" Loveframes wrappers
	-- Love Frames - A GUI library for LOVE --
	-- Copyright (c) 2012-2014 Kenny Shields --
	https://github.com/linux-man/LoveFrames
Copyright (C) 2019-today Alejandro F. Reimondo <aleReimondo@smalltalking.net>
MIT License - See license.txt file
"!

LoveObject subclass: #LoveFramesObject category: #loveframes!

LoveFramesObject subclass: #LoveFrames!
LoveFrames class instanceVariableNames: #current!
#(Object Skin Template View) do: [:each|
	LoveFramesObject subclass: #LF ,each
]!

! LoveFrames class methodsFor: #accessing !
current
	" Private - Returns the current instance. "

	current isNil ifTrue: [ current := self new ].
	^current! !

! LoveFrames class methodsFor: #accessing !
object: aName
	" Returns the object with aName (or nil). "

	^self current objectNamed: aName! !

! LoveFrames class methodsFor: #accessing !
skin: aName
	" Returns the skin with aName (or nil). "

	^self current skinNamed: aName! !

! LoveFrames class methodsFor: #accessing !
template: aName
	" Returns the template with aName (or nil). "

	^self current templateNamed: aName! !

! LoveFrames class methodsFor: #accessing !
objects
	" Returns the registered objects. "

	^self current objects! !

! LoveFrames class methodsFor: #accessing !
skins
	" Returns the registered skins. "

	^self current skins! !

! LoveFrames class methodsFor: #accessing !
templates
	" Returns the registered templates. "

	^self current templates! !

! LoveFrames methodsFor: #initialize !
initializeHandle
	" Private - Initialize the handle of the receiver. "

	self handle: (# #require: #loveframes).
	handle isNil ifTrue: [ self error: 'Missing loveframes support' ] ! !

LoveFrames buildTypedGetters: #(
	#(base Lua)
)!
LoveFrames buildTypedAccessors: #(
	author
	#(basicfont Font)
	#(basicfontsmall Font)
	collisioncount
	#(collisions Array)
	#(config PoolDictionary)
	downobject
	dragobject
	drawcount
	hover
	#(hoverobject Lua)
	input_cursor_set
	inputobject
	objectcount
	#(utf8 Lua)
	resizeobject
	version
	stage state
 	modalobject
)!
LoveFrames buildLuaFunctions: #(
	#(require #(moduleName))
	draw
	#(keypressed #(key isrepeat))
	#(keyreleased #(key))
	#(mousepressed #(x y button))
	#(mousereleased #(x y button))
	#(update #(dt))
	#(textinput #(text))
	#(wheelmoved #(x y))
	GetState #(SetState #(name))
	#(GetActiveSkin LFSkin) #(SetActiveSkin #(name))
	#(BoundingBox #(x1 x2 y1 y2 w1 w2 h1 h2))
	#(GetCollisions #(_object into))
	#(GetAllObjects #(_object into))
	#(GetDirectoryContents #(dir into))
	RemoveAll
	GetCollisionCount
	GetHover #(GetHoverObject LFObject)
	IsCtrlDown
	#(Color #(s)) #(Color #(s a))
	DebugDraw
	#(Create #(_data) LFView) #(Create #(_data _parent) LFView)
	#(NewObject #(id name)) #(NewObject #(id name inheritFromBase))
	#(LoadObjects #(dir))
	#(ApplyTemplatesToObject #(_object))
	#(LoadTemplates #(dir))
	#(RegisterSkin #(_skin))
	#(LoadSkins #(dir))
)!

! LoveFrames methodsFor: #accessing !
collisions: object
	" Return the collisions to object. "

	^self collisions: object into: Array new! !

! LoveFrames methodsFor: #accessing !
allObjects: object
	" Return all active objects. "

	^self allObjects: object into: Array new! !

! LoveFrames methodsFor: #accessing !
directoryContents: dir
	" Return directory contents. "

	^self directoryContents: dir into: Array new! !

! LoveFrames methodsFor: #accessing !
cls
	" Return the (loveframes)class of the receiver. "

	^self handle basicAt: #class! !

! LoveFrames methodsFor: #accessing !
skins
	" Return a dictionary with the skins of the receiver. "

	| h |
	h := Lua @ (self handle basicAt: #skins).
	h isNil ifTrue: [ ^PoolDictionary new ].
	^h keys	inject: PoolDictionary new into: [:total :each|
		total at: each put: LFSkin @ (h @ each); yourself ]! !

! LoveFrames methodsFor: #accessing !
skinNamed: aName
	" Return the skin of the receiver with aName (or nil). "

	| h |
	h := Lua @ (self handle basicAt: #skins).
	^h notNil ifTrue: [ ^LFSkin @ (h @ aName) ]! !

! LoveFrames methodsFor: #accessing !
templates
	" Return a dictionary with the templates of the receiver. "

	| h |
	h := Lua @ (self handle basicAt: #templates).
	h isNil ifTrue: [ ^PoolDictionary new ].
	^h keys	inject: PoolDictionary new into: [:total :each|
		total at: each put: LFTemplate @ (h @ each); yourself ]! !

! LoveFrames methodsFor: #accessing !
templateNamed: aName
	" Return the template of the receiver with aName (or nil). "

	| h |
	h := Lua @ (self handle basicAt: #templates).
	^h notNil ifTrue: [ ^LFTemplate @ (h @ aName) ]! !

! LoveFrames methodsFor: #accessing !
objects
	" Return a dictionary with the objects of the receiver. "

	| h |
	h := Lua @ (self handle basicAt: #objects).
	h isNil ifTrue: [ ^PoolDictionary new ].
	^h keys	inject: PoolDictionary new into: [:total :each|
		total at: each put: LFObject @ (h @ each); yourself ]! !

! LoveFrames methodsFor: #accessing !
objectNamed: aName
	" Return the object of the receiver with aName (or nil). "

	| h |
	h := Lua @ (self handle basicAt: #objects).
	^h notNil ifTrue: [ ^LFObject @ (h @ aName) ]! !

LFTemplate buildTypedAccessors: #(
	name
	#(properties Lua)
)!

! LFTemplate class methodsFor: #accessing !
base
	" Returns the base template. "

	^LoveFrames current templateNamed: #Base! !

! LFTemplate methodsFor: #accessing !
star
	" Return the default properties. "

	^Lua @ (self properties @ #*)! !

#(	alwaysupdate
	calledmousefunc
	collide draworder
	height hover
	internal retainsize
	state staticx staticy
	visible width
	x y
) do: [:each| "keys found in Base template star" ]!

! LFSkin class methodsFor: #accessing !
default
	" Returns the default skin. "

	^LoveFrames current skinNamed: #Default! !

LFSkin buildTypedAccessors: #(
	author name version dir
	#(directives Lua)
	#(images Lua)
	#(controls Lua)
)!

! LFSkin methodsFor: #accessing !
imageNamed: aName
	" Return the image of the receiver with aName (or nil). "

	| h |
	h := Lua @ (self handle basicAt: #images).
	^h notNil ifTrue: [ ^Image @ (h @ aName) ]! !

! LFSkin methodsFor: #accessing !
controlNamed: aName
	" Return the control of the receiver with aName (or nil). "

	| h |
	h := Lua @ (self handle basicAt: #controls).
	^h notNil ifTrue: [ ^Lua @ (h @ aName) ]! !

! LFSkin methodsFor: #accessing !
color: selector
	" Return the color of the receiver at selector (or nil). "

	| rgba |
	rgba := self controlNamed: #color_ ,selector.
	^rgba notNil ifTrue: [ Array @ rgba ]! !

! LFSkin methodsFor: #accessing !
font: selector
	" Return the font of the receiver at selector (or nil). "

	| font |
	font := self controlNamed: selector ,#font.
	^font notNil ifTrue: [ Font @ font ]! !

! LFSkin methodsFor: #accessing !
smallFont
	" Return the small font of the receiver (or nil). "

	^self font: #small! !

"...skin functions...
 OutlinedRectangle: (function: 0x34a354d8{...});
 PrintText: (function: 0x349d60a0{...});
 button: (function: 0x349f43f0{...});
 checkbox: (function: 0x34a5d6c8{...});
 closebutton: (function: 0x349f7370{...});
 collapsiblecategory: (function: 0x349ea288{...});
 columnlist: (function: 0x34a4c3a8{...});
 columnlistarea: (function: 0x34a53188{...});
 columnlistarea_over: (function: 0x34a53448{...});
 columnlistheader: (function: 0x34a42848{...});
 columnlistrow: (function: 0x34a45ae8{...});
 form: (function: 0x34a58a28{...});
 frame: (function: 0x349fb718{...});
 grid: (function: 0x349f2148{...});
 image: (function: 0x34a5b300{...});
 imagebutton: (function: 0x34a46270{...});
 linenumberspanel: (function: 0x349f1ae0{...});
 list: (function: 0x349d4c60{...});
 list_over: (function: 0x34a5b7a8{...});
 menu: (function: 0x34a490f0{...});
 menuoption: (function: 0x349ef3b0{...});
 modalbackground: (function: 0x34a59018{...});
 multichoice: (function: 0x34a3aa88{...});
 multichoicelist: (function: 0x34a3a4c0{...});
 multichoicelist_over: (function: 0x34a3a0d8{...});
 multichoicerow: (function: 0x34a42dc0{...});
 numberbox: (function: 0x349f1d88{...});
 panel: (function: 0x34a511b8{...});
 progressbar: (function: 0x34a50910{...});
 radiobutton: (function: 0x34a4d9b8{...});
 scrollarea: (function: 0x34a4d108{...});
 scrollbar: (function: 0x34a55ca8{...});
 scrollbody: (function: 0x34a52a88{...});
 scrollbutton: (function: 0x34a442f0{...});
 slider: (function: 0x34a52940{...});
 sliderbutton: (function: 0x34a509a8{...});
 tabbutton: (function: 0x34a4af50{...});
 tabpanel: (function: 0x34a62f08{...});
 tabpanel_over: (function: 0x34a4aca8{...});
 text: (function: 0x349b9098{...});
 textinput: (function: 0x349cbdc0{...});
 textinput_over: (function: 0x34a56000{...});
 tooltip: (function: 0x349c6d88{...});
 tree: (function: 0x34a0beb8{...});
 treenode: (function: 0x34a0d548{...});
 treenodebutton: (function: 0x34a0f490{...});
"

LFObject buildTypedAccessors: #(
	name
	#(static Lua) #(super Lua)
	#(subclasses 'Array[LFObject]')
	#(__declaredMethods Lua)
	#(__instanceDict Lua)
)!


! LFView class methodsFor: #instantiating !
from: data parent: aParent
	" Returns an instance of the receiver. "

	^LoveFrames current
		create: data
		parent: aParent! !

! LFView class methodsFor: #instantiating !
from: data
	" Returns an instance of the receiver. "

	^LoveFrames current create: data! !

! LFView class methodsFor: #instantiating !
type: data in: aParent
	" Returns an instance of the receiver. "

	^LoveFrames current
		create: data
		parent: aParent! !

! LFView class methodsFor: #instantiating !
type: data
	" Returns an instance of the receiver. "

	^LoveFrames current create: data! !

LFView buildTypedAccessors: #(
	alwaysupdate
	calledmousefunc
	collide draworder hover
	internal retainsize state
	staticx staticy
	visible
	height width x y
)!
LFView buildLuaFunctions: #(
	draw #(update #(dt))
	#(keypressed #(key isrepeat))
	#(keyreleased #(key))
	#(mousepressed #(x y button))
	#(mousereleased #(x y button))
	#(textinput #(text))
	#(wheelmoved #(x y))
	#(SetPos #(x y)) #(SetPos #(x y center))
	#(SetX #(x)) #(SetX #(x center))
	#(SetY #(y)) #(SetY #(y center))
	GetX GetY GetStaticX GetStaticY
	Center CenterX CenterY
	#(CenterWithinArea #(x y width height))
	#(SetWidth #(width)) #(SetWidth #(width relative))
	#(SetHeight #(height)) #(SetHeight #(height relative))
	GetWidth GetHeight GetVisible
	#(SetVisible #(aBoolean))
	GetParent #(SetParent #(_parent))
	Remove RemoveClickBounds InClickBounds
	#(SetClickBounds #(x y width height))
	#(GetBaseParent #(_t) LFObject)
	CheckHover #(GetHover LFView)
	#(GetChildren 'Array[LFView]')
	#(GetInternals 'Array[LFView]')
	IsTopList IsTopChild
	MoveToTop SetDrawFunc
	GetSkinName #(GetSkin LFSkin) #(SetSkin #(name))
	GetAlwaysUpdate #(SetAlwaysUpdate #(bool))
	GetRetainSize #(SetRetainSize #(bool))
	IsActive
	#(GetParents 'Array[LFView]')
	IsInternal IsTopInternal
	GetType GetDrawOrder #(SetDrawOrder  #(ignored))
	#(SetProperty #(name _value)) #(GetProperty #(name))
	IsInList
	GetState #(SetState #(name))
)!

! LFView methodsFor: #accessing !
pos
	| x y |
{'	x, y = self:handle():GetPos()'}.
	^Array with: x with: y! !

! LFView methodsFor: #accessing !
position
	| x y |
{'	x, y = self:handle():GetPos()'}.
	^x@y! !

! LFView methodsFor: #accessing !
position: aPoint
	self pos: aPoint x y: aPoint y! !

! LFView methodsFor: #accessing !
staticPosition
	| x y |
{'	x, y = self:handle():GetStaticPos()'}.
	^x@y! !

! LFView methodsFor: #accessing !
staticPos
	| x y |
{'	x, y = self:handle():GetStaticPos()'}.
	^Array with: x with: y! !

! LFView methodsFor: #layout !
centerWithinArea: aRectangle
	self centerWithinArea: aRectangle left y: aRectangle top
		width: aRectangle width height: aRectangle height! !

! LFView methodsFor: #accessing !
extent
	| x y |
{'	x, y = self:handle():GetSize()'}.
	^x@y! !

! LFView methodsFor: #accessing !
size
	| x y |
{'	x, y = self:handle():GetSize()'}.
	^Array with: x with: y! !
