" Extension to S8's FileIn framework for stringified content.
  @2018 Alejandro F. Reimondo aleReimondo@smalltalking.net
 See license.txt file
"

FileInReader
	subclass: #StringifiedReader
	instanceVariableNames: ' action objectifier'
	category: 'Sharing-FileIn' !

! StringifiedReader methodsFor: #accessing !
action: aBlock
	" Set the action to evaluate on each object read. "

	action := aBlock! !

! StringifiedReader methodsFor: #accessing !
objectifier: anObjectifier
	" Set the objectifier in use to read objects. "

	objectifier := anObjectifier! !

! StringifiedReader methodsFor: #accessing !
objectifier
	" Returns the objectifier in use to read objects. "

	objectifier isNil ifTrue: [ objectifier := Objectifier new ].
	^objectifier! !

! StringifiedReader methodsFor: #fileIn !
scanFrom: aStream
	" FileIn - Parse next chunk in aStream as stringified contents, evaluating action on the object read. "

	| chunk object |
	chunk := aStream nextChunk.
	chunk isEmpty ifTrue: [ ^nil ].
	object := self objectifier objectFrom: chunk.
	action isNil ifTrue: [ ^object ].
	^action evaluateWith: object! !

! Smalltalk class methodsFor: 'Sharing-FileIn' !
objectsDoing: aBlock
	" FileIn - Return a reader to fileIn stringified objects.
	aBlock is evaluated with each object read.
	"

	^StringifiedReader new
		action: aBlock;
		yourself! !

! Smalltalk class methodsFor: 'Sharing-FileIn' !
objects
	" FileIn - Return a reader to fileIn stringified objects. "

	^StringifiedReader new! !