/*
	S8 - A modern Smalltalk environment running on the "invisible machine".
	Copyright (C) 2023-today Alejandro F. Reimondo <aleReimondo@smalltalking.net>
	MIT License - See license.txt file
	
	Version 1.7.0 - Stable
	
	
	For questions and bug reports join us at S8 Facebook group
	https://www.facebook.com/groups/s8smalltalk/
*/
function StObject(){}
function StBehavior(){}
function StClass(){}
function StMethod(){}
function StMetaclass(){
	this.$meta = true;
}
function StUndefinedObject(){}
var nil = new StUndefinedObject()
nil.toString = function(){ return "nil" }
nil.isNil_ = function (x){
	switch(typeof x) {
		case "undefined": 
			return true
		case "string": case "boolean": 
		case "number": case "function": 
			return false
		
	}
	if (x===null || x===this) return true
	if (x===smalltalk) return false
	if (x.$klass && "function" === typeof x.isNil) return x.isNil()
	return false
}
nil.is_eqeq_ = function (a,b){
	if (a===b) return true
	if ((this.isNil_(a)) || (this.isNil_(b))) return false
	return (a.valueOf()) === (b.valueOf())
}
function StSystemDictionary(){
	this.$imageFileName = "s8.image.js"
	this.$nativeLanguage = "js"
	this.useManager = false
	this.isNil = function (x){
		return (x || nil) === nil
	}
	this.klass = function (spec){// Private
	
		var aClass
		if (spec.meta) {
			aClass = new StMetaclass()
		} else {
			var metaSpec = {meta:true}
			metaSpec.className = spec.className + " class"
			if (!this.isNil(spec.superclass)) {
				metaSpec.superclass = spec.superclass.$klass
			}
			var metaClass = this.klass(metaSpec)
			aClass = new metaClass.$fn()
		}
		this.setClass(aClass,spec)
		aClass.$fn.prototype.$klass = aClass
		aClass.$fn.prototype.$methods = {}
		if (aClass.$className == "Metaclass") {
			this.bindRoots(aClass)
		}
		return aClass
	}
	this.setProtoOf_to_ = function (nativeObject,proto){// Private
	
		nativeObject.__proto__ = proto
	}
	this.bindRoots = function (metaclass){// Private
	
		var objectClassPrototype  = this.Object.$klass.$fn.prototype
		this.setProtoOf_to_(objectClassPrototype,this.Class.$fn.prototype)
		this.setProtoOf_to_(this.Smalltalk.$klass.$fn.prototype,objectClassPrototype)
		this.setProtoOf_to_(this.Behavior.$klass.$fn.prototype,objectClassPrototype)
		this.Object.$klass.$superclass = this.Class
		metaclass.$superclass = this.Behavior
	}
	this.setVarNames = function (aClass,iVarNames,aValue){// Private
	
		aClass.$iVarNames = iVarNames
		var i = iVarNames.length
		while(i--){
			aClass.$fn.prototype["@"+iVarNames[i]] = aValue
		}
	}
	this.setClass = function (aClass,spec){// Private
	
		aClass.$className = spec.className
		if (!spec.meta) {
			aClass.$klass["@instanceClass"] = aClass
		}
		aClass.$category = spec.category || ""
		aClass.$superclass = spec.superclass || nil
		if (!this.isNil(aClass.$superclass)) {
			delete aClass.$superclass["@subclasses"]
			if (!this.isNil(aClass.$klass)) {
				if (!spec.meta) {
					aClass.$klass.$superclass = aClass.$superclass.$klass
				}
				if (aClass.$klass.$superclass) {
					delete aClass.$klass.$superclass["@subclasses"]
				}
				
			}
			
		}
		aClass.$fn = this.fnInSpec(spec)
		this.setVarNames(aClass,spec.iVarNames||[],nil)
		return aClass
	}
	this.fnInSpec = function (spec){// Private
	
		if (spec.meta && (this.isNil(spec.superclass))) {
			spec.superclass = this.Class
		}
		if (spec.fn) {
			if (!this.isNil(spec.superclass)) {
				this.setProtoOf_to_(spec.fn.prototype,spec.superclass.$fn.prototype)
			}
			return spec.fn
		}
		var $s8Constructor = function(){}
		if (!this.isNil(spec.superclass)) {
			$s8Constructor.prototype = new spec.superclass.$fn()
		}
		return $s8Constructor
	}
	this.classes = function (){
		var classes = []
		for(var i in this){
			if ((this.hasOwnProperty(i)) && i != "$klass") {
				var cls = this[i]
				if (!this.isNil(cls)) {
					if (cls.$klass && cls.$className) {
						classes.push(cls)
					}
				}
			}
			
		}
		return classes
	}
	this.subclasses = function (aClass){
		if (aClass.hasOwnProperty("@subclasses")) return aClass["@subclasses"]
		var classes = this.classes()
		var i = classes.length
		while(i--){
			classes[i]["@subclasses"] = []
			classes[i].$klass["@subclasses"] = []
			
		}
		i = classes.length
		while(i--){
			var cls = classes[i]
			if (!this.isNil(cls.$superclass)) {
				cls.$superclass["@subclasses"].push(cls)
				cls.$superclass.$klass["@subclasses"].push(cls.$klass)
			}
		}
		if (this.Class) {
			this.Class["@subclasses"] = []
			this.$klass["@subclasses"] = []
		}
		return aClass["@subclasses"]
	}
	this.allSubclassesDo = function (aClass,aBlock){
		var subclasses = this.subclasses(aClass)
		var i = subclasses.length
		while(i--){
			var cls = subclasses[i]
			aBlock(cls)
			this.allSubclassesDo(cls,aBlock)
		}
		return this
	}
	this.mapClassName = function (className,category,fn,superclass){
		var spec = {
			className: className,
			category: category,
			superclass: superclass,
			fn: fn
		}
		if (this[className]) return this.setClass(this[className],spec)
		this[className] = this.klass(spec)
		return this[className]
	}
	this.sameIVContents = function (array1,array2){
		if (array1 == array2) return true
		if (array1.length === array2.length) {
			return array1.every(function (element,index){
				return element == array2[index]
			})
		}
		return false
	}
	this.addClass = function (className,superclass,iVarNames,category){
		this.debugAddClass(className,superclass,iVarNames,category)
		if (this[className]) {
			var aClass = this[className]
			var sameShape = (aClass.$superclass == superclass) && (this.sameIVContents(aClass.$iVarNames,iVarNames))
			if (sameShape && (aClass.$category == category)) return aClass
			this.aboutToChangeClass(aClass,superclass,iVarNames,category)
			if (aClass.$superclass) {
				delete aClass.$superclass["@subclasses"]
				var aMeta = aClass.$klass
				if (aMeta && superclass) {
					delete aMeta.$superclass["@subclasses"]
					aMeta.$superclass = superclass.$klass
				}
			}
			aClass.$superclass = superclass
			if (superclass && superclass["@subclasses"]) {
				if (this.isNil(superclass["@subclasses"])) {
					superclass["@subclasses"] = []
				}
				superclass["@subclasses"].push(aClass)
			}
			this.setVarNames(aClass,iVarNames||[],nil)
			aClass.$category = category || aClass.$category
			if (!sameShape) {
				this.classShapeChanged(aClass)
			}
		} else {
			this.aboutToCreateClass(className,superclass,iVarNames,category)
			this[className] = this.klass({
				className: className,
				iVarNames: iVarNames,
				superclass: superclass
			})
			this[className].$category = category || ""
			
		}
		this.classChanged(this[className])
		return this[className]
	}
	this.renameClass = function (aClass,aString){
		delete this[aClass.$className]
		aClass.$className = aString
		this[aString] = aClass
		this.classChanged(aClass)
	}
	this.removeClass = function (aClass){
		var deleted = "Deleted-"
		if (aClass.$superclass) {
			delete aClass.$superclass["@subclasses"]
		}
		delete this[aClass.$className]
		aClass.$className = deleted + aClass.$className
		aClass.$klass.$className = deleted + aClass.$klass.$className
		this.classChanged(aClass)
	}
	this.metaVarNames = function (aMetaClass,iVarNames){
		if (this.sameIVContents(aMetaClass.$iVarNames,iVarNames)) return false
		this.setVarNames(aMetaClass,iVarNames||[],nil)
		this.classShapeChanged(aMetaClass)
		this.classChanged(aMetaClass)
		return true
	}
	this.smDo = function (block){
		if (!this.useManager) return nil
		if (!this.SystemManager) return nil
		if (!this.SystemManager.currentOrNil) return nil
		var sm = this.SystemManager.currentOrNil()
		if (!this.isNil(sm)) return block(sm)
	}
	this.classChanged = function (aBehavior){
		this.smDo(function (sm){
			sm.classChanged_(aBehavior)
		})
	}
	this.classShapeChanged = function (aBehavior){
		this.smDo(function (sm){
			sm.classShapeChanged_(aBehavior)
		})
	}
	this.aboutToCreateClass = function (className,superclass,iVarNames,category){
		this.smDo(function (sm){
			sm.aboutToCreateClass_superclass_iVarNames_category_(className,superclass,iVarNames,category)
		})
	}
	this.aboutToChangeClass = function (aClass,superclass,iVarNames,category){
		this.smDo(function (sm){
			sm.aboutToChange_superclass_iVarNames_category_(aClass,superclass,iVarNames,category)
		})
	}
	this.methodBoundTo = function (method,klass){
		this.smDo(function (sm){
			sm.methodBound_to_(method,klass)
		})
		return method
	}
	this.aboutToBindTo = function (method,klass){
		this.smDo(function (sm){
			sm.aboutToBind_to_(method,klass)
		})
		return method
	}
	this.aboutToRemoveFrom = function (method,klass){
		this.smDo(function (sm){
			sm.aboutToRemove_from_(method,klass)
		})
		return method
	}
	this.methodRemovedFrom = function (method,klass){
		this.smDo(function (sm){
			sm.methodRemoved_from_(method,klass)
		})
		return method
	}
	this.objectNotFound = function (object,indexOrKey,collection){
		this.smDo(function (sm){
			sm.object_at_notFoundIn_(object,indexOrKey,collection)
		})
		return collection
	}
	this.whoImplements = function (klass,jsSelector){
		var cls = klass
		do {
			if (cls.$fn.prototype.hasOwnProperty(jsSelector)) return cls
			cls = cls.$superclass
		} while (cls)
		return nil
	}
	this.superImplementor = function (klass,jsSelector){
		if ((this.isNil(klass.$superclass)) || (this.isNil(klass.$superclass.$fn))) return klass.missingImplementorFor_(jsSelector)
		var cls = this.whoImplements(klass.$superclass,jsSelector)
		if (this.isNil(cls)) return klass.$superclass.missingImplementorFor_(jsSelector)
		return cls.$fn.prototype[jsSelector]
	}
	this.addMethod = function (jsSelector,method,klass){
		this.aboutToBindTo(method,klass)
		klass.$fn.prototype[jsSelector] = method.$fn
		klass.$fn.prototype.$methods[method.$selector] = method
		method["@classField"] = klass
		return this.methodBoundTo(method,klass)
	}
	this.bind = function (klass,stSelector,jsSelector,jsFunction,category,stSource,stamp){
		this.debugBind(klass,stSelector,jsSelector,jsFunction,category,stSource,stamp)
		var method = new StMethod()
		
		method.$selector = stSelector
		method.$fn = jsFunction
		method.$category = category
		method.$source = stSource
		if (!this.isNil(stamp)) {
			method.$stamp = stamp
		}
		method.$generation = this.$state
		if (!this.isNil(klass)) {
			if (this.isNil(jsSelector)) {
				jsSelector = stSelector.replace(/\:/g,"_")
			}
			this.addMethod(jsSelector,method,klass)
		}
		return method
	}
	this.removeMethod = function (aClass,aMethod){
		this.aboutToRemoveFrom(aMethod,aClass)
		delete aClass.$fn.prototype[aMethod.$selector.asSelector()]
		delete aClass.$fn.prototype.$methods[aMethod.$selector]
		return this.methodRemovedFrom(aMethod,aClass)
	}
	this.newMethod = function (currentClass,selector,fn,source){
		return this.bind(nil,selector,nil,fn,nil,source,nil)
	}
	this.readJSObject = function (js){
		if (!js) return nil
		var object = js
		var readObject = js.constructor === Object
		var readArray = js.constructor === Array
		if (readObject) {
			object = this.PoolDictionary.$$new()
		}
		for(var i in js){
			if (!js.hasOwnProperty(i)) break
			if (readObject) {
				object.at_put_(i,this.readJSObject(js[i]))
			}
			if (readArray) {
				object[i] = this.readJSObject(js[i])
			}
		}
		return object
	}
	this.globalsDo = function (action){
		for(key in this){
			if (!this.hasOwnProperty(key)) break
			var aValue = this[key]
			if (typeof(aValue) != "function" && (aValue.isClass && (!aValue.isClass()))) {
				action(key)
			}
			
		}
		return this
	}
	this.startup = function (){
		this.$state = "booting"
		this.OverwriteMethods = this.OverwriteMethods || true
		this.DumpOverwritenMethods = this.DumpOverwritenMethods || false
		this.mapClassName("Object","Kernel",StObject)
		this.mapClassName("Smalltalk","Kernel",StSystemDictionary,this.Object)
		this.mapClassName("Behavior","Kernel",StBehavior,this.Object)
		this.mapClassName("Class","Kernel",StClass,this.Behavior)
		this.mapClassName("Metaclass","Kernel",StMetaclass,this.Behavior)
		this.mapClassName("CompiledMethod","Kernel",StMethod,this.Object)
		this.mapClassName("Number","Kernel",Number,this.Object)
		this.mapClassName("BlockClosure","Kernel",Function,this.Object)
		this.mapClassName("Boolean","Kernel",Boolean,this.Object)
		this.mapClassName("DateTime","Kernel",Date,this.Object)
		this.mapClassName("UndefinedObject","Kernel",StUndefinedObject,this.Object)
		this.mapClassName("Collection","Kernel",undefined,this.Object)
		this.mapClassName("SequenceableCollection","Kernel",undefined,this.Collection)
		this.mapClassName("String","Kernel",String,this.SequenceableCollection)
		this.mapClassName("Array","Kernel",Array,this.SequenceableCollection)
		this.mapClassName("RegularExpression","Kernel",RegExp,this.Object)
		this.$state = "startup"
		return this
	}
	this.launchImage = function (imageName){
		this.useManager = true
		if (!this.isNil(imageName)) {
			this.$imageFileName = imageName
			load(imageName)
		}
		this.$state = "launching"
		if (this.launch && typeof this.launch == "function") {
			this.launch()
		}
		this.$state = "running"
		return this
	}
	this.is_kindOf_ = function (anObject,aClass){
		if ((nil.isNil_(anObject)) && (nil !== anObject)) return false
		if ("undefined" === typeof anObject.isKindOf_) return false
		return anObject.isKindOf_(aClass)
	}
	this.responds_to_ = function (anObject,aSelector){
		if ((nil.isNil_(anObject)) && (nil !== anObject)) return false
		if ("undefined" === typeof anObject.respondsTo_) return false
		return anObject.respondsTo_(aSelector)
	}
	this.debugBind = function (klass,stSelector,jsSelector,jsFunction,category,stSource,stamp){
		return this
	}
	this.debugAddClass = function (className,superclass,iVarNames,category){
		return this
	}
	this.initWithSubclasses = function (aClass){
		return aClass
	}
	
}
var smalltalk = new StSystemDictionary()
smalltalk.toString = function() { return "smalltalk" }
smalltalk.startup()
