" FlashAir Library wrappers
Copyright (C) 2019-today Alejandro F. Reimondo <aleReimondo@smalltalking.net>
MIT License - See license.txt file
See also https://www.flashair-developers.com/en/documents/api/lua/
"!

NativeObject subclass: #FlashAir category: #FlashAir!

! FlashAir class methodsFor: #accessing !
nativeClass
	" Private - Returns the native module. "

	^Lua @ #{fa} ! !

! FlashAir methodsFor: #initialize !
initializeHandle
	" Private - Initializze the handle of the receiver. "

	handle := #{fa} ! !

! FlashAir class methodsFor: #converting !
stringToFatDateTime: aString
	" Convert the date and time of the string to FAT format. "

{'	local pattern = "(%d+)/(%d+)/(%d+)%s+(%d+):(%d+):(%d+)"
	local year,month,day,hour,min,sec = string.match(aString, pattern)
	year = year - 1980
	sec = bit32.rshift(sec, 1)
	local date_fat = bit32.bor(bit32.lshift(year, 9), bit32.lshift(month, 5), day)
	local time_fat = bit32.bor(bit32.lshift(hour, 11), bit32.lshift(min, 5), sec)
	local datetime_fat = bit32.bor(bit32.lshift(date_fat, 16), time_fat)
	if true then return datetime_fat end
'}.	self error: 'Should not happen'! !

! FlashAir class methodsFor: #converting !
fatDateTimeToString: aNumber
	" Convert the date and time of the string to FAT format. "

{'	local function getbits(x, from, to)
		local mask = bit32.lshift(1, to - from + 1) - 1
		local shifted = bit32.rshift(x, from)
		return bit32.band(shifted, mask)
	end
	local fatdate = bit32.rshift(aNumber, 16)
	local day = getbits(fatdate, 0, 4)
	local month = getbits(fatdate, 5, 8)
	local year = getbits(fatdate, 9, 15) + 1980
	local fattime = getbits(datetime_fat, 0, 15)
	local sec = getbits(fattime, 0, 4) * 2
	local min = getbits(fattime, 5, 10)
	local hour = getbits(fattime, 11, 15)
	if true then
		return string.format("%02d/%02d/%02d %02d:%02d:%02d", year, month, day, hour, min, sec)
	end
'}.	self error: 'Should not happen'! !

FlashAir buildLuaStaticFunctions: #(
	#(Bridge #(ssid networkkey encmode brgssid brgnetworkkey) )
	#(Connect #(ssid networkkey))
	Disconnect
	#(Establish #(ssid networkkey encmode))
	#(ftp #(cmd uri filename))
	#(HTTPGetFile #(uri filepath user pass) Boolean)
	#(HTTPGetFile #(uri filepath) Boolean)
	#(ip #(address subnet gateway))
	#(MailSend #(from headers body server user password attachment contentType))
	#(ping #(ip) Boolean)
	ReadStatusReg
	Scan #(Scan #(ssid))
	#(SetCert #(filename) Boolean)
	#(SetChannel #(channelNo))
	#(WlanLink #() Boolean)
)!

FlashAir buildLuaFunctions: #(
	#(control #(key))
	#(control #(key value))
	#(hash #(name data key))
	#(remove #(filepath))
	#(rename:to: #(filePath newFilepath))
	#(sharedmemory #(command addr len wdata))
	#(spi #(command data))
	#(strconvert #(format orgstr))
	#(watchdog #(command data))
	#(watchdog #(command))
)!

! FlashAir class methodsFor: #accessing !
websocket: options
	" WebSocket communication. "

	| res type payload |
{'	res, type, payload = fa.websocket(options:json()) '}.
	^Array with: res with: type with: payload! !

! FlashAir class methodsFor: #accessing !
search: type path: path searchtime: searchtime
	" Searches for files based on the updated date and time in the specified directory.
	It is possible to search for a file whose with the latest date and time, or to search for a file with the update date specified by argument. "

	| result filelist time |
{'	result, filelist, time = fa.search(type, path, searchtime) '}.
	result = 1 ifTrue: [
		^filelist asArrayOfSubstringsSeparatedBy: $,
	].
	result = -1 ifTrue: [ ^self error: 'Overflow when creating file list' ].
	result = -2 ifTrue: [ ^self error: 'String buffer overflow' ].
	result = -3 ifTrue: [ ^self error: 'Directory open error' ].
	result = -4 ifTrue: [ ^self error: 'Directory read error' ].
	^self error: 'Unexpected result ',result! !

! FlashAir class methodsFor: #accessing !
connectedStations
	" Returns anArray with information about connected stations. "

	| cnt tbl |
{'	cnt,tbl = fa.ConnectedSTA() '}.
	^Array @ tbl collect: [:each| Lua @ each ]! !

! FlashAir class methodsFor: #accessing !
getScanInfo: num
	" Returns result of SSID scan. "

	| ssid info |
{'	ssid, info = fa.GetScanInfo(num) '}.
	^ssid -> (PoolDictionary @ info)! !

! FlashAir methodsFor: #control !
fioget
	" Get on/off state of wireless LAN. "

	^self handle #control: #fioget! !

! FlashAir methodsFor: #control !
fioset: aBoolean
	" Turn on / off the wireless LAN. "

	^self handle #control: #fioset with: aBoolean! !

! FlashAir methodsFor: #control !
hidChange: oldPsw password: newPsw
	" Change the password set for the hidden area. "

	^self handle #control: #hid_change_pass from: oldPsw to: newPsw! !

! FlashAir methodsFor: #control !
hidClear
	" Delete all script files saved by password, API. "

	^self handle #control: #hid_clear! !

! FlashAir methodsFor: #control !
hidGet: filename as: dstfilename password: password
	" Get the script saved in the hidden area. "

	^self handle #control: #hid_get
		filename: filename as: dstfilename
		password: password! !

! FlashAir methodsFor: #control !
hidHash: aString
	" Create a hash string from the password set in the hidden area and the specified character string. "

	^self handle #control: #hid_hash with: aString! !

! FlashAir methodsFor: #control !
hidPassword: password
	" Sets the password for the preservation of the secret area. "

	^self handle #control: #hid_set_pass with: password! !

! FlashAir methodsFor: #control !
hidStore: fileName password: password
	" Sets the password for the preservation of the secret area. "

	^self handle #control: #hid_store with: fileName with: password! !

! FlashAir methodsFor: #control !
time
	"  Returns the date and time memo.
	A a 32-bit integer value, the upper 16 bits are the date and the lower 16 bits are the time.
	"

	^self handle #control: #time! !

! FlashAir methodsFor: #control !
time: saveTime
	"  Set the date and time memo. "

	^self handle #control: #time with: saveTime! !

! FlashAir class methodsFor: #accessing !
i2c: options
	" Returns result of I2C operation from FlashAir. i2c works as Master. "

	| values |
{'	values = (table.pack or pack) (fa.i2c(options:json())) '}.
	^Array @ values! !

! FlashAir class methodsFor: #accessing !
ip
	" Returns the ip address of FlashAir "

	| ip mask gw |
{'	ip, mask, gw = fa.ip() '}.
	^Array with: ip with: mask with: gw! !

! FlashAir class methodsFor: #accessing !
pio: ctrl data: data
	" Controls the SD interface with PIO (Programmed Input/Output). "

	| s indata |
{'	s, indata = fa.pio(ctrl, data) '}.
	^s == 1 ifTrue: [ indata ]! !

! FlashAir class methodsFor: #accessing !
request: url
	method: method headers: headers
	file: file body: body bufsize: bufsize
	redirect: redirect rcvbuff: rcvbuff rcvfile: rcvfile
	" Issues an HTTP (Hyper Text Transfer Protocol) request. "

	| resBody status header |
{'	b, c, h = fa.request(url ,method ,headers ,file ,body ,bufsize ,redirect ,rcvbuff ,rcvfile) '}.
	^Array with: resBody with: status with: header ! !

! FlashAir class methodsFor: #accessing !
request: url
	method: method headers: headers
	file: file body: body bufsize: bufsize
	redirect: redirect rcvbuff: rcvbuff
	" Issues an HTTP (Hyper Text Transfer Protocol) request. "

	| resBody status header |
{'	b, c, h = fa.request(url ,method ,headers ,file ,body ,bufsize ,redirect ,rcvbuff) '}.
	^Array with: resBody with: status with: header ! !

! FlashAir class methodsFor: #accessing !
request: url
	method: method headers: headers
	file: file body: body bufsize: bufsize
	redirect: redirect
	" Issues an HTTP (Hyper Text Transfer Protocol) request. "

	| resBody status header |
{'	b, c, h = fa.request(url ,method ,headers ,file ,body ,bufsize ,redirect) '}.
	^Array with: resBody with: status with: header ! !

! FlashAir class methodsFor: #accessing !
request: url
	method: method headers: headers
	file: file body: body bufsize: bufsize
	" Issues an HTTP (Hyper Text Transfer Protocol) request. "

	| resBody status header |
{'	b, c, h = fa.request(url ,method ,headers ,file ,body ,bufsize) '}.
	^Array with: resBody with: status with: header ! !

! FlashAir class methodsFor: #accessing !
request: url
	method: method headers: headers
	file: file body: body
	" Issues an HTTP (Hyper Text Transfer Protocol) request. "

	| resBody status header |
{'	b, c, h = fa.request(url ,method ,headers ,file ,body) '}.
	^Array with: resBody with: status with: header ! !

! FlashAir class methodsFor: #accessing !
request: url method: method headers: headers file: file
	" Issues an HTTP (Hyper Text Transfer Protocol) request. "

	| resBody status header |
{'	b, c, h = fa.request(url ,method ,headers ,file) '}.
	^Array with: resBody with: status with: header ! !

! FlashAir class methodsFor: #accessing !
request: url method: method headers: headers
	" Issues an HTTP (Hyper Text Transfer Protocol) request. "

	| resBody status header |
{'	b, c, h = fa.request(url ,method ,headers) '}.
	^Array with: resBody with: status with: header ! !

! FlashAir class methodsFor: #accessing !
request: url method: method
	" Issues an HTTP (Hyper Text Transfer Protocol) request. "

	| resBody status header |
{'	b, c, h = fa.request(url ,method) '}.
	^Array with: resBody with: status with: header ! !

! FlashAir class methodsFor: #accessing !
request: url
	" Issues a GET request. "

	| resBody status header |
{'	b, c, h = fa.request(url) '}.
	^Array with: resBody with: status with: header ! !