
Object
	subclass: #LuaStudioExporter
	instanceVariableNames:
		'  window template linkToContents linkToResources'
		,' idField userField progressField'
		,' platformField destinationField'
	category: #LuaStudioExporter !
LuaStudioExporter buildAccessors: #(
	#(window initialize 'self defaultWindow')
	#(template initialize '#Template')
	#(linkToContents initialize 'String new')
	#(linkToResources initialize 'String new')
	idField userField progressField platformField destinationField
)!

! LuaStudioExporter class methodsFor: #gui !
open
	" Open the receiver "

	^self new open! !

! LuaStudioExporter methodsFor: #defaults !
defaultWindow
	" Return default window "

	| defaultWindow screenDpi sx |
	defaultWindow := Display stage.
	screenDpi := Capabilities screenDPI. 
	sx := screenDpi > 120 ifTrue: [ screenDpi / 120 ] ifFalse: [ 1 ].
	defaultWindow scale: sx.
	^defaultWindow! !

! LuaStudioExporter methodsFor: #accessing !
platform
	" Returns the target platform "

	self platformField isNil ifTrue: [ ^#Windows ].
	^self platformField text! !

! LuaStudioExporter methodsFor: #defaults !
defaultDestination
	" Private - Returns the default path to export. "

	^LuaStudioLibrary pathToM8n
		,self contributionUser
		,$/ ,self contributionId
		,$/ ! !

! LuaStudioExporter methodsFor: #accessing !
destination
	" Returns the destination folder "

	self destinationField isNil ifTrue: [ ^self defaultDestination ].
	^self destinationField text! !

! LuaStudioExporter methodsFor: #constants !
contributionURL
	" Private - Returns the URL of the contribution set by U8 service. "

	^Smalltalk at: #ContributionURL ifAbsent: [
		self u8URL ,'smalltalking/000/'
	]! !

! LuaStudioExporter methodsFor: #accessing !
contributionId
	" Returns the contribution id "

	self idField isNil ifTrue: [ ^(self contributionURL upToLast: $/) fromLast: $/ ].
	^self idField text! !

! LuaStudioExporter methodsFor: #constants !
u8URL
	" Private - Returns the URL of the profile folder in U8 service. "

	^'http://u8.smalltalking.net/u8/profile/'! !

! LuaStudioExporter methodsFor: #accessing !
contributionUser
	" Returns the contribution user "

	self userField isNil ifTrue: [ ^((self contributionURL upToLast: $/) upToLast: $/) fromLast: $/ ].
	^self userField text! !

! LuaStudioExporter methodsFor: #downloading !
contentsAtURL: url
	" Returns the contents at url "

	^Http requestUrl: url! !

! LuaStudioExporter methodsFor: #gui !
open
	" Open the receiver "

	self
		openForm;
		addExportOptions;
		progress: 'Please, enter contribution values'! !

! LuaStudioExporter methodsFor: #gui !
openForm
	" Private - Open the form to prompt export settings. "

	self addTitle: 'Contribution User' at: 10@30.
	userField := TextField input text: self contributionUser position: 10@50 extent: 240@20.
	self window addChild: userField.
	
	self addTitle: 'Contribution Id' at: 10@80.
	idField := TextField input text: self contributionId position: 10@100 extent: 240@20.
	self window addChild: idField.
	
	self addTitle: 'Target platform (iOS, Android, Windows, Mac, WindowsJIT, MacJIT)' at: 10@150.
	platformField := TextField input text: self platform position: 10@170 extent: 240@20.
	self window addChild: platformField.
	
	self addTitle: 'Destination folder' at: 10@200.
	destinationField := TextField input text: self destination position: 10@220 extent: 240@20.
	self window addChild: destinationField.! !

! LuaStudioExporter methodsFor: #gui !
addExportOptions
	" Private - Add [Export] option "

	| button |
	button := TextField new
		border: true; borderColor: 16r0000FF;
		background: true;  backgroundColor: 16rF0F0FF;
		autoSize: TextFieldAutoSize left;
		selectable: false;
		text: 'EXPORT'; position: 10@250; extent: 500@100;
		yourself.
	self window addChild: button.
	button addEventListener: MouseEvent click with: [:e| self export. nil ].

	progressField := TextField label text: '' position: 10@280 extent: 240@60.
	self window addChild: progressField.! !

! LuaStudioExporter methodsFor: #gui !
addTitle: text at: aPoint
	" Private - Add title "

	self window addChild: (TextField new
		autoSize: TextFieldAutoSize left;
		position: aPoint; width: 240@20;
		text: text; border: false;
		background: true; backgroundColor: 16rF0F0FF;
		selectable: false;
		yourself)! !

! LuaStudioExporter methodsFor: #accessing !
progress: message
	" Set exportation progress "
	
	self progressField text: message! !

! LuaStudioExporter methodsFor: #accessing !
sourceURL
	" Private - Returns the URL of contribution to be exported. "

	^self u8URL
		,self contributionUser
		,$/ ,self contributionId
		,$/ ! !

! LuaStudioExporter methodsFor: #accessing !
templateRoot
	" Returns the local path to template folder "

	^self destination ,self template ,$/ ! !

! LuaStudioExporter methodsFor: #accessing !
contentsRoot
	" Returns the local path to contents folder "

	^self templateRoot ,self linkToContents ! !

! LuaStudioExporter methodsFor: #accessing !
resourcesRoot
	" Returns the local path to resources folder "

	^self contentsRoot ,self linkToResources ! !

! LuaStudioExporter methodsFor: #accessing !
projectRoot
	" Returns the local path to project folder "

	^self resourcesRoot ,'assets/project/' ! !

! LuaStudioExporter methodsFor: #export !
exportURL
	" Private - Returns the URL of Export platform dependent files "

	^self u8URL ,'aleReimondo/m8n/LuaStudio/Export/'! !

! LuaStudioExporter methodsFor: #export !
export
	" Export platform files "

	self	progress: 'Exporting for ' ,self platform;
		unzip: self platform,'.zip' in: self exportURL;
		run: self platform,'.st'! !

! LuaStudioExporter methodsFor: #export !
exportCoreFiles
	" Private - Export m8n core Files "

	| aleReimondo |
	aleReimondo := self u8URL ,'aleReimondo/'.
	self	exportFile: 's8.snapshot.lua' in: aleReimondo ,'m8n/' toPath: self projectRoot ,'m8n/';
		exportFile: 'LuaStudio.st.lua' in: aleReimondo ,'m8n/LuaStudio/' toPath: self projectRoot ,'m8n/LuaStudio/';
		exportFile: 'LuaStudioSamples.st.lua' in: aleReimondo ,'m8n/LuaStudio/' toPath: self projectRoot ,'m8n/LuaStudio/'! !

! LuaStudioExporter methodsFor: #export !
run: scriptFileName
	" Private - Run the script file. "

	(self contentsAtURL: self exportURL ,scriptFileName) stream fileInto: self! !

! LuaStudioExporter methodsFor: #export !
exportAssets
	" Export contribution resources to local assets "

	^self exportAssetsTo: (self projectRoot ,'assets/')! !

! LuaStudioExporter methodsFor: #export !
exportAssetsTo: folderPath
	" Export contribution resources to local assets "

	| resourceNames stream |
	resourceNames := self contentsAtURL: self sourceURL ,'ResourceNames.txt'.
	stream := resourceNames stream.
	[ stream atEnd ] whileFalse: [ | fName |
		fName := (stream nextLine upTo: $;) trimBlanks.
		self exportFile: fName in: self sourceURL toPath: folderPath
	]! !

! LuaStudioExporter methodsFor: #export !
exportFile: fileName in: folderURL toPath: path
	" Export file in server at folderURL to local path "

	^self exportFile: fileName fromUrl: folderURL ,fileName toPath: path! !

! LuaStudioExporter methodsFor: #export !
exportFile: fileName fromUrl: url toPath: path
	" Private - Export url to path "

	| data pathToFile |
	(Sys fileSystem exists: path) ifFalse: [ 
		Sys fileSystem createDirectory: path 
	].
	pathToFile := path ,fileName.
	Http get: url onBytes: [:hBytes|
		"self note: 'Downloading ' ,pathToFile."
		IOFile new
			saveBytes: pathToFile
			bytes: (ByteArray fromBytes: Bytes @ hBytes).
	] onError: [:err|
		# #print: err with: url.
		"^self error: err" ^false
	].
	^true! !

! LuaStudioExporter methodsFor: #export !
exportProject
	" Export the project "

	| path |
	path := self projectRoot.
	(Sys fileSystem exists: path) ifFalse: [ 
		Sys fileSystem createDirectory: path
	].
	IOFile new 
		saveContent: path ,'Main.lua' 
		content: self mainContents! !

! LuaStudioExporter methodsFor: #config !
saveConfigFile
	" Save config file "

	^self saveConfigFile: 'config.xml' in: self resourcesRoot ,'assets/'! !

! LuaStudioExporter methodsFor: #config !
saveConfigFile: fileName in: folder
	" Save config file "

	(Sys fileSystem exists: folder) ifFalse: [ Sys fileSystem createDirectory: folder ].
	IOFile new
		saveContent: folder ,fileName
		content: '<project title="S8 project" startPath="/Main.lua"> </project>'! !

! LuaStudioExporter methodsFor: #accessing !
mainContents
	" Private - Returns the code for Main.lua "

	^'
-- drawbox test prologue
Display = Lib.Media.Display
Capabilities = Lib.Media.Capabilities
stage = Display.stage
stage.scaleX = Capabilities.screenDPI > 120 and Capabilities.screenDPI / 120 or 1
stage.scaleY = stage.scaleX
gfx = stage.graphics
gfx.beginFill(0x008800, 0.3)
gfx.drawRect(120, 60, 120, 320)

local m8n="' ,self projectRoot ,'m8n/"
dofile(m8n.."s8.snapshot.lua")
dofile(m8n.."LuaStudio/LuaStudio.st.lua")
dofile(m8n.."LuaStudio/LuaStudioSamples.st.lua")
--dofile(m8n.."LuaStudio/AssetsPathOverwrite.st.lua")
--dofile(m8n.."LuaStudio/WorkInProgress.st.lua")

code=[[
	Display stage graphics "drawbox test epilogue"
		beginFill: 16r880000 alpha: 0.5;
		drawRect: 60 y: 100 width: 400 height: 80.

"	AssetsPath :=''' ,self projectRoot ,'assets/''.	"
'		,self startupExpression ,'
]]
code:fileIn()'! !

! LuaStudioExporter methodsFor: #expression !
startupExpression
	" Private - Returns the expression to run contribution at startup. "

	| content |
	content := self contentsAtURL: self sourceURL ,'startup.lua'.
	^content stream upTo: 'code=[['; upTo: ']]'! !

! LuaStudioExporter methodsFor: #export !
unzip: fileName in: folderURL
	" Private - Unzip the contents of fileName downloading from folderURL. "
	
	(self exportFile: fileName in: folderURL toPath: self destination) ifTrue: [
		Zip unzip: self destination ,fileName to: self destination.
		Sys fileSystem deleteFile: self destination ,fileName
	]! !

! LuaStudioExporter methodsFor: #templates !
renameTemplates
	" Private - Rename Template mask from all exported contents "

"	| binaryFolder |
	binaryFolder := self destination ,self template ,$/.
	Sys fileSystem
		rename: binaryFolder ,self template ,'.exe' 
		newPath: binaryFolder ,'app.exe'
"! !
