" This code is published under MIT license.
Has been written to run in LuaStudio as demostration on how to write S8 applications that run on Lua VMs.
Read more:
	swiki:s8-media/sampleLuaStudioIndex
	swiki:U8/m8nS8OnLua
"!

"-U8ContributionSample model--------------------------------------------------------------"
Object
	subclass: #U8ContributionSample
	instanceVariableNames: ' userName id '
	category: #LuaStudioSamples !
U8ContributionSample buildAccessors!

! U8ContributionSample class methodsFor: #install !
user: user id: id
	" Returns an instance of the receiver "

	^self number: id of: user! !

! U8ContributionSample class methodsFor: #install !
number: id of: user
	" Returns an instance of the receiver "

	^self new
		userName: user;
		id: id;
		yourself! !

! U8ContributionSample methodsFor: #install !
install
	" Install the receiver "

	self	copyFolder: #assets;
		copyFile: 'Main.lua';
		yourself! !

! U8ContributionSample methodsFor: #accessing !
userID
	" Private - Return the user+ID path "

	^$/ ,self userName
	 ,$/ ,self id asString
	 ,$/! !

! U8ContributionSample methodsFor: #constants !
root
	" Private - Returns the root folder of the receiver. "

	(#(android ios) includes: System systemName) ifTrue: [
		^MediaFile documentsDirectory nativePath ,$/
	].
	^$/! !

! U8ContributionSample methodsFor: #constants !
installationPath
	" Private - Return the installation path (including separator) "

	^self root ,#m8n ,self userID! !

! U8ContributionSample methodsFor: #constants !
url
	" Private - Return U8 service url string "

	^'http://u8.smalltalking.net/u8/profile'
		,self userID! !

! U8ContributionSample methodsFor: #copying !
filesIn: folder
	" Returns anArray with the pathNames of the contents at folder. "

	| stream result root |
	result := #().
	root := $/ ,self id ,$/.
	stream := (Http get: self url ,folder ,$/) stream.
	[stream atEnd] whileFalse: [ | path |
		path := stream upTo: '<A HREF="'; upTo: '">'.
		stream peek = $[ ifFalse: [
			path := path fromLast: root.
			path notEmpty ifTrue: [ result add: path ].
		]
	].
	^result! !

! U8ContributionSample methodsFor: #copying !
copyFolder: folder
	" Copy the contents of (relative) folder. "

	self copyFiles: (self filesIn: folder)! !

! U8ContributionSample methodsFor: #copying !
copyFiles: pathNames
	" Private - Copy the files in pathNames. "

	pathNames do: [:path|
		path last = $/
		ifTrue: [ self copyFolder: (path upToLast: $/) ]
		ifFalse: [ self copyFile: path ]
	]! !

! U8ContributionSample methodsFor: #copying !
copyFile: pathName
	" Copy the contents of file at (relative) pathName. "

	| fileURL filePath dir |
	fileURL := self url ,pathName.
	filePath := self installationPath ,pathName.
	(Sys fileSystem exists: filePath) ifTrue: [
		self note: 'Skipped ' ,filePath
	].
	(filePath includes: $/) ifTrue: [
		dir := filePath upToLast: $/.
		dir notEmpty ifTrue: [ Sys fileSystem createDirectory: dir ].
	].
	Http	get: fileURL
		onBytes: [:hBytes|
			self note: 'Saving ' ,filePath.
			IOFile new
				saveBytes: filePath
				bytes: (ByteArray fromBytes: Bytes @ hBytes).
		]
		onError: [:err| # #print: err with: fileURL ]! !

"(U8ContributionSample number: 516 of: #aleReimondo) install"!

"=ShaderSamples----------------------------------------------------------"
Object
	subclass: #ShaderSample
	instanceVariableNames: ' stage3D context3D program '
	category: #LuaStudioSamples!

! ShaderSample class methodsFor: #sample !
example
	" Run a sample. "

	^self new run! !

! ShaderSample methodsFor: #sample !
run
	" Execute the receiver. "

	stage3D := Display stage stage3Ds first.
	stage3D	addEventListener: Event context3dCreate
			doing: [:e| self contextCreated: e. nil ];
		addEventListener: ErrorEvent error
			doing: [:e| self traceError: e. nil ].
	self requestContext! !

! ShaderSample methodsFor: #private !
requestContext

	"self note: 'About to create context'."
	stage3D requestContext3D: ''! !

! ShaderSample methodsFor: #private !
traceError: e

# #print: #error with: e.
	Lib sys trace: e! !

! ShaderSample methodsFor: #private !
contextCreated: e

	self	captureContext;
		startProgram;
		yourself.! !

! ShaderSample methodsFor: #private !
captureContext

	context3D := stage3D context3D.
	context3D configureBackBuffer: Display stage stageWidth
		height: Display stage stageHeight
		antiAlias: 0 enableDepthAndStencil: false;
		enableErrorChecking: true;
		yourself.
	"self note: #Context3DCreated"! !

! ShaderSample methodsFor: #private !
createProgram
	program := context3D createProgram.! !

! ShaderSample methodsFor: #private !
startProgram
	self	createProgram;
		uploadShaders;
		startRendering! !

! ShaderSample methodsFor: #private !
startRendering

	"self note: #startRendering."
	self createModels.
	context3D setRenderMethod: [:rect| self renderIn: rect ].! !

! ShaderSample methodsFor: #private !
uploadShaders

	"self note: #uploadShaders"! !

! ShaderSample methodsFor: #private !
createModels

	"self note: #createModels"! !

! ShaderSample methodsFor: #private !
renderIn: rect
	" Signal a rendering step. "

	View3D aboutToRender: nil ! !

"--Away3DSample----------------------------------------------------------"!
Object
	subclass: #Away3DSample
	instanceVariableNames:
		 ' stage scene view random '
		,' camera cameraController '
		,' plane planeMaterial '
		,' lightPicker '
	category: #LuaStudioSamples!
Away3DSample comment: '
	http://away3d.com/tutorials/Setting_Up_Your_Scene
'!

! Away3DSample class methodsFor: #sample !
example
	self new run! !

! Away3DSample methodsFor: #run !
run
	" Execute the receiver "

	self	initEngine;
		initLights;
		initMaterials;
		initParticles;
		initObjects;
		initListeners;
		initFPS;
		yourself! !

! Away3DSample methodsFor: #initialize !
initEngine

	"self note: 'Engine start'."
	stage := Display stage.
	random := Random new.
	scene := Scene3D new.
	camera := Camera3D new.
	view := View3D new.
	view
		antiAlias: 4;
		scene: scene;
		camera: camera.
	stage addChild: view.

	cameraController := HoverController
		target: camera lookAt: nil
		panAngle: 0 tiltAngle: 90 distance: 1000
		minTiltAngle: -90 maxTiltAngle: 90
		minPanAngle: nil maxPanAngle: nil
		steps: 8 yFactor: 2
		wrapPanAngle: false.
	cameraController
		distance: 1000;
		minTiltAngle: 0;
		maxTiltAngle: 90;
		panAngle: 45;
		tiltAngle: 20;
		yourself ! !

! Away3DSample methodsFor: #initialize !
initFPS
	stage addChild: (AwayFPS
		xOff: 10 yOff: 10
		color: 16rFFFFFF scale: 3) ! !

! Away3DSample methodsFor: #initialize !
initLights

	"self note: 'Lights start'."! !

! Away3DSample methodsFor: #query !
usePlane
	^true! !

! Away3DSample methodsFor: #constants !
planeMipmap
	^false! !

! Away3DSample methodsFor: #constants !
planeAnisotropy
	^nil! !

! Away3DSample methodsFor: #initialize !
initMaterials

	"self note: 'Materials start'."
	self usePlane ifTrue: [
		planeMaterial := TextureMultiPassMaterial
			texture: BitmapTexture @ 'floor_diffuse.jpg'
			smooth: true repeat: false mipmap: true anisotropy: nil.
		planeMaterial
			specularMap: BitmapTexture @ 'floor_specular.jpg';
			normalMap: BitmapTexture @ 'floor_normal.jpg';
			repeat: true; mipmap: self planeMipmap;
			specular: 10.
		self planeAnisotropy notNil ifTrue: [
			planeMaterial anisotropy: self planeAnisotropy
		].
		lightPicker notNil ifTrue: [
			planeMaterial lightPicker: lightPicker
		].
		"self note: 'PlaneMaterial created'."
	].! !

! Away3DSample methodsFor: #initialize !
initParticles

	"self note: 'Particles start'." ! !

! Away3DSample methodsFor: #constants !
planeSize

	^1000@1000 ! !

! Away3DSample methodsFor: #constants !
scaleUV

	^2@2 ! !

! Away3DSample methodsFor: #initialize !
initObjects

	"self note: 'Objects start'."
	self usePlane ifTrue: [
		plane := Mesh
			geometry: (PlaneGeometry
				width: self planeSize x height: self planeSize y
				segmentsW: 1 segmentsH: 1
				yUp: true doubleSided: false)
			material: planeMaterial.
		plane geometry scaleUV: self scaleUV x scaleV: self scaleUV y.
		plane y: -20.
		scene addChild: plane.
		"self note: 'Plane created'."
	].! !

! Away3DSample methodsFor: #render !
renderStep: rect moving: moving
	" Private - Render step "

"	cameraController notNil ifTrue: [
		self note: 'pan=' ,cameraController panAngle ,' tilt=' ,cameraController tiltAngle.
	]."! !

! Away3DSample methodsFor: #initialize !
initListeners

	| moving onResize camMove
	 lastPanAngle lastTiltAngle lastMouseX lastMouseY |
	"self note: 'Listeners start'."
	lastPanAngle
		:= lastTiltAngle
		:= lastMouseX
		:= lastMouseY
		:= 0.
	moving := false.
	camMove := [:e| moving ifTrue: [
		cameraController
			panAngle: stage mouseX - lastMouseX * 0.3 + lastPanAngle;
			tiltAngle:   stage mouseY - lastMouseY * 0.3 + lastTiltAngle
		]. nil
	].
	view setRenderCallback: [:rect |
		camMove value: nil.
		self renderStep: rect moving: moving.
		view render.
	].

	stage addEventListener: MouseEvent mouseDown doing: [:e|
		lastPanAngle := cameraController panAngle.
		lastTiltAngle := cameraController tiltAngle.
		lastMouseX := stage mouseX.
		lastMouseY := stage mouseY.
		moving := true. nil
	].
	stage addEventListener: MouseEvent mouseMove doing: camMove.
	stage addEventListener: MouseEvent mouseUp doing: [:e|
		moving := false. nil
	].

	onResize := [:e|
		view width: stage stageWidth.
		view height: stage stageHeight.
		nil
	].

	stage addEventListener: Event resize doing: onResize.
	onResize value.

	"self note: 'Listeners ON'."! !

"-View--------------------------------------------------------------------------------"
Away3DSample subclass: #ViewSample!

! ViewSample methodsFor: #initialize !
initEngine

	super initEngine.
	camera
		z: -1000;	y: 300;
		lookAt: Vector3D zero upAxis: nil;
		yourself. ! !

! ViewSample methodsFor: #render !
renderStep: rect moving: moving
	" Private - Render step "

	super renderStep: rect moving: moving.
	plane rotationY: plane rotationY + 1! !

"-Particles--------------------------------------------------------------------------------"
Away3DSample
	subclass: #ParticlesSample
	instanceVariableNames:
		 ' geometrySet particleAnimationSet '
		,' material particleAnimator particleMesh '!
ParticlesSample comment: '
	http://away3d.com/tutorials/Introduction_to_Particles
'!

! ParticlesSample methodsFor: #initialize !
initEngine

	"self note: 'Engine start'."
	stage := Display stage.
	random := Random new.
	view := View3D new.
	camera := view camera.
	stage addChild: view.
	scene := view scene.

	cameraController := HoverController
		target: camera lookAt: nil
		panAngle: 45 tiltAngle: 2 distance: 1000
		minTiltAngle: -90 maxTiltAngle: 90
		minPanAngle: nil maxPanAngle: nil
		steps: 8 yFactor: 2
		wrapPanAngle: false. ! !

! ParticlesSample methodsFor: #query !
usePlane
	^false! !

! ParticlesSample methodsFor: #initialize !
initObjects

	"self note: 'Objects start'."
	plane := PlaneGeometry
		width: 10 height: 10
		segmentsW: 1 segmentsH: 1
		yUp: false doubleSided: false.
	"self note: 'Plane created'."
	geometrySet := Array new.
	1 to: 20000 do: [:i| geometrySet at: i put: plane ].

	particleAnimationSet := ParticleAnimationSet
		usesDuration: true usesLooping: true usesDelay: false.
	particleAnimationSet
		addAnimation: ParticleBillboardNode new;
		addAnimation: (ParticleVelocityNode
			mode: ParticlePropertiesMode localStatic
			velocity: nil);
		initParticleFunc: [:prop| | degree1 degree2 r k v |
			(ParticleAnimationData @ prop)
				startTime: random next * 5 - 5;
				duration: 5.
			degree1 := random next * Number pi.
			degree2 := random next * Number pi * 2.
			r := random next * 50 + 400.
			v := Vector3D
				x: r * degree1 sin * degree2 cos
				y: r * degree1 cos * degree2 cos
				z: r * degree2 sin w: 0.
			k := ParticleVelocityNode velocityVector3D.
			{' prop.nodes.set(k,v:handle()) '}.
		].

	material :=  TextureMaterial
		texture: BitmapTexture @ 'blue.png'
		smooth: true repeat: false mipmap: true anisotropy: nil.
	material blendMode: BlendMode add.

	particleAnimator :=  ParticleAnimator new: particleAnimationSet.
	particleMesh := Mesh
		geometry: (ParticleGeometry geometries: geometrySet)
		material: material.
	particleMesh animator: particleAnimator.
	scene addChild: particleMesh.

	particleAnimator start.
	"self note: 'Objects ON'."! !

"-Fire------------------------------------------------------------------------------------"
Away3DSample
	subclass: #FireSample
	instanceVariableNames:
		 ' directionalLight timer '
		,' particleMaterial particleGeometry '
		,' fireAnimationSet fireObjects '!
Object
	subclass: #FireVO
	instanceVariableNames: 'mesh animator light strength'
	category: #LuaStudioSamples!
FireVO buildAccessors!

! FireVO class methodsFor: #instantiation !
mesh: aMesh animator: anAnimator
	^self new
		mesh: aMesh;
		animator: anAnimator;
		strength: 0;
		yourself! !

! FireSample methodsFor: #initialize !
initLights

	super initLights.
	directionalLight := DirectionalLight xDir: 0 yDir: -1 zDir: 1.
	directionalLight
		castsShadows: false;
		color: 16rEEDDDD; diffuse: 0.5;
		ambient: 0.5; specular: 0;
		ambientColor: 16r808090.
	scene addChild: directionalLight.
	lightPicker := StaticLightPicker lights: (Array with: directionalLight).
	"self note: 'Lights ON'."! !

! FireSample methodsFor: #initialize !
initMaterials

	super initMaterials.
	particleMaterial :=  TextureMaterial
		texture: BitmapTexture @ 'blue.png'
		smooth: true repeat: false mipmap: true anisotropy: nil.
	particleMaterial blendMode: BlendMode add.
	"self note: 'Materials ON'."! !

! FireSample methodsFor: #initialize !
initParticles

	"self note: 'Particles start'."
	self	initAnimationSet;
		initGeometrySet;
		yourself.
	"self note: 'Particles ON '."! !

! FireSample methodsFor: #initialize !
initGeometrySet

	| particle geometrySet |
	"self note: 'GeometrySet start'."
	particle := PlaneGeometry
		width: 10 height: 10
		segmentsW: 1 segmentsH: 1
		yUp: false doubleSided: false.
	geometrySet := Array new.
	1 to: 500 do: [:i| geometrySet at: i put: particle ].
	particleGeometry := ParticleGeometry geometries: geometrySet.
	"self note: 'GeometrySet ON '."! !

! FireSample methodsFor: #initialize !
initAnimationSet

	fireAnimationSet := ParticleAnimationSet
		usesDuration: true usesLooping: true usesDelay: false.

	fireAnimationSet
		addAnimation: ParticleBillboardNode new;
		addAnimation: (ParticleScaleNode
			mode: ParticlePropertiesMode global
			usesCycle: false usesPhase: false
			minScale: 2.5 maxScale: 0.5
			cycleDuration: 1 cyclePhase: 0);
		addAnimation: (ParticleVelocityNode
			mode: ParticlePropertiesMode global
			velocity: Vector3D @ #(0 80 0 0));
		addAnimation: (ParticleColorNode
			mode: ParticlePropertiesMode global
			usesMultiplier: true usesOffset: true
			usesCycle: false usesPhase: false
			startColor: ColorTransform @ #(0 0 0 1 16rFF 16r33 1 0)
			endColor: ColorTransform @ #(0 0 0 1 16r99 0 0 0)
			cycleDuration: 1 cyclePhase: 0);
		addAnimation: (ParticleVelocityNode
			mode: ParticlePropertiesMode localStatic
			velocity: nil);
		initParticleFunc: [:prop| | degree1 degree2 r v k |
			(ParticleAnimationData @ prop)
				startTime: random next * 5;
				duration: random next * 4 + 0.1.
			degree1 := random next * Number pi * 2.
			degree2 := random next * Number pi * 2.
			r := 15.
			v := Vector3D
				x: r * degree1 sin * degree2 cos
				y: r * degree1 cos * degree2 cos
				z: r * degree2 sin w: 0.
			k := ParticleVelocityNode velocityVector3D.
			{' prop.nodes.set(k,v:handle()) '}.
		];
		yourself.! !

! FireSample methodsFor: #initialize !
initFires

	| numFires |
	numFires := 10.
	"self note: #initFires."
	fireObjects := Array new.
	1 to: numFires do: [:i| | particleMesh animator degree |
		particleMesh := Mesh geometry: particleGeometry material: particleMaterial.
		animator :=  ParticleAnimator new: fireAnimationSet.
		particleMesh animator: animator.
		degree := i - 1 / numFires * Number pi * 2.
		particleMesh
			x: degree sin * 400;
			z: degree cos * 400;
			y: 5.
		fireObjects at: i put: (FireVO mesh: particleMesh animator: animator).
		scene addChild: particleMesh.
	].
	"self note: 'Fires ON'."! !

! FireSample methodsFor: #initialize !
initTimer

	"self note: #initTimer."
	timer := Timer delay: 1000 repeatCount: fireObjects size.
	timer addEventListener: TimerEvent timer doing: [:e|
		| fireObject light lights |
		fireObject := fireObjects at: timer currentCount.
		fireObject animator start.
		light := PointLight new.
		light
			color: 16rFF3301;
			diffuse: 0; specular: 0;
			position: fireObject mesh position;
			yourself.
		fireObject light: light.
		lights := Array with: directionalLight.
		lights addAll: ((fireObjects
			collect: [:one| one light ])
			select:  [:each| each notNil]).
		 lightPicker lights: lights.
		nil
	].
	timer start.
	"self note: 'Timer ON'."! !

! FireSample methodsFor: #initialize !
initObjects

	super initObjects.
	self initFires.
	self initTimer.
	"self note: 'Objects ON'."! !

! FireSample methodsFor: #render !
renderStep: rect moving: moving
	" Private - Render step "

	super renderStep: rect moving: moving.
	fireObjects do: [:each|
		each light notNil ifTrue: [
			each strength < 1 ifTrue: [
				each strength: each strength + 0.1
			].
			each light
				fallOff: random next * 20 + 380;
				radius: random next * 30 + 200;
				diffuse: random next * 0.2 + each strength;
				specular: each light diffuse
		].
	].! !

"-SkyBox--------------------------------------------------------------------------------"
Away3DSample
	subclass: #SkyBoxSample
	instanceVariableNames:
		 ' cubeTexture material '
		,' torus skyBox '!
SkyBoxSample comment: '
	A skybox is used to provide a 3D background image for distant objects such as landscapes, horizons, sky & clouds, star-fields or other unreachbale objects. These distant objects are represented as a cube and the images are mapped on to the inside surface of that cube with the camera viewpoint being in the centre of the skybox cube.
	http://away3d.com/tutorials/Using_A_Skybox
'!

! SkyBoxSample methodsFor: #initialize !
initEngine

	super initEngine.
	camera
		z: -600;	y: 0;
		lookAt: Vector3D zero upAxis: nil;
		lens: (PerspectiveLens new: 90)
		yourself.
	"self note: 'Engine OK'."! !

! SkyBoxSample methodsFor: #query !
usePlane
	^false! !

! SkyBoxSample methodsFor: #initialize !
initMaterials

	"self note: 'Materials start'."
	cubeTexture := BitmapCubeTexture @ #(
		'snow_positive_x.jpg'
		'snow_negative_x.jpg'
		'snow_positive_y.jpg'
		'snow_negative_y.jpg'
		'snow_positive_z.jpg'
		'snow_negative_z.jpg'
	).
	material := ColorMaterial color: 16rFFFFFF alpha: 1.
	material
		specular: 0.5;	ambient: 0.25;
		ambientColor: 16r111199;
		ambient: 1;
		addMethod: (EnvMapMethod envMap: cubeTexture alpha: 1);
		yourself.
	"self note: 'Materials OK'."! !

! SkyBoxSample methodsFor: #initialize !
initObjects

	"self note: 'Objects start'."
	torus := Mesh geometry: (TorusGeometry
			radius: 150 tubeRadius: 60
			segmentsR: 40 segmentsT: 20
			yUp: true)
		material: material.
	scene addChild: torus.
	"self note: 'Torus OK'."
	skyBox := SkyBox cubeMap: cubeTexture.
	scene addChild: skyBox.
	"self note: 'SkyBox OK'."! !

! SkyBoxSample methodsFor: #render !
renderStep: rect moving: moving
	" Private - Render step "

	super renderStep: rect moving: moving.
	torus
		rotationX: torus rotationX + 2;
		rotationY: torus rotationY + 1.
	camera
		position: Vector3D zero;
		rotationY: stage mouseX - stage stageWidth / 2 * 0.5  / 800 + camera rotationY;
		moveBackward: 600.! !

"-Shading--------------------------------------------------------------------------------"
Away3DSample
	subclass: #ShadingSample
	instanceVariableNames:
		 ' light1 light2 '
		,' sphereMaterial sphere '
		,' cubeMaterial cube '
		,' torusMaterial torus '!
ShadingSample comment: '
	http://away3d.com/tutorials/Introduction_to_Materials
'!

! ShadingSample methodsFor: #initialize !
initLights

	super initLights.
	light1 := DirectionalLight xDir: 0 yDir: -1 zDir: 1.
	light1
		direction: Vector3D @ #( 0 -1 0 0 );
		ambient: 0.1; diffuse: 0.7;
		yourself.
	scene addChild: light1.
	light2 := DirectionalLight xDir: 0 yDir: -1 zDir: 1.
	light2
		direction: Vector3D @ #( 0 -1 0 0 );
		ambient: 0.1; diffuse: 0.7;
		yourself."
		castsShadows: false;
		color: 16rEEDDDD; diffuse: 0.5;
		ambient: 0.5; specular: 0;
		ambientColor: 16r808090."
	scene addChild: light2.
	lightPicker := StaticLightPicker lights: (Array with: light1 with: light2).
	"self note: 'Lights ON'."! !

! ShadingSample methodsFor: #render !
renderStep: rect moving: moving
	" Private - Render step "

	| k dir max |
	super renderStep: rect moving: moving.
	max := 150000.
	k := Lib sys getTime / 10000.
	dir := #( 0 1000 0 0).
	dir at: 1 put: k sin * max.
	dir at: 3 put: k cos * max.
	light1 direction: Vector3D @ dir! !

! ShadingSample methodsFor: #initialize !
initMaterials

	| weaveDiffuseTexture |
	super initMaterials.

	sphereMaterial :=  TextureMaterial
		texture: BitmapTexture @ 'beachball_diffuse.jpg'
		smooth: true repeat: false mipmap: true anisotropy: nil.
	sphereMaterial
		specularMap: BitmapTexture @ 'beachball_specular.jpg';
		lightPicker: lightPicker.

	cubeMaterial :=  TextureMaterial
		texture: BitmapTexture @ 'trinket_diffuse.jpg'
		smooth: true repeat: false mipmap: true anisotropy: nil.
	cubeMaterial
		specularMap: BitmapTexture @ 'trinket_specular.jpg';
		normalMap: BitmapTexture @ 'trinket_normal.jpg';
		lightPicker: lightPicker;
		mipmap: false.

	weaveDiffuseTexture := BitmapTexture @ 'weave_diffuse.jpg'.
	torusMaterial :=  TextureMaterial
		texture: weaveDiffuseTexture
		smooth: true repeat: false mipmap: true anisotropy: nil.
	torusMaterial
		specularMap: weaveDiffuseTexture;
		normalMap: BitmapTexture @ 'weave_normal.jpg';
		lightPicker: lightPicker;
		repeat: true.

	"self note: 'Materials ON'."! !

! ShadingSample methodsFor: #initialize !
initObjects

	super initObjects.

	sphere := Mesh
		geometry: (SphereGeometry
			radius: 150
			segmentsW: 40 segmentsH: 20
			yUp: true)
		material: sphereMaterial.
	sphere x: 300 y: 160 z: 300.
	scene addChild: sphere.

	cube := Mesh
		geometry: (CubeGeometry
			width: 200 height: 200 depth: 200
			segmentsW: 1 segmentsH: 1 segmentsD: 1
			tile6: false)
		material: cubeMaterial.
	cube x: 300 y: 160 z: -250.
	scene addChild: cube.

	torus := Mesh
		geometry: (TorusGeometry
			radius: 150 tubeRadius: 60
			segmentsR: 40 segmentsT: 20
			yUp: true)
		material: torusMaterial.
	torus geometry scaleUV: 10 scaleV: 5.
	torus x: -250 y: 160 z: -250.
	scene addChild: torus.

	"self note: 'Objects ON'."! !

"-MipMapping-----------------------------------------------------------------------------"
Away3DSample
	subclass: #MipMapping
	instanceVariableNames: ' lastTime text '!

! MipMapping methodsFor: #initialize !
initEngine

	super initEngine.
	camera
		z: 0; y: 25;
		lookAt: Vector3D @ #(0 -500 2000 0) ! !

! MipMapping methodsFor: #constants !
planeSize

	^20000@20000 ! !

! MipMapping methodsFor: #constants !
scaleUV

	^150@150 ! !

! MipMapping methodsFor: #constants !
planeMipmap
	^false! !

! MipMapping methodsFor: #constants !
planeAnisotropy
	^Anisotropy none! !

! MipMapping methodsFor: #constants !
anisotropies
	^#(	none
		anisotropic2x anisotropic4x
		anisotropic8x anisotropic16x
	)! !

! MipMapping methodsFor: #accessing !
details

	^'mip:' ,planeMaterial mipmap
	,' ' ,self anisotropyString! !

! MipMapping methodsFor: #accessing !
anisotropyString

	| value |
	value := planeMaterial anisotropy.
	self anisotropies do: [:constant|
		(Anisotropy perform: constant) = value
			ifTrue: [ ^constant ]
	].
	^#unknown! !

! MipMapping methodsFor: #initialize !
initMaterials

	"self note: 'Materials start'."
	self usePlane ifTrue: [
		planeMaterial := TextureMaterial
			texture: BitmapTexture @ 'floor_diffuse.jpg'
			smooth: true repeat: false mipmap: true
			anisotropy: nil.
		planeMaterial repeat: true; mipmap: self planeMipmap.
		self planeAnisotropy notNil ifTrue: [
			planeMaterial anisotropy: self planeAnisotropy
		].
		lightPicker notNil ifTrue: [
			planeMaterial lightPicker: lightPicker
		].
		"self note: 'PlaneMaterial created'."
	].! !

! MipMapping methodsFor: #render !
renderStep: rect moving: moving
	" Private - Render step "

	| t delta |
	super renderStep: rect moving: moving.
	t := Lib sys getTime.
	lastTime isNil ifTrue: [ lastTime := t ].
	delta := t - lastTime.
	plane rotationY: delta / 50 + plane rotationY.
	lastTime := t! !

! MipMapping methodsFor: #initialize !
initObjects

	| clickText |
	super initObjects.

	clickText := TextField new.
	clickText
		x: 200; y: 50;
		defaultTextFormat: (TextFormat
			font: '_sans' size: 32
			color: 16rFFFFFF
			bold: nil italic: nil underline: nil);
		width: stage stageWidth - 200;
		text: 'Click anywhere to change'.
	stage addChild: clickText.

	text := TextField new.
	text	x: 300; y: 25;
		defaultTextFormat: (TextFormat
			font: '_sans' size: 24
			color: 16r4422FF
			bold: nil italic: nil underline: nil);
		width: stage stageWidth - 200;
		text: 'No Mip-mapping'.
	stage addChild: text.! !	

! MipMapping methodsFor: #initialize !
initListeners

	super initListeners.

	stage addEventListener: MouseEvent click doing: [:e|
		| mip ani all |
		all := self anisotropies.
		mip := planeMaterial mipmap.
		ani := self anisotropyString.
		mip ifTrue: [
			ani = all last
			ifTrue: [ ani := all first. mip := false ]
			ifFalse: [ ani := all at: (all indexOf: ani) + 1 ].
		] ifFalse: [ mip := true ].
"# #print: mip with: ani with: (Anisotropy perform: ani) with: 'next=',(ani=all last ifTrue: [#none] ifFalse: [all at: (all indexOf: ani) + 1])."
		planeMaterial mipmap: mip.
		planeMaterial anisotropy: (Anisotropy perform: ani).
		text text: self details.
		nil
	].
	"self note: 'Click ON'."! !

"----------------------------------------------------------------------------------------"!
! Accelerometer class methodsFor: #samples !
example
	" Dump accelerometer response "

	| fmt text sample |
	fmt := TextFormat
		font: '_sans' size: 30 color: 16r660000
		bold: false italic: false underline: false.
	text := TextField new.
	text
		defaultTextFormat: fmt;
		autoSize: TextFieldAutoSize left;
		x: 10; y: 10; width: 400;
		wordWrap:true.
	Display stage addChild: text.
	Accelerometer isSupported ifFalse: [
		^self print: 'Accelerometer feature not supported'
	].

	sample := Accelerometer new.
	sample
		addEventListener: AccelerometerEvent update
		doing: [:e| text text:
			'orientation: ' ,Display stage getOrientation ,'
	#',e#timestamp,'
	x=',e#accelerationX,'
	y=',e#accelerationY,'
	z=',e#accelerationZ.
	nil
].! !

! Bitmap class methodsFor: #samples !
cacheAsBitmap
	" Cache as bitmap, example "

	| circle stage gfx colours alphas mtx focal shape bmp bitmap combined matrix ratios f strip |
	stage  := Display stage.
	circle := Display sprite new.
	gfx     := circle graphics.
	colours := #(16rFF0000 16r000000).
	alphas := #(1 1).
	ratios := #(0 255).
	mtx := Matrix a: 1 b: 0 c: 0 d: 1 tx: 0 ty: 0.
	focal := -0.9.
	mtx createGradientBox: 100 height: 100 rotation: 0 tx: 0 ty: 0.
	gfx	beginGradientFill: GradientType radial
		colors: colours alphas: alphas ratios: ratios 
		matrix: mtx spreadMethod: SpreadMethod repeat 
		interpolationMethod: InterpolationMethod linearRgb
		focalPointRatio: focal.
	gfx drawRect: 0 y: 0 width: 100 height: 100.
	stage addChild: circle.

	circle cacheAsBitmap: true; x: 200; y: 200.
	f := DropShadowFilter
		distance: "4"8 angle: 45 
		color: 0 alpha: "1"0.75 blurX: 4 blurY: 4
		strength: 1 quality: 1
		inner: false knockout: false
		hideObject: false.
	circle filters: (Array with: f).

	shape := Shape new.
	gfx := shape graphics.
	gfx	lineStyle: 3 color: 16r0000FF alpha: 1 
		pixelHinting: false scaleMode: nil 
		caps: nil joints: nil miterLimit: 3.
	gfx moveTo:  5 y:  5.
	gfx lineTo: 25 y: 25.
	bmp := BitmapData inWidth: 32 inHeight: 32 
		inTransparent: true fillColor: 0 
		inGPUMode: nil.
	bmp	draw: shape matrix: nil colorTransform: nil
		blendMode: nil clipRect: nil smoothing: false.
	bitmap := Bitmap
		bitmapData: bmp pixelSnapping: nil
		smoothing: false.
	bitmap x: 50; y: 50. "translation of small blue box \ "
	stage addChild: bitmap.

	combined := BitmapData
		inWidth: 200 inHeight: 200 
		inTransparent: true fillColor: 0 
		inGPUMode: nil.
	matrix := Matrix a: 1 b: 0 c: 0 d: 1 tx: 0 ty: 0.
	0 to: 4 do: [:x |
		0 to: 4 do: [:y |
			matrix tx: x * 20; ty: y * 20.
			combined draw: bmp matrix: matrix 
				colorTransform: nil 
				blendMode: nil clipRect: nil 
				smoothing: false.
		]
	].
	bitmap := Bitmap 
		bitmapData: combined 
		pixelSnapping: nil
		smoothing: false.
	bitmap x: 150; y: 50. "translation of blue \\\\ box"
	stage addChild: bitmap.

	strip := self createStrip: InterpolationMethod linearRgb.
	strip x: 20; y: 300.
	stage addChild: strip.

	strip := self createStrip: InterpolationMethod rgb.
	strip x: 20; y: 400.
	stage addChild: strip.

	stage addEventListener: Event enterFrame with: [:e |
		circle rotation: circle rotation + 1. nil
	]! !

! Bitmap class methodsFor: #samples !
createStrip: inMethod
	" Private - Create strip of gradient sample. "
	
	| shape gfx mtx colours alphas ratios |
	shape := Display sprite new.
	gfx := shape graphics.
	mtx := Matrix a: 1 b: 0 c: 0 d: 1 tx: 0 ty: 0.
	mtx createGradientBox: 250 height: 50 rotation: 0 tx: 0 ty: 0.
	colours := #( 16rFF0000 16r00FF00 16r0000FF ).
	alphas  := #( 1 1 1 ).
	ratios   := #( 0 128 255 ).
	gfx beginGradientFill: GradientType linear
		colors: colours alphas: alphas ratios: ratios 
		matrix: mtx spreadMethod: SpreadMethod repeat 
		interpolationMethod: inMethod
		focalPointRatio: -0.9.
	gfx drawRect: 0 y: 0 width: 250 height: 50.
	shape addEventListener: MouseEvent click with: [:e |
		# #print: #click. nil
	].
	^shape! !

! Clipboard class methodsFor: #samples !
example
	" Clipboard example "
	
	| stage screenDpi sx input clickTxt clipboard paste clickTxt2 |
	stage := Display stage.
	screenDpi := Capabilities screenDPI. 
	sx := screenDpi > 120 ifTrue: [ screenDpi / 120 ] ifFalse: [ 1 ].
	stage scaleX: sx.
	stage scaleY: sx.
	input := Text textField new
		x: 10; y: 10;
		type: TextFieldType input;
		wordWrap: true; multiline: true;
		width: 240; height: 100;
		text: 'Text for clipboard';
		border: true; borderColor: 0;
		background: true; 
		backgroundColor: 4294967295;
		yourself.

	stage addChild: input.
	clickTxt := Text textField new
		autoSize: TextFieldAutoSize left;
		x: 10; y: 120;
		width: 240; height: 100;
		text: 'Click to copy';
		border: true; borderColor: 255;
		background: true; 
		backgroundColor: 15790335;
		selectable: false;
		yourself.

	stage addChild: clickTxt.
	clipboard := Clipboard generalClipboard.
	clickTxt addEventListener: MouseEvent click with: [:e |
		clipboard
			setData: ClipboardFormats textFormat
			data: input text serializable: true.
		nil
	].
	paste := Text textField new
		x: 10; y: 150;
		wordWrap: true; multiline: true;
		width: 240; height: 100;
		border: true; borderColor: 0; 
		background: true;
		backgroundColor: 16777215;
		yourself.
	stage addChild: paste.
	clickTxt2 := Text textField new
		autoSize: TextFieldAutoSize left;
		x: 10; y: 260;
		width: 240; height: 100;
		text: 'Click to paste';
		border: true; borderColor: 255;
		background: true; 
		backgroundColor: 15790335;
		selectable: false;
		yourself.
	stage addChild: clickTxt2.
	clickTxt2 addEventListener: MouseEvent click with: [:e |
		(clipboard hasFormat: ClipboardFormats textFormat) ifTrue: [
			paste text: (clipboard getData: ClipboardFormats textFormat transferMode: nil)
		].
		nil
	]! !

! Crypto class methodsFor: #samples !
example
	" Encode/decode test "

	self print: (Crypto base64 encode: (Bytes ofString: #test) complement: true).
	self print: (Crypto base64 decode: 'dGVzdA==' complement: true) toString.
	#(Md5 Sha1 Sha224 Sha256) do: [:each|
		self print: each ,' ' ,((Smalltalk at: each) new encode: #test)
	]! !

! EReg class methodsFor: #samples !
example
	" Regular expressions example. "

	| expr text |
	expr := EReg with: '("(?:\\.|[^\\\n"])*")'.
	text := '123"test1" 456 "test2"end'.
	(expr matchSub: text pos: 0 len: text size) ifTrue: [ | pos len |
		pos := expr matchedPos pos.
		len := expr matchedPos len.
		self print: 'Match ' ,pos ,' ' ,len ,' value=' ,(expr matched: 0).
		(expr matchSub: text pos: pos + len len: text size - (pos + len)) ifTrue: [
			self print: 'match again ' ,(expr matched: 0)
		]
	]! !

"-Http--------------------------------------------------------------------------"!
! Http class methodsFor: #samples !
example
	" Print the result string of a simple request "

	self print: (self requestUrl: 'https://www.w3.org/')! !

! Http class methodsFor: #samples !
example2
	" Get data and bytes from a request. "

	| url |
	url := 'https://www.w3.org/'.
	(Http @ url)
		"note: 'Configure Http instance';"
		onData: [:data| # #print: 'data=' ,data ];
		onError: [:msg| # #print: 'Error=' ,msg ];
		onStatus: [:code| # #print: 'status code: ',code ];
		onBytesData: [:hBytes| | bytes |
			bytes := Bytes @ hBytes.
			# #print: 'bytes length: ',bytes length.
			# #print: 'first byte: ',(bytes get: 0).
		];
		note: 'Sending http request to ' ,url;
		request: false;
		yourself.
	self print: #done.! !

! Http class methodsFor: #sample !
showU8Logo
	" Show an image reading bytes from U8 service. "

	self	get: 'http://u8.smalltalking.net/imgs/u8logo3.png'
		onBytes: [:hBytes| | bytes data bmp half |
			bytes := ByteArray fromBytes: Bytes @ hBytes.
			data := BitmapData bytes: bytes rawAlpha: nil.
			half := Display stage stageExtent - data extent / 2 max: 0@0.
			bmp := (Bitmap 
				bitmapData: data 
				pixelSnapping: PixelSnapping auto
				smoothing: false)
				x: half x; y: half y;
				yourself.
			Display stage addChild: bmp.
		]
		onError: [:err| self error: err ]! !

! Json class methodsFor: #samples !
example
	" Parse example. "

	| json |
	json := Json parse: '{
	"glossary": {
		"title": "example glossary",
		"GlossDiv": {
			"title": "S",
			"GlossList": {
				"GlossEntry": {
					"ID": "SGML",
					"SortAs": "SGML",
					"GlossTerm": "Standard Generalized Markup Language",
					"Acronym": "SGML",
					"Abbrev": "ISO 8879:1986",
					"GlossDef": {
		"para": "A meta-markup language, used to create markup languages such as DocBook.",
		"GlossSeeAlso": ["GML", "XML"]
					},
					"GlossSee": "markup"
				}
			}
		}
	}
}'.	json isNil ifTrue: [ ^self print: 'Parse result was nil' ].
	self print: 'title=', (json @@ 'glossary.title').
	self print: (json @@ 'glossary.GlossDiv.GlossList.GlossEntry.GlossDef.GlossSeeAlso' basicAt: 1) toString.
	self print: (Json stringify: json).! !

! SQLight class methodsFor: #samples !
example
	" SQL memory table example. "

	((SQLight connection new: ':memory:')
		note: 'creating table';
		request: 'CREATE TABLE IF NOT EXISTS Test ( field1 INTEGER PRIMARY KEY NOT NULL, field2 TEXT, field3 BLOB )';
		note: 'filling the table';
		request: 'INSERT INTO Test (field1, field2) VALUES (?, ?)' with: #(1 test1);
		request: 'INSERT INTO Test (field1, field2) VALUES (?, ?)' with: #(2 test2);
		note: 'walking the table';
		request: 'SELECT field1, field2 FROM Test') do: [:tuple|
			self print: Lua @> tuple
		]! !

! SSL class methodsFor: #samples !
example
	" talk.google.com example. "

	| socket bufSize buf pos len |
	socket := SSL new.
	socket
		note: 'Connecting to talk.google.com';
		connect: Host @ 'talk.google.com' port: 5223;
		note: 'Connected. Writing data...';
		write: '<?xml version="1.0" encoding="UTF-8"?><stream:stream xmlns:stream="http://etherx.jabber.org/streams" xmlns="jabber:client" to="talk.google.com" xml:lang="en" version="1.0">';
		note: 'Waiting for incoming XMPP stream...'.
	bufSize := 32.
	buf := Bytes new: 4096.
	pos := 0. len := -1.
	[ len := socket input readBytes: buf pos: pos len: bufSize.
	  len < bufSize ] whileFalse: [ pos := pos + len ].
	self note:  'Recieved xmpp stream: ' ,buf toString.
	socket
		note: 'Finishing connection';
		write: '</stream>';
		close! !

! Sprite class methodsFor: #samples !
example
	" Sprite example "

	| sprite |
	sprite := Sprite new
		name: 'Sprite 1';
		buttonMode: true;
		yourself.
	sprite graphics
		lineStyle: 2 color: 16r00FF00 alpha: 1.0 pixelHinting: false scaleMode: nil caps: nil joints: nil miterLimit: 3;
		beginFill: 16rFFFF00 alpha: 1.0;
		drawRect: 0 y: 0 width: 200 height: 100;
		endFill.
	Display stage addChild: sprite.

	sprite	x: 10;y: 10;
		addEventListener: MouseEvent click
		with: [:e| sprite print: 'Click on ', e#target#name. nil ]! !

! Sprite class methodsFor: #samples !
example2
	" Sprite example "
	" I.N.: handler listener on sprite object 
	is not working "
		
	| sprite stage graphics |
	sprite := Display sprite new.
	stage := Display stage.
	sprite name: 'Sprite 1'.
	graphics := sprite graphics.
	graphics lineStyle: 2 color: 65280 alpha: 1.0 
		pixelHinting: false scaleMode: nil 
		caps: nil joints: nil miterLimit: 3.
	graphics beginFill: 16776960 alpha: 1.0.
	graphics drawRect: 0 y: 0 width: 200 height: 100.
	graphics endFill.
	sprite buttonMode: true.
	stage addChild: sprite.
	sprite x: 10; y: 10.
	sprite	addEventListener: MouseEvent click
		with: [:e | # #print: e#target#name. nil ]! !

! Timer class methodsFor: #samples !
example
	" Simple timer example "

	| startTime timer i |
	startTime := Lib sys getTime.
	i := 1.
	timer := Timer delay: 500 repeatCount: 10.
	timer	addEventListener: TimerEvent timer
		doing: [:e| 
			self print: 'timer: ', i.
			i = 10 ifTrue: [
				timer stop.
				self print: 'Finished ', (Lib sys getTime - startTime).
			].
			i := i + 1. nil
		].
	timer start! !

! Bitmap class methodsFor: #samples !
trianglesExample
	" Triangles example "
	
	| s0 s1 s2 s3 t0 data stage doUpdate |
	stage := Display stage.
	s0 := Display sprite new.
	s1 := Display sprite new.
	s2 := Display sprite new.
	s3 := Display sprite new.
	stage addChild: s0.
	stage addChild: s1.
	stage addChild: s2.
	stage addChild: s3.
	s0 scaleX: 0.5; scaleY: 0.5.
	s1 scaleX: 0.5; scaleY: 0.5.
	s2 scaleX: 0.5; scaleY: 0.5.
	s3 scaleX: 0.5; scaleY: 0.5.
	s1 x: 550 / 2.
	s3 x: s1 x.
	s2 y: 400 / 2.
	s3 y: s2 y.
	t0 := Lib sys getTime / 1000.
	data := BitmapData @ 'Image.jpg'.

	doUpdate := [ | sx sy theta sin cos z w0 w1 x0 y0 vertices indices tex_uvt tex_uv cols gfx |
		sx := 1 / data width.
		sy := 1 / data height.
		theta := (Lib sys getTime / 1000) - t0.
		cos := theta cos.
		sin := theta sin.
		z := sin * 100. 
		w0 := 150 / (200 + z).
		w1 := 150 / (200 - z).
		x0 := 200.
		y0 := 200.
		vertices := #().
		vertices at: 1 put: x0 + (100 * cos * w0).
		vertices at: 2 put: y0 - (100 * w0).
		vertices at: 3 put: x0 + (100 * cos * w0).
		vertices at: 4 put: y0 + (100 * w0).
		vertices at: 5 put: x0 - (100 * cos * w1).
		vertices at: 6 put: y0 + (100 * w1).
		vertices at: 7 put: x0 - (100 * cos * w1).
		vertices at: 8 put: y0 - (100 * w1).
		indices := #(0 1 2 2 3 0).
		tex_uv := #().
		tex_uv at: 1 put: 100 * sx.
		tex_uv at: 2 put: 0.
		tex_uv at: 3 put: 100 * sx.
		tex_uv at: 4 put: 200 * sy.
		tex_uv at: 5 put: 300 * sx.
		tex_uv at: 6 put: 200 * sy.
		tex_uv at: 7 put: 300 * sx.
		tex_uv at: 8 put: 0.
		tex_uvt := #().
		tex_uvt at: 1 put: 100 * sx.
		tex_uvt at: 2 put: 0.
		tex_uvt at: 3 put: w0.
		tex_uvt at: 4 put: 100 * sx.
		tex_uvt at: 5 put: 200 * sy.
		tex_uvt at: 6 put: w0.
		tex_uvt at: 7 put: 300 * sx.
		tex_uvt at: 8 put: 200 * sy.
		tex_uvt at: 9 put: w1.
		tex_uvt at: 10 put: 300 * sx.
		tex_uvt at: 11 put: 0.
		tex_uvt at: 12 put: w1.

		cols := #( 16rFFFF0000 16rFF00FF00 16rFF0000FF 16rFFFFFFFF ).

		gfx := s0 graphics.
		gfx clear.
		gfx beginBitmapFill: data matrix: nil repeat: true smooth: false.
		gfx lineStyle: 4 color: 16r0000FF alpha: 1 
			pixelHinting: false scaleMode: nil 
			caps: nil joints: nil miterLimit: 3.
		gfx drawTriangles: vertices indices: indices  
			uvtData: tex_uvt culling: nil colours: nil
			blendMode: 0.

		gfx := s1 graphics.
		gfx clear.
		gfx beginBitmapFill: data matrix: nil repeat: true smooth: false.
		gfx lineStyle: 4 color: 15r0000FF alpha: 1 
			pixelHinting: false scaleMode: nil 
			caps: nil joints: nil miterLimit: 3.
		gfx drawTriangles: vertices indices: indices  
			uvtData: tex_uv culling: nil colours: nil
			blendMode: 0.

		gfx := s2 graphics.
		gfx clear.
		gfx beginBitmapFill: data matrix: nil repeat: true smooth: false.
		gfx lineStyle: 4 color: 16r808080 alpha: 1 
			pixelHinting: false scaleMode: nil 
			caps: nil joints: nil miterLimit: 3.
		gfx drawTriangles: vertices indices: indices  
			uvtData: tex_uv culling: nil colours: nil
			blendMode: 0.

		gfx := s3 graphics.
		gfx clear.
		gfx beginBitmapFill: data matrix: nil repeat: true smooth: false.
		gfx lineStyle: 4 color: 16r808080 alpha: 1 
			pixelHinting: false scaleMode: nil 
			caps: nil joints: nil miterLimit: 3.
		gfx drawTriangles: vertices indices: indices  
			uvtData: tex_uvt culling: nil colours: nil
			blendMode: 0.
	].

	stage
		addEventListener: Event enterFrame
		with: [:e | doUpdate value. nil ] ! !

! Utf8 class methodsFor: #samples !
example
	" Utf8 example "

	| s utf8 |
"	s := #(8021 945 955 959 957) inject: '' into: [:total :c| total , c asCharacter ].
	self print: s,' size=',s size.
	self print: 'validate=',(Utf8 validate: s).
	self print: 'compare=',(Utf8 compare: s with: '',s).
	Utf8 iter: s doing: [:c| self print: c ].
	self print: 'first=', (Uft8 charCodeAt: s index: 0).
	self print: 'sub=', (Uft8 sub: s pos: 1 len: 2).
"	utf8 := Utf8 new: 0.
	#(8021 945 955 959 957) do: [:i| utf8 addChar: i ].
	self print: 'utf8=' ,utf8 toString! ! 

! Bytes class methodsFor: #samples !
example
	" Bytes example. "

	| bytes bytes2 |
	bytes := Bytes new: 5.
	self print: 'size=',bytes size.
	bytes fill: 0 len: 2 value: 65.
	bytes fill: 2 len: 3 value: 67.
	self print: 'AACCC=',bytes toString.
	self print: '4141434343=',bytes toHex.
	self print: 'CCC=',(bytes getString: 2 len: 3).
	bytes2 := bytes sub: 0 len: 2.
	self print: 'AA=',bytes2 toString.
	bytes set: 0 value: 69.
	self print: '69=',(bytes get: 0) toString.
	bytes blit: 2 src: bytes2 srcPos: 0 len: 2.
	self print: 'EAAAC=',bytes toString! !

! Loader class methodsFor: #samples !
example
	" Loader example "

	| loader root req |
	loader := Display loader new.
	loader contentLoaderInfo
		addEventListener: Event complete
		with: [:e| loader content bitmapData. nil ].
	root := MediaFile applicationStorageDirectory nativePath.
	req := URLRequest url: root ,'/Image.jpg'.
	loader load: req context: nil.
	self print: '#done @ ',root! !

! Stage3D class methodsFor: #samples !
example
	" Stage3D example "

	| stage3D |
	stage3D := Lib media display stage stage3Ds first.
	stage3D
		addEventListener: Event context3dCreate
		with: [:e|
			self print: 'Context 3D created'.
			stage3D context3D. nil
		].
	stage3D requestContext3D: ''! !

! AGALMiniAssembler class methodsFor: #samples !
example
	" AGALMiniAssembler example "

	| stage stage3D |
	stage := Lib media display stage.
	stage3D := stage stage3Ds first.
	stage3D
		addEventListener: Event context3dCreate
		with: [:e| | context3D program assembler |
			self print: 'Context 3D created'.
			context3D := stage3D context3D.
			context3D
				configureBackBuffer: stage stageWidth height: stage stageHeight
					antiAlias: 4 enableDepthAndStencil: true;
				enableErrorChecking: true.
			context3D createProgram
				upload: (AGALMiniAssembler new
					assemble: Context3DProgramType vertex
					source: 'mov op, va0' ,String cr , 'mov v0, va1')
				fragmentShader: (AGALMiniAssembler new
					assemble: Context3DProgramType fragment
					source: 'mov oc, v0' ,String cr).
			nil
		].
	stage3D requestContext3D: ''! !

! GLSLFragmentShader class methodsFor: #samples !
example
	" GLSLFragmentShader example "

	^GLSLFragmentShader source: ' void main(void) { gl_FragColor = vec4(1.0, 1.0, 1.0, 1.0); }'! !

! MediaFile class methodsFor: #samples !
example
	" MediaFile folders example "

	#( applicationDirectory applicationStorageDirectory
	   desktopDirectory documentsDirectory userDirectory
	) do: [:selector|
		self print: selector,'=', (self perform: selector) nativePath
	]! !

! Bitmap class methodsFor: #samples !
drawSprite: sprite
	" Draw sprite "

	| red green blue size graphics colours |
	red := 16rFF0000.
	green := 16r00FF00.
	blue := 16r0000FF.
	size := 100.
	colours := Array with: red with: blue with: green.
	graphics := sprite graphics.
	graphics beginGradientFill: GradientType linear
		colors: colours alphas: #(1 0.5 1) 
		ratios: #(0 200 255) matrix: nil	
		spreadMethod: nil
		interpolationMethod: nil
		focalPointRatio: 0.
	graphics drawRect: 0 y: 0 width:100 height: 100.
	graphics beginFill: 16r808080 alpha: 1.
	graphics drawRect: 80 y: 80 width: 100 height: 100.! !

! Bitmap class methodsFor: #samples !
colorTransformExample
	" Color transform example "
	
	| stage target data bmp bmp2 target box s data2 |
	stage := Display stage.
	target := Display sprite new.
	self drawSprite: target.
	stage addChild: target.

	data := BitmapData @ 'Image.jpg'.
	bmp := (Bitmap
		bitmapData: data 
		pixelSnapping: PixelSnapping auto
		smoothing: false)
		alpha: 0.5; x: 50; y: 50;
		yourself.
	target addChild: bmp.
	target alpha: 0.5.
	box := Display sprite new.
	box alpha: 0.2.
	self drawSprite: box.
	box x: 160; y: 160.
	stage addChild: box.
	data := BitmapData inWidth: 100 inHeight: 100 
		inTransparent: true fillColor: 16r00 
		inGPUMode: nil.
	s := Shape new.
	s graphics
		beginFill: 16r00FF00 alpha: 1;
		drawCircle: 0 y: 0 radius: 100.
	data draw: s matrix: nil colorTransform: nil blendMode: nil
		clipRect: nil smoothing: false.
	data2 := BitmapData inWidth: 100 inHeight: 100 
		inTransparent: true fillColor: 16r00 
		inGPUMode: nil.
	data2 draw: (Bitmap 
			bitmapData: data 
			pixelSnapping: PixelSnapping auto
			smoothing: false
		)
		matrix: nil 
		colorTransform: (ColorTransform 
			redMultiplier: 0 greenMultiplier: 0
			blueMultiplier: 0 alphaMultiplier: -1
			redOffset: 0 greenOffset: 0
			blueOffset: 0 alphaOffset: 255
		) 
		blendMode: nil clipRect: nil smoothing: false.
	bmp2 := (Bitmap 
		bitmapData: data2 
		pixelSnapping: PixelSnapping auto
		smoothing: false)
		x: 200;
		yourself.
	stage addChild: bmp2.
	target addEventListener: MouseEvent click
		with: [:e | | t rOffset bOffset a |
		t := stage transform.
		rOffset := t colorTransform redOffset + 25.
		bOffset := t colorTransform redOffset + 25.
		a := stage alpha.
		t colorTransform: (ColorTransform 
			redMultiplier: 1 greenMultiplier: 1
			blueMultiplier: 1 alphaMultiplier: a * 0.9
			redOffset: rOffset greenOffset: 0
			blueOffset: bOffset alphaOffset: 0).
		nil
	]! !

! FileFilter class methodsFor: #samples !
example
	" FileFilter example "

	| result |
	result := MediaFile new
		browseForOpen: 'Open Archives'
		typeFilter: (FileFilter
			description: 'Archives *.zip;*.rar'
			extension: '*.zip;*.rar'
			macType: 'zip;rar').
	self print: 'Result=' ,result! !

! Zip class methodsFor: #samples !
example
	" Compress/decompress contents"

	| text bytes compressed uncompressed |
	text := 'test test test test test'.
	bytes := Bytes ofString: text.
	compressed := Zip compress: bytes  level: 7.
	self note: '[' ,compressed size ,'] ' ,compressed toHex ,'.'.
	uncompressed := Zip uncompress: compressed.
	self note: '[' ,uncompressed size ,'] ' ,uncompressed toString ,'.'.! !

! ZipReader class methodsFor: #samples !
example
	" Scan zip file contents "

	| bytes input entries |
	bytes := IOFile bytesAt: 'sample.zip'.
	self note: 'sample.zip file size=',bytes size.
	input := BytesInput bytes: bytes pos: 0 len: bytes size.
	[	entries := Zip reader readZip: input.
		entries do: [:each| | fName type |
			fName := each fileName.
			type := (fName endsWith: $/) ifTrue: [#folder] ifFalse: [#file].
			self note: type ,' ' ,fName.
			type = #file ifTrue: [ | data bytes contents k |
				data := Zip reader unzip: each.
				bytes := ByteArray fromBytes: data.
				contents := bytes asString.
				k := contents size min: 10.
				self note: '  size=' ,contents size
					,' first[',k,']=' ,(contents copyFrom: 1 to: k)
					,(contents size>k ifTrue: ['...'] ifFalse: [''])
			].
		]
	] ensure: [ input close ]! !

! Stage class methodsFor: #samples !
example
	" Sample "

	| scale stage sp bd bm shape gfx |
	stage := Display stage.
	scale := Capabilities screenDPI > 120 ifTrue: [ 
	Capabilities screenDPI / 120 ] ifFalse: [ 1 ].
	stage scaleX: scale; scaleY: scale; addChild: FPS new.
	sp := Display sprite new.
	sp graphics beginFill: 0 alpha: 1.
	sp graphics drawCircle: 50 y: 50 radius: 50.
	sp graphics endFill.
	stage frameRate: 60.
	bd := BitmapData inWidth: 100 inHeight: 100 
		inTransparent: true fillColor: 16rCCCCCCCC 
		inGPUMode: nil.
	bd draw: sp matrix: nil colorTransform: nil 
		blendMode: nil  clipRect: nil smoothing: false.
	bm := Bitmap bitmapData: bd 
		pixelSnapping: nil smoothing: false.
	stage addChild: bm.
	bm x: 100.

	shape := Display sprite new.
	gfx := shape graphics.
	gfx lineStyle: 1 color: 16rFF0000 alpha: 1
		pixelHinting: false scaleMode: nil 
		caps: nil joints: nil miterLimit: 3.
	gfx beginFill: 16rFFFFFF alpha: 1.
	gfx drawRect: 0 y: 0 width: 20 height: 40.
	shape x: 100; y: 100; rotation: 10. 
	stage addChild: shape.

	stage addEventListener: Event enterFrame
		with: [:e | shape rotation: shape rotation + (360/60/10). nil ].
	stage addEventListener: MouseEvent click
		with: [:e | shape rotation: 0. nil ].
	stage addEventListener: MouseEvent mouseMove
		with: [:e | shape x: e#stageX - 10; y: e#stageY - 20. nil ].! !

"----Scrollbar-----------------------------------------------------------------------------------------------"
Object
	subclass: #Scrollbar
	instanceVariableNames: ' sprite '
	category: #LuaStudioSamples!
Scrollbar buildAccessors!

! Scrollbar class methodsFor: #instantiation !
fromInWidth: inWidth
	inHeight: inHeight inRange: inRange 
	inPage: inPage window: inWindow
	" Returns an instance of the receiver.  "
	
	| sprite result
	thumb mThumbHeight onScroll thumbStop |

	result := self new.
	sprite := Sprite new.
	result sprite: sprite.

	sprite graphics
		lineStyle: 1 color: 16r404040 alpha: 1 
			pixelHinting: false scaleMode: nil 
			caps: nil joints: nil miterLimit: 3;
		beginFill: 16rEEEEEE alpha: 1;
		drawRect: 0 y: 0 width: inWidth height: inHeight.

	mThumbHeight := inHeight * inPage / inRange.

	thumb := Sprite new.
	thumb graphics
		lineStyle: 1 color: 16r000000 alpha: 1 
			pixelHinting: false scaleMode: nil 
			caps: nil joints: nil miterLimit: 3;
		beginFill: 16rFFFFFF alpha: 1;
		drawRect: 0 y: 0 width: inWidth height: mThumbHeight.
	sprite addChild: thumb.

	onScroll := [:e | | denom |
		denom := inHeight - mThumbHeight.
		denom > 0 ifTrue: [ | y r |
			y := thumb y / denom * inRange.
			r := ARectangle x: 0 y: y width: 440 height: 400.
			inWindow scrollRect: r
		].
	].

	thumbStop := [:e |
		thumb	stopDrag.
		thumb
			removeEventListener: MouseEvent mouseUp 
			doing: thumbStop capture: false.
		Display stage
			removeEventListener: MouseEvent mouseUp
				doing: thumbStop capture: false;
			removeEventListener: MouseEvent mouseMove
				doing: onScroll capture: false.
	].

	thumb addEventListener: MouseEvent mouseDown doing: [:e | | r |
		r := ARectangle x: 0 y: 0 width: 0 height: inHeight - mThumbHeight.
		Display stage
			addEventListener: MouseEvent mouseUp doing: thumbStop;
			addEventListener: MouseEvent mouseMove doing: onScroll.
		thumb	addEventListener: MouseEvent mouseUp doing: thumbStop.
		thumb	startDrag: false bounds: r.
		nil
	].
	^result! !

"----Masks--------------------------------------------------------------------------"
Object
	subclass: #MasksSample
	category: #LuaStudioSamples!

! MasksSample class methodsFor: #samples !
example
	" MasksSample example "

	| stage window scrollbarObj gfx bg line glow tf 
	mask_obj mask_child scrollbar |
	stage := Display stage.
	window := Sprite new.
	stage addChild: window.

	scrollbarObj := Scrollbar fromInWidth: 20 
		inHeight: 404 inRange: 1024 - 400 
		inPage: 404 * 400 / 1024 window: window.
	scrollbar := scrollbarObj sprite.
	scrollbar x: 76; y: 38.
	stage addChild: scrollbar.

	gfx := stage graphics.
	gfx lineStyle: 1 color: 16r000000 alpha: 1 
		pixelHinting: false scaleMode: nil 
		caps: nil joints: nil miterLimit: 3.
	gfx drawRect: 98 y: 38 width: 444 height: 404.
	window x: 100; y: 40.

	bg := Sprite new.
	gfx := bg graphics.
	gfx beginFill: 16r808080 alpha: 1.
	gfx drawRect: 0 y: 0 width: 1024 height: 1024.
	window addChild: bg.

	line := Sprite new.
	gfx := line graphics.
	gfx lineStyle: 20 color: 16rFFFFFF alpha: 1 
		pixelHinting: false scaleMode: nil 
		caps: nil joints: nil miterLimit: 3.
	gfx moveTo: 20 y: 20.
	gfx lineTo: 250 y: 250.
	window addChild: line.

	glow := GlowFilter color: 16r00FF00 alpha: 1
		blurX: 3 blurY: 3
		strength: 1 quality: 1
		inner: false knockout: false.
	line filters: (Array with: glow).

	line := Sprite new.
	gfx := line graphics.
	gfx lineStyle: 5 color: 16r000000 alpha: 1 
		pixelHinting: false scaleMode: nil 
		caps: nil joints: nil miterLimit: 3.
	gfx moveTo: 5 y: 5.
	gfx lineTo: 250 y: 250.
	window addChild: line.

	tf := TextField new
		text: 'Lorem ipsum dolor sit amet, consectetur adipisicing elit, 
		sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. ';
		selectable: false; wordWrap: true;
		width: 150; name: 'text1';
		cacheAsBitmap: true; 
		yourself.
	window addChild: tf.

	mask_obj := Sprite new.
	mask_obj graphics
		beginFill: 16rFF0000 alpha: 1;
		drawCircle: 0 y: 0 radius: 40;
		endFill.
	mask_obj name: 'mask_obj'.
	window addChild: mask_obj.

	mask_child := Shape new.
	gfx := mask_child graphics.
	gfx beginFill: 16r00FF00 alpha: 1.
	gfx drawRect: -60 y: -10 width: 120 height: 20.
	mask_obj addChild: mask_child.

	window	mask: mask_obj;
		addEventListener: MouseEvent mouseDown
			doing: [:e | mask_obj startDrag: false bounds: nil. nil ];
		addEventListener: MouseEvent mouseUp
			doing: [:e | mask_obj stopDrag. nil ].

	tf x: 100; y: 100.
	mask_obj x: 100; y: 100! !

! Text class methodsFor: #samples !
example
	" Text example "

	| stage scale input textRuns |
	stage := Display stage.
	scale := Capabilities screenDPI > 120 ifTrue: [ 
		Capabilities screenDPI / 120 ] ifFalse: [ 1 ].
	stage scaleX: scale; scaleY: scale.
	stage graphics
		beginFill: 16r000000 alpha: 1;
		drawRect: 120 y: 0 width: 120 height: 320.
	0 to: 1 do: [:side | | colA colB col text fmt |
		colA := side * 16rFF + 16rFF.
		colB := 16rFFFFFF.
		col := colA - ((colA / colB) truncated * colB).
		text := TextField new
			x: side * 120 + 10; y: 10;
			textColor: col; width: 100; wordWrap: true;
			text: 'Hello !!', 
			String lf, 'From this multi-line, wordwrapped, centred text box!!';
			yourself.
		fmt := TextFormat font: '_sans' size: 12 
			color: 16r969696 bold: nil 
			italic: nil underline: nil.
		fmt align: TextFormatAlign center.
		text setTextFormat: fmt beginIndex: 0 endIndex: text length.
		fmt := TextFormat font: '_sans' size: 12 
			color: 16r660000 bold: nil 
			italic: nil underline: nil.
		text setTextFormat: fmt beginIndex: 6 endIndex: 12.	
		fmt color: 16rFF00FF.
		text setTextFormat: fmt beginIndex: 18 endIndex: 19.
		stage addChild: text.

		text := TextField new
			x: side * 120 + 10; y: 120;
			textColor: col; 
			htmlText: '<font size=''16''>Hello !!</font>';
			yourself.
		stage addChild: text.

		text := TextField new
			x: side * 120 + 10; y: 170;
			textColor: col; 
			htmlText: '<font size=''24''>Hello !!</font>';
			yourself.
		stage addChild: text.
		
		text := TextField new
			x: side * 120 + 10; y: 220;
			textColor: col; 
			htmlText: '<font size=''36''>Hello !!</font>';
			yourself.
		stage addChild: text.
	].

	input := TextField new
		x: 2 * 120 + 20; y: 10;
		type: TextFieldType input;
		wordWrap: true; multiline: true;
		width: 240; height: 300;
		htmlText: 'Input <u>underline</u> <sup>superscript</sup> <s>strikethrough</s> <sub>subscript</sub> <b><i>bold italic</i></b> <font size=''24'' color=''#0000FF'' face=''_sans''>blue 24</font>';
		border: true; borderColor: 16r000000;
		background: true; backgroundColor: 16rF0F0FF;
		softKeyboard: SoftKeyboardType default;
		yourself.
	stage addChild: input.

	textRuns := input getTextRuns: 0 endIndex: 7.
	1 to: textRuns size do: [:i | | textRun textFormat underline |
		textRun := textRuns at: i.
		textFormat := textRun textFormat.
		self print: 'textFormat[',i,'] begin index: ' ,textRun beginIndex
			,' end index: ', textRun endIndex
			,' underline: ', textFormat underline
	].

	stage	addEventListener: FocusEvent mouseFocusChange
		with: [:e | Lib sys trace: (FocusEvent @ e) relatedObject. nil ].
	stage	addEventListener: SoftKeyboardEvent softKeyboardActivate
		with: [:e |
			# #print: ' SOFT KEYBOARD ACTIVATE '
			,stage softKeyboardRect height. nil ].
	stage	addEventListener: SoftKeyboardEvent softKeyboardDeactivate
		with: [:e |
			# #print: ' SOFT KEYBOARD DEACTIVATE '
			,stage softKeyboardRect height. nil ].! !

! System class methodsFor: #samples !
example
	" System example "

	| hJIT |	
	self print: 'Lua Studio version: ', Lib sys version.
	self print: '_VERSION=' ,#{_VERSION}.

	hJIT := #{jit}.
	((self typeOf: hJIT) = #table) ifTrue: [
		self print: 'jit version=' ,hJIT#version
	].

	#(	pixelAspectRatio screenDPI
		screenResolutionX screenResolutionY
		language
	) do: [:each| self print: each ,'=' ,(Capabilities perform: each) ].

	#(	deviceID totalMemory
		systemName exeName
		getLocalIpAddress
	) do: [:each| self print: each ,'=' ,(System perform: each) ].

	" Lib garbage collector "
	System gc.
	
	" Exit "
	" System restart: 0. "
	" System exit: 0. "

	System getUrl: (URLRequest new: 'https://www.wikipedia.org/') target: '_blank'.! !


"-SquareTexture-----------------------------------------------------------------------------------"
ShaderSample
	subclass: #SquareTexture
	instanceVariableNames: ' vertices indices '
		,' vertexBuffer indexBuffer bitmap texture '
		,' assembler vertexShader fragmentShader'!

! SquareTexture methodsFor: #private !
uploadShaders

	super uploadShaders.
	assembler := AGALMiniAssembler new.
	vertexShader := assembler
		assemble: Context3DProgramType vertex
		source: 'm44 op, va0, vc0' ,String lf ,'mov v0, va1'.
	fragmentShader := assembler
		assemble: Context3DProgramType fragment
		source: 'tex ft1, v0, fs0 <2d,linear, nomip>;' ,String lf ,'mov oc, ft1'.
	program
		upload: vertexShader
		fragmentShader: fragmentShader.
	"self note: #updateShadersOk." ! !

! SquareTexture methodsFor: #private !
createModels

	super createModels.
	vertices := #(
		-0.5 -0.5 0 0 0
		-0.5  0.5 0 0 1
		 0.5  0.5 0 1 1
		 0.5 -0.5 0 1 0 ).
	indices := #( 0 1 2 2 3 0 ).
	vertexBuffer := context3D createVertexBuffer: 4 data32PerVertex: 5.
	vertexBuffer uploadFromVector: vertices startOffset: 0 count: 4.
	indexBuffer := context3D createIndexBuffer: 6.
	indexBuffer uploadFromVector: indices startOffset: 0 count: 6.

	bitmap := Bitmap @ 'RockSmooth.jpg'.
	texture := context3D
		createTexture: bitmap bitmapData width
		height: bitmap bitmapData height
		format: Context3DTextureFormat bgra
		optimize: false levels: 0.
	texture	uploadFromBitmapData: bitmap bitmapData miplevel: 0.
	"self note: #createModelsOK."! !

! SquareTexture methodsFor: #private !
renderIn: rect

	| m |
	context3D
		clear: 1 green: 1 blue: 1 alpha: 1
		depth: 1 stencil: 0 mask: Context3DClearMask all;
		setProgram: program;
		setVertexBufferAt: 0 buffer: vertexBuffer
			bufferOffset: 0 format: Context3DVertexBufferFormat float3;
		setVertexBufferAt: 1 buffer: vertexBuffer
			bufferOffset: 3 format: Context3DVertexBufferFormat float2;
		setTextureAt: 0 texture: texture;
		yourself.
	m := Matrix3D new.
	m appendRotation: Lib sys getTime / 50 axis: Vector3D zAxis pivotPoint: nil.
	context3D
		setProgramConstantsFromMatrix: Context3DProgramType vertex
		firstRegister: 0 matrix: m transposedMatrix: true;
		drawTriangles: indexBuffer firstIndex: 0 numTriangles: -1;
		present.
	super renderIn: rect.
"# #print: #presented" ! !

"-Stage3D-----------------------------------------------------------------------------------"
ShaderSample
	subclass: #Stage3DSample
	instanceVariableNames: ' vertices indices '
		,' vertexBuffer indexBuffer '
		,' vertexShader fragmentShader'!

! Stage3DSample methodsFor: #private !
createProgram
	program := GLSLProgram new: context3D! !

! Stage3DSample methodsFor: #private !
uploadShaders

	super uploadShaders.
	vertexShader := GLSLVertexShader source: '
		attribute vec3 vertexPosition;
		uniform mat4 mvpMatrix;
		void main(void) { gl_Position = mvpMatrix * vec4(vertexPosition, 1.0); }
		' opt: nil.
	fragmentShader := GLSLFragmentShader source: '
		void main(void) { gl_FragColor = vec4(1.0, 1.0, 1.0, 1.0); }
		' opt: nil.
	program
		upload: vertexShader
		fragmentShader: fragmentShader.
	"self note: #updateShadersOk." ! !

! Stage3DSample methodsFor: #private !
createModels

	super createModels.
	vertices := #(
		100 100 0	-100 100 0
		100 -100 0	-100 -100 0
	).
	indices := #( 0 1 2 2 3 1 ).
	vertexBuffer := context3D createVertexBuffer: 4 data32PerVertex: 3.
	vertexBuffer uploadFromVector: vertices startOffset: 0 count: 4.
	indexBuffer := context3D createIndexBuffer: 6.
	indexBuffer uploadFromVector: indices startOffset: 0 count: 6.
	"self note: #createModelsOK."! !

! Stage3DSample methodsFor: #private !
renderIn: rect

	| m extent center projectionMatrix modelViewMatrix mvpMatrix |
	extent := Display stage stageExtent.
	center := extent / 2.

	projectionMatrix := Matrix3D createOrtho: (Array with: 0 with: extent x with: extent y) ,#( 0 1000 -1000 ).
	modelViewMatrix := Matrix3D create2D: (Array with: center x with: center y with: 1 with: 0).
	mvpMatrix := modelViewMatrix clone.
	mvpMatrix append: projectionMatrix.

	program
		attach;
		setVertexUniformFromMatrix: #mvpMatrix with: mvpMatrix transposedMatrix: true;
		setVertexBufferAt: #vertexPosition with: vertexBuffer
			ofsset: 0 format: Context3DVertexBufferFormat float3;
		yourself.

	context3D
		clear: 0 green: 0 blue: 0 alpha: 1
		depth: 1 stencil: 0 mask: Context3DClearMask all;
		drawTriangles: indexBuffer firstIndex: 0 numTriangles: -1;
		present.
	super renderIn: rect.
"# #print: #presented" ! !

"-Stage3DAGAL-----------------------------------------------------------------------------------"
ShaderSample
	subclass: #AGALSample
	instanceVariableNames: ' vertices indices '
		,' vertexBuffer indexBuffer bitmap texture '
		,' assembler vertexShader fragmentShader'!

! AGALSample methodsFor: #private !
uploadShaders

	super uploadShaders.
	assembler := AGALMiniAssembler new.
	vertexShader := assembler
		assemble: Context3DProgramType vertex
		source:	 'mov op, va0' ,String lf "Move the Vertex Attribute 0 (va0), which is our Vertex Coordinate, to the Output Point"
			,'mov v0, va1'."Move the Vertex Attribute 1 (va1), which is our Vertex Color, to the variable register v0
		Variable register are memory space shared between your Vertex Shader and your Fragment Shader"
	fragmentShader := assembler
		assemble: Context3DProgramType fragment
		source: 'mov oc, v0'."Move the Variable register 0 (v0) where we copied our Vertex Color, to the output color"
	program
		upload: vertexShader
		fragmentShader: fragmentShader.
	"self note: #updateShadersOk." ! !

! AGALSample methodsFor: #private !
createModels

	super createModels.
	vertices := #(
		-0.3 -0.3 0 1 0 0	"1st vertex x,y,z,r,g,b"
		0 0.3 0 0 1 0	"2nd vertex x,y,z,r,g,b"
		0.3 -0.3 0 0 0 1	"3rd vertex x,y,z,r,g,b"
	).
	indices := #( 0 1 2 ).
	vertexBuffer := context3D createVertexBuffer: 3 data32PerVertex: 6.
	vertexBuffer uploadFromVector: vertices startOffset: 0 count: 3.
	indexBuffer := context3D createIndexBuffer: 3.
	indexBuffer uploadFromVector: indices startOffset: 0 count: 3.
	"self note: #createModelsOK."! !

! AGALSample methodsFor: #private !
renderIn: rect

	context3D
		setProgram: program;
		setVertexBufferAt: 0 buffer: vertexBuffer
			bufferOffset: 0 format: Context3DVertexBufferFormat float3;
		setVertexBufferAt: 1 buffer: vertexBuffer
			bufferOffset: 3 format: Context3DVertexBufferFormat float3;
		yourself.
	context3D
		clear: 1 green: 1 blue: 1 alpha: 1
		depth: 1 stencil: 0 mask: Context3DClearMask all;
		drawTriangles: indexBuffer firstIndex: 0 numTriangles: -1;
		present.
	super renderIn: rect.
"# #print: #presented" ! !


! WebServer class methodsFor: #constants !
sampleCode
	" Private - Returns the code string for example file. "

	^'--web response
	Request = Lib.Web.Request
	Response = Lib.Web.Response
	Response.addHeader("Content-Type", "text/html")
	Response.writeString("<html><body>")
	Response.writeString(Request.requestType.."<br/>")
	Response.writeString(Request.url.."<br/>")
	Response.writeString("Headers:<br/>")
	headers = Lib.Reflect.fields(Request.headers)
	for k, v in pairs(headers) do
		Response.writeString(k.." "..v.."="..Lib.Reflect.field(Request.headers, v))
		Response.writeString("<br/>")
	end
	Response.writeString("QueryString:<br/>")
	queryStringFields = Lib.Reflect.fields(Request.queryString)
	for k, v in pairs(queryStringFields) do
		Response.writeString(k.." "..v.."="..Lib.Reflect.field(Request.queryString, v))
		Response.writeString("<br/>")
	end
	Response.writeString("Form:<br/>")
	formFields = Lib.Reflect.fields(Request.form)
	for k, v in pairs(formFields) do
		Response.writeString(k.." "..v.."="..Lib.Reflect.field(Request.form, v))
		Response.writeString("<br/>")
	end
	Response.writeString("</body></html>")' ! !

! WebServer class methodsFor: #sample !
example
	" WebServer example "
	
	| root serverSideLuaFileName path webServer webTrack frameCheck |
	root := MediaFile applicationStorageDirectory nativePath.
	serverSideLuaFileName := 'webservercheck.lua'.
	path := root, '\\', serverSideLuaFileName.

	"self note: 'writing servlet at ', path."
	IOFile new saveContent: path content: self sampleCode.
	System systemName = #ios ifTrue: [ (MediaFile new: path) preventBackup: true ].

	webServer := Lib webServer.
"# #print: #webServer with: #aboutToStart with: webServer toString with: webServer handle."
	webTrack := [:url :code :status | # #print: url with: code with: status ].
	webServer 
		start: #localhost port: 2001 
		path: root oncomplete: webTrack.
"# #print: #webServer with: #OK."

	frameCheck := [:e |
"# #print: #webServer with: #frameCheck."
		(webServer isStarted: #localhost port: 2001) ifTrue: [  | url |
"# #print: #webServer with: #started."
			url := 'http://localhost:2001/', serverSideLuaFileName, '?test=true'.
			Lib webView isSupported ifTrue: [
# #print: #webView with: #aboutToOpen.
				Lib webView
					open: url userAgent: nil floating: true 
					urlWhiteList: nil urlBlackList: nil.
# #print: #webView with: #opened.
			] ifFalse: [
				"System getURL: (URLRequest new: url) target: #_blank"
				# #print: 'Serving ', url
			].
			Display stage removeEventListener: Event enterFrame doing: frameCheck. 
		]. nil
	].
	Display stage addEventListener: Event enterFrame with: frameCheck.

	System systemName = #ios ifTrue: [
		Display stage
			addEventListener: Event deactivate doing: [:e |
				# #print: #deactivated.
				(webServer isStarted: #localhost port: 2001) ifTrue: [
					webServer stop: #localhost port: 2001
				]. nil
			];
			addEventListener: Event activate doing: [:e |
				# #print: #activated.
				(webServer isStarted: #localhost port: 2001) ifTrue: [
					webServer
						start: #localhost port: 2001 
						path: root oncomplete: webTrack
				]. nil
			].
	].! !

"-BitmapFont-----------------------------------------------------------------------------------"!
Object
	subclass: #BitmapFontSample
	instanceVariableNames: ' view container '
	category: #LuaStudioSamples!

! BitmapFontSample class methodsFor: #sample !
example
	" BitmapFontSample example. "

	^self new run! !

! BitmapFontSample methodsFor: #sample !
run
	" Execute the receiver. "

	self	setupScene;
		setContents;
		render! !

! BitmapFontSample methodsFor: #setup !
setupScene

	view := View3D new.
	Display stage addChild: view.
	view antiAlias: 0.

	view camera
		z: -600; y: 200;
		lookAt: Vector3D zero upAxis: nil;
		yourself.

	container := ObjectContainer3D new.
	view scene addChild: container.! !

! BitmapFontSample methodsFor: #constants !
count
	^14! !

! BitmapFontSample methodsFor: #setup !
setContents

	| bmTexture xml random |
	random := Random new.
	self initFPS.
	bmTexture := BitmapTexture
		bitmapData: BitmapData @ 'BerberRevKC_260.png'
		generateMipmaps: false.
	xml := Xml pathName: 'BerberRevKC_260.fnt'.
	self count timesRepeat: [:i|
		self
			setBanner: i texture: bmTexture
			font: xml with: random
	].! !

! BitmapFontSample methodsFor: #private !
setBanner: i texture: texture font: xml with: random

	| textContainer material font field |
	textContainer := ObjectContainer3D new.
	textContainer rotationY: i - 1 * 360 / self count.
	container addChild: textContainer.

	material := TextureMaterial
		texture: texture smooth: true
		repeat: false mipmap: true
		anisotropy: nil.
	material
		alphaBlending: true;
		bothSides: true.
	font := BitmapFont
		texture: material
		fontXml: xml.
			
	field := TextField
		width: 800 height: 600
		text: 'This is a Test' asUppercase "this font has only uppercase letters defined"
		bitmapFont: font fontSize: 100
		color: (random next * 16rFFFFFF) rounded
		bold: false hAlign: HAlign center.
	field	x: -400;
		y: i - 1 * 60 - 300;
		rotationX: 90;
		yourself.

	textContainer addChild: field.
	"self note: #contentsOK."! !

! BitmapFontSample methodsFor: #initialize !
initFPS
	Display stage addChild: (AwayFPS
		xOff: 10 yOff: 10
		color: 16rFFFFFF scale: 3) ! !

! BitmapFontSample methodsFor: #private !
render
	| resize |
	resize := [ view extent: Display stage stageExtent. nil ].
	view setRenderCallback: [:rect|
		container rotationY: container rotationY + 1.
		view render. nil
	].
	Display stage
		addEventListener: Event resize
		doing: resize.
	resize value.! !

! Sound class methodsFor: #constants !
fileExtension
	" Returns a proper file extension for sound files. "

	^(#(android ios) includes: System systemName)
		ifTrue: [ #mp3 ]
		ifFalse: [ #ogg ]! !

! Sound class methodsFor: #samples !
example
	" Sound example "

	| path sound channel |
	path := 'Party_Gu-Jeremy_S-8250_hifi.' ,self fileExtension.
	Display stage addChild: Bitmap @ 'drum_kit.jpg'.
	sound := Sound @ path.
	channel := sound play: 0 loops: 1 sndTransform: nil.
	channel isNil ifTrue: [ ^self error: 'Playing ',path	].
	channel	addEventListener: Event soundComplete
		doing: [:e| # #print: '--Sound Complete'. nil ]! !

! VM class methodsFor: #samples !
example
	" VM example "

	| thread |
	thread := Thread @ '
		local main = Lib.Sys.VM.Thread.readMessage(true)
		for ii=0, 5, 1 do print("--step #"..ii) end
		main.sendMessage("thread done")
	'.
	thread sendMessage: Thread current.
	Display stage
		addEventListener: Event enterFrame
		with: [:e| | msg |
			msg := Thread readMessage: false.
			msg notNil ifTrue: [ # #print: msg ].
			nil
		]! !

! VM class methodsFor: #samples !
example2
	" VM example2 - Message Passing Example "

	| generator consumer |
	generator := Thread @ '-- Message generator
		Thread = Lib.Sys.VM.Thread
		local main = Thread.readMessage(true)
		local t2 = Thread.readMessage(true)
		for ii=0, 5, 1 do
			print("Sender sending message #"..ii)
			t2.sendMessage(ii)
			Lib.Sys.sleep(0.5)
		end
		main.sendMessage("Message sender done")
	'.
	consumer := Thread @ '-- Message consumer
		Thread = Lib.Sys.VM.Thread
		local main = Thread.readMessage(true)
		for ii=0, 5, 1 do
			print("Consumer waiting...")
			print("Got message #"..Thread.readMessage(true))
		end
		main.sendMessage("Message consumer done")
	'.
	generator sendMessage: Thread current.
	generator sendMessage: consumer.
	consumer sendMessage: Thread current.
	Display stage
		addEventListener: Event enterFrame
		with: [:e| | msg |
			msg := Thread readMessage: false.
			msg notNil ifTrue: [ # #print: msg ].
			nil
		]! !

"-BitmapFont-----------------------------------------------------------------------------------"!
Object
	subclass: #FontSample
	instanceVariableNames: ' font bytes field textFormat timer '
	category: #LuaStudioSamples!

! FontSample class methodsFor: #sample !
example
	" FontSample example. "

	^self new run! !

! FontSample methodsFor: #sample !
run
	" Execute the receiver. "

	self
		createFont;
		describeFont;
		registerFont;
		render;
		initFPS;
		destruct! !

! FontSample methodsFor: #private !
createFont

	font := Font
		name: #Purisa
		style: FontStyle regular
		type: FontType embedded.
	bytes := IOFile bytesAt: 'Purisa.ttf'.! !

! FontSample methodsFor: #private !
describeFont
	| data glyphs kerning |
	data := Font loadBytes: bytes.
	#(	has_kerning is_fixed_width
		has_glyph_names is_italic
		is_bold num_glyphs
		family_name style_name
		em_size ascend descend
		height
	) do: [:each| # #print: each with: (data basicAt: each) ].

	glyphs := Array @ (data basicAt: #glyphs).
	glyphs size >= 65 ifTrue: [ | glyph |
		glyph := glyphs at: 65.
		#(	char_code advance
			min_x max_x min_y max_y
		) do: [:each| # #print: '@65' with: each with: (glyph basicAt: each) ].
		# #print: #points with: (Array @ glyph#points) asLiteral
	].

	data#has_kerning ifTrue: [
		kerning := Array @ (data basicAt: #kerning).
		# #print: 'kerning size' with: kerning size
	] ifFalse: [ self note: 'No kerning' ]! !

! FontSample methodsFor: #private !
initFPS
	Display stage addChild: (AwayFPS
		xOff: Display stage stageWidth / 2 yOff: 10
		color: 16rFFFFFF scale: 3) ! !

! FontSample methodsFor: #private !
registerFont
	| fonts |
	Font registerFontData: font bytes: bytes.
	(Font enumerateFonts: false) do: [:each|
		self print: each toString
	].
	self print: #registered! !

! FontSample methodsFor: #private !
render
	| textFormat |
	textFormat := TextFormat
		font: #Purisa size: 24 color: 0
		bold: false italic: false underline: false.
	field := TextField new.
	field	defaultTextFormat: textFormat;
		border: true; text: 'Text with Purisa font';
		x: 5; y: 5; width: 300; height: 30.
	Display stage addChild: field.
	self print: #rendered! !

! FontSample methodsFor: #private !
destruct
	| count |
	count := 5.
	self note: 'Destruct in ' ,count ,' seconds'.
	timer := Timer delay: 1000 repeatCount: count + 1.
	timer addEventListener: TimerEvent timer doing: [:e|
		# #print: '...' with: count with: #seconds.
		count = 0 ifTrue: [
			# #print: #aboutToDestruct.
			Display stage removeChild: field.
			field := nil.
			textFormat := nil.
			Font unregisterFontData: font.
			# #collectgarbage: #collect.
			# #print: #destructed with: $& with: #GCollected.
		].
		count := count - 1. nil
	].
	timer start.
	"self note: #timerStarted"! !

"-File-------------------------------------------------------------------------------------"!
! IOFile class methodsFor: #sample !
example
	" Write ByteArray to File. "

	| root path fileSystem dkdkd dkdkd2 ba ba2|
	fileSystem := FileSystem new.
	root := MediaFile applicationStorageDirectory nativePath.
	(fileSystem exists: root) ifFalse: [
		fileSystem createDirectory: root
	].

	path := root, '/README.txt'.
	IOFile new saveContent: path content: 'A SiMpLe FiLe'.
	System systemName = 'ios' ifTrue: [
		(MediaFile new: path) preventBackup: true
	].

	ba := ByteArray fromBytes: (IOFile new getBytes: path).
# #print: 'length: ', ba length.
	dkdkd := ba asString.
# #print: 'file contents: ', dkdkd.
	ba writeUTFBytes: 'MODIFIED FILE ///////////////////////'.
# #print: 'now writing as another file and re-reading'.
	IOFile new saveBytes: path bytes: ba.
	ba2 := ByteArray fromBytes: (IOFile new getBytes: path).
# #print: 'length: ',  ba2 length.
	dkdkd2 := ba2 asString.
# #print: 'file contents: ', dkdkd2.! !

"-Camera-------------------------------------------------------------------------------------"!
! Camera class methodsFor: #samples !
showCameras
	" Camera showCameras "

	| names |
	names := Camera names.
	names size = 0 ifTrue: [ ^self note: 'No cameras found' ].
	names do: [:each| | cam |
		cam := Camera named: each.
		self note: (#(width height position) inject: cam name into: [:total :prop|
			total ,' ' ,prop ,'=' ,(cam perform: prop)
		]).
		cam destroy.
	].! !

! Camera class methodsFor: #samples !
example
	" Camera example "

	| camera onFrame bitmap setBmpSize |
	self showCameras.
	Camera names size = 0 ifTrue: [ ^nil ].
	camera := Camera default.
	camera isNil ifTrue: [ ^self note: 'Missing default camera' ].

	setBmpSize := [
		| sextent extent |
		bitmap isNil ifTrue: [ ^nil ].
		sextent := Display stage stageExtent.
		extent := bitmap extent.
		extent x * sextent y > (extent y * sextent x) ifTrue: [ | sh |
			bitmap width: sextent x.
			sh := extent y * sextent x / extent x.
			bitmap	height: sh; x: 0;
				y: sextent y - sh * 0.5
		] ifFalse: [ | sw |
			bitmap height: sextent y.
			sw := extent x * sextent y / extent y.
			bitmap	width: sw; y: 0;
				x: sextent x - sw * 0.5
		].
	].

	self note: 'Default camera: ' ,camera name ,' [' ,camera extent ,'] at ' ,camera position.
	onFrame := [:e|
		camera removeEventListener: Event videoFrame doing: onFrame.
		bitmap := Bitmap
			bitmapData: camera bitmapData
			pixelSnapping: nil smoothing: true.
		Display stage addChild: bitmap.
		setBmpSize value. nil
	].
	camera addEventListener: Event videoFrame doing: onFrame.
	Display stage addEventListener: Event resize doing: setBmpSize.
	camera activate.! !

"-Tiles-----------------------------------------------------------------------------------"!
Object
	subclass: #Particle
	instanceVariableNames: ' x y angle dx dy da daspect aspect red green blue alpha size '
	category: #LuaStudioSamples !
Particle class instanceVariableNames: #data!
Particle class buildAccessors: #(
	#(data initialize 'Array new')
)!

! Particle class methodsFor: #defaults !
initialDefaultsArrayForInstanceVars
	" Answer an array with defaults for instance variables initialization "

	| initial iVars |
	initial := #().
	iVars := self instVarNames.
	iVars do: [:iVar | initial add: (Array with: iVar
		with: 'initialize' with: 'self default' ,(iVar asSmalltalkName: true)) 
	].
	^initial! !

Particle buildAccessors: Particle initialDefaultsArrayForInstanceVars! 

	| iVars implementations |
	implementations := #(320 240 0
		'{''math.random() * 2.0 - 1.0''}'
		'{''math.random() * 2.0 - 1.0''}' 
		'{''math.random() * 0.2 - 0.1''}'
		'{''math.random() * 0.1''}'
		'{''math.random() * 1 + 0.25''}' 
		'{''math.random()''}'
		'{''math.random()''}'
		'{''math.random()''}'
		'{''math.random()''}' 
		'{''math.random() * 1.9 + 0.1''}'
	).
	iVars := Particle instVarNames.
	1 to: iVars size do: [:index |
		Particle
			implement: 'default', ((iVars at: index) asSmalltalkName: true)
			comment: 'Returns the default value for ' , (iVars at: index)
			as: '^', (implementations at: index) ,$
			category: #defaults
	]!

! Particle methodsFor: #data !
data
	" Return data "

	^self class data! !

! Particle methodsFor: #data !
addSimple
	" Add simple data "

	^self data
		add: self x; 
		add: self y;
		add: 0;
		yourself! !

! Particle methodsFor: #data !
add
	" Add data "

	^self addSimple
		add: self size; add: self angle;
		add: self red; add: self green;
		add: self blue; add: self alpha;
		yourself! !
		
! Particle methodsFor: #data !
addTrans
	" Add trans data "

	| t00 t01 t10 t11 wobble |
	t00 := self size + self angle cos.
	t01 := self size + self angle sin.
	t10 := t01 negated.
	t11 := t00.
	wobble := 1 + (self aspect cos * 0.75).
	^self addSimple
		add: t00 * wobble; 
		add: t01 * wobble;
		add: t10; add: t11;
		add: self red; add: self green;
		add: self blue; add: self alpha;
		yourself! !

! Particle methodsFor: #data !
addColoured
	" Add coloured data "

	| t00 t01 t10 t11 wobble |
	self addSimple
		add: self red; add: self green;
		add: self blue; add: self alpha;
		yourself! !

! Particle methodsFor: #data !
move
	" Move "

	| rad |
	rad := 30 * self size.
	self x: self x + self dx.
	self x < rad ifTrue: [
		self x: rad.
		self dx: self dx negated
	].
	self x > (640 - rad) ifTrue: [
		self x: 640 - rad.
		self dx: self dx negated
	].
	self y: self y + self dy.
	self y < rad ifTrue: [
		self y: rad.
		self dy: self dy negated
	].
	self y > (480 - rad) ifTrue: [
		self y: 480 - rad.
		self dy: self dy negated
	].
	self angle: self angle + self da.
	self aspect: self aspect + self daspect.
	! !

Object
	subclass: #Tiles
	instanceVariableNames: ''
	category: #LuaStudioSamples !

! Tiles class methodsFor: #samples !
example
	" Tiles example "
	
	| stage shape bmp tilesheet particles tileFlags |
	tileFlags := Tilesheet tileTrans2x2
		+ Tilesheet tileAlpha
		+ Tilesheet tileRgb.
	stage := Display stage.

	shape := Shape new.
	shape graphics
		beginFill: 16rFFFFFF alpha: 1;
		lineStyle: 1 color: 16r000000 alpha: 1 
			pixelHinting: false scaleMode: nil 
			caps: nil joints: nil miterLimit: 3;
		drawCircle: 32 y: 32 radius: 30;
		endFill;
		moveTo: 32 y: 32;
		lineTo: 62 y: 62;
		yourself.

	bmp := BitmapData inWidth: 64 inHeight: 64 
		inTransparent: true fillColor: 16r000000 
		inGPUMode: nil.
	bmp draw: shape matrix: nil colorTransform: nil 
		blendMode: nil clipRect: nil smoothing: false.
	tilesheet := Tilesheet new: bmp.
	tilesheet	addTileRect: (ARectangle x: 0 y: 0 width: 64 height: 64)
		centerPoint: (APoint x: 32 y: 32).

	particles := #().
	1 to: 100 do: [:i| particles at: i put: Particle new ].

	stage addEventListener: Event enterFrame doing: [:e |
		particles first addSimple.
		stage graphics 
			drawTiles: tilesheet xyid: Particle data 
			smooth: true flags: 0 count: -1.
		Particle data: #().
		particles do: [:each| each move; addTrans ].
		stage graphics clear;
			drawTiles: tilesheet xyid: Particle data 
			smooth: true flags: tileFlags count: -1.
		nil
	]! !

"-URLLoader--------------------------------------------------------------------------------"!
! URLLoader class methodsFor: #sample !
example
	" sample "
	
	| stage xmlTextField postTextField loader 
	imageLoader request post postLoad vars |
	stage := Display stage.
	xmlTextField := TextField new.
	xmlTextField
		x: 10; y: 100;
		background: true; border: true;
		autoSize: TextFieldAutoSize left.
	stage addChild: xmlTextField.

	postTextField := TextField new.
	postTextField
		x: 10; y: 10;
		border: true; background: true;
		autoSize: TextFieldAutoSize left.
	stage addChild: postTextField.

	request := URLRequest new: 'https://www.w3.org/'.
	request basicAuth: #basic value: #basic.
	request cookieString: 'name=value'.

	loader := URLLoader request: nil.
	loader	addEventListener: IOErrorEvent ioError doing: [:e |
			# #print: #Html with: #load with: #error. nil ];
		addEventListener: Event complete doing: [:e | | data |
			data := loader data.
			xmlTextField text: (data copyFrom: 1 to: 2000). nil
		]; addEventListener: ProgressEvent progress doing: [:e |
			# #print: 'Html Loaded ', (e#bytesLoaded / e#bytesTotal * 100) rounded ,'%'. nil
		].
	loader load: request.
		
	imageLoader := Loader new.
	imageLoader contentLoaderInfo
		addEventListener: Event complete  doing: [:e | | bmp bdata |
# #print: #image with: #complete."
			bmp := imageLoader content as: Bitmap.
			bdata := bmp bitmapData."
			# #print: 'Image Loaded' ", bdata width, 'x', bmp bdata height ". nil
		].
	imageLoader addEventListener: IOErrorEvent ioError doing: [:e |
		# #print: 'Error loading image'. nil
	].

	request := URLRequest new: 'https://picsum.photos/200/300/?random'.
	request contentType: 'image/jpeg'.
	imageLoader load: request context: nil; 
	x: 15; y: 190; scaleX: 0.5; scaleY: 0.5.
	stage addChild: imageLoader.

	post := URLRequest new: 'https://www.w3schools.com/php/welcome.php'.
	vars := URLVariables new: nil.
	vars
		name: 'Milla';
		email: 'Jovovich@hotmail.com';
		submit: '1'.
	post
		method: URLRequestMethod post;
		data: vars.
	Lib sys trace: post data.
	postLoad := URLLoader request: nil.
	postLoad	addEventListener: IOErrorEvent ioError doing: [:e |
		# #print: 'form post error: ', e #_toString. nil
	];	addEventListener: Event complete doing: [:e | | data |
# #print: #postLoad with: #complete.
		data := postLoad data.
"# #print: #data with: data class name with: data size.
"		postTextField text: data. nil
	].
	postLoad load: post! !

"-Xml-----------------------------------------------------------------"!
! Xml class methodsFor: #private !
typeName: nodeType
	" Private - Returns the name of noteType. "

	#(Element PCData CData Comment DocType ProcessingInstruction Document) do: [:each|
		(Xml type: each) = nodeType ifTrue: [ ^each ].
	].
	^#Unknown! !

! Xml class methodsFor: #samples !
example
	" Answer node type name "

	| xml html body bodyElements |
	xml := Xml parse: '<html><body><a href=''#test''>test1</a><div><!![','CDATA','[test2]',']></div></body></html>'.
	html := xml firstElement.
	self note: html nodeName ,' type=' ,(self typeName: html nodeType) ,' class=' ,html class name.
	body := html firstElement.
	self note: body nodeName ,' type=' ,(self typeName: body nodeType) ,' class=' ,body class name.
	body do: [:element |
		self note: element nodeName ,' ' ,(self typeName: element nodeType) ,' class=' ,element class name.
		(element exists: #href) ifTrue: [
			element attributes do: [:value| self note: 'attr=' ,value ]
		] ifFalse: [ element set: #width value: 10 ].
		(element iterator asArrayOf: Xml) do: [:child|
			self note: 'child ' ,child nodeValue ,' ' ,(self typeName: child nodeType)
		]
	].
	body addChild: (Xml createElement: #br).
	self print:  xml toString! !

"-Multitouch-----------------------------------------------------------------------"!
! Multitouch class methodsFor: #sample !
example
	" sample "
	
	| sprite radius sp sp2 primaryTouch stage updatePosition |
	stage := Display stage.
	radius := 50.
	
	sp := Display sprite new.
	sp graphics
		beginFill: 16r00FF00 alpha: 1;
		drawCircle: radius y: radius radius: radius;
		endFill.
	sp cacheAsBitmap: true.
	
	sp2 := Display sprite new.
	sp2 graphics 
		beginFill: 16r0000FF alpha: 1;
		drawCircle: radius y: radius radius: radius;
		endFill.
	sp2 cacheAsBitmap: true.

	Multitouch supportsTouchEvents ifTrue: [
		Lib sys trace: Multitouch inputMode.
		# #print: Multitouch maxTouchPoints.
		
		stage 
			addChild: sp;
			addChild: sp2.
		
		primaryTouch := nil.
		
		updatePosition := [:e | | touchEvent |
			touchEvent := TouchEvent @ e.
			touchEvent touchPointID = primaryTouch ifTrue: [ 
				sp x: touchEvent stageX - radius.
				sp y: touchEvent stageY - radius.
			] ifFalse: [
			sp2 x: touchEvent stageX - radius.
			sp2 y: touchEvent stageY - radius.
			]. nil
		].
		
		stage addEventListener: TouchEvent touchBegin with: [:e | | event |
			event := TouchEvent @ e.
			primaryTouch = nil ifTrue: [
				primaryTouch := event touchPointID
			]. nil
		].
		stage addEventListener: TouchEvent touchMove 
			with: updatePosition.
		stage addEventListener: TouchEvent touchEnd with: [:e | | event |
			event := TouchEvent @ e.
			event touchPointID = primaryTouch ifTrue: [
				primaryTouch := nil
			]. nil
		]
	] ifFalse: [
		self print: 'Multitouch not supported on this device'.
		stage addChild: sp.
		stage addEventListener: MouseEvent mouseMove with: [:e | | event |
			event := MouseEvent @ e.
			sp x: event stageX - radius.
			sp y: event stageY - radius.
			nil
		]
	]! !

"-POC-------------------------------------------------------------------"!
Object
	subclass: #POC
	category: #LuaStudioSamples!

! POC class methodsFor: #samples !
example
	" POC example "

	^self new open! !

! POC methodsFor: #gui !
open
	" Open the receiver "
	
	| window screenDpi sx input target button title |
	window := Display stage.
	screenDpi := Capabilities screenDPI. 
	sx := screenDpi > 120 ifTrue: [ screenDpi / 120 ] ifFalse: [ 1 ].
	window scaleX: sx.
	window scaleY: sx.
	
	self addTitle: window text: 'Input Text' x: 10 y: 30.
	
	input := Text textField new
		x: 10; y: 50;
		type: TextFieldType input;
		wordWrap: true; multiline: true;
		width: 240; height: 50;
		text: '';
		border: true; borderColor: 16r000000;
		background: true; 
		backgroundColor: 16rFFFFFF;
		yourself.
	window addChild: input.
	
	self addTitle: window text: 'Output Text' x: 10 y: 130.
	target := Text textField new
		x: 10; y: 150;
		wordWrap: true; multiline: true;
		width: 240; height: 50;
		text: '';
		border: true; borderColor: 16r000000; 
		background: true;
		backgroundColor: 16rFFFFFF;
		yourself.
	window addChild: target.
	
	button := Text textField new
		autoSize: TextFieldAutoSize left;
		x: 10; y: 220;
		width: 240; height: 100;
		text: 'UPPERCASE STRING';
		border: true; borderColor: 16r0000FF;
		background: true; 
		backgroundColor: 16rF0F0FF;
		selectable: false;
		yourself.
	window addChild: button.
	
	input addEventListener: Event change with: [:e | 
		target text: input text reversed. nil
	].
	button addEventListener: MouseEvent click with: [:e |
		target text: target text asUpperCase. nil
	]! !

! POC methodsFor: #gui !
addTitle: parent text: text x: x y: y
	" Private - Add title "

	parent addChild: (Text textField new
		autoSize: TextFieldAutoSize left;
		x: x; y: y; width: 240; height: 20;
		text: text;
		border: false; background: true; 
		backgroundColor: 16rF0F0FF;
		selectable: false;
		yourself)! !

"-HTTP GET Server (blocking)-------------------------------------------------------------------"!
! Host class methodsFor: #sample !
example
	" server - Implements a simple (blocking) HTTP GET Server.
	The server will respond with details on each request.
	Until it is requested /quit or /exit
	"

	| host port socket client crlf |
	crlf := 13 asCharacter ,10 asCharacter.
	port := 8088.
	host := Host @ '192.168.1.137'.
	socket := Socket new.
	socket bind: host port: port.
	socket listen: 10.
	[ client := socket accept.
	  client notNil ] whileTrue: [ | data line result |
		[	line := client input readLine.
			line notEmpty ] whileTrue: [
			data := data isNil ifTrue: [line] ifFalse: [ data ,crlf ,line ]
		].
		"# #print: #data with: data."
		result := '<html><head><title>Hello</title></head><body>Hello S8 world'
			,'<hr><pre>' ,(data replace: crlf with: '<br>'),'</pre>'
			,'</body></html>'.
		client write:
			 'HTTP/1.1 200 OK' ,crlf
			,'Access-Control-Allow-Origin: *' ,crlf
			,'Content-Type: text/html' ,crlf
			,'Content-Length: ' ,result size ,crlf
			,crlf ,result.
		client close.
		(#(exit quit) includes: (data stream upTo: $/;upTo: $ )) ifTrue: [
			# #print: #exit.
			^socket close
		].
		# #print: #done.
	].
	socket close.
	# #print: #OK! !