
Object	subclass: #LoveApplication
	category: 'LoveApplication'!
LoveApplication class instanceVariableNames: #current!
LoveApplication class buildAccessors: #(
	#(current initialize 'self new')
)!

! LoveApplication class methodsFor: #events !
constructEventsTriggered
	" Private - Return the events managed by instances of the receiver. "

	^#(	aboutToInstall installed
		load draw update:
		focus: mouseFocus:
		lowMemory quit thread:error:
		directoryDropped: fileDropped:
		resize: visible:
		keyDown:scancode:isRepeat:
		keyUp:scancode:
		textEdited:start:length: textInput:
		mouse:moved:isTouch:
		mouse:pressed:isTouch:presses:
		mouse:released:isTouch:presses:
		wheelMoved:
		gamepad:axis:value: gamepadDown:button: gamepadUp:button:
		joystickAdded: joystickRemoved:
		joystick:axis:value: joystick:hat:direction:
		joystickDown:button: joystickUp:button:
		touch:moved:by:pressure:
		touch:down:by:pressure:
		touch:up:by:pressure:
	) ,super constructEventsTriggered! !

! LoveApplication methodsFor: #installation !
installGeneralCallbacks
	" Private - Set callbacks of this type "

	Love	on: #load do: [ self triggerEvent: #load ];
		on: #draw do: [ self triggerEvent: #draw ];
		on: #lowmemory do: [ self triggerEvent: #lowMemory ];
		on: #update do: [:dt | self triggerEvent: #update: with: dt ];
		on: #quit do: [ false ~~ (self triggerEvent: #quit) ];
		on: #threaderror do: [:thread :errorString|
			self triggerEvent: #thread:error: withArguments: (Array
				with: Thread @ thread
				with: errorString)
		];
		yourself! !

! LoveApplication methodsFor: #installation !
installWindowCallbacks
	" Private - Set callbacks of this type "

	Love	on: #directorydropped do: [:path |
			self triggerEvent: #directoryDropped: with: path
		];
		on: #filedropped do: [:file |
			self triggerEvent: #fileDropped: with: File @ file
		];
		on: #focus do: [:focus | self triggerEvent: #focus: with: focus ];
		on: #mousefocus do: [:focus | self triggerEvent: #mouseFocus: with: focus ];
		on: #resize do: [:w :h | self triggerEvent: #resize: with: w@h ];
		on: #visible do: [:visible | self triggerEvent: #visible: with: visible ];
		yourself! !

! LoveApplication methodsFor: #installation !
installKeyboardCallbacks
	" Private - Set callbacks of this type "

	Love	on: #keypressed do: [:key :scancode :isrepeat|
			self triggerEvent: #keyDown:scancode:isRepeat: withArguments: (Array
				with: key with: scancode with: isrepeat)
		];
		on: #keyreleased do: [:key :scancode|
			self triggerEvent: #keyUp:scancode: withArguments: (Array
				with: key with: scancode)
		];
		on: #textedited do: [:text :start :length|
			self triggerEvent: #textEdited:start:length: withArguments: (Array
				with: text with: start with: length)
		];
		on: #textinput do: [:text| self triggerEvent: #textInput: with: text ];
		yourself! !

! LoveApplication methodsFor: #installation !
installMouseCallbacks
	" Private - Set callbacks of this type "

	Love	on: #mousemoved do: [:x :y :dx :dy :isTouch|
			self triggerEvent: #mouse:moved:isTouch: withArguments: (Array
				with: x@y with: dx@dy with: isTouch)
		];
		on: #mousepressed do: [:x :y :button :isTouch :presses|
			self triggerEvent: #mouse:pressed:isTouch:presses: withArguments: (Array
				with: x@y with: button with: isTouch with: presses)
		];
		on: #mousereleased do: [:x :y :button :isTouch :presses|
			self triggerEvent: #mouse:released:isTouch:presses: withArguments: (Array
				with: x@y with: button with: isTouch with: presses)
		];
		on: #wheelmoved do: [:x :y |
			self triggerEvent: #wheelMoved: with: x@y
		];
		yourself! !

! LoveApplication methodsFor: #installation !
installJoystickCallbacks
	" Private - Set callbacks of this type "

	Love	on: #gamepadaxis do: [:joystick :axis :value|
			self triggerEvent: #gamepad:axis:value: withArguments: (Array
				with: Joystick @ joystick with: axis with: value)
		];
		on: #gamepadpressed do: [:joystick :button|
			self triggerEvent: #gamepadDown:button: withArguments: (Array
				with: Joystick @ joystick with: button)
		];
		on: #gamepadreleased do: [:joystick :button|
			self triggerEvent: #gamepadUp:button: withArguments: (Array
				with: Joystick @ joystick with: button)
		];
		on: #joystickadded do: [:joystick|
			self triggerEvent: #joystickAdded: with: Joystick @ joystick
		];
		on: #joystickaxis do: [:joystick :axis :value|
			self triggerEvent: #joystick:axis:value: withArguments: (Array
				with: Joystick @ joystick with: axis with: value)
		];
		on: #joystickhat do: [:joystick :hat :direction|
			self triggerEvent: #joystick:hat:direction: withArguments: (Array
				with: Joystick @ joystick with: hat with: direction)
		];
		on: #joystickpressed do: [:joystick :button|
			self triggerEvent: #joystickDown:button: withArguments: (Array
				with: Joystick @ joystick with: button)
		];
		on: #joystickreleased do: [:joystick :button|
			self triggerEvent: #joystickUp:button: withArguments: (Array
				with: Joystick @ joystick with: button)
		];
		on: #joystickremoved do: [:joystick|
			self triggerEvent: #joystickRemoved: with: Joystick @ joystick
		];
		yourself! !

! LoveApplication methodsFor: #installation !
installTouchCallbacks
	" Private - Set callbacks of this type "

	Love	on: #touchmoved do: [:id :x :y :dx :dy :pressure|
			self triggerEvent: #touch:moved:by:pressure: withArguments: (Array
				with: id with: x@y with: dx@dy with: pressure)
		];
		on: #touchpressed do: [:id :x :y :dx :dy :pressure|
			self triggerEvent: #touch:down:by:pressure: withArguments: (Array
				with: id with: x@y with: dx@dy with: pressure)
		];
		on: #touchreleased do: [:id :x :y :dx :dy :pressure|
			self triggerEvent: #touch:up:by:pressure: withArguments: (Array
				with: id with: x@y with: dx@dy with: pressure)
		];
		yourself! !

! LoveApplication methodsFor: #installation !
install
	" Private - Set callbacks "

	self	triggerEvent: #aboutToInstall;
		installGeneralCallbacks;
		installWindowCallbacks;
		installKeyboardCallbacks;
		installMouseCallbacks;
		installJoystickCallbacks;
		installTouchCallbacks;
		triggerEvent: #installed! !

