" This code is published under MIT license.
Has been written to run in Lve as demostration on how to write S8 applications that run on Lua VMs.
Read more:
	swiki:s8-media/sampleLoveIndex
	swiki:U8/m8nS8OnLua
"!

Object
	subclass: #SampleObject
	category: #LoveSamples!

SampleObject
	subclass: #Player
	instanceVariableNames: ' body shape fixture img speed jump yVelocity x y '!
Player buildAccessors!

SampleObject
	subclass: #Platform
	instanceVariableNames: ' body shape fixture '!
Platform buildAccessors!

SampleObject
	subclass: #Block
	instanceVariableNames: ' body shape fixture '!
Block buildAccessors!

SampleObject
	subclass: #BouncingU8
	instanceVariableNames: ' player platform block world '!
BouncingU8 buildAccessors: #(
	#(player initialize Player)
	#(platform initialize Platform)
	#(block initialize Block)
	#world
)!

! BouncingU8 class methodsFor: #samples !
open
	" Open the receiver "

	BouncingU8 new open! !

! BouncingU8 class methodsFor: #samples !
canw
	" Answer canw "

	^800! !

! BouncingU8 class methodsFor: #samples !
canv
	" Answer canv "

	^650! !

! BouncingU8 methodsFor: #actions !
open
	" Open the receiver "

	self install! !

! BouncingU8 methodsFor: #samples !
canw
	" Answer canw "

	^self class canw! !

! BouncingU8 methodsFor: #samples !
canv
	" Answer canv "

	^self class canv! !

! BouncingU8 methodsFor: #installation !
install
	" Private - Set callbacks "

	Love
		on: #load do: [ self load ];
		on: #draw do: [ self draw ];
		on: #update do: [:dt | self update: dt ];
		yourself! !

! BouncingU8 methodsFor: #private !
load
	" Private - Load the receiver "

	Physics meter: 64. "1 meter = 64px"
	self world: (Physics newWorld: 0 yg: 9*64 sleep: true). "hgrav=0,vgrav=9"
	self platform
		body: (Physics newBody: self world x: self canw/2 y: self canv-(50/2));
		shape: (Physics newRectangleShape: self canw y: 50);
		fixture: (Physics newFixture: self platform body shape: self platform shape).
	self player
		img: (Graphics newImage: 'images/u8.png');
		body: (Physics newBody: self world x: self canw/2 y: self canv/2 type: 'dynamic');
		shape: (Physics
			newRectangleShape: self player img width
			y: self player img height);
		fixture: (Physics
			newFixture: self player body
			shape: self player shape
			density: 1).
	self player fixture restitution: 1.
	self player
		speed: 200;
		yVelocity: 0;
		jump: -300;
		y: self canv-(50/2);
		x: Graphics width / 2.
	self block
		body: (Physics newBody: self world x: 200 y: 550 type: 'dynamic');
		shape: (Physics newRectangleShape: 0 y: 0 width: 50 height: 100);
		fixture: (Physics
			newFixture: self block body
			shape: self block shape
			density: 5).
	Graphics backgroundColor: 104 green: 136 blue: 248.
	Window mode: self canw height: self canv.! !

! BouncingU8 methodsFor: #private !
draw
	" Private - Draw the receiver. "

	| s |
"Debug-"	SystemServer current step.
	Graphics color: 246 green: 246 blue: 250.
	s := self platform shape.
{'	love.graphics.polygon("fill", self:platform():body():handle():getWorldPoints(s:handle():getPoints()))
'}.
	Graphics
		color: 255 green: 255 blue: 255;
		draw: self player img x: self player body x
		y: self player body y r: self player body angle
		sx: 1 sy: 1 ox: self player img width / 2
		oy: self player img height / 2.
	Graphics	color: 50 green: 50 blue: 50.
	s := self block shape.
{'	love.graphics.polygon("fill", self:block():body():handle():getWorldPoints(s:handle():getPoints()))
'}.
	Graphics
		color: 0 green: 0 blue: 0;
		print: self player body y x: self canw/2 y: self canv/2;
		print: self platform body y x: self canw/3 y: self canv/2.! !

! BouncingU8 methodsFor: #private !
update: dt
	" Private - Update the receiver. "

	self world update: dt. "start the world"
	Graphics
		print: self player body y x: self canw/2 y: self canv/2;
		print: self platform body y x: self canw/3 y: self canv/2.
	(Keyboard isDown: 'right') ifTrue: [
		self player img: (Graphics newImage: 'images/u8.png').
		(Graphics width - self player img width) > self player body x ifTrue: [
			self player body
				linearVelocity: self player speed
				y: 0 + (self player body y / 3)
		] ifFalse: [
			(Keyboard isDown: 'left') ifTrue: [
				self player img: (Graphics newImage: 'images/u8r.png').
				self player x > 0 ifTrue: [
					self player body
						linearVelocity: -1 * self player speed
						y: 0 + (self player body y / 3)
				]
			]
		]
	].
	(Keyboard isDown: 'space') ifTrue: [
		self player body y + 200 > self platform body y ifTrue: [
			self player body linearVelocity: 0 y: self player jump
		]
	].
	(Keyboard isDown: 'down') ifTrue: [
		self player body position: 650/2 y: 650/2.
		self player body linearVelocity: 0 y: 0.
	].! !