! Object methodsFor: #Mobile-debug !
-- text
	" DEBUG - Output text onto debug console.
	The default action is to do nothing.
	This method can be overwritten by debugging tools (e.g. MobileLog).
	"! !

EventManager
	subclass: #MobileApplication
	instanceVariableNames:
		 ' settings screen mainView title '
		,' fullScreen views templates current '
		,' model threads '
	category: #Mobile !
MobileApplication comment: '
	Application designed to run on aMobileDevice.
	@2011 Alejandro Reimondo - aleReimondo@smalltalking.net
'!
MobileApplication buildAccessors: #(
	#( #settings #initialize 'self triggerEvent: #needsSettings with: self ifNotHandled: [ self defaultSettings ]' )
	#( #screen #initialize 'self device screenFor: self' )
	#( #fullScreen #initialize 'self defaultFullScreen' )
	#( #title #initialize 'self class name ,#@ ,self device name' )
	#( #views #initialize 'self triggerEvent: #needsViews with: self ifNotHandled: [ self defaultViews ]' )
	#( #templates #initialize 'PoolDictionary new' )
	#( #threads #initialize '#()')
	#( #mainView #initialize 'self triggerEvent: #needsMainView with: self ifNotHandled: [ self defaultMainView ]' )
	#( #model #initialize 'self triggerEvent: #needsModel with: self' )
	#current
).!

! MobileApplication class methodsFor: #events !
constructEventsTriggered
	" Private - Return the events managed by instances of the receiver. "

	^#(	#aboutToOpen
		#opened
		#initialized
		#aboutToClose #aboutToClose:
		#closed
		#needsMainView
		#needsModel
		#needsViews
		#needsSettings
		#needsImageNamed:
		#needsSourceForImage:
	) ,super constructEventsTriggered! !

! MobileApplication class methodsFor: #instantiation !
fromJSon: aJSon
	" Return an instance of the receiver set with aJSon. "

	^self new fromJSon: aJSon! !
		
! MobileApplication class methodsFor: #instantiation !
@ aJSon
	" Return an instance of the receiver set with aJSon. "

	^self new fromJSon: aJSon! !
		
! MobileApplication class methodsFor: #gui !
open: aJSon
	" Open an instance of the receiver set with aJSon. "

	^(self @ aJSon) open! !

! MobileApplication class methodsFor: #gui !
open
	" Open an instance of the receiver. "

	^self new open! !

! MobileApplication class methodsFor: #titles !
defaultTitleFor: label
	" Return the default title for label. "

	^(label upTo: $-) asGlobalName! !

! MobileApplication class methodsFor: #private !
imageLibrary
	" Private - Return the name of image repository. "

	^nil! !

! MobileApplication class methodsFor: #defaults !
mainViewSupport
	" Private - Return the support for mainView(or nil). "

	^TopPane! !

! MobileApplication class methodsFor: #gui !
newImage: fileName
	" Return an unbound image widget with image obtained from image repository. "

	^self device imageWidgetSupport
		fileName: fileName
		in: self imageLibrary! !

! MobileApplication class methodsFor: #gui !
openFullScreen
	" Open an instance of the receiver. "

	^self new openFullScreen! !

! MobileApplication methodsFor: #accessing !
device
	" Return the device where the receiver is running. "

	^MobileDevice current! !

! MobileApplication methodsFor: #gui !
defaultFullScreen
	" Private - Return true if the receiver should be opened in fullScreen mode. "

	^self device fullScreen! !

! MobileApplication methodsFor: #gui !
aboutToOpen
	" The receiver is to be opened. "

	self triggerEvent: #aboutToOpen! !

! MobileApplication methodsFor: #gui !
bringToFront
	" Bring the mainView of the receiver to front of opened windows (at same level). "

	^self mainViewOrNil notNil ifTrue: [
		self mainView bringToFront
	]! !

! MobileApplication methodsFor: #gui !
close
	" Private - Close the receiver. "

	self triggerEvent: #aboutToClose.
	self isOpen ifFalse: [ ^self ].
	self closeMainView.
	self triggerEvent: #closed.
	mainView := nil.! !

! MobileApplication methodsFor: #private !
closeMainView
	" Private - Close the mainView of the receiver. "

	mainView notNil ifTrue: [ mainView close ].! !

! MobileApplication methodsFor: #defaults !
defaultMainView
	" Private - Return a new mainView for the receiver(or nil). "

	^self device mainViewFor: self! !

! MobileApplication methodsFor: #accessing !
actionsLiteral
	" Private - Return the specification of actions to be arranged in the actions bar. "

	^#()! !

! MobileApplication methodsFor: #private !
imageLibrary
	" Private - Return the name of image repository. "

	^self class imageLibrary! !

! MobileApplication methodsFor: #initialize !
initializeMainView
	" Private - Initialize the mainView of the receiver. "

	mainView := self
		triggerEvent: #needsMainView
		ifNotHandled: [ self defaultMainView ].! !

! MobileApplication methodsFor: #query !
isOpen
	" Return true if the receiver is open. "

	^mainView notNil and: [ mainView isOpen ]! !

! MobileApplication methodsFor: #accessing !
mainView: aWidget
	" Set the mainView of the receiver. "

	mainView notNil ifTrue: [ mainView close ].
	mainView := aWidget! !

! MobileApplication methodsFor: #query !
mainViewOrNil
	" Return the mainView of the receiver(or nil). "

	^mainView! !

! MobileApplication methodsFor: #defaults !
mainViewSupport
	" Private - Return the support for mainView of the receiver(or nil). "

	^self class mainViewSupport! !

! MobileApplication methodsFor: #gui !
openFullScreen
	" Private - Open the receiver in fullScreen mode. "

	self fullScreen: true.
	^self open! !

! MobileApplication methodsFor: #gui !
open
	" Private - Open the GUI of receiver. "

	^self perform: (self device selectorToOpen: self)! !

! MobileApplication methodsFor: #private !
basicOpen
	" Private - Open the GUI of receiver. "

	^self
		aboutToOpen;
		mainView;
		setInitialContents;
		triggerEvent: #initialized;
		openMainView;
		opened;
		yourself! !

! MobileApplication methodsFor: #gui !
openIn: aMainView
	" Open the receiver contained/embedded in aMainView.
	Note the mainView will NOT be closed when the receiver is closed; it must be handled by client.
	"

	self mainView: aMainView.
	self when: #aboutToClose do: [
		self aboutToClose.
		mainView := nil
	].
	^self open! !

! MobileApplication methodsFor: #gui !
openTabs
	" Open the GUI of receiver in a tabbedUI. "

	^self device openTabsFor: self! !

! MobileApplication methodsFor: #gui !
aboutToClose
	" The receiver is to be closed. "

	self triggerEvent: #aboutToClose:
		withArguments: (Array with: self)! !

! MobileApplication methodsFor: #private !
openMainView
	" Private - Open the mainView of the receiver. "

	mainView notNil ifTrue: [ mainView open ].! !

! MobileApplication methodsFor: #private !
scaleMainView
	" Private - Scale the mainView of the receiver. "

	^mainView notNil ifTrue: [
		self device scaleMainViewFor: self
	]! !

! MobileApplication methodsFor: #gui !
opened
	" The receiver has just been opened. "

	self mainView 
		when: #aboutToClose
		send: #aboutToClose
		to: self.
	self	triggerEvent: #opened with: self
		ifNotHandled: [
			self threads notEmpty ifTrue: [ self threads first activate ].
			self scaleMainView.
		].! !

! MobileApplication methodsFor: #accessing !
screenExtent
	" Private - Return the extent of the device screen (in device units). "

	^self device screenExtent! !

! MobileApplication methodsFor: #accessing !
statusBarHeight
	" Private - Return the status bar height (in device units). "

	self fullScreen ifTrue: [ ^0 ].
	^self device statusBarHeight! !

! MobileApplication methodsFor: #accessing !
screenUnits
	" Private - Return the units of the device screen. "

	^self device units! !

! MobileApplication methodsFor: #initialize !
setInitialContents
	" Private - Initialize the initial contents of the receiver.
	The default action set the screen to render contents, and open startup task.
	"

	| theScreen |
	theScreen := self screen.
	theScreen when: #configured send: #renderContents to: self.
	self mainView addChild: theScreen! !

! MobileApplication methodsFor: #defaults !
defaultViews
	" Private - Return the default views of the receiver.
	This method can be refined by subclasses to customize app views at startup.
	"

	^Array new! !

! MobileApplication methodsFor: #defaults !
defaultSettings
	" Private - Return the default settings of the receiver.
	This method can be refined by subclasses to load settings from external storage.
	"

	^PoolDictionary new! !

! MobileApplication methodsFor: #views !
@ aName
	" Return the task(aCoordinator) named aName (or nil). "

	^self views
		detect: [:each| each name = aName ]
		ifNone: [ nil ]! !

! MobileApplication methodsFor: #views !
active
	" Private - Return the active task(aCoordinator) of the receiver (or nil). "

	^self views notEmpty ifTrue: [ self views first ]! !

! MobileApplication methodsFor: #gui !
activate: task
	" Activate task(aCoordinator) in the receiver. "

	self device
		activate: task for: self
		doing: [
			self views
				remove: task ifAbsent: [];
				addFirst: task.
			self update.
		]! !

! MobileApplication methodsFor: #gui !
renderContents
	" Render the contents of the receiver. "

	self device renderApplicationContents: self! !

! MobileApplication methodsFor: #gui !
update
	" Update the receiver. "

	self screen update.! !

! MobileApplication methodsFor: #gui !
clearScreen
	" Clear the screen contents. "

	self screen clear! !

! MobileApplication methodsFor: #gui !
defaultSourceForImage: aSymbol

	^self device sourceForImage: aSymbol! !	

! MobileApplication methodsFor: #gui !
defaultImageNamed: aSymbol

	^self device imageNamed: aSymbol! !

! MobileApplication methodsFor: #gui !
imageNamed: aSymbol

	| result |
	result:= self triggerEvent: #needsImageNamed:
		withArguments: (Array with: self with: aSymbol)
		ifNotHandled: [ nil ].
	result isNil ifTrue: [ ^self defaultImageNamed: aSymbol ].
	^result! !

! MobileApplication methodsFor: #gui !
sourceForImage: aSymbol

	| result |
	result:= self triggerEvent: #needsSourceForImage:
			withArguments: (Array with: self with: aSymbol)
			ifNotHandled: [ nil ].
	result isNil ifTrue: [ ^self defaultSourceForImage: aSymbol ].
	^result! !

! MobileApplication methodsFor: #gui !
openTask: aTask
	" Open aTask on screen. "

	(aTask isNil or: [self isOpen not]) ifTrue: [ ^nil ].
	aTask isString ifTrue: [ ^self openTask: self @ aTask ].
	^aTask openIn: self! !

! MobileApplication methodsFor: #views !
wakeup: aBlock in: milliseconds
	" Evaluate aBlock after milliseconds intervals.
	Return the timer ID that can be used to cancel with
		# #clearTimeout: id
	"

	^milliseconds notNil ifTrue: [
		aBlock valueWithInterval: milliseconds
	]! !

! MobileApplication methodsFor: #gui !
warning: explanation
	" Open a modal dialog with explanation shown to the user. "

	^self device warning: self text: explanation doing: [ "nothing" ]! !

! MobileApplication methodsFor: #gui !
confirm: question action: actionAssoc
	" Prompt the user for confirmation and evaluates actionAssoc when confirmed. "

	^self device confirm: self text: question action: actionAssoc! !

! MobileApplication methodsFor: #gui !
dialog: explanation
	custom: customBlock
	doing: anAction
	closeOnClick: closeOnClick
	" Open a modal dialog customized by customBlock. "

	^self device 
		dialog: self
		text: explanation
		custom: customBlock
		doing: anAction
		closeOnClick: closeOnClick! !

! MobileApplication methodsFor: #prompt !
peekA: aTitle selecting: initialSelection in: options doing: anAction
	" Open a list peeker prompting aValue from list of options. "

	^self	peekA: aTitle
		selecting: initialSelection in: options
		doing: anAction onClose: [ "nothing" ]! !

! MobileApplication methodsFor: #prompt !
peekA: aTitle
	selecting: initialSelection in: options
	doing: anAction onClose: closeBlock
	" Open a list peeker prompting aValue from list of options. "

	^self device
		peek: self title: aTitle
		selecting: initialSelection in: options
		doing: anAction onClose: closeBlock! !

! MobileApplication methodsFor: #geo !
instructionsToTarget: aLocation
	latitude: latitude longitude: longitude
	on: aTaskDirectionsView
	" Asist the user with directions to target. "

	^self device
		instructionsToTarget: aLocation
		latitude: latitude longitude: longitude
		on: aTaskDirectionsView! !

! MobileApplication methodsFor: #gui !
activateModal: selector
	" Open/activate (as modal UI) the template at selector. "

self --('// activateModal: ',selector toString).
	^MobileThreadModal new
		application: self;
		fromJSon: selector;
		activate! !

! MobileApplication methodsFor: #spec !
fromJSon: jsObject
	" Private - Set the receiver from jsObject. "

	self basicAt: #spec put: jsObject.
	JSonConfigurator configure: self fromJSon: jsObject! !

! MobileApplication methodsFor: #spec !
listeners: jsObject
	" Private - Set the listeners of the receiver from jsObject. "

	JS @ jsObject propertiesDo: [:key :value |
		self when: key do: value
	]! !

! MobileApplication methodsFor: #spec !
templates: jsObject
	" Private - Set the templates of the receiver from jsObject. "

	JS @ jsObject propertiesDo: [:key :value|
		value isNil ifTrue: [ self -- ('Missing template #',key) ].
		self templates
			at: key
			put: (MobileCoordinatorTemplate fromJSon: value)
	]! !

! MobileApplication methodsFor: #spec !
threads: jsObject
	" Private - Set the threads of the receiver from jsObject. "

	threads := #().
	JS @ jsObject propertiesDo: [:key :value|
		value isNil ifTrue: [ self -- ('Missing thread #',key) ].
		threads add: (
			(self device newThreadFor: self)
				fromJSon: value;
				yourself)
	]! !

! MobileApplication methodsFor: #error !
templateNotFoundAt: indexOrKey
	" Private - Generates an error.
	The object at key/index was not found in the receiver's contents.
	"
self--(#templateNotFoundAt: ,indexOrKey printString).
	#{smalltalk} #objectNotFound: nil at: indexOrKey in: self.
	^self error: 'Template not found at ',indexOrKey toString,' in ',self printString! !

! MobileApplication methodsFor: #accessing !
templateAt: aKey ifAbsent: aBlock
	" Return the template of the receiver at aKey, or the result of evaluating aBlock (if not found). "

	^self templates at: aKey ifAbsent: [ "can be a global"
		Smalltalk at: aKey ifAbsent: aBlock
	]! !

! MobileApplication methodsFor: #accessing !
templateAt: aKey
	" Return the template of the receiver at aKey. "

	^self templateAt: aKey ifAbsent: [ self templateNotFoundAt: aKey ]! !

! MobileApplication methodsFor: #multimedia !
openFileAt: pathName
	" Open the (local) file at pathName. "

	^self device openFileAt: pathName! !

! MobileApplication methodsFor: #multimedia !
openFileAtUrl: url
	" Download and open the file at url. "

	^self device openFileAtUrl: url! !

! MobileApplication methodsFor: #multimedia !
openFileContents: fileContents type: filetype version: version
	" Save the contents in local storage and open as a document of specified type. "

	^self device openFileContents: fileContents type: filetype version: version! !

! MobileApplication methodsFor: #debug !
debugUI: aBoolean
	" Set the receiver to debug UI operations.
	The default action is to do nothing.
	This method can be overwritten by debugging tools (e.g. MobileTracer).
	" ! !

! MobileApplication methodsFor: #private !
refresh
	" Refresh receivers UI "

	self fromJSon: (self basicAt: #spec).
	self open! !

! MobileApplication methodsFor: #private !
refreshThread: aMobileThread key: key
	" Refresh aMobileThread "

	| js |
	js := JS @ (self basicAt: #spec).
	(JS @ (js @ #threads)) 
		handleAt: key 
		put: (aMobileThread basicAt: #spec)! !

! MobileApplication methodsFor: #spec !
spec: jsObject
	" Update the receiver from jsObject. "

	self basicAt: #spec put: jsObject.
	self refresh! !

! MobileApplication methodsFor: #private !
keyForThread: aMobileThread
	" Answer the key of aMobileThread in the receiver "
	
	^self threads 
		indexOf: aMobileThread 
		ifAbsent: [ nil ]! !

EventManager
	subclass: #MobileCoordinator
	instanceVariableNames: ' name thread back template '
	category: #Mobile !
MobileCoordinator buildAccessors: #(
	#( #name #initialize 'self class name' )
	thread back template
	)!

! MobileCoordinator class methodsFor: #events !
constructEventsTriggered
	" Private - Return the smalltalk events managed by instances of the receiver. "

	^#(
	#built: closed
	) , super constructEventsTriggered! !

! MobileCoordinator methodsFor: #accessing !
applicationModel
	" Return the current application model of the receiver. "

	^self application model! !

! MobileCoordinator methodsFor: #accessing !
application
	" Return the current application of the receiver. "

	^self thread application! !

! MobileCoordinator methodsFor: #accessing !
device
	" Return the device where the receiver is running. "

	^self application device! !

! MobileCoordinator methodsFor: #accessing !
navigationTemplate
	" Return the navigationTemplate of the receiver. "

	^self template navigationTemplate! !

! MobileCoordinator methodsFor: #accessing !
widgetTemplates
	" Return the widgetTemplates of the receiver. "

	^self template widgetTemplates! !

! MobileCoordinator methodsFor: #gui !
openIn: aMobileApplication
	" Set the mobile app to show the receiver's view at current state. "

	^aMobileApplication activate: self! !

! MobileCoordinator methodsFor: #gui !
aboutToActivateIn: aMobileApplication
	" The receiver will be activated in aMobileApplication. "! !

! MobileCoordinator methodsFor: #gui !
activatedIn: aMobileApplication
	" The receiver has been activated in aMobileApplication. "! !

! MobileCoordinator methodsFor: #gui !
triggerClosed

	self -- (Array with: #closed with: self with: self title).
	self	triggerEvent: #closed
		withArguments: (Array with: self)
		ifNotHandled: [ ]! !

! MobileCoordinator methodsFor: #gui !
aboutClosed
	" OBSOLETE - will be removed"

	self	triggerClosed! !

! MobileCoordinator methodsFor: #navigation !
backActionIn: aMobileApplication
	" Return the action to return to previous page in history list. "

	^[ self thread back ]! !

! MobileCoordinator methodsFor: #gui !
buildContentsIn: aMobileApplication
	" Build the receiver's views at current state in aMobileApplication. "

	| screen |
	screen := aMobileApplication screen.
	self
		buildNavigationBar: screen navigationBar in: aMobileApplication;
		buildScreenContents: screen contents in: aMobileApplication;
		buildActionsBar: screen actionsBar in: aMobileApplication;
		yourself.
	self triggerEvent: #built: withArguments: (Array with: self)! !

! MobileCoordinator methodsFor: #gui !
buildNavigationBar: aNavigationBar in: aMobileApplication
	" Private - Build the navigation bar of the receiver.
	The default action do nothing (expose an empty bar).
	This method can be refined to expose custom GUI elements.
	"! !

! MobileCoordinator methodsFor: #gui !
buildActionsBar: anActionsBar in: aMobileApplication
	" Private - Build the actions bar of the receiver.
	The default action do nothing (expose an empty bar).
	This method can be refined to expose custom GUI elements.
	"! !

! MobileCoordinator methodsFor: #gui !
buildScreenContents: aWidget in: aMobileApplication
	" Private - Build the screen contents of the receiver.
	The default action do nothing (leave the screen blank).
	This method should be refined to let the user interact from views.
	"! !

! MobileCoordinator methodsFor: #navigation !
success: aMobileApplication
	" Private - Activate the next task. "

	aMobileApplication views size < 2 ifTrue: [
		^self error: 'Missing next view in #success:'
	].
	aMobileApplication views add: aMobileApplication views removeFirst.
	aMobileApplication openTask: aMobileApplication views first.! !

! MobileCoordinator methodsFor: #gui !
skipContentsIn: aMobileApplication
	" Modal coordinators skip to next w/o showing the contents. "

	| conf action |
self -- '//ToDo MobileCoordinator>>#skipContentsIn: & senders'.
	conf := JS @ self navigationTemplate.
	conf isNil ifTrue: [ ^self error: 'Missing navigation template in modal coordinator' ].
	conf := JS @ (conf @ #right).
	conf isNil ifTrue: [ ^self error: 'Missing #right in modal coordinator' ].
	conf keys do: [:key | action := [ conf @ key value: self ] ].
	self triggerEvent: #built: withArguments: (Array with: self).
	^action evaluate! !

! MobileCoordinator methodsFor: #drawing !
configure: cell title: aTitle
	" Private - Default configuration for cells in groups. "

	^self configure: cell title: aTitle description: ''! !

! MobileCoordinator methodsFor: #drawing !
configure: cell title: aTitle description: aDescription
	" Private - Default configuration for cells in groups. "

	^self configure: cell title: aTitle description: aDescription detail: nil! !

! MobileCoordinator methodsFor: #drawing !
configure: cell title: aTitle description: aDescription detail: aDetail
	" Private - Default configuration for cells in groups. "

	^self application device
		configure: cell title: aTitle
		description: aDescription detail: aDetail ! !

! MobileCoordinator methodsFor: #drawing !
configure: cell title: aTitle detail: aDetail
	" Private - Default configuration for cells in groups. "

	^self
		configure: cell title: aTitle
		description: nil detail: aDetail! !

! MobileCoordinator methodsFor: #drawing !
iconSourceFor: iconName group: aGroup
	" Private - Return the icon for item in a group. "

	^self application sourceForImage: iconName! !

! MobileCoordinator methodsFor: #navigation !
leftActionIn: aMobileApplication
	" Return the action of the button on this side.
	Return the action to return to previous page by default.
	"

	^self back notNil ifTrue: [ self backActionIn: aMobileApplication ]! !

! MobileCoordinator methodsFor: #accessing !
leftTitle
	" Return the title of the navigation button on this side (or nil). "

	^#back! !

! MobileCoordinator methodsFor: #navigation !
rightActionIn: aMobileApplication
	" Return the action of the button on this side.
	Return nil by default (no action).
	The method must be refined by subclasses with navigation actions on this side.
	"

	^nil! !

! MobileCoordinator methodsFor: #accessing !
rightTitle
	" Return the title of the navigation button on this side (or nil). "

	^#next! !

! MobileCoordinator methodsFor: #gui !
hide: what

	what isArray ifTrue: [ ^what do: [:each| self hide: each ] ].
	what isString ifTrue: [ ^self hide: self @ what ].
	^what hide! !

! MobileCoordinator methodsFor: #gui !
show: what

	what isArray ifTrue: [ ^what do: [:each| self show: each ] ].
	what isString ifTrue: [ ^self show: self @ what ].
	^what show! !

! MobileCoordinator methodsFor: #gui !
refresh: what

	what isArray ifTrue: [ ^what do: [:each| self refresh: each ] ].
	what isString ifTrue: [ ^self refresh: self @ what ].
	^what refresh! !



MobileCoordinator
	subclass: #RecurrentCoordinator
	instanceVariableNames: ' timeout '
	category: #Mobile !
RecurrentCoordinator comment: '
	Task to be activated after a timeout.
'!

! RecurrentCoordinator methodsFor: #cancelling !
cancel
	" Private - Cancel activation of the receiver. "

	^timeout notNil ifTrue: [ # clearTimeout: timeout ]! !

! RecurrentCoordinator methodsFor: #activation !
wakeup: aMobileApplication in: milliseconds
	" Set the receiver to activate/reopen after milliseconds (if it is not already set). "

	timeout isNil ifTrue: [
		timeout := aMobileApplication
			wakeup: [ timeout := nil. self wakeupIn: aMobileApplication ]
			in: milliseconds
	].! !

! RecurrentCoordinator methodsFor: #activation !
wakeupIn: aMobileApplication
	" Private - Wakeup/reactivate the receiver. "

	aMobileApplication openTask: self! !

MobileCoordinator subclass: #MobilePage !
MobilePage comment: '
	A page coordinator for one application stage.
	Enables support for back/next buttons and action bar.
	'!

MobilePage subclass: #MobileStartPage !
MobileStartPage comment: '
	A page coordinator of a start page (and/or action bar page). '!

! MobileStartPage methodsFor: #navigation !
leftTitle
	" Return the title of the navigation button on this side. "

	^nil "we do not have left button/action by default"! !

! MobilePage methodsFor: #accessing !
title
	" Return the title of the receiver. "

	^self implementedBySubclass: #title! !

! MobilePage methodsFor: #accessing !
rightTitle
	" Return the title of the navigation button on this side (or nil). "

	^#next! !

! MobilePage methodsFor: #accessing !
leftTitle
	" Return the title of the navigation button on this side (or nil). "

	^#back! !

! MobilePage methodsFor: #gui !
aboutToActivateIn: aMobileApplication
	" The receiver will be activated in aMobileApplication. "

	self back isNil ifTrue: [
		self back: aMobileApplication active
	].
	^super aboutToActivateIn: aMobileApplication! !

! MobilePage methodsFor: #gui !
buildActionsBar: anActionsBar in: aMobileApplication
	" Private - Build the actions/tasks bar of the receiver. "

	self -- (Array with: #create with: anActionsBar with: self).
	anActionsBar contents: (self actionsIn: aMobileApplication).! !

! MobilePage methodsFor: #actions !
actionsIn: aMobileApplication
	" Private - Return the actions to be arranged in task bar. "

	(aMobileApplication model respondsTo: #actionsLiteral) ifTrue: [ "Obsolete"
		^aMobileApplication model actionsLiteral collect: [:tuple|
			aMobileApplication device 
				tool: tuple first
				for: aMobileApplication
				doing: [ aMobileApplication openTask: tuple last ]
		]
	].
	^aMobileApplication threads collect: [:each|
		aMobileApplication device 
			tool: each tool
			for: aMobileApplication
			doing: [ each activate ]
	]! !

! MobilePage methodsFor: #navigation !
buildNavigationBar: aNavigationBar in: aMobileApplication
	" Private - Build the navigation bar of the receiver. "

	aNavigationBar title: self title.
	self -- (Array with: #create with: aNavigationBar with: self).
	self
		buildRight: aNavigationBar in: aMobileApplication;
		buildLeft: aNavigationBar in: aMobileApplication;
		yourself! !

! MobilePage methodsFor: #navigation !
buildLeft: aNavigationBar in: aMobileApplication
	" Private - Build the left side navigation bar of the receiver. "

	aNavigationBar left: self leftTitle action: (self leftActionIn: aMobileApplication).! !

! MobilePage methodsFor: #navigation !
buildRight: aNavigationBar in: aMobileApplication
	" Private - Build the right side navigation bar of the receiver. "

	aNavigationBar right: self rightTitle action: (self rightActionIn: aMobileApplication).! !

! MobilePage methodsFor: #navigation !
rightActionIn: aMobileApplication
	" Return the action of the button on this side.
	Return nil by default (no action).
	The method must be refined by subclasses with navigation actions on this side.
	"

	^nil! !

! MobilePage methodsFor: #navigation !
leftActionIn: aMobileApplication
	" Return the action of the button on this side. "

	^self back notNil ifTrue: [
		super backActionIn: aMobileApplication
	]! !

! MobilePage methodsFor: #navigation !
backActionIn: aMobileApplication
	" Return the action to return to previous page in history list. "

	^[ aMobileApplication openTask: self back ]! !

! MobilePage methodsFor: #drawing !
configure: cell title: aTitle description: aDescription detail: aDetail
	" Private - Default configuration for cells in groups. "

	(cell respondsTo: #group) ifTrue: [
		^cell group
			configure: cell title: aTitle
			description: aDescription detail: aDetail
	].
	^super configure: cell title: aTitle description: aDescription detail: aDetail! !

! MobilePage methodsFor: #drawing !
configure: cell title: aTitle detail: aDetail
	" Private - Default configuration for cells in groups. "

	^self configure: cell title: aTitle description: nil detail: aDetail! !

! MobilePage methodsFor: #drawing !
configure: cell title: aTitle description: aDescription
	" Private - Default configuration for cells in groups. "

	^self configure: cell title: aTitle description: aDescription detail: nil! !

! MobilePage methodsFor: #drawing !
configure: cell title: aTitle
	" Private - Default configuration for cells in groups. "

	^self configure: cell title: aTitle description: ''! !

! MobilePage methodsFor: #drawing !
iconSourceFor: iconName group: aGroup
	" Private - Return the icon for item in a group. "

	^self"aGroup" application sourceForImage: iconName! !

Object
	subclass: #MobileCoordinatorTemplate
	instanceVariableNames:
		 ' coordinatorClass title '
		,' widgetTemplates navigationTemplate '
		,' name listeners configuration '
	category: #Mobile !
MobileCoordinatorTemplate comment: '
	A template that contains information to create coordinators.'!
MobileCoordinatorTemplate buildAccessors.!

! MobileCoordinatorTemplate class methodsFor: #instantiation !
fromJSon: aJSon
	" Return an instance of the receiver set with aJSon. "

	^self new
		fromJSon: aJSon;
		yourself! !

! MobileCoordinatorTemplate methodsFor: #spec !
fromJSon: jsObject
	" Private - Set the receiver from jsObject. "

	self basicAt: #spec put: jsObject.
	configuration := JS @ jsObject.
	self coordinatorClass: ((configuration isNil
		or: [ (configuration @ #class) isNil ])
		ifTrue: [ #MobileCoordinator ]
		ifFalse: [ configuration @ #class ]).
	configuration := JS @ (configuration @ #configuration).
	configuration isNil ifFalse: [
		self title: configuration @ #title.
		self name: configuration @ #name.
		self widgetTemplates: (JS @ (configuration @ #widgets)).
		self navigationTemplate: configuration @ #navigationBar.
		self listeners: JS @ (configuration @ #listeners).
	]! !

! MobileCoordinatorTemplate methodsFor: #private !
actionToHandle: eventName in: subject doing: aBlock with: data
	" Private - Return the block to handle the event. "

	^aBlock! !

! MobileCoordinatorTemplate methodsFor: #instantiation !
newCoordinator
	" Return a new coordinator from the receiver. "

	| answer support |
	support := self coordinatorClass.
	support isNil ifTrue: [
		^self error: 'Missing coordinatorClass'
	].
	support := Smalltalk at: support ifAbsent: [
		^self error: 'Missing coordinator #',support
	].
	answer := support new.
	answer template: self.
	self listeners notNil ifTrue: [
		self listeners properties do: [:tuple |
			answer when: tuple first do:
				(self	actionToHandle: tuple first in: answer
					doing: tuple last with: configuration @ #title)
		]
	].
	configuration keys do: [:key |
		( #( #title #name #widgets #navigationBar #listeners ) includes: key)
		ifFalse: [ answer perform: key ,':' with: configuration @ key ]
	].
	^answer! !

! MobileCoordinatorTemplate methodsFor: #private !
spec: jsObject
	" Update the receiver from jsObject. "

	self fromJSon: jsObject.! !

Object
	subclass: #MobileThread
	instanceVariableNames: ' application coordinator '
	category: #Mobile !
MobileThread comment: '
	A process that holds a coordinator with visual focus. 
	Has an image and name attached via the thread selection bar.
'!
MobileThread buildAccessors: #(
	application
)!

! MobileThread methodsFor: #accessing !
coordinator
	" Return the coordinator of the receiver. "

	^coordinator! !

! MobileThread methodsFor: #accessing !
coordinator: anObject
	" Set the coordinator of the receiver. "

	coordinator := anObject.
	coordinator thread: self! !

! MobileThread methodsFor: #accessing !
back

	| back |
	self coordinator triggerClosed.
	back := self coordinator back.
	back notNil ifTrue: [ self coordinator: back ].
	self activate! !

! MobileThread methodsFor: #accessing !
next: aCoordinator

	aCoordinator back: self coordinator.
	self coordinator: aCoordinator.
	self coordinator thread: self.
	self activate! !

! MobileThread methodsFor: #accessing !
update

	self activate! !

! MobileThread methodsFor: #ui !
actionsBarDo: aBlock
	" Private - Evaluate aBlock with current actionsBar (if present). "

	^self application device thread: self actionsBarDo: aBlock! !

! MobileThread methodsFor: #instantiation !
newCoordinatorFrom: spec

	(self isObject: spec) ifFalse: [
		^(MobileCoordinatorTemplate fromJSon: spec) newCoordinator
	].
	spec isString ifTrue: [
		^(self application templateAt: spec) newCoordinator
	].
	^spec new! !

MobileThread subclass: #MobileThreadModal !

! MobileThread methodsFor: #gui !
activate
	" Activate the receiver (render UI). "

	self application clearScreen.
	self coordinator buildContentsIn: self application.
	self application current: self.
	self -- (Array 
		with: #threadActivated with: self 
		with: self coordinator class name ,' ('
			,self coordinator title ,')'
		)! !

! MobileThreadModal methodsFor: #spec !
fromJSon: jsObject
	" Private - Set the receiver from jsObject. "

	self basicAt: #spec put: jsObject.
	self coordinator: (self newCoordinatorFrom: jsObject).! !

! MobileThreadModal methodsFor: #gui !
activate
	" Activate the receiver (render UI). "

	super activate.
	self actionsBarDo: [:bar| bar isOpen ifTrue: [ bar close ] ].! !

MobileThread
	subclass: #MobileThreadAction
	instanceVariableNames: 'tool task'!
MobileThreadAction buildAccessors!

! MobileThreadAction class methodsFor: #instantiation !
fromJSon: aJSon
	" Return an instance of the receiver set with aJSon. "

	^self new
		fromJSon: aJSon;
		yourself! !

! MobileThreadAction methodsFor: #accessing !
activate
	" Activate the receiver (render UI). "

	super activate.
	self actionsBarDo: [:bar| bar isOpen ifFalse: [ bar open ] ].! !

! MobileThreadAction methodsFor: #spec !
fromJSon: jsObject
	" Private - Set the receiver from jsObject. "

	| js |
	self basicAt: #spec put: jsObject.
	js := JS @ jsObject.
	self
		tool: js @ #tool;
		task: js @ #task;
		coordinator: (self newCoordinatorFrom: js @ #coordinator);
		yourself.! !

! MobileThreadAction methodsFor: #private !
refreshCoordinator: aMobilePageJSon
	" Refresh a aMobilePageJSon "

	(JS @ (self basicAt: #spec))
		handleAt: #coordinator
		put: aMobilePageJSon spec! !

! MobileThreadAction methodsFor: #spec !
spec: jsObject
	" Update the receiver from jsObject. "

	| js threadKey |
	self basicAt: #spec put: jsObject.
	threadKey := self application keyForThread: self.
	js := JS @ jsObject.
	self
		tool: js @ #tool;
		task: js @ #task;
		coordinator: (self coordinator spec: js @ #coordinator).
	threadKey isNil ifFalse: [
		self application 
			refreshThread: self 
			key: threadKey.
		self refresh
	]! !

! MobileThreadAction methodsFor: #private !
refresh
	" Refresh receivers UI "

	self update! !

MobilePage
	subclass: #MobilePageJSon
	instanceVariableNames:
		 ' widgets model buttons background navigationItem'
		,' top height lineHeight'!
MobilePageJSon buildAccessors: #(
	#(widgets initialize #PoolDictionary)
	#(buttons initialize #PoolDictionary)
	background height top model navigationItem
)!
MobilePageJSon subclass: #MobileEditor !
MobilePageJSon subclass: #MobileBrowser !

! MobilePageJSon class methodsFor: #navigation !
constructEventsTriggered
	" Private - Return the events managed by instances of the receiver. "

	^#(	#needsTitle
	) , super constructEventsTriggered! !

! MobilePageJSon methodsFor: #accessing !
widgetsMap
	" Returns the widgets (anArray of associations) of the reciver sorted by order. "

	^self widgets associations sorted: [:a :b | a value order < b value order ]! !

! MobilePageJSon methodsFor: #accessing !
suggestedHeight
	" Return the suggested height to render the receiver. "

	^self height ! !

! MobilePageJSon methodsFor: #accessing !
suggestedTop
	" Return the suggested top to render the receiver. "

	^self top ! !

! MobilePageJSon methodsFor: #accessing !
lines: aNumber
	" Set the lines of the receiver. "

	self height: aNumber * self lineHeight + (self lineHeight / 2)! !

! MobilePageJSon methodsFor: #accessing !
lineHeight
	" Return the height of lines of the receiver. "

	lineHeight isNil ifTrue: [ ^MobileDevice current lineHeight ].
	^lineHeight! !

! MobilePageJSon methodsFor: #accessing !
lineHeight: aNumber
	" Set the height of lines of the receiver. "

	lineHeight := aNumber! !

! MobilePageJSon methodsFor: #accessing !
title: aString
	" Set the title of the receiver. "

	self when: #needsTitle do: [ aString ]! !

! MobilePageJSon methodsFor: #navigation !
buildLeft: aNavigationBar in: aMobileApplication
	" Private - Build the left side navigation bar of the receiver. "

	| conf |
	self buttons at: #back put: (aNavigationBar
		left: self leftTitle
		action: (self leftActionIn: aMobileApplication)).

	conf := JS @ self navigationTemplate.
	conf isNil ifTrue: [^nil].

	conf := JS @ (conf @ #left).
	conf isNil ifTrue: [^nil].

	conf keys do: [:key |
		self buttons at: key put: (aNavigationBar
			left: key
			action: [ conf @ key value: self ])
	].! !

! MobilePageJSon methodsFor: #navigation !
buildRight: aNavigationBar in: aMobileApplication
	" Private - Build the left side navigation bar of the receiver. "

	| conf |
	aNavigationBar
		right: self rightTitle
		action: (self rightActionIn: aMobileApplication).

	conf := JS @ self navigationTemplate.
	conf isNil ifTrue: [^nil].

	conf := JS @ (conf @ #right).
	conf isNil ifTrue: [^nil].

	conf keys do: [:key |
		self buttons at: key put: (aNavigationBar
			right: key
			action: [ conf @ key value: self ])
	].! !

! MobilePageJSon methodsFor: #gui !
buildScreenContents: aWidget in: aMobileApplication
	" Private - Build the screen contents of the receiver. "

	| widgetTemplate current spec |
	super buildScreenContents: aWidget in: aMobileApplication.
	aWidget isNil ifTrue: [ ^nil ].
	self widgetTemplates isNil ifTrue: [ ^nil ].
	self background notNil ifTrue: [
		aWidget background: self background
	].
	self widgetTemplates keys do: [:key | | json |
		json := self widgetTemplates @ key.
		current := self device
			newWidgetFromJSon: json
			for: self.
		self -- (Array with: #create with: current with: self).
		spec := JS @ json @ #configuration.
		spec isNil ifFalse: [ current fromJSon: spec ].
		self widgets at: key put: current.
	].
	(self widgets values asArray
		sorted: [:before :after | before order < after order])
		do: [:pose | aWidget addChild: pose ].! !
		
! MobilePageJSon methodsFor: #actions !
click: aButtonName

	^(self button: aButtonName) triggerClick ! !

! MobilePageJSon methodsFor: #accessing !
button: selector

	^self buttons at: selector ifAbsent: [
		self error: 'Missing button #',selector
	]! !

! MobilePageJSon methodsFor: #accessing !
buttonBack

	^self button: #back ! !

! MobilePageJSon methodsFor: #accessing !
device
	" Return the title of the receiver. "

	^self thread application device! !

! MobilePageJSon methodsFor: #navigation !
newTemplateAt: aString

	^(self thread application templates at: aString) newCoordinator! !

! MobilePageJSon methodsFor: #GUI !
refreshTitle

	| target |
	target := self navigationItem "self thread application screen navigationBar".
self -- ('#refreshTitle target=' ,target).
	target notNil ifTrue: [ target title: self title ]! !

! MobilePageJSon methodsFor: #accessing !
title
	" Return the title of the receiver. "

	| answer |
	answer := self template title.
	answer isNil ifTrue: [
		answer := self
			triggerEvent: #needsTitle
			withArguments: (Array with: self)
			ifNotHandled: [ '' ].
	].
	^answer! !

! MobilePageJSon methodsFor: #accessing !
widgetAt: aString

	^self widgets at: aString! !

! MobilePageJSon methodsFor: #accessing !
@ aString

	^self widgets
		at: aString
		ifAbsent: [ self error: '// Missing widget #',aString
			,' ',self widgets keys asArray printString
		]! !

! MobilePageJSon methodsFor: #spec !
spec: jsObject
	" Update the receiver from jsObject. "

	self template spec: jsObject.
	self refresh.! !

! MobilePageJSon methodsFor: #private !
refresh
	" Refresh receivers UI "

	self thread refreshCoordinator: self.
	self application clearScreen. 
	"self buildContentsIn: self application."
	self device refreshCoordinator: self.! !

! MobilePageJSon methodsFor: #private !
spec
	" Answer the spec of the receiver. "

	^self template basicAt: #spec! !

! MobilePageJSon methodsFor: #private !
refreshWidget: aWI8Widget key: key
	" Refresh aWI8Widget "

	(JS @ (self widgetTemplates @ key))
		handleAt: #configuration 
		put: (aWI8Widget basicAt: #spec)! !

! MobilePageJSon methodsFor: #private !
keyForWidget: aWI8Widget
	" Answer the key of aWI8Widget in the receiver "
	
	^self widgets 
		keyForValue: aWI8Widget 
		ifAbsent: [ nil ]! !

Object
	subclass: #JSonConfigurator
	instanceVariableNames: ''
	category: #Mobile !
JSonConfigurator class instanceVariableNames: ' default ' !

! JSonConfigurator class methodsFor: #configuration !
configure: target fromJSon: aJSon
	" Configure the target from aJSon "

	self default configure: target fromJSon: aJSon! !

! JSonConfigurator class methodsFor: #accessing !
default
	" Return the default instance of the receiver. "

	default isNil ifTrue: [ default := self new ].
	^default! !

! JSonConfigurator methodsFor: #accessing !
configure: target fromJSon: aJSon
	" Configure the target from aJSon. "

	| configuration receiver styleSelectors |
	styleSelectors := ((target respondsTo: #styleSelectors)
		and: [ target respondsTo: #style ])
		ifTrue: [ target styleSelectors ]
		ifFalse: [ #() ].
	configuration := JS @ aJSon.
	configuration keys do: [:key |
		(styleSelectors includes: key)
			ifTrue: [ receiver := target style ]
			ifFalse: [ receiver := target ].
		receiver perform: key ,$: with: configuration @ key
 	]! !