
MobileDevice
	subclass: #LoveDevice
	category: #LoveDevice !

! LoveDevice class methodsFor: #defaults !
isInstalled
	" Private - Return true if the receiver implements installed support. "

	^true! !

! LoveDevice methodsFor: #accessing !
id
	" Private - Return the device ID of the receiver. "

	^#Love-DEBUG! !

! LoveDevice methodsFor: #gui !
screenFor: aMobileApp
	" Private - Return a main screen to contain the UI elements for aMobileApp. "

	^LoveScreen for: aMobileApp! !

! LoveDevice methodsFor: #gui !
makeupGroupList: aList
	" Private - Makeup the list of a group. "

	! !

! LoveDevice methodsFor: #barcode !
userScanCodeFor: aMobileApp doing: aTwoArgumentsBlock onClose: onClose onError: errorBlock
	" Let the user enter scan barcode from keyboard. "
	" Pending implementation "

	self mainViewSupport 
		scanCodeDoing: aTwoArgumentsBlock 
		onClose: onClose 
		onError: errorBlock! !

! LoveDevice methodsFor: #camera !
openCameraDoing: action onError: errorBlock with: options
	" Evaluate action with snapshot captured from camera. "

	self mainViewSupport 
		openCameraDoing: action 
		onError: errorBlock 
		with: options! !

! LoveDevice methodsFor: #gui !
confirm: aMobileApp text: question action: actionAssoc cancel: cancelAction close: closeAction

	| builder editTxt |
	editTxt := TextView fromMainContext.
	builder := AlertDialog builder fromMainContext.
	builder
		message: question; title: '';
		positive: actionAssoc key buttonDo: [:iDialog|
			iDialog dismiss.
			actionAssoc value evaluate.
			closeAction evaluate
		];
		negative: #cancel buttonDo: [:iDialog|
			iDialog cancel.
			closeAction evaluate.
		];
		view: editTxt.
	^builder create
		cancelable: false;
		show! !

! LoveDevice methodsFor: #gui !
newWidgetFromJSon: aJSon for: aMobileCoordinator
	
	| support |
	support := self mainViewSupport current.
	aMobileCoordinator widgets isEmpty ifTrue: [
		aMobileCoordinator when: #built: do: [:aMobileCoordinator |
			support configureView: aMobileCoordinator
		]
	].
	^super newWidgetFromJSon: aJSon for: aMobileCoordinator. ! !

! LoveDevice methodsFor: #list !
configure: cell title: aTitle description: aDescription detail: aDetail
	" Private - Default configuration for cells in groups. "

	| rowText text rest |
	aDescription isNil ifTrue: [ rest := aDetail ] 
	ifFalse: [ rest := aDescription ].
	rowText := aTitle, String tab, rest.
	text:= TextWidget text: rowText.
	cell addChild: text.
	^cell ! !

! LoveDevice class methodsFor: #defaults !
mainViewSupport
	" Private - Return the support for mainView of the receiver(or nil). "

	^LoveApplication! !

! LoveDevice methodsFor: #defaults !
mainViewSupport
	" Private - Return the support for mainView of the receiver(or nil). "

	^self class mainViewSupport! !

! LoveDevice methodsFor: #ui !
hideKeyboard
	" hide keyboard "

	self mainViewSupport hideKeyboard! !

! LoveDevice methodsFor: #defaults !
customizeMainView: aMainView application: anApplication
	" Private - Customize the mainView. "
	" I.N.: initialization of menu items could be done here -
	I'd prefer so, because at this point we have the thread initialized
	and also menu items initialization is more encapsulated in LoveApplication,
	but mobile frw does it in 
	MobilePage>>buildActionsBar: anActionsBar in: aMobileApplication 
	exposing the UI elements, this could be reviewed.
	By now added a NavigationMenu>>contents: anObject "

	"anApplication threads 
		do: [:thread | aMainView addNavigationMenuItem: thread tool ]"
	
	anApplication threads 
		do: [:thread | aMainView addThread: thread ]! !

! LoveDevice methodsFor: #accessing !
mainViewFor: anApplication
	" Return a mainView (GUI container) for the receiver(or nil). "

	| support result |
	support := self mainViewSupport.
	support isNil ifTrue: [ ^nil ].
	result := support current. " a new instance of LoveApplication 
	to support anApplication may be needed. "
	self customizeMainView: result application: anApplication.
	^result! !

! LoveDevice methodsFor: #gui !
scaleMainViewFor: anApplication
	" Private - Scale the main view of anApplication. "
	" Implementation Note: temporary implementation "

	! !

! LoveDevice methodsFor: #gui !
button: title for: imageProvider doing: aBlock
	" Private - Return a button with title evaluating anAction when clicked. "
		
	| result tool |
	" implementation note: by now imageProvider is nil, 
	until MenuBuilder is replaced by MobileNavigationBar "
	" temporary implementation: "
	" tool := self sourceForImage: title. "
	tool := imageProvider imageNamed: title. 
	tool notNil ifTrue: [
		^ImageButtonMobile 
			withAction: aBlock 
			title: title 
			tool: tool
	] ifFalse: [ ^title ]! !

! LoveDevice methodsFor: #defaults !
iconNames
	" Private - Return the names of icons in resources storage. "

	^super iconNames ,#(
		add back next right gps barcode
		trashcan delete exit finish
		#Cancel #Delete #Commit 'Start Task'
		fullBlack fullRed fullWhite
		nextItem prevItem
		refresh reject snapshot start save

		tasks tasksSelected
		transfer transferSelected
		help helpSelected
		settings settingsSelected
	)! !

! LoveDevice methodsFor: #defaults !
sourceForImage: imageName
	"  Return the URL for source of resource image at imageName. "

	^R drawable @ imageName asLowercase! !

! LoveDevice methodsFor: #initialize !
initializeTypeWidgetMapping
	" Private - Initialize the typeWidgetMapping of the receiver. "

	typeWidgetMapping := Dictionary new.
	typeWidgetMapping
		at: #label	put: #MobileText;
		at: #textField	put: #MobileEntryField;
		at: #button	put: #MobileButtonWidget;
		at: #list	put: #MobileSelector;
		yourself.! !

! LoveDevice methodsFor: #gui !
navigationBarFor: anApplication
	" Private - Return the/a navigationBar installed on the receiver. "

	^MobileNavigationBar new
		" application: anApplication;
		when: #makeup: do: [:widget| self makeupNavigationBar: widget ];
		yourself "! !

! LoveDevice methodsFor: #gui !
toolbarWith: left and: right for: imageProvider
	" Return a simple toolbar with two elements. "
	
	self print: '(#toolbarWith:and:for:) left=', left ,' right=', right ,' imageProvider=', imageProvider.

	"| toolbar onLeft onRight |
	onLeft := self button: left key for: imageProvider doing: left value.
	onRight := self button: right key for: imageProvider doing: right value.
	onLeft style
		background: #darkgray; color: #white;
		position: #absolute; left: #2pt;
		yourself.
	onRight style
		background: #green; color: #white;
		position: #absolute; right: #2pt;
		yourself.
	toolbar := HTMLWidget with: onLeft with: onRight.
	toolbar style
		left: 0; right: 0; height: #26pt;
		background: 'linear-gradient(gray,black)';		
		yourself.
	^toolbar"! !

! LoveDevice methodsFor: #gui !
dialog: aMobileApp
	text: explanation
	custom: customBlock
	doing: anAction
	closeOnClick: closeOnClick
	" Open a modal dialog customized by customBlock. "

	"| cover pane close text |"

	self print: 'dialog:', aMobileApp.
	self print: 'text:', explanation.
	self print: 'custom:', customBlock.
	self print: 'doing:', anAction.
	self print: 'closeOnClick:', closeOnClick.

	"close := [ anAction evaluate. pane close. cover close. ].
	cover := HTMLWidget new.
	cover style
		background: #white; opacity: 0.3;
		position: #absolute; top: 0; bottom: 0; left: 0; right: 0;
		yourself.
	pane := HTMLWidget new.
	pane style
		background: #white; border: '1px solid #black'; color: #black;
		position: #absolute; left: 0; right: 0;
		bottom: 0;
		opacity: 1;
		yourself.
	explanation isString ifTrue: [
		text := HTMLWidget with: explanation.
		text style
			marginTop: #5%;
			marginBottom: #5%;
			marginRight: #5%;
			marginLeft: #5%;
			yourself.
		pane addChild: text.
	]ifFalse: [
		explanation notNil ifTrue: [ pane addChild: explanation ]
	].
	customBlock value: pane.
	closeOnClick ifTrue: [
		cover when: #click: do: close.
		pane  when: #click: do: close.
		text notNil ifTrue: [ text when: #click: do: close. ].
	].
	aMobileApp screen
		addChild: cover;
		addChild: pane;
		yourself.
	^close"! !

EventManager
	subclass: #LoveScreen
	instanceVariableNames: ' application navigationBar actionsBar contents statusBarHeight parent '
	category: #LoveDevice !
LoveScreen buildAccessors: #(
	#application
	#navigationBar
	#actionsBar
	#contents
	#statusBarHeight
	#parent
)!
LoveScreen comment: '
	Polimorphic -at certain point- with (web) MobileScreen'!

! LoveScreen class methodsFor: #instantiation !
for: aMobileApp
	" Return an instance of the receiver. "
	" I.N.: added #configure "

	^self new
		application: aMobileApp;
		configure;
		yourself! !

! LoveScreen methodsFor: #update !
clear
	" Clear the receiver. "

	! !

! LoveScreen methodsFor: #accessing !
device
	" Return the device where the application is running. "

	^self application device! !

! LoveScreen methodsFor: #defaults !
defaultNavigationBar
	" Return the default navigation bar for the receiver. "

	^self device navigationBarFor: self application! !

! LoveScreen methodsFor: #configuration !
configure
	" Private - Configure the receiver's internal structure. "

	"super configure."
	self navigationBar: (self triggerEvent: #needsNavigationBar ifNotHandled: [ self defaultNavigationBar ]).
	"self contents: (self triggerEvent: #needsContents ifNotHandled: [ self defaultContents ]).
	self actionsBar: (self triggerEvent: #needsActionsBar ifNotHandled: [ self defaultActionsBar ]).
	self triggerEvent: #configured"! !

! LoveScreen methodsFor: #configuration !
navigationBar
	" Answer the navigation bar of the receiver. "

	^navigationBar " self parent navigationBar "! !

! LoveScreen methodsFor: #accessing !
actionsBar
	" Return the actionsBar of the receiver. "

	^self parent actionsBar! !

! LoveScreen methodsFor: #accessing !
contents
	" Return the contents of the receiver. "

	"^self parent contents"
	| ui |
	ui := LoveWidget new
		parent: self parent;
		yourself.
	self print: 'LoveScreen>>contents'.
	self print: 'parent=', self parent.
	" self parent activeView: ui. "
	^ui! !

! LoveScreen class methodsFor: #events !
localEvents
	" Private - Return the smalltalk events managed by instances of the receiver. "

	^#(
	#getBackground
	#needsStatusBarHeight
	#needsNavigationBar #needsActionsBar
	#needsContents #getContents
	) ! !

self ignoreFileInUpTo: #EndOfAndroid!
! AndroidApplication methodsFor: #gui !
open
	" Open the receiver. "
	
	^self! !

! AndroidApplication methodsFor: 'AndroidDevice-add' !
addChild: aChild
	" Add a child to the receiver. Return the added element. "
	
	self screen: aChild.
	aChild parent: self.
	^aChild! !
	
! MenuBuilder methodsFor: 'LoveDevice-Imlt8_configuration' !
triggerClick

	"self triggerEvent: #click:"! !

! MenuBuilder methodsFor: 'LoveDevice-accessing' !
device
	" Return the device where the application is running. "

	^MobileDevice current! !

! NavigationMenu methodsFor: 'LoveDevice-accessing' !
contents: anObject
	" Set the contents of the receiver. "

	| items itemOrder |
	items := self menuItems.
	itemOrder := self handle #size.
	anObject do: [:item | | string |
		string := item isString ifTrue: [ item ] ifFalse: [ item title ].
		(items includes: string) ifFalse: [
		self handle #add: 0 itemId: itemOrder order: itemOrder titleRes: string.
		itemOrder := itemOrder + 1.
		]
	]! !

! NavigationMenu methodsFor: 'LoveDevice-query' !
isOpen
	" Return true if the receiver is open. "

	^true! !

! NavigationMenu methodsFor: 'LoveDevice-accessing' !
menuItems
	" Private - Return the the navigation menu items. "

	| itemsCount items |
	items := #().
	itemsCount := self handle #size. 
	1 to: itemsCount do: [:i | 
		items add: (self handle #findItem: i-1) #getTitle ].
	^items! !
----------------------------------------------------------
#EndOfAndroid!

! LoveDevice methodsFor: #multimedia !
openFileAtUrl: url
	" Download and open the file at url. "

	^self mainViewSupport openFileAtUrl: url! !

! LoveDevice methodsFor: #multimedia !
openFileAt: pathName
	" Open the (local) file at pathName. "

	self mainViewSupport openFileAt: pathName! !

! LoveDevice methodsFor: #multimedia !
openFileContents: fileContents type: filetype version: version
	" Save the contents in local storage and open as a document of specified type. "

	self mainViewSupport 
		openFileContents: fileContents 
		type: filetype 
		version: version! !

! LoveDevice methodsFor: #accessing !
peek: aMobileApplication title: aTitle
	selecting: initialSelection in: options
	doing: anAction onClose: closeBlock
	" Open an instance of the receiver prompting aValue from list of options. "

	^MobileWebListEditor
		peek: aMobileApplication title: aTitle
		selecting: initialSelection in: options
		doing: anAction onClose: closeBlock! !

! LoveDevice methodsFor: #liveUpdate !
refreshCoordinator: coordinator
	" Private - refresh coordinator. "

	coordinator buildContentsIn: coordinator application.! !
