
EventManager
	subclass: #LoveWidget
	instanceVariableNames: ' name parent opened handle children id view '
	category: #LoveWidget !
LoveWidget buildAccessors: #(
	#(view initialize 'self defaultView')
	#(id initialize 'self defaultId')
	#(children initialize 'Array new')
	#name #parent #opened #handle
)!

EventManager
	subclass: #LoveListConfiguration
	instanceVariableNames: ' initialElements stringSelector layout listSelectorLayout listViewLayout coordinator container owner clickAction imageNames '
	category: #LoveWidget !
LoveListConfiguration buildAccessors: #(
	#initialElements
	#stringSelector 
	#(layout initialize 'self defaultLayout')
	#listSelectorLayout 
	#listViewLayout 
	#coordinator 
	#container
	#owner
	#clickAction
	#(imageNames initialize 'self defaultImageNames')
)!

LoveListConfiguration
	subclass: #MultiViewListConfiguration
	instanceVariableNames: ' spec '
	category: #LoveWidget !
MultiViewListConfiguration buildAccessors: #(
	#spec
)!

LoveWidget
	subclass: #LoveListPart
	instanceVariableNames: ' '
	category: #LoveWidget !

LoveListPart
	subclass: #LoveListItem
	instanceVariableNames: ' owner object items '
	category: #LoveWidget !
LoveListItem buildAccessors: #(
	#owner #object #items
)!

LoveListPart
	subclass: #LoveList
	instanceVariableNames: ' configuration list elements index isDirty noLayoutListView '
	category: #LoveWidget !
LoveList buildAccessors: #(
	#configuration
	#(list initialize 'self defaultList')
	#(elements initialize 'self defaultElements')
	#index
	#(isDirty initialize 'self defaultIsDirty')
	#(noLayoutListView initialize 'self defaultNoLayoutListView')
)!

LoveList
	subclass: #LoveSelector
	instanceVariableNames: ' mobileSelector '
	category: #LoveWidget !
LoveSelector buildAccessors: #(
	#mobileSelector
)!

LoveSelector
	subclass: #LoveGrid
	instanceVariableNames: ''
	category: #LoveWidget !

LoveWidget
	subclass: #MobileGroup
	instanceVariableNames: ' selected '
	category: #LoveWidget !
MobileGroup buildAccessors: #(
	#selected
)!

LoveWidget
	subclass: #TextWidget
	instanceVariableNames: ' text readOnly '
	category: #LoveWidget !
TextWidget buildAccessors: #(
	#text 
)!

! TextWidget methodsFor: #value !
emptyValue
	" Return the empty (text) value of the receiver. "

	^self triggerEvent: #needsEmptyValue ifNotHandled: [$ ]! !

! TextWidget methodsFor: #ui !
textView
	" Answer the textView. "

	self parent class = MobileSelector ifTrue: [
	^self parent view findViewById: R id @ #title.
	] ifFalse: [ id isNil ifTrue: [ ^self view ].
	^self activeCoordinator findViewById: self id ]! !

! TextWidget methodsFor: 'LoveMobile-ui' !
configureView
	" Configure the ui of the receiver "

	self parent class = MobileSelector ifTrue: [
	self textView textSize: 21.
	self textView setTypeface: nil style: Typeface bold
	]! !

MobileGroup
	subclass: #MobileSelector
	instanceVariableNames: ' coordinator order  '
	category: #LoveWidget !
MobileSelector buildAccessors: #(
	#coordinator #order
)!

MobileGroup
	subclass: #MobileGrid
	instanceVariableNames: ' coordinator order '
	category: #LoveWidget !

LoveWidget
	subclass: #MobileGroupCell
	instanceVariableNames: ' group '
	category: #LoveWidget !
MobileGroupCell buildAccessors: #(
	#group
)!

! MobileGrid methodsFor: #configuration !
configureList
	" Private - Configure the contents of the receiver to arrange a list of items. "

	| items list |
	items := self 
		triggerEvent: #needsItems 
		withArguments: (Array with: self coordinator)
		ifNotHandled: [ nil ].
	items isNil ifTrue: [ ^nil ].
	list := self 
		triggerEvent: #needsList 
		ifNotHandled: [ self defaultListFor: items ].
	self
		triggerEvent: #configureList:for:
		withArguments: (Array with: list with: items)
		ifNotHandled: [ self defaultConfigureList: list for: items ].
	self addChild: list.
	^list! !
	
! MobileGrid methodsFor: #defaults !
defaultView
	" Answer a default view for the receiver "

	^LoveApplication current activeCoordinator
		defaultViewFor: self xmlLayout! !

! MobileGrid methodsFor: #ui !
setViewId
	" Set the view id of the receiver "
	
	! !

! MobileGrid methodsFor: 'LoveMobile-ui' !
configureView
	" Configure the ui of the receiver "
	
	| list textWidget |
	list := self list. 
	list isNil ifTrue: [ ^self ].
	list isNil ifFalse: [ list configure ]. 
	textWidget := self text.
	textWidget isString ifTrue: [ list setTitle: textWidget ] 
	ifFalse: [ textWidget configureView.
	list setTitle: textWidget text ]! !

! MobileGrid methodsFor: 'LoveMobile-ui' !
list
	" Answer the list of the receiver "
	
	^self 
		childKindOf: LoveGrid 
		ifNone: [ nil ]! !

! MobileGrid methodsFor: 'LoveMobile-ui' !
lines: anObject
	" This message is neither not sent nor implemented, 
	in IMLT8 web "
	
	! !

! MobileGrid methodsFor: 'LoveMobile-ui' !
text
	" Answer the text of the receiver "

	^self 
		childKindOf: TextWidget 
		ifNone: [ String space ]! !

! MobileGrid methodsFor: 'LoveMobile-ui' !
childKindOf: aClass ifNone: aBlock
	" Answer the child of kind aClass if none evaluate aBlock. "
	
	^self children 
		detect: [ :child | child isKindOf: aClass ]
		ifNone: aBlock! !

! MobileGrid methodsFor: #selecting !
select: item
	" Private - Process the selection of item in the receiver. "

	self selected: item.
	self triggerEvent: #selectionChanged.
	self triggerEvent: #selected: withArguments: (Array with: self selected with: self coordinator).
	LoveApplication current activeCoordinator open.! !

! MobileGrid methodsFor: #stage2 !
xmlLayout
	" Return the proper layout "

	^#grid! !

! MobileGrid methodsFor: #configuration !
configureTitle
	" Private - Configure the title of the receiver. "

	| title |
	title := self triggerEvent: #needsTitle withArguments: (Array with: self coordinator) ifNotHandled: [ nil ].
	title isNil ifTrue: [ ^nil ].
	" title isString ifTrue: [ title := HTMLWidget text: '<h2>',title,'</h2>' ]. "
	self addChild: title! !

! MobileGrid methodsFor: #accessing !
coordinator
	" Return the coordinator of the receiver. "

	^coordinator! !

! MobileGrid methodsFor: #accessing !
coordinator: anObject
	" Set the coordinator of the receiver. "

	coordinator := anObject! !

! MobileGrid methodsFor: #list !
defaultCellFor: item in: list
	" Private - Return the cell to render item in the list. "

	| cell |
	cell := MobileGroupCell new.
	cell	group: self.
		" href: '#'; when: #click: do: [:evt|
			self select: item in: cell event: DOMMouseEvent @ evt 
		]."
	cell := self
		triggerEvent: #configureCell:for:in:coordinator:
		withArguments: (Array with: cell with: item with: list with: self coordinator)
		ifNotHandled: [ self defaultConfigureCell: cell for: item in: list ].
	^cell! !
	
! MobileGrid methodsFor: #selecting !
	defaultConfigureCell: cell for: item in: list

	! !

! MobileGrid methodsFor: #selecting !
defaultSetIconFor: anLoveListItem in: aList
	" Set the icon to use as expander of the item.
	Note that click: event must be set if it is bound an action on the icon "

	| expander iconSource iconSize |
	iconSource := self
		triggerEvent: #needsIconSourceFor:in:coordinator:
		withArguments: (Array with: anLoveListItem with: aList with: self coordinator)
		ifNotHandled: [ nil ].
	expander := anLoveListItem expander.
	expander isNil ifTrue: [
		anLoveListItem expander: anLoveListItem defaultExpander.
		expander := anLoveListItem expander.
	].
	iconSource isNil ifTrue: [
		expander style width: #0px.
		^self
	].
	expander src: iconSource.
	iconSize := self device iconSize.
	expander style
		width: iconSize;
		height: iconSize;
		yourself.! !

! MobileGrid methodsFor: #fileIn !
listeners: jsObject

	(JS @ jsObject) properties do: [:listener |
		self when: listener first do: listener last
	]! !

! MobileGrid methodsFor: #gui !
openIn: aParent before: aWidget
	" Open the receiver before element inside aParent. "

	super openIn: aParent before: aWidget.
	self refresh. ! !

! MobileGrid methodsFor: #accessing !
order
	" Return the order of the receiver. "

	^order! !

! MobileGrid methodsFor: #accessing !
order: anObject
	" Set the order of the receiver. "

	order := anObject! !

! MobileGrid methodsFor: #accessing !
outline
	" Set the receiver to none display. "

	self display: #none! !

! MobileGrid methodsFor: #accessing !
container
	" Answer the container. "

	^self view findViewById: R id @ #mobileSelector ! !

! MobileGrid methodsFor: #accessing !
refresh

	| items |
	(self children notEmpty and: [self children last isKindOf: ListWidget])
		ifFalse: [ self configure ]
		ifTrue: [
			self children last reset.
			items := self
				triggerEvent: #needsItems 
				withArguments: (Array with: self coordinator)
				ifNotHandled: [ nil ].
			self children last when: #needsRoots do: [ items ].
			self children last configure.
			self defaultConfigureList: self children last for: items
	].
	super refresh! !

! LoveWidget class methodsFor: #events !
constructEventsTriggered
	" Private - Return the events managed by instances of the receiver. "

	^self localEvents
	,super constructEventsTriggered! !

! LoveWidget class methodsFor: #inspect !
addWidgetNamed: widgetName withSpec: aComposedAssociations coordinator: coordinator
	
	coordinator addWidgetNamed: widgetName withSpec: aComposedAssociations! !

! LoveWidget class methodsFor: #inspect !
aspects
	" Return the aspects of the receiver. "

	^#()! !

! LoveWidget class methodsFor: #inspect !
prototypeSpec
	" Return a prototype spec of the receiver. "

	^self subclassResponsibility: #prototypeSpec! !

! LoveWidget class methodsFor: #inspect !
inspectionInstruments
	" Anwser inspection instruments for the receiver. "

	^#()! !

! LoveWidget class methodsFor: #inspect !
support
	" Return the support for the receiver. "

	^LoveDevice mainViewSupport! !

! LoveWidget class methodsFor: 'inspect-accessing' !
inspectableWidgets
	" Answer inspectable widget classes. "
	
	^LoveWidget allSubclasses select: 
		[:cls | cls needsInspection 
	]! !

! LoveWidget methodsFor: #defaults !
mainViewSupport
	" Private - Return the support for mainView of the receiver(or nil). "

	^self device mainViewSupport! !

! LoveWidget methodsFor: #accessing !
application
	" Return the application of the receiver. "

	^self support application.! !

! LoveWidget methodsFor: #accessing !
support
	" Return the support for the receiver. "

	^self device mainViewSupport! !

! LoveWidget methodsFor: #configuration !
configure
	" Private - Set the GUI element(handle) to host the receiver. "

	self basicConfigure.
	self makeup! !

! LoveWidget methodsFor: #accessing !
defaultId
	" Answer the default id for the receiver "
	" I.N.: generateViewId should be used, but it did not 
	worked "
	
	| now |
	now := DateTime now asNumber asString.
	^(now copyFrom: 5 to: now size) asNumber.! !

! LoveWidget methodsFor: #style !
background: background
	" Set the style of receiver. "

	"self style background: background"! !

! LoveWidget methodsFor: #accessing !
device
	" Return the device where the application is running. "

	^MobileDevice current! !

! LoveWidget methodsFor: 'LoveMobile-ui' !
configureView
	" Configure the ui of the receiver "

	! !

! LoveWidget methodsFor: 'LoveMobile-ui' !
setProperties
	" Set the properties of the receiver "

	! !

! LoveWidget methodsFor: #ui !
openIn: anLoveCoordinator
	" Open the receiver in aViewGroup. "
	" I.N.: give the receiver a chance to do anything before 
	opening "
	
	self setViewId.
	anLoveCoordinator showChild: self! !

! LoveWidget methodsFor: #ui !
setViewId
	" Set the view id of the receiver "! !

! LoveWidget methodsFor: #initialize !
initializeHandle
	" Private - Initialize the handle of the receiver.
	Bind the receiver to element inside parent's contents. "

	self handle: self newHandle.
	"self configure." ! !

! LoveWidget methodsFor: #private !
handle
	" Return the native handle. "

	handle isNil ifTrue: [ self initializeHandle ].
	^handle! !

! LoveWidget methodsFor: #private !
configureListenerForView: aView
	" Configure listener for aView "

	aView onDragListener: self dragListenerForView.
	! !

! LoveWidget methodsFor: #private !
dragListenerForView
	" Answer a drag listener for view "

	| listener actionDrop actionDragStarted |
	actionDrop := DragEvent actionDrop.  
	actionDragStarted := DragEvent actionDragStarted.
	listener := Java
		newProxy: 'Love.view.View$OnDragListener'
		functions: #onDrag -> [:v :event | 
			(event action = actionDrop) 
				ifTrue: [ [ self inspectWidget ] valueDeferred. 
						true ] 
				ifFalse: [ event action = actionDragStarted 
						ifTrue: [ true ] ifFalse: [ false ] 
				]	
		].
	^listener! !

! LoveWidget methodsFor: #private !
newHandle
	" Private - Return a handle obtained from new GUI element created to host receiver's contents.
	Actually we try to find an element matching the receiver's id and in that case, use that handle.
	In case of there is no element to host the receiver, we create an element in the parent's handle.
	"

	^self parent handle! !

! LoveWidget methodsFor: #inspect !
inspectWidget
	" Inspect the receiver. "

	self preInspectionActions.
	self inspect! !

! LoveWidget methodsFor: #inspect !
preInspectionActions
	" Pre inspection actions. "

	self support context preInspectionActions.! !

! LoveWidget methodsFor: #inspect !
needsInspection
	" Anwser if the receiver needs inspection. "

	^false! !

! LoveWidget methodsFor: #accessing !
handle: aHandle
	" Set the handle of the receiver. "

	handle := aHandle! !

! LoveWidget methodsFor: #delegation !
height
	" Delegate the resolution of height. "

	! !

! LoveWidget methodsFor: #delegation !
height: anObject
	" Delegate the resolution of height:. "

	! !

! LoveWidget methodsFor: 'WebWidget-gui' !
hide
	" Hide the receiver "

	self view visibility: View invisible! !

! LoveWidget methodsFor: #configuration !
makeup
	" Private - Set the style of the receiver. "

	self defaultMakeup.
	self triggerEvent: #makeup.
	self triggerEvent: #makeup: withArguments: (Array with: self).! !

! LoveWidget methodsFor: #private !
basicConfigure
	" Private - Configure the receiver's internal structure with default settings."

	self children: #().! !

! LoveWidget class methodsFor: #events !
localEvents
	" Private - Return the smalltalk events managed by instances of the receiver. "

	^#()! !

! LoveWidget methodsFor: #configuration !
defaultMakeup
	" Private - Set the style of the receiver. "

	! !

! LoveWidget methodsFor: #configuration !
fromJSon: aJSon
	" Configure the receiver from a literal. "

	JSonConfigurator
		configure: self
		fromJSon: aJSon.
	self configure! !

! LoveWidget methodsFor: #add !
addChild: aChild
	" Add a child to the receiver. Return the added element. "
	" I.N.: by now, not including HTMLWidget>>addChild: aChild where
	aChild open is sent, to render aChild in the receiver "

	aChild isNil ifTrue: [ ^nil ].
	
	aChild isString ifTrue: [
		^self addChild: (TextWidget text: aChild)
	].

	(aChild isKindOf: Collection) ifTrue: [
		^self addChildren: aChild
	].

	"self configure."
	"self handle." "ensure handle initialized"
	^self children add: (aChild asChildOf: self)! !

! LoveWidget methodsFor: #add !
addChildren: aCollection
	" Add a collection (of widgets) as child of the receiver.
	Return the receiver.
	"

	aCollection do: [:each| self addChild: each ].! !

! LoveWidget methodsFor: #add !
asChildOf: aParent
	" Private - Return the receiver as a child of aParent. "

	parent := aParent! !

! LoveWidget methodsFor: #gui !
show
	" Show the receiver "

	self view visibility: View visible! !

! LoveWidget methodsFor: #accessing !
refresh
	" Refresh the receiver "

	self configureView! !

! MobileGroup class methodsFor: #events !
localEvents
	" Private - Return the smalltalk events managed by instances of the receiver. "

	^#(
	#needsTitle
	#needsContainer
	#needsItems
	#needsList #configureList:for:
	#setIconFor:in: #needsIconSourceFor:in:
	#needsCellFor:in: #configureCell:for:in:
	#selectionChanged #selected:
	)! !

! TextWidget class methodsFor: #configuration !
text: aString
	" Answer a new instance of the receiver with aString as its contents. "

	^self new
		text: aString;
		yourself! !

! MobileGroup methodsFor: #configuration !
configure
	" Private - Configure the receiver's internal structure. "

	super configure.
	self configureTitle;
		configureContents;
		yourself! !

! MobileGroup methodsFor: #configuration !
configureContents
	" Private - Configure the contents of the receiver. "

	| custom |
	custom := self
		triggerEvent: #needsContainer
		ifNotHandled: [ nil ].
	custom isNil ifTrue: [ ^self configureList ].
	^self addChild: custom! !

! MobileGroup methodsFor: #list !
configure: cell title: aTitle description: aDescription detail: aDetail
	" Private - Default configuration for cells in groups. "

	^self device
		configure: cell title: aTitle
		description: aDescription detail: aDetail ! !

! MobileGroup methodsFor: #configuration !
configureList
	" Private - Configure the contents of the receiver to arrange a list of items. "

	| items list |
	items := self triggerEvent: #needsItems ifNotHandled: [ nil ].
	items isNil ifTrue: [ ^nil ].
	list := self triggerEvent: #needsList ifNotHandled: [ self defaultListFor: items ].
	self	triggerEvent: #configureList:for:
		withArguments: (Array with: list with: items)
		ifNotHandled: [ self defaultConfigureList: list for: items ].
	self addChild: list.
	^list! !

! MobileGroup methodsFor: #list !
defaultListFor: items
	" Private - Return a widget to render items in the receiver. "

	| tree |
	tree := LoveSelector @ self.
	tree	
		flat;
		when: #makeup do: [ self device makeupGroupList: tree ];
		when: #needsRoots do: [ items ];
		when: #needsTitleOf: do: [:item|
			self triggerEvent: #needsCellFor:in:
				withArguments: (Array with: item with: tree)
				ifNotHandled: [ self defaultCellFor: item in: tree ].
		]. 
	^tree! !

! MobileGroup methodsFor: #list !
defaultConfigureList: list for: items
	" Private - Configure the list. "
	" in self defaultSetIconFor: item in: list, event triggered attended by spec listener "

	! !

! LoveSelector class methodsFor: 'LoveMobile-instantiation' !
@ aMobileSelector
	" Return a new instance of the receiver wrapping aMobileSelector. "
	" create an instance of LoveListConfiguration and @ message to build 
	the instance of LoveList "

	"LoveList := self @ self defaultConfiguration."
	^self new 
		mobileSelector: aMobileSelector;
		yourself.! !

! LoveSelector class methodsFor: #private !
defaultConfiguration
	" Answer a list configuration for the receiver "

	^self notImplementedYet: #defaultConfiguration! !

! LoveSelector methodsFor: #accessing !
container
	" Answer the list container of the receiver "

	^self mobileSelector container.! !


! LoveSelector methodsFor: 'LoveMobile-configuration' !
configure
	" Private - Configure the receiver's internal structure. "

	| listItems |
	super configure.
	listItems := (self children 
		collect: [:item | item configure]).
			"collect: [:item | 
					| cell row |
					cell := item children first.
					row := cell children first.
					row text 
			]."
	self setConfiguration.
	self setContents: listItems.! !

! LoveSelector methodsFor: #accessing !
onItemClickAction
	" Private - an item in the receiver was clicked, do an action "

	self mobileSelector select: self selectedElement object! !

! LoveSelector methodsFor: #private !
defaultConfiguration
	" Answer a list configuration for the receiver "
	" I.N.: to be changed: get coordinator by means of handle or parent,
	encapsulate "
 
	| conf |
	conf := self class defaultConfiguration.
	conf container: self container.
	conf owner: self.
	^conf! !

! LoveSelector methodsFor: #accessing !
title
	" Answer the title of the receiver "
	
	" ^self coordinator viewNamed: #title "
	^self container findViewById: R id @ #title! !

! LoveSelector methodsFor: #ui !
setTitle: aString
	" Set the title of the reciever. "

	^self title text: aString! !

! LoveSelector methodsFor: #instantiation !
setConfiguration
	" Configure the receiver. "
	
	self 
		configuration: self defaultConfiguration;
		configureListView! !

! LoveGrid class methodsFor: #private !
defaultConfiguration
	" Answer a list configuration for the receiver "

	^self notImplementedYet: #defaultConfiguration! !

! LoveGrid class methodsFor: #private !
configurationSpec
	" Answer the configuration spec of the receiver "
 
	^(#itemName -> [ #temp ])
	,(#icon -> [ R drawable @ #browse ])! !

! MobileSelector class methodsFor: #inspect !
updateSpecBaseOn: dictionary prototypeSpec: aComposedAssociations coordinator: coordinator
	" Private - answer a new -composedAssociations- spec based on dictionary 
	and prototype spec"
	
	| result widgetName |
	result := aComposedAssociations copy.
	dictionary keysAndValuesDo: [:key :value| | newValue |
		newValue := value text toString.
		(self listenerAspects includes: key) ifTrue: [
			newValue := newValue doIt.
		].
		key = #order ifTrue: [ newValue := newValue asNumber ].
		key = #name ifTrue: [ widgetName := newValue ] ifFalse: [
		(result 
			associationAt: key ifAbsent: [ nil ]) value: newValue.
		]
	].
	self addWidgetNamed: widgetName withSpec: result coordinator: coordinator! !
	
! MobileSelector class methodsFor: #inspect !
prototypeSpec
	" Return a prototype spec of the receiver. "

	^(#class -> #MobileSelector),
	(#configuration -> (
		  (#order -> 0), (#height -> 0),
		  (#listeners -> (
			(#needsTitle -> '[:coordinator | ''title'']' ),
			(#needsItems -> '[:coordinator | ]' ),
			(#configureCell:for:in:coordinator: -> '[:cell :item :aList :coordinator | ]'),
			(#needsIconSourceFor:in:coordinator: ->  '[:item :aList :coordinator | ]'),
			(#selected: -> '[:item :coordinator | ]')
		   ))
	))! !

! MobileSelector class methodsFor: #inspect !
aspects
	" Return the aspects of the receiver. "

	^#(#order #name #needsTitle #needsItems),  
	self listenerAspects! !

! MobileSelector class methodsFor: #inspect !
listenerAspects
	" Return the listener aspects of the receiver. "

	^#(#needsTitle #needsItems 
	#configureCell:for:in:coordinator: 
	#needsIconSourceFor:in:coordinator:
	#selected:)
	! !

! MobileSelector methodsFor: #private !
dragListenerForView
	" Answer a drag listener for view "

	| listener actionDrop actionDragStarted actionDragLocation |
	actionDrop := DragEvent actionDrop.  
	actionDragStarted := DragEvent actionDragStarted.
	actionDragLocation := DragEvent actionDragLocation.
	listener := Java
		newProxy: 'Love.view.View$OnDragListener'
		functions: #onDrag -> [:v :event | 
			(event action = actionDrop) 
				ifTrue: [ [ self inspectWidget ] valueDeferred. 
						true ] 
				ifFalse: [ (event action = actionDragStarted 
							or: [ (v class = TextView or: [ v class superclass = TextView ]) 
								and: [ event action = actionDragLocation ] ])
						ifTrue: [ true ] ifFalse: [ false ] 
				]	
		].
	^listener! !

! MobileSelector methodsFor: #private !
configureListenerForView: aView
	" Configure listener for view "

	| listView titleView |
	listView := aView findViewById: R id @ #selectorListView.
	titleView := aView findViewById: R id @ #title.
	listView onDragListener: self dragListenerForView.
	titleView onDragListener: self dragListenerForView.
	! !

! MobileSelector methodsFor: #configuration !
configureList
	" Private - Configure the contents of the receiver to arrange a list of items. "

	| items list |
	items := self 
		triggerEvent: #needsItems 
		withArguments: (Array with: self coordinator)
		ifNotHandled: [ nil ].
	items isNil ifTrue: [ ^nil ].
	list := self 
		triggerEvent: #needsList 
		ifNotHandled: [ self defaultListFor: items ].
	self
		triggerEvent: #configureList:for:
		withArguments: (Array with: list with: items)
		ifNotHandled: [ self defaultConfigureList: list for: items ].
	self addChild: list.
	^list! !
	
! MobileSelector methodsFor: #defaults !
defaultView
	" Answer a default view for the receiver "

	^LoveApplication current activeCoordinator
		defaultViewFor: self xmlLayout! !

! MobileSelector methodsFor: #ui !
setViewId
	" Set the view id of the receiver "! !

! MobileSelector methodsFor: 'LoveMobile-ui' !
configureView
	" Configure the ui of the receiver "
	
	| list textWidget |
	list := self list. 
	list isNil ifTrue: [ ^self ].
	list isNil ifFalse: [ list configure ]. 
	textWidget := self text.
	textWidget isString ifTrue: [ list setTitle: textWidget ] 
	ifFalse: [ textWidget configureView.
	list setTitle: textWidget text ].
	self configureListenerForView: self view! !

! MobileSelector methodsFor: 'LoveMobile-ui' !
list
	" Answer the list of the receiver "
	
	^self 
		childKindOf: LoveSelector 
		ifNone: [ nil ]! !

! MobileSelector methodsFor: 'LoveMobile-ui' !
lines: anObject
	" This message is neither not sent nor implemented, in IMLT8 web "! !

! MobileSelector methodsFor: 'LoveMobile-ui' !
text
	" Answer the text of the receiver "

	^self 
		childKindOf: TextWidget 
		ifNone: [ String space ]! !

! MobileSelector methodsFor: 'LoveMobile-ui' !
childKindOf: aClass ifNone: aBlock
	" Answer the child of kind aClass if none evaluate aBlock. "
	
	^self children 
		detect: [ :child | child isKindOf: aClass ]
		ifNone: aBlock! !

! MobileSelector methodsFor: #selecting !
select: item
	" Private - Process the selection of item in the receiver. "

	self selected: item.
	self triggerEvent: #selectionChanged.
	self triggerEvent: #selected: withArguments: (Array with: self selected with: self coordinator).
	LoveApplication current activeCoordinator open.! !

! MobileSelector methodsFor: #stage2 !
xmlLayout
	" Return the proper layout "

	^#mobile_selector! !

! MobileSelector methodsFor: #configuration !
configureTitle
	" Private - Configure the title of the receiver. "

	| title |
	title := self triggerEvent: #needsTitle withArguments: (Array with: self coordinator) ifNotHandled: [ nil ].
	title isNil ifTrue: [ ^nil ].
	" title isString ifTrue: [ title := HTMLWidget text: '<h2>',title,'</h2>' ]. "
	self addChild: title! !

! MobileSelector methodsFor: #accessing !
coordinator
	" Return the coordinator of the receiver. "

	^coordinator! !

! MobileSelector methodsFor: #accessing !
coordinator: anObject
	" Set the coordinator of the receiver. "

	coordinator := anObject! !

! MobileSelector methodsFor: #list !
defaultCellFor: item in: list
	" Private - Return the cell to render item in the list. "

	| cell |
	cell := MobileGroupCell new.
	cell	group: self.
		" href: '#'; when: #click: do: [:evt|
			self select: item in: cell event: DOMMouseEvent @ evt 
		]."
	cell := self
		triggerEvent: #configureCell:for:in:coordinator:
		withArguments: (Array with: cell with: item with: list with: self coordinator)
		ifNotHandled: [ self defaultConfigureCell: cell for: item in: list ].
	^cell! !

! MobileSelector methodsFor: #selecting !
defaultSetIconFor: anLoveListItem in: aList
	" Set the icon to use as expander of the item.
	Note that click: event must be set if it is bound an action on the icon "

	| expander iconSource iconSize |
	iconSource := self
		triggerEvent: #needsIconSourceFor:in:coordinator:
		withArguments: (Array with: anLoveListItem with: aList with: self coordinator)
		ifNotHandled: [ nil ].
	expander := anLoveListItem expander.
	expander isNil ifTrue: [
		anLoveListItem expander: anLoveListItem defaultExpander.
		expander := anLoveListItem expander.
	].
	iconSource isNil ifTrue: [
		expander style width: #0px.
		^self
	].
	expander src: iconSource.
	iconSize := self device iconSize.
	expander style
		width: iconSize;
		height: iconSize;
		yourself.! !

! MobileSelector methodsFor: #fileIn !
listeners: jsObject

	(JS @ jsObject) properties do: [:listener |
		self when: listener first do: listener last
	]! !

! MobileSelector methodsFor: #gui !
openIn: aParent before: aWidget
	" Open the receiver before element inside aParent. "

	super openIn: aParent before: aWidget.
	self refresh. ! !

! MobileSelector methodsFor: #accessing !
order
	" Return the order of the receiver. "

	^order! !

! MobileSelector methodsFor: #accessing !
order: anObject
	" Set the order of the receiver. "

	order := anObject! !

! MobileSelector methodsFor: #accessing !
outline
	" Set the receiver to none display. "

	self display: #none! !

! MobileSelector methodsFor: #accessing !
container
	" Answer the container. "

	^self view findViewById: R id @ #mobileSelector ! !

! MobileSelector methodsFor: #accessing !
refresh

	| items |
	(self children notEmpty and: [self children last isKindOf: ListWidget])
		ifFalse: [ self configure ]
		ifTrue: [
			self children last reset.
			items := self
				triggerEvent: #needsItems 
				withArguments: (Array with: self coordinator)
				ifNotHandled: [ nil ].
			self children last when: #needsRoots do: [ items ].
			self children last configure.
			self defaultConfigureList: self children last for: items
	].
	super refresh! !

! LoveList class methodsFor: #instantiation !
@ aLoveListConfiguration
	" Return a new instance of the receiver based on anLoveListConfiguration. "
	
	| loveList |
	loveList := self new
		configuration: aLoveListConfiguration;
		yourself.
	aLoveListConfiguration owner: LoveList.
	loveList configureListView.
	^loveList! !

! LoveList class methodsFor: #events !
localEvents
	" Private - Return the smalltalk events managed by instances of the receiver. "

	^#(
	#getContents #needsRoots
	#hasContents: #needsContentsOf:
	#needsTitleOf: #makeupList:
	#needsConfiguration
	)! !

! LoveList methodsFor: #ui !
iconFor: object
	" Answer a proper icon representation for object. "

	self coordinator iconFor: object! !

! LoveList methodsFor: 'LoveMobile-accessing' !
owner
	" Private - Return the owner of the receiver. "

	^self! !

! LoveList methodsFor: #configuration !
flat
	" Set the receiver to be flat (no children). "

	self
		when: #hasContents: do: [:anItem| false ];
		when: #needsContentsOf: do: [:anItem| #() ]! !

! LoveList methodsFor: #accessing !
setContents: anArray
	" set the contents of the receiver "

	self configuration isNil ifTrue: [ self basicSetContents: anArray ]
	ifFalse: [ self configuration setContents: anArray ]! !

! LoveList methodsFor: #private !
basicSetContents: anArray
	" set the contents of the receiver "

	self elements: anArray.
	" instead of creating a new ArrayList each time, reset current 
	array list elements "
	self setList.
	"self configureListView"! !

! LoveList methodsFor: 'LoveMobile-accessing' !
titleOf: anObject
	" Private - Return the title used to show reference to anObject. "

	| title |
	title := self triggerEvent: #needsTitleOf:
		withArguments: (Array with: anObject)
		ifNotHandled: [ anObject ].
	^title! !

! LoveList methodsFor: 'LoveMobile-configuration' !
configure
	" Private - Configure the receiver's internal structure. "

	super configure.
	self triggerEvent: #getContents ifNotHandled: [
		self addChildren: (self itemsFrom:
			(self triggerEvent: #needsRoots ifNotHandled: [ #() ]))
	]! !

! LoveListPart methodsFor: 'LoveMobile-converting' !
itemsFrom: objects
	" Return a collection of items built from objects. "

	^objects collect: [:each| self itemFrom: each ]! !

! LoveListPart methodsFor: 'LoveMobile-converting' !
itemFrom: anObject
	" Return an item built from anObject. "

	^self triggerEvent: #needsItemFrom:
		withArguments: (Array with: anObject)
		ifNotHandled: [ self defaultItemFrom: anObject ]! !

! LoveListPart methodsFor: #converting !
defaultItemFrom: anObject
	" Return a (default) item built from anObject. "

	^LoveListItem for: anObject in: self owner! !

! LoveListItem class methodsFor: 'LoveMobile-instantiation' !
for: anObject in: aTree
	" Return an instance of the receiver. "

	^self new
		owner: aTree;
		object: anObject;
		yourself! !

! LoveListItem methodsFor: 'LoveMobile-configuration' !
configure
	" Private - Configure the receiver's internal structure. "

	super configure.
	self triggerEvent: #getTitle ifNotHandled: [ self addChild: self defaultTitle ].
	self triggerEvent: #getContents.! !

! LoveListItem methodsFor: 'LoveMobile-gui' !
description
	" Return the description of the receiver. "

	| cell row |
	cell := self children first.
	row := cell children first.
	^row text! !

! LoveListItem methodsFor: 'LoveMobile-gui' !
defaultTitle
	" Return the (default) title of the receiver. "

	^self owner titleOf: object! !
	
! LoveList methodsFor: #private !
setList
	" (re)Set the list of the receiver. "

	| strings |
	self list clear.
	self configuration setList.
	self elementsChanged! !

! LoveList methodsFor: #ui !
refreshView
	" (re)Set the list of the receiver. "

	self setList! !

! LoveList methodsFor: #ui !
open
	" open the receiver. "

	^self! !

! LoveList methodsFor: #ui !
reset
	" reset the receiver. "

	^self setList! !
	
! LoveList methodsFor: #private !
elementsChanged
	" Refresh the list view of the receiver because 
	the list of the receiver has changed. "

	self listView adapter notifyDataSetChanged! !

! LoveList methodsFor: #defaults !
defaultIsDirty
	" Private - Return default isDirty "
	
	^false! !

! LoveList methodsFor: #defaults !
defaultElements
	" Private - Return default elements "
	
	^self configuration initialElements! !

! LoveList methodsFor: #defaults !
defaultList
	" Private - Return default list "
	
	"| strings newList |
	strings := self elements collect: [:element | element perform: self stringSelector ].
	newList := ArrayList initialCapacity: strings size.
	strings do: [:element | newList add: element].
	^newList"
	^self configuration defaultList! !

! LoveList methodsFor: #accessing !
onItemClickAction
	" Private - an item in the receiver was clicked, do an action "

	self clickAction notNil ifTrue: [
		self coordinator isNil ifTrue: [
			self clickAction 
				value: self selectedElement
		] ifFalse: [
		self clickAction 
			value: self coordinator 
			value: self selectedElement
		]
	] ifFalse: [
		self coordinator onItemClickAction: self selectedElement
	]! !

! LoveList methodsFor: #accessing !
onItemLongClickAction
	" Private - an item in the receiver was long clicked, do an action "

	self coordinator onItemLongClickAction: self selectedElement! !

! LoveList methodsFor: #accessing !
layout
	" Private - Answer the layout of the receiver "

	^self configuration layout! !

! LoveList methodsFor: #accessing !
listViewLayout
	" Private - Answer the listViewLayout of the receiver "

	^self configuration listViewLayout! !

! LoveList methodsFor: #accessing !
listSelectorLayout
	" Private - Answer the listSelectorLayout of the receiver "

	^self configuration listSelectorLayout! !

! LoveList methodsFor: #accessing !
coordinator
	" Private - Answer the coordinator of the receiver "

	^self configuration coordinator! !

! LoveList methodsFor: #accessing !
container
	" Private - Answer the container of the receiver "

	^self configuration container! !

! LoveList methodsFor: #accessing !
stringSelector
	" Private - Answer the stringSelector of the receiver "

	^self configuration stringSelector! !

! LoveList methodsFor: #accessing !
clickAction
	" Private - Answer the clickAction of the receiver "

	^self configuration clickAction! !

! LoveList methodsFor: #accessing !
context
	" Private - Answer the context of the receiver "

	^self coordinator isNil 
		ifTrue: [ MainActivity singleton ] ifFalse: [
	self coordinator context ]! !

! LoveList methodsFor: #defaults !
configureListView
	" Private - configure the list view of the receiver "
	
	| lAdapter "resource" |
	" resource := R layout @ self layout. "
	lAdapter := self configuration adapter.
	
	" lAdapter := ArrayAdapter 
		context: self context 
		resource: resource 
		listObjects: self list. "
		
	self listView
		selector: R drawable @ self listSelectorLayout;
		adapter: lAdapter.
	self setListenersForListView! !

! LoveList methodsFor: #ui !
listView
	" Answer the list view of the receiver "
	
	| layout |
	layout := self listViewLayout.
	layout isNil ifTrue: [ ^self noLayoutListView ].
	self container notNil ifTrue: [
		^self container findViewById: R id @ layout
	].
	^self coordinator viewNamed: layout! !

! LoveList methodsFor: #ui !
hideView
	" Hide the list view of the receiver "
	
	self setVisibility: View gone! !
	
! LoveList methodsFor: #ui !
showView
	" Show the list view of the receiver "
	
	self setVisibility: View visible! !
	
! LoveList methodsFor: #ui !
setVisibility: aNumber
	" Set list view visibility for the receiver "

	" self listView handle #setVisibility: aNumber "
	self listView visibility: aNumber! !

! LoveList methodsFor: #ui !
setListenersForListView
	" Private - Set listener for the list view of the receiver "
	
	self setOnClickListenerForListView.
	self setOnLongClickListenerForListView! !

! LoveList methodsFor: #ui !
setOnClickListenerForListView
	" Private - Set listener for the list view of the receiver "
	
	| listener |
	listener := AdapterView onItemClick: [:parent :view :position :id | | offset |
		offset := position.
		[	self index: offset + 1.
			self listView selection: offset.
			self onItemClickAction.
		] valueDeferred
	].
	self listView onItemClickListener: listener.! !

! LoveList methodsFor: #ui !
setOnLongClickListenerForListView
	" Private - Set listener for the list view of the receiver "
	
	| listener |
	listener := AdapterView onItemLongClick: [:parent :view :position :id | | offset |
		offset := position.
		[	self index: offset + 1.
			self listView selection: offset.
			self onItemLongClickAction.
		] valueDeferred. true
	].
	self listView onItemLongClickListener: listener.! !

! LoveList methodsFor: #private !
defaultNoLayoutListView
	" Answer a default listView for the receiver 
	if there is no listViewLayout. "
	
	^ListView context: self context! !
	
! LoveList methodsFor: #private !
refreshList
	" Refresh the list of the receiver. "
	
	self coordinator refreshList: self! !

! LoveList methodsFor: #accessing !
selectedElement
	" Return the selected element "
	
	^self elements at: index! !

! LoveList methodsFor: #accessing !
selectedIndex
	" Return the selected index "
	
	^index! !

! LoveList methodsFor: #ui !
selectItemAt: anInteger
	" Select and highlight item at index anInteger "
	
	self index: anInteger + 1.
	self listView  
		setItemChecked: anInteger
		value: true! !

! LoveList methodsFor: #ui !
firstVisiblePosition
	" Answer the first visible position of the receiver "
	" I.N.: it's zero-indexed "
	
	^self listView firstVisiblePosition! !

! LoveList methodsFor: #ui !
checkedItemPosition
	" Answer the checked item position in the receiver "
	
	^self listView checkedItemPosition! !

! LoveList methodsFor: #ui !
childAt: position
	" Answer the view at position in the receiver "
	
	^self listView getChildAt: position! !

! LoveList methodsFor: #ui !
add: anObject
	" Add anObject to the receiver "
	
	self list add: anObject! !

! LoveList methodsFor: #ui !
remove: anInteger
	" Remove the object at index anInteger from receiver "
	
	self list remove: anInteger! !

! LoveList methodsFor: #ui !
set: anInteger to: anObject
	" Put anObject at index anInteger in the receiver "
	
	self list set: anInteger to: anObject! !

! LoveListConfiguration methodsFor: #private !
setContents: anArray
	" set the contents of the receiver "

	self owner basicSetContents: anArray! !

! LoveListConfiguration methodsFor: #private !
setList
	" (re)Set the list of the receiver. "

	| strings |
	strings := self elements 
		collect: [:element | self stringRepresentationFor: element ].
	strings do: [:element | self list add: element ]! !

! LoveListConfiguration methodsFor: #private !
stringRepresentationFor: element
	" Answer a string representation for element. "

	^self stringSelector isString 
		ifTrue: [ element perform: self stringSelector ]
		ifFalse: [ self stringSelector value: element 
	].! !

! LoveListConfiguration methodsFor: #private !
adapter
	" answer the adapter for receiver's owner "

	| resource |
	resource := R layout @ self layout.
	^ArrayAdapter 
		context: self context 
		resource: resource 
		listObjects: self list.! !

! LoveListConfiguration methodsFor: #private !
context
	" Private - return context "

	^self owner context! !

! LoveListConfiguration methodsFor: #private !
list
	" Private - Return list "

	^self owner list! !

! LoveListConfiguration methodsFor: #private !
elements
	" Private - Return elements "

	^self owner elements! !

! LoveListConfiguration methodsFor: #private !
defaultLayout
	" Private - Return layout "

	^#workspace_list_chunk! !

! LoveListConfiguration methodsFor: #default !
defaultList
	" Private - Return default list "
	
	| strings newList |
	strings := self elements 
		collect: [:element | self stringRepresentationFor: element ].
	newList := ArrayList initialCapacity: strings size.
	strings do: [:element | newList add: element].
	^newList! !

! LoveListConfiguration methodsFor: #default !
defaultImageNames
	" Private - Return default imageNames "

	^Array new! !

! MultiViewListConfiguration methodsFor: #private !
defaultLayout
	" Private - Return layout "

	^#item_with_view! !

! MultiViewListConfiguration methodsFor: #default !
defaultList
	" Private - Return default list "

	| array newList |
	array := self listElements.
	" arrayListElements := self elements 
		collect: [:element | element perform: self stringSelector ]. "
	newList := ArrayList initialCapacity: array size.
	array do: [:element | newList add: element].
	^newList! !

! MultiViewListConfiguration methodsFor: #private !
setList
	" (re)Set the list of the receiver. "

	| array |
	array := self listElements.
	array do: [:element | self list add: element].! !

! MultiViewListConfiguration methodsFor: #private !
listElements
	" Private - Return list elements "

	| keys |
	^self imageNames notEmpty 
		ifTrue: [ self elementsBasedOnImageNames ]
		ifFalse: [
		keys := (JS @ self spec) keys.
		self elements 
			collect: [:element | | hm |
				hm := HashMap new.
				keys
					do: [:key | | value |
						value := ((JS @ self spec) @ key ) value: element value: self.
						hm put: key value: value.
				].
				hm
		].
	]
	"^self elements 
		collect: [:element | | hm |
			hm := HashMap new.
			hm put: #itemName value: element name.
			hm put: #icon value: R drawable @ (self iconNameFor: element).
			hm
	]."! !

! MultiViewListConfiguration methodsFor: #private !
elementsBasedOnImageNames
	" Answer elements based on receiver's image names "
	"I.N.: its assumed that the receiver have the same
	number of elements and image names "

	| hms |
	hms := Array new.
	1 to: self elements size
	do: [:i| | hm |
			hm := HashMap new.
			hm 
				put: #itemName 
				value: (self elements at: i).
			hm 
				put: #icon 
				value: R drawable @ (self imageNames at: i).
			hms add: hm
	].
	^hms! !

! MultiViewListConfiguration methodsFor: #private !
adapter
	" answer the adapter for receiver's owner "

	| resource from to |
	from := JavaObject of: String withAll: #(#itemName #icon).
	to := JavaObject of: Integer withAll: (Array with: R id @ #itemName with: R id @ #icon). 
	resource := R layout @ self layout.
	^MainActivity singleton  
		createSimpleAdapter: self context 
		data: self list 
		resource: resource
		from: from
		to: to. ! !

LoveWidget
	subclass: #MobileJsonWidget
	instanceVariableNames: ' order coordinator '
	category: #LoveWidget !

! MobileJsonWidget methodsFor: #accessing !
coordinator
	" Return the coordinator of the receiver. "

	^coordinator! !

! MobileJsonWidget methodsFor: #accessing !
coordinator: anObject
	" Set the coordinator of the receiver. "

	coordinator := anObject! !

! MobileJsonWidget methodsFor: #accessing !
listeners: jsObject

	(JS @ jsObject) properties do: [:listener |
		self when: listener first do: listener last 
	]! !

! MobileJsonWidget methodsFor: #accessing !
order
	" Return the order of the receiver. "

	^order! !

! MobileJsonWidget methodsFor: #accessing !
order: anObject
	" Set the order of the receiver. "

	order := anObject! !

MobileJsonWidget
	subclass: #MobileAlignWidget
	instanceVariableNames: ' align content '
	category: #LoveWidget !

! MobileAlignWidget methodsFor: #accessing !
align
	" Return the align of the receiver. "

	^align! !

! MobileAlignWidget methodsFor: #accessing !
align: anObject
	" Set the align of the receiver. "

	align := anObject! !

! MobileAlignWidget methodsFor: #accessing !
content
	" Return the content of the receiver. "

	^content! !

! MobileAlignWidget methodsFor: #accessing !
content: anObject
	" Set the content of the receiver. "

	content := anObject! !

! MobileJsonWidget methodsFor: #private !
dragListenerForView
	" Answer a drag listener for view "

	| listener actionDrop actionDragStarted actionDragLocation |
	actionDrop := DragEvent actionDrop.  
	actionDragStarted := DragEvent actionDragStarted.
	actionDragLocation := DragEvent actionDragLocation.
	listener := Java
		newProxy: 'Love.view.View$OnDragListener'
		functions: #onDrag -> [:v :event | 
			(event action = actionDrop) 
				ifTrue: [ [ self inspectWidget ] valueDeferred. 
						true ] 
				ifFalse: [ (event action = actionDragStarted 
							or: [ event action = actionDragLocation ])
						ifTrue: [ true ] ifFalse: [ false ] 
				]	
		].
	^listener! !

! MobileAlignWidget methodsFor: #style !
inline
	" Set the receiver to none display. "

	self content display: #inline! !

! MobileAlignWidget methodsFor: #gui !
open
	" Open the receiver. "

	super open.
	self opened! !

! MobileAlignWidget methodsFor: #style !
outline
	" Set the receiver to none display. "

	self content display: #none! !

! MobileAlignWidget methodsFor: #private !
tag
	" Private - Return the HTML tag to host the receiver. "

	^self align isNil
		ifTrue: [ super tag ]
		ifFalse: [ self align ]! !

MobileAlignWidget
	subclass: #MobileEntryField
	instanceVariableNames: ''
	category: #WebWidget !

! MobileEntryField methodsFor: #accessing !
autofocus: boolean
	" Return an instance of the receiver. "

	boolean ifTrue: [ self content autofocus ]! !

! MobileEntryField methodsFor: #accessing !
changed 

	self 
		triggerEvent: #changed:
		withArguments: (Array with: self coordinator)! !

! MobileEntryField methodsFor: #accessing !
content
	" Return the content of the receiver. "

	content isNil ifTrue: [
		content := MobileTextInput new.
		content when: #changed send: #changed to: self.
	].
	^content! !

! MobileEntryField methodsFor: #accessing !
done 

	self 
		triggerEvent: #done:
		withArguments: (Array with: self coordinator with: self)! !

! MobileEntryField methodsFor: #accessing !
opened

	self addChild: self content! !

! MobileEntryField methodsFor: #accessing !
size: charCount
	" Return an instance of the receiver. "

	self content size: charCount! !

! MobileEntryField methodsFor: #ui !
editText
	" Answer the editText. "

	id isNil ifTrue: [ ^self view ].
	^self activeCoordinator findViewById: self id! !

! MobileEntryField methodsFor: #accessing !
text
	" Return the contents of the receiver. "

	^self editText text toString! !

! MobileEntryField methodsFor: #accessing !
text: aString
	" Set receivers contents to aString. "

	^self editText text: aString! !

! MobileEntryField methodsFor: #accessing !
value
	" Return the (text) value of the receiver. "

	^self content value! !

! MobileEntryField methodsFor: #accessing !
value: aString
	" Set the value the (text) value of the receiver 
	to aString. "

	^self content value: aString! !

! MobileEntryField methodsFor: #defaults !
defaultView
	" Answer a default view for the receiver "

	^EditText context: self support context.
	! !

! MobileEntryField methodsFor: #accessing !
activeCoordinator
	" Answer the active coordinator "
	
	^LoveApplication current activeCoordinator! !

! MobileEntryField methodsFor: #accessing !
autocapitalizationType: anObject

! !

! MobileEntryField methodsFor: #accessing !
autocorrectionType: anObject

! !

! MobileEntryField methodsFor: #accessing !
background: anObject

! !

! MobileEntryField methodsFor: #accessing !
border: anObject

! !

! MobileEntryField methodsFor: #accessing !
spellCheckingType: anObject

! !

! MobileEntryField methodsFor: #accessing !
returnKeyType: anObject

! !

! MobileEntryField methodsFor: #accessing !
color: anObject

! !

! MobileEntryField methodsFor: #accessing !
defaultText: aString
	self text: aString

! !

! MobileEntryField methodsFor: #accessing !
height: anObject

! !

! MobileEntryField methodsFor: #accessing !
weight: anObject

! !

! MobileEntryField methodsFor: #accessing !
keyboardType: anObject

! !

! MobileEntryField methodsFor: #accessing !
width: anObject

! !

! MobileEntryField methodsFor: #accessing !
height: anObject

! !

MobileAlignWidget
	subclass: #MobileText
	instanceVariableNames: ''
	category: #WebWidget !

! MobileText class methodsFor: #inspect !
updateSpecBaseOn: dictionary prototypeSpec: aComposedAssociations coordinator: coordinator
	" Private - update -composedAssociations- spec based on dictionary 
	and prototype spec, in coordinator "
	
	| result widgetName |
	result := aComposedAssociations copy.
	dictionary keysAndValuesDo: [:key :value| | newValue |
		newValue := value text toString.
		key = #order ifTrue: [ newValue := newValue asNumber ].
		key = #name ifTrue: [ widgetName := newValue ] ifFalse: [
		(result 
			associationAt: key ifAbsent: [ nil ]) value: newValue.
		]
	].
	self addWidgetNamed: widgetName withSpec: result coordinator: coordinator! !

! MobileText class methodsFor: #inspect !
prototypeSpec
	" Return a prototype spec of the receiver. "

	^(#class -> #MobileText),
		(#configuration -> ( (#order -> 0),(#text -> 'undefined') )
	)! !

! MobileText class methodsFor: #inspect !
aspects
	" Return the aspects of the receiver. "

	^#(#order #name #text)
	! !

! MobileText methodsFor: #accessing !
content
	" Return the content of the receiver. "

	content isNil ifTrue: [ content := HTMLWidget text: '-' ].
	^content! !

! MobileText methodsFor: #accessing !
opened

	| text |
	self addChild: self content.
	text := self
		triggerEvent: #needsText:
		withArguments: (Array with: self coordinator)
		ifNotHandled: [nil].
	text isNil ifFalse: [ self text: text ]! !

! MobileText methodsFor: #accessing !
refresh

	| text |
	text := self
		triggerEvent: #needsText:
		withArguments: (Array with: self coordinator)
		ifNotHandled: [nil].
	text isNil ifFalse: [ self text: text ].
	super refresh! !

! MobileText methodsFor: #accessing !
text: aString
	" Set receivers text to aString. "

	" self content innerHtml: aString "
	self textView text: aString! !

! MobileText methodsFor: 'LoveMobile-ui' !
top: anObject
	" This message is neither not sent nor implemented, 
	in IMLT8 web "
	
	! !

! MobileText methodsFor: #defaults !
defaultView
	" Answer a default view for the receiver "

	| tv |
	tv := TextView context: self support context.
	self configureListenerForView: tv.
	^tv! !

! MobileText methodsFor: #ui !
textView
	" Answer the textView. "

	id isNil ifTrue: [ ^self view ].
	^self activeCoordinator findViewById: self id! !

! MobileText methodsFor: #accessing !
height: anObject

! !

! MobileText methodsFor: #inspect !
needsInspection
	" Anwser if the receiver needs inspection. "

	^true! !

MobileAlignWidget
	subclass: #MobileLine
	instanceVariableNames: ''
	category: #WebWidget !

! MobileLine methodsFor: #accessing !
content
	" Return the content of the receiver. "

	content isNil ifTrue: [ content := HTMLWidget text: '<hr>' ].
	^content! !

! MobileLine methodsFor: #accessing !
opened

	self addChild: self content! !

! MobileLine methodsFor: #style !
styleSelectors
	" Return the style selectors for the receiver. "

	^ #() 
	! !

! MobileLine methodsFor: #accessing !
background: anObject

! !

! MobileLine methodsFor: #accessing !
height: anObject

! !

! MobileLine methodsFor: #ui !
openIn: anLoveCoordinator
	" Open the receiver in aViewGroup. "
	" I.N.: give the receiver a chance to do anything before 
	opening "
	
	! !

TextWidget
	subclass: #TextAreaWidget
	instanceVariableNames: ''
	category: #LoveWidget !

! TextAreaWidget class methodsFor: #accessing !
htmlSupport
	" Return the support to handle instances the receiver. "

	^HTMLTextarea! !

! TextAreaWidget class methodsFor: #events !
localEvents
	" Private - Return the smalltalk events managed by instances of the receiver. "

	^#(
	#needsRows #needsCols #needsResize
	#getContents #getConfiguration
	) ,super localEvents! !


! TextAreaWidget methodsFor: #selection !
addText: aString
	" Add aString to the text after current selection. "

	self
		replaceSelectedTextWith: aString;
		collapseSelection! !

! TextAreaWidget methodsFor: #selection !
appendSelectedText: aString
	" Append text to the current selection. "

	| selection text aHandle point |
	aHandle := self htmlHandle.
	selection := self document handle basicAt: #selection.
	selection notNil ifTrue: [ " IE detected we use document selection "
		| range |
		range := selection createRange.
		range parentElement == aHandle ifFalse: [
			self print: 'selected text outside edit area'.
			^self value: self value , aString
		].
		range #collapse: false.
		range basicAt: #text put: aString.
		range select.
		^aString
	].
	text := self value.
	text isNil ifTrue: [ ^self value: aString ].
	point := aHandle basicAt: #selectionEnd. "try mozilla way"
	point isNil ifTrue: [ point := 0 ].
	aHandle basicAt: #selectionStart put: point.
	aString isEmpty ifTrue: [ ^self ].
	text :=  (text copyFrom: 1 to: point) ,aString
		,(text copyFrom: point + 1 to: text size).
	self value: text.
	aHandle basicAt: #selectionStart put: point.
	aHandle basicAt: #selectionEnd put: point + aString size.
	^aString! !

! TextAreaWidget methodsFor: #delegation !
autocapitalize
	" Delegate the resolution of autocapitalize. "

	^self handle autocapitalize! !

! TextAreaWidget methodsFor: #delegation !
autocapitalize: anObject
	" Delegate the resolution of autocapitalize:. "

	^self handle autocapitalize: anObject! !

! TextAreaWidget methodsFor: #delegation !
autocomplete
	" Delegate the resolution of autocomplete. "

	^self handle autocomplete! !

! TextAreaWidget methodsFor: #delegation !
autocomplete: anObject
	" Delegate the resolution of autocomplete:. "

	^self handle autocomplete: anObject! !

! TextAreaWidget methodsFor: #delegation !
autocorrect
	" Delegate the resolution of autocorrect. "

	^self handle autocorrect! !

! TextAreaWidget methodsFor: #delegation !
autocorrect: anObject
	" Delegate the resolution of autocorrect:. "

	^self handle autocorrect: anObject! !

! TextAreaWidget methodsFor: #selection !
collapseSelection
	" Collapse the current selection. "

	^self collapseSelection: false! !

! TextAreaWidget methodsFor: #selection !
collapseSelection: atStart
	" Collapse the current selection. "

	| selection aHandle point |
	aHandle := self htmlHandle.
	selection := self document handle basicAt: #selection.
	selection notNil ifTrue: [ " IE detected we use document selection "
		| range |
		range := selection createRange.
		range parentElement == aHandle ifFalse: [
			self print: 'selected text outside edit area'.
			^self
		].
		range #collapse: atStart.
		^self
	].
	point := atStart
		ifTrue: [ aHandle basicAt: #selectionStart ]
		ifFalse: [ aHandle basicAt: #selectionEnd ]. "try mozilla way"
	point isNil ifTrue: [ ^self ].
	aHandle basicAt: #selectionStart put: point.
	aHandle basicAt: #selectionEnd put: point.! !

! TextAreaWidget methodsFor: #delegation !
cols
	" Delegate the resolution of cols. "

	^self handle cols! !

! TextAreaWidget methodsFor: #delegation !
cols: anObject
	" Delegate the resolution of cols:. "

	^self handle cols: anObject! !

! TextAreaWidget methodsFor: #configuration !
configure
	" Private - Configure the receiver's internal structure. "

	| value |
	super configure.
	self triggerEvent: #getConfiguration ifNotHandled: [ self getDefaultConfiguration ].
	value := self triggerEvent: #needsCols ifNotHandled: [ 60 ].
	value notNil ifTrue: [ self cols: value ].
	value := self triggerEvent: #needsRows ifNotHandled: [ 10 ].
	value notNil ifTrue: [ self rows: value ].
	value := self triggerEvent: #needsResize ifNotHandled: [ nil ].
	value notNil ifTrue: [ self resize: value ].
	self triggerEvent: #getContents.! !

! TextAreaWidget methodsFor: #delegation !
defaultValue
	" Delegate the resolution of defaultValue. "

	^self handle defaultValue! !

! TextAreaWidget methodsFor: #delegation !
defaultValue: anObject
	" Delegate the resolution of defaultValue:. "

	^self handle defaultValue: anObject! !

! TextAreaWidget methodsFor: #delegation !
disabled
	" Delegate the resolution of disabled. "

	^self handle disabled! !

! TextAreaWidget methodsFor: #delegation !
disabled: anObject
	" Delegate the resolution of disabled:. "

	^self handle disabled: anObject! !

! TextAreaWidget methodsFor: #configuration !
getDefaultConfiguration
	" Private - Configure the receiver's internal structure with default settings. "
! !

! TextAreaWidget methodsFor: #delegation !
host
	" Delegate the resolution of host. "

	^self handle host! !

! TextAreaWidget methodsFor: #delegation !
host: anObject
	" Delegate the resolution of host:. "

	^self handle host: anObject! !

! TextAreaWidget methodsFor: #delegation !
hostName
	" Delegate the resolution of hostName. "

	^self handle hostName! !

! TextAreaWidget methodsFor: #delegation !
hostName: anObject
	" Delegate the resolution of hostName:. "

	^self handle hostName: anObject! !

! TextAreaWidget methodsFor: #delegation !
href
	" Delegate the resolution of href. "

	^self handle href! !

! TextAreaWidget methodsFor: #delegation !
href: anObject
	" Delegate the resolution of href:. "

	^self handle href: anObject! !

! TextAreaWidget methodsFor: #delegation !
pathname
	" Delegate the resolution of pathname. "

	^self handle pathname! !

! TextAreaWidget methodsFor: #delegation !
pathname: anObject
	" Delegate the resolution of pathname:. "

	^self handle pathname: anObject! !

! TextAreaWidget methodsFor: #delegation !
port
	" Delegate the resolution of port. "

	^self handle port! !

! TextAreaWidget methodsFor: #delegation !
port: anObject
	" Delegate the resolution of port:. "

	^self handle port: anObject! !

! TextAreaWidget methodsFor: #delegation !
protocol
	" Delegate the resolution of protocol. "

	^self handle protocol! !

! TextAreaWidget methodsFor: #delegation !
protocol: anObject
	" Delegate the resolution of protocol:. "

	^self handle protocol: anObject! !

! TextAreaWidget methodsFor: #selection !
replaceSelectedTextWith: aString
	" Replace the current selection with aString. "

	| selection text aHandle start end |
	aString isEmpty ifTrue: [ ^self collapseSelection ].
	aHandle := self htmlHandle.
	selection := self document handle basicAt: #selection.
	selection notNil ifTrue: [ " IE detected we use document selection "
		| range |
		range := selection createRange.
		range parentElement == aHandle ifFalse: [
			self print: 'selected text outside edit area'.
			^self value: self value , aString
		].
		range basicAt: #text put: aString.
		range #collapse: false.
		^aString
	].
	text := self value.
	text isNil ifTrue: [ ^self value: aString ].
	start := aHandle basicAt: #selectionStart. "try mozilla way"
	start isNil ifTrue: [ start := text size ].
	end := aHandle basicAt: #selectionEnd. "try mozilla way"
	end isNil ifTrue: [ end := start ].
	aHandle basicAt: #selectionStart put: start.
	text :=  (text copyFrom: 1 to: start) ,aString
		,(text copyFrom: end + 1 to: text size).
	self value: text.
	aHandle basicAt: #selectionStart put: start.
	aHandle basicAt: #selectionEnd put: start + aString size.
	^aString! !

! TextAreaWidget methodsFor: #resize !
resize: aSymbol
	" Set the resize property of the receiver.
	aSymbol can be nil or one of #( none both vertical horizontal )
	Note: curretly supported in Chrome, Safari, Firefox.
	"

	self style resize: (aSymbol isNil ifTrue: [ #both ] ifFalse: [ aSymbol ])! !

! TextAreaWidget methodsFor: #delegation !
rows
	" Delegate the resolution of rows. "

	^self handle rows! !

! TextAreaWidget methodsFor: #delegation !
rows: anObject
	" Delegate the resolution of rows:. "

	^self handle rows: anObject! !

! TextAreaWidget methodsFor: #delegation !
search
	" Delegate the resolution of search. "

	^self handle search! !

! TextAreaWidget methodsFor: #delegation !
search: anObject
	" Delegate the resolution of search:. "

	^self handle search: anObject! !

! TextAreaWidget methodsFor: #selection !
selectCurrentLine
	" Select the line where the cursor is over. "

	| text selection point current stream aHandle |
	aHandle := self htmlHandle.
	selection := self document handle basicAt: #selection.
	selection notNil ifTrue: [ " IE detected we use document selection "
		"how to do it in iE ?"
		^self
	].
	(text := self value) isNil ifTrue: [ ^self ].
	point := aHandle basicAt: #selectionStart. "try mozilla way"
	point isNil ifTrue: [ point := 0 ].
	stream := text stream.
	[	current := stream position.
		stream nextLine.
		stream position >= point
	] whileFalse: [  stream atEnd ifTrue: [ ^self ] ].
	aHandle basicAt: #selectionStart put: current.
	aHandle basicAt: #selectionEnd put: stream position.! !

! TextAreaWidget methodsFor: #selection !
selectedText
	" Return the selected text. "

	| empty text selection start end aHandle |
	empty := ''.
	aHandle := self htmlHandle.
	selection := self document handle basicAt: #selection.
	selection notNil ifTrue: [ " IE detected we use document selection "
		| range |
		range := selection createRange.
		range parentElement == aHandle ifFalse: [
			self print: 'selected text outside edit area'.
			^empty
		].
		text := range basicAt: #text.
		^text isNil ifTrue: [ empty ] ifFalse: [ text ]
	].
	text := self value.
	text isNil ifTrue: [ ^empty ].
	start := aHandle basicAt: #selectionStart. "try mozilla way"
	start isNil ifTrue: [ start := 0 ].
	^text copyFrom: start + 1 to: (aHandle basicAt: #selectionEnd)! !

! TextAreaWidget methodsFor: #delegation !
spellcheck
	" Delegate the resolution of spellcheck. "

	^self handle spellcheck! !

! TextAreaWidget methodsFor: #delegation !
spellcheck: anObject
	" Delegate the resolution of spellcheck:. "

	^self handle spellcheck: anObject! !

! TextAreaWidget methodsFor: #delegation !
type
	" Delegate the resolution of type. "

	^self handle type! !

! TextAreaWidget methodsFor: #delegation !
type: anObject
	" Delegate the resolution of type:. "

	^self handle type: anObject! !


TextAreaWidget
	subclass: #MobileTextArea
	instanceVariableNames: ' order coordinator '
	category: #LoveWidget !



! MobileTextArea class methodsFor: #fileIn !
fromJSon: configuration

	^self new
		fromJSon: configuration;
		yourself! !

! MobileTextArea class methodsFor: #events !
localEvents
	" Private - Return the smalltalk events managed by instances of the receiver. "

	^#( #needsValue ) ,super localEvents! !


! MobileTextArea methodsFor: #accessing !
coordinator
	" Return the coordinator of the receiver. "

	^coordinator! !

! MobileTextArea methodsFor: #accessing !
coordinator: anObject
	" Set the coordinator of the receiver. "

	coordinator := anObject! !

! MobileTextArea methodsFor: #fileIn !
fromJSon: aJSon
	" I.N.: set receivers (default) view properties here "
	
	super fromJSon: aJSon.
	! !

! MobileTextArea methodsFor: #accessing !
listeners: jsObject

	(JS @ jsObject) properties do: [:listener |
		self when: listener first do: listener last 
	]! !

! MobileTextArea methodsFor: #accessing !
open

	| aValue |
	super open.
	aValue := self 
		triggerEvent: #needsValue 
		withArguments: (Array with: self coordinator) 
		ifNotHandled: [ nil ].
	aValue isNil ifFalse: [ self value: aValue ]! !

! MobileTextArea methodsFor: #accessing !
order
	" Return the order of the receiver. "

	^order! !

! MobileTextArea methodsFor: #accessing !
order: anObject
	" Set the order of the receiver. "

	order := anObject! !

! MobileTextArea methodsFor: #configuration !
styleSelectors
	" Return the style selectors for the receiver. "

	^ #(#top #bottom #left #right #position #height #width #border )
	! !

! MobileTextArea methodsFor: 'LoveMobile-ui' !
setProperties
	" Set the properties of the receiver "

	! !

! MobileTextArea methodsFor: #defaults !
defaultView
	" Answer a default view for the receiver "

	| widgetClass |
	(readOnly isNil or: [ readOnly not ])
		ifTrue: [ widgetClass := EditText ]
		ifFalse: [ widgetClass := TextView.
	].
	^widgetClass context: self support context.! !

! MobileTextArea methodsFor: #accessing !
text: aString
	" Set receivers text to aString. "

	self textView text: aString! !

! MobileTextArea methodsFor: #accessing !
text
	" Answer the text of the receiver. "

	^self textView text toString! !

! MobileTextArea methodsFor: #value !
value
	" Return the (text) value of the receiver. "

	^self text! !

! MobileTextArea methodsFor: #value !
value: aValue
	" Set the (text) value of the receiver. "

	^self textView text: (aValue isNil
		ifTrue: [ self emptyValue ]
		ifFalse: [ aValue asString ])! !

! MobileTextArea methodsFor: #ui !
textView
	" Answer the textView. "

	id isNil ifTrue: [ ^self view ].
	^self activeCoordinator findViewById: self id! !

! MobileTextArea methodsFor: #configuration !
configure
	" Private - Configure the receiver's internal structure. "

	| aValue |
	aValue := self 
		triggerEvent: #needsValue 
		withArguments: (Array with: self coordinator) 
		ifNotHandled: [ nil ].
	aValue isNil ifFalse: [ self value: aValue ]! !

LoveWidget
	subclass: #BreakWidget
	instanceVariableNames: ''
	category: #LoveWidget !

! BreakWidget class methodsFor: #accessing !
htmlSupport
	" Return the support to handle instances the receiver. "

	^HTMLBR! !

BreakWidget
	subclass: #MobileBreak
	instanceVariableNames: ' order '
	category: #LoveWidget !

! MobileBreak methodsFor: #accessing !
coordinator: ignored
	" Set the coordinator of the receiver. "! !

! MobileBreak methodsFor: #accessing !
order
	" Return the order of the receiver. "

	^order! !

! MobileBreak methodsFor: #accessing !
order: anObject
	" Set the order of the receiver. "

	order := anObject! !

! MobileBreak methodsFor: #configuration !
styleSelectors
	" Return the style selectors for the receiver. "

	^ #() 
	! !

! MobileBreak methodsFor: #ui !
openIn: anLoveCoordinator
	" Open the receiver in aViewGroup. "
	" I.N.: give the receiver a chance to do anything before 
	opening "
	
	! !

LoveWidget
	subclass: #LoveImageWidget
	instanceVariableNames: ''
	category: #LoveWidget!



! LoveImageWidget class methodsFor: #instantiation !
fileName: fileName in: library
	" Return an instance of the receiver with image at fileName in library (or nil). "

	^self newDefault src: fileName in: library! !

! LoveImageWidget class methodsFor: #accessing !
htmlSupport
	" Return the support to handle instances the receiver. "

	" ^HTMLImage" ! !

! LoveImageWidget class methodsFor: #instantiation !
icon: iconSpec
	" Return an instance of the receiver set with icon (or nil). "

	^self newDefault icon: iconSpec! !

! LoveImageWidget class methodsFor: #instantiation !
newDefault
	" Private - Return a default instance of the receiver. "

	| result |
	result := self inline.
	result style verticalAlign: #middle.
	^result! !


! LoveImageWidget methodsFor: #delegation !
alt
	" Delegate the resolution of alt. "

	^self handle alt! !

! LoveImageWidget methodsFor: #delegation !
alt: anObject
	" Delegate the resolution of alt:. "

	^self handle alt: anObject! !

! LoveImageWidget methodsFor: #delegation !
border
	" Delegate the resolution of border. "

	^self handle border! !

! LoveImageWidget methodsFor: #delegation !
border: anObject
	" Delegate the resolution of border:. "

	^self handle border: anObject! !

! LoveImageWidget methodsFor: #delegation !
complete
	" Delegate the resolution of complete. "

	^self handle complete! !

! LoveImageWidget methodsFor: #delegation !
complete: anObject
	" Delegate the resolution of complete:. "

	^self handle complete: anObject! !

! LoveImageWidget methodsFor: #delegation !
extent
	" Delegate the resolution of extent. "

	^self handle extent! !

! LoveImageWidget methodsFor: #delegation !
extent: anObject
	" Delegate the resolution of extent:. "

	^self handle extent: anObject! !

! LoveImageWidget methodsFor: #accessing !
handle: aHandle
	" Set the handle of the receiver. "

	aHandle isString ifTrue: [ ^self src: aHandle ].
	^super handle: aHandle! !

! LoveImageWidget methodsFor: #delegation !
hspace
	" Delegate the resolution of hspace. "

	^self handle hspace! !

! LoveImageWidget methodsFor: #delegation !
hspace: anObject
	" Delegate the resolution of hspace:. "

	^self handle hspace: anObject! !

! LoveImageWidget methodsFor: #accessing !
icon: iconSpec
	" Set the receiver's image from iconSpec. "

	| iconName iconLibrary |
	iconName := iconSpec isString
		ifTrue: [ iconSpec ]
		ifFalse: [
			iconLibrary := iconSpec last.
			iconSpec first
		].
	^self src: iconName in: iconLibrary! !

! LoveImageWidget methodsFor: #delegation !
isMap
	" Delegate the resolution of isMap. "

	^self handle isMap! !

! LoveImageWidget methodsFor: #delegation !
isMap: anObject
	" Delegate the resolution of isMap:. "

	^self handle isMap: anObject! !

! LoveImageWidget methodsFor: #delegation !
longDesc
	" Delegate the resolution of longDesc. "

	^self handle longDesc! !

! LoveImageWidget methodsFor: #delegation !
longDesc: anObject
	" Delegate the resolution of longDesc:. "

	^self handle longDesc: anObject! !

! LoveImageWidget methodsFor: #delegation !
lowscr
	" Delegate the resolution of lowscr. "

	^self handle lowscr! !

! LoveImageWidget methodsFor: #delegation !
lowscr: anObject
	" Delegate the resolution of lowscr:. "

	^self handle lowscr: anObject! !

! LoveImageWidget methodsFor: #delegation !
spaceExtent
	" Delegate the resolution of spaceExtent. "

	^self handle spaceExtent! !

! LoveImageWidget methodsFor: #delegation !
spaceExtent: anObject
	" Delegate the resolution of spaceExtent:. "

	^self handle spaceExtent: anObject! !

! LoveImageWidget methodsFor: #delegation !
src
	" Delegate the resolution of src. "

	^self handle src! !

! LoveImageWidget methodsFor: #delegation !
src: anObject
	" Delegate the resolution of src:. "

	^self handle src: anObject! !

! LoveImageWidget methodsFor: #accessing !
src: fileName in: library
	" Set the receiver with src image at fileName in library.
	Return the receiver or nil if the image is missing. "

	| url |
	url := self urlFor: library image: fileName.
	url isNil ifTrue: [ ^nil ].
	self src: url.! !

! LoveImageWidget methodsFor: #delegation !
useMap
	" Delegate the resolution of useMap. "

	^self handle useMap! !

! LoveImageWidget methodsFor: #delegation !
useMap: anObject
	" Delegate the resolution of useMap:. "

	^self handle useMap: anObject! !

! LoveImageWidget methodsFor: #delegation !
vspace
	" Delegate the resolution of vspace. "

	^self handle vspace! !

! LoveImageWidget methodsFor: #delegation !
vspace: anObject
	" Delegate the resolution of vspace:. "

	^self handle vspace: anObject! !

LoveImageWidget
	subclass: #ImageButtonMobile
	instanceVariableNames: ' action title tool side visibility '
	category: #LoveWidget !
ImageButtonMobile buildAccessors: #(
	#action #title #tool #side #visibility
)!

! ImageButtonMobile class methodsFor: #instantiation !
withAction: aBlock title: aTitle tool: aDrawable
	" Private - Return a new instance of the receiver. "

	^self new
		title: aTitle;
		action: aBlock;
		tool: aDrawable;
		yourself.! !

! ImageButtonMobile methodsFor: #configuration !
triggerClick
	" Trigger click on the receiver "

	" self action value. "
	self handle json #performClick ! !

! ImageButtonMobile methodsFor: #gui !
show
	" Show the receiver "

	"| args selector |
	args := Array with: self title with: self action.
	selector := (self side replace: #_ with: #:), #:.
	self parent 
		perform: selector
		withArguments: args"
	self handle visibility: View visible.
	self visibility: View visible! !

! ImageButtonMobile methodsFor: #gui !
hide
	" Hide the receiver "

	"| args selector |
	args := Array with: self title with: nil.
	selector := (self side replace: #_ with: #:), #:.
	self parent 
		perform: selector
		withArguments: args"
	self handle visibility: View gone.
	self visibility: View gone! !

LoveWidget
	subclass: #MobileToolbar
	instanceVariableNames: ''
	category: #LoveWidget !

! MobileToolbar class methodsFor: #events !
localEvents
	" Private - Return the smalltalk events managed by instances of the receiver. "

	^#(
	#getContents
	#aboutToRender
	) ,super localEvents! !


! MobileToolbar methodsFor: #configuration !
configure
	" Private - Configure the receiver's internal structure. "

	super configure.
	self	triggerEvent: #getContents;
		triggerEvent: #aboutToRender
		ifNotHandled: [ self updateLayout ].! !

! MobileToolbar methodsFor: #layout !
updateLayout
	" Private - Render the receiver. "

"	self isEmpty
		ifTrue: [ self hideWindow ]
		ifFalse: [ self showWindow ]
"! !

MobileToolbar
	subclass: #MobileNavigationBar
	instanceVariableNames: ' left title right '
	category: #LoveWidget !
MobileNavigationBar buildAccessors: #(
	#(#left initialize 'self defaultLeft')
	#(#right initialize 'self defaultRight')
	#title
)!	

! MobileNavigationBar methodsFor: #accessing !
application: anApplication

	"ignored"! !

! MobileNavigationBar methodsFor: #gui !
back: anAction
	" Set the back button to evaluate anAction when clicked. "

	^self left: #back action: anAction! !

! MobileNavigationBar methodsFor: #defaults !
defaultLeft
	" Answer a default left for the receiver "

	^LoveWidget new
		view: self leftLayout
		yourself! !

! MobileNavigationBar methodsFor: #defaults !
defaultRight
	" Answer a default right for the receiver "

	^LoveWidget new
		view: self rightLayout
		yourself! !

! MobileNavigationBar methodsFor: #private !
rightLayout
	" Answer the right layout container "

	^self support findViewNamed: #right_toolbar_item_container! !

! MobileNavigationBar methodsFor: #private !
leftLayout
	" Answer the left layout container "

	^self support findViewNamed: #left_toolbar_item_container! !

! MobileNavigationBar methodsFor: #private !
clear
	" Clear the receiver. "

	self title: ''.
	self left removeChildren.
	self right removeChildren.! !

! MobileNavigationBar methodsFor: #configuration !
defaultMakeup
	" Private - Set the style of the receiver. "

	" self title: self emptyTitle.
	left  := HTMLWidget inline.
	left style position: #absolute; top: #4pt; bottom: #4pt; left: #4pt.
	self addChild: left.
	right := HTMLWidget inline.
	right style position: #absolute; top: #4pt; bottom: #4pt; right: #4pt.
	self addChild: right "! !

! MobileNavigationBar methodsFor: #private !
emptyTitle
	" Private - Return an empty tile widget. "

	| result |
	result := HTMLWidget inline.
	result align: #center.
	result style position: #absolute; left: 0; right: 0.
	^result! !

! MobileNavigationBar methodsFor: #query !
hasTitle
	" Return true if the title is set/not empty. "

	| inner |
	title isNil ifTrue: [ ^false ].
	inner := title innerHTML.
	^inner notNil and: [ inner trimBlanks notEmpty ]! !

! MobileNavigationBar methodsFor: #query !
isEmpty
	" Returns true if the receiver is empty. "

	(left notNil and: [ left notEmpty ]) ifTrue: [ ^false ].
	(right notNil and: [ right notEmpty ]) ifTrue: [ ^false ].
	self hasTitle ifTrue: [ ^false ].
	^true! !

! MobileNavigationBar methodsFor: #gui !
next: anAction
	" Set the next button to evaluate anAction when clicked. "

	^self right: #next action: anAction! !

! MobileNavigationBar methodsFor: #accessing !
title
	" Return the title of the receiver. "

	^title! !

! MobileNavigationBar methodsFor: #accessing !
title: aTitle
	" Set the title of the receiver. "

	self mainViewSupport title: aTitle.

	" aTitle isString ifTrue: [
		title isNil ifTrue: [
			title := self emptyTitle.
			self addChild: title.
		].
		title style color: #white.
		title innerHTML: '<h2>',aTitle.
		^title
	].
	title notNil ifTrue: [ title close. title := nil ].
 	title := aTitle.
	title notNil ifTrue: [ self addChild: aTitle ]"! !

! MobileNavigationBar methodsFor: 'LoveMobile-gui' !
right: label action: anAction
	" Set the next button to evaluate anAction when clicked. "
	" I.N.: pending add a wrapper on the receiver, to be in 
	LoveWidget hierarchy, and in particular understand #application "

	| button imageButton isIncluded coll |
	label = #next ifTrue: [
		self right view removeAllViews.
		self right children
			do: [:child | child visibility: View gone ].
	].
	(anAction isNil or: [ label isNil ]) ifTrue: [ 
			" imageButton := self support findViewNamed: #right_action. "
			imageButton := ImageButton context: self support context.
			imageButton visibility: View invisible. 
			" self right: nil. "
	" self left update. " ^self  ].
	label isString ifTrue: [
		button := self device
			button: label
			for: self application
			doing: anAction.
	].
	button isString ifTrue: [ self print: 'button=', button ]
	ifFalse: [ self 
				configureActionForButton: button 
				side: #right_action.
				coll := self right children 
					collect: [:e | e title].
				isIncluded := coll includes: label.
				isIncluded
					ifTrue: [ coll first class = ImageButtonMobile
								ifTrue: [
										coll first handle: button handle.
										coll first visibility: button handle visibility ]
					]
					ifFalse: [ self left addChild: button ].
				self support							
					addView: button handle
					toContainer: self right view.
			" self right: button. "
	].
		" ^self 
		label: label 
		action: anAction 
		order: 99 
		icon: R drawable @ #ic_action_back "
	^button! !

! MobileNavigationBar methodsFor: 'LoveMobile-gui' !
left: label action: anAction
	" Set the back button to evaluate anAction when clicked. "
	" I.N.: pending add a wrapper on the receiver, to be in 
	LoveWidget hierarchy, and in particular understand #application "

	| button imageButton isIncluded coll |
	label = #back ifTrue: [
		self left view removeAllViews.
		self left children
			do: [:child | child visibility: View gone ].
	].
	(anAction isNil or: [ label isNil ]) ifTrue: [ 
			" imageButton := self support findViewNamed: #left_action. "
			imageButton := ImageButton context: self support context.
			imageButton visibility: View invisible. 
			" self left: nil. "
	" self left update. " ^self  ].
	label isString ifTrue: [
		button := self device
			button: label
			for: self application
			doing: anAction.
	].
	button isString ifTrue: [ self print: 'button=', button ]
	ifFalse: [ self 
				configureActionForButton: button 
				side: #left_action.
				coll := self left children 
					collect: [:e | e title].
				isIncluded := coll includes: label.
				isIncluded
					ifTrue: [ coll first class = ImageButtonMobile
								ifTrue: [
										coll first handle: button handle.
										coll first visibility: button handle visibility ]
					]
					ifFalse: [ self left addChild: button ].
				self support							
					addView: button handle
					toContainer: self left view.
			" self left: button. "
	].
		" ^self 
		label: label 
		action: anAction 
		order: 100 
		icon: R drawable @ #ic_action_back "
	^button! !
	
! MobileNavigationBar methodsFor: 'LoveMobile-gui' !
configureActionForButton: anImageButtonMobile side: side
	" Configure anImageButtonMobile. "

	| imageButton listener support |
	" imageButton := self support findViewNamed: side. "
	imageButton := ImageButton context: self support context.
	imageButton visibility: View visible. 
	imageButton imageResource: anImageButtonMobile tool.
	imageButton backgroundColor: JavaColor transparent.
	listener:= self buttonListenerWithAction: anImageButtonMobile action.
	imageButton onClickListener: listener. 
	anImageButtonMobile handle: imageButton.
	anImageButtonMobile visibility: View visible.
	anImageButtonMobile parent: self.
	anImageButtonMobile side: side! !

! MobileNavigationBar methodsFor: 'LoveMobile-gui' !
buttonListenerWithAction: anAction
	" Returns the button listener. "

	^View onClick: [:view | anAction valueDeferred ]! !

MobileGroup
	subclass: #MobileOptions
	instanceVariableNames: ' coordinator options order '
	category: #LoveWidget !

! MobileOptions methodsFor: #configuration !
configureList
	" Private - Configure the contents of the receiver to arrange a list of items. "

	| items list |
	items := self options.
	list := self defaultListFor: items.
	 self defaultConfigureList: list for: items.
	self addChild: list.
	^list! !

! MobileOptions methodsFor: #configuration !
configureTitle
	" Private - Configure the title of the receiver. "

	| title |
	title := self triggerEvent: #needsTitle withArguments: (Array with: self coordinator) ifNotHandled: [ nil ].
	title isNil ifTrue: [ ^nil ].
	" title isString ifTrue: [ title := HTMLWidget text: '<h2>',title,'</h2>' ]. "
	self addChild: title! !

! MobileOptions methodsFor: #accessing !
coordinator
	" Return the coordinator of the receiver. "

	^coordinator! !

! MobileOptions methodsFor: #accessing !
coordinator: anObject
	" Set the coordinator of the receiver. "

	coordinator := anObject! !

! MobileOptions methodsFor: #list !
defaultCellFor: item in: list
	" Private - Return the cell to render item in the list. "

	| cell |
	cell := MobileGroupCell new.
	cell	group: self.
		" href: '#'; 
                when: #click: do: [:evt|
		    self select: item in: cell event: DOMMouseEvent @ evt
		]. "
	^item configure: cell coordinator: self coordinator! !

! MobileOptions methodsFor: #selecting !
defaultSetIconFor: aListItemWidget in: aList
	" Set the icon to use as expander of the item.
	Note that click: event must be set if it is bound an action on the icon
	"

	| expander iconSource iconSize |
	aListItemWidget object icon isNil ifFalse: [
		iconSource := aListItemWidget object iconSource: self coordinator group: self
	].

	expander := aListItemWidget expander.
	expander isNil ifTrue: [
		aListItemWidget expander: aListItemWidget defaultExpander.
		expander := aListItemWidget expander.
	].
	iconSource isNil ifTrue: [
		expander style width: #0px.
		^self
	].
	expander src: iconSource.
	iconSize := self device iconSize.
	expander style
		width: iconSize;
		height: iconSize;
		yourself.! !

! MobileOptions methodsFor: #fileIn !
listeners: jsObject

	(JS @ jsObject) properties do: [:listener |
		self when: listener first do: listener last 
	]! !

! MobileOptions methodsFor: #fileIn !
options

	^options! !

! MobileOptions methodsFor: #fileIn !
options: aJSon

	| configuration list opt |
	configuration := JS @ aJSon.
	list := #().
	configuration keys do: [:key |
		opt := MobileOptionItem new.
		opt fromJSon: configuration @ key.
		opt order: key.
		list add: opt
	].
	options := list sorted: [:before :after | before order < after order ]! !

! MobileOptions methodsFor: #accessing !
order
	" Return the order of the receiver. "

	^order! !

! MobileOptions methodsFor: #accessing !
order: anObject
	" Set the order of the receiver. "

	order := anObject! !

! MobileOptions methodsFor: #accessing !
outline
	" Set the receiver to none display. "

	self display: #none! !

! MobileOptions methodsFor: #accessing !
refresh

	| items |
	(self children notEmpty and: [self children last isKindOf: ListWidget])
		ifFalse: [ self configure ]
		ifTrue: [
			self children last reset.
			items := self
				triggerEvent: #needsItems 
				withArguments: (Array with: self coordinator)
				ifNotHandled: [ nil ].
			self children last when: #needsRoots do: [ items ].
			self children last configure.
			self defaultConfigureList: self children last for: items
	].
	super refresh! !

! MobileOptions methodsFor: #selecting !
select: item
	" Private - Process the selection of item in the receiver. "

	self selected: item.
	self triggerEvent: #selectionChanged.
	self 
		triggerEvent: #selected: 
		withArguments: (Array with: self selected with: self coordinator)
		ifNotHandled: [ item triggerSelected: coordinator ] 

	" LoveApplication current activeCoordinator open. "! !

! MobileOptions methodsFor: #stage2 !
xmlLayout
	" Return the proper layout "

	^#mobile_selector! !

! MobileOptions methodsFor: #defaults !
defaultView
	" Answer a default view for the receiver "

	^LoveApplication current activeCoordinator
		defaultViewFor: self xmlLayout! !

! MobileOptions methodsFor: 'LoveMobile-ui' !
list
	" Answer the list of the receiver "
	
	^self 
		childKindOf: LoveSelector 
		ifNone: [ nil ]! !

! MobileOptions methodsFor: 'LoveMobile-ui' !
childKindOf: aClass ifNone: aBlock
	" Answer the child of kind aClass if none evaluate aBlock. "
	
	^self children 
		detect: [ :child | child isKindOf: aClass ]
		ifNone: aBlock! !

! MobileOptions methodsFor: 'LoveMobile-ui' !
text
	" Answer the text of the receiver "

	^self 
		childKindOf: TextWidget 
		ifNone: [ String space ]! !

! MobileOptions methodsFor: 'LoveMobile-ui' !
configureView
	" Configure the ui of the receiver "
	
	| list textWidget |
	list := self list. 
	list isNil ifFalse: [ list configure ]. 
	textWidget := self text.
	textWidget isString ifTrue: [ list setTitle: textWidget ] 
	ifFalse: [ list setTitle: textWidget text ]! !

! MobileOptions methodsFor: #accessing !
container
	" Answer the container. "

	^self view findViewById: R id @ #mobileSelector ! !

Object
	subclass: #MobileOptionItem
	instanceVariableNames: ' title detail icon order description selected '
	category: #LoveWidget !



! MobileOptionItem methodsFor: #accessing !
configure: cell coordinator: coordinator 

	^coordinator
		configure: cell
		title: (self title isNil ifFalse: [ self title value: self value: coordinator ])
		description: ( self description isNil ifFalse: [self description value: self value: coordinator])
		detail: (self detail isNil ifFalse: [self detail value: self value: coordinator])! !

! MobileOptionItem methodsFor: #accessing !
description
	" Return the description of the receiver. "

	^description! !

! MobileOptionItem methodsFor: #accessing !
description: anObject
	" Set the description of the receiver. "

	description := anObject! !

! MobileOptionItem methodsFor: #accessing !
detail
	" Return the detail of the receiver. "

	^detail! !

! MobileOptionItem methodsFor: #accessing !
detail: anObject
	" Set the detail of the receiver. "

	detail := anObject! !

! MobileOptionItem methodsFor: #accessing !
fromJSon: aJSon
	
	"MobileWidgetConfigurator" JSonConfigurator configure: self fromJSon: aJSon.
	! !

! MobileOptionItem methodsFor: #accessing !
icon
	" Return the icon of the receiver. "

	^icon! !

! MobileOptionItem methodsFor: #accessing !
icon: anObject
	" Set the icon of the receiver. "

	icon := anObject! !

! MobileOptionItem methodsFor: #accessing !
iconSource: coordinator group: group

	self icon isNil ifTrue: [ ^nil ].
	^coordinator
		iconSourceFor: (self icon value: self value: coordinator )
		group: group! !

! MobileOptionItem methodsFor: #accessing !
order
	" Return the order of the receiver. "

	^order! !

! MobileOptionItem methodsFor: #accessing !
order: anObject
	" Set the order of the receiver. "

	order := anObject! !

! MobileOptionItem methodsFor: #accessing !
selected
	" Return the selected of the receiver. "

	^selected! !

! MobileOptionItem methodsFor: #accessing !
selected: anObject
	" Set the selected of the receiver. "

	selected := anObject! !

! MobileOptionItem methodsFor: #accessing !
title
	" Return the title of the receiver. "

	^title! !

! MobileOptionItem methodsFor: #accessing !
title: anObject
	" Set the title of the receiver. "

	title := anObject! !

! MobileOptionItem methodsFor: #accessing !
triggerSelected: coordinator 

	"self triggerEvent: #selected: withArguments: (Array with: self selected with: self coordinator)."
	self selected isNil ifTrue: [ ^nil ].
	self selected evaluateWithArguments: (Array with: self with: coordinator)! !
