" Turbo.lua API implementation
@2018-today Alejandro Reimondo aleReimondo@smalltalking.net
https://turbo.readthedocs.io
"
NativeObject subclass: #TurboObject category: #Turbo!
TurboObject subclass: #Turbo category: #Turbo!

! TurboObject class methodsFor: #builders !
buildConstants: constants
	" Build the methods for constants. "

	^self build: #Constants -> constants doing: [
		constants isString ifTrue: [ | stream |
			stream := constants stream.
			[stream atEnd] whileFalse: [
				self buildConstant: stream nextLine trimBlanks
			]
		] ifFalse: [ constants do: [:each| self buildConstant: each ] ]
	]! !

! TurboObject class methodsFor: #builders !
buildConstant: spec
	" Private - Build the accessor for constant spec. "

	| selector |
	spec isEmpty ifTrue: [ ^self ].
	selector := (spec fromLast: $.) asSmalltalkName: false.
	self class
		implement: selector
		comment: 'Return the constant value for ' ,spec
		as: '^{' ,$' ,spec ,$' ,'}' category: #constants! !

TurboObject class methodsFor: #accessing !
subclass: clsName implementing: impl
	" Returns a wrapper to native class inheriting from the receiver's nativeClass. "

	| result |
	# #tostring: (self nativeClass basicAt: #name).
	result := # #class: clsName subclassOf: self nativeClass.
	impl isAssociation
	ifTrue:  [ result basicAt: impl key put: impl value ]
	ifFalse: [ impl do: [:assoc| result basicAt: assoc key put: assoc value ] ].
	^NativeHandle @ result! !

Turbo class instanceVariableNames: #current!
Turbo class buildAccessor: #(current initialize 'self new handle; yourself "ensure instance is initialized"')!

! Turbo methodsFor: #initialize !
initializeHandle
	" Private - Initialize the handle of the receiver. "

	handle := # #require: #turbo! !

Turbo buildGetters: #(
	MINOR_VERSION MICRO_VERSION
	VERSION VERSION_HEX
	MAJOR_VERSION
)!

#(	log platform ioloop escape
	httputil tcpserver httpserver
	iostream iosimple crypto
	async web util coctx
	websocket socket sockutil hash
	inotify fs signal syscall thread
	structs
) do: [:selector| | clsName |
	clsName := selector asSmalltalkName: true.
	TurboObject subclass: clsName.
	Turbo
		implement: selector asLocalName
		comment: 'Returns a wrapper for library'
		as: '^' ,clsName ,' new' category: #accessing.
	(Smalltalk at: clsName)
		implement: #initializeHandle
		comment: 'Private - Initialize the handle of the receiver.'
		as: 'handle := Turbo current @#' ,selector
		category: #accessing.
]!

Log buildTypedGetters: #(
	#(categories 'PoolDictionary@@(')
)!
Log buildLuaFunctions: #(
	disable_all
	#(success #(str))
	#(notice #(str))
	#(warning #(str))
	#(error #(str))
	#(debug #(str))
	#(devel #(str))
	#(stringify #(t name indent))
	#(dump #(stuff description))
	#(stacktrace #(str))
)! 

#(	__WINDOWS__
	__UNIX__ __LINUX__ __DARWIN__
	__ABI32__ __ABI64__
	__X86__ __X64__
	__PPC__ __PPC64__
	__ARM__ __MIPSEL__
) do: [:each|
	Platform
		implement: (each asSmalltalkName: false)
		comment: 'Returns the value of ',each
		as: '^self handle basicAt: #',each
		category: #accessing

]!

Ioloop buildConstants: '
	smalltalk.Turbo:current():handle().ioloop.READ
	smalltalk.Turbo:current():handle().ioloop.WRITE
	smalltalk.Turbo:current():handle().ioloop.PRI
	smalltalk.Turbo:current():handle().ioloop.ERROR
'!
Ioloop buildLuaFunctions: #(
	"#(instance #(func) IOLoop)"
	#(instance IOLoop)
)!

! Ioloop class methodsFor: #instantiating !
instance
	" Returns an instance of the receiver. "

	^Turbo current ioloop instance! !

TurboObject subclass: #IOLoop!
IOLoop buildFunctions: #(
	initialize running start close
	#(add_handler #(fd events handler arg))
	#(update_handler #(fd events))
	#(remove_handler #(fd))
	#(add_callback #(callback arg))
	#(finalize_coroutine_context #(coctx))
	#(add_timeout #(timestamp func arg))
	#(remove_timeout #(ref))
	#(set_interval #(msec func arg))
	#(clear_interval #(ref))
	#(add_signal_handler #(signo handler arg))
	#(remove_signal_handler #(signo))
	#(_event_poll #(poll_timeout))
	#(wait #(timeout))
)!

Escape buildLuaFunctions: #(
	#(json_encode #(_t))
	#(json_decode #(s))
	#(unescape #(s))
	#(escape #(s))
	#(html_escape #(s))
	#(trim #(s))
	#(ltrim #(s))
	#(rtrim #(s))
	#(base64_decode #(str sz))
	#(base64_encode #(str sz disable_break))
)!

Httputil buildTypedGetters: #(
	#(UF 'PoolDictionary@@(')
	#(hdr_t 'PoolDictionary@@(')
)!
Httputil buildLuaFunctions: #(
	#(parse_post_arguments #(data))
	#(parse_multipart_data #(data boundary))
)!

TurboObject subclass: #HTTPParser !
! HTTPParser class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Httputil new handle basicAt: self name! !

! HTTPParser class methodsFor: #instantiation !
string: hdr_str type: hdr_t
	" Returns an instance of the receiver. "

	^self @ (self nativeClass value: hdr_str value: hdr_t)! !

! HTTPParser class methodsFor: #instantiation !
string: hdr_str
	" Returns an instance of the receiver. "

	^self @ (self nativeClass value: hdr_str)! !

! HTTPParser methodsFor: #initialize !
initializeHandle
	" Private - Initialize the handle of the receiver. "

	handle := self class nativeClass value! !

HTTPParser buildFunctions: #(
	#(initialize #(hdr_str hdr_t))
	#(parse_url #(url))
	#(get_url_field #(uf_prop))
	get_url get_method
	get_version get_status_code
	#(get_argument #(argument))
	get_arguments
	#(get #(key caseinsensitive))
	#(parse_header #(hdr_str hdr_t))
	#(parse_response_header #(raw_headers))
	#(parse_request_header #(raw_headers))
)!

TurboObject subclass: #HTTPHeaders !
! HTTPHeaders class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Httputil new handle basicAt: self name! !

! HTTPHeaders methodsFor: #initialize !
initializeHandle
	" Private - Initialize the handle of the receiver. "

	| constructor |
	constructor := self class nativeClass.
	handle := {'constructor()'}! !

HTTPHeaders buildFunctions: #(
	get_uri get_method
	get_version get_status_code
	#(set_method #(method))
	#(set_version #(version))
	#(set_status_code #(code))
	#(get #(key caseinsensitive))
	#(add #(key value))
	#(set #(key value caseinsensitive))
	#(remove #(key caseinsensitive))
	stringify_as_request
	stringify_as_response
)!

TurboObject subclass: #TCPServer !
! TCPServer class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Tcpserver new handle basicAt: self name! !

! TCPServer class methodsFor: #instantiation !
ioLoop: io_loop sslOptions: ssl_options maxBufferSize: max_buffer_size
	" Returns an instance of the receiver. "

	^self @ (self nativeClass
		value: io_loop handle
		value: ssl_options
		value: max_buffer_size)! !

TCPServer buildFunctions: #(
	#(initialize #(_io_loop ssl_options max_buffer_size))
	#(listen #(port address backlog family))
	#(add_sockets #(sockets))
	#(start #(procs))
	stop
)!

TurboObject subclass: #HTTPServer !
! HTTPServer class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Httpserver new handle basicAt: self name! !

! HTTPServer class methodsFor: #instantiation !
request_callback: request_callback
	no_keep_alive: no_keep_alive io_loop: io_loop
	xheaders: xheaders kwargs: kwargs
	" Returns an instance of the receiver. "

	| array |
	array := Array
		with: request_callback with: no_keep_alive
		with: io_loop handle with: xheaders.
	kwargs isNil ifFalse: [ array add: kwargs ].
	^self @ (self nativeClass evaluateWithArguments: array)! !

! HTTPServer class methodsFor: #instantiation !
new: request_callback
	" Returns an instance of the receiver. "

	^self @ (self nativeClass #new: request_callback)! !

HTTPServer buildFunctions: #(
	#(handle_stream #(stream address))
	#(listen #(port))
)!

TurboObject subclass: #HTTPRequest !
! HTTPRequest class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Httpserver new handle basicAt: self name! !

HTTPRequest buildTypedAccessors: #(
	method version uri
	path #(headers HTTPHeaders)
	body #(connection HTTPConection)
	files host arguments
)!
HTTPRequest buildFunctions: #(
	supports_http_1_1
	#(write #(chunk))
	#(write #(chunk callback))
	#(write #(chunk callback arg))
	#(write_buffer #(buf callback arg))
	#(write_zero_copy #(buf callback arg))
	finish full_url request_time
)!

TurboObject subclass: #HTTPConnection !
! HTTPConnection class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Httpserver new handle basicAt: self name! !

HTTPConnection buildFunctions: #(
	#(write #(chunk))
	#(write #(chunk callback))
	#(write #(chunk callback arg))
	#(write_buffer #(buf callback arg))
	#(write_zero_copy #(buf callback arg))
	finish
)!

TurboObject subclass: #IOStream !
! IOStream class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Iostream new handle basicAt: self name! !

IOStream buildFunctions: #(
	#(initialize #(fd _io_loop max_buffer_size args))
	#(connect #(address port family callback fail_callback arg))
	#(read_until #(delimiter  callback arg))
	#(read_until_pattern #(pattern callback arg))
	#(read_bytes #(num_bytes callback arg streaming_callback streaming_arg))
	#(read_until_close #(callback arg streaming_callback streaming_arg))
	#(write #(_data callback arg))
	#(write_buffer #(_buf callback arg))
	#(write_zero_copy #(_buf callback arg))
	reading writing close closed
	#(set_close_callback #(callback arg))
	#(set_maxed_buffer_callback #(callback arg))
	#(set_max_buffer_size #(sz))
)!

Iosimple buildLuaFunctions: #(
	#(dial #(address ssl _io))
)!

TurboObject subclass: #IOSimple !
! IOSimple class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Iostream new handle basicAt: self name! !

! IOSimple class methodsFor: #instantiation !
new: stream
	" Returns an instance of the receiver. "

	^self @ (self nativeClass #new: stream handle)! !

IOSimple buildFunctions: #(
	close get_iostream
	#(write #(_data))
	#(read_until #(delimiter))
	#(read_bytes #(_bytes))
	#(read_until_pattern #(pattern))
	read_until_close
)!

Crypto buildLuaFunctions: #(
	#(ssl_create_client_context #(cert_file prv_file ca_cert_path verify sslv))
	#(ssl_create_server_context #(cert_file prv_file ca_cert_path sslv))
	#(ssl_new #(_ctx fd_sock _client))
	#(ssl_do_handshake #(_aSSLIOStream))
)!

Async buildLuaFunctions: #(
	#(task #(func))
	#(task:with: #(func arg1))
	#(task:with:with: #(func arg1 arg2))
	#(task:with:with:with: #(func arg1 arg2 arg3))
	#(task:with:with:with:with: #(func arg1 arg2 arg3 arg4))
)!

TurboObject subclass: #HTTPClient !
! HTTPClient class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Async new handle basicAt: self name! !

HTTPClient buildTypedGetters: #(
	#(errors 'PoolDictionary@@(')
)!
HTTPClient buildFunctions: #(
	#(fetch #(url kwargs) "CoroutineContext"HTTPResponse)
)!

TurboObject subclass: #HTTPResponse !
! HTTPResponse class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Async new handle basicAt: self name! !

TurboObject subclass: #RequestHandler !
! RequestHandler class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Web new handle basicAt: self name! !

RequestHandler buildFunctions: #(
	#(initialize #(_application _request url_args options))
	settings prepare
	#(on_create #(kwargs))
	on_finish set_default_headers
	head get post delete put options
	#(get_argument #(name default strip))
	#(get_arguments #(name strip))
	#(get_json #(force))
	clear get_status
	#(add_header #(name value))
	#(set_header #(name value))
	#(get_header #(key))
	#(set_status #(status_code))
	#(redirect #(url permanent))
	#(send_error #(status_code msg))
	finished
	#(get_cookie #(name default))
	#(get_secure_cookie #(name default max_age))
	#(set_cookie #(name value domain expire_hours))
	#(set_secure_cookie #(name value domain expire_hours))
	#(clear_cookie #(name))
	#(set_async #(bool))
	set_chunked_write
	#(write #(chunk))
	#(flush #(callback arg))
)!

TurboObject subclass: #StaticFileHandler !
! StaticFileHandler class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Web new handle basicAt: self name! !

StaticFileHandler buildFunctions: #(
	prepare
	#(get #(path))
	#(head #(path))
)!

TurboObject subclass: #HTTPError !
! HTTPError class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Web new handle basicAt: self name! !

! HTTPError class methodsFor: #instantiation !
code: code message: message
	" Returns an instance of the receiver. "

	^self @ (self nativeClass value: code value: message)! !

RequestHandler subclass: #ErrorHandler !
! ErrorHandler class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Web new handle basicAt: self name! !

! ErrorHandler class methodsFor: #instantiation !
app: app request: request code: code message: message
	" Returns an instance of the receiver. "

	^self @ (self nativeClass
		value: app handle value: request handle
		value: code value: message)! !

RequestHandler subclass: #RedirectHandler !
! RedirectHandler class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Web new handle basicAt: self name! !

TurboObject subclass: #Application !
! Application class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Web new handle basicAt: self name! !

! Application class methodsFor: #instantiation !
handlers: handlers kwargs: kwargs
	" Returns an instance of the receiver. "

	| hResult |
	hResult := {'self:nativeClass()(handlers:json(),kwargs)'}.
	^self @ hResult! !

! Application class methodsFor: #instantiation !
handlers: handlers
	" Returns an instance of the receiver. "

	| hResult |
	hResult := {'self:nativeClass()(handlers:json())'}.
	^self @ hResult! !

Application buildFunctions: #(
	#(set_server_name #(name))
	get_server_name
	#(add_handler #(pattern handler arg))
	#(add_handler #(pattern handler))
	#(listen #(port))
	#(listen #(port address))
	#(listen #(port address kwargs))
)!

Util buildLuaFunctions: #(
	#(strsplit #(str sep max pattern))
	#(strstrip #(str))
	#(strsubstr #(str from to))
	#(rand_str #(len))
	#(tablemerge #(t1 t2))
	#(join #(delimiter list))
	#(is_in #(needle haystack))
	#(funpack #(t i))
	gettimeofday
	#(time_format_cookie #(epoch))
	#(time_format_http_header #(time_t))
	#(file_exists #(path))
	#(read_all #(file))
	#(strcasecmp #(str1 str2))
	#(str_find #(s p slen plen))
	#(hex #(num))
	#(mem_dump #(ptr sz))
	#(load_libtffi #(name))
)!

Coctx buildTypedGetters: #(
	#(states 'PoolDictionary@@(')
)!

TurboObject subclass: #CoroutineContext !
! CoroutineContext class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Coctx new handle basicAt: self name! !

! CoroutineContext class methodsFor: #instantiation !
new: io_loop
	" Returns an instance of the receiver. "

	^self @ (self nativeClass #new: io_loop handle)! !

CoroutineContext buildFunctions: #(
	#(set_arguments #(args))
	#(set_state #(state))
	#(get_state #(state))
	finalize_context
	get_coroutine_arguments
)!

Websocket buildTypedGetters: #(
	MAGIC
	#(opcode 'PoolDictionary@@(')
	#(errors 'PoolDictionary@@(')
)!


TurboObject subclass: #WebSocketStream !
! WebSocketStream class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Websocket new handle basicAt: self name! !

WebSocketStream buildFunctions: #(
	#(write_message #(msg binary))
	#(pong #(data))
	#(ping #(data callback callback_arg))
	close closed
)!

RequestHandler subclass: #WebSocketHandler !
! WebSocketHandler class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Websocket new handle basicAt: self name! !

WebSocketHandler buildFunctions: #(
	open on_close prepare
	#(on_message #(msg))
	#(on_error #(msg))
	#(subprotocol #(protocols))
)!

TurboObject subclass: #WebSocketClient !
! WebSocketClient class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Websocket new handle basicAt: self name! !

! WebSocketClient class methodsFor: #instantiation !
address: address kwargs: kwargs
	" Returns an instance of the receiver. "

	| hResult |
	hResult := {'self:nativeClass()(address,kwargs)'}.
	^self @ hResult! !

TurboObject subclass: #Sockutils !
Sockutils buildLuaFunctions: #(
	#(create_server_address #(port address family))
	#(connect_addrinfo #(sock p))
	#(bind_sockets #(port address backlog family))
	#(add_accept_handler #(_sock callback io_loop arg))
)!

TurboObject subclass: #SHA1 !
! SHA1 class methodsFor: #accessing !
nativeClass
	" Private - Returns the native instantiator. "

	^Hash new handle basicAt: self name! !

Hash buildConstants: '
	self.nativeClass().SHA_DIGEST_LENGTH
'!
Hash buildLuaFunctions: #(
	#(HMAC #(key digest raw))
	#(HMAC #(key digest))
)!

! SHA1 class methodsFor: #instantiation !
new: str
	" Returns an instance of the receiver. "

	| hResult |
	hResult := {'self:nativeClass()(str)'}.
	^self @ hResult! !

SHA1 buildFunctions: #(
	hex #(update #(str))
	finalize
)!

Inotify buildConstants: '
	self.nativeClass().IN_ACCESS
	self.nativeClass().IN_MODIFY
	self.nativeClass().IN_ATTRIB
	self.nativeClass().IN_CLOSE_WRITE
	self.nativeClass().IN_CLOSE_NOWRITE
	self.nativeClass().IN_CLOSE
	self.nativeClass().IN_OPEN
	self.nativeClass().IN_MOVED_FROM
	self.nativeClass().IN_MOVED_TO
	self.nativeClass().IN_MOVE
	self.nativeClass().IN_CREATE
	self.nativeClass().IN_DELETE
	self.nativeClass().IN_DELETE_SELF
	self.nativeClass().IN_MOVE_SELF
	self.nativeClass().IN_IGNORED
'!
Inotify buildFunctions: #(
	#(watch_file #(filePath))
	#(watch_dir #(dirPath))
	#(watch_all #(path ignore))
	#(watch_remove #(pathOrwd))
	#(rewatch_if_ignored #(event path))
	#(get_watched_file #(wd))
	get_watched_files
	close
)!

Fs buildConstants: '
	self.nativeClass().NAME_MAX
	self.nativeClass().PATH_MAX
'!
Fs buildFunctions: #(
	#(stat #(path buf))
	#(is_file #(path))
	#(is_dir #(path))
	#(glob #(pattern))
)!