" Turbo.lua API Samples
https://turbo.readthedocs.io
"

! HTTPServer class methodsFor: #samples !
example
	" Example usage of HTTPServer
	https://turbo.readthedocs.io/en/latest/httpserver.html
	"

	| port server |
	port := 8080.
	server := HTTPServer new: [:hRequest| | hdr request msg buf crlf |
		crlf := String crlf.
		request := HTTPRequest @ hRequest.
		msg := 'You requested: ',request path.
		hdr := HTTPHeaders new.
		hdr
			add: 'Content-Length' value: (# #tostring: msg size);
			add: 'Connection' value: 'Keep-Alive';
			add: 'Access-Control-Allow-Origin' value: $*;
			setStatusCode: 200;
			setVersion: 'HTTP/1.1';
			yourself.
		buf := hdr stringifyAsResponse ,crlf ,msg.
		request write: buf; finish
	].
	server listen: port.
	self note: server class name,' example on port ',port.
	"Ioloop instance start"! !

! Application class methodsFor: #samples !
example
	" Hello world server example
	https://turbo.readthedocs.io
	"

	| port handler |
	port := 8080.
	handler := RequestHandler
		subclass: #ExampleHandler
		implementing: #get -> [:self :args :opts |
			"# #print: #get with: self with: args with: opts."
			RequestHandler @ self write: '<h1>Application example</h1><h2>Object selectors</h2>',Object selectors sorted asLiteral
		].
	(Application handlers: (Array
		with: (Array with: '^/$' with: handler))
	) listen: port.
	self note: self name,' example on port ',port.
	"Ioloop instance start"! !

! Application class methodsFor: #samples !
exampleRoutes
	" Pattern matching example
	https://turbo.readthedocs.io/en/latest/get_started.html#routes
	"

	| port indexHandler userHandler addHandler |
	port := 8080.
	indexHandler := RequestHandler
		subclass: #IndexHandler
		implementing: #get -> [:self|
			"# #print: #index with: self."
			RequestHandler @ self write: 'This is the index'
		].
	userHandler := RequestHandler
		subclass: #UserHandler
		implementing: #get -> [:self :username|
			"# #print: #user with: self with: username."
			RequestHandler @ self write: 'Hello ' ,username
		].
	addHandler := RequestHandler
		subclass: #AddHandler
		implementing: #get -> [:self :a1 :a2|
			"# #print: #user with: self with: a1 with: a2."
			RequestHandler @ self write: ' ' ,a1 ,'+' ,a2 ,'=' ,(a1+a2)
		].
	(Application handlers: (Array
		with: (Array with: '/$' with: indexHandler)
		with: (Array with: '/user/(.*)$' with: userHandler)
		with: (Array with: '/add/(%d+)/(%d+)$' with: addHandler))
	) listen: port.
	self note: self name,' exampleRoutes on port ',port.
	"Ioloop instance start"! !

! Application class methodsFor: #samples !
exampleStatic
	" Serving Static Files
	https://turbo.readthedocs.io/en/latest/get_started.html#serving-static-files
	"

	| port |
	port := 8080.
	(Application handlers: (Array with: (Array
		with: '/static/(.*)$'
		with: StaticFileHandler nativeClass
		with: '/var/www/')
	)) listen: port.
	self note: self name,' exampleStatic on port ',port.
	"Ioloop instance start"! !

! Application class methodsFor: #samples !
exampleJSON
	" Serving JSON contents
	https://turbo.readthedocs.io/en/latest/get_started.html#json-output
	"

	| port handler |
	port := 8080.
	handler := RequestHandler
		subclass: #JSONHandler
		implementing: #get -> [:self|
			"# #print: #get with: self."
			RequestHandler @ self write: (#hello -> #json) json
		].
	(Application handlers: (Array
		with: (Array with: '/hello' with: handler))
	) listen: port.
	self note: self name,' exampleJSON on port ',port.
	"Ioloop instance start"! !