BootSector
	subclass: #JuliaBoot
	category: #jlBoot !

! JuliaBoot class methodsFor: #instantiation !
supportsLanguage: aLanguage
	" Private - Return true if instances of the receiver can encode to aLanguage. "

	^#jl = aLanguage! !

! JuliaBoot methodsFor: #contents !
contents
	" Return the contents (aString) to be dump as source of the receiver. "

	^super contents
		,self testExpressions
		,self notes! !

! JuliaBoot methodsFor: #constants !
version
	" Private - Returns the version string (dotted number and date). "

	^'0.7.2023.11' ! !

! JuliaBoot methodsFor: #constants !
versionDetail
	" Private - Returns this value. "

	^'Note that this is a pre-release of Julia execution support under social development(2021-today).'! !

! JuliaBoot methodsFor: #encoding !
lineComment: aString
	" Private - Encode aString as line comment. "

	^self nextPutAll: '# ' ,aString;cr ! !

! JuliaBoot methodsFor: #encoding !
multilineComment: array
	" Private - Encode an array as a multiline comment. "

	^self encode: [
		self nextPutAll: '#=' ;indent.
		array do: [:line| self cr; nextPutAll: line ].
		self dedent; cr; nextPutAll: '=#'; cr
	]! !

! JuliaBoot methodsFor: #contents !
notes
	" Private - Return the encoding for printing notes on this implementation. "

	^self encode: [
		self comment: '-------------------------------------------------------------------------------------'.
		self call: #println with: '"This is a pre-release of Julia execution support for S8 (@2021-today)."';cr.
		self call: #println with: '"Join our S8 Facebook group https://www.facebook.com/groups/s8smalltalk/"';cr.
		self call: #println with: '"boot(.jl) ready to load image."';cr.
	]! !

! JuliaBoot methodsFor: #contents !
startupExpression
	" Private - Return the encoding for startup expression of the receiver. "

	^self encode: [
		self define: #smalltalk asInstanceOf: #SystemDictionary.
		self send: #startup to: #smalltalk;cr.
		self assign: #nil to: [ self send: #basicNewInstance_ to: 'smalltalk.UndefinedObject' ];cr.
	]! !

! JuliaBoot methodsFor: #definitions !
initialDefinition
	" Private - Return the encoding for initializing execution environment. "

	^super initialDefinition ,(self encode: [
		self import: 'Base.string';cr.
		self comment: 'import Pkg; Pkg.add("HTTP")'.
		self comment: 'import Pkg; Pkg.add("JSON3")'.
		self comment: 'import Pkg; Pkg.add("Dates")'.
		self comment: 'import Pkg; Pkg.add("Debugger")'.
		self comment: 'using Debugger #uncomment for #halt inline'.
		self comment: 'using Dates # required by DateTime inlines'.
	]) ! !

! JuliaBoot methodsFor: #definitions !
nilDefinition
	" Private - Return the encoding for nil. "

	^self encode: [
		self assign: #nil to: #nothing;cr.
	]! !

! JuliaBoot methodsFor: #definitions !
implementStObject
	" Private - Implement support for StObject. "

	self defineClass: #StObject implementing: #(
		#('Base.getproperty' #('aSymbol::Symbol'))
		#('Base.setproperty!!' #('aSymbol::Symbol' object))
		#('Base.setindex!!' #(object aSymbol))
	);cr.
	self	bind: #StObject function: 'Base.getindex'
		arguments: #(aName) sending: #stObjectBaseGetindexName.
	self	bind: #StObject function: 'Base.getindex'
		arguments: #('ind::Int') sending: #stObjectBaseGetindexInt.
	self implementClass: #Any api: #(
		'handle(self::Any) = self'
		'asParameter(self::Any) = handle(self)'
	).
	self implementClass: #StObject api: #(
		'handle(self::StObject) = self["@handle"]'
		'#asParameter(self::StObject) = self["@handle"]'
		#(handleOr #(forcedHandle))
	);cr. ! !

! JuliaBoot methodsFor: #definitions !
implementStGlobalScope
	" Private - Implement support for StObject. "

	self defineClass: #StGlobalScope implementing: #(
		#('Base.getproperty' #('aSymbol::Symbol'))
		#('Base.getindex' #(aName))
		#('Base.setproperty!!' #('aSymbol::Symbol' object))
		#('Base.setindex!!' #(object aSymbol))
	);cr.
	self	define: ''
		struct: #(S8GlobalScope #(#(cookie Any)))
		inheriting: #StGlobalScope.
	self assign: 'S8GlobalScope()' to: 'S8GlobalScope(nil)'; cr! !

! JuliaBoot methodsFor: #definitions !
smalltalkDefinition
	" Private - Return the encoding for smalltalk. "

	^self encode: [
		self bind: #Any api: #(klass #('aValue::Any')); cr.
		self implementStObject.
		#(	#(#(StBehavior StObject) #(
				#(basicNewInstance_ #('fields=Dict{String,Any}()'))
				#(nativeNew_ #('handle=nil'))
				#(>> #(selector))
				#(>>> #(selector))
			))
			#(#(StMetaclass StBehavior #(
				S8Metaclass #(
					#(dict 'Dict{String,Any}')
				))) #( "empty api" ))
			#(#(StClass StBehavior #(
				S8Class #(
					#(dict 'Dict{String,Any}')
				))) #( "empty api" ))
			#(#(StMethod StObject) #( "empty api" ))
		) do: [:tuple| self defineClass: tuple first implementing: tuple last;cr ].
		self implementStGlobalScope.
		self implementSystemDictionary.
	]! !

! JuliaBoot methodsFor: #definitions !
implementSystemDictionary
	" Private - Implement support for SystemDictionary. "


	self	define: #mutable struct: #(StSystemDictionary #(
			#(dict 'Dict{String,Any}')
			#(state String)
			#(useManager Bool)
		)) inheriting: #StObject.
	self assign: 'StSystemDictionary()' to: 'StSystemDictionary(Dict(),"boot",false)'; cr.
	self implementClass: 'io:IO' api: #(
		'Base.show(io::IO,st::StSystemDictionary) = print(io,"smalltalk")'
		'Base.show(io::IO,o::StObject) = print(io, tostring(o))'
	).
	self implementClass: #Any api: #(
		'tostring(o::Any) = to_String(smalltalk,o)'
	).
	self implementClass: #StSystemDictionary api: #(
		state
		#(state: #(aState))
		'isNil_(self::StSystemDictionary, aValue::Nothing) = true'
		'isNil_(self::StSystemDictionary, aValue::Any) = aValue === nil'
		'notNil_(self::StSystemDictionary, aValue::Nothing) = false'
		'notNil_(self::StSystemDictionary, aValue::Any) = aValue !!== nil'
		'classOf_(self::StSystemDictionary, aValue) = klass(aValue)'
		#(do:onError: #(aBlock errorBlock))
		'basicAt_(self::StSystemDictionary, key) = basic_at_or_(self, self, key, nil)'
		#(basic:at:or: #(object key default))
		#(basic:at:ifAbsent: #(object key aBlock))
		'basic_at_(self::StSystemDictionary, object, key) = basic_at_or_(self, object, key, nil)'
		#(basic:at:put: #(object key value))
		#(basic:delete: #(object key))
		#(ifUndefined:do: #(object aBlock))
		#(times:repeat: #(count aBlock))
		#(orNil: #(object))
		#(if:nil:notNil: #(anObject block1 block2))
		#(if:true:false: #(aBoolean blk1 blk2))
		#(while:do: #(condition aBlock))
		#(whileNot:do: #(condition aBlock))
		#(createType #('aName::Symbol' superName))
	).
	self	defineClass: #(StObject_class StClass #(S8Object_class #( #(dict 'Dict{String,Any}') )))
		implementing: #( "empty api" ).
	self implementClass: #StSystemDictionary api: #(
		#(mapClassName #(className category 'native=nil' 'superclass=nil'))
		#(setVarNames #(aClass iVarNames aValue))
		#(setClass #(aClass spec))
		#(fnInSpec #(spec))
		nativeClasses
		classes computeSubclasses
		#(subclasses #(aClass))
		#(allSubclassesDo #(aClass aBlock))
		#(bindRoots #('ignored=nil'))
		#(isArray:equalTo: #(table1 table2))
		#(object:or: #(receiver option))
		#(isObject:equalTo: #(receiver anObject))
		#(object:comma: #(receiver anObject))
		'object_comma_(self::StSystemDictionary, receiver::Symbol, anObject) = object_comma_(self, string(receiver), anObject)'
		#(boolean:and: #(receiver option))
		#(boolean:or: #(receiver option))
		#(booleanNot: #(receiver))
		#(asCharacter: #(receiver))
		'to_String(self::StSystemDictionary, receiver::Nothing) = "nil"'
		'to_String(self::StSystemDictionary, receiver::String) = receiver'
		'to_String(self::StSystemDictionary, receiver::Number) = string(receiver)'
		#(to_String #(receiver))
		#(printString: #(receiver))
		#(asString: #(receiver))
		#(mustBeNumber: #(receiver))
		#(math:max: #(receiver aNumber))
		#(math:min: #(receiver aNumber))
		#(addClass #(className superclass iVarNames 'category=nil'))
		#(renameClass #(aClass aString))
		#(removeClass #(aClass))
		#(metaVarNames #(aMetaClass iVarNames))
		#(smDo #(block))
		#(classChanged #(aBehavior))
		#(classShapeChanged #(aBehavior))
		#(aboutToCreateClass #(className superclass iVarNames category))
		#(aboutToChangeClass #(aClass superclass iVarNames category))
		#(methodBoundTo #(method klass))
		#(aboutToBindTo #(method klass))
		#(aboutToRemoveFrom #(method klass))
		#(methodRemovedFrom #(method klass))
		#(objectNotFound #(object indexOrKey collection))
		#(whoImplements #( klass jsSelector))
		#(superImplementor #(klass jsSelector))
		#(instVarAt #(object ivName 'default=nil'))
		#(addMethod #(jsSelector method klass 'stSelector=nil'))
		#(bind #(klass stSelector jsSelector jsFunction category stSource 'nativeSource=nil' 'stamp=nil'))
		#(removeNativeFunction:for: #(fn aClass))
		#(removeMethod #(aClass aMethod))
		#(newMethod #(currentClass selector fn source 'nativeSource=nil' 'stamp=nil'))
"OBSOLETE	#(method #(spec))"
		#(readJSObject #(js))
		#(basen #(n b))
		#(globalsDo #(action))
		#(launchImage #('imageName=nil'))
		#(is:kindOf: #(anObject aClass))
		'responds_to_(self::StSystemDictionary, anObject::Nothing, aSelector)=false'
		#(responds:to: #(anObject aSelector))
		#(handleOf #(anObject))
		'handleOf_or_(self::StSystemDictionary, anObject::Nothing, default)=default'
		#(handleOf:or: #(anObject default))
		newHash
		stObject
		#(debugBind #(klass stSelector jsSelector jsFunction category stSource 'nativeSource=nil' 'stamp=nil'))
		#(debugAddClass #(className superclass iVarNames category))
		toString
		'quit() = Base.exit()'
		quit
		version
		#(folderContents #(fileName))
		httpSupport
		#(httpContents #(fileName))
		#(fileContents #(fileName))
	).
	self	bind: #fileWrite
		arguments: #('fileName::String' contents mode)
		doing: [ self perform: #fileWriteString ].
	self implementClass: #StSystemDictionary api: #(
		#(fileIn #(fileName))
		#(emmit #(fileName))
		#(fileInJS #(fileName))
		#(exportCategory #(aString))
		saveImage
		#(saveImageAs #(fileName))
		#(isFileOlderThan #(fileName referenceFileName))
		startup
		niy #(niy: #(msg))
		linkNatives
		bindCore
		#(invalidMessage: #(msg))
		obsolete
		#(linkNative:to: #(instance aClass))
		#(newConstructor #(superC))
		#(will:be: #(anObject className))
		#(newArray #('object=[]'))
		#(newArrayOfSize #(size))
		#(arrayAdapt #(array cookie))
		#(arrayPack #(array))
		#(become:a: #(anObject aClass))
		#(setProtoOf:to: #(nativeObject proto))
		#(klass #(spec))
		globalScope
		#(__f__ #(source))
		#(__b__ #(sources locals))
		stacktrace
		datesSupport
		#(block:valueWithPossibleArguments: #(block args))
		#(block_argumentCount #(block))
		#(pool_key #(h k))
		#(pool:at:put: #(pool aKey aValue))
		#(pool:removeKey: #(pool aKey))
		#(pool:at:ifAbsent: #(pool aKey aBlock))
		#(pool:includesKey: #(pool aKey))
		#(pool_keys #(pool))
		#(string_toNumber #(aString))
		#(string:copyFrom:to: #(aString fromIndex toIndex))
	);cr. ! !

! JuliaBoot methodsFor: #encoding !
define: aName asInstanceOf: clsName
	" Private - Return the encoding for defining a variable named aName. "

	self assign: aName to: #St ,clsName ,'()'; cr ! !

! JuliaBoot methodsFor: #encoding !
implementClass: typeName api: functions
	" Private - API functions. "

	functions do: [:spec|
		(spec isString and: [spec includes: $=])
		ifTrue: [ self nextPutAll: spec; cr ]
		ifFalse: [ | mySpec |
			mySpec := spec.
			mySpec isString ifTrue: [ mySpec := Array with: mySpec with: #() ].
			(mySpec isArray and: [mySpec last isArray]) ifTrue: [
				mySpec := Array
					with: mySpec first
					with: (Array with: 'self::' ,typeName) ,mySpec last
			] ifFalse: [ self error: 'Unspected spec - ' ,spec asLiteral ].
			self bind: typeName api: mySpec
		]
	]! !

! JuliaBoot methodsFor: #encoding !
defineClass: header implementing: functions
	" Private - jlClass definition. "

	header isString ifTrue: [
		^self	defineClass: (Array with: header with: #() with: #())
			implementing: functions
	].
	header isArray ifFalse: [
		^self error: 'Invalid/corrupt class spec - ' ,header asLiteral
	].
	header size = 2 ifTrue: [
		^self	defineClass: header ,(Array with: #())
			implementing: functions
	].
	header size = 3 ifFalse: [
		^self error: 'Invalid class spec size - ' ,header asLiteral
	].

	self define: #abstract type: header first inheriting: header second.
	header third notEmpty ifTrue: [
		self define: #mutable struct: header third inheriting: header first
	].

	self implementClass: header first api: functions! !

! JuliaBoot methodsFor: #api !
defineStructFields: spec
	" Private - Define fields in spec. "

	spec do: [:tuple|
		self nextPutAll: tuple first; nextPutAll: '::'; nextPutAll: tuple last
	] separatedBy: [ self cr ]! !

! JuliaBoot methodsFor: #api !
define: kind struct: spec inheriting: parentSpec
	" Private - Define a new struct. "

	| def fields tags |
	spec isArray ifFalse: [
		^self error: 'Invalid/corrupt struct spec - ' ,spec asLiteral
	].
	spec size = 2 ifFalse: [
		^self error: 'Invalid struct spec size - ' ,spec asLiteral
	].
	def := spec first ,' <: ' ,parentSpec.
	def := (kind isEmpty ifTrue: [ 'struct ' ] ifFalse: [ kind ,' struct ' ]) ,def.
	tags := Array with: def with: #end.
	self between: tags do: [ self defineStructFields: spec last ] indent: true; cr ! !

! JuliaBoot methodsFor: #api !
define: kind type: typeName inheriting: parentSpec
	" Private - Define a new type. "

	| tags |
	tags := Array with: kind ,' type ' with: #end.
	self between: tags do: [
		self nextPutAll: typeName; space.
		parentSpec notEmpty ifTrue: [
			self nextPutAll: #<:; space.
			self nextPutAll: parentSpec; space.
		]] indent: false; cr ! !

! JuliaBoot methodsFor: #api !
api: typeName selector: functionName
	" Private - Returns the selector message to find implementation of function of typeName. "

	| input output |
	(functionName includes: $.) ifFalse: [
		^functionName asSelector ,typeName
	].
	input := (typeName ,$_ ,functionName asSelector) readStream.
	output := '' writeStream.
	[input atEnd] whileFalse: [
		output nextPutAll: (input upTo: $.).
		input atEnd ifFalse: [ output nextPut: $_ ].
	].
	^output contents asLocalName
		replace: $!! with: $8 ! !

! JuliaBoot methodsFor: #api !
bind: typeName api: spec
	" Private - Encoding for basic message implementation. "

	| rcv selector action |
	spec isString ifTrue: [ ^self bind: typeName api: (Array with: spec with: #()) ].
	spec last isArray ifFalse: [
		^self error: 'Invalid/corrupt API spec ' ,spec asLiteral
	].
	selector := self api: typeName selector: spec first.
	action := [ rcv perform: selector ].
	rcv := (self respondsTo: selector) ifTrue: [ self ] ifFalse: [
		(self class respondsTo: selector) ifTrue: [ self class ] ifFalse: [
			" self error: 'Missing API implementation for ' ,selector asLiteral "
			action := [ self comment: '*ToDo* - API ' ,selector asLiteral ,' Not Implemented Yet.' ]
		]
	].
	self bind: spec first arguments: spec last doing: action ! !

! JuliaBoot methodsFor: #api !
bind: typeName function: functionName arguments: args sending: selector
	" Private - Encoding of custom API function implementation. "

	self	bind: functionName
		arguments: (Array with: 'self::',typeName) ,args
		doing: [ self perform: selector ] ! !

! JuliaBoot methodsFor: #asm !
bind: selector arguments: args doing: implementation comment: aComment
	" Private - Encoding for internal method implementation. "

	self nextPutAll: #function;space.
	self nextPutAll: ((selector startsWith: $>)
		ifTrue: [ selector] ifFalse: [ selector asSelector ]).
	self pars: [ self nextPutArguments: args ].
	self between: #('' end) do: implementation indent: true comment: aComment.
	self cr! !

! JuliaBoot methodsFor: #encoding !
sendMessage: aMessage
	" Private - Return the encoding for sending aMessage. "

	| rcv args |
	rcv := aMessage receiver.
	(rcv isString or: [rcv isValuable])
	ifTrue: [ args := (Array with: rcv) ,aMessage arguments ]
	ifFalse: [
		rcv isMessage ifFalse: [ self error: 'Receiver must be aMessage or aString' ].
		self pars: [ self sendMessage: rcv ].
		args := (Array with: rcv) ,aMessage arguments
	].
	self nextPutAll: aMessage selector asSelector.
	self pars: [ self sendMessageArguments: args ] ! !

! JuliaBoot methodsFor: #asm !
import: what
	" Private - Encoding for import declaration. "

	self nextPutAll: #import; space.
	self nextPutAll: what ! !

! JuliaBoot methodsFor: #asm !
expr: expr1 is: expr2
	" Private - Encoding for <: expression. "

	self expr: expr1 op: #<: with: expr2! !

! JuliaBoot methodsFor: #asm !
expr: expr1 isa: expr2
	" Private - Encoding for isa expression. "

	self expr: expr1 op: #isa with: expr2! !

! JuliaBoot methodsFor: #asm !
expr: expr1 eqgt: expr2
	" Private - Encoding for => expression. "

	self expr: expr1 op: #=> with: expr2! !

! JuliaBoot methodsFor: #asm !
expr: expr1 gt: expr2
	" Private - Encoding for > expression. "

	self expr: expr1 op: #> with: expr2! !

! JuliaBoot methodsFor: #asm !
expr: expr1 lt: expr2
	" Private - Encoding for < expression. "

	self expr: expr1 op: #< with: expr2! !

! JuliaBoot methodsFor: #asm !
expr: expr1 dashgt: expr2
	" Private - Encoding for -> expression. "

	self expr: expr1 op: #-> with: expr2! !

! JuliaBoot methodsFor: #asm !
expr: expr1 return: expr2
	" Private - Encoding for %% return expression. "

	self expr: expr1 and: [self return: expr2] ! !

! JuliaBoot methodsFor: #asm !
expr: expr1 and: expr2 return: expr3
	" Private - Encoding for &&..&& expression. "

	self expr: [self expr: expr1 and: expr2] return: expr3 ! !

! JuliaBoot methodsFor: #asm !
if: condition do: trueBlock else: falseBlock
	" Private - Encoding for conditional. "

	self if: condition; space.
	self between: nil do: trueBlock indent: true.
	falseBlock notNil ifTrue: [
		self nextPutAll: self elseOP.
		self between: nil do: falseBlock indent: true.
	].
	self nextPutAll: #end;cr! !

! JuliaBoot methodsFor: #asm-macros !
succ: arg
	" Private - Encoding for +1 expression. "

	self expr: arg op: #+ with: #1 ! !

! JuliaBoot methodsFor: #asm-macros !
prev: arg
	" Private - Encoding for -1 expression. "

	self expr: arg op: #- with: #1 ! !

! JuliaBoot methodsFor: #asm !
if: condition return: expression
	" Private - Encoding for conditional expression return. "

	self if: condition do: [self return: expression]! !

! JuliaBoot methodsFor: #asm !
valueIf: expr1 do: expr2
	" Private - Encoding for (valueIf) expression. "

	self expr: expr1 op: $? with: expr2 ! !

! JuliaBoot methodsFor: #asm !
valueIf: expr1 do: expr2 else: expr3
	" Private - Encoding for (valueIf:else:) expression. "

	self expr: [self valueIf: expr1 do: expr2] op: $: with: expr3 ! !

! JuliaBoot methodsFor: #asm-constants !
blockOPs
	" Private - Encoding for expression block OPs "

	^#('' end) ! !

! JuliaBoot methodsFor: #asm-macros !
object: src at: index
	" Private - Encoding for brackets indexing. "

	self nextPutAll: src.
	self between: '[]' do: index indent: false! !

! JuliaBoot methodsFor: #asm !
try: block catch: varName doing: exBlock
	" Private - Encoding for try/catch expression. "

	self between: #(try catch) do: block indent: true.
	varName notNil ifTrue: [
		self space; nextPutAll: varName
	].
	self between: self expressionsOPs do: exBlock indent: true! !

! JuliaBoot methodsFor: #asm !
try: block catch: exBlock
	" Private - Encoding for try/catch expression. "

	self try: block catch: nil doing: exBlock! !

! JuliaBoot methodsFor: #asm-macros !
concat: array
	" Private - Encoding to concatenate expressions inlined. "

	array do: [:each| self nextPutAll: each ] separatedBy: [self nextPutAll: $*] ! !

! JuliaBoot methodsFor: #asm-macros !
sum: array
	" Private - Encoding to concatenate expressions inlined. "

	array do: [:each| self nextPutAll: each ] separatedBy: [self nextPutAll: $+] ! !

! JuliaBoot methodsFor: #asm-constants !
conditionPars: expression
	" Private - Encoding for condition expression(or nil) "

	^nil ! !

! JuliaBoot methodsFor: #asm-constants !
opPars: expression
	" Private - Encoding for expression in operation(or nil) "

	^nil ! !

! JuliaBoot methodsFor: #asm-macros !
symbol: aName
	" Private - Encoding for returning a Symbol instance."

	^self call: #Symbol with: aName ! !

! JuliaBoot methodsFor: #asm-macros !
typeof: anObject
	" Private - Encoding for returning the type of anObject instance."

	^self call: #typeof with: anObject ! !

! JuliaBoot methodsFor: #asm-macros !
hasfield: args
	" Private - Encoding for #hasfield expression. "

	(args isArray and: [args size = 2]) ifTrue: [
		^self call: #hasfield withArguments: args
	].
	^self error: 'Invalid arguments for #hasfield' ! !

! JuliaBoot methodsFor: #asm-macros !
hasfieldTypeof: src with: key
	" Private - Encoding for #hasfield(typeof(src),key) expression. "

	^self hasfield: (Array
		with: [self typeof: src]
		with: key) ! !

! JuliaBoot methodsFor: #asm-macros !
from: from to: to
	" Private - Encoding for from:to expression. "

	self	nextPutAll: from;
		nextPutAll: #:;
		nextPutAll: to ! !

! JuliaBoot methodsFor: #asm-constants !
whileDoOPs
	" Private - Return the OPs for while-do expression. "

	^Array with: #while with: '' ! !

! JuliaBoot methodsFor: #asm-constants !
forDoOPs
	" Private - Return the OPs for for-do expression. "

	^Array with: #for with: '' ! !

! JuliaBoot methodsFor: #asm-constants !
forDoPars
	" Private - Return the parentesis for for-do expression. "

	^nil ! !

! JuliaBoot methodsFor: 'asm-constants' !
forInDoPars
	" Private - Return the parentesis for for-in-do expression. "

	^nil! !

! JuliaBoot methodsFor: #asm-constants !
forInDoOPs
	" Private - Return the OPs for for-in-do expression. "

	^Array with: #for with: #in with: '' ! !

! BootSector methodsFor: #asm-constants !
forInDoPars
	" Private - Return the parentesis for for-in-do expression. "

	^nil! !

! JuliaBoot methodsFor: #asm-macros !
for: varName from: from to: to do: aBlock
	" Private - Encoding for #hasfield(typeof(src),key) expression. "

	^self for: [self assign: varName
		to: [self from: from to: to]]
		do: aBlock ! !

! JuliaBoot methodsFor: #asm-constants !
fieldOP
	" Private - Return the OP for tuple separator. "

	^#=> ! !

! JuliaBoot methodsFor: #asm-constants !
objectOPs
	" Private - Return the OPs for literal object definition. "

	^'()' ! !

! JuliaBoot methodsFor: #asm-macros !
anyDict: keyType object: array
	" Private - Return the encoding for literal dictionary definition. "

	self	nextPutAll: 'Dict{';
		nextPutAll: keyType;
		nextPutAll: ',Any}';
		object: array ! !

! JuliaBoot methodsFor: #asm-macros !
on: expr with: varName do: aBlock
	" Private - Encoding for 'expr do var blk' expression. "

	self nextPutAll: expr.
	self space; nextPutAll: #do.
	self space; nextPutAll: varName.
	self between: self expressionsOPs do: aBlock indent: true! !



! JuliaBoot methodsFor: #api !
klassAny
	" Private - Implementation of api function "

	self assign: #t to: [self typeof: #aValue];cr.
	self expr: [self hasfield: #(t :dict)]
		and: [self call: #haskey withArguments: #('aValue.dict' '"\$klass"')]
		return: 'aValue.dict["\$klass"]';cr.
	self expr: [self expr: #t eqeqeq: #String]
		return: 'smalltalk.String';cr.
	self expr: [self expr: #t eqeqeq: #Bool] 
		return: 'smalltalk.Boolean';cr.
	self expr: [self expr: #t is: #Function] 
		return: 'smalltalk.BlockClosure';cr.
	self expr: [self expr: #t is: #Number] 
		return: 'smalltalk.Number';cr.
	self expr: [self expr: #t eqeqeq: #StSystemDictionary] 
		return: 'smalltalk.Smalltalk';cr.
	self return: #nil ! !

! JuliaBoot methodsFor: #api !
stObjectBaseGetproperty
	" Private - Implementation of api function "

	self if: [self hasfieldTypeof: #self with: #aSymbol]
		return: [self call: #getfield withArguments: #(self aSymbol)];cr.
	self if: [self call: #haskey withArguments: (Array
			with: 'self.dict'
			with: [self call: #string with: #aSymbol])]
		return: [self call: #get withArguments: (Array
			with: 'self.dict'
			with: [self call: #string with: #aSymbol]
			with: #nil)];cr.
	self return: [self call: #getfield withArguments: #(self aSymbol)] ! !

! JuliaBoot methodsFor: #api !
stObjectBaseSetproperty8
	" Private - Implementation of api function "

	self comment: 'println("setproperty!! [",string(aSymbol),"]=",object)'.
	self if: [self hasfieldTypeof: #self with: #aSymbol]
		do: [ self call: 'setfield!!' withArguments: #(self aSymbol object) ]
		else: [ self assign: 'self.dict[string(aSymbol)]' to: #object] ! !

! JuliaBoot methodsFor: #api !
stObjectBaseSetindex8
	" Private - Implementation of api function "

	self comment: 'println("setindex!! [",string(aSymbol),"]=",object)'.
	self assign: 'self.dict[string(aSymbol)]' to: #object ! !

! JuliaBoot methodsFor: #api !
stObjectBaseGetindexName 
	" Private - Implementation of api function "

	self return: [self call: #get withArguments: (Array
		with: 'self.dict'
		with: [self call: #string with: #aName]
		with: #nil)] ! !

! JuliaBoot methodsFor: #api !
stObjectBaseGetindexInt
	" Private - Implementation of api function "

	self if: [self call: #haskey withArguments: #('self.dict' '"@handle"')] do: [
		self assign: #h to: 'self.dict["@handle"]';cr.
		self return: [self call: 'Base.getindex' withArguments: #(h ind)]
	];cr.
	self return: [self call: 'Base.getindex' withArguments: #(self ind)] ! !

! JuliaBoot methodsFor: #api !
handleOrStObject
	" Private - Implementation of api function "

	self assign: #result to: [self call: #handle with: #self];cr.
	self if: [self send: #isNil: to: #smalltalk with: #result] do: [
		self assign: 'self["@handle"]' to: #forcedHandle;cr.
		self return: #forcedHandle
	];cr.
	self return: #result ! !

! JuliaBoot methodsFor: #api !
basicNewInstance_StBehavior
	" Private - Implementation of api function "

	self assign: #result to: [self call: 'self.dict["\$constructor"]' with: #fields];cr.
	self assign: 'result.dict["\$klass"]' to: #self;cr.
	self return: #result ! !

! JuliaBoot methodsFor: #api !
nativeNew_StBehavior
	" Private - Implementation of api function "

	self assign: #fields to: [self
		valueIf: [self send: #isNil: to: #smalltalk with: #handle]
		do: [self call: 'Dict{String,Any}']
		else: [self call: 'Dict{String,Any}' with: [self expr: '"@handle"' eqgt: #handle]]
	];cr.
	self return: [self send: #basicNewInstance: to: #self with: #fields] ! !

! JuliaBoot methodsFor: #api !
_gt_gtStBehavior
	" Private - Implementation of api function "

	self return: [self send: #compiledMethodAt: to: #self with: #selector] ! !

! JuliaBoot methodsFor: #api !
_gt_gt_gtStBehavior
	" Private - Implementation of api function "

	self return: [self send: #>>> to: #self with: #selector] ! !

! JuliaBoot methodsFor: #api !
stGlobalScopeBaseGetproperty
	" Private - Implementation of api function "

	self try: [
		self return: [self call: #eval with: #aSymbol]
	] catch: #ex doing: [
		self call: #println withArguments: #('"StGlobalScope getproperty["' aSymbol '"] error="' ex);cr.
		self return: #nil
	] ! !

! JuliaBoot methodsFor: #api !
stGlobalScopeBaseGetindex
	" Private - Implementation of api function "

	self return: [ self send: 'Base.getproperty' with: [self symbol: #aName] ] ! !

! JuliaBoot methodsFor: #api !
stGlobalScopeBaseSetproperty8
	" Private - Implementation of api function "

	self assign: #key to: [self call: #string with: #aSymbol];cr.
	self try: [
		self assign: 'global cookie' to: #object;cr.
		self assign: #code to: [self concat: (Array
			with: '"global "'
			with: [self call: #string with: #aSymbol]
			with: '"=cookie"')
		];cr.
		self call: #eval with: [self call: 'Meta.parse' with: #code];cr.
		self return: #object
	] catch: #ex doing: [
		self call: #println withArguments: #('"StGlobalScope[set] error "' ex);cr.
		self return: #nil
	] ! !

! JuliaBoot methodsFor: #api !
stGlobalScopeBaseSetindex8
	" Private - Implementation of api function "

	self return: [self send: 'Base.setproperty!!' to: #self withArguments: (Array
		with: [self symbol: #aSymbol]
		with: #object)] ! !

! JuliaBoot methodsFor: #api !
stateStSystemDictionary
	" Private - Implementation of api function "

	self return: 'self.state' ! !

! JuliaBoot methodsFor: #api !
state_StSystemDictionary
	" Private - Implementation of api function "

	self assign: 'self.state' to: #aState ! !

! JuliaBoot methodsFor: #api !
do_onError_StSystemDictionary
	" Private - Implementation of api function "

	self send: #invalidMessage: with: '"do:onError:"' ! !

! JuliaBoot methodsFor: #api !
basic_at_or_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #skey to: [self symbol: #key];cr.
	self if: [self not: [self hasfieldTypeof: #object with: #:dict]] do: [
			self if: [self expr: [self typeof: #object] noteq: #S8GlobalScope] do: [
				self if: [self not: [self call: #hasproperty withArguments: #(object skey)]]
					return: #default
			];cr.
			self try: [ self return: [self
				call: #getproperty
				withArguments: #(object skey)]
			] catch: #ex doing: [ self return: #default ]
	];cr.
	self assign: #keyString to: [self call: #string with: #key];cr.
	self expr: [self call: #haskey withArguments: #('object.dict' keyString)]
		return: 'object.dict[keyString]';cr.
	self return: #default ! !

! JuliaBoot methodsFor: #api !
basic_at_ifAbsent_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #sentinel to: [self call: #newHash with: #self];cr.
	self assign: #result to: [self
		send: #basic:at:or:
		withArguments: #(object key sentinel)
	];cr.
	self return: [self expr: #result eqeq: [
		self valueIf: #sentinel do: [
			self call: #evaluate with: #aBlock
			] else: #result
	]]! !

! JuliaBoot methodsFor: #api !
basic_at_put_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #skey to: [self symbol: #key];cr.
	self if: [self not: [self hasfieldTypeof: #object with: #:dict]] do: [
		self call: 'setproperty!!' withArguments: #(object skey value);cr.
		self return: #value
	];cr.
	self assign: #keyString to: [self call: #string with: #key];cr.
	self assign: 'object.dict[keyString]' to: #value;cr.
	self return: #value ! !

! JuliaBoot methodsFor: #api !
basic_delete_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self not:  [self hasfieldTypeof: #object with: #:dict]] do: [
		self call: 'setproperty!!' withArguments: #(object skey #nil).
	] else: [
		self call: 'delete!!' withArguments: (Array
			with: 'object.dict'
			with: [self call: #string with: #key])
	] ! !

! JuliaBoot methodsFor: #api !
ifUndefined_do_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self send: #isNil: to: #self with: #object] and: [self return: #nil];cr.
	self return: #object ! !

! JuliaBoot methodsFor: #api !
times_repeat_StSystemDictionary
	" Private - Implementation of api function "

	self for: #i from: #1 to: #count do: [
		self send: #evaluateWith: to: #aBlock with: #i
	];cr.
	self return: #count ! !

! JuliaBoot methodsFor: #api !
orNil_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self send: #isNil: to: #self with: #object] and: [self return: #nil];cr.
	self return: #object ! !

! JuliaBoot methodsFor: #api !
if_nil_notNil_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self send: #isNil: to: #self with: #anObject]
		and: [self return: [self send: #evaluate to: #block1]];cr.
	self return: [self send: #evaluate to: #block2] ! !

! JuliaBoot methodsFor: #api !
if_true_false_StSystemDictionary
	" Private - Implementation of api function "

	self if: #aBoolean do: [
		self expr: [self send: #notNil: to: #self with: #blk1]
			and: [self return: [self call: #blk1]]
	] else: [
		self expr: [self send: #notNil: to: #self with: #blk2]
			and: [self return: [self call: #blk2]]
	];cr.
	self return: #nil ! !

! JuliaBoot methodsFor: #api !
while_do_StSystemDictionary
	" Private - Implementation of api function "

	self while: [self call: #condition] do: [
		self expr: [self send: #notNil: to: #self with: #aBlock]
			and: [self send: #evaluate to: #aBlock]
	];cr.
	self return: #nil ! !

! JuliaBoot methodsFor: #api !
whileNot_do_StSystemDictionary
	" Private - Implementation of api function "

	self while: [self not: [self call: #condition]] do: [
		self expr: [self send: #notNil: to: #self with: #aBlock]
			and: [self send: #evaluate to: #aBlock]
	];cr.
	self return: #nil ! !

! JuliaBoot methodsFor: #api !
createTypeStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'println("createType ",aName,"<:",superName)'.
	self assign: #stName to: [self concat: (Array with: '"St"' with: [self call: #String with: #aName])];cr.
	self assign: #s8Name to: [self concat: (Array with: '"S8"' with: [self call: #String with: #aName])];cr.
	self try: [
		self call: #eval with: [self call: 'Meta.parse' with: #stName];cr.
		self comment: 'println("FOUND " ,stName)'
	] catch: [
		self if: [self expr: [self call: #length with: #superName] gt: #0] do: [
			self assign: #superName to: [self concat: #('" <: St"' superName)]
		];cr.
		self assign: #code to: [self concat: #('"abstract type "' stName superName '" end"')];cr.
		self comment: 'println(code)'.
		self call: #eval with: [self call: 'Meta.parse' with: #code];cr.
	];cr.
	self try: [
		self call: #eval with: [self call: 'Meta.parse' with: #s8Name];cr.
		self comment: 'println("FOUND " ,s8Name)'
	] catch: [
		self assign: #code to: [self concat: #('"mutable struct "' s8Name '" <: "' stName)];cr.
		self assign: #code to: [self concat: #(code '"\n	dict::Dict{String,Any}"')];cr.
		self assign: #code to: [self concat: #(code '"\nend"')];cr.
		self comment: 'println(code)'.
		self call: #eval with: [self call: 'Meta.parse' with: #code];cr.
	];cr.
	self return: [self call: #eval with: [self call: 'Meta.parse' with: #s8Name]] ! !

! JuliaBoot methodsFor: #api !
mapClassNameStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'println("mapClassName ",className ) #," <: ",superclass)'.
	self assign: #mcName to: [self symbol: [self concat: #(className '"_class"')]];cr.
	self assign: #mcDict to: [self anyDict: #String object:
		'"\$klassName"' -> [self call: #string with: #mcName]
		,('"\$className"' -> [self call: #string with: [self concat: #(className '" class"')]])
		,('"\$klass"' -> 'self["Metaclass"]')
		,('"@subclasses"' -> #nil)
		,('"\$methods"' -> [self call: 'Dict{String,Any}'])
	];cr.
	self if: [self send: #isNil: with: #superclass] do: [
		self assign: #superCls to: '""';cr.
		self assign: 'mcDict["\$superclass"]' to: #StClass;cr.
		self assign: 'mcDict["\$constructor"]' to: #S8Object_class;cr.
	] else: [
		self assign: 'superclass["@subclasses"]' to: #nil;cr.
		self assign: #superCls to: [self send: #basic:at: withArguments: #(superclass '"\$className"')];cr.
		self assign: #smc to: [self send: #classOf: with: #superclass];cr.
		self assign: 'mcDict["\$superclass"]' to: #smc;cr.
		self assign: #superMc to: [self send: #basic:at: withArguments: #(smc '"\$klassName"')];cr.
		self assign: #mcType to: [self send: #createType withArguments: #(mcName superMc)];cr.
		self assign: 'mcDict["\$constructor"]' to: #mcType
	];cr.
	self assign: #mClass to: [self call: #S8Metaclass with: #mcDict];cr.
	self assign: #clsName to: [self symbol: #className];cr.
	self assign: #clsType to: [self send: #createType withArguments: #(clsName superCls)];cr.
	self assign: #clsDict to: [self anyDict: #String object:
		'"\$klass"' -> #mClass
		,('"\$constructor"' -> #clsType)
		,('"\$className"' -> [self call: #string with: #clsName])
		,('"\$category"' -> #category)
		,('"\$superclass"' -> #superclass)
		,('"@subclasses"' -> #nil)
		,('"\$comment"' -> #nil)
		,('"\$methods"' -> [self call: 'Dict{String,Any}'])
	];cr.
	self if: [self send: #notNil: with: #native] do: [
		self assign: 'clsDict["\$native"]' to: #native
	];cr.
	self assign: #cls to: [self call: 'Base.invokelatest' withArguments: #(basicNewInstance_ mClass clsDict)];cr.
	self assign: 'mClass["@instanceClass"]' to: #cls;cr.
	self assign: 'cls["@instanceClass"]' to: #cls;cr.
	self assign: 'self.dict[string(clsName)]' to: #cls;cr.
	self comment: 'println("Class created ",string(clsName))'.
	self return: #cls ! !

! JuliaBoot methodsFor: #api !
setVarNamesStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'println("setVarNames ",iVarNames)'.
	self assign: #hArray to: [self send: #handleOf:or: withArguments: #(iVarNames '[]')];cr.
	self send: #basic:at:put: withArguments: (Array
		with: #aClass with: '"\$iVarNames"'
		with: [self send: #newArray with: #hArray]);cr.
	self for: #iv in: #hArray do: [
		self send: #basic:at:put: withArguments: (Array
			with: #aClass
			with: [self concat: #('"@"' iv)]
			with: #aValue)
	] ! !

! JuliaBoot methodsFor: #api !
setClassStSystemDictionary
	" Private - Implementation of api function "

	self send: #obsolete ! !

! JuliaBoot methodsFor: #api !
fnInSpecStSystemDictionary
	" Private - Implementation of api function "

	self send: #obsolete ! !

! JuliaBoot methodsFor: #api !
nativeClassesStSystemDictionary
	" Private - Implementation of api function "

	self assign: #keys to: [self call: #filter withArguments: (Array
		with: [self expr: #k dashgt: [self call: #isa withArguments: #('self[k]' StBehavior)]]
		with: [self send: 'Base.keys' to: 'self.dict'])
	];cr.
	self assign: #result to: [self emptyArray];cr.
	self for: #k in: #keys do: [
		self send: 'Base.push!!' to: #result with: 'self[k]'
	];cr.
	self return: #result ! !

! JuliaBoot methodsFor: #api !
classesStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #newArray with: [self send: #nativeClasses]] ! !

! JuliaBoot methodsFor: #api !
computeSubclassesStSystemDictionary
	" Private - Implementation of api function "

	self assign: #all to: [self send: #handleOf with: [self send: #classes]];cr.
	self for: #aClass in: #all do: [
		self assign: 'aClass["@subclasses"]' to: [self send: #newArray with: [self emptyArray]];cr.
		self assign: 'class(aClass)["@subclasses"]' to: [self send: #newArray with: [self emptyArray]]
	];cr.
	self for: #aClass in: #all do: [
		self assign: #parent to: 'aClass["\$superclass"]';cr.
		self if: [self send: #notNil: with: #parent] do: [
			self send: #add: to: 'parent["@subclasses"]' with: #aClass;cr.
			self send: #add: to: 'class(parent)["@subclasses"]' with: [self call: #class with: #aClass]
		]
	] ! !

! JuliaBoot methodsFor: #api !
subclassesStSystemDictionary
	" Private - Implementation of api function "

	self assign: #result to: 'aClass["@subclasses"]';cr.
	self if: [self send: #isNil: with: #result] do: [
		self send: #computeSubclasses;cr.
		self return: 'aClass["@subclasses"]'
	];cr.
	self return: #result ! !

! JuliaBoot methodsFor: #api !
allSubclassesDoStSystemDictionary
	" Private - Implementation of api function "

	self	send: #do:
		to: [self send: #subclasses to: #aClass]
		with: [self implementFunction: '' arguments: #(cls) doing: [
			self send: #evaluateWith: to: #aBlock with: #cls;cr.
			self send: #allSubclassesDo withArguments: #(cls aBlock)
		]];cr.
	self return: #self ! !

! JuliaBoot methodsFor: #api !
bindRootsStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Private - Link the system roots to ensure proper delegation chains'.
	self send: #invalidMessage: with: '"bindRoots"' ! !

! JuliaBoot methodsFor: #api !
isArray_equalTo_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self expr: #table1 eqeq: #table2] and: [self return: #true];cr.
	self return: [self
		expr: [self send: #handleOf:or: withArguments: #(table1 #1)]
		eqeq: [self send: #handleOf:or: withArguments: #(table2 #2)]] ! !

! JuliaBoot methodsFor: #api !
object_or_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self send: #isNil: with: #receiver] and: [self return: #option];cr.
	self expr: [self expr: #receiver eqeqeq: #false] and: [self return: #option];cr.
	self return: #receiver ! !

! JuliaBoot methodsFor: #api !
isObject_equalTo_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self
		expr: #receiver eqeqeq: #anObject]
		and: [self return: #true];cr.
	self comment: 'h1 = handleOf(self,receiver)'.
	self comment: 'h2 = handleOf(self,anObject)'.
	self comment: 'notNil_(self,h1) && return h1==h2'.
	self comment: 'notNil_(self,h2) && return h1==h2'.
	self expr: [
		self send: #responds:to: withArguments: #(receiver '"="')]
		and: [self return: [self send: #= to: #receiver with: #anObject]];cr.
	self expr: [
		self send: #responds:to: withArguments: #(anObject '"="')]
		and: [self return: [self send: #= to: #anObject with: #receiver]];cr.
	self return: #false ! !

! JuliaBoot methodsFor: #api !
object_comma_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h1 to:[self send: #handleOf with: #receiver];cr.
	self if: [self expr: [self call: #typeof with: #h1] eqeq: #String] do: [
		self expr: [self send: #isNil: with: #anObject] and: [self return: #receiver];cr.
		self assign: #h2 to: [self send: #handleOf with: #anObject];cr.
		self if: [self expr: [self call: #typeof with: #h2] eqeq: #String]
			return: [self concat: #(h1 h2)];cr.
		self if: [self call: #isa withArguments: #(anObject StObject)]
			return: [self concat: (Array with: #h1
				with: [self send: #handleOf with: [self
					send: #to_String with: #anObject]])];cr.
		self return: [self concat: (Array with: #h1 with: [self call: #string with: #h2])]
	];cr.
	self expr: [
		self send: #isa to: #receiver with: #StObject]
		and: [self return: [self send: #, to: #receiver with: #anObject]];cr.
	self return: [self send: #object:comma: withArguments: (Array
		with: [self call: #string with: #h1]
		with: #anObject)] ! !

! JuliaBoot methodsFor: #api !
boolean_and_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self expr: #receiver noteq: #true] and: [self return: #false];cr.
	self expr: [self expr: [self call: #typeof with: #option] eqeq: #Bool] and: [self return: #option];cr.
	self return: [self expr: [self send: #evaluate to: #option] eqeq: #true] ! !

! JuliaBoot methodsFor: #api !
boolean_or_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self expr: #receiver eqeq: #true] and: [self return: #true];cr.
	self expr: [self expr: [self call: #typeof with: #option] eqeq: #Bool]
		and: [self return: #option];cr.
	self return: [self expr: [self send: #evaluate to: #option] eqeq: #true] ! !

! JuliaBoot methodsFor: #api !
booleanNot_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self expr: #receiver eqeq: #true] and: [self return: #false];cr.
	self expr: [self expr: #receiver eqeq: #false] and: [self return: #true];cr.
	self return: [self send: #not to: #receiver] ! !

! JuliaBoot methodsFor: #api !
asCharacter_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self send: #isa to: #receiver with: #Number]
		and: [self return: [self call: #string with: [self call: #Char with: #receiver]]];cr.
	self return: [self send: #asCharacter to: #receiver] ! !

! JuliaBoot methodsFor: #api !
to_StringStSystemDictionary
	" Private - Implementation of api function "

	self if: [self send: #isa to: #receiver with: #Bool] do: [
		self expr: #receiver and: [self return: '"true"'];cr.
		self return: '"false"'
	];cr.
	self expr: [self send: #isNil: with: #receiver] and: [self return: '"nil"'];cr.
	self if: [self send: #isa to: #receiver with: #StObject] do: [
		self expr: [self send: #isNil: with: [self send: #klass to: #receiver]]
			and: [self return: [self concat: (Array
				with: '"StObject("'
				with: [self call: #string with: 'receiver.dict']
				with: '")"')]];cr.
		self return: [self send: #toString to: #receiver]
	];cr.
	self return: [self send: #toString: to: 'smalltalk.Object' with: #receiver] ! !

! JuliaBoot methodsFor: #api !
printString_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self send: #responds:to: withArguments: #(receiver '"printString"')]
		and: [self return: [self send: #printString to: #receiver]];cr.
	self return: [self call: #string with: #receiver] ! !

! JuliaBoot methodsFor: #api !
asString_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self send: #responds:to: withArguments: #(receiver '"asString"')]
		and: [self return: [self send: #asString to: #receiver]];cr.
	self return: [self call: #string with: #receiver] ! !

! JuliaBoot methodsFor: #api !
mustBeNumber_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self send: #isa to: #receiver with: #Number]
		and: [self return: #receiver];cr.
	self return: [self send: #mustBeNumber to: #receiver] ! !

! JuliaBoot methodsFor: #api !
math_max_StSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #niy] ! !

! JuliaBoot methodsFor: #api !
math_min_StSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #niy] ! !

! JuliaBoot methodsFor: #api !
addClassStSystemDictionary
	" Private - Implementation of api function "

	self send: #debugAddClass withArguments: #(className superclass iVarNames category);cr.
	self assign: #aClass to: [self send: #basic:at:or: withArguments: #(self className #nil)];cr.
	self assign: #parent to: [self send: #basic:at:or: withArguments: #(aClass '"\$superclass"' #nil)];cr.
	self if: [self send: #isNil: with: #parent] do: [
		self assign: #parent to: #nil
	];cr.
	self if: [self send: #isNil: with: #iVarNames] do: [
		self assign: #iVarNames to: [self emptyArray]
	];cr.
	self if: [self send: #notNil: with: #aClass] do: [
		self if: [self expr: #superclass eqeq: #parent] do: [
			self if: [self send: #isArray:equalTo: withArguments: (Array
				with: [self send: #basic:at:or:
					withArguments: (Array
						with: #aClass
						with: '"\$iVarNames"'
						with: [self emptyArray])]
				with: #iVarNames)] do: [
				self if: [self expr: #category eqeq: [
					self send: #basic:at:
					withArguments: #(aClass '"\$category"')]
					] return: #aClass
			]
		];cr.
	self assign: #sameShape to: #false;cr.
	self send: #aboutToChangeClass withArguments: #(aClass superclass iVarNames category);cr.
	self if: [self expr: #parent noteqeq: #superclass] do: [
		self comment: 'move in hierarchy'.
		self comment: 'disconnect from parent'.
		self send: #basic:at:put: withArguments: #(parent '"@subclasses"' #nil);cr.
		self send: #basic:at:put: withArguments: (Array
			with: [self call: #class with: #parent]
			with: '"@subclasses"' with: #nil);cr.
		self comment: 'link to new parent'.
		self send: #basic:at:put: withArguments: #(aClass '"\$superclass"' superclass);cr.
		self assign: #aMeta to: [self call: #class with: #aClass];cr.
		self if: [self expr: [self send: #notNil: with: #aMeta]
			and: [self send: #notNil: with: #superclass]] do: [
			self assign: #msuper to: [self send: #basic:at:
				withArguments: #(aMeta '"\$superclass"')];cr.
			self send: #basic_at_put_ withArguments: #(msuper '"@subclasses"' #nil);cr.
			self send: #basic_at_put_ withArguments: (Array
				with: #aMeta with: '"\$superclass"'
				with: [self call: #class with: #superclass])
		];cr.
		self comment: 'add to subclasses of parent'.
		self if: [self send: #isNil: with: [self send: #basic:at:or:
			withArguments: #(superclass '"@subclasses"' #nil)]] do: [
				self send: #basic:at:put: withArguments: (Array
					with: #superclass with: '"@subclasses"'
					with: [self send: #newArray])
		];cr.
		self send: #add:
			to: [self send: #basic:at: withArguments: #(superclass '"@subclasses"')]
			with: #aClass;cr.
		self if: [self send:#isNil: with: [self send: #basic:at:or: withArguments: (Array
				with: [self call: #class with: #superclass]
				with: '"@subclasses"' with: #nil)
			]] do: [
			self send: #basic:at:put: withArguments: (Array
				with: [self call: #class with: #superclass]
				with: '"@subclasses"'
				with: [self send: #newArray])
		];cr.
		self send: #add: to: [self send: #basic:at: withArguments: (Array
				with: [self call: #class with: #superclass]
				with: '"@subclasses"')]
			with: [self call: #class with: #aClass]
		];cr.
		self if: [self send: #isNil: with: #category] do: [
			self assign: #category to: [self send: #basic:at:or:
				withArguments: #(aClass '"\$category"' '""')]
		];cr.
		self send: #basic:at:put: withArguments: #(aClass '"\$category"' category);cr.
		self send: #setVarNames withArguments: #(aClass iVarNames #nil);cr.
		self if: [self not: #sameShape] do: [
			self send: #classShapeChanged with: #aClass
		]
	] else: [
		self send: #aboutToCreateClass withArguments: #(className superclass iVarNames category);cr.
		self send: #mapClassName withArguments: #(className category #nil superclass);cr.
		self assign: #aClass to: [self send: #basicAt: with: #className];cr.
		self send: #setVarNames withArguments: #(aClass iVarNames #nil)
	];cr.
	self send: #classChanged with: #aClass;cr.
	self return: #aClass ! !

! JuliaBoot methodsFor: #api !
renameClassStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #niy] ! !

! JuliaBoot methodsFor: #api !
removeClassStSystemDictionary
	" Private - Implementation of api function "

	self assign: #aName to: [self send: #name to: #aClass];cr.
	self assign: #deleted to: [self concat: #('"Deleted-"' aName)];cr.
	self assign: #parent to: [self send: #superclass to: #aClass];cr.
	self if: [self send: #notNil: with: #parent] do: [
		self assign: #children to: [self send: #basic:at:or: withArguments: #(parent '"@subclasses"' #nil)];cr.
		self expr: [self send: #notNil: with: #children]
			and: [self send: #remove: to: #children with: #aClass ]
	];cr.
	self send: #basic:at:put: withArguments: (Array
		with: [self call: #class with: #aClass]
		with: '"\$className"'
		with: [self concat: #(deleted '" class"')]);cr.
	self send: #basic:at:put: withArguments: #(aClass '"\$className"' deleted);cr.
	self send: #basicDelete: with: #aName;cr.
	self send: #classChanged with: #aClass;cr.
	self return: #self ! !

! JuliaBoot methodsFor: #api !
metaVarNamesStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Set the instance variables of aMetaclass';cr.
	self send: #setVarNames withArguments: #(aMetaClass iVarNames #nil);cr.
	self send: #classShapeChanged with: #aMetaClass;cr.
	self send: #classChanged with: #aMetaClass;cr.
	self return: #true ! !

! JuliaBoot methodsFor: #api !
smDoStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'println("in smDo: from ",string(Base.stacktrace()[2]))';cr.
	self expr: [self not: 'self.useManager'] and: [self return: #nil];cr.
	self assign: #sm to: [self send: #basicAt: with: '"SystemManager"'];cr.
	self expr: [self send: #notNil: with: #sm] and: [self return: #nil];cr.
	self expr: [self not: [self send: #responds:to:
		to: #smalltalk withArguments: #(sm :currentOrNil)]]
		and: [self return: #nil];cr.
	self assign: #sm to: [self send: #currentOrNil to: #sm];cr.
	self expr: [self send: #notNil: with: #sm]
		and: [self return: [self call: #block with: #sm]] ! !

! JuliaBoot methodsFor: #api !
classChangedStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self implementFunction: '' arguments: #(sm) doing: [
		self send: #classChanged: to: #sm with: #aBehavior
	]] ! !

! JuliaBoot methodsFor: #api !
classShapeChangedStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self implementFunction: '' arguments: #(sm) doing: [
		self send: #classShapeChanged: to: #sm with: #aBehavior
	]] ! !

! JuliaBoot methodsFor: #api !
aboutToCreateClassStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self implementFunction: '' arguments: #(sm) doing: [
		self send: #aboutToCreateClass:superclass:iVarNames:category: to: #sm
			withArguments: #(className superclass iVarNames category)
	]] ! !

! JuliaBoot methodsFor: #api !
aboutToChangeClassStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self implementFunction: '' arguments: #(sm) doing: [
		self send: #aboutToChange:superclass:iVarNames:category: to: #sm
			withArguments: #(aClass superclass iVarNames category)
	]] ! !

! JuliaBoot methodsFor: #api !
methodBoundToStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self implementFunction: '' arguments: #(sm) doing: [
		self send: #methodBound:to: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method ! !

! JuliaBoot methodsFor: #api !
aboutToBindToStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self implementFunction: '' arguments: #(sm) doing: [
		self send: #aboutToBind:to: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method ! !

! JuliaBoot methodsFor: #api !
aboutToRemoveFromStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self implementFunction: '' arguments: #(sm) doing: [
		self send: #aboutToRemove:from: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method ! 

! JuliaBoot methodsFor: #api !
methodRemovedFromStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self implementFunction: '' arguments: #(sm) doing: [
		self send: #methodRemoved:from: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method ! 

! JuliaBoot methodsFor: #api !
objectNotFoundStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self implementFunction: '' arguments: #(sm) doing: [
		self send: #object:at:notFoundIn: to: #sm withArguments: #(object indexOrKey collection)
	]];cr.
	self return: #collection ! 

! JuliaBoot methodsFor: #api !
whoImplementsStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #niy] ! !

! JuliaBoot methodsFor: #api !
superImplementorStSystemDictionary
	" Private - Implementation of api function "

	self return: [self pars: [
		self implementFunction: '' arguments: #(aReceiver 'args...') doing: [
			self assign: #ff to: [self pars: [
				self implementFunction: '' arguments: #(i) doing: [
					self expr: [self expr: #i eqeq: #1]
						and: [self return: [self send: #basicNew to: [
							self send: #superclass to: #klass]]];cr.
					self return: 'args[i-1]'
				]]];cr.
		self assign: #ikTypes to: [self call: #typeof with: [
			self call: 'Base.ntuple' withArguments: (Array
				with: #ff with: [self succ: [self call: #length with: #args]])
		]];cr.
		self assign: #fun to: [self call: #fn with: [
			self send: #>>> to: [self send: #superclass to: #klass] with: #jsSelector
		]];cr.
		self call: #println withArguments: #('"###about to invoke "' aReceiver '" "' fun '" on "' ikTypes);cr.
		self return: [self call: 'Base.invoke' withArguments: #(fun ikTypes aReceiver 'args...')]
	]]] ! !

! JuliaBoot methodsFor: #api !
instVarAtStSystemDictionary
	" Private - Implementation of api function "

	self assign: #key to: [self concat: (Array with: '"\$"' with: [self call: #string with: #ivName])];cr.
	self comment: 'println("instVarAt ",key ," in a " ,typeof(object) ," object=",object)';cr.
	self expr: [self not: [self call: #hasfield withArguments: (Array
		with: [self call: #typeof with: #object] with: #:dict)]]
		and: [self return: #default];cr.
	self assign: #result to: [self send: #basic:at:or: withArguments: #(object key #nil)];cr.
	self expr: [ self send: #notNil: with: #result] and: [self return: #result];cr.
	self assign: #result to: [self send: #basic:at:or:
		withArguments: #(object '"@handle"' #nil)];cr.
	self if: [self send: #notNil: with: #result] do: [
		self call: #println withArguments: #('"instVarAt "' key '" in @handle"');cr.
		self return: [self send: #basic:at:or: withArguments: #(result key default)]
	];cr.
	self return: #default ! !

! JuliaBoot methodsFor: #api !
addMethodStSystemDictionary
	" Private - Implementation of api function "

	self send: #aboutToBindTo withArguments: #(method klass);cr.
	self assign: #fn to: [self send: #instVarAt withArguments: #(method '"fn"')];cr.
	self comment: 'basic_at_put_(self, fn, "\$s8-method", method)';cr.
	self send: #basic:at:put: withArguments: #(klass jsSelector fn);cr.
	self if: [self expr: [self send: #isNil: with: #stSelector]
		or: [self expr: #stSelector eqeq: #0]] do: [
		self assign: #stSelector to: [self send: #asNativeSelector to: #jsSelector]
	];cr.
	self assign: [self
		object: [self send: #basic:at: withArguments: #(klass '"\$methods"')]
		at: [self call: #string with: #stSelector]]
		to: #method;cr.
	self send: #basic:at:put: withArguments: #(method '"@classField"' klass);cr.
	self return: [self send: #methodBoundTo withArguments: #(method klass)] ! !

! JuliaBoot methodsFor: #api !
bindStSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: [self send: #isNil: with: #jsSelector]
		or: [self expr: #jsSelector eqeq: #0]] do: [
		self assign: #jsSelector to: [self call: 'Base.replace' withArguments: (Array
			with: #stSelector with: [self expr: '":"' eqgt: '"_"'])]
	];cr.
	self send: #debugBind withArguments: #(
		klass stSelector jsSelector jsFunction
		category stSource nativeSource stamp);cr.
	self comment: 'println("bind - " ,stSelector )';cr.
	self assign: #spec to: [self anyDict: #String object:
		'"\$selector"' -> #stSelector
		,('"\$fn"' -> #jsFunction)
		,('"\$category"' -> #category)
		,('"\$source"' -> #stSource)
		,('"\$nativeSource"' -> #nativeSource)
		,('"\$stamp"' -> #stamp)
		,('"\$generation"' -> [self send: #state])
	];cr.
	self if: [self send: #notNil: with: [
		self send: #basic:at:or: withArguments: #(self '"CompiledMethod"' #nil)
		]] do: [
		self assign: #method to: [self send: #basicNewInstance: to: 'self.CompiledMethod' with: #spec]
		] else: [ self assign: #method to: #spec ];cr.
	self if: [self send: #notNil: with: #klass] do: [
		self send: #addMethod withArguments: #(jsSelector method klass stSelector)
	];cr.
	self return: #method ! !

! JuliaBoot methodsFor: #api !
removeNativeFunction_for_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self send: #isNil: with: #fn] and: [self return: #false];cr.
	self assign: #done to: #false;cr.
	self assign: #stClass to: [self call: #supertype with: 'aClass["\$constructor"]'];cr.
	self comment: 'mmm it should be the encoding types to match here'.
	self comment: 'e.g. Any for Object'.
	self comment: 'see methods(yourself).ms[1].sig.types'.
	self for: #mth in: [self call: #methods with: #fn] do: [
		self comment: 'println("Walking method ",mth)'.
		self assign: #rcvType to: 'mth.sig.types[2]';cr.
		self if: [self expr: #rcvType eqeq: #stClass] do: [
			self call: 'Base.delete_method' with: #mth;cr.
			self comment: 'println("Deleting method ",mth)'.
			self assign: #done to: #true
		]
	];cr.
	self return: #done ! !

! JuliaBoot methodsFor: #api !
removeMethodStSystemDictionary
	" Private - Implementation of api function "

	self send: #aboutToRemoveFrom withArguments: #(aMethod aClass);cr.
	self assign: #stSelector to: [self send: #selector to: #aMethod];cr.
	self send: #basic:delete: withArguments: (Array
		with: #aClass
		with: [self send: #asSelector to: #stSelector]);cr.
	self send: 'delete!!' to: [self send: #basic:at:
		withArguments: #(aClass '"\$methods"')]
		with: [self call: #string with: #stSelector];cr.
	self send: #removeNativeFunction:for: withArguments: (Array
		with: [self send: #fn to: #aMethod]
		with: #aClass);cr.
	self return: [self send: #methodRemovedFrom withArguments: #(aMethod aClass)] ! !

! JuliaBoot methodsFor: #api !
newMethodStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #bind withArguments: #(
		#nil selector #nil fn
		#nil source nativeSource
		stamp)] ! !

! JuliaBoot methodsFor: #api !
methodStSystemDictionary
	" Private - Implementation of api function "

	self comment: #OBSOLETE.
	self return: [self send: #bind withArguments: #(
		#nil 'spec.selector'
		#nil 'spec.fn' 'spec.category'
		'spec.source' 'spec["\$nativeSource"]'
		'spec["\$stamp"]')] ! !

! JuliaBoot methodsFor: #api !
readJSObjectStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #niy] ! !

! JuliaBoot methodsFor: #api !
basenStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #niy] ! !

! JuliaBoot methodsFor: #api !
globalsDoStSystemDictionary
	" Private - Implementation of api function "

	self for: #aPair in: 'self.dict' do: [
		self assign: #obj to: 'aPair[2]';cr.
		self if: [self send: #isObject: with: #obj] do: [
			self expr: [self expr: [
				self send: #respondsTo: to: #obj with: '"isClass"'] 
				and: [self not: [self send: #isClass to: #obj]]
				] and: [self call: #action with: 'aPair[1]']
		]
	] ! !

! JuliaBoot methodsFor: #api !
launchImageStSystemDictionary
	" Private - Implementation of api function "

	self assign: 'self.useManager' to: #true;cr.
	self if: [self send: #notNil: with: #imageName] do: [
		self assign: 'self["\$imageFileName"]' to: #imageName;cr.
		self send: #load with: #imageName
	];cr.
	self send: #state: with: '"launching"';cr.
	self expr: [self send: #respondsTo: with: '"launch"']
		and: [self send: #launch];cr.
	self send: #state: with: '"running"';cr.
	self return: #self ! !

! JuliaBoot methodsFor: #api !
is_kindOf_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #cls to: [self send: #classOf: with: #anObject];cr.
	self while: #true do: [
		self expr: [self expr: #cls eqeq: #aClass] and: [self return: #true];cr.
		self expr: [self send: #isNil: with: #cls] and: [self return: #false];cr.
		self assign: #cls to: [self send: #superclass to: #cls]
	] ! !

! JuliaBoot methodsFor: #api !
responds_to_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #cls to: [self send: #klass to: #anObject];cr.
	self while: [self send: #notNil: with: #cls] do: [
		self expr: [self send: #implements: to: #cls with: #aSelector]
			and: [self return: #true];cr.
		self assign: #cls to: [self send: #superclass to: #cls]
	];cr.
	self return: #false ! !

! JuliaBoot methodsFor: #api !
handleOfStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #handleOf:or: withArguments: #(anObject anObject)] ! !

! JuliaBoot methodsFor: #api !
handleOf_or_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: #anObject isa: #StObject] return: [
		self send: #basic:at:or: withArguments: #(anObject '"@handle"' default)
	];cr.
	self return: #default ! !

! JuliaBoot methodsFor: #api !
newHashStSystemDictionary
	" Private - Implementation of api function "

	self return: [self call: #S8Object with: [self call: 'Dict{String, Any}']] ! !

! JuliaBoot methodsFor: #api !
stObjectStSystemDictionary
	" Private - Implementation of api function "

	self return: #StObject ! !

! JuliaBoot methodsFor: #api !
debugBindStSystemDictionary
	" Private - Implementation of api function "

	self return: #self ! !

! JuliaBoot methodsFor: #api !
debugAddClassStSystemDictionary
	" Private - Implementation of api function "

	self return: #self ! !

! JuliaBoot methodsFor: #api !
toStringStSystemDictionary
	" Private - Implementation of api function "

	self return: '"smalltalk"' ! !

! JuliaBoot methodsFor: #api !
quitStSystemDictionary
	" Private - Implementation of api function "

	self call: 'Base.exit' ! !

! JuliaBoot methodsFor: #api !
versionStSystemDictionary
	" Private - Implementation of api function "

	self return: #VERSION ! !

! JuliaBoot methodsFor: #api !
folderContentsStSystemDictionary
	" Private - Implementation of api function "

	self assign: #result to: [self emptyArray];cr.
	self for: #each in: [self call: 'Base.Filesystem.readdir' with: #fileName] do: [
		self assign: #pathName to: [
			self call: 'Base.Filesystem.joinpath'
				withArguments: #(fileName each)
		];cr.
		self assign: #entry to: [self emptyArray];cr.
		self if: [self call: 'Base.Filesystem.isdir' with: #pathName] do: [
			self assign: #entry to: [self array: #('"folder"' #0 each)]
		] else: [
			self assign: #entry to: [self array: (Array
				with: '"file"'
				with: [self call: 'Base.filesize' with: #pathName]
				with: #each)]
		];cr.
		self send: 'Base.push!!' to: #result with: [self send: #newArray with: #entry]
	];cr.
	self return: [self send: #newArray with: #result] ! !

! JuliaBoot methodsFor: #api !
httpSupportStSystemDictionary
	" Private - Implementation of api function "

	self return: [self call: #eval with: [self call: 'Meta.parse' with: '"using HTTP; HTTP"']] ! !

! JuliaBoot methodsFor: #api !
httpContentsStSystemDictionary
	" Private - Implementation of api function "

	self try: [
		self assign: #response to: [
			self call: 'httpSupport(self).request'
				withArguments: #('"GET"' fileName)
		];cr.
		self return: [self call: #String with: 'response.body']
	] catch: #e doing: [ self return: #nil ] ! !

! JuliaBoot methodsFor: #api !
fileContentsStSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: [self call: #length with: #fileName] gt: #3] do: [
		self if: [self expr: [self send: #lowercase to: 'fileName[1:4]'] eqeq: '"http"']
			return: [self send: #httpContents with: #fileName]
	];cr.
	self expr: [self call: 'Base.Filesystem.isdir' with: #fileName]
		and: [self return: [self send: #folderContents with: #fileName]];cr.
	self expr: [self call: 'Base.Filesystem.isfile' with: #fileName]
		and: [self return: [self call: #read withArguments: #(fileName String)]];cr.
	self return: #nil ! !

! JuliaBoot methodsFor: #api !
fileWriteString
	" Private - Implementation of api function "

	self assign: #jmode to: [
		self send: #replace:with: to: #mode
			withArguments: #('"b"' '""')
	];cr.
	self	on: [self call: #open withArguments: #(fileName jmode)]
		with: #f do: [ self call: #write withArguments: #(f contents)] ! !

! JuliaBoot methodsFor: #api !
fileInStSystemDictionary
	" Private - Implementation of api function "

	self assign: #contents to: [self send: #fileContents with: #fileName];cr.
	self expr: [self send: #isNil: with: #contents]
		and: [self return: [self send: #error: with: [self concat: #('"Missing file "' fileName)]
		]];cr.
	self return: [self send: #fileIn to: #contents] ! !

! JuliaBoot methodsFor: #api !
emmitStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #emmit: with: #fileName] ! !

! JuliaBoot methodsFor: #api !
fileInJSStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'contents = fileContents(self,fileName)'.
	self comment: 'isNil_(self,contents) && return error_(self,"Missing file " * fileName)'.
	self comment: 'return Base.include_string(Main,contents,fileName)'.
	self return: [self send: 'Base.include' to: #Main with: #fileName] ! !

! JuliaBoot methodsFor: #api !
exportCategoryStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #exportCategory: with: #aString] ! !

! JuliaBoot methodsFor: #api !
saveImageStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #saveImage] ! !

! JuliaBoot methodsFor: #api !
saveImageAsStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #saveImageAs: with: #fileName] ! !

! JuliaBoot methodsFor: #api !
isFileOlderThanStSystemDictionary
	" Private - Implementation of api function "

	self if: [self not: [self call: 'Base.Filesystem.isfile' with: #fileName]] return: #false;cr.
	self if: [self not: [self call: 'Base.Filesystem.isfile' with: #referenceFileName]] return: #false;cr.
	self comment: 'note_(self,"File timestamp matching NIY")'.
	self return: [self expr: [self call: 'Base.Filesystem.mtime' with: #fileName]
		lt: [self call: 'Base.Filesystem.mtime' with: #referenceFileName]] ! !

! JuliaBoot methodsFor: #api !
startupStSystemDictionary
	" Private - Implementation of api function "

	self send: #state: with: '"booting"';cr.
	self assign: 'self["\$nativeLanguage"]' to: '"jl"';cr.
	self assign: 'self["OverwriteMethods"]' to: #true;cr.
	self assign: 'self["DumpOverwritenMethods"]' to: #false;cr.
	self send: #state: with: '"singularity"';cr.
	self send: #mapClassName withArguments: #('"Object"' '"Kernel"' StObject);cr.
	self send: #mapClassName withArguments: #('"Smalltalk"' '"Kernel"' StSystemDictionary 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Behavior"' '"Kernel"' StBehavior 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Class"' '"Kernel"' StClass 'self.Behavior');cr.
	self send: #mapClassName withArguments: #('"Metaclass"' '"Kernel"' StMetaclass 'self.Behavior');cr.
	self send: #mapClassName withArguments: #('"CompiledMethod"' '"Kernel"' StMethod 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Number"' '"Kernel"' Number 'self.Object');cr.
	self send: #mapClassName withArguments: #('"BlockClosure"' '"Kernel"' Function 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Boolean"' '"Kernel"' Bool 'self.Object');cr.
	self comment: 'mapClassName(self,"DateTime","Kernel",Date,self.Object)'.
	self send: #mapClassName withArguments: #('"UndefinedObject"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Collection"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"SequenceableCollection"' '"Kernel"' #nil 'self.Collection');cr.
	self send: #mapClassName withArguments: #('"String"' '"Kernel"' String 'self.SequenceableCollection');cr.
	self send: #mapClassName withArguments: #('"Array"' '"Kernel"' #nil 'self.SequenceableCollection');cr.
	self comment: 'mapClassName(self,"RegularExpression","Kernel",RegExp,self.Object)'.
	self send: #state: with: '"whiteHole"';cr.
	self send: #linkNatives;cr.
	self send: #state: with: '"wormHole"';cr.
	self send: #bindCore;cr.
	self send: #state: with: '"startup"';cr.
	self return: #self ! !

! JuliaBoot methodsFor: #api !
niyStSystemDictionary
	" Private - Implementation of api function "

	self assign: #text to: [self concat: #('"NIY - "' 'string(Base.stacktrace()[2])')];cr.
	self call: #println with: #text;cr.
	self return: #text ! !

! JuliaBoot methodsFor: #api !
niy_StSystemDictionary
	" Private - Implementation of api function "

	self call: #println with: [self call: #string
		with: [self concat: #('"NIY #"'  msg)]] ! !

! JuliaBoot methodsFor: #api !
linkNativesStSystemDictionary
	" Private - Implementation of api function "

	self assign: #theMetaclass to: 'self["Metaclass"]';cr.
	self for: #cls in: [self send: #nativeClasses] do: [
		self if: [self expr: [self object: [self send: #classOf: with: #cls] at: '"\$klass"'] eqeq: #nil] do: [
			self comment: 'println("Linking ",classOf_(self,cls)["\$className"])'.
			self assign: [self object: [self send: #classOf: with: #cls] at: '"\$klass"'] to: #theMetaclass
		]
	];cr.
	self assign: [self object: [self send: #classOf: with: 'self.Object'] at: '"\$superclass"'] to: 'self.Class';cr.
	self return: #true ! !

! JuliaBoot methodsFor: #api !
bindCoreStSystemDictionary
	" Private - Implementation of api function "

	self comment: '--bind the minimal functions(e.g i/o)'.
	self if: [self expr: 'self["\$imageFileName"]' eqeq: #nil] do: [
		self assign: 'self["\$imageFileName"]' to: [self concat: #('"s8.image."' 'self["\$nativeLanguage"]')]
	];cr.
	self return: #true ! !

! JuliaBoot methodsFor: #api !
invalidMessage_StSystemDictionary
	" Private - Implementation of api function "

	self call: #error with: [self concat: #('"Invalid message @"' 'string(Base.stacktrace()[2])')];cr.
	self call: #error with: [self concat: #('"Invalid message #"' msg)] ! !

! JuliaBoot methodsFor: #api !
obsoleteStSystemDictionary
	" Private - Implementation of api function "

	self assign: #text to: [self concat: #('"OBSOLETE - "' 'string(Base.stacktrace()[2])')];cr.
	self call: #println with: #text;cr.
	self return: #text ! !

! JuliaBoot methodsFor: #api !
linkNative_to_StSystemDictionary
	" Private - Implementation of api function "

	self comment: 'link native species of instance to delegate to aClass implementation'.
	self send: #obsolete ! !

! JuliaBoot methodsFor: #api !
newConstructorStSystemDictionary
	" Private - Implementation of api function "

	self send: #obsolete ! !

! JuliaBoot methodsFor: #api !
will_be_StSystemDictionary
	" Private - Implementation of api function "

	self send: #obsolete ! !

! JuliaBoot methodsFor: #api !
newArrayStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'return a new Array instance with contents in object'.
	self assign: #contents to: [self
		valueIf: [self send: #isNil: with: #object]
		do: [self emptyArray]
		else: #object];cr.
	self return: [self send: #nativeNew: to: 'self.Array' with: #contents] ! !

! JuliaBoot methodsFor: #api !
newArrayOfSizeStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #newArray with: 'Array{Any}(nothing,size)'] ! !

! JuliaBoot methodsFor: #api !
arrayAdaptStSystemDictionary
	" Private - Implementation of api function "

	self expr: [self expr: #cookie isa: [
		self call: #eltype with: [self send: #handle to: #array]]]
		and: [self return: #array];cr.
	self assign: 'array.dict["@handle"]' to: [self call: #convert withArguments: (Array
		with: 'Array{Any,1}' with: [self send: #handle to: #array])] ! !

! JuliaBoot methodsFor: #api !
arrayPackStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'currently disabled to avoid issues adding objects of other types than contents'.
	self comment: 'array.dict["@handle"] = [ x for x in handle(array) ]' ! !

! JuliaBoot methodsFor: #api !
become_a_StSystemDictionary
	" Private - Implementation of api function "

	self send: #obsolete ! !

! JuliaBoot methodsFor: #api !
setProtoOf_to_StSystemDictionary
	" Private - Implementation of api function "

	self send: #obsolete ! !

! JuliaBoot methodsFor: #api !
klassStSystemDictionary
	" Private - Implementation of api function "

	self call: #println with: [self sum: #('"OBSOLETE - klass "' 'spec.className')];cr.
	self send: #obsolete ! !

! JuliaBoot methodsFor: #api !
globalScopeStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Returns an object to access global scope'.
	self return: [self call: #S8GlobalScope] ! !

! JuliaBoot methodsFor: #api !
__f__StSystemDictionary
	" Private - Implementation of api function "

	self send: #obsolete ! !

! JuliaBoot methodsFor: #api !
__b__StSystemDictionary
	" Private - Implementation of api function "

	self send: #obsolete ! !

! JuliaBoot methodsFor: #api !
stacktraceStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #newArray with: [self call: 'Base.stacktrace']] ! !

! JuliaBoot methodsFor: #api !
datesSupportStSystemDictionary
	" Private - Implementation of api function "

	self return: [self call: #eval with: [self call: 'Meta.parse' with: '"using Dates; Dates"']] ! !

! JuliaBoot methodsFor: #api !
block_valueWithPossibleArguments_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #count to: [self send: #argumentCount to: #block];cr.
	self if: [self expr: #count eqeq: #0] return: [self call: #block];cr.
	self if: [self expr: #count eqeq: [self send: #length to: #args]] return: [self call: #block with: 'args...'];cr.
	self if: [self expr: [self send: #length to: #args] gt: #count] do: [
		self assign: #params to: '@view args[1:count]'
	] else: [
		self assign: #params to: [self call: #convert withArguments: #('Array{Any,1}' args)];cr.
		self while: [self expr: [self send: #length to: #params] lt: #count]
			do: [ self send: 'push!!' to: #params with: #nil ]
	];cr.
	self return: [self call: #block with: 'params...'] ! !

! JuliaBoot methodsFor: #api !
block_argumentCountStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'println("Block ",block," argumentCount=",methods(block).ms[1].nargs - 1)'.
	self return: [self prev: 'methods(block).ms[1].nargs'] ! !

! JuliaBoot methodsFor: #api !
pool_keyStSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: #h isa: #Dict] do: [
		self if: [self expr: #h isa: 'Dict{Symbol,Any}'] return: [self symbol: #k];cr.
		self if: [self expr: #h isa: 'Dict{String,Any}'] return: [self call: #string with: #k]
	];cr.
	self return: [self call: #string with: #k] ! !

! JuliaBoot methodsFor: #api !
pool_at_put_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: 'pool["@hash"]';cr.
	self assign: #k to: [self send: #pool_key withArguments: (Array
		with: #h with: [self send: #wrap: to: #pool with: #aKey])];cr.
	self assign: #v to: [self send: #wrap: to: #pool with: #aValue];cr.
	self assign: 'h[k]' to: #v;cr.
	self return: #aValue ! !

! JuliaBoot methodsFor: #api !
pool_removeKey_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: 'pool["@hash"]';cr.
	self assign: #k to: [self send: #wrap: to: #pool with: #aKey];cr.
	self if: [self expr: #h isa: #StObject] do: [
		self assign: #h to: 'h.dict'
	];cr.
	self if: [self expr: #h isa: #Dict] do: [
		self send: 'delete!!' to: #h with: [
			self send: #pool_key withArguments: #(h k)
		]
	] else: [
		self assign: 'h[k]' to: #nil
	] ! !

! JuliaBoot methodsFor: #api !
pool_at_ifAbsent_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: 'pool["@hash"]';cr.
	self assign: #k to: [self send: #pool_key withArguments: (Array
		with: #h with: [self send: #wrap: to: #pool with: #aKey])];cr.
	self assign: #found to: #nil;cr.
	self try: [
		self if: [self expr: #h isa: #Dict] do: [
			self assign: #found to: [self call: #get withArguments: #(h k #nil)]
		] else: [self assign: #found to: 'h[k]']
	] catch: '' doing: [];cr.
	self if: [self expr: #found noteq: #nil] do: [
		self return: [self send: #unwrap: to: #pool with: #found]
	] else: [self return: [self send: #evaluate to: #aBlock]] ! !

! JuliaBoot methodsFor: #api !
pool_includesKey_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: 'pool["@hash"]';cr.
	self assign: #k to: [self send: #pool_key withArguments: (Array
		with: #h with: [ self send: #wrap: to: #pool with: #aKey])];cr.
	self expr: [self expr: #h isa: #Dict]
		return: [self call: #haskey withArguments: #(h k)];cr.
	self expr: [self call: #haskey withArguments: #('h.dict' k)]
		return: [self expr: 'h.dict[k]' noteq: #nil];cr.
	self return: #false ! !

! JuliaBoot methodsFor: #api !
pool_keysStSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: 'pool["@hash"]';cr.
	self if: [self expr: #h isa: #StObject] do: [
		self assign: #h to: 'h.dict'
	];cr.
	self return: [self send: #newArray to: #smalltalk with: [
		self call: #collect with: [
		self call: 'Base.keys' with: #h]]] ! !

! JuliaBoot methodsFor: #api !
string_toNumberStSystemDictionary
	" Private - Implementation of api function "

	self expr: [self expr: [self call: #findfirst withArguments: #('"."' aString)] noteqeq: #nothing]
		return: [self call: #parse withArguments: #(Float64 aString)];cr.
	self return: [self call: #parse withArguments: #(Int64 aString)] ! !

! JuliaBoot methodsFor: #api !
string_copyFrom_to_StSystemDictionary
	" Private - Implementation of api function "

	self expr: [self expr: #fromIndex gt: #toIndex]
		return: '""';cr.
	self assign: #fromIndex to: [self call: 'Base.max' withArguments: #(fromIndex #1)];cr.
	self assign: #toIndex to: [self call: 'Base.min' withArguments: (Array
		with: #toIndex with: [self send: #size to: #aString])];cr.
	self return: [self object: #aString at: [self from: #fromIndex to: #toIndex]] ! !

! JuliaBoot methodsFor: #contents !
testExpressions
	" Private - Return the encoding for testing expressions. "

	^self encode: [
		self comment: '-------------------------------------------------------------------------------------'.
		self call: #println with: '"boot(.jl) section for testing support - Installing..."';cr.
		self comment: 'o = basicNewInstance_(smalltalk.Object)'.
		self comment: 'p = basicNewInstance_(smalltalk.Point); x_(p,3);y_(p,4); x(p)+y(p)==7'.
		self implementClass: #StSystemDictionary api: #(
			'halt() = halt(basicNewInstance_(smalltalk.Object))'
			'testAll() = testAll(smalltalk)'
			'test(i) = test(smalltalk,i)'
			't() = testFrom(smalltalk,1)'
			't(i) = t(smalltalk,i)'
			'testFrom(i) = testFrom(smalltalk,i)'
			'testFrom(i,j) = testFrom(smalltalk,i,j)'
			testAll
			#(test #(what))
			#(testFrom #(first 'last=1000'))
			#(t #(number))
		).
		self	bind: #src
			arguments: #('aClass::StBehavior' selector)
			doing: [ self perform: #srcStBehavior ].
		self cr.
	]! !

! JuliaBoot methodsFor: #api !
srcStBehavior
	" Private - Implementation of api function "

	self assign: #mth to: 'aClass["\$methods"][string(selector)]';cr.
	self return: 'mth["\$nativeSource"]' ! !

! JuliaBoot methodsFor: #api !
testAllStSystemDictionary
	" Private - Implementation of api function "

	self assign: #o to: [self send: #basicNewInstance: to: 'self.Object'];cr.
	self return: [self send: #testAll to: #o] ! !

! JuliaBoot methodsFor: #api !
testStSystemDictionary
	" Private - Implementation of api function "

	self assign: #o to: [self send: #basicNewInstance: to: 'self.Object'];cr.
	self assign: #selector to: [self symbol: [self concat: #('"test"' 'string(what)')]];cr.
	self assign: #mth to: [self send: #basic:at: to: #smalltalk withArguments: (Array
		with: [self call: #class with: #o] with: #selector)];cr.
	self expr: [self send: #isNil: with: #mth] return: #false;cr.
	self comment: 'println("test",what)'.
	self if: [self not: [self send: #mth to: #o]] do: [
		self call: #println withArguments: #('"test"' what '"  FAILED"');cr.
		self return: #true
	];cr.
	self return: #true ! !

! JuliaBoot methodsFor: #api !
testFromStSystemDictionary
	" Private - Implementation of api function "

	self for: #i from: #first to: #last do: [
		self try: [
			self send: #test with: #i
		] catch: #e doing: [
			self call: #println withArguments: #('"Error in test"' i '" - "' e)
		]
	];cr.
	self call: #println withArguments: #('"testFrom"' first '" to "' last '" complete"') ! !

! JuliaBoot methodsFor: #api !
tStSystemDictionary
	" Private - Implementation of api function "

	self assign: #o to: [self send: #basicNewInstance: to: 'self.Object'];cr.
	self assign: #selector to: [self concat: #('"test"' 'string(number)')];cr.
	self assign: #mth to: 'class(o)["\$methods"][string(selector)]';cr.
	self call: #println with: 'mth["\$source"]';cr.
	self call: #println with: '"------------------------------------------------------"';cr.
	self call: #println with: 'mth["\$nativeSource"]' ! !
