BootSector
	subclass: #JSBoot
	category: #jsBoot !

! JSBoot class methodsFor: #instantiation !
supportsLanguage: aLanguage
	" Private - Return true if instances of the receiver can encode to aLanguage. "

	^JSEncoder supportsLanguage: aLanguage! !

! JSBoot methodsFor: #constants !
version
	" Private - Returns the version string (dotted number and date). "

	^'1.7.0 - Stable'! !

! JSBoot methodsFor: #encoding !
lineComment: aString
	" Private - Encode aString as line comment. "

	^self nextPutAll: '// ' ,aString;cr ! !

! JSBoot methodsFor: #encoding !
multilineComment: array
	" Private - Encode an array as a multiline comment. "

	^self encode: [
		self nextPutAll: '/*' ;indent.
		array do: [:line| self cr; nextPutAll: line ].
		self dedent; cr; nextPutAll: '*/'; cr
	]! !

! JSBoot methodsFor: #encoding !
bind: selector to: object with: expression
	" Private - Bind message selector to object to execute expression when the message is received. "

	self assign: object ,$. ,selector asSelector to: expression ! !

! JSBoot methodsFor: #encoding !
bind: selector to: object doing: expression
	" Private - Bind message selector to object to execute expression when the message is received. "

	self bind: selector to: object with: expression; cr ! !

! JSBoot methodsFor: #definitions !
initialDefinition
	" Private - Return the encoding for initializing execution environment. "

	^self encode: [
		#(Object Behavior Class Method) do: [:cls|
			self implementFunction: #St ,cls;cr
		].
		self implementFunction: #StMetaclass doing: 'this.$meta = true;';cr
	] ! !

! JSBoot methodsFor: #asm-constants !
blockOPs
	" Private - Encoding for expression block OPs "

	^'{}' ! !

! JSBoot methodsFor: #asm-constants !
localOP
	" Private - Return the OP for defining new local variable (or nil). "

	^#var ! !

! JSBoot methodsFor: #asm-constants !
fieldOP
	" Private - Return the OP for tuple separator. "

	^$: ! !

! JSBoot methodsFor: #asm !
cases: array
	" Private - Encoding option cases. "

	array do: [:assoc| | options |
		options := assoc key isArray
			ifTrue: [ assoc key ]
			ifFalse: [ Array with: assoc key].
		options do: [:each|
			each isString
			ifTrue: [ self nextPutAll: 'case "', each ,'": ' ]
			ifFalse: [ each do: [:one| self nextPutAll: 'case "', one ,'": ' ] ].
		] separatedBy: [ self cr ].
		self	indent;cr;
			nextPutAll: assoc value;
			dedent;cr
	]! !

! JSBoot methodsFor: #asm-constants !
emptyString
	" Private - Encoding for empty string literal. "

	^'""' ! !

! JSBoot methodsFor: #asm-constants !
emptyArray
	" Private - Encoding for empty Array literal. "

	^'[]' ! !

! JSBoot methodsFor: #asm !
break
	" Private - Encoding for break expression. "

	^self space; nextPutAll: #break ! !

! JSBoot methodsFor: #asm-constants !
whileDoOPs
	" Private - Return the OPs for while-do expression. "

	^Array with: #while with: '' ! !

! JSBoot methodsFor: #asm-constants !
forInDoOPs
	" Private - Return the OPs for for-in-do expression. "

	^Array with: #for with: #in with: ''! !

! JSBoot methodsFor: #asm-constants !
forInDoPars
	" Private - Return the parentesis for for-in-do expression. "

	^'()'! !

! JSBoot methodsFor: #asm-constants !
deleteOP
	" Private - Return the OP for delete operator. "

	^#delete ! !

! JSBoot methodsFor: #asm !
delete: expression
	" Private - Encoding for delete expression. "

	self nextPutAll: self deleteOP; space; nextPutAll: expression ! !

! JSBoot methodsFor: #asm !
switch: condition cases: array
	" Private - Encoding for switch expression. "

	self nextPutAll: #switch.
	self between: '()' do: condition indent: false; space.
	self between: '{}' do: [ self cases: array ] indent: true.! !

! JSBoot methodsFor: #asm !
assign: obj property: key to: expression
	" Private - Encoding for assignment to object property. "

	self assign: obj ,$. ,key to: expression ! !

! JSBoot methodsFor: #asm !
assign: rcv iv: ivName to: expression
	" Private - Encoding for assignment to object instance variable. "

	self assign: rcv property: $@ ,ivName to: expression ! !

! JSBoot methodsFor: #asm !
assign: rcv pv: pvName to: expression
	" Private - Encoding for assignment to object private variable. "

	self assign: rcv property: $$ ,pvName to: expression ! !

! JSBoot methodsFor: #asm-constants !
selfOP
	" Private - Encoding for self. "

	^#this ! !

! JSBoot methodsFor: #asm !
bind: selector arguments: args doing: implementation comment: aComment
	" Private - Encoding for internal method implementation. "

	self bind: selector to: #this doing: [
		self	implementFunction: '' arguments: args
			doing: implementation comment: aComment
	]! !

! JSBoot methodsFor: #private !
bindAPI: spec
	" Private - Encoding for basic message implementation. "

	| rcv selector action |
	spec isString ifTrue: [ ^self bindAPI: (Array with: spec with: #()) ].
	spec last isArray ifFalse: [
		^self error: 'Invalid/corrupt API spec ' ,spec asLiteral
	].
	selector := spec first asSelector ,#Implementation.
	action := [ rcv perform: selector ].
	rcv := (self respondsTo: selector) ifTrue: [ self ] ifFalse: [
		(self class respondsTo: selector) ifTrue: [ self class ] ifFalse: [
			" self error: 'Missing API implementation for ' ,selector asLiteral "
			action := [ self comment: '*ToDo* - API ' ,selector asLiteral ,' Not Implemented Yet.' ]
		]
	].
	self bind: spec first arguments: spec last doing: action ! !

! JSBoot methodsFor: #implementations !
nilImplementation
	" Private - Return the encoding for nil implementation. "

	^self encode: [
		self
			bind: #toString to: #nil doing: 'function(){ return "nil" }';
			bind: #isNil: to: #nil doing: [ self nilIsNilImplementation ];
			bind: #is:eqeq: to: #nil doing: [ self nilIsEqeqImplementation ];
			yourself
	] ! !

! JSBoot methodsFor: #implementations !
nilIsNilImplementation
	" Private - Encoding for #isNil: implementation. "

	^self implementFunction: '' arguments: #(x) doing: [
		self switch: 'typeof x' cases: (
			#undefined -> [self return: #true]
			,(#( #(string boolean) #(number function)) -> [self return: #false] )
			) asArray;cr.
		self if: [self expr: #x eqeqeq: #null] or: [self expr: #x eqeqeq: #this] return: #true;cr.
		self if: [self expr: #x eqeqeq: #smalltalk] return: #false;cr.
		self if: 'x.$klass' and: [self typeOf: 'x.isNil' is: #function] return: [ self send: #isNil to: #x ];cr.
		self return: #false
	] ! !

! JSBoot methodsFor: #implementations !
nilIsEqeqImplementation
	" Private - Encoding for #is:eqeq: implementation. "

	^self implementFunction: '' arguments: #(a b) doing: [
		self if: [self expr: #a eqeqeq: #b] return: #true;cr.
		self	if: [self send: #isNil: with: #a]
			or: [self send: #isNil: with: #b]
			return: #false;cr.
		self return: [
			self	expr: [ self send: #valueOf to: #a ]
				eqeqeq: [ self send: #valueOf to: #b ]
		]
	] ! !

! JSBoot methodsFor: #implementations !
smalltalkImplementation
	" Private - Return the encoding for smalltalk implementation. "

	^self encode: [
		self bind: #toString to: #smalltalk doing: 'function() { return "smalltalk" }'
	] ! !

! JSBoot methodsFor: #implementations !
systemDictionaryPrivates
	" Private - Private encoding for SystemDictionary. "

	self bind: #klass arguments: #(spec) private: [
		self klassImplementation
	].
	self bind: #setProtoOf:to: arguments: #(nativeObject proto) private: [
		self assign: #nativeObject property: #__proto__ to: #proto
	].
	self bind: #bindRoots arguments: #(metaclass) private: [
		self bindRootsImplementation
	].
	self bind: #setVarNames arguments: #(aClass iVarNames aValue) private: [
		self setVarNamesImplementation
	].
	self bind: #setClass arguments: #(aClass spec) private: [
		self setClassImplementation
	].
	self bind: #fnInSpec arguments: #(spec) private: [
		self fnInSpecImplementation
	].! !

! JSBoot methodsFor: #implementations !
klassImplementation
	" Private - Implementation of private function klass. "

	self local: #aClass; cr.
	self if: 'spec.meta'
	  do: [ self assign: #aClass to: [ self newInstanceOf: #Metaclass ] ]
	  else: [
		self local: #metaSpec with: '{meta:true}';cr.
		self assign: #metaSpec property: #className to: 'spec.className + " class"';cr.
		self
			if: [ self not: [ self send: #isNil with: 'spec.superclass'  ] ]
			do: [ self assign: #metaSpec property: #superclass to: 'spec.superclass.$klass' ];cr.
		self local: #metaClass with: [ self send: #klass with: #metaSpec ];cr.
		self assign: #aClass to: [self call: 'new metaClass.$fn'].
	]; cr.
	self send: #setClass withArguments: #(aClass spec);cr.
	self assign: 'aClass.$fn.prototype' pv: #klass to: #aClass;cr.
	self assign: 'aClass.$fn.prototype' pv: #methods to: '{}';cr.
	self	if: [ self expr: 'aClass.$className' eqeq: '"Metaclass"' ]
		do: [ self send: #bindRoots with: #aClass ];cr.
	self return: #aClass! !

! JSBoot methodsFor: #implementations !
bindRootsImplementation
	" Private - Implementation of private function bindRoots. "

	self local: 'objectClassPrototype ' with: 'this.Object.$klass.$fn.prototype';cr.
	self send: #setProtoOf:to: withArguments: #(objectClassPrototype 'this.Class.$fn.prototype');cr.
	self send: #setProtoOf:to: withArguments: #('this.Smalltalk.$klass.$fn.prototype' objectClassPrototype);cr.
	self send: #setProtoOf:to: withArguments: #('this.Behavior.$klass.$fn.prototype' objectClassPrototype);cr.
	self assign: 'this.Object.$klass.$superclass' to: 'this.Class';cr.
	self assign: 'metaclass.$superclass' to: 'this.Behavior'.! !

! JSBoot methodsFor: #implementations !
setVarNamesImplementation
	" Private - Implementation of private function setVarNames. "

	self assign: #aClass pv: #iVarNames to: #iVarNames;cr.
	self local: #i with: 'iVarNames.length';cr.
	self while: 'i--' do: [
		self assign: 'aClass.$fn.prototype["@"+iVarNames[i]]' to: #aValue
	]! !

! JSBoot methodsFor: #implementations !
setClassImplementation
	" Private - Implementation of private function setClass. "

	self assign: #aClass pv: #className to: 'spec.className';cr.
	self if: [self not: 'spec.meta'] do: [ self assign: 'aClass.$klass["@instanceClass"]' to: #aClass ];cr.
	self assign: #aClass pv: #category to: [ self expr: 'spec.category' or: '""' ];cr.
	self assign: #aClass pv: #superclass to: [ self expr: 'spec.superclass' or: #nil ];cr.
	self	if: [ self not: [self send: #isNil with: 'aClass.$superclass' ]]
		do: [	self delete: 'aClass.$superclass["@subclasses"]';cr.
			self	if: [self not: [ self send: #isNil with: 'aClass.$klass' ]]
				do: [
					self if: [self not: 'spec.meta'] do: [
						self	assign: 'aClass.$klass'
							pv: #superclass
							to: 'aClass.$superclass.$klass'
					];cr.
					self if: 'aClass.$klass.$superclass' do: [
						self delete: 'aClass.$klass.$superclass["@subclasses"]'
					];cr
				];cr
		];cr.
	self assign: #aClass pv: #fn to: [self send: #fnInSpec with: #spec];cr.
	self send: #setVarNames withArguments: #(
		aClass 'spec.iVarNames||[]' #nil
	);cr.
	self return: #aClass! !

! JSBoot methodsFor: #implementations !
fnInSpecImplementation
	" Private - Implementation of private function fnInSpec. "

	self if: 'spec.meta' and: [self send: #isNil with: 'spec.superclass'] do: [
		self assign: #spec property: #superclass to: 'this.Class'
	];cr.
	self if: 'spec.fn' do: [
		self	if: [self not: [ self send: #isNil with: 'spec.superclass']]
			do: [ self send: #setProtoOf:to: to: #this
				withArguments: #('spec.fn.prototype' 'spec.superclass.$fn.prototype')
			];cr.
		self return: 'spec.fn'
	];cr.
	self local: '$s8Constructor' with: 'function(){}';cr.
	self if: [ self not: [self send: #isNil with: 'spec.superclass' ]] do: [
		self assign: '$s8Constructor' property: #prototype to: [self call: 'new spec.superclass.$fn']
	];cr.
	self return: '$s8Constructor' ! !

! JSBoot methodsFor: #implementations !
classesImplementation
	" Private - Encoding for basic message implementation. "

	self local: #classes with: self emptyArray;cr.
	self for: 'var i' in: #this do: [
		self if: [self expr: [self send: #hasOwnProperty with: #i]
			and: [self expr: #i noteq: '"$klass"']] do: [
			self local: #cls with: 'this[i]';cr.
			self if: [self not: [self send: #isNil with: #cls]] do: [
				self if: [self expr: 'cls.$klass' and: 'cls.$className'] do: [
					self send: #push to: #classes with: #cls
				]
			]
		];cr.
	];cr.
	self return: #classes ! !

! JSBoot methodsFor: #implementations !
subclassesImplementation
	" Private - Encoding for basic message implementation. "

	self	if: [self send: #hasOwnProperty to: #aClass with: '"@subclasses"']
		return: 'aClass["@subclasses"]';cr.
	self local: #classes with: [self send: #classes to: #this];cr.
	self local: #i with: 'classes.length';cr.
	self while: 'i--' do: [
		self assign: 'classes[i]["@subclasses"]' to: self emptyArray;cr.
		self assign: 'classes[i].$klass["@subclasses"]' to: self emptyArray;cr.
	];cr.
	self assign: #i to: 'classes.length';cr.
	self while: 'i--' do: [
		self local: #cls with: 'classes[i]';cr.
		self if: [self not: [self send: #isNil with: 'cls.$superclass']] do: [
			self send: #push to: 'cls.$superclass["@subclasses"]' with: #cls;cr.
			self send: #push to: 'cls.$superclass.$klass["@subclasses"]' with: 'cls.$klass'
		].
	];cr.
	self if: 'this.Class' do: [
		self assign: 'this.Class["@subclasses"]' to: '[]';cr.
		self assign: 'this.$klass["@subclasses"]' to: '[]'
	];cr.
	self return: 'aClass["@subclasses"]'! !

! JSBoot methodsFor: #implementations !
allSubclassesDoImplementation
	" Private - Encoding for basic message implementation. "

	self local: #subclasses with: [self send: #subclasses with: #aClass];cr.
	self local: #i with: 'subclasses.length';cr.
	self while: 'i--' do: [
		self local: #cls with: 'subclasses[i]';cr.
		self call: #aBlock with: #cls;cr.
		self send: #allSubclassesDo withArguments: #(cls aBlock)
	];cr.
	self return: #this! !

! JSBoot methodsFor: #implementations !
mapClassNameImplementation
	" Private - Encoding for basic message implementation. "

	self local: #spec with: [
		self object: #className -> #className
			,(#category -> #category)
			,(#superclass -> #superclass)
			,(#fn -> #fn)
	];cr.
	self if: 'this[className]' return: [
		self	send: #setClass to: #this
			withArguments: #('this[className]' spec)
	];cr.
	self assign: 'this[className]' to: [self send: #klass with: #spec];cr.
	self return: 'this[className]'! !

! JSBoot methodsFor: #implementations !
sameIVContentsImplementation
	" Private - Encoding for basic message implementation. "

	self if: [ self expr: #array1 eqeq: #array2 ] return: #true;cr.
	self if: [ self expr: 'array1.length' eqeqeq: 'array2.length' ] do: [
		self return: [ self send:  #every to: #array1 with: [
			self implementFunction: '' arguments: #(element index) doing: [
				self return: [self expr: #element eqeq: 'array2[index]']
			]]
		]
	];cr.
	self return: #false! !

! JSBoot methodsFor: #implementations !
addClassImplementation
	" Private - Encoding for basic message implementation. "

	self send: #debugAddClass withArguments: #(
		className superclass iVarNames category
	);cr.
	self if: 'this[className]' do: [
		self local: #aClass with: 'this[className]';cr.
		self local: #sameShape with: [
			self expr: [
				self expr: 'aClass.$superclass' eqeq: #superclass
				] and: [ self send: #sameIVContents to: #this
					withArguments: #('aClass.$iVarNames' iVarNames)
				]
		];cr.
		self if: [self expr: #sameShape and: [
			self expr: 'aClass.$category' eqeq: #category
		]] return: #aClass;cr.
		self send: #aboutToChangeClass to: #this
			withArguments: #(aClass superclass iVarNames category);cr.
		self if: 'aClass.$superclass' do: [
			self delete: 'aClass.$superclass["@subclasses"]';cr.
			self local: #aMeta with: 'aClass.$klass';cr.
			self if: [self expr: #aMeta and: #superclass] do: [
				self delete: 'aMeta.$superclass["@subclasses"]';cr.
				self assign: #aMeta pv: #superclass to: 'superclass.$klass'
			].
		];cr.
		self assign: #aClass pv: #superclass to: #superclass;cr.
		self if: [self expr: #superclass and: 'superclass["@subclasses"]'] do: [
			self if: [self send: #isNil with: 'superclass["@subclasses"]'] do: [
				self assign: 'superclass["@subclasses"]' to: self emptyArray
			];cr.
			self send: #push to: 'superclass["@subclasses"]' with: #aClass
		];cr.
		self send: #setVarNames withArguments: #(
			aClass 'iVarNames||[]' #nil
		);cr.
		self assign: #aClass pv: #category to: [
			self expr: #category or: 'aClass.$category'
		];cr.
		self if: [self not: #sameShape] do: [
			self send: #classShapeChanged with: #aClass
		].
	] else: [
		self send: #aboutToCreateClass withArguments: #(
			className superclass iVarNames category
		);cr.
		self assign: 'this[className]' to: [
			self send: #klass with: [
				self object: #className -> #className
					,(#iVarNames -> #iVarNames)
					,(#superclass -> #superclass)
			];cr.
			self assign: 'this[className]' pv: #category to: [
				self expr: #category or: self emptyString
			];cr
		]
	];cr.
	self send: #classChanged with: 'this[className]';cr.
	self return: 'this[className]'! !

! JSBoot methodsFor: #implementations !
renameClassImplementation
	" Private - Encoding for basic message implementation. "

	self delete: 'this[aClass.$className]';cr.
	self assign: #aClass pv: #className to: #aString;cr.
	self assign: 'this[aString]' to: #aClass;cr.
	self send: #classChanged with: #aClass! !

! JSBoot methodsFor: #implementations !
removeClassImplementation
	" Private - Encoding for basic message implementation. "

	self local: #deleted with: '"Deleted-"';cr.
	self if: 'aClass.$superclass' do: [
		self delete: 'aClass.$superclass["@subclasses"]'
	];cr.
	self delete: 'this[aClass.$className]';cr.
	self assign: #aClass pv: #className to: 'deleted + aClass.$className';cr.
	self assign: #aClass pv: 'klass.$className' to: 'deleted + aClass.$klass.$className';cr.
	self send: #classChanged with: #aClass! !

! JSBoot methodsFor: #implementations !
metaVarNamesImplementation
	" Private - Encoding for basic message implementation. "

	self if: [self send: #sameIVContents withArguments: #(
		'aMetaClass.$iVarNames' iVarNames
		) ] return: #false;cr.
	self send: #setVarNames withArguments: #(aMetaClass 'iVarNames||[]' #nil);cr.
	self send: #classShapeChanged with: #aMetaClass;cr.
	self send: #classChanged with: #aMetaClass;cr.
	self return: #true! !

! JSBoot methodsFor: #implementations !
smDoImplementation
	" Private - Encoding for basic message implementation. "

	self if: [self not: 'this.useManager'] return: #nil;cr.
	self if: [self not: 'this.SystemManager'] return: #nil;cr.
	self if: [self not: 'this.SystemManager.currentOrNil'] return: #nil;cr.
	self local: #sm with: [ self send: #currentOrNil to: 'this.SystemManager' ];cr.
	self if: [self not: [self send: #isNil with: #sm]] return: [self call: #block with: #sm].! !

! JSBoot methodsFor: #implementations !
smDo: selector withArguments: array
	" Private - Encoding for smDo send expression. "

	self send: #smDo with: [
		self implementFunction: '' arguments: #(sm) doing: [
			self send: selector to: #sm withArguments: array
		]
	]! !

! JSBoot methodsFor: #implementations !
classChangedImplementation
	" Private - Encoding for basic message implementation. "

	self smDo: #classChanged: withArguments: #(aBehavior)! !

! JSBoot methodsFor: #implementations !
classShapeChangedImplementation
	" Private - Encoding for basic message implementation. "

	self smDo: #classShapeChanged: withArguments: #(aBehavior)! !

! JSBoot methodsFor: #implementations !
aboutToCreateClassImplementation
	" Private - Encoding for basic message implementation. "

	self	smDo: #aboutToCreateClass:superclass:iVarNames:category:
		withArguments: #(className superclass iVarNames category)! !

! JSBoot methodsFor: #implementations !
aboutToChangeClassImplementation
	" Private - Encoding for basic message implementation. "

	self	smDo: #aboutToChange:superclass:iVarNames:category:
		withArguments: #(aClass superclass iVarNames category)! !

! JSBoot methodsFor: #implementations !
methodBoundToImplementation
	" Private - Encoding for basic message implementation. "

	self	smDo: #methodBound:to:
		withArguments: #(method klass);cr.
	self return: #method! !

! JSBoot methodsFor: #implementations !
aboutToBindToImplementation
	" Private - Encoding for basic message implementation. "

	self	smDo: #aboutToBind:to:
		withArguments: #(method klass);cr.
	self return: #method! !

! JSBoot methodsFor: #implementations !
aboutToRemoveFromImplementation
	" Private - Encoding for basic message implementation. "

	self	smDo: #aboutToRemove:from:
		withArguments: #(method klass);cr.
	self return: #method! !

! JSBoot methodsFor: #implementations !
methodRemovedFromImplementation
	" Private - Encoding for basic message implementation. "

	self	smDo: #methodRemoved:from:
		withArguments: #(method klass);cr.
	self return: #method! !

! JSBoot methodsFor: #implementations !
objectNotFoundImplementation
	" Private - Encoding for basic message implementation. "

	self	smDo: #object:at:notFoundIn:
		withArguments: #(object indexOrKey collection);cr.
	self return: #collection! !

! JSBoot methodsFor: #implementations !
whoImplementsImplementation
	" Private - Encoding for basic message implementation. "

	self local: #cls with: #klass;cr.
	self do: [
		self if: [self send: #hasOwnProperty to: 'cls.$fn.prototype' with: #jsSelector] return: #cls;cr.
		self assign: #cls to: 'cls.$superclass'
	] while: #cls;cr.
	self return: #nil ! !

! JSBoot methodsFor: #implementations !
superImplementorImplementation
	" Private - Encoding for basic message implementation. "

	self 	if: [self expr: [self send: #isNil with: 'klass.$superclass']
		or: [self send: #isNil with: 'klass.$superclass.$fn']]
		return: [self send: #missingImplementorFor: to: #klass with: #jsSelector];cr.
	self local: #cls with: [
		self send: #whoImplements to: #this
			withArguments: #('klass.$superclass' jsSelector)
	];cr.
	self	if: [self send: #isNil with: #cls]
		return: [self send: #missingImplementorFor: to: 'klass.$superclass' with: #jsSelector];cr.
	self return: 'cls.$fn.prototype[jsSelector]'! !

! JSBoot methodsFor: #implementations !
addMethodImplementation
	" Private - Encoding for basic message implementation. "

	self send: #aboutToBindTo withArguments: #(method klass);cr.
	self assign: 'klass.$fn.prototype[jsSelector]' to: 'method.$fn';cr.
	self assign: 'klass.$fn.prototype.$methods[method.$selector]' to: #method;cr.
	self assign: 'method["@classField"]' to: #klass;cr.
	self return: [self send: #methodBoundTo withArguments: #(method klass)]! !

! JSBoot methodsFor: #implementations !
bindImplementation
	" Private - Encoding for basic message implementation. "

	self send: #debugBind withArguments: #(
		klass stSelector jsSelector jsFunction category stSource stamp
	);cr.
	self define: #method asInstanceOf: #Method;cr.
	self assign: #method pv: #selector to: #stSelector;cr.
	self assign: #method pv: #fn to: #jsFunction;cr.
	self assign: #method pv: #category to: #category;cr.
	self assign: #method pv: #source to: #stSource;cr.
	self if: [self not: [self send: #isNil with: #stamp]] do: [
		self assign: #method pv: #stamp to: #stamp
	];cr.
	self assign: #method pv: #generation to: 'this.$state';cr.
	self if: [self not: [self send: #isNil with: #klass]] do: [
	self if: [self send: #isNil with: #jsSelector] do: [
		self assign: #jsSelector to: [
			self send: #replace to: #stSelector with: '/\:/g,"_"'
		]
	];cr.
	self send: #addMethod withArguments: #(jsSelector method klass)
	];cr.
	self return: #method! !

! JSBoot methodsFor: #implementations !
removeMethodImplementation
	" Private - Encoding for basic message implementation. "

	self send: #aboutToRemoveFrom withArguments: #(aMethod aClass);cr.
	self delete: 'aClass.$fn.prototype[aMethod.$selector.asSelector()]';cr.
	self delete: 'aClass.$fn.prototype.$methods[aMethod.$selector]';cr.
	self return: [ self	send: #methodRemovedFrom to: #this
			withArguments: #(aMethod aClass)
	]! !

! JSBoot methodsFor: #implementations !
newMethodImplementation
	" Private - Encoding for basic message implementation. "

	self return: [ self	send: #bind to: #this
			withArguments: #(#nil selector #nil fn #nil source #nil)
	]! !

! JSBoot methodsFor: #implementations !
methodImplementation
	" Private - Encoding for basic message implementation. "

	self return: [ self	send: #bind to: #this
			withArguments: #(#nil 'spec.selector'
				#nil 'spec.fn' 'spec.category'
				'spec.source' 'spec.stamp')
	]! 

! JSBoot methodsFor: #implementations !
readJSObjectImplementation
	" Private - Encoding for basic message implementation. "

	self if: [self not: #js] return: #nil;cr.
	self local: #object with: #js;cr.
	self local: #readObject with: [self expr: 'js.constructor' eqeqeq: #Object];cr.
	self local: #readArray with: [self expr: 'js.constructor' eqeqeq: #Array];cr.
	self if: #readObject do: [
		self assign: #object to: [self call: 'this.PoolDictionary.$$new']
	];cr.
	self for: 'var i' in: #js do: [
		self if: [self not: [self send: #hasOwnProperty to: #js with: #i]].
		self break;cr.
		self if: #readObject do: [
			self send: #at:put: to: #object withArguments: (Array with: #i with: [
				self send: #readJSObject with: 'js[i]'
			])
		];cr.
		self if: #readArray do: [
			self assign: 'object[i]' to: [
				self send: #readJSObject with: 'js[i]'
			]
		].
	];cr.
	self return: #object! !

! JSBoot methodsFor: #implementations !
globalsDoImplementation
	" Private - Encoding for basic message implementation. "

	self for: #key in: #this do: [
		self if: [self not: [self send: #hasOwnProperty with: #key]];break;cr.
		self local: #aValue with: 'this[key]';cr.
		self if: [	self expr: [self typeOf: #aValue isNot: #function] and: [
			self expr: 'aValue.isClass' and: [
			self not: [self send: #isClass to: #aValue]]]
		] do: [ self call: #action with: #key ];cr.
	];cr.
	self return: #this! !

! JSBoot methodsFor: #asm !
mapClassName: args
	" Private - Encoding for mapClassName expression. "

	self send: #mapClassName withArguments: args! !

! JSBoot methodsFor: #implementations !
startupImplementation
	" Private - Encoding for basic message implementation. "

	self assign: #this pv: #state to: '"booting"';cr.
	self assign: 'this.OverwriteMethods' to: [
		self expr: 'this.OverwriteMethods' or: #true
	];cr.
	self assign: 'this.DumpOverwritenMethods' to: [
		self expr: 'this.DumpOverwritenMethods' or: #false
	];cr.
	self mapClassName: #('"Object"' '"Kernel"' StObject);cr.
	self mapClassName: #('"Smalltalk"' '"Kernel"' StSystemDictionary 'this.Object');cr.
	self mapClassName: #('"Behavior"' '"Kernel"' StBehavior 'this.Object');cr.
	self mapClassName: #('"Class"' '"Kernel"' StClass 'this.Behavior');cr.
	self mapClassName: #('"Metaclass"' '"Kernel"' StMetaclass 'this.Behavior');cr.
	self mapClassName: #('"CompiledMethod"' '"Kernel"' StMethod 'this.Object');cr.
	self mapClassName: #('"Number"' '"Kernel"' Number 'this.Object');cr.
	self mapClassName: #('"BlockClosure"' '"Kernel"' Function 'this.Object');cr.
	self mapClassName: #('"Boolean"' '"Kernel"' Boolean 'this.Object');cr.
	self mapClassName: #('"DateTime"' '"Kernel"' Date 'this.Object');cr.
	self mapClassName: #('"UndefinedObject"' '"Kernel"' StUndefinedObject 'this.Object');cr.
	self mapClassName: #('"Collection"' '"Kernel"' undefined 'this.Object');cr.
	self mapClassName: #('"SequenceableCollection"' '"Kernel"' undefined 'this.Collection');cr.
	self mapClassName: #('"String"' '"Kernel"' String 'this.SequenceableCollection');cr.
	self mapClassName: #('"Array"' '"Kernel"' Array 'this.SequenceableCollection');cr.
	self mapClassName: #('"RegularExpression"' '"Kernel"' RegExp 'this.Object');cr.
	self assign: #this pv: #state to: '"startup"';cr.
	self return: #this! !

! JSBoot methodsFor: #implementations !
launchImageImplementation
	" Private - Encoding for basic message implementation. "

	self assign: #this property: #useManager to: #true;cr.
	self if: [self not: [self send: #isNil with: #imageName]] do: [
		self assign: #this pv: #imageFileName to: #imageName;cr.
		self call: #load with: #imageName
	];cr.
	self assign: #this pv: #state to: '"launching"';cr.
	self if: [self expr: 'this.launch'
		and: [self typeOf: 'this.launch' is: #function]] do: [
		self send: #launch to: #this
	];cr.
	self assign: #this pv: #state to: '"running"';cr.
	self return: #this! !

! JSBoot methodsFor: #implementations !
is_kindOf_Implementation
	" Private - Encoding for basic message implementation. "

	self if: [self expr: [self
		send:#isNil: to: #nil with: #anObject
		] and: [self expr: #nil noteqeq: #anObject]] return: #false;cr.
	self	if: [self typeOf: 'anObject.isKindOf_' is: #undefined]
		return: #false;cr.
	self return: [ self send: #isKindOf: to: #anObject with: #aClass ]! !

! JSBoot methodsFor: #implementations !
responds_to_Implementation
	" Private - Encoding for basic message implementation. "

	self if: [self expr: [self
		send:#isNil: to: #nil with: #anObject
		] and: [self expr: #nil noteqeq: #anObject]] return: #false;cr.
	self	if: [self typeOf: 'anObject.respondsTo_' is: #undefined]
		return: #false;cr.
	self return: [ self send: #respondsTo: to: #anObject with: #aSelector ]! !

! JSBoot methodsFor: #implementations !
debugBindImplementation
	" Private - Encoding for basic message implementation. "

	self return: #this! !

! JSBoot methodsFor: #implementations !
debugAddClassImplementation
	" Private - Encoding for basic message implementation. "

	self return: #this! !

! JSBoot methodsFor: #implementations !
initWithSubclassesImplementation
	" Private - Encoding for basic message implementation. "

	self return: #aClass! !

! JSBoot methodsFor: #implementations !
systemDictionaryAPI
	" Private - Internal encoding for basic SystemDictionary messages. "

	#(	#classes
		#(subclasses #(aClass))
		#(allSubclassesDo #(aClass aBlock))
		#(mapClassName #(className category fn superclass))
		#(sameIVContents #(array1 array2))
		#(addClass #(className superclass iVarNames category))
		#(renameClass #(aClass aString))
		#(removeClass #(aClass))
		#(metaVarNames #(aMetaClass iVarNames))
	"Private"	#(smDo #(block))
	"Private"	#(classChanged #(aBehavior))
	"Private"	#(classShapeChanged #(aBehavior))
	"Private"	#(aboutToCreateClass #(className superclass iVarNames category))
	"Private"	#(aboutToChangeClass #(aClass superclass iVarNames category))
	"Private"	#(methodBoundTo #(method klass))
	"Private"	#(aboutToBindTo #(method klass))
	"Private"	#(aboutToRemoveFrom #(method klass))
	"Private"	#(methodRemovedFrom #(method klass))
	"Private"	#(objectNotFound #(object indexOrKey collection))
		#(whoImplements #(klass jsSelector))
		#(superImplementor #(klass jsSelector))
		#(addMethod #(jsSelector method klass))
		#(bind #(klass stSelector jsSelector jsFunction category stSource stamp))
		#(removeMethod #(aClass aMethod))
		#(newMethod #(currentClass selector fn source))
"OBSOLETE	#(method #(spec))"
	"js"	#(readJSObject #(js))
		#(globalsDo #(action))
		#(startup #())
		#(launchImage #(imageName))
		#(is:kindOf: #(anObject aClass))
		#(responds:to: #(anObject aSelector))
	"DEBUG"	#(debugBind #(klass stSelector jsSelector jsFunction category stSource stamp))
	"DEBUG"	#(debugAddClass #(className superclass iVarNames category))
		#(initWithSubclasses #(aClass))
	) do: [:tuple| self bindAPI: tuple ]! !

! JSBoot methodsFor: #implementations !
systemDictionaryInternals
	" Private - Internal encoding for SystemDictionary. "

	self assign: #this pv: #imageFileName to: '"s8.image.js"';cr.
	self assign: #this pv: #nativeLanguage to: '"js"';cr.
	self assign: #this property: #useManager to: #false;cr.
	self bind: #isNil arguments: #(x) doing: [
		self return: [self expr: [self expr: #x or: #nil ] eqeqeq: #nil ]
	].
	self nextPutAll: [ self systemDictionaryPrivates ].
	self nextPutAll: [ self systemDictionaryAPI ].! !

! JSBoot methodsFor: #encoding !
define: parentClass subclass: aName doing: expression
	" Private - Return the encoding for defining a new S8 class named aName. "

	self implementFunction: #St ,aName doing: expression;cr ! !

! JSBoot methodsFor: #encoding !
newInstanceOf: clsName
	" Private - Encoding for defining a variable named aName. "

	self call: 'new St' ,clsName ! !

! JSBoot methodsFor: #encoding !
define: aName asInstanceOf: clsName
	" Private - Return the encoding for defining a variable named aName. "

	self	nextPutAll: #var; space;
		assign: aName to: [ self newInstanceOf: clsName; cr ] ! !

! JSBoot methodsFor: #encoding !
sendMessageArguments: args
	" Private - Encoding for aMessage arguments. "

	args size = 0 ifTrue: [ ^self ].
	self nextPutArguments: args ! !