BootSector
	subclass: #LuaBoot
	category: #luaBoot !

! LuaBoot class methodsFor: #instantiation !
supportsLanguage: aLanguage
	" Private - Return true if instances of the receiver can encode to aLanguage. "

	^#lua = aLanguage! !

! LuaBoot methodsFor: #constants !
version
	" Private - Returns the version string (dotted number and date). "

	^'1.0 - Stable'! !

! LuaBoot methodsFor: #encoding !
lineComment: aString
	" Private - Encode aString as line comment. "

	^self nextPutAll: '-- ' ,aString;cr ! !

! LuaBoot methodsFor: #encoding !
multilineComment: array
	" Private - Encode an array as a multiline comment. "

	^self encode: [
		self nextPutAll: '--[[' ;indent.
		array do: [:line| self cr; nextPutAll: line ].
		self dedent; cr; nextPutAll: '--]]'; cr
	]! !

! LuaBoot methodsFor: #contents !
startupExpression
	" Private - Return the encoding for startup expression of the receiver. "

	^self encode: [
		self return: [ self send: #startup to: #smalltalk ]
	]! !

! LuaBoot methodsFor: #definitions !
nilDefinition
	" Private - Return the encoding for nil. "

	^'' "we share nil instance with Lua space" ! !

! LuaBoot methodsFor: #definitions !
smalltalkDefinition
	" Private - Return the encoding for smalltalk. "

	^(self encode: [
		self define: #smalltalk as: '{}'
	]) ,self smalltalkImplementation ! !

! LuaBoot methodsFor: #encoding !
define: aName as: expression
	" Private - Return the encoding for defining a variable named aName. "

	self	nextPutAll: #local; space;
		assign: aName to: expression; cr ! !

! LuaBoot methodsFor: #asm-constants !
sendMessageOP
	" Private - Encoding for send message OP "

	^$: ! !

! LuaBoot methodsFor: #implementations !
smalltalkImplementation
	" Private - Return the encoding for smalltalk implementation. "

	^self encode: [
		self assign: #smalltalk pv: #nativeLanguage to: '"lua"';cr.
		self assign: #smalltalk property: #useManager to: #false;cr.
		self systemDictionaryInternals.
		self rootDefinition.
		self systemDictionaryPrivates.
		self systemDictionaryAPI.
	] ! !

! LuaBoot methodsFor: #definitions !
rootDefinition
	" Private - Return the encoding for root classes. "

	self define: #StObject as: [ self send: #newConstructor to: #smalltalk ].
	self define: #StBehavior as: [ self send: #newConstructor to: #smalltalk with: #StObject ].
	self define: #StClass as: [ self send: #newConstructor to: #smalltalk with: #StBehavior ].
	self define: #StMetaclass as: [ self send: #newConstructor to: #smalltalk with: #StBehavior ].
	self assign: 'StMetaclass.prototype' pv: #meta to: 'true';cr.
	self define: #StMethod as: [ self send: #newConstructor to: #smalltalk with: #StObject ].
	self define: #StSystemDictionary as: [ self send: #newConstructor to: #smalltalk with: #StObject ]. ! !

! LuaBoot methodsFor: #implementations !
systemDictionaryInternals
	" Private - Internal encoding for smalltalk. "

	#(	#(do:onError: #(aBlock errorBlock))
		#(isNil: #(aValue))
		#(notNil: #(aValue))
		#(basic:at:ifAbsent: #(receiver key aBlock))
		#(basic:at:put: #(receiver key aValue))
		#(ifUndefined:do: #(receiver aBlock))
		#(orNil: #(receiver))
		#(newConstructor #(super))
	) do: [:tuple| self bindAPI: tuple ]! !

! LuaBoot methodsFor: #asm-constants !
localOP
	" Private - Return the OP for defining new local variable (or nil). "

	^#local ! !

! LuaBoot methodsFor: #asm-constants !
notOP
	" Private - Return the OP for negation. "

	^'not ' ! !

! LuaBoot methodsFor: #asm-constants !
orOP
	" Private - Encoding for operand. "

	^#or ! !

! LuaBoot methodsFor: #asm-constants !
andOP
	" Private - Encoding for operand. "

	^#and ! !

! LuaBoot methodsFor: #asm-constants !
noteqOP
	" Private - Encoding for operand. "

	^#~= ! !

! LuaBoot methodsFor: #asm-constants !
blockOPs
	" Private - Encoding for expression block OPs "

	^'{}' ! !

! LuaBoot methodsFor: #asm-constants !
expressionsOPs
	" Private - Encoding for expression list OPs "

	^#('' end) ! !

! LuaBoot methodsFor: #asm-constants !
fieldOP
	" Private - Return the OP for tuple separator. "

	^$= ! !

! LuaBoot methodsFor: #asm-constants !
emptyString
	" Private - Encoding for empty string literal. "

	^'""' ! !

! LuaBoot methodsFor: #asm-constants !
forInDoPars
	" Private - Return the parentesis for for-in-do expression. "

	^''! !

! LuaBoot methodsFor: #asm-constants !
forDoPars
	" Private - Return the parentesis for for-do expression. "

	^''! !

! LuaBoot methodsFor: #asm-constants !
typeofOPs
	" Private - Return the OP for typeOf expression. "

	^#('type(' $)) ! !

! LuaBoot methodsFor: #asm !
if: condition do: trueBlock else: falseBlock
	" Private - Encoding for conditional. "

	self if: condition; space.
	self between: (Array
		with: #then
		with: (falseBlock isNil ifTrue: [#end] ifFalse: ['']))
		do: trueBlock indent: true.
	falseBlock notNil ifTrue: [
		self space; nextPutAll: self elseOP; space.
		self between: #('' end) do: falseBlock indent: true.
	]! !

! LuaBoot methodsFor: #asm !
if: condition return: expression
	" Private - Encoding for conditional expression return. "

	self	if: condition; space;
		between: #(then end)
		do: [ self space; return: expression; space ]
		indent: false! !

! LuaBoot methodsFor: #asm !
assign: obj property: key to: expression
	" Private - Encoding for assignment to object property. "

	self assign: obj ,$. ,key to: expression ! !

! LuaBoot methodsFor: #asm !
assign: obj index: key to: expression
	" Private - Encoding for assignment to indexed property. "

	self assign: obj ,$[ ,key ,$] to: expression ! !

! LuaBoot methodsFor: #asm !
assign: rcv iv: ivName to: expression
	" Private - Encoding for assignment to object instance variable. "

	self assign: rcv index: $" ,$@ ,ivName ,$" to: expression ! !

! LuaBoot methodsFor: #asm !
assign: rcv pv: pvName to: expression
	" Private - Encoding for assignment to object private variable. "

	self assign: rcv index: $" ,$$ ,pvName ,$" to: expression ! !

! LuaBoot methodsFor: #asm !
rawFunction: aName arguments: args doing: implementation
	" Private - Encoding for function (w/o self definition) implementation. "

	^super	implementFunction: aName
		arguments: args doing: implementation
		comment: nil ! !

! LuaBoot methodsFor: #encoding !
callPrint: arg1
	" Private - Encoding for print expression. "

	^self call: #print with: arg1 ! !

! LuaBoot methodsFor: #encoding !
concat: rcv with: arg1
	" Private - Encoding for contatenation expression. "

	self	nextPutAll: rcv;
		space; nextPutAll: '..';space;
		nextPutAll: arg1 ! !

! LuaBoot methodsFor: #asm !
bind: selector arguments: args doing: implementation comment: aComment
	" Private - Encoding for internal method implementation. "

	self	implementFunction: 'smalltalk.' ,selector asSelector
		arguments: args doing: implementation
		comment: aComment ! !

! LuaBoot methodsFor: #encoding !
bind: selector to: object with: expression
	" Private - Bind message selector to object to execute expression when the message is received. "

	self assign: object ,$. ,selector asSelector to: expression ! !

! LuaBoot methodsFor: #encoding !
bind: selector to: object doing: expression
	" Private - Bind message selector to object to execute expression when the message is received. "

	self bind: selector to: object with: expression; cr ! !

! LuaBoot methodsFor: #private !
bindAPI: spec
	" Private - Encoding for basic message implementation. "

	| rcv selector action |
	spec isString ifTrue: [ ^self bindAPI: (Array with: spec with: #()) ].
	spec last isArray ifFalse: [
		^self error: 'Invalid/corrupt API spec ' ,spec asLiteral
	].
	selector := spec first asSelector ,#Implementation.
	action := [ rcv perform: selector ].
	rcv := (self respondsTo: selector) ifTrue: [ self ] ifFalse: [
		(self class respondsTo: selector) ifTrue: [ self class ] ifFalse: [
			" self error: 'Missing API implementation for ' ,selector asLiteral "
			action := [ self comment: '*ToDo* - API ' ,selector asLiteral ,' Not Implemented Yet.' ]
		]
	].
	self bind: spec first arguments: spec last doing: action ! !

! LuaBoot methodsFor: #encoding !
implementFunction: aName arguments: args doing: expression comment: aComment
	" Private - Define the function named aName doing expression. "

	self	nextPutAll: #function; space; nextPutAll: aName;
		pars: [ self nextPutArguments: #(self) ,args ];
		between: #('' end) do: expression indent: true comment: aComment;
		cr! !

! LuaBoot methodsFor: #lua-internal !
do_onError_Implementation
	" Private - Implementation of internal function. "

	self local: 'status, result' with: [self call: #pcall with: #aBlock];cr.
	self if: #status return: #result;cr.
	self return: [self call: #errorBlock with: #result] ! !

! LuaBoot methodsFor: #lua-internal !
isNil_Implementation
	" Private - Implementation of internal function. "

	self if: [self not: #aValue] return: [self expr: #aValue eqeq: #nil];cr.
	self if: [self typeOf: #aValue is: #table] do: [
		self if: [self typeOf: 'aValue.isNil' is: #function] do: [
			self return: [self send: #isNil to: #aValue]
		]
	];cr.
	self return: #false! !

! LuaBoot methodsFor: #lua-internal !
notNil_Implementation
	" Private - Implementation of internal function. "

	self if: [self not: #aValue] return: [self expr: #aValue noteq: #nil];cr.
	self if: [self typeOf: #aValue is: #table] do: [
		self if: [self typeOf: 'aValue.notNil' is: #function] do: [
			self return: [self send: #notNil to: #aValue]
		]
	];cr.
	self return: #true! !

! LuaBoot methodsFor: #lua-internal !
basic_at_ifAbsent_Implementation
	" Private - Implementation of internal function. "

	self local: #result with: 'receiver[key]';cr.
	self if: [self send: #isNil: with: #result] do: [
		self return: [self send: #value to: #aBlock]
	];cr.
	self return: #result! !

! LuaBoot methodsFor: #lua-internal !
basic_at_put_Implementation
	" Private - Implementation of internal function. "

	self assign: 'receiver[key]' to: #aValue;cr.
	self return: #aValue! !

! LuaBoot methodsFor: #lua-internal !
ifUndefined_do_Implementation
	" Private - Implementation of internal function. "

	self local: #result with: #receiver;cr.
	self if: [self send: #isNil: with: #result] do: [
		self return: [self send: #value to: #aBlock]
	];cr.
	self return: #result! !

! LuaBoot methodsFor: #lua-internal !
orNil_Implementation
	" Private - Implementation of internal function. "

	self local: #result with: #receiver;cr.
	self if: [self send: #isNil: with: #result] return: #nil;cr.
	self return: #result! !

! LuaBoot methodsFor: #lua-internal !
inlineDebugDNU
	" Private - Inline code to debug dump on DNU. "

	self comment: 'DEBUG';cr.
	self callPrint: [self concat: '"--DNU "' with: [self call: #tostring with: 'smalltalk.dnu']];cr.
	self if: [self send: #isUppercase to: [
		self send: #first to: [self call: #tostring with: 'smalltalk.dnu']]]
		return: #nil;cr.
	self comment: 'DEBUG DNU detected dump stack trace';cr.
	self if: #debug do: [
		self callPrint: [self call: 'debug.traceback' with: '"Stack trace"'];cr.
		self callPrint: [self call: 'debug.getinfo' with: #1];cr.
		self callPrint: '"Stack trace end"'
	];cr.
	self comment: 'if smalltalk.dnu=="arguments" then error("DNU exiting...") end';cr.
	self call: #error with: '"DNU"' ! !

! LuaBoot methodsFor: #lua-internal !
newConstructorImplementation
	" Private - Implementation of internal function. "

	self comment: 'return a new constructor of instances inheriting from super (constructor)';cr.
	self comment: 'super can be nil to return a constructor for a root class';cr.

	self local: #constructor with: [self emptyObject];cr.
	self local: #prototype with: [self emptyObject];cr.
	self assign: #prototype property: #__index to: #prototype;cr.
	self if: #super
	 do: [self call: #setmetatable withArguments: #(prototype 'super.prototype')]
	 else: [
		self local: #dnu with: [self emptyObject];cr.
		self assign: #dnu property: #__index to: [
			self rawFunction: '' arguments: #(rootPrototype key) doing: [
				self comment: 'print("--keyNotFound: "..key)';cr.
				self assign: #smalltalk property: #dnu to: #key;cr.
				self return: #nil
			]
		];cr.
		self call: #setmetatable withArguments: #(prototype dnu)
	];cr.
	self call: #setmetatable withArguments: #(constructor prototype);cr.
	self assign: #constructor property: #prototype to: #prototype;cr.
	self assign: #prototype property: #__call to: [
		self rawFunction: '' arguments: #(f rcv '...') doing: [
			self if: [self expr: [self expr: #f eqeq: #nil] and: 'smalltalk.dnu'] do: [
				self if: [self expr: [
					self call: 'string.find' withArguments: #(
						'smalltalk.dnu'
						'"[%a_%+%-%/%*%|%&%=%<%>%,%~%\\]"'
					)] noteq: #1]
					return: #nil;cr.
				self inlineDebugDNU;cr.
				self comment: 'Syntetize a doesNotUnderstand message send';cr.
				self local: #args with: [
					self call: [self expr: #pack or: 'table.pack'] with: '...'
				];cr.
				self local: #msg with: [
					self send: #receiver:selector:arguments:
						to: 'smalltalk.Message'
						withArguments: (Array with: #rcv with: 'smalltalk.dnu'
							with: [self send: #newArray to: #smalltalk with: #args])
				];cr.
				self assign: #smalltalk property: #dnu to: #nil;cr.
				self local: #result with: #nil;cr.
				self if: [self send: #isObject: to: #smalltalk with: #rcv]
					do: [ self assign: #result to: [self send: #doesNotUnderstand: to: #rcv with: #msg]]
					else: [	self comment: 'DNU goes to smalltalk for native objects';cr.
						self assign: #result to: [self send: #doesNotUnderstand: to: #smalltalk with: #msg]
				];cr.
				self return: #result
			];cr.
			self local: #result with: [self emptyObject];cr.
			self call: #setmetatable withArguments: #(result prototype);cr.
			self return: #result
		]
	];cr.
	self return: #constructor! !

! LuaBoot methodsFor: #implementations !
systemDictionaryPrivates
	" Private - Private encoding for SystemDictionary. "

	#(	#(will:be: #(anObject className))
		#(linkNative:to: #(instance aClass))
		#(newArray #(object))
		#(become:a: #(object aClass))
		#(setProtoOf:to: #(nativeObject proto))
		#(bindRoots #(metaclass))
		#(klass #(spec))
		#(setVarNames #(aClass iVarNames aValue))
		#(setClass #(aClass spec))
		#(fnInSpec #(spec))
	) do: [:tuple| self bindAPI: tuple ]! !

! LuaBoot methodsFor: #lua-private !
will_be_Implementation
	" Private - Implementation of private function. "

	self comment: 'return an object that will become an instance of class (named className) when accessed';cr.
	self local: #result with: #anObject;cr.
	self local: #mt with: [self emptyObject];cr.
	self assign: #mt property: #__index to: [
		self rawFunction: '' arguments: #(table key) doing: [
			self if: 'self[className]' do: [
				self send: #become:a: withArguments: #(table 'self[className]')
			];cr.
			self return: 'table[key]'
		]
	];cr.
	self call: #setmetatable withArguments: #(result mt);cr.
	self return: #result! !

! LuaBoot methodsFor: #lua-private !
linkNative_to_Implementation
	" Private - Implementation of private function. "

	self comment: 'link native species of instance to delegate to aClass instance';cr.
	self local: #mt with: [self call: #getmetatable with: #instance];cr.
	self if: #mt do: [
		self local: #index with: 'mt["__index"]';cr.
		self call: #setmetatable withArguments: #(index 'aClass["$fn"].prototype');cr.
	] else: [
		self if: #debug do: [
			self send: #do:onError: withArguments: (Array
				with: [self rawFunction: '' arguments: #() doing: [
					self	call: 'debug.setmetatable'
						withArguments: #(instance 'aClass["$fn"].prototype')
				]]
				with: [self rawFunction: '' arguments: #() doing: [
					self callPrint: [self concat: '"-- ERROR cant set metatable for "'
						with: [self call: #tostring with: #instance]]
				]])
		] else: [ self callPrint: '"-- WARNING: debug support missing"' ]
	] ! !

! LuaBoot methodsFor: #lua-private !
newArrayImplementation
	" Private - Implementation of private function. "

	self comment: 'return a new Array instance mutating the provided object';cr.
	self return: [ self send: #will:be: withArguments: (Array
		with: [self expr: #object or: [self emptyObject]]
		with: '"Array"')] ! !

! LuaBoot methodsFor: #lua-private !
become_a_Implementation
	" Private - Implementation of private function. "

	self comment: 'mutate object to an instance of aClass';cr.
	self call: #setmetatable withArguments: #(object 'aClass["$fn"].prototype');cr.
	self return: #object ! !

! LuaBoot methodsFor: #lua-private !
setProtoOf_to_Implementation
	" Private - Implementation of private function. "

	self local: #mt with: [self emptyObject];cr.
	self assign: #mt property: #__index to: #proto;cr.
	self call: #setmetatable withArguments: #(nativeObject mt)! !

! LuaBoot methodsFor: #lua-private !
bindRootsImplementation
	" Private - Implementation of private function. "

	self comment: 'Private - Link the system roots to ensure proper delegation chains';cr.
	self local: #objectClassPrototype with: 'self.Object["$klass"]["$fn"].prototype';cr.
	self send: #setProtoOf:to: withArguments: #(
		objectClassPrototype
		'self.Class["$fn"].prototype');cr.
	self send: #setProtoOf:to: withArguments: #(
		'self.Smalltalk["$klass"]["$fn"].prototype'
		objectClassPrototype);cr.
	self send: #setProtoOf:to: withArguments: #(
		'self.Behavior["$klass"]["$fn"].prototype'
		objectClassPrototype);cr.
	self assign: 'self.Object["$klass"]["$superclass"]' to: 'self.Class';cr.
	self send: #become:a: withArguments: #(
		smalltalk
		'self.Smalltalk')! !

! LuaBoot methodsFor: #lua-private !
klassImplementation
	" Private - Implementation of private function. "

	self local: #aClass;cr.
	self if: 'spec.meta' do: [
		self assign: #aClass to: [self call: #StMetaclass]
	] else: [
		self local: #metaSpec with: [self object: (Array with: #meta -> #true)];cr.
		self assign: #metaSpec property: #className to: [self concat: 'spec.className' with: '" class"'];cr.
		self if: 'spec.superclass' do: [
			self assign: #metaSpec property: #superclass to: 'spec.superclass["$klass"]'
		];cr.
		self local: #metaclass with: [ self send: #klass with: #metaSpec];cr.
		self assign: #aClass to: [self call: 'metaclass["$fn"]'];cr.
		self if: [self expr: 'aClass["$klass"]' noteq: #metaclass] do: [
			self comment: 'this should NOT happen';cr.
			self callPrint: '"--!! "..(spec.className).." BROKEN metaclass="..tostring(aClass["$klass"])';cr.
			self assign: 'aClass["$klass"]' to: #metaclass
		]
	];cr.
	self send: #setClass withArguments: #(aClass spec);cr.
	self assign: 'aClass["$fn"].prototype["$klass"]' to: #aClass;cr.
	self assign: 'aClass["$fn"].prototype["$methods"]' to: [self emptyObject];cr.
	self if: [self expr: 'aClass["$className"]' eqeq: '"Metaclass"'] do: [
		self send: #bindRoots with: #aClass
	];cr.
	self return: #aClass ! !

! LuaBoot methodsFor: #lua-private !
setVarNamesImplementation
	" Private - Implementation of private function. "

	self if: [self expr: #iVarNames eqeq: #nil] do: [
		self assign: #iVarNames to: [self send: #newArray ]
	];cr.
	self assign: 'aClass["$iVarNames"]' to: #iVarNames;cr.
	self local: #i with: '#iVarNames';cr.
	self while: 'i > 0' do: [
		self assign: 'aClass["$fn"].prototype["@"..iVarNames[i]]' to: #aValue;cr.
		self assign: #i to: 'i - 1'
	]. ! !

! LuaBoot methodsFor: #lua-private !
setClassImplementation
	" Private - Implementation of private function. "

	self assign: 'aClass["$className"]' to: 'spec.className';cr.
	self if: [self expr: 'spec.meta' eqeq: #nil] do: [
		self assign: 'aClass["$klass"]["@instanceClass"]' to: #aClass
	];cr.
	self assign: 'aClass["$category"]' to: [self expr: 'spec.category' or: self emptyString];cr.
	self assign: 'aClass["$superclass"]' to: 'spec.superclass --or nil';cr.
	self if: 'aClass["$superclass"]' do: [
		self assign: 'aClass["$superclass"]["@subclasses"]' to: #nil;cr.
		self if: 'aClass["$klass"]' do: [
			self if: [self expr: 'spec.meta'
				and: [self expr: 'aClass["$klass"]' noteq: 'self.Metaclass']] do: [
				self assign: 'aClass["$klass"]["$superclass"]' to: 'aClass["$superclass"]["$klass"]'
			];cr.
			self if: [self expr: 'self.Metaclass' noteq: #nil] do: [ 
				self comment: 'OBSOLETE - Metaclass validation & repair';cr.
				self if: [self expr: 'self.Metaclass["$superclass"]' noteq: 'self.Behavior'] do: [
					self callPrint: '"-- Warning broken Metaclass spec.meta="..tostring(spec.meta).." aClass="..aClass["$className"]';cr.
					self assign: 'self.Metaclass["$superclass"]' to: 'self.Behavior'
				].
			];cr.
			self if: 'aClass["$klass"]["$superclass"]' do: [
				self assign: 'aClass["$klass"]["$superclass"]["@subclasses"]' to: #nil
			].
		].
	];cr.
	self assign: 'aClass["$fn"]' to: [self send: #fnInSpec with: #spec];cr.
	self send: #setVarNames withArguments: #(aClass 'spec.iVarNames' #nil);cr.
	self return: #aClass ! !

! LuaBoot methodsFor: #lua-private !
fnInSpecImplementation
	" Private - Implementation of private function. "

	self if: [self expr: 'spec.meta' and: [self expr: 'spec.superclass' eqeq: #nil]] do: [
		self assign: 'spec.superclass' to: 'self.Class'
	];cr.
	self if: 'spec.fn' do: [
		self if: 'spec.superclass' do: [
			self send: #setProtoOf:to: withArguments: #(
				'spec.fn.prototype' 'spec.superclass["$fn"].prototype'
			)
		];cr.
		self return: 'spec.fn'
	];cr.
	self local: #superC with: #nil;cr.
	self if: 'spec.superclass' do: [
		self assign: #superC to: 'spec.superclass["$fn"]'
	];cr.
	self return: [self send: #newConstructor with: #superC] ! !

! LuaBoot methodsFor: #implementations !
systemDictionaryAPI
	" Private - Internal encoding for SystemDictionary messages. "

	#(	#classes
		#computeSubclasses
		#(subclasses #(aClass))
		#(allSubclassesDo #(aClass aBlock))
		#(mapClassName #(className category fn superclass))
		#(isArray:equalTo: #(table1 table2))
		#(isObject:equalTo: #(receiver anObject))
		#(object:comma: #(receiver anObject))
		#(boolean:and: #(receiver option))
		#(boolean:or: #(receiver option))
		#(booleanNot: #(receiver))
		#(times:repeat: #(count aBlock))
		#(if:nil:notNil: #(anObject block1 block2))
		#(if:true:false: #(aBoolean blk1 blk2))
		#(while:do: #(condition aBlock))
		#(whileNot:do: #(condition aBlock))
		#(asCharacter: #(receiver))
		#(toString: #(receiver))
		#(printString: #(receiver))
		#(asString: #(receiver))
		#(mustBeNumber: #(receiver))
		#(math:max: #(receiver aNumber))
		#(math:min: #(receiver aNumber))
		#(addClass #(className superclass iVarNames category))
		#(renameClass #(aClass aString))
		#(removeClass #(aClass))
		#(metaVarNames #(aMetaClass iVarNames))
		#(smDo #(block))
		#(classChanged #(aBehavior))
		#(classShapeChanged #(aBehavior))
		#(aboutToCreateClass #(className superclass iVarNames category))
		#(aboutToChangeClass #(aClass superclass iVarNames category))
		#(methodBoundTo #(method klass))
		#(aboutToBindTo #(method klass))
		#(aboutToRemoveFrom #(method klass))
		#(methodRemovedFrom #(method klass))
		#(objectNotFound #(object indexOrKey collection))
		#(whoImplements #(klass jsSelector))
		#(superImplementor #(klass jsSelector))
		#(addMethod #(jsSelector method klass))
		#(bind #(klass stSelector jsSelector jsFunction category stSource nativeSource stamp))
		#(removeMethod #(aClass aMethod))
		#(newMethod #(currentClass selector fn source nativeSource stamp))
"OBSOLETE	#(method #(spec))"
		#(readJSObject #(js))
		#(basen #(n b))
		#(globalsDo #(action))
		#(launchImage #(imageName))
		#(is:kindOf: #(anObject aClass))
		#(responds:to: #(anObject aSelector))
		#(debugBind #(klass stSelector jsSelector jsFunction category stSource nativeSource stamp))
		#(debugAddClass #(className superclass iVarNames category))
		#toString
		#bindCore
		#startup
	) do: [:tuple| self bindAPI: tuple ]! !

! LuaBoot methodsFor: #api !
classesImplementation
	" Private - Implementation of API function. "

	self local: #result with: [self send: #newArray];cr.
	self for: 'key, aClass' in: [self call: #pairs with: #smalltalk] do: [
		self if: [self typeOf: #aClass is: #table] do: [
			self if: [self expr: 'aClass.isClass' and: [self send: #isClass to: #aClass]] do: [
				self assign: 'result[#result+1]' to: #aClass
			]
		]
	];cr.
	self return: #result ! !

! LuaBoot methodsFor: #api !
computeSubclassesImplementation
	" Private - Implementation of API function. "

	self if: [self expr: 'self.Metaclass' eqeq: #nil] do: [
		self callPrint: '"-- Warning system broken/incomplete"'
	];cr.
	self if: [self expr: 'self.Metaclass["$superclass"]' noteq: 'self.Behavior'] do: [
		self comment: 'OBSOLETE - Metaclass validation & repair';cr.
		self callPrint: '"-- Warning broken Metaclass"';cr.
		self assign: 'self.Metaclass["$superclass"]' to: 'self.Behavior'
	];cr.
	self local: #classes with: [self send: #classes];cr.
	self for: '_,cls' in: [self call: #pairs with: #classes] do: [
		self assign: 'cls["@subclasses"]' to: [self send: #newArray];cr.
		self assign: 'cls["$klass"]["@subclasses"]' to: [self send: #newArray];cr.
	];cr.
	self for: '_,cls' in: [self call: #pairs with: #classes] do: [
		self if: [self expr: 'cls["$superclass"]' noteq: #nil] do: [
			self local: #array with: 'cls["$superclass"]["@subclasses"]';cr.
			self assign: 'array[#array+1]' to: #cls;cr.
			self assign: #array to: 'cls["$superclass"]["$klass"]["@subclasses"]';cr.
			self assign: 'array[#array+1]' to: 'cls["$klass"]'
		]
	];cr.
	self if: 'self.Class' do: [
		self assign: 'self.Class["@subclasses"]' to: [self send: #newArray];cr.
		self assign: 'self.Class["$klass"]["@subclasses"]' to: [self send: #newArray];cr.
	] ! !

! LuaBoot methodsFor: #api !
subclassesImplementation
	" Private - Implementation of API function. "

	self if: [self expr: 'aClass["@subclasses"]' eqeq: #nil] do: [
		self send: #computeSubclasses
	];cr.
	self return: 'aClass["@subclasses"]' ! !

! LuaBoot methodsFor: #api !
allSubclassesDoImplementation
	" Private - Implementation of API function. "

	self for: '_,cls' in: [self call: #pairs with: [self send: #subclasses with: #aClass]] do: [
		self call: #aBlock with: #cls;cr.
		self send: #allSubclassesDo withArguments: #(cls aBlock)
	]! !

! LuaBoot methodsFor: #api !
mapClassNameImplementation
	" Private - Implementation of API function. "

	self local: #spec with: [self object:
		#className -> #className
		,(#category -> #category) 
		,(#superclass -> #superclass)
		,(#fn -> #fn)];cr.
	self if: 'self[className]' do: [
		self return: [self send: #setClass withArguments: #('self[className]' spec)]
	];cr.
	self assign: 'self[className]' to: [self send: #klass with: #spec];cr.
	self return: 'self[className]'! !

! LuaBoot methodsFor: #api !
isArray_equalTo_Implementation
	" Private - Implementation of API function. "

	self if: [self expr: #table1 eqeq: #table2] return: #true;cr.
	self if: [self expr: '#table1' noteq: '#table2'] return: #false;cr.
	self local: #i with: '#table1';cr.
	self while: 'i > 0' do: [
		self if: [self expr: 'table1[i]' noteq: 'table2[i]'] return: #false;cr.
		self assign: #i to: 'i - 1'
	];cr.
	self return: #true ! !

! LuaBoot methodsFor: #api !
isObject_equalTo_Implementation
	" Private - Implementation of API function. "

	self if: [self expr: #receiver eqeq: #anObject] return: #true;cr.
	self if: [self expr: [self typeOf: #receiver is: #table]
		 and: [ self typeOf: #anObject is: #table]] do: [
		self if: 'receiver["_eq"]' do: [
			self return: [self send: #= to: #receiver with: #anObject]
		].
	];cr.
	self return: #false ! !

! LuaBoot methodsFor: #api !
object_comma_Implementation
	" Private - Implementation of API function. "

	self if: [self typeOf: #receiver is: #string] do: [
		self if: [self typeOf: #anObject is: #string] return: [self concat: #receiver with: #anObject];cr.
		self if: [self typeOf: #anObject is: #number] return: [self concat: #receiver
			with: [self call: #tostring with: #anObject]];cr.
		self return: [self send: #object:comma: withArguments: (Array with: #receiver
			with: [self send: #toString: to: #receiver with: #anObject])]
	];cr.
	self if: [self typeOf: #receiver is: #number] do: [
		self return: [self send: #object:comma:
			withArguments: (Array with: [self call: #tostring with: #receiver] with: #anObject)]
	];cr.
	self if: [self not: [self typeOf: #receiver is: #table]] do: [
		self return: [self send: #object:comma: withArguments: (Array
				with: [self send: #toString: with: #receiver]
				with: #anObject)]
	];cr.
	self return: [self send: #, to: #receiver with: #anObject] ! !

! LuaBoot methodsFor: #api !
boolean_and_Implementation
	" Private - Implementation of API function. "

	self if: [self expr: #receiver noteq: #true] return: #false;cr.
	self if: [self typeOf: #option is: #function] return: [self call: #option];cr.
	self if: [self typeOf: #option is: #boolean] return: #option;cr.
	self return: [self expr: [self send: #evaluate to: #option] eqeq: #true]! !

! LuaBoot methodsFor: #api !
boolean_or_Implementation
	" Private - Implementation of API function. "

	self if: [self expr: #receiver eqeq: #true] return: #true;cr.
	self if: [self typeOf: #option is: #function] return: [self call: #option];cr.
	self if: [self typeOf: #option is: #boolean] return: #option;cr.
	self return: [self expr: [self send: #evaluate to: #option] eqeq: #true]! !

! LuaBoot methodsFor: #api !
booleanNot_Implementation
	" Private - Implementation of API function. "

	self if: [self expr: #receiver eqeq: #true] return: #false;cr.
	self if: [self typeOf: #receiver isNot: #table] return: 'not receiver';cr.
	self return: [self send: '__not' to: #receiver]! !

! LuaBoot methodsFor: #api !
times_repeat_Implementation
	" Private - Implementation of API function. "

	self for: 'i=1,count' do: [
		self call: #aBlock with: #i
	];cr.
	self return: #count! !

! LuaBoot methodsFor: #api !
if_nil_notNil_Implementation
	" Private - Implementation of API function. "

	self if: [self send: #isNil: with: #anObject] do: [
		self return: [self send: #evaluate to: #block1]
	];cr.
	self return: [self send: #evaluate to: #block2] !

! LuaBoot methodsFor: #api !
if_true_false_Implementation
	" Private - Implementation of API function. "

	self if: [self expr: #aBoolean eqeq: #true] do: [
		self if: #blk1 return: [self call: #blk1]
	] else: [
		self if: #blk2 return: [self call: #blk2]
	];cr.
	self return: #nil ! !

! LuaBoot methodsFor: #api !
while_do_Implementation
	" Private - Implementation of API function. "

	self while: [self expr: [self call: #condition] eqeq: #true] do: [
		self if: #aBlock do: [ self call: #aBlock ]
	];cr.
	self return: #nil ! !

! LuaBoot methodsFor: #api !
whileNot_do_Implementation
	" Private - Implementation of API function. "

	self while: [self expr: [self call: #condition] eqeq: #false] do: [
		self if: #aBlock do: [ self call: #aBlock ]
	];cr.
	self return: #nil ! !

! LuaBoot methodsFor: #api !
asCharacter_Implementation
	" Private - Implementation of API function. "

	self if: [self typeOf: #receiver is: #number] do: [
		self return: [self call: 'string.char' with: #receiver]
	];cr.
	self return: [self send: #asCharacter to: #receiver]! !

! LuaBoot methodsFor: #api !
toString_Implementation
	" Private - Implementation of API function. "

	self local: #aType with: [self call: #type with: #receiver];cr.
	self if: [self expr: #aType eqeq: '"string"'] return: #receiver;cr.
	self if: [self expr: #aType eqeq: '"number"'] return: [self call: #tostring with: #receiver];cr.
	self if: [self expr: #aType eqeq: '"boolean"'] do: [
		self if: #receiver return: '"true"';cr.
		self return: '"false"'
	];cr.
	self if: [self typeOf: 'receiver.toString' is: #function]
		return: [self send: #toString to: #receiver ];cr.
	self return: [self call: #tostring with: #receiver] ! !

! LuaBoot methodsFor: #api !
printString_Implementation
	" Private - Implementation of API function. "

	self if: [self typeOf: #receiver is: #string] return: [self send: #printString to: #receiver];cr.
	self if: [self expr: [self typeOf: #receiver is: #table]
		and: 'receiver.printString'] return: [self send: #printString to: #receiver];cr.
	self return: [self call: #tostring with: #receiver] ! !

! LuaBoot methodsFor: #api !
asString_Implementation
	" Private - Implementation of API function. "

	self if: [self expr: [self typeOf: #receiver is: #table]
		and: 'receiver.asString'] return: [self send: #asString to: #receiver];cr.
	self return: [self call: #tostring with: #receiver] ! !

! LuaBoot methodsFor: #api !
mustBeNumber_Implementation
	" Private - Implementation of API function. "

	self if: [self typeOf: #receiver is: #number] return: #receiver;cr.
	self if: [self typeOf: #receiver is: #table] return: [self send: #mustBeNumber to: #receiver];cr.
	self return: [self send: #error: with: '"Must be a Number"'] ! !

! LuaBoot methodsFor: #api !
math_max_Implementation
	" Private - Implementation of API function. "

	self if: [self typeOf: #receiver isNot: #number] return: [self send: #max: to: #receiver with: #aNumber];cr.
	self if: [self typeOf: #aNumber isNot: #number] return: [self send: #max: to: #aNumber with: #receiver];cr.
	self if: 'receiver>aNumber' return: #receiver;cr.
	self return: #aNumber ! !

! LuaBoot methodsFor: #api !
math_min_Implementation
	" Private - Implementation of API function. "

	self if: [self typeOf: #receiver isNot: #number] return: [self send: #min: to: #receiver with: #aNumber];cr.
	self if: [self typeOf: #aNumber isNot: #number] return: [self send: #min: to: #aNumber with: #receiver];cr.
	self if: 'receiver<aNumber' return: #receiver;cr.
	self return: #aNumber ! !

! LuaBoot methodsFor: #api !
addClassImplementation
	" Private - Implementation of API function. "

	self send: #debugAddClass withArguments: #(className superclass iVarNames category);cr.
	self if: 'self[className]' do: [
		self local: #aClass with: 'self[className]';cr.
		self local: #sameShape with: [
			self expr: [self expr: 'aClass["$superclass"]' eqeq: #superclass]
			      and: [ self send: #isArray:equalTo: withArguments: #('aClass["$iVarNames"]' iVarNames)
			]];cr.
		self if: [self expr: #sameShape and: [
			self expr: 'aClass["$category"]' eqeq: #category]] do: [
			self return: #aClass
		];cr.
		self send: #aboutToChangeClass withArguments: #(aClass superclass iVarNames category);cr.
		self if: 'aClass["$superclass"]' do: [
			self assign: 'aClass["$superclass"]["@subclasses"]' to: #nil;cr.
			self local: #aMeta with: 'aClass["$klass"]';cr.
			self if: [self expr: #aMeta and: #superclass] do: [
				self assign: 'aMeta["$superclass"]["@subclasses"]' to: #nil;cr.
				self assign: 'aMeta["$superclass"]' to: 'superclass["$klass"]'
			]
		];cr.
		self assign: 'aClass["$superclass"]' to: #superclass;cr.
		self if: [self expr: #superclass and: 'superclass["@subclasses"]'] do: [
			self if: [self expr: 'superclass["@subclasses"]' eqeq: #nil] do: [
				self assign: 'superclass["@subclasses"]' to: [self send: #newArray]
			];cr.
			self call: 'superclass["@subclasses"].push' with: #aClass
		];cr.
		self send: #setVarNames withArguments: #(aClass iVarNames #nil);cr.
		self assign: 'aClass["$category"]' to: [self expr: #category or: 'aClass["$category"]'];cr.
		self if: [self expr: #sameShape eqeq: #nil] do: [
			self send: #classShapeChanged with: #aClass
		]
	] else: [
		self send: #aboutToCreateClass withArguments: #(className superclass iVarNames category);cr.
		self assign: 'self[className]' to: [ self send: #klass with: [
			self object: #className -> #className
				,(#iVarNames -> #iVarNames)
				,(#superclass -> #superclass)
		]];cr.
		self assign: 'self[className]["$category"]' to: [self expr: #category or: self emptyString]
	];cr.
	self send: #classChanged with: 'self[className]';cr.
	self return: 'self[className]' ! !

! LuaBoot methodsFor: #api !
renameClassImplementation
	" Private - Implementation of API function. "

	self assign: 'self[aClass["$className"]]' to: #nil;cr.
	self assign: 'aClass["$className"]' to: #aString;cr.
	self assign: 'self[aString]' to: #aClass;cr.
	self send: #classChanged with: #aClass ! !

! LuaBoot methodsFor: #api !
removeClassImplementation
	" Private - Implementation of API function. "

	self local: #deleted with: '"Deleted-"';cr.
	self if: 'aClass["$superclass"]' do: [
		self assign: 'aClass["$superclass"]["@subclasses"]' to: #nil
	];cr.
	self assign: 'self[aClass["$className"]]' to: #nil;cr.
	self assign: 'aClass["$className"]'
		to: [self concat: 'deleted ' with: 'aClass["$className"]'];cr.
	self assign: 'aClass["$klass"]["$className"]'
		to: [self concat: 'deleted ' with: 'aClass["$klass"]["$className"]'];cr.
	self send: #classChanged with: #aClass ! !

! LuaBoot methodsFor: #api !
metaVarNamesImplementation
	" Private - Implementation of API function. "

	self if: [self send: #isArray:equalTo: withArguments: #('aMetaClass["$iVarNames"]' iVarNames)] return: #false;cr.
	self send: #setVarNames withArguments: #(aMetaClass iVarNames #nil);cr.
	self send: #classShapeChanged with: #aMetaClass;cr.
	self send: #classChanged with: #aMetaClass;cr.
	self return: #true ! !

! LuaBoot methodsFor: #api !
smDoImplementation
	" Private - Implementation of API function. "

	self if: [self expr: [self expr: 'self.useManager' and: 'self.SystemManager']
		and: 'self.SystemManager.currentOrNil'] do: [
		self local: #sm with: [self send: #currentOrNil to: 'self.SystemManager'];cr.
		self if: #sm return: [self call: #block with: #sm]
	]! !

! LuaBoot methodsFor: #api !
classChangedImplementation
	" Private - Implementation of API function. "

	self send: #smDo with: [self rawFunction: '' arguments: #(sm) doing: [
		self send: #classChanged: to: #sm with: #aBehavior
	]]! !

! LuaBoot methodsFor: #api !
classShapeChangedImplementation
	" Private - Implementation of API function. "

	self send: #smDo with: [self rawFunction: '' arguments: #(sm) doing: [
		self send: #classShapeChanged: to: #sm with: #aBehavior
	]]! !

! LuaBoot methodsFor: #api !
aboutToCreateClassImplementation
	" Private - Implementation of API function. "

	self send: #smDo with: [self rawFunction: '' arguments: #(sm) doing: [
		self send: #aboutToCreateClass:superclass:iVarNames:category:
			to: #sm withArguments: #(className superclass iVarNames category)
	]]! !

! LuaBoot methodsFor: #api !
aboutToChangeClassImplementation
	" Private - Implementation of API function. "

	self send: #smDo with: [self rawFunction: '' arguments: #(sm) doing: [
		self send: #aboutToChange:superclass:iVarNames:category:
			to: #sm withArguments: #(aClass superclass iVarNames category)
	]]! !

! LuaBoot methodsFor: #api !
methodBoundToImplementation
	" Private - Implementation of API function. "

	self send: #smDo with: [self rawFunction: '' arguments: #(sm) doing: [
		self send: #methodBound:to: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method! !

! LuaBoot methodsFor: #api !
aboutToBindToImplementation
	" Private - Implementation of API function. "

	self send: #smDo with: [self rawFunction: '' arguments: #(sm) doing: [
		self send: #aboutToBind:to: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method! !

! LuaBoot methodsFor: #api !
aboutToRemoveFromImplementation
	" Private - Implementation of API function. "

	self send: #smDo with: [self rawFunction: '' arguments: #(sm) doing: [
		self send: #aboutToRemove:from: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method! !

! LuaBoot methodsFor: #api !
methodRemovedFromImplementation
	" Private - Implementation of API function. "

	self send: #smDo with: [self rawFunction: '' arguments: #(sm) doing: [
		self send: #methodRemoved:from: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method! !

! LuaBoot methodsFor: #api !
objectNotFoundImplementation
	" Private - Implementation of API function. "

	self send: #smDo with: [self rawFunction: '' arguments: #(sm) doing: [
		self send: #object:at:notFoundIn: to: #sm withArguments: #(object indexOrKey collection)
	]];cr.
	self return: #collection! !

! LuaBoot methodsFor: #api !
whoImplementsImplementation
	" Private - Implementation of API function. "

	self local: #cls with: #klass;cr.
	self repeat: [
		self if: [self call: #rawget withArguments: #('cls["$fn"].prototype' jsSelector)] return: #cls;cr.
		self assign: #cls to: 'cls["$superclass"]'
	] until: [self expr: #cls eqeq: #nil];cr.
	self return: #nil ! !

! LuaBoot methodsFor: #api !
superImplementorImplementation
	" Private - Implementation of API function. "

	self if: [self expr: [self send: #isNil: with: 'klass["$superclass"]']
		or: [ self send: #isNil: with: 'klass["$superclass"]["$fn"]']] do: [
		self return: [self send: #missingImplementorFor: to: #klass with: #jsSelector]
	];cr.
	self local: #cls with: [self send: #whoImplements withArguments: #('klass["$superclass"]' jsSelector)];cr.
	self if: [self send: #isNil: with: #cls] do: [
		self return: [self send: #missingImplementorFor: to: 'klass["$superclass"]' with: #jsSelector]
	];cr.
	self return: 'cls["$fn"].prototype[jsSelector]' ! !

! LuaBoot methodsFor: #api !
addMethodImplementation
	" Private - Implementation of API function. "

	self send: #aboutToBindTo withArguments: #(method klass);cr.
	self assign: 'klass["$fn"].prototype[jsSelector]' to: 'method["$fn"]';cr.
	self assign: 'klass["$fn"].prototype["$methods"][method["$selector"]]' to: #method;cr.
	self assign: 'method["@classField"]' to: #klass;cr.
	self return: [self send: #methodBoundTo withArguments: #(method klass)] ! !

! LuaBoot methodsFor: #api !
bindImplementation
	" Private - Implementation of API function. "

	self send: #debugBind withArguments: #(
		klass stSelector jsSelector jsFunction
		category stSource nativeSource
		stamp);cr.
	self local: #method with: [self call: 'self.CompiledMethod["$fn"]'];cr.
	self assign: 'method["$selector"]' to: #stSelector;cr.
	self assign: 'method["$fn"]' to: #jsFunction;cr.
	self assign: 'method["$category"]' to: #category;cr.
	self assign: 'method["$source"]' to: #stSource;cr.
	self assign: 'method["$nativeSource"]' to: #nativeSource;cr.
	self if: [self send: #notNil: with: #stamp] do: [
		self assign: 'method["$stamp"]' to: #stamp
	];cr.
	self assign: 'method["$generation"]' to: 'self["$state"]';cr.
	self if: [self send: #notNil: with: #klass] do: [
		self if: [self expr: #jsSelector eqeq: #0] do: [
			self assign: #jsSelector to: [
				self	call: 'string.gsub'
					withArguments: #(stSelector '":"' '"_"') ]
		];cr.
		self send: #addMethod withArguments: #(jsSelector method klass)
	];cr.
	self return: #method ! !

! LuaBoot methodsFor: #api !
removeMethodImplementation
	" Private - Implementation of API function. "

	self send: #aboutToRemoveFrom withArguments: #(aMethod aClass);cr.
	self assign: 'aClass["$fn"].prototype[aMethod["$selector"]:asSelector()]' to: #nil;cr.
	self assign: 'aClass["$fn"].prototype["$methods"][aMethod["$selector"]]' to: #nil;cr.
	self return: [self send: #methodRemovedFrom withArguments: #(aMethod aClass)] ! !

! LuaBoot methodsFor: #api !
newMethodImplementation
	" Private - Implementation of API function. "

	self return: [self send: #bind withArguments: #(
		#nil selector #nil fn
		#nil source nativeSource stamp)] ! !

! LuaBoot methodsFor: #api !
methodImplementation
	" Private - Implementation of API function. "

	self comment: #OBSOLETE;cr.
	self return: [self send: #bind withArguments: #(
		#nil 'spec.selector' #nil 'spec.fn' 'spec.category'
		'spec.source' 'spec.nativeSource' 'spec.stamp')] ! !

! LuaBoot methodsFor: #api !
readJSObjectImplementation
	" Private - Implementation of API function. "

	self callPrint: '"--[NIY] smalltalk:readJSObject()"';cr.
	self return: #nil ! !

! LuaBoot methodsFor: #api !
basenImplementation
	" Private - Implementation of API function. "

	self local: 'floor,insert' with: 'math.floor, table.insert';cr.
	self assign: #n to: [self call: #floor with: #n];cr.
	self if: 'not b or b == 10' return: [self call: #tostring with: #n];cr.
	self local: #digits with: '"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"';cr.
	self local: #t with: [self emptyObject];cr.
	self local: #sign with: self emptyString;cr.
	self if: 'n < 0' do: [
		self assign: #sign to: '"-"';cr.
		self assign: #n to: '-n'
	];cr.
	self repeat: [
		self local: #d with: '(n % b) + 1';cr.
		self assign: #n to: [self call: #floor with: 'n / b'];cr.
		self call: #insert withArguments: #(t #1 'digits:sub(d,d)');cr.
	] until: [self expr: #n eqeq: #0];cr.
	self return: [self concat: #sign with: [
		self call: 'table.concat' withArguments: #(t '""')
	]]! !

! LuaBoot methodsFor: #api !
globalsDoImplementation
	" Private - Implementation of API function. "

	self for: 'key, aValue' in: [self call: #pairs with: #smalltalk] do: [
		self if: [self typeOf: #aValue is: #table] do: [
			self if: [self expr: 'aValue.isClass'
				and: [self not: [self send: #isClass to: #aValue]]]
				do: [ self call: #action with: #key ]
		];cr.
	];cr.
	self return: #smalltalk ! !

! LuaBoot methodsFor: #api !
launchImageImplementation
	" Private - Implementation of API function. "

	self assign: 'self.useManager' to: #true;cr.
	self if: [self send: #notNil: with: #imageName] do: [
		self assign: 'self["$imageFileName"]' to: #imageName;cr.
		self call: #dofile with: #imageName; comment: 'load(imageName)'
	];cr.
	self assign: 'self["$state"]' to: '"launching"';cr.
	self if: 'self.launch' do: [
		self send: #launch
	];cr.
	self assign: 'self["$state"]' to: '"running"';cr.
	self return: #smalltalk ! !

! LuaBoot methodsFor: #api !
is_kindOf_Implementation
	" Private - Implementation of API function. "

	self if: [self expr: [self send: #isNil: with: #anObject]
		or: [self send: #isNil: with: 'anObject.isKindOf_']] return: #false;cr.
	self return: [self send: #isKindOf: to: #anObject with: #aClass] ! !

! LuaBoot methodsFor: #api !
responds_to_Implementation
	" Private - Implementation of API function. "

	self if: [self expr: [self send: #isNil: with: #anObject]
		or: [self send: #isNil: with: 'anObject.respondsTo_']] return: #false;cr.
	self return: [self send: #respondsTo: to: #anObject with: #aSelector] ! !

! LuaBoot methodsFor: #api !
debugBindImplementation
	" Private - Implementation of API function. "

	self return: #self ! !

! LuaBoot methodsFor: #api !
debugAddClassImplementation
	" Private - Implementation of API function. "

	self return: #self ! !

! LuaBoot methodsFor: #api !
toStringImplementation
	" Private - Implementation of API function. "

	self return: '"smalltalk"' ! !

! LuaBoot methodsFor: #api !
bindCoreImplementation
	" Private - Implementation of API function. "

	self comment: 'bind the minimal functions (e.g i/o)';cr.
	self local: #global with: #_G;cr.
	self if: [self expr: #global eqeq: #nil] return: #false;cr.
	self rawFunction: 'global.quit' arguments: #() doing: [
		self call: 'os.exit';cr.
		self return: #false
	];cr.
	self rawFunction: 'global.version' arguments: #() doing: [
		self return: #_VERSION
	];cr.
	self rawFunction: 'global.fileContents' arguments: #(fileName) doing: [
		self local: 'f,err' with: [self call: 'io.open' withArguments: #(fileName '"rb"')];cr.
		self if: [self expr: #f eqeq: #nil] return: #nil;cr.
		self local: #content with: [self send: #read to: #f with: '"*all"'];cr.
		self send: #close to: #f;cr.
		self return: #content
	];cr.
	self assign: 'global.read' to: 'global.fileContents';cr.
	self rawFunction: 'global.fileWrite' arguments: #(fileName contents mode) doing: [
		self local: #f with: [self call: #assert with: [self call: 'io.open' withArguments: #(fileName mode)]];cr.
		self send: #write to: #f with: #contents;cr.
		self send: #close to: #f;cr.
		self return: [self send: #size to: #contents]
	];cr.
	self rawFunction: 'global.fileIn' arguments: #(fileName) doing: [
		self local: #contents with: [self call: #read with: #fileName];cr.
		self if: [self expr: #contents eqeq: #nil] do: [
			self send: #error: to: #smalltalk with: [
				self	concat: '"Missing file "'
					with: [self call: #tostring with: #fileName]
			]
		];cr.
		self return: [self send: #fileIn to: #contents]
	];cr.
	self rawFunction: 'global.emmit' arguments: #(fileName) doing: [
		self return: [self send: #emmit: to: #smalltalk with: #fileName]
	];cr.
	self rawFunction: 'global.fileInJS' arguments: #(fileName) doing: [
		self return: [self send: #eval: to: #smalltalk
			with: [self call: #read with: #fileName]]
	];cr.
	self rawFunction: 'global.exportCategory' arguments: #(aString) doing: [
		self return: [self send: #exportCategory: to: 'smalltalk.Exporter' with: #aString]
	];cr.
	self rawFunction: 'global.saveImage' arguments: #() doing: [
		self return: [self send: #saveImage to: #smalltalk ]
	];cr.
	self rawFunction: 'global.saveImageAs' arguments: #(fileName) doing: [
		self return: [self send: #saveImageAs: to: #smalltalk with: #fileName]
	];cr.
	self rawFunction: 'global.isFileOlderThan' arguments: #(fileName referenceFileName) doing: [
		self local: 'f,err' with: [self call: 'io.open' withArguments: #(fileName '"r"')];cr.
		self if: [self expr: #f eqeq: #nil] return: #false;cr.
		self send: #close to: #f;cr.
		self assign: 'f,err' to: [self call: 'io.open' withArguments: #(referenceFileName '"r"')];cr.
		self if: [self expr: #f eqeq: #nil] return: #false;cr.
		self send: #close to: #f;cr.
		self comment: 'there is NO way to get the datetime of a file in Lua';cr.
		self return: #true
	];cr.
	self assign: 'self["$imageFileName"]' to: [self expr: 'self["$imageFileName"]' or: '"s8.image.lua"'];cr.
	self return: #true ! !

! LuaBoot methodsFor: #api !
startupImplementation
	" Private - Implementation of API function. "

	self assign: 'self["$state"]' to: '"booting"';cr.
	self assign: 'self.OverwriteMethods' to: [self expr: 'self.OverwriteMethods' or: #true];cr.
	self assign: 'self.DumpOverwritenMethods' to: [self expr: 'self.DumpOverwritenMethods' or: #false];cr.
	self assign: 'self["$state"]' to: '"singularity"';cr.
	self send: #mapClassName withArguments: #('"Object"' '"Kernel"' StObject);cr.
	self send: #mapClassName withArguments: #('"Smalltalk"' '"Kernel"' StSystemDictionary 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Behavior"' '"Kernel"' StBehavior 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Class"' '"Kernel"' StClass 'self.Behavior');cr.
	self send: #mapClassName withArguments: #('"Metaclass"' '"Kernel"' StMetaclass 'self.Behavior');cr.
	self send: #mapClassName withArguments: #('"CompiledMethod"' '"Kernel"' StMethod 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Number"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"BlockClosure"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Boolean"' '"Kernel"' #nil 'self.Object');cr.
	self comment: 'self:mapClassName("DateTime","Kernel",Date,self.Object)';cr.
	self send: #mapClassName withArguments: #('"UndefinedObject"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Collection"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"SequenceableCollection"' '"Kernel"' #nil 'self.Collection');cr.
	self send: #mapClassName withArguments: #('"String"' '"Kernel"' #nil 'self.SequenceableCollection');cr.
	self send: #mapClassName withArguments: #('"Array"' '"Kernel"' #nil 'self.SequenceableCollection');cr.
	self comment: 'self:mapClassName("RegularExpression","Kernel",RegExp,self.Object)';cr.
	self assign: 'self["$state"]' to: '"whiteHole"';cr.
	self send: #linkNative:to: withArguments: (Array with: self emptyString with: 'self.String');cr.
	self send: #linkNative:to: withArguments: #('function()end' 'self.BlockClosure');cr.
	self send: #linkNative:to: withArguments: #(#true 'self.Boolean');cr.
	self send: #linkNative:to: withArguments: #(#nil 'self.UndefinedObject');cr.
	self send: #linkNative:to: withArguments: #(#1 'self.Number');cr.
	self assign: 'self["$state"]' to: '"wormHole"';cr.
	self send: #bindCore;cr.
	self assign: 'self["$state"]' to: '"startup"';cr.
	self return: #self! !
