BootSector
	subclass: #PythonBoot
	category: #pyBoot !

! PythonBoot class methodsFor: #instantiation !
supportsLanguage: aLanguage
	" Private - Return true if instances of the receiver can encode to aLanguage. "

	^#py = aLanguage! !

! PythonBoot methodsFor: #constants !
version
	" Private - Returns the version string (dotted number and date). "

	^'1.0 - Stable'! !

! PythonBoot methodsFor: #encoding !
lineComment: aString
	" Private - Encode aString as line comment. "

	^self nextPutAll: '# ' ,aString;cr ! !

! PythonBoot methodsFor: #encoding !
multilineComment: array
	" Private - Encode an array as a multiline comment. "

	^self encode: [
		self nextPutAll: '"""' ;indent.
		array do: [:line| self cr; nextPutAll: line ].
		self dedent; cr; nextPutAll: '"""'; cr
	]! !

! PythonBoot methodsFor: #contents !
startupExpression
	" Private - Return the encoding for startup expression of the receiver. "

	^self encode: [
		self assign: 'smalltalk, nil' to: [ self send: #startup to: #smalltalk ] ;cr
	]! !

! PythonBoot methodsFor: #definitions !
nilDefinition
	" Private - Return the encoding for nil. "

	^self encode: [
		self assign: #nil to: #None ;cr
	]! !

! PythonBoot methodsFor: #definitions !
smalltalkDefinition
	" Private - Return the encoding for smalltalk. "

	^self encode: [
		#(	#(StObject #(
				#(__init__ #('aHandle=nil'))
				asParameter handle halt klass
				toString __str__ __repr__
			))
			#(#(SuperForClass #(object)) #(
				#(__init__ #( type 'obj=None'))
				#(__get__ #( obj 'type=None'))
				#(__getattr__ #(attr))
			))
			#(#(StMetaType #(type)) #(
				#(lookup:in: #(selector aClass))
				#(__getattribute__ #(item))
			))
			#(StSystemDictionary #(
				__init__
				#(__setitem__ #(item value))
				#(superReceiver #(implementor receiver))
				#(linkRoute #(receiver parent))
				#(mapClassName #(className category 'native=nil' 'superclass=nil'))
				#(available #('*modules'))
				#(moduleAt #(module field))
				#(state: #(aState))
				#(do:onError: #(aBlock errorBlock))
				#(is_Nil #(aValue))
				#(isNil: #(aValue))
				#(not_Nil #(aValue))
				#(notNil: #(aValue))
				#(stringOf #(nativeString))
				#(b #(aBoolean))
				#(b_ #(aBool))
				#(asObject: #(aNativeObject))
				#(asNativeValue: #(anObject))
				#(nativeCall #(anObject selector '*args'))
				#(blk #(func 'argumentCount=nil' 'info=nil'))
				#(setVarNames #(aClass iVarNames 'aValue=nil'))
				#(setPropertyNames #(aClass pNames 'aValue=nil'))
				#(setClass #(aClass spec))
				#(fnInSpec #(spec))
				classes
				#(computeMetaSubclassesOf: #(aClass))
				computeSubclasses
				#(subclasses #('aClass=nil'))
				#(allSubclassesDo #(aClass aBlock))
				#(bindRoots #('ignored=None'))
				#(isArray_equalTo #(table1 table2))
				#(is_Number #(x))
				#(isObject_equalTo #(receiver anObject))
				#(isObject:equalTo: #(receiver anObject))
				#(isObject_eqeq #(receiver anObject))
				#(isObject:eqeq: #(receiver anObject))
				#(object:comma: #(receiver anObject))
				#(basic_at_ifAbsent #(receiver key aBlock))
				#(basic:at:ifAbsent: #(receiver key aBlock))
				#(basic:at: #(receiver key))
				#(basic_at_put #(receiver key value))
				#(basic:at:put: #(receiver key value))
				#(callBlock0: #(aBlock 'noBlockValue=nil'))
				#(ifUndefined:do: #(receiver aBlock))
				#(times:repeat: #(count aBlock))
				#(isBoolean: #(receiver))
				 #(boolean:and: #(receiver option))
				 #(boolean:or: #(receiver option))
				#(booleanNot: #(receiver))
				#(asCharacter: #(receiver))
				#(to_String #(receiver))
				#(printString: #(receiver))
				#(asString: #(receiver))
				#(mustBeNumber: #(receiver))
				#(math:max: #(receiver aNumber))
				#(math:min: #(receiver aNumber))
				#(addClass #(className superclass 'iVarNames=[]' 'category=""'))
				#(renameClass #(aClass aString))
				#(removeClass #(aClass))
				#(metaVarNames #(aMetaClass iVarNames))
				#(smDo #(block))
				#(classChanged #(aBehavior))
				#(classShapeChanged #(aBehavior))
				#(aboutToCreateClass #(className superclass iVarNames category))
				#(aboutToChangeClass #(aClass superclass iVarNames category))
				#(methodBoundTo #(method klass))
				#(aboutToBindTo #(method klass))
				#(aboutToRemoveFrom #(method klass))
				#(methodRemovedFrom #(method klass))
				#(objectNotFound #(object indexOrKey collection))
				#(whoImplements #(klass jsSelector))
				#(superImplementor #(klass jsSelector))
				#(instVarAt #(object ivName 'default=nil'))
				#(fn_Of #(klass))
				#(prototype_Of #(klass))
				#(methods_Of #(klass))
				#(register:method:in: #(stSelector aMethod klass))
				#(unregister:in: #(stSelector klass))
				#(addMethod #(jsSelector method klass 'stSelector=nil'))
				#(bind #(klass stSelector jsSelector jsFunction category stSource 'nativeSource=nil' 'stamp=nil'))
				#(removeMethod #(aClass aMethod))
				#(newMethod #(currentClass selector fn source 'nativeSource=nil' 'stamp=nil'))
"OBSOLETE			#(method #(spec))"
				#(readJSObject #(js))
				#(basen #(n bas 'numerals="0123456789abcdefghijklmnopqrstuvwxyz"'))
				#(globalsDo #(action))
				#(launchImage #('imageName=nil'))
				#(is:kindOf: #(anObject aClass))
				#(responds_to #(anObject jsSelector))
				#(responds:to: #(anObject aSelector))
				#(isObject #(anObject))
				#(handleOf #(anObject))
				#(handleOf:or: #(anObject default))
				newHash newContext stObject
				#(debugBind #(klass stSelector jsSelector jsFunction category stSource 'nativeSource=nil' 'stamp=nil'))
				#(debugAddClass #(className superclass iVarNames category))
				toString quit version
				#(httpContents #(fileName))
				#(folderContents #(fileName))
				#(fileContents #(fileName))
				"#(read #(fileName))"
				#(fileWrite #(fileName contents mode))
				#(fileIn #(fileName))
				#(emmit #(fileName))
				#(fileInJS #(fileName))
				#(exportCategory #(aString))
				saveImage
				#(saveImageAs #(fileName))
				#(isFileOlderThan #(fileName referenceFileName))
				#(copyTo #(anInstance))
				startup
				niy
				#(niy: #(msg))
				#(invalidMessage: #(msg))
				obsolete

				#(toObjects #(list))
				#(reshape:of: #(field anObject))
				#(reshapeClass: #(aClass))
				reshapeClasses
				#(linkNative:to: #(instance aClass))
				#(arguments:toCall: #(args aBlock))
				#(objects:toCall: #(anArray aBlock))
				#(makeCallable: #(aClass))
				#(makeStringable: #(aClass))
				linkNatives
				bindCore

				#(newArray #(object))
				#(newConstructor #(superC))
				#(will:be: #(anObject className))
				#(become:a: #(anObject aClass))
				#(setProtoOf:to: #(nativeObject proto))
				#(klass #(spec))
				#(__f__ #(source 'tag=nil'))
				#(import: #(libraryName))
				#(from:import: #(libraryName aName))
"OBSOLETE			#(__b__OBSOLETE #(source locals))"
				#(if:nil:notNil: #(anObject block1 block2))
				#(if:true:false: #(aBoolean blk1 blk2))
				#(while:do: #(condition aBlock))
				#(whileNot:do: #(condition aBlock))
				#(raise:result: #(_s8Ret_ result))
				#(exceptionResult: #(__ex))
				#(exception_at #(__ex target))

				#(value:withTimeout: #(aBlock aNumber))
				#(value:withInterval: #(aBlock aNumber))
				#(argumentCount: #(aCallable))
				#(pool:at:put: #(pool aKey aValue))
				#(pool:removeKey: #(pool aKey))
				#(pool:at:ifAbsent: #(pool aKey aBlock))
				#(pool:includesKey: #(pool aKey))
				#(pool_keys #(pool))
				#(arrayHasNativeContents #(anArray))
				#(arrayContents #(anArray))
				#(arrayHandles #(anArray))
				#(array:at: #(anArray anIndex))
				#(array:at:ifAbsent: #(anArray anIndex aBlock))
				#(array:at:put: #(anArray anIndex anObject))
				#(array:join: #(anArray aString))
				#(array:copyWithAll: #(receiver anArray))
				#(array:sort: #(anArray aBlock))
				#(array:remove:ifAbsent: #(receiver anObject aBlock))
				#(array:atAllPut: #(receiver anObject))
				#(hashedSet:privateAdd: #(aSet cookie))
				#(hashedSet:privateRemove: #(aSet cookie))
				#(string:isLiteral: #(receiver))
				#(string:match #(receiver aRegexp))
				#(string:occurrencesOf: #(receiver aString))
				#(string:startsWith: #(receiver aString))
				#(string:asNumber: #(receiver))
				#(string:asPrivateSelector: #(receiver))

				outerFramesCount
				asPoolDictionary
				#(debug: #(aBoolean))
				startDebug
				debug

				testAll
				#(test #(what))
				#(testFrom #(first 'last=1000'))
				#(t #(number))
			))
		) do: [:tuple| self defineClass: tuple first implementing: tuple last;cr ].
		self define: #smalltalk asInstanceOf: #SystemDictionary
	]! !

! PythonBoot methodsFor: #encoding !
define: aName asInstanceOf: clsName
	" Private - Return the encoding for defining a variable named aName. "

	self assign: aName to: #St ,clsName ,'()'; cr ! !

! PythonBoot methodsFor: #encoding !
defineClass: header implementing: functions
	" Private - pyClasss definition. "

	header isString ifTrue: [
		^self	defineClass: (Array with: header with: #())
			implementing: functions
	].
	self nextPutAll: #class; space.
	self nextPutAll: header first.
	self pars: [ self nextPutArguments: header last ].
	self between: #($: '') do: [
		functions do: [:spec| self bind: header first api: spec ]
	] indent: true! !

! PythonBoot methodsFor: #api !
bind: className api: spec
	" Private - Encoding for basic message implementation. "

	| rcv selector action |
	spec isString ifTrue: [ ^self bind: className api: (Array with: spec with: #()) ].
	spec last isArray ifFalse: [
		^self error: 'Invalid/corrupt API spec ' ,spec asLiteral
	].
	selector := spec first asSelector ,className.
	action := [ rcv perform: selector ].
	rcv := (self respondsTo: selector) ifTrue: [ self ] ifFalse: [
		(self class respondsTo: selector) ifTrue: [ self class ] ifFalse: [
			" self error: 'Missing API implementation for ' ,selector asLiteral "
			action := [ self comment: '*ToDo* - API ' ,selector asLiteral ,' Not Implemented Yet.' ]
		]
	].
	self bind: spec first arguments: spec last doing: action ! !

! PythonBoot methodsFor: #asm !
implementFunction: selector arguments: args doing: implementation comment: aComment
	" Private - Encoding for internal method implementation. "

	self nextPutAll: #def;space; nextPutAll: selector asSelector.
	self pars: [ self nextPutArguments: args ].
	self between: #($: '') do: implementation indent: true comment: aComment! !

! PythonBoot methodsFor: #asm !
bind: selector arguments: args doing: implementation comment: aComment
	" Private - Encoding for internal method implementation. "

	^self	implementFunction: selector arguments: #(self) ,args
		doing: implementation comment: aComment ! !

! PythonBoot methodsFor: #asm !
with: expression as: vars do: aBlock
	" Private - Encoding for with expression. "

	self	nextPutAll: #with;space;
		nextPutAll: expression;space;
		nextPutAll: #as;space;
		nextPutAll: vars;space;
		between: self expressionsOPs do: aBlock indent: true! !

! PythonBoot methodsFor: #asm-constants !
blockOPs
	" Private - Encoding for expression block OPs "

	^Array with: $: with: '' ! !

! PythonBoot methodsFor: #asm-constants !
objectOPs
	" Private - Return the OPs for literal object definition. "

	^'{}' ! !

! PythonBoot methodsFor: #asm-constants !
fieldOP
	" Private - Return the OP for tuple separator. "

	^$: ! !

! PythonBoot methodsFor: #asm-constants !
andOP
	" Private - Encoding for operand. "

	^#and ! !

! PythonBoot methodsFor: #asm-constants !
orOP
	" Private - Encoding for operand. "

	^#or ! !

! PythonBoot methodsFor: #asm-constants !
notOP
	" Private - Return the OP for negation. "

	^'not ' ! !

! PythonBoot methodsFor: #asm-constants !
expressionPars
	" Private - Encoding for expression (or nil) "

	^nil ! !

! PythonBoot methodsFor: 'asm-constants' !
conditionPars: expression
	" Private - Encoding for condition expression(or nil) "

	expression isString ifTrue: [ ^nil ].
	^'()' "self expressionPars" ! !

! PythonBoot methodsFor: 'asm-constants' !
opPars: expression
	" Private - Encoding for expression in operation(or nil) "

	^'()' ! !

! PythonBoot methodsFor: #asm-constants !
whileDoOPs
	" Private - Return the OPs for while-do expression. "

	^Array with: #while with: '' ! !

! PythonBoot methodsFor: #asm-constants !
forDoOPs
	" Private - Return the OPs for for-do expression. "

	^Array with: #for with: ''! !

! PythonBoot methodsFor: #asm-constants !
forInDoOPs
	" Private - Return the OPs for for-in-do expression. "

	^Array with: #for with: #in with: ''! !

! PythonBoot methodsFor: #asm-constants !
forInDoPars
	" Private - Return the parentesis for for-in-do expression. "

	^Array with: '' with: ''! !

! PythonBoot methodsFor: #asm !
assign: obj property: key to: expression
	" Private - Encoding for assignment to object property. "

	self assign: obj ,$. ,key to: expression ! !

! PythonBoot methodsFor: #asm !
if: condition return: expression
	" Private - Encoding for conditional expression return. "

	self if: condition do: [self return: expression]! !

! PythonBoot methodsFor: #asm !
return: first and: second
	" Private - Encoding for returning two expressions. "

	self return: first; space; nextPutAll: $,; nextPutAll: second ! !

! PythonBoot methodsFor: #asm !
expressions: array
	" Private - Encoding for arrayed expressions. "

	self indent.
	array do: [:each| self nextPutAll: each ] separatedBy: [self nextPutAll: $\;cr].
	self dedent.! !

! PythonBoot methodsFor: #asm !
dot: expression
	" Private - Encoding for linked expression. "

	self nextPutAll: $.; nextPutAll: expression! !

! PythonBoot methodsFor: #asm !
expr: first cr: second
	" Private - Encoding for arrayed expressions. "

	self expressions: (Array with: first with: second)! !

! PythonBoot methodsFor: #asm !
expr: first cr: second cr: third
	" Private - Encoding for arrayed expressions. "

	self expressions: (Array with: first with: second with: third)! !

! PythonBoot methodsFor: #asm !
expr: first plus: second
	" Private - Encoding for sum expression. "

	self	nextPutAll: first; space;
		nextPutAll: #+; space;
		nextPutAll: second ! !

! PythonBoot methodsFor: #asm !
expr: first plus: second plus: third
	" Private - Encoding for arrayed sum expressions. "

	self expressions: (Array with: first
		with: [self nextPutAll: #+; nextPutAll: second]
		with: [self nextPutAll: #+; nextPutAll: third])! !

! PythonBoot methodsFor: #asm !
raise: what
	" Private - Encoding for raising exceptions. "

	self nextPutAll: #raise; space; nextPutAll: what ! !

! PythonBoot methodsFor: #asm !
format: aString withArguments: args
	" Private - Encoding for traditional format string expressions. "

	self nextPutAll: aString.
	self space; nextPutAll: #%; space.
	self pars: [ self nextPutArguments: args ] ! !

! PythonBoot methodsFor: #asm !
sum: array
	" Private - Encoding for sum expressions inlined. "

	array do: [:each| self nextPutAll: each ] separatedBy: [self nextPutAll: $+] ! !

! PythonBoot methodsFor: #asm !
property: aName of: rcv
	" Private - Encoding for accessing a property. "

	self nextPutAll: rcv.
	self nextPutAll: $..
	self nextPutAll: aName ! !

! PythonBoot methodsFor: #asm !
is: what in: container
	" Private - Encoding for in expression. "

	self nextPutAll: what.
	self space; nextPutAll: #in;space.
	self nextPutAll: container ! !

! PythonBoot methodsFor: #asm !
pass
	" Private - Encoding for pass. "

	self nextPutAll: #pass ! !

! PythonBoot methodsFor: #asm !
import: what
	" Private - Encoding for import declaration. "

	self nextPutAll: #import; space.
	self nextPutAll: what ! !

! PythonBoot methodsFor: #asm !
from: library import: what
	" Private - Encoding for from-import declaration. "

	self nextPutAll: #from; space.
	self nextPutAll: library; space.
	self import: what ! !

! PythonBoot methodsFor: #asm !
expr: rcv lt: expression
	" Private - Compare expression. "

	self	expr: rcv op: #< with: expression
		pars: #(#('' '') #('' ''))! !

! PythonBoot methodsFor: #asm !
expr: rcv lteq: expression
	" Private - Compare expression. "

	self	expr: rcv op: #<= with: expression
		pars: #(#('' '') #('' ''))! !

! PythonBoot methodsFor: #asm !
expr: rcv gt: expression
	" Private - Compare expression. "

	self	expr: rcv op: #> with: expression
		pars: #(#('' '') #('' ''))! !

! PythonBoot methodsFor: #asm !
expr: rcv less: expression
	" Private - Decrement expression. "

	self	expr: rcv op: #- with: expression
		pars: #(#('' '') #('' ''))! !

! PythonBoot methodsFor: #asm !
expr: rcv gteq: expression
	" Private - Compare expression. "

	self	expr: rcv op: #>= with: expression
		pars: #(#('' '') #('' ''))! !

! PythonBoot methodsFor: #asm !
try: block except: tags as: varName doing: exBlock
	" Private - Encoding for try/except expression. "

	self between: #('try:' except) do: block indent: true.
	self space; nextPutAll: tags.
	varName notNil ifTrue: [
		self nextPutAll: ' as '; nextPutAll: varName
	].
	self between: (Array with: $: with: nil) do: exBlock indent: true!

! PythonBoot methodsFor: #asm !
expr: expr1 is: expr2
	" Private - Encoding for is not expression. "

	self expr: expr1 op: #is with: expr2! !

! PythonBoot methodsFor: #asm !
expr: expr1 isNot: expr2
	" Private - Encoding for is not expression. "

	self expr: expr1 op: 'is not' with: expr2! !

! PythonBoot methodsFor: #asm !
expr: expr1 in: expr2
	" Private - Encoding for is not expression. "

	self expr: expr1 op: #in with: expr2! !

! PythonBoot methodsFor: #asm-constants !
typeofOPs
	" Private - Return the OP for typeOf expression. "

	^#('type(' $)) ! !

! PythonBoot methodsFor: #asm-constants !
typeofArg: token
	" Private - Return the OP for typeOf argument. "

	^token ! !

! PythonBoot methodsFor: #asm !
lambda: arg expr: expression
	" Private - Encoding for lambda expression. "

	self nextPutAll: #lambda; space.
	self nextPutAll: arg; space.
	self nextPutAll: $:;space.
	self nextPutAll: expression! !

! PythonBoot methodsFor: #asm !
pointerTo: expression
	" Private - Encoding for pointer to expression. "

	self nextPutAll: $*; nextPutAll: expression ! !

! PythonBoot methodsFor: #api !
__b__OBSOLETEStSystemDictionary
	" Private - Implementation of api function "

	self call: #exec withArguments: (Array
		with: '"def __b__"+source'
		with: [self call: #globals]
		with: #locals).
	self call: #setattr withArguments: #(
		'locals["__b__"]'
		'"$nativeSource"'
		source).
	self return: 'locals["__b__"]' ! !

! PythonBoot methodsFor: #api !
__f__StSystemDictionary
	" Private - Implementation of api function "

	self if: [self send: #is_Nil with: #tag]
		do: [ self assign: #fName to: '"__f__"' ]
		else: [ self assign: #fName to: #tag ];cr.
	self comment: 'print("def "+fName,source)'.
	self assign: #locals to: [self call: #dict];cr.
	self try: [
		self call: #exec withArguments: (Array
			with: [self sum: #('"def "' fName source)]
			with: [self call: #globals]
			with: #locals)
	] except: #Exception as: #ex doing: [
		self call: #print withArguments: (Array with: [self sum: #('"def "' fName)] with: #source);cr.
		self call: #print with: #ex
	];cr.
	self assign: #delim to: $' ,'"""' ,$';cr.
	self if: [self is: #delim in: #source] do: [
		self assign: #delim to: $" ,$' ,$' ,$' ,$"
	];cr.
	self assign: #source to: [self send: #replace to: #source withArguments: #('"\\"' '"\\\\"')];cr.
	self assign: #nativeSource to: [self
		sum: #(	'"smalltalk.__f__("'
			delim source delim
			),(Array with: $' ,$, ,$" ,$'
			with: [self call: #str with: #tag]
			with: $' ,$" ,$) ,$')
	];cr.
	self call: #setattr withArguments: #('locals[fName]' '"$nativeSource"' nativeSource);cr.
	self return: 'locals[fName]' ! !

! PythonBoot methodsFor: #api !
__get__SuperForClass
	" Private - Implementation of api function "

	self 	if: [self expr: [self expr: 'self.__obj__' is: #None]
			and: [self expr: #obj isNot: #None]]
		do: [self return: [self call: #super withArguments: #('self.__type__' obj)]]
		else: [self return: #self] ! !

! PythonBoot methodsFor: #api !
lookup_in_StMetaType
	" Private - Implementation of api function "

	self comment: 'print("[look]",selector,aClass,"meta=",getattr(self,"$meta","missing"))'.
	self try: [
		self assign: #result to: [self call: #getattr withArguments: #(self selector)];cr.
		self comment: 'print("[look]local",selector,result)'.
		self	if: [self call: #callable with: #result]
			return: [self send: #__get__ to: 'result.__func__' with: #aClass ].
		self return: #result
	] except: #AttributeError as: #ex doing: [
		self comment: 'print("[look]missing@",item ,"ex=",ex.args)'.
		self return: #None
	]! !

! PythonBoot methodsFor: #api !
__getattr__SuperForClass
	" Private - Implementation of api function "

	self try: [
		self return: [self call: #getattr withArguments: (Array
			with: [self call: #super withArguments: #('self.__type__' 'self.__obj__')]
			with: #attr)]
	] except: #AttributeError as: #ex doing: [
		self comment: 'print("[SuperForClass]missing@",attr ,"ex=",ex.args)'.
		self pass
	];cr.
	self if: [self call: #hasattr withArguments: #('smalltalk.Class' attr)] do: [
		self comment: 'print("[SuperForClass]found as Class@",attr)'.
		self assign: #theAttr to: [self call: #getattr withArguments: #('smalltalk.Class' attr)];cr.
		self comment: 'print("[SuperForClass] got ",theAttr," binding to ",self.__obj__)'.
		self return: [self send: #__get__ to: 'theAttr.__func__' with: 'self.__obj__']
	];cr.
	self raise: [self call: #AttributeError with: [
		self format: '"%s has no attribute %s."'
			withArguments: #('self.__type__' attr)]
	] ! !

! PythonBoot methodsFor: #api !
__getattribute__StMetaType
	" Private - Implementation of api function "

	self if: [self expr: [self expr:
		[self expr: 'item[0:2]' eqeq: '"__"']
		or: [self expr: 'item[0]' eqeq: '"$"']]
		or: [self expr: 'item[0]' eqeq: '"@"']]
	do: [
		self if: [self expr: #item eqeq: '"__class__"'] return: 'smalltalk.Metaclass'.
		self return: [self send: #__getattribute__ to: [self call: #super] with: #item]
	];cr.

	self comment: 'print("[meta]"+str(self)+"@"+item,"meta=",getattr(self,"$meta","missing"))'.
	self try: [
		self assign: #result to: [self send: #__getattribute__ to: [self call: #super] with: #item];cr.
		self comment: 'print("[meta]super",item)'.
		self return: [self send: #__get__ to: #result with: #self ]
	] except: #AttributeError as: #ex doing: [
		self comment: 'print("[meta]@",item ,"missing ex=",ex.args)'.
		self assign: #result to: #None
	];cr.

	self assign: #parent to: '"Metaclass"';cr.
	self assign: #delegate to: [self call: #getattr withArguments: #(smalltalk parent None)];cr.
	self if: #delegate do: [
		self assign: #delegate to: [self call: #delegate with: #None];cr.
		self comment: 'print("[meta]as"+parent)'.
		self assign: #result to: [self call: #getattr withArguments: #(delegate item None)];cr.
		self if: [self call: #callable with: #result] do: [
			self assign: #result to: 'result.__func__';cr.
			self comment: 'print("[meta]found("+parent+"@"+item+") src=",result.__method__.src())'.
			self comment: 'print("[meta]calling ",result.__get__(self))'.
			self return: [self send: #__get__ to: #result with: #self]
		]
	];cr.

	self assign: #parent to: '"Class"';cr.
	self assign: #delegate to: [self call: #getattr withArguments: #(smalltalk parent None)];cr.
	self if: #delegate do: [
		self assign: #delegate to: [self call: #delegate with: #None];cr.
		self comment: 'print("[meta]as"+parent)'.
		self assign: #result to: [self call: #getattr withArguments: #(delegate item None)];cr.
		self if: [self call: #callable with: #result] do: [
			self assign: #result to: 'result.__func__';cr.
			self comment: 'print("[meta]found("+parent+"@"+item+") src=",result.__method__.src())'.
			self comment: 'print("[meta]calling ",result.__get__(self))'.
			self return: [self send: #__get__ to: #result with: #self]
		]
	];cr.
	self raise: [self call: #AttributeError with: [
		self format: '"%s has no attribute %s."'
			withArguments: #('self.__name__' item)
	]]! !

! PythonBoot methodsFor: #api !
__init__StObject
	" Private - Implementation of api function "

	self	if: [self send: #not_Nil to: #smalltalk with: #aHandle]
		do: [self call: #setattr withArguments: #(self '"@handle"' aHandle)] ! !

! PythonBoot methodsFor: #api !
__init__StSystemDictionary
	" Private - Implementation of api function "

	self call: #setattr withArguments: #(self '"$nativeLanguage"' '"py"');cr.
	self assign: #self property: #useManager to: #False;cr.
	self send: #state: with: '"__init__"';cr.
	self comment: 'self.st = self #OBSOLETE' ! !

! PythonBoot methodsFor: #api !
__init__SuperForClass
	" Private - Implementation of api function "

	self comment: 'print("SuperForClass[init] type=",type,"obj=",obj)'.
	self assign: #self property: #__type__ to: #type;cr.
	self assign: #self property: #__obj__ to: #obj ! !

! PythonBoot methodsFor: #api !
__repr__StObject
	" Private - Implementation of api function "

	self assign: #h to: [self call: #getattr withArguments: #(self '"@handle"' None)];cr.
	self if: [self send: #is_Nil to: #smalltalk with: #h] return: [
		self send: #__repr__ to: [self call: #super]
	].
	self return: [self expr: '"smalltalk."'
		plus: [self send: #handleOf to: #smalltalk with: [
			self call: #getattr withArguments: #('self.__class__' '"$className"' '"???"')
		]] plus: [self sum: (Array with: '"("'
			with: [self call: #repr with: #h]
			with: '")"' )]
		] ! !

! PythonBoot methodsFor: #api !
__setitem__StSystemDictionary
	" Private - Implementation of api function "

	self call: #setattr withArguments: #(self item value)! !

! PythonBoot methodsFor: #api !
__str__StObject
	" Private - Implementation of api function "

	self if: [self expr: #self is: #nil] do: [self return: '"nil"'].
	self assign: #h to: [self call: #getattr withArguments: #(self '"@handle"' None)];cr.
	self if: [self expr: [self typeOf: #h is: #str]
		 or: [self send: #is_Number to: #smalltalk with: #h]]
		return: [self call: #str with: #h ].
	self if: [self typeOf: #h is: #bool] do: [
		self if: [self expr: #h eqeq: #True] return: '"true"'.
		self return: '"false"'
	];cr.
	self return: [self send: #__str__ to: [self call: #super]]! !

! PythonBoot methodsFor: #api !
aboutToBindToStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self lambda: #sm expr: [
		self send: #aboutToBind:to: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method ! !

! PythonBoot methodsFor: #api !
aboutToChangeClassStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self lambda: #sm expr: [
		self send: #aboutToChange:superclass:iVarNames:category: to: #sm
			withArguments: #(aClass superclass iVarNames category)
	]];cr ! !

! PythonBoot methodsFor: #api !
aboutToCreateClassStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self lambda: #sm expr: [
		self send: #aboutToCreateClass:superclass:iVarNames:category: to: #sm
			withArguments: #(className superclass iVarNames category)
	]];cr ! !

! PythonBoot methodsFor: #api !
aboutToRemoveFromStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self lambda: #sm expr: [
		self send: #aboutToRemove:from: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method ! !

! PythonBoot methodsFor: #api !
addClassStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Register a Class onto the system'.
	self send: #debugAddClass withArguments: #(className superclass iVarNames category);cr.
	self assign: #aClass to: [self call: #getattr withArguments: #(self className #nil)];cr.
	self if: [self send: #not_Nil with: #aClass] do: [
		self assign: #parent to: [self call: #getattr withArguments: #(aClass '"$superclass"' #nil)];cr.
		self if: [self send: #is_Nil with: #parent] do: [
			self assign: #parent to: #nil
		] else: [
			self comment: 'invalidate subclasses of parent'.
			self call: #setattr withArguments: #(parent '"@subclasses"' #nil);cr.
			self call: #setattr withArguments: #('parent.__class__' '"@subclasses"' #nil)
		];cr.
		self  if: [self send: #is_Nil with: #superclass] do: [
			self assign: #superclass to: #nil
		];cr.
		self assign: #sameShape to: [self expr: 
			[self expr: #superclass eqeq: #parent]
			and: [self send: #isArray_equalTo withArguments: (Array
				with: [self call: #getattr withArguments: #(aClass '"$iVarNames"' '[]')]
				with: #iVarNames)]];cr.
		self if: [self expr: #sameShape and: [
			self expr: #category eqeq: [
				self call: #getattr withArguments: #(aClass '"$category"' #nil)]
			]] do: [
			self send: #reshapeClass: with: #aClass;cr.
			self return: #aClass
		];cr.
		self send: #aboutToChangeClass withArguments: #(aClass superclass iVarNames category);cr.
		self if: [self not: [self expr: #parent is: #superclass]] do: [
			self comment: 'move in hierarchy'.
			self comment: 'link to new parent'.
			self call: #setattr withArguments: #(aClass '"$superclass"' superclass).
			self comment: 'do we need to move in native hierarchy?'.
			self assign: #aMeta to: 'aClass.__class__';cr.
			self if: [self expr: #aMeta and: #superclass] do: [
				self assign: #msuper to: [self call: #getattr withArguments: #(aMeta '"$superclass"')];cr.
				self call: #setattr withArguments: #(msuper '"@subclasses"' #nil);cr.
				self call: #setattr withArguments: #(aMeta '"$superclass"' 'superclass.__class__')
			];cr.
			self comment: 'invalidate subclasses of superclass'.
			self call: #setattr withArguments: #(superclass '"@subclasses"' #nil);cr.
			self call: #setattr withArguments: #('superclass.__class__' '"@subclasses"' #nil)
		];cr.
		self call: #setattr withArguments: (Array with: #aClass with: '"$category"'
			with: [self expr: #category
				or: [self call: #getattr withArguments: #(aClass '"$category"' '""')]]);cr.
		self send: #setVarNames withArguments: #(aClass iVarNames #nil);cr.
		self if: [self not: #sameShape] do: [ self send: #classShapeChanged with: #aClass]
	] else: [
		self send: #aboutToCreateClass withArguments: #(className superclass iVarNames category);cr.
		self send: #mapClassName withArguments: #(className category #nil superclass);cr.
		self assign: #aClass to: [self call: #getattr withArguments: #(self className)];cr.
		self send: #setVarNames withArguments: #(aClass iVarNames #nil);cr.
		self if: [self send: #not_Nil with: #superclass] do: [
			self comment: 'invalidate subclasses of superclass'.
			self call: #setattr withArguments: #(superclass '"@subclasses"' #nil);cr.
			self call: #setattr withArguments: #('superclass.__class__' '"@subclasses"' #nil)
		]
	];cr.
	self send: #classChanged with: #aClass;cr.
	self send: #reshapeClass: with: #aClass;cr.
	self return: #aClass ! !

! PythonBoot methodsFor: #api !
addMethodStSystemDictionary
	" Private - Implementation of api function "

	self send: #aboutToBindTo withArguments: #(method klass);cr.
	self assign: #fn to: [self send: #instVarAt withArguments: #(method '"fn"') ];cr.
	self comment: 'setattr(fn,"$s8-method",method) # needed? for back refs to method'.
	self assign: 'fn.__method__' to: #method;cr.
	self call: #setattr withArguments: (Array
		with: #klass
		with: [self send: #handleOf with: #jsSelector]
		with: #fn);cr.
	self call: #setattr withArguments: #(method '"@classField"' klass);cr.
	self call: #setattr withArguments: #(method '"$fn"' fn);cr.
	self if: [self expr: [self send: #is_Nil with: #stSelector] or: [self not: #stSelector]] do: [
		self assign: #stSelector to: [self send: #asNativeSelector to: #jsSelector]
	];cr.
	self send: #register:method:in: withArguments: #(stSelector method klass);cr.
	self return: [self send: #methodBoundTo withArguments: #(method klass)] ! !

! PythonBoot methodsFor: #api !
allSubclassesDoStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Evaluates aBlock with all the subclasses of aClass'.
	self implementFunction: #__blk1__ arguments: #(cls) doing: [
		self call: #aBlock with: #cls;cr.
		self send: #allSubclassesDo withArguments: #(cls aBlock)
	] comment: nil;cr.
	self send: #do:
		to: [self send: #subclasses with: #aClass]
		with: [self send: #blk withArguments: #(__blk1__ #1)];cr.
	self return: #self ! !

! PythonBoot methodsFor: #api !
argumentCount_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self send: #debug to: #smalltalk] do: [
		self call: #print withArguments: (Array
			with: '"@argumentCount - "'
			with: [self call: #getattr
				withArguments: #(aCallable '"@argumentCount"' #nil)])
	];cr.
	self assign: #count to: [self call: #getattr
		withArguments: #(aCallable '"@argumentCount"' #nil)];cr.
	self if: [self expr: #count is: #nil] do: [
		self try: [
			self from: #inspect import: #signature;cr.
			self assign: #f to: [self send: #handleOf with: #aCallable];cr.
			self assign: #count to: [self call: #len with: [
				self property: #parameters of: [self call: #signature with: #f]
			]]
		] except: #Exception as: #ex doing: [
			self return: #nil
		]
	];cr.
	self return: [self call: 'self.Number' with: #count] ! !

! PythonBoot methodsFor: #api !
arguments_toCall_StSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Returns arguments adapted to call aBlock'.
	self if: [self send: #debug to: #smalltalk] do: [
		self call: #print withArguments: #('"arguments_toCall_ - "' args)
	];cr.
	self assign: #count to: [self send: #handleOf with: [
		self send: #argumentCount: with: #aBlock]
	];cr.
	self if: [self send: #is_Nil with: #count] return: #args.
	self if: [self expr: #count eqeq: [self call: #len with: #args]] return: #args.
	self assign: #largs to: [self send: #arrayContents with: 'list(args)[:count]'];cr.
	self while: [self expr: [self call: #len with: #largs] lt: #count] do: [
		self send: #append to: #largs with: #nil
	];cr.
	self return: [self call: #tuple with: #largs] ! !

! PythonBoot methodsFor: #api !
arrayContentsStSystemDictionary
	" Private - Implementation of api function "

	self assign: #result to: '[]';cr.
	self for: #each in: [self call: #list with: [self send: #handleOf with: #anArray]] do: [
		self send: #append to: #result with: [self send: #asObject: with: #each]
	];cr.
	self return: #result ! !

! PythonBoot methodsFor: #api !
arrayHandlesStSystemDictionary
	" Private - Implementation of api function "

	self assign: #result to: '[]';cr.
	self for: #each in: [self call: #list with: [self send: #handleOf with: #anArray]] do: [
		self send: #append to: #result with: [self send: #handleOf with: #each]
	];cr.
	self return: #result ! !

! PythonBoot methodsFor: #api !
arrayHasNativeContentsStSystemDictionary
	" Private - Implementation of api function "

	self for: #each in: [self send: #handleOf with: #anArray] do: [
		self if: [self send: #isObject with: #each] do: [
			self return: #False
		] else: [ self return: #True ]
	];cr.
	self return: #False ! !

! PythonBoot methodsFor: #api !
array_atAllPut_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #array to: [self send: #handleOf with: #receiver];cr.
	self assign: #o to: [self send: #asNativeValue: with: #anObject];cr.
	self for: #i in: [self call: #range with: [self call: #len with: #array]] do: [
		self assign: 'array[i]' to: #o
	]! !

! PythonBoot methodsFor: #api !
array_at_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #array to: [self send: #handleOf with: #anArray];cr.
	self assign: #offset to: [self expr: [self send: #handleOf with: #anIndex] less: #1];cr.
	self if: [	self expr: [self expr: #offset lt: #0]
		or: [self expr: [self call: #len with: #array] lteq: #offset]]
		return: [self send: #objectNotFoundAt: to: #anArray with: #anIndex ].
	self return: [self send: #asObject: with: 'array[offset]']! !

! PythonBoot methodsFor: #api !
array_at_ifAbsent_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #array to: [self send: #handleOf with: #anArray];cr.
	self assign: #offset to: [self expr: [self send: #handleOf with: #anIndex] less: #1];cr.
	self if: [	self expr: [self expr: #offset lt: #0]
		or: [self expr: [self call: #len with: #array] lteq: #offset]]
		return: [self send: #callBlock0: with: #aBlock ].
	self return: [self send: #asObject: with: 'array[offset]']! !

! PythonBoot methodsFor: #api !
array_at_put_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #array to: [self send: #handleOf with: #anArray];cr.
	self assign: #offset to: [self expr: [self send: #handleOf with: #anIndex] less: #1];cr.
	self if: [self expr: [self expr: #offset lt: #0]
		or: [self expr: [self call: #len with: #array] lt: #offset]]
		return: [self send: #invalidIndex: to: #anArray with: #anIndex].
	self if: [self expr: [self call: #len with: #array] eqeq: #offset]
		return: [self send: #add: to: #anArray with: #anObject].
	self assign: 'array[offset]' to: [self send: #asNativeValue: with: #anObject];cr.
	self return: #anObject ! !

! PythonBoot methodsFor: #api !
array_copyWithAll_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #result to: [self send: #shallowCopy to: #receiver];cr.
	self send: #addAll: to: #result with: #anArray;cr.
	self return: #result ! !

! PythonBoot methodsFor: #api !
array_join_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #glue to: [self send: #handleOf with: [self send: #to_String with: #aString]];cr.

	self implementFunction: #eachString arguments: #(each) doing: [
		self return: [
			self send: #handleOf with: [
			self send: #to_String with: [
			self send: #asObject: with: #each
		]]]
	] comment: nil;cr.
	self assign: #parts to: [
		self send: #collect: to: #anArray with: [
		self send: #blk withArguments: #(eachString #1)]
	];cr.
	self return: [self call: 'self.String' with: [
		self send: #join to: #glue with: [
		self send: #handleOf with: #parts
	]]];cr.
	self comment: 'return self.String(glue.join(self.handleOf(anArray)))' ! !

! PythonBoot methodsFor: #api !
array_remove_ifAbsent_StSystemDictionary
	" Private - Implementation of api function "

	self try: [
		self	send: #remove
			to: [self send: #handleOf with: #receiver]
			with: [self send: #asNativeValue: with: #anObject]
	] except: #ValueError as: #ex doing: [
		self return: [self send: #callBlock0: with: #aBlock
	]];cr.
	self return: #anObject ! !

! PythonBoot methodsFor: #api !
asCharacter_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self call: #isinstance withArguments: #(receiver 'self.Number')]
		return: [	self call: 'self.String' with: [
			self call: #chr with: [
			self send: #handleOf with: #receiver
	]]];cr.
	self return: [self send: #asCharacter to: #receiver] ! !

! PythonBoot methodsFor: #api !
asNativeValue_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self send: #is_Nil with: #anObject] return: #None.
	self if: [self call: #isinstance withArguments: #(anObject '(self.String,self.Number,self.Boolean)')]
		return: [self send: #handleOf with: #anObject].
	self if: [self not: [self call: #hasattr withArguments: #(self '"Array"')]]
		return: #anObject.
	self assign: #cls to: [self call: #getattr withArguments: #(anObject '"__class__"' #None)];cr.
	self if: [self expr: #cls is: 'self.Array']
		return: [self send: #handleOf with: #anObject ].
	self return: #anObject ! !

! PythonBoot methodsFor: #api !
asObject_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self send: #isObject with: #aNativeObject]
		return: #aNativeObject.
	self if: [self not: [self call: #hasattr withArguments: #(self '"Array"')]]
		return: #aNativeObject.
	self assign: #h to: [self send: #handleOf with: #aNativeObject];cr.
	self if: [self send: #is_Number with: #h]
		return: [self call: 'self.Number' with: #h].
	self if: [self send: #is_Nil with: #h]
		return: #nil.
	self assign: #t to: [self call: #type with: #h];cr.
	self if: [self expr: #t eqeq: #bool]
		return: [self send: #b: with: #h].
	self if: [self expr: #t eqeq: #str]
		return: [self call: 'self.String' with: #h].
	self if: [self call: #isinstance withArguments: #(h '(list,tuple)')]
		return: [self call: 'self.Array' with: #h].
	self if: [self call: #callable with: #h]
		return: [self call: 'self.BlockClosure' with: #h].
	self return: #aNativeObject ! !

! PythonBoot methodsFor: #api !
asParameterStObject
	" Private - Implementation of api function "

	self return: [self call: #getattr
		withArguments: #(self '"@handle"' #nil)
	] ! !

! PythonBoot methodsFor: #api !
asPoolDictionaryStSystemDictionary
	" Private - Implementation of api function "

	self comment: #DEBUG.
	self assign: #result to: [self call: 'self.PoolDictionary.new'];cr.
	self for: #k in: [self call: #dir with: #self] do: [
		self send: #at:put: to: #result withArguments: (
			Array	with: #k
				with: [self call: #getattr withArguments: #(self k)]
			)
	];cr.
	self return: #result ! !

! PythonBoot methodsFor: #api !
asString_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self send: #responds_to withArguments: #(receiver '"asString"')]
		return: [self send: #asString to: #receiver].
	self return: [self call: 'self.String' with: [
		self call: #str with: [
		self send: #handleOf with: #receiver
	]]]! !

! PythonBoot methodsFor: #api !
availableStSystemDictionary
	" Private - Implementation of api function "

	self try: [
		self for: #module in: #modules do: [
			self send: #import: with: #module
		]
	] except: #Exception as: #ex doing: [
			self call: #print with: #ex;cr.
			self return: #False
	];cr.
	self return: #True ! !

! PythonBoot methodsFor: #api !
bStSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: [self expr: #aBoolean is: 'self.true']
		or: [self expr: #aBoolean is: 'self.false']]
		return: [self expr: #aBoolean is: 'self.true'].
	self return: [self expr: #aBoolean is: #True] ! !

! PythonBoot methodsFor: #api !
b_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: [self expr: #aBool is: #True]
		or: [self expr: #aBool is: 'self.true']]
		return: 'self.true'.
	self return: 'self.false' ! !

! PythonBoot methodsFor: #api !
basenStSystemDictionary
	" Private - Implementation of api function "

	self assign: #b to: [self send: #handleOf with: #bas];cr.
	self implementFunction: #baseN arguments: #(num) doing: [
		self return: [self expr: [ self expr: [
			self expr: #num eqeq: #0] 
			and: 'numerals[0]']
			or: [self expr: [
				self send: #lstrip to: [
					self call: #baseN with: 'num // b'
					] with: 'numerals[0]'
				] plus: 'numerals[num % b]'
			]]
	];cr.
	self if: [self expr: [self send: #handleOf with: #n] lt: #0]
		return: [self send: #invalidMessage: with: '"basen - must be positive"' ].
	self return: [
		self send: #String to: #smalltalk with: [
		self call: #baseN with: [
		self send: #handleOf with: #n
	]]]! !

! PythonBoot methodsFor: #api !
basic_at_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: [self send: #basic_at_ifAbsent withArguments: #(receiver key #nil)];cr.
	self return: [self send: #asObject: with: #h]! !

! PythonBoot methodsFor: #api !
basic_at_ifAbsentStSystemDictionary
	" Private - Implementation of api function "

	self assign: #k to: [ self call: #str with: [self send: #handleOf with: #key]];cr.
	self if: [self typeOf: #receiver is: #dict] do: [
		self if: [self expr: #k in: #receiver] return: 'receiver[k]'.
		self return: [self send: #callBlock0: with: #aBlock]
	];cr.
	self assign: #absent to: [self call: #StObject with: #nil];cr.
	self assign: #result to: [self call: #getattr withArguments: #(receiver k absent)];cr.
	self if: [self expr: #result is: #absent] return: [self send: #callBlock0: with: #aBlock].
	self return: #result ! !

! PythonBoot methodsFor: #api !
basic_at_ifAbsent_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: [self send: #basic_at_ifAbsent withArguments: #(receiver key aBlock)];cr.
	self return: [self send: #asObject: with: #h] ! !

! PythonBoot methodsFor: #api !
basic_at_putStSystemDictionary
	" Private - Implementation of api function "

	self assign: #k to: [self call: #str with: [self send: #handleOf with: #key]];cr.
	self if: [self typeOf: #receiver is: #dict] do: [
		self if: [self expr: #value is: #None]
			return: [self send: #pop to: #receiver withArguments: #(k #None)].
		self assign: 'receiver[k]' to: #value;cr.
		self return: #value
	];cr.
	self if: [self expr: #value is: #None] do: [
		self if: [self call: #hasattr withArguments: #(receiver k)] do: [
			self if: [self send: #isObject with: #receiver] do: [
				self call: #delattr withArguments: #(receiver k);cr.
				self return: #value
			];cr.
			self call: #setattr withArguments: #(receiver k value)
		];cr.
		self return: #value
	];cr.
	self call: #setattr withArguments: #(receiver k value);cr.
	self return: #value ! !

! PythonBoot methodsFor: #api !
basic_at_put_StSystemDictionary
	" Private - Implementation of api function "

	self send: #basic_at_put withArguments: (Array
		with: #receiver with: #key
		with: [self send: #asNativeValue: with: #value]);cr.
	self return: #value ! !

! PythonBoot methodsFor: #api !
become_a_StSystemDictionary
	" Private - Implementation of api function "

	self comment: 'mutate object to an instance of aClass'.
	self send: #invalidMessage: with: '"become:a:"' ! !

! PythonBoot methodsFor: #api !
bindCoreStSystemDictionary
	" Private - Implementation of api function "

	self comment: '--bind the minimal functions (e.g i/o)'.
	self implementFunction: #stringOf arguments: #(param) doing: [
		self return: [self call: 'self.String' with: [self send: #handleOf with: #param]]
	];cr.
	self assign: #g to: [self call: #globals];cr.
	self implementFunction: #read arguments: #(fileName) doing: [
		self return: [self send: #fileContents with: [self call: #stringOf with: #fileName]]
	];cr.
	self assign: 'g["read"]' to: #read;cr.
	self assign: 'g["fileContents"]' to: #read;cr.
	self implementFunction: #fileWrite arguments: #(fileName contents mode) doing: [
		self return: [self send: #fileWrite withArguments: (Array
			with: [self call: #stringOf with: #fileName]
			with: [self call: #stringOf with: #contents]
			with: [self call: #stringOf with: #mode]
		)]
	];cr.
	self assign: 'g["fileWrite"]' to: #fileWrite;cr.
	self implementFunction: #fileIn arguments: #(fileName) doing: [
		self return: [self send: #fileIn with:
			[self call: #stringOf with: #fileName]
		]
	];cr.
	self assign: 'g["fileIn"]' to: #fileIn;cr.
	self implementFunction: #fileInJS arguments: #(fileName) doing: [
		self return: [self send: #fileInJS with:
			[self call: #stringOf with: #fileName]
		]
	];cr.
	self assign: 'g["fileInJS"]' to: #fileInJS;cr.
	self implementFunction: #doit arguments: #(aString) doing: [
		self return: [self send: #doIt to: [self call: #stringOf with: #aString]]
	];cr.
	self assign: 'g["doit"]' to: #doit;cr.
	self implementFunction: #exportCategory arguments: #(aString) doing: [
		self return: [self
			send: #exportCategory: to: 'self.Exporter'
			with: [self call: #stringOf with: #aString]
		]
	];cr.
	self assign: 'g["exportCategory"]' to: #exportCategory;cr.
	self implementFunction: #saveImage arguments: #(ignored) doing: [
		self return: [self send: #saveImage]
	];cr.
	self assign: 'g["saveImage"]' to: #saveImage;cr.
	self implementFunction: #saveImageAs arguments: #(fileName) doing: [
		self return: [self send: #saveImageAs: with:
			[self call: #stringOf with: #fileName]
		]
	];cr.
	self assign: 'g["saveImageAs"]' to: #saveImageAs;cr.
	self implementFunction: #isFileOlderThan arguments: #(fileName referenceFileName) doing: [
		self return: [self send: #isFileOlderThan
			with: [self call: #stringOf with: #fileName]
			with: [self call: #stringOf with: #referenceFileName]
		]
	];cr.
	self assign: 'g["isFileOlderThan"]' to: #isFileOlderThan;cr.
	self if: [self not: [self call: #hasattr withArguments: #(self '"$imageFileName"')]] do: [
		self call: #setattr withArguments: (Array
			with: #self with: '"$imageFileName"'
			with: [self call: 'self.String' with: [
				self expr: '"s8.image."' plus: [
					self call: #getattr
						withArguments: #(self '"$nativeLanguage"')
				]]
			])
	];cr.
	self return: #True ! !

! PythonBoot methodsFor: #api !
bindRootsStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Private - Link the system roots to ensure proper delegation chains'.
	self send: #invalidMessage: with: '"bindRoots"' ! !

! PythonBoot methodsFor: #api !
bindStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Bind an implementation for message stSelector of klass instances'.
	self if: [self expr: [self send: #is_Nil with: #jsSelector] or: [self not: #jsSelector]] do: [
		self assign: #jsSelector to: [
			self send: #replace to: #stSelector withArguments: #('":"' '"_"')
		]
	];cr.
	self assign: #stSelector to: [self send: #asObject: with: #stSelector];cr.
	self assign: #category to: [self send: #asObject: with: #category];cr.
	self assign: #stSource to: [self send: #asObject: with: #stSource];cr.
	self assign: #nativeSource to: [self send: #asObject: with: #nativeSource];cr.
	self if: [self send: #is_Nil with: #nativeSource] do: [
		self assign: #nativeSource to: [
			self call: #getattr withArguments: #(jsFunction '"$nativeSource"' #nil)
		]
	];cr.
	self assign: #stamp to: [self send: #asObject: with: #stamp];cr.
	self send: #debugBind withArguments: #(
		klass stSelector jsSelector jsFunction
		category stSource nativeSource stamp);cr.
	self assign: #generation to: [
		self call: #getattr withArguments: #(self '"$state"' #nil)
	];cr.
	self assign: #spec to: [self object:
		'"$fn"' -> #jsFunction
		,('#"$selector"' -> #stSelector)
		,('#"$category"' -> #category)
		,('#"$source"' -> #stSource)
		,('#"$nativeSource"' -> #nativeSource)
		,('#"$stamp"' -> #stamp)
		,('#"$generation"' -> 'generation,')
	];cr.
	self if: [self call: #hasattr withArguments: #(self '"CompiledMethod"')] do: [
		self assign: #method to: [self call: 'self.CompiledMethod' with: #spec]
		] else: [ self assign: #method to: #spec ];cr.
	self call: #setattr withArguments: #(method '"$selector"' stSelector);cr.
	self call: #setattr withArguments: #(method '"$category"' category);cr.
	self call: #setattr withArguments: #(method '"$source"' stSource);cr.
	self call: #setattr withArguments: #(method '"$nativeSource"' nativeSource);cr.
	self if: [self send: #not_Nil with: #stamp] do: [
		self call: #setattr withArguments: #(method '"$stamp"' stamp);cr.
	].
		self call: #setattr withArguments: #(method '"$generation"' generation);cr.
	self if: [self send: #not_Nil with: #klass] do: [
		self send: #addMethod withArguments: #(jsSelector method klass stSelector)
	];cr.
	self return: #method ! !

! PythonBoot methodsFor: #api !
blkStSystemDictionary
	" Private - Implementation of api function "

	self assign: #result to: [self call: 'self.BlockClosure' with: #func];cr.
	self if: [self not: [self expr: #argumentCount is: #nil]] do: [
		self comment: 'self.not_Nil(argumentCount):'.
		self call: #setattr withArguments: #(result '"@argumentCount"' argumentCount)
	];cr.
	self if: [self not: [self expr: #info is: #nil]] do: [
		self comment: 'self.not_Nil(info):'.
		self call: #setattr withArguments: #(result '"$info"' info)
	];cr.
	self return: #result ! !

! PythonBoot methodsFor: #api !
booleanNot_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: #receiver is: 'self.true']
		return: 'self.false'.
	self if: [self expr: #receiver is: 'self.false']
		return: 'self.true'.
	self assign: #aHandle to: [self send: #handleOf with: #receiver];cr.
	self if: [self send: #responds_to withArguments: #(receiver '"_8not"')]
		return: [self send: #_8not to: #receiver ].
	self return: [self send: #b: with: [self not: #aHandle]] ! !

! PythonBoot methodsFor: #api !
boolean_and_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: #receiver is: 'self.false']
		return: #receiver.
	self return: [self send: #callBlock0: with: #option] ! !

! PythonBoot methodsFor: #api !
boolean_or_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: #receiver is: 'self.true']
		return: #receiver.
	self return: [self send: #callBlock0: with: #option] ! !

! PythonBoot methodsFor: #api !
callBlock0_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: #aBlock is: #nil] return: #noBlockValue.
	self import: #types;cr.
	self if: [self typeOf: #aBlock is: 'types.FunctionType'] return: [self call: #aBlock].
	self assign: #hBlock to: [self send: #handle to: #aBlock];cr.
	self if: [self expr: #hBlock is: #nil] do: [
		self call: #print withArguments: #('"Corrupted block"' aBlock);cr.
		self return: #nil
	];cr.
	self return: [self call: #hBlock] ! !

! PythonBoot methodsFor: #api !
classChangedStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self lambda: #sm expr: [
		self send: #classChanged: to: #sm with: #aBehavior
	]] ! !

! PythonBoot methodsFor: #api !
classShapeChangedStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self lambda: #sm expr: [
		self send: #classShapeChanged: to: #sm with: #aBehavior
	]] ! !

! PythonBoot methodsFor: #api !
classesStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Returns an Array with the classes defined in the system'.
	self assign: #result to: '[]';cr.
	self for: #k in: [self call: #dir with: #smalltalk] do: [
		self if: [self send: #isupper to: 'k[0]'] do: [
			self assign: #cls to: [self call: #getattr withArguments: #(smalltalk k)];cr.
			self if: [self
				expr: [self send: #isObject to: #smalltalk with: #cls]
				and: [self send: #b to: #smalltalk with: [
				self send: #isClass to: #cls ]]] do: [
					self send: #append to: #result with: #cls
			]
		]
	];cr.
	self return: [self send: #newArray to: #smalltalk with: #result]! !

! PythonBoot methodsFor: #api !
computeMetaSubclassesOf_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #result to: [self send: #newArray with: '[]'];cr.
	self implementFunction: #addSubMeta arguments: #(cls) doing: [
		self send: #add: to: #result with: 'cls.__class__'
	];cr.
	self assign: #array to: [self call: #getattr withArguments: #(aClass '"@subclasses"')];cr.
	self if: [self expr: [self send: #handle to: [self send: #size to: #array]] gt: #0] do: [
		self for: #cls in: [self send: #handle to: #array] do: [
			self call: #addSubMeta with: #cls
		]
	];cr.
	self call: #setattr withArguments: #('aClass.__class__' '"@subclasses"' result)! !

! PythonBoot methodsFor: #api !
computeSubclassesStSystemDictionary
	" Private - Implementation of api function "

	self for: #aClass in: [self send: #handleOf with: [self send: #classes]] do: [
		self assign: #result to: [self call: #list with: [self send: #__subclasses__ to: #aClass]];cr.
		self call: #setattr withArguments: (Array
			with: #aClass with: '"@subclasses"'
			with: [self send: #newArray with: #result]);cr.
		self send: #computeMetaSubclassesOf: with: #aClass
	]! !

! PythonBoot methodsFor: #api !
copyToStSystemDictionary
	" Private - Implementation of api function "

	self assign: #source to: 'self.__dict__';cr.
	self assign: #target to: 'anInstance.__dict__';cr.
	self for: #k in: [self send: #keys to: #source] do: [
		self assign: 'target[k]' to: 'source[k]'
	];cr.
	self if: #False do: [
		self comment: #True:.
		self call: #print with: '"smalltalk replacement with new instance notImplemented. Yet"';cr.
		self return: #self
	];cr.
	self return: #anInstance ! !

! PythonBoot methodsFor: #api !
debugAddClassStSystemDictionary
	" Private - Implementation of api function "

	self return: #self ! !

! PythonBoot methodsFor: #api !
debugBindStSystemDictionary
	" Private - Implementation of api function "

	self return: #self ! !

! PythonBoot methodsFor: #api !
debugStSystemDictionary
	" Private - Implementation of api function "

	self comment: #DEBUG.
	self assign: #dbg to: [self
		expr: [self call: #getattr withArguments: #(self '"DEBUG"' #nil)]
		is: 'self.true'];cr.
	self comment: 'if dbg: print("# outerFramesCount",self.outerFramesCount())'.
	self return: #dbg ! !

! PythonBoot methodsFor: #api !
debug_StSystemDictionary
	" Private - Implementation of api function "

	self comment: #DEBUG.
	self call: #setattr withArguments: #(self '"DEBUG"' aBoolean) ! !

! PythonBoot methodsFor: #api !
do_onError_StSystemDictionary
	" Private - Implementation of api function "

	self send: #invalidMessage: with: '"do:onError:"' ! !

! PythonBoot methodsFor: #api !
emmitStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #emmit: to: #smalltalk with: #fileName] ! !

! PythonBoot methodsFor: #api !
exceptionResult_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #_s8Ret_ to: '__ex.args[0]';cr.
	self return: '_s8Ret_.result' ! !

! PythonBoot methodsFor: #api !
exception_atStSystemDictionary
	" Private - Implementation of api function "

	self if: [self send: #is_Nil with: #__ex] return: #False.
	self if: [self not: [self call: #hasattr withArguments: #(__ex '"args"')]] return: #False.
	self assign: #arg to: '__ex.args[0]';cr.
	self return: [self expr: #arg eqeq: #target] ! !

! PythonBoot methodsFor: #api !
exportCategoryStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #exportCategory: with: #aString] ! !

! PythonBoot methodsFor: #api !
fileContentsStSystemDictionary
	" Private - Implementation of api function "

	self assign: #filename to: [self send: #handleOf with: #fileName];cr.
	self if: [self expr: [self send: #lower to: 'filename[:4]'] eqeq: '"http"']
		return: [self send: #httpContents with: #filename].
	self import: #os;cr.
	self if: [self send: #isdir to: 'os.path' with: #filename]
		return: [self send: #folderContents with: #filename].
	self	with: [self call: #open
			withArguments: #(filename
				'encoding="utf-8"'
				'errors="ignore"')]
		as: #f do: [
			self return: [self
				call: 'self.String'
				with: [self send: #read to: #f]]
	]! !

! PythonBoot methodsFor: #api !
fileInJSStSystemDictionary
	" Private - Implementation of api function "

	self assign: #sName to: [self call: #str with: [self send: #handleOf with: #fileName]];cr.
	self assign: #contents to: [self send: #fileContents to: [self call: 'self.String' with: #sName]];cr.
	self if: [self send: #is_Nil to: #smalltalk with: #contents] do: [
		self send: #error: to: #smalltalk with: [self call: 'self.String' with:
			[self expr: '"Missing file "' plus: #sName]]
	];cr.
	self comment: 'was: return smalltalk.eval_(contents)'.
	self assign: #locals to: [self call: #dict];cr.
	self try: [
		self call: #exec withArguments: (Array
			with: [self send: #handleOf with: #contents]
			with: [self call: #globals]
			with: #locals)
	] except: #Exception as: #ex doing: [
		self call: #print with: #ex
	]! !

! PythonBoot methodsFor: #api !
fileInStSystemDictionary
	" Private - Implementation of api function "

	self assign: #sName to: [self call: #str with: [self send: #handleOf with: #fileName]];cr.
	self assign: #contents to: [self send: #fileContents to: [self call: 'self.String' with: #sName]];cr.
	self if: [self send: #is_Nil to: #smalltalk with: #contents] do: [
		self send: #error: to: #smalltalk with: [self call: 'self.String' with:
			[self expr: '"Missing file "' plus: #sName]]
	];cr.
	self return: [self send: #fileIn to: #contents] ! !

! PythonBoot methodsFor: #api !
fileWriteStSystemDictionary
	" Private - Implementation of api function "

	self with: [self call: #open withArguments: (Array
		with: [self send: #handleOf with: #fileName]
		with: [self send: #handleOf with: #mode])
	] as: #f do: [
		self send: #write to: #f with: [
			self call: #bytes withArguments: (Array
				with: [self send: #handleOf with: #contents]
				with: 'encoding="utf8"'
				with: 'errors="ignore"')
		]
	]! !

! PythonBoot methodsFor: #api !
fnInSpecStSystemDictionary
	" Private - Implementation of api function "

	self call: #print with: [self
		expr: '"OBSOLETE - fnInSpec "'
		plus: 'spec.className'];cr.
	self send: #obsolete ! !

! PythonBoot methodsFor: #api !
fn_OfStSystemDictionary
	" Private - Implementation of api function "

	self return: [self call: #getattr withArguments: #(klass '"$fn"' #nil)] ! !

! PythonBoot methodsFor: #api !
folderContentsStSystemDictionary
	" Private - Implementation of api function "

	self import: #os;cr.
	self assign: #filename to: [self send: #handleOf with: #fileName];cr.
	self assign: #result to: [self send: #newArray with: '[]'];cr.
	self for: #each in: [self call: 'os.listdir' with: #filename] do: [
		self assign: #fullName to: [self call: 'os.path.join' withArguments: #(filename each)];cr.
		self if: [self call: 'os.path.isdir' with: #fullName] do: [
			self send: #add: to: #result with: [
				self send: #newArray with: '["folder",0,each]'
			]
		] else: [
			self assign: #fSize to: [self call: 'os.path.getsize' with: #fullName];cr.
			self send: #add: to: #result with: [
				self send: #newArray with: '["file",fSize,each]'
			]
		]
	];cr.
	self return: #result ! !

! PythonBoot methodsFor: #api !
from_import_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #hLibName to: [self send: #handleOf with: #libraryName];cr.
	self assign: #hName to: [self send: #handleOf with: #aName];cr.
	self return: [self call: [
		self send: #__f__ with: [self sum: #(
			'"():from "' hLibName
			'" import "' hName
			'"; return "' hName)]]]! !

! PythonBoot methodsFor: #api !
globalsDoStSystemDictionary
	" Private - Implementation of api function "

	self for: #k in: 'self.__dict__' do: [
		self if: [self send: #isupper to: 'k[0]'] do: [
			self assign: #aValue to: [self call: #getattr withArguments: #(self k #nil)];cr.
			self if: [self expr: [
				self call: #callable with: [
				self call: #getattr withArguments: #(aValue '"isClass"' #nil)]
				] and: [self expr: [self send: #isClass to: #aValue] is: 'self.false']]
				do: [self call: #action with: [self call: 'self.String' with: #k]]
		]
	];cr.
	self return: #self ! !

! PythonBoot methodsFor: #api !
haltStObject
	" Private - Implementation of api function "

	self comment: 'import pdb; pdb.set_trace()'.
	self return: #self ! !

! PythonBoot methodsFor: #api !
handleOfStSystemDictionary
	" Private - Implementation of api function "

	self return: [self call: #getattr withArguments: #(anObject '"@handle"' anObject)] ! !

! PythonBoot methodsFor: #api !
handleOf_or_StSystemDictionary
	" Private - Implementation of api function "

	self return: [self call: #getattr withArguments: #(anObject '"@handle"' default)] ! !

! PythonBoot methodsFor: #api !
handleStObject
	" Private - Implementation of api function "

	self return: [self call: #getattr withArguments: #(self '"@handle"' #nil)] ! !

! PythonBoot methodsFor: #api !
hashedSet_privateAdd_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #k to: [self call: #str with: [
		self send: #handleOf to: #smalltalk with: #cookie
	]];cr.
	self assign: 'getattr(aSet,"@hash")[k]' to: #cookie ! !

! PythonBoot methodsFor: #api !
hashedSet_privateRemove_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #contents to: [self call: #getattr withArguments: #(aSet '"@contents"')];cr.
	self implementFunction: #remotion arguments: #(each) doing: [
		self if: [self send: #isObject_equalTo to: #smalltalk withArguments: #(each cookie)] do: [
			self send: #remove: to: #contents with: #each;cr.
			self raise: [self call: #Exception with: '"done"']
		]
	];cr.
	self assign: #k to: [self call: #str with: [self send: #handleOf to: #smalltalk with: #cookie]];cr.
	self send: #pop
		to: [self call: #getattr withArguments: #(aSet '"@hash"')]
		withArguments: #(k None) ;cr.
	self try: [
		self send: #do: to: #contents with: [
			self send: #blk to: #smalltalk withArguments: #(remotion #1)
		]
	] except: #Exception as: #ex doing: [self pass]! !

! PythonBoot methodsFor: #api !
httpContentsStSystemDictionary
	" Private - Implementation of api function "

	self assign: #filename to: [self send: #handleOf with: #fileName];cr.
	self import: 'urllib.request';cr.
	self with: [ self call: 'urllib.request.urlopen' with: #filename] as: #f do: [
		self return: [self call: 'self.String' with: [
			self send: #decode to: [self send: #read to: #f]
				withArguments: #('"utf-8"' 'errors="ignore"')
		]]
	]! !

! PythonBoot methodsFor: #api !
ifUndefined_do_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self send: #is_Nil with: #receiver]
		return: [self send: #callBlock0: with: #aBlock].
	self return: #receiver ! !

! PythonBoot methodsFor: #api !
if_nil_notNil_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self send: #is_Nil with: #anObject]
		return: [self send: #callBlock0: with: #block1].
	self return: [self send: #callBlock0: with: #block2] ! !

! PythonBoot methodsFor: #api !
if_true_false_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self send: #b with: #aBoolean] do: [
		self if: [self send: #not_Nil with: #blk1]
			return: [self send: #callBlock0: with: #blk1]
	] else: [
		self if: [self send: #not_Nil with: #blk2]
			return: [self send: #callBlock0: with: #blk2]
	];cr.
	self return: #nil ! !

! PythonBoot methodsFor: #api !
import_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #hName to: [self send: #handleOf with: #libraryName];cr.
	self return: [self call: [self send: #__f__
		with: [self sum: #('"():import "' hName '"; return "' hName)]]]! !

! PythonBoot methodsFor: #api !
instVarAtStSystemDictionary
	" Private - Implementation of api function "

	self assign: #key to: [self sum: #('"$"' ivName)];cr.
	self if: [self call: #hasattr withArguments: #(object key)]
		return: [self call: #getattr withArguments: #(object key)].
	self if: [self call: #hasattr withArguments: #(object '"@handle"')] do: [
		self assign: #h to: [self call: #getattr withArguments: #(object '"@handle"')];cr.
		self return: 'h[key]'
	];cr.
	self return: #default ! !

! PythonBoot methodsFor: #api !
invalidMessage_StSystemDictionary
	" Private - Implementation of api function "

	self raise: [self call: #Exception with: [self sum: (Array
		with: '"Invalid message #"'
		with: [self call: #str with: [self send: #handleOf with: #msg]])
	]]! !

! PythonBoot methodsFor: #api !
isArray_equalToStSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: #table1 is: #table2] return: #True.
	self return: [self
		expr: [self send: #handleOf:or: withArguments: #(table1 #1)]
		eqeq: [self send: #handleOf:or: withArguments: #(table2 #2)]
	] ! !

! PythonBoot methodsFor: #api !
isBoolean_StSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #b: with: [
		self expr: [self call: #type with: [self send: #handleOf with: #receiver]]
			is: #bool
	]] ! !

! PythonBoot methodsFor: #api !
isFileOlderThanStSystemDictionary
	" Private - Implementation of api function "

	self try: [
		self import: #os;cr.
		self if: [self expr: [self call: 'os.path.getmtime'
				with: [self send: #handleOf with: #fileName]]
			lt: [self call: 'os.path.getmtime'
				with: [self send: #handleOf with: #referenceFileName]]]
			return: 'self.true'
	] except: #FileNotFoundError as: #ex doing: [
		self comment: 'print(ex)'.
		self pass
	];cr.
	self return: 'self.false' ! !

! PythonBoot methodsFor: #api !
isNil_StSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #b: with: [self send: #is_Nil with: #aValue]] ! !

! PythonBoot methodsFor: #api !
isObjectStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'return isinstance(anObject,StObject)'.
	self if: [self call: #isinstance withArguments: #(anObject StObject)]
		return: #True.
	self return: [self send: #responds_to withArguments: #(anObject '"yourself"')] ! !

! PythonBoot methodsFor: #api !
isObject_eqeqStSystemDictionary
	" Private - Implementation of api function "

	self comment: #OBSOLETE.
	self if: [self expr: #receiver is: #anObject] return: #True.
	self if: [self expr: [self send: #is_Nil with: #receiver]
		or: [self send: #is_Nil with: #anObject]]
		return: [self expr: [self send: #is_Nil with: #receiver]
			and: [self send: #is_Nil with: #anObject]];cr.
	self return: [self
		expr: [self send: #handleOf with: #receiver]
		is: [self send: #handleOf with: #anObject]] ! !

! PythonBoot methodsFor: #api !
isObject_eqeq_StSystemDictionary
	" Private - Implementation of api function "

	self comment: #OBSOLETE.
	self return: [self send: #b: with: [self
		send: #isObject_eqeq
		withArguments: #(receiver anObject)]] ! !

! PythonBoot methodsFor: #api !
isObject_equalToStSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: #receiver is: #anObject] return: #True.
	self if: [self expr: [self send: #is_Nil with: #receiver]
		or: [self send: #is_Nil with: #anObject]]
		return: [self expr: [self send: #is_Nil with: #receiver]
			and: [self send: #is_Nil with: #anObject]].
	self assign: #h1 to: [self send: #handleOf with: #receiver];cr.
	self assign: #h2 to: [self send: #handleOf with: #anObject];cr.
	self if: [self expr: [self expr: #h1 noteq: #nil]
		or: [self expr: #h2 noteq: #nil]] do: [
		self if: [self expr: #h1 eqeq: #h2] do: [
			self if: [self expr: [self send: #is_Number with: #h1]
				and: [self send: #is_Number with: #h2]]
				return: #True.
			self if: [self expr: [self call: #type with: #h1]
				noteq: [self call: #type with: #h2]]
				return: #False.
			self return: #True
		]
	];cr.
	self if: [self send: #responds_to withArguments: #(receiver '"_eq"')]
		return: [self expr: [self send: #= to: #receiver with: #anObject]
			is: 'self.true'];cr.
	self if: [self send: #responds_to withArguments: #(anObject '"_eq"')]
		return: [self expr: [self send: #= to: #anObject with: #receiver]
			is: 'self.true'];cr.
	self return: #False ! !

! PythonBoot methodsFor: #api !
isObject_equalTo_StSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #b: with: [self
		send: #isObject_equalTo
		withArguments: #(receiver anObject)]]! !

! PythonBoot methodsFor: #api !
is_NilStSystemDictionary
	" Private - Implementation of api function "

	self return: [self expr: [self expr: #aValue is: #nil]
		or: [self expr: #aValue is: #None]] ! !

! PythonBoot methodsFor: #api !
is_NumberStSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: [self call: #type with: #x] is: #bool] return: #False.
	self return: [self call: #isinstance withArguments: #(x '(int,float)')] ! !

! PythonBoot methodsFor: #api !
is_kindOf_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self not: [self call: #isinstance withArguments: #(aClass '(type,tuple)')]]
		return: 'self.false'.
	self if: [self call: #isinstance withArguments: #(anObject aClass)]
		return: 'self.true'.
	self return: [self send: #isKindOf: to: #anObject with: #aClass] ! !

! PythonBoot methodsFor: #api !
klassStObject
	" Private - Implementation of api function "

	self return: 'self.__class__' ! !

! PythonBoot methodsFor: #api !
klassStSystemDictionary
	" Private - Implementation of api function "

	self call: #print with: [self sum: #('"OBSOLETE - klass "' 'spec.className')];cr.
	self send: #obsolete ! !

! PythonBoot methodsFor: #api !
launchImageStSystemDictionary
	" Private - Implementation of api function "

	self assign: 'self.useManager' to: #True;cr.
	self if: [self send: #not_Nil with: #imageName] do: [
		self call: #setattr withArguments: #(self '"$imageFileName"' imageName);cr.
		self call: #load with: #imageName
	];cr.
	self send: #state: with: '"launching"';cr.
	self if: [self call: #callable with: [
		self call: #getattr withArguments: #(self '"launch"' #nil)]] do: [
		self send: #launch
	];cr.
	self send: #state: with: '"running"';cr.
	self return: #self ! !

! PythonBoot methodsFor: #api !
linkNative_to_StSystemDictionary
	" Private - Implementation of api function "

	self comment: 'link native species of instance to delegate to aClass implementation'.
	self send: #obsolete ! !

! PythonBoot methodsFor: #api !
linkNativesStSystemDictionary
	" Private - Implementation of api function "

	self comment: '--bind the native linkings'.
	self import: #sys;cr.
	self send: #setrecursionlimit to: #sys with: '10**6';cr.
	self import: #math;cr.
	self assign: 'self.math' to: #math;cr.
	self assign: 'self.true' to: [self call: 'self.Boolean' with: #True];cr.
	self assign: 'self.false' to: [self call: 'self.Boolean' with: #False];cr.
	self send: #makeCallable: with: 'self.BlockClosure';cr.
	self send: #makeStringable: with: 'self.String';cr.
	self call: #setattr withArguments: #('self.Object.__class__' '"$superclass"' 'self.Class');cr.
	self call: #setattr withArguments: #('self.Metaclass.__class__' '"@instanceClass"' 'self.Metaclass');cr.
	self if: [self not: [self call: #hasattr withArguments: #(self '"OverwriteMethods"')]] do: [
		self assign: 'self.OverwriteMethods' to: 'self.true'
	];cr.
	self if: [self not: [self call: #hasattr withArguments: #(self '"DumpOverwritenMethods"')]] do: [
		self assign: 'self.DumpOverwritenMethods' to: 'self.false'
	];cr.
	self return: #True ! !

! PythonBoot methodsFor: #api !
linkRouteStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Extend lookup on classes'.
	self implementFunction: #lookup arguments: #(self item) doing: [
		self if: [self expr: 
			[self expr: [self expr: 'item[0:2]' eqeq: '"__"']
				or: [self expr: 'item[0]' eqeq: '"$"']]
			or: [self expr: 'item[0]' eqeq: '"@"']
		] return: [self send: #__getattribute__
				to: [self call: #super withArguments: #(receiver self)]
				with: #item].
		self comment: 'print("[info]"+str(self)+"@"+item)'.
		self try: [
			self assign: #result to: [self send: #__getattribute__
				to: [self call: #super withArguments: #(receiver self)]
				with: #item];cr.
			self comment: 'print("[info]local@"+item) #,result.__method__.src())'
		] except: #AttributeError as: #ex doing: [
			self comment: 'print("[info]missing@",item)'.
			self assign: #result to: #None
		];cr.
		self assign: #implementor to: #nil;cr.
		self assign: #delegate to: [self
			call: #getattr
			withArguments: #(smalltalk parent None)
		];cr.
		self if: #result do: [

			self if: [self expr: #delegate is: #self] do: [
				self comment: 'print("[info]exit ",delegate)'.
				self return: [self send: #__get__ to: #result with: #self]
			];cr.
			self assign: #mth to: [
				self call: #getattr withArguments: #(result '"__method__"' #nil)
			];cr.
			self if: [self send: #is_Nil to: #smalltalk with: #mth]
				return: [self send: #__get__ to: #result with: #self].
			self assign: #implementor to: [
				self call: #getattr withArguments: #(mth '"@classField"' #nil)
			];cr.
			self comment: 'print("[info]implementor=",implementor)'.
			self if: [self expr: [self expr: #implementor is: #self]
				or: [self send: #b to: #smalltalk with: [
					self send: #isClass to: #implementor]]] do: [
				self comment: 'print("[info] instance method - IGNORED")'.
				self assign: #implementor to: #nil
			];cr.
			self if: [self expr: [self
					expr: [self expr: #implementor is: 'smalltalk.Object']
					or: [self expr: #implementor is: 'smalltalk.Metaclass']]
				or: [self expr: #implementor is: 'smalltalk.Behavior']] do: [
				self comment: 'print("[info] abstract implementor - IGNORED")'.
				self assign: #implementor to: #nil
			]
		];cr.
		self if: [self expr: #implementor is: #nil] do: [
			self comment: 'print("[info]as",parent)'.
			self if: [self send: #not_Nil to: #smalltalk with: #delegate] do: [
				self assign: #delegate to: [self call: #delegate with: #nil];cr.
				self assign: #citem to: [self call: #getattr
					withArguments: #(delegate item None)];cr.
				self if: [self call: #callable with: #citem] do: [
					self comment: 'print("[info]@"+item+" as "+parent)'.
					self comment: 'print("[info]calling",citem.__func__.__method__.src())'.
					self return: [self send: #__get__ to: 'citem.__func__' with: #self]
				]
			]
		];cr.
		self assign: #delegate to: 'self.__class__';cr.
		self comment: 'print("[cls] lookup "+delegate.__name__+"@",item,"implementor=",implementor)'.
		self assign: #citem to: [self send: #lookup:in: to: #delegate withArguments: #(item self)];cr.
		self if: [self call: #callable with: #citem] do: [
			self comment: 'print("[cls]"+delegate.__name__+"@"+item)'.
			self comment: 'print("[cls]calling",citem.__func__.__method__.src())'.
			self return: #citem
		];cr.
		self if: #result do: [
			self comment: 'print("[info]result@"+item)'.
			self return: [self send: #__get__ to: #result with: #self]
		];cr.
		self raise: [self call: #AttributeError with: '"%s has no attribute %s. "'
			,$% ,' (self.__class__.__name__,item)']
	];cr.
	self call: #setattr withArguments: #(receiver '"__getattribute__"' lookup);cr.
	self implementFunction: #removeItem arguments: #(self item) doing: [
		self comment: 'print("[info]-delitem[",item,"] on ",self)'.
		self if: [self expr: [self expr: 'item[0]' eqeq: '"$"']
			or: [self expr: 'item[0]' eqeq: '"@"']] do: [
				self assign: #old to: [self call: #getattr withArguments: #(self item None)];cr.
				self if: #old do: [self call: #delattr withArguments: #(self item)];cr.
				self return: #old
		];cr.
		self return: [self send: #__delitem__ to: [
			self call: #super withArguments: #(receiver self)
			] with: #item]
	];cr.
	self call: #setattr withArguments: #(receiver '"__delitem__"' removeItem) ! !

! PythonBoot methodsFor: #api !
makeCallable_StSystemDictionary
	" Private - Implementation of api function "

	self comment: 'implement callable protocoll for instances of aClass'.
	self assign: #st to: #self;cr.
	self implementFunction: #callS8Callable arguments: #(self '*args' '**kwargs') doing: [
		self assign: #hBlock to: [self send: #handle];cr.
		self if: [self send: #is_Nil to: #st with: #hBlock] do: [
			self call: #print withArguments: #('"Corrupted callable"' self);cr.
			self return: #nil
		];cr.
		self return: [self call: #hBlock withArguments: #(
			'*(st.arguments_toCall_(args,self))'
			'**kwargs')]
	];cr.
	self call: #setattr withArguments: #(aClass '"__call__"' callS8Callable) ! !

! PythonBoot methodsFor: #api !
makeStringable_StSystemDictionary
	" Private - Implementation of api function "

	self comment: 'implement string protocoll for instances of aClass'.
	self assign: #st to: #self;cr.
	self implementFunction: #stringBlock arguments: #(self '*args') doing: [
		self assign: #h to: [self send: #handleOf to: #st with: #self];cr.
		self if: [self expr: [self call: #type with: #h] is: #str] return: #h.
		self return: [self send: #__str__ to: [self call: #super] with: #self]
	];cr.
	self call: #setattr withArguments: #(aClass '"__str__"' stringBlock) ! !

! PythonBoot methodsFor: #api !
mapClassNameStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Add a new class mapping to the system'.
	self comment: 'print("mapClassName",className)'.
	self assign: #mi to: #None;cr.
	self if: [self not: #superclass] do: [
		self assign: #mi to: [self call: #type
			withArguments: (Array
				with: [self sum: #(className '"$info"')]
				with: '(type,)'
				with: [self call: #dict])
		];cr.
		self send: #linkRoute withArguments: #(mi '"Class"');cr.
		self assign: #delegate to: '(mi,)'
	] else: [
		self assign: #delegate to: '(superclass.__class__,)'
	];cr.
	self assign: #mc to: [ self call: #StMetaType
		withArguments: (Array
			with: [self sum: #(className '" class"')]
			with: #delegate
			with: [self object: (Array with: '"$meta"' -> #True)])
	];cr.
	self if: [self send: #is_Nil with: #superclass] do: [
		self assign: #delegate to: '()'
	] else: [	self assign: #delegate to: '(superclass,)'
	];cr.
	self if: [self send: #not_Nil with: #native] do: [
		self if: [self typeOf: #native is: #tuple] do: [
			self assign: #delegate to: [self sum: #(delegate native)]
		] else: [
			self assign: #delegate to: [self sum: #(delegate '(native,)')]
	] ];cr.
	self assign: #cls to: [self call: #mc
		withArguments: (Array
			with: #className with: #delegate
			with: [self object: (Array with: '"$meta"' -> #False)])
	];cr.
	self call: #setattr withArguments: (Array
		with: #mc with: '"$className"'
		with: [self send: #stringOf with: 'mc.__name__']);cr.
	self call: #setattr withArguments: #(mc '"@instanceClass"' cls);cr.
	self call: #setattr withArguments: #(mc '"@subclasses"' #nil);cr.
	self call: #setattr withArguments: #(mc '"$iVarNames"' '[]');cr.
	self call: #setattr withArguments: (Array
		with: #mc with: '"$methods"'
		with: [self call: #dict]);cr.
	self if: [self send: #not_Nil with: #superclass] do: [
		self call: #setattr withArguments: #(mc '"$superclass"' 'superclass.__class__')
	];cr.
	self call: #setattr withArguments: (Array
		with: #cls with: '"$className"'
		with: [self send: #stringOf with: 'cls.__name__']);cr.
	self call: #setattr withArguments: #(cls '"$category"' category);cr.
	self call: #setattr withArguments: #(cls '"$superclass"' superclass);cr.
	self call: #setattr withArguments: #(cls '"@subclasses"' #nil);cr.
	self call: #setattr withArguments: (Array
		with: #cls with: '"$methods"'
		with: [self call: #dict]);cr.
	self call: #setattr withArguments: #(cls '"$comment"' #nil);cr.
	self call: #setattr withArguments: #(self className cls);cr.
	self return: #cls ! !

! PythonBoot methodsFor: #api !
math_max_StSystemDictionary
	" Private - Implementation of api function "

	self send: #obsolete ! !

! PythonBoot methodsFor: #api !
math_min_StSystemDictionary
	" Private - Implementation of api function "

	self send: #obsolete ! !

! PythonBoot methodsFor: #api !
metaVarNamesStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Set the instance variables of aMetaclass'.
	self if: [self send: #isArray_equalTo withArguments: (Array
		with: [self call: #getattr withArguments: #(aMetaClass '"$iVarNames"' '[]')]
		with: #iVarNames)] return: 'self.false'.
	self assign: #hArray to: [self send: #handleOf with: #iVarNames];cr.
	self if: [self send: #is_Nil with: #hArray] do: [
		self assign: #hArray to: '[]'
	];cr.
	self send: #setVarNames withArguments: #(aMetaClass hArray #nil);cr.
	self send: #classShapeChanged with: #aMetaClass;cr.
	self send: #classChanged with: #aMetaClass;cr.
	self return: 'self.true' ! !

! PythonBoot methodsFor: #api !
methodBoundToStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self lambda: #sm expr: [
		self send: #methodBound:to: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method ! !

! PythonBoot methodsFor: #api !
methodRemovedFromStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self lambda: #sm expr: [
		self send: #methodRemoved:from: to: #sm withArguments: #(method klass)
	]];cr.
	self return: #method ! !

! PythonBoot methodsFor: #api !
methodStSystemDictionary
	" Private - Implementation of api function "

	self comment: #OBSOLETE.
	self return: [self send: #bind withArguments:
		#(#nil 'spec.selector' #nil 'spec.fn' 'spec.category' 'spec.source')
		,(Array
			with: [self call: #getattr withArguments: #(spec '"$nativeSource"' #nil)]
			with: [self call: #getattr withArguments: #(spec '"$stamp"' #nil)])
	] ! !

! PythonBoot methodsFor: #api !
methods_OfStSystemDictionary
	" Private - Implementation of api function "

	self assign: #proto to: [self send: #prototype_Of with: #klass];cr.
	self return: [self call: #getattr withArguments: #(proto '"$methods"' #nil)] ! !

! PythonBoot methodsFor: #api !
moduleAtStSystemDictionary
	" Private - Implementation of api function "

	self assign: #result to: #object;cr.
	self try: [
		self assign: #m to: [self send: #import: with: #module];cr.
		self assign: #result to: [self call: #getattr with: 'm.field']
	] except: #Exception as: #ex doing: [
		self call: #print with: #ex
	];cr.
	self return: #result! !

! PythonBoot methodsFor: #api !
mustBeNumber_StSystemDictionary
	" Private - Implementation of api function "

	self from: #numbers import: #Number;cr.
	self if: [self call: #isinstance withArguments: (Array
		with: [self send: #handleOf with: #receiver]
		with: #Number)] return: #receiver.
	self return: [self send: #error: to: #receiver
		with: [self send: #String with: '"Must be a Number"']] ! !

! PythonBoot methodsFor: #api !
nativeCallStSystemDictionary
	" Private - Implementation of api function "

	self assign: #func to: #anObject;cr.
	self if: [self send: #not_Nil with: #selector] do: [
		self assign: #func to: [self call: #getattr
			withArguments: (Array with: #anObject
				with: [self send: #handleOf with: #selector])
		]
	];cr.
	self if: [self send: #isObject with: #anObject]
		return: [self call: #func with: '*args'].
	self assign: #handles to: '[]';cr.
	self for: #arg in: #args do: [
		self send: #append to: #handles with: [
			self send: #handleOf with: #arg
		]
	];cr.
	self assign: #result to: [self call: #func with: [
		self pointerTo: [self call: #tuple with: #handles]]
	];cr.
	self return: [self send: #asObject: with: #result] ! !

! PythonBoot methodsFor: #api !
newArrayStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'return a new Array instance with contents in object'.
	self return: [self send: #Array with: [self expr: #object or: '[]']] ! !

! PythonBoot methodsFor: #api !
newConstructorStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'return a new constructor of instances inheriting from superC (constructor)'.
	self comment: 'superC can be nil to return a constructor for a root class'.
	self send: #invalidMessage: with: '"newConstructor"' ! !

! PythonBoot methodsFor: #api !
newContextStSystemDictionary
	" Private - Implementation of api function "

	self return: [self call: #StObject with: #nil] ! !

! PythonBoot methodsFor: #api !
newHashStSystemDictionary
	" Private - Implementation of api function "

	self return: [self object: #()]! !

! PythonBoot methodsFor: #api !
newMethodStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #bind withArguments: #(
		#nil selector #nil fn #nil source
		nativeSource stamp)] ! !

! PythonBoot methodsFor: #api !
niyStSystemDictionary
	" Private - Implementation of api function "

	self import: #inspect;cr.
	self assign: #text to: [self sum: #(
		'"NIY #"' 'inspect.stack()[1].function'
	)];cr.
	self call: #print with: #text;cr.
	self return: #text ! !

! PythonBoot methodsFor: #api !
niy_StSystemDictionary
	" Private - Implementation of api function "

	self call: #print with: [self sum: #('"NIY #"' msg)] ! !

! PythonBoot methodsFor: #api !
notNil_StSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #b: with: [self send: #not_Nil with: #aValue]] ! !

! PythonBoot methodsFor: #api !
not_NilStSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: [self expr: #aValue is: #nil]
		or: [self expr: #aValue is: #None]]
		return: #False.
	self return: #True ! !

! PythonBoot methodsFor: #api !
objectNotFoundStSystemDictionary
	" Private - Implementation of api function "

	self send: #smDo with: [self lambda: #sm expr: [
		self send: #object:at:notFoundIn: to: #sm
			withArguments: #(object indexOrKey collection)
	]];cr.
	self return: #collection ! !

! PythonBoot methodsFor: #api !
object_comma_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h1 to: [self send: #handleOf with: #receiver];cr.
	self assign: #h2 to: [self send: #handleOf with: #anObject];cr.
	self if: [self expr: [self call: #type with: #h1] is: #str] do: [
		self if: [self send: #is_Nil with: #anObject]
			return: [self send: #String with: [self sum: #(h1 '""')]].
		self if: [self expr: [self call: #type with: #h2] is: #str]
			return: [self send: #String with: [self sum: #(h1 h2)]].
		self if: [self send: #isObject with: #anObject]
			return: [self send: #String with: [self sum: (Array with: #h1 
				with: [self send: #handleOf with: [
					self send: #to_String with: #anObject
				]])]].
		self return: [self send: #String with: [
			self sum: (Array with: #h1 with: [
				self call: #str with: #h2])]]
	];cr.
	self if: [self not: [self send: #isObject with: #receiver]]
		return: [self send: #object:comma:
			with: [self send: #to_String with: #receiver]
			with: #anObject].
	self return: [self send: #, to: #receiver with: #anObject] ! !

! PythonBoot methodsFor: #api !
objects_toCall_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #result to: '[]';cr.
	self for: #x in: [self send: #handleOf with: #anArray] do: [
		self send: #append to: #result with: [
			self send: #asObject: with: #x
		]
	];cr.
	self return: #result ! !

! PythonBoot methodsFor: #api !
obsoleteStSystemDictionary
	" Private - Implementation of api function "

	self import: #inspect;cr.
	self assign: #text to: [self sum: #('"OBSOLETE #"' 'inspect.stack()[1].function')];cr.
	self call: #print with: #text;cr.
	self return: #text! !

! PythonBoot methodsFor: #api !
outerFramesCountStSystemDictionary
	" Private - Implementation of api function "

	self comment: #DEBUG.
	self from: #inspect import: 'getouterframes, currentframe';cr.
	self return: [self call: #len with: [
		self call: #getouterframes with: [
		self call: #currentframe]]] ! !

! PythonBoot methodsFor: #api !
pool_at_ifAbsent_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: [self call: #getattr withArguments: #(pool '"@hash"')];cr.
	self assign: #k to: [self call: #str with: [self send: #handleOf with: #aKey]];cr.
	self if: [self typeOf: #h is: #dict] do: [
		self if: [self expr: #k in: #h]
			return: [self send: #asObject: with: 'h[k]']
	] else: [
		self if: [self call: #hasattr withArguments: #(h k)]
			return: [self send: #asObject: with: [
				self call: #getattr withArguments: #(h k)]]
	];cr.
	self comment: 'print("pool key=",k,"missing") # in ",h)'.
	self if: [self send: #is_Nil with: #aBlock] return: #nil.
	self comment: 'info = getattr(aBlock,"$info","no-info")'.
	self if: [self call: #callable with: #aBlock]
		return: [self send: #callBlock0: with: #aBlock].
	self return: [self send: #evaluate to: #aBlock] ! !

! PythonBoot methodsFor: #api !
pool_at_put_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: [self
		call: #getattr withArguments: #(pool '"@hash"')
	];cr.
	self assign: #k to: [self call: #str with: [
		self send: #handleOf with: #aKey]
	];cr.
	self assign: #v to: [
		self send: #wrap: to: #pool with: #aValue
	];cr.
	self if: [self typeOf: #h is: #dict] do: [
		self assign: 'h[k]' to: #v
	] else: [
		self call: #setattr withArguments: #(h k v)
	];cr.
	self return: #aValue ! !

! PythonBoot methodsFor: #api !
pool_includesKey_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: [self call: #getattr withArguments: #(pool '"@hash"') ];cr.
	self assign: #k to: [self call: #str with: [self send: #handleOf with: #aKey]];cr.
	self if: [self typeOf: #h is: #dict] do: [
		self if: [self expr: #k in: #h] return: 'self.true'
	] else: [
		self if: [self call: #hasattr withArguments: #(h k)] return: 'self.true'
	];cr.
	self return: 'self.false' ! !

! PythonBoot methodsFor: #api !
pool_keysStSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: [self call: #getattr withArguments: #(pool '"@hash"')];cr.
	self if: [self typeOf: #h is: #dict] return: [
		self send: #newArray with: [
			self call: #list with: [
				self send: #keys to: #h]]
	].
	self if: [self typeOf: #h is: #list] return: [self send: #newArray with: #h].
	self assign: #h to: [self send: #keys to: 'h.__dict__'];cr.
	self return: [self send: #newArray with: [self call: #list with: #h]] ! !

! PythonBoot methodsFor: #api !
pool_removeKey_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #h to: [
		self call: #getattr withArguments: #(pool '"@hash"')
	];cr.
	self assign: #k to: [
		self call: #str with: [self send: #handleOf with: #aKey]
	];cr.
	self if: [self typeOf: #h is: #dict] do: [
		self send: #pop to: #h withArguments: #(k #nil)
	] else: [ self call: #delattr withArguments: #(h k) ] ! !

! PythonBoot methodsFor: #api !
printString_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self expr: 'receiver.__class__' is: 'self.String']
		return: [self send: #String with: [self sum: (Array
			with: $",$',$"
			with: [self send: #handleOf with: #receiver]
			with: $",$',$")]].
	self if: [self send: #responds_to withArguments: #(receiver '"printString"')]
		return: [self send: #printString to: #receiver].
	self return: [self send: #String with: [
		self call: #repr with: [
			self send: #handleOf with: #receiver
	]]] ! !

! PythonBoot methodsFor: #api !
prototype_OfStSystemDictionary
	" Private - Implementation of api function "

	self assign: #fn to: [self send: #fn_Of with: #klass];cr.
	self if: [self send: #is_Nil with: #fn] return: #klass.
	self return: [self call: #getattr withArguments: #(fn '"prototype"' #nil)] ! !

! PythonBoot methodsFor: #api !
quitStSystemDictionary
	" Private - Implementation of api function "

	self raise: #SystemExit! !

! PythonBoot methodsFor: #api !
raise_result_StSystemDictionary
	" Private - Implementation of api function "

	self assign: '_s8Ret_.result' to: #result;cr.
	self raise: [self call: #Exception with: #_s8Ret_];cr.
	self return: #result ! !

! PythonBoot methodsFor: #api !
readJSObjectStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #invalidMessage: with: '"readJSObject"'] ! !

! PythonBoot methodsFor: #api !
register_method_in_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #methods to: [self send: #methods_Of with: #klass];cr.
	self if: [self send: #not_Nil with: #methods] do: [
		self assign: #key to: [self send: #handleOf with: #stSelector];cr.
		self assign: 'methods[key]' to: #aMethod
	]! !

! PythonBoot methodsFor: #api !
removeClassStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Remove aClass from system'.
	self assign: #deleted to: [self send: #String with: [self sum: (Array
		with: '"Deleted-"' with: [self send: #handleOf with: [
			self call: #getattr withArguments: #(aClass '"$className"')]]
		)]];cr.
	self assign: #array to: [self call: #getattr withArguments: (Array
		with: [self call: #getattr withArguments: #(aClass '"$superclass"' #nil)]
		with: '"@subclasses"' with: #None)
	];cr.
	self if: #array do: [
		self send: #remove:ifAbsent: to: #array withArguments: (Array
			with: #aClass
			with: [self send: #empty to: 'self.BlockClosure'])
	];cr.
	self assign: #meta to: 'aClass.__class__';cr.
	self assign: #info to: [self call: #getattr withArguments: #(meta '"$info"' None)];cr.
	self call: #delattr withArguments: (Array with: #self
		with: [self send: #handleOf with: [
			self call: #getattr withArguments: #(aClass '"$className"')]
	]);cr.
	self if: #info do: [self call: #setattr withArguments: (Array
		with: #info with: '"$className"'
		with: [self sum: #(deleted '" class"')])
	];cr.
	self call: #setattr withArguments: #(meta '"$className"' deleted);cr.
	self send: #classChanged with: #aClass;cr.
	self return: #self ! !

! PythonBoot methodsFor: #api !
removeMethodStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Remove aMethod implementation from aClass'.
	self send: #aboutToRemoveFrom withArguments: #(aMethod aClass);cr.
	self assign: #stSelector to: [self send: #selector to: #aMethod];cr.
	self assign: #jsSelector to: [self send: #handleOf
		with: [self send: #asSelector to: #stSelector]
	];cr.
	self send: #unregister:in: withArguments: #(stSelector aClass);cr.
	self call: #delattr withArguments: #(aClass jsSelector);cr.
	self return: [self send: #methodRemovedFrom withArguments: #(aMethod aClass)] ! !

! PythonBoot methodsFor: #api !
renameClassStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Rename aClass to aString'.
	self assign: #meta to: 'aClass.__class__';cr.
	self assign: #info to: [self call: #getattr withArguments: #(meta '"$info"' None)];cr.
	self call: #delattr withArguments: (Array with: #self
		with: [self call: #getattr withArguments: #(aClass '"$className"')]);cr.
	self if: #info do: [self call: #setattr withArguments: (Array
		with: #info with: '"$className"'
		with: [self sum: #(aString '" class"')])];cr.
	self call: #setattr withArguments: #(meta '"$className"' aString);cr.
	self call: #setattr withArguments: #(self aString aClass);cr.
	self send: #classChanged with: #aClass;cr.
	self return: #self ! !

! PythonBoot methodsFor: #api !
reshapeClass_StSystemDictionary
	" Private - Implementation of api function "

	self send: #reshape:of: with: '"$className"' with: #aClass;cr.
	self send: #reshape:of: with: '"$category"' with: #aClass;cr.
	self send: #reshape:of: with: '"$iVarNames"' with: #aClass;cr ! !

! PythonBoot methodsFor: #api !
reshapeClassesStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'mutate class shapes to my implementation'.
	self for: #cls in: [self send: #values to: 'self.__dict__'] do: [
		self if: [self expr: [self call: #type with: [self call: #type with: #cls]] is: #StMetaType] do: [
			self send: #reshapeClass: with: #cls;cr.
			self send: #reshapeClass: with: 'cls.__class__'
		]
	]! !

! PythonBoot methodsFor: #api !
reshape_of_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #value to: [self call: #getattr withArguments: #(anObject field None)];cr.
	self if: [self expr: #value is: #None] return: ''.
	self if: [self expr: [self call: #type with: #value] is: #str] do: [
		self call: #setattr withArguments: (Array
			with: #anObject with: #field
			with: [self send: #String with: #value])
	];cr.
	self if: [self expr: [self call: #type with: #value] is: #list] do: [
		self send: #toObjects with: #value;cr.
		self call: #setattr withArguments: (Array
			with: #anObject with: #field
			with: [self send: #newArray with: #value])
	];cr.
	self if: [self expr: 'value.__class__' eqeq: 'self.Array'] do: [
		self send: #toObjects with: [
			self send: #handleOf with: #value]
	]! !

! PythonBoot methodsFor: #api !
responds_toStSystemDictionary
	" Private - Implementation of api function "

	self assign: #selector to: [self send: #handleOf with: #jsSelector];cr.
	self if: [self expr: [self call: #type with: #selector] noteq: #str] return: #False.
	self return: [self call: #callable with: [
		self call: #getattr withArguments: #(anObject selector None)
	]] ! !

! PythonBoot methodsFor: #api !
responds_to_StSystemDictionary
	" Private - Implementation of api function "

	self if: [self not: [self expr: [self call: #type with: #aSelector] is: 'self.String']] return: 'self.false'.
	self return: [self send: #b: with: [
		self send: #responds_to with: #anObject
			with: [self send: #asSelector to: #aSelector]
	]] ! !

! PythonBoot methodsFor: #api !
saveImageAsStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #saveImageAs: to: #smalltalk with: #fileName] ! !

! PythonBoot methodsFor: #api !
saveImageStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #saveImage to: #smalltalk] ! !

! PythonBoot methodsFor: #api !
setClassStSystemDictionary
	" Private - Implementation of api function "

	self call: #print with: [self sum: #('"OBSOLETE - setClass "' 'spec.className')];cr.
	self send: #obsolete ! !

! PythonBoot methodsFor: #api !
setPropertyNamesStSystemDictionary
	" Private - Implementation of api function "

	self assign: #hArray to: [self send: #handleOf with: #pNames];cr.
	self if: [self send: #is_Nil with: #hArray] do: [
		self assign: #hArray to: '[]'
	];cr.
	self call: #setattr withArguments: (Array
		with: #aClass with: '"$propertyNames"'
		with: [self send: #newArray with: #hArray]) ! !

! PythonBoot methodsFor: #api !
setProtoOf_to_StSystemDictionary
	" Private - Implementation of api function "

	self send: #invalidMessage: with: '"setProtoOf:to:"' ! !

! PythonBoot methodsFor: #api !
setVarNamesStSystemDictionary
	" Private - Implementation of api function "

	self assign: #hArray to: [self send: #handleOf with: #iVarNames];cr.
	self if: [self send: #is_Nil with: #hArray] do: [
		self assign: #hArray to: '[]'
	];cr.
	self call: #setattr withArguments: (Array
		with: #aClass with: '"$iVarNames"'
		with: [self send: #newArray with: #hArray]);cr.
	self for: #iv in: #hArray do: [
		self assign: #key to: [self sum: (Array
			with: '"@"'
			with: [self send: #handleOf with: #iv])];cr.
		self if: [self not: [self call: #hasattr withArguments: #(aClass key)]] do: [
			self call: #setattr withArguments: #(aClass key aValue)
		]
	]! !

! PythonBoot methodsFor: #api !
smDoStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Notify the system about executing aBlock'.
	self if: [self call: #getattr withArguments: #(self '"useManager"' False)] do: [
		self if: [self call: #hasattr withArguments: #(self '"SystemManager"')] do: [
			self if: [self call: #hasattr withArguments: #('self.SystemManager' '"currentOrNil"')] do: [
				self assign: #sm to: [self send: #currentOrNil to: #'self.SystemManager'];cr.
				self if: [self send: #not_Nil with: #sm] return: [self call: #block with: #sm]
			]
		]
	]! !

! PythonBoot methodsFor: #api !
stObjectStSystemDictionary
	" Private - Implementation of api function "

	self return: #StObject ! !

! PythonBoot methodsFor: #api !
startDebugStSystemDictionary
	" Private - Implementation of api function "

	self comment: #DEBUG.
	self call: #setattr withArguments: #(self '"DEBUG"' 'self.true');cr.
	self call: #print with: '"# DEBUG started"';cr.
	self comment: 'print("# outerFramesCount",self.outerFramesCount())'.
	self comment: 'import gc'.
	self comment: 'gc.collect()'.
	self comment: 'print("# collection finished")'.
	self comment: 'gc.set_debug(gc.DEBUG_LEAK)' ! !

! PythonBoot methodsFor: #api !
startupStSystemDictionary
	" Private - Implementation of api function "

	self send: #state: with: '"booting"';cr.
	self send: #state: with: '"singularity"';cr.
	self send: #mapClassName withArguments: #('"Object"' '"Kernel"' StObject);cr.
	self send: #mapClassName withArguments: #('"Smalltalk"' '"Kernel"' StSystemDictionary 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Behavior"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Class"' '"Kernel"' #nil 'self.Behavior');cr.
	self send: #mapClassName withArguments: #('"Metaclass"' '"Kernel"' #nil 'self.Behavior');cr.
	self send: #mapClassName withArguments: #('"CompiledMethod"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Number"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"BlockClosure"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Boolean"' '"Kernel"' #nil 'self.Object');cr.
	self comment: 'self.mapClassName("DateTime","Kernel",Date,self.Object)'.
	self send: #mapClassName withArguments: #('"UndefinedObject"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"Collection"' '"Kernel"' #nil 'self.Object');cr.
	self send: #mapClassName withArguments: #('"SequenceableCollection"' '"Kernel"' #nil 'self.Collection');cr.
	self send: #mapClassName withArguments: #('"String"' '"Kernel"' #nil 'self.SequenceableCollection');cr.
	self send: #mapClassName withArguments: #('"Array"' '"Kernel"' #nil 'self.SequenceableCollection');cr.
	self comment: 'self.mapClassName("RegularExpression","Kernel",RegExp,self.Object)'.
	self send: #state: with: '"whiteHole"';cr.
	self send: #linkNatives;cr.
	self send: #reshapeClasses;cr.
	self send: #state: with: '"wormHole"';cr.
	self send: #bindCore;cr.
	self send: #state: with: '"startup"';cr.
	self return: [self send: #copyTo with: [self send: #Smalltalk with: #nil]]
		and: [self send: #UndefinedObject with: #nil]! !

! PythonBoot methodsFor: #api !
state_StSystemDictionary
	" Private - Implementation of api function "

	self call: #setattr withArguments: #(self '"$state"' aState) ! !

! PythonBoot methodsFor: #api !
stringOfStSystemDictionary
	" Private - Implementation of api function "

	self if: [self not: [self expr: [self call: #type with: #nativeString] is: #str]]
		return: [self send: #invalidMessage: with: '"Must be a native string"'].
	self if: [self call: #hasattr withArguments: #(self '"String"')]
		return: [self send: #String with: #nativeString].
	self return: #nativeString ! !

! PythonBoot methodsFor: #api !
string_asNumber_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #s to: [self send: #handleOf with: #receiver];cr.
	self if: [self expr: '"."' in: #s] do: [
		self return: [self send: #Number with: [self call: #float with: #s]]
	] else: [
		self return: [self send: #Number with: [self call: #int with: #s]]
	]! !

! PythonBoot methodsFor: #api !
string_isLiteral_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #s to: [self send: #handleOf with: #receiver];cr.
	self assign: #l to: [self call: #len with: #s];cr.
	self if: [self expr: #l eqeq: #0] return: 'self.false'.
	self if: [self send: #b with: [self send: #isBinarySelector: to: 'self.String' with: #receiver]] return: 'self.true'.
	self import: #re;cr.
	self assign: #p to: [self send: #compile to: #re with: '"[\w:]*"'];cr.
	self assign: #match to: [self send: #match to: #p with: #s];cr.
	self if: [self expr: #match and: [self expr: [self send: #end to: #match] eqeq: #l]] return: 'self.true'.
	self return: 'self.false' ! !

! PythonBoot methodsFor: #api !
string_matchStSystemDictionary
	" Private - Implementation of api function "

	self assign: #s to: [self send: #handleOf with: #receiver];cr.
	self assign: #regexp to: [self send: #handleOf with: #aRegexp];cr.
	self import: #re;cr.
	self assign: #p to: [self send: #compile to: #re with: #regexp];cr.
	self return: [self send: #match to: #p with: #s] ! !

! PythonBoot methodsFor: #api !
string_occurrencesOf_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #s to: [self send: #handleOf with: #aString];cr.
	self if: [self expr: [self call: #type with: #s] eqeq: #str] do: [
		self return: [self send: #Number with: [
			self send: #count
				to: [self send: #handleOf with: #receiver]
				with: #s
			]]
	] else: [
		self return: [self send: #Number with: #0]
	]! !

! PythonBoot methodsFor: #api !
string_startsWith_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #s to: [self send: #handleOf with: #aString];cr.
	self if: [self expr: [self call: #type with: #s] eqeq: #str] do: [
		self return: [self send: #b: with: [
			self send: #startswith
				to: [self send: #handleOf with: #receiver]
				with: #s
		]]
	] else: [
		self return: 'self.false'
	]! !

! PythonBoot methodsFor: #api !
subclassesStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'Boot - Return the subclasses of aClass'.
	self if: [self send: #is_Nil with: #aClass]
		return: [self send: #subclasses to: #smalltalk with: 'smalltalk.Smalltalk'].
	self assign: #result to: [self call: #getattr withArguments: #(aClass '"@subclasses"' #nil)];cr.
	self if: [self send: #is_Nil with: #result] do: [
		self send: #computeSubclasses
	];cr.
	self return: [self call: #getattr withArguments: #(aClass '"@subclasses"')] ! !

! PythonBoot methodsFor: #api !
superImplementorStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #invalidMessage: with: '"superImplementor"'] ! !

! PythonBoot methodsFor: #api !
superReceiverStSystemDictionary
	" Private - Implementation of api function "

	self assign: #meta to: [self call: #getattr withArguments: #(receiver '"$meta"' False)];cr.
	self comment: 'print("superReceiver",implementor,receiver ,"meta=",meta,"isClass=",receiver.isClass())'.
	self if: #meta return: [self call: #super withArguments: (Array
		with: #implementor
		with: [self send: #instanceClass to: #receiver])].
	self if: [self send: #b with: [self send: #isClass to: #receiver]]
		return: [self call: #SuperForClass withArguments: #(implementor receiver)].
	self return: [self call: #super withArguments: #(implementor receiver)] ! !

! PythonBoot methodsFor: #api !
tStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'TESTING - print the native sources of test method'.
	self assign: #test to: [self sum: (Array
		with: '"test"'
		with: [self call: #str with: #number])];cr.
	self call: #print with: [
		self send: #src to: [self
			property: #__method__
			of: [self call: #getattr withArguments: #('self.true' test)]
		]
	] ! !

! PythonBoot methodsFor: #api !
testAllStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'TESTING - Run all tests'.
	self return: [self send: #testAll to: 'self.true'] ! !

! PythonBoot methodsFor: #api !
testFromStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'TESTING - Run from first test'.
	self for: #i in: [self call: #range withArguments: #(first last)] do: [
		self try: [ self send: #test with: #i
		] except: #Exception as: #ex doing: [
			self call: #print withArguments: (Array
				with: [self sum: (Array
					with: '"test"'
					with: [self call: #str with: #i])]
				with: 'ex.args')
		]
	] ! !

! PythonBoot methodsFor: #api !
testStSystemDictionary
	" Private - Implementation of api function "

	self comment: 'TESTING - Returns the result of running test (or nil)'.
	self assign: #f to: [self call: #getattr withArguments: (Array
		with: 'smalltalk.true'
		with: [self sum: (Array
			with: '"test"'
			with: [self call: #str with: #what])]
		with: #nil)
	];cr.
	self if: [self send: #is_Nil with: #f] return: #nil.
	self comment: 'print("test",what)'.
	self if: [self expr: [self call: #f] is: 'smalltalk.true'] return: #True.
	self comment: 'print("FAILED test",what)'.
	self call: #print withArguments: (Array
		with: '"FAILED"'
		with: [self send: #src to: [self
			property: #__method__
			of: #f]]) ! !

! PythonBoot methodsFor: #api !
times_repeat_StSystemDictionary
	" Private - Implementation of api function "

	self for: #i in: [self call: #range with: [self send: #asParameter to: #count]] do: [
		self call: #aBlock with: [self send: #Number with: [self sum: #(i #1)]]
	];cr.
	self return: #count ! !

! PythonBoot methodsFor: #api !
toObjectsStSystemDictionary
	" Private - Implementation of api function "

	self for: #i in: [self call: #range withArguments: (Array
		with: #0 with: [self call: #len with: #list])] do: [
		self if: [self expr: 'list[i]' is: #None] do: [
			self assign: 'list[i]' to: #nil
		];cr.
		self if: [self expr: [self call: #type with: 'list[i]'] is: #str] do: [
			self assign: 'list[i]' to: [self send: #String with: 'list[i]']
		];cr.
		self if: [self send: #is_Number with: 'list[i]'] do: [
			self assign: 'list[i]' to: [self send: #Number with: 'list[i]']
		]
	] ! !

! PythonBoot methodsFor: #api !
toStringStObject
	" Private - Implementation of api function "

	self return: [self send: #String to: #smalltalk with: [self send: #__str__]] ! !

! PythonBoot methodsFor: #api !
toStringStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #String with: '"smalltalk"'] ! !

! PythonBoot methodsFor: #api !
to_StringStSystemDictionary
	" Private - Implementation of api function "

	self if: [self send: #is_Nil with: #receiver] return: [self send: #String with: '"nil"'].
	self if: [self call: #isinstance withArguments: #(receiver 'self.Boolean')]
		return: [self send: #String with: [self call: #str with: #receiver]].
	self if: [self send: #responds_to withArguments: #(receiver '"toString"')]
		return: [self send: #toString to: #receiver].
	self return: [self send: #String with: [
		self call: #str with: [
		self send: #handleOf with: #receiver
	]]] ! !

! PythonBoot methodsFor: #api !
unregister_in_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #methods to: [self send: #methods_Of with: #klass];cr.
	self if: [self send: #not_Nil with: #methods] do: [
		self assign: #key to: [self send: #handleOf with: #stSelector];cr.
		self send: #pop to: #methods with: #key
	] ! !

! PythonBoot methodsFor: #api !
value_withInterval_StSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #invalidMessage: with: '"value_withInterval_"'] ! !

! PythonBoot methodsFor: #api !
value_withTimeout_StSystemDictionary
	" Private - Implementation of api function "

	self from: #threading import: #Timer;cr.
	self assign: #t to: [self call: #Timer withArguments: (Array
		with: [self send: #asParameter to: #aNumber]
		with: [self send: #asParameter to: #aBlock])
	];cr.
	self send: #start to: #t;cr.
	self return: #t ! !

! PythonBoot methodsFor: #api !
versionStSystemDictionary
	" Private - Implementation of api function "

	self import: #sys;cr.
	self return: 'sys.version' ! !

! PythonBoot methodsFor: #api !
whileNot_do_StSystemDictionary
	" Private - Implementation of api function "

	self while: [self expr: [self call: #condition] is: 'self.false'] do: [
		self send: #callBlock0: with: #aBlock
	];cr.
	self return: #nil ! !

! PythonBoot methodsFor: #api !
while_do_StSystemDictionary
	" Private - Implementation of api function "

	self while: [self expr: [self call: #condition] is: 'self.true'] do: [
		self send: #callBlock0: with: #aBlock
	];cr.
	self return: #nil ! !

! PythonBoot methodsFor: #api !
whoImplementsStSystemDictionary
	" Private - Implementation of api function "

	self return: [self send: #invalidMessage: with: '"whoImplements"'] ! !

! PythonBoot methodsFor: #api !
will_be_StSystemDictionary
	" Private - Implementation of api function "

	self comment: 'return an object that will become an instance of class (named className) when accessed'.
	self send: #invalidMessage: with: '"will:be:"' ! !

! PythonBoot methodsFor: #api !
string_asPrivateSelector_StSystemDictionary
	" Private - Implementation of api function "

	self assign: #s to: [
		self expressions: (Array
			with: [self send: #handleOf with: #receiver]
			with: [	self dot: [self call: #replace withArguments: #('":"' '"_"')].
				self dot: [self call: #replace withArguments: #('"+"' '"_plus"')] ]
			with: [	self dot: [self call: #replace withArguments: #('"-"' '"_minus"')].
				self dot: [self call: #replace withArguments: #('"*"' '"_star"')] ]
			with: [	self dot: [self call: #replace withArguments: #('"/"' '"_slash"')].
				self dot: [self call: #replace withArguments: #('"|"' '"_bar"')] ]
			),(Array
			with: [	self dot: [self call: #replace withArguments: #('"&"' '"_amp"')].
				self dot: [self call: #replace withArguments: #('">"' '"_gt"')] ]
			with: [	self dot: [self call: #replace withArguments: #('"<"' '"_lt"')].
				self dot: [self call: #replace withArguments: #('"="' '"_eq"')] ]
			with: [	self dot: [self call: #replace withArguments: #('","' '"_comma"')].
				self dot: [self call: #replace withArguments: #('"@"' '"_at"')] ]
			with: [	self dot: [self call: #replace withArguments: #('"~"' '"_no"')].
				self dot: [self call: #replace withArguments: #('"\\"' '"_bk"')] ]
			)
	];cr.
	self return: [self send: #String with: #s] ! !

! PythonBoot methodsFor: #api !
array_sort_StSystemDictionary
	" Private - Implementation of api function "

	self nextPutLines: '
def merge(left, right):
	if len(left) == 0:
		return right
	if len(right) == 0:
		return left
	result = []
	index_left = index_right = 0
	while len(result) < len(left) + len(right):
		#if left[index_left] <= right[index_right]:
		if self.b(aBlock.value_value_(anArray.unwrap_(left[index_left]),anArray.unwrap_(right[index_right]))):
			result.append(left[index_left])
			index_left += 1
		else:
			result.append(right[index_right])
			index_right += 1
		if index_right == len(right):
			result += left[index_left:]
			break
		if index_left == len(left):
			result += right[index_right:]
			break
	return result
def insertion_sort(array, left=0, right=None):
	if right is None:
		right = len(array) - 1
	for i in range(left + 1, right + 1):
		key_item = array[i]
		j = i - 1
		#while j >= left and key_item < array[j]:
		while j >= left and self.b(aBlock.value_value_(anArray.unwrap_(key_item),anArray.unwrap_(array[j]))):
			array[j + 1] = array[j]
			j -= 1
		array[j + 1] = key_item
	return array
def timsort(array):
	min_run = 32
	n = len(array)
	for i in range(0, n, min_run):
		insertion_sort(array, i, min((i + min_run - 1), n - 1))
	size = min_run
	while size < n:
		for start in range(0, n, size * 2):
			midpoint = start + size - 1
			end = min((start + size * 2 - 1), (n-1))
			merged_array = merge(
				left=array[start:midpoint + 1],
				right=array[midpoint + 1:end + 1])
			array[start:start + len(merged_array)] = merged_array
		size *= 2
	return array
if self.is_Nil(aBlock):
	# def defaultBlock(a,b): return a._lt(b) 
	# aBlock = self.blk(defaultBlock,2)
	aBlock = self.BlockClosure.defaultSortBlock()
timsort(self.handleOf(anArray))
return anArray' trimBlanks! !
