" FFI extension for LuaJIT
Copyright (C) 2019-today Alejandro F. Reimondo <aleReimondo@smalltalking.net>
MIT License - See license.txt file
See http://luajit.org/ext_ffi.html
"!

NativeObject subclass: #FFI category: #ffi!
Lua subclass: #CObject category: #ffi!
CObject subclass: #CData!

!CData methodsFor: #converting !
toNumber
	^# #tonumber: self handle! !

!CData methodsFor: #converting !
toString
	^# #tostring: self handle! !

CData subclass: #CType!
CType buildFunctions: #(
	free
	#(set #(func))
)!

CObject subclass: #CLib!

FFI class instanceVariableNames: #nativeClass!
FFI class buildAccessors: #(
	#(nativeClass initialize 'Lua @ (# #require: #ffi)')
)!

FFI buildNativeStaticFunctions: #(
	#(cdef #(source))
	#(load:global: #(libName aBoolean) CLib)
	#(load #(libName ) CLib)
	#(new: #(_ct) CData)
	#(new:count: #(_ct nelem) CData)
	#(new:count:with: #(_ct nelem init1 ) CData)
	#(new:count:with:with: #(_ct nelem init1 init2 ) CData)
	#(new:count:with:with:with: #(_ct nelem init1 init2 init3 ) CData)
	#(new:count:with:with:with:with: #(_ct nelem init1 init2 init3 init4) CData)
	#(typeof #(_ct) CType)
	#(cast #(_ct with) CData)
	#(metatype #(_ct _metatable) CType)
	#(gc #(_cdata finalizer) CData)
	#(sizeof #(_ct))
	#(sizeof #(_ct count))
	#(alignof #(_ct))
	#(offsetof #(_ct field))
	#(istype #(_ct _obj))
	errno #(errno #(newerr))
	#(string #(_ptr))
	#(string #(_ptr len))
	#(copy #(_dst _src len))
	#(copy #(_dst _str))
	#(fill #(_dst len))
	#(fill #(_dst len c))
	#(abi #(param))
)!

! FFI class methodsFor: #loading !
load: libraryFileName cdef: source
	" Returns the result of loading the C library at libraryFileName. "

	self cdef: source.
	^self load: libraryFileName ! !

! FFI class methodsFor: #accessing !
c
	" Returns the default C library namespace (ffi.C).
	It binds to the default set of symbols or libraries on the target system.
	These are more or less the same as a C compiler would offer by default, without specifying extra link libraries.
	"

	^Lua @ (self nativeClass @ #C) ! !

! FFI class methodsFor: #accessing !
os
	" Returns the target OS name. "

	^self nativeClass @ #os ! !

! FFI class methodsFor: #accessing !
arch
	" Returns the target architecture name. "

	^self nativeClass @ #arch ! !