
" Implement build information method "
| date |
[ date := Smalltalk nativeLanguage = #js
	ifTrue: [ (DateTime today asString upTo: $() trimBlanks ]
	"ifFalse: [ Lua os date ]"
] on: Exception do: [:ex| ^self note: ex description ].
date isNil ifTrue: [ ^self ].
Smalltalk
	implement: #buildString
	comment: 'Return the birth information of this system.'
	as: $^ ,date printString
	category: #version ! !

! Smalltalk methodsFor: #platform-tools !
defaultWorkspaceList
	" Private - Return the (captured) workspace list of the receiver.
	Return an array of tuples; each tuple is anArray of the form
		#( title code )
	where:
		title is aString to be shown as a title of the workspace page.
		code is the text to be shown as contents of workspace page.
	"

	| result |
	result := #().
	(Smalltalk includesKey: #WorkspaceList) ifTrue: [
		result addAll: (Smalltalk at: #WorkspaceList)
	].
	(Smalltalk includesKey: #U8) ifTrue: [
		result addAll: U8 commonExpressions
	].
	^result! !

! Smalltalk methodsFor: #platform-tools !
workspaceList
	" Return the (captured) workspace list of the receiver. "

	^self defaultWorkspaceList! !

! Smalltalk methodsFor: #platform-tools !
defaultAppList
	" Private - Return the (captured) apps of the receiver.
	Return an array of tuples; each tuple is anArray of the form
		#( title comment code )
	where:
		title is aString that can include an icon name inside brackets.
		comment is aString with short description of the app.
		code is the code to evaluate to open the app.
	"

	| result |
	result := #().
	(Smalltalk includesKey: #AppList) ifTrue: [
		result addAll: (Smalltalk at: #AppList)
	].
	(Smalltalk includesKey: #U8Configuration) ifTrue: [
		result addAll: U8Configuration appList
	].
	result addAll: (self allClassMethodsList: #(sample samples)).
	^result! !

! Smalltalk methodsFor: #platform-tools !
appList
	" Return the (captured) apps of the receiver. "

	^self defaultAppList! !

! Smalltalk methodsFor: #platform-tools !
allClassMethodsList: choices
	" Private - Return the list of all (class) methods in the receiver matching choices. "

	| result |
	result := Array new.
	self classes do: [:cls|
		cls class methodsDo: [:mth| | comment |
			(mth selector argumentCount = 0 and: [
			((choices includes: mth category) or: [
			  choices includes: mth selector
			]) and: [ comment := mth comment.
			 ((comment asString upTo: $-) asLowercase includes: #private) not
			]]) ifTrue: [ | title code |
				title := code := cls name ,$  ,mth selector.
				result add: (Array
					with: title with: (comment upTo: $.)
					with: code)
			].
		].
	].
	^result sorted: [:a :b| a first < b first ]! !

! Smalltalk methodsFor: #platform-tools !
exampleList
	" Return the (captured) examples of the receiver. "

	^self allClassMethodsList: #(example)! !

! Smalltalk methodsFor: #platform-tools !
screen: aBlock
	" Return the (captured) screen of the receiver. "

	^'S8 system running as console application'! !

! Smalltalk methodsFor: #platform-tools !
fileContentsAt: pathName
	" Return the contents of file at pathName (or nil). "

	^[ pathName fileContents ] on: Exception do: [:ex| nil ]! !

! Smalltalk methodsFor: #platform-tools !
output: aString toFile: pathName
	" Save aString onto a file at pathName. "

	^aString outputToFile: pathName! !

! Smalltalk methodsFor: #platform-tools !
append: aString toFile: pathName
	" Append aString onto a file contents at pathName. "

	^aString appendToFile: pathName! !