" NOTE: This file is optional an experimental.
  Compiler templates for native access & calls
"

! APIBuilder methodsFor: #templates-iv !
ivGet
	" Template - Getter of instance variable. "

	^token! !

! APIBuilder class methodsFor: #templates-iv !
ivGet: ivName
	" Template - Return the template node to access instance variable. "

	^self templateFrom: self>>#ivGet doing: [:node|
		(node isVariable and: [node value = #token])
			ifTrue: [ node value: ivName ]
	] on: [:node| node selector: ivName ]! !

! APIBuilder methodsFor: #templates-iv !
ivSet: value
	" Template - Setter of instance variable. "

	token := value! !

! APIBuilder methodsFor: #templates-parsing !
nextLiteralArrayIn: stream
	" Private - Return the next literal array read from stream (or nil). "

	| result |
	stream skipSeparators.
	stream peek = $# ifFalse: [ ^nil ]. stream next.
	stream peek = $( ifFalse: [ ^nil ]. stream next.
	result := Array new.
	[ stream skipSeparators.
	  stream peek = $) ifTrue: [ stream next.^result ].
	  stream peek = $# ifFalse: [ ^nil ]. stream next.
	  result add: (stream peek = $(
		ifTrue: [
			stream position: stream position - 1.
			self nextLiteralArrayIn: stream
		] ifFalse: [ stream upTo: $  ]).
	  stream atEnd
	] whileFalse.
	^nil! !

! APIBuilder class methodsFor: #templates-iv !
ivSet: ivName
	" Template - Return the template node to access instance variable. "

	^self templateFrom: self>>#ivSet: doing: [:node|
		(node isVariable and: [node value = #token])
			ifTrue: [ node value: ivName ]
	] on: [:mth| mth selector: ivName ,$: ]! !


! APIBuilder methodsFor: #templates-call !
apiCall: arg1
	" Template - Basic native call. "

	^self handle #fnCall: arg1! !

! APIBuilder methodsFor: #templates-call !
wrapCall: arg1
	" Template - Wrapped native call. "

	^NativeObject @ (self handle #fnCall: arg1)! !

! APIBuilder methodsFor: #templates-call !
apiArgumentNames: args
	" Private - Return the arguments names in args. "

	^args collect: [:arg|
		arg first = $_
		ifTrue: [ arg copyFrom: 2 to: arg size ]
		ifFalse: [ arg ]
	]! !

! APIBuilder methodsFor: #templates-call !
apiArgumentNode: arg
	" Private - Return the node to return api argument (or nil). "

	arg first = $_ ifTrue: [
		^SendNode new
			receiver: (VariableNode new value: (arg copyFrom: 2 to: arg size));
			selector: #json;
			yourself
	].
	^VariableNode new value: arg! !

! APIBuilder methodsFor: #templates-call !
apiWrap: aGlobalReferenceNode to: type
	" Private - Return true if the node can be wrapped with type. "

	aGlobalReferenceNode value: type.
	^true! !

! APIBuilder methodsFor: #templates-call !
apiTransform: root selector: selector arguments: args type: type
	" Private - Return true if the receiver can transform node to host a function call. "

	| send |
	send := root.
	type notNil ifTrue: [ "Type @ (apiCall)"
		self apiWrap: send receiver to: type.
		send := send arguments first.
	].
	send selector: selector.
	send arguments: (args collect: [:arg| | node |
		node := self apiArgumentNode: arg.
		node isNil ifTrue: [ ^false ].
		node
	]).
	^true! !

! APIBuilder class methodsFor: #templates-call !
fnCall: parameters
	" Template - Return the template node to call a native function. "

	| spec selector args type method result builder |
	builder := self new.
	spec := builder nextLiteralArrayIn: parameters stream.
	spec isArray ifFalse: [ ^nil ].
	(spec size between: 2 and: 3) ifFalse: [ ^nil ].
	selector := spec first.
	args := spec second.
	type := spec size = 3 ifTrue: [ spec last ].
	method := type isNil
		ifTrue: [ self >> #apiCall: ]
		ifFalse: [ self >> #wrapCall: ].
	result := self templateFrom: method with: [:mth|
		mth
			selector: selector;
			arguments: (builder apiArgumentNames: args).
		(builder apiTransform: mth nodes first
				nodes first
				nodes first
			selector: $# ,selector
			arguments: args
			type: type) ifFalse: [ ^nil ].
		mth
	].
	^result! !

! Behavior methodsFor: #templates-iv !
buildGetterCode: aSelector
	" Private - Return source code to get aSelector. "

	^'^' ,aSelector ,'
"#api ivGet: ',aSelector ,'"'! !

! Behavior methodsFor: #templates-iv !
buildSetterCode: aSelector
	" Private - Return source code to get aSelector. "

	^aSelector ,' := anObject
"#api ivSet: ',aSelector ,'"'! !

"--------------------------------------------------------------------------"

! NativeObject class methodsFor: #templates-call !
buildFunction: spec arguments: array prologue: prologue epilogue: epilogue 
	" Private - Generate code to delegate message on instance handle. "

	| header code delegate selector args |
	header := '' stream. code := '' stream.
	args := self apiArgumentsFrom: array.
	selector := self apiSelector: spec forArguments: args.
	delegate := 'self handle'.
	self
	  apiEncode: delegate
		call: selector arguments: args
		on: code header: header;
	  implement: (self stMessageHeader: header contents)
		comment: 'Delegate ' ,spec ,' to handle.'
		as: '^'	,prologue ,code contents ,epilogue
			,(self	apiTemplateFor: delegate
				call: selector arguments: args
				prologue: prologue epilogue: epilogue)
		category: #wrappers! !

! NativeObject class methodsFor: #templates-call !
apiEncode: delegate
	call: selector arguments: args
	on: code header: header
	" Private - Generate code to call function onto code&header streams. "

	| input argStream msg |
	input := selector stream.
	argStream := args stream.
	code	nextPutAll: delegate ,' #'.
	code	nextPut: input peek.
	header	nextPut: input next.
	[ argStream atEnd ] whileFalse: [ | param expr |
		argStream peek isString
		ifTrue: [ param := argStream next. expr := param ]
		ifFalse: [ param := argStream peek first. expr := param ,' ' ,argStream next last. ].
		msg := (input upTo: $:) ,': '.
		code   nextPutAll: msg, expr ,' '.
		header nextPutAll: msg, param ,' '.
	].
	msg := input upToEnd.
	code   nextPutAll: msg.
	header nextPutAll: msg.! !

! NativeObject class methodsFor: #templates-call !
apiTemplateFor: delegate
	call: selector arguments: args
	prologue: prologue epilogue: epilogue
	" Private - Return the spec for code template to build the function call. "

	| empty type parameters |
	empty := ''.
	prologue notEmpty ifTrue: [
		(prologue includes: $@) ifFalse: [ ^empty ].
		type := (prologue upTo: $@) trimBlanks.
		(Smalltalk includesKey: type) ifFalse: [ ^empty ].
	].
	parameters := Array
		with: selector
		with: (args collect: [:arg|
			arg isString ifTrue: [ arg ]
			ifFalse: [
				(arg size = 2 and: [ arg last = #json ])
					ifFalse: [ ^empty ].
				'_' ,arg first
			]
		]).
	type notNil ifTrue: [ parameters add: type ].
	^'
"#api fnCall: ',parameters asLiteral ,'"'! !


! self cancelFileIn !
"---DEBUG AND TESTING----------------------------------------------------------------------------"
! CompiledMethod methodsFor: #debug !
compilerNodes
	" Return compilation nodes result of parsing the receiver's source. "

	^self classField compiler parseCode: self source! !

NativeObject subclass: #Mine instanceVariableNames: 'x y z'!
Mine buildFunctions: #(
	#(between #(x y))
	#(toPoint #(x y) Point)
	#(to:test: #(_x _y) Test)
)!
Mine buildAccessors; browse!


(Mine>>#between:y:) compilerNodes inspect!

| api |
api := (1 to: 100) collect: [:i| Array with: #test ,i with: #(x y _t #(u json) ) with: #Point ].
Mine selectors do: [:each| Mine removeSelector: each ].
[Mine buildFunctions: api] ms
