smalltalk = (function() -- Snapshot built on Fri Mar 15 2019 10:00:57 GMT-0300 ()
-- Embedded image follows
--[[
S8 - A modern Smalltalk environment running on the "invisible machine".
Copyright (C) 2018 Alejandro F. Reimondo <aleReimondo@smalltalking.net>
MIT License - See license.txt file

Alpha - Sept 2018

Note that this is the Beta version of Lua execution support
 and is under social development (Jan/2019).
For questions and bug reports please join us at S8 Facebook group
	https://www.facebook.com/groups/s8smalltalk/
--]]

local smalltalk = {}
smalltalk["$nativeLanguage"] = "lua"
smalltalk.useManager = false

function smalltalk.do_onError_(self, aBlock, errorBlock)
	local status, result = pcall(aBlock)
	if status then return result end
	return errorBlock(result) 
end

function smalltalk.isNil_(self, aValue)
	if (aValue==nil) then return true end
	if (aValue==smalltalk) then return false end
	if (aValue.isNil==nil) then return false end
	return aValue:isNil()==true 
end

function smalltalk.notNil_(self, aValue)
	if (aValue==nil) then return false end
	if (aValue==smalltalk) then return true end
	if (aValue.isNil==nil) then return true end
	return aValue:isNil()~=true
end

function smalltalk.newConstructor (self, super)
	-- return a new constructor of instances inheriting from super (constructor)
	-- super can be nil to return a constructor for a root class
	local constructor = {}
	local prototype = {}
	prototype.__index = prototype
	if (super)	then setmetatable(prototype,super.prototype)
	else
		local dnu = {}
		dnu.__index = function (rootPrototype, key)
			--print("--keyNotFound: "..key)
			smalltalk.dnu=key
			return nil
		end
		setmetatable(prototype,dnu)
	end
	setmetatable(constructor,prototype)
	constructor.prototype = prototype
	prototype.__call = function (f,rcv,...)
		if (f==nil and smalltalk.dnu) then
			if (string.find(smalltalk.dnu,"[%a_%+%-%/%*%|%&%=%<%>%,%~%\\]")~=1) then return nil end
--DEBUG
print("--DNU "..tostring(smalltalk.dnu))
if (tostring(smalltalk.dnu):first():isUppercase()) then 
  return nil
end
--DEBUG DNU detected dump stack trace
if debug then
	print(debug.traceback("Stack trace"))
	print(debug.getinfo(1))
	print("Stack trace end")
end
--if (smalltalk.dnu=="arguments") then error("DNU exiting...") end
error("DNU")
			-- Syntetize a doesNotUnderstand message send
			local args = (pack or table.pack)(...)
			local msg = smalltalk.Message:receiver_selector_arguments_(
				rcv, smalltalk.dnu,
				smalltalk:newArray(args)
			)
			smalltalk.dnu = nil
			local result = nil
			if (smalltalk.isObject_(rcv)) then
				result = rcv:doesNotUnderstand_( msg )
			else -- DNU goes to smalltalk for native objects
				result = smalltalk:doesNotUnderstand_( msg )
			end
			return result
		end
		local result = {}
		setmetatable(result,prototype)
		return result
	end
	return constructor
end

local StObject = smalltalk:newConstructor()
local StBehavior = smalltalk:newConstructor(StObject)
local StClass = smalltalk:newConstructor(StBehavior)
local StMetaclass = smalltalk:newConstructor(StBehavior)
StMetaclass.prototype["$meta"] = true
local StMethod = smalltalk:newConstructor(StObject)
local StSystemDictionary = smalltalk:newConstructor(StObject)

function smalltalk.will_be_(self, anObject, className)
	-- return an object that will become an instance of class (named className) when accessed
	local result = anObject
	local mt = {}
	mt.__index = function(table, key)
		if (self[className]) then
			self:become_a_(table,self[className])
		end
		return table[key]
	end
	setmetatable(result,mt)
	return result
end

function smalltalk.linkNative_to_ (self, instance, aClass)
	-- link native species of instance to delegate to aClass instance
	local mt=getmetatable(instance)
	if (mt)	then
		local index=mt["__index"]
		setmetatable(index,aClass["$fn"].prototype)
	else
		if debug then
			self:do_onError_(function()
				debug.setmetatable(instance,aClass["$fn"].prototype)
			end ,function()
				aFile:write("\n-- ERROR cant set metatable for "..tostring(instance))
			end)
		else
			print("-- WARNING: debug support missing")
		end
	end
end

function smalltalk.newArray(self, object)
	-- return a new Array instance mutating the provided object
	return self:will_be_(object or {},"Array")
end

function smalltalk.become_a_(self, object, aClass)
	-- mutate object to an instance of aClass
	setmetatable(object,aClass["$fn"].prototype)
	return object
end

function smalltalk.setProtoOf_to_ (self, jsObject, proto)
	local mt = {}
	mt.__index = proto
	setmetatable(jsObject,mt) -- jsObject.__proto__ = proto
end

function smalltalk.bindRoots(self, metaclass)
	-- Private - Link the system roots to ensure proper delegation chains
	local objectClassPrototype = self.Object["$klass"]["$fn"].prototype
	self:setProtoOf_to_(objectClassPrototype,self.Class["$fn"].prototype)
	self:setProtoOf_to_(self.Smalltalk["$klass"]["$fn"].prototype,objectClassPrototype)
	self:setProtoOf_to_(self.Behavior["$klass"]["$fn"].prototype,objectClassPrototype)
	self.Object["$klass"]["$superclass"] = self.Class
	self:become_a_(smalltalk,self.Smalltalk)
end

function smalltalk.klass(self, spec)
	local aClass
	if (spec.meta) then
		aClass = StMetaclass()
	else
		local metaSpec = {meta=true}
		metaSpec.className = spec.className .." class"
		if (spec.superclass) then
			metaSpec.superclass = spec.superclass["$klass"]
		end
		local metaclass = self:klass(metaSpec)
		aClass = metaclass["$fn"]()
		if (aClass["$klass"]~=metaclass) then -- this should NOT happen
			print("--! "..(spec.className).." BROKEN metaclass="..tostring(aClass["$klass"]))
			aClass["$klass"] = metaclass
		end
	end
	self:setClass(aClass,spec)
	aClass["$fn"].prototype["$klass"] = aClass
	aClass["$fn"].prototype["$methods"] = {}
	if (aClass["$className"] == "Metaclass") then
		self:bindRoots(aClass)
	end
	return aClass
end

function smalltalk.setVarNames(self, aClass, iVarNames, aValue)
	if (iVarNames==nil) then iVarNames = self:newArray() end
	aClass["$iVarNames"] = iVarNames
	local i = #iVarNames
	while (i > 0) do
		aClass["$fn"].prototype["@"..iVarNames[i]] = aValue
		i = i - 1
	end
end

function smalltalk.setClass (self, aClass, spec)
	aClass["$className"] = spec.className
	if (spec.meta==nil) then
		aClass["$klass"]["@instanceClass"] = aClass
	end
	aClass["$category"] = spec.category or ""
	aClass["$superclass"] = spec.superclass --or nil
	if (aClass["$superclass"]) then
		aClass["$superclass"]["@subclasses"] = nil
		if (aClass["$klass"]) then
			if (spec.meta and (aClass["$klass"]~=self.Metaclass)) then
				aClass["$klass"]["$superclass"] = aClass["$superclass"]["$klass"]
			end

			if (self.Metaclass~=nil) then -- OBSOLETE - Metaclass validation & repair
				if (self.Metaclass["$superclass"]~=self.Behavior) then
					print("-- Warning broken Metaclass spec.meta="..tostring(spec.meta).." aClass="..aClass["$className"])
					self.Metaclass["$superclass"]=self.Behavior
				end
			end

			if (aClass["$klass"]["$superclass"]) then
				aClass["$klass"]["$superclass"]["@subclasses"] = nil
			end
		end
	end
	aClass["$fn"] = self:fnInSpec(spec)
	self:setVarNames(aClass,spec.iVarNames,nil)
	return aClass
end

function smalltalk.fnInSpec (self, spec)
	if (spec.meta and (spec.superclass == nil)) then
		spec.superclass = self.Class
	end
	if (spec.fn) then
		if (spec.superclass) then
			self:setProtoOf_to_(spec.fn.prototype,spec.superclass["$fn"].prototype)
		end
		return spec.fn
	end
	local superC = nil
	if (spec.superclass) then
		superC = spec.superclass["$fn"]
	end
	return self:newConstructor(superC)
end

function smalltalk.classes (self)
	local result = self:newArray()
	for key, aClass in pairs(smalltalk) do
		if (type(aClass)=="table") then
			if (aClass.isClass and aClass:isClass()) then
				result[#result+1] = aClass
			end
		end
	end
	return result
end

function smalltalk.computeSubclasses (self)
	if (self.Metaclass==nil) then print("-- Warning system broken/incomplete") end
	if (self.Metaclass["$superclass"]~=self.Behavior) then -- OBSOLETE - Metaclass validation & repair
		print("-- Warning broken Metaclass")
		self.Metaclass["$superclass"]=self.Behavior
	end
	local classes = self:classes()
	for _,cls in pairs(classes) do
		cls["@subclasses"] = self:newArray()
		cls["$klass"]["@subclasses"] = self:newArray()
	end
	for _,cls in pairs(classes) do
		if (cls["$superclass"]~=nil) then
			local array = cls["$superclass"]["@subclasses"]
			array[#array+1] = cls
			array = cls["$superclass"]["$klass"]["@subclasses"]
			array[#array+1] = cls["$klass"]
		end
	end
	if (self.Class) then
		self.Class["@subclasses"] = self:newArray()
		self.Class["$klass"]["@subclasses"] = self:newArray()
	end
end

function smalltalk.subclasses (self, aClass)
	if (aClass["@subclasses"]==nil) then
		self:computeSubclasses()
	end
	return aClass["@subclasses"]
end

function smalltalk.allSubclassesDo (self, aClass, aBlock)
	for _,cls in pairs(self:subclasses(aClass)) do
		aBlock(cls)
		self:allSubclassesDo(cls,aBlock)
	end
end

function smalltalk.mapClassName (self, className, category, fn, superclass)
	local spec = {
		className = className, 
		category = category, 
		superclass = superclass,
		fn = fn }
	if (self[className]) then
		return self:setClass(self[className],spec)
	end
	self[className] = self:klass(spec)
	return self[className]
end

function smalltalk.isArray_equalTo_(self, table1, table2)
	if (table1==table2) then return true end
	if (#table1 ~= #table2) then return false end
	local i = #table1
	while (i > 0) do
		if (table1[i] ~= table2[i]) then return false end
		i = i - 1
	end
	return true
end

function smalltalk.isObject_equalTo_(self, receiver, anObject)
	if (receiver==anObject) then return true end
	if (type(receiver)=="table" and type(anObject)=="table") then
		if (receiver["_eq"]) then
			return receiver:_eq(anObject)
		end
	end
	return false
end

function smalltalk.object_comma_(self, receiver, anObject)
	if (type(receiver)=="string") then
		if (type(anObject)=="string") then return receiver..anObject end
		return self:object_comma_(receiver,tostring(anObject))
	end
	if (not (type(receiver)=="table")) then
		return self:object_comma_(tostring(receiver),anObject)
	end
	return receiver:_comma(anObject)
end

function smalltalk.boolean_and_(self, receiver ,option)
	if (receiver~=true) then return false end
	if (type(option)=="function") then return option() end
	if (type(option)=="boolean") then return option end
	return option:evaluate()==true;
end

function smalltalk.boolean_or_(self, receiver, option)
	if (receiver==true) then return true end
	if (type(option)=="function") then return option() end
	if (type(option)=="boolean") then return option end
	return option:evaluate()==true;
end

function smalltalk.booleanNot_(self, receiver)
	if (receiver==true) then return false end
	if (type(receiver)~="table") then return not receiver end
	return receiver:__not()
end

function smalltalk.asCharacter_(self, receiver)
	if (type(receiver)=="number") then return string.char(receiver) end
	return receiver:asCharacter()
end

function smalltalk.toString_(self, receiver)
	if (type(receiver)=="table" and receiver.toString) then return receiver:toString() end
	return tostring(receiver)
end

function smalltalk.printString_(self, receiver)
	if (type(receiver)=="table" and receiver.printString) then return receiver:printString() end
	return tostring(receiver)
end

function smalltalk.asString_(self, receiver)
	if (type(receiver)=="table" and receiver.asString) then return receiver:asString() end
	return tostring(receiver)
end

function smalltalk.mustBeNumber_(self, receiver)
	if (type(receiver)=="number") then return receiver end
	if (type(receiver)=="table") then return receiver:mustBeNumber() end
	return self:error_("Must be a Number")
end

function smalltalk.math_max_(self, receiver, aNumber)
	if (type(receiver)~="number") then return receiver:max_(aNumber) end
	if (type(aNumber)~="number") then return aNumber:max_(receiver) end
	if receiver>aNumber then return receiver end
	return aNumber
end

function smalltalk.math_min_(self, receiver, aNumber)
	if (type(receiver)~="number") then return receiver:min_(aNumber) end
	if (type(aNumber)~="number") then return aNumber:min_(receiver) end
	if receiver<aNumber then return receiver end
	return aNumber
end

function smalltalk.addClass (self, className, superclass, iVarNames, category)
	self:debugAddClass(className, superclass, iVarNames, category)
	if(self[className]) then
		local aClass = self[className]
		local sameShape = (aClass["$superclass"] == superclass)
			and (self:isArray_equalTo_(aClass["$iVarNames"],iVarNames))
		if (sameShape and (aClass["$category"] == category)) then
			return aClass
		end
		self:aboutToChangeClass(aClass,superclass,iVarNames,category)
		if(aClass["$superclass"]) then
			aClass["$superclass"]["@subclasses"] = nil
			local aMeta = aClass["$klass"]
			if (aMeta and superclass) then
				aMeta["$superclass"]["@subclasses"] = nil
				aMeta["$superclass"] = superclass["$klass"]
			end
		end
		aClass["$superclass"] = superclass
		if(superclass and (superclass["@subclasses"])) then
			if (superclass["@subclasses"] == nil) then
				superclass["@subclasses"] = self:newArray()
			end
			superclass["@subclasses"].push(aClass) 
		end
		self:setVarNames(aClass,iVarNames,nil)
		aClass["$category"] = category or aClass["$category"]
		if (sameShape == nil) then self:classShapeChanged(aClass) end
	else
		self:aboutToCreateClass(className,superclass,iVarNames,category)
		self[className] = self:klass({
			className = className, 
			iVarNames = iVarNames,
			superclass= superclass
		})
		self[className]["$category"] = category or ""
	end
	self:classChanged(self[className])
	return self[className]
end

function smalltalk.renameClass (self, aClass, aString)
	self[aClass["$className"]] = nil
	aClass["$className"] = aString
	self[aString] = aClass
	self:classChanged(aClass)
end

function smalltalk.removeClass (self, aClass)
	local deleted = "Deleted-"
	if (aClass["$superclass"]) then aClass["$superclass"]["@subclasses"] = nil end
	self[aClass["$className"]] = nil
	aClass["$className"] = deleted .. aClass["$className"]
	aClass["$klass"]["$className"] = deleted .. aClass["$klass"]["$className"]
	self:classChanged(aClass)
end

function smalltalk.metaVarNames (self, aMetaClass, iVarNames)
	if (self:isArray_equalTo_(aMetaClass["$iVarNames"],iVarNames)) then return false end
	self:setVarNames(aMetaClass,iVarNames,nil)
	self:classShapeChanged(aMetaClass)
	self:classChanged(aMetaClass)
	return true
end

function smalltalk.smDo (self, block)
	if (self.useManager and self.SystemManager
		and self.SystemManager.currentOrNil) then
		local sm = self.SystemManager:currentOrNil()
		if (sm) then return block(sm) end
	end
end

function smalltalk.classChanged (self, aBehavior)
	self:smDo(function(sm) sm:classChanged_(aBehavior) end)
end
function smalltalk.classShapeChanged (self, aBehavior)
	self:smDo(function(sm) sm:classShapeChanged_(aBehavior) end)
end
function smalltalk.aboutToCreateClass (self, className, superclass, iVarNames, category)
	self:smDo(function(sm)
		sm:aboutToCreateClass_superclass_iVarNames_category_(className,superclass,iVarNames,category)
	end)
end
function smalltalk.aboutToChangeClass (self, aClass, superclass, iVarNames, category)
	self:smDo(function(sm)
		sm:aboutToChange_superclass_iVarNames_category_(aClass,superclass,iVarNames,category)
	end)
end
function smalltalk.methodBoundTo (self, method, klass)
	self:smDo(function(sm) sm:methodBound_to_(method,klass) end)
	return method
end
function smalltalk.aboutToBindTo (self, method,klass)
	self:smDo(function(sm) sm:aboutToBind_to_(method,klass) end)
	return method
end
function smalltalk.aboutToRemoveFrom (self, method, klass)
	self:smDo(function(sm) sm:aboutToRemove_from_(method,klass) end)
	return method
end
function smalltalk.methodRemovedFrom (self, method, klass)
	self:smDo(function(sm) sm:methodRemoved_from_(method,klass) end)
	return method
end
function smalltalk.objectNotFound (self, object, indexOrKey, collection)
	self:smDo(function(sm) sm:object_at_notFoundIn_(object,indexOrKey,collection) end)
	return collection
end

function smalltalk.whoImplements (self, klass, jsSelector)
	local cls = klass
	repeat
		if(rawget(cls["$fn"].prototype,jsSelector)) then return cls end
		cls = cls["$superclass"]
	until (cls==nil)
	return nil
end

function smalltalk.superImplementor (self, klass, jsSelector)
	if (self:isNil_(klass["$superclass"])
		or self:isNil_(klass["$superclass"]["$fn"])) then
		return klass:missingImplementorFor_(jsSelector) end
	local cls = self:whoImplements(klass["$superclass"],jsSelector)
	if (self:isNil_(cls)) then
		return klass["$superclass"]:missingImplementorFor_(jsSelector)
	end
	return cls["$fn"].prototype[jsSelector]
end

function smalltalk.addMethod (self, jsSelector, method, klass)
	self:aboutToBindTo(method,klass)
	klass["$fn"].prototype[jsSelector] = method["$fn"]
	klass["$fn"].prototype["$methods"][method["$selector"]] = method
	method["@classField"] = klass
	return self:methodBoundTo(method,klass)
end

function smalltalk.bind (self, klass, stSelector, jsSelector, jsFunction, category, stSource, nativeSource)
	self:debugBind(klass,stSelector,jsSelector,jsFunction,category,stSource,nativeSource)
	local method = self.CompiledMethod["$fn"]()
	method["$selector"] = stSelector
	method["$fn"]	= jsFunction
	method["$category"] = category
	method["$source"] = stSource
	method["$nativeSource"] = nativeSource
	method["$generation"] = self["$state"]
	if (self:notNil_(klass)) then
		if(jsSelector == 0) then jsSelector = string.gsub(stSelector,":","_") end
		self:addMethod(jsSelector,method,klass)
	end
	return method
end

function smalltalk.removeMethod (self, aClass, aMethod)
	self:aboutToRemoveFrom(aMethod,aClass)
	aClass["$fn"].prototype[aMethod["$selector"]:asSelector()] = nil
	aClass["$fn"].prototype["$methods"][aMethod["$selector"]] = nil
	return self:methodRemovedFrom(aMethod,aClass)
end

function smalltalk.newMethod (self, currentClass, selector, fn, source, nativeSource)
	return self:bind(nil,selector,nil,fn,nil,source,nativeSource)
end
function smalltalk.method (self, spec) -- OBSOLETE
	return self:bind(nil,spec.selector,nil,spec.fn,spec.category,spec.source,spec.nativeSource)
end

function smalltalk.readJSObject (self, js)
	print("--[NIY] smalltalk:readJSObject()")
	return nil
end

function smalltalk.basen(self, n, b)
	local floor,insert = math.floor, table.insert
	n = floor(n)
	if not b or b == 10 then return tostring(n) end
	local digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	local t = {}
	local sign = ""
	if n < 0 then
		sign = "-"
		n = -n
	end
	repeat
		local d = (n % b) + 1
		n = floor(n / b)
		insert(t, 1, digits:sub(d,d))
	until n == 0
	return sign .. table.concat(t,"")
end

function smalltalk.globalsDo (self, action)
	for key, aValue in pairs(smalltalk) do
		if (type(aValue)=="table") then
			if (aValue.isClass and not aValue:isClass())
				then action(key) end
		end
	end
	return smalltalk
end

function smalltalk.launchImage (self, imageName)
	self.useManager = true
	if(self:notNil_(imageName)) then
		self["$imageFileName"] = imageName
		dofile(imageName) --load(imageName)
	end
	self["$state"] = "launching"
	if(self.launch) then self:launch() end
	self["$state"] = "running"
	return smalltalk
end

function smalltalk.is_kindOf_ (self, anObject, aClass)
	if(self:isNil_(anObject) or (self:isNil_(anObject.isKindOf_))) then return false end
	return anObject:isKindOf_(aClass)
end

function smalltalk.responds_to_ (self, anObject, aSelector)
	if(self:isNil_(anObject) or (self:isNil_(anObject.respondsTo_))) then return false end
	return anObject:respondsTo_(aSelector)
end

function smalltalk.debugBind (self,klass,stSelector,jsSelector,jsFunction,category,stSource,nativeSource) return self end
function smalltalk.debugAddClass (self,className, superclass, iVarNames, category) return self end

function smalltalk.toString (self)
	return "smalltalk"
end

function smalltalk.bindCore (self) --bind the minimal functions (e.g i/o)
	local global=_G
	if (global==nil) then return false end

	function global.quit ()
		os.exit()
		return false
	end
	function global.version ()
		return _VERSION
	end
	function global.fileContents (fileName)
		local f,err = io.open(fileName, "rb")
		if (f==nil) then return nil end
		local content = f:read("*all")
		f:close()
		return content
	end
	global.read=global.fileContents
	function global.fileWrite (fileName,contents,mode)
		local f = assert(io.open(fileName, mode))
		f:write(contents)
		f:close()
		return contents:size()
	end

	function global.fileIn (fileName)
		local contents = read(fileName)
		if (contents==nil) then 
			smalltalk:error_("Missing file "..tostring(fileName)) 
		end
		return contents:fileIn()
	end
	function global.emmit (fileName)
		return smalltalk:emmit_(fileName)
	end
	function global.fileInJS (fileName)
		return smalltalk:eval_(read(fileName))
	end

	function global.exportCategory (aString)
		return smalltalk.Exporter:exportCategory_(aString)
	end
	function global.saveImage ()
		return smalltalk:saveImage()
	end
	function global.saveImageAs (fileName)
		return smalltalk:saveImageAs_(fileName)
	end

	function global.isFileOlderThan (fileName,referenceFileName)
		local f,err = io.open(fileName, "r")
		if (f==nil) then return false end
		f:close()
		f,err = io.open(referenceFileName, "r")
		if (f==nil) then return false end
		f:close()
		-- there is NO way to get the datetime of a file in Lua
		return true
	end

	self["$imageFileName"] = self["$imageFileName"] or "s8.image.lua"
	return true
end

function smalltalk.startup (self)
	self["$state"] = "booting"
	self.OverwriteMethods = self.OverwriteMethods or true
	self.DumpOverwritenMethods = self.DumpOverwritenMethods or false

	self["$state"] = "singularity"
	self:mapClassName('Object','Kernel',StObject)
	self:mapClassName('Smalltalk','Kernel',StSystemDictionary,self.Object)
	self:mapClassName('Behavior','Kernel',StBehavior,self.Object)
	self:mapClassName('Class','Kernel',StClass,self.Behavior)
	self:mapClassName('Metaclass','Kernel',StMetaclass,self.Behavior)
	self:mapClassName('CompiledMethod','Kernel',StMethod,self.Object)
	self:mapClassName('Number','Kernel',nil,self.Object)
	self:mapClassName('BlockClosure','Kernel',nil,self.Object)
	self:mapClassName('Boolean','Kernel',nil,self.Object)
--	self:mapClassName('DateTime','Kernel',Date,self.Object)
	self:mapClassName('UndefinedObject','Kernel',nil,self.Object)
	self:mapClassName('Collection','Kernel',nil,self.Object)
	self:mapClassName('SequenceableCollection','Kernel',nil,self.Collection)
	self:mapClassName('String','Kernel',nil,self.SequenceableCollection)
	self:mapClassName('Array','Kernel',nil,self.SequenceableCollection)
--	self:mapClassName('RegularExpression','Kernel',RegExp,self.Object)
	self["$state"] = "whiteHole"
	self:linkNative_to_("",self.String)
	self:linkNative_to_(function()end,self.BlockClosure)
	self:linkNative_to_(true,self.Boolean)
	self:linkNative_to_(nil,self.UndefinedObject)
	self:linkNative_to_(1,self.Number)
	self["$state"] = "wormHole"
	self:bindCore()
	self["$state"] = "startup"
	return self
end

-- DEBUG ----
function xprint(obj,...)
   if type(obj) == 'table' then
      local mt = getmetatable(obj)
      if mt and mt.__tostring__ then
         io.write(mt.__tostring__(obj))
      else
         local tos = tostring(obj)
         local obj_w_usage = false
         if tos and not string.find(tos,'table: ') then
            if obj.usage and type(obj.usage) == 'string' then
               io.write(obj.usage)
               io.write('\n\nFIELDS:\n')
               obj_w_usage = true
            else
               io.write(tos .. ':\n')
            end
         end
         io.write('{')
         local tab = ''
         local idx = 1
         for k,v in pairs(obj) do
            if idx > 1 then io.write(',\n') end
            if type(v) == 'userdata' then
               io.write(tab .. '[' .. k .. ']' .. ' = <userdata>')
            else
               local tostr = tostring(v):gsub('\n','\\n')
               if #tostr>40 then
                  local tostrshort = tostr:sub(1,40)
                  io.write(tab .. '[' .. tostring(k) .. ']' .. ' = ' .. tostrshort .. ' ... ')
               else
                  io.write(tab .. '[' .. tostring(k) .. ']' .. ' = ' .. tostr)
               end
            end
            tab = ' '
            idx = idx + 1
         end
         io.write('}')
         if obj_w_usage then
            io.write('')
         end
      end
   else
      io.write(tostring(obj))
   end
   if select('#',...) > 0 then
      io.write('    ')
      print(...)
   else
      io.write('\n')
   end
end
-- DEBUG ----

return smalltalk:startup()
-- End of embedded image
end)()
smalltalk:addClass("Object",nil,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Smalltalk",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Behavior",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Class",smalltalk.Behavior,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Metaclass",smalltalk.Behavior,smalltalk:newArray({"instanceClass"}),"Kernel")
smalltalk:addClass("CompiledMethod",smalltalk.Object,smalltalk:newArray({"classField"}),"Kernel")
smalltalk:addClass("Number",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("BlockClosure",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Boolean",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("DateTime",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("UndefinedObject",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Collection",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("SequenceableCollection",smalltalk.Collection,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("String",smalltalk.SequenceableCollection,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Array",smalltalk.SequenceableCollection,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("ArrayedContents",smalltalk.SequenceableCollection,smalltalk:newArray({"contents"}),"Kernel")
smalltalk:addClass("ComposedAssociations",smalltalk.ArrayedContents,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("ComposedObjects",smalltalk.ArrayedContents,smalltalk:newArray({"species"}),"Kernel")
smalltalk:addClass("Set",smalltalk.Collection,smalltalk:newArray({"contents"}),"Kernel")
smalltalk:addClass("HashedSet",smalltalk.Set,smalltalk:newArray({"hash"}),"Kernel")
smalltalk:addClass("KeyedCollection",smalltalk.Collection,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Dictionary",smalltalk.KeyedCollection,smalltalk:newArray({"keys","values"}),"Kernel")
smalltalk:addClass("PoolDictionary",smalltalk.KeyedCollection,smalltalk:newArray({"hash","size"}),"Kernel")
smalltalk:addClass("RegularExpression",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Exception",smalltalk.Object,smalltalk:newArray({"messageText"}),"Kernel")
smalltalk:addClass("Error",smalltalk.Exception,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("MessageNotUnderstood",smalltalk.Error,smalltalk:newArray({"message","receiver"}),"Kernel")
smalltalk:addClass("Association",smalltalk.Object,smalltalk:newArray({"key","value"}),"Kernel")
smalltalk:addClass("ClassBuilder",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Stream",smalltalk.Object,smalltalk:newArray({"collection","position","readLimit"}),"Kernel")
smalltalk:addClass("StringStream",smalltalk.Stream,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("StringWriteStream",smalltalk.Stream,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Random",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk:addClass("Point",smalltalk.Object,smalltalk:newArray({"x","y"}),"Kernel")
smalltalk:addClass("Rectangle",smalltalk.Object,smalltalk:newArray({"leftTop","rightBottom"}),"Kernel")
smalltalk:addClass("EvaluableAction",smalltalk.Object,smalltalk:newArray({}),"Kernel")
smalltalk.EvaluableAction["$comment"]="\r\n\tAction model, evaluable and composable actions are implemented to support evaluations.\r\n"
smalltalk:addClass("Message",smalltalk.EvaluableAction,smalltalk:newArray({"selector","arguments","receiver"}),"Kernel")
smalltalk:addClass("LinkMessage",smalltalk.Message,smalltalk:newArray({"resultAction"}),"Kernel")
smalltalk.LinkMessage["$comment"]="\r\n\tImplement evaluation cascading argument evaluations."
smalltalk:addClass("ActionSequence",smalltalk.Object,smalltalk:newArray({"contents"}),"Kernel")
smalltalk:addClass("EventManager",smalltalk.Object,smalltalk:newArray({"handlers"}),"Kernel")
smalltalk.EventManager["$comment"]="\r\n\tEvent system, properties and change/update/release mechanism.\r\n\tWe hook $properties and $handlers internal properties to manage properties and events at Object level.\r\n\tEventManager instances implement events handler as pure instance variable.\r\n"
smalltalk.EventManager["$klass"]["$iVarNames"]=smalltalk:newArray({"eventsTriggered"});
smalltalk:addClass("S8Encoder",smalltalk.EventManager,smalltalk:newArray({"compiler"}),"Compiler")
smalltalk:addClass("OptimizingEncoder",smalltalk.S8Encoder,smalltalk:newArray({"optimizationMap","knownOptimizations"}),"Compiler")
smalltalk:addClass("JSEncoder",smalltalk.OptimizingEncoder,smalltalk:newArray({"stream","startOffset","nestedBlocks","earlyReturn","functionName"}),"JS")
smalltalk:addClass("LuaEncoder",smalltalk.OptimizingEncoder,smalltalk:newArray({"stream","startOffset","nestedBlocks","earlyReturn","functionName"}),"Lua")
smalltalk:addClass("Compiler",smalltalk.EventManager,smalltalk:newArray({"currentClass","encoder","unknownVariables","tempVariables","transformers","useTemplates"}),"Compiler")
smalltalk:addClass("SourceCodeParser",smalltalk.EventManager,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("SmalltalkParser",smalltalk.SourceCodeParser,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("MethodReferences",smalltalk.EventManager,smalltalk:newArray({"rootClasses","results"}),"Tools-References")
smalltalk:addClass("SystemManager",smalltalk.EventManager,smalltalk:newArray({}),"Snapshot")
smalltalk.SystemManager["$classVariableNames"]="Current"
smalltalk:addClass("CompilerNode",smalltalk.Object,smalltalk:newArray({"nodes","start","end","annotations"}),"Compiler")
smalltalk:addClass("MethodNode",smalltalk.CompilerNode,smalltalk:newArray({"selector","arguments","source"}),"Compiler")
smalltalk:addClass("SendNode",smalltalk.CompilerNode,smalltalk:newArray({"selector","arguments","receiver"}),"Compiler")
smalltalk:addClass("CascadeNode",smalltalk.CompilerNode,smalltalk:newArray({"receiver"}),"Compiler")
smalltalk:addClass("AssignmentNode",smalltalk.CompilerNode,smalltalk:newArray({"left","right"}),"Compiler")
smalltalk:addClass("BlockNode",smalltalk.CompilerNode,smalltalk:newArray({"parameters"}),"Compiler")
smalltalk:addClass("SequenceNode",smalltalk.CompilerNode,smalltalk:newArray({"temps"}),"Compiler")
smalltalk:addClass("BlockSequenceNode",smalltalk.SequenceNode,smalltalk:newArray({}),"Compiler")
smalltalk:addClass("ExtendedSequenceNode",smalltalk.SequenceNode,smalltalk:newArray({"directives"}),"Compiler")
smalltalk:addClass("ReturnNode",smalltalk.CompilerNode,smalltalk:newArray({}),"Compiler")
smalltalk:addClass("ValueNode",smalltalk.CompilerNode,smalltalk:newArray({"value"}),"Compiler")
smalltalk:addClass("VariableNode",smalltalk.ValueNode,smalltalk:newArray({"indices"}),"Compiler")
smalltalk:addClass("GlobalReferenceNode",smalltalk.VariableNode,smalltalk:newArray({}),"Compiler")
smalltalk:addClass("NoReceiverNode",smalltalk.VariableNode,smalltalk:newArray({}),"Compiler")
smalltalk:addClass("AlienNode",smalltalk.VariableNode,smalltalk:newArray({}),"Compiler")
smalltalk:addClass("InlineNode",smalltalk.CompilerNode,smalltalk:newArray({"sources"}),"Compiler")
smalltalk:addClass("DirectiveNode",smalltalk.CompilerNode,smalltalk:newArray({}),"Compiler")
smalltalk:addClass("JSDirectiveNode",smalltalk.DirectiveNode,smalltalk:newArray({"code","protected"}),"Compiler")
smalltalk:addClass("InlineOptimizationNode",smalltalk.DirectiveNode,smalltalk:newArray({"args","remove"}),"Compiler")
smalltalk:addClass("PPFailure",smalltalk.Object,smalltalk:newArray({"position","reason"}),"Compiler-Parser")
smalltalk:addClass("PPParser",smalltalk.Object,smalltalk:newArray({"memo"}),"Compiler-Parser")
smalltalk:addClass("PPEOFParser",smalltalk.PPParser,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("PPAnyParser",smalltalk.PPParser,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("PPEpsilonParser",smalltalk.PPParser,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("PPStringParser",smalltalk.PPParser,smalltalk:newArray({"string"}),"Compiler-Parser")
smalltalk:addClass("PPStStringParser",smalltalk.PPParser,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("PPCharacterParser",smalltalk.PPParser,smalltalk:newArray({"regexp","theString"}),"Compiler-Parser")
smalltalk.PPCharacterParser["$classVariableNames"]="RegExpCache"
smalltalk:addClass("PPListParser",smalltalk.PPParser,smalltalk:newArray({"parsers"}),"Compiler-Parser")
smalltalk:addClass("PPSequenceParser",smalltalk.PPListParser,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("PPChoiceParser",smalltalk.PPListParser,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("PPDelegateParser",smalltalk.PPParser,smalltalk:newArray({"parser"}),"Compiler-Parser")
smalltalk:addClass("PPAndParser",smalltalk.PPDelegateParser,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("PPNotParser",smalltalk.PPAndParser,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("PPActionParser",smalltalk.PPDelegateParser,smalltalk:newArray({"block"}),"Compiler-Parser")
smalltalk:addClass("PPFlattenParser",smalltalk.PPDelegateParser,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("PPSourceParser",smalltalk.PPDelegateParser,smalltalk:newArray({}),"Compiler-Parser")
smalltalk:addClass("PPRepeatingParser",smalltalk.PPDelegateParser,smalltalk:newArray({"min"}),"Compiler-Parser")
smalltalk:addClass("PPInlineParser",smalltalk.PPParser,smalltalk:newArray({"terminator"}),"Compiler-Parser")
smalltalk:addClass("FileInReader",smalltalk.Object,smalltalk:newArray({}),"Tools-FileIn")
smalltalk.FileInReader["$comment"]="\r\n\tPerform sequential reading from aStream in chunk format.\r\n\tInstances implement #scanFrom: message, advancing aStream upto next reader instantiation chunk."
smalltalk:addClass("ChunkReader",smalltalk.FileInReader,smalltalk:newArray({"actionBlock","terminationBlock"}),"Tools-FileIn")
smalltalk.ChunkReader["$comment"]="\r\n\tPluggable FileInReader.\r\n\tactionBlock is evaluated with input stream as argument.\r\n\tRead chunks until empty chunk, terminationBlock is nil or evaluates to true."
smalltalk:addClass("ClassCategoryReader",smalltalk.FileInReader,smalltalk:newArray({"class","category"}),"Tools-FileIn")
smalltalk:addClass("ClassCommentReader",smalltalk.FileInReader,smalltalk:newArray({"class"}),"Tools-FileIn")
smalltalk:addClass("SkipFileInReader",smalltalk.FileInReader,smalltalk:newArray({"limit"}),"Tools-FileIn")
smalltalk:addClass("AssociationsReader",smalltalk.FileInReader,smalltalk:newArray({}),"Tools-FileIn")
smalltalk:addClass("PoolDictionaryReader",smalltalk.AssociationsReader,smalltalk:newArray({"scope"}),"Tools-FileIn")
smalltalk:addClass("NativeReader",smalltalk.AssociationsReader,smalltalk:newArray({"object"}),"Tools-FileIn")
smalltalk:addClass("Exporter",smalltalk.Object,smalltalk:newArray({"encoder"}),"Tools-FileOut")
smalltalk:addClass("APIBuilder",smalltalk.Object,smalltalk:newArray({"parents","token","bound"}),"Builders")
smalltalk.APIBuilder["$klass"]["$iVarNames"]=smalltalk:newArray({"default","saveBuilders"});
smalltalk:addClass("Snapshot",smalltalk.Object,smalltalk:newArray({}),"Snapshot")
smalltalk.Snapshot["$comment"]="\r\n\tDump a binary representation of this S8 system onto an image (String).\r\n\tThe default implementation export full/runnable images containing boot sections.\r\n\tThe subclasses can refine behavior to select/customize exports and format.\r\n\t@2011-2018 Alejandro Reimondo aleReimondo@smalltalking.net"
smalltalk:addClass("Transcript",smalltalk.Object,smalltalk:newArray({"streams"}),"Transcript")
smalltalk.Transcript["$comment"]="\r\n\tImplement Transcript streamming.\r\n\tThis implementation emmit output delegating onto stream instances.\r\n\tThe streams must understand #cr and #nextPutAll:\r\n"
smalltalk.Transcript["$classVariableNames"]="Current"
smalltalk:addClass("NativeObject",smalltalk.Object,smalltalk:newArray({"handle"}),"Natives")
smalltalk.NativeObject["$comment"]="\r\n\tWrapper to native object.\r\n\t@2011 Alejandro F. Reimondo\r\n\taleReimondo@smalltalking.net\r\n"
smalltalk:addClass("NativeHandle",smalltalk.NativeObject,smalltalk:newArray({}),"Natives")
smalltalk.NativeHandle["$comment"]="\r\n\tWrapper for native objects.\r\n\t@2018 Alejandro F. Reimondo\r\n\taleReimondo@smalltalking.net\r\n"
smalltalk:addClass("JS",smalltalk.NativeHandle,smalltalk:newArray({}),"JS")
smalltalk.JS["$comment"]="\r\n\tImplementation of wrappers to native Javascript objects.\r\n\t@2011 Alejandro F. Reimondo\r\n\taleReimondo@smalltalking.net"
smalltalk:addClass("Lua",smalltalk.NativeHandle,smalltalk:newArray({}),"Lua")
smalltalk:addClass("NativeLibraryObject",smalltalk.NativeObject,smalltalk:newArray({}),"JS")
smalltalk.NativeLibraryObject["$comment"]="\r\n\tAbstract wrapper to javascript external library.\r\n\t@2012 Alejandro Reimondo - aleReimondo@smalltalking.net\r\n"
smalltalk.NativeLibraryObject["$classVariableNames"]="BindingListeners"

smalltalk:bind(smalltalk.Object["$klass"],"eventsTriggered",0
,(
function(self)
return ((self):constructEventsTriggered())
;
end
)
,"events","eventsTriggered\r\n\t\" Return all the events managed by instances of the receiver. \"\r\n\r\n\t^self constructEventsTriggered","(\r\nfunction(self)\r\nreturn ((self):constructEventsTriggered())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object["$klass"],"constructEventsTriggered",0
,(
function(self)
return (smalltalk:newArray({"changed"}))
;
end
)
,"events","constructEventsTriggered\r\n\t\" Private - Return the events managed by instances of the receiver.\r\n\tThe default implementation return support for MVC change/update mechanism.\r\n\tThis method can be refined by the subclasses to extend event support.\r\n\t\"\r\n\r\n\t^#( #changed )","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"changed\"}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object["$klass"],"canTriggerEvent:",0
,(
function(self,anEvent)
return (((self):eventsTriggered()):includes_(anEvent))
;
end
)
,"events","canTriggerEvent: anEvent\r\n\t\" Return true if instances of the receiver can trigger anEvent.\"\r\n\r\n\t^self eventsTriggered includes: anEvent","(\r\nfunction(self,anEvent)\r\nreturn (((self):eventsTriggered()):includes_(anEvent))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"print:",0
,(
function(self,aString)
print(aString)
;
return (self)
;
end
)
,"printing","print: aString\r\n\t\" Output aString to output device. \"\r\n\r\n\t# #print: aString","(\r\nfunction(self,aString)\r\nprint(aString)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"fileIn:",0
,(
function(self,fileName)
(self):note_(smalltalk:object_comma_((smalltalk:object_comma_(("fileIn(\""),fileName)),"\");"))
;
return (fileIn(fileName))
;
end
)
,"Lua-fileIn","fileIn: fileName\r\n\t\" FileIn contents of file at fileName. \"\r\n\r\n\tself note: 'fileIn(\"', fileName, '\");'.\r\n\t^# #fileIn: fileName","(\r\nfunction(self,fileName)\r\n(self):note_(smalltalk:object_comma_((smalltalk:object_comma_((\"fileIn(\\\"\"),fileName)),\"\\\");\"))\r\n;\r\nreturn (fileIn(fileName))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"fileInJS:",0
,(
function(self,fileName)
(self):note_(smalltalk:object_comma_((smalltalk:object_comma_(("fileInJS(\""),fileName)),"\");"))
;
return (fileInJS(fileName))
;
end
)
,"Lua-fileIn","fileInJS: fileName\r\n\t\" Load native code from file at fileName. \"\r\n\r\n\tself note: 'fileInJS(\"', fileName, '\");'.\r\n\t^# #fileInJS: fileName","(\r\nfunction(self,fileName)\r\n(self):note_(smalltalk:object_comma_((smalltalk:object_comma_((\"fileInJS(\\\"\"),fileName)),\"\\\");\"))\r\n;\r\nreturn (fileInJS(fileName))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"smalltalk",0
,(
function(self)
return ((function() local _1_=((self)["$smalltalk"]);if (_1_==nil) then return ((function()
return (smalltalk)
end)):value() end; return _1_ end)())
;
end
)
,"system","smalltalk\r\n\t\" Private - Return the hosting context of the receiver.\r\n\tThe smalltalk host is used for evaluations in the receiver. \"\r\n\r\n\t^self basicAt: '$smalltalk' ifAbsent: [ #{smalltalk} ]","(\r\nfunction(self)\r\nreturn ((function() local _1_=((self)[\"$smalltalk\"]);if (_1_==nil) then return ((function()\r\nreturn (smalltalk)\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"nativeLanguage",0
,(
function(self)
return ((function() local _1_=(((self):smalltalk())["$nativeLanguage"]);if (_1_==nil) then return ((function()
return ("js")
end)):value() end; return _1_ end)())
;
end
)
,"environment","nativeLanguage\r\n\t\" Private - Returns the native language of the host. \"\r\n\r\n\t^self smalltalk\r\n\t\tbasicAt: '$nativeLanguage'\r\n\t\tifAbsent: [ #js ]","(\r\nfunction(self)\r\nreturn ((function() local _1_=(((self):smalltalk())[\"$nativeLanguage\"]);if (_1_==nil) then return ((function()\r\nreturn (\"js\")\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"halt",0
,(
function(self)
	debug.debug()
	-- Enters an interactive mode with the user, running each string that the user enters.
	-- Using simple commands and other debug facilities, the user can inspect global and local variables, change their values, evaluate expressions, and so on.
	-- A line containing only the word cont finishes this function, so that the caller continues its execution.

;
return (self)
;
end
)
,"debug","halt\r\n\t\" Private - Halt execution entering debugger. \"\r\n\r\n{'\tdebugger'\r\nlua'\tdebug.debug()\r\n\t-- Enters an interactive mode with the user, running each string that the user enters.\r\n\t-- Using simple commands and other debug facilities, the user can inspect global and local variables, change their values, evaluate expressions, and so on.\r\n\t-- A line containing only the word cont finishes this function, so that the caller continues its execution.\r\n'}","(\r\nfunction(self)\r\n\tdebug.debug()\r\n\t-- Enters an interactive mode with the user, running each string that the user enters.\r\n\t-- Using simple commands and other debug facilities, the user can inspect global and local variables, change their values, evaluate expressions, and so on.\r\n\t-- A line containing only the word cont finishes this function, so that the caller continues its execution.\r\n\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"note:",0
,(
function(self,aString)
(self):print_((smalltalk:asString_((aString))):asComment())
;
return (self)
;
end
)
,"development","note: aString\r\n\t\" Print a note (aString) to the user. \"\r\n\r\n\tself print: aString asString asComment","(\r\nfunction(self,aString)\r\n(self):print_((smalltalk:asString_((aString))):asComment())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"notImplementedYet",0
,(
function(self)
(self):error_(("Not implemented. Yet"):halt())
;
return (self)
;
end
)
,"development","notImplementedYet\r\n\t\" Private - Halt execution and/or generate an error reporting implementation incomplete. \"\r\n\r\n\tself error: 'Not implemented. Yet' halt","(\r\nfunction(self)\r\n(self):error_((\"Not implemented. Yet\"):halt())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"notImplementedYet:",0
,(
function(self,aSelector)
(self):error_(smalltalk:object_comma_(((aSelector):asLiteral()),(" Not implemented. Yet"):halt()))
;
return (self)
;
end
)
,"development","notImplementedYet: aSelector\r\n\t\" Private - Halt execution and/or generate an error reporting implementation incomplete. \"\r\n\r\n\tself error: aSelector asLiteral ,' Not implemented. Yet' halt","(\r\nfunction(self,aSelector)\r\n(self):error_(smalltalk:object_comma_(((aSelector):asLiteral()),(\" Not implemented. Yet\"):halt()))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"accept:",0
,(
function(self,what)
(self):print_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("[bug] "),smalltalk:asString_((self))))," accept: ")),smalltalk:asString_((what))))
;
return ((self):error_("#accept: out of control."))
;
end
)
,"parserBug","accept: what\r\n\t\" Private - Report buggy send by compiler. \"\r\n\r\n\tself print: '[bug] ',self asString,' accept: ',what asString.\r\n\t^self error: '#accept: out of control.'","(\r\nfunction(self,what)\r\n(self):print_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"[bug] \"),smalltalk:asString_((self)))),\" accept: \")),smalltalk:asString_((what))))\r\n;\r\nreturn ((self):error_(\"#accept: out of control.\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"=","_eq"
,(
function(self,anObject)
return (((self)==anObject))
;
end
)
,"comparing","= anObject\r\n\t\" Return true if the receiver is equal to argument. \"\r\n\r\n\t^self == anObject","(\r\nfunction(self,anObject)\r\nreturn (((self)==anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"~=","_no_eq"
,(
function(self,anObject)
return ((((smalltalk:isObject_equalTo_((self),anObject)))==false))
;
end
)
,"comparing","~= anObject\r\n\t\" Return true if the receiver is not equal to anObject. \"\r\n\r\n\t^(self = anObject) == false","(\r\nfunction(self,anObject)\r\nreturn ((((smalltalk:isObject_equalTo_((self),anObject)))==false))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"initialize",0
,(
function(self)
return (self)
;
end
)
,"initialize","initialize\r\n\t\" Private - Initialize the receiver.\r\n\tThe default implementation do nothing.\r\n\tThis method can be refined by subclasses to support creation time initialization of collaborators.\r\n\t\"","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"yourself",0
,(
function(self)
return (self)
;
end
)
,"identity","yourself\r\n\t\" Return the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"class",0
,(
function(self)
return (((self)["$klass"]))
;
end
)
,"accessing","class\r\n\t\" Return the class of the receiver. \"\r\n\r\n\t^self basicAt: '$klass'","(\r\nfunction(self)\r\nreturn (((self)[\"$klass\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"size",0
,(
function(self)
return ((0))
;
end
)
,"accessing","size\r\n\t\" Return the size of the receiver.\r\n\tThe default implementation return zero.\r\n\tThis method can be refined by subclasses to support indexed access.\r\n\t\"\r\n\r\n\t^0","(\r\nfunction(self)\r\nreturn ((0))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"copy",0
,(
function(self)
return ((self):shallowCopy())
;
end
)
,"copying","copy\r\n\t\" Return a copy of the receiver. \"\r\n\r\n\t^self shallowCopy","(\r\nfunction(self)\r\nreturn ((self):shallowCopy())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"shallowCopy",0
,(
function(self)
local result=nil
(function()
	local _s8_value=(((self)["$klass"]):basicNew())
	result=_s8_value; return _s8_value
end)()
;
((self)["$klass"]):allInstVarNamesDo_((function(instVarName)
return ((function() local _1_=(((self)[smalltalk:object_comma_(("@"),instVarName)])); (result)[smalltalk:object_comma_(("@"),instVarName)]=_1_; return _1_ end)())
end))
;
return (result)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t| result |\r\n\tresult := self class basicNew.\r\n\tself class allInstVarNamesDo: [:instVarName|\r\n\t\tresult\tbasicAt: $@,instVarName\r\n\t\t\tput: (self basicAt: $@,instVarName)\r\n\t].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):basicNew())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((self)[\"$klass\"]):allInstVarNamesDo_((function(instVarName)\r\nreturn ((function() local _1_=(((self)[smalltalk:object_comma_((\"@\"),instVarName)])); (result)[smalltalk:object_comma_((\"@\"),instVarName)]=_1_; return _1_ end)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"deepCopyOf:",0
,(
function(self,what)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((what))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (what)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((self):isObject_(what)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((what):deepCopy())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (what)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"copying","deepCopyOf: what\r\n\t\" Private - Return a deep copy of what. \"\r\n\r\n\twhat isNil ifTrue: [ ^what ].\r\n\t(self isObject: what) ifTrue: [ ^what deepCopy ].\r\n\t^what","(\r\nfunction(self,what)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((what))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (what)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((self):isObject_(what)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((what):deepCopy())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (what)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"deepCopy",0
,(
function(self)
local result=nil
local value=nil
(function()
	local _s8_value=(((self)["$klass"]):basicNew())
	result=_s8_value; return _s8_value
end)()
;
((self)["$klass"]):allInstVarNamesDo_((function(iv)
(function()
	local _s8_value=((self):deepCopyOf_(((self)[smalltalk:object_comma_(("@"),iv)])))
	value=_s8_value; return _s8_value
end)();return ((function() local _1_=(value); (result)[smalltalk:object_comma_(("@"),iv)]=_1_; return _1_ end)())
end))
;
return (result)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t| result value |\r\n\tresult := self class basicNew.\r\n\tself class allInstVarNamesDo: [:iv|\r\n\t\tvalue := self deepCopyOf: (self basicAt: $@,iv).\r\n\t\tresult\tbasicAt: $@,iv put: value\r\n\t].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\nlocal value=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):basicNew())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((self)[\"$klass\"]):allInstVarNamesDo_((function(iv)\r\n(function()\r\n\tlocal _s8_value=((self):deepCopyOf_(((self)[smalltalk:object_comma_((\"@\"),iv)])))\r\n\tvalue=_s8_value; return _s8_value\r\nend)();return ((function() local _1_=(value); (result)[smalltalk:object_comma_((\"@\"),iv)]=_1_; return _1_ end)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"->","_minus_gt"
,(
function(self,anObject)
return ((smalltalk.Association):key_value_(self, anObject))
;
end
)
,"converting","-> anObject\r\n\t\" Return an association of the receiver to anObject. \"\r\n\r\n\t^Association key: self value: anObject","(\r\nfunction(self,anObject)\r\nreturn ((smalltalk.Association):key_value_(self, anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asString",0
,(
function(self)
return (smalltalk:printString_((self)))
;
end
)
,"converting","asString\r\n\t\" Return the receiver as a String. \"\r\n\r\n\t^self printString","(\r\nfunction(self)\r\nreturn (smalltalk:printString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"toString",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_(("{st:a"),((self)["$klass"]):name())),"}"))
;
end
)
,"converting","toString\r\n\t\" Convert the receiver to a String. \"\r\n\r\n\t^'{st:a',self class name,$}","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"{st:a\"),((self)[\"$klass\"]):name())),\"}\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asNative:",0
,(
function(self,lang)
return (smalltalk:asString_((self)))
;
end
)
,"converting","asNative: lang\r\n\t\" Return the native representation of the receiver in target language lang.\r\n\tThe default implementation return the receiver as a String.\r\n\tThis method must be refined by subclasses implementing native literal representation.\r\n\t\"\r\n\r\n\t^self asString","(\r\nfunction(self,lang)\r\nreturn (smalltalk:asString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asNative",0
,(
function(self)
return ((self):asNative_((self):nativeLanguage()))
;
end
)
,"converting","asNative\r\n\t\" Return the native representation of the receiver.\r\n\tThe default implementation return the receiver as a String.\r\n\tThis method must be refined by subclasses implementing native literal representation.\r\n\t\"\r\n\r\n\t^self asNative: self nativeLanguage","(\r\nfunction(self)\r\nreturn ((self):asNative_((self):nativeLanguage()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asJSON",0
,(
function(self)
local json=nil
(function()
	local _s8_value=((smalltalk.Smalltalk):jsObjectAt_("JSON"))
	json=_s8_value; return _s8_value
end)()
;
return ((json):stringify((self):asJSONObject()))
;
end
)
,"json","asJSON\r\n\t\" Return the JSON representation of the receiver. \"\r\n\r\n\t| json |\r\n\tjson := Smalltalk jsObjectAt: #JSON.\r\n\t^json #stringify: self asJSONObject","(\r\nfunction(self)\r\nlocal json=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):jsObjectAt_(\"JSON\"))\r\n\tjson=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((json):stringify((self):asJSONObject()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"escapedCode",0
,(
function(self)
return ((self):asNative())
;
end
)
,"converting","escapedCode\r\n\t\" Return the code to reproduce receiver from code. \"\r\n\r\n\t^self asNative","(\r\nfunction(self)\r\nreturn ((self):asNative())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"perform:",0
,(
function(self,aSymbol)
return ((self):perform_withArguments_(aSymbol, smalltalk:newArray({})))
;
end
)
,"perform","perform: aSymbol\r\n\t\" Return the result of sending message aSymbol to the receiver. \"\r\n\r\n\t^self perform: aSymbol withArguments: #()","(\r\nfunction(self,aSymbol)\r\nreturn ((self):perform_withArguments_(aSymbol, smalltalk:newArray({})))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"perform:with:",0
,(
function(self,aSymbol,argument)
return ((self):perform_withArguments_(aSymbol, (smalltalk.Array):with_(argument)))
;
end
)
,"perform","perform: aSymbol with: argument\r\n\t\" Return the result of sending message aSymbol to the receiver. \"\r\n\r\n\t^self perform: aSymbol withArguments: (Array with: argument)","(\r\nfunction(self,aSymbol,argument)\r\nreturn ((self):perform_withArguments_(aSymbol, (smalltalk.Array):with_(argument)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"perform:withArguments:",0
,(
function(self,aSymbol,aCollection)
return ((self):basicPerform_withArguments_((aSymbol):asSelector(), (aCollection):asArray()))
;
end
)
,"perform","perform: aSymbol withArguments: aCollection\r\n\t\" Return the result of sending message aSymbol to the receiver. \"\r\n\r\n\"\tWarning: To do NOT check arguments size is a feature\r\n\t\tif we do strict matching U8 tools will break because they are using the feature.\r\n\taSymbol numberOfArguments = aCollection size ifFalse: [\r\n\t\t^self error: 'Invalid arguments size'\r\n\t].\r\n\"\t^self basicPerform: aSymbol asSelector withArguments: aCollection asArray","(\r\nfunction(self,aSymbol,aCollection)\r\nreturn ((self):basicPerform_withArguments_((aSymbol):asSelector(), (aCollection):asArray()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"basicAt:",0
,(
function(self,aString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function()
	local _s8_value=(self[aString])
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((((result))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","basicAt: aString\r\n\t\" Return the property of the receiver known as aString (or nil). \"\r\n\r\n\t| result |\r\n\tresult := self[aString].\r\n\tresult isNil ifTrue: [ ^nil ].\r\n\t^result","(\r\nfunction(self,aString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(self[aString])\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((result))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"basicAt:ifAbsent:",0
,(
function(self,aString,aBlock)
return ((function() local _1_=((self[aString]));if (_1_==nil) then return ((function()
return ((aBlock):value())
end)):value() end; return _1_ end)())
;
end
)
,"accessing","basicAt: aString ifAbsent: aBlock\r\n\t\" Return the property of the receiver known as aString (or the result of evaluating aBlock). \"\r\n\r\n\t^self[aString] ifUndefined: [ aBlock value ]","(\r\nfunction(self,aString,aBlock)\r\nreturn ((function() local _1_=((self[aString]));if (_1_==nil) then return ((function()\r\nreturn ((aBlock):value())\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"basicAt:ifAbsentPut:",0
,(
function(self,aString,aBlock)
return ((function() local _1_=((self)[aString]);if (_1_==nil) then return ((function()
return ((function() local _1_=((aBlock):value()); (self)[aString]=_1_; return _1_ end)())
end)):value() end; return _1_ end)())
;
end
)
,"accessing","basicAt: aString ifAbsentPut: aBlock\r\n\t\" Return the property of the receiver known as aString (or set it with the result of evaluating aBlock). \"\r\n\r\n\t^self basicAt: aString ifAbsent: [ self basicAt: aString put: aBlock value ]","(\r\nfunction(self,aString,aBlock)\r\nreturn ((function() local _1_=((self)[aString]);if (_1_==nil) then return ((function()\r\nreturn ((function() local _1_=((aBlock):value()); (self)[aString]=_1_; return _1_ end)())\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"instVarAt:",0
,(
function(self,instVarName)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((instVarName):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):instVarAt_((((self)["$klass"]):allInstVarNames()):at_(instVarName)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self)[smalltalk:object_comma_(("@"),instVarName)]))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","instVarAt: instVarName\r\n\t\" Return the internal collaborator of the receiver known as instVarName. \"\r\n\r\n\tinstVarName isNumber ifTrue: [\r\n\t\t^self instVarAt: (self class allInstVarNames at: instVarName)\r\n\t].\r\n\t^self basicAt: '@', instVarName","(\r\nfunction(self,instVarName)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((instVarName):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):instVarAt_((((self)[\"$klass\"]):allInstVarNames()):at_(instVarName)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self)[smalltalk:object_comma_((\"@\"),instVarName)]))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"instVarAt:put:",0
,(
function(self,instVarName,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((instVarName):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):instVarAt_put_((((self)["$klass"]):allInstVarNames()):at_(instVarName), anObject))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function() local _1_=(anObject); (self)[smalltalk:object_comma_(("@"),instVarName)]=_1_; return _1_ end)())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","instVarAt: instVarName put: anObject\r\n\t\" Set the internal collaborator of the receiver known as instVarName to anObject. \"\r\n\r\n\tinstVarName isNumber ifTrue: [\r\n\t\t^self instVarAt: (self class allInstVarNames at: instVarName) put: anObject\r\n\t].\r\n\t^self basicAt: '@', instVarName put: anObject","(\r\nfunction(self,instVarName,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((instVarName):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):instVarAt_put_((((self)[\"$klass\"]):allInstVarNames()):at_(instVarName), anObject))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function() local _1_=(anObject); (self)[smalltalk:object_comma_((\"@\"),instVarName)]=_1_; return _1_ end)())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"basicAt:put:",0
,(
function(self,aString,anObject)
return ((function()
	local _s8_value=(anObject)
	self[aString]=_s8_value; return _s8_value
end)())
;
end
)
,"accessing","basicAt: aString put: anObject\r\n\t\" Set the property of the receiver known as aString. \"\r\n\r\n\t^self[aString] := anObject","(\r\nfunction(self,aString,anObject)\r\nreturn ((function()\r\n\tlocal _s8_value=(anObject)\r\n\tself[aString]=_s8_value; return _s8_value\r\nend)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"error:",0
,(
function(self,aString)
return ((smalltalk.Error):signal_(aString))
;
end
)
,"error","error: aString\r\n\t\" Return the result of managing error(aString) in the receiver. \"\r\n\r\n\t^Error signal: aString","(\r\nfunction(self,aString)\r\nreturn ((smalltalk.Error):signal_(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"subclassResponsibility",0
,(
function(self)
return ((self):subclassResponsibility_(nil))
;
end
)
,"error","subclassResponsibility\r\n\t\" Private - The concrete implementation on the receiver is not complete. \"\r\n\r\n\t^self subclassResponsibility: nil","(\r\nfunction(self)\r\nreturn ((self):subclassResponsibility_(nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"implementedBySubclass",0
,(
function(self)
return ((self):subclassResponsibility())
;
end
)
,"error","implementedBySubclass\r\n\t\" Private - The concrete implementation on the receiver is not complete. \"\r\n\r\n\t^self subclassResponsibility","(\r\nfunction(self)\r\nreturn ((self):subclassResponsibility())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"shouldNotImplement",0
,(
function(self)
return ((self):shouldNotImplement_(nil))
;
end
)
,"error","shouldNotImplement\r\n\t\" The receiver received a message that is not expected. \"\r\n\r\n\t^self shouldNotImplement: nil","(\r\nfunction(self)\r\nreturn ((self):shouldNotImplement_(nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"invalidMessage",0
,(
function(self)
return ((self):invalidMessage_(nil))
;
end
)
,"error","invalidMessage\r\n\t\" The receiver received a message that is invalid (not expected for this object). \"\r\n\r\n\t^self invalidMessage: nil","(\r\nfunction(self)\r\nreturn ((self):invalidMessage_(nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"subclassResponsibility:",0
,(
function(self,aSelector)
return (((self):halt()):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("Implementation of"),(function() if ((((((aSelector))==nil)))==true) then return ((function()
return (" ")
end))() else return ((function()
return (smalltalk:object_comma_((" #"),aSelector))
end))() end end)()))," is a responsibility of the subclass(")),((self)["$klass"]):name())),").")))
;
end
)
,"error","subclassResponsibility: aSelector\r\n\t\" Private - The concrete implementation on the receiver is not complete. \"\r\n\r\n\t^self halt error:\r\n\t\t 'Implementation of'\r\n\t\t,(aSelector isNil ifTrue: [' '] ifFalse: [' #',aSelector])\r\n\t\t,' is a responsibility of the subclass(',self class name,').'","(\r\nfunction(self,aSelector)\r\nreturn (((self):halt()):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"Implementation of\"),(function() if ((((((aSelector))==nil)))==true) then return ((function()\r\nreturn (\" \")\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((\" #\"),aSelector))\r\nend))() end end)())),\" is a responsibility of the subclass(\")),((self)[\"$klass\"]):name())),\").\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"implementedBySubclass:",0
,(
function(self,aSelector)
return ((self):subclassResponsibility_(aSelector))
;
end
)
,"error","implementedBySubclass: aSelector\r\n\t\" Private - The concrete implementation on the receiver is not complete. \"\r\n\r\n\t^self subclassResponsibility: aSelector","(\r\nfunction(self,aSelector)\r\nreturn ((self):subclassResponsibility_(aSelector))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"shouldNotImplement:",0
,(
function(self,aSelector)
return ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("Message"),(function() if ((((((aSelector))==nil)))==true) then return ((function()
return (" ")
end))() else return ((function()
return (smalltalk:object_comma_((" #"),aSelector))
end))() end end)()))," should not be implemented for instances of ")),((self)["$klass"]):name())))
;
end
)
,"error","shouldNotImplement: aSelector\r\n\t\" The receiver received a message that is not expected. \"\r\n\r\n\t^self error:\r\n\t\t 'Message'\r\n\t\t,(aSelector isNil ifTrue: [' '] ifFalse: [' #',aSelector])\r\n\t\t,' should not be implemented for instances of ', self class name","(\r\nfunction(self,aSelector)\r\nreturn ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"Message\"),(function() if ((((((aSelector))==nil)))==true) then return ((function()\r\nreturn (\" \")\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((\" #\"),aSelector))\r\nend))() end end)())),\" should not be implemented for instances of \")),((self)[\"$klass\"]):name())))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"invalidMessage:",0
,(
function(self,aSelector)
return ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("Message"),(function() if ((((((aSelector))==nil)))==true) then return ((function()
return (" ")
end))() else return ((function()
return (smalltalk:object_comma_((" #"),aSelector))
end))() end end)()))," should not be sent to instances of ")),((self)["$klass"]):name())))
;
end
)
,"error","invalidMessage: aSelector\r\n\t\" The receiver received a message that is invalid (not expected for this object). \"\r\n\r\n\t^self error: \r\n\t\t 'Message'\r\n\t\t,(aSelector isNil ifTrue: [' '] ifFalse: [' #',aSelector])\r\n\t\t,' should not be sent to instances of ', self class name","(\r\nfunction(self,aSelector)\r\nreturn ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"Message\"),(function() if ((((((aSelector))==nil)))==true) then return ((function()\r\nreturn (\" \")\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((\" #\"),aSelector))\r\nend))() end end)())),\" should not be sent to instances of \")),((self)[\"$klass\"]):name())))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"smalltalkErrorOf:",0
,(
function(self,aNativeException)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function() if ((((self):isObject_(aNativeException)))==true) then return ((function()
(function() if (((smalltalk:is_kindOf_((aNativeException),(smalltalk.Exception))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (aNativeException)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((smalltalk.Error):new()):messageText_(aNativeException))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=(((aNativeException)["smalltalkError"]))
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((((result))~=nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((
function(_s8_rec)
(_s8_rec):messageText_(((aNativeException["description"]) or aNativeException["message"]));
(_s8_rec):nativeException_(aNativeException);
return ((_s8_rec):yourself())
end
)((smalltalk.Error):new()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","smalltalkErrorOf: aNativeException\r\n\t\" Private, Internal - Return anException associated with a native exception. \"\r\n\r\n\t| result |\r\n\t(self isObject: aNativeException) ifTrue: [\r\n\t\t(aNativeException isKindOf: Exception) ifTrue: [ ^aNativeException ].\r\n\t\t^Error new messageText: aNativeException\r\n\t].\r\n\tresult := aNativeException basicAt: #smalltalkError.\r\n\tresult notNil ifTrue: [ ^result ].\r\n\t^Error new\r\n\t\tmessageText: aNativeException#description || aNativeException#message;\r\n\t\tnativeException: aNativeException;\r\n\t\tyourself","(\r\nfunction(self,aNativeException)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function() if ((((self):isObject_(aNativeException)))==true) then return ((function()\r\n(function() if (((smalltalk:is_kindOf_((aNativeException),(smalltalk.Exception))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (aNativeException)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((smalltalk.Error):new()):messageText_(aNativeException))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((aNativeException)[\"smalltalkError\"]))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((result))~=nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):messageText_(((aNativeException[\"description\"]) or aNativeException[\"message\"]));\r\n(_s8_rec):nativeException_(aNativeException);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.Error):new()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"inspect",0
,(
function(self)
return ((self):basicInspect())
;
end
)
,"inspection","inspect\r\n\t\" Open an inspector on the receiver. \"\r\n\r\n\t^self basicInspect","(\r\nfunction(self)\r\nreturn ((self):basicInspect())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"basicInspect",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk.Smalltalk):includesKey_("JS")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):print_((smalltalk.JS):_at_gt(self)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"inspection","basicInspect\r\n\t\" Private - Open an inspector on the receiver. \"\r\n\r\n\t(Smalltalk includesKey: #JS) ifTrue: [ \"print native representation\"\r\n\t\t^self print: JS@> self\r\n\t].\r\n\t^self","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk.Smalltalk):includesKey_(\"JS\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):print_((smalltalk.JS):_at_gt(self)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"printString",0
,(
function(self)
return (smalltalk:object_comma_(("a "),((self)["$klass"]):name()))
;
end
)
,"printing","printString\r\n\t\" Return a printable representation of the receiver. \"\r\n\r\n\t^'a ', self class name","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((\"a \"),((self)[\"$klass\"]):name()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isKindOf:",0
,(
function(self,aClass)
return (((self)["$klass"]):inheritsFrom_(aClass))
;
end
)
,"testing","isKindOf: aClass\r\n\t\" Return true if the receiver is kind of aClass. \"\r\n\r\n\t^self class inheritsFrom: aClass","(\r\nfunction(self,aClass)\r\nreturn (((self)[\"$klass\"]):inheritsFrom_(aClass))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isMemberOf:",0
,(
function(self,aClass)
return ((((self)["$klass"])==aClass))
;
end
)
,"testing","isMemberOf: aClass\r\n\t\" Return true if the receiver is member of aClass. \"\r\n\r\n\t^self class == aClass","(\r\nfunction(self,aClass)\r\nreturn ((((self)[\"$klass\"])==aClass))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"ifNil:",0
,(
function(self,aBlock)
return (self)
;
end
)
,"testing","ifNil: aBlock\r\n\t\" Return the receiver if it is not nil, or the result of evaluating aBlock (if it is nil). \"\r\n\r\n\t^self","(\r\nfunction(self,aBlock)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"ifNil:ifNotNil:",0
,(
function(self,aBlock,anotherBlock)
return ((anotherBlock):value())
;
end
)
,"testing","ifNil: aBlock ifNotNil: anotherBlock\r\n\t\" Return the result of evaluating aBlock if the receiver is nil or the result of evaluating anotherBlock (if not nil). \"\r\n\r\n\t^anotherBlock value","(\r\nfunction(self,aBlock,anotherBlock)\r\nreturn ((anotherBlock):value())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"ifNotNil:",0
,(
function(self,aBlock)
return ((aBlock):value())
;
end
)
,"testing","ifNotNil: aBlock\r\n\t\" Return the result of evaluating aBlock if the receiver is not nil, or nil if it is nil. \"\r\n\r\n\t^aBlock value","(\r\nfunction(self,aBlock)\r\nreturn ((aBlock):value())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"ifNotNil:ifNil:",0
,(
function(self,aBlock,anotherBlock)
return ((aBlock):value())
;
end
)
,"testing","ifNotNil: aBlock ifNil: anotherBlock\r\n\t\" Return the result of evaluating aBlock if the receiver is not nil, or the result of evaluating anotherBlock if it is nil. \"\r\n\r\n\t^aBlock value","(\r\nfunction(self,aBlock,anotherBlock)\r\nreturn ((aBlock):value())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isNil",0
,(
function(self)
return (false)
;
end
)
,"testing","isNil\r\n\t\" Return true if the receiver is nil. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"notNil",0
,(
function(self)
return ((smalltalk:booleanNot_(((((self))==nil)))))
;
end
)
,"testing","notNil\r\n\t\" Return true if the receiver is not nil. \"\r\n\r\n\t^self isNil not","(\r\nfunction(self)\r\nreturn ((smalltalk:booleanNot_(((((self))==nil)))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isClass",0
,(
function(self)
return (false)
;
end
)
,"testing","isClass\r\n\t\" Return true if the receiver is a Class. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isMetaclass",0
,(
function(self)
return (false)
;
end
)
,"testing","isMetaclass\r\n\t\" Return true if the receiver is a metaclass. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isNumber",0
,(
function(self)
return (false)
;
end
)
,"testing","isNumber\r\n\t\" Return true if the receiver is a Number. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isString",0
,(
function(self)
return (false)
;
end
)
,"testing","isString\r\n\t\" Return true if the receiver is a String. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isArray",0
,(
function(self)
return (false)
;
end
)
,"testing","isArray\r\n\t\" Return true if the receiver is an Array. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isObject:",0
,(
function(self,anObject)
return (anObject["$klass"]~=nil)
;
end
)
,"testing","isObject: anObject\r\n\t\" Return true if anObject is a smalltalk object.\r\n\tReturn false if the object is a native object.\r\n\t\"\r\n\r\n\t^{'(anObject==0 || anObject && anObject.$klass)?true:false'\r\n\tlua'anObject[\"$klass\"]~=nil'}","(\r\nfunction(self,anObject)\r\nreturn (anObject[\"$klass\"]~=nil)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"typeOf:",0
,(
function(self,anObject)
return (type(anObject))
;
end
)
,"utilities","typeOf: anObject\r\n\t\" Return the (native) type of anObject. \"\r\n\r\n\t^{'typeof anObject'\r\n\tlua'type(anObject)'}","(\r\nfunction(self,anObject)\r\nreturn (type(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"methodFor:",0
,(
function(self,aSymbol)
return (((self)["$klass"]):_gt_gt_gt(aSymbol))
;
end
)
,"messages","methodFor: aSymbol \r\n\t\" Return the method implementing aSymbol (or nil). \"\r\n\r\n\t^self class >>> aSymbol","(\r\nfunction(self,aSymbol)\r\nreturn (((self)[\"$klass\"]):_gt_gt_gt(aSymbol))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"respondsTo:",0
,(
function(self,aSymbol)
return (((((self):methodFor_(aSymbol)))~=nil))
;
end
)
,"messages","respondsTo: aSymbol \r\n\t\" Return true if the receiver respond to message aSymbol. \"\r\n\r\n\t^(self methodFor: aSymbol) notNil","(\r\nfunction(self,aSymbol)\r\nreturn (((((self):methodFor_(aSymbol)))~=nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"basicDelete:",0
,(
function(self,aString)
	self[aString]=nil
;
return (self)
;
end
)
,"accessing","basicDelete: aString\r\n\t\" Private - Basic implementation of deletion of property aString of the receiver. \"\r\n\r\n{'\tdelete self[aString]'\r\nlua'\tself[aString]=nil'}","(\r\nfunction(self,aString)\r\n\tself[aString]=nil\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"doesNotUnderstand:",0
,(
function(self,aMessage)
return ((
function(_s8_rec)
(_s8_rec):receiver_(self);
(_s8_rec):message_(aMessage);
return ((_s8_rec):signal())
end
)((smalltalk.MessageNotUnderstood):new()))
;
end
)
,"dnu","doesNotUnderstand: aMessage\r\n\t\" Private - Basic implementation of a message sent and not understood by the receiver. \"\r\n\r\n\t^MessageNotUnderstood new\r\n\t\treceiver: self;\r\n\t\tmessage: aMessage;\r\n\t\tsignal","(\r\nfunction(self,aMessage)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):receiver_(self);\r\n(_s8_rec):message_(aMessage);\r\nreturn ((_s8_rec):signal())\r\nend\r\n)((smalltalk.MessageNotUnderstood):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"dnu:withArguments:",0
,(
function(self,selector,anArray)
return ((self):doesNotUnderstand_((smalltalk.Message):selector_arguments_(selector, anArray)))
;
end
)
,"dnu","dnu: selector withArguments: anArray\r\n\t\" Private - Basic implementation of a message sent and not understood by the receiver. \"\r\n\r\n\t^self doesNotUnderstand: (Message\r\n\t\tselector: selector\r\n\t\targuments: anArray)","(\r\nfunction(self,selector,anArray)\r\nreturn ((self):doesNotUnderstand_((smalltalk.Message):selector_arguments_(selector, anArray)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"basicPerform:withArguments:",0
,(
function(self,jsSelector,anArray)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local fn=nil
(function()
	local _s8_value=((function() local _1_=((self)[jsSelector]);if (_1_==nil) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):dnu_withArguments_((jsSelector):asStSelector(), anArray))
end
)()
error(_s8Ret_,0)
end)())
end)):value() end; return _1_ end)())
	fn=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (fn(self,(table.unpack or unpack)(anArray)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","basicPerform: jsSelector withArguments: anArray\r\n\t\" Private - Basic implementation of #perform... \"\r\n\r\n\t| fn |\r\n\tfn := self basicAt: jsSelector ifAbsent: [\r\n\t\t^self dnu: jsSelector asStSelector withArguments: anArray\r\n\t].\r\n\t^{'fn.apply(self,anArray)'\r\n\tlua'fn(self,(table.unpack or unpack)(anArray))'}","(\r\nfunction(self,jsSelector,anArray)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal fn=nil\r\n(function()\r\n\tlocal _s8_value=((function() local _1_=((self)[jsSelector]);if (_1_==nil) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):dnu_withArguments_((jsSelector):asStSelector(), anArray))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)):value() end; return _1_ end)())\r\n\tfn=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (fn(self,(table.unpack or unpack)(anArray)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"json",0
,(
function(self)
return ((self):asJSONObject())
;
end
)
,"json","json\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self asJSONObject","(\r\nfunction(self)\r\nreturn ((self):asJSONObject())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asJson:",0
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((anObject))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((nil):json())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function() if ((((self):isObject_(anObject)))==true) then return ((function()
return ((anObject):json())
end))() else return ((function()
return (anObject)
end))() end end)())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"json","asJson: anObject\r\n\t\" Return anObject as a hash object. \"\r\n\r\n\tanObject isNil ifTrue: [ ^nil json ].\r\n\t^(self isObject: anObject)\r\n\t\tifTrue: [ anObject json ]\r\n\t\tifFalse: [ anObject ]","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((anObject))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((nil):json())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function() if ((((self):isObject_(anObject)))==true) then return ((function()\r\nreturn ((anObject):json())\r\nend))() else return ((function()\r\nreturn (anObject)\r\nend))() end end)())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asJSONObject",0
,(
function(self)
local result=nil
(function()
	local _s8_value=({})
	result=_s8_value; return _s8_value
end)()
;
(self):jsonFieldsDo_ignoring_((function(field,value)
return ((function() if ((((smalltalk:boolean_or_(((((value))~=nil)),(function()
return ((self):jsonExportNil())
end)))))==true) then return ((function()
return ((function() local _1_=((self):asJson_(value)); (result)[field]=_1_; return _1_ end)())
end))() else return nil end end)())
end), (self):jsonHiddenNames())
;
return (result)
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t| result |\r\n\tresult := {'{}' lua'{}'}.\r\n\tself\tjsonFieldsDo: [:field :value|\r\n\t\t(value notNil or: [ self jsonExportNil ]) ifTrue: [\r\n\t\t\tresult basicAt: field put: (self asJson: value)\r\n\t\t].\r\n\t]\tignoring: self jsonHiddenNames.\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=({})\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):jsonFieldsDo_ignoring_((function(field,value)\r\nreturn ((function() if ((((smalltalk:boolean_or_(((((value))~=nil)),(function()\r\nreturn ((self):jsonExportNil())\r\nend)))))==true) then return ((function()\r\nreturn ((function() local _1_=((self):asJson_(value)); (result)[field]=_1_; return _1_ end)())\r\nend))() else return nil end end)())\r\nend), (self):jsonHiddenNames())\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"jsonHiddenNames",0
,(
function(self)
return (smalltalk:newArray({}))
;
end
)
,"converting","jsonHiddenNames\r\n\t\" Private - Return the names of the fields to skip when mapping the receiver to JSON structure. \"\r\n\r\n\t^#()","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"jsonFieldsDo:ignoring:",0
,(
function(self,aBlock,ignored)
return (((self)["$klass"]):allInstVarNamesDo_((function(each)
return ((function() if ((((smalltalk:boolean_and_(((((ignored))~=nil)),(function()
return ((ignored):includes_(each))
end)))))==true) then return nil else return ((function()
return ((aBlock):value_value_(each, (self):instVarAt_(each)))
end))() end end)())
end)))
;
end
)
,"converting","jsonFieldsDo: aBlock ignoring: ignored\r\n\t\" Private - Evaluates a(TwoArguments)Block with the fields to map the receiver to JSON structure. \"\r\n\r\n\t^self class allInstVarNamesDo: [:each|\r\n\t\t(ignored notNil and: [ ignored includes: each ]) ifFalse: [\r\n\t\t\taBlock value: each value: (self instVarAt: each)\r\n\t\t].\r\n\t]","(\r\nfunction(self,aBlock,ignored)\r\nreturn (((self)[\"$klass\"]):allInstVarNamesDo_((function(each)\r\nreturn ((function() if ((((smalltalk:boolean_and_(((((ignored))~=nil)),(function()\r\nreturn ((ignored):includes_(each))\r\nend)))))==true) then return nil else return ((function()\r\nreturn ((aBlock):value_value_(each, (self):instVarAt_(each)))\r\nend))() end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"jsonExportNil",0
,(
function(self)
return (false)
;
end
)
,"converting","jsonExportNil\r\n\t\" Private - Return true if the receiver export nil values. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"mustBeBoolean",0
,(
function(self)
return ((self):error_("Must be boolean."))
;
end
)
,"testing","mustBeBoolean\r\n\t\" Signal an error if the receiver is not a Boolean. \"\r\n\r\n\t^self error: 'Must be boolean.'","(\r\nfunction(self)\r\nreturn ((self):error_(\"Must be boolean.\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"mustBeNumber",0
,(
function(self)
return ((self):error_("Must be a Number."))
;
end
)
,"testing","mustBeNumber\r\n\t\" Signal an error if the receiver is not a Number. \"\r\n\r\n\t^self error: 'Must be a Number.'","(\r\nfunction(self)\r\nreturn ((self):error_(\"Must be a Number.\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isBoolean",0
,(
function(self)
return (false)
;
end
)
,"query","isBoolean\r\n\t\" Return true if the receiver is a boolean. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asLiteral",0
,(
function(self)
return (smalltalk:printString_((self)))
;
end
)
,"converting","asLiteral\r\n\t\" Return the receiver in literal notation. \"\r\n\r\n\t^self printString","(\r\nfunction(self)\r\nreturn (smalltalk:printString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isCharacter",0
,(
function(self)
return (false)
;
end
)
,"testing","isCharacter\r\n\t\" Return true if the receiver is a character. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isAssociation",0
,(
function(self)
return (false)
;
end
)
,"testing","isAssociation\r\n\t\" Return true if the receiver is anAssociation. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isDictionary",0
,(
function(self)
return (false)
;
end
)
,"query","isDictionary\r\n\t\" Return true if the receiver is a dictionary. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isRectangle",0
,(
function(self)
return (false)
;
end
)
,"testing","isRectangle\r\n\t\" Return true if receiver is a Rectangle. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isMessage",0
,(
function(self)
return (false)
;
end
)
,"query","isMessage\r\n\t\" Return true if the receiver is a message. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"argumentCount",0
,(
function(self)
return ((0))
;
end
)
,"evaluating","argumentCount\r\n\t\" Return the number of arguments defined by the receiver. \"\r\n\r\n\t^0","(\r\nfunction(self)\r\nreturn ((0))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"numberOfArguments",0
,(
function(self)
return ((self):argumentCount())
;
end
)
,"evaluating","numberOfArguments\r\n\t\" Return the number of arguments defined by the receiver. \"\r\n\r\n\t^self argumentCount","(\r\nfunction(self)\r\nreturn ((self):argumentCount())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"evaluate",0
,(
function(self)
return (self)
;
end
)
,"evaluating","evaluate\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"evaluateDeferred",0
,(
function(self)
((function()
return ((self):evaluate())
end)):valueDeferred()
;
return (self)
;
end
)
,"evaluating","evaluateDeferred\r\n\t\" Evaluating the receiver deferred in time.\r\n\tReturn the receiver. \"\r\n\r\n\t[ self evaluate ] valueDeferred.","(\r\nfunction(self)\r\n((function()\r\nreturn ((self):evaluate())\r\nend)):valueDeferred()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"evaluateWithArguments:",0
,(
function(self,anArray)
return (self)
;
end
)
,"evaluating","evaluateWithArguments: anArray\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self,anArray)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"evaluateWith:",0
,(
function(self,anObject)
return ((self):evaluateWithArguments_((smalltalk.Array):with_(anObject)))
;
end
)
,"evaluating","evaluateWith: anObject\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t^self evaluateWithArguments: (Array with: anObject)","(\r\nfunction(self,anObject)\r\nreturn ((self):evaluateWithArguments_((smalltalk.Array):with_(anObject)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"logEvaluate:",0
,(
function(self,source)
local result=nil
(function()
	local _s8_value=((self):evaluate())
	result=_s8_value; return _s8_value
end)()
;
(smalltalk.SystemManager):log_(source)
;
return (result)
;
end
)
,"evaluating","logEvaluate: source\r\n\t\" Return the result of evaluating the receiver.\r\n\tLog the source expression as reference of evaluation.\r\n\t\"\r\n\r\n\t| result |\r\n\tresult := self evaluate.\r\n\tSystemManager log: source.\r\n\t^result","(\r\nfunction(self,source)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):evaluate())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(smalltalk.SystemManager):log_(source)\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asActionSequence",0
,(
function(self)
return ((smalltalk.ActionSequence):with_(self))
;
end
)
,"converting","asActionSequence\r\n\t\" Return an ActionSequence from the receiver. \"\r\n\r\n\t^ActionSequence with: self","(\r\nfunction(self)\r\nreturn ((smalltalk.ActionSequence):with_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"eventTable",0
,(
function(self)
return ((function() local _1_=((self)["@handlers"]);if (_1_==nil) then return ((function()
return ((self):eventTableCreate())
end)):value() end; return _1_ end)())
;
end
)
,"events","eventTable\r\n\t\" Private - Return the mapping event names to actions of the receiver. \"\r\n\r\n\t^self basicAt: '@handlers' ifAbsent: [ self eventTableCreate ]","(\r\nfunction(self)\r\nreturn ((function() local _1_=((self)[\"@handlers\"]);if (_1_==nil) then return ((function()\r\nreturn ((self):eventTableCreate())\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"eventTableForEdit",0
,(
function(self)
return ((function() local _1_=((self)["@handlers"]);if (_1_==nil) then return ((function()
return ((function() local _1_=((self):eventTableCreate()); (self)["@handlers"]=_1_; return _1_ end)())
end)):value() end; return _1_ end)())
;
end
)
,"events","eventTableForEdit\r\n\t\" Private - Return the mapping event names to actions of the receiver.\r\n\tSet the internal map if not present.\r\n\t\"\r\n\r\n\t^self basicAt: '@handlers' ifAbsent: [\r\n\t\tself basicAt: '@handlers' put: self eventTableCreate\r\n\t]","(\r\nfunction(self)\r\nreturn ((function() local _1_=((self)[\"@handlers\"]);if (_1_==nil) then return ((function()\r\nreturn ((function() local _1_=((self):eventTableCreate()); (self)[\"@handlers\"]=_1_; return _1_ end)())\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"releaseEventTable",0
,(
function(self)
(self):basicDelete_("@handlers")
;
return (self)
;
end
)
,"events","releaseEventTable\r\n\t\" Private - Release the event table. \"\r\n\r\n\tself basicDelete: '@handlers'","(\r\nfunction(self)\r\n(self):basicDelete_(\"@handlers\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"release",0
,(
function(self)
(self):releaseEventTable()
;
return (self)
;
end
)
,"release","release\r\n\t\" Release the receiver. \"\r\n\r\n\tself releaseEventTable ","(\r\nfunction(self)\r\n(self):releaseEventTable()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"eventTableCreate",0
,(
function(self)
return ((smalltalk.PoolDictionary):new())
;
end
)
,"events","eventTableCreate\r\n\t\" Private - Return a new mapping for the receiver. \"\r\n\r\n\t^PoolDictionary new","(\r\nfunction(self)\r\nreturn ((smalltalk.PoolDictionary):new())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"canTriggerEvent:",0
,(
function(self,anEvent)
return (((self)["$klass"]):canTriggerEvent_(anEvent))
;
end
)
,"events","canTriggerEvent: anEvent\r\n\t\" Return true if the receiver can trigger anEvent.\"\r\n\r\n\t^self class canTriggerEvent: anEvent","(\r\nfunction(self,anEvent)\r\nreturn (((self)[\"$klass\"]):canTriggerEvent_(anEvent))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"invalidEvent:",0
,(
function(self,anEvent)
return (self)
;
end
)
,"events","invalidEvent: anEvent\r\n\t\" Private - Notify the situation where an unknown event is registered.\r\n\tDefault implementation ignore error during unknown event registration.\r\n\t\"\r\n\r\n\t^self\r\n\"DEBUG-\t^self print: self class name,' invalidEvent: #',anEvent asString. \"","(\r\nfunction(self,anEvent)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"when:evaluate:",0
,(
function(self,anEvent,anAction)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local actions=nil
(function() if ((((self):canTriggerEvent_(anEvent)))==true) then return nil else return ((function()
return ((self):invalidEvent_(anEvent))
end))() end end)()
;
(function()
	local _s8_value=((self):actionListForEvent_(anEvent))
	actions=_s8_value; return _s8_value
end)()
;
(function() if ((((actions):includes_(anAction)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(self):setActionList_forEvent_((actions):copyWith_(anAction), anEvent)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"events","when: anEvent evaluate: anAction\r\n\t\" Register anAction to the be evaluated when the receiver triggers the event. \"\r\n\r\n\t| actions |\r\n\t(self canTriggerEvent: anEvent) ifFalse: [ self invalidEvent: anEvent ].\r\n\tactions := self actionListForEvent: anEvent.\r\n\t(actions includes: anAction) ifTrue: [ ^self ].\r\n\tself\r\n\t\tsetActionList: (actions copyWith: anAction)\r\n\t\tforEvent: anEvent","(\r\nfunction(self,anEvent,anAction)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal actions=nil\r\n(function() if ((((self):canTriggerEvent_(anEvent)))==true) then return nil else return ((function()\r\nreturn ((self):invalidEvent_(anEvent))\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):actionListForEvent_(anEvent))\r\n\tactions=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((actions):includes_(anAction)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self):setActionList_forEvent_((actions):copyWith_(anAction), anEvent)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"actionListForEvent:",0
,(
function(self,anEvent)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function()
	local _s8_value=(((self):eventTable()):at_ifAbsent_(anEvent, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.ActionSequence):new())
end
)()
error(_s8Ret_,0)
end)())
end)))
	result=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((result):asActionSequence())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"events","actionListForEvent: anEvent\r\n\t\" Private - Return a collection of actions to evaluate when the event is triggered. \"\r\n\r\n\t| result |\r\n\tresult := self eventTable at: anEvent ifAbsent: [ ^ActionSequence new ].\r\n\t^result asActionSequence","(\r\nfunction(self,anEvent)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(((self):eventTable()):at_ifAbsent_(anEvent, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.ActionSequence):new())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((result):asActionSequence())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"setActionList:forEvent:",0
,(
function(self,actionSequence,anEvent)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local action=nil
(function()
	local _s8_value=(((actionSequence):asActionSequence()):reduced())
	action=_s8_value; return _s8_value
end)()
;
(function() if ((((((action))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):removeActionsForEvent_(anEvent))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
((self):eventTableForEdit()):at_put_(anEvent, action)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"events","setActionList: actionSequence forEvent: anEvent\r\n\t\" Private - Set the list of actions to evaluate when the event is triggered by the receiver. \"\r\n\r\n\t| action |\r\n\taction := actionSequence asActionSequence reduced.\r\n\taction isNil ifTrue: [ ^self removeActionsForEvent: anEvent ].\r\n\tself eventTableForEdit at: anEvent put: action.","(\r\nfunction(self,actionSequence,anEvent)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal action=nil\r\n(function()\r\n\tlocal _s8_value=(((actionSequence):asActionSequence()):reduced())\r\n\taction=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((action))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):removeActionsForEvent_(anEvent))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n((self):eventTableForEdit()):at_put_(anEvent, action)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"removeActionsSatisfying:forEvent:",0
,(
function(self,aBlock,anEvent)
(self):setActionList_forEvent_((((self):actionListForEvent_(anEvent)):asArray()):reject_(aBlock), anEvent)
;
return (self)
;
end
)
,"events","removeActionsSatisfying: aBlock forEvent: anEvent\r\n\t\" Private - Remove all actions for the event that evaluates aBlock to true.\"\r\n\r\n\tself\r\n\t\tsetActionList: ((self actionListForEvent: anEvent) asArray reject: aBlock)\r\n\t\tforEvent: anEvent","(\r\nfunction(self,aBlock,anEvent)\r\n(self):setActionList_forEvent_((((self):actionListForEvent_(anEvent)):asArray()):reject_(aBlock), anEvent)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"removeActionsWithReceiver:forEvent:",0
,(
function(self,anObject,anEvent)
(self):removeActionsSatisfying_forEvent_((function(each)
return ((smalltalk:boolean_and_((smalltalk:responds_to_((each),("receiver"))),(function()
return ((((each):receiver())==anObject))
end))))
end), anEvent)
;
return (self)
;
end
)
,"events","removeActionsWithReceiver: anObject forEvent: anEvent\r\n\t\" Remove all actions for the event in the receiver's event table which have anObject as receiver. \"\r\n\r\n\tself\r\n\t\tremoveActionsSatisfying: [:each| (each respondsTo: #receiver) and: [ each receiver == anObject ] ]\r\n\t\tforEvent: anEvent","(\r\nfunction(self,anObject,anEvent)\r\n(self):removeActionsSatisfying_forEvent_((function(each)\r\nreturn ((smalltalk:boolean_and_((smalltalk:responds_to_((each),(\"receiver\"))),(function()\r\nreturn ((((each):receiver())==anObject))\r\nend))))\r\nend), anEvent)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"removeAction:forEvent:",0
,(
function(self,anAction,anEvent)
(self):removeActionsSatisfying_forEvent_((function(each)
return ((smalltalk:isObject_equalTo_((each),anAction)))
end), anEvent)
;
return (self)
;
end
)
,"events","removeAction: anAction forEvent: anEvent\r\n\t\" Remove anAction for the event in the receiver's event table which have anObject as receiver. \"\r\n\r\n\tself\r\n\t\tremoveActionsSatisfying: [:each| each = anAction ]\r\n\t\tforEvent: anEvent","(\r\nfunction(self,anAction,anEvent)\r\n(self):removeActionsSatisfying_forEvent_((function(each)\r\nreturn ((smalltalk:isObject_equalTo_((each),anAction)))\r\nend), anEvent)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"when:do:",0
,(
function(self,anEvent,aBlock)
(self):when_evaluate_(anEvent, aBlock)
;
return (self)
;
end
)
,"events","when: anEvent do: aBlock\r\n\t\" Evaluate aBlock when the receiver triggers the event. \"\r\n\r\n\tself when: anEvent evaluate: aBlock","(\r\nfunction(self,anEvent,aBlock)\r\n(self):when_evaluate_(anEvent, aBlock)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"when:send:to:withArguments:",0
,(
function(self,anEvent,selector,anObject,array)
(self):when_evaluate_(anEvent, (smalltalk.LinkMessage):receiver_selector_arguments_(anObject, selector, array))
;
return (self)
;
end
)
,"events","when: anEvent send: selector to: anObject withArguments: array\r\n\t\" Send the message when the receiver triggers the event. \"\r\n\r\n\tself\r\n\t\twhen: anEvent\r\n\t\tevaluate: (LinkMessage\r\n\t\t\treceiver: anObject\r\n\t\t\tselector: selector\r\n\t\t\targuments: array)","(\r\nfunction(self,anEvent,selector,anObject,array)\r\n(self):when_evaluate_(anEvent, (smalltalk.LinkMessage):receiver_selector_arguments_(anObject, selector, array))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"when:send:to:",0
,(
function(self,anEvent,selector,anObject)
(self):when_evaluate_(anEvent, (smalltalk.Message):receiver_selector_arguments_(anObject, selector, (smalltalk.Array):new_((selector):argumentCount())))
;
return (self)
;
end
)
,"events","when: anEvent send: selector to: anObject\r\n\t\" Send the message when the receiver triggers the event. \"\r\n\r\n\tself\r\n\t\twhen: anEvent\r\n\t\tevaluate: (Message\r\n\t\t\treceiver: anObject\r\n\t\t\tselector: selector\r\n\t\t\targuments: (Array new: selector argumentCount))","(\r\nfunction(self,anEvent,selector,anObject)\r\n(self):when_evaluate_(anEvent, (smalltalk.Message):receiver_selector_arguments_(anObject, selector, (smalltalk.Array):new_((selector):argumentCount())))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"triggerEvent:ifNotHandled:",0
,(
function(self,anEvent,aBlock)
return ((((self):eventTable()):at_ifAbsent_(anEvent, (function()
return (aBlock)
end))):evaluate())
;
end
)
,"events","triggerEvent: anEvent ifNotHandled: aBlock\r\n\t\" Trigger the event.\r\n\tIf the event is not handled, return the value of evaluating aBlock,\r\n        or the value returned by the most recently defined event handler.\r\n\t\"\r\n\r\n\t^(self eventTable at: anEvent ifAbsent: [ aBlock ]) evaluate","(\r\nfunction(self,anEvent,aBlock)\r\nreturn ((((self):eventTable()):at_ifAbsent_(anEvent, (function()\r\nreturn (aBlock)\r\nend))):evaluate())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"actionForEvent:",0
,(
function(self,anEvent)
return (((self):eventTable()):at_ifAbsent_(anEvent, (function()
return (nil)
end)))
;
end
)
,"events","actionForEvent: anEvent\r\n\t\" Return the action to evaluate when the event is triggered by the receiver (or nil). \"\r\n\r\n\t^self eventTable at: anEvent ifAbsent: [ nil ]","(\r\nfunction(self,anEvent)\r\nreturn (((self):eventTable()):at_ifAbsent_(anEvent, (function()\r\nreturn (nil)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"triggerEvent:",0
,(
function(self,anEvent)
return (((self):actionForEvent_(anEvent)):evaluate())
;
end
)
,"events","triggerEvent: anEvent\r\n\t\" Return the result of triggering the event. \"\r\n\r\n\t^(self actionForEvent: anEvent) evaluate","(\r\nfunction(self,anEvent)\r\nreturn (((self):actionForEvent_(anEvent)):evaluate())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"triggerEvent:withArguments:",0
,(
function(self,anEvent,array)
return (((self):actionForEvent_(anEvent)):evaluateWithArguments_(array))
;
end
)
,"events","triggerEvent: anEvent withArguments: array\r\n\t\" Return the result of triggering the event. \"\r\n\r\n\t^(self actionForEvent: anEvent) evaluateWithArguments: array","(\r\nfunction(self,anEvent,array)\r\nreturn (((self):actionForEvent_(anEvent)):evaluateWithArguments_(array))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"triggerEvent:with:",0
,(
function(self,anEvent,anObject)
return ((self):triggerEvent_withArguments_(anEvent, (smalltalk.Array):with_(anObject)))
;
end
)
,"events","triggerEvent: anEvent with: anObject\r\n\t\" Return the result of triggering the event. \"\r\n\r\n\t^self triggerEvent: anEvent withArguments: (Array with: anObject)","(\r\nfunction(self,anEvent,anObject)\r\nreturn ((self):triggerEvent_withArguments_(anEvent, (smalltalk.Array):with_(anObject)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"triggerEvent:with:with:",0
,(
function(self,anEvent,anObject,anotherObject)
return ((self):triggerEvent_withArguments_(anEvent, (smalltalk.Array):with_with_(anObject, anotherObject)))
;
end
)
,"events","triggerEvent: anEvent with: anObject with: anotherObject\r\n\t\" Return the result of triggering the event. \"\r\n\r\n\t^self triggerEvent: anEvent withArguments: (Array with: anObject with: anotherObject)","(\r\nfunction(self,anEvent,anObject,anotherObject)\r\nreturn ((self):triggerEvent_withArguments_(anEvent, (smalltalk.Array):with_with_(anObject, anotherObject)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"triggerEvent:withArguments:ifNotHandled:",0
,(
function(self,anEvent,array,aBlock)
return ((((self):eventTable()):at_ifAbsent_(anEvent, (function()
return (aBlock)
end))):evaluateWithArguments_(array))
;
end
)
,"events","triggerEvent: anEvent withArguments: array ifNotHandled: aBlock\r\n\t\" Return the result of triggering the event with the arguments.\r\n\tIf the event is not handled, return the value of evaluating aBlock,\r\n\tor the value returned by the most recently defined event handler action.\r\n\t\"\r\n\r\n\t^(self eventTable at: anEvent ifAbsent: [ aBlock ]) evaluateWithArguments: array","(\r\nfunction(self,anEvent,array,aBlock)\r\nreturn ((((self):eventTable()):at_ifAbsent_(anEvent, (function()\r\nreturn (aBlock)\r\nend))):evaluateWithArguments_(array))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"triggerEvent:with:ifNotHandled:",0
,(
function(self,anEvent,anObject,aBlock)
return ((self):triggerEvent_withArguments_ifNotHandled_(anEvent, (smalltalk.Array):with_(anObject), aBlock))
;
end
)
,"events","triggerEvent: anEvent with: anObject ifNotHandled: aBlock\r\n\t\" Return the result of triggering the event with anObject. \"\r\n\r\n\t^self triggerEvent: anEvent withArguments: (Array with: anObject) ifNotHandled: aBlock","(\r\nfunction(self,anEvent,anObject,aBlock)\r\nreturn ((self):triggerEvent_withArguments_ifNotHandled_(anEvent, (smalltalk.Array):with_(anObject), aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"hasActionForEvent:",0
,(
function(self,anEvent)
return (((self):eventTable()):includesKey_(anEvent))
;
end
)
,"events","hasActionForEvent: anEvent\r\n\t\" Return true if the receiver has an action registered for the event. \"\r\n\r\n\t^self eventTable includesKey: anEvent","(\r\nfunction(self,anEvent)\r\nreturn (((self):eventTable()):includesKey_(anEvent))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"eventsHandled",0
,(
function(self)
return (((self):eventTable()):keys())
;
end
)
,"events","eventsHandled\r\n\t\" Return the events for which there are actions registered in the receiver. \"\r\n\r\n\t^self eventTable keys","(\r\nfunction(self)\r\nreturn (((self):eventTable()):keys())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"removeAllActionsWithReceiver:",0
,(
function(self,anObject)
(((self):eventTable()):keys()):do_((function(each)
return ((self):removeActionsWithReceiver_forEvent_(anObject, each))
end))
;
return (self)
;
end
)
,"events","removeAllActionsWithReceiver: anObject\r\n\t\" Remove all actions for all events in the receiver's event table with anObject as receiver. \"\r\n\r\n\tself eventTable keys do: [:each|self removeActionsWithReceiver: anObject forEvent: each ]","(\r\nfunction(self,anObject)\r\n(((self):eventTable()):keys()):do_((function(each)\r\nreturn ((self):removeActionsWithReceiver_forEvent_(anObject, each))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"removeActionsForEvent:",0
,(
function(self,anEvent)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local actions=nil
(function()
	local _s8_value=((self):eventTable())
	actions=_s8_value; return _s8_value
end)()
;
(actions):removeKey_ifAbsent_(anEvent, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))
;
(function() if ((((actions):isEmpty()))==true) then return ((function()
return ((self):releaseEventTable())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"events","removeActionsForEvent: anEvent\r\n\t\" Remove all actions for the event. \"\r\n\r\n\t| actions |\r\n\tactions := self eventTable.\r\n\tactions removeKey: anEvent ifAbsent: [ ^self ].\r\n\tactions isEmpty ifTrue: [ self releaseEventTable ].","(\r\nfunction(self,anEvent)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal actions=nil\r\n(function()\r\n\tlocal _s8_value=((self):eventTable())\r\n\tactions=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(actions):removeKey_ifAbsent_(anEvent, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))\r\n;\r\n(function() if ((((actions):isEmpty()))==true) then return ((function()\r\nreturn ((self):releaseEventTable())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"addDependent:",0
,(
function(self,anObject)
(self):when_send_to_("changed", "update", anObject)
;
return (self)
;
end
)
,"changes","addDependent: anObject\r\n\t\" Make anObject dependent of the receiver. \"\r\n\r\n\tself when: #changed send: #update to: anObject","(\r\nfunction(self,anObject)\r\n(self):when_send_to_(\"changed\", \"update\", anObject)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"changed",0
,(
function(self)
(self):triggerEvent_("changed")
;
return (self)
;
end
)
,"changes","changed\r\n\t\" Update the dependents of the receiver. \"\r\n\r\n\tself triggerEvent: #changed","(\r\nfunction(self)\r\n(self):triggerEvent_(\"changed\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"updateEvent:selector:arguments:",0
,(
function(self,anEvent,selector,array)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local action=nil
(function()
	local _s8_value=(((self):eventTable()):at_ifAbsent_(anEvent, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end)))
	action=_s8_value; return _s8_value
end)()
;
((action):asActionSequence()):evaluateWithSelector_withArguments_(selector, array)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"changes","updateEvent: anEvent selector: selector arguments: array\r\n\t\" Private - Trigger the event with the message selector and the arguments. \"\r\n\r\n\t| action |\r\n\taction := self eventTable at: anEvent ifAbsent: [ ^nil ].\r\n\taction asActionSequence\r\n            evaluateWithSelector: selector\r\n            withArguments: array","(\r\nfunction(self,anEvent,selector,array)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal action=nil\r\n(function()\r\n\tlocal _s8_value=(((self):eventTable()):at_ifAbsent_(anEvent, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)))\r\n\taction=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((action):asActionSequence()):evaluateWithSelector_withArguments_(selector, array)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"changed:",0
,(
function(self,aspect)
(self):updateEvent_selector_arguments_("changed", "update:", (smalltalk.Array):with_(aspect))
;
return (self)
;
end
)
,"changes","changed: aspect\r\n\t\" Update the dependents on aspect of the receiver. \"\r\n\r\n\tself updateEvent: #changed selector: #update: arguments: (Array with: aspect)","(\r\nfunction(self,aspect)\r\n(self):updateEvent_selector_arguments_(\"changed\", \"update:\", (smalltalk.Array):with_(aspect))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"changed:with:",0
,(
function(self,aspect,anObject)
(self):updateEvent_selector_arguments_("changed", "update:with:", (smalltalk.Array):with_with_(aspect, anObject))
;
return (self)
;
end
)
,"changes","changed: aspect with: anObject\r\n\t\" Update the dependents on aspect of the receiver. \"\r\n\r\n\tself updateEvent: #changed selector: #update:with: arguments: (Array with: aspect with: anObject)","(\r\nfunction(self,aspect,anObject)\r\n(self):updateEvent_selector_arguments_(\"changed\", \"update:with:\", (smalltalk.Array):with_with_(aspect, anObject))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"changed:with:with:",0
,(
function(self,aspect,anObject,anotherObject)
(self):updateEvent_selector_arguments_("changed", "update:with:with:", (smalltalk.Array):with_with_with_(aspect, anObject, anotherObject))
;
return (self)
;
end
)
,"changes","changed: aspect with: anObject with: anotherObject\r\n\t\" Update the dependents on aspect of the receiver. \"\r\n\r\n\tself updateEvent: #changed selector: #update:with:with: arguments: (Array with: aspect with: anObject with: anotherObject)","(\r\nfunction(self,aspect,anObject,anotherObject)\r\n(self):updateEvent_selector_arguments_(\"changed\", \"update:with:with:\", (smalltalk.Array):with_with_with_(aspect, anObject, anotherObject))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"dependents",0
,(
function(self)
return (((self):actionListForEvent_("changed")):collect_((function(each)
return ((each):receiver())
end)))
;
end
)
,"changes","dependents\r\n\t\" Return the dependents of the receiver. \"\r\n\r\n\t^(self actionListForEvent: #changed) collect: [:each | each receiver ]","(\r\nfunction(self)\r\nreturn (((self):actionListForEvent_(\"changed\")):collect_((function(each)\r\nreturn ((each):receiver())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"update:with:",0
,(
function(self,anObject,anotherObject)
return (self)
;
end
)
,"changes","update: anObject with: anotherObject\r\n\t\" Update the receiver.\r\n\tDefault implementation do nothing.\r\n\tThis method must be refined by subclasses to support change/update protocoll.\r\n\t\"","(\r\nfunction(self,anObject,anotherObject)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"update:",0
,(
function(self,anObject)
return (self)
;
end
)
,"changes","update: anObject\r\n\t\" Update the receiver.\r\n\tDefault implementation do nothing.\r\n\tThis method must be refined by subclasses to support change/update protocoll.\r\n\t\"","(\r\nfunction(self,anObject)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"printingWith:do:",0
,(
function(self,printBlock,aBlock)
local backup=nil
local result=nil
(function()
	local _s8_value=(print)
	backup=_s8_value; return _s8_value
end)()
;
((function()
(printBlock):installAs_("print");return ((function()
	local _s8_value=((aBlock):value())
	result=_s8_value; return _s8_value
end)())
end)):ensure_((function()
return ((backup):installAs_("print"))
end))
;
return (result)
;
end
)
,"printing","printingWith: printBlock do: aBlock\r\n\t\" Return the result of evaluating aBlock, redirecting output to printBlock. \"\r\n\r\n\t| backup result |\r\n\tbackup := #{print}.\r\n\t[\tprintBlock installAs: #print.\r\n\t\tresult := aBlock value.\r\n\t] ensure: [ backup installAs: #print ].\r\n\t^result","(\r\nfunction(self,printBlock,aBlock)\r\nlocal backup=nil\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(print)\r\n\tbackup=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((function()\r\n(printBlock):installAs_(\"print\");return ((function()\r\n\tlocal _s8_value=((aBlock):value())\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend)):ensure_((function()\r\nreturn ((backup):installAs_(\"print\"))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isParseFailure",0
,(
function(self)
return (false)
;
end
)
,"Compiler-testing","isParseFailure\r\n\t\" Return true if the receiver is a parse failure. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asParser",0
,(
function(self)
return ((self):subclassResponsibility_("asParser"))
;
end
)
,"Compiler-converting","asParser\r\n\t\" Return a parser on the receiver. \"\r\n\r\n\t^self subclassResponsibility: #asParser","(\r\nfunction(self)\r\nreturn ((self):subclassResponsibility_(\"asParser\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"ifUndefined:",0
,(
function(self,aBlock)
return (self)
;
end
)
,"Compiler-inlines","ifUndefined: aBlock\r\n\t\" Return the receiver or the result of evaluating aBlock if it is undefined.\r\n\tNotes:\t-this message is inlined by compiler (it is not sent)\r\n\t\t-aBlock is evaluated ONLY in case of undefined.\r\n\t\"\r\n\r\n\t^self","(\r\nfunction(self,aBlock)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"orNil",0
,(
function(self)
return (self)
;
end
)
,"Compiler-inlines","orNil\r\n\t\" Return the receiver or nil.\r\n\tNotes:\t-this message is inlined by compiler (it is not sent)\r\n\t\t-nil, null and undefined returns nil.\r\n\t\"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"scanFrom:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local chunk=nil
(function()
	local _s8_value=((aStream):nextChunk())
	chunk=_s8_value; return _s8_value
end)()
;
(function() if ((((chunk):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((chunk):evaluateIn_(self))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Tools-fileIn","scanFrom: aStream\r\n\t\" FileIn - Return the result of evaluating the next chunk of data from aStream.\r\n\tThe method can be refined by chunk readers to support custom fileIn features.\r\n\tMust return the next reader to use for reading remaining contents from aStream.\r\n\t\"\r\n\r\n\t| chunk |\r\n\tchunk := aStream nextChunk.\r\n\tchunk isEmpty ifTrue: [ ^nil ].\r\n\t^chunk evaluateIn: self","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal chunk=nil\r\n(function()\r\n\tlocal _s8_value=((aStream):nextChunk())\r\n\tchunk=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((chunk):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((chunk):evaluateIn_(self))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"javascriptCodeFollows",0
,(
function(self)
return ((smalltalk.ChunkReader):javascript())
;
end
)
,"Tools-fileIn","javascriptCodeFollows\r\n\t\" FileIn - FileIn javascript code on current stream.\r\n\tConsume stream contents upto end of stream evaluating the contents as javascript code.\r\n\t\"\r\n\r\n\t^ChunkReader javascript","(\r\nfunction(self)\r\nreturn ((smalltalk.ChunkReader):javascript())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"cancelFileIn",0
,(
function(self)
return ((self):ignoreFileInUpTo_(nil))
;
end
)
,"Tools-fileIn","cancelFileIn\r\n\t\" FileIn - Cancel fileIn on current stream. \"\r\n\r\n\t^self ignoreFileInUpTo: nil","(\r\nfunction(self)\r\nreturn ((self):ignoreFileInUpTo_(nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"ignoreFileInUpTo:",0
,(
function(self,aToken)
return ((smalltalk.SkipFileInReader):upTo_(aToken))
;
end
)
,"Tools-fileIn","ignoreFileInUpTo: aToken\r\n\t\" FileIn - Ignore fileIn on current stream upto token. \"\r\n\r\n\t^SkipFileInReader upTo: aToken","(\r\nfunction(self,aToken)\r\nreturn ((smalltalk.SkipFileInReader):upTo_(aToken))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"==","_eq_eq"
,(
function(self,anObject)
return (((anObject)==self))
;
end
)
,"Tools-comparing","== anObject\r\n\t^anObject == self","(\r\nfunction(self,anObject)\r\nreturn (((anObject)==self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"~~","_no_no"
,(
function(self,anObject)
return (((((self)==anObject))==false))
;
end
)
,"Tools-comparing","~~ anObject\r\n\t^(self == anObject) == false","(\r\nfunction(self,anObject)\r\nreturn (((((self)==anObject))==false))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"isFile:olderThan:",0
,(
function(self,fileName,referenceFileName)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((((((smalltalk.Smalltalk):global())["isFileOlderThan"])))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (isFileOlderThan(fileName, referenceFileName))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Builders-emmiting","isFile: fileName olderThan: referenceFileName\r\n\t\" Private - Return true if the file at pathName is older than file at referenceFileName. \"\r\n\r\n\t(Smalltalk global basicAt: #isFileOlderThan) isNil ifTrue: [ ^false ].\r\n\t^# #isFileOlderThan: fileName with: referenceFileName","(\r\nfunction(self,fileName,referenceFileName)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((((((smalltalk.Smalltalk):global())[\"isFileOlderThan\"])))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (isFileOlderThan(fileName, referenceFileName))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"emmit:startingWith:",0
,(
function(self,fileName,initial)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local output=nil
local cr=nil
local nativeFileName=nil
(function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((fileName),".")),(smalltalk.String):nativeFileExtension()))
	nativeFileName=_s8_value; return _s8_value
end)()
;
(function() if ((((self):isFile_olderThan_(fileName, nativeFileName)))==true) then return ((function()
(self):fileInJS_(nativeFileName);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (fileContents(nativeFileName))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=(initial)
	output=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=("\r\n")
	cr=_s8_value; return _s8_value
end)()
;
(self):note_(smalltalk:object_comma_((smalltalk:object_comma_(("emmit('"),fileName)),"')"))
;
((function()
(self):printingWith_do_((function(what)
return ((function()
	local _s8_value=((function() if ((((((output))==nil)))==true) then return ((function()
return (what)
end))() else return ((function()
return (smalltalk:object_comma_((smalltalk:object_comma_((output),cr)),what))
end))() end end)())
	output=_s8_value; return _s8_value
end)())
end), (function()
return ((self):fileIn_(fileName))
end));return ((output):outputToFile_(nativeFileName))
end)):on_do_(smalltalk.Error, (function(ex)
(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:asString_((output))),cr)),smalltalk:toString_((ex)))):outputToFile_(smalltalk:object_comma_((fileName),".error.txt"));(self):print_(smalltalk:toString_((ex)));return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:toString_((ex)))
end
)()
error(_s8Ret_,0)
end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (output)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Builders-emmiting","emmit: fileName startingWith: initial\r\n\t\" FileIn the contents of file at fileName redirecting output(emmited code) to a native file. \"\r\n\r\n\t| output cr nativeFileName |\r\n\tnativeFileName := fileName ,$. ,String nativeFileExtension.\r\n\t(self isFile: fileName olderThan: nativeFileName) ifTrue: [\r\n\t\tself fileInJS: nativeFileName.\r\n\t\t^# #fileContents: nativeFileName\r\n\t].\r\n\toutput := initial. cr := '\r\n'.\tself note: 'emmit(''',fileName,''')'.\r\n[\tself printingWith: [:what|\r\n\t\toutput := output isNil ifTrue: [what] ifFalse: [ output ,cr ,what ]\r\n\t] do: [ self fileIn: fileName ].\r\n\toutput outputToFile: nativeFileName.\r\n]\ton: Error do: [:ex|\r\n\t(output asString ,cr ,ex toString) outputToFile: fileName ,'.error.txt'.\r\n\tself print: ex toString.\r\n\t^ex toString\r\n].\t^output","(\r\nfunction(self,fileName,initial)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal output=nil\r\nlocal cr=nil\r\nlocal nativeFileName=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((fileName),\".\")),(smalltalk.String):nativeFileExtension()))\r\n\tnativeFileName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((self):isFile_olderThan_(fileName, nativeFileName)))==true) then return ((function()\r\n(self):fileInJS_(nativeFileName);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (fileContents(nativeFileName))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(initial)\r\n\toutput=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(\"\\r\\n\")\r\n\tcr=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):note_(smalltalk:object_comma_((smalltalk:object_comma_((\"emmit('\"),fileName)),\"')\"))\r\n;\r\n((function()\r\n(self):printingWith_do_((function(what)\r\nreturn ((function()\r\n\tlocal _s8_value=((function() if ((((((output))==nil)))==true) then return ((function()\r\nreturn (what)\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((output),cr)),what))\r\nend))() end end)())\r\n\toutput=_s8_value; return _s8_value\r\nend)())\r\nend), (function()\r\nreturn ((self):fileIn_(fileName))\r\nend));return ((output):outputToFile_(nativeFileName))\r\nend)):on_do_(smalltalk.Error, (function(ex)\r\n(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:asString_((output))),cr)),smalltalk:toString_((ex)))):outputToFile_(smalltalk:object_comma_((fileName),\".error.txt\"));(self):print_(smalltalk:toString_((ex)));return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:toString_((ex)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (output)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"emmit:",0
,(
function(self,fileName)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((fileName):isString()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((fileName):asArray()):emmit())
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):emmit_startingWith_(fileName, nil))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Builders-emmiting","emmit: fileName\r\n\t\" FileIn the contents of file at fileName redirecting output(emmited code) to a native file. \"\r\n\r\n\tfileName isString ifFalse: [ ^fileName asArray emmit ].\r\n\t^self emmit: fileName startingWith: nil","(\r\nfunction(self,fileName)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((fileName):isString()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((fileName):asArray()):emmit())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):emmit_startingWith_(fileName, nil))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asJS",0
,(
function(self)
return ((smalltalk.JS):_at(self))
;
end
)
,"Natives-converting","asJS\r\n\t\" Convert the receiver to JS instance (for reflection). \"\r\n\r\n\t^JS @ self","(\r\nfunction(self)\r\nreturn ((smalltalk.JS):_at(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asNativeHandle",0
,(
function(self)
return (((smalltalk.NativeHandle):support()):_at(self))
;
end
)
,"Natives-converting","asNativeHandle\r\n\t\" Convert the receiver to NativeHandle instance (for reflection). \"\r\n\r\n\t^NativeHandle support @ self","(\r\nfunction(self)\r\nreturn (((smalltalk.NativeHandle):support()):_at(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"handleOf:",0
,(
function(self,nativeObject)
return ((function() if ((((((nativeObject))~=nil)))==true) then return ((function()
return ((function() if ((((smalltalk:boolean_and_(((self):isObject_(nativeObject)),(function()
return ((((nativeObject["handle"]))~=nil))
end)))))==true) then return ((function()
return ((nativeObject):handle())
end))() else return ((function()
return (nativeObject)
end))() end end)())
end))() else return ((function()
return ((smalltalk.NativeObject):undefined())
end))() end end)())
;
end
)
,"Natives-converting","handleOf: nativeObject\r\n\t\" Private - Return the handle of nativeObject (or the nativeObject). \"\r\n\r\n\t^nativeObject notNil ifTrue: [\r\n\t\t((self isObject: nativeObject)\r\n\t\tand: [ nativeObject[#handle] notNil ])\r\n\t\tifTrue: [ nativeObject handle ]\r\n\t\tifFalse: [ nativeObject ]\r\n\t] ifFalse: [ NativeObject undefined ]","(\r\nfunction(self,nativeObject)\r\nreturn ((function() if ((((((nativeObject))~=nil)))==true) then return ((function()\r\nreturn ((function() if ((((smalltalk:boolean_and_(((self):isObject_(nativeObject)),(function()\r\nreturn ((((nativeObject[\"handle\"]))~=nil))\r\nend)))))==true) then return ((function()\r\nreturn ((nativeObject):handle())\r\nend))() else return ((function()\r\nreturn (nativeObject)\r\nend))() end end)())\r\nend))() else return ((function()\r\nreturn ((smalltalk.NativeObject):undefined())\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asJavascript",0
,(
function(self)
return ((self):asNative())
;
end
)
,"js","asJavascript\r\n\t\" OBSOLETE - Return the native representation of the receiver. \"\r\n\r\n\t^self asNative","(\r\nfunction(self)\r\nreturn ((self):asNative())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"asLua",0
,(
function(self)
return (smalltalk:asString_((self)))
;
end
)
,"Lua-lua","asLua\r\n\t\" Return the native representation of the receiver.\r\n\tThe default implementation return the receiver as a String.\r\n\tThis method must be refined by subclasses implementing native literal representation.\r\n\t\"\r\n\r\n\t^self asString","(\r\nfunction(self)\r\nreturn (smalltalk:asString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Object,"__tostring",0
,(
function(self)
return (smalltalk:toString_((self)))
;
end
)
,"Lua-printing","__tostring\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"current",0
,(
function(self)
return (smalltalk)
;
end
)
,"accessing","current\r\n\t\" Return the current instance of the receiver. \"\r\n\r\n\t^#{smalltalk}","(\r\nfunction(self)\r\nreturn (smalltalk)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"at:ifAbsent:",0
,(
function(self,aGlobalName,aBlock)
return ((function() local _1_=(((self):current())[aGlobalName]);if (_1_==nil) then return (aBlock):value() end; return _1_ end)())
;
end
)
,"accessing","at: aGlobalName ifAbsent: aBlock\r\n\t\" Return the global known as aGlobalName in the receiver or the result of evaluating aBlock. \"\r\n\r\n\t^self current basicAt: aGlobalName ifAbsent: aBlock","(\r\nfunction(self,aGlobalName,aBlock)\r\nreturn ((function() local _1_=(((self):current())[aGlobalName]);if (_1_==nil) then return (aBlock):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"includesKey:",0
,(
function(self,aGlobalName)
local none=nil
(function()
	local _s8_value=({})
	none=_s8_value; return _s8_value
end)()
;
return ((smalltalk:booleanNot_(((((function() local _1_=(((smalltalk.Smalltalk):current())[aGlobalName]);if (_1_==nil) then return ((function()
return (none)
end)):value() end; return _1_ end)())==none)))))
;
end
)
,"accessing","includesKey: aGlobalName\r\n\t\" Return true if the global aGlobalName is defined in the receiver. \"\r\n\r\n\t| none |\r\n\tnone := {'undefined'\tlua'{}'}.\r\n\t^((Smalltalk current\r\n\t\tbasicAt: aGlobalName ifAbsent: [none])\r\n\t\t== none) not","(\r\nfunction(self,aGlobalName)\r\nlocal none=nil\r\n(function()\r\n\tlocal _s8_value=({})\r\n\tnone=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((smalltalk:booleanNot_(((((function() local _1_=(((smalltalk.Smalltalk):current())[aGlobalName]);if (_1_==nil) then return ((function()\r\nreturn (none)\r\nend)):value() end; return _1_ end)())==none)))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"at:",0
,(
function(self,aGlobalName)
return ((function() local _1_=(((self):current())[aGlobalName]);if (_1_==nil) then return ((function()
return (nil)
end)):value() end; return _1_ end)())
;
end
)
,"accessing","at: aGlobalName\r\n\t\" Return the global known as aGlobalName in the receiver or nil. \"\r\n\r\n\t^self current basicAt: aGlobalName ifAbsent: [ nil ]","(\r\nfunction(self,aGlobalName)\r\nreturn ((function() local _1_=(((self):current())[aGlobalName]);if (_1_==nil) then return ((function()\r\nreturn (nil)\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"at:put:",0
,(
function(self,aGlobalName,anObject)
return ((function() local _1_=(anObject); ((self):current())[aGlobalName]=_1_; return _1_ end)())
;
end
)
,"accessing","at: aGlobalName put: anObject\r\n\t\" Set the global known as aGlobalName in the receiver top anObject. \"\r\n\r\n\t^self current basicAt: aGlobalName put: anObject","(\r\nfunction(self,aGlobalName,anObject)\r\nreturn ((function() local _1_=(anObject); ((self):current())[aGlobalName]=_1_; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"at:ifAbsentPut:",0
,(
function(self,aGlobalName,aBlock)
return ((self):at_ifAbsent_(aGlobalName, (function()
return ((self):at_put_(aGlobalName, (aBlock):value()))
end)))
;
end
)
,"accessing","at: aGlobalName ifAbsentPut: aBlock\r\n\t\" Return the global known as aGlobalName in the receiver or the result of evaluating aBlock.\r\n\tSet the global to return value if was absent.\r\n\t\"\r\n\r\n\t^self at: aGlobalName ifAbsent: [\r\n\t\tself at: aGlobalName put: aBlock value\r\n\t]","(\r\nfunction(self,aGlobalName,aBlock)\r\nreturn ((self):at_ifAbsent_(aGlobalName, (function()\r\nreturn ((self):at_put_(aGlobalName, (aBlock):value()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"removeKey:ifAbsent:",0
,(
function(self,aKey,aBlock)
return (((self):current()):removeKey_ifAbsent_(aKey, aBlock))
;
end
)
,"removing","removeKey: aKey ifAbsent: aBlock\r\n\t\" Remove the binding at aKey. \"\r\n\r\n\t^self current removeKey: aKey ifAbsent: aBlock","(\r\nfunction(self,aKey,aBlock)\r\nreturn (((self):current()):removeKey_ifAbsent_(aKey, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"@","_at"
,(
function(self,aGlobalName)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local binding=nil
(function()
	local _s8_value=((self):at_ifAbsent_(aGlobalName, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Array):new())
end
)()
error(_s8Ret_,0)
end)())
end)))
	binding=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Array):with_(binding))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","@ aGlobalName\r\n\t\" Return a collection with the objects bound to aGlobalName (or an empty collection if global is not defined).\r\n\tIt is used in expressions like:\r\n\t\tSmalltalk @ #MyGlobal do: [:global| ... ]\r\n\t\"\r\n\r\n\t| binding |\r\n\tbinding := self at: aGlobalName ifAbsent: [ ^Array new ].\r\n\t^Array with: binding","(\r\nfunction(self,aGlobalName)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal binding=nil\r\n(function()\r\n\tlocal _s8_value=((self):at_ifAbsent_(aGlobalName, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Array):new())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)))\r\n\tbinding=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Array):with_(binding))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"definePool:with:",0
,(
function(self,poolName,tuples)
return (((self):current()):definePool_with_(poolName, tuples))
;
end
)
,"accessing","definePool: poolName with: tuples\r\n\t\" Define the pool dictionary. \"\r\n\r\n\t^self current definePool: poolName with: tuples","(\r\nfunction(self,poolName,tuples)\r\nreturn (((self):current()):definePool_with_(poolName, tuples))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"poolDictionary:ifAbsent:",0
,(
function(self,aPoolName,aBlock)
return (((self):current()):poolDictionary_ifAbsent_(aPoolName, aBlock))
;
end
)
,"accessing","poolDictionary: aPoolName ifAbsent: aBlock\r\n\t\" Return the pool dictionary named aPoolName or the result of evaluating aBlock. \"\r\n\r\n\t^self current poolDictionary: aPoolName ifAbsent: aBlock","(\r\nfunction(self,aPoolName,aBlock)\r\nreturn (((self):current()):poolDictionary_ifAbsent_(aPoolName, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"objectAt:ifAbsent:",0
,(
function(self,aName,aBlock)
return (((self):current()):objectAt_ifAbsent_(aName, aBlock))
;
end
)
,"accessing","objectAt: aName ifAbsent: aBlock\r\n\t\" Return the (global scoped) smalltalk object at aName or the result of evaluating aBlock (if undefined or not an Object instance). \"\r\n\r\n\t^self current objectAt: aName ifAbsent: aBlock","(\r\nfunction(self,aName,aBlock)\r\nreturn (((self):current()):objectAt_ifAbsent_(aName, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"jsObjectAt:ifAbsent:",0
,(
function(self,aName,aBlock)
return (((self):current()):jsObjectAt_ifAbsent_(aName, aBlock))
;
end
)
,"accessing","jsObjectAt: aName ifAbsent: aBlock\r\n\t\" Return the (global) object at aName or the result of evaluating aBlock (if undefined). \"\r\n\r\n\t^self current jsObjectAt: aName ifAbsent: aBlock","(\r\nfunction(self,aName,aBlock)\r\nreturn (((self):current()):jsObjectAt_ifAbsent_(aName, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"jsObjectAt:",0
,(
function(self,aName)
return ((self):jsObjectAt_ifAbsent_(aName, (function()
return ((self):error_(smalltalk:object_comma_(("Undefined "),aName)))
end)))
;
end
)
,"accessing","jsObjectAt: aName\r\n\t\" Return the (global) object at aName. \"\r\n\r\n\t^self jsObjectAt: aName ifAbsent: [ self error: 'Undefined ',aName ]","(\r\nfunction(self,aName)\r\nreturn ((self):jsObjectAt_ifAbsent_(aName, (function()\r\nreturn ((self):error_(smalltalk:object_comma_((\"Undefined \"),aName)))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"global",0
,(
function(self)
return (_G)
;
end
)
,"accessing","global\r\n\t\" Return the global context object. \"\r\n\r\n\t^{'(function(){return this;}).call(null)'\r\n\tlua'_G'}","(\r\nfunction(self)\r\nreturn (_G)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"classes",0
,(
function(self)
return (((self):current()):classes())
;
end
)
,"accessing","classes\r\n\t\" Return the classes defined in current instance. \"\r\n\r\n\t^self current classes","(\r\nfunction(self)\r\nreturn (((self):current()):classes())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"rootClasses",0
,(
function(self)
return (((self):current()):rootClasses())
;
end
)
,"accessing","rootClasses\r\n\t\" Return the collection of topLevel classes in the receiver. \"\r\n\r\n\t^self current rootClasses","(\r\nfunction(self)\r\nreturn (((self):current()):rootClasses())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"implementorsOf:",0
,(
function(self,aSymbol)
return (((self):current()):implementorsOf_(aSymbol))
;
end
)
,"tools","implementorsOf: aSymbol\r\n\t\" Return the implementors of aSymbol. \"\r\n\r\n\t^self current implementorsOf: aSymbol","(\r\nfunction(self,aSymbol)\r\nreturn (((self):current()):implementorsOf_(aSymbol))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"isRunningOn:",0
,(
function(self,lang)
return ((smalltalk:isObject_equalTo_((lang),(self):nativeLanguage())))
;
end
)
,"environment","isRunningOn: lang\r\n\t\" Private - Returns true when running hosted by native lang execution environment. \"\r\n\r\n\t^lang = self nativeLanguage","(\r\nfunction(self,lang)\r\nreturn ((smalltalk:isObject_equalTo_((lang),(self):nativeLanguage())))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"gc",0
,(
function(self)
	if collectgarbage() then return true end
;
return (self)
;
end
)
,"tools","gc\r\n\t\" Collect garbage. \"\r\n\r\n{'\tif(typeof gc!=\"undefined\"){gc(); return true}\r\n\treturn false\r\n'lua'\tif collectgarbage() then return true end'}","(\r\nfunction(self)\r\n\tif collectgarbage() then return true end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"allMethodsDo:",0
,(
function(self,aBlock)
((self):classes()):do_((function(each)
(each):methodsDo_(aBlock);return (((each)["$klass"]):methodsDo_(aBlock))
end))
;
return (self)
;
end
)
,"iterating","allMethodsDo: aBlock\r\n\t\" Evaluates aBlock for all the methods in the current instance. \"\r\n\r\n\tself classes do: [:each|\r\n\t\teach methodsDo: aBlock.\r\n\t\teach class methodsDo: aBlock.\r\n\t]","(\r\nfunction(self,aBlock)\r\n((self):classes()):do_((function(each)\r\n(each):methodsDo_(aBlock);return (((each)[\"$klass\"]):methodsDo_(aBlock))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"allSelectorsDo:",0
,(
function(self,aBlock)
local found=nil
(function()
	local _s8_value=((smalltalk.HashedSet):new())
	found=_s8_value; return _s8_value
end)()
;
(self):allMethodsDo_((function(each)
local selector=nil
(function()
	local _s8_value=((each):selector())
	selector=_s8_value; return _s8_value
end)();return ((function() if ((((found):includes_(selector)))==true) then return nil else return ((function()
(aBlock):evaluateWith_(selector);return ((found):add_(selector))
end))() end end)())
end))
;
return (found)
;
end
)
,"iterating","allSelectorsDo: aBlock\r\n\t\" Evaluates aBlock for all the selectors defined in the current instance. \"\r\n\r\n\t| found |\r\n\tfound := HashedSet new.\r\n\tself allMethodsDo: [:each| | selector |\r\n\t\tselector := each selector.\r\n\t\t(found includes: selector) ifFalse: [\r\n\t\t\taBlock evaluateWith: selector.\r\n\t\t\tfound add: selector.\r\n\t\t]\r\n\t].\r\n\t^found","(\r\nfunction(self,aBlock)\r\nlocal found=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.HashedSet):new())\r\n\tfound=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):allMethodsDo_((function(each)\r\nlocal selector=nil\r\n(function()\r\n\tlocal _s8_value=((each):selector())\r\n\tselector=_s8_value; return _s8_value\r\nend)();return ((function() if ((((found):includes_(selector)))==true) then return nil else return ((function()\r\n(aBlock):evaluateWith_(selector);return ((found):add_(selector))\r\nend))() end end)())\r\nend))\r\n;\r\nreturn (found)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"allSelectors",0
,(
function(self)
return (((self):allSelectorsDo_(nil)):sorted())
;
end
)
,"iterating","allSelectors\r\n\t\" Return all the selectors (sorted) defined in the current instance. \"\r\n\r\n\t^(self allSelectorsDo: nil) sorted","(\r\nfunction(self)\r\nreturn (((self):allSelectorsDo_(nil)):sorted())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"bindingsFor:",0
,(
function(self,aScope)
return (((smalltalk.PoolDictionaryReader):new()):scope_(aScope))
;
end
)
,"Tools-pool variables","bindingsFor: aScope\r\n\t\" FileIn - Return a reader to fileIn bindings in a (global) scope.\r\n\taScope can be a string or a dictionary.\r\n\tIn case of aScope been a String, it will be considered a PoolDictionaryName\r\n\tSee PoolDictionaryReader>>#scanFrom:\r\n\t\"\r\n\r\n\t^PoolDictionaryReader new scope: aScope","(\r\nfunction(self,aScope)\r\nreturn (((smalltalk.PoolDictionaryReader):new()):scope_(aScope))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"sendersOf:",0
,(
function(self,selector)
return (((self):current()):sendersOf_(selector))
;
end
)
,"Tools-query","sendersOf: selector\r\n\t\" Return the senders of message selector. \"\r\n\r\n\t^self current sendersOf: selector","(\r\nfunction(self,selector)\r\nreturn (((self):current()):sendersOf_(selector))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"saveImageAs:",0
,(
function(self,fileName)
((self):current()):saveImageAs_(fileName)
;
return (self)
;
end
)
,"Snapshot-snapshot","saveImageAs: fileName\r\n\t\" Dump the image contents onto file at fileName. \"\r\n\r\n\tself current saveImageAs: fileName","(\r\nfunction(self,fileName)\r\n((self):current()):saveImageAs_(fileName)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"saveImage",0
,(
function(self)
((self):current()):saveImage()
;
return (self)
;
end
)
,"Snapshot-snapshot","saveImage\r\n\t\" Dump the image contents onto image file. \"\r\n\r\n\tself current saveImage","(\r\nfunction(self)\r\n((self):current()):saveImage()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk["$klass"],"exit",0
,(
function(self)
((self):current()):exit()
;
return (self)
;
end
)
,"Snapshot-exiting","exit\r\n\t\" Exit to OS system. \"\r\n\r\n\t\r\n\tself current exit","(\r\nfunction(self)\r\n((self):current()):exit()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"globalAt:",0
,(
function(self,aKey)
return (((self)[aKey]))
;
end
)
,"accessing","globalAt: aKey\r\n\t\" Return the global at aKey (or nil). \"\r\n\r\n\t^self basicAt: aKey","(\r\nfunction(self,aKey)\r\nreturn (((self)[aKey]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"globalAt:ifAbsent:",0
,(
function(self,aKey,aBlock)
return ((function() local _1_=((self)[aKey]);if (_1_==nil) then return (aBlock):value() end; return _1_ end)())
;
end
)
,"accessing","globalAt: aKey ifAbsent: aBlock\r\n\t\" Return the global at aKey (or the result of evaluating aBlock). \"\r\n\r\n\t^self basicAt: aKey ifAbsent: aBlock","(\r\nfunction(self,aKey,aBlock)\r\nreturn ((function() local _1_=((self)[aKey]);if (_1_==nil) then return (aBlock):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"rootClasses",0
,(
function(self)
return (((self):classes()):select_((function(each)
return (((((each):superclass()))==nil))
end)))
;
end
)
,"accessing","rootClasses\r\n\t\" Return the collection of topLevel classes in the receiver. \"\r\n\r\n\t^self classes select: [:each| each superclass isNil ]","(\r\nfunction(self)\r\nreturn (((self):classes()):select_((function(each)\r\nreturn (((((each):superclass()))==nil))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"implementorsOf:",0
,(
function(self,aSymbol)
local result=nil
(function()
	local _s8_value=(smalltalk:newArray({}))
	result=_s8_value; return _s8_value
end)()
;
((self):rootClasses()):do_((function(each)
return ((result):addAll_((each):implementorsOf_(aSymbol)))
end))
;
return (result)
;
end
)
,"testing","implementorsOf: aSymbol\r\n\t\" Return the implementors of aSymbol. \"\r\n\r\n\t| result |\r\n\tresult := #().\r\n\tself rootClasses do: [:each|\r\n\t\tresult addAll: (each implementorsOf: aSymbol)\r\n\t].\r\n\t^result","(\r\nfunction(self,aSymbol)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((self):rootClasses()):do_((function(each)\r\nreturn ((result):addAll_((each):implementorsOf_(aSymbol)))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"objectAt:ifAbsent:",0
,(
function(self,aName,aBlock)
local result=nil
(function()
	local _s8_value=((self):jsObjectAt_ifAbsent_(aName, aBlock))
	result=_s8_value; return _s8_value
end)()
;
return ((function() if ((((((result["yourself"]))~=nil)))==true) then return ((function()
return (result)
end))() else return nil end end)())
;
end
)
,"accessing","objectAt: aName ifAbsent: aBlock\r\n\t\" Return the (global scoped) smalltalk object at aName or the result of evaluating aBlock (if undefined or not an Object instance). \"\r\n\r\n\t| result |\r\n\tresult := self jsObjectAt: aName ifAbsent: aBlock.\r\n\t^result#yourself notNil ifTrue: [ result ]","(\r\nfunction(self,aName,aBlock)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):jsObjectAt_ifAbsent_(aName, aBlock))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((function() if ((((((result[\"yourself\"]))~=nil)))==true) then return ((function()\r\nreturn (result)\r\nend))() else return nil end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"jsObjectAt:ifAbsent:",0
,(
function(self,aName,aBlock)
return ((function() local _1_=(((smalltalk.Smalltalk):global())[aName]);if (_1_==nil) then return (aBlock):value() end; return _1_ end)())
;
end
)
,"accessing","jsObjectAt: aName ifAbsent: aBlock\r\n\t\" Return the (global) object at aName or the result of evaluating aBlock (if undefined). \"\r\n\r\n\t^Smalltalk global basicAt: aName ifAbsent: aBlock","(\r\nfunction(self,aName,aBlock)\r\nreturn ((function() local _1_=(((smalltalk.Smalltalk):global())[aName]);if (_1_==nil) then return (aBlock):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"jsObjectAt:",0
,(
function(self,aName)
return ((self):jsObjectAt_ifAbsent_(aName, (function()
return ((self):error_(smalltalk:object_comma_(("Undefined "),aName)))
end)))
;
end
)
,"accessing","jsObjectAt: aName\r\n\t\" Return the (global) object at aName. \"\r\n\r\n\t^self jsObjectAt: aName ifAbsent: [ self error: 'Undefined ',aName ]","(\r\nfunction(self,aName)\r\nreturn ((self):jsObjectAt_ifAbsent_(aName, (function()\r\nreturn ((self):error_(smalltalk:object_comma_((\"Undefined \"),aName)))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"removeKey:ifAbsent:",0
,(
function(self,aKey,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() local _1_=((self)[aKey]);if (_1_==nil) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):value())
end
)()
error(_s8Ret_,0)
end)())
end)):value() end; return _1_ end)()
;
(self):basicDelete_(aKey)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"removing","removeKey: aKey ifAbsent: aBlock\r\n\t\" Remove the binding at aKey. \"\r\n\r\n\tself basicAt: aKey ifAbsent: [ ^aBlock value ].\r\n\tself basicDelete: aKey","(\r\nfunction(self,aKey,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() local _1_=((self)[aKey]);if (_1_==nil) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)):value() end; return _1_ end)()\r\n;\r\n(self):basicDelete_(aKey)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"definePool:",0
,(
function(self,poolName)
return ((self):definePool_with_(poolName, (smalltalk.Array):new()))
;
end
)
,"accessing","definePool: poolName\r\n\t\" Define the pool dictionary (if it is not defined yet). \"\r\n\r\n\t^self definePool: poolName with: Array new","(\r\nfunction(self,poolName)\r\nreturn ((self):definePool_with_(poolName, (smalltalk.Array):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"poolDictionary:ifAbsent:",0
,(
function(self,aPoolName,aBlock)
return ((function() local _1_=((self)[aPoolName]);if (_1_==nil) then return ((function()
return ((aBlock):value())
end)):value() end; return _1_ end)())
;
end
)
,"accessing","poolDictionary: aPoolName ifAbsent: aBlock\r\n\t\" Return the pool dictionary named aPoolName or the result of evaluating aBlock. \"\r\n\r\n\t^self basicAt: aPoolName ifAbsent: [ aBlock value ]","(\r\nfunction(self,aPoolName,aBlock)\r\nreturn ((function() local _1_=((self)[aPoolName]);if (_1_==nil) then return ((function()\r\nreturn ((aBlock):value())\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"poolDictionary:",0
,(
function(self,aPoolName)
return ((self):poolDictionary_ifAbsent_(aPoolName, (function()
return ((self):definePool_(aPoolName))
end)))
;
end
)
,"accessing","poolDictionary: aPoolName\r\n\t\" Return the pool dictionary named aPoolName. \"\r\n\r\n\t^self poolDictionary: aPoolName ifAbsent: [\r\n\t\t\"self error: 'Missing Pool dictionary: ',aPoolName. \"\r\n\t\tself definePool: aPoolName\r\n\t]","(\r\nfunction(self,aPoolName)\r\nreturn ((self):poolDictionary_ifAbsent_(aPoolName, (function()\r\nreturn ((self):definePool_(aPoolName))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"printDefinePool:with:",0
,(
function(self,poolName,tuples)
(self):print_(smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk.definePool_with_(\""),poolName)),"\",["))
;
(tuples):do_((function(tuple)
return ((function() if ((((tuple):isArray()))==true) then return ((function()
return ((self):print_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("\t["),smalltalk:printString_(((tuple):first())))),",")),((tuple):last()):asNative())),"],")))
end))() else return ((function()
return ((self):print_(smalltalk:object_comma_((smalltalk:object_comma_(("\t"),(tuple):asNative())),",")))
end))() end end)())
end))
;
(self):print_("\t]);")
;
return (self)
;
end
)
,"compiling","printDefinePool: poolName with: tuples\r\n\t\" DEBUG - Dump definition of the pool dictionary. \"\r\n\r\n\tself print: 'smalltalk.definePool_with_(\"',poolName,'\",['.\r\n\ttuples do: [:tuple|\r\n\t\ttuple isArray ifTrue: [\r\n\t\t\tself print: '\t[',tuple first printString,',',tuple last asNative,'],'\r\n\t\t] ifFalse: [\r\n\t\t\tself print: '\t',tuple asNative ,','\r\n\t\t]\r\n\t].\r\n\tself print: '\t]);'.","(\r\nfunction(self,poolName,tuples)\r\n(self):print_(smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.definePool_with_(\\\"\"),poolName)),\"\\\",[\"))\r\n;\r\n(tuples):do_((function(tuple)\r\nreturn ((function() if ((((tuple):isArray()))==true) then return ((function()\r\nreturn ((self):print_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"\\t[\"),smalltalk:printString_(((tuple):first())))),\",\")),((tuple):last()):asNative())),\"],\")))\r\nend))() else return ((function()\r\nreturn ((self):print_(smalltalk:object_comma_((smalltalk:object_comma_((\"\\t\"),(tuple):asNative())),\",\")))\r\nend))() end end)())\r\nend))\r\n;\r\n(self):print_(\"\\t]);\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"definePool:with:",0
,(
function(self,poolName,tuples)
local dictionary=nil
local offset=nil
(function()
	local _s8_value=((function() local _1_=((self)[poolName]);if (_1_==nil) then return ((function()
return ((function() local _1_=((smalltalk.PoolDictionary):new()); (self)[poolName]=_1_; return _1_ end)())
end)):value() end; return _1_ end)())
	dictionary=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((0))
	offset=_s8_value; return _s8_value
end)()
;
(tuples):do_((function(each)
return ((function() if ((((each):isArray()))==true) then return ((function()
(dictionary):at_put_((each):first(), (each):last());return ((function() if (((((each):last()):isNumber()))==true) then return ((function()
return ((function()
	local _s8_value=(((each):last()):_plus((1)))
	offset=_s8_value; return _s8_value
end)())
end))() else return nil end end)())
end))() else return ((function()
return ((function() if ((((each):isNumber()))==true) then return ((function()
return ((function()
	local _s8_value=(each)
	offset=_s8_value; return _s8_value
end)())
end))() else return ((function()
(dictionary):at_put_(each, offset);return ((function()
	local _s8_value=((offset):_plus((1)))
	offset=_s8_value; return _s8_value
end)())
end))() end end)())
end))() end end)())
end))
;
return (dictionary)
;
end
)
,"compiling","definePool: poolName with: tuples\r\n\t\" Define the pool dictionary. \"\r\n\r\n\t| dictionary offset |\r\n\tdictionary := self basicAt: poolName ifAbsent: [\r\n\t\tself basicAt: poolName put: PoolDictionary new\r\n\t].\r\n\toffset := 0.\r\n\ttuples do: [:each|\r\n\t\teach isArray ifTrue: [\r\n\t\t\tdictionary at: each first put: each last.\r\n\t\t\teach last isNumber ifTrue: [ offset := each last + 1 ].\r\n\t\t] ifFalse: [\r\n\t\t\teach isNumber ifTrue: [ offset := each ]\r\n\t\t\tifFalse: [\r\n\t\t\t\tdictionary at: each put: offset.\r\n\t\t\t\toffset := offset + 1.\r\n\t\t\t]\r\n\t\t]\r\n\t].\r\n\t^dictionary","(\r\nfunction(self,poolName,tuples)\r\nlocal dictionary=nil\r\nlocal offset=nil\r\n(function()\r\n\tlocal _s8_value=((function() local _1_=((self)[poolName]);if (_1_==nil) then return ((function()\r\nreturn ((function() local _1_=((smalltalk.PoolDictionary):new()); (self)[poolName]=_1_; return _1_ end)())\r\nend)):value() end; return _1_ end)())\r\n\tdictionary=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((0))\r\n\toffset=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(tuples):do_((function(each)\r\nreturn ((function() if ((((each):isArray()))==true) then return ((function()\r\n(dictionary):at_put_((each):first(), (each):last());return ((function() if (((((each):last()):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(((each):last()):_plus((1)))\r\n\toffset=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return ((function()\r\nreturn ((function() if ((((each):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(each)\r\n\toffset=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\n(dictionary):at_put_(each, offset);return ((function()\r\n\tlocal _s8_value=((offset):_plus((1)))\r\n\toffset=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)())\r\nend))() end end)())\r\nend))\r\n;\r\nreturn (dictionary)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"state",0
,(
function(self)
return (((self)["$state"]))
;
end
)
,"accessing","state\r\n\t\" Return the state of the receiver (or nil). \"\r\n\r\n\t^self basicAt: '$state'","(\r\nfunction(self)\r\nreturn (((self)[\"$state\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"state:",0
,(
function(self,aState)
(function() local _1_=(aState); (self)["$state"]=_1_; return _1_ end)()
;
return (self)
;
end
)
,"accessing","state: aState\r\n\t\" Set the state of the receiver. \"\r\n\r\n\tself basicAt: '$state' put: aState","(\r\nfunction(self,aState)\r\n(function() local _1_=(aState); (self)[\"$state\"]=_1_; return _1_ end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"withState:do:",0
,(
function(self,aState,aBlock)
local old=nil
(function()
	local _s8_value=((self):state())
	old=_s8_value; return _s8_value
end)()
;
return (((function()
(self):state_(aState);return ((aBlock):value())
end)):ensure_((function()
return ((self):state_(old))
end)))
;
end
)
,"accessing","withState: aState do: aBlock\r\n\t\" Set the state of the receiver to aState during execution of aBlock. \"\r\n\r\n\t| old |\r\n\told := self state.\r\n\t^[\tself state: aState.\r\n\t\taBlock value\r\n\t] ensure: [\r\n\t\tself state: old\r\n\t]","(\r\nfunction(self,aState,aBlock)\r\nlocal old=nil\r\n(function()\r\n\tlocal _s8_value=((self):state())\r\n\told=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (((function()\r\n(self):state_(aState);return ((aBlock):value())\r\nend)):ensure_((function()\r\nreturn ((self):state_(old))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"eval:",0
,(
function(self,anExpression)
	local fn,err  = load(anExpression)
	if fn then return fn() else
		self:print_("-- eval: Error "..err) --the expression must be a lua expression (including return)
		return nil
	end
;
return (self)
;
end
)
,"eval","eval: anExpression\r\n\t\" Return the result of evaluating a native expression in global context. \"\r\n\r\n{'\treturn eval(anExpression)\r\n'lua'\tlocal fn,err  = load(anExpression)\r\n\tif fn then return fn() else\r\n\t\tself:print_(\"-- eval: Error \"..err) --the expression must be a lua expression (including return)\r\n\t\treturn nil\r\n\tend'}","(\r\nfunction(self,anExpression)\r\n\tlocal fn,err  = load(anExpression)\r\n\tif fn then return fn() else\r\n\t\tself:print_(\"-- eval: Error \"..err) --the expression must be a lua expression (including return)\r\n\t\treturn nil\r\n\tend\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"eval:as:",0
,(
function(self,anExpression,aState)
return ((self):withState_do_(aState, (function()
return ((self):eval_(anExpression))
end)))
;
end
)
,"eval","eval: anExpression as: aState\r\n\t\" Return the result of evaluating a native expression in global context.\r\n\tThe state of the system change to state while evaluating the expression.\r\n\t\"\r\n\r\n\t^self withState: aState do: [ self eval: anExpression ]","(\r\nfunction(self,anExpression,aState)\r\nreturn ((self):withState_do_(aState, (function()\r\nreturn ((self):eval_(anExpression))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"capture:with:",0
,(
function(self,aSymbol,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((aBlock))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):capture_with_(aSymbol, (function(result)
return (result)
end)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((((aSymbol))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):evaluateWith_(nil))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((aSymbol):startsWith_("!")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):evaluateWith_(((aSymbol):copyFrom_to_((2), (aSymbol):size())):doIt()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if (((smalltalk:responds_to_((self),(aSymbol))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):evaluateWith_(nil))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((aSymbol):argumentCount()),(1)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):perform_with_(aSymbol, aBlock))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):evaluateWith_((self):perform_(aSymbol)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"tools","capture: aSymbol with: aBlock\r\n\t\" Capture aSymbol of the receiver and evaluate aBlock.\r\n\tDefault action will try to capture aSymbol and (potentially) evaluate aBlock with the result. \"\r\n\r\n\taBlock isNil ifTrue: [ ^self capture: aSymbol with: [:result | result ] ].\r\n\taSymbol isNil ifTrue: [ ^aBlock evaluateWith: nil ].\r\n\t(aSymbol startsWith: $!) ifTrue: [\r\n\t\t^aBlock evaluateWith: (aSymbol copyFrom: 2 to: aSymbol size) doIt\r\n\t].\r\n\t(self respondsTo: aSymbol) ifFalse: [ ^aBlock evaluateWith: nil ].\r\n\taSymbol argumentCount = 1 ifTrue: [ ^self perform: aSymbol with: aBlock ].\r\n\t^aBlock evaluateWith: (self perform: aSymbol)","(\r\nfunction(self,aSymbol,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((aBlock))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):capture_with_(aSymbol, (function(result)\r\nreturn (result)\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((aSymbol))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):evaluateWith_(nil))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((aSymbol):startsWith_(\"!\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):evaluateWith_(((aSymbol):copyFrom_to_((2), (aSymbol):size())):doIt()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if (((smalltalk:responds_to_((self),(aSymbol))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):evaluateWith_(nil))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((aSymbol):argumentCount()),(1)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):perform_with_(aSymbol, aBlock))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):evaluateWith_((self):perform_(aSymbol)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"removeSources",0
,(
function(self)
((self):classes()):do_((function(each)
return ((each):removeSources())
end))
;
return (self)
;
end
)
,"tools","removeSources\r\n\t\" Remove all development sources of the receiver. \"\r\n\r\n\tself classes do: [:each | each removeSources ]","(\r\nfunction(self)\r\n((self):classes()):do_((function(each)\r\nreturn ((each):removeSources())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"readJSON:",0
,(
function(self,anObject)
return ((self):readJSObject(anObject))
;
end
)
,"json","readJSON: anObject\r\n\t\" Return an object read from JSON map object anObject. \"\r\n\r\n\t^self #readJSObject: anObject","(\r\nfunction(self,anObject)\r\nreturn ((self):readJSObject(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"shallowCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"deepCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"recompileAll",0
,(
function(self)
((self):classes()):do_((function(each)
return ((each):recompile())
end))
;
return (self)
;
end
)
,"Compiler-compiling","recompileAll\r\n\t\" Recompile all classes in the receiver. \"\r\n\r\n\tself classes do: [:each | each recompile ]","(\r\nfunction(self)\r\n((self):classes()):do_((function(each)\r\nreturn ((each):recompile())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"compilerSupport",0
,(
function(self)
return (smalltalk.Compiler)
;
end
)
,"Compiler-accessing","compilerSupport\r\n\t\" Return the compiler support for the receiver. \"\r\n\r\n\t^Compiler","(\r\nfunction(self)\r\nreturn (smalltalk.Compiler)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"fileInCode:",0
,(
function(self,code)
return (((code):stream()):fileIn())
;
end
)
,"Tools-fileIn","fileInCode: code\r\n\t\" FileIn the code into the receiver. \"\r\n\r\n\t^code stream fileIn","(\r\nfunction(self,code)\r\nreturn (((code):stream()):fileIn())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"methodReferences",0
,(
function(self)
return ((smalltalk.MethodReferences):new())
;
end
)
,"Tools-accessing","methodReferences\r\n\t\" Return the methodReferences tool of the receiver. \"\r\n\r\n\t^MethodReferences new","(\r\nfunction(self)\r\nreturn ((smalltalk.MethodReferences):new())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"sendersOf:",0
,(
function(self,selector)
return (((self):methodReferences()):sendersOf_(selector))
;
end
)
,"Tools-query","sendersOf: selector\r\n\t\" Return the senders of message selector. \"\r\n\r\n\t^self methodReferences sendersOf: selector","(\r\nfunction(self,selector)\r\nreturn (((self):methodReferences()):sendersOf_(selector))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"exit",0
,(
function(self)
((self):classes()):do_((function(each)
return ((each):aboutToExit_(self))
end))
;
quit()
;
return (self)
;
end
)
,"Snapshot-exiting","exit\r\n\t\" Exit to OS system. \"\r\n\r\n\tself classes do: [:each| each aboutToExit: self ].\r\n\t# quit","(\r\nfunction(self)\r\n((self):classes()):do_((function(each)\r\nreturn ((each):aboutToExit_(self))\r\nend))\r\n;\r\nquit()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"product",0
,(
function(self)
return ("S8")
;
end
)
,"Snapshot-accessing","product\r\n\t\" Private - Return the name of the system. \"\r\n\r\n\t^#S8","(\r\nfunction(self)\r\nreturn (\"S8\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"description",0
,(
function(self)
return ("A modern Smalltalk environment running on the \"invisible machine\".")
;
end
)
,"Snapshot-accessing","description\r\n\t\" Return the description of the receiver. \"\r\n\r\n\t^'A modern Smalltalk environment running on the \"invisible machine\".'","(\r\nfunction(self)\r\nreturn (\"A modern Smalltalk environment running on the \\\"invisible machine\\\".\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"licenseComment",0
,(
function(self)
return ("\r\nThe text you are reading here is the residue of something that has run before.\r\nKeep the system running with responsibility. Smalltalk is not the words,\r\n nor any definition, nor it current contents...\r\nSmalltalk include your feelings when you see it running.\r\nSmalltalk is what happens when time flows and you \"are\" here.\r\nStop reading/writing code (thinking you are outside the system),\r\n change your world with resposibility, one object at a time, with your hands,\r\n during all your life, in family.\r\n")
;
end
)
,"Snapshot-license","licenseComment\r\n\t\" Private - Comment note about license of this product. \"\r\n\r\n\t^'\r\nThe text you are reading here is the residue of something that has run before.\r\nKeep the system running with responsibility. Smalltalk is not the words,\r\n nor any definition, nor it current contents...\r\nSmalltalk include your feelings when you see it running.\r\nSmalltalk is what happens when time flows and you \"are\" here.\r\nStop reading/writing code (thinking you are outside the system),\r\n change your world with resposibility, one object at a time, with your hands,\r\n during all your life, in family.\r\n'","(\r\nfunction(self)\r\nreturn (\"\\r\\nThe text you are reading here is the residue of something that has run before.\\r\\nKeep the system running with responsibility. Smalltalk is not the words,\\r\\n nor any definition, nor it current contents...\\r\\nSmalltalk include your feelings when you see it running.\\r\\nSmalltalk is what happens when time flows and you \\\"are\\\" here.\\r\\nStop reading/writing code (thinking you are outside the system),\\r\\n change your world with resposibility, one object at a time, with your hands,\\r\\n during all your life, in family.\\r\\n\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"licenseTerms",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):product())," - ")),(self):description())),"\r\n")),(self):licenseMoreHolders())),"\r\n")),(self):licenseHolders())),"\r\nPermission is hereby granted, free of charge, to any person obtaining\r\n a copy of this software and associated documentation files (the \"Software\"),\r\n to deal in the Software without restriction, including without limitation\r\n the rights to use, copy, modify, merge, publish, distribute, sublicense,\r\n and/or sell copies of the Software, and to permit persons to whom the\r\n Software is furnished to do so, subject to the following conditions:\r\n\r\nThe above copyright notice and this permission notice shall be included\r\n in all copies or substantial portions of the Software.\r\n\r\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\r\n EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES\r\n OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.\r\n IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY\r\n CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,\r\n TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE\r\n OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\r\n\r\n")),(self):licenseComment()))
;
end
)
,"Snapshot-license","licenseTerms\r\n\t\" Private - Return the license terms to use this system. \"\r\n\r\n\t^self product ,' - ' ,self description ,'\r\n' ,self licenseMoreHolders ,'\r\n' ,self licenseHolders ,'\r\nPermission is hereby granted, free of charge, to any person obtaining\r\n a copy of this software and associated documentation files (the \"Software\"),\r\n to deal in the Software without restriction, including without limitation\r\n the rights to use, copy, modify, merge, publish, distribute, sublicense,\r\n and/or sell copies of the Software, and to permit persons to whom the\r\n Software is furnished to do so, subject to the following conditions:\r\n\r\nThe above copyright notice and this permission notice shall be included\r\n in all copies or substantial portions of the Software.\r\n\r\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\r\n EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES\r\n OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.\r\n IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY\r\n CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,\r\n TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE\r\n OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\r\n\r\n' ,self licenseComment","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):product()),\" - \")),(self):description())),\"\\r\\n\")),(self):licenseMoreHolders())),\"\\r\\n\")),(self):licenseHolders())),\"\\r\\nPermission is hereby granted, free of charge, to any person obtaining\\r\\n a copy of this software and associated documentation files (the \\\"Software\\\"),\\r\\n to deal in the Software without restriction, including without limitation\\r\\n the rights to use, copy, modify, merge, publish, distribute, sublicense,\\r\\n and/or sell copies of the Software, and to permit persons to whom the\\r\\n Software is furnished to do so, subject to the following conditions:\\r\\n\\r\\nThe above copyright notice and this permission notice shall be included\\r\\n in all copies or substantial portions of the Software.\\r\\n\\r\\nTHE SOFTWARE IS PROVIDED \\\"AS IS\\\", WITHOUT WARRANTY OF ANY KIND,\\r\\n EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES\\r\\n OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.\\r\\n IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY\\r\\n CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,\\r\\n TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE\\r\\n OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\\r\\n\\r\\n\")),(self):licenseComment()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"licenseYear",0
,(
function(self)
local year=nil
((function()
return ((function()
	local _s8_value=(smalltalk:toString_((((smalltalk.DateTime):today()):year())))
	year=_s8_value; return _s8_value
end)())
end)):on_do_(smalltalk.Error, (function()
return ((function()
	local _s8_value=("today")
	year=_s8_value; return _s8_value
end)())
end))
;
return (year)
;
end
)
,"Snapshot-license","licenseYear\r\n\t\" Private - Return the year license information. \"\r\n\r\n\t| year |\r\n\t[ year := DateTime today year toString\r\n\t] on: Error do: [ year := #today ].\r\n\t^year","(\r\nfunction(self)\r\nlocal year=nil\r\n((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:toString_((((smalltalk.DateTime):today()):year())))\r\n\tyear=_s8_value; return _s8_value\r\nend)())\r\nend)):on_do_(smalltalk.Error, (function()\r\nreturn ((function()\r\n\tlocal _s8_value=(\"today\")\r\n\tyear=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\nreturn (year)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"licenseMoreHolders",0
,(
function(self)
return ("")
;
end
)
,"Snapshot-license","licenseMoreHolders\r\n\t\" Private - Return additional license holders information. \"\r\n\r\n\t^''","(\r\nfunction(self)\r\nreturn (\"\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"licenseHolders",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("Copyright (C) "),smalltalk:asString_(((self):licenseYear()))))," ")),"Alejandro F. Reimondo <aleReimondo@smalltalking.net>\r\n\r\nParts of source code written for Jtalk (Sept 2011),\r\nCopyright (C) 2011 by Nicolas Petton <petton.nicolas@gmail.com>\r\nCopyright (C) 2011 Jtalk contributors\r\n\tDavid Graham <dgraham@iphouse.com>\r\n\tGoran Krampe <goran@krampe.se>\r\nAlso code and ideas from Clamato (http://clamato.net), written by Avi Byrant.\r\nThe PetitParser library, published by Lukas Renggli (http://lukas-renggli.ch) and released under the MIT license.\r\nAnd people that contributed to Smalltalk on diverse media years(decades) before this license holders claimed ownership.\r\n"))
;
end
)
,"Snapshot-license","licenseHolders\r\n\t\" Private - Return the license holders information. \"\r\n\r\n\t^'Copyright (C) '\r\n\t,self licenseYear asString ,' '\r\n\t,'Alejandro F. Reimondo <aleReimondo@smalltalking.net>\r\n\r\nParts of source code written for Jtalk (Sept 2011),\r\nCopyright (C) 2011 by Nicolas Petton <petton.nicolas@gmail.com>\r\nCopyright (C) 2011 Jtalk contributors\r\n\tDavid Graham <dgraham@iphouse.com>\r\n\tGoran Krampe <goran@krampe.se>\r\nAlso code and ideas from Clamato (http://clamato.net), written by Avi Byrant.\r\nThe PetitParser library, published by Lukas Renggli (http://lukas-renggli.ch) and released under the MIT license.\r\nAnd people that contributed to Smalltalk on diverse media years(decades) before this license holders claimed ownership.\r\n'","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"Copyright (C) \"),smalltalk:asString_(((self):licenseYear())))),\" \")),\"Alejandro F. Reimondo <aleReimondo@smalltalking.net>\\r\\n\\r\\nParts of source code written for Jtalk (Sept 2011),\\r\\nCopyright (C) 2011 by Nicolas Petton <petton.nicolas@gmail.com>\\r\\nCopyright (C) 2011 Jtalk contributors\\r\\n\\tDavid Graham <dgraham@iphouse.com>\\r\\n\\tGoran Krampe <goran@krampe.se>\\r\\nAlso code and ideas from Clamato (http://clamato.net), written by Avi Byrant.\\r\\nThe PetitParser library, published by Lukas Renggli (http://lukas-renggli.ch) and released under the MIT license.\\r\\nAnd people that contributed to Smalltalk on diverse media years(decades) before this license holders claimed ownership.\\r\\n\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"globalsDo:",0
,(
function(self,aBlock)
return ((self):globalsDo(aBlock))
;
end
)
,"Snapshot-evaluating","globalsDo: aBlock\r\n\t\" Private - Evaluates aBlock for each globals defined in the receiver. \"\r\n\r\n\t^self #globalsDo: aBlock","(\r\nfunction(self,aBlock)\r\nreturn ((self):globalsDo(aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"globalsDo:separatedBy:",0
,(
function(self,aBlock,anotherBlock)
local first=nil
(function()
	local _s8_value=(true)
	first=_s8_value; return _s8_value
end)()
;
return ((self):globalsDo((function(each)
(function() if (((first))==true) then return ((function()
return ((function()
	local _s8_value=(false)
	first=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((anotherBlock):value())
end))() end end)();return ((aBlock):value_(each))
end)))
;
end
)
,"Snapshot-evaluating","globalsDo: aBlock separatedBy: anotherBlock\r\n\t\" Private - Evaluates aBlock for each globals defined in the receiver. \"\r\n\r\n\t| first | first := true.\r\n\t^self #globalsDo: [:each|\r\n\t    first\r\n\t\tifTrue: [ first := false ]\r\n\t\tifFalse: [ anotherBlock value ].\r\n\t    aBlock value: each\r\n\t]","(\r\nfunction(self,aBlock,anotherBlock)\r\nlocal first=nil\r\n(function()\r\n\tlocal _s8_value=(true)\r\n\tfirst=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((self):globalsDo((function(each)\r\n(function() if (((first))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(false)\r\n\tfirst=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((anotherBlock):value())\r\nend))() end end)();return ((aBlock):value_(each))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"saveImageOn:",0
,(
function(self,stream)
local exporter=nil
local allClasses=nil
(
function(_s8_rec)
(_s8_rec):nextPutAll_(((self):licenseTerms()):asComment());
return ((_s8_rec):cr())
end
)(stream)
;
(function()
	local _s8_value=((((((self):classes()):select_((function(each)
return (((((each):superclass()))==nil))
end))):inject_into_(smalltalk:newArray({}), (function(total,each)
(total):addAll_((each):withAllSubclasses());return (total)
end))):collect_((function(each)
return ((each):aboutToSaveImage_on_(self, stream))
end))):select_((function(each)
return ((((each))~=nil))
end)))
	allClasses=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.Exporter):new())
	exporter=_s8_value; return _s8_value
end)()
;
(
function(_s8_rec)
(_s8_rec):cr();
(_s8_rec):nextPutAll_(("Smalltalk Classes."):asComment());
return ((_s8_rec):cr())
end
)(stream)
;
(exporter):exportClasses_on_(allClasses, stream)
;
(
function(_s8_rec)
(_s8_rec):cr();
return ((_s8_rec):nextPutAll_(("Smalltalk Globals set-up."):asComment()))
end
)(stream)
;
(self):globalsDo_((function(key)
return ((
function(_s8_rec)
(_s8_rec):cr();
return ((_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk."),key)),"=")),(exporter):exportObject_(((self)[key])))))
end
)(stream))
end))
;
(allClasses):do_((function(each)
return ((each):imageSaved_on_(self, stream))
end))
;
return (self)
;
end
)
,"Snapshot-writing","saveImageOn: stream\r\n\t\" Dump the receiver's contents onto stream. \"\r\n\r\n\t| exporter allClasses |\r\n\tstream nextPutAll: self licenseTerms asComment; cr.\r\n\tallClasses := (((self classes\r\n\t\tselect: [:each| each superclass isNil ])\r\n\t\tinject: #() into: [:total :each| total addAll: each withAllSubclasses. total ])\r\n\t\tcollect: [:each| each aboutToSaveImage: self on: stream ])\r\n\t\tselect: [:each| each notNil ].\r\n\texporter := Exporter new.\r\n\"\tstream nextPutAll: 'Smalltalk Globals (pre)definition.' asComment; cr.\r\n\tself\r\n\t\tglobalsDo: [:key| stream nextPutAll: 'smalltalk.',key ]\r\n\t\tseparatedBy: [ stream nextPutAll: ' , ' ].\r\n\tstream nextPutAll: '=nil'; cr.\r\n\"\tstream cr; nextPutAll: 'Smalltalk Classes.' asComment; cr.\r\n\texporter exportClasses: allClasses on: stream.\r\n\tstream cr; nextPutAll: 'Smalltalk Globals set-up.' asComment.\r\n\tself globalsDo: [:key|\r\n\t\tstream cr; nextPutAll: 'smalltalk.',key,'='\r\n\t\t\t,(exporter exportObject: (self basicAt: key))\r\n\t].\r\n\tallClasses do: [:each| each imageSaved: self on: stream ].","(\r\nfunction(self,stream)\r\nlocal exporter=nil\r\nlocal allClasses=nil\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(((self):licenseTerms()):asComment());\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream)\r\n;\r\n(function()\r\n\tlocal _s8_value=((((((self):classes()):select_((function(each)\r\nreturn (((((each):superclass()))==nil))\r\nend))):inject_into_(smalltalk:newArray({}), (function(total,each)\r\n(total):addAll_((each):withAllSubclasses());return (total)\r\nend))):collect_((function(each)\r\nreturn ((each):aboutToSaveImage_on_(self, stream))\r\nend))):select_((function(each)\r\nreturn ((((each))~=nil))\r\nend)))\r\n\tallClasses=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Exporter):new())\r\n\texporter=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\n(_s8_rec):nextPutAll_((\"Smalltalk Classes.\"):asComment());\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream)\r\n;\r\n(exporter):exportClasses_on_(allClasses, stream)\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\nreturn ((_s8_rec):nextPutAll_((\"Smalltalk Globals set-up.\"):asComment()))\r\nend\r\n)(stream)\r\n;\r\n(self):globalsDo_((function(key)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\nreturn ((_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.\"),key)),\"=\")),(exporter):exportObject_(((self)[key])))))\r\nend\r\n)(stream))\r\nend))\r\n;\r\n(allClasses):do_((function(each)\r\nreturn ((each):imageSaved_on_(self, stream))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"imageContents",0
,(
function(self)
local stream=nil
(function()
	local _s8_value=(((smalltalk.String):new()):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):saveImageOn_(stream)
;
return ((stream):contents())
;
end
)
,"Snapshot-contents","imageContents\r\n\t\" Return a dump of the system contents. \"\r\n\r\n\t| stream |\r\n\tstream := String new writeStream.\r\n\tself saveImageOn: stream.\r\n\t^stream contents","(\r\nfunction(self)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.String):new()):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):saveImageOn_(stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"snapshot",0
,(
function(self)
return ((smalltalk.Snapshot):contents())
;
end
)
,"Snapshot-contents","snapshot\r\n\t\" Return a system snapshot. \"\r\n\r\n\t^Snapshot contents","(\r\nfunction(self)\r\nreturn ((smalltalk.Snapshot):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"imageFileName",0
,(
function(self)
return (((self)["$imageFileName"]))
;
end
)
,"Snapshot-fileName","imageFileName\r\n\t\" Return the image file name. \"\r\n\r\n\t^self basicAt: '$imageFileName'","(\r\nfunction(self)\r\nreturn (((self)[\"$imageFileName\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"imageFileName:",0
,(
function(self,fileName)
return ((function() local _1_=(fileName); (self)["$imageFileName"]=_1_; return _1_ end)())
;
end
)
,"Snapshot-fileName","imageFileName: fileName\r\n\t\" Set the image file name. \"\r\n\r\n\t^self basicAt: '$imageFileName' put: fileName","(\r\nfunction(self,fileName)\r\nreturn ((function() local _1_=(fileName); (self)[\"$imageFileName\"]=_1_; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"saveImage",0
,(
function(self)
(self):saveImageAs_((self):imageFileName())
;
return (self)
;
end
)
,"Snapshot-save","saveImage\r\n\t\" Dump the image contents onto default image file. \"\r\n\r\n\tself saveImageAs: self imageFileName","(\r\nfunction(self)\r\n(self):saveImageAs_((self):imageFileName())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"saveImageAs:",0
,(
function(self,fileName)
((self):imageContents()):outputToFile_(fileName)
;
(self):imageFileName_(fileName)
;
return (self)
;
end
)
,"Snapshot-save","saveImageAs: fileName\r\n\t\" Dump the image contents onto file at fileName. \"\r\n\r\n\tself imageContents outputToFile: fileName.\r\n\tself imageFileName: fileName.","(\r\nfunction(self,fileName)\r\n((self):imageContents()):outputToFile_(fileName)\r\n;\r\n(self):imageFileName_(fileName)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"launch",0
,(
function(self)
((self):classes()):do_((function(each)
return ((each):launch())
end))
;
return (((smalltalk.Smalltalk):at_ifAbsent_("LaunchExpression", (function()
return (nil)
end))):evaluate())
;
end
)
,"Snapshot-startup","launch\r\n\t\" The system has been launched and start execution.\r\n\tThe default action is to propagate #launch to current classes and evaluate #LaunchExpression.\r\n\t\"\r\n\r\n\tself classes do: [:each| each launch ].\r\n\t^(Smalltalk at: #LaunchExpression ifAbsent: []) evaluate","(\r\nfunction(self)\r\n((self):classes()):do_((function(each)\r\nreturn ((each):launch())\r\nend))\r\n;\r\nreturn (((smalltalk.Smalltalk):at_ifAbsent_(\"LaunchExpression\", (function()\r\nreturn (nil)\r\nend))):evaluate())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"defaultWorkspaceList",0
,(
function(self)
local result=nil
(function()
	local _s8_value=(smalltalk:newArray({}))
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk.Smalltalk):includesKey_("WorkspaceList")))==true) then return ((function()
return ((result):addAll_((smalltalk.Smalltalk):at_("WorkspaceList")))
end))() else return nil end end)()
;
(function() if ((((smalltalk.Smalltalk):includesKey_("U8")))==true) then return ((function()
return ((result):addAll_((smalltalk.U8):commonExpressions()))
end))() else return nil end end)()
;
return (result)
;
end
)
,"platform-tools","defaultWorkspaceList\r\n\t\" Private - Returns the (captured) workspace list of the receiver.\r\n\tReturns an array of tuples; each tuple is anArray of the form\r\n\t\t#( title code )\r\n\twhere:\r\n\t\ttitle is aString to be shown as a title of the workspace page.\r\n\t\tcode is the text to be shown as contents of workspace page.\r\n\t\"\r\n\r\n\t| result |\r\n\tresult := #().\r\n\t(Smalltalk includesKey: #WorkspaceList) ifTrue: [\r\n\t\tresult addAll: (Smalltalk at: #WorkspaceList)\r\n\t].\r\n\t(Smalltalk includesKey: #U8) ifTrue: [\r\n\t\tresult addAll: U8 commonExpressions\r\n\t].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk.Smalltalk):includesKey_(\"WorkspaceList\")))==true) then return ((function()\r\nreturn ((result):addAll_((smalltalk.Smalltalk):at_(\"WorkspaceList\")))\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk.Smalltalk):includesKey_(\"U8\")))==true) then return ((function()\r\nreturn ((result):addAll_((smalltalk.U8):commonExpressions()))\r\nend))() else return nil end end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"workspaceList",0
,(
function(self)
return ((self):defaultWorkspaceList())
;
end
)
,"platform-tools","workspaceList\r\n\t\" Returns the (captured) workspace list of the receiver. \"\r\n\r\n\t^self defaultWorkspaceList","(\r\nfunction(self)\r\nreturn ((self):defaultWorkspaceList())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"defaultAppList",0
,(
function(self)
local result=nil
(function()
	local _s8_value=(smalltalk:newArray({}))
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk.Smalltalk):includesKey_("AppList")))==true) then return ((function()
return ((result):addAll_((smalltalk.Smalltalk):at_("AppList")))
end))() else return nil end end)()
;
(function() if ((((smalltalk.Smalltalk):includesKey_("U8Configuration")))==true) then return ((function()
return ((result):addAll_((smalltalk.U8Configuration):appList()))
end))() else return nil end end)()
;
(result):addAll_((self):allClassMethodsList_(smalltalk:newArray({"sample", "samples"})))
;
return (result)
;
end
)
,"platform-tools","defaultAppList\r\n\t\" Private - Returns the (captured) apps of the receiver.\r\n\tReturns an array of tuples; each tuple is anArray of the form\r\n\t\t#( title comment code )\r\n\twhere:\r\n\t\ttitle is aString that can include an icon name inside brackets.\r\n\t\tcomment is aString with short description of the app.\r\n\t\tcode is the code to evaluate to open the app.\r\n\t\"\r\n\r\n\t| result |\r\n\tresult := #().\r\n\t(Smalltalk includesKey: #AppList) ifTrue: [\r\n\t\tresult addAll: (Smalltalk at: #AppList)\r\n\t].\r\n\t(Smalltalk includesKey: #U8Configuration) ifTrue: [\r\n\t\tresult addAll: U8Configuration appList\r\n\t].\r\n\tresult addAll: (self allClassMethodsList: #(sample samples)).\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk.Smalltalk):includesKey_(\"AppList\")))==true) then return ((function()\r\nreturn ((result):addAll_((smalltalk.Smalltalk):at_(\"AppList\")))\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk.Smalltalk):includesKey_(\"U8Configuration\")))==true) then return ((function()\r\nreturn ((result):addAll_((smalltalk.U8Configuration):appList()))\r\nend))() else return nil end end)()\r\n;\r\n(result):addAll_((self):allClassMethodsList_(smalltalk:newArray({\"sample\", \"samples\"})))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"appList",0
,(
function(self)
return ((self):defaultAppList())
;
end
)
,"platform-tools","appList\r\n\t\" Returns the (captured) apps of the receiver. \"\r\n\r\n\t^self defaultAppList","(\r\nfunction(self)\r\nreturn ((self):defaultAppList())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"allClassMethodsList:",0
,(
function(self,choices)
local result=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	result=_s8_value; return _s8_value
end)()
;
((smalltalk.Smalltalk):classes()):do_((function(cls)
return (((cls)["$klass"]):methodsDo_((function(mth)
local comment=nil
return ((function() if ((((smalltalk:boolean_and_(((smalltalk:boolean_or_(((choices):includes_((mth):category())),(function()
return ((choices):includes_((mth):selector()))
end)))),(function()
(function()
	local _s8_value=((mth):comment())
	comment=_s8_value; return _s8_value
end)();return ((smalltalk:booleanNot_(((((smalltalk:asString_((comment))):upTo_("-")):asLowercase()):includes_("private")))))
end)))))==true) then return ((function()
local title=nil
(function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_(((cls):name())," ")),(mth):selector()))
	title=_s8_value; return _s8_value
end)();return ((result):add_((smalltalk.Array):with_with_with_(title, comment, smalltalk:object_comma_((smalltalk:object_comma_(((cls):name())," ")),(mth):selector()))))
end))() else return nil end end)())
end)))
end))
;
return ((result):sorted_((function(a,b)
return ((((a):first()) < (b):first()))
end)))
;
end
)
,"platform-tools","allClassMethodsList: choices\r\n\t\" Private - Returns the list of all (class) methods in the receiver matching choices. \"\r\n\r\n\t| result |\r\n\tresult := Array new.\r\n\tSmalltalk classes do: [:cls|\r\n\t\tcls class methodsDo: [:mth| | comment |\r\n\t\t\t(((choices includes: mth category) or: [\r\n\t\t\t  choices includes: mth selector\r\n\t\t\t]) and: [ comment := mth comment.\r\n\t\t\t ((comment asString upTo: $-) asLowercase includes: #private) not\r\n\t\t\t]) ifTrue: [ | title |\r\n\t\t\t\ttitle := cls name ,' ' ,mth selector.\r\n\t\t\t\tresult add: (Array\r\n\t\t\t\t\twith: title with: comment\r\n\t\t\t\t\twith: cls name,' ', mth selector)\r\n\t\t\t].\r\n\t\t].\r\n\t].\r\n\t^result sorted: [:a :b| a first < b first ]","(\r\nfunction(self,choices)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((smalltalk.Smalltalk):classes()):do_((function(cls)\r\nreturn (((cls)[\"$klass\"]):methodsDo_((function(mth)\r\nlocal comment=nil\r\nreturn ((function() if ((((smalltalk:boolean_and_(((smalltalk:boolean_or_(((choices):includes_((mth):category())),(function()\r\nreturn ((choices):includes_((mth):selector()))\r\nend)))),(function()\r\n(function()\r\n\tlocal _s8_value=((mth):comment())\r\n\tcomment=_s8_value; return _s8_value\r\nend)();return ((smalltalk:booleanNot_(((((smalltalk:asString_((comment))):upTo_(\"-\")):asLowercase()):includes_(\"private\")))))\r\nend)))))==true) then return ((function()\r\nlocal title=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_(((cls):name()),\" \")),(mth):selector()))\r\n\ttitle=_s8_value; return _s8_value\r\nend)();return ((result):add_((smalltalk.Array):with_with_with_(title, comment, smalltalk:object_comma_((smalltalk:object_comma_(((cls):name()),\" \")),(mth):selector()))))\r\nend))() else return nil end end)())\r\nend)))\r\nend))\r\n;\r\nreturn ((result):sorted_((function(a,b)\r\nreturn ((((a):first()) < (b):first()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"exampleList",0
,(
function(self)
return ((self):allClassMethodsList_(smalltalk:newArray({"example"})))
;
end
)
,"platform-tools","exampleList\r\n\t\" Returns the (captured) examples of the receiver. \"\r\n\r\n\t^self allClassMethodsList: #(example)","(\r\nfunction(self)\r\nreturn ((self):allClassMethodsList_(smalltalk:newArray({\"example\"})))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Smalltalk,"screen:",0
,(
function(self,aBlock)
return ("S8 system running in as console application")
;
end
)
,"platform-tools","screen: aBlock\r\n\t\" Returns the (captured) screen of the receiver. \"\r\n\r\n\t^'S8 system running in as console application'","(\r\nfunction(self,aBlock)\r\nreturn (\"S8 system running in as console application\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"new",0
,(
function(self)
return (((self):basicNew()):initialize())
;
end
)
,"instantiation","new\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self basicNew initialize","(\r\nfunction(self)\r\nreturn (((self):basicNew()):initialize())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"new:",0
,(
function(self,size)
return ((self):basicNew_(size))
;
end
)
,"instantiation","new: size\r\n\t\" Return an instance of the receiver of specified size. \"\r\n\r\n\t^self basicNew: size","(\r\nfunction(self,size)\r\nreturn ((self):basicNew_(size))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"basicNew:",0
,(
function(self,size)
return ((self):invalidMessage_("basicNew:"))
;
end
)
,"instantiation","basicNew: size\r\n\t\" Return an instance of the receiver of specified size. \"\r\n\r\n\t^self invalidMessage: #basicNew:","(\r\nfunction(self,size)\r\nreturn ((self):invalidMessage_(\"basicNew:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"basicNew",0
,(
function(self)
return (self["$fn"]())
;
end
)
,"instantiation","basicNew\r\n\t\" Return a new (non-initialized) instance of the receiver. \"\r\n\r\n\t^{'new self.$fn()'\r\n\tlua'self[\"$fn\"]()'}","(\r\nfunction(self)\r\nreturn (self[\"$fn\"]())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"name",0
,(
function(self)
return (((self)["$className"]))
;
end
)
,"accessing","name\r\n\t\" Return the name of the receiver. \"\r\n\r\n\t^self basicAt: '$className'","(\r\nfunction(self)\r\nreturn (((self)[\"$className\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"superclass",0
,(
function(self)
return (((self)["$superclass"]))
;
end
)
,"accessing","superclass\r\n\t\" Return the superclass of the receiver. \"\r\n\r\n\t^self basicAt: '$superclass'","(\r\nfunction(self)\r\nreturn (((self)[\"$superclass\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"subclasses",0
,(
function(self)
return (((self):smalltalk()):subclasses(self))
;
end
)
,"accessing","subclasses\r\n\t\" Return the subclasses of the receiver. \"\r\n\r\n\t^self smalltalk #subclasses: self","(\r\nfunction(self)\r\nreturn (((self):smalltalk()):subclasses(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"allSubclasses",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	result=_s8_value; return _s8_value
end)()
;
(self):allSubclassesDo_((function(each)
return ((result):add_(each))
end))
;
return (result)
;
end
)
,"testing","allSubclasses\r\n\t\" Return all the subclasses of the receiver. \"\r\n\r\n\t| result |\r\n\tresult := Array new.\r\n\tself allSubclassesDo: [:each| result add: each ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):allSubclassesDo_((function(each)\r\nreturn ((result):add_(each))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"allSubclassesDo:",0
,(
function(self,aBlock)
((self):smalltalk()):allSubclassesDo(self, aBlock)
;
return (self)
;
end
)
,"testing","allSubclassesDo: aBlock\r\n\t\" Evaluates aBlock with all the subclasses of the receiver. \"\r\n\r\n\tself smalltalk #allSubclassesDo: self value: aBlock","(\r\nfunction(self,aBlock)\r\n((self):smalltalk()):allSubclassesDo(self, aBlock)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"withAllSubclassesDo:",0
,(
function(self,aBlock)
(aBlock):value_(self)
;
(self):allSubclassesDo_(aBlock)
;
return (self)
;
end
)
,"testing","withAllSubclassesDo: aBlock\r\n\t\" Evaluates aBlock with the receiver and all the subclasses. \"\r\n\r\n\taBlock value: self.\r\n\tself allSubclassesDo: aBlock","(\r\nfunction(self,aBlock)\r\n(aBlock):value_(self)\r\n;\r\n(self):allSubclassesDo_(aBlock)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"withAllSubclasses",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((smalltalk.Array):with_(self))
	result=_s8_value; return _s8_value
end)()
;
(self):allSubclassesDo_((function(each)
return ((result):add_(each))
end))
;
return (result)
;
end
)
,"accessing","withAllSubclasses\r\n\t\" Return a collection with the receiver and all subclasses. \"\r\n\r\n\t| result |\r\n\tresult := Array with: self.\r\n\tself allSubclassesDo: [:each| result add: each ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):with_(self))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):allSubclassesDo_((function(each)\r\nreturn ((result):add_(each))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"allSuperclassesDo:",0
,(
function(self,aBlock)
local current=nil
(function()
	local _s8_value=((self):superclass())
	current=_s8_value; return _s8_value
end)()
;
(function() while(((function()
return ((((current))~=nil))
end))()) do (function()
(aBlock):value_(current);return ((function()
	local _s8_value=((current):superclass())
	current=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
return (self)
;
end
)
,"accessing","allSuperclassesDo: aBlock\r\n\t\" Evaluate aBlock with the superclasses of the receiver.\r\n\tThe superclasses are walked in inverse hierarchical order, i.e class Object is last.\r\n\t\"\r\n\r\n\t| current |\r\n\tcurrent := self superclass.\r\n\t[ current notNil ] whileTrue: [\r\n\t        aBlock value: current.\r\n\t\tcurrent := current superclass.\r\n\t]","(\r\nfunction(self,aBlock)\r\nlocal current=nil\r\n(function()\r\n\tlocal _s8_value=((self):superclass())\r\n\tcurrent=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\nreturn ((((current))~=nil))\r\nend))()) do (function()\r\n(aBlock):value_(current);return ((function()\r\n\tlocal _s8_value=((current):superclass())\r\n\tcurrent=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"withAllSuperclassesDo:",0
,(
function(self,aBlock)
(aBlock):value_(self)
;
return ((self):allSuperclassesDo_(aBlock))
;
end
)
,"accessing","withAllSuperclassesDo: aBlock\r\n\t\" Evaluate aBlock with the receiver and it's superclasses.\r\n\tThe superclasses are walked in inverse hierarchical order, i.e class Object is last.\r\n\t\"\r\n\r\n\taBlock value: self.\r\n\t^self allSuperclassesDo: aBlock","(\r\nfunction(self,aBlock)\r\n(aBlock):value_(self)\r\n;\r\nreturn ((self):allSuperclassesDo_(aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"withAllSuperclasses",0
,(
function(self)
return (smalltalk:object_comma_(((smalltalk.Array):with_(self)),(self):allSuperclasses()))
;
end
)
,"accessing","withAllSuperclasses\r\n\t\" Return a collection with the receiver and all superclasses. \"\r\n\r\n\t^(Array with: self) , self allSuperclasses","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_(((smalltalk.Array):with_(self)),(self):allSuperclasses()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"allSuperclasses",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	result=_s8_value; return _s8_value
end)()
;
(self):allSuperclassesDo_((function(each)
return ((result):add_(each))
end))
;
return (result)
;
end
)
,"accessing","allSuperclasses\r\n\t\" Return a collection of all the superclasses of the receiver.\r\n\tThe superclasses are in inverse hierarchical order, i.e class Object is last.\r\n\t\"\r\n\r\n\t| result |\r\n\tresult := Array new.\r\n\tself allSuperclassesDo: [:each| result add: each ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):allSuperclassesDo_((function(each)\r\nreturn ((result):add_(each))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"inheritsFrom:",0
,(
function(self,aClass)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((self)==aClass)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(self):allSuperclassesDo_((function(each)
return ((function() if (((((aClass)==each)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"testing","inheritsFrom: aClass\r\n\t\" Return true if the receiver inherits from aClass. \"\r\n\r\n\tself == aClass ifTrue: [ ^true ].\r\n\tself allSuperclassesDo: [:each|\r\n\t\taClass == each ifTrue: [ ^true ]\r\n\t].\r\n\t^false","(\r\nfunction(self,aClass)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((self)==aClass)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self):allSuperclassesDo_((function(each)\r\nreturn ((function() if (((((aClass)==each)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"prototype",0
,(
function(self)
return (((self["$fn"])["prototype"]))
;
end
)
,"accessing","prototype\r\n\t\" Return the prototype of the receiver (or nil). \"\r\n\r\n\t^self['$fn'] basicAt: #prototype","(\r\nfunction(self)\r\nreturn (((self[\"$fn\"])[\"prototype\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"constructorName",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
local fn=nil
(function()
	local _s8_value=(self["$fn"])
	fn=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((fn)["name"]))
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((((result))~=nil)))==true) then return ((function()
(function() if ((((result):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();(function() if ((((smalltalk:isObject_equalTo_((result),"$s8Constructor"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=(((smalltalk:toString_((fn))):upTo_("(")):asArrayOfSubstrings())
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((result):size()),(2)))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((result):last())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","constructorName\r\n\t\" Return the name of the constructor of the receiver (or nil). \"\r\n\r\n\t| result fn |\r\n\tfn := self['$fn'].\r\n\tresult := fn basicAt: #name.\r\n\tresult notNil ifTrue: [\r\n\t\tresult isEmpty ifTrue: [ ^nil ].\r\n\t\tresult = '$s8Constructor' ifTrue: [ ^nil ].\r\n\t\t^result\r\n\t].\r\n\tresult := (fn toString upTo: $() asArrayOfSubstrings.\r\n\tresult size = 2 ifFalse: [ ^nil ].\r\n\t^result last","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\nlocal fn=nil\r\n(function()\r\n\tlocal _s8_value=(self[\"$fn\"])\r\n\tfn=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((fn)[\"name\"]))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((result))~=nil)))==true) then return ((function()\r\n(function() if ((((result):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();(function() if ((((smalltalk:isObject_equalTo_((result),\"$s8Constructor\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:toString_((fn))):upTo_(\"(\")):asArrayOfSubstrings())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((result):size()),(2)))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((result):last())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"methodsDo:",0
,(
function(self,aBlock)
	for i,v in pairs(self["$fn"].prototype["$methods"]) do aBlock(v) end
;
return (self)
;
end
)
,"accessing","methodsDo: aBlock\r\n\t\" Evaluates aBlock for all the methods defined by the receiver. \"\r\n\r\n{'\tvar methods = self.$fn.prototype.$methods;\r\n\tfor(var i in methods) {\r\n\t\tif (!methods.hasOwnProperty(i)) break;\r\n\t\taBlock(methods[i]);\r\n\t}'\r\nlua'\tfor i,v in pairs(self[\"$fn\"].prototype[\"$methods\"]) do aBlock(v) end'}","(\r\nfunction(self,aBlock)\r\n\tfor i,v in pairs(self[\"$fn\"].prototype[\"$methods\"]) do aBlock(v) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"compiledMethods",0
,(
function(self)
local result=nil
(function()
	local _s8_value=(smalltalk:newArray({}))
	result=_s8_value; return _s8_value
end)()
;
(self):methodsDo_((function(mth)
return ((result):add_(mth))
end))
;
return (result)
;
end
)
,"accessing","compiledMethods\r\n\t\" Returns a collection with all the methods defined by the receiver. \"\r\n\r\n\t| result |\r\n\tresult := #().\r\n\tself methodsDo: [:mth| result add: mth ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):methodsDo_((function(mth)\r\nreturn ((result):add_(mth))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"selectorsDo:",0
,(
function(self,aBlock)
	for i,v in pairs(self["$fn"].prototype["$methods"]) do aBlock(i) end
;
return (self)
;
end
)
,"accessing","selectorsDo: aBlock\r\n\t\" Evaluate aBlock for all the selectors implemented by the receiver. \"\r\n\r\n{'\tvar methods = self.$fn.prototype.$methods;\r\n\tfor(var i in methods) {\r\n\t\tif (!methods.hasOwnProperty(i)) break;\r\n\t\taBlock(methods[i].$selector);\r\n\t}'\r\nlua'\tfor i,v in pairs(self[\"$fn\"].prototype[\"$methods\"]) do aBlock(i) end'}","(\r\nfunction(self,aBlock)\r\n\tfor i,v in pairs(self[\"$fn\"].prototype[\"$methods\"]) do aBlock(i) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"methodDictionary",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((smalltalk.PoolDictionary):new())
	result=_s8_value; return _s8_value
end)()
;
(self):methodsDo_((function(each)
return ((result):at_put_((each):selector(), each))
end))
;
return (result)
;
end
)
,"accessing","methodDictionary\r\n\t\" Return a dictionary with the methods implemented by the receiver. \"\r\n\r\n\t| result |\r\n\tresult := PoolDictionary new.\r\n\tself methodsDo: [:each| result at: each selector put: each ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.PoolDictionary):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):methodsDo_((function(each)\r\nreturn ((result):at_put_((each):selector(), each))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"selectors",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	result=_s8_value; return _s8_value
end)()
;
(self):selectorsDo_((function(each)
return ((result):add_(each))
end))
;
return (result)
;
end
)
,"accessing","selectors\r\n\t\" Return the selectors implemented by the receiver. \"\r\n\r\n\t| result |\r\n\tresult := Array new.\r\n\tself selectorsDo: [:each| result add: each ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):selectorsDo_((function(each)\r\nreturn ((result):add_(each))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"categorizedSelectors",0
,(
function(self)
local pool=nil
(function()
	local _s8_value=((smalltalk.PoolDictionary):new())
	pool=_s8_value; return _s8_value
end)()
;
(self):methodsDo_((function(mth)
local category=nil
(function()
	local _s8_value=((mth):category())
	category=_s8_value; return _s8_value
end)();(function() if ((((((category))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=("undefined")
	category=_s8_value; return _s8_value
end)())
end))() else return nil end end)();return (((pool):at_ifAbsentPut_(category, (function()
return ((smalltalk.Array):new())
end))):add_((mth):selector()))
end))
;
return ((((pool):keys()):sorted()):collect_((function(key)
return ((smalltalk.Array):with_with_(key, ((pool):at_(key)):sorted()))
end)))
;
end
)
,"accessing","categorizedSelectors\r\n\t\" Return the selectors classified by category. \"\r\n\r\n\t| pool |\r\n\tpool := PoolDictionary new.\r\n\tself methodsDo: [:mth| | category |\r\n\t\tcategory := mth category.\r\n\t\tcategory isNil ifTrue: [ category := #undefined ].\r\n\t\t(pool at: category ifAbsentPut: [Array new]) add: mth selector\r\n\t].\r\n\t^pool keys sorted collect: [:key|\r\n\t\tArray with: key with: (pool at: key) sorted\r\n\t]","(\r\nfunction(self)\r\nlocal pool=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.PoolDictionary):new())\r\n\tpool=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):methodsDo_((function(mth)\r\nlocal category=nil\r\n(function()\r\n\tlocal _s8_value=((mth):category())\r\n\tcategory=_s8_value; return _s8_value\r\nend)();(function() if ((((((category))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(\"undefined\")\r\n\tcategory=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)();return (((pool):at_ifAbsentPut_(category, (function()\r\nreturn ((smalltalk.Array):new())\r\nend))):add_((mth):selector()))\r\nend))\r\n;\r\nreturn ((((pool):keys()):sorted()):collect_((function(key)\r\nreturn ((smalltalk.Array):with_with_(key, ((pool):at_(key)):sorted()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"implements:",0
,(
function(self,aSymbol)
return ((self["$fn"].prototype["$methods"][aSymbol]~=nil))
;
end
)
,"testing","implements: aSymbol\r\n\t\" Return true if the receiver implements message aSymbol. \"\r\n\r\n\t^{'Object.prototype.hasOwnProperty.call(self.$fn.prototype.$methods,aSymbol)'\r\n\tlua'(self[\"$fn\"].prototype[\"$methods\"][aSymbol]~=nil)'}","(\r\nfunction(self,aSymbol)\r\nreturn ((self[\"$fn\"].prototype[\"$methods\"][aSymbol]~=nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"compiledMethodAt:",0
,(
function(self,aSymbol)
	if true then return (self["$fn"].prototype["$methods"][aSymbol]) end
;
return (self)
;
end
)
,"accessing","compiledMethodAt: aSymbol\r\n\t\" Return the compiled method at aSymbol implemented by the receiver (or nil). \"\r\n\r\n{'\tvar methods = self.$fn.prototype.$methods;\r\n\tvar found = Object.prototype.hasOwnProperty.call(methods,aSymbol);\r\n\tif (found) return methods[aSymbol];\r\n\treturn nil'\r\nlua'\tif true then return (self[\"$fn\"].prototype[\"$methods\"][aSymbol]) end'}","(\r\nfunction(self,aSymbol)\r\n\tif true then return (self[\"$fn\"].prototype[\"$methods\"][aSymbol]) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,">>","_gt_gt"
,(
function(self,aSymbol)
return ((self):compiledMethodAt_(aSymbol))
;
end
)
,"accessing",">> aSymbol\r\n\t\" Return the method at aSymbol (or nil if not implemented by the receiver). \"\r\n\r\n\t^self compiledMethodAt: aSymbol","(\r\nfunction(self,aSymbol)\r\nreturn ((self):compiledMethodAt_(aSymbol))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,">>>","_gt_gt_gt"
,(
function(self,aSymbol)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function()
	local _s8_value=((self):compiledMethodAt_(aSymbol))
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:boolean_and_(((((result))==nil)),(function()
return (((((self):superclass()))~=nil))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self):superclass()):_gt_gt_gt(aSymbol))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing",">>> aSymbol\r\n\t\" Return the method at aSymbol searching up in receiver's hierarchy (or nil if not implemented by the receiver nor its superclasses). \"\r\n\r\n\t| result |\r\n\tresult := self compiledMethodAt: aSymbol.\r\n\t(result isNil and: [self superclass notNil]) ifTrue: [\r\n\t\t^self superclass >>> aSymbol\r\n\t].\r\n\t^result","(\r\nfunction(self,aSymbol)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):compiledMethodAt_(aSymbol))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((((result))==nil)),(function()\r\nreturn (((((self):superclass()))~=nil))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self):superclass()):_gt_gt_gt(aSymbol))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"instVarNames",0
,(
function(self)
return (((self)["$iVarNames"]))
;
end
)
,"accessing","instVarNames\r\n\t\" Return the instance variable names of the receiver. \"\r\n\r\n\t^self basicAt: '$iVarNames'","(\r\nfunction(self)\r\nreturn (((self)[\"$iVarNames\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"allInstVarNames",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((self):instVarNames())
	result=_s8_value; return _s8_value
end)()
;
(self):allSuperclassesDo_((function(each)
return ((function()
	local _s8_value=(smalltalk:object_comma_(((each):instVarNames()),result))
	result=_s8_value; return _s8_value
end)())
end))
;
return (result)
;
end
)
,"accessing","allInstVarNames\r\n\t\" Return all of the instance variable names defined in the receiver and its superclasses. \"\r\n\r\n\t| result |\r\n\tresult := self instVarNames.\r\n\tself allSuperclassesDo: [ :each | result := each instVarNames , result ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):instVarNames())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):allSuperclassesDo_((function(each)\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_(((each):instVarNames()),result))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"instanceVariableNames",0
,(
function(self)
return (((self)["$iVarNames"]))
;
end
)
,"accessing","instanceVariableNames\r\n\t\" Return the instance variable names defined by the receiver. \"\r\n\r\n\t^self basicAt: '$iVarNames'","(\r\nfunction(self)\r\nreturn (((self)[\"$iVarNames\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"propertyNames",0
,(
function(self)
return ((function() local _1_=((self)["$propertyNames"]);if (_1_==nil) then return ((function()
return ((smalltalk.Array):new())
end)):value() end; return _1_ end)())
;
end
)
,"accessing","propertyNames\r\n\t\" Return the property names of the receiver. \"\r\n\r\n\t^self basicAt: '$propertyNames' ifAbsent: [ Array new ]","(\r\nfunction(self)\r\nreturn ((function() local _1_=((self)[\"$propertyNames\"]);if (_1_==nil) then return ((function()\r\nreturn ((smalltalk.Array):new())\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"propertyNames:",0
,(
function(self,anArray)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((anArray))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):basicDelete_("$propertyNames"))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function() local _1_=(anArray); (self)["$propertyNames"]=_1_; return _1_ end)())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","propertyNames: anArray\r\n\t\" Set the property names of the receiver. \"\r\n\r\n\tanArray isNil ifTrue: [ ^self basicDelete: '$propertyNames' ].\r\n\t^self basicAt: '$propertyNames' put: anArray","(\r\nfunction(self,anArray)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((anArray))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):basicDelete_(\"$propertyNames\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function() local _1_=(anArray); (self)[\"$propertyNames\"]=_1_; return _1_ end)())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"allPropertyNamesDo:",0
,(
function(self,aBlock)
((self):propertyNames()):do_(aBlock)
;
(self):allSuperclassesDo_((function(each)
return (((each):propertyNames()):do_(aBlock))
end))
;
return (self)
;
end
)
,"accessing","allPropertyNamesDo: aBlock\r\n\t\" Evaluates aBlock for the property names defined in the receiver and its superclasses. \"\r\n\r\n\tself propertyNames do: aBlock.\r\n\tself allSuperclassesDo: [ :each | each propertyNames do: aBlock ].","(\r\nfunction(self,aBlock)\r\n((self):propertyNames()):do_(aBlock)\r\n;\r\n(self):allSuperclassesDo_((function(each)\r\nreturn (((each):propertyNames()):do_(aBlock))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"allInstVarNamesDo:",0
,(
function(self,aBlock)
((self):instVarNames()):do_(aBlock)
;
(self):allSuperclassesDo_((function(each)
return (((each):instVarNames()):do_(aBlock))
end))
;
return (self)
;
end
)
,"accessing","allInstVarNamesDo: aBlock\r\n\t\" Evaluates aBlock for the instance variable names defined in the receiver and its superclasses. \"\r\n\r\n\tself instVarNames do: aBlock.\r\n\tself allSuperclassesDo: [ :each | each instVarNames do: aBlock ].","(\r\nfunction(self,aBlock)\r\n((self):instVarNames()):do_(aBlock)\r\n;\r\n(self):allSuperclassesDo_((function(each)\r\nreturn (((each):instVarNames()):do_(aBlock))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"comment",0
,(
function(self)
return ((function() local _1_=(((self):instanceClass())["$comment"]);if (_1_==nil) then return ((function()
return ("")
end)):value() end; return _1_ end)())
;
end
)
,"accessing","comment\r\n\t\" Return the comment of the receiver. \"\r\n\r\n\t^self instanceClass basicAt: '$comment' ifAbsent: [ '' ]","(\r\nfunction(self)\r\nreturn ((function() local _1_=(((self):instanceClass())[\"$comment\"]);if (_1_==nil) then return ((function()\r\nreturn (\"\")\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"comment:",0
,(
function(self,aString)
(function() local _1_=(aString); ((self):instanceClass())["$comment"]=_1_; return _1_ end)()
;
return (self)
;
end
)
,"accessing","comment: aString\r\n\t\" Set the comment of the receiver. \"\r\n\r\n\tself instanceClass basicAt: '$comment' put: aString","(\r\nfunction(self,aString)\r\n(function() local _1_=(aString); ((self):instanceClass())[\"$comment\"]=_1_; return _1_ end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"removeSources",0
,(
function(self)
((self):instanceClass()):removeSources()
;
return (self)
;
end
)
,"tools","removeSources\r\n\t\" Remove all development sources of the receiver. \"\r\n\r\n\tself instanceClass removeSources","(\r\nfunction(self)\r\n((self):instanceClass()):removeSources()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"poolDictionaryNames",0
,(
function(self)
return (((self):instanceClass()):poolDictionaryNames())
;
end
)
,"accessing","poolDictionaryNames\r\n\t\" Return the shared pool names of the receiver. \"\r\n\r\n\t^self instanceClass poolDictionaryNames","(\r\nfunction(self)\r\nreturn (((self):instanceClass()):poolDictionaryNames())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"sharedPoolsAndNamesDo:",0
,(
function(self,aBlock)
local pool=nil
((self):instanceClass()):withAllSuperclassesDo_((function(each)
return (((each):poolDictionaryNames()):do_((function(aName)
(function()
	local _s8_value=((smalltalk.Smalltalk):poolDictionary_ifAbsent_(aName, (function()
return (nil)
end)))
	pool=_s8_value; return _s8_value
end)();return ((function() if ((((((pool))~=nil)))==true) then return ((function()
return ((aBlock):value_value_(aName, pool))
end))() else return nil end end)())
end)))
end))
;
return (self)
;
end
)
,"accessing","sharedPoolsAndNamesDo: aBlock\r\n\t\" Execute aBlock for the pool dictionaries known by the receiver. \"\r\n\r\n\t| pool |\r\n\tself instanceClass withAllSuperclassesDo: [:each|\r\n\t\teach poolDictionaryNames do: [:aName|\r\n\t\t\tpool := Smalltalk poolDictionary: aName ifAbsent: [ nil ].\r\n\t\t\tpool notNil ifTrue: [ aBlock value: aName value: pool ]\r\n\t\t].\r\n\t]","(\r\nfunction(self,aBlock)\r\nlocal pool=nil\r\n((self):instanceClass()):withAllSuperclassesDo_((function(each)\r\nreturn (((each):poolDictionaryNames()):do_((function(aName)\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):poolDictionary_ifAbsent_(aName, (function()\r\nreturn (nil)\r\nend)))\r\n\tpool=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((pool))~=nil)))==true) then return ((function()\r\nreturn ((aBlock):value_value_(aName, pool))\r\nend))() else return nil end end)())\r\nend)))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"sharedPoolsDo:",0
,(
function(self,aBlock)
(self):sharedPoolsAndNamesDo_((function(aName,aPool)
return ((aBlock):value_(aPool))
end))
;
return (self)
;
end
)
,"accessing","sharedPoolsDo: aBlock\r\n\t\" Execute aBlock for the pool dictionaries known by the receiver. \"\r\n\r\n\tself sharedPoolsAndNamesDo: [:aName :aPool| aBlock value: aPool ]","(\r\nfunction(self,aBlock)\r\n(self):sharedPoolsAndNamesDo_((function(aName,aPool)\r\nreturn ((aBlock):value_(aPool))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"classVariablesDo:",0
,(
function(self,aBlock)
local bindings=nil
return (((self):instanceClass()):withAllSuperclassesDo_((function(each)
(function()
	local _s8_value=((each):classVariables())
	bindings=_s8_value; return _s8_value
end)();return ((function() if ((((((bindings))~=nil)))==true) then return ((function()
return ((bindings):associationsDo_((function(assoc)
return ((aBlock):value_(assoc))
end)))
end))() else return nil end end)())
end)))
;
end
)
,"accessing","classVariablesDo: aBlock\r\n\t\" Execute aBlock for the classVariables bindings known by the receiver. \"\r\n\r\n\t| bindings |\r\n\t^self instanceClass withAllSuperclassesDo: [:each|\r\n\t\tbindings := each classVariables.\r\n\t\tbindings notNil ifTrue: [\r\n\t\t\tbindings associationsDo: [:assoc| aBlock value: assoc ]\r\n\t\t]\r\n\t]","(\r\nfunction(self,aBlock)\r\nlocal bindings=nil\r\nreturn (((self):instanceClass()):withAllSuperclassesDo_((function(each)\r\n(function()\r\n\tlocal _s8_value=((each):classVariables())\r\n\tbindings=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((bindings))~=nil)))==true) then return ((function()\r\nreturn ((bindings):associationsDo_((function(assoc)\r\nreturn ((aBlock):value_(assoc))\r\nend)))\r\nend))() else return nil end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"classVariableNamesDo:",0
,(
function(self,aBlock)
local bindings=nil
return (((self):instanceClass()):withAllSuperclassesDo_((function(each)
(function()
	local _s8_value=((each):classVariables())
	bindings=_s8_value; return _s8_value
end)();return ((function() if ((((((bindings))~=nil)))==true) then return ((function()
return ((bindings):keysDo_((function(aName)
return ((aBlock):value_(aName))
end)))
end))() else return nil end end)())
end)))
;
end
)
,"accessing","classVariableNamesDo: aBlock\r\n\t\" Execute aBlock for the classVariable names known by the receiver. \"\r\n\r\n\t| bindings |\r\n\t^self instanceClass withAllSuperclassesDo: [:each|\r\n\t\tbindings := each classVariables.\r\n\t\tbindings notNil ifTrue: [\r\n\t\t\tbindings keysDo: [:aName| aBlock value: aName ]\r\n\t\t]\r\n\t]","(\r\nfunction(self,aBlock)\r\nlocal bindings=nil\r\nreturn (((self):instanceClass()):withAllSuperclassesDo_((function(each)\r\n(function()\r\n\tlocal _s8_value=((each):classVariables())\r\n\tbindings=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((bindings))~=nil)))==true) then return ((function()\r\nreturn ((bindings):keysDo_((function(aName)\r\nreturn ((aBlock):value_(aName))\r\nend)))\r\nend))() else return nil end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"classVariableAt:put:",0
,(
function(self,aName,anObject)
return ((((self):instanceClass()):classVariables()):at_put_(aName, anObject))
;
end
)
,"accessing","classVariableAt: aName put: anObject\r\n\t\" Set the classVariable of the receiver to anObject. \"\r\n\r\n\t^self instanceClass classVariables at: aName put: anObject","(\r\nfunction(self,aName,anObject)\r\nreturn ((((self):instanceClass()):classVariables()):at_put_(aName, anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"classVariableAt:ifAbsent:",0
,(
function(self,aName,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local bindings=nil
((self):instanceClass()):withAllSuperclassesDo_((function(each)
(function()
	local _s8_value=((each):classVariables())
	bindings=_s8_value; return _s8_value
end)();return ((function() if ((((((bindings))~=nil)))==true) then return ((function()
return ((function() if ((((bindings):includesKey_(aName)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((bindings):at_(aName))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):evaluate())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","classVariableAt: aName ifAbsent: aBlock\r\n\t\" Return the value of classVariable named aName. \"\r\n\r\n\t| bindings |\r\n\tself instanceClass withAllSuperclassesDo: [:each|\r\n\t\tbindings := each classVariables.\r\n\t\tbindings notNil ifTrue: [\r\n\t\t\t(bindings includesKey: aName) ifTrue: [\r\n\t\t\t\t^bindings at: aName\r\n\t\t\t]\r\n\t\t]\r\n\t].\r\n\t^aBlock evaluate","(\r\nfunction(self,aName,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal bindings=nil\r\n((self):instanceClass()):withAllSuperclassesDo_((function(each)\r\n(function()\r\n\tlocal _s8_value=((each):classVariables())\r\n\tbindings=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((bindings))~=nil)))==true) then return ((function()\r\nreturn ((function() if ((((bindings):includesKey_(aName)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((bindings):at_(aName))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):evaluate())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"classVariableAt:",0
,(
function(self,aName)
return ((self):classVariableAt_ifAbsent_(aName, (function()
return ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("Undefined class variable "),aName))," in ")),(self):name())))
end)))
;
end
)
,"accessing","classVariableAt: aName\r\n\t\" Return the value of classVariable named aName or generates an error. \"\r\n\r\n\t^self classVariableAt: aName ifAbsent: [\r\n\t\tself error: 'Undefined class variable ',aName ,' in ' ,self name\r\n\t]","(\r\nfunction(self,aName)\r\nreturn ((self):classVariableAt_ifAbsent_(aName, (function()\r\nreturn ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"Undefined class variable \"),aName)),\" in \")),(self):name())))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"pool:variableAt:put:",0
,(
function(self,poolName,aName,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local pool=nil
(function()
	local _s8_value=((smalltalk.Smalltalk):poolDictionary_ifAbsent_(poolName, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("Undefined pool variable "),poolName))," in ")),(self):name())))
end
)()
error(_s8Ret_,0)
end)())
end)))
	pool=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((pool):at_put_(aName, anObject))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","pool: poolName variableAt: aName put: anObject\r\n\t\" Set the pool Variable of the receiver to anObject. \"\r\n\r\n\t| pool |\r\n\tpool := Smalltalk\r\n\t\tpoolDictionary: poolName\r\n\t\tifAbsent: [ ^self error: 'Undefined pool variable ',poolName ,' in ' ,self name ].\r\n\t^pool at: aName put: anObject","(\r\nfunction(self,poolName,aName,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal pool=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):poolDictionary_ifAbsent_(poolName, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"Undefined pool variable \"),poolName)),\" in \")),(self):name())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)))\r\n\tpool=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((pool):at_put_(aName, anObject))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"loadRequirementsInto:",0
,(
function(self,aCollection)
return (self)
;
end
)
,"loading","loadRequirementsInto: aCollection\r\n\t\" Add the requirements to load the receiver from sources to aCollection.\r\n\tDefault action do nothing, this method can be refined by the subclasses.\r\n\t\"\r\n\r\n\t^self","(\r\nfunction(self,aCollection)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"removeSelectors:",0
,(
function(self,aCollection)
(aCollection):do_((function(each)
return ((self):removeSelector_(each))
end))
;
return (self)
;
end
)
,"methods","removeSelectors: aCollection\r\n\t\" Remove the methods for messages in aCollection, implemented by the receiver. \"\r\n\r\n\taCollection do: [:each| self removeSelector: each ]","(\r\nfunction(self,aCollection)\r\n(aCollection):do_((function(each)\r\nreturn ((self):removeSelector_(each))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"removeCompiledMethod:",0
,(
function(self,aMethod)
((self):smalltalk()):removeMethod(self, aMethod)
;
return (self)
;
end
)
,"private","removeCompiledMethod: aMethod\r\n\t\" Remove aMethod from the receiver. \"\r\n\r\n\tself smalltalk #removeMethod: self method: aMethod","(\r\nfunction(self,aMethod)\r\n((self):smalltalk()):removeMethod(self, aMethod)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"missingImplementorFor:",0
,(
function(self,jsSelector)
return ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("Missing implementor for "),((jsSelector):asStSelector()):asLiteral()))," in ")),(self):name())))
;
end
)
,"private","missingImplementorFor: jsSelector\r\n\t\" Private - Handle the situation where there is no implementor for jsSelector.\r\n\tThis can happen when there is no implementation for jsSelector and the message\r\n\t has been dispatched through a superSend.\r\n\t\"\r\n\r\n\t^self error: 'Missing implementor for '\r\n\t\t, jsSelector asStSelector asLiteral\r\n\t\t,' in ',self name","(\r\nfunction(self,jsSelector)\r\nreturn ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"Missing implementor for \"),((jsSelector):asStSelector()):asLiteral())),\" in \")),(self):name())))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"removeSelector:",0
,(
function(self,aSelector)
local method=nil
(function()
	local _s8_value=((self):compiledMethodAt_(aSelector))
	method=_s8_value; return _s8_value
end)()
;
(function() if ((((((method))~=nil)))==true) then return ((function()
return ((self):removeCompiledMethod_(method))
end))() else return nil end end)()
;
return (self)
;
end
)
,"methods","removeSelector: aSelector\r\n\t\" Remove the method for message aSelector implemented by the receiver. \"\r\n\r\n\t| method |\r\n\tmethod := self compiledMethodAt: aSelector.\r\n\tmethod notNil ifTrue: [ self removeCompiledMethod: method ].","(\r\nfunction(self,aSelector)\r\nlocal method=nil\r\n(function()\r\n\tlocal _s8_value=((self):compiledMethodAt_(aSelector))\r\n\tmethod=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((method))~=nil)))==true) then return ((function()\r\nreturn ((self):removeCompiledMethod_(method))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"removeFromSystem",0
,(
function(self)
((self):instanceClass()):removeFromSystem()
;
return (self)
;
end
)
,"remove","removeFromSystem\r\n\t\" Remove the receiver from system. \"\r\n\r\n\tself instanceClass removeFromSystem","(\r\nfunction(self)\r\n((self):instanceClass()):removeFromSystem()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"implementorsOf:",0
,(
function(self,aSymbol)
local result=nil
(function()
	local _s8_value=(smalltalk:newArray({}))
	result=_s8_value; return _s8_value
end)()
;
(self):withAllSubclassesDo_((function(each)
local method=nil
(function()
	local _s8_value=((each):_gt_gt(aSymbol))
	method=_s8_value; return _s8_value
end)();(function() if ((((((method))~=nil)))==true) then return ((function()
return ((result):add_(method))
end))() else return nil end end)();(function()
	local _s8_value=(((each)["$klass"]):_gt_gt(aSymbol))
	method=_s8_value; return _s8_value
end)();return ((function() if ((((((method))~=nil)))==true) then return ((function()
return ((result):add_(method))
end))() else return nil end end)())
end))
;
return (result)
;
end
)
,"testing","implementorsOf: aSymbol\r\n\t\" Return the implementors of aSymbol in the receiver and its subclasses. \"\r\n\r\n\t| result |\r\n\tresult := #().\r\n\tself withAllSubclassesDo: [:each | | method |\r\n\t\tmethod := each >> aSymbol.\r\n\t\tmethod notNil ifTrue: [ result add: method ].\r\n\t\tmethod := each class >> aSymbol.\r\n\t\tmethod notNil ifTrue: [ result add: method ].\r\n\t].\r\n\t^result","(\r\nfunction(self,aSymbol)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):withAllSubclassesDo_((function(each)\r\nlocal method=nil\r\n(function()\r\n\tlocal _s8_value=((each):_gt_gt(aSymbol))\r\n\tmethod=_s8_value; return _s8_value\r\nend)();(function() if ((((((method))~=nil)))==true) then return ((function()\r\nreturn ((result):add_(method))\r\nend))() else return nil end end)();(function()\r\n\tlocal _s8_value=(((each)[\"$klass\"]):_gt_gt(aSymbol))\r\n\tmethod=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((method))~=nil)))==true) then return ((function()\r\nreturn ((result):add_(method))\r\nend))() else return nil end end)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"asJSONObject",0
,(
function(self)
return (self)
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"shallowCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"deepCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"toString",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_(("{st:"),(self):name())),"}"))
;
end
)
,"converting","toString\r\n\t\" Convert the receiver to a String. \"\r\n\r\n\t^'{st:',self name,$}","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"{st:\"),(self):name())),\"}\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"basicAddCompiledMethod:",0
,(
function(self,aMethod)
(smalltalk.APIBuilder):register_(aMethod)
;
(smalltalk):addMethod(((aMethod):selector()):asSelector(), aMethod, self)
;
return (self)
;
end
)
,"Builders-private","basicAddCompiledMethod: aMethod\r\n\t\" Private - Effectivelly add aMethod to the receiver. \"\r\n\r\n\tAPIBuilder register: aMethod.\r\n\t#{smalltalk} #addMethod: aMethod selector asSelector method: aMethod to: self","(\r\nfunction(self,aMethod)\r\n(smalltalk.APIBuilder):register_(aMethod)\r\n;\r\n(smalltalk):addMethod(((aMethod):selector()):asSelector(), aMethod, self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"addCompiledMethod:",0
,(
function(self,aMethod)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local isDoIt=nil
(function()
	local _s8_value=((smalltalk:isObject_equalTo_(((aMethod):selector()),(smalltalk.String):doItSelector())))
	isDoIt=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:boolean_and_(((((smalltalk.OverwriteMethods))~=nil)),(function()
return (smalltalk.OverwriteMethods)
end)))))==true) then return nil else return ((function()
return ((function() if ((((self):implements_((aMethod):selector())))==true) then return ((function()
return ((function() if (((isDoIt))==true) then return nil else return ((function()
(function() if ((((smalltalk:boolean_and_(((((smalltalk.DumpOverwritenMethods))~=nil)),(function()
return ((smalltalk:boolean_and_((smalltalk.DumpOverwritenMethods),(function()
return ((((smalltalk.Exporter))~=nil))
end))))
end)))))==true) then return ((function()
(self):note_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("? "),(self):name())),">>#")),(aMethod):selector()));return ((self):print_(((smalltalk.Exporter):new()):export_method_(self, aMethod)))
end))() else return ((function()
return ((self):note_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("-Overwrite Conflict- "),(self):name())),">>#")),(aMethod):selector())))
end))() end end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)())
end))() else return nil end end)())
end))() end end)()
;
(function() if ((((smalltalk:boolean_and_(((((smalltalk.Exporter))~=nil)),(function()
return ((smalltalk:booleanNot_((isDoIt))))
end)))))==true) then return ((function()
return ((self):print_(((smalltalk.Exporter):new()):export_method_(self, aMethod)))
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):basicAddCompiledMethod_(aMethod))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"methods","addCompiledMethod: aMethod\r\n\t\" Add the compiled method, if appropiate, and register the method in recent methods. \"\r\n\r\n\t| isDoIt |\r\n\tisDoIt := aMethod selector = String doItSelector.\r\n\t(OverwriteMethods notNil and: [\tOverwriteMethods ]) ifFalse: [\r\n\t\t(self implements: aMethod selector) ifTrue: [\r\n\t\t\tisDoIt ifFalse: [\r\n\t\t\t\t(DumpOverwritenMethods notNil\r\n\t\t\t\tand: [ DumpOverwritenMethods\r\n\t\t\t\tand: [ Exporter notNil ]]) ifTrue: [\r\n\t\t\t\t\tself note: '? ',self name,'>>#',aMethod selector.\r\n\t\t\t\t\tself print: (Exporter new export: self method: aMethod).\r\n\t\t\t\t] ifFalse: [\r\n\t\t\t\t\tself note: '-Overwrite Conflict- ',self name,'>>#',aMethod selector.\r\n\t\t\t\t].\r\n\t\t\t\t^self\r\n\t\t\t].\r\n\t\t].\r\n\t].\r\n\t(Exporter notNil and: [ isDoIt not ]) ifTrue: [\r\n\" DEBUG -\tself note: self name,'>>#',aMethod selector.\"\r\n\t\tself print: (Exporter new export: self method: aMethod).\r\n\t].\r\n\t^self basicAddCompiledMethod: aMethod","(\r\nfunction(self,aMethod)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal isDoIt=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk:isObject_equalTo_(((aMethod):selector()),(smalltalk.String):doItSelector())))\r\n\tisDoIt=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((((smalltalk.OverwriteMethods))~=nil)),(function()\r\nreturn (smalltalk.OverwriteMethods)\r\nend)))))==true) then return nil else return ((function()\r\nreturn ((function() if ((((self):implements_((aMethod):selector())))==true) then return ((function()\r\nreturn ((function() if (((isDoIt))==true) then return nil else return ((function()\r\n(function() if ((((smalltalk:boolean_and_(((((smalltalk.DumpOverwritenMethods))~=nil)),(function()\r\nreturn ((smalltalk:boolean_and_((smalltalk.DumpOverwritenMethods),(function()\r\nreturn ((((smalltalk.Exporter))~=nil))\r\nend))))\r\nend)))))==true) then return ((function()\r\n(self):note_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"? \"),(self):name())),\">>#\")),(aMethod):selector()));return ((self):print_(((smalltalk.Exporter):new()):export_method_(self, aMethod)))\r\nend))() else return ((function()\r\nreturn ((self):note_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"-Overwrite Conflict- \"),(self):name())),\">>#\")),(aMethod):selector())))\r\nend))() end end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)())\r\nend))() else return nil end end)())\r\nend))() end end)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((((smalltalk.Exporter))~=nil)),(function()\r\nreturn ((smalltalk:booleanNot_((isDoIt))))\r\nend)))))==true) then return ((function()\r\nreturn ((self):print_(((smalltalk.Exporter):new()):export_method_(self, aMethod)))\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):basicAddCompiledMethod_(aMethod))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"collection",0
,(
function(self)
return ((
function(_s8_rec)
(_s8_rec):species_(self);
return ((_s8_rec):yourself())
end
)((smalltalk.ComposedObjects):new()))
;
end
)
,"composing","collection\r\n\t\" Return a collection to compose instances of the receiver. \"\r\n\r\n\t^ComposedObjects new\r\n\t\tspecies: self;\r\n\t\tyourself","(\r\nfunction(self)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):species_(self);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.ComposedObjects):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"codeFor:accessing:",0
,(
function(self,anEncoder,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local aName=nil
(function()
	local _s8_value=((aNode):value())
	aName=_s8_value; return _s8_value
end)()
;
(self):allInstVarNamesDo_((function(each)
return ((function() if ((((smalltalk:isObject_equalTo_((aName),each))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((anEncoder):codeToAccessInstanceVariable_(aName))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(self):allPropertyNamesDo_((function(each)
return ((function() if ((((smalltalk:isObject_equalTo_((aName),each))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((anEncoder):codeToAccessProperty_(aName))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Compiler-generation","codeFor: anEncoder accessing: aNode\r\n\t\" Return the code to access variable in aNode (aVariableNode).\r\n\tReturn nil if the code to access variable can t be generated.\r\n\tThe default implementation return code to access instance variables.\r\n\t\"\r\n\r\n\t| aName |\r\n\taName := aNode value.\r\n\tself allInstVarNamesDo: [:each|\r\n\t\taName = each ifTrue: [\r\n\t\t\t^anEncoder codeToAccessInstanceVariable: aName\r\n\t\t].\r\n\t].\r\n\tself allPropertyNamesDo: [:each|\r\n\t\taName = each ifTrue: [\r\n\t\t\t^anEncoder codeToAccessProperty: aName\r\n\t\t].\r\n\t].\r\n\t^nil","(\r\nfunction(self,anEncoder,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal aName=nil\r\n(function()\r\n\tlocal _s8_value=((aNode):value())\r\n\taName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):allInstVarNamesDo_((function(each)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((aName),each))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((anEncoder):codeToAccessInstanceVariable_(aName))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(self):allPropertyNamesDo_((function(each)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((aName),each))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((anEncoder):codeToAccessProperty_(aName))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"compiler",0
,(
function(self)
return ((((self):smalltalk()):compilerSupport()):for_(self))
;
end
)
,"Compiler-compiling","compiler\r\n\t\" Return a compiler for receiver's source code. \"\r\n\r\n\t^self smalltalk compilerSupport for: self","(\r\nfunction(self)\r\nreturn ((((self):smalltalk()):compilerSupport()):for_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"recompile",0
,(
function(self)
((self):compiler()):recompile_(self)
;
return (self)
;
end
)
,"Compiler-compiling","recompile\r\n\t\" Recompile all the methods of the receiver. \"\r\n\r\n\tself compiler recompile: self","(\r\nfunction(self)\r\n((self):compiler()):recompile_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"recompileAll",0
,(
function(self)
((self):instanceClass()):withAllSubclassesDo_((function(each)
return ((each):recompile())
end))
;
return (self)
;
end
)
,"Compiler-compiling","recompileAll\r\n\t\" Recompile all the methods of the receiver and its subclasses.\t\"\r\n\r\n\tself instanceClass withAllSubclassesDo: [:each| each recompile ]","(\r\nfunction(self)\r\n((self):instanceClass()):withAllSubclassesDo_((function(each)\r\nreturn ((each):recompile())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"recompile:",0
,(
function(self,what)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((what):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):recompile_((smalltalk.Array):with_(what)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(what):do_((function(selector)
return (((self):_gt_gt(selector)):recompile())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Compiler-compiling","recompile: what\r\n\t\" Recompile the method(s) at selector(s) of the receiver.\t\"\r\n\r\n\twhat isString ifTrue: [ ^self recompile: (Array with: what) ].\r\n\twhat do: [:selector| (self >> selector) recompile ]","(\r\nfunction(self,what)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((what):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):recompile_((smalltalk.Array):with_(what)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(what):do_((function(selector)\r\nreturn (((self):_gt_gt(selector)):recompile())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"templateFrom:doing:on:",0
,(
function(self,host,aBlock,methodBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local template=nil
(function() if ((((((host))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((function() local _1_=((host)["s8Template"]);if (_1_==nil) then return ((function()
return (nil)
end)):value() end; return _1_ end)())
	template=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:boolean_and_(((((template))==nil)),(function()
return (smalltalk:is_kindOf_((host),(smalltalk.CompiledMethod)))
end)))))==true) then return ((function()
(function()
	local _s8_value=((
function(_s8_rec)
(_s8_rec):useTemplates_(false);
(_s8_rec):currentClass_((host):classField());
return ((_s8_rec):parse_((host):source()))
end
)((smalltalk.Compiler):new()))
	template=_s8_value; return _s8_value
end)();return ((function() local _1_=(template); (host)["s8Template"]=_1_; return _1_ end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function() if ((((((template))~=nil)))==true) then return ((function()
(function()
	local _s8_value=((template):deepCopy())
	template=_s8_value; return _s8_value
end)();(function() if ((((((aBlock))~=nil)))==true) then return ((function()
return ((template):do_(aBlock))
end))() else return nil end end)();return ((function() if ((((((methodBlock))~=nil)))==true) then return ((function()
return ((methodBlock):value_(template))
end))() else return ((function()
return (template)
end))() end end)())
end))() else return nil end end)())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Compiler-templates","templateFrom: host doing: aBlock on: methodBlock\r\n\t\" Return the result of collecting mutations in template cached in host (or nil). \"\r\n\r\n\t| template |\r\n\thost isNil ifTrue: [ ^nil ].\r\n\ttemplate := host basicAt: #s8Template ifAbsent: [].\r\n\t(template isNil and: [host isKindOf: CompiledMethod]) ifTrue: [\r\n\t\ttemplate := Compiler new\r\n\t\t\tuseTemplates: false;\r\n\t\t\tcurrentClass: host classField;\r\n\t\t\tparse: host source.\r\n\t\thost basicAt: #s8Template put: template.\r\n\t].\r\n\t^template notNil ifTrue: [\r\n\t\ttemplate := template deepCopy.\r\n\t\taBlock notNil ifTrue: [ template do: aBlock ].\r\n\t\tmethodBlock notNil\r\n\t\t\tifTrue: [ methodBlock value: template ]\r\n\t\t\tifFalse: [ template ]\r\n\t]","(\r\nfunction(self,host,aBlock,methodBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal template=nil\r\n(function() if ((((((host))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function() local _1_=((host)[\"s8Template\"]);if (_1_==nil) then return ((function()\r\nreturn (nil)\r\nend)):value() end; return _1_ end)())\r\n\ttemplate=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((((template))==nil)),(function()\r\nreturn (smalltalk:is_kindOf_((host),(smalltalk.CompiledMethod)))\r\nend)))))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((\r\nfunction(_s8_rec)\r\n(_s8_rec):useTemplates_(false);\r\n(_s8_rec):currentClass_((host):classField());\r\nreturn ((_s8_rec):parse_((host):source()))\r\nend\r\n)((smalltalk.Compiler):new()))\r\n\ttemplate=_s8_value; return _s8_value\r\nend)();return ((function() local _1_=(template); (host)[\"s8Template\"]=_1_; return _1_ end)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function() if ((((((template))~=nil)))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((template):deepCopy())\r\n\ttemplate=_s8_value; return _s8_value\r\nend)();(function() if ((((((aBlock))~=nil)))==true) then return ((function()\r\nreturn ((template):do_(aBlock))\r\nend))() else return nil end end)();return ((function() if ((((((methodBlock))~=nil)))==true) then return ((function()\r\nreturn ((methodBlock):value_(template))\r\nend))() else return ((function()\r\nreturn (template)\r\nend))() end end)())\r\nend))() else return nil end end)())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"templateFrom:with:",0
,(
function(self,host,methodBlock)
return ((self):templateFrom_doing_on_(host, nil, methodBlock))
;
end
)
,"Compiler-templates","templateFrom: host with: methodBlock\r\n\t\" Return the template cached in host (or nil). \"\r\n\r\n\t^self templateFrom: host doing: nil on: methodBlock","(\r\nfunction(self,host,methodBlock)\r\nreturn ((self):templateFrom_doing_on_(host, nil, methodBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"methodsFor:",0
,(
function(self,aString)
return (((smalltalk.ClassCategoryReader):new()):class_category_(self, aString))
;
end
)
,"Tools-fileIn","methodsFor: aString\r\n\t\" FileIn - Return a chunk reader to fileIn methods onto the receiver. \"\r\n\r\n\t^ClassCategoryReader new class: self category: aString","(\r\nfunction(self,aString)\r\nreturn (((smalltalk.ClassCategoryReader):new()):class_category_(self, aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"methods",0
,(
function(self)
return ((self):methodsFor_("unclassified"))
;
end
)
,"Tools-fileIn","methods\r\n\t\" FileIn - Return a chunk reader to fileIn methods onto the receiver. \"\r\n\r\n\t^self methodsFor: #unclassified ","(\r\nfunction(self)\r\nreturn ((self):methodsFor_(\"unclassified\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"commentStamp",0
,(
function(self)
return (((smalltalk.ClassCommentReader):new()):class_(self))
;
end
)
,"Tools-fileIn","commentStamp\r\n\t\" Return a comment reader for the receiver. \"\r\n\r\n\t^ClassCommentReader new class: self","(\r\nfunction(self)\r\nreturn (((smalltalk.ClassCommentReader):new()):class_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"fileOut",0
,(
function(self)
return ((smalltalk.Exporter):fileOut_((self):instanceClass()))
;
end
)
,"Tools-fileOut","fileOut\r\n\t\" Return fileOut definition and code of a class. \"\r\n\r\n\t^Exporter fileOut: self instanceClass","(\r\nfunction(self)\r\nreturn ((smalltalk.Exporter):fileOut_((self):instanceClass()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"export",0
,(
function(self)
return ((smalltalk.Exporter):export_((self):instanceClass()))
;
end
)
,"Tools-export","export\r\n\t\" Return the result of exporting definition and source code of a class. \"\r\n\r\n\t^Exporter export: self instanceClass","(\r\nfunction(self)\r\nreturn ((smalltalk.Exporter):export_((self):instanceClass()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"references",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((smalltalk.PoolDictionary):new())
	result=_s8_value; return _s8_value
end)()
;
(self):methodsDo_((function(mth)
local refs=nil
(function()
	local _s8_value=((mth):references())
	refs=_s8_value; return _s8_value
end)();return (((refs):keys()):do_((function(key)
return (((result):at_ifAbsentPut_(key, (function()
return ((smalltalk.HashedSet):new())
end))):addAll_((refs):at_(key)))
end)))
end))
;
return ((result):collect_((function(each)
return ((each):asArray())
end)))
;
end
)
,"Tools-references","references\r\n\t\" Return all the references in the receiver. \"\r\n\r\n\t| result |\r\n\tresult := PoolDictionary new.\r\n\tself methodsDo: [:mth| | refs |\r\n\t\trefs := mth references.\r\n\t\trefs keys do: [:key|\r\n\t\t\t(result at: key ifAbsentPut: [HashedSet new])\r\n\t\t\t\taddAll: (refs at: key)\r\n\t\t]\r\n\t].\r\n\t^result collect: [:each| each asArray ]","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.PoolDictionary):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):methodsDo_((function(mth)\r\nlocal refs=nil\r\n(function()\r\n\tlocal _s8_value=((mth):references())\r\n\trefs=_s8_value; return _s8_value\r\nend)();return (((refs):keys()):do_((function(key)\r\nreturn (((result):at_ifAbsentPut_(key, (function()\r\nreturn ((smalltalk.HashedSet):new())\r\nend))):addAll_((refs):at_(key)))\r\nend)))\r\nend))\r\n;\r\nreturn ((result):collect_((function(each)\r\nreturn ((each):asArray())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"messagesSent",0
,(
function(self)
local result=nil
local cls=nil
(function()
	local _s8_value=((smalltalk.HashedSet):new())
	result=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):instanceClass())
	cls=_s8_value; return _s8_value
end)()
;
(cls):methodsDo_((function(each)
return ((result):addAll_((each):messages()))
end))
;
((cls)["$klass"]):methodsDo_((function(each)
return ((result):addAll_((each):messages()))
end))
;
return (result)
;
end
)
,"Tools-messages","messagesSent\r\n\t\" Return the messages sent by the receiver. \"\r\n\r\n\t| result cls |\r\n\tresult := HashedSet new.\r\n\tcls := self instanceClass.\r\n\tcls methodsDo: [:each| result addAll: each messages ].\r\n\tcls class methodsDo: [:each| result addAll: each messages ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\nlocal cls=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.HashedSet):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):instanceClass())\r\n\tcls=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(cls):methodsDo_((function(each)\r\nreturn ((result):addAll_((each):messages()))\r\nend))\r\n;\r\n((cls)[\"$klass\"]):methodsDo_((function(each)\r\nreturn ((result):addAll_((each):messages()))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"allMessagesSent",0
,(
function(self)
return (((self):allSubclasses()):inject_into_((self):messagesSent(), (function(total,each)
return ((
function(_s8_rec)
(_s8_rec):addAll_((each):messagesSent());
return ((_s8_rec):yourself())
end
)(total))
end)))
;
end
)
,"Tools-messages","allMessagesSent\r\n\t\" Return the messages sent by the receiver and its subclasses. \"\r\n\r\n\t^self allSubclasses\r\n\t\tinject: self messagesSent\r\n\t\tinto: [:total :each| total addAll: each messagesSent; yourself ]","(\r\nfunction(self)\r\nreturn (((self):allSubclasses()):inject_into_((self):messagesSent(), (function(total,each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):addAll_((each):messagesSent());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(total))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"implementedMessages",0
,(
function(self)
local result=nil
local cls=nil
(function()
	local _s8_value=((smalltalk.HashedSet):new())
	result=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):instanceClass())
	cls=_s8_value; return _s8_value
end)()
;
(result):addAll_((cls):selectors())
;
(result):addAll_(((cls)["$klass"]):selectors())
;
return (result)
;
end
)
,"Tools-messages","implementedMessages\r\n\t\" Return the messages implemented by the receiver. \"\r\n\r\n\t| result cls |\r\n\tresult := HashedSet new.\r\n\tcls := self instanceClass.\r\n\tresult addAll: cls selectors.\r\n\tresult addAll: cls class selectors.\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\nlocal cls=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.HashedSet):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):instanceClass())\r\n\tcls=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(result):addAll_((cls):selectors())\r\n;\r\n(result):addAll_(((cls)[\"$klass\"]):selectors())\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"allImplementedMessages",0
,(
function(self)
return (((self):allSubclasses()):inject_into_((self):implementedMessages(), (function(total,each)
return ((
function(_s8_rec)
(_s8_rec):addAll_((each):implementedMessages());
return ((_s8_rec):yourself())
end
)(total))
end)))
;
end
)
,"Tools-messages","allImplementedMessages\r\n\t\" Return the messages implemented by the receiver and its subclasses. \"\r\n\r\n\t^self allSubclasses\r\n\t\tinject: self implementedMessages\r\n\t\tinto: [:total :each| total addAll: each implementedMessages; yourself ]","(\r\nfunction(self)\r\nreturn (((self):allSubclasses()):inject_into_((self):implementedMessages(), (function(total,each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):addAll_((each):implementedMessages());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(total))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"implement:comment:as:category:",0
,(
function(self,messageHeader,aComment,sourceCode,aCategory)
local code=nil
(function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("! "),(self):name())),(function() if ((((((aCategory))==nil)))==true) then return ((function()
return (" methods")
end))() else return ((function()
return (smalltalk:object_comma_((" methodsFor: "),(aCategory):asLiteral()))
end))() end end)()))," !\r\n")),messageHeader)),(function() if ((((((aComment))==nil)))==true) then return ((function()
return ("")
end))() else return ((function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("\r\n\t\" "),aComment))," \""))
end))() end end)())),"\r\n\r\n\t")),sourceCode)),"! !\r\n\r\n"))
	code=_s8_value; return _s8_value
end)()
;
return ((
function(_s8_rec)
(_s8_rec):fileIn();
return ((_s8_rec):yourself())
end
)(code))
;
end
)
,"Builders-implementing","implement: messageHeader\r\n\tcomment: aComment\r\n\tas: sourceCode\r\n\tcategory: aCategory\r\n\t\" Private - Implement a method in the receiver.\r\n\tNote: sourceCode must be the source code in chunk format.\r\n\t\"\r\n\r\n\t| code |\r\n\tcode := '! ',self name,\r\n\t\t(aCategory isNil\r\n\t\t\tifTrue: [ ' methods' ]\r\n\t\t\tifFalse: [ ' methodsFor: ', aCategory asLiteral ])\r\n,' !\r\n',messageHeader, (aComment isNil ifTrue: [ '' ] ifFalse: ['\r\n\t\" ', aComment, ' \"']),'\r\n\r\n\t',sourceCode ,'! !\r\n\r\n'.\t^code fileIn; yourself","(\r\nfunction(self,messageHeader,aComment,sourceCode,aCategory)\r\nlocal code=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"! \"),(self):name())),(function() if ((((((aCategory))==nil)))==true) then return ((function()\r\nreturn (\" methods\")\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((\" methodsFor: \"),(aCategory):asLiteral()))\r\nend))() end end)())),\" !\\r\\n\")),messageHeader)),(function() if ((((((aComment))==nil)))==true) then return ((function()\r\nreturn (\"\")\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"\\r\\n\\t\\\" \"),aComment)),\" \\\"\"))\r\nend))() end end)())),\"\\r\\n\\r\\n\\t\")),sourceCode)),\"! !\\r\\n\\r\\n\"))\r\n\tcode=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):fileIn();\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(code))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"implement:comment:as:",0
,(
function(self,messageHeader,aComment,sourceCode)
return ((self):implement_comment_as_category_(messageHeader, aComment, sourceCode, nil))
;
end
)
,"Builders-implementing","implement: messageHeader comment: aComment as: sourceCode\r\n\t\" Private - Implement a method in the receiver. \"\r\n\r\n\t^self\r\n\t\timplement: messageHeader\r\n\t\tcomment: aComment\r\n\t\tas: sourceCode\r\n\t\tcategory: nil","(\r\nfunction(self,messageHeader,aComment,sourceCode)\r\nreturn ((self):implement_comment_as_category_(messageHeader, aComment, sourceCode, nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"implement:as:",0
,(
function(self,messageHeader,sourceCode)
return ((self):implement_comment_as_category_(messageHeader, nil, sourceCode, nil))
;
end
)
,"Builders-implementing","implement: messageHeader as: sourceCode\r\n\t\" Private - Implement a method in the receiver. \"\r\n\r\n\t^self\r\n\t\timplement: messageHeader\r\n\t\tcomment: nil\r\n\t\tas: sourceCode\r\n\t\tcategory: nil","(\r\nfunction(self,messageHeader,sourceCode)\r\nreturn ((self):implement_comment_as_category_(messageHeader, nil, sourceCode, nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"implement:as:category:",0
,(
function(self,messageHeader,sourceCode,aCategory)
return ((self):implement_comment_as_category_(messageHeader, nil, sourceCode, aCategory))
;
end
)
,"Builders-implementing","implement: messageHeader as: sourceCode category: aCategory\r\n\t\" Private - Implement a method in the receiver. \"\r\n\r\n\t^self\r\n\t\timplement: messageHeader\r\n\t\tcomment: nil\r\n\t\tas: sourceCode\r\n\t\tcategory: aCategory","(\r\nfunction(self,messageHeader,sourceCode,aCategory)\r\nreturn ((self):implement_comment_as_category_(messageHeader, nil, sourceCode, aCategory))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"class:implement:comment:as:category:",0
,(
function(self,aClass,message,aComment,sourceCode,aCategory)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aClass):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):class_implement_comment_as_category_((aClass):evaluateIn_(nil), message, aComment, sourceCode, aCategory))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aClass):implement_comment_as_category_(message, aComment, sourceCode, aCategory))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Builders-implementing","class: aClass implement: message\r\n\tcomment: aComment as: sourceCode\r\n\tcategory: aCategory\r\n\t\" Private - Implement a method in aClass. \"\r\n\r\n\taClass isString ifTrue: [\r\n\t\t^self\r\n\t\t\tclass: (aClass evaluateIn: nil)\r\n\t\t\timplement: message\r\n\t\t\tcomment: aComment\r\n\t\t\tas: sourceCode\r\n\t\t\tcategory: aCategory\r\n\t].\r\n\t^aClass\r\n\t\timplement: message\r\n\t\tcomment: aComment\r\n\t\tas: sourceCode\r\n\t\tcategory: aCategory","(\r\nfunction(self,aClass,message,aComment,sourceCode,aCategory)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aClass):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):class_implement_comment_as_category_((aClass):evaluateIn_(nil), message, aComment, sourceCode, aCategory))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aClass):implement_comment_as_category_(message, aComment, sourceCode, aCategory))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"build:doing:",0
,(
function(self,token,aBlock)
return ((smalltalk.APIBuilder):build_doing_(token, aBlock))
;
end
)
,"Builders-private","build: token doing: aBlock\r\n\t\" Process a (build...: spec) definition marking methods generated while evaluating aBlock.\r\n\tReturn the result of evaluating aBlock. \"\r\n\r\n\t^APIBuilder build: token doing: aBlock","(\r\nfunction(self,token,aBlock)\r\nreturn ((smalltalk.APIBuilder):build_doing_(token, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildAPI:",0
,(
function(self,specs)
(specs):do_((function(spec)
return ((function() if ((((spec):isString()))==true) then return ((function()
return ((spec):evaluateIn_(self))
end))() else return ((function()
return ((self):perform_withArguments_((spec):first(), (spec):copyFrom_to_((2), (spec):size())))
end))() end end)())
end))
;
return (self)
;
end
)
,"Builders-obsolete","buildAPI: specs\r\n\t\" Generate code to implement API protocoll defined by specs.\r\n\tWARNING: OBSOLETE - this builder generate changes that will not be filedOut to builder code.\r\n\tspecs must be an array of :\r\n\t\t- string : it will be considered as an evaluable expression to be executed in the context of the receiver.\r\n\t\t- anArray : of #( selector arg1 arg2 arg3 arg4... ) to be performed for code generation/setup of the receiver\r\n\t\t\twhere selector is frequently a build...: selector.\r\n\t\"\r\n\r\n\tspecs do: [:spec|\r\n\t\tspec isString\r\n\t\tifTrue: [ spec evaluateIn: self ]\r\n\t\tifFalse: [\r\n\t\t\tself\tperform: spec first\r\n\t\t\t\twithArguments: (spec copyFrom: 2 to: spec size)\r\n\t\t]\r\n\t]","(\r\nfunction(self,specs)\r\n(specs):do_((function(spec)\r\nreturn ((function() if ((((spec):isString()))==true) then return ((function()\r\nreturn ((spec):evaluateIn_(self))\r\nend))() else return ((function()\r\nreturn ((self):perform_withArguments_((spec):first(), (spec):copyFrom_to_((2), (spec):size())))\r\nend))() end end)())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"stMessageHeader:",0
,(
function(self,msgHeader)
return (msgHeader)
;
end
)
,"Builders-private","stMessageHeader: msgHeader\r\n\t\" Private - Return a message header string with nice smalltalk looking.\r\n\tDefault implementation leave the message header as is.\r\n\tThis method can be refined by subclasses to transform message pattern for better code generation.\r\n\t\"\r\n\r\n\t^msgHeader","(\r\nfunction(self,msgHeader)\r\nreturn (msgHeader)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildAccessor:",0
,(
function(self,accessor)
return ((self):build_doing_(("Accessor"):_minus_gt(accessor), (function()
return ((function() if ((((smalltalk:boolean_or_(((accessor):isString()),(function()
return ((smalltalk:boolean_and_(((accessor):isArray()),(function()
return ((smalltalk:boolean_and_(((((accessor):size()) > (1))),(function()
return ((smalltalk:isObject_equalTo_(((accessor):second()),"initialize")))
end))))
end))))
end)))))==true) then return ((function()
return (smalltalk:object_comma_(((self):buildGetter_(accessor)),(self):buildSetter_(accessor)))
end))() else return ((function()
return ((accessor):compose_((function(tuple)
return ((self):implement_comment_as_category_((tuple):first(), (tuple):at_((2)), (tuple):at_((3)), "accessing"))
end)))
end))() end end)())
end)))
;
end
)
,"Builders-api","buildAccessor: accessor\r\n\t\" Build (instance) method to access native property. \"\r\n\r\n\t^self build: #Accessor -> accessor doing: [\r\n\t\t(accessor isString or: [\r\n\t\t\taccessor isArray and: [\r\n\t\t\taccessor size > 1 and: [\r\n\t\t\taccessor second = #initialize ] ]\r\n\t\t]) ifTrue: [\r\n\t\t\t(self buildGetter: accessor)\r\n\t\t\t,(self buildSetter: accessor)\r\n\t\t] ifFalse: [\r\n\t\t\taccessor compose: [:tuple|\r\n\t\t\t\tself implement: tuple first comment: (tuple at: 2)\r\n\t\t\t\t      as: (tuple at: 3) category: #accessing\r\n\t\t\t]\r\n\t\t]\r\n\t]","(\r\nfunction(self,accessor)\r\nreturn ((self):build_doing_((\"Accessor\"):_minus_gt(accessor), (function()\r\nreturn ((function() if ((((smalltalk:boolean_or_(((accessor):isString()),(function()\r\nreturn ((smalltalk:boolean_and_(((accessor):isArray()),(function()\r\nreturn ((smalltalk:boolean_and_(((((accessor):size()) > (1))),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((accessor):second()),\"initialize\")))\r\nend))))\r\nend))))\r\nend)))))==true) then return ((function()\r\nreturn (smalltalk:object_comma_(((self):buildGetter_(accessor)),(self):buildSetter_(accessor)))\r\nend))() else return ((function()\r\nreturn ((accessor):compose_((function(tuple)\r\nreturn ((self):implement_comment_as_category_((tuple):first(), (tuple):at_((2)), (tuple):at_((3)), \"accessing\"))\r\nend)))\r\nend))() end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildGetterCode:",0
,(
function(self,aSelector)
return (smalltalk:object_comma_(("^"),aSelector))
;
end
)
,"Builders-private","buildGetterCode: aSelector\r\n\t\" Private - Return source code to get aSelector. \"\r\n\r\n\t^'^' ,aSelector","(\r\nfunction(self,aSelector)\r\nreturn (smalltalk:object_comma_((\"^\"),aSelector))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildGetter:",0
,(
function(self,spec)
return ((self):build_doing_(("Getter"):_minus_gt(spec), (function()
return ((function() if ((((smalltalk:boolean_and_(((spec):isArray()),(function()
return ((smalltalk:boolean_and_(((((spec):size()) > (1))),(function()
return ((smalltalk:isObject_equalTo_(((spec):second()),"initialize")))
end))))
end)))))==true) then return ((function()
return ((self):buildGetter_initialize_((spec):first(), (spec):last()))
end))() else return ((function()
return ((self):implement_comment_as_category_((self):stMessageHeader_(spec), smalltalk:object_comma_((smalltalk:object_comma_(("Return the "),spec))," of the receiver."), (self):buildGetterCode_(spec), "accessing"))
end))() end end)())
end)))
;
end
)
,"Builders-api","buildGetter: spec\r\n\t\" Build method for getter. \"\r\n\r\n\t^self build: #Getter -> spec doing: [\r\n\t\t(spec isArray and: [ spec size > 1 and: [\r\n\t\t spec second = #initialize ]]) ifTrue: [\r\n\t\t\tself buildGetter: spec first initialize: spec last\r\n\t\t] ifFalse: [\r\n\t\t\tself\r\n\t\t\t\timplement: (self stMessageHeader: spec)\r\n\t\t\t\tcomment: 'Return the ' ,spec ,' of the receiver.'\r\n\t\t\t\tas: (self buildGetterCode: spec) category: #accessing\r\n\t\t]\r\n\t]","(\r\nfunction(self,spec)\r\nreturn ((self):build_doing_((\"Getter\"):_minus_gt(spec), (function()\r\nreturn ((function() if ((((smalltalk:boolean_and_(((spec):isArray()),(function()\r\nreturn ((smalltalk:boolean_and_(((((spec):size()) > (1))),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((spec):second()),\"initialize\")))\r\nend))))\r\nend)))))==true) then return ((function()\r\nreturn ((self):buildGetter_initialize_((spec):first(), (spec):last()))\r\nend))() else return ((function()\r\nreturn ((self):implement_comment_as_category_((self):stMessageHeader_(spec), smalltalk:object_comma_((smalltalk:object_comma_((\"Return the \"),spec)),\" of the receiver.\"), (self):buildGetterCode_(spec), \"accessing\"))\r\nend))() end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildInitialize:with:",0
,(
function(self,aSelector,initializer)
return ((self):implement_comment_as_category_(smalltalk:object_comma_(("initialize"),(aSelector):asSmalltalkName_(true)), smalltalk:object_comma_((smalltalk:object_comma_(("Private - Initialize the "),aSelector))," of the receiver."), initializer, "initialize"))
;
end
)
,"Builders-private","buildInitialize: aSelector with: initializer\r\n\t\" Private - Build method to initialize instance variable aSelector. \"\r\n\r\n\t^self\r\n\t\timplement: #initialize ,(aSelector asSmalltalkName: true)\r\n\t\tcomment: 'Private - Initialize the ' ,aSelector ,' of the receiver.'\r\n\t\tas: initializer\r\n\t\tcategory: #initialize","(\r\nfunction(self,aSelector,initializer)\r\nreturn ((self):implement_comment_as_category_(smalltalk:object_comma_((\"initialize\"),(aSelector):asSmalltalkName_(true)), smalltalk:object_comma_((smalltalk:object_comma_((\"Private - Initialize the \"),aSelector)),\" of the receiver.\"), initializer, \"initialize\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildGetter:initialize:",0
,(
function(self,aSelector,initializer)
local code=nil
(function()
	local _s8_value=((function() if ((((smalltalk:isObject_equalTo_((initializer),"initialize"))))==true) then return ((function()
return ("^self implementedBySubclass: #initialize")
end))() else return ((function()
return ((function() if ((((initializer):notEmpty()))==true) then return ((function()
return ((function() if ((((smalltalk:isObject_equalTo_(((initializer):first()),"."))))==true) then return ((function()
return ((initializer):copyFrom_to_((2), (initializer):size()))
end))() else return ((function()
return (smalltalk:object_comma_((smalltalk:object_comma_((aSelector)," := ")),(function() if ((((smalltalk:boolean_and_((((initializer):first()):isUppercase()),(function()
return ((smalltalk:isObject_equalTo_((((initializer):asArrayOfSubstrings()):size()),(1))))
end)))))==true) then return ((function()
return (smalltalk:object_comma_((initializer)," new"))
end))() else return ((function()
return (initializer)
end))() end end)()))
end))() end end)())
end))() else return nil end end)())
end))() end end)())
	code=_s8_value; return _s8_value
end)()
;
return (smalltalk:object_comma_(((function() if ((((((code))~=nil)))==true) then return ((function()
return ((self):buildInitialize_with_(aSelector, code))
end))() else return ((function()
return ("")
end))() end end)()),(self):implement_comment_as_category_((self):stMessageHeader_(aSelector), smalltalk:object_comma_((smalltalk:object_comma_(("Return the "),aSelector))," of the receiver."), smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((aSelector)," isNil ifTrue: [ self ")),"initialize")),(aSelector):asSmalltalkName_(true)))," ].\r\n\t^")),aSelector), "accessing")))
;
end
)
,"Builders-private","buildGetter: aSelector initialize: initializer\r\n\t\" Private - Build method to get instance aSelector w/deferred initialization mode.\r\n\tNote: #initializer can be:\r\n\t\t#initialize to generate default/deferred implementation.\r\n\t\tempty string to do NOT generate #initialize... method.\r\n\t\tsmalltalk code starting with $. to use that code as method implementation.\r\n\t\tsmalltalk expression for asignment (to be used as assignment to instance).\r\n\t\tGlobalName if code do not have blanks and start with uppercase, it will be used as class to instantiate during initialization.\r\n\t\"\r\n\r\n\t| code |\r\n\tcode := initializer = #initialize\r\n\t  ifTrue: [ '^self implementedBySubclass: #initialize' ] ifFalse: [\r\n\t\tinitializer notEmpty ifTrue: [\r\n\t\t\tinitializer first = $.\r\n\t\t\tifTrue: [ initializer copyFrom: 2 to: initializer size ] ifFalse: [\r\n\t\t\t\taSelector ,' := ' ,(\r\n\t\t\t\t\t(initializer first isUppercase and: [ initializer asArrayOfSubstrings size = 1 ])\r\n\t\t\t\t\tifTrue: [ initializer ,' new' ]\r\n\t\t\t\t\tifFalse: [ initializer ])\r\n\t\t\t]\r\n\t\t]\r\n\t  ].\r\n\t^(code notNil ifTrue: [ self buildInitialize: aSelector with: code ] ifFalse: [ '' ])\r\n\t,(self\r\n\t\timplement: (self stMessageHeader: aSelector)\r\n\t\tcomment: 'Return the ' ,aSelector ,' of the receiver.'\r\n\t\tas: aSelector,' isNil ifTrue: [ self ' , #initialize ,(aSelector asSmalltalkName: true) ,' ].\r\n\t^' ,aSelector category: #accessing)","(\r\nfunction(self,aSelector,initializer)\r\nlocal code=nil\r\n(function()\r\n\tlocal _s8_value=((function() if ((((smalltalk:isObject_equalTo_((initializer),\"initialize\"))))==true) then return ((function()\r\nreturn (\"^self implementedBySubclass: #initialize\")\r\nend))() else return ((function()\r\nreturn ((function() if ((((initializer):notEmpty()))==true) then return ((function()\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_(((initializer):first()),\".\"))))==true) then return ((function()\r\nreturn ((initializer):copyFrom_to_((2), (initializer):size()))\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((aSelector),\" := \")),(function() if ((((smalltalk:boolean_and_((((initializer):first()):isUppercase()),(function()\r\nreturn ((smalltalk:isObject_equalTo_((((initializer):asArrayOfSubstrings()):size()),(1))))\r\nend)))))==true) then return ((function()\r\nreturn (smalltalk:object_comma_((initializer),\" new\"))\r\nend))() else return ((function()\r\nreturn (initializer)\r\nend))() end end)()))\r\nend))() end end)())\r\nend))() else return nil end end)())\r\nend))() end end)())\r\n\tcode=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (smalltalk:object_comma_(((function() if ((((((code))~=nil)))==true) then return ((function()\r\nreturn ((self):buildInitialize_with_(aSelector, code))\r\nend))() else return ((function()\r\nreturn (\"\")\r\nend))() end end)()),(self):implement_comment_as_category_((self):stMessageHeader_(aSelector), smalltalk:object_comma_((smalltalk:object_comma_((\"Return the \"),aSelector)),\" of the receiver.\"), smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((aSelector),\" isNil ifTrue: [ self \")),\"initialize\")),(aSelector):asSmalltalkName_(true))),\" ].\\r\\n\\t^\")),aSelector), \"accessing\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildSetterCode:",0
,(
function(self,aSelector)
return (smalltalk:object_comma_((aSelector)," := anObject"))
;
end
)
,"Builders-private","buildSetterCode: aSelector\r\n\t\" Private - Return source code to set aSelector. \"\r\n\r\n\t^aSelector ,' := anObject'","(\r\nfunction(self,aSelector)\r\nreturn (smalltalk:object_comma_((aSelector),\" := anObject\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildSetter:",0
,(
function(self,spec)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:boolean_and_(((spec):isArray()),(function()
return ((smalltalk:boolean_and_(((((spec):size()) > (1))),(function()
return ((smalltalk:isObject_equalTo_(((spec):second()),"initialize")))
end))))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildSetter_((spec):first()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("Setter"):_minus_gt(spec), (function()
return ((self):implement_comment_as_category_(smalltalk:object_comma_(((self):stMessageHeader_(spec)),": anObject"), smalltalk:object_comma_((smalltalk:object_comma_(("Set the "),spec))," of the receiver."), (self):buildSetterCode_(spec), "accessing"))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Builders-api","buildSetter: spec\r\n\t\" Build method for setter. \"\r\n\r\n\t(spec isArray and: [ spec size > 1 and: [\r\n\t spec second = #initialize ]]) ifTrue: [\r\n\t\t^self buildSetter: spec first\r\n\t].\r\n\t^self build: #Setter -> spec doing: [\r\n\t\tself\r\n\t\t\timplement: (self stMessageHeader: spec) ,': anObject'\r\n\t\t\tcomment: 'Set the ' ,spec ,' of the receiver.'\r\n\t\t\tas: (self buildSetterCode: spec)\r\n\t\t\tcategory: #accessing\r\n\t]","(\r\nfunction(self,spec)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:boolean_and_(((spec):isArray()),(function()\r\nreturn ((smalltalk:boolean_and_(((((spec):size()) > (1))),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((spec):second()),\"initialize\")))\r\nend))))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildSetter_((spec):first()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"Setter\"):_minus_gt(spec), (function()\r\nreturn ((self):implement_comment_as_category_(smalltalk:object_comma_(((self):stMessageHeader_(spec)),\": anObject\"), smalltalk:object_comma_((smalltalk:object_comma_((\"Set the \"),spec)),\" of the receiver.\"), (self):buildSetterCode_(spec), \"accessing\"))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildGetters:",0
,(
function(self,specs)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((specs):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildGetters_((specs):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("Getters"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildGetter_(each))
end)))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Builders-api","buildGetters: specs\r\n\t\" Build methods to get instances. \"\r\n\r\n\tspecs isString ifTrue: [ ^self buildGetters: specs asArrayOfSubstrings ].\r\n\t^self build: #Getters -> specs doing: [\r\n\t\tspecs compose: [:each| self buildGetter: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((specs):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildGetters_((specs):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"Getters\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildGetter_(each))\r\nend)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildSetters:",0
,(
function(self,specs)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((specs):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildSetters_((specs):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("Setters"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildSetter_(each))
end)))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Builders-api","buildSetters: specs\r\n\t\" Build methods to set instances. \"\r\n\r\n\tspecs isString ifTrue: [ ^self buildSetters: specs asArrayOfSubstrings ].\r\n\t^self build: #Setters -> specs doing: [\r\n\t\tspecs compose: [:each| self buildSetter: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((specs):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildSetters_((specs):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"Setters\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildSetter_(each))\r\nend)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildAccessors:",0
,(
function(self,specs)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((specs):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildAccessors_((specs):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("Accessors"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildAccessor_(each))
end)))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Builders-api","buildAccessors: specs\r\n\t\" Build methods to access instances. \"\r\n\r\n\tspecs isString ifTrue: [ ^self buildAccessors: specs asArrayOfSubstrings ].\r\n\t^self build: #Accessors -> specs doing: [\r\n\t\tspecs compose: [:each| self buildAccessor: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((specs):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildAccessors_((specs):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"Accessors\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildAccessor_(each))\r\nend)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildAccessors",0
,(
function(self)
return ((self):build_doing_(("Accessors"):_minus_gt(smalltalk:newArray({})), (function()
return ((self):buildAccessors_((self):instanceVariableNames()))
end)))
;
end
)
,"Builders-api","buildAccessors\r\n\t\" Private - Build methods to access all instance variables. \"\r\n\r\n\t^self build: #Accessors -> #() doing: [\r\n\t\tself buildAccessors: self instanceVariableNames\r\n\t]","(\r\nfunction(self)\r\nreturn ((self):build_doing_((\"Accessors\"):_minus_gt(smalltalk:newArray({})), (function()\r\nreturn ((self):buildAccessors_((self):instanceVariableNames()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"buildPool:with:",0
,(
function(self,poolName,spec)
return ((self):build_doing_(("Pool"):_minus_gt(spec), (function()
return ((
function(_s8_rec)
(_s8_rec):definePool_with_(poolName, spec);
(_s8_rec):printDefinePool_with_(poolName, spec);
return ((_s8_rec):yourself())
end
)((smalltalk.Smalltalk):current()))
end)))
;
end
)
,"Builders-api","buildPool: poolName with: spec\r\n\t\" Private - Build a PoolDictionary with spec (and log definition in log stream). \"\r\n\r\n\t^self build: #Pool -> spec doing: [\r\n\t\tSmalltalk current\r\n\t\t\tdefinePool: poolName with: spec;\r\n\t\t\tprintDefinePool: poolName with: spec;\r\n\t\t\tyourself\r\n\t]","(\r\nfunction(self,poolName,spec)\r\nreturn ((self):build_doing_((\"Pool\"):_minus_gt(spec), (function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):definePool_with_(poolName, spec);\r\n(_s8_rec):printDefinePool_with_(poolName, spec);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.Smalltalk):current()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Behavior,"__tostring",0
,(
function(self)
return (smalltalk:toString_((self)))
;
end
)
,"Lua-printing","__tostring\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class["$klass"],"categories",0
,(
function(self)
return ((((smalltalk.Smalltalk):classes()):collect_((function(each)
return ((each):category())
end))):asHashedSet())
;
end
)
,"export","categories\r\n\t\" Return a collection with all defined categories. \"\r\n\r\n\t^(Smalltalk classes collect: [:each | each category ]) asHashedSet","(\r\nfunction(self)\r\nreturn ((((smalltalk.Smalltalk):classes()):collect_((function(each)\r\nreturn ((each):category())\r\nend))):asHashedSet())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class["$klass"],"uncategorized",0
,(
function(self)
return (((smalltalk.Smalltalk):classes()):select_((function(each)
return ((smalltalk:isObject_equalTo_(((each):category()),"unclassified")))
end)))
;
end
)
,"export","uncategorized\r\n\t\" Return a collection with classes without category. \"\r\n\r\n\t^Smalltalk classes select: [:each | each category = #unclassified ]","(\r\nfunction(self)\r\nreturn (((smalltalk.Smalltalk):classes()):select_((function(each)\r\nreturn ((smalltalk:isObject_equalTo_(((each):category()),\"unclassified\")))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"category",0
,(
function(self)
return (((self)["$category"]))
;
end
)
,"accessing","category\r\n\t\" Return the category of the receiver. \"\r\n\r\n\t^self basicAt: '$category'","(\r\nfunction(self)\r\nreturn (((self)[\"$category\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"category:",0
,(
function(self,aString)
(function() local _1_=(aString); (self)["$category"]=_1_; return _1_ end)()
;
return (self)
;
end
)
,"accessing","category: aString\r\n\t\" Set the category of the receiver. \"\r\n\r\n\tself basicAt: '$category' put: aString","(\r\nfunction(self,aString)\r\n(function() local _1_=(aString); (self)[\"$category\"]=_1_; return _1_ end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"classVariables",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local names=nil
(function()
	local _s8_value=((function() local _1_=((self)["$classVariableNames"]);if (_1_==nil) then return ((function()
return (nil)
end)):value() end; return _1_ end)())
	names=_s8_value; return _s8_value
end)()
;
(function() if ((((((names))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self)["$classVariables"]))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(self):basicDelete_("$classVariableNames")
;
(self):classVariableNames_(names)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self)["$classVariables"]))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","classVariables\r\n\t\" Return the class variables of the receiver. \"\r\n\r\n\t| names |\r\n\tnames := self basicAt: '$classVariableNames' ifAbsent: [].\r\n\tnames isNil ifTrue: [ ^self basicAt: '$classVariables' ].\r\n\tself basicDelete: '$classVariableNames'.\r\n\tself classVariableNames: names.\r\n\t^self basicAt: '$classVariables'","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal names=nil\r\n(function()\r\n\tlocal _s8_value=((function() local _1_=((self)[\"$classVariableNames\"]);if (_1_==nil) then return ((function()\r\nreturn (nil)\r\nend)):value() end; return _1_ end)())\r\n\tnames=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((names))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self)[\"$classVariables\"]))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self):basicDelete_(\"$classVariableNames\")\r\n;\r\n(self):classVariableNames_(names)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self)[\"$classVariables\"]))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"classVariables:",0
,(
function(self,aDictionary)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:boolean_or_(((((aDictionary))==nil)),(function()
return ((aDictionary):isEmpty())
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):basicDelete_("$classVariables"))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function() local _1_=(aDictionary); (self)["$classVariables"]=_1_; return _1_ end)())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","classVariables: aDictionary\r\n\t\" Set the class variables of the receiver. \"\r\n\r\n\t(aDictionary isNil or: [ aDictionary isEmpty ]) ifTrue: [\r\n\t\t^self basicDelete: '$classVariables'\r\n\t].\r\n\t^self basicAt: '$classVariables' put: aDictionary","(\r\nfunction(self,aDictionary)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:boolean_or_(((((aDictionary))==nil)),(function()\r\nreturn ((aDictionary):isEmpty())\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):basicDelete_(\"$classVariables\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function() local _1_=(aDictionary); (self)[\"$classVariables\"]=_1_; return _1_ end)())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"poolDictionaries",0
,(
function(self)
return (((self)["$poolDictionaries"]))
;
end
)
,"accessing","poolDictionaries\r\n\t\" Return the poolDictionaries of the receiver (anArray or nil). \"\r\n\r\n\t^self basicAt: '$poolDictionaries'","(\r\nfunction(self)\r\nreturn (((self)[\"$poolDictionaries\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"poolDictionaries:",0
,(
function(self,anArray)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:boolean_and_(((((anArray))~=nil)),(function()
return ((anArray):isEmpty())
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):poolDictionaries_(nil))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function() local _1_=(anArray); (self)["$poolDictionaries"]=_1_; return _1_ end)())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","poolDictionaries: anArray\r\n\t\" Set the poolDictionaries of the receiver. \"\r\n\r\n\t(anArray notNil and: [ anArray isEmpty ]) ifTrue: [\r\n\t\t^self poolDictionaries: nil\r\n\t].\r\n\t^self basicAt: '$poolDictionaries' put: anArray","(\r\nfunction(self,anArray)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:boolean_and_(((((anArray))~=nil)),(function()\r\nreturn ((anArray):isEmpty())\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):poolDictionaries_(nil))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function() local _1_=(anArray); (self)[\"$poolDictionaries\"]=_1_; return _1_ end)())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"classVariableNames",0
,(
function(self)
return ((function() if (((((((self):classVariables()))==nil)))==true) then return ((function()
return ((smalltalk.Array):new())
end))() else return ((function()
return (((self):classVariables()):keys())
end))() end end)())
;
end
)
,"accessing","classVariableNames\r\n\t\" Return the class variable names of the receiver. \"\r\n\r\n\t^self classVariables isNil\r\n\t\tifTrue: [ Array new ]\r\n\t\tifFalse: [ self classVariables keys ]","(\r\nfunction(self)\r\nreturn ((function() if (((((((self):classVariables()))==nil)))==true) then return ((function()\r\nreturn ((smalltalk.Array):new())\r\nend))() else return ((function()\r\nreturn (((self):classVariables()):keys())\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"classVariablesString",0
,(
function(self)
return (((self):classVariableNames()):join_(" "))
;
end
)
,"accessing","classVariablesString\r\n\t\" Return the class variable names of the receiver as aString. \"\r\n\r\n\t^self classVariableNames join: ' '","(\r\nfunction(self)\r\nreturn (((self):classVariableNames()):join_(\" \"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"poolDictionaryNames",0
,(
function(self)
return ((function() if (((((((self):poolDictionaries()))==nil)))==true) then return ((function()
return ((smalltalk.Array):new())
end))() else return ((function()
return ((self):poolDictionaries())
end))() end end)())
;
end
)
,"accessing","poolDictionaryNames\r\n\t\" Return the pool dictionary names of the receiver. \"\r\n\r\n\t^self poolDictionaries isNil\r\n\t\tifTrue: [ Array new ]\r\n\t\tifFalse: [ self poolDictionaries ]","(\r\nfunction(self)\r\nreturn ((function() if (((((((self):poolDictionaries()))==nil)))==true) then return ((function()\r\nreturn ((smalltalk.Array):new())\r\nend))() else return ((function()\r\nreturn ((self):poolDictionaries())\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"poolDictionariesString",0
,(
function(self)
return (((self):poolDictionaryNames()):join_(" "))
;
end
)
,"accessing","poolDictionariesString\r\n\t\" Return the poolDictionary names of the receiver as aString. \"\r\n\r\n\t^self poolDictionaryNames join: ' '","(\r\nfunction(self)\r\nreturn (((self):poolDictionaryNames()):join_(\" \"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"classVariableNames:",0
,(
function(self,classVarNames)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local names=nil
local dictionary=nil
local old=nil
(function() if ((((((classVarNames))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):classVariables_(nil))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((function() if ((((classVarNames):isString()))==true) then return ((function()
return ((classVarNames):asArrayOfSubstrings())
end))() else return ((function()
return (classVarNames)
end))() end end)())
	names=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.PoolDictionary):new())
	dictionary=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):classVariables())
	old=_s8_value; return _s8_value
end)()
;
(function() if ((((((old))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=(dictionary)
	old=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(names):do_((function(each)
return ((dictionary):at_put_(each, (old):at_ifAbsent_(each, (function()
return (nil)
end))))
end))
;
(self):classVariables_(dictionary)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","classVariableNames: classVarNames\r\n\t\" Set the variable names defined in the receiver. \"\r\n\r\n\t| names dictionary old |\r\n\tclassVarNames isNil ifTrue: [ ^self classVariables: nil ].\r\n\tnames := classVarNames isString\r\n\t\tifTrue: [ classVarNames asArrayOfSubstrings ]\r\n\t\tifFalse: [ classVarNames ].\r\n\tdictionary := PoolDictionary new.\r\n\told := self classVariables. old isNil ifTrue: [ old := dictionary ].\r\n\tnames do: [:each|\r\n\t\tdictionary at: each put: (old at: each ifAbsent: [ nil ])\r\n\t].\r\n\tself classVariables: dictionary.","(\r\nfunction(self,classVarNames)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal names=nil\r\nlocal dictionary=nil\r\nlocal old=nil\r\n(function() if ((((((classVarNames))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):classVariables_(nil))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function() if ((((classVarNames):isString()))==true) then return ((function()\r\nreturn ((classVarNames):asArrayOfSubstrings())\r\nend))() else return ((function()\r\nreturn (classVarNames)\r\nend))() end end)())\r\n\tnames=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.PoolDictionary):new())\r\n\tdictionary=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):classVariables())\r\n\told=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((old))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(dictionary)\r\n\told=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(names):do_((function(each)\r\nreturn ((dictionary):at_put_(each, (old):at_ifAbsent_(each, (function()\r\nreturn (nil)\r\nend))))\r\nend))\r\n;\r\n(self):classVariables_(dictionary)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"poolDictionaryNames:",0
,(
function(self,poolVarNames)
(self):poolDictionaries_((function() if ((((poolVarNames):isString()))==true) then return ((function()
return ((poolVarNames):asArrayOfSubstrings())
end))() else return ((function()
return (poolVarNames)
end))() end end)())
;
return (self)
;
end
)
,"accessing","poolDictionaryNames: poolVarNames\r\n\t\" Set the pool dictionary names known to the receiver. \"\r\n\r\n\tself poolDictionaries: (poolVarNames isString\r\n\t\tifTrue: [ poolVarNames asArrayOfSubstrings ]\r\n\t\tifFalse: [ poolVarNames ]).","(\r\nfunction(self,poolVarNames)\r\n(self):poolDictionaries_((function() if ((((poolVarNames):isString()))==true) then return ((function()\r\nreturn ((poolVarNames):asArrayOfSubstrings())\r\nend))() else return ((function()\r\nreturn (poolVarNames)\r\nend))() end end)())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:instanceVariableNames:",0
,(
function(self,aString,aString2)
return ((self):subclass_instanceVariableNames_classVariableNames_(aString, aString2, ""))
;
end
)
,"instantiation","subclass: aString instanceVariableNames: aString2\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: ''","(\r\nfunction(self,aString,aString2)\r\nreturn ((self):subclass_instanceVariableNames_classVariableNames_(aString, aString2, \"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:category:",0
,(
function(self,aString,aCategory)
return ((self):subclass_instanceVariableNames_category_(aString, "", aCategory))
;
end
)
,"instantiation","subclass: aString category: aCategory\r\n\t\" Create a subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: ''\r\n\t\tcategory: aCategory","(\r\nfunction(self,aString,aCategory)\r\nreturn ((self):subclass_instanceVariableNames_category_(aString, \"\", aCategory))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:instanceVariableNames:category:",0
,(
function(self,aString,aString2,aString3)
return (((smalltalk.ClassBuilder):new()):superclass_subclass_instanceVariableNames_category_(self, aString, aString2, aString3))
;
end
)
,"instantiation","subclass: aString instanceVariableNames: aString2 category: aString3\r\n\t\" Create a subclass of the receiver. \"\r\n\r\n\t^ClassBuilder new\r\n\t\tsuperclass: self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tcategory: aString3","(\r\nfunction(self,aString,aString2,aString3)\r\nreturn (((smalltalk.ClassBuilder):new()):superclass_subclass_instanceVariableNames_category_(self, aString, aString2, aString3))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"isClass",0
,(
function(self)
return (true)
;
end
)
,"testing","isClass\r\n\t\" Return true if the receiver is a Class. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"printString",0
,(
function(self)
return ((self):name())
;
end
)
,"printing","printString\r\n\t\" Return a printable representation of the receiver. \"\r\n\r\n\t^self name","(\r\nfunction(self)\r\nreturn ((self):name())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"instanceClass",0
,(
function(self)
return (self)
;
end
)
,"accessing","instanceClass\r\n\t\" Return the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"rename:",0
,(
function(self,aString)
((self):smalltalk()):renameClass(self, aString)
;
return (self)
;
end
)
,"renaming","rename: aString\r\n\t\" Rename the receiver to aString. \"\r\n\r\n\tself smalltalk #renameClass: self to: aString","(\r\nfunction(self,aString)\r\n((self):smalltalk()):renameClass(self, aString)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"removeFromSystem",0
,(
function(self)
((self):subclasses()):do_((function(each)
return ((each):removeFromSystem())
end))
;
((self):smalltalk()):removeClass(self)
;
return (self)
;
end
)
,"remove","removeFromSystem\r\n\t\" Remove the receiver from system. \"\r\n\r\n\tself subclasses do: [:each| each removeFromSystem ].\r\n\tself smalltalk #removeClass: self","(\r\nfunction(self)\r\n((self):subclasses()):do_((function(each)\r\nreturn ((each):removeFromSystem())\r\nend))\r\n;\r\n((self):smalltalk()):removeClass(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"removeSources",0
,(
function(self)
((self)["$klass"]):methodsDo_((function(each)
return ((each):removeSources())
end))
;
(self):methodsDo_((function(each)
return ((each):removeSources())
end))
;
(
function(_s8_rec)
(_s8_rec):comment_("");
return ((_s8_rec):category_("s8"))
end
)(self)
;
return (self)
;
end
)
,"tools","removeSources\r\n\t\" Remove all development sources of the receiver. \"\r\n\r\n\tself class methodsDo: [:each | each removeSources ].\r\n\tself methodsDo: [:each | each removeSources ].\r\n\tself comment: ''; category: #s8","(\r\nfunction(self)\r\n((self)[\"$klass\"]):methodsDo_((function(each)\r\nreturn ((each):removeSources())\r\nend))\r\n;\r\n(self):methodsDo_((function(each)\r\nreturn ((each):removeSources())\r\nend))\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):comment_(\"\");\r\nreturn ((_s8_rec):category_(\"s8\"))\r\nend\r\n)(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"variableByteSubclass:classVariableNames:poolDictionaries:",0
,(
function(self,aString,classVarNames,poolVarNames)
return ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, "", classVarNames, poolVarNames))
;
end
)
,"instantiation","variableByteSubclass: aString\r\n\tclassVariableNames: classVarNames\r\n\tpoolDictionaries: poolVarNames\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\"\tself print: 'WARNING: ',aString,' defined as variableByteSubclass:...'.\r\n\"\t^self \r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: ''\r\n\t\tclassVariableNames: classVarNames\r\n\t\tpoolDictionaries: poolVarNames","(\r\nfunction(self,aString,classVarNames,poolVarNames)\r\nreturn ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, \"\", classVarNames, poolVarNames))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"variableSubclass:instanceVariableNames:classVariableNames:poolDictionaries:",0
,(
function(self,aString,aString2,classVarNames,poolVarNames)
(self):note_(smalltalk:object_comma_((smalltalk:object_comma_(("WARNING: "),aString))," defined as variableSubclass:..."))
;
return ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, aString2, classVarNames, poolVarNames))
;
end
)
,"instantiation","variableSubclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tclassVariableNames: classVarNames\r\n\tpoolDictionaries: poolVarNames\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\tself note: 'WARNING: ',aString,' defined as variableSubclass:...'.\r\n\t^self \r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: classVarNames\r\n\t\tpoolDictionaries: poolVarNames","(\r\nfunction(self,aString,aString2,classVarNames,poolVarNames)\r\n(self):note_(smalltalk:object_comma_((smalltalk:object_comma_((\"WARNING: \"),aString)),\" defined as variableSubclass:...\"))\r\n;\r\nreturn ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, aString2, classVarNames, poolVarNames))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:",0
,(
function(self,aString,aString2,classVarNames,poolVarNames,aCategory)
(function() if ((((((((self):smalltalk()):globalAt_(aString)))~=nil)))==true) then return ((function()
return ((self):note_(smalltalk:object_comma_(("WARNING: class REdefinition for "),aString)))
end))() else return nil end end)()
;
return (((smalltalk.ClassBuilder):new()):superclass_subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(self, aString, aString2, classVarNames, poolVarNames, aCategory))
;
end
)
,"instantiation","subclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tclassVariableNames: classVarNames\r\n\tpoolDictionaries: poolVarNames\r\n\tcategory: aCategory\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t(self smalltalk globalAt: aString) notNil ifTrue: [\r\n\t\tself note: 'WARNING: class REdefinition for ',aString.\r\n\t\t\"^self smalltalk globalAt: aString\"\r\n\t].\r\n\r\n\"\tself note: 'Creating ',aString.\"\r\n\t^ClassBuilder new\r\n\t\tsuperclass: self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: classVarNames\r\n\t\tpoolDictionaries: poolVarNames\r\n\t\tcategory: aCategory","(\r\nfunction(self,aString,aString2,classVarNames,poolVarNames,aCategory)\r\n(function() if ((((((((self):smalltalk()):globalAt_(aString)))~=nil)))==true) then return ((function()\r\nreturn ((self):note_(smalltalk:object_comma_((\"WARNING: class REdefinition for \"),aString)))\r\nend))() else return nil end end)()\r\n;\r\nreturn (((smalltalk.ClassBuilder):new()):superclass_subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(self, aString, aString2, classVarNames, poolVarNames, aCategory))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:instanceVariableNames:classVariableNames:category:",0
,(
function(self,aString,aString2,classVarNames,aCategory)
return ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString, aString2, classVarNames, "", aCategory))
;
end
)
,"instantiation","subclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tclassVariableNames: classVarNames\r\n\tcategory: aCategory\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self subclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: classVarNames\r\n\t\tpoolDictionaries: ''\r\n\t\tcategory: aCategory","(\r\nfunction(self,aString,aString2,classVarNames,aCategory)\r\nreturn ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString, aString2, classVarNames, \"\", aCategory))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:instanceVariableNames:classVariableNames:poolDictionaries:",0
,(
function(self,aString,aString2,classVarNames,poolVarNames)
return ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString, aString2, classVarNames, poolVarNames, nil))
;
end
)
,"instantiation","subclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tclassVariableNames: classVarNames\r\n\tpoolDictionaries: poolVarNames\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: classVarNames\r\n\t\tpoolDictionaries: poolVarNames\r\n\t\tcategory: nil","(\r\nfunction(self,aString,aString2,classVarNames,poolVarNames)\r\nreturn ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString, aString2, classVarNames, poolVarNames, nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:instanceVariableNames:poolDictionaries:category:",0
,(
function(self,aString,aString2,poolVarNames,aCategory)
return ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString, aString2, "", poolVarNames, aCategory))
;
end
)
,"instantiation","subclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tpoolDictionaries: poolVarNames\r\n\tcategory: aCategory\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: ''\r\n\t\tpoolDictionaries: poolVarNames\r\n\t\tcategory: aCategory","(\r\nfunction(self,aString,aString2,poolVarNames,aCategory)\r\nreturn ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString, aString2, \"\", poolVarNames, aCategory))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"variableByteSubclass:classVariableNames:",0
,(
function(self,aString,classVarNames)
return ((self):variableByteSubclass_classVariableNames_poolDictionaries_(aString, classVarNames, ""))
;
end
)
,"instantiation","variableByteSubclass: aString classVariableNames: classVarNames\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tvariableByteSubclass: aString\r\n\t\tclassVariableNames: classVarNames\r\n\t\tpoolDictionaries: ''","(\r\nfunction(self,aString,classVarNames)\r\nreturn ((self):variableByteSubclass_classVariableNames_poolDictionaries_(aString, classVarNames, \"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"variableByteSubclass:poolDictionaries:",0
,(
function(self,aString,aString2)
return ((self):variableByteSubclass_classVariableNames_poolDictionaries_(aString, "", aString2))
;
end
)
,"instantiation","variableByteSubclass: aString poolDictionaries: aString2\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tvariableByteSubclass: aString\r\n\t\tclassVariableNames: ''\r\n\t\tpoolDictionaries: aString2","(\r\nfunction(self,aString,aString2)\r\nreturn ((self):variableByteSubclass_classVariableNames_poolDictionaries_(aString, \"\", aString2))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"variableByteSubclass:",0
,(
function(self,aString)
return ((self):variableByteSubclass_classVariableNames_(aString, ""))
;
end
)
,"instantiation","variableByteSubclass: aString\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tvariableByteSubclass: aString\r\n\t\tclassVariableNames: ''","(\r\nfunction(self,aString)\r\nreturn ((self):variableByteSubclass_classVariableNames_(aString, \"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"variableSubclass:instanceVariableNames:classVariableNames:",0
,(
function(self,aString,aString2,classVarNames)
return ((self):variableSubclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, aString2, classVarNames, ""))
;
end
)
,"instantiation","variableSubclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tclassVariableNames: classVarNames\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self \r\n\t\tvariableSubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: classVarNames\r\n\t\tpoolDictionaries: ''","(\r\nfunction(self,aString,aString2,classVarNames)\r\nreturn ((self):variableSubclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, aString2, classVarNames, \"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"variableSubclass:instanceVariableNames:",0
,(
function(self,aString,aString2)
return ((self):variableSubclass_instanceVariableNames_classVariableNames_(aString, aString2, ""))
;
end
)
,"instantiation","variableSubclass: aString instanceVariableNames: aString2\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self \r\n\t\tvariableSubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: ''","(\r\nfunction(self,aString,aString2)\r\nreturn ((self):variableSubclass_instanceVariableNames_classVariableNames_(aString, aString2, \"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"variableSubclass:classVariableNames:",0
,(
function(self,aString,aString2)
return ((self):variableSubclass_instanceVariableNames_classVariableNames_(aString, "", aString2))
;
end
)
,"instantiation","variableSubclass: aString classVariableNames: aString2\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self \r\n\t\tvariableSubclass: aString\r\n\t\tinstanceVariableNames: ''\r\n\t\tclassVariableNames: aString2","(\r\nfunction(self,aString,aString2)\r\nreturn ((self):variableSubclass_instanceVariableNames_classVariableNames_(aString, \"\", aString2))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"variableSubclass:poolDictionaries:",0
,(
function(self,aString,aString2)
return ((self):variableSubclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, "", "", aString2))
;
end
)
,"instantiation","variableSubclass: aString poolDictionaries: aString2\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self \r\n\t\tvariableSubclass: aString\r\n\t\tinstanceVariableNames: ''\r\n\t\tclassVariableNames: ''\r\n\t\tpoolDictionaries: aString2","(\r\nfunction(self,aString,aString2)\r\nreturn ((self):variableSubclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, \"\", \"\", aString2))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"variableSubclass:",0
,(
function(self,aString)
return ((self):variableSubclass_instanceVariableNames_(aString, ""))
;
end
)
,"instantiation","variableSubclass: aString\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self \r\n\t\tvariableSubclass: aString\r\n\t\tinstanceVariableNames: ''","(\r\nfunction(self,aString)\r\nreturn ((self):variableSubclass_instanceVariableNames_(aString, \"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:instanceVariableNames:classVariableNames:",0
,(
function(self,aString,aString2,classVarNames)
return ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, aString2, classVarNames, ""))
;
end
)
,"instantiation","subclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tclassVariableNames: classVarNames\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: classVarNames\r\n\t\tpoolDictionaries: ''","(\r\nfunction(self,aString,aString2,classVarNames)\r\nreturn ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, aString2, classVarNames, \"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:instanceVariableNames:poolDictionaries:",0
,(
function(self,aString,aString2,aString3)
return ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, aString2, "", aString3))
;
end
)
,"instantiation","subclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tpoolDictionaries: aString3\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: ''\r\n\t\tpoolDictionaries: aString3","(\r\nfunction(self,aString,aString2,aString3)\r\nreturn ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, aString2, \"\", aString3))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:classVariableNames:",0
,(
function(self,aString,aString2)
return ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, "", aString2, ""))
;
end
)
,"instantiation","subclass: aString classVariableNames: aString2\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: ''\r\n\t\tclassVariableNames: aString2\r\n\t\tpoolDictionaries: ''","(\r\nfunction(self,aString,aString2)\r\nreturn ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, \"\", aString2, \"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:poolDictionaries:",0
,(
function(self,aString,aString2)
return ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, "", "", aString2))
;
end
)
,"instantiation","subclass: aString poolDictionaries: aString2\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: ''\r\n\t\tclassVariableNames: ''\r\n\t\tpoolDictionaries: aString2","(\r\nfunction(self,aString,aString2)\r\nreturn ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString, \"\", \"\", aString2))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"subclass:",0
,(
function(self,aString)
return ((self):subclass_instanceVariableNames_(aString, ""))
;
end
)
,"instantiation","subclass: aString\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: ''","(\r\nfunction(self,aString)\r\nreturn ((self):subclass_instanceVariableNames_(aString, \"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"hackDNU",0
,(
function(self)
local all=nil
(function()
	local _s8_value=((smalltalk.HashedSet):new())
	all=_s8_value; return _s8_value
end)()
;
((smalltalk.Smalltalk):rootClasses()):do_((function(root)
return (((root):allSubclasses()):do_((function(each)
return ((
function(_s8_rec)
(_s8_rec):addAll_((each):selectors());
return ((_s8_rec):addAll_(((each)["$klass"]):selectors()))
end
)(all))
end)))
end))
;
((smalltalk.Smalltalk):rootClasses()):do_((function(root)
return (((root):selectors()):do_((function(each)
return ((all):remove_ifAbsent_(each, (function()
return (nil)
end)))
end)))
end))
;
(smalltalk:newArray({"handleOrNil", "start:end:"})):do_((function(each)
return ((all):remove_ifAbsent_(each, (function()
return (nil)
end)))
end))
;
(all):do_((function(selector)
local mth=nil
(function()
	local _s8_value=((smalltalk.CompiledMethod):new())
	mth=_s8_value; return _s8_value
end)();(
function(_s8_rec)
(_s8_rec):selector_(selector);
(_s8_rec):classField_(self);
(_s8_rec):category_("hackDNU");
return ((_s8_rec):source_(smalltalk:object_comma_((selector),"(dnu)")))
end
)(mth);self:notImplementedYet_("hackDNU");return ((self):basicAddCompiledMethod_(mth))
end))
;
return (self)
;
end
)
,"Tools-hacking","hackDNU\r\n\t\" Catch DNU condition in the receiver and its subclasses.\r\n\tNote: hack will persist until #unhackDNU is sent to the receiver.\r\n\t\"\r\n\r\n\t| all |\r\n\tall := HashedSet new.\r\n\tSmalltalk rootClasses do: [:root|\r\n\t\troot allSubclasses do: [:each|\r\n\t\t\tall addAll: each selectors; addAll: each class selectors\r\n\t\t].\r\n\t].\r\n\tSmalltalk rootClasses do: [:root| root selectors do: [:each| all remove: each ifAbsent: []] ].\r\n\t#( #handleOrNil #start:end: ) do: [:each| all remove: each ifAbsent: []].\r\n\tall do: [:selector| | mth |\r\n\t\tmth := CompiledMethod new.\r\n\t\tmth selector: selector; classField: self; category: #hackDNU; source: selector,'(dnu)'.\r\n\t\t{'mth.fn_(function(){\r\n\t\t\tthis.print_(\"//DNU \"+selector);\r\n\t\t\tthis.doesNotUnderstand_(smalltalk.Message.receiver_selector_arguments_(this,selector,[].slice.call(arguments)))});\r\n\t\t' lua'self:notImplementedYet_(\"hackDNU\")'}.\r\n\t\tself basicAddCompiledMethod: mth.\r\n\t].","(\r\nfunction(self)\r\nlocal all=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.HashedSet):new())\r\n\tall=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((smalltalk.Smalltalk):rootClasses()):do_((function(root)\r\nreturn (((root):allSubclasses()):do_((function(each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):addAll_((each):selectors());\r\nreturn ((_s8_rec):addAll_(((each)[\"$klass\"]):selectors()))\r\nend\r\n)(all))\r\nend)))\r\nend))\r\n;\r\n((smalltalk.Smalltalk):rootClasses()):do_((function(root)\r\nreturn (((root):selectors()):do_((function(each)\r\nreturn ((all):remove_ifAbsent_(each, (function()\r\nreturn (nil)\r\nend)))\r\nend)))\r\nend))\r\n;\r\n(smalltalk:newArray({\"handleOrNil\", \"start:end:\"})):do_((function(each)\r\nreturn ((all):remove_ifAbsent_(each, (function()\r\nreturn (nil)\r\nend)))\r\nend))\r\n;\r\n(all):do_((function(selector)\r\nlocal mth=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.CompiledMethod):new())\r\n\tmth=_s8_value; return _s8_value\r\nend)();(\r\nfunction(_s8_rec)\r\n(_s8_rec):selector_(selector);\r\n(_s8_rec):classField_(self);\r\n(_s8_rec):category_(\"hackDNU\");\r\nreturn ((_s8_rec):source_(smalltalk:object_comma_((selector),\"(dnu)\")))\r\nend\r\n)(mth);self:notImplementedYet_(\"hackDNU\");return ((self):basicAddCompiledMethod_(mth))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"unhackDNU",0
,(
function(self)
(self):removeSelectors_(((self):selectors()):select_((function(each)
return ((smalltalk:isObject_equalTo_((((smalltalk.Object):_gt_gt(each)):category()),"hackDNU")))
end)))
;
return (self)
;
end
)
,"Tools-hacking","unhackDNU\r\n\t\" Remove Catch DNU hacking on the receiver. \"\r\n\r\n\tself removeSelectors: (self selectors\r\n\t\tselect: [:each| (Object >> each) category = #hackDNU ])","(\r\nfunction(self)\r\n(self):removeSelectors_(((self):selectors()):select_((function(each)\r\nreturn ((smalltalk:isObject_equalTo_((((smalltalk.Object):_gt_gt(each)):category()),\"hackDNU\")))\r\nend)))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"hackDNUWhile:",0
,(
function(self,aBlock)
local result=nil
((function()
(self):hackDNU();return ((function()
	local _s8_value=((aBlock):evaluate())
	result=_s8_value; return _s8_value
end)())
end)):ensure_((function()
return ((self):unhackDNU())
end))
;
return (result)
;
end
)
,"Tools-hacking","hackDNUWhile: aBlock\r\n\t\" Catch DNU hacking on the receiver while evaluating aBlock. \"\r\n\r\n\t| result |\r\n\t[\tself hackDNU.\r\n\t\tresult := aBlock evaluate\r\n\t] ensure: [ self unhackDNU ].\r\n\t^result","(\r\nfunction(self,aBlock)\r\nlocal result=nil\r\n((function()\r\n(self):hackDNU();return ((function()\r\n\tlocal _s8_value=((aBlock):evaluate())\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend)):ensure_((function()\r\nreturn ((self):unhackDNU())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"launch",0
,(
function(self)
return (self)
;
end
)
,"Snapshot-startup","launch\r\n\t\" Private - The system has been launched and start execution.\r\n\tThe default action is to do nothing.\r\n\tSubclasses can refine this method to execute actions on startup.\r\n\t\"","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"aboutToSaveImage:on:",0
,(
function(self,snapshot,stream)
return (self)
;
end
)
,"Snapshot-snapshot","aboutToSaveImage: snapshot on: stream\r\n\t\" Private - A system snapshot will be taken, the receiver can prepare the contents to be stored on snapshot.\r\n\tMust return the receiver or a replacement to act as the receiver when persisting data.\r\n\tThe default action return the receiver.\r\n\t\"\r\n\r\n\t^self","(\r\nfunction(self,snapshot,stream)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"imageSaved:on:",0
,(
function(self,snapshot,stream)
return (self)
;
end
)
,"Snapshot-snapshot","imageSaved: snapshot on: stream\r\n\t\" Private - A system snapshot has been taken, the receiver can emmit code to restore state.\r\n\tThe default action do nothing.\r\n\tThis method can be refined by the subclasses.\r\n\t\"","(\r\nfunction(self,snapshot,stream)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Class,"aboutToExit:",0
,(
function(self,aSystemDictionary)
return (self)
;
end
)
,"Snapshot-snapshot","aboutToExit: aSystemDictionary\r\n\t\" Private - A system will exit, and the receiver can act executing actions just before exiting.\r\n\tThe default action do nothing.\r\n\tThis method can be refined by the subclasses.\r\n\t\"","(\r\nfunction(self,aSystemDictionary)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Metaclass,"instanceClass",0
,(
function(self)
return (self['@instanceClass'])
;
end
)
,"accessing","instanceClass\r\n\t\" Return the instance class of receiver. \"\r\n\r\n\t^instanceClass","(\r\nfunction(self)\r\nreturn (self['@instanceClass'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Metaclass,"instanceVariableNames:",0
,(
function(self,aCollection)
((smalltalk.ClassBuilder):new()):class_instanceVariableNames_(self, aCollection)
;
return (self)
;
end
)
,"accessing","instanceVariableNames: aCollection\r\n\t\" Set the instance variable names of the receiver. \"\r\n\r\n\tClassBuilder new\r\n\t\tclass: self\r\n\t\tinstanceVariableNames: aCollection","(\r\nfunction(self,aCollection)\r\n((smalltalk.ClassBuilder):new()):class_instanceVariableNames_(self, aCollection)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Metaclass,"isMetaclass",0
,(
function(self)
return (true)
;
end
)
,"testing","isMetaclass\r\n\t\" Return true if the receiver is a metaclass. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Metaclass,"printString",0
,(
function(self)
return (smalltalk:object_comma_((((self):instanceClass()):name())," class"))
;
end
)
,"printing","printString\r\n\t\" Return a printable representation of the receiver. \"\r\n\r\n\t^self instanceClass name, ' class'","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((((self):instanceClass()):name()),\" class\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Metaclass,"poolDictionaries",0
,(
function(self)
return (((self):instanceClass()):poolDictionaries())
;
end
)
,"accessing","poolDictionaries\r\n\t\" Return the poolDictionaries of the receiver. \"\r\n\r\n\t^self instanceClass poolDictionaries","(\r\nfunction(self)\r\nreturn (((self):instanceClass()):poolDictionaries())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Metaclass,"classVariables",0
,(
function(self)
return (((self):instanceClass()):classVariables())
;
end
)
,"accessing","classVariables\r\n\t\" Return the class variables of the receiver. \"\r\n\r\n\t^self instanceClass classVariables","(\r\nfunction(self)\r\nreturn (((self):instanceClass()):classVariables())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"source",0
,(
function(self)
return ((function() local _1_=((self)["$source"]);if (_1_==nil) then return ((function()
return ("")
end)):value() end; return _1_ end)())
;
end
)
,"accessing","source\r\n\t\" Return the source code of the receiver. \"\r\n\r\n\t^self basicAt: '$source' ifAbsent: ['']","(\r\nfunction(self)\r\nreturn ((function() local _1_=((self)[\"$source\"]);if (_1_==nil) then return ((function()\r\nreturn (\"\")\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"source:",0
,(
function(self,aString)
(function() local _1_=(aString); (self)["$source"]=_1_; return _1_ end)()
;
return (self)
;
end
)
,"accessing","source: aString\r\n\t\" Set the source code of the receiver. \"\r\n\r\n\tself basicAt: '$source' put: aString","(\r\nfunction(self,aString)\r\n(function() local _1_=(aString); (self)[\"$source\"]=_1_; return _1_ end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"category",0
,(
function(self)
return ((function() local _1_=((self)["$category"]);if (_1_==nil) then return ((function()
return ("")
end)):value() end; return _1_ end)())
;
end
)
,"accessing","category\r\n\t\" Return the category of the receiver. \"\r\n\r\n\t^self basicAt: '$category' ifAbsent: ['']","(\r\nfunction(self)\r\nreturn ((function() local _1_=((self)[\"$category\"]);if (_1_==nil) then return ((function()\r\nreturn (\"\")\r\nend)):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"category:",0
,(
function(self,aString)
(function() local _1_=(aString); (self)["$category"]=_1_; return _1_ end)()
;
return (self)
;
end
)
,"accessing","category: aString\r\n\t\" Set the category of the receiver. \"\r\n\r\n\tself basicAt: '$category' put: aString","(\r\nfunction(self,aString)\r\n(function() local _1_=(aString); (self)[\"$category\"]=_1_; return _1_ end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"selector",0
,(
function(self)
return (((self)["$selector"]))
;
end
)
,"accessing","selector\r\n\t\" Return the selector of the receiver. \"\r\n\r\n\t^self basicAt: '$selector'","(\r\nfunction(self)\r\nreturn (((self)[\"$selector\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"selector:",0
,(
function(self,aString)
(function() local _1_=(aString); (self)["$selector"]=_1_; return _1_ end)()
;
return (self)
;
end
)
,"accessing","selector: aString\r\n\t\" Set the selector of the receiver. \"\r\n\r\n\tself basicAt: '$selector' put: aString","(\r\nfunction(self,aString)\r\n(function() local _1_=(aString); (self)[\"$selector\"]=_1_; return _1_ end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"fn",0
,(
function(self)
return (((self)["$fn"]))
;
end
)
,"accessing","fn\r\n\t\" Return the function of the receiver. \"\r\n\r\n\t^self basicAt: '$fn'","(\r\nfunction(self)\r\nreturn (((self)[\"$fn\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"fn:",0
,(
function(self,aBlock)
(function() local _1_=(aBlock); (self)["$fn"]=_1_; return _1_ end)()
;
return (self)
;
end
)
,"accessing","fn: aBlock\r\n\t\" Set the function of the receiver. \"\r\n\r\n\tself basicAt: '$fn' put: aBlock","(\r\nfunction(self,aBlock)\r\n(function() local _1_=(aBlock); (self)[\"$fn\"]=_1_; return _1_ end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"compiledSource",0
,(
function(self)
return (((self):fn()):compiledSourceAt_(self))
;
end
)
,"accessing","compiledSource\r\n\t\" Return the native source code of the receiver. \"\r\n\r\n\t^self fn compiledSourceAt: self","(\r\nfunction(self)\r\nreturn (((self):fn()):compiledSourceAt_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"classField",0
,(
function(self)
return (self['@classField'])
;
end
)
,"accessing","classField\r\n\t\" Return the implementor of the receiver. \"\r\n\r\n\t^classField","(\r\nfunction(self)\r\nreturn (self['@classField'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"classField:",0
,(
function(self,aClass)
(function()
	local _s8_value=(aClass)
	self['@classField']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","classField: aClass\r\n\t\" Set the implementor of the receiver. \"\r\n\r\n\tclassField := aClass","(\r\nfunction(self,aClass)\r\n(function()\r\n\tlocal _s8_value=(aClass)\r\n\tself['@classField']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"generation",0
,(
function(self)
return (((self)["$generation"]))
;
end
)
,"accessing","generation\r\n\t\" Return the generation of the receiver. \"\r\n\r\n\t^self basicAt: '$generation'","(\r\nfunction(self)\r\nreturn (((self)[\"$generation\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"comment",0
,(
function(self)
return (((
function(_s8_rec)
(_s8_rec):upTo_("\"");
return ((_s8_rec):upTo_("\""))
end
)(((self):source()):stream())):trimBlanks())
;
end
)
,"accessing","comment\r\n\t\" Return the comment of the receiver. \"\r\n\r\n\t^(self source stream upTo: $\"; upTo: $\") trimBlanks","(\r\nfunction(self)\r\nreturn (((\r\nfunction(_s8_rec)\r\n(_s8_rec):upTo_(\"\\\"\");\r\nreturn ((_s8_rec):upTo_(\"\\\"\"))\r\nend\r\n)(((self):source()):stream())):trimBlanks())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"toString",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_(((function() if ((((((self['@classField']))==nil)))==true) then return ((function()
return (smalltalk:toString_((self['@classField'])))
end))() else return ((function()
return ((self['@classField']):name())
end))() end end)()),">>#")),smalltalk:toString_(((self):selector()))))
;
end
)
,"printing","toString\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^(classField isNil\r\n\t\tifTrue: [classField toString]\r\n\t\tifFalse: [classField name])\r\n\t\t,'>>#' ,self selector toString","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_(((function() if ((((((self['@classField']))==nil)))==true) then return ((function()\r\nreturn (smalltalk:toString_((self['@classField'])))\r\nend))() else return ((function()\r\nreturn ((self['@classField']):name())\r\nend))() end end)()),\">>#\")),smalltalk:toString_(((self):selector()))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"printString",0
,(
function(self)
return (smalltalk:toString_((self)))
;
end
)
,"printing","printString\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"asJSONObject",0
,(
function(self)
return (self)
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"shallowCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"deepCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"removeSources",0
,(
function(self)
(
function(_s8_rec)
(_s8_rec):source_("");
(_s8_rec):category_("s8");
return ((_s8_rec):builder_(nil))
end
)(self)
;
return (self)
;
end
)
,"Builders-tools","removeSources\r\n\t\" Remove all development sources of the receiver. \"\r\n\r\n\tself source: ''; category: #s8; builder: nil","(\r\nfunction(self)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):source_(\"\");\r\n(_s8_rec):category_(\"s8\");\r\nreturn ((_s8_rec):builder_(nil))\r\nend\r\n)(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"removeFromSystem",0
,(
function(self)
(function() if ((((smalltalk:isObject_equalTo_((((self):classField()):_gt_gt((self):selector())),self))))==true) then return ((function()
return (((self):classField()):removeSelector_((self):selector()))
end))() else return nil end end)()
;
return (self)
;
end
)
,"remove","removeFromSystem\r\n\t\" Remove the receiver from system. \"\r\n\r\n\tself classField >> self selector = self ifTrue: [\r\n\t\tself classField removeSelector: self selector\r\n\t]","(\r\nfunction(self)\r\n(function() if ((((smalltalk:isObject_equalTo_((((self):classField()):_gt_gt((self):selector())),self))))==true) then return ((function()\r\nreturn (((self):classField()):removeSelector_((self):selector()))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"recompile",0
,(
function(self)
(((self):classField()):compiler()):compile_forClass_category_((self):source(), (self):classField(), (self):category())
;
return (self)
;
end
)
,"Compiler-compiling","recompile\r\n\t\" Recompile the source of the receiver and install the new compiled method. \"\r\n\r\n\tself classField compiler\r\n\t\tcompile: self source\r\n\t\tforClass: self classField\r\n\t\tcategory: self category","(\r\nfunction(self)\r\n(((self):classField()):compiler()):compile_forClass_category_((self):source(), (self):classField(), (self):category())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"parseNodes",0
,(
function(self)
return ((((self):classField()):compiler()):parse_((self):source()))
;
end
)
,"Compiler-inspecting","parseNodes\r\n\t\" Return the result of parsing the source of the receiver. \"\r\n\r\n\t^self classField compiler\r\n\t\tparse: self source","(\r\nfunction(self)\r\nreturn ((((self):classField()):compiler()):parse_((self):source()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"changeCommentWith:",0
,(
function(self,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local stream=nil
local header=nil
local old=nil
local new=nil
local nl=nil
(function()
	local _s8_value=(((self):source()):stream())
	stream=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((smalltalk.SmalltalkParser):new()):methodHeaderParser()):parse_(stream))
	header=_s8_value; return _s8_value
end)()
;
(function() if ((((header):isParseFailure()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(stream):skipSeparators()
;
(function()
	local _s8_value=(((stream):collection()):copyFrom_to_((1), (stream):position()))
	header=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((stream):peek()),"\""))))==true) then return ((function()
(stream):next();(function()
	local _s8_value=((stream):upTo_("\""))
	old=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=((aBlock):evaluateWith_(old))
	new=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((function()
	local _s8_value=((aBlock):evaluateWith_(""))
	new=_s8_value; return _s8_value
end)())
end))() end end)()
;
(function() if ((((((new))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((function() if ((((((old))==nil)))==true) then return ((function()
return ("\r\n\t")
end))() else return ((function()
return ("")
end))() end end)())
	nl=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((nl),"\" ")),(new):trimBlanks()))," \"")),nl))
	new=_s8_value; return _s8_value
end)()
;
(
function(_s8_rec)
(_s8_rec):source_(smalltalk:object_comma_((smalltalk:object_comma_((header),new)),(stream):upToEnd()));
return ((_s8_rec):recompile())
end
)(self)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Compiler-transforms","changeCommentWith: aBlock\r\n\t\" Private - Change the comment of the receiver with aBlock.\r\n\taBlock must be a one argument block to be evaluated with current comment.\r\n\taBlock must return the new/empty comment (or nil, to cancel change).\r\n\t\"\r\n\r\n\t| stream header old new nl |\r\n\tstream := self source stream.\r\n\theader := SmalltalkParser new methodHeaderParser parse: stream.\r\n\theader isParseFailure ifTrue: [ ^false ].\r\n\tstream skipSeparators.\r\n\theader := stream collection copyFrom: 1 to: stream position.\r\n\tstream peek = $\" ifTrue: [\r\n\t\tstream next.\r\n\t\told := stream upTo: $\".\r\n\t\tnew := aBlock evaluateWith: old.\r\n\t] ifFalse: [\r\n\t\tnew := aBlock evaluateWith: ''.\r\n\t].\r\n\tnew isNil ifTrue: [ ^false ].\r\n\tnl := old isNil ifTrue: [ '\r\n\t' ] ifFalse: [ '' ].\r\n\tnew := nl ,'\" ' ,new trimBlanks ,' \"' ,nl.\r\n\r\n\tself\r\n\t\tsource: header ,new ,stream upToEnd;\r\n\t\trecompile","(\r\nfunction(self,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal stream=nil\r\nlocal header=nil\r\nlocal old=nil\r\nlocal new=nil\r\nlocal nl=nil\r\n(function()\r\n\tlocal _s8_value=(((self):source()):stream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((smalltalk.SmalltalkParser):new()):methodHeaderParser()):parse_(stream))\r\n\theader=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((header):isParseFailure()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(stream):skipSeparators()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((stream):collection()):copyFrom_to_((1), (stream):position()))\r\n\theader=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((stream):peek()),\"\\\"\"))))==true) then return ((function()\r\n(stream):next();(function()\r\n\tlocal _s8_value=((stream):upTo_(\"\\\"\"))\r\n\told=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=((aBlock):evaluateWith_(old))\r\n\tnew=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((aBlock):evaluateWith_(\"\"))\r\n\tnew=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((((new))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function() if ((((((old))==nil)))==true) then return ((function()\r\nreturn (\"\\r\\n\\t\")\r\nend))() else return ((function()\r\nreturn (\"\")\r\nend))() end end)())\r\n\tnl=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((nl),\"\\\" \")),(new):trimBlanks())),\" \\\"\")),nl))\r\n\tnew=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):source_(smalltalk:object_comma_((smalltalk:object_comma_((header),new)),(stream):upToEnd()));\r\nreturn ((_s8_rec):recompile())\r\nend\r\n)(self)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"tags",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(self):changeCommentWith_((function(old)
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((((old):upTo_("-")):asArrayOfSubstringsSeparatedBy_(",")):collect_((function(tag)
return ((tag):trimBlanks())
end)))
end
)()
error(_s8Ret_,0)
end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Array):new())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Compiler-tags","tags\r\n\t\" Development - Return the tags stored in the receiver's comment. \"\r\n\r\n\tself changeCommentWith: [:old|\r\n\t\t^((old upTo: $-)\r\n\t\t\tasArrayOfSubstringsSeparatedBy: $,)\r\n\t\t\tcollect: [:tag| tag trimBlanks ]\r\n\t].\r\n\t^Array new","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(self):changeCommentWith_((function(old)\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((((old):upTo_(\"-\")):asArrayOfSubstringsSeparatedBy_(\",\")):collect_((function(tag)\r\nreturn ((tag):trimBlanks())\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Array):new())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"comment:",0
,(
function(self,aString)
return ((self):changeCommentWith_((function(old)
return (aString)
end)))
;
end
)
,"Compiler-accessing","comment: aString\r\n\t\" Change the comment in the receiver to aString. \"\r\n\r\n\t^self changeCommentWith: [:old| aString ]","(\r\nfunction(self,aString)\r\nreturn ((self):changeCommentWith_((function(old)\r\nreturn (aString)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"messageComment",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local line=nil
(function()
	local _s8_value=((((self):comment()):stream()):nextLine())
	line=_s8_value; return _s8_value
end)()
;
(function() if ((((line):includes_("-")))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (line)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((
function(_s8_rec)
(_s8_rec):upTo_("-");
return ((_s8_rec):upToEnd())
end
)((line):stream())):trimBlanks())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Compiler-accessing","messageComment\r\n\t\" Return the comment of the message implemented by the receiver.\r\n\tThe messageComment is obtained from first line of comment of the receiver.\r\n\tNote: it is expected that all the methods implementing the same message in this system have the same messageComment.\r\n\t\"\r\n\r\n\t| line |\r\n\tline := self comment stream nextLine.\r\n\t(line includes: $-) ifFalse: [ ^line ].\r\n\t^(line stream upTo: $-; upToEnd) trimBlanks","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal line=nil\r\n(function()\r\n\tlocal _s8_value=((((self):comment()):stream()):nextLine())\r\n\tline=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((line):includes_(\"-\")))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (line)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((\r\nfunction(_s8_rec)\r\n(_s8_rec):upTo_(\"-\");\r\nreturn ((_s8_rec):upToEnd())\r\nend\r\n)((line):stream())):trimBlanks())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"messageComment:",0
,(
function(self,aString)
return ((self):changeCommentWith_((function(old)
local line=nil
local tags=nil
local stream=nil
(function()
	local _s8_value=((old):stream())
	stream=_s8_value; return _s8_value
end)();(function()
	local _s8_value=((stream):nextLine())
	line=_s8_value; return _s8_value
end)();(function() if ((((line):includes_("-")))==true) then return ((function()
return ((function()
	local _s8_value=(smalltalk:object_comma_(((line):upTo_("-")),"- "))
	tags=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((function()
	local _s8_value=("")
	tags=_s8_value; return _s8_value
end)())
end))() end end)();return (smalltalk:object_comma_((smalltalk:object_comma_((tags),aString)),(function() if ((((stream):atEnd()))==true) then return ((function()
return ("")
end))() else return ((function()
return (smalltalk:object_comma_(("\r\n"),(stream):upToEnd()))
end))() end end)()))
end)))
;
end
)
,"Compiler-transforms","messageComment: aString\r\n\t\" Set the comment of the message implemented by the receiver. \"\r\n\r\n\t^self changeCommentWith: [:old| | line tags stream |\r\n\t\tstream := old stream.\r\n\t\tline := stream nextLine.\r\n\t\t(line includes: $-) ifTrue: [\r\n\t\t\ttags := (line upTo: $-) ,'- '\r\n\t\t] ifFalse: [ tags := '' ].\r\n\t\ttags\t,aString ,(stream atEnd ifTrue: [''] ifFalse: ['\r\n'\t\t\t,stream upToEnd])\r\n\t]","(\r\nfunction(self,aString)\r\nreturn ((self):changeCommentWith_((function(old)\r\nlocal line=nil\r\nlocal tags=nil\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=((old):stream())\r\n\tstream=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=((stream):nextLine())\r\n\tline=_s8_value; return _s8_value\r\nend)();(function() if ((((line):includes_(\"-\")))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_(((line):upTo_(\"-\")),\"- \"))\r\n\ttags=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(\"\")\r\n\ttags=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)();return (smalltalk:object_comma_((smalltalk:object_comma_((tags),aString)),(function() if ((((stream):atEnd()))==true) then return ((function()\r\nreturn (\"\")\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((\"\\r\\n\"),(stream):upToEnd()))\r\nend))() end end)()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"messages",0
,(
function(self)
return ((((self):smalltalk()):methodReferences()):messagesOf_(self))
;
end
)
,"Tools-accessing","messages\r\n\t\" Return all the messages that can be sent during execution of the receiver. \"\r\n\r\n\t^self smalltalk methodReferences messagesOf: self","(\r\nfunction(self)\r\nreturn ((((self):smalltalk()):methodReferences()):messagesOf_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"globalReferences",0
,(
function(self)
return ((((self):smalltalk()):methodReferences()):globalReferencesOf_(self))
;
end
)
,"Tools-accessing","globalReferences\r\n\t\" Return all the global references in the receiver. \"\r\n\r\n\t^self smalltalk methodReferences globalReferencesOf: self","(\r\nfunction(self)\r\nreturn ((((self):smalltalk()):methodReferences()):globalReferencesOf_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"references",0
,(
function(self)
return ((
function(_s8_rec)
(_s8_rec):at_put_("messages", (self):messages());
(_s8_rec):at_put_("globalReferences", (self):globalReferences());
return ((_s8_rec):yourself())
end
)((smalltalk.PoolDictionary):new()))
;
end
)
,"Tools-accessing","references\r\n\t\" Return all the references in the receiver. \"\r\n\r\n\t^PoolDictionary new\r\n\t\tat: #messages put: self messages;\r\n\t\tat: #globalReferences put: self globalReferences;\r\n\t\tyourself","(\r\nfunction(self)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):at_put_(\"messages\", (self):messages());\r\n(_s8_rec):at_put_(\"globalReferences\", (self):globalReferences());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.PoolDictionary):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"builder:",0
,(
function(self,cookie)
(function() local _1_=(cookie); (self)["$builder"]=_1_; return _1_ end)()
;
return (self)
;
end
)
,"Builders-accessing","builder: cookie\r\n\t\" Private - Set the builder cookie of the receiver. \"\r\n\r\n\tself basicAt: '$builder' put: cookie","(\r\nfunction(self,cookie)\r\n(function() local _1_=(cookie); (self)[\"$builder\"]=_1_; return _1_ end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"builder",0
,(
function(self)
return (((self)["$builder"]))
;
end
)
,"Builders-accessing","builder\r\n\t\" Private - Return the builder cookie of the receiver. \"\r\n\r\n\t^self basicAt: '$builder'","(\r\nfunction(self)\r\nreturn (((self)[\"$builder\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompiledMethod,"__tostring",0
,(
function(self)
return (smalltalk:toString_((self)))
;
end
)
,"Lua-printing","__tostring\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number["$klass"],"pi",0
,(
function(self)
return (math.pi)
;
end
)
,"instantiation","pi\r\n\t\" Return the singular value PI. \"\r\n\r\n\t^{'Math.PI'\r\n\tlua'math.pi'}","(\r\nfunction(self)\r\nreturn (math.pi)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number["$klass"],"readFrom:radix:",0
,(
function(self,aStream,radix)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local value=nil
local char=nil
(function()
	local _s8_value=((0))
	value=_s8_value; return _s8_value
end)()
;
(function() while(((function()
(function()
	local _s8_value=((aStream):peek())
	char=_s8_value; return _s8_value
end)();return ((smalltalk:boolean_and_(((((char))~=nil)),(function()
return ((smalltalk:boolean_or_(((char):isDigit()),(function()
return ((char):isUpperCase())
end))))
end))))
end))()) do (function()
(function() if (((((radix) < (char):digitValue())))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (value)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();(function()
	local _s8_value=(((value):_star(radix)):_plus((char):digitValue()))
	value=_s8_value; return _s8_value
end)();return ((aStream):next())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (value)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","readFrom: aStream radix: radix\r\n\t\" Private - Read an integer from <aStream> and return it. \"\r\n\r\n\t| value char |\r\n\tvalue := 0.\r\n\t[\tchar := aStream peek.\r\n\t\tchar notNil and: [ char isDigit or: [ char isUpperCase ] ]\r\n\t] whileTrue: [\r\n\t\tradix <  char digitValue ifTrue: [ ^value ].\r\n\t\tvalue := value * radix + char digitValue.\r\n\t\taStream next\r\n\t].\r\n\t^value","(\r\nfunction(self,aStream,radix)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal value=nil\r\nlocal char=nil\r\n(function()\r\n\tlocal _s8_value=((0))\r\n\tvalue=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\n(function()\r\n\tlocal _s8_value=((aStream):peek())\r\n\tchar=_s8_value; return _s8_value\r\nend)();return ((smalltalk:boolean_and_(((((char))~=nil)),(function()\r\nreturn ((smalltalk:boolean_or_(((char):isDigit()),(function()\r\nreturn ((char):isUpperCase())\r\nend))))\r\nend))))\r\nend))()) do (function()\r\n(function() if (((((radix) < (char):digitValue())))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (value)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();(function()\r\n\tlocal _s8_value=(((value):_star(radix)):_plus((char):digitValue()))\r\n\tvalue=_s8_value; return _s8_value\r\nend)();return ((aStream):next())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (value)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Number["$klass"],"readFrom:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local value=nil
local char=nil
(function()
	local _s8_value=((0))
	value=_s8_value; return _s8_value
end)()
;
(function() while(((function()
(function()
	local _s8_value=((aStream):peek())
	char=_s8_value; return _s8_value
end)();return ((smalltalk:boolean_and_(((((char))~=nil)),(function()
return ((char):isDigit())
end))))
end))()) do (function()
(function()
	local _s8_value=(((value):_star((10))):_plus((char):digitValue()))
	value=_s8_value; return _s8_value
end)();return ((aStream):next())
end)() end;return nil end)()
;
(function() if (((((char)=="r")))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (value)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(aStream):next()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):readFrom_radix_(aStream, value))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","readFrom: aStream\r\n\t\" Private - Read an integer from <aStream> and return it. \"\r\n\r\n\t| value char |\r\n\tvalue := 0.\r\n\t[\tchar := aStream peek.\r\n\t\tchar notNil and: [ char isDigit ]\r\n\t] whileTrue: [\r\n\t\tvalue := value * 10 + char digitValue.\r\n\t\taStream next\r\n\t].\r\n\tchar == \"$r\"'r' ifFalse: [ ^value ].\r\n\taStream next. \"skip the r\"\r\n\t^self readFrom: aStream radix: value","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal value=nil\r\nlocal char=nil\r\n(function()\r\n\tlocal _s8_value=((0))\r\n\tvalue=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\n(function()\r\n\tlocal _s8_value=((aStream):peek())\r\n\tchar=_s8_value; return _s8_value\r\nend)();return ((smalltalk:boolean_and_(((((char))~=nil)),(function()\r\nreturn ((char):isDigit())\r\nend))))\r\nend))()) do (function()\r\n(function()\r\n\tlocal _s8_value=(((value):_star((10))):_plus((char):digitValue()))\r\n\tvalue=_s8_value; return _s8_value\r\nend)();return ((aStream):next())\r\nend)() end;return nil end)()\r\n;\r\n(function() if (((((char)==\"r\")))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (value)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(aStream):next()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):readFrom_radix_(aStream, value))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"=","_eq"
,(
function(self,anObject)
	if (type(anObject)=="number")
		then return self==anObject
		else return false
		end
;
return (self)
;
end
)
,"comparing","= anObject\r\n    \" Return true if the receiver is equal to argument. \"\r\n\r\n{'\tif (nil.isNil_(anObject)) return false;\r\n\tif (!(anObject.isNumber)) return false;\r\n\tif (!(anObject.isNumber())) return false;\r\n\tif (self == anObject) return true;\r\n\treturn ((0+self-anObject) == 0)'\r\nlua'\tif (type(anObject)==\"number\")\r\n\t\tthen return self==anObject\r\n\t\telse return false\r\n\t\tend'}","(\r\nfunction(self,anObject)\r\n\tif (type(anObject)==\"number\")\r\n\t\tthen return self==anObject\r\n\t\telse return false\r\n\t\tend\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"mustBeNumber",0
,(
function(self)
return (self)
;
end
)
,"testing","mustBeNumber\r\n\t\" Signal an error if the receiver is not a Number.\r\n\tWARNING: this check impose a severe loss of perfomance (five times slower).\r\n\tIf you are sure the operation/compare arguments are ok (a number),\r\n\t comment the implementation of this method making the code empty.\r\n\t\"\r\n\r\n\t\"(# #isNaN: self) ifTrue: [ super mustBeNumber ]\"","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,">","_gt"
,(
function(self,aNumber)
return (self>aNumber)
;
end
)
,"comparing","> aNumber\r\n\t\" Return true if the receiver is greater than argument. \"\r\n\r\n\t^{'self > aNumber.mustBeNumber()'\r\n\tlua'self>aNumber'}","(\r\nfunction(self,aNumber)\r\nreturn (self>aNumber)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"<","_lt"
,(
function(self,aNumber)
return (self<aNumber)
;
end
)
,"comparing","< aNumber\r\n\t\" Return true if the receiver is smaller than argument. \"\r\n\r\n\t^{'self < aNumber.mustBeNumber()'\r\n\tlua'self<aNumber'}","(\r\nfunction(self,aNumber)\r\nreturn (self<aNumber)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,">=","_gt_eq"
,(
function(self,aNumber)
return (self>=aNumber)
;
end
)
,"comparing",">= aNumber\r\n\t\" Return true if the receiver is greater or equal to argument. \"\r\n\r\n\t^{'self >= aNumber.mustBeNumber()'\r\n\tlua'self>=aNumber'}","(\r\nfunction(self,aNumber)\r\nreturn (self>=aNumber)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"<=","_lt_eq"
,(
function(self,aNumber)
return (self<=aNumber)
;
end
)
,"comparing","<= aNumber\r\n\t\" Return true if the receiver is smaller or equal to argument. \"\r\n\r\n\t^{'self <= aNumber.mustBeNumber()'\r\n\tlua'self<=aNumber'}","(\r\nfunction(self,aNumber)\r\nreturn (self<=aNumber)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"+","_plus"
,(
function(self,aNumber)
return (self+aNumber)
;
end
)
,"arithmetic","+ aNumber\r\n\t\" Return the result of operation. \"\r\n\r\n\t^{'self + aNumber.mustBeNumber()'\r\n\tlua'self+aNumber'}","(\r\nfunction(self,aNumber)\r\nreturn (self+aNumber)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"-","_minus"
,(
function(self,aNumber)
return (self-aNumber)
;
end
)
,"arithmetic","- aNumber\r\n\t\" Return the result of operation. \"\r\n\r\n\t^{'self - aNumber.mustBeNumber()'\r\n\tlua'self-aNumber'}","(\r\nfunction(self,aNumber)\r\nreturn (self-aNumber)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"*","_star"
,(
function(self,aNumber)
return (self*aNumber)
;
end
)
,"arithmetic","* aNumber\r\n\t\" Return the result of operation. \"\r\n\r\n\t^{'self * aNumber.mustBeNumber()'\r\n\tlua'self*aNumber'}","(\r\nfunction(self,aNumber)\r\nreturn (self*aNumber)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"/","_slash"
,(
function(self,aNumber)
return (self/aNumber)
;
end
)
,"arithmetic","/ aNumber\r\n\t\" Return the result of operation. \"\r\n\r\n\t^{'self / aNumber.mustBeNumber()'\r\n\tlua'self/aNumber'}","(\r\nfunction(self,aNumber)\r\nreturn (self/aNumber)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"divideByZero",0
,(
function(self)
return ((self):error_("Divide by Zero"))
;
end
)
,"signaling","divideByZero\r\n\t\" Signal divide by cero attempt. \"\r\n\r\n\t^self error: 'Divide by Zero'","(\r\nfunction(self)\r\nreturn ((self):error_(\"Divide by Zero\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"\\\\","_bk_bk"
,(
function(self,aNumber)
(aNumber):mustBeNumber()
;
(function() if ((((smalltalk:isObject_equalTo_((aNumber),(0)))))==true) then return ((function()
return ((self):divideByZero())
end))() else return nil end end)()
;
return ((self):_minus(((self):_slash_slash(aNumber)):_star(aNumber)))
;
end
)
,"arithmetic","\\\\\\\\ aNumber\r\n\t\" Return the result of operation. \"\r\n\r\n\taNumber mustBeNumber.\r\n\taNumber = 0 ifTrue: [ self divideByZero ].\r\n\t^self - (self // aNumber * aNumber)","(\r\nfunction(self,aNumber)\r\n(aNumber):mustBeNumber()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((aNumber),(0)))))==true) then return ((function()\r\nreturn ((self):divideByZero())\r\nend))() else return nil end end)()\r\n;\r\nreturn ((self):_minus(((self):_slash_slash(aNumber)):_star(aNumber)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"//","_slash_slash"
,(
function(self,aNumber)
(aNumber):mustBeNumber()
;
(function() if ((((smalltalk:isObject_equalTo_((aNumber),(0)))))==true) then return ((function()
return ((self):divideByZero())
end))() else return nil end end)()
;
return (((self):_slash(aNumber)):truncated())
;
end
)
,"arithmetic","// aNumber\r\n\t\" Return the result of operation. \"\r\n\r\n\taNumber mustBeNumber.\r\n\taNumber = 0 ifTrue: [ self divideByZero ].\r\n\t^(self / aNumber) truncated","(\r\nfunction(self,aNumber)\r\n(aNumber):mustBeNumber()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((aNumber),(0)))))==true) then return ((function()\r\nreturn ((self):divideByZero())\r\nend))() else return nil end end)()\r\n;\r\nreturn (((self):_slash(aNumber)):truncated())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"sin",0
,(
function(self)
return (math.sin(self))
;
end
)
,"arithmetic","sin\r\n\t^{'Math.sin(self)'\r\n\tlua'math.sin(self)'}","(\r\nfunction(self)\r\nreturn (math.sin(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"cos",0
,(
function(self)
return (math.cos(self))
;
end
)
,"arithmetic","cos\r\n\t^{'Math.cos(self)'\r\n\tlua'math.cos(self)'}","(\r\nfunction(self)\r\nreturn (math.cos(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"tan",0
,(
function(self)
return (math.tan(self))
;
end
)
,"arithmetic","tan\r\n\t^{'Math.tan(self)'\r\n\tlua'math.tan(self)'}","(\r\nfunction(self)\r\nreturn (math.tan(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"arcCos",0
,(
function(self)
return (math.acos(self))
;
end
)
,"arithmetic","arcCos\r\n\t^{'Math.acos(self)'\r\n\tlua'math.acos(self)'}","(\r\nfunction(self)\r\nreturn (math.acos(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"arcSin",0
,(
function(self)
return (math.asin(self))
;
end
)
,"arithmetic","arcSin\r\n\t^{'Math.asin(self)'\r\n\tlua'math.asin(self)'}","(\r\nfunction(self)\r\nreturn (math.asin(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"arcTan",0
,(
function(self)
return (math.atan(self))
;
end
)
,"arithmetic","arcTan\r\n\t^{'Math.atan(self)'\r\n\tlua'math.atan(self)'}","(\r\nfunction(self)\r\nreturn (math.atan(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"atan2:",0
,(
function(self,aNumber)
(aNumber):mustBeNumber()
;
return (math.atan2(self,aNumber))
;
end
)
,"arithmetic","atan2: aNumber\r\n\r\n\taNumber mustBeNumber.\r\n\t^{'Math.atan2(self,aNumber)'\r\n\tlua'math.atan2(self,aNumber)'}","(\r\nfunction(self,aNumber)\r\n(aNumber):mustBeNumber()\r\n;\r\nreturn (math.atan2(self,aNumber))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"negated",0
,(
function(self)
return (((-1)):_star(self))
;
end
)
,"arithmetic","negated\r\n\t^-1 * self ","(\r\nfunction(self)\r\nreturn (((-1)):_star(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"squared",0
,(
function(self)
return ((self):_star(self))
;
end
)
,"arithmetic","squared\r\n\t^self * self ","(\r\nfunction(self)\r\nreturn ((self):_star(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"sqrt",0
,(
function(self)
return (math.sqrt(self))
;
end
)
,"arithmetic","sqrt\r\n\t^{'Math.sqrt(self)'\r\n\tlua'math.sqrt(self)'}","(\r\nfunction(self)\r\nreturn (math.sqrt(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"<<","_lt_lt"
,(
function(self,aNumber)
return (bit.lshift(self,aNumber))
;
end
)
,"arithmetic","<< aNumber\r\n\t\" Return the receiver shifted aNumber of bits to the left. \"\r\n\r\n\t^{'self << aNumber.mustBeNumber()'\r\n\tlua'bit.lshift(self,aNumber)'}","(\r\nfunction(self,aNumber)\r\nreturn (bit.lshift(self,aNumber))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,">>","_gt_gt"
,(
function(self,aNumber)
return (bit.rshift(self,aNumber))
;
end
)
,"arithmetic",">> aNumber\r\n\t\" Return the receiver shifted aNumber of bits to the right. \"\r\n\r\n\t^{'self >> aNumber.mustBeNumber()'\r\n\tlua'bit.rshift(self,aNumber)'}","(\r\nfunction(self,aNumber)\r\nreturn (bit.rshift(self,aNumber))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"bitShift:",0
,(
function(self,anInteger)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((anInteger) < (0))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):_gt_gt((anInteger):negated()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):_lt_lt(anInteger))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"arithmetic","bitShift: anInteger\r\n\t\" Return the receiver shifted left anInteger number of bits if anInteger is positive, or shifted right for anInteger negated number of bits if anInteger is negative.\"\r\n\r\n\tanInteger < 0 ifTrue: [ ^self >> anInteger negated ].\r\n\t^self << anInteger","(\r\nfunction(self,anInteger)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((anInteger) < (0))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):_gt_gt((anInteger):negated()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):_lt_lt(anInteger))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"max:",0
,(
function(self,aNumber)
return (math.max(self,aNumber))
;
end
)
,"arithmetic","max: aNumber\r\n\t\" Return the maximum between the receiver and argument. \"\r\n\r\n\t^{'Math.max(self,aNumber)'\r\n\tlua'math.max(self,aNumber)'}","(\r\nfunction(self,aNumber)\r\nreturn (math.max(self,aNumber))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"min:",0
,(
function(self,aNumber)
return (math.min(self,aNumber))
;
end
)
,"arithmetic","min: aNumber\r\n\t\" Return the minimum between the receiver and argument. \"\r\n\r\n\t^{'Math.min(self,aNumber)'\r\n\tlua'math.min(self,aNumber)'}","(\r\nfunction(self,aNumber)\r\nreturn (math.min(self,aNumber))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"asNumber",0
,(
function(self)
return ((self):_plus((0)))
;
end
)
,"converting","asNumber\r\n\t\" Return the receiver as a Number. \"\r\n\r\n\t^self + 0","(\r\nfunction(self)\r\nreturn ((self):_plus((0)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"abs",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((self) < (0))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):negated())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","abs\r\n\t\" Return the absolute value of the receiver. \"\r\n\r\n\tself < 0 ifTrue: [ ^self negated ].\r\n\t^self","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((self) < (0))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):negated())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"rounded",0
,(
function(self)
return (self + 0.5 - (self + 0.5) % 1)
;
end
)
,"converting","rounded\r\n\t\" Return the receiver rounded. \"\r\n\r\n\t^{'Math.round(self)'\r\n\tlua'self + 0.5 - (self + 0.5) % 1'}","(\r\nfunction(self)\r\nreturn (self + 0.5 - (self + 0.5) % 1)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"truncated",0
,(
function(self)
return (math.floor(self))
;
end
)
,"converting","truncated\r\n\t\" Return the receiver truncated. \"\r\n\r\n\t^{'Math.floor(self)'\r\n\tlua'math.floor(self)'}","(\r\nfunction(self)\r\nreturn (math.floor(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"asCharacter",0
,(
function(self)
return ((smalltalk.String):fromCharCode_(self))
;
end
)
,"converting","asCharacter\r\n\t\" Return the character with the receiver's code. \"\r\n\r\n\t^String fromCharCode: self","(\r\nfunction(self)\r\nreturn ((smalltalk.String):fromCharCode_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"to:",0
,(
function(self,aNumber)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local array=nil
local first=nil
local last=nil
local count=nil
(function()
	local _s8_value=((self):truncated())
	first=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((aNumber):truncated()):_plus((1)))
	last=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((1))
	count=_s8_value; return _s8_value
end)()
;
(function() if (((((first) <= last)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Wrong interval"))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
	local _s8_value=((smalltalk.Array):new())
	array=_s8_value; return _s8_value
end)()
;
(function() for _1_=1,(((last):_minus(first))) do (function()
(array):at_put_(count, first);(function()
	local _s8_value=((count):_plus((1)))
	count=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=((first):_plus((1)))
	first=_s8_value; return _s8_value
end)())
end)(_1_) end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (array)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","to: aNumber\r\n\t\" Return a collection of numbers from the receiver to aNumber. \"\r\n\r\n\t| array first last count |\r\n\tfirst := self truncated.\r\n\tlast := aNumber truncated + 1.\r\n\tcount := 1.\r\n\tfirst <= last ifFalse: [ ^self error: 'Wrong interval' ].\r\n\tarray := Array new.\r\n\tlast - first timesRepeat: [\r\n\t\tarray at: count put: first.\r\n\t\tcount := count + 1.\r\n\t\tfirst := first + 1\r\n\t].\r\n\t^array","(\r\nfunction(self,aNumber)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal array=nil\r\nlocal first=nil\r\nlocal last=nil\r\nlocal count=nil\r\n(function()\r\n\tlocal _s8_value=((self):truncated())\r\n\tfirst=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((aNumber):truncated()):_plus((1)))\r\n\tlast=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((1))\r\n\tcount=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((((first) <= last)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Wrong interval\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tarray=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() for _1_=1,(((last):_minus(first))) do (function()\r\n(array):at_put_(count, first);(function()\r\n\tlocal _s8_value=((count):_plus((1)))\r\n\tcount=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=((first):_plus((1)))\r\n\tfirst=_s8_value; return _s8_value\r\nend)())\r\nend)(_1_) end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (array)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"timesRepeat:",0
,(
function(self,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local count=nil
	if true then
		for i=1,self do aBlock(i) end
		return self
	end
;
(function() if ((((smalltalk:isObject_equalTo_(((aBlock):argumentCount()),(1)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((1)):to_do_(self, aBlock))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((1))
	count=_s8_value; return _s8_value
end)()
;
(function() while(not ((function()
return (((count) > self))
end))()) do (function()
(aBlock):value();return ((function()
	local _s8_value=((count):_plus((1)))
	count=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"enumerating","timesRepeat: aBlock\r\n\t\" Evaluates aBlock a number of times given by the receiver.\r\n\tReturn the receiver.\r\n\t\"\r\n\r\n\t| count |\r\n{''lua'\tif true then\r\n\t\tfor i=1,self do aBlock(i) end\r\n\t\treturn self\r\n\tend'}.\r\n\taBlock argumentCount = 1 ifTrue: [\r\n\t\t^1 to: self do: aBlock\r\n\t].\r\n\tcount := 1.\r\n\t[ count > self ] whileFalse: [\r\n\t\taBlock value.\r\n\t\tcount := count + 1\r\n\t]","(\r\nfunction(self,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal count=nil\r\n\tif true then\r\n\t\tfor i=1,self do aBlock(i) end\r\n\t\treturn self\r\n\tend\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((aBlock):argumentCount()),(1)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((1)):to_do_(self, aBlock))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((1))\r\n\tcount=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(not ((function()\r\nreturn (((count) > self))\r\nend))()) do (function()\r\n(aBlock):value();return ((function()\r\n\tlocal _s8_value=((count):_plus((1)))\r\n\tcount=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"to:do:",0
,(
function(self,aNumber,aBlock)
	for i=self,aNumber do aBlock(i) end
;
return (self)
;
end
)
,"iterating","to: aNumber do: aBlock\r\n\t\" Evaluate aBlock from the receiver to aNumber.\r\n\tReturn the receiver.\r\n\t\"\r\n\r\n{'\tfor(var i=self;i<=aNumber;i++){aBlock(i);}'\r\nlua'\tfor i=self,aNumber do aBlock(i) end'}","(\r\nfunction(self,aNumber,aBlock)\r\n\tfor i=self,aNumber do aBlock(i) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"to:by:do:",0
,(
function(self,aNumber,increment,aBlock)
	for i=self,aNumber,increment do aBlock(i) end
;
return (self)
;
end
)
,"iterating","to: aNumber by: increment do: aBlock\r\n\t\" Evaluate aBlock for the numbers between the receiver and aNumber where each number is the previous number plus increment.\r\n\tReturn the receiver.\r\n\t\"\r\n\r\n{'\tif (increment>0) { for(var i=self;i<=aNumber;i=i+increment){aBlock(i);} }\r\n\telse { for(var i=self;aNumber<=i;i=i+increment){aBlock(i);} }'\r\nlua'\tfor i=self,aNumber,increment do aBlock(i) end'}","(\r\nfunction(self,aNumber,increment,aBlock)\r\n\tfor i=self,aNumber,increment do aBlock(i) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"asString",0
,(
function(self)
return (smalltalk:printString_((self)))
;
end
)
,"converting","asString\r\n\t\" Return the receiver as a String. \"\r\n\r\n\t^self printString","(\r\nfunction(self)\r\nreturn (smalltalk:printString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"asNative:",0
,(
function(self,lang)
return (smalltalk:object_comma_((smalltalk:object_comma_(("("),smalltalk:printString_((self)))),")"))
;
end
)
,"converting","asNative: lang\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\t^'(', self printString, ')'","(\r\nfunction(self,lang)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"(\"),smalltalk:printString_((self)))),\")\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"printString",0
,(
function(self)
return (tostring(self))
;
end
)
,"printing","printString\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^{'String(self)'\r\n\tlua'tostring(self)'}","(\r\nfunction(self)\r\nreturn (tostring(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"toString:",0
,(
function(self,radix)
return (smalltalk:basen(self,radix))
;
end
)
,"printing","toString: radix\r\n\t\" Return the ASCII representation of the receiver in base radix. \"\r\n\r\n\t^{'self.toString(radix)'\r\n\tlua'smalltalk:basen(self,radix)'}","(\r\nfunction(self,radix)\r\nreturn (smalltalk:basen(self,radix))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"printPaddedWith:to:base:",0
,(
function(self,aCharacter,anInteger,aRadix)
local result=nil
(function()
	local _s8_value=((self):toString_(aRadix))
	result=_s8_value; return _s8_value
end)()
;
(function() while(((function()
return ((((result):size()) < anInteger))
end))()) do (function()
return ((function()
	local _s8_value=(smalltalk:object_comma_((aCharacter),result))
	result=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
return (result)
;
end
)
,"printing","printPaddedWith: aCharacter to: anInteger base: aRadix\r\n\t\" Return the ASCII representation of the receiver padded on the left with aCharacter to be at least anInteger characters. \"\r\n\r\n\t| result |\r\n\tresult := self toString: aRadix.\r\n\t[ result size < anInteger ] whileTrue: [\r\n\t\tresult := aCharacter ,result\r\n\t].\r\n\t^result","(\r\nfunction(self,aCharacter,anInteger,aRadix)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):toString_(aRadix))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\nreturn ((((result):size()) < anInteger))\r\nend))()) do (function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((aCharacter),result))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"isNumber",0
,(
function(self)
	if (self~=self) then return false else return (self~=math.huge and self~=-math.huge) end
;
return (self)
;
end
)
,"testing","isNumber\r\n\t\" Return true if the receiver is a Number. \"\r\n\r\n{'\treturn isNaN(self).not()'\r\nlua'\tif (self~=self) then return false else return (self~=math.huge and self~=-math.huge) end'}","(\r\nfunction(self)\r\n\tif (self~=self) then return false else return (self~=math.huge and self~=-math.huge) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"atRandom",0
,(
function(self)
return ((((((smalltalk.Random):new()):next()):_star(self)):truncated()):_plus((1)))
;
end
)
,"converting","atRandom\r\n\t\" Return a random integer in the interval from 1 to the receiver. \"\r\n\r\n\t^(Random new next * self) truncated + 1","(\r\nfunction(self)\r\nreturn ((((((smalltalk.Random):new()):next()):_star(self)):truncated()):_plus((1)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"@","_at"
,(
function(self,aNumber)
return ((smalltalk.Point):x_y_(self, (function() if ((((((aNumber))==nil)))==true) then return ((function()
return ((0))
end))() else return ((function()
return (aNumber)
end))() end end)()))
;
end
)
,"converting","@ aNumber\r\n\t\" Return a Point with coordinates given by the receiver and aNumber. \"\r\n\r\n\t^Point x: self y: (aNumber isNil ifTrue: [0] ifFalse: [aNumber])","(\r\nfunction(self,aNumber)\r\nreturn ((smalltalk.Point):x_y_(self, (function() if ((((((aNumber))==nil)))==true) then return ((function()\r\nreturn ((0))\r\nend))() else return ((function()\r\nreturn (aNumber)\r\nend))() end end)()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"asPoint",0
,(
function(self)
return ((smalltalk.Point):x_y_(self, self))
;
end
)
,"converting","asPoint\r\n\t\" Return the receiver as a Point. \"\r\n\r\n\t^Point x: self y: self","(\r\nfunction(self)\r\nreturn ((smalltalk.Point):x_y_(self, self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"asJSONObject",0
,(
function(self)
return ((self):_plus((0)))
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self + 0\r\n\"Note:\twe need to normalize result to do not destroy typeof checkings\r\n\tDo NOT modify this method if you do not know what you are doing.\r\n\tIf you remove the + 0 computation, the returned object will return 'object' and not 'number' as result of typeof operation.\r\n\"","(\r\nfunction(self)\r\nreturn ((self):_plus((0)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"between:and:",0
,(
function(self,min,max)
return ((smalltalk:boolean_and_((((min) <= self)),(function()
return (((self) <= max))
end))))
;
end
)
,"testing","between: min and: max\r\n\t\" Return true if the receiver is between arguments. \"\r\n\r\n\t^min <= self and: [ self <= max ]","(\r\nfunction(self,min,max)\r\nreturn ((smalltalk:boolean_and_((((min) <= self)),(function()\r\nreturn (((self) <= max))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"even",0
,(
function(self)
return ((smalltalk:isObject_equalTo_(((self):_bk_bk((2))),(0))))
;
end
)
,"testing","even\r\n\t\" Return true if the receiver is an even number. \"\r\n\r\n\t^self \\\\ 2 = 0","(\r\nfunction(self)\r\nreturn ((smalltalk:isObject_equalTo_(((self):_bk_bk((2))),(0))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"odd",0
,(
function(self)
return ((smalltalk:isObject_equalTo_(((self):_bk_bk((2))),(1))))
;
end
)
,"testing","odd\r\n\t\" Return true if the receiver is an odd number. \"\r\n\r\n\t^self \\\\ 2 = 1","(\r\nfunction(self)\r\nreturn ((smalltalk:isObject_equalTo_(((self):_bk_bk((2))),(1))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"clearInterval",0
,(
function(self)
self:notImplementedYet_("clearInterval")
;
return (self)
;
end
)
,"intervals","clearInterval\r\n\r\n\t{'clearInterval(Number(self))'\r\n\tlua'self:notImplementedYet_(\"clearInterval\")'}","(\r\nfunction(self)\r\nself:notImplementedYet_(\"clearInterval\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"shallowCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"deepCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"asLua",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_(("("),smalltalk:printString_((self)))),")"))
;
end
)
,"Lua-lua","asLua\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\t^'(', self printString, ')'","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"(\"),smalltalk:printString_((self)))),\")\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Number,"s8DoIt",0
,(
function(self)
return (((function()
local image=nil
local tag=nil
local source=nil
(function()
	local _s8_value=((
function(_s8_rec)
(_s8_rec):encoder_((smalltalk.LuaEncoder):new());
return ((_s8_rec):exportAll())
end
)((smalltalk.Exporter):new()))
	image=_s8_value; return _s8_value
end)();(image):outputToFile_("../s8.image.lua");(self):print_("// Collecting methods with errors...");(function()
	local _s8_value=("NIY:#luaMethodBody: ")
	tag=_s8_value; return _s8_value
end)();(self):print_(smalltalk:object_comma_(("// ToDo list for "),tag));(function()
	local _s8_value=((image):stream())
	source=_s8_value; return _s8_value
end)();return ((function() while(not ((function()
return ((source):atEnd())
end))()) do (function()
(source):upTo_(tag);return ((function() if ((((source):atEnd()))==true) then return nil else return ((function()
return ((self):print_(smalltalk:object_comma_(("\t"),(source):upTo_("\""))))
end))() end end)())
end)() end;return nil end)())
end)):value())
;
end
)
,"","s8DoIt ^[| image tag source |\r\nimage := Exporter new\r\n\tencoder: LuaEncoder new;\r\n\texportAll.\r\nimage outputToFile: '../s8.image.lua'.\r\n\r\nself print: '// Collecting methods with errors...'.\r\ntag := 'NIY:#luaMethodBody: '.\r\nself print: '// ToDo list for ',tag.\r\nsource := image stream.\r\n[source atEnd] whileFalse: [\r\n\tsource upTo: tag.\r\n\tsource atEnd ifFalse: [\r\n\t\tself print: '\t',(source upTo: $\")\r\n\t].\r\n].] value","(\r\nfunction(self)\r\nreturn (((function()\r\nlocal image=nil\r\nlocal tag=nil\r\nlocal source=nil\r\n(function()\r\n\tlocal _s8_value=((\r\nfunction(_s8_rec)\r\n(_s8_rec):encoder_((smalltalk.LuaEncoder):new());\r\nreturn ((_s8_rec):exportAll())\r\nend\r\n)((smalltalk.Exporter):new()))\r\n\timage=_s8_value; return _s8_value\r\nend)();(image):outputToFile_(\"../s8.image.lua\");(self):print_(\"// Collecting methods with errors...\");(function()\r\n\tlocal _s8_value=(\"NIY:#luaMethodBody: \")\r\n\ttag=_s8_value; return _s8_value\r\nend)();(self):print_(smalltalk:object_comma_((\"// ToDo list for \"),tag));(function()\r\n\tlocal _s8_value=((image):stream())\r\n\tsource=_s8_value; return _s8_value\r\nend)();return ((function() while(not ((function()\r\nreturn ((source):atEnd())\r\nend))()) do (function()\r\n(source):upTo_(tag);return ((function() if ((((source):atEnd()))==true) then return nil else return ((function()\r\nreturn ((self):print_(smalltalk:object_comma_((\"\\t\"),(source):upTo_(\"\\\"\"))))\r\nend))() end end)())\r\nend)() end;return nil end)())\r\nend)):value())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure["$klass"],"hasDeferredSupport",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
((function()
return ((((setTimeout))~=nil))
end)):on_do_(smalltalk.Error, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((((setTimeout))~=nil))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"deferredEvaluation","hasDeferredSupport\r\n\t\" Return true if the platform can defer instance evaluation. \"\r\n\r\n\t[ #{setTimeout} notNil ] on: Error do: [ ^false ].\r\n\t^#{setTimeout} notNil","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n((function()\r\nreturn ((((setTimeout))~=nil))\r\nend)):on_do_(smalltalk.Error, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((((setTimeout))~=nil))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"compiledSourceAt:",0
,(
function(self,aCompiledMethod)
local cached=nil
(function()
	local _s8_value=((function() if ((((((aCompiledMethod))~=nil)))==true) then return ((function()
return ((function() local _1_=((aCompiledMethod)["$nativeSource"]);if (_1_==nil) then return ((function()
return (nil)
end)):value() end; return _1_ end)())
end))() else return nil end end)())
	cached=_s8_value; return _s8_value
end)()
;
return ((function() if ((((((cached))==nil)))==true) then return ((function()
return ((self):compiledSource())
end))() else return ((function()
return (cached)
end))() end end)())
;
end
)
,"accessing","compiledSourceAt: aCompiledMethod\r\n\t\" Return the native source code of the receiver. \"\r\n\r\n\t| cached |\r\n\tcached := aCompiledMethod notNil ifTrue: [\r\n\t\taCompiledMethod basicAt: '$nativeSource' ifAbsent: []\r\n\t].\r\n\t^cached isNil ifTrue: [ self compiledSource ] ifFalse: [ cached ]","(\r\nfunction(self,aCompiledMethod)\r\nlocal cached=nil\r\n(function()\r\n\tlocal _s8_value=((function() if ((((((aCompiledMethod))~=nil)))==true) then return ((function()\r\nreturn ((function() local _1_=((aCompiledMethod)[\"$nativeSource\"]);if (_1_==nil) then return ((function()\r\nreturn (nil)\r\nend)):value() end; return _1_ end)())\r\nend))() else return nil end end)())\r\n\tcached=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((function() if ((((((cached))==nil)))==true) then return ((function()\r\nreturn ((self):compiledSource())\r\nend))() else return ((function()\r\nreturn (cached)\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"compiledSource",0
,(
function(self)
return (smalltalk:toString_((self)))
;
end
)
,"accessing","compiledSource\r\n\t\" Return the native source code of the receiver. \"\r\n\r\n\t^self toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"whileTrue:",0
,(
function(self,aBlock)
	while self() do aBlock() end
;
return (self)
;
end
)
,"iterating","whileTrue: aBlock\r\n\t\" Evaluate aBlock while the result of evaluating the receiver is true. \"\r\n\r\n{'\twhile(self()) {aBlock()}'\r\nlua'\twhile self() do aBlock() end'}","(\r\nfunction(self,aBlock)\r\n\twhile self() do aBlock() end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"whileFalse:",0
,(
function(self,aBlock)
	while (self()==false) do aBlock() end
;
return (self)
;
end
)
,"iterating","whileFalse: aBlock\r\n\t\" Evaluate aBlock while the result of evaluating the receiver is false. \"\r\n\r\n{'\twhile(!self()) {aBlock()}'\r\nlua'\twhile (self()==false) do aBlock() end'}","(\r\nfunction(self,aBlock)\r\n\twhile (self()==false) do aBlock() end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"value",0
,(
function(self)
return (self())
;
end
)
,"evaluating","value\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t^# #self","(\r\nfunction(self)\r\nreturn (self())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"value:",0
,(
function(self,anObject)
return (self(anObject))
;
end
)
,"evaluating","value: anObject\r\n\t\" Return the result of evaluating the receiver with anObject. \"\r\n\r\n\t^# #self: anObject","(\r\nfunction(self,anObject)\r\nreturn (self(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"value:value:",0
,(
function(self,firstArgument,secondArgument)
return (self(firstArgument, secondArgument))
;
end
)
,"evaluating","value: firstArgument value: secondArgument\r\n\t\" Return the result of evaluating the receiver with arguments. \"\r\n\r\n\t^# #self: firstArgument value: secondArgument","(\r\nfunction(self,firstArgument,secondArgument)\r\nreturn (self(firstArgument, secondArgument))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"value:value:value:",0
,(
function(self,firstArgument,secondArgument,thirdArgument)
return (self(firstArgument, secondArgument, thirdArgument))
;
end
)
,"evaluating","value: firstArgument value: secondArgument value: thirdArgument\r\n\t\" Return the result of evaluating the receiver with arguments. \"\r\n\r\n\t^# #self: firstArgument value: secondArgument value: thirdArgument","(\r\nfunction(self,firstArgument,secondArgument,thirdArgument)\r\nreturn (self(firstArgument, secondArgument, thirdArgument))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"value:value:value:value:",0
,(
function(self,firstArgument,secondArgument,thirdArgument,fourthArgument)
return (self(firstArgument, secondArgument, thirdArgument, fourthArgument))
;
end
)
,"evaluating","value: firstArgument value: secondArgument value: thirdArgument value: fourthArgument\r\n\t\" Return the result of evaluating the receiver with arguments. \"\r\n\r\n\t^# #self: firstArgument value: secondArgument value: thirdArgument value: fourthArgument","(\r\nfunction(self,firstArgument,secondArgument,thirdArgument,fourthArgument)\r\nreturn (self(firstArgument, secondArgument, thirdArgument, fourthArgument))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"valueWithPossibleArguments:",0
,(
function(self,aCollection)
(function() if ((((aCollection):isArray()))==true) then return nil else return ((function()
return ((self):error_(smalltalk:object_comma_(("Invalid arguments in #valueWithPossibleArguments: (must be anArray). receiver="),smalltalk:toString_((self)))))
end))() end end)()
;
return (self((table.unpack or unpack)(aCollection)))
;
end
)
,"evaluating","valueWithPossibleArguments: aCollection\r\n\t\" Return the result of evaluating the receiver with potential arguments. \"\r\n\r\n\taCollection isArray ifFalse: [\r\n\t\tself error: 'Invalid arguments in #valueWithPossibleArguments: (must be anArray). receiver=',self toString\r\n\t].\r\n\t^{'self.apply(null,aCollection)'\r\n\tlua'self((table.unpack or unpack)(aCollection))'}","(\r\nfunction(self,aCollection)\r\n(function() if ((((aCollection):isArray()))==true) then return nil else return ((function()\r\nreturn ((self):error_(smalltalk:object_comma_((\"Invalid arguments in #valueWithPossibleArguments: (must be anArray). receiver=\"),smalltalk:toString_((self)))))\r\nend))() end end)()\r\n;\r\nreturn (self((table.unpack or unpack)(aCollection)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"on:do:",0
,(
function(self,anExceptionType,aBlock)
return ((self):try_catch_(self, (function(error)
return ((function() if (((smalltalk:is_kindOf_((error),(anExceptionType))))==true) then return ((function()
return ((aBlock):value_(error))
end))() else return ((function()
return ((error):signal())
end))() end end)())
end)))
;
end
)
,"error","on: anExceptionType do: aBlock\r\n\t\" Return the result of evaluating the receiver catching exceptions. \"\r\n\r\n\t^self try: self catch: [:error |\r\n\t\t(error isKindOf: anExceptionType)\r\n\t\tifTrue: [ aBlock value: error ]\r\n\t\tifFalse: [ error signal ]\r\n\t]","(\r\nfunction(self,anExceptionType,aBlock)\r\nreturn ((self):try_catch_(self, (function(error)\r\nreturn ((function() if (((smalltalk:is_kindOf_((error),(anExceptionType))))==true) then return ((function()\r\nreturn ((aBlock):value_(error))\r\nend))() else return ((function()\r\nreturn ((error):signal())\r\nend))() end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"try:catch:",0
,(
function(self,aBlock,anotherBlock)
	local success, result = pcall(function() return aBlock() end)
	if success
		then return result
		else return anotherBlock( self:smalltalkErrorOf_(result) ) end

;
return (self)
;
end
)
,"private","try: aBlock catch: anotherBlock\r\n\t\" Private - Try to evaluate aBlock catching exceptions with catchBlock. \"\r\n\r\n{'\tvar result;\r\n\ttry{ result = aBlock() } catch(e) {\r\n\t\tresult = anotherBlock( self.smalltalkErrorOf_(e) )\r\n\t};\r\n\treturn result'\r\nlua'\tlocal success, result = pcall(function() return aBlock() end)\r\n\tif success\r\n\t\tthen return result\r\n\t\telse return anotherBlock( self:smalltalkErrorOf_(result) ) end\r\n'}","(\r\nfunction(self,aBlock,anotherBlock)\r\n\tlocal success, result = pcall(function() return aBlock() end)\r\n\tif success\r\n\t\tthen return result\r\n\t\telse return anotherBlock( self:smalltalkErrorOf_(result) ) end\r\n\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"valueDeferred:",0
,(
function(self,milliseconds)
(self):valueWithTimeout_(milliseconds)
;
return (self)
;
end
)
,"evaluating","valueDeferred: milliseconds\r\n\t\" Evaluate the receiver deferred in time.\r\n\tReturn the receiver.\r\n\t\"\r\n\r\n\tself valueWithTimeout: milliseconds","(\r\nfunction(self,milliseconds)\r\n(self):valueWithTimeout_(milliseconds)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"valueDeferred",0
,(
function(self)
(self):valueDeferred_((0))
;
return (self)
;
end
)
,"evaluating","valueDeferred\r\n\t\" Evaluate the receiver deferred in time.\r\n\tReturn the receiver.\r\n\t\"\r\n\r\n\tself valueDeferred: 0","(\r\nfunction(self)\r\n(self):valueDeferred_((0))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"valueWithTimeout:",0
,(
function(self,aNumber)
return (self:notImplementedYet_("valueWithTimeout:"))
;
end
)
,"timeout","valueWithTimeout: aNumber\r\n\t\" Evaluate the receiver with time out. \"\r\n\r\n\t^{'setTimeout(self,aNumber)'\r\n\tlua'self:notImplementedYet_(\"valueWithTimeout:\")'}","(\r\nfunction(self,aNumber)\r\nreturn (self:notImplementedYet_(\"valueWithTimeout:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"valueWithInterval:",0
,(
function(self,aNumber)
return (self:notImplementedYet_("valueWithInterval:"))
;
end
)
,"timeout","valueWithInterval: aNumber\r\n\t\" Evaluate the receiver with interval. \"\r\n\r\n\t^{'setInterval(self,aNumber)'\r\n\tlua'self:notImplementedYet_(\"valueWithInterval:\")'}","(\r\nfunction(self,aNumber)\r\nreturn (self:notImplementedYet_(\"valueWithInterval:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"ensure:",0
,(
function(self,aBlock)
	local success, result = pcall(function() return self() end)
	aBlock()
	if (true) then return result end

;
return (self)
;
end
)
,"error","ensure: aBlock\r\n\t\" Return the result of evaluating the receiver.\r\n\tExecute aBlock after the receiver is executed.\r\n\t\"\r\n\r\n{'\ttry{return self();} finally {aBlock();}'\r\nlua'\tlocal success, result = pcall(function() return self() end)\r\n\taBlock()\r\n\tif (true) then return result end\r\n'}","(\r\nfunction(self,aBlock)\r\n\tlocal success, result = pcall(function() return self() end)\r\n\taBlock()\r\n\tif (true) then return result end\r\n\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"ifCurtailed:",0
,(
function(self,exceptionBlock)
local curtailed=nil
local result=nil
(function()
	local _s8_value=(true)
	curtailed=_s8_value; return _s8_value
end)()
;
	local curtailed = true
	local success, result = pcall(function()
		local r = self()
		curtailed = false
		return r
	end)
	if (curtailed)
		then exceptionBlock()
		else return result
		end
;
return (self)
;
end
)
,"evaluating","ifCurtailed: exceptionBlock\r\n\t\" Return the result of evaluating the receiver.\r\n\tIf evaluation is limited by exiting before completion, the exception block is evaluated.\r\n\t\"\r\n\r\n\t| curtailed result |\r\n\tcurtailed := true.\r\n{'\ttry{\tresult = self.value();\r\n\t\tcurtailed = false;\r\n\t} finally {\r\n\t\tif (curtailed) exceptionBlock.value();\r\n\t};\r\n\treturn result'\r\nlua'\tlocal curtailed = true\r\n\tlocal success, result = pcall(function()\r\n\t\tlocal r = self()\r\n\t\tcurtailed = false\r\n\t\treturn r\r\n\tend)\r\n\tif (curtailed)\r\n\t\tthen exceptionBlock()\r\n\t\telse return result\r\n\t\tend'}","(\r\nfunction(self,exceptionBlock)\r\nlocal curtailed=nil\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(true)\r\n\tcurtailed=_s8_value; return _s8_value\r\nend)()\r\n;\r\n\tlocal curtailed = true\r\n\tlocal success, result = pcall(function()\r\n\t\tlocal r = self()\r\n\t\tcurtailed = false\r\n\t\treturn r\r\n\tend)\r\n\tif (curtailed)\r\n\t\tthen exceptionBlock()\r\n\t\telse return result\r\n\t\tend\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"argumentCount",0
,(
function(self)
return (self:notImplementedYet_("argumentCount"))
;
end
)
,"evaluating","argumentCount\r\n\t\" Return the number of arguments defined by the receiver. \"\r\n\r\n\t^{'self.length'\r\n\tlua'self:notImplementedYet_(\"argumentCount\")'}","(\r\nfunction(self)\r\nreturn (self:notImplementedYet_(\"argumentCount\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"installAs:",0
,(
function(self,functionName)
return ((function() local _1_=(self); ((smalltalk.Smalltalk):global())[functionName]=_1_; return _1_ end)())
;
end
)
,"installing","installAs: functionName\r\n\t\" Install the receiver as a function in global scope. \"\r\n\r\n\t^Smalltalk global basicAt: functionName put: self","(\r\nfunction(self,functionName)\r\nreturn ((function() local _1_=(self); ((smalltalk.Smalltalk):global())[functionName]=_1_; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"asJSONObject",0
,(
function(self)
return (self)
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"shallowCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"deepCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"ms",0
,(
function(self)
return ((smalltalk.DateTime):millisecondsToRun_(self))
;
end
)
,"timming","ms\r\n\t\" Return the milliseconds taken to evaluate the receiver. \"\r\n\r\n\t^DateTime millisecondsToRun: self","(\r\nfunction(self)\r\nreturn ((smalltalk.DateTime):millisecondsToRun_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"evaluateFor:",0
,(
function(self,anObject)
return (self(anObject))
;
end
)
,"evaluating","evaluateFor: anObject\r\n\t\" Return the result of evaluating the receiver with anObject. \"\r\n\r\n\t^# #self: anObject","(\r\nfunction(self,anObject)\r\nreturn (self(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"evaluate",0
,(
function(self)
return (self())
;
end
)
,"evaluating","evaluate\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t^# self","(\r\nfunction(self)\r\nreturn (self())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"evaluateWithArguments:",0
,(
function(self,anArray)
return ((self):valueWithPossibleArguments_(anArray))
;
end
)
,"evaluating","evaluateWithArguments: anArray\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t^self valueWithPossibleArguments: anArray","(\r\nfunction(self,anArray)\r\nreturn ((self):valueWithPossibleArguments_(anArray))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"valueWithArguments:",0
,(
function(self,anArray)
return ((self):valueWithPossibleArguments_(anArray))
;
end
)
,"evaluating","valueWithArguments: anArray\r\n\t\" Return the result of evaluating the receiver with arguments. \"\r\n\r\n\t^self valueWithPossibleArguments: anArray","(\r\nfunction(self,anArray)\r\nreturn ((self):valueWithPossibleArguments_(anArray))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockClosure,"valueWithDNU",0
,(
function(self)
return ((smalltalk.Object):hackDNUWhile_(self))
;
end
)
,"Tools-hacking","valueWithDNU\r\n\t\" Catch DNU hacking while evaluating the receiver. \"\r\n\r\n\t^Object hackDNUWhile: self","(\r\nfunction(self)\r\nreturn ((smalltalk.Object):hackDNUWhile_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean["$klass"],"@","_at"
,(
function(self,aBOOL)
	if (aBOOL==true) then return true end
	if (aBOOL==1) then return true end
	if (aBOOL==false) then return false end
	if (aBOOL+0==1) then return true end

;
return (false)
;
end
)
,"Natives-converting","@ aBOOL\r\n\t\" Return aBOOL as a Boolean instance. \"\r\n\r\n{'\tif (aBOOL === true) return true;\r\n\tif (aBOOL === 1) return true;\r\n\tif ((1 & aBOOL) + 0 == 1) return true'\r\nlua'\tif (aBOOL==true) then return true end\r\n\tif (aBOOL==1) then return true end\r\n\tif (aBOOL==false) then return false end\r\n\tif (aBOOL+0==1) then return true end\r\n'}.\t^false","(\r\nfunction(self,aBOOL)\r\n\tif (aBOOL==true) then return true end\r\n\tif (aBOOL==1) then return true end\r\n\tif (aBOOL==false) then return false end\r\n\tif (aBOOL+0==1) then return true end\r\n\r\n;\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"mustBeBoolean",0
,(
function(self)
return (self)
;
end
)
,"testing","mustBeBoolean\r\n\t\" Signal an error if the receiver is not a Boolean. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"=","_eq"
,(
function(self,anObject)
	if (anObject==nil) then return false end
	if (true) then return self==anObject end
;
return (self)
;
end
)
,"comparing","= anObject\r\n    \" Return true if the receiver is equal to argument. \"\r\n\r\n{'\tif (nil.isNil_(anObject)) return false;\r\n\tif (!(self.$klass === anObject.$klass)) return false;\r\n\treturn (self.valueOf() == anObject.valueOf())'\r\nlua'\tif (anObject==nil) then return false end\r\n\tif (true) then return self==anObject end'}","(\r\nfunction(self,anObject)\r\n\tif (anObject==nil) then return false end\r\n\tif (true) then return self==anObject end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"ifTrue:",0
,(
function(self,aBlock)
return ((function() if (((self))==true) then return ((function()
return ((aBlock):value())
end))() else return ((function()
return (nil)
end))() end end)())
;
end
)
,"control","ifTrue: aBlock\r\n\t\" Return the result of evaluating aBlock if the receiver is true, else return nil. \"\r\n\r\n\t^self ifTrue: [ aBlock value ] ifFalse: []","(\r\nfunction(self,aBlock)\r\nreturn ((function() if (((self))==true) then return ((function()\r\nreturn ((aBlock):value())\r\nend))() else return ((function()\r\nreturn (nil)\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"ifFalse:",0
,(
function(self,aBlock)
return ((function() if (((self))==true) then return ((function()
return (nil)
end))() else return ((function()
return ((aBlock):value())
end))() end end)())
;
end
)
,"control","ifFalse: aBlock\r\n\t\" Return the result of evaluating aBlock if the receiver is false, else return nil. \"\r\n\r\n\t^self ifTrue: [] ifFalse: [ aBlock value ]","(\r\nfunction(self,aBlock)\r\nreturn ((function() if (((self))==true) then return ((function()\r\nreturn (nil)\r\nend))() else return ((function()\r\nreturn ((aBlock):value())\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"ifFalse:ifTrue:",0
,(
function(self,aBlock,anotherBlock)
return ((self):ifTrue_ifFalse_(anotherBlock, (function()
return ((aBlock):value())
end)))
;
end
)
,"control","ifFalse: aBlock ifTrue: anotherBlock\r\n\t\" Return the result of evaluating aBlock if the receiver is false, else the result of evaluating anotherBlock. \"\r\n\r\n\t^self ifTrue: anotherBlock ifFalse: [ aBlock value ]","(\r\nfunction(self,aBlock,anotherBlock)\r\nreturn ((self):ifTrue_ifFalse_(anotherBlock, (function()\r\nreturn ((aBlock):value())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"ifTrue:ifFalse:",0
,(
function(self,aBlock,anotherBlock)
	if (self==true)
		then return aBlock()
		else return anotherBlock() end
;
return (self)
;
end
)
,"control","ifTrue: aBlock ifFalse: anotherBlock\r\n\t\" Return the result of evaluating aBlock if the receiver is true, else the result of evaluating anotherBlock. \"\r\n\r\n{'\tif(self == true) { return aBlock.value(); } else { return anotherBlock.value(); }'\r\nlua'\tif (self==true)\r\n\t\tthen return aBlock()\r\n\t\telse return anotherBlock() end'}","(\r\nfunction(self,aBlock,anotherBlock)\r\n\tif (self==true)\r\n\t\tthen return aBlock()\r\n\t\telse return anotherBlock() end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"and:",0
,(
function(self,aBlock)
return ((function() if (((self))==true) then return ((function()
return ((aBlock):value())
end))() else return ((function()
return (false)
end))() end end)())
;
end
)
,"control","and: aBlock\r\n\t\" Return true if the receiver is true and the result of evaluating aBlock is also true. \"\r\n\r\n\t^self ifTrue: [ aBlock value ] ifFalse: [false]","(\r\nfunction(self,aBlock)\r\nreturn ((function() if (((self))==true) then return ((function()\r\nreturn ((aBlock):value())\r\nend))() else return ((function()\r\nreturn (false)\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"or:",0
,(
function(self,aBlock)
return ((function() if (((self))==true) then return ((function()
return (true)
end))() else return ((function()
return ((aBlock):value())
end))() end end)())
;
end
)
,"control","or: aBlock\r\n\t\" Return true if the receiver is true or the result of evaluating aBlock is true. \"\r\n\r\n\t^self ifTrue: [true] ifFalse: [ aBlock value ]","(\r\nfunction(self,aBlock)\r\nreturn ((function() if (((self))==true) then return ((function()\r\nreturn (true)\r\nend))() else return ((function()\r\nreturn ((aBlock):value())\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"not","__not"
,(
function(self)
return (	self == false)
;
end
)
,"control","not\r\n\t\" Return true if the receiver is false. \"\r\n\r\n\t^{' self == false '\r\nlua'\tself == false'}","(\r\nfunction(self)\r\nreturn (\tself == false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"printString",0
,(
function(self)
return (tostring(self))
;
end
)
,"printing","printString\r\n\t\" Return a printable representation of the receiver. \"\r\n\r\n\t^{'self.toString()'\r\n\tlua'tostring(self)'}","(\r\nfunction(self)\r\nreturn (tostring(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"asJSONObject",0
,(
function(self)
return ((smalltalk:boolean_and_((self),(function()
return (true)
end))))
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self and: [ true ]","(\r\nfunction(self)\r\nreturn ((smalltalk:boolean_and_((self),(function()\r\nreturn (true)\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"shallowCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"deepCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Boolean,"isBoolean",0
,(
function(self)
return (true)
;
end
)
,"query","isBoolean\r\n\t\" Return true if the receiver is a boolean. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime["$klass"],"new:",0
,(
function(self,anObject)
return (self:notImplementedYet_("new:"))
;
end
)
,"instantiation","new: anObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^{'new Date(anObject)'\r\n\tlua'self:notImplementedYet_(\"new:\")'}","(\r\nfunction(self,anObject)\r\nreturn (self:notImplementedYet_(\"new:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime["$klass"],"fromString:",0
,(
function(self,aString)
return ((self):new_(aString))
;
end
)
,"instantiation","fromString: aString\r\n\t\" Return an instance of the receiver.\r\n\tDateTime fromString: '1962/12/15 01:40:00'\r\n\t\"\r\n\r\n\t^self new: aString","(\r\nfunction(self,aString)\r\nreturn ((self):new_(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime["$klass"],"fromSeconds:",0
,(
function(self,aNumber)
return ((self):fromMilliseconds_((aNumber):_star((1000))))
;
end
)
,"instantiation","fromSeconds: aNumber\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self fromMilliseconds: aNumber * 1000","(\r\nfunction(self,aNumber)\r\nreturn ((self):fromMilliseconds_((aNumber):_star((1000))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime["$klass"],"fromMilliseconds:",0
,(
function(self,aNumber)
return ((self):new_(aNumber))
;
end
)
,"instantiation","fromMilliseconds: aNumber\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new: aNumber","(\r\nfunction(self,aNumber)\r\nreturn ((self):new_(aNumber))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime["$klass"],"today",0
,(
function(self)
return ((self):new())
;
end
)
,"instantiation","today\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new","(\r\nfunction(self)\r\nreturn ((self):new())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime["$klass"],"now",0
,(
function(self)
return ((self):today())
;
end
)
,"instantiation","now\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self today","(\r\nfunction(self)\r\nreturn ((self):today())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime["$klass"],"msNow",0
,(
function(self)
	if true then return os.clock()*1000 end
;
return ((self):new())
;
end
)
,"instantiation","msNow\r\n\t\" Private - Return a magnitude to measure milliseconds. \"\r\n\r\n{''lua'\tif true then return os.clock()*1000 end'}.\r\n\t^self new","(\r\nfunction(self)\r\n\tif true then return os.clock()*1000 end\r\n;\r\nreturn ((self):new())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime["$klass"],"millisecondsToRun:",0
,(
function(self,aBlock)
local start=nil
(function()
	local _s8_value=((self):msNow())
	start=_s8_value; return _s8_value
end)()
;
(aBlock):value()
;
return (((self):msNow()):_minus(start))
;
end
)
,"instantiation","millisecondsToRun: aBlock\r\n\t\" Return the time elapsed during evaluation of aBlock. \"\r\n\r\n\t| start |\r\n\tstart := self msNow.\r\n\taBlock value.\r\n\t^self msNow - start","(\r\nfunction(self,aBlock)\r\nlocal start=nil\r\n(function()\r\n\tlocal _s8_value=((self):msNow())\r\n\tstart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aBlock):value()\r\n;\r\nreturn (((self):msNow()):_minus(start))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"year",0
,(
function(self)
return ((self):getFullYear())
;
end
)
,"accessing","year\r\n\t\" Return the year of the receiver. \"\r\n\r\n\t^self #getFullYear","(\r\nfunction(self)\r\nreturn ((self):getFullYear())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"month",0
,(
function(self)
return (((self):getMonth()):_plus((1)))
;
end
)
,"accessing","month\r\n\t\" Return the month of the receiver. \"\r\n\r\n\t^self #getMonth + 1","(\r\nfunction(self)\r\nreturn (((self):getMonth()):_plus((1)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"month:",0
,(
function(self,aNumber)
(self):setMonth((aNumber):_minus((1)))
;
return (self)
;
end
)
,"accessing","month: aNumber\r\n\t\" Set the month of the receiver. \"\r\n\r\n\tself #setMonth: aNumber - 1","(\r\nfunction(self,aNumber)\r\n(self):setMonth((aNumber):_minus((1)))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"day",0
,(
function(self)
return ((self):dayOfWeek())
;
end
)
,"accessing","day\r\n\t\" Return the day of the receiver. \"\r\n\r\n\t^self dayOfWeek","(\r\nfunction(self)\r\nreturn ((self):dayOfWeek())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"dayOfWeek",0
,(
function(self)
return (((self):getDay()):_plus((1)))
;
end
)
,"accessing","dayOfWeek\r\n\t\" Return the day of week of the receiver. \"\r\n\r\n\t^self #getDay + 1","(\r\nfunction(self)\r\nreturn (((self):getDay()):_plus((1)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"dayOfWeek:",0
,(
function(self,aNumber)
(self):setDay((aNumber):_minus((1)))
;
return (self)
;
end
)
,"accessing","dayOfWeek: aNumber\r\n\t\" Set the day of week of the receiver. \"\r\n\r\n\tself #setDay: aNumber - 1","(\r\nfunction(self,aNumber)\r\n(self):setDay((aNumber):_minus((1)))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"day:",0
,(
function(self,aNumber)
(self):day_(aNumber)
;
return (self)
;
end
)
,"accessing","day: aNumber\r\n\t\" Set the day of the receiver. \"\r\n\r\n\tself day: aNumber","(\r\nfunction(self,aNumber)\r\n(self):day_(aNumber)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"year:",0
,(
function(self,aNumber)
(self):setFullYear(aNumber)
;
return (self)
;
end
)
,"accessing","year: aNumber\r\n\t\" Set the year of the receiver. \"\r\n\r\n\tself #setFullYear: aNumber","(\r\nfunction(self,aNumber)\r\n(self):setFullYear(aNumber)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"dayOfMonth",0
,(
function(self)
return ((self):getDate())
;
end
)
,"accessing","dayOfMonth\r\n\t\" Return the day of month of the receiver. \"\r\n\r\n\t^self #getDate","(\r\nfunction(self)\r\nreturn ((self):getDate())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"dayOfMonth:",0
,(
function(self,aNumber)
(self):setDate(aNumber)
;
return (self)
;
end
)
,"accessing","dayOfMonth: aNumber\r\n\t\" Set the day of month of the receiver. \"\r\n\r\n\tself #setDate: aNumber","(\r\nfunction(self,aNumber)\r\n(self):setDate(aNumber)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"asString",0
,(
function(self)
return ((self):toString())
;
end
)
,"converting","asString\r\n\t\" Return the text representation of the receiver. \"\r\n\r\n\t^self #toString","(\r\nfunction(self)\r\nreturn ((self):toString())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"printString",0
,(
function(self)
return (smalltalk:asString_((self)))
;
end
)
,"printing","printString\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self asString","(\r\nfunction(self)\r\nreturn (smalltalk:asString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"asMilliseconds",0
,(
function(self)
return ((self):time())
;
end
)
,"converting","asMilliseconds\r\n\t\" Return the receiver as milliseconds. \"\r\n\r\n\t^self time","(\r\nfunction(self)\r\nreturn ((self):time())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"time",0
,(
function(self)
return ((self):getTime())
;
end
)
,"accessing","time\r\n\t\" Return the time of the receiver. \"\r\n\r\n\t^self #getTime","(\r\nfunction(self)\r\nreturn ((self):getTime())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"time:",0
,(
function(self,aNumber)
(self):setTime(aNumber)
;
return (self)
;
end
)
,"accessing","time: aNumber\r\n\t\" Set the time of the receiver. \"\r\n\r\n\tself #setTime: aNumber","(\r\nfunction(self,aNumber)\r\n(self):setTime(aNumber)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"asDateString",0
,(
function(self)
return ((self):toDateString())
;
end
)
,"converting","asDateString\r\n\t\" Return the date string of the receiver. \"\r\n\r\n\t^self #toDateString","(\r\nfunction(self)\r\nreturn ((self):toDateString())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"asTimeString",0
,(
function(self)
return ((self):toTimeString())
;
end
)
,"converting","asTimeString\r\n\t\" Return the time string of the receiver. \"\r\n\r\n\t^self #toTimeString","(\r\nfunction(self)\r\nreturn ((self):toTimeString())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"asLocaleString",0
,(
function(self)
return ((self):toLocaleString())
;
end
)
,"converting","asLocaleString\r\n\t\" Return the locale string of the receiver. \"\r\n\r\n\t^self #toLocaleString","(\r\nfunction(self)\r\nreturn ((self):toLocaleString())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"asNumber",0
,(
function(self)
return ((self):asMilliseconds())
;
end
)
,"converting","asNumber\r\n\t\" Return the receiver as a Number. \"\r\n\r\n\t^self asMilliseconds","(\r\nfunction(self)\r\nreturn ((self):asMilliseconds())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"hours:",0
,(
function(self,aNumber)
return ((self):setHours(aNumber))
;
end
)
,"accessing","hours: aNumber\r\n\t\" Set the hours of the receiver. \"\r\n\r\n\t^self #setHours: aNumber","(\r\nfunction(self,aNumber)\r\nreturn ((self):setHours(aNumber))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"minutes:",0
,(
function(self,aNumber)
(self):setMinutes(aNumber)
;
return (self)
;
end
)
,"accessing","minutes: aNumber\r\n\t\" Set the minutes of the receiver. \"\r\n\r\n\tself #setMinutes: aNumber","(\r\nfunction(self,aNumber)\r\n(self):setMinutes(aNumber)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"seconds:",0
,(
function(self,aNumber)
(self):setSeconds(aNumber)
;
return (self)
;
end
)
,"accessing","seconds: aNumber\r\n\t\" Set the seconds of the receiver. \"\r\n\r\n\tself #setSeconds: aNumber","(\r\nfunction(self,aNumber)\r\n(self):setSeconds(aNumber)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"milliseconds:",0
,(
function(self,aNumber)
(self):setMilliseconds(aNumber)
;
return (self)
;
end
)
,"accessing","milliseconds: aNumber\r\n\t\" Set the milliseconds of the receiver. \"\r\n\r\n\tself #setMilliseconds: aNumber","(\r\nfunction(self,aNumber)\r\n(self):setMilliseconds(aNumber)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"hours",0
,(
function(self)
return ((self):getHours())
;
end
)
,"accessing","hours\r\n\t\" Return the hours of the receiver. \"\r\n\r\n\t^self #getHours","(\r\nfunction(self)\r\nreturn ((self):getHours())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"minutes",0
,(
function(self)
return ((self):getMinutes())
;
end
)
,"accessing","minutes\r\n\t\" Return the minutes of the receiver. \"\r\n\r\n\t^self getMinutes","(\r\nfunction(self)\r\nreturn ((self):getMinutes())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"seconds",0
,(
function(self)
return ((self):getSeconds())
;
end
)
,"accessing","seconds\r\n\t\" Return the seconds of the receiver. \"\r\n\r\n\t^self #getSeconds","(\r\nfunction(self)\r\nreturn ((self):getSeconds())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"milliseconds",0
,(
function(self)
return ((self):getMilliseconds())
;
end
)
,"accessing","milliseconds\r\n\t\" Return the milliseconds of the receiver. \"\r\n\r\n\t^self #getMilliseconds","(\r\nfunction(self)\r\nreturn ((self):getMilliseconds())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"<","_lt"
,(
function(self,aDateTime)
return (self:notImplementedYet_("<"))
;
end
)
,"comparing","< aDateTime\r\n\t\" Return true if the receiver is smaller than argument. \"\r\n\r\n\t^{'self < aDateTime'\r\n\tlua'self:notImplementedYet_(\"<\")'}","(\r\nfunction(self,aDateTime)\r\nreturn (self:notImplementedYet_(\"<\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,">","_gt"
,(
function(self,aDateTime)
return (self:notImplementedYet_(">"))
;
end
)
,"comparing","> aDateTime\r\n\t\" Return true if the receiver is greater than argument. \"\r\n\r\n\t^{'self > aDateTime'\r\n\tlua'self:notImplementedYet_(\">\")'}","(\r\nfunction(self,aDateTime)\r\nreturn (self:notImplementedYet_(\">\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"<=","_lt_eq"
,(
function(self,aDateTime)
return (self:notImplementedYet_("<="))
;
end
)
,"comparing","<= aDateTime\r\n\t\" Return true if the receiver is smaller or equal to argument. \"\r\n\r\n\t^{'self <= aDateTime'\r\n\tlua'self:notImplementedYet_(\"<=\")'}","(\r\nfunction(self,aDateTime)\r\nreturn (self:notImplementedYet_(\"<=\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,">=","_gt_eq"
,(
function(self,aDateTime)
return (self:notImplementedYet_(">="))
;
end
)
,"comparing",">= aDateTime\r\n\t\" Return true if the receiver is greater or equal to argument. \"\r\n\r\n\t^{'self >= aDateTime'\r\n\tlua'self:notImplementedYet_(\">=\")'}","(\r\nfunction(self,aDateTime)\r\nreturn (self:notImplementedYet_(\">=\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"-","_minus"
,(
function(self,aDateTime)
return (self:notImplementedYet_("-"))
;
end
)
,"arithmetic","- aDateTime\r\n\t\" Return the result of operation. \"\r\n\r\n\t^{'self - aDateTime'\r\n\tlua'self:notImplementedYet_(\"-\")'}","(\r\nfunction(self,aDateTime)\r\nreturn (self:notImplementedYet_(\"-\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"+","_plus"
,(
function(self,aDateTime)
return (self:notImplementedYet_("+"))
;
end
)
,"arithmetic","+ aDateTime\r\n\t\" Return the result of operation. \"\r\n\r\n\t^{'self + aDateTime'\r\n\tlua'self:notImplementedYet_(\"+\")'}","(\r\nfunction(self,aDateTime)\r\nreturn (self:notImplementedYet_(\"+\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"isValid",0
,(
function(self)
return (self:notImplementedYet_("isValid"))
;
end
)
,"testing","isValid\r\n\t\" Return true if the receiver is a valid instance. \"\r\n\r\n\t^{' isFinite(self)'\r\n\tlua'self:notImplementedYet_(\"isValid\")'}","(\r\nfunction(self)\r\nreturn (self:notImplementedYet_(\"isValid\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DateTime,"asJSONObject",0
,(
function(self)
return (self)
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject["$klass"],"new",0
,(
function(self)
return ((self):error_("UndefinedObject can't be instantiated."))
;
end
)
,"instantiation","new\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self error: 'UndefinedObject can''t be instantiated.'","(\r\nfunction(self)\r\nreturn ((self):error_(\"UndefinedObject can't be instantiated.\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject["$klass"],"alias",0
,(
function(self)
return ((self):basicAt_ifAbsentPut_("@alias", (function()
return ((self):basicNew())
end)))
;
end
)
,"alias","alias\r\n\t\" Private - Returns an alias of nil.\r\n\tNote that is is a hack used to wrap nil in an alias instance to make possible for Lua collections to contain nil\r\n\t\"\r\n\r\n\t^self basicAt: '@alias' ifAbsentPut: [ self basicNew ]","(\r\nfunction(self)\r\nreturn ((self):basicAt_ifAbsentPut_(\"@alias\", (function()\r\nreturn ((self):basicNew())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"ifNil:",0
,(
function(self,aBlock)
return ((aBlock):value())
;
end
)
,"testing","ifNil: aBlock\r\n\t\" Return the receiver if it is not nil, or the result of evaluating aBlock (if it is nil). \"\r\n\r\n\t^aBlock value","(\r\nfunction(self,aBlock)\r\nreturn ((aBlock):value())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"ifNotNil:",0
,(
function(self,aBlock)
return (self)
;
end
)
,"testing","ifNotNil: aBlock\r\n\t\" Return the receiver if it is nil, or the result of evaluating aBlock. \"\r\n\r\n\t^self","(\r\nfunction(self,aBlock)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"ifNil:ifNotNil:",0
,(
function(self,aBlock,anotherBlock)
return ((aBlock):value())
;
end
)
,"testing","ifNil: aBlock ifNotNil: anotherBlock\r\n\t\" Return the receiver if it is nil, or the result of evaluating aBlock. \"\r\n\r\n\t^aBlock value","(\r\nfunction(self,aBlock,anotherBlock)\r\nreturn ((aBlock):value())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"ifNotNil:ifNil:",0
,(
function(self,aBlock,anotherBlock)
return ((anotherBlock):value())
;
end
)
,"testing","ifNotNil: aBlock ifNil: anotherBlock\r\n\t\" Return the result of evaluating anotherBlock if teh receiver is nil, or the result of evaluating aBlock. \"\r\n\r\n\t^anotherBlock value","(\r\nfunction(self,aBlock,anotherBlock)\r\nreturn ((anotherBlock):value())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"isNil",0
,(
function(self)
return (true)
;
end
)
,"testing","isNil\r\n\t\" Return true if the receiver is nil. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"notNil",0
,(
function(self)
return (false)
;
end
)
,"testing","notNil\r\n\t\" Return true if the receiver is not nil. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"printString",0
,(
function(self)
return ("nil")
;
end
)
,"printing","printString\r\n\t\" Return the text representation of the receiver. \"\r\n\r\n\t^#nil","(\r\nfunction(self)\r\nreturn (\"nil\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"subclass:instanceVariableNames:",0
,(
function(self,aString,anotherString)
return ((self):subclass_instanceVariableNames_category_(aString, anotherString, nil))
;
end
)
,"instantiation","subclass: aString instanceVariableNames: anotherString\r\n\t\" Create a subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: anotherString\r\n\t\tcategory: nil","(\r\nfunction(self,aString,anotherString)\r\nreturn ((self):subclass_instanceVariableNames_category_(aString, anotherString, nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"subclass:instanceVariableNames:category:",0
,(
function(self,aString,aString2,aString3)
return (((smalltalk.ClassBuilder):new()):superclass_subclass_instanceVariableNames_category_(self, aString, aString2, aString3))
;
end
)
,"instantiation","subclass: aString instanceVariableNames: aString2 category: aString3\r\n\t\" Create a subclass of the receiver. \"\r\n\r\n\t^ClassBuilder new\r\n\t\tsuperclass: self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tcategory: aString3","(\r\nfunction(self,aString,aString2,aString3)\r\nreturn (((smalltalk.ClassBuilder):new()):superclass_subclass_instanceVariableNames_category_(self, aString, aString2, aString3))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"asJSONObject",0
,(
function(self)
return (nil)
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^{'null'\tlua'nil'}","(\r\nfunction(self)\r\nreturn (nil)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"shallowCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"deepCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"value",0
,(
function(self)
return (self)
;
end
)
,"evaluating","value\r\n\t\" Return the result of evaluating the receiver. \"","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"@","_at"
,(
function(self,aNumber)
return ((smalltalk.Point):x_y_((0), aNumber))
;
end
)
,"converting","@ aNumber\r\n\t\" Return a Point with coordinates given by the receiver and aNumber. \"\r\n\r\n\t^Point x: 0 y: aNumber","(\r\nfunction(self,aNumber)\r\nreturn ((smalltalk.Point):x_y_((0), aNumber))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"=","_eq"
,(
function(self,anObject)
return ((((anObject))==nil))
;
end
)
,"comparing","= anObject\r\n\t\" Return true if the receiver is equal to argument. \"\r\n\r\n\t^anObject isNil","(\r\nfunction(self,anObject)\r\nreturn ((((anObject))==nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"~=","_no_eq"
,(
function(self,anObject)
return ((((anObject))~=nil))
;
end
)
,"comparing","~= anObject\r\n\t\" Return true if the receiver is not equal to anObject. \"\r\n\r\n\t^anObject notNil","(\r\nfunction(self,anObject)\r\nreturn ((((anObject))~=nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"category:",0
,(
function(self,ignored)
return (self)
;
end
)
,"instantiation","category: ignored\r\n\t\" BUG - throw an error in ClassBuilder. \" ","(\r\nfunction(self,ignored)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:",0
,(
function(self,aString,aString2,classVarNames,poolVarNames,aCategory)
(function() if ((((((((self):smalltalk()):globalAt_(aString)))~=nil)))==true) then return ((function()
return ((self):note_(smalltalk:object_comma_(("WARNING: class REdefinition for "),aString)))
end))() else return nil end end)()
;
return (((smalltalk.ClassBuilder):new()):superclass_subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(self, aString, aString2, classVarNames, poolVarNames, aCategory))
;
end
)
,"instantiation","subclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tclassVariableNames: classVarNames\r\n\tpoolDictionaries: poolVarNames\r\n\tcategory: aCategory\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t(self smalltalk globalAt: aString) notNil ifTrue: [\r\n\t\tself note: 'WARNING: class REdefinition for ',aString.\r\n\t\t\"^self smalltalk globalAt: aString\"\r\n\t].\r\n\r\n\"\tself note: 'Creating ROOT class ',aString.\"\r\n\t^ClassBuilder new\r\n\t\tsuperclass: self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: classVarNames\r\n\t\tpoolDictionaries: poolVarNames\r\n\t\tcategory: aCategory","(\r\nfunction(self,aString,aString2,classVarNames,poolVarNames,aCategory)\r\n(function() if ((((((((self):smalltalk()):globalAt_(aString)))~=nil)))==true) then return ((function()\r\nreturn ((self):note_(smalltalk:object_comma_((\"WARNING: class REdefinition for \"),aString)))\r\nend))() else return nil end end)()\r\n;\r\nreturn (((smalltalk.ClassBuilder):new()):superclass_subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(self, aString, aString2, classVarNames, poolVarNames, aCategory))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"subclass:instanceVariableNames:classVariableNames:poolDictionaries:",0
,(
function(self,aString,aString2,classVarNames,poolVarNames)
return ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString, aString2, classVarNames, poolVarNames, nil))
;
end
)
,"instantiation","subclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tclassVariableNames: classVarNames\r\n\tpoolDictionaries: poolVarNames\r\n\t\" Create a new subclass of the receiver. \"\r\n\r\n\t^self\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: classVarNames\r\n\t\tpoolDictionaries: poolVarNames\r\n\t\tcategory: nil","(\r\nfunction(self,aString,aString2,classVarNames,poolVarNames)\r\nreturn ((self):subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString, aString2, classVarNames, poolVarNames, nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"compiler",0
,(
function(self)
return ((((self):smalltalk()):compilerSupport()):for_(nil))
;
end
)
,"Tools-printing","compiler\r\n\t\" Return the compiler of the receiver. \"\r\n\r\n\t^self smalltalk compilerSupport for: nil","(\r\nfunction(self)\r\nreturn ((((self):smalltalk()):compilerSupport()):for_(nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"==","_eq_eq"
,(
function(self,anObject)
return ((((anObject))==nil))
;
end
)
,"Tools-comparing","== anObject\r\n\t^anObject isNil","(\r\nfunction(self,anObject)\r\nreturn ((((anObject))==nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"~~","_no_no"
,(
function(self,anObject)
return ((((anObject))~=nil))
;
end
)
,"Tools-comparing","~~ anObject\r\n\t^anObject notNil","(\r\nfunction(self,anObject)\r\nreturn ((((anObject))~=nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"handles",0
,(
function(self)
return (self)
;
end
)
,"Natives-converting","handles\r\n\t\" Return the result of collecting the handles of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.UndefinedObject,"handle",0
,(
function(self)
return ((self):asJSONObject())
;
end
)
,"Natives-converting","handle\r\n\t\" Return the handle of the receiver. \"\r\n\r\n\t^self asJSONObject","(\r\nfunction(self)\r\nreturn ((self):asJSONObject())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection["$klass"],"streamClass",0
,(
function(self)
return (smalltalk.Stream)
;
end
)
,"streams","streamClass\r\n\t\" Private - Return the support for streamming on the receiver's instances. \"\r\n\r\n\t^Stream","(\r\nfunction(self)\r\nreturn (smalltalk.Stream)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection["$klass"],"with:",0
,(
function(self,anObject)
return ((
function(_s8_rec)
(_s8_rec):add_(anObject);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","with: anObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tadd: anObject;\r\n\t\tyourself","(\r\nfunction(self,anObject)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(anObject);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection["$klass"],"with:with:",0
,(
function(self,anObject,anotherObject)
return ((
function(_s8_rec)
(_s8_rec):add_(anObject);
(_s8_rec):add_(anotherObject);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","with: anObject with: anotherObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tadd: anObject;\r\n\t\tadd: anotherObject;\r\n\t\tyourself","(\r\nfunction(self,anObject,anotherObject)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(anObject);\r\n(_s8_rec):add_(anotherObject);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection["$klass"],"with:with:with:",0
,(
function(self,firstObject,secondObject,thirdObject)
return ((
function(_s8_rec)
(_s8_rec):add_(firstObject);
(_s8_rec):add_(secondObject);
(_s8_rec):add_(thirdObject);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","with: firstObject with: secondObject with: thirdObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tadd: firstObject;\r\n\t\tadd: secondObject;\r\n\t\tadd: thirdObject;\r\n\t\tyourself","(\r\nfunction(self,firstObject,secondObject,thirdObject)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(firstObject);\r\n(_s8_rec):add_(secondObject);\r\n(_s8_rec):add_(thirdObject);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection["$klass"],"with:with:with:with:",0
,(
function(self,firstObject,secondObject,thirdObject,fourthObject)
return ((
function(_s8_rec)
(_s8_rec):add_(firstObject);
(_s8_rec):add_(secondObject);
(_s8_rec):add_(thirdObject);
(_s8_rec):add_(fourthObject);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","with: firstObject with: secondObject with: thirdObject with: fourthObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tadd: firstObject;\r\n\t\tadd: secondObject;\r\n\t\tadd: thirdObject;\r\n\t\tadd: fourthObject;\r\n\t\tyourself","(\r\nfunction(self,firstObject,secondObject,thirdObject,fourthObject)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(firstObject);\r\n(_s8_rec):add_(secondObject);\r\n(_s8_rec):add_(thirdObject);\r\n(_s8_rec):add_(fourthObject);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection["$klass"],"withAll:",0
,(
function(self,aCollection)
return ((
function(_s8_rec)
(_s8_rec):addAll_(aCollection);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","withAll: aCollection\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\taddAll: aCollection;\r\n\t\tyourself","(\r\nfunction(self,aCollection)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):addAll_(aCollection);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection["$klass"],"basicNew:",0
,(
function(self,size)
return ((self):subclassResponsibility_("basicNew:"))
;
end
)
,"instantiation","basicNew: size\r\n\t\" Return an instance of the receiver of specified size. \"\r\n\r\n\t^self subclassResponsibility: #basicNew:","(\r\nfunction(self,size)\r\nreturn ((self):subclassResponsibility_(\"basicNew:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection["$klass"],"of:withAll:",0
,(
function(self,aType,handles)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):isObject_(handles)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):of_withAll_(aType, (smalltalk.Array):_at(handles)))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):withAll_((handles):collect_((function(each)
return ((aType):_at(each))
end))))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Natives-instantiation","of: aType withAll: handles\r\n\t\" Return an instance of the receiver wrapping handles with a type (or nil). \"\r\n\r\n\t(self isObject: handles) ifFalse: [ ^self of: aType withAll: Array @ handles ].\r\n\t^self withAll: (handles collect: [:each| aType @ each ])","(\r\nfunction(self,aType,handles)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):isObject_(handles)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):of_withAll_(aType, (smalltalk.Array):_at(handles)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):withAll_((handles):collect_((function(each)\r\nreturn ((aType):_at(each))\r\nend))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"wrap:",0
,(
function(self,anObject)
	if (anObject==nil)
		then return smalltalk.UndefinedObject:alias()
		else return anObject end
;
return (self)
;
end
)
,"alias","wrap: anObject\r\n\t\" Private - Return an element holding anObject. \"\r\n\r\n{'\treturn anObject'\r\nlua'\tif (anObject==nil)\r\n\t\tthen return smalltalk.UndefinedObject:alias()\r\n\t\telse return anObject end'}","(\r\nfunction(self,anObject)\r\n\tif (anObject==nil)\r\n\t\tthen return smalltalk.UndefinedObject:alias()\r\n\t\telse return anObject end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"unwrap:",0
,(
function(self,anObject)
	if (anObject:isNil())
		then return nil
		else return anObject end
;
return (self)
;
end
)
,"alias","unwrap: anObject\r\n\t\" Private - Return object held by anObject. \"\r\n\r\n{'\treturn anObject'\r\nlua'\tif (anObject:isNil())\r\n\t\tthen return nil\r\n\t\telse return anObject end'}","(\r\nfunction(self,anObject)\r\n\tif (anObject:isNil())\r\n\t\tthen return nil\r\n\t\telse return anObject end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"size",0
,(
function(self)
return ((self):subclassResponsibility_("size"))
;
end
)
,"accessing","size\r\n\t\" Return the size of the receiver. \"\r\n\r\n\t^self subclassResponsibility: #size","(\r\nfunction(self)\r\nreturn ((self):subclassResponsibility_(\"size\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"readStream",0
,(
function(self)
return ((self):stream())
;
end
)
,"streams","readStream\r\n\t\" Return a read stream on the receiver. \"\r\n\r\n\t^self stream","(\r\nfunction(self)\r\nreturn ((self):stream())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"writeStream",0
,(
function(self)
return ((self):stream())
;
end
)
,"streams","writeStream\r\n\t\" Return a write stream on the receiver. \"\r\n\r\n\t^self stream","(\r\nfunction(self)\r\nreturn ((self):stream())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"stream",0
,(
function(self)
return (((self):streamClass()):on_(self))
;
end
)
,"streams","stream\r\n\t\" Return a stream on the receiver. \"\r\n\r\n\t^self streamClass on: self","(\r\nfunction(self)\r\nreturn (((self):streamClass()):on_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"streamClass",0
,(
function(self)
return (((self)["$klass"]):streamClass())
;
end
)
,"streams","streamClass\r\n\t\" Private - Return the support for streamming on the receiver. \"\r\n\r\n\t^self class streamClass","(\r\nfunction(self)\r\nreturn (((self)[\"$klass\"]):streamClass())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"errorNotFound",0
,(
function(self)
return ((self):objectNotFound())
;
end
)
,"error","errorNotFound\r\n\t\" DEPRECATED - Generates an error.\r\n\tAn object was not found in the receiver's contents.\r\n\t\"\r\n\r\n\t^self objectNotFound","(\r\nfunction(self)\r\nreturn ((self):objectNotFound())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"objectNotFound",0
,(
function(self)
((self):smalltalk()):objectNotFound(nil, nil, self)
;
return ((self):error_("Object is not in the collection"))
;
end
)
,"error","objectNotFound\r\n\t\" Private - Generates an error.\r\n\tAn object was not found in the receiver's contents.\r\n\t\"\r\n\r\n\tself smalltalk #objectNotFound: nil at: nil in: self.\r\n\t^self error: 'Object is not in the collection'","(\r\nfunction(self)\r\n((self):smalltalk()):objectNotFound(nil, nil, self)\r\n;\r\nreturn ((self):error_(\"Object is not in the collection\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"objectNotFound:",0
,(
function(self,anObject)
((self):smalltalk()):objectNotFound(anObject, nil, self)
;
return ((self):error_(smalltalk:object_comma_(("Object not found: "),smalltalk:toString_((anObject)))))
;
end
)
,"error","objectNotFound: anObject\r\n\t\" Private - Generates an error.\r\n\tanObject was not found in the receiver's contents.\r\n\t\"\r\n\r\n\tself smalltalk #objectNotFound: anObject at: nil in: self.\r\n\t^self error: 'Object not found: ',anObject toString","(\r\nfunction(self,anObject)\r\n((self):smalltalk()):objectNotFound(anObject, nil, self)\r\n;\r\nreturn ((self):error_(smalltalk:object_comma_((\"Object not found: \"),smalltalk:toString_((anObject)))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"objectNotFoundAt:",0
,(
function(self,indexOrKey)
((self):smalltalk()):objectNotFound(nil, indexOrKey, self)
;
return ((self):error_(smalltalk:object_comma_(("Object not found at "),smalltalk:toString_((indexOrKey)))))
;
end
)
,"error","objectNotFoundAt: indexOrKey\r\n\t\" Private - Generates an error.\r\n\tThe object at key/index was not found in the receiver's contents.\r\n\t\"\r\n\r\n\tself smalltalk #objectNotFound: nil at: indexOrKey in: self.\r\n\t^self error: 'Object not found at ',indexOrKey toString","(\r\nfunction(self,indexOrKey)\r\n((self):smalltalk()):objectNotFound(nil, indexOrKey, self)\r\n;\r\nreturn ((self):error_(smalltalk:object_comma_((\"Object not found at \"),smalltalk:toString_((indexOrKey)))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"add:",0
,(
function(self,anObject)
return ((self):subclassResponsibility_("add:"))
;
end
)
,"adding","add: anObject\r\n\t\" Add anObject to the receiver. \"\r\n\r\n\t^self subclassResponsibility: #add:","(\r\nfunction(self,anObject)\r\nreturn ((self):subclassResponsibility_(\"add:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"addAll:",0
,(
function(self,aCollection)
(aCollection):do_((function(each)
return ((self):add_(each))
end))
;
return (aCollection)
;
end
)
,"adding","addAll: aCollection\r\n\t\" Add objects in aCollection to the receiver. \"\r\n\r\n\taCollection do: [:each | self add: each ].\r\n\t^aCollection","(\r\nfunction(self,aCollection)\r\n(aCollection):do_((function(each)\r\nreturn ((self):add_(each))\r\nend))\r\n;\r\nreturn (aCollection)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,",","_comma"
,(
function(self,aCollection)
return ((
function(_s8_rec)
(_s8_rec):addAll_(aCollection);
return ((_s8_rec):yourself())
end
)((self):copy()))
;
end
)
,"copying",", aCollection\r\n\t\" Return a copy of the receiver with all contents of aCollection. \"\r\n\r\n\t^self copy\r\n\t\taddAll: aCollection;\r\n\t\tyourself","(\r\nfunction(self,aCollection)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):addAll_(aCollection);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):copy()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"copyWith:",0
,(
function(self,anObject)
return ((
function(_s8_rec)
(_s8_rec):add_(anObject);
return ((_s8_rec):yourself())
end
)((self):copy()))
;
end
)
,"copying","copyWith: anObject\r\n\t\" Return a copy of the receiver with anObject. \"\r\n\r\n\t^self copy add: anObject; yourself","(\r\nfunction(self,anObject)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(anObject);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):copy()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"copyWithAll:",0
,(
function(self,aCollection)
return ((
function(_s8_rec)
(_s8_rec):addAll_(aCollection);
return ((_s8_rec):yourself())
end
)((self):copy()))
;
end
)
,"copying","copyWithAll: aCollection\r\n\t\" Return a copy of the receiver with all contents of aCollection. \"\r\n\r\n\t^self copy\r\n\t\taddAll: aCollection;\r\n\t\tyourself","(\r\nfunction(self,aCollection)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):addAll_(aCollection);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):copy()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"asArray",0
,(
function(self)
local result=nil
local index=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	result=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((0))
	index=_s8_value; return _s8_value
end)()
;
(self):do_((function(each)
(function()
	local _s8_value=((index):_plus((1)))
	index=_s8_value; return _s8_value
end)();return ((result):at_put_(index, each))
end))
;
return (result)
;
end
)
,"converting","asArray\r\n\t\" Return an array with the contents of the receiver. \"\r\n\r\n\t| result index |\r\n\tresult := Array new.\r\n\tindex := 0.\r\n\tself do: [:each |\r\n\t    index := index + 1.\r\n\t    result at: index put: each\r\n\t].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((0))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):do_((function(each)\r\n(function()\r\n\tlocal _s8_value=((index):_plus((1)))\r\n\tindex=_s8_value; return _s8_value\r\nend)();return ((result):at_put_(index, each))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"do:",0
,(
function(self,aBlock)
return ((self):subclassResponsibility_("do:"))
;
end
)
,"enumerating","do: aBlock\r\n\t\" Evaluate aBlock with the contents of the receiver. \"\r\n\r\n\t^self subclassResponsibility: #do:","(\r\nfunction(self,aBlock)\r\nreturn ((self):subclassResponsibility_(\"do:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"detect:ifNone:",0
,(
function(self,aBlock,anotherBlock)
return ((self):subclassResponsibility_("detect:ifNone:"))
;
end
)
,"enumerating","detect: aBlock ifNone: anotherBlock\r\n\t\" Return the object in the receiver that return true when evaluated with aBlock or the result of evaluating anotherBlock. \"\r\n\r\n\t^self subclassResponsibility: #detect:ifNone:","(\r\nfunction(self,aBlock,anotherBlock)\r\nreturn ((self):subclassResponsibility_(\"detect:ifNone:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"reverseDo:",0
,(
function(self,aBlock)
return ((self):subclassResponsibility_("reverseDo:"))
;
end
)
,"enumerating","reverseDo: aBlock\r\n\t\" Evaluate aBlock with the contents of the receiver in reverse order of iteration. \"\r\n\r\n\t^self subclassResponsibility: #reverseDo:","(\r\nfunction(self,aBlock)\r\nreturn ((self):subclassResponsibility_(\"reverseDo:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"collect:",0
,(
function(self,aBlock)
local result=nil
(function()
	local _s8_value=(((self)["$klass"]):new())
	result=_s8_value; return _s8_value
end)()
;
(self):do_((function(each)
return ((result):add_((aBlock):value_(each)))
end))
;
return (result)
;
end
)
,"enumerating","collect: aBlock\r\n\t\" Return a collection with the result of evaluating aBlock with the contents of the receiver. \"\r\n\r\n\t| result |\r\n\tresult := self class new.\r\n\tself do: [:each | result add: (aBlock value: each) ].\r\n\t^result","(\r\nfunction(self,aBlock)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):do_((function(each)\r\nreturn ((result):add_((aBlock):value_(each)))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"select:",0
,(
function(self,aBlock)
local stream=nil
(function()
	local _s8_value=((((self)["$klass"]):new()):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):do_((function(each)
return ((function() if ((((aBlock):value_(each)))==true) then return ((function()
return ((stream):nextPut_(each))
end))() else return nil end end)())
end))
;
return ((stream):contents())
;
end
)
,"enumerating","select: aBlock\r\n\t\" Return the contents of the receiver wich evaluates aBlock to true. \"\r\n\r\n\t| stream |\r\n\tstream := self class new writeStream.\r\n\tself do: [:each |\r\n\t    (aBlock value: each) ifTrue: [ stream nextPut: each ]\r\n\t].\r\n\t^stream contents","(\r\nfunction(self,aBlock)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=((((self)[\"$klass\"]):new()):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):do_((function(each)\r\nreturn ((function() if ((((aBlock):value_(each)))==true) then return ((function()\r\nreturn ((stream):nextPut_(each))\r\nend))() else return nil end end)())\r\nend))\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"detect:",0
,(
function(self,aBlock)
return ((self):detect_ifNone_(aBlock, (function()
return ((self):objectNotFound())
end)))
;
end
)
,"enumerating","detect: aBlock\r\n\t\" Return the object in the receiver that return true when evaluated with aBlock.\r\n\tGenerates an error if there is no object that evaluates aBlock to true.\r\n\t\"\r\n\r\n\t^self detect: aBlock ifNone: [ self objectNotFound ]","(\r\nfunction(self,aBlock)\r\nreturn ((self):detect_ifNone_(aBlock, (function()\r\nreturn ((self):objectNotFound())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"do:andBetweenDo:",0
,(
function(self,aBlock,separatorBlock)
return ((self):do_separatedBy_(aBlock, separatorBlock))
;
end
)
,"enumerating","do: aBlock andBetweenDo: separatorBlock\r\n\t\" For each element in the receiver, evaluate aBlock with that element as the argument.\r\n\tEvaluate the separatorBlock between successive elements of the receiver.\r\n\t\"\r\n\r\n\t^self do: aBlock separatedBy: separatorBlock","(\r\nfunction(self,aBlock,separatorBlock)\r\nreturn ((self):do_separatedBy_(aBlock, separatorBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"do:separatedBy:",0
,(
function(self,aBlock,anotherBlock)
local first=nil
(function()
	local _s8_value=(true)
	first=_s8_value; return _s8_value
end)()
;
(self):do_((function(each)
(function() if (((first))==true) then return ((function()
return ((function()
	local _s8_value=(false)
	first=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((anotherBlock):value())
end))() end end)();return ((aBlock):value_(each))
end))
;
return (self)
;
end
)
,"enumerating","do: aBlock separatedBy: anotherBlock\r\n\t\" Evaluate aBlock for contents of the receiver evaluating anotherBlock inbetween aBlock evaluations. \"\r\n\r\n\t| first |\r\n\tfirst := true.\r\n\tself do: [:each |\r\n\t    first\r\n\t\tifTrue: [ first := false ]\r\n\t\tifFalse: [ anotherBlock value ].\r\n\t    aBlock value: each\r\n\t]","(\r\nfunction(self,aBlock,anotherBlock)\r\nlocal first=nil\r\n(function()\r\n\tlocal _s8_value=(true)\r\n\tfirst=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):do_((function(each)\r\n(function() if (((first))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(false)\r\n\tfirst=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((anotherBlock):value())\r\nend))() end end)();return ((aBlock):value_(each))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"inject:into:",0
,(
function(self,anObject,aBlock)
local result=nil
(function()
	local _s8_value=(anObject)
	result=_s8_value; return _s8_value
end)()
;
(self):do_((function(each)
return ((function()
	local _s8_value=((aBlock):value_value_(result, each))
	result=_s8_value; return _s8_value
end)())
end))
;
return (result)
;
end
)
,"enumerating","inject: anObject into: aBlock\r\n\t\" Evaluate aBlock for result of previous evaluation and contents of the receiver.\r\n\tOn first evaluation anObject is used as first argument to evaluate aBlock.\r\n\t\"\r\n\r\n\t| result |\r\n\tresult := anObject.\r\n\tself do: [:each | result := aBlock value: result value: each ].\r\n\t^result","(\r\nfunction(self,anObject,aBlock)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(anObject)\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):do_((function(each)\r\nreturn ((function()\r\n\tlocal _s8_value=((aBlock):value_value_(result, each))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"reject:",0
,(
function(self,aBlock)
return ((self):select_((function(each)
return ((smalltalk:isObject_equalTo_(((aBlock):value_(each)),false)))
end)))
;
end
)
,"enumerating","reject: aBlock\r\n\t\" Return the contents of the receiver wich evaluates aBlock to false. \"\r\n\r\n\t^self select: [:each | (aBlock value: each) = false]","(\r\nfunction(self,aBlock)\r\nreturn ((self):select_((function(each)\r\nreturn ((smalltalk:isObject_equalTo_(((aBlock):value_(each)),false)))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"select:thenCollect:",0
,(
function(self,selectBlock,collectBlock)
return (((self):select_(selectBlock)):collect_(collectBlock))
;
end
)
,"enumerating","select: selectBlock thenCollect: collectBlock\r\n\t\" Return a new collection with the result of evaluating collectBlock on the contents of the receiver wich evaluates selectBlock to true. \"\r\n\r\n\t^(self select: selectBlock) collect: collectBlock","(\r\nfunction(self,selectBlock,collectBlock)\r\nreturn (((self):select_(selectBlock)):collect_(collectBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"collect:thenSelect:",0
,(
function(self,collectBlock,selectBlock)
return (((self):collect_(collectBlock)):select_(selectBlock))
;
end
)
,"enumerating","collect: collectBlock thenSelect: selectBlock\r\n\t\" Return a new collection with the result of evaluating collectBlock on the contents of the receiver wich evaluates selectBlock to true. \"\r\n\r\n\t^(self collect: collectBlock) select: selectBlock","(\r\nfunction(self,collectBlock,selectBlock)\r\nreturn (((self):collect_(collectBlock)):select_(selectBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"reversed",0
,(
function(self)
return ((self):subclassResponsibility_("reversed"))
;
end
)
,"copying","reversed\r\n\t\" Return an array with the contents of the receiver in reverse order. \"\r\n\r\n\t^self subclassResponsibility: #reversed","(\r\nfunction(self)\r\nreturn ((self):subclassResponsibility_(\"reversed\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"includes:",0
,(
function(self,anObject)
return ((self):subclassResponsibility_("includes:"))
;
end
)
,"testing","includes: anObject\r\n\t\" Return true if anObject is in the receiver. \"\r\n\r\n\t^self subclassResponsibility: #includes:","(\r\nfunction(self,anObject)\r\nreturn ((self):subclassResponsibility_(\"includes:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"notEmpty",0
,(
function(self)
return ((smalltalk:booleanNot_(((self):isEmpty()))))
;
end
)
,"testing","notEmpty\r\n\t\" Return true if the receiver is not empty. \"\r\n\r\n\t^self isEmpty not","(\r\nfunction(self)\r\nreturn ((smalltalk:booleanNot_(((self):isEmpty()))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"isEmpty",0
,(
function(self)
return ((((self):size())==(0)))
;
end
)
,"testing","isEmpty\r\n\t\" Return true if the receiver is empty. \"\r\n\r\n\t^self size == 0","(\r\nfunction(self)\r\nreturn ((((self):size())==(0)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"remove:",0
,(
function(self,anObject)
return ((self):remove_ifAbsent_(anObject, (function()
return ((self):objectNotFound_(anObject))
end)))
;
end
)
,"removing","remove: anObject\r\n\t\" Remove anObject from receiver, return the removed object. \"\r\n\r\n\t^self remove: anObject ifAbsent: [ self objectNotFound: anObject ]","(\r\nfunction(self,anObject)\r\nreturn ((self):remove_ifAbsent_(anObject, (function()\r\nreturn ((self):objectNotFound_(anObject))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"remove:ifAbsent:",0
,(
function(self,anObject,aBlock)
return ((self):subclassResponsibility_("remove:ifAbsent:"))
;
end
)
,"removing","remove: anObject ifAbsent: aBlock\r\n\t\" Remove anObject from receiver.\r\n\tReturn the removed object or the result of evaluating aBlock (if the object is not found).\r\n\t\"\r\n\r\n\t^self subclassResponsibility: #remove:ifAbsent:","(\r\nfunction(self,anObject,aBlock)\r\nreturn ((self):subclassResponsibility_(\"remove:ifAbsent:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"removeAll:",0
,(
function(self,aCollection)
(aCollection):do_((function(each)
return ((self):remove_(each))
end))
;
return (aCollection)
;
end
)
,"removing","removeAll: aCollection\r\n\t\" Remove aCollection from receiver. \"\r\n\r\n\taCollection do: [:each| self remove: each ].\r\n\t^aCollection","(\r\nfunction(self,aCollection)\r\n(aCollection):do_((function(each)\r\nreturn ((self):remove_(each))\r\nend))\r\n;\r\nreturn (aCollection)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"removeAll",0
,(
function(self)
(self):removeAll_((self):shallowCopy())
;
return (self)
;
end
)
,"removing","removeAll\r\n\t\" Remove all elements in the receiver. \"\r\n\r\n\tself removeAll: self shallowCopy","(\r\nfunction(self)\r\n(self):removeAll_((self):shallowCopy())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"occurrencesOf:",0
,(
function(self,anObject)
return ((self):inject_into_((0), (function(total,each)
return ((function() if ((((smalltalk:isObject_equalTo_((each),anObject))))==true) then return ((function()
return ((total):_plus((1)))
end))() else return ((function()
return (total)
end))() end end)())
end)))
;
end
)
,"counting","occurrencesOf: anObject\r\n\t\" Return the occurrences of anObject in the receiver. \"\r\n\r\n\t^self inject: 0 into: [:total :each|\r\n\t\teach = anObject ifTrue: [ total + 1 ] ifFalse: [total]\r\n\t]","(\r\nfunction(self,anObject)\r\nreturn ((self):inject_into_((0), (function(total,each)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((each),anObject))))==true) then return ((function()\r\nreturn ((total):_plus((1)))\r\nend))() else return ((function()\r\nreturn (total)\r\nend))() end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"shallowCopy",0
,(
function(self)
return ((self):collect_((function(each)
return (each)
end)))
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self collect: [:each| each ]","(\r\nfunction(self)\r\nreturn ((self):collect_((function(each)\r\nreturn (each)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"deepCopy",0
,(
function(self)
return ((self):collect_((function(each)
return ((each):deepCopy())
end)))
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self collect: [:each| each deepCopy ]","(\r\nfunction(self)\r\nreturn ((self):collect_((function(each)\r\nreturn ((each):deepCopy())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"asSet",0
,(
function(self)
return ((smalltalk.Set):withAll_(self))
;
end
)
,"converting","asSet\r\n\t\" Return the receiver contents in a Set. \"\r\n\r\n\t^Set withAll: self","(\r\nfunction(self)\r\nreturn ((smalltalk.Set):withAll_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"asHashedSet",0
,(
function(self)
return ((smalltalk.HashedSet):withAll_(self))
;
end
)
,"converting","asHashedSet\r\n\t\" Return the receiver contents in a HashedSet. \"\r\n\r\n\t^HashedSet withAll: self","(\r\nfunction(self)\r\nreturn ((smalltalk.HashedSet):withAll_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"asParser",0
,(
function(self)
return ((smalltalk.PPChoiceParser):withAll_((self):asArray()))
;
end
)
,"Compiler-converting","asParser\r\n\t\" Return a parser on the receiver. \"\r\n\r\n\t^PPChoiceParser withAll: self asArray","(\r\nfunction(self)\r\nreturn ((smalltalk.PPChoiceParser):withAll_((self):asArray()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"fileOut",0
,(
function(self)
local cr=nil
(function()
	local _s8_value=("\r\n")
	cr=_s8_value; return _s8_value
end)()
;
return (((self):inject_into_(smalltalk:newArray({}), (function(total,each)
return ((
function(_s8_rec)
(_s8_rec):add_(cr);
(_s8_rec):add_((each):fileOut());
return ((_s8_rec):yourself())
end
)(total))
end))):join_(""))
;
end
)
,"Tools-fileOut","fileOut\r\n\t\" Return fileOut definition and code of classes in the receiver. \"\r\n\r\n\t| cr |\r\n\tcr := '\r\n'.\t^(self inject: #() into: [:total :each|\r\n\t\ttotal add: cr; add: each fileOut; yourself\r\n\t\t]) join: ''","(\r\nfunction(self)\r\nlocal cr=nil\r\n(function()\r\n\tlocal _s8_value=(\"\\r\\n\")\r\n\tcr=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (((self):inject_into_(smalltalk:newArray({}), (function(total,each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(cr);\r\n(_s8_rec):add_((each):fileOut());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(total))\r\nend))):join_(\"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"export",0
,(
function(self)
local cr=nil
(function()
	local _s8_value=("\r\n")
	cr=_s8_value; return _s8_value
end)()
;
return (((self):inject_into_("", (function(total,each)
return ((
function(_s8_rec)
(_s8_rec):add_(cr);
(_s8_rec):add_((each):export());
return ((_s8_rec):yourself())
end
)(total))
end))):join_(""))
;
end
)
,"Tools-export","export\r\n\t\" Return the result of exporting definition and source code of classes in the receiver. \"\r\n\r\n\t| cr |\r\n\tcr := '\r\n'.\t^(self inject: '' into: [:total :each|\r\n\t\ttotal add: cr; add: each export; yourself\r\n\t\t]) join: ''","(\r\nfunction(self)\r\nlocal cr=nil\r\n(function()\r\n\tlocal _s8_value=(\"\\r\\n\")\r\n\tcr=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (((self):inject_into_(\"\", (function(total,each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(cr);\r\n(_s8_rec):add_((each):export());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(total))\r\nend))):join_(\"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"emmit",0
,(
function(self)
return (((self):asArray()):compose_with_((function(each)
return ((self):emmit_(each))
end), "\r\n"))
;
end
)
,"Builders-emmiting","emmit\r\n\t\" Emmit (fileIn & save generated code of) the fileNames in the receiver. \"\r\n\r\n\t^self asArray compose: [:each| self emmit: each ] with: '\r\n'","(\r\nfunction(self)\r\nreturn (((self):asArray()):compose_with_((function(each)\r\nreturn ((self):emmit_(each))\r\nend), \"\\r\\n\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"emmitSt",0
,(
function(self)
return (((self):collect_((function(module)
return (smalltalk:object_comma_((smalltalk:object_comma_((module),".")),(smalltalk.String):sourceFileExtension()))
end))):emmit())
;
end
)
,"Builders-emmiting","emmitSt\r\n\t\" Emmit the code of source modules (st files) in the receiver. \"\r\n\r\n\t^(self collect: [:module| module ,$. ,String sourceFileExtension ]) emmit","(\r\nfunction(self)\r\nreturn (((self):collect_((function(module)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((module),\".\")),(smalltalk.String):sourceFileExtension()))\r\nend))):emmit())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Collection,"handles",0
,(
function(self)
return (((self):asArray()):handles())
;
end
)
,"Natives-converting","handles\r\n\t\" Return an array with the handles of the receiver. \"\r\n\r\n\t^self asArray handles","(\r\nfunction(self)\r\nreturn (((self):asArray()):handles())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"at:",0
,(
function(self,anIndex)
return ((self):at_ifAbsent_(anIndex, (function()
return ((self):objectNotFoundAt_(anIndex))
end)))
;
end
)
,"accessing","at: anIndex\r\n\t\" Return the object in the receiver at anIndex position. \"\r\n\r\n\t^self at: anIndex ifAbsent: [ self objectNotFoundAt: anIndex ]","(\r\nfunction(self,anIndex)\r\nreturn ((self):at_ifAbsent_(anIndex, (function()\r\nreturn ((self):objectNotFoundAt_(anIndex))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"at:ifAbsent:",0
,(
function(self,anIndex,aBlock)
return ((self):subclassResponsibility_("at:ifAbsent:"))
;
end
)
,"accessing","at: anIndex ifAbsent: aBlock\r\n\t\" Return the object in the receiver at anIndex position or the result of evaluating aBlock. \"\r\n\r\n\t^self subclassResponsibility: #at:ifAbsent:","(\r\nfunction(self,anIndex,aBlock)\r\nreturn ((self):subclassResponsibility_(\"at:ifAbsent:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"at:put:",0
,(
function(self,anIndex,anObject)
return ((self):subclassResponsibility_("at:put:"))
;
end
)
,"accessing","at: anIndex put: anObject\r\n\t\" Set the object in the receiver at anIndex position. \"\r\n\r\n\t^self subclassResponsibility: #at:put:","(\r\nfunction(self,anIndex,anObject)\r\nreturn ((self):subclassResponsibility_(\"at:put:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"at:ifAbsentPut:",0
,(
function(self,aKey,aBlock)
return ((self):at_ifAbsent_(aKey, (function()
return ((self):at_put_(aKey, (aBlock):value()))
end)))
;
end
)
,"accessing","at: aKey ifAbsentPut: aBlock\r\n\t\" Return the value at aKey, or add the result of evaluating aBlock to the receiver if it is absent. \"\r\n\r\n\t^self at: aKey ifAbsent: [ self at: aKey put: aBlock value ]","(\r\nfunction(self,aKey,aBlock)\r\nreturn ((self):at_ifAbsent_(aKey, (function()\r\nreturn ((self):at_put_(aKey, (aBlock):value()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"copyFrom:to:",0
,(
function(self,anIndex,anotherIndex)
return ((self):subclassResponsibility_("copyFrom:to:"))
;
end
)
,"copying","copyFrom: anIndex to: anotherIndex\r\n\t\" Return a copy of the receiver. \"\r\n\r\n\t^self subclassResponsibility: #copyFrom:to:","(\r\nfunction(self,anIndex,anotherIndex)\r\nreturn ((self):subclassResponsibility_(\"copyFrom:to:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"indexOf:",0
,(
function(self,anObject)
return ((self):indexOf_ifAbsent_(anObject, (function()
return ((0))
end)))
;
end
)
,"testing","indexOf: anObject\r\n\t\" Return the position of anObject in the receiver, or 0. \"\r\n\r\n\t^self indexOf: anObject ifAbsent: [ 0 ]","(\r\nfunction(self,anObject)\r\nreturn ((self):indexOf_ifAbsent_(anObject, (function()\r\nreturn ((0))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"indexOf:ifAbsent:",0
,(
function(self,anObject,aBlock)
	for i,value in ipairs(self) do
		if (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then return i end
	end
	if true then return aBlock:value() end
;
return (self)
;
end
)
,"testing","indexOf: anObject ifAbsent: aBlock\r\n\t\" Return the position of anObject in the receiver, or the result of evaluating aBlock. \"\r\n\r\n{'\tfor(var i=0;i<self.length;i++){ if (self[i]._eq(anObject)) { return i + 1; } }\r\n\treturn aBlock()'\r\nlua'\tfor i,value in ipairs(self) do\r\n\t\tif (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then return i end\r\n\tend\r\n\tif true then return aBlock:value() end'}","(\r\nfunction(self,anObject,aBlock)\r\n\tfor i,value in ipairs(self) do\r\n\t\tif (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then return i end\r\n\tend\r\n\tif true then return aBlock:value() end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"do:",0
,(
function(self,aBlock)
	for _,value in ipairs(self) do
		aBlock(self:unwrap_(value))
	end
;
return (self)
;
end
)
,"enumerating","do: aBlock\r\n\t\" Evaluate aBlock with the contents of the receiver. \"\r\n\r\n{'\tfor(var i=0;i<self.length;i++){aBlock(self[i]);}'\r\nlua'\tfor _,value in ipairs(self) do\r\n\t\taBlock(self:unwrap_(value))\r\n\tend'}","(\r\nfunction(self,aBlock)\r\n\tfor _,value in ipairs(self) do\r\n\t\taBlock(self:unwrap_(value))\r\n\tend\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"detect:ifNone:",0
,(
function(self,aBlock,anotherBlock)
	for _,value in ipairs(self) do
		local v=self:unwrap_(value)
		if (aBlock(v)) then return v end
	end
	if true then return anotherBlock:value() end
;
return (self)
;
end
)
,"enumerating","detect: aBlock ifNone: anotherBlock\r\n\t\" Return the object in the receiver that return true when evaluated with aBlock or the result of evaluating anotherBlock. \"\r\n\r\n{'\tfor(var i = 0; i < self.length; i++) if(aBlock(self[i])) return self[i];\r\n\treturn anotherBlock()'\r\nlua'\tfor _,value in ipairs(self) do\r\n\t\tlocal v=self:unwrap_(value)\r\n\t\tif (aBlock(v)) then return v end\r\n\tend\r\n\tif true then return anotherBlock:value() end'}","(\r\nfunction(self,aBlock,anotherBlock)\r\n\tfor _,value in ipairs(self) do\r\n\t\tlocal v=self:unwrap_(value)\r\n\t\tif (aBlock(v)) then return v end\r\n\tend\r\n\tif true then return anotherBlock:value() end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"reversed",0
,(
function(self)
	local result = smalltalk.Array:new()
	local top = #self + 1
	for i,v in ipairs(self) do result[top - i] = v end
	if true then return result end
;
return (self)
;
end
)
,"copying","reversed\r\n\t\" Return an array with the contents of the receiver in reverse order. \"\r\n\r\n{'\tvar result = new Array();\r\n\tvar len=self.length; var i=len;\r\n\twhile(i--){result[len-i-1]= self[i];}\r\n\treturn result'\r\nlua'\tlocal result = smalltalk.Array:new()\r\n\tlocal top = #self + 1\r\n\tfor i,v in ipairs(self) do result[top - i] = v end\r\n\tif true then return result end'}","(\r\nfunction(self)\r\n\tlocal result = smalltalk.Array:new()\r\n\tlocal top = #self + 1\r\n\tfor i,v in ipairs(self) do result[top - i] = v end\r\n\tif true then return result end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"isEmpty",0
,(
function(self)
return (#self==0)
;
end
)
,"testing","isEmpty\r\n\t\" Return true if the receiver is empty.\r\n\tNote: ^self size = 0 is slower than inlined impl.\r\n\t\"\r\n\r\n\t^{'(this.length)?false:true'\r\n\tlua'#self==0'}","(\r\nfunction(self)\r\nreturn (#self==0)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"reverseDo:",0
,(
function(self,aBlock)
	for i = #self,1,-1 do aBlock(self:unwrap_(self[i])) end
;
return (self)
;
end
)
,"enumerating","reverseDo: aBlock\r\n\t\" Evaluate aBlock with the contents of the receiver in reverse order of iteration. \"\r\n\r\n{'\tvar i=self.length;while(i--){aBlock(self[i]);}'\r\nlua'\tfor i = #self,1,-1 do aBlock(self:unwrap_(self[i])) end'}","(\r\nfunction(self,aBlock)\r\n\tfor i = #self,1,-1 do aBlock(self:unwrap_(self[i])) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"includes:",0
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
	for _,value in ipairs(self) do
		if (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then return true end
	end
	if true then return false end
;
(function() if ((((self):isObject_(anObject)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((((self):indexOf(anObject)) > (-1)))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
	if true then end
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"testing","includes: anObject\r\n\t\" Return true if anObject is in the receiver. \"\r\n\r\n{'\t'\r\nlua'\tfor _,value in ipairs(self) do\r\n\t\tif (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then return true end\r\n\tend\r\n\tif true then return false end'}.\r\n\r\n\t\" Note: the indexOf() function fail to search in case of string literals in Array contents. \"\r\n\t(self isObject: anObject) ifFalse: [\r\n\t\t^(self #indexOf: anObject) > -1\r\n\t].\r\n{'\tif (anObject.isNumber() && (self.indexOf(anObject) > -1)) return true;\r\n\tif (anObject.isString() && (self.indexOf(anObject) > -1)) return true;\r\n\tvar i = self.length;\r\n\twhile (i--) { if (anObject._eq(self[i])) { return true; } }\r\n\treturn false'\r\nlua'\tif true then end'}","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n\tfor _,value in ipairs(self) do\r\n\t\tif (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then return true end\r\n\tend\r\n\tif true then return false end\r\n;\r\n(function() if ((((self):isObject_(anObject)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((((self):indexOf(anObject)) > (-1)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n\tif true then end\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"occurrencesOf:",0
,(
function(self,anObject)
	local count = 0
	for _,value in ipairs(self) do
		if (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then count = count + 1 end
	end
	if true then return count end
;
return (self)
;
end
)
,"counting","occurrencesOf: anObject\r\n\t\" Return the occurrences of anObject in the receiver. \"\r\n\r\n{'\tvar i = self.length; var count = 0;\r\n\twhile (i--) { if (self[i]._eq(anObject)) { count++; } }\r\n\treturn count'\r\nlua'\tlocal count = 0\r\n\tfor _,value in ipairs(self) do\r\n\t\tif (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then count = count + 1 end\r\n\tend\r\n\tif true then return count end'}","(\r\nfunction(self,anObject)\r\n\tlocal count = 0\r\n\tfor _,value in ipairs(self) do\r\n\t\tif (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then count = count + 1 end\r\n\tend\r\n\tif true then return count end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"first",0
,(
function(self)
return ((self):at_((1)))
;
end
)
,"accessing","first\r\n\t\" Return the object in the receiver. \"\r\n\r\n\t^self at: 1","(\r\nfunction(self)\r\nreturn ((self):at_((1)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"fourth",0
,(
function(self)
return ((self):at_((4)))
;
end
)
,"accessing","fourth\r\n\t\" Return the object in the receiver. \"\r\n\r\n\t^self at: 4","(\r\nfunction(self)\r\nreturn ((self):at_((4)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"last",0
,(
function(self)
return ((self):at_((self):size()))
;
end
)
,"accessing","last\r\n\t\" Return the object in the receiver. \"\r\n\r\n\t^self at: self size","(\r\nfunction(self)\r\nreturn ((self):at_((self):size()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"second",0
,(
function(self)
return ((self):at_((2)))
;
end
)
,"accessing","second\r\n\t\" Return the object in the receiver. \"\r\n\r\n\t^self at: 2","(\r\nfunction(self)\r\nreturn ((self):at_((2)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"third",0
,(
function(self)
return ((self):at_((3)))
;
end
)
,"accessing","third\r\n\t\" Return the object in the receiver. \"\r\n\r\n\t^self at: 3","(\r\nfunction(self)\r\nreturn ((self):at_((3)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"removeLast",0
,(
function(self)
return ((self):remove_((self):last()))
;
end
)
,"removing","removeLast\r\n\t\" Remove the last object in the receiver. \"\r\n\r\n\t^self remove: self last","(\r\nfunction(self)\r\nreturn ((self):remove_((self):last()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"removeFirst",0
,(
function(self)
return ((self):remove_((self):first()))
;
end
)
,"removing","removeFirst\r\n\t\" Remove the first object in the receiver. \"\r\n\r\n\t^self remove: self first","(\r\nfunction(self)\r\nreturn ((self):remove_((self):first()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"addLast:",0
,(
function(self,anObject)
return ((self):add_(anObject))
;
end
)
,"adding","addLast: anObject\r\n\t\" Add anObject to the receiver (at end). \"\r\n\r\n\t^self add: anObject","(\r\nfunction(self,anObject)\r\nreturn ((self):add_(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"atAll:put:",0
,(
function(self,indices,anObject)
(indices):do_((function(index)
return ((self):at_put_(index, anObject))
end))
;
return (self)
;
end
)
,"accessing","atAll: indices put: anObject\r\n\t\" Return the receiver after replacing those elements, indexed by the indices, with anObject. \"\r\n\r\n\tindices do: [ :index | self at: index put: anObject ]","(\r\nfunction(self,indices,anObject)\r\n(indices):do_((function(index)\r\nreturn ((self):at_put_(index, anObject))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"atAllPut:",0
,(
function(self,anObject)
(((1)):to_((self):size())):do_((function(index)
return ((self):at_put_(index, anObject))
end))
;
return (self)
;
end
)
,"accessing","atAllPut: anObject\r\n\t\" Replace all elements of the receiver with anObject. \"\r\n\r\n\t(1 to: self size) do: [ :index | self at: index put: anObject ]","(\r\nfunction(self,anObject)\r\n(((1)):to_((self):size())):do_((function(index)\r\nreturn ((self):at_put_(index, anObject))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"withIndexDo:",0
,(
function(self,aBlock)
(((1)):to_((self):size())):do_((function(index)
return ((aBlock):value_value_((self):at_(index), index))
end))
;
return (self)
;
end
)
,"enumerating","withIndexDo: aBlock\r\n\t\" Evaluate aBlock with all the index positions in the receiver. \"\r\n\r\n\t(1 to: self size) do: [ :index |\r\n\t\taBlock value: (self at: index) value: index\r\n\t]","(\r\nfunction(self,aBlock)\r\n(((1)):to_((self):size())):do_((function(index)\r\nreturn ((aBlock):value_value_((self):at_(index), index))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"=","_eq"
,(
function(self,aCollection)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local index=nil
(function() if (((((self)==aCollection)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((((self)["$klass"])==(aCollection)["$klass"])))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
	local _s8_value=((self):size())
	index=_s8_value; return _s8_value
end)()
;
(function() if ((((not (smalltalk:isObject_equalTo_((index),(aCollection):size())))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() while(not ((function()
return (((index) <= (0)))
end))()) do (function()
(function() if ((((smalltalk:isObject_equalTo_(((self):at_(index)),(aCollection):at_(index)))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)();return ((function()
	local _s8_value=((index):_minus((1)))
	index=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"testing","= aCollection\r\n\t\" Return true if the receiver is equal to argument. \"\r\n\r\n\t| index |\r\n\tself == aCollection ifTrue: [ ^true ].\r\n\tself class == aCollection class ifFalse: [ ^false ].\r\n\tindex := self size.\r\n\tindex ~= aCollection size ifTrue: [ ^false ].\r\n\t[ index <= 0 ] whileFalse: [\r\n\t\t(self at: index) = (aCollection at: index) ifFalse: [ ^false ].\r\n\t\tindex := index - 1\r\n\t].\r\n\t^true","(\r\nfunction(self,aCollection)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal index=nil\r\n(function() if (((((self)==aCollection)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((self)[\"$klass\"])==(aCollection)[\"$klass\"])))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):size())\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((not (smalltalk:isObject_equalTo_((index),(aCollection):size())))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() while(not ((function()\r\nreturn (((index) <= (0)))\r\nend))()) do (function()\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):at_(index)),(aCollection):at_(index)))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)();return ((function()\r\n\tlocal _s8_value=((index):_minus((1)))\r\n\tindex=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"beginsWith:",0
,(
function(self,aCollection)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((self):size()) < (aCollection):size())))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((self):size()),(aCollection):size()))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk:isObject_equalTo_((aCollection),self)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk:isObject_equalTo_((aCollection),(self):copyFrom_to_((1), (aCollection):size()))))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"testing","beginsWith: aCollection\r\n\t\" Return true if the receiver begins with aCollection. \"\r\n\r\n\tself size < aCollection size ifTrue: [ ^false ].\r\n\tself size = aCollection size ifTrue: [ ^aCollection = self ].\r\n\t^aCollection = (self copyFrom: 1 to: aCollection size)","(\r\nfunction(self,aCollection)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((self):size()) < (aCollection):size())))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):size()),(aCollection):size()))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk:isObject_equalTo_((aCollection),self)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk:isObject_equalTo_((aCollection),(self):copyFrom_to_((1), (aCollection):size()))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"endsWith:",0
,(
function(self,aCollection)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((self):size()) < (aCollection):size())))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((self):size()),(aCollection):size()))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk:isObject_equalTo_((aCollection),self)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk:isObject_equalTo_((aCollection),(self):copyFrom_to_((((self):size()):_minus((aCollection):size())):_plus((1)), (self):size()))))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"testing","endsWith: aCollection\r\n\t\" Return true if the receiver ends with aCollection. \"\r\n\r\n\tself size < aCollection size ifTrue: [ ^false ].\r\n\tself size = aCollection size ifTrue: [ ^aCollection = self ].\r\n\t^aCollection = (self copyFrom: self size - aCollection size + 1 to: self size)","(\r\nfunction(self,aCollection)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((self):size()) < (aCollection):size())))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):size()),(aCollection):size()))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk:isObject_equalTo_((aCollection),self)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk:isObject_equalTo_((aCollection),(self):copyFrom_to_((((self):size()):_minus((aCollection):size())):_plus((1)), (self):size()))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"copyReplaceFrom:to:with:",0
,(
function(self,start,stop,aCollection)
return (smalltalk:object_comma_((smalltalk:object_comma_(((self):copyFrom_to_((1), (start):_minus((1)))),aCollection)),(self):copyFrom_to_((stop):_plus((1)), (self):size())))
;
end
)
,"copying","copyReplaceFrom: start to: stop with: aCollection\r\n\t\" Return a collection containing a copy of the receiver with the elements at index positions from start through stop replaced with the elements of aCollection. \"\r\n\r\n\t^(self copyFrom: 1 to: start - 1)\r\n\t\t, aCollection\r\n\t\t, (self copyFrom: stop + 1 to: self size)","(\r\nfunction(self,start,stop,aCollection)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_(((self):copyFrom_to_((1), (start):_minus((1)))),aCollection)),(self):copyFrom_to_((stop):_plus((1)), (self):size())))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"copyWithout:",0
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local index=nil
(function()
	local _s8_value=((self):indexOf_ifAbsent_(anObject, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):copy())
end
)()
error(_s8Ret_,0)
end)())
end)))
	index=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_(((self):copyFrom_to_((1), (index):_minus((1)))),((self):copyFrom_to_((index):_plus((1)), (self):size())):copyWithout_(anObject)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"copying","copyWithout: anObject\r\n\t\" Return a copy of the receiver excluding anObject, if any. \"\r\n\r\n\t| index |\r\n\tindex := self indexOf: anObject ifAbsent: [ ^self copy ].\r\n\t^(self copyFrom: 1 to: index - 1)\r\n\t\t,((self copyFrom: index + 1 to: self size)\r\n\t\t\tcopyWithout: anObject)","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((self):indexOf_ifAbsent_(anObject, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):copy())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_(((self):copyFrom_to_((1), (index):_minus((1)))),((self):copyFrom_to_((index):_plus((1)), (self):size())):copyWithout_(anObject)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"copyWithAll:",0
,(
function(self,aCollection)
return (smalltalk:object_comma_((self),aCollection))
;
end
)
,"copying","copyWithAll: aCollection\r\n\t\" Return a copy of the receiver with all contents of aCollection. \"\r\n\r\n\t^self , aCollection ","(\r\nfunction(self,aCollection)\r\nreturn (smalltalk:object_comma_((self),aCollection))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"findFirst:",0
,(
function(self,aBlock)
return ((self):findFirst_ifAbsent_(aBlock, (function()
return ((self):errorAbsentObject())
end)))
;
end
)
,"searching","findFirst: aBlock\r\n\t\" Return the index of the first element of the receiver that causes aBlock to evaluate to true.\r\n         If no such element is found, report an error.\r\n\t\"\r\n\r\n\t^self findFirst: aBlock ifAbsent: [ self errorAbsentObject ]","(\r\nfunction(self,aBlock)\r\nreturn ((self):findFirst_ifAbsent_(aBlock, (function()\r\nreturn ((self):errorAbsentObject())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"findFirst:ifAbsent:",0
,(
function(self,aBlock,absentBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local index=nil
local size=nil
(function()
	local _s8_value=((self):size())
	size=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((1))
	index=_s8_value; return _s8_value
end)()
;
(function() while(((function()
return (((index) <= size))
end))()) do (function()
(function() if ((((aBlock):value_((self):at_(index))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (index)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function()
	local _s8_value=((index):_plus((1)))
	index=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((absentBlock):value())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"searching","findFirst: aBlock ifAbsent: absentBlock\r\n\t\" Return the index of the first element of the receiver that causes aBlock to evaluate to true.\r\n\tIf no such element is found, return the result of evaluating the argument block.\r\n\t\"\r\n\r\n\t| index size |\r\n\tsize := self size.\r\n\tindex := 1.\r\n\t[ index <= size ] whileTrue: [\r\n\t\t(aBlock value: (self at: index)) ifTrue: [ ^index ].\r\n\t\tindex := index + 1\r\n\t].\r\n\t^absentBlock value","(\r\nfunction(self,aBlock,absentBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal index=nil\r\nlocal size=nil\r\n(function()\r\n\tlocal _s8_value=((self):size())\r\n\tsize=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((1))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\nreturn (((index) <= size))\r\nend))()) do (function()\r\n(function() if ((((aBlock):value_((self):at_(index))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (index)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n\tlocal _s8_value=((index):_plus((1)))\r\n\tindex=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((absentBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"findLast:",0
,(
function(self,aBlock)
return ((self):findLast_ifAbsent_(aBlock, (function()
return ((self):errorAbsentObject())
end)))
;
end
)
,"searching","findLast: aBlock\r\n\t\" Return the index of the last element of the receiver that causes aBlock to evaluate to true.\r\n\tIf no such element is found, report an error.\r\n\t\"\r\n\r\n\t^self findLast: aBlock ifAbsent: [ self errorAbsentObject ]","(\r\nfunction(self,aBlock)\r\nreturn ((self):findLast_ifAbsent_(aBlock, (function()\r\nreturn ((self):errorAbsentObject())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"findLast:ifAbsent:",0
,(
function(self,aBlock,absentBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local index=nil
(function()
	local _s8_value=((self):size())
	index=_s8_value; return _s8_value
end)()
;
(function() while(((function()
return (((index) > (0)))
end))()) do (function()
(function() if ((((aBlock):value_((self):at_(index))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (index)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function()
	local _s8_value=((index):_minus((1)))
	index=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((absentBlock):value())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"searching","findLast: aBlock ifAbsent: absentBlock\r\n\t\" Return the index of the last element of the receiver that causes aBlock to evaluate to true.\r\n\tIf no such element is found, return the result of evaluating the argument block.\r\n\t\"\r\n\r\n\t| index |\r\n\tindex := self size.\r\n\t[ index > 0 ] whileTrue: [\r\n\t\t(aBlock value: (self at: index)) ifTrue: [ ^index ].\r\n\t\tindex := index - 1\r\n\t].\r\n\t^absentBlock value","(\r\nfunction(self,aBlock,absentBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((self):size())\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\nreturn (((index) > (0)))\r\nend))()) do (function()\r\n(function() if ((((aBlock):value_((self):at_(index))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (index)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n\tlocal _s8_value=((index):_minus((1)))\r\n\tindex=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((absentBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"replaceFrom:to:with:",0
,(
function(self,start,stop,aCollection)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:isObject_equalTo_((((stop):_minus(start)):_plus((1))),(aCollection):size()))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Replacement has wrong size"))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(self):replaceFrom_to_with_startingAt_(start, stop, aCollection, (1))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"replace","replaceFrom: start to: stop with: aCollection\r\n\t\" Return the receiver.\r\n\tReplace the elements of the receiver at index positions start through stop, with the elements of aCollection. \"\r\n\r\n\tstop - start + 1 = aCollection size ifFalse: [\r\n\t\t^self error: 'Replacement has wrong size'\r\n\t].\r\n\tself\r\n\t\treplaceFrom: start\r\n\t\tto: stop\r\n\t\twith: aCollection\r\n\t\tstartingAt: 1","(\r\nfunction(self,start,stop,aCollection)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:isObject_equalTo_((((stop):_minus(start)):_plus((1))),(aCollection):size()))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Replacement has wrong size\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(self):replaceFrom_to_with_startingAt_(start, stop, aCollection, (1))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"replaceFrom:to:with:startingAt:",0
,(
function(self,start,stop,aCollection,repStart)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local index1=nil
local index2=nil
(function() if ((((smalltalk:boolean_and_((((self)==aCollection)),(function()
return (((repStart) < start))
end)))))==true) then return ((function()
(function()
	local _s8_value=(((repStart):_plus(stop)):_minus(start))
	index2=_s8_value; return _s8_value
end)();(function()
	local _s8_value=(stop)
	index1=_s8_value; return _s8_value
end)();(function() while(((function()
return (((start) <= index1))
end))()) do (function()
(self):at_put_(index1, (aCollection):at_(index2));(function()
	local _s8_value=((index1):_minus((1)))
	index1=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=((index2):_minus((1)))
	index2=_s8_value; return _s8_value
end)())
end)() end;return nil end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=(repStart)
	index2=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(start)
	index1=_s8_value; return _s8_value
end)()
;
(function() while(((function()
return (((index1) <= stop))
end))()) do (function()
(self):at_put_(index1, (aCollection):at_(index2));(function()
	local _s8_value=((index1):_plus((1)))
	index1=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=((index2):_plus((1)))
	index2=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"replace","replaceFrom: start\r\n    to: stop\r\n    with: aCollection\r\n    startingAt: repStart\r\n\t\" Return the receiver.\r\n\tReplace the elements of the receiver at index positions start through stop with consecutive\r\n         elements of aCollection beginning at index position repStart.\r\n\t\"\r\n\r\n\t| index1 index2 |\r\n\t(self == aCollection and: [ repStart < start ]) ifTrue: [ \"do backward move for same object\"\r\n\t\tindex2 := repStart + stop - start.\r\n\t\tindex1 := stop.\r\n\t\t[ start <= index1 ] whileTrue: [\r\n\t\t\tself at: index1 put: (aCollection at: index2).\r\n\t\t\tindex1 := index1 - 1.\r\n\t\t\tindex2 := index2 - 1\r\n\t\t].\r\n\t\t^self\r\n\t].\r\n\tindex2 := repStart.\r\n\tindex1 := start.\r\n\t[ index1 <= stop ] whileTrue: [\r\n\t\tself at: index1 put: (aCollection at: index2).\r\n\t\tindex1 := index1 + 1.\r\n\t\tindex2 := index2 + 1\r\n\t]","(\r\nfunction(self,start,stop,aCollection,repStart)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal index1=nil\r\nlocal index2=nil\r\n(function() if ((((smalltalk:boolean_and_((((self)==aCollection)),(function()\r\nreturn (((repStart) < start))\r\nend)))))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=(((repStart):_plus(stop)):_minus(start))\r\n\tindex2=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=(stop)\r\n\tindex1=_s8_value; return _s8_value\r\nend)();(function() while(((function()\r\nreturn (((start) <= index1))\r\nend))()) do (function()\r\n(self):at_put_(index1, (aCollection):at_(index2));(function()\r\n\tlocal _s8_value=((index1):_minus((1)))\r\n\tindex1=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=((index2):_minus((1)))\r\n\tindex2=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(repStart)\r\n\tindex2=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(start)\r\n\tindex1=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\nreturn (((index1) <= stop))\r\nend))()) do (function()\r\n(self):at_put_(index1, (aCollection):at_(index2));(function()\r\n\tlocal _s8_value=((index1):_plus((1)))\r\n\tindex1=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=((index2):_plus((1)))\r\n\tindex2=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"replaceFrom:to:with:startingWith:",0
,(
function(self,start,stop,replacement,replacementStart)
return ((self):replaceFrom_to_with_startingAt_(start, stop, replacement, replacementStart))
;
end
)
,"replace","replaceFrom: start\r\n    to: stop\r\n    with: replacement\r\n    startingWith: replacementStart\r\n\t\" Replace the elements of the receiver. \"\r\n\r\n\t^self\r\n\t\treplaceFrom: start\r\n\t\tto: stop\r\n\t\twith: replacement\r\n\t\tstartingAt: replacementStart","(\r\nfunction(self,start,stop,replacement,replacementStart)\r\nreturn ((self):replaceFrom_to_with_startingAt_(start, stop, replacement, replacementStart))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"replaceFrom:to:withObject:",0
,(
function(self,start,stop,anObject)
local index=nil
(function()
	local _s8_value=(start)
	index=_s8_value; return _s8_value
end)()
;
(function() while(((function()
return (((index) <= stop))
end))()) do (function()
(self):at_put_(index, anObject);return ((function()
	local _s8_value=((index):_plus((1)))
	index=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
return (anObject)
;
end
)
,"replace","replaceFrom: start to: stop withObject: anObject\r\n\t\"Replace each of the elements of the receiver at index positions start through stop with anObject. \"\r\n\r\n\t| index |\r\n\tindex := start.\r\n\t[ index <= stop ] whileTrue: [\r\n\t\tself at: index put: anObject.\r\n\t\tindex := index + 1\r\n\t].\r\n\t^anObject","(\r\nfunction(self,start,stop,anObject)\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=(start)\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\nreturn (((index) <= stop))\r\nend))()) do (function()\r\n(self):at_put_(index, anObject);return ((function()\r\n\tlocal _s8_value=((index):_plus((1)))\r\n\tindex=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\nreturn (anObject)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"with:do:",0
,(
function(self,aCollection,aBlock)
local first=nil
local second=nil
(function()
	local _s8_value=((self):stream())
	first=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((aCollection):stream())
	second=_s8_value; return _s8_value
end)()
;
(function() while(not ((function()
return ((smalltalk:boolean_or_(((first):atEnd()),(function()
return ((second):atEnd())
end))))
end))()) do (function()
return ((aBlock):value_value_((first):next(), (second):next()))
end)() end;return nil end)()
;
return (self)
;
end
)
,"iterating","with: aCollection do: aBlock\r\n\t\" For each pair of elements evaluate aBlock with those elements as the arguments. \"\r\n\r\n\t| first second |\r\n\tfirst\t:= self stream.\r\n\tsecond\t:= aCollection stream.\r\n\t[ first atEnd or: [ second atEnd ] ] whileFalse: [\r\n\t\taBlock value: first next value: second next\r\n\t].","(\r\nfunction(self,aCollection,aBlock)\r\nlocal first=nil\r\nlocal second=nil\r\n(function()\r\n\tlocal _s8_value=((self):stream())\r\n\tfirst=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aCollection):stream())\r\n\tsecond=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(not ((function()\r\nreturn ((smalltalk:boolean_or_(((first):atEnd()),(function()\r\nreturn ((second):atEnd())\r\nend))))\r\nend))()) do (function()\r\nreturn ((aBlock):value_value_((first):next(), (second):next()))\r\nend)() end;return nil end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"sort",0
,(
function(self)
(self):sort_(nil)
;
return (self)
;
end
)
,"sorting","sort\r\n\t\" Sort (inplace) the receiver's contents with default criteria. \"\r\n\r\n\tself sort: nil","(\r\nfunction(self)\r\n(self):sort_(nil)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"sort:",0
,(
function(self,aBlock)
(self):implementedBySubclass_("sort:")
;
return (self)
;
end
)
,"sorting","sort: aBlock\r\n\t\" Sort (inplace) the receiver's contents with aBlock. \"\r\n\r\n\tself implementedBySubclass: #sort:","(\r\nfunction(self,aBlock)\r\n(self):implementedBySubclass_(\"sort:\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceableCollection,"asActionSequence",0
,(
function(self)
return ((smalltalk.ActionSequence):withAll_(self))
;
end
)
,"converting","asActionSequence\r\n\t\" Return an ActionSequence from the receiver. \"\r\n\r\n\t^ActionSequence withAll: self","(\r\nfunction(self)\r\nreturn ((smalltalk.ActionSequence):withAll_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"fromString:",0
,(
function(self,aString)
return (tostring(aString))
;
end
)
,"instantiation","fromString: aString\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^{'new self.$fn(aString)'\r\n\tlua'tostring(aString)'}","(\r\nfunction(self,aString)\r\nreturn (tostring(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"new",0
,(
function(self)
return ("")
;
end
)
,"instantiation","new\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^{'\"\"'\r\n\tlua'\"\"'}","(\r\nfunction(self)\r\nreturn (\"\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"cr",0
,(
function(self)
return ("\r")
;
end
)
,"constants","cr\r\n\t\" Return the singular instance. \"\r\n\r\n\t^{'\"\\r\"'\r\n\tlua'\"\\r\"'}","(\r\nfunction(self)\r\nreturn (\"\\r\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"lf",0
,(
function(self)
return ("\n")
;
end
)
,"constants","lf\r\n\t\" Return the singular instance. \"\r\n\r\n\t^{'\"\\n\"'\r\n\tlua'\"\\n\"'}","(\r\nfunction(self)\r\nreturn (\"\\n\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"crlf",0
,(
function(self)
return ("\r\n")
;
end
)
,"constants","crlf\r\n\t\" Return the singular instance. \"\r\n\r\n\t^{'\"\\r\\n\"'\r\n\tlua'\"\\r\\n\"'}","(\r\nfunction(self)\r\nreturn (\"\\r\\n\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"space",0
,(
function(self)
return (" ")
;
end
)
,"constants","space\r\n\t\" Return the singular instance. \"\r\n\r\n\t^{'\" \"'\r\n\tlua'\" \"'}","(\r\nfunction(self)\r\nreturn (\" \")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"tab",0
,(
function(self)
return ("\t")
;
end
)
,"constants","tab\r\n\t\" Return the singular instance. \"\r\n\r\n\t^{'\"\\t\"'\r\n\tlua'\"\\t\"'}","(\r\nfunction(self)\r\nreturn (\"\\t\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"zero",0
,(
function(self)
return (string.char(0))
;
end
)
,"constants","zero\r\n\t\" Return the singular instance. \"\r\n\r\n\t^{'\"\\0\"'\r\n\tlua'string.char(0)'}","(\r\nfunction(self)\r\nreturn (string.char(0))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"with:",0
,(
function(self,aCharacter)
return (aCharacter)
;
end
)
,"instantiation","with: aCharacter\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^aCharacter","(\r\nfunction(self,aCharacter)\r\nreturn (aCharacter)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"with:with:",0
,(
function(self,aCharacter,bCharacter)
return (smalltalk:object_comma_((aCharacter),bCharacter))
;
end
)
,"instantiation","with: aCharacter with: bCharacter\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^aCharacter , bCharacter","(\r\nfunction(self,aCharacter,bCharacter)\r\nreturn (smalltalk:object_comma_((aCharacter),bCharacter))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"with:with:with:",0
,(
function(self,aCharacter,bCharacter,cCharacter)
return (smalltalk:object_comma_((smalltalk:object_comma_((aCharacter),bCharacter)),cCharacter))
;
end
)
,"instantiation","with: aCharacter with: bCharacter with: cCharacter\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^aCharacter , bCharacter , cCharacter","(\r\nfunction(self,aCharacter,bCharacter,cCharacter)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((aCharacter),bCharacter)),cCharacter))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"with:with:with:with:",0
,(
function(self,aCharacter,bCharacter,cCharacter,dCharacter)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((aCharacter),bCharacter)),cCharacter)),dCharacter))
;
end
)
,"instantiation","with: aCharacter with: bCharacter\r\n\twith: cCharacter with: dCharacter\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^aCharacter , bCharacter , cCharacter , dCharacter","(\r\nfunction(self,aCharacter,bCharacter,cCharacter,dCharacter)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((aCharacter),bCharacter)),cCharacter)),dCharacter))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"fromCharCode:",0
,(
function(self,anArrayOrNumber)
	if (type(anArrayOrNumber)=="number") then
		return string.char(anArrayOrNumber)
	end
	if true then
		local fup = table.unpack or unpack
		return (string.char(fup(anArrayOrNumber)))
	end

;
return (self)
;
end
)
,"instantiation","fromCharCode: anArrayOrNumber\r\n\t\" Return an instance of the receiver. \"\r\n\r\n{'\tif (anArrayOrNumber.isNumber()) return String.fromCharCode(anArrayOrNumber);\r\n\treturn String.fromCharCode.apply(null,anArrayOrNumber.asArray());'\r\nlua'\tif (type(anArrayOrNumber)==\"number\") then\r\n\t\treturn string.char(anArrayOrNumber)\r\n\tend\r\n\tif true then\r\n\t\tlocal fup = table.unpack or unpack\r\n\t\treturn (string.char(fup(anArrayOrNumber)))\r\n\tend\r\n'}","(\r\nfunction(self,anArrayOrNumber)\r\n\tif (type(anArrayOrNumber)==\"number\") then\r\n\t\treturn string.char(anArrayOrNumber)\r\n\tend\r\n\tif true then\r\n\t\tlocal fup = table.unpack or unpack\r\n\t\treturn (string.char(fup(anArrayOrNumber)))\r\n\tend\r\n\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"doItSelector",0
,(
function(self)
return ("s8DoIt")
;
end
)
,"doIt","doItSelector\r\n\t\" Return the selector to use for doIt evaluations.\r\n\tWarning: this selector will not be dump on image nor change log expressions.\r\n\t\"\r\n\r\n\t^#s8DoIt","(\r\nfunction(self)\r\nreturn (\"s8DoIt\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"reservedArray",0
,(
function(self)
return (("abstract as boolean break byte case catch char class\r\n\tcontinue const debugger default delete do double else\r\n\tenum export extends false final finally float for function\r\n\tgoto if implements import in instanceof int interface is\r\n\tlong namespace native new null package private\r\n\tprotected public return short static super switch\r\n\tsynchronized this throw throws transient true try typeof\r\n\tuse var void volatile while with"):asArrayOfSubstrings())
;
end
)
,"js","reservedArray\r\n\t\" Private - Return the reserved names of native language. \"\r\n\r\n\t^'abstract as boolean break byte case catch char class\r\n\tcontinue const debugger default delete do double else\r\n\tenum export extends false final finally float for function\r\n\tgoto if implements import in instanceof int interface is\r\n\tlong namespace native new null package private\r\n\tprotected public return short static super switch\r\n\tsynchronized this throw throws transient true try typeof\r\n\tuse var void volatile while with' asArrayOfSubstrings","(\r\nfunction(self)\r\nreturn ((\"abstract as boolean break byte case catch char class\\r\\n\\tcontinue const debugger default delete do double else\\r\\n\\tenum export extends false final finally float for function\\r\\n\\tgoto if implements import in instanceof int interface is\\r\\n\\tlong namespace native new null package private\\r\\n\\tprotected public return short static super switch\\r\\n\\tsynchronized this throw throws transient true try typeof\\r\\n\\tuse var void volatile while with\"):asArrayOfSubstrings())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"reservedWords",0
,(
function(self)
return ((self):basicAt_ifAbsentPut_("$reservedSet", (function()
return ((smalltalk.HashedSet):withAll_((self):reservedArray()))
end)))
;
end
)
,"js","reservedWords\r\n\t\" Private - Return the reserved names of native language. \"\r\n\r\n\t^self basicAt: '$reservedSet' ifAbsentPut: [\r\n\t\tHashedSet withAll: self reservedArray\r\n\t]","(\r\nfunction(self)\r\nreturn ((self):basicAt_ifAbsentPut_(\"$reservedSet\", (function()\r\nreturn ((smalltalk.HashedSet):withAll_((self):reservedArray()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"streamClass",0
,(
function(self)
return (smalltalk.StringStream)
;
end
)
,"streams","streamClass\r\n\t\" Return the streamming support for the receiver's instances. \"\r\n\r\n\t^StringStream","(\r\nfunction(self)\r\nreturn (smalltalk.StringStream)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"nativeFileExtension",0
,(
function(self)
return ((self):nativeLanguage())
;
end
)
,"Builders-emmiting","nativeFileExtension\r\n\t\" Return the file extension to append to fileName when reading/writing native files. \"\r\n\r\n\t^self nativeLanguage","(\r\nfunction(self)\r\nreturn ((self):nativeLanguage())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"sourceFileExtension",0
,(
function(self)
return ("st")
;
end
)
,"Builders-emmiting","sourceFileExtension\r\n\t\" Return the file extension to append to fileName when reading/writing source files. \"\r\n\r\n\t^#st","(\r\nfunction(self)\r\nreturn (\"st\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"@","_at"
,(
function(self,aHandle)
return ((function() if ((((((aHandle))~=nil)))==true) then return ((function()
return (smalltalk:toString_((aHandle)))
end))() else return nil end end)())
;
end
)
,"Natives-converting","@ aHandle\r\n\t\" Returns an instance of the receiver from aHandle. \"\r\n\r\n\t^aHandle notNil ifTrue: [ aHandle toString ]","(\r\nfunction(self,aHandle)\r\nreturn ((function() if ((((((aHandle))~=nil)))==true) then return ((function()\r\nreturn (smalltalk:toString_((aHandle)))\r\nend))() else return nil end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"reservedLuaWords",0
,(
function(self)
return (smalltalk:newArray({"and", "break", "do", "else", "elseif", "end", "false", "for", "function", "if", "in", "local", "nil", "not", "or", "repeat", "return", "then", "true", "until", "while"}))
;
end
)
,"Lua-lua","reservedLuaWords\r\n\t\" Private - Return the reserved names of Lua language. \"\r\n\r\n\t^#(\r\n\tand\tbreak\tdo\telse\telseif\r\n\tend\t#false\tfor\tfunction\tif\r\n\tin\tlocal\t#nil\tnot\tor\r\n\trepeat\treturn\tthen\t#true\tuntil\r\n\twhile\r\n\t)","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"and\", \"break\", \"do\", \"else\", \"elseif\", \"end\", \"false\", \"for\", \"function\", \"if\", \"in\", \"local\", \"nil\", \"not\", \"or\", \"repeat\", \"return\", \"then\", \"true\", \"until\", \"while\"}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String["$klass"],"forbiddenLuaSelectors",0
,(
function(self)
return (smalltalk:newArray({"class", "new"}))
;
end
)
,"Lua-lua","forbiddenLuaSelectors\r\n\t\" Private - Return the platform forbidden selectors. \"\r\n\r\n\t^#( class new )","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"class\", \"new\"}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"=","_eq"
,(
function(self,anObject)
	if true then return self==anObject end
;
return (self)
;
end
)
,"comparing","= anObject\r\n    \" Return true if the receiver is equal to argument. \"\r\n\r\n{'\tif (nil.isNil_(anObject)) return false;\r\n\tif (!(anObject.isString)) return false;\r\n\tif (!(anObject.isString())) return false;\r\n\tif (self == anObject) return true;\r\n\treturn ((\"\"+self) == (\"\"+anObject))'\r\nlua'\tif true then return self==anObject end'}","(\r\nfunction(self,anObject)\r\n\tif true then return self==anObject end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"size",0
,(
function(self)
return (#self)
;
end
)
,"accessing","size\r\n\t\" Return the size of the receiver. \"\r\n\r\n\t^{'self.length'\r\n\tlua'#self'}","(\r\nfunction(self)\r\nreturn (#self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"at:",0
,(
function(self,anIndex)
return (self:sub(anIndex,anIndex))
;
end
)
,"accessing","at: anIndex\r\n\t\" Return the receiver at anIndex. \"\r\n\r\n\t^{'self.charAt(anIndex - 1)'\r\n\tlua'self:sub(anIndex,anIndex)'}","(\r\nfunction(self,anIndex)\r\nreturn (self:sub(anIndex,anIndex))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"at:put:",0
,(
function(self,anIndex,anObject)
return ((self):errorReadOnly())
;
end
)
,"accessing","at: anIndex put: anObject\r\n\t\" Set the receiver at anIndex. \"\r\n\r\n\t^self errorReadOnly","(\r\nfunction(self,anIndex,anObject)\r\nreturn ((self):errorReadOnly())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"at:ifAbsent:",0
,(
function(self,anIndex,aBlock)
return ((function() local _1_=(((self):at_(anIndex)));if (_1_==nil) then return (aBlock):value() end; return _1_ end)())
;
end
)
,"accessing","at: anIndex ifAbsent: aBlock\r\n\t\" Return the receiver at anIndex position or the result of evaluating aBlock. \"\r\n\r\n\t^(self at: anIndex) ifNil: aBlock","(\r\nfunction(self,anIndex,aBlock)\r\nreturn ((function() local _1_=(((self):at_(anIndex)));if (_1_==nil) then return (aBlock):value() end; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"do:",0
,(
function(self,aBlock)
	for c in self:gmatch(".") do aBlock(c) end
;
return (self)
;
end
)
,"enumerating","do: aBlock\r\n\t\" Evaluate aBlock with the contents of the receiver. \"\r\n\r\n{'\tfor(var i=0;i<self.length;i++){aBlock(self.charAt(i));}'\r\nlua'\tfor c in self:gmatch(\".\") do aBlock(c) end'}","(\r\nfunction(self,aBlock)\r\n\tfor c in self:gmatch(\".\") do aBlock(c) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"detect:ifNone:",0
,(
function(self,aBlock,anotherBlock)
	for c in self:gmatch(".") do
		if (aBlock(c)) then return c end
	end
	if true then return anotherBlock() end
;
return (self)
;
end
)
,"enumerating","detect: aBlock ifNone: anotherBlock\r\n\t\" Return the object in the receiver that return true when evaluated with aBlock or the result of evaluating anotherBlock. \"\r\n\r\n{'\tfor(var i = 0; i < self.length; i++)\r\n\t\tif(aBlock(self.charAt(i))) return self.charAt(i);\r\n\treturn anotherBlock()'\r\nlua'\tfor c in self:gmatch(\".\") do\r\n\t\tif (aBlock(c)) then return c end\r\n\tend\r\n\tif true then return anotherBlock() end'}","(\r\nfunction(self,aBlock,anotherBlock)\r\n\tfor c in self:gmatch(\".\") do\r\n\t\tif (aBlock(c)) then return c end\r\n\tend\r\n\tif true then return anotherBlock() end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"includes:",0
,(
function(self,anObject)
return ((((self):indexOf_(anObject)) > (0)))
;
end
)
,"testing","includes: anObject\r\n\t\" Return true if anObject is in the receiver. \"\r\n\r\n\t^(self indexOf: anObject) > 0","(\r\nfunction(self,anObject)\r\nreturn ((((self):indexOf_(anObject)) > (0)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"withIndexDo:",0
,(
function(self,aBlock)
	for i=1,#self do aBlock(self:sub(i,i),i) end
;
return (self)
;
end
)
,"enumerating","withIndexDo: aBlock\r\n\t\" Evaluate aBlock with all the index positions in the receiver. \"\r\n\r\n{'\tfor(var i=0;i<self.length;i++){aBlock(self.charAt(i), i+1);}'\r\nlua'\tfor i=1,#self do aBlock(self:sub(i,i),i) end'}","(\r\nfunction(self,aBlock)\r\n\tfor i=1,#self do aBlock(self:sub(i,i),i) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"copyWithout:",0
,(
function(self,anObject)
return ((self):replace_with_(anObject, ""))
;
end
)
,"copying","copyWithout: anObject\r\n\t\" Return a copy of the receiver excluding anObject, if any. \"\r\n\r\n\t^self replace: anObject with: ''","(\r\nfunction(self,anObject)\r\nreturn ((self):replace_with_(anObject, \"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asSymbol",0
,(
function(self)
return (self)
;
end
)
,"conversion","asSymbol\r\n\t\" Return a symbol with the same characters as the receiver. \"\r\n\r\n\t^self \"we do not discriminate singular strings\"","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"argumentCount",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((0))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if (((("|&\\\\+*/=><,@%~-"):includes_((self):first())))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((1))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):occurrencesOf_(":"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"evaluating","argumentCount\r\n\t\" Return the number of arguments defined by the receiver. \"\r\n\r\n\tself isEmpty ifTrue: [ ^0 ].\r\n\t('|&\\\\+*/=><,@%~-' includes: self first) ifTrue: [ ^1 ].\r\n\t^self occurrencesOf: $:","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((0))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((\"|&\\\\\\\\+*/=><,@%~-\"):includes_((self):first())))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((1))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):occurrencesOf_(\":\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isLiteral",0
,(
function(self)
	if #self==0 then return false end
	if true then return string.find(self,"[^a-zA-Z0-9_:]+")==nil end

;
return ((smalltalk:boolean_and_(((self):notEmpty()),(function()
return ((((((smalltalk.RegularExpression):fromString_("[^a-zA-Z0-9_:]")):exec_(self)))==nil))
end))))
;
end
)
,"testing","isLiteral\r\n\t\" Return true if the receiver can be output using literal notation. \"\r\n\r\n{''lua'\tif #self==0 then return false end\r\n\tif true then return string.find(self,\"[^a-zA-Z0-9_:]+\")==nil end\r\n'}.\r\n\t^self notEmpty and: [\r\n\t\t((RegularExpression fromString: '[^a-zA-Z0-9_:]') exec: self) isNil\r\n\t]","(\r\nfunction(self)\r\n\tif #self==0 then return false end\r\n\tif true then return string.find(self,\"[^a-zA-Z0-9_:]+\")==nil end\r\n\r\n;\r\nreturn ((smalltalk:boolean_and_(((self):notEmpty()),(function()\r\nreturn ((((((smalltalk.RegularExpression):fromString_(\"[^a-zA-Z0-9_:]\")):exec_(self)))==nil))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asLiteral",0
,(
function(self)
return ((function() if ((((self):isLiteral()))==true) then return ((function()
return (smalltalk:object_comma_(("#"),self))
end))() else return ((function()
return (smalltalk:printString_((self)))
end))() end end)())
;
end
)
,"converting","asLiteral\r\n\t\" Return the receiver in literal notation. \"\r\n\r\n\t^self isLiteral ifTrue: [ $# ,self ] ifFalse: [ self printString ]","(\r\nfunction(self)\r\nreturn ((function() if ((((self):isLiteral()))==true) then return ((function()\r\nreturn (smalltalk:object_comma_((\"#\"),self))\r\nend))() else return ((function()\r\nreturn (smalltalk:printString_((self)))\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"occurrencesOf:",0
,(
function(self,aString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
local index=nil
	local j
	local k,count = 0,0
	repeat	j, k = self:find(aString, k + 1, true)
		if (j~=nil) then count=count+1 end
	until j == nil
	if true then return count end

;
(function() if ((((aString):isString()))==true) then return nil else return ((function()
(function()
	local _s8_value=((self):match(aString))
	result=_s8_value; return _s8_value
end)();(function() if ((((((result))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((0))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result["length"])
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((aString):size()),(1)))))==true) then return ((function()
(function()
	local _s8_value=((self):match((smalltalk.RegularExpression):fromString_flag_(smalltalk:object_comma_((smalltalk:object_comma_(("["),aString)),"]"), "g")))
	result=_s8_value; return _s8_value
end)();(function() if ((((((result))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((0))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result["length"])
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((0))
	result=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((0))
	index=_s8_value; return _s8_value
end)()
;
(function() while(((function()
(function()
	local _s8_value=((self):indexOf_startingAt_(aString, (index):_plus((1))))
	index=_s8_value; return _s8_value
end)();return (((index) > (1)))
end))()) do (function()
return ((function()
	local _s8_value=((result):_plus((1)))
	result=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"testing","occurrencesOf: aString\r\n\t\" Return the times aString can be found in the receiver. \"\r\n\r\n\t| result index |\r\n{''lua'\tlocal j\r\n\tlocal k,count = 0,0\r\n\trepeat\tj, k = self:find(aString, k + 1, true)\r\n\t\tif (j~=nil) then count=count+1 end\r\n\tuntil j == nil\r\n\tif true then return count end\r\n'}.\taString isString ifFalse: [\r\n\t\tresult := self #match: aString.\r\n\t\tresult isNil ifTrue: [ ^0 ].\r\n\t\t^result#length\r\n\t].\r\n\taString size = 1 ifTrue: [\r\n\t\tresult := self #match: (RegularExpression\r\n\t\t\t\tfromString: '[',aString,']'\r\n\t\t\t\tflag: 'g').\r\n\t\tresult isNil ifTrue: [ ^0 ].\r\n\t\t^result#length\r\n\t].\r\n\tresult := 0. index := 0.\r\n\t[\tindex := self indexOf: aString startingAt: index + 1.\r\n\t\tindex > 1 ] whileTrue: [ result := result + 1 ].\r\n\t^result","(\r\nfunction(self,aString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\nlocal index=nil\r\n\tlocal j\r\n\tlocal k,count = 0,0\r\n\trepeat\tj, k = self:find(aString, k + 1, true)\r\n\t\tif (j~=nil) then count=count+1 end\r\n\tuntil j == nil\r\n\tif true then return count end\r\n\r\n;\r\n(function() if ((((aString):isString()))==true) then return nil else return ((function()\r\n(function()\r\n\tlocal _s8_value=((self):match(aString))\r\n\tresult=_s8_value; return _s8_value\r\nend)();(function() if ((((((result))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((0))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result[\"length\"])\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((aString):size()),(1)))))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((self):match((smalltalk.RegularExpression):fromString_flag_(smalltalk:object_comma_((smalltalk:object_comma_((\"[\"),aString)),\"]\"), \"g\")))\r\n\tresult=_s8_value; return _s8_value\r\nend)();(function() if ((((((result))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((0))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result[\"length\"])\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((0))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((0))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\n(function()\r\n\tlocal _s8_value=((self):indexOf_startingAt_(aString, (index):_plus((1))))\r\n\tindex=_s8_value; return _s8_value\r\nend)();return (((index) > (1)))\r\nend))()) do (function()\r\nreturn ((function()\r\n\tlocal _s8_value=((result):_plus((1)))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"startsWith:",0
,(
function(self,aString)
	if true then return (self:sub(1, #aString) == aString) end
;
return ((self):beginsWith_(aString))
;
end
)
,"testing","startsWith: aString\r\n\t\" Return true if the receiver begins with the characters of aString. \"\r\n\r\n{'\tif (self.startsWith) { return self.startsWith(aString) }'\r\nlua'\tif true then return (self:sub(1, #aString) == aString) end'}.\r\n\t^self beginsWith: aString","(\r\nfunction(self,aString)\r\n\tif true then return (self:sub(1, #aString) == aString) end\r\n;\r\nreturn ((self):beginsWith_(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"escaped",0
,(
function(self)
return (self:notImplementedYet_("escaped"))
;
end
)
,"converting","escaped\r\n\t\" Return the receiver with escape codes. \"\r\n\r\n\t^{'escape(self)'\r\n\tlua'self:notImplementedYet_(\"escaped\")'}","(\r\nfunction(self)\r\nreturn (self:notImplementedYet_(\"escaped\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"unescaped",0
,(
function(self)
	local x16 = function(x) return string.char(tonumber(x, 16)) end
	if true then return self:gsub("%%(%x%x)", x16) end

;
return (self)
;
end
)
,"converting","unescaped\r\n\t\" Return the receiver from encoded URI escape codes. \"\r\n\r\n{'\treturn unescape(self)\r\n'lua'\tlocal x16 = function(x) return string.char(tonumber(x, 16)) end\r\n\tif true then return self:gsub(\"%%(%x%x)\", x16) end\r\n'}","(\r\nfunction(self)\r\n\tlocal x16 = function(x) return string.char(tonumber(x, 16)) end\r\n\tif true then return self:gsub(\"%%(%x%x)\", x16) end\r\n\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"escapedCode",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local escaped=nil
(function()
	local _s8_value=((self):escaped())
	escaped=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_((escaped),self))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("\""),escaped)),"\""))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("unescape(\""),escaped)),"\")"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","escapedCode\r\n\t\" Return the code to reproduce receiver from code. \"\r\n\r\n\t| escaped |\r\n\tescaped := self escaped.\r\n\tescaped = self ifTrue: [ ^$\" ,escaped ,$\" ].\r\n\t^'unescape(\"' ,escaped ,'\")'","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal escaped=nil\r\n(function()\r\n\tlocal _s8_value=((self):escaped())\r\n\tescaped=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((escaped),self))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"\\\"\"),escaped)),\"\\\"\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"unescape(\\\"\"),escaped)),\"\\\")\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asEncodedURI",0
,(
function(self)
return (self:notImplementedYet_("asEncodedURI"))
;
end
)
,"converting","asEncodedURI\r\n\t\" Return the receiver encoded as an URI component.\r\n\tEncodes special characters and it encodes the following characters: , / ? : @ & = + $ #\r\n\t\"\r\n\r\n\t^{'encodeURIComponent(self)'\r\n\tlua'self:notImplementedYet_(\"asEncodedURI\")'}","(\r\nfunction(self)\r\nreturn (self:notImplementedYet_(\"asEncodedURI\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asDecodedURI",0
,(
function(self)
return (self:notImplementedYet_("asDecodedURI"))
;
end
)
,"converting","asDecodedURI\r\n\t\" Return the receiver decoded as an URI component. \"\r\n\r\n\t^{'decodeURIComponent(self)'\r\n\tlua'self:notImplementedYet_(\"asDecodedURI\")'}","(\r\nfunction(self)\r\nreturn (self:notImplementedYet_(\"asDecodedURI\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"add:",0
,(
function(self,anObject)
return ((self):errorReadOnly())
;
end
)
,"adding","add: anObject\r\n\t\" Add anObject to the receiver. \"\r\n\r\n\t^self errorReadOnly","(\r\nfunction(self,anObject)\r\nreturn ((self):errorReadOnly())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"-->","_minus_minus_gt"
,(
function(self,anObject)
return ((smalltalk.Association):key_value_((self):asArrayOfSubstringsSeparatedBy_("."), anObject))
;
end
)
,"converting","--> anObject\r\n\t\" Return a far-association of the receiver to anObject. \"\r\n\r\n\t^Association key: (self asArrayOfSubstringsSeparatedBy: $.) value: anObject","(\r\nfunction(self,anObject)\r\nreturn ((smalltalk.Association):key_value_((self):asArrayOfSubstringsSeparatedBy_(\".\"), anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,",","_comma"
,(
function(self,aCollection)
	if true then return self..(smalltalk:asString_(aCollection)) end
;
return (self)
;
end
)
,"copying",", aCollection\r\n\t\" Return a copy of the receiver with all contents of aCollection. \"\r\n\r\n{'\tif (nil.isNil_(aCollection)) return self+\"\"; // we need this ugly patch for JSC under iOS\r\n\treturn self + aCollection.toString()'\r\nlua'\tif true then return self..(smalltalk:asString_(aCollection)) end'}","(\r\nfunction(self,aCollection)\r\n\tif true then return self..(smalltalk:asString_(aCollection)) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"copyWith:",0
,(
function(self,anObject)
return (self..(smalltalk:asString_(aCollection)))
;
end
)
,"copying","copyWith: anObject\r\n\t\" Return a copy of the receiver with anObject. \"\r\n\r\n\t^{'self + anObject.toString()'\r\n\tlua'self..(smalltalk:asString_(aCollection))'}","(\r\nfunction(self,anObject)\r\nreturn (self..(smalltalk:asString_(aCollection)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"copyFrom:to:",0
,(
function(self,anIndex,anotherIndex)
return (self:sub(anIndex,anotherIndex))
;
end
)
,"copying","copyFrom: anIndex to: anotherIndex\r\n\t\" Return the receiver between limits. \"\r\n\r\n\t^{'self.substring(anIndex - 1,anotherIndex)'\r\n\tlua'self:sub(anIndex,anotherIndex)'}","(\r\nfunction(self,anIndex,anotherIndex)\r\nreturn (self:sub(anIndex,anotherIndex))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"shallowCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"deepCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"reversed",0
,(
function(self)
local result=nil
	if true then return self:reverse() end

;
(function()
	local _s8_value=("")
	result=_s8_value; return _s8_value
end)()
;
(self):reverseDo_((function(c)
return ((function()
	local _s8_value=(smalltalk:object_comma_((result),c))
	result=_s8_value; return _s8_value
end)())
end))
;
return (result)
;
end
)
,"copying","reversed\r\n\t\" Return a string with the contents of the receiver in reverse order. \"\r\n\r\n\t| result |\r\n{''lua'\tif true then return self:reverse() end\r\n'}.\r\n\tresult := ''.\r\n\tself reverseDo: [:c| result := result , c ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n\tif true then return self:reverse() end\r\n\r\n;\r\n(function()\r\n\tlocal _s8_value=(\"\")\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):reverseDo_((function(c)\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((result),c))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"indexOf:",0
,(
function(self,aString)
	local index = self:find(aString,1,true)
	if not index
		then return 0
		else return index end
;
return (self)
;
end
)
,"accessing","indexOf: aString\r\n\t\" Return the position of first occurrence of aString (or 0 if not present). \"\r\n\r\n{'\treturn self.indexOf(aString) + 1\r\n'lua'\tlocal index = self:find(aString,1,true)\r\n\tif not index\r\n\t\tthen return 0\r\n\t\telse return index end'}","(\r\nfunction(self,aString)\r\n\tlocal index = self:find(aString,1,true)\r\n\tif not index\r\n\t\tthen return 0\r\n\t\telse return index end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"indexOf:ifAbsent:",0
,(
function(self,aString,aBlock)
local result=nil
(function()
	local _s8_value=((self):indexOf_(aString))
	result=_s8_value; return _s8_value
end)()
;
return (((smalltalk:isObject_equalTo_((result),(0)))):ifTrue_ifFalse_(aBlock, (function()
return (result)
end)))
;
end
)
,"accessing","indexOf: aString ifAbsent: aBlock\r\n\t\" Return the position of first occurrence of aString (or the result of evaluating aBlock). \"\r\n\r\n\t| result |\r\n\tresult := self indexOf: aString.\r\n\t^result = 0 ifTrue: aBlock ifFalse: [ result ]","(\r\nfunction(self,aString,aBlock)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):indexOf_(aString))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (((smalltalk:isObject_equalTo_((result),(0)))):ifTrue_ifFalse_(aBlock, (function()\r\nreturn (result)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"indexOf:startingAt:",0
,(
function(self,aString,position)
	local index = self:find(aString,position,true)
	if not index
		then return 0
		else return index end
;
return (self)
;
end
)
,"accessing","indexOf: aString startingAt: position\r\n\t\" Return the position of first occurrence of aString after position (or 0 if not present). \"\r\n\r\n{'\treturn self.indexOf(aString,position-1) + 1\r\n'lua'\tlocal index = self:find(aString,position,true)\r\n\tif not index\r\n\t\tthen return 0\r\n\t\telse return index end'}","(\r\nfunction(self,aString,position)\r\n\tlocal index = self:find(aString,position,true)\r\n\tif not index\r\n\t\tthen return 0\r\n\t\telse return index end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"lastIndexOf:",0
,(
function(self,aString)
	local i = 0
	local j = 0
	local k = 0
	repeat	i = j
		j, k = self:find(aString, k + 1, true)
	until j == nil
	if true then return i end
;
return (self)
;
end
)
,"accessing","lastIndexOf: aString\r\n\t\" Return the position of last occurrence of aString (or 0 if not present). \"\r\n\r\n{'\treturn self.lastIndexOf(aString) + 1'\r\nlua'\tlocal i = 0\r\n\tlocal j = 0\r\n\tlocal k = 0\r\n\trepeat\ti = j\r\n\t\tj, k = self:find(aString, k + 1, true)\r\n\tuntil j == nil\r\n\tif true then return i end'}","(\r\nfunction(self,aString)\r\n\tlocal i = 0\r\n\tlocal j = 0\r\n\tlocal k = 0\r\n\trepeat\ti = j\r\n\t\tj, k = self:find(aString, k + 1, true)\r\n\tuntil j == nil\r\n\tif true then return i end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"lastIndexOf:ifAbsent:",0
,(
function(self,aString,aBlock)
local result=nil
(function()
	local _s8_value=((self):lastIndexOf_(aString))
	result=_s8_value; return _s8_value
end)()
;
return (((smalltalk:isObject_equalTo_((result),(0)))):ifTrue_ifFalse_(aBlock, (function()
return (result)
end)))
;
end
)
,"accessing","lastIndexOf: aString ifAbsent: aBlock\r\n\t\" Return the position of last occurrence of aString (or the result of evaluating aBlock). \"\r\n\r\n\t| result |\r\n\tresult := self lastIndexOf: aString.\r\n\t^result = 0 ifTrue: aBlock ifFalse: [ result ]","(\r\nfunction(self,aString,aBlock)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):lastIndexOf_(aString))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (((smalltalk:isObject_equalTo_((result),(0)))):ifTrue_ifFalse_(aBlock, (function()\r\nreturn (result)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"lastIndexOf:startingAt:",0
,(
function(self,aString,position)
	local i, j
	local k = position
	repeat	i = j
		j, k = self:find(aString, k + 1, true)
	until j == nil
	if true then return i end
;
return (self)
;
end
)
,"accessing","lastIndexOf: aString startingAt: position\r\n\t\" Return the position of last occurrence of aString before position (or 0 if not present). \"\r\n\r\n{'\treturn self.lastIndexOf(aString,position-1) + 1'\r\nlua'\tlocal i, j\r\n\tlocal k = position\r\n\trepeat\ti = j\r\n\t\tj, k = self:find(aString, k + 1, true)\r\n\tuntil j == nil\r\n\tif true then return i end'}","(\r\nfunction(self,aString,position)\r\n\tlocal i, j\r\n\tlocal k = position\r\n\trepeat\ti = j\r\n\t\tj, k = self:find(aString, k + 1, true)\r\n\tuntil j == nil\r\n\tif true then return i end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"upTo:startingAt:",0
,(
function(self,aString,position)
local index=nil
(function()
	local _s8_value=((self):indexOf_startingAt_(aString, position))
	index=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((function() if ((((smalltalk:isObject_equalTo_((index),(0)))))==true) then return ((function()
return ((self):size())
end))() else return ((function()
return ((index):_minus((1)))
end))() end end)())
	index=_s8_value; return _s8_value
end)()
;
return ((self):copyFrom_to_(position, index))
;
end
)
,"accessing","upTo: aString startingAt: position\r\n\t\" Return the receiver upto first occurrence of aString after position. \"\r\n\r\n\t| index |\r\n\tindex := self indexOf: aString startingAt: position.\r\n\tindex := index = 0 ifTrue: [ self size ] ifFalse: [ index - 1 ].\r\n\t^self copyFrom: position to: index","(\r\nfunction(self,aString,position)\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((self):indexOf_startingAt_(aString, position))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function() if ((((smalltalk:isObject_equalTo_((index),(0)))))==true) then return ((function()\r\nreturn ((self):size())\r\nend))() else return ((function()\r\nreturn ((index):_minus((1)))\r\nend))() end end)())\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((self):copyFrom_to_(position, index))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"upTo:",0
,(
function(self,aCharacter)
return ((self):upTo_startingAt_(aCharacter, (1)))
;
end
)
,"accessing","upTo: aCharacter\r\n\t\" Return the receiver upto first occurrence of aCharacter. \"\r\n\r\n\t^self upTo: aCharacter startingAt: 1","(\r\nfunction(self,aCharacter)\r\nreturn ((self):upTo_startingAt_(aCharacter, (1)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"upToLast:",0
,(
function(self,aString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local index=nil
(function()
	local _s8_value=((self):lastIndexOf_ifAbsent_(aString, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end)))
	index=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):copyFrom_to_((1), (index):_minus((1))))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","upToLast: aString\r\n\t\" Return the receiver upto last occurrence of aString. \"\r\n\r\n\t| index |\r\n\tindex := self lastIndexOf: aString ifAbsent: [ ^self ].\r\n\t^self copyFrom: 1 to: index - 1","(\r\nfunction(self,aString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((self):lastIndexOf_ifAbsent_(aString, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):copyFrom_to_((1), (index):_minus((1))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"fromLast:",0
,(
function(self,aString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local index=nil
(function()
	local _s8_value=((self):lastIndexOf_ifAbsent_(aString, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end)))
	index=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):copyFrom_to_((index):_plus((aString):size()), (self):size()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","fromLast: aString\r\n\t\" Return the receiver contents from last occurrence of aString. \"\r\n\r\n\t| index |\r\n\tindex := self lastIndexOf: aString ifAbsent: [ ^self ].\r\n\t^self copyFrom: index + aString size to: self size","(\r\nfunction(self,aString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((self):lastIndexOf_ifAbsent_(aString, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):copyFrom_to_((index):_plus((aString):size()), (self):size()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asNativeName",0
,(
function(self)
return (self:asLuaName())
;
end
)
,"js","asNativeName\r\n\t\" Return the receiver as a valid native name. \"\r\n\r\n\t^{'self.asJavascriptName()'\r\n\tlua'self:asLuaName()'}","(\r\nfunction(self)\r\nreturn (self:asLuaName())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asJavascriptName",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((smalltalk.String):reservedWords()):includes_(self)))==true) then return ((function()
return ((function() if ((((smalltalk:isObject_equalTo_((self),"toString"))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_(("$$"),self))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"js","asJavascriptName\r\n\t\" Private - Return the receiver as a valid native name. \"\r\n\r\n\t(String reservedWords includes: self) ifTrue: [\r\n\t\tself = #toString ifFalse: [ ^'$$' ,self ]\r\n\t].\r\n\t^self","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((smalltalk.String):reservedWords()):includes_(self)))==true) then return ((function()\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((self),\"toString\"))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((\"$$\"),self))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asSelector",0
,(
function(self)
	if (self:startsWith_("#")) then return self:upTo_startingAt_(":",2) end
	local map = {
		[":"]="_",
		["+"]="_plus",
		["-"]="_minus",
		["*"]="_star",
		["/"]="_slash",
		["|"]="_bar",
		["&"]="_amp",
		[">"]="_gt",
		["<"]="_lt",
		["="]="_eq",
		[","]="_comma",
		["@"]="_at",
		["~"]="_no",
		["\\"]="_bk"
	};
	local result = self:gsub(".",function(c) return map[c] end)
	if true then return result:asLuaName() end
;
return (self)
;
end
)
,"converting","asSelector\r\n\t\" Private - Return the receiver as a valid native selector. \"\r\n\r\n{'\tif (self[0]==\"#\") return self.upTo_startingAt_(\":\",2);\r\n\treturn self\r\n\t\t.replace(/:/g,\"_\")\r\n\t\t.replace(/[+]/g,\"_plus\")\r\n\t\t.replace(/-/g,\"_minus\")\r\n\t\t.replace(/[*]/g,\"_star\")\r\n\t\t.replace(/[/]/g,\"_slash\")\r\n\t\t.replace(/[|]/g,\"_bar\")\r\n\t\t.replace(/[&]/g,\"_amp\")\r\n\t\t.replace(/>/g,\"_gt\")\r\n\t\t.replace(/</g,\"_lt\")\r\n\t\t.replace(/=/g,\"_eq\")\r\n\t\t.replace(/,/g,\"_comma\")\r\n\t\t.replace(/[@]/g,\"_at\")\r\n\t\t.replace(/~/g,\"_no\")\r\n\t\t.replace(/\\\\/g,\"_bk\")\r\n\t\t.asJavascriptName()'\r\nlua'\tif (self:startsWith_(\"#\")) then return self:upTo_startingAt_(\":\",2) end\r\n\tlocal map = {\r\n\t\t[\":\"]=\"_\",\r\n\t\t[\"+\"]=\"_plus\",\r\n\t\t[\"-\"]=\"_minus\",\r\n\t\t[\"*\"]=\"_star\",\r\n\t\t[\"/\"]=\"_slash\",\r\n\t\t[\"|\"]=\"_bar\",\r\n\t\t[\"&\"]=\"_amp\",\r\n\t\t[\">\"]=\"_gt\",\r\n\t\t[\"<\"]=\"_lt\",\r\n\t\t[\"=\"]=\"_eq\",\r\n\t\t[\",\"]=\"_comma\",\r\n\t\t[\"@\"]=\"_at\",\r\n\t\t[\"~\"]=\"_no\",\r\n\t\t[\"\\\\\"]=\"_bk\"\r\n\t};\r\n\tlocal result = self:gsub(\".\",function(c) return map[c] end)\r\n\tif true then return result:asLuaName() end'}","(\r\nfunction(self)\r\n\tif (self:startsWith_(\"#\")) then return self:upTo_startingAt_(\":\",2) end\r\n\tlocal map = {\r\n\t\t[\":\"]=\"_\",\r\n\t\t[\"+\"]=\"_plus\",\r\n\t\t[\"-\"]=\"_minus\",\r\n\t\t[\"*\"]=\"_star\",\r\n\t\t[\"/\"]=\"_slash\",\r\n\t\t[\"|\"]=\"_bar\",\r\n\t\t[\"&\"]=\"_amp\",\r\n\t\t[\">\"]=\"_gt\",\r\n\t\t[\"<\"]=\"_lt\",\r\n\t\t[\"=\"]=\"_eq\",\r\n\t\t[\",\"]=\"_comma\",\r\n\t\t[\"@\"]=\"_at\",\r\n\t\t[\"~\"]=\"_no\",\r\n\t\t[\"\\\\\"]=\"_bk\"\r\n\t};\r\n\tlocal result = self:gsub(\".\",function(c) return map[c] end)\r\n\tif true then return result:asLuaName() end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asStSelector",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local operations=nil
(function() if ((((smalltalk:isObject_equalTo_((self),"$"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((self),"_bk"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:asCharacter_(((92))))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((self):startsWith_("$$")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self):copyFrom_to_((3), (self):size())):asStSelector())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=(smalltalk:newArray({smalltalk:newArray({"+", "_plus"}), smalltalk:newArray({"-", "_minus"}), smalltalk:newArray({"*", "_star"}), smalltalk:newArray({"/", "_slash"}), smalltalk:newArray({"|", "_bar"}), smalltalk:newArray({"&", "_amp"}), smalltalk:newArray({">", "_gt"}), smalltalk:newArray({"<", "_lt"}), smalltalk:newArray({"=", "_eq"}), smalltalk:newArray({",", "_comma"}), smalltalk:newArray({"@", "_at"}), smalltalk:newArray({"~", "_no"})}))
	operations=_s8_value; return _s8_value
end)()
;
(operations):add_((smalltalk.Array):with_with_(smalltalk:asCharacter_(((92))), "_bk"))
;
(operations):do_((function(tuple)
return ((function() if ((((self):startsWith_((tuple):last())))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((operations):reversed()):inject_into_(self, (function(result,each)
return ((result):replace_with_((each):last(), (each):first()))
end)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):replace_with_("_", ":"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","asStSelector\r\n\t\" Return a smalltalk selector from a native function name.\r\n\tNote: it must be the reverse operation of #asSelector\r\n\t\"\r\n\r\n\t| operations |\r\n\tself = '$' ifTrue: [ ^'' ].\r\n\tself = #_bk ifTrue: [ ^92 asCharacter ].\r\n\t(self startsWith: '$$') ifTrue: [ ^(self copyFrom: 3 to: self size) asStSelector ].\r\n\toperations := #(\r\n\t\t#( + _plus )\r\n\t\t#( - _minus )\r\n\t\t#( * _star )\r\n\t\t#( / _slash )\r\n\t\t#( | _bar )\r\n\t\t#( & _amp )\r\n\t\t#( > _gt )\r\n\t\t#( < _lt )\r\n\t\t#( = _eq )\r\n\t\t#( , _comma )\r\n\t\t#( @ _at )\r\n\t\t#( ~ _no )\r\n\t).\r\n\toperations add: (Array with: 92 asCharacter with: #_bk).\r\n\toperations do: [:tuple|\r\n\t\t(self startsWith: tuple last) ifTrue: [\r\n\t\t\t^operations reversed inject: self into: [:result :each|\r\n\t\t\t\tresult replace: each last with: each first\r\n\t\t\t]\r\n\t\t].\r\n\t].\r\n\t^self replace: $_ with: $:","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal operations=nil\r\n(function() if ((((smalltalk:isObject_equalTo_((self),\"$\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((self),\"_bk\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:asCharacter_(((92))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((self):startsWith_(\"$$\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self):copyFrom_to_((3), (self):size())):asStSelector())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({smalltalk:newArray({\"+\", \"_plus\"}), smalltalk:newArray({\"-\", \"_minus\"}), smalltalk:newArray({\"*\", \"_star\"}), smalltalk:newArray({\"/\", \"_slash\"}), smalltalk:newArray({\"|\", \"_bar\"}), smalltalk:newArray({\"&\", \"_amp\"}), smalltalk:newArray({\">\", \"_gt\"}), smalltalk:newArray({\"<\", \"_lt\"}), smalltalk:newArray({\"=\", \"_eq\"}), smalltalk:newArray({\",\", \"_comma\"}), smalltalk:newArray({\"@\", \"_at\"}), smalltalk:newArray({\"~\", \"_no\"})}))\r\n\toperations=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(operations):add_((smalltalk.Array):with_with_(smalltalk:asCharacter_(((92))), \"_bk\"))\r\n;\r\n(operations):do_((function(tuple)\r\nreturn ((function() if ((((self):startsWith_((tuple):last())))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((operations):reversed()):inject_into_(self, (function(result,each)\r\nreturn ((result):replace_with_((each):last(), (each):first()))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):replace_with_(\"_\", \":\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asNative:",0
,(
function(self,lang)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((lang)=="lua")))==true) then return ((function()
local encoded=nil
local bar=nil
(function()
	local _s8_value=(smalltalk:asCharacter_(((92))))
	bar=_s8_value; return _s8_value
end)();(function()
	local _s8_value=(self)
	encoded=_s8_value; return _s8_value
end)();(function()
	local _s8_value=((encoded):replace_with_(bar, smalltalk:object_comma_((bar),bar)))
	encoded=_s8_value; return _s8_value
end)();(function()
	local _s8_value=((encoded):replace_with_((smalltalk.String):lf(), smalltalk:object_comma_((bar),"n")))
	encoded=_s8_value; return _s8_value
end)();(function()
	local _s8_value=((encoded):replace_with_((smalltalk.String):cr(), smalltalk:object_comma_((bar),"r")))
	encoded=_s8_value; return _s8_value
end)();(function()
	local _s8_value=((encoded):replace_with_((smalltalk.String):tab(), smalltalk:object_comma_((bar),"t")))
	encoded=_s8_value; return _s8_value
end)();(function()
	local _s8_value=((encoded):replace_with_("\"", smalltalk:object_comma_((bar),"\"")))
	encoded=_s8_value; return _s8_value
end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("\""),encoded)),"\""))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
self:notImplementedYet_("asNative:")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","asNative: lang\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\tlang == #lua ifTrue: [ | encoded bar |\r\n\t\tbar := 92 asCharacter.\r\n\t\tencoded := self.\r\n\t\tencoded := encoded replace: bar with: bar ,bar.\r\n\t\tencoded := encoded replace: String lf with: bar ,$n.\r\n\t\tencoded := encoded replace: String cr with: bar ,$r.\r\n\t\tencoded := encoded replace: String tab with: bar ,$t.\r\n\t\tencoded := encoded replace: $\" with: bar ,$\".\r\n\t\t^$\" ,encoded ,$\"\r\n\t].\r\n{'\tvar bar=unescape(\"%22\");\r\n\tif(self.search(/^[a-zA-Z0-9_:.$ ]*$/) == -1)\r\n\t\treturn \"unescape(\" +bar +escape(self) +bar +\")\";\r\n\telse\r\n\t\treturn bar +self +bar' lua'self:notImplementedYet_(\"asNative:\")'}","(\r\nfunction(self,lang)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((lang)==\"lua\")))==true) then return ((function()\r\nlocal encoded=nil\r\nlocal bar=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:asCharacter_(((92))))\r\n\tbar=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=(self)\r\n\tencoded=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=((encoded):replace_with_(bar, smalltalk:object_comma_((bar),bar)))\r\n\tencoded=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=((encoded):replace_with_((smalltalk.String):lf(), smalltalk:object_comma_((bar),\"n\")))\r\n\tencoded=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=((encoded):replace_with_((smalltalk.String):cr(), smalltalk:object_comma_((bar),\"r\")))\r\n\tencoded=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=((encoded):replace_with_((smalltalk.String):tab(), smalltalk:object_comma_((bar),\"t\")))\r\n\tencoded=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=((encoded):replace_with_(\"\\\"\", smalltalk:object_comma_((bar),\"\\\"\")))\r\n\tencoded=_s8_value; return _s8_value\r\nend)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"\\\"\"),encoded)),\"\\\"\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nself:notImplementedYet_(\"asNative:\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"replace:with:",0
,(
function(self,aString,anotherString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
	local result=""
	local i,j
	local k = 0
	repeat	i=k+1
		j, k = self:find(aString, i, true)
		if (j==nil) then
			result=result..(self:sub(i))
		else
			result=result..(self:sub(i,j-1)..anotherString)
		end
	until j == nil
	if true then return result end

;
(function() if (((((aString)=="\\")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):replace_with_("\\\\", anotherString))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):replaceRegexp_with_((smalltalk.RegularExpression):fromString_flag_(aString, "g"), anotherString))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"regular expressions","replace: aString with: anotherString\r\n\t\" Return the result of replacement of aString with anotherString in the receiver. \"\r\n\r\n{''lua'\tlocal result=\"\"\r\n\tlocal i,j\r\n\tlocal k = 0\r\n\trepeat\ti=k+1\r\n\t\tj, k = self:find(aString, i, true)\r\n\t\tif (j==nil) then\r\n\t\t\tresult=result..(self:sub(i))\r\n\t\telse\r\n\t\t\tresult=result..(self:sub(i,j-1)..anotherString)\r\n\t\tend\r\n\tuntil j == nil\r\n\tif true then return result end\r\n'}.\taString == $\\ ifTrue: [ ^self replace: '\\\\' with: anotherString ].\r\n\t^self\r\n\t\treplaceRegexp: (RegularExpression fromString: aString flag: 'g')\r\n\t\twith: anotherString","(\r\nfunction(self,aString,anotherString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n\tlocal result=\"\"\r\n\tlocal i,j\r\n\tlocal k = 0\r\n\trepeat\ti=k+1\r\n\t\tj, k = self:find(aString, i, true)\r\n\t\tif (j==nil) then\r\n\t\t\tresult=result..(self:sub(i))\r\n\t\telse\r\n\t\t\tresult=result..(self:sub(i,j-1)..anotherString)\r\n\t\tend\r\n\tuntil j == nil\r\n\tif true then return result end\r\n\r\n;\r\n(function() if (((((aString)==\"\\\\\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):replace_with_(\"\\\\\\\\\", anotherString))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):replaceRegexp_with_((smalltalk.RegularExpression):fromString_flag_(aString, \"g\"), anotherString))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"replaceRegexp:with:",0
,(
function(self,aRegexp,aString)
return (self:notImplementedYet_("replaceRegexp:with:"))
;
end
)
,"regular expressions","replaceRegexp: aRegexp with: aString\r\n\t\" Return the result of replacement of aRegexp with aString in the receiver. \"\r\n\r\n\t^{'self.replace(aRegexp,aString)'\r\n\tlua'self:notImplementedYet_(\"replaceRegexp:with:\")'}","(\r\nfunction(self,aRegexp,aString)\r\nreturn (self:notImplementedYet_(\"replaceRegexp:with:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"tokenize:",0
,(
function(self,aString)
	local result=smalltalk.Array:new()
	local i,j
	local k = 0
	repeat	i=k+1
		j, k = self:find(aString, i, true)
		if (j==nil) then
			result:add_(self:sub(i))
		else
			result:add_(self:sub(i,j-#aString))
		end
	until j == nil
	if true then return result end
;
return (self)
;
end
)
,"converting","tokenize: aString\r\n\t\" Return the split tokenization of the receiver. \"\r\n\r\n{'\treturn self.split(aString)\r\n'lua'\tlocal result=smalltalk.Array:new()\r\n\tlocal i,j\r\n\tlocal k = 0\r\n\trepeat\ti=k+1\r\n\t\tj, k = self:find(aString, i, true)\r\n\t\tif (j==nil) then\r\n\t\t\tresult:add_(self:sub(i))\r\n\t\telse\r\n\t\t\tresult:add_(self:sub(i,j-#aString))\r\n\t\tend\r\n\tuntil j == nil\r\n\tif true then return result end'}","(\r\nfunction(self,aString)\r\n\tlocal result=smalltalk.Array:new()\r\n\tlocal i,j\r\n\tlocal k = 0\r\n\trepeat\ti=k+1\r\n\t\tj, k = self:find(aString, i, true)\r\n\t\tif (j==nil) then\r\n\t\t\tresult:add_(self:sub(i))\r\n\t\telse\r\n\t\t\tresult:add_(self:sub(i,j-#aString))\r\n\t\tend\r\n\tuntil j == nil\r\n\tif true then return result end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asArrayOfSubstrings",0
,(
function(self)
local result=nil
local start=nil
local i=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	result=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((1))
	i=_s8_value; return _s8_value
end)()
;
(self):do_((function(c)
(function() if ((((((start))==nil)))==true) then return ((function()
return ((function() if ((((c):isSeparator()))==true) then return nil else return ((function()
return ((function()
	local _s8_value=(i)
	start=_s8_value; return _s8_value
end)())
end))() end end)())
end))() else return ((function()
return ((function() if ((((c):isSeparator()))==true) then return ((function()
(result):add_((self):copyFrom_to_(start, (i):_minus((1))));return ((function()
	local _s8_value=(nil)
	start=_s8_value; return _s8_value
end)())
end))() else return nil end end)())
end))() end end)();return ((function()
	local _s8_value=((i):_plus((1)))
	i=_s8_value; return _s8_value
end)())
end))
;
(function() if ((((((start))~=nil)))==true) then return ((function()
return ((result):add_((self):copyFrom_to_(start, (self):size())))
end))() else return nil end end)()
;
return (result)
;
end
)
,"converting","asArrayOfSubstrings\r\n\t\" Return an array of substrings from the receiver.\r\n\tThe receiver is divided into substrings by one or more occurrences of separators.\r\n\t\"\r\n\r\n\t| result start i |\r\n\tresult := Array new. i := 1.\r\n\tself do: [:c|\r\n\t\tstart isNil ifTrue: [ \"in separator\"\r\n\t\t\tc isSeparator ifFalse: [ \"start of substring\"\r\n\t\t\t\tstart := i.\r\n\t\t\t]\r\n\t\t] ifFalse: [ \"in word\"\r\n\t\t\tc isSeparator ifTrue: [ \"end of substring\"\r\n\t\t\t\tresult add: (self copyFrom: start to: i - 1).\r\n\t\t\t\tstart := nil.\r\n\t\t\t].\r\n\t\t].\r\n\t\ti := i + 1.\r\n\t].\r\n\tstart notNil ifTrue: [\r\n\t\tresult add: (self copyFrom: start to: self size)\r\n\t].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\nlocal start=nil\r\nlocal i=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((1))\r\n\ti=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):do_((function(c)\r\n(function() if ((((((start))==nil)))==true) then return ((function()\r\nreturn ((function() if ((((c):isSeparator()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(i)\r\n\tstart=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)())\r\nend))() else return ((function()\r\nreturn ((function() if ((((c):isSeparator()))==true) then return ((function()\r\n(result):add_((self):copyFrom_to_(start, (i):_minus((1))));return ((function()\r\n\tlocal _s8_value=(nil)\r\n\tstart=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)())\r\nend))() end end)();return ((function()\r\n\tlocal _s8_value=((i):_plus((1)))\r\n\ti=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\n(function() if ((((((start))~=nil)))==true) then return ((function()\r\nreturn ((result):add_((self):copyFrom_to_(start, (self):size())))\r\nend))() else return nil end end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asArrayOfSubstringsSeparatedBy:",0
,(
function(self,aCharacter)
return ((self):tokenize_(aCharacter))
;
end
)
,"converting","asArrayOfSubstringsSeparatedBy: aCharacter\r\n\t\" Return an array of substrings from the receiver.\r\n\tThe receiver is divided into substrings by one or more occurrences of aCharacter.\r\n\t\"\r\n\r\n\t^self tokenize: aCharacter","(\r\nfunction(self,aCharacter)\r\nreturn ((self):tokenize_(aCharacter))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"match:",0
,(
function(self,aRegexp)
return (self:notImplementedYet_("match:"))
;
end
)
,"regular expressions","match: aRegexp\r\n\t\" Return the match index of aRegexp in the receiver. \"\r\n\r\n\t^{'self.search(aRegexp) != -1'\r\n\tlua'self:notImplementedYet_(\"match:\")'}","(\r\nfunction(self,aRegexp)\r\nreturn (self:notImplementedYet_(\"match:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asString",0
,(
function(self)
return (self)
;
end
)
,"converting","asString\r\n\t\" Return the text representation of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"between:and:",0
,(
function(self,min,max)
return ((smalltalk:boolean_and_((((min) <= self)),(function()
return (((self) <= max))
end))))
;
end
)
,"testing","between: min and: max\r\n\t\" Return true if the receiver is between arguments. \"\r\n\r\n\t^min <= self and: [ self <= max ]","(\r\nfunction(self,min,max)\r\nreturn ((smalltalk:boolean_and_((((min) <= self)),(function()\r\nreturn (((self) <= max))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isLetter",0
,(
function(self)
return (string.find(self,"%a")==1 and #self==1)
;
end
)
,"testing","isLetter\r\n\t\" Return true if the receiver is a one character letter. \"\r\n\r\n\t^{'self.search(\"[A-Za-z]$\")==0'\r\n\tlua'string.find(self,\"%a\")==1 and #self==1'}","(\r\nfunction(self)\r\nreturn (string.find(self,\"%a\")==1 and #self==1)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isDigit",0
,(
function(self)
return (string.find(self,"%d")==1 and #self==1)
;
end
)
,"testing","isDigit\r\n\t\" Return true if the receiver is one character numeric digit. \"\r\n\r\n\t^{'self.search(\"[0-9]$\")==0'\r\n\tlua'string.find(self,\"%d\")==1 and #self==1'}","(\r\nfunction(self)\r\nreturn (string.find(self,\"%d\")==1 and #self==1)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isAlphaNumeric",0
,(
function(self)
return ((smalltalk:boolean_or_(((self):isDigit()),(function()
return ((self):isLetter())
end))))
;
end
)
,"testing","isAlphaNumeric\r\n\t\" Return true if the receiver is an alphanumeric character. \"\r\n\r\n\t^self isDigit or: [self isLetter]","(\r\nfunction(self)\r\nreturn ((smalltalk:boolean_or_(((self):isDigit()),(function()\r\nreturn ((self):isLetter())\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isUppercase",0
,(
function(self)
return ((self):isUpperCase())
;
end
)
,"testing","isUppercase\r\n\t\" Return true if all the receiver characters are uppercase. \"\r\n\r\n\t^self isUpperCase","(\r\nfunction(self)\r\nreturn ((self):isUpperCase())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isUpperCase",0
,(
function(self)
	if (#self==0) then return false end
	for c in self:gmatch(".") do
		if (c<"A" or c>"Z") then return false end
	end
	if true then return true end
;
return ((smalltalk:booleanNot_(((self):match_("[^A-Z]")))))
;
end
)
,"testing","isUpperCase\r\n\t\" Return true if all the receiver characters are uppercase. \"\r\n\r\n{''lua'\tif (#self==0) then return false end\r\n\tfor c in self:gmatch(\".\") do\r\n\t\tif (c<\"A\" or c>\"Z\") then return false end\r\n\tend\r\n\tif true then return true end'}.\r\n\t^(self match: '[^A-Z]') not","(\r\nfunction(self)\r\n\tif (#self==0) then return false end\r\n\tfor c in self:gmatch(\".\") do\r\n\t\tif (c<\"A\" or c>\"Z\") then return false end\r\n\tend\r\n\tif true then return true end\r\n;\r\nreturn ((smalltalk:booleanNot_(((self):match_(\"[^A-Z]\")))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isLowerCase",0
,(
function(self)
	if (#self==0) then return false end
	for c in self:gmatch(".") do
		if (c<"a" or c>"z") then return false end
	end
	if true then return true end
;
return ((smalltalk:booleanNot_(((self):match_("[^a-z]")))))
;
end
)
,"testing","isLowerCase\r\n\t\" Return true if all the receiver characters are lowercase. \"\r\n\r\n{''lua'\tif (#self==0) then return false end\r\n\tfor c in self:gmatch(\".\") do\r\n\t\tif (c<\"a\" or c>\"z\") then return false end\r\n\tend\r\n\tif true then return true end'}.\r\n\t^(self match: '[^a-z]') not","(\r\nfunction(self)\r\n\tif (#self==0) then return false end\r\n\tfor c in self:gmatch(\".\") do\r\n\t\tif (c<\"a\" or c>\"z\") then return false end\r\n\tend\r\n\tif true then return true end\r\n;\r\nreturn ((smalltalk:booleanNot_(((self):match_(\"[^a-z]\")))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isLowercase",0
,(
function(self)
return ((self):isLowerCase())
;
end
)
,"testing","isLowercase\r\n\t\" Return true if the receiver is one character lowercase. \"\r\n\r\n\t^self isLowerCase","(\r\nfunction(self)\r\nreturn ((self):isLowerCase())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asUppercase",0
,(
function(self)
return (self:upper())
;
end
)
,"converting","asUppercase\r\n\t\" Return a copy of the receiver converted to uppercase. \"\r\n\r\n\t^{'self.toUpperCase()'\r\n\tlua'self:upper()'}","(\r\nfunction(self)\r\nreturn (self:upper())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asUpperCase",0
,(
function(self)
return ((self):asUppercase())
;
end
)
,"converting","asUpperCase\r\n\t\" Return a copy of the receiver converted to uppercase. \"\r\n\r\n\t^self asUppercase","(\r\nfunction(self)\r\nreturn ((self):asUppercase())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asLowercase",0
,(
function(self)
return (self:lower())
;
end
)
,"converting","asLowercase\r\n\t\" Return a copy of the receiver converted to lowercase. \"\r\n\r\n\t^{'self.toLowerCase()'\r\n\tlua'self:lower()'}","(\r\nfunction(self)\r\nreturn (self:lower())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asLowerCase",0
,(
function(self)
return ((self):asLowercase())
;
end
)
,"converting","asLowerCase\r\n\t\" Return a copy of the receiver converted to lowercase. \"\r\n\r\n\t^self asLowercase","(\r\nfunction(self)\r\nreturn ((self):asLowercase())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asSmalltalkName:",0
,(
function(self,upCase)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local up=nil
local result=nil
(function() if ((((smalltalk:boolean_and_(((smalltalk:booleanNot_((upCase)))),(function()
return (((((self):detect_ifNone_((function(c)
return (((c) >= "a"))
end), (function()
return (nil)
end))))==nil))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self):asLowercase()):asSmalltalkName_(upCase))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=(upCase)
	up=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):inject_into_("", (function(result,each)
return ((function() if (((("_:.&%$?"):includes_(each)))==true) then return ((function()
(function()
	local _s8_value=(true)
	up=_s8_value; return _s8_value
end)();return (result)
end))() else return ((function()
return (smalltalk:object_comma_((result),(function() if (((up))==true) then return ((function()
(function()
	local _s8_value=(false)
	up=_s8_value; return _s8_value
end)();return ((each):asUppercase())
end))() else return ((function()
return (each)
end))() end end)()))
end))() end end)())
end)))
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((result):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("_")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if (((upCase))==true) then return ((function()
return ((function() if (((((result):first()):isLowercase()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((((result):first()):asUppercase()),(result):copyFrom_to_((2), (result):size())))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() else return ((function()
return ((function() if (((((result):first()):isUppercase()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((((result):first()):asLowercase()),(result):copyFrom_to_((2), (result):size())))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","asSmalltalkName: upCase\r\n\t\" Return a copy of the receiver converted to a smalltalk global name. \"\r\n\r\n\t| up result |\r\n\t(upCase not and: [\r\n\t\t(self detect: [:c| c >= $a ] ifNone: []) isNil\r\n\t]) ifTrue: [\r\n\t\t^self asLowercase asSmalltalkName: upCase\r\n\t].\r\n\tup := upCase.\r\n\tresult := self inject: '' into: [:result :each|\r\n\t\t('_:.&%$?' includes: each) ifTrue: [ up := true. result ]\r\n\t\tifFalse: [ result , (up\r\n\t\t\tifTrue: [  up := false. each asUppercase ]\r\n\t\t\tifFalse: [each])\r\n\t\t]\r\n\t].\r\n\tresult isEmpty ifTrue: [ ^$_ ].\r\n\tupCase ifTrue: [\r\n\t\tresult first isLowercase ifTrue: [\r\n\t\t\t^result first asUppercase ,(result copyFrom: 2 to: result size)\r\n\t\t]\r\n\t] ifFalse: [\r\n\t\tresult first isUppercase ifTrue: [\r\n\t\t\t^result first asLowercase ,(result copyFrom: 2 to: result size)\r\n\t\t]\r\n\t].\r\n\t^result","(\r\nfunction(self,upCase)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal up=nil\r\nlocal result=nil\r\n(function() if ((((smalltalk:boolean_and_(((smalltalk:booleanNot_((upCase)))),(function()\r\nreturn (((((self):detect_ifNone_((function(c)\r\nreturn (((c) >= \"a\"))\r\nend), (function()\r\nreturn (nil)\r\nend))))==nil))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self):asLowercase()):asSmalltalkName_(upCase))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(upCase)\r\n\tup=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):inject_into_(\"\", (function(result,each)\r\nreturn ((function() if ((((\"_:.&%$?\"):includes_(each)))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=(true)\r\n\tup=_s8_value; return _s8_value\r\nend)();return (result)\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((result),(function() if (((up))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=(false)\r\n\tup=_s8_value; return _s8_value\r\nend)();return ((each):asUppercase())\r\nend))() else return ((function()\r\nreturn (each)\r\nend))() end end)()))\r\nend))() end end)())\r\nend)))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((result):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"_\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if (((upCase))==true) then return ((function()\r\nreturn ((function() if (((((result):first()):isLowercase()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((((result):first()):asUppercase()),(result):copyFrom_to_((2), (result):size())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return ((function()\r\nreturn ((function() if (((((result):first()):isUppercase()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((((result):first()):asLowercase()),(result):copyFrom_to_((2), (result):size())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asGlobalName",0
,(
function(self)
return ((self):asSmalltalkName_(true))
;
end
)
,"converting","asGlobalName\r\n\t\" Return a copy of the receiver converted to a smalltalk global name. \"\r\n\r\n\t^self asSmalltalkName: true","(\r\nfunction(self)\r\nreturn ((self):asSmalltalkName_(true))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asLocalName",0
,(
function(self)
return ((self):asSmalltalkName_(false))
;
end
)
,"converting","asLocalName\r\n\t\" Return a copy of the receiver converted to a smalltalk local name. \"\r\n\r\n\t^self asSmalltalkName: false","(\r\nfunction(self)\r\nreturn ((self):asSmalltalkName_(false))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"digitValue",0
,(
function(self)
return (((("0123456789ABCDEF"):indexOf_(self)):_minus((1))):max_(((0)):_minus((1))))
;
end
)
,"converting","digitValue\r\n\t\" Return the digit value of the receiver (or -1). \"\r\n\r\n\t^('0123456789ABCDEF' indexOf: self) - 1 max: 0 - 1","(\r\nfunction(self)\r\nreturn ((((\"0123456789ABCDEF\"):indexOf_(self)):_minus((1))):max_(((0)):_minus((1))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asNumber",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):includes_("r")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Number):readFrom_((self):readStream()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (tonumber(self))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","asNumber\r\n\t\" Return the receiver as a Number. \"\r\n\r\n\t(self includes: $r) ifTrue: [\r\n\t\t^Number readFrom: self readStream\r\n\t].\r\n\t^{'Number(self)'\r\n\tlua'tonumber(self)'}","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):includes_(\"r\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Number):readFrom_((self):readStream()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (tonumber(self))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"errorReadOnly",0
,(
function(self)
return ((self):error_(smalltalk:object_comma_(("Strings are inmutable. Attempt to change "),smalltalk:printString_((self)))))
;
end
)
,"error","errorReadOnly\r\n\t\" Private - The receiver is read-only instance. \"\r\n\r\n\t^self error: 'Strings are inmutable. Attempt to change ',self printString","(\r\nfunction(self)\r\nreturn ((self):error_(smalltalk:object_comma_((\"Strings are inmutable. Attempt to change \"),smalltalk:printString_((self)))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"printString",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_(("'"),self)),"'"))
;
end
)
,"printing","printString\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^'''', self, ''''","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"'\"),self)),\"'\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isString",0
,(
function(self)
return (true)
;
end
)
,"testing","isString\r\n\t\" Return true if the receiver is a String. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isBackSlash",0
,(
function(self)
return ((smalltalk:isObject_equalTo_(((self):asciiValue()),(92))))
;
end
)
,"testing","isBackSlash\r\n\t\" Return true if the receiver is a back-slash string. \"\r\n\r\n\t^self asciiValue = 92 ","(\r\nfunction(self)\r\nreturn ((smalltalk:isObject_equalTo_(((self):asciiValue()),(92))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,">","_gt"
,(
function(self,aString)
return (self > aString)
;
end
)
,"comparing","> aString\r\n\t\" Return true if the receiver is greater than argument. \"\r\n\r\n\t^{'String(self) > aString'\r\n\tlua'self > aString'}","(\r\nfunction(self,aString)\r\nreturn (self > aString)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"<","_lt"
,(
function(self,aString)
return (self < aString)
;
end
)
,"comparing","< aString\r\n\t\" Return true if the receiver is smaller than argument. \"\r\n\r\n\t^{'String(self) < aString'\r\n\tlua'self < aString'}","(\r\nfunction(self,aString)\r\nreturn (self < aString)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,">=","_gt_eq"
,(
function(self,aString)
return (self >= aString)
;
end
)
,"comparing",">= aString\r\n\t\" Return true if the receiver is greater or equal to argument. \"\r\n\r\n\t^{'String(self) >= aString'\r\n\tlua'self >= aString'}","(\r\nfunction(self,aString)\r\nreturn (self >= aString)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"<=","_lt_eq"
,(
function(self,aString)
return (self <= aString)
;
end
)
,"comparing","<= aString\r\n\t\" Return true if the receiver is smaller or equal to argument. \"\r\n\r\n\t^{'String(self) <= aString'\r\n\tlua'self <= aString'}","(\r\nfunction(self,aString)\r\nreturn (self <= aString)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"remove:ifAbsent:",0
,(
function(self,anObject,aBlock)
return ((self):errorReadOnly())
;
end
)
,"removing","remove: anObject ifAbsent: aBlock\r\n\t\" Remove anObject from the receiver. \"\r\n\r\n\t^self errorReadOnly","(\r\nfunction(self,anObject,aBlock)\r\nreturn ((self):errorReadOnly())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asJSONObject",0
,(
function(self)
return (smalltalk:object_comma_((self),""))
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self , ''\r\n\"Note:\twe need to normalize result to do not destroy typeof checkings\r\n\tDo NOT modify this method if you do not know what you are doing.\r\n\tIf you remove the ,'' computation, the returned object will return 'object' and not 'string' as result of typeof operation.\r\n\"","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((self),\"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isCharacter",0
,(
function(self)
return (#self == 1)
;
end
)
,"testing","isCharacter\r\n\t\" Implemented to support Character protocoll for one-character Strings. \"\r\n\r\n\t^{'self.length == 1'\r\n\tlua'#self == 1'}","(\r\nfunction(self)\r\nreturn (#self == 1)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asciiValue",0
,(
function(self)
return (string.byte(self))
;
end
)
,"testing","asciiValue\r\n\t\" Return the number corresponding to the ASCII encoding of the first character of the receiver. \"\r\n\r\n\t^{'self.charCodeAt(0)'\r\n\tlua'string.byte(self)'}","(\r\nfunction(self)\r\nreturn (string.byte(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"isSeparator",0
,(
function(self)
return (#self==1 and self:find("%s")==1)
;
end
)
,"testing","isSeparator\r\n\t\" Return true if the receiver is either a space, tab, new-line, form-feed character, or double-byte space.\r\n\tNote that new-line has been extended to support NEL character.\r\n\tSee https://en.wikipedia.org/wiki/Newline\r\n\t\"\r\n\r\n\t^{'(new RegExp(\"\\\\s|\\\\u0085\")).test(self)'\r\n\tlua'#self==1 and self:find(\"%s\")==1'}","(\r\nfunction(self)\r\nreturn (#self==1 and self:find(\"%s\")==1)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asAsciiZ",0
,(
function(self)
return (smalltalk:object_comma_((self),((self)["$klass"]):zero()))
;
end
)
,"converting","asAsciiZ\r\n\t\" Return the receiver with ascii zero appended. \"\r\n\r\n\t^self , self class zero","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((self),((self)[\"$klass\"]):zero()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"trimFirst:last:",0
,(
function(self,firstBlock,lastBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local start=nil
local stop=nil
(function()
	local _s8_value=((1))
	start=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):size())
	stop=_s8_value; return _s8_value
end)()
;
(function() while(((function()
(function() if (((((start) > stop)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.String):new())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((firstBlock):value_((self):at_(start)))
end))()) do (function()
return ((function()
	local _s8_value=((start):_plus((1)))
	start=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
(function() while(((function()
(function() if (((((stop) < start)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.String):new())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((lastBlock):value_((self):at_(stop)))
end))()) do (function()
return ((function()
	local _s8_value=((stop):_minus((1)))
	stop=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):copyFrom_to_(start, stop))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"selecting","trimFirst: firstBlock last: lastBlock\r\n\t\" Return a String containing the result of trimming characters matching blocks from both sides of the receiver. \"\r\n\r\n\t| start stop |\r\n\tstart := 1. stop := self size.\r\n\t[\tstart > stop ifTrue: [ ^String new ].\r\n\t\tfirstBlock value: (self at: start)\r\n\t] whileTrue: [ start := start + 1 ].\r\n\t[\tstop < start ifTrue: [ ^String new ].\r\n\t\tlastBlock value: (self at: stop)\r\n\t] whileTrue: [ stop := stop - 1 ].\r\n\t^self copyFrom: start to: stop","(\r\nfunction(self,firstBlock,lastBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal start=nil\r\nlocal stop=nil\r\n(function()\r\n\tlocal _s8_value=((1))\r\n\tstart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):size())\r\n\tstop=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\n(function() if (((((start) > stop)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.String):new())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((firstBlock):value_((self):at_(start)))\r\nend))()) do (function()\r\nreturn ((function()\r\n\tlocal _s8_value=((start):_plus((1)))\r\n\tstart=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\n(function() while(((function()\r\n(function() if (((((stop) < start)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.String):new())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((lastBlock):value_((self):at_(stop)))\r\nend))()) do (function()\r\nreturn ((function()\r\n\tlocal _s8_value=((stop):_minus((1)))\r\n\tstop=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):copyFrom_to_(start, stop))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"trim:",0
,(
function(self,aBlock)
return ((self):trimFirst_last_(aBlock, aBlock))
;
end
)
,"selecting","trim: aBlock\r\n\t\" Return a String containing the result of trimming characters matching aBlock from both sides of the receiver. \"\r\n\r\n\t^self trimFirst: aBlock last: aBlock","(\r\nfunction(self,aBlock)\r\nreturn ((self):trimFirst_last_(aBlock, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"trimFirst:",0
,(
function(self,aBlock)
return ((self):trimFirst_last_(aBlock, (function(c)
return (false)
end)))
;
end
)
,"selecting","trimFirst: aBlock\r\n\t\" Return a String containing the result of trimming characters matching aBlock from start of the receiver. \"\r\n\r\n\t^self trimFirst: aBlock last: [:c| false ]","(\r\nfunction(self,aBlock)\r\nreturn ((self):trimFirst_last_(aBlock, (function(c)\r\nreturn (false)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"trimLast:",0
,(
function(self,aBlock)
return ((self):trimFirst_last_((function(c)
return (false)
end), aBlock))
;
end
)
,"selecting","trimLast: aBlock\r\n\t\" Return a String containing the result of trimming characters matching aBlock from end of the receiver. \"\r\n\r\n\t^self trimFirst: [:c| false ] last: aBlock","(\r\nfunction(self,aBlock)\r\nreturn ((self):trimFirst_last_((function(c)\r\nreturn (false)\r\nend), aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"trimBlanks",0
,(
function(self)
return ((self):trim_((function(c)
return ((c):isSeparator())
end)))
;
end
)
,"selecting","trimBlanks\r\n\t\" Return a String containing the receiver string with leading and trailing blanks removed. \"\r\n\r\n\t^self trim: [:c| c isSeparator ]","(\r\nfunction(self)\r\nreturn ((self):trim_((function(c)\r\nreturn ((c):isSeparator())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"writeStream",0
,(
function(self)
return ((smalltalk.StringWriteStream):on_(self))
;
end
)
,"streams","writeStream\r\n\t\" Return a write stream on the receiver. \"\r\n\r\n\t^StringWriteStream on: self","(\r\nfunction(self)\r\nreturn ((smalltalk.StringWriteStream):on_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"evaluateFor:",0
,(
function(self,anObject)
return ((anObject):perform_(self))
;
end
)
,"evaluating","evaluateFor: anObject\r\n\t\" Return the result of evaluating the receiver with anObject. \"\r\n\r\n\t^anObject perform: self","(\r\nfunction(self,anObject)\r\nreturn ((anObject):perform_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asParser",0
,(
function(self)
return ((function() if ((((self):isCharacter()))==true) then return ((function()
return ((smalltalk.PPCharacterParser):in_(self))
end))() else return ((function()
return (((smalltalk.PPStringParser):new()):string_(self))
end))() end end)())
;
end
)
,"Compiler-converting","asParser\r\n\t\" Return a parser on the receiver. \"\r\n\r\n\t^self isCharacter\r\n\tifTrue:  [ PPCharacterParser in: self ]\r\n\tifFalse: [ PPStringParser new string: self ]","(\r\nfunction(self)\r\nreturn ((function() if ((((self):isCharacter()))==true) then return ((function()\r\nreturn ((smalltalk.PPCharacterParser):in_(self))\r\nend))() else return ((function()\r\nreturn (((smalltalk.PPStringParser):new()):string_(self))\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asChoiceParser",0
,(
function(self)
return ((smalltalk.PPChoiceParser):withAll_(((self):asArray()):collect_((function(each)
return ((each):asParser())
end))))
;
end
)
,"Compiler-converting","asChoiceParser\r\n\t\" Return a parser of choices for each character the receiver. \"\r\n\r\n\t^PPChoiceParser withAll: (self asArray collect: [:each | each asParser ])","(\r\nfunction(self)\r\nreturn ((smalltalk.PPChoiceParser):withAll_(((self):asArray()):collect_((function(each)\r\nreturn ((each):asParser())\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asCharacterParser",0
,(
function(self)
return (((smalltalk.PPCharacterParser):new()):string_(self))
;
end
)
,"Compiler-converting","asCharacterParser\r\n\t\" Return a parser on the characters of the receiver. \"\r\n\r\n\t^PPCharacterParser new string: self","(\r\nfunction(self)\r\nreturn (((smalltalk.PPCharacterParser):new()):string_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"showIt",0
,(
function(self)
return ((self):print_((self):logEvaluate()))
;
end
)
,"Tools-evaluating","showIt\r\n\t\" Show/Print the result of evaluating the receiver. \"\r\n\r\n\t^self print: self logEvaluate","(\r\nfunction(self)\r\nreturn ((self):print_((self):logEvaluate()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"logEvaluate",0
,(
function(self)
return (((function()
return ((self):evaluate())
end)):logEvaluate_(self))
;
end
)
,"Tools-evaluating","logEvaluate\r\n\t\" Return the result of evaluating the receiver.\r\n\tThe source of evaluation will be logged.\r\n\t\"\r\n\r\n\t^[ self evaluate ] logEvaluate: self ","(\r\nfunction(self)\r\nreturn (((function()\r\nreturn ((self):evaluate())\r\nend)):logEvaluate_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"evaluate",0
,(
function(self)
return ((((self):smalltalk()):compilerSupport()):evaluate_(self))
;
end
)
,"Tools-evaluating","evaluate\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t^self smalltalk compilerSupport evaluate: self ","(\r\nfunction(self)\r\nreturn ((((self):smalltalk()):compilerSupport()):evaluate_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"evaluateIn:",0
,(
function(self,aReceiver)
local target=nil
(function()
	local _s8_value=((function() if ((((smalltalk:boolean_and_(((self):isObject_(aReceiver)),(function()
return ((((aReceiver))~=nil))
end)))))==true) then return ((function()
return (aReceiver)
end))() else return nil end end)())
	target=_s8_value; return _s8_value
end)()
;
return ((((target):smalltalk()):compilerSupport()):evaluate_to_(self, target))
;
end
)
,"Tools-evaluating","evaluateIn: aReceiver\r\n\t\" Return the result of evaluating the receiver in the context of aReceiver. \"\r\n\r\n\t| target |\r\n\ttarget := ((self isObject: aReceiver) and: [aReceiver notNil]) ifTrue: [ aReceiver ].\r\n\t^target smalltalk compilerSupport evaluate: self to: target","(\r\nfunction(self,aReceiver)\r\nlocal target=nil\r\n(function()\r\n\tlocal _s8_value=((function() if ((((smalltalk:boolean_and_(((self):isObject_(aReceiver)),(function()\r\nreturn ((((aReceiver))~=nil))\r\nend)))))==true) then return ((function()\r\nreturn (aReceiver)\r\nend))() else return nil end end)())\r\n\ttarget=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((((target):smalltalk()):compilerSupport()):evaluate_to_(self, target))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"doIt",0
,(
function(self)
return ((((self):smalltalk()):compilerSupport()):doIt_(self))
;
end
)
,"Tools-evaluating","doIt\r\n\t\" Return the result of evaluating the receiver or the error occurred during compilation or evaluation. \"\r\n\r\n\t^self smalltalk compilerSupport doIt: self","(\r\nfunction(self)\r\nreturn ((((self):smalltalk()):compilerSupport()):doIt_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"fileIn",0
,(
function(self)
return (((self):stream()):fileIn())
;
end
)
,"Tools-fileIn","fileIn\r\n\t\" FileIn the contents of the receiver. \"\r\n\r\n\t^self stream fileIn","(\r\nfunction(self)\r\nreturn (((self):stream()):fileIn())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"fileOut",0
,(
function(self)
return ((smalltalk.Exporter):fileOut_(self))
;
end
)
,"Tools-fileOut","fileOut\r\n\t\" FileOut the classes/categories in the receiver. \"\r\n\r\n\t^Exporter fileOut: self","(\r\nfunction(self)\r\nreturn ((smalltalk.Exporter):fileOut_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"export",0
,(
function(self)
return ((smalltalk.Exporter):export_(self))
;
end
)
,"Tools-fileOut","export\r\n\t\" Export the classes/categories in the receiver. \"\r\n\r\n\t^Exporter export: self","(\r\nfunction(self)\r\nreturn ((smalltalk.Exporter):export_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asComment",0
,(
function(self)
local prologue=nil
local epilogue=nil
(function() if ((((smalltalk:boolean_or_(((self):includes_((smalltalk.String):cr())),(function()
return ((self):includes_((smalltalk.String):lf()))
end)))))==true) then return ((function()
(function()
	local _s8_value=("--[[ ")
	prologue=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=(" --]]")
	epilogue=_s8_value; return _s8_value
end)())
end))() else return ((function()
(function()
	local _s8_value=("-- ")
	prologue=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=("")
	epilogue=_s8_value; return _s8_value
end)())
end))() end end)()
;
return (smalltalk:object_comma_((smalltalk:object_comma_((prologue),self)),epilogue))
;
end
)
,"Tools-converting","asComment\r\n\t\" Return the receiver as a source code comment. \"\r\n\r\n\t| prologue epilogue |\r\n\t((self includes: String cr) or: [self includes: String lf]) ifTrue: [\r\n\t\tprologue := {'\"/* \"'\tlua'\"--[[ \"'}.\r\n\t\tepilogue := {'\"*/ \"'\tlua'\" --]]\"'}.\r\n\t] ifFalse: [\r\n\t\tprologue := {'\"// \"'\tlua'\"-- \"'}.\r\n\t\tepilogue := ''\r\n\t].\r\n\t^prologue ,self ,epilogue","(\r\nfunction(self)\r\nlocal prologue=nil\r\nlocal epilogue=nil\r\n(function() if ((((smalltalk:boolean_or_(((self):includes_((smalltalk.String):cr())),(function()\r\nreturn ((self):includes_((smalltalk.String):lf()))\r\nend)))))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=(\"--[[ \")\r\n\tprologue=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=(\" --]]\")\r\n\tepilogue=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\n(function()\r\n\tlocal _s8_value=(\"-- \")\r\n\tprologue=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=(\"\")\r\n\tepilogue=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)()\r\n;\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((prologue),self)),epilogue))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"senders",0
,(
function(self)
return ((smalltalk.Smalltalk):sendersOf_(self))
;
end
)
,"Tools-query","senders\r\n\t\" Return the senders of the receiver. \"\r\n\r\n\t^Smalltalk sendersOf: self","(\r\nfunction(self)\r\nreturn ((smalltalk.Smalltalk):sendersOf_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"implementors",0
,(
function(self)
return ((smalltalk.Smalltalk):implementorsOf_(self))
;
end
)
,"Tools-query","implementors\r\n\t\" Return the implementors of the receiver. \"\r\n\r\n\t^Smalltalk implementorsOf: self","(\r\nfunction(self)\r\nreturn ((smalltalk.Smalltalk):implementorsOf_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"emmitFiles",0
,(
function(self)
local folder=nil
local fileNames=nil
local stream=nil
local line=nil
(function()
	local _s8_value=(self)
	folder=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(smalltalk:newArray({}))
	fileNames=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((folder),"emmit.txt")):fileContents()):stream())
	stream=_s8_value; return _s8_value
end)()
;
(function() while(not ((function()
return ((stream):atEnd())
end))()) do (function()
(function()
	local _s8_value=((((stream):nextLine()):upTo_(";")):trimBlanks())
	line=_s8_value; return _s8_value
end)();return ((function() if ((((line):notEmpty()))==true) then return ((function()
return ((function() if ((((smalltalk:isObject_equalTo_(((line):first()),"@"))))==true) then return ((function()
(function()
	local _s8_value=(((line):copyFrom_to_((2), (line):size())):trimBlanks())
	folder=_s8_value; return _s8_value
end)();return ((function() if ((((folder):isEmpty()))==true) then return ((function()
return ((function()
	local _s8_value=(self)
	folder=_s8_value; return _s8_value
end)())
end))() else return nil end end)())
end))() else return ((function()
return ((fileNames):add_(smalltalk:object_comma_((folder),line)))
end))() end end)())
end))() else return nil end end)())
end)() end;return nil end)()
;
(fileNames):emmitAll()
;
return (self)
;
end
)
,"Builders-emmiting","emmitFiles\r\n\t\" Emmit all the files listed in emmit.txt \"\r\n\r\n\t| folder fileNames stream line |\r\n\tfolder := self.\r\n\tfileNames := #().\r\n\tstream := (folder ,'emmit.txt') fileContents stream.\r\n\t[ stream atEnd ] whileFalse: [\r\n\t\tline := (stream nextLine upTo: $;) trimBlanks.\r\n\t\tline notEmpty ifTrue: [\r\n\t\t\tline first = $@ ifTrue: [\r\n\t\t\t\tfolder := (line copyFrom: 2 to: line size) trimBlanks.\r\n\t\t\t\tfolder isEmpty ifTrue: [ folder := self ]\r\n\t\t\t] ifFalse: [ fileNames add: folder ,line ]\r\n\t\t]\r\n\t].\r\n\tfileNames emmitAll","(\r\nfunction(self)\r\nlocal folder=nil\r\nlocal fileNames=nil\r\nlocal stream=nil\r\nlocal line=nil\r\n(function()\r\n\tlocal _s8_value=(self)\r\n\tfolder=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tfileNames=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((folder),\"emmit.txt\")):fileContents()):stream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(not ((function()\r\nreturn ((stream):atEnd())\r\nend))()) do (function()\r\n(function()\r\n\tlocal _s8_value=((((stream):nextLine()):upTo_(\";\")):trimBlanks())\r\n\tline=_s8_value; return _s8_value\r\nend)();return ((function() if ((((line):notEmpty()))==true) then return ((function()\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_(((line):first()),\"@\"))))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=(((line):copyFrom_to_((2), (line):size())):trimBlanks())\r\n\tfolder=_s8_value; return _s8_value\r\nend)();return ((function() if ((((folder):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(self)\r\n\tfolder=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return ((function()\r\nreturn ((fileNames):add_(smalltalk:object_comma_((folder),line)))\r\nend))() end end)())\r\nend))() else return nil end end)())\r\nend)() end;return nil end)()\r\n;\r\n(fileNames):emmitAll()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"outputToFile:",0
,(
function(self,fileName)
return (fileWrite(fileName, self, "wb"))
;
end
)
,"Snapshot-io","outputToFile: fileName\r\n\t\" Save the receiver's contents onto a file at fileName. \"\r\n\r\n\t^# #fileWrite: fileName string: self mode: #wb","(\r\nfunction(self,fileName)\r\nreturn (fileWrite(fileName, self, \"wb\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"appendToFile:",0
,(
function(self,fileName)
return (fileWrite(fileName, self, "ab"))
;
end
)
,"Snapshot-io","appendToFile: fileName\r\n\t\" Append the receiver's contents onto a file at fileName. \"\r\n\r\n\t^# #fileWrite: fileName string: self mode: #ab","(\r\nfunction(self,fileName)\r\nreturn (fileWrite(fileName, self, \"ab\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asLuaName",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:isObject_equalTo_(((self):first()),"_"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if (((((smalltalk.String):reservedLuaWords()):includes_(self)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_(("__"),self))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Lua-lua","asLuaName\r\n\t\" Private - Return the receiver as a valid Lua name. \"\r\n\r\n\tself first = $_ ifTrue: [ ^self ].\r\n\t(String reservedLuaWords includes: self) ifTrue: [ ^'__',self ].\r\n\t^self","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):first()),\"_\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if (((((smalltalk.String):reservedLuaWords()):includes_(self)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((\"__\"),self))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asLuaSelector",0
,(
function(self)
	if (self:startsWith_("#")) then return self:upTo_startingAt_(":",2) end
	local map = {
		[":"]="_",
		["+"]="_plus",
		["-"]="_minus",
		["*"]="_star",
		["/"]="_slash",
		["|"]="_bar",
		["&"]="_amp",
		[">"]="_gt",
		["<"]="_lt",
		["="]="_eq",
		[","]="_comma",
		["@"]="_at",
		["~"]="_no",
		["\\"]="_bk"
	};
	local result = self:gsub(".",function(c) return map[c] end)
	if true then return result:asLuaName() end
;
return (self)
;
end
)
,"Lua-lua","asLuaSelector\r\n\t\" Private - Return the receiver as a valid native selector. \"\r\n\r\n{'\tif (self[0]==\"#\") return self.upTo_startingAt_(\":\",2);\r\n\treturn self\r\n\t\t.replace(/:/g,\"_\")\r\n\t\t.replace(/[+]/g,\"_plus\")\r\n\t\t.replace(/-/g,\"_minus\")\r\n\t\t.replace(/[*]/g,\"_star\")\r\n\t\t.replace(/[/]/g,\"_slash\")\r\n\t\t.replace(/[|]/g,\"_bar\")\r\n\t\t.replace(/[&]/g,\"_amp\")\r\n\t\t.replace(/>/g,\"_gt\")\r\n\t\t.replace(/</g,\"_lt\")\r\n\t\t.replace(/=/g,\"_eq\")\r\n\t\t.replace(/,/g,\"_comma\")\r\n\t\t.replace(/[@]/g,\"_at\")\r\n\t\t.replace(/~/g,\"_no\")\r\n\t\t.replace(/\\\\/g,\"_bk\")\r\n\t\t.asLuaName()'\r\nlua'\tif (self:startsWith_(\"#\")) then return self:upTo_startingAt_(\":\",2) end\r\n\tlocal map = {\r\n\t\t[\":\"]=\"_\",\r\n\t\t[\"+\"]=\"_plus\",\r\n\t\t[\"-\"]=\"_minus\",\r\n\t\t[\"*\"]=\"_star\",\r\n\t\t[\"/\"]=\"_slash\",\r\n\t\t[\"|\"]=\"_bar\",\r\n\t\t[\"&\"]=\"_amp\",\r\n\t\t[\">\"]=\"_gt\",\r\n\t\t[\"<\"]=\"_lt\",\r\n\t\t[\"=\"]=\"_eq\",\r\n\t\t[\",\"]=\"_comma\",\r\n\t\t[\"@\"]=\"_at\",\r\n\t\t[\"~\"]=\"_no\",\r\n\t\t[\"\\\\\"]=\"_bk\"\r\n\t};\r\n\tlocal result = self:gsub(\".\",function(c) return map[c] end)\r\n\tif true then return result:asLuaName() end'}","(\r\nfunction(self)\r\n\tif (self:startsWith_(\"#\")) then return self:upTo_startingAt_(\":\",2) end\r\n\tlocal map = {\r\n\t\t[\":\"]=\"_\",\r\n\t\t[\"+\"]=\"_plus\",\r\n\t\t[\"-\"]=\"_minus\",\r\n\t\t[\"*\"]=\"_star\",\r\n\t\t[\"/\"]=\"_slash\",\r\n\t\t[\"|\"]=\"_bar\",\r\n\t\t[\"&\"]=\"_amp\",\r\n\t\t[\">\"]=\"_gt\",\r\n\t\t[\"<\"]=\"_lt\",\r\n\t\t[\"=\"]=\"_eq\",\r\n\t\t[\",\"]=\"_comma\",\r\n\t\t[\"@\"]=\"_at\",\r\n\t\t[\"~\"]=\"_no\",\r\n\t\t[\"\\\\\"]=\"_bk\"\r\n\t};\r\n\tlocal result = self:gsub(\".\",function(c) return map[c] end)\r\n\tif true then return result:asLuaName() end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"asLua",0
,(
function(self)
local encoded=nil
local bar=nil
(function()
	local _s8_value=(smalltalk:asCharacter_(((92))))
	bar=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(self)
	encoded=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((encoded):replace_with_(bar, smalltalk:object_comma_((bar),bar)))
	encoded=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((encoded):replace_with_((smalltalk.String):lf(), smalltalk:object_comma_((bar),"n")))
	encoded=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((encoded):replace_with_((smalltalk.String):cr(), smalltalk:object_comma_((bar),"r")))
	encoded=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((encoded):replace_with_((smalltalk.String):tab(), smalltalk:object_comma_((bar),"t")))
	encoded=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((encoded):replace_with_("\"", smalltalk:object_comma_((bar),"\"")))
	encoded=_s8_value; return _s8_value
end)()
;
return (smalltalk:object_comma_((smalltalk:object_comma_(("\""),encoded)),"\""))
;
end
)
,"Lua-lua","asLua\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\t| encoded bar |\r\n\tbar := 92 asCharacter.\r\n\tencoded := self.\r\n\tencoded := encoded replace: bar with: bar ,bar.\r\n\tencoded := encoded replace: String lf with: bar ,$n.\r\n\tencoded := encoded replace: String cr with: bar ,$r.\r\n\tencoded := encoded replace: String tab with: bar ,$t.\r\n\tencoded := encoded replace: $\" with: bar ,$\".\r\n\t^$\" ,encoded ,$\"","(\r\nfunction(self)\r\nlocal encoded=nil\r\nlocal bar=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:asCharacter_(((92))))\r\n\tbar=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(self)\r\n\tencoded=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((encoded):replace_with_(bar, smalltalk:object_comma_((bar),bar)))\r\n\tencoded=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((encoded):replace_with_((smalltalk.String):lf(), smalltalk:object_comma_((bar),\"n\")))\r\n\tencoded=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((encoded):replace_with_((smalltalk.String):cr(), smalltalk:object_comma_((bar),\"r\")))\r\n\tencoded=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((encoded):replace_with_((smalltalk.String):tab(), smalltalk:object_comma_((bar),\"t\")))\r\n\tencoded=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((encoded):replace_with_(\"\\\"\", smalltalk:object_comma_((bar),\"\\\"\")))\r\n\tencoded=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"\\\"\"),encoded)),\"\\\"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.String,"fileContents",0
,(
function(self)
return (read(self))
;
end
)
,"io","fileContents\r\n\t\" Returns the contents of file at (fileName given by) the receiver.\r\n\tReturn nil if the file do not exist or contents cant be read.\r\n\t\"\r\n\r\n\t^# #read: self","(\r\nfunction(self)\r\nreturn (read(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array["$klass"],"new",0
,(
function(self)
return (smalltalk:newArray({}))
;
end
)
,"instantiation","new\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^#()","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array["$klass"],"basicNew:",0
,(
function(self,size)
return (smalltalk:newArray())
;
end
)
,"instantiation","basicNew: size\r\n\t\" Return an instance of the receiver of specified size. \"\r\n\r\n\t^{'new Array(size)'\r\n\tlua'smalltalk:newArray()'}","(\r\nfunction(self,size)\r\nreturn (smalltalk:newArray())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array["$klass"],"new:",0
,(
function(self,size)
return (((self):basicNew_(size)):atAllPut_(nil))
;
end
)
,"instantiation","new: size\r\n\t\" Return an instance of the receiver of specified size. \"\r\n\r\n\t^(self basicNew: size) atAllPut: nil","(\r\nfunction(self,size)\r\nreturn (((self):basicNew_(size)):atAllPut_(nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array["$klass"],"with:",0
,(
function(self,anObject)
local unused
;
return ((
function(_s8_rec)
(_s8_rec):add_(anObject);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","with: anObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n{'\treturn [ anObject ]' lua'local unused'}.\r\n\t^self new\r\n\t\tadd: anObject;\r\n\t\tyourself","(\r\nfunction(self,anObject)\r\nlocal unused\r\n;\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(anObject);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array["$klass"],"with:with:",0
,(
function(self,firstObject,secondObject)
local unused
;
return ((
function(_s8_rec)
(_s8_rec):add_(firstObject);
(_s8_rec):add_(secondObject);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","with: firstObject with: secondObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n{'\treturn [ firstObject, secondObject ]' lua'local unused'}.\r\n\t^self new\r\n\t\tadd: firstObject;\r\n\t\tadd: secondObject;\r\n\t\tyourself","(\r\nfunction(self,firstObject,secondObject)\r\nlocal unused\r\n;\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(firstObject);\r\n(_s8_rec):add_(secondObject);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array["$klass"],"with:with:with:",0
,(
function(self,firstObject,secondObject,thirdObject)
local unused
;
return ((
function(_s8_rec)
(_s8_rec):add_(firstObject);
(_s8_rec):add_(secondObject);
(_s8_rec):add_(thirdObject);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","with: firstObject with: secondObject with: thirdObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n{'\treturn [ firstObject, secondObject, thirdObject ]' lua'local unused'}.\r\n\t^self new\r\n\t\tadd: firstObject;\r\n\t\tadd: secondObject;\r\n\t\tadd: thirdObject;\r\n\t\tyourself","(\r\nfunction(self,firstObject,secondObject,thirdObject)\r\nlocal unused\r\n;\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(firstObject);\r\n(_s8_rec):add_(secondObject);\r\n(_s8_rec):add_(thirdObject);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array["$klass"],"with:with:with:with:",0
,(
function(self,firstObject,secondObject,thirdObject,fourthObject)
local unused
;
return ((
function(_s8_rec)
(_s8_rec):add_(firstObject);
(_s8_rec):add_(secondObject);
(_s8_rec):add_(thirdObject);
(_s8_rec):add_(fourthObject);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","with: firstObject with: secondObject with: thirdObject with: fourthObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n{'\treturn [ firstObject, secondObject, thirdObject, fourthObject ]' lua'local unused'}.\r\n\t^self new\r\n\t\tadd: firstObject;\r\n\t\tadd: secondObject;\r\n\t\tadd: thirdObject;\r\n\t\tadd: fourthObject;\r\n\t\tyourself","(\r\nfunction(self,firstObject,secondObject,thirdObject,fourthObject)\r\nlocal unused\r\n;\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(firstObject);\r\n(_s8_rec):add_(secondObject);\r\n(_s8_rec):add_(thirdObject);\r\n(_s8_rec):add_(fourthObject);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array["$klass"],"withAll:",0
,(
function(self,aCollection)
	local result = aCollection:asArray()
	if (aCollection==result)
		then return result:copy()
		else return result
	end
;
return (self)
;
end
)
,"instantiation","withAll: aCollection\r\n\t\" Return an instance of the receiver. \"\r\n\r\n{'\treturn [].concat(aCollection.asArray())'\r\nlua'\tlocal result = aCollection:asArray()\r\n\tif (aCollection==result)\r\n\t\tthen return result:copy()\r\n\t\telse return result\r\n\tend'}","(\r\nfunction(self,aCollection)\r\n\tlocal result = aCollection:asArray()\r\n\tif (aCollection==result)\r\n\t\tthen return result:copy()\r\n\t\telse return result\r\n\tend\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array["$klass"],"slice:",0
,(
function(self,anIterable)
return (self:notImplementedYet_("slice:"))
;
end
)
,"util","slice: anIterable\r\n\t\" Return an Array instance from contents of anIterable. \"\r\n\r\n\t^{'Array.prototype.slice.call(anIterable)'\r\n\tlua'self:notImplementedYet_(\"slice:\")'}","(\r\nfunction(self,anIterable)\r\nreturn (self:notImplementedYet_(\"slice:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array["$klass"],"@","_at"
,(
function(self,native)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function()
	local _s8_value=((self):new())
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((((native))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
	for _,value in pairs(native) do result:add_(value) end

;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Lua-instantiation","@ native\r\n\t\" Returns an instance of the receiver containing the values in native object. \"\r\n\r\n\t| result |\r\n\tresult := self new.\r\n\tnative isNil ifTrue: [ ^result ].\r\n{'\tresult.addAll_(native)'\r\nlua'\tfor _,value in pairs(native) do result:add_(value) end\r\n'}.\t^result","(\r\nfunction(self,native)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((native))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n\tfor _,value in pairs(native) do result:add_(value) end\r\n\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"size",0
,(
function(self)
return (#self)
;
end
)
,"accessing","size\r\n\t\" Return the size of the receiver. \"\r\n\r\n\t^{'self.length'\r\n\tlua'#self'}","(\r\nfunction(self)\r\nreturn (#self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"at:",0
,(
function(self,anIndex)
	local value = self[anIndex]
	if (value==nil) then return self:objectNotFoundAt_(anIndex) end
	if (true) then return self:unwrap_(value) end
;
return (self)
;
end
)
,"accessing","at: anIndex\r\n\t\" Return the contents of the receiver at anIndex. \"\r\n\r\n{'\tvar value = self[anIndex - 1];\r\n\tif(value === undefined) { return self.objectNotFoundAt_(anIndex); }\r\n\treturn value'\r\nlua'\tlocal value = self[anIndex]\r\n\tif (value==nil) then return self:objectNotFoundAt_(anIndex) end\r\n\tif (true) then return self:unwrap_(value) end'}","(\r\nfunction(self,anIndex)\r\n\tlocal value = self[anIndex]\r\n\tif (value==nil) then return self:objectNotFoundAt_(anIndex) end\r\n\tif (true) then return self:unwrap_(value) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"at:ifAbsent:",0
,(
function(self,anIndex,aBlock)
	local value = self[anIndex]
	if (value==nil) then return aBlock:value() end
	if (true) then return self:unwrap_(value) end
;
return (self)
;
end
)
,"accessing","at: anIndex ifAbsent: aBlock\r\n\t\" Return the contents of the receiver at anIndex or the result of evaluating aBlock. \"\r\n\r\n{'\tvar value = self[anIndex - 1];\r\n\tif(value === undefined) { return aBlock(); }\r\n\treturn value'\r\nlua'\tlocal value = self[anIndex]\r\n\tif (value==nil) then return aBlock:value() end\r\n\tif (true) then return self:unwrap_(value) end'}","(\r\nfunction(self,anIndex,aBlock)\r\n\tlocal value = self[anIndex]\r\n\tif (value==nil) then return aBlock:value() end\r\n\tif (true) then return self:unwrap_(value) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"at:put:",0
,(
function(self,anIndex,anObject)
	self[anIndex]=self:wrap_(anObject)
	if true then return anObject end
;
return (self)
;
end
)
,"accessing","at: anIndex put: anObject\r\n\t\" Set the contents of the receiver at anIndex. \"\r\n\r\n{'\treturn self[anIndex - 1] = anObject'\r\nlua'\tself[anIndex]=self:wrap_(anObject)\r\n\tif true then return anObject end'}","(\r\nfunction(self,anIndex,anObject)\r\n\tself[anIndex]=self:wrap_(anObject)\r\n\tif true then return anObject end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"do:",0
,(
function(self,aBlock)
	for _,value in ipairs(self) do
		aBlock(self:unwrap_(value))
	end
;
return (self)
;
end
)
,"enumerating","do: aBlock\r\n\t\" Evaluate aBlock with the contents of the receiver. \"\r\n\r\n{'\tfor(var i=0;i<self.length;i++){aBlock(self[i]);}'\r\nlua'\tfor _,value in ipairs(self) do\r\n\t\taBlock(self:unwrap_(value))\r\n\tend'}","(\r\nfunction(self,aBlock)\r\n\tfor _,value in ipairs(self) do\r\n\t\taBlock(self:unwrap_(value))\r\n\tend\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"isArray",0
,(
function(self)
return ((((self)["$klass"])==smalltalk.Array))
;
end
)
,"testing","isArray\r\n\t\" Return true if the receiver is an Array. \"\r\n\r\n\t^self class == Array","(\r\nfunction(self)\r\nreturn ((((self)[\"$klass\"])==smalltalk.Array))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"addAll:",0
,(
function(self,aCollection)
	local array = aCollection:asArray()
	for i=1,#array do self[#self+1]=self:wrap_(array[i]) end

;
return (aCollection)
;
end
)
,"adding","addAll: aCollection\r\n\t\" Add objects in aCollection to the receiver. \"\r\n\r\n{'\tself.push.apply(self, aCollection.asArray())'\r\nlua'\tlocal array = aCollection:asArray()\r\n\tfor i=1,#array do self[#self+1]=self:wrap_(array[i]) end\r\n'}.\t^aCollection","(\r\nfunction(self,aCollection)\r\n\tlocal array = aCollection:asArray()\r\n\tfor i=1,#array do self[#self+1]=self:wrap_(array[i]) end\r\n\r\n;\r\nreturn (aCollection)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"shallowCopy",0
,(
function(self)
local result=nil
	result = self:class():new()
	for i=1,#self do result[#result+1]=self[i] end

;
return (result)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t| result |\r\n{'\tresult = self.slice(0)'\r\nlua'\tresult = self:class():new()\r\n\tfor i=1,#self do result[#result+1]=self[i] end\r\n'}.\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n\tresult = self:class():new()\r\n\tfor i=1,#self do result[#result+1]=self[i] end\r\n\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"deepCopy",0
,(
function(self)
return ((self):collect_((function(each)
return ((each):deepCopy())
end)))
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self collect: [:each | each deepCopy ]","(\r\nfunction(self)\r\nreturn ((self):collect_((function(each)\r\nreturn ((each):deepCopy())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"copyFrom:to:",0
,(
function(self,anIndex,anotherIndex)
local result=nil
(function()
	local _s8_value=(((self)["$klass"]):new())
	result=_s8_value; return _s8_value
end)()
;
(anIndex):to_do_(anotherIndex, (function(each)
return ((result):add_((self):at_(each)))
end))
;
return (result)
;
end
)
,"copying","copyFrom: anIndex to: anotherIndex\r\n\t\" Return the receiver contents in the interval. \"\r\n\r\n\t| result |\r\n\tresult := self class new.\r\n\tanIndex to: anotherIndex do: [:each | result add: (self at: each) ].\r\n\t^result","(\r\nfunction(self,anIndex,anotherIndex)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(anIndex):to_do_(anotherIndex, (function(each)\r\nreturn ((result):add_((self):at_(each)))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"copyWithAll:",0
,(
function(self,aCollection)
	local result = self:copy()
	result:addAll_(aCollection)
	if (true) then return result end

;
return (self)
;
end
)
,"copying","copyWithAll: aCollection\r\n\t\" Return a copy of the receiver with all contents of aCollection. \"\r\n\r\n{'\treturn self.concat(aCollection.asArray())'\r\nlua'\tlocal result = self:copy()\r\n\tresult:addAll_(aCollection)\r\n\tif (true) then return result end\r\n'}","(\r\nfunction(self,aCollection)\r\n\tlocal result = self:copy()\r\n\tresult:addAll_(aCollection)\r\n\tif (true) then return result end\r\n\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,",","_comma"
,(
function(self,aCollection)
	local result = self:copy()
	result:addAll_(aCollection)
	if (true) then return result end

;
return (self)
;
end
)
,"copying",", aCollection\r\n\t\" Return a copy of the receiver with all contents of aCollection. \"\r\n\r\n{'\treturn self.concat(aCollection.asArray())'\r\nlua'\tlocal result = self:copy()\r\n\tresult:addAll_(aCollection)\r\n\tif (true) then return result end\r\n'}","(\r\nfunction(self,aCollection)\r\n\tlocal result = self:copy()\r\n\tresult:addAll_(aCollection)\r\n\tif (true) then return result end\r\n\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"join:",0
,(
function(self,aString)
return (table.concat(self,aString))
;
end
)
,"composing","join: aString\r\n\t\" Return a string with the receiver's contents separated by aString. \"\r\n\r\n\t^{'self.join(aString)'\r\n\tlua'table.concat(self,aString)'}","(\r\nfunction(self,aString)\r\nreturn (table.concat(self,aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"compose:with:",0
,(
function(self,aBlock,aString)
return (((self):collect_(aBlock)):join_(aString))
;
end
)
,"composing","compose: aBlock with: aString\r\n\t\" Return a string composed with the result of evaluating aBlock into receiver's contents. \"\r\n\r\n\t^(self collect: aBlock) join: aString","(\r\nfunction(self,aBlock,aString)\r\nreturn (((self):collect_(aBlock)):join_(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"compose:",0
,(
function(self,aBlock)
return (((self):collect_(aBlock)):join_(""))
;
end
)
,"composing","compose: aBlock\r\n\t\" Return a string composed with the result of evaluating aBlock into receiver's contents. \"\r\n\r\n\t^(self collect: aBlock) join: ''","(\r\nfunction(self,aBlock)\r\nreturn (((self):collect_(aBlock)):join_(\"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"asNative:",0
,(
function(self,lang)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local list=nil
(function()
	local _s8_value=(((self):collect_((function(each)
return ((each):asNative_(lang))
end))):join_(", "))
	list=_s8_value; return _s8_value
end)()
;
(function() if (((((lang)=="lua")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk:newArray({"),list)),"})"))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("["),list)),"]"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","asNative: lang\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\t| list |\r\n\tlist := (self collect: [:each | each asNative: lang ]) join: ', '.\r\n\tlang == #lua ifTrue: [ ^'smalltalk:newArray({' ,list ,'})' ].\r\n\t^$[ ,list ,$]","(\r\nfunction(self,lang)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal list=nil\r\n(function()\r\n\tlocal _s8_value=(((self):collect_((function(each)\r\nreturn ((each):asNative_(lang))\r\nend))):join_(\", \"))\r\n\tlist=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((((lang)==\"lua\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk:newArray({\"),list)),\"})\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"[\"),list)),\"]\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"sort:",0
,(
function(self,aBlock)
	table.sort(self,aBlock)
;
return (self)
;
end
)
,"sorting","sort: aBlock\r\n\t\" Sort (inplace) the receiver's contents with aBlock. \"\r\n\r\n{'\tif (nil.isNil_(aBlock)) { self.sort(); return self }\r\n\tvar criteria = function(a, b) {\r\n\t\tif(aBlock(a,b)) {return -1} else {return 1}\r\n\t}\r\n\tself.sort(criteria)'\r\nlua'\ttable.sort(self,aBlock)'}","(\r\nfunction(self,aBlock)\r\n\ttable.sort(self,aBlock)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"asArray",0
,(
function(self)
return (self)
;
end
)
,"converting","asArray\r\n\t\" Return an array with the contents of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"remove:ifAbsent:",0
,(
function(self,anObject,aBlock)
	for i,value in ipairs(self) do
		if (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then return table.remove(self,i) end
	end
	if true then return aBlock:value() end
;
return (self)
;
end
)
,"removing","remove: anObject ifAbsent: aBlock\r\n\t\" Remove anObject from the receiver.\r\n\tReturn the result of evaluating aBlock if anObject is not in the receiver.\r\n\t\"\r\n\r\n{'\tfor(var i=0;i<self.length;i++) {\r\n\t\tif(self[i] == anObject) {\r\n\t\t\tself.splice(i,1); return anObject;\r\n\t\t}\r\n\t}; return aBlock.value()'\r\nlua'\tfor i,value in ipairs(self) do\r\n\t\tif (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then return table.remove(self,i) end\r\n\tend\r\n\tif true then return aBlock:value() end'}","(\r\nfunction(self,anObject,aBlock)\r\n\tfor i,value in ipairs(self) do\r\n\t\tif (smalltalk:isObject_equalTo_(self:unwrap_(value),anObject)) then return table.remove(self,i) end\r\n\tend\r\n\tif true then return aBlock:value() end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"add:",0
,(
function(self,anObject)
	self[#self+1]=self:wrap_(anObject)

;
return (anObject)
;
end
)
,"adding","add: anObject\r\n\t\" Add anObject to the receiver. \"\r\n\r\n{'\tself.push(anObject);'\r\nlua'\tself[#self+1]=self:wrap_(anObject)\r\n'}.\t^anObject","(\r\nfunction(self,anObject)\r\n\tself[#self+1]=self:wrap_(anObject)\r\n\r\n;\r\nreturn (anObject)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"addFirst:",0
,(
function(self,anObject)
	table.insert(self,1,self:wrap_(anObject))

;
return (anObject)
;
end
)
,"adding","addFirst: anObject\r\n\t\" Add anObject to the receiver. \"\r\n\r\n{'\tself.splice(0,0,anObject)'\r\nlua'\ttable.insert(self,1,self:wrap_(anObject))\r\n'}.\t^anObject","(\r\nfunction(self,anObject)\r\n\ttable.insert(self,1,self:wrap_(anObject))\r\n\r\n;\r\nreturn (anObject)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"addLast:",0
,(
function(self,anObject)
	self[#self+1]=self:wrap_(anObject)

;
return (anObject)
;
end
)
,"adding","addLast: anObject\r\n\t\" Add anObject to the receiver (at end). \"\r\n\r\n{'\tself.push(anObject)'\r\nlua'\tself[#self+1]=self:wrap_(anObject)\r\n'}.\t^anObject","(\r\nfunction(self,anObject)\r\n\tself[#self+1]=self:wrap_(anObject)\r\n\r\n;\r\nreturn (anObject)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"removeAll",0
,(
function(self)
	for i=#self,1,-1 do self[i]=nil end
;
return (self)
;
end
)
,"removing","removeAll\r\n\t\" Remove all the objects in the receiver. \"\r\n\r\n{'\tself.splice(0,self.size())'\r\nlua'\tfor i=#self,1,-1 do self[i]=nil end'}","(\r\nfunction(self)\r\n\tfor i=#self,1,-1 do self[i]=nil end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"removeFirst",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((self):first())
	result=_s8_value; return _s8_value
end)()
;
	table.remove(self,1)
;
return (result)
;
end
)
,"removing","removeFirst\r\n\t\" Remove the first object in the receiver. \"\r\n\r\n\t| result |\r\n\tresult := self first.\r\n{'\tself.splice(0,1)'\r\nlua'\ttable.remove(self,1)'}.\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):first())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n\ttable.remove(self,1)\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"removeIndex:",0
,(
function(self,index)
	table.remove(self,index)
;
return (self)
;
end
)
,"removing","removeIndex: index\r\n\t\" Remove the element at index. \"\r\n\r\n{'\tself.splice(index-1,1)'\r\nlua'\ttable.remove(self,index)'}","(\r\nfunction(self,index)\r\n\ttable.remove(self,index)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"removeLast",0
,(
function(self)
return (self:unwrap_(table.remove(self)))
;
end
)
,"removing","removeLast\r\n\t\" Remove the last object in the receiver. \"\r\n\r\n\t^{'self.pop()'\r\n\tlua'self:unwrap_(table.remove(self))'}","(\r\nfunction(self)\r\nreturn (self:unwrap_(table.remove(self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"sorted",0
,(
function(self)
return (((self):copy()):sort())
;
end
)
,"sorting","sorted\r\n\t\" Return a sorted copy of the receiver. \"\r\n\r\n\t^self copy sort","(\r\nfunction(self)\r\nreturn (((self):copy()):sort())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"sorted:",0
,(
function(self,aBlock)
return (((self):copy()):sort_(aBlock))
;
end
)
,"sorting","sorted: aBlock\r\n\t\" Return a sorted copy of the receiver. \"\r\n\r\n\t^self copy sort: aBlock","(\r\nfunction(self,aBlock)\r\nreturn (((self):copy()):sort_(aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"atAllPut:",0
,(
function(self,anObject)
	local what=self:wrap_(anObject)
	for i=1,#self do self[i]=what end
;
return (self)
;
end
)
,"accessing","atAllPut: anObject\r\n\t\" Replace all elements of the receiver with anObject. \"\r\n\r\n{'\tfor(var i=0;i<self.length;i++){ self[i] = anObject; }'\r\nlua'\tlocal what=self:wrap_(anObject)\r\n\tfor i=1,#self do self[i]=what end'}","(\r\nfunction(self,anObject)\r\n\tlocal what=self:wrap_(anObject)\r\n\tfor i=1,#self do self[i]=what end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"first",0
,(
function(self)
	local value=self[1]
	if (value~=nil) then return self:unwrap_(value) end

;
return ((self):at_((1)))
;
end
)
,"accessing","first\r\n\t\" Return the object in the receiver. \"\r\n\r\n{'\tvar result=self[0];\r\n\tif (typeof(result)!=\"undefined\") return result\r\n'lua'\tlocal value=self[1]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n'}.\t^self at: 1","(\r\nfunction(self)\r\n\tlocal value=self[1]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n\r\n;\r\nreturn ((self):at_((1)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"second",0
,(
function(self)
	local value=self[2]
	if (value~=nil) then return self:unwrap_(value) end

;
return ((self):at_((2)))
;
end
)
,"accessing","second\r\n\t\" Return the object in the receiver. \"\r\n\r\n{'\tvar result=self[1];\r\n\tif (typeof(result)!=\"undefined\") return result\r\n'lua'\tlocal value=self[2]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n'}.\t^self at: 2","(\r\nfunction(self)\r\n\tlocal value=self[2]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n\r\n;\r\nreturn ((self):at_((2)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"third",0
,(
function(self)
	local value=self[3]
	if (value~=nil) then return self:unwrap_(value) end

;
return ((self):at_((3)))
;
end
)
,"accessing","third\r\n\t\" Return the object in the receiver. \"\r\n\r\n{'\tvar result=self[2];\r\n\tif (typeof(result)!=\"undefined\") return result\r\n'lua'\tlocal value=self[3]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n'}.\t^self at: 3","(\r\nfunction(self)\r\n\tlocal value=self[3]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n\r\n;\r\nreturn ((self):at_((3)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"fourth",0
,(
function(self)
	local value=self[4]
	if (value~=nil) then return self:unwrap_(value) end

;
return ((self):at_((4)))
;
end
)
,"accessing","fourth\r\n\t\" Return the object in the receiver. \"\r\n\r\n{'\tvar result=self[3];\r\n\tif (typeof(result)!=\"undefined\") return result\r\n'lua'\tlocal value=self[4]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n'}.\t^self at: 4","(\r\nfunction(self)\r\n\tlocal value=self[4]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n\r\n;\r\nreturn ((self):at_((4)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"fifth",0
,(
function(self)
	local value=self[5]
	if (value~=nil) then return self:unwrap_(value) end

;
return ((self):at_((5)))
;
end
)
,"accessing","fifth\r\n\t\" Return the fifth object in the receiver. \"\r\n\r\n{'\tvar result=self[4];\r\n\tif (typeof(result)!=\"undefined\") return result\r\n'lua'\tlocal value=self[5]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n'}.\t^self at: 5","(\r\nfunction(self)\r\n\tlocal value=self[5]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n\r\n;\r\nreturn ((self):at_((5)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"sixth",0
,(
function(self)
	local value=self[6]
	if (value~=nil) then return self:unwrap_(value) end

;
return ((self):at_((6)))
;
end
)
,"accessing","sixth\r\n\t\" Return the sixth object in the receiver. \"\r\n\r\n{'\tvar result=self[5];\r\n\tif (typeof(result)!=\"undefined\") return result\r\n'lua'\tlocal value=self[6]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n'}.\t^self at: 6","(\r\nfunction(self)\r\n\tlocal value=self[6]\r\n\tif (value~=nil) then return self:unwrap_(value) end\r\n\r\n;\r\nreturn ((self):at_((6)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"withIndexDo:",0
,(
function(self,aBlock)
	for i=1,#self do aBlock(self:unwrap_(self[i]),i) end
;
return (self)
;
end
)
,"enumerating","withIndexDo: aBlock\r\n\t\" Evaluate aBlock with all the index positions in the receiver. \"\r\n\r\n{'\tfor(var i=0;i<self.length;i++){aBlock(self[i], i+1);}'\r\nlua'\tfor i=1,#self do aBlock(self:unwrap_(self[i]),i) end'}","(\r\nfunction(self,aBlock)\r\n\tfor i=1,#self do aBlock(self:unwrap_(self[i]),i) end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"printString",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function() if ((((self):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("#()")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=("#( ")
	result=_s8_value; return _s8_value
end)()
;
(self):do_((function(each)
(function() if ((((((result):size()) > (10000))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((result),"...)"))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((result),(function() if ((((self):isObject_(each)))==true) then return ((function()
return (smalltalk:printString_((each)))
end))() else return ((function()
return (each)
end))() end end)()))," "))
	result=_s8_value; return _s8_value
end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((result),")"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"printing","printString\r\n\t\" DEBUG - Temporary printing method (must check for recursion). \"\r\n\r\n\t| result |\r\n\tself isEmpty ifTrue: [ ^'#()' ].\r\n\tresult := '#( '.\r\n\tself do: [:each|\r\n\t\tresult size > 10000 ifTrue: [ ^result,'...)' ].\r\n\t\tresult := result , (\r\n\t\t\t(self isObject: each)\r\n\t\t\tifTrue: [ each printString ]\r\n\t\t\tifFalse: [ each ]) ,' '\r\n\t].\r\n\t^result,')'","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function() if ((((self):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"#()\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(\"#( \")\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):do_((function(each)\r\n(function() if ((((((result):size()) > (10000))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((result),\"...)\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((result),(function() if ((((self):isObject_(each)))==true) then return ((function()\r\nreturn (smalltalk:printString_((each)))\r\nend))() else return ((function()\r\nreturn (each)\r\nend))() end end)())),\" \"))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((result),\")\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"asLiteral",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function() if ((((self):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("#()")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=("#( ")
	result=_s8_value; return _s8_value
end)()
;
(self):do_((function(each)
return ((function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((result),(function() if ((((self):isObject_(each)))==true) then return ((function()
return ((each):asLiteral())
end))() else return ((function()
return (each)
end))() end end)()))," "))
	result=_s8_value; return _s8_value
end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((result),")"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","asLiteral\r\n\t\" Return the receiver in literal notation. \"\r\n\r\n\t| result |\r\n\tself isEmpty ifTrue: [ ^'#()' ].\r\n\tresult := '#( '.\r\n\tself do: [:each|\r\n\t\tresult := result , (\r\n\t\t\t(self isObject: each)\r\n\t\t\tifTrue: [ each asLiteral ]\r\n\t\t\tifFalse: [ each ]) ,' '\r\n\t].\r\n\t^result,')'","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function() if ((((self):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"#()\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(\"#( \")\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):do_((function(each)\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((result),(function() if ((((self):isObject_(each)))==true) then return ((function()\r\nreturn ((each):asLiteral())\r\nend))() else return ((function()\r\nreturn (each)\r\nend))() end end)())),\" \"))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((result),\")\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"asJSONObject",0
,(
function(self)
return ((self):collect_((function(each)
return ((function() if ((((self):isObject_(each)))==true) then return ((function()
return ((each):asJSONObject())
end))() else return ((function()
return (each)
end))() end end)())
end)))
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self collect: [:each| \r\n\t\t(self isObject: each)\r\n\t\tifTrue: [ each asJSONObject ]\r\n\t\tifFalse: [ each ]\r\n\t]","(\r\nfunction(self)\r\nreturn ((self):collect_((function(each)\r\nreturn ((function() if ((((self):isObject_(each)))==true) then return ((function()\r\nreturn ((each):asJSONObject())\r\nend))() else return ((function()\r\nreturn (each)\r\nend))() end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"json",0
,(
function(self)
return ((self):collect_((function(each)
return ((function() if ((((self):isObject_(each)))==true) then return ((function()
return ((each):json())
end))() else return ((function()
return (each)
end))() end end)())
end)))
;
end
)
,"converting","json\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self collect: [:each| \r\n\t\t(self isObject: each)\r\n\t\tifTrue: [ each json ]\r\n\t\tifFalse: [ each ]\r\n\t]","(\r\nfunction(self)\r\nreturn ((self):collect_((function(each)\r\nreturn ((function() if ((((self):isObject_(each)))==true) then return ((function()\r\nreturn ((each):json())\r\nend))() else return ((function()\r\nreturn (each)\r\nend))() end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"emmitAllIn:",0
,(
function(self,folder)
return (((self):collect_((function(each)
return (smalltalk:object_comma_((folder),each))
end))):emmitAll())
;
end
)
,"Builders-emmiting","emmitAllIn: folder\r\n\t\" Emmit(fileIn & save generated code of) the fileNames in the receiver.\r\n\tNote the receiver MUST include the file separator at end\r\n\t\"\r\n\r\n\t^(self collect: [:each| folder , each ]) emmitAll","(\r\nfunction(self,folder)\r\nreturn (((self):collect_((function(each)\r\nreturn (smalltalk:object_comma_((folder),each))\r\nend))):emmitAll())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"emmitAll",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
((function()
(self):emmit_((smalltalk.Array):with_((self):removeFirst()));return ((self):emmitAll())
end)):valueDeferred_((500))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Builders-emmiting","emmitAll\r\n\t\" Deferred (consuming) emmit of fileNames in the receiver. \"\r\n\r\n\tself isEmpty ifTrue: [ ^self ].\r\n\t[\tself emmit: (Array with: self removeFirst).\r\n\t\tself emmitAll\r\n\t] valueDeferred: 500","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n((function()\r\n(self):emmit_((smalltalk.Array):with_((self):removeFirst()));return ((self):emmitAll())\r\nend)):valueDeferred_((500))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"handles",0
,(
function(self)
return ((self):collect_((function(each)
return ((each):handle())
end)))
;
end
)
,"Natives-converting","handles\r\n\t\" Return the result of collecting the handles of the receiver. \"\r\n\r\n\t^self collect: [:each| each handle ]","(\r\nfunction(self)\r\nreturn ((self):collect_((function(each)\r\nreturn ((each):handle())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"asLua",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk:newArray({"),((self):collect_((function(each)
return ((each):asLua())
end))):join_(", "))),"})"))
;
end
)
,"Lua-lua","asLua\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\t^'smalltalk:newArray({'\r\n\t\t, ((self collect: [:each | each asLua ]) join: ', ')\r\n\t, '})'","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk:newArray({\"),((self):collect_((function(each)\r\nreturn ((each):asLua())\r\nend))):join_(\", \"))),\"})\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Array,"__tostring",0
,(
function(self)
return ((self):asLiteral())
;
end
)
,"Lua-printing","__tostring\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self asLiteral","(\r\nfunction(self)\r\nreturn ((self):asLiteral())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"contents",0
,(
function(self)
(function() if ((((((self['@contents']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.Array):new())
	self['@contents']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (self['@contents'])
;
end
)
,"private","contents\r\n\t\" Private - Return the contents of the receiver. \"\r\n\r\n\tcontents isNil ifTrue: [ contents := Array new ].\r\n\t^contents","(\r\nfunction(self)\r\n(function() if ((((((self['@contents']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tself['@contents']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@contents'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"add:",0
,(
function(self,anObject)
return (((self):contents()):add_(anObject))
;
end
)
,"adding","add: anObject\r\n\t\" Add anObject to the receiver. \"\r\n\r\n\t^self contents add: anObject","(\r\nfunction(self,anObject)\r\nreturn (((self):contents()):add_(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"at:ifAbsent:",0
,(
function(self,anIndex,aBlock)
return (((self):contents()):at_ifAbsent_(anIndex, aBlock))
;
end
)
,"accessing","at: anIndex ifAbsent: aBlock\r\n\t\" Return the contents of the receiver at anIndex or the result of evaluating aBlock. \"\r\n\r\n\t^self contents at: anIndex ifAbsent: aBlock","(\r\nfunction(self,anIndex,aBlock)\r\nreturn (((self):contents()):at_ifAbsent_(anIndex, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"at:put:",0
,(
function(self,anIndex,anObject)
return (((self):contents()):at_put_(anIndex, anObject))
;
end
)
,"accessing","at: anIndex put: anObject\r\n\t\" Set the contents of the receiver at anIndex. \"\r\n\r\n\t^self contents at: anIndex put: anObject","(\r\nfunction(self,anIndex,anObject)\r\nreturn (((self):contents()):at_put_(anIndex, anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"atAllPut:",0
,(
function(self,anObject)
return (((self):contents()):atAllPut_(anObject))
;
end
)
,"accessing","atAllPut: anObject\r\n\t\" Replace all elements of the receiver with anObject. \"\r\n\r\n\t^self contents atAllPut: anObject","(\r\nfunction(self,anObject)\r\nreturn (((self):contents()):atAllPut_(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"copyFrom:to:",0
,(
function(self,anIndex,anotherIndex)
local result=nil
(function()
	local _s8_value=(((self)["$klass"]):new())
	result=_s8_value; return _s8_value
end)()
;
(anIndex):to_do_(anotherIndex, (function(each)
return ((result):add_((self):at_(each)))
end))
;
return (result)
;
end
)
,"copying","copyFrom: anIndex to: anotherIndex\r\n\t\" Return the receiver contents in the interval. \"\r\n\r\n\t| result |\r\n\tresult := self class new.\r\n\tanIndex to: anotherIndex do: [:each | result add: (self at: each) ].\r\n\t^result","(\r\nfunction(self,anIndex,anotherIndex)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(anIndex):to_do_(anotherIndex, (function(each)\r\nreturn ((result):add_((self):at_(each)))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"indexOf:ifAbsent:",0
,(
function(self,anObject,aBlock)
return (((self):contents()):indexOf_ifAbsent_(anObject, aBlock))
;
end
)
,"testing","indexOf: anObject ifAbsent: aBlock\r\n\t\" Return the position of anObject in the receiver, or the result of evaluating aBlock. \"\r\n\r\n\t^self contents indexOf: anObject ifAbsent: aBlock","(\r\nfunction(self,anObject,aBlock)\r\nreturn (((self):contents()):indexOf_ifAbsent_(anObject, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"detect:ifNone:",0
,(
function(self,aBlock,anotherBlock)
return (((self):contents()):detect_ifNone_(aBlock, anotherBlock))
;
end
)
,"enumerating","detect: aBlock ifNone: anotherBlock\r\n\t\" Return the object in the receiver that return true when evaluated with aBlock or the result of evaluating anotherBlock. \"\r\n\r\n\t^self contents detect: aBlock ifNone: anotherBlock","(\r\nfunction(self,aBlock,anotherBlock)\r\nreturn (((self):contents()):detect_ifNone_(aBlock, anotherBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"do:",0
,(
function(self,aBlock)
return (((self):contents()):do_(aBlock))
;
end
)
,"enumerating","do: aBlock\r\n\t\" Evaluate aBlock with the contents of the receiver. \"\r\n\r\n\t^self contents do: aBlock","(\r\nfunction(self,aBlock)\r\nreturn (((self):contents()):do_(aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"includes:",0
,(
function(self,anObject)
return (((self):contents()):includes_(anObject))
;
end
)
,"testing","includes: anObject\r\n\t\" Return true if anObject is in the receiver. \"\r\n\r\n\t^self contents includes: anObject","(\r\nfunction(self,anObject)\r\nreturn (((self):contents()):includes_(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"isEmpty",0
,(
function(self)
return (((self):contents()):isEmpty())
;
end
)
,"testing","isEmpty\r\n\t\" Return true if the receiver is empty. \"\r\n\r\n\t^self contents isEmpty","(\r\nfunction(self)\r\nreturn (((self):contents()):isEmpty())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"occurrencesOf:",0
,(
function(self,anObject)
return (((self):contents()):occurrencesOf_(anObject))
;
end
)
,"counting","occurrencesOf: anObject\r\n\t\" Return the occurrences of anObject in the receiver. \"\r\n\r\n\t^self contents occurrencesOf: anObject","(\r\nfunction(self,anObject)\r\nreturn (((self):contents()):occurrencesOf_(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"reverseDo:",0
,(
function(self,aBlock)
return (((self):contents()):reverseDo_(aBlock))
;
end
)
,"enumerating","reverseDo: aBlock\r\n\t\" Evaluate aBlock with the contents of the receiver in reverse order of iteration. \"\r\n\r\n\t^self contents reverseDo: aBlock","(\r\nfunction(self,aBlock)\r\nreturn (((self):contents()):reverseDo_(aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"reversed",0
,(
function(self)
return (((self)["$klass"]):withAll_(((self):contents()):reversed()))
;
end
)
,"copying","reversed\r\n\t\" Return an array with the contents of the receiver in reverse order. \"\r\n\r\n\t^self class withAll: self contents reversed","(\r\nfunction(self)\r\nreturn (((self)[\"$klass\"]):withAll_(((self):contents()):reversed()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"remove:ifAbsent:",0
,(
function(self,anObject,aBlock)
return (((self):contents()):remove_ifAbsent_(anObject, aBlock))
;
end
)
,"removing","remove: anObject ifAbsent: aBlock\r\n\t\" Remove anObject from the receiver.\r\n\tReturn the result of evaluating aBlock if anObject is not in the receiver.\r\n\t\"\r\n\r\n\t^self contents remove: anObject ifAbsent: aBlock","(\r\nfunction(self,anObject,aBlock)\r\nreturn (((self):contents()):remove_ifAbsent_(anObject, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"removeAll",0
,(
function(self)
(function()
	local _s8_value=(nil)
	self['@contents']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"removing","removeAll\r\n\t\" Remove all the objects in the receiver. \"\r\n\r\n\tcontents := nil","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=(nil)\r\n\tself['@contents']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"removeFirst",0
,(
function(self)
return (((self):contents()):removeFirst())
;
end
)
,"removing","removeFirst\r\n\t\" Remove the first object in the receiver. \"\r\n\r\n\t^self contents removeFirst","(\r\nfunction(self)\r\nreturn (((self):contents()):removeFirst())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"removeLast",0
,(
function(self)
return (((self):contents()):removeLast())
;
end
)
,"removing","removeLast\r\n\t\" Remove the last object in the receiver. \"\r\n\r\n\t^self contents removeLast","(\r\nfunction(self)\r\nreturn (((self):contents()):removeLast())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"removeIndex:",0
,(
function(self,index)
return (((self):contents()):removeIndex_(index))
;
end
)
,"removing","removeIndex: index\r\n\t\" Remove the element at index. \"\r\n\r\n\t^self contents removeIndex: index","(\r\nfunction(self,index)\r\nreturn (((self):contents()):removeIndex_(index))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"size",0
,(
function(self)
return (((self):contents()):size())
;
end
)
,"accesing","size\r\n\t\" Return the size of the receiver. \"\r\n\r\n\t^self contents size","(\r\nfunction(self)\r\nreturn (((self):contents()):size())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ArrayedContents,"withIndexDo:",0
,(
function(self,aBlock)
return (((self):contents()):withIndexDo_(aBlock))
;
end
)
,"enumerating","withIndexDo: aBlock\r\n\t\" Evaluate aBlock with all the index positions in the receiver. \"\r\n\r\n\t^self contents withIndexDo: aBlock","(\r\nfunction(self,aBlock)\r\nreturn (((self):contents()):withIndexDo_(aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ComposedAssociations,",","_comma"
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((anObject):isAssociation()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((
function(_s8_rec)
(_s8_rec):add_(anObject);
return ((_s8_rec):yourself())
end
)(self))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function() if ((((((anObject))==nil)))==true) then return ((function()
return (self)
end))() else return ((function()
return (smalltalk:superImplementor(smalltalk.ComposedAssociations,"_comma")(self,anObject))
end))() end end)())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"composing",", anObject\r\n\t\" Return the receiver composed with the argument. \"\r\n\r\n\tanObject isAssociation ifTrue: [\r\n\t\t^self add: anObject; yourself\r\n\t].\r\n\t^anObject isNil ifTrue: [ self ] ifFalse: [ super , anObject ]","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((anObject):isAssociation()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(anObject);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(self))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function() if ((((((anObject))==nil)))==true) then return ((function()\r\nreturn (self)\r\nend))() else return ((function()\r\nreturn (smalltalk:superImplementor(smalltalk.ComposedAssociations,\"_comma\")(self,anObject))\r\nend))() end end)())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.ComposedAssociations,"hashObject",0
,(
function(self)
return ({})
;
end
)
,"private","hashObject\r\n\t\" Private - Return an empty hash object. \"\r\n\r\n\t^{'{}' lua'{}'}","(\r\nfunction(self)\r\nreturn ({})\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ComposedAssociations,"install:value:into:",0
,(
function(self,key,value,jsObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:boolean_and_(((self):isObject_(key)),(function()
return ((key):isArray())
end)))))==true) then return ((function()
(function() if ((((smalltalk:isObject_equalTo_(((key):size()),(1)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):install_value_into_((key):first(), value, jsObject))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function() if ((((((key):size()) > (1))))==true) then return ((function()
local target=nil
(function()
	local _s8_value=((function() local _1_=((jsObject)[(key):first()]);if (_1_==nil) then return ((function()
return ((function() local _1_=((self):hashObject()); (jsObject)[smalltalk:toString_(((key):first()))]=_1_; return _1_ end)())
end)):value() end; return _1_ end)())
	target=_s8_value; return _s8_value
end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):install_value_into_((key):copyFrom_to_((2), (key):size()), value, target))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function() local _1_=(value); (jsObject)[smalltalk:toString_((key))]=_1_; return _1_ end)())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","install: key value: value into: jsObject\r\n\t\" Private - Install a key/value pair into jsObject. \"\r\n\r\n\t((self isObject: key) and: [ key isArray ]) ifTrue: [\r\n\t\tkey size = 1 ifTrue: [ ^self install: key first value: value into: jsObject ].\r\n\t\tkey size > 1 ifTrue: [ | target |\r\n\t\t\ttarget := jsObject basicAt: key first ifAbsent: [\r\n\t\t\t\tjsObject basicAt: key first toString put: self hashObject\r\n\t\t\t].\r\n\t\t\t^self install: (key copyFrom: 2 to: key size) value: value into: target\r\n\t\t].\r\n\t].\r\n\t^jsObject\r\n\t\tbasicAt: key toString \"it is IMPORTANT to send #toString here (v8{4.7.0} bug will expose {object} instead of {string} Ale-14/oct/2015)\"\r\n\t\tput: value","(\r\nfunction(self,key,value,jsObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:boolean_and_(((self):isObject_(key)),(function()\r\nreturn ((key):isArray())\r\nend)))))==true) then return ((function()\r\n(function() if ((((smalltalk:isObject_equalTo_(((key):size()),(1)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):install_value_into_((key):first(), value, jsObject))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function() if ((((((key):size()) > (1))))==true) then return ((function()\r\nlocal target=nil\r\n(function()\r\n\tlocal _s8_value=((function() local _1_=((jsObject)[(key):first()]);if (_1_==nil) then return ((function()\r\nreturn ((function() local _1_=((self):hashObject()); (jsObject)[smalltalk:toString_(((key):first()))]=_1_; return _1_ end)())\r\nend)):value() end; return _1_ end)())\r\n\ttarget=_s8_value; return _s8_value\r\nend)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):install_value_into_((key):copyFrom_to_((2), (key):size()), value, target))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function() local _1_=(value); (jsObject)[smalltalk:toString_((key))]=_1_; return _1_ end)())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.ComposedAssociations,"asJSONObject",0
,(
function(self)
return ((self):json())
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self json","(\r\nfunction(self)\r\nreturn ((self):json())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ComposedAssociations,"json",0
,(
function(self)
local result=nil
local value=nil
(function()
	local _s8_value=((self):hashObject())
	result=_s8_value; return _s8_value
end)()
;
(self):do_((function(assoc)
(function()
	local _s8_value=((function() if ((((self):isObject_((assoc):value())))==true) then return ((function()
return (((assoc):value()):json())
end))() else return ((function()
return ((assoc):value())
end))() end end)())
	value=_s8_value; return _s8_value
end)();return ((self):install_value_into_((assoc):key(), value, result))
end))
;
return (result)
;
end
)
,"converting","json\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t| result value |\r\n\tresult := self hashObject.\r\n\tself do: [:assoc |\r\n\t\tvalue := (self isObject: assoc value)\r\n\t\t\tifTrue: [ assoc value json ]\r\n\t\t\tifFalse: [ assoc value ].\r\n\t\tself install: assoc key value: value into: result\r\n\t].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\nlocal value=nil\r\n(function()\r\n\tlocal _s8_value=((self):hashObject())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):do_((function(assoc)\r\n(function()\r\n\tlocal _s8_value=((function() if ((((self):isObject_((assoc):value())))==true) then return ((function()\r\nreturn (((assoc):value()):json())\r\nend))() else return ((function()\r\nreturn ((assoc):value())\r\nend))() end end)())\r\n\tvalue=_s8_value; return _s8_value\r\nend)();return ((self):install_value_into_((assoc):key(), value, result))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ComposedAssociations,"associationAt:ifAbsent:",0
,(
function(self,aKey,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(self):reverseDo_((function(assoc)
local value=nil
(function() if ((((smalltalk:isObject_equalTo_(((assoc):key()),aKey))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (assoc)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();(function()
	local _s8_value=((assoc):value())
	value=_s8_value; return _s8_value
end)();return ((function() if ((((smalltalk:boolean_and_(((self):isObject_(value)),(function()
return (smalltalk:is_kindOf_((value),((self)["$klass"])))
end)))))==true) then return ((function()
(function()
	local _s8_value=((value):associationAt_ifAbsent_(aKey, (function()
return (nil)
end)))
	value=_s8_value; return _s8_value
end)();return ((function() if ((((((value))~=nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (value)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):evaluate())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","associationAt: aKey ifAbsent: aBlock\r\n\t\" Return the association at aKey (or the result of evaluating aBlock).\r\n\tNote: the search will introspect in case of composite values.\r\n\t\"\r\n\r\n\tself reverseDo: [:assoc| | value |\r\n\t\tassoc key = aKey ifTrue: [ ^assoc ].\r\n\t\tvalue := assoc value.\r\n\t\t((self isObject: value) and: [value isKindOf: self class]) ifTrue: [\r\n\t\t\tvalue := value associationAt: aKey ifAbsent: [].\r\n\t\t\tvalue notNil ifTrue: [ ^value ].\r\n\t\t]\r\n\t].\r\n\t^aBlock evaluate","(\r\nfunction(self,aKey,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(self):reverseDo_((function(assoc)\r\nlocal value=nil\r\n(function() if ((((smalltalk:isObject_equalTo_(((assoc):key()),aKey))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (assoc)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();(function()\r\n\tlocal _s8_value=((assoc):value())\r\n\tvalue=_s8_value; return _s8_value\r\nend)();return ((function() if ((((smalltalk:boolean_and_(((self):isObject_(value)),(function()\r\nreturn (smalltalk:is_kindOf_((value),((self)[\"$klass\"])))\r\nend)))))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((value):associationAt_ifAbsent_(aKey, (function()\r\nreturn (nil)\r\nend)))\r\n\tvalue=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((value))~=nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (value)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):evaluate())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.ComposedAssociations,"@","_at"
,(
function(self,aKey)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local assoc=nil
(function()
	local _s8_value=((self):associationAt_ifAbsent_(aKey, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end)))
	assoc=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((assoc):value())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","@ aKey\r\n\t\" Return the value at aKey (or nil).\r\n\tNote: the search will introspect in case of composite values.\r\n\t\"\r\n\r\n\t| assoc |\r\n\tassoc := self associationAt: aKey ifAbsent: [ ^nil ].\r\n\t^assoc value","(\r\nfunction(self,aKey)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal assoc=nil\r\n(function()\r\n\tlocal _s8_value=((self):associationAt_ifAbsent_(aKey, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)))\r\n\tassoc=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((assoc):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.ComposedObjects,",","_comma"
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((anObject))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:boolean_or_(((anObject):isArray()),(function()
return (smalltalk:responds_to_((anObject),("asArray")))
end)))))==true) then return ((function()
((anObject):asArray()):do_((function(each)
return (smalltalk:object_comma_((self),each))
end));return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:boolean_and_(((((self['@species']))~=nil)),(function()
return ((smalltalk:booleanNot_((smalltalk:is_kindOf_((anObject),(self['@species']))))))
end)))))==true) then return ((function()
return ((self):error_("Invalid object to compose"))
end))() else return nil end end)()
;
(self):add_(anObject)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"composing",", anObject\r\n\t\" Return the receiver composed with the argument. \"\r\n\r\n\tanObject isNil ifTrue: [ ^self ].\r\n\t(anObject isArray or: [ anObject respondsTo: #asArray ]) ifTrue: [\r\n\t\tanObject asArray do: [:each| self , each ].\r\n\t\t^self\r\n\t].\r\n\r\n\t(species notNil and: [ (anObject isKindOf: species) not ]) ifTrue: [\r\n\t\tself error: 'Invalid object to compose'\r\n\t].\r\n\r\n\tself add: anObject.","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((anObject))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:boolean_or_(((anObject):isArray()),(function()\r\nreturn (smalltalk:responds_to_((anObject),(\"asArray\")))\r\nend)))))==true) then return ((function()\r\n((anObject):asArray()):do_((function(each)\r\nreturn (smalltalk:object_comma_((self),each))\r\nend));return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((((self['@species']))~=nil)),(function()\r\nreturn ((smalltalk:booleanNot_((smalltalk:is_kindOf_((anObject),(self['@species']))))))\r\nend)))))==true) then return ((function()\r\nreturn ((self):error_(\"Invalid object to compose\"))\r\nend))() else return nil end end)()\r\n;\r\n(self):add_(anObject)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.ComposedObjects,"species:",0
,(
function(self,aBehavior)
(function()
	local _s8_value=(aBehavior)
	self['@species']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","species: aBehavior\r\n\t\" Set the species to compose in the receiver. \"\r\n\r\n\tspecies := aBehavior","(\r\nfunction(self,aBehavior)\r\n(function()\r\n\tlocal _s8_value=(aBehavior)\r\n\tself['@species']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set["$klass"],"newContents:",0
,(
function(self,contents)
return (((self):basicNew()):initialize_(contents))
;
end
)
,"private","newContents: contents\r\n\t\" Return an instance of the receiver initialized from given contents. \"\r\n\r\n\t^self basicNew initialize: contents","(\r\nfunction(self,contents)\r\nreturn (((self):basicNew()):initialize_(contents))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set["$klass"],"basicNew:",0
,(
function(self,size)
return ((self):new())
;
end
)
,"instantiation","basicNew: size\r\n\t\" Return an instance of the receiver of specified size. \"\r\n\r\n\t^self new","(\r\nfunction(self,size)\r\nreturn ((self):new())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"initialize",0
,(
function(self)
smalltalk:superImplementor(smalltalk.Set,"initialize")(self)
;
(function()
	local _s8_value=(smalltalk:newArray({}))
	self['@contents']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"initialize","initialize\r\n\t\" Private - Initialize the receiver. \"\r\n\r\n\tsuper initialize.\r\n\tcontents := #().","(\r\nfunction(self)\r\nsmalltalk:superImplementor(smalltalk.Set,\"initialize\")(self)\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tself['@contents']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"initialize:",0
,(
function(self,initializer)
(function()
	local _s8_value=(initializer)
	self['@contents']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"initialize","initialize: initializer\r\n\t\" Private - Initialize the receiver. \"\r\n\r\n\tcontents := initializer.","(\r\nfunction(self,initializer)\r\n(function()\r\n\tlocal _s8_value=(initializer)\r\n\tself['@contents']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"size",0
,(
function(self)
return ((self['@contents']):size())
;
end
)
,"accessing","size\r\n\t\" Return the size of the receiver. \"\r\n\r\n\t^contents size","(\r\nfunction(self)\r\nreturn ((self['@contents']):size())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"basicAdd:",0
,(
function(self,anObject)
(self['@contents']):add_(anObject)
;
return (anObject)
;
end
)
,"private","basicAdd: anObject\r\n\t\" Private - Add anObject to the receiver's contents. \"\r\n\r\n\tcontents add: anObject.\r\n\t^anObject","(\r\nfunction(self,anObject)\r\n(self['@contents']):add_(anObject)\r\n;\r\nreturn (anObject)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"includes:",0
,(
function(self,anObject)
return ((self['@contents']):includes_(anObject))
;
end
)
,"testing","includes: anObject\r\n\t\" Return true if anObject is in the receiver. \"\r\n\r\n\t^contents includes: anObject","(\r\nfunction(self,anObject)\r\nreturn ((self['@contents']):includes_(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"add:",0
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):includes_(anObject)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (anObject)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):basicAdd_(anObject))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"adding","add: anObject\r\n\t\" Add anObject to the receiver. \"\r\n\r\n\t(self includes: anObject) ifTrue: [ ^anObject ].\r\n\t^self basicAdd: anObject","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):includes_(anObject)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (anObject)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):basicAdd_(anObject))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"remove:ifAbsent:",0
,(
function(self,anObject,aBlock)
return ((self['@contents']):remove_ifAbsent_(anObject, aBlock))
;
end
)
,"removing","remove: anObject ifAbsent: aBlock\r\n\t\" Remove anObject from the receiver.\r\n\tReturn the result of evaluating aBlock if anObject is not in the receiver.\r\n\t\"\r\n\r\n\t^contents remove: anObject ifAbsent: aBlock","(\r\nfunction(self,anObject,aBlock)\r\nreturn ((self['@contents']):remove_ifAbsent_(anObject, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"copyFrom:to:",0
,(
function(self,anIndex,anotherIndex)
return ((self):shouldNotImplement_("copyFrom:to:"))
;
end
)
,"copying","copyFrom: anIndex to: anotherIndex\r\n\t\" Return a copy of the receiver from anIndex to anotherIndex. \"\r\n\r\n\t^self shouldNotImplement: #copyFrom:to:","(\r\nfunction(self,anIndex,anotherIndex)\r\nreturn ((self):shouldNotImplement_(\"copyFrom:to:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"asNative:",0
,(
function(self,lang)
local send=nil
(function()
	local _s8_value=((function() if (((((lang)=="lua")))==true) then return ((function()
return (":")
end))() else return ((function()
return (".")
end))() end end)())
	send=_s8_value; return _s8_value
end)()
;
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk."),((self)["$klass"]):name())),send)),"newContents_(")),(self['@contents']):asNative_(lang))),")"))
;
end
)
,"converting","asNative: lang\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\t| send |\r\n\tsend := lang == #lua ifTrue: [ $: ] ifFalse: [ $. ].\r\n\t^'smalltalk.' ,self class name\r\n\t\t,send ,'newContents_('\r\n\t\t,(contents asNative: lang)\r\n\t\t,')'","(\r\nfunction(self,lang)\r\nlocal send=nil\r\n(function()\r\n\tlocal _s8_value=((function() if (((((lang)==\"lua\")))==true) then return ((function()\r\nreturn (\":\")\r\nend))() else return ((function()\r\nreturn (\".\")\r\nend))() end end)())\r\n\tsend=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.\"),((self)[\"$klass\"]):name())),send)),\"newContents_(\")),(self['@contents']):asNative_(lang))),\")\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"=","_eq"
,(
function(self,aSet)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:isObject_equalTo_(((self)["$klass"]),(aSet)["$klass"]))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((self):size()),(aSet):size()))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(self):do_((function(each)
return ((function() if ((((aSet):includes_(each)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"comparing","= aSet\r\n\t\" Return true if the receiver is equal to argument. \"\r\n\r\n\tself class = aSet class ifFalse: [^false].\r\n\tself size = aSet size ifFalse: [^false].\r\n\tself do: [:each | (aSet includes: each) ifFalse: [^false] ].\r\n\t^true","(\r\nfunction(self,aSet)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:isObject_equalTo_(((self)[\"$klass\"]),(aSet)[\"$klass\"]))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):size()),(aSet):size()))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(self):do_((function(each)\r\nreturn ((function() if ((((aSet):includes_(each)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"asArray",0
,(
function(self)
return ((self['@contents']):copy())
;
end
)
,"converting","asArray\r\n\t\" Return an array with the contents of the receiver. \"\r\n\r\n\t^contents copy","(\r\nfunction(self)\r\nreturn ((self['@contents']):copy())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"toString",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_((((self)["$klass"]):name())," withAll: ")),smalltalk:printString_(((self):asArray()))))
;
end
)
,"converting","toString\r\n\t\" Convert the receiver to a String. \"\r\n\r\n\t^self class name,' withAll: ',self asArray printString","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((((self)[\"$klass\"]):name()),\" withAll: \")),smalltalk:printString_(((self):asArray()))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"sorted",0
,(
function(self)
return (((self):asArray()):sort())
;
end
)
,"sorting","sorted\r\n\t\" Return the contents of the receiver sorted. \"\r\n\r\n\t^self asArray sort","(\r\nfunction(self)\r\nreturn (((self):asArray()):sort())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"sorted:",0
,(
function(self,aBlock)
return (((self):asArray()):sort_(aBlock))
;
end
)
,"sorting","sorted: aBlock\r\n\t\" Return a sorted copy of the receiver. \"\r\n\r\n\t^self asArray sort: aBlock","(\r\nfunction(self,aBlock)\r\nreturn (((self):asArray()):sort_(aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"do:",0
,(
function(self,aBlock)
return ((self['@contents']):do_(aBlock))
;
end
)
,"enumerating","do: aBlock\r\n\t\" Evaluate aBlock with the contents of the receiver. \"\r\n\r\n\t^contents do: aBlock","(\r\nfunction(self,aBlock)\r\nreturn ((self['@contents']):do_(aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"detect:ifNone:",0
,(
function(self,aBlock,anotherBlock)
return ((self['@contents']):detect_ifNone_(aBlock, anotherBlock))
;
end
)
,"enumerating","detect: aBlock ifNone: anotherBlock\r\n\t\" Return the object in the receiver that return true when evaluated with aBlock or the result of evaluating anotherBlock. \"\r\n\r\n\t^contents detect: aBlock ifNone: anotherBlock","(\r\nfunction(self,aBlock,anotherBlock)\r\nreturn ((self['@contents']):detect_ifNone_(aBlock, anotherBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"reverseDo:",0
,(
function(self,aBlock)
return ((self['@contents']):reverseDo_(aBlock))
;
end
)
,"enumerating","reverseDo: aBlock\r\n\t\" Evaluate aBlock with the contents of the receiver in reverse order of iteration. \"\r\n\r\n\t^contents reverseDo: aBlock","(\r\nfunction(self,aBlock)\r\nreturn ((self['@contents']):reverseDo_(aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"collect:",0
,(
function(self,aBlock)
return (((self)["$klass"]):withAll_((self['@contents']):collect_(aBlock)))
;
end
)
,"enumerating","collect: aBlock\r\n\t\" Return a collection with the result of evaluating aBlock with the contents of the receiver. \"\r\n\r\n\t^self class withAll: (contents collect: aBlock)","(\r\nfunction(self,aBlock)\r\nreturn (((self)[\"$klass\"]):withAll_((self['@contents']):collect_(aBlock)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"do:separatedBy:",0
,(
function(self,aBlock,anotherBlock)
return ((self['@contents']):do_separatedBy_(aBlock, anotherBlock))
;
end
)
,"enumerating","do: aBlock separatedBy: anotherBlock\r\n\t\" Evaluate aBlock for contents of the receiver evaluating anotherBlock inbetween aBlock evaluations. \"\r\n\r\n\t^contents do: aBlock separatedBy: anotherBlock","(\r\nfunction(self,aBlock,anotherBlock)\r\nreturn ((self['@contents']):do_separatedBy_(aBlock, anotherBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"inject:into:",0
,(
function(self,anObject,aBlock)
return ((self['@contents']):inject_into_(anObject, aBlock))
;
end
)
,"enumerating","inject: anObject into: aBlock\r\n\t\" Evaluate aBlock for result of previous evaluation and contents of the receiver.\r\n\tOn first evaluation anObject is used as first argument to evaluate aBlock.\r\n\t\"\r\n\r\n\t^contents inject: anObject into: aBlock","(\r\nfunction(self,anObject,aBlock)\r\nreturn ((self['@contents']):inject_into_(anObject, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"select:",0
,(
function(self,aBlock)
return (((self)["$klass"]):withAll_((self['@contents']):select_(aBlock)))
;
end
)
,"enumerating","select: aBlock\r\n\t\" Return a copy of the receiver with elements wich evaluates aBlock to true. \"\r\n\r\n\t^self class withAll: (contents select: aBlock)","(\r\nfunction(self,aBlock)\r\nreturn (((self)[\"$klass\"]):withAll_((self['@contents']):select_(aBlock)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"reject:",0
,(
function(self,aBlock)
return (((self)["$klass"]):withAll_((self['@contents']):reject_(aBlock)))
;
end
)
,"enumerating","reject: aBlock\r\n\t\" Return the contents of the receiver wich evaluates aBlock to false. \"\r\n\r\n\t^self class withAll: (contents reject: aBlock)","(\r\nfunction(self,aBlock)\r\nreturn (((self)[\"$klass\"]):withAll_((self['@contents']):reject_(aBlock)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"reversed",0
,(
function(self)
return ((self['@contents']):reversed())
;
end
)
,"copying","reversed\r\n\t\" Return an array with the contents of the receiver in reverse order. \"\r\n\r\n\t^contents reversed","(\r\nfunction(self)\r\nreturn ((self['@contents']):reversed())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"notEmpty",0
,(
function(self)
return ((self['@contents']):notEmpty())
;
end
)
,"testing","notEmpty\r\n\t\" Return true if the receiver is not empty. \"\r\n\r\n\t^contents notEmpty","(\r\nfunction(self)\r\nreturn ((self['@contents']):notEmpty())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"isEmpty",0
,(
function(self)
return ((self['@contents']):isEmpty())
;
end
)
,"testing","isEmpty\r\n\t\" Return true if the receiver is empty. \"\r\n\r\n\t^contents isEmpty","(\r\nfunction(self)\r\nreturn ((self['@contents']):isEmpty())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"occurrencesOf:",0
,(
function(self,anObject)
return ((function() if ((((self):includes_(anObject)))==true) then return ((function()
return ((1))
end))() else return ((function()
return ((0))
end))() end end)())
;
end
)
,"counting","occurrencesOf: anObject\r\n\t\" Return the occurrences of anObject in the receiver. \"\r\n\r\n\t^(self includes: anObject) ifTrue: [ 1 ] ifFalse: [ 0 ]","(\r\nfunction(self,anObject)\r\nreturn ((function() if ((((self):includes_(anObject)))==true) then return ((function()\r\nreturn ((1))\r\nend))() else return ((function()\r\nreturn ((0))\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"asSet",0
,(
function(self)
return (self)
;
end
)
,"converting","asSet\r\n\t\" Return the receiver contents in a Set. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"shallowCopy",0
,(
function(self)
return (((self)["$klass"]):newContents_((self['@contents']):copy()))
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self class newContents: contents copy","(\r\nfunction(self)\r\nreturn (((self)[\"$klass\"]):newContents_((self['@contents']):copy()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"deepCopy",0
,(
function(self)
return (((self)["$klass"]):newContents_((self['@contents']):deepCopy()))
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self class newContents: contents deepCopy","(\r\nfunction(self)\r\nreturn (((self)[\"$klass\"]):newContents_((self['@contents']):deepCopy()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"asLua",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk."),((self)["$klass"]):name())),":newContents_(")),(self['@contents']):asLua())),")"))
;
end
)
,"Lua-lua","asLua\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\t^'smalltalk.' ,self class name\r\n\t, ':newContents_('\r\n\t\t, contents asLua\r\n\t,')'","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.\"),((self)[\"$klass\"]):name())),\":newContents_(\")),(self['@contents']):asLua())),\")\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Set,"__tostring",0
,(
function(self)
return (smalltalk:toString_((self)))
;
end
)
,"Lua-printing","__tostring\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.HashedSet,"initialize",0
,(
function(self)
smalltalk:superImplementor(smalltalk.HashedSet,"initialize")(self)
;
(self):initializeHash()
;
return (self)
;
end
)
,"initialize","initialize\r\n\t\" Private - Initialize the receiver. \"\r\n\r\n\tsuper initialize.\r\n\tself initializeHash.","(\r\nfunction(self)\r\nsmalltalk:superImplementor(smalltalk.HashedSet,\"initialize\")(self)\r\n;\r\n(self):initializeHash()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.HashedSet,"initialize:",0
,(
function(self,initializer)
(function()
	local _s8_value=(initializer)
	self['@contents']=_s8_value; return _s8_value
end)()
;
(self):rehash()
;
return (self)
;
end
)
,"initialize","initialize: initializer\r\n\t\" Private - Initialize the receiver. \"\r\n\r\n\tcontents := initializer.\r\n\tself rehash.","(\r\nfunction(self,initializer)\r\n(function()\r\n\tlocal _s8_value=(initializer)\r\n\tself['@contents']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):rehash()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.HashedSet,"initializeHash",0
,(
function(self)
(function()
	local _s8_value=({})
	self['@hash']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"private","initializeHash\r\n\t\" Private - Initialize the hash of the receiver. \"\r\n\r\n\thash :={'{}' lua'{}'}.","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=({})\r\n\tself['@hash']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.HashedSet,"rehash",0
,(
function(self)
(self):initializeHash()
;
(self['@contents']):do_((function(each)
local cookie=nil
(function()
	local _s8_value=((self):wrap_(each))
	cookie=_s8_value; return _s8_value
end)();return ((function() local _1_=(cookie); (self['@hash'])[cookie]=_1_; return _1_ end)())
end))
;
return (self)
;
end
)
,"private","rehash\r\n\t\" Private - Rehash the receiver contents. \"\r\n\r\n\tself initializeHash.\r\n\tcontents do: [:each| | cookie |\r\n\t\tcookie := self wrap: each.\r\n\t\thash basicAt: cookie put: cookie\r\n\t]","(\r\nfunction(self)\r\n(self):initializeHash()\r\n;\r\n(self['@contents']):do_((function(each)\r\nlocal cookie=nil\r\n(function()\r\n\tlocal _s8_value=((self):wrap_(each))\r\n\tcookie=_s8_value; return _s8_value\r\nend)();return ((function() local _1_=(cookie); (self['@hash'])[cookie]=_1_; return _1_ end)())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.HashedSet,"asHashedSet",0
,(
function(self)
return (self)
;
end
)
,"converting","asHashedSet\r\n\t\" Return the receiver contents in a HashedSet. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.HashedSet,"includes:",0
,(
function(self,anObject)
return ((self["@hash"][self:wrap_(anObject)])~=nil)
;
end
)
,"testing","includes: anObject\r\n\t\" Return true if anObject is in the receiver. \"\r\n\r\n\t^{'Object.prototype.hasOwnProperty.call(self[\"@hash\"],anObject)'\r\n\tlua'(self[\"@hash\"][self:wrap_(anObject)])~=nil'}","(\r\nfunction(self,anObject)\r\nreturn ((self[\"@hash\"][self:wrap_(anObject)])~=nil)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.HashedSet,"indexOf:ifAbsent:",0
,(
function(self,anObject,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):includes_(anObject)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):value())
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self['@contents']):indexOf_ifAbsent_(anObject, aBlock))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","indexOf: anObject ifAbsent: aBlock\r\n\t\" Return the position of anObject in the receiver, or the result of evaluating aBlock. \"\r\n\r\n\t(self includes: anObject) ifFalse: [ ^aBlock value ].\r\n\t^contents indexOf: anObject ifAbsent: aBlock","(\r\nfunction(self,anObject,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):includes_(anObject)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self['@contents']):indexOf_ifAbsent_(anObject, aBlock))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.HashedSet,"withIndexDo:",0
,(
function(self,aBlock)
return ((self['@contents']):withIndexDo_(aBlock))
;
end
)
,"private","withIndexDo: aBlock\r\n\t\" Evaluate aBlock with all the index positions in the receiver. \"\r\n\r\n\t^contents withIndexDo: aBlock","(\r\nfunction(self,aBlock)\r\nreturn ((self['@contents']):withIndexDo_(aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.HashedSet,"removeIndex:",0
,(
function(self,index)
return ((self):remove_((self['@contents']):at_(index)))
;
end
)
,"private","removeIndex: index\r\n\t\" Remove the element at index. \"\r\n\r\n\t^self remove: (contents at: index)","(\r\nfunction(self,index)\r\nreturn ((self):remove_((self['@contents']):at_(index)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.HashedSet,"add:",0
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local cookie=nil
(function() if ((((self):includes_(anObject)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (anObject)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((self):wrap_(anObject))
	cookie=_s8_value; return _s8_value
end)()
;
(function() local _1_=(cookie); (self['@hash'])[cookie]=_1_; return _1_ end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):basicAdd_(anObject))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"adding","add: anObject\r\n\t\" Add anObject to the receiver. \"\r\n\r\n\t| cookie |\r\n\t(self includes: anObject) ifTrue: [ ^anObject ].\r\n\tcookie := self wrap: anObject.\r\n\thash basicAt: cookie put: cookie.\r\n\t^self basicAdd: anObject","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal cookie=nil\r\n(function() if ((((self):includes_(anObject)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (anObject)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):wrap_(anObject))\r\n\tcookie=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() local _1_=(cookie); (self['@hash'])[cookie]=_1_; return _1_ end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):basicAdd_(anObject))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.HashedSet,"remove:ifAbsent:",0
,(
function(self,anObject,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):includes_(anObject)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):value())
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(self['@contents']):remove_ifAbsent_(anObject, (function()
return (nil)
end))
;
	self["@hash"][self:wrap_(anObject)] = nil
	if true then return anObject end
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"removing","remove: anObject ifAbsent: aBlock\r\n\t\" Remove anObject from the receiver.\r\n\tReturn the result of evaluating aBlock if anObject is not in the receiver.\r\n\t\"\r\n\r\n\t(self includes: anObject) ifFalse: [ ^aBlock value ].\r\n\tcontents remove: anObject ifAbsent: [ ].\r\n{'\tdelete self[\"@hash\"][anObject];\r\n\treturn anObject'\r\nlua'\tself[\"@hash\"][self:wrap_(anObject)] = nil\r\n\tif true then return anObject end'}.","(\r\nfunction(self,anObject,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):includes_(anObject)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(self['@contents']):remove_ifAbsent_(anObject, (function()\r\nreturn (nil)\r\nend))\r\n;\r\n\tself[\"@hash\"][self:wrap_(anObject)] = nil\r\n\tif true then return anObject end\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection["$klass"],"basicNew:",0
,(
function(self,size)
return ((self):new())
;
end
)
,"instantiation","basicNew: size\r\n\t\" Return an instance of the receiver of specified size. \"\r\n\r\n\t^self new","(\r\nfunction(self,size)\r\nreturn ((self):new())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection["$klass"],"@","_at"
,(
function(self,jsObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
local parts=nil
(function() if ((((((jsObject))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((self):new())
	result=_s8_value; return _s8_value
end)()
;
	parts = smalltalk.Array:new()
		for k,_ in pairs(jsObject) do parts:add_(k) end
;
(parts):do_((function(key)
return ((result):at_put_(key, ((jsObject)[key])))
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"instantiation","@ jsObject\r\n\t\" Return an instance of the receiver with the properties owned by jsObject (or nil). \"\r\n\r\n\t| result parts |\r\n\tjsObject isNil ifTrue: [ ^nil ].\r\n\tresult := self new.\r\n\t{'parts=Object.getOwnPropertyNames(jsObject)'\r\n\tlua'\tparts = smalltalk.Array:new()\r\n\t\tfor k,_ in pairs(jsObject) do parts:add_(k) end'}.\r\n\tparts do: [:key|\r\n\t\tresult at: key put: (jsObject basicAt: key)\r\n\t].\r\n\t^result","(\r\nfunction(self,jsObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\nlocal parts=nil\r\n(function() if ((((((jsObject))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n\tparts = smalltalk.Array:new()\r\n\t\tfor k,_ in pairs(jsObject) do parts:add_(k) end\r\n;\r\n(parts):do_((function(key)\r\nreturn ((result):at_put_(key, ((jsObject)[key])))\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection["$klass"],"fromTuples:",0
,(
function(self,tuples)
local result=nil
(function()
	local _s8_value=((self):new())
	result=_s8_value; return _s8_value
end)()
;
(tuples):do_((function(tuple)
return ((result):at_put_((tuple):first(), (tuple):last()))
end))
;
return (result)
;
end
)
,"instantiation","fromTuples: tuples\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t| result |\r\n\tresult := self new.\r\n\ttuples do: [:tuple| result at: tuple first put: tuple last ].\r\n\t^result","(\r\nfunction(self,tuples)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(tuples):do_((function(tuple)\r\nreturn ((result):at_put_((tuple):first(), (tuple):last()))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,",","_comma"
,(
function(self,aCollection)
return ((self):shouldNotImplement_(","))
;
end
)
,"copying",", aCollection\r\n\t\" Return a copy of the receiver with all contents of aCollection. \"\r\n\r\n\t^self shouldNotImplement: #,","(\r\nfunction(self,aCollection)\r\nreturn ((self):shouldNotImplement_(\",\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"=","_eq"
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:isObject_equalTo_(((self)["$klass"]),(anObject)["$klass"]))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((self):size()),(anObject):size()))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(self):keysAndValuesDo_((function(key,value)
return ((function() if ((((smalltalk:isObject_equalTo_(((anObject):at_ifAbsent_(key, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))),value))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"comparing","= anObject\r\n\t\" Return true if the receiver is equal to argument. \"\r\n\r\n\tself class = anObject class ifFalse: [^false].\r\n\tself size = anObject size ifFalse: [^false].\r\n\tself keysAndValuesDo: [:key :value |\r\n\t\t(anObject at: key ifAbsent: [^false])\r\n\t\t\t= value ifFalse: [^false]\r\n\t].\r\n\t^true","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:isObject_equalTo_(((self)[\"$klass\"]),(anObject)[\"$klass\"]))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):size()),(anObject):size()))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(self):keysAndValuesDo_((function(key,value)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_(((anObject):at_ifAbsent_(key, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))),value))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"add:",0
,(
function(self,anAssociation)
(self):at_put_((anAssociation):key(), (anAssociation):value())
;
return (anAssociation)
;
end
)
,"adding","add: anAssociation\r\n\t\" Bind anAssociation to the receiver. \"\r\n\r\n\tself at: anAssociation key put: anAssociation value.\r\n\t^anAssociation","(\r\nfunction(self,anAssociation)\r\n(self):at_put_((anAssociation):key(), (anAssociation):value())\r\n;\r\nreturn (anAssociation)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"addAll:",0
,(
function(self,aKeyedCollection)
smalltalk:superImplementor(smalltalk.KeyedCollection,"addAll_")(self,(aKeyedCollection):associations())
;
return (aKeyedCollection)
;
end
)
,"adding","addAll: aKeyedCollection\r\n\t\" Bind all associations in aKeyedCollection to the receiver. \"\r\n\r\n\tsuper addAll: aKeyedCollection associations.\r\n\t^aKeyedCollection","(\r\nfunction(self,aKeyedCollection)\r\nsmalltalk:superImplementor(smalltalk.KeyedCollection,\"addAll_\")(self,(aKeyedCollection):associations())\r\n;\r\nreturn (aKeyedCollection)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"newHash",0
,(
function(self)
return ({})
;
end
)
,"private","newHash\r\n\t\" Private - Return a new hash object. \"\r\n\r\n\t^{'{}' lua'{}'}","(\r\nfunction(self)\r\nreturn ({})\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"asJSONObject",0
,(
function(self)
local result=nil
local value=nil
(function()
	local _s8_value=((self):newHash())
	result=_s8_value; return _s8_value
end)()
;
(self):keysAndValuesDo_((function(key,aValue)
(function()
	local _s8_value=((function() if ((((self):isObject_(aValue)))==true) then return ((function()
return ((aValue):asJSONObject())
end))() else return ((function()
return (aValue)
end))() end end)())
	value=_s8_value; return _s8_value
end)();return ((function() local _1_=(value); (result)[key]=_1_; return _1_ end)())
end))
;
return (result)
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t| result value |\r\n\tresult := self newHash.\r\n\tself keysAndValuesDo: [:key :aValue |\r\n\t\tvalue := (self isObject: aValue)\r\n\t\t\tifTrue: [ aValue asJSONObject ]\r\n\t\t\tifFalse: [ aValue ].\r\n\t\tresult basicAt: key put: value\r\n\t].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\nlocal value=nil\r\n(function()\r\n\tlocal _s8_value=((self):newHash())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):keysAndValuesDo_((function(key,aValue)\r\n(function()\r\n\tlocal _s8_value=((function() if ((((self):isObject_(aValue)))==true) then return ((function()\r\nreturn ((aValue):asJSONObject())\r\nend))() else return ((function()\r\nreturn (aValue)\r\nend))() end end)())\r\n\tvalue=_s8_value; return _s8_value\r\nend)();return ((function() local _1_=(value); (result)[key]=_1_; return _1_ end)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"associations",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	result=_s8_value; return _s8_value
end)()
;
(self):associationsDo_((function(each)
return ((result):add_(each))
end))
;
return (result)
;
end
)
,"accessing","associations\r\n\t\" Return the associations of the receiver. \"\r\n\r\n\t| result |\r\n\tresult := Array new.\r\n\tself associationsDo: [:each| result add: each ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):associationsDo_((function(each)\r\nreturn ((result):add_(each))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"associationsDo:",0
,(
function(self,aBlock)
(self):keysAndValuesDo_((function(key,value)
return ((aBlock):value_((smalltalk.Association):key_value_(key, value)))
end))
;
return (self)
;
end
)
,"enumerating","associationsDo: aBlock\r\n\t\" Evaluate aBlock for all the receiver's associations. \"\r\n\r\n\tself keysAndValuesDo: [:key :value|\r\n\t\taBlock value: (Association key: key value: value)\r\n\t].","(\r\nfunction(self,aBlock)\r\n(self):keysAndValuesDo_((function(key,value)\r\nreturn ((aBlock):value_((smalltalk.Association):key_value_(key, value)))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"at:",0
,(
function(self,aKey)
return ((self):at_ifAbsent_(aKey, (function()
return ((self):objectNotFoundAt_(aKey))
end)))
;
end
)
,"accessing","at: aKey\r\n\t\" Return the value at aKey.\r\n\tSignal an error if the key is not present.\r\n\t\"\r\n\r\n\t^self at: aKey ifAbsent: [ self objectNotFoundAt: aKey ]","(\r\nfunction(self,aKey)\r\nreturn ((self):at_ifAbsent_(aKey, (function()\r\nreturn ((self):objectNotFoundAt_(aKey))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"at:ifAbsent:",0
,(
function(self,aKey,aBlock)
return ((self):implementedBySubclass_("at:ifAbsent:"))
;
end
)
,"accessing","at: aKey ifAbsent: aBlock\r\n\t\" Return the value at aKey or the result of evaluating aBlock. \"\r\n\r\n\t^self implementedBySubclass: #at:ifAbsent:","(\r\nfunction(self,aKey,aBlock)\r\nreturn ((self):implementedBySubclass_(\"at:ifAbsent:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"values",0
,(
function(self)
return ((self):implementedBySubclass_("values"))
;
end
)
,"accessing","values\r\n\t\" Return the values of the receiver. \"\r\n\r\n\t^self implementedBySubclass: #values","(\r\nfunction(self)\r\nreturn ((self):implementedBySubclass_(\"values\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"at:ifAbsentPut:",0
,(
function(self,aKey,aBlock)
return ((self):at_ifAbsent_(aKey, (function()
return ((self):at_put_(aKey, (aBlock):value()))
end)))
;
end
)
,"accessing","at: aKey ifAbsentPut: aBlock\r\n\t\" Return the value at aKey, or add the result of evaluating aBlock to the receiver if it is absent. \"\r\n\r\n\t^self at: aKey ifAbsent: [ self at: aKey put: aBlock value ]","(\r\nfunction(self,aKey,aBlock)\r\nreturn ((self):at_ifAbsent_(aKey, (function()\r\nreturn ((self):at_put_(aKey, (aBlock):value()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"at:put:",0
,(
function(self,aKey,aValue)
return ((self):implementedBySubclass_("at:put:"))
;
end
)
,"accessing","at: aKey put: aValue\r\n\t\" Set the value at aKey of the receiver. \"\r\n\r\n\t^self implementedBySubclass: #at:put:","(\r\nfunction(self,aKey,aValue)\r\nreturn ((self):implementedBySubclass_(\"at:put:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"detect:ifNone:",0
,(
function(self,aBlock,anotherBlock)
return ((self):implementedBySubclass_("detect:ifNone:"))
;
end
)
,"enumerating","detect: aBlock ifNone: anotherBlock\r\n\t\" Detect the value who matches aBlock.\r\n\tReturn the result of evaluating aBlock if no value matches detection block.\r\n\t\"\r\n\r\n\t^self implementedBySubclass: #detect:ifNone:","(\r\nfunction(self,aBlock,anotherBlock)\r\nreturn ((self):implementedBySubclass_(\"detect:ifNone:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"do:",0
,(
function(self,aBlock)
return ((self):implementedBySubclass_("do:"))
;
end
)
,"enumerating","do: aBlock\r\n\t\" Evaluates aBlock for all values in the receiver. \"\r\n\r\n\t^self implementedBySubclass: #do:","(\r\nfunction(self,aBlock)\r\nreturn ((self):implementedBySubclass_(\"do:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"includes:",0
,(
function(self,anObject)
return ((self):implementedBySubclass_("includes:"))
;
end
)
,"testing","includes: anObject\r\n\t\" Return true if anObject is in the receiver. \"\r\n\r\n\t^self implementedBySubclass: #includes:","(\r\nfunction(self,anObject)\r\nreturn ((self):implementedBySubclass_(\"includes:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"keyForValue:ifAbsent:",0
,(
function(self,anObject,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(self):keysAndValuesDo_((function(key,value)
return ((function() if ((((smalltalk:isObject_equalTo_((anObject),value))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (key)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):value())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","keyForValue: anObject ifAbsent: aBlock\r\n\t\" Return the key in the receiver for value anObject, or the result of evaluating aBlock. \"\r\n\r\n\tself keysAndValuesDo: [:key :value| anObject = value ifTrue: [ ^key ] ].\r\n\t^aBlock value","(\r\nfunction(self,anObject,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(self):keysAndValuesDo_((function(key,value)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((anObject),value))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (key)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"keys",0
,(
function(self)
return ((self):implementedBySubclass_("keys"))
;
end
)
,"accessing","keys\r\n\t\" Return the keys of the receiver. \"\r\n\r\n\t^self implementedBySubclass: #keys","(\r\nfunction(self)\r\nreturn ((self):implementedBySubclass_(\"keys\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"keysAndValuesDo:",0
,(
function(self,aBlock)
return ((self):implementedBySubclass_("keysAndValuesDo:"))
;
end
)
,"enumerating","keysAndValuesDo: aBlock\r\n\t\" Evaluate aBlock for key/value pairs of the receiver. \"\r\n\r\n\t^self implementedBySubclass: #keysAndValuesDo:","(\r\nfunction(self,aBlock)\r\nreturn ((self):implementedBySubclass_(\"keysAndValuesDo:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"keysDo:",0
,(
function(self,aBlock)
return ((self):implementedBySubclass_("keysDo:"))
;
end
)
,"enumerating","keysDo: aBlock\r\n\t\" Evaluate aBlock with the keys of the receiver. \"\r\n\r\n\t^self implementedBySubclass: #keysDo:","(\r\nfunction(self,aBlock)\r\nreturn ((self):implementedBySubclass_(\"keysDo:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"includesKey:",0
,(
function(self,aKey)
return ((self):implementedBySubclass_("includesKey:"))
;
end
)
,"testing","includesKey: aKey\r\n\t\" Return true if the receiver's keys include aKey. \"\r\n\r\n\t^self implementedBySubclass: #includesKey:","(\r\nfunction(self,aKey)\r\nreturn ((self):implementedBySubclass_(\"includesKey:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"removeKey:ifAbsent:",0
,(
function(self,aKey,aBlock)
return ((self):implementedBySubclass_("removeKey:ifAbsent:"))
;
end
)
,"removing","removeKey: aKey ifAbsent: aBlock\r\n\t\" Remove the binding at aKey. \"\r\n\r\n\t^self implementedBySubclass: #removeKey:ifAbsent:","(\r\nfunction(self,aKey,aBlock)\r\nreturn ((self):implementedBySubclass_(\"removeKey:ifAbsent:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"isEmpty",0
,(
function(self)
return ((smalltalk:isObject_equalTo_(((self):size()),(0))))
;
end
)
,"testing","isEmpty\r\n\t\" Return true if the receiver is empty. \"\r\n\r\n\t^self size = 0","(\r\nfunction(self)\r\nreturn ((smalltalk:isObject_equalTo_(((self):size()),(0))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"at:ifPresent:",0
,(
function(self,aKey,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local aValue=nil
(function()
	local _s8_value=((self):at_ifAbsent_(aKey, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end)))
	aValue=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):value_(aValue))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","at: aKey ifPresent: aBlock\r\n\t\" Return the result of evaluating aBlock if there is a value at aKey (or nil). \"\r\n\r\n\t| aValue |\r\n\taValue := self at: aKey ifAbsent: [ ^nil ].\r\n\t^aBlock value: aValue","(\r\nfunction(self,aKey,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal aValue=nil\r\n(function()\r\n\tlocal _s8_value=((self):at_ifAbsent_(aKey, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)))\r\n\taValue=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):value_(aValue))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"at:ifPresent:ifAbsent:",0
,(
function(self,aKey,aBlock,anotherBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local aValue=nil
(function()
	local _s8_value=((self):at_ifAbsent_(aKey, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((anotherBlock):value())
end
)()
error(_s8Ret_,0)
end)())
end)))
	aValue=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):value_(aValue))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","at: aKey ifPresent: aBlock ifAbsent: anotherBlock\r\n\t\" Return the result of evaluating aBlock if there is a value at aKey (or the result of evaluating anotherBlock). \"\r\n\r\n\t| aValue |\r\n\taValue := self at: aKey ifAbsent: [ ^anotherBlock value ].\r\n\t^aBlock value: aValue","(\r\nfunction(self,aKey,aBlock,anotherBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal aValue=nil\r\n(function()\r\n\tlocal _s8_value=((self):at_ifAbsent_(aKey, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((anotherBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)))\r\n\taValue=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):value_(aValue))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"collect:",0
,(
function(self,aBlock)
local result=nil
(function()
	local _s8_value=(((self)["$klass"]):new())
	result=_s8_value; return _s8_value
end)()
;
(self):keysAndValuesDo_((function(key,value)
return ((result):at_put_(key, (aBlock):value_(value)))
end))
;
return (result)
;
end
)
,"enumerating","collect: aBlock\r\n\t\" Return aKeyedCollection with keys and result of evaluating aBlock with value. \"\r\n\r\n\t| result |\r\n\tresult := self class new.\r\n\tself keysAndValuesDo: [:key :value |\r\n\t\tresult at: key put: (aBlock value: value)\r\n\t].\r\n\t^result","(\r\nfunction(self,aBlock)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):keysAndValuesDo_((function(key,value)\r\nreturn ((result):at_put_(key, (aBlock):value_(value)))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"select:",0
,(
function(self,aBlock)
local result=nil
(function()
	local _s8_value=(((self)["$klass"]):new())
	result=_s8_value; return _s8_value
end)()
;
(self):keysAndValuesDo_((function(key,value)
return ((function() if ((((aBlock):value_(value)))==true) then return ((function()
return ((result):at_put_(key, value))
end))() else return nil end end)())
end))
;
return (result)
;
end
)
,"enumerating","select: aBlock\r\n\t\" Return the contents of the receiver wich evaluates aBlock to true. \"\r\n\r\n\t| result |\r\n\tresult := self class new.\r\n\tself keysAndValuesDo: [:key :value |\r\n\t\t(aBlock value: value) ifTrue: [ result at: key put: value ]\r\n\t].\r\n\t^result","(\r\nfunction(self,aBlock)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):keysAndValuesDo_((function(key,value)\r\nreturn ((function() if ((((aBlock):value_(value)))==true) then return ((function()\r\nreturn ((result):at_put_(key, value))\r\nend))() else return nil end end)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"shallowCopy",0
,(
function(self)
local result=nil
(function()
	local _s8_value=(((self)["$klass"]):new())
	result=_s8_value; return _s8_value
end)()
;
(self):keysAndValuesDo_((function(key,value)
return ((result):at_put_(key, value))
end))
;
return (result)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t| result |\r\n\tresult := self class new.\r\n\tself keysAndValuesDo: [:key :value | result at: key put: value ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):keysAndValuesDo_((function(key,value)\r\nreturn ((result):at_put_(key, value))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"tuples",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	result=_s8_value; return _s8_value
end)()
;
(self):keysAndValuesDo_((function(key,value)
return ((result):add_((smalltalk.Array):with_with_(key, value)))
end))
;
return (result)
;
end
)
,"accessing","tuples\r\n\t\" Return an array of tuples built from the receiver's contents. \"\r\n\r\n\t| result |\r\n\tresult := Array new.\r\n\tself keysAndValuesDo: [:key :value|\r\n\t\tresult add: (Array with: key with: value)\r\n\t].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):keysAndValuesDo_((function(key,value)\r\nreturn ((result):add_((smalltalk.Array):with_with_(key, value)))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"copyFrom:to:",0
,(
function(self,anIndex,anotherIndex)
return ((self):shouldNotImplement_("copyFrom:to:"))
;
end
)
,"copying","copyFrom: anIndex to: anotherIndex\r\n\t\" Return a copy of the receiver from anIndex to anotherIndex. \"\r\n\r\n\t^self shouldNotImplement: #copyFrom:to:","(\r\nfunction(self,anIndex,anotherIndex)\r\nreturn ((self):shouldNotImplement_(\"copyFrom:to:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"deepCopy",0
,(
function(self)
local result=nil
(function()
	local _s8_value=(((self)["$klass"]):new())
	result=_s8_value; return _s8_value
end)()
;
(self):keysAndValuesDo_((function(key,value)
return ((result):at_put_(key, (value):deepCopy()))
end))
;
return (result)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t| result |\r\n\tresult := self class new.\r\n\tself keysAndValuesDo: [:key :value |\r\n\t\tresult at: key put: value deepCopy\r\n\t].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):keysAndValuesDo_((function(key,value)\r\nreturn ((result):at_put_(key, (value):deepCopy()))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"remove:",0
,(
function(self,aKey)
(self):removeKey_(aKey)
;
return (self)
;
end
)
,"removing","remove: aKey\r\n\t\" Remove the binding at aKey. \"\r\n\r\n\tself removeKey: aKey","(\r\nfunction(self,aKey)\r\n(self):removeKey_(aKey)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"removeKey:",0
,(
function(self,aKey)
return ((self):removeKey_ifAbsent_(aKey, (function()
return ((self):objectNotFoundAt_(aKey))
end)))
;
end
)
,"removing","removeKey: aKey\r\n\t\" Remove the binding at aKey.\r\n\tSignal an error if the key is not present.\r\n\t\"\r\n\r\n\t^self removeKey: aKey ifAbsent: [ self objectNotFoundAt: aKey ]","(\r\nfunction(self,aKey)\r\nreturn ((self):removeKey_ifAbsent_(aKey, (function()\r\nreturn ((self):objectNotFoundAt_(aKey))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.KeyedCollection,"isDictionary",0
,(
function(self)
return (true)
;
end
)
,"query","isDictionary\r\n\t\" Return true if the receiver is a dictionary. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary["$klass"],"basicNew",0
,(
function(self)
	if true then return smalltalk.PoolDictionary:new() end 
;
return (smalltalk:superImplementor(smalltalk.Dictionary["$klass"],"basicNew")(self))
;
end
)
,"instantiation","basicNew\r\n\t\" Return an instance of the receiver. \"\r\n\r\n{''lua'\tif true then return smalltalk.PoolDictionary:new() end '}.\r\n\t^super basicNew","(\r\nfunction(self)\r\n\tif true then return smalltalk.PoolDictionary:new() end \r\n;\r\nreturn (smalltalk:superImplementor(smalltalk.Dictionary[\"$klass\"],\"basicNew\")(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary["$klass"],"fromKeys:and:",0
,(
function(self,keys,values)
return (((self):basicNew()):initializeKeys_values_(keys, values))
;
end
)
,"instantiation","fromKeys: keys and: values\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self basicNew initializeKeys: keys values: values","(\r\nfunction(self,keys,values)\r\nreturn (((self):basicNew()):initializeKeys_values_(keys, values))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"initialize",0
,(
function(self)
smalltalk:superImplementor(smalltalk.Dictionary,"initialize")(self)
;
(function()
	local _s8_value=((smalltalk.HashedSet):new())
	self['@keys']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(smalltalk:newArray({}))
	self['@values']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"initialize","initialize\r\n\t\" Private - Initialize the receiver. \"\r\n\r\n\tsuper initialize.\r\n\tkeys := HashedSet new.\r\n\tvalues := #().","(\r\nfunction(self)\r\nsmalltalk:superImplementor(smalltalk.Dictionary,\"initialize\")(self)\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.HashedSet):new())\r\n\tself['@keys']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tself['@values']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"initializeKeys:values:",0
,(
function(self,privateKeys,privateValues)
smalltalk:superImplementor(smalltalk.Dictionary,"initialize")(self)
;
(function()
	local _s8_value=((smalltalk.HashedSet):newContents_(privateKeys))
	self['@keys']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((privateValues):asArray())
	self['@values']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"initialize","initializeKeys: privateKeys values: privateValues\r\n\t\" Private - Initialize the receiver. \"\r\n\r\n\tsuper initialize.\r\n\tkeys := HashedSet newContents: privateKeys.\r\n\tvalues := privateValues asArray.","(\r\nfunction(self,privateKeys,privateValues)\r\nsmalltalk:superImplementor(smalltalk.Dictionary,\"initialize\")(self)\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.HashedSet):newContents_(privateKeys))\r\n\tself['@keys']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((privateValues):asArray())\r\n\tself['@values']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"size",0
,(
function(self)
return ((self['@keys']):size())
;
end
)
,"accessing","size\r\n\t\" Return the size of the receiver. \"\r\n\r\n\t^keys size","(\r\nfunction(self)\r\nreturn ((self['@keys']):size())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"isEmpty",0
,(
function(self)
return ((self['@keys']):isEmpty())
;
end
)
,"testing","isEmpty\r\n\t\" Return true if the receiver is empty. \"\r\n\r\n\t^keys isEmpty","(\r\nfunction(self)\r\nreturn ((self['@keys']):isEmpty())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"lookUpKey:",0
,(
function(self,aKey)
return ((self['@keys']):indexOf_ifAbsent_(aKey, (function()
return (nil)
end)))
;
end
)
,"private","lookUpKey: aKey\r\n\t\" Private - Return the index where aKey is found, or nil. \"\r\n\r\n\t^keys indexOf: aKey ifAbsent: [ nil ]","(\r\nfunction(self,aKey)\r\nreturn ((self['@keys']):indexOf_ifAbsent_(aKey, (function()\r\nreturn (nil)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"keysAndValuesDo:",0
,(
function(self,aBlock)
(self['@keys']):withIndexDo_((function(aKey,index)
return ((aBlock):value_value_(aKey, (self['@values']):at_(index)))
end))
;
return (self)
;
end
)
,"enumerating","keysAndValuesDo: aBlock\r\n\t\" Evaluate aBlock for key/value pairs of the receiver. \"\r\n\r\n\tkeys withIndexDo: [:aKey :index | aBlock value: aKey value: (values at: index) ]","(\r\nfunction(self,aBlock)\r\n(self['@keys']):withIndexDo_((function(aKey,index)\r\nreturn ((aBlock):value_value_(aKey, (self['@values']):at_(index)))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"keys",0
,(
function(self)
return ((self['@keys']):copy())
;
end
)
,"accessing","keys\r\n\t\" Return the keys of the receiver. \"\r\n\r\n\t^keys copy","(\r\nfunction(self)\r\nreturn ((self['@keys']):copy())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"keysDo:",0
,(
function(self,aBlock)
(self['@keys']):do_(aBlock)
;
return (self)
;
end
)
,"enumerating","keysDo: aBlock\r\n\t\" Evaluate aBlock with the keys of the receiver. \"\r\n\r\n\tkeys do: aBlock","(\r\nfunction(self,aBlock)\r\n(self['@keys']):do_(aBlock)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"values",0
,(
function(self)
return ((self['@values']):copy())
;
end
)
,"accessing","values\r\n\t\" Return the values of the receiver. \"\r\n\r\n\t^values copy","(\r\nfunction(self)\r\nreturn ((self['@values']):copy())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"at:put:",0
,(
function(self,aKey,aValue)
local index=nil
(function()
	local _s8_value=((self):lookUpKey_(aKey))
	index=_s8_value; return _s8_value
end)()
;
(function() if ((((((index))==nil)))==true) then return ((function()
(self['@keys']):add_(aKey);return ((self['@values']):add_(aValue))
end))() else return ((function()
return ((self['@values']):at_put_(index, aValue))
end))() end end)()
;
return (aValue)
;
end
)
,"accessing","at: aKey put: aValue\r\n\t\" Set the value at aKey of the receiver. \"\r\n\r\n\t| index |\r\n\tindex := self lookUpKey: aKey.\r\n\tindex isNil\r\n\t\tifTrue: [ keys add: aKey. values add: aValue.]\r\n\t\tifFalse: [ values at: index put: aValue ].\r\n\t^aValue","(\r\nfunction(self,aKey,aValue)\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((self):lookUpKey_(aKey))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((index))==nil)))==true) then return ((function()\r\n(self['@keys']):add_(aKey);return ((self['@values']):add_(aValue))\r\nend))() else return ((function()\r\nreturn ((self['@values']):at_put_(index, aValue))\r\nend))() end end)()\r\n;\r\nreturn (aValue)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"at:ifAbsent:",0
,(
function(self,aKey,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local index=nil
(function()
	local _s8_value=((self):lookUpKey_(aKey))
	index=_s8_value; return _s8_value
end)()
;
(function() if ((((((index))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):value())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self['@values']):at_(index))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","at: aKey ifAbsent: aBlock\r\n\t\" Return the value at aKey or the result of evaluating aBlock. \"\r\n\r\n\t| index |\r\n\tindex := self lookUpKey: aKey.\r\n\tindex isNil ifTrue: [ ^aBlock value ].\r\n\t^values at: index","(\r\nfunction(self,aKey,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((self):lookUpKey_(aKey))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((index))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self['@values']):at_(index))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"at:ifAbsentPut:",0
,(
function(self,aKey,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local index=nil
(function()
	local _s8_value=((self):lookUpKey_(aKey))
	index=_s8_value; return _s8_value
end)()
;
(function() if ((((((index))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):at_put_(aKey, (aBlock):value()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self['@values']):at_(index))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","at: aKey ifAbsentPut: aBlock\r\n\t\" Return the value at aKey, or add the result of evaluating aBlock to the receiver if it is absent. \"\r\n\r\n\t| index |\r\n\tindex := self lookUpKey: aKey.\r\n\tindex isNil ifTrue: [ ^self at: aKey put: aBlock value ].\r\n\t^values at: index","(\r\nfunction(self,aKey,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((self):lookUpKey_(aKey))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((index))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):at_put_(aKey, (aBlock):value()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self['@values']):at_(index))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"do:",0
,(
function(self,aBlock)
return ((self['@values']):do_(aBlock))
;
end
)
,"enumerating","do: aBlock\r\n\t\" Evaluates aBlock for all values in the receiver. \"\r\n\r\n\t^values do: aBlock","(\r\nfunction(self,aBlock)\r\nreturn ((self['@values']):do_(aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"detect:ifNone:",0
,(
function(self,aBlock,anotherBlock)
return ((self['@values']):detect_ifNone_(aBlock, anotherBlock))
;
end
)
,"enumerating","detect: aBlock ifNone: anotherBlock\r\n\t\" Detect the value who matches aBlock.\r\n\tReturn the result of evaluating aBlock if no value matches detection block.\r\n\t\"\r\n\r\n\t^values detect: aBlock ifNone: anotherBlock","(\r\nfunction(self,aBlock,anotherBlock)\r\nreturn ((self['@values']):detect_ifNone_(aBlock, anotherBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"includes:",0
,(
function(self,anObject)
return ((self['@values']):includes_(anObject))
;
end
)
,"testing","includes: anObject\r\n\t\" Return true if anObject is in the receiver. \"\r\n\r\n\t^values includes: anObject","(\r\nfunction(self,anObject)\r\nreturn ((self['@values']):includes_(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"includesKey:",0
,(
function(self,aKey)
return ((self['@keys']):includes_(aKey))
;
end
)
,"enumerating","includesKey: aKey\r\n\t\" Return true if the receiver's keys include aKey. \"\r\n\r\n\t^keys includes: aKey","(\r\nfunction(self,aKey)\r\nreturn ((self['@keys']):includes_(aKey))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"removeKey:ifAbsent:",0
,(
function(self,aKey,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local index=nil
(function()
	local _s8_value=((self):lookUpKey_(aKey))
	index=_s8_value; return _s8_value
end)()
;
(function() if ((((((index))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):value())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(self['@keys']):removeIndex_(index)
;
(self['@values']):removeIndex_(index)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"removing","removeKey: aKey ifAbsent: aBlock\r\n\t\" Remove the binding at aKey. \"\r\n\r\n\t| index |\r\n\tindex := self lookUpKey: aKey.\r\n\tindex isNil ifTrue: [ ^aBlock value ].\r\n\tkeys removeIndex: index.\r\n\tvalues removeIndex: index.","(\r\nfunction(self,aKey,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((self):lookUpKey_(aKey))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((index))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self['@keys']):removeIndex_(index)\r\n;\r\n(self['@values']):removeIndex_(index)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"keyForValue:ifAbsent:",0
,(
function(self,anObject,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(self):keysAndValuesDo_((function(key,aValue)
return ((function() if (((((anObject)==aValue)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (key)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):value())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","keyForValue: anObject ifAbsent: aBlock\r\n\t\" Return the key in the receiver for value anObject, or the result of evaluating aBlock. \"\r\n\r\n\tself keysAndValuesDo: [:key :aValue| anObject == aValue ifTrue: [ ^key ] ].\r\n\t^aBlock value","(\r\nfunction(self,anObject,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(self):keysAndValuesDo_((function(key,aValue)\r\nreturn ((function() if (((((anObject)==aValue)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (key)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"asNative:",0
,(
function(self,lang)
local send=nil
(function()
	local _s8_value=((function() if (((((lang)=="lua")))==true) then return ((function()
return (":")
end))() else return ((function()
return (".")
end))() end end)())
	send=_s8_value; return _s8_value
end)()
;
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk."),((self)["$klass"]):name())),send)),"fromKeys_and_(")),((self['@keys']):asArray()):asNative_(lang))),",")),(self['@values']):asNative_(lang))),")"))
;
end
)
,"converting","asNative: lang\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\t| send |\r\n\tsend := lang == #lua ifTrue: [$:] ifFalse: [$.].\r\n\t^'smalltalk.' ,self class name\r\n\t\t,send ,'fromKeys_and_('\r\n\t\t\t,(keys asArray asNative: lang)\r\n\t\t\t,',' ,(values asNative: lang)\r\n\t\t,')'","(\r\nfunction(self,lang)\r\nlocal send=nil\r\n(function()\r\n\tlocal _s8_value=((function() if (((((lang)==\"lua\")))==true) then return ((function()\r\nreturn (\":\")\r\nend))() else return ((function()\r\nreturn (\".\")\r\nend))() end end)())\r\n\tsend=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.\"),((self)[\"$klass\"]):name())),send)),\"fromKeys_and_(\")),((self['@keys']):asArray()):asNative_(lang))),\",\")),(self['@values']):asNative_(lang))),\")\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Dictionary,"asLua",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk."),((self)["$klass"]):name())),":fromKeys_and_(")),((self['@keys']):asArray()):asLua())),",")),(self['@values']):asLua())),")"))
;
end
)
,"Lua-lua","asLua\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\t^'smalltalk.' ,self class name\r\n\t, ':fromKeys_and_('\r\n\t\t, keys asArray asLua\r\n\t\t,',' ,values asLua\r\n\t,')'","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.\"),((self)[\"$klass\"]):name())),\":fromKeys_and_(\")),((self['@keys']):asArray()):asLua())),\",\")),(self['@values']):asLua())),\")\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary["$klass"],"@@","_at_at"
,(
function(self,jsObject)
return ((
function(_s8_rec)
(_s8_rec):instVarAt_put_("hash", jsObject);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","@@ jsObject\r\n\t\" Returns an instance of the receiver. \"\r\n\t\" The instance will hack jsObject violating encapsulation of jsObject and the instance.\r\n\t Both will share internal state. \"\r\n\r\n\t^self new\r\n\t\tinstVarAt: #hash put: jsObject;\r\n\t\tyourself","(\r\nfunction(self,jsObject)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):instVarAt_put_(\"hash\", jsObject);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"initialize",0
,(
function(self)
smalltalk:superImplementor(smalltalk.PoolDictionary,"initialize")(self)
;
(function()
	local _s8_value=((self):newHash())
	self['@hash']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"initialize","initialize\r\n\t\" Private - Initialize the receiver. \"\r\n\r\n\tsuper initialize.\r\n\thash := self newHash.","(\r\nfunction(self)\r\nsmalltalk:superImplementor(smalltalk.PoolDictionary,\"initialize\")(self)\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):newHash())\r\n\tself['@hash']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"at:put:",0
,(
function(self,aKey,value)
(function()
	local _s8_value=(nil)
	self['@size']=_s8_value; return _s8_value
end)()
;
return ((function() local _1_=((self):wrap_(value)); (self['@hash'])[(self):wrap_(aKey)]=_1_; return _1_ end)())
;
end
)
,"accessing","at: aKey put: value\r\n\t\" Set the value at aKey of the receiver. \"\r\n\r\n\t\"(self typeOf: aKey) = #string ifFalse: [ ^self error: 'Invalid key (must be aString).' ].\"\r\n\tsize := nil.\r\n\t^hash\tbasicAt: (self wrap: aKey)\r\n\t\tput: (self wrap: value)","(\r\nfunction(self,aKey,value)\r\n(function()\r\n\tlocal _s8_value=(nil)\r\n\tself['@size']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((function() local _1_=((self):wrap_(value)); (self['@hash'])[(self):wrap_(aKey)]=_1_; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"removeKey:ifAbsent:",0
,(
function(self,aKey,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):includesKey_(aKey)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):value())
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
	local _s8_value=(nil)
	self['@size']=_s8_value; return _s8_value
end)()
;
	self["@hash"][self:wrap_(aKey)]=nil
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"removing","removeKey: aKey ifAbsent: aBlock\r\n\t\" Remove the binding at aKey. \"\r\n\r\n\t(self includesKey: aKey) ifFalse: [ ^aBlock value ].\r\n\tsize := nil.\r\n{'\tdelete self[\"@hash\"][aKey]'\r\nlua'\tself[\"@hash\"][self:wrap_(aKey)]=nil'}","(\r\nfunction(self,aKey,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):includesKey_(aKey)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(nil)\r\n\tself['@size']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n\tself[\"@hash\"][self:wrap_(aKey)]=nil\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"size",0
,(
function(self)
(function() if ((((((self['@size']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=(((self):keys()):size())
	self['@size']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (self['@size'])
;
end
)
,"accessing","size\r\n\t\" Return the size of the receiver. \"\r\n\r\n\tsize isNil ifTrue: [ size := self keys size ].\r\n\t^size","(\r\nfunction(self)\r\n(function() if ((((((self['@size']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(((self):keys()):size())\r\n\tself['@size']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@size'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"at:ifAbsent:",0
,(
function(self,aKey,aBlock)
	local found = self["@hash"][self:wrap_(aKey)]
	if found~=nil
		then return self:unwrap_(found)
		else return aBlock()
	end
;
return (self)
;
end
)
,"accessing","at: aKey ifAbsent: aBlock\r\n\t\" Return the value at aKey or the result of evaluating aBlock. \"\r\n\r\n{'\tvar found = Object.prototype.hasOwnProperty.call(self[\"@hash\"],aKey);\r\n\tif(found) return self[\"@hash\"][aKey];\r\n\treturn aBlock()'\r\nlua'\tlocal found = self[\"@hash\"][self:wrap_(aKey)]\r\n\tif found~=nil\r\n\t\tthen return self:unwrap_(found)\r\n\t\telse return aBlock()\r\n\tend'}.","(\r\nfunction(self,aKey,aBlock)\r\n\tlocal found = self[\"@hash\"][self:wrap_(aKey)]\r\n\tif found~=nil\r\n\t\tthen return self:unwrap_(found)\r\n\t\telse return aBlock()\r\n\tend\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"includesKey:",0
,(
function(self,aKey)
return (self["@hash"][self:wrap_(aKey)]~=nil)
;
end
)
,"testing","includesKey: aKey\r\n\t\" Return true if the receiver's keys include aKey. \"\r\n\r\n\t^{'Object.prototype.hasOwnProperty.call(self[\"@hash\"],aKey)'\r\n\tlua'self[\"@hash\"][self:wrap_(aKey)]~=nil'}","(\r\nfunction(self,aKey)\r\nreturn (self[\"@hash\"][self:wrap_(aKey)]~=nil)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"keysAndValuesDo:",0
,(
function(self,aBlock)
((self):keys()):do_((function(key)
local value=nil
(function()
	local _s8_value=((self):unwrap_(((self['@hash'])[key])))
	value=_s8_value; return _s8_value
end)();return ((aBlock):value_value_(key, value))
end))
;
return (self)
;
end
)
,"enumerating","keysAndValuesDo: aBlock\r\n\t\" Evaluate aBlock for key/value pairs of the receiver. \"\r\n\r\n\tself keys do: [:key | | value |\r\n\t\tvalue := self unwrap: (hash basicAt: key).\r\n\t\taBlock value: key value: value\r\n\t]","(\r\nfunction(self,aBlock)\r\n((self):keys()):do_((function(key)\r\nlocal value=nil\r\n(function()\r\n\tlocal _s8_value=((self):unwrap_(((self['@hash'])[key])))\r\n\tvalue=_s8_value; return _s8_value\r\nend)();return ((aBlock):value_value_(key, value))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"keys",0
,(
function(self)
return ((function()
		local result=smalltalk:newArray()
		for k,_ in pairs(self["@hash"]) do result[#result+1]=k end
		return result
	end)())
;
end
)
,"accessing","keys\r\n\t\" Return the keys of the receiver. \"\r\n\r\n\t^{'Object.getOwnPropertyNames(self[\"@hash\"])'\r\n\tlua'(function()\r\n\t\tlocal result=smalltalk:newArray()\r\n\t\tfor k,_ in pairs(self[\"@hash\"]) do result[#result+1]=k end\r\n\t\treturn result\r\n\tend)()'}","(\r\nfunction(self)\r\nreturn ((function()\r\n\t\tlocal result=smalltalk:newArray()\r\n\t\tfor k,_ in pairs(self[\"@hash\"]) do result[#result+1]=k end\r\n\t\treturn result\r\n\tend)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"values",0
,(
function(self)
return (((self):keys()):collect_((function(key)
return ((self):unwrap_(((self['@hash'])[key])))
end)))
;
end
)
,"accessing","values\r\n\t\" Return the values of the receiver. \"\r\n\r\n\t^self keys collect: [:key| self unwrap: (hash basicAt: key) ]","(\r\nfunction(self)\r\nreturn (((self):keys()):collect_((function(key)\r\nreturn ((self):unwrap_(((self['@hash'])[key])))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"asNative:",0
,(
function(self,lang)
local send=nil
(function()
	local _s8_value=((function() if (((((lang)=="lua")))==true) then return ((function()
return (":")
end))() else return ((function()
return (".")
end))() end end)())
	send=_s8_value; return _s8_value
end)()
;
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk."),((self)["$klass"]):name())),send)),"fromTuples_(")),((self):tuples()):asNative_(lang))),")"))
;
end
)
,"converting","asNative: lang\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\t| send |\r\n\tsend := lang == #lua ifTrue: [$:] ifFalse: [$.].\r\n\t^'smalltalk.' ,self class name\r\n\t\t,send ,'fromTuples_('\r\n\t\t,(self tuples asNative: lang)\r\n\t\t,')'","(\r\nfunction(self,lang)\r\nlocal send=nil\r\n(function()\r\n\tlocal _s8_value=((function() if (((((lang)==\"lua\")))==true) then return ((function()\r\nreturn (\":\")\r\nend))() else return ((function()\r\nreturn (\".\")\r\nend))() end end)())\r\n\tsend=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.\"),((self)[\"$klass\"]):name())),send)),\"fromTuples_(\")),((self):tuples()):asNative_(lang))),\")\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"detect:ifNone:",0
,(
function(self,aBlock,anotherBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(self):keysAndValuesDo_((function(key,value)
return ((function() if ((((aBlock):value_(value)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (value)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((anotherBlock):value())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"enumerating","detect: aBlock ifNone: anotherBlock\r\n\t\" Detect the value who matches aBlock.\r\n\tReturn the result of evaluating aBlock if no value matches detection block.\r\n\t\"\r\n\r\n\tself keysAndValuesDo: [:key :value |\r\n\t\t(aBlock value: value) ifTrue: [ ^value ]\r\n\t].\r\n\t^anotherBlock value","(\r\nfunction(self,aBlock,anotherBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(self):keysAndValuesDo_((function(key,value)\r\nreturn ((function() if ((((aBlock):value_(value)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (value)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((anotherBlock):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"do:",0
,(
function(self,aBlock)
(self):keysAndValuesDo_((function(key,value)
return ((aBlock):value_(value))
end))
;
return (self)
;
end
)
,"enumerating","do: aBlock\r\n\t\" Evaluates aBlock for all values in the receiver. \"\r\n\r\n\tself keysAndValuesDo: [:key :value | aBlock value: value ]","(\r\nfunction(self,aBlock)\r\n(self):keysAndValuesDo_((function(key,value)\r\nreturn ((aBlock):value_(value))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"includes:",0
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(self):keysAndValuesDo_((function(key,value)
return ((function() if ((((smalltalk:isObject_equalTo_((anObject),value))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"testing","includes: anObject\r\n\t\" Return true if anObject is in the receiver. \"\r\n\r\n\tself keysAndValuesDo: [:key :value |\r\n\t\tanObject = value ifTrue: [ ^true ].\r\n\t].\r\n\t^false","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(self):keysAndValuesDo_((function(key,value)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((anObject),value))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"keysDo:",0
,(
function(self,aBlock)
((self):keys()):do_(aBlock)
;
return (self)
;
end
)
,"enumerating","keysDo: aBlock\r\n\t\" Evaluate aBlock with the keys of the receiver. \"\r\n\r\n\tself keys do: aBlock","(\r\nfunction(self,aBlock)\r\n((self):keys()):do_(aBlock)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"@","_at"
,(
function(self,keyOrPath)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function() if ((((keyOrPath):isArray()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):at_ifAbsent_(keyOrPath, (function()
return (nil)
end)))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
	local _s8_value=((keyOrPath):inject_into_(self, (function(pool,key)
local value=nil
(function() if ((((((((function()
	local _s8_value=((pool):_at(key))
	value=_s8_value; return _s8_value
end)())))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return (value)
end)))
	result=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","@ keyOrPath\r\n\t\" Returns the result of searching the receiver at key or path (or nil). \"\r\n\r\n\t| result |\r\n\tkeyOrPath isArray ifFalse: [ ^self at: keyOrPath ifAbsent: [] ].\r\n\tresult := keyOrPath inject: self into: [:pool :key | | value |\r\n\t\t(value := pool @ key) isNil ifTrue: [ ^nil ].\r\n\t\tvalue\r\n\t].\r\n\t^result","(\r\nfunction(self,keyOrPath)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function() if ((((keyOrPath):isArray()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):at_ifAbsent_(keyOrPath, (function()\r\nreturn (nil)\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((keyOrPath):inject_into_(self, (function(pool,key)\r\nlocal value=nil\r\n(function() if ((((((((function()\r\n\tlocal _s8_value=((pool):_at(key))\r\n\tvalue=_s8_value; return _s8_value\r\nend)())))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return (value)\r\nend)))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionary,"asLua",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk."),((self)["$klass"]):name())),":fromTuples_(")),((self):tuples()):asLua())),")"))
;
end
)
,"Lua-lua","asLua\r\n\t\" Return the native representation of the receiver. \"\r\n\r\n\t^'smalltalk.',self class name\r\n\t, ':fromTuples_('\r\n\t\t, self tuples asLua\r\n\t,')'","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.\"),((self)[\"$klass\"]):name())),\":fromTuples_(\")),((self):tuples()):asLua())),\")\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.RegularExpression["$klass"],"fromString:flag:",0
,(
function(self,aString,anotherString)
return (self:notImplementedYet_("fromString:flag:"))
;
end
)
,"instantiation","fromString: aString flag: anotherString\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^{'new RegExp(aString, anotherString)'\r\n\tlua'self:notImplementedYet_(\"fromString:flag:\")'}","(\r\nfunction(self,aString,anotherString)\r\nreturn (self:notImplementedYet_(\"fromString:flag:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.RegularExpression["$klass"],"fromString:",0
,(
function(self,aString)
return ((self):fromString_flag_(aString, ""))
;
end
)
,"instantiation","fromString: aString\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t    ^self fromString: aString flag: ''","(\r\nfunction(self,aString)\r\nreturn ((self):fromString_flag_(aString, \"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.RegularExpression,"compile:",0
,(
function(self,aString)
return ((self):compile(aString))
;
end
)
,"evaluating","compile: aString\r\n\t\" Return the result of compilation of aString. \"\r\n\r\n\t^self #compile: aString","(\r\nfunction(self,aString)\r\nreturn ((self):compile(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.RegularExpression,"exec:",0
,(
function(self,aString)
return ((self):exec(aString))
;
end
)
,"evaluating","exec: aString\r\n\t\" Return the result of excecution of aString. \"\r\n\r\n\t^self #exec: aString","(\r\nfunction(self,aString)\r\nreturn ((self):exec(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.RegularExpression,"test:",0
,(
function(self,aString)
return ((self):test(aString))
;
end
)
,"evaluating","test: aString\r\n\t\" Return the result of testing of aString. \"\r\n\r\n\t^self #test: aString","(\r\nfunction(self,aString)\r\nreturn ((self):test(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.RegularExpression,"shallowCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.RegularExpression,"deepCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.RegularExpression,"asJSONObject",0
,(
function(self)
return (self)
;
end
)
,"json","asJSONObject\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exception,"nativeException",0
,(
function(self)
return (((self)["@nativeException"]))
;
end
)
,"private","nativeException\r\n\t\" Private, Internal - Return the native exception associated with the receiver (or nil). \"\r\n\r\n\t^self basicAt: '@nativeException'","(\r\nfunction(self)\r\nreturn (((self)[\"@nativeException\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exception,"nativeException:",0
,(
function(self,nex)
return ((function() local _1_=(nex); (self)["@nativeException"]=_1_; return _1_ end)())
;
end
)
,"private","nativeException: nex\r\n\t\" Private, Internal - Set the native exception associated with the receiver. \"\r\n\r\n\t^self basicAt: '@nativeException' put: nex","(\r\nfunction(self,nex)\r\nreturn ((function() local _1_=(nex); (self)[\"@nativeException\"]=_1_; return _1_ end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exception,"stack",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local nex=nil
(function()
	local _s8_value=((self):nativeException())
	nex=_s8_value; return _s8_value
end)()
;
(function() if ((((((nex))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((nex["stack"]) or ""))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"stack","stack\r\n\t\" Return a stack trace string. \"\r\n\r\n\t| nex |\r\n\tnex := self nativeException.\r\n\tnex isNil ifTrue: [ ^'' ].\r\n\t^nex#stack || ''","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal nex=nil\r\n(function()\r\n\tlocal _s8_value=((self):nativeException())\r\n\tnex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((nex))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((nex[\"stack\"]) or \"\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Exception,"toString",0
,(
function(self)
local txt=nil
local stk=nil
(function()
	local _s8_value=((self):messageText())
	txt=_s8_value; return _s8_value
end)()
;
(function() if ((((((txt))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=("")
	txt=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((self):stack())
	stk=_s8_value; return _s8_value
end)()
;
(function() if ((((((stk))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=("")
	stk=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((function()
	local _s8_value=(smalltalk:object_comma_(("\r\n"),stk))
	stk=_s8_value; return _s8_value
end)())
end))() end end)()
;
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("{st:a"),((self)["$klass"]):name()))," - ")),txt)),"}")),stk))
;
end
)
,"converting","toString\r\n\t\" Convert the receiver to a String. \"\r\n\r\n\t| txt stk |\r\n\ttxt := self messageText. txt isNil ifTrue: [ txt := '' ].\r\n\tstk := self stack.\r\n\tstk isNil ifTrue: [ stk := '' ] ifFalse: [ stk := '\r\n',stk ].\r\n\t^'{st:a' ,self class name ,' - ' ,txt ,$} ,stk","(\r\nfunction(self)\r\nlocal txt=nil\r\nlocal stk=nil\r\n(function()\r\n\tlocal _s8_value=((self):messageText())\r\n\ttxt=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((txt))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(\"\")\r\n\ttxt=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):stack())\r\n\tstk=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((stk))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(\"\")\r\n\tstk=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((\"\\r\\n\"),stk))\r\n\tstk=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)()\r\n;\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"{st:a\"),((self)[\"$klass\"]):name())),\" - \")),txt)),\"}\")),stk))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exception,"messageText",0
,(
function(self)
return (self['@messageText'])
;
end
)
,"accessing","messageText\r\n\t\" Return the messageText of the receiver. \"\r\n\r\n\t^messageText","(\r\nfunction(self)\r\nreturn (self['@messageText'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exception,"description",0
,(
function(self)
return (smalltalk:toString_(((self):messageText())))
;
end
)
,"accessing","description\r\n\t\" Return the description of the receiver. \"\r\n\r\n\t^self messageText toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_(((self):messageText())))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exception,"messageText:",0
,(
function(self,aString)
(function()
	local _s8_value=(aString)
	self['@messageText']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","messageText: aString\r\n\t\" Set the messageText of the receiver. \"\r\n\r\n\tmessageText := aString","(\r\nfunction(self,aString)\r\n(function()\r\n\tlocal _s8_value=(aString)\r\n\tself['@messageText']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exception,"signal",0
,(
function(self)
(self):note_(smalltalk:object_comma_((smalltalk:object_comma_((((self)["$klass"]):name()),": ")),(self):description()))
;
	error(self)
;
return (self)
;
end
)
,"signaling","signal\r\n\t\" Signal/throw the receiver. \"\r\n\r\n\tself note: self class name ,': ' ,self description.\r\n{'\tthrow(self)'\r\nlua'\terror(self)'}","(\r\nfunction(self)\r\n(self):note_(smalltalk:object_comma_((smalltalk:object_comma_((((self)[\"$klass\"]):name()),\": \")),(self):description()))\r\n;\r\n\terror(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exception,"__tostring",0
,(
function(self)
return (smalltalk:toString_((self)))
;
end
)
,"Lua-printing","__tostring\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Error["$klass"],"signal:",0
,(
function(self,aString)
return ((
function(_s8_rec)
(_s8_rec):messageText_(aString);
return ((_s8_rec):signal())
end
)((self):new()))
;
end
)
,"instantiation","signal: aString\r\n\t\" Signal an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tmessageText: aString;\r\n\t\tsignal","(\r\nfunction(self,aString)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):messageText_(aString);\r\nreturn ((_s8_rec):signal())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MessageNotUnderstood,"message",0
,(
function(self)
return (self['@message'])
;
end
)
,"accessing","message\r\n\t\" Return the message of the receiver. \"\r\n\r\n\t^message","(\r\nfunction(self)\r\nreturn (self['@message'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MessageNotUnderstood,"message:",0
,(
function(self,aMessage)
(function()
	local _s8_value=(aMessage)
	self['@message']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","message: aMessage\r\n\t\" Set the message of the receiver. \"\r\n\r\n\tmessage := aMessage","(\r\nfunction(self,aMessage)\r\n(function()\r\n\tlocal _s8_value=(aMessage)\r\n\tself['@message']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MessageNotUnderstood,"receiver",0
,(
function(self)
return (self['@receiver'])
;
end
)
,"accessing","receiver\r\n\t\" Return the receiver of the receiver. \"\r\n\r\n\t^receiver","(\r\nfunction(self)\r\nreturn (self['@receiver'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MessageNotUnderstood,"receiver:",0
,(
function(self,anObject)
(function()
	local _s8_value=(anObject)
	self['@receiver']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","receiver: anObject\r\n\t\" Set the receiver of the receiver. \"\r\n\r\n\treceiver := anObject","(\r\nfunction(self,anObject)\r\n(function()\r\n\tlocal _s8_value=(anObject)\r\n\tself['@receiver']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MessageNotUnderstood,"messageTextOBSOLETE",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:toString_(((self):receiver())))," does not understand #")),((self):message()):selector()))
;
end
)
,"accessing","messageTextOBSOLETE\r\n\t\" Return the message text of the receiver. \"\r\n\r\n\t^self receiver toString, ' does not understand #', self message selector","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:toString_(((self):receiver()))),\" does not understand #\")),((self):message()):selector()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MessageNotUnderstood,"messageText",0
,(
function(self)
local type=nil
(function()
	local _s8_value=((function() if ((((self):isObject_((self):receiver())))==true) then return ((function()
return (smalltalk:object_comma_(("instance of "),(((self):receiver())["$klass"]):name()))
end))() else return ((function()
return ("native object")
end))() end end)())
	type=_s8_value; return _s8_value
end)()
;
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("#"),((self):message()):selector()))," not understood by ")),type))
;
end
)
,"accessing","messageText\r\n\t\" Return the message text of the receiver. \"\r\n\r\n\t| type |\r\n\ttype := (self isObject: self receiver)\r\n\t\tifTrue: [ 'instance of ' ,self receiver class name ]\r\n\t\tifFalse: [ 'native object' ].\r\n\t^$# ,self message selector ,' not understood by ' ,type","(\r\nfunction(self)\r\nlocal type=nil\r\n(function()\r\n\tlocal _s8_value=((function() if ((((self):isObject_((self):receiver())))==true) then return ((function()\r\nreturn (smalltalk:object_comma_((\"instance of \"),(((self):receiver())[\"$klass\"]):name()))\r\nend))() else return ((function()\r\nreturn (\"native object\")\r\nend))() end end)())\r\n\ttype=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"#\"),((self):message()):selector())),\" not understood by \")),type))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association["$klass"],"key:",0
,(
function(self,aKey)
return (((self):new()):key_(aKey))
;
end
)
,"instantiation","key: aKey\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new key: aKey","(\r\nfunction(self,aKey)\r\nreturn (((self):new()):key_(aKey))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association["$klass"],"key:value:",0
,(
function(self,aKey,aValue)
return (((self):key_(aKey)):value_(aValue))
;
end
)
,"instantiation","key: aKey value: aValue\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^(self key: aKey) value: aValue","(\r\nfunction(self,aKey,aValue)\r\nreturn (((self):key_(aKey)):value_(aValue))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association,"=","_eq"
,(
function(self,anAssociation)
return ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((self)["$klass"]),(anAssociation)["$klass"]))),(function()
return ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((self):key()),(anAssociation):key()))),(function()
return ((smalltalk:isObject_equalTo_(((self):value()),(anAssociation):value())))
end))))
end))))
;
end
)
,"comparing","= anAssociation\r\n\t\" Return true if the receiver is equal to argument. \"\r\n\r\n\t^self class = anAssociation class and: [\r\n\t    self key = anAssociation key and: [\r\n\t\tself value = anAssociation value]]","(\r\nfunction(self,anAssociation)\r\nreturn ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((self)[\"$klass\"]),(anAssociation)[\"$klass\"]))),(function()\r\nreturn ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((self):key()),(anAssociation):key()))),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((self):value()),(anAssociation):value())))\r\nend))))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association,"key:",0
,(
function(self,aKey)
(function()
	local _s8_value=(aKey)
	self['@key']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","key: aKey\r\n\t\" Set the key of the receiver. \"\r\n\r\n\tkey := aKey","(\r\nfunction(self,aKey)\r\n(function()\r\n\tlocal _s8_value=(aKey)\r\n\tself['@key']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association,"key",0
,(
function(self)
return (self['@key'])
;
end
)
,"accessing","key\r\n\t\" Return the key of the receiver. \"\r\n\r\n\t^key","(\r\nfunction(self)\r\nreturn (self['@key'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association,"value:",0
,(
function(self,aValue)
(function()
	local _s8_value=(aValue)
	self['@value']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","value: aValue\r\n\t\" Set the value of the receiver. \"\r\n\r\n\tvalue := aValue","(\r\nfunction(self,aValue)\r\n(function()\r\n\tlocal _s8_value=(aValue)\r\n\tself['@value']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association,"value",0
,(
function(self)
return (self['@value'])
;
end
)
,"accessing","value\r\n\t\" Return the value of the receiver. \"\r\n\r\n\t^value","(\r\nfunction(self)\r\nreturn (self['@value'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association,",","_comma"
,(
function(self,anAssociation)
return ((smalltalk.ComposedAssociations):with_with_(self, anAssociation))
;
end
)
,"composing",", anAssociation\r\n\t\" Return the receiver composed with the argument. \"\r\n\r\n\t^ComposedAssociations with: self with: anAssociation","(\r\nfunction(self,anAssociation)\r\nreturn ((smalltalk.ComposedAssociations):with_with_(self, anAssociation))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association,"json",0
,(
function(self)
return (((smalltalk.ComposedAssociations):with_(self)):json())
;
end
)
,"converting","json\r\n\t\" Return the receiver as a hash object. \"\r\n\r\n\t^(ComposedAssociations with: self) json","(\r\nfunction(self)\r\nreturn (((smalltalk.ComposedAssociations):with_(self)):json())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association,"isAssociation",0
,(
function(self)
return (true)
;
end
)
,"testing","isAssociation\r\n\t\" Return true if the receiver is anAssociation. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association,"printString",0
,(
function(self)
local literal=nil
(function()
	local _s8_value=((function() if ((((self):isObject_((self):key())))==true) then return ((function()
return (((self):key()):asLiteral())
end))() else return ((function()
return (smalltalk:toString_(((self):key())))
end))() end end)())
	literal=_s8_value; return _s8_value
end)()
;
return (smalltalk:object_comma_((smalltalk:object_comma_((literal)," -> ")),smalltalk:printString_(((self):value()))))
;
end
)
,"printing","printString\r\n\t\" Return a printable representation of the receiver. \"\r\n\r\n\t| literal |\r\n\tliteral := (self isObject: self key)\r\n\t\tifTrue: [ self key asLiteral ]\r\n\t\tifFalse: [ self key toString ].\r\n\t^literal ,' -> ' ,self value printString","(\r\nfunction(self)\r\nlocal literal=nil\r\n(function()\r\n\tlocal _s8_value=((function() if ((((self):isObject_((self):key())))==true) then return ((function()\r\nreturn (((self):key()):asLiteral())\r\nend))() else return ((function()\r\nreturn (smalltalk:toString_(((self):key())))\r\nend))() end end)())\r\n\tliteral=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((literal),\" -> \")),smalltalk:printString_(((self):value()))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association,"toString",0
,(
function(self)
local literal=nil
(function()
	local _s8_value=((function() if ((((self):isObject_((self):key())))==true) then return ((function()
return (((self):key()):asLiteral())
end))() else return ((function()
return (smalltalk:toString_(((self):key())))
end))() end end)())
	literal=_s8_value; return _s8_value
end)()
;
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("("),literal))," -> ")),smalltalk:toString_(((self):value())))),")"))
;
end
)
,"printing","toString\r\n\t\" Return a printable representation of the receiver. \"\r\n\r\n\t| literal |\r\n\tliteral := (self isObject: self key)\r\n\t\tifTrue: [ self key asLiteral ]\r\n\t\tifFalse: [ self key toString ].\r\n\t^'(', literal ,' -> ' ,self value toString ,')'","(\r\nfunction(self)\r\nlocal literal=nil\r\n(function()\r\n\tlocal _s8_value=((function() if ((((self):isObject_((self):key())))==true) then return ((function()\r\nreturn (((self):key()):asLiteral())\r\nend))() else return ((function()\r\nreturn (smalltalk:toString_(((self):key())))\r\nend))() end end)())\r\n\tliteral=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"(\"),literal)),\" -> \")),smalltalk:toString_(((self):value())))),\")\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Association,"__tostring",0
,(
function(self)
return (smalltalk:toString_((self)))
;
end
)
,"Lua-printing","__tostring\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassBuilder,"superclass:subclass:",0
,(
function(self,aClass,aString)
return ((self):superclass_subclass_instanceVariableNames_category_(aClass, aString, "", nil))
;
end
)
,"instantiation","superclass: aClass subclass: aString\r\n\t\" Create/Register a new class in the system. \"\r\n\r\n\t^self\r\n\t\tsuperclass: aClass\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: ''\r\n\t\tcategory: nil","(\r\nfunction(self,aClass,aString)\r\nreturn ((self):superclass_subclass_instanceVariableNames_category_(aClass, aString, \"\", nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassBuilder,"superclass:subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:",0
,(
function(self,aClass,aString,aString2,classVarNames,poolVarNames,aString3)
local result=nil
local aCategory=nil
(function()
	local _s8_value=((function() if ((((((aString3))==nil)))==true) then return ((function()
return ((function() if ((((((aClass))==nil)))==true) then return ((function()
return ("unclassified")
end))() else return ((function()
return ((aClass):category())
end))() end end)())
end))() else return ((function()
return (aString3)
end))() end end)())
	aCategory=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):addSubclassOf_named_instanceVariableNames_(aClass, aString, (self):instanceVariableNamesFor_(aString2)))
	result=_s8_value; return _s8_value
end)()
;
(self):setupClass_(result)
;
(
function(_s8_rec)
(_s8_rec):classVariableNames_(classVarNames);
(_s8_rec):poolDictionaryNames_(poolVarNames);
return ((_s8_rec):category_(aCategory))
end
)(result)
;
(function() if ((((smalltalk:boolean_and_(((((aClass))~=nil)),(function()
return ((smalltalk.Smalltalk):includesKey_("Exporter"))
end)))))==true) then return ((function()
return ((self):print_(((smalltalk.Exporter):new()):exportNew_(result)))
end))() else return nil end end)()
;
return (result)
;
end
)
,"instantiation","superclass: aClass\r\n\tsubclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tclassVariableNames: classVarNames\r\n\tpoolDictionaries: poolVarNames\r\n\tcategory: aString3\r\n\t\" Create/Register a new class in the system. \"\r\n\r\n\t| result aCategory |\r\n\taCategory := aString3 isNil ifTrue: [\r\n\t\t\taClass isNil ifTrue: [ #unclassified ]\r\n\t\t\tifFalse: [ aClass category ]\r\n\t\t] ifFalse: [ aString3 ].\r\n\tresult := self\r\n\t\taddSubclassOf: aClass named: aString\r\n\t\tinstanceVariableNames: (self instanceVariableNamesFor: aString2).\r\n\tself setupClass: result.\r\n\tresult \r\n\t\tclassVariableNames: classVarNames;\r\n\t\tpoolDictionaryNames: poolVarNames;\r\n\t\tcategory: aCategory.\r\n\t(aClass notNil and: [ Smalltalk includesKey: #Exporter ]) ifTrue: [\r\n\t\tself print: (Exporter new exportNew: result)\r\n\t].\r\n\t^result","(\r\nfunction(self,aClass,aString,aString2,classVarNames,poolVarNames,aString3)\r\nlocal result=nil\r\nlocal aCategory=nil\r\n(function()\r\n\tlocal _s8_value=((function() if ((((((aString3))==nil)))==true) then return ((function()\r\nreturn ((function() if ((((((aClass))==nil)))==true) then return ((function()\r\nreturn (\"unclassified\")\r\nend))() else return ((function()\r\nreturn ((aClass):category())\r\nend))() end end)())\r\nend))() else return ((function()\r\nreturn (aString3)\r\nend))() end end)())\r\n\taCategory=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):addSubclassOf_named_instanceVariableNames_(aClass, aString, (self):instanceVariableNamesFor_(aString2)))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):setupClass_(result)\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):classVariableNames_(classVarNames);\r\n(_s8_rec):poolDictionaryNames_(poolVarNames);\r\nreturn ((_s8_rec):category_(aCategory))\r\nend\r\n)(result)\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((((aClass))~=nil)),(function()\r\nreturn ((smalltalk.Smalltalk):includesKey_(\"Exporter\"))\r\nend)))))==true) then return ((function()\r\nreturn ((self):print_(((smalltalk.Exporter):new()):exportNew_(result)))\r\nend))() else return nil end end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassBuilder,"superclass:subclass:instanceVariableNames:category:",0
,(
function(self,aClass,aString,aString2,aString3)
return ((self):superclass_subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aClass, aString, aString2, "", "", aString3))
;
end
)
,"instantiation","superclass: aClass\r\n\tsubclass: aString\r\n\tinstanceVariableNames: aString2\r\n\tcategory: aString3\r\n\t\" Create/Register a new class in the system. \"\r\n\r\n\t^self\r\n\t\tsuperclass: aClass\r\n\t\tsubclass: aString\r\n\t\tinstanceVariableNames: aString2\r\n\t\tclassVariableNames: ''\r\n\t\tpoolDictionaries: ''\r\n\t\tcategory: aString3","(\r\nfunction(self,aClass,aString,aString2,aString3)\r\nreturn ((self):superclass_subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aClass, aString, aString2, \"\", \"\", aString3))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassBuilder,"basicClass:instanceVariableNames:",0
,(
function(self,aMetaClass,anArray)
return (((self):smalltalk()):metaVarNames(aMetaClass, anArray))
;
end
)
,"private","basicClass: aMetaClass instanceVariableNames: anArray\r\n\t\" Private - Register instance variables onto aMetaClass. \"\r\n\r\n\t^self smalltalk #metaVarNames: aMetaClass array: anArray","(\r\nfunction(self,aMetaClass,anArray)\r\nreturn (((self):smalltalk()):metaVarNames(aMetaClass, anArray))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassBuilder,"class:instanceVariableNames:",0
,(
function(self,aMetaClass,aString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local varNames=nil
(function() if ((((aMetaClass):isMetaclass()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_(smalltalk:object_comma_(((aMetaClass):name())," is not a metaclass")))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
	local _s8_value=((self):instanceVariableNamesFor_(aString))
	varNames=_s8_value; return _s8_value
end)()
;
(function() if ((((self):basicClass_instanceVariableNames_(aMetaClass, varNames)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(self):setupClass_(aMetaClass)
;
(function() if ((((smalltalk.Smalltalk):includesKey_("Exporter")))==true) then return ((function()
return ((self):print_(((smalltalk.Exporter):new()):exportMetaDefinitionOf_((aMetaClass):instanceClass())))
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"metaclass","class: aMetaClass instanceVariableNames: aString\r\n\t\" Register instance variable names onto aMetaClass. \"\r\n\r\n\t| varNames |\r\n\taMetaClass isMetaclass ifFalse: [\r\n\t\t^self error: aMetaClass name, ' is not a metaclass'\r\n\t].\r\n\tvarNames := self instanceVariableNamesFor: aString.\r\n\t(self basicClass: aMetaClass instanceVariableNames: varNames) ifFalse: [ ^self ].\r\n\tself setupClass: aMetaClass.\r\n\t(Smalltalk includesKey: #Exporter) ifTrue: [\r\n\t\tself print: (Exporter new exportMetaDefinitionOf: aMetaClass instanceClass)\r\n\t].","(\r\nfunction(self,aMetaClass,aString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal varNames=nil\r\n(function() if ((((aMetaClass):isMetaclass()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(smalltalk:object_comma_(((aMetaClass):name()),\" is not a metaclass\")))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):instanceVariableNamesFor_(aString))\r\n\tvarNames=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((self):basicClass_instanceVariableNames_(aMetaClass, varNames)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(self):setupClass_(aMetaClass)\r\n;\r\n(function() if ((((smalltalk.Smalltalk):includesKey_(\"Exporter\")))==true) then return ((function()\r\nreturn ((self):print_(((smalltalk.Exporter):new()):exportMetaDefinitionOf_((aMetaClass):instanceClass())))\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassBuilder,"instanceVariableNamesFor:",0
,(
function(self,aString)
return ((aString):asArrayOfSubstrings())
;
end
)
,"private","instanceVariableNamesFor: aString\r\n\t\" Private - Return the instance variable names string. \"\r\n\r\n\t^aString asArrayOfSubstrings","(\r\nfunction(self,aString)\r\nreturn ((aString):asArrayOfSubstrings())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassBuilder,"addSubclassOf:named:instanceVariableNames:",0
,(
function(self,aClass,aString,aCollection)
	smalltalk:addClass(aString, aClass, aCollection)
	if true then return smalltalk[aString] end
;
return (self)
;
end
)
,"private","addSubclassOf: aClass named: aString instanceVariableNames: aCollection\r\n\t\" Private - Add the subclass to system. \"\r\n\r\n{'\tsmalltalk.addClass(aString, aClass, aCollection);\r\n\treturn smalltalk[aString]'\r\nlua'\tsmalltalk:addClass(aString, aClass, aCollection)\r\n\tif true then return smalltalk[aString] end'}","(\r\nfunction(self,aClass,aString,aCollection)\r\n\tsmalltalk:addClass(aString, aClass, aCollection)\r\n\tif true then return smalltalk[aString] end\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassBuilder,"setupClass:",0
,(
function(self,aClass)
return (self)
;
end
)
,"private","setupClass: aClass\r\n\t\" Private - Bind/initialize the class delegation chain. \"","(\r\nfunction(self,aClass)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream["$klass"],"on:",0
,(
function(self,aCollection)
return ((
function(_s8_rec)
(_s8_rec):collection_(aCollection);
(_s8_rec):readLimit_((aCollection):size());
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","on: aCollection\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new \r\n\t\tcollection: aCollection;\r\n\t\treadLimit: aCollection size;\r\n\t\tyourself","(\r\nfunction(self,aCollection)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):collection_(aCollection);\r\n(_s8_rec):readLimit_((aCollection):size());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"collection",0
,(
function(self)
return (self['@collection'])
;
end
)
,"accessing","collection\r\n\t\" Private - Return the collection of the receiver. \"\r\n\r\n\t^collection","(\r\nfunction(self)\r\nreturn (self['@collection'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"collection:",0
,(
function(self,aCollection)
(function()
	local _s8_value=(aCollection)
	self['@collection']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","collection: aCollection\r\n\t\" Private - Set the collection of the receiver. \"\r\n\r\n\tcollection := aCollection","(\r\nfunction(self,aCollection)\r\n(function()\r\n\tlocal _s8_value=(aCollection)\r\n\tself['@collection']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"setCollection:",0
,(
function(self,aCollection)
(self):collection_(aCollection)
;
return (self)
;
end
)
,"accessing","setCollection: aCollection\r\n\t\" Private - Set the receiver stream collection to aCollection. \"\r\n\r\n\tself collection: aCollection","(\r\nfunction(self,aCollection)\r\n(self):collection_(aCollection)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"initialize",0
,(
function(self)
smalltalk:superImplementor(smalltalk.Stream,"initialize")(self)
;
(function()
	local _s8_value=((0))
	self['@position']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"initialize","initialize\r\n\t\" Private - Initialize the receiver. \"\r\n\r\n\tsuper initialize.\r\n\tposition := 0.","(\r\nfunction(self)\r\nsmalltalk:superImplementor(smalltalk.Stream,\"initialize\")(self)\r\n;\r\n(function()\r\n\tlocal _s8_value=((0))\r\n\tself['@position']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"position",0
,(
function(self)
return (self['@position'])
;
end
)
,"accessing","position\r\n\t\" Return the position of the receiver. \"\r\n\r\n\t^position","(\r\nfunction(self)\r\nreturn (self['@position'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"position:",0
,(
function(self,anInteger)
(function()
	local _s8_value=(anInteger)
	self['@position']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","position: anInteger\r\n\t\" Set the position of the receiver. \"\r\n\r\n\tposition := anInteger","(\r\nfunction(self,anInteger)\r\n(function()\r\n\tlocal _s8_value=(anInteger)\r\n\tself['@position']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"readLimit",0
,(
function(self)
(function() if (((((self['@readLimit'])==nil)))==true) then return ((function()
return ((function()
	local _s8_value=(((self):collection()):size())
	self['@readLimit']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (self['@readLimit'])
;
end
)
,"accessing","readLimit\r\n\t\" Private - Return the read limit of the receiver. \"\r\n\r\n\treadLimit == nil ifTrue: [ readLimit := self collection size ].\r\n\t^readLimit","(\r\nfunction(self)\r\n(function() if (((((self['@readLimit'])==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(((self):collection()):size())\r\n\tself['@readLimit']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@readLimit'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"readLimit:",0
,(
function(self,anInteger)
(function()
	local _s8_value=(anInteger)
	self['@readLimit']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","readLimit: anInteger\r\n\t\" Private - Set the stream size of the receiver. \"\r\n\r\n\treadLimit := anInteger","(\r\nfunction(self,anInteger)\r\n(function()\r\n\tlocal _s8_value=(anInteger)\r\n\tself['@readLimit']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"contents",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function()
	local _s8_value=((self):collection())
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((self):readLimit()),(result):size()))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((result):copyFrom_to_((1), (self):readLimit()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","contents\r\n\t\" Return the contents of the receiver. \"\r\n\r\n\t| result |\r\n\tresult := self collection.\r\n\tself readLimit = result size ifTrue: [ ^result ].\r\n\t^result copyFrom: 1 to: self readLimit","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):collection())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):readLimit()),(result):size()))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((result):copyFrom_to_((1), (self):readLimit()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"size",0
,(
function(self)
return ((self):readLimit())
;
end
)
,"accessing","size\r\n\t\" Return the size of the receiver. \"\r\n\r\n\t^self readLimit","(\r\nfunction(self)\r\nreturn ((self):readLimit())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"reset",0
,(
function(self)
(self):position_((0))
;
return (self)
;
end
)
,"actions","reset\r\n\t\" Reset the receiver. \"\r\n\r\n\tself position: 0","(\r\nfunction(self)\r\n(self):position_((0))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"close",0
,(
function(self)
return (self)
;
end
)
,"actions","close\r\n\t\" Close the receiver. \"","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"flush",0
,(
function(self)
return (self)
;
end
)
,"io","flush\r\n\t\" Flush the receiver. \"","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"resetContents",0
,(
function(self)
(
function(_s8_rec)
(_s8_rec):reset();
return ((_s8_rec):readLimit_((0)))
end
)(self)
;
return (self)
;
end
)
,"actions","resetContents\r\n\t\" Reset the receiver contents. \"\r\n\r\n\tself reset; readLimit: 0","(\r\nfunction(self)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):reset();\r\nreturn ((_s8_rec):readLimit_((0)))\r\nend\r\n)(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"do:",0
,(
function(self,aBlock)
(function() while(not ((function()
return ((self):atEnd())
end))()) do (function()
return ((aBlock):value_((self):next()))
end)() end;return nil end)()
;
return (self)
;
end
)
,"enumerating","do: aBlock\r\n\t\" Evaluates aBlock streaming on the receiver. \"\r\n\r\n\t[ self atEnd ] whileFalse: [ aBlock value: self next ]","(\r\nfunction(self,aBlock)\r\n(function() while(not ((function()\r\nreturn ((self):atEnd())\r\nend))()) do (function()\r\nreturn ((aBlock):value_((self):next()))\r\nend)() end;return nil end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"setToEnd",0
,(
function(self)
(self):position_((self):size())
;
return (self)
;
end
)
,"positioning","setToEnd\r\n\t\" Position the receiver at end. \"\r\n\r\n\tself position: self size","(\r\nfunction(self)\r\n(self):position_((self):size())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"skip:",0
,(
function(self,anInteger)
(self):position_((((anInteger):_plus((self):position())):min_((self):size())):max_((0)))
;
return (self)
;
end
)
,"positioning","skip: anInteger\r\n\t\" Skip next anInteger readable elements in the receiver. \"\r\n\r\n\tself position: ((anInteger + self position min: self size) max: 0)","(\r\nfunction(self,anInteger)\r\n(self):position_((((anInteger):_plus((self):position())):min_((self):size())):max_((0)))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"next",0
,(
function(self)
(self):position_(((self):position()):_plus((1)))
;
return ((self['@collection']):at_((self):position()))
;
end
)
,"reading","next\r\n\t\" Return the next readable element in the receiver (or nil). \"\r\n\r\n\tself position: self position + 1. \r\n\t^collection at: self position","(\r\nfunction(self)\r\n(self):position_(((self):position()):_plus((1)))\r\n;\r\nreturn ((self['@collection']):at_((self):position()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"next:",0
,(
function(self,anInteger)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function()
	local _s8_value=((((self):collection())["$klass"]):new())
	result=_s8_value; return _s8_value
end)()
;
(function() for _1_=1,((anInteger)) do (function()
(function() if ((((self):atEnd()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((result):add_((self):next()))
end)(_1_) end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"reading","next: anInteger\r\n\t\" Return the next anInteger elements read from the receiver.\r\n\tIf the receiver reach its end, the returned collection has less elements.\r\n\t\"\r\n\r\n\t| result |\r\n\tresult := self collection class new.\r\n\tanInteger timesRepeat: [\r\n\t\tself atEnd ifTrue: [ ^result ].\r\n\t\tresult add: self next.\r\n\t].\r\n\t^result","(\r\nfunction(self,anInteger)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((((self):collection())[\"$klass\"]):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() for _1_=1,((anInteger)) do (function()\r\n(function() if ((((self):atEnd()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((result):add_((self):next()))\r\nend)(_1_) end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"nextPut:",0
,(
function(self,anObject)
(self):position_(((self):position()):_plus((1)))
;
((self):collection()):at_put_(self['@position'], anObject)
;
(self):readLimit_(((self):readLimit()):max_(self['@position']))
;
return (self)
;
end
)
,"writing","nextPut: anObject\r\n\t\" Put anObject onto the receiver. \"\r\n\r\n\tself position: self position + 1.\r\n\tself collection at: position put: anObject.\r\n\tself readLimit: (self readLimit max: position)","(\r\nfunction(self,anObject)\r\n(self):position_(((self):position()):_plus((1)))\r\n;\r\n((self):collection()):at_put_(self['@position'], anObject)\r\n;\r\n(self):readLimit_(((self):readLimit()):max_(self['@position']))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"nextPutAll:",0
,(
function(self,aCollection)
(aCollection):do_((function(each)
return ((self):nextPut_(each))
end))
;
return (self)
;
end
)
,"writing","nextPutAll: aCollection\r\n\t\" Put aCollection onto the receiver. \"\r\n\r\n\taCollection do: [:each | self nextPut: each ]","(\r\nfunction(self,aCollection)\r\n(aCollection):do_((function(each)\r\nreturn ((self):nextPut_(each))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"cr",0
,(
function(self)
(self):nextPutAll_((smalltalk.String):crlf())
;
return (self)
;
end
)
,"writing","cr\r\n\t\" Put a line break sequence onto the receiver.\r\n\tNote: this implementation MUST be consistent with #nextLine\r\n\t\"\r\n\r\n\tself nextPutAll: String crlf","(\r\nfunction(self)\r\n(self):nextPutAll_((smalltalk.String):crlf())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"space",0
,(
function(self)
(self):nextPutAll_((smalltalk.String):space())
;
return (self)
;
end
)
,"writing","space\r\n\t\" Put a space sequence onto the receiver. \"\r\n\r\n\tself nextPutAll: String space","(\r\nfunction(self)\r\n(self):nextPutAll_((smalltalk.String):space())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"tab",0
,(
function(self)
(self):nextPutAll_((smalltalk.String):tab())
;
return (self)
;
end
)
,"writing","tab\r\n\t\" Put a tab sequence onto the receiver. \"\r\n\r\n\tself nextPutAll: String tab","(\r\nfunction(self)\r\n(self):nextPutAll_((smalltalk.String):tab())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"peek",0
,(
function(self)
return ((function() if ((((self):atEnd()))==true) then return nil else return ((function()
return (((self):collection()):at_(((self):position()):_plus((1))))
end))() end end)())
;
end
)
,"reading","peek\r\n\t\" Return the next readable element from the receiver without advancing the receiver; or nil. \"\r\n\r\n\t^self atEnd ifFalse: [\r\n\t    self collection at: self position + 1\r\n\t]","(\r\nfunction(self)\r\nreturn ((function() if ((((self):atEnd()))==true) then return nil else return ((function()\r\nreturn (((self):collection()):at_(((self):position()):_plus((1))))\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"peekFor:",0
,(
function(self,anObject)
return ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((anObject),(self):peek()))),(function()
(function() if ((((self):atEnd()))==true) then return nil else return ((function()
return ((self):next())
end))() end end)();return (true)
end))))
;
end
)
,"reading","peekFor: anObject\r\n\t\" Return true if the next readable element from the receiver equals anObject.\r\n\tAdvance the receiver if the result is true.\r\n\t\"\r\n\r\n\t^anObject = self peek and: [\r\n\t\tself atEnd ifFalse: [ self next ].\r\n\t\ttrue\r\n\t]","(\r\nfunction(self,anObject)\r\nreturn ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((anObject),(self):peek()))),(function()\r\n(function() if ((((self):atEnd()))==true) then return nil else return ((function()\r\nreturn ((self):next())\r\nend))() end end)();return (true)\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"peekForAll:",0
,(
function(self,aCollection)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local back=nil
(function()
	local _s8_value=((self):position())
	back=_s8_value; return _s8_value
end)()
;
(aCollection):do_((function(each)
return ((function() if ((((self):peekFor_(each)))==true) then return nil else return ((function()
(self):position_(back);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"reading","peekForAll: aCollection\r\n\t\" Return true if the next readable elements from the receiver matches aCollection.\r\n\tAdvance the receiver if the result is true.\r\n\t\"\r\n\r\n\t| back |\r\n\tback := self position.\r\n\taCollection do: [:each|\r\n\t\t(self peekFor: each) ifFalse: [\r\n\t\t\tself position: back.\r\n\t\t\t^false\r\n\t\t].\r\n\t].\r\n\t^true","(\r\nfunction(self,aCollection)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal back=nil\r\n(function()\r\n\tlocal _s8_value=((self):position())\r\n\tback=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aCollection):do_((function(each)\r\nreturn ((function() if ((((self):peekFor_(each)))==true) then return nil else return ((function()\r\n(self):position_(back);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"atEnd",0
,(
function(self)
return ((((self):position()) >= (self):size()))
;
end
)
,"testing","atEnd\r\n\t\" Return true if the receiver is at end. \"\r\n\r\n\t^self position >= self size","(\r\nfunction(self)\r\nreturn ((((self):position()) >= (self):size()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"atStart",0
,(
function(self)
return ((smalltalk:isObject_equalTo_(((self):position()),(0))))
;
end
)
,"testing","atStart\r\n\t\" Return true if the receiver is at start position. \"\r\n\r\n\t^self position = 0","(\r\nfunction(self)\r\nreturn ((smalltalk:isObject_equalTo_(((self):position()),(0))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"isEmpty",0
,(
function(self)
return ((smalltalk:isObject_equalTo_(((self):size()),(0))))
;
end
)
,"testing","isEmpty\r\n\t\" Return true if the receiver is empty. \"\r\n\r\n\t^self size = 0","(\r\nfunction(self)\r\nreturn ((smalltalk:isObject_equalTo_(((self):size()),(0))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"notEmpty",0
,(
function(self)
return ((smalltalk:booleanNot_(((self):isEmpty()))))
;
end
)
,"testing","notEmpty\r\n\t\" Return true if the receiver is not empty. \"\r\n\r\n\t^self isEmpty not","(\r\nfunction(self)\r\nreturn ((smalltalk:booleanNot_(((self):isEmpty()))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"upTo:",0
,(
function(self,anObject)
local result=nil
(function()
	local _s8_value=((((self):collection())["$klass"]):new())
	result=_s8_value; return _s8_value
end)()
;
(function() while(not ((function()
return ((smalltalk:boolean_or_(((self):atEnd()),(function()
return ((smalltalk:isObject_equalTo_(((self):peek()),anObject)))
end))))
end))()) do (function()
return ((result):add_((self):next()))
end)() end;return nil end)()
;
(function() if ((((self):atEnd()))==true) then return nil else return ((function()
return ((self):next())
end))() end end)()
;
return (result)
;
end
)
,"reading","upTo: anObject\r\n\t\" Return the elements read from the receiver upto anObject (or end). \"\r\n\r\n\t| result |\r\n\tresult := self collection class new.\r\n\t[ self atEnd or: [ self peek = anObject ]] whileFalse: [\r\n\t    result add: self next\r\n\t].\r\n\tself atEnd ifFalse: [ self next ].\r\n\t^result","(\r\nfunction(self,anObject)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((((self):collection())[\"$klass\"]):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(not ((function()\r\nreturn ((smalltalk:boolean_or_(((self):atEnd()),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((self):peek()),anObject)))\r\nend))))\r\nend))()) do (function()\r\nreturn ((result):add_((self):next()))\r\nend)() end;return nil end)()\r\n;\r\n(function() if ((((self):atEnd()))==true) then return nil else return ((function()\r\nreturn ((self):next())\r\nend))() end end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"upToEnd",0
,(
function(self)
local result=nil
(function()
	local _s8_value=(((self):collection()):copyFrom_to_(((self):position()):_plus((1)), (self):readLimit()))
	result=_s8_value; return _s8_value
end)()
;
(self):setToEnd()
;
return (result)
;
end
)
,"reading","upToEnd\r\n\t\" Return the contents of the receiver upto end. \"\r\n\r\n\t| result |\r\n\tresult := self collection copyFrom: self position + 1 to: self readLimit.\r\n\tself setToEnd.\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(((self):collection()):copyFrom_to_(((self):position()):_plus((1)), (self):readLimit()))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):setToEnd()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"nextLine",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
local start=nil
(function()
	local _s8_value=((self):position())
	start=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):upTo_((smalltalk.String):cr()))
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((result):includes_((smalltalk.String):lf())))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((
function(_s8_rec)
(_s8_rec):position_(start);
return ((_s8_rec):upTo_((smalltalk.String):lf()))
end
)(self))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((self):peek()),(smalltalk.String):lf()))))==true) then return ((function()
return ((self):next())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"reading","nextLine\r\n\t\" Return the characters read from the receiver up to the line delimiter. \"\r\n\r\n\t| result start |\r\n\tstart := self position.\r\n\tresult := self upTo: String cr.\r\n\t(result includes: String lf) ifTrue: [\r\n\t\t^self position: start; upTo: String lf\r\n\t].\r\n\t(self peek = String lf) ifTrue: [ self next ].\r\n\t^result","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\nlocal start=nil\r\n(function()\r\n\tlocal _s8_value=((self):position())\r\n\tstart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):upTo_((smalltalk.String):cr()))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((result):includes_((smalltalk.String):lf())))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):position_(start);\r\nreturn ((_s8_rec):upTo_((smalltalk.String):lf()))\r\nend\r\n)(self))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):peek()),(smalltalk.String):lf()))))==true) then return ((function()\r\nreturn ((self):next())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"skipTo:",0
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() while(not ((function()
return ((self):atEnd())
end))()) do (function()
return ((function() if ((((smalltalk:isObject_equalTo_(((self):next()),anObject))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"reading","skipTo: anObject\r\n\t\" Advance the receiver upto anObject (or end). \"\r\n\r\n\t[ self atEnd ] whileFalse: [\r\n\t\tself next = anObject ifTrue: [ ^true ].\r\n\t].\r\n\t^false","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() while(not ((function()\r\nreturn ((self):atEnd())\r\nend))()) do (function()\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_(((self):next()),anObject))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"skipSeparators",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() while(((function()
(function() if ((((self):atEnd()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return (((self):peek()):isSeparator())
end))()) do (function()
return ((self):next())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"reading","skipSeparators\r\n\t\" Advance the receiver upto a non-separator (or end).\r\n\tReturn true  if a non-separator contents is found,\r\n\treturn false if run upto end of stream.\r\n\t\"\r\n\r\n\t[ self atEnd ifTrue: [ ^false ].\r\n\t  self peek isSeparator\r\n\t] whileTrue: [ self next ].\r\n\t^true","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() while(((function()\r\n(function() if ((((self):atEnd()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return (((self):peek()):isSeparator())\r\nend))()) do (function()\r\nreturn ((self):next())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"nextChunk",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((self):upTo_("!"))
	result=_s8_value; return _s8_value
end)()
;
(function() while(((function()
return ((smalltalk:isObject_equalTo_(((self):peek()),"!")))
end))()) do (function()
(self):next();return ((function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((result),"!")),(self):upTo_("!")))
	result=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
return ((result):trimFirst_((function(c)
return ((((c):asciiValue()) <= (32)))
end)))
;
end
)
,"Tools-fileIn","nextChunk\r\n\t\" FileIn - Return the next chunk read from the receiver.\r\n\tTrailing non-printing characters are skipped. \"\r\n\r\n\t| result |\r\n\tresult := self upTo: $!.\r\n\t[ self peek = $! ] whileTrue: [\r\n\t\tself next.\r\n\t\tresult := result, '!', (self upTo: $!)\r\n\t].\r\n\t^result trimFirst: [:c| c asciiValue <= 32 ]","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):upTo_(\"!\"))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\nreturn ((smalltalk:isObject_equalTo_(((self):peek()),\"!\")))\r\nend))()) do (function()\r\n(self):next();return ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((result),\"!\")),(self):upTo_(\"!\")))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\nreturn ((result):trimFirst_((function(c)\r\nreturn ((((c):asciiValue()) <= (32)))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"fileInto:",0
,(
function(self,aReceiver)
local scanner=nil
(function() while(not ((function()
return ((self):atEnd())
end))()) do (function()
(function() if ((((((scanner))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=(aReceiver)
	scanner=_s8_value; return _s8_value
end)())
end))() else return nil end end)();return ((function()
	local _s8_value=((scanner):scanFrom_(self))
	scanner=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
return (scanner)
;
end
)
,"Tools-fileIn","fileInto: aReceiver\r\n\t\" FileIn - FileIn the receiver's contents (in chunk format) using aReceiver as default scanner. \"\r\n\r\n\t| scanner |\r\n\t[self atEnd] whileFalse: [\r\n\t\tscanner isNil ifTrue: [ scanner := aReceiver ].\r\n\t\tscanner := scanner scanFrom: self\r\n\t].\r\n\t^scanner","(\r\nfunction(self,aReceiver)\r\nlocal scanner=nil\r\n(function() while(not ((function()\r\nreturn ((self):atEnd())\r\nend))()) do (function()\r\n(function() if ((((((scanner))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(aReceiver)\r\n\tscanner=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n\tlocal _s8_value=((scanner):scanFrom_(self))\r\n\tscanner=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\nreturn (scanner)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"fileIn",0
,(
function(self)
return ((self):fileInto_(nil))
;
end
)
,"Tools-fileIn","fileIn\r\n\t\" FileIn - FileIn the receiver's contents (in chunk format). \"\r\n\r\n\t^self fileInto: nil","(\r\nfunction(self)\r\nreturn ((self):fileInto_(nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Stream,"nextChunkPut:",0
,(
function(self,aString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local eoc=nil
local parts=nil
(function()
	local _s8_value=("!")
	eoc=_s8_value; return _s8_value
end)()
;
(function() if ((((((aString))==nil)))==true) then return ((function()
(self):nextPut_(eoc);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((aString):isEmpty()))==true) then return ((function()
return ((self):nextPut_(" "))
end))() else return nil end end)()
;
((aString):asArrayOfSubstringsSeparatedBy_(eoc)):do_separatedBy_((function(each)
return ((self):nextPutAll_(each))
end), (function()
return ((self):nextPutAll_(smalltalk:object_comma_((eoc),eoc)))
end))
;
(self):nextPut_(eoc)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (aString)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"Tools-fileOut","nextChunkPut: aString\r\n\t\" Output aString in chunk format. \"\r\n\r\n\t| eoc parts |\r\n\teoc := $!.\r\n\taString isNil ifTrue: [  self nextPut: eoc. ^nil ].\r\n\taString isEmpty ifTrue: [ self nextPut: $ ].\r\n\t(aString asArrayOfSubstringsSeparatedBy: eoc)\r\n\t\tdo: [:each| self nextPutAll: each ]\r\n\t\tseparatedBy: [ self nextPutAll: eoc,eoc ].\r\n\tself nextPut: eoc.\r\n\t^aString","(\r\nfunction(self,aString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal eoc=nil\r\nlocal parts=nil\r\n(function()\r\n\tlocal _s8_value=(\"!\")\r\n\teoc=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((aString))==nil)))==true) then return ((function()\r\n(self):nextPut_(eoc);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((aString):isEmpty()))==true) then return ((function()\r\nreturn ((self):nextPut_(\" \"))\r\nend))() else return nil end end)()\r\n;\r\n((aString):asArrayOfSubstringsSeparatedBy_(eoc)):do_separatedBy_((function(each)\r\nreturn ((self):nextPutAll_(each))\r\nend), (function()\r\nreturn ((self):nextPutAll_(smalltalk:object_comma_((eoc),eoc)))\r\nend))\r\n;\r\n(self):nextPut_(eoc)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (aString)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.StringStream,"next:",0
,(
function(self,anInteger)
local start=nil
local result=nil
local stop=nil
(function()
	local _s8_value=((self):position())
	start=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((start):_plus(anInteger)):min_((self):readLimit()))
	stop=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((self):collection()):copyFrom_to_((start):_plus((1)), stop))
	result=_s8_value; return _s8_value
end)()
;
(self):position_(stop)
;
return (result)
;
end
)
,"reading","next: anInteger\r\n\t\" Return the next anInteger elements read from the receiver.\r\n\tIf the receiver reach its end, the returned collection has less elements.\r\n\t\"\r\n\r\n\t| start result stop |\r\n\tstart := self position.\r\n\tstop := start + anInteger min: self readLimit.\r\n\tresult := self collection copyFrom: start + 1 to: stop.\r\n\tself position: stop.\r\n\t^result","(\r\nfunction(self,anInteger)\r\nlocal start=nil\r\nlocal result=nil\r\nlocal stop=nil\r\n(function()\r\n\tlocal _s8_value=((self):position())\r\n\tstart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((start):_plus(anInteger)):min_((self):readLimit()))\r\n\tstop=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((self):collection()):copyFrom_to_((start):_plus((1)), stop))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):position_(stop)\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringStream,"nextPut:",0
,(
function(self,aString)
(self):nextPutAll_(aString)
;
return (self)
;
end
)
,"writing","nextPut: aString\r\n\t\" Put aString onto the receiver. \"\r\n\r\n\tself nextPutAll: aString","(\r\nfunction(self,aString)\r\n(self):nextPutAll_(aString)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringStream,"nextPutAll:",0
,(
function(self,aString)
local result=nil
(function() if ((((smalltalk:isObject_equalTo_(((self):position()),((self):collection()):size()))))==true) then return ((function()
return ((function()
	local _s8_value=(smalltalk:object_comma_(((self):collection()),aString))
	result=_s8_value; return _s8_value
end)())
end))() else return ((function()
local substring=nil
(function()
	local _s8_value=(((self):collection()):copyFrom_to_((1), (self):position()))
	substring=_s8_value; return _s8_value
end)();(function()
	local _s8_value=((function() if ((((substring):isEmpty()))==true) then return ((function()
return (aString)
end))() else return ((function()
return (smalltalk:object_comma_((substring),aString))
end))() end end)())
	result=_s8_value; return _s8_value
end)();(function()
	local _s8_value=(((self):collection()):copyFrom_to_((((self):position()):_plus((1))):_plus((aString):size()), ((self):collection()):size()))
	substring=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=((function() if ((((substring):isEmpty()))==true) then return ((function()
return (result)
end))() else return ((function()
return (smalltalk:object_comma_((result),substring))
end))() end end)())
	result=_s8_value; return _s8_value
end)())
end))() end end)()
;
(self):collection_(result)
;
(self):position_(((self):position()):_plus((aString):size()))
;
(self):readLimit_(((self):readLimit()):max_(self['@position']))
;
return (self)
;
end
)
,"writing","nextPutAll: aString\r\n\t\" Put aString onto the receiver at current position; expanding internal contents. \"\r\n\r\n\t| result |\r\n\tself position = self collection size\r\n\tifTrue: [ result := self collection , aString ]\r\n\tifFalse: [ | substring |\r\n\t\tsubstring := self collection copyFrom: 1 to: self position.\r\n\t\tresult := substring isEmpty ifTrue: [ aString ] ifFalse: [ substring , aString ].\r\n\t\tsubstring := self collection\r\n\t\t\tcopyFrom: self position + 1 + aString size\r\n\t\t\tto: self collection size.\r\n\t\tresult := substring isEmpty ifTrue: [ result ] ifFalse: [ result , substring ].\r\n\t].\r\n\tself collection: result.\r\n\tself position: self position + aString size.\r\n\tself readLimit: (self readLimit max: position)","(\r\nfunction(self,aString)\r\nlocal result=nil\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):position()),((self):collection()):size()))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_(((self):collection()),aString))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nlocal substring=nil\r\n(function()\r\n\tlocal _s8_value=(((self):collection()):copyFrom_to_((1), (self):position()))\r\n\tsubstring=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=((function() if ((((substring):isEmpty()))==true) then return ((function()\r\nreturn (aString)\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((substring),aString))\r\nend))() end end)())\r\n\tresult=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=(((self):collection()):copyFrom_to_((((self):position()):_plus((1))):_plus((aString):size()), ((self):collection()):size()))\r\n\tsubstring=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=((function() if ((((substring):isEmpty()))==true) then return ((function()\r\nreturn (result)\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((result),substring))\r\nend))() end end)())\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)()\r\n;\r\n(self):collection_(result)\r\n;\r\n(self):position_(((self):position()):_plus((aString):size()))\r\n;\r\n(self):readLimit_(((self):readLimit()):max_(self['@position']))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringStream,"skipTo:",0
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local index=nil
(function() if ((((anObject):isString()))==true) then return nil else return ((function()
(self):setToEnd();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
	local _s8_value=(((self):collection()):indexOf_startingAt_(anObject, ((self):position()):_plus((1))))
	index=_s8_value; return _s8_value
end)()
;
(function() if ((((index):between_and_(((self):position()):_plus((1)), (self):readLimit())))==true) then return ((function()
(self):position_(((index):_plus((anObject):size())):_minus((1)));return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(self):setToEnd()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"reading","skipTo: anObject\r\n\t\" Advance the receiver upto anObject (or end). \"\r\n\r\n\t| index |\r\n\tanObject isString ifFalse: [ self setToEnd. ^false ].\r\n\tindex := self collection indexOf: anObject startingAt: self position + 1.\r\n\t(index between: self position + 1 and: self readLimit) ifTrue: [\r\n\t\tself position: index + anObject size - 1.\r\n\t\t^true\r\n\t].\r\n\tself setToEnd.\r\n\t^false","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal index=nil\r\n(function() if ((((anObject):isString()))==true) then return nil else return ((function()\r\n(self):setToEnd();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((self):collection()):indexOf_startingAt_(anObject, ((self):position()):_plus((1))))\r\n\tindex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((index):between_and_(((self):position()):_plus((1)), (self):readLimit())))==true) then return ((function()\r\n(self):position_(((index):_plus((anObject):size())):_minus((1)));return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self):setToEnd()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.StringStream,"upTo:",0
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local start=nil
local found=nil
(function() if ((((anObject):isString()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):upToEnd())
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
	local _s8_value=(((self):position()):_plus((1)))
	start=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((function() if ((((self):skipTo_(anObject)))==true) then return ((function()
return ((anObject):size())
end))() else return ((function()
return ((0))
end))() end end)())
	found=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self):collection()):copyFrom_to_(start, ((self):position()):_minus(found)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"reading","upTo: anObject\r\n\t\" Return the elements read from the receiver upto anObject (or end). \"\r\n\r\n\t| start found |\r\n\tanObject isString ifFalse: [ ^self upToEnd ].\r\n\tstart := self position + 1.\r\n\tfound := (self skipTo: anObject) ifTrue: [ anObject size ] ifFalse: [ 0 ].\r\n\t^self collection copyFrom: start to: self position - found","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal start=nil\r\nlocal found=nil\r\n(function() if ((((anObject):isString()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):upToEnd())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((self):position()):_plus((1)))\r\n\tstart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function() if ((((self):skipTo_(anObject)))==true) then return ((function()\r\nreturn ((anObject):size())\r\nend))() else return ((function()\r\nreturn ((0))\r\nend))() end end)())\r\n\tfound=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self):collection()):copyFrom_to_(start, ((self):position()):_minus(found)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.StringStream,"peek",0
,(
function(self)
	local c = self["@collection"]:at_(self:position()+1)
	if c=="" then return nil else return c end

;
return (self)
;
end
)
,"reading","peek\r\n\t\" Return the next readable element from the receiver without advancing (or nil). \"\r\n\r\n{'\treturn self[\"@collection\"].charAt(self.position())||nil'\r\nlua'\tlocal c = self[\"@collection\"]:at_(self:position()+1)\r\n\tif c==\"\" then return nil else return c end\r\n'}","(\r\nfunction(self)\r\n\tlocal c = self[\"@collection\"]:at_(self:position()+1)\r\n\tif c==\"\" then return nil else return c end\r\n\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringStream,"next",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((self):peek())
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((((result))~=nil)))==true) then return ((function()
return ((function()
	local _s8_value=((self['@position']):_plus((1)))
	self['@position']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (result)
;
end
)
,"reading","next\r\n\t\" Return the next readable element in the receiver (or nil). \"\r\n\r\n\t| result |\r\n\tresult := self peek.\r\n\tresult notNil ifTrue: [ position := position + 1 ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):peek())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((result))~=nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((self['@position']):_plus((1)))\r\n\tself['@position']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringStream,"peekForAll:",0
,(
function(self,aCollection)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aCollection):isString()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:superImplementor(smalltalk.StringStream,"peekForAll_")(self,aCollection))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
	local start=self:position()
	if (self["@collection"]:sub(start+1,start+#aCollection)~=aCollection) then return false end	

;
(function()
	local _s8_value=((self['@position']):_plus((aCollection):size()))
	self['@position']=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"reading","peekForAll: aCollection\r\n\t\" Return true if the next readable elements from the receiver matches aCollection.\r\n\tAdvance the receiver if the result is true.\r\n\t\"\r\n\r\n\taCollection isString ifFalse: [ ^super peekForAll: aCollection ].\r\n{'\tvar start = self.position();\r\n\tfor (var i=0;i<aCollection.length; i++){\r\n\t\tif (self[\"@collection\"].charCodeAt(start+i) != aCollection.charCodeAt(i)) return false;\r\n\t}'\r\nlua'\tlocal start=self:position()\r\n\tif (self[\"@collection\"]:sub(start+1,start+#aCollection)~=aCollection) then return false end\t\r\n'}.\tposition := position + aCollection size.\r\n\t^true","(\r\nfunction(self,aCollection)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aCollection):isString()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:superImplementor(smalltalk.StringStream,\"peekForAll_\")(self,aCollection))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n\tlocal start=self:position()\r\n\tif (self[\"@collection\"]:sub(start+1,start+#aCollection)~=aCollection) then return false end\t\r\n\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@position']):_plus((aCollection):size()))\r\n\tself['@position']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.StringStream,"upToSeparator",0
,(
function(self)
local start=nil
(function()
	local _s8_value=(((self):position()):_plus((1)))
	start=_s8_value; return _s8_value
end)()
;
(function() while(((function()
return ((smalltalk:boolean_and_(((smalltalk:booleanNot_(((self):atEnd())))),(function()
return ((smalltalk:booleanNot_((((self):peek()):isSeparator()))))
end))))
end))()) do (function()
return ((self):next())
end)() end;return nil end)()
;
return (((self):collection()):copyFrom_to_(start, (self):position()))
;
end
)
,"reading","upToSeparator\r\n\t\" Return the elements read from the receiver upto a separator (or end). \"\r\n\r\n\t| start |\r\n\tstart := self position + 1.\r\n\t[self atEnd not and: [self peek isSeparator not]] whileTrue: [\r\n\t\tself next\r\n\t].\r\n\t^self collection copyFrom: start to: self position","(\r\nfunction(self)\r\nlocal start=nil\r\n(function()\r\n\tlocal _s8_value=(((self):position()):_plus((1)))\r\n\tstart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\nreturn ((smalltalk:boolean_and_(((smalltalk:booleanNot_(((self):atEnd())))),(function()\r\nreturn ((smalltalk:booleanNot_((((self):peek()):isSeparator()))))\r\nend))))\r\nend))()) do (function()\r\nreturn ((self):next())\r\nend)() end;return nil end)()\r\n;\r\nreturn (((self):collection()):copyFrom_to_(start, (self):position()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringStream,"nextMessagePattern",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local part=nil
local selector=nil
local args=nil
(function()
	local _s8_value=(smalltalk:newArray({}))
	args=_s8_value; return _s8_value
end)()
;
(self):skipSeparators()
;
(function()
	local _s8_value=((self):upToSeparator())
	selector=_s8_value; return _s8_value
end)()
;
(function() if ((((selector):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((((selector):argumentCount()) > (0))))==true) then return ((function()
local part=nil
return ((function() while(((function()
(self):skipSeparators();(args):add_((self):upToSeparator());(self):skipSeparators();(function()
	local _s8_value=((function() if ((((smalltalk:isObject_equalTo_(((self):peek()),"\""))))==true) then return ((function()
return ("")
end))() else return ((function()
return ((self):upToSeparator())
end))() end end)())
	part=_s8_value; return _s8_value
end)();return ((smalltalk:boolean_and_(((part):notEmpty()),(function()
return ((smalltalk:isObject_equalTo_(((part):last()),":")))
end))))
end))()) do (function()
return ((function()
	local _s8_value=(smalltalk:object_comma_((selector),part))
	selector=_s8_value; return _s8_value
end)())
end)() end;return nil end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((smalltalk.Message):selector_arguments_(selector, args))
	part=_s8_value; return _s8_value
end)()
;
(self):skipSeparators()
;
(function() if ((((smalltalk:isObject_equalTo_(((self):peek()),"\""))))==true) then return ((function()
return ((part):comment_(((
function(_s8_rec)
(_s8_rec):next();
return ((_s8_rec):upTo_("\""))
end
)(self)):trimBlanks()))
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (part)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"tools","nextMessagePattern\r\n\t\" Return a message pattern read from the receiver (or nil).\r\n\tNote: it is assumed that there is a correct message header in the receiver.\r\n\t\"\r\n\r\n\t| part selector args |\r\n\targs := #().\r\n\tself skipSeparators.\r\n\tselector := self upToSeparator.\r\n\tselector isEmpty ifTrue: [ ^nil ].\r\n\tselector argumentCount > 0 ifTrue: [ | part |\r\n\t\t[ self skipSeparators.\r\n\t\t  args add: self upToSeparator.\r\n\t\t  self skipSeparators.\r\n\t\t  part := self peek = $\" ifTrue: [ '' ]\r\n\t\t\tifFalse: [ self upToSeparator ].\r\n\t\t  part notEmpty and: [ part last = $: ]\r\n\t\t] whileTrue: [\r\n\t\t  selector := selector , part\r\n\t\t].\r\n\t].\r\n\tpart := Message selector: selector arguments: args.\r\n\tself skipSeparators.\r\n\tself peek = $\" ifTrue: [\r\n\t\tpart comment: (self next; upTo: $\") trimBlanks\r\n\t].\r\n\t^part","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal part=nil\r\nlocal selector=nil\r\nlocal args=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\targs=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):skipSeparators()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):upToSeparator())\r\n\tselector=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((selector):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((selector):argumentCount()) > (0))))==true) then return ((function()\r\nlocal part=nil\r\nreturn ((function() while(((function()\r\n(self):skipSeparators();(args):add_((self):upToSeparator());(self):skipSeparators();(function()\r\n\tlocal _s8_value=((function() if ((((smalltalk:isObject_equalTo_(((self):peek()),\"\\\"\"))))==true) then return ((function()\r\nreturn (\"\")\r\nend))() else return ((function()\r\nreturn ((self):upToSeparator())\r\nend))() end end)())\r\n\tpart=_s8_value; return _s8_value\r\nend)();return ((smalltalk:boolean_and_(((part):notEmpty()),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((part):last()),\":\")))\r\nend))))\r\nend))()) do (function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((selector),part))\r\n\tselector=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Message):selector_arguments_(selector, args))\r\n\tpart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):skipSeparators()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):peek()),\"\\\"\"))))==true) then return ((function()\r\nreturn ((part):comment_(((\r\nfunction(_s8_rec)\r\n(_s8_rec):next();\r\nreturn ((_s8_rec):upTo_(\"\\\"\"))\r\nend\r\n)(self)):trimBlanks()))\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (part)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"collection",0
,(
function(self)
return ((self):invalidMessage_("collection"))
;
end
)
,"accessing","collection\r\n\t\" Private - Return the collection of the receiver. \"\r\n\r\n\t^self invalidMessage: #collection","(\r\nfunction(self)\r\nreturn ((self):invalidMessage_(\"collection\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"collection:",0
,(
function(self,aCollection)
(function() if ((((aCollection):isString()))==true) then return nil else return ((function()
return ((self):error_("Collection must be a String"))
end))() end end)()
;
(function()
	local _s8_value=((smalltalk.Array):with_(aCollection))
	self['@collection']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","collection: aCollection\r\n\t\" Private - Set the collection of the receiver. \"\r\n\r\n\taCollection isString ifFalse: [ self error: 'Collection must be a String' ].\r\n\tcollection := Array with: aCollection","(\r\nfunction(self,aCollection)\r\n(function() if ((((aCollection):isString()))==true) then return nil else return ((function()\r\nreturn ((self):error_(\"Collection must be a String\"))\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):with_(aCollection))\r\n\tself['@collection']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"contents",0
,(
function(self)
return ((self['@collection']):join_(""))
;
end
)
,"accessing","contents\r\n\t\" Return the contents of the receiver. \"\r\n\r\n\t^collection join: ''","(\r\nfunction(self)\r\nreturn ((self['@collection']):join_(\"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"position",0
,(
function(self)
return ((self):readLimit())
;
end
)
,"accessing","position\r\n\t\" Return the position of the receiver. \"\r\n\r\n\t^self readLimit","(\r\nfunction(self)\r\nreturn ((self):readLimit())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"position:",0
,(
function(self,anInteger)
return ((self):invalidMessage_("position:"))
;
end
)
,"accessing","position: anInteger\r\n\t\" Set the position of the receiver. \"\r\n\r\n\t^self invalidMessage: #position:","(\r\nfunction(self,anInteger)\r\nreturn ((self):invalidMessage_(\"position:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"readLimit",0
,(
function(self)
return (self['@readLimit'])
;
end
)
,"accessing","readLimit\r\n\t\" Private - Return the read limit of the receiver. \"\r\n\r\n\t^readLimit","(\r\nfunction(self)\r\nreturn (self['@readLimit'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"readLimit:",0
,(
function(self,anInteger)
(function()
	local _s8_value=(anInteger)
	self['@readLimit']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","readLimit: anInteger\r\n\t\" Private - Set the stream size of the receiver. \"\r\n\r\n\treadLimit := anInteger","(\r\nfunction(self,anInteger)\r\n(function()\r\n\tlocal _s8_value=(anInteger)\r\n\tself['@readLimit']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"setCollection:",0
,(
function(self,aCollection)
return ((self):invalidMessage_("setCollection:"))
;
end
)
,"accessing","setCollection: aCollection\r\n\t\" Private - Set the receiver stream collection to aCollection. \"\r\n\r\n\t^self invalidMessage: #setCollection:","(\r\nfunction(self,aCollection)\r\nreturn ((self):invalidMessage_(\"setCollection:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"size",0
,(
function(self)
return ((self):readLimit())
;
end
)
,"accessing","size\r\n\t\" Return the size of the receiver. \"\r\n\r\n\t^self readLimit","(\r\nfunction(self)\r\nreturn ((self):readLimit())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"close",0
,(
function(self)
return (self)
;
end
)
,"actions","close\r\n\t\" Close the receiver. \"","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"reset",0
,(
function(self)
(self):initialize()
;
return (self)
;
end
)
,"actions","reset\r\n\t\" Reset the receiver. \"\r\n\r\n\tself initialize","(\r\nfunction(self)\r\n(self):initialize()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"do:",0
,(
function(self,aBlock)
return ((self):invalidMessage_("do:"))
;
end
)
,"enumerating","do: aBlock\r\n\t\" Evaluates aBlock streaming on the receiver. \"\r\n\r\n\t^self invalidMessage: #do:","(\r\nfunction(self,aBlock)\r\nreturn ((self):invalidMessage_(\"do:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"initialize",0
,(
function(self)
smalltalk:superImplementor(smalltalk.StringWriteStream,"initialize")(self)
;
(function()
	local _s8_value=((smalltalk.Array):new())
	self['@collection']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((function()
	local _s8_value=((0))
	self['@position']=_s8_value; return _s8_value
end)())
	self['@readLimit']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"initialize","initialize\r\n\t\" Private - Initialize the receiver. \"\r\n\r\n\tsuper initialize.\r\n\tcollection := Array new.\r\n\treadLimit := position := 0.","(\r\nfunction(self)\r\nsmalltalk:superImplementor(smalltalk.StringWriteStream,\"initialize\")(self)\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tself['@collection']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function()\r\n\tlocal _s8_value=((0))\r\n\tself['@position']=_s8_value; return _s8_value\r\nend)())\r\n\tself['@readLimit']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"setToEnd",0
,(
function(self)
return (self)
;
end
)
,"positioning","setToEnd\r\n\t\" Position the receiver at end. \"","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"skip:",0
,(
function(self,anInteger)
return ((self):invalidMessage_("skip:"))
;
end
)
,"positioning","skip: anInteger\r\n\t\" Skip next anInteger readable elements in the receiver. \"\r\n\r\n\t^self invalidMessage: #skip:","(\r\nfunction(self,anInteger)\r\nreturn ((self):invalidMessage_(\"skip:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"next",0
,(
function(self)
return ((self):invalidMessage_("next"))
;
end
)
,"reading","next\r\n\t\" Return the next readable element in the receiver (or nil). \"\r\n\r\n\t^self invalidMessage: #next","(\r\nfunction(self)\r\nreturn ((self):invalidMessage_(\"next\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"next:",0
,(
function(self,anInteger)
return ((self):invalidMessage_("next"))
;
end
)
,"reading","next: anInteger\r\n\t\" Return the next anInteger elements read from the receiver. \"\r\n\r\n\t^self invalidMessage: #next","(\r\nfunction(self,anInteger)\r\nreturn ((self):invalidMessage_(\"next\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"nextLine",0
,(
function(self)
return ((self):invalidMessage_("nextLine"))
;
end
)
,"reading","nextLine\r\n\t\" Return the characters read from the receiver up to the line delimiter. \"\r\n\r\n\t^self invalidMessage: #nextLine","(\r\nfunction(self)\r\nreturn ((self):invalidMessage_(\"nextLine\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"peek",0
,(
function(self)
return ((self):invalidMessage_("peek"))
;
end
)
,"reading","peek\r\n\t\" Return the next readable element from the receiver without advancing the receiver; or nil. \"\r\n\r\n\t^self invalidMessage: #peek","(\r\nfunction(self)\r\nreturn ((self):invalidMessage_(\"peek\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"upTo:",0
,(
function(self,anObject)
return ((self):invalidMessage_("upTo:"))
;
end
)
,"reading","upTo: anObject\r\n\t\" Return the elements read from the receiver upto anObject (or end). \"\r\n\r\n\t^self invalidMessage: #upTo:","(\r\nfunction(self,anObject)\r\nreturn ((self):invalidMessage_(\"upTo:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"upToEnd",0
,(
function(self)
return ((self):invalidMessage_("upToEnd"))
;
end
)
,"reading","upToEnd\r\n\t\" Return the contents of the receiver upto end. \"\r\n\r\n\t^self invalidMessage: #upToEnd","(\r\nfunction(self)\r\nreturn ((self):invalidMessage_(\"upToEnd\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"atEnd",0
,(
function(self)
return (true)
;
end
)
,"testing","atEnd\r\n\t\" Return true if the receiver is at end. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"nextPut:",0
,(
function(self,anObject)
local string=nil
(function()
	local _s8_value=(smalltalk:asString_((anObject)))
	string=_s8_value; return _s8_value
end)()
;
(self['@collection']):add_(string)
;
(function()
	local _s8_value=((self['@readLimit']):_plus((string):size()))
	self['@readLimit']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"writing","nextPut: anObject\r\n\t\" Put anObject onto the receiver. \"\r\n\r\n\t| string |\r\n\tstring :=  anObject asString.\r\n\tcollection add: string.\r\n\treadLimit := readLimit + string size","(\r\nfunction(self,anObject)\r\nlocal string=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:asString_((anObject)))\r\n\tstring=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self['@collection']):add_(string)\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@readLimit']):_plus((string):size()))\r\n\tself['@readLimit']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"nextPutAll:",0
,(
function(self,aCollection)
(self):nextPut_(aCollection)
;
return (self)
;
end
)
,"writing","nextPutAll: aCollection\r\n\t\" Put aCollection onto the receiver. \"\r\n\r\n\tself nextPut: aCollection","(\r\nfunction(self,aCollection)\r\n(self):nextPut_(aCollection)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.StringWriteStream,"removeLastInput",0
,(
function(self)
(function()
	local _s8_value=((self['@readLimit']):_minus(((self['@collection']):removeLast()):size()))
	self['@readLimit']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"back","removeLastInput\r\n\t\" Remove last input recorded in the receiver. \"\r\n\r\n\treadLimit := readLimit - collection removeLast size","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=((self['@readLimit']):_minus(((self['@collection']):removeLast()):size()))\r\n\tself['@readLimit']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Random,"next",0
,(
function(self)
return (math.random())
;
end
)
,"accessing","next\r\n\t\" Return the next random number. \"\r\n\r\n\t^{'Math.random()'\r\n\tlua'math.random()'}","(\r\nfunction(self)\r\nreturn (math.random())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Random,"next:",0
,(
function(self,anInteger)
return ((((1)):to_(anInteger)):collect_((function(each)
return ((self):next())
end)))
;
end
)
,"accessing","next: anInteger\r\n\t\" Return the next anInteger random numbers. \"\r\n\r\n\t^(1 to: anInteger) collect: [:each | self next ]","(\r\nfunction(self,anInteger)\r\nreturn ((((1)):to_(anInteger)):collect_((function(each)\r\nreturn ((self):next())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point["$klass"],"x:y:",0
,(
function(self,x,y)
return ((
function(_s8_rec)
(_s8_rec):x_(x);
return ((_s8_rec):y_(y))
end
)((self):basicNew()))
;
end
)
,"instantiation","x: x y: y\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self basicNew x: x; y: y","(\r\nfunction(self,x,y)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):x_(x);\r\nreturn ((_s8_rec):y_(y))\r\nend\r\n)((self):basicNew()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point["$klass"],"zero",0
,(
function(self)
return (((self):basicNew()):x_y_((0), (0)))
;
end
)
,"instantiation","zero\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self basicNew x: 0 y: 0","(\r\nfunction(self)\r\nreturn (((self):basicNew()):x_y_((0), (0)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"x",0
,(
function(self)
return (self['@x'])
;
end
)
,"accessing","x\r\n\t\" Return the x of the receiver. \"\r\n\r\n\t^x","(\r\nfunction(self)\r\nreturn (self['@x'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"y",0
,(
function(self)
return (self['@y'])
;
end
)
,"accessing","y\r\n\t\" Return the y of the receiver. \"\r\n\r\n\t^y","(\r\nfunction(self)\r\nreturn (self['@y'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"x:y:",0
,(
function(self,xValue,yValue)
(function()
	local _s8_value=(xValue)
	self['@x']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(yValue)
	self['@y']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","x: xValue y: yValue\r\n\t\" Set the receiver's coordinates. \"\r\n\r\n\tx := xValue.\r\n\ty := yValue","(\r\nfunction(self,xValue,yValue)\r\n(function()\r\n\tlocal _s8_value=(xValue)\r\n\tself['@x']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(yValue)\r\n\tself['@y']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"y:",0
,(
function(self,aNumber)
(function()
	local _s8_value=((function() if ((((((aNumber))==nil)))==true) then return ((function()
return ((0))
end))() else return ((function()
return (aNumber)
end))() end end)())
	self['@y']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","y: aNumber\r\n\t\" Set the y of the receiver. \"\r\n\r\n\ty := aNumber isNil ifTrue: [ 0 ] ifFalse: [ aNumber ]","(\r\nfunction(self,aNumber)\r\n(function()\r\n\tlocal _s8_value=((function() if ((((((aNumber))==nil)))==true) then return ((function()\r\nreturn ((0))\r\nend))() else return ((function()\r\nreturn (aNumber)\r\nend))() end end)())\r\n\tself['@y']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"x:",0
,(
function(self,aNumber)
(function()
	local _s8_value=((function() if ((((((aNumber))==nil)))==true) then return ((function()
return ((0))
end))() else return ((function()
return (aNumber)
end))() end end)())
	self['@x']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","x: aNumber\r\n\t\" Set the x of the receiver. \"\r\n\r\n\tx := aNumber isNil ifTrue: [ 0 ] ifFalse: [ aNumber ]","(\r\nfunction(self,aNumber)\r\n(function()\r\n\tlocal _s8_value=((function() if ((((((aNumber))==nil)))==true) then return ((function()\r\nreturn ((0))\r\nend))() else return ((function()\r\nreturn (aNumber)\r\nend))() end end)())\r\n\tself['@x']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"*","_star"
,(
function(self,aPoint)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aPoint):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@x']):_star(aPoint)):_at((self['@y']):_star(aPoint)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Point):x_y_((self['@x']):_star(((aPoint):asPoint()):x()), (self['@y']):_star(((aPoint):asPoint()):y())))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"arithmetic","* aPoint\r\n\t\" Return the result of operation. \"\r\n\r\n\taPoint isNumber ifTrue: [ ^x * aPoint @ (y * aPoint) ].\r\n\t^Point\tx: x * aPoint asPoint x\r\n\t\ty: y * aPoint asPoint y","(\r\nfunction(self,aPoint)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aPoint):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@x']):_star(aPoint)):_at((self['@y']):_star(aPoint)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Point):x_y_((self['@x']):_star(((aPoint):asPoint()):x()), (self['@y']):_star(((aPoint):asPoint()):y())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"+","_plus"
,(
function(self,aPoint)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aPoint):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@x']):_plus(aPoint)):_at((self['@y']):_plus(aPoint)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Point):x_y_((self['@x']):_plus(((aPoint):asPoint()):x()), (self['@y']):_plus(((aPoint):asPoint()):y())))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"arithmetic","+ aPoint\r\n\t\" Return the result of operation. \"\r\n\r\n\taPoint isNumber ifTrue: [ ^x + aPoint @ (y + aPoint) ].\r\n\t^Point\tx: x + aPoint asPoint x\r\n\t\ty: y + aPoint asPoint y","(\r\nfunction(self,aPoint)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aPoint):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@x']):_plus(aPoint)):_at((self['@y']):_plus(aPoint)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Point):x_y_((self['@x']):_plus(((aPoint):asPoint()):x()), (self['@y']):_plus(((aPoint):asPoint()):y())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"-","_minus"
,(
function(self,aPoint)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aPoint):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@x']):_minus(aPoint)):_at((self['@y']):_minus(aPoint)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Point):x_y_((self['@x']):_minus(((aPoint):asPoint()):x()), (self['@y']):_minus(((aPoint):asPoint()):y())))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"arithmetic","- aPoint\r\n\t\" Return the result of operation. \"\r\n\r\n\taPoint isNumber ifTrue: [ ^x - aPoint @ (y - aPoint) ].\r\n\t^Point\tx: x - aPoint asPoint x\r\n\t\ty: y - aPoint asPoint y","(\r\nfunction(self,aPoint)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aPoint):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@x']):_minus(aPoint)):_at((self['@y']):_minus(aPoint)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Point):x_y_((self['@x']):_minus(((aPoint):asPoint()):x()), (self['@y']):_minus(((aPoint):asPoint()):y())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"/","_slash"
,(
function(self,aPoint)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aPoint):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@x']):_slash(aPoint)):_at((self['@y']):_slash(aPoint)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Point):x_y_((self['@x']):_slash(((aPoint):asPoint()):x()), (self['@y']):_slash(((aPoint):asPoint()):y())))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"arithmetic","/ aPoint\r\n\t\" Return the result of operation. \"\r\n\r\n\taPoint isNumber ifTrue: [ ^x / aPoint @ (y / aPoint) ].\r\n\t^Point\tx: x / aPoint asPoint x\r\n\t\ty: y / aPoint asPoint y","(\r\nfunction(self,aPoint)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aPoint):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@x']):_slash(aPoint)):_at((self['@y']):_slash(aPoint)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Point):x_y_((self['@x']):_slash(((aPoint):asPoint()):x()), (self['@y']):_slash(((aPoint):asPoint()):y())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"//","_slash_slash"
,(
function(self,aPoint)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aPoint):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@x']):_slash_slash(aPoint)):_at((self['@y']):_slash_slash(aPoint)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Point):x_y_((self['@x']):_slash_slash(((aPoint):asPoint()):x()), (self['@y']):_slash_slash(((aPoint):asPoint()):y())))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"arithmetic","// aPoint\r\n\t\" Return the result of operation. \"\r\n\r\n\taPoint isNumber ifTrue: [ ^x // aPoint @ (y // aPoint) ].\r\n\t^Point\tx: x // aPoint asPoint x\r\n\t\ty: y // aPoint asPoint y","(\r\nfunction(self,aPoint)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aPoint):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@x']):_slash_slash(aPoint)):_at((self['@y']):_slash_slash(aPoint)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Point):x_y_((self['@x']):_slash_slash(((aPoint):asPoint()):x()), (self['@y']):_slash_slash(((aPoint):asPoint()):y())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"\\\\","_bk_bk"
,(
function(self,aPoint)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aPoint):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@x']):_bk_bk(aPoint)):_at((self['@y']):_bk_bk(aPoint)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Point):x_y_((self['@x']):_bk_bk(((aPoint):asPoint()):x()), (self['@y']):_bk_bk(((aPoint):asPoint()):y())))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"arithmetic","\\\\\\\\ aPoint\r\n\t\" Return the result of operation. \"\r\n\r\n\taPoint isNumber ifTrue: [ ^x \\\\ aPoint @ (y \\\\ aPoint) ].\r\n\t^Point\tx: x \\\\ aPoint asPoint x\r\n\t\ty: y \\\\ aPoint asPoint y","(\r\nfunction(self,aPoint)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aPoint):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@x']):_bk_bk(aPoint)):_at((self['@y']):_bk_bk(aPoint)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Point):x_y_((self['@x']):_bk_bk(((aPoint):asPoint()):x()), (self['@y']):_bk_bk(((aPoint):asPoint()):y())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"<","_lt"
,(
function(self,aPoint)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aPoint):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self) < (aPoint):asPoint()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk:boolean_and_((((self['@x']) < (aPoint):x())),(function()
return (((self['@y']) < (aPoint):y()))
end))))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"comparing","< aPoint\r\n\t\" Return true if the receiver is smaller than argument. \"\r\n\r\n\taPoint isNumber ifTrue: [ ^self < aPoint asPoint ].\r\n\t^x < aPoint x and: [ y < aPoint y ]","(\r\nfunction(self,aPoint)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aPoint):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self) < (aPoint):asPoint()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk:boolean_and_((((self['@x']) < (aPoint):x())),(function()\r\nreturn (((self['@y']) < (aPoint):y()))\r\nend))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"<=","_lt_eq"
,(
function(self,aPoint)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aPoint):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self) <= (aPoint):asPoint()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk:boolean_and_((((self['@x']) <= (aPoint):x())),(function()
return (((self['@y']) <= (aPoint):y()))
end))))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"comparing","<= aPoint\r\n\t\" Return true if the receiver is smaller or equal to argument. \"\r\n\r\n\taPoint isNumber ifTrue: [ ^self <= aPoint asPoint ].\r\n\t^x <= aPoint x and: [ y <= aPoint y ]","(\r\nfunction(self,aPoint)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aPoint):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self) <= (aPoint):asPoint()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk:boolean_and_((((self['@x']) <= (aPoint):x())),(function()\r\nreturn (((self['@y']) <= (aPoint):y()))\r\nend))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,">","_gt"
,(
function(self,aPoint)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aPoint):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self) > (aPoint):asPoint()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk:boolean_and_((((self['@x']) > (aPoint):x())),(function()
return (((self['@y']) > (aPoint):y()))
end))))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"comparing","> aPoint\r\n\t\" Return true if the receiver is greater than argument. \"\r\n\r\n\taPoint isNumber ifTrue: [ ^self > aPoint asPoint ].\r\n\t^x > aPoint x and: [ y > aPoint y ]","(\r\nfunction(self,aPoint)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aPoint):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self) > (aPoint):asPoint()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk:boolean_and_((((self['@x']) > (aPoint):x())),(function()\r\nreturn (((self['@y']) > (aPoint):y()))\r\nend))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,">=","_gt_eq"
,(
function(self,aPoint)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aPoint):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self) >= (aPoint):asPoint()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk:boolean_and_((((self['@x']) >= (aPoint):x())),(function()
return (((self['@y']) >= (aPoint):y()))
end))))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"comparing",">= aPoint\r\n\t\" Return true if the receiver is greater or equal to argument. \"\r\n\r\n\taPoint isNumber ifTrue: [ ^self >= aPoint asPoint ].\r\n\t^x >= aPoint x and: [ y >= aPoint y ]","(\r\nfunction(self,aPoint)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aPoint):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self) >= (aPoint):asPoint()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk:boolean_and_((((self['@x']) >= (aPoint):x())),(function()\r\nreturn (((self['@y']) >= (aPoint):y()))\r\nend))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"rounded",0
,(
function(self)
return (((self['@x']):rounded()):_at((self['@y']):rounded()))
;
end
)
,"converting","rounded\r\n\t\" Return the receiver rounded. \"\r\n\r\n\t^x rounded @ y rounded","(\r\nfunction(self)\r\nreturn (((self['@x']):rounded()):_at((self['@y']):rounded()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"truncated",0
,(
function(self)
return (((self['@x']):truncated()):_at((self['@y']):truncated()))
;
end
)
,"converting","truncated\r\n\t\" Return the receiver truncated. \"\r\n\r\n\t^x truncated @ y truncated","(\r\nfunction(self)\r\nreturn (((self['@x']):truncated()):_at((self['@y']):truncated()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"asPoint",0
,(
function(self)
return (self)
;
end
)
,"converting","asPoint\r\n\t\" Return the receiver as a Point. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"printString",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:toString_((self['@x'])))," @ ")),smalltalk:toString_((self['@y']))))
;
end
)
,"printing","printString\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^x toString ,' @ ' ,y toString","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:toString_((self['@x']))),\" @ \")),smalltalk:toString_((self['@y']))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"toString",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:toString_((self['@x']))),"@")),smalltalk:toString_((self['@y']))))
;
end
)
,"printing","toString\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^x toString ,'@' ,y toString","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:toString_((self['@x']))),\"@\")),smalltalk:toString_((self['@y']))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"shallowCopy",0
,(
function(self)
return ((((self)["$klass"]):basicNew()):x_y_((self):x(), (self):y()))
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self class basicNew\r\n\t\tx: self x y: self y","(\r\nfunction(self)\r\nreturn ((((self)[\"$klass\"]):basicNew()):x_y_((self):x(), (self):y()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"=","_eq"
,(
function(self,aPoint)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((aPoint))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((aPoint):isMemberOf_((self)["$klass"])))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk:boolean_and_((smalltalk:responds_to_((aPoint),("asPoint"))),(function()
return ((smalltalk:isObject_equalTo_(((aPoint):asPoint()),self)))
end))))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((self['@x']),(aPoint):x()))),(function()
return ((smalltalk:isObject_equalTo_((self['@y']),(aPoint):y())))
end))))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"testing","= aPoint\r\n\t\" Return true if the receiver is equal to argument. \"\r\n\r\n\taPoint isNil ifTrue: [ ^false ].\r\n\t(aPoint isMemberOf: self class) ifFalse: [\r\n\t\t^(aPoint respondsTo: #asPoint) and: [\r\n\t\t\taPoint asPoint = self\r\n\t\t]\r\n\t].\r\n\t^x = aPoint x and: [ y = aPoint y ]","(\r\nfunction(self,aPoint)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((aPoint))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((aPoint):isMemberOf_((self)[\"$klass\"])))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk:boolean_and_((smalltalk:responds_to_((aPoint),(\"asPoint\"))),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((aPoint):asPoint()),self)))\r\nend))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((self['@x']),(aPoint):x()))),(function()\r\nreturn ((smalltalk:isObject_equalTo_((self['@y']),(aPoint):y())))\r\nend))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"min:",0
,(
function(self,aPoint)
return (((self['@x']):min_((aPoint):x())):_at((self['@y']):min_((aPoint):y())))
;
end
)
,"magnitude","min: aPoint\r\n\t\" Return the minimum of the receiver and aPoint. \"\r\n\r\n\t^(x min: aPoint x) @ (y min: aPoint y)","(\r\nfunction(self,aPoint)\r\nreturn (((self['@x']):min_((aPoint):x())):_at((self['@y']):min_((aPoint):y())))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"max:",0
,(
function(self,aPoint)
return (((self['@x']):max_((aPoint):x())):_at((self['@y']):max_((aPoint):y())))
;
end
)
,"magnitude","max: aPoint\r\n\t\" Return the maximum of the receiver and aPoint. \"\r\n\r\n\t^(x max: aPoint x) @ (y max: aPoint y)","(\r\nfunction(self,aPoint)\r\nreturn (((self['@x']):max_((aPoint):x())):_at((self['@y']):max_((aPoint):y())))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"extent:",0
,(
function(self,extent)
return ((smalltalk.Rectangle):leftTop_rightBottom_(self, (self):_plus(extent)))
;
end
)
,"converting","extent: extent\r\n\t\" Return a Rectangle from the receiver with specified extent. \"\r\n\r\n\t^Rectangle leftTop: self rightBottom: self + extent","(\r\nfunction(self,extent)\r\nreturn ((smalltalk.Rectangle):leftTop_rightBottom_(self, (self):_plus(extent)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"rightBottom:",0
,(
function(self,rightBottom)
return ((smalltalk.Rectangle):leftTop_rightBottom_(self, rightBottom))
;
end
)
,"converting","rightBottom: rightBottom\r\n\t\" Return a Rectangle from the receiver to rightBottom. \"\r\n\r\n\t^Rectangle leftTop: self rightBottom: rightBottom","(\r\nfunction(self,rightBottom)\r\nreturn ((smalltalk.Rectangle):leftTop_rightBottom_(self, rightBottom))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"to:",0
,(
function(self,rightBottom)
return ((smalltalk.Rectangle):leftTop_rightBottom_(self, rightBottom))
;
end
)
,"converting","to: rightBottom\r\n\t\" Return a Rectangle from the receiver to rightBottom. \"\r\n\r\n\t^Rectangle leftTop: self rightBottom: rightBottom","(\r\nfunction(self,rightBottom)\r\nreturn ((smalltalk.Rectangle):leftTop_rightBottom_(self, rightBottom))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Point,"__tostring",0
,(
function(self)
return (smalltalk:toString_((self)))
;
end
)
,"Lua-printing","__tostring\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle["$klass"],"leftTop:extent:",0
,(
function(self,leftTop,extent)
return (((self):new()):leftTop_rightBottom_(leftTop, (leftTop):_plus(extent)))
;
end
)
,"instantiation","leftTop: leftTop extent: extent\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new leftTop: leftTop rightBottom: leftTop + extent","(\r\nfunction(self,leftTop,extent)\r\nreturn (((self):new()):leftTop_rightBottom_(leftTop, (leftTop):_plus(extent)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle["$klass"],"leftTop:rightBottom:",0
,(
function(self,leftTop,rightBottom)
return (((self):new()):leftTop_rightBottom_(leftTop, rightBottom))
;
end
)
,"instantiation","leftTop: leftTop rightBottom: rightBottom\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new leftTop: leftTop rightBottom: rightBottom","(\r\nfunction(self,leftTop,rightBottom)\r\nreturn (((self):new()):leftTop_rightBottom_(leftTop, rightBottom))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"=","_eq"
,(
function(self,aRectangle)
return ((smalltalk:boolean_and_(((aRectangle):isRectangle()),(function()
return ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((self['@leftTop']),(aRectangle):leftTop()))),(function()
return ((smalltalk:isObject_equalTo_((self['@rightBottom']),(aRectangle):rightBottom())))
end))))
end))))
;
end
)
,"testing","= aRectangle\r\n\t\" Return true if the receiver is equal to argument. \"\r\n\r\n\t^aRectangle isRectangle\r\n\tand: [ leftTop = aRectangle leftTop\r\n\tand: [ rightBottom = aRectangle rightBottom ]]","(\r\nfunction(self,aRectangle)\r\nreturn ((smalltalk:boolean_and_(((aRectangle):isRectangle()),(function()\r\nreturn ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((self['@leftTop']),(aRectangle):leftTop()))),(function()\r\nreturn ((smalltalk:isObject_equalTo_((self['@rightBottom']),(aRectangle):rightBottom())))\r\nend))))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"area",0
,(
function(self)
return (((self):height()):_star((self):width()))
;
end
)
,"accessing","area\r\n\t\" Return the area of the receiver.\"\r\n\r\n\t^self height * self width","(\r\nfunction(self)\r\nreturn (((self):height()):_star((self):width()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"areasOutside:",0
,(
function(self,aRectangle)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):intersects_(aRectangle)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Array):with_(self))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((smalltalk:boolean_and_(((aRectangle):containsPoint_((self):topLeft())),(function()
return ((aRectangle):containsPoint_((self):bottomRight()))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:newArray({}))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((smalltalk.Array):with_with_with_with_((function() if ((((((aRectangle):top()) > (self):top())))==true) then return ((function()
return (((self):topLeft()):rightBottom_(((self):right()):_at((aRectangle):top())))
end))() else return nil end end)(), (function() if ((((((aRectangle):bottom()) < (self):bottom())))==true) then return ((function()
return ((((self):left()):_at((aRectangle):bottom())):rightBottom_((self):bottomRight()))
end))() else return nil end end)(), (function() if ((((((aRectangle):left()) > (self):left())))==true) then return ((function()
return ((((self):left()):_at(((self):top()):max_((aRectangle):top()))):rightBottom_(((aRectangle):left()):_at(((self):bottom()):min_((aRectangle):bottom()))))
end))() else return nil end end)(), (function() if ((((((aRectangle):right()) < (self):right())))==true) then return ((function()
return ((((aRectangle):right()):_at(((self):top()):max_((aRectangle):top()))):rightBottom_(((self):right()):_at(((self):bottom()):min_((aRectangle):bottom()))))
end))() else return nil end end)())):select_((function(each)
return ((((each))~=nil))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","areasOutside: aRectangle\r\n\t\"Return a collection of rectangles where the receiver do not intersect aRectangle. \"\r\n\r\n\t(self intersects: aRectangle) ifFalse: [ ^Array with: self ].\r\n\t((aRectangle containsPoint: self topLeft)\r\n\tand: [aRectangle containsPoint: self bottomRight])\r\n        ifTrue: [ ^#() ].\r\n\t^(Array\r\n\twith: (aRectangle top > self top ifTrue: [ self topLeft rightBottom: self right @ aRectangle top])\r\n\twith: (aRectangle bottom < self bottom ifTrue: [ self left @ aRectangle bottom rightBottom: self bottomRight])\r\n\twith: (aRectangle left > self left ifTrue: [ self left @ (self top max: aRectangle top) rightBottom: aRectangle left @ (self bottom min: aRectangle bottom) ])\r\n\twith: (aRectangle right < self right ifTrue: [ aRectangle right @ (self top max: aRectangle top) rightBottom: self right @ (self bottom min: aRectangle bottom) ])\r\n\t) select: [:each| each notNil ]","(\r\nfunction(self,aRectangle)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):intersects_(aRectangle)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Array):with_(self))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((aRectangle):containsPoint_((self):topLeft())),(function()\r\nreturn ((aRectangle):containsPoint_((self):bottomRight()))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:newArray({}))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((smalltalk.Array):with_with_with_with_((function() if ((((((aRectangle):top()) > (self):top())))==true) then return ((function()\r\nreturn (((self):topLeft()):rightBottom_(((self):right()):_at((aRectangle):top())))\r\nend))() else return nil end end)(), (function() if ((((((aRectangle):bottom()) < (self):bottom())))==true) then return ((function()\r\nreturn ((((self):left()):_at((aRectangle):bottom())):rightBottom_((self):bottomRight()))\r\nend))() else return nil end end)(), (function() if ((((((aRectangle):left()) > (self):left())))==true) then return ((function()\r\nreturn ((((self):left()):_at(((self):top()):max_((aRectangle):top()))):rightBottom_(((aRectangle):left()):_at(((self):bottom()):min_((aRectangle):bottom()))))\r\nend))() else return nil end end)(), (function() if ((((((aRectangle):right()) < (self):right())))==true) then return ((function()\r\nreturn ((((aRectangle):right()):_at(((self):top()):max_((aRectangle):top()))):rightBottom_(((self):right()):_at(((self):bottom()):min_((aRectangle):bottom()))))\r\nend))() else return nil end end)())):select_((function(each)\r\nreturn ((((each))~=nil))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"bottom",0
,(
function(self)
return ((self['@rightBottom']):y())
;
end
)
,"accessing","bottom\r\n\t\" Return the bottom of the receiver.\"\r\n\r\n\t^rightBottom y","(\r\nfunction(self)\r\nreturn ((self['@rightBottom']):y())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"bottomCenter",0
,(
function(self)
return ((((self):center()):x()):_at((self):bottom()))
;
end
)
,"accessing","bottomCenter\r\n\t\" Return the center of the bottom edge of the receiver. \"\r\n\r\n\t^self center x @ self bottom","(\r\nfunction(self)\r\nreturn ((((self):center()):x()):_at((self):bottom()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"bottomLeft",0
,(
function(self)
return (((self):left()):_at((self):bottom()))
;
end
)
,"accessing","bottomLeft\r\n\t\" Return the bottom-left corner of the receiver. \"\r\n\r\n\t^self left @ self bottom","(\r\nfunction(self)\r\nreturn (((self):left()):_at((self):bottom()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"bottomRight",0
,(
function(self)
return (self['@rightBottom'])
;
end
)
,"accessing","bottomRight\r\n\t\" Return the bottom-right corner of the receiver. \"\r\n\r\n\t^rightBottom","(\r\nfunction(self)\r\nreturn (self['@rightBottom'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"center",0
,(
function(self)
return (((self['@leftTop']):_plus(self['@rightBottom'])):_slash_slash((2)))
;
end
)
,"accessing","center\r\n\t\" Return the center of the receiver. \"\r\n\r\n\t^leftTop + rightBottom // 2","(\r\nfunction(self)\r\nreturn (((self['@leftTop']):_plus(self['@rightBottom'])):_slash_slash((2)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"centerTop",0
,(
function(self)
return (((((self):right()):_plus((self):left())):_slash_slash((2))):_at((self):top()))
;
end
)
,"accessing","centerTop\r\n\t\" Return the center top of the receiver. \"\r\n\r\n\t^self right + self left // 2 @ self top","(\r\nfunction(self)\r\nreturn (((((self):right()):_plus((self):left())):_slash_slash((2))):_at((self):top()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"contains:",0
,(
function(self,aRectangle)
return ((smalltalk:boolean_and_(((((self):left()) <= (aRectangle):left())),(function()
return ((smalltalk:boolean_and_(((((aRectangle):right()) <= (self):right())),(function()
return ((smalltalk:boolean_and_(((((self):bottom()) >= (aRectangle):bottom())),(function()
return ((((aRectangle):top()) >= (self):top()))
end))))
end))))
end))))
;
end
)
,"testing","contains: aRectangle\r\n\t\" Return true if aRectangle lies completely within or on the edge of the receiver. \"\r\n\r\n\t^self left <= aRectangle left\r\n\tand: [ aRectangle right <= self right\r\n\tand: [ self bottom >= aRectangle bottom\r\n\tand: [ aRectangle top >= self top ]]]","(\r\nfunction(self,aRectangle)\r\nreturn ((smalltalk:boolean_and_(((((self):left()) <= (aRectangle):left())),(function()\r\nreturn ((smalltalk:boolean_and_(((((aRectangle):right()) <= (self):right())),(function()\r\nreturn ((smalltalk:boolean_and_(((((self):bottom()) >= (aRectangle):bottom())),(function()\r\nreturn ((((aRectangle):top()) >= (self):top()))\r\nend))))\r\nend))))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"containsPoint:",0
,(
function(self,aPoint)
return ((smalltalk:boolean_and_(((((self):left()) <= (aPoint):x())),(function()
return ((smalltalk:boolean_and_(((((aPoint):x()) < (self):right())),(function()
return ((smalltalk:boolean_and_(((((self):bottom()) > (aPoint):y())),(function()
return ((((aPoint):y()) >= (self):top()))
end))))
end))))
end))))
;
end
)
,"testing","containsPoint: aPoint\r\n\t\" Return true if aPoint is contained within the receiver. \"\r\n\r\n\t^(self left <= aPoint x)\r\n\tand: [ aPoint x < self right\r\n\tand: [ self bottom > aPoint y\r\n\tand: [ aPoint y >= self top ]]]","(\r\nfunction(self,aPoint)\r\nreturn ((smalltalk:boolean_and_(((((self):left()) <= (aPoint):x())),(function()\r\nreturn ((smalltalk:boolean_and_(((((aPoint):x()) < (self):right())),(function()\r\nreturn ((smalltalk:boolean_and_(((((self):bottom()) > (aPoint):y())),(function()\r\nreturn ((((aPoint):y()) >= (self):top()))\r\nend))))\r\nend))))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"expandBy:",0
,(
function(self,delta)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((delta):isRectangle()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@leftTop']):_minus((delta):leftTop())):rightBottom_((self['@rightBottom']):_plus((delta):rightBottom())))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@leftTop']):_minus(delta)):rightBottom_((self['@rightBottom']):_plus(delta)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","expandBy: delta\r\n\t\" Return a Rectangle obtained from the receiver expanded by delta. \"\r\n\r\n\tdelta isRectangle ifTrue: [\r\n            ^leftTop - delta leftTop rightBottom: rightBottom + delta rightBottom\r\n\t].\r\n\t^leftTop - delta rightBottom: rightBottom + delta","(\r\nfunction(self,delta)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((delta):isRectangle()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@leftTop']):_minus((delta):leftTop())):rightBottom_((self['@rightBottom']):_plus((delta):rightBottom())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@leftTop']):_minus(delta)):rightBottom_((self['@rightBottom']):_plus(delta)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"extent",0
,(
function(self)
return (((self):width()):_at((self):height()))
;
end
)
,"accessing","extent\r\n\t\" Return receiver's width and height.\"\r\n\r\n\t^self width @ self height","(\r\nfunction(self)\r\nreturn (((self):width()):_at((self):height()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"extent:",0
,(
function(self,aPoint)
(self):width_((aPoint):x())
;
(self):height_((aPoint):y())
;
return (self)
;
end
)
,"accessing","extent: aPoint\r\n\t\" Set the extent of receiver to aPoint. \"\r\n\r\n\tself width: aPoint x.\r\n\tself height: aPoint y","(\r\nfunction(self,aPoint)\r\n(self):width_((aPoint):x())\r\n;\r\n(self):height_((aPoint):y())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"height",0
,(
function(self)
return ((((self['@leftTop']):y()):_minus((self['@rightBottom']):y())):abs())
;
end
)
,"accessing","height\r\n\t\" Return the height of the receiver. \"\r\n\r\n\t^(leftTop y - rightBottom y) abs","(\r\nfunction(self)\r\nreturn ((((self['@leftTop']):y()):_minus((self['@rightBottom']):y())):abs())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"height:",0
,(
function(self,aNumber)
(self['@rightBottom']):y_(((self['@leftTop']):y()):_plus(aNumber))
;
return (self)
;
end
)
,"accessing","height: aNumber\r\n\t\" Change the height of the receiver. \"\r\n\r\n\trightBottom y: leftTop y + aNumber","(\r\nfunction(self,aNumber)\r\n(self['@rightBottom']):y_(((self['@leftTop']):y()):_plus(aNumber))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"insetBy:",0
,(
function(self,delta)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((delta):isRectangle()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@leftTop']):_plus((delta):leftTop())):rightBottom_((self['@rightBottom']):_minus((delta):rightBottom())))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@leftTop']):_plus(delta)):rightBottom_((self['@rightBottom']):_minus(delta)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","insetBy: delta\r\n\t\" Return the receiver inset by delta. \"\r\n\r\n\tdelta isRectangle ifTrue: [\r\n            ^leftTop + delta leftTop rightBottom: rightBottom - delta rightBottom\r\n\t].\r\n\t^leftTop + delta rightBottom: rightBottom - delta","(\r\nfunction(self,delta)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((delta):isRectangle()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@leftTop']):_plus((delta):leftTop())):rightBottom_((self['@rightBottom']):_minus((delta):rightBottom())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@leftTop']):_plus(delta)):rightBottom_((self['@rightBottom']):_minus(delta)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"intersect:",0
,(
function(self,aRectangle)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((aRectangle))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@leftTop']):max_((aRectangle):leftTop())):rightBottom_((self['@rightBottom']):min_((aRectangle):rightBottom())))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","intersect: aRectangle\r\n\t\" Return the rectangle in which the receiver and aRectangle overlap. \"\r\n\r\n\taRectangle isNil ifTrue: [ ^self ].\r\n\t^(leftTop max: aRectangle leftTop)\r\n        rightBottom: (rightBottom min: aRectangle rightBottom)","(\r\nfunction(self,aRectangle)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((aRectangle))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@leftTop']):max_((aRectangle):leftTop())):rightBottom_((self['@rightBottom']):min_((aRectangle):rightBottom())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"intersects:",0
,(
function(self,aRectangle)
return ((smalltalk:boolean_and_((((self['@leftTop']) < (aRectangle):rightBottom())),(function()
return ((((aRectangle):leftTop()) < self['@rightBottom']))
end))))
;
end
)
,"accessing","intersects: aRectangle\r\n\t\" Return true if the receiver and aRectangle have any area in common. \"\r\n\r\n\t^leftTop < aRectangle rightBottom\r\n\tand: [ aRectangle leftTop < rightBottom ]","(\r\nfunction(self,aRectangle)\r\nreturn ((smalltalk:boolean_and_((((self['@leftTop']) < (aRectangle):rightBottom())),(function()\r\nreturn ((((aRectangle):leftTop()) < self['@rightBottom']))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"isRectangle",0
,(
function(self)
return (true)
;
end
)
,"testing","isRectangle\r\n\t\" Return true if receiver is a Rectangle. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"left",0
,(
function(self)
return ((self['@leftTop']):x())
;
end
)
,"accessing","left\r\n\t\" Return the left of the receiver. \"\r\n\r\n\t^leftTop x","(\r\nfunction(self)\r\nreturn ((self['@leftTop']):x())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"leftBottom",0
,(
function(self)
return (((self):left()):_at((self):bottom()))
;
end
)
,"accessing","leftBottom\r\n\t\" Return the left bottom corner of the receiver. \"\r\n\r\n\t^self left @ self bottom","(\r\nfunction(self)\r\nreturn (((self):left()):_at((self):bottom()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"leftCenter",0
,(
function(self)
return (((self):left()):_at(((self):center()):y()))
;
end
)
,"accessing","leftCenter\r\n\t\" Return the center of the left edge of the receiver. \"\r\n\r\n\t^self left @ self center y","(\r\nfunction(self)\r\nreturn (((self):left()):_at(((self):center()):y()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"leftTop",0
,(
function(self)
return (self['@leftTop'])
;
end
)
,"accessing","leftTop\r\n\t\" Return the left top of the receiver. \"\r\n\r\n\t^leftTop","(\r\nfunction(self)\r\nreturn (self['@leftTop'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"leftTop:",0
,(
function(self,aPoint)
(function()
	local _s8_value=(aPoint)
	self['@leftTop']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","leftTop: aPoint\r\n\t\" Set the leftTop of the receiver.\"\r\n\r\n\tleftTop := aPoint","(\r\nfunction(self,aPoint)\r\n(function()\r\n\tlocal _s8_value=(aPoint)\r\n\tself['@leftTop']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"leftTop:rightBottom:",0
,(
function(self,leftTopPoint,rightBottomPoint)
(function()
	local _s8_value=(leftTopPoint)
	self['@leftTop']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(rightBottomPoint)
	self['@rightBottom']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","leftTop: leftTopPoint rightBottom: rightBottomPoint\r\n\t\"Set the receiver's coordinates. \"\r\n\r\n\tleftTop := leftTopPoint.\r\n\trightBottom := rightBottomPoint.","(\r\nfunction(self,leftTopPoint,rightBottomPoint)\r\n(function()\r\n\tlocal _s8_value=(leftTopPoint)\r\n\tself['@leftTop']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(rightBottomPoint)\r\n\tself['@rightBottom']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"merge:",0
,(
function(self,aRectangle)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((aRectangle))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self['@leftTop']):min_((aRectangle):leftTop())):rightBottom_((self['@rightBottom']):max_((aRectangle):rightBottom())))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","merge: aRectangle\r\n\t\" Return a Rectangle representing the area in which the receiver and aRectangle merged together. \"\r\n\r\n\taRectangle isNil ifTrue: [ ^self ].\r\n\t^(leftTop min: aRectangle leftTop) rightBottom: (rightBottom max: aRectangle rightBottom)","(\r\nfunction(self,aRectangle)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((aRectangle))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self['@leftTop']):min_((aRectangle):leftTop())):rightBottom_((self['@rightBottom']):max_((aRectangle):rightBottom())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"moveBy:",0
,(
function(self,aPoint)
(function()
	local _s8_value=((self['@leftTop']):_plus(aPoint))
	self['@leftTop']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self['@rightBottom']):_plus(aPoint))
	self['@rightBottom']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","moveBy: aPoint\r\n\t\" Move the receiver by aPoint. \"\r\n\r\n\tleftTop := leftTop + aPoint.\r\n\trightBottom := rightBottom + aPoint","(\r\nfunction(self,aPoint)\r\n(function()\r\n\tlocal _s8_value=((self['@leftTop']):_plus(aPoint))\r\n\tself['@leftTop']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@rightBottom']):_plus(aPoint))\r\n\tself['@rightBottom']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"moveTo:",0
,(
function(self,aPoint)
(self):moveBy_((aPoint):_minus(self['@leftTop']))
;
return (self)
;
end
)
,"accessing","moveTo: aPoint\r\n\t\" Move the receiver to aPoint. \"\r\n\r\n\tself moveBy: aPoint - leftTop","(\r\nfunction(self,aPoint)\r\n(self):moveBy_((aPoint):_minus(self['@leftTop']))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"moveToLeftTop:",0
,(
function(self,aPoint)
(self):moveBy_((aPoint):_minus((self):leftTop()))
;
return (self)
;
end
)
,"accessing","moveToLeftTop: aPoint\r\n\t\" Move the receiver to leftTop. \"\r\n\r\n\tself moveBy: aPoint - self leftTop","(\r\nfunction(self,aPoint)\r\n(self):moveBy_((aPoint):_minus((self):leftTop()))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"nonIntersections:",0
,(
function(self,aRectangle)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local areas=nil
(function() if ((((self):intersects_(aRectangle)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Array):with_(self))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((smalltalk:boolean_and_(((aRectangle):containsPoint_(self['@leftTop'])),(function()
return ((aRectangle):containsPoint_(self['@rightBottom']))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:newArray({}))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((smalltalk.Array):with_with_with_with_((function() if ((((((aRectangle):top()) > (self):top())))==true) then return ((function()
return ((self['@leftTop']):rightBottom_(((self['@rightBottom']):x()):_at((aRectangle):top())))
end))() else return nil end end)(), (function() if ((((((aRectangle):bottom()) < (self):bottom())))==true) then return ((function()
return ((((self['@leftTop']):x()):_at((aRectangle):bottom())):rightBottom_(self['@rightBottom']))
end))() else return nil end end)(), (function() if ((((((aRectangle):left()) > (self):left())))==true) then return ((function()
return ((((self['@leftTop']):x()):_at(((self['@leftTop']):y()):max_((aRectangle):top()))):rightBottom_(((aRectangle):left()):_at(((self['@rightBottom']):y()):min_((aRectangle):bottom()))))
end))() else return nil end end)(), (function() if ((((((aRectangle):right()) < (self):right())))==true) then return ((function()
return ((((aRectangle):right()):_at(((self['@leftTop']):y()):max_((aRectangle):top()))):rightBottom_(((self['@rightBottom']):x()):_at(((self['@rightBottom']):y()):min_((aRectangle):bottom()))))
end))() else return nil end end)())):select_((function(each)
return ((((each))~=nil))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","nonIntersections: aRectangle\r\n\t\" Return the rectangles describing areas of the receiver outside aRectangle. \"\r\n\r\n\t| areas |\r\n\t(self intersects: aRectangle) ifFalse: [ ^Array with: self ].\r\n\t((aRectangle containsPoint: leftTop)\r\n\t\tand: [ aRectangle containsPoint: rightBottom ])\r\n\t\tifTrue: [ ^#() ].\r\n\t^(Array\r\n\twith: (aRectangle top > self top ifTrue: [ leftTop rightBottom: rightBottom x @ aRectangle top ])\r\n\twith: (aRectangle bottom < self bottom ifTrue: [ leftTop x @ aRectangle bottom rightBottom: rightBottom ])\r\n\twith: (aRectangle left > self left ifTrue: [ leftTop x @ (leftTop y max: aRectangle top)\r\n                rightBottom: aRectangle left @ (rightBottom y min: aRectangle bottom) ])\r\n\twith: (aRectangle right < self right ifTrue: [ aRectangle right @ (leftTop y max: aRectangle top)\r\n                rightBottom: rightBottom x @ (rightBottom y min: aRectangle bottom) ])\r\n\t) select: [:each| each notNil ]","(\r\nfunction(self,aRectangle)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal areas=nil\r\n(function() if ((((self):intersects_(aRectangle)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Array):with_(self))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((aRectangle):containsPoint_(self['@leftTop'])),(function()\r\nreturn ((aRectangle):containsPoint_(self['@rightBottom']))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:newArray({}))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((smalltalk.Array):with_with_with_with_((function() if ((((((aRectangle):top()) > (self):top())))==true) then return ((function()\r\nreturn ((self['@leftTop']):rightBottom_(((self['@rightBottom']):x()):_at((aRectangle):top())))\r\nend))() else return nil end end)(), (function() if ((((((aRectangle):bottom()) < (self):bottom())))==true) then return ((function()\r\nreturn ((((self['@leftTop']):x()):_at((aRectangle):bottom())):rightBottom_(self['@rightBottom']))\r\nend))() else return nil end end)(), (function() if ((((((aRectangle):left()) > (self):left())))==true) then return ((function()\r\nreturn ((((self['@leftTop']):x()):_at(((self['@leftTop']):y()):max_((aRectangle):top()))):rightBottom_(((aRectangle):left()):_at(((self['@rightBottom']):y()):min_((aRectangle):bottom()))))\r\nend))() else return nil end end)(), (function() if ((((((aRectangle):right()) < (self):right())))==true) then return ((function()\r\nreturn ((((aRectangle):right()):_at(((self['@leftTop']):y()):max_((aRectangle):top()))):rightBottom_(((self['@rightBottom']):x()):_at(((self['@rightBottom']):y()):min_((aRectangle):bottom()))))\r\nend))() else return nil end end)())):select_((function(each)\r\nreturn ((((each))~=nil))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"toString",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:toString_((self['@leftTop'])))," to: ")),smalltalk:toString_((self['@rightBottom']))))
;
end
)
,"printing","toString\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^leftTop toString ,' to: ' ,rightBottom toString","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:toString_((self['@leftTop']))),\" to: \")),smalltalk:toString_((self['@rightBottom']))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"relativeTo:",0
,(
function(self,aRectangle)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local extent=nil
(function()
	local _s8_value=((aRectangle):extent())
	extent=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:boolean_or_(((smalltalk:isObject_equalTo_((((aRectangle):extent()):x()),(0)))),(function()
return ((smalltalk:isObject_equalTo_((((aRectangle):extent()):y()),(0))))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((((self['@leftTop']):_minus((aRectangle):leftTop())):_slash(extent)):extent_(((self):extent()):_slash(extent)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","relativeTo: aRectangle\r\n\t\" Return a Rectangle proportionally relative to aRectangle. \"\r\n\r\n\t| extent |\r\n\textent := aRectangle extent.\r\n\t(aRectangle extent x = 0 or: [ aRectangle extent y = 0 ]) ifTrue: [ ^self ].\r\n\t^leftTop - aRectangle leftTop / extent extent: self extent / extent","(\r\nfunction(self,aRectangle)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal extent=nil\r\n(function()\r\n\tlocal _s8_value=((aRectangle):extent())\r\n\textent=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:boolean_or_(((smalltalk:isObject_equalTo_((((aRectangle):extent()):x()),(0)))),(function()\r\nreturn ((smalltalk:isObject_equalTo_((((aRectangle):extent()):y()),(0))))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((((self['@leftTop']):_minus((aRectangle):leftTop())):_slash(extent)):extent_(((self):extent()):_slash(extent)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"right",0
,(
function(self)
return ((self['@rightBottom']):x())
;
end
)
,"accessing","right\r\n\t\" Return the receiver's right side. \"\r\n\r\n\t^rightBottom x","(\r\nfunction(self)\r\nreturn ((self['@rightBottom']):x())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"rightBottom",0
,(
function(self)
return (self['@rightBottom'])
;
end
)
,"accessing","rightBottom\r\n\t\" Return the right bottom corner. \"\r\n\r\n\t^rightBottom","(\r\nfunction(self)\r\nreturn (self['@rightBottom'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"rightBottom:",0
,(
function(self,aPoint)
(function()
	local _s8_value=(aPoint)
	self['@rightBottom']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","rightBottom: aPoint\r\n\t\" Set the right bottom corner of the receiver. \"\r\n\r\n\trightBottom := aPoint","(\r\nfunction(self,aPoint)\r\n(function()\r\n\tlocal _s8_value=(aPoint)\r\n\tself['@rightBottom']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"rightCenter",0
,(
function(self)
return (((self):right()):_at(((self):center()):y()))
;
end
)
,"accessing","rightCenter\r\n\t\" Return the center of the right edge of the receiver. \"\r\n\r\n\t^self right @ self center y","(\r\nfunction(self)\r\nreturn (((self):right()):_at(((self):center()):y()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"rightTop",0
,(
function(self)
return (((self):right()):_at((self):top()))
;
end
)
,"accessing","rightTop\r\n        \" Return the right top of the receiver. \"\r\n\r\n\t^self right @ self top","(\r\nfunction(self)\r\nreturn (((self):right()):_at((self):top()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"rounded",0
,(
function(self)
(function()
	local _s8_value=((self['@leftTop']):rounded())
	self['@leftTop']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self['@rightBottom']):rounded())
	self['@rightBottom']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","rounded\r\n\t\" Set the receiver with coordinates rounded to integers. \"\r\n\r\n\tleftTop := leftTop rounded.\r\n\trightBottom := rightBottom rounded.","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=((self['@leftTop']):rounded())\r\n\tself['@leftTop']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@rightBottom']):rounded())\r\n\tself['@rightBottom']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"scaleBy:",0
,(
function(self,delta)
return (((self['@leftTop']):_star(delta)):rightBottom_((self['@rightBottom']):_star(delta)))
;
end
)
,"accessing","scaleBy: delta\r\n\t\" Return the receiver multiplied by delta. \"\r\n\r\n\t^leftTop * delta rightBottom: rightBottom * delta","(\r\nfunction(self,delta)\r\nreturn (((self['@leftTop']):_star(delta)):rightBottom_((self['@rightBottom']):_star(delta)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"scaleTo:",0
,(
function(self,aRectangle)
return ((((aRectangle):scaleBy_((self):extent())):truncated()):moveBy_((self):leftTop()))
;
end
)
,"accessing","scaleTo: aRectangle\r\n\t\" Return a Rectangle whose size is proportional to the receiver with ratios specified by aRectangle. \"\r\n\r\n\t^(aRectangle scaleBy: self extent) truncated moveBy: self leftTop","(\r\nfunction(self,aRectangle)\r\nreturn ((((aRectangle):scaleBy_((self):extent())):truncated()):moveBy_((self):leftTop()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"top",0
,(
function(self)
return ((self['@leftTop']):y())
;
end
)
,"accessing","top\r\n\t\" Return the top side of the receiver. \"\r\n\r\n\t^leftTop y","(\r\nfunction(self)\r\nreturn ((self['@leftTop']):y())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"topCenter",0
,(
function(self)
return ((((self):center()):x()):_at((self):top()))
;
end
)
,"accessing","topCenter\r\n\t\" Return the center of the top edge of the receiver. \"\r\n\r\n\t^self center x @ self top","(\r\nfunction(self)\r\nreturn ((((self):center()):x()):_at((self):top()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"topLeft",0
,(
function(self)
return (self['@leftTop'])
;
end
)
,"accessing","topLeft\r\n\t\" Return the top-left corner of the receiver. \"\r\n\r\n\t^leftTop","(\r\nfunction(self)\r\nreturn (self['@leftTop'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"topLeft:",0
,(
function(self,aPoint)
(function()
	local _s8_value=(aPoint)
	self['@leftTop']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","topLeft: aPoint\r\n\t\"Set the top left corner of the receiver. \"\r\n\r\n\tleftTop := aPoint","(\r\nfunction(self,aPoint)\r\n(function()\r\n\tlocal _s8_value=(aPoint)\r\n\tself['@leftTop']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"topRight",0
,(
function(self)
return (((self):right()):_at((self):top()))
;
end
)
,"accessing","topRight\r\n\t\" Return the top-right corner of the receiver. \"\r\n\r\n\t^self right @ self top","(\r\nfunction(self)\r\nreturn (((self):right()):_at((self):top()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"translateBy:",0
,(
function(self,delta)
return (((self['@leftTop']):_plus(delta)):rightBottom_((self['@rightBottom']):_plus(delta)))
;
end
)
,"accessing","translateBy: delta\r\n\t\" Return the receiver with position incremented by delta. \"\r\n\r\n\t^leftTop + delta rightBottom: rightBottom + delta","(\r\nfunction(self,delta)\r\nreturn (((self['@leftTop']):_plus(delta)):rightBottom_((self['@rightBottom']):_plus(delta)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"truncated",0
,(
function(self)
(function()
	local _s8_value=((self['@leftTop']):truncated())
	self['@leftTop']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self['@rightBottom']):truncated())
	self['@rightBottom']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","truncated\r\n\t\" Set the receiver with the coordinates truncated to integers. \"\r\n\r\n\tleftTop := leftTop truncated.\r\n\trightBottom := rightBottom truncated.","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=((self['@leftTop']):truncated())\r\n\tself['@leftTop']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@rightBottom']):truncated())\r\n\tself['@rightBottom']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"width",0
,(
function(self)
return ((((self['@rightBottom']):x()):_minus((self['@leftTop']):x())):abs())
;
end
)
,"accessing","width\r\n\t\" Return the width of the receiver. \"\r\n\r\n\t^(rightBottom x - leftTop x) abs","(\r\nfunction(self)\r\nreturn ((((self['@rightBottom']):x()):_minus((self['@leftTop']):x())):abs())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"width:",0
,(
function(self,anInteger)
(self['@rightBottom']):x_(((self['@leftTop']):x()):_plus(anInteger))
;
return (self)
;
end
)
,"accessing","width: anInteger\r\n\t\" Set the width of the receiver. \"\r\n\r\n\trightBottom x: leftTop x + anInteger","(\r\nfunction(self,anInteger)\r\n(self['@rightBottom']):x_(((self['@leftTop']):x()):_plus(anInteger))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Rectangle,"__tostring",0
,(
function(self)
return (smalltalk:toString_((self)))
;
end
)
,"Lua-printing","__tostring\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.EvaluableAction,",","_comma"
,(
function(self,anAction)
return ((smalltalk.ActionSequence):with_with_(self, anAction))
;
end
)
,"composing",", anAction\r\n\t\" Return the receiver composed with the argument. \"\r\n\r\n\t^ActionSequence with: self with: anAction","(\r\nfunction(self,anAction)\r\nreturn ((smalltalk.ActionSequence):with_with_(self, anAction))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message["$klass"],"selector:arguments:",0
,(
function(self,aString,anArray)
return ((
function(_s8_rec)
(_s8_rec):selector_(aString);
(_s8_rec):arguments_(anArray);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","selector: aString arguments: anArray\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tselector: aString;\r\n\t\targuments: anArray;\r\n\t\tyourself","(\r\nfunction(self,aString,anArray)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):selector_(aString);\r\n(_s8_rec):arguments_(anArray);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message["$klass"],"receiver:selector:arguments:",0
,(
function(self,anObject,aString,anArray)
return ((
function(_s8_rec)
(_s8_rec):receiver_(anObject);
(_s8_rec):selector_(aString);
(_s8_rec):arguments_(anArray);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","receiver: anObject selector: aString arguments: anArray\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\treceiver: anObject;\r\n\t\tselector: aString;\r\n\t\targuments: anArray;\r\n\t\tyourself","(\r\nfunction(self,anObject,aString,anArray)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):receiver_(anObject);\r\n(_s8_rec):selector_(aString);\r\n(_s8_rec):arguments_(anArray);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"selector",0
,(
function(self)
return (self['@selector'])
;
end
)
,"accessing","selector\r\n\t\" Return the selector of the receiver. \"\r\n\r\n\t^selector","(\r\nfunction(self)\r\nreturn (self['@selector'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"selector:",0
,(
function(self,aString)
(function()
	local _s8_value=(aString)
	self['@selector']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","selector: aString\r\n\t\" Set the selector of the receiver. \"\r\n\r\n\tselector := aString","(\r\nfunction(self,aString)\r\n(function()\r\n\tlocal _s8_value=(aString)\r\n\tself['@selector']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"arguments:",0
,(
function(self,anArray)
(function()
	local _s8_value=(anArray)
	self['@arguments']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","arguments: anArray\r\n\t\" Set the arguments of the receiver. \"\r\n\r\n\targuments := anArray","(\r\nfunction(self,anArray)\r\n(function()\r\n\tlocal _s8_value=(anArray)\r\n\tself['@arguments']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"arguments",0
,(
function(self)
return (self['@arguments'])
;
end
)
,"accessing","arguments\r\n\t\" Return the arguments of the receiver. \"\r\n\r\n\t^arguments","(\r\nfunction(self)\r\nreturn (self['@arguments'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"receiver",0
,(
function(self)
return (self['@receiver'])
;
end
)
,"accessing","receiver\r\n\t\" Return the receiver of the receiver. \"\r\n\r\n\t^receiver","(\r\nfunction(self)\r\nreturn (self['@receiver'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"receiver:",0
,(
function(self,anObject)
(function()
	local _s8_value=(anObject)
	self['@receiver']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","receiver: anObject\r\n\t\" Set the receiver of the receiver. \"\r\n\r\n\treceiver := anObject","(\r\nfunction(self,anObject)\r\n(function()\r\n\tlocal _s8_value=(anObject)\r\n\tself['@receiver']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"comment",0
,(
function(self)
return (((self)["@comment"]))
;
end
)
,"accessing","comment\r\n\t\" Return the comment of the receiver (or nil). \"\r\n\r\n\t^self basicAt: '@comment'","(\r\nfunction(self)\r\nreturn (((self)[\"@comment\"]))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"comment:",0
,(
function(self,aString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((aString))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):basicDelete_("@comment"))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() local _1_=(aString); (self)["@comment"]=_1_; return _1_ end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","comment: aString\r\n\t\" Set the comment of the receiver. \"\r\n\r\n\taString isNil ifTrue: [ ^self basicDelete: '@comment' ].\r\n\tself basicAt: '@comment' put: aString","(\r\nfunction(self,aString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((aString))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):basicDelete_(\"@comment\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() local _1_=(aString); (self)[\"@comment\"]=_1_; return _1_ end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"perform",0
,(
function(self)
return (((self):receiver()):perform_withArguments_((self):selector(), (self):arguments()))
;
end
)
,"private","perform\r\n\t\" Return the result of sending the receiver. \"\r\n\r\n\t^self receiver\r\n\t\tperform: self selector\r\n\t\twithArguments: self arguments","(\r\nfunction(self)\r\nreturn (((self):receiver()):perform_withArguments_((self):selector(), (self):arguments()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"performDeferred",0
,(
function(self)
((function()
return ((self):perform())
end)):valueDeferred()
;
return (self)
;
end
)
,"evaluating","performDeferred\r\n\t\" Evaluate the receiver deferred in time.\r\n\tReturn the receiver.\r\n\t\"\r\n\r\n\t[ self perform ] valueDeferred.","(\r\nfunction(self)\r\n((function()\r\nreturn ((self):perform())\r\nend)):valueDeferred()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"evaluate",0
,(
function(self)
return ((self):perform())
;
end
)
,"evaluating","evaluate\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t^self perform","(\r\nfunction(self)\r\nreturn ((self):perform())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"evaluateFor:",0
,(
function(self,anObject)
return ((self):evaluateWithArguments_((smalltalk.Array):with_(anObject)))
;
end
)
,"evaluating","evaluateFor: anObject\r\n\t\" Return the result of evaluating the receiver with anObject. \"\r\n\r\n\t^self evaluateWithArguments: (Array with: anObject)","(\r\nfunction(self,anObject)\r\nreturn ((self):evaluateWithArguments_((smalltalk.Array):with_(anObject)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"evaluateWithArguments:",0
,(
function(self,anArray)
return (((self):receiver()):perform_withArguments_((self):selector(), (self):collectArguments_(anArray)))
;
end
)
,"evaluating","evaluateWithArguments: anArray\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t^self receiver\r\n\t\tperform: self selector\r\n\t\twithArguments: (self collectArguments: anArray)","(\r\nfunction(self,anArray)\r\nreturn (((self):receiver()):perform_withArguments_((self):selector(), (self):collectArguments_(anArray)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"initializeArguments",0
,(
function(self)
(function()
	local _s8_value=((smalltalk.Array):new_(((self):selector()):argumentCount()))
	self['@arguments']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"private","initializeArguments\r\n\t\" Private - Initialize the arguments of the receiver. \"\r\n\r\n\targuments := Array new: self selector argumentCount","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new_(((self):selector()):argumentCount()))\r\n\tself['@arguments']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"argumentCount",0
,(
function(self)
(function() if ((((((self['@arguments']))==nil)))==true) then return ((function()
return ((self):initializeArguments())
end))() else return nil end end)()
;
return ((self['@arguments']):size())
;
end
)
,"private","argumentCount\r\n\t\" Return the number of arguments defined by the receiver. \"\r\n\r\n\targuments isNil ifTrue: [ self initializeArguments ].\r\n\t^arguments size","(\r\nfunction(self)\r\n(function() if ((((((self['@arguments']))==nil)))==true) then return ((function()\r\nreturn ((self):initializeArguments())\r\nend))() else return nil end end)()\r\n;\r\nreturn ((self['@arguments']):size())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"collectArguments:",0
,(
function(self,evaluationArguments)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:isObject_equalTo_(((self['@arguments']):size()),(evaluationArguments):size()))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (evaluationArguments)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((evaluationArguments):copyFrom_to_((1), ((self):argumentCount()):min_((evaluationArguments):size())))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","collectArguments: evaluationArguments\r\n\t\" Private - Return the arguments to evaluate the receiver. \"\r\n\r\n\targuments size = evaluationArguments size ifTrue: [ ^evaluationArguments ].\r\n\t^evaluationArguments copyFrom: 1 to: (self argumentCount min: evaluationArguments size)","(\r\nfunction(self,evaluationArguments)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:isObject_equalTo_(((self['@arguments']):size()),(evaluationArguments):size()))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (evaluationArguments)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((evaluationArguments):copyFrom_to_((1), ((self):argumentCount()):min_((evaluationArguments):size())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"resolve:with:",0
,(
function(self,what,aPoolOrBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aPoolOrBlock):isMemberOf_(smalltalk.PoolDictionary)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aPoolOrBlock):evaluateWith_(what))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((((what))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (what)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((self):isObject_(what)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (what)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((what):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aPoolOrBlock):at_ifAbsent_(what, (function()
return (what)
end)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (what)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"resolution","resolve: what with: aPoolOrBlock\r\n\t\" Private - Returns the receiver to be evaluated with mappings given by aPoolOrBlock. \"\r\n\r\n\t(aPoolOrBlock isMemberOf: PoolDictionary) ifFalse: [\r\n\t\t^aPoolOrBlock evaluateWith: what\r\n\t].\r\n\twhat isNil ifTrue: [ ^what ].\r\n\t(self isObject: what) ifFalse: [ ^what ].\r\n\twhat isString ifTrue: [ ^aPoolOrBlock at: what ifAbsent: [ what ] ].\r\n\t^what","(\r\nfunction(self,what,aPoolOrBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aPoolOrBlock):isMemberOf_(smalltalk.PoolDictionary)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aPoolOrBlock):evaluateWith_(what))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((((what))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (what)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((self):isObject_(what)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (what)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((what):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aPoolOrBlock):at_ifAbsent_(what, (function()\r\nreturn (what)\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (what)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"resolveWith:",0
,(
function(self,aPoolOrBlock)
local args=nil
(function()
	local _s8_value=((function() if ((((((self['@arguments']))~=nil)))==true) then return ((function()
return ((self['@arguments']):collect_((function(each)
return ((self):resolve_with_(each, aPoolOrBlock))
end)))
end))() else return nil end end)())
	args=_s8_value; return _s8_value
end)()
;
return (((self)["$klass"]):receiver_selector_arguments_((self):resolve_with_(self['@receiver'], aPoolOrBlock), (self):selector(), args))
;
end
)
,"resolution","resolveWith: aPoolOrBlock\r\n\t\" Return a message set to be evaluated with mappings given by aPoolOrBlock. \"\r\n\r\n\t| args |\r\n\targs := arguments notNil ifTrue: [\r\n\t\targuments collect: [:each|\r\n\t\t\tself resolve: each with: aPoolOrBlock\r\n\t\t]\r\n\t].\r\n\t^self class\r\n\t\treceiver: (self resolve: receiver with: aPoolOrBlock)\r\n\t\tselector: self selector\r\n\t\targuments: args","(\r\nfunction(self,aPoolOrBlock)\r\nlocal args=nil\r\n(function()\r\n\tlocal _s8_value=((function() if ((((((self['@arguments']))~=nil)))==true) then return ((function()\r\nreturn ((self['@arguments']):collect_((function(each)\r\nreturn ((self):resolve_with_(each, aPoolOrBlock))\r\nend)))\r\nend))() else return nil end end)())\r\n\targs=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (((self)[\"$klass\"]):receiver_selector_arguments_((self):resolve_with_(self['@receiver'], aPoolOrBlock), (self):selector(), args))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Message,"isMessage",0
,(
function(self)
return (true)
;
end
)
,"query","isMessage\r\n\t\" Return true if the receiver is a message. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LinkMessage,"resultAction",0
,(
function(self)
return (self['@resultAction'])
;
end
)
,"accessing","resultAction\r\n\t\" Return the resultAction of the receiver. \"\r\n\r\n\t^resultAction","(\r\nfunction(self)\r\nreturn (self['@resultAction'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LinkMessage,"resultAction:",0
,(
function(self,anObject)
(function()
	local _s8_value=(anObject)
	self['@resultAction']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","resultAction: anObject\r\n\t\" Set the resultAction of the receiver. \"\r\n\r\n\tresultAction := anObject","(\r\nfunction(self,anObject)\r\n(function()\r\n\tlocal _s8_value=(anObject)\r\n\tself['@resultAction']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LinkMessage,"evaluate",0
,(
function(self)
return ((self):evaluateWithArguments_(smalltalk:newArray({})))
;
end
)
,"evaluating","evaluate\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t^self evaluateWithArguments: #()","(\r\nfunction(self)\r\nreturn ((self):evaluateWithArguments_(smalltalk:newArray({})))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LinkMessage,"collectArguments:",0
,(
function(self,evaluationArguments)
return (((self):arguments()):collect_((function(each)
return ((each):evaluateWithArguments_(evaluationArguments))
end)))
;
end
)
,"evaluating","collectArguments: evaluationArguments\r\n\t\" Private - Return the arguments to evaluate the receiver. \"\r\n\r\n\t^self arguments collect: [:each| each evaluateWithArguments: evaluationArguments ]","(\r\nfunction(self,evaluationArguments)\r\nreturn (((self):arguments()):collect_((function(each)\r\nreturn ((each):evaluateWithArguments_(evaluationArguments))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence["$klass"],"withAll:",0
,(
function(self,aCollection)
return ((
function(_s8_rec)
(_s8_rec):contents_(aCollection);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","withAll: aCollection\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tcontents: aCollection;\r\n\t\tyourself","(\r\nfunction(self,aCollection)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):contents_(aCollection);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence["$klass"],"with:",0
,(
function(self,anObject)
return ((self):withAll_((smalltalk.Array):with_(anObject)))
;
end
)
,"instantiation","with: anObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self withAll: (Array with: anObject)","(\r\nfunction(self,anObject)\r\nreturn ((self):withAll_((smalltalk.Array):with_(anObject)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence["$klass"],"with:with:",0
,(
function(self,anObject,bObject)
return ((self):withAll_((smalltalk.Array):with_with_(anObject, bObject)))
;
end
)
,"instantiation","with: anObject with: bObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self withAll: (Array with: anObject with: bObject)","(\r\nfunction(self,anObject,bObject)\r\nreturn ((self):withAll_((smalltalk.Array):with_with_(anObject, bObject)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence["$klass"],"with:with:with:",0
,(
function(self,anObject,bObject,cObject)
return ((self):withAll_((smalltalk.Array):with_with_with_(anObject, bObject, cObject)))
;
end
)
,"instantiation","with: anObject with: bObject with: cObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self withAll: (Array with: anObject with: bObject with: cObject)","(\r\nfunction(self,anObject,bObject,cObject)\r\nreturn ((self):withAll_((smalltalk.Array):with_with_with_(anObject, bObject, cObject)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"contents:",0
,(
function(self,aCollection)
(function()
	local _s8_value=(aCollection)
	self['@contents']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"private","contents: aCollection\r\n\t\" Private - Set the contents of the receiver. \"\r\n\r\n\tcontents := aCollection","(\r\nfunction(self,aCollection)\r\n(function()\r\n\tlocal _s8_value=(aCollection)\r\n\tself['@contents']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"contents",0
,(
function(self)
(function() if ((((((self['@contents']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.Array):new())
	self['@contents']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (self['@contents'])
;
end
)
,"private","contents\r\n\t\" Private - Return the contents of the receiver. \"\r\n\r\n\tcontents isNil ifTrue: [ contents := Array new ].\r\n\t^contents","(\r\nfunction(self)\r\n(function() if ((((((self['@contents']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tself['@contents']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@contents'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"isEmpty",0
,(
function(self)
return ((smalltalk:boolean_or_(((((self['@contents']))==nil)),(function()
return ((self['@contents']):isEmpty())
end))))
;
end
)
,"testing","isEmpty\r\n\t\" Return true if the receiver is empty. \"\r\n\r\n\t^contents isNil or: [ contents isEmpty ]","(\r\nfunction(self)\r\nreturn ((smalltalk:boolean_or_(((((self['@contents']))==nil)),(function()\r\nreturn ((self['@contents']):isEmpty())\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"notEmpty",0
,(
function(self)
return ((smalltalk:boolean_and_(((((self['@contents']))~=nil)),(function()
return ((self['@contents']):notEmpty())
end))))
;
end
)
,"testing","notEmpty\r\n\t\" Return true if the receiver is not empty. \"\r\n\r\n\t^contents notNil and: [ contents notEmpty ]","(\r\nfunction(self)\r\nreturn ((smalltalk:boolean_and_(((((self['@contents']))~=nil)),(function()\r\nreturn ((self['@contents']):notEmpty())\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"asArray",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:newArray({}))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self['@contents']):asArray())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","asArray\r\n\t\" Return an array with the contents of the receiver. \"\r\n\r\n\tself isEmpty ifTrue: [ ^#() ].\r\n\t^contents asArray","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:newArray({}))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self['@contents']):asArray())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"asActionSequence",0
,(
function(self)
return (self)
;
end
)
,"converting","asActionSequence\r\n\t\" Return an ActionSequence from the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,",","_comma"
,(
function(self,anAction)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((anAction):asActionSequence())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self)["$klass"]):withAll_(smalltalk:object_comma_(((self):contents()),((anAction):asActionSequence()):contents())))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting",", anAction\r\n\t\" Return an action composed with the receiver and anAction. \"\r\n\r\n\tself isEmpty ifTrue: [ ^anAction asActionSequence ].\r\n\t^self class withAll: self contents , anAction asActionSequence contents","(\r\nfunction(self,anAction)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((anAction):asActionSequence())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self)[\"$klass\"]):withAll_(smalltalk:object_comma_(((self):contents()),((anAction):asActionSequence()):contents())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"copyWith:",0
,(
function(self,anObject)
return (((self)["$klass"]):withAll_(((self):contents()):copyWith_(anObject)))
;
end
)
,"converting","copyWith: anObject\r\n\t\" Return an action composed with the receiver and anObject. \"\r\n\r\n\t^self class withAll: (self contents copyWith: anObject)","(\r\nfunction(self,anObject)\r\nreturn (((self)[\"$klass\"]):withAll_(((self):contents()):copyWith_(anObject)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"do:",0
,(
function(self,aBlock)
return ((function() if ((((self):notEmpty()))==true) then return ((function()
return (((self):contents()):do_(aBlock))
end))() else return nil end end)())
;
end
)
,"converting","do: aBlock\r\n\t\" Evaluate aBlock for each object in the receiver's contents. \"\r\n\r\n\t^self notEmpty ifTrue: [ self contents do: aBlock ]","(\r\nfunction(self,aBlock)\r\nreturn ((function() if ((((self):notEmpty()))==true) then return ((function()\r\nreturn (((self):contents()):do_(aBlock))\r\nend))() else return nil end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"includes:",0
,(
function(self,anObject)
return ((smalltalk:boolean_and_(((self):notEmpty()),(function()
return (((self):contents()):includes_(anObject))
end))))
;
end
)
,"testing","includes: anObject\r\n\t\" Return true if anObject is in the receiver. \"\r\n\r\n\t^self notEmpty and: [ self contents includes: anObject ]","(\r\nfunction(self,anObject)\r\nreturn ((smalltalk:boolean_and_(((self):notEmpty()),(function()\r\nreturn (((self):contents()):includes_(anObject))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"select:",0
,(
function(self,aBlock)
return (((self)["$klass"]):withAll_(((self):asArray()):select_(aBlock)))
;
end
)
,"enumerating","select: aBlock\r\n\t\" Return the a copy of the receiver with elements wich evaluates aBlock to true. \"\r\n\r\n\t^self class withAll: (self asArray select: aBlock)","(\r\nfunction(self,aBlock)\r\nreturn (((self)[\"$klass\"]):withAll_(((self):asArray()):select_(aBlock)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"reject:",0
,(
function(self,aBlock)
return (((self)["$klass"]):withAll_(((self):asArray()):reject_(aBlock)))
;
end
)
,"enumerating","reject: aBlock\r\n\t\" Return the a copy of the receiver with elements wich evaluates aBlock to false. \"\r\n\r\n\t^self class withAll: (self asArray reject: aBlock)","(\r\nfunction(self,aBlock)\r\nreturn (((self)[\"$klass\"]):withAll_(((self):asArray()):reject_(aBlock)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"evaluate",0
,(
function(self)
local result=nil
(self):do_((function(each)
return ((function()
	local _s8_value=((each):evaluate())
	result=_s8_value; return _s8_value
end)())
end))
;
return (result)
;
end
)
,"evaluating","evaluate\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t| result |\r\n\tself do: [:each| result := each evaluate ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(self):do_((function(each)\r\nreturn ((function()\r\n\tlocal _s8_value=((each):evaluate())\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"evaluateWithArguments:",0
,(
function(self,anArray)
local result=nil
(self):do_((function(each)
return ((function()
	local _s8_value=((each):evaluateWithArguments_(anArray))
	result=_s8_value; return _s8_value
end)())
end))
;
return (result)
;
end
)
,"evaluating","evaluateWithArguments: anArray\r\n\t\" Return the result of evaluating the receiver. \"\r\n\r\n\t| result |\r\n\tself do: [:each| result := each evaluateWithArguments: anArray ].\r\n\t^result","(\r\nfunction(self,anArray)\r\nlocal result=nil\r\n(self):do_((function(each)\r\nreturn ((function()\r\n\tlocal _s8_value=((each):evaluateWithArguments_(anArray))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"evaluateFor:",0
,(
function(self,anObject)
local result=nil
(self):do_((function(each)
return ((function()
	local _s8_value=((each):evaluateFor_(anObject))
	result=_s8_value; return _s8_value
end)())
end))
;
return (result)
;
end
)
,"evaluating","evaluateFor: anObject\r\n\t\" Return the result of evaluating the receiver with anObject. \"\r\n\r\n\t| result |\r\n\tself do: [:each| result := each evaluateFor: anObject ].\r\n\t^result","(\r\nfunction(self,anObject)\r\nlocal result=nil\r\n(self):do_((function(each)\r\nreturn ((function()\r\n\tlocal _s8_value=((each):evaluateFor_(anObject))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"reduced",0
,(
function(self)
return ((function() if ((((self):notEmpty()))==true) then return ((function()
return ((function() if ((((smalltalk:isObject_equalTo_(((self['@contents']):size()),(1)))))==true) then return ((function()
return ((self['@contents']):first())
end))() else return ((function()
return (self)
end))() end end)())
end))() else return nil end end)())
;
end
)
,"events","reduced\r\n\t\" Return the receiver reduced to minimal evaluable expression. \"\r\n\r\n\t^self notEmpty ifTrue: [\r\n\t\tcontents size = 1\r\n\t\tifTrue: [ contents first ]\r\n\t\tifFalse: [ self ]\r\n\t]","(\r\nfunction(self)\r\nreturn ((function() if ((((self):notEmpty()))==true) then return ((function()\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_(((self['@contents']):size()),(1)))))==true) then return ((function()\r\nreturn ((self['@contents']):first())\r\nend))() else return ((function()\r\nreturn (self)\r\nend))() end end)())\r\nend))() else return nil end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"evaluateWithSelector:withArguments:",0
,(
function(self,aSelector,array)
local result=nil
(self):do_((function(each)
return ((function()
	local _s8_value=(((each):receiver()):perform_withArguments_(aSelector, array))
	result=_s8_value; return _s8_value
end)())
end))
;
return (result)
;
end
)
,"changes","evaluateWithSelector: aSelector withArguments: array\r\n\t\" Private - Return the result of evaluating the receiver with aSelector and arguments array. \"\r\n\r\n\t| result |\r\n\tself do: [:each| result := each receiver perform: aSelector withArguments: array ].\r\n\t^result","(\r\nfunction(self,aSelector,array)\r\nlocal result=nil\r\n(self):do_((function(each)\r\nreturn ((function()\r\n\tlocal _s8_value=(((each):receiver()):perform_withArguments_(aSelector, array))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ActionSequence,"resolveWith:",0
,(
function(self,aPoolOrBlock)
return (((self)["$klass"]):withAll_(((self):asArray()):collect_((function(each)
return ((function() if (((smalltalk:responds_to_((each),("resolveWith:"))))==true) then return ((function()
return ((each):resolveWith_(aPoolOrBlock))
end))() else return ((function()
return (each)
end))() end end)())
end))))
;
end
)
,"resolution","resolveWith: aPoolOrBlock\r\n\t\" Return an instance set to be evaluated with mappings given by aPoolOrBlock. \"\r\n\r\n\t^self class withAll: (self asArray collect: [:each|\r\n\t\t(each respondsTo: #resolveWith:)\r\n\t\tifTrue: [ each resolveWith: aPoolOrBlock ]\r\n\t\tifFalse: [ each ]\r\n\t])","(\r\nfunction(self,aPoolOrBlock)\r\nreturn (((self)[\"$klass\"]):withAll_(((self):asArray()):collect_((function(each)\r\nreturn ((function() if (((smalltalk:responds_to_((each),(\"resolveWith:\"))))==true) then return ((function()\r\nreturn ((each):resolveWith_(aPoolOrBlock))\r\nend))() else return ((function()\r\nreturn (each)\r\nend))() end end)())\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.EventManager["$klass"],"eventsTriggered",0
,(
function(self)
(function() if ((((((self['@eventsTriggered']))==nil)))==true) then return ((function()
return ((self):initializeEventsTriggered())
end))() else return nil end end)()
;
return (self['@eventsTriggered'])
;
end
)
,"events","eventsTriggered\r\n\t\" Return all the events managed by instances of the receiver. \"\r\n\r\n\teventsTriggered isNil ifTrue: [ self initializeEventsTriggered ].\r\n\t^eventsTriggered","(\r\nfunction(self)\r\n(function() if ((((((self['@eventsTriggered']))==nil)))==true) then return ((function()\r\nreturn ((self):initializeEventsTriggered())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@eventsTriggered'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.EventManager["$klass"],"eventsTriggered:",0
,(
function(self,aCollection)
(function()
	local _s8_value=(aCollection)
	self['@eventsTriggered']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"events","eventsTriggered: aCollection\r\n\t\" Set all the events managed by instances of the receiver. \"\r\n\r\n\teventsTriggered := aCollection","(\r\nfunction(self,aCollection)\r\n(function()\r\n\tlocal _s8_value=(aCollection)\r\n\tself['@eventsTriggered']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.EventManager["$klass"],"initializeEventsTriggered",0
,(
function(self)
(self):eventsTriggered_((self):constructEventsTriggered())
;
return (self)
;
end
)
,"events","initializeEventsTriggered\r\n\t\" Private - Initialize all the events managed by instances of the receiver. \"\r\n\r\n\tself eventsTriggered: self constructEventsTriggered","(\r\nfunction(self)\r\n(self):eventsTriggered_((self):constructEventsTriggered())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.EventManager,"eventTable",0
,(
function(self)
return ((function() if ((((((self['@handlers']))==nil)))==true) then return ((function()
return ((self):eventTableCreate())
end))() else return ((function()
return (self['@handlers'])
end))() end end)())
;
end
)
,"events","eventTable\r\n\t\" Private - Return the mapping event names to actions of the receiver. \"\r\n\r\n\t^handlers isNil\r\n\t\tifTrue: [ self eventTableCreate ]\r\n\t\tifFalse: [ handlers ]","(\r\nfunction(self)\r\nreturn ((function() if ((((((self['@handlers']))==nil)))==true) then return ((function()\r\nreturn ((self):eventTableCreate())\r\nend))() else return ((function()\r\nreturn (self['@handlers'])\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.EventManager,"eventTableForEdit",0
,(
function(self)
(function() if ((((((self['@handlers']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((self):eventTableCreate())
	self['@handlers']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (self['@handlers'])
;
end
)
,"events","eventTableForEdit\r\n\t\" Private - Return the mapping event names to actions of the receiver.\r\n\tSet the internal map if not present.\r\n\t\"\r\n\r\n\thandlers isNil ifTrue: [ handlers := self eventTableCreate ].\r\n\t^handlers","(\r\nfunction(self)\r\n(function() if ((((((self['@handlers']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((self):eventTableCreate())\r\n\tself['@handlers']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@handlers'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.EventManager,"releaseEventTable",0
,(
function(self)
(function()
	local _s8_value=(nil)
	self['@handlers']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"events","releaseEventTable\r\n\t\" Private - Release the event table. \"\r\n\r\n\thandlers := nil","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=(nil)\r\n\tself['@handlers']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder["$klass"],"for:",0
,(
function(self,aCompiler)
return ((((self):support()):new()):compiler_(aCompiler))
;
end
)
,"instantiation","for: aCompiler\r\n\t\" Private - Returns an instance of the receiver. \"\r\n\r\n\t^self support new compiler: aCompiler","(\r\nfunction(self,aCompiler)\r\nreturn ((((self):support()):new()):compiler_(aCompiler))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder["$klass"],"support",0
,(
function(self)
local lang=nil
(function()
	local _s8_value=((smalltalk.Smalltalk):nativeLanguage())
	lang=_s8_value; return _s8_value
end)()
;
return (((self):allSubclasses()):detect_ifNone_((function(cls)
return ((cls):supportsLanguage_(lang))
end), (function()
return ((self):invalidMessage_("support"))
end)))
;
end
)
,"instantiation","support\r\n\t\" Private - Return the support to encode for native platform. \"\r\n\r\n\t| lang |\r\n\tlang := Smalltalk nativeLanguage.\r\n\t^self allSubclasses\r\n\t\tdetect: [:cls| cls supportsLanguage: lang ]\r\n\t\tifNone: [ self invalidMessage: #support ]","(\r\nfunction(self)\r\nlocal lang=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):nativeLanguage())\r\n\tlang=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (((self):allSubclasses()):detect_ifNone_((function(cls)\r\nreturn ((cls):supportsLanguage_(lang))\r\nend), (function()\r\nreturn ((self):invalidMessage_(\"support\"))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder["$klass"],"supportsLanguage:",0
,(
function(self,aLanguage)
return (false)
;
end
)
,"instantiation","supportsLanguage: aLanguage\r\n\t\" Private - Return true if instances of the receiver can encode to aLanguage. \"\r\n\r\n\t^false","(\r\nfunction(self,aLanguage)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder["$klass"],"smalltalkNameFor:",0
,(
function(self,aClass)
return ((self):implementedBySubclass_("smalltalkNameFor:"))
;
end
)
,"export","smalltalkNameFor: aClass\r\n\t\" Private - Return the global name of aClass (can be nil). \"\r\n\r\n\t^self implementedBySubclass: #smalltalkNameFor: ","(\r\nfunction(self,aClass)\r\nreturn ((self):implementedBySubclass_(\"smalltalkNameFor:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"smalltalkNameFor:",0
,(
function(self,aClass)
return (((self)["$klass"]):smalltalkNameFor_(aClass))
;
end
)
,"export","smalltalkNameFor: aClass\r\n\t\" Private - Return the global name of aClass (can be nil). \"\r\n\r\n\t^self class smalltalkNameFor: aClass","(\r\nfunction(self,aClass)\r\nreturn (((self)[\"$klass\"]):smalltalkNameFor_(aClass))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"compiler",0
,(
function(self)
return (self['@compiler'])
;
end
)
,"accessing","compiler\r\n\t\" Return the compiler of the receiver. \"\r\n\r\n\t^compiler","(\r\nfunction(self)\r\nreturn (self['@compiler'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"compiler:",0
,(
function(self,aCompiler)
(function()
	local _s8_value=(aCompiler)
	self['@compiler']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","compiler: aCompiler\r\n\t\" Set the compiler of the receiver. \"\r\n\r\n\tcompiler := aCompiler","(\r\nfunction(self,aCompiler)\r\n(function()\r\n\tlocal _s8_value=(aCompiler)\r\n\tself['@compiler']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"compilerEvent:",0
,(
function(self,anEvent)
return ((self['@compiler']):triggerEvent_(anEvent))
;
end
)
,"events","compilerEvent: anEvent\r\n\t\" Return the result of triggering the compiler's event. \"\r\n\r\n\t^compiler triggerEvent: anEvent","(\r\nfunction(self,anEvent)\r\nreturn ((self['@compiler']):triggerEvent_(anEvent))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"compilerEvent:ifNotHandled:",0
,(
function(self,anEvent,aBlock)
return ((self['@compiler']):triggerEvent_ifNotHandled_(anEvent, aBlock))
;
end
)
,"events","compilerEvent: anEvent ifNotHandled: aBlock\r\n\t\" Return the result of triggering the compiler's event. \"\r\n\r\n\t^compiler triggerEvent: anEvent ifNotHandled: aBlock","(\r\nfunction(self,anEvent,aBlock)\r\nreturn ((self['@compiler']):triggerEvent_ifNotHandled_(anEvent, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"compilerEvent:with:",0
,(
function(self,anEvent,anObject)
return ((self['@compiler']):triggerEvent_with_(anEvent, anObject))
;
end
)
,"events","compilerEvent: anEvent with: anObject\r\n\t\" Return the result of triggering the compiler's event with anObject. \"\r\n\r\n\t^compiler\r\n\t\ttriggerEvent: anEvent\r\n\t\twith: anObject","(\r\nfunction(self,anEvent,anObject)\r\nreturn ((self['@compiler']):triggerEvent_with_(anEvent, anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"compilerEvent:with:ifNotHandled:",0
,(
function(self,anEvent,anObject,aBlock)
return ((self['@compiler']):triggerEvent_with_ifNotHandled_(anEvent, anObject, aBlock))
;
end
)
,"events","compilerEvent: anEvent with: anObject ifNotHandled: aBlock\r\n\t\" Return the result of triggering the compiler's event with anObject. \"\r\n\r\n\t^compiler\r\n\t\ttriggerEvent: anEvent\r\n\t\twith: anObject\r\n\t\tifNotHandled: aBlock","(\r\nfunction(self,anEvent,anObject,aBlock)\r\nreturn ((self['@compiler']):triggerEvent_with_ifNotHandled_(anEvent, anObject, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"compilerEvent:withArguments:",0
,(
function(self,anEvent,array)
return ((self['@compiler']):triggerEvent_withArguments_(anEvent, array))
;
end
)
,"events","compilerEvent: anEvent withArguments: array\r\n\t\" Return the result of triggering the compiler's event with the arguments. \"\r\n\r\n\t^compiler\r\n\t\ttriggerEvent: anEvent\r\n\t\twithArguments: array","(\r\nfunction(self,anEvent,array)\r\nreturn ((self['@compiler']):triggerEvent_withArguments_(anEvent, array))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"compilerEvent:withArguments:ifNotHandled:",0
,(
function(self,anEvent,array,aBlock)
return ((self['@compiler']):triggerEvent_withArguments_ifNotHandled_(anEvent, array, aBlock))
;
end
)
,"events","compilerEvent: anEvent withArguments: array ifNotHandled: aBlock\r\n\t\" Return the result of triggering the compiler's event with the arguments. \"\r\n\r\n\t^compiler\r\n\t\ttriggerEvent: anEvent\r\n\t\twithArguments: array\r\n\t\tifNotHandled: aBlock","(\r\nfunction(self,anEvent,array,aBlock)\r\nreturn ((self['@compiler']):triggerEvent_withArguments_ifNotHandled_(anEvent, array, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"currentClass",0
,(
function(self)
return ((self['@compiler']):currentClass())
;
end
)
,"accessing","currentClass\r\n\t\" Return the current class of the receiver. \"\r\n\r\n\t^compiler currentClass","(\r\nfunction(self)\r\nreturn ((self['@compiler']):currentClass())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"ignore:",0
,(
function(self,aNode)
return (self)
;
end
)
,"visiting","ignore: aNode","(\r\nfunction(self,aNode)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"knownVariables",0
,(
function(self)
return ((self['@compiler']):knownVariables())
;
end
)
,"variables","knownVariables\r\n\t\" Return the knownVariables of the receiver. \"\r\n\r\n\t^compiler knownVariables","(\r\nfunction(self)\r\nreturn ((self['@compiler']):knownVariables())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"localName:",0
,(
function(self,aName)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((self):pseudoVariables()):includes_(aName)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (aName)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aName):asSelector())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","localName: aName\r\n\t\" Private - Return a valid (native, local) name for aName. \"\r\n\r\n\t(self pseudoVariables includes: aName) ifTrue: [ ^aName ].\r\n\t^aName asSelector","(\r\nfunction(self,aName)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((self):pseudoVariables()):includes_(aName)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (aName)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aName):asSelector())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"nativeLanguage",0
,(
function(self)
return (nil)
;
end
)
,"accessing","nativeLanguage\r\n\t\" Return the nativeLanguage of the receiver.\r\n\tThe default action returns nil (the default native language).\r\n\t\"\r\n\r\n\t^nil","(\r\nfunction(self)\r\nreturn (nil)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"emmitedCode",0
,(
function(self)
return ((self):implementedBySubclass_("emmitedCode"))
;
end
)
,"results","emmitedCode\r\n\t\" Return the emmited code during compilation (or nil). \"\r\n\r\n\t^self implementedBySubclass: #emmitedCode","(\r\nfunction(self)\r\nreturn ((self):implementedBySubclass_(\"emmitedCode\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"emmitCode:",0
,(
function(self,aString)
return ((self):implementedBySubclass_("emmitCode:"))
;
end
)
,"private","emmitCode: aString\r\n\t\" Dump aString onto code stream. \"\r\n\r\n\t^self implementedBySubclass: #emmitCode:","(\r\nfunction(self,aString)\r\nreturn ((self):implementedBySubclass_(\"emmitCode:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"endCompiling:",0
,(
function(self,aNode)
return (self)
;
end
)
,"private","endCompiling: aNode\r\n\t\" Private - Finish emmiting code for aNode. \"","(\r\nfunction(self,aNode)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"startCompiling:",0
,(
function(self,aNode)
return (self)
;
end
)
,"private","startCompiling: aNode\r\n\t\" Private - Set the receiver to start emmiting code for aNode. \"","(\r\nfunction(self,aNode)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"pseudoVariables",0
,(
function(self)
return ((self['@compiler']):pseudoVariables())
;
end
)
,"variables","pseudoVariables\r\n\t\" Return the pseudoVariables of the receiver. \"\r\n\r\n\t^compiler pseudoVariables","(\r\nfunction(self)\r\nreturn ((self['@compiler']):pseudoVariables())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"tempVariables",0
,(
function(self)
return ((self['@compiler']):tempVariables())
;
end
)
,"variables","tempVariables\r\n\t\" Return the tempVariables of the receiver. \"\r\n\r\n\t^compiler tempVariables","(\r\nfunction(self)\r\nreturn ((self['@compiler']):tempVariables())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"unknownVariables",0
,(
function(self)
return ((self['@compiler']):unknownVariables())
;
end
)
,"variables","unknownVariables\r\n\t\" Return the unknownVariables of the receiver. \"\r\n\r\n\t^compiler unknownVariables","(\r\nfunction(self)\r\nreturn ((self['@compiler']):unknownVariables())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visit:",0
,(
function(self,aNode)
(aNode):accept_(self)
;
return (self)
;
end
)
,"visiting","visit: aNode\r\n\taNode accept: self","(\r\nfunction(self,aNode)\r\n(aNode):accept_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitAssignmentNode:",0
,(
function(self,aNode)
(self):visitNode_(aNode)
;
return (self)
;
end
)
,"visiting","visitAssignmentNode: aNode\r\n\tself visitNode: aNode","(\r\nfunction(self,aNode)\r\n(self):visitNode_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitBlockNode:",0
,(
function(self,aNode)
(self):visitNode_(aNode)
;
return (self)
;
end
)
,"visiting","visitBlockNode: aNode\r\n\tself visitNode: aNode","(\r\nfunction(self,aNode)\r\n(self):visitNode_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitBlockSequenceNode:",0
,(
function(self,aNode)
(self):visitSequenceNode_(aNode)
;
return (self)
;
end
)
,"visiting","visitBlockSequenceNode: aNode\r\n\tself visitSequenceNode: aNode","(\r\nfunction(self,aNode)\r\n(self):visitSequenceNode_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitCascadeNode:",0
,(
function(self,aNode)
(self):visitNode_(aNode)
;
return (self)
;
end
)
,"visiting","visitCascadeNode: aNode\r\n\tself visitNode: aNode","(\r\nfunction(self,aNode)\r\n(self):visitNode_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitMacroAssignment:to:",0
,(
function(self,aVariableNode,anExpressionNode)
return (false)
;
end
)
,"visiting","visitMacroAssignment: aVariableNode to: anExpressionNode\r\n\t\" Return true if the receiver generated code to a macro assignment to aVariableNode. \"\r\n\r\n\t^false","(\r\nfunction(self,aVariableNode,anExpressionNode)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitMethodNode:",0
,(
function(self,aNode)
(self):visitNode_(aNode)
;
return (self)
;
end
)
,"visiting","visitMethodNode: aNode\r\n\t\" Visit a method node. \"\r\n\r\n\tself visitNode: aNode","(\r\nfunction(self,aNode)\r\n(self):visitNode_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitNode:",0
,(
function(self,aNode)
(self):ignore_(aNode)
;
return (self)
;
end
)
,"visiting","visitNode: aNode\r\n\tself ignore: aNode","(\r\nfunction(self,aNode)\r\n(self):ignore_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitReturnNode:",0
,(
function(self,aNode)
(self):visitNode_(aNode)
;
return (self)
;
end
)
,"visiting","visitReturnNode: aNode\r\n\tself visitNode: aNode","(\r\nfunction(self,aNode)\r\n(self):visitNode_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitSendNode:",0
,(
function(self,aNode)
(self):visitNode_(aNode)
;
return (self)
;
end
)
,"visiting","visitSendNode: aNode\r\n\tself visitNode: aNode","(\r\nfunction(self,aNode)\r\n(self):visitNode_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitSequenceNode:",0
,(
function(self,aNode)
(self):visitNode_(aNode)
;
return (self)
;
end
)
,"visiting","visitSequenceNode: aNode\r\n\tself visitNode: aNode","(\r\nfunction(self,aNode)\r\n(self):visitNode_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitValueNode:",0
,(
function(self,aNode)
(self):visitNode_(aNode)
;
return (self)
;
end
)
,"visiting","visitValueNode: aNode\r\n\tself visitNode: aNode","(\r\nfunction(self,aNode)\r\n(self):visitNode_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"visitVariableNode:",0
,(
function(self,aNode)
(self):ignore_(aNode)
;
return (self)
;
end
)
,"visiting","visitVariableNode: aNode\r\n\tself ignore: aNode","(\r\nfunction(self,aNode)\r\n(self):ignore_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"loadRequirementsOf:",0
,(
function(self,aClass)
return ((self):implementedBySubclass_("loadRequirementsOf:"))
;
end
)
,"export","loadRequirementsOf: aClass\r\n\t\" Private - Return the requirements to load aClass. \"\r\n\r\n\t^self implementedBySubclass: #loadRequirementsOf: ","(\r\nfunction(self,aClass)\r\nreturn ((self):implementedBySubclass_(\"loadRequirementsOf:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"exportDefinitionOf:on:",0
,(
function(self,aClass,aStream)
return ((self):implementedBySubclass_("exportDefinitionOf:on:"))
;
end
)
,"export","exportDefinitionOf: aClass on: aStream\r\n\t\" Export definition of aClass onto aStream. \"\r\n\r\n\t^self implementedBySubclass: #exportDefinitionOf:on: ","(\r\nfunction(self,aClass,aStream)\r\nreturn ((self):implementedBySubclass_(\"exportDefinitionOf:on:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"exportMetaDefinitionOf:on:",0
,(
function(self,aClass,aStream)
return ((self):implementedBySubclass_("exportMetaDefinitionOf:on:"))
;
end
)
,"export","exportMetaDefinitionOf: aClass on: aStream\r\n\t\" Export class side definition of aClass onto aStream. \"\r\n\r\n\t^self implementedBySubclass: #exportMetaDefinitionOf:on: ","(\r\nfunction(self,aClass,aStream)\r\nreturn ((self):implementedBySubclass_(\"exportMetaDefinitionOf:on:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"export:method:on:",0
,(
function(self,aClass,aMethod,aStream)
return ((self):implementedBySubclass_("export:method:on:"))
;
end
)
,"export","export: aClass method: aMethod on: aStream\r\n\t\" Export a method definition of aClass onto aStream. \"\r\n\r\n\t^self implementedBySubclass: #export:method:on: ","(\r\nfunction(self,aClass,aMethod,aStream)\r\nreturn ((self):implementedBySubclass_(\"export:method:on:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"arrayLiteralStart",0
,(
function(self)
return ((self):implementedBySubclass_("arrayLiteralStart"))
;
end
)
,"literals","arrayLiteralStart\r\n\t\" Private - Returns the code string to start exporting an array literal. \"\r\n\r\n\t^self implementedBySubclass: #arrayLiteralStart","(\r\nfunction(self)\r\nreturn ((self):implementedBySubclass_(\"arrayLiteralStart\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"arrayLiteralEnd",0
,(
function(self)
return ((self):implementedBySubclass_("arrayLiteralEnd"))
;
end
)
,"literals","arrayLiteralEnd\r\n\t\" Private - Returns the code string to end exporting an array literal. \"\r\n\r\n\t^self implementedBySubclass: #arrayLiteralEnd","(\r\nfunction(self)\r\nreturn ((self):implementedBySubclass_(\"arrayLiteralEnd\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"escaped:",0
,(
function(self,string)
return (smalltalk:object_comma_((smalltalk:object_comma_(("\""),string)),"\""))
;
end
)
,"literals","escaped: string\r\n\t\" Private - Returns the code string to export string literal. \"\r\n\r\n\t^$\" ,string ,$\"","(\r\nfunction(self,string)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"\\\"\"),string)),\"\\\"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.S8Encoder,"exportArrayLiteral:on:newLine:",0
,(
function(self,anArray,aStream,newLine)
(aStream):nextPutAll_((self):arrayLiteralStart())
;
(anArray):do_separatedBy_((function(each)
return ((aStream):nextPutAll_((function() if ((((each):isString()))==true) then return ((function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("\""),each)),"\""))
end))() else return ((function()
return (smalltalk:printString_((each)))
end))() end end)()))
end), (function()
return ((aStream):nextPutAll_(","))
end))
;
(aStream):nextPutAll_((self):arrayLiteralEnd())
;
(function() if (((newLine))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_(";");
return ((_s8_rec):cr())
end
)(aStream))
end))() else return nil end end)()
;
return (self)
;
end
)
,"literals","exportArrayLiteral: anArray on: aStream newLine: newLine\r\n\t\" Private - Export the native code for elements in anArray. \"\r\n\r\n\taStream nextPutAll: self arrayLiteralStart.\r\n\tanArray\r\n\t    do: [:each | aStream nextPutAll: (\r\n\t\teach isString ifTrue: [ $\" ,each ,$\" ] ifFalse: [ each printString ]\r\n\t    ) ] separatedBy: [ aStream nextPutAll: ',' ].\r\n\taStream nextPutAll: self arrayLiteralEnd.\r\n\tnewLine ifTrue: [ aStream nextPutAll: $;; cr ]","(\r\nfunction(self,anArray,aStream,newLine)\r\n(aStream):nextPutAll_((self):arrayLiteralStart())\r\n;\r\n(anArray):do_separatedBy_((function(each)\r\nreturn ((aStream):nextPutAll_((function() if ((((each):isString()))==true) then return ((function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"\\\"\"),each)),\"\\\"\"))\r\nend))() else return ((function()\r\nreturn (smalltalk:printString_((each)))\r\nend))() end end)()))\r\nend), (function()\r\nreturn ((aStream):nextPutAll_(\",\"))\r\nend))\r\n;\r\n(aStream):nextPutAll_((self):arrayLiteralEnd())\r\n;\r\n(function() if (((newLine))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\";\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(aStream))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"addOptimizationLiteral:",0
,(
function(self,aLiteral)
((self):optimizationMap()):add_(aLiteral)
;
return (self)
;
end
)
,"optimization","addOptimizationLiteral: aLiteral\r\n\t\" Private - Add the optimization from the receiver. \"\r\n\r\n\tself optimizationMap add: aLiteral ","(\r\nfunction(self,aLiteral)\r\n((self):optimizationMap()):add_(aLiteral)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"addOptimizations:",0
,(
function(self,array)
(array):do_((function(opt)
return (((self):optimizationLiteralsFor_(opt)):do_((function(each)
return ((self):addOptimizationLiteral_(each))
end)))
end))
;
return (self)
;
end
)
,"optimization","addOptimizations: array\r\n\t\" Private - Add an array of optimizations to the receiver. \"\r\n\r\n\tarray do: [:opt|\r\n\t\t(self optimizationLiteralsFor: opt) do: [:each|\r\n\t\t\tself addOptimizationLiteral: each\r\n\t\t]\r\n\t]","(\r\nfunction(self,array)\r\n(array):do_((function(opt)\r\nreturn (((self):optimizationLiteralsFor_(opt)):do_((function(each)\r\nreturn ((self):addOptimizationLiteral_(each))\r\nend)))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"allBlockNoArguments:",0
,(
function(self,aNode)
return ((smalltalk:boolean_and_(((self):receiverIsBlockNoArguments_(aNode)),(function()
return ((self):hasBlockNoArguments_(aNode))
end))))
;
end
)
,"optimization","allBlockNoArguments: aNode\r\n\t\" Private - Return true if receiver and arguments of aNode are blocks with no arguments. \"\r\n\r\n\t^(self receiverIsBlockNoArguments: aNode) and: [\r\n\t\tself hasBlockNoArguments: aNode\r\n\t]","(\r\nfunction(self,aNode)\r\nreturn ((smalltalk:boolean_and_(((self):receiverIsBlockNoArguments_(aNode)),(function()\r\nreturn ((self):hasBlockNoArguments_(aNode))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"arg1HasNumber:",0
,(
function(self,aNode)
return ((((aNode):arguments()):at_((1))):hasNumber())
;
end
)
,"optimization","arg1HasNumber: aNode\r\n\t\" Private - Return true if argument of aNode is a number literal. \"\r\n\r\n\t^(aNode arguments at: 1) hasNumber","(\r\nfunction(self,aNode)\r\nreturn ((((aNode):arguments()):at_((1))):hasNumber())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"arg2HasNumber:",0
,(
function(self,aNode)
return ((((aNode):arguments()):at_((2))):hasNumber())
;
end
)
,"optimization","arg2HasNumber: aNode\r\n\t\" Private - Return true if argument of aNode is a number literal. \"\r\n\r\n\t^(aNode arguments at: 2) hasNumber","(\r\nfunction(self,aNode)\r\nreturn ((((aNode):arguments()):at_((2))):hasNumber())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"arg3HasNumber:",0
,(
function(self,aNode)
return ((((aNode):arguments()):at_((3))):hasNumber())
;
end
)
,"optimization","arg3HasNumber: aNode\r\n\t\" Private - Return true if argument of aNode is a number literal. \"\r\n\r\n\t^(aNode arguments at: 3) hasNumber","(\r\nfunction(self,aNode)\r\nreturn ((((aNode):arguments()):at_((3))):hasNumber())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"argumentsHasNumber:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
((aNode):arguments()):do_((function(each)
return ((function() if ((((each):hasNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"optimization","argumentsHasNumber: aNode\r\n\t\" Private - Return true if any argument of aNode is a number literal. \"\r\n\r\n\taNode arguments do: [:each|\r\n\t\teach hasNumber ifTrue: [ ^true ]\r\n\t].\r\n\t^false","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n((aNode):arguments()):do_((function(each)\r\nreturn ((function() if ((((each):hasNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"argumentsHasString:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
((aNode):arguments()):do_((function(each)
return ((function() if ((((each):hasString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"optimization","argumentsHasString: aNode\r\n\t\" Private - Return true if any argument of aNode is a string literal. \"\r\n\r\n\taNode arguments do: [:each|\r\n\t\teach hasString ifTrue: [ ^true ]\r\n\t].\r\n\t^false","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n((aNode):arguments()):do_((function(each)\r\nreturn ((function() if ((((each):hasString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"defaultKnownOptimizations",0
,(
function(self)
return (smalltalk:newArray({}))
;
end
)
,"defaults","defaultKnownOptimizations\r\n\t\" Private - Returns the default knownOptimizations of the receiver. \"\r\n\r\n\t^#()","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"defaultOptimizations",0
,(
function(self)
return (smalltalk:newArray({}))
;
end
)
,"defaults","defaultOptimizations\r\n\t\" Private - Returns the default optimizations of the receiver. \"\r\n\r\n\t^#()","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"hasBlockArguments:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
((aNode):arguments()):do_((function(each)
return ((function() if (((smalltalk:is_kindOf_((each),(smalltalk.BlockNode))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"optimization","hasBlockArguments: aNode\r\n\t\" Private - Return true if arguments of aNode are blocks. \"\r\n\r\n\taNode arguments do: [:each|\r\n\t\t(each isKindOf: BlockNode) ifFalse: [ ^false ].\r\n\t].\r\n\t^true","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n((aNode):arguments()):do_((function(each)\r\nreturn ((function() if (((smalltalk:is_kindOf_((each),(smalltalk.BlockNode))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"hasBlockNoArguments:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):hasBlockArguments_(aNode)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
((aNode):arguments()):do_((function(each)
return ((function() if (((((each):parameters()):isEmpty()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"optimization","hasBlockNoArguments: aNode\r\n\t\" Private - Return true if arguments of aNode are blocks with no parameters. \"\r\n\r\n\t(self hasBlockArguments: aNode) ifFalse: [ ^false ].\r\n\taNode arguments do: [:each|\r\n\t\teach parameters isEmpty ifFalse: [ ^false ].\r\n\t].\r\n\t^true","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):hasBlockArguments_(aNode)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n((aNode):arguments()):do_((function(each)\r\nreturn ((function() if (((((each):parameters()):isEmpty()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"hasBlockOneArguments:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):hasBlockArguments_(aNode)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
((aNode):arguments()):do_((function(each)
return ((function() if ((((smalltalk:isObject_equalTo_((((each):parameters()):size()),(1)))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"optimization","hasBlockOneArguments: aNode\r\n\t\" Private - Return true if arguments of aNode are blocks with ONE parameter. \"\r\n\r\n\t(self hasBlockArguments: aNode) ifFalse: [ ^false ].\r\n\taNode arguments do: [:each|\r\n\t\teach parameters size = 1 ifFalse: [ ^false ].\r\n\t].\r\n\t^true","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):hasBlockArguments_(aNode)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n((aNode):arguments()):do_((function(each)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((((each):parameters()):size()),(1)))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"hasNumber:",0
,(
function(self,aNode)
return ((smalltalk:boolean_or_(((self):receiverHasNumber_(aNode)),(function()
return ((self):argumentsHasNumber_(aNode))
end))))
;
end
)
,"optimization","hasNumber: aNode\r\n\t\" Private - Return true if the receiver or arguments in aNode has numeric value. \"\r\n\r\n\t^(self receiverHasNumber: aNode) or: [\r\n\t\tself argumentsHasNumber: aNode\r\n\t]","(\r\nfunction(self,aNode)\r\nreturn ((smalltalk:boolean_or_(((self):receiverHasNumber_(aNode)),(function()\r\nreturn ((self):argumentsHasNumber_(aNode))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"hasString:",0
,(
function(self,aNode)
return ((smalltalk:boolean_or_(((self):receiverHasString_(aNode)),(function()
return ((self):argumentsHasString_(aNode))
end))))
;
end
)
,"optimization","hasString: aNode\r\n\t\" Private - Return true if the receiver or arguments in aNode has string value. \"\r\n\r\n\t^(self receiverHasString: aNode) or: [\r\n\t\tself argumentsHasString: aNode\r\n\t]","(\r\nfunction(self,aNode)\r\nreturn ((smalltalk:boolean_or_(((self):receiverHasString_(aNode)),(function()\r\nreturn ((self):argumentsHasString_(aNode))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"initializeKnownOptimizations",0
,(
function(self)
(function()
	local _s8_value=((self):defaultKnownOptimizations())
	self['@knownOptimizations']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"initialize","initializeKnownOptimizations\r\n\t\" Private - Initialize the knownOptimizations of the receiver. \"\r\n\r\n\tknownOptimizations := self defaultKnownOptimizations","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=((self):defaultKnownOptimizations())\r\n\tself['@knownOptimizations']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"initializeOptimizationMap",0
,(
function(self)
(function()
	local _s8_value=((self):compilerEvent_ifNotHandled_("needsOptimizationMap", (function()
return (smalltalk:newArray({}))
end)))
	self['@optimizationMap']=_s8_value; return _s8_value
end)()
;
(self):compilerEvent_ifNotHandled_("initializeOptimizations", (function()
return ((self):addOptimizations_((self):defaultOptimizations()))
end))
;
(self):compilerEvent_("customizeOptimizations")
;
return (self)
;
end
)
,"initialize","initializeOptimizationMap\r\n\t\" Private - Initialize the optimization map of the receiver. \"\r\n\r\n\toptimizationMap := self\r\n\t\tcompilerEvent: #needsOptimizationMap\r\n\t\tifNotHandled: [ #() ].\r\n\tself compilerEvent: #initializeOptimizations ifNotHandled: [\r\n\t\tself addOptimizations: self defaultOptimizations\r\n\t].\r\n\tself compilerEvent: #customizeOptimizations","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=((self):compilerEvent_ifNotHandled_(\"needsOptimizationMap\", (function()\r\nreturn (smalltalk:newArray({}))\r\nend)))\r\n\tself['@optimizationMap']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):compilerEvent_ifNotHandled_(\"initializeOptimizations\", (function()\r\nreturn ((self):addOptimizations_((self):defaultOptimizations()))\r\nend))\r\n;\r\n(self):compilerEvent_(\"customizeOptimizations\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"knownOptimizations",0
,(
function(self)
(function() if ((((((self['@knownOptimizations']))==nil)))==true) then return ((function()
return ((self):initializeKnownOptimizations())
end))() else return nil end end)()
;
return (self['@knownOptimizations'])
;
end
)
,"accessing","knownOptimizations\r\n\t\" Return the knownOptimizations of the receiver. \"\r\n\r\n\tknownOptimizations isNil ifTrue: [ self initializeKnownOptimizations ].\r\n\t^knownOptimizations","(\r\nfunction(self)\r\n(function() if ((((((self['@knownOptimizations']))==nil)))==true) then return ((function()\r\nreturn ((self):initializeKnownOptimizations())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@knownOptimizations'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"knownOptimizations:",0
,(
function(self,anObject)
(function()
	local _s8_value=(anObject)
	self['@knownOptimizations']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","knownOptimizations: anObject\r\n\t\" Set the knownOptimizations of the receiver. \"\r\n\r\n\tknownOptimizations := anObject","(\r\nfunction(self,anObject)\r\n(function()\r\n\tlocal _s8_value=(anObject)\r\n\tself['@knownOptimizations']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"matchOptimizedCondition:with:",0
,(
function(self,condition,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:newArray({"hasNumber:", "receiverHasNumber:", "argumentsHasNumber:", "hasString:", "receiverHasString:", "argumentsHasString:", "arg1HasNumber:", "arg2HasNumber:", "arg3HasNumber:", "hasBlockArguments:", "hasBlockNoArguments:", "hasBlockOneArguments:", "allBlockNoArguments:", "receiverIsBlockNoArguments:"})):includes_(condition)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):perform_withArguments_(condition, (smalltalk.Array):with_(aNode)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((condition),"true"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((condition),"false"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((condition):isMemberOf_(smalltalk.Array)))==true) then return ((function()
(condition):detect_ifNone_((function(one)
return ((smalltalk:isObject_equalTo_((false),(self):matchOptimizedCondition_with_(one, aNode))))
end), (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end));return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(self):print_(smalltalk:object_comma_((">>> unknown optimize-condition: "),smalltalk:toString_((condition))))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"optimization","matchOptimizedCondition: condition with: aNode\r\n\t\" Private - Return true if aNode matches the condition. \"\r\n\r\n\t(#(\r\n\t#hasNumber: #receiverHasNumber: #argumentsHasNumber:\r\n\t#hasString:  #receiverHasString: #argumentsHasString:\r\n\t#arg1HasNumber: #arg2HasNumber: #arg3HasNumber:\r\n\t#hasBlockArguments: #hasBlockNoArguments: #hasBlockOneArguments:\r\n\t#allBlockNoArguments: #receiverIsBlockNoArguments:\r\n\t) includes: condition) ifTrue: [\r\n\t\t^self perform: condition withArguments: (Array with: aNode)\r\n\t].\r\n\tcondition = #true ifTrue: [ ^true ].\r\n\tcondition = #false ifTrue: [ ^false ].\r\n\t(condition isMemberOf: Array) ifTrue: [\r\n\t\tcondition detect: [:one|\r\n\t\t\tfalse = (self matchOptimizedCondition: one with: aNode)\r\n\t\t] ifNone: [ ^true ].\r\n\t\t^false\r\n\t].\r\n\tself print: '>>> unknown optimize-condition: ',condition toString.\r\n\t^false","(\r\nfunction(self,condition,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:newArray({\"hasNumber:\", \"receiverHasNumber:\", \"argumentsHasNumber:\", \"hasString:\", \"receiverHasString:\", \"argumentsHasString:\", \"arg1HasNumber:\", \"arg2HasNumber:\", \"arg3HasNumber:\", \"hasBlockArguments:\", \"hasBlockNoArguments:\", \"hasBlockOneArguments:\", \"allBlockNoArguments:\", \"receiverIsBlockNoArguments:\"})):includes_(condition)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):perform_withArguments_(condition, (smalltalk.Array):with_(aNode)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((condition),\"true\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((condition),\"false\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((condition):isMemberOf_(smalltalk.Array)))==true) then return ((function()\r\n(condition):detect_ifNone_((function(one)\r\nreturn ((smalltalk:isObject_equalTo_((false),(self):matchOptimizedCondition_with_(one, aNode))))\r\nend), (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend));return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self):print_(smalltalk:object_comma_((\">>> unknown optimize-condition: \"),smalltalk:toString_((condition))))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"optimizationLiteralsFor:",0
,(
function(self,opt)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local array=nil
(function() if ((((opt):isString()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Array):with_(opt))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
	local _s8_value=((self):knownOptimizations())
	array=_s8_value; return _s8_value
end)()
;
(array):do_((function(pair)
return ((function() if ((((smalltalk:isObject_equalTo_(((pair):first()),opt))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((pair):last()):inject_into_(smalltalk:newArray({}), (function(total,tuple)
return ((
function(_s8_rec)
(_s8_rec):addAll_((self):optimizationLiteralsFor_(tuple));
return ((_s8_rec):yourself())
end
)(total))
end)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(array):do_((function(pair)
return (((pair):last()):do_((function(tuple)
return ((function() if ((((smalltalk:boolean_and_(((tuple):isArray()),(function()
return ((smalltalk:isObject_equalTo_(((tuple):first()),opt)))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Array):with_(tuple))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end)))
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:newArray({}))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"optimization","optimizationLiteralsFor: opt\r\n\t\" Private - Return the optimization literals array defined for opt.\r\n\topt can be an optimization family or message selector\r\n\t\"\r\n\r\n\t| array |\r\n\topt isString ifFalse: [ ^Array with: opt ].\r\n\tarray := self knownOptimizations.\r\n\tarray do: [:pair|\r\n\t\tpair first = opt ifTrue: [\r\n\t\t\t^pair last inject: #() into: [:total :tuple|\r\n\t\t\t\ttotal addAll: (self optimizationLiteralsFor: tuple); yourself\r\n\t\t\t]\r\n\t\t].\r\n\t].\r\n\tarray do: [:pair|\r\n\t\tpair last do: [:tuple|\r\n\t\t\t(tuple isArray and: [tuple first = opt]) ifTrue: [ ^Array with: tuple ]\r\n\t\t].\r\n\t].\r\n\t^#()","(\r\nfunction(self,opt)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal array=nil\r\n(function() if ((((opt):isString()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Array):with_(opt))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):knownOptimizations())\r\n\tarray=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(array):do_((function(pair)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_(((pair):first()),opt))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((pair):last()):inject_into_(smalltalk:newArray({}), (function(total,tuple)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):addAll_((self):optimizationLiteralsFor_(tuple));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(total))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(array):do_((function(pair)\r\nreturn (((pair):last()):do_((function(tuple)\r\nreturn ((function() if ((((smalltalk:boolean_and_(((tuple):isArray()),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((tuple):first()),opt)))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Array):with_(tuple))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend)))\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:newArray({}))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"optimizationMap",0
,(
function(self)
(function() if ((((((self['@optimizationMap']))==nil)))==true) then return ((function()
return ((self):initializeOptimizationMap())
end))() else return nil end end)()
;
return (self['@optimizationMap'])
;
end
)
,"accessing","optimizationMap\r\n\t\" Return the message send optimization map of the receiver. \"\r\n\r\n\toptimizationMap isNil ifTrue: [ self initializeOptimizationMap ].\r\n\t^optimizationMap","(\r\nfunction(self)\r\n(function() if ((((((self['@optimizationMap']))==nil)))==true) then return ((function()\r\nreturn ((self):initializeOptimizationMap())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@optimizationMap'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"optimizationMap:",0
,(
function(self,aSpec)
(function()
	local _s8_value=(aSpec)
	self['@optimizationMap']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","optimizationMap: aSpec\r\n\t\" Set the current optimizationMap of the receiver. \"\r\n\r\n\toptimizationMap := aSpec","(\r\nfunction(self,aSpec)\r\n(function()\r\n\tlocal _s8_value=(aSpec)\r\n\tself['@optimizationMap']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"optimize:macro:",0
,(
function(self,aNode,template)
local input=nil
(function()
	local _s8_value=((template):readStream())
	input=_s8_value; return _s8_value
end)()
;
(input):do_((function(char)
return ((function() if ((((smalltalk:isObject_equalTo_((char),"%"))))==true) then return ((function()
return ((self):visitMacroIn_with_(input, aNode))
end))() else return ((function()
return ((self):emmitCode_(smalltalk:asString_((char))))
end))() end end)())
end))
;
return (self)
;
end
)
,"optimizing","optimize: aNode macro: template\r\n\t\" Private - Generate the code for aNode using a macro template.\r\n\tDump code expanding variables in macro template.\r\n\tVariables are defined between %%\r\n\t\"\r\n\r\n\t| input |\r\n\tinput := template readStream.\r\n\tinput do: [:char|\r\n\t\tchar = $%\r\n\t\tifTrue: [ self visitMacroIn: input with: aNode ]\r\n\t\tifFalse: [ self emmitCode: char asString ]\r\n\t]","(\r\nfunction(self,aNode,template)\r\nlocal input=nil\r\n(function()\r\n\tlocal _s8_value=((template):readStream())\r\n\tinput=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(input):do_((function(char)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((char),\"%\"))))==true) then return ((function()\r\nreturn ((self):visitMacroIn_with_(input, aNode))\r\nend))() else return ((function()\r\nreturn ((self):emmitCode_(smalltalk:asString_((char))))\r\nend))() end end)())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"receiverHasNumber:",0
,(
function(self,aNode)
return (((aNode):receiver()):hasNumber())
;
end
)
,"optimization","receiverHasNumber: aNode\r\n\t\" Private - Return true if the receiver of aNode is a number literal. \"\r\n\r\n\t^aNode receiver hasNumber","(\r\nfunction(self,aNode)\r\nreturn (((aNode):receiver()):hasNumber())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"receiverHasString:",0
,(
function(self,aNode)
return (((aNode):receiver()):hasString())
;
end
)
,"optimization","receiverHasString: aNode\r\n\t\" Private - Return true if the receiver of aNode is a string literal. \"\r\n\r\n\t^aNode receiver hasString","(\r\nfunction(self,aNode)\r\nreturn (((aNode):receiver()):hasString())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"receiverIsBlockNoArguments:",0
,(
function(self,aNode)
return ((smalltalk:boolean_and_((smalltalk:is_kindOf_(((aNode):receiver()),(smalltalk.BlockNode))),(function()
return ((((aNode):receiver()):parameters()):isEmpty())
end))))
;
end
)
,"optimization","receiverIsBlockNoArguments: aNode\r\n\t\" Private - Return true if receiver of aNode is block with no arguments. \"\r\n\r\n\t^(aNode receiver isKindOf: BlockNode) and: [\r\n\t\taNode receiver parameters isEmpty\r\n\t]","(\r\nfunction(self,aNode)\r\nreturn ((smalltalk:boolean_and_((smalltalk:is_kindOf_(((aNode):receiver()),(smalltalk.BlockNode))),(function()\r\nreturn ((((aNode):receiver()):parameters()):isEmpty())\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"removeOptimizationLiteral:",0
,(
function(self,aLiteral)
((self):optimizationMap()):remove_(aLiteral)
;
return (self)
;
end
)
,"optimization","removeOptimizationLiteral: aLiteral\r\n\t\" Private - Remove the optimization from the receiver. \"\r\n\r\n\tself optimizationMap remove: aLiteral ","(\r\nfunction(self,aLiteral)\r\n((self):optimizationMap()):remove_(aLiteral)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"removeOptimizations:",0
,(
function(self,array)
(array):do_((function(opt)
return (((self):optimizationLiteralsFor_(opt)):do_((function(each)
return ((self):removeOptimizationLiteral_(each))
end)))
end))
;
return (self)
;
end
)
,"optimization","removeOptimizations: array\r\n\t\" Private - Remove an array of optimizations to the receiver. \"\r\n\r\n\tarray do: [:opt|\r\n\t\t(self optimizationLiteralsFor: opt) do: [:each|\r\n\t\t\tself removeOptimizationLiteral: each\r\n\t\t]\r\n\t]","(\r\nfunction(self,array)\r\n(array):do_((function(opt)\r\nreturn (((self):optimizationLiteralsFor_(opt)):do_((function(each)\r\nreturn ((self):removeOptimizationLiteral_(each))\r\nend)))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"missingNativeCodeFor:as:",0
,(
function(self,aNode,lang)
return ((self):error_(smalltalk:object_comma_(("Missing native code for language "),lang)))
;
end
)
,"inline","missingNativeCodeFor: aNode as: lang\r\n\t\" Private - Returns the native language code for missing expression (or nil). \"\r\n\r\n\t^self error: 'Missing native code for language ' ,lang","(\r\nfunction(self,aNode,lang)\r\nreturn ((self):error_(smalltalk:object_comma_((\"Missing native code for language \"),lang)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"visitInlineNode:",0
,(
function(self,aNode)
local string=nil
local lang=nil
(function()
	local _s8_value=((self):nativeLanguage())
	lang=_s8_value; return _s8_value
end)()
;
(function() if ((((((lang))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.Smalltalk):nativeLanguage())
	lang=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((aNode):sourceFor_(lang))
	string=_s8_value; return _s8_value
end)()
;
(function() if ((((((string))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((self):missingNativeCodeFor_as_(aNode, lang))
	string=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function() if ((((((string))~=nil)))==true) then return ((function()
return ((self):emmitCode_(string))
end))() else return nil end end)()
;
return (self)
;
end
)
,"inline","visitInlineNode: aNode\r\n\t\" Private - Encode an inline node containing native language code. \"\r\n\r\n\t| string lang |\r\n\tlang := self nativeLanguage.\r\n\tlang isNil ifTrue:  [ lang := Smalltalk nativeLanguage ].\r\n\tstring := aNode sourceFor: lang.\r\n\tstring isNil ifTrue:  [ string := self missingNativeCodeFor: aNode as: lang ].\r\n\tstring notNil ifTrue: [ self emmitCode: string ].","(\r\nfunction(self,aNode)\r\nlocal string=nil\r\nlocal lang=nil\r\n(function()\r\n\tlocal _s8_value=((self):nativeLanguage())\r\n\tlang=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((lang))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):nativeLanguage())\r\n\tlang=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aNode):sourceFor_(lang))\r\n\tstring=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((string))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((self):missingNativeCodeFor_as_(aNode, lang))\r\n\tstring=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((string))~=nil)))==true) then return ((function()\r\nreturn ((self):emmitCode_(string))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"visitInlined:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((smalltalk:is_kindOf_((aNode),(smalltalk.BlockNode))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_(("Invalid block node ("),smalltalk:toString_((aNode)))),").")))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
((aNode):nodes()):do_((function(each)
return ((self):visit_(each))
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"optimization","visitInlined: aNode\r\n\t\" Private - Encode aNode as inlined sequence of statements. \"\r\n\r\n\t(aNode isKindOf: BlockNode) ifFalse: [\r\n\t\t^self error: 'Invalid block node (',aNode toString,').'\r\n\t].\r\n\taNode nodes do: [:each | self visit: each ].","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((smalltalk:is_kindOf_((aNode),(smalltalk.BlockNode))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((\"Invalid block node (\"),smalltalk:toString_((aNode)))),\").\")))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n((aNode):nodes()):do_((function(each)\r\nreturn ((self):visit_(each))\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"visitMacroIn:with:",0
,(
function(self,input,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local varName=nil
local char=nil
local args=nil
(function()
	local _s8_value=((smalltalk.String):new())
	varName=_s8_value; return _s8_value
end)()
;
(function() while(not ((function()
(function()
	local _s8_value=((input):next())
	char=_s8_value; return _s8_value
end)();return ((smalltalk:isObject_equalTo_((char),"%")))
end))()) do (function()
return ((function()
	local _s8_value=(smalltalk:object_comma_((varName),smalltalk:asString_((char))))
	varName=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
(function() if ((((smalltalk:isObject_equalTo_((varName),"receiver"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitReceiver_((aNode):receiver()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((varName):startsWith_("arg")))==true) then return ((function()
(function()
	local _s8_value=((aNode):arguments())
	args=_s8_value; return _s8_value
end)();return (((1)):to_do_((args):size(), (function(i)
return ((function() if ((((smalltalk:isObject_equalTo_((smalltalk:object_comma_(("arg"),smalltalk:asString_((i)))),varName))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visit_((args):at_(i)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end)))
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((varName),"inlinedReceiver"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitInlined_((aNode):receiver()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((varName):startsWith_("inlined")))==true) then return ((function()
(function()
	local _s8_value=((aNode):arguments())
	args=_s8_value; return _s8_value
end)();return (((1)):to_do_((args):size(), (function(i)
return ((function() if ((((smalltalk:isObject_equalTo_((smalltalk:object_comma_(("inlined"),smalltalk:asString_((i)))),varName))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitInlined_((args):at_(i)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end)))
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_(("Invalid macro variable["),varName)),"].")))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"optimizing","visitMacroIn: input with: aNode\r\n\t\" Private - Dump the value of next variable defined in input. \"\r\n\r\n\t| varName char args |\r\n\tvarName := String new.\r\n\t[ char := input next. char = $% ] whileFalse: [\r\n\t\tvarName := varName ,char asString\r\n\t].\r\n\r\n\tvarName = #receiver ifTrue: [ ^self visitReceiver: aNode receiver ].\r\n\t(varName startsWith: #arg) ifTrue: [\r\n\t\targs := aNode arguments.\r\n\t\t1 to: args size do: [:i|\r\n\t\t\t#arg ,i asString = varName ifTrue: [\r\n\t\t\t\t^self visit: (args at: i)\r\n\t\t\t].\r\n\t\t].\r\n\t].\r\n\tvarName = #inlinedReceiver ifTrue: [ ^self visitInlined: aNode receiver ].\r\n\t(varName startsWith: #inlined) ifTrue: [\r\n\t\targs := aNode arguments.\r\n\t\t1 to: args size do: [:i|\r\n\t\t\t#inlined ,i asString = varName ifTrue: [\r\n\t\t\t\t^self visitInlined: (args at: i)\r\n\t\t\t].\r\n\t\t].\r\n\t].\r\n\t^self error: 'Invalid macro variable[',varName,'].'","(\r\nfunction(self,input,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal varName=nil\r\nlocal char=nil\r\nlocal args=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.String):new())\r\n\tvarName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(not ((function()\r\n(function()\r\n\tlocal _s8_value=((input):next())\r\n\tchar=_s8_value; return _s8_value\r\nend)();return ((smalltalk:isObject_equalTo_((char),\"%\")))\r\nend))()) do (function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((varName),smalltalk:asString_((char))))\r\n\tvarName=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((varName),\"receiver\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitReceiver_((aNode):receiver()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((varName):startsWith_(\"arg\")))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((aNode):arguments())\r\n\targs=_s8_value; return _s8_value\r\nend)();return (((1)):to_do_((args):size(), (function(i)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((smalltalk:object_comma_((\"arg\"),smalltalk:asString_((i)))),varName))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visit_((args):at_(i)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend)))\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((varName),\"inlinedReceiver\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitInlined_((aNode):receiver()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((varName):startsWith_(\"inlined\")))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((aNode):arguments())\r\n\targs=_s8_value; return _s8_value\r\nend)();return (((1)):to_do_((args):size(), (function(i)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((smalltalk:object_comma_((\"inlined\"),smalltalk:asString_((i)))),varName))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitInlined_((args):at_(i)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend)))\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((\"Invalid macro variable[\"),varName)),\"].\")))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"visitOptimizedNode:as:",0
,(
function(self,aNode,mapping)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((mapping):size()),(2)))),(function()
return (((mapping):last()):isString())
end)))))==true) then return ((function()
(self):optimize_macro_(aNode, (mapping):last());return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((mapping):size()),(3)))),(function()
return (((mapping):last()):isString())
end)))))==true) then return ((function()
(function() if ((((self):matchOptimizedCondition_with_((mapping):at_((2)), aNode)))==true) then return ((function()
return ((self):optimize_macro_(aNode, (mapping):last()))
end))() else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(self):print_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((">>> compiler optimization")," for #")),(aNode):selector()))," - not implemented yet. Mapping=")),smalltalk:asString_((mapping))))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitOptimizedNode: aNode as: mapping\r\n\t\" Private - Generate the code for aNode using custom mapping.\r\n\tReturn true if the code can be generated.\r\n\tIf the optimization fails, the receiver remains intact.\r\n\t\"\r\n\r\n\t(mapping size = 2 and: [mapping last isString]) ifTrue: [ \"(selector macro)\"\r\n\t\tself optimize: aNode macro: mapping last.\r\n\t\t^true\r\n\t].\r\n\t(mapping size = 3 and: [mapping last isString]) ifTrue: [\"(selector condition macro)\"\r\n\t\t(self matchOptimizedCondition: (mapping at:2) with: aNode)\r\n\t\t\tifTrue: [ self optimize: aNode macro: mapping last ]\r\n\t\t\tifFalse: [ ^false ].\r\n\t\t^true\r\n\t].\r\n\r\n\tself print: '>>> compiler optimization'\r\n\t\t,' for #',aNode selector\r\n\t\t,' - not implemented yet. Mapping='\r\n\t\t,mapping asString.\r\n\t^false","(\r\nfunction(self,aNode,mapping)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((mapping):size()),(2)))),(function()\r\nreturn (((mapping):last()):isString())\r\nend)))))==true) then return ((function()\r\n(self):optimize_macro_(aNode, (mapping):last());return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((mapping):size()),(3)))),(function()\r\nreturn (((mapping):last()):isString())\r\nend)))))==true) then return ((function()\r\n(function() if ((((self):matchOptimizedCondition_with_((mapping):at_((2)), aNode)))==true) then return ((function()\r\nreturn ((self):optimize_macro_(aNode, (mapping):last()))\r\nend))() else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self):print_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\">>> compiler optimization\"),\" for #\")),(aNode):selector())),\" - not implemented yet. Mapping=\")),smalltalk:asString_((mapping))))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"visitOptimizedSend:",0
,(
function(self,aNode)
return ((self):visitOptimizedSend_with_(aNode, (self):optimizationMap()))
;
end
)
,"visiting","visitOptimizedSend: aNode\r\n\t\" Private - Generate optimal code for aNode using optimization map.\r\n\tReturn true if optimization was successfull.\r\n\t\"\r\n\r\n\t^self visitOptimizedSend: aNode with: self optimizationMap","(\r\nfunction(self,aNode)\r\nreturn ((self):visitOptimizedSend_with_(aNode, (self):optimizationMap()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"visitOptimizedSend:with:",0
,(
function(self,aNode,mapping)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local argCount=nil
(function()
	local _s8_value=(((aNode):arguments()):size())
	argCount=_s8_value; return _s8_value
end)()
;
(mapping):do_((function(tuple)
return ((function() if ((((smalltalk:boolean_and_((((tuple):first()):isNumber()),(function()
return ((smalltalk:isObject_equalTo_(((tuple):first()),argCount)))
end)))))==true) then return ((function()
return ((function() if ((((self):visitOptimizedSend_with_(aNode, (tuple):last())))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() else return ((function()
return ((function() if ((((smalltalk:isObject_equalTo_(((tuple):first()),(aNode):selector()))))==true) then return ((function()
return ((function() if ((((self):visitOptimizedNode_as_(aNode, tuple)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() else return nil end end)())
end))() end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitOptimizedSend: aNode with: mapping\r\n\t\" Private - Generate optimal code for aNode using mapping.\r\n\tReturn true if the code has been generated.\r\n\tLeave the receiver intact if the optimization do not apply.\r\n\t\"\r\n\r\n\t| argCount |\r\n\targCount := aNode arguments size.\r\n\tmapping do: [:tuple|\r\n\t\t(tuple first isNumber and: [ tuple first = argCount ]) ifTrue: [ \"arguments size match\"\r\n\t\t\t(self visitOptimizedSend: aNode with: tuple last) ifTrue: [ ^true ].\r\n\t\t] ifFalse: [\r\n\t\t\ttuple first = aNode selector ifTrue: [ \"selector match\"\r\n\t\t\t\t(self visitOptimizedNode: aNode as: tuple) ifTrue: [ ^true ].\r\n\t\t\t].\r\n\t\t].\r\n\t].\r\n\t^false","(\r\nfunction(self,aNode,mapping)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal argCount=nil\r\n(function()\r\n\tlocal _s8_value=(((aNode):arguments()):size())\r\n\targCount=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(mapping):do_((function(tuple)\r\nreturn ((function() if ((((smalltalk:boolean_and_((((tuple):first()):isNumber()),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((tuple):first()),argCount)))\r\nend)))))==true) then return ((function()\r\nreturn ((function() if ((((self):visitOptimizedSend_with_(aNode, (tuple):last())))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return ((function()\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_(((tuple):first()),(aNode):selector()))))==true) then return ((function()\r\nreturn ((function() if ((((self):visitOptimizedNode_as_(aNode, tuple)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return nil end end)())\r\nend))() end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"visitReceiver:",0
,(
function(self,aNode)
return ((self):implementedBySubclass_("visitReceiver:"))
;
end
)
,"visiting","visitReceiver: aNode\r\n\t\" Private - Visit the node (as receiver) emmitting output. \"\r\n\r\n\t^self implementedBySubclass: #visitReceiver:","(\r\nfunction(self,aNode)\r\nreturn ((self):implementedBySubclass_(\"visitReceiver:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.OptimizingEncoder,"visitSendNode:",0
,(
function(self,aNode)
return ((self):implementedBySubclass_("visitSendNode:"))
;
end
)
,"visiting","visitSendNode: aNode\r\n\t\" Private - Visit the node (message send node) emmitting output. \"\r\n\r\n\t^self implementedBySubclass: #visitSendNode:","(\r\nfunction(self,aNode)\r\nreturn ((self):implementedBySubclass_(\"visitSendNode:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder["$klass"],"supportsLanguage:",0
,(
function(self,aLanguage)
return ((smalltalk:boolean_or_(((((aLanguage))==nil)),(function()
return ((smalltalk:newArray({"js", "javascript"})):includes_(aLanguage))
end))))
;
end
)
,"instantiation","supportsLanguage: aLanguage\r\n\t\" Private - Return true if instances of the receiver can encode to aLanguage. \"\r\n\r\n\t^aLanguage isNil or: [ #(js javascript) includes: aLanguage ]","(\r\nfunction(self,aLanguage)\r\nreturn ((smalltalk:boolean_or_(((((aLanguage))==nil)),(function()\r\nreturn ((smalltalk:newArray({\"js\", \"javascript\"})):includes_(aLanguage))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder["$klass"],"smalltalkNameFor:",0
,(
function(self,aClass)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((aClass))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("undefined")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_(("smalltalk."),(function() if ((((aClass):isMetaclass()))==true) then return ((function()
return (smalltalk:object_comma_((((aClass):instanceClass()):name()),".$klass"))
end))() else return ((function()
return ((aClass):name())
end))() end end)()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","smalltalkNameFor: aClass\r\n\t\" Private - Return the global name of aClass (can be nil). \"\r\n\r\n\taClass isNil ifTrue: [ ^#undefined ].\r\n\t^'smalltalk.', (aClass isMetaclass\r\n\t    ifTrue: [ aClass instanceClass name, '.$klass' ]\r\n\t    ifFalse: [ aClass name ])","(\r\nfunction(self,aClass)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((aClass))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"undefined\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((\"smalltalk.\"),(function() if ((((aClass):isMetaclass()))==true) then return ((function()\r\nreturn (smalltalk:object_comma_((((aClass):instanceClass()):name()),\".$klass\"))\r\nend))() else return ((function()\r\nreturn ((aClass):name())\r\nend))() end end)()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder["$klass"],"unknownVariable:in:",0
,(
function(self,aNode,anEncoder)
(self):note_(smalltalk:object_comma_((">>> Unknown variable: "),smalltalk:asString_(((aNode):value()))))
;
return ((self):error_(smalltalk:object_comma_(("Missing reference "),(aNode):value())))
;
end
)
,"variables","unknownVariable: aNode in: anEncoder\r\n\t\" Private - anEncoder has found an unknown name reference.\r\n\tThe default action will thrown an error (stop compilation),\r\n\t this method can be refined/changed to silently ignore the\r\n\t unknown to let the compiler produce code to refer code\r\n\t in outer native contexts.\r\n\t\"\r\n\r\nself note: '>>> Unknown variable: ' ,aNode value asString.\r\n\t^self error: 'Missing reference ',aNode value","(\r\nfunction(self,aNode,anEncoder)\r\n(self):note_(smalltalk:object_comma_((\">>> Unknown variable: \"),smalltalk:asString_(((aNode):value()))))\r\n;\r\nreturn ((self):error_(smalltalk:object_comma_((\"Missing reference \"),(aNode):value())))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"nativeLanguage",0
,(
function(self)
return ("js")
;
end
)
,"accessing","nativeLanguage\r\n\t\" Return the nativeLanguage of the receiver. \"\r\n\r\n\t^#js","(\r\nfunction(self)\r\nreturn (\"js\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"missingNativeCodeFor:as:",0
,(
function(self,aNode,lang)
return ("(self.invalidMessage())")
;
end
)
,"inline","missingNativeCodeFor: aNode as: lang\r\n\t\" Private - Returns the native language code for missing expression (or nil). \"\r\n\r\n\"\tself note: '>>> Missing native code for language ' ,lang.\r\n\t^nil\r\n\"\t^'(self.invalidMessage())'","(\r\nfunction(self,aNode,lang)\r\nreturn (\"(self.invalidMessage())\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"acceptAssignment:to:",0
,(
function(self,aNode,assignee)
return ((self):compilerEvent_with_ifNotHandled_("acceptAssignmentTo:", assignee, (function()
return (true)
end)))
;
end
)
,"assignment","acceptAssignment: aNode to: assignee\r\n\t\" Private - Return true if aNode expression can be assigned to assignee (aVariableNode). \"\r\n\r\n\t^self\tcompilerEvent: #acceptAssignmentTo:\r\n\t\twith: assignee\r\n\t\tifNotHandled: [ true ]","(\r\nfunction(self,aNode,assignee)\r\nreturn ((self):compilerEvent_with_ifNotHandled_(\"acceptAssignmentTo:\", assignee, (function()\r\nreturn (true)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"codeToAccessInstanceVariable:",0
,(
function(self,aName)
return ((self):codeToAccessProperty_(smalltalk:object_comma_(("@"),aName)))
;
end
)
,"generation","codeToAccessInstanceVariable: aName\r\n\t\" Private - Return the source code to access instance variable aName. \"\r\n\r\n\t^self codeToAccessProperty: '@', aName","(\r\nfunction(self,aName)\r\nreturn ((self):codeToAccessProperty_(smalltalk:object_comma_((\"@\"),aName)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"codeToAccessProperty:",0
,(
function(self,aName)
return (smalltalk:object_comma_((smalltalk:object_comma_(("self[\""),aName)),"\"]"))
;
end
)
,"generation","codeToAccessProperty: aName\r\n\t\" Private - Return the source code to access property aName. \"\r\n\r\n\t^'self[\"',aName,'\"]'","(\r\nfunction(self,aName)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"self[\\\"\"),aName)),\"\\\"]\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"defaultKnownOptimizations",0
,(
function(self)
return (smalltalk:newArray({smalltalk:newArray({"core", smalltalk:newArray({smalltalk:newArray({"class", "%receiver%.$klass"}), smalltalk:newArray({"isNil", "(nil.isNil_(%receiver%))"}), smalltalk:newArray({"notNil", "(nil.isNil_(%receiver%)===false)"}), smalltalk:newArray({"==", "(nil.is_eqeq_(%receiver%,%arg1%))"}), smalltalk:newArray({"===", "(%receiver% === %arg1%)"}), smalltalk:newArray({"||", "(%receiver% || %arg1%)"}), smalltalk:newArray({"basicAt:", "((function(){var $1$=(%receiver%[%arg1%]);if(nil.isNil_($1$))return nil;return $1$;})())"}), smalltalk:newArray({"ifUndefined:", "hasBlockNoArguments:", "(function(){var $1$=(%receiver%);if (typeof($1$)==\"undefined\") return %arg1%();return $1$;})()"}), smalltalk:newArray({"basicAt:ifAbsent:", "((function(){var $1$=(%receiver%[%arg1%]);if(nil.isNil_($1$))return (%arg2%.value());return $1$;})())"}), smalltalk:newArray({"basicAt:put:", "(%receiver%[%arg1%]=(%arg2%))"})})}), smalltalk:newArray({"s8", smalltalk:newArray({"core", smalltalk:newArray({"whileTrue", "receiverIsBlockNoArguments:", "(function(){while(%receiver%()){};return nil})()"}), smalltalk:newArray({"whileFalse", "receiverIsBlockNoArguments:", "(function(){while(!%receiver%()){};return nil})()"}), smalltalk:newArray({"orNil", "(function(){var $1$=(%receiver%);return (nil.isNil_($1$)) ? nil : $1$;})()"}), smalltalk:newArray({"ifTrue:", "hasBlockNoArguments:", "((%receiver%).mustBeBoolean()==true ? %arg1%() : nil)"}), smalltalk:newArray({"ifFalse:", "hasBlockNoArguments:", "((%receiver%).mustBeBoolean()==true ? nil : %arg1%())"}), smalltalk:newArray({"whileTrue:", "allBlockNoArguments:", "(function(){while(%receiver%()){%arg1%()};return nil})()"}), smalltalk:newArray({"whileFalse:", "allBlockNoArguments:", "(function(){while(!%receiver%()){%arg1%()};return nil})()"}), smalltalk:newArray({"ifNil:", "hasBlockArguments:", "(function(){var $1$=(%receiver%);return (nil.isNil_($1$)) ? %arg1%() : $1$;})()"}), smalltalk:newArray({"ifNotNil:", "hasBlockArguments:", "(function(){var $1$=(%receiver%);return (nil.isNil_($1$)===false) ? %arg1%() : $1$;})()"}), smalltalk:newArray({"timesRepeat:", "hasBlockNoArguments:", "(function(){for(var $1$=(%receiver%);$1$>0;$1$--){%arg1%()};return nil})()"}), smalltalk:newArray({"isKindOf:", "smalltalk.is_kindOf_(%receiver%,(%arg1%))"}), smalltalk:newArray({"respondsTo:", "smalltalk.responds_to_(%receiver%,(%arg1%))"}), smalltalk:newArray({"ifTrue:ifFalse:", "hasBlockNoArguments:", "((%receiver%).mustBeBoolean()==true ? %arg1%() : %arg2%())"}), smalltalk:newArray({"ifFalse:ifTrue:", "hasBlockNoArguments:", "((%receiver%).mustBeBoolean()==true ? %arg2%() : %arg1%())"}), smalltalk:newArray({"ifNil:ifNotNil:", "hasBlockNoArguments:", "(nil.isNil_(%receiver%)) ? %arg1%() : %arg2%()"}), smalltalk:newArray({"ifNotNil:ifNil:", "hasBlockNoArguments:", "(nil.isNil_(%receiver%)===false) ? %arg1%() : %arg2%()"})})}), smalltalk:newArray({"magnitude", smalltalk:newArray({smalltalk:newArray({">", "hasNumber:", "(%receiver% > %arg1%)"}), smalltalk:newArray({"<", "hasNumber:", "(%receiver% < %arg1%)"}), smalltalk:newArray({">=", "hasNumber:", "(%receiver% >= %arg1%)"}), smalltalk:newArray({"<=", "hasNumber:", "(%receiver% <= %arg1%)"})})}), smalltalk:newArray({"math", smalltalk:newArray({smalltalk:newArray({"+", "hasNumber:", "(%receiver% + %arg1%)"}), smalltalk:newArray({"-", "hasNumber:", "(%receiver% - %arg1%)"}), smalltalk:newArray({"*", "hasNumber:", "(%receiver% * %arg1%)"}), smalltalk:newArray({"/", "hasNumber:", "(%receiver% / %arg1%)"}), smalltalk:newArray({"=", "hasNumber:", "(%receiver% == %arg1%)"})})})}))
;
end
)
,"defaults","defaultKnownOptimizations\r\n\t\" Private - Returns the default knownOptimizations of the receiver. \"\r\n\r\n\t^#(\t#(core  #(\r\n\t\t\t#(class\t'%receiver%.$klass' )\r\n\t\t\t#(isNil\t'(nil.isNil_(%receiver%))' )\r\n\t\t\t#(notNil '(nil.isNil_(%receiver%)===false)' )\r\n\t\t\t#(==\t'(nil.is_eqeq_(%receiver%,%arg1%))' )\r\n\t\t\t#(===\t'(%receiver% === %arg1%)' )\r\n\t\t\t#(||\t'(%receiver% || %arg1%)' )\r\n\t\t\t#(basicAt:\t'((function(){var $1$=(%receiver%[%arg1%]);if(nil.isNil_($1$))return nil;return $1$;})())' )\r\n\t\t\t#(ifUndefined: hasBlockNoArguments: '(function(){var $1$=(%receiver%);if (typeof($1$)==\"undefined\") return %arg1%();return $1$;})()' )\r\n\t\t\t#(basicAt:ifAbsent: '((function(){var $1$=(%receiver%[%arg1%]);if(nil.isNil_($1$))return (%arg2%.value());return $1$;})())' )\r\n\t\t\t#(basicAt:put: '(%receiver%[%arg1%]=(%arg2%))' )\r\n\t\t))\r\n\t\t#(s8\t#(\r\n\t\t\tcore\r\n\t\t\t#(whileTrue receiverIsBlockNoArguments: '(function(){while(%receiver%()){};return nil})()' )\r\n\t\t\t#(whileFalse receiverIsBlockNoArguments: '(function(){while(!%receiver%()){};return nil})()' )\r\n\t\t\t#(orNil '(function(){var $1$=(%receiver%);return (nil.isNil_($1$)) ? nil : $1$;})()' )\r\n\t\t\t#(ifTrue: hasBlockNoArguments: '((%receiver%).mustBeBoolean()==true ? %arg1%() : nil)' )\r\n\t\t\t#(ifFalse: hasBlockNoArguments: '((%receiver%).mustBeBoolean()==true ? nil : %arg1%())' )\r\n\t\t\t#(whileTrue: allBlockNoArguments: '(function(){while(%receiver%()){%arg1%()};return nil})()' )\r\n\t\t\t#(whileFalse: allBlockNoArguments: '(function(){while(!%receiver%()){%arg1%()};return nil})()' )\r\n\t\t\t#(ifNil: hasBlockArguments: '(function(){var $1$=(%receiver%);return (nil.isNil_($1$)) ? %arg1%() : $1$;})()' )\r\n\t\t\t#(ifNotNil: hasBlockArguments: '(function(){var $1$=(%receiver%);return (nil.isNil_($1$)===false) ? %arg1%() : $1$;})()' )\r\n\t\t\t#(timesRepeat: hasBlockNoArguments: '(function(){for(var $1$=(%receiver%);$1$>0;$1$--){%arg1%()};return nil})()' )\r\n\t\t\t#(isKindOf: 'smalltalk.is_kindOf_(%receiver%,(%arg1%))' )\r\n\t\t\t#(respondsTo: 'smalltalk.responds_to_(%receiver%,(%arg1%))' )\r\n\t\t\t#(ifTrue:ifFalse: hasBlockNoArguments: '((%receiver%).mustBeBoolean()==true ? %arg1%() : %arg2%())' )\r\n\t\t\t#(ifFalse:ifTrue: hasBlockNoArguments: '((%receiver%).mustBeBoolean()==true ? %arg2%() : %arg1%())' )\r\n\t\t\t#(ifNil:ifNotNil: hasBlockNoArguments: '(nil.isNil_(%receiver%)) ? %arg1%() : %arg2%()' )\r\n\t\t\t#(ifNotNil:ifNil: hasBlockNoArguments: '(nil.isNil_(%receiver%)===false) ? %arg1%() : %arg2%()' )\r\n\t\t))\r\n\t\t#(magnitude #(\r\n\t\t\t#(>  hasNumber: '(%receiver% > %arg1%)' )\r\n\t\t\t#(<  hasNumber: '(%receiver% < %arg1%)' )\r\n\t\t\t#(>= hasNumber: '(%receiver% >= %arg1%)' )\r\n\t\t\t#(<= hasNumber: '(%receiver% <= %arg1%)' )\r\n\t\t))\r\n\t\t#(math\t#(\r\n\t\t\t#(+ hasNumber: '(%receiver% + %arg1%)' )\r\n\t\t\t#(#- hasNumber: '(%receiver% - %arg1%)' )\r\n\t\t\t#(* hasNumber: '(%receiver% * %arg1%)' )\r\n\t\t\t#(/ hasNumber: '(%receiver% / %arg1%)' )\r\n\t\t\t#(= hasNumber: '(%receiver% == %arg1%)' )\r\n\t\t))\r\n\t)","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({smalltalk:newArray({\"core\", smalltalk:newArray({smalltalk:newArray({\"class\", \"%receiver%.$klass\"}), smalltalk:newArray({\"isNil\", \"(nil.isNil_(%receiver%))\"}), smalltalk:newArray({\"notNil\", \"(nil.isNil_(%receiver%)===false)\"}), smalltalk:newArray({\"==\", \"(nil.is_eqeq_(%receiver%,%arg1%))\"}), smalltalk:newArray({\"===\", \"(%receiver% === %arg1%)\"}), smalltalk:newArray({\"||\", \"(%receiver% || %arg1%)\"}), smalltalk:newArray({\"basicAt:\", \"((function(){var $1$=(%receiver%[%arg1%]);if(nil.isNil_($1$))return nil;return $1$;})())\"}), smalltalk:newArray({\"ifUndefined:\", \"hasBlockNoArguments:\", \"(function(){var $1$=(%receiver%);if (typeof($1$)==\\\"undefined\\\") return %arg1%();return $1$;})()\"}), smalltalk:newArray({\"basicAt:ifAbsent:\", \"((function(){var $1$=(%receiver%[%arg1%]);if(nil.isNil_($1$))return (%arg2%.value());return $1$;})())\"}), smalltalk:newArray({\"basicAt:put:\", \"(%receiver%[%arg1%]=(%arg2%))\"})})}), smalltalk:newArray({\"s8\", smalltalk:newArray({\"core\", smalltalk:newArray({\"whileTrue\", \"receiverIsBlockNoArguments:\", \"(function(){while(%receiver%()){};return nil})()\"}), smalltalk:newArray({\"whileFalse\", \"receiverIsBlockNoArguments:\", \"(function(){while(!%receiver%()){};return nil})()\"}), smalltalk:newArray({\"orNil\", \"(function(){var $1$=(%receiver%);return (nil.isNil_($1$)) ? nil : $1$;})()\"}), smalltalk:newArray({\"ifTrue:\", \"hasBlockNoArguments:\", \"((%receiver%).mustBeBoolean()==true ? %arg1%() : nil)\"}), smalltalk:newArray({\"ifFalse:\", \"hasBlockNoArguments:\", \"((%receiver%).mustBeBoolean()==true ? nil : %arg1%())\"}), smalltalk:newArray({\"whileTrue:\", \"allBlockNoArguments:\", \"(function(){while(%receiver%()){%arg1%()};return nil})()\"}), smalltalk:newArray({\"whileFalse:\", \"allBlockNoArguments:\", \"(function(){while(!%receiver%()){%arg1%()};return nil})()\"}), smalltalk:newArray({\"ifNil:\", \"hasBlockArguments:\", \"(function(){var $1$=(%receiver%);return (nil.isNil_($1$)) ? %arg1%() : $1$;})()\"}), smalltalk:newArray({\"ifNotNil:\", \"hasBlockArguments:\", \"(function(){var $1$=(%receiver%);return (nil.isNil_($1$)===false) ? %arg1%() : $1$;})()\"}), smalltalk:newArray({\"timesRepeat:\", \"hasBlockNoArguments:\", \"(function(){for(var $1$=(%receiver%);$1$>0;$1$--){%arg1%()};return nil})()\"}), smalltalk:newArray({\"isKindOf:\", \"smalltalk.is_kindOf_(%receiver%,(%arg1%))\"}), smalltalk:newArray({\"respondsTo:\", \"smalltalk.responds_to_(%receiver%,(%arg1%))\"}), smalltalk:newArray({\"ifTrue:ifFalse:\", \"hasBlockNoArguments:\", \"((%receiver%).mustBeBoolean()==true ? %arg1%() : %arg2%())\"}), smalltalk:newArray({\"ifFalse:ifTrue:\", \"hasBlockNoArguments:\", \"((%receiver%).mustBeBoolean()==true ? %arg2%() : %arg1%())\"}), smalltalk:newArray({\"ifNil:ifNotNil:\", \"hasBlockNoArguments:\", \"(nil.isNil_(%receiver%)) ? %arg1%() : %arg2%()\"}), smalltalk:newArray({\"ifNotNil:ifNil:\", \"hasBlockNoArguments:\", \"(nil.isNil_(%receiver%)===false) ? %arg1%() : %arg2%()\"})})}), smalltalk:newArray({\"magnitude\", smalltalk:newArray({smalltalk:newArray({\">\", \"hasNumber:\", \"(%receiver% > %arg1%)\"}), smalltalk:newArray({\"<\", \"hasNumber:\", \"(%receiver% < %arg1%)\"}), smalltalk:newArray({\">=\", \"hasNumber:\", \"(%receiver% >= %arg1%)\"}), smalltalk:newArray({\"<=\", \"hasNumber:\", \"(%receiver% <= %arg1%)\"})})}), smalltalk:newArray({\"math\", smalltalk:newArray({smalltalk:newArray({\"+\", \"hasNumber:\", \"(%receiver% + %arg1%)\"}), smalltalk:newArray({\"-\", \"hasNumber:\", \"(%receiver% - %arg1%)\"}), smalltalk:newArray({\"*\", \"hasNumber:\", \"(%receiver% * %arg1%)\"}), smalltalk:newArray({\"/\", \"hasNumber:\", \"(%receiver% / %arg1%)\"}), smalltalk:newArray({\"=\", \"hasNumber:\", \"(%receiver% == %arg1%)\"})})})}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"defaultOptimizations",0
,(
function(self)
return (smalltalk:newArray({"s8"}))
;
end
)
,"defaults","defaultOptimizations\r\n\t\" Private - Returns the default optimizations of the receiver. \"\r\n\r\n\t^#( s8 \"magnitude math\" )","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"s8\"}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"defaultRequirementsFor:",0
,(
function(self,aClass)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local constructorName=nil
local code=nil
(function()
	local _s8_value=((aClass):constructorName())
	constructorName=_s8_value; return _s8_value
end)()
;
(function() if ((((((constructorName))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:newArray({}))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Array):with_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("if("),constructorName))," && !")),(self):smalltalkNameFor_(aClass))),") smalltalk.mapClassName(\"")),(aClass):name())),"\",\"")),(aClass):category())),"\",")),constructorName)),",")),(self):smalltalkNameFor_((aClass):superclass()))),");")))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"export","defaultRequirementsFor: aClass\r\n\t\" Private - Return the default requirements to load aClass.\r\n\tDetect native bindings and generate code to rebind before class definition.\r\n\t\"\r\n\r\n\t| constructorName code |\r\n\tconstructorName := aClass constructorName.\r\n\tconstructorName isNil ifTrue: [ ^#() ].\r\n\t^Array with:\r\n\t\t'if(',constructorName,' && !'\r\n\t\t, (self smalltalkNameFor: aClass)\r\n\t\t,') smalltalk.mapClassName(\"'\r\n\t\t,aClass name ,'\",\"'\r\n\t\t,aClass category, '\",'\r\n\t\t,constructorName, ','\r\n\t\t,(self smalltalkNameFor: aClass superclass)\r\n\t\t,');'","(\r\nfunction(self,aClass)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal constructorName=nil\r\nlocal code=nil\r\n(function()\r\n\tlocal _s8_value=((aClass):constructorName())\r\n\tconstructorName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((constructorName))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:newArray({}))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Array):with_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"if(\"),constructorName)),\" && !\")),(self):smalltalkNameFor_(aClass))),\") smalltalk.mapClassName(\\\"\")),(aClass):name())),\"\\\",\\\"\")),(aClass):category())),\"\\\",\")),constructorName)),\",\")),(self):smalltalkNameFor_((aClass):superclass()))),\");\")))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"emmitCode:",0
,(
function(self,aString)
(self['@stream']):nextPutAll_(aString)
;
return (self)
;
end
)
,"private","emmitCode: aString\r\n\t\" Dump aString onto code stream. \"\r\n\r\n\tstream nextPutAll: aString","(\r\nfunction(self,aString)\r\n(self['@stream']):nextPutAll_(aString)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"emmitEnterMethod:",0
,(
function(self,aNode)
(self['@stream']):nextPutAll_("var self=this;\r\n")
;
return (self)
;
end
)
,"generation","emmitEnterMethod: aNode\r\n\t\" Private - Generate home context definitions, entering method aNode. \"\r\n\r\n\tstream nextPutAll: 'var self=this;\r\n'","(\r\nfunction(self,aNode)\r\n(self['@stream']):nextPutAll_(\"var self=this;\\r\\n\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"emmitFunction:arguments:doing:",0
,(
function(self,fName,argList,aBlock)
(self):compilerEvent_withArguments_ifNotHandled_("startFunction:with:", (smalltalk.Array):with_with_(fName, argList), (function()
(self['@stream']):nextPutAll_("function");(self):compilerEvent_with_("startFunctionName:", fName);return ((self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(("("),argList)),")")))
end))
;
(self['@stream']):nextPutAll_("{")
;
(self):compilerEvent_withArguments_("startFunctionBody:with:", (smalltalk.Array):with_with_(fName, argList))
;
(aBlock):evaluate()
;
(self['@stream']):nextPutAll_("}")
;
(self):compilerEvent_with_("endFunction:", fName)
;
return (self)
;
end
)
,"generation","emmitFunction: fName arguments: argList doing: aBlock\r\n\t\" Private - Emmit function on receiver's code stream. \"\r\n\r\n\tself\r\n\t\tcompilerEvent: #startFunction:with:\r\n\t\twithArguments: (Array with: fName with: argList)\r\n\t\tifNotHandled: [\r\n\t\t\tstream nextPutAll: 'function'.\r\n\t\t\tself compilerEvent: #startFunctionName: with: fName.\r\n\t\t\tstream nextPutAll: '(' ,argList ,$)\r\n\t\t].\r\n\tstream nextPutAll: ${.\r\n\tself\tcompilerEvent: #startFunctionBody:with:\r\n\t\twithArguments: (Array with: fName with: argList).\r\n\taBlock evaluate.\r\n\tstream nextPutAll: $}.\r\n\tself compilerEvent: #endFunction: with: fName.","(\r\nfunction(self,fName,argList,aBlock)\r\n(self):compilerEvent_withArguments_ifNotHandled_(\"startFunction:with:\", (smalltalk.Array):with_with_(fName, argList), (function()\r\n(self['@stream']):nextPutAll_(\"function\");(self):compilerEvent_with_(\"startFunctionName:\", fName);return ((self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\"(\"),argList)),\")\")))\r\nend))\r\n;\r\n(self['@stream']):nextPutAll_(\"{\")\r\n;\r\n(self):compilerEvent_withArguments_(\"startFunctionBody:with:\", (smalltalk.Array):with_with_(fName, argList))\r\n;\r\n(aBlock):evaluate()\r\n;\r\n(self['@stream']):nextPutAll_(\"}\")\r\n;\r\n(self):compilerEvent_with_(\"endFunction:\", fName)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"emmitFunction:node:arguments:doing:",0
,(
function(self,fName,aNode,argumentNames,aBlock)
local args=nil
(function()
	local _s8_value=((""):writeStream())
	args=_s8_value; return _s8_value
end)()
;
(argumentNames):do_separatedBy_((function(each)
((self):tempVariables()):add_(each);return ((args):nextPutAll_((self):localName_(each)))
end), (function()
return ((args):nextPutAll_(","))
end))
;
(self):emmitFunction_arguments_doing_(fName, (args):contents(), aBlock)
;
return (self)
;
end
)
,"generation","emmitFunction: fName node: aNode arguments: argumentNames doing: aBlock\r\n\t\" Private - Generate function for aNode. \"\r\n\r\n\t| args |\r\n\targs := '' writeStream.\r\n\targumentNames do: [:each | \r\n\t\tself tempVariables add: each.\r\n\t\targs nextPutAll: (self localName: each) ]\r\n\t    separatedBy: [ args nextPutAll: ',' ].\r\n\tself\temmitFunction: fName\r\n\t\targuments: args contents\r\n\t\tdoing: aBlock","(\r\nfunction(self,fName,aNode,argumentNames,aBlock)\r\nlocal args=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\targs=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(argumentNames):do_separatedBy_((function(each)\r\n((self):tempVariables()):add_(each);return ((args):nextPutAll_((self):localName_(each)))\r\nend), (function()\r\nreturn ((args):nextPutAll_(\",\"))\r\nend))\r\n;\r\n(self):emmitFunction_arguments_doing_(fName, (args):contents(), aBlock)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"emmitFunction:node:doing:",0
,(
function(self,fName,aNode,aBlock)
return ((self):emmitFunction_node_arguments_doing_(fName, aNode, (aNode):argumentNames(), aBlock))
;
end
)
,"generation","emmitFunction: fName node: aNode doing: aBlock\r\n\t\" Private - Generate function for aNode. \"\r\n\r\n\t^self\temmitFunction: fName\r\n\t\tnode: aNode arguments: aNode argumentNames\r\n\t\tdoing: aBlock","(\r\nfunction(self,fName,aNode,aBlock)\r\nreturn ((self):emmitFunction_node_arguments_doing_(fName, aNode, (aNode):argumentNames(), aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"emmitFunctionDoing:",0
,(
function(self,aBlock)
return ((self):emmitFunction_arguments_doing_("", "", aBlock))
;
end
)
,"generation","emmitFunctionDoing: aBlock\r\n\t\" Private - Emmit function on receiver's code stream. \"\r\n\r\n\t^self emmitFunction: '' arguments: '' doing: aBlock","(\r\nfunction(self,aBlock)\r\nreturn ((self):emmitFunction_arguments_doing_(\"\", \"\", aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"emmitMethod:",0
,(
function(self,aMethodNode)
local code=nil
(function()
	local _s8_value=((0))
	self['@nestedBlocks']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(false)
	self['@earlyReturn']=_s8_value; return _s8_value
end)()
;
((self):compiler()):resetVariables()
;
(function()
	local _s8_value=(((aMethodNode):selector()):asSelector())
	self['@functionName']=_s8_value; return _s8_value
end)()
;
(aMethodNode):validateArguments()
;
(self):emmitFunction_node_doing_(self['@functionName'], aMethodNode, (function()
(self):compilerEvent_with_("startMethod:", aMethodNode);(function()
	local _s8_value=((self):generatedCodeFor_while_(aMethodNode, (function()
(self):compilerEvent_withArguments_ifNotHandled_("enterMethod:", (smalltalk.Array):with_(aMethodNode), (function()
return ((self):emmitEnterMethod_(aMethodNode))
end));return (((aMethodNode):nodes()):do_((function(each)
return ((self):visit_(each))
end)))
end)))
	code=_s8_value; return _s8_value
end)();(self):generate_early_return_(self['@functionName'], self['@earlyReturn'], code);return ((self):compilerEvent_with_("endMethod:", aMethodNode))
end))
;
return (self)
;
end
)
,"generation","emmitMethod: aMethodNode\r\n\t\" Private - Generate code of aMethodNode. \"\r\n\r\n\t| code |\r\n\tnestedBlocks := 0.\r\n\tearlyReturn := false.\r\n\tself compiler resetVariables.\r\n\r\n\tfunctionName := aMethodNode selector asSelector.\r\n\taMethodNode validateArguments.\r\n\tself emmitFunction: functionName node: aMethodNode doing: [\r\n\t\tself compilerEvent: #startMethod: with: aMethodNode.\r\n\t\tcode := self generatedCodeFor: aMethodNode while: [\r\n\t\t\tself compilerEvent: #enterMethod: withArguments: (Array with: aMethodNode)\r\n\t\t\t\tifNotHandled: [ self emmitEnterMethod: aMethodNode ].\r\n\t\t\taMethodNode nodes do: [:each | self visit: each ].\r\n\t\t].\r\n\t\tself generate: functionName early: earlyReturn return: code.\r\n\t\tself compilerEvent: #endMethod: with: aMethodNode.\r\n\t]","(\r\nfunction(self,aMethodNode)\r\nlocal code=nil\r\n(function()\r\n\tlocal _s8_value=((0))\r\n\tself['@nestedBlocks']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(false)\r\n\tself['@earlyReturn']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((self):compiler()):resetVariables()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((aMethodNode):selector()):asSelector())\r\n\tself['@functionName']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aMethodNode):validateArguments()\r\n;\r\n(self):emmitFunction_node_doing_(self['@functionName'], aMethodNode, (function()\r\n(self):compilerEvent_with_(\"startMethod:\", aMethodNode);(function()\r\n\tlocal _s8_value=((self):generatedCodeFor_while_(aMethodNode, (function()\r\n(self):compilerEvent_withArguments_ifNotHandled_(\"enterMethod:\", (smalltalk.Array):with_(aMethodNode), (function()\r\nreturn ((self):emmitEnterMethod_(aMethodNode))\r\nend));return (((aMethodNode):nodes()):do_((function(each)\r\nreturn ((self):visit_(each))\r\nend)))\r\nend)))\r\n\tcode=_s8_value; return _s8_value\r\nend)();(self):generate_early_return_(self['@functionName'], self['@earlyReturn'], code);return ((self):compilerEvent_with_(\"endMethod:\", aMethodNode))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"emmitReturn:",0
,(
function(self,aBlockOrString)
return ((self):emmitReturn_early_(aBlockOrString, false))
;
end
)
,"generation","emmitReturn: aBlockOrString\r\n\t\" Private - Generate return code. \"\r\n\r\n\t^self emmitReturn: aBlockOrString early: false","(\r\nfunction(self,aBlockOrString)\r\nreturn ((self):emmitReturn_early_(aBlockOrString, false))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"emmitReturn:early:",0
,(
function(self,aBlockOrString,early)
(self):compilerEvent_with_("aboutReturn:", aBlockOrString)
;
(self['@stream']):nextPutAll_("return ")
;
(self):compilerEvent_with_("return:", aBlockOrString)
;
(function() if ((((aBlockOrString):isString()))==true) then return ((function()
return ((self['@stream']):nextPutAll_(aBlockOrString))
end))() else return ((function()
return ((aBlockOrString):evaluate())
end))() end end)()
;
(function() if ((((((aBlockOrString))~=nil)))==true) then return ((function()
(self):compilerEvent_withArguments_("returned:early:", (smalltalk.Array):with_with_(aBlockOrString, early));return ((self['@stream']):nextPutAll_(";"))
end))() else return nil end end)()
;
return (self)
;
end
)
,"generation","emmitReturn: aBlockOrString early: early\r\n\t\" Private - Generate return code. \"\r\n\r\n\tself compilerEvent: #aboutReturn: with: aBlockOrString.\r\n\tstream nextPutAll: 'return '.\r\n\tself compilerEvent: #return: with: aBlockOrString.\r\n\taBlockOrString isString\r\n\t\tifTrue: [ stream nextPutAll: aBlockOrString ]\r\n\t\tifFalse: [ aBlockOrString evaluate ].\r\n\taBlockOrString notNil ifTrue: [\r\n\t\tself\tcompilerEvent: #returned:early:\r\n\t\t\twithArguments: (Array with: aBlockOrString with: early).\r\n\t\tstream nextPutAll: $;.\r\n\t]","(\r\nfunction(self,aBlockOrString,early)\r\n(self):compilerEvent_with_(\"aboutReturn:\", aBlockOrString)\r\n;\r\n(self['@stream']):nextPutAll_(\"return \")\r\n;\r\n(self):compilerEvent_with_(\"return:\", aBlockOrString)\r\n;\r\n(function() if ((((aBlockOrString):isString()))==true) then return ((function()\r\nreturn ((self['@stream']):nextPutAll_(aBlockOrString))\r\nend))() else return ((function()\r\nreturn ((aBlockOrString):evaluate())\r\nend))() end end)()\r\n;\r\n(function() if ((((((aBlockOrString))~=nil)))==true) then return ((function()\r\n(self):compilerEvent_withArguments_(\"returned:early:\", (smalltalk.Array):with_with_(aBlockOrString, early));return ((self['@stream']):nextPutAll_(\";\"))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"emmitedCode",0
,(
function(self)
return ((function() if ((((((self['@stream']))~=nil)))==true) then return ((function()
return ((self['@stream']):contents())
end))() else return nil end end)())
;
end
)
,"results","emmitedCode\r\n\t\" Return the emmited code during compilation (or nil). \"\r\n\r\n\t^stream notNil ifTrue: [ stream contents ]","(\r\nfunction(self)\r\nreturn ((function() if ((((((self['@stream']))~=nil)))==true) then return ((function()\r\nreturn ((self['@stream']):contents())\r\nend))() else return nil end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"arrayLiteralStart",0
,(
function(self)
return ("[")
;
end
)
,"literals","arrayLiteralStart\r\n\t\" Private - Returns the code string to start exporting an array literal. \"\r\n\r\n\t^$[","(\r\nfunction(self)\r\nreturn (\"[\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"arrayLiteralEnd",0
,(
function(self)
return ("]")
;
end
)
,"literals","arrayLiteralEnd\r\n\t\" Private - Returns the code string to end exporting an array literal. \"\r\n\r\n\t^$]","(\r\nfunction(self)\r\nreturn (\"]\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"escaped:",0
,(
function(self,string)
return ((string):escapedCode())
;
end
)
,"literals","escaped: string\r\n\t\" Private - Returns the code string to export string literal. \"\r\n\r\n\t^string escapedCode","(\r\nfunction(self,string)\r\nreturn ((string):escapedCode())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"nativeSourceFor:",0
,(
function(self,aMethod)
return ((aMethod):compiledSource())
;
end
)
,"private","nativeSourceFor: aMethod\r\n\t\" Private - Return the code of native function implementing aMethod. \"\r\n\r\n\t^aMethod compiledSource","(\r\nfunction(self,aMethod)\r\nreturn ((aMethod):compiledSource())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"export:method:on:",0
,(
function(self,aClass,aMethod,aStream)
local functionName=nil
local selectorName=nil
local bar=nil
(function()
	local _s8_value=(smalltalk:asString_(("\"")))
	bar=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((aMethod):selector())
	functionName=_s8_value; return _s8_value
end)()
;
(function() if (((((functionName):first()):isBackSlash()))==true) then return ((function()
(function()
	local _s8_value=((self):escaped_((aMethod):selector()))
	selectorName=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=((self):escaped_((functionName):asSelector()))
	functionName=_s8_value; return _s8_value
end)())
end))() else return ((function()
(function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((bar),(aMethod):selector())),bar))
	selectorName=_s8_value; return _s8_value
end)();return ((function() if ((((smalltalk:isObject_equalTo_(((functionName):asSelector()),((aMethod):selector()):replace_with_(":", "_")))))==true) then return ((function()
return ((function()
	local _s8_value=("0")
	functionName=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((bar),(functionName):asSelector())),bar))
	functionName=_s8_value; return _s8_value
end)())
end))() end end)())
end))() end end)()
;
(aStream):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk.bind("),(self):smalltalkNameFor_(aClass))),",")),selectorName)),",")),functionName)),"\r\n,")),(self):nativeSourceFor_(aMethod)))
;
(function() if ((((smalltalk:boolean_and_((((((aMethod):source()))~=nil)),(function()
return (((aMethod):source()):notEmpty())
end)))))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):cr();
return ((_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((","),(self):escaped_((aMethod):category()))),",")),(self):escaped_((aMethod):source()))))
end
)(aStream))
end))() else return nil end end)()
;
(aStream):nextPutAll_(");\r\n")
;
return (self)
;
end
)
,"export","export: aClass method: aMethod on: aStream\r\n\t\" Export a method definition of aClass onto aStream. \"\r\n\r\n\t| functionName selectorName bar |\r\n\tbar := $\" asString.\r\n\tfunctionName := aMethod selector.\r\n\tfunctionName first isBackSlash ifTrue: [\r\n\t\tselectorName := self escaped: aMethod selector.\r\n\t\tfunctionName := self escaped: functionName asSelector.\r\n\t] ifFalse: [\r\n\t\tselectorName := bar ,aMethod selector, bar.\r\n\t\tfunctionName asSelector = (aMethod selector replace: $: with: $_) ifTrue: [\r\n\t\t\tfunctionName := '0' \"we exploit the fact that 0 isNil to signal a compact selector\"\r\n\t\t] ifFalse: [\r\n\t\t\tfunctionName := bar ,functionName asSelector, bar\r\n\t\t].\r\n\t].\r\n\taStream\r\n\t\tnextPutAll: 'smalltalk.bind(', (self smalltalkNameFor: aClass)\r\n\t\t\t,',', selectorName, ',', functionName ,'\r\n,'\t\t\t, (self nativeSourceFor: aMethod).\r\n\t(aMethod source notNil and: [ aMethod source notEmpty ]) ifTrue: [\r\n\t    aStream cr; \"warning: this cr is important for debugger\"\r\n\t\tnextPutAll: ',', (self escaped: aMethod category)\r\n\t\t\t,',', (self escaped: aMethod source).\r\n\t].\r\n\taStream nextPutAll: ');\r\n'","(\r\nfunction(self,aClass,aMethod,aStream)\r\nlocal functionName=nil\r\nlocal selectorName=nil\r\nlocal bar=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:asString_((\"\\\"\")))\r\n\tbar=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aMethod):selector())\r\n\tfunctionName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((((functionName):first()):isBackSlash()))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((self):escaped_((aMethod):selector()))\r\n\tselectorName=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=((self):escaped_((functionName):asSelector()))\r\n\tfunctionName=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((bar),(aMethod):selector())),bar))\r\n\tselectorName=_s8_value; return _s8_value\r\nend)();return ((function() if ((((smalltalk:isObject_equalTo_(((functionName):asSelector()),((aMethod):selector()):replace_with_(\":\", \"_\")))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(\"0\")\r\n\tfunctionName=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((bar),(functionName):asSelector())),bar))\r\n\tfunctionName=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)())\r\nend))() end end)()\r\n;\r\n(aStream):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.bind(\"),(self):smalltalkNameFor_(aClass))),\",\")),selectorName)),\",\")),functionName)),\"\\r\\n,\")),(self):nativeSourceFor_(aMethod)))\r\n;\r\n(function() if ((((smalltalk:boolean_and_((((((aMethod):source()))~=nil)),(function()\r\nreturn (((aMethod):source()):notEmpty())\r\nend)))))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\nreturn ((_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\",\"),(self):escaped_((aMethod):category()))),\",\")),(self):escaped_((aMethod):source()))))\r\nend\r\n)(aStream))\r\nend))() else return nil end end)()\r\n;\r\n(aStream):nextPutAll_(\");\\r\\n\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"exportDefinitionOf:on:",0
,(
function(self,aClass,aStream)
((self):loadRequirementsOf_(aClass)):do_((function(each)
return ((function() if ((((each):isString()))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_(each);
return ((_s8_rec):cr())
end
)(aStream))
end))() else return ((function()
return ((aStream):fileOutOn_(aStream))
end))() end end)())
end))
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk.addClass(\""),(aClass):name())),"\","));
(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass):superclass()));
return ((_s8_rec):nextPutAll_(","))
end
)(aStream)
;
(self):exportArrayLiteral_on_newLine_((aClass):instanceVariableNames(), aStream, false)
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((",\""),(aClass):category())),"\""));
(_s8_rec):nextPutAll_(");");
return ((_s8_rec):cr())
end
)(aStream)
;
(function() if ((((smalltalk:boolean_and_((((((aClass):comment()))~=nil)),(function()
return (((aClass):comment()):notEmpty())
end)))))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
(_s8_rec):nextPutAll_(smalltalk:object_comma_((".$comment="),(self):escaped_((aClass):comment())));
(_s8_rec):nextPutAll_(";");
return ((_s8_rec):cr())
end
)(aStream))
end))() else return nil end end)()
;
(function() if (((((((aClass):classVariables()))~=nil)))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
(_s8_rec):nextPutAll_(smalltalk:object_comma_((".$classVariableNames="),(self):escaped_((aClass):classVariablesString())));
(_s8_rec):nextPutAll_(";");
return ((_s8_rec):cr())
end
)(aStream))
end))() else return nil end end)()
;
(function() if (((((((aClass):poolDictionaries()))~=nil)))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
return ((_s8_rec):nextPutAll_(".$poolDictionaries="))
end
)(aStream);return ((self):exportArrayLiteral_on_newLine_((aClass):poolDictionaries(), aStream, true))
end))() else return nil end end)()
;
(function() if (((((aClass):propertyNames()):notEmpty()))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
return ((_s8_rec):nextPutAll_(".$propertyNames="))
end
)(aStream);return ((self):exportArrayLiteral_on_newLine_((aClass):propertyNames(), aStream, true))
end))() else return nil end end)()
;
return (self)
;
end
)
,"export","exportDefinitionOf: aClass on: aStream\r\n\t\" Export definition of aClass onto aStream. \"\r\n\r\n\t(self loadRequirementsOf: aClass) do: [:each|\r\n\t\teach isString\r\n\t\tifTrue: [ aStream nextPutAll: each;cr ]\r\n\t\tifFalse: [ aStream fileOutOn: aStream ].\r\n\t].\r\n\taStream \r\n\t    nextPutAll: 'smalltalk.addClass(\"', aClass name, '\",';\r\n\t    nextPutAll: (self smalltalkNameFor: aClass superclass);\r\n\t    nextPutAll: ','.\r\n\tself exportArrayLiteral: aClass instanceVariableNames on: aStream newLine: false.\r\n\taStream\r\n\t    nextPutAll: ',\"' ,aClass category, $\";\r\n\t    nextPutAll: ');'; cr.\r\n\t(aClass comment notNil and: [aClass comment notEmpty]) ifTrue: [\r\n\t    aStream\r\n\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\tnextPutAll: '.$comment=' ,(self escaped: aClass comment);\r\n\t\tnextPutAll: ';'; cr\r\n\t].\r\n\taClass classVariables notNil ifTrue: [\r\n\t    aStream\r\n\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\tnextPutAll: '.$classVariableNames=' ,(self escaped: aClass classVariablesString);\r\n\t\tnextPutAll: ';'; cr\r\n\t].\r\n\taClass poolDictionaries notNil ifTrue: [\r\n\t    aStream\r\n\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\tnextPutAll: '.$poolDictionaries='.\r\n\t   self exportArrayLiteral: aClass poolDictionaries on: aStream newLine: true.\r\n\t].\r\n\taClass propertyNames notEmpty ifTrue: [\r\n\t    aStream\r\n\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\tnextPutAll: '.$propertyNames='.\r\n\t   self exportArrayLiteral: aClass propertyNames on: aStream newLine: true.\r\n\t].","(\r\nfunction(self,aClass,aStream)\r\n((self):loadRequirementsOf_(aClass)):do_((function(each)\r\nreturn ((function() if ((((each):isString()))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(each);\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(aStream))\r\nend))() else return ((function()\r\nreturn ((aStream):fileOutOn_(aStream))\r\nend))() end end)())\r\nend))\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.addClass(\\\"\"),(aClass):name())),\"\\\",\"));\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass):superclass()));\r\nreturn ((_s8_rec):nextPutAll_(\",\"))\r\nend\r\n)(aStream)\r\n;\r\n(self):exportArrayLiteral_on_newLine_((aClass):instanceVariableNames(), aStream, false)\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\",\\\"\"),(aClass):category())),\"\\\"\"));\r\n(_s8_rec):nextPutAll_(\");\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(aStream)\r\n;\r\n(function() if ((((smalltalk:boolean_and_((((((aClass):comment()))~=nil)),(function()\r\nreturn (((aClass):comment()):notEmpty())\r\nend)))))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((\".$comment=\"),(self):escaped_((aClass):comment())));\r\n(_s8_rec):nextPutAll_(\";\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(aStream))\r\nend))() else return nil end end)()\r\n;\r\n(function() if (((((((aClass):classVariables()))~=nil)))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((\".$classVariableNames=\"),(self):escaped_((aClass):classVariablesString())));\r\n(_s8_rec):nextPutAll_(\";\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(aStream))\r\nend))() else return nil end end)()\r\n;\r\n(function() if (((((((aClass):poolDictionaries()))~=nil)))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\nreturn ((_s8_rec):nextPutAll_(\".$poolDictionaries=\"))\r\nend\r\n)(aStream);return ((self):exportArrayLiteral_on_newLine_((aClass):poolDictionaries(), aStream, true))\r\nend))() else return nil end end)()\r\n;\r\n(function() if (((((aClass):propertyNames()):notEmpty()))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\nreturn ((_s8_rec):nextPutAll_(\".$propertyNames=\"))\r\nend\r\n)(aStream);return ((self):exportArrayLiteral_on_newLine_((aClass):propertyNames(), aStream, true))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"exportMetaDefinitionOf:on:",0
,(
function(self,aClass,aStream)
(function() if ((((((aClass)["$klass"]):instanceVariableNames()):notEmpty()))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass)["$klass"]));
return ((_s8_rec):nextPutAll_(".$iVarNames="))
end
)(aStream);return ((self):exportArrayLiteral_on_newLine_(((aClass)["$klass"]):instanceVariableNames(), aStream, true))
end))() else return nil end end)()
;
(function() if ((((((aClass)["$klass"]):propertyNames()):notEmpty()))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass)["$klass"]));
return ((_s8_rec):nextPutAll_(".$propertyNames="))
end
)(aStream);return ((self):exportArrayLiteral_on_newLine_(((aClass)["$klass"]):propertyNames(), aStream, true))
end))() else return nil end end)()
;
return (self)
;
end
)
,"export","exportMetaDefinitionOf: aClass on: aStream\r\n\t\" Export class side definition of aClass onto aStream. \"\r\n\r\n\taClass class instanceVariableNames notEmpty ifTrue: [\r\n\t\taStream \r\n\t\t\tnextPutAll: (self smalltalkNameFor: aClass class);\r\n\t\t\tnextPutAll: '.$iVarNames='.\r\n\t\tself exportArrayLiteral: aClass class instanceVariableNames on: aStream newLine: true.\r\n\t].\r\n\r\n\taClass class propertyNames notEmpty ifTrue: [\r\n\t\taStream \r\n\t\t\tnextPutAll: (self smalltalkNameFor: aClass class);\r\n\t\t\tnextPutAll: '.$propertyNames='.\r\n\t\tself exportArrayLiteral: aClass class propertyNames on: aStream newLine: true.\r\n\t].","(\r\nfunction(self,aClass,aStream)\r\n(function() if ((((((aClass)[\"$klass\"]):instanceVariableNames()):notEmpty()))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass)[\"$klass\"]));\r\nreturn ((_s8_rec):nextPutAll_(\".$iVarNames=\"))\r\nend\r\n)(aStream);return ((self):exportArrayLiteral_on_newLine_(((aClass)[\"$klass\"]):instanceVariableNames(), aStream, true))\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((aClass)[\"$klass\"]):propertyNames()):notEmpty()))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass)[\"$klass\"]));\r\nreturn ((_s8_rec):nextPutAll_(\".$propertyNames=\"))\r\nend\r\n)(aStream);return ((self):exportArrayLiteral_on_newLine_(((aClass)[\"$klass\"]):propertyNames(), aStream, true))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"generate:early:return:",0
,(
function(self,fName,early,code)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((early))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self['@stream']):nextPutAll_(code))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_("var $s8Ret$={name:\"stReturn\"};try{");
(_s8_rec):nextPutAll_(code);
return ((_s8_rec):nextPutAll_("} catch($$ex) {if($$ex === $s8Ret$){"))
end
)(self['@stream'])
;
(self):emmitReturn_early_("$$ex.result", true)
;
(self['@stream']):nextPutAll_("} throw($$ex);}")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"generation","generate: fName early: early return: code\r\n\t\" Private - Generate code to catch early return of code expression in fName. \"\r\n\r\n\tearly ifFalse: [ ^stream nextPutAll: code ].\r\n\tstream\tnextPutAll: 'var $s8Ret$={name:\"stReturn\"};try{';\r\n\t\tnextPutAll: code;\r\n\t\tnextPutAll: '} catch($$ex) {if($$ex === $s8Ret$){'.\r\n\tself emmitReturn: '$$ex.result' early: true.\r\n\tstream\tnextPutAll: '} throw($$ex);}'","(\r\nfunction(self,fName,early,code)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((early))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self['@stream']):nextPutAll_(code))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\"var $s8Ret$={name:\\\"stReturn\\\"};try{\");\r\n(_s8_rec):nextPutAll_(code);\r\nreturn ((_s8_rec):nextPutAll_(\"} catch($$ex) {if($$ex === $s8Ret$){\"))\r\nend\r\n)(self['@stream'])\r\n;\r\n(self):emmitReturn_early_(\"$$ex.result\", true)\r\n;\r\n(self['@stream']):nextPutAll_(\"} throw($$ex);}\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"generatedCodeFor:while:",0
,(
function(self,aNode,aBlock)
local backup=nil
local result=nil
(function()
	local _s8_value=((smalltalk.Array):with_with_(self['@stream'], self['@startOffset']))
	backup=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self['@stream']):position())
	self['@startOffset']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((""):writeStream())
	self['@stream']=_s8_value; return _s8_value
end)()
;
(self):visit_doing_(aNode, aBlock)
;
(function()
	local _s8_value=((self['@stream']):contents())
	result=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((backup):first())
	self['@stream']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((backup):last())
	self['@startOffset']=_s8_value; return _s8_value
end)()
;
return (result)
;
end
)
,"generation","generatedCodeFor: aNode while: aBlock\r\n\t\" Private - Return the code generated during evaluation of aBlock. \"\r\n\r\n\t| backup result |\r\n\tbackup := Array with: stream with: startOffset.\r\n\tstartOffset := stream position.\r\n\tstream := '' writeStream.\r\n\tself visit: aNode doing: aBlock.\r\n\tresult := stream contents.\r\n\tstream := backup first.\r\n\tstartOffset := backup last.\r\n\t^result ","(\r\nfunction(self,aNode,aBlock)\r\nlocal backup=nil\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):with_with_(self['@stream'], self['@startOffset']))\r\n\tbackup=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@stream']):position())\r\n\tself['@startOffset']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tself['@stream']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):visit_doing_(aNode, aBlock)\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@stream']):contents())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((backup):first())\r\n\tself['@stream']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((backup):last())\r\n\tself['@startOffset']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"includeSourceCode",0
,(
function(self)
return (true)
;
end
)
,"generation","includeSourceCode\r\n\t\" Private - Return true if source code is included as method argument. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"isOperation:",0
,(
function(self,aNode)
return ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((((aNode):arguments()):size()),(1)))),(function()
return ((smalltalk:boolean_and_((((((aNode):selector()):size()) > (1))),(function()
return ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((((aNode):selector()):first()),"#"))),(function()
return (("|&\\\\+*/=><,@%~-"):includes_(((aNode):selector()):at_((2))))
end))))
end))))
end))))
;
end
)
,"operations","isOperation: aNode\r\n\t\" Private - Return true if aNode is an operation. \"\r\n\r\n\t^aNode arguments size = 1 and: [\r\n\t\taNode selector size > 1 and: [\r\n\t\taNode selector first = $# and: [\r\n\t\t'|&\\\\+*/=><,@%~-' includes: (aNode selector at: 2)\r\n\t\t]]]","(\r\nfunction(self,aNode)\r\nreturn ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((((aNode):arguments()):size()),(1)))),(function()\r\nreturn ((smalltalk:boolean_and_((((((aNode):selector()):size()) > (1))),(function()\r\nreturn ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((((aNode):selector()):first()),\"#\"))),(function()\r\nreturn ((\"|&\\\\\\\\+*/=><,@%~-\"):includes_(((aNode):selector()):at_((2))))\r\nend))))\r\nend))))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"loadRequirementsOf:",0
,(
function(self,aClass)
local requirements=nil
(function()
	local _s8_value=((self):defaultRequirementsFor_(aClass))
	requirements=_s8_value; return _s8_value
end)()
;
(aClass):loadRequirementsInto_(requirements)
;
return (requirements)
;
end
)
,"export","loadRequirementsOf: aClass\r\n\t\" Private - Return the requirements to load aClass.\r\n\tIt must be a collection of requirements.\r\n\taRequirement must implement #fileOutOn: or must be aString(native code) to execute just before defining aClass.\r\n\t\"\r\n\r\n\t| requirements |\r\n\trequirements := self defaultRequirementsFor: aClass.\r\n\taClass loadRequirementsInto: requirements.\r\n\t^requirements","(\r\nfunction(self,aClass)\r\nlocal requirements=nil\r\n(function()\r\n\tlocal _s8_value=((self):defaultRequirementsFor_(aClass))\r\n\trequirements=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aClass):loadRequirementsInto_(requirements)\r\n;\r\nreturn (requirements)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"newMethodFunctionName",0
,(
function(self)
return ("smalltalk.newMethod")
;
end
)
,"generation","newMethodFunctionName\r\n\t\" Private - Return the function to call when installing a new method. \"\r\n\r\n\t^'smalltalk.newMethod'","(\r\nfunction(self)\r\nreturn (\"smalltalk.newMethod\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"position",0
,(
function(self)
return (((self['@stream']):position()):_plus(self['@startOffset']))
;
end
)
,"accessing","position\r\n\t\" Return the current position in output stream. \"\r\n\r\n\t^stream position + startOffset","(\r\nfunction(self)\r\nreturn (((self['@stream']):position()):_plus(self['@startOffset']))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"startCompiling:",0
,(
function(self,aNode)
smalltalk:superImplementor(smalltalk.JSEncoder,"startCompiling_")(self,aNode)
;
(function()
	local _s8_value=((""):writeStream())
	self['@stream']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self['@stream']):position())
	self['@startOffset']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"private","startCompiling: aNode\r\n\t\" Private - Set the receiver to start emmiting code for aNode. \"\r\n\r\n\tsuper startCompiling: aNode.\r\n\tstream := '' writeStream.\r\n\tstartOffset := stream position.","(\r\nfunction(self,aNode)\r\nsmalltalk:superImplementor(smalltalk.JSEncoder,\"startCompiling_\")(self,aNode)\r\n;\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tself['@stream']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@stream']):position())\r\n\tself['@startOffset']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"stream",0
,(
function(self)
return (self['@stream'])
;
end
)
,"accessing","stream\r\n\t\" Return the current stream (or nil). \"\r\n\r\n\t^stream","(\r\nfunction(self)\r\nreturn (self['@stream'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visit:",0
,(
function(self,aNode)
return ((self):visit_doing_(aNode, (function()
return (smalltalk:superImplementor(smalltalk.JSEncoder,"visit_")(self,aNode))
end)))
;
end
)
,"visiting","visit: aNode\r\n\t^self visit: aNode doing: [ super visit: aNode ]","(\r\nfunction(self,aNode)\r\nreturn ((self):visit_doing_(aNode, (function()\r\nreturn (smalltalk:superImplementor(smalltalk.JSEncoder,\"visit_\")(self,aNode))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visit:doing:",0
,(
function(self,aNode,aBlock)
local cookie=nil
local result=nil
(function()
	local _s8_value=((self):compilerEvent_with_ifNotHandled_("needsCookie:", aNode, (function()
return (nil)
end)))
	cookie=_s8_value; return _s8_value
end)()
;
(function() if ((((((cookie))~=nil)))==true) then return ((function()
return ((self):compilerEvent_with_("startVisit:", cookie))
end))() else return nil end end)()
;
(function()
	local _s8_value=((aBlock):evaluate())
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((((cookie))~=nil)))==true) then return ((function()
return ((self):compilerEvent_with_("endVisit:", cookie))
end))() else return nil end end)()
;
return (result)
;
end
)
,"visiting","visit: aNode doing: aBlock\r\n\t| cookie result |\r\n\tcookie := self\r\n\t\tcompilerEvent: #needsCookie: with: aNode\r\n\t\tifNotHandled: [ nil ].\r\n\tcookie notNil ifTrue: [ self compilerEvent: #startVisit: with: cookie ].\r\n\tresult := aBlock evaluate.\r\n\tcookie notNil ifTrue: [ self compilerEvent: #endVisit: with: cookie ].\r\n\t^result","(\r\nfunction(self,aNode,aBlock)\r\nlocal cookie=nil\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):compilerEvent_with_ifNotHandled_(\"needsCookie:\", aNode, (function()\r\nreturn (nil)\r\nend)))\r\n\tcookie=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((cookie))~=nil)))==true) then return ((function()\r\nreturn ((self):compilerEvent_with_(\"startVisit:\", cookie))\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aBlock):evaluate())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((cookie))~=nil)))==true) then return ((function()\r\nreturn ((self):compilerEvent_with_(\"endVisit:\", cookie))\r\nend))() else return nil end end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitArgument:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aNode):isAssignment()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitLocalAssignment_(aNode))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visit_(aNode))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitArgument: aNode\r\n\t\" Private - Visit the argument emmitting output. \"\r\n\r\n\taNode isAssignment ifTrue: [ ^self visitLocalAssignment: aNode. ].\r\n\t^self visit: aNode","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aNode):isAssignment()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitLocalAssignment_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visit_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitAssignmentNode:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local assignee=nil
(function() if ((((self):visitMacroAssignment_(aNode)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((aNode):left())
	assignee=_s8_value; return _s8_value
end)()
;
(function() if ((((self):acceptAssignment_to_((aNode):right(), assignee)))==true) then return ((function()
(self):visit_(assignee);(self['@stream']):nextPutAll_("=");return ((self):visit_((aNode):right()))
end))() else return ((function()
return ((self):compilerEvent_with_ifNotHandled_("invalidAssignment:", aNode, (function()
return ((self):error_(smalltalk:object_comma_(("Invalid assignment to "),(assignee):value())))
end)))
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitAssignmentNode: aNode\r\n\r\n\t| assignee |\r\n\t(self visitMacroAssignment: aNode) ifTrue: [ ^self ].\r\n\tassignee := aNode left.\r\n\t(self acceptAssignment: aNode right to: assignee) ifTrue: [\r\n\t\tself visit: assignee.\r\n\t\tstream nextPutAll: '='.\r\n\t\tself visit: aNode right\r\n\t] ifFalse: [\r\n\t\tself\tcompilerEvent: #invalidAssignment: with: aNode\r\n\t\t\tifNotHandled: [ self error: 'Invalid assignment to ' ,assignee value ]\r\n\t]","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal assignee=nil\r\n(function() if ((((self):visitMacroAssignment_(aNode)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aNode):left())\r\n\tassignee=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((self):acceptAssignment_to_((aNode):right(), assignee)))==true) then return ((function()\r\n(self):visit_(assignee);(self['@stream']):nextPutAll_(\"=\");return ((self):visit_((aNode):right()))\r\nend))() else return ((function()\r\nreturn ((self):compilerEvent_with_ifNotHandled_(\"invalidAssignment:\", aNode, (function()\r\nreturn ((self):error_(smalltalk:object_comma_((\"Invalid assignment to \"),(assignee):value())))\r\nend)))\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitBlockNode:",0
,(
function(self,aNode)
(self['@stream']):nextPutAll_("(")
;
(self):emmitFunction_node_arguments_doing_("", aNode, (aNode):parameterNames(), (function()
return (((aNode):nodes()):do_((function(each)
return ((self):visit_(each))
end)))
end))
;
(self['@stream']):nextPutAll_(")")
;
return (self)
;
end
)
,"visiting","visitBlockNode: aNode\r\n\r\n\tstream nextPutAll: $(.\r\n\tself\temmitFunction: '' node: aNode\r\n\t\targuments: aNode parameterNames\r\n\t\tdoing: [ aNode nodes do: [:each | self visit: each ] ].\r\n\tstream nextPutAll: $).","(\r\nfunction(self,aNode)\r\n(self['@stream']):nextPutAll_(\"(\")\r\n;\r\n(self):emmitFunction_node_arguments_doing_(\"\", aNode, (aNode):parameterNames(), (function()\r\nreturn (((aNode):nodes()):do_((function(each)\r\nreturn ((self):visit_(each))\r\nend)))\r\nend))\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitBlockSequenceNode:",0
,(
function(self,aNode)
local index=nil
(function()
	local _s8_value=((self['@nestedBlocks']):_plus((1)))
	self['@nestedBlocks']=_s8_value; return _s8_value
end)()
;
(function() if (((((aNode):nodes()):isEmpty()))==true) then return ((function()
return ((self):emmitReturn_("nil"))
end))() else return ((function()
((aNode):tempNames()):do_((function(each)
((self):tempVariables()):add_(each);return ((self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(("var "),(self):localName_(each))),"=nil;")))
end));(function()
	local _s8_value=((0))
	index=_s8_value; return _s8_value
end)();return (((aNode):nodes()):do_((function(each)
(function()
	local _s8_value=((index):_plus((1)))
	index=_s8_value; return _s8_value
end)();return ((function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()
return ((self):emmitReturn_((function()
return ((self):visit_(each))
end)))
end))() else return ((function()
(self):visit_(each);return ((self['@stream']):nextPutAll_(";"))
end))() end end)())
end)))
end))() end end)()
;
(function()
	local _s8_value=((self['@nestedBlocks']):_minus((1)))
	self['@nestedBlocks']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"visiting","visitBlockSequenceNode: aNode\r\n\r\n\t| index |\r\n\tnestedBlocks := nestedBlocks + 1.\r\n\taNode nodes isEmpty\r\n\t    ifTrue: [ self emmitReturn: #nil ]\r\n\t    ifFalse: [\r\n\t\taNode tempNames do: [:each |\r\n\t\t    self tempVariables add: each.\r\n\t\t    stream nextPutAll: 'var ', (self localName: each), '=nil;'\r\n\t\t].\r\n\t\tindex := 0.\r\n\t\taNode nodes do: [:each |\r\n\t\t    index := index + 1.\r\n\t\t    index = aNode nodes size\r\n\t\t\tifTrue: [ self emmitReturn: [ self visit: each ] ]\r\n\t\t    \tifFalse: [ self visit: each. stream nextPutAll: $; ]\r\n\t\t].\r\n\t].\r\n\tnestedBlocks := nestedBlocks - 1","(\r\nfunction(self,aNode)\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((self['@nestedBlocks']):_plus((1)))\r\n\tself['@nestedBlocks']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((((aNode):nodes()):isEmpty()))==true) then return ((function()\r\nreturn ((self):emmitReturn_(\"nil\"))\r\nend))() else return ((function()\r\n((aNode):tempNames()):do_((function(each)\r\n((self):tempVariables()):add_(each);return ((self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\"var \"),(self):localName_(each))),\"=nil;\")))\r\nend));(function()\r\n\tlocal _s8_value=((0))\r\n\tindex=_s8_value; return _s8_value\r\nend)();return (((aNode):nodes()):do_((function(each)\r\n(function()\r\n\tlocal _s8_value=((index):_plus((1)))\r\n\tindex=_s8_value; return _s8_value\r\nend)();return ((function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()\r\nreturn ((self):emmitReturn_((function()\r\nreturn ((self):visit_(each))\r\nend)))\r\nend))() else return ((function()\r\n(self):visit_(each);return ((self['@stream']):nextPutAll_(\";\"))\r\nend))() end end)())\r\nend)))\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@nestedBlocks']):_minus((1)))\r\n\tself['@nestedBlocks']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitCascadeNode:",0
,(
function(self,aNode)
(function() if (((((self):tempVariables()):includes_("$rec")))==true) then return nil else return ((function()
return (((self):tempVariables()):add_("$rec"))
end))() end end)()
;
(self):compilerEvent_with_("aboutCascade:", aNode)
;
(self['@stream']):nextPutAll_("(")
;
(self):emmitFunction_arguments_doing_("", "$rec", (function()
local index=nil
local rec=nil
(function()
	local _s8_value=((0))
	index=_s8_value; return _s8_value
end)();return (((aNode):nodes()):do_((function(each)
(function()
	local _s8_value=((index):_plus((1)))
	index=_s8_value; return _s8_value
end)();(function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()
return ((self):emmitReturn_(nil))
end))() else return nil end end)();(function()
	local _s8_value=((smalltalk.VariableNode):new())
	rec=_s8_value; return _s8_value
end)();(
function(_s8_rec)
(_s8_rec):value_("$rec");
return ((_s8_rec):start_end_((each):start(), (each):start()))
end
)(rec);(each):receiver_(rec);(self):visit_(each);(function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()
return ((self):compilerEvent_withArguments_("returned:early:", (smalltalk.Array):with_with_(nil, false)))
end))() else return nil end end)();return ((self['@stream']):nextPutAll_(";"))
end)))
end))
;
(self['@stream']):nextPutAll_(")(")
;
(self):visit_((aNode):receiver())
;
(self['@stream']):nextPutAll_(")")
;
(self):compilerEvent_with_("cascade:", aNode)
;
return (self)
;
end
)
,"visiting","visitCascadeNode: aNode\r\n\r\n\t(self tempVariables includes: '$rec') ifFalse: [ self tempVariables add: '$rec' ].\r\n\tself compilerEvent: #aboutCascade: with: aNode.\r\n\tstream nextPutAll: $(.\r\n\tself emmitFunction: '' arguments: '$rec' doing: [ | index rec |\r\n\t\tindex := 0.\r\n\t\taNode nodes do: [:each |\r\n\t\t    index := index + 1.\r\n\t\t    index = aNode nodes size ifTrue: [ self emmitReturn: nil ].\r\n\t\t    rec := VariableNode new.\r\n\t\t    rec value: '$rec'; start: each start end: each start.\r\n\t\t    each receiver: rec.\r\n\t\t    self visit: each.\r\n\t\t    index = aNode nodes size ifTrue: [\r\n\t\t\tself compilerEvent: #returned:early: withArguments: (Array with: nil with: false)\r\n\t\t    ].\r\n\t\t    stream nextPutAll: ';'\r\n\t\t].\r\n\t].\r\n\tstream nextPutAll: ')('.\r\n\tself visit: aNode receiver.\r\n\tstream nextPutAll: $).\r\n\tself compilerEvent: #cascade: with: aNode.","(\r\nfunction(self,aNode)\r\n(function() if (((((self):tempVariables()):includes_(\"$rec\")))==true) then return nil else return ((function()\r\nreturn (((self):tempVariables()):add_(\"$rec\"))\r\nend))() end end)()\r\n;\r\n(self):compilerEvent_with_(\"aboutCascade:\", aNode)\r\n;\r\n(self['@stream']):nextPutAll_(\"(\")\r\n;\r\n(self):emmitFunction_arguments_doing_(\"\", \"$rec\", (function()\r\nlocal index=nil\r\nlocal rec=nil\r\n(function()\r\n\tlocal _s8_value=((0))\r\n\tindex=_s8_value; return _s8_value\r\nend)();return (((aNode):nodes()):do_((function(each)\r\n(function()\r\n\tlocal _s8_value=((index):_plus((1)))\r\n\tindex=_s8_value; return _s8_value\r\nend)();(function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()\r\nreturn ((self):emmitReturn_(nil))\r\nend))() else return nil end end)();(function()\r\n\tlocal _s8_value=((smalltalk.VariableNode):new())\r\n\trec=_s8_value; return _s8_value\r\nend)();(\r\nfunction(_s8_rec)\r\n(_s8_rec):value_(\"$rec\");\r\nreturn ((_s8_rec):start_end_((each):start(), (each):start()))\r\nend\r\n)(rec);(each):receiver_(rec);(self):visit_(each);(function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()\r\nreturn ((self):compilerEvent_withArguments_(\"returned:early:\", (smalltalk.Array):with_with_(nil, false)))\r\nend))() else return nil end end)();return ((self['@stream']):nextPutAll_(\";\"))\r\nend)))\r\nend))\r\n;\r\n(self['@stream']):nextPutAll_(\")(\")\r\n;\r\n(self):visit_((aNode):receiver())\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\n(self):compilerEvent_with_(\"cascade:\", aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitClassVariableAssignment:in:to:",0
,(
function(self,aClassVariableName,aClass,anExpressionNode)
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
(_s8_rec):nextPutAll_(".classVariableAt_put_(\"");
(_s8_rec):nextPutAll_(aClassVariableName);
return ((_s8_rec):nextPutAll_("\","))
end
)(self['@stream'])
;
(self):visit_(anExpressionNode)
;
(self['@stream']):nextPutAll_(")")
;
return (self)
;
end
)
,"visiting","visitClassVariableAssignment: aClassVariableName in: aClass to: anExpressionNode\r\n\r\n\tstream\r\n\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\tnextPutAll: '.classVariableAt_put_(\"';\r\n\t\tnextPutAll: aClassVariableName; nextPutAll: '\",'.\r\n\tself visit: anExpressionNode.\r\n\tstream\tnextPutAll: ')'","(\r\nfunction(self,aClassVariableName,aClass,anExpressionNode)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\n(_s8_rec):nextPutAll_(\".classVariableAt_put_(\\\"\");\r\n(_s8_rec):nextPutAll_(aClassVariableName);\r\nreturn ((_s8_rec):nextPutAll_(\"\\\",\"))\r\nend\r\n)(self['@stream'])\r\n;\r\n(self):visit_(anExpressionNode)\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitExtendedSequenceNode:",0
,(
function(self,aNode)
((aNode):directives()):do_((function(each)
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_((each):asNative_for_on_(self['@functionName'], self, self['@stream']));
return ((_s8_rec):cr())
end
)(self['@stream']))
end))
;
return ((self):visitSequenceNode_(aNode))
;
end
)
,"visiting","visitExtendedSequenceNode: aNode\r\n\t\" Private - Visit a directive node. \"\r\n\r\n\taNode directives do: [:each|\r\n\t\tstream nextPutAll: (each\r\n\t\t\tasNative: functionName\r\n\t\t\tfor: self on: stream); cr.\r\n\t].\r\n\t^self visitSequenceNode: aNode","(\r\nfunction(self,aNode)\r\n((aNode):directives()):do_((function(each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((each):asNative_for_on_(self['@functionName'], self, self['@stream']));\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(self['@stream']))\r\nend))\r\n;\r\nreturn ((self):visitSequenceNode_(aNode))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitGlobalReferenceNode:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local aName=nil
local aClass=nil
local lookupPools=nil
(function()
	local _s8_value=((aNode):value())
	aName=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((self):currentClass()):instanceClass())
	aClass=_s8_value; return _s8_value
end)()
;
(aClass):classVariableNamesDo_((function(each)
return ((function() if ((((smalltalk:isObject_equalTo_((each),aName))))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
(_s8_rec):nextPutAll_(".classVariableAt_(\"");
(_s8_rec):nextPutAll_(aName);
return ((_s8_rec):nextPutAll_("\")"))
end
)(self['@stream']);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(aClass):sharedPoolsAndNamesDo_((function(poolName,pool)
return ((function() if ((((pool):includesKey_(aName)))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_("smalltalk.poolDictionary_(\"");
(_s8_rec):nextPutAll_(poolName);
(_s8_rec):nextPutAll_("\").at_(\"");
(_s8_rec):nextPutAll_(aName);
return ((_s8_rec):nextPutAll_("\")"))
end
)(self['@stream']);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_("smalltalk.");
return ((_s8_rec):nextPutAll_((aNode):value()))
end
)(self['@stream'])
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitGlobalReferenceNode: aNode\r\n\r\n\t| aName aClass lookupPools |\r\n\taName := aNode value.\r\n\taClass := self currentClass instanceClass.\r\n\taClass classVariableNamesDo: [:each|\r\n\t\teach = aName ifTrue: [\r\n\t\t\tstream\r\n\t\t\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\t\t\tnextPutAll: '.classVariableAt_(\"';\r\n\t\t\t\tnextPutAll: aName;\r\n\t\t\t\tnextPutAll: '\")'.\r\n\t\t\t^self\r\n\t\t]\r\n\t].\r\n\taClass sharedPoolsAndNamesDo: [:poolName :pool|\r\n\t\t(pool includesKey: aName) ifTrue: [\r\n\t\t\tstream\r\n\t\t\t\tnextPutAll: 'smalltalk.poolDictionary_(\"';\r\n\t\t\t\tnextPutAll: poolName;\r\n\t\t\t\tnextPutAll: '\").at_(\"';\r\n\t\t\t\tnextPutAll: aName;\r\n\t\t\t\tnextPutAll: '\")'.\r\n\t\t\t^self\r\n\t\t].\r\n\t].\r\n\tstream \r\n\t\tnextPutAll: 'smalltalk.';\r\n\t\tnextPutAll: aNode value.","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal aName=nil\r\nlocal aClass=nil\r\nlocal lookupPools=nil\r\n(function()\r\n\tlocal _s8_value=((aNode):value())\r\n\taName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((self):currentClass()):instanceClass())\r\n\taClass=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aClass):classVariableNamesDo_((function(each)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((each),aName))))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\n(_s8_rec):nextPutAll_(\".classVariableAt_(\\\"\");\r\n(_s8_rec):nextPutAll_(aName);\r\nreturn ((_s8_rec):nextPutAll_(\"\\\")\"))\r\nend\r\n)(self['@stream']);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(aClass):sharedPoolsAndNamesDo_((function(poolName,pool)\r\nreturn ((function() if ((((pool):includesKey_(aName)))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\"smalltalk.poolDictionary_(\\\"\");\r\n(_s8_rec):nextPutAll_(poolName);\r\n(_s8_rec):nextPutAll_(\"\\\").at_(\\\"\");\r\n(_s8_rec):nextPutAll_(aName);\r\nreturn ((_s8_rec):nextPutAll_(\"\\\")\"))\r\nend\r\n)(self['@stream']);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\"smalltalk.\");\r\nreturn ((_s8_rec):nextPutAll_((aNode):value()))\r\nend\r\n)(self['@stream'])\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitLocalAssignment:",0
,(
function(self,aNode)
(self['@stream']):nextPutAll_("(")
;
(self):visit_(aNode)
;
(self['@stream']):nextPutAll_(")")
;
return (self)
;
end
)
,"visiting","visitLocalAssignment: aNode\r\n\t\" Private - Visit a node that is an assignment in a local expression. \"\r\n\r\n\tstream nextPutAll: $(.\r\n\tself visit: aNode.\r\n\tstream nextPutAll: $).","(\r\nfunction(self,aNode)\r\n(self['@stream']):nextPutAll_(\"(\")\r\n;\r\n(self):visit_(aNode)\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitMacroAssignment:",0
,(
function(self,aNode)
return (((aNode):left()):acceptMacroAssignment_with_((aNode):right(), self))
;
end
)
,"visiting","visitMacroAssignment: aNode\r\n\t^aNode left acceptMacroAssignment: aNode right with: self","(\r\nfunction(self,aNode)\r\nreturn (((aNode):left()):acceptMacroAssignment_with_((aNode):right(), self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitMacroAssignment:to:",0
,(
function(self,aVariableNode,anExpressionNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local aName=nil
local aClass=nil
(function() if (((((aVariableNode):indices()):notEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((aVariableNode):value())
	aName=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((self):currentClass()):instanceClass())
	aClass=_s8_value; return _s8_value
end)()
;
(aClass):classVariableNamesDo_((function(each)
return ((function() if ((((smalltalk:isObject_equalTo_((each),aName))))==true) then return ((function()
(self):visitClassVariableAssignment_in_to_(each, aClass, anExpressionNode);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(aClass):sharedPoolsAndNamesDo_((function(poolName,pool)
return ((function() if ((((pool):includesKey_(aName)))==true) then return ((function()
(self):visitPoolVariableAssignment_in_to_(aName, poolName, anExpressionNode);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitMacroAssignment: aVariableNode to: anExpressionNode\r\n\t\" Return true if the receiver generated code to a macro assignment to aVariableNode. \"\r\n\r\n\t| aName aClass |\r\n\taVariableNode indices notEmpty ifTrue: [ ^false ].\r\n\taName := aVariableNode value.\r\n\taClass := self currentClass instanceClass.\r\n\taClass classVariableNamesDo: [:each|\r\n\t\teach = aName ifTrue: [\r\n\t\t\tself visitClassVariableAssignment: each\r\n\t\t\t\tin: aClass\r\n\t\t\t\tto: anExpressionNode.\r\n\t\t\t^true\r\n\t\t]\r\n\t].\r\n\taClass sharedPoolsAndNamesDo: [:poolName :pool|\r\n\t\t(pool includesKey: aName) ifTrue: [\r\n\t\t\tself visitPoolVariableAssignment: aName\r\n\t\t\t\tin: poolName\r\n\t\t\t\tto: anExpressionNode.\r\n\t\t\t^true\r\n\t\t].\r\n\t].\r\n\t^false","(\r\nfunction(self,aVariableNode,anExpressionNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal aName=nil\r\nlocal aClass=nil\r\n(function() if (((((aVariableNode):indices()):notEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aVariableNode):value())\r\n\taName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((self):currentClass()):instanceClass())\r\n\taClass=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aClass):classVariableNamesDo_((function(each)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((each),aName))))==true) then return ((function()\r\n(self):visitClassVariableAssignment_in_to_(each, aClass, anExpressionNode);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(aClass):sharedPoolsAndNamesDo_((function(poolName,pool)\r\nreturn ((function() if ((((pool):includesKey_(aName)))==true) then return ((function()\r\n(self):visitPoolVariableAssignment_in_to_(aName, poolName, anExpressionNode);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitMethodNode:",0
,(
function(self,aMethodNode)
(
function(_s8_rec)
(_s8_rec):nextPutAll_(smalltalk:object_comma_(((self):newMethodFunctionName()),"("));
(_s8_rec):nextPutAll_((self):smalltalkNameFor_((self):currentClass()));
(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((",\""),(aMethodNode):selector())),"\","));
return ((_s8_rec):cr())
end
)(self['@stream'])
;
(self):emmitMethod_(aMethodNode)
;
(function() if ((((self):includeSourceCode()))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):cr();
return ((_s8_rec):nextPutAll_(smalltalk:object_comma_((","),(self):escaped_((aMethodNode):source()))))
end
)(self['@stream']))
end))() else return nil end end)()
;
(self['@stream']):nextPutAll_(")")
;
return (self)
;
end
)
,"visiting","visitMethodNode: aMethodNode\r\n\t\" Visit a method node. \"\r\n\r\n\tstream\r\n\t\tnextPutAll: self newMethodFunctionName, '(';\r\n\t\tnextPutAll: (self smalltalkNameFor: self currentClass);\r\n\t\tnextPutAll: ',\"', aMethodNode selector, '\",'; cr.\r\n\tself emmitMethod: aMethodNode.\r\n\tself includeSourceCode ifTrue: [\r\n\t\tstream\tcr; \"warning: this cr is important for debugger\"\r\n\t\t\tnextPutAll: ',' ,(self escaped: aMethodNode source)\r\n\t].\r\n\tstream nextPutAll: ')'","(\r\nfunction(self,aMethodNode)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_(((self):newMethodFunctionName()),\"(\"));\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_((self):currentClass()));\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\",\\\"\"),(aMethodNode):selector())),\"\\\",\"));\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(self['@stream'])\r\n;\r\n(self):emmitMethod_(aMethodNode)\r\n;\r\n(function() if ((((self):includeSourceCode()))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\nreturn ((_s8_rec):nextPutAll_(smalltalk:object_comma_((\",\"),(self):escaped_((aMethodNode):source()))))\r\nend\r\n)(self['@stream']))\r\nend))() else return nil end end)()\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitNode:index:",0
,(
function(self,aNode,index)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local selector=nil
(function() if ((((smalltalk:boolean_and_(((index):isMemberOf_(smalltalk.ValueNode)),(function()
return ((index):hasString())
end)))))==true) then return ((function()
(function()
	local _s8_value=(((index):value()):asSelector())
	selector=_s8_value; return _s8_value
end)();return ((function() if ((((smalltalk:boolean_or_(((selector):includes_(" ")),(function()
return ((selector):includes_("_"))
end)))))==true) then return nil else return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_(".");
return ((_s8_rec):nextPutAll_(selector))
end
)(self['@stream']);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)())
end))() else return nil end end)()
;
(self['@stream']):nextPutAll_("[")
;
(self):visit_(index)
;
(self['@stream']):nextPutAll_("]")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitNode: aNode index: index\r\n\t\" Private - Generate code for direct property access. \"\r\n\r\n\t| selector |\r\n\t((index isMemberOf: ValueNode) and: [ index hasString ]) ifTrue: [\r\n\t\tselector := index value asSelector.\r\n\t\t((selector includes: $ ) or: [ selector includes: $_ ]) ifFalse: [\r\n\t\t\tstream nextPutAll: '.'; nextPutAll: selector.\r\n\t\t\t^self\r\n\t\t].\r\n\t].\r\n\tstream nextPutAll: '['.\r\n\tself visit: index.\r\n\tstream nextPutAll: ']'.","(\r\nfunction(self,aNode,index)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal selector=nil\r\n(function() if ((((smalltalk:boolean_and_(((index):isMemberOf_(smalltalk.ValueNode)),(function()\r\nreturn ((index):hasString())\r\nend)))))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=(((index):value()):asSelector())\r\n\tselector=_s8_value; return _s8_value\r\nend)();return ((function() if ((((smalltalk:boolean_or_(((selector):includes_(\" \")),(function()\r\nreturn ((selector):includes_(\"_\"))\r\nend)))))==true) then return nil else return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\".\");\r\nreturn ((_s8_rec):nextPutAll_(selector))\r\nend\r\n)(self['@stream']);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)())\r\nend))() else return nil end end)()\r\n;\r\n(self['@stream']):nextPutAll_(\"[\")\r\n;\r\n(self):visit_(index)\r\n;\r\n(self['@stream']):nextPutAll_(\"]\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitNodeIndices:",0
,(
function(self,aNode)
((aNode):indices()):do_((function(each)
return ((self):visitNode_index_(aNode, each))
end))
;
return (self)
;
end
)
,"visiting","visitNodeIndices: aNode\r\n\t\" Private - Generate code for direct property access. \"\r\n\r\n\taNode indices do: [:each| self visitNode: aNode index: each ].","(\r\nfunction(self,aNode)\r\n((aNode):indices()):do_((function(each)\r\nreturn ((self):visitNode_index_(aNode, each))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitNormalSend:",0
,(
function(self,aNode)
(self):compilerEvent_with_("aboutSend:", aNode)
;
(self):visitReceiver_((aNode):receiver())
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_(((aNode):receiver()):messageSendConnector());
(_s8_rec):nextPutAll_(((aNode):selector()):asSelector());
return ((_s8_rec):nextPutAll_("("))
end
)(self['@stream'])
;
(self):visitSendArguments_(aNode)
;
(self['@stream']):nextPutAll_(")")
;
(self):compilerEvent_with_("send:", aNode)
;
return (self)
;
end
)
,"visiting","visitNormalSend: aNode\r\n\t\" Generate code for direct/inlined message send.\"\r\n\r\n\tself compilerEvent: #aboutSend: with: aNode.\r\n\tself visitReceiver: aNode receiver.\r\n\tstream\r\n\t\tnextPutAll: aNode receiver messageSendConnector;\r\n\t\tnextPutAll: aNode selector asSelector;\r\n\t\tnextPutAll: '('.\r\n\tself visitSendArguments: aNode.\r\n\tstream\tnextPutAll: ')'.\r\n\tself compilerEvent: #send: with: aNode.","(\r\nfunction(self,aNode)\r\n(self):compilerEvent_with_(\"aboutSend:\", aNode)\r\n;\r\n(self):visitReceiver_((aNode):receiver())\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(((aNode):receiver()):messageSendConnector());\r\n(_s8_rec):nextPutAll_(((aNode):selector()):asSelector());\r\nreturn ((_s8_rec):nextPutAll_(\"(\"))\r\nend\r\n)(self['@stream'])\r\n;\r\n(self):visitSendArguments_(aNode)\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\n(self):compilerEvent_with_(\"send:\", aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitOperation:",0
,(
function(self,aNode)
(self):compilerEvent_with_("aboutOperation:", aNode)
;
(self['@stream']):nextPutAll_("(")
;
(self):visitReceiver_((aNode):receiver())
;
(self['@stream']):nextPutAll_(")")
;
(self['@stream']):nextPutAll_(((aNode):selector()):asSelector())
;
(self):visitSendArguments_(aNode)
;
(self):compilerEvent_with_("operation:", aNode)
;
return (self)
;
end
)
,"visiting","visitOperation: aNode\r\n\t\" Private - aNode is an operation and selector is mapped to target operation. \"\r\n\r\n\tself compilerEvent: #aboutOperation: with: aNode.\r\n\tstream nextPutAll: $(.\r\n\tself visitReceiver: aNode receiver.\r\n\tstream nextPutAll: $).\r\n\tstream nextPutAll: aNode selector asSelector.\r\n\tself visitSendArguments: aNode.\r\n\tself compilerEvent: #operation: with: aNode.","(\r\nfunction(self,aNode)\r\n(self):compilerEvent_with_(\"aboutOperation:\", aNode)\r\n;\r\n(self['@stream']):nextPutAll_(\"(\")\r\n;\r\n(self):visitReceiver_((aNode):receiver())\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\n(self['@stream']):nextPutAll_(((aNode):selector()):asSelector())\r\n;\r\n(self):visitSendArguments_(aNode)\r\n;\r\n(self):compilerEvent_with_(\"operation:\", aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitPoolVariableAssignment:in:to:",0
,(
function(self,aName,poolName,anExpressionNode)
(
function(_s8_rec)
(_s8_rec):nextPutAll_("smalltalk.poolDictionary_(\"");
(_s8_rec):nextPutAll_(poolName);
(_s8_rec):nextPutAll_("\").at_put_(\"");
(_s8_rec):nextPutAll_(aName);
return ((_s8_rec):nextPutAll_("\","))
end
)(self['@stream'])
;
(self):visit_(anExpressionNode)
;
(self['@stream']):nextPutAll_(")")
;
return (self)
;
end
)
,"visiting","visitPoolVariableAssignment: aName in: poolName to: anExpressionNode\r\n\r\n\tstream\r\n\t\tnextPutAll: 'smalltalk.poolDictionary_(\"';\r\n\t\tnextPutAll: poolName; nextPutAll: '\").at_put_(\"';\r\n\t\tnextPutAll: aName; nextPutAll: '\",'.\r\n\tself visit: anExpressionNode.\r\n\tstream\tnextPutAll: ')'","(\r\nfunction(self,aName,poolName,anExpressionNode)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\"smalltalk.poolDictionary_(\\\"\");\r\n(_s8_rec):nextPutAll_(poolName);\r\n(_s8_rec):nextPutAll_(\"\\\").at_put_(\\\"\");\r\n(_s8_rec):nextPutAll_(aName);\r\nreturn ((_s8_rec):nextPutAll_(\"\\\",\"))\r\nend\r\n)(self['@stream'])\r\n;\r\n(self):visit_(anExpressionNode)\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitReceiver:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aNode):isAssignment()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitLocalAssignment_(aNode))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visit_(aNode))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitReceiver: aNode\r\n\t\" Private - Visit the node (as receiver) emmitting output. \"\r\n\r\n\taNode isAssignment ifTrue: [ ^self visitLocalAssignment: aNode. ].\r\n\t^self visit: aNode","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aNode):isAssignment()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitLocalAssignment_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visit_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitReturnNode:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local theReturn=nil
(function() if (((((self['@nestedBlocks']) > (0))))==true) then return ((function()
return ((function()
	local _s8_value=(true)
	self['@earlyReturn']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((function()
return ((self):emmitReturn_((function()
return (((aNode):nodes()):do_((function(each)
return ((self):visit_(each))
end)))
end)))
end))
	theReturn=_s8_value; return _s8_value
end)()
;
(function() if (((self['@earlyReturn']))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((theReturn):value())
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(self['@stream']):nextPutAll_("(")
;
(self):emmitFunctionDoing_((function()
(self['@stream']):nextPutAll_("$s8Ret$.result=(");(self):emmitFunctionDoing_(theReturn);return ((self['@stream']):nextPutAll_(")();throw($s8Ret$)"))
end))
;
(self['@stream']):nextPutAll_(")()")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitReturnNode: aNode\r\n\r\n\t| theReturn |\r\n\tnestedBlocks > 0 ifTrue: [ earlyReturn := true ].\r\n\ttheReturn := [\r\n\t\tself emmitReturn: [\r\n\t\t\taNode nodes do: [:each | self visit: each ]\r\n\t\t]\r\n\t].\r\n\tearlyReturn ifFalse: [ ^theReturn value ].\r\n\tstream nextPutAll: $(.\r\n\tself emmitFunctionDoing: [\r\n\t\tstream nextPutAll: '$s8Ret$.result=('.\r\n\t\tself emmitFunctionDoing: theReturn.\r\n\t\tstream nextPutAll: ')();throw($s8Ret$)'\r\n\t].\r\n\tstream nextPutAll: ')()'.","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal theReturn=nil\r\n(function() if (((((self['@nestedBlocks']) > (0))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(true)\r\n\tself['@earlyReturn']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function()\r\nreturn ((self):emmitReturn_((function()\r\nreturn (((aNode):nodes()):do_((function(each)\r\nreturn ((self):visit_(each))\r\nend)))\r\nend)))\r\nend))\r\n\ttheReturn=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((self['@earlyReturn']))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((theReturn):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(self['@stream']):nextPutAll_(\"(\")\r\n;\r\n(self):emmitFunctionDoing_((function()\r\n(self['@stream']):nextPutAll_(\"$s8Ret$.result=(\");(self):emmitFunctionDoing_(theReturn);return ((self['@stream']):nextPutAll_(\")();throw($s8Ret$)\"))\r\nend))\r\n;\r\n(self['@stream']):nextPutAll_(\")()\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitSendArguments:",0
,(
function(self,aSendNode)
((aSendNode):arguments()):do_separatedBy_((function(each)
return ((self):visitArgument_(each))
end), (function()
return ((self['@stream']):nextPutAll_(", "))
end))
;
return (self)
;
end
)
,"visiting","visitSendArguments: aSendNode\r\n\r\n\taSendNode arguments \r\n\t    do: [:each | self visitArgument: each ]\r\n\t    separatedBy: [ stream nextPutAll: ', ' ].","(\r\nfunction(self,aSendNode)\r\n((aSendNode):arguments()):do_separatedBy_((function(each)\r\nreturn ((self):visitArgument_(each))\r\nend), (function()\r\nreturn ((self['@stream']):nextPutAll_(\", \"))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitSendNode:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((aNode):receiver()):isSuper()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitSendToSuper_(aNode))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((self):visitOptimizedSend_(aNode)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((self):isOperation_(aNode)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitOperation_(aNode))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitNormalSend_(aNode))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitSendNode: aNode\r\n\t\" Private - Visit the node (message send node) emmitting output. \"\r\n\r\n\taNode receiver isSuper ifTrue: [ ^self visitSendToSuper: aNode ].\r\n\t(self visitOptimizedSend: aNode) ifTrue: [ ^self ].\r\n\t(self isOperation: aNode) ifTrue: [ ^self visitOperation: aNode ].\r\n\t^self visitNormalSend: aNode","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((aNode):receiver()):isSuper()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitSendToSuper_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((self):visitOptimizedSend_(aNode)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((self):isOperation_(aNode)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitOperation_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitNormalSend_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitSendToSuper:",0
,(
function(self,aNode)
(self):compilerEvent_with_("aboutSend:", aNode)
;
(self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk.superImplementor("),(self):smalltalkNameFor_((self):currentClass()))),",\"")),((aNode):selector()):asSelector())),"\").apply(self, ["))
;
(self):visitSendArguments_(aNode)
;
(self['@stream']):nextPutAll_("])")
;
(self):compilerEvent_with_("send:", aNode)
;
return (self)
;
end
)
,"visiting","visitSendToSuper: aNode\r\n\tself compilerEvent: #aboutSend: with: aNode.\r\n\tstream \r\n\t\tnextPutAll: 'smalltalk.superImplementor('\r\n\t\t\t,(self smalltalkNameFor: self currentClass)\r\n\t\t\t,',\"',aNode selector asSelector\r\n\t\t\t,'\").apply(self, ['.\r\n\tself visitSendArguments: aNode.\r\n\tstream nextPutAll: '])'.\r\n\tself compilerEvent: #send: with: aNode.","(\r\nfunction(self,aNode)\r\n(self):compilerEvent_with_(\"aboutSend:\", aNode)\r\n;\r\n(self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.superImplementor(\"),(self):smalltalkNameFor_((self):currentClass()))),\",\\\"\")),((aNode):selector()):asSelector())),\"\\\").apply(self, [\"))\r\n;\r\n(self):visitSendArguments_(aNode)\r\n;\r\n(self['@stream']):nextPutAll_(\"])\")\r\n;\r\n(self):compilerEvent_with_(\"send:\", aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitSequenceNode:",0
,(
function(self,aNode)
((aNode):tempNames()):do_((function(each)
((self):tempVariables()):add_(each);return ((self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(("var "),(self):localName_(each))),"=nil;")))
end))
;
((aNode):nodes()):do_separatedBy_((function(each)
(self):visit_(each);return ((self['@stream']):nextPutAll_(";"))
end), (function()
return ((self['@stream']):cr())
end))
;
return (self)
;
end
)
,"visiting","visitSequenceNode: aNode\r\n\t\" Private - Visit the node sequence emmitting output. \"\r\n\r\n\taNode tempNames do: [:each |\r\n\t\tself tempVariables add: each.\r\n\t\tstream nextPutAll: 'var ', (self localName: each), '=nil;'\r\n\t].\r\n\taNode nodes do: [:each |\r\n\t\tself visit: each.\r\n\t\tstream nextPutAll: ';'\r\n\t] separatedBy: [stream cr]","(\r\nfunction(self,aNode)\r\n((aNode):tempNames()):do_((function(each)\r\n((self):tempVariables()):add_(each);return ((self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\"var \"),(self):localName_(each))),\"=nil;\")))\r\nend))\r\n;\r\n((aNode):nodes()):do_separatedBy_((function(each)\r\n(self):visit_(each);return ((self['@stream']):nextPutAll_(\";\"))\r\nend), (function()\r\nreturn ((self['@stream']):cr())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitValueNode:",0
,(
function(self,aNode)
(self['@stream']):nextPutAll_(((aNode):value()):asNative_((self):nativeLanguage()))
;
return (self)
;
end
)
,"visiting","visitValueNode: aNode\r\n\t\" Private - Visit the node emmitting output. \"\r\n\r\n\tstream nextPutAll: (aNode value asNative: self nativeLanguage)","(\r\nfunction(self,aNode)\r\n(self['@stream']):nextPutAll_(((aNode):value()):asNative_((self):nativeLanguage()))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSEncoder,"visitVariableNode:",0
,(
function(self,aNode)
local code=nil
(function()
	local _s8_value=(((self):currentClass()):codeFor_accessing_(self, aNode))
	code=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:boolean_and_(((((code))==nil)),(function()
return ((aNode):needsVerification())
end)))))==true) then return ((function()
return ((function() if (((((self):knownVariables()):includes_((aNode):value())))==true) then return nil else return ((function()
return ((self):compilerEvent_with_ifNotHandled_("unknownVariable:", aNode, (function()
((self):unknownVariables()):add_((aNode):value());return (((self)["$klass"]):unknownVariable_in_(aNode, self))
end)))
end))() end end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:boolean_or_(((((code))==nil)),(function()
return (((self):knownVariables()):includes_((aNode):value()))
end)))))==true) then return ((function()
return ((function()
	local _s8_value=((aNode):localNameIn_(self))
	code=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(self['@stream']):nextPutAll_(code)
;
(self):visitNodeIndices_(aNode)
;
return (self)
;
end
)
,"visiting","visitVariableNode: aNode\r\n\t\" Generate code to access variable aNode. \"\r\n\r\n\t| code |\r\n\tcode := self currentClass codeFor: self accessing: aNode.\r\n\t(code isNil and: [aNode needsVerification]) ifTrue: [\r\n\t\t(self knownVariables includes: aNode value) ifFalse: [\r\n\t\t\tself\tcompilerEvent: #unknownVariable: with: aNode\r\n\t\t\t\tifNotHandled: [\r\n\t\t\t\t\tself unknownVariables add: aNode value.\r\n\t\t\t\t\tself class unknownVariable: aNode in: self.\r\n\t\t\t\t].\r\n\t\t].\r\n\t].\r\n\t(code isNil or: [self knownVariables includes: aNode value]) ifTrue: [\r\n\t\tcode := aNode localNameIn: self\r\n\t].\r\n\r\n\tstream nextPutAll: code.\r\n\tself visitNodeIndices: aNode.","(\r\nfunction(self,aNode)\r\nlocal code=nil\r\n(function()\r\n\tlocal _s8_value=(((self):currentClass()):codeFor_accessing_(self, aNode))\r\n\tcode=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((((code))==nil)),(function()\r\nreturn ((aNode):needsVerification())\r\nend)))))==true) then return ((function()\r\nreturn ((function() if (((((self):knownVariables()):includes_((aNode):value())))==true) then return nil else return ((function()\r\nreturn ((self):compilerEvent_with_ifNotHandled_(\"unknownVariable:\", aNode, (function()\r\n((self):unknownVariables()):add_((aNode):value());return (((self)[\"$klass\"]):unknownVariable_in_(aNode, self))\r\nend)))\r\nend))() end end)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:boolean_or_(((((code))==nil)),(function()\r\nreturn (((self):knownVariables()):includes_((aNode):value()))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((aNode):localNameIn_(self))\r\n\tcode=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self['@stream']):nextPutAll_(code)\r\n;\r\n(self):visitNodeIndices_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder["$klass"],"supportsLanguage:",0
,(
function(self,aLanguage)
return ((smalltalk:isObject_equalTo_((aLanguage),"lua")))
;
end
)
,"instantiation","supportsLanguage: aLanguage\r\n\t\" Private - Return true if instances of the receiver can encode to aLanguage. \"\r\n\r\n\t^aLanguage = #lua","(\r\nfunction(self,aLanguage)\r\nreturn ((smalltalk:isObject_equalTo_((aLanguage),\"lua\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder["$klass"],"smalltalkNameFor:",0
,(
function(self,aClass)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((aClass))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("nil")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_(("smalltalk."),(function() if ((((aClass):isMetaclass()))==true) then return ((function()
return (smalltalk:object_comma_((((aClass):instanceClass()):name()),"[\"$klass\"]"))
end))() else return ((function()
return ((aClass):name())
end))() end end)()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","smalltalkNameFor: aClass\r\n\t\" Private - Return the global name of aClass (can be nil). \"\r\n\r\n\taClass isNil ifTrue: [ ^#nil ].\r\n\t^'smalltalk.', (aClass isMetaclass\r\n\t    ifTrue: [ aClass instanceClass name, '[\"$klass\"]' ]\r\n\t    ifFalse: [ aClass name ])","(\r\nfunction(self,aClass)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((aClass))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"nil\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((\"smalltalk.\"),(function() if ((((aClass):isMetaclass()))==true) then return ((function()\r\nreturn (smalltalk:object_comma_((((aClass):instanceClass()):name()),\"[\\\"$klass\\\"]\"))\r\nend))() else return ((function()\r\nreturn ((aClass):name())\r\nend))() end end)()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder["$klass"],"unknownVariable:in:",0
,(
function(self,aNode,anEncoder)
(self):note_(smalltalk:object_comma_((">>> Unknown variable: "),smalltalk:asString_(((aNode):value()))))
;
return ((self):error_(smalltalk:object_comma_(("Missing reference "),(aNode):value())))
;
end
)
,"variables","unknownVariable: aNode in: anEncoder\r\n\t\" Private - anEncoder has found an unknown name reference.\r\n\tThe default action will thrown an error (stop compilation),\r\n\t this method can be refined/changed to silently ignore the\r\n\t unknown to let the compiler produce code to refer code\r\n\t in outer native contexts.\r\n\t\"\r\n\r\nself note: '>>> Unknown variable: ' ,aNode value asString.\r\n\t^self error: 'Missing reference ',aNode value","(\r\nfunction(self,aNode,anEncoder)\r\n(self):note_(smalltalk:object_comma_((\">>> Unknown variable: \"),smalltalk:asString_(((aNode):value()))))\r\n;\r\nreturn ((self):error_(smalltalk:object_comma_((\"Missing reference \"),(aNode):value())))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"missingNativeCodeFor:as:",0
,(
function(self,aNode,lang)
return ("(self:invalidMessage())")
;
end
)
,"inline","missingNativeCodeFor: aNode as: lang\r\n\t\" Private - Returns the native language code for missing expression (or nil). \"\r\n\r\n\"\tself note: '>>> Missing native code for language ' ,lang.\r\n\t^nil\r\n\"\t^'(self:invalidMessage())'","(\r\nfunction(self,aNode,lang)\r\nreturn (\"(self:invalidMessage())\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"canExportMethod:",0
,(
function(self,aMethod)
return (true)
;
end
)
,"private","canExportMethod: aMethod\r\n\t\" Private - Return true if the receiver can/want to export method aMethod. \"\r\n\r\n\t^true","(\r\nfunction(self,aMethod)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"nativeSourceFor:",0
,(
function(self,aMethod)
return ((function() if ((((self):canExportMethod_(aMethod)))==true) then return ((function()
return ((self):luaSourceFor_(aMethod))
end))() else return nil end end)())
;
end
)
,"private","nativeSourceFor: aMethod\r\n\t\" Private - Return the code of native function implementing aMethod. \"\r\n\r\n\t^(self canExportMethod: aMethod) ifTrue: [\r\n\t\tself luaSourceFor: aMethod\r\n\t]","(\r\nfunction(self,aMethod)\r\nreturn ((function() if ((((self):canExportMethod_(aMethod)))==true) then return ((function()\r\nreturn ((self):luaSourceFor_(aMethod))\r\nend))() else return nil end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"export:method:on:",0
,(
function(self,aClass,aMethod,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local functionName=nil
local selectorName=nil
local bar=nil
local source=nil
(function()
	local _s8_value=((self):nativeSourceFor_(aMethod))
	source=_s8_value; return _s8_value
end)()
;
(function() if ((((((source))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((((aMethod))==nil)))==true) then return ((function()
return ((self):error_("Method is missing"))
end))() else return nil end end)()
;
(function()
	local _s8_value=("\"")
	bar=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((aMethod):selector())
	functionName=_s8_value; return _s8_value
end)()
;
(function() if (((((functionName):first()):isBackSlash()))==true) then return ((function()
(function()
	local _s8_value=((self):escaped_((aMethod):selector()))
	selectorName=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=((self):escaped_((functionName):asLuaSelector()))
	functionName=_s8_value; return _s8_value
end)())
end))() else return ((function()
(function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((bar),(aMethod):selector())),bar))
	selectorName=_s8_value; return _s8_value
end)();return ((function() if ((((smalltalk:isObject_equalTo_(((functionName):asLuaSelector()),((aMethod):selector()):replace_with_(":", "_")))))==true) then return ((function()
return ((function()
	local _s8_value=((0))
	functionName=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((bar),(functionName):asLuaSelector())),bar))
	functionName=_s8_value; return _s8_value
end)())
end))() end end)())
end))() end end)()
;
(aStream):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk:bind("),(self):smalltalkNameFor_(aClass))),",")),selectorName)),",")),functionName)),"\r\n,")),source))
;
(function() if ((((smalltalk:boolean_and_((((((aMethod):source()))~=nil)),(function()
return (((aMethod):source()):notEmpty())
end)))))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):cr();
return ((_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((","),(self):escaped_((aMethod):category()))),",")),(self):escaped_((aMethod):source()))),",")),(self):escaped_(source))))
end
)(aStream))
end))() else return nil end end)()
;
(aStream):nextPutAll_(")\r\n")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"export","export: aClass method: aMethod on: aStream\r\n\t\" Export a method definition of aClass onto aStream. \"\r\n\r\n\t| functionName selectorName bar source |\r\n\tsource := self nativeSourceFor: aMethod.\r\n\tsource isNil ifTrue: [ ^false ].\r\n\taMethod isNil ifTrue: [ self error: 'Method is missing' ].\r\n\tbar := $\".\r\n\tfunctionName := aMethod selector.\r\n\tfunctionName first isBackSlash ifTrue: [\r\n\t\tselectorName := self escaped: aMethod selector.\r\n\t\tfunctionName := self escaped: functionName asLuaSelector.\r\n\t] ifFalse: [\r\n\t\tselectorName := bar ,aMethod selector, bar.\r\n\t\tfunctionName asLuaSelector = (aMethod selector replace: $: with: $_) ifTrue: [\r\n\t\t\tfunctionName := 0\r\n\t\t] ifFalse: [\r\n\t\t\tfunctionName := bar ,functionName asLuaSelector, bar\r\n\t\t].\r\n\t].\r\n\taStream\r\n\t\tnextPutAll: 'smalltalk:bind(', (self smalltalkNameFor: aClass)\r\n\t\t\t,',', selectorName, ',', functionName ,'\r\n,'\t\t\t, source.\r\n\t(aMethod source notNil and: [ aMethod source notEmpty ]) ifTrue: [\r\n\t    aStream cr; \"warning: this cr is important for debugger\"\r\n\t\tnextPutAll: ',', (self escaped: aMethod category)\r\n\t\t\t,',', (self escaped: aMethod source)\r\n\t\t\t,',', (self escaped: source).\r\n\t].\r\n\taStream nextPutAll: ')\r\n'.\t^true","(\r\nfunction(self,aClass,aMethod,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal functionName=nil\r\nlocal selectorName=nil\r\nlocal bar=nil\r\nlocal source=nil\r\n(function()\r\n\tlocal _s8_value=((self):nativeSourceFor_(aMethod))\r\n\tsource=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((source))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((aMethod))==nil)))==true) then return ((function()\r\nreturn ((self):error_(\"Method is missing\"))\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(\"\\\"\")\r\n\tbar=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aMethod):selector())\r\n\tfunctionName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((((functionName):first()):isBackSlash()))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((self):escaped_((aMethod):selector()))\r\n\tselectorName=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=((self):escaped_((functionName):asLuaSelector()))\r\n\tfunctionName=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((bar),(aMethod):selector())),bar))\r\n\tselectorName=_s8_value; return _s8_value\r\nend)();return ((function() if ((((smalltalk:isObject_equalTo_(((functionName):asLuaSelector()),((aMethod):selector()):replace_with_(\":\", \"_\")))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((0))\r\n\tfunctionName=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((bar),(functionName):asLuaSelector())),bar))\r\n\tfunctionName=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)())\r\nend))() end end)()\r\n;\r\n(aStream):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk:bind(\"),(self):smalltalkNameFor_(aClass))),\",\")),selectorName)),\",\")),functionName)),\"\\r\\n,\")),source))\r\n;\r\n(function() if ((((smalltalk:boolean_and_((((((aMethod):source()))~=nil)),(function()\r\nreturn (((aMethod):source()):notEmpty())\r\nend)))))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\nreturn ((_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\",\"),(self):escaped_((aMethod):category()))),\",\")),(self):escaped_((aMethod):source()))),\",\")),(self):escaped_(source))))\r\nend\r\n)(aStream))\r\nend))() else return nil end end)()\r\n;\r\n(aStream):nextPutAll_(\")\\r\\n\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"arrayLiteralStart",0
,(
function(self)
return ("smalltalk:newArray({")
;
end
)
,"arrays","arrayLiteralStart\r\n\t\" Private - Returns the code string to start exporting an array literal. \"\r\n\r\n\t^'smalltalk:newArray({'","(\r\nfunction(self)\r\nreturn (\"smalltalk:newArray({\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"arrayLiteralEnd",0
,(
function(self)
return ("})")
;
end
)
,"arrays","arrayLiteralEnd\r\n\t\" Private - Returns the code string to end exporting an array literal. \"\r\n\r\n\t^'})'","(\r\nfunction(self)\r\nreturn (\"})\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"escaped:",0
,(
function(self,string)
return ((string):asLua())
;
end
)
,"literals","escaped: string\r\n\t\" Private - Returns the code string to export string literal. \"\r\n\r\n\t^string asLua","(\r\nfunction(self,string)\r\nreturn ((string):asLua())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"exportDefinitionOf:on:",0
,(
function(self,aClass,aStream)
((self):loadRequirementsOf_(aClass)):do_((function(each)
return ((function() if ((((each):isString()))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_(each);
return ((_s8_rec):cr())
end
)(aStream))
end))() else return ((function()
return ((aStream):fileOutOn_(aStream))
end))() end end)())
end))
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk:addClass(\""),(aClass):name())),"\","));
(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass):superclass()));
return ((_s8_rec):nextPutAll_(","))
end
)(aStream)
;
(self):exportArrayLiteral_on_newLine_((aClass):instanceVariableNames(), aStream, false)
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((",\""),(aClass):category())),"\""));
(_s8_rec):nextPutAll_(")");
return ((_s8_rec):cr())
end
)(aStream)
;
(function() if ((((smalltalk:boolean_and_((((((aClass):comment()))~=nil)),(function()
return (((aClass):comment()):notEmpty())
end)))))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
(_s8_rec):nextPutAll_(smalltalk:object_comma_(("[\"$comment\"]="),(self):escaped_((aClass):comment())));
return ((_s8_rec):cr())
end
)(aStream))
end))() else return nil end end)()
;
(function() if (((((((aClass):classVariables()))~=nil)))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
(_s8_rec):nextPutAll_(smalltalk:object_comma_(("[\"$classVariableNames\"]="),(self):escaped_((aClass):classVariablesString())));
return ((_s8_rec):cr())
end
)(aStream))
end))() else return nil end end)()
;
(function() if (((((((aClass):poolDictionaries()))~=nil)))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
return ((_s8_rec):nextPutAll_("[\"$poolDictionaries\"]="))
end
)(aStream);return ((self):exportArrayLiteral_on_newLine_((aClass):poolDictionaries(), aStream, true))
end))() else return nil end end)()
;
(function() if (((((aClass):propertyNames()):notEmpty()))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
return ((_s8_rec):nextPutAll_("[\"$propertyNames\"]="))
end
)(aStream);return ((self):exportArrayLiteral_on_newLine_((aClass):propertyNames(), aStream, true))
end))() else return nil end end)()
;
return (self)
;
end
)
,"export","exportDefinitionOf: aClass on: aStream\r\n\t\" Export definition of aClass onto aStream. \"\r\n\r\n\t(self loadRequirementsOf: aClass) do: [:each|\r\n\t\teach isString\r\n\t\tifTrue: [ aStream nextPutAll: each;cr ]\r\n\t\tifFalse: [ aStream fileOutOn: aStream ].\r\n\t].\r\n\taStream \r\n\t    nextPutAll: 'smalltalk:addClass(\"', aClass name, '\",';\r\n\t    nextPutAll: (self smalltalkNameFor: aClass superclass);\r\n\t    nextPutAll: ','.\r\n\tself exportArrayLiteral: aClass instanceVariableNames on: aStream newLine: false.\r\n\taStream\r\n\t    nextPutAll: ',\"' ,aClass category, $\";\r\n\t    nextPutAll: ')'; cr.\r\n\t(aClass comment notNil and: [aClass comment notEmpty]) ifTrue: [\r\n\t    aStream\r\n\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\tnextPutAll: '[\"$comment\"]=' ,(self escaped: aClass comment); cr\r\n\t].\r\n\taClass classVariables notNil ifTrue: [\r\n\t    aStream\r\n\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\tnextPutAll: '[\"$classVariableNames\"]=' ,(self escaped: aClass classVariablesString); cr\r\n\t].\r\n\taClass poolDictionaries notNil ifTrue: [\r\n\t    aStream\r\n\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\tnextPutAll: '[\"$poolDictionaries\"]='.\r\n\t   self exportArrayLiteral: aClass poolDictionaries on: aStream newLine: true.\r\n\t].\r\n\taClass propertyNames notEmpty ifTrue: [\r\n\t    aStream\r\n\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\tnextPutAll: '[\"$propertyNames\"]='.\r\n\t   self exportArrayLiteral: aClass propertyNames on: aStream newLine: true.\r\n\t].","(\r\nfunction(self,aClass,aStream)\r\n((self):loadRequirementsOf_(aClass)):do_((function(each)\r\nreturn ((function() if ((((each):isString()))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(each);\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(aStream))\r\nend))() else return ((function()\r\nreturn ((aStream):fileOutOn_(aStream))\r\nend))() end end)())\r\nend))\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk:addClass(\\\"\"),(aClass):name())),\"\\\",\"));\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass):superclass()));\r\nreturn ((_s8_rec):nextPutAll_(\",\"))\r\nend\r\n)(aStream)\r\n;\r\n(self):exportArrayLiteral_on_newLine_((aClass):instanceVariableNames(), aStream, false)\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\",\\\"\"),(aClass):category())),\"\\\"\"));\r\n(_s8_rec):nextPutAll_(\")\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(aStream)\r\n;\r\n(function() if ((((smalltalk:boolean_and_((((((aClass):comment()))~=nil)),(function()\r\nreturn (((aClass):comment()):notEmpty())\r\nend)))))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((\"[\\\"$comment\\\"]=\"),(self):escaped_((aClass):comment())));\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(aStream))\r\nend))() else return nil end end)()\r\n;\r\n(function() if (((((((aClass):classVariables()))~=nil)))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((\"[\\\"$classVariableNames\\\"]=\"),(self):escaped_((aClass):classVariablesString())));\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(aStream))\r\nend))() else return nil end end)()\r\n;\r\n(function() if (((((((aClass):poolDictionaries()))~=nil)))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\nreturn ((_s8_rec):nextPutAll_(\"[\\\"$poolDictionaries\\\"]=\"))\r\nend\r\n)(aStream);return ((self):exportArrayLiteral_on_newLine_((aClass):poolDictionaries(), aStream, true))\r\nend))() else return nil end end)()\r\n;\r\n(function() if (((((aClass):propertyNames()):notEmpty()))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\nreturn ((_s8_rec):nextPutAll_(\"[\\\"$propertyNames\\\"]=\"))\r\nend\r\n)(aStream);return ((self):exportArrayLiteral_on_newLine_((aClass):propertyNames(), aStream, true))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"exportMetaDefinitionOf:on:",0
,(
function(self,aClass,aStream)
(function() if ((((((aClass)["$klass"]):instanceVariableNames()):notEmpty()))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass)["$klass"]));
return ((_s8_rec):nextPutAll_("[\"$iVarNames\"]="))
end
)(aStream);return ((self):exportArrayLiteral_on_newLine_(((aClass)["$klass"]):instanceVariableNames(), aStream, true))
end))() else return nil end end)()
;
(function() if ((((((aClass)["$klass"]):propertyNames()):notEmpty()))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass)["$klass"]));
return ((_s8_rec):nextPutAll_("[\"$propertyNames\"]="))
end
)(aStream);return ((self):exportArrayLiteral_on_newLine_(((aClass)["$klass"]):propertyNames(), aStream, true))
end))() else return nil end end)()
;
return (self)
;
end
)
,"export","exportMetaDefinitionOf: aClass on: aStream\r\n\t\" Export class side definition of aClass onto aStream. \"\r\n\r\n\taClass class instanceVariableNames notEmpty ifTrue: [\r\n\t\taStream \r\n\t\t\tnextPutAll: (self smalltalkNameFor: aClass class);\r\n\t\t\tnextPutAll: '[\"$iVarNames\"]='.\r\n\t\tself exportArrayLiteral: aClass class instanceVariableNames on: aStream newLine: true.\r\n\t].\r\n\r\n\taClass class propertyNames notEmpty ifTrue: [\r\n\t\taStream \r\n\t\t\tnextPutAll: (self smalltalkNameFor: aClass class);\r\n\t\t\tnextPutAll: '[\"$propertyNames\"]='.\r\n\t\tself exportArrayLiteral: aClass class propertyNames on: aStream newLine: true.\r\n\t].","(\r\nfunction(self,aClass,aStream)\r\n(function() if ((((((aClass)[\"$klass\"]):instanceVariableNames()):notEmpty()))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass)[\"$klass\"]));\r\nreturn ((_s8_rec):nextPutAll_(\"[\\\"$iVarNames\\\"]=\"))\r\nend\r\n)(aStream);return ((self):exportArrayLiteral_on_newLine_(((aClass)[\"$klass\"]):instanceVariableNames(), aStream, true))\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((aClass)[\"$klass\"]):propertyNames()):notEmpty()))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_((aClass)[\"$klass\"]));\r\nreturn ((_s8_rec):nextPutAll_(\"[\\\"$propertyNames\\\"]=\"))\r\nend\r\n)(aStream);return ((self):exportArrayLiteral_on_newLine_(((aClass)[\"$klass\"]):propertyNames(), aStream, true))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"luaSourceFor:",0
,(
function(self,aMethod)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local code=nil
(function() if ((((smalltalk.Smalltalk):isRunningOn_("lua")))==true) then return ((function()
(function()
	local _s8_value=((function() local _1_=((aMethod)["$nativeSource"]);if (_1_==nil) then return ((function()
return (nil)
end)):value() end; return _1_ end)())
	code=_s8_value; return _s8_value
end)();return ((function() if ((((((code))~=nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (code)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=(((function()
return ((self):luaCodeFor_(aMethod))
end)):on_do_(smalltalk.Error, (function(ex)
(self):note_(aMethod);return ((self):luaNIYSourceFor_with_(aMethod, (ex):description()))
end)))
	code=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("(\r\n"),code)),"\r\n)"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"lua","luaSourceFor: aMethod\r\n\t\" Private - Returns the lua code implementing aMethod. \"\r\n\r\n\t| code |\r\n\t(Smalltalk isRunningOn: #lua) ifTrue: [\r\n\t\tcode := aMethod basicAt: '$nativeSource' ifAbsent: [].\r\n\t\tcode notNil ifTrue: [ ^code ].\r\n\t].\r\n\r\n\tcode := [ self luaCodeFor: aMethod ] on: Error do: [:ex|\r\n\t\tself note: aMethod.\r\n\t\tself luaNIYSourceFor: aMethod with: ex description\r\n\t].\r\n\t^'(\r\n'\t,code ,'\r\n)'","(\r\nfunction(self,aMethod)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal code=nil\r\n(function() if ((((smalltalk.Smalltalk):isRunningOn_(\"lua\")))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((function() local _1_=((aMethod)[\"$nativeSource\"]);if (_1_==nil) then return ((function()\r\nreturn (nil)\r\nend)):value() end; return _1_ end)())\r\n\tcode=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((code))~=nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (code)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((function()\r\nreturn ((self):luaCodeFor_(aMethod))\r\nend)):on_do_(smalltalk.Error, (function(ex)\r\n(self):note_(aMethod);return ((self):luaNIYSourceFor_with_(aMethod, (ex):description()))\r\nend)))\r\n\tcode=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"(\\r\\n\"),code)),\"\\r\\n)\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"luaNIYSourceFor:with:",0
,(
function(self,aMethod,description)
return (smalltalk:object_comma_(((self):luaMethodDefinition_(aMethod)),(self):luaMethodBody_with_(aMethod, description)))
;
end
)
,"niy","luaNIYSourceFor: aMethod with: description\r\n\t\" Private - Returns the lua code implementing aMethod. \"\r\n\r\n\t^(self luaMethodDefinition: aMethod)\r\n\t,(self luaMethodBody: aMethod with: description)","(\r\nfunction(self,aMethod,description)\r\nreturn (smalltalk:object_comma_(((self):luaMethodDefinition_(aMethod)),(self):luaMethodBody_with_(aMethod, description)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"luaMethodDefinition:",0
,(
function(self,aMethod)
local stream=nil
local args=nil
(function()
	local _s8_value=((
function(_s8_rec)
(_s8_rec):upTo_("(");
return ((_s8_rec):upTo_(")"))
end
)(((aMethod):compiledSource()):stream()))
	args=_s8_value; return _s8_value
end)()
;
(function() if ((((args):notEmpty()))==true) then return ((function()
return ((function()
	local _s8_value=(((args):asArrayOfSubstringsSeparatedBy_(",")):inject_into_("", (function(total,each)
return (smalltalk:object_comma_((smalltalk:object_comma_((total),",")),(each):asLuaName()))
end)))
	args=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (smalltalk:object_comma_((smalltalk:object_comma_(("function(self"),args)),")"))
;
end
)
,"niy","luaMethodDefinition: aMethod\r\n\t\" Private - Return the (lua) method definition of aMethod. \"\r\n\r\n\t| stream args |\r\n\targs := aMethod compiledSource stream upTo: $(; upTo: $).\r\n\targs notEmpty ifTrue: [\r\n\t\targs := (args asArrayOfSubstringsSeparatedBy: $,)\r\n\t\t\tinject: '' into: [:total :each| total ,',' ,each asLuaName ].\r\n\t].\r\n\t^'function(self' ,args ,')'","(\r\nfunction(self,aMethod)\r\nlocal stream=nil\r\nlocal args=nil\r\n(function()\r\n\tlocal _s8_value=((\r\nfunction(_s8_rec)\r\n(_s8_rec):upTo_(\"(\");\r\nreturn ((_s8_rec):upTo_(\")\"))\r\nend\r\n)(((aMethod):compiledSource()):stream()))\r\n\targs=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((args):notEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(((args):asArrayOfSubstringsSeparatedBy_(\",\")):inject_into_(\"\", (function(total,each)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((total),\",\")),(each):asLuaName()))\r\nend)))\r\n\targs=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"function(self\"),args)),\")\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"luaMethodBody:with:",0
,(
function(self,aMethod,description)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("\r\n\tprint(\"NIY:#luaMethodBody: "),aMethod))," - ")),description)),"\")\r\n\treturn self\r\nend"))
;
end
)
,"niy","luaMethodBody: aMethod with: description\r\n\t\" Private - Return the (lua) body of aMethod. \"\r\n\r\n\t^'\r\n\tprint(\"NIY:#luaMethodBody: ',aMethod,' - ',description,'\")\r\n\treturn self\r\nend'","(\r\nfunction(self,aMethod,description)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"\\r\\n\\tprint(\\\"NIY:#luaMethodBody: \"),aMethod)),\" - \")),description)),\"\\\")\\r\\n\\treturn self\\r\\nend\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"luaCodeFor:",0
,(
function(self,aMethod)
(self):compiler_(((aMethod):classField()):compiler())
;
return ((
function(_s8_rec)
(_s8_rec):encoder_(self);
return ((_s8_rec):compile_((aMethod):source()))
end
)((self):compiler()))
;
end
)
,"compiling","luaCodeFor: aMethod\r\n\t\" Private - Returns the lua code translated from sources of aMethod. \"\r\n\r\n\tself compiler: aMethod classField compiler.\r\n\t^self compiler\r\n\t\tencoder: self;\r\n\t\tcompile: aMethod source","(\r\nfunction(self,aMethod)\r\n(self):compiler_(((aMethod):classField()):compiler())\r\n;\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):encoder_(self);\r\nreturn ((_s8_rec):compile_((aMethod):source()))\r\nend\r\n)((self):compiler()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"nativeLanguage",0
,(
function(self)
return ("lua")
;
end
)
,"accessing","nativeLanguage\r\n\t\" Return the nativeLanguage of the receiver. \"\r\n\r\n\t^#lua","(\r\nfunction(self)\r\nreturn (\"lua\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"acceptAssignment:to:",0
,(
function(self,aNode,assignee)
return ((self):compilerEvent_with_ifNotHandled_("acceptAssignmentTo:", assignee, (function()
return (true)
end)))
;
end
)
,"assignment","acceptAssignment: aNode to: assignee\r\n\t\" Private - Return true if aNode expression can be assigned to assignee (aVariableNode). \"\r\n\r\n\t^self\tcompilerEvent: #acceptAssignmentTo:\r\n\t\twith: assignee\r\n\t\tifNotHandled: [ true ]","(\r\nfunction(self,aNode,assignee)\r\nreturn ((self):compilerEvent_with_ifNotHandled_(\"acceptAssignmentTo:\", assignee, (function()\r\nreturn (true)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"codeToAccessInstanceVariable:",0
,(
function(self,aName)
return ((self):codeToAccessProperty_(smalltalk:object_comma_(("@"),aName)))
;
end
)
,"generation","codeToAccessInstanceVariable: aName\r\n\t\" Private - Return the source code to access instance variable aName. \"\r\n\r\n\t^self codeToAccessProperty: '@', aName","(\r\nfunction(self,aName)\r\nreturn ((self):codeToAccessProperty_(smalltalk:object_comma_((\"@\"),aName)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"codeToAccessProperty:",0
,(
function(self,aName)
return (smalltalk:object_comma_((smalltalk:object_comma_(("self['"),aName)),"']"))
;
end
)
,"generation","codeToAccessProperty: aName\r\n\t\" Private - Return the source code to access property aName. \"\r\n\r\n\t^'self[''',aName,''']'","(\r\nfunction(self,aName)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"self['\"),aName)),\"']\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"defaultRequirementsFor:",0
,(
function(self,aClass)
return (smalltalk:newArray({}))
;
end
)
,"export","defaultRequirementsFor: aClass\r\n\t\" Private - Return the default requirements to load aClass.\r\n\tDetect native bindings and generate code to rebind before class definition.\r\n\t\"\r\n\r\n\t^#() ","(\r\nfunction(self,aClass)\r\nreturn (smalltalk:newArray({}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitCode:",0
,(
function(self,aString)
(self['@stream']):nextPutAll_(aString)
;
return (self)
;
end
)
,"private","emmitCode: aString\r\n\t\" Dump aString onto code stream. \"\r\n\r\n\tstream nextPutAll: aString","(\r\nfunction(self,aString)\r\n(self['@stream']):nextPutAll_(aString)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitEnterMethod:",0
,(
function(self,aNode)
return (self)
;
end
)
,"generation","emmitEnterMethod: aNode\r\n\t\" Private - Generate home context definitions, entering method aNode. \"","(\r\nfunction(self,aNode)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitFunction:arguments:doing:",0
,(
function(self,fName,argList,aBlock)
(self):compilerEvent_withArguments_ifNotHandled_("startFunction:with:", (smalltalk.Array):with_with_(fName, argList), (function()
(self['@stream']):nextPutAll_("function");(self):compilerEvent_with_("startFunctionName:", fName);return ((self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(("("),argList)),")")))
end))
;
(self['@stream']):cr()
;
(self):compilerEvent_withArguments_("startFunctionBody:with:", (smalltalk.Array):with_with_(fName, argList))
;
(aBlock):evaluate()
;
(
function(_s8_rec)
(_s8_rec):cr();
return ((_s8_rec):nextPutAll_("end"))
end
)(self['@stream'])
;
(self):compilerEvent_with_("endFunction:", fName)
;
return (self)
;
end
)
,"generation","emmitFunction: fName arguments: argList doing: aBlock\r\n\t\" Private - Emmit function on receiver's code stream. \"\r\n\r\n\tself\r\n\t\tcompilerEvent: #startFunction:with:\r\n\t\twithArguments: (Array with: fName with: argList)\r\n\t\tifNotHandled: [\r\n\t\t\tstream nextPutAll: 'function'.\r\n\t\t\tself compilerEvent: #startFunctionName: with: fName.\r\n\t\t\tstream nextPutAll: '(' ,argList ,$)\r\n\t\t].\r\n\tstream cr.\r\n\tself\tcompilerEvent: #startFunctionBody:with:\r\n\t\twithArguments: (Array with: fName with: argList).\r\n\taBlock evaluate.\r\n\tstream cr;nextPutAll: #end.\r\n\tself compilerEvent: #endFunction: with: fName.","(\r\nfunction(self,fName,argList,aBlock)\r\n(self):compilerEvent_withArguments_ifNotHandled_(\"startFunction:with:\", (smalltalk.Array):with_with_(fName, argList), (function()\r\n(self['@stream']):nextPutAll_(\"function\");(self):compilerEvent_with_(\"startFunctionName:\", fName);return ((self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\"(\"),argList)),\")\")))\r\nend))\r\n;\r\n(self['@stream']):cr()\r\n;\r\n(self):compilerEvent_withArguments_(\"startFunctionBody:with:\", (smalltalk.Array):with_with_(fName, argList))\r\n;\r\n(aBlock):evaluate()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\nreturn ((_s8_rec):nextPutAll_(\"end\"))\r\nend\r\n)(self['@stream'])\r\n;\r\n(self):compilerEvent_with_(\"endFunction:\", fName)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitFunction:node:withSelf:arguments:doing:",0
,(
function(self,fName,aNode,aBoolean,argumentNames,aBlock)
local args=nil
(function()
	local _s8_value=((""):writeStream())
	args=_s8_value; return _s8_value
end)()
;
(function() if (((aBoolean))==true) then return ((function()
(args):nextPutAll_("self");return ((function() if ((((argumentNames):notEmpty()))==true) then return ((function()
return ((args):nextPutAll_(","))
end))() else return nil end end)())
end))() else return nil end end)()
;
(argumentNames):do_separatedBy_((function(each)
((self):tempVariables()):add_(each);return ((args):nextPutAll_((self):localName_(each)))
end), (function()
return ((args):nextPutAll_(","))
end))
;
(self):emmitFunction_arguments_doing_(fName, (args):contents(), aBlock)
;
return (self)
;
end
)
,"generation","emmitFunction: fName node: aNode withSelf: aBoolean arguments: argumentNames doing: aBlock\r\n\t\" Private - Generate function for aNode. \"\r\n\r\n\t| args |\r\n\targs := '' writeStream.\r\n\taBoolean ifTrue: [\r\n\t\targs nextPutAll: #self.\r\n\t\targumentNames notEmpty ifTrue: [ args nextPutAll: $, ].\r\n\t].\r\n\targumentNames do: [:each | \r\n\t\tself tempVariables add: each.\r\n\t\targs nextPutAll: (self localName: each) ]\r\n\t    separatedBy: [ args nextPutAll: ',' ].\r\n\tself\temmitFunction: fName\r\n\t\targuments: args contents\r\n\t\tdoing: aBlock","(\r\nfunction(self,fName,aNode,aBoolean,argumentNames,aBlock)\r\nlocal args=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\targs=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((aBoolean))==true) then return ((function()\r\n(args):nextPutAll_(\"self\");return ((function() if ((((argumentNames):notEmpty()))==true) then return ((function()\r\nreturn ((args):nextPutAll_(\",\"))\r\nend))() else return nil end end)())\r\nend))() else return nil end end)()\r\n;\r\n(argumentNames):do_separatedBy_((function(each)\r\n((self):tempVariables()):add_(each);return ((args):nextPutAll_((self):localName_(each)))\r\nend), (function()\r\nreturn ((args):nextPutAll_(\",\"))\r\nend))\r\n;\r\n(self):emmitFunction_arguments_doing_(fName, (args):contents(), aBlock)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitFunction:node:arguments:doing:",0
,(
function(self,fName,aNode,argumentNames,aBlock)
return ((self):emmitFunction_node_withSelf_arguments_doing_(fName, aNode, false, argumentNames, aBlock))
;
end
)
,"generation","emmitFunction: fName node: aNode arguments: argumentNames doing: aBlock\r\n\t\" Private - Generate function for aNode. \"\r\n\r\n\t^self\temmitFunction: fName node: aNode\r\n\t\twithSelf: false arguments: argumentNames\r\n\t\tdoing: aBlock","(\r\nfunction(self,fName,aNode,argumentNames,aBlock)\r\nreturn ((self):emmitFunction_node_withSelf_arguments_doing_(fName, aNode, false, argumentNames, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitFunction:node:doing:",0
,(
function(self,fName,aNode,aBlock)
return ((self):emmitFunction_node_withSelf_arguments_doing_(fName, aNode, true, (aNode):argumentNames(), aBlock))
;
end
)
,"generation","emmitFunction: fName node: aNode doing: aBlock\r\n\t\" Private - Generate function for aNode. \"\r\n\r\n\t^self\temmitFunction: fName node: aNode\r\n\t\twithSelf: true arguments: aNode argumentNames\r\n\t\tdoing: aBlock","(\r\nfunction(self,fName,aNode,aBlock)\r\nreturn ((self):emmitFunction_node_withSelf_arguments_doing_(fName, aNode, true, (aNode):argumentNames(), aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitFunctionDoing:",0
,(
function(self,aBlock)
return ((self):emmitFunction_arguments_doing_("", "", aBlock))
;
end
)
,"generation","emmitFunctionDoing: aBlock\r\n\t\" Private - Emmit function on receiver's code stream. \"\r\n\r\n\t^self emmitFunction: '' arguments: '' doing: aBlock","(\r\nfunction(self,aBlock)\r\nreturn ((self):emmitFunction_arguments_doing_(\"\", \"\", aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"position",0
,(
function(self)
return (((self['@stream']):position()):_plus(self['@startOffset']))
;
end
)
,"accessing","position\r\n\t\" Return the current position in output stream. \"\r\n\r\n\t^stream position + startOffset","(\r\nfunction(self)\r\nreturn (((self['@stream']):position()):_plus(self['@startOffset']))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"startCompiling:",0
,(
function(self,aNode)
smalltalk:superImplementor(smalltalk.LuaEncoder,"startCompiling_")(self,aNode)
;
(function()
	local _s8_value=((""):writeStream())
	self['@stream']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self['@stream']):position())
	self['@startOffset']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"private","startCompiling: aNode\r\n\t\" Private - Set the receiver to start emmiting code for aNode. \"\r\n\r\n\tsuper startCompiling: aNode.\r\n\tstream := '' writeStream.\r\n\tstartOffset := stream position.","(\r\nfunction(self,aNode)\r\nsmalltalk:superImplementor(smalltalk.LuaEncoder,\"startCompiling_\")(self,aNode)\r\n;\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tself['@stream']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@stream']):position())\r\n\tself['@startOffset']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"stream",0
,(
function(self)
return (self['@stream'])
;
end
)
,"accessing","stream\r\n\t\" Return the current stream (or nil). \"\r\n\r\n\t^stream","(\r\nfunction(self)\r\nreturn (self['@stream'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visit:",0
,(
function(self,aNode)
return ((self):visit_doing_(aNode, (function()
return (smalltalk:superImplementor(smalltalk.LuaEncoder,"visit_")(self,aNode))
end)))
;
end
)
,"visiting","visit: aNode\r\n\t^self visit: aNode doing: [ super visit: aNode ]","(\r\nfunction(self,aNode)\r\nreturn ((self):visit_doing_(aNode, (function()\r\nreturn (smalltalk:superImplementor(smalltalk.LuaEncoder,\"visit_\")(self,aNode))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visit:doing:",0
,(
function(self,aNode,aBlock)
local cookie=nil
local result=nil
(function()
	local _s8_value=((self):compilerEvent_with_ifNotHandled_("needsCookie:", aNode, (function()
return (nil)
end)))
	cookie=_s8_value; return _s8_value
end)()
;
(function() if ((((((cookie))~=nil)))==true) then return ((function()
return ((self):compilerEvent_with_("startVisit:", cookie))
end))() else return nil end end)()
;
(function()
	local _s8_value=((aBlock):evaluate())
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((((cookie))~=nil)))==true) then return ((function()
return ((self):compilerEvent_with_("endVisit:", cookie))
end))() else return nil end end)()
;
return (result)
;
end
)
,"visiting","visit: aNode doing: aBlock\r\n\t| cookie result |\r\n\tcookie := self\r\n\t\tcompilerEvent: #needsCookie: with: aNode\r\n\t\tifNotHandled: [ nil ].\r\n\tcookie notNil ifTrue: [ self compilerEvent: #startVisit: with: cookie ].\r\n\tresult := aBlock evaluate.\r\n\tcookie notNil ifTrue: [ self compilerEvent: #endVisit: with: cookie ].\r\n\t^result","(\r\nfunction(self,aNode,aBlock)\r\nlocal cookie=nil\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):compilerEvent_with_ifNotHandled_(\"needsCookie:\", aNode, (function()\r\nreturn (nil)\r\nend)))\r\n\tcookie=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((cookie))~=nil)))==true) then return ((function()\r\nreturn ((self):compilerEvent_with_(\"startVisit:\", cookie))\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aBlock):evaluate())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((cookie))~=nil)))==true) then return ((function()\r\nreturn ((self):compilerEvent_with_(\"endVisit:\", cookie))\r\nend))() else return nil end end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitArgument:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aNode):isAssignment()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitLocalAssignment_(aNode))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visit_(aNode))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitArgument: aNode\r\n\t\" Private - Visit the argument emmitting output. \"\r\n\r\n\taNode isAssignment ifTrue: [ ^self visitLocalAssignment: aNode. ].\r\n\t^self visit: aNode","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aNode):isAssignment()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitLocalAssignment_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visit_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"includeSourceCode",0
,(
function(self)
return (true)
;
end
)
,"generation","includeSourceCode\r\n\t\" Private - Return true if source code is included as method argument. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"isOperation:",0
,(
function(self,aNode)
return ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((((aNode):arguments()):size()),(1)))),(function()
return ((smalltalk:boolean_and_((((((aNode):selector()):size()) > (1))),(function()
return ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((((aNode):selector()):first()),"#"))),(function()
return (("|&\\\\+*/=><,@%~-"):includes_(((aNode):selector()):at_((2))))
end))))
end))))
end))))
;
end
)
,"operations","isOperation: aNode\r\n\t\" Private - Return true if aNode is an operation. \"\r\n\r\n\t^aNode arguments size = 1 and: [\r\n\t\taNode selector size > 1 and: [\r\n\t\taNode selector first = $# and: [\r\n\t\t'|&\\\\+*/=><,@%~-' includes: (aNode selector at: 2)\r\n\t\t]]]","(\r\nfunction(self,aNode)\r\nreturn ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((((aNode):arguments()):size()),(1)))),(function()\r\nreturn ((smalltalk:boolean_and_((((((aNode):selector()):size()) > (1))),(function()\r\nreturn ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((((aNode):selector()):first()),\"#\"))),(function()\r\nreturn ((\"|&\\\\\\\\+*/=><,@%~-\"):includes_(((aNode):selector()):at_((2))))\r\nend))))\r\nend))))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"loadRequirementsOf:",0
,(
function(self,aClass)
local requirements=nil
(function()
	local _s8_value=((self):defaultRequirementsFor_(aClass))
	requirements=_s8_value; return _s8_value
end)()
;
(aClass):loadRequirementsInto_(requirements)
;
return (requirements)
;
end
)
,"export","loadRequirementsOf: aClass\r\n\t\" Private - Return the requirements to load aClass.\r\n\tIt must be a collection of requirements.\r\n\taRequirement must implement #fileOutOn: or must be aString(native code) to execute just before defining aClass.\r\n\t\"\r\n\r\n\t| requirements |\r\n\trequirements := self defaultRequirementsFor: aClass.\r\n\taClass loadRequirementsInto: requirements.\r\n\t^requirements","(\r\nfunction(self,aClass)\r\nlocal requirements=nil\r\n(function()\r\n\tlocal _s8_value=((self):defaultRequirementsFor_(aClass))\r\n\trequirements=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aClass):loadRequirementsInto_(requirements)\r\n;\r\nreturn (requirements)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"newMethodFunctionName",0
,(
function(self)
return ("smalltalk:newMethod")
;
end
)
,"generation","newMethodFunctionName\r\n\t\" Private - Return the function to call when installing a new method. \"\r\n\r\n\t^'smalltalk:newMethod'","(\r\nfunction(self)\r\nreturn (\"smalltalk:newMethod\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitMethodNode:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local fnStart=nil
local fnEnd=nil
local nativeSource=nil
(function() if ((((smalltalk.Smalltalk):isRunningOn_("lua")))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):emmitMethod_(aNode))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_(smalltalk:object_comma_(((self):newMethodFunctionName()),"("));
(_s8_rec):nextPutAll_((self):smalltalkNameFor_((self):currentClass()));
(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((",\""),(aNode):selector())),"\","));
return ((_s8_rec):cr())
end
)(self['@stream'])
;
(function()
	local _s8_value=((self['@stream']):position())
	fnStart=_s8_value; return _s8_value
end)()
;
(self):emmitMethod_(aNode)
;
(function()
	local _s8_value=((self['@stream']):position())
	fnEnd=_s8_value; return _s8_value
end)()
;
(function() if ((((self):includeSourceCode()))==true) then return ((function()
(function()
	local _s8_value=(((self['@stream']):contents()):copyFrom_to_((fnStart):_plus((1)), (fnEnd):_plus((1))))
	nativeSource=_s8_value; return _s8_value
end)();return ((
function(_s8_rec)
(_s8_rec):cr();
(_s8_rec):nextPutAll_(smalltalk:object_comma_((","),(self):escaped_((aNode):source())));
return ((_s8_rec):nextPutAll_(smalltalk:object_comma_((","),(self):escaped_(nativeSource))))
end
)(self['@stream']))
end))() else return nil end end)()
;
(self['@stream']):nextPutAll_(")")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitMethodNode: aNode\r\n\t\" Visit a method node. \"\r\n\r\n\t| fnStart fnEnd nativeSource |\r\n\t(Smalltalk isRunningOn: #lua) ifFalse: [ ^self emmitMethod: aNode ].\r\n\r\n\t\"return the code for method ready to be injected in the system\"\r\n\tstream\r\n\t\tnextPutAll: self newMethodFunctionName, '(';\r\n\t\tnextPutAll: (self smalltalkNameFor: self currentClass);\r\n\t\tnextPutAll: ',\"', aNode selector, '\",'; cr.\r\n\tfnStart := stream position.\r\n\tself emmitMethod: aNode.\r\n\tfnEnd := stream position.\r\n\tself includeSourceCode ifTrue: [\r\n\t\tnativeSource  := stream contents copyFrom: fnStart+1 to: fnEnd+1.\r\n\t\tstream\tcr; \"warning: this cr is important for debugger\"\r\n\t\t\tnextPutAll: ',' ,(self escaped: aNode source)\r\n\t\t\t;nextPutAll: ',' ,(self escaped: nativeSource)\r\n\t].\r\n\tstream nextPutAll: ')'","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal fnStart=nil\r\nlocal fnEnd=nil\r\nlocal nativeSource=nil\r\n(function() if ((((smalltalk.Smalltalk):isRunningOn_(\"lua\")))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):emmitMethod_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_(((self):newMethodFunctionName()),\"(\"));\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_((self):currentClass()));\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\",\\\"\"),(aNode):selector())),\"\\\",\"));\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(self['@stream'])\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@stream']):position())\r\n\tfnStart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):emmitMethod_(aNode)\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@stream']):position())\r\n\tfnEnd=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((self):includeSourceCode()))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=(((self['@stream']):contents()):copyFrom_to_((fnStart):_plus((1)), (fnEnd):_plus((1))))\r\n\tnativeSource=_s8_value; return _s8_value\r\nend)();return ((\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((\",\"),(self):escaped_((aNode):source())));\r\nreturn ((_s8_rec):nextPutAll_(smalltalk:object_comma_((\",\"),(self):escaped_(nativeSource))))\r\nend\r\n)(self['@stream']))\r\nend))() else return nil end end)()\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitMethod:",0
,(
function(self,aNode)
local code=nil
(function()
	local _s8_value=((0))
	self['@nestedBlocks']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(false)
	self['@earlyReturn']=_s8_value; return _s8_value
end)()
;
((self):compiler()):resetVariables()
;
(function()
	local _s8_value=(((aNode):selector()):asLuaSelector())
	self['@functionName']=_s8_value; return _s8_value
end)()
;
(aNode):validateArguments()
;
(self):emmitFunction_node_doing_(self['@functionName'], aNode, (function()
(self):compilerEvent_with_("startMethod:", aNode);(function()
	local _s8_value=((self):generatedCodeFor_while_(aNode, (function()
(self):compilerEvent_withArguments_ifNotHandled_("enterMethod:", (smalltalk.Array):with_(aNode), (function()
return ((self):emmitEnterMethod_(aNode))
end));return (((aNode):nodes()):do_((function(each)
return ((self):visit_(each))
end)))
end)))
	code=_s8_value; return _s8_value
end)();(self):generate_early_return_(self['@functionName'], self['@earlyReturn'], code);return ((self):compilerEvent_with_("endMethod:", aNode))
end))
;
return (self)
;
end
)
,"generation","emmitMethod: aNode\r\n\t\" Private - Generate code of method aNode. \"\r\n\r\n\t| code |\r\n\tnestedBlocks := 0.\r\n\tearlyReturn := false.\r\n\tself compiler resetVariables.\r\n\r\n\tfunctionName := aNode selector asLuaSelector.\r\n\taNode validateArguments.\r\n\tself emmitFunction: functionName node: aNode doing: [\r\n\t\tself compilerEvent: #startMethod: with: aNode.\r\n\t\tcode := self generatedCodeFor: aNode while: [\r\n\t\t\tself compilerEvent: #enterMethod: withArguments: (Array with: aNode)\r\n\t\t\t\tifNotHandled: [ self emmitEnterMethod: aNode ].\r\n\t\t\taNode nodes do: [:each | self visit: each ].\r\n\t\t].\r\n\t\tself generate: functionName early: earlyReturn return: code.\r\n\t\tself compilerEvent: #endMethod: with: aNode.\r\n\t]","(\r\nfunction(self,aNode)\r\nlocal code=nil\r\n(function()\r\n\tlocal _s8_value=((0))\r\n\tself['@nestedBlocks']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(false)\r\n\tself['@earlyReturn']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((self):compiler()):resetVariables()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((aNode):selector()):asLuaSelector())\r\n\tself['@functionName']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aNode):validateArguments()\r\n;\r\n(self):emmitFunction_node_doing_(self['@functionName'], aNode, (function()\r\n(self):compilerEvent_with_(\"startMethod:\", aNode);(function()\r\n\tlocal _s8_value=((self):generatedCodeFor_while_(aNode, (function()\r\n(self):compilerEvent_withArguments_ifNotHandled_(\"enterMethod:\", (smalltalk.Array):with_(aNode), (function()\r\nreturn ((self):emmitEnterMethod_(aNode))\r\nend));return (((aNode):nodes()):do_((function(each)\r\nreturn ((self):visit_(each))\r\nend)))\r\nend)))\r\n\tcode=_s8_value; return _s8_value\r\nend)();(self):generate_early_return_(self['@functionName'], self['@earlyReturn'], code);return ((self):compilerEvent_with_(\"endMethod:\", aNode))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"generatedCodeFor:while:",0
,(
function(self,aNode,aBlock)
local backup=nil
local result=nil
(function()
	local _s8_value=((smalltalk.Array):with_with_(self['@stream'], self['@startOffset']))
	backup=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self['@stream']):position())
	self['@startOffset']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((""):writeStream())
	self['@stream']=_s8_value; return _s8_value
end)()
;
(self):visit_doing_(aNode, aBlock)
;
(function()
	local _s8_value=((self['@stream']):contents())
	result=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((backup):first())
	self['@stream']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((backup):last())
	self['@startOffset']=_s8_value; return _s8_value
end)()
;
return (result)
;
end
)
,"generation","generatedCodeFor: aNode while: aBlock\r\n\t\" Private - Return the code generated during evaluation of aBlock. \"\r\n\r\n\t| backup result |\r\n\tbackup := Array with: stream with: startOffset.\r\n\tstartOffset := stream position.\r\n\tstream := '' writeStream.\r\n\tself visit: aNode doing: aBlock.\r\n\tresult := stream contents.\r\n\tstream := backup first.\r\n\tstartOffset := backup last.\r\n\t^result ","(\r\nfunction(self,aNode,aBlock)\r\nlocal backup=nil\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):with_with_(self['@stream'], self['@startOffset']))\r\n\tbackup=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@stream']):position())\r\n\tself['@startOffset']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tself['@stream']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):visit_doing_(aNode, aBlock)\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@stream']):contents())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((backup):first())\r\n\tself['@stream']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((backup):last())\r\n\tself['@startOffset']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitExtendedSequenceNode:",0
,(
function(self,aNode)
((aNode):directives()):do_((function(each)
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_((each):asNative_for_on_(self['@functionName'], self, self['@stream']));
return ((_s8_rec):cr())
end
)(self['@stream']))
end))
;
return ((self):visitSequenceNode_(aNode))
;
end
)
,"visiting","visitExtendedSequenceNode: aNode\r\n\t\" Private - Visit a directive node. \"\r\n\r\n\taNode directives do: [:each|\r\n\t\tstream nextPutAll: (each\r\n\t\t\tasNative: functionName\r\n\t\t\tfor: self on: stream); cr.\r\n\t].\r\n\t^self visitSequenceNode: aNode","(\r\nfunction(self,aNode)\r\n((aNode):directives()):do_((function(each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((each):asNative_for_on_(self['@functionName'], self, self['@stream']));\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(self['@stream']))\r\nend))\r\n;\r\nreturn ((self):visitSequenceNode_(aNode))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitSequenceNode:",0
,(
function(self,aNode)
((aNode):tempNames()):do_((function(each)
((self):tempVariables()):add_(each);return ((
function(_s8_rec)
(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(("local "),(self):localName_(each))),"=nil"));
return ((_s8_rec):cr())
end
)(self['@stream']))
end))
;
((aNode):nodes()):do_separatedBy_((function(each)
(self):visit_(each);return ((
function(_s8_rec)
(_s8_rec):cr();
return ((_s8_rec):nextPutAll_(";"))
end
)(self['@stream']))
end), (function()
return ((self['@stream']):cr())
end))
;
return (self)
;
end
)
,"visiting","visitSequenceNode: aNode\r\n\r\n\taNode tempNames do: [:each |\r\n\t\tself tempVariables add: each.\r\n\t\tstream nextPutAll: 'local ', (self localName: each), '=nil';cr\r\n\t].\r\n\taNode nodes do: [:each |\r\n\t\tself visit: each.\r\n\t\tstream cr\t ;nextPutAll: ';'\r\n\t] separatedBy: [stream cr]","(\r\nfunction(self,aNode)\r\n((aNode):tempNames()):do_((function(each)\r\n((self):tempVariables()):add_(each);return ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\"local \"),(self):localName_(each))),\"=nil\"));\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(self['@stream']))\r\nend))\r\n;\r\n((aNode):nodes()):do_separatedBy_((function(each)\r\n(self):visit_(each);return ((\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\nreturn ((_s8_rec):nextPutAll_(\";\"))\r\nend\r\n)(self['@stream']))\r\nend), (function()\r\nreturn ((self['@stream']):cr())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitedCode",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((self['@stream']))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk.Smalltalk):isRunningOn_("lua")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_(("return "),(self['@stream']):contents()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self['@stream']):contents())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"results","emmitedCode\r\n\t\" Return the emmited code during compilation (or nil). \"\r\n\r\n\tstream isNil ifTrue: [ ^nil ].\r\n\t(Smalltalk isRunningOn: #lua) ifTrue: [\r\n\t\t\"the code must be valid Lua code including return expression\"\r\n\t\t^'return ' ,stream contents\r\n\t].\r\n\t^stream contents","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((self['@stream']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk.Smalltalk):isRunningOn_(\"lua\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((\"return \"),(self['@stream']):contents()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self['@stream']):contents())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitReturn:",0
,(
function(self,aBlockOrString)
return ((self):emmitReturn_early_(aBlockOrString, false))
;
end
)
,"generation","emmitReturn: aBlockOrString\r\n\t\" Private - Generate return code. \"\r\n\r\n\t^self emmitReturn: aBlockOrString early: false","(\r\nfunction(self,aBlockOrString)\r\nreturn ((self):emmitReturn_early_(aBlockOrString, false))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitOpenReturn:",0
,(
function(self,aBlockOrString)
(self):compilerEvent_with_("aboutReturn:", aBlockOrString)
;
(self['@stream']):nextPutAll_("return (")
;
return (self)
;
end
)
,"generation","emmitOpenReturn: aBlockOrString\r\n\t\" Private - Generate code to open return. \"\r\n\r\n\tself compilerEvent: #aboutReturn: with: aBlockOrString.\r\n\tstream nextPutAll: 'return ('.","(\r\nfunction(self,aBlockOrString)\r\n(self):compilerEvent_with_(\"aboutReturn:\", aBlockOrString)\r\n;\r\n(self['@stream']):nextPutAll_(\"return (\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitCloseReturn:",0
,(
function(self,aBlockOrString)
(self['@stream']):nextPutAll_(")")
;
return (self)
;
end
)
,"generation","emmitCloseReturn: aBlockOrString\r\n\t\" Private - Generate code to close return. \"\r\n\r\n\tstream nextPutAll: $)","(\r\nfunction(self,aBlockOrString)\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitReturn:early:",0
,(
function(self,aBlockOrString,early)
(self):emmitOpenReturn_(aBlockOrString)
;
(self):compilerEvent_with_("return:", aBlockOrString)
;
(function() if ((((aBlockOrString):isString()))==true) then return ((function()
return ((self['@stream']):nextPutAll_(aBlockOrString))
end))() else return ((function()
return ((aBlockOrString):evaluate())
end))() end end)()
;
(function() if ((((((aBlockOrString))~=nil)))==true) then return ((function()
return ((self):compilerEvent_withArguments_("returned:early:", (smalltalk.Array):with_with_(aBlockOrString, early)))
end))() else return nil end end)()
;
(self):emmitCloseReturn_(aBlockOrString)
;
return (self)
;
end
)
,"generation","emmitReturn: aBlockOrString early: early\r\n\t\" Private - Generate return code. \"\r\n\r\n\tself emmitOpenReturn: aBlockOrString.\r\n\tself compilerEvent: #return: with: aBlockOrString.\r\n\taBlockOrString isString\r\n\t\tifTrue: [ stream nextPutAll: aBlockOrString ]\r\n\t\tifFalse: [ aBlockOrString evaluate ].\r\n\taBlockOrString notNil ifTrue: [\r\n\t\tself\tcompilerEvent: #returned:early:\r\n\t\t\twithArguments: (Array with: aBlockOrString with: early).\r\n\t].\r\n\tself emmitCloseReturn: aBlockOrString","(\r\nfunction(self,aBlockOrString,early)\r\n(self):emmitOpenReturn_(aBlockOrString)\r\n;\r\n(self):compilerEvent_with_(\"return:\", aBlockOrString)\r\n;\r\n(function() if ((((aBlockOrString):isString()))==true) then return ((function()\r\nreturn ((self['@stream']):nextPutAll_(aBlockOrString))\r\nend))() else return ((function()\r\nreturn ((aBlockOrString):evaluate())\r\nend))() end end)()\r\n;\r\n(function() if ((((((aBlockOrString))~=nil)))==true) then return ((function()\r\nreturn ((self):compilerEvent_withArguments_(\"returned:early:\", (smalltalk.Array):with_with_(aBlockOrString, early)))\r\nend))() else return nil end end)()\r\n;\r\n(self):emmitCloseReturn_(aBlockOrString)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitSendNode:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((aNode):receiver()):isSuper()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitSendToSuper_(aNode))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((self):visitOptimizedSend_(aNode)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((self):isOperation_(aNode)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitOperation_(aNode))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):visitNormalSend_(aNode))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitSendNode: aNode\r\n\t\" Private - Visit the node (message send node) emmitting output. \"\r\n\r\n\taNode receiver isSuper ifTrue: [ ^self visitSendToSuper: aNode ].\r\n\t(self visitOptimizedSend: aNode) ifTrue: [ ^self ].\r\n\t(self isOperation: aNode) ifTrue: [ ^self visitOperation: aNode ].\r\n\t^self visitNormalSend: aNode","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((aNode):receiver()):isSuper()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitSendToSuper_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((self):visitOptimizedSend_(aNode)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((self):isOperation_(aNode)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitOperation_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):visitNormalSend_(aNode))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitValueNode:",0
,(
function(self,aNode)
(self['@stream']):nextPutAll_(((aNode):value()):asLua())
;
return (self)
;
end
)
,"visiting","visitValueNode: aNode\r\n\t\" Private - Visit the node emmitting output. \"\r\n\r\n\tstream nextPutAll: aNode value asLua","(\r\nfunction(self,aNode)\r\n(self['@stream']):nextPutAll_(((aNode):value()):asLua())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitVariableNode:",0
,(
function(self,aNode)
local code=nil
(function()
	local _s8_value=(((self):currentClass()):codeFor_accessing_(self, aNode))
	code=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:boolean_and_(((((code))==nil)),(function()
return ((aNode):needsVerification())
end)))))==true) then return ((function()
return ((function() if (((((self):knownVariables()):includes_((aNode):value())))==true) then return nil else return ((function()
return ((self):compilerEvent_with_ifNotHandled_("unknownVariable:", aNode, (function()
((self):unknownVariables()):add_((aNode):value());return (((self)["$klass"]):unknownVariable_in_(aNode, self))
end)))
end))() end end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:boolean_or_(((((code))==nil)),(function()
return (((self):knownVariables()):includes_((aNode):value()))
end)))))==true) then return ((function()
return ((function()
	local _s8_value=((aNode):localNameIn_(self))
	code=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(self['@stream']):nextPutAll_(code)
;
(self):visitNodeIndices_(aNode)
;
return (self)
;
end
)
,"visiting","visitVariableNode: aNode\r\n\t\" Generate code to access variable aNode. \"\r\n\r\n\t| code |\r\n\tcode := self currentClass codeFor: self accessing: aNode.\r\n\t(code isNil and: [aNode needsVerification]) ifTrue: [\r\n\t\t(self knownVariables includes: aNode value) ifFalse: [\r\n\t\t\tself\tcompilerEvent: #unknownVariable: with: aNode\r\n\t\t\t\tifNotHandled: [\r\n\t\t\t\t\tself unknownVariables add: aNode value.\r\n\t\t\t\t\tself class unknownVariable: aNode in: self.\r\n\t\t\t\t].\r\n\t\t].\r\n\t].\r\n\t(code isNil or: [self knownVariables includes: aNode value]) ifTrue: [\r\n\t\tcode := aNode localNameIn: self\r\n\t].\r\n\r\n\tstream nextPutAll: code.\r\n\tself visitNodeIndices: aNode.","(\r\nfunction(self,aNode)\r\nlocal code=nil\r\n(function()\r\n\tlocal _s8_value=(((self):currentClass()):codeFor_accessing_(self, aNode))\r\n\tcode=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((((code))==nil)),(function()\r\nreturn ((aNode):needsVerification())\r\nend)))))==true) then return ((function()\r\nreturn ((function() if (((((self):knownVariables()):includes_((aNode):value())))==true) then return nil else return ((function()\r\nreturn ((self):compilerEvent_with_ifNotHandled_(\"unknownVariable:\", aNode, (function()\r\n((self):unknownVariables()):add_((aNode):value());return (((self)[\"$klass\"]):unknownVariable_in_(aNode, self))\r\nend)))\r\nend))() end end)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:boolean_or_(((((code))==nil)),(function()\r\nreturn (((self):knownVariables()):includes_((aNode):value()))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((aNode):localNameIn_(self))\r\n\tcode=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self['@stream']):nextPutAll_(code)\r\n;\r\n(self):visitNodeIndices_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitNode:index:",0
,(
function(self,aNode,index)
local selector=nil
(self['@stream']):nextPutAll_("[")
;
(self):visit_(index)
;
(self['@stream']):nextPutAll_("]")
;
return (self)
;
end
)
,"visiting","visitNode: aNode index: index\r\n\t\" Private - Generate code for direct property access. \"\r\n\r\n\t| selector |\r\n\"ToDo -\t((index isMemberOf: ValueNode) and: [ index hasString ]) ifTrue: [\r\n\t\tselector := index value asLuaSelector.\r\n\t\t((selector includes: $ ) or: [ selector includes: $_ ]) ifFalse: [\r\n\t\t\tstream nextPutAll: '.'; nextPutAll: selector.\r\n\t\t\t^self\r\n\t\t].\r\n\t].\"\r\n\tstream nextPutAll: '['.\r\n\tself visit: index.\r\n\tstream nextPutAll: ']'.","(\r\nfunction(self,aNode,index)\r\nlocal selector=nil\r\n(self['@stream']):nextPutAll_(\"[\")\r\n;\r\n(self):visit_(index)\r\n;\r\n(self['@stream']):nextPutAll_(\"]\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitNodeIndices:",0
,(
function(self,aNode)
((aNode):indices()):do_((function(each)
return ((self):visitNode_index_(aNode, each))
end))
;
return (self)
;
end
)
,"visiting","visitNodeIndices: aNode\r\n\t\" Private - Generate code for direct property access. \"\r\n\r\n\taNode indices do: [:each| self visitNode: aNode index: each ].","(\r\nfunction(self,aNode)\r\n((aNode):indices()):do_((function(each)\r\nreturn ((self):visitNode_index_(aNode, each))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"luaCallEscape",0
,(
function(self)
return ("#_")
;
end
)
,"accessing","luaCallEscape\r\n\t\" Private - Returns the selector sequence to escape OO lua message send selector, and produce code for function call.\r\n\tNote that an alien selector starting with two $$ are understood as a lua function call to function starting with $\r\n\t\"\r\n\r\n\t^'#_'","(\r\nfunction(self)\r\nreturn (\"#_\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"messageSendConnector:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:isObject_equalTo_((((aNode):receiver()):messageSendConnector()),""))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if (((((aNode):selector()):startsWith_("#")))==true) then return ((function()
(function() if (((((aNode):selector()):startsWith_((self):luaCallEscape())))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (".")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (":")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (":")
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","messageSendConnector: aNode\r\n\t\" Private - Returns the caracter to use for sending message. \"\r\n\r\n\taNode receiver messageSendConnector = '' ifTrue: [ ^'' ].\r\n\t(aNode selector startsWith: $#) ifTrue: [ \"alien call\"\r\n\t\t(aNode selector startsWith: self luaCallEscape) ifTrue: [\r\n\t\t\t^$. \"a Lua function call\"\r\n\t\t].\r\n\t\t^$:  \"a Lua OO message send\"\r\n\t].\r\n\t^$:","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:isObject_equalTo_((((aNode):receiver()):messageSendConnector()),\"\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if (((((aNode):selector()):startsWith_(\"#\")))==true) then return ((function()\r\n(function() if (((((aNode):selector()):startsWith_((self):luaCallEscape())))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\".\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\":\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\":\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"asLuaSelector:",0
,(
function(self,aSelector)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function()
	local _s8_value=((aSelector):asLuaSelector())
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((aSelector):startsWith_((self):luaCallEscape())))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((result):copyFrom_to_((2), (result):size()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","asLuaSelector: aSelector\r\n\t\" Private - Validate and returns a valid Lua selector \"\r\n\r\n\t| result |\r\n\"DEBUG -\t(aSelector startsWith: $#) ifTrue: [ ^self error: 'Alien calls are not allowed. Yet' ].\r\nLOG -\t(aSelector startsWith: $#) ifTrue: [ self note: 'Calling to Alien ',aSelector ].\r\n\"\tresult := aSelector asLuaSelector.\r\n\t(aSelector startsWith: self luaCallEscape) ifTrue: [ \"call to an alien starting with _\"\r\n\t\t^result copyFrom: 2 to: result size\r\n\t].\r\n\t^result","(\r\nfunction(self,aSelector)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((aSelector):asLuaSelector())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((aSelector):startsWith_((self):luaCallEscape())))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((result):copyFrom_to_((2), (result):size()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitLuaMessage:",0
,(
function(self,aNode)
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):messageSendConnector_(aNode));
return ((_s8_rec):nextPutAll_((self):asLuaSelector_((aNode):selector())))
end
)(self['@stream'])
;
return (self)
;
end
)
,"visiting","visitLuaMessage: aNode\r\n\t\" Private - Generate code for lua message send.\"\r\n\r\n\tstream\r\n\t\tnextPutAll: (self messageSendConnector: aNode);\r\n\t\tnextPutAll: (self asLuaSelector: aNode selector)","(\r\nfunction(self,aNode)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):messageSendConnector_(aNode));\r\nreturn ((_s8_rec):nextPutAll_((self):asLuaSelector_((aNode):selector())))\r\nend\r\n)(self['@stream'])\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitNormalSend:",0
,(
function(self,aNode)
(self):compilerEvent_with_("aboutSend:", aNode)
;
(self):visitReceiver_((aNode):receiver())
;
(self):visitLuaMessage_(aNode)
;
(self['@stream']):nextPutAll_("(")
;
(self):visitSendArguments_(aNode)
;
(self['@stream']):nextPutAll_(")")
;
(self):compilerEvent_with_("send:", aNode)
;
return (self)
;
end
)
,"visiting","visitNormalSend: aNode\r\n\t\" Generate code for direct/inlined message send.\"\r\n\r\n\tself compilerEvent: #aboutSend: with: aNode.\r\n\tself visitReceiver: aNode receiver.\r\n\tself visitLuaMessage: aNode.\r\n\tstream nextPutAll: $(.\r\n\tself visitSendArguments: aNode.\r\n\tstream nextPutAll: $).\r\n\tself compilerEvent: #send: with: aNode.","(\r\nfunction(self,aNode)\r\n(self):compilerEvent_with_(\"aboutSend:\", aNode)\r\n;\r\n(self):visitReceiver_((aNode):receiver())\r\n;\r\n(self):visitLuaMessage_(aNode)\r\n;\r\n(self['@stream']):nextPutAll_(\"(\")\r\n;\r\n(self):visitSendArguments_(aNode)\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\n(self):compilerEvent_with_(\"send:\", aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitSendArguments:",0
,(
function(self,aNode)
((aNode):arguments()):do_separatedBy_((function(each)
return ((self):visitArgument_(each))
end), (function()
return ((self['@stream']):nextPutAll_(", "))
end))
;
return (self)
;
end
)
,"visiting","visitSendArguments: aNode\r\n\r\n\taNode arguments \r\n\t    do: [:each | self visitArgument: each ]\r\n\t    separatedBy: [ stream nextPutAll: ', ' ].","(\r\nfunction(self,aNode)\r\n((aNode):arguments()):do_separatedBy_((function(each)\r\nreturn ((self):visitArgument_(each))\r\nend), (function()\r\nreturn ((self['@stream']):nextPutAll_(\", \"))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitReceiver:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local last=nil
(self['@stream']):nextPutAll_("(")
;
(function()
	local _s8_value=((self['@stream']):position())
	last=_s8_value; return _s8_value
end)()
;
(function() if ((((aNode):isAssignment()))==true) then return ((function()
return ((self):visitLocalAssignment_(aNode))
end))() else return ((function()
return ((self):visit_(aNode))
end))() end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((self['@stream']):position()),last))))==true) then return ((function()
(self['@stream']):removeLastInput();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(self['@stream']):nextPutAll_(")")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitReceiver: aNode\r\n\t\" Private - Visit the node (as receiver) emmitting output. \"\r\n\r\n\t| last |\r\n\tstream nextPutAll: $(.\r\n\tlast := stream position.\r\n\taNode isAssignment\r\n\t\tifTrue: [ self visitLocalAssignment: aNode ]\r\n\t\tifFalse: [ self visit: aNode ].\r\n\tstream position = last ifTrue: [\r\n\t\tstream removeLastInput.\r\n\t\t^self\r\n\t].\r\n\tstream nextPutAll: $)","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal last=nil\r\n(self['@stream']):nextPutAll_(\"(\")\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@stream']):position())\r\n\tlast=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((aNode):isAssignment()))==true) then return ((function()\r\nreturn ((self):visitLocalAssignment_(aNode))\r\nend))() else return ((function()\r\nreturn ((self):visit_(aNode))\r\nend))() end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self['@stream']):position()),last))))==true) then return ((function()\r\n(self['@stream']):removeLastInput();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"localName:",0
,(
function(self,aName)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((self):pseudoVariables()):includes_(aName)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (aName)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aName):asLuaSelector())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","localName: aName\r\n\t\" Private - Return a valid (native, local) name for aName. \"\r\n\r\n\t(self pseudoVariables includes: aName) ifTrue: [ ^aName ].\r\n\t^aName asLuaSelector","(\r\nfunction(self,aName)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((self):pseudoVariables()):includes_(aName)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (aName)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aName):asLuaSelector())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitGlobalReferenceNode:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local aName=nil
local aClass=nil
local lookupPools=nil
(function()
	local _s8_value=((aNode):value())
	aName=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((self):currentClass()):instanceClass())
	aClass=_s8_value; return _s8_value
end)()
;
(aClass):classVariableNamesDo_((function(each)
return ((function() if ((((smalltalk:isObject_equalTo_((each),aName))))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
(_s8_rec):nextPutAll_(":classVariableAt_(\"");
(_s8_rec):nextPutAll_(aName);
return ((_s8_rec):nextPutAll_("\")"))
end
)(self['@stream']);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(aClass):sharedPoolsAndNamesDo_((function(poolName,pool)
return ((function() if ((((pool):includesKey_(aName)))==true) then return ((function()
(
function(_s8_rec)
(_s8_rec):nextPutAll_("(smalltalk:poolDictionary_(\"");
(_s8_rec):nextPutAll_(poolName);
(_s8_rec):nextPutAll_("\")):at_(\"");
(_s8_rec):nextPutAll_(aName);
return ((_s8_rec):nextPutAll_("\")"))
end
)(self['@stream']);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_("smalltalk.");
return ((_s8_rec):nextPutAll_((aNode):value()))
end
)(self['@stream'])
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitGlobalReferenceNode: aNode\r\n\r\n\t| aName aClass lookupPools |\r\n\taName := aNode value.\r\n\taClass := self currentClass instanceClass.\r\n\taClass classVariableNamesDo: [:each|\r\n\t\teach = aName ifTrue: [\r\n\t\t\tstream\r\n\t\t\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\t\t\tnextPutAll: ':classVariableAt_(\"';\r\n\t\t\t\tnextPutAll: aName;\r\n\t\t\t\tnextPutAll: '\")'.\r\n\t\t\t^self\r\n\t\t]\r\n\t].\r\n\taClass sharedPoolsAndNamesDo: [:poolName :pool|\r\n\t\t(pool includesKey: aName) ifTrue: [\r\n\t\t\tstream\r\n\t\t\t\tnextPutAll: '(smalltalk:poolDictionary_(\"';\r\n\t\t\t\tnextPutAll: poolName;\r\n\t\t\t\tnextPutAll: '\")):at_(\"';\r\n\t\t\t\tnextPutAll: aName;\r\n\t\t\t\tnextPutAll: '\")'.\r\n\t\t\t^self\r\n\t\t].\r\n\t].\r\n\tstream \r\n\t\tnextPutAll: 'smalltalk.';\r\n\t\tnextPutAll: aNode value.","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal aName=nil\r\nlocal aClass=nil\r\nlocal lookupPools=nil\r\n(function()\r\n\tlocal _s8_value=((aNode):value())\r\n\taName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((self):currentClass()):instanceClass())\r\n\taClass=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aClass):classVariableNamesDo_((function(each)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((each),aName))))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\n(_s8_rec):nextPutAll_(\":classVariableAt_(\\\"\");\r\n(_s8_rec):nextPutAll_(aName);\r\nreturn ((_s8_rec):nextPutAll_(\"\\\")\"))\r\nend\r\n)(self['@stream']);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(aClass):sharedPoolsAndNamesDo_((function(poolName,pool)\r\nreturn ((function() if ((((pool):includesKey_(aName)))==true) then return ((function()\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\"(smalltalk:poolDictionary_(\\\"\");\r\n(_s8_rec):nextPutAll_(poolName);\r\n(_s8_rec):nextPutAll_(\"\\\")):at_(\\\"\");\r\n(_s8_rec):nextPutAll_(aName);\r\nreturn ((_s8_rec):nextPutAll_(\"\\\")\"))\r\nend\r\n)(self['@stream']);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\"smalltalk.\");\r\nreturn ((_s8_rec):nextPutAll_((aNode):value()))\r\nend\r\n)(self['@stream'])\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitBlockNode:",0
,(
function(self,aNode)
(self['@stream']):nextPutAll_("(")
;
(self):emmitFunction_node_arguments_doing_("", aNode, (aNode):parameterNames(), (function()
return (((aNode):nodes()):do_((function(each)
return ((self):visit_(each))
end)))
end))
;
(self['@stream']):nextPutAll_(")")
;
return (self)
;
end
)
,"visiting","visitBlockNode: aNode\r\n\r\n\tstream nextPutAll: $(.\r\n\tself\temmitFunction: '' node: aNode\r\n\t\targuments: aNode parameterNames\r\n\t\tdoing: [ aNode nodes do: [:each | self visit: each ] ].\r\n\tstream nextPutAll: $).","(\r\nfunction(self,aNode)\r\n(self['@stream']):nextPutAll_(\"(\")\r\n;\r\n(self):emmitFunction_node_arguments_doing_(\"\", aNode, (aNode):parameterNames(), (function()\r\nreturn (((aNode):nodes()):do_((function(each)\r\nreturn ((self):visit_(each))\r\nend)))\r\nend))\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitBlockSequenceNode:",0
,(
function(self,aNode)
local index=nil
(function()
	local _s8_value=((self['@nestedBlocks']):_plus((1)))
	self['@nestedBlocks']=_s8_value; return _s8_value
end)()
;
(function() if (((((aNode):nodes()):isEmpty()))==true) then return ((function()
return ((self):emmitReturn_("nil"))
end))() else return ((function()
((aNode):tempNames()):do_((function(each)
((self):tempVariables()):add_(each);return ((
function(_s8_rec)
(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(("local "),(self):localName_(each))),"=nil"));
return ((_s8_rec):cr())
end
)(self['@stream']))
end));(function()
	local _s8_value=((0))
	index=_s8_value; return _s8_value
end)();return (((aNode):nodes()):do_((function(each)
(function()
	local _s8_value=((index):_plus((1)))
	index=_s8_value; return _s8_value
end)();return ((function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()
return ((self):emmitReturn_((function()
return ((self):visit_(each))
end)))
end))() else return ((function()
(self):visit_(each);return ((self['@stream']):nextPutAll_(";"))
end))() end end)())
end)))
end))() end end)()
;
(function()
	local _s8_value=((self['@nestedBlocks']):_minus((1)))
	self['@nestedBlocks']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"visiting","visitBlockSequenceNode: aNode\r\n\r\n\t| index |\r\n\tnestedBlocks := nestedBlocks + 1.\r\n\taNode nodes isEmpty\r\n\t    ifTrue: [ self emmitReturn: #nil ]\r\n\t    ifFalse: [\r\n\t\taNode tempNames do: [:each |\r\n\t\t    self tempVariables add: each.\r\n\t\t    stream nextPutAll: 'local ', (self localName: each), '=nil';cr\r\n\t\t].\r\n\t\tindex := 0.\r\n\t\taNode nodes do: [:each |\r\n\t\t    index := index + 1.\r\n\t\t    index = aNode nodes size\r\n\t\t\tifTrue: [ self emmitReturn: [ self visit: each ] ]\r\n\t\t    \tifFalse: [ self visit: each. stream nextPutAll: $; ]\r\n\t\t].\r\n\t].\r\n\tnestedBlocks := nestedBlocks - 1","(\r\nfunction(self,aNode)\r\nlocal index=nil\r\n(function()\r\n\tlocal _s8_value=((self['@nestedBlocks']):_plus((1)))\r\n\tself['@nestedBlocks']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((((aNode):nodes()):isEmpty()))==true) then return ((function()\r\nreturn ((self):emmitReturn_(\"nil\"))\r\nend))() else return ((function()\r\n((aNode):tempNames()):do_((function(each)\r\n((self):tempVariables()):add_(each);return ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\"local \"),(self):localName_(each))),\"=nil\"));\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(self['@stream']))\r\nend));(function()\r\n\tlocal _s8_value=((0))\r\n\tindex=_s8_value; return _s8_value\r\nend)();return (((aNode):nodes()):do_((function(each)\r\n(function()\r\n\tlocal _s8_value=((index):_plus((1)))\r\n\tindex=_s8_value; return _s8_value\r\nend)();return ((function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()\r\nreturn ((self):emmitReturn_((function()\r\nreturn ((self):visit_(each))\r\nend)))\r\nend))() else return ((function()\r\n(self):visit_(each);return ((self['@stream']):nextPutAll_(\";\"))\r\nend))() end end)())\r\nend)))\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@nestedBlocks']):_minus((1)))\r\n\tself['@nestedBlocks']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitMacroAssignment:",0
,(
function(self,aNode)
return (((aNode):left()):acceptMacroAssignment_with_((aNode):right(), self))
;
end
)
,"visiting","visitMacroAssignment: aNode\r\n\t^aNode left acceptMacroAssignment: aNode right with: self","(\r\nfunction(self,aNode)\r\nreturn (((aNode):left()):acceptMacroAssignment_with_((aNode):right(), self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitMacroAssignment:to:",0
,(
function(self,aVariableNode,anExpressionNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local aName=nil
local aClass=nil
(function() if (((((aVariableNode):indices()):notEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((aVariableNode):value())
	aName=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((self):currentClass()):instanceClass())
	aClass=_s8_value; return _s8_value
end)()
;
(aClass):classVariableNamesDo_((function(each)
return ((function() if ((((smalltalk:isObject_equalTo_((each),aName))))==true) then return ((function()
(self):visitClassVariableAssignment_in_to_(each, aClass, anExpressionNode);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(aClass):sharedPoolsAndNamesDo_((function(poolName,pool)
return ((function() if ((((pool):includesKey_(aName)))==true) then return ((function()
(self):visitPoolVariableAssignment_in_to_(aName, poolName, anExpressionNode);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitMacroAssignment: aVariableNode to: anExpressionNode\r\n\t\" Return true if the receiver generated code to a macro assignment to aVariableNode. \"\r\n\r\n\t| aName aClass |\r\n\taVariableNode indices notEmpty ifTrue: [ ^false ].\r\n\taName := aVariableNode value.\r\n\taClass := self currentClass instanceClass.\r\n\taClass classVariableNamesDo: [:each|\r\n\t\teach = aName ifTrue: [\r\n\t\t\tself visitClassVariableAssignment: each\r\n\t\t\t\tin: aClass\r\n\t\t\t\tto: anExpressionNode.\r\n\t\t\t^true\r\n\t\t]\r\n\t].\r\n\taClass sharedPoolsAndNamesDo: [:poolName :pool|\r\n\t\t(pool includesKey: aName) ifTrue: [\r\n\t\t\tself visitPoolVariableAssignment: aName\r\n\t\t\t\tin: poolName\r\n\t\t\t\tto: anExpressionNode.\r\n\t\t\t^true\r\n\t\t].\r\n\t].\r\n\t^false","(\r\nfunction(self,aVariableNode,anExpressionNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal aName=nil\r\nlocal aClass=nil\r\n(function() if (((((aVariableNode):indices()):notEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aVariableNode):value())\r\n\taName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((self):currentClass()):instanceClass())\r\n\taClass=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aClass):classVariableNamesDo_((function(each)\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((each),aName))))==true) then return ((function()\r\n(self):visitClassVariableAssignment_in_to_(each, aClass, anExpressionNode);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(aClass):sharedPoolsAndNamesDo_((function(poolName,pool)\r\nreturn ((function() if ((((pool):includesKey_(aName)))==true) then return ((function()\r\n(self):visitPoolVariableAssignment_in_to_(aName, poolName, anExpressionNode);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitCascadeNode:",0
,(
function(self,aNode)
local target=nil
(function()
	local _s8_value=("_s8_rec")
	target=_s8_value; return _s8_value
end)()
;
(function() if (((((self):tempVariables()):includes_(target)))==true) then return nil else return ((function()
return (((self):tempVariables()):add_(target))
end))() end end)()
;
(self):compilerEvent_with_("aboutCascade:", aNode)
;
(self['@stream']):nextPutAll_("(\r\n")
;
(self):emmitFunction_arguments_doing_("", target, (function()
local index=nil
local rec=nil
(function()
	local _s8_value=((0))
	index=_s8_value; return _s8_value
end)();return (((aNode):nodes()):do_((function(each)
(function()
	local _s8_value=((index):_plus((1)))
	index=_s8_value; return _s8_value
end)();(function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()
return ((self):emmitOpenReturn_(nil))
end))() else return nil end end)();(function()
	local _s8_value=((smalltalk.VariableNode):new())
	rec=_s8_value; return _s8_value
end)();(
function(_s8_rec)
(_s8_rec):value_(target);
return ((_s8_rec):start_end_((each):start(), (each):start()))
end
)(rec);(each):receiver_(rec);(self):visit_(each);return ((function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()
(self):compilerEvent_withArguments_("returned:early:", (smalltalk.Array):with_with_(nil, false));return ((self):emmitCloseReturn_(nil))
end))() else return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_(";");
return ((_s8_rec):cr())
end
)(self['@stream']))
end))() end end)())
end)))
end))
;
(self['@stream']):nextPutAll_("\r\n)(")
;
(self):visit_((aNode):receiver())
;
(self['@stream']):nextPutAll_(")")
;
(self):compilerEvent_with_("cascade:", aNode)
;
return (self)
;
end
)
,"visiting","visitCascadeNode: aNode\r\n\r\n\t| target |\r\n\ttarget := #_s8_rec.\r\n\t(self tempVariables includes: target) ifFalse: [ self tempVariables add: target ].\r\n\tself compilerEvent: #aboutCascade: with: aNode.\r\n\tstream nextPutAll: '(\r\n'.\tself emmitFunction: '' arguments: target doing: [ | index rec |\r\n\t\tindex := 0.\r\n\t\taNode nodes do: [:each |\r\n\t\t    index := index + 1.\r\n\t\t    index = aNode nodes size ifTrue: [ self emmitOpenReturn: nil ].\r\n\t\t    rec := VariableNode new.\r\n\t\t    rec value: target; start: each start end: each start.\r\n\t\t    each receiver: rec.\r\n\t\t    self visit: each.\r\n\t\t    index = aNode nodes size ifTrue: [\r\n\t\t\t\tself\tcompilerEvent: #returned:early:\r\n\t\t\t\t\twithArguments: (Array with: nil with: false).\r\n\t\t\t\tself emmitCloseReturn: nil.\r\n\t\t    \t] ifFalse: [ stream nextPutAll: ';';cr ]\r\n\t\t].\r\n\t].\r\n\tstream nextPutAll: '\r\n)('.\tself visit: aNode receiver.\r\n\tstream nextPutAll: $).\r\n\tself compilerEvent: #cascade: with: aNode.","(\r\nfunction(self,aNode)\r\nlocal target=nil\r\n(function()\r\n\tlocal _s8_value=(\"_s8_rec\")\r\n\ttarget=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((((self):tempVariables()):includes_(target)))==true) then return nil else return ((function()\r\nreturn (((self):tempVariables()):add_(target))\r\nend))() end end)()\r\n;\r\n(self):compilerEvent_with_(\"aboutCascade:\", aNode)\r\n;\r\n(self['@stream']):nextPutAll_(\"(\\r\\n\")\r\n;\r\n(self):emmitFunction_arguments_doing_(\"\", target, (function()\r\nlocal index=nil\r\nlocal rec=nil\r\n(function()\r\n\tlocal _s8_value=((0))\r\n\tindex=_s8_value; return _s8_value\r\nend)();return (((aNode):nodes()):do_((function(each)\r\n(function()\r\n\tlocal _s8_value=((index):_plus((1)))\r\n\tindex=_s8_value; return _s8_value\r\nend)();(function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()\r\nreturn ((self):emmitOpenReturn_(nil))\r\nend))() else return nil end end)();(function()\r\n\tlocal _s8_value=((smalltalk.VariableNode):new())\r\n\trec=_s8_value; return _s8_value\r\nend)();(\r\nfunction(_s8_rec)\r\n(_s8_rec):value_(target);\r\nreturn ((_s8_rec):start_end_((each):start(), (each):start()))\r\nend\r\n)(rec);(each):receiver_(rec);(self):visit_(each);return ((function() if ((((smalltalk:isObject_equalTo_((index),((aNode):nodes()):size()))))==true) then return ((function()\r\n(self):compilerEvent_withArguments_(\"returned:early:\", (smalltalk.Array):with_with_(nil, false));return ((self):emmitCloseReturn_(nil))\r\nend))() else return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\";\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(self['@stream']))\r\nend))() end end)())\r\nend)))\r\nend))\r\n;\r\n(self['@stream']):nextPutAll_(\"\\r\\n)(\")\r\n;\r\n(self):visit_((aNode):receiver())\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\n(self):compilerEvent_with_(\"cascade:\", aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitLocalAssignment:",0
,(
function(self,aNode)
(self['@stream']):nextPutAll_("(")
;
(self):visitAssignmentNode_local_(aNode, true)
;
(self['@stream']):nextPutAll_(")")
;
return (self)
;
end
)
,"visiting","visitLocalAssignment: aNode\r\n\t\" Private - Visit a node that is an assignment in a local expression. \"\r\n\r\n\tstream nextPutAll: $(.\r\n\tself visitAssignmentNode: aNode local: true.\r\n\tstream nextPutAll: $).","(\r\nfunction(self,aNode)\r\n(self['@stream']):nextPutAll_(\"(\")\r\n;\r\n(self):visitAssignmentNode_local_(aNode, true)\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitAssignmentNode:",0
,(
function(self,aNode)
return ((self):visitAssignmentNode_local_(aNode, true))
;
end
)
,"visiting","visitAssignmentNode: aNode\r\n\t\" Private - Visit an assignment node. \"\r\n\r\n\t^self visitAssignmentNode: aNode local: true\"false\"","(\r\nfunction(self,aNode)\r\nreturn ((self):visitAssignmentNode_local_(aNode, true))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"emmitAssignment:to:local:",0
,(
function(self,assignee,right,wrap)
(function() if (((wrap))==true) then return ((function()
(self['@stream']):nextPutAll_("(function()\r\n\tlocal _s8_value=(");(self):visit_(right);(self['@stream']):nextPutAll_(")\r\n\t");(self):visit_(assignee);return ((self['@stream']):nextPutAll_("=_s8_value; return _s8_value\r\nend)()"))
end))() else return ((function()
(self):visit_(assignee);(self['@stream']):nextPutAll_("=");return ((self):visit_(right))
end))() end end)()
;
return (self)
;
end
)
,"generation","emmitAssignment: assignee to: right local: wrap\r\n\t\" Private - Emmit the code for assignment. \"\r\n\r\n\twrap ifTrue: [\r\n\t\tstream nextPutAll: '(function()\r\n\tlocal _s8_value=('.\r\n\t\tself visit: right.\r\n\t\tstream nextPutAll: ')\r\n\t'.\tself visit: assignee.\r\n\t\tstream nextPutAll: '=_s8_value; return _s8_value\r\nend)()'\r\n\t] ifFalse: [\r\n\t\tself visit: assignee.\r\n\t\tstream nextPutAll: '='.\r\n\t\tself visit: right\r\n\t]","(\r\nfunction(self,assignee,right,wrap)\r\n(function() if (((wrap))==true) then return ((function()\r\n(self['@stream']):nextPutAll_(\"(function()\\r\\n\\tlocal _s8_value=(\");(self):visit_(right);(self['@stream']):nextPutAll_(\")\\r\\n\\t\");(self):visit_(assignee);return ((self['@stream']):nextPutAll_(\"=_s8_value; return _s8_value\\r\\nend)()\"))\r\nend))() else return ((function()\r\n(self):visit_(assignee);(self['@stream']):nextPutAll_(\"=\");return ((self):visit_(right))\r\nend))() end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitAssignmentNode:local:",0
,(
function(self,aNode,__local)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local assignee=nil
(function() if ((((self):visitMacroAssignment_(aNode)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((aNode):left())
	assignee=_s8_value; return _s8_value
end)()
;
(function() if ((((self):acceptAssignment_to_((aNode):right(), assignee)))==true) then return ((function()
return ((self):emmitAssignment_to_local_(assignee, (aNode):right(), __local))
end))() else return ((function()
return ((self):compilerEvent_with_ifNotHandled_("invalidAssignment:", aNode, (function()
return ((self):error_(smalltalk:object_comma_(("Invalid assignment to "),(assignee):value())))
end)))
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitAssignmentNode: aNode local: local\r\n\r\n\t| assignee |\r\n\t(self visitMacroAssignment: aNode) ifTrue: [ ^self ].\r\n\tassignee := aNode left.\r\n\t(self acceptAssignment: aNode right to: assignee) ifTrue: [\r\n\t\tself emmitAssignment: assignee to: aNode right local: local\r\n\t] ifFalse: [\r\n\t\tself\tcompilerEvent: #invalidAssignment: with: aNode\r\n\t\t\tifNotHandled: [ self error: 'Invalid assignment to ' ,assignee value ]\r\n\t]","(\r\nfunction(self,aNode,__local)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal assignee=nil\r\n(function() if ((((self):visitMacroAssignment_(aNode)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aNode):left())\r\n\tassignee=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((self):acceptAssignment_to_((aNode):right(), assignee)))==true) then return ((function()\r\nreturn ((self):emmitAssignment_to_local_(assignee, (aNode):right(), __local))\r\nend))() else return ((function()\r\nreturn ((self):compilerEvent_with_ifNotHandled_(\"invalidAssignment:\", aNode, (function()\r\nreturn ((self):error_(smalltalk:object_comma_((\"Invalid assignment to \"),(assignee):value())))\r\nend)))\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitClassVariableAssignment:in:to:",0
,(
function(self,aClassVariableName,aClass,anExpressionNode)
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));
(_s8_rec):nextPutAll_(":classVariableAt_put_(\"");
(_s8_rec):nextPutAll_(aClassVariableName);
return ((_s8_rec):nextPutAll_("\","))
end
)(self['@stream'])
;
(self):visit_(anExpressionNode)
;
(self['@stream']):nextPutAll_(")")
;
return (self)
;
end
)
,"visiting","visitClassVariableAssignment: aClassVariableName in: aClass to: anExpressionNode\r\n\r\n\tstream\r\n\t\tnextPutAll: (self smalltalkNameFor: aClass);\r\n\t\tnextPutAll: ':classVariableAt_put_(\"';\r\n\t\tnextPutAll: aClassVariableName; nextPutAll: '\",'.\r\n\tself visit: anExpressionNode.\r\n\tstream\tnextPutAll: ')'","(\r\nfunction(self,aClassVariableName,aClass,anExpressionNode)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):smalltalkNameFor_(aClass));\r\n(_s8_rec):nextPutAll_(\":classVariableAt_put_(\\\"\");\r\n(_s8_rec):nextPutAll_(aClassVariableName);\r\nreturn ((_s8_rec):nextPutAll_(\"\\\",\"))\r\nend\r\n)(self['@stream'])\r\n;\r\n(self):visit_(anExpressionNode)\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitPoolVariableAssignment:in:to:",0
,(
function(self,aName,poolName,anExpressionNode)
(
function(_s8_rec)
(_s8_rec):nextPutAll_("smalltalk:poolDictionary_(\"");
(_s8_rec):nextPutAll_(poolName);
(_s8_rec):nextPutAll_("\"):at_put_(\"");
(_s8_rec):nextPutAll_(aName);
return ((_s8_rec):nextPutAll_("\","))
end
)(self['@stream'])
;
(self):visit_(anExpressionNode)
;
(self['@stream']):nextPutAll_(")")
;
return (self)
;
end
)
,"visiting","visitPoolVariableAssignment: aName in: poolName to: anExpressionNode\r\n\r\n\tstream\r\n\t\tnextPutAll: 'smalltalk:poolDictionary_(\"';\r\n\t\tnextPutAll: poolName; nextPutAll: '\"):at_put_(\"';\r\n\t\tnextPutAll: aName; nextPutAll: '\",'.\r\n\tself visit: anExpressionNode.\r\n\tstream\tnextPutAll: ')'","(\r\nfunction(self,aName,poolName,anExpressionNode)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\"smalltalk:poolDictionary_(\\\"\");\r\n(_s8_rec):nextPutAll_(poolName);\r\n(_s8_rec):nextPutAll_(\"\\\"):at_put_(\\\"\");\r\n(_s8_rec):nextPutAll_(aName);\r\nreturn ((_s8_rec):nextPutAll_(\"\\\",\"))\r\nend\r\n)(self['@stream'])\r\n;\r\n(self):visit_(anExpressionNode)\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitOperation:",0
,(
function(self,aNode)
(self):compilerEvent_with_("aboutOperation:", aNode)
;
(self['@stream']):nextPutAll_("(")
;
(self):visitReceiver_((aNode):receiver())
;
(self['@stream']):nextPutAll_(")")
;
(self['@stream']):nextPutAll_((self):asLuaSelector_((aNode):selector()))
;
(self):visitSendArguments_(aNode)
;
(self):compilerEvent_with_("operation:", aNode)
;
return (self)
;
end
)
,"visiting","visitOperation: aNode\r\n\t\" Private - aNode is an operation and selector is mapped to target operation. \"\r\n\r\n\tself compilerEvent: #aboutOperation: with: aNode.\r\n\tstream nextPutAll: $(.\r\n\tself visitReceiver: aNode receiver.\r\n\tstream nextPutAll: $).\r\n\tstream nextPutAll: (self asLuaSelector: aNode selector).\r\n\tself visitSendArguments: aNode.\r\n\tself compilerEvent: #operation: with: aNode.","(\r\nfunction(self,aNode)\r\n(self):compilerEvent_with_(\"aboutOperation:\", aNode)\r\n;\r\n(self['@stream']):nextPutAll_(\"(\")\r\n;\r\n(self):visitReceiver_((aNode):receiver())\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\n(self['@stream']):nextPutAll_((self):asLuaSelector_((aNode):selector()))\r\n;\r\n(self):visitSendArguments_(aNode)\r\n;\r\n(self):compilerEvent_with_(\"operation:\", aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitSendToSuper:",0
,(
function(self,aNode)
(self):compilerEvent_with_("aboutSend:", aNode)
;
(self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk:superImplementor("),(self):smalltalkNameFor_((self):currentClass()))),",\"")),(self):asLuaSelector_((aNode):selector()))),"\")(self"))
;
(function() if (((((aNode):arguments()):notEmpty()))==true) then return ((function()
(self['@stream']):nextPutAll_(",");return ((self):visitSendArguments_(aNode))
end))() else return nil end end)()
;
(self['@stream']):nextPutAll_(")")
;
(self):compilerEvent_with_("send:", aNode)
;
return (self)
;
end
)
,"visiting","visitSendToSuper: aNode\r\n\r\n\tself compilerEvent: #aboutSend: with: aNode.\r\n\tstream \r\n\t\tnextPutAll: 'smalltalk:superImplementor('\r\n\t\t\t,(self smalltalkNameFor: self currentClass)\r\n\t\t\t,',\"',(self asLuaSelector: aNode selector)\r\n\t\t\t,'\")(self'.\r\n\taNode arguments notEmpty ifTrue: [\r\n\t\tstream nextPutAll: ','.\r\n\t\tself visitSendArguments: aNode\r\n\t].\r\n\tstream nextPutAll: ')'.\r\n\tself compilerEvent: #send: with: aNode.","(\r\nfunction(self,aNode)\r\n(self):compilerEvent_with_(\"aboutSend:\", aNode)\r\n;\r\n(self['@stream']):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk:superImplementor(\"),(self):smalltalkNameFor_((self):currentClass()))),\",\\\"\")),(self):asLuaSelector_((aNode):selector()))),\"\\\")(self\"))\r\n;\r\n(function() if (((((aNode):arguments()):notEmpty()))==true) then return ((function()\r\n(self['@stream']):nextPutAll_(\",\");return ((self):visitSendArguments_(aNode))\r\nend))() else return nil end end)()\r\n;\r\n(self['@stream']):nextPutAll_(\")\")\r\n;\r\n(self):compilerEvent_with_(\"send:\", aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"visitReturnNode:",0
,(
function(self,aNode)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local theReturn=nil
(function() if (((((self['@nestedBlocks']) > (0))))==true) then return ((function()
return ((function()
	local _s8_value=(true)
	self['@earlyReturn']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((function()
return ((self):emmitReturn_((function()
return (((aNode):nodes()):do_((function(each)
return ((self):visit_(each))
end)))
end)))
end))
	theReturn=_s8_value; return _s8_value
end)()
;
(function() if (((self['@earlyReturn']))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((theReturn):value())
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(self['@stream']):nextPutAll_("(")
;
(self):emmitFunctionDoing_((function()
(self['@stream']):nextPutAll_("_s8Ret_.hasResult=true\r\n_s8Ret_.result=(");(self):emmitFunctionDoing_(theReturn);return ((self['@stream']):nextPutAll_("\r\n)()\r\nerror(_s8Ret_,0)"))
end))
;
(self['@stream']):nextPutAll_(")()")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"visiting","visitReturnNode: aNode\r\n\r\n\t| theReturn |\r\n\tnestedBlocks > 0 ifTrue: [ earlyReturn := true ].\r\n\ttheReturn := [\r\n\t\tself emmitReturn: [\r\n\t\t\taNode nodes do: [:each | self visit: each ]\r\n\t\t]\r\n\t].\r\n\tearlyReturn ifFalse: [ ^theReturn value ].\r\n\r\n\tstream nextPutAll: $(.\r\n\tself emmitFunctionDoing: [\r\n\t\tstream nextPutAll: '_s8Ret_.hasResult=true\r\n_s8Ret_.result=('.\r\n\t\tself emmitFunctionDoing: theReturn.\r\n\t\tstream nextPutAll: '\r\n)()\r\nerror(_s8Ret_,0)'\r\n\t].\r\n\tstream nextPutAll: ')()'.","(\r\nfunction(self,aNode)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal theReturn=nil\r\n(function() if (((((self['@nestedBlocks']) > (0))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(true)\r\n\tself['@earlyReturn']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function()\r\nreturn ((self):emmitReturn_((function()\r\nreturn (((aNode):nodes()):do_((function(each)\r\nreturn ((self):visit_(each))\r\nend)))\r\nend)))\r\nend))\r\n\ttheReturn=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((self['@earlyReturn']))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((theReturn):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(self['@stream']):nextPutAll_(\"(\")\r\n;\r\n(self):emmitFunctionDoing_((function()\r\n(self['@stream']):nextPutAll_(\"_s8Ret_.hasResult=true\\r\\n_s8Ret_.result=(\");(self):emmitFunctionDoing_(theReturn);return ((self['@stream']):nextPutAll_(\"\\r\\n)()\\r\\nerror(_s8Ret_,0)\"))\r\nend))\r\n;\r\n(self['@stream']):nextPutAll_(\")()\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"generate:early:return:",0
,(
function(self,fName,early,code)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((early))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self['@stream']):nextPutAll_(code))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_("\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n");
return ((_s8_rec):nextPutAll_(code))
end
)(self['@stream'])
;
(self['@stream']):nextPutAll_("\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      ")
;
(self):emmitReturn_early_("_s8Ret_.result", true)
;
(self['@stream']):nextPutAll_("\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"generation","generate: fName early: early return: code\r\n\t\" Private - Generate code to catch early return of code expression in fName. \"\r\n\r\n\tearly ifFalse: [ ^stream nextPutAll: code ].\r\n\r\n\tstream nextPutAll: '\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n';\tnextPutAll: code.\r\n\tstream nextPutAll: '\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      '.\tself emmitReturn: '_s8Ret_.result' early: true.\r\n\tstream nextPutAll: '\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n'","(\r\nfunction(self,fName,early,code)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((early))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self['@stream']):nextPutAll_(code))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\"\\r\\nlocal _s8Ret_={[\\\"name\\\"]=\\\"stReturn\\\"}\\r\\nlocal _s8_f = function()\\r\\n\");\r\nreturn ((_s8_rec):nextPutAll_(code))\r\nend\r\n)(self['@stream'])\r\n;\r\n(self['@stream']):nextPutAll_(\"\\r\\nend\\r\\nlocal _s8_status, _s8_result = pcall(_s8_f)\\r\\nif _s8_status then return _s8_result\\r\\nelse\\r\\n    if (_s8Ret_.hasResult) then\\r\\n      \")\r\n;\r\n(self):emmitReturn_early_(\"_s8Ret_.result\", true)\r\n;\r\n(self['@stream']):nextPutAll_(\"\\r\\n    end\\r\\nend\\r\\nerror(_s8_result,0)\\r\\n\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"coreOptimizations",0
,(
function(self)
return (smalltalk:newArray({"core", smalltalk:newArray({smalltalk:newArray({"class", "%receiver%[\"$klass\"]"}), smalltalk:newArray({"isNil", "((%receiver%)==nil)"}), smalltalk:newArray({"notNil", "((%receiver%)~=nil)"}), smalltalk:newArray({"=", "(smalltalk:isObject_equalTo_(%receiver%,%arg1%))"}), smalltalk:newArray({"~=", "(not (smalltalk:isObject_equalTo_(%receiver%,%arg1%)))"}), smalltalk:newArray({"==", "(%receiver%==%arg1%)"}), smalltalk:newArray({"~~", "(%receiver%~=%arg1%)"}), smalltalk:newArray({"===", "(%receiver% == %arg1%)"}), smalltalk:newArray({"||", "(%receiver% or %arg1%)"}), smalltalk:newArray({",", "smalltalk:object_comma_(%receiver%,%arg1%)"}), smalltalk:newArray({"basicAt:", "(%receiver%[%arg1%])"}), smalltalk:newArray({"basicAt:ifAbsent:", "(function() local _1_=(%receiver%[%arg1%]);if (_1_==nil) then return (%arg2%):value() end; return _1_ end)()"}), smalltalk:newArray({"basicAt:put:", "(function() local _1_=(%arg2%); %receiver%[%arg1%]=_1_; return _1_ end)()"}), smalltalk:newArray({"ifUndefined:", "(function() local _1_=(%receiver%);if (_1_==nil) then return (%arg1%):value() end; return _1_ end)()"})})}))
;
end
)
,"optimization","coreOptimizations\r\n\t\" Private - Return the optimization set. \"\r\n\r\n\t^#(core  #(\r\n\t\t#(class\t'%receiver%[\"$klass\"]' )\r\n\t\t#(isNil\t'((%receiver%)==nil)' )\r\n\t\t#(notNil\t'((%receiver%)~=nil)' )\r\n\t\t#(=\t'(smalltalk:isObject_equalTo_(%receiver%,%arg1%))' )\r\n\t\t#(~=\t'(not (smalltalk:isObject_equalTo_(%receiver%,%arg1%)))' )\r\n\t\t#(==\t'(%receiver%==%arg1%)' )\r\n\t\t#(~~\t'(%receiver%~=%arg1%)' )\r\n\t\t#(===\t'(%receiver% == %arg1%)' )\r\n\t\t#(||\t'(%receiver% or %arg1%)' )\r\n\t\t#(,\t'smalltalk:object_comma_(%receiver%,%arg1%)' )\r\n\t\t#(basicAt:\t'(%receiver%[%arg1%])' )\r\n\t\t#(basicAt:ifAbsent: '(function() local _1_=(%receiver%[%arg1%]);if (_1_==nil) then return (%arg2%):value() end; return _1_ end)()' )\r\n\t\t#(basicAt:put: '(function() local _1_=(%arg2%); %receiver%[%arg1%]=_1_; return _1_ end)()' )\r\n\t\t#(ifUndefined: '(function() local _1_=(%receiver%);if (_1_==nil) then return (%arg1%):value() end; return _1_ end)()' )\r\n\t))","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"core\", smalltalk:newArray({smalltalk:newArray({\"class\", \"%receiver%[\\\"$klass\\\"]\"}), smalltalk:newArray({\"isNil\", \"((%receiver%)==nil)\"}), smalltalk:newArray({\"notNil\", \"((%receiver%)~=nil)\"}), smalltalk:newArray({\"=\", \"(smalltalk:isObject_equalTo_(%receiver%,%arg1%))\"}), smalltalk:newArray({\"~=\", \"(not (smalltalk:isObject_equalTo_(%receiver%,%arg1%)))\"}), smalltalk:newArray({\"==\", \"(%receiver%==%arg1%)\"}), smalltalk:newArray({\"~~\", \"(%receiver%~=%arg1%)\"}), smalltalk:newArray({\"===\", \"(%receiver% == %arg1%)\"}), smalltalk:newArray({\"||\", \"(%receiver% or %arg1%)\"}), smalltalk:newArray({\",\", \"smalltalk:object_comma_(%receiver%,%arg1%)\"}), smalltalk:newArray({\"basicAt:\", \"(%receiver%[%arg1%])\"}), smalltalk:newArray({\"basicAt:ifAbsent:\", \"(function() local _1_=(%receiver%[%arg1%]);if (_1_==nil) then return (%arg2%):value() end; return _1_ end)()\"}), smalltalk:newArray({\"basicAt:put:\", \"(function() local _1_=(%arg2%); %receiver%[%arg1%]=_1_; return _1_ end)()\"}), smalltalk:newArray({\"ifUndefined:\", \"(function() local _1_=(%receiver%);if (_1_==nil) then return (%arg1%):value() end; return _1_ end)()\"})})}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"s8Optimizations",0
,(
function(self)
return (smalltalk:newArray({"s8", smalltalk:newArray({"core", smalltalk:newArray({"ifTrue:", "hasBlockNoArguments:", "(function() if ((%receiver%)==true) then return (%arg1%)() else return nil end end)()"}), smalltalk:newArray({"ifFalse:", "hasBlockNoArguments:", "(function() if ((%receiver%)==true) then return nil else return (%arg1%)() end end)()"}), smalltalk:newArray({"ifTrue:ifFalse:", "hasBlockNoArguments:", "(function() if ((%receiver%)==true) then return (%arg1%)() else return (%arg2%)() end end)()"}), smalltalk:newArray({"ifFalse:ifTrue:", "hasBlockNoArguments:", "(function() if ((%receiver%)==true) then return (%arg2%)() else return (%arg1%)() end end)()"}), smalltalk:newArray({"whileTrue", "receiverIsBlockNoArguments:", "(function() while(%receiver%()) do end;return nil end)()"}), smalltalk:newArray({"whileFalse", "receiverIsBlockNoArguments:", "(function() while(not %receiver%()) do end;return nil end)()"}), smalltalk:newArray({"orNil", "(function() local _1_=(%receiver%); if (_1_==nil) then return nil end; return _1_ end)()"}), smalltalk:newArray({"whileTrue:", "receiverIsBlockNoArguments:", "(function() while(%receiver%()) do %arg1%() end;return nil end)()"}), smalltalk:newArray({"whileFalse:", "receiverIsBlockNoArguments:", "(function() while(not %receiver%()) do %arg1%() end;return nil end)()"}), smalltalk:newArray({"timesRepeat:", "(function() for _1_=1,(%receiver%) do %arg1%(_1_) end;return nil end)()"}), smalltalk:newArray({"isKindOf:", "smalltalk:is_kindOf_(%receiver%,(%arg1%))"}), smalltalk:newArray({"respondsTo:", "smalltalk:responds_to_(%receiver%,(%arg1%))"}), smalltalk:newArray({"ifNil:", "(function() local _1_=(%receiver%);if (_1_==nil) then return (%arg1%):value() end; return _1_ end)()"}), smalltalk:newArray({"ifNotNil:", "(function() local _1_=(%receiver%);if (_1_~=nil) then return (%arg1%):value() end; return _1_ end)()"}), smalltalk:newArray({"ifNil:ifNotNil:", "(function() if ((%receiver%)==nil) then return (%arg1%):value() end; return (%arg2%):value() end)()"}), smalltalk:newArray({"ifNotNil:ifNil:", "(function() if ((%receiver%)~=nil) then return (%arg1%):value() end; return (%arg2%):value() end)()"})})}))
;
end
)
,"optimization","s8Optimizations\r\n\t\" Private - Return the optimization set. \"\r\n\r\n\t^#(s8\t#(\r\n\t\tcore\r\n\t\t#(ifTrue: hasBlockNoArguments: '(function() if ((%receiver%)==true) then return (%arg1%)() else return nil end end)()' )\r\n\t\t#(ifFalse: hasBlockNoArguments: '(function() if ((%receiver%)==true) then return nil else return (%arg1%)() end end)()' )\r\n\t\t#(ifTrue:ifFalse: hasBlockNoArguments: '(function() if ((%receiver%)==true) then return (%arg1%)() else return (%arg2%)() end end)()' )\r\n\t\t#(ifFalse:ifTrue: hasBlockNoArguments: '(function() if ((%receiver%)==true) then return (%arg2%)() else return (%arg1%)() end end)()' )\r\n\t\t#(whileTrue receiverIsBlockNoArguments: '(function() while(%receiver%()) do end;return nil end)()' )\r\n\t\t#(whileFalse receiverIsBlockNoArguments: '(function() while(not %receiver%()) do end;return nil end)()' )\r\n\t\t#(orNil '(function() local _1_=(%receiver%); if (_1_==nil) then return nil end; return _1_ end)()' )\r\n\t\t#(whileTrue: receiverIsBlockNoArguments: \"allBlockNoArguments:\" '(function() while(%receiver%()) do %arg1%() end;return nil end)()' )\r\n\t\t#(whileFalse: receiverIsBlockNoArguments: \"allBlockNoArguments:\" '(function() while(not %receiver%()) do %arg1%() end;return nil end)()' )\r\n\t\t#(timesRepeat: '(function() for _1_=1,(%receiver%) do %arg1%(_1_) end;return nil end)()' )\r\n\t\t#(isKindOf: 'smalltalk:is_kindOf_(%receiver%,(%arg1%))' )\r\n\t\t#(respondsTo: 'smalltalk:responds_to_(%receiver%,(%arg1%))' )\r\n\t\t#(ifNil: '(function() local _1_=(%receiver%);if (_1_==nil) then return (%arg1%):value() end; return _1_ end)()' )\r\n\t\t#(ifNotNil: '(function() local _1_=(%receiver%);if (_1_~=nil) then return (%arg1%):value() end; return _1_ end)()' )\r\n\t\t#(ifNil:ifNotNil: '(function() if ((%receiver%)==nil) then return (%arg1%):value() end; return (%arg2%):value() end)()' )\r\n\t\t#(ifNotNil:ifNil: '(function() if ((%receiver%)~=nil) then return (%arg1%):value() end; return (%arg2%):value() end)()' )\r\n\t))","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"s8\", smalltalk:newArray({\"core\", smalltalk:newArray({\"ifTrue:\", \"hasBlockNoArguments:\", \"(function() if ((%receiver%)==true) then return (%arg1%)() else return nil end end)()\"}), smalltalk:newArray({\"ifFalse:\", \"hasBlockNoArguments:\", \"(function() if ((%receiver%)==true) then return nil else return (%arg1%)() end end)()\"}), smalltalk:newArray({\"ifTrue:ifFalse:\", \"hasBlockNoArguments:\", \"(function() if ((%receiver%)==true) then return (%arg1%)() else return (%arg2%)() end end)()\"}), smalltalk:newArray({\"ifFalse:ifTrue:\", \"hasBlockNoArguments:\", \"(function() if ((%receiver%)==true) then return (%arg2%)() else return (%arg1%)() end end)()\"}), smalltalk:newArray({\"whileTrue\", \"receiverIsBlockNoArguments:\", \"(function() while(%receiver%()) do end;return nil end)()\"}), smalltalk:newArray({\"whileFalse\", \"receiverIsBlockNoArguments:\", \"(function() while(not %receiver%()) do end;return nil end)()\"}), smalltalk:newArray({\"orNil\", \"(function() local _1_=(%receiver%); if (_1_==nil) then return nil end; return _1_ end)()\"}), smalltalk:newArray({\"whileTrue:\", \"receiverIsBlockNoArguments:\", \"(function() while(%receiver%()) do %arg1%() end;return nil end)()\"}), smalltalk:newArray({\"whileFalse:\", \"receiverIsBlockNoArguments:\", \"(function() while(not %receiver%()) do %arg1%() end;return nil end)()\"}), smalltalk:newArray({\"timesRepeat:\", \"(function() for _1_=1,(%receiver%) do %arg1%(_1_) end;return nil end)()\"}), smalltalk:newArray({\"isKindOf:\", \"smalltalk:is_kindOf_(%receiver%,(%arg1%))\"}), smalltalk:newArray({\"respondsTo:\", \"smalltalk:responds_to_(%receiver%,(%arg1%))\"}), smalltalk:newArray({\"ifNil:\", \"(function() local _1_=(%receiver%);if (_1_==nil) then return (%arg1%):value() end; return _1_ end)()\"}), smalltalk:newArray({\"ifNotNil:\", \"(function() local _1_=(%receiver%);if (_1_~=nil) then return (%arg1%):value() end; return _1_ end)()\"}), smalltalk:newArray({\"ifNil:ifNotNil:\", \"(function() if ((%receiver%)==nil) then return (%arg1%):value() end; return (%arg2%):value() end)()\"}), smalltalk:newArray({\"ifNotNil:ifNil:\", \"(function() if ((%receiver%)~=nil) then return (%arg1%):value() end; return (%arg2%):value() end)()\"})})}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"magnitudeOptimizations",0
,(
function(self)
return (smalltalk:newArray({"magnitude", smalltalk:newArray({smalltalk:newArray({">", "(%receiver% > %arg1%)"}), smalltalk:newArray({"<", "(%receiver% < %arg1%)"}), smalltalk:newArray({">=", "(%receiver% >= %arg1%)"}), smalltalk:newArray({"<=", "(%receiver% <= %arg1%)"})})}))
;
end
)
,"optimization","magnitudeOptimizations\r\n\t\" Private - Return the optimization set. \"\r\n\r\n\t^#(magnitude #(\r\n\t\t#(> \t'(%receiver% > %arg1%)' )\r\n\t\t#(< \t'(%receiver% < %arg1%)' )\r\n\t\t#(>=\t'(%receiver% >= %arg1%)' )\r\n\t\t#(<=\t'(%receiver% <= %arg1%)' )\r\n\t))","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"magnitude\", smalltalk:newArray({smalltalk:newArray({\">\", \"(%receiver% > %arg1%)\"}), smalltalk:newArray({\"<\", \"(%receiver% < %arg1%)\"}), smalltalk:newArray({\">=\", \"(%receiver% >= %arg1%)\"}), smalltalk:newArray({\"<=\", \"(%receiver% <= %arg1%)\"})})}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"mathOptimizations",0
,(
function(self)
return (smalltalk:newArray({"math", smalltalk:newArray({smalltalk:newArray({"max:", "smalltalk:math_max_(%receiver%,%arg1%)"}), smalltalk:newArray({"min:", "smalltalk:math_min_(%receiver%,%arg1%)"}), smalltalk:newArray({"mustBeNumber", "smalltalk:mustBeNumber_(%receiver%)"}), smalltalk:newArray({"+", "(%receiver% + %arg1%)"}), smalltalk:newArray({"-", "(%receiver% - %arg1%)"}), smalltalk:newArray({"*", "(%receiver% * %arg1%)"}), smalltalk:newArray({"/", "(%receiver% / %arg1%)"})})}))
;
end
)
,"optimization","mathOptimizations\r\n\t\" Private - Return the optimization set. \"\r\n\r\n\t^#(math #(\r\n\t\t#(max:\t'smalltalk:math_max_(%receiver%,%arg1%)' )\r\n\t\t#(min:\t'smalltalk:math_min_(%receiver%,%arg1%)' )\r\n\t\t#(mustBeNumber\t'smalltalk:mustBeNumber_(%receiver%)' )\r\n\t\t#(+\t'(%receiver% + %arg1%)' )\r\n\t\t#(#-\t'(%receiver% - %arg1%)' )\r\n\t\t#(*\t'(%receiver% * %arg1%)' )\r\n\t\t#(/\t'(%receiver% / %arg1%)' )\r\n\t))","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"math\", smalltalk:newArray({smalltalk:newArray({\"max:\", \"smalltalk:math_max_(%receiver%,%arg1%)\"}), smalltalk:newArray({\"min:\", \"smalltalk:math_min_(%receiver%,%arg1%)\"}), smalltalk:newArray({\"mustBeNumber\", \"smalltalk:mustBeNumber_(%receiver%)\"}), smalltalk:newArray({\"+\", \"(%receiver% + %arg1%)\"}), smalltalk:newArray({\"-\", \"(%receiver% - %arg1%)\"}), smalltalk:newArray({\"*\", \"(%receiver% * %arg1%)\"}), smalltalk:newArray({\"/\", \"(%receiver% / %arg1%)\"})})}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"printingOptimizations",0
,(
function(self)
return (smalltalk:newArray({"printing", smalltalk:newArray({smalltalk:newArray({"toString", "smalltalk:toString_(%receiver%)"}), smalltalk:newArray({"printString", "smalltalk:printString_(%receiver%)"}), smalltalk:newArray({"asString", "smalltalk:asString_(%receiver%)"}), smalltalk:newArray({"asCharacter", "smalltalk:asCharacter_(%receiver%)"})})}))
;
end
)
,"optimization","printingOptimizations\r\n\t\" Private - Return the optimization set. \"\r\n\r\n\t^#(printing #(\r\n\t\t#(toString\t'smalltalk:toString_(%receiver%)' )\r\n\t\t#(printString\t'smalltalk:printString_(%receiver%)' )\r\n\t\t#(asString\t'smalltalk:asString_(%receiver%)' )\r\n\t\t#(asCharacter\t'smalltalk:asCharacter_(%receiver%)' )\r\n\t))","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"printing\", smalltalk:newArray({smalltalk:newArray({\"toString\", \"smalltalk:toString_(%receiver%)\"}), smalltalk:newArray({\"printString\", \"smalltalk:printString_(%receiver%)\"}), smalltalk:newArray({\"asString\", \"smalltalk:asString_(%receiver%)\"}), smalltalk:newArray({\"asCharacter\", \"smalltalk:asCharacter_(%receiver%)\"})})}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"booleanOptimizations",0
,(
function(self)
return (smalltalk:newArray({"boolean", smalltalk:newArray({smalltalk:newArray({"isBoolean", "(type(%receiver%)==\"boolean\")"}), smalltalk:newArray({"and:", "hasBlockNoArguments:", "(smalltalk:boolean_and_(%receiver%,%arg1%))"}), smalltalk:newArray({"or:", "hasBlockNoArguments:", "(smalltalk:boolean_or_(%receiver%,%arg1%))"}), smalltalk:newArray({"not", "(smalltalk:booleanNot_(%receiver%))"})})}))
;
end
)
,"optimization","booleanOptimizations\r\n\t\" Private - Return the optimization set. \"\r\n\r\n\t^#(boolean #(\r\n\t\t#(isBoolean '(type(%receiver%)==\"boolean\")' )\r\n\t\t#(and: hasBlockNoArguments: '(smalltalk:boolean_and_(%receiver%,%arg1%))' )\r\n\t\t#(or: hasBlockNoArguments:  '(smalltalk:boolean_or_(%receiver%,%arg1%))' )\r\n\t\t#(not '(smalltalk:booleanNot_(%receiver%))' )\r\n\t))","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"boolean\", smalltalk:newArray({smalltalk:newArray({\"isBoolean\", \"(type(%receiver%)==\\\"boolean\\\")\"}), smalltalk:newArray({\"and:\", \"hasBlockNoArguments:\", \"(smalltalk:boolean_and_(%receiver%,%arg1%))\"}), smalltalk:newArray({\"or:\", \"hasBlockNoArguments:\", \"(smalltalk:boolean_or_(%receiver%,%arg1%))\"}), smalltalk:newArray({\"not\", \"(smalltalk:booleanNot_(%receiver%))\"})})}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"evaluationOptimizations",0
,(
function(self)
return (smalltalk:newArray({"evaluation", smalltalk:newArray({"value", "value_", "value_value_", "value_value_value_", "value_value_value_value_", "valueWithArguments_", "evaluate", "evaluateWith_", "evaluateWithArguments_"})}))
;
end
)
,"optimization","evaluationOptimizations\r\n\t\" Private - Return the optimization set. \"\r\n\r\n\t^#(evaluation #(\r\n\"\t\t#(value 'smalltalk:block_value_sending_(%receiver%,{},\"value\")' )\r\n\t\t#(value: 'smalltalk:block_value_sending_(%receiver%,{%arg1%},\"value_\")' )\r\n\t\t#(value:value: 'smalltalk:block_value_sending_(%receiver%,{%arg1%,%arg2%},\"value_value_\")' )\r\n\t\t#(value:value:value: 'smalltalk:block_value_sending_(%receiver%,{%arg1%,%arg2%,%arg3%},\"value_value_value_\")' )\r\n\t\t#(value:value:value:value: 'smalltalk:block_value_sending_(%receiver%,{%arg1%,%arg2%,%arg3%,%arg4%},\"value_value_value_value_\")' )\r\n\t\t#(valueWithArguments: 'smalltalk:block_value_sending_(%receiver%,{%arg1%},\"valueWithArguments_\")' )\r\n\t\t#(evaluate 'smalltalk:block_value_sending_(%receiver%,{},\"evaluate\")' )\r\n\t\t#(evaluateWith: 'smalltalk:block_value_sending_(%receiver%,{%arg1%},\"evaluateWith_\")' )\r\n\t\t#(evaluateWithArguments: 'smalltalk:block_value_sending_(%receiver%,{%arg1%},\"evaluateWithArguments_\")' )\r\n\"\t))","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"evaluation\", smalltalk:newArray({\"value\", \"value_\", \"value_value_\", \"value_value_value_\", \"value_value_value_value_\", \"valueWithArguments_\", \"evaluate\", \"evaluateWith_\", \"evaluateWithArguments_\"})}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"defaultKnownOptimizations",0
,(
function(self)
return ((
function(_s8_rec)
(_s8_rec):add_((self):s8Optimizations());
(_s8_rec):add_((self):magnitudeOptimizations());
(_s8_rec):add_((self):booleanOptimizations());
(_s8_rec):add_((self):printingOptimizations());
return ((_s8_rec):yourself())
end
)((smalltalk.Array):with_((self):coreOptimizations())))
;
end
)
,"defaults","defaultKnownOptimizations\r\n\t\" Private - Returns the default knownOptimizations of the receiver. \"\r\n\r\n\t^(Array\twith: self coreOptimizations)\r\n\t\tadd: self s8Optimizations;\r\n\t\tadd: self magnitudeOptimizations;\r\n\"\t\tadd: self mathOptimizations;\"\r\n\t\tadd: self booleanOptimizations;\r\n\"\t\tadd: self evaluationOptimizations;\"\r\n\t\tadd: self printingOptimizations;\r\n\t\tyourself","(\r\nfunction(self)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_((self):s8Optimizations());\r\n(_s8_rec):add_((self):magnitudeOptimizations());\r\n(_s8_rec):add_((self):booleanOptimizations());\r\n(_s8_rec):add_((self):printingOptimizations());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.Array):with_((self):coreOptimizations())))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.LuaEncoder,"defaultOptimizations",0
,(
function(self)
return (smalltalk:newArray({"s8", "magnitude", "boolean", "printing"}))
;
end
)
,"defaults","defaultOptimizations\r\n\t\" Private - Returns the default optimizations of the receiver. \"\r\n\r\n\t^#( s8 magnitude \"math\" boolean \"evaluation\" printing )","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"s8\", \"magnitude\", \"boolean\", \"printing\"}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"constructEventsTriggered",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:newArray({"needsCookie:", "startVisit:", "endVisit:", "startMethod:", "enterMethod:", "endMethod:", "startFunction:with:", "startFunctionName:", "startFunctionBody:with:", "endFunction:", "aboutReturn:", "return:", "returned:early:", "aboutSend:", "send:", "aboutOperation:", "operation:", "aboutCascade:", "cascade:", "aboutToCompile:", "setTransformers:", "needsParser", "aboutToParse:", "needsEncoder", "aboutToEncode:", "aboutToRecompile:", "recompiled:", "startCompiling:", "endCompiling:", "needsOptimizationMap", "initializeOptimizations", "customizeOptimizations", "unknownVariable:", "acceptAssignmentTo:", "invalidAssignment:"})),smalltalk:superImplementor(smalltalk.Compiler["$klass"],"constructEventsTriggered")(self)))
;
end
)
,"events","constructEventsTriggered\r\n\t\" Private - Return the events managed by instances of the receiver. \"\r\n\r\n\t^#(\t#needsCookie: #startVisit: #endVisit:\r\n\t\t#startMethod: #enterMethod: #endMethod:\r\n\t\t#startFunction:with: #startFunctionName: #startFunctionBody:with: #endFunction:\r\n\t\t#aboutReturn: #return: #returned:early:\r\n\t\t#aboutSend: #send:\r\n\t\t#aboutOperation: #operation:\r\n\t\t#aboutCascade: #cascade:\r\n\t\t#aboutToCompile: #setTransformers:\r\n\t\t#needsParser #aboutToParse:\r\n\t\t#needsEncoder #aboutToEncode:\r\n\t\t#aboutToRecompile: #recompiled:\r\n\t\t#startCompiling: #endCompiling:\r\n\t\t#needsOptimizationMap #initializeOptimizations #customizeOptimizations\r\n\t\t#unknownVariable: #acceptAssignmentTo: #invalidAssignment:\r\n\t) ,super constructEventsTriggered","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:newArray({\"needsCookie:\", \"startVisit:\", \"endVisit:\", \"startMethod:\", \"enterMethod:\", \"endMethod:\", \"startFunction:with:\", \"startFunctionName:\", \"startFunctionBody:with:\", \"endFunction:\", \"aboutReturn:\", \"return:\", \"returned:early:\", \"aboutSend:\", \"send:\", \"aboutOperation:\", \"operation:\", \"aboutCascade:\", \"cascade:\", \"aboutToCompile:\", \"setTransformers:\", \"needsParser\", \"aboutToParse:\", \"needsEncoder\", \"aboutToEncode:\", \"aboutToRecompile:\", \"recompiled:\", \"startCompiling:\", \"endCompiling:\", \"needsOptimizationMap\", \"initializeOptimizations\", \"customizeOptimizations\", \"unknownVariable:\", \"acceptAssignmentTo:\", \"invalidAssignment:\"})),smalltalk:superImplementor(smalltalk.Compiler[\"$klass\"],\"constructEventsTriggered\")(self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"debugCookieFrom:",0
,(
function(self,aStream)
local nextOffset=nil
(function()
	local _s8_value=((function()
return (((aStream):nextChunk()):asNumber())
end))
	nextOffset=_s8_value; return _s8_value
end)()
;
return ((smalltalk.Array):with_with_with_with_((aStream):nextChunk(), ((nextOffset):value()):_at((nextOffset):value()), ((nextOffset):value()):_at((nextOffset):value()), ((aStream):nextChunk()):asArrayOfSubstrings()))
;
end
)
,"private","debugCookieFrom: aStream\r\n\t\" Private - Return the next debug cookie encoded in aStream (or nil). \"\r\n\r\n\t| nextOffset |\r\n\tnextOffset := [ aStream nextChunk asNumber ].\r\n\t^Array\t\"ref. #debugInfo:forClass: \"\r\n\t\twith: aStream nextChunk\r\n\t\twith: nextOffset value @ nextOffset value\r\n\t\twith: nextOffset value @ nextOffset value\r\n\t\twith: aStream nextChunk asArrayOfSubstrings","(\r\nfunction(self,aStream)\r\nlocal nextOffset=nil\r\n(function()\r\n\tlocal _s8_value=((function()\r\nreturn (((aStream):nextChunk()):asNumber())\r\nend))\r\n\tnextOffset=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((smalltalk.Array):with_with_with_with_((aStream):nextChunk(), ((nextOffset):value()):_at((nextOffset):value()), ((nextOffset):value()):_at((nextOffset):value()), ((aStream):nextChunk()):asArrayOfSubstrings()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"debugCookiesFrom:",0
,(
function(self,aStream)
local result=nil
local cookie=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	result=_s8_value; return _s8_value
end)()
;
(function() while(not ((function()
return ((
function(_s8_rec)
(_s8_rec):skipSeparators();
return ((_s8_rec):atEnd())
end
)(aStream))
end))()) do (function()
(function()
	local _s8_value=((self):debugCookieFrom_(aStream))
	cookie=_s8_value; return _s8_value
end)();return ((function() if ((((((cookie))~=nil)))==true) then return ((function()
return ((result):add_(cookie))
end))() else return nil end end)())
end)() end;return nil end)()
;
return (result)
;
end
)
,"private","debugCookiesFrom: aStream\r\n\t\" Private - Return the debug cookies encoded in aStream. \"\r\n\r\n\t| result cookie |\r\n\tresult := Array new.\r\n\t[ aStream skipSeparators; atEnd ] whileFalse: [\r\n\t\tcookie := self debugCookieFrom: aStream.\r\n\t\tcookie notNil ifTrue: [ result add: cookie ].\r\n\t].\r\n\t^result","(\r\nfunction(self,aStream)\r\nlocal result=nil\r\nlocal cookie=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(not ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):skipSeparators();\r\nreturn ((_s8_rec):atEnd())\r\nend\r\n)(aStream))\r\nend))()) do (function()\r\n(function()\r\n\tlocal _s8_value=((self):debugCookieFrom_(aStream))\r\n\tcookie=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((cookie))~=nil)))==true) then return ((function()\r\nreturn ((result):add_(cookie))\r\nend))() else return nil end end)())\r\nend)() end;return nil end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"debugInfo:",0
,(
function(self,aMethod)
return ((aMethod):basicAt_ifAbsentPut_("$debugInfo", (function()
return ((self):debugInfo_forClass_((aMethod):source(), (aMethod):classField()))
end)))
;
end
)
,"debug","debugInfo: aMethod\r\n\t\" Return the debug information of aMethod. \"\r\n\r\n\t^aMethod basicAt: '$debugInfo' ifAbsentPut: [\r\n\t\tself debugInfo: aMethod source forClass: aMethod classField\r\n\t]","(\r\nfunction(self,aMethod)\r\nreturn ((aMethod):basicAt_ifAbsentPut_(\"$debugInfo\", (function()\r\nreturn ((self):debugInfo_forClass_((aMethod):source(), (aMethod):classField()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"debugInfo:forClass:",0
,(
function(self,aString,aClass)
local compiler=nil
local output=nil
local cookies=nil
local aNode=nil
(function()
	local _s8_value=((""):writeStream())
	output=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(smalltalk:newArray({}))
	cookies=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):new())
	compiler=_s8_value; return _s8_value
end)()
;
(
function(_s8_rec)
(_s8_rec):useTemplates_(false);
(_s8_rec):when_do_("needsCookie:", (function(aNode)
return ((cookies):add_((smalltalk.Array):with_with_(aNode, ((compiler):encoder()):position())))
end));
return ((_s8_rec):when_do_("endVisit:", (function(cookie)
return ((cookie):add_(((compiler):encoder()):position()))
end)))
end
)(compiler)
;
(
function(_s8_rec)
(_s8_rec):nextChunkPut_((aClass):name());
(_s8_rec):nextChunkPut_(aString);
(_s8_rec):nextChunkPut_((compiler):compile_forClass_(aString, aClass));
return ((_s8_rec):yourself())
end
)(output)
;
(cookies):do_((function(tuple)
(function()
	local _s8_value=((tuple):first())
	aNode=_s8_value; return _s8_value
end)();(
function(_s8_rec)
(_s8_rec):nextChunkPut_(((aNode)["$klass"]):name());
(_s8_rec):nextChunkPut_(smalltalk:asString_(((aNode):start())));
(_s8_rec):nextChunkPut_(smalltalk:asString_(((aNode):__end())));
(_s8_rec):nextChunkPut_(smalltalk:asString_(((tuple):second())));
(_s8_rec):nextChunkPut_(smalltalk:asString_(((tuple):third())));
return ((_s8_rec):yourself())
end
)(output);((aNode):bindings()):do_((function(each)
return ((output):nextPutAll_(smalltalk:object_comma_((" "),each)))
end));return ((output):nextChunkPut_(" "))
end))
;
return ((output):contents())
;
end
)
,"debug","debugInfo: aString forClass: aClass\r\n\t\" Private - Return aString with information of compiling aString as a method implemented in aClass. \"\r\n\r\n\t| compiler output cookies aNode |\r\n\toutput := '' writeStream.\r\n\tcookies := #().\r\n\tcompiler := self new.\r\n\tcompiler\r\n\t\tuseTemplates: false;\r\n\t\twhen: #needsCookie: do: [:aNode| cookies add: (Array with: aNode with: compiler encoder position) ];\r\n\t\twhen: #endVisit: do: [:cookie| cookie add: compiler encoder position ].\r\n\toutput\r\n\t\tnextChunkPut: aClass name;\r\n\t\tnextChunkPut: aString;\r\n\t\tnextChunkPut: (compiler compile: aString forClass: aClass);\r\n\t\tyourself.\r\n\tcookies do: [:tuple|\r\n\t\taNode := tuple first.\r\n\t\toutput\r\n\t\t\tnextChunkPut: aNode class name;\r\n\t\t\tnextChunkPut: aNode start asString;\r\n\t\t\tnextChunkPut: aNode end asString;\r\n\t\t\tnextChunkPut: tuple second asString;\r\n\t\t\tnextChunkPut: tuple third asString;\r\n\t\t\tyourself.\r\n\t\taNode bindings do: [:each| output nextPutAll: ' ' ,each ].\r\n\t\toutput\tnextChunkPut: ' '.\r\n\t].\r\n\t^output contents","(\r\nfunction(self,aString,aClass)\r\nlocal compiler=nil\r\nlocal output=nil\r\nlocal cookies=nil\r\nlocal aNode=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\toutput=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tcookies=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):new())\r\n\tcompiler=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):useTemplates_(false);\r\n(_s8_rec):when_do_(\"needsCookie:\", (function(aNode)\r\nreturn ((cookies):add_((smalltalk.Array):with_with_(aNode, ((compiler):encoder()):position())))\r\nend));\r\nreturn ((_s8_rec):when_do_(\"endVisit:\", (function(cookie)\r\nreturn ((cookie):add_(((compiler):encoder()):position()))\r\nend)))\r\nend\r\n)(compiler)\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextChunkPut_((aClass):name());\r\n(_s8_rec):nextChunkPut_(aString);\r\n(_s8_rec):nextChunkPut_((compiler):compile_forClass_(aString, aClass));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(output)\r\n;\r\n(cookies):do_((function(tuple)\r\n(function()\r\n\tlocal _s8_value=((tuple):first())\r\n\taNode=_s8_value; return _s8_value\r\nend)();(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextChunkPut_(((aNode)[\"$klass\"]):name());\r\n(_s8_rec):nextChunkPut_(smalltalk:asString_(((aNode):start())));\r\n(_s8_rec):nextChunkPut_(smalltalk:asString_(((aNode):__end())));\r\n(_s8_rec):nextChunkPut_(smalltalk:asString_(((tuple):second())));\r\n(_s8_rec):nextChunkPut_(smalltalk:asString_(((tuple):third())));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(output);((aNode):bindings()):do_((function(each)\r\nreturn ((output):nextPutAll_(smalltalk:object_comma_((\" \"),each)))\r\nend));return ((output):nextChunkPut_(\" \"))\r\nend))\r\n;\r\nreturn ((output):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"debugInfoFrom:",0
,(
function(self,aString)
return ((self):debugInfoFromStream_((aString):stream()))
;
end
)
,"debug","debugInfoFrom: aString\r\n\t\" Private - Return the debug information decoding aString contents.\r\n\tNote: aString must be result of sending #getInfo: to the receiver.\r\n\t\"\r\n\r\n\t^self debugInfoFromStream: aString stream","(\r\nfunction(self,aString)\r\nreturn ((self):debugInfoFromStream_((aString):stream()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"debugInfoFromStream:",0
,(
function(self,aStream)
return ((smalltalk.Array):with_with_with_with_((aStream):nextChunk(), (aStream):nextChunk(), (aStream):nextChunk(), (self):debugCookiesFrom_(aStream)))
;
end
)
,"private","debugInfoFromStream: aStream\r\n\t\" Private - Return the debug information encoded in aStream. \"\r\n\r\n\t^Array\t\"ref. #debugInfo:forClass: \"\r\n\t\twith: aStream nextChunk\r\n\t\twith: aStream nextChunk\r\n\t\twith: aStream nextChunk\r\n\t\twith: (self debugCookiesFrom: aStream)","(\r\nfunction(self,aStream)\r\nreturn ((smalltalk.Array):with_with_with_with_((aStream):nextChunk(), (aStream):nextChunk(), (aStream):nextChunk(), (self):debugCookiesFrom_(aStream)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"doIt:",0
,(
function(self,smalltalkExpression)
return ((self):doIt_to_(smalltalkExpression, nil))
;
end
)
,"evaluation","doIt: smalltalkExpression\r\n\t\" Return the result of evaluating the smalltalk expression in the global context or a description of an error situation occurred evaluating the code. \"\r\n\r\n\t^self doIt: smalltalkExpression to: nil","(\r\nfunction(self,smalltalkExpression)\r\nreturn ((self):doIt_to_(smalltalkExpression, nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"doIt:to:",0
,(
function(self,smalltalkExpression,aReceiver)
local result=nil
((function()
return ((function()
	local _s8_value=((function() if ((((((aReceiver))==nil)))==true) then return ((function()
return ((self):evaluate_(smalltalkExpression))
end))() else return ((function()
return ((self):evaluate_to_(smalltalkExpression, aReceiver))
end))() end end)())
	result=_s8_value; return _s8_value
end)())
end)):on_do_(smalltalk.Error, (function(error)
(function()
	local _s8_value=(error)
	result=_s8_value; return _s8_value
end)();(function() if ((((smalltalk:boolean_and_(((self):isObject_(error)),(function()
return ((smalltalk:booleanNot_(((error):isString()))))
end)))))==true) then return ((function()
return ((function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((((error)["$klass"]):name()),": ")),(error):messageText()))
	result=_s8_value; return _s8_value
end)())
end))() else return nil end end)();return ((self):note_(result))
end))
;
return (result)
;
end
)
,"evaluation","doIt: smalltalkExpression to: aReceiver\r\n\t\" Return the result of evaluating the smalltalk expression in the context of aReceiver or a description of an error situation occurred evaluating the code. \"\r\n\r\n\t| result |\r\n\t[ result := aReceiver isNil\r\n\t\t\tifTrue:  [ self evaluate: smalltalkExpression ]\r\n\t\t\tifFalse: [ self evaluate: smalltalkExpression to: aReceiver ]\r\n\t] on: Error do: [:error|\r\n\t\tresult := error.\r\n\t\t((self isObject: error) and: [ error isString not ])\r\n\t\tifTrue: [ result := error class name ,': ' ,error messageText ].\r\n\t\tself note: result\r\n\t].\r\n\t^result","(\r\nfunction(self,smalltalkExpression,aReceiver)\r\nlocal result=nil\r\n((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((function() if ((((((aReceiver))==nil)))==true) then return ((function()\r\nreturn ((self):evaluate_(smalltalkExpression))\r\nend))() else return ((function()\r\nreturn ((self):evaluate_to_(smalltalkExpression, aReceiver))\r\nend))() end end)())\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend)):on_do_(smalltalk.Error, (function(error)\r\n(function()\r\n\tlocal _s8_value=(error)\r\n\tresult=_s8_value; return _s8_value\r\nend)();(function() if ((((smalltalk:boolean_and_(((self):isObject_(error)),(function()\r\nreturn ((smalltalk:booleanNot_(((error):isString()))))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((((error)[\"$klass\"]):name()),\": \")),(error):messageText()))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)();return ((self):note_(result))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"evaluate:",0
,(
function(self,smalltalkExpression)
return (((self):new()):evaluate_(smalltalkExpression))
;
end
)
,"evaluation","evaluate: smalltalkExpression\r\n\t\" Return the result of evaluating the smalltalk expression in a global context. \"\r\n\r\n\t^self new evaluate: smalltalkExpression","(\r\nfunction(self,smalltalkExpression)\r\nreturn (((self):new()):evaluate_(smalltalkExpression))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"evaluate:in:to:",0
,(
function(self,expression,aClass,aReceiver)
return (((aClass):compiler()):evaluate_in_to_(expression, aClass, aReceiver))
;
end
)
,"evaluation","evaluate: expression in: aClass to: aReceiver\r\n\t\" Return the result of evaluating the expression with aReceiver. \"\r\n\r\n\t^aClass compiler evaluate: expression in: aClass to: aReceiver","(\r\nfunction(self,expression,aClass,aReceiver)\r\nreturn (((aClass):compiler()):evaluate_in_to_(expression, aClass, aReceiver))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"evaluate:to:",0
,(
function(self,expression,aReceiver)
return ((self):evaluate_in_to_(expression, (aReceiver)["$klass"], aReceiver))
;
end
)
,"evaluation","evaluate: expression to: aReceiver\r\n\t\" Return the result of evaluating the expression with aReceiver. \"\r\n\r\n\t^self evaluate: expression in: aReceiver class to: aReceiver","(\r\nfunction(self,expression,aReceiver)\r\nreturn ((self):evaluate_in_to_(expression, (aReceiver)[\"$klass\"], aReceiver))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"for:",0
,(
function(self,aBehavior)
return ((
function(_s8_rec)
(_s8_rec):currentClass_(aBehavior);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","for: aBehavior\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tcurrentClass: aBehavior;\r\n\t\tyourself","(\r\nfunction(self,aBehavior)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):currentClass_(aBehavior);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler["$klass"],"getInfo:",0
,(
function(self,aString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local stream=nil
local aClass=nil
local selector=nil
local source=nil
local aMethod=nil
(function()
	local _s8_value=((aString):stream())
	stream=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((stream):nextChunk()):doIt())
	aClass=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((stream):nextChunk())
	selector=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((stream):nextChunk())
	source=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((aClass):_gt_gt(selector))
	aMethod=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:boolean_and_(((((aMethod))~=nil)),(function()
return ((smalltalk:boolean_or_((((source):trimBlanks()):isEmpty()),(function()
return ((smalltalk:isObject_equalTo_(((aMethod):source()),source)))
end))))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):debugInfo_(aMethod))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):debugInfo_forClass_(source, aClass))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"debug","getInfo: aString\r\n\t\" Private - Return the information of compiling aString as a chunked spec. \"\r\n\r\n\t| stream aClass selector source aMethod |\r\n\tstream := aString stream.\r\n\taClass := stream nextChunk doIt.\r\n\tselector := stream nextChunk.\r\n\tsource := stream nextChunk.\r\n\taMethod := aClass >> selector.\r\n\t(aMethod notNil and: [ source trimBlanks isEmpty or: [\r\n\t\taMethod source = source ]]) ifTrue: [\r\n\t\t^self debugInfo: aMethod\r\n\t].\r\n\t^self debugInfo: source forClass: aClass","(\r\nfunction(self,aString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal stream=nil\r\nlocal aClass=nil\r\nlocal selector=nil\r\nlocal source=nil\r\nlocal aMethod=nil\r\n(function()\r\n\tlocal _s8_value=((aString):stream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((stream):nextChunk()):doIt())\r\n\taClass=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((stream):nextChunk())\r\n\tselector=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((stream):nextChunk())\r\n\tsource=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aClass):_gt_gt(selector))\r\n\taMethod=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((((aMethod))~=nil)),(function()\r\nreturn ((smalltalk:boolean_or_((((source):trimBlanks()):isEmpty()),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((aMethod):source()),source)))\r\nend))))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):debugInfo_(aMethod))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):debugInfo_forClass_(source, aClass))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"parse:",0
,(
function(self,aString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(self):triggerEvent_with_ifNotHandled_("aboutToParse:", aString, (function()
(function()
	local _s8_value=((self):nodeFromTemplateIn_(aString))
	result=_s8_value; return _s8_value
end)();return ((function() if ((((((result))~=nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
	local _s8_value=(((self):parser()):parse_((aString):readStream()))
	result=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","parse: aString\r\n\t\" Return the result of parsing aString. \"\r\n\r\n\t| result |\r\n\tself\ttriggerEvent: #aboutToParse:\r\n\t\twith: aString ifNotHandled: [\r\n\t\t\tresult := self nodeFromTemplateIn: aString.\r\n\t\t\tresult notNil ifTrue: [ ^result ]\r\n\t\t].\r\n\tresult := self parser parse: aString readStream.\r\n\t^result","(\r\nfunction(self,aString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(self):triggerEvent_with_ifNotHandled_(\"aboutToParse:\", aString, (function()\r\n(function()\r\n\tlocal _s8_value=((self):nodeFromTemplateIn_(aString))\r\n\tresult=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((result))~=nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n\tlocal _s8_value=(((self):parser()):parse_((aString):readStream()))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"parseCode:",0
,(
function(self,aString)
return ((self):parse_(aString))
;
end
)
,"parsing","parseCode: aString\r\n\t\" OBSOLETE - Return the result of parsing aString. \"\r\n\r\n\t^self parse: aString","(\r\nfunction(self,aString)\r\nreturn ((self):parse_(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"compile:",0
,(
function(self,aString)
local aNode=nil
(function()
	local _s8_value=((self):parse_(aString))
	aNode=_s8_value; return _s8_value
end)()
;
(self):triggerEvent_with_("aboutToCompile:", aNode)
;
return ((self):emmitNode_(aNode))
;
end
)
,"private","compile: aString\r\n\t\" Return the native code result of compiling aString for currentClass. \"\r\n\r\n\t| aNode |\r\n\taNode := self parse: aString.\r\n\tself triggerEvent: #aboutToCompile: with: aNode.\r\n\t^self emmitNode: aNode","(\r\nfunction(self,aString)\r\nlocal aNode=nil\r\n(function()\r\n\tlocal _s8_value=((self):parse_(aString))\r\n\taNode=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):triggerEvent_with_(\"aboutToCompile:\", aNode)\r\n;\r\nreturn ((self):emmitNode_(aNode))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"compile:forClass:",0
,(
function(self,aString,aClass)
(self):currentClass_(aClass)
;
return ((self):compile_(aString))
;
end
)
,"compiling","compile: aString forClass: aClass\r\n\t\" Return the native code result of compiling aString as a method implemented in aClass. \"\r\n\r\n\tself currentClass: aClass.\r\n\t^self compile: aString","(\r\nfunction(self,aString,aClass)\r\n(self):currentClass_(aClass)\r\n;\r\nreturn ((self):compile_(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"compile:forClass:category:",0
,(
function(self,aString,aClass,category)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local method=nil
(function() if ((((smalltalk:boolean_or_(((((aString))==nil)),(function()
return ((aString):isEmpty())
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((self):load_forClass_(aString, aClass))
	method=_s8_value; return _s8_value
end)()
;
(method):category_(category)
;
(aClass):addCompiledMethod_(method)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (method)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"compiling","compile: aString forClass: aClass category: category\r\n\t\" Return the compiled method result of installing a new method compiled from aString in aClass. \"\r\n\r\n\t| method |\r\n\t(aString isNil or: [ aString isEmpty ]) ifTrue: [ ^nil ].\r\n\tmethod := self load: aString forClass: aClass.\r\n\tmethod category: category.\r\n\taClass addCompiledMethod: method.\r\n\t^method","(\r\nfunction(self,aString,aClass,category)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal method=nil\r\n(function() if ((((smalltalk:boolean_or_(((((aString))==nil)),(function()\r\nreturn ((aString):isEmpty())\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):load_forClass_(aString, aClass))\r\n\tmethod=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(method):category_(category)\r\n;\r\n(aClass):addCompiledMethod_(method)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (method)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"compileNodeObsolete:",0
,(
function(self,aNode)
return ((self):error_("OBSOLETE - Send #emmitNode: instead of #compileNode:"))
;
end
)
,"private","compileNodeObsolete: aNode\r\n\t\" OBSOLETE - \"\r\n\r\n\t^self error: 'OBSOLETE - Send #emmitNode: instead of #compileNode:'","(\r\nfunction(self,aNode)\r\nreturn ((self):error_(\"OBSOLETE - Send #emmitNode: instead of #compileNode:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"currentClass",0
,(
function(self)
return (self['@currentClass'])
;
end
)
,"accessing","currentClass\r\n\t\" Return the current class of the receiver. \"\r\n\r\n\t^currentClass","(\r\nfunction(self)\r\nreturn (self['@currentClass'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"currentClass:",0
,(
function(self,aClass)
(function()
	local _s8_value=(aClass)
	self['@currentClass']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","currentClass: aClass\r\n\t\" Set the current class of the receiver. \"\r\n\r\n\tcurrentClass := aClass","(\r\nfunction(self,aClass)\r\n(function()\r\n\tlocal _s8_value=(aClass)\r\n\tself['@currentClass']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"emmitNode:",0
,(
function(self,aNode)
(self):startCompiling_(aNode)
;
(self):prepare_(aNode)
;
(self):triggerEvent_with_("aboutToEncode:", aNode)
;
((self):encoder()):visit_(aNode)
;
(self):endCompiling_(aNode)
;
return (((self):encoder()):emmitedCode())
;
end
)
,"private","emmitNode: aNode\r\n\t\" Private - Return the native code for aNode. \"\r\n\r\n\tself startCompiling: aNode.\r\n\tself prepare: aNode.\r\n\tself triggerEvent: #aboutToEncode: with: aNode.\r\n\tself encoder visit: aNode.\r\n\tself endCompiling: aNode.\r\n\t^self encoder emmitedCode","(\r\nfunction(self,aNode)\r\n(self):startCompiling_(aNode)\r\n;\r\n(self):prepare_(aNode)\r\n;\r\n(self):triggerEvent_with_(\"aboutToEncode:\", aNode)\r\n;\r\n((self):encoder()):visit_(aNode)\r\n;\r\n(self):endCompiling_(aNode)\r\n;\r\nreturn (((self):encoder()):emmitedCode())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"emmitedCodeObsolete",0
,(
function(self)
return ((self):error_("OBSOLETE - Use expression: self encoder emmitedCode"))
;
end
)
,"results","emmitedCodeObsolete\r\n\t\" OBSOLETE - \"\r\n\r\n\t^self error: 'OBSOLETE - Use expression: self encoder emmitedCode'","(\r\nfunction(self)\r\nreturn ((self):error_(\"OBSOLETE - Use expression: self encoder emmitedCode\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"encoder",0
,(
function(self)
(function() if ((((((self['@encoder']))==nil)))==true) then return ((function()
return ((self):initializeEncoder())
end))() else return nil end end)()
;
return (self['@encoder'])
;
end
)
,"accessing","encoder\r\n\t\" Return the encoder of the receiver. \"\r\n\r\n\tencoder isNil ifTrue: [ self initializeEncoder ].\r\n\t^encoder","(\r\nfunction(self)\r\n(function() if ((((((self['@encoder']))==nil)))==true) then return ((function()\r\nreturn ((self):initializeEncoder())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@encoder'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"encoder:",0
,(
function(self,anEncoder)
(function()
	local _s8_value=(anEncoder)
	self['@encoder']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","encoder: anEncoder\r\n\t\" Set the encoder of the receiver. \"\r\n\r\n\tencoder := anEncoder","(\r\nfunction(self,anEncoder)\r\n(function()\r\n\tlocal _s8_value=(anEncoder)\r\n\tself['@encoder']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"endCompiling:",0
,(
function(self,aNode)
((self):encoder()):endCompiling_(aNode)
;
(self):triggerEvent_with_("endCompiling:", aNode)
;
return (self)
;
end
)
,"private","endCompiling: aNode\r\n\t\" Private - Finish emmiting code for aNode. \"\r\n\r\n\tself encoder endCompiling: aNode.\r\n\tself triggerEvent: #endCompiling: with: aNode.","(\r\nfunction(self,aNode)\r\n((self):encoder()):endCompiling_(aNode)\r\n;\r\n(self):triggerEvent_with_(\"endCompiling:\", aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"eval:",0
,(
function(self,nativeExpression)
return (((smalltalk.Smalltalk):current()):eval_(nativeExpression))
;
end
)
,"evaluation","eval: nativeExpression\r\n\t\" Return the result of evaluating a native expression in global context. \"\r\n\r\n\t^Smalltalk current eval: nativeExpression","(\r\nfunction(self,nativeExpression)\r\nreturn (((smalltalk.Smalltalk):current()):eval_(nativeExpression))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"eval:as:",0
,(
function(self,nativeExpression,aState)
return (((smalltalk.Smalltalk):current()):eval_as_(nativeExpression, aState))
;
end
)
,"evaluation","eval: nativeExpression as: aState\r\n\t\" Return the result of evaluating a native expression in global context.\r\n\tThe state of the system change to state while evaluating the expression.\r\n\t\"\r\n\r\n\t^Smalltalk current eval: nativeExpression as: aState","(\r\nfunction(self,nativeExpression,aState)\r\nreturn (((smalltalk.Smalltalk):current()):eval_as_(nativeExpression, aState))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"evaluate:",0
,(
function(self,smalltalkExpression)
return ((self):evaluate_in_to_(smalltalkExpression, (nil)["$klass"], nil))
;
end
)
,"doIt","evaluate: smalltalkExpression\r\n\t\" Return the result of evaluating the smalltalk expression in global context. \"\r\n\r\n\t^self evaluate: smalltalkExpression in: nil class to: nil","(\r\nfunction(self,smalltalkExpression)\r\nreturn ((self):evaluate_in_to_(smalltalkExpression, (nil)[\"$klass\"], nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"evaluate:in:to:",0
,(
function(self,smalltalkExpression,aClass,aReceiver)
local result=nil
local doIt=nil
local node=nil
local nativeCode=nil
(function()
	local _s8_value=((smalltalk.String):doItSelector())
	doIt=_s8_value; return _s8_value
end)()
;
(self):currentClass_(aClass)
;
(function()
	local _s8_value=((self):parse_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((doIt)," ^[")),smalltalkExpression)),"] value")))
	node=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):emmitNode_(node))
	nativeCode=_s8_value; return _s8_value
end)()
;
(aClass):addCompiledMethod_((self):eval_(nativeCode))
;
((function()
return ((function()
	local _s8_value=((aReceiver):perform_(doIt))
	result=_s8_value; return _s8_value
end)())
end)):ensure_((function()
return ((aClass):removeSelector_(doIt))
end))
;
return (result)
;
end
)
,"evaluation","evaluate: smalltalkExpression in: aClass to: aReceiver\r\n\t\" Return the result of evaluating the smalltalk expression with aReceiver. \"\r\n\r\n\t| result doIt node nativeCode |\r\n\tdoIt := String doItSelector.\r\n\tself currentClass: aClass.\r\n\tnode := self parse: doIt,' ^[', smalltalkExpression, '] value'.\r\n\tnativeCode := self emmitNode: node.\r\n\taClass addCompiledMethod: (self eval: nativeCode).\r\n\t[ result := aReceiver perform: doIt.\r\n\t] ensure: [ aClass removeSelector: doIt. ].\r\n\t^result","(\r\nfunction(self,smalltalkExpression,aClass,aReceiver)\r\nlocal result=nil\r\nlocal doIt=nil\r\nlocal node=nil\r\nlocal nativeCode=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.String):doItSelector())\r\n\tdoIt=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):currentClass_(aClass)\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):parse_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((doIt),\" ^[\")),smalltalkExpression)),\"] value\")))\r\n\tnode=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):emmitNode_(node))\r\n\tnativeCode=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aClass):addCompiledMethod_((self):eval_(nativeCode))\r\n;\r\n((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((aReceiver):perform_(doIt))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend)):ensure_((function()\r\nreturn ((aClass):removeSelector_(doIt))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"initialize",0
,(
function(self)
smalltalk:superImplementor(smalltalk.Compiler,"initialize")(self)
;
(self):resetVariables()
;
return (self)
;
end
)
,"initialize","initialize\r\n\t\" Private - Initialize the receiver. \"\r\n\r\n\tsuper initialize.\r\n\tself resetVariables.","(\r\nfunction(self)\r\nsmalltalk:superImplementor(smalltalk.Compiler,\"initialize\")(self)\r\n;\r\n(self):resetVariables()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"initializeEncoder",0
,(
function(self)
(function()
	local _s8_value=((self):triggerEvent_ifNotHandled_("needsEncoder", (function()
return ((smalltalk.S8Encoder):for_(self))
end)))
	self['@encoder']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"private","initializeEncoder\r\n\t\" Private - Initialize the encoder of the receiver. \"\r\n\r\n\tencoder := self\r\n\t\ttriggerEvent: #needsEncoder\r\n\t\tifNotHandled: [ S8Encoder for: self ]","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=((self):triggerEvent_ifNotHandled_(\"needsEncoder\", (function()\r\nreturn ((smalltalk.S8Encoder):for_(self))\r\nend)))\r\n\tself['@encoder']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"knownVariables",0
,(
function(self)
return (smalltalk:object_comma_(((self):pseudoVariables()),(self):tempVariables()))
;
end
)
,"variables","knownVariables\r\n\t\" Return the knownVariables of the receiver. \"\r\n\r\n\t^self pseudoVariables , self tempVariables","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_(((self):pseudoVariables()),(self):tempVariables()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"load:forClass:",0
,(
function(self,aString,aClass)
local nativeCode=nil
(function()
	local _s8_value=((self):compile_forClass_(aString, aClass))
	nativeCode=_s8_value; return _s8_value
end)()
;
return ((self):eval_(nativeCode))
;
end
)
,"compiling","load: aString forClass: aClass\r\n\t\" Return the CompiledMethod result of compiling aString as a method implemented in aClass. \"\r\n\r\n\t| nativeCode |\r\n\tnativeCode := self compile: aString forClass: aClass.\r\n\t^self eval: nativeCode","(\r\nfunction(self,aString,aClass)\r\nlocal nativeCode=nil\r\n(function()\r\n\tlocal _s8_value=((self):compile_forClass_(aString, aClass))\r\n\tnativeCode=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((self):eval_(nativeCode))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"localName:",0
,(
function(self,aName)
return (((self):encoder()):localName_(aName))
;
end
)
,"private","localName: aName\r\n\t\" Private - Return a valid (native, local) name for aName. \"\r\n\r\n\t^self encoder localName: aName","(\r\nfunction(self,aName)\r\nreturn (((self):encoder()):localName_(aName))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"nativeLanguage",0
,(
function(self)
return (((self):encoder()):nativeLanguage())
;
end
)
,"accessing","nativeLanguage\r\n\r\n\t^self encoder nativeLanguage","(\r\nfunction(self)\r\nreturn (((self):encoder()):nativeLanguage())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"nodeFromTemplateIn:",0
,(
function(self,aString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local src=nil
local driver=nil
local selector=nil
local count=nil
local found=nil
(function() if ((((smalltalk:boolean_or_(((aString):isEmpty()),(function()
return ((smalltalk:booleanNot_(((smalltalk:isObject_equalTo_(((aString):last()),"\""))))))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((((aString):upToLast_("\"")):fromLast_("\"")):stream())
	src=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((src):peek()),"#"))))==true) then return ((function()
return ((src):next())
end))() else return ((function()
return ((function() if ((((smalltalk:isObject_equalTo_(("template"),((src):nextLine()):upTo_(" ")))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)())
end))() end end)()
;
(function()
	local _s8_value=((src):upTo_(" "))
	driver=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_((driver),"self"))))==true) then return ((function()
return ((function()
	local _s8_value=(self)
	driver=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((driver),"class"))))==true) then return ((function()
return ((function()
	local _s8_value=((self):currentClass())
	driver=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((driver),"api"))))==true) then return ((function()
return ((function()
	local _s8_value=("APIBuilder")
	driver=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function() if ((((driver):isString()))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.Smalltalk):at_ifAbsent_(driver, (function()
return (nil)
end)))
	driver=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((src):upTo_(" "))
	selector=_s8_value; return _s8_value
end)()
;
(function() if (((smalltalk:responds_to_((driver),(selector))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
	local _s8_value=((selector):occurrencesOf_(":"))
	count=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((function(mth)
return ((function() if ((((((mth))~=nil)))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):source_(aString);
return ((_s8_rec):yourself())
end
)(mth))
end))() else return nil end end)())
end))
	found=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_((count),(0)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((found):value_((driver):perform_(selector)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((count),(1)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((found):value_((driver):perform_with_(selector, (src):upToEnd())))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"templates","nodeFromTemplateIn: aString\r\n\t\" Return the nodes syntetized from inlined templates in aString (or nil).\r\n\tInlined templates are detected from last comment in source.\r\n\t\"\r\n\r\n\t| src driver selector count found |\r\n\t(aString isEmpty or: [(aString last = $\") not]) ifTrue: [ ^nil ].\r\n\tsrc := ((aString upToLast: $\") fromLast: $\") stream.\r\n\tsrc peek = $# ifTrue: [ src next ] ifFalse: [\r\n\t\t#template = (src nextLine upTo: $ ) ifFalse: [ ^nil ]\r\n\t].\r\n\tdriver := src upTo: $ .\r\n\tdriver = #self  ifTrue: [ driver := self ].\r\n\tdriver = #class ifTrue: [ driver := self currentClass ].\r\n\tdriver = #api   ifTrue: [ driver := #APIBuilder ].\r\n\tdriver isString ifTrue: [ driver := Smalltalk at: driver ifAbsent: [] ].\r\n\tselector := src upTo: $ .\r\n\t(driver respondsTo: selector) ifFalse: [ ^nil ].\r\n\tcount := selector occurrencesOf: $:.\r\n\tfound := [:mth| mth notNil ifTrue: [ mth source: aString; yourself ] ].\r\n\tcount = 0 ifTrue: [ ^found value: (driver perform: selector) ].\r\n\tcount = 1 ifTrue: [ ^found value: (driver perform: selector with: src upToEnd) ].\r\n\t^nil","(\r\nfunction(self,aString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal src=nil\r\nlocal driver=nil\r\nlocal selector=nil\r\nlocal count=nil\r\nlocal found=nil\r\n(function() if ((((smalltalk:boolean_or_(((aString):isEmpty()),(function()\r\nreturn ((smalltalk:booleanNot_(((smalltalk:isObject_equalTo_(((aString):last()),\"\\\"\"))))))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((aString):upToLast_(\"\\\"\")):fromLast_(\"\\\"\")):stream())\r\n\tsrc=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((src):peek()),\"#\"))))==true) then return ((function()\r\nreturn ((src):next())\r\nend))() else return ((function()\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_((\"template\"),((src):nextLine()):upTo_(\" \")))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)())\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((src):upTo_(\" \"))\r\n\tdriver=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((driver),\"self\"))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(self)\r\n\tdriver=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((driver),\"class\"))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((self):currentClass())\r\n\tdriver=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((driver),\"api\"))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(\"APIBuilder\")\r\n\tdriver=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((driver):isString()))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):at_ifAbsent_(driver, (function()\r\nreturn (nil)\r\nend)))\r\n\tdriver=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((src):upTo_(\" \"))\r\n\tselector=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((smalltalk:responds_to_((driver),(selector))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((selector):occurrencesOf_(\":\"))\r\n\tcount=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function(mth)\r\nreturn ((function() if ((((((mth))~=nil)))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):source_(aString);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(mth))\r\nend))() else return nil end end)())\r\nend))\r\n\tfound=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((count),(0)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((found):value_((driver):perform_(selector)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((count),(1)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((found):value_((driver):perform_with_(selector, (src):upToEnd())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"parser",0
,(
function(self)
return ((self):triggerEvent_ifNotHandled_("needsParser", (function()
return ((smalltalk.SmalltalkParser):new())
end)))
;
end
)
,"private","parser\r\n\t\" Private - Return the parser of the receiver. \"\r\n\r\n\t^self\ttriggerEvent: #needsParser\r\n\t\tifNotHandled: [ SmalltalkParser new ]","(\r\nfunction(self)\r\nreturn ((self):triggerEvent_ifNotHandled_(\"needsParser\", (function()\r\nreturn ((smalltalk.SmalltalkParser):new())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"prepare:",0
,(
function(self,aNode)
((self):transformers()):do_((function(each)
return ((each):visit_(aNode))
end))
;
return (self)
;
end
)
,"private","prepare: aNode\r\n\t\" Private - Apply tranformations/optimization to aNode. \"\r\n\r\n\tself transformers do: [:each| each visit: aNode ].","(\r\nfunction(self,aNode)\r\n((self):transformers()):do_((function(each)\r\nreturn ((each):visit_(aNode))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"pseudoVariables",0
,(
function(self)
return (smalltalk:newArray({"self", "super", "true", "false", "nil", "thisContext"}))
;
end
)
,"variables","pseudoVariables\r\n\t\" Return the pseudoVariables of the receiver. \"\r\n\r\n\t^#( #self #super #true #false #nil #thisContext )","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"self\", \"super\", \"true\", \"false\", \"nil\", \"thisContext\"}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"recompile:",0
,(
function(self,aClass)
(self):triggerEvent_with_("aboutToRecompile:", aClass)
;
((aClass):methodDictionary()):do_((function(each)
return ((self):compile_forClass_category_((each):source(), aClass, (each):category()))
end))
;
(self):triggerEvent_with_("recompiled:", aClass)
;
(function() if ((((aClass):isMetaclass()))==true) then return nil else return ((function()
return ((self):recompile_((aClass)["$klass"]))
end))() end end)()
;
return (self)
;
end
)
,"compiling","recompile: aClass\r\n\t\" Re-compile all the methods defined by aClass. \"\r\n\r\n\tself triggerEvent: #aboutToRecompile: with: aClass.\r\n\taClass methodDictionary do: [:each |\r\n\"Debug\t\tself note: each.\"\r\n\t\tself compile: each source forClass: aClass category: each category\r\n\t].\r\n\tself triggerEvent: #recompiled: with: aClass.\r\n\taClass isMetaclass ifFalse: [ self recompile: aClass class ].","(\r\nfunction(self,aClass)\r\n(self):triggerEvent_with_(\"aboutToRecompile:\", aClass)\r\n;\r\n((aClass):methodDictionary()):do_((function(each)\r\nreturn ((self):compile_forClass_category_((each):source(), aClass, (each):category()))\r\nend))\r\n;\r\n(self):triggerEvent_with_(\"recompiled:\", aClass)\r\n;\r\n(function() if ((((aClass):isMetaclass()))==true) then return nil else return ((function()\r\nreturn ((self):recompile_((aClass)[\"$klass\"]))\r\nend))() end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"resetVariables",0
,(
function(self)
(function()
	local _s8_value=(smalltalk:newArray({}))
	self['@unknownVariables']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(smalltalk:newArray({}))
	self['@tempVariables']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"variables","resetVariables\r\n\t\" Reset the variables of the receiver. \"\r\n\r\n\tunknownVariables := #().\r\n\ttempVariables := #().","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tself['@unknownVariables']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tself['@tempVariables']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"smalltalkNameFor:",0
,(
function(self,aClass)
return (((self):encoder()):smalltalkNameFor_(aClass))
;
end
)
,"names","smalltalkNameFor: aClass\r\n\t\" Return the global name of aClass (can be nil). \"\r\n\r\n\t^self encoder smalltalkNameFor: aClass","(\r\nfunction(self,aClass)\r\nreturn (((self):encoder()):smalltalkNameFor_(aClass))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"startCompiling:",0
,(
function(self,aNode)
(self):triggerEvent_with_("startCompiling:", aNode)
;
((self):encoder()):startCompiling_(aNode)
;
return (self)
;
end
)
,"private","startCompiling: aNode\r\n\t\" Private - Set the receiver to start emmiting code for aNode. \"\r\n\r\n\tself triggerEvent: #startCompiling: with: aNode.\r\n\tself encoder startCompiling: aNode.","(\r\nfunction(self,aNode)\r\n(self):triggerEvent_with_(\"startCompiling:\", aNode)\r\n;\r\n((self):encoder()):startCompiling_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"tempVariables",0
,(
function(self)
return (self['@tempVariables'])
;
end
)
,"variables","tempVariables\r\n\t\" Return the tempVariables of the receiver. \"\r\n\r\n\t^tempVariables","(\r\nfunction(self)\r\nreturn (self['@tempVariables'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"transformers",0
,(
function(self)
(function() if ((((((self['@transformers']))==nil)))==true) then return ((function()
(function()
	local _s8_value=((smalltalk.Array):new())
	self['@transformers']=_s8_value; return _s8_value
end)();return ((self):triggerEvent_with_("setTransformers:", self['@transformers']))
end))() else return nil end end)()
;
return (self['@transformers'])
;
end
)
,"accessing","transformers\r\n\t\" Return the node transformers installed in the receiver. \"\r\n\r\n\ttransformers isNil ifTrue: [\r\n\t\ttransformers := Array new.\r\n\t\tself triggerEvent: #setTransformers: with: transformers.\r\n\t].\r\n\t^transformers","(\r\nfunction(self)\r\n(function() if ((((((self['@transformers']))==nil)))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tself['@transformers']=_s8_value; return _s8_value\r\nend)();return ((self):triggerEvent_with_(\"setTransformers:\", self['@transformers']))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@transformers'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"unknownVariables",0
,(
function(self)
return (self['@unknownVariables'])
;
end
)
,"variables","unknownVariables\r\n\t\" Return the unknownVariables of the receiver. \"\r\n\r\n\t^unknownVariables","(\r\nfunction(self)\r\nreturn (self['@unknownVariables'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"useTemplates",0
,(
function(self)
return ((smalltalk:boolean_and_(((((self['@useTemplates']))~=nil)),(function()
return (self['@useTemplates'])
end))))
;
end
)
,"accessing","useTemplates\r\n\t\" Return true if the receiver use tamplates. \"\r\n\r\n\t^useTemplates notNil and: [ useTemplates ]","(\r\nfunction(self)\r\nreturn ((smalltalk:boolean_and_(((((self['@useTemplates']))~=nil)),(function()\r\nreturn (self['@useTemplates'])\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Compiler,"useTemplates:",0
,(
function(self,aBoolean)
(function()
	local _s8_value=(aBoolean)
	self['@useTemplates']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","useTemplates: aBoolean\r\n\t\" Set the receiver to consider inlined templates. \"\r\n\r\n\tuseTemplates := aBoolean","(\r\nfunction(self,aBoolean)\r\n(function()\r\n\tlocal _s8_value=(aBoolean)\r\n\tself['@useTemplates']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SourceCodeParser["$klass"],"parse:",0
,(
function(self,aStream)
return (((self):new()):parse_(aStream))
;
end
)
,"instantiation","parse: aStream\r\n\t\" Return the result of parsing contents in aStream. \"\r\n\r\n\t^self new parse: aStream ","(\r\nfunction(self,aStream)\r\nreturn (((self):new()):parse_(aStream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SourceCodeParser["$klass"],"debug:",0
,(
function(self,aStream)
return ((smalltalk.PPParser):debugWhile_((function()
return ((self):parse_(aStream))
end)))
;
end
)
,"debug","debug: aStream\r\n\t^PPParser debugWhile: [\r\n\t\tself parse: aStream\r\n\t]","(\r\nfunction(self,aStream)\r\nreturn ((smalltalk.PPParser):debugWhile_((function()\r\nreturn ((self):parse_(aStream))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SourceCodeParser,"parse:",0
,(
function(self,aStream)
return (((self):parser()):parse_(aStream))
;
end
)
,"parsing","parse: aStream\r\n\t\" Return the result of parsing contents in aStream. \"\r\n\r\n\t^self parser parse: aStream","(\r\nfunction(self,aStream)\r\nreturn (((self):parser()):parse_(aStream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SourceCodeParser,"gc",0
,(
function(self)
(function() if ((((smalltalk.Smalltalk):includesKey_("JSC")))==true) then return ((function()
return ((smalltalk.Smalltalk):gc())
end))() else return nil end end)()
;
return (self)
;
end
)
,"gc","gc\r\n\t\" Private - Perform (preventive) garbage collection.\r\n\tNote: JavascriptCore VM can crash reporting\r\n\t 'memory issues' if we do not GC asap.\r\n\t This issue will persist while Apple continue\r\n\t  distributing a version of JSC with issue reported\r\n\t\thttp://alereimondo.no-ip.org/U8/232\r\n\t\thttps://bugs.webkit.org/show_bug.cgi?id=160027\r\n\t\"\r\n\r\n\t(Smalltalk includesKey: #JSC) ifTrue: [ Smalltalk gc ].","(\r\nfunction(self)\r\n(function() if ((((smalltalk.Smalltalk):includesKey_(\"JSC\")))==true) then return ((function()\r\nreturn ((smalltalk.Smalltalk):gc())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SourceCodeParser,"separator",0
,(
function(self)
return ((smalltalk.PPCharacterParser):separator())
;
end
)
,"parsers","separator\r\n\t\" Return the parser for separator. \"\r\n\r\n\t^PPCharacterParser separator","(\r\nfunction(self)\r\nreturn ((smalltalk.PPCharacterParser):separator())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SourceCodeParser,"parser",0
,(
function(self)
return ((self):implementedBySubclass_("parser"))
;
end
)
,"parsers","parser\r\n\t\" Return the method parser. \"\r\n\r\n\t^self implementedBySubclass: #parser","(\r\nfunction(self)\r\nreturn ((self):implementedBySubclass_(\"parser\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SourceCodeParser,"charParser:",0
,(
function(self,options)
return ((smalltalk.PPCharacterParser):for_doing_(smalltalk:object_comma_(("a-zA-Z0-9"),options), (function(c)
return ((smalltalk:boolean_or_(((c):between_and_("a", "z")),(function()
return ((smalltalk:boolean_or_(((c):between_and_("A", "Z")),(function()
return ((smalltalk:boolean_or_(((c):between_and_("0", "9")),(function()
return ((options):includes_(c))
end))))
end))))
end))))
end)))
;
end
)
,"parsers","charParser: options\r\n\t\" Private - Return the character parser. \"\r\n\r\n\t^PPCharacterParser for: 'a-zA-Z0-9',options doing: [:c|\r\n\t\t(c between: $a and: $z)\r\n\t\tor: [ (c between: $A and: $Z)\r\n\t\tor: [ (c between: $0 and: $9)\r\n\t\tor: [ options includes: c ]]]\r\n\t]","(\r\nfunction(self,options)\r\nreturn ((smalltalk.PPCharacterParser):for_doing_(smalltalk:object_comma_((\"a-zA-Z0-9\"),options), (function(c)\r\nreturn ((smalltalk:boolean_or_(((c):between_and_(\"a\", \"z\")),(function()\r\nreturn ((smalltalk:boolean_or_(((c):between_and_(\"A\", \"Z\")),(function()\r\nreturn ((smalltalk:boolean_or_(((c):between_and_(\"0\", \"9\")),(function()\r\nreturn ((options):includes_(c))\r\nend))))\r\nend))))\r\nend))))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SourceCodeParser,",","_comma"
,(
function(self,anObject)
return (smalltalk:object_comma_((smalltalk.PPParser),anObject))
;
end
)
,"parsers",", anObject\r\n\t\" Private - Return the parser to start parsing anObject. \"\r\n\r\n\t^PPParser , anObject","(\r\nfunction(self,anObject)\r\nreturn (smalltalk:object_comma_((smalltalk.PPParser),anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SourceCodeParser,"optional:",0
,(
function(self,anObject)
return ((smalltalk.PPParser):optional_(anObject))
;
end
)
,"parsers","optional: anObject\r\n\t\" Private - Return the parser to parse an optional element anObject. \"\r\n\r\n\t^PPParser optional: anObject","(\r\nfunction(self,anObject)\r\nreturn ((smalltalk.PPParser):optional_(anObject))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SourceCodeParser,"gc:",0
,(
function(self,anObject)
return ((smalltalk:object_comma_((self),anObject)):_eq_eq_gt((function(node)
(self):gc();return (node)
end)))
;
end
)
,"parsers","gc: anObject\r\n\t\" Private - Return the parser to parse anObject. \"\r\n\r\n\t^(self ,anObject) ==> [:node| self gc. node ]","(\r\nfunction(self,anObject)\r\nreturn ((smalltalk:object_comma_((self),anObject)):_eq_eq_gt((function(node)\r\n(self):gc();return (node)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser["$klass"],"constructEventsTriggered",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:newArray({"needsDirectiveParser:ws:"})),smalltalk:superImplementor(smalltalk.SmalltalkParser["$klass"],"constructEventsTriggered")(self)))
;
end
)
,"events","constructEventsTriggered\r\n\t\" Private - Return the events managed by instances of the receiver. \"\r\n\r\n\t^#(\t#needsDirectiveParser:ws:\r\n\t) , super constructEventsTriggered","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:newArray({\"needsDirectiveParser:ws:\"})),smalltalk:superImplementor(smalltalk.SmalltalkParser[\"$klass\"],\"constructEventsTriggered\")(self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"missingDirective:ws:",0
,(
function(self,number,ws)
return (((smalltalk:object_comma_(((((smalltalk.PPCharacterParser):notIn_(">")):star()):flatten()),">")):_at_at("missingDirective")):_eq_eq_gt((function(node)
return (((node):first()):error_("Invalid directive."))
end)))
;
end
)
,"directives","missingDirective: number ws: ws\r\n\t\" Private - Return the parser for missing support of directives. \"\r\n\r\n\t^(PPCharacterParser notIn: $>) star flatten\r\n\t, $> @@#missingDirective\r\n\t==> [:node| node first error: 'Invalid directive.' ]","(\r\nfunction(self,number,ws)\r\nreturn (((smalltalk:object_comma_(((((smalltalk.PPCharacterParser):notIn_(\">\")):star()):flatten()),\">\")):_at_at(\"missingDirective\")):_eq_eq_gt((function(node)\r\nreturn (((node):first()):error_(\"Invalid directive.\"))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"jsDirective:ws:",0
,(
function(self,number,ws)
return (((smalltalk:object_comma_(((smalltalk:object_comma_((self),"js:")):_slash("javascript:")),(smalltalk.PPInlineParser):upTo_(":>"))):_at_at("jsDirective")):_eq_eq_gt((function(node)
return ((smalltalk.JSDirectiveNode):code_protected_((node):second(), (smalltalk:isObject_equalTo_(((node):first()),"javascript:"))))
end)))
;
end
)
,"directives","jsDirective: number ws: ws\r\n\t\" Private - Return the parser for native directives.\r\n\t\"\r\n\r\n\t^self\t, #js: / #javascript:\r\n\t\t, (PPInlineParser upTo: ':>')\r\n\t\t@@#jsDirective\r\n\t==> [:node| JSDirectiveNode\r\n\t\tcode: node second\r\n\t\tprotected: node first = #javascript:\r\n\t]","(\r\nfunction(self,number,ws)\r\nreturn (((smalltalk:object_comma_(((smalltalk:object_comma_((self),\"js:\")):_slash(\"javascript:\")),(smalltalk.PPInlineParser):upTo_(\":>\"))):_at_at(\"jsDirective\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.JSDirectiveNode):code_protected_((node):second(), (smalltalk:isObject_equalTo_(((node):first()),\"javascript:\"))))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"optDirective:ws:",0
,(
function(self,number,ws)
return (((smalltalk:object_comma_((((smalltalk:object_comma_((self),"opt:")):_slash("opt+:")):_slash("opt-:")),(smalltalk.PPInlineParser):upTo_(":>"))):_at_at("optDirective")):_eq_eq_gt((function(node)
return ((
function(_s8_rec)
(_s8_rec):args_(((node):second()):asArrayOfSubstrings());
(_s8_rec):remove_((smalltalk:isObject_equalTo_(((node):first()),"opt-:")));
return ((_s8_rec):yourself())
end
)((smalltalk.InlineOptimizationNode):new()))
end)))
;
end
)
,"directives","optDirective: number ws: ws\r\n\t\" Private - Return the parser for compiler optimizations directive.\r\n\t\"\r\n\r\n\t^self\t, #opt: / #opt+: / #opt-:\r\n\t\t,(PPInlineParser upTo: ':>')\r\n\t\t@@#optDirective\r\n\t==> [:node| InlineOptimizationNode new\r\n\t\targs: node second asArrayOfSubstrings;\r\n\t\tremove: node first = #opt-:;\r\n\t\tyourself\r\n\t]","(\r\nfunction(self,number,ws)\r\nreturn (((smalltalk:object_comma_((((smalltalk:object_comma_((self),\"opt:\")):_slash(\"opt+:\")):_slash(\"opt-:\")),(smalltalk.PPInlineParser):upTo_(\":>\"))):_at_at(\"optDirective\")):_eq_eq_gt((function(node)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):args_(((node):second()):asArrayOfSubstrings());\r\n(_s8_rec):remove_((smalltalk:isObject_equalTo_(((node):first()),\"opt-:\")));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.InlineOptimizationNode):new()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"defaultDirective:ws:",0
,(
function(self,number,ws)
return ((((self):optDirective_ws_(number, ws)):_slash((self):jsDirective_ws_(number, ws))):_slash((self):missingDirective_ws_(number, ws)))
;
end
)
,"directives","defaultDirective: number ws: ws\r\n\t\" Private - Return the parser for default support of directives.\r\n\tWe support inlined native directives.\r\n\t\"\r\n\r\n\t^ (self optDirective: number ws: ws)\r\n\t/ (self jsDirective: number ws: ws)\r\n\t/ (self missingDirective: number ws: ws)","(\r\nfunction(self,number,ws)\r\nreturn ((((self):optDirective_ws_(number, ws)):_slash((self):jsDirective_ws_(number, ws))):_slash((self):missingDirective_ws_(number, ws)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"directive:ws:",0
,(
function(self,number,ws)
local parser=nil
(function()
	local _s8_value=((self):triggerEvent_withArguments_ifNotHandled_("needsDirectiveParser:ws:", (smalltalk.Array):with_with_(number, ws), (function()
return ((self):defaultDirective_ws_(number, ws))
end)))
	parser=_s8_value; return _s8_value
end)()
;
return (((smalltalk:object_comma_((smalltalk:object_comma_((self),"<")),smalltalk:object_comma_((smalltalk:object_comma_((ws),parser)),ws))):_at_at("directive")):_eq_eq_gt((function(node)
return (((node):second()):second())
end)))
;
end
)
,"directives","directive: number ws: ws\r\n\t\" Private - Return the parser for directives. \"\r\n\r\n\t| parser |\r\n\tparser := self\r\n\t\ttriggerEvent: #needsDirectiveParser:ws:\r\n\t\twithArguments: (Array with: number with: ws)\r\n\t\tifNotHandled: [ self defaultDirective: number ws: ws ].\r\n\r\n\t^self\t, $< ,(ws, parser, ws) @@#directive\r\n\t\t==> [:node | node second second ]","(\r\nfunction(self,number,ws)\r\nlocal parser=nil\r\n(function()\r\n\tlocal _s8_value=((self):triggerEvent_withArguments_ifNotHandled_(\"needsDirectiveParser:ws:\", (smalltalk.Array):with_with_(number, ws), (function()\r\nreturn ((self):defaultDirective_ws_(number, ws))\r\nend)))\r\n\tparser=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (((smalltalk:object_comma_((smalltalk:object_comma_((self),\"<\")),smalltalk:object_comma_((smalltalk:object_comma_((ws),parser)),ws))):_at_at(\"directive\")):_eq_eq_gt((function(node)\r\nreturn (((node):second()):second())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"binarySelectorChars",0
,(
function(self)
return ("|&\\\\+*/=><,@%~-")
;
end
)
,"constants","binarySelectorChars\r\n\t\" Return the binary selector characters mask. \"\r\n\r\n\t^'|&\\\\+*/=><,@%~-'","(\r\nfunction(self)\r\nreturn (\"|&\\\\\\\\+*/=><,@%~-\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"binaryCharParser",0
,(
function(self)
return (((smalltalk.PPCharacterParser):in_((self):binarySelectorChars())):_at_at("binaryChar"))
;
end
)
,"parsers","binaryCharParser\r\n\t\" Return the binary selector character parser. \"\r\n\r\n\t^(PPCharacterParser in: self binarySelectorChars) @@#binaryChar","(\r\nfunction(self)\r\nreturn (((smalltalk.PPCharacterParser):in_((self):binarySelectorChars())):_at_at(\"binaryChar\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"symbolCharParser",0
,(
function(self)
return (((self):charParser_("_")):_at_at("symbolChar"))
;
end
)
,"parsers","symbolCharParser\r\n\t\" Return the symbol character parser. \"\r\n\r\n\t^(self charParser: $_) @@#symbolChar","(\r\nfunction(self)\r\nreturn (((self):charParser_(\"_\")):_at_at(\"symbolChar\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"jsSymbolCharParser",0
,(
function(self)
return (((self):charParser_("_$")):_at_at("jsSymbolChar"))
;
end
)
,"parsers","jsSymbolCharParser\r\n\t\" Private - Return the js symbol character parser. \"\r\n\r\n\t^(self charParser: '_$') @@#jsSymbolChar","(\r\nfunction(self)\r\nreturn (((self):charParser_(\"_$\")):_at_at(\"jsSymbolChar\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"extendedSymbolCharParser",0
,(
function(self)
return (((self):charParser_(smalltalk:object_comma_(("_:"),(self):binarySelectorChars()))):_at_at("extendedSymbolChar"))
;
end
)
,"parsers","extendedSymbolCharParser\r\n\t\" Private - Return the extended symbol character parser. \"\r\n\r\n\t^(self charParser: '_:' , self binarySelectorChars) @@#extendedSymbolChar","(\r\nfunction(self)\r\nreturn (((self):charParser_(smalltalk:object_comma_((\"_:\"),(self):binarySelectorChars()))):_at_at(\"extendedSymbolChar\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"globalNameParser",0
,(
function(self)
return (((smalltalk:object_comma_(((smalltalk.PPCharacterParser):between_and_("A", "Z")),((self):symbolCharParser()):star())):flatten()):_at_at("globalNameParser"))
;
end
)
,"parsers","globalNameParser\r\n\t\" Return the parser for global names. \"\r\n\r\n\t^((PPCharacterParser between: $A and: $Z)\r\n\t\t, self symbolCharParser star)\r\n\t\tflatten @@#globalNameParser","(\r\nfunction(self)\r\nreturn (((smalltalk:object_comma_(((smalltalk.PPCharacterParser):between_and_(\"A\", \"Z\")),((self):symbolCharParser()):star())):flatten()):_at_at(\"globalNameParser\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"comment",0
,(
function(self)
return ((((smalltalk:object_comma_((smalltalk:object_comma_((("\""):asParser()),(smalltalk:object_comma_(((smalltalk:booleanNot_((("\""):asParser())))),(smalltalk.PPAnyParser):new())):star())),"\"")):_at_at("stComment")):flatten()):_at_at("comment"))
;
end
)
,"parsers","comment\r\n\t\" Return the parser for comment. \"\r\n\r\n\t^($\" asParser\r\n\t,($\" asParser not, PPAnyParser new) star\r\n\t, $\" @@#stComment) flatten\r\n\t@@#comment","(\r\nfunction(self)\r\nreturn ((((smalltalk:object_comma_((smalltalk:object_comma_(((\"\\\"\"):asParser()),(smalltalk:object_comma_(((smalltalk:booleanNot_(((\"\\\"\"):asParser())))),(smalltalk.PPAnyParser):new())):star())),\"\\\"\")):_at_at(\"stComment\")):flatten()):_at_at(\"comment\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"nameStart",0
,(
function(self)
return (((smalltalk.PPCharacterParser):for_doing_("a-z_", (function(c)
return ((smalltalk:boolean_or_(((c):between_and_("a", "z")),(function()
return ((smalltalk:isObject_equalTo_((c),"_")))
end))))
end))):_at_at("nameStart"))
;
end
)
,"parsers","nameStart\r\n\t\" Return the parser for name start.\"\r\n\r\n\t^(PPCharacterParser for: 'a-z_' doing: [:c|\r\n\t\t(c between: $a and: $z)\r\n\t\tor: [ c = $_ ]\r\n\t]) @@#nameStart","(\r\nfunction(self)\r\nreturn (((smalltalk.PPCharacterParser):for_doing_(\"a-z_\", (function(c)\r\nreturn ((smalltalk:boolean_or_(((c):between_and_(\"a\", \"z\")),(function()\r\nreturn ((smalltalk:isObject_equalTo_((c),\"_\")))\r\nend))))\r\nend))):_at_at(\"nameStart\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"upNameStart",0
,(
function(self)
return (((smalltalk.PPCharacterParser):for_doing_("A-Za-z_", (function(c)
return ((smalltalk:boolean_or_(((c):between_and_("A", "Z")),(function()
return ((smalltalk:boolean_or_(((c):between_and_("a", "z")),(function()
return ((smalltalk:isObject_equalTo_((c),"_")))
end))))
end))))
end))):_at_at("upNameStart"))
;
end
)
,"parsers","upNameStart\r\n\t\" Return the parser for name start (can be uppercase).\"\r\n\r\n\t^(PPCharacterParser for: 'A-Za-z_' doing: [:c|\r\n\t\t(c between: $A and: $Z)\r\n\t\tor: [ (c between: $a and: $z)\r\n\t\tor: [ c = $_ ]]\r\n\t]) @@#upNameStart","(\r\nfunction(self)\r\nreturn (((smalltalk.PPCharacterParser):for_doing_(\"A-Za-z_\", (function(c)\r\nreturn ((smalltalk:boolean_or_(((c):between_and_(\"A\", \"Z\")),(function()\r\nreturn ((smalltalk:boolean_or_(((c):between_and_(\"a\", \"z\")),(function()\r\nreturn ((smalltalk:isObject_equalTo_((c),\"_\")))\r\nend))))\r\nend))))\r\nend))):_at_at(\"upNameStart\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"digits",0
,(
function(self)
return (((smalltalk.PPCharacterParser):between_and_("0", "9")):plus())
;
end
)
,"parsers","digits\r\n\t\" Return the parser for numeric digits.\"\r\n\r\n\t^(PPCharacterParser between: $0 and: $9) plus","(\r\nfunction(self)\r\nreturn (((smalltalk.PPCharacterParser):between_and_(\"0\", \"9\")):plus())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"decimals",0
,(
function(self)
return ((smalltalk:object_comma_((smalltalk:object_comma_((self),".")),(self):digits())):_at_at("decimals"))
;
end
)
,"parsers","decimals\r\n\t\" Return the parser for number decimals.\"\r\n\r\n\t^self\t, $.\r\n\t\t, self digits\r\n\t\t@@#decimals","(\r\nfunction(self)\r\nreturn ((smalltalk:object_comma_((smalltalk:object_comma_((self),\".\")),(self):digits())):_at_at(\"decimals\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"exponent",0
,(
function(self)
return ((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),"e")),(self):optional_("-"))),(self):digits())):_at_at("exponent"))
;
end
)
,"parsers","exponent\r\n\t\" Return the parser for number exponent.\"\r\n\r\n\t^self\t, $e\r\n\t\t, (self optional: $-)\r\n\t\t, self digits\r\n\t\t@@#exponent","(\r\nfunction(self)\r\nreturn ((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),\"e\")),(self):optional_(\"-\"))),(self):digits())):_at_at(\"exponent\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"radix",0
,(
function(self)
return ((smalltalk:object_comma_((smalltalk:object_comma_((self),"r")),((smalltalk.PPCharacterParser):for_doing_("0-9A-Z", (function(c)
return ((smalltalk:boolean_or_(((c):between_and_("0", "9")),(function()
return ((c):between_and_("A", "Z"))
end))))
end))):plus())):_at_at("radix"))
;
end
)
,"parsers","radix\r\n\t\" Return the parser for number radix.\"\r\n\r\n\t^self\t, $r\r\n\t\t, (PPCharacterParser for: '0-9A-Z' doing: [:c|\r\n\t\t\t(c between: $0 and: $9)\r\n\t\t\tor: [ c between: $A and: $Z ]\r\n\t\t]) plus\r\n\t\t@@#radix","(\r\nfunction(self)\r\nreturn ((smalltalk:object_comma_((smalltalk:object_comma_((self),\"r\")),((smalltalk.PPCharacterParser):for_doing_(\"0-9A-Z\", (function(c)\r\nreturn ((smalltalk:boolean_or_(((c):between_and_(\"0\", \"9\")),(function()\r\nreturn ((c):between_and_(\"A\", \"Z\"))\r\nend))))\r\nend))):plus())):_at_at(\"radix\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"number",0
,(
function(self)
return ((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):optional_("-")),(self):digits())),((self):decimals()):optional())),(((self):exponent()):_slash((self):radix())):optional())):_at_at("number"))
;
end
)
,"parsers","number\r\n\t\" Return the parser for number.\"\r\n\r\n\t^(self optional: $-)\r\n\t\t, self digits , self decimals optional\r\n\t\t, (self exponent / self radix) optional\r\n\t\t@@#number","(\r\nfunction(self)\r\nreturn ((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):optional_(\"-\")),(self):digits())),((self):decimals()):optional())),(((self):exponent()):_slash((self):radix())):optional())):_at_at(\"number\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"methodParserParts",0
,(
function(self)
local expression=nil
local separator=nil
local comment=nil
local ws=nil
local identifier=nil
local keyword=nil
local stSelectorPart=nil
local string=nil
local symbol=nil
local number=nil
local variable=nil
local reference=nil
local globalReference=nil
local literal=nil
local returnExpression=nil
local expressionParser=nil
local keyword=nil
local unarySelector=nil
local binarySelector=nil
local keywordPattern=nil
local unaryPattern=nil
local binaryPattern=nil
local assignment=nil
local temps=nil
local blockParamList=nil
local block=nil
local expression=nil
local expressions=nil
local subexpression=nil
local statements=nil
local sequence=nil
local operand=nil
local unaryMessage=nil
local unarySend=nil
local unaryTail=nil
local binaryMessage=nil
local binarySend=nil
local binaryTail=nil
local keywordMessage=nil
local keywordSend=nil
local keywordPair=nil
local cascade=nil
local message=nil
local inline=nil
local jsIndex=nil
local character=nil
local directive=nil
local arrayLiteral=nil
local arrayVariable=nil
local literalArray=nil
local aSymbol=nil
local stIdentifier=nil
local jsIdentifier=nil
local noReference=nil
local alienReference=nil
local alienIdentifier=nil
local emptyArray=nil
local hashedString=nil
local annotation=nil
local argument=nil
(function()
	local _s8_value=((self):separator())
	separator=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):comment())
	comment=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((((separator):_slash(comment)):_at_at("sp")):star()):_at_at("ws"))
	ws=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((self),"'")),(smalltalk.PPStStringParser):new())):_at_at("string")):_eq_eq_gt((function(node)
return ((smalltalk.ValueNode):with_((node):second()))
end)))
	string=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((self),smalltalk:object_comma_(("#"),"'"))),(smalltalk.PPStStringParser):new())):_at_at("hashedString")):_eq_eq_gt((function(node)
return ((smalltalk.ValueNode):with_((node):second()))
end)))
	hashedString=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((smalltalk:object_comma_(((self):nameStart()),((self):symbolCharParser()):star())):_at_at("stId")):flatten()):_at_at("stIdentifier"))
	stIdentifier=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((smalltalk:object_comma_(((self):upNameStart()),((self):symbolCharParser()):star())):_at_at("stSel")):flatten()):_at_at("stSelectorPart"))
	stSelectorPart=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((smalltalk:object_comma_((smalltalk:object_comma_((self),"#")),((self):jsSymbolCharParser()):plus())):_at_at("jsId")):flatten()):_at_at("jsIdentifier"))
	jsIdentifier=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((((self):jsSymbolCharParser()):plus()):flatten()):_at_at("alienIdentifier"))
	alienIdentifier=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((jsIdentifier):_slash(stIdentifier)):_at_at("identifier"))
	identifier=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_(((stSelectorPart):_slash(identifier)),":")):flatten()):_at_at("keyword"))
	keyword=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((string):_slash(stIdentifier))
	annotation=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk:object_comma_((identifier),(smalltalk:object_comma_((smalltalk:object_comma_((self),"@")),annotation)):star())):_eq_eq_gt((function(node)
return (((smalltalk.ValueNode):with_((node):first())):annotations_((node):last()))
end)))
	argument=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((self),"#")),(((self):extendedSymbolCharParser()):plus()):flatten())):_at_at("symbol")):_eq_eq_gt((function(node)
return ((smalltalk.ValueNode):with_((node):second()))
end)))
	symbol=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((self):number()):flatten()):_eq_eq_gt((function(node)
return ((smalltalk.ValueNode):with_((node):asNumber()))
end)))
	number=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((self),"$")),(smalltalk.PPAnyParser):new())):_at_at("character")):_eq_eq_gt((function(node)
return ((smalltalk.ValueNode):with_((node):at_((2))))
end)))
	character=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):directive_ws_(number, ws))
	directive=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((identifier):_eq_eq_gt((function(token)
return (((smalltalk.VariableNode):new()):value_(token))
end)))
	variable=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((self),"#")):_at_at("noReference")):_eq_eq_gt((function(token)
return ((smalltalk.NoReceiverNode):new())
end)))
	noReference=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk.PPDelegateParser):new()):_at_at("literalDelegate"))
	literal=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk.PPDelegateParser):new()):_at_at("expressionDelegate"))
	expression=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((((smalltalk:object_comma_((self),"true")):_eq_eq_gt((function(node)
return ((smalltalk.ValueNode):with_(true))
end))):_slash((smalltalk:object_comma_((self),"false")):_eq_eq_gt((function(node)
return ((smalltalk.ValueNode):with_(false))
end)))):_slash((smalltalk:object_comma_((self),"nil")):_eq_eq_gt((function(node)
return ((smalltalk.ValueNode):with_(nil))
end)))):_at_at("arrayVariable"))
	arrayVariable=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((((self):extendedSymbolCharParser()):plus()):flatten()):_at_at("aSymbol")):_eq_eq_gt((function(node)
return ((smalltalk.ValueNode):with_(node))
end)))
	aSymbol=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((literal):_slash(arrayVariable)):_slash(aSymbol)):_slash(variable))
	arrayLiteral=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_(((self):gc_("#(")),(smalltalk:object_comma_((smalltalk:object_comma_((ws),arrayLiteral)),ws)):star())),")")):_at_at("literalArray")):_eq_eq_gt((function(node)
return ((smalltalk.ValueNode):with_((smalltalk.Array):withAll_(((node):second()):collect_((function(each)
return (((each):second()):value())
end)))))
end)))
	literalArray=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),"#(")),ws)),")")):_at_at("emptyArray")):_eq_eq_gt((function(node)
return ((smalltalk.ValueNode):with_((smalltalk.Array):new()))
end)))
	emptyArray=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((self):globalNameParser()):_at_at("globalReference")):_eq_eq_gt((function(token)
(self):gc();return (((smalltalk.GlobalReferenceNode):new()):value_(token))
end)))
	globalReference=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),"#{")),alienIdentifier)),"}")):_at_at("alienReference")):_eq_eq_gt((function(node)
return (((smalltalk.AlienNode):new()):value_((node):second()))
end)))
	alienReference=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk:object_comma_((smalltalk:object_comma_(((self):gc_("[")),expression)),"]")):_slash(smalltalk:object_comma_((smalltalk:object_comma_((self),"#")),identifier)))
	jsIndex=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_(((((variable):_slash(globalReference)):_slash(alienReference)):_at_at("ref")),(jsIndex):star())):_at_at("reference")):_eq_eq_gt((function(nodes)
return (((nodes):first()):indexedAs_(nodes))
end)))
	reference=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_(((self):optional_("#")),((self):binaryCharParser()):plus())):flatten()):_at_at("binarySelector"))
	binarySelector=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((stSelectorPart):_slash(identifier)):_at_at("unarySelector"))
	unarySelector=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((((smalltalk:object_comma_((smalltalk:object_comma_(((smalltalk:object_comma_((ws),keyword)):_at_at("kw")),ws)),argument)):_at_at("kwId")):plus()):_at_at("keywordPattern")):_eq_eq_gt((function(nodes)
return ((smalltalk.Array):with_with_(((nodes):collect_((function(each)
return ((each):at_((2)))
end))):join_(""), (nodes):collect_((function(each)
return ((each):at_((4)))
end))))
end)))
	keywordPattern=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((ws),binarySelector)),ws)),argument)):_at_at("binaryPattern")):_eq_eq_gt((function(node)
return ((smalltalk.Array):with_with_((node):second(), (smalltalk.Array):with_((node):fourth())))
end)))
	binaryPattern=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((ws),unarySelector)):_at_at("unaryPattern")):_eq_eq_gt((function(node)
return ((smalltalk.Array):with_with_((node):second(), (smalltalk.Array):new()))
end)))
	unaryPattern=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((expression),(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((ws),".")),ws)),expression)):_at_at("anExpression")):_eq_eq_gt((function(node)
return ((node):fourth())
end))):star())):_at_at("expressions")):_eq_eq_gt((function(node)
return (smalltalk:object_comma_(((smalltalk.Array):with_((node):first())),(node):second()))
end)))
	expressions=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((reference),ws)),":=")),ws)),expression)):_at_at("assignment")):_eq_eq_gt((function(node)
return ((
function(_s8_rec)
(_s8_rec):left_((node):first());
return ((_s8_rec):right_((node):at_((5))))
end
)((smalltalk.AssignmentNode):new()))
end)))
	assignment=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((((self):gc_("^")):_at_at("caret")),(smalltalk:object_comma_((smalltalk:object_comma_((self),"@")),annotation)):star())),ws)),expression)),ws)),(("."):asParser()):optional())):_at_at("returnExpression")):_eq_eq_gt((function(node)
return ((
function(_s8_rec)
(_s8_rec):addNode_(((node):at_((4))):annotations_((node):second()));
return ((_s8_rec):yourself())
end
)((smalltalk.ReturnNode):new()))
end)))
	returnExpression=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),"|")),(smalltalk:object_comma_((ws),argument)):star())),ws)),"|")):_at_at("temps")):_eq_eq_gt((function(node)
return (((node):second()):collect_((function(each)
return ((each):second())
end)))
end)))
	temps=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),":")),ws)),argument)),ws)):plus()),"|")):_at_at("blockParamList")):_eq_eq_gt((function(node)
return (((node):first()):collect_((function(each)
return ((each):third())
end)))
end)))
	blockParamList=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):gc_("(")),ws)),expression)),ws)),")")):_at_at("subexpression")):_eq_eq_gt((function(node)
return ((node):third())
end)))
	subexpression=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((((returnExpression):_at_at("stat1")):_eq_eq_gt((function(node)
return ((smalltalk.Array):with_(node))
end))):_slash(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((expressions),ws)),".")),ws)),returnExpression)):_at_at("stat2")):_eq_eq_gt((function(node)
return ((
function(_s8_rec)
(_s8_rec):add_((node):at_((5)));
return ((_s8_rec):yourself())
end
)((node):first()))
end)))):_slash(((smalltalk:object_comma_((expressions),(self):optional_("."))):_at_at("stat3")):_eq_eq_gt((function(node)
return ((node):first())
end))))
	statements=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((((temps):optional()):_at_at("temps")),ws)),(directive):star())),ws)),(statements):optional())),ws)):_at_at("sequence")):_eq_eq_gt((function(node)
return ((
function(_s8_rec)
(_s8_rec):temps_((node):first());
(_s8_rec):directives_((node):third());
(_s8_rec):nodes_((node):at_((5)));
return ((_s8_rec):yourself())
end
)((smalltalk.ExtendedSequenceNode):new()))
end)))
	sequence=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):gc_("[")),ws)),(blockParamList):optional())),ws)),(sequence):optional())),ws)),"]")):_at_at("block")):_eq_eq_gt((function(node)
return ((
function(_s8_rec)
(_s8_rec):parameters_((node):third());
return ((_s8_rec):addNode_(((node):at_((5))):asBlockSequenceNode()))
end
)((smalltalk.BlockNode):new()))
end)))
	block=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((((block):_slash(literal)):_slash(reference)):_slash(noReference)):_slash(subexpression)):_at_at("operand"))
	operand=_s8_value; return _s8_value
end)()
;
(literal):parser_((((((((symbol):_slash(number)):_slash(string)):_slash(character)):_slash(emptyArray)):_slash(literalArray)):_slash(hashedString)):_at_at("literalDelegate"))
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((ws),unarySelector)),(smalltalk:booleanNot_((smalltalk:object_comma_((self),":")))))):_at_at("unaryMessage")):_eq_eq_gt((function(node)
return (((smalltalk.SendNode):new()):selector_((node):second()))
end)))
	unaryMessage=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk.PPDelegateParser):new()):_at_at("unaryTailDelegate"))
	unaryTail=_s8_value; return _s8_value
end)()
;
(unaryTail):parser_(((smalltalk:object_comma_((unaryMessage),(unaryTail):optional())):_at_at("unaryTailDelegate")):_eq_eq_gt((function(node)
return ((function() if (((((((node):second()))==nil)))==true) then return ((function()
return ((node):first())
end))() else return ((function()
return (((node):second()):valueForReceiver_((node):first()))
end))() end end)())
end)))
;
(function()
	local _s8_value=(((smalltalk:object_comma_((operand),(unaryTail):optional())):_at_at("unarySend")):_eq_eq_gt((function(node)
return ((function() if (((((((node):second()))==nil)))==true) then return ((function()
return ((node):first())
end))() else return ((function()
return (((node):second()):valueForReceiver_((node):first()))
end))() end end)())
end)))
	unarySend=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((ws),binarySelector)),ws)),(unarySend):_slash(operand))):_at_at("binaryMessage")):_eq_eq_gt((function(node)
return ((
function(_s8_rec)
(_s8_rec):selector_((node):second());
return ((_s8_rec):arguments_((smalltalk.Array):with_((node):fourth())))
end
)((smalltalk.SendNode):new()))
end)))
	binaryMessage=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk.PPDelegateParser):new()):_at_at("binaryTailDelegate"))
	binaryTail=_s8_value; return _s8_value
end)()
;
(binaryTail):parser_((smalltalk:object_comma_((binaryMessage),(binaryTail):optional())):_eq_eq_gt((function(node)
return ((function() if (((((((node):second()))==nil)))==true) then return ((function()
return ((node):first())
end))() else return ((function()
return (((node):second()):valueForReceiver_((node):first()))
end))() end end)())
end)))
;
(function()
	local _s8_value=(((smalltalk:object_comma_((unarySend),(binaryTail):optional())):_at_at("binarySend")):_eq_eq_gt((function(node)
return ((function() if (((((((node):second()))==nil)))==true) then return ((function()
return ((node):first())
end))() else return ((function()
return (((node):second()):valueForReceiver_((node):first()))
end))() end end)())
end)))
	binarySend=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk:object_comma_((smalltalk:object_comma_((keyword),ws)),binarySend)):_at_at("keywordPair"))
	keywordPair=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((smalltalk:object_comma_((ws),keywordPair)):plus()):_at_at("keywordMessage")):_eq_eq_gt((function(nodes)
return ((
function(_s8_rec)
(_s8_rec):selector_(((nodes):collect_((function(each)
return (((each):second()):first())
end))):join_(""));
return ((_s8_rec):arguments_((nodes):collect_((function(each)
return (((each):second()):third())
end))))
end
)((smalltalk.SendNode):new()))
end)))
	keywordMessage=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((binarySend),keywordMessage)):_at_at("keywordSend")):_eq_eq_gt((function(node)
return (((node):second()):valueForReceiver_((node):first()))
end)))
	keywordSend=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((binaryMessage):_slash(unaryMessage)):_slash(keywordMessage)):_at_at("message"))
	message=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((((keywordSend):_slash(binarySend)):_at_at("preCascade")),(smalltalk:object_comma_((smalltalk:object_comma_((ws),";")),message)):plus())):_at_at("cascade")):_eq_eq_gt((function(node)
return (((node):first()):cascadeNodeWithMessages_(((node):second()):collect_((function(each)
return ((each):third())
end))))
end)))
	cascade=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),"{")),(smalltalk:object_comma_((smalltalk:object_comma_(((stIdentifier):optional()),string)),ws)):plus())),"}")):_at_at("inline")):_eq_eq_gt((function(node)
return (((smalltalk.InlineNode):new()):sources_(((node):second()):collect_((function(each)
return ((smalltalk.Array):with_with_((each):first(), (((each):second()):value()):replace_with_("''", "'")))
end))))
end)))
	inline=_s8_value; return _s8_value
end)()
;
(expression):parser_((((((assignment):_slash(cascade)):_slash(keywordSend)):_slash(binarySend)):_slash(inline)):_at_at("expression"))
;
return ((smalltalk.Array):with_with_with_(ws, (((keywordPattern):_slash(binaryPattern)):_slash(unaryPattern)):_at_at("header"), ((sequence):optional()):_at_at("body")))
;
end
)
,"private","methodParserParts\r\n\t\" Private - Return a parser for parts of a method. \"\r\n\r\n | expression separator comment ws identifier keyword stSelectorPart\r\n  string symbol number variable reference globalReference\r\n  literal returnExpression expressionParser keyword unarySelector\r\n  binarySelector keywordPattern unaryPattern binaryPattern assignment\r\n  temps blockParamList block expression expressions subexpression\r\n  statements sequence operand unaryMessage unarySend unaryTail\r\n  binaryMessage binarySend binaryTail keywordMessage keywordSend\r\n  keywordPair cascade message inline jsIndex character directive\r\n  arrayLiteral arrayVariable literalArray aSymbol\r\n  stIdentifier jsIdentifier noReference alienReference\r\n  alienIdentifier emptyArray hashedString annotation argument\r\n |\r\n\r\n separator := self separator.\r\n comment := self comment.\r\n ws := (separator / comment @@#sp) star @@#ws.\r\n \r\n string := self ,$' ,PPStStringParser new @@#string\r\n  ==> [:node| ValueNode with: node second ].\r\n \r\n hashedString := self ,($#,$') ,PPStStringParser new @@#hashedString\r\n  ==> [:node| ValueNode with: node second ].\r\n\r\n stIdentifier := (self nameStart, self symbolCharParser star @@#stId) flatten @@#stIdentifier.\r\n stSelectorPart := (self upNameStart, self symbolCharParser star @@#stSel) flatten @@#stSelectorPart.\r\n jsIdentifier := (self ,$# ,self jsSymbolCharParser plus @@#jsId) flatten @@#jsIdentifier.\r\n alienIdentifier := self jsSymbolCharParser plus flatten @@#alienIdentifier.\r\n identifier := jsIdentifier / stIdentifier @@#identifier.\r\n\r\n keyword := (stSelectorPart / identifier, $:) flatten @@#keyword.\r\n\r\n annotation := string / stIdentifier.\r\n argument := identifier ,(self ,$@ ,annotation) star ==> [:node|\r\n\t(ValueNode with: node first) annotations: node last\r\n ].\r\n\r\n symbol := self ,$# ,self extendedSymbolCharParser plus flatten @@#symbol\r\n  ==> [:node | ValueNode with: node second ].\r\n\r\n number := self number flatten\r\n  ==> [:node | ValueNode with: node asNumber ].\r\n\r\n character := self ,$$ ,PPAnyParser new @@#character\r\n  ==> [:node | ValueNode with: (node at: 2) ].\r\n\r\n directive := self directive: number ws: ws.\r\n\r\n variable := identifier ==> [:token | VariableNode new value: token ].\r\n noReference := self ,$# @@#noReference ==> [:token | NoReceiverNode new ].\r\n literal := PPDelegateParser new @@#literalDelegate.\r\n expression := PPDelegateParser new @@#expressionDelegate.\r\n\r\n arrayVariable :=\r\n    (self ,#true  ==> [:node| ValueNode with: true  ])\r\n  / (self ,#false ==> [:node| ValueNode with: false ])\r\n  / (self ,#nil   ==> [:node| ValueNode with: nil   ])\r\n  @@#arrayVariable.\r\n\r\n aSymbol := self extendedSymbolCharParser plus flatten @@#aSymbol\r\n  ==> [:node| ValueNode with: node ].\r\n\r\n arrayLiteral := literal / arrayVariable / aSymbol / variable.\r\n literalArray := (self gc: '#(')\r\n\t,(ws ,arrayLiteral ,ws) star ,$)\r\n\t@@#literalArray\r\n  ==> [:node | ValueNode with: (Array\r\n\twithAll: (node second\r\n\t\tcollect: [:each | each second value])) ].\r\n emptyArray := self ,'#(' ,ws ,$) @@#emptyArray\r\n  ==> [:node | ValueNode with: Array new ].\r\n\r\n globalReference := self globalNameParser @@#globalReference\r\n  ==> [:token | self gc. GlobalReferenceNode new value: token ].\r\n alienReference := self ,'#{' ,alienIdentifier, $} @@#alienReference\r\n  ==> [:node | AlienNode new value: node second ].\r\n\r\n jsIndex := (self gc: $[) ,expression, $] / (self ,$# ,identifier).\r\n reference := (variable / globalReference / alienReference @@#ref)\r\n\t,jsIndex star @@#reference\r\n  ==> [:nodes | nodes first indexedAs: nodes ].\r\n\r\n binarySelector := (\r\n\t(self optional: $#)\r\n\t,self binaryCharParser plus\r\n\t) flatten @@#binarySelector.\r\n\r\n unarySelector := stSelectorPart / identifier @@#unarySelector.\r\n\r\n keywordPattern :=\r\n\t(ws, keyword  @@#kw ,\r\n\t ws, argument @@#kwId\r\n\t) plus @@#keywordPattern\r\n  ==> [:nodes | Array\r\n   with: ((nodes collect: [:each | each at: 2]) join: '')\r\n   with: (nodes collect: [:each | each at: 4]) ].\r\n\r\n binaryPattern := ws, binarySelector, ws, argument @@#binaryPattern\r\n  ==> [:node | Array with: node second with: (Array with: node fourth)].\r\n\r\n unaryPattern := ws, unarySelector @@#unaryPattern\r\n  ==> [:node | Array with: node second with: Array new].\r\n \r\n expressions := expression, ((ws, $., ws, expression) @@#anExpression\r\n  ==> [:node | node fourth]) star @@#expressions\r\n  ==> [:node | (Array with: node first) ,node second ].\r\n\r\n assignment := reference, ws, ':=', ws, expression @@#assignment\r\n  ==> [:node | AssignmentNode new left: node first; right: (node at: 5)].\r\n\r\nreturnExpression := (self gc: $^) @@#caret\r\n\t,(self ,$@ ,annotation) star\r\n\t, ws, expression, ws\r\n\t, $. asParser optional\t@@#returnExpression\r\n     ==> [:node | ReturnNode new\r\n\t\taddNode: ((node at:4) annotations: node second);\r\n\t\tyourself ].\r\n\r\n temps := self, $|, (ws, argument) star, ws, $| @@#temps\r\n  ==> [:node | node second collect: [:each | each second ] ].\r\n\r\n blockParamList := (self ,$: ,ws ,argument ,ws) plus ,$| @@#blockParamList\r\n  ==> [:node | node first collect: [:each | each third ] ].\r\n\r\n subexpression := (self gc: $() ,ws ,expression ,ws ,$) @@#subexpression\r\n  ==> [:node | node third ].\r\n\r\n statements := (returnExpression @@#stat1 ==> [:node | Array with: node])\r\n  / (expressions ,ws ,$. ,ws ,returnExpression @@#stat2 ==> [:node | node first add: (node at: 5); yourself ])\r\n  / (expressions ,(self optional: $.) @@#stat3 ==> [:node | node first ]).\r\n\r\n sequence := temps optional @@#temps\r\n\t, ws, directive star\r\n\t, ws, statements optional\r\n\t, ws @@#sequence\r\n  ==> [:node | ExtendedSequenceNode new\r\n\ttemps: node first;\r\n\tdirectives: node third;\r\n\tnodes: (node at: 5);\r\n\tyourself ].\r\n\r\n block := (self gc: $[)\r\n\t,ws ,blockParamList optional\r\n\t,ws ,sequence optional\r\n\t,ws ,$] @@#block\r\n  ==> [:node | BlockNode new\r\n\tparameters: node third;\r\n\taddNode: (node at: 5) asBlockSequenceNode ].\r\n\r\n operand := block / literal / reference / noReference / subexpression @@#operand.\r\n\r\n literal parser: symbol / number / string / character\r\n  / emptyArray / literalArray / hashedString @@#literalDelegate.\r\n\r\n unaryMessage := ws ,unarySelector ,(self ,$:) not @@#unaryMessage\r\n  ==> [:node | SendNode new selector: node second ].\r\n\r\n unaryTail := PPDelegateParser new @@#unaryTailDelegate.\r\n unaryTail parser: (unaryMessage, unaryTail optional @@#unaryTailDelegate\r\n  ==> [:node | node second isNil ifTrue: [ node first ]\r\n   ifFalse: [ node second valueForReceiver: node first ] ]).\r\n\r\n unarySend := operand, unaryTail optional @@#unarySend\r\n  ==> [:node | node second isNil ifTrue: [ node first ]\r\n   ifFalse: [ node second valueForReceiver: node first ] ].\r\n\r\n binaryMessage := ws, binarySelector, ws, (unarySend / operand) @@#binaryMessage\r\n  ==> [:node | SendNode new\r\n   selector: node second;\r\n   arguments: (Array with: node fourth) ].\r\n\r\n binaryTail := PPDelegateParser new @@#binaryTailDelegate.\r\n binaryTail parser: (binaryMessage, binaryTail optional\r\n  ==> [:node | node second isNil ifTrue: [ node first ]\r\n   ifFalse: [ node second valueForReceiver: node first ] ]).\r\n\r\n binarySend := unarySend, binaryTail optional @@#binarySend\r\n  ==> [:node | node second isNil ifTrue: [ node first ]\r\n   ifFalse: [ node second valueForReceiver: node first ] ].\r\n\r\n keywordPair := keyword, ws, binarySend @@#keywordPair.\r\n\r\n keywordMessage := (ws, keywordPair) plus @@#keywordMessage\r\n  ==> [:nodes | SendNode new\r\n   selector: ((nodes collect: [:each | each second first]) join: '');\r\n   arguments: (nodes collect: [:each | each second third])].\r\n\r\n keywordSend := binarySend, keywordMessage @@#keywordSend\r\n  ==> [:node | node second valueForReceiver: node first].\r\n\r\n message := binaryMessage / unaryMessage / keywordMessage @@#message.\r\n\r\n cascade := (keywordSend / binarySend @@#preCascade), (ws, $;, message) plus @@#cascade\r\n  ==> [:node | node first cascadeNodeWithMessages: \r\n   (node second collect: [:each | each third ])].\r\n\r\n inline := self ,${\r\n\t,(stIdentifier optional ,string ,ws) plus\r\n\t,$} @@#inline\r\n  ==> [:node | InlineNode new sources: (node second collect: [:each|\r\n\t\tArray\twith: each first with:\r\n\t\t\t(each second value replace: '''''' with: '''')\r\n\t\t])\r\n\t].\r\n\r\n expression parser: assignment / cascade / keywordSend / binarySend / inline @@#expression.\r\n\r\n\t^Array\twith: ws\r\n\t\twith: keywordPattern / binaryPattern / unaryPattern @@#header\r\n\t\twith: sequence optional @@#body","(\r\nfunction(self)\r\nlocal expression=nil\r\nlocal separator=nil\r\nlocal comment=nil\r\nlocal ws=nil\r\nlocal identifier=nil\r\nlocal keyword=nil\r\nlocal stSelectorPart=nil\r\nlocal string=nil\r\nlocal symbol=nil\r\nlocal number=nil\r\nlocal variable=nil\r\nlocal reference=nil\r\nlocal globalReference=nil\r\nlocal literal=nil\r\nlocal returnExpression=nil\r\nlocal expressionParser=nil\r\nlocal keyword=nil\r\nlocal unarySelector=nil\r\nlocal binarySelector=nil\r\nlocal keywordPattern=nil\r\nlocal unaryPattern=nil\r\nlocal binaryPattern=nil\r\nlocal assignment=nil\r\nlocal temps=nil\r\nlocal blockParamList=nil\r\nlocal block=nil\r\nlocal expression=nil\r\nlocal expressions=nil\r\nlocal subexpression=nil\r\nlocal statements=nil\r\nlocal sequence=nil\r\nlocal operand=nil\r\nlocal unaryMessage=nil\r\nlocal unarySend=nil\r\nlocal unaryTail=nil\r\nlocal binaryMessage=nil\r\nlocal binarySend=nil\r\nlocal binaryTail=nil\r\nlocal keywordMessage=nil\r\nlocal keywordSend=nil\r\nlocal keywordPair=nil\r\nlocal cascade=nil\r\nlocal message=nil\r\nlocal inline=nil\r\nlocal jsIndex=nil\r\nlocal character=nil\r\nlocal directive=nil\r\nlocal arrayLiteral=nil\r\nlocal arrayVariable=nil\r\nlocal literalArray=nil\r\nlocal aSymbol=nil\r\nlocal stIdentifier=nil\r\nlocal jsIdentifier=nil\r\nlocal noReference=nil\r\nlocal alienReference=nil\r\nlocal alienIdentifier=nil\r\nlocal emptyArray=nil\r\nlocal hashedString=nil\r\nlocal annotation=nil\r\nlocal argument=nil\r\n(function()\r\n\tlocal _s8_value=((self):separator())\r\n\tseparator=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):comment())\r\n\tcomment=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((((separator):_slash(comment)):_at_at(\"sp\")):star()):_at_at(\"ws\"))\r\n\tws=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((self),\"'\")),(smalltalk.PPStStringParser):new())):_at_at(\"string\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.ValueNode):with_((node):second()))\r\nend)))\r\n\tstring=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((self),smalltalk:object_comma_((\"#\"),\"'\"))),(smalltalk.PPStStringParser):new())):_at_at(\"hashedString\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.ValueNode):with_((node):second()))\r\nend)))\r\n\thashedString=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((smalltalk:object_comma_(((self):nameStart()),((self):symbolCharParser()):star())):_at_at(\"stId\")):flatten()):_at_at(\"stIdentifier\"))\r\n\tstIdentifier=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((smalltalk:object_comma_(((self):upNameStart()),((self):symbolCharParser()):star())):_at_at(\"stSel\")):flatten()):_at_at(\"stSelectorPart\"))\r\n\tstSelectorPart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((smalltalk:object_comma_((smalltalk:object_comma_((self),\"#\")),((self):jsSymbolCharParser()):plus())):_at_at(\"jsId\")):flatten()):_at_at(\"jsIdentifier\"))\r\n\tjsIdentifier=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((((self):jsSymbolCharParser()):plus()):flatten()):_at_at(\"alienIdentifier\"))\r\n\talienIdentifier=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((jsIdentifier):_slash(stIdentifier)):_at_at(\"identifier\"))\r\n\tidentifier=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_(((stSelectorPart):_slash(identifier)),\":\")):flatten()):_at_at(\"keyword\"))\r\n\tkeyword=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((string):_slash(stIdentifier))\r\n\tannotation=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk:object_comma_((identifier),(smalltalk:object_comma_((smalltalk:object_comma_((self),\"@\")),annotation)):star())):_eq_eq_gt((function(node)\r\nreturn (((smalltalk.ValueNode):with_((node):first())):annotations_((node):last()))\r\nend)))\r\n\targument=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((self),\"#\")),(((self):extendedSymbolCharParser()):plus()):flatten())):_at_at(\"symbol\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.ValueNode):with_((node):second()))\r\nend)))\r\n\tsymbol=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((self):number()):flatten()):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.ValueNode):with_((node):asNumber()))\r\nend)))\r\n\tnumber=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((self),\"$\")),(smalltalk.PPAnyParser):new())):_at_at(\"character\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.ValueNode):with_((node):at_((2))))\r\nend)))\r\n\tcharacter=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):directive_ws_(number, ws))\r\n\tdirective=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((identifier):_eq_eq_gt((function(token)\r\nreturn (((smalltalk.VariableNode):new()):value_(token))\r\nend)))\r\n\tvariable=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((self),\"#\")):_at_at(\"noReference\")):_eq_eq_gt((function(token)\r\nreturn ((smalltalk.NoReceiverNode):new())\r\nend)))\r\n\tnoReference=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.PPDelegateParser):new()):_at_at(\"literalDelegate\"))\r\n\tliteral=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.PPDelegateParser):new()):_at_at(\"expressionDelegate\"))\r\n\texpression=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((((smalltalk:object_comma_((self),\"true\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.ValueNode):with_(true))\r\nend))):_slash((smalltalk:object_comma_((self),\"false\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.ValueNode):with_(false))\r\nend)))):_slash((smalltalk:object_comma_((self),\"nil\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.ValueNode):with_(nil))\r\nend)))):_at_at(\"arrayVariable\"))\r\n\tarrayVariable=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((((self):extendedSymbolCharParser()):plus()):flatten()):_at_at(\"aSymbol\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.ValueNode):with_(node))\r\nend)))\r\n\taSymbol=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((literal):_slash(arrayVariable)):_slash(aSymbol)):_slash(variable))\r\n\tarrayLiteral=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_(((self):gc_(\"#(\")),(smalltalk:object_comma_((smalltalk:object_comma_((ws),arrayLiteral)),ws)):star())),\")\")):_at_at(\"literalArray\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.ValueNode):with_((smalltalk.Array):withAll_(((node):second()):collect_((function(each)\r\nreturn (((each):second()):value())\r\nend)))))\r\nend)))\r\n\tliteralArray=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),\"#(\")),ws)),\")\")):_at_at(\"emptyArray\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.ValueNode):with_((smalltalk.Array):new()))\r\nend)))\r\n\temptyArray=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((self):globalNameParser()):_at_at(\"globalReference\")):_eq_eq_gt((function(token)\r\n(self):gc();return (((smalltalk.GlobalReferenceNode):new()):value_(token))\r\nend)))\r\n\tglobalReference=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),\"#{\")),alienIdentifier)),\"}\")):_at_at(\"alienReference\")):_eq_eq_gt((function(node)\r\nreturn (((smalltalk.AlienNode):new()):value_((node):second()))\r\nend)))\r\n\talienReference=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk:object_comma_((smalltalk:object_comma_(((self):gc_(\"[\")),expression)),\"]\")):_slash(smalltalk:object_comma_((smalltalk:object_comma_((self),\"#\")),identifier)))\r\n\tjsIndex=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_(((((variable):_slash(globalReference)):_slash(alienReference)):_at_at(\"ref\")),(jsIndex):star())):_at_at(\"reference\")):_eq_eq_gt((function(nodes)\r\nreturn (((nodes):first()):indexedAs_(nodes))\r\nend)))\r\n\treference=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_(((self):optional_(\"#\")),((self):binaryCharParser()):plus())):flatten()):_at_at(\"binarySelector\"))\r\n\tbinarySelector=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((stSelectorPart):_slash(identifier)):_at_at(\"unarySelector\"))\r\n\tunarySelector=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((((smalltalk:object_comma_((smalltalk:object_comma_(((smalltalk:object_comma_((ws),keyword)):_at_at(\"kw\")),ws)),argument)):_at_at(\"kwId\")):plus()):_at_at(\"keywordPattern\")):_eq_eq_gt((function(nodes)\r\nreturn ((smalltalk.Array):with_with_(((nodes):collect_((function(each)\r\nreturn ((each):at_((2)))\r\nend))):join_(\"\"), (nodes):collect_((function(each)\r\nreturn ((each):at_((4)))\r\nend))))\r\nend)))\r\n\tkeywordPattern=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((ws),binarySelector)),ws)),argument)):_at_at(\"binaryPattern\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.Array):with_with_((node):second(), (smalltalk.Array):with_((node):fourth())))\r\nend)))\r\n\tbinaryPattern=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((ws),unarySelector)):_at_at(\"unaryPattern\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.Array):with_with_((node):second(), (smalltalk.Array):new()))\r\nend)))\r\n\tunaryPattern=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((expression),(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((ws),\".\")),ws)),expression)):_at_at(\"anExpression\")):_eq_eq_gt((function(node)\r\nreturn ((node):fourth())\r\nend))):star())):_at_at(\"expressions\")):_eq_eq_gt((function(node)\r\nreturn (smalltalk:object_comma_(((smalltalk.Array):with_((node):first())),(node):second()))\r\nend)))\r\n\texpressions=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((reference),ws)),\":=\")),ws)),expression)):_at_at(\"assignment\")):_eq_eq_gt((function(node)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):left_((node):first());\r\nreturn ((_s8_rec):right_((node):at_((5))))\r\nend\r\n)((smalltalk.AssignmentNode):new()))\r\nend)))\r\n\tassignment=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((((self):gc_(\"^\")):_at_at(\"caret\")),(smalltalk:object_comma_((smalltalk:object_comma_((self),\"@\")),annotation)):star())),ws)),expression)),ws)),((\".\"):asParser()):optional())):_at_at(\"returnExpression\")):_eq_eq_gt((function(node)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):addNode_(((node):at_((4))):annotations_((node):second()));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.ReturnNode):new()))\r\nend)))\r\n\treturnExpression=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),\"|\")),(smalltalk:object_comma_((ws),argument)):star())),ws)),\"|\")):_at_at(\"temps\")):_eq_eq_gt((function(node)\r\nreturn (((node):second()):collect_((function(each)\r\nreturn ((each):second())\r\nend)))\r\nend)))\r\n\ttemps=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),\":\")),ws)),argument)),ws)):plus()),\"|\")):_at_at(\"blockParamList\")):_eq_eq_gt((function(node)\r\nreturn (((node):first()):collect_((function(each)\r\nreturn ((each):third())\r\nend)))\r\nend)))\r\n\tblockParamList=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):gc_(\"(\")),ws)),expression)),ws)),\")\")):_at_at(\"subexpression\")):_eq_eq_gt((function(node)\r\nreturn ((node):third())\r\nend)))\r\n\tsubexpression=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((((returnExpression):_at_at(\"stat1\")):_eq_eq_gt((function(node)\r\nreturn ((smalltalk.Array):with_(node))\r\nend))):_slash(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((expressions),ws)),\".\")),ws)),returnExpression)):_at_at(\"stat2\")):_eq_eq_gt((function(node)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_((node):at_((5)));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((node):first()))\r\nend)))):_slash(((smalltalk:object_comma_((expressions),(self):optional_(\".\"))):_at_at(\"stat3\")):_eq_eq_gt((function(node)\r\nreturn ((node):first())\r\nend))))\r\n\tstatements=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((((temps):optional()):_at_at(\"temps\")),ws)),(directive):star())),ws)),(statements):optional())),ws)):_at_at(\"sequence\")):_eq_eq_gt((function(node)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):temps_((node):first());\r\n(_s8_rec):directives_((node):third());\r\n(_s8_rec):nodes_((node):at_((5)));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.ExtendedSequenceNode):new()))\r\nend)))\r\n\tsequence=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):gc_(\"[\")),ws)),(blockParamList):optional())),ws)),(sequence):optional())),ws)),\"]\")):_at_at(\"block\")):_eq_eq_gt((function(node)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):parameters_((node):third());\r\nreturn ((_s8_rec):addNode_(((node):at_((5))):asBlockSequenceNode()))\r\nend\r\n)((smalltalk.BlockNode):new()))\r\nend)))\r\n\tblock=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((((block):_slash(literal)):_slash(reference)):_slash(noReference)):_slash(subexpression)):_at_at(\"operand\"))\r\n\toperand=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(literal):parser_((((((((symbol):_slash(number)):_slash(string)):_slash(character)):_slash(emptyArray)):_slash(literalArray)):_slash(hashedString)):_at_at(\"literalDelegate\"))\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((ws),unarySelector)),(smalltalk:booleanNot_((smalltalk:object_comma_((self),\":\")))))):_at_at(\"unaryMessage\")):_eq_eq_gt((function(node)\r\nreturn (((smalltalk.SendNode):new()):selector_((node):second()))\r\nend)))\r\n\tunaryMessage=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.PPDelegateParser):new()):_at_at(\"unaryTailDelegate\"))\r\n\tunaryTail=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(unaryTail):parser_(((smalltalk:object_comma_((unaryMessage),(unaryTail):optional())):_at_at(\"unaryTailDelegate\")):_eq_eq_gt((function(node)\r\nreturn ((function() if (((((((node):second()))==nil)))==true) then return ((function()\r\nreturn ((node):first())\r\nend))() else return ((function()\r\nreturn (((node):second()):valueForReceiver_((node):first()))\r\nend))() end end)())\r\nend)))\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((operand),(unaryTail):optional())):_at_at(\"unarySend\")):_eq_eq_gt((function(node)\r\nreturn ((function() if (((((((node):second()))==nil)))==true) then return ((function()\r\nreturn ((node):first())\r\nend))() else return ((function()\r\nreturn (((node):second()):valueForReceiver_((node):first()))\r\nend))() end end)())\r\nend)))\r\n\tunarySend=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((ws),binarySelector)),ws)),(unarySend):_slash(operand))):_at_at(\"binaryMessage\")):_eq_eq_gt((function(node)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):selector_((node):second());\r\nreturn ((_s8_rec):arguments_((smalltalk.Array):with_((node):fourth())))\r\nend\r\n)((smalltalk.SendNode):new()))\r\nend)))\r\n\tbinaryMessage=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.PPDelegateParser):new()):_at_at(\"binaryTailDelegate\"))\r\n\tbinaryTail=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(binaryTail):parser_((smalltalk:object_comma_((binaryMessage),(binaryTail):optional())):_eq_eq_gt((function(node)\r\nreturn ((function() if (((((((node):second()))==nil)))==true) then return ((function()\r\nreturn ((node):first())\r\nend))() else return ((function()\r\nreturn (((node):second()):valueForReceiver_((node):first()))\r\nend))() end end)())\r\nend)))\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((unarySend),(binaryTail):optional())):_at_at(\"binarySend\")):_eq_eq_gt((function(node)\r\nreturn ((function() if (((((((node):second()))==nil)))==true) then return ((function()\r\nreturn ((node):first())\r\nend))() else return ((function()\r\nreturn (((node):second()):valueForReceiver_((node):first()))\r\nend))() end end)())\r\nend)))\r\n\tbinarySend=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk:object_comma_((smalltalk:object_comma_((keyword),ws)),binarySend)):_at_at(\"keywordPair\"))\r\n\tkeywordPair=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((smalltalk:object_comma_((ws),keywordPair)):plus()):_at_at(\"keywordMessage\")):_eq_eq_gt((function(nodes)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):selector_(((nodes):collect_((function(each)\r\nreturn (((each):second()):first())\r\nend))):join_(\"\"));\r\nreturn ((_s8_rec):arguments_((nodes):collect_((function(each)\r\nreturn (((each):second()):third())\r\nend))))\r\nend\r\n)((smalltalk.SendNode):new()))\r\nend)))\r\n\tkeywordMessage=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((binarySend),keywordMessage)):_at_at(\"keywordSend\")):_eq_eq_gt((function(node)\r\nreturn (((node):second()):valueForReceiver_((node):first()))\r\nend)))\r\n\tkeywordSend=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((binaryMessage):_slash(unaryMessage)):_slash(keywordMessage)):_at_at(\"message\"))\r\n\tmessage=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((((keywordSend):_slash(binarySend)):_at_at(\"preCascade\")),(smalltalk:object_comma_((smalltalk:object_comma_((ws),\";\")),message)):plus())):_at_at(\"cascade\")):_eq_eq_gt((function(node)\r\nreturn (((node):first()):cascadeNodeWithMessages_(((node):second()):collect_((function(each)\r\nreturn ((each):third())\r\nend))))\r\nend)))\r\n\tcascade=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((self),\"{\")),(smalltalk:object_comma_((smalltalk:object_comma_(((stIdentifier):optional()),string)),ws)):plus())),\"}\")):_at_at(\"inline\")):_eq_eq_gt((function(node)\r\nreturn (((smalltalk.InlineNode):new()):sources_(((node):second()):collect_((function(each)\r\nreturn ((smalltalk.Array):with_with_((each):first(), (((each):second()):value()):replace_with_(\"''\", \"'\")))\r\nend))))\r\nend)))\r\n\tinline=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(expression):parser_((((((assignment):_slash(cascade)):_slash(keywordSend)):_slash(binarySend)):_slash(inline)):_at_at(\"expression\"))\r\n;\r\nreturn ((smalltalk.Array):with_with_with_(ws, (((keywordPattern):_slash(binaryPattern)):_slash(unaryPattern)):_at_at(\"header\"), ((sequence):optional()):_at_at(\"body\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"parser",0
,(
function(self)
local parts=nil
local ws=nil
local header=nil
local body=nil
local method=nil
(function()
	local _s8_value=((self):methodParserParts())
	parts=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((parts):first())
	ws=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((parts):second())
	header=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((parts):third())
	body=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((ws),header)),ws)),body)),ws)):_at_at("impl")):withSource()):_eq_eq_gt((function(node)
return ((
function(_s8_rec)
(_s8_rec):selector_((((node):first()):second()):first());
(_s8_rec):arguments_((((node):first()):second()):second());
(_s8_rec):addNode_((((node):first()):fourth()):shouldReturn());
(_s8_rec):source_((node):second());
return ((_s8_rec):yourself())
end
)((smalltalk.MethodNode):new()))
end)))
	method=_s8_value; return _s8_value
end)()
;
(self):gc()
;
return (((smalltalk:object_comma_((method),(smalltalk.PPEOFParser):new())):_at_at("method")):_eq_eq_gt((function(nodes)
return ((nodes):first())
end)))
;
end
)
,"parsers","parser\r\n\t\" Return the method parser. \"\r\n\r\n\t| parts ws header body method |\r\n\tparts := self methodParserParts.\r\n\tws := parts first.\r\n\theader := parts second.\r\n\tbody := parts third.\r\n\r\n\tmethod := (ws ,header\r\n\t\t,ws ,body\r\n\t\t,ws @@#impl) withSource\r\n\t  ==> [:node | MethodNode new\r\n\t\tselector: node first second first;\r\n\t\targuments: node first second second;\r\n\t\taddNode: node first fourth shouldReturn;\r\n\t\tsource: node second;\r\n\t\tyourself ].\r\n \r\n\tself gc.\r\n \t^method ,PPEOFParser new @@#method\r\n\t   ==> [:nodes | nodes first ]","(\r\nfunction(self)\r\nlocal parts=nil\r\nlocal ws=nil\r\nlocal header=nil\r\nlocal body=nil\r\nlocal method=nil\r\n(function()\r\n\tlocal _s8_value=((self):methodParserParts())\r\n\tparts=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((parts):first())\r\n\tws=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((parts):second())\r\n\theader=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((parts):third())\r\n\tbody=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((ws),header)),ws)),body)),ws)):_at_at(\"impl\")):withSource()):_eq_eq_gt((function(node)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):selector_((((node):first()):second()):first());\r\n(_s8_rec):arguments_((((node):first()):second()):second());\r\n(_s8_rec):addNode_((((node):first()):fourth()):shouldReturn());\r\n(_s8_rec):source_((node):second());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.MethodNode):new()))\r\nend)))\r\n\tmethod=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):gc()\r\n;\r\nreturn (((smalltalk:object_comma_((method),(smalltalk.PPEOFParser):new())):_at_at(\"method\")):_eq_eq_gt((function(nodes)\r\nreturn ((nodes):first())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SmalltalkParser,"methodHeaderParser",0
,(
function(self)
local parts=nil
local ws=nil
local header=nil
(function()
	local _s8_value=((self):methodParserParts())
	parts=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((parts):first())
	ws=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((parts):second())
	header=_s8_value; return _s8_value
end)()
;
return (((smalltalk:object_comma_((ws),header)):_at_at("methodHeader")):_eq_eq_gt((function(nodes)
return ((nodes):second())
end)))
;
end
)
,"parsers","methodHeaderParser\r\n\t\" Return a parser for method header. \"\r\n\r\n\t| parts ws header |\r\n\tparts := self methodParserParts.\r\n\tws := parts first.\r\n\theader := parts second.\r\n\t^ws ,header @@#methodHeader\r\n\t\t==> [:nodes| nodes second ]","(\r\nfunction(self)\r\nlocal parts=nil\r\nlocal ws=nil\r\nlocal header=nil\r\n(function()\r\n\tlocal _s8_value=((self):methodParserParts())\r\n\tparts=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((parts):first())\r\n\tws=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((parts):second())\r\n\theader=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (((smalltalk:object_comma_((ws),header)):_at_at(\"methodHeader\")):_eq_eq_gt((function(nodes)\r\nreturn ((nodes):second())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences["$klass"],"constructEventsTriggered",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:newArray({"found:", "found:pattern:in:", "needsMethods", "searchStarted", "searchFinished"})),smalltalk:superImplementor(smalltalk.MethodReferences["$klass"],"constructEventsTriggered")(self)))
;
end
)
,"events","constructEventsTriggered\r\n\t\" Private - Return the events managed by instances of the receiver. \"\r\n\r\n\t^#(\t#found: #found:pattern:in:\r\n\t\t#needsMethods\r\n\t\t#searchStarted #searchFinished\r\n\t) , super constructEventsTriggered","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:newArray({\"found:\", \"found:pattern:in:\", \"needsMethods\", \"searchStarted\", \"searchFinished\"})),smalltalk:superImplementor(smalltalk.MethodReferences[\"$klass\"],\"constructEventsTriggered\")(self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences["$klass"],"of:",0
,(
function(self,selector)
return (((self):new()):referencesOf_(selector))
;
end
)
,"testing","of: selector\r\n\t\" Return the references to selector. \"\r\n\r\n\t^self new referencesOf: selector","(\r\nfunction(self,selector)\r\nreturn (((self):new()):referencesOf_(selector))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences["$klass"],"ofGlobal:",0
,(
function(self,globalName)
return (((self):new()):referencesOfGlobal_(globalName))
;
end
)
,"testing","ofGlobal: globalName\r\n\t\" Return the references to global globalName. \"\r\n\r\n\t^self new referencesOfGlobal: globalName","(\r\nfunction(self,globalName)\r\nreturn (((self):new()):referencesOfGlobal_(globalName))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences["$klass"],"findClasses:",0
,(
function(self,what)
return (((self):new()):findClasses_(what))
;
end
)
,"testing","findClasses: what\r\n\t\" Find classes in the system matching what. \"\r\n\r\n\t^self new findClasses: what","(\r\nfunction(self,what)\r\nreturn (((self):new()):findClasses_(what))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences["$klass"],"globals",0
,(
function(self)
local tool=nil
local result=nil
(function()
	local _s8_value=((self):new())
	tool=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.PoolDictionary):new())
	result=_s8_value; return _s8_value
end)()
;
(tool):when_do_("found:pattern:in:", (function(mth,mask,stream)
local global=nil
(function()
	local _s8_value=((((smalltalk.SmalltalkParser):new()):globalNameParser()):parse_(stream))
	global=_s8_value; return _s8_value
end)();return ((function() if ((((global):isParseFailure()))==true) then return nil else return ((function()
return (((result):at_ifAbsentPut_(global, (function()
return ((smalltalk.Array):new())
end))):add_(mth))
end))() end end)())
end))
;
(tool):searchGlobalReferences()
;
return (result)
;
end
)
,"testing","globals\r\n\t\" Return the references to all global names. \"\r\n\r\n\t| tool result |\r\n\ttool := self new.\r\n\tresult := PoolDictionary new.\r\n\ttool when: #found:pattern:in: do: [:mth :mask :stream| | global |\r\n\t\tglobal := SmalltalkParser new globalNameParser parse: stream.\r\n\t\tglobal isParseFailure ifFalse: [\r\n\t\t\t(result at: global ifAbsentPut: [Array new]) add: mth\r\n\t\t].\r\n\t].\r\n\ttool searchGlobalReferences.\r\n\t^result","(\r\nfunction(self)\r\nlocal tool=nil\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):new())\r\n\ttool=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.PoolDictionary):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(tool):when_do_(\"found:pattern:in:\", (function(mth,mask,stream)\r\nlocal global=nil\r\n(function()\r\n\tlocal _s8_value=((((smalltalk.SmalltalkParser):new()):globalNameParser()):parse_(stream))\r\n\tglobal=_s8_value; return _s8_value\r\nend)();return ((function() if ((((global):isParseFailure()))==true) then return nil else return ((function()\r\nreturn (((result):at_ifAbsentPut_(global, (function()\r\nreturn ((smalltalk.Array):new())\r\nend))):add_(mth))\r\nend))() end end)())\r\nend))\r\n;\r\n(tool):searchGlobalReferences()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences["$klass"],"methodOverwrites",0
,(
function(self)
return (((self):new()):methodOverwrites())
;
end
)
,"testing","methodOverwrites\r\n\t\" Find the methods that are implemented/overwritten by subclasses. \"\r\n\r\n\t^self new methodOverwrites","(\r\nfunction(self)\r\nreturn (((self):new()):methodOverwrites())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences["$klass"],"variableOverwrites",0
,(
function(self)
return (((self):new()):variableOverwrites())
;
end
)
,"testing","variableOverwrites\r\n\t\" Find the methods that has potential overwrite of variable names. \"\r\n\r\n\t^self new variableOverwrites","(\r\nfunction(self)\r\nreturn (((self):new()):variableOverwrites())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences["$klass"],"findMessages:",0
,(
function(self,what)
return (((self):new()):findMessages_(what))
;
end
)
,"testing","findMessages: what\r\n\t\" Find messages in the system matching what. \"\r\n\r\n\t^self new findMessages: what","(\r\nfunction(self,what)\r\nreturn (((self):new()):findMessages_(what))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"initializeResults",0
,(
function(self)
(function()
	local _s8_value=((smalltalk.Array):new())
	self['@results']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"initialize","initializeResults\r\n\t\" Private - Initialize the results of the receiver. \"\r\n\r\n\tresults := Array new","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tself['@results']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"reset",0
,(
function(self)
(self):results_(nil)
;
return (self)
;
end
)
,"initialize","reset\r\n\t\" Reset the receiver. \"\r\n\r\n\tself results: nil","(\r\nfunction(self)\r\n(self):results_(nil)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"rootClasses:",0
,(
function(self,array)
(function()
	local _s8_value=(array)
	self['@rootClasses']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","rootClasses: array\r\n\t\" Set the rootClasses of the receiver. \"\r\n\r\n\trootClasses := array","(\r\nfunction(self,array)\r\n(function()\r\n\tlocal _s8_value=(array)\r\n\tself['@rootClasses']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"results",0
,(
function(self)
(function() if ((((((self['@results']))==nil)))==true) then return ((function()
return ((self):initializeResults())
end))() else return nil end end)()
;
return (self['@results'])
;
end
)
,"accessing","results\r\n\t\" Return the results of the receiver. \"\r\n\r\n\tresults isNil ifTrue: [ self initializeResults ].\r\n\t^results","(\r\nfunction(self)\r\n(function() if ((((((self['@results']))==nil)))==true) then return ((function()\r\nreturn ((self):initializeResults())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@results'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"results:",0
,(
function(self,array)
(function()
	local _s8_value=(array)
	self['@results']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","results: array\r\n\t\" Set the results of the receiver. \"\r\n\r\n\tresults := array","(\r\nfunction(self,array)\r\n(function()\r\n\tlocal _s8_value=(array)\r\n\tself['@results']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"rootClasses",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((self['@rootClasses']))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Smalltalk):rootClasses())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self['@rootClasses'])
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","rootClasses\r\n\t\" Return the rootClasses of the receiver. \"\r\n\r\n\trootClasses isNil ifTrue: [ ^Smalltalk rootClasses ].\r\n\t^rootClasses","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((self['@rootClasses']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Smalltalk):rootClasses())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self['@rootClasses'])\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"allMethodsIn:",0
,(
function(self,aCollection)
local result=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	result=_s8_value; return _s8_value
end)()
;
((self):rootClasses()):do_((function(root)
return ((((root):instanceClass()):withAllSubclasses()):do_((function(cls)
(cls):methodsDo_((function(each)
return ((result):add_(each))
end));return (((cls)["$klass"]):methodsDo_((function(each)
return ((result):add_(each))
end)))
end)))
end))
;
return (result)
;
end
)
,"accessing","allMethodsIn: aCollection\r\n\t\" Return the methods defined by aCollection of classes. \"\r\n\r\n\t| result |\r\n\tresult := Array new.\r\n\tself rootClasses do: [:root|\r\n\t\troot instanceClass withAllSubclasses do: [:cls|\r\n\t\t\tcls methodsDo: [:each| result add: each ].\r\n\t\t\tcls class methodsDo: [:each| result add: each ].\r\n\t\t]\r\n\t].\r\n\t^result","(\r\nfunction(self,aCollection)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((self):rootClasses()):do_((function(root)\r\nreturn ((((root):instanceClass()):withAllSubclasses()):do_((function(cls)\r\n(cls):methodsDo_((function(each)\r\nreturn ((result):add_(each))\r\nend));return (((cls)[\"$klass\"]):methodsDo_((function(each)\r\nreturn ((result):add_(each))\r\nend)))\r\nend)))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"allMethods",0
,(
function(self)
return ((self):triggerEvent_ifNotHandled_("needsMethods", (function()
return ((self):allMethodsIn_((self):rootClasses()))
end)))
;
end
)
,"accessing","allMethods\r\n\t\" Return the methods to search the receiver. \"\r\n\r\n\t^self\ttriggerEvent: #needsMethods\r\n\t\tifNotHandled: [ self allMethodsIn: self rootClasses ]","(\r\nfunction(self)\r\nreturn ((self):triggerEvent_ifNotHandled_(\"needsMethods\", (function()\r\nreturn ((self):allMethodsIn_((self):rootClasses()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"search:with:",0
,(
function(self,what,targetBlock)
local atoms=nil
(function()
	local _s8_value=((self):allMethods())
	atoms=_s8_value; return _s8_value
end)()
;
(self):triggerEvent_("searchStarted")
;
(function()
	local _s8_value=((atoms):select_((function(aMethod)
local source=nil
(function()
	local _s8_value=((targetBlock):value_(aMethod))
	source=_s8_value; return _s8_value
end)();return ((function() if ((((smalltalk:boolean_and_(((((source))~=nil)),(function()
return ((smalltalk:toString_((source))):includes_(what))
end)))))==true) then return ((function()
((self):results()):add_(aMethod);(self):triggerEvent_with_("found:", aMethod);(function() if ((((self):hasActionForEvent_("found:pattern:in:")))==true) then return ((function()
local stream=nil
local args=nil
(function()
	local _s8_value=((smalltalk:toString_((source))):stream())
	stream=_s8_value; return _s8_value
end)();(function()
	local _s8_value=((smalltalk.Array):with_with_with_(aMethod, what, stream))
	args=_s8_value; return _s8_value
end)();return ((function() while(not ((function()
return ((
function(_s8_rec)
(_s8_rec):upTo_(what);
return ((_s8_rec):atEnd())
end
)(stream))
end))()) do (function()
return ((self):triggerEvent_withArguments_("found:pattern:in:", args))
end)() end;return nil end)())
end))() else return nil end end)();return (true)
end))() else return ((function()
return (false)
end))() end end)())
end)))
	atoms=_s8_value; return _s8_value
end)()
;
(self):triggerEvent_("searchFinished")
;
return (atoms)
;
end
)
,"searching","search: what with: targetBlock\r\n\t\" Find what on target of selected methods. \"\r\n\r\n\t| atoms |\r\n\tatoms := self allMethods.\r\n\tself triggerEvent: #searchStarted.\r\n\tatoms := atoms select: [:aMethod| | source |\r\n\t\tsource := targetBlock value: aMethod.\r\n\t\t(source notNil and: [source toString includes: what]) ifTrue: [\r\n\t\t\tself results add: aMethod.\r\n\t\t\tself triggerEvent: #found: with: aMethod.\r\n\t\t\t(self hasActionForEvent: #found:pattern:in:) ifTrue: [ | stream args |\r\n\t\t\t\tstream := source toString stream.\r\n\t\t\t\targs := Array with: aMethod with: what with: stream.\r\n\t\t\t\t[ stream upTo: what; atEnd ] whileFalse: [\r\n\t\t\t\t\tself triggerEvent: #found:pattern:in: withArguments: args\r\n\t\t\t\t]\r\n\t\t\t].\r\n\t\t\ttrue\r\n\t\t] ifFalse: [ false ]\r\n\t].\r\n\tself triggerEvent: #searchFinished.\r\n\t^atoms","(\r\nfunction(self,what,targetBlock)\r\nlocal atoms=nil\r\n(function()\r\n\tlocal _s8_value=((self):allMethods())\r\n\tatoms=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):triggerEvent_(\"searchStarted\")\r\n;\r\n(function()\r\n\tlocal _s8_value=((atoms):select_((function(aMethod)\r\nlocal source=nil\r\n(function()\r\n\tlocal _s8_value=((targetBlock):value_(aMethod))\r\n\tsource=_s8_value; return _s8_value\r\nend)();return ((function() if ((((smalltalk:boolean_and_(((((source))~=nil)),(function()\r\nreturn ((smalltalk:toString_((source))):includes_(what))\r\nend)))))==true) then return ((function()\r\n((self):results()):add_(aMethod);(self):triggerEvent_with_(\"found:\", aMethod);(function() if ((((self):hasActionForEvent_(\"found:pattern:in:\")))==true) then return ((function()\r\nlocal stream=nil\r\nlocal args=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk:toString_((source))):stream())\r\n\tstream=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=((smalltalk.Array):with_with_with_(aMethod, what, stream))\r\n\targs=_s8_value; return _s8_value\r\nend)();return ((function() while(not ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):upTo_(what);\r\nreturn ((_s8_rec):atEnd())\r\nend\r\n)(stream))\r\nend))()) do (function()\r\nreturn ((self):triggerEvent_withArguments_(\"found:pattern:in:\", args))\r\nend)() end;return nil end)())\r\nend))() else return nil end end)();return (true)\r\nend))() else return ((function()\r\nreturn (false)\r\nend))() end end)())\r\nend)))\r\n\tatoms=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):triggerEvent_(\"searchFinished\")\r\n;\r\nreturn (atoms)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"search:",0
,(
function(self,what)
return ((self):search_with_(what, (function(aMethod)
return ((aMethod):compiledSource())
end)))
;
end
)
,"searching","search: what\r\n\t\" Find references to what in selected methods.\r\n\tNote: search is realized on native source of native code of methods.\r\n\t\"\r\n\r\n\t^self search: what with: [:aMethod| aMethod compiledSource ]","(\r\nfunction(self,what)\r\nreturn ((self):search_with_(what, (function(aMethod)\r\nreturn ((aMethod):compiledSource())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"searchSource:",0
,(
function(self,what)
return ((self):search_with_(what, (function(aMethod)
return ((aMethod):source())
end)))
;
end
)
,"searching","searchSource: what\r\n\t\" Find what in source of selected methods. \"\r\n\r\n\t^self search: what with: [:aMethod| aMethod source ]","(\r\nfunction(self,what)\r\nreturn ((self):search_with_(what, (function(aMethod)\r\nreturn ((aMethod):source())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"inlines",0
,(
function(self)
return ((self):searchSource_("{'"))
;
end
)
,"searching","inlines\r\n\t\" Find native inlines in selected methods. \"\r\n\r\n\t^self searchSource: '{'''","(\r\nfunction(self)\r\nreturn ((self):searchSource_(\"{'\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"searchCategory:",0
,(
function(self,aName)
local found=nil
(self):triggerEvent_("searchStarted")
;
(function()
	local _s8_value=(((smalltalk.Exporter):new()):matchCategory_from_(aName, (self):rootClasses()))
	found=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(smalltalk:object_comma_(((found):first()),(found):last()))
	found=_s8_value; return _s8_value
end)()
;
(found):do_((function(each)
((self):results()):add_(each);return ((self):triggerEvent_with_("found:", each))
end))
;
(self):triggerEvent_("searchFinished")
;
return (found)
;
end
)
,"searching","searchCategory: aName\r\n\t\" Find (source) references to category starting from root classes. \"\r\n\r\n\t| found |\r\n\tself triggerEvent: #searchStarted.\r\n\tfound := Exporter new matchCategory: aName from: self rootClasses.\r\n\tfound := found first , found last.\r\n\tfound do: [:each|\r\n\t\t\tself results add: each.\r\n\t\t\tself triggerEvent: #found: with: each.\r\n\t].\r\n\tself triggerEvent: #searchFinished.\r\n\t^found","(\r\nfunction(self,aName)\r\nlocal found=nil\r\n(self):triggerEvent_(\"searchStarted\")\r\n;\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.Exporter):new()):matchCategory_from_(aName, (self):rootClasses()))\r\n\tfound=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_(((found):first()),(found):last()))\r\n\tfound=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(found):do_((function(each)\r\n((self):results()):add_(each);return ((self):triggerEvent_with_(\"found:\", each))\r\nend))\r\n;\r\n(self):triggerEvent_(\"searchFinished\")\r\n;\r\nreturn (found)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"sendPatternsOf:",0
,(
function(self,aSelector)
return ((smalltalk.Array):with_with_((self):normalSendPatternOf_(aSelector), (self):superSendPatternOf_(aSelector)))
;
end
)
,"patterns","sendPatternsOf: aSelector\r\n\t\" Private - Return the search patterns for references of aSelector. \"\r\n\r\n\t^Array\r\n\t\twith: (self normalSendPatternOf: aSelector)\r\n\t\twith: (self superSendPatternOf: aSelector)","(\r\nfunction(self,aSelector)\r\nreturn ((smalltalk.Array):with_with_((self):normalSendPatternOf_(aSelector), (self):superSendPatternOf_(aSelector)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"dot",0
,(
function(self)
return (":")
;
end
)
,"patterns","dot\r\n\t\" Private - Return the message send pattern starter. \"\r\n\r\n\t^{'\".\"' lua'\":\"'}","(\r\nfunction(self)\r\nreturn (\":\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"localDef",0
,(
function(self)
return ("local ")
;
end
)
,"patterns","localDef\r\n\t\" Private - Return the variable definition starter. \"\r\n\r\n\t^{'\"var \"' lua'\"local \"'}","(\r\nfunction(self)\r\nreturn (\"local \")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"normalSendPatternOf:",0
,(
function(self,aSelector)
return (smalltalk:object_comma_((smalltalk:object_comma_(((self):dot()),(aSelector):asSelector())),"("))
;
end
)
,"patterns","normalSendPatternOf: aSelector\r\n\t\" Private - Return the search pattern for normal senders of aSelector. \"\r\n\r\n\t^self dot ,aSelector asSelector ,'('","(\r\nfunction(self,aSelector)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_(((self):dot()),(aSelector):asSelector())),\"(\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"superJSSendPatternOf:",0
,(
function(self,aSelector)
return (smalltalk:object_comma_((smalltalk:object_comma_((".$superclass.$fn.prototype['"),(aSelector):asSelector())),"'].apply("))
;
end
)
,"patterns","superJSSendPatternOf: aSelector\r\n\t\" Private - Return the search pattern for senders (to super) of aSelector. \"\r\n\r\n\t^'.$superclass.$fn.prototype[''' ,aSelector asSelector ,'''].apply('","(\r\nfunction(self,aSelector)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\".$superclass.$fn.prototype['\"),(aSelector):asSelector())),\"'].apply(\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"superLuaSendPatternOf:",0
,(
function(self,aSelector)
return (smalltalk:object_comma_((smalltalk:object_comma_(("\""),(aSelector):asSelector())),"\")(self)"))
;
end
)
,"patterns","superLuaSendPatternOf: aSelector\r\n\t\" Private - Return the search pattern for senders (to super) of aSelector. \"\r\n\r\n\t^\"'smalltalk:superImplementor(smalltalk.' ,theClassName ,',' \"\r\n\t\t'\"' ,aSelector asSelector ,'\")(self)'","(\r\nfunction(self,aSelector)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"\\\"\"),(aSelector):asSelector())),\"\\\")(self)\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"superSendPatternOf:",0
,(
function(self,aSelector)
return (self:superLuaSendPatternOf_(aSelector))
;
end
)
,"patterns","superSendPatternOf: aSelector\r\n\t\" Private - Return the search pattern for senders (to super) of aSelector. \"\r\n\r\n\t^{'self.superJSSendPatternOf_(aSelector)'\r\n\tlua'self:superLuaSendPatternOf_(aSelector)'}","(\r\nfunction(self,aSelector)\r\nreturn (self:superLuaSendPatternOf_(aSelector))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"instVarPatternOf:",0
,(
function(self,instVarName)
return (smalltalk:object_comma_((smalltalk:object_comma_(("self['@"),(instVarName):asNativeName())),"']"))
;
end
)
,"patterns","instVarPatternOf: instVarName\r\n\t\" Private - Return the search pattern for instance variable access. \"\r\n\r\n\t^'self[''@' ,instVarName asNativeName ,''']'","(\r\nfunction(self,instVarName)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"self['@\"),(instVarName):asNativeName())),\"']\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"globalPatternOf:",0
,(
function(self,globalName)
return (smalltalk:object_comma_(("smalltalk."),globalName))
;
end
)
,"patterns","globalPatternOf: globalName\r\n\t\" Private - Return the search pattern for global variable access. \"\r\n\r\n\t^'smalltalk.' ,globalName","(\r\nfunction(self,globalName)\r\nreturn (smalltalk:object_comma_((\"smalltalk.\"),globalName))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"types",0
,(
function(self)
return (smalltalk:newArray({"referencesOf:", "referencesOfGlobal:", "referencesOfClassVariable:", "referencesOfInstanceVariable:", "findClasses:", "findMessages:", "searchCategory:", "search:", "searchSource:", "globalReferences", "missingGlobals", "inlines", "methodOverwrites", "variableOverwrites"}))
;
end
)
,"types","types\r\n\t\" Private - Return the types of search implemented by the receiver. \"\r\n\r\n\t^#(\t#referencesOf: #referencesOfGlobal:\r\n\t\t#referencesOfClassVariable: \r\n\t\t#referencesOfInstanceVariable:\r\n\t\t#findClasses: #findMessages:\r\n\t\t#searchCategory:\r\n\t\t#search: #searchSource:\r\n\t\t#globalReferences #missingGlobals #inlines\r\n\t\t#methodOverwrites #variableOverwrites\r\n\t)","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"referencesOf:\", \"referencesOfGlobal:\", \"referencesOfClassVariable:\", \"referencesOfInstanceVariable:\", \"findClasses:\", \"findMessages:\", \"searchCategory:\", \"search:\", \"searchSource:\", \"globalReferences\", \"missingGlobals\", \"inlines\", \"methodOverwrites\", \"variableOverwrites\"}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"search:type:",0
,(
function(self,aName,typeSelector)
local selector=nil
(function()
	local _s8_value=((function() if ((((typeSelector):includes_(":")))==true) then return ((function()
return (smalltalk:object_comma_(((typeSelector):upTo_(":")),":"))
end))() else return ((function()
return ((typeSelector):upTo_(" "))
end))() end end)())
	selector=_s8_value; return _s8_value
end)()
;
return ((function() if (((smalltalk:responds_to_((self),(selector))))==true) then return ((function()
return ((self):perform_with_(selector, (aName):trimBlanks()))
end))() else return nil end end)())
;
end
)
,"searching","search: aName type: typeSelector\r\n\t\" Return the result of searching for aName as a search of specified type (or nil). \"\r\n\r\n\t| selector |\r\n\tselector := (typeSelector includes: $:)\r\n\tifTrue: [ (typeSelector upTo: $:) ,$: ]\r\n\tifFalse: [ typeSelector upTo: $  ].\r\n\t^(self respondsTo: selector) ifTrue: [\r\n\t\tself perform: selector with: aName trimBlanks\r\n\t]","(\r\nfunction(self,aName,typeSelector)\r\nlocal selector=nil\r\n(function()\r\n\tlocal _s8_value=((function() if ((((typeSelector):includes_(\":\")))==true) then return ((function()\r\nreturn (smalltalk:object_comma_(((typeSelector):upTo_(\":\")),\":\"))\r\nend))() else return ((function()\r\nreturn ((typeSelector):upTo_(\" \"))\r\nend))() end end)())\r\n\tselector=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((function() if (((smalltalk:responds_to_((self),(selector))))==true) then return ((function()\r\nreturn ((self):perform_with_(selector, (aName):trimBlanks()))\r\nend))() else return nil end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"sendersOf:",0
,(
function(self,aSelector)
return (((self):sendPatternsOf_(aSelector)):inject_into_(smalltalk:newArray({}), (function(total,each)
return ((
function(_s8_rec)
(_s8_rec):addAll_((self):search_(each));
return ((_s8_rec):yourself())
end
)(total))
end)))
;
end
)
,"searching","sendersOf: aSelector\r\n\t\" Find the requested elements from root classes. \"\r\n\r\n\t^(self sendPatternsOf: aSelector) inject: #() into: [:total :each|\r\n\t\ttotal addAll: (self search: each); yourself\r\n\t]","(\r\nfunction(self,aSelector)\r\nreturn (((self):sendPatternsOf_(aSelector)):inject_into_(smalltalk:newArray({}), (function(total,each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):addAll_((self):search_(each));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(total))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"referencesOf:",0
,(
function(self,aSelector)
(
function(_s8_rec)
(_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_(("--- Implementors of #"),aSelector))," ---"));
(_s8_rec):addAll_((self):implementorsOf_(aSelector));
return ((_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_(("--- Senders of #"),aSelector))," ---")))
end
)((self):results())
;
(self):sendersOf_(aSelector)
;
return (self['@results'])
;
end
)
,"searching","referencesOf: aSelector\r\n\t\" Find the references to message (aSelector). \"\r\n\r\n\tself results\r\n\t\tadd: '--- Implementors of #' ,aSelector ,' ---';\r\n\t\taddAll: (self implementorsOf: aSelector);\r\n\t\tadd: '--- Senders of #' ,aSelector ,' ---'.\r\n\tself sendersOf: aSelector.\r\n\t^results","(\r\nfunction(self,aSelector)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_((\"--- Implementors of #\"),aSelector)),\" ---\"));\r\n(_s8_rec):addAll_((self):implementorsOf_(aSelector));\r\nreturn ((_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_((\"--- Senders of #\"),aSelector)),\" ---\")))\r\nend\r\n)((self):results())\r\n;\r\n(self):sendersOf_(aSelector)\r\n;\r\nreturn (self['@results'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"referencesOfInstanceVariable:",0
,(
function(self,instVarName)
return ((self):search_((self):instVarPatternOf_(instVarName)))
;
end
)
,"searching","referencesOfInstanceVariable: instVarName\r\n\t\" Find the requested elements from root classes. \"\r\n\r\n\t^self search: (self instVarPatternOf: instVarName)","(\r\nfunction(self,instVarName)\r\nreturn ((self):search_((self):instVarPatternOf_(instVarName)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"settersOfInstanceVariable:",0
,(
function(self,instVarName)
return ((self):search_(smalltalk:object_comma_(((self):instVarPatternOf_(instVarName)),"=")))
;
end
)
,"searching","settersOfInstanceVariable: instVarName\r\n\t\" Find the requested elements from root classes. \"\r\n\r\n\t^self search: (self instVarPatternOf: instVarName) ,'='","(\r\nfunction(self,instVarName)\r\nreturn ((self):search_(smalltalk:object_comma_(((self):instVarPatternOf_(instVarName)),\"=\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"referencesOfGlobal:",0
,(
function(self,globalName)
return ((self):search_((self):globalPatternOf_(globalName)))
;
end
)
,"searching","referencesOfGlobal: globalName\r\n\t\" Find the requested elements from root classes. \"\r\n\r\n\t^self search: (self globalPatternOf: globalName)","(\r\nfunction(self,globalName)\r\nreturn ((self):search_((self):globalPatternOf_(globalName)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"searchGlobalReferences",0
,(
function(self)
return ((self):search_((self):globalPatternOf_("")))
;
end
)
,"searching","searchGlobalReferences\r\n\t\" Find the requested elements from root classes. \"\r\n\r\n\t^self search: (self globalPatternOf: '')","(\r\nfunction(self)\r\nreturn ((self):search_((self):globalPatternOf_(\"\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"settersOfGlobalVariable:",0
,(
function(self,globalName)
return ((self):search_(smalltalk:object_comma_(((self):globalPatternOf_(globalName)),"=")))
;
end
)
,"searching","settersOfGlobalVariable: globalName\r\n\t\" Find the requested elements from root classes. \"\r\n\r\n\t^self search: (self globalPatternOf: globalName) ,'='","(\r\nfunction(self,globalName)\r\nreturn ((self):search_(smalltalk:object_comma_(((self):globalPatternOf_(globalName)),\"=\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"referencesOfClassVariable:",0
,(
function(self,classVarName)
return ((
function(_s8_rec)
(_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_(("--- Getters of "),classVarName))," ---"));
(_s8_rec):addAll_((self):gettersOfClassVariable_(classVarName));
(_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_(("--- Setters of "),classVarName))," ---"));
(_s8_rec):addAll_((self):settersOfClassVariable_(classVarName));
return ((_s8_rec):yourself())
end
)((self):results()))
;
end
)
,"searching","referencesOfClassVariable: classVarName\r\n\t\" Find the requested elements from root classes. \"\r\n\r\n\t^self results\r\n\t\tadd: '--- Getters of ' ,classVarName ,' ---';\r\n\t\taddAll: (self gettersOfClassVariable: classVarName);\r\n\t\tadd: '--- Setters of ' ,classVarName ,' ---';\r\n\t\taddAll: (self settersOfClassVariable: classVarName);\r\n\t\tyourself","(\r\nfunction(self,classVarName)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_((\"--- Getters of \"),classVarName)),\" ---\"));\r\n(_s8_rec):addAll_((self):gettersOfClassVariable_(classVarName));\r\n(_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_((\"--- Setters of \"),classVarName)),\" ---\"));\r\n(_s8_rec):addAll_((self):settersOfClassVariable_(classVarName));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):results()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"gettersOfClassVariable:",0
,(
function(self,classVarName)
return ((self):search_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):dot()),"classVariableAt_(\"")),classVarName)),"\")")))
;
end
)
,"searching","gettersOfClassVariable: classVarName\r\n\t\" Find the requested elements from root classes. \"\r\n\r\n\t^self search: self dot,'classVariableAt_(\"',classVarName,'\")'","(\r\nfunction(self,classVarName)\r\nreturn ((self):search_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):dot()),\"classVariableAt_(\\\"\")),classVarName)),\"\\\")\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"settersOfClassVariable:",0
,(
function(self,classVarName)
return ((self):search_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):dot()),"classVariableAt_put_(\"")),classVarName)),"\",")))
;
end
)
,"searching","settersOfClassVariable: classVarName\r\n\t\" Find the requested elements from root classes. \"\r\n\r\n\t^self search: self dot,'classVariableAt_put_(\"',classVarName,'\",'","(\r\nfunction(self,classVarName)\r\nreturn ((self):search_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((self):dot()),\"classVariableAt_put_(\\\"\")),classVarName)),\"\\\",\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"globalReferences",0
,(
function(self)
local pool=nil
(function()
	local _s8_value=(((self)["$klass"]):globals())
	pool=_s8_value; return _s8_value
end)()
;
(((pool):keys()):sorted()):do_((function(aName)
return ((
function(_s8_rec)
(_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_(("--- References to: "),aName))," ---"));
return ((_s8_rec):addAll_((pool):at_(aName)))
end
)((self):results()))
end))
;
return (self['@results'])
;
end
)
,"searching","globalReferences\r\n\t\" Find the references to globals. \"\r\n\r\n\t| pool |\r\n\tpool := self class globals.\r\n\tpool keys sorted do: [:aName|\r\n\t\tself results\r\n\t\t\tadd: '--- References to: ',aName,' ---';\r\n\t\t\taddAll: (pool at: aName)\r\n\t].\r\n\t^results","(\r\nfunction(self)\r\nlocal pool=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):globals())\r\n\tpool=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(((pool):keys()):sorted()):do_((function(aName)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_((\"--- References to: \"),aName)),\" ---\"));\r\nreturn ((_s8_rec):addAll_((pool):at_(aName)))\r\nend\r\n)((self):results()))\r\nend))\r\n;\r\nreturn (self['@results'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"missingGlobals",0
,(
function(self)
local pool=nil
(function()
	local _s8_value=(((self)["$klass"]):globals())
	pool=_s8_value; return _s8_value
end)()
;
((((pool):keys()):sorted()):reject_((function(key)
return ((smalltalk.Smalltalk):includesKey_(key))
end))):do_((function(aName)
return ((
function(_s8_rec)
(_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_(("--- Missing global: "),aName))," ---"));
return ((_s8_rec):addAll_((pool):at_(aName)))
end
)((self):results()))
end))
;
(function() if (((((self):results()):isEmpty()))==true) then return ((function()
return (((self):results()):add_("--- All global references are bound to Smalltalk ---"))
end))() else return nil end end)()
;
return (self['@results'])
;
end
)
,"searching","missingGlobals\r\n\t\" Find the references to missing globals. \"\r\n\r\n\t| pool |\r\n\tpool := self class globals.\r\n\t(pool keys sorted\r\n\t\treject: [:key| (Smalltalk includesKey: key) ])\r\n\t\tdo: [:aName|\r\n\t\t\tself results\r\n\t\t\t\tadd: '--- Missing global: ',aName,' ---';\r\n\t\t\t\taddAll: (pool at: aName)\r\n\t\t].\r\n\tself results isEmpty ifTrue: [\r\n\t\tself results add: '--- All global references are bound to Smalltalk ---'\r\n\t].\r\n\t^results","(\r\nfunction(self)\r\nlocal pool=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):globals())\r\n\tpool=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((((pool):keys()):sorted()):reject_((function(key)\r\nreturn ((smalltalk.Smalltalk):includesKey_(key))\r\nend))):do_((function(aName)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_(smalltalk:object_comma_((smalltalk:object_comma_((\"--- Missing global: \"),aName)),\" ---\"));\r\nreturn ((_s8_rec):addAll_((pool):at_(aName)))\r\nend\r\n)((self):results()))\r\nend))\r\n;\r\n(function() if (((((self):results()):isEmpty()))==true) then return ((function()\r\nreturn (((self):results()):add_(\"--- All global references are bound to Smalltalk ---\"))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@results'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"implementorsOf:",0
,(
function(self,aSelector)
return (((smalltalk.Smalltalk):current()):implementorsOf_(aSelector))
;
end
)
,"private","implementorsOf: aSelector\r\n\t\" Private - Return the implementors of aSelector. \"\r\n\r\n\t^Smalltalk current implementorsOf: aSelector","(\r\nfunction(self,aSelector)\r\nreturn (((smalltalk.Smalltalk):current()):implementorsOf_(aSelector))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"compiledSourceStart",0
,(
function(self)
return ("(")
;
end
)
,"patterns","compiledSourceStart\r\n\t\" Private - Return the token for start of compiled code. \"\r\n\r\n\t^{'\"{\"' lua'\"(\"'}","(\r\nfunction(self)\r\nreturn (\"(\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"messagesOf:",0
,(
function(self,aCompiledMethod)
local stream=nil
local selectors=nil
local string=nil
(function()
	local _s8_value=(((aCompiledMethod):compiledSource()):stream())
	stream=_s8_value; return _s8_value
end)()
;
(stream):upTo_((self):compiledSourceStart())
;
(function()
	local _s8_value=((smalltalk.HashedSet):new())
	selectors=_s8_value; return _s8_value
end)()
;
(function() while(not ((function()
return ((stream):atEnd())
end))()) do (function()
return ((selectors):add_(((((
function(_s8_rec)
(_s8_rec):upTo_((self):dot());
return ((_s8_rec):upTo_("("))
end
)(stream)):asArrayOfSubstringsSeparatedBy_((self):dot())):last()):asStSelector()))
end)() end;return nil end)()
;
(selectors):remove_ifAbsent_("", (function()
return (nil)
end))
;
return ((((selectors):asArray()):select_((function(each)
return ((each):isLiteral())
end))):sorted())
;
end
)
,"accessing","messagesOf: aCompiledMethod\r\n\t\" Return all the messages that can be sent during execution of aCompiledMethod. \"\r\n\r\n\t| stream selectors string |\r\n\tstream := aCompiledMethod compiledSource stream.\r\n\tstream upTo: self compiledSourceStart.\r\n\tselectors := HashedSet new.\r\n\t[stream atEnd] whileFalse: [\r\n\t\tselectors add: ((\r\n\t\t\tstream upTo: self dot;upTo: $(\r\n\t\t\t) asArrayOfSubstringsSeparatedBy: self dot)\r\n\t\t\tlast asStSelector\r\n\t].\r\n\tselectors remove: '' ifAbsent: [].\r\n\t^(selectors asArray select: [:each| each isLiteral ]) sorted","(\r\nfunction(self,aCompiledMethod)\r\nlocal stream=nil\r\nlocal selectors=nil\r\nlocal string=nil\r\n(function()\r\n\tlocal _s8_value=(((aCompiledMethod):compiledSource()):stream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(stream):upTo_((self):compiledSourceStart())\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.HashedSet):new())\r\n\tselectors=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(not ((function()\r\nreturn ((stream):atEnd())\r\nend))()) do (function()\r\nreturn ((selectors):add_(((((\r\nfunction(_s8_rec)\r\n(_s8_rec):upTo_((self):dot());\r\nreturn ((_s8_rec):upTo_(\"(\"))\r\nend\r\n)(stream)):asArrayOfSubstringsSeparatedBy_((self):dot())):last()):asStSelector()))\r\nend)() end;return nil end)()\r\n;\r\n(selectors):remove_ifAbsent_(\"\", (function()\r\nreturn (nil)\r\nend))\r\n;\r\nreturn ((((selectors):asArray()):select_((function(each)\r\nreturn ((each):isLiteral())\r\nend))):sorted())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"globalReferencesOf:",0
,(
function(self,aCompiledMethod)
local result=nil
(function()
	local _s8_value=((smalltalk.HashedSet):new())
	result=_s8_value; return _s8_value
end)()
;
(
function(_s8_rec)
(_s8_rec):when_do_("needsMethods", (function()
return ((smalltalk.Array):with_(aCompiledMethod))
end));
(_s8_rec):when_do_("found:pattern:in:", (function(mth,mask,stream)
local global=nil
(function()
	local _s8_value=((((smalltalk.SmalltalkParser):new()):globalNameParser()):parse_(stream))
	global=_s8_value; return _s8_value
end)();return ((function() if ((((global):isParseFailure()))==true) then return nil else return ((function()
return ((result):add_(global))
end))() end end)())
end));
return ((_s8_rec):yourself())
end
)(self)
;
(self):searchGlobalReferences()
;
return (((result):asArray()):sorted())
;
end
)
,"accessing","globalReferencesOf: aCompiledMethod\r\n\t\" Return all the global reference names in aCompiledMethod. \"\r\n\r\n\t| result |\r\n\tresult := HashedSet new.\r\n\tself\r\n\t\twhen: #needsMethods do: [ Array with: aCompiledMethod ];\r\n\t\twhen: #found:pattern:in: do: [:mth :mask :stream| | global |\r\n\t\t\tglobal := SmalltalkParser new globalNameParser parse: stream.\r\n\t\t\tglobal isParseFailure ifFalse: [ result add: global ].\r\n\t\t];\r\n\t\tyourself.\r\n\tself searchGlobalReferences.\r\n\t^result asArray sorted","(\r\nfunction(self,aCompiledMethod)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.HashedSet):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):when_do_(\"needsMethods\", (function()\r\nreturn ((smalltalk.Array):with_(aCompiledMethod))\r\nend));\r\n(_s8_rec):when_do_(\"found:pattern:in:\", (function(mth,mask,stream)\r\nlocal global=nil\r\n(function()\r\n\tlocal _s8_value=((((smalltalk.SmalltalkParser):new()):globalNameParser()):parse_(stream))\r\n\tglobal=_s8_value; return _s8_value\r\nend)();return ((function() if ((((global):isParseFailure()))==true) then return nil else return ((function()\r\nreturn ((result):add_(global))\r\nend))() end end)())\r\nend));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(self)\r\n;\r\n(self):searchGlobalReferences()\r\n;\r\nreturn (((result):asArray()):sorted())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"findClasses:",0
,(
function(self,what)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local keys=nil
local system=nil
(function() if ((((what):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):findClasses_(((what):trimBlanks()):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(self):triggerEvent_("searchStarted")
;
(function()
	local _s8_value=((what):collect_((function(each)
return ((smalltalk:asString_((each))):asLowercase())
end)))
	keys=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.Smalltalk):current())
	system=_s8_value; return _s8_value
end)()
;
(function() if ((((keys):notEmpty()))==true) then return ((function()
return (((system):classes()):do_((function(aClass)
local key=nil
(function()
	local _s8_value=(((aClass):name()):asLowercase())
	key=_s8_value; return _s8_value
end)();return ((function() if (((((((keys):detect_ifNone_((function(one)
return ((key):includes_(one))
end), (function()
return (nil)
end))))~=nil)))==true) then return ((function()
((self):results()):add_(aClass);return ((self):triggerEvent_with_("found:", aClass))
end))() else return nil end end)())
end)))
end))() else return nil end end)()
;
(self):triggerEvent_("searchFinished")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):results())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"searching","findClasses: what\r\n\t\" Find classes matching what. \"\r\n\r\n\t| keys system |\r\n\twhat isString ifTrue: [\r\n\t\t^self findClasses: what trimBlanks asArrayOfSubstrings\r\n\t].\r\n\tself triggerEvent: #searchStarted.\r\n\tkeys := what collect: [:each| each asString asLowercase ].\r\n\tsystem := Smalltalk current.\r\n\tkeys notEmpty ifTrue: [\r\n\t\tsystem classes do: [:aClass| | key |\r\n\t\t\tkey := aClass name asLowercase.\r\n\t\t\t(keys detect: [:one| key includes: one ] ifNone: []) notNil ifTrue: [\r\n\t\t\t\tself results add: aClass.\r\n\t\t\t\tself triggerEvent: #found: with: aClass.\r\n\t\t\t]\r\n\t\t].\r\n\t].\r\n\tself triggerEvent: #searchFinished.\r\n\t^self results","(\r\nfunction(self,what)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal keys=nil\r\nlocal system=nil\r\n(function() if ((((what):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):findClasses_(((what):trimBlanks()):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self):triggerEvent_(\"searchStarted\")\r\n;\r\n(function()\r\n\tlocal _s8_value=((what):collect_((function(each)\r\nreturn ((smalltalk:asString_((each))):asLowercase())\r\nend)))\r\n\tkeys=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):current())\r\n\tsystem=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((keys):notEmpty()))==true) then return ((function()\r\nreturn (((system):classes()):do_((function(aClass)\r\nlocal key=nil\r\n(function()\r\n\tlocal _s8_value=(((aClass):name()):asLowercase())\r\n\tkey=_s8_value; return _s8_value\r\nend)();return ((function() if (((((((keys):detect_ifNone_((function(one)\r\nreturn ((key):includes_(one))\r\nend), (function()\r\nreturn (nil)\r\nend))))~=nil)))==true) then return ((function()\r\n((self):results()):add_(aClass);return ((self):triggerEvent_with_(\"found:\", aClass))\r\nend))() else return nil end end)())\r\nend)))\r\nend))() else return nil end end)()\r\n;\r\n(self):triggerEvent_(\"searchFinished\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):results())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"methodOverwritesOf:",0
,(
function(self,aClass)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local map=nil
local subclasses=nil
(function()
	local _s8_value=((aClass):allSubclasses())
	subclasses=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.PoolDictionary):new())
	map=_s8_value; return _s8_value
end)()
;
(function() if ((((subclasses):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (map)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(aClass):methodsDo_((function(mth)
local selector=nil
(function()
	local _s8_value=((mth):selector())
	selector=_s8_value; return _s8_value
end)();(map):at_put_(selector, (smalltalk.Array):with_(mth));return ((subclasses):do_((function(each)
return ((function() if (((((each):selectors()):includes_(selector)))==true) then return ((function()
return (((map):at_(selector)):add_((each):_gt_gt(selector)))
end))() else return nil end end)())
end)))
end))
;
(subclasses):do_((function(each)
return (((self):methodOverwritesOf_(each)):keysAndValuesDo_((function(key,value)
return (((map):at_ifAbsentPut_(key, (function()
return ((smalltalk.Array):new())
end))):addAll_(value))
end)))
end))
;
((((map):keys()):asArray()):select_((function(key)
return (((((map):at_(key)):size()) <= (1)))
end))):do_((function(key)
return ((map):removeKey_(key))
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (map)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","methodOverwritesOf: aClass\r\n\t\" Private - Find the methods subimplemented by subclasses of aClass. \"\r\n\r\n\t| map subclasses |\r\n\tsubclasses := aClass allSubclasses.\r\n\tmap := PoolDictionary new.\r\n\tsubclasses isEmpty ifTrue: [ ^map ].\r\n\taClass methodsDo: [:mth| | selector |\r\n\t\tselector := mth selector.\r\n\t\tmap at: selector put: (Array with: mth).\r\n\t\tsubclasses do: [:each|\r\n\t\t\t(each selectors includes: selector) ifTrue: [\r\n\t\t\t\t(map at: selector) add: each >> selector\r\n\t\t\t].\r\n\t\t].\r\n\t].\r\n\tsubclasses do: [:each|\r\n\t\t(self methodOverwritesOf: each) keysAndValuesDo: [:key :value|\r\n\t\t\t(map at: key ifAbsentPut: [Array new]) addAll: value\r\n\t\t].\r\n\t].\r\n\t(map keys asArray\r\n\t\tselect: [:key| (map at: key) size <= 1 ]\r\n\t\t) do: [:key| map removeKey: key ].\r\n\t^map","(\r\nfunction(self,aClass)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal map=nil\r\nlocal subclasses=nil\r\n(function()\r\n\tlocal _s8_value=((aClass):allSubclasses())\r\n\tsubclasses=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.PoolDictionary):new())\r\n\tmap=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((subclasses):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (map)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(aClass):methodsDo_((function(mth)\r\nlocal selector=nil\r\n(function()\r\n\tlocal _s8_value=((mth):selector())\r\n\tselector=_s8_value; return _s8_value\r\nend)();(map):at_put_(selector, (smalltalk.Array):with_(mth));return ((subclasses):do_((function(each)\r\nreturn ((function() if (((((each):selectors()):includes_(selector)))==true) then return ((function()\r\nreturn (((map):at_(selector)):add_((each):_gt_gt(selector)))\r\nend))() else return nil end end)())\r\nend)))\r\nend))\r\n;\r\n(subclasses):do_((function(each)\r\nreturn (((self):methodOverwritesOf_(each)):keysAndValuesDo_((function(key,value)\r\nreturn (((map):at_ifAbsentPut_(key, (function()\r\nreturn ((smalltalk.Array):new())\r\nend))):addAll_(value))\r\nend)))\r\nend))\r\n;\r\n((((map):keys()):asArray()):select_((function(key)\r\nreturn (((((map):at_(key)):size()) <= (1)))\r\nend))):do_((function(key)\r\nreturn ((map):removeKey_(key))\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (map)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"addMethodOverwrites:",0
,(
function(self,map)
(map):associationsDo_((function(assoc)
return (((assoc):value()):do_((function(each)
(self):triggerEvent_with_("found:", each);return (((self):results()):add_(each))
end)))
end))
;
return (self)
;
end
)
,"private","addMethodOverwrites: map\r\n\t\" Private - Register the methods overwrites in map. \"\r\n\r\n\tmap associationsDo: [:assoc|\r\n\t\tassoc value do: [:each|\r\n\t\t\tself triggerEvent: #found: with: each.\r\n\t\t\tself results add: each.\r\n\t\t]\r\n\t]","(\r\nfunction(self,map)\r\n(map):associationsDo_((function(assoc)\r\nreturn (((assoc):value()):do_((function(each)\r\n(self):triggerEvent_with_(\"found:\", each);return (((self):results()):add_(each))\r\nend)))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"methodOverwrites",0
,(
function(self)
(self):triggerEvent_("searchStarted")
;
((smalltalk.Smalltalk):rootClasses()):do_((function(aClass)
(self):addMethodOverwrites_((self):methodOverwritesOf_(aClass));return ((self):addMethodOverwrites_((self):methodOverwritesOf_((aClass)["$klass"])))
end))
;
(self):triggerEvent_("searchFinished")
;
return ((self):results())
;
end
)
,"searching","methodOverwrites\r\n\t\" Find the methods that are implemented/overwritten by subclasses. \"\r\n\r\n\tself triggerEvent: #searchStarted.\r\n\tSmalltalk rootClasses do: [:aClass|\r\n\t\tself addMethodOverwrites: (self methodOverwritesOf: aClass).\r\n\t\tself addMethodOverwrites: (self methodOverwritesOf: aClass class).\r\n\t].\r\n\tself triggerEvent: #searchFinished.\r\n\t^self results","(\r\nfunction(self)\r\n(self):triggerEvent_(\"searchStarted\")\r\n;\r\n((smalltalk.Smalltalk):rootClasses()):do_((function(aClass)\r\n(self):addMethodOverwrites_((self):methodOverwritesOf_(aClass));return ((self):addMethodOverwrites_((self):methodOverwritesOf_((aClass)[\"$klass\"])))\r\nend))\r\n;\r\n(self):triggerEvent_(\"searchFinished\")\r\n;\r\nreturn ((self):results())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"hasArgumentOverwrites:in:",0
,(
function(self,names,stream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local args=nil
(function()
	local _s8_value=(((
function(_s8_rec)
(_s8_rec):upTo_("(");
return ((_s8_rec):upTo_(")"))
end
)(stream)):asArrayOfSubstringsSeparatedBy_(","))
	args=_s8_value; return _s8_value
end)()
;
(args):do_((function(each)
return ((function() if ((((names):includes_(each)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","hasArgumentOverwrites: names in: stream\r\n\t\" Private - Return true if stream of code has argument name conflicts. \"\r\n\r\n\t| args |\r\n\targs := (stream upTo: $(; upTo: $)) asArrayOfSubstringsSeparatedBy: $,.\r\n\targs do: [:each|\r\n\t\t(names includes: each) ifTrue: [ ^true ].\r\n\t].\r\n\t^false","(\r\nfunction(self,names,stream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal args=nil\r\n(function()\r\n\tlocal _s8_value=(((\r\nfunction(_s8_rec)\r\n(_s8_rec):upTo_(\"(\");\r\nreturn ((_s8_rec):upTo_(\")\"))\r\nend\r\n)(stream)):asArrayOfSubstringsSeparatedBy_(\",\"))\r\n\targs=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(args):do_((function(each)\r\nreturn ((function() if ((((names):includes_(each)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"hasLocalOverwrites:in:",0
,(
function(self,names,stream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local candidate=nil
(function() while(not ((function()
return ((stream):atEnd())
end))()) do (function()
(function()
	local _s8_value=(((
function(_s8_rec)
(_s8_rec):upTo_((self):localDef());
return ((_s8_rec):upTo_("="))
end
)(stream)):trimBlanks())
	candidate=_s8_value; return _s8_value
end)();return ((function() if ((((names):includes_(candidate)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","hasLocalOverwrites: names in: stream\r\n\t\" Private - Return true if stream of code has local name conflicts. \"\r\n\r\n\t| candidate |\r\n\t[stream atEnd] whileFalse: [\r\n\t\tcandidate := (stream upTo: self localDef; upTo: $=) trimBlanks.\r\n\t\t(names includes: candidate) ifTrue: [\r\n\t\t\t^true\r\n\t\t].\r\n\t].\r\n\t^false","(\r\nfunction(self,names,stream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal candidate=nil\r\n(function() while(not ((function()\r\nreturn ((stream):atEnd())\r\nend))()) do (function()\r\n(function()\r\n\tlocal _s8_value=(((\r\nfunction(_s8_rec)\r\n(_s8_rec):upTo_((self):localDef());\r\nreturn ((_s8_rec):upTo_(\"=\"))\r\nend\r\n)(stream)):trimBlanks())\r\n\tcandidate=_s8_value; return _s8_value\r\nend)();return ((function() if ((((names):includes_(candidate)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"addVariableOverwrites:in:",0
,(
function(self,names,mth)
local stream=nil
(function()
	local _s8_value=(((mth):compiledSource()):stream())
	stream=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:boolean_or_(((self):hasArgumentOverwrites_in_(names, stream)),(function()
return ((self):hasLocalOverwrites_in_(names, stream))
end)))))==true) then return ((function()
(self):triggerEvent_with_("found:", mth);return (((self):results()):add_(mth))
end))() else return nil end end)()
;
return (self)
;
end
)
,"private","addVariableOverwrites: names in: mth\r\n\t\" Private - Register the methods that has name overwrites. \"\r\n\r\n\t| stream |\r\n\tstream := mth compiledSource stream.\r\n\t((self hasArgumentOverwrites: names in: stream)\r\n\tor: [ self hasLocalOverwrites: names in: stream ])\r\n\tifTrue: [\r\n\t\tself triggerEvent: #found: with: mth.\r\n\t\tself results add: mth.\r\n\t]","(\r\nfunction(self,names,mth)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=(((mth):compiledSource()):stream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:boolean_or_(((self):hasArgumentOverwrites_in_(names, stream)),(function()\r\nreturn ((self):hasLocalOverwrites_in_(names, stream))\r\nend)))))==true) then return ((function()\r\n(self):triggerEvent_with_(\"found:\", mth);return (((self):results()):add_(mth))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"variableOverwrites",0
,(
function(self)
(self):triggerEvent_("searchStarted")
;
(((smalltalk.Smalltalk):current()):classes()):do_((function(aClass)
local names=nil
(function()
	local _s8_value=((aClass):allInstVarNames())
	names=_s8_value; return _s8_value
end)();(aClass):methodsDo_((function(mth)
return ((self):addVariableOverwrites_in_(names, mth))
end));(function()
	local _s8_value=(((aClass)["$klass"]):allInstVarNames())
	names=_s8_value; return _s8_value
end)();return (((aClass)["$klass"]):methodsDo_((function(mth)
return ((self):addVariableOverwrites_in_(names, mth))
end)))
end))
;
(self):triggerEvent_("searchFinished")
;
return ((self):results())
;
end
)
,"searching","variableOverwrites\r\n\t\" Find the methods that has potential overwrite of variable names. \"\r\n\r\n\tself triggerEvent: #searchStarted.\r\n\tSmalltalk current classes do: [:aClass| | names |\r\n\t\tnames := aClass allInstVarNames.\r\n\t\taClass methodsDo: [:mth|\r\n\t\t\tself addVariableOverwrites: names in: mth.\r\n\t\t].\r\n\t\tnames := aClass class allInstVarNames.\r\n\t\taClass class methodsDo: [:mth|\r\n\t\t\tself addVariableOverwrites: names in: mth.\r\n\t\t].\r\n\t].\r\n\tself triggerEvent: #searchFinished.\r\n\t^self results","(\r\nfunction(self)\r\n(self):triggerEvent_(\"searchStarted\")\r\n;\r\n(((smalltalk.Smalltalk):current()):classes()):do_((function(aClass)\r\nlocal names=nil\r\n(function()\r\n\tlocal _s8_value=((aClass):allInstVarNames())\r\n\tnames=_s8_value; return _s8_value\r\nend)();(aClass):methodsDo_((function(mth)\r\nreturn ((self):addVariableOverwrites_in_(names, mth))\r\nend));(function()\r\n\tlocal _s8_value=(((aClass)[\"$klass\"]):allInstVarNames())\r\n\tnames=_s8_value; return _s8_value\r\nend)();return (((aClass)[\"$klass\"]):methodsDo_((function(mth)\r\nreturn ((self):addVariableOverwrites_in_(names, mth))\r\nend)))\r\nend))\r\n;\r\n(self):triggerEvent_(\"searchFinished\")\r\n;\r\nreturn ((self):results())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodReferences,"findMessages:",0
,(
function(self,what)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local keys=nil
(function() if ((((what):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):findMessages_(((what):trimBlanks()):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(self):triggerEvent_("searchStarted")
;
(function()
	local _s8_value=((what):collect_((function(each)
return (smalltalk:asString_((each)))
end)))
	keys=_s8_value; return _s8_value
end)()
;
(function() if ((((keys):notEmpty()))==true) then return ((function()
local all=nil
local collect=nil
(function()
	local _s8_value=((self):allMethods())
	all=_s8_value; return _s8_value
end)();return ((all):do_((function(mth)
local current=nil
(function()
	local _s8_value=((mth):selector())
	current=_s8_value; return _s8_value
end)();return ((function() if (((((((keys):detect_ifNone_((function(one)
return ((current):includes_(one))
end), (function()
return (nil)
end))))~=nil)))==true) then return ((function()
((self):results()):add_(mth);return ((self):triggerEvent_with_("found:", mth))
end))() else return nil end end)())
end)))
end))() else return nil end end)()
;
(self):triggerEvent_("searchFinished")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):results())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"searching","findMessages: what\r\n\t\" Find messages matching what. \"\r\n\r\n\t| keys |\r\n\twhat isString ifTrue: [\r\n\t\t^self findMessages: what trimBlanks asArrayOfSubstrings\r\n\t].\r\n\tself triggerEvent: #searchStarted.\r\n\tkeys := what collect: [:each| each asString ].\r\n\tkeys notEmpty ifTrue: [ | all collect |\r\n\t\tall := self allMethods.\r\n\t\tall do: [:mth| | current |\r\n\t\t\tcurrent := mth selector.\r\n\t\t\t(keys detect: [:one| current includes: one ] ifNone: []) notNil ifTrue: [\r\n\t\t\t\tself results add: mth.\r\n\t\t\t\tself triggerEvent: #found: with: mth.\r\n\t\t\t].\r\n\t\t].\r\n\t].\r\n\tself triggerEvent: #searchFinished.\r\n\t^self results","(\r\nfunction(self,what)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal keys=nil\r\n(function() if ((((what):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):findMessages_(((what):trimBlanks()):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self):triggerEvent_(\"searchStarted\")\r\n;\r\n(function()\r\n\tlocal _s8_value=((what):collect_((function(each)\r\nreturn (smalltalk:asString_((each)))\r\nend)))\r\n\tkeys=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((keys):notEmpty()))==true) then return ((function()\r\nlocal all=nil\r\nlocal collect=nil\r\n(function()\r\n\tlocal _s8_value=((self):allMethods())\r\n\tall=_s8_value; return _s8_value\r\nend)();return ((all):do_((function(mth)\r\nlocal current=nil\r\n(function()\r\n\tlocal _s8_value=((mth):selector())\r\n\tcurrent=_s8_value; return _s8_value\r\nend)();return ((function() if (((((((keys):detect_ifNone_((function(one)\r\nreturn ((current):includes_(one))\r\nend), (function()\r\nreturn (nil)\r\nend))))~=nil)))==true) then return ((function()\r\n((self):results()):add_(mth);return ((self):triggerEvent_with_(\"found:\", mth))\r\nend))() else return nil end end)())\r\nend)))\r\nend))() else return nil end end)()\r\n;\r\n(self):triggerEvent_(\"searchFinished\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):results())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager["$klass"],"constructEventsTriggered",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:superImplementor(smalltalk.SystemManager["$klass"],"constructEventsTriggered")(self)),smalltalk:newArray({"aboutToBind:to:", "methodBound:to:", "aboutToRemove:from:", "methodRemoved:from:", "aboutToChange:to:", "aboutToCreateClass:with:", "classChanged:", "classShapeChanged:", "object:at:notFoundIn:", "log:", "memoryWarning:"})))
;
end
)
,"events","constructEventsTriggered\r\n\t\" Private - Return the events managed by instances of the receiver. \"\r\n\r\n\t^super constructEventsTriggered ,#(\r\n\t\t#aboutToBind:to:\r\n\t\t#methodBound:to:\r\n\t\t#aboutToRemove:from:\r\n\t\t#methodRemoved:from:\r\n\t\t#aboutToChange:to:\r\n\t\t#aboutToCreateClass:with:\r\n\t\t#classChanged:\r\n\t\t#classShapeChanged:\r\n\t\t#object:at:notFoundIn:\r\n\t\t#log:\r\n\t\t#memoryWarning:\r\n\t)","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:superImplementor(smalltalk.SystemManager[\"$klass\"],\"constructEventsTriggered\")(self)),smalltalk:newArray({\"aboutToBind:to:\", \"methodBound:to:\", \"aboutToRemove:from:\", \"methodRemoved:from:\", \"aboutToChange:to:\", \"aboutToCreateClass:with:\", \"classChanged:\", \"classShapeChanged:\", \"object:at:notFoundIn:\", \"log:\", \"memoryWarning:\"})))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager["$klass"],"currentOrNil",0
,(
function(self)
return ((self):classVariableAt_ifAbsent_("Current", (function()
return (nil)
end)))
;
end
)
,"singleton","currentOrNil\r\n\t\" Return the current instance of the receiver. \"\r\n\r\n\t^self classVariableAt: #Current ifAbsent: [ nil ]","(\r\nfunction(self)\r\nreturn ((self):classVariableAt_ifAbsent_(\"Current\", (function()\r\nreturn (nil)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager["$klass"],"current",0
,(
function(self)
(function() if ((((((smalltalk.SystemManager:classVariableAt_("Current")))==nil)))==true) then return ((function()
return ((self):current_((self):new()))
end))() else return nil end end)()
;
return (smalltalk.SystemManager:classVariableAt_("Current"))
;
end
)
,"singleton","current\r\n\t\" Return the current instance of the receiver. \"\r\n\r\n\tCurrent isNil ifTrue: [ self current: self new ].\r\n\t^Current","(\r\nfunction(self)\r\n(function() if ((((((smalltalk.SystemManager:classVariableAt_(\"Current\")))==nil)))==true) then return ((function()\r\nreturn ((self):current_((self):new()))\r\nend))() else return nil end end)()\r\n;\r\nreturn (smalltalk.SystemManager:classVariableAt_(\"Current\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager["$klass"],"current:",0
,(
function(self,aSystemManager)
smalltalk.SystemManager:classVariableAt_put_("Current",aSystemManager)
;
(aSystemManager):install()
;
return (self)
;
end
)
,"singleton","current: aSystemManager\r\n\t\" Set the current instance of the receiver. \"\r\n\r\n\tCurrent := aSystemManager.\r\n\taSystemManager install.","(\r\nfunction(self,aSystemManager)\r\nsmalltalk.SystemManager:classVariableAt_put_(\"Current\",aSystemManager)\r\n;\r\n(aSystemManager):install()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager["$klass"],"log:",0
,(
function(self,aString)
(function() if ((((((smalltalk.SystemManager:classVariableAt_("Current")))~=nil)))==true) then return ((function()
return ((smalltalk.SystemManager:classVariableAt_("Current")):log_(aString))
end))() else return nil end end)()
;
return (self)
;
end
)
,"log","log: aString\r\n\t\" Log aString onto loggin stream. \"\r\n\r\n\tCurrent notNil ifTrue: [ Current log: aString ]","(\r\nfunction(self,aString)\r\n(function() if ((((((smalltalk.SystemManager:classVariableAt_(\"Current\")))~=nil)))==true) then return ((function()\r\nreturn ((smalltalk.SystemManager:classVariableAt_(\"Current\")):log_(aString))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager["$klass"],"memoryWarning:",0
,(
function(self,anArgument)
(function() if ((((((smalltalk.SystemManager:classVariableAt_("Current")))~=nil)))==true) then return ((function()
return ((smalltalk.SystemManager:classVariableAt_("Current")):memoryWarning_(anArgument))
end))() else return nil end end)()
;
return (self)
;
end
)
,"memory","memoryWarning: anArgument\r\n\t\" Trigger a memory warning event. \"\r\n\r\n\tCurrent notNil ifTrue: [ Current memoryWarning: anArgument ]","(\r\nfunction(self,anArgument)\r\n(function() if ((((((smalltalk.SystemManager:classVariableAt_(\"Current\")))~=nil)))==true) then return ((function()\r\nreturn ((smalltalk.SystemManager:classVariableAt_(\"Current\")):memoryWarning_(anArgument))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"install",0
,(
function(self)
return (self)
;
end
)
,"install","install\r\n\t\" Private - Install the receiver as the source manager of the system.\r\n\tThe default action is to do nothing.\r\n\tThis method can be refined by subclasses.\r\n\t\"","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"aboutToBind:to:",0
,(
function(self,aMethod,aClass)
return ((self):triggerEvent_with_with_("aboutToBind:to:", aMethod, aClass))
;
end
)
,"tracking-methods","aboutToBind: aMethod to: aClass\r\n\t\" Private - The method will be bound to aClass. \"\r\n\r\n\t^self triggerEvent: #aboutToBind:to: with: aMethod with: aClass ","(\r\nfunction(self,aMethod,aClass)\r\nreturn ((self):triggerEvent_with_with_(\"aboutToBind:to:\", aMethod, aClass))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"methodBound:to:",0
,(
function(self,aMethod,aClass)
return ((self):triggerEvent_with_with_("methodBound:to:", aMethod, aClass))
;
end
)
,"tracking-methods","methodBound: aMethod to: aClass\r\n\t\" Private - The method has been bound to aClass. \"\r\n\r\n\t^self triggerEvent: #methodBound:to: with: aMethod with: aClass ","(\r\nfunction(self,aMethod,aClass)\r\nreturn ((self):triggerEvent_with_with_(\"methodBound:to:\", aMethod, aClass))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"aboutToRemove:from:",0
,(
function(self,aMethod,aClass)
return ((self):triggerEvent_with_with_("aboutToRemove:from:", aMethod, aClass))
;
end
)
,"tracking-methods","aboutToRemove: aMethod from: aClass\r\n\t\" Private - The method will be removed from aClass. \"\r\n\r\n\t^self triggerEvent: #aboutToRemove:from: with: aMethod with: aClass ","(\r\nfunction(self,aMethod,aClass)\r\nreturn ((self):triggerEvent_with_with_(\"aboutToRemove:from:\", aMethod, aClass))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"methodRemoved:from:",0
,(
function(self,aMethod,aClass)
return ((self):triggerEvent_with_with_("methodRemoved:from:", aMethod, aClass))
;
end
)
,"tracking-methods","methodRemoved: aMethod from: aClass\r\n\t\" Private - The method has been removed from aClass. \"\r\n\r\n\t^self triggerEvent: #methodRemoved:from: with: aMethod with: aClass ","(\r\nfunction(self,aMethod,aClass)\r\nreturn ((self):triggerEvent_with_with_(\"methodRemoved:from:\", aMethod, aClass))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"aboutToChange:superclass:iVarNames:category:",0
,(
function(self,aClass,superclass,iVarNames,category)
return ((self):triggerEvent_with_with_("aboutToChange:to:", aClass, (smalltalk.Array):with_with_with_(superclass, iVarNames, category)))
;
end
)
,"tracking-classes","aboutToChange: aClass superclass: superclass iVarNames: iVarNames category: category\r\n\t\" Private - aClass is about to be changed. \"\r\n\r\n\t^self\ttriggerEvent: #aboutToChange:to:\r\n\t\twith: aClass\r\n\t\twith: (Array with: superclass with: iVarNames with: category)","(\r\nfunction(self,aClass,superclass,iVarNames,category)\r\nreturn ((self):triggerEvent_with_with_(\"aboutToChange:to:\", aClass, (smalltalk.Array):with_with_with_(superclass, iVarNames, category)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"aboutToCreateClass:superclass:iVarNames:category:",0
,(
function(self,globalName,superclass,iVarNames,category)
return ((self):triggerEvent_with_with_("aboutToCreateClass:with:", globalName, (smalltalk.Array):with_with_with_(superclass, iVarNames, category)))
;
end
)
,"tracking-classes","aboutToCreateClass: globalName superclass: superclass iVarNames: iVarNames category: category\r\n\t\" Private - Attemp to create Class with name globalName. \"\r\n\r\n\t^self\ttriggerEvent: #aboutToCreateClass:with:\r\n\t\twith: globalName\r\n\t\twith: (Array with: superclass with: iVarNames with: category)","(\r\nfunction(self,globalName,superclass,iVarNames,category)\r\nreturn ((self):triggerEvent_with_with_(\"aboutToCreateClass:with:\", globalName, (smalltalk.Array):with_with_with_(superclass, iVarNames, category)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"classChanged:",0
,(
function(self,aBehavior)
return ((self):triggerEvent_with_("classChanged:", aBehavior))
;
end
)
,"tracking-classes","classChanged: aBehavior\r\n\t\" Private - aBehavior has been changed. \"\r\n\r\n\t^self triggerEvent: #classChanged: with: aBehavior ","(\r\nfunction(self,aBehavior)\r\nreturn ((self):triggerEvent_with_(\"classChanged:\", aBehavior))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"classShapeChanged:",0
,(
function(self,aBehavior)
return ((self):triggerEvent_with_ifNotHandled_("classShapeChanged:", aBehavior, (function()
return ((aBehavior):recompileAll())
end)))
;
end
)
,"tracking-classes","classShapeChanged: aBehavior\r\n\t\" Private - aBehavior has changed shape. \"\r\n\r\n\t^self\ttriggerEvent: #classShapeChanged:\r\n\t\twith: aBehavior\r\n\t\tifNotHandled: [ aBehavior recompileAll ]","(\r\nfunction(self,aBehavior)\r\nreturn ((self):triggerEvent_with_ifNotHandled_(\"classShapeChanged:\", aBehavior, (function()\r\nreturn ((aBehavior):recompileAll())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"log:",0
,(
function(self,aString)
return ((self):triggerEvent_with_("log:", aString))
;
end
)
,"log","log: aString\r\n\t\" Private - Log aString onto loggin device. \"\r\n\r\n\t^self triggerEvent: #log: with: aString","(\r\nfunction(self,aString)\r\nreturn ((self):triggerEvent_with_(\"log:\", aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"nextPutAll:",0
,(
function(self,aString)
return ((self):log_(aString))
;
end
)
,"streaming","nextPutAll: aString\r\n\t\" Log aString when streaming. \"\r\n\r\n\t^self log: aString","(\r\nfunction(self,aString)\r\nreturn ((self):log_(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"cr",0
,(
function(self)
return (self)
;
end
)
,"streaming","cr\r\n\t\" Ignore the output, we log as line oriented streaming service. \"","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"memoryWarning:",0
,(
function(self,anArgument)
return ((self):triggerEvent_with_("memoryWarning:", anArgument))
;
end
)
,"memory","memoryWarning: anArgument\r\n\t\" Trigger a memory warning event.\r\n\tThe execution environment can signal memory warning/low conditions, we trigger an event to be handled at application level.\r\n\tNote: It is frequent procedure to clean cached resources that can be reconstructed later, if required.\r\n\t\"\r\n\r\n\t^self triggerEvent: #memoryWarning: with: anArgument","(\r\nfunction(self,anArgument)\r\nreturn ((self):triggerEvent_with_(\"memoryWarning:\", anArgument))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SystemManager,"object:at:notFoundIn:",0
,(
function(self,anObject,indexOrKey,aCollection)
return ((self):triggerEvent_withArguments_ifNotHandled_("object:at:notFoundIn:", (smalltalk.Array):with_with_with_(anObject, indexOrKey, aCollection), (function()
return ((self):print_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("Object/key("),smalltalk:toString_(((function() if ((((((indexOrKey))==nil)))==true) then return ((function()
return (anObject)
end))() else return ((function()
return (indexOrKey)
end))() end end)())))),") not found in ")),smalltalk:printString_((aCollection)))))
end)))
;
end
)
,"failures","object: anObject at: indexOrKey notFoundIn: aCollection\r\n\t\" Trigger a missing object event. \"\r\n\r\n\t^self triggerEvent: #object:at:notFoundIn:\r\n\t\twithArguments: (Array with: anObject with: indexOrKey with: aCollection)\r\n\t\tifNotHandled: [\r\n\t\t\tself print: 'Object/key('\r\n\t\t\t\t,(indexOrKey isNil ifTrue: [anObject] ifFalse: [indexOrKey]) toString\r\n\t\t\t\t,') not found in ',aCollection printString\r\n\t\t]","(\r\nfunction(self,anObject,indexOrKey,aCollection)\r\nreturn ((self):triggerEvent_withArguments_ifNotHandled_(\"object:at:notFoundIn:\", (smalltalk.Array):with_with_with_(anObject, indexOrKey, aCollection), (function()\r\nreturn ((self):print_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"Object/key(\"),smalltalk:toString_(((function() if ((((((indexOrKey))==nil)))==true) then return ((function()\r\nreturn (anObject)\r\nend))() else return ((function()\r\nreturn (indexOrKey)\r\nend))() end end)())))),\") not found in \")),smalltalk:printString_((aCollection)))))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"nodes",0
,(
function(self)
(function() if ((((((self['@nodes']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.Array):new())
	self['@nodes']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (self['@nodes'])
;
end
)
,"accessing","nodes\r\n\tnodes isNil ifTrue: [ nodes := Array new ].\r\n\t^nodes","(\r\nfunction(self)\r\n(function() if ((((((self['@nodes']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tself['@nodes']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@nodes'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"nodes:",0
,(
function(self,aCollection)
(function()
	local _s8_value=(aCollection)
	self['@nodes']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","nodes: aCollection\r\n\tnodes := aCollection","(\r\nfunction(self,aCollection)\r\n(function()\r\n\tlocal _s8_value=(aCollection)\r\n\tself['@nodes']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"annotations",0
,(
function(self)
(function() if ((((((self['@annotations']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.Array):new())
	self['@annotations']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (self['@annotations'])
;
end
)
,"accessing","annotations\r\n\tannotations isNil ifTrue: [ annotations := Array new ].\r\n\t^annotations","(\r\nfunction(self)\r\n(function() if ((((((self['@annotations']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tself['@annotations']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@annotations'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"annotations:",0
,(
function(self,aCollection)
(function()
	local _s8_value=(aCollection)
	self['@annotations']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","annotations: aCollection\r\n\tannotations := aCollection","(\r\nfunction(self,aCollection)\r\n(function()\r\n\tlocal _s8_value=(aCollection)\r\n\tself['@annotations']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"addNode:",0
,(
function(self,aNode)
((self):nodes()):add_(aNode)
;
return (self)
;
end
)
,"accessing","addNode: aNode\r\n\tself nodes add: aNode","(\r\nfunction(self,aNode)\r\n((self):nodes()):add_(aNode)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"start",0
,(
function(self)
return (self['@start'])
;
end
)
,"accessing","start\r\n\t^start","(\r\nfunction(self)\r\nreturn (self['@start'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"end","__end"
,(
function(self)
return (self['@end'])
;
end
)
,"accessing","end\r\n\t^end","(\r\nfunction(self)\r\nreturn (self['@end'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"start:end:",0
,(
function(self,startOffset,endOffset)
(function()
	local _s8_value=(startOffset)
	self['@start']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(endOffset)
	self['@end']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","start: startOffset end: endOffset\r\n\t\" Set the offsets of the receiver in source code. \"\r\n\r\n\tstart := startOffset.\r\n\tend := endOffset.","(\r\nfunction(self,startOffset,endOffset)\r\n(function()\r\n\tlocal _s8_value=(startOffset)\r\n\tself['@start']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(endOffset)\r\n\tself['@end']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"acceptMacroAssignment:with:",0
,(
function(self,anExpressionNode,aVisitor)
return ((aVisitor):visitMacroAssignment_to_(self, anExpressionNode))
;
end
)
,"visiting","acceptMacroAssignment: anExpressionNode with: aVisitor\r\n\t\" Return true if the receiver can resolve assignment on aVisitor. \"\r\n\r\n\t^aVisitor visitMacroAssignment: self to: anExpressionNode","(\r\nfunction(self,anExpressionNode,aVisitor)\r\nreturn ((aVisitor):visitMacroAssignment_to_(self, anExpressionNode))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"isSuper",0
,(
function(self)
return (false)
;
end
)
,"testing","isSuper\r\n\t\" Return true if the receiver is super. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"hasNumber",0
,(
function(self)
return (false)
;
end
)
,"testing","hasNumber\r\n\t\" Return true if the receiver has a numeric literal value. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"hasString",0
,(
function(self)
return (false)
;
end
)
,"testing","hasString\r\n\t\" Return true if the receiver has a string literal value. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"isAssignment",0
,(
function(self)
return (false)
;
end
)
,"testing","isAssignment\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"isBlock",0
,(
function(self)
return (false)
;
end
)
,"testing","isBlock\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"isCascade",0
,(
function(self)
return (false)
;
end
)
,"testing","isCascade\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"isInline",0
,(
function(self)
return (false)
;
end
)
,"testing","isInline\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"isReturn",0
,(
function(self)
return (false)
;
end
)
,"testing","isReturn\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"isSend",0
,(
function(self)
return (false)
;
end
)
,"testing","isSend\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"isSequence",0
,(
function(self)
return (false)
;
end
)
,"testing","isSequence\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"isValue",0
,(
function(self)
return (false)
;
end
)
,"testing","isValue\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"isVariable",0
,(
function(self)
return (false)
;
end
)
,"testing","isVariable\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"bindings",0
,(
function(self)
return (smalltalk:newArray({}))
;
end
)
,"bindings","bindings\r\n\t\" Return the bindings of the receiver. \"\r\n\r\n\t^#()","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"do:",0
,(
function(self,aBlock)
(aBlock):value_(self)
;
((self):nodes()):do_((function(each)
return ((each):do_(aBlock))
end))
;
return (self)
;
end
)
,"iteration","do: aBlock\r\n\t\" Evaluate aBlock for all contents of the receiver. \"\r\n\r\n\taBlock value: self.\r\n\tself nodes do: [:each| each do: aBlock ].","(\r\nfunction(self,aBlock)\r\n(aBlock):value_(self)\r\n;\r\n((self):nodes()):do_((function(each)\r\nreturn ((each):do_(aBlock))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CompilerNode,"messageSendConnector",0
,(
function(self)
return (".")
;
end
)
,"accessing","messageSendConnector\r\n\t\" Return the binder used to send message to the receiver. \"\r\n\r\n\t^'.'","(\r\nfunction(self)\r\nreturn (\".\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodNode,"selector",0
,(
function(self)
return (self['@selector'])
;
end
)
,"accessing","selector\r\n\t^selector","(\r\nfunction(self)\r\nreturn (self['@selector'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodNode,"selector:",0
,(
function(self,aString)
(function()
	local _s8_value=(aString)
	self['@selector']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","selector: aString\r\n\tselector := aString","(\r\nfunction(self,aString)\r\n(function()\r\n\tlocal _s8_value=(aString)\r\n\tself['@selector']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodNode,"arguments",0
,(
function(self)
return ((function() if ((((((self['@arguments']))==nil)))==true) then return ((function()
return (smalltalk:newArray({}))
end))() else return ((function()
return (self['@arguments'])
end))() end end)())
;
end
)
,"accessing","arguments\r\n\t^arguments isNil ifTrue: [ #() ] ifFalse: [ arguments ]","(\r\nfunction(self)\r\nreturn ((function() if ((((((self['@arguments']))==nil)))==true) then return ((function()\r\nreturn (smalltalk:newArray({}))\r\nend))() else return ((function()\r\nreturn (self['@arguments'])\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodNode,"arguments:",0
,(
function(self,aCollection)
(function()
	local _s8_value=(aCollection)
	self['@arguments']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","arguments: aCollection\r\n\targuments := aCollection","(\r\nfunction(self,aCollection)\r\n(function()\r\n\tlocal _s8_value=(aCollection)\r\n\tself['@arguments']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodNode,"argumentNames",0
,(
function(self)
return (((self):arguments()):collect_((function(each)
return ((function() if ((((each):isString()))==true) then return ((function()
return (each)
end))() else return ((function()
return ((each):value())
end))() end end)())
end)))
;
end
)
,"accessing","argumentNames\r\n\t\" Return the names of the arguments of the receiver. \"\r\n\r\n\t^self arguments collect: [:each|\r\n\t\teach isString\r\n\t\tifTrue: [each]\r\n\t\tifFalse: [each value]\r\n\t]","(\r\nfunction(self)\r\nreturn (((self):arguments()):collect_((function(each)\r\nreturn ((function() if ((((each):isString()))==true) then return ((function()\r\nreturn (each)\r\nend))() else return ((function()\r\nreturn ((each):value())\r\nend))() end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodNode,"source",0
,(
function(self)
return (self['@source'])
;
end
)
,"accessing","source\r\n\t^source","(\r\nfunction(self)\r\nreturn (self['@source'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodNode,"source:",0
,(
function(self,aString)
(function()
	local _s8_value=(aString)
	self['@source']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","source: aString\r\n\tsource := aString","(\r\nfunction(self,aString)\r\n(function()\r\n\tlocal _s8_value=(aString)\r\n\tself['@source']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitMethodNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitMethodNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitMethodNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.MethodNode,"validateArguments",0
,(
function(self)
local names=nil
(function()
	local _s8_value=((smalltalk.HashedSet):withAll_((self):argumentNames()))
	names=_s8_value; return _s8_value
end)()
;
((self):nodes()):do_((function(each)
return ((each):validateNames_(names))
end))
;
return (self)
;
end
)
,"validation","validateArguments\r\n\t\" Ensure the nodes of the receiver can be emmited with the defined arguments. \"\r\n\r\n\t| names |\r\n\tnames := HashedSet withAll: self argumentNames.\r\n\tself nodes do: [:each| each validateNames: names ]","(\r\nfunction(self)\r\nlocal names=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.HashedSet):withAll_((self):argumentNames()))\r\n\tnames=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((self):nodes()):do_((function(each)\r\nreturn ((each):validateNames_(names))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SendNode,"selector",0
,(
function(self)
return (self['@selector'])
;
end
)
,"accessing","selector\r\n\t^selector","(\r\nfunction(self)\r\nreturn (self['@selector'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SendNode,"selector:",0
,(
function(self,aString)
(function()
	local _s8_value=(aString)
	self['@selector']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","selector: aString\r\n\tselector := aString","(\r\nfunction(self,aString)\r\n(function()\r\n\tlocal _s8_value=(aString)\r\n\tself['@selector']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SendNode,"arguments",0
,(
function(self)
return ((function() if ((((((self['@arguments']))==nil)))==true) then return ((function()
return (smalltalk:newArray({}))
end))() else return ((function()
return (self['@arguments'])
end))() end end)())
;
end
)
,"accessing","arguments\r\n\t^arguments isNil ifTrue: [ #() ] ifFalse: [ arguments ]","(\r\nfunction(self)\r\nreturn ((function() if ((((((self['@arguments']))==nil)))==true) then return ((function()\r\nreturn (smalltalk:newArray({}))\r\nend))() else return ((function()\r\nreturn (self['@arguments'])\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SendNode,"arguments:",0
,(
function(self,aCollection)
(function()
	local _s8_value=(aCollection)
	self['@arguments']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","arguments: aCollection\r\n\targuments := aCollection","(\r\nfunction(self,aCollection)\r\n(function()\r\n\tlocal _s8_value=(aCollection)\r\n\tself['@arguments']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SendNode,"receiver",0
,(
function(self)
return (self['@receiver'])
;
end
)
,"accessing","receiver\r\n\t^receiver","(\r\nfunction(self)\r\nreturn (self['@receiver'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SendNode,"receiver:",0
,(
function(self,aNode)
(function()
	local _s8_value=(aNode)
	self['@receiver']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","receiver: aNode\r\n\treceiver := aNode","(\r\nfunction(self,aNode)\r\n(function()\r\n\tlocal _s8_value=(aNode)\r\n\tself['@receiver']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SendNode,"valueForReceiver:",0
,(
function(self,anObject)
return ((
function(_s8_rec)
(_s8_rec):receiver_((function() if (((((((self):receiver()))==nil)))==true) then return ((function()
return (anObject)
end))() else return ((function()
return (((self):receiver()):valueForReceiver_(anObject))
end))() end end)());
(_s8_rec):selector_((self):selector());
(_s8_rec):arguments_((self):arguments());
(_s8_rec):start_end_((self):start(), (self):__end());
return ((_s8_rec):yourself())
end
)((smalltalk.SendNode):new()))
;
end
)
,"accessing","valueForReceiver: anObject\r\n\t^SendNode new\r\n\t    receiver: (self receiver isNil ifTrue: [anObject]\r\n\t\tifFalse: [self receiver valueForReceiver: anObject]);\r\n\t    selector: self selector;\r\n\t    arguments: self arguments;\r\n\t    start: self start end: self end;\r\n\t    yourself","(\r\nfunction(self,anObject)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):receiver_((function() if (((((((self):receiver()))==nil)))==true) then return ((function()\r\nreturn (anObject)\r\nend))() else return ((function()\r\nreturn (((self):receiver()):valueForReceiver_(anObject))\r\nend))() end end)());\r\n(_s8_rec):selector_((self):selector());\r\n(_s8_rec):arguments_((self):arguments());\r\n(_s8_rec):start_end_((self):start(), (self):__end());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.SendNode):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SendNode,"cascadeNodeWithMessages:",0
,(
function(self,aCollection)
local first=nil
local sends=nil
(function()
	local _s8_value=((
function(_s8_rec)
(_s8_rec):selector_((self):selector());
(_s8_rec):arguments_((self):arguments());
(_s8_rec):start_end_((self):start(), (self):__end());
return ((_s8_rec):yourself())
end
)((smalltalk.SendNode):new()))
	first=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(smalltalk:object_comma_(((smalltalk.Array):with_(first)),aCollection))
	sends=_s8_value; return _s8_value
end)()
;
return ((
function(_s8_rec)
(_s8_rec):receiver_((self):receiver());
(_s8_rec):nodes_(sends);
(_s8_rec):start_end_(((sends):first()):start(), ((sends):last()):__end());
return ((_s8_rec):yourself())
end
)((smalltalk.CascadeNode):new()))
;
end
)
,"accessing","cascadeNodeWithMessages: aCollection\r\n\t| first sends |\r\n\tfirst := SendNode new\r\n\t    selector: self selector;\r\n\t    arguments: self arguments;\r\n\t    start: self start end: self end;\r\n\t    yourself.\r\n\tsends := (Array with: first), aCollection.\r\n\t^CascadeNode new\r\n\t    receiver: self receiver;\r\n\t    nodes: sends;\r\n\t    start: sends first start end: sends last end;\r\n\t    yourself","(\r\nfunction(self,aCollection)\r\nlocal first=nil\r\nlocal sends=nil\r\n(function()\r\n\tlocal _s8_value=((\r\nfunction(_s8_rec)\r\n(_s8_rec):selector_((self):selector());\r\n(_s8_rec):arguments_((self):arguments());\r\n(_s8_rec):start_end_((self):start(), (self):__end());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.SendNode):new()))\r\n\tfirst=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_(((smalltalk.Array):with_(first)),aCollection))\r\n\tsends=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):receiver_((self):receiver());\r\n(_s8_rec):nodes_(sends);\r\n(_s8_rec):start_end_(((sends):first()):start(), ((sends):last()):__end());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.CascadeNode):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SendNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitSendNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitSendNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitSendNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SendNode,"isSend",0
,(
function(self)
return (true)
;
end
)
,"testing","isSend\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SendNode,"do:",0
,(
function(self,aBlock)
smalltalk:superImplementor(smalltalk.SendNode,"do_")(self,aBlock)
;
(function() if ((((((self['@receiver']))~=nil)))==true) then return ((function()
return ((self['@receiver']):do_(aBlock))
end))() else return nil end end)()
;
((self):arguments()):do_((function(each)
return ((each):do_(aBlock))
end))
;
return (self)
;
end
)
,"iteration","do: aBlock\r\n\t\" Evaluate aBlock for all contents of the receiver. \"\r\n\r\n\tsuper do: aBlock.\r\n\treceiver notNil ifTrue: [ receiver do: aBlock ].\r\n\tself arguments do: [:each| each do: aBlock ].","(\r\nfunction(self,aBlock)\r\nsmalltalk:superImplementor(smalltalk.SendNode,\"do_\")(self,aBlock)\r\n;\r\n(function() if ((((((self['@receiver']))~=nil)))==true) then return ((function()\r\nreturn ((self['@receiver']):do_(aBlock))\r\nend))() else return nil end end)()\r\n;\r\n((self):arguments()):do_((function(each)\r\nreturn ((each):do_(aBlock))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CascadeNode,"receiver",0
,(
function(self)
return (self['@receiver'])
;
end
)
,"accessing","receiver\r\n\t^receiver","(\r\nfunction(self)\r\nreturn (self['@receiver'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CascadeNode,"receiver:",0
,(
function(self,aNode)
(function()
	local _s8_value=(aNode)
	self['@receiver']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","receiver: aNode\r\n\treceiver := aNode","(\r\nfunction(self,aNode)\r\n(function()\r\n\tlocal _s8_value=(aNode)\r\n\tself['@receiver']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CascadeNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitCascadeNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitCascadeNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitCascadeNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CascadeNode,"isCascade",0
,(
function(self)
return (true)
;
end
)
,"testing","isCascade\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.CascadeNode,"do:",0
,(
function(self,aBlock)
smalltalk:superImplementor(smalltalk.CascadeNode,"do_")(self,aBlock)
;
(function() if ((((((self['@receiver']))~=nil)))==true) then return ((function()
return ((self['@receiver']):do_(aBlock))
end))() else return nil end end)()
;
return (self)
;
end
)
,"iteration","do: aBlock\r\n\t\" Evaluate aBlock for all contents of the receiver. \"\r\n\r\n\tsuper do: aBlock.\r\n\treceiver notNil ifTrue: [ receiver do: aBlock ].","(\r\nfunction(self,aBlock)\r\nsmalltalk:superImplementor(smalltalk.CascadeNode,\"do_\")(self,aBlock)\r\n;\r\n(function() if ((((((self['@receiver']))~=nil)))==true) then return ((function()\r\nreturn ((self['@receiver']):do_(aBlock))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AssignmentNode,"left",0
,(
function(self)
return (self['@left'])
;
end
)
,"accessing","left\r\n\t^left","(\r\nfunction(self)\r\nreturn (self['@left'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AssignmentNode,"left:",0
,(
function(self,aNode)
(function()
	local _s8_value=(aNode)
	self['@left']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","left: aNode\r\n\tleft := aNode","(\r\nfunction(self,aNode)\r\n(function()\r\n\tlocal _s8_value=(aNode)\r\n\tself['@left']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AssignmentNode,"right",0
,(
function(self)
return (self['@right'])
;
end
)
,"accessing","right\r\n\t^right","(\r\nfunction(self)\r\nreturn (self['@right'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AssignmentNode,"right:",0
,(
function(self,aNode)
(function()
	local _s8_value=(aNode)
	self['@right']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","right: aNode\r\n\tright := aNode","(\r\nfunction(self,aNode)\r\n(function()\r\n\tlocal _s8_value=(aNode)\r\n\tself['@right']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AssignmentNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitAssignmentNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitAssignmentNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitAssignmentNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AssignmentNode,"isAssignment",0
,(
function(self)
return (true)
;
end
)
,"testing","isAssignment\r\n\t\" Return true if the receiver is an assignment node. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AssignmentNode,"do:",0
,(
function(self,aBlock)
smalltalk:superImplementor(smalltalk.AssignmentNode,"do_")(self,aBlock)
;
(function() if ((((((self['@left']))~=nil)))==true) then return ((function()
return ((self['@left']):do_(aBlock))
end))() else return nil end end)()
;
(function() if ((((((self['@right']))~=nil)))==true) then return ((function()
return ((self['@right']):do_(aBlock))
end))() else return nil end end)()
;
return (self)
;
end
)
,"iteration","do: aBlock\r\n\t\" Evaluate aBlock for all contents of the receiver. \"\r\n\r\n\tsuper do: aBlock.\r\n\tleft  notNil ifTrue: [ left  do: aBlock ].\r\n\tright notNil ifTrue: [ right do: aBlock ].","(\r\nfunction(self,aBlock)\r\nsmalltalk:superImplementor(smalltalk.AssignmentNode,\"do_\")(self,aBlock)\r\n;\r\n(function() if ((((((self['@left']))~=nil)))==true) then return ((function()\r\nreturn ((self['@left']):do_(aBlock))\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((self['@right']))~=nil)))==true) then return ((function()\r\nreturn ((self['@right']):do_(aBlock))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockNode,"parameterNames",0
,(
function(self)
return (((self):parameters()):collect_((function(each)
return ((function() if ((((each):isString()))==true) then return ((function()
return (each)
end))() else return ((function()
return ((each):value())
end))() end end)())
end)))
;
end
)
,"accessing","parameterNames\r\n\t\" Return the names of the parameters of the receiver. \"\r\n\r\n\t^self parameters collect: [:each|\r\n\t\teach isString\r\n\t\tifTrue: [each]\r\n\t\tifFalse: [each value]\r\n\t]","(\r\nfunction(self)\r\nreturn (((self):parameters()):collect_((function(each)\r\nreturn ((function() if ((((each):isString()))==true) then return ((function()\r\nreturn (each)\r\nend))() else return ((function()\r\nreturn ((each):value())\r\nend))() end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockNode,"parameters",0
,(
function(self)
(function() if ((((((self['@parameters']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.Array):new())
	self['@parameters']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (self['@parameters'])
;
end
)
,"accessing","parameters\r\n\tparameters isNil ifTrue: [ parameters := Array new ].\r\n\t^parameters","(\r\nfunction(self)\r\n(function() if ((((((self['@parameters']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tself['@parameters']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@parameters'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockNode,"parameters:",0
,(
function(self,aCollection)
(function()
	local _s8_value=(aCollection)
	self['@parameters']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","parameters: aCollection\r\n\tparameters := aCollection","(\r\nfunction(self,aCollection)\r\n(function()\r\n\tlocal _s8_value=(aCollection)\r\n\tself['@parameters']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitBlockNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitBlockNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitBlockNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockNode,"bindings",0
,(
function(self)
return ((self):parameterNames())
;
end
)
,"bindings","bindings\r\n\t\" Return the bindings of the receiver. \"\r\n\r\n\t^self parameterNames","(\r\nfunction(self)\r\nreturn ((self):parameterNames())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockNode,"isBlock",0
,(
function(self)
return (true)
;
end
)
,"testing","isBlock\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockNode,"do:",0
,(
function(self,aBlock)
smalltalk:superImplementor(smalltalk.BlockNode,"do_")(self,aBlock)
;
((self):parameters()):do_((function(each)
return ((each):do_(aBlock))
end))
;
return (self)
;
end
)
,"iteration","do: aBlock\r\n\t\" Evaluate aBlock for all contents of the receiver. \"\r\n\r\n\tsuper do: aBlock.\r\n\tself parameters do: [:each| each do: aBlock ].","(\r\nfunction(self,aBlock)\r\nsmalltalk:superImplementor(smalltalk.BlockNode,\"do_\")(self,aBlock)\r\n;\r\n((self):parameters()):do_((function(each)\r\nreturn ((each):do_(aBlock))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceNode,"tempNames",0
,(
function(self)
return (((self):temps()):collect_((function(each)
return ((function() if ((((each):isString()))==true) then return ((function()
return (each)
end))() else return ((function()
return ((each):value())
end))() end end)())
end)))
;
end
)
,"accessing","tempNames\r\n\t\" Return the names of the temps in the receiver. \"\r\n\r\n\t^self temps collect: [:each|\r\n\t\teach isString\r\n\t\tifTrue: [each]\r\n\t\tifFalse: [each value]\r\n\t]","(\r\nfunction(self)\r\nreturn (((self):temps()):collect_((function(each)\r\nreturn ((function() if ((((each):isString()))==true) then return ((function()\r\nreturn (each)\r\nend))() else return ((function()\r\nreturn ((each):value())\r\nend))() end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceNode,"temps",0
,(
function(self)
return ((function() if ((((((self['@temps']))==nil)))==true) then return ((function()
return (smalltalk:newArray({}))
end))() else return ((function()
return (self['@temps'])
end))() end end)())
;
end
)
,"accessing","temps\r\n\t^temps isNil ifTrue: [ #() ] ifFalse: [ temps ]","(\r\nfunction(self)\r\nreturn ((function() if ((((((self['@temps']))==nil)))==true) then return ((function()\r\nreturn (smalltalk:newArray({}))\r\nend))() else return ((function()\r\nreturn (self['@temps'])\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceNode,"temps:",0
,(
function(self,aCollection)
(function()
	local _s8_value=(aCollection)
	self['@temps']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","temps: aCollection\r\n\ttemps := aCollection","(\r\nfunction(self,aCollection)\r\n(function()\r\n\tlocal _s8_value=(aCollection)\r\n\tself['@temps']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceNode,"asBlockSequenceNode",0
,(
function(self)
return ((
function(_s8_rec)
(_s8_rec):nodes_((self):nodes());
(_s8_rec):temps_((self):temps());
(_s8_rec):start_end_((self):start(), (self):__end());
return ((_s8_rec):yourself())
end
)((smalltalk.BlockSequenceNode):new()))
;
end
)
,"converting","asBlockSequenceNode\r\n\t^BlockSequenceNode new\r\n\t    nodes: self nodes;\r\n\t    temps: self temps;\r\n\t    start: self start end: self end;\r\n\t    yourself","(\r\nfunction(self)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nodes_((self):nodes());\r\n(_s8_rec):temps_((self):temps());\r\n(_s8_rec):start_end_((self):start(), (self):__end());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.BlockSequenceNode):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitSequenceNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitSequenceNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitSequenceNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceNode,"bindings",0
,(
function(self)
return ((self):tempNames())
;
end
)
,"bindings","bindings\r\n\t\" Return the bindings of the receiver. \"\r\n\r\n\t^self tempNames","(\r\nfunction(self)\r\nreturn ((self):tempNames())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceNode,"isSequence",0
,(
function(self)
return (true)
;
end
)
,"testing","isSequence\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceNode,"shouldReturn",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:boolean_and_((((self):nodes()):notEmpty()),(function()
return ((((self):nodes()):last()):isReturn())
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(self):addNode_((smalltalk.ReturnNode):variable_at_("self", (self):__end()))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"return","shouldReturn\r\n\t\" Ensure the receiver includes a return node. \"\r\n\r\n\t(self nodes notEmpty and: [\r\n\t\tself nodes last isReturn\r\n\t]) ifTrue: [ ^self].\r\n\r\n\tself addNode: (ReturnNode variable: #self at: self end).","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:boolean_and_((((self):nodes()):notEmpty()),(function()\r\nreturn ((((self):nodes()):last()):isReturn())\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self):addNode_((smalltalk.ReturnNode):variable_at_(\"self\", (self):__end()))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.SequenceNode,"validateNames:",0
,(
function(self,names)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
((self):tempNames()):do_((function(temp)
return ((function() if ((((names):includes_(temp)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_(("Invalid temp ("),temp)),")")))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"validation","validateNames: names\r\n\t\" Validate the receiver can be emmited in a context with names. \"\r\n\r\n\tself tempNames do: [:temp|\r\n\t\t(names includes: temp) ifTrue: [\r\n\t\t\t^self error: 'Invalid temp (',temp,')'\r\n\t\t].\r\n\t]","(\r\nfunction(self,names)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n((self):tempNames()):do_((function(temp)\r\nreturn ((function() if ((((names):includes_(temp)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((\"Invalid temp (\"),temp)),\")\")))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.BlockSequenceNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitBlockSequenceNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitBlockSequenceNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitBlockSequenceNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ExtendedSequenceNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitExtendedSequenceNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitExtendedSequenceNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitExtendedSequenceNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ExtendedSequenceNode,"directives",0
,(
function(self)
return ((function() if ((((((self['@directives']))==nil)))==true) then return ((function()
return (smalltalk:newArray({}))
end))() else return ((function()
return (self['@directives'])
end))() end end)())
;
end
)
,"accessing","directives\r\n\t\" Return the directives of the receiver. \"\r\n\r\n\t^directives isNil\r\n\t\tifTrue: [ #() ]\r\n\t\tifFalse: [directives]","(\r\nfunction(self)\r\nreturn ((function() if ((((((self['@directives']))==nil)))==true) then return ((function()\r\nreturn (smalltalk:newArray({}))\r\nend))() else return ((function()\r\nreturn (self['@directives'])\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ExtendedSequenceNode,"directives:",0
,(
function(self,aCollection)
(function()
	local _s8_value=(aCollection)
	self['@directives']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","directives: aCollection\r\n\t\" Set the directives of the receiver. \"\r\n\r\n\tdirectives := aCollection","(\r\nfunction(self,aCollection)\r\n(function()\r\n\tlocal _s8_value=(aCollection)\r\n\tself['@directives']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ReturnNode["$klass"],"variable:at:",0
,(
function(self,aName,offset)
return ((
function(_s8_rec)
(_s8_rec):addNode_((
function(_s8_rec)
(_s8_rec):value_(aName);
(_s8_rec):start_end_(offset, offset);
return ((_s8_rec):yourself())
end
)((smalltalk.VariableNode):new()));
(_s8_rec):start_end_(offset, offset);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","variable: aName at: offset\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\taddNode: (VariableNode new\r\n\t\t\tvalue: aName;\r\n\t\t\tstart: offset end: offset;\r\n\t\t\tyourself);\r\n\t\tstart: offset end: offset;\r\n\t\tyourself","(\r\nfunction(self,aName,offset)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):addNode_((\r\nfunction(_s8_rec)\r\n(_s8_rec):value_(aName);\r\n(_s8_rec):start_end_(offset, offset);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((smalltalk.VariableNode):new()));\r\n(_s8_rec):start_end_(offset, offset);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ReturnNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitReturnNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitReturnNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitReturnNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ReturnNode,"isReturn",0
,(
function(self)
return (true)
;
end
)
,"testing","isReturn\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ValueNode["$klass"],"with:",0
,(
function(self,aValue)
return (((self):new()):value_(aValue))
;
end
)
,"instantiation","with: aValue\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new value: aValue","(\r\nfunction(self,aValue)\r\nreturn (((self):new()):value_(aValue))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ValueNode,"value",0
,(
function(self)
return (self['@value'])
;
end
)
,"accessing","value\r\n\t^value","(\r\nfunction(self)\r\nreturn (self['@value'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ValueNode,"value:",0
,(
function(self,anObject)
(function()
	local _s8_value=(anObject)
	self['@value']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","value: anObject\r\n\tvalue := anObject","(\r\nfunction(self,anObject)\r\n(function()\r\n\tlocal _s8_value=(anObject)\r\n\tself['@value']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ValueNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitValueNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitValueNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitValueNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ValueNode,"hasNumber",0
,(
function(self)
return ((self['@value']):isNumber())
;
end
)
,"testing","hasNumber\r\n\t\" Return true if the receiver has a numeric literal value. \"\r\n\r\n\t^value isNumber","(\r\nfunction(self)\r\nreturn ((self['@value']):isNumber())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ValueNode,"hasString",0
,(
function(self)
return ((self['@value']):isString())
;
end
)
,"testing","hasString\r\n\t\" Return true if the receiver has a string literal value. \"\r\n\r\n\t^value isString","(\r\nfunction(self)\r\nreturn ((self['@value']):isString())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ValueNode,"isValue",0
,(
function(self)
return (true)
;
end
)
,"testing","isValue\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.VariableNode,"localNameIn:",0
,(
function(self,aCompiler)
return ((aCompiler):localName_((self):value()))
;
end
)
,"accessing","localNameIn: aCompiler\r\n\t\" Return the name of the receiver to be used by aCompiler locally. \"\r\n\r\n\t^aCompiler localName: self value ","(\r\nfunction(self,aCompiler)\r\nreturn ((aCompiler):localName_((self):value()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.VariableNode,"needsVerification",0
,(
function(self)
return (true)
;
end
)
,"testing","needsVerification\r\n\t\" Return true if the receiver need verification. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.VariableNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitVariableNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitVariableNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitVariableNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.VariableNode,"isSuper",0
,(
function(self)
return ((smalltalk:isObject_equalTo_((self['@value']),"super")))
;
end
)
,"testing","isSuper\r\n\t\" Return true if the receiver is super. \"\r\n\r\n\t^value = 'super'","(\r\nfunction(self)\r\nreturn ((smalltalk:isObject_equalTo_((self['@value']),\"super\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.VariableNode,"indices",0
,(
function(self)
return ((function() if ((((((self['@indices']))==nil)))==true) then return ((function()
return ((smalltalk.Array):new())
end))() else return ((function()
return (self['@indices'])
end))() end end)())
;
end
)
,"accessing","indices\r\n\t\" Return the indexing references of the receiver. \"\r\n\r\n\t^indices isNil ifTrue: [ Array new ] ifFalse: [ indices ]","(\r\nfunction(self)\r\nreturn ((function() if ((((((self['@indices']))==nil)))==true) then return ((function()\r\nreturn ((smalltalk.Array):new())\r\nend))() else return ((function()\r\nreturn (self['@indices'])\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.VariableNode,"indexedAs:",0
,(
function(self,chain)
(function()
	local _s8_value=((function() if ((((smalltalk:boolean_and_(((((chain):size()) > (1))),(function()
return (((chain):second()):notEmpty())
end)))))==true) then return ((function()
return (((chain):second()):collect_((function(tuple)
return ((function() if (((((tuple):second()):isString()))==true) then return ((function()
return (((smalltalk.ValueNode):new()):value_((tuple):second()))
end))() else return ((function()
return ((tuple):second())
end))() end end)())
end)))
end))() else return nil end end)())
	self['@indices']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"visiting","indexedAs: chain\r\n\t\" The receiver has indexing references defined by chain. \"\r\n\r\n\tindices := (chain size > 1 and: [ chain second notEmpty ]) ifTrue: [\r\n\t\tchain second collect: [:tuple|\r\n\t\t\ttuple second isString\r\n\t\t\tifTrue: [ ValueNode new value: tuple second ]\r\n\t\t\tifFalse: [ tuple second ]\r\n\t\t]\r\n\t].\r\n\t^self","(\r\nfunction(self,chain)\r\n(function()\r\n\tlocal _s8_value=((function() if ((((smalltalk:boolean_and_(((((chain):size()) > (1))),(function()\r\nreturn (((chain):second()):notEmpty())\r\nend)))))==true) then return ((function()\r\nreturn (((chain):second()):collect_((function(tuple)\r\nreturn ((function() if (((((tuple):second()):isString()))==true) then return ((function()\r\nreturn (((smalltalk.ValueNode):new()):value_((tuple):second()))\r\nend))() else return ((function()\r\nreturn ((tuple):second())\r\nend))() end end)())\r\nend)))\r\nend))() else return nil end end)())\r\n\tself['@indices']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.VariableNode,"isVariable",0
,(
function(self)
return (true)
;
end
)
,"testing","isVariable\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.GlobalReferenceNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitGlobalReferenceNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitGlobalReferenceNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitGlobalReferenceNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NoReceiverNode,"accept:",0
,(
function(self,aVisitor)
return (self)
;
end
)
,"visiting","accept: aVisitor","(\r\nfunction(self,aVisitor)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NoReceiverNode,"messageSendConnector",0
,(
function(self)
return ((smalltalk.String):new())
;
end
)
,"accessing","messageSendConnector\r\n\t\" Return the binder used to send message to the receiver. \"\r\n\r\n\t^String new","(\r\nfunction(self)\r\nreturn ((smalltalk.String):new())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AlienNode,"localNameIn:",0
,(
function(self,aCompiler)
return ((self):value())
;
end
)
,"accessing","localNameIn: aCompiler\r\n\t\" Return the name of the receiver to be used by aCompiler locally. \"\r\n\r\n\t^self value ","(\r\nfunction(self,aCompiler)\r\nreturn ((self):value())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AlienNode,"needsVerification",0
,(
function(self)
return (false)
;
end
)
,"testing","needsVerification\r\n\t\" Return true if the receiver need verification. \"\r\n\r\n\t^false","(\r\nfunction(self)\r\nreturn (false)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.InlineNode,"sources:",0
,(
function(self,tuples)
(function()
	local _s8_value=(tuples)
	self['@sources']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","sources: tuples\r\n\tsources := tuples","(\r\nfunction(self,tuples)\r\n(function()\r\n\tlocal _s8_value=(tuples)\r\n\tself['@sources']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.InlineNode,"sourceFor:",0
,(
function(self,lang)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local default=nil
(self['@sources']):do_((function(tuple)
(function() if ((((smalltalk:isObject_equalTo_(((tuple):first()),lang))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((tuple):last())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function() if (((((((tuple):first()))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((tuple):last())
	default=_s8_value; return _s8_value
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function() if ((((((default))~=nil)))==true) then return ((function()
return (default)
end))() else return nil end end)())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","sourceFor: lang\r\n\t\" Returns the native source code to inline (or nil). \"\r\n\r\n\t| default |\r\n\tsources do: [:tuple|\r\n\t\ttuple first = lang ifTrue: [ ^tuple last ].\r\n\t\ttuple first isNil ifTrue: [ default := tuple last ].\r\n\t].\r\n\t^default notNil ifTrue: [ default ]","(\r\nfunction(self,lang)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal default=nil\r\n(self['@sources']):do_((function(tuple)\r\n(function() if ((((smalltalk:isObject_equalTo_(((tuple):first()),lang))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((tuple):last())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function() if (((((((tuple):first()))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((tuple):last())\r\n\tdefault=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function() if ((((((default))~=nil)))==true) then return ((function()\r\nreturn (default)\r\nend))() else return nil end end)())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.InlineNode,"accept:",0
,(
function(self,aVisitor)
(aVisitor):visitInlineNode_(self)
;
return (self)
;
end
)
,"visiting","accept: aVisitor\r\n\taVisitor visitInlineNode: self","(\r\nfunction(self,aVisitor)\r\n(aVisitor):visitInlineNode_(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.InlineNode,"isInline",0
,(
function(self)
return (true)
;
end
)
,"testing","isInline\r\n\t\" Return true if the receiver is of this type. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DirectiveNode,"description",0
,(
function(self)
return (((self)["$klass"]):name())
;
end
)
,"visiting","description\r\n\t\" Return the description of the receiver. \"\r\n\r\n\t^self class name","(\r\nfunction(self)\r\nreturn (((self)[\"$klass\"]):name())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DirectiveNode,"selector",0
,(
function(self)
return (("directive:arguments:"):asSelector())
;
end
)
,"visiting","selector\r\n\t\" Private - Return the internal selector used to execute directive. \"\r\n\r\n\t^#directive:arguments: asSelector","(\r\nfunction(self)\r\nreturn ((\"directive:arguments:\"):asSelector())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DirectiveNode,"literal",0
,(
function(self)
return ((self):implementedBySubclass())
;
end
)
,"visiting","literal\r\n\t\" Private - Return the literal to embedd in directive call. \"\r\n\r\n\t^self implementedBySubclass","(\r\nfunction(self)\r\nreturn ((self):implementedBySubclass())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DirectiveNode,"asJavascript:for:on:",0
,(
function(self,selector,anEncoder,aStream)
return ((self):tryExpressionDoing_((self):asJavascript_(selector)))
;
end
)
,"code generation","asJavascript: selector for: anEncoder on: aStream\r\n\t\" OBSOLETE - Return the native code to emmit the receiver. \"\r\n\r\n\t^self tryExpressionDoing: (self asJavascript: selector)","(\r\nfunction(self,selector,anEncoder,aStream)\r\nreturn ((self):tryExpressionDoing_((self):asJavascript_(selector)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DirectiveNode,"asNative:for:on:",0
,(
function(self,selector,anEncoder,aStream)
return ((self):tryExpressionDoing_((self):asJavascript_(selector)))
;
end
)
,"code generation","asNative: selector for: anEncoder on: aStream\r\n\t\" Return the native code to emmit the receiver. \"\r\n\r\n\t^self tryExpressionDoing: (self asJavascript: selector)","(\r\nfunction(self,selector,anEncoder,aStream)\r\nreturn ((self):tryExpressionDoing_((self):asJavascript_(selector)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DirectiveNode,"tryExpressionDoing:",0
,(
function(self,sourceCode)
return (smalltalk:object_comma_((smalltalk:object_comma_(("try{"),sourceCode)),"} catch($$e) {\r\n\tif ($$e.directiveError) { return(this.error_($$e.directiveError)); }\r\n\tif (!$$e.directiveFailed) { throw($$e); } }"))
;
end
)
,"code generation","tryExpressionDoing: sourceCode\r\n\t\" Private - Return the try/catch code wrapping sourceCode. \"\r\n\r\n\t^'try{' ,sourceCode ,'} catch($$e) {\r\n\tif ($$e.directiveError) { return(this.error_($$e.directiveError)); }\r\n\tif (!$$e.directiveFailed) { throw($$e); } }'","(\r\nfunction(self,sourceCode)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"try{\"),sourceCode)),\"} catch($$e) {\\r\\n\\tif ($$e.directiveError) { return(this.error_($$e.directiveError)); }\\r\\n\\tif (!$$e.directiveFailed) { throw($$e); } }\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.DirectiveNode,"asJavascript:",0
,(
function(self,selector)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("return this."),(self):selector())),"(")),(self):literal())),",")),"arguments")),");"))
;
end
)
,"code generation","asJavascript: selector\r\n\t\" Private - Return the native code to emmit the receiver. \"\r\n\r\n\t^'return this.',self selector,'(',self literal,',','arguments',');'","(\r\nfunction(self,selector)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"return this.\"),(self):selector())),\"(\")),(self):literal())),\",\")),\"arguments\")),\");\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSDirectiveNode["$klass"],"code:protected:",0
,(
function(self,aCode,useTry)
return ((
function(_s8_rec)
(_s8_rec):code_(aCode);
(_s8_rec):protected_(useTry);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","code: aCode protected: useTry\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tcode: aCode;\r\n\t\tprotected: useTry;\r\n\t\tyourself","(\r\nfunction(self,aCode,useTry)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):code_(aCode);\r\n(_s8_rec):protected_(useTry);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSDirectiveNode,"code:",0
,(
function(self,aCode)
(function()
	local _s8_value=(aCode)
	self['@code']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","code: aCode\r\n\t\" Set the code of the receiver. \"\r\n\r\n\tcode := aCode","(\r\nfunction(self,aCode)\r\n(function()\r\n\tlocal _s8_value=(aCode)\r\n\tself['@code']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSDirectiveNode,"jsCodeFor:",0
,(
function(self,anEncoder)
return ((function() if ((((self['@code']):isString()))==true) then return ((function()
return (self['@code'])
end))() else return ((function()
return ((self['@code']):evaluateWith_(anEncoder))
end))() end end)())
;
end
)
,"accessing","jsCodeFor: anEncoder\r\n\t\" Private - Return the native code of the receiver. \"\r\n\r\n\t^code isString\r\n\t\tifTrue: [ code ]\r\n\t\tifFalse: [ code evaluateWith: anEncoder ]","(\r\nfunction(self,anEncoder)\r\nreturn ((function() if ((((self['@code']):isString()))==true) then return ((function()\r\nreturn (self['@code'])\r\nend))() else return ((function()\r\nreturn ((self['@code']):evaluateWith_(anEncoder))\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSDirectiveNode,"protected:",0
,(
function(self,aBoolean)
(function()
	local _s8_value=(aBoolean)
	self['@protected']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","protected: aBoolean\r\n\t\" Set the receiver to use try/catch to handle failure. \"\r\n\r\n\tprotected := aBoolean","(\r\nfunction(self,aBoolean)\r\n(function()\r\n\tlocal _s8_value=(aBoolean)\r\n\tself['@protected']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JSDirectiveNode,"asNative:for:on:",0
,(
function(self,selector,anEncoder,aStream)
local jsCode=nil
(function()
	local _s8_value=((self):jsCodeFor_(anEncoder))
	jsCode=_s8_value; return _s8_value
end)()
;
return ((function() if (((((self['@protected'])==true)))==true) then return ((function()
return ((self):tryExpressionDoing_(jsCode))
end))() else return ((function()
return (jsCode)
end))() end end)())
;
end
)
,"code generation","asNative: selector for: anEncoder on: aStream\r\n\t\" Return the native code to emmit the receiver. \"\r\n\r\n\t| jsCode |\r\n\tjsCode := self jsCodeFor: anEncoder.\r\n\t^protected == true\r\n\t\tifTrue: [ self tryExpressionDoing: jsCode ]\r\n\t\tifFalse: [ jsCode ]","(\r\nfunction(self,selector,anEncoder,aStream)\r\nlocal jsCode=nil\r\n(function()\r\n\tlocal _s8_value=((self):jsCodeFor_(anEncoder))\r\n\tjsCode=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((function() if (((((self['@protected'])==true)))==true) then return ((function()\r\nreturn ((self):tryExpressionDoing_(jsCode))\r\nend))() else return ((function()\r\nreturn (jsCode)\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.InlineOptimizationNode,"args",0
,(
function(self)
return (self['@args'])
;
end
)
,"accessing","args\r\n\t\" Return the args of the receiver. \"\r\n\r\n\t^args","(\r\nfunction(self)\r\nreturn (self['@args'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.InlineOptimizationNode,"args:",0
,(
function(self,anObject)
(function()
	local _s8_value=(anObject)
	self['@args']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","args: anObject\r\n\t\" Set the args of the receiver. \"\r\n\r\n\targs := anObject","(\r\nfunction(self,anObject)\r\n(function()\r\n\tlocal _s8_value=(anObject)\r\n\tself['@args']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.InlineOptimizationNode,"remove",0
,(
function(self)
return (self['@remove'])
;
end
)
,"accessing","remove\r\n\t\" Return the remove of the receiver. \"\r\n\r\n\t^remove","(\r\nfunction(self)\r\nreturn (self['@remove'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.InlineOptimizationNode,"remove:",0
,(
function(self,anObject)
(function()
	local _s8_value=(anObject)
	self['@remove']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","remove: anObject\r\n\t\" Set the remove of the receiver. \"\r\n\r\n\tremove := anObject","(\r\nfunction(self,anObject)\r\n(function()\r\n\tlocal _s8_value=(anObject)\r\n\tself['@remove']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.InlineOptimizationNode,"asNative:for:on:",0
,(
function(self,selector,anEncoder,aStream)
(function() if ((((self):remove()))==true) then return ((function()
return ((anEncoder):removeOptimizations_(self['@args']))
end))() else return ((function()
return ((anEncoder):addOptimizations_(self['@args']))
end))() end end)()
;
return ("")
;
end
)
,"code generation","asNative: selector for: anEncoder on: aStream\r\n\t\" Return the native code to emmit the receiver. \"\r\n\r\n\tself remove\r\n\t\tifTrue: [ anEncoder removeOptimizations: args ]\r\n\t\tifFalse: [ anEncoder addOptimizations: args ].\r\n\t^''","(\r\nfunction(self,selector,anEncoder,aStream)\r\n(function() if ((((self):remove()))==true) then return ((function()\r\nreturn ((anEncoder):removeOptimizations_(self['@args']))\r\nend))() else return ((function()\r\nreturn ((anEncoder):addOptimizations_(self['@args']))\r\nend))() end end)()\r\n;\r\nreturn (\"\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPFailure["$klass"],"reason:at:",0
,(
function(self,aString,anInteger)
return (((self):new()):reason_at_(aString, anInteger))
;
end
)
,"instantiation","reason: aString at: anInteger\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new reason: aString at: anInteger","(\r\nfunction(self,aString,anInteger)\r\nreturn (((self):new()):reason_at_(aString, anInteger))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPFailure,"position",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((self['@position']))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((0))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self['@position'])
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","position\r\n\t\" Return the position of the receiver. \"\r\n\r\n\tposition isNil ifTrue: [ ^0 ].\r\n\t^position","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((self['@position']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((0))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self['@position'])\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPFailure,"position:",0
,(
function(self,aNumber)
(function()
	local _s8_value=(aNumber)
	self['@position']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","position: aNumber\r\n\t\" Set the position of the receiver. \"\r\n\r\n\tposition := aNumber","(\r\nfunction(self,aNumber)\r\n(function()\r\n\tlocal _s8_value=(aNumber)\r\n\tself['@position']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPFailure,"toString",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("Failure "),smalltalk:printString_(((self):reason())))),"(@")),(self):position())),")"))
;
end
)
,"printing","toString\r\n\t\" Return the receiver converted to String. \"\r\n\r\n\t^'Failure ',self reason printString ,'(@',self position,')'","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"Failure \"),smalltalk:printString_(((self):reason())))),\"(@\")),(self):position())),\")\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPFailure,"reason",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((self['@reason']))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.String):new())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self['@reason'])
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","reason\r\n\t\" Return the reason of the receiver. \"\r\n\r\n\treason isNil ifTrue: [ ^String new ].\r\n\t^reason","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((self['@reason']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.String):new())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self['@reason'])\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPFailure,"reason:",0
,(
function(self,aString)
(function()
	local _s8_value=(smalltalk:toString_((aString)))
	self['@reason']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","reason: aString\r\n\t\" Set the reason of the receiver. \"\r\n\r\n\treason := aString toString","(\r\nfunction(self,aString)\r\n(function()\r\n\tlocal _s8_value=(smalltalk:toString_((aString)))\r\n\tself['@reason']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPFailure,"reason:at:",0
,(
function(self,aString,anInteger)
(
function(_s8_rec)
(_s8_rec):reason_(aString);
return ((_s8_rec):position_(anInteger))
end
)(self)
;
return (self)
;
end
)
,"accessing","reason: aString at: anInteger\r\n\t\" Set the reason and position of the receiver. \"\r\n\r\n\tself\r\n\t\treason: aString; \r\n\t\tposition: anInteger","(\r\nfunction(self,aString,anInteger)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):reason_(aString);\r\nreturn ((_s8_rec):position_(anInteger))\r\nend\r\n)(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPFailure,"isParseFailure",0
,(
function(self)
return (true)
;
end
)
,"testing","isParseFailure\r\n\t\" Return true if the receiver is a parse failure. \"\r\n\r\n\t^true","(\r\nfunction(self)\r\nreturn (true)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPFailure,"accept:",0
,(
function(self,anEncoder)
local description=nil
(function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("PPFailure: "),(self):reason()))," at: ")),smalltalk:asString_(((self):position()))))," source: ")),((anEncoder):stream()):contents()))
	description=_s8_value; return _s8_value
end)()
;
(self):print_(description)
;
(self):error_(description)
;
return (self)
;
end
)
,"bug","accept: anEncoder\r\n\t\" Report compilation failure and error. \"\r\n\r\n\t| description |\r\n\tdescription := 'PPFailure: ', self reason\r\n\t\t,' at: ' ,self position asString\r\n\t\t,' source: ' ,anEncoder stream contents.\r\n\tself print: description.\r\n\tself error: description","(\r\nfunction(self,anEncoder)\r\nlocal description=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"PPFailure: \"),(self):reason())),\" at: \")),smalltalk:asString_(((self):position())))),\" source: \")),((anEncoder):stream()):contents()))\r\n\tdescription=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):print_(description)\r\n;\r\n(self):error_(description)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPFailure,"__tostring",0
,(
function(self)
return (smalltalk:toString_((self)))
;
end
)
,"Lua-printing","__tostring\r\n\t\" Return the printable representation of the receiver. \"\r\n\r\n\t^self toString","(\r\nfunction(self)\r\nreturn (smalltalk:toString_((self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser["$klass"],",","_comma"
,(
function(self,anObject)
return ((anObject):asParser())
;
end
)
,"instantiation",", anObject\r\n\t\" Return the receiver composed with the argument. \"\r\n\r\n\t^anObject asParser","(\r\nfunction(self,anObject)\r\nreturn ((anObject):asParser())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser["$klass"],"optional:",0
,(
function(self,anObject)
return ((smalltalk:object_comma_((self),anObject)):optional())
;
end
)
,"instantiation","optional: anObject\r\n\t\" Return an instance of the receiver to parse an optional element anObject. \"\r\n\r\n\t^(self ,anObject) optional","(\r\nfunction(self,anObject)\r\nreturn ((smalltalk:object_comma_((self),anObject)):optional())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser["$klass"],"debug:or:",0
,(
function(self,action,aBlock)
local show=nil
local result=nil
(function()
	local _s8_value=((function() local _1_=((smalltalk.PPParser)["debugOn"]);if (_1_==nil) then return ((function()
(function()
	local _s8_value=((aBlock):evaluate())
	result=_s8_value; return _s8_value
end)();return (false)
end)):value() end; return _1_ end)())
	show=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_((show),true))))==true) then return ((function()
return ((function()
	local _s8_value=((action):evaluate())
	result=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (result)
;
end
)
,"debug","debug: action or: aBlock\r\n\t\" Log evaluation of action if in debugging mode,\r\n\t else returns the result of evaluation of aBlock.\r\n\t\"\r\n\r\n\t| show result |\r\n\tshow := PPParser basicAt: #debugOn ifAbsent:[ result := aBlock evaluate. false ].\r\n\tshow = true ifTrue: [ result := action evaluate ].\r\n\t^result","(\r\nfunction(self,action,aBlock)\r\nlocal show=nil\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((function() local _1_=((smalltalk.PPParser)[\"debugOn\"]);if (_1_==nil) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((aBlock):evaluate())\r\n\tresult=_s8_value; return _s8_value\r\nend)();return (false)\r\nend)):value() end; return _1_ end)())\r\n\tshow=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((show),true))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((action):evaluate())\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser["$klass"],"debugWhile:",0
,(
function(self,aBlock)
return (((function()
(function() local _1_=(true); (smalltalk.PPParser)["debugOn"]=_1_; return _1_ end)();return ((aBlock):evaluate())
end)):ensure_((function()
return ((function() local _1_=(false); (smalltalk.PPParser)["debugOn"]=_1_; return _1_ end)())
end)))
;
end
)
,"debug","debugWhile: aBlock\r\n\t^[\tPPParser basicAt: #debugOn put: true.\r\n\t\taBlock evaluate\r\n\t] ensure: [\r\n\t\tPPParser basicAt: #debugOn put: false\r\n\t]","(\r\nfunction(self,aBlock)\r\nreturn (((function()\r\n(function() local _1_=(true); (smalltalk.PPParser)[\"debugOn\"]=_1_; return _1_ end)();return ((aBlock):evaluate())\r\nend)):ensure_((function()\r\nreturn ((function() local _1_=(false); (smalltalk.PPParser)[\"debugOn\"]=_1_; return _1_ end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"set:start:end:",0
,(
function(self,anElement,start,__end)
(function() if (((smalltalk:responds_to_((anElement),("start:end:"))))==true) then return ((function()
return ((anElement):start_end_(start, __end))
end))() else return nil end end)()
;
return (self)
;
end
)
,"private","set: anElement start: start end: end\r\n\t\" Private - Set start&end source offest of anElement. \"\r\n\r\n\t(anElement respondsTo: #start:end:) ifTrue: [\r\n\t\tanElement start: start end: end\r\n\t]","(\r\nfunction(self,anElement,start,__end)\r\n(function() if (((smalltalk:responds_to_((anElement),(\"start:end:\"))))==true) then return ((function()\r\nreturn ((anElement):start_end_(start, __end))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"flatten",0
,(
function(self)
return ((smalltalk.PPFlattenParser):on_(self))
;
end
)
,"operations","flatten\r\n\t\" Return a flattened version of the receiver. \"\r\n\r\n\t^PPFlattenParser on: self","(\r\nfunction(self)\r\nreturn ((smalltalk.PPFlattenParser):on_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"withSource",0
,(
function(self)
return ((smalltalk.PPSourceParser):on_(self))
;
end
)
,"operations","withSource\r\n\t\" Return a source parser on the receiver. \"\r\n\r\n\t^PPSourceParser on: self","(\r\nfunction(self)\r\nreturn ((smalltalk.PPSourceParser):on_(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"==>","_eq_eq_gt"
,(
function(self,aBlock)
return ((smalltalk.PPActionParser):on_block_(self, aBlock))
;
end
)
,"operations","==> aBlock\r\n\t\" Return an action parser on the receiver. \"\r\n\r\n\t^PPActionParser on: self block: aBlock","(\r\nfunction(self,aBlock)\r\nreturn ((smalltalk.PPActionParser):on_block_(self, aBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,",","_comma"
,(
function(self,aParser)
return ((smalltalk.PPSequenceParser):with_with_(self, aParser))
;
end
)
,"operations",", aParser\r\n\t\" Return the receiver composed with the argument. \"\r\n\r\n\t^PPSequenceParser with: self with: aParser","(\r\nfunction(self,aParser)\r\nreturn ((smalltalk.PPSequenceParser):with_with_(self, aParser))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"/","_slash"
,(
function(self,aParser)
return ((smalltalk.PPChoiceParser):with_with_(self, aParser))
;
end
)
,"operations","/ aParser\r\n\t\" Return a choice composite version of the receiver and aParser. \"\r\n\r\n\t^PPChoiceParser with: self with: aParser","(\r\nfunction(self,aParser)\r\nreturn ((smalltalk.PPChoiceParser):with_with_(self, aParser))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"@","_at"
,(
function(self,tag)
(function() local _1_=(tag); (self)["debugTag"]=_1_; return _1_ end)()
;
return ((self):_eq_eq_gt((function(node)
(self):print_(smalltalk:object_comma_((smalltalk:object_comma_((tag),"==>")),smalltalk:printString_((node))));return (node)
end)))
;
end
)
,"debug","@ tag\r\n\t\" DEBUG - Return a notification wrapper of the receiver. \"\r\n\r\n\tself basicAt: #debugTag put: tag.\r\n\t^self ==> [:node| self print: tag ,'==>',node printString. node ]","(\r\nfunction(self,tag)\r\n(function() local _1_=(tag); (self)[\"debugTag\"]=_1_; return _1_ end)()\r\n;\r\nreturn ((self):_eq_eq_gt((function(node)\r\n(self):print_(smalltalk:object_comma_((smalltalk:object_comma_((tag),\"==>\")),smalltalk:printString_((node))));return (node)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"@@","_at_at"
,(
function(self,tag)
(function() local _1_=(tag); (self)["debugTag"]=_1_; return _1_ end)()
;
return (self)
;
end
)
,"debug","@@ tag\r\n\t\" DEBUG - Tag the receiver for debugging. \"\r\n\r\n\tself basicAt: #debugTag put: tag","(\r\nfunction(self,tag)\r\n(function() local _1_=(tag); (self)[\"debugTag\"]=_1_; return _1_ end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"plus",0
,(
function(self)
return (((smalltalk.PPRepeatingParser):on_min_(self, (1))):_at_at("plus"))
;
end
)
,"operations","plus\r\n\t\" Return an iterative version of the receiver. \"\r\n\r\n\t^(PPRepeatingParser on: self min: 1) @@#plus","(\r\nfunction(self)\r\nreturn (((smalltalk.PPRepeatingParser):on_min_(self, (1))):_at_at(\"plus\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"star",0
,(
function(self)
return (((smalltalk.PPRepeatingParser):on_min_(self, (0))):_at_at("star"))
;
end
)
,"operations","star\r\n\t\" Return an iterative version of the receiver. \"\r\n\r\n\t^(PPRepeatingParser on: self min: 0) @@#star","(\r\nfunction(self)\r\nreturn (((smalltalk.PPRepeatingParser):on_min_(self, (0))):_at_at(\"star\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"not","__not"
,(
function(self)
return (((smalltalk.PPNotParser):on_(self)):_at_at("not"))
;
end
)
,"operations","not\r\n\t\" Return the negation of the receiver. \"\r\n\r\n\t^(PPNotParser on: self) @@#not","(\r\nfunction(self)\r\nreturn (((smalltalk.PPNotParser):on_(self)):_at_at(\"not\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"optional",0
,(
function(self)
return (((self):_slash((smalltalk.PPEpsilonParser):new())):_at_at("opt"))
;
end
)
,"operations","optional\r\n\t\" Return an optional version of the receiver. \"\r\n\r\n\t^self / PPEpsilonParser new @@#opt","(\r\nfunction(self)\r\nreturn (((self):_slash((smalltalk.PPEpsilonParser):new())):_at_at(\"opt\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"isParseFailure:",0
,(
function(self,what)
return ((what):isParseFailure())
;
end
)
,"test","isParseFailure: what\r\n\t\" Private - Return true if what is a parsing failure. \"\r\n\r\n\t^what isParseFailure","(\r\nfunction(self,what)\r\nreturn ((what):isParseFailure())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"valueToMemoize:start:",0
,(
function(self,aStream,start)
local __end=nil
local node=nil
(function()
	local _s8_value=((self):parse_(aStream))
	node=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((aStream):position())
	__end=_s8_value; return _s8_value
end)()
;
(self):set_start_end_(node, start, __end)
;
return ((smalltalk.Array):with_with_(node, __end))
;
end
)
,"memo","valueToMemoize: aStream start: start\r\n\t\" Private - Return the value to memoize as result of parsing en element in aStream. \"\r\n\r\n\t| end node |\r\n\tnode := self parse: aStream.\r\n\tend := aStream position.\r\n\tself set: node start: start end: end.\r\n\t^Array with: node with: end","(\r\nfunction(self,aStream,start)\r\nlocal __end=nil\r\nlocal node=nil\r\n(function()\r\n\tlocal _s8_value=((self):parse_(aStream))\r\n\tnode=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aStream):position())\r\n\t__end=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):set_start_end_(node, start, __end)\r\n;\r\nreturn ((smalltalk.Array):with_with_(node, __end))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"memoizedParse:",0
,(
function(self,aStream)
local start=nil
local value=nil
(function() if ((((((self['@memo']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.PoolDictionary):new())
	self['@memo']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((aStream):position())
	start=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self['@memo']):at_ifAbsentPut_(start, (function()
return ((self):valueToMemoize_start_(aStream, start))
end)))
	value=_s8_value; return _s8_value
end)()
;
(aStream):position_((value):second())
;
return ((value):first())
;
end
)
,"memo","memoizedParse: aStream\r\n\t\" Private - Evaluate the receiver. \"\r\n\r\n\t| start value |\r\n\tmemo isNil ifTrue: [ memo := PoolDictionary new ].\r\n\tstart := aStream position.\r\n\tvalue := memo at: start ifAbsentPut: [\r\n\t\tself valueToMemoize: aStream start: start\r\n\t].\r\n\taStream position: value second.\r\n\t^value first","(\r\nfunction(self,aStream)\r\nlocal start=nil\r\nlocal value=nil\r\n(function() if ((((((self['@memo']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.PoolDictionary):new())\r\n\tself['@memo']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aStream):position())\r\n\tstart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self['@memo']):at_ifAbsentPut_(start, (function()\r\nreturn ((self):valueToMemoize_start_(aStream, start))\r\nend)))\r\n\tvalue=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aStream):position_((value):second())\r\n;\r\nreturn ((value):first())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"basicParse:",0
,(
function(self,aStream)
(self):subclassResponsibility_("basicParse:")
;
return (self)
;
end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\tself subclassResponsibility: #basicParse:","(\r\nfunction(self,aStream)\r\n(self):subclassResponsibility_(\"basicParse:\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"parseOld:",0
,(
function(self,aStream)
return ((self):basicParse_(aStream))
;
end
)
,"parsing","parseOld: aStream\r\n\t\" Parse contents in aStream. \"\r\n\r\n\t^self basicParse: aStream","(\r\nfunction(self,aStream)\r\nreturn ((self):basicParse_(aStream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"parseLog:",0
,(
function(self,aStream)
local result=nil
local old=nil
(function()
	local _s8_value=((aStream):position())
	old=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):basicParse_(aStream))
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((self):isParseFailure_(result)))==true) then return nil else return ((function()
return ((self):note_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("["),old)),"-")),(aStream):position())),"] ")),smalltalk:printString_((self)))))
end))() end end)()
;
return (result)
;
end
)
,"parsing","parseLog: aStream\r\n\t\" Parse contents in aStream. \"\r\n\r\n\t| result old |\r\n\told := aStream position.\r\n\tresult := self basicParse: aStream.\r\n\t(self isParseFailure: result) ifFalse: [\r\n\t\tself note: '[',old,'-',aStream position,'] ',self printString\r\n\t].\r\n\t^result","(\r\nfunction(self,aStream)\r\nlocal result=nil\r\nlocal old=nil\r\n(function()\r\n\tlocal _s8_value=((aStream):position())\r\n\told=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):basicParse_(aStream))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((self):isParseFailure_(result)))==true) then return nil else return ((function()\r\nreturn ((self):note_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"[\"),old)),\"-\")),(aStream):position())),\"] \")),smalltalk:printString_((self)))))\r\nend))() end end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"parse:",0
,(
function(self,aStream)
return ((smalltalk.PPParser):debug_or_((function()
return ((self):parseLog_(aStream))
end), (function()
return ((self):basicParse_(aStream))
end)))
;
end
)
,"parsing","parse: aStream\r\n\t\" Parse contents in aStream. \"\r\n\r\n\t^PPParser\r\n\t\tdebug: [self parseLog: aStream]\r\n\t\tor: [self basicParse: aStream]","(\r\nfunction(self,aStream)\r\nreturn ((smalltalk.PPParser):debug_or_((function()\r\nreturn ((self):parseLog_(aStream))\r\nend), (function()\r\nreturn ((self):basicParse_(aStream))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"parseAll:",0
,(
function(self,aStream)
local result=nil
(function()
	local _s8_value=(((smalltalk.PPSequenceParser):with_with_(self, (smalltalk.PPEOFParser):new())):memoizedParse_(aStream))
	result=_s8_value; return _s8_value
end)()
;
return ((function() if ((((self):isParseFailure_(result)))==true) then return ((function()
return ((self):error_((result):messageFor_((aStream):contents())))
end))() else return ((function()
return ((result):first())
end))() end end)())
;
end
)
,"parsing","parseAll: aStream\r\n\t\" Parse contents in aStream. \"\r\n\r\n\t| result |\r\n\tresult := (PPSequenceParser\r\n\t\twith: self with: PPEOFParser new)\r\n\t\tmemoizedParse: aStream.\r\n\t^(self isParseFailure: result)\r\n\t    ifTrue: [ self error: (result messageFor: aStream contents) ]\r\n\t    ifFalse: [ result first ]","(\r\nfunction(self,aStream)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.PPSequenceParser):with_with_(self, (smalltalk.PPEOFParser):new())):memoizedParse_(aStream))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((function() if ((((self):isParseFailure_(result)))==true) then return ((function()\r\nreturn ((self):error_((result):messageFor_((aStream):contents())))\r\nend))() else return ((function()\r\nreturn ((result):first())\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"asParser",0
,(
function(self)
return (self)
;
end
)
,"converting","asParser\r\n\t\" Return a parser on the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"printStringDetail",0
,(
function(self)
return ("")
;
end
)
,"printing","printStringDetail\r\n\t\" Private - Return a printable detail of the receiver. \"\r\n\r\n\t^''","(\r\nfunction(self)\r\nreturn (\"\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPParser,"printString",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local tag=nil
(function()
	local _s8_value=(((self)["debugTag"]))
	tag=_s8_value; return _s8_value
end)()
;
(function() if ((((((tag))~=nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("@@"),tag)),"@@"))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("a"),((self)["$klass"]):name())),(function() if ((((((tag))==nil)))==true) then return ((function()
return ((self):printStringDetail())
end))() else return ((function()
return (smalltalk:object_comma_(("@@"),tag))
end))() end end)()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"printing","printString\r\n\t\" Return a printable representation of the receiver. \"\r\n\r\n\t| tag |\r\n\ttag := self basicAt: #debugTag.\r\n\ttag notNil ifTrue: [ ^#@@ ,tag ,#@@ ]. \"compact notation\"\r\n\r\n\t^#a ,self class name ,(tag isNil\r\n\t\tifTrue: [ self printStringDetail ]\r\n\t\tifFalse: [ '@@',tag ])","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal tag=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"debugTag\"]))\r\n\ttag=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((tag))~=nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"@@\"),tag)),\"@@\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"a\"),((self)[\"$klass\"]):name())),(function() if ((((((tag))==nil)))==true) then return ((function()\r\nreturn ((self):printStringDetail())\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((\"@@\"),tag))\r\nend))() end end)()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPEOFParser,"basicParse:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aStream):atEnd()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.PPFailure):reason_at_("EOF expected (or invalid character in input)", (aStream):position()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\taStream atEnd ifTrue: [ ^nil ] .\r\n\t^PPFailure reason: 'EOF expected (or invalid character in input)' at: aStream position","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aStream):atEnd()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.PPFailure):reason_at_(\"EOF expected (or invalid character in input)\", (aStream):position()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPAnyParser,"basicParse:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aStream):atEnd()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aStream):next())
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.PPFailure):reason_at_("End of stream", (aStream):position()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\taStream atEnd ifFalse: [ ^aStream next ].\r\n\t^PPFailure reason: 'End of stream' at: aStream position","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aStream):atEnd()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aStream):next())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.PPFailure):reason_at_(\"End of stream\", (aStream):position()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPEpsilonParser,"basicParse:",0
,(
function(self,aStream)
return (nil)
;
end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t^nil","(\r\nfunction(self,aStream)\r\nreturn (nil)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPStringParser,"string",0
,(
function(self)
return (self['@string'])
;
end
)
,"accessing","string\r\n\t\" Return the string of the receiver. \"\r\n\r\n\t^string","(\r\nfunction(self)\r\nreturn (self['@string'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPStringParser,"string:",0
,(
function(self,aString)
(function()
	local _s8_value=(aString)
	self['@string']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","string: aString\r\n\t\" Set the string of the receiver. \"\r\n\r\n\tstring := aString","(\r\nfunction(self,aString)\r\n(function()\r\n\tlocal _s8_value=(aString)\r\n\tself['@string']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPStringParser,"basicParse:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local back=nil
local reason=nil
(function() if ((((self['@string']):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self['@string'])
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((aStream):position())
	back=_s8_value; return _s8_value
end)()
;
(function() if ((((aStream):peekForAll_(self['@string'])))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self['@string'])
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("Expecting "),smalltalk:printString_((self['@string']))))," at ")),smalltalk:printString_((back))))," read ")),smalltalk:printString_(((aStream):next_((self['@string']):size())))))
	reason=_s8_value; return _s8_value
end)()
;
(aStream):position_(back)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.PPFailure):reason_at_(reason, back))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t| back reason |\r\n\tstring isEmpty ifTrue: [ ^string ].\r\n\tback := aStream position.\r\n\t(aStream peekForAll: string) ifTrue: [ ^string ].\r\n\treason := 'Expecting ', string printString\r\n\t\t,' at ',back printString\r\n\t\t,' read ',(aStream next: string size) printString.\r\n\taStream position: back.\r\n\t^PPFailure reason: reason at: back","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal back=nil\r\nlocal reason=nil\r\n(function() if ((((self['@string']):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self['@string'])\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aStream):position())\r\n\tback=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((aStream):peekForAll_(self['@string'])))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self['@string'])\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"Expecting \"),smalltalk:printString_((self['@string'])))),\" at \")),smalltalk:printString_((back)))),\" read \")),smalltalk:printString_(((aStream):next_((self['@string']):size())))))\r\n\treason=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aStream):position_(back)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.PPFailure):reason_at_(reason, back))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPStringParser,"printStringDetail",0
,(
function(self)
return (smalltalk:printString_((self['@string'])))
;
end
)
,"printing","printStringDetail\r\n\t\" Private - Return a printable detail of the receiver. \"\r\n\r\n\t^string printString","(\r\nfunction(self)\r\nreturn (smalltalk:printString_((self['@string'])))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPStStringParser,"basicParse:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local position=nil
local result=nil
(function()
	local _s8_value=((aStream):position())
	position=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=("")
	result=_s8_value; return _s8_value
end)()
;
(function() while(not ((function()
(function()
	local _s8_value=(smalltalk:object_comma_((result),(aStream):upTo_("'")))
	result=_s8_value; return _s8_value
end)();return ((aStream):atEnd())
end))()) do (function()
(function() if ((((smalltalk:isObject_equalTo_(((aStream):peek()),"'"))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)();return ((function()
	local _s8_value=(smalltalk:object_comma_((result),(aStream):next()))
	result=_s8_value; return _s8_value
end)())
end)() end;return nil end)()
;
(function() if ((((((aStream):position()) > position)))==true) then return ((function()
(aStream):position_(((aStream):position()):_minus((1)));return ((function() if ((((smalltalk:isObject_equalTo_(((aStream):next()),"'"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() else return nil end end)()
;
(aStream):position_(position)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.PPFailure):reason_at_("Unfinished String", position))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t| position result |\r\n\tposition := aStream position.\r\n\tresult := ''.\r\n\t[ result := result , (aStream upTo: $').\r\n\t  aStream atEnd ] whileFalse: [\r\n\t\taStream peek = $' ifFalse: [ ^result ].\r\n\t\tresult := result ,aStream next.\r\n\t].\r\n\taStream position > position ifTrue: [\r\n\t\taStream position: aStream position - 1.\r\n\t\taStream next = $' ifTrue: [ ^result ].\r\n\t].\r\n\taStream position: position.\r\n\t^PPFailure reason: 'Unfinished String' at: position","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal position=nil\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((aStream):position())\r\n\tposition=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(\"\")\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(not ((function()\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((result),(aStream):upTo_(\"'\")))\r\n\tresult=_s8_value; return _s8_value\r\nend)();return ((aStream):atEnd())\r\nend))()) do (function()\r\n(function() if ((((smalltalk:isObject_equalTo_(((aStream):peek()),\"'\"))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)();return ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((result),(aStream):next()))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend)() end;return nil end)()\r\n;\r\n(function() if ((((((aStream):position()) > position)))==true) then return ((function()\r\n(aStream):position_(((aStream):position()):_minus((1)));return ((function() if ((((smalltalk:isObject_equalTo_(((aStream):next()),\"'\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return nil end end)()\r\n;\r\n(aStream):position_(position)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.PPFailure):reason_at_(\"Unfinished String\", position))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser["$klass"],"separator",0
,(
function(self)
return (((self):new()):separator())
;
end
)
,"constants","separator\r\n\t\" Return the parser for separator. \"\r\n\r\n\t^self new separator","(\r\nfunction(self)\r\nreturn (((self):new()):separator())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser["$klass"],"in:",0
,(
function(self,aCollection)
return ((self):for_doing_(smalltalk:asString_((aCollection)), (function(c)
return ((aCollection):includes_(c))
end)))
;
end
)
,"constants","in: aCollection\r\n\t\" Return the parser for input matching any element in aCollection. \"\r\n\r\n\t^self\tfor: aCollection asString\r\n\t\tdoing: [:c| aCollection includes: c ]","(\r\nfunction(self,aCollection)\r\nreturn ((self):for_doing_(smalltalk:asString_((aCollection)), (function(c)\r\nreturn ((aCollection):includes_(c))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser["$klass"],"notIn:",0
,(
function(self,aCollection)
return ((self):for_doing_(smalltalk:asString_((aCollection)), (function(c)
return ((smalltalk:booleanNot_(((aCollection):includes_(c)))))
end)))
;
end
)
,"constants","notIn: aCollection\r\n\t\" Return the parser for input matching any element in aCollection. \"\r\n\r\n\t^self\tfor: aCollection asString\r\n\t\tdoing: [:c| (aCollection includes: c) not ]","(\r\nfunction(self,aCollection)\r\nreturn ((self):for_doing_(smalltalk:asString_((aCollection)), (function(c)\r\nreturn ((smalltalk:booleanNot_(((aCollection):includes_(c)))))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser["$klass"],"between:and:",0
,(
function(self,start,__end)
return ((self):for_doing_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((""),start)),"-")),__end), (function(c)
return ((c):between_and_(start, __end))
end)))
;
end
)
,"constants","between: start and: end\r\n\t\" Return the parser for input between start and end. \"\r\n\r\n\t^self\tfor: '',start,'-',end\r\n\t\tdoing: [:c| c between: start and: end ]","(\r\nfunction(self,start,__end)\r\nreturn ((self):for_doing_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"\"),start)),\"-\")),__end), (function(c)\r\nreturn ((c):between_and_(start, __end))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser["$klass"],"for:doing:",0
,(
function(self,label,aBlock)
return (((self):new()):do_for_(aBlock, label))
;
end
)
,"constants","for: label doing: aBlock\r\n\t\" Return a parser that evaluates aBlock to match label. \"\r\n\r\n\t^self new do: aBlock for: label","(\r\nfunction(self,label,aBlock)\r\nreturn (((self):new()):do_for_(aBlock, label))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser["$klass"],"regExpCache",0
,(
function(self)
(function() if ((((((smalltalk.PPCharacterParser:classVariableAt_("RegExpCache")))==nil)))==true) then return ((function()
return (smalltalk.PPCharacterParser:classVariableAt_put_("RegExpCache",(smalltalk.PoolDictionary):new()))
end))() else return nil end end)()
;
return (smalltalk.PPCharacterParser:classVariableAt_("RegExpCache"))
;
end
)
,"constants","regExpCache\r\n\t\" Private - Return the regexp cache. \"\r\n\r\n\tRegExpCache isNil ifTrue: [\r\n\t\tRegExpCache := PoolDictionary new\r\n\t].\r\n\t^RegExpCache","(\r\nfunction(self)\r\n(function() if ((((((smalltalk.PPCharacterParser:classVariableAt_(\"RegExpCache\")))==nil)))==true) then return ((function()\r\nreturn (smalltalk.PPCharacterParser:classVariableAt_put_(\"RegExpCache\",(smalltalk.PoolDictionary):new()))\r\nend))() else return nil end end)()\r\n;\r\nreturn (smalltalk.PPCharacterParser:classVariableAt_(\"RegExpCache\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser,"do:for:",0
,(
function(self,aBlock,aString)
(function()
	local _s8_value=(aString)
	self['@theString']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(aBlock)
	self['@regexp']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","do: aBlock for: aString\r\n\t\" Set the receiver to match aString using aBlock. \"\r\n\r\n\ttheString := aString.\r\n\tregexp := aBlock","(\r\nfunction(self,aBlock,aString)\r\n(function()\r\n\tlocal _s8_value=(aString)\r\n\tself['@theString']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(aBlock)\r\n\tself['@regexp']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser,"string:",0
,(
function(self,aString)
(function()
	local _s8_value=(aString)
	self['@theString']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((self)["$klass"]):regExpCache()):at_ifAbsentPut_(aString, (function()
return ((smalltalk.RegularExpression):fromString_(smalltalk:object_comma_((smalltalk:object_comma_(("["),aString)),"]")))
end)))
	self['@regexp']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","string: aString\r\n\t\" Set the string of the receiver. \"\r\n\r\n\ttheString := aString.\r\n\tregexp := self class regExpCache at: aString ifAbsentPut: [\r\n\t\tRegularExpression fromString: '[', aString, ']'\r\n\t]","(\r\nfunction(self,aString)\r\n(function()\r\n\tlocal _s8_value=(aString)\r\n\tself['@theString']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((self)[\"$klass\"]):regExpCache()):at_ifAbsentPut_(aString, (function()\r\nreturn ((smalltalk.RegularExpression):fromString_(smalltalk:object_comma_((smalltalk:object_comma_((\"[\"),aString)),\"]\")))\r\nend)))\r\n\tself['@regexp']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser,"match:",0
,(
function(self,aString)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((aString))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((((self['@regexp']))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aString):isSeparator())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((self):typeOf_(self['@regexp'])),"function"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self['@regexp']):value_(aString))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((self['@theString']),aString))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aString):match_(self['@regexp']))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","match: aString\r\n\t\" Private - Match the receiver to string. \"\r\n\r\n\taString isNil ifTrue: [ ^false ].\r\n\tregexp isNil ifTrue: [ ^aString isSeparator ].\r\n\t(self typeOf: regexp) = #function ifTrue: [ ^regexp value: aString ].\r\n\ttheString = aString ifTrue: [ ^true ].\r\n\t^aString match: regexp","(\r\nfunction(self,aString)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((aString))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((self['@regexp']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aString):isSeparator())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):typeOf_(self['@regexp'])),\"function\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self['@regexp']):value_(aString))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((self['@theString']),aString))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aString):match_(self['@regexp']))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser,"matchFailureReason",0
,(
function(self)
return (smalltalk:object_comma_(("Could not match "),(function() if ((((((self['@theString']))==nil)))==true) then return ((function()
return ("separator")
end))() else return ((function()
return (smalltalk:printString_((self['@theString'])))
end))() end end)()))
;
end
)
,"accessing","matchFailureReason\r\n\t\" Private - Return the reason for mismatch. \"\r\n\r\n\t^'Could not match ' ,(theString isNil\r\n\t\tifTrue: [#separator]\r\n\t\tifFalse: [theString printString])","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((\"Could not match \"),(function() if ((((((self['@theString']))==nil)))==true) then return ((function()\r\nreturn (\"separator\")\r\nend))() else return ((function()\r\nreturn (smalltalk:printString_((self['@theString'])))\r\nend))() end end)()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser,"basicParse:",0
,(
function(self,aStream)
return ((function() if ((((self):match_((aStream):peek())))==true) then return ((function()
return ((aStream):next())
end))() else return ((function()
return ((smalltalk.PPFailure):reason_at_((self):matchFailureReason(), (aStream):position()))
end))() end end)())
;
end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t^(self match: aStream peek)\r\n\t ifTrue:  [ aStream next ]\r\n\t ifFalse: [ PPFailure\r\n\t\treason: self matchFailureReason\r\n\t\tat: aStream position\r\n\t]","(\r\nfunction(self,aStream)\r\nreturn ((function() if ((((self):match_((aStream):peek())))==true) then return ((function()\r\nreturn ((aStream):next())\r\nend))() else return ((function()\r\nreturn ((smalltalk.PPFailure):reason_at_((self):matchFailureReason(), (aStream):position()))\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser,"separator",0
,(
function(self)
(function()
	local _s8_value=((function()
	local _s8_value=(nil)
	self['@regexp']=_s8_value; return _s8_value
end)())
	self['@theString']=_s8_value; return _s8_value
end)()
;
(self):_at_at("separator")
;
return (self)
;
end
)
,"separator","separator\r\n\t\" Set the receiver to parse separator. \"\r\n\r\n\ttheString := regexp := nil.\r\n\tself @@#separator","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=((function()\r\n\tlocal _s8_value=(nil)\r\n\tself['@regexp']=_s8_value; return _s8_value\r\nend)())\r\n\tself['@theString']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):_at_at(\"separator\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPCharacterParser,"printStringDetail",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self['@theString']):isCharacter()))==true) then return ((function()
(function() if ((((self['@theString']):between_and_("0", "z")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_(("$"),self['@theString']))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_(("#"),(self['@theString']):asciiValue()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:printString_((self['@theString'])))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"printing","printStringDetail\r\n\t\" Private - Return a printable detail of the receiver. \"\r\n\r\n\ttheString isCharacter ifTrue: [\r\n\t\t(theString between: $0 and: $z) ifTrue: [\r\n\t\t\t^$$ ,theString\r\n\t\t].\r\n\t\t^$# ,theString asciiValue\r\n\t].\r\n\t^theString printString","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self['@theString']):isCharacter()))==true) then return ((function()\r\n(function() if ((((self['@theString']):between_and_(\"0\", \"z\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((\"$\"),self['@theString']))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((\"#\"),(self['@theString']):asciiValue()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:printString_((self['@theString'])))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPListParser["$klass"],"withAll:",0
,(
function(self,aCollection)
return ((
function(_s8_rec)
(_s8_rec):parsers_(aCollection);
return ((_s8_rec):yourself())
end
)((self):basicNew()))
;
end
)
,"instantiation","withAll: aCollection\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t    ^self basicNew\r\n\t\tparsers: aCollection;\r\n\t\tyourself","(\r\nfunction(self,aCollection)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):parsers_(aCollection);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):basicNew()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPListParser["$klass"],"with:with:",0
,(
function(self,aParser,anotherParser)
return ((self):withAll_((smalltalk.Array):with_with_(aParser, anotherParser)))
;
end
)
,"instantiation","with: aParser with: anotherParser\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self withAll: (Array with: aParser with: anotherParser)","(\r\nfunction(self,aParser,anotherParser)\r\nreturn ((self):withAll_((smalltalk.Array):with_with_(aParser, anotherParser)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPListParser,"parsers",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((self['@parsers']))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:newArray({}))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self['@parsers'])
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","parsers\r\n\t\" Return the parsers of the receiver. \"\r\n\r\n\tparsers isNil ifTrue: [ ^#() ].\r\n\t^parsers","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((self['@parsers']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:newArray({}))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self['@parsers'])\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPListParser,"parsers:",0
,(
function(self,aCollection)
(function()
	local _s8_value=((aCollection):collect_((function(each)
return ((each):asParser())
end)))
	self['@parsers']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","parsers: aCollection\r\n\t\" Set the parsers of the receiver. \"\r\n\r\n\tparsers := aCollection collect: [:each| each asParser ]","(\r\nfunction(self,aCollection)\r\n(function()\r\n\tlocal _s8_value=((aCollection):collect_((function(each)\r\nreturn ((each):asParser())\r\nend)))\r\n\tself['@parsers']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPListParser,"copyWith:",0
,(
function(self,aParser)
return (((self)["$klass"]):withAll_(((self):parsers()):copyWith_(aParser)))
;
end
)
,"copying","copyWith: aParser\r\n\t\" Return a copy of the receiver with aParser. \"\r\n\r\n\t^self class withAll: (self parsers copyWith: aParser)","(\r\nfunction(self,aParser)\r\nreturn (((self)[\"$klass\"]):withAll_(((self):parsers()):copyWith_(aParser)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPListParser,"op",0
,(
function(self)
return ("+")
;
end
)
,"printing","op\r\n\t\" Private - Return the operator of the receiver. \"\r\n\r\n\t^$+","(\r\nfunction(self)\r\nreturn (\"+\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPListParser,"printString",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local stream=nil
(function() if ((((((((self)["debugTag"])))~=nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:superImplementor(smalltalk.PPListParser,"printString")(self))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=(("{"):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
((self):parsers()):do_separatedBy_((function(each)
return ((stream):nextPutAll_(smalltalk:printString_((each))))
end), (function()
return ((stream):nextPutAll_((self):op()))
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((stream):contents())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"printing","printString\r\n\t\" Return a printable representation of the receiver. \"\r\n\r\n\t| stream |\r\n\t(self basicAt: #debugTag) notNil ifTrue: [ ^super printString ].\r\n\tstream := '{' writeStream.\r\n\tself parsers\r\n\t\tdo: [:each| stream nextPutAll: each printString ]\r\n\t\tseparatedBy: [stream nextPutAll: self op].\r\n\t^stream contents","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal stream=nil\r\n(function() if ((((((((self)[\"debugTag\"])))~=nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:superImplementor(smalltalk.PPListParser,\"printString\")(self))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((\"{\"):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((self):parsers()):do_separatedBy_((function(each)\r\nreturn ((stream):nextPutAll_(smalltalk:printString_((each))))\r\nend), (function()\r\nreturn ((stream):nextPutAll_((self):op()))\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((stream):contents())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPSequenceParser,",","_comma"
,(
function(self,aRule)
return ((self):copyWith_(aRule))
;
end
)
,"copying",", aRule\r\n\t\" Return the receiver composed with the argument. \"\r\n\r\n\t^self copyWith: aRule","(\r\nfunction(self,aRule)\r\nreturn ((self):copyWith_(aRule))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPSequenceParser,"basicParse:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local start=nil
local result=nil
local element=nil
(function()
	local _s8_value=(smalltalk:newArray({}))
	result=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((aStream):position())
	start=_s8_value; return _s8_value
end)()
;
((self):parsers()):do_((function(each)
(function()
	local _s8_value=((each):memoizedParse_(aStream))
	element=_s8_value; return _s8_value
end)();(result):add_(element);return ((function() if ((((self):isParseFailure_(element)))==true) then return ((function()
(aStream):position_(start);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (element)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t| start result element |\r\n\tresult := #().\r\n\tstart := aStream position.\r\n\tself parsers do: [:each |\r\n\t\telement := each memoizedParse: aStream.\r\n\t\tresult add: element.\r\n\t\t(self isParseFailure: element) ifTrue: [\r\n\t\t\taStream position: start.\r\n\t\t\t^element\r\n\t\t]\r\n\t].\r\n\t^result","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal start=nil\r\nlocal result=nil\r\nlocal element=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:newArray({}))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aStream):position())\r\n\tstart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((self):parsers()):do_((function(each)\r\n(function()\r\n\tlocal _s8_value=((each):memoizedParse_(aStream))\r\n\telement=_s8_value; return _s8_value\r\nend)();(result):add_(element);return ((function() if ((((self):isParseFailure_(element)))==true) then return ((function()\r\n(aStream):position_(start);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (element)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPSequenceParser,"op",0
,(
function(self)
return (",")
;
end
)
,"printing","op\r\n\t\" Private - Return the operator of the receiver. \"\r\n\r\n\t^$,","(\r\nfunction(self)\r\nreturn (\",\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPChoiceParser,"/","_slash"
,(
function(self,aRule)
return ((self):copyWith_(aRule))
;
end
)
,"copying","/ aRule\r\n\t\" Return the receiver composed with aRule. \"\r\n\r\n\t^self copyWith: aRule","(\r\nfunction(self,aRule)\r\nreturn ((self):copyWith_(aRule))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPChoiceParser,"basicParse:",0
,(
function(self,aStream)
local result=nil
((self):parsers()):detect_ifNone_((function(each)
(function()
	local _s8_value=((each):memoizedParse_(aStream))
	result=_s8_value; return _s8_value
end)();return ((smalltalk:booleanNot_(((self):isParseFailure_(result)))))
end), (function()
return (nil)
end))
;
return (result)
;
end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t| result |\r\n\tself parsers detect: [:each |\r\n\t\tresult := each memoizedParse: aStream.\r\n\t\t(self isParseFailure: result) not\r\n\t] ifNone: [].\r\n\t^result","(\r\nfunction(self,aStream)\r\nlocal result=nil\r\n((self):parsers()):detect_ifNone_((function(each)\r\n(function()\r\n\tlocal _s8_value=((each):memoizedParse_(aStream))\r\n\tresult=_s8_value; return _s8_value\r\nend)();return ((smalltalk:booleanNot_(((self):isParseFailure_(result)))))\r\nend), (function()\r\nreturn (nil)\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPChoiceParser,"op",0
,(
function(self)
return ("/")
;
end
)
,"printing","op\r\n\t\" Private - Return the operator of the receiver. \"\r\n\r\n\t^$/","(\r\nfunction(self)\r\nreturn (\"/\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPDelegateParser["$klass"],"on:",0
,(
function(self,aParser)
return ((
function(_s8_rec)
(_s8_rec):parser_(aParser);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","on: aParser\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t    ^self new\r\n\t\tparser: aParser;\r\n\t\tyourself","(\r\nfunction(self,aParser)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):parser_(aParser);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPDelegateParser,"parser",0
,(
function(self)
return (self['@parser'])
;
end
)
,"accessing","parser\r\n\t\" Return the parser of the receiver. \"\r\n\r\n\t^parser","(\r\nfunction(self)\r\nreturn (self['@parser'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPDelegateParser,"parser:",0
,(
function(self,aParser)
(function()
	local _s8_value=(aParser)
	self['@parser']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","parser: aParser\r\n\t\" Set the parser of the receiver. \"\r\n\r\n\tparser := aParser","(\r\nfunction(self,aParser)\r\n(function()\r\n\tlocal _s8_value=(aParser)\r\n\tself['@parser']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPDelegateParser,"basicParse:",0
,(
function(self,aStream)
return (((self):parser()):memoizedParse_(aStream))
;
end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t^self parser memoizedParse: aStream","(\r\nfunction(self,aStream)\r\nreturn (((self):parser()):memoizedParse_(aStream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPDelegateParser,"printStringDetail",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_(("["),smalltalk:printString_((self['@parser'])))),"]"))
;
end
)
,"printing","printStringDetail\r\n\t\" Private - Return a printable detail of the receiver. \"\r\n\r\n\t^'[' ,parser printString ,']'","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"[\"),smalltalk:printString_((self['@parser'])))),\"]\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPAndParser,"opParse:",0
,(
function(self,aStream)
local element=nil
local position=nil
(function()
	local _s8_value=((aStream):position())
	position=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((self):parser()):memoizedParse_(aStream))
	element=_s8_value; return _s8_value
end)()
;
(aStream):position_(position)
;
return (element)
;
end
)
,"parsing","opParse: aStream\r\n\t\" Parse contents in aStream. \"\r\n\r\n\t| element position |\r\n\tposition := aStream position.\r\n\telement := self parser memoizedParse: aStream.\r\n\taStream position: position.\r\n\t^element","(\r\nfunction(self,aStream)\r\nlocal element=nil\r\nlocal position=nil\r\n(function()\r\n\tlocal _s8_value=((aStream):position())\r\n\tposition=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((self):parser()):memoizedParse_(aStream))\r\n\telement=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aStream):position_(position)\r\n;\r\nreturn (element)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPAndParser,"basicParse:",0
,(
function(self,aStream)
return ((self):opParse_(aStream))
;
end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t^self opParse: aStream","(\r\nfunction(self,aStream)\r\nreturn ((self):opParse_(aStream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPNotParser,"basicParse:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local element=nil
(function()
	local _s8_value=((self):opParse_(aStream))
	element=_s8_value; return _s8_value
end)()
;
(function() if ((((self):isParseFailure_(element)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.PPFailure):reason_at_(element, (aStream):position()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t| element |\r\n\telement := self opParse: aStream.\r\n\t(self isParseFailure: element) ifTrue: [ ^nil ].\r\n\t^PPFailure reason: element at: aStream position","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal element=nil\r\n(function()\r\n\tlocal _s8_value=((self):opParse_(aStream))\r\n\telement=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((self):isParseFailure_(element)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.PPFailure):reason_at_(element, (aStream):position()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPActionParser["$klass"],"on:block:",0
,(
function(self,aParser,aBlock)
return ((
function(_s8_rec)
(_s8_rec):parser_(aParser);
(_s8_rec):block_(aBlock);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","on: aParser block: aBlock\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tparser: aParser;\r\n\t\tblock: aBlock;\r\n\t\tyourself","(\r\nfunction(self,aParser,aBlock)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):parser_(aParser);\r\n(_s8_rec):block_(aBlock);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPActionParser,"block",0
,(
function(self)
return (self['@block'])
;
end
)
,"accessing","block\r\n\t\" Return the block of the receiver. \"\r\n\r\n\t^block","(\r\nfunction(self)\r\nreturn (self['@block'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPActionParser,"block:",0
,(
function(self,aBlock)
(function()
	local _s8_value=(aBlock)
	self['@block']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","block: aBlock\r\n\t\" Set the block of the receiver. \"\r\n\r\n\tblock := aBlock","(\r\nfunction(self,aBlock)\r\n(function()\r\n\tlocal _s8_value=(aBlock)\r\n\tself['@block']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPActionParser,"basicParse:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local element=nil
(function()
	local _s8_value=(((self):parser()):memoizedParse_(aStream))
	element=_s8_value; return _s8_value
end)()
;
(function() if ((((self):isParseFailure_(element)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (element)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self):block()):value_(element))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t| element |\r\n\telement := self parser memoizedParse: aStream.\r\n\t(self isParseFailure: element) ifTrue: [ ^element ].\r\n\t^self block value: element","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal element=nil\r\n(function()\r\n\tlocal _s8_value=(((self):parser()):memoizedParse_(aStream))\r\n\telement=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((self):isParseFailure_(element)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (element)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self):block()):value_(element))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPActionParser,"printString",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_(("{"),smalltalk:printString_((self['@parser'])))),"}"))
;
end
)
,"printing","printString\r\n\t\" Return a printable representation of the receiver. \"\r\n\r\n\t^'{', parser printString ,'}'","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"{\"),smalltalk:printString_((self['@parser'])))),\"}\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPFlattenParser,"basicParse:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local start=nil
local element=nil
(function()
	local _s8_value=((aStream):position())
	start=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((self):parser()):memoizedParse_(aStream))
	element=_s8_value; return _s8_value
end)()
;
(function() if ((((self):isParseFailure_(element)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (element)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((aStream):collection()):copyFrom_to_((start):_plus((1)), (aStream):position()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t| start element |\r\n\tstart := aStream position.\r\n\telement := self parser memoizedParse: aStream.\r\n\t(self isParseFailure: element) ifTrue: [ ^element ].\r\n\t^aStream collection \r\n\t\tcopyFrom: start + 1 \r\n\t\tto: aStream position","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal start=nil\r\nlocal element=nil\r\n(function()\r\n\tlocal _s8_value=((aStream):position())\r\n\tstart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((self):parser()):memoizedParse_(aStream))\r\n\telement=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((self):isParseFailure_(element)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (element)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((aStream):collection()):copyFrom_to_((start):_plus((1)), (aStream):position()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPSourceParser,"basicParse:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local start=nil
local element=nil
local result=nil
(function()
	local _s8_value=((aStream):position())
	start=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((self):parser()):memoizedParse_(aStream))
	element=_s8_value; return _s8_value
end)()
;
(function() if ((((self):isParseFailure_(element)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (element)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=(((aStream):collection()):copyFrom_to_((start):_plus((1)), (aStream):position()))
	result=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Array):with_with_(element, result))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t| start element result |\r\n\tstart := aStream position.\r\n\telement := self parser memoizedParse: aStream.\r\n\t(self isParseFailure: element) ifTrue: [ ^element ].\r\n\tresult := aStream collection\r\n\t\tcopyFrom: start + 1\r\n\t\tto: aStream position.\r\n\t^Array with: element with: result","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal start=nil\r\nlocal element=nil\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((aStream):position())\r\n\tstart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((self):parser()):memoizedParse_(aStream))\r\n\telement=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((self):isParseFailure_(element)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (element)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((aStream):collection()):copyFrom_to_((start):_plus((1)), (aStream):position()))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Array):with_with_(element, result))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPRepeatingParser["$klass"],"on:min:",0
,(
function(self,aParser,aNumber)
return ((
function(_s8_rec)
(_s8_rec):parser_(aParser);
(_s8_rec):min_(aNumber);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","on: aParser min: aNumber\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tparser: aParser;\r\n\t\tmin: aNumber;\r\n\t\tyourself","(\r\nfunction(self,aParser,aNumber)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):parser_(aParser);\r\n(_s8_rec):min_(aNumber);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPRepeatingParser,"min",0
,(
function(self)
return (self['@min'])
;
end
)
,"accessing","min\r\n\t\" Return the min of the receiver. \"\r\n\r\n\t^min","(\r\nfunction(self)\r\nreturn (self['@min'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPRepeatingParser,"min:",0
,(
function(self,aNumber)
(function()
	local _s8_value=(aNumber)
	self['@min']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","min: aNumber\r\n\t\" Set the min of the receiver. \"\r\n\r\n\tmin := aNumber","(\r\nfunction(self,aNumber)\r\n(function()\r\n\tlocal _s8_value=(aNumber)\r\n\tself['@min']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPRepeatingParser,"basicParse:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local start=nil
local element=nil
local result=nil
local failure=nil
(function()
	local _s8_value=((aStream):position())
	start=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.Array):new())
	result=_s8_value; return _s8_value
end)()
;
(function() while(((function()
return ((smalltalk:boolean_and_(((((result):size()) < (self):min())),(function()
return ((((failure))==nil))
end))))
end))()) do (function()
(function()
	local _s8_value=(((self):parser()):memoizedParse_(aStream))
	element=_s8_value; return _s8_value
end)();return ((function() if ((((self):isParseFailure_(element)))==true) then return ((function()
(aStream):position_(start);return ((function()
	local _s8_value=(element)
	failure=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((result):addLast_(element))
end))() end end)())
end)() end;return nil end)()
;
(function() if ((((((failure))~=nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (failure)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() while(((function()
return ((((failure))==nil))
end))()) do (function()
(function()
	local _s8_value=(((self):parser()):memoizedParse_(aStream))
	element=_s8_value; return _s8_value
end)();return ((function() if ((((self):isParseFailure_(element)))==true) then return ((function()
return ((function()
	local _s8_value=(element)
	failure=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((result):addLast_(element))
end))() end end)())
end)() end;return nil end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t| start element result failure |\r\n\tstart := aStream position.\r\n\tresult := Array new.\r\n\t[ (result size < self min) and: [ failure isNil ]\r\n\t] whileTrue: [\r\n\t    element := self parser memoizedParse: aStream.\r\n\t    (self isParseFailure: element)\r\n\t\tifFalse: [ result addLast: element ]\r\n\t\tifTrue: [ aStream position: start.\r\n\t\t\t  failure := element ]\r\n\t].\r\n\tfailure notNil ifTrue: [ ^failure ].\r\n\t[failure isNil] whileTrue: [\r\n\t\telement := self parser memoizedParse: aStream.\r\n\t \t(self isParseFailure: element)\r\n\t\t\tifTrue: [ failure := element ]\r\n\t\t\tifFalse: [ result addLast: element ]\r\n\t].\r\n\t^result","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal start=nil\r\nlocal element=nil\r\nlocal result=nil\r\nlocal failure=nil\r\n(function()\r\n\tlocal _s8_value=((aStream):position())\r\n\tstart=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\nreturn ((smalltalk:boolean_and_(((((result):size()) < (self):min())),(function()\r\nreturn ((((failure))==nil))\r\nend))))\r\nend))()) do (function()\r\n(function()\r\n\tlocal _s8_value=(((self):parser()):memoizedParse_(aStream))\r\n\telement=_s8_value; return _s8_value\r\nend)();return ((function() if ((((self):isParseFailure_(element)))==true) then return ((function()\r\n(aStream):position_(start);return ((function()\r\n\tlocal _s8_value=(element)\r\n\tfailure=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((result):addLast_(element))\r\nend))() end end)())\r\nend)() end;return nil end)()\r\n;\r\n(function() if ((((((failure))~=nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (failure)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() while(((function()\r\nreturn ((((failure))==nil))\r\nend))()) do (function()\r\n(function()\r\n\tlocal _s8_value=(((self):parser()):memoizedParse_(aStream))\r\n\telement=_s8_value; return _s8_value\r\nend)();return ((function() if ((((self):isParseFailure_(element)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(element)\r\n\tfailure=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((result):addLast_(element))\r\nend))() end end)())\r\nend)() end;return nil end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PPRepeatingParser,"printStringDetail",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_(("[min="),self['@min'])),"]"))
;
end
)
,"printing","printStringDetail\r\n\t\" Private - Return a printable detail of the receiver. \"\r\n\r\n\t^'[min=' ,min ,']'","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"[min=\"),self['@min'])),\"]\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPInlineParser["$klass"],"upTo:",0
,(
function(self,aString)
return ((
function(_s8_rec)
(_s8_rec):terminator_(aString);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","upTo: aString\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new terminator: aString; yourself","(\r\nfunction(self,aString)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):terminator_(aString);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPInlineParser,"terminator:",0
,(
function(self,aString)
(function()
	local _s8_value=(aString)
	self['@terminator']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","terminator: aString\r\n\t\" Set terminator of the receiver. \"\r\n\r\n\tterminator := aString","(\r\nfunction(self,aString)\r\n(function()\r\n\tlocal _s8_value=(aString)\r\n\tself['@terminator']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PPInlineParser,"basicParse:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function()
	local _s8_value=((aStream):upTo_(self['@terminator']))
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((aStream):atEnd()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.PPFailure):reason_at_(smalltalk:object_comma_((smalltalk:object_comma_(("Missing inline terminator ("),self['@terminator'])),")"), (aStream):position()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"parsing","basicParse: aStream\r\n\t\" Private - Parse contents in aStream. \"\r\n\r\n\t| result |\r\n\tresult := aStream upTo: terminator.\r\n\taStream atEnd ifFalse: [ ^result ].\r\n\t^PPFailure\r\n\t\treason: 'Missing inline terminator (',terminator,')'\r\n\t\tat: aStream position","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((aStream):upTo_(self['@terminator']))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((aStream):atEnd()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.PPFailure):reason_at_(smalltalk:object_comma_((smalltalk:object_comma_((\"Missing inline terminator (\"),self['@terminator'])),\")\"), (aStream):position()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.FileInReader,"scanFrom:",0
,(
function(self,aStream)
return ((self):subclassResponsibility_("scanFrom:"))
;
end
)
,"fileIn","scanFrom: aStream\r\n\t\" FileIn - Return the result of evaluating the next chunk of data from aStream.\r\n\tThe method must be implemented by chunk readers to support custom fileIn features.\r\n\t\"\r\n\r\n\t^self subclassResponsibility: #scanFrom:","(\r\nfunction(self,aStream)\r\nreturn ((self):subclassResponsibility_(\"scanFrom:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ChunkReader["$klass"],"doing:until:",0
,(
function(self,aBlock,terminationBlock)
return ((
function(_s8_rec)
(_s8_rec):actionBlock_(aBlock);
(_s8_rec):terminationBlock_(terminationBlock);
return ((_s8_rec):yourself())
end
)((self):new()))
;
end
)
,"instantiation","doing: aBlock until: terminationBlock\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new\r\n\t\tactionBlock: aBlock;\r\n\t\tterminationBlock: terminationBlock;\r\n\t\tyourself","(\r\nfunction(self,aBlock,terminationBlock)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):actionBlock_(aBlock);\r\n(_s8_rec):terminationBlock_(terminationBlock);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)((self):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ChunkReader["$klass"],"doing:",0
,(
function(self,aBlock)
return ((self):doing_until_(aBlock, (function()
return (true)
end)))
;
end
)
,"instantiation","doing: aBlock\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self doing: aBlock until: [ true ]","(\r\nfunction(self,aBlock)\r\nreturn ((self):doing_until_(aBlock, (function()\r\nreturn (true)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ChunkReader["$klass"],"until:",0
,(
function(self,terminationBlock)
return ((self):doing_until_(nil, terminationBlock))
;
end
)
,"instantiation","until: terminationBlock\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self doing: nil until: terminationBlock","(\r\nfunction(self,terminationBlock)\r\nreturn ((self):doing_until_(nil, terminationBlock))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ChunkReader["$klass"],"ignore",0
,(
function(self)
return ((self):new())
;
end
)
,"instantiation","ignore\r\n\t\" Return an instance of the receiver.\r\n\tThe returned reader will ignore/absorb next chunk.\r\n\t\"\r\n\r\n\t^self new","(\r\nfunction(self)\r\nreturn ((self):new())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ChunkReader["$klass"],"javascriptChunk",0
,(
function(self)
return ((self):doing_((function(aStream)
return (((smalltalk.Smalltalk):current()):eval_((aStream):nextChunk()))
end)))
;
end
)
,"instantiation","javascriptChunk\r\n\t\" Return an instance of the receiver.\r\n\tThe returned reader will evaluate next chunk as native code (in global context).\r\n\t\"\r\n\r\n\t^self doing: [:aStream| Smalltalk current eval: aStream nextChunk ]","(\r\nfunction(self)\r\nreturn ((self):doing_((function(aStream)\r\nreturn (((smalltalk.Smalltalk):current()):eval_((aStream):nextChunk()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ChunkReader["$klass"],"javascript",0
,(
function(self)
return ((self):doing_((function(aStream)
return (((smalltalk.Smalltalk):current()):eval_((aStream):upToEnd()))
end)))
;
end
)
,"instantiation","javascript\r\n\t\" Return an instance of the receiver.\r\n\tThe returned reader will evaluate following contents as native code (in global context).\r\n\tThe stream will be consumed up to end.\r\n\t\"\r\n\r\n\t^self doing: [:aStream| Smalltalk current eval: aStream upToEnd ]","(\r\nfunction(self)\r\nreturn ((self):doing_((function(aStream)\r\nreturn (((smalltalk.Smalltalk):current()):eval_((aStream):upToEnd()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ChunkReader,"actionBlock",0
,(
function(self)
return (self['@actionBlock'])
;
end
)
,"accessing","actionBlock\r\n\t\" Return the actionBlock of the receiver. \"\r\n\r\n\t^actionBlock","(\r\nfunction(self)\r\nreturn (self['@actionBlock'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ChunkReader,"actionBlock:",0
,(
function(self,aBlock)
(function()
	local _s8_value=(aBlock)
	self['@actionBlock']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","actionBlock: aBlock\r\n\t\" Set the actionBlock of the receiver. \"\r\n\r\n\tactionBlock := aBlock","(\r\nfunction(self,aBlock)\r\n(function()\r\n\tlocal _s8_value=(aBlock)\r\n\tself['@actionBlock']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ChunkReader,"terminationBlock",0
,(
function(self)
return (self['@terminationBlock'])
;
end
)
,"accessing","terminationBlock\r\n\t\" Return the terminationBlock of the receiver. \"\r\n\r\n\t^terminationBlock","(\r\nfunction(self)\r\nreturn (self['@terminationBlock'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ChunkReader,"terminationBlock:",0
,(
function(self,aBlock)
(function()
	local _s8_value=(aBlock)
	self['@terminationBlock']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","terminationBlock: aBlock\r\n\t\" Set the terminationBlock of the receiver. \"\r\n\r\n\tterminationBlock := aBlock","(\r\nfunction(self,aBlock)\r\n(function()\r\n\tlocal _s8_value=(aBlock)\r\n\tself['@terminationBlock']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ChunkReader,"scanFrom:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local chunk=nil
(function() if (((((((self):actionBlock()))==nil)))==true) then return ((function()
(function()
	local _s8_value=((aStream):nextChunk())
	chunk=_s8_value; return _s8_value
end)();return ((function() if ((((chunk):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() else return ((function()
return (((self):actionBlock()):value_(aStream))
end))() end end)()
;
(function() if ((((smalltalk:boolean_or_((((((self):terminationBlock()))==nil)),(function()
return (((true)==((self):terminationBlock()):value()))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"fileIn","scanFrom: aStream\r\n\t\" Scan the next chunk of data from aStream. \"\r\n\r\n\t| chunk |\r\n\tself actionBlock isNil ifTrue: [\r\n\t\tchunk := aStream nextChunk.\r\n\t\tchunk isEmpty ifTrue: [ ^nil ].\r\n\t] ifFalse: [ self actionBlock value: aStream ].\r\n\t(self terminationBlock isNil\r\n\t\tor: [ true == self terminationBlock value ])\r\n\t\tifTrue: [ ^nil ].\r\n\t^self","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal chunk=nil\r\n(function() if (((((((self):actionBlock()))==nil)))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((aStream):nextChunk())\r\n\tchunk=_s8_value; return _s8_value\r\nend)();return ((function() if ((((chunk):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return ((function()\r\nreturn (((self):actionBlock()):value_(aStream))\r\nend))() end end)()\r\n;\r\n(function() if ((((smalltalk:boolean_or_((((((self):terminationBlock()))==nil)),(function()\r\nreturn (((true)==((self):terminationBlock()):value()))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassCategoryReader,"class:category:",0
,(
function(self,aClass,aString)
(function()
	local _s8_value=(aClass)
	self['@class']=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(aString)
	self['@category']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","class: aClass category: aString\r\n\t\" Private - Set the class and category of the receiver. \"\r\n\r\n\tclass := aClass.\r\n\tcategory := aString","(\r\nfunction(self,aClass,aString)\r\n(function()\r\n\tlocal _s8_value=(aClass)\r\n\tself['@class']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(aString)\r\n\tself['@category']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassCategoryReader,"scanFrom:",0
,(
function(self,aStream)
local chunk=nil
(function()
	local _s8_value=((aStream):nextChunk())
	chunk=_s8_value; return _s8_value
end)()
;
return (((function()
return ((function() if ((((chunk):isEmpty()))==true) then return nil else return ((function()
return ((self):compileMethod_(chunk))
end))() end end)())
end)):logEvaluate_(chunk))
;
end
)
,"fileIn","scanFrom: aStream\r\n\t\" Scan the next chunk of data from aStream. \"\r\n\r\n\t| chunk |\r\n\tchunk := aStream nextChunk.\r\n\t^[ chunk isEmpty ifFalse: [ self compileMethod: chunk ]\r\n\t] logEvaluate: chunk","(\r\nfunction(self,aStream)\r\nlocal chunk=nil\r\n(function()\r\n\tlocal _s8_value=((aStream):nextChunk())\r\n\tchunk=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (((function()\r\nreturn ((function() if ((((chunk):isEmpty()))==true) then return nil else return ((function()\r\nreturn ((self):compileMethod_(chunk))\r\nend))() end end)())\r\nend)):logEvaluate_(chunk))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassCategoryReader,"compileMethod:",0
,(
function(self,aString)
((self['@class']):compiler()):compile_forClass_category_(aString, self['@class'], self['@category'])
;
return (self)
;
end
)
,"private","compileMethod: aString\r\n\t\" Compile aString as a method source. \"\r\n\r\n\tclass compiler\r\n\t\tcompile: aString\r\n\t\tforClass: class\r\n\t\tcategory: category","(\r\nfunction(self,aString)\r\n((self['@class']):compiler()):compile_forClass_category_(aString, self['@class'], self['@category'])\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassCommentReader,"class:",0
,(
function(self,aClass)
(function()
	local _s8_value=(aClass)
	self['@class']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","class: aClass\r\n\t\" Set the destination class of the receiver. \"\r\n\r\n\tclass := aClass","(\r\nfunction(self,aClass)\r\n(function()\r\n\tlocal _s8_value=(aClass)\r\n\tself['@class']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassCommentReader,"scanFrom:",0
,(
function(self,aStream)
(self):comment_((aStream):nextChunk())
;
return (nil)
;
end
)
,"fileIn","scanFrom: aStream\r\n\t\" FileIn - Parse next chunk in aStream as a comment for target class. \"\r\n\r\n\tself comment: aStream nextChunk.\r\n\t^nil","(\r\nfunction(self,aStream)\r\n(self):comment_((aStream):nextChunk())\r\n;\r\nreturn (nil)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.ClassCommentReader,"comment:",0
,(
function(self,aString)
(self['@class']):comment_((function() if ((((aString):isEmpty()))==true) then return nil else return ((function()
return (aString)
end))() end end)())
;
return (self)
;
end
)
,"private","comment: aString\r\n\t\" Set the comment of target class. \"\r\n\r\n\tclass comment: (aString isEmpty ifFalse: [ aString ])","(\r\nfunction(self,aString)\r\n(self['@class']):comment_((function() if ((((aString):isEmpty()))==true) then return nil else return ((function()\r\nreturn (aString)\r\nend))() end end)())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SkipFileInReader["$klass"],"upTo:",0
,(
function(self,aToken)
return (((self):new()):limit_(aToken))
;
end
)
,"instantiation","upTo: aToken\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self new limit: aToken","(\r\nfunction(self,aToken)\r\nreturn (((self):new()):limit_(aToken))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SkipFileInReader,"scanFrom:",0
,(
function(self,aStream)
(function() if ((((((self['@limit']))==nil)))==true) then return ((function()
return ((aStream):setToEnd())
end))() else return ((function()
return ((aStream):skipTo_(self['@limit']))
end))() end end)()
;
return (nil)
;
end
)
,"fileIn","scanFrom: aStream\r\n\t\" Consume/Ignore contents in aStream. \"\r\n\r\n\tlimit isNil\r\n\t\tifTrue: [ aStream setToEnd ]\r\n\t\tifFalse: [ aStream skipTo: limit ].\r\n\t^nil","(\r\nfunction(self,aStream)\r\n(function() if ((((((self['@limit']))==nil)))==true) then return ((function()\r\nreturn ((aStream):setToEnd())\r\nend))() else return ((function()\r\nreturn ((aStream):skipTo_(self['@limit']))\r\nend))() end end)()\r\n;\r\nreturn (nil)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.SkipFileInReader,"limit:",0
,(
function(self,aToken)
(function()
	local _s8_value=(aToken)
	self['@limit']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","limit: aToken\r\n\t\" Set the token limit. \"\r\n\r\n\tlimit := aToken","(\r\nfunction(self,aToken)\r\n(function()\r\n\tlocal _s8_value=(aToken)\r\n\tself['@limit']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AssociationsReader,"keyFrom:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:isObject_equalTo_(((aStream):peek()),"#"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((
function(_s8_rec)
(_s8_rec):next();
return ((_s8_rec):upTo_(" "))
end
)(aStream)):asSymbol())
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((aStream):peek()),"'"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((
function(_s8_rec)
(_s8_rec):next();
return ((_s8_rec):upTo_("'"))
end
)(aStream))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aStream):upTo_(" "))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"reading","keyFrom: aStream\r\n\t\" Private - Return the name of the current binding in aStream. \"\r\n\r\n\taStream peek = $# ifTrue: [ ^(aStream next; upTo: $ ) asSymbol ].\r\n\taStream peek = $' ifTrue: [ ^aStream next; upTo: $' ].\r\n\t^aStream upTo: $ ","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:isObject_equalTo_(((aStream):peek()),\"#\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((\r\nfunction(_s8_rec)\r\n(_s8_rec):next();\r\nreturn ((_s8_rec):upTo_(\" \"))\r\nend\r\n)(aStream)):asSymbol())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((aStream):peek()),\"'\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):next();\r\nreturn ((_s8_rec):upTo_(\"'\"))\r\nend\r\n)(aStream))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aStream):upTo_(\" \"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.AssociationsReader,"valueFrom:",0
,(
function(self,aStream)
return (((self):compilerSupport()):evaluate_((aStream):upToEnd()))
;
end
)
,"reading","valueFrom: aStream\r\n\t\" Private - Return the value of the current binding in aStream. \"\r\n\r\n\t^self compilerSupport evaluate: aStream upToEnd ","(\r\nfunction(self,aStream)\r\nreturn (((self):compilerSupport()):evaluate_((aStream):upToEnd()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AssociationsReader,"compilerSupport",0
,(
function(self)
return (((self):smalltalk()):compilerSupport())
;
end
)
,"evaluating","compilerSupport\r\n\t\" Private - Return compiler in use to get values from stream. \"\r\n\r\n\t^self smalltalk compilerSupport","(\r\nfunction(self)\r\nreturn (((self):smalltalk()):compilerSupport())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AssociationsReader,"bind:",0
,(
function(self,tuples)
return ((self):subclassResponsibility_("bind:"))
;
end
)
,"binding","bind: tuples\r\n\t\" Private - Bind the filedIn tuples to target.\r\n\tMust return the result of the fileIn/binding process.\r\n\t\"\r\n\r\n\t^self subclassResponsibility: #bind: ","(\r\nfunction(self,tuples)\r\nreturn ((self):subclassResponsibility_(\"bind:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.AssociationsReader,"scanFrom:",0
,(
function(self,aStream)
local chunk=nil
local tuples=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	tuples=_s8_value; return _s8_value
end)()
;
(function() while(((function()
return ((smalltalk:boolean_and_(((smalltalk:booleanNot_(((aStream):atEnd())))),(function()
(function()
	local _s8_value=(((aStream):nextChunk()):trimBlanks())
	chunk=_s8_value; return _s8_value
end)();return ((chunk):notEmpty())
end))))
end))()) do (function()
(function()
	local _s8_value=((chunk):stream())
	chunk=_s8_value; return _s8_value
end)();return ((tuples):add_((smalltalk.Array):with_with_((self):keyFrom_(chunk), (self):valueFrom_(chunk))))
end)() end;return nil end)()
;
return ((self):bind_(tuples))
;
end
)
,"fileIn","scanFrom: aStream\r\n\t\" Consume chunks in aStream until empty chunk.\r\n\tThe chunks are considered associative bindings with the syntax:\r\n\t\t<key> <expression>\r\n\tWhere\r\n\t\tkey is the keyword of the binding (name, string or symbol).\r\n\t\texpression is a valid smalltalk expression that will be evaluated to generate the value of the binding.\r\n\t\"\r\n\r\n\t| chunk tuples |\r\n\ttuples := Array new.\r\n\t[ aStream atEnd not and: [\r\n\t  chunk := aStream nextChunk trimBlanks.\r\n\t  chunk notEmpty ]] whileTrue: [\r\n\t  chunk := chunk stream.\r\n\t  tuples add: (Array\r\n\t\twith: (self keyFrom: chunk)\r\n\t\twith: (self valueFrom: chunk)).\r\n\t].\r\n\t^self bind: tuples","(\r\nfunction(self,aStream)\r\nlocal chunk=nil\r\nlocal tuples=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\ttuples=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(((function()\r\nreturn ((smalltalk:boolean_and_(((smalltalk:booleanNot_(((aStream):atEnd())))),(function()\r\n(function()\r\n\tlocal _s8_value=(((aStream):nextChunk()):trimBlanks())\r\n\tchunk=_s8_value; return _s8_value\r\nend)();return ((chunk):notEmpty())\r\nend))))\r\nend))()) do (function()\r\n(function()\r\n\tlocal _s8_value=((chunk):stream())\r\n\tchunk=_s8_value; return _s8_value\r\nend)();return ((tuples):add_((smalltalk.Array):with_with_((self):keyFrom_(chunk), (self):valueFrom_(chunk))))\r\nend)() end;return nil end)()\r\n;\r\nreturn ((self):bind_(tuples))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionaryReader,"scope:",0
,(
function(self,aScope)
(function()
	local _s8_value=(aScope)
	self['@scope']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","scope: aScope\r\n\t\" Set the scope of the receiver. \"\r\n\r\n\tscope := aScope","(\r\nfunction(self,aScope)\r\n(function()\r\n\tlocal _s8_value=(aScope)\r\n\tself['@scope']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionaryReader,"bind:",0
,(
function(self,tuples)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self['@scope']):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Smalltalk):definePool_with_(self['@scope'], tuples))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((((self['@scope']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.PoolDictionary):new())
	self['@scope']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(tuples):do_((function(each)
return ((self['@scope']):at_put_((each):first(), (each):last()))
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self['@scope'])
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"binding","bind: tuples\r\n\t\" Private - Bind the filedIn tuples to target.\r\n\tMust return the result of the fileIn/binding process.\r\n\t\"\r\n\r\n\tscope isString ifTrue: [\r\n\t\t^Smalltalk definePool: scope with: tuples\r\n\t].\r\n\tscope isNil ifTrue: [ scope := PoolDictionary new ].\r\n\ttuples do: [:each| scope at: each first put: each last ].\r\n\t^scope","(\r\nfunction(self,tuples)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self['@scope']):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Smalltalk):definePool_with_(self['@scope'], tuples))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((self['@scope']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.PoolDictionary):new())\r\n\tself['@scope']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(tuples):do_((function(each)\r\nreturn ((self['@scope']):at_put_((each):first(), (each):last()))\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self['@scope'])\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.PoolDictionaryReader,"valueFrom:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local code=nil
local result=nil
local word=nil
(function()
	local _s8_value=((aStream):upToEnd())
	code=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((code):trimBlanks())
	word=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:boolean_or_(((smalltalk:isObject_equalTo_((word),"nil"))),(function()
return ((word):isEmpty())
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((word):asNumber())
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((result):isNumber()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:superImplementor(smalltalk.PoolDictionaryReader,"valueFrom_")(self,(code):stream()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"reading","valueFrom: aStream\r\n\t\" Private - Return the value of the current binding in aStream. \"\r\n\r\n\t| code result word |\r\n\tcode := aStream upToEnd.\r\n\tword := code trimBlanks.\r\n\t(word = #nil or: [ word isEmpty ]) ifTrue: [ ^nil ].\r\n\tresult := word asNumber.\r\n\tresult isNumber ifTrue: [ ^result ].\r\n\t^super valueFrom: code stream ","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal code=nil\r\nlocal result=nil\r\nlocal word=nil\r\n(function()\r\n\tlocal _s8_value=((aStream):upToEnd())\r\n\tcode=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((code):trimBlanks())\r\n\tword=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:boolean_or_(((smalltalk:isObject_equalTo_((word),\"nil\"))),(function()\r\nreturn ((word):isEmpty())\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((word):asNumber())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((result):isNumber()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:superImplementor(smalltalk.PoolDictionaryReader,\"valueFrom_\")(self,(code):stream()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeReader,"bind:",0
,(
function(self,tuples)
(function() if ((((((self['@object']))==nil)))==true) then return ((function()
return ((self):initializeObject())
end))() else return nil end end)()
;
(tuples):do_((function(each)
return ((function() local _1_=((each):last()); (self['@object'])[(each):first()]=_1_; return _1_ end)())
end))
;
return (self['@object'])
;
end
)
,"binding","bind: tuples\r\n\t\" Private - Bind the filedIn tuples to target.\r\n\tMust return the result of the fileIn/binding process.\r\n\t\"\r\n\r\n\tobject isNil ifTrue: [ self initializeObject ].\r\n\ttuples do: [:each| object basicAt: each first put: each last ].\r\n\t^object","(\r\nfunction(self,tuples)\r\n(function() if ((((((self['@object']))==nil)))==true) then return ((function()\r\nreturn ((self):initializeObject())\r\nend))() else return nil end end)()\r\n;\r\n(tuples):do_((function(each)\r\nreturn ((function() local _1_=((each):last()); (self['@object'])[(each):first()]=_1_; return _1_ end)())\r\nend))\r\n;\r\nreturn (self['@object'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeReader,"object:",0
,(
function(self,anObject)
(function()
	local _s8_value=(anObject)
	self['@object']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","object: anObject\r\n\t\" Set the object where the bindings will be stored. \"\r\n\r\n\tobject := anObject","(\r\nfunction(self,anObject)\r\n(function()\r\n\tlocal _s8_value=(anObject)\r\n\tself['@object']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeReader,"initializeObject",0
,(
function(self)
(function()
	local _s8_value=({})
	self['@object']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"initialize","initializeObject\r\n\t\" Private - Initialize the object where the bindings will be stored. \"\r\n\r\n\tobject := {'{}' lua'{}'}","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=({})\r\n\tself['@object']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter["$klass"],"fileOutCategories:",0
,(
function(self,categories)
return (((self):new()):fileOutCategories_(categories))
;
end
)
,"fileOut","fileOutCategories: categories\r\n\t\" Return the result of fileOut all the classes in categories. \"\r\n\r\n\t^self new fileOutCategories: categories","(\r\nfunction(self,categories)\r\nreturn (((self):new()):fileOutCategories_(categories))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter["$klass"],"fileOutCategory:",0
,(
function(self,aString)
return (((self):new()):fileOutCategory_(aString))
;
end
)
,"fileOut","fileOutCategory: aString\r\n\t\" Return the result of fileOut all the classes in category aString. \"\r\n\r\n\t^self new fileOutCategory: aString","(\r\nfunction(self,aString)\r\nreturn (((self):new()):fileOutCategory_(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter["$klass"],"fileOutAll:",0
,(
function(self,elements)
return (((self):new()):fileOutAll_(elements))
;
end
)
,"fileOut","fileOutAll: elements\r\n\t\" Return the result of fileOut elements (classes, methods and/or categories). \"\r\n\r\n\t^self new fileOutAll: elements","(\r\nfunction(self,elements)\r\nreturn (((self):new()):fileOutAll_(elements))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter["$klass"],"fileOutAll",0
,(
function(self)
return ((self):fileOutAll_((self):allClasses()))
;
end
)
,"fileOut","fileOutAll\r\n\t\" Return the result of fileOut of all classes in the system. \"\r\n\r\n\t^self fileOutAll: self allClasses","(\r\nfunction(self)\r\nreturn ((self):fileOutAll_((self):allClasses()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter["$klass"],"fileOut:",0
,(
function(self,spec)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local aName=nil
(function() if ((((((spec))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((spec):isString()))==true) then return nil else return ((function()
(function() if (((smalltalk:is_kindOf_((spec),(smalltalk.Behavior))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self):new()):fileOut_(spec))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((spec):asArray()):fileOut())
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((spec):includes_(",")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):fileOut_((spec):asArrayOfSubstringsSeparatedBy_(",")))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((spec):includes_(" ")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):fileOut_((spec):asArrayOfSubstringsSeparatedBy_(" ")))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((spec):trimBlanks())
	aName=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk.Smalltalk):includesKey_(aName)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):fileOut_((smalltalk.Smalltalk):at_(aName)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):fileOutCategories_(aName))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"fileOut","fileOut: spec\r\n\t\" Return the result of fileOut of literals in spec. \"\r\n\r\n\t| aName |\r\n\tspec isNil ifTrue: [ ^'' ].\r\n\tspec isString ifFalse: [\r\n\t\t(spec isKindOf: Behavior) ifTrue: [ ^self new fileOut: spec ].\r\n\t\t^spec asArray fileOut\r\n\t].\r\n\t(spec includes: $,) ifTrue: [ ^self fileOut: (spec asArrayOfSubstringsSeparatedBy: $,) ].\r\n\t(spec includes: $ ) ifTrue: [ ^self fileOut: (spec asArrayOfSubstringsSeparatedBy: $ ) ].\r\n\taName := spec trimBlanks.\r\n\t(Smalltalk includesKey: aName) ifTrue: [ ^self fileOut: (Smalltalk at: aName)].\r\n\t^self fileOutCategories: aName","(\r\nfunction(self,spec)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal aName=nil\r\n(function() if ((((((spec))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((spec):isString()))==true) then return nil else return ((function()\r\n(function() if (((smalltalk:is_kindOf_((spec),(smalltalk.Behavior))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self):new()):fileOut_(spec))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((spec):asArray()):fileOut())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((spec):includes_(\",\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):fileOut_((spec):asArrayOfSubstringsSeparatedBy_(\",\")))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((spec):includes_(\" \")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):fileOut_((spec):asArrayOfSubstringsSeparatedBy_(\" \")))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((spec):trimBlanks())\r\n\taName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk.Smalltalk):includesKey_(aName)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):fileOut_((smalltalk.Smalltalk):at_(aName)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):fileOutCategories_(aName))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter["$klass"],"exportCategories:",0
,(
function(self,categories)
return (((self):new()):exportCategories_(categories))
;
end
)
,"export","exportCategories: categories\r\n\t\" Return the result of exporting all the classes in categories. \"\r\n\r\n\t^self new exportCategories: categories","(\r\nfunction(self,categories)\r\nreturn (((self):new()):exportCategories_(categories))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter["$klass"],"exportCategory:",0
,(
function(self,aString)
return (((self):new()):exportCategory_(aString))
;
end
)
,"export","exportCategory: aString\r\n\t\" Return the result of exporting all the classes in category aString. \"\r\n\r\n\t^self new exportCategory: aString","(\r\nfunction(self,aString)\r\nreturn (((self):new()):exportCategory_(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter["$klass"],"export:",0
,(
function(self,spec)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local aName=nil
(function() if ((((((spec))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((spec):isString()))==true) then return nil else return ((function()
(function() if (((smalltalk:is_kindOf_((spec),(smalltalk.Behavior))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((self):new()):export_(spec))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((spec):asArray()):export())
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((spec):includes_(",")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):export_((spec):asArrayOfSubstringsSeparatedBy_(",")))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((spec):includes_(" ")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):export_((spec):asArrayOfSubstringsSeparatedBy_(" ")))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((spec):trimBlanks())
	aName=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk.Smalltalk):includesKey_(aName)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):export_((smalltalk.Smalltalk):at_(aName)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):exportCategories_(aName))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"export","export: spec\r\n\t\" Return the result of exporting literals in spec. \"\r\n\r\n\t| aName |\r\n\tspec isNil ifTrue: [ ^'' ].\r\n\tspec isString ifFalse: [\r\n\t\t(spec isKindOf: Behavior) ifTrue: [ ^self new export: spec ].\r\n\t\t^spec asArray export\r\n\t].\r\n\t(spec includes: $,) ifTrue: [ ^self export: (spec asArrayOfSubstringsSeparatedBy: $,) ].\r\n\t(spec includes: $ ) ifTrue: [ ^self export: (spec asArrayOfSubstringsSeparatedBy: $ ) ].\r\n\taName := spec trimBlanks.\r\n\t(Smalltalk includesKey: aName) ifTrue: [ ^self export: (Smalltalk at: aName)].\r\n\t^self exportCategories: aName","(\r\nfunction(self,spec)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal aName=nil\r\n(function() if ((((((spec))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((spec):isString()))==true) then return nil else return ((function()\r\n(function() if (((smalltalk:is_kindOf_((spec),(smalltalk.Behavior))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((self):new()):export_(spec))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((spec):asArray()):export())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((spec):includes_(\",\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):export_((spec):asArrayOfSubstringsSeparatedBy_(\",\")))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((spec):includes_(\" \")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):export_((spec):asArrayOfSubstringsSeparatedBy_(\" \")))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((spec):trimBlanks())\r\n\taName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk.Smalltalk):includesKey_(aName)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):export_((smalltalk.Smalltalk):at_(aName)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):exportCategories_(aName))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter["$klass"],"exportAll",0
,(
function(self)
return (((self):new()):exportAll())
;
end
)
,"export","exportAll\r\n\t\" Return the result of exporting all classes in the system. \"\r\n\r\n\t^self new exportAll","(\r\nfunction(self)\r\nreturn (((self):new()):exportAll())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter["$klass"],"exportAll:",0
,(
function(self,elements)
return (((self):new()):exportAll_(elements))
;
end
)
,"export","exportAll: elements\r\n\t\" Return the result of exporting all elements (classes, methods and/or categories). \"\r\n\r\n\t^self new exportAll: elements","(\r\nfunction(self,elements)\r\nreturn (((self):new()):exportAll_(elements))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter["$klass"],"allClasses",0
,(
function(self)
return (((smalltalk.Smalltalk):rootClasses()):inject_into_(smalltalk:newArray({}), (function(total,each)
return ((
function(_s8_rec)
(_s8_rec):addAll_((each):withAllSubclasses());
return ((_s8_rec):yourself())
end
)(total))
end)))
;
end
)
,"accessing","allClasses\r\n\t\" Private - Return all the classes in the system (in fileIn order). \"\r\n\r\n\t^Smalltalk rootClasses inject: #() into: [:total :each|\r\n\t\ttotal addAll: each withAllSubclasses; yourself\r\n\t]","(\r\nfunction(self)\r\nreturn (((smalltalk.Smalltalk):rootClasses()):inject_into_(smalltalk:newArray({}), (function(total,each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):addAll_((each):withAllSubclasses());\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(total))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"encoder:",0
,(
function(self,aS8Encoder)
(function()
	local _s8_value=(aS8Encoder)
	self['@encoder']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","encoder: aS8Encoder\r\n\t\" Set the encoder of the receiver. \"\r\n\r\n\tencoder := aS8Encoder","(\r\nfunction(self,aS8Encoder)\r\n(function()\r\n\tlocal _s8_value=(aS8Encoder)\r\n\tself['@encoder']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"setEncoderFor:",0
,(
function(self,aClass)
(self):encoder_(((aClass):compiler()):encoder())
;
return (self)
;
end
)
,"accessing","setEncoderFor: aClass\r\n\t\" Set the encoder of the receiver to emmit code for aClass. \"\r\n\r\n\tself encoder: aClass compiler encoder","(\r\nfunction(self,aClass)\r\n(self):encoder_(((aClass):compiler()):encoder())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"encoderFor:",0
,(
function(self,aClass)
(function() if ((((((self['@encoder']))==nil)))==true) then return ((function()
return ((self):setEncoderFor_(aClass))
end))() else return nil end end)()
;
return (self['@encoder'])
;
end
)
,"accessing","encoderFor: aClass\r\n\t\" Return the encoder of the receiver.\r\n\tIf the encoder is nil, use the default encoder for aClass.\r\n\t\"\r\n\r\n\tencoder isNil ifTrue: [ self setEncoderFor: aClass ].\r\n\t^encoder","(\r\nfunction(self,aClass)\r\n(function() if ((((((self['@encoder']))==nil)))==true) then return ((function()\r\nreturn ((self):setEncoderFor_(aClass))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@encoder'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"encoder",0
,(
function(self)
(function() if ((((((self['@encoder']))==nil)))==true) then return ((function()
return ((self):setEncoderFor_(smalltalk.Object))
end))() else return nil end end)()
;
return (self['@encoder'])
;
end
)
,"accessing","encoder\r\n\t\" Return the encoder of the receiver. \"\r\n\r\n\tencoder isNil ifTrue: [ self setEncoderFor: Object ].\r\n\t^encoder","(\r\nfunction(self)\r\n(function() if ((((((self['@encoder']))==nil)))==true) then return ((function()\r\nreturn ((self):setEncoderFor_(smalltalk.Object))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@encoder'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"allClasses",0
,(
function(self)
return (((self)["$klass"]):allClasses())
;
end
)
,"accessing","allClasses\r\n\t\" Private - Return all the classes in the system (in fileIn order). \"\r\n\r\n\t^self class allClasses","(\r\nfunction(self)\r\nreturn (((self)[\"$klass\"]):allClasses())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"matchCategory:",0
,(
function(self,aString)
return ((self):matchCategory_from_(aString, (smalltalk.Smalltalk):rootClasses()))
;
end
)
,"categories","matchCategory: aString\r\n\t\" Return the result of filtering all the classes (and orphan methods) in category aString. \"\r\n\r\n\t^self matchCategory: aString from: Smalltalk rootClasses","(\r\nfunction(self,aString)\r\nreturn ((self):matchCategory_from_(aString, (smalltalk.Smalltalk):rootClasses()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"matchCategory:from:",0
,(
function(self,aString,rootClasses)
local selection=nil
local selected=nil
local rejected=nil
local orphanMethods=nil
local fileOutMethods=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	selected=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.Array):new())
	rejected=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.Array):new())
	orphanMethods=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((function() if ((((aString):includes_("-")))==true) then return ((function()
return ((function(each)
return ((smalltalk:isObject_equalTo_((smalltalk:asString_(((each):category()))),aString)))
end))
end))() else return ((function()
return ((function(each)
return ((smalltalk:isObject_equalTo_(((smalltalk:asString_(((each):category()))):upTo_("-")),aString)))
end))
end))() end end)())
	selection=_s8_value; return _s8_value
end)()
;
(rootClasses):do_((function(root)
return ((((root):instanceClass()):withAllSubclasses()):do_((function(each)
return (((function() if ((((selection):value_(each)))==true) then return ((function()
return (selected)
end))() else return ((function()
return (rejected)
end))() end end)()):add_(each))
end)))
end))
;
(function()
	local _s8_value=((function(cls)
return ((cls):methodsDo_((function(aMethod)
return ((function() if ((((selection):value_(aMethod)))==true) then return ((function()
return ((orphanMethods):add_(aMethod))
end))() else return nil end end)())
end)))
end))
	fileOutMethods=_s8_value; return _s8_value
end)()
;
(rejected):do_((function(each)
return ((
function(_s8_rec)
(_s8_rec):value_(each);
return ((_s8_rec):value_((each)["$klass"]))
end
)(fileOutMethods))
end))
;
return ((smalltalk.Array):with_with_with_(selected, rejected, orphanMethods))
;
end
)
,"categories","matchCategory: aString from: rootClasses\r\n\t\" Return the result of filtering all the classes (and orphan methods) in category aString. \"\r\n\r\n\t| selection selected rejected orphanMethods fileOutMethods |\r\n\tselected := Array new.\r\n\trejected := Array new.\r\n\torphanMethods := Array new.\r\n\tselection := (aString includes: $-)\r\n\t\tifTrue: [ [:each| each category asString = aString ] ]\r\n\t\tifFalse: [ [:each| (each category asString upTo: $-) = aString ] ].\r\n\trootClasses do: [:root|\r\n\t\troot instanceClass withAllSubclasses do: [:each|\r\n\t\t\t((selection value: each)\r\n\t\t\t\tifTrue: [ selected ] ifFalse: [ rejected ])\r\n\t\t\t\tadd: each\r\n\t\t]\r\n\t].\r\n\tfileOutMethods := [:cls|\r\n\t\tcls methodsDo: [:aMethod |\r\n\t\t\t(selection value: aMethod) ifTrue: [ orphanMethods add: aMethod ]\r\n\t\t].\r\n\t].\r\n\trejected do: [:each| fileOutMethods value: each; value: each class ].\r\n\t^Array with: selected with: rejected with: orphanMethods","(\r\nfunction(self,aString,rootClasses)\r\nlocal selection=nil\r\nlocal selected=nil\r\nlocal rejected=nil\r\nlocal orphanMethods=nil\r\nlocal fileOutMethods=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tselected=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\trejected=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\torphanMethods=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function() if ((((aString):includes_(\"-\")))==true) then return ((function()\r\nreturn ((function(each)\r\nreturn ((smalltalk:isObject_equalTo_((smalltalk:asString_(((each):category()))),aString)))\r\nend))\r\nend))() else return ((function()\r\nreturn ((function(each)\r\nreturn ((smalltalk:isObject_equalTo_(((smalltalk:asString_(((each):category()))):upTo_(\"-\")),aString)))\r\nend))\r\nend))() end end)())\r\n\tselection=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(rootClasses):do_((function(root)\r\nreturn ((((root):instanceClass()):withAllSubclasses()):do_((function(each)\r\nreturn (((function() if ((((selection):value_(each)))==true) then return ((function()\r\nreturn (selected)\r\nend))() else return ((function()\r\nreturn (rejected)\r\nend))() end end)()):add_(each))\r\nend)))\r\nend))\r\n;\r\n(function()\r\n\tlocal _s8_value=((function(cls)\r\nreturn ((cls):methodsDo_((function(aMethod)\r\nreturn ((function() if ((((selection):value_(aMethod)))==true) then return ((function()\r\nreturn ((orphanMethods):add_(aMethod))\r\nend))() else return nil end end)())\r\nend)))\r\nend))\r\n\tfileOutMethods=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(rejected):do_((function(each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):value_(each);\r\nreturn ((_s8_rec):value_((each)[\"$klass\"]))\r\nend\r\n)(fileOutMethods))\r\nend))\r\n;\r\nreturn ((smalltalk.Array):with_with_with_(selected, rejected, orphanMethods))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutCategories:",0
,(
function(self,categories)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((categories):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):fileOutCategories_((categories):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((((categories):asArray()):inject_into_(smalltalk:newArray({}), (function(total,each)
return ((
function(_s8_rec)
(_s8_rec):add_((self):fileOutCategory_(each));
return ((_s8_rec):yourself())
end
)(total))
end))):join_(""))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"fileOut","fileOutCategories: categories\r\n\t\" Return the result of fileOut all the classes in categories. \"\r\n\r\n\tcategories isString ifTrue: [ ^self fileOutCategories: categories asArrayOfSubstrings ].\r\n\t^(categories asArray inject: #() into: [:total :each|\r\n\t\ttotal add: (self fileOutCategory: each); yourself\r\n\t]) join: ''","(\r\nfunction(self,categories)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((categories):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):fileOutCategories_((categories):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((((categories):asArray()):inject_into_(smalltalk:newArray({}), (function(total,each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_((self):fileOutCategory_(each));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(total))\r\nend))):join_(\"\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutCategory:",0
,(
function(self,aString)
local stream=nil
local match=nil
(function()
	local _s8_value=((""):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):matchCategory_(aString))
	match=_s8_value; return _s8_value
end)()
;
(self):fileOutAll_on_((match):first(), stream)
;
(self):fileOutMethods_on_((match):last(), stream)
;
return ((stream):contents())
;
end
)
,"fileOut","fileOutCategory: aString\r\n\t\" Return the result of fileOut all the classes in category aString. \"\r\n\r\n\t| stream match |\r\n\tstream := '' writeStream.\r\n\tmatch := self matchCategory: aString.\r\n\tself fileOutAll: match first on: stream.\r\n\tself fileOutMethods: match last on: stream.\r\n\t^stream contents","(\r\nfunction(self,aString)\r\nlocal stream=nil\r\nlocal match=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):matchCategory_(aString))\r\n\tmatch=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):fileOutAll_on_((match):first(), stream)\r\n;\r\n(self):fileOutMethods_on_((match):last(), stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOut:",0
,(
function(self,aClass)
local stream=nil
(function()
	local _s8_value=((""):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):fileOut_on_(aClass, stream)
;
return ((stream):contents())
;
end
)
,"fileOut","fileOut: aClass\r\n\t\" Return fileOut definition and code of a class. \"\r\n\r\n\t| stream |\r\n\tstream := '' writeStream.\r\n\tself fileOut: aClass on: stream.\r\n\t^stream contents","(\r\nfunction(self,aClass)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):fileOut_on_(aClass, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutClass:",0
,(
function(self,aClass)
return ((self):fileOut_(aClass))
;
end
)
,"fileOut","fileOutClass: aClass\r\n\t\" Return fileOut definition and code of a class. \"\r\n\r\n\t^self fileOut: aClass","(\r\nfunction(self,aClass)\r\nreturn ((self):fileOut_(aClass))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutAll:",0
,(
function(self,elements)
local stream=nil
(function()
	local _s8_value=((""):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):fileOutAll_on_(elements, stream)
;
return ((stream):contents())
;
end
)
,"fileOut","fileOutAll: elements\r\n\t\" Return fileOut definition and code of elements (classes, methods and/or categories). \"\r\n\r\n\t| stream |\r\n\tstream := '' writeStream.\r\n\tself fileOutAll: elements on: stream.\r\n\t^stream contents","(\r\nfunction(self,elements)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):fileOutAll_on_(elements, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutMethod:of:",0
,(
function(self,aMethod,aClass)
local stream=nil
(function()
	local _s8_value=((""):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):fileOutMethod_of_on_(aMethod, aClass, stream)
;
return ((stream):contents())
;
end
)
,"fileOut","fileOutMethod: aMethod of: aClass\r\n\t\" Return fileOut method of a class. \"\r\n\r\n\t| stream |\r\n\tstream := '' writeStream.\r\n\tself fileOutMethod: aMethod of: aClass on: stream.\r\n\t^stream contents","(\r\nfunction(self,aMethod,aClass)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):fileOutMethod_of_on_(aMethod, aClass, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutAllMethodsOf:on:",0
,(
function(self,aClass,stream)
(self):fileOutMethodsOf_on_((aClass)["$klass"], stream)
;
(self):fileOutMethodsOf_on_(aClass, stream)
;
return (self)
;
end
)
,"fileOut","fileOutAllMethodsOf: aClass on: stream\r\n\t\" FileOut all methods of aClass (and metaclass) onto stream. \"\r\n\r\n\tself fileOutMethodsOf: aClass class on: stream.\r\n\tself fileOutMethodsOf: aClass on: stream.","(\r\nfunction(self,aClass,stream)\r\n(self):fileOutMethodsOf_on_((aClass)[\"$klass\"], stream)\r\n;\r\n(self):fileOutMethodsOf_on_(aClass, stream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutAll:on:",0
,(
function(self,elements,stream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((elements):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):fileOutAll_on_((elements):asArrayOfSubstrings(), stream))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(elements):do_((function(each)
return ((function() if (((smalltalk:is_kindOf_((each),(smalltalk.Behavior))))==true) then return ((function()
return ((self):fileOutDefinitionOfClass_on_(each, stream))
end))() else return nil end end)())
end))
;
(stream):cr()
;
(elements):do_((function(each)
return ((function() if (((smalltalk:is_kindOf_((each),(smalltalk.Behavior))))==true) then return ((function()
return ((self):fileOutAllMethodsOf_on_(each, stream))
end))() else return ((function()
return ((function() if ((((each):isString()))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_((self):fileOutCategory_(each));
return ((_s8_rec):cr())
end
)(stream))
end))() else return ((function()
return ((self):fileOutMethod_of_on_(each, (each):classField(), stream))
end))() end end)())
end))() end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"fileOut","fileOutAll: elements on: stream\r\n\t\" FileOut definition and code of elements (classes, methods and/or categories) onto stream. \"\r\n\r\n\telements isString ifTrue: [ ^self fileOutAll: elements asArrayOfSubstrings on: stream ].\r\n\telements do: [:each|\r\n\t\t(each isKindOf: Behavior) ifTrue: [\r\n\t\t\tself fileOutDefinitionOfClass: each on: stream\r\n\t\t].\r\n\t].\tstream cr.\r\n\telements do: [:each|\r\n\t\t(each isKindOf: Behavior) ifTrue: [\r\n\t\t\tself fileOutAllMethodsOf: each on: stream\r\n\t\t] ifFalse: [\r\n\t\t\teach isString\r\n\t\t\tifTrue: [ stream nextPutAll: (self fileOutCategory: each);cr ]\r\n\t\t\tifFalse: [ self fileOutMethod: each of: each classField on: stream ]\r\n\t\t]\r\n\t].","(\r\nfunction(self,elements,stream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((elements):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):fileOutAll_on_((elements):asArrayOfSubstrings(), stream))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(elements):do_((function(each)\r\nreturn ((function() if (((smalltalk:is_kindOf_((each),(smalltalk.Behavior))))==true) then return ((function()\r\nreturn ((self):fileOutDefinitionOfClass_on_(each, stream))\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(stream):cr()\r\n;\r\n(elements):do_((function(each)\r\nreturn ((function() if (((smalltalk:is_kindOf_((each),(smalltalk.Behavior))))==true) then return ((function()\r\nreturn ((self):fileOutAllMethodsOf_on_(each, stream))\r\nend))() else return ((function()\r\nreturn ((function() if ((((each):isString()))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):fileOutCategory_(each));\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream))\r\nend))() else return ((function()\r\nreturn ((self):fileOutMethod_of_on_(each, (each):classField(), stream))\r\nend))() end end)())\r\nend))() end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutDefinitionOfClass:on:",0
,(
function(self,aClass,stream)
(self):fileOutDefinitionOf_on_(aClass, stream)
;
(self):fileOutDefinitionOf_on_((aClass)["$klass"], stream)
;
return (self)
;
end
)
,"fileOut","fileOutDefinitionOfClass: aClass on: stream\r\n\t\" FileOut definition of aClass (and metaclass) onto stream. \"\r\n\r\n\tself fileOutDefinitionOf: aClass on: stream.\r\n\tself fileOutDefinitionOf: aClass class on: stream","(\r\nfunction(self,aClass,stream)\r\n(self):fileOutDefinitionOf_on_(aClass, stream)\r\n;\r\n(self):fileOutDefinitionOf_on_((aClass)[\"$klass\"], stream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutDefinitionOf:",0
,(
function(self,aClass)
local stream=nil
(function()
	local _s8_value=((""):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):fileOutDefinitionOfClass_on_((aClass):instanceClass(), stream)
;
return ((stream):contents())
;
end
)
,"fileOut","fileOutDefinitionOf: aClass\r\n\t\" Return the definition of a class. \"\r\n\r\n\t| stream |\r\n\tstream := '' writeStream.\r\n\tself fileOutDefinitionOfClass: aClass instanceClass on: stream.\r\n\t^stream contents","(\r\nfunction(self,aClass)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):fileOutDefinitionOfClass_on_((aClass):instanceClass(), stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOut:on:",0
,(
function(self,aClass,stream)
(self):fileOutAll_on_((smalltalk.Array):with_(aClass), stream)
;
return (self)
;
end
)
,"fileOut","fileOut: aClass on: stream\r\n\t\" FileOut definition and code of a class onto stream. \"\r\n\r\n\tself fileOutAll: (Array with: aClass) on: stream.","(\r\nfunction(self,aClass,stream)\r\n(self):fileOutAll_on_((smalltalk.Array):with_(aClass), stream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutDefinitionOf:on:",0
,(
function(self,aClass,stream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aClass):isMetaclass()))==true) then return ((function()
(function() if ((((self):fileOutStrings_of_on_("instanceVariableNames", aClass, stream)))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextChunkPut_("");
return ((_s8_rec):cr())
end
)(stream))
end))() else return nil end end)();(function() if ((((self):fileOutArray_of_on_("propertyNames", aClass, stream)))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextChunkPut_("");
return ((_s8_rec):cr())
end
)(stream))
end))() else return nil end end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_(smalltalk:asString_(((aClass):superclass())));
(_s8_rec):cr();
(_s8_rec):tab();
return ((_s8_rec):nextPutAll_("subclass: #"))
end
)(stream)
;
(function() if ((((self):fileOutStrings_of_on_("instanceVariableNames", aClass, stream)))==true) then return nil else return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_((self):globalNameFor_(aClass));
(_s8_rec):cr();
(_s8_rec):tab();
(_s8_rec):nextPutAll_("instanceVariableNames: ");
return ((_s8_rec):nextPutAll_(smalltalk:printString_((""))))
end
)(stream))
end))() end end)()
;
(function() if (((((((aClass):classVariables()))~=nil)))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):cr();
(_s8_rec):tab();
(_s8_rec):nextPutAll_("classVariableNames: ");
return ((_s8_rec):nextPutAll_(smalltalk:printString_(((aClass):classVariablesString()))))
end
)(stream))
end))() else return nil end end)()
;
(function() if (((((((aClass):poolDictionaries()))~=nil)))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):cr();
(_s8_rec):tab();
(_s8_rec):nextPutAll_("poolDictionaries: ");
return ((_s8_rec):nextPutAll_(smalltalk:printString_(((aClass):poolDictionariesString()))))
end
)(stream))
end))() else return nil end end)()
;
(
function(_s8_rec)
(_s8_rec):cr();
(_s8_rec):tab();
(_s8_rec):nextPutAll_("category: ");
return ((_s8_rec):nextPutAll_(((aClass):category()):asLiteral()))
end
)(stream)
;
(
function(_s8_rec)
(_s8_rec):nextChunkPut_("");
return ((_s8_rec):cr())
end
)(stream)
;
(function() if ((((self):fileOutArray_of_on_("propertyNames", aClass, stream)))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextChunkPut_("");
return ((_s8_rec):cr())
end
)(stream))
end))() else return nil end end)()
;
(function() if ((((smalltalk:boolean_and_((((((aClass):comment()))~=nil)),(function()
return (((aClass):comment()):notEmpty())
end)))))==true) then return ((function()
(stream):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(((self):globalNameFor_(aClass))," comment: ")),smalltalk:printString_(((aClass):comment()))));return ((
function(_s8_rec)
(_s8_rec):nextChunkPut_("");
return ((_s8_rec):cr())
end
)(stream))
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"fileOut","fileOutDefinitionOf: aClass on: stream\r\n\t\" FileOut definition of a class onto stream. \"\r\n\r\n\taClass isMetaclass ifTrue: [\r\n\t\t(self fileOutStrings: #instanceVariableNames of: aClass on: stream) ifTrue: [\r\n\t\t\tstream nextChunkPut: '';cr.\r\n\t\t].\r\n\t\t(self fileOutArray: #propertyNames of: aClass on: stream) ifTrue: [\r\n\t\t\tstream nextChunkPut: '';cr.\r\n\t\t].\r\n\t\t^self\r\n\t].\r\n\r\n\tstream\r\n\t\tnextPutAll: aClass superclass asString; cr;tab;\r\n\t\tnextPutAll: 'subclass: #'.\r\n\t(self fileOutStrings: #instanceVariableNames of: aClass on: stream) ifFalse: [\r\n\t\tstream\r\n\t\t\tnextPutAll: (self globalNameFor: aClass); cr;tab;\r\n\t\t\tnextPutAll: 'instanceVariableNames: ';\r\n\t\t\tnextPutAll: '' printString\r\n\t].\r\n\taClass classVariables notNil ifTrue: [\r\n\t    stream\t cr;tab;\r\n\t\tnextPutAll: 'classVariableNames: ';\r\n\t\tnextPutAll: aClass classVariablesString printString\r\n\t].\r\n\taClass poolDictionaries notNil ifTrue: [\r\n\t    stream\t cr;tab;\r\n\t\tnextPutAll: 'poolDictionaries: ';\r\n\t\tnextPutAll: aClass poolDictionariesString printString\r\n\t].\r\n\tstream cr;tab;\r\n\t\tnextPutAll: 'category: ';\r\n\t\tnextPutAll: aClass category asLiteral.\r\n\tstream nextChunkPut: '';cr.\r\n\t(self fileOutArray: #propertyNames of: aClass on: stream) ifTrue: [\r\n\t\tstream nextChunkPut: '';cr.\r\n\t].\r\n\t(aClass comment notNil and: [aClass comment notEmpty]) ifTrue: [\r\n\t\tstream nextPutAll: (self globalNameFor: aClass),' comment: ', aClass comment printString.\r\n\t\tstream nextChunkPut: '';cr.\r\n\t].","(\r\nfunction(self,aClass,stream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aClass):isMetaclass()))==true) then return ((function()\r\n(function() if ((((self):fileOutStrings_of_on_(\"instanceVariableNames\", aClass, stream)))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextChunkPut_(\"\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream))\r\nend))() else return nil end end)();(function() if ((((self):fileOutArray_of_on_(\"propertyNames\", aClass, stream)))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextChunkPut_(\"\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream))\r\nend))() else return nil end end)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(smalltalk:asString_(((aClass):superclass())));\r\n(_s8_rec):cr();\r\n(_s8_rec):tab();\r\nreturn ((_s8_rec):nextPutAll_(\"subclass: #\"))\r\nend\r\n)(stream)\r\n;\r\n(function() if ((((self):fileOutStrings_of_on_(\"instanceVariableNames\", aClass, stream)))==true) then return nil else return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):globalNameFor_(aClass));\r\n(_s8_rec):cr();\r\n(_s8_rec):tab();\r\n(_s8_rec):nextPutAll_(\"instanceVariableNames: \");\r\nreturn ((_s8_rec):nextPutAll_(smalltalk:printString_((\"\"))))\r\nend\r\n)(stream))\r\nend))() end end)()\r\n;\r\n(function() if (((((((aClass):classVariables()))~=nil)))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\n(_s8_rec):tab();\r\n(_s8_rec):nextPutAll_(\"classVariableNames: \");\r\nreturn ((_s8_rec):nextPutAll_(smalltalk:printString_(((aClass):classVariablesString()))))\r\nend\r\n)(stream))\r\nend))() else return nil end end)()\r\n;\r\n(function() if (((((((aClass):poolDictionaries()))~=nil)))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\n(_s8_rec):tab();\r\n(_s8_rec):nextPutAll_(\"poolDictionaries: \");\r\nreturn ((_s8_rec):nextPutAll_(smalltalk:printString_(((aClass):poolDictionariesString()))))\r\nend\r\n)(stream))\r\nend))() else return nil end end)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\n(_s8_rec):tab();\r\n(_s8_rec):nextPutAll_(\"category: \");\r\nreturn ((_s8_rec):nextPutAll_(((aClass):category()):asLiteral()))\r\nend\r\n)(stream)\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextChunkPut_(\"\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream)\r\n;\r\n(function() if ((((self):fileOutArray_of_on_(\"propertyNames\", aClass, stream)))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextChunkPut_(\"\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream))\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_((((((aClass):comment()))~=nil)),(function()\r\nreturn (((aClass):comment()):notEmpty())\r\nend)))))==true) then return ((function()\r\n(stream):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(((self):globalNameFor_(aClass)),\" comment: \")),smalltalk:printString_(((aClass):comment()))));return ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextChunkPut_(\"\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream))\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutStrings:of:on:",0
,(
function(self,selector,aClass,stream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local strings=nil
(function()
	local _s8_value=((aClass):perform_(selector))
	strings=_s8_value; return _s8_value
end)()
;
(function() if ((((strings):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):globalNameFor_(aClass));
(_s8_rec):cr();
(_s8_rec):tab();
return ((_s8_rec):nextPutAll_(smalltalk:object_comma_((selector),": '")))
end
)(stream)
;
(strings):do_((function(each)
return ((stream):nextPutAll_(smalltalk:object_comma_((" "),each)))
end))
;
(stream):nextPutAll_(" '")
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"fileOut","fileOutStrings: selector of: aClass on: stream\r\n\t\" Private - FileOut selector strings of a class onto stream.\r\n\tReturn true if code has been filedOut.\r\n\t\"\r\n\r\n\t| strings |\r\n\tstrings := aClass perform: selector.\r\n\tstrings isEmpty ifTrue: [ ^false ].\r\n\tstream\r\n\t\tnextPutAll: (self globalNameFor: aClass); cr;tab;\r\n\t\tnextPutAll: selector,': '''.\r\n\tstrings do: [:each| stream nextPutAll: ' ',each ].\r\n\tstream nextPutAll: ' '''.\r\n\t^true","(\r\nfunction(self,selector,aClass,stream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal strings=nil\r\n(function()\r\n\tlocal _s8_value=((aClass):perform_(selector))\r\n\tstrings=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((strings):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):globalNameFor_(aClass));\r\n(_s8_rec):cr();\r\n(_s8_rec):tab();\r\nreturn ((_s8_rec):nextPutAll_(smalltalk:object_comma_((selector),\": '\")))\r\nend\r\n)(stream)\r\n;\r\n(strings):do_((function(each)\r\nreturn ((stream):nextPutAll_(smalltalk:object_comma_((\" \"),each)))\r\nend))\r\n;\r\n(stream):nextPutAll_(\" '\")\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutArray:of:on:",0
,(
function(self,selector,aClass,stream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local strings=nil
(function()
	local _s8_value=((aClass):perform_(selector))
	strings=_s8_value; return _s8_value
end)()
;
(function() if ((((strings):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_((self):globalNameFor_(aClass));
return ((_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((" "),selector)),": ")),(strings):asLiteral())))
end
)(stream)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"fileOut","fileOutArray: selector of: aClass on: stream\r\n\t\" Private - FileOut selector arrayed strings of a class onto stream.\r\n\tReturn true if code has been filedOut.\r\n\t\"\r\n\r\n\t| strings |\r\n\tstrings := aClass perform: selector.\r\n\tstrings isEmpty ifTrue: [ ^false ].\r\n\tstream\r\n\t\tnextPutAll: (self globalNameFor: aClass);\r\n\t\tnextPutAll: ' ',selector,': ', strings asLiteral.\r\n\t^true","(\r\nfunction(self,selector,aClass,stream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal strings=nil\r\n(function()\r\n\tlocal _s8_value=((aClass):perform_(selector))\r\n\tstrings=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((strings):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):globalNameFor_(aClass));\r\nreturn ((_s8_rec):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\" \"),selector)),\": \")),(strings):asLiteral())))\r\nend\r\n)(stream)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutMethodsOf:on:",0
,(
function(self,aClass,stream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local selectors=nil
(function()
	local _s8_value=((aClass):selectors())
	selectors=_s8_value; return _s8_value
end)()
;
(function() if ((((selectors):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(stream):cr()
;
(self):fileOutSelectors_of_on_(selectors, aClass, stream)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"fileOut","fileOutMethodsOf: aClass on: stream\r\n\t\" FileOut methods of a class onto stream. \"\r\n\r\n\t| selectors |\r\n\tselectors := aClass selectors.\r\n\tselectors isEmpty ifTrue: [ ^self ].\r\n\tstream cr.\r\n\tself\r\n\t\tfileOutSelectors: selectors\r\n\t\tof: aClass on: stream","(\r\nfunction(self,aClass,stream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal selectors=nil\r\n(function()\r\n\tlocal _s8_value=((aClass):selectors())\r\n\tselectors=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((selectors):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(stream):cr()\r\n;\r\n(self):fileOutSelectors_of_on_(selectors, aClass, stream)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutClusters",0
,(
function(self)
return (((smalltalk:newArray({"APIBuilder"})):select_((function(key)
return ((smalltalk.Smalltalk):includesKey_(key))
end))):collect_((function(each)
return (((smalltalk.Smalltalk):at_(each)):default())
end)))
;
end
)
,"private","fileOutClusters\r\n\t\" Private - Return the fileOut chustering dumpers of the receiver.\r\n\tA fileOutCluster is an object that potentially dump clustered actions for selectors implemented by aClass.\r\n\tThe fileOut write (smalltalk)code for multiple selectors removing them from selectors collection.\r\n\tNote: the selectors collection is modified during fileOut.\r\n\tThe default answer is an empty collection of custed or a collection with default APIBuilder.\r\n\tThis method can be refined by the subclasses.\r\n\t\"\r\n\r\n\t^(#( #APIBuilder )\r\n\t\tselect: [:key| Smalltalk includesKey: key ])\r\n\t\tcollect: [:each| (Smalltalk at: each) default ]","(\r\nfunction(self)\r\nreturn (((smalltalk:newArray({\"APIBuilder\"})):select_((function(key)\r\nreturn ((smalltalk.Smalltalk):includesKey_(key))\r\nend))):collect_((function(each)\r\nreturn (((smalltalk.Smalltalk):at_(each)):default())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutSelectors:of:on:",0
,(
function(self,selectors,aClass,stream)
((self):fileOutClusters()):do_((function(each)
return ((each):fileOut_selectors_of_on_(self, selectors, aClass, stream))
end))
;
(self):fileOutMethods_on_((selectors):collect_((function(each)
return ((aClass):compiledMethodAt_(each))
end)), stream)
;
return (self)
;
end
)
,"fileOut","fileOutSelectors: selectors of: aClass on: stream\r\n\t\" FileOut methods implementing selectors of a class onto stream. \"\r\n\r\n\tself fileOutClusters do: [:each|\r\n\t\teach\r\n\t\t\tfileOut: self selectors: selectors\r\n\t\t\tof: aClass on: stream\r\n\t].\r\n\tself\tfileOutMethods: (selectors collect: [:each|\r\n\t\t\taClass compiledMethodAt: each ])\r\n\t\ton: stream","(\r\nfunction(self,selectors,aClass,stream)\r\n((self):fileOutClusters()):do_((function(each)\r\nreturn ((each):fileOut_selectors_of_on_(self, selectors, aClass, stream))\r\nend))\r\n;\r\n(self):fileOutMethods_on_((selectors):collect_((function(each)\r\nreturn ((aClass):compiledMethodAt_(each))\r\nend)), stream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutMethods:on:",0
,(
function(self,array,stream)
local sorted=nil
(function()
	local _s8_value=((array):sorted_((function(a,b)
return ((smalltalk:boolean_or_(((((a):category()) < (b):category())),(function()
return ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((a):category()),(b):category()))),(function()
return ((((a):selector()) < (b):selector()))
end))))
end))))
end)))
	sorted=_s8_value; return _s8_value
end)()
;
(sorted):do_((function(each)
return ((self):fileOutMethod_of_on_(each, (each):classField(), stream))
end))
;
return (self)
;
end
)
,"fileOut","fileOutMethods: array on: stream\r\n\t\" FileOut array of methods of a class onto stream. \"\r\n\r\n\t| sorted |\r\n\tsorted := array sorted: [:a :b|\r\n\t\ta category < b category\r\n\t\tor: [ a category = b category\r\n\t\tand: [ a selector < b selector ]]\r\n\t].\r\n\r\n\tsorted do: [:each|\r\n\t\tself\tfileOutMethod: each\r\n\t\t\tof: each classField on: stream\r\n\t].","(\r\nfunction(self,array,stream)\r\nlocal sorted=nil\r\n(function()\r\n\tlocal _s8_value=((array):sorted_((function(a,b)\r\nreturn ((smalltalk:boolean_or_(((((a):category()) < (b):category())),(function()\r\nreturn ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((a):category()),(b):category()))),(function()\r\nreturn ((((a):selector()) < (b):selector()))\r\nend))))\r\nend))))\r\nend)))\r\n\tsorted=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(sorted):do_((function(each)\r\nreturn ((self):fileOutMethod_of_on_(each, (each):classField(), stream))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"isASymbol:",0
,(
function(self,aString)
return ((((((smalltalk.RegularExpression):fromString_("[^a-zA-Z0-9_:]")):exec_(aString)))==nil))
;
end
)
,"private","isASymbol: aString\r\n\t\" Private - Return true if aString can be output with symbol notation. \"\r\n\r\n\t^((RegularExpression fromString: '[^a-zA-Z0-9_:]') exec: aString) isNil","(\r\nfunction(self,aString)\r\nreturn ((((((smalltalk.RegularExpression):fromString_(\"[^a-zA-Z0-9_:]\")):exec_(aString)))==nil))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutMethod:on:",0
,(
function(self,aMethod,aStream)
return ((self):fileOutMethod_of_on_(aMethod, (aMethod):classField(), aStream))
;
end
)
,"fileOut","fileOutMethod: aMethod on: aStream\r\n\t\" Return the fileOut of aMethod. \"\r\n\r\n\t^self\r\n\t\tfileOutMethod: aMethod\r\n\t\tof: aMethod classField\r\n\t\ton: aStream","(\r\nfunction(self,aMethod,aStream)\r\nreturn ((self):fileOutMethod_of_on_(aMethod, (aMethod):classField(), aStream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutMethod:",0
,(
function(self,aMethod)
local stream=nil
(function()
	local _s8_value=((""):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):fileOutMethod_on_(aMethod, stream)
;
return ((stream):contents())
;
end
)
,"fileOut","fileOutMethod: aMethod\r\n\t\" Return the fileOut of aMethod. \"\r\n\r\n\t| stream |\r\n\tstream := '' writeStream.\r\n\tself fileOutMethod: aMethod on: stream.\r\n\t^stream contents","(\r\nfunction(self,aMethod)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):fileOutMethod_on_(aMethod, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"fileOutMethod:of:on:",0
,(
function(self,aMethod,aClass,stream)
(
function(_s8_rec)
(_s8_rec):cr();
(_s8_rec):nextChunkPut_(nil);
(_s8_rec):nextChunkPut_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((" "),(self):globalNameFor_(aClass)))," methodsFor: ")),((aMethod):category()):asLiteral()))," "));
(_s8_rec):cr();
(_s8_rec):nextChunkPut_((aMethod):source());
(_s8_rec):nextChunkPut_(" ");
return ((_s8_rec):cr())
end
)(stream)
;
return (self)
;
end
)
,"fileOut","fileOutMethod: aMethod of: aClass on: stream\r\n\t\" FileOut method of a class onto stream. \"\r\n\r\n\tstream\tcr; nextChunkPut: nil;\r\n\t\tnextChunkPut: ' '\r\n\t\t\t,(self globalNameFor: aClass)\r\n\t\t\t,' methodsFor: ' ,aMethod category asLiteral\r\n\t\t\t,' '; cr;\r\n\t\tnextChunkPut: aMethod source;\r\n\t\tnextChunkPut: ' '; cr","(\r\nfunction(self,aMethod,aClass,stream)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\n(_s8_rec):nextChunkPut_(nil);\r\n(_s8_rec):nextChunkPut_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\" \"),(self):globalNameFor_(aClass))),\" methodsFor: \")),((aMethod):category()):asLiteral())),\" \"));\r\n(_s8_rec):cr();\r\n(_s8_rec):nextChunkPut_((aMethod):source());\r\n(_s8_rec):nextChunkPut_(\" \");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportCategory:",0
,(
function(self,aString)
local stream=nil
local match=nil
(function()
	local _s8_value=((""):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):matchCategory_(aString))
	match=_s8_value; return _s8_value
end)()
;
((match):first()):do_((function(each)
return ((stream):nextPutAll_((self):export_(each)))
end))
;
((match):last()):do_((function(aMethod)
return ((self):export_method_on_((aMethod):classField(), aMethod, stream))
end))
;
return ((stream):contents())
;
end
)
,"export","exportCategory: aString\r\n\t\" Return the result of exporting all the classes in category aString. \"\r\n\r\n\t| stream match |\r\n\tstream := '' writeStream.\r\n\tmatch := self matchCategory: aString.\r\n\tmatch first do: [:each| stream nextPutAll: (self export: each) ].\r\n\tmatch last do: [:aMethod |\r\n\t\tself export: aMethod classField method: aMethod on: stream\r\n\t].\r\n\t^stream contents","(\r\nfunction(self,aString)\r\nlocal stream=nil\r\nlocal match=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):matchCategory_(aString))\r\n\tmatch=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((match):first()):do_((function(each)\r\nreturn ((stream):nextPutAll_((self):export_(each)))\r\nend))\r\n;\r\n((match):last()):do_((function(aMethod)\r\nreturn ((self):export_method_on_((aMethod):classField(), aMethod, stream))\r\nend))\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportCategories:",0
,(
function(self,categories)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((categories):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):exportCategories_((categories):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((((categories):asArray()):inject_into_(smalltalk:newArray({}), (function(total,each)
return ((
function(_s8_rec)
(_s8_rec):add_((self):exportCategory_(each));
return ((_s8_rec):yourself())
end
)(total))
end))):join_(""))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"export","exportCategories: categories\r\n\t\" Return the result of exporting all the classes in categories. \"\r\n\r\n\tcategories isString ifTrue: [ ^self exportCategories: categories asArrayOfSubstrings ].\r\n\t^(categories asArray inject: #() into: [:total :each|\r\n\t\ttotal add: (self exportCategory: each); yourself\r\n\t]) join: ''","(\r\nfunction(self,categories)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((categories):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):exportCategories_((categories):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((((categories):asArray()):inject_into_(smalltalk:newArray({}), (function(total,each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_((self):exportCategory_(each));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(total))\r\nend))):join_(\"\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportAllMethodsOf:on:",0
,(
function(self,aClass,stream)
(self):exportMethodsOf_on_((aClass)["$klass"], stream)
;
(self):exportMethodsOf_on_(aClass, stream)
;
return (self)
;
end
)
,"export","exportAllMethodsOf: aClass on: stream\r\n\t\" Export all methods of a class (and metaclass) onto stream. \"\r\n\r\n\tself exportMethodsOf: aClass class on: stream.\r\n\tself exportMethodsOf: aClass on: stream.","(\r\nfunction(self,aClass,stream)\r\n(self):exportMethodsOf_on_((aClass)[\"$klass\"], stream)\r\n;\r\n(self):exportMethodsOf_on_(aClass, stream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportAllMethodsOn:",0
,(
function(self,stream)
((self):allClasses()):do_((function(aClass)
return ((self):exportAllMethodsOf_on_(aClass, stream))
end))
;
return (self)
;
end
)
,"export","exportAllMethodsOn: stream\r\n\t\" Export all methods in the system onto stream. \"\r\n\r\n\tself allClasses do: [:aClass|\r\n\t\tself exportAllMethodsOf: aClass on: stream\r\n\t]","(\r\nfunction(self,stream)\r\n((self):allClasses()):do_((function(aClass)\r\nreturn ((self):exportAllMethodsOf_on_(aClass, stream))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportAllMethods",0
,(
function(self)
local stream=nil
(function()
	local _s8_value=((""):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):exportAllMethodsOn_(stream)
;
return ((stream):contents())
;
end
)
,"export","exportAllMethods\r\n\t\" Return the result of exporting all methods in the system. \"\r\n\r\n\t| stream |\r\n\tstream := '' writeStream.\r\n\tself exportAllMethodsOn: stream.\r\n\t^stream contents","(\r\nfunction(self)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):exportAllMethodsOn_(stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportAllDefinitionsOn:",0
,(
function(self,stream)
((self):allClasses()):do_((function(aClass)
return ((self):exportDefinitionOfClass_on_(aClass, stream))
end))
;
return (self)
;
end
)
,"export","exportAllDefinitionsOn: stream\r\n\t\" Export all class definitions in the system onto stream. \"\r\n\r\n\tself allClasses do: [:aClass|\r\n\t\tself exportDefinitionOfClass: aClass on: stream\r\n\t]","(\r\nfunction(self,stream)\r\n((self):allClasses()):do_((function(aClass)\r\nreturn ((self):exportDefinitionOfClass_on_(aClass, stream))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportAllDefinitions",0
,(
function(self)
local stream=nil
(function()
	local _s8_value=((""):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):exportAllDefinitionsOn_(stream)
;
return ((stream):contents())
;
end
)
,"export","exportAllDefinitions\r\n\t\" Return the result of exporting all class definitions in the system. \"\r\n\r\n\t| stream |\r\n\tstream := '' writeStream.\r\n\tself exportAllDefinitionsOn: stream.\r\n\t^stream contents","(\r\nfunction(self)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):exportAllDefinitionsOn_(stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportDefinitionOfClass:on:",0
,(
function(self,aClass,stream)
(self):exportDefinitionOf_on_(aClass, stream)
;
(self):exportMetaDefinitionOf_on_(aClass, stream)
;
return (self)
;
end
)
,"export","exportDefinitionOfClass: aClass on: stream\r\n\t\" Export definition of aClass (and metaclass) onto stream. \"\r\n\r\n\tself exportDefinitionOf: aClass on: stream.\r\n\tself exportMetaDefinitionOf: aClass on: stream.","(\r\nfunction(self,aClass,stream)\r\n(self):exportDefinitionOf_on_(aClass, stream)\r\n;\r\n(self):exportMetaDefinitionOf_on_(aClass, stream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"export:",0
,(
function(self,aClass)
local stream=nil
(function()
	local _s8_value=((""):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):exportDefinitionOfClass_on_(aClass, stream)
;
(self):exportAllMethodsOf_on_(aClass, stream)
;
return ((stream):contents())
;
end
)
,"export","export: aClass\r\n\t\" Return the result of exporting definition and source code of a class. \"\r\n\r\n\t| stream |\r\n\tstream := '' writeStream.\r\n\tself exportDefinitionOfClass: aClass on: stream.\r\n\tself exportAllMethodsOf: aClass on: stream.\r\n\t^stream contents","(\r\nfunction(self,aClass)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):exportDefinitionOfClass_on_(aClass, stream)\r\n;\r\n(self):exportAllMethodsOf_on_(aClass, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportNew:",0
,(
function(self,aClass)
return ((self):exportDefinitionOf_(aClass))
;
end
)
,"export","exportNew: aClass\r\n\t\" Return the result of exporting definition of aClass. \"\r\n\r\n\t^self exportDefinitionOf: aClass","(\r\nfunction(self,aClass)\r\nreturn ((self):exportDefinitionOf_(aClass))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportDefinitionOf:on:",0
,(
function(self,aClass,aStream)
return (((self):encoderFor_(aClass)):exportDefinitionOf_on_(aClass, aStream))
;
end
)
,"private","exportDefinitionOf: aClass on: aStream\r\n\t\" Export definition of aClass onto aStream. \"\r\n\r\n\t^(self encoderFor: aClass) exportDefinitionOf: aClass on: aStream","(\r\nfunction(self,aClass,aStream)\r\nreturn (((self):encoderFor_(aClass)):exportDefinitionOf_on_(aClass, aStream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportMetaDefinitionOf:on:",0
,(
function(self,aClass,aStream)
return (((self):encoderFor_(aClass)):exportMetaDefinitionOf_on_(aClass, aStream))
;
end
)
,"private","exportMetaDefinitionOf: aClass on: aStream\r\n\t\" Export class side definition of aClass onto aStream. \"\r\n\r\n\t^(self encoderFor: aClass) exportMetaDefinitionOf: aClass on: aStream","(\r\nfunction(self,aClass,aStream)\r\nreturn (((self):encoderFor_(aClass)):exportMetaDefinitionOf_on_(aClass, aStream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"smalltalkNameFor:",0
,(
function(self,aClass)
return (((self):encoderFor_(aClass)):smalltalkNameFor_(aClass))
;
end
)
,"private","smalltalkNameFor: aClass\r\n\t\" Private - Return the native reference expression of aClass (or nil). \"\r\n\r\n\t^(self encoderFor: aClass) smalltalkNameFor: aClass","(\r\nfunction(self,aClass)\r\nreturn (((self):encoderFor_(aClass)):smalltalkNameFor_(aClass))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"globalNameFor:",0
,(
function(self,aClass)
return ((function() if ((((((aClass))==nil)))==true) then return ((function()
return ("nil")
end))() else return ((function()
return ((aClass):name())
end))() end end)())
;
end
)
,"private","globalNameFor: aClass\r\n\t\" Private - Return the smalltalk name of aClass (or nil). \"\r\n\r\n\t^aClass isNil ifTrue: [ #nil ] ifFalse: [ aClass name ]","(\r\nfunction(self,aClass)\r\nreturn ((function() if ((((((aClass))==nil)))==true) then return ((function()\r\nreturn (\"nil\")\r\nend))() else return ((function()\r\nreturn ((aClass):name())\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportAll",0
,(
function(self)
return ((self):exportClasses_((self):allClasses()))
;
end
)
,"export","exportAll\r\n\t\" Return the result of exporting all classes in the system. \"\r\n\r\n\t^self exportClasses: self allClasses","(\r\nfunction(self)\r\nreturn ((self):exportClasses_((self):allClasses()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportAllCategories",0
,(
function(self)
local categories=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	categories=_s8_value; return _s8_value
end)()
;
((self):allClasses()):do_((function(each)
return ((function() if ((((categories):includes_((each):category())))==true) then return nil else return ((function()
return ((categories):add_((each):category()))
end))() end end)())
end))
;
return (((categories):inject_into_(smalltalk:newArray({}), (function(total,each)
return ((
function(_s8_rec)
(_s8_rec):add_((self):exportCategory_(each));
return ((_s8_rec):yourself())
end
)(total))
end))):join_(""))
;
end
)
,"export","exportAllCategories\r\n\t\" Return the result of exporting all categories in the system. \"\r\n\r\n\t| categories |\r\n\tcategories := Array new.\r\n\tself allClasses do: [:each |\r\n\t\t(categories includes: each category) ifFalse: [\r\n\t\t\tcategories add: each category\r\n\t\t]\r\n\t].\r\n\t^(categories inject: #() into: [:total :each | total add: (self exportCategory: each); yourself ]) join: ''","(\r\nfunction(self)\r\nlocal categories=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tcategories=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((self):allClasses()):do_((function(each)\r\nreturn ((function() if ((((categories):includes_((each):category())))==true) then return nil else return ((function()\r\nreturn ((categories):add_((each):category()))\r\nend))() end end)())\r\nend))\r\n;\r\nreturn (((categories):inject_into_(smalltalk:newArray({}), (function(total,each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):add_((self):exportCategory_(each));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(total))\r\nend))):join_(\"\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportDefinitionOf:",0
,(
function(self,aClass)
local stream=nil
(function()
	local _s8_value=(((smalltalk.String):new()):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):exportDefinitionOf_on_(aClass, stream)
;
return ((stream):contents())
;
end
)
,"export","exportDefinitionOf: aClass\r\n\t\" Return the result of exporting definition of aClass. \"\r\n\r\n\t| stream |\r\n\tstream := String new writeStream.\r\n\tself exportDefinitionOf: aClass on: stream.\r\n\t^stream contents","(\r\nfunction(self,aClass)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.String):new()):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):exportDefinitionOf_on_(aClass, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportMetaDefinitionOf:",0
,(
function(self,aClass)
local stream=nil
(function()
	local _s8_value=(((smalltalk.String):new()):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):exportMetaDefinitionOf_on_(aClass, stream)
;
return ((stream):contents())
;
end
)
,"export","exportMetaDefinitionOf: aClass\r\n\t\" Return the result of exporting class side definition of aClass. \"\r\n\r\n\t| stream |\r\n\tstream := String new writeStream.\r\n\tself exportMetaDefinitionOf: aClass on: stream.\r\n\t^stream contents","(\r\nfunction(self,aClass)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.String):new()):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):exportMetaDefinitionOf_on_(aClass, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportClasses:on:",0
,(
function(self,classes,stream)
return ((self):exportAll_on_(classes, stream))
;
end
)
,"export","exportClasses: classes on: stream\r\n\t\" Return the result of exporting class definitions onto stream. \"\r\n\r\n\t^self exportAll: classes on: stream","(\r\nfunction(self,classes,stream)\r\nreturn ((self):exportAll_on_(classes, stream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportAll:on:",0
,(
function(self,elements,stream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((elements):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):exportAll_on_((elements):asArrayOfSubstrings(), stream))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(elements):do_((function(each)
return ((function() if (((smalltalk:is_kindOf_((each),(smalltalk.Behavior))))==true) then return ((function()
return ((self):exportDefinitionOfClass_on_(each, stream))
end))() else return nil end end)())
end))
;
(stream):cr()
;
(elements):do_((function(each)
return ((function() if (((smalltalk:is_kindOf_((each),(smalltalk.Behavior))))==true) then return ((function()
return ((self):exportAllMethodsOf_on_(each, stream))
end))() else return ((function()
return ((function() if ((((each):isString()))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):nextPutAll_((self):exportCategory_(each));
return ((_s8_rec):cr())
end
)(stream))
end))() else return ((function()
return ((self):exportMethod_on_(each, stream))
end))() end end)())
end))() end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"export","exportAll: elements on: stream\r\n\t\" Return the result of exporting elements (classes, methods and/or categories) onto stream. \"\r\n\r\n\telements isString ifTrue: [ ^self exportAll: elements asArrayOfSubstrings on: stream ].\r\n\telements do: [:each|\r\n\t\t(each isKindOf: Behavior) ifTrue: [\r\n\t\t\tself exportDefinitionOfClass: each on: stream\r\n\t\t].\r\n\t].\r\n\tstream cr.\r\n\telements do: [:each|\r\n\t\t(each isKindOf: Behavior) ifTrue: [\r\n\t\t\tself exportAllMethodsOf: each on: stream\r\n\t\t] ifFalse: [\r\n\t\t\teach isString\r\n\t\t\tifTrue: [ stream nextPutAll: (self exportCategory: each);cr ]\r\n\t\t\tifFalse: [ self exportMethod: each on: stream ]\r\n\t\t]\r\n\t].","(\r\nfunction(self,elements,stream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((elements):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):exportAll_on_((elements):asArrayOfSubstrings(), stream))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(elements):do_((function(each)\r\nreturn ((function() if (((smalltalk:is_kindOf_((each),(smalltalk.Behavior))))==true) then return ((function()\r\nreturn ((self):exportDefinitionOfClass_on_(each, stream))\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(stream):cr()\r\n;\r\n(elements):do_((function(each)\r\nreturn ((function() if (((smalltalk:is_kindOf_((each),(smalltalk.Behavior))))==true) then return ((function()\r\nreturn ((self):exportAllMethodsOf_on_(each, stream))\r\nend))() else return ((function()\r\nreturn ((function() if ((((each):isString()))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_((self):exportCategory_(each));\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream))\r\nend))() else return ((function()\r\nreturn ((self):exportMethod_on_(each, stream))\r\nend))() end end)())\r\nend))() end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportAll:",0
,(
function(self,elements)
local stream=nil
(function()
	local _s8_value=(((smalltalk.String):new()):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):exportAll_on_(elements, stream)
;
return ((stream):contents())
;
end
)
,"export","exportAll: elements\r\n\t\" Return the result of exporting elements (classes, methods and/or categories). \"\r\n\r\n\t| stream |\r\n\tstream := String new writeStream.\r\n\tself exportAll: elements on: stream.\r\n\t^stream contents","(\r\nfunction(self,elements)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.String):new()):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):exportAll_on_(elements, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportClasses:",0
,(
function(self,classes)
local stream=nil
(function()
	local _s8_value=(((smalltalk.String):new()):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):exportClasses_on_(classes, stream)
;
return ((stream):contents())
;
end
)
,"export","exportClasses: classes\r\n\t\" Return the result of exporting classes. \"\r\n\r\n\t| stream |\r\n\tstream := String new writeStream.\r\n\tself exportClasses: classes on: stream.\r\n\t^stream contents","(\r\nfunction(self,classes)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.String):new()):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):exportClasses_on_(classes, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportObject:on:",0
,(
function(self,anObject,stream)
(stream):nextPutAll_((anObject):asNative())
;
return (self)
;
end
)
,"export","exportObject: anObject on: stream\r\n\t\" Export a (native)literal representation of anObject onto stream. \"\r\n\r\n\tstream nextPutAll: anObject asNative","(\r\nfunction(self,anObject,stream)\r\n(stream):nextPutAll_((anObject):asNative())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportObject:",0
,(
function(self,anObject)
local stream=nil
(function()
	local _s8_value=(((smalltalk.String):new()):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):exportObject_on_(anObject, stream)
;
return ((stream):contents())
;
end
)
,"export","exportObject: anObject\r\n\t\" Return the result of exporting a literal representation of anObject. \"\r\n\r\n\t| stream |\r\n\tstream := String new writeStream.\r\n\tself exportObject: anObject on: stream.\r\n\t^stream contents","(\r\nfunction(self,anObject)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.String):new()):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):exportObject_on_(anObject, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"export:method:",0
,(
function(self,aClass,aMethod)
local stream=nil
(function()
	local _s8_value=(((smalltalk.String):new()):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):export_method_on_(aClass, aMethod, stream)
;
return ((stream):contents())
;
end
)
,"export","export: aClass method: aMethod\r\n\t\" Return the result of exporting aClass method. \"\r\n\r\n\t| stream |\r\n\tstream := String new writeStream.\r\n\tself export: aClass method: aMethod on: stream.\r\n\t^stream contents","(\r\nfunction(self,aClass,aMethod)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.String):new()):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):export_method_on_(aClass, aMethod, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportMethod:",0
,(
function(self,aMethod)
return ((self):export_method_((aMethod):classField(), aMethod))
;
end
)
,"export","exportMethod: aMethod\r\n\t\" Return the result of exporting aMethod. \"\r\n\r\n\t^self export: aMethod classField method: aMethod","(\r\nfunction(self,aMethod)\r\nreturn ((self):export_method_((aMethod):classField(), aMethod))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportMethod:on:",0
,(
function(self,aMethod,aStream)
return ((self):export_method_on_((aMethod):classField(), aMethod, aStream))
;
end
)
,"export","exportMethod: aMethod on: aStream\r\n\t\" Return the result of exporting aMethod on stream. \"\r\n\r\n\"\t(aMethod respondsTo: #classField) ifFalse: [ ^self print: '//Warning: broken method ',aMethod ].\r\n\"\t^self export: aMethod classField method: aMethod on: aStream","(\r\nfunction(self,aMethod,aStream)\r\nreturn ((self):export_method_on_((aMethod):classField(), aMethod, aStream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportMethods:on:",0
,(
function(self,aCollection,aStream)
(aCollection):do_((function(each)
return ((self):exportMethod_on_(each, aStream))
end))
;
return (self)
;
end
)
,"export","exportMethods: aCollection on: aStream\r\n\t\" Return the result of exporting aCollection of methods on stream. \"\r\n\r\n\taCollection do: [:each| self exportMethod: each on: aStream ]","(\r\nfunction(self,aCollection,aStream)\r\n(aCollection):do_((function(each)\r\nreturn ((self):exportMethod_on_(each, aStream))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportMethods:",0
,(
function(self,aCollection)
local stream=nil
(function()
	local _s8_value=(((smalltalk.String):new()):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):exportMethods_on_(aCollection, stream)
;
return ((stream):contents())
;
end
)
,"export","exportMethods: aCollection\r\n\t\" Return the result of exporting aCollection of methods. \"\r\n\r\n\t| stream |\r\n\tstream := String new writeStream.\r\n\tself exportMethods: aCollection on: stream.\r\n\t^stream contents","(\r\nfunction(self,aCollection)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.String):new()):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):exportMethods_on_(aCollection, stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"exportMethodsOf:on:",0
,(
function(self,aClass,aStream)
(aClass):methodsDo_((function(each)
return ((self):export_method_on_(aClass, each, aStream))
end))
;
return (self)
;
end
)
,"private","exportMethodsOf: aClass on: aStream\r\n\t\" Export methods of aClass onto aStream. \"\r\n\r\n\taClass methodsDo: [:each|\r\n\t\tself export: aClass method: each on: aStream\r\n\t]","(\r\nfunction(self,aClass,aStream)\r\n(aClass):methodsDo_((function(each)\r\nreturn ((self):export_method_on_(aClass, each, aStream))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Exporter,"export:method:on:",0
,(
function(self,aClass,aMethod,aStream)
return (((self):encoderFor_(aClass)):export_method_on_(aClass, aMethod, aStream))
;
end
)
,"export","export: aClass method: aMethod on: aStream\r\n\t\" Export a method definition of aClass onto aStream. \"\r\n\r\n\t^(self encoderFor: aClass) export: aClass method: aMethod on: aStream","(\r\nfunction(self,aClass,aMethod,aStream)\r\nreturn (((self):encoderFor_(aClass)):export_method_on_(aClass, aMethod, aStream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"default",0
,(
function(self)
(function() if ((((((self['@default']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((self):new())
	self['@default']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (self['@default'])
;
end
)
,"accessing","default\r\n\t\" Return the default instance of the receiver. \"\r\n\r\n\tdefault isNil ifTrue: [ default := self new ].\r\n\t^default","(\r\nfunction(self)\r\n(function() if ((((((self['@default']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((self):new())\r\n\tself['@default']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@default'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"saveBuilders",0
,(
function(self)
return (((self['@saveBuilders'])==true))
;
end
)
,"accessing","saveBuilders\r\n\t\" Return true if the receiver is set to save builders in snapshot. \"\r\n\r\n\t^saveBuilders == true","(\r\nfunction(self)\r\nreturn (((self['@saveBuilders'])==true))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"saveBuilders:",0
,(
function(self,aBoolean)
(function()
	local _s8_value=(aBoolean)
	self['@saveBuilders']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","saveBuilders: aBoolean\r\n\t\" Set to true if the receiver save builders in snapshot. \"\r\n\r\n\tsaveBuilders := aBoolean","(\r\nfunction(self,aBoolean)\r\n(function()\r\n\tlocal _s8_value=(aBoolean)\r\n\tself['@saveBuilders']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"build:doing:",0
,(
function(self,token,aBlock)
local builder=nil
local result=nil
local success=nil
(function()
	local _s8_value=((self):default())
	builder=_s8_value; return _s8_value
end)()
;
(builder):open_(token)
;
(function()
	local _s8_value=(false)
	success=_s8_value; return _s8_value
end)()
;
((function()
(function()
	local _s8_value=((aBlock):value())
	result=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=(true)
	success=_s8_value; return _s8_value
end)())
end)):ensure_((function()
return ((builder):close_success_(token, success))
end))
;
return (result)
;
end
)
,"building","build: token doing: aBlock\r\n\t\" Process a (build...: spec) definition marking methods generated while evaluating aBlock.\r\n\tReturn the result of evaluating aBlock. \"\r\n\r\n\t| builder result success |\r\n\tbuilder := self default.\r\n\tbuilder open: token.\r\n\tsuccess := false.\r\n\t[\tresult := aBlock value.\r\n\t\tsuccess := true.\r\n\t] ensure: [ builder close: token success: success ].\r\n\t^result","(\r\nfunction(self,token,aBlock)\r\nlocal builder=nil\r\nlocal result=nil\r\nlocal success=nil\r\n(function()\r\n\tlocal _s8_value=((self):default())\r\n\tbuilder=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(builder):open_(token)\r\n;\r\n(function()\r\n\tlocal _s8_value=(false)\r\n\tsuccess=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((function()\r\n(function()\r\n\tlocal _s8_value=((aBlock):value())\r\n\tresult=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=(true)\r\n\tsuccess=_s8_value; return _s8_value\r\nend)())\r\nend)):ensure_((function()\r\nreturn ((builder):close_success_(token, success))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"register:",0
,(
function(self,aMethod)
return ((smalltalk:boolean_and_(((((self['@default']))~=nil)),(function()
return ((self['@default']):register_(aMethod))
end))))
;
end
)
,"building","register: aMethod\r\n\t\" Register aMethod as related to current builder. \"\r\n\r\n\t^default notNil and: [ default register: aMethod ]","(\r\nfunction(self,aMethod)\r\nreturn ((smalltalk:boolean_and_(((((self['@default']))~=nil)),(function()\r\nreturn ((self['@default']):register_(aMethod))\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"imageSaved:on:",0
,(
function(self,snapshot,stream)
local code=nil
(function() if ((((smalltalk:boolean_and_(((self):saveBuilders()),(function()
return ((((function()
	local _s8_value=((self):codeForBuilders())
	code=_s8_value; return _s8_value
end)())):notEmpty())
end)))))==true) then return ((function()
return ((
function(_s8_rec)
(_s8_rec):cr();
return ((_s8_rec):nextPutAll_(code))
end
)(stream))
end))() else return nil end end)()
;
return (smalltalk:superImplementor(smalltalk.APIBuilder["$klass"],"imageSaved_on_")(self,snapshot, stream))
;
end
)
,"snapshot","imageSaved: snapshot on: stream\r\n\t\" Private - A system snapshot has been taken, the receiver can restore state. \"\r\n\r\n\t| code |\r\n\t(self saveBuilders and: [\r\n\t\t(code := self codeForBuilders) notEmpty ])\r\n\t\tifTrue: [ stream cr;nextPutAll: code ].\r\n\t^super imageSaved: snapshot on: stream","(\r\nfunction(self,snapshot,stream)\r\nlocal code=nil\r\n(function() if ((((smalltalk:boolean_and_(((self):saveBuilders()),(function()\r\nreturn ((((function()\r\n\tlocal _s8_value=((self):codeForBuilders())\r\n\tcode=_s8_value; return _s8_value\r\nend)())):notEmpty())\r\nend)))))==true) then return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\nreturn ((_s8_rec):nextPutAll_(code))\r\nend\r\n)(stream))\r\nend))() else return nil end end)()\r\n;\r\nreturn (smalltalk:superImplementor(smalltalk.APIBuilder[\"$klass\"],\"imageSaved_on_\")(self,snapshot, stream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"codeForBuilders",0
,(
function(self)
return ((self):codeFor_((smalltalk.Smalltalk):classes()))
;
end
)
,"code","codeForBuilders\r\n\t\" Return the (native) code to export the builders of the system. \"\r\n\r\n\t^self codeFor: Smalltalk classes","(\r\nfunction(self)\r\nreturn ((self):codeFor_((smalltalk.Smalltalk):classes()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"codeFor:",0
,(
function(self,classes)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local builders=nil
(function()
	local _s8_value=((self):buildersFor_(classes))
	builders=_s8_value; return _s8_value
end)()
;
(function() if ((((builders):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk."),(self):name())),".builders_(\r\n")),((smalltalk.Exporter):new()):exportObject_(builders))),"\r\n);"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"code","codeFor: classes\r\n\t\" Return the (native) code to export the builders defined by a collection of classes. \"\r\n\r\n\t| builders |\r\n\tbuilders := self buildersFor: classes.\r\n\tbuilders isEmpty ifTrue: [ ^'' ].\r\n\t^'smalltalk.', self name ,'.builders_(\r\n'\t,(Exporter new exportObject: builders) ,'\r\n);'","(\r\nfunction(self,classes)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal builders=nil\r\n(function()\r\n\tlocal _s8_value=((self):buildersFor_(classes))\r\n\tbuilders=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((builders):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.\"),(self):name())),\".builders_(\\r\\n\")),((smalltalk.Exporter):new()):exportObject_(builders))),\"\\r\\n);\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"buildersFor:",0
,(
function(self,classes)
return (((classes):collect_((function(cls)
return ((smalltalk.Array):with_with_with_((cls):name(), (self):buildersIn_((cls)["$klass"]), (self):buildersIn_(cls)))
end))):reject_((function(tuple)
return ((smalltalk:boolean_and_((((tuple):second()):isEmpty()),(function()
return (((tuple):third()):isEmpty())
end))))
end)))
;
end
)
,"private","buildersFor: classes\r\n\t\" Private - Return the builders defined by a collection of classes.\r\n\t#( className #(--classbuilders--) #(--instanceBuilders---))\r\n\t\"\r\n\r\n\t^(classes collect: [:cls|\r\n\t\tArray\r\n\t\t\twith: cls name\r\n\t\t\twith: (self buildersIn: cls class)\r\n\t\t\twith: (self buildersIn: cls)\r\n\t]) reject: [:tuple|\r\n\t\ttuple second isEmpty and: [tuple third isEmpty]\r\n\t]","(\r\nfunction(self,classes)\r\nreturn (((classes):collect_((function(cls)\r\nreturn ((smalltalk.Array):with_with_with_((cls):name(), (self):buildersIn_((cls)[\"$klass\"]), (self):buildersIn_(cls)))\r\nend))):reject_((function(tuple)\r\nreturn ((smalltalk:boolean_and_((((tuple):second()):isEmpty()),(function()\r\nreturn (((tuple):third()):isEmpty())\r\nend))))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"builders",0
,(
function(self)
return ((self):buildersFor_((smalltalk.Smalltalk):classes()))
;
end
)
,"private","builders\r\n\t\" Private - Return the builders defined in this system. \"\r\n\r\n\t^self buildersFor: Smalltalk classes","(\r\nfunction(self)\r\nreturn ((self):buildersFor_((smalltalk.Smalltalk):classes()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"buildersIn:",0
,(
function(self,aClass)
local map=nil
(function()
	local _s8_value=((smalltalk.Array):new())
	map=_s8_value; return _s8_value
end)()
;
(aClass):methodsDo_((function(mth)
local assoc=nil
(function()
	local _s8_value=((mth):builder())
	assoc=_s8_value; return _s8_value
end)();return ((function() if ((((((assoc))~=nil)))==true) then return ((function()
return ((((map):detect_ifNone_((function(tuple)
return ((smalltalk:isObject_equalTo_(((tuple):first()),assoc)))
end), (function()
return ((map):add_((smalltalk.Array):with_with_(assoc, (smalltalk.Array):new())))
end))):second()):add_((mth):selector()))
end))() else return nil end end)())
end))
;
return ((map):collect_((function(tuple)
return ((smalltalk.Array):with_with_with_(((tuple):first()):key(), ((tuple):first()):value(), (tuple):last()))
end)))
;
end
)
,"private","buildersIn: aClass\r\n\t\" Private - Return the builders used in aClass.\r\n\t#( builderType builderSpec #( selectors ) )\r\n\t\"\r\n\r\n\t| map |\r\n\tmap := Array new.\r\n\taClass methodsDo: [:mth| | assoc |\r\n\t\tassoc := mth builder.\r\n\t\tassoc notNil ifTrue: [\r\n\t\t\t(map\tdetect: [:tuple| tuple first = assoc ]\r\n\t\t\t\tifNone: [ map add: (Array with: assoc with: Array new) ])\r\n\t\t\t\tsecond add: mth selector\r\n\t\t]\r\n\t].\r\n\t^map collect: [:tuple| Array\r\n\t\twith: tuple first key\r\n\t\twith: tuple first value\r\n\t\twith: tuple last\r\n\t]","(\r\nfunction(self,aClass)\r\nlocal map=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tmap=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aClass):methodsDo_((function(mth)\r\nlocal assoc=nil\r\n(function()\r\n\tlocal _s8_value=((mth):builder())\r\n\tassoc=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((assoc))~=nil)))==true) then return ((function()\r\nreturn ((((map):detect_ifNone_((function(tuple)\r\nreturn ((smalltalk:isObject_equalTo_(((tuple):first()),assoc)))\r\nend), (function()\r\nreturn ((map):add_((smalltalk.Array):with_with_(assoc, (smalltalk.Array):new())))\r\nend))):second()):add_((mth):selector()))\r\nend))() else return nil end end)())\r\nend))\r\n;\r\nreturn ((map):collect_((function(tuple)\r\nreturn ((smalltalk.Array):with_with_with_(((tuple):first()):key(), ((tuple):first()):value(), (tuple):last()))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"builders:",0
,(
function(self,spec)
(spec):do_((function(builder)
local cls=nil
(function()
	local _s8_value=((smalltalk.Smalltalk):at_ifAbsent_((builder):first(), (function()
return (nil)
end)))
	cls=_s8_value; return _s8_value
end)();return ((function() if ((((((cls))~=nil)))==true) then return ((function()
(self):setBuilders_in_((builder):second(), (cls)["$klass"]);return ((self):setBuilders_in_((builder):third(), cls))
end))() else return nil end end)())
end))
;
return (self)
;
end
)
,"private","builders: spec\r\n\t\" Private - Set the builders on this system according to bindings defined in spec.\r\n\tNote that missing bindings will be ignored and no check is done to ensure the bindings are in sync with target.\r\n\t\"\r\n\r\n\tspec do: [:builder| | cls |\r\n\t\tcls := Smalltalk at: builder first ifAbsent: [].\r\n\t\tcls notNil ifTrue: [\r\n\t\t\tself setBuilders: builder second in: cls class.\r\n\t\t\tself setBuilders: builder third in: cls.\r\n\t\t].\r\n\t]","(\r\nfunction(self,spec)\r\n(spec):do_((function(builder)\r\nlocal cls=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):at_ifAbsent_((builder):first(), (function()\r\nreturn (nil)\r\nend)))\r\n\tcls=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((cls))~=nil)))==true) then return ((function()\r\n(self):setBuilders_in_((builder):second(), (cls)[\"$klass\"]);return ((self):setBuilders_in_((builder):third(), cls))\r\nend))() else return nil end end)())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder["$klass"],"setBuilders:in:",0
,(
function(self,array,aClass)
local builder=nil
(array):do_((function(map)
(function()
	local _s8_value=((smalltalk.Association):key_value_((map):first(), (map):second()))
	builder=_s8_value; return _s8_value
end)();return (((map):last()):do_((function(selector)
local mth=nil
(function()
	local _s8_value=((aClass):compiledMethodAt_(selector))
	mth=_s8_value; return _s8_value
end)();return ((function() if ((((((mth))~=nil)))==true) then return ((function()
return ((mth):builder_(builder))
end))() else return nil end end)())
end)))
end))
;
return (self)
;
end
)
,"private","setBuilders: array in: aClass\r\n\t\" Private - Set the builders in array on aClass. \"\r\n\r\n\t| builder |\r\n\tarray do: [:map|\r\n\t\tbuilder := Association key: map first value: map second.\r\n\t\tmap last do: [:selector| | mth |\r\n\t\t\tmth := aClass compiledMethodAt: selector.\r\n\t\t\tmth notNil ifTrue: [ mth builder: builder ].\r\n\t\t]\r\n\t]","(\r\nfunction(self,array,aClass)\r\nlocal builder=nil\r\n(array):do_((function(map)\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Association):key_value_((map):first(), (map):second()))\r\n\tbuilder=_s8_value; return _s8_value\r\nend)();return (((map):last()):do_((function(selector)\r\nlocal mth=nil\r\n(function()\r\n\tlocal _s8_value=((aClass):compiledMethodAt_(selector))\r\n\tmth=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((mth))~=nil)))==true) then return ((function()\r\nreturn ((mth):builder_(builder))\r\nend))() else return nil end end)())\r\nend)))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder,"register:",0
,(
function(self,aMethod)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((self['@token']))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((((self['@bound']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.Array):new())
	self['@bound']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(self['@bound']):add_(aMethod)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","register: aMethod\r\n\t\" Private - Register aMethod as related to the receiver. \"\r\n\r\n\ttoken isNil ifTrue: [ ^false ].\r\n\tbound isNil ifTrue: [ bound := Array new ].\r\n\tbound add: aMethod.\r\n\t^true","(\r\nfunction(self,aMethod)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((self['@token']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((self['@bound']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tself['@bound']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(self['@bound']):add_(aMethod)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder,"open:",0
,(
function(self,spec)
(function() if ((((((self['@parents']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.Array):new())
	self['@parents']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function() if ((((((self['@token']))~=nil)))==true) then return ((function()
return ((self['@parents']):addLast_(self['@token']))
end))() else return nil end end)()
;
(function()
	local _s8_value=(spec)
	self['@token']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"private","open: spec\r\n\t\" Private - The spec will be generated.\r\n\tPrepare the receiver to track generated methods while building spec.\r\n\t\"\r\n\r\n\tparents isNil ifTrue: [ parents := Array new ].\r\n\ttoken notNil ifTrue: [ parents addLast: token ].\r\n\ttoken := spec.","(\r\nfunction(self,spec)\r\n(function() if ((((((self['@parents']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tself['@parents']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((self['@token']))~=nil)))==true) then return ((function()\r\nreturn ((self['@parents']):addLast_(self['@token']))\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(spec)\r\n\tself['@token']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder,"close:success:",0
,(
function(self,spec,success)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:isObject_equalTo_((self['@token']),spec))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Builders out of sync."))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((smalltalk:boolean_and_(((((self['@parents']))~=nil)),(function()
return ((self['@parents']):notEmpty())
end)))))==true) then return ((function()
(function()
	local _s8_value=((self['@parents']):removeLast())
	self['@token']=_s8_value; return _s8_value
end)();return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (success)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((((self['@bound']))~=nil)))==true) then return ((function()
(self['@bound']):do_((function(each)
return ((self):mark_(each))
end));return ((function()
	local _s8_value=((function()
	local _s8_value=((function()
	local _s8_value=(nil)
	self['@bound']=_s8_value; return _s8_value
end)())
	self['@parents']=_s8_value; return _s8_value
end)())
	self['@token']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (success)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","close: spec success: success\r\n\t\" Private - The spec has been implemented, backtrack to parent or mark collected methods. \"\r\n\r\n\ttoken = spec ifFalse: [ ^self error: 'Builders out of sync.' ].\r\n\t(parents notNil and: [ parents notEmpty ]) ifTrue: [\r\n\t\ttoken := parents removeLast.\r\n\t\t^success\r\n\t].\r\n\tbound notNil ifTrue: [\r\n\t\tbound do: [:each| self mark: each ].\r\n\t\ttoken := parents := bound := nil.\r\n\t].\r\n\t^success","(\r\nfunction(self,spec,success)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:isObject_equalTo_((self['@token']),spec))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Builders out of sync.\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((smalltalk:boolean_and_(((((self['@parents']))~=nil)),(function()\r\nreturn ((self['@parents']):notEmpty())\r\nend)))))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((self['@parents']):removeLast())\r\n\tself['@token']=_s8_value; return _s8_value\r\nend)();return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (success)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((((self['@bound']))~=nil)))==true) then return ((function()\r\n(self['@bound']):do_((function(each)\r\nreturn ((self):mark_(each))\r\nend));return ((function()\r\n\tlocal _s8_value=((function()\r\n\tlocal _s8_value=((function()\r\n\tlocal _s8_value=(nil)\r\n\tself['@bound']=_s8_value; return _s8_value\r\nend)())\r\n\tself['@parents']=_s8_value; return _s8_value\r\nend)())\r\n\tself['@token']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (success)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder,"mark:",0
,(
function(self,aMethod)
(aMethod):builder_(self['@token'])
;
return (self)
;
end
)
,"private","mark: aMethod\r\n\t\" Private - Mark aMethod as dependent of current token. \"\r\n\r\n\taMethod builder: token","(\r\nfunction(self,aMethod)\r\n(aMethod):builder_(self['@token'])\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder,"fileOut:selectors:of:on:",0
,(
function(self,exporter,selectors,aClass,stream)
local specs=nil
(function()
	local _s8_value=((smalltalk.Set):new())
	specs=_s8_value; return _s8_value
end)()
;
((selectors):copy()):do_((function(selector)
local assoc=nil
(function()
	local _s8_value=(((aClass):compiledMethodAt_(selector)):builder())
	assoc=_s8_value; return _s8_value
end)();return ((function() if ((((((assoc))~=nil)))==true) then return ((function()
(specs):add_(assoc);return ((selectors):remove_(selector))
end))() else return nil end end)())
end))
;
(((specs):asArray()):sorted_((function(a,b)
return ((((a):key()) <= (b):key()))
end))):do_((function(assoc)
(stream):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(((aClass):name())," build")),(assoc):key()));(function() if (((((assoc):value()):notEmpty()))==true) then return ((function()
(stream):nextPut_(":");return ((self):fileOutSpec_on_((assoc):value(), stream))
end))() else return nil end end)();return ((
function(_s8_rec)
(_s8_rec):nextChunkPut_(" ");
return ((_s8_rec):cr())
end
)(stream))
end))
;
return (self)
;
end
)
,"fileOut","fileOut: exporter selectors: selectors of: aClass on: stream\r\n\t\" Fileout clustered sources for selectors onto stream.\r\n\tNote: Remove the clustered message selectors from selectors collection while fileOut.\r\n\t\"\r\n\r\n\t| specs |\r\n\tspecs := Set new.\r\n\tselectors copy do: [:selector| | assoc |\r\n\t\tassoc := (aClass compiledMethodAt: selector) builder.\r\n\t\tassoc notNil ifTrue: [\r\n\t\t\tspecs add: assoc.\r\n\t\t\tselectors remove: selector.\r\n\t\t]\r\n\t].\r\n\t(specs asArray sorted: [:a :b| a key <= b key ]) do: [:assoc|\r\n\t\tstream nextPutAll: aClass name ,' build' ,assoc key.\r\n\t\tassoc value notEmpty ifTrue: [\r\n\t\t\tstream nextPut: $:.\r\n\t\t\tself fileOutSpec: assoc value on: stream\r\n\t\t].\r\n\t\tstream nextChunkPut: $ ;cr\r\n\t].","(\r\nfunction(self,exporter,selectors,aClass,stream)\r\nlocal specs=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Set):new())\r\n\tspecs=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((selectors):copy()):do_((function(selector)\r\nlocal assoc=nil\r\n(function()\r\n\tlocal _s8_value=(((aClass):compiledMethodAt_(selector)):builder())\r\n\tassoc=_s8_value; return _s8_value\r\nend)();return ((function() if ((((((assoc))~=nil)))==true) then return ((function()\r\n(specs):add_(assoc);return ((selectors):remove_(selector))\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(((specs):asArray()):sorted_((function(a,b)\r\nreturn ((((a):key()) <= (b):key()))\r\nend))):do_((function(assoc)\r\n(stream):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(((aClass):name()),\" build\")),(assoc):key()));(function() if (((((assoc):value()):notEmpty()))==true) then return ((function()\r\n(stream):nextPut_(\":\");return ((self):fileOutSpec_on_((assoc):value(), stream))\r\nend))() else return nil end end)();return ((\r\nfunction(_s8_rec)\r\n(_s8_rec):nextChunkPut_(\" \");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.APIBuilder,"fileOutSpec:on:",0
,(
function(self,array,stream)
(
function(_s8_rec)
(_s8_rec):nextPutAll_(" #(");
return ((_s8_rec):cr())
end
)(stream)
;
(array):do_((function(each)
return ((
function(_s8_rec)
(_s8_rec):tab();
(_s8_rec):nextPutAll_((each):asLiteral());
return ((_s8_rec):cr())
end
)(stream))
end))
;
(stream):nextPut_(")")
;
return (self)
;
end
)
,"fileOut","fileOutSpec: array on: stream\r\n\t\" Private - Fileout API spec in array onto stream. \"\r\n\r\n\tstream nextPutAll: ' #(';cr.\r\n\tarray do: [:each| stream tab; nextPutAll: each asLiteral;cr ].\r\n\tstream \"tab;\"nextPut: $).","(\r\nfunction(self,array,stream)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\" #(\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream)\r\n;\r\n(array):do_((function(each)\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):tab();\r\n(_s8_rec):nextPutAll_((each):asLiteral());\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(stream))\r\nend))\r\n;\r\n(stream):nextPut_(\")\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot["$klass"],"contents",0
,(
function(self)
return (((self):new()):contents())
;
end
)
,"accessing","contents\r\n\t\" Return a snapshot of current system contents. \"\r\n\r\n\t^self new contents","(\r\nfunction(self)\r\nreturn (((self):new()):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot["$klass"],"outputToFile:",0
,(
function(self,fileName)
((self):contents()):outputToFile_(fileName)
;
return (self)
;
end
)
,"saveImage","outputToFile: fileName\r\n\t\" Save a complete system snapshot in a file at fileName. \"\r\n\r\n\tself contents outputToFile: fileName","(\r\nfunction(self,fileName)\r\n((self):contents()):outputToFile_(fileName)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"contents",0
,(
function(self)
local stream=nil
(function()
	local _s8_value=(((smalltalk.String):new()):writeStream())
	stream=_s8_value; return _s8_value
end)()
;
(self):saveImageOn_(stream)
;
return ((stream):contents())
;
end
)
,"accessing","contents\r\n\t\" Return a snapshot of current system contents. \"\r\n\r\n\t| stream |\r\n\tstream := String new writeStream.\r\n\tself saveImageOn: stream.\r\n\t^stream contents","(\r\nfunction(self)\r\nlocal stream=nil\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.String):new()):writeStream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):saveImageOn_(stream)\r\n;\r\nreturn ((stream):contents())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"saveImageOn:",0
,(
function(self,stream)
return ((self):saveImage_on_((self):smalltalk(), stream))
;
end
)
,"saveImage","saveImageOn: stream\r\n\t\" Save the current system contents onto stream. \"\r\n\r\n\t^self saveImage: self smalltalk on: stream","(\r\nfunction(self,stream)\r\nreturn ((self):saveImage_on_((self):smalltalk(), stream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"saveImage:on:",0
,(
function(self,aSmalltalk,stream)
(
function(_s8_rec)
(_s8_rec):writeLicense_on_(aSmalltalk, stream);
(_s8_rec):writeBootOn_(stream);
(_s8_rec):writePrologueOn_(stream);
(_s8_rec):writeImageContents_on_(aSmalltalk, stream);
(_s8_rec):writeEpilogueOn_(stream);
return ((_s8_rec):yourself())
end
)(self)
;
return (self)
;
end
)
,"saveImage","saveImage: aSmalltalk on: stream\r\n\t\" Save aSmalltalk system contents onto stream. \"\r\n\r\n\tself\r\n\t\twriteLicense: aSmalltalk on: stream;\r\n\t\twriteBootOn: stream;\r\n\t\twritePrologueOn: stream;\r\n\t\twriteImageContents: aSmalltalk on: stream;\r\n\t\twriteEpilogueOn: stream;\r\n\t\tyourself","(\r\nfunction(self,aSmalltalk,stream)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):writeLicense_on_(aSmalltalk, stream);\r\n(_s8_rec):writeBootOn_(stream);\r\n(_s8_rec):writePrologueOn_(stream);\r\n(_s8_rec):writeImageContents_on_(aSmalltalk, stream);\r\n(_s8_rec):writeEpilogueOn_(stream);\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"writeLicense:on:",0
,(
function(self,aSmalltalk,aStream)
(
function(_s8_rec)
(_s8_rec):nextPutAll_(((aSmalltalk):licenseTerms()):asComment());
return ((_s8_rec):cr())
end
)(aStream)
;
return (self)
;
end
)
,"writing","writeLicense: aSmalltalk on: aStream\r\n\t\" Private - Write the license terms onto aStream. \"\r\n\r\n\taStream nextPutAll: aSmalltalk licenseTerms asComment; cr","(\r\nfunction(self,aSmalltalk,aStream)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(((aSmalltalk):licenseTerms()):asComment());\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(aStream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"writeJSBootOn:",0
,(
function(self,aStream)
(self):writeGlobals_on_(smalltalk:newArray({"StObject", "StBehavior", "StClass", "StMetaclass", "StMethod", "StUndefinedObject"}), aStream)
;
(self):writeNilDefinitionOn_(aStream)
;
(self):writeGlobals_on_(smalltalk:newArray({"StSystemDictionary"}), aStream)
;
(self):writeSystemDefinitionOn_(aStream)
;
(self):write_mappingsOn_("Kernel", aStream)
;
return (self)
;
end
)
,"javascript","writeJSBootOn: aStream\r\n\t\" Private - Write the (javascript) boot section of the image onto aStream. \"\r\n\r\n\tself writeGlobals: #(\r\n\t\t#StObject #StBehavior #StClass #StMetaclass\r\n\t\t#StMethod\r\n\t\t#StUndefinedObject\r\n\t\t) on: aStream.\r\n\tself writeNilDefinitionOn: aStream.\r\n\tself writeGlobals: #( #StSystemDictionary ) on: aStream.\r\n\tself writeSystemDefinitionOn: aStream.\r\n\tself write: #Kernel mappingsOn: aStream.","(\r\nfunction(self,aStream)\r\n(self):writeGlobals_on_(smalltalk:newArray({\"StObject\", \"StBehavior\", \"StClass\", \"StMetaclass\", \"StMethod\", \"StUndefinedObject\"}), aStream)\r\n;\r\n(self):writeNilDefinitionOn_(aStream)\r\n;\r\n(self):writeGlobals_on_(smalltalk:newArray({\"StSystemDictionary\"}), aStream)\r\n;\r\n(self):writeSystemDefinitionOn_(aStream)\r\n;\r\n(self):write_mappingsOn_(\"Kernel\", aStream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"bootHeader",0
,(
function(self)
return ([[smalltalk = (function() 
]])
;
end
)
,"encoding","bootHeader\r\n\t\" Private - Returns the header for boot contents. \"\r\n\r\n\t^{'\"\"' lua'[[smalltalk = (function() \r\n]]'}","(\r\nfunction(self)\r\nreturn ([[smalltalk = (function() \r\n]])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"bootFooter",0
,(
function(self)
return ([[
end)()
]])
;
end
)
,"encoding","bootFooter\r\n\t\" Private - Returns the footer for boot contents. \"\r\n\r\n\t^{'\"\"' lua'[[\r\nend)()\r\n]]'}","(\r\nfunction(self)\r\nreturn ([[\r\nend)()\r\n]])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"writeBootOn:",0
,(
function(self,aStream)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local boot=nil
local bootName=nil
(function()
	local _s8_value=(smalltalk:object_comma_(("S8_Boot_"),(smalltalk.String):nativeFileExtension()))
	bootName=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.Smalltalk):at_ifAbsent_(bootName, (function()
return (nil)
end)))
	boot=_s8_value; return _s8_value
end)()
;
(function() if ((((((boot))~=nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):write_on_(boot, aStream))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((self):nativeLanguage()),"js"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):writeJSBootOn_(aStream))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=(read(smalltalk:object_comma_(("s8/start/boot."),(smalltalk.String):nativeFileExtension())))
	boot=_s8_value; return _s8_value
end)()
;
(function() if ((((((boot))~=nil)))==true) then return ((function()
(function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_(((self):bootHeader()),boot)),(self):bootFooter()))
	boot=_s8_value; return _s8_value
end)();(smalltalk.Smaltalk):at_put_(bootName, boot);return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):write_on_(boot, aStream))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Missing boot resource"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"writing","writeBootOn: aStream\r\n\t\" Private - Write the boot section of the image onto aStream. \"\r\n\r\n\t| boot bootName |\r\n\tbootName := 'S8_Boot_' ,String nativeFileExtension.\r\n\tboot := Smalltalk at: bootName ifAbsent: [].\r\n\tboot notNil ifTrue: [ ^self write: boot on: aStream ].\r\n\tself nativeLanguage = #js ifTrue: [\r\n\t\t^self writeJSBootOn: aStream\r\n\t].\r\n\tboot := # #read: 's8/start/boot.' ,String nativeFileExtension.\r\n\tboot notNil ifTrue: [\r\n\t\tboot := self bootHeader ,boot ,self bootFooter.\r\n\t\tSmaltalk at: bootName put: boot.\r\n\t\t^self write: boot on: aStream\r\n\t].\r\n\t^self error: 'Missing boot resource'","(\r\nfunction(self,aStream)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal boot=nil\r\nlocal bootName=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((\"S8_Boot_\"),(smalltalk.String):nativeFileExtension()))\r\n\tbootName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):at_ifAbsent_(bootName, (function()\r\nreturn (nil)\r\nend)))\r\n\tboot=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((boot))~=nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):write_on_(boot, aStream))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):nativeLanguage()),\"js\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):writeJSBootOn_(aStream))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(read(smalltalk:object_comma_((\"s8/start/boot.\"),(smalltalk.String):nativeFileExtension())))\r\n\tboot=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((boot))~=nil)))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_(((self):bootHeader()),boot)),(self):bootFooter()))\r\n\tboot=_s8_value; return _s8_value\r\nend)();(smalltalk.Smaltalk):at_put_(bootName, boot);return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):write_on_(boot, aStream))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Missing boot resource\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"writeJSPrologueOn:",0
,(
function(self,aStream)
local defaultImageName=nil
(function()
	local _s8_value=(smalltalk:object_comma_(("s8.image."),(smalltalk.String):nativeFileExtension()))
	defaultImageName=_s8_value; return _s8_value
end)()
;
(self):write_on_("smalltalk.$state = \"initialize\"", aStream)
;
(self):writeGlobals_on_(smalltalk:newArray({"smalltalk.st", "smalltalk.doIt", "fileIn", "fileInJS", "doit", "exportCategory", "saveImage", "saveImageAs"}), aStream)
;
(aStream):cr()
;
return (self)
;
end
)
,"javascript","writeJSPrologueOn: aStream\r\n\t\" Private - Write the (javascript) prologue section of the image onto aStream. \"\r\n\r\n\t| defaultImageName|\r\n\tdefaultImageName := 's8.image.' ,String nativeFileExtension.\r\n\tself write: 'smalltalk.$state = \"initialize\"' on: aStream.\r\n\tself writeGlobals: #(\r\n\t\t'smalltalk.st' 'smalltalk.doIt'\r\n\t\t#fileIn #fileInJS\r\n\t\t#doit #exportCategory\r\n\t\t#saveImage #saveImageAs\r\n\t\t) on: aStream.\r\n\taStream cr.","(\r\nfunction(self,aStream)\r\nlocal defaultImageName=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((\"s8.image.\"),(smalltalk.String):nativeFileExtension()))\r\n\tdefaultImageName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):write_on_(\"smalltalk.$state = \\\"initialize\\\"\", aStream)\r\n;\r\n(self):writeGlobals_on_(smalltalk:newArray({\"smalltalk.st\", \"smalltalk.doIt\", \"fileIn\", \"fileInJS\", \"doit\", \"exportCategory\", \"saveImage\", \"saveImageAs\"}), aStream)\r\n;\r\n(aStream):cr()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"writePrologueOn:",0
,(
function(self,aStream)
(aStream):cr()
;
(function() if ((((smalltalk:isObject_equalTo_(((self):nativeLanguage()),"js"))))==true) then return ((function()
return ((self):writeJSPrologueOn_(aStream))
end))() else return nil end end)()
;
(self):write_on_("smalltalk[\"$state\"]=\"startup\"", aStream)
;
return (self)
;
end
)
,"writing","writePrologueOn: aStream\r\n\t\" Private - Write the prologue section of the image onto aStream. \"\r\n\r\n\taStream cr.\r\n\tself nativeLanguage = #js ifTrue: [\r\n\t\tself writeJSPrologueOn: aStream\r\n\t].\r\n\tself write: 'smalltalk[\"$state\"]=\"startup\"' on: aStream.","(\r\nfunction(self,aStream)\r\n(aStream):cr()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):nativeLanguage()),\"js\"))))==true) then return ((function()\r\nreturn ((self):writeJSPrologueOn_(aStream))\r\nend))() else return nil end end)()\r\n;\r\n(self):write_on_(\"smalltalk[\\\"$state\\\"]=\\\"startup\\\"\", aStream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"launchExpression",0
,(
function(self)
return ("smalltalk:launchImage()" )
;
end
)
,"encoding","launchExpression\r\n\t\" Private - Returns the native launch expression. \"\r\n\r\n\t^{' \"smalltalk.launchImage()\" '\r\n\tlua'\"smalltalk:launchImage()\" '}","(\r\nfunction(self)\r\nreturn (\"smalltalk:launchImage()\" )\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"writeEpilogueOn:",0
,(
function(self,aStream)
local launch=nil
(function()
	local _s8_value=((self):launchExpression())
	launch=_s8_value; return _s8_value
end)()
;
(function() if ((((((launch))~=nil)))==true) then return ((function()
return ((self):write_on_(launch, aStream))
end))() else return nil end end)()
;
return (self)
;
end
)
,"writing","writeEpilogueOn: aStream\r\n\t\" Private - Write the epilogue section of the image onto aStream. \"\r\n\r\n\t| launch |\r\n\tlaunch := self launchExpression.\r\n\tlaunch notNil ifTrue: [ self write: launch on: aStream ].","(\r\nfunction(self,aStream)\r\nlocal launch=nil\r\n(function()\r\n\tlocal _s8_value=((self):launchExpression())\r\n\tlaunch=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((launch))~=nil)))==true) then return ((function()\r\nreturn ((self):write_on_(launch, aStream))\r\nend))() else return nil end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"write:on:",0
,(
function(self,aString,aStream)
(
function(_s8_rec)
(_s8_rec):cr();
return ((_s8_rec):nextPutAll_(aString))
end
)(aStream)
;
return (self)
;
end
)
,"writing","write: aString on: aStream\r\n\t\" Private - Write aString onto aStream. \"\r\n\r\n\taStream cr; nextPutAll: aString","(\r\nfunction(self,aString,aStream)\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):cr();\r\nreturn ((_s8_rec):nextPutAll_(aString))\r\nend\r\n)(aStream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"globalNotFound:",0
,(
function(self,aGlobal)
return ((self):error_(smalltalk:object_comma_(("Global not found: "),aGlobal)))
;
end
)
,"error","globalNotFound: aGlobal\r\n\t\" Private - A global at native level has not been found. \"\r\n\r\n\t^self error: 'Global not found: ',aGlobal","(\r\nfunction(self,aGlobal)\r\nreturn ((self):error_(smalltalk:object_comma_((\"Global not found: \"),aGlobal)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"writeGlobals:on:",0
,(
function(self,globals,aStream)
local object=nil
local varName=nil
(globals):do_((function(each)
(function()
	local _s8_value=(((smalltalk.Smalltalk):current()):eval_(each))
	object=_s8_value; return _s8_value
end)();(function() if ((((((object))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((self):globalNotFound_(each))
	object=_s8_value; return _s8_value
end)())
end))() else return nil end end)();return ((function() if ((((((object))~=nil)))==true) then return ((function()
(function()
	local _s8_value=((function() if ((((each):includes_(".")))==true) then return ((function()
return (each)
end))() else return ((function()
return (smalltalk:object_comma_(("var "),each))
end))() end end)())
	varName=_s8_value; return _s8_value
end)();return ((self):write_on_(smalltalk:object_comma_((smalltalk:object_comma_((varName)," = ")),smalltalk:toString_((object))), aStream))
end))() else return nil end end)())
end))
;
return (self)
;
end
)
,"writing","writeGlobals: globals on: aStream\r\n\t\" Private - Write the prologue section of the image onto aStream. \"\r\n\r\n\t| object varName |\r\n\tglobals do: [:each|\r\n\t\tobject := Smalltalk current eval: each.\r\n\t\tobject isNil ifTrue: [ object := self globalNotFound: each ].\r\n\t\tobject notNil ifTrue: [\r\n\t\t\tvarName := (each includes: $.) ifTrue: [ each ] ifFalse: [ 'var ',each ].\r\n\t\t\tself write: varName,' = ', object toString on: aStream.\r\n\t\t]\r\n\t]","(\r\nfunction(self,globals,aStream)\r\nlocal object=nil\r\nlocal varName=nil\r\n(globals):do_((function(each)\r\n(function()\r\n\tlocal _s8_value=(((smalltalk.Smalltalk):current()):eval_(each))\r\n\tobject=_s8_value; return _s8_value\r\nend)();(function() if ((((((object))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((self):globalNotFound_(each))\r\n\tobject=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)();return ((function() if ((((((object))~=nil)))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((function() if ((((each):includes_(\".\")))==true) then return ((function()\r\nreturn (each)\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((\"var \"),each))\r\nend))() end end)())\r\n\tvarName=_s8_value; return _s8_value\r\nend)();return ((self):write_on_(smalltalk:object_comma_((smalltalk:object_comma_((varName),\" = \")),smalltalk:toString_((object))), aStream))\r\nend))() else return nil end end)())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"writeNilDefinitionOn:",0
,(
function(self,aStream)
local sample=nil
(function()
	local _s8_value=(nil)
	sample=_s8_value; return _s8_value
end)()
;
(self):write_on_("var nil = new StUndefinedObject()", aStream)
;
(self):write_on_(smalltalk:object_comma_(("nil.toString = "),smalltalk:toString_((sample["toString"]))), aStream)
;
(self):write_on_(smalltalk:object_comma_(("nil.isNil_ = "),smalltalk:toString_((sample["isNil_"]))), aStream)
;
(self):write_on_(smalltalk:object_comma_(("nil.is_eqeq_ = "),smalltalk:toString_((sample["is_eqeq_"]))), aStream)
;
return (self)
;
end
)
,"javascript","writeNilDefinitionOn: aStream\r\n\t\" Private - Write the definition of nil object onto aStream. \"\r\n\r\n\t| sample |\r\n\tsample := nil.\r\n\tself write: 'var nil = new StUndefinedObject()' on: aStream.\r\n\tself write: 'nil.toString = ', sample#toString toString on: aStream.\r\n\tself write: 'nil.isNil_ = ', sample#isNil_ toString on: aStream.\r\n\tself write: 'nil.is_eqeq_ = ', sample#is_eqeq_ toString on: aStream.","(\r\nfunction(self,aStream)\r\nlocal sample=nil\r\n(function()\r\n\tlocal _s8_value=(nil)\r\n\tsample=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):write_on_(\"var nil = new StUndefinedObject()\", aStream)\r\n;\r\n(self):write_on_(smalltalk:object_comma_((\"nil.toString = \"),smalltalk:toString_((sample[\"toString\"]))), aStream)\r\n;\r\n(self):write_on_(smalltalk:object_comma_((\"nil.isNil_ = \"),smalltalk:toString_((sample[\"isNil_\"]))), aStream)\r\n;\r\n(self):write_on_(smalltalk:object_comma_((\"nil.is_eqeq_ = \"),smalltalk:toString_((sample[\"is_eqeq_\"]))), aStream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"writeSystemDefinitionOn:",0
,(
function(self,aStream)
(self):write_on_("var smalltalk = new StSystemDictionary()", aStream)
;
(self):write_on_("smalltalk.toString = function() { return \"smalltalk\" }", aStream)
;
(self):write_on_("smalltalk.$state = \"booting\"", aStream)
;
(self):write_on_("smalltalk.OverwriteMethods = smalltalk.OverwriteMethods || true", aStream)
;
(self):write_on_("smalltalk.DumpOverwritenMethods = smalltalk.DumpOverwritenMethods || false", aStream)
;
return (self)
;
end
)
,"javascript","writeSystemDefinitionOn: aStream\r\n\t\" Private - Write the definition of smalltalk object onto aStream. \"\r\n\r\n\tself write: 'var smalltalk = new StSystemDictionary()' on: aStream.\r\n\tself write: 'smalltalk.toString = function() { return \"smalltalk\" }' on: aStream.\r\n\tself write: 'smalltalk.$state = \"booting\"' on: aStream.\r\n\tself write: 'smalltalk.OverwriteMethods = smalltalk.OverwriteMethods || true' on: aStream.\r\n\tself write: 'smalltalk.DumpOverwritenMethods = smalltalk.DumpOverwritenMethods || false' on: aStream.","(\r\nfunction(self,aStream)\r\n(self):write_on_(\"var smalltalk = new StSystemDictionary()\", aStream)\r\n;\r\n(self):write_on_(\"smalltalk.toString = function() { return \\\"smalltalk\\\" }\", aStream)\r\n;\r\n(self):write_on_(\"smalltalk.$state = \\\"booting\\\"\", aStream)\r\n;\r\n(self):write_on_(\"smalltalk.OverwriteMethods = smalltalk.OverwriteMethods || true\", aStream)\r\n;\r\n(self):write_on_(\"smalltalk.DumpOverwritenMethods = smalltalk.DumpOverwritenMethods || false\", aStream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"write:mappingsOn:",0
,(
function(self,aCategory,aStream)
local mapped=nil
local mappings=nil
local done=nil
local register=nil
local row=nil
(function()
	local _s8_value=((((smalltalk.Smalltalk):current()):classes()):select_((function(each)
return ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((each):category()),aCategory))),(function()
return (((((each):constructorName()))~=nil))
end))))
end)))
	mapped=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.Array):new())
	mappings=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.Array):new())
	done=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((function(aClass)
return ((function() if ((((smalltalk:boolean_or_(((((aClass))==nil)),(function()
return ((done):includes_(aClass))
end)))))==true) then return nil else return ((function()
(register):value_((aClass):superclass());(function()
	local _s8_value=((smalltalk.Array):with_((aClass):name()))
	row=_s8_value; return _s8_value
end)();(row):add_((function() if (((((((aClass):constructorName()))==nil)))==true) then return ((function()
return ("undefined")
end))() else return ((function()
return ((aClass):constructorName())
end))() end end)());(function() if (((((((aClass):superclass()))~=nil)))==true) then return ((function()
return ((row):add_(((aClass):superclass()):name()))
end))() else return nil end end)();(mappings):add_(row);return ((done):add_(aClass))
end))() end end)())
end))
	register=_s8_value; return _s8_value
end)()
;
(mapped):do_(register)
;
return ((self):write_mappings_on_(aCategory, mappings, aStream))
;
end
)
,"writing","write: aCategory mappingsOn: aStream\r\n\t\" Private - Write the mappings for system classes in aCategory onto aStream. \"\r\n\r\n\t| mapped mappings done register row |\r\n\tmapped := Smalltalk current classes select: [:each|\r\n\t\teach category = aCategory\r\n\t\tand: [ each constructorName notNil ]\r\n\t\t].\r\n\tmappings := Array new.\r\n\tdone := Array new.\r\n\tregister := [:aClass|\r\n\t\t(aClass isNil or: [ done includes: aClass ]) ifFalse: [\r\n\t\t\tregister value: aClass superclass.\r\n\t\t\trow := Array with: aClass name.\r\n\t\t\trow add: (aClass constructorName isNil\r\n\t\t\t\tifTrue: [ #undefined ]\r\n\t\t\t\tifFalse: [ aClass constructorName ]).\r\n\t\t\taClass superclass notNil ifTrue: [\r\n\t\t\t\trow add: aClass superclass name.\r\n\t\t\t].\r\n\t\t\tmappings add: row.\r\n\t\t\tdone add: aClass.\r\n\t\t].\r\n\t].\r\n\tmapped do: register.\r\n\t^self write: aCategory mappings: mappings on: aStream","(\r\nfunction(self,aCategory,aStream)\r\nlocal mapped=nil\r\nlocal mappings=nil\r\nlocal done=nil\r\nlocal register=nil\r\nlocal row=nil\r\n(function()\r\n\tlocal _s8_value=((((smalltalk.Smalltalk):current()):classes()):select_((function(each)\r\nreturn ((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((each):category()),aCategory))),(function()\r\nreturn (((((each):constructorName()))~=nil))\r\nend))))\r\nend)))\r\n\tmapped=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tmappings=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tdone=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function(aClass)\r\nreturn ((function() if ((((smalltalk:boolean_or_(((((aClass))==nil)),(function()\r\nreturn ((done):includes_(aClass))\r\nend)))))==true) then return nil else return ((function()\r\n(register):value_((aClass):superclass());(function()\r\n\tlocal _s8_value=((smalltalk.Array):with_((aClass):name()))\r\n\trow=_s8_value; return _s8_value\r\nend)();(row):add_((function() if (((((((aClass):constructorName()))==nil)))==true) then return ((function()\r\nreturn (\"undefined\")\r\nend))() else return ((function()\r\nreturn ((aClass):constructorName())\r\nend))() end end)());(function() if (((((((aClass):superclass()))~=nil)))==true) then return ((function()\r\nreturn ((row):add_(((aClass):superclass()):name()))\r\nend))() else return nil end end)();(mappings):add_(row);return ((done):add_(aClass))\r\nend))() end end)())\r\nend))\r\n\tregister=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(mapped):do_(register)\r\n;\r\nreturn ((self):write_mappings_on_(aCategory, mappings, aStream))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"write:mappings:on:",0
,(
function(self,category,mappings,aStream)
local className=nil
local fn=nil
local superclass=nil
(mappings):do_((function(tuple)
(function()
	local _s8_value=((tuple):first())
	className=_s8_value; return _s8_value
end)();(function()
	local _s8_value=((tuple):at_((2)))
	fn=_s8_value; return _s8_value
end)();(function()
	local _s8_value=((function() if ((((smalltalk:isObject_equalTo_(((tuple):size()),(2)))))==true) then return ((function()
return ("")
end))() else return ((function()
return (smalltalk:object_comma_((",smalltalk."),(tuple):last()))
end))() end end)())
	superclass=_s8_value; return _s8_value
end)();return ((self):write_on_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk.mapClassName("),smalltalk:printString_((className)))),",")),smalltalk:printString_((category)))),",")),smalltalk:asString_((fn)))),superclass)),")"), aStream))
end))
;
return (self)
;
end
)
,"writing","write: category mappings: mappings on: aStream\r\n\t\" Private - Write the mappings onto aStream. \"\r\n\r\n\t| className fn superclass |\r\n\tmappings do: [:tuple|\r\n\t\tclassName := tuple first.\r\n\t\tfn := tuple at: 2.\r\n\t\tsuperclass := tuple size = 2 ifTrue: [ '' ]\r\n\t\t\tifFalse: [ ',smalltalk.',tuple last ]. \r\n\t\tself write: 'smalltalk.mapClassName('\r\n\t\t\t,className printString, ','\r\n\t\t\t,category printString, ','\r\n\t\t\t,fn asString ,superclass\r\n\t\t\t,')' on: aStream.\r\n\t]","(\r\nfunction(self,category,mappings,aStream)\r\nlocal className=nil\r\nlocal fn=nil\r\nlocal superclass=nil\r\n(mappings):do_((function(tuple)\r\n(function()\r\n\tlocal _s8_value=((tuple):first())\r\n\tclassName=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=((tuple):at_((2)))\r\n\tfn=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=((function() if ((((smalltalk:isObject_equalTo_(((tuple):size()),(2)))))==true) then return ((function()\r\nreturn (\"\")\r\nend))() else return ((function()\r\nreturn (smalltalk:object_comma_((\",smalltalk.\"),(tuple):last()))\r\nend))() end end)())\r\n\tsuperclass=_s8_value; return _s8_value\r\nend)();return ((self):write_on_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk.mapClassName(\"),smalltalk:printString_((className)))),\",\")),smalltalk:printString_((category)))),\",\")),smalltalk:asString_((fn)))),superclass)),\")\"), aStream))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"writeImageContents:on:",0
,(
function(self,aSmalltalk,aStream)
local exporter=nil
local globals=nil
local classes=nil
(aStream):cr()
;
(function()
	local _s8_value=((self):exporterFor_(aSmalltalk))
	exporter=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):globalsIn_(aSmalltalk))
	globals=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((((self):classesToExportFrom_with_(aSmalltalk, exporter)):collect_((function(each)
return ((each):aboutToSaveImage_on_(self, aStream))
end))):select_((function(each)
return ((((each))~=nil))
end)))
	classes=_s8_value; return _s8_value
end)()
;
(exporter):exportClasses_on_(classes, aStream)
;
(self):exportGlobals_with_on_(globals, exporter, aStream)
;
(classes):do_((function(each)
return ((each):imageSaved_on_(self, aStream))
end))
;
return (self)
;
end
)
,"writing","writeImageContents: aSmalltalk on: aStream\r\n\t\" Private - Write the image section of aSmalltalk system onto aStream. \"\r\n\r\n\t| exporter globals classes |\r\n\taStream cr.\r\n\texporter := self exporterFor: aSmalltalk.\r\n\tglobals := self globalsIn: aSmalltalk.\r\n\tclasses := ((self classesToExportFrom: aSmalltalk with: exporter)\r\n\t\tcollect: [:each| each aboutToSaveImage: self on: aStream ])\r\n\t\tselect: [:each| each notNil ].\r\n\"\tself exportGlobalNames: globals keys sorted in: aSmalltalk on: aStream.\r\n\"\texporter exportClasses: classes on: aStream.\r\n\tself exportGlobals: globals with: exporter on: aStream.\r\n\tclasses do: [:each| each imageSaved: self on: aStream ].","(\r\nfunction(self,aSmalltalk,aStream)\r\nlocal exporter=nil\r\nlocal globals=nil\r\nlocal classes=nil\r\n(aStream):cr()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):exporterFor_(aSmalltalk))\r\n\texporter=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):globalsIn_(aSmalltalk))\r\n\tglobals=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((self):classesToExportFrom_with_(aSmalltalk, exporter)):collect_((function(each)\r\nreturn ((each):aboutToSaveImage_on_(self, aStream))\r\nend))):select_((function(each)\r\nreturn ((((each))~=nil))\r\nend)))\r\n\tclasses=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(exporter):exportClasses_on_(classes, aStream)\r\n;\r\n(self):exportGlobals_with_on_(globals, exporter, aStream)\r\n;\r\n(classes):do_((function(each)\r\nreturn ((each):imageSaved_on_(self, aStream))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"exporterFor:",0
,(
function(self,aSmalltalk)
return ((smalltalk.Exporter):new())
;
end
)
,"exporting","exporterFor: aSmalltalk\r\n\t\" Private - Return the exporter to use to write contents of aSmalltalk system. \"\r\n\r\n\t^Exporter new","(\r\nfunction(self,aSmalltalk)\r\nreturn ((smalltalk.Exporter):new())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"globalsIn:",0
,(
function(self,aSmalltalk)
local result=nil
(function()
	local _s8_value=((smalltalk.PoolDictionary):new())
	result=_s8_value; return _s8_value
end)()
;
(aSmalltalk):globalsDo_((function(key)
return ((result):at_put_(key, ((aSmalltalk)[key])))
end))
;
return (result)
;
end
)
,"exporting","globalsIn: aSmalltalk\r\n\t\" Private - Return the globals defined in aSmalltalk system. \"\r\n\r\n\t| result |\r\n\tresult := PoolDictionary new.\r\n\taSmalltalk globalsDo: [:key| result at: key put: (aSmalltalk basicAt: key) ].\r\n\t^result","(\r\nfunction(self,aSmalltalk)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.PoolDictionary):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(aSmalltalk):globalsDo_((function(key)\r\nreturn ((result):at_put_(key, ((aSmalltalk)[key])))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"classesToExportFrom:with:",0
,(
function(self,aSmalltalk,exporter)
return ((((aSmalltalk):classes()):select_((function(each)
return (((((each):superclass()))==nil))
end))):inject_into_(smalltalk:newArray({}), (function(total,each)
(total):addAll_((each):withAllSubclasses());return (total)
end)))
;
end
)
,"exporting","classesToExportFrom: aSmalltalk with: exporter\r\n\t\" Private - Return the classes to export from aSmalltalk system. \"\r\n\r\n\t^(aSmalltalk classes\r\n\t\tselect: [:each| each superclass isNil ])\r\n\t\tinject: #() into: [:total :each| total addAll: each withAllSubclasses. total ]","(\r\nfunction(self,aSmalltalk,exporter)\r\nreturn ((((aSmalltalk):classes()):select_((function(each)\r\nreturn (((((each):superclass()))==nil))\r\nend))):inject_into_(smalltalk:newArray({}), (function(total,each)\r\n(total):addAll_((each):withAllSubclasses());return (total)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"exportGlobalNames:in:on:",0
,(
function(self,names,aSmalltalk,aStream)
(names):do_separatedBy_((function(key)
return ((aStream):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk[\""),key)),"\"]")))
end), (function()
return ((aStream):nextPutAll_(" , "))
end))
;
(
function(_s8_rec)
(_s8_rec):nextPutAll_("=nil");
return ((_s8_rec):cr())
end
)(aStream)
;
return (self)
;
end
)
,"exporting","exportGlobalNames: names in: aSmalltalk on: aStream\r\n\t\" Private - Export global names in aSmalltalk system. \"\r\n\r\n\tnames\r\n\t\tdo: [:key| aStream nextPutAll: 'smalltalk[\"',key,'\"]' ]\r\n\t\tseparatedBy: [ aStream nextPutAll: ' , ' ].\r\n\taStream nextPutAll: '=nil'; cr.","(\r\nfunction(self,names,aSmalltalk,aStream)\r\n(names):do_separatedBy_((function(key)\r\nreturn ((aStream):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk[\\\"\"),key)),\"\\\"]\")))\r\nend), (function()\r\nreturn ((aStream):nextPutAll_(\" , \"))\r\nend))\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):nextPutAll_(\"=nil\");\r\nreturn ((_s8_rec):cr())\r\nend\r\n)(aStream)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Snapshot,"exportGlobals:with:on:",0
,(
function(self,globals,exporter,aStream)
(globals):associationsDo_((function(binding)
return ((self):write_on_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("smalltalk[\""),(binding):key())),"\"]=")),(exporter):exportObject_((binding):value())), aStream))
end))
;
return (self)
;
end
)
,"exporting","exportGlobals: globals with: exporter on: aStream\r\n\t\" Private - Export global objects onto aStream. \"\r\n\r\n\tglobals associationsDo: [:binding|\r\n\t\tself write: 'smalltalk[\"',binding key,'\"]='\r\n\t\t\t,(exporter exportObject: binding value)\r\n\t\t\ton: aStream\r\n\t]","(\r\nfunction(self,globals,exporter,aStream)\r\n(globals):associationsDo_((function(binding)\r\nreturn ((self):write_on_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"smalltalk[\\\"\"),(binding):key())),\"\\\"]=\")),(exporter):exportObject_((binding):value())), aStream))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript["$klass"],"current",0
,(
function(self)
(function() if ((((((smalltalk.Transcript:classVariableAt_("Current")))==nil)))==true) then return ((function()
return ((self):current_((self):new()))
end))() else return nil end end)()
;
return (smalltalk.Transcript:classVariableAt_("Current"))
;
end
)
,"singleton","current\r\n\t\" Return the current instance of the receiver. \"\r\n\r\n\tCurrent isNil ifTrue: [ self current: self new ].\r\n\t^Current","(\r\nfunction(self)\r\n(function() if ((((((smalltalk.Transcript:classVariableAt_(\"Current\")))==nil)))==true) then return ((function()\r\nreturn ((self):current_((self):new()))\r\nend))() else return nil end end)()\r\n;\r\nreturn (smalltalk.Transcript:classVariableAt_(\"Current\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript["$klass"],"current:",0
,(
function(self,aTranscript)
smalltalk.Transcript:classVariableAt_put_("Current",aTranscript)
;
return (self)
;
end
)
,"singleton","current: aTranscript\r\n\t\" Set the current instance of the receiver. \"\r\n\r\n\tCurrent := aTranscript","(\r\nfunction(self,aTranscript)\r\nsmalltalk.Transcript:classVariableAt_put_(\"Current\",aTranscript)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript["$klass"],"cr",0
,(
function(self)
return (((self):current()):cr())
;
end
)
,"io","cr\r\n\t\" Output a line break onto the current instance. \"\r\n\r\n\t^self current cr","(\r\nfunction(self)\r\nreturn (((self):current()):cr())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript["$klass"],"nextPutAll:",0
,(
function(self,aString)
return (((self):current()):nextPutAll_(aString))
;
end
)
,"io","nextPutAll: aString\r\n\t\" Output aString onto the current instance. \"\r\n\r\n\t^self current nextPutAll: aString","(\r\nfunction(self,aString)\r\nreturn (((self):current()):nextPutAll_(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript["$klass"],"show:",0
,(
function(self,aString)
return (((self):current()):show_(aString))
;
end
)
,"io","show: aString\r\n\t\" Output aString onto the current instance (ensure result is shown). \"\r\n\r\n\t^self current show: aString","(\r\nfunction(self,aString)\r\nreturn (((self):current()):show_(aString))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript,"initializeStreams",0
,(
function(self)
(function()
	local _s8_value=((smalltalk.Array):new())
	self['@streams']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"initialize","initializeStreams\r\n\t\" Private - Initialize the streams of the receiver. \"\r\n\r\n\tstreams := Array new","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Array):new())\r\n\tself['@streams']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript,"streams",0
,(
function(self)
(function() if ((((((self['@streams']))==nil)))==true) then return ((function()
return ((self):initializeStreams())
end))() else return nil end end)()
;
return (self['@streams'])
;
end
)
,"accessing","streams\r\n\t\" Return the streams of the receiver. \"\r\n\r\n\tstreams isNil ifTrue: [ self initializeStreams ].\r\n\t^streams","(\r\nfunction(self)\r\n(function() if ((((((self['@streams']))==nil)))==true) then return ((function()\r\nreturn ((self):initializeStreams())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@streams'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript,"streams:",0
,(
function(self,anObject)
(function()
	local _s8_value=(anObject)
	self['@streams']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"accessing","streams: anObject\r\n\t\" Set the streams of the receiver. \"\r\n\r\n\tstreams := anObject","(\r\nfunction(self,anObject)\r\n(function()\r\n\tlocal _s8_value=(anObject)\r\n\tself['@streams']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript,"cr",0
,(
function(self)
((self):streams()):do_((function(each)
return ((each):cr())
end))
;
return (self)
;
end
)
,"io","cr\r\n\t\" Output a line break onto the receiver. \"\r\n\r\n\tself streams do: [:each| each cr ]","(\r\nfunction(self)\r\n((self):streams()):do_((function(each)\r\nreturn ((each):cr())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript,"nextPutAll:",0
,(
function(self,aString)
((self):streams()):do_((function(each)
return ((each):nextPutAll_(smalltalk:asString_((aString))))
end))
;
return (self)
;
end
)
,"io","nextPutAll: aString\r\n\t\" Output aString onto the receiver. \"\r\n\r\n\tself streams do: [:each| each nextPutAll: aString asString ]","(\r\nfunction(self,aString)\r\n((self):streams()):do_((function(each)\r\nreturn ((each):nextPutAll_(smalltalk:asString_((aString))))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript,"show:",0
,(
function(self,aString)
(self):nextPutAll_(aString)
;
return (self)
;
end
)
,"io","show: aString\r\n\t\" Output aString onto the receiver (ensure result is shown). \"\r\n\r\n\tself nextPutAll: aString","(\r\nfunction(self,aString)\r\n(self):nextPutAll_(aString)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript,"flush",0
,(
function(self)
((self):streams()):do_((function(each)
return ((function() if (((smalltalk:responds_to_((each),("flush"))))==true) then return ((function()
return ((each):flush())
end))() else return nil end end)())
end))
;
return (self)
;
end
)
,"io","flush\r\n\t\" OBSOLETE - Flush the receiver. \"\r\n\r\n\tself streams do: [:each|\r\n\t\t(each respondsTo: #flush) ifTrue: [ each flush ]\r\n\t]","(\r\nfunction(self)\r\n((self):streams()):do_((function(each)\r\nreturn ((function() if (((smalltalk:responds_to_((each),(\"flush\"))))==true) then return ((function()\r\nreturn ((each):flush())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Transcript,"inUse",0
,(
function(self)
return ((smalltalk:boolean_and_(((((self['@streams']))~=nil)),(function()
return ((self['@streams']):notEmpty())
end))))
;
end
)
,"query","inUse\r\n\t\" Returns true if the receiver is in use. \"\r\n\r\n\t^streams notNil and: [streams notEmpty]","(\r\nfunction(self)\r\nreturn ((smalltalk:boolean_and_(((((self['@streams']))~=nil)),(function()\r\nreturn ((self['@streams']):notEmpty())\r\nend))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"undefined",0
,(
function(self)
return ( nil)
;
end
)
,"constants","undefined\r\n\t\" Private - Return the native representation for undefined. \"\r\n\r\n\t^{' undefined'\tlua' nil'}","(\r\nfunction(self)\r\nreturn ( nil)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"nativeNameFromSelector:",0
,(
function(self,aSymbol)
return (aSymbol)
;
end
)
,"builders","nativeNameFromSelector: aSymbol\r\n\t\" Private - Return the property name for selector aSymbol. \"\r\n\r\n\t^aSymbol","(\r\nfunction(self,aSymbol)\r\nreturn (aSymbol)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"jsNameFromSelector:",0
,(
function(self,aSymbol)
return ((self):nativeNameFromSelector_(aSymbol))
;
end
)
,"builders","jsNameFromSelector: aSymbol\r\n\t\" OBSOLETE - Return the property name for selector aSymbol. \"\r\n\r\n\t^self nativeNameFromSelector: aSymbol","(\r\nfunction(self,aSymbol)\r\nreturn ((self):nativeNameFromSelector_(aSymbol))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildGetterCode:",0
,(
function(self,aSelector)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((self):instanceVariableNames()):includes_(aSelector)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:superImplementor(smalltalk.NativeObject["$klass"],"buildGetterCode_")(self,aSelector))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_(("^self handle basicAt: "),((self):nativeNameFromSelector_(aSelector)):asLiteral()))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders-private","buildGetterCode: aSelector\r\n\t\" Private - Return source code to get aSelector. \"\r\n\r\n\t(self instanceVariableNames includes: aSelector) ifTrue: [\r\n\t\t^super buildGetterCode: aSelector\r\n\t].\r\n\t^'^self handle basicAt: '\r\n\t\t,(self nativeNameFromSelector: aSelector) asLiteral","(\r\nfunction(self,aSelector)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((self):instanceVariableNames()):includes_(aSelector)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:superImplementor(smalltalk.NativeObject[\"$klass\"],\"buildGetterCode_\")(self,aSelector))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((\"^self handle basicAt: \"),((self):nativeNameFromSelector_(aSelector)):asLiteral()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildGetter:",0
,(
function(self,spec)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:boolean_or_(((spec):isString()),(function()
return ((smalltalk:boolean_and_(((spec):isArray()),(function()
return ((smalltalk:isObject_equalTo_(((spec):second()),"initialize")))
end))))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:superImplementor(smalltalk.NativeObject["$klass"],"buildGetter_")(self,spec))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildTypedGetter_(spec))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildGetter: spec\r\n\t\" Build method for getter. \"\r\n\r\n\t(spec isString or: [\r\n\t\tspec isArray and: [\r\n\t\tspec second = #initialize ]]) ifTrue: [\r\n\t\t^super buildGetter: spec\r\n\t].\r\n\t^self buildTypedGetter: spec","(\r\nfunction(self,spec)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:boolean_or_(((spec):isString()),(function()\r\nreturn ((smalltalk:boolean_and_(((spec):isArray()),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((spec):second()),\"initialize\")))\r\nend))))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:superImplementor(smalltalk.NativeObject[\"$klass\"],\"buildGetter_\")(self,spec))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildTypedGetter_(spec))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildSetterCode:",0
,(
function(self,aSelector)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((self):instanceVariableNames()):includes_(aSelector)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:superImplementor(smalltalk.NativeObject["$klass"],"buildSetterCode_")(self,aSelector))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("self handle basicAt: "),((self):nativeNameFromSelector_(aSelector)):asLiteral()))," put: anObject"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders-private","buildSetterCode: aSelector\r\n\t\" Private - Return source code to set aSelector. \"\r\n\r\n\t(self instanceVariableNames includes: aSelector) ifTrue: [\r\n\t\t^super buildSetterCode: aSelector\r\n\t].\r\n\t^'self handle basicAt: '\r\n\t\t,(self nativeNameFromSelector: aSelector) asLiteral\r\n\t\t, ' put: anObject'","(\r\nfunction(self,aSelector)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((self):instanceVariableNames()):includes_(aSelector)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:superImplementor(smalltalk.NativeObject[\"$klass\"],\"buildSetterCode_\")(self,aSelector))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"self handle basicAt: \"),((self):nativeNameFromSelector_(aSelector)):asLiteral())),\" put: anObject\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildSetter:",0
,(
function(self,spec)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:boolean_or_(((spec):isString()),(function()
return ((smalltalk:boolean_and_(((spec):isArray()),(function()
return ((smalltalk:isObject_equalTo_(((spec):second()),"initialize")))
end))))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:superImplementor(smalltalk.NativeObject["$klass"],"buildSetter_")(self,spec))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildTypedSetter_(spec))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildSetter: spec\r\n\t\" Private - Return source code for setter. \"\r\n\r\n\t(spec isString or: [\r\n\t\tspec isArray and: [\r\n\t\tspec second = #initialize ]]) ifTrue: [\r\n\t\t^super buildSetter: spec\r\n\t].\r\n\t^self buildTypedSetter: spec","(\r\nfunction(self,spec)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:boolean_or_(((spec):isString()),(function()\r\nreturn ((smalltalk:boolean_and_(((spec):isArray()),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((spec):second()),\"initialize\")))\r\nend))))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:superImplementor(smalltalk.NativeObject[\"$klass\"],\"buildSetter_\")(self,spec))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildTypedSetter_(spec))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildTypedGetter:",0
,(
function(self,tuple)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((tuple):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildGetter_(tuple))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("TypedGetter"):_minus_gt(tuple), (function()
return ((self):implement_comment_as_category_((self):stMessageHeader_((tuple):first()), smalltalk:object_comma_((smalltalk:object_comma_(("Return the "),(tuple):first()))," of the receiver."), smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("^"),(self):apiPrologueFor_((tuple):last()))),"self handle basicAt: ")),((self):nativeNameFromSelector_((tuple):first())):asLiteral())),")"), "accessing"))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildTypedGetter: tuple\r\n\t\" Build (instance) method to get property from spec array. \"\r\n\r\n\ttuple isString ifTrue: [ ^self buildGetter: tuple ].\r\n\t^self build: #TypedGetter -> tuple doing: [\r\n\t\tself\r\n\t\t\timplement: (self stMessageHeader: tuple first)\r\n\t\t\tcomment: 'Return the ' ,tuple first ,' of the receiver.'\r\n\t\t\tas: '^'\t,(self apiPrologueFor: tuple last)\r\n\t\t\t\t,'self handle basicAt: '\r\n\t\t\t\t,(self nativeNameFromSelector: tuple first) asLiteral\r\n\t\t\t\t,')'\r\n\t\t\tcategory: #accessing\r\n\t]","(\r\nfunction(self,tuple)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((tuple):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildGetter_(tuple))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"TypedGetter\"):_minus_gt(tuple), (function()\r\nreturn ((self):implement_comment_as_category_((self):stMessageHeader_((tuple):first()), smalltalk:object_comma_((smalltalk:object_comma_((\"Return the \"),(tuple):first())),\" of the receiver.\"), smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"^\"),(self):apiPrologueFor_((tuple):last()))),\"self handle basicAt: \")),((self):nativeNameFromSelector_((tuple):first())):asLiteral())),\")\"), \"accessing\"))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildTypedSetter:",0
,(
function(self,tuple)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((tuple):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildSetter_(tuple))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("TypedSetter"):_minus_gt(tuple), (function()
return ((self):implement_comment_as_category_((self):stMessageHeader_(smalltalk:object_comma_(((tuple):first()),": anObject")), smalltalk:object_comma_((smalltalk:object_comma_(("Set the "),(tuple):first()))," of the receiver."), smalltalk:object_comma_((smalltalk:object_comma_(("self handle basicAt: "),((self):nativeNameFromSelector_((tuple):first())):asLiteral()))," put: anObject handle"), "accessing"))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildTypedSetter: tuple\r\n\t\" Build (instance) method to set property to spec array. \"\r\n\r\n\ttuple isString ifTrue: [ ^self buildSetter: tuple ].\r\n\t^self build: #TypedSetter -> tuple doing: [\r\n\t\tself\r\n\t\t\timplement: (self stMessageHeader: tuple first , ': anObject')\r\n\t\t\tcomment: 'Set the ' ,tuple first ,' of the receiver.'\r\n\t\t\tas: 'self handle basicAt: '\r\n\t\t\t\t,(self nativeNameFromSelector: tuple first) asLiteral\r\n\t\t\t\t,' put: anObject handle'\r\n\t\t\tcategory: #accessing\r\n\t]","(\r\nfunction(self,tuple)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((tuple):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildSetter_(tuple))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"TypedSetter\"):_minus_gt(tuple), (function()\r\nreturn ((self):implement_comment_as_category_((self):stMessageHeader_(smalltalk:object_comma_(((tuple):first()),\": anObject\")), smalltalk:object_comma_((smalltalk:object_comma_((\"Set the \"),(tuple):first())),\" of the receiver.\"), smalltalk:object_comma_((smalltalk:object_comma_((\"self handle basicAt: \"),((self):nativeNameFromSelector_((tuple):first())):asLiteral())),\" put: anObject handle\"), \"accessing\"))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildTypedGetters:",0
,(
function(self,specs)
return ((self):build_doing_(("TypedGetters"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildTypedGetter_(each))
end)))
end)))
;
end
)
,"builders","buildTypedGetters: specs\r\n\t\" Build (instance) methods to get properties. \"\r\n\r\n\t^self build: #TypedGetters -> specs doing: [\r\n\t\tspecs compose: [:each| self buildTypedGetter: each ]\r\n\t]","(\r\nfunction(self,specs)\r\nreturn ((self):build_doing_((\"TypedGetters\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildTypedGetter_(each))\r\nend)))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildTypedAccessors:",0
,(
function(self,tuples)
return ((self):build_doing_(("TypedAccessors"):_minus_gt(tuples), (function()
return ((tuples):compose_((function(each)
return (smalltalk:object_comma_(((self):buildTypedGetter_(each)),(self):buildTypedSetter_(each)))
end)))
end)))
;
end
)
,"builders","buildTypedAccessors: tuples\r\n\t\" Build (instance) methods to get/set property from/to tuples. \"\r\n\t\r\n\t^self build: #TypedAccessors -> tuples doing: [\r\n\t\ttuples compose: [:each|\r\n\t\t\t(self buildTypedGetter: each)\r\n\t\t\t,(self buildTypedSetter: each)\r\n\t\t]\r\n\t]","(\r\nfunction(self,tuples)\r\nreturn ((self):build_doing_((\"TypedAccessors\"):_minus_gt(tuples), (function()\r\nreturn ((tuples):compose_((function(each)\r\nreturn (smalltalk:object_comma_(((self):buildTypedGetter_(each)),(self):buildTypedSetter_(each)))\r\nend)))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildFastGetter:",0
,(
function(self,aSelector)
local type=nil
local jsName=nil
(function() if ((((aSelector):isString()))==true) then return ((function()
(function()
	local _s8_value=("")
	type=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=((self):nativeNameFromSelector_(aSelector))
	jsName=_s8_value; return _s8_value
end)())
end))() else return ((function()
(function()
	local _s8_value=((aSelector):last())
	type=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=((self):nativeNameFromSelector_((aSelector):first()))
	jsName=_s8_value; return _s8_value
end)())
end))() end end)()
;
return ((self):build_doing_(("FastGetter"):_minus_gt(aSelector), (function()
return ((self):implement_comment_as_category_((self):stMessageHeader_(aSelector), smalltalk:object_comma_((smalltalk:object_comma_(("Return the "),aSelector))," of the receiver."), smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("^"),type)),"handle#")),jsName), "accessing"))
end)))
;
end
)
,"builders","buildFastGetter: aSelector\r\n\t\" Build (instance) method to get native property. \"\r\n\r\n\t| type jsName |\r\n\taSelector isString\r\n\t\tifTrue: [ type := ''. jsName := self nativeNameFromSelector: aSelector ]\r\n\t\tifFalse: [ type := aSelector last. jsName := self nativeNameFromSelector: aSelector first ].\r\n\t^self build: #FastGetter -> aSelector doing: [\r\n\t\tself\r\n\t\t\timplement: (self stMessageHeader: aSelector)\r\n\t\t\tcomment: 'Return the ' ,aSelector ,' of the receiver.'\r\n\t\t\tas: '^' ,type ,'handle#' ,jsName category: #accessing\r\n\t]","(\r\nfunction(self,aSelector)\r\nlocal type=nil\r\nlocal jsName=nil\r\n(function() if ((((aSelector):isString()))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=(\"\")\r\n\ttype=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=((self):nativeNameFromSelector_(aSelector))\r\n\tjsName=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\n(function()\r\n\tlocal _s8_value=((aSelector):last())\r\n\ttype=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=((self):nativeNameFromSelector_((aSelector):first()))\r\n\tjsName=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)()\r\n;\r\nreturn ((self):build_doing_((\"FastGetter\"):_minus_gt(aSelector), (function()\r\nreturn ((self):implement_comment_as_category_((self):stMessageHeader_(aSelector), smalltalk:object_comma_((smalltalk:object_comma_((\"Return the \"),aSelector)),\" of the receiver.\"), smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"^\"),type)),\"handle#\")),jsName), \"accessing\"))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildFastSetter:",0
,(
function(self,aSelector)
local jsName=nil
(function() if ((((aSelector):isString()))==true) then return ((function()
return ((function()
	local _s8_value=((self):nativeNameFromSelector_(aSelector))
	jsName=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((function()
	local _s8_value=((self):nativeNameFromSelector_((aSelector):first()))
	jsName=_s8_value; return _s8_value
end)())
end))() end end)()
;
return ((self):build_doing_(("FastSetter"):_minus_gt(aSelector), (function()
return ((self):implement_comment_as_category_(smalltalk:object_comma_(((self):stMessageHeader_(aSelector)),": anObject"), smalltalk:object_comma_((smalltalk:object_comma_(("Set the "),aSelector))," of the receiver."), smalltalk:object_comma_((smalltalk:object_comma_(("handle#"),jsName))," := anObject"), "accessing"))
end)))
;
end
)
,"builders","buildFastSetter: aSelector\r\n\t\" Build (instance) method to set native property. \"\r\n\r\n\t| jsName |\r\n\taSelector isString\r\n\t\tifTrue: [ jsName := self nativeNameFromSelector: aSelector ]\r\n\t\tifFalse: [ jsName := self nativeNameFromSelector: aSelector first ].\r\n\t^self build: #FastSetter -> aSelector doing: [\r\n\t\tself\r\n\t\t\timplement: (self stMessageHeader: aSelector) ,': anObject'\r\n\t\t\tcomment: 'Set the ' ,aSelector ,' of the receiver.'\r\n\t\t\tas: 'handle#' ,jsName ,' := anObject' category: #accessing\r\n\t]","(\r\nfunction(self,aSelector)\r\nlocal jsName=nil\r\n(function() if ((((aSelector):isString()))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((self):nativeNameFromSelector_(aSelector))\r\n\tjsName=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((self):nativeNameFromSelector_((aSelector):first()))\r\n\tjsName=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)()\r\n;\r\nreturn ((self):build_doing_((\"FastSetter\"):_minus_gt(aSelector), (function()\r\nreturn ((self):implement_comment_as_category_(smalltalk:object_comma_(((self):stMessageHeader_(aSelector)),\": anObject\"), smalltalk:object_comma_((smalltalk:object_comma_((\"Set the \"),aSelector)),\" of the receiver.\"), smalltalk:object_comma_((smalltalk:object_comma_((\"handle#\"),jsName)),\" := anObject\"), \"accessing\"))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildFastAccessor:",0
,(
function(self,accessor)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((accessor):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("FastAccessor"):_minus_gt(accessor), (function()
return (smalltalk:object_comma_(((self):buildFastGetter_(accessor)),(self):buildFastSetter_(accessor)))
end)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Invalid (Fast)accessors."))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildFastAccessor: accessor\r\n\t\" Build (instance) method to access native property. \"\r\n\r\n\taccessor isString ifTrue: [\r\n\t\t^self build: #FastAccessor -> accessor doing: [\r\n\t\t\t(self buildFastGetter: accessor)\r\n\t\t\t,(self buildFastSetter: accessor)\r\n\t\t]\r\n\t].\r\n\t^self error: 'Invalid (Fast)accessors.'","(\r\nfunction(self,accessor)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((accessor):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"FastAccessor\"):_minus_gt(accessor), (function()\r\nreturn (smalltalk:object_comma_(((self):buildFastGetter_(accessor)),(self):buildFastSetter_(accessor)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Invalid (Fast)accessors.\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildFastGetters:",0
,(
function(self,specs)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((specs):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildFastGetters_((specs):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("FastGetters"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildFastGetter_(each))
end)))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildFastGetters: specs\r\n\t\" Build methods to get properties. \"\r\n\r\n\tspecs isString ifTrue: [ ^self buildFastGetters: specs asArrayOfSubstrings ].\r\n\t^self build: #FastGetters -> specs doing: [\r\n\t\tspecs compose: [:each| self buildFastGetter: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((specs):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildFastGetters_((specs):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"FastGetters\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildFastGetter_(each))\r\nend)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildFastAccessors:",0
,(
function(self,specs)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((specs):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildFastAccessors_((specs):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("FastAccessors"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildFastAccessor_(each))
end)))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildFastAccessors: specs\r\n\t\" Build methods to access properties. \"\r\n\r\n\tspecs isString ifTrue: [ ^self buildFastAccessors: specs asArrayOfSubstrings ].\r\n\t^self build: #FastAccessors -> specs doing: [\r\n\t\tspecs compose: [:each| self buildFastAccessor: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((specs):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildFastAccessors_((specs):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"FastAccessors\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildFastAccessor_(each))\r\nend)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"orZero",0
,(
function(self)
return (" or 0")
;
end
)
,"constants","orZero\r\n\t\" Private - Return the code for value or zero. \"\r\n\r\n\t^{'\" || 0\"'\tlua'\" or 0\"'}","(\r\nfunction(self)\r\nreturn (\" or 0\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"handleAt:",0
,(
function(self,aSelector)
return (smalltalk:object_comma_(("handle#"),(self):nativeNameFromSelector_(aSelector)))
;
end
)
,"constants","handleAt: aSelector\r\n\t\" Private - Return the code for accessing instance handle. \"\r\n\r\n\t^'handle#' ,(self nativeNameFromSelector: aSelector)","(\r\nfunction(self,aSelector)\r\nreturn (smalltalk:object_comma_((\"handle#\"),(self):nativeNameFromSelector_(aSelector)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildNumberGetter:",0
,(
function(self,aSelector)
return ((self):build_doing_(("NumberGetter"):_minus_gt(aSelector), (function()
return ((self):implement_comment_as_category_((self):stMessageHeader_(aSelector), smalltalk:object_comma_((smalltalk:object_comma_(("Return the "),aSelector))," of the receiver."), smalltalk:object_comma_((smalltalk:object_comma_(("^"),(self):handleAt_(aSelector))),(self):orZero()), "accessing"))
end)))
;
end
)
,"builders","buildNumberGetter: aSelector\r\n\t\" Build (instance) method to get native property. \"\r\n\r\n\t^self build: #NumberGetter -> aSelector doing: [\r\n\t\tself\r\n\t\t\timplement: (self stMessageHeader: aSelector)\r\n\t\t\tcomment: 'Return the ' ,aSelector ,' of the receiver.'\r\n\t\t\tas: $^ ,(self handleAt: aSelector) ,self orZero\r\n\t\t\tcategory: #accessing\r\n\t]","(\r\nfunction(self,aSelector)\r\nreturn ((self):build_doing_((\"NumberGetter\"):_minus_gt(aSelector), (function()\r\nreturn ((self):implement_comment_as_category_((self):stMessageHeader_(aSelector), smalltalk:object_comma_((smalltalk:object_comma_((\"Return the \"),aSelector)),\" of the receiver.\"), smalltalk:object_comma_((smalltalk:object_comma_((\"^\"),(self):handleAt_(aSelector))),(self):orZero()), \"accessing\"))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildNumberAccessor:",0
,(
function(self,accessor)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((accessor):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("NumberAccessor"):_minus_gt(accessor), (function()
return (smalltalk:object_comma_(((self):buildNumberGetter_(accessor)),(self):buildSetter_(accessor)))
end)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Invalid (Number)accessors."))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildNumberAccessor: accessor\r\n\t\" Build (instance) method to access native property. \"\r\n\r\n\taccessor isString ifTrue: [\r\n\t\t^self build: #NumberAccessor -> accessor doing: [\r\n\t\t\t(self buildNumberGetter: accessor)\r\n\t\t\t,(self buildSetter: accessor)\r\n\t\t]\r\n\t].\r\n\t^self error: 'Invalid (Number)accessors.'","(\r\nfunction(self,accessor)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((accessor):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"NumberAccessor\"):_minus_gt(accessor), (function()\r\nreturn (smalltalk:object_comma_(((self):buildNumberGetter_(accessor)),(self):buildSetter_(accessor)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Invalid (Number)accessors.\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildNumberGetters:",0
,(
function(self,specs)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((specs):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildNumberGetters_((specs):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("NumberGetters"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildNumberGetter_(each))
end)))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildNumberGetters: specs\r\n\t\" Build methods to get properties. \"\r\n\r\n\tspecs isString ifTrue: [ ^self buildNumberGetters: specs asArrayOfSubstrings ].\r\n\t^self build: #NumberGetters -> specs doing: [\r\n\t\tspecs compose: [:each| self buildNumberGetter: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((specs):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildNumberGetters_((specs):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"NumberGetters\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildNumberGetter_(each))\r\nend)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildNumberAccessors:",0
,(
function(self,specs)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((specs):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildNumberAccessors_((specs):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("NumberAccessors"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildNumberAccessor_(each))
end)))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildNumberAccessors: specs\r\n\t\" Build methods to access properties. \"\r\n\r\n\tspecs isString ifTrue: [ ^self buildNumberAccessors: specs asArrayOfSubstrings ].\r\n\t^self build: #NumberAccessors -> specs doing: [\r\n\t\tspecs compose: [:each| self buildNumberAccessor: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((specs):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildNumberAccessors_((specs):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"NumberAccessors\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildNumberAccessor_(each))\r\nend)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildBooleanGetter:",0
,(
function(self,aSelector)
return ((self):build_doing_(("BooleanGetter"):_minus_gt(aSelector), (function()
return ((self):implement_comment_as_category_((self):stMessageHeader_(aSelector), smalltalk:object_comma_((smalltalk:object_comma_(("Return the "),aSelector))," of the receiver."), smalltalk:object_comma_(("^true == "),(self):handleAt_(aSelector)), "accessing"))
end)))
;
end
)
,"builders","buildBooleanGetter: aSelector\r\n\t\" Build (instance) method to get native property. \"\r\n\r\n\t^self build: #BooleanGetter -> aSelector doing: [\r\n\t\tself\r\n\t\t\timplement: (self stMessageHeader: aSelector)\r\n\t\t\tcomment: 'Return the ' ,aSelector ,' of the receiver.'\r\n\t\t\tas: '^true == ' ,(self handleAt: aSelector)\r\n\t\t\tcategory: #accessing\r\n\t]","(\r\nfunction(self,aSelector)\r\nreturn ((self):build_doing_((\"BooleanGetter\"):_minus_gt(aSelector), (function()\r\nreturn ((self):implement_comment_as_category_((self):stMessageHeader_(aSelector), smalltalk:object_comma_((smalltalk:object_comma_((\"Return the \"),aSelector)),\" of the receiver.\"), smalltalk:object_comma_((\"^true == \"),(self):handleAt_(aSelector)), \"accessing\"))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildBooleanAccessor:",0
,(
function(self,accessor)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((accessor):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("BooleanAccessor"):_minus_gt(accessor), (function()
return (smalltalk:object_comma_(((self):buildBooleanGetter_(accessor)),(self):buildSetter_(accessor)))
end)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Invalid (Boolean)accessors."))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildBooleanAccessor: accessor\r\n\t\" Build (instance) method to access native property. \"\r\n\r\n\taccessor isString ifTrue: [\r\n\t\t^self build: #BooleanAccessor -> accessor doing: [\r\n\t\t\t(self buildBooleanGetter: accessor)\r\n\t\t\t,(self buildSetter: accessor)\r\n\t\t]\r\n\t].\r\n\t^self error: 'Invalid (Boolean)accessors.'","(\r\nfunction(self,accessor)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((accessor):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"BooleanAccessor\"):_minus_gt(accessor), (function()\r\nreturn (smalltalk:object_comma_(((self):buildBooleanGetter_(accessor)),(self):buildSetter_(accessor)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Invalid (Boolean)accessors.\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildBooleanGetters:",0
,(
function(self,specs)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((specs):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildBooleanGetters_((specs):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("BooleanGetters"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildBooleanGetter_(each))
end)))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildBooleanGetters: specs\r\n\t\" Build methods to get properties. \"\r\n\r\n\tspecs isString ifTrue: [ ^self buildBooleanGetters: specs asArrayOfSubstrings ].\r\n\t^self build: #BooleanGetters -> specs doing: [\r\n\t\tspecs compose: [:each| self buildBooleanGetter: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((specs):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildBooleanGetters_((specs):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"BooleanGetters\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildBooleanGetter_(each))\r\nend)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildBooleanAccessors:",0
,(
function(self,specs)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((specs):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildBooleanAccessors_((specs):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("BooleanAccessors"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildBooleanAccessor_(each))
end)))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildBooleanAccessors: specs\r\n\t\" Build methods to access properties. \"\r\n\r\n\tspecs isString ifTrue: [ ^self buildBooleanAccessors: specs asArrayOfSubstrings ].\r\n\t^self build: #BooleanAccessors -> specs doing: [\r\n\t\tspecs compose: [:each| self buildBooleanAccessor: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((specs):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildBooleanAccessors_((specs):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"BooleanAccessors\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildBooleanAccessor_(each))\r\nend)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildIsBooleanGetter:",0
,(
function(self,aSelector)
return ((self):build_doing_(("IsBooleanGetter"):_minus_gt(aSelector), (function()
local isSelector=nil
(function()
	local _s8_value=(smalltalk:object_comma_(("is"),(aSelector):asGlobalName()))
	isSelector=_s8_value; return _s8_value
end)();return ((self):implement_comment_as_category_((self):stMessageHeader_(isSelector), smalltalk:object_comma_((smalltalk:object_comma_(("Return the "),aSelector))," of the receiver."), smalltalk:object_comma_(("^true == handle#"),(self):nativeNameFromSelector_(isSelector)), "accessing"))
end)))
;
end
)
,"builders","buildIsBooleanGetter: aSelector\r\n\t\" Build (instance) method to get native property. \"\r\n\r\n\t^self build: #IsBooleanGetter -> aSelector doing: [ | isSelector |\r\n\t\tisSelector :=  #is ,aSelector asGlobalName.\r\n\t\tself\r\n\t\t\timplement: (self stMessageHeader: isSelector)\r\n\t\t\tcomment: 'Return the ' ,aSelector ,' of the receiver.'\r\n\t\t\tas: '^true == handle#' ,(self nativeNameFromSelector: isSelector)\r\n\t\t\tcategory: #accessing\r\n\t]","(\r\nfunction(self,aSelector)\r\nreturn ((self):build_doing_((\"IsBooleanGetter\"):_minus_gt(aSelector), (function()\r\nlocal isSelector=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((\"is\"),(aSelector):asGlobalName()))\r\n\tisSelector=_s8_value; return _s8_value\r\nend)();return ((self):implement_comment_as_category_((self):stMessageHeader_(isSelector), smalltalk:object_comma_((smalltalk:object_comma_((\"Return the \"),aSelector)),\" of the receiver.\"), smalltalk:object_comma_((\"^true == handle#\"),(self):nativeNameFromSelector_(isSelector)), \"accessing\"))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildIsBooleanAccessor:",0
,(
function(self,accessor)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((accessor):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("IsBooleanAccessor"):_minus_gt(accessor), (function()
return (smalltalk:object_comma_(((self):buildIsBooleanGetter_(accessor)),(self):buildSetter_(accessor)))
end)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Invalid (IsBoolean)accessors."))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildIsBooleanAccessor: accessor\r\n\t\" Build (instance) method to access native property. \"\r\n\r\n\taccessor isString ifTrue: [\r\n\t\t^self build: #IsBooleanAccessor -> accessor doing: [\r\n\t\t\t(self buildIsBooleanGetter: accessor)\r\n\t\t\t,(self buildSetter: accessor)\r\n\t\t]\r\n\t].\r\n\t^self error: 'Invalid (IsBoolean)accessors.'","(\r\nfunction(self,accessor)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((accessor):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"IsBooleanAccessor\"):_minus_gt(accessor), (function()\r\nreturn (smalltalk:object_comma_(((self):buildIsBooleanGetter_(accessor)),(self):buildSetter_(accessor)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Invalid (IsBoolean)accessors.\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildIsBooleanGetters:",0
,(
function(self,specs)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((specs):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildIsBooleanGetters_((specs):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("IsBooleanGetters"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildIsBooleanGetter_(each))
end)))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildIsBooleanGetters: specs\r\n\t\" Build methods to get properties. \"\r\n\r\n\tspecs isString ifTrue: [ ^self buildIsBooleanGetters: specs asArrayOfSubstrings ].\r\n\t^self build: #IsBooleanGetters -> specs doing: [\r\n\t\tspecs compose: [:each| self buildIsBooleanGetter: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((specs):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildIsBooleanGetters_((specs):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"IsBooleanGetters\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildIsBooleanGetter_(each))\r\nend)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildIsBooleanAccessors:",0
,(
function(self,specs)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((specs):isString()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildIsBooleanAccessors_((specs):asArrayOfSubstrings()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):build_doing_(("IsBooleanAccessors"):_minus_gt(specs), (function()
return ((specs):compose_((function(each)
return ((self):buildIsBooleanAccessor_(each))
end)))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildIsBooleanAccessors: specs\r\n\t\" Build methods to access properties. \"\r\n\r\n\tspecs isString ifTrue: [ ^self buildIsBooleanAccessors: specs asArrayOfSubstrings ].\r\n\t^self build: #IsBooleanAccessors -> specs doing: [\r\n\t\tspecs compose: [:each| self buildIsBooleanAccessor: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((specs):isString()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildIsBooleanAccessors_((specs):asArrayOfSubstrings()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):build_doing_((\"IsBooleanAccessors\"):_minus_gt(specs), (function()\r\nreturn ((specs):compose_((function(each)\r\nreturn ((self):buildIsBooleanAccessor_(each))\r\nend)))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildSubclasses:",0
,(
function(self,aCollection)
local result=nil
(aCollection):do_((function(tuple)
return ((function()
	local _s8_value=((function() if ((((tuple):isString()))==true) then return ((function()
return ((self):buildSubclass_arguments_typed_(tuple, smalltalk:newArray({}), smalltalk:newArray({})))
end))() else return ((function()
return ((function() if ((((smalltalk:isObject_equalTo_(((tuple):size()),(2)))))==true) then return ((function()
return ((self):buildSubclass_arguments_typed_((tuple):first(), (tuple):second(), smalltalk:newArray({})))
end))() else return ((function()
return ((self):buildSubclass_arguments_typed_((tuple):first(), (tuple):second(), (tuple):last()))
end))() end end)())
end))() end end)())
	result=_s8_value; return _s8_value
end)())
end))
;
return (result)
;
end
)
,"builder","buildSubclasses: aCollection\r\n\t\" Private - Build aCollection of subclasses of the receiver as wrappers to the library.\r\n\tReturn the last class created (or nil).\r\n\t\"\r\n \r\n\t| result |\r\n\taCollection do: [:tuple|\r\n\t\tresult := tuple isString\r\n\t\tifTrue: [ self buildSubclass: tuple arguments: #() typed: #() ]\r\n\t\tifFalse: [\r\n\t\t\ttuple size = 2\r\n\t\t\tifTrue: [ self buildSubclass: tuple first arguments: tuple second typed: #() ]\r\n\t\t\tifFalse: [ self buildSubclass: tuple first arguments: tuple second typed: tuple last ]\r\n\t\t ]\r\n\t].\r\n\t^result","(\r\nfunction(self,aCollection)\r\nlocal result=nil\r\n(aCollection):do_((function(tuple)\r\nreturn ((function()\r\n\tlocal _s8_value=((function() if ((((tuple):isString()))==true) then return ((function()\r\nreturn ((self):buildSubclass_arguments_typed_(tuple, smalltalk:newArray({}), smalltalk:newArray({})))\r\nend))() else return ((function()\r\nreturn ((function() if ((((smalltalk:isObject_equalTo_(((tuple):size()),(2)))))==true) then return ((function()\r\nreturn ((self):buildSubclass_arguments_typed_((tuple):first(), (tuple):second(), smalltalk:newArray({})))\r\nend))() else return ((function()\r\nreturn ((self):buildSubclass_arguments_typed_((tuple):first(), (tuple):second(), (tuple):last()))\r\nend))() end end)())\r\nend))() end end)())\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildHierarchy:",0
,(
function(self,aCollection)
local stream=nil
local aClass=nil
(function()
	local _s8_value=((aCollection):stream())
	stream=_s8_value; return _s8_value
end)()
;
(function() while(not ((function()
return ((stream):atEnd())
end))()) do (function()
(function()
	local _s8_value=((self):buildSubclasses_((smalltalk.Array):with_((stream):next())))
	aClass=_s8_value; return _s8_value
end)();return ((function() if (((((stream):peek()):isArray()))==true) then return ((function()
return ((aClass):buildHierarchy_((stream):next()))
end))() else return nil end end)())
end)() end;return nil end)()
;
return (self)
;
end
)
,"builder","buildHierarchy: aCollection\r\n\t\" Private - Build aCollection of subclasses of the receiver as wrappers to the library. \"\r\n \r\n\t| stream aClass |\r\n\tstream := aCollection stream.\r\n\t[ stream atEnd ] whileFalse: [\r\n\t\taClass := self buildSubclasses: (Array with: stream next).\r\n\t\tstream peek isArray ifTrue: [\r\n\t\t\taClass buildHierarchy: stream next\r\n\t\t].\r\n\t]","(\r\nfunction(self,aCollection)\r\nlocal stream=nil\r\nlocal aClass=nil\r\n(function()\r\n\tlocal _s8_value=((aCollection):stream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() while(not ((function()\r\nreturn ((stream):atEnd())\r\nend))()) do (function()\r\n(function()\r\n\tlocal _s8_value=((self):buildSubclasses_((smalltalk.Array):with_((stream):next())))\r\n\taClass=_s8_value; return _s8_value\r\nend)();return ((function() if (((((stream):peek()):isArray()))==true) then return ((function()\r\nreturn ((aClass):buildHierarchy_((stream):next()))\r\nend))() else return nil end end)())\r\nend)() end;return nil end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildSubclass:arguments:typed:",0
,(
function(self,globalClassName,argList,typedArguments)
local cls=nil
local clsName=nil
(function()
	local _s8_value=(smalltalk:object_comma_((((globalClassName):first()):asUppercase()),(globalClassName):copyFrom_to_((2), (globalClassName):size())))
	clsName=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(smalltalk:object_comma_(((self):classToken()),clsName))
	clsName=_s8_value; return _s8_value
end)()
;
(self):subclass_instanceVariableNames_category_(clsName, "", (self):category())
;
return ((smalltalk.Smalltalk):at_(clsName))
;
end
)
,"builder","buildSubclass: globalClassName arguments: argList typed: typedArguments\r\n\t\" Private - Build a subclass of the receiver implementing a wrapper to the library.\r\n\tNote: argList and typed arguments are ignored; see subclasses to find concrete implementors.\r\n\tReturn the created class.\r\n\t\"\r\n\r\n\t| cls clsName |\r\n\tclsName := globalClassName first asUppercase\r\n\t\t\t,(globalClassName copyFrom: 2 to: globalClassName size).\r\n\tclsName := self classToken ,clsName.\r\n\tself\r\n\t\tsubclass: clsName\r\n\t\tinstanceVariableNames: ''\r\n\t\tcategory: self category.\r\n\t^Smalltalk at: clsName","(\r\nfunction(self,globalClassName,argList,typedArguments)\r\nlocal cls=nil\r\nlocal clsName=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((((globalClassName):first()):asUppercase()),(globalClassName):copyFrom_to_((2), (globalClassName):size())))\r\n\tclsName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_(((self):classToken()),clsName))\r\n\tclsName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):subclass_instanceVariableNames_category_(clsName, \"\", (self):category())\r\n;\r\nreturn ((smalltalk.Smalltalk):at_(clsName))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"classToken",0
,(
function(self)
return ("")
;
end
)
,"builder","classToken\r\n\t\" Private - Return the classToken to prepend/tag the receiver.\r\n\tThe default answer is empty.\r\n\tThe subclasses can subclass to build tagged subclasses.\r\n\t\"\r\n\r\n\t^''","(\r\nfunction(self)\r\nreturn (\"\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"classToken:",0
,(
function(self,aToken)
return ((self):buildClassToken_(aToken))
;
end
)
,"builder","classToken: aToken\r\n\t\" Private - Generate code to return the classToken of the receiver. \"\r\n\r\n\t^self buildClassToken: aToken","(\r\nfunction(self,aToken)\r\nreturn ((self):buildClassToken_(aToken))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildClassToken:",0
,(
function(self,aToken)
(self):build_doing_(("ClassToken"):_minus_gt(aToken), (function()
return ((self):implement_comment_as_category_("classToken", "Private - Return the classToken to prepend/tag the receiver.", smalltalk:object_comma_(("^"),(aToken):asLiteral()), "builder"))
end))
;
return (self)
;
end
)
,"builder","buildClassToken: aToken\r\n\t\" Generate code to return the classToken of the receiver. \"\r\n\r\n\tself build: #ClassToken -> aToken doing: [\r\n\t\tself\r\n\t\t\timplement: #classToken\r\n\t\t\tcomment: 'Private - Return the classToken to prepend/tag the receiver.'\r\n\t\t\tas: $^ ,aToken asLiteral\r\n\t\t\tcategory: #builder\r\n\t]","(\r\nfunction(self,aToken)\r\n(self):build_doing_((\"ClassToken\"):_minus_gt(aToken), (function()\r\nreturn ((self):implement_comment_as_category_(\"classToken\", \"Private - Return the classToken to prepend/tag the receiver.\", smalltalk:object_comma_((\"^\"),(aToken):asLiteral()), \"builder\"))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildFunctions:",0
,(
function(self,specs)
(self):build_doing_(("Functions"):_minus_gt(specs), (function()
return ((specs):do_((function(spec)
return ((self):buildFunction_(spec))
end)))
end))
;
return (self)
;
end
)
,"builders","buildFunctions: specs\r\n\t\" Generate code from specs as functions implemented by instance handle. \"\r\n\r\n\tself build: #Functions -> specs doing: [\r\n\t\tspecs do: [:spec| self buildFunction: spec ]\r\n\t]","(\r\nfunction(self,specs)\r\n(self):build_doing_((\"Functions\"):_minus_gt(specs), (function()\r\nreturn ((specs):do_((function(spec)\r\nreturn ((self):buildFunction_(spec))\r\nend)))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"apiPrologueFor:",0
,(
function(self,aReturnType)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((aReturnType):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ("self ignoreReturn: (")
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((aReturnType):last()),"("))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (aReturnType)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((aReturnType):last()),"+"))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):apiPrologueFor_(smalltalk:object_comma_(((aReturnType):upTo_("+")),"[]")))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((aReturnType):includes_("[")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):apiPrologueForArray_(aReturnType))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((aReturnType):includes_("<")))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):apiPrologueForTuple_(aReturnType))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_(((self):apiWrapperFor_(aReturnType))," @ ("))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","apiPrologueFor: aReturnType\r\n\t\" Private - Return the prologue to use to wrap a return.\r\n\taReturnType can be:\r\n\t\t- empty string to ignore api return value (will return the receiver)\r\n\t\t- suffixed by [...] or + to return an arrayed result\r\n\t\t- suffixed by type spec GlobalName\r\n\t\t- smalltalk code ending with (\r\n\t\t- a global name understanding #@\r\n\t\"\r\n\r\n\taReturnType isEmpty ifTrue: [ ^'self ignoreReturn: (' ].\r\n\taReturnType last = $( ifTrue: [ ^aReturnType ].\r\n\taReturnType last = $+ ifTrue: [\r\n\t\t^self apiPrologueFor: (aReturnType upTo: $+) ,'[]'\r\n\t].\r\n\t(aReturnType includes: $[) ifTrue: [\r\n\t\t^self apiPrologueForArray: aReturnType\r\n\t].\r\n\t(aReturnType includes: $<) ifTrue: [\r\n\t\t^self apiPrologueForTuple: aReturnType\r\n\t].\r\n\r\n\t^(self apiWrapperFor: aReturnType) ,' @ ('","(\r\nfunction(self,aReturnType)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((aReturnType):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (\"self ignoreReturn: (\")\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((aReturnType):last()),\"(\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (aReturnType)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((aReturnType):last()),\"+\"))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):apiPrologueFor_(smalltalk:object_comma_(((aReturnType):upTo_(\"+\")),\"[]\")))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((aReturnType):includes_(\"[\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):apiPrologueForArray_(aReturnType))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((aReturnType):includes_(\"<\")))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):apiPrologueForTuple_(aReturnType))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_(((self):apiWrapperFor_(aReturnType)),\" @ (\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"apiPrologueForArray:",0
,(
function(self,aReturnType)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local type=nil
local stream=nil
local kind=nil
(function()
	local _s8_value=((aReturnType):stream())
	stream=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):apiWrapperFor_((stream):upTo_("[")))
	type=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((stream):upTo_("]"))
	kind=_s8_value; return _s8_value
end)()
;
(function() if ((((stream):atEnd()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Invalid brackets for array"))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((kind):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((type)," withAll: ("))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((kind):includes_(",")))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((type)," of: ")),(self):apiWrapperFor_(kind)))," withAll: ("))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((type)," of: ")),smalltalk:printString_((kind))))," withAll: ("))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","apiPrologueForArray: aReturnType\r\n\t\" Private - Return the prologue to use to wrap an arrayed return.\r\n\tThe returned code will be like:\r\n\t\tGlobalName withAll: (\r\n\t\tGlobalName of: ElementClass withAll: (\r\n\t\tGlobalName of: 'Type1,Type2,Type3' withAll: (\r\n\t\"\r\n\r\n\t| type stream kind |\r\n\tstream := aReturnType stream.\r\n\ttype := self apiWrapperFor: (stream upTo: $[).\r\n\tkind := stream upTo: $].\r\n\tstream atEnd ifFalse: [ ^self error: 'Invalid brackets for array' ].\r\n\tkind isEmpty ifTrue: [ ^type ,' withAll: ('].\r\n\t(kind includes: $,) ifFalse: [\r\n\t\t^type ,' of: ' ,(self apiWrapperFor: kind) ,' withAll: ('\r\n\t].\t\r\n\t^type ,' of: ' ,kind printString ,' withAll: ('","(\r\nfunction(self,aReturnType)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal type=nil\r\nlocal stream=nil\r\nlocal kind=nil\r\n(function()\r\n\tlocal _s8_value=((aReturnType):stream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):apiWrapperFor_((stream):upTo_(\"[\")))\r\n\ttype=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((stream):upTo_(\"]\"))\r\n\tkind=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((stream):atEnd()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Invalid brackets for array\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((kind):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((type),\" withAll: (\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((kind):includes_(\",\")))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((type),\" of: \")),(self):apiWrapperFor_(kind))),\" withAll: (\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((type),\" of: \")),smalltalk:printString_((kind)))),\" withAll: (\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"apiPrologueForTuple:",0
,(
function(self,aReturnType)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local type=nil
local stream=nil
local kind=nil
(function()
	local _s8_value=((aReturnType):stream())
	stream=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):apiWrapperFor_((stream):upTo_("<")))
	type=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((stream):upTo_(">"))
	kind=_s8_value; return _s8_value
end)()
;
(function() if ((((stream):atEnd()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Invalid code for typle"))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function() if ((((kind):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Empty types for tuple"))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((kind):includes_(",")))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((type)," of: ")),(self):apiWrapperFor_(kind)))," with: ("))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((type)," of: ")),smalltalk:printString_((kind))))," with: ("))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","apiPrologueForTuple: aReturnType\r\n\t\" Private - Return the prologue to use to wrap a tuple return.\r\n\tThe returned code will be like:\r\n\t\tGlobalName of: 'Type1,Type2,Type3' with: (\r\n\t\"\r\n\r\n\t| type stream kind |\r\n\tstream := aReturnType stream.\r\n\ttype := self apiWrapperFor: (stream upTo: $<).\r\n\tkind := stream upTo: $>.\r\n\tstream atEnd ifFalse: [ ^self error: 'Invalid code for typle' ].\r\n\tkind isEmpty ifTrue:  [ ^self error: 'Empty types for tuple' ].\r\n\t(kind includes: $,) ifFalse: [\r\n\t\t^type ,' of: ' ,(self apiWrapperFor: kind) ,' with: ('\r\n\t].\t\r\n\t^type ,' of: ' ,kind printString ,' with: ('","(\r\nfunction(self,aReturnType)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal type=nil\r\nlocal stream=nil\r\nlocal kind=nil\r\n(function()\r\n\tlocal _s8_value=((aReturnType):stream())\r\n\tstream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):apiWrapperFor_((stream):upTo_(\"<\")))\r\n\ttype=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((stream):upTo_(\">\"))\r\n\tkind=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((stream):atEnd()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Invalid code for typle\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function() if ((((kind):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Empty types for tuple\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((kind):includes_(\",\")))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((type),\" of: \")),(self):apiWrapperFor_(kind))),\" with: (\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((type),\" of: \")),smalltalk:printString_((kind)))),\" with: (\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"apiWrapperFor:",0
,(
function(self,aName)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(smalltalk.Smalltalk):at_ifAbsent_(aName, (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):defaultAPIWrapperFor_(aName))
end
)()
error(_s8Ret_,0)
end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (aName)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","apiWrapperFor: aName\r\n\t\" Private - Return the name to use to wrap API structures of type aName. \"\r\n\r\n\tSmalltalk at: aName ifAbsent: [ ^self defaultAPIWrapperFor: aName ].\r\n\t^aName","(\r\nfunction(self,aName)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(smalltalk.Smalltalk):at_ifAbsent_(aName, (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):defaultAPIWrapperFor_(aName))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (aName)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"defaultAPIWrapperFor:",0
,(
function(self,aName)
return (aName)
;
end
)
,"builders","defaultAPIWrapperFor: aName\r\n\t\" Private - Return the default name to use to wrap API structures of type aName.\r\n\tDefault implementation will defer lookup upto runtime.\r\n\t\"\r\n\r\n\t^aName","(\r\nfunction(self,aName)\r\nreturn (aName)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildFunction:",0
,(
function(self,spec)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((spec):isArray()))==true) then return ((function()
(function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((spec):size()),(3)))),(function()
return (((spec):last()):isString())
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildFunction_((smalltalk.Array):with_with_((smalltalk.Array):with_with_((spec):first(), (spec):second()), (spec):last())))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((spec):size()),(2)))),(function()
return (((spec):last()):isString())
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildFunction_prologue_epilogue_((spec):first(), (self):apiPrologueFor_((spec):second()), ")"))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildFunction_prologue_epilogue_(spec, "", ""))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildFunction: spec\r\n\t\" Private - Generate code from spec as functions on instance handle.\r\n\tspec can be:\r\n\t\taSymbol (e.g. any smalltalk message selector)\r\n\t\t#( selector #ReturnType )\r\n\t\t#( selector #( argumentNames ) #ReturnType )\r\n\t\t#( #( selector #( argumentNames ) ) #ReturnType )\r\n\t\t#( selector #( argumentNames ) )\r\n\targumentNames must match selector pattern\r\n\tif argumentNames is not given, it will be computed from selector pattern\r\n\tThe function will return the result of calling the handle, wrapped to a ReturnType\r\n\tSee implementation of #apiPrologueFor: for documentation on #ReturnType\r\n\t\"\r\n\r\n\tspec isArray ifTrue: [\r\n\t    (spec size = 3 and: [spec last isString]) ifTrue: [ \"#( selector #( argumentNames ) #ReturnType )\"\r\n\t\t^self buildFunction: (Array\r\n\t\t\twith: (Array with: spec first with: spec second)\r\n\t\t\twith: spec last)\r\n\t    ]. \r\n\t    (spec size = 2 and: [spec last isString]) ifTrue: [ \"#( selector #ReturnType )\"\r\n\t\t^self\r\n\t\t\tbuildFunction: spec first\r\n\t\t\tprologue: (self apiPrologueFor: spec second)\r\n\t\t\tepilogue: ')'\r\n\t    ]. \r\n\t].\r\n\t^self buildFunction: spec prologue: '' epilogue: ''","(\r\nfunction(self,spec)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((spec):isArray()))==true) then return ((function()\r\n(function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((spec):size()),(3)))),(function()\r\nreturn (((spec):last()):isString())\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildFunction_((smalltalk.Array):with_with_((smalltalk.Array):with_with_((spec):first(), (spec):second()), (spec):last())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((spec):size()),(2)))),(function()\r\nreturn (((spec):last()):isString())\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildFunction_prologue_epilogue_((spec):first(), (self):apiPrologueFor_((spec):second()), \")\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildFunction_prologue_epilogue_(spec, \"\", \"\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildFunction:prologue:epilogue:",0
,(
function(self,spec,prologue,epilogue)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local args=nil
(function() if ((((spec):isArray()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildFunction_arguments_prologue_epilogue_((spec):first(), (spec):last(), prologue, epilogue))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((((1)):to_((spec):occurrencesOf_(":"))):collect_((function(i)
return (smalltalk:object_comma_(("arg"),smalltalk:asString_((i))))
end)))
	args=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((args):size()),(1)))))==true) then return ((function()
return ((function()
	local _s8_value=(smalltalk:newArray({"anObject"}))
	args=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildFunction_arguments_prologue_epilogue_(spec, args, prologue, epilogue))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildFunction: spec prologue: prologue epilogue: epilogue \r\n\t\" Private - Generate code for spec to apply on instance handle. \"\r\n\r\n\t| args |\r\n\tspec isArray ifTrue: [ \"#( selector #(argumentNames) )\"\r\n\t\t^self\tbuildFunction: spec first\r\n\t\t\targuments: spec last\r\n\t\t\tprologue: prologue epilogue: epilogue\r\n\t].\r\n\targs :=  (1 to: (spec occurrencesOf: $:)) collect: [:i| #arg , i asString ].\r\n\targs size = 1 ifTrue: [ args := #( #anObject ) ].\r\n\t^self\r\n\t\tbuildFunction: spec arguments: args\r\n\t\tprologue: prologue epilogue: epilogue  ","(\r\nfunction(self,spec,prologue,epilogue)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal args=nil\r\n(function() if ((((spec):isArray()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildFunction_arguments_prologue_epilogue_((spec):first(), (spec):last(), prologue, epilogue))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((1)):to_((spec):occurrencesOf_(\":\"))):collect_((function(i)\r\nreturn (smalltalk:object_comma_((\"arg\"),smalltalk:asString_((i))))\r\nend)))\r\n\targs=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((args):size()),(1)))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:newArray({\"anObject\"}))\r\n\targs=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildFunction_arguments_prologue_epilogue_(spec, args, prologue, epilogue))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"apiSelector:forArguments:",0
,(
function(self,callName,args)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local parts=nil
(function() if ((((args):isEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (callName)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((callName):asArrayOfSubstringsSeparatedBy_(":"))
	parts=_s8_value; return _s8_value
end)()
;
(function() if (((((parts):last()):isEmpty()))==true) then return ((function()
return ((parts):removeLast())
end))() else return nil end end)()
;
(((parts):size()):_plus((1))):to_do_((args):size(), (function(i)
local argName=nil
(function()
	local _s8_value=((args):at_(i))
	argName=_s8_value; return _s8_value
end)();(function() if ((((argName):isArray()))==true) then return ((function()
return ((function()
	local _s8_value=((argName):first())
	argName=_s8_value; return _s8_value
end)())
end))() else return nil end end)();return ((parts):add_(argName))
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((parts):inject_into_("", (function(total,part)
return (smalltalk:object_comma_((smalltalk:object_comma_((total),part)),":"))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","apiSelector: callName forArguments: args\r\n\t\" Private - Return the api selector to implement a native call with arguments. \"\r\n\r\n\t| parts |\r\n\targs isEmpty ifTrue: [ ^callName ].\r\n\tparts := callName asArrayOfSubstringsSeparatedBy: $:.\r\n\tparts last isEmpty ifTrue: [ parts removeLast ].\r\n\tparts size + 1 to: args size do: [:i| | argName |\r\n\t\targName := args at: i.\r\n\t\targName isArray ifTrue: [ argName := argName first ].\r\n\t\tparts add: argName\r\n\t].\r\n\t^parts inject: '' into: [:total :part| total ,part ,$: ]","(\r\nfunction(self,callName,args)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal parts=nil\r\n(function() if ((((args):isEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (callName)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((callName):asArrayOfSubstringsSeparatedBy_(\":\"))\r\n\tparts=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((((parts):last()):isEmpty()))==true) then return ((function()\r\nreturn ((parts):removeLast())\r\nend))() else return nil end end)()\r\n;\r\n(((parts):size()):_plus((1))):to_do_((args):size(), (function(i)\r\nlocal argName=nil\r\n(function()\r\n\tlocal _s8_value=((args):at_(i))\r\n\targName=_s8_value; return _s8_value\r\nend)();(function() if ((((argName):isArray()))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((argName):first())\r\n\targName=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)();return ((parts):add_(argName))\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((parts):inject_into_(\"\", (function(total,part)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((total),part)),\":\"))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"apiArgumentsFrom:",0
,(
function(self,array)
local tag=nil
(function()
	local _s8_value=("_")
	tag=_s8_value; return _s8_value
end)()
;
return ((array):collect_((function(each)
return ((function() if ((((smalltalk:boolean_and_(((each):isString()),(function()
return ((smalltalk:isObject_equalTo_(((each):first()),tag)))
end)))))==true) then return ((function()
local varName=nil
(function()
	local _s8_value=((each):copyFrom_to_((2), (each):size()))
	varName=_s8_value; return _s8_value
end)();return ((function() if ((((smalltalk:isObject_equalTo_(((each):last()),tag))))==true) then return ((function()
return ((smalltalk.Array):with_with_((varName):copyFrom_to_((1), ((varName):size()):_minus((1))), "handles"))
end))() else return ((function()
return ((smalltalk.Array):with_with_(varName, "json"))
end))() end end)())
end))() else return ((function()
return (each)
end))() end end)())
end)))
;
end
)
,"builders","apiArgumentsFrom: array\r\n\t\" Private - Return the arguments for API call. \"\r\n\r\n\t| tag |\r\n\ttag := $_.\r\n\t^array collect: [:each|\r\n\t\t(each isString and: [ each first = tag ]) ifTrue: [ | varName |\r\n\t\t\tvarName := each copyFrom: 2 to: each size.\r\n\t\t\teach last = tag\r\n\t\t\tifTrue: [ Array with: (varName copyFrom: 1 to: varName size - 1) with: #handles ]\r\n\t\t\tifFalse: [ Array with: varName with: #json ]\r\n\t\t] ifFalse: [ each ]\r\n\t]","(\r\nfunction(self,array)\r\nlocal tag=nil\r\n(function()\r\n\tlocal _s8_value=(\"_\")\r\n\ttag=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((array):collect_((function(each)\r\nreturn ((function() if ((((smalltalk:boolean_and_(((each):isString()),(function()\r\nreturn ((smalltalk:isObject_equalTo_(((each):first()),tag)))\r\nend)))))==true) then return ((function()\r\nlocal varName=nil\r\n(function()\r\n\tlocal _s8_value=((each):copyFrom_to_((2), (each):size()))\r\n\tvarName=_s8_value; return _s8_value\r\nend)();return ((function() if ((((smalltalk:isObject_equalTo_(((each):last()),tag))))==true) then return ((function()\r\nreturn ((smalltalk.Array):with_with_((varName):copyFrom_to_((1), ((varName):size()):_minus((1))), \"handles\"))\r\nend))() else return ((function()\r\nreturn ((smalltalk.Array):with_with_(varName, \"json\"))\r\nend))() end end)())\r\nend))() else return ((function()\r\nreturn (each)\r\nend))() end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"apiSelectorAndFunctionName:forArguments:",0
,(
function(self,callName,args)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local msgSelector=nil
local apiSelector=nil
(function() if ((((smalltalk:boolean_or_(((callName):isString()),(function()
return ((callName):isArray())
end)))))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("CallName must be aString or anArray"))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
	local _s8_value=((self):apiSelector_forArguments_((function() if ((((callName):isString()))==true) then return ((function()
return ((self):apiMessage_for_(callName, args))
end))() else return ((function()
return ((self):apiMessage_for_((callName):first(), args))
end))() end end)(), args))
	msgSelector=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):apiSelector_forArguments_((function() if ((((callName):isString()))==true) then return ((function()
return (callName)
end))() else return ((function()
return ((callName):last())
end))() end end)(), args))
	apiSelector=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Array):with_with_(msgSelector, apiSelector))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","apiSelectorAndFunctionName: callName forArguments: args\r\n\t\" Private - Return the selector and api call to implement a native call with arguments. \"\r\n\r\n\t| msgSelector apiSelector |\r\n\t(callName isString or: [callName isArray]) ifFalse: [\r\n\t\t^self error: 'CallName must be aString or anArray'\r\n\t].\r\n\tmsgSelector := self apiSelector: (callName isString\r\n\t\tifTrue: [ self apiMessage: callName for: args ]\r\n\t\tifFalse: [ self apiMessage: callName first for: args ]) forArguments: args.\r\n\tapiSelector := self apiSelector: (callName isString\r\n\t\tifTrue: [ callName ] ifFalse: [ callName last ]) forArguments: args.\r\n\r\n\t^Array with: msgSelector with: apiSelector","(\r\nfunction(self,callName,args)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal msgSelector=nil\r\nlocal apiSelector=nil\r\n(function() if ((((smalltalk:boolean_or_(((callName):isString()),(function()\r\nreturn ((callName):isArray())\r\nend)))))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"CallName must be aString or anArray\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):apiSelector_forArguments_((function() if ((((callName):isString()))==true) then return ((function()\r\nreturn ((self):apiMessage_for_(callName, args))\r\nend))() else return ((function()\r\nreturn ((self):apiMessage_for_((callName):first(), args))\r\nend))() end end)(), args))\r\n\tmsgSelector=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):apiSelector_forArguments_((function() if ((((callName):isString()))==true) then return ((function()\r\nreturn (callName)\r\nend))() else return ((function()\r\nreturn ((callName):last())\r\nend))() end end)(), args))\r\n\tapiSelector=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Array):with_with_(msgSelector, apiSelector))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"apiMessage:for:",0
,(
function(self,callName,args)
local result=nil
(function()
	local _s8_value=(callName)
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((self):nativeLanguage()),"lua"))))==true) then return ((function()
return ((function() if ((((callName):startsWith_("_")))==true) then return ((function()
return ((function()
	local _s8_value=((callName):copyFrom_to_((2), (callName):size()))
	result=_s8_value; return _s8_value
end)())
end))() else return nil end end)())
end))() else return nil end end)()
;
return ((function() if ((((result):includes_(":")))==true) then return ((function()
return (result)
end))() else return ((function()
return ((result):asLocalName())
end))() end end)())
;
end
)
,"builders","apiMessage: callName for: args\r\n\t\" Private - Return the message to implement for calling function callName.\r\n\tThe default action use the callName as local name.\r\n\tIt can be refined by subclasses to edit the name for better smalltalk looking of the message.\r\n\t\"\r\n\r\n\t| result |\r\n\tresult := callName.\r\n\tself nativeLanguage = #lua ifTrue: [\r\n\t\t(callName startsWith: $_) ifTrue: [ result := callName copyFrom: 2 to: callName size ].\r\n\t].\r\n\t^(result includes: $:) ifTrue: [ result ] ifFalse: [ result asLocalName ]","(\r\nfunction(self,callName,args)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=(callName)\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((self):nativeLanguage()),\"lua\"))))==true) then return ((function()\r\nreturn ((function() if ((((callName):startsWith_(\"_\")))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((callName):copyFrom_to_((2), (callName):size()))\r\n\tresult=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return nil end end)()\r\n;\r\nreturn ((function() if ((((result):includes_(\":\")))==true) then return ((function()\r\nreturn (result)\r\nend))() else return ((function()\r\nreturn ((result):asLocalName())\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"apiEncode:call:arguments:on:header:",0
,(
function(self,delegate,aSelector,array,code,header)
local pair=nil
local args=nil
local msgStream=nil
local apiStream=nil
local argStream=nil
local msg=nil
(function()
	local _s8_value=((self):apiArgumentsFrom_(array))
	args=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((self):apiSelectorAndFunctionName_forArguments_(aSelector, args))
	pair=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((pair):first()):stream())
	msgStream=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(((pair):last()):stream())
	apiStream=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((args):stream())
	argStream=_s8_value; return _s8_value
end)()
;
(code):nextPutAll_(delegate)
;
(code):nextPut_((apiStream):next())
;
(header):nextPut_((msgStream):next())
;
(function() while(not ((function()
return ((argStream):atEnd())
end))()) do (function()
local param=nil
local expr=nil
(function() if (((((argStream):peek()):isString()))==true) then return ((function()
(function()
	local _s8_value=((argStream):next())
	param=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=(param)
	expr=_s8_value; return _s8_value
end)())
end))() else return ((function()
(function()
	local _s8_value=(((argStream):peek()):first())
	param=_s8_value; return _s8_value
end)();return ((function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((param)," ")),((argStream):next()):last()))
	expr=_s8_value; return _s8_value
end)())
end))() end end)();(code):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((apiStream):upTo_(":")),": ")),expr))," "));return ((header):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((msgStream):upTo_(":")),": ")),param))," ")))
end)() end;return nil end)()
;
(code):nextPutAll_((apiStream):upToEnd())
;
(header):nextPutAll_((msgStream):upToEnd())
;
return (self)
;
end
)
,"builders","apiEncode: delegate\r\n\tcall: aSelector arguments: array\r\n\ton: code header: header\r\n\t\" Private - Generate code to call function onto code&header streams. \"\r\n\r\n\t| pair args msgStream apiStream argStream msg |\r\n\targs := self apiArgumentsFrom: array.\r\n\tpair := self apiSelectorAndFunctionName: aSelector forArguments: args.\r\n\tmsgStream := pair first stream.\r\n\tapiStream := pair last stream.\r\n\targStream := args stream.\r\n\r\n\tcode\tnextPutAll: delegate.\r\n\tcode\tnextPut: apiStream next.\r\n\theader\tnextPut: msgStream next.\r\n\r\n\t[ argStream atEnd ] whileFalse: [ | param expr |\r\n\t\targStream peek isString\r\n\t\tifTrue: [ param := argStream next. expr := param ]\r\n\t\tifFalse: [ param := argStream peek first. expr := param ,' ' ,argStream next last. ].\r\n\t\tcode   nextPutAll: (apiStream upTo: $:) ,': ' ,expr ,' '.\r\n\t\theader nextPutAll: (msgStream upTo: $:) ,': ' ,param ,' '.\r\n\t].\r\n\r\n\tcode   nextPutAll: apiStream upToEnd.\r\n\theader nextPutAll: msgStream upToEnd.","(\r\nfunction(self,delegate,aSelector,array,code,header)\r\nlocal pair=nil\r\nlocal args=nil\r\nlocal msgStream=nil\r\nlocal apiStream=nil\r\nlocal argStream=nil\r\nlocal msg=nil\r\n(function()\r\n\tlocal _s8_value=((self):apiArgumentsFrom_(array))\r\n\targs=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((self):apiSelectorAndFunctionName_forArguments_(aSelector, args))\r\n\tpair=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((pair):first()):stream())\r\n\tmsgStream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(((pair):last()):stream())\r\n\tapiStream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((args):stream())\r\n\targStream=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(code):nextPutAll_(delegate)\r\n;\r\n(code):nextPut_((apiStream):next())\r\n;\r\n(header):nextPut_((msgStream):next())\r\n;\r\n(function() while(not ((function()\r\nreturn ((argStream):atEnd())\r\nend))()) do (function()\r\nlocal param=nil\r\nlocal expr=nil\r\n(function() if (((((argStream):peek()):isString()))==true) then return ((function()\r\n(function()\r\n\tlocal _s8_value=((argStream):next())\r\n\tparam=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=(param)\r\n\texpr=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\n(function()\r\n\tlocal _s8_value=(((argStream):peek()):first())\r\n\tparam=_s8_value; return _s8_value\r\nend)();return ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((param),\" \")),((argStream):next()):last()))\r\n\texpr=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)();(code):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((apiStream):upTo_(\":\")),\": \")),expr)),\" \"));return ((header):nextPutAll_(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(((msgStream):upTo_(\":\")),\": \")),param)),\" \")))\r\nend)() end;return nil end)()\r\n;\r\n(code):nextPutAll_((apiStream):upToEnd())\r\n;\r\n(header):nextPutAll_((msgStream):upToEnd())\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"codeForInstanceReceiver",0
,(
function(self)
return ("self handle #")
;
end
)
,"private","codeForInstanceReceiver\r\n\t\" Private - Returns the code for the receiver of instance function call.\r\n\tDefault answer asumes a wrapper calling with syntax sugar.\r\n\tThis method can be refined to adapt to custom platform calls.\r\n\t\"\r\n\r\n\t^'self handle #'","(\r\nfunction(self)\r\nreturn (\"self handle #\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildFunction:arguments:prologue:epilogue:",0
,(
function(self,spec,args,prologue,epilogue)
local header=nil
local code=nil
(function()
	local _s8_value=((""):writeStream())
	header=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((""):writeStream())
	code=_s8_value; return _s8_value
end)()
;
(
function(_s8_rec)
(_s8_rec):apiEncode_call_arguments_on_header_((self):codeForInstanceReceiver(), spec, args, code, header);
return ((_s8_rec):implement_comment_as_category_((self):stMessageHeader_((header):contents()), smalltalk:object_comma_((smalltalk:object_comma_(("Delegate "),spec))," to handle."), smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("^"),prologue)),(code):contents())),epilogue), "wrappers"))
end
)(self)
;
return (self)
;
end
)
,"builders","buildFunction: spec arguments: args prologue: prologue epilogue: epilogue \r\n\t\" Private - Generate code to delegate message on instance handle. \"\r\n\r\n\t| header code |\r\n\theader := '' writeStream. code := '' writeStream.\r\n\tself\r\n\t  apiEncode: self codeForInstanceReceiver\r\n\t\tcall: spec arguments: args\r\n\t\ton: code header: header;\r\n\t  implement: (self stMessageHeader: header contents)\r\n\t\tcomment: 'Delegate ' ,spec ,' to handle.'\r\n\t\tas: '^'\t, prologue\r\n\t\t\t, code contents\r\n\t\t\t, epilogue\r\n\t\tcategory: #wrappers","(\r\nfunction(self,spec,args,prologue,epilogue)\r\nlocal header=nil\r\nlocal code=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\theader=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tcode=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(\r\nfunction(_s8_rec)\r\n(_s8_rec):apiEncode_call_arguments_on_header_((self):codeForInstanceReceiver(), spec, args, code, header);\r\nreturn ((_s8_rec):implement_comment_as_category_((self):stMessageHeader_((header):contents()), smalltalk:object_comma_((smalltalk:object_comma_((\"Delegate \"),spec)),\" to handle.\"), smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"^\"),prologue)),(code):contents())),epilogue), \"wrappers\"))\r\nend\r\n)(self)\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"codeForStaticReceiver",0
,(
function(self)
return ("self nativeClass handle #")
;
end
)
,"private","codeForStaticReceiver\r\n\t\" Private - Returns the code for the receiver of static function call.\r\n\tDefault answer asumes a nativeClass wrapper calling with syntax sugar.\r\n\tThis method can be refined to adapt to custom platform calls.\r\n\t\"\r\n\r\n\t^'self nativeClass handle #'","(\r\nfunction(self)\r\nreturn (\"self nativeClass handle #\")\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"nativeClass",0
,(
function(self)
return ((self):implementedBySubclass_("nativeClass"))
;
end
)
,"private","nativeClass\r\n\t\" Private - Returns the wrapper for native binding of the receiver. \"\r\n\r\n\t^self implementedBySubclass: #nativeClass","(\r\nfunction(self)\r\nreturn ((self):implementedBySubclass_(\"nativeClass\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildStaticFunctions:",0
,(
function(self,specs)
(self):build_doing_(("StaticFunctions"):_minus_gt(specs), (function()
return ((specs):do_((function(each)
return ((self):buildStaticFunction_(each))
end)))
end))
;
return (self)
;
end
)
,"builders","buildStaticFunctions: specs\r\n\t\" Generate code from specs as functions implemented implemented by native class. \"\r\n\r\n\tself build: #StaticFunctions -> specs doing: [\r\n\t\tspecs do: [:each| self buildStaticFunction: each ]\r\n\t]","(\r\nfunction(self,specs)\r\n(self):build_doing_((\"StaticFunctions\"):_minus_gt(specs), (function()\r\nreturn ((specs):do_((function(each)\r\nreturn ((self):buildStaticFunction_(each))\r\nend)))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildStaticFunction:",0
,(
function(self,spec)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((spec):isArray()))==true) then return ((function()
(function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((spec):size()),(3)))),(function()
return (((spec):last()):isString())
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildStaticFunction_((smalltalk.Array):with_with_((smalltalk.Array):with_with_((spec):first(), (spec):second()), (spec):last())))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)();return ((function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((spec):size()),(2)))),(function()
return (((spec):last()):isString())
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildStaticFunction_prologue_epilogue_((spec):first(), (self):apiPrologueFor_((spec):second()), ")"))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildStaticFunction_prologue_epilogue_(spec, "", ""))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildStaticFunction: spec\r\n\t\" Private - Generate code to delegate message implemented as functions on native class. \"\r\n\r\n\tspec isArray ifTrue: [\r\n\t    (spec size = 3 and: [spec last isString]) ifTrue: [ \"#( selector #( argumentNames ) #RETURNClass )\"\r\n\t\t^self buildStaticFunction: (Array\r\n\t\t\twith: (Array with: spec first with: spec second)\r\n\t\t\twith: spec last)\r\n\t    ]. \r\n\t    (spec size = 2 and: [spec last isString]) ifTrue: [ \"#( selector #RETURNClass )\"\r\n\t\t^self\r\n\t\t\tbuildStaticFunction: spec first\r\n\t\t\tprologue: (self apiPrologueFor: spec second)\r\n\t\t\tepilogue: ')'\r\n\t    ]. \r\n\t].\r\n\t^self buildStaticFunction: spec prologue: '' epilogue: ''","(\r\nfunction(self,spec)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((spec):isArray()))==true) then return ((function()\r\n(function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((spec):size()),(3)))),(function()\r\nreturn (((spec):last()):isString())\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildStaticFunction_((smalltalk.Array):with_with_((smalltalk.Array):with_with_((spec):first(), (spec):second()), (spec):last())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)();return ((function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_(((spec):size()),(2)))),(function()\r\nreturn (((spec):last()):isString())\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildStaticFunction_prologue_epilogue_((spec):first(), (self):apiPrologueFor_((spec):second()), \")\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildStaticFunction_prologue_epilogue_(spec, \"\", \"\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildStaticFunction:prologue:epilogue:",0
,(
function(self,spec,prologue,epilogue)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local args=nil
(function() if ((((spec):isArray()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildStaticFunction_arguments_prologue_epilogue_((spec):first(), (spec):last(), prologue, epilogue))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((((1)):to_((spec):occurrencesOf_(":"))):collect_((function(i)
return (smalltalk:object_comma_(("arg"),smalltalk:asString_((i))))
end)))
	args=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_(((args):size()),(1)))))==true) then return ((function()
return ((function()
	local _s8_value=(smalltalk:newArray({"anObject"}))
	args=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):buildStaticFunction_arguments_prologue_epilogue_(spec, args, prologue, epilogue))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builders","buildStaticFunction: spec prologue: prologue epilogue: epilogue \r\n\t\" Private - Generate code to delegate message on native class. \"\r\n\r\n\t| args |\r\n\tspec isArray ifTrue: [ \"#( selector #(argumentNames) )\"\r\n\t\t^self\tbuildStaticFunction: spec first\r\n\t\t\targuments: spec last\r\n\t\t\tprologue: prologue epilogue: epilogue\r\n\t].\r\n\targs :=  (1 to: (spec occurrencesOf: $:)) collect: [:i| #arg , i asString ].\r\n\targs size = 1 ifTrue: [ args := #( #anObject ) ].\r\n\t^self\r\n\t\tbuildStaticFunction: spec arguments: args\r\n\t\tprologue: prologue epilogue: epilogue ","(\r\nfunction(self,spec,prologue,epilogue)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal args=nil\r\n(function() if ((((spec):isArray()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildStaticFunction_arguments_prologue_epilogue_((spec):first(), (spec):last(), prologue, epilogue))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((((1)):to_((spec):occurrencesOf_(\":\"))):collect_((function(i)\r\nreturn (smalltalk:object_comma_((\"arg\"),smalltalk:asString_((i))))\r\nend)))\r\n\targs=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_(((args):size()),(1)))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:newArray({\"anObject\"}))\r\n\targs=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):buildStaticFunction_arguments_prologue_epilogue_(spec, args, prologue, epilogue))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildStaticFunction:arguments:prologue:epilogue:",0
,(
function(self,spec,args,prologue,epilogue)
local header=nil
local code=nil
(function()
	local _s8_value=((""):writeStream())
	header=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((""):writeStream())
	code=_s8_value; return _s8_value
end)()
;
(self):apiEncode_call_arguments_on_header_((self):codeForStaticReceiver(), spec, args, code, header)
;
((self)["$klass"]):implement_comment_as_category_((self):stMessageHeader_((header):contents()), "Delegated to native class.", smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("^"),prologue)),(code):contents())),epilogue), "wrappers")
;
return (self)
;
end
)
,"builders","buildStaticFunction: spec arguments: args prologue: prologue epilogue: epilogue \r\n\t\" Private - Generate code to delegate message on native class. \"\r\n\r\n\t| header code |\r\n\theader := '' writeStream. code := '' writeStream.\r\n\tself\r\n\t  apiEncode: self codeForStaticReceiver\r\n\t\tcall: spec arguments: args\r\n\t\ton: code header: header.\r\n\tself class\r\n\t\timplement: (self stMessageHeader: header contents)\r\n\t\t  comment: 'Delegated to native class.'\r\n\t\t  as: '^'\r\n\t\t\t, prologue\r\n\t\t\t, code contents\r\n\t\t\t, epilogue\r\n\t\t  category: #wrappers","(\r\nfunction(self,spec,args,prologue,epilogue)\r\nlocal header=nil\r\nlocal code=nil\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\theader=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((\"\"):writeStream())\r\n\tcode=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):apiEncode_call_arguments_on_header_((self):codeForStaticReceiver(), spec, args, code, header)\r\n;\r\n((self)[\"$klass\"]):implement_comment_as_category_((self):stMessageHeader_((header):contents()), \"Delegated to native class.\", smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"^\"),prologue)),(code):contents())),epilogue), \"wrappers\")\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"@","_at"
,(
function(self,handle)
return (((self):basicNew()):handle_(handle))
;
end
)
,"instantiation","@ handle\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self basicNew handle: handle","(\r\nfunction(self,handle)\r\nreturn (((self):basicNew()):handle_(handle))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"empty",0
,(
function(self)
return ((self):_at(((smalltalk.Object):new()):json()))
;
end
)
,"instantiation","empty\r\n\t\" Return an empty instance of the receiver. \"\r\n\r\n\t^self @ Object new json","(\r\nfunction(self)\r\nreturn ((self):_at(((smalltalk.Object):new()):json()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"for:",0
,(
function(self,handle)
return (((self):basicNew()):handle_(handle))
;
end
)
,"instantiation","for: handle\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t^self basicNew handle: handle","(\r\nfunction(self,handle)\r\nreturn (((self):basicNew()):handle_(handle))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"withAll:",0
,(
function(self,handles)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):isObject_(handles)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):withAll_((smalltalk.Array):_at(handles)))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((handles):collect_((function(each)
return ((self):_at(each))
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"instantiation","withAll: handles\r\n\t\" Return a collection of instance of the receiver with handles. \"\r\n\r\n\t(self isObject: handles) ifFalse: [ ^self withAll: Array @ handles ].\r\n\t^handles collect: [:each| self @ each ]","(\r\nfunction(self,handles)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):isObject_(handles)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):withAll_((smalltalk.Array):_at(handles)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((handles):collect_((function(each)\r\nreturn ((self):_at(each))\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"of:withAll:",0
,(
function(self,aType,handles)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((self):isObject_(handles)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):of_withAll_(aType, (smalltalk.Array):_at(handles)))
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):withAll_(handles))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"instantiation","of: aType withAll: handles\r\n\t\" Return an instance of the receiver wrapping handles with a type (or nil).\r\n\tDefault implementation ignore the type.\r\n\tThis method can be refined by subclasses to support typed wrappers.\r\n\taType can be a class for type of elements or aString of comma separated type names.\r\n\t\"\r\n\r\n\t(self isObject: handles) ifFalse: [ ^self of: aType withAll: Array @ handles ].\r\n\t^self withAll: handles","(\r\nfunction(self,aType,handles)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((self):isObject_(handles)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):of_withAll_(aType, (smalltalk.Array):_at(handles)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):withAll_(handles))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"of:with:",0
,(
function(self,aType,aHandle)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((((aHandle))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):basicOf_with_(aType, aHandle))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"instantiation","of: aType with: aHandle\r\n\t\" Return an instance of the receiver wrapping aHandle with a type (or nil). \"\r\n\r\n\taHandle isNil ifTrue: [ ^nil ].\r\n\t^self basicOf: aType with: aHandle","(\r\nfunction(self,aType,aHandle)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((((aHandle))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):basicOf_with_(aType, aHandle))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"basicOf:with:",0
,(
function(self,aType,aHandle)
return ((self):implementedBySubclass_("basicOf:with:"))
;
end
)
,"instantiation","basicOf: aType with: aHandle\r\n\t\" Private - Return an instance of the receiver wrapping aHandle with a type.\r\n\taType can be a class for type of element or aString of comma separated type names.\r\n\t\"\r\n\r\n\t^self implementedBySubclass: #basicOf:with:","(\r\nfunction(self,aType,aHandle)\r\nreturn ((self):implementedBySubclass_(\"basicOf:with:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"html5Events",0
,(
function(self)
return (smalltalk:newArray({}))
;
end
)
,"events","html5Events\r\n\t\" Private - Return the html5 events+notifications managed by instances of the receiver.\r\n\tDefault answer is empty, this method can be refined by subclasses to handle html5 events.\r\n\t\"\r\n\t^#()","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"localEvents",0
,(
function(self)
return (smalltalk:newArray({}))
;
end
)
,"events","localEvents\r\n\t\" Private - Return the smalltalk events managed by instances of the receiver. \"\r\n\r\n\t^#()","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"constructEventsTriggered",0
,(
function(self)
return (smalltalk:object_comma_((smalltalk:object_comma_(((self):html5Events()),(self):localEvents())),smalltalk:superImplementor(smalltalk.NativeObject["$klass"],"constructEventsTriggered")(self)))
;
end
)
,"events","constructEventsTriggered\r\n\t\" Private - Return the events managed by instances of the receiver. \"\r\n\r\n\t^self html5Events\r\n\t,self localEvents\r\n\t,super constructEventsTriggered","(\r\nfunction(self)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_(((self):html5Events()),(self):localEvents())),smalltalk:superImplementor(smalltalk.NativeObject[\"$klass\"],\"constructEventsTriggered\")(self)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildHTML5Events:",0
,(
function(self,selectors)
(self):build_doing_(("HTML5Events"):_minus_gt(selectors), (function()
((self)["$klass"]):implement_comment_as_category_("html5Events", "Private - Return the html5 events managed by instances of the receiver.", smalltalk:object_comma_((smalltalk:object_comma_(("^#( "),(selectors):inject_into_("", (function(total,each)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((total),"#")),each))," "))
end)))),") , super html5Events"), "events");return ((self):buildGetters_((selectors):collect_((function(each)
return ((self):selectorForHTML5Event_(each))
end))))
end))
;
return (self)
;
end
)
,"builder","buildHTML5Events: selectors\r\n\t\" Private - Generate code to route events to handle. \"\r\n\r\n\tself build: #HTML5Events -> selectors doing: [\r\n\t\tself class\r\n\t\t\timplement: #html5Events\r\n\t\t\tcomment: 'Private - Return the html5 events managed by instances of the receiver.'\r\n\t\t\tas: '^#( ' ,(selectors inject: '' into: [:total :each|\r\n\t\t\t\ttotal ,'#',each ,' '\r\n\t\t\t\t]),') , super html5Events'\r\n\t\t\tcategory: #events.\r\n\t\tself buildGetters: (selectors collect: [:each|\r\n\t\t\tself selectorForHTML5Event: each\r\n\t\t]).\r\n\t]","(\r\nfunction(self,selectors)\r\n(self):build_doing_((\"HTML5Events\"):_minus_gt(selectors), (function()\r\n((self)[\"$klass\"]):implement_comment_as_category_(\"html5Events\", \"Private - Return the html5 events managed by instances of the receiver.\", smalltalk:object_comma_((smalltalk:object_comma_((\"^#( \"),(selectors):inject_into_(\"\", (function(total,each)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((total),\"#\")),each)),\" \"))\r\nend)))),\") , super html5Events\"), \"events\");return ((self):buildGetters_((selectors):collect_((function(each)\r\nreturn ((self):selectorForHTML5Event_(each))\r\nend))))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"selectorForEvent:",0
,(
function(self,anEvent)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((self):localEvents()):includes_(anEvent)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function() if (((((self):html5Events()):includes_(anEvent)))==true) then return ((function()
return ((self):selectorForHTML5Event_(anEvent))
end))() else return nil end end)())
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"events","selectorForEvent: anEvent\r\n\t\" Private - Return the on... selector to use to register binding to anEvent. \"\r\n\r\n\t(self localEvents includes: anEvent) ifTrue: [ ^nil ].\r\n\t^(self html5Events includes: anEvent) ifTrue: [\r\n\t\tself selectorForHTML5Event: anEvent\r\n\t]","(\r\nfunction(self,anEvent)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((self):localEvents()):includes_(anEvent)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function() if (((((self):html5Events()):includes_(anEvent)))==true) then return ((function()\r\nreturn ((self):selectorForHTML5Event_(anEvent))\r\nend))() else return nil end end)())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"selectorForHTML5Event:",0
,(
function(self,anEvent)
return (smalltalk:object_comma_(("on"),((anEvent):upTo_(":")):asSmalltalkName_(true)))
;
end
)
,"events","selectorForHTML5Event: anEvent\r\n\t\" Private - Return the on... selector to use to register binding to anEvent. \"\r\n\r\n\t^#on , ((anEvent upTo: $:) asSmalltalkName: true)","(\r\nfunction(self,anEvent)\r\nreturn (smalltalk:object_comma_((\"on\"),((anEvent):upTo_(\":\")):asSmalltalkName_(true)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildLuaFunctions:",0
,(
function(self,specs)
(self):build_doing_(("LuaFunctions"):_minus_gt(specs), (function()
local tuples=nil
(function()
	local _s8_value=((specs):collect_((function(spec)
return ((function() if ((((spec):isString()))==true) then return ((function()
return (smalltalk:object_comma_(("_"),spec))
end))() else return ((function()
local selector=nil
(function()
	local _s8_value=((function() if (((((spec):first()):isString()))==true) then return ((function()
return (smalltalk:object_comma_(("_"),(spec):first()))
end))() else return ((function()
return ((
function(_s8_rec)
(_s8_rec):at_put_((1), smalltalk:object_comma_(("_"),((spec):first()):first()));
return ((_s8_rec):yourself())
end
)(((spec):first()):copy()))
end))() end end)())
	selector=_s8_value; return _s8_value
end)();return (smalltalk:object_comma_(((smalltalk.Array):with_(selector)),(spec):copyFrom_to_((2), (spec):size())))
end))() end end)())
end)))
	tuples=_s8_value; return _s8_value
end)();return ((tuples):do_((function(spec)
return ((self):buildFunction_(spec))
end)))
end))
;
return (self)
;
end
)
,"Lua-builders","buildLuaFunctions: specs\r\n\t\" Generate code from specs as raw functions implemented by instance handle. \"\r\n\r\n\tself build: #LuaFunctions -> specs doing: [ | tuples |\r\n\t\ttuples := specs collect: [:spec|\r\n\t\t\tspec isString ifTrue: [ $_ ,spec ] ifFalse: [ | selector |\r\n\t\t\t\tselector := spec first isString ifTrue: [$_ ,spec first] ifFalse: [\r\n\t\t\t\t\tspec first copy at: 1 put: ($_ ,spec first first); yourself\r\n\t\t\t\t].\r\n\t\t\t\t(Array with: selector), (spec copyFrom: 2 to: spec size)\r\n\t\t\t]\r\n\t\t].\r\n\t\ttuples do: [:spec| self buildFunction: spec ]\r\n\t]","(\r\nfunction(self,specs)\r\n(self):build_doing_((\"LuaFunctions\"):_minus_gt(specs), (function()\r\nlocal tuples=nil\r\n(function()\r\n\tlocal _s8_value=((specs):collect_((function(spec)\r\nreturn ((function() if ((((spec):isString()))==true) then return ((function()\r\nreturn (smalltalk:object_comma_((\"_\"),spec))\r\nend))() else return ((function()\r\nlocal selector=nil\r\n(function()\r\n\tlocal _s8_value=((function() if (((((spec):first()):isString()))==true) then return ((function()\r\nreturn (smalltalk:object_comma_((\"_\"),(spec):first()))\r\nend))() else return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):at_put_((1), smalltalk:object_comma_((\"_\"),((spec):first()):first()));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(((spec):first()):copy()))\r\nend))() end end)())\r\n\tselector=_s8_value; return _s8_value\r\nend)();return (smalltalk:object_comma_(((smalltalk.Array):with_(selector)),(spec):copyFrom_to_((2), (spec):size())))\r\nend))() end end)())\r\nend)))\r\n\ttuples=_s8_value; return _s8_value\r\nend)();return ((tuples):do_((function(spec)\r\nreturn ((self):buildFunction_(spec))\r\nend)))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildLuaStaticFunctions:",0
,(
function(self,specs)
(self):build_doing_(("LuaStaticFunctions"):_minus_gt(specs), (function()
local tuples=nil
(function()
	local _s8_value=((specs):collect_((function(spec)
return ((function() if ((((spec):isString()))==true) then return ((function()
return (smalltalk:object_comma_(("_"),spec))
end))() else return ((function()
local selector=nil
(function()
	local _s8_value=((function() if (((((spec):first()):isString()))==true) then return ((function()
return (smalltalk:object_comma_(("_"),(spec):first()))
end))() else return ((function()
return ((
function(_s8_rec)
(_s8_rec):at_put_((1), smalltalk:object_comma_(("_"),((spec):first()):first()));
return ((_s8_rec):yourself())
end
)(((spec):first()):copy()))
end))() end end)())
	selector=_s8_value; return _s8_value
end)();return (smalltalk:object_comma_(((smalltalk.Array):with_(selector)),(spec):copyFrom_to_((2), (spec):size())))
end))() end end)())
end)))
	tuples=_s8_value; return _s8_value
end)();return ((tuples):do_((function(spec)
return ((self):buildStaticFunction_(spec))
end)))
end))
;
return (self)
;
end
)
,"Lua-builders","buildLuaStaticFunctions: specs\r\n\t\" Generate code from specs as raw functions implemented by class handle. \"\r\n\r\n\tself build: #LuaStaticFunctions -> specs doing: [ | tuples |\r\n\t\ttuples := specs collect: [:spec|\r\n\t\t\tspec isString ifTrue: [ $_ ,spec ] ifFalse: [ | selector |\r\n\t\t\t\tselector := spec first isString ifTrue: [$_ ,spec first] ifFalse: [\r\n\t\t\t\t\tspec first copy at: 1 put: ($_ ,spec first first); yourself\r\n\t\t\t\t].\r\n\t\t\t\t(Array with: selector), (spec copyFrom: 2 to: spec size)\r\n\t\t\t]\r\n\t\t].\r\n\t\ttuples do: [:spec| self buildStaticFunction: spec ]\r\n\t]","(\r\nfunction(self,specs)\r\n(self):build_doing_((\"LuaStaticFunctions\"):_minus_gt(specs), (function()\r\nlocal tuples=nil\r\n(function()\r\n\tlocal _s8_value=((specs):collect_((function(spec)\r\nreturn ((function() if ((((spec):isString()))==true) then return ((function()\r\nreturn (smalltalk:object_comma_((\"_\"),spec))\r\nend))() else return ((function()\r\nlocal selector=nil\r\n(function()\r\n\tlocal _s8_value=((function() if (((((spec):first()):isString()))==true) then return ((function()\r\nreturn (smalltalk:object_comma_((\"_\"),(spec):first()))\r\nend))() else return ((function()\r\nreturn ((\r\nfunction(_s8_rec)\r\n(_s8_rec):at_put_((1), smalltalk:object_comma_((\"_\"),((spec):first()):first()));\r\nreturn ((_s8_rec):yourself())\r\nend\r\n)(((spec):first()):copy()))\r\nend))() end end)())\r\n\tselector=_s8_value; return _s8_value\r\nend)();return (smalltalk:object_comma_(((smalltalk.Array):with_(selector)),(spec):copyFrom_to_((2), (spec):size())))\r\nend))() end end)())\r\nend)))\r\n\ttuples=_s8_value; return _s8_value\r\nend)();return ((tuples):do_((function(spec)\r\nreturn ((self):buildStaticFunction_(spec))\r\nend)))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject["$klass"],"buildMethods:",0
,(
function(self,specs)
(self):build_doing_(("Methods"):_minus_gt(specs), (function()
return ((self):buildFunctions_(specs))
end))
;
return (self)
;
end
)
,"Lua-builders","buildMethods: specs\r\n\t\" Generate code from specs as OO methods implemented by instance handle. \"\r\n\r\n\tself build: #Methods -> specs doing: [\r\n\t\tself buildFunctions: specs\r\n\t]","(\r\nfunction(self,specs)\r\n(self):build_doing_((\"Methods\"):_minus_gt(specs), (function()\r\nreturn ((self):buildFunctions_(specs))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"ignoreReturn:",0
,(
function(self,value)
return (self)
;
end
)
,"api","ignoreReturn: value\r\n\t\" Private - Ignore the argument value.\r\n\tReturn the receiver. \"","(\r\nfunction(self,value)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"@@","_at_at"
,(
function(self,path)
return (((path):asArrayOfSubstringsSeparatedBy_(".")):inject_into_((self):handle(), (function(h,each)
return ((function() if ((((((h))~=nil)))==true) then return ((function()
return (((h)[each]))
end))() else return nil end end)())
end)))
;
end
)
,"accessing","@@ path\r\n\t\" Return the property at path of the receiver (or nil). \"\r\n\r\n\t^(path asArrayOfSubstringsSeparatedBy: $.)\r\n\t\tinject: self handle into: [:h :each|\r\n\t\t\th notNil ifTrue: [ h basicAt: each ]\r\n\t\t]","(\r\nfunction(self,path)\r\nreturn (((path):asArrayOfSubstringsSeparatedBy_(\".\")):inject_into_((self):handle(), (function(h,each)\r\nreturn ((function() if ((((((h))~=nil)))==true) then return ((function()\r\nreturn (((h)[each]))\r\nend))() else return nil end end)())\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"as:",0
,(
function(self,aNativeObjectClass)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function() if (((smalltalk:is_kindOf_((self),(aNativeObjectClass))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((aNativeObjectClass):_at((self):handle()))
	result=_s8_value; return _s8_value
end)()
;
(function() if (((((result)==self)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(result):copyPropertiesFrom_(self)
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"converting","as: aNativeObjectClass\r\n\t\" Return the receiver casted to aNativeObjectClass. \"\r\n\r\n\t| result |\r\n\t(self isKindOf: aNativeObjectClass) ifTrue: [ ^self ].\r\n\tresult := aNativeObjectClass @ self handle.\r\n\tresult == self ifTrue: [ ^self ].\r\n\tresult copyPropertiesFrom: self.\r\n\t^result","(\r\nfunction(self,aNativeObjectClass)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function() if (((smalltalk:is_kindOf_((self),(aNativeObjectClass))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((aNativeObjectClass):_at((self):handle()))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((((result)==self)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(result):copyPropertiesFrom_(self)\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"handle",0
,(
function(self)
(function() if ((((((self['@handle']))==nil)))==true) then return ((function()
return ((self):initializeHandle())
end))() else return nil end end)()
;
return (self['@handle'])
;
end
)
,"accessing","handle\r\n\t\" Return the native handle. \"\r\n\r\n\thandle isNil ifTrue: [ self initializeHandle ].\r\n\t^handle","(\r\nfunction(self)\r\n(function() if ((((((self['@handle']))==nil)))==true) then return ((function()\r\nreturn ((self):initializeHandle())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@handle'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"handleOrNil",0
,(
function(self)
return (self['@handle'])
;
end
)
,"private","handleOrNil\r\n\t\" Return the native handle (or nil). \"\r\n\r\n\t^handle","(\r\nfunction(self)\r\nreturn (self['@handle'])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"=","_eq"
,(
function(self,anObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((self)==anObject)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((self):isObject_(anObject)))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (false)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk:boolean_and_((smalltalk:responds_to_((anObject),("handleOrNil"))),(function()
return (((self['@handle'])==(anObject):handleOrNil()))
end))))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"testing","= anObject\r\n\r\n\tself == anObject ifTrue: [ ^true ].\r\n\t(self isObject: anObject) ifFalse: [ ^false ].\r\n\t^(anObject respondsTo: #handleOrNil) and: [\r\n\t\thandle == anObject handleOrNil\r\n\t] ","(\r\nfunction(self,anObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((self)==anObject)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((self):isObject_(anObject)))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (false)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk:boolean_and_((smalltalk:responds_to_((anObject),(\"handleOrNil\"))),(function()\r\nreturn (((self['@handle'])==(anObject):handleOrNil()))\r\nend))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"handle:",0
,(
function(self,anAlien)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function()
	local _s8_value=(anAlien)
	self['@handle']=_s8_value; return _s8_value
end)()
;
(function() if ((((((self['@handle']))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","handle: anAlien\r\n\t\" Set the native handle to a potentialy native(non-S8) object.\r\n\tReturn the receiver, or nil if the receiver is void.\r\n\t\"\r\n\r\n\thandle := anAlien.\r\n\thandle isNil ifTrue: [ ^nil ].","(\r\nfunction(self,anAlien)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function()\r\n\tlocal _s8_value=(anAlien)\r\n\tself['@handle']=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((self['@handle']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"jsonHiddenNames",0
,(
function(self)
return (smalltalk:newArray({"handle"}))
;
end
)
,"converting","jsonHiddenNames\r\n\t\" Private - Return the names of the instance variables to skip when mapping the receiver to JSON structure. \"\r\n\r\n\t^#( #handle )","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({\"handle\"}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"basicJson",0
,(
function(self)
return (smalltalk:superImplementor(smalltalk.NativeObject,"json")(self))
;
end
)
,"converting","basicJson\r\n\t\" Convert the receiver to JSON structure. \"\r\n\r\n\t^super json","(\r\nfunction(self)\r\nreturn (smalltalk:superImplementor(smalltalk.NativeObject,\"json\")(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"json",0
,(
function(self)
return ((self):handle())
;
end
)
,"converting","json\r\n\t\" Convert the receiver to JSON structure. \"\r\n\r\n\t^self handle","(\r\nfunction(self)\r\nreturn ((self):handle())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"jsObject",0
,(
function(self)
return (self:invalidMessage())
;
end
)
,"private","jsObject\r\n\t\" Private - Return the (native)Object. \"\r\n\r\n\t^{'Object'\r\n\tlua'self:invalidMessage()'}","(\r\nfunction(self)\r\nreturn (self:invalidMessage())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"ownPropertyNames",0
,(
function(self)
	local result = smalltalk.Array:new()
	for k,_ in pairs(self["@handle"]) do
		result:add_(k)
	end
	if true then return result end

;
return (self)
;
end
)
,"accessing","ownPropertyNames\r\n\t\" Return the owned property names of the receiver. \"\r\n\r\n{'\treturn Object.getOwnPropertyNames(self[\"@handle\"])'\r\nlua'\tlocal result = smalltalk.Array:new()\r\n\tfor k,_ in pairs(self[\"@handle\"]) do\r\n\t\tresult:add_(k)\r\n\tend\r\n\tif true then return result end\r\n'}","(\r\nfunction(self)\r\n\tlocal result = smalltalk.Array:new()\r\n\tfor k,_ in pairs(self[\"@handle\"]) do\r\n\t\tresult:add_(k)\r\n\tend\r\n\tif true then return result end\r\n\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"@","_at"
,(
function(self,propertyName)
return (self['@handle'][propertyName])
;
end
)
,"accessing","@ propertyName\r\n\t\" Return the property at propertyName of the receiver. \"\r\n\r\n\t^handle[propertyName]","(\r\nfunction(self,propertyName)\r\nreturn (self['@handle'][propertyName])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"handleAt:",0
,(
function(self,propertyName)
return (self['@handle'][propertyName])
;
end
)
,"accessing","handleAt: propertyName\r\n\t\" Return the property at propertyName of the receiver. \"\r\n\r\n\t^handle[propertyName]","(\r\nfunction(self,propertyName)\r\nreturn (self['@handle'][propertyName])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"handleAt:put:",0
,(
function(self,propertyName,anObject)
return ((function()
	local _s8_value=(anObject)
	self['@handle'][propertyName]=_s8_value; return _s8_value
end)())
;
end
)
,"accessing","handleAt: propertyName put: anObject\r\n\t\" Set the property at propertyName of the receiver. \"\r\n\r\n\t^handle[propertyName] := anObject","(\r\nfunction(self,propertyName,anObject)\r\nreturn ((function()\r\n\tlocal _s8_value=(anObject)\r\n\tself['@handle'][propertyName]=_s8_value; return _s8_value\r\nend)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"initializeHandle",0
,(
function(self)
return ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_(("Missing "),((self)["$klass"]):name())),">>#initializeHandle")))
;
end
)
,"initialize","initializeHandle\r\n\t\" Private - Initialize the native handle.\r\n\tThis method must be implemented by subclasses.\r\n\t\"\r\n\r\n\t^self error: 'Missing '\r\n\t\t,self class name\r\n\t\t,'>>#initializeHandle'","(\r\nfunction(self)\r\nreturn ((self):error_(smalltalk:object_comma_((smalltalk:object_comma_((\"Missing \"),((self)[\"$klass\"]):name())),\">>#initializeHandle\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"releaseHandle",0
,(
function(self)
(function()
	local _s8_value=(nil)
	self['@handle']=_s8_value; return _s8_value
end)()
;
return (self)
;
end
)
,"release","releaseHandle\r\n\t\" Private - Release the native handle.\r\n\tThe default implementation clear handle.\r\n\tThis method can be refined by subclasses to free external resources.\r\n\t\"\r\n\r\n\thandle := nil","(\r\nfunction(self)\r\n(function()\r\n\tlocal _s8_value=(nil)\r\n\tself['@handle']=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"release",0
,(
function(self)
(self):releaseHandle()
;
return (smalltalk:superImplementor(smalltalk.NativeObject,"release")(self))
;
end
)
,"release","release\r\n\t\" Private - Release the receiver. \"\r\n\r\n\tself releaseHandle.\r\n\t^super release","(\r\nfunction(self)\r\n(self):releaseHandle()\r\n;\r\nreturn (smalltalk:superImplementor(smalltalk.NativeObject,\"release\")(self))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"copyPropertiesFrom:",0
,(
function(self,aNativeObject)
((aNativeObject):ownPropertyNames()):do_((function(key)
return ((function() local _1_=(((aNativeObject)[key])); (self)[key]=_1_; return _1_ end)())
end))
;
return (self)
;
end
)
,"copying","copyPropertiesFrom: aNativeObject\r\n\t\" Private - Copy receiver's properties from aNativeObject. \"\r\n\r\n\taNativeObject ownPropertyNames do: [:key|\r\n\t\tself basicAt: key put: (aNativeObject basicAt: key)\r\n\t]","(\r\nfunction(self,aNativeObject)\r\n((aNativeObject):ownPropertyNames()):do_((function(key)\r\nreturn ((function() local _1_=(((aNativeObject)[key])); (self)[key]=_1_; return _1_ end)())\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"shallowCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","shallowCopy\r\n\t\" Return a shallow copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"deepCopy",0
,(
function(self)
return (self)
;
end
)
,"copying","deepCopy\r\n\t\" Return a deep copy of the receiver. \"\r\n\r\n\t^self","(\r\nfunction(self)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"closureForEvent:",0
,(
function(self,selector)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local argumentCount=nil
(function()
	local _s8_value=((selector):argumentCount())
	argumentCount=_s8_value; return _s8_value
end)()
;
(function() if ((((smalltalk:isObject_equalTo_((argumentCount),(0)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function()
return ((self):triggerEvent_(selector))
end))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((argumentCount),(1)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function(arg1)
return ((self):triggerEvent_with_(selector, arg1))
end))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((argumentCount),(2)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function(arg1,arg2)
return ((self):triggerEvent_with_with_(selector, arg1, arg2))
end))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((argumentCount),(3)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function(arg1,arg2,arg3)
return ((self):triggerEvent_withArguments_(selector, (smalltalk.Array):with_with_with_(arg1, arg2, arg3)))
end))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((smalltalk:isObject_equalTo_((argumentCount),(4)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((function(arg1,arg2,arg3,arg4)
return ((self):triggerEvent_withArguments_(selector, (smalltalk.Array):with_with_with_with_(arg1, arg2, arg3, arg4)))
end))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Invalid argument count"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"events","closureForEvent: selector\r\n\t\" Private - Return the closure to handle event selector. \"\r\n\r\n\t| argumentCount |\r\n\targumentCount := selector argumentCount.\r\n\targumentCount = 0 ifTrue: [ ^[ self triggerEvent: selector ] ].\r\n\targumentCount = 1 ifTrue: [ ^[:arg1| self triggerEvent: selector with: arg1 ] ].\r\n\targumentCount = 2 ifTrue: [ ^[:arg1 :arg2| self triggerEvent: selector with: arg1 with: arg2 ] ].\r\n\targumentCount = 3 ifTrue: [ ^[:arg1 :arg2 :arg3| self triggerEvent: selector withArguments: (Array with: arg1 with: arg2 with: arg3) ] ].\r\n\targumentCount = 4 ifTrue: [ ^[:arg1 :arg2 :arg3 :arg4| self triggerEvent: selector withArguments: (Array with: arg1 with: arg2 with: arg3 with: arg4) ] ].\r\n\t^self error: 'Invalid argument count'","(\r\nfunction(self,selector)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal argumentCount=nil\r\n(function()\r\n\tlocal _s8_value=((selector):argumentCount())\r\n\targumentCount=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((argumentCount),(0)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function()\r\nreturn ((self):triggerEvent_(selector))\r\nend))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((argumentCount),(1)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function(arg1)\r\nreturn ((self):triggerEvent_with_(selector, arg1))\r\nend))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((argumentCount),(2)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function(arg1,arg2)\r\nreturn ((self):triggerEvent_with_with_(selector, arg1, arg2))\r\nend))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((argumentCount),(3)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function(arg1,arg2,arg3)\r\nreturn ((self):triggerEvent_withArguments_(selector, (smalltalk.Array):with_with_with_(arg1, arg2, arg3)))\r\nend))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((smalltalk:isObject_equalTo_((argumentCount),(4)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((function(arg1,arg2,arg3,arg4)\r\nreturn ((self):triggerEvent_withArguments_(selector, (smalltalk.Array):with_with_with_with_(arg1, arg2, arg3, arg4)))\r\nend))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Invalid argument count\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"when:evaluate:",0
,(
function(self,anEvent,anAction)
local result=nil
local needsRegistration=nil
(function()
	local _s8_value=((((((self):eventTable()):at_ifAbsent_(anEvent, (function()
return (nil)
end))))==nil))
	needsRegistration=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(smalltalk:superImplementor(smalltalk.NativeObject,"when_evaluate_")(self,anEvent, anAction))
	result=_s8_value; return _s8_value
end)()
;
(function() if (((needsRegistration))==true) then return ((function()
return ((self):registerEvent_doing_(anEvent, (self):closureForEvent_(anEvent)))
end))() else return nil end end)()
;
return (result)
;
end
)
,"events","when: anEvent evaluate: anAction\r\n\t\" Register anAction to the be evaluated when the receiver triggers the event. \"\r\n\r\n\t| result needsRegistration |\r\n\tneedsRegistration := (self eventTable at: anEvent ifAbsent: [nil]) isNil.\r\n\tresult := super when: anEvent evaluate: anAction.\r\n\tneedsRegistration ifTrue: [\r\n\t\tself\tregisterEvent: anEvent\r\n\t\t\tdoing: (self closureForEvent: anEvent).\r\n\t].\r\n\t^result","(\r\nfunction(self,anEvent,anAction)\r\nlocal result=nil\r\nlocal needsRegistration=nil\r\n(function()\r\n\tlocal _s8_value=((((((self):eventTable()):at_ifAbsent_(anEvent, (function()\r\nreturn (nil)\r\nend))))==nil))\r\n\tneedsRegistration=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:superImplementor(smalltalk.NativeObject,\"when_evaluate_\")(self,anEvent, anAction))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if (((needsRegistration))==true) then return ((function()\r\nreturn ((self):registerEvent_doing_(anEvent, (self):closureForEvent_(anEvent)))\r\nend))() else return nil end end)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"registerEvent:doing:",0
,(
function(self,anEvent,closure)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local onSelector=nil
(function()
	local _s8_value=(((self)["$klass"]):selectorForEvent_(anEvent))
	onSelector=_s8_value; return _s8_value
end)()
;
(function() if ((((((onSelector))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):bindEvent_doing_(onSelector, closure))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"events","registerEvent: anEvent doing: closure\r\n\t\" Private - Register basic event handler. \"\r\n\r\n\t| onSelector |\r\n\tonSelector := self class selectorForEvent: anEvent.\r\n\tonSelector isNil ifTrue: [ ^self ]. \"local/smalltalk event\"\r\n\t^self bindEvent: onSelector doing: closure","(\r\nfunction(self,anEvent,closure)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal onSelector=nil\r\n(function()\r\n\tlocal _s8_value=(((self)[\"$klass\"]):selectorForEvent_(anEvent))\r\n\tonSelector=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((onSelector))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):bindEvent_doing_(onSelector, closure))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"bindEvent:doing:",0
,(
function(self,onSelector,closure)
local native=nil
(function()
	local _s8_value=((self):handle())
	native=_s8_value; return _s8_value
end)()
;
(function() if ((((((native[onSelector]))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=(closure)
	native[onSelector]=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((native[onSelector]):addListener((closure):json()))
end))() end end)()
;
return (self)
;
end
)
,"events","bindEvent: onSelector doing: closure\r\n\t\" Private - Bind the event handler. \"\r\n\r\n\t| native |\r\n\tnative := self handle.\r\n\tnative[onSelector] isNil ifTrue: [\r\n\t\tnative[onSelector] := closure\r\n\t] ifFalse: [\r\n\t\tnative[onSelector] #addListener: closure json\r\n\t]","(\r\nfunction(self,onSelector,closure)\r\nlocal native=nil\r\n(function()\r\n\tlocal _s8_value=((self):handle())\r\n\tnative=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((native[onSelector]))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(closure)\r\n\tnative[onSelector]=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((native[onSelector]):addListener((closure):json()))\r\nend))() end end)()\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"description",0
,(
function(self)
return ((((smalltalk.NativeHandle):support()):_at((self):handle())):description())
;
end
)
,"debugging","description\r\n\t\" Return a string description from receiver's properties. \"\r\n\r\n\t^(NativeHandle support @ self handle) description","(\r\nfunction(self)\r\nreturn ((((smalltalk.NativeHandle):support()):_at((self):handle())):description())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"asDictionary",0
,(
function(self)
local result=nil
(function()
	local _s8_value=((smalltalk.PoolDictionary):new())
	result=_s8_value; return _s8_value
end)()
;
((self):ownPropertyNames()):do_((function(key)
return ((result):at_put_(key, self['@handle'][key]))
end))
;
return (result)
;
end
)
,"converting","asDictionary\r\n\t\" Convert the receiver to a(Pool)Dictionary. \"\r\n\r\n\t| result |\r\n\tresult := PoolDictionary new.\r\n\tself ownPropertyNames do: [:key| result at: key put: handle[key] ].\r\n\t^result","(\r\nfunction(self)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.PoolDictionary):new())\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((self):ownPropertyNames()):do_((function(key)\r\nreturn ((result):at_put_(key, self['@handle'][key]))\r\nend))\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"asNativeHandle",0
,(
function(self)
return (((smalltalk.NativeHandle):support()):_at((self):handle()))
;
end
)
,"converting","asNativeHandle\r\n\t\" Convert the receiver to NativeHandle instance (for reflection). \"\r\n\r\n\t^NativeHandle support @ self handle","(\r\nfunction(self)\r\nreturn (((smalltalk.NativeHandle):support()):_at((self):handle()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeObject,"asJS",0
,(
function(self)
return ((smalltalk.JS):_at((self):handle()))
;
end
)
,"converting","asJS\r\n\t\" Convert the receiver to JS instance (for reflection). \"\r\n\r\n\t^JS @ self handle","(\r\nfunction(self)\r\nreturn ((smalltalk.JS):_at((self):handle()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle["$klass"],"support",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local lang=nil
(function()
	local _s8_value=((self):nativeLanguage())
	lang=_s8_value; return _s8_value
end)()
;
(smalltalk:newArray({smalltalk:newArray({"js", "JS"}), smalltalk:newArray({"lua", "Lua"})})):do_((function(tuple)
return ((function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((lang),(tuple):first()))),(function()
return ((smalltalk.Smalltalk):includesKey_((tuple):last()))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Smalltalk):at_((tuple):last()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):error_("Native handle support missing"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"instantiation","support\r\n\t\" Private - Return the support for hosting execution environment. \"\r\n\r\n\t| lang |\r\n\tlang := self nativeLanguage.\r\n\t#(#(js JS) #(lua Lua)) do: [:tuple|\r\n\t\t(lang = tuple first and: [Smalltalk includesKey: tuple last]) ifTrue: [\r\n\t\t\t^Smalltalk at: tuple last\r\n\t\t]\r\n\t].\r\n\t^self error: 'Native handle support missing'","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal lang=nil\r\n(function()\r\n\tlocal _s8_value=((self):nativeLanguage())\r\n\tlang=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(smalltalk:newArray({smalltalk:newArray({\"js\", \"JS\"}), smalltalk:newArray({\"lua\", \"Lua\"})})):do_((function(tuple)\r\nreturn ((function() if ((((smalltalk:boolean_and_(((smalltalk:isObject_equalTo_((lang),(tuple):first()))),(function()\r\nreturn ((smalltalk.Smalltalk):includesKey_((tuple):last()))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Smalltalk):at_((tuple):last()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):error_(\"Native handle support missing\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle["$klass"],"@>","_at_gt"
,(
function(self,nativeObject)
local result=nil
(function()
	local _s8_value=((self):_at(nativeObject))
	result=_s8_value; return _s8_value
end)()
;
return ((function() if ((((((result))~=nil)))==true) then return ((function()
return ((result):description())
end))() else return ((function()
return ("nil")
end))() end end)())
;
end
)
,"converting","@> nativeObject\r\n\t\" Return the description of nativeObject. \"\r\n\r\n\t| result |\r\n\tresult := self @ nativeObject.\r\n\t^result notNil\r\n\t\tifTrue: [ result description ]\r\n\t\tifFalse: [ #nil ]","(\r\nfunction(self,nativeObject)\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((self):_at(nativeObject))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn ((function() if ((((((result))~=nil)))==true) then return ((function()\r\nreturn ((result):description())\r\nend))() else return ((function()\r\nreturn (\"nil\")\r\nend))() end end)())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle["$klass"],"@","_at"
,(
function(self,nativeObject)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((smalltalk:boolean_and_(((self):isObject_(nativeObject)),(function()
return ((nativeObject):isString())
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:superImplementor(smalltalk.NativeHandle["$klass"],"_at")(self,(smalltalk.Smalltalk):jsObjectAt_ifAbsent_(nativeObject, (function()
return (nil)
end))))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (smalltalk:superImplementor(smalltalk.NativeHandle["$klass"],"_at")(self,nativeObject))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"instantiation","@ nativeObject\r\n\t\" Return an instance of the receiver. \"\r\n\r\n\t((self isObject: nativeObject) and: [ nativeObject isString ]) ifTrue: [\r\n\t\t^super @ (Smalltalk jsObjectAt: nativeObject ifAbsent: [])\r\n\t].\r\n\t^super @ nativeObject","(\r\nfunction(self,nativeObject)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((smalltalk:boolean_and_(((self):isObject_(nativeObject)),(function()\r\nreturn ((nativeObject):isString())\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:superImplementor(smalltalk.NativeHandle[\"$klass\"],\"_at\")(self,(smalltalk.Smalltalk):jsObjectAt_ifAbsent_(nativeObject, (function()\r\nreturn (nil)\r\nend))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (smalltalk:superImplementor(smalltalk.NativeHandle[\"$klass\"],\"_at\")(self,nativeObject))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle["$klass"],"chunks",0
,(
function(self)
return ((smalltalk.NativeReader):new())
;
end
)
,"utilities","chunks\r\n\t\" Return a NativeReader to fileIn properties in new native object. \"\r\n\r\n\t^NativeReader new","(\r\nfunction(self)\r\nreturn ((smalltalk.NativeReader):new())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle,"propertyNames",0
,(
function(self)
return ((self):implementedBySubclass_("propertyNames"))
;
end
)
,"accessing","propertyNames\r\n\t\" Return the property names of the receiver. \"\r\n\r\n\t^self implementedBySubclass: #propertyNames","(\r\nfunction(self)\r\nreturn ((self):implementedBySubclass_(\"propertyNames\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle,"keys",0
,(
function(self)
return (((self):propertyNames()):sorted())
;
end
)
,"accessing","keys\r\n\t\" Return the sorted property names of the receiver. \"\r\n\r\n\t^self propertyNames sorted","(\r\nfunction(self)\r\nreturn (((self):propertyNames()):sorted())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle,"properties",0
,(
function(self)
return (((self):keys()):collect_((function(each)
return ((smalltalk.Array):with_with_(each, self['@handle'][each]))
end)))
;
end
)
,"accessing","properties\r\n\t\" Return an array with property name and values. \"\r\n\r\n\t^self keys collect: [:each|\r\n\t\tArray with: each with: handle[each]\r\n\t]","(\r\nfunction(self)\r\nreturn (((self):keys()):collect_((function(each)\r\nreturn ((smalltalk.Array):with_with_(each, self['@handle'][each]))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle,"propertiesDo:",0
,(
function(self,aBlock)
((self):propertyNames()):do_((function(each)
return ((aBlock):value_value_(each, self['@handle'][each]))
end))
;
return (self)
;
end
)
,"iterating","propertiesDo: aBlock\r\n\t\" Evaluate aTwoArgumentBlock with each property (name & value) of the receiver. \"\r\n\r\n\tself propertyNames do: [:each| aBlock value: each value: handle[each] ]","(\r\nfunction(self,aBlock)\r\n((self):propertyNames()):do_((function(each)\r\nreturn ((aBlock):value_value_(each, self['@handle'][each]))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle,"description",0
,(
function(self)
local text=nil
local data=nil
(function()
	local _s8_value=((function(h)
return ((function() if ((((((h))==nil)))==true) then return nil else return ((function()
return ((function() if ((((self):isObject_(h)))==true) then return ((function()
return ((function() if ((((((self):typeOf_(h))=="function")))==true) then return ((function()
return (smalltalk:object_comma_((smalltalk:object_comma_(("("),(smalltalk:toString_((h))):upTo_("{"))),"{...})"))
end))() else return ((function()
return (smalltalk:printString_((h)))
end))() end end)())
end))() else return ((function()
return (((function()
return (smalltalk:toString_((h)))
end)):on_do_(smalltalk.Error, (function(err)
return ("???")
end)))
end))() end end)())
end))() end end)())
end))
	text=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((function(key)
local value=nil
((function()
return ((function()
	local _s8_value=(((self['@handle'])[key]))
	value=_s8_value; return _s8_value
end)())
end)):on_do_(smalltalk.Error, (function(ex)
return ((function()
	local _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_(("ERROR ACCESSING["),key)),"] ")),(ex):description()))
	value=_s8_value; return _s8_value
end)())
end));return (value)
end))
	data=_s8_value; return _s8_value
end)()
;
return (smalltalk:object_comma_(((((self):propertyNames()):sorted()):inject_into_("{", (function(total,each)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((total),"\r\n ")),each)),": ")),(text):value_((data):value_(each)))),";"))
end))),"\r\n}"))
;
end
)
,"debugging","description\r\n\t\" Return the description of the receiver. \"\r\n\r\n\t| text data |\r\n\ttext := [:h|\r\n\t\th isNil ifFalse: [\r\n\t\t\t(self isObject: h) ifTrue: [\r\n\t\t\t\t(self typeOf: h) == #function\r\n\t\t\t\tifTrue: [ $( ,(h toString upTo: ${) ,'{...})' ]\r\n\t\t\t\tifFalse: [ h printString ]\r\n\t\t\t] ifFalse: [\r\n\t\t\t\t[ h toString ] on: Error do: [:err| '???' ]\r\n\t\t\t]\r\n\t\t]\r\n\t].\r\n\tdata := [:key| | value |\r\n\t\t[ value := handle basicAt: key ] on: Error do: [:ex|\r\n\t\t\tvalue := 'ERROR ACCESSING[',key,'] ' ,ex description\r\n\t\t]. value\r\n\t].\r\n\t^(self propertyNames sorted inject: '{' into: [:total :each| total,'\r\n ' ,each ,': ' ,(text value: (data value:each)),';' ]) ,'\r\n}'","(\r\nfunction(self)\r\nlocal text=nil\r\nlocal data=nil\r\n(function()\r\n\tlocal _s8_value=((function(h)\r\nreturn ((function() if ((((((h))==nil)))==true) then return nil else return ((function()\r\nreturn ((function() if ((((self):isObject_(h)))==true) then return ((function()\r\nreturn ((function() if ((((((self):typeOf_(h))==\"function\")))==true) then return ((function()\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((\"(\"),(smalltalk:toString_((h))):upTo_(\"{\"))),\"{...})\"))\r\nend))() else return ((function()\r\nreturn (smalltalk:printString_((h)))\r\nend))() end end)())\r\nend))() else return ((function()\r\nreturn (((function()\r\nreturn (smalltalk:toString_((h)))\r\nend)):on_do_(smalltalk.Error, (function(err)\r\nreturn (\"???\")\r\nend)))\r\nend))() end end)())\r\nend))() end end)())\r\nend))\r\n\ttext=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function(key)\r\nlocal value=nil\r\n((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(((self['@handle'])[key]))\r\n\tvalue=_s8_value; return _s8_value\r\nend)())\r\nend)):on_do_(smalltalk.Error, (function(ex)\r\nreturn ((function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((\"ERROR ACCESSING[\"),key)),\"] \")),(ex):description()))\r\n\tvalue=_s8_value; return _s8_value\r\nend)())\r\nend));return (value)\r\nend))\r\n\tdata=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (smalltalk:object_comma_(((((self):propertyNames()):sorted()):inject_into_(\"{\", (function(total,each)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((total),\"\\r\\n \")),each)),\": \")),(text):value_((data):value_(each)))),\";\"))\r\nend))),\"\\r\\n}\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle,"functionNames",0
,(
function(self)
return (((self):propertyNames()):select_((function(each)
return ((((self):typeOf_(self['@handle'][each]))=="function"))
end)))
;
end
)
,"accessing","functionNames\r\n\t\" Return the names of property of the receiver of type function. \"\r\n\r\n\t^self propertyNames select: [:each| (self typeOf: handle[each]) == #function ]","(\r\nfunction(self)\r\nreturn (((self):propertyNames()):select_((function(each)\r\nreturn ((((self):typeOf_(self['@handle'][each]))==\"function\"))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle,"functions",0
,(
function(self)
return (((self):functionNames()):collect_((function(each)
return ((smalltalk.Array):with_with_(each, self['@handle'][each]))
end)))
;
end
)
,"accessing","functions\r\n\t\" Return the functions of the receiver as a collection of tuples #(functionName aClosure). \"\r\n\r\n\t^self functionNames collect: [:each| Array with: each with: handle[each] ]","(\r\nfunction(self)\r\nreturn (((self):functionNames()):collect_((function(each)\r\nreturn ((smalltalk.Array):with_with_(each, self['@handle'][each]))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle,"ownFunctionNames",0
,(
function(self)
return (((self):ownPropertyNames()):select_((function(each)
return ((((self):typeOf_(self['@handle'][each]))=="function"))
end)))
;
end
)
,"accessing","ownFunctionNames\r\n\t\" Return the names of functions owned/implemented by the receiver. \"\r\n\r\n\t^self ownPropertyNames select: [:each| (self typeOf: handle[each]) == #function ]","(\r\nfunction(self)\r\nreturn (((self):ownPropertyNames()):select_((function(each)\r\nreturn ((((self):typeOf_(self['@handle'][each]))==\"function\"))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle,"hasOwnProperty:",0
,(
function(self,aName)
return (rawget(self["@handle"],aName)~=nil)
;
end
)
,"testing","hasOwnProperty: aName\r\n\t\" Return true if receiver own a property named aName. \"\r\n\r\n\t^{'Object.prototype.hasOwnProperty.call(self[\"@handle\"],aName)'\r\n\tlua'rawget(self[\"@handle\"],aName)~=nil'}","(\r\nfunction(self,aName)\r\nreturn (rawget(self[\"@handle\"],aName)~=nil)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeHandle,"delete:",0
,(
function(self,aName)
	self["@handle"][aName]=nil
;
return (self)
;
end
)
,"actions","delete: aName\r\n\t\" Delete the property named aName in the receiver. \"\r\n\r\n{'\tdelete self[\"@handle\"][aName]'\r\nlua'\tself[\"@handle\"][aName]=nil'}","(\r\nfunction(self,aName)\r\n\tself[\"@handle\"][aName]=nil\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"propertyNames",0
,(
function(self)
return (self:invalidMessage_("propertyNames"))
;
end
)
,"accessing","propertyNames\r\n\t\" Return the property names of the receiver. \"\r\n\r\n\t^{'Object.keys(self.handle())'\r\n\tlua'self:invalidMessage_(\"propertyNames\")'}","(\r\nfunction(self)\r\nreturn (self:invalidMessage_(\"propertyNames\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"prototype",0
,(
function(self)
return (self:invalidMessage_("prototype"))
;
end
)
,"accessing","prototype\r\n\t\" Return the prototype of the receiver. \"\r\n\r\n\t^{'smalltalk.JS._at(Object.getPrototypeOf(self[\"@handle\"]))'\r\n\tlua'self:invalidMessage_(\"prototype\")'}","(\r\nfunction(self)\r\nreturn (self:invalidMessage_(\"prototype\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"functionNamesAndArguments",0
,(
function(self)
local argList=nil
(function()
	local _s8_value=((function(source)
return ((((
function(_s8_rec)
(_s8_rec):upTo_("(");
return ((_s8_rec):upTo_(")"))
end
)(source)):asArrayOfSubstringsSeparatedBy_(",")):collect_((function(arg)
return ((arg):trimBlanks())
end)))
end))
	argList=_s8_value; return _s8_value
end)()
;
return (((self):functions()):collect_((function(tuple)
return ((smalltalk.Array):with_with_((tuple):first(), (argList):value_((smalltalk:toString_(((tuple):last()))):stream())))
end)))
;
end
)
,"accessing","functionNamesAndArguments\r\n\t\" Return the functions of the receiver as a collection of tuples #(functionName argumentsArray). \"\r\n\r\n\t| argList |\r\n\targList := [:source|\r\n\t\t((source upTo: $(; upTo: $)) asArrayOfSubstringsSeparatedBy: $,)\r\n\t\t\tcollect: [:arg| arg trimBlanks ]\r\n\t].\r\n\t^self functions collect: [:tuple| Array with: tuple first with: (argList value: tuple last toString stream) ]","(\r\nfunction(self)\r\nlocal argList=nil\r\n(function()\r\n\tlocal _s8_value=((function(source)\r\nreturn ((((\r\nfunction(_s8_rec)\r\n(_s8_rec):upTo_(\"(\");\r\nreturn ((_s8_rec):upTo_(\")\"))\r\nend\r\n)(source)):asArrayOfSubstringsSeparatedBy_(\",\")):collect_((function(arg)\r\nreturn ((arg):trimBlanks())\r\nend)))\r\nend))\r\n\targList=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (((self):functions()):collect_((function(tuple)\r\nreturn ((smalltalk.Array):with_with_((tuple):first(), (argList):value_((smalltalk:toString_(((tuple):last()))):stream())))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"defineProperty:value:writable:enumerable:configurable:",0
,(
function(self,aName,aValue,writable,enumerable,configurable)
return ((self):defineProperty_as_(aName, (smalltalk.Array):with_with_with_with_((smalltalk.Array):with_with_("value", aValue), (smalltalk.Array):with_with_("writable", writable), (smalltalk.Array):with_with_("enumerable", enumerable), (smalltalk.Array):with_with_("configurable", configurable))))
;
end
)
,"properties","defineProperty: aName\r\n\tvalue: aValue\r\n\twritable: writable\r\n\tenumerable: enumerable\r\n\tconfigurable: configurable\r\n\t\" Define the property named aName in the receiver. \"\r\n\r\n\t^self defineProperty: aName as: ( Array\r\n\t\twith: (Array with: #value with: aValue)\r\n\t\twith: (Array with: #writable with: writable)\r\n\t\twith: (Array with: #enumerable with: enumerable)\r\n\t\twith: (Array with: #configurable with: configurable)\r\n\t)","(\r\nfunction(self,aName,aValue,writable,enumerable,configurable)\r\nreturn ((self):defineProperty_as_(aName, (smalltalk.Array):with_with_with_with_((smalltalk.Array):with_with_(\"value\", aValue), (smalltalk.Array):with_with_(\"writable\", writable), (smalltalk.Array):with_with_(\"enumerable\", enumerable), (smalltalk.Array):with_with_(\"configurable\", configurable))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"defineAccessor:get:set:enumerable:configurable:",0
,(
function(self,aName,aBlock,oneArgumentBlock,enumerable,configurable)
return ((self):defineProperty_as_(aName, (smalltalk.Array):with_with_with_with_((smalltalk.Array):with_with_("get", aBlock), (smalltalk.Array):with_with_("set", oneArgumentBlock), (smalltalk.Array):with_with_("enumerable", enumerable), (smalltalk.Array):with_with_("configurable", configurable))))
;
end
)
,"properties","defineAccessor: aName\r\n\tget: aBlock\r\n\tset: oneArgumentBlock\r\n\tenumerable: enumerable\r\n\tconfigurable: configurable\r\n\t\" Define the Accessor property named aName in the receiver. \"\r\n\r\n\t^self defineProperty: aName as: ( Array\r\n\t\twith: (Array with: #get with: aBlock)\r\n\t\twith: (Array with: #set with: oneArgumentBlock)\r\n\t\twith: (Array with: #enumerable with: enumerable)\r\n\t\twith: (Array with: #configurable with: configurable)\r\n\t)","(\r\nfunction(self,aName,aBlock,oneArgumentBlock,enumerable,configurable)\r\nreturn ((self):defineProperty_as_(aName, (smalltalk.Array):with_with_with_with_((smalltalk.Array):with_with_(\"get\", aBlock), (smalltalk.Array):with_with_(\"set\", oneArgumentBlock), (smalltalk.Array):with_with_(\"enumerable\", enumerable), (smalltalk.Array):with_with_(\"configurable\", configurable))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"defineProperty:value:writable:enumerable:",0
,(
function(self,aName,aValue,writable,enumerable)
return ((self):defineProperty_as_(aName, (smalltalk.Array):with_with_with_((smalltalk.Array):with_with_("value", aValue), (smalltalk.Array):with_with_("writable", writable), (smalltalk.Array):with_with_("enumerable", enumerable))))
;
end
)
,"properties","defineProperty: aName\r\n\tvalue: aValue\r\n\twritable: writable\r\n\tenumerable: enumerable\r\n\t\" Define the property named aName in the receiver. \"\r\n\r\n\t^self defineProperty: aName as: ( Array\r\n\t\twith: (Array with: #value with: aValue)\r\n\t\twith: (Array with: #writable with: writable)\r\n\t\twith: (Array with: #enumerable with: enumerable)\r\n\t)","(\r\nfunction(self,aName,aValue,writable,enumerable)\r\nreturn ((self):defineProperty_as_(aName, (smalltalk.Array):with_with_with_((smalltalk.Array):with_with_(\"value\", aValue), (smalltalk.Array):with_with_(\"writable\", writable), (smalltalk.Array):with_with_(\"enumerable\", enumerable))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"defineProperty:value:writable:",0
,(
function(self,aName,aValue,writable)
return ((self):defineProperty_as_(aName, (smalltalk.Array):with_with_((smalltalk.Array):with_with_("value", aValue), (smalltalk.Array):with_with_("writable", writable))))
;
end
)
,"properties","defineProperty: aName\r\n\tvalue: aValue\r\n\twritable: writable\r\n\t\" Define the property named aName in the receiver. \"\r\n\r\n\t^self defineProperty: aName as: ( Array\r\n\t\twith: (Array with: #value with: aValue)\r\n\t\twith: (Array with: #writable with: writable)\r\n\t)","(\r\nfunction(self,aName,aValue,writable)\r\nreturn ((self):defineProperty_as_(aName, (smalltalk.Array):with_with_((smalltalk.Array):with_with_(\"value\", aValue), (smalltalk.Array):with_with_(\"writable\", writable))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"defineProperty:value:",0
,(
function(self,aName,aValue)
return ((self):defineProperty_as_(aName, (smalltalk.Array):with_((smalltalk.Array):with_with_("value", aValue))))
;
end
)
,"properties","defineProperty: aName value: aValue\r\n\t\" Define the property named aName in the receiver. \"\r\n\r\n\t^self defineProperty: aName as: ( Array\r\n\t\twith: (Array with: #value with: aValue)\r\n\t)","(\r\nfunction(self,aName,aValue)\r\nreturn ((self):defineProperty_as_(aName, (smalltalk.Array):with_((smalltalk.Array):with_with_(\"value\", aValue))))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"defineProperty:",0
,(
function(self,aName)
return ((self):defineProperty_as_(aName, (smalltalk.Object):new()))
;
end
)
,"properties","defineProperty: aName\r\n\t\" Define the property named aName in the receiver. \"\r\n\r\n\t^self defineProperty: aName as: Object new","(\r\nfunction(self,aName)\r\nreturn ((self):defineProperty_as_(aName, (smalltalk.Object):new()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"defineProperty:as:",0
,(
function(self,aName,descriptor)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if ((((descriptor):isArray()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):defineProperty_as_(aName, (descriptor):inject_into_((smalltalk.Object):new(), (function(result,tuple)
(function() if (((((((tuple):last()))~=nil)))==true) then return ((function()
return ((function() local _1_=((tuple):last()); (result)[(tuple):first()]=_1_; return _1_ end)())
end))() else return nil end end)();return (result)
end))))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self:invalidMessage_("defineProperty:as:"))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"properties","defineProperty: aName as: descriptor\r\n\t\" Define the property named aName in the receiver. \"\r\n\r\n\tdescriptor isArray ifTrue: [\r\n\t\t^self defineProperty: aName as: (descriptor\r\n\t\t\tinject: Object new into: [:result :tuple|\r\n\t\t\t\ttuple last notNil ifTrue: [\r\n\t\t\t\t\tresult basicAt: tuple first put: tuple last\r\n\t\t\t\t].\r\n\t\t\t\tresult\r\n\t\t\t])\r\n\t].\r\n\r\n\t^{'Object.defineProperty(self[\"@handle\"],aName,descriptor)'\r\n\tlua'self:invalidMessage_(\"defineProperty:as:\")'}","(\r\nfunction(self,aName,descriptor)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if ((((descriptor):isArray()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):defineProperty_as_(aName, (descriptor):inject_into_((smalltalk.Object):new(), (function(result,tuple)\r\n(function() if (((((((tuple):last()))~=nil)))==true) then return ((function()\r\nreturn ((function() local _1_=((tuple):last()); (result)[(tuple):first()]=_1_; return _1_ end)())\r\nend))() else return nil end end)();return (result)\r\nend))))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self:invalidMessage_(\"defineProperty:as:\"))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"defineProperties:",0
,(
function(self,descriptors)
return (self:invalidMessage_("defineProperties:"))
;
end
)
,"properties","defineProperties: descriptors\r\n\t\" Define the properties in the receiver. \"\r\n\r\n\t^{'Object.defineProperties(self[\"@handle\"],descriptors)'\r\n\tlua'self:invalidMessage_(\"defineProperties:\")'}","(\r\nfunction(self,descriptors)\r\nreturn (self:invalidMessage_(\"defineProperties:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"preventExtensions",0
,(
function(self)
return (self:invalidMessage_("preventExtensions"))
;
end
)
,"actions","preventExtensions\r\n\t\" Set the receiver as non-extensible. \"\r\n\r\n\t^{'Object.preventExtensions(self[\"@handle\"])'\r\n\tlua'self:invalidMessage_(\"preventExtensions\")'}","(\r\nfunction(self)\r\nreturn (self:invalidMessage_(\"preventExtensions\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"freeze",0
,(
function(self)
return (self:invalidMessage_("freeze"))
;
end
)
,"actions","freeze\r\n\t\" Prevents the modification of existing property attributes and values, and prevents the addition of new properties. \"\r\n\r\n\t^{'Object.freeze(self[\"@handle\"])'\r\n\tlua'self:invalidMessage_(\"freeze\")'}","(\r\nfunction(self)\r\nreturn (self:invalidMessage_(\"freeze\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"seal",0
,(
function(self)
return (self:invalidMessage_("seal"))
;
end
)
,"actions","seal\r\n\t\" Prevents the modification of attributes of existing properties, and prevents the addition of new properties. \"\r\n\r\n\t^{'Object.seal(self[\"@handle\"])'\r\n\tlua'self:invalidMessage_(\"seal\")'}","(\r\nfunction(self)\r\nreturn (self:invalidMessage_(\"seal\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"isExtensible",0
,(
function(self)
return (self:invalidMessage_("isExtensible"))
;
end
)
,"testing","isExtensible\r\n\t\" Return true if the receiver is extensible. \"\r\n\r\n\t^{'Object.isExtensible(self[\"@handle\"])'\r\n\tlua'self:invalidMessage_(\"isExtensible\")'}","(\r\nfunction(self)\r\nreturn (self:invalidMessage_(\"isExtensible\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"isFrozen",0
,(
function(self)
return (self:invalidMessage_("isFrozen"))
;
end
)
,"testing","isFrozen\r\n\t\" Return true if the receiver is frozen. \"\r\n\r\n\t^{'Object.isFrozen(self[\"@handle\"])'\r\n\tlua'self:invalidMessage_(\"isFrozen\")'}","(\r\nfunction(self)\r\nreturn (self:invalidMessage_(\"isFrozen\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"isSealed",0
,(
function(self)
return (self:invalidMessage_("isSealed"))
;
end
)
,"testing","isSealed\r\n\t\" Return true if the receiver is sealed. \"\r\n\r\n\t^{'Object.isSealed(self[\"@handle\"])'\r\n\tlua'self:invalidMessage_(\"isSealed\")'}","(\r\nfunction(self)\r\nreturn (self:invalidMessage_(\"isSealed\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"create:with:",0
,(
function(self,prototype,descriptors)
return (self:invalidMessage_("create:with:"))
;
end
)
,"creation","create: prototype with: descriptors\r\n\t\" Return a JS object that has been created with the specified prototype, and that optionally contains specified properties. \"\r\n\r\n\t^{'smalltalk.JS._at(Object.create(prototype,descriptors))'\r\n\tlua'self:invalidMessage_(\"create:with:\")'}","(\r\nfunction(self,prototype,descriptors)\r\nreturn (self:invalidMessage_(\"create:with:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.JS,"createWith:",0
,(
function(self,descriptors)
return ((self):create_with_(self['@handle'], descriptors))
;
end
)
,"creation","createWith: descriptors\r\n\t\" Return a JS object that has been created with the handle prototype, and that optionally contains specified properties. \"\r\n\r\n\t^self create: handle with: descriptors","(\r\nfunction(self,descriptors)\r\nreturn ((self):create_with_(self['@handle'], descriptors))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Lua["$klass"],"global",0
,(
function(self)
return ((self):_at((smalltalk.Smalltalk):global()))
;
end
)
,"accessing","global\r\n\t\" Return an instance wrapping global. \"\r\n\r\n\t^self @ Smalltalk global","(\r\nfunction(self)\r\nreturn ((self):_at((smalltalk.Smalltalk):global()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Lua["$klass"],"version",0
,(
function(self)
return (((self):global()):_at("_VERSION"))
;
end
)
,"accessing","version\r\n\t\" Returns the host version. \"\r\n\r\n\t^self global @ #_VERSION","(\r\nfunction(self)\r\nreturn (((self):global()):_at(\"_VERSION\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Lua["$klass"],"arg",0
,(
function(self)
return ((smalltalk.Array):_at(((self):global()):_at("arg")))
;
end
)
,"accessing","arg\r\n\t\" Returns the main arguments (anArray of strings). \"\r\n\r\n\t^Array @ (self global @ #arg)","(\r\nfunction(self)\r\nreturn ((smalltalk.Array):_at(((self):global()):_at(\"arg\")))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.Lua,"propertyNames",0
,(
function(self)
return ((self):ownPropertyNames())
;
end
)
,"accessing","propertyNames\r\n\t\" Return the property names of the receiver. \"\r\n\r\n\t^self ownPropertyNames","(\r\nfunction(self)\r\nreturn ((self):ownPropertyNames())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"globalName",0
,(
function(self)
return ((self):subclassResponsibility_("globalName"))
;
end
)
,"private","globalName\r\n\t\" Private - Return the name of the global to detect/access the library of the receiver. \"\r\n\r\n\t^self subclassResponsibility: #globalName","(\r\nfunction(self)\r\nreturn ((self):subclassResponsibility_(\"globalName\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"topGlobal",0
,(
function(self)
return ((smalltalk.Smalltalk):jsObjectAt_ifAbsent_("window", (function()
return ((self):error_("Missing window support"))
end)))
;
end
)
,"private","topGlobal\r\n\t\" Private - Return the object to access the global during detection stage.\r\n\tThe default implementation return window global (present when running in a web browser).\r\n\tThis method must be refined by the subclasses requiring other root dependences.\r\n\tCan return nil to do not check dependents.\r\n\t\"\r\n\r\n\t^Smalltalk jsObjectAt: #window ifAbsent: [ self error: 'Missing window support' ].","(\r\nfunction(self)\r\nreturn ((smalltalk.Smalltalk):jsObjectAt_ifAbsent_(\"window\", (function()\r\nreturn ((self):error_(\"Missing window support\"))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"requiredFeatures",0
,(
function(self)
return (smalltalk:newArray({}))
;
end
)
,"private","requiredFeatures\r\n\t\" Private - Return the names of the required properties/features to execute the receiver.\r\n\tThe default implementation do not check dependents.\r\n\tThis method can be refined by subclasses to ensure features to run the receiver are present in this system.\r\n\t\"\r\n\r\n\t^#()","(\r\nfunction(self)\r\nreturn (smalltalk:newArray({}))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"classNamed:",0
,(
function(self,className)
local library=nil
(function()
	local _s8_value=((self):library())
	library=_s8_value; return _s8_value
end)()
;
return (library[className])
;
end
)
,"private","classNamed: className\r\n\t\" Private - Return the class named className in the receiver's library.\r\n\tThe default implementation get the class from library.\r\n\tSubclasses can refine to obtain classes from anywhere\r\n\t\te.g. global scope with: ^self global: className.\r\n\t\"\r\n\r\n\t| library |\r\n\tlibrary := self library.\r\n\t^library[className]","(\r\nfunction(self,className)\r\nlocal library=nil\r\n(function()\r\n\tlocal _s8_value=((self):library())\r\n\tlibrary=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (library[className])\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"sourceCode",0
,(
function(self)
return ((self):subclassResponsibility_("sourceCode"))
;
end
)
,"private","sourceCode\r\n\t\" Private - Return the (native) source code of the library as a String.\r\n\tThis method can be implemented by subclasses loading sources from external media or simply returning aString embedded in this method.\r\n\tA reference/link to source origin is appreciated by users.\r\n\tCAUTION: S8 is MIT, so you MUST embedd here MIT compatible sources.\r\n\t\"\r\n\r\n\t^self subclassResponsibility: #sourceCode","(\r\nfunction(self)\r\nreturn ((self):subclassResponsibility_(\"sourceCode\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"libraryBound:",0
,(
function(self,aClassName)
((self):bindingListeners()):do_((function(each)
return ((each):libraryBound_(aClassName))
end))
;
return (self)
;
end
)
,"binding","libraryBound: aClassName\r\n\t\" Private - The library of aClassName has just been bound to this system. \"\r\n\r\n\tself bindingListeners do: [:each| each libraryBound: aClassName ]","(\r\nfunction(self,aClassName)\r\n((self):bindingListeners()):do_((function(each)\r\nreturn ((each):libraryBound_(aClassName))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"bindingListeners",0
,(
function(self)
(function() if ((((((smalltalk.NativeLibraryObject:classVariableAt_("BindingListeners")))==nil)))==true) then return ((function()
return (smalltalk.NativeLibraryObject:classVariableAt_put_("BindingListeners",(smalltalk.Array):new()))
end))() else return nil end end)()
;
return (smalltalk.NativeLibraryObject:classVariableAt_("BindingListeners"))
;
end
)
,"binding","bindingListeners\r\n\t\" Private - Return the collection of listeners to library binding notifications of this system. \"\r\n\r\n\tBindingListeners isNil ifTrue: [ BindingListeners := Array new ].\r\n\t^BindingListeners","(\r\nfunction(self)\r\n(function() if ((((((smalltalk.NativeLibraryObject:classVariableAt_(\"BindingListeners\")))==nil)))==true) then return ((function()\r\nreturn (smalltalk.NativeLibraryObject:classVariableAt_put_(\"BindingListeners\",(smalltalk.Array):new()))\r\nend))() else return nil end end)()\r\n;\r\nreturn (smalltalk.NativeLibraryObject:classVariableAt_(\"BindingListeners\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"globalName:",0
,(
function(self,aName)
return (((self)["$klass"]):implement_comment_as_category_("globalName", "Private - Return the name of the global to detect/access the library of the receiver.", smalltalk:object_comma_(("^"),smalltalk:printString_((aName))), "private"))
;
end
)
,"builder","globalName: aName\r\n\t\" Generate code to return the name of the global to detect/access the library of the receiver. \"\r\n\r\n\t^self class\r\n\t\timplement: #globalName\r\n\t\tcomment: 'Private - Return the name of the global to detect/access the library of the receiver.'\r\n\t\tas: '^' ,aName printString\r\n\t\tcategory: #private","(\r\nfunction(self,aName)\r\nreturn (((self)[\"$klass\"]):implement_comment_as_category_(\"globalName\", \"Private - Return the name of the global to detect/access the library of the receiver.\", smalltalk:object_comma_((\"^\"),smalltalk:printString_((aName))), \"private\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"libraryURL:",0
,(
function(self,anURL)
return (((self)["$klass"]):implement_comment_as_category_("libraryURL", "Private - Return the URL to use when loading library of the receiver.", smalltalk:object_comma_(("^"),smalltalk:printString_((anURL))), "private"))
;
end
)
,"builder","libraryURL: anURL\r\n\t\" Generate the method to return libraryURL. \"\r\n\r\n\t^self class\r\n\t\timplement: #libraryURL\r\n\t\tcomment: 'Private - Return the URL to use when loading library of the receiver.'\r\n\t\tas: '^' ,anURL printString\r\n\t\tcategory: #private","(\r\nfunction(self,anURL)\r\nreturn (((self)[\"$klass\"]):implement_comment_as_category_(\"libraryURL\", \"Private - Return the URL to use when loading library of the receiver.\", smalltalk:object_comma_((\"^\"),smalltalk:printString_((anURL))), \"private\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"globalClassNameFrom:",0
,(
function(self,globalClassName)
return (globalClassName)
;
end
)
,"private","globalClassNameFrom: globalClassName\r\n\t\" Private - Return the global class name for global globalClassName.\r\n\tIt must return the (global) name of the constructor used to instantiate instance handles.\r\n\tThe default action is to return the globalClassName.\r\n\tThis method can be refined by subclasses to adapt name to (case sensitive) library constructor names.\r\n\t\"\r\n\r\n\t^globalClassName","(\r\nfunction(self,globalClassName)\r\nreturn (globalClassName)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"selectorForConstantNamed:",0
,(
function(self,aName)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(aName):detect_ifNone_((function(c)
return ((c):isLowercase())
end), (function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((aName):asLowercase()):asSmalltalkName_(false))
end
)()
error(_s8Ret_,0)
end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aName):asSmalltalkName_(false))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builder","selectorForConstantNamed: aName\r\n\t\" Private - Return the selector to use for constant aName. \"\r\n\r\n\taName detect: [:c| c isLowercase ] ifNone: [\r\n\t\t^aName asLowercase asSmalltalkName: false\r\n\t].\r\n\t^aName asSmalltalkName: false","(\r\nfunction(self,aName)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(aName):detect_ifNone_((function(c)\r\nreturn ((c):isLowercase())\r\nend), (function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((aName):asLowercase()):asSmalltalkName_(false))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aName):asSmalltalkName_(false))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"constants:",0
,(
function(self,names)
(names):do_((function(each)
return (((self)["$klass"]):implement_comment_as_category_((self):selectorForConstantNamed_(each), smalltalk:object_comma_(("Return the value of "),each), smalltalk:object_comma_(("^self library basicAt: #"),each), "accessing"))
end))
;
return (self)
;
end
)
,"builder","constants: names\r\n\t\" Private - Build methods to access library globals. \"\r\n\r\n\tnames do: [:each|\r\n\t\tself class\r\n\t\t\timplement: (self selectorForConstantNamed: each)\r\n\t\t\tcomment: 'Return the value of ',each\r\n\t\t\tas: '^self library basicAt: #',each\r\n\t\t\tcategory: #accessing\r\n\t].","(\r\nfunction(self,names)\r\n(names):do_((function(each)\r\nreturn (((self)[\"$klass\"]):implement_comment_as_category_((self):selectorForConstantNamed_(each), smalltalk:object_comma_((\"Return the value of \"),each), smalltalk:object_comma_((\"^self library basicAt: #\"),each), \"accessing\"))\r\nend))\r\n;\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"buildSubclass:arguments:typed:",0
,(
function(self,globalClassName,argList,typedArguments)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local cls=nil
local clsName=nil
(function()
	local _s8_value=(smalltalk:object_comma_((((globalClassName):first()):asUppercase()),(globalClassName):copyFrom_to_((2), (globalClassName):size())))
	clsName=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=(smalltalk:object_comma_(((self):classToken()),clsName))
	clsName=_s8_value; return _s8_value
end)()
;
(self):subclass_instanceVariableNames_category_(clsName, "", (self):category())
;
(function() if ((((smalltalk:boolean_or_(((((argList))==nil)),(function()
return ((smalltalk:isObject_equalTo_((argList),"nil")))
end)))))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((smalltalk.Smalltalk):at_(clsName))
	cls=_s8_value; return _s8_value
end)()
;
((cls)["$klass"]):implement_comment_as_category_("globalClassName", "Private - Return the internal class name.", smalltalk:object_comma_(("^self globalClassNameFrom: #"),globalClassName), "private")
;
(function() if ((((argList):isEmpty()))==true) then return ((function()
return ((cls):implement_comment_as_category_("initializeHandle", "Private - Initialize the receiver's handle.", "self handle: (self newHandleOf: self class globalClassName)", "initialize"))
end))() else return ((function()
local selector=nil
local args=nil
local current=nil
(function()
	local _s8_value=((argList):inject_into_("", (function(total,each)
return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((total),each)),": ")),each))," "))
end)))
	selector=_s8_value; return _s8_value
end)();(function()
	local _s8_value=((argList):inject_into_("", (function(total,each)
(function() if ((((smalltalk:boolean_or_(((smalltalk:isObject_equalTo_((typedArguments),"all"))),(function()
return ((typedArguments):includes_(each))
end)))))==true) then return ((function()
return ((function()
	local _s8_value=(" handle ")
	current=_s8_value; return _s8_value
end)())
end))() else return ((function()
return ((function()
	local _s8_value=(" ")
	current=_s8_value; return _s8_value
end)())
end))() end end)();return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((total),"with: ")),each)),current))
end)))
	args=_s8_value; return _s8_value
end)();((cls)["$klass"]):implement_comment_as_category_(selector, "Return an instance of the receiver.", smalltalk:object_comma_(("^self basicNew "),selector), "instantiation");return ((cls):implement_comment_as_category_(selector, "Private - Initialize the receiver.", smalltalk:object_comma_((smalltalk:object_comma_(("self handle: (self newHandleOf: self class globalClassName "),args)),")"), "initialize"))
end))() end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (self)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"builder","buildSubclass: globalClassName arguments: argList typed: typedArguments\r\n\t\" Private - Build a subclass of the receiver implementing a wrapper to the library. \"\r\n\r\n\t| cls clsName |\r\n\tclsName := globalClassName first asUppercase\r\n\t\t\t,(globalClassName copyFrom: 2 to: globalClassName size).\r\n\tclsName := self classToken ,clsName.\r\n\tself\r\n\t\tsubclass: clsName\r\n\t\tinstanceVariableNames: ''\r\n\t\tcategory: self category.\r\n\t(argList isNil or: [ argList = #nil ]) ifTrue: [ ^self ].\r\n\r\n\tcls := Smalltalk at: clsName.\r\n\tcls class\r\n\t\timplement: #globalClassName\r\n\t\tcomment: 'Private - Return the internal class name.'\r\n\t\tas: '^self globalClassNameFrom: #' ,globalClassName\r\n\t\tcategory: #private.\r\n\targList isEmpty ifTrue: [\r\n\t\tcls\r\n\t\t\timplement: #initializeHandle\r\n\t\t\tcomment: 'Private - Initialize the receiver''s handle.'\r\n\t\t\tas: 'self handle: (self newHandleOf: self class globalClassName)'\r\n\t\t\tcategory: #initialize.\r\n\t] ifFalse: [ | selector args current |\r\n\t\tselector := argList inject: '' into: [:total :each| total ,each,': ',each,' ' ].\r\n\t\targs := argList inject: '' into: [:total :each|\r\n\t\t\t(typedArguments = #all\r\n\t\t\t  or: [ typedArguments includes: each ])\r\n\t\t\t  ifTrue: [ current := ' handle ' ]\r\n\t\t\t  ifFalse: [ current := ' ' ].\r\n\t\t\ttotal ,'with: ',each, current\r\n\t\t].\r\n\t\tcls class\r\n\t\t\timplement: selector\r\n\t\t\tcomment: 'Return an instance of the receiver.'\r\n\t\t\tas: '^self basicNew ',selector\r\n\t\t\tcategory: #instantiation.\r\n\t\tcls\r\n\t\t\timplement: selector\r\n\t\t\tcomment: 'Private - Initialize the receiver.'\r\n\t\t\tas: 'self handle: (self newHandleOf: self class globalClassName ' ,args ,')'\r\n\t\t\tcategory: #initialize.\r\n\t].","(\r\nfunction(self,globalClassName,argList,typedArguments)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal cls=nil\r\nlocal clsName=nil\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_((((globalClassName):first()):asUppercase()),(globalClassName):copyFrom_to_((2), (globalClassName):size())))\r\n\tclsName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=(smalltalk:object_comma_(((self):classToken()),clsName))\r\n\tclsName=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(self):subclass_instanceVariableNames_category_(clsName, \"\", (self):category())\r\n;\r\n(function() if ((((smalltalk:boolean_or_(((((argList))==nil)),(function()\r\nreturn ((smalltalk:isObject_equalTo_((argList),\"nil\")))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):at_(clsName))\r\n\tcls=_s8_value; return _s8_value\r\nend)()\r\n;\r\n((cls)[\"$klass\"]):implement_comment_as_category_(\"globalClassName\", \"Private - Return the internal class name.\", smalltalk:object_comma_((\"^self globalClassNameFrom: #\"),globalClassName), \"private\")\r\n;\r\n(function() if ((((argList):isEmpty()))==true) then return ((function()\r\nreturn ((cls):implement_comment_as_category_(\"initializeHandle\", \"Private - Initialize the receiver's handle.\", \"self handle: (self newHandleOf: self class globalClassName)\", \"initialize\"))\r\nend))() else return ((function()\r\nlocal selector=nil\r\nlocal args=nil\r\nlocal current=nil\r\n(function()\r\n\tlocal _s8_value=((argList):inject_into_(\"\", (function(total,each)\r\nreturn (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((total),each)),\": \")),each)),\" \"))\r\nend)))\r\n\tselector=_s8_value; return _s8_value\r\nend)();(function()\r\n\tlocal _s8_value=((argList):inject_into_(\"\", (function(total,each)\r\n(function() if ((((smalltalk:boolean_or_(((smalltalk:isObject_equalTo_((typedArguments),\"all\"))),(function()\r\nreturn ((typedArguments):includes_(each))\r\nend)))))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(\" handle \")\r\n\tcurrent=_s8_value; return _s8_value\r\nend)())\r\nend))() else return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=(\" \")\r\n\tcurrent=_s8_value; return _s8_value\r\nend)())\r\nend))() end end)();return (smalltalk:object_comma_((smalltalk:object_comma_((smalltalk:object_comma_((total),\"with: \")),each)),current))\r\nend)))\r\n\targs=_s8_value; return _s8_value\r\nend)();((cls)[\"$klass\"]):implement_comment_as_category_(selector, \"Return an instance of the receiver.\", smalltalk:object_comma_((\"^self basicNew \"),selector), \"instantiation\");return ((cls):implement_comment_as_category_(selector, \"Private - Initialize the receiver.\", smalltalk:object_comma_((smalltalk:object_comma_((\"self handle: (self newHandleOf: self class globalClassName \"),args)),\")\"), \"initialize\"))\r\nend))() end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (self)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"global:",0
,(
function(self,globalName)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local result=nil
(function()
	local _s8_value=((smalltalk.Smalltalk):jsObjectAt_ifAbsent_(globalName, (function()
return (nil)
end)))
	result=_s8_value; return _s8_value
end)()
;
(function() if ((((((result))~=nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (result)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function() if ((((self):isValid()))==true) then return nil else return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (nil)
end
)()
error(_s8Ret_,0)
end)())
end))() end end)()
;
(self):libraryLoaded_((self):loadLibrary())
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((smalltalk.Smalltalk):jsObjectAt_ifAbsent_(globalName, (function()
return (nil)
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"accessing","global: globalName\r\n\t\" Private - Return the native library global object (or nil). \"\r\n\r\n\t| result |\r\n\tresult := Smalltalk jsObjectAt: globalName ifAbsent: [ nil ].\r\n\tresult notNil ifTrue: [ ^result ].\r\n\tself isValid ifFalse: [ ^nil ].\r\n\tself libraryLoaded: self loadLibrary.\r\n\t^Smalltalk jsObjectAt: globalName ifAbsent: [ nil ]","(\r\nfunction(self,globalName)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal result=nil\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):jsObjectAt_ifAbsent_(globalName, (function()\r\nreturn (nil)\r\nend)))\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((result))~=nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (result)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function() if ((((self):isValid()))==true) then return nil else return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (nil)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() end end)()\r\n;\r\n(self):libraryLoaded_((self):loadLibrary())\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((smalltalk.Smalltalk):jsObjectAt_ifAbsent_(globalName, (function()\r\nreturn (nil)\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"library",0
,(
function(self)
return ((self):global_((self):globalName()))
;
end
)
,"validation","library\r\n\t\" Check support to run this framework and try to load the library implementation.\r\n\tReturn the native library object if success.\r\n\t\"\r\n\r\n\t^self global: self globalName","(\r\nfunction(self)\r\nreturn ((self):global_((self):globalName()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"libraryOrNil",0
,(
function(self)
return ((smalltalk.Smalltalk):jsObjectAt_ifAbsent_((self):globalName(), (function()
return (nil)
end)))
;
end
)
,"validation","libraryOrNil\r\n\t\" Quick Check - Return true if the library is already bound. \"\r\n\r\n\t^Smalltalk jsObjectAt: self globalName ifAbsent: [ nil ]","(\r\nfunction(self)\r\nreturn ((smalltalk.Smalltalk):jsObjectAt_ifAbsent_((self):globalName(), (function()\r\nreturn (nil)\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"nativeClass",0
,(
function(self)
return ((smalltalk.JS):_at((self):library()))
;
end
)
,"private","nativeClass\r\n\t\" Private - Returns the wrapper for native binding of the receiver.\r\n\tThe default result is a JS instance wrapping the receiver's library,\r\n\t so static(class side) builders generate bindings accessing receiver's library.\r\n\t\"\r\n\r\n\t^JS @ self library","(\r\nfunction(self)\r\nreturn ((smalltalk.JS):_at((self):library()))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"libraryURLs",0
,(
function(self)
return (((smalltalk.Array):with_((self):libraryURL())):select_((function(each)
return ((((each))~=nil))
end)))
;
end
)
,"private","libraryURLs\r\n\t\" Private - Return a collection of URL to load library of the receiver.\r\n\tReturn an empty collection, the subclasses can complement addresses to load library from external resources.\r\n\t\"\r\n\r\n\t^(Array with: self libraryURL) select: [:each| each notNil ]","(\r\nfunction(self)\r\nreturn (((smalltalk.Array):with_((self):libraryURL())):select_((function(each)\r\nreturn ((((each))~=nil))\r\nend)))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"libraryURL",0
,(
function(self)
return (nil)
;
end
)
,"private","libraryURL\r\n\t\" Private - Return the URL to use when loading library of the receiver.\r\n\tDefault implementation do not provide support to load library.\r\n\t\"\r\n\r\n\t^nil","(\r\nfunction(self)\r\nreturn (nil)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"basicLoad:",0
,(
function(self,url)
return (self:notImplementedYet_("basicLoad:"))
;
end
)
,"loading","basicLoad: url\r\n\t\" Private - Try to load the library at url. \"\r\n\r\n\t^{'load(url)'\r\n\tlua'self:notImplementedYet_(\"basicLoad:\")'}","(\r\nfunction(self,url)\r\nreturn (self:notImplementedYet_(\"basicLoad:\"))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"onLoad:",0
,(
function(self,aBlock)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local dom=nil
local success=nil
(function() if (((((((self):libraryOrNil()))~=nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((aBlock):value_((self):library()))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
	local _s8_value=((function()
(smalltalk.NativeLibraryObject):libraryBound_((self):name());return ((aBlock):value_((self):library()))
end))
	success=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=((smalltalk.Smalltalk):at_ifAbsent_("DOM", (function()
((self):libraryURLs()):do_((function(each)
return ((self):basicLoad_(each))
end));return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((success):value())
end
)()
error(_s8Ret_,0)
end)())
end)))
	dom=_s8_value; return _s8_value
end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((dom):document()):loadAll_onLoad_((self):libraryURLs(), (function()
return ((success):value())
end)))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","onLoad: aBlock\r\n\t\" Private - Load the library of the receiver and evaluates aBlock. \"\r\n\r\n\t| dom success |\r\n\tself libraryOrNil notNil ifTrue: [ ^aBlock value: self library ].\r\n\tsuccess := [\r\n\t\tNativeLibraryObject libraryBound: self name.\r\n\t\taBlock value: self library\r\n\t].\r\n\tdom := Smalltalk at: #DOM ifAbsent: [\r\n\t\tself libraryURLs do: [:each| self basicLoad: each ].\r\n\t\t^success value\r\n\t].\r\n\t^dom document\r\n\t\tloadAll: self libraryURLs\r\n\t\tonLoad: [ success value ]","(\r\nfunction(self,aBlock)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal dom=nil\r\nlocal success=nil\r\n(function() if (((((((self):libraryOrNil()))~=nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((aBlock):value_((self):library()))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((function()\r\n(smalltalk.NativeLibraryObject):libraryBound_((self):name());return ((aBlock):value_((self):library()))\r\nend))\r\n\tsuccess=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=((smalltalk.Smalltalk):at_ifAbsent_(\"DOM\", (function()\r\n((self):libraryURLs()):do_((function(each)\r\nreturn ((self):basicLoad_(each))\r\nend));return ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((success):value())\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend)))\r\n\tdom=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((dom):document()):loadAll_onLoad_((self):libraryURLs(), (function()\r\nreturn ((success):value())\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"loadLibrary",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
(function() if (((((self):libraryURLs()):notEmpty()))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return ((self):onLoad_((function(ignored)
return (nil)
end)))
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (((smalltalk.Smalltalk):current()):eval_as_((self):sourceCode(), (smalltalk.Array):with_with_("loadLibrary", (self):name())))
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"private","loadLibrary\r\n\t\" Private - Load the library of the receiver.\r\n\tDefault implementation load the library from remote URL or sourceCode.\r\n\t\"\r\n\r\n\tself libraryURLs notEmpty ifTrue: [ ^self onLoad: [:ignored| ] ].\r\n\t^Smalltalk current\r\n\t\teval: self sourceCode\r\n\t\tas: (Array with: #loadLibrary with: self name)","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\n(function() if (((((self):libraryURLs()):notEmpty()))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn ((self):onLoad_((function(ignored)\r\nreturn (nil)\r\nend)))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (((smalltalk.Smalltalk):current()):eval_as_((self):sourceCode(), (smalltalk.Array):with_with_(\"loadLibrary\", (self):name())))\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"libraryLoaded:",0
,(
function(self,loadResult)
return (self)
;
end
)
,"private","libraryLoaded: loadResult\r\n\t\" Private - The library of the receiver has been loaded.\r\n\tDefault implementation do nothing.\r\n\tThis method can be refined by subclasses to perform postLoading actions.\r\n\tThe loadResult argument is the result of the just evaluated load process.\r\n\t\"","(\r\nfunction(self,loadResult)\r\nreturn (self)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject["$klass"],"isValid",0
,(
function(self)

local _s8Ret_={["name"]="stReturn"}
local _s8_f = function()
local global=nil
(function()
	local _s8_value=((self):topGlobal())
	global=_s8_value; return _s8_value
end)()
;
(function() if ((((((global))==nil)))==true) then return ((function()
return ((function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)())
end))() else return nil end end)()
;
((self):requiredFeatures()):do_((function(each)
return ((function() if ((((((global[each]))==nil)))==true) then return ((function()
return ((self):error_(smalltalk:object_comma_(("Missing support for "),each)))
end))() else return nil end end)())
end))
;
(function()
_s8Ret_.hasResult=true
_s8Ret_.result=(function()
return (true)
end
)()
error(_s8Ret_,0)
end)()
;
end
local _s8_status, _s8_result = pcall(_s8_f)
if _s8_status then return _s8_result
else
    if (_s8Ret_.hasResult) then
      return (_s8Ret_.result)
    end
end
error(_s8_result,0)

end
)
,"validation","isValid\r\n\t\" Return true if the receiver is valid in this (browser) context.\r\n\tGenerate error if the minimum support to run the library is not available.\r\n\t\"\r\n\r\n\t| global |\r\n\tglobal := self topGlobal.\r\n\tglobal isNil ifTrue: [ ^true ]. \"always valid\"\r\n\tself requiredFeatures do: [:each|\r\n\t  global[each] isNil ifTrue: [\r\n\t    self error: 'Missing support for ',each\r\n\t  ].\r\n\t].\r\n\t^true","(\r\nfunction(self)\r\n\r\nlocal _s8Ret_={[\"name\"]=\"stReturn\"}\r\nlocal _s8_f = function()\r\nlocal global=nil\r\n(function()\r\n\tlocal _s8_value=((self):topGlobal())\r\n\tglobal=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function() if ((((((global))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\n((self):requiredFeatures()):do_((function(each)\r\nreturn ((function() if ((((((global[each]))==nil)))==true) then return ((function()\r\nreturn ((self):error_(smalltalk:object_comma_((\"Missing support for \"),each)))\r\nend))() else return nil end end)())\r\nend))\r\n;\r\n(function()\r\n_s8Ret_.hasResult=true\r\n_s8Ret_.result=(function()\r\nreturn (true)\r\nend\r\n)()\r\nerror(_s8Ret_,0)\r\nend)()\r\n;\r\nend\r\nlocal _s8_status, _s8_result = pcall(_s8_f)\r\nif _s8_status then return _s8_result\r\nelse\r\n    if (_s8Ret_.hasResult) then\r\n      return (_s8Ret_.result)\r\n    end\r\nend\r\nerror(_s8_result,0)\r\n\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject,"library",0
,(
function(self)
return (((self)["$klass"]):library())
;
end
)
,"private","library\r\n\t\" Private - Return the library of the receiver. \"\r\n\r\n\t^self class library","(\r\nfunction(self)\r\nreturn (((self)[\"$klass\"]):library())\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject,"classNamed:",0
,(
function(self,className)
return (((self)["$klass"]):classNamed_(className))
;
end
)
,"private","classNamed: className\r\n\t\" Private - Return the class named className in the receiver's library. \"\r\n\r\n\t^self class classNamed: className","(\r\nfunction(self,className)\r\nreturn (((self)[\"$klass\"]):classNamed_(className))\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject,"newHandleOf:",0
,(
function(self,className)
local result=nil
local aConstructor=nil
(function()
	local _s8_value=((self):classNamed_(className))
	aConstructor=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=( aConstructor() )
	result=_s8_value; return _s8_value
end)()
;
return (result)
;
end
)
,"private","newHandleOf: className\r\n\t\" Private - Return a new instance of the class named className in the receiver's library. \"\r\n\r\n\t| result aConstructor |\r\n\taConstructor := self classNamed: className.\r\n\tresult := {' new aConstructor() ' lua' aConstructor() '}.\r\n\t^result","(\r\nfunction(self,className)\r\nlocal result=nil\r\nlocal aConstructor=nil\r\n(function()\r\n\tlocal _s8_value=((self):classNamed_(className))\r\n\taConstructor=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=( aConstructor() )\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject,"newHandleOf:with:",0
,(
function(self,className,arg1)
local result=nil
local aConstructor=nil
(function()
	local _s8_value=((self):classNamed_(className))
	aConstructor=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=( aConstructor(arg1) )
	result=_s8_value; return _s8_value
end)()
;
return (result)
;
end
)
,"private","newHandleOf: className with: arg1\r\n\t\" Private - Return a new instance of the class named className in the receiver's library. \"\r\n\r\n\t| result aConstructor |\r\n\taConstructor := self classNamed: className.\r\n\tresult := {' new aConstructor(arg1) ' lua' aConstructor(arg1) '}.\r\n\t^result","(\r\nfunction(self,className,arg1)\r\nlocal result=nil\r\nlocal aConstructor=nil\r\n(function()\r\n\tlocal _s8_value=((self):classNamed_(className))\r\n\taConstructor=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=( aConstructor(arg1) )\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:",0
,(
function(self,className,arg1,arg2)
local result=nil
local aConstructor=nil
(function()
	local _s8_value=((self):classNamed_(className))
	aConstructor=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=( aConstructor(arg1,arg2) )
	result=_s8_value; return _s8_value
end)()
;
return (result)
;
end
)
,"private","newHandleOf: className with: arg1 with: arg2\r\n\t\" Private - Return a new instance of the class named className in the receiver's library. \"\r\n\r\n\t| result aConstructor |\r\n\taConstructor := self classNamed: className.\r\n\tresult := {' new aConstructor(arg1,arg2) ' lua' aConstructor(arg1,arg2) '}.\r\n\t^result","(\r\nfunction(self,className,arg1,arg2)\r\nlocal result=nil\r\nlocal aConstructor=nil\r\n(function()\r\n\tlocal _s8_value=((self):classNamed_(className))\r\n\taConstructor=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=( aConstructor(arg1,arg2) )\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:",0
,(
function(self,className,arg1,arg2,arg3)
local result=nil
local aConstructor=nil
(function()
	local _s8_value=((self):classNamed_(className))
	aConstructor=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=( aConstructor(arg1,arg2,arg3) )
	result=_s8_value; return _s8_value
end)()
;
return (result)
;
end
)
,"private","newHandleOf: className with: arg1 with: arg2 with: arg3\r\n\t\" Private - Return a new instance of the class named className in the receiver's library. \"\r\n\r\n\t| result aConstructor |\r\n\taConstructor := self classNamed: className.\r\n\tresult := {' new aConstructor(arg1,arg2,arg3) ' lua' aConstructor(arg1,arg2,arg3) '}.\r\n\t^result","(\r\nfunction(self,className,arg1,arg2,arg3)\r\nlocal result=nil\r\nlocal aConstructor=nil\r\n(function()\r\n\tlocal _s8_value=((self):classNamed_(className))\r\n\taConstructor=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=( aConstructor(arg1,arg2,arg3) )\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:with:",0
,(
function(self,className,arg1,arg2,arg3,arg4)
local result=nil
local aConstructor=nil
(function()
	local _s8_value=((self):classNamed_(className))
	aConstructor=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=( aConstructor(arg1,arg2,arg3,arg4) )
	result=_s8_value; return _s8_value
end)()
;
return (result)
;
end
)
,"private","newHandleOf: className with: arg1 with: arg2 with: arg3 with: arg4\r\n\t\" Private - Return a new instance of the class named className in the receiver's library. \"\r\n\r\n\t| result aConstructor |\r\n\taConstructor := self classNamed: className.\r\n\tresult := {' new aConstructor(arg1,arg2,arg3,arg4) ' lua' aConstructor(arg1,arg2,arg3,arg4) '}.\r\n\t^result","(\r\nfunction(self,className,arg1,arg2,arg3,arg4)\r\nlocal result=nil\r\nlocal aConstructor=nil\r\n(function()\r\n\tlocal _s8_value=((self):classNamed_(className))\r\n\taConstructor=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=( aConstructor(arg1,arg2,arg3,arg4) )\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:with:with:",0
,(
function(self,className,arg1,arg2,arg3,arg4,arg5)
local result=nil
local aConstructor=nil
(function()
	local _s8_value=((self):classNamed_(className))
	aConstructor=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=( aConstructor(arg1,arg2,arg3,arg4,arg5) )
	result=_s8_value; return _s8_value
end)()
;
return (result)
;
end
)
,"private","newHandleOf: className with: arg1 with: arg2 with: arg3 with: arg4 with: arg5\r\n\t\" Private - Return a new instance of the class named className in the receiver's library. \"\r\n\r\n\t| result aConstructor |\r\n\taConstructor := self classNamed: className.\r\n\tresult := {' new aConstructor(arg1,arg2,arg3,arg4,arg5) ' lua' aConstructor(arg1,arg2,arg3,arg4,arg5) '}.\r\n\t^result","(\r\nfunction(self,className,arg1,arg2,arg3,arg4,arg5)\r\nlocal result=nil\r\nlocal aConstructor=nil\r\n(function()\r\n\tlocal _s8_value=((self):classNamed_(className))\r\n\taConstructor=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=( aConstructor(arg1,arg2,arg3,arg4,arg5) )\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:with:with:with:",0
,(
function(self,className,arg1,arg2,arg3,arg4,arg5,arg6)
local result=nil
local aConstructor=nil
(function()
	local _s8_value=((self):classNamed_(className))
	aConstructor=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=( aConstructor(arg1,arg2,arg3,arg4,arg5,arg6) )
	result=_s8_value; return _s8_value
end)()
;
return (result)
;
end
)
,"private","newHandleOf: className with: arg1 with: arg2 with: arg3 with: arg4 with: arg5 with: arg6\r\n\t\" Private - Return a new instance of the class named className in the receiver's library. \"\r\n\r\n\t| result aConstructor |\r\n\taConstructor := self classNamed: className.\r\n\tresult := {' new aConstructor(arg1,arg2,arg3,arg4,arg5,arg6) ' lua' aConstructor(arg1,arg2,arg3,arg4,arg5,arg6) '}.\r\n\t^result","(\r\nfunction(self,className,arg1,arg2,arg3,arg4,arg5,arg6)\r\nlocal result=nil\r\nlocal aConstructor=nil\r\n(function()\r\n\tlocal _s8_value=((self):classNamed_(className))\r\n\taConstructor=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=( aConstructor(arg1,arg2,arg3,arg4,arg5,arg6) )\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:with:with:with:with:",0
,(
function(self,className,arg1,arg2,arg3,arg4,arg5,arg6,arg7)
local result=nil
local aConstructor=nil
(function()
	local _s8_value=((self):classNamed_(className))
	aConstructor=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=( aConstructor(arg1,arg2,arg3,arg4,arg5,arg6,arg7) )
	result=_s8_value; return _s8_value
end)()
;
return (result)
;
end
)
,"private","newHandleOf: className with: arg1 with: arg2 with: arg3 with: arg4 with: arg5 with: arg6 with: arg7\r\n\t\" Private - Return a new instance of the class named className in the receiver's library. \"\r\n\r\n\t| result aConstructor |\r\n\taConstructor := self classNamed: className.\r\n\tresult := {' new aConstructor(arg1,arg2,arg3,arg4,arg5,arg6,arg7) ' lua' aConstructor(arg1,arg2,arg3,arg4,arg5,arg6,arg7) '}.\r\n\t^result","(\r\nfunction(self,className,arg1,arg2,arg3,arg4,arg5,arg6,arg7)\r\nlocal result=nil\r\nlocal aConstructor=nil\r\n(function()\r\n\tlocal _s8_value=((self):classNamed_(className))\r\n\taConstructor=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=( aConstructor(arg1,arg2,arg3,arg4,arg5,arg6,arg7) )\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
smalltalk:bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:with:with:with:with:with:",0
,(
function(self,className,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8)
local result=nil
local aConstructor=nil
(function()
	local _s8_value=((self):classNamed_(className))
	aConstructor=_s8_value; return _s8_value
end)()
;
(function()
	local _s8_value=( aConstructor(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8) )
	result=_s8_value; return _s8_value
end)()
;
return (result)
;
end
)
,"private","newHandleOf: className with: arg1 with: arg2 with: arg3 with: arg4 with: arg5 with: arg6 with: arg7 with: arg8\r\n\t\" Private - Return a new instance of the class named className in the receiver's library. \"\r\n\r\n\t| result aConstructor |\r\n\taConstructor := self classNamed: className.\r\n\tresult := {' new aConstructor(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8) ' lua' aConstructor(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8) '}.\r\n\t^result","(\r\nfunction(self,className,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8)\r\nlocal result=nil\r\nlocal aConstructor=nil\r\n(function()\r\n\tlocal _s8_value=((self):classNamed_(className))\r\n\taConstructor=_s8_value; return _s8_value\r\nend)()\r\n;\r\n(function()\r\n\tlocal _s8_value=( aConstructor(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8) )\r\n\tresult=_s8_value; return _s8_value\r\nend)()\r\n;\r\nreturn (result)\r\n;\r\nend\r\n)")
