-- fileIn("s8/library/SimpleJson.st");
smalltalk:addClass("Json",smalltalk.NativeObject,smalltalk:newArray({"api"}),"SimpleJson")

smalltalk.Json["$klass"]["$iVarNames"]=smalltalk:newArray({"nativeClass"});

smalltalk:bind(smalltalk.Json["$klass"],"code",0
,function(self)
return ("\r\n--\r\n-- json.lua\r\n--\r\n-- Copyright (c) 2018 rxi\r\n--\r\n-- Permission is hereby granted, free of charge, to any person obtaining a copy of\r\n-- this software and associated documentation files (the \"Software\"), to deal in\r\n-- the Software without restriction, including without limitation the rights to\r\n-- use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies\r\n-- of the Software, and to permit persons to whom the Software is furnished to do\r\n-- so, subject to the following conditions:\r\n--\r\n-- The above copyright notice and this permission notice shall be included in all\r\n-- copies or substantial portions of the Software.\r\n--\r\n-- THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\r\n-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\r\n-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\r\n-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\r\n-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\r\n-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\r\n-- SOFTWARE.\r\n--\r\n\r\nlocal json = { _version = \"0.1.1\" }\r\n\r\n-------------------------------------------------------------------------------\r\n-- Encode\r\n-------------------------------------------------------------------------------\r\n\r\nlocal encode\r\n\r\nlocal escape_char_map = {\r\n  [ \"\\\\\" ] = \"\\\\\\\\\",\r\n  [ \"\\\"\" ] = \"\\\\\\\"\",\r\n  [ \"\\b\" ] = \"\\\\b\",\r\n  [ \"\\f\" ] = \"\\\\f\",\r\n  [ \"\\n\" ] = \"\\\\n\",\r\n  [ \"\\r\" ] = \"\\\\r\",\r\n  [ \"\\t\" ] = \"\\\\t\",\r\n}\r\n\r\nlocal escape_char_map_inv = { [ \"\\\\/\" ] = \"/\" }\r\nfor k, v in pairs(escape_char_map) do\r\n  escape_char_map_inv[v] = k\r\nend\r\n\r\n\r\nlocal function escape_char(c)\r\n  return escape_char_map[c] or string.format(\"\\\\u%04x\", c:byte())\r\nend\r\n\r\n\r\nlocal function encode_nil(val)\r\n  return \"null\"\r\nend\r\n\r\n\r\nlocal function encode_table(val, stack)\r\n  local res = {}\r\n  stack = stack or {}\r\n\r\n  -- Circular reference?\r\n  if stack[val] then error(\"circular reference\") end\r\n\r\n  stack[val] = true\r\n\r\n  if val[1] ~= nil or next(val) == nil then\r\n    -- Treat as array -- check keys are valid and it is not sparse\r\n    local n = 0\r\n    for k in pairs(val) do\r\n      if type(k) ~= \"number\" then\r\n        error(\"invalid table: mixed or invalid key types\")\r\n      end\r\n      n = n + 1\r\n    end\r\n    if n ~= #val then\r\n      error(\"invalid table: sparse array\")\r\n    end\r\n    -- Encode\r\n    for i, v in ipairs(val) do\r\n      table.insert(res, encode(v, stack))\r\n    end\r\n    stack[val] = nil\r\n    return \"[\" .. table.concat(res, \",\") .. \"]\"\r\n\r\n  else\r\n    -- Treat as an object\r\n    for k, v in pairs(val) do\r\n      if type(k) ~= \"string\" then\r\n        error(\"invalid table: mixed or invalid key types\")\r\n      end\r\n      table.insert(res, encode(k, stack) .. \":\" .. encode(v, stack))\r\n    end\r\n    stack[val] = nil\r\n    return \"{\" .. table.concat(res, \",\") .. \"}\"\r\n  end\r\nend\r\n\r\n\r\nlocal function encode_string(val)\r\n  return '\"' .. val:gsub('[%z\\1-\\31\\\\\"]', escape_char) .. '\"'\r\nend\r\n\r\n\r\nlocal function encode_number(val)\r\n  -- Check for NaN, -inf and inf\r\n  if val ~= val or val <= -math.huge or val >= math.huge then\r\n    error(\"unexpected number value '\" .. tostring(val) .. \"'\")\r\n  end\r\n  return string.format(\"%.14g\", val)\r\nend\r\n\r\n\r\nlocal type_func_map = {\r\n  [ \"nil\"     ] = encode_nil,\r\n  [ \"table\"   ] = encode_table,\r\n  [ \"string\"  ] = encode_string,\r\n  [ \"number\"  ] = encode_number,\r\n  [ \"boolean\" ] = tostring,\r\n}\r\n\r\n\r\nencode = function(val, stack)\r\n  local t = type(val)\r\n  local f = type_func_map[t]\r\n  if f then\r\n    return f(val, stack)\r\n  end\r\n  error(\"unexpected type '\" .. t .. \"'\")\r\nend\r\n\r\n\r\nfunction json.encode(val)\r\n  return ( encode(val) )\r\nend\r\n\r\n\r\n-------------------------------------------------------------------------------\r\n-- Decode\r\n-------------------------------------------------------------------------------\r\n\r\nlocal parse\r\n\r\nlocal function create_set(...)\r\n  local res = {}\r\n  for i = 1, select(\"#\", ...) do\r\n    res[ select(i, ...) ] = true\r\n  end\r\n  return res\r\nend\r\n\r\nlocal space_chars   = create_set(\" \", \"\\t\", \"\\r\", \"\\n\")\r\nlocal delim_chars   = create_set(\" \", \"\\t\", \"\\r\", \"\\n\", \"]\", \"}\", \",\")\r\nlocal escape_chars  = create_set(\"\\\\\", \"/\", '\"', \"b\", \"f\", \"n\", \"r\", \"t\", \"u\")\r\nlocal literals      = create_set(\"true\", \"false\", \"null\")\r\n\r\nlocal literal_map = {\r\n  [ \"true\"  ] = true,\r\n  [ \"false\" ] = false,\r\n  [ \"null\"  ] = nil,\r\n}\r\n\r\n\r\nlocal function next_char(str, idx, set, negate)\r\n  for i = idx, #str do\r\n    if set[str:sub(i, i)] ~= negate then\r\n      return i\r\n    end\r\n  end\r\n  return #str + 1\r\nend\r\n\r\n\r\nlocal function decode_error(str, idx, msg)\r\n  local line_count = 1\r\n  local col_count = 1\r\n  for i = 1, idx - 1 do\r\n    col_count = col_count + 1\r\n    if str:sub(i, i) == \"\\n\" then\r\n      line_count = line_count + 1\r\n      col_count = 1\r\n    end\r\n  end\r\n  error( string.format(\"%s at line %d col %d\", msg, line_count, col_count) )\r\nend\r\n\r\n\r\nlocal function codepoint_to_utf8(n)\r\n  -- http://scripts.sil.org/cms/scripts/page.php?site_id=nrsi&id=iws-appendixa\r\n  local f = math.floor\r\n  if n <= 0x7f then\r\n    return string.char(n)\r\n  elseif n <= 0x7ff then\r\n    return string.char(f(n / 64) + 192, n % 64 + 128)\r\n  elseif n <= 0xffff then\r\n    return string.char(f(n / 4096) + 224, f(n % 4096 / 64) + 128, n % 64 + 128)\r\n  elseif n <= 0x10ffff then\r\n    return string.char(f(n / 262144) + 240, f(n % 262144 / 4096) + 128,\r\n                       f(n % 4096 / 64) + 128, n % 64 + 128)\r\n  end\r\n  error( string.format(\"invalid unicode codepoint '%x'\", n) )\r\nend\r\n\r\n\r\nlocal function parse_unicode_escape(s)\r\n  local n1 = tonumber( s:sub(3, 6),  16 )\r\n  local n2 = tonumber( s:sub(9, 12), 16 )\r\n  -- Surrogate pair?\r\n  if n2 then\r\n    return codepoint_to_utf8((n1 - 0xd800) * 0x400 + (n2 - 0xdc00) + 0x10000)\r\n  else\r\n    return codepoint_to_utf8(n1)\r\n  end\r\nend\r\n\r\n\r\nlocal function parse_string(str, i)\r\n  local has_unicode_escape = false\r\n  local has_surrogate_escape = false\r\n  local has_escape = false\r\n  local last\r\n  for j = i + 1, #str do\r\n    local x = str:byte(j)\r\n\r\n    if x < 32 then\r\n      decode_error(str, j, \"control character in string\")\r\n    end\r\n\r\n    if last == 92 then -- \"\\\\\" (escape char)\r\n      if x == 117 then -- \"u\" (unicode escape sequence)\r\n        local hex = str:sub(j + 1, j + 5)\r\n        if not hex:find(\"%x%x%x%x\") then\r\n          decode_error(str, j, \"invalid unicode escape in string\")\r\n        end\r\n        if hex:find(\"^[dD][89aAbB]\") then\r\n          has_surrogate_escape = true\r\n        else\r\n          has_unicode_escape = true\r\n        end\r\n      else\r\n        local c = string.char(x)\r\n        if not escape_chars[c] then\r\n          decode_error(str, j, \"invalid escape char '\" .. c .. \"' in string\")\r\n        end\r\n        has_escape = true\r\n      end\r\n      last = nil\r\n\r\n    elseif x == 34 then -- '\"' (end of string)\r\n      local s = str:sub(i + 1, j - 1)\r\n      if has_surrogate_escape then\r\n        s = s:gsub(\"\\\\u[dD][89aAbB]..\\\\u....\", parse_unicode_escape)\r\n      end\r\n      if has_unicode_escape then\r\n        s = s:gsub(\"\\\\u....\", parse_unicode_escape)\r\n      end\r\n      if has_escape then\r\n        s = s:gsub(\"\\\\.\", escape_char_map_inv)\r\n      end\r\n      return s, j + 1\r\n\r\n    else\r\n      last = x\r\n    end\r\n  end\r\n  decode_error(str, i, \"expected closing quote for string\")\r\nend\r\n\r\n\r\nlocal function parse_number(str, i)\r\n  local x = next_char(str, i, delim_chars)\r\n  local s = str:sub(i, x - 1)\r\n  local n = tonumber(s)\r\n  if not n then\r\n    decode_error(str, i, \"invalid number '\" .. s .. \"'\")\r\n  end\r\n  return n, x\r\nend\r\n\r\n\r\nlocal function parse_literal(str, i)\r\n  local x = next_char(str, i, delim_chars)\r\n  local word = str:sub(i, x - 1)\r\n  if not literals[word] then\r\n    decode_error(str, i, \"invalid literal '\" .. word .. \"'\")\r\n  end\r\n  return literal_map[word], x\r\nend\r\n\r\n\r\nlocal function parse_array(str, i)\r\n  local res = {}\r\n  local n = 1\r\n  i = i + 1\r\n  while 1 do\r\n    local x\r\n    i = next_char(str, i, space_chars, true)\r\n    -- Empty / end of array?\r\n    if str:sub(i, i) == \"]\" then\r\n      i = i + 1\r\n      break\r\n    end\r\n    -- Read token\r\n    x, i = parse(str, i)\r\n    res[n] = x\r\n    n = n + 1\r\n    -- Next token\r\n    i = next_char(str, i, space_chars, true)\r\n    local chr = str:sub(i, i)\r\n    i = i + 1\r\n    if chr == \"]\" then break end\r\n    if chr ~= \",\" then decode_error(str, i, \"expected ']' or ','\") end\r\n  end\r\n  return res, i\r\nend\r\n\r\n\r\nlocal function parse_object(str, i)\r\n  local res = {}\r\n  i = i + 1\r\n  while 1 do\r\n    local key, val\r\n    i = next_char(str, i, space_chars, true)\r\n    -- Empty / end of object?\r\n    if str:sub(i, i) == \"}\" then\r\n      i = i + 1\r\n      break\r\n    end\r\n    -- Read key\r\n    if str:sub(i, i) ~= '\"' then\r\n      decode_error(str, i, \"expected string for key\")\r\n    end\r\n    key, i = parse(str, i)\r\n    -- Read ':' delimiter\r\n    i = next_char(str, i, space_chars, true)\r\n    if str:sub(i, i) ~= \":\" then\r\n      decode_error(str, i, \"expected ':' after key\")\r\n    end\r\n    i = next_char(str, i + 1, space_chars, true)\r\n    -- Read value\r\n    val, i = parse(str, i)\r\n    -- Set\r\n    res[key] = val\r\n    -- Next token\r\n    i = next_char(str, i, space_chars, true)\r\n    local chr = str:sub(i, i)\r\n    i = i + 1\r\n    if chr == \"}\" then break end\r\n    if chr ~= \",\" then decode_error(str, i, \"expected '}' or ','\") end\r\n  end\r\n  return res, i\r\nend\r\n\r\n\r\nlocal char_func_map = {\r\n  [ '\"' ] = parse_string,\r\n  [ \"0\" ] = parse_number,\r\n  [ \"1\" ] = parse_number,\r\n  [ \"2\" ] = parse_number,\r\n  [ \"3\" ] = parse_number,\r\n  [ \"4\" ] = parse_number,\r\n  [ \"5\" ] = parse_number,\r\n  [ \"6\" ] = parse_number,\r\n  [ \"7\" ] = parse_number,\r\n  [ \"8\" ] = parse_number,\r\n  [ \"9\" ] = parse_number,\r\n  [ \"-\" ] = parse_number,\r\n  [ \"t\" ] = parse_literal,\r\n  [ \"f\" ] = parse_literal,\r\n  [ \"n\" ] = parse_literal,\r\n  [ \"[\" ] = parse_array,\r\n  [ \"{\" ] = parse_object,\r\n}\r\n\r\n\r\nparse = function(str, idx)\r\n  local chr = str:sub(idx, idx)\r\n  local f = char_func_map[chr]\r\n  if f then\r\n    return f(str, idx)\r\n  end\r\n  decode_error(str, idx, \"unexpected character '\" .. chr .. \"'\")\r\nend\r\n\r\n\r\nfunction json.decode(str)\r\n  if type(str) ~= \"string\" then\r\n    error(\"expected argument of type string, got \" .. type(str))\r\n  end\r\n  local res, idx = parse(str, next_char(str, 1, space_chars, true))\r\n  idx = next_char(str, idx, space_chars, true)\r\n  if idx <= #str then\r\n    decode_error(str, idx, \"trailing garbage\")\r\n  end\r\n  return res\r\nend\r\n\r\n\r\nreturn json\r\n")
;
end
,"constants","code\r\n\t\" Returns the embedded source code of the library. \"\r\n\t^'\r\n--\r\n-- json.lua\r\n--\r\n-- Copyright (c) 2018 rxi\r\n--\r\n-- Permission is hereby granted, free of charge, to any person obtaining a copy of\r\n-- this software and associated documentation files (the \"Software\"), to deal in\r\n-- the Software without restriction, including without limitation the rights to\r\n-- use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies\r\n-- of the Software, and to permit persons to whom the Software is furnished to do\r\n-- so, subject to the following conditions:\r\n--\r\n-- The above copyright notice and this permission notice shall be included in all\r\n-- copies or substantial portions of the Software.\r\n--\r\n-- THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\r\n-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\r\n-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\r\n-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\r\n-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\r\n-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\r\n-- SOFTWARE.\r\n--\r\n\r\nlocal json = { _version = \"0.1.1\" }\r\n\r\n-------------------------------------------------------------------------------\r\n-- Encode\r\n-------------------------------------------------------------------------------\r\n\r\nlocal encode\r\n\r\nlocal escape_char_map = {\r\n  [ \"\\\\\" ] = \"\\\\\\\\\",\r\n  [ \"\\\"\" ] = \"\\\\\\\"\",\r\n  [ \"\\b\" ] = \"\\\\b\",\r\n  [ \"\\f\" ] = \"\\\\f\",\r\n  [ \"\\n\" ] = \"\\\\n\",\r\n  [ \"\\r\" ] = \"\\\\r\",\r\n  [ \"\\t\" ] = \"\\\\t\",\r\n}\r\n\r\nlocal escape_char_map_inv = { [ \"\\\\/\" ] = \"/\" }\r\nfor k, v in pairs(escape_char_map) do\r\n  escape_char_map_inv[v] = k\r\nend\r\n\r\n\r\nlocal function escape_char(c)\r\n  return escape_char_map[c] or string.format(\"\\\\u%04x\", c:byte())\r\nend\r\n\r\n\r\nlocal function encode_nil(val)\r\n  return \"null\"\r\nend\r\n\r\n\r\nlocal function encode_table(val, stack)\r\n  local res = {}\r\n  stack = stack or {}\r\n\r\n  -- Circular reference?\r\n  if stack[val] then error(\"circular reference\") end\r\n\r\n  stack[val] = true\r\n\r\n  if val[1] ~= nil or next(val) == nil then\r\n    -- Treat as array -- check keys are valid and it is not sparse\r\n    local n = 0\r\n    for k in pairs(val) do\r\n      if type(k) ~= \"number\" then\r\n        error(\"invalid table: mixed or invalid key types\")\r\n      end\r\n      n = n + 1\r\n    end\r\n    if n ~= #val then\r\n      error(\"invalid table: sparse array\")\r\n    end\r\n    -- Encode\r\n    for i, v in ipairs(val) do\r\n      table.insert(res, encode(v, stack))\r\n    end\r\n    stack[val] = nil\r\n    return \"[\" .. table.concat(res, \",\") .. \"]\"\r\n\r\n  else\r\n    -- Treat as an object\r\n    for k, v in pairs(val) do\r\n      if type(k) ~= \"string\" then\r\n        error(\"invalid table: mixed or invalid key types\")\r\n      end\r\n      table.insert(res, encode(k, stack) .. \":\" .. encode(v, stack))\r\n    end\r\n    stack[val] = nil\r\n    return \"{\" .. table.concat(res, \",\") .. \"}\"\r\n  end\r\nend\r\n\r\n\r\nlocal function encode_string(val)\r\n  return ''\"'' .. val:gsub(''[%z\\1-\\31\\\\\"]'', escape_char) .. ''\"''\r\nend\r\n\r\n\r\nlocal function encode_number(val)\r\n  -- Check for NaN, -inf and inf\r\n  if val ~= val or val <= -math.huge or val >= math.huge then\r\n    error(\"unexpected number value ''\" .. tostring(val) .. \"''\")\r\n  end\r\n  return string.format(\"%.14g\", val)\r\nend\r\n\r\n\r\nlocal type_func_map = {\r\n  [ \"nil\"     ] = encode_nil,\r\n  [ \"table\"   ] = encode_table,\r\n  [ \"string\"  ] = encode_string,\r\n  [ \"number\"  ] = encode_number,\r\n  [ \"boolean\" ] = tostring,\r\n}\r\n\r\n\r\nencode = function(val, stack)\r\n  local t = type(val)\r\n  local f = type_func_map[t]\r\n  if f then\r\n    return f(val, stack)\r\n  end\r\n  error(\"unexpected type ''\" .. t .. \"''\")\r\nend\r\n\r\n\r\nfunction json.encode(val)\r\n  return ( encode(val) )\r\nend\r\n\r\n\r\n-------------------------------------------------------------------------------\r\n-- Decode\r\n-------------------------------------------------------------------------------\r\n\r\nlocal parse\r\n\r\nlocal function create_set(...)\r\n  local res = {}\r\n  for i = 1, select(\"#\", ...) do\r\n    res[ select(i, ...) ] = true\r\n  end\r\n  return res\r\nend\r\n\r\nlocal space_chars   = create_set(\" \", \"\\t\", \"\\r\", \"\\n\")\r\nlocal delim_chars   = create_set(\" \", \"\\t\", \"\\r\", \"\\n\", \"]\", \"}\", \",\")\r\nlocal escape_chars  = create_set(\"\\\\\", \"/\", ''\"'', \"b\", \"f\", \"n\", \"r\", \"t\", \"u\")\r\nlocal literals      = create_set(\"true\", \"false\", \"null\")\r\n\r\nlocal literal_map = {\r\n  [ \"true\"  ] = true,\r\n  [ \"false\" ] = false,\r\n  [ \"null\"  ] = nil,\r\n}\r\n\r\n\r\nlocal function next_char(str, idx, set, negate)\r\n  for i = idx, #str do\r\n    if set[str:sub(i, i)] ~= negate then\r\n      return i\r\n    end\r\n  end\r\n  return #str + 1\r\nend\r\n\r\n\r\nlocal function decode_error(str, idx, msg)\r\n  local line_count = 1\r\n  local col_count = 1\r\n  for i = 1, idx - 1 do\r\n    col_count = col_count + 1\r\n    if str:sub(i, i) == \"\\n\" then\r\n      line_count = line_count + 1\r\n      col_count = 1\r\n    end\r\n  end\r\n  error( string.format(\"%s at line %d col %d\", msg, line_count, col_count) )\r\nend\r\n\r\n\r\nlocal function codepoint_to_utf8(n)\r\n  -- http://scripts.sil.org/cms/scripts/page.php?site_id=nrsi&id=iws-appendixa\r\n  local f = math.floor\r\n  if n <= 0x7f then\r\n    return string.char(n)\r\n  elseif n <= 0x7ff then\r\n    return string.char(f(n / 64) + 192, n % 64 + 128)\r\n  elseif n <= 0xffff then\r\n    return string.char(f(n / 4096) + 224, f(n % 4096 / 64) + 128, n % 64 + 128)\r\n  elseif n <= 0x10ffff then\r\n    return string.char(f(n / 262144) + 240, f(n % 262144 / 4096) + 128,\r\n                       f(n % 4096 / 64) + 128, n % 64 + 128)\r\n  end\r\n  error( string.format(\"invalid unicode codepoint ''%x''\", n) )\r\nend\r\n\r\n\r\nlocal function parse_unicode_escape(s)\r\n  local n1 = tonumber( s:sub(3, 6),  16 )\r\n  local n2 = tonumber( s:sub(9, 12), 16 )\r\n  -- Surrogate pair?\r\n  if n2 then\r\n    return codepoint_to_utf8((n1 - 0xd800) * 0x400 + (n2 - 0xdc00) + 0x10000)\r\n  else\r\n    return codepoint_to_utf8(n1)\r\n  end\r\nend\r\n\r\n\r\nlocal function parse_string(str, i)\r\n  local has_unicode_escape = false\r\n  local has_surrogate_escape = false\r\n  local has_escape = false\r\n  local last\r\n  for j = i + 1, #str do\r\n    local x = str:byte(j)\r\n\r\n    if x < 32 then\r\n      decode_error(str, j, \"control character in string\")\r\n    end\r\n\r\n    if last == 92 then -- \"\\\\\" (escape char)\r\n      if x == 117 then -- \"u\" (unicode escape sequence)\r\n        local hex = str:sub(j + 1, j + 5)\r\n        if not hex:find(\"%x%x%x%x\") then\r\n          decode_error(str, j, \"invalid unicode escape in string\")\r\n        end\r\n        if hex:find(\"^[dD][89aAbB]\") then\r\n          has_surrogate_escape = true\r\n        else\r\n          has_unicode_escape = true\r\n        end\r\n      else\r\n        local c = string.char(x)\r\n        if not escape_chars[c] then\r\n          decode_error(str, j, \"invalid escape char ''\" .. c .. \"'' in string\")\r\n        end\r\n        has_escape = true\r\n      end\r\n      last = nil\r\n\r\n    elseif x == 34 then -- ''\"'' (end of string)\r\n      local s = str:sub(i + 1, j - 1)\r\n      if has_surrogate_escape then\r\n        s = s:gsub(\"\\\\u[dD][89aAbB]..\\\\u....\", parse_unicode_escape)\r\n      end\r\n      if has_unicode_escape then\r\n        s = s:gsub(\"\\\\u....\", parse_unicode_escape)\r\n      end\r\n      if has_escape then\r\n        s = s:gsub(\"\\\\.\", escape_char_map_inv)\r\n      end\r\n      return s, j + 1\r\n\r\n    else\r\n      last = x\r\n    end\r\n  end\r\n  decode_error(str, i, \"expected closing quote for string\")\r\nend\r\n\r\n\r\nlocal function parse_number(str, i)\r\n  local x = next_char(str, i, delim_chars)\r\n  local s = str:sub(i, x - 1)\r\n  local n = tonumber(s)\r\n  if not n then\r\n    decode_error(str, i, \"invalid number ''\" .. s .. \"''\")\r\n  end\r\n  return n, x\r\nend\r\n\r\n\r\nlocal function parse_literal(str, i)\r\n  local x = next_char(str, i, delim_chars)\r\n  local word = str:sub(i, x - 1)\r\n  if not literals[word] then\r\n    decode_error(str, i, \"invalid literal ''\" .. word .. \"''\")\r\n  end\r\n  return literal_map[word], x\r\nend\r\n\r\n\r\nlocal function parse_array(str, i)\r\n  local res = {}\r\n  local n = 1\r\n  i = i + 1\r\n  while 1 do\r\n    local x\r\n    i = next_char(str, i, space_chars, true)\r\n    -- Empty / end of array?\r\n    if str:sub(i, i) == \"]\" then\r\n      i = i + 1\r\n      break\r\n    end\r\n    -- Read token\r\n    x, i = parse(str, i)\r\n    res[n] = x\r\n    n = n + 1\r\n    -- Next token\r\n    i = next_char(str, i, space_chars, true)\r\n    local chr = str:sub(i, i)\r\n    i = i + 1\r\n    if chr == \"]\" then break end\r\n    if chr ~= \",\" then decode_error(str, i, \"expected '']'' or '',''\") end\r\n  end\r\n  return res, i\r\nend\r\n\r\n\r\nlocal function parse_object(str, i)\r\n  local res = {}\r\n  i = i + 1\r\n  while 1 do\r\n    local key, val\r\n    i = next_char(str, i, space_chars, true)\r\n    -- Empty / end of object?\r\n    if str:sub(i, i) == \"}\" then\r\n      i = i + 1\r\n      break\r\n    end\r\n    -- Read key\r\n    if str:sub(i, i) ~= ''\"'' then\r\n      decode_error(str, i, \"expected string for key\")\r\n    end\r\n    key, i = parse(str, i)\r\n    -- Read '':'' delimiter\r\n    i = next_char(str, i, space_chars, true)\r\n    if str:sub(i, i) ~= \":\" then\r\n      decode_error(str, i, \"expected '':'' after key\")\r\n    end\r\n    i = next_char(str, i + 1, space_chars, true)\r\n    -- Read value\r\n    val, i = parse(str, i)\r\n    -- Set\r\n    res[key] = val\r\n    -- Next token\r\n    i = next_char(str, i, space_chars, true)\r\n    local chr = str:sub(i, i)\r\n    i = i + 1\r\n    if chr == \"}\" then break end\r\n    if chr ~= \",\" then decode_error(str, i, \"expected ''}'' or '',''\") end\r\n  end\r\n  return res, i\r\nend\r\n\r\n\r\nlocal char_func_map = {\r\n  [ ''\"'' ] = parse_string,\r\n  [ \"0\" ] = parse_number,\r\n  [ \"1\" ] = parse_number,\r\n  [ \"2\" ] = parse_number,\r\n  [ \"3\" ] = parse_number,\r\n  [ \"4\" ] = parse_number,\r\n  [ \"5\" ] = parse_number,\r\n  [ \"6\" ] = parse_number,\r\n  [ \"7\" ] = parse_number,\r\n  [ \"8\" ] = parse_number,\r\n  [ \"9\" ] = parse_number,\r\n  [ \"-\" ] = parse_number,\r\n  [ \"t\" ] = parse_literal,\r\n  [ \"f\" ] = parse_literal,\r\n  [ \"n\" ] = parse_literal,\r\n  [ \"[\" ] = parse_array,\r\n  [ \"{\" ] = parse_object,\r\n}\r\n\r\n\r\nparse = function(str, idx)\r\n  local chr = str:sub(idx, idx)\r\n  local f = char_func_map[chr]\r\n  if f then\r\n    return f(str, idx)\r\n  end\r\n  decode_error(str, idx, \"unexpected character ''\" .. chr .. \"''\")\r\nend\r\n\r\n\r\nfunction json.decode(str)\r\n  if type(str) ~= \"string\" then\r\n    error(\"expected argument of type string, got \" .. type(str))\r\n  end\r\n  local res, idx = parse(str, next_char(str, 1, space_chars, true))\r\n  idx = next_char(str, idx, space_chars, true)\r\n  if idx <= #str then\r\n    decode_error(str, idx, \"trailing garbage\")\r\n  end\r\n  return res\r\nend\r\n\r\n\r\nreturn json\r\n'","function(self)\r\nreturn (\"\\r\\n--\\r\\n-- json.lua\\r\\n--\\r\\n-- Copyright (c) 2018 rxi\\r\\n--\\r\\n-- Permission is hereby granted, free of charge, to any person obtaining a copy of\\r\\n-- this software and associated documentation files (the \\\"Software\\\"), to deal in\\r\\n-- the Software without restriction, including without limitation the rights to\\r\\n-- use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies\\r\\n-- of the Software, and to permit persons to whom the Software is furnished to do\\r\\n-- so, subject to the following conditions:\\r\\n--\\r\\n-- The above copyright notice and this permission notice shall be included in all\\r\\n-- copies or substantial portions of the Software.\\r\\n--\\r\\n-- THE SOFTWARE IS PROVIDED \\\"AS IS\\\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\\r\\n-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\\r\\n-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\\r\\n-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\\r\\n-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\\r\\n-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\\r\\n-- SOFTWARE.\\r\\n--\\r\\n\\r\\nlocal json = { _version = \\\"0.1.1\\\" }\\r\\n\\r\\n-------------------------------------------------------------------------------\\r\\n-- Encode\\r\\n-------------------------------------------------------------------------------\\r\\n\\r\\nlocal encode\\r\\n\\r\\nlocal escape_char_map = {\\r\\n  [ \\\"\\\\\\\\\\\" ] = \\\"\\\\\\\\\\\\\\\\\\\",\\r\\n  [ \\\"\\\\\\\"\\\" ] = \\\"\\\\\\\\\\\\\\\"\\\",\\r\\n  [ \\\"\\\\b\\\" ] = \\\"\\\\\\\\b\\\",\\r\\n  [ \\\"\\\\f\\\" ] = \\\"\\\\\\\\f\\\",\\r\\n  [ \\\"\\\\n\\\" ] = \\\"\\\\\\\\n\\\",\\r\\n  [ \\\"\\\\r\\\" ] = \\\"\\\\\\\\r\\\",\\r\\n  [ \\\"\\\\t\\\" ] = \\\"\\\\\\\\t\\\",\\r\\n}\\r\\n\\r\\nlocal escape_char_map_inv = { [ \\\"\\\\\\\\/\\\" ] = \\\"/\\\" }\\r\\nfor k, v in pairs(escape_char_map) do\\r\\n  escape_char_map_inv[v] = k\\r\\nend\\r\\n\\r\\n\\r\\nlocal function escape_char(c)\\r\\n  return escape_char_map[c] or string.format(\\\"\\\\\\\\u%04x\\\", c:byte())\\r\\nend\\r\\n\\r\\n\\r\\nlocal function encode_nil(val)\\r\\n  return \\\"null\\\"\\r\\nend\\r\\n\\r\\n\\r\\nlocal function encode_table(val, stack)\\r\\n  local res = {}\\r\\n  stack = stack or {}\\r\\n\\r\\n  -- Circular reference?\\r\\n  if stack[val] then error(\\\"circular reference\\\") end\\r\\n\\r\\n  stack[val] = true\\r\\n\\r\\n  if val[1] ~= nil or next(val) == nil then\\r\\n    -- Treat as array -- check keys are valid and it is not sparse\\r\\n    local n = 0\\r\\n    for k in pairs(val) do\\r\\n      if type(k) ~= \\\"number\\\" then\\r\\n        error(\\\"invalid table: mixed or invalid key types\\\")\\r\\n      end\\r\\n      n = n + 1\\r\\n    end\\r\\n    if n ~= #val then\\r\\n      error(\\\"invalid table: sparse array\\\")\\r\\n    end\\r\\n    -- Encode\\r\\n    for i, v in ipairs(val) do\\r\\n      table.insert(res, encode(v, stack))\\r\\n    end\\r\\n    stack[val] = nil\\r\\n    return \\\"[\\\" .. table.concat(res, \\\",\\\") .. \\\"]\\\"\\r\\n\\r\\n  else\\r\\n    -- Treat as an object\\r\\n    for k, v in pairs(val) do\\r\\n      if type(k) ~= \\\"string\\\" then\\r\\n        error(\\\"invalid table: mixed or invalid key types\\\")\\r\\n      end\\r\\n      table.insert(res, encode(k, stack) .. \\\":\\\" .. encode(v, stack))\\r\\n    end\\r\\n    stack[val] = nil\\r\\n    return \\\"{\\\" .. table.concat(res, \\\",\\\") .. \\\"}\\\"\\r\\n  end\\r\\nend\\r\\n\\r\\n\\r\\nlocal function encode_string(val)\\r\\n  return '\\\"' .. val:gsub('[%z\\\\1-\\\\31\\\\\\\\\\\"]', escape_char) .. '\\\"'\\r\\nend\\r\\n\\r\\n\\r\\nlocal function encode_number(val)\\r\\n  -- Check for NaN, -inf and inf\\r\\n  if val ~= val or val <= -math.huge or val >= math.huge then\\r\\n    error(\\\"unexpected number value '\\\" .. tostring(val) .. \\\"'\\\")\\r\\n  end\\r\\n  return string.format(\\\"%.14g\\\", val)\\r\\nend\\r\\n\\r\\n\\r\\nlocal type_func_map = {\\r\\n  [ \\\"nil\\\"     ] = encode_nil,\\r\\n  [ \\\"table\\\"   ] = encode_table,\\r\\n  [ \\\"string\\\"  ] = encode_string,\\r\\n  [ \\\"number\\\"  ] = encode_number,\\r\\n  [ \\\"boolean\\\" ] = tostring,\\r\\n}\\r\\n\\r\\n\\r\\nencode = function(val, stack)\\r\\n  local t = type(val)\\r\\n  local f = type_func_map[t]\\r\\n  if f then\\r\\n    return f(val, stack)\\r\\n  end\\r\\n  error(\\\"unexpected type '\\\" .. t .. \\\"'\\\")\\r\\nend\\r\\n\\r\\n\\r\\nfunction json.encode(val)\\r\\n  return ( encode(val) )\\r\\nend\\r\\n\\r\\n\\r\\n-------------------------------------------------------------------------------\\r\\n-- Decode\\r\\n-------------------------------------------------------------------------------\\r\\n\\r\\nlocal parse\\r\\n\\r\\nlocal function create_set(...)\\r\\n  local res = {}\\r\\n  for i = 1, select(\\\"#\\\", ...) do\\r\\n    res[ select(i, ...) ] = true\\r\\n  end\\r\\n  return res\\r\\nend\\r\\n\\r\\nlocal space_chars   = create_set(\\\" \\\", \\\"\\\\t\\\", \\\"\\\\r\\\", \\\"\\\\n\\\")\\r\\nlocal delim_chars   = create_set(\\\" \\\", \\\"\\\\t\\\", \\\"\\\\r\\\", \\\"\\\\n\\\", \\\"]\\\", \\\"}\\\", \\\",\\\")\\r\\nlocal escape_chars  = create_set(\\\"\\\\\\\\\\\", \\\"/\\\", '\\\"', \\\"b\\\", \\\"f\\\", \\\"n\\\", \\\"r\\\", \\\"t\\\", \\\"u\\\")\\r\\nlocal literals      = create_set(\\\"true\\\", \\\"false\\\", \\\"null\\\")\\r\\n\\r\\nlocal literal_map = {\\r\\n  [ \\\"true\\\"  ] = true,\\r\\n  [ \\\"false\\\" ] = false,\\r\\n  [ \\\"null\\\"  ] = nil,\\r\\n}\\r\\n\\r\\n\\r\\nlocal function next_char(str, idx, set, negate)\\r\\n  for i = idx, #str do\\r\\n    if set[str:sub(i, i)] ~= negate then\\r\\n      return i\\r\\n    end\\r\\n  end\\r\\n  return #str + 1\\r\\nend\\r\\n\\r\\n\\r\\nlocal function decode_error(str, idx, msg)\\r\\n  local line_count = 1\\r\\n  local col_count = 1\\r\\n  for i = 1, idx - 1 do\\r\\n    col_count = col_count + 1\\r\\n    if str:sub(i, i) == \\\"\\\\n\\\" then\\r\\n      line_count = line_count + 1\\r\\n      col_count = 1\\r\\n    end\\r\\n  end\\r\\n  error( string.format(\\\"%s at line %d col %d\\\", msg, line_count, col_count) )\\r\\nend\\r\\n\\r\\n\\r\\nlocal function codepoint_to_utf8(n)\\r\\n  -- http://scripts.sil.org/cms/scripts/page.php?site_id=nrsi&id=iws-appendixa\\r\\n  local f = math.floor\\r\\n  if n <= 0x7f then\\r\\n    return string.char(n)\\r\\n  elseif n <= 0x7ff then\\r\\n    return string.char(f(n / 64) + 192, n % 64 + 128)\\r\\n  elseif n <= 0xffff then\\r\\n    return string.char(f(n / 4096) + 224, f(n % 4096 / 64) + 128, n % 64 + 128)\\r\\n  elseif n <= 0x10ffff then\\r\\n    return string.char(f(n / 262144) + 240, f(n % 262144 / 4096) + 128,\\r\\n                       f(n % 4096 / 64) + 128, n % 64 + 128)\\r\\n  end\\r\\n  error( string.format(\\\"invalid unicode codepoint '%x'\\\", n) )\\r\\nend\\r\\n\\r\\n\\r\\nlocal function parse_unicode_escape(s)\\r\\n  local n1 = tonumber( s:sub(3, 6),  16 )\\r\\n  local n2 = tonumber( s:sub(9, 12), 16 )\\r\\n  -- Surrogate pair?\\r\\n  if n2 then\\r\\n    return codepoint_to_utf8((n1 - 0xd800) * 0x400 + (n2 - 0xdc00) + 0x10000)\\r\\n  else\\r\\n    return codepoint_to_utf8(n1)\\r\\n  end\\r\\nend\\r\\n\\r\\n\\r\\nlocal function parse_string(str, i)\\r\\n  local has_unicode_escape = false\\r\\n  local has_surrogate_escape = false\\r\\n  local has_escape = false\\r\\n  local last\\r\\n  for j = i + 1, #str do\\r\\n    local x = str:byte(j)\\r\\n\\r\\n    if x < 32 then\\r\\n      decode_error(str, j, \\\"control character in string\\\")\\r\\n    end\\r\\n\\r\\n    if last == 92 then -- \\\"\\\\\\\\\\\" (escape char)\\r\\n      if x == 117 then -- \\\"u\\\" (unicode escape sequence)\\r\\n        local hex = str:sub(j + 1, j + 5)\\r\\n        if not hex:find(\\\"%x%x%x%x\\\") then\\r\\n          decode_error(str, j, \\\"invalid unicode escape in string\\\")\\r\\n        end\\r\\n        if hex:find(\\\"^[dD][89aAbB]\\\") then\\r\\n          has_surrogate_escape = true\\r\\n        else\\r\\n          has_unicode_escape = true\\r\\n        end\\r\\n      else\\r\\n        local c = string.char(x)\\r\\n        if not escape_chars[c] then\\r\\n          decode_error(str, j, \\\"invalid escape char '\\\" .. c .. \\\"' in string\\\")\\r\\n        end\\r\\n        has_escape = true\\r\\n      end\\r\\n      last = nil\\r\\n\\r\\n    elseif x == 34 then -- '\\\"' (end of string)\\r\\n      local s = str:sub(i + 1, j - 1)\\r\\n      if has_surrogate_escape then\\r\\n        s = s:gsub(\\\"\\\\\\\\u[dD][89aAbB]..\\\\\\\\u....\\\", parse_unicode_escape)\\r\\n      end\\r\\n      if has_unicode_escape then\\r\\n        s = s:gsub(\\\"\\\\\\\\u....\\\", parse_unicode_escape)\\r\\n      end\\r\\n      if has_escape then\\r\\n        s = s:gsub(\\\"\\\\\\\\.\\\", escape_char_map_inv)\\r\\n      end\\r\\n      return s, j + 1\\r\\n\\r\\n    else\\r\\n      last = x\\r\\n    end\\r\\n  end\\r\\n  decode_error(str, i, \\\"expected closing quote for string\\\")\\r\\nend\\r\\n\\r\\n\\r\\nlocal function parse_number(str, i)\\r\\n  local x = next_char(str, i, delim_chars)\\r\\n  local s = str:sub(i, x - 1)\\r\\n  local n = tonumber(s)\\r\\n  if not n then\\r\\n    decode_error(str, i, \\\"invalid number '\\\" .. s .. \\\"'\\\")\\r\\n  end\\r\\n  return n, x\\r\\nend\\r\\n\\r\\n\\r\\nlocal function parse_literal(str, i)\\r\\n  local x = next_char(str, i, delim_chars)\\r\\n  local word = str:sub(i, x - 1)\\r\\n  if not literals[word] then\\r\\n    decode_error(str, i, \\\"invalid literal '\\\" .. word .. \\\"'\\\")\\r\\n  end\\r\\n  return literal_map[word], x\\r\\nend\\r\\n\\r\\n\\r\\nlocal function parse_array(str, i)\\r\\n  local res = {}\\r\\n  local n = 1\\r\\n  i = i + 1\\r\\n  while 1 do\\r\\n    local x\\r\\n    i = next_char(str, i, space_chars, true)\\r\\n    -- Empty / end of array?\\r\\n    if str:sub(i, i) == \\\"]\\\" then\\r\\n      i = i + 1\\r\\n      break\\r\\n    end\\r\\n    -- Read token\\r\\n    x, i = parse(str, i)\\r\\n    res[n] = x\\r\\n    n = n + 1\\r\\n    -- Next token\\r\\n    i = next_char(str, i, space_chars, true)\\r\\n    local chr = str:sub(i, i)\\r\\n    i = i + 1\\r\\n    if chr == \\\"]\\\" then break end\\r\\n    if chr ~= \\\",\\\" then decode_error(str, i, \\\"expected ']' or ','\\\") end\\r\\n  end\\r\\n  return res, i\\r\\nend\\r\\n\\r\\n\\r\\nlocal function parse_object(str, i)\\r\\n  local res = {}\\r\\n  i = i + 1\\r\\n  while 1 do\\r\\n    local key, val\\r\\n    i = next_char(str, i, space_chars, true)\\r\\n    -- Empty / end of object?\\r\\n    if str:sub(i, i) == \\\"}\\\" then\\r\\n      i = i + 1\\r\\n      break\\r\\n    end\\r\\n    -- Read key\\r\\n    if str:sub(i, i) ~= '\\\"' then\\r\\n      decode_error(str, i, \\\"expected string for key\\\")\\r\\n    end\\r\\n    key, i = parse(str, i)\\r\\n    -- Read ':' delimiter\\r\\n    i = next_char(str, i, space_chars, true)\\r\\n    if str:sub(i, i) ~= \\\":\\\" then\\r\\n      decode_error(str, i, \\\"expected ':' after key\\\")\\r\\n    end\\r\\n    i = next_char(str, i + 1, space_chars, true)\\r\\n    -- Read value\\r\\n    val, i = parse(str, i)\\r\\n    -- Set\\r\\n    res[key] = val\\r\\n    -- Next token\\r\\n    i = next_char(str, i, space_chars, true)\\r\\n    local chr = str:sub(i, i)\\r\\n    i = i + 1\\r\\n    if chr == \\\"}\\\" then break end\\r\\n    if chr ~= \\\",\\\" then decode_error(str, i, \\\"expected '}' or ','\\\") end\\r\\n  end\\r\\n  return res, i\\r\\nend\\r\\n\\r\\n\\r\\nlocal char_func_map = {\\r\\n  [ '\\\"' ] = parse_string,\\r\\n  [ \\\"0\\\" ] = parse_number,\\r\\n  [ \\\"1\\\" ] = parse_number,\\r\\n  [ \\\"2\\\" ] = parse_number,\\r\\n  [ \\\"3\\\" ] = parse_number,\\r\\n  [ \\\"4\\\" ] = parse_number,\\r\\n  [ \\\"5\\\" ] = parse_number,\\r\\n  [ \\\"6\\\" ] = parse_number,\\r\\n  [ \\\"7\\\" ] = parse_number,\\r\\n  [ \\\"8\\\" ] = parse_number,\\r\\n  [ \\\"9\\\" ] = parse_number,\\r\\n  [ \\\"-\\\" ] = parse_number,\\r\\n  [ \\\"t\\\" ] = parse_literal,\\r\\n  [ \\\"f\\\" ] = parse_literal,\\r\\n  [ \\\"n\\\" ] = parse_literal,\\r\\n  [ \\\"[\\\" ] = parse_array,\\r\\n  [ \\\"{\\\" ] = parse_object,\\r\\n}\\r\\n\\r\\n\\r\\nparse = function(str, idx)\\r\\n  local chr = str:sub(idx, idx)\\r\\n  local f = char_func_map[chr]\\r\\n  if f then\\r\\n    return f(str, idx)\\r\\n  end\\r\\n  decode_error(str, idx, \\\"unexpected character '\\\" .. chr .. \\\"'\\\")\\r\\nend\\r\\n\\r\\n\\r\\nfunction json.decode(str)\\r\\n  if type(str) ~= \\\"string\\\" then\\r\\n    error(\\\"expected argument of type string, got \\\" .. type(str))\\r\\n  end\\r\\n  local res, idx = parse(str, next_char(str, 1, space_chars, true))\\r\\n  idx = next_char(str, idx, space_chars, true)\\r\\n  if idx <= #str then\\r\\n    decode_error(str, idx, \\\"trailing garbage\\\")\\r\\n  end\\r\\n  return res\\r\\nend\\r\\n\\r\\n\\r\\nreturn json\\r\\n\")\r\n;\r\nend")

smalltalk:bind(smalltalk.Json["$klass"],"nativeClass",0
,function(self)
(function() if ((((((self['@nativeClass']))==nil)))==true) then return ((function()
return ((function()
	local _s8_value=((smalltalk.Lua):_at(((smalltalk.Smalltalk):current()):eval_((self):code())))
	self['@nativeClass']=_s8_value; return _s8_value
end)())
end))() else return nil end end)()
;
return (self['@nativeClass'])
;
end
,"private","nativeClass\r\n\t\" Private - Returns the wrapper for native binding of the receiver. \"\r\n\r\n\tnativeClass isNil ifTrue: [ nativeClass := Lua @ (Smalltalk current eval: self code) ].\r\n\t^nativeClass","function(self)\r\n(function() if ((((((self['@nativeClass']))==nil)))==true) then return ((function()\r\nreturn ((function()\r\n\tlocal _s8_value=((smalltalk.Lua):_at(((smalltalk.Smalltalk):current()):eval_((self):code())))\r\n\tself['@nativeClass']=_s8_value; return _s8_value\r\nend)())\r\nend))() else return nil end end)()\r\n;\r\nreturn (self['@nativeClass'])\r\n;\r\nend")

smalltalk:bind(smalltalk.Json["$klass"],"decode:",0
,function(self,string)
return ((smalltalk.Lua):_at((((self):nativeClass()):handle()).decode(string)))
;
end
,"wrappers","decode: string \r\n\t\" Delegated to native class. \"\r\n\r\n\t^Lua @ (self nativeClass handle #_decode: string )","function(self,string)\r\nreturn ((smalltalk.Lua):_at((((self):nativeClass()):handle()).decode(string)))\r\n;\r\nend")

smalltalk:bind(smalltalk.Json["$klass"],"encode:",0
,function(self,aLua)
return ((((self):nativeClass()):handle()).encode((aLua):json()))
;
end
,"wrappers","encode: aLua \r\n\t\" Delegated to native class. \"\r\n\r\n\t^self nativeClass handle #_encode: aLua json ","function(self,aLua)\r\nreturn ((((self):nativeClass()):handle()).encode((aLua):json()))\r\n;\r\nend")

smalltalk:bind(smalltalk.Json["$klass"],"parse:",0
,function(self,string)
return ((self):decode_(string))
;
end
,"converting","parse: string\r\n\t^self decode: string","function(self,string)\r\nreturn ((self):decode_(string))\r\n;\r\nend")

smalltalk:bind(smalltalk.Json["$klass"],"stringify:",0
,function(self,aLua)
return ((self):encode_(aLua))
;
end
,"converting","stringify: aLua\r\n\t^self encode: aLua","function(self,aLua)\r\nreturn ((self):encode_(aLua))\r\n;\r\nend")
