function print(text) { return console.log(text); }

/* S8 - A modern Smalltalk environment running on the "invisible machine".

Copyright (C) 2024 Alejandro F. Reimondo <aleReimondo@smalltalking.net>

Parts of source code written for Jtalk (Sept 2011),
Copyright (C) 2011 by Nicolas Petton <petton.nicolas@gmail.com>
Copyright (C) 2011 Jtalk contributors
	David Graham <dgraham@iphouse.com>
	Goran Krampe <goran@krampe.se>
Also code and ideas from Clamato (http://clamato.net), written by Avi Byrant.
The PetitParser library, published by Lukas Renggli (http://lukas-renggli.ch) and released under the MIT license.
And people that contributed to Smalltalk on diverse media years(decades) before this license holders claimed ownership.

Permission is hereby granted, free of charge, to any person obtaining
 a copy of this software and associated documentation files (the "Software"),
 to deal in the Software without restriction, including without limitation
 the rights to use, copy, modify, merge, publish, distribute, sublicense,
 and/or sell copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


The text you are reading here is the residue of something that has run before.
Keep the system running with responsibility. Smalltalk is not the words,
 nor any definition, nor it current contents...
Smalltalk include your feelings when you see it running.
Smalltalk is what happens when time flows and you "are" here.
Stop reading/writing code (thinking you are outside the system),
 change your world with responsibility, one object at a time, with your hands,
 during all your life, in family.
*/ 

var StObject = function StObject(){}
var StBehavior = function StBehavior(){}
var StClass = function StClass(){}
var StMetaclass = function StMetaclass(){
	this.$meta = true;
}
var StMethod = function StMethod(){}
var StUndefinedObject = function StUndefinedObject(){}
var nil = new StUndefinedObject()
nil.toString = function(){ return "nil" }
nil.isNil_ = function (x){
	switch(typeof x) {
		case "undefined": 
			return true
		case "string": case "boolean": 
		case "number": case "function": 
			return false
		
	}
	if (x===null || x===this) return true
	if (x===smalltalk) return false
	if (x.$klass && "function" === typeof x.isNil) return x.isNil()
	return false
}
nil.is_eqeq_ = function (a,b){
	if (a===b) return true
	if ((this.isNil_(a)) || (this.isNil_(b))) return false
	return (a.valueOf()) === (b.valueOf())
}
var StSystemDictionary = function StSystemDictionary(){
	this.$imageFileName = "s8.image.js"
	this.$nativeLanguage = "js"
	this.useManager = false
	this.isNil = function (x){
		return (x || nil) === nil
	}
	this.klass = function (spec){// Private
	
		var aClass
		if (spec.meta) {
			aClass = new StMetaclass()
		} else {
			var metaSpec = {meta:true}
			metaSpec.className = spec.className + " class"
			if (!this.isNil(spec.superclass)) {
				metaSpec.superclass = spec.superclass.$klass
			}
			var metaClass = this.klass(metaSpec)
			aClass = new metaClass.$fn()
		}
		this.setClass(aClass,spec)
		aClass.$fn.prototype.$klass = aClass
		aClass.$fn.prototype.$methods = {}
		if (aClass.$className == "Metaclass") {
			this.bindRoots(aClass)
		}
		return aClass
	}
	this.setProtoOf_to_ = function (nativeObject,proto){// Private
	
		nativeObject.__proto__ = proto
	}
	this.bindRoots = function (metaclass){// Private
	
		var objectClassPrototype  = this.Object.$klass.$fn.prototype
		this.setProtoOf_to_(objectClassPrototype,this.Class.$fn.prototype)
		this.setProtoOf_to_(this.Smalltalk.$klass.$fn.prototype,objectClassPrototype)
		this.setProtoOf_to_(this.Behavior.$klass.$fn.prototype,objectClassPrototype)
		this.Object.$klass.$superclass = this.Class
		metaclass.$superclass = this.Behavior
	}
	this.setVarNames = function (aClass,iVarNames,aValue){// Private
	
		aClass.$iVarNames = iVarNames
		var i = iVarNames.length
		while(i--){
			aClass.$fn.prototype["@"+iVarNames[i]] = aValue
		}
	}
	this.setClass = function (aClass,spec){// Private
	
		aClass.$className = spec.className
		if (!spec.meta) {
			aClass.$klass["@instanceClass"] = aClass
		}
		aClass.$category = spec.category || ""
		aClass.$superclass = spec.superclass || nil
		if (!this.isNil(aClass.$superclass)) {
			delete aClass.$superclass["@subclasses"]
			if (!this.isNil(aClass.$klass)) {
				if (!spec.meta) {
					aClass.$klass.$superclass = aClass.$superclass.$klass
				}
				if (aClass.$klass.$superclass) {
					delete aClass.$klass.$superclass["@subclasses"]
				}
				
			}
			
		}
		aClass.$fn = this.fnInSpec(spec)
		this.setVarNames(aClass,spec.iVarNames||[],nil)
		return aClass
	}
	this.fnInSpec = function (spec){// Private
	
		if (spec.meta && (this.isNil(spec.superclass))) {
			spec.superclass = this.Class
		}
		if (spec.fn) {
			if (!this.isNil(spec.superclass)) {
				this.setProtoOf_to_(spec.fn.prototype,spec.superclass.$fn.prototype)
			}
			return spec.fn
		}
		var $s8Constructor = function(){}
		if (!this.isNil(spec.superclass)) {
			$s8Constructor.prototype = new spec.superclass.$fn()
		}
		return $s8Constructor
	}
	this.classes = function (){
		var classes = []
		for(var i in this){
			if ((this.hasOwnProperty(i)) && i != "$klass") {
				var cls = this[i]
				if (!this.isNil(cls)) {
					if (cls.$klass && cls.$className) {
						classes.push(cls)
					}
				}
			}
			
		}
		return classes
	}
	this.subclasses = function (aClass){
		if (aClass.hasOwnProperty("@subclasses")) return aClass["@subclasses"]
		var classes = this.classes()
		var i = classes.length
		while(i--){
			classes[i]["@subclasses"] = []
			classes[i].$klass["@subclasses"] = []
			
		}
		i = classes.length
		while(i--){
			var cls = classes[i]
			if (!this.isNil(cls.$superclass)) {
				cls.$superclass["@subclasses"].push(cls)
				cls.$superclass.$klass["@subclasses"].push(cls.$klass)
			}
		}
		if (this.Class) {
			this.Class["@subclasses"] = []
			this.$klass["@subclasses"] = []
		}
		return aClass["@subclasses"]
	}
	this.allSubclassesDo = function (aClass,aBlock){
		var subclasses = this.subclasses(aClass)
		var i = subclasses.length
		while(i--){
			var cls = subclasses[i]
			aBlock(cls)
			this.allSubclassesDo(cls,aBlock)
		}
		return this
	}
	this.mapClassName = function (className,category,fn,superclass){
		var spec = {
			className: className,
			category: category,
			superclass: superclass,
			fn: fn
		}
		if (this[className]) return this.setClass(this[className],spec)
		this[className] = this.klass(spec)
		return this[className]
	}
	this.sameIVContents = function (array1,array2){
		if (array1 == array2) return true
		if (array1.length === array2.length) {
			return array1.every(function (element,index){
				return element == array2[index]
			})
		}
		return false
	}
	this.addClass = function (className,superclass,iVarNames,category){
		this.debugAddClass(className,superclass,iVarNames,category)
		if (this[className]) {
			var aClass = this[className]
			var sameShape = (aClass.$superclass == superclass) && (this.sameIVContents(aClass.$iVarNames,iVarNames))
			if (sameShape && (aClass.$category == category)) return aClass
			this.aboutToChangeClass(aClass,superclass,iVarNames,category)
			if (aClass.$superclass) {
				delete aClass.$superclass["@subclasses"]
				var aMeta = aClass.$klass
				if (aMeta && superclass) {
					delete aMeta.$superclass["@subclasses"]
					aMeta.$superclass = superclass.$klass
				}
			}
			aClass.$superclass = superclass
			if (superclass && superclass["@subclasses"]) {
				if (this.isNil(superclass["@subclasses"])) {
					superclass["@subclasses"] = []
				}
				superclass["@subclasses"].push(aClass)
			}
			this.setVarNames(aClass,iVarNames||[],nil)
			aClass.$category = category || aClass.$category
			if (!sameShape) {
				this.classShapeChanged(aClass)
			}
		} else {
			this.aboutToCreateClass(className,superclass,iVarNames,category)
			this[className] = this.klass({
				className: className,
				iVarNames: iVarNames,
				superclass: superclass
			})
			this[className].$category = category || ""
			
		}
		this.classChanged(this[className])
		return this[className]
	}
	this.renameClass = function (aClass,aString){
		delete this[aClass.$className]
		aClass.$className = aString
		this[aString] = aClass
		this.classChanged(aClass)
	}
	this.removeClass = function (aClass){
		var deleted = "Deleted-"
		if (aClass.$superclass) {
			delete aClass.$superclass["@subclasses"]
		}
		delete this[aClass.$className]
		aClass.$className = deleted + aClass.$className
		aClass.$klass.$className = deleted + aClass.$klass.$className
		this.classChanged(aClass)
	}
	this.metaVarNames = function (aMetaClass,iVarNames){
		if (this.sameIVContents(aMetaClass.$iVarNames,iVarNames)) return false
		this.setVarNames(aMetaClass,iVarNames||[],nil)
		this.classShapeChanged(aMetaClass)
		this.classChanged(aMetaClass)
		return true
	}
	this.smDo = function (block){
		if (!this.useManager) return nil
		if (!this.SystemManager) return nil
		if (!this.SystemManager.currentOrNil) return nil
		var sm = this.SystemManager.currentOrNil()
		if (!this.isNil(sm)) return block(sm)
	}
	this.classChanged = function (aBehavior){
		this.smDo(function (sm){
			sm.classChanged_(aBehavior)
		})
	}
	this.classShapeChanged = function (aBehavior){
		this.smDo(function (sm){
			sm.classShapeChanged_(aBehavior)
		})
	}
	this.aboutToCreateClass = function (className,superclass,iVarNames,category){
		this.smDo(function (sm){
			sm.aboutToCreateClass_superclass_iVarNames_category_(className,superclass,iVarNames,category)
		})
	}
	this.aboutToChangeClass = function (aClass,superclass,iVarNames,category){
		this.smDo(function (sm){
			sm.aboutToChange_superclass_iVarNames_category_(aClass,superclass,iVarNames,category)
		})
	}
	this.methodBoundTo = function (method,klass){
		this.smDo(function (sm){
			sm.methodBound_to_(method,klass)
		})
		return method
	}
	this.aboutToBindTo = function (method,klass){
		this.smDo(function (sm){
			sm.aboutToBind_to_(method,klass)
		})
		return method
	}
	this.aboutToRemoveFrom = function (method,klass){
		this.smDo(function (sm){
			sm.aboutToRemove_from_(method,klass)
		})
		return method
	}
	this.methodRemovedFrom = function (method,klass){
		this.smDo(function (sm){
			sm.methodRemoved_from_(method,klass)
		})
		return method
	}
	this.objectNotFound = function (object,indexOrKey,collection){
		this.smDo(function (sm){
			sm.object_at_notFoundIn_(object,indexOrKey,collection)
		})
		return collection
	}
	this.whoImplements = function (klass,jsSelector){
		var cls = klass
		do {
			if (cls.$fn.prototype.hasOwnProperty(jsSelector)) return cls
			cls = cls.$superclass
		} while (cls)
		return nil
	}
	this.superImplementor = function (klass,jsSelector){
		if ((this.isNil(klass.$superclass)) || (this.isNil(klass.$superclass.$fn))) return klass.missingImplementorFor_(jsSelector)
		var cls = this.whoImplements(klass.$superclass,jsSelector)
		if (this.isNil(cls)) return klass.$superclass.missingImplementorFor_(jsSelector)
		return cls.$fn.prototype[jsSelector]
	}
	this.addMethod = function (jsSelector,method,klass){
		this.aboutToBindTo(method,klass)
		klass.$fn.prototype[jsSelector] = method.$fn
		klass.$fn.prototype.$methods[method.$selector] = method
		method["@classField"] = klass
		return this.methodBoundTo(method,klass)
	}
	this.bind = function (klass,stSelector,jsSelector,jsFunction,category,stSource,stamp){
		this.debugBind(klass,stSelector,jsSelector,jsFunction,category,stSource,stamp)
		var method = new StMethod()
		
		method.$selector = stSelector
		method.$fn = jsFunction
		method.$category = category
		method.$source = stSource
		if (!this.isNil(stamp)) {
			method.$stamp = stamp
		}
		method.$generation = this.$state
		if (!this.isNil(klass)) {
			if (this.isNil(jsSelector)) {
				jsSelector = stSelector.replace(/\:/g,"_")
			}
			this.addMethod(jsSelector,method,klass)
		}
		return method
	}
	this.removeMethod = function (aClass,aMethod){
		this.aboutToRemoveFrom(aMethod,aClass)
		delete aClass.$fn.prototype[aMethod.$selector.asSelector()]
		delete aClass.$fn.prototype.$methods[aMethod.$selector]
		return this.methodRemovedFrom(aMethod,aClass)
	}
	this.newMethod = function (currentClass,selector,fn,source){
		return this.bind(nil,selector,nil,fn,nil,source,nil)
	}
	this.readJSObject = function (js){
		if (!js) return nil
		var object = js
		var readObject = js.constructor === Object
		var readArray = js.constructor === Array
		if (readObject) {
			object = this.PoolDictionary.$$new()
		}
		for(var i in js){
			if (!js.hasOwnProperty(i)) break
			if (readObject) {
				object.at_put_(i,this.readJSObject(js[i]))
			}
			if (readArray) {
				object[i] = this.readJSObject(js[i])
			}
		}
		return object
	}
	this.globalsDo = function (action){
		for(key in this){
			if (!this.hasOwnProperty(key)) break
			var aValue = this[key]
			if (typeof(aValue) != "function" && (aValue.isClass && (!aValue.isClass()))) {
				action(key)
			}
			
		}
		return this
	}
	this.startup = function (){
		this.$state = "booting"
		this.OverwriteMethods = this.OverwriteMethods || true
		this.DumpOverwritenMethods = this.DumpOverwritenMethods || false
		this.mapClassName("Object","Kernel",StObject)
		this.mapClassName("Smalltalk","Kernel",StSystemDictionary,this.Object)
		this.mapClassName("Behavior","Kernel",StBehavior,this.Object)
		this.mapClassName("Class","Kernel",StClass,this.Behavior)
		this.mapClassName("Metaclass","Kernel",StMetaclass,this.Behavior)
		this.mapClassName("CompiledMethod","Kernel",StMethod,this.Object)
		this.mapClassName("Number","Kernel",Number,this.Object)
		this.mapClassName("BlockClosure","Kernel",Function,this.Object)
		this.mapClassName("Boolean","Kernel",Boolean,this.Object)
		this.mapClassName("DateTime","Kernel",Date,this.Object)
		this.mapClassName("UndefinedObject","Kernel",StUndefinedObject,this.Object)
		this.mapClassName("Collection","Kernel",undefined,this.Object)
		this.mapClassName("SequenceableCollection","Kernel",undefined,this.Collection)
		this.mapClassName("String","Kernel",String,this.SequenceableCollection)
		this.mapClassName("Array","Kernel",Array,this.SequenceableCollection)
		this.mapClassName("RegularExpression","Kernel",RegExp,this.Object)
		this.$state = "startup"
		return this
	}
	this.launchImage = function (imageName){
		this.useManager = true
		if (!this.isNil(imageName)) {
			this.$imageFileName = imageName
			load(imageName)
		}
		this.$state = "launching"
		if (this.launch && typeof this.launch == "function") {
			this.launch()
		}
		this.$state = "running"
		return this
	}
	this.is_kindOf_ = function (anObject,aClass){
		if ((nil.isNil_(anObject)) && (nil !== anObject)) return false
		if ("undefined" === typeof anObject.isKindOf_) return false
		return anObject.isKindOf_(aClass)
	}
	this.responds_to_ = function (anObject,aSelector){
		if ((nil.isNil_(anObject)) && (nil !== anObject)) return false
		if ("undefined" === typeof anObject.respondsTo_) return false
		return anObject.respondsTo_(aSelector)
	}
	this.debugBind = function (klass,stSelector,jsSelector,jsFunction,category,stSource,stamp){
		return this
	}
	this.debugAddClass = function (className,superclass,iVarNames,category){
		return this
	}
	this.initWithSubclasses = function (aClass){
		return aClass
	}
	
}
var smalltalk = new StSystemDictionary()
smalltalk.toString = function() { return "smalltalk" }
smalltalk.startup();

smalltalk.$state = "js-toolset"
smalltalk.st = function(code) { return code.evaluate(); }
smalltalk.doIt = function(code) { return code.doIt() }
var fileIn = function(fileName) { return read(fileName).fileIn(); }
var fileInJS = function(fileName) { return eval(read(fileName)); }
var doit = function(what) { if (what) { return smalltalk.st(what); } else { return vse("workspace"); } }
var exportCategory = function(aString) { return smalltalk.Exporter.exportCategory_(aString); }
var saveImage = function() { return smalltalk.saveImage(); }
var saveImageAs = function(fileName) { return smalltalk.saveImageAs_(fileName); }
smalltalk.$state = "initialized"

if(StObject && !smalltalk.Object) smalltalk.mapClassName("Object","Kernel",StObject,undefined);
smalltalk.addClass("Object",undefined,[],"Kernel");
if(StSystemDictionary && !smalltalk.Smalltalk) smalltalk.mapClassName("Smalltalk","Kernel",StSystemDictionary,smalltalk.Object);
smalltalk.addClass("Smalltalk",smalltalk.Object,[],"Kernel");
if(StBehavior && !smalltalk.Behavior) smalltalk.mapClassName("Behavior","Kernel",StBehavior,smalltalk.Object);
smalltalk.addClass("Behavior",smalltalk.Object,[],"Kernel");
if(StClass && !smalltalk.Class) smalltalk.mapClassName("Class","Kernel",StClass,smalltalk.Behavior);
smalltalk.addClass("Class",smalltalk.Behavior,[],"Kernel");
if(StMetaclass && !smalltalk.Metaclass) smalltalk.mapClassName("Metaclass","Kernel",StMetaclass,smalltalk.Behavior);
smalltalk.addClass("Metaclass",smalltalk.Behavior,["instanceClass"],"Kernel");
if(StMethod && !smalltalk.CompiledMethod) smalltalk.mapClassName("CompiledMethod","Kernel",StMethod,smalltalk.Object);
smalltalk.addClass("CompiledMethod",smalltalk.Object,["classField"],"Kernel");
if(Number && !smalltalk.Number) smalltalk.mapClassName("Number","Kernel",Number,smalltalk.Object);
smalltalk.addClass("Number",smalltalk.Object,[],"Kernel");
if(Function && !smalltalk.BlockClosure) smalltalk.mapClassName("BlockClosure","Kernel",Function,smalltalk.Object);
smalltalk.addClass("BlockClosure",smalltalk.Object,[],"Kernel");
if(Boolean && !smalltalk.Boolean) smalltalk.mapClassName("Boolean","Kernel",Boolean,smalltalk.Object);
smalltalk.addClass("Boolean",smalltalk.Object,[],"Kernel");
if(Date && !smalltalk.DateTime) smalltalk.mapClassName("DateTime","Kernel",Date,smalltalk.Object);
smalltalk.addClass("DateTime",smalltalk.Object,[],"Kernel");
if(StUndefinedObject && !smalltalk.UndefinedObject) smalltalk.mapClassName("UndefinedObject","Kernel",StUndefinedObject,smalltalk.Object);
smalltalk.addClass("UndefinedObject",smalltalk.Object,[],"Kernel");
smalltalk.addClass("Collection",smalltalk.Object,[],"Kernel");
smalltalk.addClass("SequenceableCollection",smalltalk.Collection,[],"Kernel");
if(String && !smalltalk.String) smalltalk.mapClassName("String","Kernel",String,smalltalk.SequenceableCollection);
smalltalk.addClass("String",smalltalk.SequenceableCollection,[],"Kernel");
if(Array && !smalltalk.Array) smalltalk.mapClassName("Array","Kernel",Array,smalltalk.SequenceableCollection);
smalltalk.addClass("Array",smalltalk.SequenceableCollection,[],"Kernel");
smalltalk.addClass("ArrayedContents",smalltalk.SequenceableCollection,["contents"],"Kernel");
smalltalk.addClass("ComposedAssociations",smalltalk.ArrayedContents,[],"Kernel");
smalltalk.addClass("ComposedObjects",smalltalk.ArrayedContents,["species"],"Kernel");
smalltalk.addClass("Interval",smalltalk.Collection,["start","end","step","value"],"Kernel");
smalltalk.addClass("Set",smalltalk.Collection,["contents"],"Kernel");
smalltalk.addClass("HashedSet",smalltalk.Set,["hash"],"Kernel");
smalltalk.addClass("KeyedCollection",smalltalk.Collection,[],"Kernel");
smalltalk.addClass("Dictionary",smalltalk.KeyedCollection,["keys","values"],"Kernel");
smalltalk.addClass("PoolDictionary",smalltalk.KeyedCollection,["hash","size"],"Kernel");
if(RegExp && !smalltalk.RegularExpression) smalltalk.mapClassName("RegularExpression","Kernel",RegExp,smalltalk.Object);
smalltalk.addClass("RegularExpression",smalltalk.Object,[],"Kernel");
smalltalk.addClass("Exception",smalltalk.Object,["messageText"],"Kernel");
smalltalk.addClass("Error",smalltalk.Exception,[],"Kernel");
smalltalk.addClass("MessageNotUnderstood",smalltalk.Error,["message","receiver"],"Kernel");
smalltalk.addClass("Association",smalltalk.Object,["key","value"],"Kernel");
smalltalk.addClass("ClassBuilder",smalltalk.Object,[],"Kernel");
smalltalk.addClass("Stream",smalltalk.Object,["collection","position","readLimit"],"Kernel");
smalltalk.addClass("StringStream",smalltalk.Stream,[],"Kernel");
smalltalk.addClass("StringWriteStream",smalltalk.Stream,[],"Kernel");
smalltalk.addClass("Random",smalltalk.Object,[],"Kernel");
smalltalk.addClass("Point",smalltalk.Object,["x","y"],"Kernel");
smalltalk.addClass("Rectangle",smalltalk.Object,["leftTop","rightBottom"],"Kernel");
smalltalk.addClass("EvaluableAction",smalltalk.Object,[],"Kernel");
smalltalk.EvaluableAction.$comment=unescape("%0D%0A%09Action%20model%2C%20evaluable%20and%20composable%20actions%20are%20implemented%20to%20support%20evaluations.%0D%0A");
smalltalk.addClass("Message",smalltalk.EvaluableAction,["selector","arguments","receiver"],"Kernel");
smalltalk.addClass("LinkMessage",smalltalk.Message,["resultAction"],"Kernel");
smalltalk.LinkMessage.$comment=unescape("%0D%0A%09Implement%20evaluation%20cascading%20argument%20evaluations.");
smalltalk.addClass("ActionSequence",smalltalk.Object,["contents"],"Kernel");
smalltalk.addClass("EventManager",smalltalk.Object,["handlers"],"Kernel");
smalltalk.EventManager.$comment=unescape("%0D%0A%09Event%20system%2C%20properties%20and%20change/update/release%20mechanism.%0D%0A%09We%20hook%20%24properties%20and%20%24handlers%20internal%20properties%20to%20manage%20properties%20and%20events%20at%20Object%20level.%0D%0A%09EventManager%20instances%20implement%20events%20handler%20as%20pure%20instance%20variable.%0D%0A");
smalltalk.EventManager.$klass.$iVarNames=["eventsTriggered"]
smalltalk.addClass("S8Encoder",smalltalk.EventManager,["compiler"],"Compiler-Encoding");
smalltalk.addClass("OptimizingEncoder",smalltalk.S8Encoder,["optimizationMap","knownOptimizations","inlines"],"Compiler-Encoding");
smalltalk.OptimizingEncoder.$klass.$iVarNames=["inlines"]
smalltalk.addClass("NativeEncoder",smalltalk.OptimizingEncoder,["stream","indentation","startOffset","nestedBlocks","earlyReturn","functionName"],"Compiler-Encoding");
smalltalk.addClass("JSEncoder",smalltalk.NativeEncoder,[],"JS");
smalltalk.addClass("LuaEncoder",smalltalk.NativeEncoder,[],"Lua");
smalltalk.addClass("Compiler",smalltalk.EventManager,["currentClass","selector","encoder","unknownVariables","tempVariables","transformers","useTemplates"],"Compiler");
smalltalk.addClass("SourceCodeParser",smalltalk.EventManager,[],"Compiler-Parsing");
smalltalk.addClass("SmalltalkParser",smalltalk.SourceCodeParser,[],"Compiler-Parsing");
smalltalk.addClass("Exporter",smalltalk.EventManager,["encoder"],"Tools-fileOut");
smalltalk.addClass("MethodReferences",smalltalk.EventManager,["rootClasses","results"],"Tools-references");
smalltalk.addClass("SystemManager",smalltalk.EventManager,[],"Snapshot");
smalltalk.SystemManager.$classVariableNames="Current";
smalltalk.addClass("CompilerNode",smalltalk.Object,["nodes","start","end","annotations","client"],"Compiler-Nodes");
smalltalk.addClass("MethodNode",smalltalk.CompilerNode,["selector","arguments","source"],"Compiler-Nodes");
smalltalk.addClass("SendNode",smalltalk.CompilerNode,["selector","arguments","receiver","action"],"Compiler-Nodes");
smalltalk.addClass("ScopedNode",smalltalk.CompilerNode,[],"Compiler-Nodes");
smalltalk.addClass("CascadeNode",smalltalk.ScopedNode,["receiver"],"Compiler-Nodes");
smalltalk.addClass("BlockNode",smalltalk.ScopedNode,["parameters"],"Compiler-Nodes");
smalltalk.addClass("SequenceNode",smalltalk.ScopedNode,["temps"],"Compiler-Nodes");
smalltalk.addClass("BlockSequenceNode",smalltalk.SequenceNode,[],"Compiler-Nodes");
smalltalk.addClass("ExtendedSequenceNode",smalltalk.SequenceNode,["directives"],"Compiler-Nodes");
smalltalk.addClass("AssignmentNode",smalltalk.CompilerNode,["left","right"],"Compiler-Nodes");
smalltalk.addClass("ReturnNode",smalltalk.CompilerNode,["exitBlock"],"Compiler-Nodes");
smalltalk.addClass("ValueNode",smalltalk.CompilerNode,["value","binding"],"Compiler-Nodes");
smalltalk.addClass("VariableNode",smalltalk.ValueNode,["indices"],"Compiler-Nodes");
smalltalk.addClass("GlobalReferenceNode",smalltalk.VariableNode,[],"Compiler-Nodes");
smalltalk.addClass("NoReceiverNode",smalltalk.VariableNode,[],"Compiler-Nodes");
smalltalk.addClass("AlienNode",smalltalk.VariableNode,[],"Compiler-Nodes");
smalltalk.addClass("InlineNode",smalltalk.CompilerNode,["sources"],"Compiler-Nodes");
smalltalk.addClass("DirectiveNode",smalltalk.CompilerNode,[],"Compiler-Nodes");
smalltalk.addClass("JSDirectiveNode",smalltalk.DirectiveNode,["code","protected"],"Compiler-Nodes");
smalltalk.addClass("InlineOptimizationNode",smalltalk.DirectiveNode,["args","remove"],"Compiler-Nodes");
smalltalk.addClass("PPFailure",smalltalk.Object,["position","reason"],"Compiler-Parsing");
smalltalk.addClass("PPParser",smalltalk.Object,["memo"],"Compiler-Parsing");
smalltalk.addClass("PPEOFParser",smalltalk.PPParser,[],"Compiler-Parsing");
smalltalk.addClass("PPAnyParser",smalltalk.PPParser,[],"Compiler-Parsing");
smalltalk.addClass("PPEpsilonParser",smalltalk.PPParser,[],"Compiler-Parsing");
smalltalk.addClass("PPStringParser",smalltalk.PPParser,["string"],"Compiler-Parsing");
smalltalk.addClass("PPStStringParser",smalltalk.PPParser,[],"Compiler-Parsing");
smalltalk.addClass("PPCharacterParser",smalltalk.PPParser,["regexp","theString"],"Compiler-Parsing");
smalltalk.PPCharacterParser.$classVariableNames="RegExpCache";
smalltalk.addClass("PPListParser",smalltalk.PPParser,["parsers"],"Compiler-Parsing");
smalltalk.addClass("PPSequenceParser",smalltalk.PPListParser,[],"Compiler-Parsing");
smalltalk.addClass("PPChoiceParser",smalltalk.PPListParser,[],"Compiler-Parsing");
smalltalk.addClass("PPDelegateParser",smalltalk.PPParser,["parser"],"Compiler-Parsing");
smalltalk.addClass("PPAndParser",smalltalk.PPDelegateParser,[],"Compiler-Parsing");
smalltalk.addClass("PPNotParser",smalltalk.PPAndParser,[],"Compiler-Parsing");
smalltalk.addClass("PPActionParser",smalltalk.PPDelegateParser,["block"],"Compiler-Parsing");
smalltalk.addClass("PPFlattenParser",smalltalk.PPDelegateParser,[],"Compiler-Parsing");
smalltalk.addClass("PPSourceParser",smalltalk.PPDelegateParser,[],"Compiler-Parsing");
smalltalk.addClass("PPRepeatingParser",smalltalk.PPDelegateParser,["min"],"Compiler-Parsing");
smalltalk.addClass("PPInlineParser",smalltalk.PPParser,["terminator"],"Compiler-Parsing");
smalltalk.addClass("FileInReader",smalltalk.Object,[],"Tools-fileIn");
smalltalk.FileInReader.$comment=unescape("%0D%0A%09Perform%20sequential%20reading%20from%20aStream%20in%20chunk%20format.%0D%0A%09Instances%20implement%20%23scanFrom%3A%20message%2C%20advancing%20aStream%20upto%20next%20reader%20instantiation%20chunk.");
smalltalk.addClass("ChunkReader",smalltalk.FileInReader,["actionBlock","terminationBlock"],"Tools-fileIn");
smalltalk.ChunkReader.$comment=unescape("%0D%0A%09Pluggable%20FileInReader.%0D%0A%09actionBlock%20is%20evaluated%20with%20input%20stream%20as%20argument.%0D%0A%09Read%20chunks%20until%20empty%20chunk%2C%20terminationBlock%20is%20nil%20or%20evaluates%20to%20true.");
smalltalk.addClass("ClassCategoryReader",smalltalk.FileInReader,["class","category","stamp"],"Tools-fileIn");
smalltalk.addClass("ClassCommentReader",smalltalk.FileInReader,["class"],"Tools-fileIn");
smalltalk.addClass("SkipFileInReader",smalltalk.FileInReader,["limit"],"Tools-fileIn");
smalltalk.addClass("AssociationsReader",smalltalk.FileInReader,[],"Tools-fileIn");
smalltalk.addClass("PoolDictionaryReader",smalltalk.AssociationsReader,["scope"],"Tools-fileIn");
smalltalk.addClass("NativeReader",smalltalk.AssociationsReader,["object"],"Tools-fileIn");
smalltalk.addClass("APIBuilder",smalltalk.Object,["parents","token","bound"],"Builders");
smalltalk.APIBuilder.$klass.$iVarNames=["default","saveBuilders"]
smalltalk.addClass("Snapshot",smalltalk.Object,[],"Snapshot");
smalltalk.Snapshot.$comment=unescape("%0D%0A%09Dump%20a%20binary%20representation%20of%20this%20S8%20system%20onto%20an%20image%20%28String%29.%0D%0A%09The%20default%20implementation%20export%20full/runnable%20images%20containing%20boot%20sections.%0D%0A%09The%20subclasses%20can%20refine%20behavior%20to%20select/customize%20exports%20and%20format.%0D%0A%09@2011-today%20Alejandro%20Reimondo%20aleReimondo@smalltalking.net");
smalltalk.addClass("Transcript",smalltalk.Object,["streams"],"Transcript");
smalltalk.Transcript.$classVariableNames="Current";
smalltalk.Transcript.$comment=unescape("%0D%0A%09Implement%20Transcript%20streamming.%0D%0A%09This%20implementation%20emmit%20output%20delegating%20onto%20stream%20instances.%0D%0A%09The%20streams%20must%20understand%20%23cr%20and%20%23nextPutAll%3A%0D%0A");
smalltalk.addClass("NativeObject",smalltalk.Object,["handle"],"Natives");
smalltalk.NativeObject.$comment=unescape("%0D%0A%09Wrapper%20to%20native%20object.%0D%0A%09@2011%20Alejandro%20F.%20Reimondo%0D%0A%09aleReimondo@smalltalking.net%0D%0A");
smalltalk.addClass("NativeHandle",smalltalk.NativeObject,[],"Natives");
smalltalk.NativeHandle.$classVariableNames="Support";
smalltalk.NativeHandle.$comment=unescape("%0D%0A%09Wrapper%20for%20native%20objects.%0D%0A%09@2018%20Alejandro%20F.%20Reimondo%0D%0A%09aleReimondo@smalltalking.net%0D%0A");
smalltalk.addClass("JS",smalltalk.NativeHandle,[],"JS");
smalltalk.JS.$comment=unescape("%0D%0A%09Implementation%20of%20wrappers%20to%20native%20Javascript%20objects.%0D%0A%09@2011%20Alejandro%20F.%20Reimondo%0D%0A%09aleReimondo@smalltalking.net");
smalltalk.addClass("Lua",smalltalk.NativeHandle,[],"Lua");
smalltalk.Lua.$comment=unescape("%0D%0A%09Implementation%20of%20wrappers%20to%20native%20Lua%20objects.%0D%0A%09@2011%20Alejandro%20F.%20Reimondo%0D%0A%09aleReimondo@smalltalking.net");
smalltalk.addClass("NativeLibraryObject",smalltalk.NativeObject,[],"JS");
smalltalk.NativeLibraryObject.$classVariableNames="BindingListeners";
smalltalk.NativeLibraryObject.$comment=unescape("%0D%0A%09Abstract%20wrapper%20to%20javascript%20external%20library.%0D%0A%09@2012%20Alejandro%20Reimondo%20-%20aleReimondo@smalltalking.net%0D%0A");

smalltalk.bind(smalltalk.Object.$klass,"eventsTriggered",0
,function(){var self=this;
return self.constructEventsTriggered();}
,"events",unescape("eventsTriggered%0D%0A%09%22%20Return%20all%20the%20events%20managed%20by%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20constructEventsTriggered"));
smalltalk.bind(smalltalk.Object.$klass,"constructEventsTriggered",0
,function(){var self=this;
return ["changed"];}
,"events",unescape("constructEventsTriggered%0D%0A%09%22%20Private%20-%20Return%20the%20events%20managed%20by%20instances%20of%20the%20receiver.%0D%0A%09The%20default%20implementation%20return%20support%20for%20MVC%20change/update%20mechanism.%0D%0A%09This%20method%20can%20be%20refined%20by%20the%20subclasses%20to%20extend%20event%20support.%0D%0A%09%22%0D%0A%0D%0A%09%5E%23%28%20%23changed%20%29"));
smalltalk.bind(smalltalk.Object.$klass,"canTriggerEvent:",0
,function(selector){var self=this;
return self.eventsTriggered().includes_(selector);}
,"events",unescape("canTriggerEvent%3A%20selector%0D%0A%09%22%20Return%20true%20if%20instances%20of%20the%20receiver%20can%20trigger%20event.%22%0D%0A%0D%0A%09%5Eself%20eventsTriggered%20includes%3A%20selector%20"));
smalltalk.bind(smalltalk.Object,"print:",0
,function(aString){var self=this;
var aBlock=nil;
aBlock=((function(){var $1$=(smalltalk.Object["printBlock"]);if(nil.isNil_($1$))return nil;return $1$;})());
if ((nil.isNil_(aBlock))){
print(aString);} else{aBlock.evaluateWith_(aString);};;
return self;}
,"s8-initialize-printing",unescape("print%3A%20aString%0D%0A%09%22%20Output%20a%20new%20line%20with%20aString%20onto%20output%20device.%20%22%0D%0A%0D%0A%09%7C%20aBlock%20%7C%0D%0A%09aBlock%20%3A%3D%20Object%20basicAt%3A%20%23printBlock.%0D%0A%09aBlock%20isNil%20ifTrue%3A%20%5B%0D%0A%09%09%7B%27self%27%7D.%20%23%20%23print%3A%20aString%0D%0A%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09aBlock%20evaluateWith%3A%20aString%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Object,"fileIn:",0
,function(fileName){var self=this;
self.note_(unescape("fileIn%28%22")._comma(fileName)._comma(unescape("%22%29")));

return fileIn(fileName);}
,"s8-initialize-fileIn",unescape("fileIn%3A%20fileName%0D%0A%09%22%20FileIn%20smalltalk%20source%20at%20fileName.%20%22%0D%0A%0D%0A%09self%20note%3A%20%27fileIn%28%22%27%2C%20fileName%2C%20%27%22%29%27.%0D%0A%09%7B%27self%27%7D.%20%5E%23%20%23fileIn%3A%20fileName"));
smalltalk.bind(smalltalk.Object,"fileInJS:",0
,function(fileName){var self=this;
self.note_(unescape("fileInJS%28%22")._comma(fileName)._comma(unescape("%22%29")));

return fileInJS(fileName);}
,"s8-initialize-fileIn",unescape("fileInJS%3A%20fileName%0D%0A%09%22%20Load%20native%20code%20from%20file%20at%20fileName.%20%22%0D%0A%0D%0A%09self%20note%3A%20%27fileInJS%28%22%27%2C%20fileName%2C%20%27%22%29%27.%0D%0A%09%7B%27self%27%7D.%20%5E%23%20%23fileInJS%3A%20fileName"));
smalltalk.bind(smalltalk.Object,"smalltalk",0
,function(){var self=this;
return ((function(){var $1$=(self["$smalltalk"]);if(nil.isNil_($1$))return ((function(){return smalltalk.Smalltalk.current();}).value());return $1$;})());}
,"system",unescape("smalltalk%0D%0A%09%22%20Private%20-%20Return%20the%20hosting%20context%20of%20the%20receiver.%0D%0A%09The%20smalltalk%20host%20is%20used%20for%20evaluations%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24smalltalk%27%20ifAbsent%3A%20%5B%20Smalltalk%20current%20%5D"));
smalltalk.bind(smalltalk.Object,"nativeLanguage",0
,function(){var self=this;
return ((function(){var $1$=(self.smalltalk()["$nativeLanguage"]);if(nil.isNil_($1$))return ((function(){return "js";}).value());return $1$;})());}
,"environment",unescape("nativeLanguage%0D%0A%09%22%20Private%20-%20Return%20the%20native%20language%20of%20the%20host.%20%22%0D%0A%0D%0A%09%5Eself%20smalltalk%0D%0A%09%09basicAt%3A%20%27%24nativeLanguage%27%0D%0A%09%09ifAbsent%3A%20%5B%20%23js%20%5D"));
smalltalk.bind(smalltalk.Object,"halt",0
,function(){var self=this;
debugger;
return self;}
,"debug",unescape("halt%0D%0A%09%22%20Private%20-%20Halt%20execution%20entering%20debugger.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27Enter%20lowlevel%20debugger%20%28or%20do%20nothing%29%27%7D"));
smalltalk.bind(smalltalk.Object,"note:",0
,function(aString){var self=this;
self.print_(aString.asString().asComment());
return self;}
,"development",unescape("note%3A%20aString%0D%0A%09%22%20Print%20a%20note%20%28aString%29%20to%20the%20user.%20%22%0D%0A%0D%0A%09self%20print%3A%20aString%20asString%20asComment"));
smalltalk.bind(smalltalk.Object,"notImplementedYet",0
,function(){var self=this;
self.error_("Not implemented. Yet".halt());
return self;}
,"development",unescape("notImplementedYet%0D%0A%09%22%20Private%20-%20Halt%20execution%20and/or%20generate%20an%20error%20reporting%20implementation%20incomplete.%20%22%0D%0A%0D%0A%09self%20error%3A%20%27Not%20implemented.%20Yet%27%20halt"));
smalltalk.bind(smalltalk.Object,"notImplementedYet:",0
,function(aSelector){var self=this;
self.error_(aSelector.asLiteral()._comma(" Not implemented. Yet").halt());
return self;}
,"development",unescape("notImplementedYet%3A%20aSelector%0D%0A%09%22%20Private%20-%20Halt%20execution%20and/or%20generate%20an%20error%20reporting%20implementation%20incomplete.%20%22%0D%0A%0D%0A%09self%20error%3A%20%28aSelector%20asLiteral%20%2C%27%20Not%20implemented.%20Yet%27%29%20halt"));
smalltalk.bind(smalltalk.Object,"=","_eq"
,function(anObject){var self=this;
return (nil.is_eqeq_(self,anObject));}
,"comparing",unescape("%3D%20anObject%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%5Eself%20%3D%3D%20anObject"));
smalltalk.bind(smalltalk.Object,"~=","_no_eq"
,function(anObject){var self=this;
return (nil.is_eqeq_(self._eq(anObject),false));}
,"comparing",unescape("%7E%3D%20anObject%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20not%20equal%20to%20anObject.%20%22%0D%0A%0D%0A%09%5E%28self%20%3D%20anObject%29%20%3D%3D%20false"));
smalltalk.bind(smalltalk.Object,"initialize",0
,function(){var self=this;
return self;}
,"initialize",unescape("initialize%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%0D%0A%09The%20default%20implementation%20do%20nothing.%0D%0A%09This%20method%20can%20be%20refined%20by%20subclasses%20to%20support%20creation%20time%20initialization%20of%20collaborators.%0D%0A%09%22"));
smalltalk.bind(smalltalk.Object,"yourself",0
,function(){var self=this;
return self;}
,"identity",unescape("yourself%0D%0A%09%22%20Return%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Object,"species",0
,function(){var self=this;
return self.$klass;}
,"species",unescape("species%0D%0A%09%22%20Return%20the%20species%20to%20instantiate%20objects%20like%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20class%20"));
smalltalk.bind(smalltalk.Object,"class","$$class"
,function(){var self=this;
return ((function(){var $1$=(self["$klass"]);if(nil.isNil_($1$))return ((function(){return self.klass();}).value());return $1$;})());}
,"accessing",unescape("class%0D%0A%09%22%20Return%20the%20class%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24klass%27%20ifAbsent%3A%20%5B%20self%20klass%20%5D"));
smalltalk.bind(smalltalk.Object,"size",0
,function(){var self=this;
return (0);}
,"accessing",unescape("size%0D%0A%09%22%20Return%20the%20size%20of%20the%20receiver.%0D%0A%09The%20default%20implementation%20return%20zero.%0D%0A%09This%20method%20can%20be%20refined%20by%20subclasses%20to%20support%20indexed%20access.%0D%0A%09%22%0D%0A%0D%0A%09%5E0"));
smalltalk.bind(smalltalk.Object,"copy",0
,function(){var self=this;
return self.shallowCopy();}
,"copying",unescape("copy%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20shallowCopy"));
smalltalk.bind(smalltalk.Object,"shallowCopy",0
,function(){var self=this;
var result=nil;
result=self.$klass.basicNew();
self.$klass.allInstVarNamesDo_((function(iv){var v=nil;
v=((function(){var $1$=(self[unescape("@")._comma(iv)]);if(nil.isNil_($1$))return nil;return $1$;})());return (((nil.isNil_(v)===false)).mustBeBoolean()==true ? (function(){return (result[unescape("@")._comma(iv)]=(v));})() : nil);}));
return result;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20class%20basicNew.%0D%0A%09self%20class%20allInstVarNamesDo%3A%20%5B%3Aiv%7C%20%7C%20v%20%7C%0D%0A%09%09v%20%3A%3D%20self%20basicAt%3A%20%24@%2Civ.%0D%0A%09%09v%20notNil%20ifTrue%3A%20%5B%20result%20basicAt%3A%20%24@%2Civ%20put%3A%20v%20%5D%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Object,"deepCopyOf:",0
,function(what){var self=this;
if ((nil.isNil_(what))){return what;};;
if (self.isObject_(what)){return what.deepCopy();};;
return what;}
,"copying",unescape("deepCopyOf%3A%20what%0D%0A%09%22%20Private%20-%20Return%20a%20deep%20copy%20of%20what.%20%22%0D%0A%0D%0A%09what%20isNil%20ifTrue%3A%20%5B%20%5Ewhat%20%5D.%0D%0A%09%28self%20isObject%3A%20what%29%20ifTrue%3A%20%5B%20%5Ewhat%20deepCopy%20%5D.%0D%0A%09%5Ewhat"));
smalltalk.bind(smalltalk.Object,"deepCopy",0
,function(){var self=this;
var result=nil;
result=self.$klass.basicNew();
self.$klass.allInstVarNamesDo_((function(iv){var v=nil;
v=self.deepCopyOf_(((function(){var $1$=(self[unescape("@")._comma(iv)]);if(nil.isNil_($1$))return nil;return $1$;})()));return (((nil.isNil_(v)===false)).mustBeBoolean()==true ? (function(){return (result[unescape("@")._comma(iv)]=(v));})() : nil);}));
return result;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20class%20basicNew.%0D%0A%09self%20class%20allInstVarNamesDo%3A%20%5B%3Aiv%7C%20%7C%20v%20%7C%0D%0A%09%09v%20%3A%3D%20self%20deepCopyOf%3A%20%28self%20basicAt%3A%20%24@%2Civ%29.%0D%0A%09%09v%20notNil%20ifTrue%3A%20%5B%20result%20basicAt%3A%20%24@%2Civ%20put%3A%20v%20%5D%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Object,"->","_minus_gt"
,function(anObject){var self=this;
return smalltalk.Association.key_value_(self,anObject);}
,"converting",unescape("-%3E%20anObject%0D%0A%09%22%20Return%20an%20association%20of%20the%20receiver%20to%20anObject.%20%22%0D%0A%0D%0A%09%5EAssociation%20key%3A%20self%20value%3A%20anObject"));
smalltalk.bind(smalltalk.Object,"asString",0
,function(){var self=this;
return self.printString();}
,"converting",unescape("asString%0D%0A%09%22%20Return%20the%20receiver%20represented%20as%20String.%20%22%0D%0A%0D%0A%09%5Eself%20printString"));
smalltalk.bind(smalltalk.Object,"toString",0
,function(){var self=this;

return unescape("%7Bst%3Aa")._comma(self.$klass.name())._comma(unescape("%7D"));}
,"converting",unescape("toString%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20String.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5E%27%7Bst%3Aa%27%2Cself%20class%20name%2C%24%7D"));
smalltalk.bind(smalltalk.Object,"toString:",0
,function(anObject){var self=this;
if ((nil.isNil_(anObject))){return "nil";};;
if (self.isObject_(anObject)){return anObject.toString();};;
return ""+anObject;}
,"converting",unescape("toString%3A%20anObject%0D%0A%09%22%20Return%20anObject%20converted%20to%20a%20String.%20%22%0D%0A%0D%0A%09anObject%20isNil%20ifTrue%3A%20%5B%20%5E%23nil%20%5D.%0D%0A%09%28self%20isObject%3A%20anObject%29%20ifTrue%3A%20%5B%20%5EanObject%20toString%20%5D.%0D%0A%09%5E%7B%27%27%20doc%27Native%20conversion%20%28default%20goes%20to%20error%29%27%7D"));
smalltalk.bind(smalltalk.Object,"asNative:",0
,function(lang){var self=this;
return self.asString();}
,"converting",unescape("asNative%3A%20lang%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver%20in%20target%20language%20lang.%0D%0A%09The%20default%20implementation%20return%20the%20receiver%20as%20a%20String.%0D%0A%09This%20method%20must%20be%20refined%20by%20subclasses%20implementing%20native%20literal%20representation.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20asString"));
smalltalk.bind(smalltalk.Object,"asNative",0
,function(){var self=this;
return self.asNative_(self.nativeLanguage());}
,"converting",unescape("asNative%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%0D%0A%09The%20default%20implementation%20return%20the%20receiver%20as%20a%20String.%0D%0A%09This%20method%20must%20be%20refined%20by%20subclasses%20implementing%20native%20literal%20representation.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20asNative%3A%20self%20nativeLanguage"));
smalltalk.bind(smalltalk.Object,"asJSON",0
,function(){var self=this;
var json=nil;
json=smalltalk.Smalltalk.nativeObjectAt_("JSON");
return json.stringify(self.asJSONObject());}
,"json",unescape("asJSON%0D%0A%09%22%20Return%20the%20JSON%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20json%20%7C%0D%0A%09json%20%3A%3D%20Smalltalk%20nativeObjectAt%3A%20%23JSON.%0D%0A%09%5Ejson%20%23stringify%3A%20self%20asJSONObject"));
smalltalk.bind(smalltalk.Object,"escapedCode",0
,function(){var self=this;
return self.asNative();}
,"converting",unescape("escapedCode%0D%0A%09%22%20Return%20the%20code%20to%20reproduce%20receiver%20from%20code.%20%22%0D%0A%0D%0A%09%5Eself%20asNative"));
smalltalk.bind(smalltalk.Object,"perform:",0
,function(aSymbol){var self=this;
return self.perform_withArguments_(aSymbol,[]);}
,"perform",unescape("perform%3A%20aSymbol%0D%0A%09%22%20Return%20the%20result%20of%20sending%20message%20aSymbol%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20perform%3A%20aSymbol%20withArguments%3A%20%23%28%29"));
smalltalk.bind(smalltalk.Object,"perform:with:",0
,function(aSymbol,argument){var self=this;
return self.perform_withArguments_(aSymbol,smalltalk.Array.with_(argument));}
,"perform",unescape("perform%3A%20aSymbol%20with%3A%20argument%0D%0A%09%22%20Return%20the%20result%20of%20sending%20message%20aSymbol%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20perform%3A%20aSymbol%20withArguments%3A%20%28Array%20with%3A%20argument%29"));
smalltalk.bind(smalltalk.Object,"perform:withArguments:",0
,function(aSymbol,aCollection){var self=this;
return self.basicPerform_withArguments_(aSymbol.asSelector(),aCollection.asArray());}
,"perform",unescape("perform%3A%20aSymbol%20withArguments%3A%20aCollection%0D%0A%09%22%20Return%20the%20result%20of%20sending%20message%20aSymbol%20to%20the%20receiver.%20%22%0D%0A%0D%0A%22%09Warning%3A%20To%20do%20NOT%20check%20arguments%20size%20is%20a%20feature%0D%0A%09%09if%20we%20do%20strict%20matching%20U8%20tools%20will%20break%20because%20they%20are%20using%20the%20feature.%0D%0A%09aSymbol%20numberOfArguments%20%3D%20aCollection%20size%20ifFalse%3A%20%5B%0D%0A%09%09%5Eself%20error%3A%20%27Invalid%20arguments%20size%27%0D%0A%09%5D.%0D%0A%22%09%5Eself%20basicPerform%3A%20aSymbol%20asSelector%20withArguments%3A%20aCollection%20asArray"));
smalltalk.bind(smalltalk.Object,"isObject:",0
,function(anObject){var self=this;
return (anObject==0 || anObject && anObject.$klass)?true:false;
return self;}
,"testing",unescape("isObject%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20a%20smalltalk%20object.%0D%0A%09Return%20false%20if%20the%20object%20is%20a%20native%20object.%0D%0A%09%22%0D%0A%0D%0A%09%7B%27%27%20doc%27Native%20detection%20%28default%20goes%20to%20error%29%27%7D"));
smalltalk.bind(smalltalk.Object,"isNumber:",0
,function(anObject){var self=this;
return self.isObject_(anObject).and_((function(){return anObject.isNumber();}));}
,"testing",unescape("isNumber%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20a%20number.%20%22%0D%0A%0D%0A%09%5E%28self%20isObject%3A%20anObject%29%20and%3A%20%5B%20anObject%20isNumber%20%5D"));
smalltalk.bind(smalltalk.Object,"basicAt:",0
,function(aString){var self=this;
var result=nil;
result=self[aString];
if ((nil.isNil_(result))){return nil;};;
return result;}
,"accessing",unescape("basicAt%3A%20aString%0D%0A%09%22%20Return%20the%20property%20of%20the%20receiver%20known%20as%20aString%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%5BaString%5D.%0D%0A%09result%20isNil%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Object,"basicAt:ifAbsent:",0
,function(aString,aBlock){var self=this;
return (function(){var $1$=(self[aString]);if (typeof($1$)=="undefined") return (function(){return aBlock.value();})();return $1$;})();}
,"accessing",unescape("basicAt%3A%20aString%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20property%20of%20the%20receiver%20known%20as%20aString%20%28or%20the%20result%20of%20evaluating%20aBlock%29.%20%22%0D%0A%0D%0A%09%5Eself%5BaString%5D%20ifUndefined%3A%20%5B%20aBlock%20value%20%5D"));
smalltalk.bind(smalltalk.Object,"basicAt:ifAbsentPut:",0
,function(aString,aBlock){var self=this;
return ((function(){var $1$=(self[aString]);if(nil.isNil_($1$))return ((function(){return (self[aString]=(aBlock.value()));}).value());return $1$;})());}
,"accessing",unescape("basicAt%3A%20aString%20ifAbsentPut%3A%20aBlock%0D%0A%09%22%20Return%20the%20property%20of%20the%20receiver%20known%20as%20aString%20%28or%20set%20it%20with%20the%20result%20of%20evaluating%20aBlock%29.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20aString%20ifAbsent%3A%20%5B%20self%20basicAt%3A%20aString%20put%3A%20aBlock%20value%20%5D"));
smalltalk.bind(smalltalk.Object,"instVarAt:",0
,function(instVarName){var self=this;
if (self.isNumber_(instVarName)){return self.instVarAt_(self.$klass.allInstVarNames().at_(instVarName));};;
return ((function(){var $1$=(self[unescape("@")._comma(instVarName)]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("instVarAt%3A%20instVarName%0D%0A%09%22%20Return%20the%20internal%20collaborator%20of%20the%20receiver%20known%20as%20instVarName.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20instVarName%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20instVarAt%3A%20%28self%20class%20allInstVarNames%20at%3A%20instVarName%29%0D%0A%09%5D.%0D%0A%09%5Eself%20basicAt%3A%20%24@%20%2CinstVarName"));
smalltalk.bind(smalltalk.Object,"instVarAt:put:",0
,function(instVarName,anObject){var self=this;
if (self.isNumber_(instVarName)){return self.instVarAt_put_(self.$klass.allInstVarNames().at_(instVarName),anObject);};;
return (self[unescape("@")._comma(instVarName)]=(anObject));}
,"accessing",unescape("instVarAt%3A%20instVarName%20put%3A%20anObject%0D%0A%09%22%20Set%20the%20internal%20collaborator%20of%20the%20receiver%20known%20as%20instVarName%20to%20anObject.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20instVarName%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20instVarAt%3A%20%28self%20class%20allInstVarNames%20at%3A%20instVarName%29%20put%3A%20anObject%0D%0A%09%5D.%0D%0A%09%5Eself%20basicAt%3A%20%24@%20%2CinstVarName%20put%3A%20anObject"));
smalltalk.bind(smalltalk.Object,"basicAt:put:",0
,function(aString,anObject){var self=this;
return self[aString]=anObject;}
,"accessing",unescape("basicAt%3A%20aString%20put%3A%20anObject%0D%0A%09%22%20Set%20the%20property%20of%20the%20receiver%20known%20as%20aString.%20%22%0D%0A%0D%0A%09%5Eself%5BaString%5D%20%3A%3D%20anObject"));
smalltalk.bind(smalltalk.Object,"error:",0
,function(aString){var self=this;
return smalltalk.Error.signal_(aString);}
,"error",unescape("error%3A%20aString%0D%0A%09%22%20Return%20the%20result%20of%20managing%20error%28aString%29%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EError%20signal%3A%20aString"));
smalltalk.bind(smalltalk.Object,"subclassResponsibility",0
,function(){var self=this;
return self.subclassResponsibility_(nil);}
,"error",unescape("subclassResponsibility%0D%0A%09%22%20Private%20-%20The%20concrete%20implementation%20on%20the%20receiver%20is%20not%20complete.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20nil"));
smalltalk.bind(smalltalk.Object,"implementedBySubclass",0
,function(){var self=this;
return self.subclassResponsibility();}
,"error",unescape("implementedBySubclass%0D%0A%09%22%20Private%20-%20The%20concrete%20implementation%20on%20the%20receiver%20is%20not%20complete.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility"));
smalltalk.bind(smalltalk.Object,"shouldNotImplement",0
,function(){var self=this;
return self.shouldNotImplement_(nil);}
,"error",unescape("shouldNotImplement%0D%0A%09%22%20The%20receiver%20received%20a%20message%20that%20is%20not%20expected.%20%22%0D%0A%0D%0A%09%5Eself%20shouldNotImplement%3A%20nil"));
smalltalk.bind(smalltalk.Object,"invalidMessage",0
,function(){var self=this;
return self.invalidMessage_(nil);}
,"error",unescape("invalidMessage%0D%0A%09%22%20The%20receiver%20received%20a%20message%20that%20is%20invalid%20%28not%20expected%20for%20this%20object%29.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20nil"));
smalltalk.bind(smalltalk.Object,"subclassResponsibility:",0
,function(aSelector){var self=this;
return self.halt().error_("Implementation of"._comma((((nil.isNil_(aSelector))).mustBeBoolean()==true ? (function(){return " ";})() : (function(){return unescape("%20%23")._comma(aSelector);})()))._comma(unescape("%20is%20a%20responsibility%20of%20the%20subclass%28"))._comma(self.$klass.name())._comma(unescape("%29.")));}
,"error",unescape("subclassResponsibility%3A%20aSelector%0D%0A%09%22%20Private%20-%20The%20concrete%20implementation%20on%20the%20receiver%20is%20not%20complete.%20%22%0D%0A%0D%0A%09%5Eself%20halt%20error%3A%0D%0A%09%09%20%27Implementation%20of%27%0D%0A%09%09%2C%28aSelector%20isNil%20ifTrue%3A%20%5B%27%20%27%5D%20ifFalse%3A%20%5B%27%20%23%27%2CaSelector%5D%29%0D%0A%09%09%2C%27%20is%20a%20responsibility%20of%20the%20subclass%28%27%2Cself%20class%20name%2C%27%29.%27"));
smalltalk.bind(smalltalk.Object,"implementedBySubclass:",0
,function(aSelector){var self=this;
return self.subclassResponsibility_(aSelector);}
,"error",unescape("implementedBySubclass%3A%20aSelector%0D%0A%09%22%20Private%20-%20The%20concrete%20implementation%20on%20the%20receiver%20is%20not%20complete.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20aSelector"));
smalltalk.bind(smalltalk.Object,"shouldNotImplement:",0
,function(aSelector){var self=this;
return self.error_("Message"._comma((((nil.isNil_(aSelector))).mustBeBoolean()==true ? (function(){return " ";})() : (function(){return unescape("%20%23")._comma(aSelector);})()))._comma(" should not be implemented for instances of ")._comma(self.$klass.name()));}
,"error",unescape("shouldNotImplement%3A%20aSelector%0D%0A%09%22%20The%20receiver%20received%20a%20message%20that%20is%20not%20expected.%20%22%0D%0A%0D%0A%09%5Eself%20error%3A%0D%0A%09%09%20%27Message%27%0D%0A%09%09%2C%28aSelector%20isNil%20ifTrue%3A%20%5B%27%20%27%5D%20ifFalse%3A%20%5B%27%20%23%27%2CaSelector%5D%29%0D%0A%09%09%2C%27%20should%20not%20be%20implemented%20for%20instances%20of%20%27%2C%20self%20class%20name"));
smalltalk.bind(smalltalk.Object,"invalidMessage:",0
,function(aSelector){var self=this;
return self.error_("Message"._comma((((nil.isNil_(aSelector))).mustBeBoolean()==true ? (function(){return " ";})() : (function(){return unescape("%20%23")._comma(aSelector);})()))._comma(" should not be sent to instances of ")._comma(self.$klass.name()));}
,"error",unescape("invalidMessage%3A%20aSelector%0D%0A%09%22%20The%20receiver%20received%20a%20message%20that%20is%20invalid%20%28not%20expected%20for%20this%20object%29.%20%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%0D%0A%09%09%20%27Message%27%0D%0A%09%09%2C%28aSelector%20isNil%20ifTrue%3A%20%5B%27%20%27%5D%20ifFalse%3A%20%5B%27%20%23%27%2CaSelector%5D%29%0D%0A%09%09%2C%27%20should%20not%20be%20sent%20to%20instances%20of%20%27%2C%20self%20class%20name"));
smalltalk.bind(smalltalk.Object,"smalltalkErrorOf:",0
,function(aNativeException){var self=this;
var result=nil;
if (self.isObject_(aNativeException)){if (smalltalk.is_kindOf_(aNativeException,(smalltalk.Exception))){return aNativeException;};;
return smalltalk.Error.$$new().messageText_(aNativeException);};;
result=((function(){var $1$=(aNativeException["smalltalkError"]);if(nil.isNil_($1$))return nil;return $1$;})());
if ((nil.isNil_(result)===false)){return result;};;
return (function($rec){$rec.messageText_((aNativeException.description || aNativeException.message));$rec.nativeException_(aNativeException);return $rec.yourself();})(smalltalk.Error.$$new());}
,"private",unescape("smalltalkErrorOf%3A%20aNativeException%0D%0A%09%22%20Private%2C%20Internal%20-%20Return%20anException%20associated%20with%20a%20native%20exception.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09%28self%20isObject%3A%20aNativeException%29%20ifTrue%3A%20%5B%0D%0A%09%09%28aNativeException%20isKindOf%3A%20Exception%29%20ifTrue%3A%20%5B%20%5EaNativeException%20%5D.%0D%0A%09%09%5EError%20new%20messageText%3A%20aNativeException%0D%0A%09%5D.%0D%0A%09result%20%3A%3D%20aNativeException%20basicAt%3A%20%23smalltalkError.%0D%0A%09result%20notNil%20ifTrue%3A%20%5B%20%5Eresult%20%5D.%0D%0A%09%5EError%20new%0D%0A%09%09messageText%3A%20aNativeException%23description%20%7C%7C%20aNativeException%23message%3B%0D%0A%09%09nativeException%3A%20aNativeException%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Object,"inspect",0
,function(){var self=this;
return self.basicInspect();}
,"inspection",unescape("inspect%0D%0A%09%22%20Open%20an%20inspector%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicInspect"));
smalltalk.bind(smalltalk.Object,"basicInspect",0
,function(){var self=this;
if (self.smalltalk().includesKey_("NativeHandle")){return self.print_(smalltalk.NativeHandle._at_gt(self));};;
return self;}
,"inspection",unescape("basicInspect%0D%0A%09%22%20Private%20-%20Open%20an%20inspector%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%28self%20smalltalk%20includesKey%3A%20%23NativeHandle%29%20ifTrue%3A%20%5B%20%22print%20native%20representation%22%0D%0A%09%09%5Eself%20print%3A%20NativeHandle%20@%3E%20self%0D%0A%09%5D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Object,"printString",0
,function(){var self=this;
return "a "._comma(self.$klass.name());}
,"printing",unescape("printString%0D%0A%09%22%20Return%20a%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%27a%20%27%2C%20self%20class%20name"));
smalltalk.bind(smalltalk.Object,"isKindOf:",0
,function(aClass){var self=this;
return self.$klass.inheritsFrom_(aClass);}
,"testing",unescape("isKindOf%3A%20aClass%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20kind%20of%20aClass.%20%22%0D%0A%0D%0A%09%5Eself%20class%20inheritsFrom%3A%20aClass"));
smalltalk.bind(smalltalk.Object,"isMemberOf:",0
,function(aClass){var self=this;
return (nil.is_eqeq_(self.$klass,aClass));}
,"testing",unescape("isMemberOf%3A%20aClass%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20member%20of%20aClass.%20%22%0D%0A%0D%0A%09%5Eself%20class%20%3D%3D%20aClass"));
smalltalk.bind(smalltalk.Object,"ifNil:",0
,function(aBlock){var self=this;
return self;}
,"testing",unescape("ifNil%3A%20aBlock%0D%0A%09%22%20Return%20the%20receiver%20if%20it%20is%20not%20nil%2C%20or%20the%20result%20of%20evaluating%20aBlock%20%28if%20it%20is%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Object,"ifNil:ifNotNil:",0
,function(aBlock,anotherBlock){var self=this;
return anotherBlock.value();}
,"testing",unescape("ifNil%3A%20aBlock%20ifNotNil%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20aBlock%20if%20the%20receiver%20is%20nil%20or%20the%20result%20of%20evaluating%20anotherBlock%20%28if%20not%20nil%29.%20%22%0D%0A%0D%0A%09%5EanotherBlock%20value"));
smalltalk.bind(smalltalk.Object,"ifNotNil:",0
,function(aBlock){var self=this;
return aBlock.value();}
,"testing",unescape("ifNotNil%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20aBlock%20if%20the%20receiver%20is%20not%20nil%2C%20or%20nil%20if%20it%20is%20nil.%20%22%0D%0A%0D%0A%09%5EaBlock%20value"));
smalltalk.bind(smalltalk.Object,"ifNotNil:ifNil:",0
,function(aBlock,anotherBlock){var self=this;
return aBlock.value();}
,"testing",unescape("ifNotNil%3A%20aBlock%20ifNil%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20aBlock%20if%20the%20receiver%20is%20not%20nil%2C%20or%20the%20result%20of%20evaluating%20anotherBlock%20if%20it%20is%20nil.%20%22%0D%0A%0D%0A%09%5EaBlock%20value"));
smalltalk.bind(smalltalk.Object,"isNil",0
,function(){var self=this;
return false;}
,"testing",unescape("isNil%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20nil.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"notNil",0
,function(){var self=this;
return (nil.isNil_(self)).not();}
,"testing",unescape("notNil%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20not%20nil.%20%22%0D%0A%0D%0A%09%5Eself%20isNil%20not"));
smalltalk.bind(smalltalk.Object,"isClass",0
,function(){var self=this;
return false;}
,"testing",unescape("isClass%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20Class.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"isMetaclass",0
,function(){var self=this;
return false;}
,"testing",unescape("isMetaclass%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20metaclass.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"isNumber",0
,function(){var self=this;
return false;}
,"testing",unescape("isNumber%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20Number.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"isString",0
,function(){var self=this;
return false;}
,"testing",unescape("isString%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20String.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"isArray",0
,function(){var self=this;
return false;}
,"testing",unescape("isArray%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20an%20Array.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"isValuable",0
,function(){var self=this;
return false;}
,"testing",unescape("isValuable%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20can%20be%20evaluated.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"typeOf:",0
,function(anObject){var self=this;
return typeof anObject;}
,"native",unescape("typeOf%3A%20anObject%0D%0A%09%22%20Return%20the%20%28native%29%20type%20of%20anObject.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27Get%20native%20type%20%28default%20goes%20to%20error%2C%20use%20to%20be%20a%20String%20instance%29%27%7D"));
smalltalk.bind(smalltalk.Object,"callableOf:",0
,function(aBlock){var self=this;

if ((nil.isNil_(aBlock))){return aBlock;};;
if (self.typeOf_(aBlock)._eq("function")){return aBlock;};;
return (function(x,y,z){return aBlock.value_value_value_(x,y,z);});}
,"native",unescape("callableOf%3A%20aBlock%0D%0A%09%22%20Return%20a%20%28native%29%20callable%20object%20for%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%7B%27self%27%20s8%27%20%5EaBlock%20%27%7D.%0D%0A%09aBlock%20isNil%20ifTrue%3A%20%5B%20%5EaBlock%20%5D.%0D%0A%09%28self%20typeOf%3A%20aBlock%29%20%3D%20%23function%20ifTrue%3A%20%5B%20%5EaBlock%20%5D.%0D%0A%09%5E%5B%3Ax%20%3Ay%20%3Az%7C%20aBlock%20value%3A%20x%20value%3A%20y%20value%3A%20z%20%5D%20"));
smalltalk.bind(smalltalk.Object,"methodFor:",0
,function(aSymbol){var self=this;
return self.$klass.compiledMethodFor_(aSymbol);}
,"messages",unescape("methodFor%3A%20aSymbol%20%0D%0A%09%22%20Return%20the%20method%20implementing%20aSymbol%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20class%20compiledMethodFor%3A%20aSymbol"));
smalltalk.bind(smalltalk.Object,"respondsTo:",0
,function(aSymbol){var self=this;
return (nil.isNil_(self.methodFor_(aSymbol))===false);}
,"messages",unescape("respondsTo%3A%20aSymbol%20%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20respond%20to%20message%20aSymbol.%20%22%0D%0A%0D%0A%09%5E%28self%20methodFor%3A%20aSymbol%29%20notNil"));
smalltalk.bind(smalltalk.Object,"basicDelete:",0
,function(aString){var self=this;
delete self[aString];
return self;}
,"accessing",unescape("basicDelete%3A%20aString%0D%0A%09%22%20Private%20-%20Basic%20implementation%20of%20deletion%20of%20property%20aString%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20deletion%27%7D"));
smalltalk.bind(smalltalk.Object,"doesNotUnderstand:",0
,function(aMessage){var self=this;
return (function($rec){$rec.receiver_(self);$rec.message_(aMessage);return $rec.signal();})(smalltalk.MessageNotUnderstood.$$new());}
,"dnu",unescape("doesNotUnderstand%3A%20aMessage%0D%0A%09%22%20Private%20-%20Basic%20implementation%20of%20a%20message%20sent%20and%20not%20understood%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EMessageNotUnderstood%20new%0D%0A%09%09receiver%3A%20self%3B%0D%0A%09%09message%3A%20aMessage%3B%0D%0A%09%09signal"));
smalltalk.bind(smalltalk.Object,"dnu:withArguments:",0
,function(selector,anArray){var self=this;
return self.doesNotUnderstand_(smalltalk.Message.selector_arguments_(selector,anArray));}
,"dnu",unescape("dnu%3A%20selector%20withArguments%3A%20anArray%0D%0A%09%22%20Private%20-%20Basic%20implementation%20of%20a%20message%20sent%20and%20not%20understood%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20doesNotUnderstand%3A%20%28Message%20selector%3A%20selector%20arguments%3A%20anArray%29"));
smalltalk.bind(smalltalk.Object,"basicPerform:withArguments:",0
,function(jsSelector,anArray){var $s8Ret$={name:"stReturn"};try{var self=this;
var fn=nil;

fn=((function(){var $1$=(self[jsSelector]);if(nil.isNil_($1$))return ((function(){return (function(){$s8Ret$.result=(function(){return self.dnu_withArguments_(jsSelector.asNativeSelector(),anArray)})();throw($s8Ret$)})();}).value());return $1$;})());
(function(){$s8Ret$.result=(function(){return fn.apply(self,anArray)})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"private",unescape("basicPerform%3A%20jsSelector%20withArguments%3A%20anArray%0D%0A%09%22%20Private%20-%20Basic%20implementation%20of%20%23perform...%20%22%0D%0A%0D%0A%09%7C%20fn%20%7C%0D%0A%09%7B%27self%27%7D.%0D%0A%09fn%20%3A%3D%20self%20basicAt%3A%20jsSelector%20ifAbsent%3A%20%5B%0D%0A%09%09%5Eself%20dnu%3A%20jsSelector%20asNativeSelector%20withArguments%3A%20anArray%0D%0A%09%5D.%0D%0A%09%5E%7B%27%27%20doc%27dynamic%20message%20send%20%28fn%20can%20be%20native%20function%20to%20call%29%27%7D"));
smalltalk.bind(smalltalk.Object,"json",0
,function(){var self=this;
return self.asJSONObject();}
,"json",unescape("json%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%5Eself%20asJSONObject"));
smalltalk.bind(smalltalk.Object,"asJson:",0
,function(anObject){var self=this;
if ((nil.isNil_(anObject))){return nil.json();};;
return ((self.isObject_(anObject)).mustBeBoolean()==true ? (function(){return anObject.json();})() : (function(){return anObject;})());}
,"json",unescape("asJson%3A%20anObject%0D%0A%09%22%20Return%20anObject%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09anObject%20isNil%20ifTrue%3A%20%5B%20%5Enil%20json%20%5D.%0D%0A%09%5E%28self%20isObject%3A%20anObject%29%0D%0A%09%09ifTrue%3A%20%5B%20anObject%20json%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20anObject%20%5D"));
smalltalk.bind(smalltalk.Object,"asJSONObject",0
,function(){var self=this;
var result=nil;
result={};
self.jsonFieldsDo_ignoring_((function(field,value){return (((nil.isNil_(value)===false).or_((function(){return self.jsonExportNil();}))).mustBeBoolean()==true ? (function(){return (result[field]=(self.asJson_(value)));})() : nil);}),self.jsonHiddenNames());
return result;}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20%7B%27%27%20doc%27get%20empty%20JSON%20collector%27%7D.%0D%0A%09self%09jsonFieldsDo%3A%20%5B%3Afield%20%3Avalue%7C%0D%0A%09%09%28value%20notNil%20or%3A%20%5B%20self%20jsonExportNil%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%09result%20basicAt%3A%20field%20put%3A%20%28self%20asJson%3A%20value%29%0D%0A%09%09%5D.%0D%0A%09%5D%09ignoring%3A%20self%20jsonHiddenNames.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Object,"jsonHiddenNames",0
,function(){var self=this;
return [];}
,"converting",unescape("jsonHiddenNames%0D%0A%09%22%20Private%20-%20Return%20the%20names%20of%20the%20fields%20to%20skip%20when%20mapping%20the%20receiver%20to%20JSON%20structure.%20%22%0D%0A%0D%0A%09%5E%23%28%29"));
smalltalk.bind(smalltalk.Object,"jsonFieldsDo:ignoring:",0
,function(aBlock,ignored){var self=this;
return self.$klass.allInstVarNamesDo_((function(each){return (((nil.isNil_(ignored)===false).and_((function(){return ignored.includes_(each);}))).mustBeBoolean()==true ? nil : (function(){return aBlock.value_value_(each,self.instVarAt_(each));})());}));}
,"converting",unescape("jsonFieldsDo%3A%20aBlock%20ignoring%3A%20ignored%0D%0A%09%22%20Private%20-%20Evaluates%20a%28TwoArguments%29Block%20with%20the%20fields%20to%20map%20the%20receiver%20to%20JSON%20structure.%20%22%0D%0A%0D%0A%09%5Eself%20class%20allInstVarNamesDo%3A%20%5B%3Aeach%7C%0D%0A%09%09%28ignored%20notNil%20and%3A%20%5B%20ignored%20includes%3A%20each%20%5D%29%20ifFalse%3A%20%5B%0D%0A%09%09%09aBlock%20value%3A%20each%20value%3A%20%28self%20instVarAt%3A%20each%29%0D%0A%09%09%5D.%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Object,"jsonExportNil",0
,function(){var self=this;
return false;}
,"converting",unescape("jsonExportNil%0D%0A%09%22%20Private%20-%20Return%20true%20if%20the%20receiver%20export%20nil%20values.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"mustBeBoolean",0
,function(){var self=this;
return self.error_("Must be boolean.");}
,"testing",unescape("mustBeBoolean%0D%0A%09%22%20Signal%20an%20error%20if%20the%20receiver%20is%20not%20a%20Boolean.%20%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%27Must%20be%20boolean.%27"));
smalltalk.bind(smalltalk.Object,"isBoolean",0
,function(){var self=this;
return false;}
,"testing",unescape("isBoolean%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20boolean.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"mustBeNumber",0
,function(){var self=this;
return self.error_("Must be a Number");}
,"testing",unescape("mustBeNumber%0D%0A%09%22%20Signal%20an%20error%20if%20the%20receiver%20is%20not%20a%20Number.%20%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%27Must%20be%20a%20Number%27"));
smalltalk.bind(smalltalk.Object,"asLiteral",0
,function(){var self=this;
return self.printString();}
,"converting",unescape("asLiteral%0D%0A%09%22%20Return%20the%20receiver%20in%20literal%20notation.%20%22%0D%0A%0D%0A%09%5Eself%20printString"));
smalltalk.bind(smalltalk.Object,"isCharacter",0
,function(){var self=this;
return false;}
,"testing",unescape("isCharacter%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20character.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"isAssociation",0
,function(){var self=this;
return false;}
,"testing",unescape("isAssociation%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20anAssociation.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"isDictionary",0
,function(){var self=this;
return false;}
,"testing",unescape("isDictionary%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20dictionary.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"isRectangle",0
,function(){var self=this;
return false;}
,"testing",unescape("isRectangle%0D%0A%09%22%20Return%20true%20if%20receiver%20is%20a%20Rectangle.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"isMessage",0
,function(){var self=this;
return false;}
,"testing",unescape("isMessage%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20message.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"argumentCount",0
,function(){var self=this;
return (0);}
,"evaluating",unescape("argumentCount%0D%0A%09%22%20Return%20the%20number%20of%20arguments%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E0"));
smalltalk.bind(smalltalk.Object,"numberOfArguments",0
,function(){var self=this;
return self.argumentCount();}
,"evaluating",unescape("numberOfArguments%0D%0A%09%22%20Return%20the%20number%20of%20arguments%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20argumentCount"));
smalltalk.bind(smalltalk.Object,"evaluate",0
,function(){var self=this;
return self;}
,"evaluating",unescape("evaluate%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Object,"evaluateDeferred",0
,function(){var self=this;
(function(){return self.evaluate();}).valueDeferred();
return self;}
,"evaluating",unescape("evaluateDeferred%0D%0A%09%22%20Evaluating%20the%20receiver%20deferred%20in%20time.%0D%0A%09Return%20the%20receiver.%20%22%0D%0A%0D%0A%09%5B%20self%20evaluate%20%5D%20valueDeferred."));
smalltalk.bind(smalltalk.Object,"evaluateWithArguments:",0
,function(anArray){var self=this;
return self;}
,"evaluating",unescape("evaluateWithArguments%3A%20anArray%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Object,"evaluateWith:",0
,function(anObject){var self=this;
return self.evaluateWithArguments_(smalltalk.Array.with_(anObject));}
,"evaluating",unescape("evaluateWith%3A%20anObject%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20evaluateWithArguments%3A%20%28Array%20with%3A%20anObject%29"));
smalltalk.bind(smalltalk.Object,"logEvaluate:",0
,function(source){var self=this;
var result=nil;
result=self.evaluate();
smalltalk.SystemManager.log_(source);
return result;}
,"evaluating",unescape("logEvaluate%3A%20source%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%0D%0A%09Log%20the%20source%20expression%20as%20reference%20of%20evaluation.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20evaluate.%0D%0A%09SystemManager%20log%3A%20source.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Object,"asActionSequence",0
,function(){var self=this;
return smalltalk.ActionSequence.with_(self);}
,"converting",unescape("asActionSequence%0D%0A%09%22%20Return%20an%20ActionSequence%20from%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EActionSequence%20with%3A%20self"));
smalltalk.bind(smalltalk.Object,"eventTable",0
,function(){var self=this;
return ((function(){var $1$=(self[unescape("@handlers")]);if(nil.isNil_($1$))return ((function(){return self.eventTableCreate();}).value());return $1$;})());}
,"events",unescape("eventTable%0D%0A%09%22%20Private%20-%20Return%20the%20mapping%20event%20names%20to%20actions%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27@handlers%27%20ifAbsent%3A%20%5B%20self%20eventTableCreate%20%5D"));
smalltalk.bind(smalltalk.Object,"eventTableForEdit",0
,function(){var self=this;
return ((function(){var $1$=(self[unescape("@handlers")]);if(nil.isNil_($1$))return ((function(){return (self[unescape("@handlers")]=(self.eventTableCreate()));}).value());return $1$;})());}
,"events",unescape("eventTableForEdit%0D%0A%09%22%20Private%20-%20Return%20the%20mapping%20event%20names%20to%20actions%20of%20the%20receiver.%0D%0A%09Set%20the%20internal%20map%20if%20not%20present.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27@handlers%27%20ifAbsent%3A%20%5B%0D%0A%09%09self%20basicAt%3A%20%27@handlers%27%20put%3A%20self%20eventTableCreate%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Object,"releaseEventTable",0
,function(){var self=this;
self.basicDelete_(unescape("@handlers"));
return self;}
,"events",unescape("releaseEventTable%0D%0A%09%22%20Private%20-%20Release%20the%20event%20table.%20%22%0D%0A%0D%0A%09self%20basicDelete%3A%20%27@handlers%27"));
smalltalk.bind(smalltalk.Object,"release",0
,function(){var self=this;
self.releaseEventTable();
return self;}
,"release",unescape("release%0D%0A%09%22%20Release%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20releaseEventTable%20"));
smalltalk.bind(smalltalk.Object,"eventTableCreate",0
,function(){var self=this;
return smalltalk.PoolDictionary.$$new();}
,"events",unescape("eventTableCreate%0D%0A%09%22%20Private%20-%20Return%20a%20new%20mapping%20for%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EPoolDictionary%20new"));
smalltalk.bind(smalltalk.Object,"canTriggerEvent:",0
,function(selector){var self=this;
return self.$klass.canTriggerEvent_(selector);}
,"events",unescape("canTriggerEvent%3A%20selector%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20can%20trigger%20event.%22%0D%0A%0D%0A%09%5Eself%20class%20canTriggerEvent%3A%20selector%20"));
smalltalk.bind(smalltalk.Object,"invalidEvent:",0
,function(selector){var self=this;
return self;}
,"events",unescape("invalidEvent%3A%20selector%0D%0A%09%22%20Private%20-%20Notify%20the%20situation%20where%20an%20unknown%20event%20is%20registered.%0D%0A%09Default%20implementation%20ignore%20error%20during%20unknown%20event%20registration.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%0D%0A%22DEBUG-%09%5Eself%20note%3A%20self%20class%20name%2C%27%20invalidEvent%3A%20%27%20%2Cselector%20asLiteral%20%22"));
smalltalk.bind(smalltalk.Object,"when:evaluate:",0
,function(selector,anAction){var self=this;
var actions=nil;
if (!(self.canTriggerEvent_(selector))){self.invalidEvent_(selector);};;
actions=self.actionListForEvent_(selector);
if (actions.includes_(anAction)){return self;};;
self.setActionList_forEvent_(actions.copyWith_(anAction),selector);
return self;}
,"events",unescape("when%3A%20selector%20evaluate%3A%20anAction%0D%0A%09%22%20Register%20anAction%20to%20the%20be%20evaluated%20when%20the%20receiver%20triggers%20the%20event.%20%22%0D%0A%0D%0A%09%7C%20actions%20%7C%0D%0A%09%28self%20canTriggerEvent%3A%20selector%29%20ifFalse%3A%20%5B%20self%20invalidEvent%3A%20selector%20%5D.%0D%0A%09actions%20%3A%3D%20self%20actionListForEvent%3A%20selector.%0D%0A%09%28actions%20includes%3A%20anAction%29%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09self%0D%0A%09%09setActionList%3A%20%28actions%20copyWith%3A%20anAction%29%0D%0A%09%09forEvent%3A%20selector"));
smalltalk.bind(smalltalk.Object,"actionListForEvent:",0
,function(selector){var $s8Ret$={name:"stReturn"};try{var self=this;
var result=nil;
result=self.eventTable().at_ifAbsent_(selector,(function(){return (function(){$s8Ret$.result=(function(){return smalltalk.ActionSequence.$$new()})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return result.asActionSequence()})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"events",unescape("actionListForEvent%3A%20selector%0D%0A%09%22%20Private%20-%20Return%20a%20collection%20of%20actions%20to%20evaluate%20when%20the%20event%20is%20triggered.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20eventTable%20at%3A%20selector%20ifAbsent%3A%20%5B%20%5EActionSequence%20new%20%5D.%0D%0A%09%5Eresult%20asActionSequence"));
smalltalk.bind(smalltalk.Object,"setActionList:forEvent:",0
,function(actionSequence,selector){var self=this;
var action=nil;
action=actionSequence.asActionSequence().reduced();
if ((nil.isNil_(action))){return self.removeActionsForEvent_(selector);};;
self.eventTableForEdit().at_put_(selector,action);
return self;}
,"events",unescape("setActionList%3A%20actionSequence%20forEvent%3A%20selector%0D%0A%09%22%20Private%20-%20Set%20the%20list%20of%20actions%20to%20evaluate%20when%20the%20event%20is%20triggered%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20action%20%7C%0D%0A%09action%20%3A%3D%20actionSequence%20asActionSequence%20reduced.%0D%0A%09action%20isNil%20ifTrue%3A%20%5B%20%5Eself%20removeActionsForEvent%3A%20selector%20%5D.%0D%0A%09self%20eventTableForEdit%20at%3A%20selector%20put%3A%20action%20"));
smalltalk.bind(smalltalk.Object,"removeActionsSatisfying:forEvent:",0
,function(aBlock,selector){var self=this;
self.setActionList_forEvent_(self.actionListForEvent_(selector).asArray().reject_(aBlock),selector);
return self;}
,"events",unescape("removeActionsSatisfying%3A%20aBlock%20forEvent%3A%20selector%0D%0A%09%22%20Private%20-%20Remove%20all%20actions%20for%20the%20event%20that%20evaluates%20aBlock%20to%20true.%22%0D%0A%0D%0A%09self%0D%0A%09%09setActionList%3A%20%28%28self%20actionListForEvent%3A%20selector%29%20asArray%20reject%3A%20aBlock%29%0D%0A%09%09forEvent%3A%20selector%20"));
smalltalk.bind(smalltalk.Object,"removeActionsWithReceiver:forEvent:",0
,function(anObject,selector){var self=this;
self.removeActionsSatisfying_forEvent_((function(each){return smalltalk.responds_to_(each,("receiver")).and_((function(){return (nil.is_eqeq_(each.receiver(),anObject));}));}),selector);
return self;}
,"events",unescape("removeActionsWithReceiver%3A%20anObject%20forEvent%3A%20selector%0D%0A%09%22%20Remove%20all%20actions%20for%20the%20event%20in%20the%20receiver%27s%20event%20table%20which%20have%20anObject%20as%20receiver.%20%22%0D%0A%0D%0A%09self%0D%0A%09%09removeActionsSatisfying%3A%20%5B%3Aeach%7C%20%28each%20respondsTo%3A%20%23receiver%29%20and%3A%20%5B%20each%20receiver%20%3D%3D%20anObject%20%5D%20%5D%0D%0A%09%09forEvent%3A%20selector%20"));
smalltalk.bind(smalltalk.Object,"removeAction:forEvent:",0
,function(anAction,selector){var self=this;
self.removeActionsSatisfying_forEvent_((function(each){return each._eq(anAction);}),selector);
return self;}
,"events",unescape("removeAction%3A%20anAction%20forEvent%3A%20selector%0D%0A%09%22%20Remove%20anAction%20for%20the%20event%20in%20the%20receiver%27s%20event%20table%20which%20have%20anObject%20as%20receiver.%20%22%0D%0A%0D%0A%09self%0D%0A%09%09removeActionsSatisfying%3A%20%5B%3Aeach%7C%20each%20%3D%20anAction%20%5D%0D%0A%09%09forEvent%3A%20selector%20"));
smalltalk.bind(smalltalk.Object,"when:do:",0
,function(selector,aBlock){var self=this;
self.when_evaluate_(selector,aBlock);
return self;}
,"events",unescape("when%3A%20selector%20do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20when%20the%20receiver%20triggers%20the%20event.%20%22%0D%0A%0D%0A%09self%20when%3A%20selector%20evaluate%3A%20aBlock"));
smalltalk.bind(smalltalk.Object,"when:send:to:withArguments:",0
,function(selector,message,anObject,array){var self=this;
self.when_evaluate_(selector,smalltalk.LinkMessage.receiver_selector_arguments_(anObject,message,array));
return self;}
,"events",unescape("when%3A%20selector%20send%3A%20message%20to%3A%20anObject%20withArguments%3A%20array%0D%0A%09%22%20Send%20the%20message%20when%20the%20receiver%20triggers%20the%20event.%20%22%0D%0A%0D%0A%09self%0D%0A%09%09when%3A%20selector%0D%0A%09%09evaluate%3A%20%28LinkMessage%0D%0A%09%09%09receiver%3A%20anObject%0D%0A%09%09%09selector%3A%20message%0D%0A%09%09%09arguments%3A%20array%29"));
smalltalk.bind(smalltalk.Object,"when:send:to:",0
,function(selector,message,anObject){var self=this;
self.when_evaluate_(selector,smalltalk.Message.receiver_selector_arguments_(anObject,message,smalltalk.Array.new_(message.argumentCount())));
return self;}
,"events",unescape("when%3A%20selector%20send%3A%20message%20to%3A%20anObject%0D%0A%09%22%20Send%20the%20message%20when%20the%20receiver%20triggers%20the%20event.%20%22%0D%0A%0D%0A%09self%0D%0A%09%09when%3A%20selector%0D%0A%09%09evaluate%3A%20%28Message%0D%0A%09%09%09receiver%3A%20anObject%20selector%3A%20message%0D%0A%09%09%09arguments%3A%20%28Array%20new%3A%20message%20argumentCount%29%29"));
smalltalk.bind(smalltalk.Object,"triggerEvent:ifNotHandled:",0
,function(selector,aBlock){var self=this;
return self.eventTable().at_ifAbsent_(selector,(function(){return aBlock;})).evaluate();}
,"events",unescape("triggerEvent%3A%20selector%20ifNotHandled%3A%20aBlock%0D%0A%09%22%20Trigger%20the%20event.%0D%0A%09If%20the%20event%20is%20not%20handled%2C%20return%20the%20value%20of%20evaluating%20aBlock%2C%0D%0A%20%20%20%20%20%20%20%20or%20the%20value%20returned%20by%20the%20most%20recently%20defined%20event%20handler.%0D%0A%09%22%0D%0A%0D%0A%09%5E%28self%20eventTable%20at%3A%20selector%20ifAbsent%3A%20%5B%20aBlock%20%5D%29%20evaluate"));
smalltalk.bind(smalltalk.Object,"actionForEvent:",0
,function(selector){var self=this;
return self.eventTable().at_ifAbsent_(selector,(function(){return nil;}));}
,"events",unescape("actionForEvent%3A%20selector%0D%0A%09%22%20Return%20the%20action%20to%20evaluate%20when%20the%20event%20is%20triggered%20by%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20eventTable%20at%3A%20selector%20ifAbsent%3A%20%5B%20nil%20%5D"));
smalltalk.bind(smalltalk.Object,"triggerEvent:",0
,function(selector){var self=this;
return self.actionForEvent_(selector).evaluate();}
,"events",unescape("triggerEvent%3A%20selector%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20event.%20%22%0D%0A%0D%0A%09%5E%28self%20actionForEvent%3A%20selector%29%20evaluate"));
smalltalk.bind(smalltalk.Object,"triggerEvent:withArguments:",0
,function(selector,array){var self=this;
return self.actionForEvent_(selector).evaluateWithArguments_(array);}
,"events",unescape("triggerEvent%3A%20selector%20withArguments%3A%20array%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20event.%20%22%0D%0A%0D%0A%09%5E%28self%20actionForEvent%3A%20selector%29%20evaluateWithArguments%3A%20array"));
smalltalk.bind(smalltalk.Object,"triggerEvent:with:",0
,function(selector,anObject){var self=this;
return self.triggerEvent_withArguments_(selector,smalltalk.Array.with_(anObject));}
,"events",unescape("triggerEvent%3A%20selector%20with%3A%20anObject%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20event.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20selector%20withArguments%3A%20%28Array%20with%3A%20anObject%29"));
smalltalk.bind(smalltalk.Object,"triggerEvent:with:with:",0
,function(selector,anObject,anotherObject){var self=this;
return self.triggerEvent_withArguments_(selector,smalltalk.Array.with_with_(anObject,anotherObject));}
,"events",unescape("triggerEvent%3A%20selector%20with%3A%20anObject%20with%3A%20anotherObject%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20event.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20selector%20withArguments%3A%20%28Array%20with%3A%20anObject%20with%3A%20anotherObject%29"));
smalltalk.bind(smalltalk.Object,"triggerEvent:withArguments:ifNotHandled:",0
,function(selector,array,aBlock){var self=this;
return self.eventTable().at_ifAbsent_(selector,(function(){return aBlock;})).evaluateWithArguments_(array);}
,"events",unescape("triggerEvent%3A%20selector%20withArguments%3A%20array%20ifNotHandled%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20event%20with%20the%20arguments.%0D%0A%09If%20the%20event%20is%20not%20handled%2C%20return%20the%20value%20of%20evaluating%20aBlock%2C%0D%0A%09or%20the%20value%20returned%20by%20the%20most%20recently%20defined%20event%20handler%20action.%0D%0A%09%22%0D%0A%0D%0A%09%5E%28self%20eventTable%20at%3A%20selector%20ifAbsent%3A%20%5B%20aBlock%20%5D%29%20evaluateWithArguments%3A%20array"));
smalltalk.bind(smalltalk.Object,"triggerEvent:with:ifNotHandled:",0
,function(selector,anObject,aBlock){var self=this;
return self.triggerEvent_withArguments_ifNotHandled_(selector,smalltalk.Array.with_(anObject),aBlock);}
,"events",unescape("triggerEvent%3A%20selector%20with%3A%20anObject%20ifNotHandled%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20event%20with%20anObject.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20selector%20withArguments%3A%20%28Array%20with%3A%20anObject%29%20ifNotHandled%3A%20aBlock"));
smalltalk.bind(smalltalk.Object,"hasActionForEvent:",0
,function(selector){var self=this;
return self.eventTable().includesKey_(selector);}
,"events",unescape("hasActionForEvent%3A%20selector%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20has%20an%20action%20registered%20for%20the%20event.%20%22%0D%0A%0D%0A%09%5Eself%20eventTable%20includesKey%3A%20selector%20"));
smalltalk.bind(smalltalk.Object,"eventsHandled",0
,function(){var self=this;
return self.eventTable().keys();}
,"events",unescape("eventsHandled%0D%0A%09%22%20Return%20the%20events%20for%20which%20there%20are%20actions%20registered%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20eventTable%20keys"));
smalltalk.bind(smalltalk.Object,"removeAllActionsWithReceiver:",0
,function(anObject){var self=this;
self.eventTable().keys().do_((function(each){return self.removeActionsWithReceiver_forEvent_(anObject,each);}));
return self;}
,"events",unescape("removeAllActionsWithReceiver%3A%20anObject%0D%0A%09%22%20Remove%20all%20actions%20for%20all%20events%20in%20the%20receiver%27s%20event%20table%20with%20anObject%20as%20receiver.%20%22%0D%0A%0D%0A%09self%20eventTable%20keys%20do%3A%20%5B%3Aeach%7Cself%20removeActionsWithReceiver%3A%20anObject%20forEvent%3A%20each%20%5D"));
smalltalk.bind(smalltalk.Object,"removeActionsForEvent:",0
,function(selector){var $s8Ret$={name:"stReturn"};try{var self=this;
var actions=nil;
actions=self.eventTable();
actions.removeKey_ifAbsent_(selector,(function(){return (function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();}));
if (actions.isEmpty()){self.releaseEventTable();};;
(function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"events",unescape("removeActionsForEvent%3A%20selector%0D%0A%09%22%20Remove%20all%20actions%20for%20the%20event.%20%22%0D%0A%0D%0A%09%7C%20actions%20%7C%0D%0A%09actions%20%3A%3D%20self%20eventTable.%0D%0A%09actions%20removeKey%3A%20selector%20ifAbsent%3A%20%5B%20%5Eself%20%5D.%0D%0A%09actions%20isEmpty%20ifTrue%3A%20%5B%20self%20releaseEventTable%20%5D."));
smalltalk.bind(smalltalk.Object,"addDependent:",0
,function(anObject){var self=this;
self.when_send_to_("changed","update",anObject);
return self;}
,"changes",unescape("addDependent%3A%20anObject%0D%0A%09%22%20Make%20anObject%20dependent%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20when%3A%20%23changed%20send%3A%20%23update%20to%3A%20anObject"));
smalltalk.bind(smalltalk.Object,"changed",0
,function(){var self=this;
self.triggerEvent_("changed");
return self;}
,"changes",unescape("changed%0D%0A%09%22%20Update%20the%20dependents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20triggerEvent%3A%20%23changed"));
smalltalk.bind(smalltalk.Object,"updateEvent:selector:arguments:",0
,function(selector,message,array){var $s8Ret$={name:"stReturn"};try{var self=this;
var action=nil;
action=self.eventTable().at_ifAbsent_(selector,(function(){return (function(){$s8Ret$.result=(function(){return nil})();throw($s8Ret$)})();}));
action.asActionSequence().evaluateWithSelector_withArguments_(message,array);
(function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"changes",unescape("updateEvent%3A%20selector%20selector%3A%20message%20arguments%3A%20array%0D%0A%09%22%20Private%20-%20Trigger%20the%20event%20with%20the%20message%20selector%20and%20the%20arguments.%20%22%0D%0A%0D%0A%09%7C%20action%20%7C%0D%0A%09action%20%3A%3D%20self%20eventTable%20at%3A%20selector%20ifAbsent%3A%20%5B%20%5Enil%20%5D.%0D%0A%09action%20asActionSequence%0D%0A%09%09evaluateWithSelector%3A%20message%0D%0A%09%09withArguments%3A%20array%20"));
smalltalk.bind(smalltalk.Object,"changed:",0
,function(aspect){var self=this;
self.updateEvent_selector_arguments_("changed","update:",smalltalk.Array.with_(aspect));
return self;}
,"changes",unescape("changed%3A%20aspect%0D%0A%09%22%20Update%20the%20dependents%20on%20aspect%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%09updateEvent%3A%20%23changed%20selector%3A%20%23update%3A%0D%0A%09%09arguments%3A%20%28Array%20with%3A%20aspect%29"));
smalltalk.bind(smalltalk.Object,"changed:with:",0
,function(aspect,anObject){var self=this;
self.updateEvent_selector_arguments_("changed","update:with:",smalltalk.Array.with_with_(aspect,anObject));
return self;}
,"changes",unescape("changed%3A%20aspect%20with%3A%20anObject%0D%0A%09%22%20Update%20the%20dependents%20on%20aspect%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%09updateEvent%3A%20%23changed%20selector%3A%20%23update%3Awith%3A%0D%0A%09%09arguments%3A%20%28Array%20with%3A%20aspect%20with%3A%20anObject%29"));
smalltalk.bind(smalltalk.Object,"changed:with:with:",0
,function(aspect,anObject,anotherObject){var self=this;
self.updateEvent_selector_arguments_("changed","update:with:with:",smalltalk.Array.with_with_with_(aspect,anObject,anotherObject));
return self;}
,"changes",unescape("changed%3A%20aspect%20with%3A%20anObject%20with%3A%20anotherObject%0D%0A%09%22%20Update%20the%20dependents%20on%20aspect%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%09updateEvent%3A%20%23changed%20selector%3A%20%23update%3Awith%3Awith%3A%0D%0A%09%09arguments%3A%20%28Array%20with%3A%20aspect%20with%3A%20anObject%20with%3A%20anotherObject%29"));
smalltalk.bind(smalltalk.Object,"dependents",0
,function(){var self=this;
return self.actionListForEvent_("changed").collect_((function(each){return each.receiver();}));}
,"changes",unescape("dependents%0D%0A%09%22%20Return%20the%20dependents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28self%20actionListForEvent%3A%20%23changed%29%20collect%3A%20%5B%3Aeach%20%7C%20each%20receiver%20%5D"));
smalltalk.bind(smalltalk.Object,"update:with:",0
,function(anObject,anotherObject){var self=this;
return self;}
,"changes",unescape("update%3A%20anObject%20with%3A%20anotherObject%0D%0A%09%22%20Update%20the%20receiver.%0D%0A%09Default%20implementation%20do%20nothing.%0D%0A%09This%20method%20must%20be%20refined%20by%20subclasses%20to%20support%20change/update%20protocoll.%0D%0A%09%22"));
smalltalk.bind(smalltalk.Object,"update:",0
,function(anObject){var self=this;
return self;}
,"changes",unescape("update%3A%20anObject%0D%0A%09%22%20Update%20the%20receiver.%0D%0A%09Default%20implementation%20do%20nothing.%0D%0A%09This%20method%20must%20be%20refined%20by%20subclasses%20to%20support%20change/update%20protocoll.%0D%0A%09%22"));
smalltalk.bind(smalltalk.Object,"printingWith:do:",0
,function(printing,aBlock){var self=this;
var old=nil;
var result=nil;

(function(){old=smalltalk.Smalltalk.install_as_(printing,"print");return result=aBlock.value();}).ensure_((function(){return smalltalk.Smalltalk.install_as_(old,"print");}));
return result;}
,"printing",unescape("printingWith%3A%20printing%20do%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20aBlock%2C%20redirecting%20output%20to%20printing%20block.%0D%0A%09Note%3A%20native%20print%28%29%20will%20be%20overwritten%20with%20printing%20block%20and%20restored%20to%20original%20after%20aBlock%20is%20evaluated.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20old%20result%20%7C%0D%0A%09%7B%27self%27%20doc%27native%20printing%27%7D.%0D%0A%09%5B%09old%20%3A%3D%20Smalltalk%20install%3A%20printing%20as%3A%20%23print.%0D%0A%09%09result%20%3A%3D%20aBlock%20value.%0D%0A%09%5D%20ensure%3A%20%5B%20Smalltalk%20install%3A%20old%20as%3A%20%23print%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Object,"isParseFailure",0
,function(){var self=this;
return false;}
,"Compiler-testing",unescape("isParseFailure%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20parse%20failure.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Object,"asParser",0
,function(){var self=this;
return self.subclassResponsibility_("asParser");}
,"Compiler-converting",unescape("asParser%0D%0A%09%22%20Return%20a%20parser%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23asParser"));
smalltalk.bind(smalltalk.Object,"ifUndefined:",0
,function(aBlock){var self=this;
return self;}
,"Compiler-inlines",unescape("ifUndefined%3A%20aBlock%0D%0A%09%22%20Return%20the%20receiver%20or%20the%20result%20of%20evaluating%20aBlock%20if%20it%20is%20undefined.%0D%0A%09Notes%3A%09-this%20message%20is%20inlined%20by%20compiler%20%28it%20is%20not%20sent%29%0D%0A%09%09-aBlock%20is%20evaluated%20ONLY%20in%20case%20of%20undefined.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Object,"orNil",0
,function(){var self=this;
return self;}
,"Compiler-inlines",unescape("orNil%0D%0A%09%22%20Return%20the%20receiver%20or%20nil.%0D%0A%09Notes%3A%09-this%20message%20is%20inlined%20by%20compiler%20%28it%20is%20not%20sent%29%0D%0A%09%09-nil%2C%20null%20and%20undefined%20returns%20nil.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Object,"accept:",0
,function(what){var self=this;
self.note_(unescape("%5Bbug%5D%20")._comma(self.asString())._comma(" accept: ")._comma(what.halt()));
return self.error_(unescape("%23accept%3A%20out%20of%20control."));}
,"Compiler-parser-bug",unescape("accept%3A%20what%0D%0A%09%22%20Private%20-%20Report%20buggy%20send%20by%20compiler.%20%22%0D%0A%0D%0A%09self%20note%3A%20%27%5Bbug%5D%20%27%20%2Cself%20asString%20%2C%27%20accept%3A%20%27%20%2Cwhat%20halt.%0D%0A%09%5Eself%20error%3A%20%27%23accept%3A%20out%20of%20control.%27"));
smalltalk.bind(smalltalk.Object,"compilerSupport",0
,function(){var self=this;
return self.smalltalk().compilerSupport();}
,"Compiler-accessing",unescape("compilerSupport%0D%0A%09%22%20Return%20the%20compiler%20of%20the%20receiver%20%22%0D%0A%0D%0A%09%5Eself%20smalltalk%20compilerSupport"));
smalltalk.bind(smalltalk.Object,"scanFrom:",0
,function(aStream){var self=this;
var chunk=nil;
chunk=aStream.nextChunk();
if (chunk.isEmpty()){return nil;};;
return chunk.evaluateIn_(self);}
,"Tools-fileIn",unescape("scanFrom%3A%20aStream%0D%0A%09%22%20FileIn%20-%20Return%20the%20result%20of%20evaluating%20the%20next%20chunk%20of%20data%20from%20aStream.%0D%0A%09The%20method%20can%20be%20refined%20by%20chunk%20readers%20to%20support%20custom%20fileIn%20features.%0D%0A%09Must%20return%20the%20next%20reader%20to%20use%20for%20reading%20remaining%20contents%20from%20aStream.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20chunk%20%7C%0D%0A%09chunk%20%3A%3D%20aStream%20nextChunk.%0D%0A%09chunk%20isEmpty%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%5Echunk%20evaluateIn%3A%20self"));
smalltalk.bind(smalltalk.Object,"javascriptCodeFollows",0
,function(){var self=this;
return self.nativeCodeFollows();}
,"Tools-fileIn",unescape("javascriptCodeFollows%0D%0A%09%22%20Obsolete%20-%20use%20%23nativeCodeFollows%22%0D%0A%0D%0A%09%5Eself%20nativeCodeFollows"));
smalltalk.bind(smalltalk.Object,"nativeCodeFollows",0
,function(){var self=this;
return smalltalk.ChunkReader.$$native();}
,"Tools-fileIn",unescape("nativeCodeFollows%0D%0A%09%22%20FileIn%20-%20FileIn%20native%20code%20on%20current%20stream.%0D%0A%09Consume%20stream%20contents%20upto%20end%20of%20stream%20evaluating%20the%20contents%20as%20native%20code.%0D%0A%09%22%0D%0A%0D%0A%09%5EChunkReader%20native"));
smalltalk.bind(smalltalk.Object,"cancelFileIn",0
,function(){var self=this;
return self.ignoreFileInUpTo_(nil);}
,"Tools-fileIn",unescape("cancelFileIn%0D%0A%09%22%20FileIn%20-%20Cancel%20fileIn%20on%20current%20stream.%20%22%0D%0A%0D%0A%09%5Eself%20ignoreFileInUpTo%3A%20nil"));
smalltalk.bind(smalltalk.Object,"ignoreFileInUpTo:",0
,function(aToken){var self=this;
return smalltalk.SkipFileInReader.upTo_(aToken);}
,"Tools-fileIn",unescape("ignoreFileInUpTo%3A%20aToken%0D%0A%09%22%20FileIn%20-%20Ignore%20fileIn%20on%20current%20stream%20upto%20token.%20%22%0D%0A%0D%0A%09%5ESkipFileInReader%20upTo%3A%20aToken"));
smalltalk.bind(smalltalk.Object,"==","_eq_eq"
,function(anObject){var self=this;
return (nil.is_eqeq_(anObject,self));}
,"Tools-comparing",unescape("%3D%3D%20anObject%0D%0A%09%5EanObject%20%3D%3D%20self"));
smalltalk.bind(smalltalk.Object,"~~","_no_no"
,function(anObject){var self=this;
return (nil.is_eqeq_((nil.is_eqeq_(self,anObject)),false));}
,"Tools-comparing",unescape("%7E%7E%20anObject%0D%0A%09%5E%28self%20%3D%3D%20anObject%29%20%3D%3D%20false"));
smalltalk.bind(smalltalk.Object,"isFile:olderThan:",0
,function(fileName,referenceFileName){var self=this;

if ((nil.isNil_(((function(){var $1$=(smalltalk.Smalltalk.global()["isFileOlderThan"]);if(nil.isNil_($1$))return nil;return $1$;})())))){return false;};;
return isFileOlderThan(fileName,referenceFileName);}
,"Builders-emmiting",unescape("isFile%3A%20fileName%20olderThan%3A%20referenceFileName%0D%0A%09%22%20Private%20-%20Return%20true%20if%20the%20file%20at%20pathName%20is%20older%20than%20file%20at%20referenceFileName.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%28Smalltalk%20global%20basicAt%3A%20%23isFileOlderThan%29%20isNil%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%5E%23%20%23isFileOlderThan%3A%20fileName%20with%3A%20referenceFileName"));
smalltalk.bind(smalltalk.Object,"emmit:startingWith:",0
,function(fileName,initial){var $s8Ret$={name:"stReturn"};try{var self=this;
var output=nil;
var nativeFileName=nil;
var text=nil;
nativeFileName=fileName._comma(".")._comma(smalltalk.String.nativeFileExtension());
if (self.isFile_olderThan_(fileName,nativeFileName)){self.fileInJS_(nativeFileName);
return fileContents(nativeFileName);};;
output="".writeStream();
if ((nil.isNil_(initial)===false)){output.nextPutAll_(initial);};;
self.note_(unescape("emmit%28%27")._comma(fileName)._comma(unescape("%27%29")));
(function(){self.printingWith_do_((function(what){text=""._comma(what);return (function($rec){$rec.cr();return $rec.nextPutAll_(text);})(output);}),(function(){return self.fileIn_(fileName);}));text=output.contents();return ((text.size()._gt((0))).mustBeBoolean()==true ? (function(){return text.outputToFile_(nativeFileName);})() : nil);}).on_do_(smalltalk.Exception,(function(ex){var reason=nil;
reason=ex.toString();(function($rec){$rec.cr();return $rec.nextPutAll_(reason);})(output).contents().outputToFile_(fileName._comma(".error.txt"));self.print_(reason);return (function(){$s8Ret$.result=(function(){return reason})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return text})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"Builders-emmiting",unescape("emmit%3A%20fileName%20startingWith%3A%20initial%0D%0A%09%22%20FileIn%20the%20contents%20of%20file%20at%20fileName%20redirecting%20output%28emmited%20code%29%20to%20a%20native%20file.%20%22%0D%0A%0D%0A%09%7C%20output%20nativeFileName%20text%20%7C%0D%0A%09nativeFileName%20%3A%3D%20fileName%20%2C%24.%20%2CString%20nativeFileExtension.%0D%0A%09%28self%20isFile%3A%20fileName%20olderThan%3A%20nativeFileName%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20fileInJS%3A%20nativeFileName.%0D%0A%09%09%5E%23%20%23fileContents%3A%20nativeFileName%0D%0A%09%5D.%0D%0A%09output%20%3A%3D%20%27%27%20writeStream.%0D%0A%09initial%20notNil%20ifTrue%3A%20%5B%20output%20nextPutAll%3A%20initial%20%5D.%0D%0A%09self%20note%3A%20%27emmit%28%27%27%27%20%2CfileName%20%2C%27%27%27%29%27.%0D%0A%5B%09self%09printingWith%3A%20%5B%3Awhat%7C%0D%0A%09%09%09text%20%3A%3D%20%27%27%20%2Cwhat.%0D%0A%09%09%09output%20cr%3B%20nextPutAll%3A%20text%0D%0A%09%09%5D%20do%3A%20%5B%20self%20fileIn%3A%20fileName%20%5D.%0D%0A%09text%20%3A%3D%20output%20contents.%0D%0A%09text%20size%20%3E%200%20ifTrue%3A%20%5B%20text%20outputToFile%3A%20nativeFileName%20%5D.%0D%0A%5D%09on%3A%20Exception%20do%3A%20%5B%3Aex%7C%20%7C%20reason%20%7C%0D%0A%09reason%20%3A%3D%20ex%20toString.%0D%0A%09%28output%20cr%3B%20nextPutAll%3A%20reason%29%20contents%20outputToFile%3A%20fileName%20%2C%27.error.txt%27.%0D%0A%09self%20print%3A%20reason.%0D%0A%09%5Ereason%0D%0A%5D.%09%5Etext"));
smalltalk.bind(smalltalk.Object,"emmit:",0
,function(fileName){var self=this;
if (!(fileName.isString())){return fileName.asArray().emmit();};;
return self.emmit_startingWith_(fileName,nil);}
,"Builders-emmiting",unescape("emmit%3A%20fileName%0D%0A%09%22%20FileIn%20the%20contents%20of%20file%20at%20fileName%20redirecting%20output%28emmited%20code%29%20to%20a%20native%20file.%20%22%0D%0A%0D%0A%09fileName%20isString%20ifFalse%3A%20%5B%20%5EfileName%20asArray%20emmit%20%5D.%0D%0A%09%5Eself%20emmit%3A%20fileName%20startingWith%3A%20nil%20"));
smalltalk.bind(smalltalk.Object,"asJS",0
,function(){var self=this;
return smalltalk.NativeHandle._at(self);}
,"Natives-converting",unescape("asJS%0D%0A%09%22%20Convert%20the%20receiver%20to%20JS%20instance%20%28for%20reflection%29.%20%22%0D%0A%0D%0A%09%5ENativeHandle%20@%20self"));
smalltalk.bind(smalltalk.Object,"asNativeHandle",0
,function(){var self=this;
return smalltalk.NativeHandle._at(self);}
,"Natives-converting",unescape("asNativeHandle%0D%0A%09%22%20Convert%20the%20receiver%20to%20NativeHandle%20instance%20%28for%20reflection%29.%20%22%0D%0A%0D%0A%09%5ENativeHandle%20@%20self"));
smalltalk.bind(smalltalk.Object,"handleOf:",0
,function(nativeObject){var self=this;
return (((nil.isNil_(nativeObject)===false)).mustBeBoolean()==true ? (function(){return ((self.isObject_(nativeObject).and_((function(){return (nil.isNil_(nativeObject.handle)===false);}))).mustBeBoolean()==true ? (function(){return nativeObject.handle();})() : (function(){return nativeObject;})());})() : (function(){return smalltalk.NativeObject.undefined();})());}
,"Natives-converting",unescape("handleOf%3A%20nativeObject%0D%0A%09%22%20Private%20-%20Return%20the%20handle%20of%20nativeObject%20%28or%20the%20nativeObject%29.%20%22%0D%0A%0D%0A%09%5EnativeObject%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%28%28self%20isObject%3A%20nativeObject%29%0D%0A%09%09and%3A%20%5B%20nativeObject%5B%23handle%5D%20notNil%20%5D%29%0D%0A%09%09ifTrue%3A%20%5B%20nativeObject%20handle%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20nativeObject%20%5D%0D%0A%09%5D%20ifFalse%3A%20%5B%20NativeObject%20undefined%20%5D"));
smalltalk.bind(smalltalk.Object,"asLua",0
,function(){var self=this;
return self.asString();}
,"Lua-lua",unescape("asLua%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%0D%0A%09The%20default%20implementation%20return%20the%20receiver%20as%20a%20String.%0D%0A%09This%20method%20must%20be%20refined%20by%20subclasses%20implementing%20native%20literal%20representation.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20asString"));
smalltalk.bind(smalltalk.Object,"__tostring",0
,function(){var self=this;
return self.toString();}
,"Lua-printing",unescape("__tostring%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.Object,"fileInOBSOLETE:",0
,function(fileName){var self=this;
self.note_(unescape("fileIn%28%22")._comma(fileName)._comma(unescape("%22%29%3B")));
return fileIn(fileName);}
,"Lua-fileIn",unescape("fileInOBSOLETE%3A%20fileName%0D%0A%09%22%20FileIn%20contents%20of%20file%20at%20fileName.%20%22%0D%0A%0D%0A%09self%20note%3A%20%27fileIn%28%22%27%2C%20fileName%2C%20%27%22%29%3B%27.%0D%0A%09%5E%23%20%23fileIn%3A%20fileName"));
smalltalk.bind(smalltalk.Object,"fileInJSOBSOLETE:",0
,function(fileName){var self=this;
self.note_(unescape("fileInJS%28%22")._comma(fileName)._comma(unescape("%22%29%3B")));
return fileInJS(fileName);}
,"Lua-fileIn",unescape("fileInJSOBSOLETE%3A%20fileName%0D%0A%09%22%20Load%20native%20code%20from%20file%20at%20fileName.%20%22%0D%0A%0D%0A%09self%20note%3A%20%27fileInJS%28%22%27%2C%20fileName%2C%20%27%22%29%3B%27.%0D%0A%09%5E%23%20%23fileInJS%3A%20fileName"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"current",0
,function(){var self=this;
return smalltalk;}
,"accessing",unescape("current%0D%0A%09%22%20Return%20the%20current%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%7Bsmalltalk%7D%20"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"at:ifAbsent:",0
,function(aGlobalName,aBlock){var self=this;
return ((function(){var $1$=(self.current()[aGlobalName]);if(nil.isNil_($1$))return (aBlock.value());return $1$;})());}
,"accessing",unescape("at%3A%20aGlobalName%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20global%20known%20as%20aGlobalName%20in%20the%20receiver%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5Eself%20current%20basicAt%3A%20aGlobalName%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"includesKey:",0
,function(aGlobalName){var self=this;
return self.current().includesKey_(aGlobalName);}
,"testing",unescape("includesKey%3A%20aGlobalName%0D%0A%09%22%20Return%20true%20if%20the%20global%20aGlobalName%20is%20defined%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20current%20includesKey%3A%20aGlobalName"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"at:",0
,function(aGlobalName){var self=this;
return ((function(){var $1$=(self.current()[aGlobalName]);if(nil.isNil_($1$))return ((function(){return nil;}).value());return $1$;})());}
,"accessing",unescape("at%3A%20aGlobalName%0D%0A%09%22%20Return%20the%20global%20known%20as%20aGlobalName%20in%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20current%20basicAt%3A%20aGlobalName%20ifAbsent%3A%20%5B%20nil%20%5D"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"at:put:",0
,function(aGlobalName,anObject){var self=this;
return (self.current()[aGlobalName]=(anObject));}
,"accessing",unescape("at%3A%20aGlobalName%20put%3A%20anObject%0D%0A%09%22%20Set%20the%20global%20known%20as%20aGlobalName%20in%20the%20receiver%20top%20anObject.%20%22%0D%0A%0D%0A%09%5Eself%20current%20basicAt%3A%20aGlobalName%20put%3A%20anObject"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"at:ifAbsentPut:",0
,function(aGlobalName,aBlock){var self=this;
return self.at_ifAbsent_(aGlobalName,(function(){return self.at_put_(aGlobalName,aBlock.value());}));}
,"accessing",unescape("at%3A%20aGlobalName%20ifAbsentPut%3A%20aBlock%0D%0A%09%22%20Return%20the%20global%20known%20as%20aGlobalName%20in%20the%20receiver%20or%20the%20result%20of%20evaluating%20aBlock.%0D%0A%09Set%20the%20global%20to%20return%20value%20if%20was%20absent.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20at%3A%20aGlobalName%20ifAbsent%3A%20%5B%0D%0A%09%09self%20at%3A%20aGlobalName%20put%3A%20aBlock%20value%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"removeKey:ifAbsent:",0
,function(aKey,aBlock){var self=this;
return self.current().removeKey_ifAbsent_(aKey,aBlock);}
,"removing",unescape("removeKey%3A%20aKey%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Remove%20the%20binding%20at%20aKey.%20%22%0D%0A%0D%0A%09%5Eself%20current%20removeKey%3A%20aKey%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"@","_at"
,function(aGlobalName){var $s8Ret$={name:"stReturn"};try{var self=this;
var binding=nil;
binding=self.at_ifAbsent_(aGlobalName,(function(){return (function(){$s8Ret$.result=(function(){return smalltalk.Array.$$new()})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return smalltalk.Array.with_(binding)})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("@%20aGlobalName%0D%0A%09%22%20Return%20a%20collection%20with%20the%20objects%20bound%20to%20aGlobalName%20%28or%20an%20empty%20collection%20if%20global%20is%20not%20defined%29.%0D%0A%09It%20is%20used%20in%20expressions%20like%3A%0D%0A%09%09Smalltalk%20@%20%23MyGlobal%20do%3A%20%5B%3Aglobal%7C%20...%20%5D%0D%0A%09%22%0D%0A%0D%0A%09%7C%20binding%20%7C%0D%0A%09binding%20%3A%3D%20self%20at%3A%20aGlobalName%20ifAbsent%3A%20%5B%20%5EArray%20new%20%5D.%0D%0A%09%5EArray%20with%3A%20binding"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"definePool:with:",0
,function(poolName,tuples){var self=this;
return self.current().definePool_with_(poolName,tuples);}
,"accessing",unescape("definePool%3A%20poolName%20with%3A%20tuples%0D%0A%09%22%20Define%20the%20pool%20dictionary.%20%22%0D%0A%0D%0A%09%5Eself%20current%20definePool%3A%20poolName%20with%3A%20tuples"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"poolDictionary:ifAbsent:",0
,function(aPoolName,aBlock){var self=this;
return self.current().poolDictionary_ifAbsent_(aPoolName,aBlock);}
,"accessing",unescape("poolDictionary%3A%20aPoolName%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20pool%20dictionary%20named%20aPoolName%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5Eself%20current%20poolDictionary%3A%20aPoolName%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"objectAt:ifAbsent:",0
,function(aName,aBlock){var self=this;
return self.current().objectAt_ifAbsent_(aName,aBlock);}
,"accessing",unescape("objectAt%3A%20aName%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20%28global%20scoped%29%20smalltalk%20object%20at%20aName%20or%20the%20result%20of%20evaluating%20aBlock%20%28if%20undefined%20or%20not%20an%20Object%20instance%29.%20%22%0D%0A%0D%0A%09%5Eself%20current%20objectAt%3A%20aName%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"jsObjectAt:ifAbsent:",0
,function(aName,aBlock){var self=this;
self.note_("OBSOLETE"._comma(unescape("%20-%20jsObjectAt%3AifAbsent%3A%20")));
return self.current().nativeObjectAt_ifAbsent_(aName,aBlock);}
,"accessing",unescape("jsObjectAt%3A%20aName%20ifAbsent%3A%20aBlock%0D%0A%09%22%20OBSOLETE%20-%20Return%20the%20%28global%29%20object%20at%20aName%20or%20the%20result%20of%20evaluating%20aBlock%20%28if%20undefined%29.%20%22%0D%0A%0D%0Aself%20note%3A%20%23OBSOLETE%20%2C%27%20-%20jsObjectAt%3AifAbsent%3A%20%27.%0D%0A%09%5Eself%20current%20nativeObjectAt%3A%20aName%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"jsObjectAt:",0
,function(aName){var self=this;
self.note_("OBSOLETE"._comma(unescape("%20-%20jsObjectAt%3A%20")));
return self.nativeObjectAt_ifAbsent_(aName,(function(){return self.error_("Undefined "._comma(aName));}));}
,"accessing",unescape("jsObjectAt%3A%20aName%0D%0A%09%22%20OBSOLETE%20-%20Return%20the%20%28global%29%20object%20at%20aName.%20%22%0D%0A%0D%0Aself%20note%3A%20%23OBSOLETE%20%2C%27%20-%20jsObjectAt%3A%20%27.%0D%0A%09%5Eself%20nativeObjectAt%3A%20aName%20ifAbsent%3A%20%5B%20self%20error%3A%20%27Undefined%20%27%2CaName%20%5D"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"nativeObjectAt:ifAbsent:",0
,function(aName,aBlock){var self=this;
return self.current().nativeObjectAt_ifAbsent_(aName,aBlock);}
,"accessing",unescape("nativeObjectAt%3A%20aName%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20%28global%29%20object%20at%20aName%20or%20the%20result%20of%20evaluating%20aBlock%20%28if%20undefined%29.%20%22%0D%0A%0D%0A%09%5Eself%20current%20nativeObjectAt%3A%20aName%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"nativeObjectAt:",0
,function(aName){var self=this;
return self.nativeObjectAt_ifAbsent_(aName,(function(){return self.error_("Undefined "._comma(aName));}));}
,"accessing",unescape("nativeObjectAt%3A%20aName%0D%0A%09%22%20Return%20the%20%28global%29%20object%20at%20aName.%20%22%0D%0A%0D%0A%09%5Eself%20nativeObjectAt%3A%20aName%20ifAbsent%3A%20%5B%20self%20error%3A%20%27Undefined%20%27%2CaName%20%5D"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"global",0
,function(){var self=this;
return (function(){return this;}).call(null);}
,"accessing",unescape("global%0D%0A%09%22%20Return%20the%20global%20context%20object.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27get%20the%20global%20context%20object%27%7D"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"install:as:",0
,function(aBlockClosure,functionName){var self=this;
var old=nil;
old=((function(){var $1$=(self.global()[functionName]);if(nil.isNil_($1$))return nil;return $1$;})());
(self.global()[functionName]=(aBlockClosure));
return old;}
,"installing",unescape("install%3A%20aBlockClosure%20as%3A%20functionName%0D%0A%09%22%20Install%20the%20aBlockClosure%20as%20a%20function%20in%20global%20scope.%20%22%0D%0A%0D%0A%09%7C%20old%20%7C%0D%0A%09old%20%3A%3D%20self%20global%20basicAt%3A%20functionName.%0D%0A%09self%20global%20basicAt%3A%20functionName%20put%3A%20aBlockClosure.%0D%0A%09%5Eold"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"classes",0
,function(){var self=this;
return self.current().classes();}
,"accessing",unescape("classes%0D%0A%09%22%20Return%20the%20classes%20defined%20in%20current%20instance.%20%22%0D%0A%0D%0A%09%5Eself%20current%20classes"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"rootClasses",0
,function(){var self=this;
return self.current().rootClasses();}
,"accessing",unescape("rootClasses%0D%0A%09%22%20Return%20the%20collection%20of%20topLevel%20classes%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20current%20rootClasses"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"implementorsOf:",0
,function(aSymbol){var self=this;
return self.current().implementorsOf_(aSymbol);}
,"tools",unescape("implementorsOf%3A%20aSymbol%0D%0A%09%22%20Return%20the%20implementors%20of%20aSymbol.%20%22%0D%0A%0D%0A%09%5Eself%20current%20implementorsOf%3A%20aSymbol"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"isRunningOn:",0
,function(lang){var self=this;
return self.nativeLanguage()._eq(lang);}
,"environment",unescape("isRunningOn%3A%20lang%0D%0A%09%22%20Private%20-%20Return%20true%20when%20running%20hosted%20by%20native%20lang%20execution%20environment.%20%22%0D%0A%0D%0A%09%5Eself%20nativeLanguage%20%3D%20lang"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"gc",0
,function(){var self=this;
if(typeof gc!="undefined"){gc(); return true};
return false;}
,"tools",unescape("gc%0D%0A%09%22%20Collect%20garbage.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27forced%20full%20garbage%20collection%27%7D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"allMethodsDo:",0
,function(aBlock){var self=this;
self.classes().do_((function(each){each.methodsDo_(aBlock);return each.$klass.methodsDo_(aBlock);}));
return self;}
,"iterating",unescape("allMethodsDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20for%20all%20the%20methods%20in%20the%20current%20instance.%20%22%0D%0A%0D%0A%09self%20classes%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20methodsDo%3A%20aBlock.%0D%0A%09%09each%20class%20methodsDo%3A%20aBlock.%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"allSelectorsDo:",0
,function(aBlock){var self=this;
var found=nil;
found=smalltalk.HashedSet.$$new();
self.allMethodsDo_((function(mth){var msg=nil;
msg=mth.selector();return ((found.includes_(msg)).mustBeBoolean()==true ? nil : (function(){if ((nil.isNil_(aBlock)===false)){aBlock.value_(msg);};;return found.add_(msg);})());}));
return found;}
,"iterating",unescape("allSelectorsDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20for%20all%20the%20selectors%20defined%20in%20the%20current%20instance.%20%22%0D%0A%0D%0A%09%7C%20found%20%7C%0D%0A%09found%20%3A%3D%20HashedSet%20new.%0D%0A%09self%20allMethodsDo%3A%20%5B%3Amth%7C%20%7C%20msg%20%7C%0D%0A%09%09msg%20%3A%3D%20mth%20selector.%0D%0A%09%09%28found%20includes%3A%20msg%29%20ifFalse%3A%20%5B%0D%0A%09%09%09aBlock%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09%09aBlock%20value%3A%20msg%0D%0A%09%09%09%5D.%0D%0A%09%09%09found%20add%3A%20msg%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Efound"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"allSelectors",0
,function(){var self=this;
return self.allSelectorsDo_(nil).sorted();}
,"iterating",unescape("allSelectors%0D%0A%09%22%20Return%20all%20the%20selectors%20%28sorted%29%20defined%20in%20the%20current%20instance.%20%22%0D%0A%0D%0A%09%5E%28self%20allSelectorsDo%3A%20nil%29%20sorted"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"bindingsFor:",0
,function(aScope){var self=this;
return smalltalk.PoolDictionaryReader.$$new().scope_(aScope);}
,"Tools-variables",unescape("bindingsFor%3A%20aScope%0D%0A%09%22%20FileIn%20-%20Return%20a%20reader%20to%20fileIn%20bindings%20in%20a%20%28global%29%20scope.%0D%0A%09aScope%20can%20be%20a%20string%20or%20a%20dictionary.%0D%0A%09In%20case%20of%20aScope%20been%20a%20String%2C%20it%20will%20be%20considered%20a%20PoolDictionaryName%0D%0A%09See%20PoolDictionaryReader%3E%3E%23scanFrom%3A%0D%0A%09%22%0D%0A%0D%0A%09%5EPoolDictionaryReader%20new%20scope%3A%20aScope"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"sendersOf:",0
,function(selector){var self=this;
return self.current().sendersOf_(selector);}
,"Tools-query",unescape("sendersOf%3A%20selector%0D%0A%09%22%20Return%20the%20senders%20of%20message%20selector.%20%22%0D%0A%0D%0A%09%5Eself%20current%20sendersOf%3A%20selector"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"saveImageAs:",0
,function(fileName){var self=this;
self.current().saveImageAs_(fileName);
return self;}
,"Snapshot-snapshot",unescape("saveImageAs%3A%20fileName%0D%0A%09%22%20Dump%20the%20image%20contents%20onto%20file%20at%20fileName.%20%22%0D%0A%0D%0A%09self%20current%20saveImageAs%3A%20fileName"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"saveImage",0
,function(){var self=this;
self.current().saveImage();
return self;}
,"Snapshot-snapshot",unescape("saveImage%0D%0A%09%22%20Dump%20the%20image%20contents%20onto%20image%20file.%20%22%0D%0A%0D%0A%09self%20current%20saveImage"));
smalltalk.bind(smalltalk.Smalltalk.$klass,"exit",0
,function(){var self=this;
self.current().exit();
return self;}
,"Snapshot-exiting",unescape("exit%0D%0A%09%22%20Exit%20to%20OS%20system.%20%22%0D%0A%0D%0A%09self%20current%20exit"));
smalltalk.bind(smalltalk.Smalltalk,"globalAt:",0
,function(aKey){var self=this;
return ((function(){var $1$=(self[aKey]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("globalAt%3A%20aKey%0D%0A%09%22%20Return%20the%20global%20at%20aKey%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20aKey"));
smalltalk.bind(smalltalk.Smalltalk,"globalAt:ifAbsent:",0
,function(aKey,aBlock){var self=this;
return ((function(){var $1$=(self[aKey]);if(nil.isNil_($1$))return (aBlock.value());return $1$;})());}
,"accessing",unescape("globalAt%3A%20aKey%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20global%20at%20aKey%20%28or%20the%20result%20of%20evaluating%20aBlock%29.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20aKey%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk,"rootClasses",0
,function(){var self=this;
return self.classes().select_((function(each){return (nil.isNil_(each.superclass()));}));}
,"accessing",unescape("rootClasses%0D%0A%09%22%20Return%20the%20collection%20of%20topLevel%20classes%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20classes%20select%3A%20%5B%3Aeach%7C%20each%20superclass%20isNil%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"at:",0
,function(aGlobalName){var self=this;
return ((function(){var $1$=(self[aGlobalName]);if(nil.isNil_($1$))return ((function(){return nil;}).value());return $1$;})());}
,"accessing",unescape("at%3A%20aGlobalName%0D%0A%09%22%20Return%20the%20global%20known%20as%20aGlobalName%20in%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20aGlobalName%20ifAbsent%3A%20%5B%20nil%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"at:ifAbsent:",0
,function(aGlobalName,aBlock){var self=this;
return ((function(){var $1$=(self[aGlobalName]);if(nil.isNil_($1$))return (aBlock.value());return $1$;})());}
,"accessing",unescape("at%3A%20aGlobalName%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20value%20at%20aKey%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20aGlobalName%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk,"at:ifAbsentPut:",0
,function(aGlobalName,aBlock){var self=this;
return self.basicAt_ifAbsentPut_(aGlobalName,aBlock);}
,"accessing",unescape("at%3A%20aGlobalName%20ifAbsentPut%3A%20aBlock%0D%0A%09%22%20Return%20the%20global%20known%20as%20aGlobalName%20in%20the%20receiver%20or%20the%20result%20of%20evaluating%20aBlock.%0D%0A%09Set%20the%20global%20to%20return%20value%20if%20was%20absent.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20aGlobalName%20ifAbsentPut%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk,"at:put:",0
,function(aGlobalName,anObject){var self=this;
return (self[aGlobalName]=(anObject));}
,"accessing",unescape("at%3A%20aGlobalName%20put%3A%20anObject%0D%0A%09%22%20Set%20the%20global%20known%20as%20aGlobalName%20in%20the%20receiver%20top%20anObject.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20aGlobalName%20put%3A%20anObject"));
smalltalk.bind(smalltalk.Smalltalk,"includesKey:",0
,function(aGlobalName){var self=this;
var none=nil;
none=undefined;
return (nil.is_eqeq_(((function(){var $1$=(self[aGlobalName]);if(nil.isNil_($1$))return ((function(){return none;}).value());return $1$;})()),none)).not();}
,"testing",unescape("includesKey%3A%20aGlobalName%0D%0A%09%22%20Return%20true%20if%20the%20global%20aGlobalName%20is%20defined%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20none%20%7C%0D%0A%09none%20%3A%3D%20%7B%27%27%20doc%27absence%20sentinel%27%7D.%0D%0A%09%5E%28%28self%0D%0A%09%09basicAt%3A%20aGlobalName%20ifAbsent%3A%20%5Bnone%5D%29%0D%0A%09%09%3D%3D%20none%29%20not"));
smalltalk.bind(smalltalk.Smalltalk,"implementorsOf:",0
,function(aSymbol){var self=this;
var result=nil;
result=[];
self.rootClasses().do_((function(each){return result.addAll_(each.implementorsOf_(aSymbol));}));
return result;}
,"accessing",unescape("implementorsOf%3A%20aSymbol%0D%0A%09%22%20Return%20the%20implementors%20of%20aSymbol.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20%23%28%29.%0D%0A%09self%20rootClasses%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09result%20addAll%3A%20%28each%20implementorsOf%3A%20aSymbol%29%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Smalltalk,"objectAt:ifAbsent:",0
,function(aName,aBlock){var self=this;
var result=nil;
result=self.nativeObjectAt_ifAbsent_(aName,aBlock);
return (((nil.isNil_(result.yourself)===false)).mustBeBoolean()==true ? (function(){return result;})() : nil);}
,"accessing",unescape("objectAt%3A%20aName%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20%28global%20scoped%29%20smalltalk%20object%20at%20aName%20or%20the%20result%20of%20evaluating%20aBlock%20%28if%20undefined%20or%20not%20an%20Object%20instance%29.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20nativeObjectAt%3A%20aName%20ifAbsent%3A%20aBlock.%0D%0A%09%5Eresult%23yourself%20notNil%20ifTrue%3A%20%5B%20result%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"jsObjectAt:ifAbsent:",0
,function(aName,aBlock){var self=this;
self.note_("OBSOLETE"._comma(unescape("%20-%20jsObjectAt%3AifAbsent%3A%20")));
return self.nativeObjectAt_ifAbsent_(aName,aBlock);}
,"accessing",unescape("jsObjectAt%3A%20aName%20ifAbsent%3A%20aBlock%0D%0A%09%22%20OBSOLETE%20-%20Return%20the%20%28global%29%20object%20at%20aName%20or%20the%20result%20of%20evaluating%20aBlock%20%28if%20undefined%29.%20%22%0D%0A%0D%0Aself%20note%3A%20%23OBSOLETE%20%2C%27%20-%20jsObjectAt%3AifAbsent%3A%20%27.%0D%0A%09%5Eself%20nativeObjectAt%3A%20aName%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk,"jsObjectAt:",0
,function(aName){var self=this;
self.note_("OBSOLETE"._comma(unescape("%20-%20jsObjectAt%3A%20")));
return self.nativeObjectAt_ifAbsent_(aName,(function(){return self.error_("Undefined "._comma(aName));}));}
,"accessing",unescape("jsObjectAt%3A%20aName%0D%0A%09%22%20OBSOLETE%20-%20Return%20the%20%28global%29%20object%20at%20aName.%20%22%0D%0A%0D%0Aself%20note%3A%20%23OBSOLETE%20%2C%27%20-%20jsObjectAt%3A%20%27.%0D%0A%09%5Eself%20nativeObjectAt%3A%20aName%20ifAbsent%3A%20%5B%20self%20error%3A%20%27Undefined%20%27%2CaName%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"nativeObjectAt:ifAbsent:",0
,function(aName,aBlock){var self=this;
return ((function(){var $1$=(smalltalk.Smalltalk.global()[aName]);if(nil.isNil_($1$))return (aBlock.value());return $1$;})());}
,"accessing",unescape("nativeObjectAt%3A%20aName%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20%28global%29%20object%20at%20aName%20or%20the%20result%20of%20evaluating%20aBlock%20%28if%20undefined%29.%20%22%0D%0A%0D%0A%09%5ESmalltalk%20global%20basicAt%3A%20aName%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk,"nativeObjectAt:",0
,function(aName){var self=this;
return self.nativeObjectAt_ifAbsent_(aName,(function(){return self.error_("Undefined "._comma(aName));}));}
,"accessing",unescape("nativeObjectAt%3A%20aName%0D%0A%09%22%20Return%20the%20%28global%29%20object%20at%20aName.%20%22%0D%0A%0D%0A%09%5Eself%20nativeObjectAt%3A%20aName%20ifAbsent%3A%20%5B%20self%20error%3A%20%27Undefined%20%27%2CaName%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"removeKey:ifAbsent:",0
,function(aKey,aBlock){var $s8Ret$={name:"stReturn"};try{var self=this;
((function(){var $1$=(self[aKey]);if(nil.isNil_($1$))return ((function(){return (function(){$s8Ret$.result=(function(){return aBlock.value()})();throw($s8Ret$)})();}).value());return $1$;})());
self.basicDelete_(aKey);
(function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"removing",unescape("removeKey%3A%20aKey%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Remove%20the%20binding%20at%20aKey.%20%22%0D%0A%0D%0A%09self%20basicAt%3A%20aKey%20ifAbsent%3A%20%5B%20%5EaBlock%20value%20%5D.%0D%0A%09self%20basicDelete%3A%20aKey"));
smalltalk.bind(smalltalk.Smalltalk,"definePool:",0
,function(poolName){var self=this;
return self.definePool_with_(poolName,smalltalk.Array.$$new());}
,"accessing",unescape("definePool%3A%20poolName%0D%0A%09%22%20Define%20the%20pool%20dictionary%20%28if%20it%20is%20not%20defined%20yet%29.%20%22%0D%0A%0D%0A%09%5Eself%20definePool%3A%20poolName%20with%3A%20Array%20new"));
smalltalk.bind(smalltalk.Smalltalk,"poolDictionary:ifAbsent:",0
,function(aPoolName,aBlock){var self=this;
return ((function(){var $1$=(self[aPoolName]);if(nil.isNil_($1$))return ((function(){return aBlock.value();}).value());return $1$;})());}
,"accessing",unescape("poolDictionary%3A%20aPoolName%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20pool%20dictionary%20named%20aPoolName%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20aPoolName%20ifAbsent%3A%20%5B%20aBlock%20value%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"poolDictionary:",0
,function(aPoolName){var self=this;
return self.poolDictionary_ifAbsent_(aPoolName,(function(){return self.definePool_(aPoolName);}));}
,"accessing",unescape("poolDictionary%3A%20aPoolName%0D%0A%09%22%20Return%20the%20pool%20dictionary%20named%20aPoolName.%20%22%0D%0A%0D%0A%09%5Eself%20poolDictionary%3A%20aPoolName%20ifAbsent%3A%20%5B%0D%0A%09%09%22self%20error%3A%20%27Missing%20Pool%20dictionary%3A%20%27%2CaPoolName.%20%22%0D%0A%09%09self%20definePool%3A%20aPoolName%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Smalltalk,"printDefinePool:with:",0
,function(poolName,tuples){var self=this;
self.print_(unescape("smalltalk.definePool_with_%28%22")._comma(poolName)._comma(unescape("%22%2C%5B")));
tuples.do_((function(tuple){return ((tuple.isArray()).mustBeBoolean()==true ? (function(){return self.print_(unescape("%09%5B")._comma(tuple.first().printString())._comma(unescape("%2C"))._comma(tuple.last().asNative())._comma(unescape("%5D%2C")));})() : (function(){return self.print_(unescape("%09")._comma(tuple.asNative())._comma(unescape("%2C")));})());}));
self.print_(unescape("%09%5D%29%3B"));
return self;}
,"compiling",unescape("printDefinePool%3A%20poolName%20with%3A%20tuples%0D%0A%09%22%20DEBUG%20-%20Dump%20definition%20of%20the%20pool%20dictionary.%20%22%0D%0A%0D%0A%09self%20print%3A%20%27smalltalk.definePool_with_%28%22%27%2CpoolName%2C%27%22%2C%5B%27.%0D%0A%09tuples%20do%3A%20%5B%3Atuple%7C%0D%0A%09%09tuple%20isArray%20ifTrue%3A%20%5B%0D%0A%09%09%09self%20print%3A%20%27%09%5B%27%2Ctuple%20first%20printString%2C%27%2C%27%2Ctuple%20last%20asNative%2C%27%5D%2C%27%0D%0A%09%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09self%20print%3A%20%27%09%27%2Ctuple%20asNative%20%2C%27%2C%27%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09self%20print%3A%20%27%09%5D%29%3B%27."));
smalltalk.bind(smalltalk.Smalltalk,"definePool:with:",0
,function(poolName,tuples){var self=this;
var dictionary=nil;
var offset=nil;
dictionary=((function(){var $1$=(self[poolName]);if(nil.isNil_($1$))return ((function(){return (self[poolName]=(smalltalk.PoolDictionary.$$new()));}).value());return $1$;})());
offset=(0);
tuples.do_((function(each){return ((each.isArray()).mustBeBoolean()==true ? (function(){dictionary.at_put_(each.first(),each.last());return ((each.last().isNumber()).mustBeBoolean()==true ? (function(){return offset=each.last()._plus((1));})() : nil);})() : (function(){return ((each.isNumber()).mustBeBoolean()==true ? (function(){return offset=each;})() : (function(){dictionary.at_put_(each,offset);return offset=offset._plus((1));})());})());}));
return dictionary;}
,"compiling",unescape("definePool%3A%20poolName%20with%3A%20tuples%0D%0A%09%22%20Define%20the%20pool%20dictionary.%20%22%0D%0A%0D%0A%09%7C%20dictionary%20offset%20%7C%0D%0A%09dictionary%20%3A%3D%20self%20basicAt%3A%20poolName%20ifAbsent%3A%20%5B%0D%0A%09%09self%20basicAt%3A%20poolName%20put%3A%20PoolDictionary%20new%0D%0A%09%5D.%0D%0A%09offset%20%3A%3D%200.%0D%0A%09tuples%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20isArray%20ifTrue%3A%20%5B%0D%0A%09%09%09dictionary%20at%3A%20each%20first%20put%3A%20each%20last.%0D%0A%09%09%09each%20last%20isNumber%20ifTrue%3A%20%5B%20offset%20%3A%3D%20each%20last%20+%201%20%5D.%0D%0A%09%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09each%20isNumber%20ifTrue%3A%20%5B%20offset%20%3A%3D%20each%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%0D%0A%09%09%09%09dictionary%20at%3A%20each%20put%3A%20offset.%0D%0A%09%09%09%09offset%20%3A%3D%20offset%20+%201.%0D%0A%09%09%09%5D%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Edictionary"));
smalltalk.bind(smalltalk.Smalltalk,"state",0
,function(){var self=this;
return ((function(){var $1$=(self["$state"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("state%0D%0A%09%22%20Return%20the%20state%20of%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24state%27"));
smalltalk.bind(smalltalk.Smalltalk,"state:",0
,function(aState){var self=this;
(self["$state"]=(aState));
return self;}
,"accessing",unescape("state%3A%20aState%0D%0A%09%22%20Set%20the%20state%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20basicAt%3A%20%27%24state%27%20put%3A%20aState"));
smalltalk.bind(smalltalk.Smalltalk,"withState:do:",0
,function(aState,aBlock){var self=this;
var old=nil;
old=self.state();
return (function(){self.state_(aState);return aBlock.value();}).ensure_((function(){return self.state_(old);}));}
,"accessing",unescape("withState%3A%20aState%20do%3A%20aBlock%0D%0A%09%22%20Set%20the%20state%20of%20the%20receiver%20to%20aState%20during%20execution%20of%20aBlock.%20%22%0D%0A%0D%0A%09%7C%20old%20%7C%0D%0A%09old%20%3A%3D%20self%20state.%0D%0A%09%5E%5B%09self%20state%3A%20aState.%0D%0A%09%09aBlock%20value%0D%0A%09%5D%20ensure%3A%20%5B%0D%0A%09%09self%20state%3A%20old%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Smalltalk,"@","_at"
,function(keyOrPath){var $s8Ret$={name:"stReturn"};try{var self=this;
var result=nil;
if (!(keyOrPath.isArray())){return self.$klass.at_ifAbsent_(keyOrPath,(function(){return nil;}));};;
result=keyOrPath.inject_into_(self,(function(pool,key){var value=nil;
if ((nil.isNil_((value=pool._at(key))))){(function(){$s8Ret$.result=(function(){return nil})();throw($s8Ret$)})();};;return value;}));
(function(){$s8Ret$.result=(function(){return result})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("@%20keyOrPath%0D%0A%09%22%20Return%20the%20result%20of%20accessing%20the%20receiver%20at%20key%20or%20path%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09keyOrPath%20isArray%20ifFalse%3A%20%5B%20%5Eself%20class%20at%3A%20keyOrPath%20ifAbsent%3A%20%5B%5D%20%5D.%0D%0A%09result%20%3A%3D%20keyOrPath%20inject%3A%20self%20into%3A%20%5B%3Apool%20%3Akey%20%7C%20%7C%20value%20%7C%0D%0A%09%09%28value%20%3A%3D%20pool%20@%20key%29%20isNil%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%09value%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Smalltalk,"eval:",0
,function(anExpression){var self=this;
return eval(anExpression);
return self;}
,"eval",unescape("eval%3A%20anExpression%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20a%20native%20expression%20in%20global%20context.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27Native%20evaluation%20hook%27%7D"));
smalltalk.bind(smalltalk.Smalltalk,"eval:as:",0
,function(anExpression,aState){var self=this;
return self.withState_do_(aState,(function(){return self.eval_(anExpression);}));}
,"eval",unescape("eval%3A%20anExpression%20as%3A%20aState%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20a%20native%20expression%20in%20global%20context.%0D%0A%09The%20state%20of%20the%20system%20change%20to%20state%20while%20evaluating%20the%20expression.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20withState%3A%20aState%20do%3A%20%5B%20self%20eval%3A%20anExpression%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"capture:with:",0
,function(keyOrPath,aBlock){var self=this;
if ((nil.isNil_(aBlock))){return self.capture_with_(keyOrPath,(function(result){return result;}));};;
if ((nil.isNil_(keyOrPath))){return aBlock.evaluateWith_(nil);};;
if (!(keyOrPath.isString())){return self._at(keyOrPath);};;
if (keyOrPath.startsWith_(unescape("%21"))){return aBlock.evaluateWith_(keyOrPath.fileIn());};;
if (keyOrPath.startsWith_(unescape("@"))){return self._at(keyOrPath.asArrayOfSubstringsSeparatedBy_(unescape("@")));};;
if (keyOrPath.startsWith_(".")){return self._at(keyOrPath.asArrayOfSubstringsSeparatedBy_("."));};;
if (!(smalltalk.responds_to_(self,(keyOrPath)))){return aBlock.evaluateWith_(nil);};;
if (keyOrPath.argumentCount()._eq((1))){return self.perform_with_(keyOrPath,aBlock);};;
return aBlock.evaluateWith_(self.perform_(keyOrPath));}
,"tools",unescape("capture%3A%20keyOrPath%20with%3A%20aBlock%0D%0A%09%22%20Capture%20keyOrPath%20of%20the%20receiver%20and%20evaluate%20aBlock.%0D%0A%09Default%20action%20will%20try%20to%20capture%20the%20object%20found%20and%20%28potentially%29%20evaluate%20aBlock%20with%20the%20result.%20%22%0D%0A%0D%0A%09aBlock%20isNil%20ifTrue%3A%20%5B%20%5Eself%20capture%3A%20keyOrPath%20with%3A%20%5B%3Aresult%20%7C%20result%20%5D%20%5D.%0D%0A%09keyOrPath%20isNil%20ifTrue%3A%20%5B%20%5EaBlock%20evaluateWith%3A%20nil%20%5D.%0D%0A%09keyOrPath%20isString%20ifFalse%3A%20%5B%20%5Eself%20@%20keyOrPath%20%5D.%0D%0A%09%28keyOrPath%20startsWith%3A%20%24%21%29%20ifTrue%3A%20%5B%20%5EaBlock%20evaluateWith%3A%20keyOrPath%20fileIn%20%5D.%0D%0A%09%28keyOrPath%20startsWith%3A%20%24@%29%20ifTrue%3A%20%5B%20%5Eself%20@%20%28keyOrPath%20asArrayOfSubstringsSeparatedBy%3A%20%24@%29%20%5D.%0D%0A%09%28keyOrPath%20startsWith%3A%20%24.%29%20ifTrue%3A%20%5B%20%5Eself%20@%20%28keyOrPath%20asArrayOfSubstringsSeparatedBy%3A%20%24.%29%20%5D.%0D%0A%09%28self%20respondsTo%3A%20keyOrPath%29%20ifFalse%3A%20%5B%20%5EaBlock%20evaluateWith%3A%20nil%20%5D.%0D%0A%09keyOrPath%20argumentCount%20%3D%201%20ifTrue%3A%20%5B%20%5Eself%20perform%3A%20keyOrPath%20with%3A%20aBlock%20%5D.%0D%0A%09%5EaBlock%20evaluateWith%3A%20%28self%20perform%3A%20keyOrPath%29"));
smalltalk.bind(smalltalk.Smalltalk,"removeSources",0
,function(){var self=this;
self.classes().do_((function(each){return each.removeSources();}));
return self;}
,"tools",unescape("removeSources%0D%0A%09%22%20Remove%20all%20development%20sources%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20classes%20do%3A%20%5B%3Aeach%20%7C%20each%20removeSources%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"readJSON:",0
,function(anObject){var self=this;
return self.readJSObject(anObject);}
,"json",unescape("readJSON%3A%20anObject%0D%0A%09%22%20Return%20an%20object%20read%20from%20JSON%20map%20object%20anObject.%20%22%0D%0A%0D%0A%09%5Eself%20%23readJSObject%3A%20anObject"));
smalltalk.bind(smalltalk.Smalltalk,"shallowCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Smalltalk,"deepCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Smalltalk,"recompileAll",0
,function(){var self=this;
self.classes().do_((function(each){return each.recompile();}));
return self;}
,"Compiler-compiling",unescape("recompileAll%0D%0A%09%22%20Recompile%20all%20classes%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20classes%20do%3A%20%5B%3Aeach%20%7C%20each%20recompile%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"compilerSupport",0
,function(){var self=this;
return smalltalk.Compiler;}
,"Compiler-accessing",unescape("compilerSupport%0D%0A%09%22%20Return%20the%20compiler%20support%20for%20the%20receiver.%20%22%0D%0A%0D%0A%09%5ECompiler"));
smalltalk.bind(smalltalk.Smalltalk,"fileInCode:",0
,function(code){var self=this;
return (((nil.isNil_(code)===false)).mustBeBoolean()==true ? (function(){return code.stream().fileIn();})() : nil);}
,"Tools-fileIn",unescape("fileInCode%3A%20code%0D%0A%09%22%20FileIn%20the%20code%20into%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ecode%20notNil%20ifTrue%3A%20%5B%20code%20stream%20fileIn%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"methodReferences",0
,function(){var self=this;
return smalltalk.MethodReferences.$$new();}
,"Tools-accessing",unescape("methodReferences%0D%0A%09%22%20Return%20the%20methodReferences%20tool%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EMethodReferences%20new"));
smalltalk.bind(smalltalk.Smalltalk,"sendersOf:",0
,function(selector){var self=this;
return self.methodReferences().sendersOf_(selector);}
,"Tools-query",unescape("sendersOf%3A%20selector%0D%0A%09%22%20Return%20the%20senders%20of%20message%20selector.%20%22%0D%0A%0D%0A%09%5Eself%20methodReferences%20sendersOf%3A%20selector"));
smalltalk.bind(smalltalk.Smalltalk,"implementorsDo:",0
,function(aBlock){var self=this;
return self.classes().do_((function(each){return each.implementorsDo_(aBlock);}));}
,"Tools-iterating",unescape("implementorsDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20for%20each%20implementor%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20classes%20do%3A%20%5B%3Aeach%7C%20each%20implementorsDo%3A%20aBlock%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"implementors",0
,function(){var self=this;
var result=nil;
result=[];
self.implementorsDo_((function(each){return result.add_(each);}));
return result;}
,"Tools-implementors",unescape("implementors%0D%0A%09%22%20Return%20a%20collection%20with%20all%20implementors%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20%23%28%29.%0D%0A%09self%20implementorsDo%3A%20%5B%3Aeach%7C%20result%20add%3A%20each%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Smalltalk,"exit",0
,function(){var self=this;
self.classes().do_((function(each){return each.aboutToExit_(self);}));
quit();
return self;}
,"Snapshot-exiting",unescape("exit%0D%0A%09%22%20Exit%20to%20OS%20system.%20%22%0D%0A%0D%0A%09self%20classes%20do%3A%20%5B%3Aeach%7C%20each%20aboutToExit%3A%20self%20%5D.%0D%0A%09%7B%27%27%20doc%27quit%20to%20OS%27%7D."));
smalltalk.bind(smalltalk.Smalltalk,"product",0
,function(){var self=this;
return "S8";}
,"Snapshot-accessing",unescape("product%0D%0A%09%22%20Private%20-%20Return%20the%20name%20of%20the%20system.%20%22%0D%0A%0D%0A%09%5E%23S8"));
smalltalk.bind(smalltalk.Smalltalk,"description",0
,function(){var self=this;
return unescape("A%20modern%20Smalltalk%20environment%20running%20on%20the%20%22invisible%20machine%22.");}
,"Snapshot-accessing",unescape("description%0D%0A%09%22%20Return%20the%20description%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%27A%20modern%20Smalltalk%20environment%20running%20on%20the%20%22invisible%20machine%22.%27"));
smalltalk.bind(smalltalk.Smalltalk,"licenseComment",0
,function(){var self=this;
return unescape("%0D%0AThe%20text%20you%20are%20reading%20here%20is%20the%20residue%20of%20something%20that%20has%20run%20before.%0D%0AKeep%20the%20system%20running%20with%20responsibility.%20Smalltalk%20is%20not%20the%20words%2C%0D%0A%20nor%20any%20definition%2C%20nor%20it%20current%20contents...%0D%0ASmalltalk%20include%20your%20feelings%20when%20you%20see%20it%20running.%0D%0ASmalltalk%20is%20what%20happens%20when%20time%20flows%20and%20you%20%22are%22%20here.%0D%0AStop%20reading/writing%20code%20%28thinking%20you%20are%20outside%20the%20system%29%2C%0D%0A%20change%20your%20world%20with%20responsibility%2C%20one%20object%20at%20a%20time%2C%20with%20your%20hands%2C%0D%0A%20during%20all%20your%20life%2C%20in%20family.%0D%0A");}
,"Snapshot-license",unescape("licenseComment%0D%0A%09%22%20Private%20-%20Comment%20note%20about%20license%20of%20this%20product.%20%22%0D%0A%0D%0A%09%5E%27%0D%0AThe%20text%20you%20are%20reading%20here%20is%20the%20residue%20of%20something%20that%20has%20run%20before.%0D%0AKeep%20the%20system%20running%20with%20responsibility.%20Smalltalk%20is%20not%20the%20words%2C%0D%0A%20nor%20any%20definition%2C%20nor%20it%20current%20contents...%0D%0ASmalltalk%20include%20your%20feelings%20when%20you%20see%20it%20running.%0D%0ASmalltalk%20is%20what%20happens%20when%20time%20flows%20and%20you%20%22are%22%20here.%0D%0AStop%20reading/writing%20code%20%28thinking%20you%20are%20outside%20the%20system%29%2C%0D%0A%20change%20your%20world%20with%20responsibility%2C%20one%20object%20at%20a%20time%2C%20with%20your%20hands%2C%0D%0A%20during%20all%20your%20life%2C%20in%20family.%0D%0A%27"));
smalltalk.bind(smalltalk.Smalltalk,"licenseTerms",0
,function(){var self=this;
return self.product()._comma(unescape("%20-%20"))._comma(self.description())._comma(unescape("%0D%0A"))._comma(self.licenseMoreHolders())._comma(unescape("%0D%0A"))._comma(self.licenseHolders())._comma(unescape("%0D%0APermission%20is%20hereby%20granted%2C%20free%20of%20charge%2C%20to%20any%20person%20obtaining%0D%0A%20a%20copy%20of%20this%20software%20and%20associated%20documentation%20files%20%28the%20%22Software%22%29%2C%0D%0A%20to%20deal%20in%20the%20Software%20without%20restriction%2C%20including%20without%20limitation%0D%0A%20the%20rights%20to%20use%2C%20copy%2C%20modify%2C%20merge%2C%20publish%2C%20distribute%2C%20sublicense%2C%0D%0A%20and/or%20sell%20copies%20of%20the%20Software%2C%20and%20to%20permit%20persons%20to%20whom%20the%0D%0A%20Software%20is%20furnished%20to%20do%20so%2C%20subject%20to%20the%20following%20conditions%3A%0D%0A%0D%0AThe%20above%20copyright%20notice%20and%20this%20permission%20notice%20shall%20be%20included%0D%0A%20in%20all%20copies%20or%20substantial%20portions%20of%20the%20Software.%0D%0A%0D%0ATHE%20SOFTWARE%20IS%20PROVIDED%20%22AS%20IS%22%2C%20WITHOUT%20WARRANTY%20OF%20ANY%20KIND%2C%0D%0A%20EXPRESS%20OR%20IMPLIED%2C%20INCLUDING%20BUT%20NOT%20LIMITED%20TO%20THE%20WARRANTIES%0D%0A%20OF%20MERCHANTABILITY%2C%20FITNESS%20FOR%20A%20PARTICULAR%20PURPOSE%20AND%20NONINFRINGEMENT.%0D%0A%20IN%20NO%20EVENT%20SHALL%20THE%20AUTHORS%20OR%20COPYRIGHT%20HOLDERS%20BE%20LIABLE%20FOR%20ANY%0D%0A%20CLAIM%2C%20DAMAGES%20OR%20OTHER%20LIABILITY%2C%20WHETHER%20IN%20AN%20ACTION%20OF%20CONTRACT%2C%0D%0A%20TORT%20OR%20OTHERWISE%2C%20ARISING%20FROM%2C%20OUT%20OF%20OR%20IN%20CONNECTION%20WITH%20THE%20SOFTWARE%0D%0A%20OR%20THE%20USE%20OR%20OTHER%20DEALINGS%20IN%20THE%20SOFTWARE.%0D%0A%0D%0A"))._comma(self.licenseComment());}
,"Snapshot-license",unescape("licenseTerms%0D%0A%09%22%20Private%20-%20Return%20the%20license%20terms%20to%20use%20this%20system.%20%22%0D%0A%0D%0A%09%5Eself%20product%20%2C%27%20-%20%27%20%2Cself%20description%20%2C%27%0D%0A%27%20%2Cself%20licenseMoreHolders%20%2C%27%0D%0A%27%20%2Cself%20licenseHolders%20%2C%27%0D%0APermission%20is%20hereby%20granted%2C%20free%20of%20charge%2C%20to%20any%20person%20obtaining%0D%0A%20a%20copy%20of%20this%20software%20and%20associated%20documentation%20files%20%28the%20%22Software%22%29%2C%0D%0A%20to%20deal%20in%20the%20Software%20without%20restriction%2C%20including%20without%20limitation%0D%0A%20the%20rights%20to%20use%2C%20copy%2C%20modify%2C%20merge%2C%20publish%2C%20distribute%2C%20sublicense%2C%0D%0A%20and/or%20sell%20copies%20of%20the%20Software%2C%20and%20to%20permit%20persons%20to%20whom%20the%0D%0A%20Software%20is%20furnished%20to%20do%20so%2C%20subject%20to%20the%20following%20conditions%3A%0D%0A%0D%0AThe%20above%20copyright%20notice%20and%20this%20permission%20notice%20shall%20be%20included%0D%0A%20in%20all%20copies%20or%20substantial%20portions%20of%20the%20Software.%0D%0A%0D%0ATHE%20SOFTWARE%20IS%20PROVIDED%20%22AS%20IS%22%2C%20WITHOUT%20WARRANTY%20OF%20ANY%20KIND%2C%0D%0A%20EXPRESS%20OR%20IMPLIED%2C%20INCLUDING%20BUT%20NOT%20LIMITED%20TO%20THE%20WARRANTIES%0D%0A%20OF%20MERCHANTABILITY%2C%20FITNESS%20FOR%20A%20PARTICULAR%20PURPOSE%20AND%20NONINFRINGEMENT.%0D%0A%20IN%20NO%20EVENT%20SHALL%20THE%20AUTHORS%20OR%20COPYRIGHT%20HOLDERS%20BE%20LIABLE%20FOR%20ANY%0D%0A%20CLAIM%2C%20DAMAGES%20OR%20OTHER%20LIABILITY%2C%20WHETHER%20IN%20AN%20ACTION%20OF%20CONTRACT%2C%0D%0A%20TORT%20OR%20OTHERWISE%2C%20ARISING%20FROM%2C%20OUT%20OF%20OR%20IN%20CONNECTION%20WITH%20THE%20SOFTWARE%0D%0A%20OR%20THE%20USE%20OR%20OTHER%20DEALINGS%20IN%20THE%20SOFTWARE.%0D%0A%0D%0A%27%20%2Cself%20licenseComment"));
smalltalk.bind(smalltalk.Smalltalk,"licenseYear",0
,function(){var self=this;
return (function(){return smalltalk.DateTime.today().year().toString();}).on_do_(smalltalk.Exception,(function(){return "today";}));}
,"Snapshot-license",unescape("licenseYear%0D%0A%09%22%20Private%20-%20Return%20the%20year%20license%20information.%20%22%0D%0A%0D%0A%09%5E%5B%20DateTime%20today%20year%20toString%0D%0A%09%5D%20on%3A%20Exception%20do%3A%20%5B%20%23today%20%5D%20"));
smalltalk.bind(smalltalk.Smalltalk,"licenseMoreHolders",0
,function(){var self=this;
return "";}
,"Snapshot-license",unescape("licenseMoreHolders%0D%0A%09%22%20Private%20-%20Return%20additional%20license%20holders%20information.%20%22%0D%0A%0D%0A%09%5E%27%27"));
smalltalk.bind(smalltalk.Smalltalk,"licenseHolders",0
,function(){var self=this;
return unescape("Copyright%20%28C%29%20")._comma(self.licenseYear().asString())._comma(" ")._comma(unescape("Alejandro%20F.%20Reimondo%20%3CaleReimondo@smalltalking.net%3E%0D%0A%0D%0AParts%20of%20source%20code%20written%20for%20Jtalk%20%28Sept%202011%29%2C%0D%0ACopyright%20%28C%29%202011%20by%20Nicolas%20Petton%20%3Cpetton.nicolas@gmail.com%3E%0D%0ACopyright%20%28C%29%202011%20Jtalk%20contributors%0D%0A%09David%20Graham%20%3Cdgraham@iphouse.com%3E%0D%0A%09Goran%20Krampe%20%3Cgoran@krampe.se%3E%0D%0AAlso%20code%20and%20ideas%20from%20Clamato%20%28http%3A//clamato.net%29%2C%20written%20by%20Avi%20Byrant.%0D%0AThe%20PetitParser%20library%2C%20published%20by%20Lukas%20Renggli%20%28http%3A//lukas-renggli.ch%29%20and%20released%20under%20the%20MIT%20license.%0D%0AAnd%20people%20that%20contributed%20to%20Smalltalk%20on%20diverse%20media%20years%28decades%29%20before%20this%20license%20holders%20claimed%20ownership.%0D%0A"));}
,"Snapshot-license",unescape("licenseHolders%0D%0A%09%22%20Private%20-%20Return%20the%20license%20holders%20information.%20%22%0D%0A%0D%0A%09%5E%27Copyright%20%28C%29%20%27%0D%0A%09%2Cself%20licenseYear%20asString%20%2C%27%20%27%0D%0A%09%2C%27Alejandro%20F.%20Reimondo%20%3CaleReimondo@smalltalking.net%3E%0D%0A%0D%0AParts%20of%20source%20code%20written%20for%20Jtalk%20%28Sept%202011%29%2C%0D%0ACopyright%20%28C%29%202011%20by%20Nicolas%20Petton%20%3Cpetton.nicolas@gmail.com%3E%0D%0ACopyright%20%28C%29%202011%20Jtalk%20contributors%0D%0A%09David%20Graham%20%3Cdgraham@iphouse.com%3E%0D%0A%09Goran%20Krampe%20%3Cgoran@krampe.se%3E%0D%0AAlso%20code%20and%20ideas%20from%20Clamato%20%28http%3A//clamato.net%29%2C%20written%20by%20Avi%20Byrant.%0D%0AThe%20PetitParser%20library%2C%20published%20by%20Lukas%20Renggli%20%28http%3A//lukas-renggli.ch%29%20and%20released%20under%20the%20MIT%20license.%0D%0AAnd%20people%20that%20contributed%20to%20Smalltalk%20on%20diverse%20media%20years%28decades%29%20before%20this%20license%20holders%20claimed%20ownership.%0D%0A%27"));
smalltalk.bind(smalltalk.Smalltalk,"globalsDo:",0
,function(aBlock){var self=this;
return self.globalsDo(aBlock);}
,"Snapshot-evaluating",unescape("globalsDo%3A%20aBlock%0D%0A%09%22%20Private%20-%20Evaluates%20aBlock%20for%20each%20globals%20defined%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20%23globalsDo%3A%20aBlock"));
smalltalk.bind(smalltalk.Smalltalk,"globalsDo:separatedBy:",0
,function(aBlock,anotherBlock){var self=this;
var first=nil;
first=true;
return self.globalsDo((function(each){if (first){first=false;} else{anotherBlock.value();};;return aBlock.value_(each);}));}
,"Snapshot-evaluating",unescape("globalsDo%3A%20aBlock%20separatedBy%3A%20anotherBlock%0D%0A%09%22%20Private%20-%20Evaluates%20aBlock%20for%20each%20globals%20defined%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20first%20%7C%20first%20%3A%3D%20true.%0D%0A%09%5Eself%20%23globalsDo%3A%20%5B%3Aeach%7C%0D%0A%09%20%20%20%20first%0D%0A%09%09ifTrue%3A%20%5B%20first%20%3A%3D%20false%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20anotherBlock%20value%20%5D.%0D%0A%09%20%20%20%20aBlock%20value%3A%20each%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Smalltalk,"saveImageOn:",0
,function(stream){var self=this;
var exporter=nil;
var allClasses=nil;
(function($rec){$rec.nextPutAll_(self.licenseTerms().asComment());return $rec.cr();})(stream);
allClasses=self.rootClasses().inject_into_([],(function(total,each){total.addAll_(each.withAllSubclasses());return total;})).collect_((function(each){return each.aboutToSaveImage_on_(self,stream);})).select_((function(each){return (nil.isNil_(each)===false);}));
exporter=smalltalk.Exporter.$$new();
(function($rec){$rec.cr();$rec.nextPutAll_("Smalltalk Classes.".asComment());return $rec.cr();})(stream);
exporter.exportClasses_on_(allClasses,stream);
(function($rec){$rec.cr();return $rec.nextPutAll_(unescape("Smalltalk%20Globals%20set-up.").asComment());})(stream);
self.globalsDo_((function(key){return (function($rec){$rec.cr();return $rec.nextPutAll_("smalltalk."._comma(key)._comma(unescape("%3D"))._comma(exporter.exportObject_(((function(){var $1$=(self[key]);if(nil.isNil_($1$))return nil;return $1$;})()))));})(stream);}));
allClasses.do_((function(each){return each.imageSaved_on_(self,stream);}));
return self;}
,"Snapshot-writing",unescape("saveImageOn%3A%20stream%0D%0A%09%22%20Dump%20the%20receiver%27s%20contents%20onto%20stream.%20%22%0D%0A%0D%0A%09%7C%20exporter%20allClasses%20%7C%0D%0A%09stream%20nextPutAll%3A%20self%20licenseTerms%20asComment%3B%20cr.%0D%0A%09allClasses%20%3A%3D%20%28%28self%20rootClasses%0D%0A%09%09inject%3A%20%23%28%29%20into%3A%20%5B%3Atotal%20%3Aeach%7C%20total%20addAll%3A%20each%20withAllSubclasses.%20total%20%5D%29%0D%0A%09%09collect%3A%20%5B%3Aeach%7C%20each%20aboutToSaveImage%3A%20self%20on%3A%20stream%20%5D%29%0D%0A%09%09select%3A%20%5B%3Aeach%7C%20each%20notNil%20%5D.%0D%0A%09exporter%20%3A%3D%20Exporter%20new.%0D%0A%22%09stream%20nextPutAll%3A%20%27Smalltalk%20Globals%20%28pre%29definition.%27%20asComment%3B%20cr.%0D%0A%09self%0D%0A%09%09globalsDo%3A%20%5B%3Akey%7C%20stream%20nextPutAll%3A%20%27smalltalk.%27%2Ckey%20%5D%0D%0A%09%09separatedBy%3A%20%5B%20stream%20nextPutAll%3A%20%27%20%2C%20%27%20%5D.%0D%0A%09stream%20nextPutAll%3A%20%27%3Dnil%27%3B%20cr.%0D%0A%22%09stream%20cr%3B%20nextPutAll%3A%20%27Smalltalk%20Classes.%27%20asComment%3B%20cr.%0D%0A%09exporter%20exportClasses%3A%20allClasses%20on%3A%20stream.%0D%0A%09stream%20cr%3B%20nextPutAll%3A%20%27Smalltalk%20Globals%20set-up.%27%20asComment.%0D%0A%09self%20globalsDo%3A%20%5B%3Akey%7C%0D%0A%09%09stream%20cr%3B%20nextPutAll%3A%20%27smalltalk.%27%2Ckey%2C%27%3D%27%0D%0A%09%09%09%2C%28exporter%20exportObject%3A%20%28self%20basicAt%3A%20key%29%29%0D%0A%09%5D.%0D%0A%09allClasses%20do%3A%20%5B%3Aeach%7C%20each%20imageSaved%3A%20self%20on%3A%20stream%20%5D."));
smalltalk.bind(smalltalk.Smalltalk,"imageContents",0
,function(){var self=this;
var stream=nil;
stream=smalltalk.String.$$new().writeStream();
self.saveImageOn_(stream);
return stream.contents();}
,"Snapshot-contents",unescape("imageContents%0D%0A%09%22%20Return%20a%20dump%20of%20the%20system%20contents.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20String%20new%20writeStream.%0D%0A%09self%20saveImageOn%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Smalltalk,"snapshot",0
,function(){var self=this;
return smalltalk.Snapshot.contents();}
,"Snapshot-contents",unescape("snapshot%0D%0A%09%22%20Return%20a%20system%20snapshot.%20%22%0D%0A%0D%0A%09%5ESnapshot%20contents"));
smalltalk.bind(smalltalk.Smalltalk,"imageFileName",0
,function(){var self=this;
return ((function(){var $1$=(self["$imageFileName"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"Snapshot-fileName",unescape("imageFileName%0D%0A%09%22%20Return%20the%20image%20file%20name.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24imageFileName%27"));
smalltalk.bind(smalltalk.Smalltalk,"imageFileName:",0
,function(fileName){var self=this;
return (self["$imageFileName"]=(fileName));}
,"Snapshot-fileName",unescape("imageFileName%3A%20fileName%0D%0A%09%22%20Set%20the%20image%20file%20name.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24imageFileName%27%20put%3A%20fileName"));
smalltalk.bind(smalltalk.Smalltalk,"saveImage",0
,function(){var self=this;
self.saveImageAs_(self.imageFileName());
return self;}
,"Snapshot-save",unescape("saveImage%0D%0A%09%22%20Dump%20the%20image%20contents%20onto%20default%20image%20file.%20%22%0D%0A%0D%0A%09self%20saveImageAs%3A%20self%20imageFileName"));
smalltalk.bind(smalltalk.Smalltalk,"saveImageAs:",0
,function(fileName){var self=this;
smalltalk.String.doItSelector().implementors().do_((function(each){return each.removeFromSystem();}));
self.imageContents().outputToFile_(fileName);
self.imageFileName_(fileName);
return self;}
,"Snapshot-save",unescape("saveImageAs%3A%20fileName%0D%0A%09%22%20Dump%20the%20image%20contents%20onto%20file%20at%20fileName.%20%22%0D%0A%0D%0A%09String%20doItSelector%20implementors%20do%3A%20%5B%3Aeach%7C%20each%20removeFromSystem%20%5D.%0D%0A%09self%20imageContents%20outputToFile%3A%20fileName.%0D%0A%09self%20imageFileName%3A%20fileName."));
smalltalk.bind(smalltalk.Smalltalk,"launch",0
,function(){var self=this;
self.classes().do_((function(each){return each.launch();}));
return smalltalk.Smalltalk.at_ifAbsent_("LaunchExpression",(function(){return nil;})).evaluate();}
,"Snapshot-startup",unescape("launch%0D%0A%09%22%20The%20system%20has%20been%20launched%20and%20start%20execution.%0D%0A%09The%20default%20action%20is%20to%20propagate%20%23launch%20to%20current%20classes%20and%20evaluate%20%23LaunchExpression.%0D%0A%09%22%0D%0A%0D%0A%09self%20classes%20do%3A%20%5B%3Aeach%7C%20each%20launch%20%5D.%0D%0A%09%5E%28Smalltalk%20at%3A%20%23LaunchExpression%20ifAbsent%3A%20%5B%5D%29%20evaluate"));
smalltalk.bind(smalltalk.Smalltalk,"buildString",0
,function(){var self=this;
return unescape("Tue%20Feb%2013%202024%2007%3A30%3A19%20GMT-0300");}
,"version",unescape("buildString%0D%0A%09%22%20Return%20the%20birth%20information%20of%20this%20system.%20%22%0D%0A%0D%0A%09%5E%27Tue%20Feb%2013%202024%2007%3A30%3A19%20GMT-0300%27"));
smalltalk.bind(smalltalk.Smalltalk,"defaultWorkspaceList",0
,function(){var self=this;
var result=nil;
result=[];
if (smalltalk.Smalltalk.includesKey_("WorkspaceList")){result.addAll_(smalltalk.Smalltalk.at_("WorkspaceList"));};;
if (smalltalk.Smalltalk.includesKey_("U8")){result.addAll_(smalltalk.U8.commonExpressions());};;
return result;}
,"platform-tools",unescape("defaultWorkspaceList%0D%0A%09%22%20Private%20-%20Return%20the%20%28captured%29%20workspace%20list%20of%20the%20receiver.%0D%0A%09Return%20an%20array%20of%20tuples%3B%20each%20tuple%20is%20anArray%20of%20the%20form%0D%0A%09%09%23%28%20title%20code%20%29%0D%0A%09where%3A%0D%0A%09%09title%20is%20aString%20to%20be%20shown%20as%20a%20title%20of%20the%20workspace%20page.%0D%0A%09%09code%20is%20the%20text%20to%20be%20shown%20as%20contents%20of%20workspace%20page.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20%23%28%29.%0D%0A%09%28Smalltalk%20includesKey%3A%20%23WorkspaceList%29%20ifTrue%3A%20%5B%0D%0A%09%09result%20addAll%3A%20%28Smalltalk%20at%3A%20%23WorkspaceList%29%0D%0A%09%5D.%0D%0A%09%28Smalltalk%20includesKey%3A%20%23U8%29%20ifTrue%3A%20%5B%0D%0A%09%09result%20addAll%3A%20U8%20commonExpressions%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Smalltalk,"workspaceList",0
,function(){var self=this;
return self.defaultWorkspaceList();}
,"platform-tools",unescape("workspaceList%0D%0A%09%22%20Return%20the%20%28captured%29%20workspace%20list%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20defaultWorkspaceList"));
smalltalk.bind(smalltalk.Smalltalk,"defaultAppList",0
,function(){var self=this;
var result=nil;
result=[];
if (smalltalk.Smalltalk.includesKey_("AppList")){result.addAll_(smalltalk.Smalltalk.at_("AppList"));};;
if (smalltalk.Smalltalk.includesKey_("U8Configuration")){result.addAll_(smalltalk.U8Configuration.appList());};;
result.addAll_(self.allClassMethodsList_(["sample", "samples"]));
return result;}
,"platform-tools",unescape("defaultAppList%0D%0A%09%22%20Private%20-%20Return%20the%20%28captured%29%20apps%20of%20the%20receiver.%0D%0A%09Return%20an%20array%20of%20tuples%3B%20each%20tuple%20is%20anArray%20of%20the%20form%0D%0A%09%09%23%28%20title%20comment%20code%20%29%0D%0A%09where%3A%0D%0A%09%09title%20is%20aString%20that%20can%20include%20an%20icon%20name%20inside%20brackets.%0D%0A%09%09comment%20is%20aString%20with%20short%20description%20of%20the%20app.%0D%0A%09%09code%20is%20the%20code%20to%20evaluate%20to%20open%20the%20app.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20%23%28%29.%0D%0A%09%28Smalltalk%20includesKey%3A%20%23AppList%29%20ifTrue%3A%20%5B%0D%0A%09%09result%20addAll%3A%20%28Smalltalk%20at%3A%20%23AppList%29%0D%0A%09%5D.%0D%0A%09%28Smalltalk%20includesKey%3A%20%23U8Configuration%29%20ifTrue%3A%20%5B%0D%0A%09%09result%20addAll%3A%20U8Configuration%20appList%0D%0A%09%5D.%0D%0A%09result%20addAll%3A%20%28self%20allClassMethodsList%3A%20%23%28sample%20samples%29%29.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Smalltalk,"appList",0
,function(){var self=this;
return self.defaultAppList();}
,"platform-tools",unescape("appList%0D%0A%09%22%20Return%20the%20%28captured%29%20apps%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20defaultAppList"));
smalltalk.bind(smalltalk.Smalltalk,"allClassMethodsList:",0
,function(choices){var self=this;
var result=nil;
result=smalltalk.Array.$$new();
self.classes().do_((function(cls){return cls.$klass.methodsDo_((function(mth){var comment=nil;
return ((mth.selector().argumentCount()._eq((0)).and_((function(){return choices.includes_(mth.category()).or_((function(){return choices.includes_(mth.selector());})).and_((function(){comment=mth.comment();return comment.asString().upTo_(unescape("-")).asLowercase().includes_("private").not();}));}))).mustBeBoolean()==true ? (function(){var title=nil;
var code=nil;
title=code=cls.name()._comma(" ")._comma(mth.selector());return result.add_(smalltalk.Array.with_with_with_(title,comment.upTo_("."),code));})() : nil);}));}));
return result.sorted_((function(a,b){return a.first()._lt(b.first());}));}
,"platform-tools",unescape("allClassMethodsList%3A%20choices%0D%0A%09%22%20Private%20-%20Return%20the%20list%20of%20all%20%28class%29%20methods%20in%20the%20receiver%20matching%20choices.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20Array%20new.%0D%0A%09self%20classes%20do%3A%20%5B%3Acls%7C%0D%0A%09%09cls%20class%20methodsDo%3A%20%5B%3Amth%7C%20%7C%20comment%20%7C%0D%0A%09%09%09%28mth%20selector%20argumentCount%20%3D%200%20and%3A%20%5B%0D%0A%09%09%09%28%28choices%20includes%3A%20mth%20category%29%20or%3A%20%5B%0D%0A%09%09%09%20%20choices%20includes%3A%20mth%20selector%0D%0A%09%09%09%5D%29%20and%3A%20%5B%20comment%20%3A%3D%20mth%20comment.%0D%0A%09%09%09%20%28%28comment%20asString%20upTo%3A%20%24-%29%20asLowercase%20includes%3A%20%23private%29%20not%0D%0A%09%09%09%5D%5D%29%20ifTrue%3A%20%5B%20%7C%20title%20code%20%7C%0D%0A%09%09%09%09title%20%3A%3D%20code%20%3A%3D%20cls%20name%20%2C%24%20%20%2Cmth%20selector.%0D%0A%09%09%09%09result%20add%3A%20%28Array%0D%0A%09%09%09%09%09with%3A%20title%20with%3A%20%28comment%20upTo%3A%20%24.%29%0D%0A%09%09%09%09%09with%3A%20code%29%0D%0A%09%09%09%5D.%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5Eresult%20sorted%3A%20%5B%3Aa%20%3Ab%7C%20a%20first%20%3C%20b%20first%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"exampleList",0
,function(){var self=this;
return self.allClassMethodsList_(["example"]);}
,"platform-tools",unescape("exampleList%0D%0A%09%22%20Return%20the%20%28captured%29%20examples%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20allClassMethodsList%3A%20%23%28example%29"));
smalltalk.bind(smalltalk.Smalltalk,"screen:",0
,function(aBlock){var self=this;
return "S8 system running as console application";}
,"platform-tools",unescape("screen%3A%20aBlock%0D%0A%09%22%20Return%20the%20%28captured%29%20screen%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%27S8%20system%20running%20as%20console%20application%27"));
smalltalk.bind(smalltalk.Smalltalk,"fileContentsAt:",0
,function(pathName){var self=this;
return (function(){return pathName.fileContents();}).on_do_(smalltalk.Exception,(function(ex){return nil;}));}
,"platform-tools",unescape("fileContentsAt%3A%20pathName%0D%0A%09%22%20Return%20the%20contents%20of%20file%20at%20pathName%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5E%5B%20pathName%20fileContents%20%5D%20on%3A%20Exception%20do%3A%20%5B%3Aex%7C%20nil%20%5D"));
smalltalk.bind(smalltalk.Smalltalk,"output:toFile:",0
,function(aString,pathName){var self=this;
return aString.outputToFile_(pathName);}
,"platform-tools",unescape("output%3A%20aString%20toFile%3A%20pathName%0D%0A%09%22%20Save%20aString%20onto%20a%20file%20at%20pathName.%20%22%0D%0A%0D%0A%09%5EaString%20outputToFile%3A%20pathName"));
smalltalk.bind(smalltalk.Smalltalk,"append:toFile:",0
,function(aString,pathName){var self=this;
return aString.appendToFile_(pathName);}
,"platform-tools",unescape("append%3A%20aString%20toFile%3A%20pathName%0D%0A%09%22%20Append%20aString%20onto%20a%20file%20contents%20at%20pathName.%20%22%0D%0A%0D%0A%09%5EaString%20appendToFile%3A%20pathName"));
smalltalk.bind(smalltalk.Behavior,"new","$$new"
,function(){var self=this;
return self.basicNew().initialize();}
,"instantiation",unescape("new%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicNew%20initialize"));
smalltalk.bind(smalltalk.Behavior,"new:",0
,function(size){var self=this;
return self.basicNew_(size);}
,"instantiation",unescape("new%3A%20size%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20of%20specified%20size.%20%22%0D%0A%0D%0A%09%5Eself%20basicNew%3A%20size"));
smalltalk.bind(smalltalk.Behavior,"basicNew:",0
,function(size){var self=this;
return self.invalidMessage_("basicNew:");}
,"instantiation",unescape("basicNew%3A%20size%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20of%20specified%20size.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23basicNew%3A"));
smalltalk.bind(smalltalk.Behavior,"basicNew",0
,function(){var self=this;
return new self.$fn();}
,"instantiation",unescape("basicNew%0D%0A%09%22%20Return%20a%20new%20%28non-initialized%29%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20instantiation%20of%20class%27%7D"));
smalltalk.bind(smalltalk.Behavior,"name",0
,function(){var self=this;
return ((function(){var $1$=(self["$className"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("name%0D%0A%09%22%20Return%20the%20name%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24className%27"));
smalltalk.bind(smalltalk.Behavior,"superclass",0
,function(){var self=this;
return ((function(){var $1$=(self["$superclass"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("superclass%0D%0A%09%22%20Return%20the%20superclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24superclass%27"));
smalltalk.bind(smalltalk.Behavior,"subclasses",0
,function(){var self=this;
return self.smalltalk().subclasses(self);}
,"accessing",unescape("subclasses%0D%0A%09%22%20Return%20the%20subclasses%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20smalltalk%20%23subclasses%3A%20self"));
smalltalk.bind(smalltalk.Behavior,"allSubclasses",0
,function(){var self=this;
var result=nil;
result=smalltalk.Array.$$new();
self.allSubclassesDo_((function(each){return result.add_(each);}));
return result;}
,"testing",unescape("allSubclasses%0D%0A%09%22%20Return%20all%20the%20subclasses%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20Array%20new.%0D%0A%09self%20allSubclassesDo%3A%20%5B%3Aeach%7C%20result%20add%3A%20each%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Behavior,"allSubclassesDo:",0
,function(aBlock){var self=this;
self.smalltalk().allSubclassesDo(self,aBlock);
return self;}
,"testing",unescape("allSubclassesDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20with%20all%20the%20subclasses%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20smalltalk%20%23allSubclassesDo%3A%20self%20value%3A%20aBlock"));
smalltalk.bind(smalltalk.Behavior,"withAllSubclassesDo:",0
,function(aBlock){var self=this;
aBlock.value_(self);
self.allSubclassesDo_(aBlock);
return self;}
,"testing",unescape("withAllSubclassesDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20with%20the%20receiver%20and%20all%20the%20subclasses.%20%22%0D%0A%0D%0A%09aBlock%20value%3A%20self.%0D%0A%09self%20allSubclassesDo%3A%20aBlock"));
smalltalk.bind(smalltalk.Behavior,"withAllSubclasses",0
,function(){var self=this;
var result=nil;
result=smalltalk.Array.with_(self);
self.allSubclassesDo_((function(each){return result.add_(each);}));
return result;}
,"accessing",unescape("withAllSubclasses%0D%0A%09%22%20Return%20a%20collection%20with%20the%20receiver%20and%20all%20subclasses.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20Array%20with%3A%20self.%0D%0A%09self%20allSubclassesDo%3A%20%5B%3Aeach%7C%20result%20add%3A%20each%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Behavior,"allSuperclassesDo:",0
,function(aBlock){var self=this;
var current=nil;
current=self.superclass();
while((function(){return (nil.isNil_(current)===false);})()){aBlock.value_(current);
current=current.superclass();};;
return self;}
,"accessing",unescape("allSuperclassesDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20superclasses%20of%20the%20receiver.%0D%0A%09The%20superclasses%20are%20walked%20in%20inverse%20hierarchical%20order%2C%20i.e%20class%20Object%20is%20last.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20current%20%7C%0D%0A%09current%20%3A%3D%20self%20superclass.%0D%0A%09%5B%20current%20notNil%20%5D%20whileTrue%3A%20%5B%0D%0A%09%20%20%20%20%20%20%20%20aBlock%20value%3A%20current.%0D%0A%09%09current%20%3A%3D%20current%20superclass.%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"withAllSuperclassesDo:",0
,function(aBlock){var self=this;
aBlock.value_(self);
return self.allSuperclassesDo_(aBlock);}
,"accessing",unescape("withAllSuperclassesDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20receiver%20and%20it%27s%20superclasses.%0D%0A%09The%20superclasses%20are%20walked%20in%20inverse%20hierarchical%20order%2C%20i.e%20class%20Object%20is%20last.%0D%0A%09%22%0D%0A%0D%0A%09aBlock%20value%3A%20self.%0D%0A%09%5Eself%20allSuperclassesDo%3A%20aBlock"));
smalltalk.bind(smalltalk.Behavior,"withAllSuperclasses",0
,function(){var self=this;
return smalltalk.Array.with_(self)._comma(self.allSuperclasses());}
,"accessing",unescape("withAllSuperclasses%0D%0A%09%22%20Return%20a%20collection%20with%20the%20receiver%20and%20all%20superclasses.%20%22%0D%0A%0D%0A%09%5E%28Array%20with%3A%20self%29%20%2C%20self%20allSuperclasses"));
smalltalk.bind(smalltalk.Behavior,"allSuperclasses",0
,function(){var self=this;
var result=nil;
result=smalltalk.Array.$$new();
self.allSuperclassesDo_((function(each){return result.add_(each);}));
return result;}
,"accessing",unescape("allSuperclasses%0D%0A%09%22%20Return%20a%20collection%20of%20all%20the%20superclasses%20of%20the%20receiver.%0D%0A%09The%20superclasses%20are%20in%20inverse%20hierarchical%20order%2C%20i.e%20class%20Object%20is%20last.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20Array%20new.%0D%0A%09self%20allSuperclassesDo%3A%20%5B%3Aeach%7C%20result%20add%3A%20each%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Behavior,"inheritsFrom:",0
,function(aClass){var $s8Ret$={name:"stReturn"};try{var self=this;
if ((nil.is_eqeq_(self,aClass))){return true;};;
self.allSuperclassesDo_((function(each){return (((nil.is_eqeq_(aClass,each))).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"testing",unescape("inheritsFrom%3A%20aClass%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20inherits%20from%20aClass.%20%22%0D%0A%0D%0A%09self%20%3D%3D%20aClass%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09self%20allSuperclassesDo%3A%20%5B%3Aeach%7C%0D%0A%09%09aClass%20%3D%3D%20each%20ifTrue%3A%20%5B%20%5Etrue%20%5D%0D%0A%09%5D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Behavior,"fn",0
,function(){var self=this;

return ((function(){var $1$=(self["$fn"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("fn%0D%0A%09%22%20Private%20-%20Return%20the%20internal%20function%20of%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself%20basicAt%3A%20%27%24fn%27%20"));
smalltalk.bind(smalltalk.Behavior,"prototype",0
,function(){var self=this;

return ((function(){var $1$=(self.fn()["prototype"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("prototype%0D%0A%09%22%20Private%20-%20Return%20the%20prototype%20of%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself%20fn%20basicAt%3A%20%23prototype%20"));
smalltalk.bind(smalltalk.Behavior,"privateMethods",0
,function(){var self=this;

return ((function(){var $1$=(self.prototype()["$methods"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("privateMethods%0D%0A%09%22%20Private%2C%20Internal%20-%20Return%20all%20the%20methods%20defined%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself%20prototype%20basicAt%3A%20%27%24methods%27%20"));
smalltalk.bind(smalltalk.Behavior,"constructorName",0
,function(){var self=this;
var result=nil;
var fn=nil;
fn=self.fn();
result=((function(){var $1$=(fn["name"]);if(nil.isNil_($1$))return nil;return $1$;})());
if ((nil.isNil_(result)===false)){if (result.isEmpty()){return nil;};;
if (result._eq("$s8Constructor")){return nil;};;
return result;};;
result=fn.toString().upTo_(unescape("%28")).asArrayOfSubstrings();
if (!(result.size()._eq((2)))){return nil;};;
return result.last();}
,"accessing",unescape("constructorName%0D%0A%09%22%20Private%20-%20Return%20the%20name%20of%20the%20constructor%20of%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20result%20fn%20%7C%0D%0A%09fn%20%3A%3D%20self%20fn.%0D%0A%09result%20%3A%3D%20fn%20basicAt%3A%20%23name.%0D%0A%09result%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09result%20isEmpty%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%09result%20%3D%20%27%24s8Constructor%27%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%09%5Eresult%0D%0A%09%5D.%0D%0A%09result%20%3A%3D%20%28fn%20toString%20upTo%3A%20%24%28%29%20asArrayOfSubstrings.%0D%0A%09result%20size%20%3D%202%20ifFalse%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%5Eresult%20last"));
smalltalk.bind(smalltalk.Behavior,"methodDictionary",0
,function(){var self=this;
return smalltalk.PoolDictionary._at_at(self.privateMethods());}
,"accessing",unescape("methodDictionary%0D%0A%09%22%20Return%20a%20%28pool%29dictionary%20with%20the%20methods%20implemented%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EPoolDictionary%20@@%20self%20privateMethods%20"));
smalltalk.bind(smalltalk.Behavior,"methodsDo:",0
,function(aBlock){var self=this;
self.methodDictionary().do_((function(mth){return aBlock.value_(mth);}));
return self;}
,"accessing",unescape("methodsDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20for%20all%20the%20methods%20defined%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20methodDictionary%20do%3A%20%5B%3Amth%7C%20aBlock%20value%3A%20mth%20%5D"));
smalltalk.bind(smalltalk.Behavior,"compiledMethods",0
,function(){var self=this;
return self.methodDictionary().values();}
,"accessing",unescape("compiledMethods%0D%0A%09%22%20Return%20a%20collection%20with%20all%20the%20methods%20defined%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20methodDictionary%20values"));
smalltalk.bind(smalltalk.Behavior,"selectorsDo:",0
,function(aBlock){var self=this;
return self.methodDictionary().keys().do_(aBlock);}
,"accessing",unescape("selectorsDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20the%20selectors%20implemented%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20methodDictionary%20keys%20do%3A%20aBlock%20"));
smalltalk.bind(smalltalk.Behavior,"selectors",0
,function(){var self=this;
var result=nil;
result=smalltalk.Array.$$new();
self.selectorsDo_((function(each){return result.add_(each);}));
return result;}
,"accessing",unescape("selectors%0D%0A%09%22%20Return%20the%20selectors%20implemented%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20Array%20new.%0D%0A%09self%20selectorsDo%3A%20%5B%3Aeach%7C%20result%20add%3A%20each%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Behavior,"categorizedSelectors",0
,function(){var self=this;
var pool=nil;
pool=smalltalk.PoolDictionary.$$new();
self.methodsDo_((function(mth){var category=nil;
category=mth.category();if ((nil.isNil_(category))){category="undefined";};;return pool.at_ifAbsentPut_(category,(function(){return smalltalk.Array.$$new();})).add_(mth.selector());}));
return pool.keys().sorted().collect_((function(key){return smalltalk.Array.with_with_(key,pool.at_(key).sorted());}));}
,"accessing",unescape("categorizedSelectors%0D%0A%09%22%20Return%20the%20selectors%20classified%20by%20category.%20%22%0D%0A%0D%0A%09%7C%20pool%20%7C%0D%0A%09pool%20%3A%3D%20PoolDictionary%20new.%0D%0A%09self%20methodsDo%3A%20%5B%3Amth%7C%20%7C%20category%20%7C%0D%0A%09%09category%20%3A%3D%20mth%20category.%0D%0A%09%09category%20isNil%20ifTrue%3A%20%5B%20category%20%3A%3D%20%23undefined%20%5D.%0D%0A%09%09%28pool%20at%3A%20category%20ifAbsentPut%3A%20%5BArray%20new%5D%29%20add%3A%20mth%20selector%0D%0A%09%5D.%0D%0A%09%5Epool%20keys%20sorted%20collect%3A%20%5B%3Akey%7C%0D%0A%09%09Array%20with%3A%20key%20with%3A%20%28pool%20at%3A%20key%29%20sorted%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"implements:",0
,function(aSymbol){var self=this;
return self.methodDictionary().includesKey_(aSymbol);}
,"testing",unescape("implements%3A%20aSymbol%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20implements%20message%20aSymbol.%20%22%0D%0A%0D%0A%09%5Eself%20methodDictionary%20includesKey%3A%20aSymbol%20"));
smalltalk.bind(smalltalk.Behavior,"compiledMethodAt:",0
,function(aSymbol){var self=this;
return self.methodDictionary().at_ifAbsent_(aSymbol,(function(){return nil;}));}
,"accessing",unescape("compiledMethodAt%3A%20aSymbol%0D%0A%09%22%20Return%20the%20compiled%20method%20at%20aSymbol%20implemented%20by%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20methodDictionary%20at%3A%20aSymbol%20ifAbsent%3A%20%5B%5D%20"));
smalltalk.bind(smalltalk.Behavior,">>","_gt_gt"
,function(aSymbol){var self=this;
return self.compiledMethodAt_(aSymbol);}
,"accessing",unescape("%3E%3E%20aSymbol%0D%0A%09%22%20Return%20the%20method%20at%20aSymbol%20%28or%20nil%20if%20not%20implemented%20by%20the%20receiver%29.%20%22%0D%0A%0D%0A%09%5Eself%20compiledMethodAt%3A%20aSymbol"));
smalltalk.bind(smalltalk.Behavior,"compiledMethodFor:",0
,function(aSymbol){var self=this;
var current=nil;
var mth=nil;
current=self;
(function(){while((function(){mth=current.compiledMethodAt_(aSymbol);current=current.superclass();return (nil.isNil_(mth)).and_((function(){return (nil.isNil_(current)===false);}));})()){};return nil})();
return mth;}
,"accessing",unescape("compiledMethodFor%3A%20aSymbol%0D%0A%09%22%20Return%20the%20method%20at%20aSymbol%20searching%20up%20in%20receiver%27s%20hierarchy%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20current%20mth%20%7C%0D%0A%09current%20%3A%3D%20self.%0D%0A%09%5B%09mth%20%3A%3D%20current%20compiledMethodAt%3A%20aSymbol.%0D%0A%09%09current%20%3A%3D%20current%20superclass.%0D%0A%09%09mth%20isNil%20and%3A%20%5B%20current%20notNil%20%5D%0D%0A%09%5D%20whileTrue.%0D%0A%09%5Emth%20"));
smalltalk.bind(smalltalk.Behavior,">>>","_gt_gt_gt"
,function(aSymbol){var self=this;
return self.compiledMethodFor_(aSymbol);}
,"accessing",unescape("%3E%3E%3E%20aSymbol%0D%0A%09%22%20OBSOLETE%20-%20Return%20the%20method%20at%20aSymbol%20searching%20up%20in%20receiver%27s%20hierarchy%20%28or%20nil%20if%20not%20implemented%20by%20the%20receiver%20nor%20its%20superclasses%29.%20%22%0D%0A%0D%0A%09%5Eself%20compiledMethodFor%3A%20aSymbol%20"));
smalltalk.bind(smalltalk.Behavior,"instVarNames",0
,function(){var self=this;
return self.basicAt_ifAbsentPut_("$iVarNames",(function(){return [];}));}
,"accessing",unescape("instVarNames%0D%0A%09%22%20Return%20the%20instance%20variable%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24iVarNames%27%20ifAbsentPut%3A%20%5B%23%28%29%5D"));
smalltalk.bind(smalltalk.Behavior,"instVarNamesDo:",0
,function(aBlock){var self=this;
var array=nil;
array=((function(){var $1$=(self["$iVarNames"]);if(nil.isNil_($1$))return nil;return $1$;})());
return (((nil.isNil_(array)===false)).mustBeBoolean()==true ? (function(){return array.do_(aBlock);})() : nil);}
,"accessing",unescape("instVarNamesDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20instance%20variable%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20array%20%7C%0D%0A%09array%20%3A%3D%20self%20basicAt%3A%20%27%24iVarNames%27.%0D%0A%09%5Earray%20notNil%20ifTrue%3A%20%5B%20array%20do%3A%20aBlock%20%5D"));
smalltalk.bind(smalltalk.Behavior,"allInstVarNames",0
,function(){var self=this;
var result=nil;
result=self.instVarNames();
self.allSuperclassesDo_((function(each){return result=each.instVarNames()._comma(result);}));
return result;}
,"accessing",unescape("allInstVarNames%0D%0A%09%22%20Return%20all%20of%20the%20instance%20variable%20names%20defined%20in%20the%20receiver%20and%20its%20superclasses.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20instVarNames.%0D%0A%09self%20allSuperclassesDo%3A%20%5B%20%3Aeach%20%7C%20result%20%3A%3D%20each%20instVarNames%20%2C%20result%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Behavior,"instanceVariableNames",0
,function(){var self=this;
return self.instVarNames();}
,"accessing",unescape("instanceVariableNames%0D%0A%09%22%20Return%20the%20instance%20variable%20names%20defined%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20instVarNames"));
smalltalk.bind(smalltalk.Behavior,"propertyNames",0
,function(){var self=this;
return ((function(){var $1$=(self["$propertyNames"]);if(nil.isNil_($1$))return ((function(){return smalltalk.Array.$$new();}).value());return $1$;})());}
,"accessing",unescape("propertyNames%0D%0A%09%22%20Return%20the%20property%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24propertyNames%27%20ifAbsent%3A%20%5B%20Array%20new%20%5D"));
smalltalk.bind(smalltalk.Behavior,"propertyNames:",0
,function(anArray){var self=this;
if ((nil.isNil_(anArray))){return self.basicDelete_("$propertyNames");};;
return (self["$propertyNames"]=(anArray));}
,"accessing",unescape("propertyNames%3A%20anArray%0D%0A%09%22%20Set%20the%20property%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09anArray%20isNil%20ifTrue%3A%20%5B%20%5Eself%20basicDelete%3A%20%27%24propertyNames%27%20%5D.%0D%0A%09%5Eself%20basicAt%3A%20%27%24propertyNames%27%20put%3A%20anArray"));
smalltalk.bind(smalltalk.Behavior,"allPropertyNamesDo:",0
,function(aBlock){var self=this;
self.propertyNames().do_(aBlock);
self.allSuperclassesDo_((function(each){return each.propertyNames().do_(aBlock);}));
return self;}
,"accessing",unescape("allPropertyNamesDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20for%20the%20property%20names%20defined%20in%20the%20receiver%20and%20its%20superclasses.%20%22%0D%0A%0D%0A%09self%20propertyNames%20do%3A%20aBlock.%0D%0A%09self%20allSuperclassesDo%3A%20%5B%20%3Aeach%20%7C%20each%20propertyNames%20do%3A%20aBlock%20%5D."));
smalltalk.bind(smalltalk.Behavior,"allInstVarNamesDo:",0
,function(aBlock){var self=this;
self.instVarNamesDo_(aBlock);
self.allSuperclassesDo_((function(each){return each.instVarNamesDo_(aBlock);}));
return self;}
,"accessing",unescape("allInstVarNamesDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20for%20the%20instance%20variable%20names%20defined%20in%20the%20receiver%20and%20its%20superclasses.%20%22%0D%0A%0D%0A%09self%20instVarNamesDo%3A%20aBlock.%0D%0A%09self%20allSuperclassesDo%3A%20%5B%20%3Aeach%20%7C%20each%20instVarNamesDo%3A%20aBlock%20%5D."));
smalltalk.bind(smalltalk.Behavior,"comment",0
,function(){var self=this;
return ((function(){var $1$=(self.instanceClass()["$comment"]);if(nil.isNil_($1$))return ((function(){return "";}).value());return $1$;})());}
,"accessing",unescape("comment%0D%0A%09%22%20Return%20the%20comment%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20instanceClass%20basicAt%3A%20%27%24comment%27%20ifAbsent%3A%20%5B%20%27%27%20%5D"));
smalltalk.bind(smalltalk.Behavior,"basicComment:",0
,function(aString){var self=this;
(self.instanceClass()["$comment"]=(aString));
return self;}
,"private",unescape("basicComment%3A%20aString%0D%0A%09%22%20Set%20the%20comment%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20instanceClass%20basicAt%3A%20%27%24comment%27%20put%3A%20aString"));
smalltalk.bind(smalltalk.Behavior,"comment:",0
,function(aString){var self=this;
smalltalk.ClassBuilder.$$new().class_comment_(self.instanceClass(),aString);
return self;}
,"accessing",unescape("comment%3A%20aString%0D%0A%09%22%20Set%20the%20comment%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09ClassBuilder%20new%0D%0A%09%09class%3A%20self%20instanceClass%0D%0A%09%09comment%3A%20aString"));
smalltalk.bind(smalltalk.Behavior,"removeSources",0
,function(){var self=this;
self.instanceClass().removeSources();
return self;}
,"tools",unescape("removeSources%0D%0A%09%22%20Remove%20all%20development%20sources%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20instanceClass%20removeSources"));
smalltalk.bind(smalltalk.Behavior,"poolDictionaries",0
,function(){var self=this;
return self.instanceClass().poolDictionaries();}
,"accessing",unescape("poolDictionaries%0D%0A%09%22%20Return%20the%20poolDictionaries%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20instanceClass%20poolDictionaries"));
smalltalk.bind(smalltalk.Behavior,"poolDictionaryNames",0
,function(){var self=this;
return self.instanceClass().poolDictionaryNames();}
,"accessing",unescape("poolDictionaryNames%0D%0A%09%22%20Return%20the%20shared%20pool%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20instanceClass%20poolDictionaryNames"));
smalltalk.bind(smalltalk.Behavior,"sharedPoolsAndNamesDo:",0
,function(aBlock){var self=this;
var pool=nil;
self.instanceClass().withAllSuperclassesDo_((function(each){return each.poolDictionaryNames().do_((function(aName){pool=smalltalk.Smalltalk.poolDictionary_ifAbsent_(aName,(function(){return nil;}));return (((nil.isNil_(pool)===false)).mustBeBoolean()==true ? (function(){return aBlock.value_value_(aName,pool);})() : nil);}));}));
return self;}
,"accessing",unescape("sharedPoolsAndNamesDo%3A%20aBlock%0D%0A%09%22%20Execute%20aBlock%20for%20the%20pool%20dictionaries%20known%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20pool%20%7C%0D%0A%09self%20instanceClass%20withAllSuperclassesDo%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20poolDictionaryNames%20do%3A%20%5B%3AaName%7C%0D%0A%09%09%09pool%20%3A%3D%20Smalltalk%20poolDictionary%3A%20aName%20ifAbsent%3A%20%5B%20nil%20%5D.%0D%0A%09%09%09pool%20notNil%20ifTrue%3A%20%5B%20aBlock%20value%3A%20aName%20value%3A%20pool%20%5D%0D%0A%09%09%5D.%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"sharedPoolsDo:",0
,function(aBlock){var self=this;
self.sharedPoolsAndNamesDo_((function(aName,aPool){return aBlock.value_(aPool);}));
return self;}
,"accessing",unescape("sharedPoolsDo%3A%20aBlock%0D%0A%09%22%20Execute%20aBlock%20for%20the%20pool%20dictionaries%20known%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20sharedPoolsAndNamesDo%3A%20%5B%3AaName%20%3AaPool%7C%20aBlock%20value%3A%20aPool%20%5D"));
smalltalk.bind(smalltalk.Behavior,"classVariablesDo:",0
,function(aBlock){var self=this;
var bindings=nil;
return self.instanceClass().withAllSuperclassesDo_((function(each){bindings=each.classVariables();return (((nil.isNil_(bindings)===false)).mustBeBoolean()==true ? (function(){return bindings.associationsDo_((function(assoc){return aBlock.value_(assoc);}));})() : nil);}));}
,"accessing",unescape("classVariablesDo%3A%20aBlock%0D%0A%09%22%20Execute%20aBlock%20for%20the%20classVariables%20bindings%20known%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20bindings%20%7C%0D%0A%09%5Eself%20instanceClass%20withAllSuperclassesDo%3A%20%5B%3Aeach%7C%0D%0A%09%09bindings%20%3A%3D%20each%20classVariables.%0D%0A%09%09bindings%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09bindings%20associationsDo%3A%20%5B%3Aassoc%7C%20aBlock%20value%3A%20assoc%20%5D%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"classVariableNamesDo:",0
,function(aBlock){var self=this;
var bindings=nil;
return self.instanceClass().withAllSuperclassesDo_((function(each){bindings=each.classVariables();return (((nil.isNil_(bindings)===false)).mustBeBoolean()==true ? (function(){return bindings.keysDo_((function(aName){return aBlock.value_(aName);}));})() : nil);}));}
,"accessing",unescape("classVariableNamesDo%3A%20aBlock%0D%0A%09%22%20Execute%20aBlock%20for%20the%20classVariable%20names%20known%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20bindings%20%7C%0D%0A%09%5Eself%20instanceClass%20withAllSuperclassesDo%3A%20%5B%3Aeach%7C%0D%0A%09%09bindings%20%3A%3D%20each%20classVariables.%0D%0A%09%09bindings%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09bindings%20keysDo%3A%20%5B%3AaName%7C%20aBlock%20value%3A%20aName%20%5D%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"classVariableAt:put:",0
,function(aName,anObject){var self=this;
return self.instanceClass().classVariables().at_put_(aName,anObject);}
,"accessing",unescape("classVariableAt%3A%20aName%20put%3A%20anObject%0D%0A%09%22%20Set%20the%20classVariable%20of%20the%20receiver%20to%20anObject.%20%22%0D%0A%0D%0A%09%5Eself%20instanceClass%20classVariables%20at%3A%20aName%20put%3A%20anObject"));
smalltalk.bind(smalltalk.Behavior,"classVariableAt:ifAbsent:",0
,function(aName,aBlock){var $s8Ret$={name:"stReturn"};try{var self=this;
var bindings=nil;
self.instanceClass().withAllSuperclassesDo_((function(each){bindings=each.classVariables();return (((nil.isNil_(bindings)===false)).mustBeBoolean()==true ? (function(){return ((bindings.includesKey_(aName)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return bindings.at_(aName)})();throw($s8Ret$)})();})() : nil);})() : nil);}));
(function(){$s8Ret$.result=(function(){return aBlock.evaluate()})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("classVariableAt%3A%20aName%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20value%20of%20classVariable%20named%20aName.%20%22%0D%0A%0D%0A%09%7C%20bindings%20%7C%0D%0A%09self%20instanceClass%20withAllSuperclassesDo%3A%20%5B%3Aeach%7C%0D%0A%09%09bindings%20%3A%3D%20each%20classVariables.%0D%0A%09%09bindings%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09%28bindings%20includesKey%3A%20aName%29%20ifTrue%3A%20%5B%0D%0A%09%09%09%09%5Ebindings%20at%3A%20aName%0D%0A%09%09%09%5D%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5EaBlock%20evaluate"));
smalltalk.bind(smalltalk.Behavior,"classVariableAt:",0
,function(aName){var self=this;
return self.classVariableAt_ifAbsent_(aName,(function(){return self.error_("Undefined class variable "._comma(aName)._comma(" in ")._comma(self.name()));}));}
,"accessing",unescape("classVariableAt%3A%20aName%0D%0A%09%22%20Return%20the%20value%20of%20classVariable%20named%20aName%20or%20generates%20an%20error.%20%22%0D%0A%0D%0A%09%5Eself%20classVariableAt%3A%20aName%20ifAbsent%3A%20%5B%0D%0A%09%09self%20error%3A%20%27Undefined%20class%20variable%20%27%2CaName%20%2C%27%20in%20%27%20%2Cself%20name%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"pool:variableAt:put:",0
,function(poolName,aName,anObject){var $s8Ret$={name:"stReturn"};try{var self=this;
var pool=nil;
pool=smalltalk.Smalltalk.poolDictionary_ifAbsent_(poolName,(function(){return (function(){$s8Ret$.result=(function(){return self.error_("Undefined pool variable "._comma(poolName)._comma(" in ")._comma(self.name()))})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return pool.at_put_(aName,anObject)})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("pool%3A%20poolName%20variableAt%3A%20aName%20put%3A%20anObject%0D%0A%09%22%20Set%20the%20pool%20Variable%20of%20the%20receiver%20to%20anObject.%20%22%0D%0A%0D%0A%09%7C%20pool%20%7C%0D%0A%09pool%20%3A%3D%20Smalltalk%0D%0A%09%09poolDictionary%3A%20poolName%0D%0A%09%09ifAbsent%3A%20%5B%20%5Eself%20error%3A%20%27Undefined%20pool%20variable%20%27%2CpoolName%20%2C%27%20in%20%27%20%2Cself%20name%20%5D.%0D%0A%09%5Epool%20at%3A%20aName%20put%3A%20anObject"));
smalltalk.bind(smalltalk.Behavior,"loadRequirementsInto:for:",0
,function(aCollection,anEncoder){var self=this;
return self;}
,"loading",unescape("loadRequirementsInto%3A%20aCollection%20for%3A%20anEncoder%0D%0A%09%22%20Add%20the%20requirements%20to%20load%20the%20receiver%20from%20sources%20to%20aCollection.%0D%0A%09Default%20action%20do%20nothing%2C%20this%20method%20can%20be%20refined%20by%20the%20subclasses.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Behavior,"removeSelectors:",0
,function(aCollection){var self=this;
aCollection.do_((function(each){return self.removeSelector_(each);}));
return self;}
,"methods",unescape("removeSelectors%3A%20aCollection%0D%0A%09%22%20Remove%20the%20methods%20for%20messages%20in%20aCollection%2C%20implemented%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09aCollection%20do%3A%20%5B%3Aeach%7C%20self%20removeSelector%3A%20each%20%5D"));
smalltalk.bind(smalltalk.Behavior,"removeCompiledMethod:",0
,function(aMethod){var self=this;
self.smalltalk().removeMethod(self,aMethod);
return self;}
,"private",unescape("removeCompiledMethod%3A%20aMethod%0D%0A%09%22%20Remove%20aMethod%20from%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20smalltalk%20%23removeMethod%3A%20self%20method%3A%20aMethod"));
smalltalk.bind(smalltalk.Behavior,"missingImplementorFor:",0
,function(jsSelector){var self=this;
return self.error_("Missing implementor for "._comma(jsSelector.asNativeSelector().asLiteral())._comma(" in ")._comma(self.name()));}
,"private",unescape("missingImplementorFor%3A%20jsSelector%0D%0A%09%22%20Private%20-%20Handle%20the%20situation%20where%20there%20is%20no%20implementor%20for%20jsSelector.%0D%0A%09This%20can%20happen%20when%20there%20is%20no%20implementation%20for%20jsSelector%20and%20the%20message%0D%0A%09%20has%20been%20dispatched%20through%20a%20superSend.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%27Missing%20implementor%20for%20%27%0D%0A%09%09%2C%20jsSelector%20asNativeSelector%20asLiteral%0D%0A%09%09%2C%27%20in%20%27%2Cself%20name"));
smalltalk.bind(smalltalk.Behavior,"removeSelector:",0
,function(aSelector){var self=this;
var method=nil;
method=self.compiledMethodAt_(aSelector);
if ((nil.isNil_(method)===false)){self.removeCompiledMethod_(method);};;
return self;}
,"methods",unescape("removeSelector%3A%20aSelector%0D%0A%09%22%20Remove%20the%20method%20for%20message%20aSelector%20implemented%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20method%20%7C%0D%0A%09method%20%3A%3D%20self%20compiledMethodAt%3A%20aSelector.%0D%0A%09method%20notNil%20ifTrue%3A%20%5B%20self%20removeCompiledMethod%3A%20method%20%5D."));
smalltalk.bind(smalltalk.Behavior,"removeFromSystem",0
,function(){var self=this;
return self.instanceClass().removeFromSystem();}
,"remove",unescape("removeFromSystem%0D%0A%09%22%20Remove%20the%20receiver%20from%20system.%20%22%0D%0A%0D%0A%09%5Eself%20instanceClass%20removeFromSystem%20"));
smalltalk.bind(smalltalk.Behavior,"implementorsOf:",0
,function(aSymbol){var self=this;
var result=nil;
result=[];
self.withAllSubclassesDo_((function(each){var method=nil;
method=each._gt_gt(aSymbol);if ((nil.isNil_(method)===false)){result.add_(method);};;method=each.$klass._gt_gt(aSymbol);return (((nil.isNil_(method)===false)).mustBeBoolean()==true ? (function(){return result.add_(method);})() : nil);}));
return result;}
,"testing",unescape("implementorsOf%3A%20aSymbol%0D%0A%09%22%20Return%20the%20implementors%20of%20aSymbol%20in%20the%20receiver%20and%20its%20subclasses.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20%23%28%29.%0D%0A%09self%20withAllSubclassesDo%3A%20%5B%3Aeach%20%7C%20%7C%20method%20%7C%0D%0A%09%09method%20%3A%3D%20each%20%3E%3E%20aSymbol.%0D%0A%09%09method%20notNil%20ifTrue%3A%20%5B%20result%20add%3A%20method%20%5D.%0D%0A%09%09method%20%3A%3D%20each%20class%20%3E%3E%20aSymbol.%0D%0A%09%09method%20notNil%20ifTrue%3A%20%5B%20result%20add%3A%20method%20%5D.%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Behavior,"asJSONObject",0
,function(){var self=this;
return self;}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Behavior,"shallowCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Behavior,"deepCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Behavior,"toString",0
,function(){var self=this;
return unescape("%7Bst%3A")._comma(self.name())._comma(unescape("%7D"));}
,"converting",unescape("toString%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20String.%20%22%0D%0A%0D%0A%09%5E%27%7Bst%3A%27%2Cself%20name%2C%24%7D"));
smalltalk.bind(smalltalk.Behavior,"aboutToAdd:",0
,function(aMethod){var self=this;
smalltalk.APIBuilder.register_(aMethod);
return self;}
,"private",unescape("aboutToAdd%3A%20aMethod%0D%0A%09%22%20Private%20-%20Prepare%20aMethod%20to%20be%20added%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09APIBuilder%20register%3A%20aMethod.%20"));
smalltalk.bind(smalltalk.Behavior,"canAdd:method:",0
,function(isDoIt,aMethod){var $s8Ret$={name:"stReturn"};try{var self=this;
if (!((nil.isNil_(smalltalk.OverwriteMethods)===false).and_((function(){return smalltalk.OverwriteMethods;})))){((self.implements_(aMethod.selector())).mustBeBoolean()==true ? (function(){return ((isDoIt).mustBeBoolean()==true ? nil : (function(){if ((nil.isNil_(smalltalk.DumpOverwritenMethods)===false).and_((function(){return smalltalk.DumpOverwritenMethods.and_((function(){return (nil.isNil_(smalltalk.Exporter)===false);}));}))){self.note_(unescape("%3F%20")._comma(self.name())._comma(unescape("%3E%3E%23"))._comma(aMethod.selector()));
self.print_(smalltalk.Exporter.$$new().export_method_(self,aMethod));} else{self.note_(unescape("-Overwrite%20Conflict-%20")._comma(self.name())._comma(unescape("%3E%3E%23"))._comma(aMethod.selector()));};;return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();})());})() : nil);};;
self.aboutToAdd_(aMethod);
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"private",unescape("canAdd%3A%20isDoIt%20method%3A%20aMethod%0D%0A%09%22%20Private%20-%20Returns%20true%20if%20aMethod%20be%20added%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09%28OverwriteMethods%20notNil%20and%3A%20%5B%20OverwriteMethods%20%5D%29%20ifFalse%3A%20%5B%0D%0A%09%09%28self%20implements%3A%20aMethod%20selector%29%20ifTrue%3A%20%5B%0D%0A%09%09%09isDoIt%20ifFalse%3A%20%5B%0D%0A%09%09%09%09%28DumpOverwritenMethods%20notNil%0D%0A%09%09%09%09and%3A%20%5B%20DumpOverwritenMethods%0D%0A%09%09%09%09and%3A%20%5B%20Exporter%20notNil%20%5D%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%09%09%09self%20note%3A%20%27%3F%20%27%2Cself%20name%2C%27%3E%3E%23%27%2CaMethod%20selector.%0D%0A%09%09%09%09%09self%20print%3A%20%28Exporter%20new%20export%3A%20self%20method%3A%20aMethod%29.%0D%0A%09%09%09%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09%09%09self%20note%3A%20%27-Overwrite%20Conflict-%20%27%2Cself%20name%2C%27%3E%3E%23%27%2CaMethod%20selector.%0D%0A%09%09%09%09%5D.%0D%0A%09%09%09%09%5Efalse%0D%0A%09%09%09%5D.%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09self%20aboutToAdd%3A%20aMethod.%0D%0A%09%5Etrue%20"));
smalltalk.bind(smalltalk.Behavior,"basicAddCompiledMethod:",0
,function(aMethod){var self=this;
self.smalltalk().addMethod(aMethod.selector().asSelector(),aMethod,self);
return self;}
,"private",unescape("basicAddCompiledMethod%3A%20aMethod%0D%0A%09%22%20Private%20-%20Register%20aMethod%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20smalltalk%0D%0A%09%09%23addMethod%3A%20aMethod%20selector%20asSelector%0D%0A%09%09method%3A%20aMethod%20to%3A%20self"));
smalltalk.bind(smalltalk.Behavior,"addCompiledMethod:",0
,function(aMethod){var self=this;
var isDoIt=nil;
isDoIt=aMethod.selector()._eq(smalltalk.String.doItSelector());
if (!(self.canAdd_method_(isDoIt,aMethod))){return nil;};;
if ((nil.isNil_(smalltalk.Exporter)===false).and_((function(){return isDoIt.not();}))){self.print_(smalltalk.Exporter.$$new().export_method_(self,aMethod));};;
return self.basicAddCompiledMethod_(aMethod);}
,"methods",unescape("addCompiledMethod%3A%20aMethod%0D%0A%09%22%20Add%20the%20compiled%20method%2C%20if%20appropiate%2C%20and%20register%20the%20method%20in%20recent%20methods.%20%22%0D%0A%0D%0A%09%7C%20isDoIt%20%7C%0D%0A%09isDoIt%20%3A%3D%20aMethod%20selector%20%3D%20String%20doItSelector.%0D%0A%09%28self%20canAdd%3A%20isDoIt%20method%3A%20aMethod%29%20ifFalse%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%28Exporter%20notNil%20and%3A%20%5B%20isDoIt%20not%20%5D%29%20ifTrue%3A%20%5B%0D%0A%22%20DEBUG%20-%09self%20note%3A%20self%20name%2C%27%3E%3E%23%27%2CaMethod%20selector.%22%0D%0A%09%09self%20print%3A%20%28Exporter%20new%20export%3A%20self%20method%3A%20aMethod%29.%0D%0A%09%5D.%0D%0A%09%5Eself%20basicAddCompiledMethod%3A%20aMethod"));
smalltalk.bind(smalltalk.Behavior,"collection",0
,function(){var self=this;
return (function($rec){$rec.species_(self);return $rec.yourself();})(smalltalk.ComposedObjects.$$new());}
,"composing",unescape("collection%0D%0A%09%22%20Return%20a%20collection%20to%20compose%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EComposedObjects%20new%0D%0A%09%09species%3A%20self%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Behavior,"codeFor:accessing:",0
,function(anEncoder,aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
var aName=nil;
aName=aNode.value();
self.allInstVarNamesDo_((function(each){return ((aName._eq(each)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return anEncoder.codeToAccessInstanceVariable_in_(aName,aNode)})();throw($s8Ret$)})();})() : nil);}));
self.allPropertyNamesDo_((function(each){return ((aName._eq(each)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return anEncoder.codeToAccessProperty_in_(aName,aNode)})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return nil})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"Compiler-generation",unescape("codeFor%3A%20anEncoder%20accessing%3A%20aNode%0D%0A%09%22%20Return%20the%20code%20to%20access%20variable%20in%20aNode%20%28aVariableNode%29.%0D%0A%09Return%20nil%20if%20the%20code%20to%20access%20variable%20can%27t%20be%20generated.%0D%0A%09The%20default%20implementation%20return%20code%20to%20access%20instance%20variables.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20aName%20%7C%0D%0A%09aName%20%3A%3D%20aNode%20value.%0D%0A%09self%20allInstVarNamesDo%3A%20%5B%3Aeach%7C%0D%0A%09%09aName%20%3D%20each%20ifTrue%3A%20%5B%0D%0A%09%09%09%5EanEncoder%20codeToAccessInstanceVariable%3A%20aName%20in%3A%20aNode%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09self%20allPropertyNamesDo%3A%20%5B%3Aeach%7C%0D%0A%09%09aName%20%3D%20each%20ifTrue%3A%20%5B%0D%0A%09%09%09%5EanEncoder%20codeToAccessProperty%3A%20aName%20in%3A%20aNode%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5Enil"));
smalltalk.bind(smalltalk.Behavior,"compiler",0
,function(){var self=this;
return self.compilerSupport().for_(self);}
,"Compiler-compiling",unescape("compiler%0D%0A%09%22%20Return%20a%20compiler%20for%20receiver%27s%20source%20code.%20%22%0D%0A%0D%0A%09%5Eself%20compilerSupport%20for%3A%20self"));
smalltalk.bind(smalltalk.Behavior,"recompile",0
,function(){var self=this;
self.compiler().recompile_(self);
return self;}
,"Compiler-compiling",unescape("recompile%0D%0A%09%22%20Recompile%20all%20the%20methods%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20compiler%20recompile%3A%20self"));
smalltalk.bind(smalltalk.Behavior,"recompileAll",0
,function(){var self=this;
self.instanceClass().withAllSubclassesDo_((function(each){return each.recompile();}));
return self;}
,"Compiler-compiling",unescape("recompileAll%0D%0A%09%22%20Recompile%20all%20the%20methods%20of%20the%20receiver%20and%20its%20subclasses.%09%22%0D%0A%0D%0A%09self%20instanceClass%20withAllSubclassesDo%3A%20%5B%3Aeach%7C%20each%20recompile%20%5D"));
smalltalk.bind(smalltalk.Behavior,"recompile:",0
,function(what){var self=this;
if (what.isString()){return self.recompile_(smalltalk.Array.with_(what));};;
what.do_((function(selector){return self._gt_gt(selector).recompile();}));
return self;}
,"Compiler-compiling",unescape("recompile%3A%20what%0D%0A%09%22%20Recompile%20the%20method%28s%29%20at%20selector%28s%29%20of%20the%20receiver.%09%22%0D%0A%0D%0A%09what%20isString%20ifTrue%3A%20%5B%20%5Eself%20recompile%3A%20%28Array%20with%3A%20what%29%20%5D.%0D%0A%09what%20do%3A%20%5B%3Aselector%7C%20%28self%20%3E%3E%20selector%29%20recompile%20%5D"));
smalltalk.bind(smalltalk.Behavior,"templateFrom:doing:on:",0
,function(host,aBlock,methodBlock){var self=this;
var template=nil;
if ((nil.isNil_(host))){return nil;};;
template=((function(){var $1$=(host["s8Template"]);if(nil.isNil_($1$))return ((function(){return nil;}).value());return $1$;})());
if ((nil.isNil_(template)).and_((function(){return smalltalk.is_kindOf_(host,(smalltalk.CompiledMethod));}))){template=(function($rec){$rec.useTemplates_(false);$rec.currentClass_(host.classField());return $rec.parseMethod_(host.source());})(smalltalk.Compiler.$$new());
(host["s8Template"]=(template));};;
return (((nil.isNil_(template)===false)).mustBeBoolean()==true ? (function(){template=template.deepCopy();if ((nil.isNil_(aBlock)===false)){template.do_(aBlock);};;return (((nil.isNil_(methodBlock)===false)).mustBeBoolean()==true ? (function(){return methodBlock.value_(template);})() : (function(){return template;})());})() : nil);}
,"Compiler-templates",unescape("templateFrom%3A%20host%20doing%3A%20aBlock%20on%3A%20methodBlock%0D%0A%09%22%20Return%20the%20result%20of%20collecting%20mutations%20in%20template%20cached%20in%20host%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20template%20%7C%0D%0A%09host%20isNil%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09template%20%3A%3D%20host%20basicAt%3A%20%23s8Template%20ifAbsent%3A%20%5B%5D.%0D%0A%09%28template%20isNil%20and%3A%20%5Bhost%20isKindOf%3A%20CompiledMethod%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09template%20%3A%3D%20Compiler%20new%0D%0A%09%09%09useTemplates%3A%20false%3B%0D%0A%09%09%09currentClass%3A%20host%20classField%3B%0D%0A%09%09%09parseMethod%3A%20host%20source.%0D%0A%09%09host%20basicAt%3A%20%23s8Template%20put%3A%20template.%0D%0A%09%5D.%0D%0A%09%5Etemplate%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09template%20%3A%3D%20template%20deepCopy.%0D%0A%09%09aBlock%20notNil%20ifTrue%3A%20%5B%20template%20do%3A%20aBlock%20%5D.%0D%0A%09%09methodBlock%20notNil%0D%0A%09%09%09ifTrue%3A%20%5B%20methodBlock%20value%3A%20template%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20template%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"templateFrom:with:",0
,function(host,methodBlock){var self=this;
return self.templateFrom_doing_on_(host,nil,methodBlock);}
,"Compiler-templates",unescape("templateFrom%3A%20host%20with%3A%20methodBlock%0D%0A%09%22%20Return%20the%20template%20cached%20in%20host%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20templateFrom%3A%20host%20doing%3A%20nil%20on%3A%20methodBlock"));
smalltalk.bind(smalltalk.Behavior,"methodsFor:",0
,function(aString){var self=this;
return smalltalk.ClassCategoryReader.$$new().class_category_(self,aString);}
,"Tools-fileIn",unescape("methodsFor%3A%20aString%0D%0A%09%22%20FileIn%20-%20Return%20a%20chunk%20reader%20to%20fileIn%20methods%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EClassCategoryReader%20new%20class%3A%20self%20category%3A%20aString"));
smalltalk.bind(smalltalk.Behavior,"methodsFor:stamp:",0
,function(aString,stamp){var self=this;
return smalltalk.ClassCategoryReader.$$new().class_category_stamp_(self,aString,stamp);}
,"Tools-fileIn",unescape("methodsFor%3A%20aString%20stamp%3A%20stamp%0D%0A%09%22%20FileIn%20-%20Return%20a%20chunk%20reader%20to%20fileIn%20methods%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EClassCategoryReader%20new%20class%3A%20self%20category%3A%20aString%20stamp%3A%20stamp"));
smalltalk.bind(smalltalk.Behavior,"methods",0
,function(){var self=this;
return self.methodsFor_("unclassified");}
,"Tools-fileIn",unescape("methods%0D%0A%09%22%20FileIn%20-%20Return%20a%20chunk%20reader%20to%20fileIn%20methods%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20methodsFor%3A%20%23unclassified%20"));
smalltalk.bind(smalltalk.Behavior,"commentStamp",0
,function(){var self=this;
return smalltalk.ClassCommentReader.$$new().class_(self);}
,"Tools-fileIn",unescape("commentStamp%0D%0A%09%22%20Return%20a%20comment%20reader%20for%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EClassCommentReader%20new%20class%3A%20self"));
smalltalk.bind(smalltalk.Behavior,"fileOut",0
,function(){var self=this;
return smalltalk.Exporter.fileOut_(self.instanceClass());}
,"Tools-fileOut",unescape("fileOut%0D%0A%09%22%20Return%20fileOut%20definition%20and%20code%20of%20a%20class.%20%22%0D%0A%0D%0A%09%5EExporter%20fileOut%3A%20self%20instanceClass"));
smalltalk.bind(smalltalk.Behavior,"export","$$export"
,function(){var self=this;
return smalltalk.Exporter.export_(self.instanceClass());}
,"Tools-export",unescape("export%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20definition%20and%20source%20code%20of%20a%20class.%20%22%0D%0A%0D%0A%09%5EExporter%20export%3A%20self%20instanceClass"));
smalltalk.bind(smalltalk.Behavior,"references",0
,function(){var self=this;
var result=nil;
result=smalltalk.PoolDictionary.$$new();
self.methodsDo_((function(mth){var refs=nil;
refs=mth.references();return refs.keys().do_((function(key){return result.at_ifAbsentPut_(key,(function(){return smalltalk.HashedSet.$$new();})).addAll_(refs.at_(key));}));}));
return result.collect_((function(each){return each.asArray();}));}
,"Tools-references",unescape("references%0D%0A%09%22%20Return%20all%20the%20references%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20PoolDictionary%20new.%0D%0A%09self%20methodsDo%3A%20%5B%3Amth%7C%20%7C%20refs%20%7C%0D%0A%09%09refs%20%3A%3D%20mth%20references.%0D%0A%09%09refs%20keys%20do%3A%20%5B%3Akey%7C%0D%0A%09%09%09%28result%20at%3A%20key%20ifAbsentPut%3A%20%5BHashedSet%20new%5D%29%0D%0A%09%09%09%09addAll%3A%20%28refs%20at%3A%20key%29%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Eresult%20collect%3A%20%5B%3Aeach%7C%20each%20asArray%20%5D"));
smalltalk.bind(smalltalk.Behavior,"messagesSent",0
,function(){var self=this;
var result=nil;
var cls=nil;
result=smalltalk.HashedSet.$$new();
cls=self.instanceClass();
cls.methodsDo_((function(each){return result.addAll_(each.messages());}));
cls.$klass.methodsDo_((function(each){return result.addAll_(each.messages());}));
return result;}
,"Tools-messages",unescape("messagesSent%0D%0A%09%22%20Return%20the%20messages%20sent%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20cls%20%7C%0D%0A%09result%20%3A%3D%20HashedSet%20new.%0D%0A%09cls%20%3A%3D%20self%20instanceClass.%0D%0A%09cls%20methodsDo%3A%20%5B%3Aeach%7C%20result%20addAll%3A%20each%20messages%20%5D.%0D%0A%09cls%20class%20methodsDo%3A%20%5B%3Aeach%7C%20result%20addAll%3A%20each%20messages%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Behavior,"allMessagesSent",0
,function(){var self=this;
return self.allSubclasses().inject_into_(self.messagesSent(),(function(total,each){return (function($rec){$rec.addAll_(each.messagesSent());return $rec.yourself();})(total);}));}
,"Tools-messages",unescape("allMessagesSent%0D%0A%09%22%20Return%20the%20messages%20sent%20by%20the%20receiver%20and%20its%20subclasses.%20%22%0D%0A%0D%0A%09%5Eself%20allSubclasses%0D%0A%09%09inject%3A%20self%20messagesSent%0D%0A%09%09into%3A%20%5B%3Atotal%20%3Aeach%7C%20total%20addAll%3A%20each%20messagesSent%3B%20yourself%20%5D"));
smalltalk.bind(smalltalk.Behavior,"implementedMessages",0
,function(){var self=this;
var result=nil;
var cls=nil;
result=smalltalk.HashedSet.$$new();
cls=self.instanceClass();
result.addAll_(cls.selectors());
result.addAll_(cls.$klass.selectors());
return result;}
,"Tools-messages",unescape("implementedMessages%0D%0A%09%22%20Return%20the%20messages%20implemented%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20cls%20%7C%0D%0A%09result%20%3A%3D%20HashedSet%20new.%0D%0A%09cls%20%3A%3D%20self%20instanceClass.%0D%0A%09result%20addAll%3A%20cls%20selectors.%0D%0A%09result%20addAll%3A%20cls%20class%20selectors.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Behavior,"allImplementedMessages",0
,function(){var self=this;
return self.allSubclasses().inject_into_(self.implementedMessages(),(function(total,each){return (function($rec){$rec.addAll_(each.implementedMessages());return $rec.yourself();})(total);}));}
,"Tools-messages",unescape("allImplementedMessages%0D%0A%09%22%20Return%20the%20messages%20implemented%20by%20the%20receiver%20and%20its%20subclasses.%20%22%0D%0A%0D%0A%09%5Eself%20allSubclasses%0D%0A%09%09inject%3A%20self%20implementedMessages%0D%0A%09%09into%3A%20%5B%3Atotal%20%3Aeach%7C%20total%20addAll%3A%20each%20implementedMessages%3B%20yourself%20%5D"));
smalltalk.bind(smalltalk.Behavior,"implementorsDo:",0
,function(aBlock){var self=this;
var cls=nil;
cls=self.instanceClass();
cls.$klass.methodsDo_(aBlock);
cls.methodsDo_(aBlock);
return self;}
,"Tools-iterating",unescape("implementorsDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20for%20each%20implementor%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20cls%20%7C%0D%0A%09cls%20%3A%3D%20self%20instanceClass.%0D%0A%09cls%20class%20methodsDo%3A%20aBlock.%0D%0A%09cls%20methodsDo%3A%20aBlock."));
smalltalk.bind(smalltalk.Behavior,"implementors",0
,function(){var self=this;
var result=nil;
result=[];
self.implementorsDo_((function(each){return result.add_(each);}));
return result;}
,"Tools-implementors",unescape("implementors%0D%0A%09%22%20Return%20a%20collection%20with%20all%20implementors%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20%23%28%29.%0D%0A%09self%20implementorsDo%3A%20%5B%3Aeach%7C%20result%20add%3A%20each%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Behavior,"implement:comment:as:category:",0
,function(messageHeader,aComment,sourceCode,aCategory){var self=this;
var code=nil;
code=unescape("%21%20")._comma(self.name())._comma((((nil.isNil_(aCategory))).mustBeBoolean()==true ? (function(){return " methods";})() : (function(){return " methodsFor: "._comma(aCategory.asLiteral());})()))._comma(unescape("%20%21%0D%0A"))._comma(messageHeader)._comma((((nil.isNil_(aComment))).mustBeBoolean()==true ? (function(){return "";})() : (function(){return unescape("%0D%0A%09%22%20")._comma(aComment)._comma(unescape("%20%22"));})()))._comma(unescape("%0D%0A%0D%0A%09"))._comma(sourceCode)._comma(unescape("%21%20%21%0D%0A%0D%0A"));
return (function($rec){$rec.fileIn();return $rec.yourself();})(code);}
,"Builders-implementing",unescape("implement%3A%20messageHeader%0D%0A%09comment%3A%20aComment%0D%0A%09as%3A%20sourceCode%0D%0A%09category%3A%20aCategory%0D%0A%09%22%20Private%20-%20Implement%20a%20method%20in%20the%20receiver.%0D%0A%09Note%3A%20sourceCode%20must%20be%20the%20source%20code%20in%20chunk%20format.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20code%20%7C%0D%0A%09code%20%3A%3D%20%27%21%20%27%2Cself%20name%2C%0D%0A%09%09%28aCategory%20isNil%0D%0A%09%09%09ifTrue%3A%20%5B%20%27%20methods%27%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20%27%20methodsFor%3A%20%27%2C%20aCategory%20asLiteral%20%5D%29%0D%0A%2C%27%20%21%0D%0A%27%2CmessageHeader%2C%20%28aComment%20isNil%20ifTrue%3A%20%5B%20%27%27%20%5D%20ifFalse%3A%20%5B%27%0D%0A%09%22%20%27%2C%20aComment%2C%20%27%20%22%27%5D%29%2C%27%0D%0A%0D%0A%09%27%2CsourceCode%20%2C%27%21%20%21%0D%0A%0D%0A%27.%09%5Ecode%20fileIn%3B%20yourself"));
smalltalk.bind(smalltalk.Behavior,"implement:comment:as:",0
,function(messageHeader,aComment,sourceCode){var self=this;
return self.implement_comment_as_category_(messageHeader,aComment,sourceCode,nil);}
,"Builders-implementing",unescape("implement%3A%20messageHeader%20comment%3A%20aComment%20as%3A%20sourceCode%0D%0A%09%22%20Private%20-%20Implement%20a%20method%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09implement%3A%20messageHeader%0D%0A%09%09comment%3A%20aComment%0D%0A%09%09as%3A%20sourceCode%0D%0A%09%09category%3A%20nil"));
smalltalk.bind(smalltalk.Behavior,"implement:as:",0
,function(messageHeader,sourceCode){var self=this;
return self.implement_comment_as_category_(messageHeader,nil,sourceCode,nil);}
,"Builders-implementing",unescape("implement%3A%20messageHeader%20as%3A%20sourceCode%0D%0A%09%22%20Private%20-%20Implement%20a%20method%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09implement%3A%20messageHeader%0D%0A%09%09comment%3A%20nil%0D%0A%09%09as%3A%20sourceCode%0D%0A%09%09category%3A%20nil"));
smalltalk.bind(smalltalk.Behavior,"implement:as:category:",0
,function(messageHeader,sourceCode,aCategory){var self=this;
return self.implement_comment_as_category_(messageHeader,nil,sourceCode,aCategory);}
,"Builders-implementing",unescape("implement%3A%20messageHeader%20as%3A%20sourceCode%20category%3A%20aCategory%0D%0A%09%22%20Private%20-%20Implement%20a%20method%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09implement%3A%20messageHeader%0D%0A%09%09comment%3A%20nil%0D%0A%09%09as%3A%20sourceCode%0D%0A%09%09category%3A%20aCategory"));
smalltalk.bind(smalltalk.Behavior,"class:implement:comment:as:category:",0
,function(aClass,message,aComment,sourceCode,aCategory){var self=this;
if (aClass.isString()){return self.class_implement_comment_as_category_(aClass.evaluateIn_(nil),message,aComment,sourceCode,aCategory);};;
return aClass.implement_comment_as_category_(message,aComment,sourceCode,aCategory);}
,"Builders-implementing",unescape("class%3A%20aClass%20implement%3A%20message%0D%0A%09comment%3A%20aComment%20as%3A%20sourceCode%0D%0A%09category%3A%20aCategory%0D%0A%09%22%20Private%20-%20Implement%20a%20method%20in%20aClass.%20%22%0D%0A%0D%0A%09aClass%20isString%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%0D%0A%09%09%09class%3A%20%28aClass%20evaluateIn%3A%20nil%29%0D%0A%09%09%09implement%3A%20message%0D%0A%09%09%09comment%3A%20aComment%0D%0A%09%09%09as%3A%20sourceCode%0D%0A%09%09%09category%3A%20aCategory%0D%0A%09%5D.%0D%0A%09%5EaClass%0D%0A%09%09implement%3A%20message%0D%0A%09%09comment%3A%20aComment%0D%0A%09%09as%3A%20sourceCode%0D%0A%09%09category%3A%20aCategory"));
smalltalk.bind(smalltalk.Behavior,"build:doing:",0
,function(token,aBlock){var self=this;
return smalltalk.APIBuilder.build_doing_(token,aBlock);}
,"Builders-private",unescape("build%3A%20token%20doing%3A%20aBlock%0D%0A%09%22%20Process%20a%20%28build...%3A%20spec%29%20definition%20marking%20methods%20generated%20while%20evaluating%20aBlock.%0D%0A%09Return%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5EAPIBuilder%20build%3A%20token%20doing%3A%20aBlock"));
smalltalk.bind(smalltalk.Behavior,"buildAPI:",0
,function(specs){var self=this;
specs.do_((function(spec){return ((spec.isString()).mustBeBoolean()==true ? (function(){return spec.evaluateIn_(self);})() : (function(){return self.perform_withArguments_(spec.first(),spec.copyFrom_to_((2),spec.size()));})());}));
return self;}
,"Builders-obsolete",unescape("buildAPI%3A%20specs%0D%0A%09%22%20Generate%20code%20to%20implement%20API%20protocoll%20defined%20by%20specs.%0D%0A%09WARNING%3A%20OBSOLETE%20-%20this%20builder%20generate%20changes%20that%20will%20not%20be%20filedOut%20to%20builder%20code.%0D%0A%09specs%20must%20be%20an%20array%20of%20%3A%0D%0A%09%09-%20string%20%3A%20it%20will%20be%20considered%20as%20an%20evaluable%20expression%20to%20be%20executed%20in%20the%20context%20of%20the%20receiver.%0D%0A%09%09-%20anArray%20%3A%20of%20%23%28%20selector%20arg1%20arg2%20arg3%20arg4...%20%29%20to%20be%20performed%20for%20code%20generation/setup%20of%20the%20receiver%0D%0A%09%09%09where%20selector%20is%20frequently%20a%20build...%3A%20selector.%0D%0A%09%22%0D%0A%0D%0A%09specs%20do%3A%20%5B%3Aspec%7C%0D%0A%09%09spec%20isString%0D%0A%09%09ifTrue%3A%20%5B%20spec%20evaluateIn%3A%20self%20%5D%0D%0A%09%09ifFalse%3A%20%5B%0D%0A%09%09%09self%09perform%3A%20spec%20first%0D%0A%09%09%09%09withArguments%3A%20%28spec%20copyFrom%3A%202%20to%3A%20spec%20size%29%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"stMessageHeader:",0
,function(msgHeader){var self=this;
return msgHeader;}
,"Builders-private",unescape("stMessageHeader%3A%20msgHeader%0D%0A%09%22%20Private%20-%20Return%20a%20message%20header%20string%20with%20nice%20smalltalk%20looking.%0D%0A%09Default%20implementation%20leave%20the%20message%20header%20as%20is.%0D%0A%09This%20method%20can%20be%20refined%20by%20subclasses%20to%20transform%20message%20pattern%20for%20better%20code%20generation.%0D%0A%09%22%0D%0A%0D%0A%09%5EmsgHeader"));
smalltalk.bind(smalltalk.Behavior,"buildAccessor:",0
,function(accessor){var self=this;
return self.build_doing_("Accessor"._minus_gt(accessor),(function(){return ((accessor.isString().or_((function(){return accessor.isArray().and_((function(){return accessor.size()._gt((1)).and_((function(){return accessor.second()._eq("initialize");}));}));}))).mustBeBoolean()==true ? (function(){return self.buildGetter_(accessor)._comma(self.buildSetter_(accessor));})() : (function(){return accessor.compose_((function(tuple){return self.implement_comment_as_category_(tuple.first(),tuple.at_((2)),tuple.at_((3)),"accessing");}));})());}));}
,"Builders-api",unescape("buildAccessor%3A%20accessor%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20access%20native%20property.%20%22%0D%0A%0D%0A%09%5Eself%20build%3A%20%23Accessor%20-%3E%20accessor%20doing%3A%20%5B%0D%0A%09%09%28accessor%20isString%20or%3A%20%5B%0D%0A%09%09%09accessor%20isArray%20and%3A%20%5B%0D%0A%09%09%09accessor%20size%20%3E%201%20and%3A%20%5B%0D%0A%09%09%09accessor%20second%20%3D%20%23initialize%20%5D%20%5D%0D%0A%09%09%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%09%28self%20buildGetter%3A%20accessor%29%0D%0A%09%09%09%2C%28self%20buildSetter%3A%20accessor%29%0D%0A%09%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09accessor%20compose%3A%20%5B%3Atuple%7C%0D%0A%09%09%09%09self%20implement%3A%20tuple%20first%20comment%3A%20%28tuple%20at%3A%202%29%0D%0A%09%09%09%09%20%20%20%20%20%20as%3A%20%28tuple%20at%3A%203%29%20category%3A%20%23accessing%0D%0A%09%09%09%5D%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"buildGetterCode:",0
,function(aSelector){var self=this;
return unescape("%5E")._comma(aSelector);}
,"Builders-private",unescape("buildGetterCode%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20source%20code%20to%20get%20aSelector.%20%22%0D%0A%0D%0A%09%5E%27%5E%27%20%2CaSelector"));
smalltalk.bind(smalltalk.Behavior,"buildGetter:",0
,function(spec){var self=this;
return self.build_doing_("Getter"._minus_gt(spec),(function(){return ((spec.isArray().and_((function(){return spec.size()._gt((1)).and_((function(){return spec.second()._eq("initialize");}));}))).mustBeBoolean()==true ? (function(){return self.buildGetter_initialize_(spec.first(),spec.last());})() : (function(){return self.implement_comment_as_category_(self.stMessageHeader_(spec),"Return the "._comma(spec)._comma(" of the receiver."),self.buildGetterCode_(spec),"accessing");})());}));}
,"Builders-api",unescape("buildGetter%3A%20spec%0D%0A%09%22%20Build%20method%20for%20getter.%20%22%0D%0A%0D%0A%09%5Eself%20build%3A%20%23Getter%20-%3E%20spec%20doing%3A%20%5B%0D%0A%09%09%28spec%20isArray%20and%3A%20%5B%20spec%20size%20%3E%201%20and%3A%20%5B%0D%0A%09%09%20spec%20second%20%3D%20%23initialize%20%5D%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%09self%20buildGetter%3A%20spec%20first%20initialize%3A%20spec%20last%0D%0A%09%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09self%0D%0A%09%09%09%09implement%3A%20%28self%20stMessageHeader%3A%20spec%29%0D%0A%09%09%09%09comment%3A%20%27Return%20the%20%27%20%2Cspec%20%2C%27%20of%20the%20receiver.%27%0D%0A%09%09%09%09as%3A%20%28self%20buildGetterCode%3A%20spec%29%20category%3A%20%23accessing%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"buildInitialize:with:",0
,function(aSelector,initializer){var self=this;
return self.implement_comment_as_category_("initialize"._comma(aSelector.asSmalltalkName_(true)),unescape("Private%20-%20Initialize%20the%20")._comma(aSelector)._comma(" of the receiver."),initializer,"initialize");}
,"Builders-private",unescape("buildInitialize%3A%20aSelector%20with%3A%20initializer%0D%0A%09%22%20Private%20-%20Build%20method%20to%20initialize%20instance%20variable%20aSelector.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09implement%3A%20%23initialize%20%2C%28aSelector%20asSmalltalkName%3A%20true%29%0D%0A%09%09comment%3A%20%27Private%20-%20Initialize%20the%20%27%20%2CaSelector%20%2C%27%20of%20the%20receiver.%27%0D%0A%09%09as%3A%20initializer%0D%0A%09%09category%3A%20%23initialize"));
smalltalk.bind(smalltalk.Behavior,"buildGetter:initialize:",0
,function(aSelector,initializer){var self=this;
var code=nil;
code=((initializer._eq("initialize")).mustBeBoolean()==true ? (function(){return unescape("%5Eself%20implementedBySubclass%3A%20%23initialize");})() : (function(){return ((initializer.notEmpty()).mustBeBoolean()==true ? (function(){return ((initializer.first()._eq(".")).mustBeBoolean()==true ? (function(){return initializer.copyFrom_to_((2),initializer.size());})() : (function(){return aSelector._comma(unescape("%20%3A%3D%20"))._comma(((initializer.first().isUppercase().and_((function(){return initializer.asArrayOfSubstrings().size()._eq((1));}))).mustBeBoolean()==true ? (function(){return initializer._comma(" new");})() : (function(){return initializer;})()));})());})() : nil);})());
return (((nil.isNil_(code)===false)).mustBeBoolean()==true ? (function(){return self.buildInitialize_with_(aSelector,code);})() : (function(){return "";})())._comma(self.implement_comment_as_category_(self.stMessageHeader_(aSelector),"Return the "._comma(aSelector)._comma(" of the receiver."),aSelector._comma(unescape("%20isNil%20ifTrue%3A%20%5B%20self%20"))._comma("initialize")._comma(aSelector.asSmalltalkName_(true))._comma(unescape("%20%5D.%0D%0A%09%5E"))._comma(aSelector),"accessing"));}
,"Builders-private",unescape("buildGetter%3A%20aSelector%20initialize%3A%20initializer%0D%0A%09%22%20Private%20-%20Build%20method%20to%20get%20instance%20aSelector%20w/deferred%20initialization%20mode.%0D%0A%09Note%3A%20%23initializer%20can%20be%3A%0D%0A%09%09%23initialize%20to%20generate%20default/deferred%20implementation.%0D%0A%09%09empty%20string%20to%20do%20NOT%20generate%20%23initialize...%20method.%0D%0A%09%09smalltalk%20code%20starting%20with%20%24.%20to%20use%20that%20code%20as%20method%20implementation.%0D%0A%09%09smalltalk%20expression%20for%20asignment%20%28to%20be%20used%20as%20assignment%20to%20instance%29.%0D%0A%09%09GlobalName%20if%20code%20do%20not%20have%20blanks%20and%20start%20with%20uppercase%2C%20it%20will%20be%20used%20as%20class%20to%20instantiate%20during%20initialization.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20code%20%7C%0D%0A%09code%20%3A%3D%20initializer%20%3D%20%23initialize%0D%0A%09%20%20ifTrue%3A%20%5B%20%27%5Eself%20implementedBySubclass%3A%20%23initialize%27%20%5D%20ifFalse%3A%20%5B%0D%0A%09%09initializer%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%09%09initializer%20first%20%3D%20%24.%0D%0A%09%09%09ifTrue%3A%20%5B%20initializer%20copyFrom%3A%202%20to%3A%20initializer%20size%20%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09%09aSelector%20%2C%27%20%3A%3D%20%27%20%2C%28%0D%0A%09%09%09%09%09%28initializer%20first%20isUppercase%20and%3A%20%5B%20initializer%20asArrayOfSubstrings%20size%20%3D%201%20%5D%29%0D%0A%09%09%09%09%09ifTrue%3A%20%5B%20initializer%20%2C%27%20new%27%20%5D%0D%0A%09%09%09%09%09ifFalse%3A%20%5B%20initializer%20%5D%29%0D%0A%09%09%09%5D%0D%0A%09%09%5D%0D%0A%09%20%20%5D.%0D%0A%09%5E%28code%20notNil%20ifTrue%3A%20%5B%20self%20buildInitialize%3A%20aSelector%20with%3A%20code%20%5D%20ifFalse%3A%20%5B%20%27%27%20%5D%29%0D%0A%09%2C%28self%0D%0A%09%09implement%3A%20%28self%20stMessageHeader%3A%20aSelector%29%0D%0A%09%09comment%3A%20%27Return%20the%20%27%20%2CaSelector%20%2C%27%20of%20the%20receiver.%27%0D%0A%09%09as%3A%20aSelector%2C%27%20isNil%20ifTrue%3A%20%5B%20self%20%27%20%2C%20%23initialize%20%2C%28aSelector%20asSmalltalkName%3A%20true%29%20%2C%27%20%5D.%0D%0A%09%5E%27%20%2CaSelector%20category%3A%20%23accessing%29"));
smalltalk.bind(smalltalk.Behavior,"buildSetterCode:",0
,function(aSelector){var self=this;
return aSelector._comma(unescape("%20%3A%3D%20anObject"));}
,"Builders-private",unescape("buildSetterCode%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20source%20code%20to%20set%20aSelector.%20%22%0D%0A%0D%0A%09%5EaSelector%20%2C%27%20%3A%3D%20anObject%27"));
smalltalk.bind(smalltalk.Behavior,"buildSetter:",0
,function(spec){var self=this;
if (spec.isArray().and_((function(){return spec.size()._gt((1)).and_((function(){return spec.second()._eq("initialize");}));}))){return self.buildSetter_(spec.first());};;
return self.build_doing_("Setter"._minus_gt(spec),(function(){return self.implement_comment_as_category_(self.stMessageHeader_(spec)._comma(": anObject"),"Set the "._comma(spec)._comma(" of the receiver."),self.buildSetterCode_(spec),"accessing");}));}
,"Builders-api",unescape("buildSetter%3A%20spec%0D%0A%09%22%20Build%20method%20for%20setter.%20%22%0D%0A%0D%0A%09%28spec%20isArray%20and%3A%20%5B%20spec%20size%20%3E%201%20and%3A%20%5B%0D%0A%09%20spec%20second%20%3D%20%23initialize%20%5D%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20buildSetter%3A%20spec%20first%0D%0A%09%5D.%0D%0A%09%5Eself%20build%3A%20%23Setter%20-%3E%20spec%20doing%3A%20%5B%0D%0A%09%09self%0D%0A%09%09%09implement%3A%20%28self%20stMessageHeader%3A%20spec%29%20%2C%27%3A%20anObject%27%0D%0A%09%09%09comment%3A%20%27Set%20the%20%27%20%2Cspec%20%2C%27%20of%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%28self%20buildSetterCode%3A%20spec%29%0D%0A%09%09%09category%3A%20%23accessing%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"buildGetters:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildGetters_(specs.asArrayOfSubstrings());};;
return self.build_doing_("Getters"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildGetter_(each);}));}));}
,"Builders-api",unescape("buildGetters%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20get%20instances.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildGetters%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23Getters%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildGetter%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"buildSetters:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildSetters_(specs.asArrayOfSubstrings());};;
return self.build_doing_("Setters"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildSetter_(each);}));}));}
,"Builders-api",unescape("buildSetters%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20set%20instances.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildSetters%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23Setters%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildSetter%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"buildAccessors:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildAccessors_(specs.asArrayOfSubstrings());};;
return self.build_doing_("Accessors"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildAccessor_(each);}));}));}
,"Builders-api",unescape("buildAccessors%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20access%20instances.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildAccessors%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23Accessors%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildAccessor%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"buildAccessors",0
,function(){var self=this;
return self.build_doing_("Accessors"._minus_gt([]),(function(){return self.buildAccessors_(self.instanceVariableNames());}));}
,"Builders-api",unescape("buildAccessors%0D%0A%09%22%20Private%20-%20Build%20methods%20to%20access%20all%20instance%20variables.%20%22%0D%0A%0D%0A%09%5Eself%20build%3A%20%23Accessors%20-%3E%20%23%28%29%20doing%3A%20%5B%0D%0A%09%09self%20buildAccessors%3A%20self%20instanceVariableNames%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"buildPool:with:",0
,function(poolName,spec){var self=this;
return self.build_doing_("Pool"._minus_gt(spec),(function(){return (function($rec){$rec.definePool_with_(poolName,spec);$rec.printDefinePool_with_(poolName,spec);return $rec.yourself();})(smalltalk.Smalltalk.current());}));}
,"Builders-api",unescape("buildPool%3A%20poolName%20with%3A%20spec%0D%0A%09%22%20Private%20-%20Build%20a%20PoolDictionary%20with%20spec%20%28and%20log%20definition%20in%20log%20stream%29.%20%22%0D%0A%0D%0A%09%5Eself%20build%3A%20%23Pool%20-%3E%20spec%20doing%3A%20%5B%0D%0A%09%09Smalltalk%20current%0D%0A%09%09%09definePool%3A%20poolName%20with%3A%20spec%3B%0D%0A%09%09%09printDefinePool%3A%20poolName%20with%3A%20spec%3B%0D%0A%09%09%09yourself%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Behavior,"__tostring",0
,function(){var self=this;
return self.toString();}
,"Lua-printing",unescape("__tostring%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.Class.$klass,"categories",0
,function(){var self=this;
return smalltalk.Smalltalk.classes().collect_((function(each){return each.category();})).asHashedSet();}
,"export",unescape("categories%0D%0A%09%22%20Return%20a%20collection%20with%20all%20defined%20categories.%20%22%0D%0A%0D%0A%09%5E%28Smalltalk%20classes%20collect%3A%20%5B%3Aeach%20%7C%20each%20category%20%5D%29%20asHashedSet"));
smalltalk.bind(smalltalk.Class.$klass,"uncategorized",0
,function(){var self=this;
return smalltalk.Smalltalk.classes().select_((function(each){return each.category()._eq("unclassified");}));}
,"export",unescape("uncategorized%0D%0A%09%22%20Return%20a%20collection%20with%20classes%20without%20category.%20%22%0D%0A%0D%0A%09%5ESmalltalk%20classes%20select%3A%20%5B%3Aeach%20%7C%20each%20category%20%3D%20%23unclassified%20%5D"));
smalltalk.bind(smalltalk.Class,"category",0
,function(){var self=this;
return ((function(){var $1$=(self["$category"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("category%0D%0A%09%22%20Return%20the%20category%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24category%27"));
smalltalk.bind(smalltalk.Class,"category:",0
,function(aString){var self=this;
(self["$category"]=(aString));
return self;}
,"accessing",unescape("category%3A%20aString%0D%0A%09%22%20Set%20the%20category%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20basicAt%3A%20%27%24category%27%20put%3A%20aString"));
smalltalk.bind(smalltalk.Class,"classVariables",0
,function(){var self=this;
var names=nil;
names=((function(){var $1$=(self["$classVariableNames"]);if(nil.isNil_($1$))return ((function(){return nil;}).value());return $1$;})());
if ((nil.isNil_(names))){return ((function(){var $1$=(self["$classVariables"]);if(nil.isNil_($1$))return nil;return $1$;})());};;
self.basicDelete_("$classVariableNames");
self.classVariableNames_(names);
return ((function(){var $1$=(self["$classVariables"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("classVariables%0D%0A%09%22%20Return%20the%20class%20variables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20names%20%7C%0D%0A%09names%20%3A%3D%20self%20basicAt%3A%20%27%24classVariableNames%27%20ifAbsent%3A%20%5B%5D.%0D%0A%09names%20isNil%20ifTrue%3A%20%5B%20%5Eself%20basicAt%3A%20%27%24classVariables%27%20%5D.%0D%0A%09self%20basicDelete%3A%20%27%24classVariableNames%27.%0D%0A%09self%20classVariableNames%3A%20names.%0D%0A%09%5Eself%20basicAt%3A%20%27%24classVariables%27"));
smalltalk.bind(smalltalk.Class,"classVariables:",0
,function(aDictionary){var self=this;
if ((nil.isNil_(aDictionary)).or_((function(){return aDictionary.isEmpty();}))){return self.basicDelete_("$classVariables");};;
return (self["$classVariables"]=(aDictionary));}
,"accessing",unescape("classVariables%3A%20aDictionary%0D%0A%09%22%20Set%20the%20class%20variables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%28aDictionary%20isNil%20or%3A%20%5B%20aDictionary%20isEmpty%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20basicDelete%3A%20%27%24classVariables%27%0D%0A%09%5D.%0D%0A%09%5Eself%20basicAt%3A%20%27%24classVariables%27%20put%3A%20aDictionary"));
smalltalk.bind(smalltalk.Class,"poolDictionaries",0
,function(){var self=this;
return ((function(){var $1$=(self["$poolDictionaries"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("poolDictionaries%0D%0A%09%22%20Return%20the%20poolDictionaries%20of%20the%20receiver%20%28anArray%20or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24poolDictionaries%27"));
smalltalk.bind(smalltalk.Class,"poolDictionaries:",0
,function(anArray){var $s8Ret$={name:"stReturn"};try{var self=this;
if ((nil.isNil_(anArray)===false)){if (anArray.isEmpty()){return self.poolDictionaries_(nil);};;
((anArray.size()._eq(anArray.asSet().size())).mustBeBoolean()==true ? nil : (function(){return (function(){$s8Ret$.result=(function(){return self.error_("Invalid pool dictionary names "._comma(anArray.asLiteral()))})();throw($s8Ret$)})();})());};;
(function(){$s8Ret$.result=(function(){return (self["$poolDictionaries"]=(anArray))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("poolDictionaries%3A%20anArray%0D%0A%09%22%20Set%20the%20poolDictionaries%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09anArray%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09anArray%20isEmpty%20ifTrue%3A%20%5B%0D%0A%09%09%09%5Eself%20poolDictionaries%3A%20nil%0D%0A%09%09%5D.%0D%0A%09%09anArray%20size%20%3D%20anArray%20asSet%20size%20ifFalse%3A%20%5B%0D%0A%09%09%09%5Eself%20error%3A%20%27Invalid%20pool%20dictionary%20names%20%27%20%2CanArray%20asLiteral%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5Eself%20basicAt%3A%20%27%24poolDictionaries%27%20put%3A%20anArray"));
smalltalk.bind(smalltalk.Class,"classVariableNames",0
,function(){var self=this;
return (((nil.isNil_(self.classVariables()))).mustBeBoolean()==true ? (function(){return smalltalk.Array.$$new();})() : (function(){return self.classVariables().keys();})());}
,"accessing",unescape("classVariableNames%0D%0A%09%22%20Return%20the%20class%20variable%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20classVariables%20isNil%0D%0A%09%09ifTrue%3A%20%5B%20Array%20new%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20self%20classVariables%20keys%20%5D"));
smalltalk.bind(smalltalk.Class,"classVariablesString",0
,function(){var self=this;
return self.classVariableNames().join_(" ");}
,"accessing",unescape("classVariablesString%0D%0A%09%22%20Return%20the%20class%20variable%20names%20of%20the%20receiver%20as%20aString.%20%22%0D%0A%0D%0A%09%5Eself%20classVariableNames%20join%3A%20%27%20%27"));
smalltalk.bind(smalltalk.Class,"poolDictionaryNames",0
,function(){var self=this;
return (((nil.isNil_(self.poolDictionaries()))).mustBeBoolean()==true ? (function(){return smalltalk.Array.$$new();})() : (function(){return self.poolDictionaries();})());}
,"accessing",unescape("poolDictionaryNames%0D%0A%09%22%20Return%20the%20pool%20dictionary%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20poolDictionaries%20isNil%0D%0A%09%09ifTrue%3A%20%5B%20Array%20new%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20self%20poolDictionaries%20%5D"));
smalltalk.bind(smalltalk.Class,"poolDictionariesString",0
,function(){var self=this;
return self.poolDictionaryNames().join_(" ");}
,"accessing",unescape("poolDictionariesString%0D%0A%09%22%20Return%20the%20poolDictionary%20names%20of%20the%20receiver%20as%20aString.%20%22%0D%0A%0D%0A%09%5Eself%20poolDictionaryNames%20join%3A%20%27%20%27"));
smalltalk.bind(smalltalk.Class,"classVariableNames:",0
,function(classVarNames){var self=this;
var names=nil;
var dictionary=nil;
var old=nil;
if ((nil.isNil_(classVarNames))){return self.classVariables_(nil);};;
names=((classVarNames.isString()).mustBeBoolean()==true ? (function(){return classVarNames.asArrayOfSubstrings();})() : (function(){return classVarNames;})());
if (!(names.size()._eq(names.asSet().size()))){return self.error_("Invalid class variable names "._comma(names.asLiteral()));};;
dictionary=smalltalk.PoolDictionary.$$new();
old=self.classVariables();
if ((nil.isNil_(old))){old=dictionary;};;
names.do_((function(each){return dictionary.at_put_(each,old.at_ifAbsent_(each,(function(){return nil;})));}));
self.classVariables_(dictionary);
return self;}
,"accessing",unescape("classVariableNames%3A%20classVarNames%0D%0A%09%22%20Set%20the%20variable%20names%20defined%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20names%20dictionary%20old%20%7C%0D%0A%09classVarNames%20isNil%20ifTrue%3A%20%5B%20%5Eself%20classVariables%3A%20nil%20%5D.%0D%0A%09names%20%3A%3D%20classVarNames%20isString%0D%0A%09%09ifTrue%3A%20%5B%20classVarNames%20asArrayOfSubstrings%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20classVarNames%20%5D.%0D%0A%09names%20size%20%3D%20names%20asSet%20size%20ifFalse%3A%20%5B%0D%0A%09%09%5Eself%20error%3A%20%27Invalid%20class%20variable%20names%20%27%20%2Cnames%20asLiteral%0D%0A%09%5D.%0D%0A%09dictionary%20%3A%3D%20PoolDictionary%20new.%0D%0A%09old%20%3A%3D%20self%20classVariables.%20old%20isNil%20ifTrue%3A%20%5B%20old%20%3A%3D%20dictionary%20%5D.%0D%0A%09names%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09dictionary%20at%3A%20each%20put%3A%20%28old%20at%3A%20each%20ifAbsent%3A%20%5B%20nil%20%5D%29%0D%0A%09%5D.%0D%0A%09self%20classVariables%3A%20dictionary."));
smalltalk.bind(smalltalk.Class,"poolDictionaryNames:",0
,function(poolVarNames){var self=this;
self.poolDictionaries_(((poolVarNames.isString()).mustBeBoolean()==true ? (function(){return poolVarNames.asArrayOfSubstrings();})() : (function(){return poolVarNames;})()));
return self;}
,"accessing",unescape("poolDictionaryNames%3A%20poolVarNames%0D%0A%09%22%20Set%20the%20pool%20dictionary%20names%20known%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20poolDictionaries%3A%20%28poolVarNames%20isString%0D%0A%09%09ifTrue%3A%20%5B%20poolVarNames%20asArrayOfSubstrings%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20poolVarNames%20%5D%29."));
smalltalk.bind(smalltalk.Class,"subclass:instanceVariableNames:",0
,function(aString,aString2){var self=this;
return self.subclass_instanceVariableNames_classVariableNames_(aString,aString2,"");}
,"instantiation",unescape("subclass%3A%20aString%20instanceVariableNames%3A%20aString2%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20%27%27"));
smalltalk.bind(smalltalk.Class,"subclass:category:",0
,function(aString,aCategory){var self=this;
return self.subclass_instanceVariableNames_category_(aString,"",aCategory);}
,"instantiation",unescape("subclass%3A%20aString%20category%3A%20aCategory%0D%0A%09%22%20Create%20a%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20%27%27%0D%0A%09%09category%3A%20aCategory"));
smalltalk.bind(smalltalk.Class,"subclass:instanceVariableNames:category:",0
,function(aString,aString2,aString3){var self=this;
return smalltalk.ClassBuilder.$$new().superclass_subclass_instanceVariableNames_category_(self,aString,aString2,aString3);}
,"instantiation",unescape("subclass%3A%20aString%20instanceVariableNames%3A%20aString2%20category%3A%20aString3%0D%0A%09%22%20Create%20a%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EClassBuilder%20new%0D%0A%09%09superclass%3A%20self%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09category%3A%20aString3"));
smalltalk.bind(smalltalk.Class,"isClass",0
,function(){var self=this;
return true;}
,"testing",unescape("isClass%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20Class.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.Class,"printString",0
,function(){var self=this;
return self.name();}
,"printing",unescape("printString%0D%0A%09%22%20Return%20a%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20name"));
smalltalk.bind(smalltalk.Class,"instanceClass",0
,function(){var self=this;
return self;}
,"accessing",unescape("instanceClass%0D%0A%09%22%20Return%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Class,"rename:",0
,function(aString){var self=this;
self.smalltalk().renameClass(self,aString);
return self;}
,"renaming",unescape("rename%3A%20aString%0D%0A%09%22%20Rename%20the%20receiver%20to%20aString.%20%22%0D%0A%0D%0A%09self%20smalltalk%20%23renameClass%3A%20self%20to%3A%20aString"));
smalltalk.bind(smalltalk.Class,"removeFromSystem",0
,function(){var self=this;
self.subclasses().do_((function(each){return each.removeFromSystem();}));
self.smalltalk().removeClass(self);
return true;}
,"remove",unescape("removeFromSystem%0D%0A%09%22%20Remove%20the%20receiver%20from%20system.%20%22%0D%0A%0D%0A%09self%20subclasses%20do%3A%20%5B%3Aeach%7C%20each%20removeFromSystem%20%5D.%0D%0A%09self%20smalltalk%20%23removeClass%3A%20self.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.Class,"removeSources",0
,function(){var self=this;
self.$klass.methodsDo_((function(each){return each.removeSources();}));
self.methodsDo_((function(each){return each.removeSources();}));
(function($rec){$rec.comment_("");return $rec.category_("s8");})(self);
return self;}
,"tools",unescape("removeSources%0D%0A%09%22%20Remove%20all%20development%20sources%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20class%20methodsDo%3A%20%5B%3Aeach%20%7C%20each%20removeSources%20%5D.%0D%0A%09self%20methodsDo%3A%20%5B%3Aeach%20%7C%20each%20removeSources%20%5D.%0D%0A%09self%20comment%3A%20%27%27%3B%20category%3A%20%23s8"));
smalltalk.bind(smalltalk.Class,"variableByteSubclass:classVariableNames:poolDictionaries:",0
,function(aString,classVarNames,poolVarNames){var self=this;
return self.subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString,"",classVarNames,poolVarNames);}
,"instantiation",unescape("variableByteSubclass%3A%20aString%0D%0A%09classVariableNames%3A%20classVarNames%0D%0A%09poolDictionaries%3A%20poolVarNames%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%22%09self%20note%3A%20%27WARNING%3A%20%27%2CaString%2C%27%20defined%20as%20variableByteSubclass%3A...%27.%0D%0A%22%09%5Eself%20%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20%27%27%0D%0A%09%09classVariableNames%3A%20classVarNames%0D%0A%09%09poolDictionaries%3A%20poolVarNames"));
smalltalk.bind(smalltalk.Class,"variableSubclass:instanceVariableNames:classVariableNames:poolDictionaries:",0
,function(aString,aString2,classVarNames,poolVarNames){var self=this;
self.note_("WARNING: "._comma(aString)._comma(" defined as variableSubclass:..."));
return self.subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString,aString2,classVarNames,poolVarNames);}
,"instantiation",unescape("variableSubclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09classVariableNames%3A%20classVarNames%0D%0A%09poolDictionaries%3A%20poolVarNames%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20note%3A%20%27WARNING%3A%20%27%2CaString%2C%27%20defined%20as%20variableSubclass%3A...%27.%0D%0A%09%5Eself%20%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20classVarNames%0D%0A%09%09poolDictionaries%3A%20poolVarNames"));
smalltalk.bind(smalltalk.Class,"subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:",0
,function(aString,aString2,classVarNames,poolVarNames,aCategory){var self=this;
if ((nil.isNil_(self.smalltalk().globalAt_(aString))===false)){self.note_("WARNING: class REdefinition for "._comma(aString));};;
return smalltalk.ClassBuilder.$$new().superclass_subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(self,aString,aString2,classVarNames,poolVarNames,aCategory);}
,"instantiation",unescape("subclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09classVariableNames%3A%20classVarNames%0D%0A%09poolDictionaries%3A%20poolVarNames%0D%0A%09category%3A%20aCategory%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%28self%20smalltalk%20globalAt%3A%20aString%29%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09self%20note%3A%20%27WARNING%3A%20class%20REdefinition%20for%20%27%2CaString.%0D%0A%09%09%22%5Eself%20smalltalk%20globalAt%3A%20aString%22%0D%0A%09%5D.%0D%0A%0D%0A%22%09self%20note%3A%20%27Creating%20%27%2CaString.%22%0D%0A%09%5EClassBuilder%20new%0D%0A%09%09superclass%3A%20self%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20classVarNames%0D%0A%09%09poolDictionaries%3A%20poolVarNames%0D%0A%09%09category%3A%20aCategory"));
smalltalk.bind(smalltalk.Class,"subclass:instanceVariableNames:classVariableNames:category:",0
,function(aString,aString2,classVarNames,aCategory){var self=this;
return self.subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString,aString2,classVarNames,"",aCategory);}
,"instantiation",unescape("subclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09classVariableNames%3A%20classVarNames%0D%0A%09category%3A%20aCategory%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20classVarNames%0D%0A%09%09poolDictionaries%3A%20%27%27%0D%0A%09%09category%3A%20aCategory"));
smalltalk.bind(smalltalk.Class,"subclass:instanceVariableNames:classVariableNames:poolDictionaries:",0
,function(aString,aString2,classVarNames,poolVarNames){var self=this;
return self.subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString,aString2,classVarNames,poolVarNames,nil);}
,"instantiation",unescape("subclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09classVariableNames%3A%20classVarNames%0D%0A%09poolDictionaries%3A%20poolVarNames%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20classVarNames%0D%0A%09%09poolDictionaries%3A%20poolVarNames%0D%0A%09%09category%3A%20nil"));
smalltalk.bind(smalltalk.Class,"subclass:instanceVariableNames:poolDictionaries:category:",0
,function(aString,aString2,poolVarNames,aCategory){var self=this;
return self.subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString,aString2,"",poolVarNames,aCategory);}
,"instantiation",unescape("subclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09poolDictionaries%3A%20poolVarNames%0D%0A%09category%3A%20aCategory%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20%27%27%0D%0A%09%09poolDictionaries%3A%20poolVarNames%0D%0A%09%09category%3A%20aCategory"));
smalltalk.bind(smalltalk.Class,"variableByteSubclass:classVariableNames:",0
,function(aString,classVarNames){var self=this;
return self.variableByteSubclass_classVariableNames_poolDictionaries_(aString,classVarNames,"");}
,"instantiation",unescape("variableByteSubclass%3A%20aString%20classVariableNames%3A%20classVarNames%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09variableByteSubclass%3A%20aString%0D%0A%09%09classVariableNames%3A%20classVarNames%0D%0A%09%09poolDictionaries%3A%20%27%27"));
smalltalk.bind(smalltalk.Class,"variableByteSubclass:poolDictionaries:",0
,function(aString,aString2){var self=this;
return self.variableByteSubclass_classVariableNames_poolDictionaries_(aString,"",aString2);}
,"instantiation",unescape("variableByteSubclass%3A%20aString%20poolDictionaries%3A%20aString2%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09variableByteSubclass%3A%20aString%0D%0A%09%09classVariableNames%3A%20%27%27%0D%0A%09%09poolDictionaries%3A%20aString2"));
smalltalk.bind(smalltalk.Class,"variableByteSubclass:",0
,function(aString){var self=this;
return self.variableByteSubclass_classVariableNames_(aString,"");}
,"instantiation",unescape("variableByteSubclass%3A%20aString%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09variableByteSubclass%3A%20aString%0D%0A%09%09classVariableNames%3A%20%27%27"));
smalltalk.bind(smalltalk.Class,"variableSubclass:instanceVariableNames:classVariableNames:",0
,function(aString,aString2,classVarNames){var self=this;
return self.variableSubclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString,aString2,classVarNames,"");}
,"instantiation",unescape("variableSubclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09classVariableNames%3A%20classVarNames%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20%0D%0A%09%09variableSubclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20classVarNames%0D%0A%09%09poolDictionaries%3A%20%27%27"));
smalltalk.bind(smalltalk.Class,"variableSubclass:instanceVariableNames:",0
,function(aString,aString2){var self=this;
return self.variableSubclass_instanceVariableNames_classVariableNames_(aString,aString2,"");}
,"instantiation",unescape("variableSubclass%3A%20aString%20instanceVariableNames%3A%20aString2%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20%0D%0A%09%09variableSubclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20%27%27"));
smalltalk.bind(smalltalk.Class,"variableSubclass:classVariableNames:",0
,function(aString,aString2){var self=this;
return self.variableSubclass_instanceVariableNames_classVariableNames_(aString,"",aString2);}
,"instantiation",unescape("variableSubclass%3A%20aString%20classVariableNames%3A%20aString2%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20%0D%0A%09%09variableSubclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20%27%27%0D%0A%09%09classVariableNames%3A%20aString2"));
smalltalk.bind(smalltalk.Class,"variableSubclass:poolDictionaries:",0
,function(aString,aString2){var self=this;
return self.variableSubclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString,"","",aString2);}
,"instantiation",unescape("variableSubclass%3A%20aString%20poolDictionaries%3A%20aString2%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20%0D%0A%09%09variableSubclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20%27%27%0D%0A%09%09classVariableNames%3A%20%27%27%0D%0A%09%09poolDictionaries%3A%20aString2"));
smalltalk.bind(smalltalk.Class,"variableSubclass:",0
,function(aString){var self=this;
return self.variableSubclass_instanceVariableNames_(aString,"");}
,"instantiation",unescape("variableSubclass%3A%20aString%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20%0D%0A%09%09variableSubclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20%27%27"));
smalltalk.bind(smalltalk.Class,"subclass:instanceVariableNames:classVariableNames:",0
,function(aString,aString2,classVarNames){var self=this;
return self.subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString,aString2,classVarNames,"");}
,"instantiation",unescape("subclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09classVariableNames%3A%20classVarNames%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20classVarNames%0D%0A%09%09poolDictionaries%3A%20%27%27"));
smalltalk.bind(smalltalk.Class,"subclass:instanceVariableNames:poolDictionaries:",0
,function(aString,aString2,aString3){var self=this;
return self.subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString,aString2,"",aString3);}
,"instantiation",unescape("subclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09poolDictionaries%3A%20aString3%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20%27%27%0D%0A%09%09poolDictionaries%3A%20aString3"));
smalltalk.bind(smalltalk.Class,"subclass:classVariableNames:",0
,function(aString,aString2){var self=this;
return self.subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString,"",aString2,"");}
,"instantiation",unescape("subclass%3A%20aString%20classVariableNames%3A%20aString2%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20%27%27%0D%0A%09%09classVariableNames%3A%20aString2%0D%0A%09%09poolDictionaries%3A%20%27%27"));
smalltalk.bind(smalltalk.Class,"subclass:poolDictionaries:",0
,function(aString,aString2){var self=this;
return self.subclass_instanceVariableNames_classVariableNames_poolDictionaries_(aString,"","",aString2);}
,"instantiation",unescape("subclass%3A%20aString%20poolDictionaries%3A%20aString2%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20%27%27%0D%0A%09%09classVariableNames%3A%20%27%27%0D%0A%09%09poolDictionaries%3A%20aString2"));
smalltalk.bind(smalltalk.Class,"subclass:",0
,function(aString){var self=this;
return self.subclass_instanceVariableNames_(aString,"");}
,"instantiation",unescape("subclass%3A%20aString%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20%27%27"));
smalltalk.bind(smalltalk.Class,"hackDNU",0
,function(){var self=this;
var all=nil;
all=smalltalk.HashedSet.$$new();
smalltalk.Smalltalk.rootClasses().do_((function(root){return root.allSubclasses().do_((function(each){return (function($rec){$rec.addAll_(each.selectors());return $rec.addAll_(each.$klass.selectors());})(all);}));}));
smalltalk.Smalltalk.rootClasses().do_((function(root){return root.selectors().do_((function(each){return all.remove_ifAbsent_(each,(function(){return nil;}));}));}));
["handleOrNil", "start:end:"].do_((function(each){return all.remove_ifAbsent_(each,(function(){return nil;}));}));
all.do_((function(selector){var mth=nil;
mth=smalltalk.CompiledMethod.$$new();(function($rec){$rec.selector_(selector);$rec.classField_(self);$rec.category_("hackDNU");return $rec.source_(selector._comma(unescape("%28dnu%29")));})(mth);mth.fn_(function(){
		this.print_("//DNU "+selector);
		this.doesNotUnderstand_(
			smalltalk.Message.receiver_selector_arguments_
				(this,selector,[].slice.call(arguments)))}
		);;return self.basicAddCompiledMethod_(mth);}));
return self;}
,"Tools-hacking",unescape("hackDNU%0D%0A%09%22%20Catch%20DNU%20condition%20in%20the%20receiver%20and%20its%20subclasses.%0D%0A%09Note%3A%20hack%20will%20persist%20until%20%23unhackDNU%20is%20sent%20to%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20all%20%7C%0D%0A%09all%20%3A%3D%20HashedSet%20new.%0D%0A%09Smalltalk%20rootClasses%20do%3A%20%5B%3Aroot%7C%0D%0A%09%09root%20allSubclasses%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%09all%20addAll%3A%20each%20selectors%3B%20addAll%3A%20each%20class%20selectors%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09Smalltalk%20rootClasses%20do%3A%20%5B%3Aroot%7C%20root%20selectors%20do%3A%20%5B%3Aeach%7C%20all%20remove%3A%20each%20ifAbsent%3A%20%5B%5D%5D%20%5D.%0D%0A%09%23%28%20%23handleOrNil%20%23start%3Aend%3A%20%29%20do%3A%20%5B%3Aeach%7C%20all%20remove%3A%20each%20ifAbsent%3A%20%5B%5D%5D.%0D%0A%09all%20do%3A%20%5B%3Aselector%7C%20%7C%20mth%20%7C%0D%0A%09%09mth%20%3A%3D%20CompiledMethod%20new.%0D%0A%09%09mth%20selector%3A%20selector%3B%20classField%3A%20self%3B%20category%3A%20%23hackDNU%3B%20source%3A%20selector%2C%27%28dnu%29%27.%0D%0A%09%09%7B%27%27%7D.%0D%0A%09%09self%20basicAddCompiledMethod%3A%20mth.%0D%0A%09%5D."));
smalltalk.bind(smalltalk.Class,"unhackDNU",0
,function(){var self=this;
self.removeSelectors_(self.selectors().select_((function(each){return smalltalk.Object._gt_gt(each).category()._eq("hackDNU");})));
return self;}
,"Tools-hacking",unescape("unhackDNU%0D%0A%09%22%20Remove%20Catch%20DNU%20hacking%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20removeSelectors%3A%20%28self%20selectors%0D%0A%09%09select%3A%20%5B%3Aeach%7C%20%28Object%20%3E%3E%20each%29%20category%20%3D%20%23hackDNU%20%5D%29"));
smalltalk.bind(smalltalk.Class,"hackDNUWhile:",0
,function(aBlock){var self=this;
var result=nil;
(function(){self.hackDNU();return result=aBlock.evaluate();}).ensure_((function(){return self.unhackDNU();}));
return result;}
,"Tools-hacking",unescape("hackDNUWhile%3A%20aBlock%0D%0A%09%22%20Catch%20DNU%20hacking%20on%20the%20receiver%20while%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09%5B%09self%20hackDNU.%0D%0A%09%09result%20%3A%3D%20aBlock%20evaluate%0D%0A%09%5D%20ensure%3A%20%5B%20self%20unhackDNU%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Class,"launch",0
,function(){var self=this;
return self;}
,"Snapshot-startup",unescape("launch%0D%0A%09%22%20Private%20-%20The%20system%20has%20been%20launched%20and%20start%20execution.%0D%0A%09The%20default%20action%20is%20to%20do%20nothing.%0D%0A%09Subclasses%20can%20refine%20this%20method%20to%20execute%20actions%20on%20startup.%0D%0A%09%22"));
smalltalk.bind(smalltalk.Class,"aboutToSaveImage:on:",0
,function(snapshot,stream){var self=this;
return self;}
,"Snapshot-snapshot",unescape("aboutToSaveImage%3A%20snapshot%20on%3A%20stream%0D%0A%09%22%20Private%20-%20A%20system%20snapshot%20will%20be%20taken%2C%20the%20receiver%20can%20prepare%20the%20contents%20to%20be%20stored%20on%20snapshot.%0D%0A%09Must%20return%20the%20receiver%20or%20a%20replacement%20to%20act%20as%20the%20receiver%20when%20persisting%20data.%0D%0A%09The%20default%20action%20return%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Class,"imageSaved:on:",0
,function(snapshot,stream){var self=this;
return self;}
,"Snapshot-snapshot",unescape("imageSaved%3A%20snapshot%20on%3A%20stream%0D%0A%09%22%20Private%20-%20A%20system%20snapshot%20has%20been%20taken%2C%20the%20receiver%20can%20emmit%20code%20to%20restore%20state.%0D%0A%09The%20default%20action%20do%20nothing.%0D%0A%09This%20method%20can%20be%20refined%20by%20the%20subclasses.%0D%0A%09%22"));
smalltalk.bind(smalltalk.Class,"aboutToExit:",0
,function(aSystemDictionary){var self=this;
return self;}
,"Snapshot-snapshot",unescape("aboutToExit%3A%20aSystemDictionary%0D%0A%09%22%20Private%20-%20A%20system%20will%20exit%2C%20and%20the%20receiver%20can%20act%20executing%20actions%20just%20before%20exiting.%0D%0A%09The%20default%20action%20do%20nothing.%0D%0A%09This%20method%20can%20be%20refined%20by%20the%20subclasses.%0D%0A%09%22"));
smalltalk.bind(smalltalk.Metaclass,"instanceClass",0
,function(){var self=this;
return self["@instanceClass"];}
,"accessing",unescape("instanceClass%0D%0A%09%22%20Return%20the%20instance%20class%20of%20receiver.%20%22%0D%0A%0D%0A%09%5EinstanceClass"));
smalltalk.bind(smalltalk.Metaclass,"instanceVariableNames:",0
,function(aString){var self=this;
smalltalk.ClassBuilder.$$new().class_instanceVariableNames_(self,aString);
return self.instanceVariableNames();}
,"accessing",unescape("instanceVariableNames%3A%20aString%0D%0A%09%22%20Set%20the%20instance%20variable%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09ClassBuilder%20new%0D%0A%09%09class%3A%20self%0D%0A%09%09instanceVariableNames%3A%20aString.%0D%0A%09%5Eself%20instanceVariableNames%20"));
smalltalk.bind(smalltalk.Metaclass,"isMetaclass",0
,function(){var self=this;
return true;}
,"testing",unescape("isMetaclass%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20metaclass.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.Metaclass,"printString",0
,function(){var self=this;
return self.instanceClass().name()._comma(" class");}
,"printing",unescape("printString%0D%0A%09%22%20Return%20a%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20instanceClass%20name%2C%20%27%20class%27"));
smalltalk.bind(smalltalk.Metaclass,"classVariables",0
,function(){var self=this;
return self.instanceClass().classVariables();}
,"accessing",unescape("classVariables%0D%0A%09%22%20Return%20the%20class%20variables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20instanceClass%20classVariables"));
smalltalk.bind(smalltalk.CompiledMethod,"source",0
,function(){var self=this;
return ((function(){var $1$=(self["$source"]);if(nil.isNil_($1$))return ((function(){return "";}).value());return $1$;})());}
,"accessing",unescape("source%0D%0A%09%22%20Return%20the%20source%20code%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24source%27%20ifAbsent%3A%20%5B%27%27%5D"));
smalltalk.bind(smalltalk.CompiledMethod,"source:",0
,function(aString){var self=this;
(self["$source"]=(aString));
return self;}
,"accessing",unescape("source%3A%20aString%0D%0A%09%22%20Set%20the%20source%20code%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20basicAt%3A%20%27%24source%27%20put%3A%20aString"));
smalltalk.bind(smalltalk.CompiledMethod,"category",0
,function(){var self=this;
return ((function(){var $1$=(self["$category"]);if(nil.isNil_($1$))return ((function(){return "";}).value());return $1$;})());}
,"accessing",unescape("category%0D%0A%09%22%20Return%20the%20category%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24category%27%20ifAbsent%3A%20%5B%27%27%5D"));
smalltalk.bind(smalltalk.CompiledMethod,"category:",0
,function(aString){var self=this;
(self["$category"]=(aString));
return self;}
,"accessing",unescape("category%3A%20aString%0D%0A%09%22%20Set%20the%20category%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20basicAt%3A%20%27%24category%27%20put%3A%20aString"));
smalltalk.bind(smalltalk.CompiledMethod,"stamp",0
,function(){var self=this;
return ((function(){var $1$=(self["$stamp"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("stamp%0D%0A%09%22%20Return%20the%20stamp%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24stamp%27"));
smalltalk.bind(smalltalk.CompiledMethod,"stamp:",0
,function(aString){var self=this;
(self["$stamp"]=(aString));
return self;}
,"accessing",unescape("stamp%3A%20aString%0D%0A%09%22%20Set%20the%20stamp%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20basicAt%3A%20%27%24stamp%27%20put%3A%20aString"));
smalltalk.bind(smalltalk.CompiledMethod,"selector",0
,function(){var self=this;
return ((function(){var $1$=(self["$selector"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("selector%0D%0A%09%22%20Return%20the%20selector%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24selector%27"));
smalltalk.bind(smalltalk.CompiledMethod,"selector:",0
,function(aString){var self=this;
(self["$selector"]=(aString));
return self;}
,"accessing",unescape("selector%3A%20aString%0D%0A%09%22%20Set%20the%20selector%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20basicAt%3A%20%27%24selector%27%20put%3A%20aString"));
smalltalk.bind(smalltalk.CompiledMethod,"fn",0
,function(){var self=this;
return ((function(){var $1$=(self["$fn"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("fn%0D%0A%09%22%20Return%20the%20native%20implementation%20of%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24fn%27"));
smalltalk.bind(smalltalk.CompiledMethod,"fn:",0
,function(aBlock){var self=this;
(self["$fn"]=(aBlock));
return self;}
,"accessing",unescape("fn%3A%20aBlock%0D%0A%09%22%20Set%20the%20native%20implementation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20basicAt%3A%20%27%24fn%27%20put%3A%20aBlock"));
smalltalk.bind(smalltalk.CompiledMethod,"asClosure",0
,function(){var self=this;

return self.fn();}
,"accessing",unescape("asClosure%0D%0A%09%22%20Private%20-%20Return%20the%20receiver%20as%20a%20native%20closure%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself%20fn"));
smalltalk.bind(smalltalk.CompiledMethod,"compiledSource",0
,function(){var self=this;
var closure=nil;
closure=self.asClosure();
return (((nil.isNil_(closure)===false)).mustBeBoolean()==true ? (function(){return closure.compiledSourceAt_(self);})() : nil);}
,"accessing",unescape("compiledSource%0D%0A%09%22%20Return%20the%20native%20source%20code%20of%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20closure%20%7C%0D%0A%09closure%20%3A%3D%20self%20asClosure.%0D%0A%09%5Eclosure%20notNil%20ifTrue%3A%20%5B%20closure%20compiledSourceAt%3A%20self%20%5D"));
smalltalk.bind(smalltalk.CompiledMethod,"classField",0
,function(){var self=this;
return self["@classField"];}
,"accessing",unescape("classField%0D%0A%09%22%20Return%20the%20implementor%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EclassField"));
smalltalk.bind(smalltalk.CompiledMethod,"classField:",0
,function(aClass){var self=this;
self["@classField"]=aClass;
return self;}
,"accessing",unescape("classField%3A%20aClass%0D%0A%09%22%20Set%20the%20implementor%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09classField%20%3A%3D%20aClass"));
smalltalk.bind(smalltalk.CompiledMethod,"generation",0
,function(){var self=this;
return ((function(){var $1$=(self["$generation"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("generation%0D%0A%09%22%20Return%20the%20generation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24generation%27"));
smalltalk.bind(smalltalk.CompiledMethod,"comment",0
,function(){var self=this;
return (function($rec){$rec.upTo_(unescape("%22"));return $rec.upTo_(unescape("%22"));})(self.source().stream()).trimBlanks();}
,"accessing",unescape("comment%0D%0A%09%22%20Return%20the%20comment%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28self%20source%20stream%20upTo%3A%20%24%22%3B%20upTo%3A%20%24%22%29%20trimBlanks"));
smalltalk.bind(smalltalk.CompiledMethod,"toString",0
,function(){var self=this;
return (((nil.isNil_(self["@classField"]))).mustBeBoolean()==true ? (function(){return self["@classField"].toString();})() : (function(){return self["@classField"].name();})())._comma(unescape("%3E%3E%23"))._comma(self.selector().toString());}
,"printing",unescape("toString%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20String.%20%22%0D%0A%0D%0A%09%5E%28classField%20isNil%0D%0A%09%09ifTrue%3A%20%5BclassField%20toString%5D%0D%0A%09%09ifFalse%3A%20%5BclassField%20name%5D%29%0D%0A%09%09%2C%27%3E%3E%23%27%20%2Cself%20selector%20toString"));
smalltalk.bind(smalltalk.CompiledMethod,"printString",0
,function(){var self=this;

return self.toString();}
,"printing",unescape("printString%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.CompiledMethod,"asJSONObject",0
,function(){var self=this;
return self;}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.CompiledMethod,"shallowCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.CompiledMethod,"deepCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.CompiledMethod,"removeSources",0
,function(){var self=this;
(function($rec){$rec.source_("");$rec.category_("s8");return $rec.builder_(nil);})(self);
return self;}
,"Builders-tools",unescape("removeSources%0D%0A%09%22%20Remove%20all%20development%20sources%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20source%3A%20%27%27%3B%20category%3A%20%23s8%3B%20builder%3A%20nil"));
smalltalk.bind(smalltalk.CompiledMethod,"removeFromSystem",0
,function(){var self=this;
if (self.classField()._gt_gt(self.selector())._eq(self)){self.classField().removeSelector_(self.selector());};;
return true;}
,"remove",unescape("removeFromSystem%0D%0A%09%22%20Remove%20the%20receiver%20from%20system.%20%22%0D%0A%0D%0A%09self%20classField%20%3E%3E%20self%20selector%20%3D%20self%20ifTrue%3A%20%5B%0D%0A%09%09self%20classField%20removeSelector%3A%20self%20selector%0D%0A%09%5D.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.CompiledMethod,"recompile",0
,function(){var self=this;
self.classField().compiler().compile_forClass_category_(self.source(),self.classField(),self.category());
return self;}
,"Compiler-compiling",unescape("recompile%0D%0A%09%22%20Recompile%20the%20source%20of%20the%20receiver%20and%20install%20the%20new%20compiled%20method.%20%22%0D%0A%0D%0A%09self%20classField%20compiler%0D%0A%09%09compile%3A%20self%20source%0D%0A%09%09forClass%3A%20self%20classField%0D%0A%09%09category%3A%20self%20category"));
smalltalk.bind(smalltalk.CompiledMethod,"parseNodes",0
,function(){var self=this;
return self.classField().compiler().parseMethod_(self.source());}
,"Compiler-inspecting",unescape("parseNodes%0D%0A%09%22%20Return%20the%20result%20of%20parsing%20the%20source%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20classField%20compiler%20parseMethod%3A%20self%20source"));
smalltalk.bind(smalltalk.CompiledMethod,"changeCommentWith:",0
,function(aBlock){var self=this;
var stream=nil;
var header=nil;
var old=nil;
var $$new=nil;
var nl=nil;
stream=self.source().stream();
header=smalltalk.SmalltalkParser.$$new().methodHeaderParser().parse_(stream);
if (header.isParseFailure()){return false;};;
stream.skipSeparators();
header=stream.collection().copyFrom_to_((1),stream.position());
if (stream.peek()._eq(unescape("%22"))){stream.next();
old=stream.upTo_(unescape("%22"));
$$new=aBlock.evaluateWith_(old);} else{$$new=aBlock.evaluateWith_("");};;
if ((nil.isNil_($$new))){return false;};;
nl=(((nil.isNil_(old))).mustBeBoolean()==true ? (function(){return unescape("%0D%0A%09");})() : (function(){return "";})());
$$new=nl._comma(unescape("%22%20"))._comma($$new.trimBlanks())._comma(unescape("%20%22"))._comma(nl);
(function($rec){$rec.source_(header._comma($$new)._comma(stream.upToEnd()));return $rec.recompile();})(self);
return self;}
,"Compiler-transforms",unescape("changeCommentWith%3A%20aBlock%0D%0A%09%22%20Private%20-%20Change%20the%20comment%20of%20the%20receiver%20with%20aBlock.%0D%0A%09aBlock%20must%20be%20a%20one%20argument%20block%20to%20be%20evaluated%20with%20current%20comment.%0D%0A%09aBlock%20must%20return%20the%20new/empty%20comment%20%28or%20nil%2C%20to%20cancel%20change%29.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20stream%20header%20old%20new%20nl%20%7C%0D%0A%09stream%20%3A%3D%20self%20source%20stream.%0D%0A%09header%20%3A%3D%20SmalltalkParser%20new%20methodHeaderParser%20parse%3A%20stream.%0D%0A%09header%20isParseFailure%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09stream%20skipSeparators.%0D%0A%09header%20%3A%3D%20stream%20collection%20copyFrom%3A%201%20to%3A%20stream%20position.%0D%0A%09stream%20peek%20%3D%20%24%22%20ifTrue%3A%20%5B%0D%0A%09%09stream%20next.%0D%0A%09%09old%20%3A%3D%20stream%20upTo%3A%20%24%22.%0D%0A%09%09new%20%3A%3D%20aBlock%20evaluateWith%3A%20old.%0D%0A%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09new%20%3A%3D%20aBlock%20evaluateWith%3A%20%27%27.%0D%0A%09%5D.%0D%0A%09new%20isNil%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09nl%20%3A%3D%20old%20isNil%20ifTrue%3A%20%5B%20%27%0D%0A%09%27%20%5D%20ifFalse%3A%20%5B%20%27%27%20%5D.%0D%0A%09new%20%3A%3D%20nl%20%2C%27%22%20%27%20%2Cnew%20trimBlanks%20%2C%27%20%22%27%20%2Cnl.%0D%0A%0D%0A%09self%0D%0A%09%09source%3A%20header%20%2Cnew%20%2Cstream%20upToEnd%3B%0D%0A%09%09recompile"));
smalltalk.bind(smalltalk.CompiledMethod,"tags",0
,function(){var $s8Ret$={name:"stReturn"};try{var self=this;
self.changeCommentWith_((function(old){return (function(){$s8Ret$.result=(function(){return old.upTo_(unescape("-")).asArrayOfSubstringsSeparatedBy_(unescape("%2C")).collect_((function(tag){return tag.trimBlanks();}))})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return smalltalk.Array.$$new()})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"Compiler-tags",unescape("tags%0D%0A%09%22%20Development%20-%20Return%20the%20tags%20stored%20in%20the%20receiver%27s%20comment.%20%22%0D%0A%0D%0A%09self%20changeCommentWith%3A%20%5B%3Aold%7C%0D%0A%09%09%5E%28%28old%20upTo%3A%20%24-%29%0D%0A%09%09%09asArrayOfSubstringsSeparatedBy%3A%20%24%2C%29%0D%0A%09%09%09collect%3A%20%5B%3Atag%7C%20tag%20trimBlanks%20%5D%0D%0A%09%5D.%0D%0A%09%5EArray%20new"));
smalltalk.bind(smalltalk.CompiledMethod,"comment:",0
,function(aString){var self=this;
return self.changeCommentWith_((function(old){return aString;}));}
,"Compiler-accessing",unescape("comment%3A%20aString%0D%0A%09%22%20Change%20the%20comment%20in%20the%20receiver%20to%20aString.%20%22%0D%0A%0D%0A%09%5Eself%20changeCommentWith%3A%20%5B%3Aold%7C%20aString%20%5D"));
smalltalk.bind(smalltalk.CompiledMethod,"messageComment",0
,function(){var self=this;
var line=nil;
line=self.comment().stream().nextLine();
if (!(line.includes_(unescape("-")))){return line;};;
return (function($rec){$rec.upTo_(unescape("-"));return $rec.upToEnd();})(line.stream()).trimBlanks();}
,"Compiler-accessing",unescape("messageComment%0D%0A%09%22%20Return%20the%20comment%20of%20the%20message%20implemented%20by%20the%20receiver.%0D%0A%09The%20messageComment%20is%20obtained%20from%20first%20line%20of%20comment%20of%20the%20receiver.%0D%0A%09Note%3A%20it%20is%20expected%20that%20all%20the%20methods%20implementing%20the%20same%20message%20in%20this%20system%20have%20the%20same%20messageComment.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20line%20%7C%0D%0A%09line%20%3A%3D%20self%20comment%20stream%20nextLine.%0D%0A%09%28line%20includes%3A%20%24-%29%20ifFalse%3A%20%5B%20%5Eline%20%5D.%0D%0A%09%5E%28line%20stream%20upTo%3A%20%24-%3B%20upToEnd%29%20trimBlanks"));
smalltalk.bind(smalltalk.CompiledMethod,"messageComment:",0
,function(aString){var self=this;
return self.changeCommentWith_((function(old){var line=nil;
var tags=nil;
var stream=nil;
stream=old.stream();line=stream.nextLine();if (line.includes_(unescape("-"))){tags=line.upTo_(unescape("-"))._comma(unescape("-%20"));} else{tags="";};;return tags._comma(aString)._comma(((stream.atEnd()).mustBeBoolean()==true ? (function(){return "";})() : (function(){return unescape("%0D%0A")._comma(stream.upToEnd());})()));}));}
,"Compiler-transforms",unescape("messageComment%3A%20aString%0D%0A%09%22%20Set%20the%20comment%20of%20the%20message%20implemented%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20changeCommentWith%3A%20%5B%3Aold%7C%20%7C%20line%20tags%20stream%20%7C%0D%0A%09%09stream%20%3A%3D%20old%20stream.%0D%0A%09%09line%20%3A%3D%20stream%20nextLine.%0D%0A%09%09%28line%20includes%3A%20%24-%29%20ifTrue%3A%20%5B%0D%0A%09%09%09tags%20%3A%3D%20%28line%20upTo%3A%20%24-%29%20%2C%27-%20%27%0D%0A%09%09%5D%20ifFalse%3A%20%5B%20tags%20%3A%3D%20%27%27%20%5D.%0D%0A%09%09tags%09%2CaString%20%2C%28stream%20atEnd%20ifTrue%3A%20%5B%27%27%5D%20ifFalse%3A%20%5B%27%0D%0A%27%09%09%09%2Cstream%20upToEnd%5D%29%0D%0A%09%5D"));
smalltalk.bind(smalltalk.CompiledMethod,"messages",0
,function(){var self=this;
return self.smalltalk().methodReferences().messagesOf_(self);}
,"Tools-accessing",unescape("messages%0D%0A%09%22%20Return%20all%20the%20messages%20that%20can%20be%20sent%20during%20execution%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20smalltalk%20methodReferences%20messagesOf%3A%20self"));
smalltalk.bind(smalltalk.CompiledMethod,"globalReferences",0
,function(){var self=this;
return self.smalltalk().methodReferences().globalReferencesOf_(self);}
,"Tools-accessing",unescape("globalReferences%0D%0A%09%22%20Return%20all%20the%20global%20references%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20smalltalk%20methodReferences%20globalReferencesOf%3A%20self"));
smalltalk.bind(smalltalk.CompiledMethod,"references",0
,function(){var self=this;
return (function($rec){$rec.at_put_("messages",self.messages());$rec.at_put_("globalReferences",self.globalReferences());return $rec.yourself();})(smalltalk.PoolDictionary.$$new());}
,"Tools-accessing",unescape("references%0D%0A%09%22%20Return%20all%20the%20references%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EPoolDictionary%20new%0D%0A%09%09at%3A%20%23messages%20put%3A%20self%20messages%3B%0D%0A%09%09at%3A%20%23globalReferences%20put%3A%20self%20globalReferences%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.CompiledMethod,"builder:",0
,function(cookie){var self=this;
(self["$builder"]=(cookie));
return self;}
,"Builders-accessing",unescape("builder%3A%20cookie%0D%0A%09%22%20Private%20-%20Set%20the%20builder%20cookie%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20basicAt%3A%20%27%24builder%27%20put%3A%20cookie"));
smalltalk.bind(smalltalk.CompiledMethod,"builder",0
,function(){var self=this;
return ((function(){var $1$=(self["$builder"]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"Builders-accessing",unescape("builder%0D%0A%09%22%20Private%20-%20Return%20the%20builder%20cookie%20of%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24builder%27"));
smalltalk.bind(smalltalk.CompiledMethod,"__tostring",0
,function(){var self=this;
return self.toString();}
,"Lua-printing",unescape("__tostring%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.Number.$klass,"pi",0
,function(){var self=this;
return Math.PI;}
,"instantiation",unescape("pi%0D%0A%09%22%20Return%20the%20singular%20value%20PI.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20constant%27%7D"));
smalltalk.bind(smalltalk.Number.$klass,"readFrom:radix:",0
,function(aStream,radix){var self=this;
var value=nil;
var $$char=nil;
value=(0);
while((function(){$$char=aStream.peek();return (nil.isNil_($$char)===false).and_((function(){return $$char.isDigit().or_((function(){return $$char.isUpperCase();}));}));})()){if (radix._lt($$char.digitValue())){return value;};;
value=value._star(radix)._plus($$char.digitValue());
aStream.next();};;
return value;}
,"converting",unescape("readFrom%3A%20aStream%20radix%3A%20radix%0D%0A%09%22%20Private%20-%20Read%20an%20integer%20from%20%3CaStream%3E%20and%20return%20it.%20%22%0D%0A%0D%0A%09%7C%20value%20char%20%7C%0D%0A%09value%20%3A%3D%200.%0D%0A%09%5B%09char%20%3A%3D%20aStream%20peek.%0D%0A%09%09char%20notNil%20and%3A%20%5B%20char%20isDigit%20or%3A%20%5B%20char%20isUpperCase%20%5D%20%5D%0D%0A%09%5D%20whileTrue%3A%20%5B%0D%0A%09%09radix%20%3C%20%20char%20digitValue%20ifTrue%3A%20%5B%20%5Evalue%20%5D.%0D%0A%09%09value%20%3A%3D%20value%20*%20radix%20+%20char%20digitValue.%0D%0A%09%09aStream%20next%0D%0A%09%5D.%0D%0A%09%5Evalue"));
smalltalk.bind(smalltalk.Number.$klass,"readFrom:",0
,function(aStream){var self=this;
var value=nil;
var $$char=nil;
value=(0);
while((function(){$$char=aStream.peek();return (nil.isNil_($$char)===false).and_((function(){return $$char.isDigit();}));})()){value=value._star((10))._plus($$char.digitValue());
aStream.next();};;
if (!("r"._eq($$char))){return value;};;
aStream.next();
return self.readFrom_radix_(aStream,value);}
,"converting",unescape("readFrom%3A%20aStream%0D%0A%09%22%20Private%20-%20Read%20an%20integer%20from%20%3CaStream%3E%20and%20return%20it.%20%22%0D%0A%0D%0A%09%7C%20value%20char%20%7C%0D%0A%09value%20%3A%3D%200.%0D%0A%09%5B%09char%20%3A%3D%20aStream%20peek.%0D%0A%09%09char%20notNil%20and%3A%20%5B%20char%20isDigit%20%5D%0D%0A%09%5D%20whileTrue%3A%20%5B%0D%0A%09%09value%20%3A%3D%20value%20*%2010%20+%20char%20digitValue.%0D%0A%09%09aStream%20next%0D%0A%09%5D.%0D%0A%09%24r%20%3D%20char%20ifFalse%3A%20%5B%20%5Evalue%20%5D.%0D%0A%09aStream%20next.%0D%0A%09%5Eself%20readFrom%3A%20aStream%20radix%3A%20value"));
smalltalk.bind(smalltalk.Number.$klass,"readFromOBSOLETE:",0
,function(aStream){var self=this;
var value=nil;
var $$char=nil;
value=(0);
while((function(){$$char=aStream.peek();return (nil.isNil_($$char)===false).and_((function(){return $$char.isDigit();}));})()){value=value._star((10))._plus($$char.digitValue());
aStream.next();};;
if (!((nil.is_eqeq_($$char,"r")))){return value;};;
aStream.next();
return self.readFrom_radix_(aStream,value);}
,"converting",unescape("readFromOBSOLETE%3A%20aStream%0D%0A%09%22%20Private%20-%20Read%20an%20integer%20from%20%3CaStream%3E%20and%20return%20it.%20%22%0D%0A%0D%0A%09%7C%20value%20char%20%7C%0D%0A%09value%20%3A%3D%200.%0D%0A%09%5B%09char%20%3A%3D%20aStream%20peek.%0D%0A%09%09char%20notNil%20and%3A%20%5B%20char%20isDigit%20%5D%0D%0A%09%5D%20whileTrue%3A%20%5B%0D%0A%09%09value%20%3A%3D%20value%20*%2010%20+%20char%20digitValue.%0D%0A%09%09aStream%20next%0D%0A%09%5D.%0D%0A%09char%20%3D%3D%20%22%24r%22%27r%27%20ifFalse%3A%20%5B%20%5Evalue%20%5D.%0D%0A%09aStream%20next.%20%22skip%20the%20r%22%0D%0A%09%5Eself%20readFrom%3A%20aStream%20radix%3A%20value"));
smalltalk.bind(smalltalk.Number.$klass,"@","_at"
,function(aHandle){var self=this;
return (((nil.isNil_(aHandle)===false)).mustBeBoolean()==true ? (function(){return aHandle.asNumber();})() : nil);}
,"Natives-converting",unescape("@%20aHandle%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20from%20aHandle.%20%22%0D%0A%0D%0A%09%5EaHandle%20notNil%20ifTrue%3A%20%5B%20aHandle%20asNumber%20%5D"));
smalltalk.bind(smalltalk.Number,"=","_eq"
,function(anObject){var self=this;

	if (nil.isNil_(anObject)) return false;
	if (!(anObject.isNumber)) return false;
	if (!(anObject.isNumber())) return false;
	if (self == anObject) return true;
	return ((0+self-anObject) == 0);
return self;}
,"comparing",unescape("%3D%20anObject%0D%0A%20%20%20%20%22%20Return%20true%20if%20the%20receiver%20is%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27number%20equality%20test%27%7D"));
smalltalk.bind(smalltalk.Number,"mustBeNumber",0
,function(){var self=this;
return self;}
,"testing",unescape("mustBeNumber%0D%0A%09%22%20Signal%20an%20error%20if%20the%20receiver%20is%20not%20a%20Number.%0D%0A%09WARNING%3A%20this%20check%20impose%20a%20severe%20loss%20of%20perfomance%20%28five%20times%20slower%29.%0D%0A%09If%20you%20are%20sure%20the%20operation/compare%20arguments%20are%20ok%20%28a%20number%29%2C%0D%0A%09%20comment%20the%20implementation%20of%20this%20method%20making%20the%20code%20empty.%0D%0A%09%22%0D%0A%0D%0A%09%22%28%23%20%23isNaN%3A%20self%29%20ifTrue%3A%20%5B%20super%20mustBeNumber%20%5D%22"));
smalltalk.bind(smalltalk.Number,">","_gt"
,function(aNumber){var self=this;
return self > aNumber.mustBeNumber();}
,"comparing",unescape("%3E%20aNumber%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20greater%20than%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20compare%27%7D"));
smalltalk.bind(smalltalk.Number,"<","_lt"
,function(aNumber){var self=this;
return self < aNumber.mustBeNumber();}
,"comparing",unescape("%3C%20aNumber%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20smaller%20than%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20compare%27%7D"));
smalltalk.bind(smalltalk.Number,">=","_gt_eq"
,function(aNumber){var self=this;
return self >= aNumber.mustBeNumber();}
,"comparing",unescape("%3E%3D%20aNumber%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20greater%20or%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20compare%27%7D"));
smalltalk.bind(smalltalk.Number,"<=","_lt_eq"
,function(aNumber){var self=this;
return self <= aNumber.mustBeNumber();}
,"comparing",unescape("%3C%3D%20aNumber%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20smaller%20or%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20compare%27%7D"));
smalltalk.bind(smalltalk.Number,"+","_plus"
,function(aNumber){var self=this;
return self + aNumber.mustBeNumber();}
,"arithmetic",unescape("+%20aNumber%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"-","_minus"
,function(aNumber){var self=this;
return self - aNumber.mustBeNumber();}
,"arithmetic",unescape("-%20aNumber%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"*","_star"
,function(aNumber){var self=this;
return self * aNumber.mustBeNumber();}
,"arithmetic",unescape("*%20aNumber%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"/","_slash"
,function(aNumber){var self=this;
return self / aNumber.mustBeNumber();}
,"arithmetic",unescape("/%20aNumber%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"divideByZero",0
,function(){var self=this;
return self.error_("Divide by Zero");}
,"signaling",unescape("divideByZero%0D%0A%09%22%20Signal%20divide%20by%20cero%20attempt.%20%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%27Divide%20by%20Zero%27"));
smalltalk.bind(smalltalk.Number,unescape("%5C%5C"),"_bk_bk"
,function(aNumber){var self=this;
aNumber.mustBeNumber();
if (aNumber._eq((0))){self.divideByZero();};;
return self._minus(self._slash_slash(aNumber)._star(aNumber));}
,"arithmetic",unescape("%5C%5C%5C%5C%20aNumber%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09aNumber%20mustBeNumber.%0D%0A%09aNumber%20%3D%200%20ifTrue%3A%20%5B%20self%20divideByZero%20%5D.%0D%0A%09%5Eself%20-%20%28self%20//%20aNumber%20*%20aNumber%29"));
smalltalk.bind(smalltalk.Number,"//","_slash_slash"
,function(aNumber){var self=this;
aNumber.mustBeNumber();
if (aNumber._eq((0))){self.divideByZero();};;
return self._slash(aNumber).truncated();}
,"arithmetic",unescape("//%20aNumber%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09aNumber%20mustBeNumber.%0D%0A%09aNumber%20%3D%200%20ifTrue%3A%20%5B%20self%20divideByZero%20%5D.%0D%0A%09%5E%28self%20/%20aNumber%29%20truncated"));
smalltalk.bind(smalltalk.Number,"sin",0
,function(){var self=this;
return Math.sin(self);}
,"arithmetic",unescape("sin%0D%0A%09%22%20Return%20the%20result%20of%20operation%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"cos",0
,function(){var self=this;
return Math.cos(self);}
,"arithmetic",unescape("cos%0D%0A%09%22%20Return%20the%20result%20of%20operation%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"tan",0
,function(){var self=this;
return Math.tan(self);}
,"arithmetic",unescape("tan%0D%0A%09%22%20Return%20the%20result%20of%20operation%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"arcCos",0
,function(){var self=this;
return Math.acos(self);}
,"arithmetic",unescape("arcCos%0D%0A%09%22%20Return%20the%20result%20of%20operation%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"arcSin",0
,function(){var self=this;
return Math.asin(self);}
,"arithmetic",unescape("arcSin%0D%0A%09%22%20Return%20the%20result%20of%20operation%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"arcTan",0
,function(){var self=this;
return Math.atan(self);}
,"arithmetic",unescape("arcTan%0D%0A%09%22%20Return%20the%20result%20of%20operation%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"atan2:",0
,function(aNumber){var self=this;
return Math.atan2(self,aNumber.mustBeNumber());}
,"arithmetic",unescape("atan2%3A%20aNumber%0D%0A%09%22%20Return%20the%20result%20of%20operation%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"negated",0
,function(){var self=this;
return (-1)._star(self);}
,"arithmetic",unescape("negated%0D%0A%09%22%20Return%20the%20receiver%20negated%22%0D%0A%0D%0A%09%5E-1%20*%20self%20"));
smalltalk.bind(smalltalk.Number,"squared",0
,function(){var self=this;
return self._star(self);}
,"arithmetic",unescape("squared%0D%0A%09%5Eself%20*%20self%20"));
smalltalk.bind(smalltalk.Number,"sqrt",0
,function(){var self=this;
return Math.sqrt(self);}
,"arithmetic",unescape("sqrt%0D%0A%09%22%20Return%20the%20result%20of%20operation%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"<<","_lt_lt"
,function(aNumber){var self=this;
return self << aNumber.mustBeNumber();}
,"arithmetic",unescape("%3C%3C%20aNumber%0D%0A%09%22%20Return%20the%20receiver%20shifted%20aNumber%20of%20bits%20to%20the%20left.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,">>","_gt_gt"
,function(aNumber){var self=this;
return self >> aNumber.mustBeNumber();}
,"arithmetic",unescape("%3E%3E%20aNumber%0D%0A%09%22%20Return%20the%20receiver%20shifted%20aNumber%20of%20bits%20to%20the%20right.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"bitShift:",0
,function(anInteger){var self=this;
if (anInteger._lt((0))){return self._gt_gt(anInteger.negated());};;
return self._lt_lt(anInteger);}
,"arithmetic",unescape("bitShift%3A%20anInteger%0D%0A%09%22%20Return%20the%20receiver%20shifted%20left%20anInteger%20number%20of%20bits%20if%20anInteger%20is%20positive%2C%20or%20shifted%20right%20for%20anInteger%20negated%20number%20of%20bits%20if%20anInteger%20is%20negative.%22%0D%0A%0D%0A%09anInteger%20%3C%200%20ifTrue%3A%20%5B%20%5Eself%20%3E%3E%20anInteger%20negated%20%5D.%0D%0A%09%5Eself%20%3C%3C%20anInteger"));
smalltalk.bind(smalltalk.Number,"raisedTo:",0
,function(aNumber){var self=this;
if (aNumber._eq((0))){return (1);};;
if (self._eq((0)).and_((function(){return aNumber._gt((0));}))){return (0);};;
return Math.pow(self,aNumber.mustBeNumber());}
,"arithmetic",unescape("raisedTo%3A%20aNumber%0D%0A%09%22%20Return%20the%20receiver%20raised%20to%20the%20power%20of%20aNumber.%20%22%0D%0A%0D%0A%09aNumber%20%3D%200%20ifTrue%3A%20%5B%20%5E1.0%20%5D.%0D%0A%09%28self%20%3D%200%20and%3A%20%5BaNumber%20%3E%200%5D%29%20ifTrue%3A%20%5B%20%5E0.0%20%5D.%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"max:",0
,function(aNumber){var self=this;
return Math.max(self,aNumber);}
,"arithmetic",unescape("max%3A%20aNumber%0D%0A%09%22%20Return%20the%20maximum%20between%20the%20receiver%20and%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"min:",0
,function(aNumber){var self=this;
return Math.min(self,aNumber);}
,"arithmetic",unescape("min%3A%20aNumber%0D%0A%09%22%20Return%20the%20minimum%20between%20the%20receiver%20and%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"asNumber",0
,function(){var self=this;
return self._plus((0));}
,"converting",unescape("asNumber%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20Number.%20%22%0D%0A%0D%0A%09%5Eself%20+%200"));
smalltalk.bind(smalltalk.Number,"abs",0
,function(){var self=this;
if (self._lt((0))){return self.negated();};;
return self;}
,"converting",unescape("abs%0D%0A%09%22%20Return%20the%20absolute%20value%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20%3C%200%20ifTrue%3A%20%5B%20%5Eself%20negated%20%5D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Number,"rounded",0
,function(){var self=this;
return Math.round(self);}
,"converting",unescape("rounded%0D%0A%09%22%20Return%20the%20receiver%20rounded.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"truncated",0
,function(){var self=this;
return Math.floor(self);}
,"converting",unescape("truncated%0D%0A%09%22%20Return%20the%20receiver%20truncated.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27number%20operation%27%7D"));
smalltalk.bind(smalltalk.Number,"asCharacter",0
,function(){var self=this;
return smalltalk.String.fromCharCode_(self);}
,"converting",unescape("asCharacter%0D%0A%09%22%20Return%20the%20character%20with%20the%20receiver%27s%20code.%20%22%0D%0A%0D%0A%09%5EString%20fromCharCode%3A%20self"));
smalltalk.bind(smalltalk.Number,"to:by:value:",0
,function(aNumber,increment,aBlock){var self=this;
return smalltalk.Interval.from_to_by_value_(self,aNumber,increment,aBlock);}
,"iterating",unescape("to%3A%20aNumber%20by%3A%20increment%20value%3A%20aBlock%0D%0A%09%22%20Private%20-%20Return%20an%20iterable%20to%20evaluate%20aBlock%20on%20each%20step%20of%20iteration.%20%22%0D%0A%0D%0A%09%5EInterval%20from%3A%20self%20to%3A%20aNumber%20by%3A%20increment%20value%3A%20aBlock%20"));
smalltalk.bind(smalltalk.Number,"to:by:",0
,function(end,increment){var self=this;
return self.to_by_value_(end,increment,(function(index,block){return block.value_(index);}));}
,"iterating",unescape("to%3A%20end%20by%3A%20increment%0D%0A%09%22%20Return%20an%20iterable%20from%20the%20receiver%20to%20end.%20%22%0D%0A%0D%0A%09%5Eself%09to%3A%20end%20by%3A%20increment%0D%0A%09%09value%3A%20%5B%3Aindex%20%3Ablock%7C%20block%20value%3A%20index%20%5D"));
smalltalk.bind(smalltalk.Number,"to:",0
,function(end){var self=this;
return self.to_by_(end,(1));}
,"iterating",unescape("to%3A%20end%0D%0A%09%22%20Return%20an%20iterable%20from%20the%20receiver%20to%20end.%20%22%0D%0A%0D%0A%09%5Eself%20to%3A%20end%20by%3A%201%20%22to%20iterate%20like%20other%20smalltalks%0D%0A%09%09%28self%20%3C%3D%20end%20ifTrue%3A%20%5B1%5D%20ifFalse%3A%20%5B-1%5D%29%22"));
smalltalk.bind(smalltalk.Number,"timesRepeat:",0
,function(aBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
for(var i=1;i<=self;i++){fn(i);};
return self;}
,"enumerating",unescape("timesRepeat%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20a%20number%20of%20times%20given%20by%20the%20receiver.%0D%0A%09Return%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27%20s8%27%281%20to%3A%20self%29%20do%3A%20aBlock%27%20doc%27iteration%27%7D"));
smalltalk.bind(smalltalk.Number,"to:do:",0
,function(aNumber,aBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
for(var i=self;i<=aNumber;i++){fn(i);};
return self;}
,"iterating",unescape("to%3A%20aNumber%20do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20from%20the%20receiver%20to%20aNumber.%0D%0A%09Return%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27s8%27%20%28self%20to%3A%20aNumber%29%20do%3A%20aBlock%27%20doc%27iteration%27%7D"));
smalltalk.bind(smalltalk.Number,"to:by:do:",0
,function(aNumber,increment,aBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
if (increment>0) { for(var i=self;i<=aNumber;i=i+increment){fn(i);} }
	else { for(var i=self;aNumber<=i;i=i+increment){fn(i);} };
return self;}
,"iterating",unescape("to%3A%20aNumber%20by%3A%20increment%20do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20the%20numbers%20between%20the%20receiver%20and%20aNumber%20where%20each%20number%20is%20the%20previous%20number%20plus%20increment.%0D%0A%09Return%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27s8%27%28self%20to%3A%20aNumber%20by%3A%20increment%29%20consume%3A%20aBlock%27%20doc%27iteration%27%7D"));
smalltalk.bind(smalltalk.Number,"asString",0
,function(){var self=this;
return self.printString();}
,"converting",unescape("asString%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20String.%20%22%0D%0A%0D%0A%09%5Eself%20printString"));
smalltalk.bind(smalltalk.Number,"asNative:",0
,function(lang){var self=this;
return unescape("%28")._comma(self.printString())._comma(unescape("%29"));}
,"converting",unescape("asNative%3A%20lang%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%27%28%27%2C%20self%20printString%2C%20%27%29%27"));
smalltalk.bind(smalltalk.Number,"printString",0
,function(){var self=this;
return String(self);}
,"printing",unescape("printString%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20conversion%27%7D"));
smalltalk.bind(smalltalk.Number,"toString:",0
,function(radix){var self=this;
if (!(radix.isNumber())){return "converting".toString_(radix);};;
return self.toString(radix);}
,"printing",unescape("toString%3A%20radix%0D%0A%09%22%20Return%20the%20ASCII%20representation%20of%20the%20receiver%20in%20base%20radix.%20%22%0D%0A%0D%0A%09radix%20isNumber%20ifFalse%3A%20%5B%0D%0A%09%09%5E%23converting%20toString%3A%20radix%0D%0A%09%5D.%0D%0A%09%5E%7B%27%27%20doc%27native%20conversion%27%7D"));
smalltalk.bind(smalltalk.Number,"printPaddedWith:to:base:",0
,function(aCharacter,anInteger,aRadix){var self=this;
var result=nil;
result=self.toString_(aRadix);
while((function(){return result.size()._lt(anInteger);})()){result=aCharacter._comma(result);};;
return result;}
,"printing",unescape("printPaddedWith%3A%20aCharacter%20to%3A%20anInteger%20base%3A%20aRadix%0D%0A%09%22%20Return%20the%20ASCII%20representation%20of%20the%20receiver%20padded%20on%20the%20left%20with%20aCharacter%20to%20be%20at%20least%20anInteger%20characters.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20toString%3A%20aRadix.%0D%0A%09%5B%20result%20size%20%3C%20anInteger%20%5D%20whileTrue%3A%20%5B%0D%0A%09%09result%20%3A%3D%20aCharacter%20%2Cresult%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Number,"isNumber",0
,function(){var self=this;
return isNaN(self).not();
return self;}
,"testing",unescape("isNumber%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20Number.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27number%20testing%27%7D"));
smalltalk.bind(smalltalk.Number,"atRandom",0
,function(){var self=this;
return smalltalk.Random.$$new().next()._star(self).truncated()._plus((1));}
,"converting",unescape("atRandom%0D%0A%09%22%20Return%20a%20random%20integer%20in%20the%20interval%20from%201%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28Random%20new%20next%20*%20self%29%20truncated%20+%201"));
smalltalk.bind(smalltalk.Number,"@","_at"
,function(aNumber){var self=this;
return smalltalk.Point.x_y_(self,(((nil.isNil_(aNumber))).mustBeBoolean()==true ? (function(){return (0);})() : (function(){return aNumber;})()));}
,"converting",unescape("@%20aNumber%0D%0A%09%22%20Return%20a%20Point%20with%20coordinates%20given%20by%20the%20receiver%20and%20aNumber.%20%22%0D%0A%0D%0A%09%5EPoint%20x%3A%20self%20y%3A%20%28aNumber%20isNil%20ifTrue%3A%20%5B0%5D%20ifFalse%3A%20%5BaNumber%5D%29"));
smalltalk.bind(smalltalk.Number,"asPoint",0
,function(){var self=this;
return smalltalk.Point.x_y_(self,self);}
,"converting",unescape("asPoint%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20Point.%20%22%0D%0A%0D%0A%09%5EPoint%20x%3A%20self%20y%3A%20self"));
smalltalk.bind(smalltalk.Number,"asJSONObject",0
,function(){var self=this;

return self._plus((0));}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself%20+%200%0D%0A%22Note%3A%09we%20need%20to%20normalize%20result%20to%20do%20not%20destroy%20typeof%20checkings%0D%0A%09Do%20NOT%20modify%20this%20method%20if%20you%20do%20not%20know%20what%20you%20are%20doing.%0D%0A%09If%20you%20remove%20the%20+%200%20computation%2C%20the%20returned%20object%20will%20return%20%27object%27%20and%20not%20%27number%27%20as%20result%20of%20typeof%20operation.%0D%0A%22"));
smalltalk.bind(smalltalk.Number,"between:and:",0
,function(min,max){var self=this;
return min._lt_eq(self).and_((function(){return self._lt_eq(max);}));}
,"testing",unescape("between%3A%20min%20and%3A%20max%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20between%20arguments.%20%22%0D%0A%0D%0A%09%5Emin%20%3C%3D%20self%20and%3A%20%5B%20self%20%3C%3D%20max%20%5D"));
smalltalk.bind(smalltalk.Number,"even",0
,function(){var self=this;
return self._bk_bk((2))._eq((0));}
,"testing",unescape("even%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20an%20even%20number.%20%22%0D%0A%0D%0A%09%5Eself%20%5C%5C%202%20%3D%200"));
smalltalk.bind(smalltalk.Number,"odd",0
,function(){var self=this;
return self._bk_bk((2))._eq((1));}
,"testing",unescape("odd%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20an%20odd%20number.%20%22%0D%0A%0D%0A%09%5Eself%20%5C%5C%202%20%3D%201"));
smalltalk.bind(smalltalk.Number,"clearInterval",0
,function(){var self=this;
clearInterval(self);
return self;}
,"intervals",unescape("clearInterval%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20timer%20cancelation%27%7D"));
smalltalk.bind(smalltalk.Number,"clearTimeout",0
,function(){var self=this;
clearTimeout(self);
return self;}
,"intervals",unescape("clearTimeout%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20timer%20cancelation%27%7D"));
smalltalk.bind(smalltalk.Number,"shallowCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Number,"deepCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Number,"int32",0
,function(){var self=this;

return self;}
,"Natives-converting",unescape("int32%0D%0A%09%22%20Return%20the%20receiver%20to%20be%20passed%20to%20API%20call%20as%2032%20bits%20integer%20value.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Number,"int64",0
,function(){var self=this;

return self;}
,"Natives-converting",unescape("int64%0D%0A%09%22%20Return%20the%20receiver%20to%20be%20passed%20to%20API%20call%20as%2064%20bits%20integer%20value.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Number,"float32",0
,function(){var self=this;

return self;}
,"Natives-converting",unescape("float32%0D%0A%09%22%20Return%20the%20receiver%20to%20be%20passed%20to%20API%20call%20as%2032%20bits%20float%20value.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Number,"float64",0
,function(){var self=this;

return self;}
,"Natives-converting",unescape("float64%0D%0A%09%22%20Return%20the%20receiver%20to%20be%20passed%20to%20API%20call%20as%2064%20bits%20float%20value.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Number,"asLua",0
,function(){var self=this;
return self.printString();}
,"Lua-lua",unescape("asLua%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20printString%20"));
smalltalk.bind(smalltalk.BlockClosure.$klass,"empty",0
,function(){var self=this;
return (function(){return nil;});}
,"instantiation",unescape("empty%0D%0A%09%22%20Return%20an%20empty%20block.%20%22%0D%0A%0D%0A%09%5E%5B%5D"));
smalltalk.bind(smalltalk.BlockClosure.$klass,"defaultSortBlock",0
,function(){var self=this;

return (function(a,b){return (function(){return a._lt(b);}).on_do_(smalltalk.Exception,(function(){return a.toString()._lt(b.toString());}));});}
,"instantiation",unescape("defaultSortBlock%0D%0A%09%22%20Private%20-%20Return%20the%20block%20used%20for%20default%20sort.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5E%5B%3Aa%20%3Ab%7C%20%5B%20a%20%3C%20b%20%5D%20on%3A%20Exception%20do%3A%20%5B%20a%20toString%20%3C%20b%20toString%20%5D%20%5D"));
smalltalk.bind(smalltalk.BlockClosure.$klass,"hasDeferredSupport",0
,function(){var $s8Ret$={name:"stReturn"};try{var self=this;
(function(){return (nil.isNil_(setTimeout)===false);}).on_do_(smalltalk.Exception,(function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return (nil.isNil_(setTimeout)===false)})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"deferredEvaluation",unescape("hasDeferredSupport%0D%0A%09%22%20Return%20true%20if%20the%20platform%20can%20defer%20instance%20evaluation.%20%22%0D%0A%0D%0A%09%5B%20%23%7BsetTimeout%7D%20notNil%20%5D%20on%3A%20Exception%20do%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%5E%23%7BsetTimeout%7D%20notNil"));
smalltalk.bind(smalltalk.BlockClosure,"compiledSourceAt:",0
,function(aCompiledMethod){var self=this;
var cached=nil;
cached=(((nil.isNil_(aCompiledMethod)===false)).mustBeBoolean()==true ? (function(){return ((function(){var $1$=(aCompiledMethod["$nativeSource"]);if(nil.isNil_($1$))return ((function(){return nil;}).value());return $1$;})());})() : nil);
return (((nil.isNil_(cached))).mustBeBoolean()==true ? (function(){return self.compiledSource();})() : (function(){return cached;})());}
,"accessing",unescape("compiledSourceAt%3A%20aCompiledMethod%0D%0A%09%22%20Return%20the%20native%20source%20code%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20cached%20%7C%0D%0A%09cached%20%3A%3D%20aCompiledMethod%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09aCompiledMethod%20basicAt%3A%20%27%24nativeSource%27%20ifAbsent%3A%20%5B%5D%0D%0A%09%5D.%0D%0A%09%5Ecached%20isNil%20ifTrue%3A%20%5B%20self%20compiledSource%20%5D%20ifFalse%3A%20%5B%20cached%20%5D"));
smalltalk.bind(smalltalk.BlockClosure,"compiledSource",0
,function(){var self=this;

return self.toString();}
,"accessing",unescape("compiledSource%0D%0A%09%22%20Return%20the%20native%20source%20code%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.BlockClosure,"whileTrue:",0
,function(aBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
while(self()) {fn()};
return self;}
,"iterating",unescape("whileTrue%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20while%20the%20result%20of%20evaluating%20the%20receiver%20is%20true.%20%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27s8%27%20self%20value%20%3D%3D%20true%20ifTrue%3A%20%5B%0D%0A%09%09aBlock%20value.%0D%0A%09%09self%20whileTrue%3A%20aBlock%0D%0A%09%5D%27%20doc%27iteration%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"whileFalse:",0
,function(aBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
while(!self()) {fn()};
return self;}
,"iterating",unescape("whileFalse%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20while%20the%20result%20of%20evaluating%20the%20receiver%20is%20false.%20%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27s8%27%09self%20value%20%3D%3D%20false%20ifTrue%3A%20%5B%0D%0A%09%09aBlock%20value.%0D%0A%09%09self%20whileFalse%3A%20aBlock%0D%0A%09%5D%27%20doc%27iteration%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"whileTrue",0
,function(){var self=this;
return self.whileTrue_((function(){return nil;}));}
,"iterating",unescape("whileTrue%0D%0A%09%22%20Evaluate%20the%20receiver%20while%20the%20result%20is%20true.%20%22%0D%0A%0D%0A%09%5Eself%20whileTrue%3A%20%5B%5D"));
smalltalk.bind(smalltalk.BlockClosure,"whileFalse",0
,function(){var self=this;
return self.whileFalse_((function(){return nil;}));}
,"iterating",unescape("whileFalse%0D%0A%09%22%20Evaluate%20the%20receiver%20while%20the%20result%20is%20false.%20%22%0D%0A%0D%0A%09%5Eself%20whileFalse%3A%20%5B%5D"));
smalltalk.bind(smalltalk.BlockClosure,"value",0
,function(){var self=this;
return self();}
,"evaluating",unescape("value%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27self%28%29%27%20doc%27native%20activation%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"value:",0
,function(anObject){var self=this;
return self(anObject);}
,"evaluating",unescape("value%3A%20anObject%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20with%20anObject.%20%22%0D%0A%0D%0A%09%5E%7B%27self%28anObject%29%27%20doc%27native%20activation%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"value:value:",0
,function(arg1,arg2){var self=this;
return self(arg1,arg2);}
,"evaluating",unescape("value%3A%20arg1%20value%3A%20arg2%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20with%20arguments.%20%22%0D%0A%0D%0A%09%5E%7B%27self%28arg1%2Carg2%29%27%20doc%27native%20activation%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"value:value:value:",0
,function(arg1,arg2,arg3){var self=this;
return self(arg1,arg2,arg3);}
,"evaluating",unescape("value%3A%20arg1%20value%3A%20arg2%20value%3A%20arg3%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20with%20arguments.%20%22%0D%0A%0D%0A%09%5E%7B%27self%28arg1%2Carg2%2Carg3%29%27%20doc%27native%20activation%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"value:value:value:value:",0
,function(arg1,arg2,arg3,arg4){var self=this;
return self(arg1,arg2,arg3,arg4);}
,"evaluating",unescape("value%3A%20arg1%20value%3A%20arg2%20value%3A%20arg3%20value%3A%20arg4%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20with%20arguments.%20%22%0D%0A%0D%0A%09%5E%7B%27self%28arg1%2Carg2%2Carg3%2Carg4%29%27%20doc%27native%20activation%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"valueWithPossibleArguments:",0
,function(array){var self=this;
if (!(array.isArray())){self.error_(unescape("Invalid%20arguments%20in%20%23valueWithPossibleArguments%3A%20%28must%20be%20anArray%29.%20receiver%3D")._comma(self.toString()));};;
return self.apply(null,array);}
,"evaluating",unescape("valueWithPossibleArguments%3A%20array%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20with%20potential%20arguments.%20%22%0D%0A%0D%0A%09array%20isArray%20ifFalse%3A%20%5B%0D%0A%09%09self%20error%3A%20%27Invalid%20arguments%20in%20%23valueWithPossibleArguments%3A%20%28must%20be%20anArray%29.%20receiver%3D%27%20%2Cself%20toString%0D%0A%09%5D.%0D%0A%09%5E%7B%27%27%20doc%27native%20activation%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"exception:matches:",0
,function(anException,type){var $s8Ret$={name:"stReturn"};try{var self=this;
if ((nil.isNil_(type))){return true;};;
if (!(self.isObject_(type))){return type._eq(anException);};;
if (type.isClass()){return smalltalk.is_kindOf_(anException,(type));};;
if (type._eq("all")){return true;};;
if (type.isArray()){type.detect_ifNone_((function(one){return self.exception_matches_(anException,one);}),(function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();};;
if (type.isString()){var cls=nil;
cls=self.smalltalk().at_ifAbsent_(cls,(function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return self.exception_matches_(anException,cls)})();throw($s8Ret$)})();};;
(function(){$s8Ret$.result=(function(){return (nil.is_eqeq_(type.evaluateWith_(anException),true))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"private",unescape("exception%3A%20anException%20matches%3A%20type%0D%0A%09%22%20Private%20-%20Return%20true%20if%20anException%20is%20of%20specified%20type.%20%22%0D%0A%0D%0A%09type%20isNil%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09%28self%20isObject%3A%20type%29%20ifFalse%3A%20%5B%20%5Etype%20%3D%20anException%20%5D.%0D%0A%09type%20isClass%20ifTrue%3A%20%5B%20%5EanException%20isKindOf%3A%20type%20%5D.%0D%0A%09type%20%3D%20%23all%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09type%20isArray%20ifTrue%3A%20%5B%0D%0A%09%09type%0D%0A%09%09%09detect%3A%20%5B%3Aone%7C%20self%20exception%3A%20anException%20matches%3A%20one%20%5D%0D%0A%09%09%09ifNone%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%09%5Etrue%0D%0A%09%5D.%0D%0A%09type%20isString%20ifTrue%3A%20%5B%20%7C%20cls%20%7C%0D%0A%09%09cls%20%3A%3D%20self%20smalltalk%20at%3A%20cls%20ifAbsent%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%09%5Eself%20exception%3A%20anException%20matches%3A%20cls%0D%0A%09%5D.%0D%0A%09%5E%28type%20evaluateWith%3A%20anException%29%20%3D%3D%20true%20"));
smalltalk.bind(smalltalk.BlockClosure,"on:do:",0
,function(type,aBlock){var self=this;
return self.try_catch_(self,(function(anException){return ((self.exception_matches_(anException,type)).mustBeBoolean()==true ? (function(){return aBlock.evaluateWith_(anException);})() : (function(){return anException.signal();})());}));}
,"evaluating",unescape("on%3A%20type%20do%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20catching%20exceptions%20of%20specified%20type.%20%22%0D%0A%0D%0A%09%5Eself%20try%3A%20self%20catch%3A%20%5B%3AanException%20%7C%0D%0A%09%09%28self%20exception%3A%20anException%20matches%3A%20type%29%0D%0A%09%09ifTrue%3A%20%5B%20aBlock%20evaluateWith%3A%20anException%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20anException%20signal%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.BlockClosure,"try:catch:",0
,function(aBlock,anotherBlock){var self=this;
var fn=nil;
var ab=nil;
fn=self.callableOf_(aBlock);
ab=self.callableOf_(anotherBlock);

	var result;
	try{ result = fn() } catch(e) {
		result = ab( self.smalltalkErrorOf_(e) )
	};
	return result;
return self;}
,"private",unescape("try%3A%20aBlock%20catch%3A%20anotherBlock%0D%0A%09%22%20Private%20-%20Try%20to%20evaluate%20aBlock%20catching%20exceptions%20with%20catchBlock.%20%22%0D%0A%0D%0A%09%7C%20fn%20ab%20%7C%0D%0A%09fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09ab%20%3A%3D%20self%20callableOf%3A%20anotherBlock.%0D%0A%09%7B%27%27%20doc%27secured%20activation%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"valueDeferred:",0
,function(milliseconds){var self=this;
self.valueWithTimeout_(milliseconds);
return self;}
,"evaluating",unescape("valueDeferred%3A%20milliseconds%0D%0A%09%22%20Evaluate%20the%20receiver%20deferred%20in%20time.%0D%0A%09Return%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09self%20valueWithTimeout%3A%20milliseconds"));
smalltalk.bind(smalltalk.BlockClosure,"valueDeferred",0
,function(){var self=this;
self.valueDeferred_((0));
return self;}
,"evaluating",unescape("valueDeferred%0D%0A%09%22%20Evaluate%20the%20receiver%20deferred%20in%20time.%0D%0A%09Return%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09self%20valueDeferred%3A%200"));
smalltalk.bind(smalltalk.BlockClosure,"valueWithTimeout:",0
,function(aNumber){var self=this;
return setTimeout(self,aNumber);}
,"timeout",unescape("valueWithTimeout%3A%20aNumber%0D%0A%09%22%20Evaluate%20the%20receiver%20with%20time%20out.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20timer%20activation%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"valueWithInterval:",0
,function(aNumber){var self=this;
return setInterval(self,aNumber);}
,"timeout",unescape("valueWithInterval%3A%20aNumber%0D%0A%09%22%20Evaluate%20the%20receiver%20with%20interval.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20timer%20activation%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"ensure:",0
,function(aBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
try{return self();} finally {fn();};
return self;}
,"evaluating",unescape("ensure%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%0D%0A%09Execute%20aBlock%20after%20the%20receiver%20is%20executed.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27%20doc%27protected%20activation%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"ifCurtailed:",0
,function(exceptionBlock){var self=this;
var curtailed=nil;
var result=nil;
curtailed=true;

	try{	result = self.value();
		curtailed = false;
	} finally {
		if (curtailed) exceptionBlock.value();
	};
	return result;
return self;}
,"evaluating",unescape("ifCurtailed%3A%20exceptionBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%0D%0A%09If%20evaluation%20is%20limited%20by%20exiting%20before%20completion%2C%20the%20exception%20block%20is%20evaluated.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20curtailed%20result%20%7C%0D%0A%09curtailed%20%3A%3D%20true.%0D%0A%09%7B%27%27%20doc%27protected%20activation%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"argumentCount",0
,function(){var self=this;
return self.length;}
,"evaluating",unescape("argumentCount%0D%0A%09%22%20Return%20the%20number%20of%20arguments%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27activation%20constants%27%7D"));
smalltalk.bind(smalltalk.BlockClosure,"installAs:",0
,function(functionName){var self=this;
return smalltalk.Smalltalk.install_as_(self,functionName);}
,"installing",unescape("installAs%3A%20functionName%0D%0A%09%22%20Install%20the%20receiver%20as%20a%20function%20in%20global%20scope.%20%22%0D%0A%0D%0A%09%5ESmalltalk%20install%3A%20self%20as%3A%20functionName"));
smalltalk.bind(smalltalk.BlockClosure,"asJSONObject",0
,function(){var self=this;
return self;}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.BlockClosure,"shallowCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.BlockClosure,"deepCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.BlockClosure,"isValuable",0
,function(){var self=this;
return true;}
,"testing",unescape("isValuable%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20can%20be%20evaluated.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.BlockClosure,"ms",0
,function(){var self=this;
return smalltalk.DateTime.millisecondsToRun_(self);}
,"accessing",unescape("ms%0D%0A%09%22%20Return%20the%20milliseconds%20taken%20to%20evaluate%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EDateTime%20millisecondsToRun%3A%20self"));
smalltalk.bind(smalltalk.BlockClosure,"evaluateFor:",0
,function(anObject){var self=this;
return self(anObject);}
,"evaluating",unescape("evaluateFor%3A%20anObject%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20with%20anObject.%20%22%0D%0A%0D%0A%09%5E%23%20%23self%3A%20anObject"));
smalltalk.bind(smalltalk.BlockClosure,"evaluate",0
,function(){var self=this;
return self();}
,"evaluating",unescape("evaluate%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%20%23self"));
smalltalk.bind(smalltalk.BlockClosure,"evaluateWithArguments:",0
,function(anArray){var self=this;
return self.valueWithPossibleArguments_(anArray);}
,"evaluating",unescape("evaluateWithArguments%3A%20anArray%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20valueWithPossibleArguments%3A%20anArray"));
smalltalk.bind(smalltalk.BlockClosure,"valueWithArguments:",0
,function(anArray){var self=this;
return self.valueWithPossibleArguments_(anArray);}
,"evaluating",unescape("valueWithArguments%3A%20anArray%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20with%20arguments.%20%22%0D%0A%0D%0A%09%5Eself%20valueWithPossibleArguments%3A%20anArray"));
smalltalk.bind(smalltalk.BlockClosure,"valueWithDNU",0
,function(){var self=this;
return smalltalk.Object.hackDNUWhile_(self);}
,"Tools-hacking",unescape("valueWithDNU%0D%0A%09%22%20Catch%20DNU%20hacking%20while%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EObject%20hackDNUWhile%3A%20self"));
smalltalk.bind(smalltalk.Boolean.$klass,"@","_at"
,function(aBOOL){var self=this;

	if (aBOOL === true) return true;
	if (aBOOL === 1) return true;
	if ((1 & aBOOL) + 0 == 1) return true;
return false;}
,"Natives-converting",unescape("@%20aBOOL%0D%0A%09%22%20Return%20aBOOL%20as%20a%20Boolean%20instance.%20%22%0D%0A%0D%0A%09%7B%27%27%7D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Boolean,"mustBeBoolean",0
,function(){var self=this;
return self;}
,"testing",unescape("mustBeBoolean%0D%0A%09%22%20Signal%20an%20error%20if%20the%20receiver%20is%20not%20a%20Boolean.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Boolean,"=","_eq"
,function(anObject){var self=this;

	if (nil.isNil_(anObject)) return false;
	if (!(self.$klass === anObject.$klass)) return false;
	return (self.valueOf() == anObject.valueOf());
return self;}
,"comparing",unescape("%3D%20anObject%0D%0A%20%20%20%20%22%20Return%20true%20if%20the%20receiver%20is%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27boolean%20test%27%7D"));
smalltalk.bind(smalltalk.Boolean,"ifTrue:ifFalse:",0
,function(aBlock,anotherBlock){var self=this;
if(self == true) { return aBlock.value(); } else { return anotherBlock.value(); };
return self;}
,"control",unescape("ifTrue%3A%20aBlock%20ifFalse%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20aBlock%20if%20the%20receiver%20is%20true%2C%20else%20the%20result%20of%20evaluating%20anotherBlock.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27boolean%20switch%27%7D"));
smalltalk.bind(smalltalk.Boolean,"ifTrue:",0
,function(aBlock){var self=this;
return ((self).mustBeBoolean()==true ? (function(){return aBlock.value();})() : (function(){return nil;})());}
,"control",unescape("ifTrue%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20aBlock%20if%20the%20receiver%20is%20true%2C%20else%20return%20nil.%20%22%0D%0A%0D%0A%09%5Eself%20ifTrue%3A%20%5B%20aBlock%20value%20%5D%20ifFalse%3A%20%5B%5D"));
smalltalk.bind(smalltalk.Boolean,"ifFalse:",0
,function(aBlock){var self=this;
return ((self).mustBeBoolean()==true ? (function(){return nil;})() : (function(){return aBlock.value();})());}
,"control",unescape("ifFalse%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20aBlock%20if%20the%20receiver%20is%20false%2C%20else%20return%20nil.%20%22%0D%0A%0D%0A%09%5Eself%20ifTrue%3A%20%5B%5D%20ifFalse%3A%20%5B%20aBlock%20value%20%5D"));
smalltalk.bind(smalltalk.Boolean,"ifFalse:ifTrue:",0
,function(aBlock,anotherBlock){var self=this;
return self.ifTrue_ifFalse_(anotherBlock,(function(){return aBlock.value();}));}
,"control",unescape("ifFalse%3A%20aBlock%20ifTrue%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20aBlock%20if%20the%20receiver%20is%20false%2C%20else%20the%20result%20of%20evaluating%20anotherBlock.%20%22%0D%0A%0D%0A%09%5Eself%20ifTrue%3A%20anotherBlock%20ifFalse%3A%20%5B%20aBlock%20value%20%5D"));
smalltalk.bind(smalltalk.Boolean,"and:",0
,function(aBlock){var self=this;
return ((self).mustBeBoolean()==true ? (function(){return aBlock.value();})() : (function(){return false;})());}
,"control",unescape("and%3A%20aBlock%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20true%20and%20the%20result%20of%20evaluating%20aBlock%20is%20also%20true.%20%22%0D%0A%0D%0A%09%5Eself%20ifTrue%3A%20%5B%20aBlock%20value%20%5D%20ifFalse%3A%20%5Bfalse%5D"));
smalltalk.bind(smalltalk.Boolean,"or:",0
,function(aBlock){var self=this;
return ((self).mustBeBoolean()==true ? (function(){return true;})() : (function(){return aBlock.value();})());}
,"control",unescape("or%3A%20aBlock%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20true%20or%20the%20result%20of%20evaluating%20aBlock%20is%20true.%20%22%0D%0A%0D%0A%09%5Eself%20ifTrue%3A%20%5Btrue%5D%20ifFalse%3A%20%5B%20aBlock%20value%20%5D"));
smalltalk.bind(smalltalk.Boolean,"not",0
,function(){var self=this;
return self == false;}
,"control",unescape("not%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20false.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27boolean%20inversion%27%7D"));
smalltalk.bind(smalltalk.Boolean,"printString",0
,function(){var self=this;
return ((self).mustBeBoolean()==true ? (function(){return "true";})() : (function(){return "false";})());}
,"printing",unescape("printString%0D%0A%09%22%20Return%20a%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20ifTrue%3A%20%5B%20%23true%20%5D%20ifFalse%3A%20%5B%20%23false%20%5D"));
smalltalk.bind(smalltalk.Boolean,"asJSONObject",0
,function(){var self=this;

return self.and_((function(){return true;}));}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself%20and%3A%20%5B%20true%20%5D"));
smalltalk.bind(smalltalk.Boolean,"shallowCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Boolean,"deepCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Boolean,"isBoolean",0
,function(){var self=this;
return true;}
,"testing",unescape("isBoolean%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20boolean.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.DateTime.$klass,"new:",0
,function(anObject){var self=this;
return new Date(anObject);
return self;}
,"instantiation",unescape("new%3A%20anObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20instantiation%27%7D"));
smalltalk.bind(smalltalk.DateTime.$klass,"fromString:",0
,function(aString){var self=this;
return self.new_(aString);}
,"instantiation",unescape("fromString%3A%20aString%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%0D%0A%09DateTime%20fromString%3A%20%271962/12/15%2001%3A40%3A00%27%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20new%3A%20aString"));
smalltalk.bind(smalltalk.DateTime.$klass,"fromSeconds:",0
,function(aNumber){var self=this;
return self.fromMilliseconds_(aNumber._star((1000)));}
,"instantiation",unescape("fromSeconds%3A%20aNumber%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20fromMilliseconds%3A%20aNumber%20*%201000"));
smalltalk.bind(smalltalk.DateTime.$klass,"fromMilliseconds:",0
,function(aNumber){var self=this;
return self.new_(aNumber);}
,"instantiation",unescape("fromMilliseconds%3A%20aNumber%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%3A%20aNumber"));
smalltalk.bind(smalltalk.DateTime.$klass,"today",0
,function(){var self=this;

return self.$$new();}
,"instantiation",unescape("today%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20instantiation%27%7D.%0D%0A%09%5Eself%20new"));
smalltalk.bind(smalltalk.DateTime.$klass,"now",0
,function(){var self=this;

return self.today();}
,"instantiation",unescape("now%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20instantiation%27%7D.%0D%0A%09%5Eself%20today"));
smalltalk.bind(smalltalk.DateTime.$klass,"msNow",0
,function(){var self=this;

return self.$$new();}
,"instantiation",unescape("msNow%0D%0A%09%22%20Private%20-%20Return%20a%20magnitude%20to%20measure%20milliseconds.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20instantiation%27%7D.%0D%0A%09%5Eself%20new"));
smalltalk.bind(smalltalk.DateTime.$klass,"millisecondsToRun:",0
,function(aBlock){var self=this;
var start=nil;
start=self.msNow();
aBlock.value();
return self.msNow()._minus(start);}
,"instantiation",unescape("millisecondsToRun%3A%20aBlock%0D%0A%09%22%20Return%20the%20time%20elapsed%20during%20evaluation%20of%20aBlock.%20%22%0D%0A%0D%0A%09%7C%20start%20%7C%0D%0A%09start%20%3A%3D%20self%20msNow.%0D%0A%09aBlock%20value.%0D%0A%09%5Eself%20msNow%20-%20start"));
smalltalk.bind(smalltalk.DateTime,"year",0
,function(){var self=this;
return self.getFullYear();}
,"accessing",unescape("year%0D%0A%09%22%20Return%20the%20year%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"month",0
,function(){var self=this;
return self.getMonth() + 1;}
,"accessing",unescape("month%0D%0A%09%22%20Return%20the%20month%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"month:",0
,function(aNumber){var self=this;
self.setMonth(aNumber - 1);
return self;}
,"accessing",unescape("month%3A%20aNumber%0D%0A%09%22%20Set%20the%20month%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"day",0
,function(){var self=this;
return self.getDay() + 1;}
,"accessing",unescape("day%0D%0A%09%22%20Return%20the%20day%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"dayOfWeek",0
,function(){var self=this;
self.invalidMessage_("dayOfWeek");
return self;}
,"accessing",unescape("dayOfWeek%0D%0A%09%22%20Return%20the%20day%20of%20week%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20invalidMessage%3A%20%23dayOfWeek."));
smalltalk.bind(smalltalk.DateTime,"dayOfWeek:",0
,function(aNumber){var self=this;
self.invalidMessage_("dayOfWeek:");
return self;}
,"accessing",unescape("dayOfWeek%3A%20aNumber%0D%0A%09%22%20Set%20the%20day%20of%20week%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20invalidMessage%3A%20%23dayOfWeek%3A."));
smalltalk.bind(smalltalk.DateTime,"day:",0
,function(aNumber){var self=this;
self.setDay(aNumber - 1);
return self;}
,"accessing",unescape("day%3A%20aNumber%0D%0A%09%22%20Set%20the%20day%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"year:",0
,function(aNumber){var self=this;
self.setFullYear(aNumber);
return self;}
,"accessing",unescape("year%3A%20aNumber%0D%0A%09%22%20Set%20the%20year%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"dayOfMonth",0
,function(){var self=this;
return self.getDate();}
,"accessing",unescape("dayOfMonth%0D%0A%09%22%20Return%20the%20day%20of%20month%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"dayOfMonth:",0
,function(aNumber){var self=this;
self.setDate(aNumber);
return self;}
,"accessing",unescape("dayOfMonth%3A%20aNumber%0D%0A%09%22%20Set%20the%20day%20of%20month%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"asString",0
,function(){var self=this;

return self.toString();}
,"converting",unescape("asString%0D%0A%09%22%20Return%20the%20text%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27self%27%20doc%27native%20conversion%27%7D.%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.DateTime,"printString",0
,function(){var self=this;
return self.asString();}
,"printing",unescape("printString%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20asString"));
smalltalk.bind(smalltalk.DateTime,"asMilliseconds",0
,function(){var self=this;
return self.time();}
,"converting",unescape("asMilliseconds%0D%0A%09%22%20Return%20the%20receiver%20as%20milliseconds.%20%22%0D%0A%0D%0A%09%5Eself%20time"));
smalltalk.bind(smalltalk.DateTime,"time",0
,function(){var self=this;
return self.getTime();}
,"accessing",unescape("time%0D%0A%09%22%20Return%20the%20time%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"time:",0
,function(aNumber){var self=this;
self.setTime(aNumber);
return self;}
,"accessing",unescape("time%3A%20aNumber%0D%0A%09%22%20Set%20the%20time%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"asDateString",0
,function(){var self=this;
return self.toDateString();}
,"converting",unescape("asDateString%0D%0A%09%22%20Return%20the%20date%20string%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20converting%27%7D"));
smalltalk.bind(smalltalk.DateTime,"asTimeString",0
,function(){var self=this;
return self.toTimeString();}
,"converting",unescape("asTimeString%0D%0A%09%22%20Return%20the%20time%20string%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20converting%27%7D"));
smalltalk.bind(smalltalk.DateTime,"asLocaleString",0
,function(){var self=this;
return self.toLocaleString();}
,"converting",unescape("asLocaleString%0D%0A%09%22%20Return%20the%20locale%20string%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20converting%27%7D"));
smalltalk.bind(smalltalk.DateTime,"asNumber",0
,function(){var self=this;
return self.asMilliseconds();}
,"converting",unescape("asNumber%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20Number.%20%22%0D%0A%0D%0A%09%5Eself%20asMilliseconds"));
smalltalk.bind(smalltalk.DateTime,"hours:",0
,function(aNumber){var self=this;
self.setHours(aNumber);
return self;}
,"accessing",unescape("hours%3A%20aNumber%0D%0A%09%22%20Set%20the%20hours%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"minutes:",0
,function(aNumber){var self=this;
self.setMinutes(aNumber);
return self;}
,"accessing",unescape("minutes%3A%20aNumber%0D%0A%09%22%20Set%20the%20minutes%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"seconds:",0
,function(aNumber){var self=this;
self.setSeconds(aNumber);
return self;}
,"accessing",unescape("seconds%3A%20aNumber%0D%0A%09%22%20Set%20the%20seconds%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"milliseconds:",0
,function(aNumber){var self=this;
self.setMilliseconds(aNumber);
return self;}
,"accessing",unescape("milliseconds%3A%20aNumber%0D%0A%09%22%20Set%20the%20milliseconds%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"hours",0
,function(){var self=this;
return self.getHours();}
,"accessing",unescape("hours%0D%0A%09%22%20Return%20the%20hours%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"minutes",0
,function(){var self=this;
return self.getMinutes();}
,"accessing",unescape("minutes%0D%0A%09%22%20Return%20the%20minutes%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"seconds",0
,function(){var self=this;
return self.getSeconds();}
,"accessing",unescape("seconds%0D%0A%09%22%20Return%20the%20seconds%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"milliseconds",0
,function(){var self=this;
return self.getMilliseconds();}
,"accessing",unescape("milliseconds%0D%0A%09%22%20Return%20the%20milliseconds%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"<","_lt"
,function(aDateTime){var self=this;
return self < aDateTime;}
,"comparing",unescape("%3C%20aDateTime%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20smaller%20than%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20compare%27%7D"));
smalltalk.bind(smalltalk.DateTime,">","_gt"
,function(aDateTime){var self=this;
return self > aDateTime;}
,"comparing",unescape("%3E%20aDateTime%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20greater%20than%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20compare%27%7D"));
smalltalk.bind(smalltalk.DateTime,"<=","_lt_eq"
,function(aDateTime){var self=this;
return self <= aDateTime;}
,"comparing",unescape("%3C%3D%20aDateTime%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20smaller%20or%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20compare%27%7D"));
smalltalk.bind(smalltalk.DateTime,">=","_gt_eq"
,function(aDateTime){var self=this;
return self >= aDateTime;}
,"comparing",unescape("%3E%3D%20aDateTime%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20greater%20or%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20compare%27%7D"));
smalltalk.bind(smalltalk.DateTime,"-","_minus"
,function(aDateTime){var self=this;
return self - aDateTime;}
,"arithmetic",unescape("-%20aDateTime%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20operation%27%7D"));
smalltalk.bind(smalltalk.DateTime,"+","_plus"
,function(aDateTime){var self=this;
return self + aDateTime;}
,"arithmetic",unescape("+%20aDateTime%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20operation%27%7D"));
smalltalk.bind(smalltalk.DateTime,"isValid",0
,function(){var self=this;
return isFinite(self);}
,"testing",unescape("isValid%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20valid%20instance.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20testing%27%7D"));
smalltalk.bind(smalltalk.DateTime,"asJSONObject",0
,function(){var self=this;

return self;}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%7B%27self%27%20doc%27native%20converting%27%7D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.UndefinedObject.$klass,"new","$$new"
,function(){var self=this;
return self.error_(unescape("UndefinedObject%20can%27t%20be%20instantiated."));}
,"instantiation",unescape("new%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%27UndefinedObject%20can%27%27t%20be%20instantiated.%27"));
smalltalk.bind(smalltalk.UndefinedObject.$klass,"alias",0
,function(){var self=this;
return self.basicAt_ifAbsentPut_(unescape("@alias"),(function(){return self.basicNew();}));}
,"alias",unescape("alias%0D%0A%09%22%20Private%20-%20Return%20an%20alias%20of%20nil.%0D%0A%09Note%20that%20is%20is%20a%20hack%20used%20to%20wrap%20nil%20in%20an%20alias%20instance%20to%20make%20possible%20for%20Lua%20collections%20to%20contain%20nil%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27@alias%27%20ifAbsentPut%3A%20%5B%20self%20basicNew%20%5D"));
smalltalk.bind(smalltalk.UndefinedObject,"ifNil:",0
,function(aBlock){var self=this;
return aBlock.value();}
,"testing",unescape("ifNil%3A%20aBlock%0D%0A%09%22%20Return%20the%20receiver%20if%20it%20is%20not%20nil%2C%20or%20the%20result%20of%20evaluating%20aBlock%20%28if%20it%20is%20nil%29.%20%22%0D%0A%0D%0A%09%5EaBlock%20value"));
smalltalk.bind(smalltalk.UndefinedObject,"ifNotNil:",0
,function(aBlock){var self=this;
return self;}
,"testing",unescape("ifNotNil%3A%20aBlock%0D%0A%09%22%20Return%20the%20receiver%20if%20it%20is%20nil%2C%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.UndefinedObject,"ifNil:ifNotNil:",0
,function(aBlock,anotherBlock){var self=this;
return aBlock.value();}
,"testing",unescape("ifNil%3A%20aBlock%20ifNotNil%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20receiver%20if%20it%20is%20nil%2C%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5EaBlock%20value"));
smalltalk.bind(smalltalk.UndefinedObject,"ifNotNil:ifNil:",0
,function(aBlock,anotherBlock){var self=this;
return anotherBlock.value();}
,"testing",unescape("ifNotNil%3A%20aBlock%20ifNil%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20anotherBlock%20if%20teh%20receiver%20is%20nil%2C%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5EanotherBlock%20value"));
smalltalk.bind(smalltalk.UndefinedObject,"isNil",0
,function(){var self=this;
return true;}
,"testing",unescape("isNil%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20nil.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.UndefinedObject,"notNil",0
,function(){var self=this;
return false;}
,"testing",unescape("notNil%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20not%20nil.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.UndefinedObject,"printString",0
,function(){var self=this;
return "nil";}
,"printing",unescape("printString%0D%0A%09%22%20Return%20the%20text%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23nil"));
smalltalk.bind(smalltalk.UndefinedObject,"subclass:instanceVariableNames:",0
,function(aString,anotherString){var self=this;
return self.subclass_instanceVariableNames_category_(aString,anotherString,nil);}
,"instantiation",unescape("subclass%3A%20aString%20instanceVariableNames%3A%20anotherString%0D%0A%09%22%20Create%20a%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20anotherString%0D%0A%09%09category%3A%20nil"));
smalltalk.bind(smalltalk.UndefinedObject,"subclass:instanceVariableNames:category:",0
,function(aString,aString2,aString3){var self=this;
return smalltalk.ClassBuilder.$$new().superclass_subclass_instanceVariableNames_category_(self,aString,aString2,aString3);}
,"instantiation",unescape("subclass%3A%20aString%20instanceVariableNames%3A%20aString2%20category%3A%20aString3%0D%0A%09%22%20Create%20a%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EClassBuilder%20new%0D%0A%09%09superclass%3A%20self%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09category%3A%20aString3"));
smalltalk.bind(smalltalk.UndefinedObject,"asJSONObject",0
,function(){var self=this;
return null;}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20constant%27%7D"));
smalltalk.bind(smalltalk.UndefinedObject,"shallowCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.UndefinedObject,"deepCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.UndefinedObject,"value",0
,function(){var self=this;
return self;}
,"evaluating",unescape("value%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22"));
smalltalk.bind(smalltalk.UndefinedObject,"@","_at"
,function(aNumber){var self=this;
return smalltalk.Point.x_y_((0),aNumber);}
,"converting",unescape("@%20aNumber%0D%0A%09%22%20Return%20a%20Point%20with%20coordinates%20given%20by%20the%20receiver%20and%20aNumber.%20%22%0D%0A%0D%0A%09%5EPoint%20x%3A%200%20y%3A%20aNumber"));
smalltalk.bind(smalltalk.UndefinedObject,"=","_eq"
,function(anObject){var self=this;
return (nil.isNil_(anObject));}
,"comparing",unescape("%3D%20anObject%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%5EanObject%20isNil"));
smalltalk.bind(smalltalk.UndefinedObject,"~=","_no_eq"
,function(anObject){var self=this;
return (nil.isNil_(anObject)===false);}
,"comparing",unescape("%7E%3D%20anObject%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20not%20equal%20to%20anObject.%20%22%0D%0A%0D%0A%09%5EanObject%20notNil"));
smalltalk.bind(smalltalk.UndefinedObject,"category:",0
,function(ignored){var self=this;
return self;}
,"instantiation",unescape("category%3A%20ignored%0D%0A%09%22%20BUG%20-%20throw%20an%20error%20in%20ClassBuilder.%20%22%20"));
smalltalk.bind(smalltalk.UndefinedObject,"subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:",0
,function(aString,aString2,classVarNames,poolVarNames,aCategory){var self=this;
if ((nil.isNil_(self.smalltalk().globalAt_(aString))===false)){self.note_("WARNING: class REdefinition for "._comma(aString));};;
return smalltalk.ClassBuilder.$$new().superclass_subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(self,aString,aString2,classVarNames,poolVarNames,aCategory);}
,"instantiation",unescape("subclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09classVariableNames%3A%20classVarNames%0D%0A%09poolDictionaries%3A%20poolVarNames%0D%0A%09category%3A%20aCategory%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%28self%20smalltalk%20globalAt%3A%20aString%29%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09self%20note%3A%20%27WARNING%3A%20class%20REdefinition%20for%20%27%2CaString.%0D%0A%09%09%22%5Eself%20smalltalk%20globalAt%3A%20aString%22%0D%0A%09%5D.%0D%0A%0D%0A%22%09self%20note%3A%20%27Creating%20ROOT%20class%20%27%2CaString.%22%0D%0A%09%5EClassBuilder%20new%0D%0A%09%09superclass%3A%20self%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20classVarNames%0D%0A%09%09poolDictionaries%3A%20poolVarNames%0D%0A%09%09category%3A%20aCategory"));
smalltalk.bind(smalltalk.UndefinedObject,"subclass:instanceVariableNames:classVariableNames:poolDictionaries:",0
,function(aString,aString2,classVarNames,poolVarNames){var self=this;
return self.subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aString,aString2,classVarNames,poolVarNames,nil);}
,"instantiation",unescape("subclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09classVariableNames%3A%20classVarNames%0D%0A%09poolDictionaries%3A%20poolVarNames%0D%0A%09%22%20Create%20a%20new%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20classVarNames%0D%0A%09%09poolDictionaries%3A%20poolVarNames%0D%0A%09%09category%3A%20nil"));
smalltalk.bind(smalltalk.UndefinedObject,"compiler",0
,function(){var self=this;
return self.smalltalk().compilerSupport().for_(nil);}
,"Tools-printing",unescape("compiler%0D%0A%09%22%20Return%20the%20compiler%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20smalltalk%20compilerSupport%20for%3A%20nil"));
smalltalk.bind(smalltalk.UndefinedObject,"==","_eq_eq"
,function(anObject){var self=this;
return (nil.isNil_(anObject));}
,"Tools-comparing",unescape("%3D%3D%20anObject%0D%0A%09%5EanObject%20isNil"));
smalltalk.bind(smalltalk.UndefinedObject,"~~","_no_no"
,function(anObject){var self=this;
return (nil.isNil_(anObject)===false);}
,"Tools-comparing",unescape("%7E%7E%20anObject%0D%0A%09%5EanObject%20notNil"));
smalltalk.bind(smalltalk.UndefinedObject,"handles",0
,function(){var self=this;
return self;}
,"Natives-converting",unescape("handles%0D%0A%09%22%20Return%20the%20result%20of%20collecting%20the%20handles%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.UndefinedObject,"handle",0
,function(){var self=this;
return self.asJSONObject();}
,"Natives-converting",unescape("handle%0D%0A%09%22%20Return%20the%20handle%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20asJSONObject"));
smalltalk.bind(smalltalk.UndefinedObject,"s8DoIt",0
,function(){var self=this;
return (function(){self.note_(unescape("Saving%20default.snapshot%20for%20browsing/debugging"));return unescape("function%20print%28text%29%20%7B%20return%20console.log%28text%29%3B%20%7D%0D%0A")._comma(smalltalk.Snapshot.contents()).outputToFile_(unescape("../default.snapshot.js"));}).value();}
,"s8DoIt",unescape("s8DoIt%20%5E%5Bself%20note%3A%20%27Saving%20default.snapshot%20for%20browsing/debugging%27.%0D%0A%27function%20print%28text%29%20%7B%20return%20console.log%28text%29%3B%20%7D%0D%0A%27%2CSnapshot%20contents%20outputToFile%3A%20%27../default.snapshot.js%27%5D%20value"));
smalltalk.bind(smalltalk.Collection.$klass,"streamClass",0
,function(){var self=this;
return smalltalk.Stream;}
,"streams",unescape("streamClass%0D%0A%09%22%20Private%20-%20Return%20the%20support%20for%20streamming%20on%20the%20receiver%27s%20instances.%20%22%0D%0A%0D%0A%09%5EStream"));
smalltalk.bind(smalltalk.Collection.$klass,"with:",0
,function(anObject){var self=this;
return (function($rec){$rec.add_(anObject);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("with%3A%20anObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09add%3A%20anObject%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Collection.$klass,"with:with:",0
,function(anObject,anotherObject){var self=this;
return (function($rec){$rec.add_(anObject);$rec.add_(anotherObject);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("with%3A%20anObject%20with%3A%20anotherObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09add%3A%20anObject%3B%0D%0A%09%09add%3A%20anotherObject%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Collection.$klass,"with:with:with:",0
,function(firstObject,secondObject,thirdObject){var self=this;
return (function($rec){$rec.add_(firstObject);$rec.add_(secondObject);$rec.add_(thirdObject);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("with%3A%20firstObject%20with%3A%20secondObject%20with%3A%20thirdObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09add%3A%20firstObject%3B%0D%0A%09%09add%3A%20secondObject%3B%0D%0A%09%09add%3A%20thirdObject%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Collection.$klass,"with:with:with:with:",0
,function(firstObject,secondObject,thirdObject,fourthObject){var self=this;
return (function($rec){$rec.add_(firstObject);$rec.add_(secondObject);$rec.add_(thirdObject);$rec.add_(fourthObject);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("with%3A%20firstObject%20with%3A%20secondObject%20with%3A%20thirdObject%20with%3A%20fourthObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09add%3A%20firstObject%3B%0D%0A%09%09add%3A%20secondObject%3B%0D%0A%09%09add%3A%20thirdObject%3B%0D%0A%09%09add%3A%20fourthObject%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Collection.$klass,"withAll:",0
,function(aCollection){var self=this;
return (function($rec){$rec.addAll_(aCollection);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("withAll%3A%20aCollection%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09addAll%3A%20aCollection%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Collection.$klass,"basicNew:",0
,function(size){var self=this;
return self.subclassResponsibility_("basicNew:");}
,"instantiation",unescape("basicNew%3A%20size%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20of%20specified%20size.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23basicNew%3A"));
smalltalk.bind(smalltalk.Collection.$klass,"of:withAll:",0
,function(aType,handles){var self=this;
if (aType.includes_(unescape("%2C"))){var stream=nil;
stream=aType.stream();
return handles.collect_((function(h){var cls=nil;
cls=stream.upTo_(unescape("%2C"));return ((cls.isEmpty()).mustBeBoolean()==true ? (function(){return h;})() : (function(){cls=smalltalk.Smalltalk.at_ifAbsent_(cls,(function(){return self.error_("Missing global "._comma(cls));}));return cls._at(h);})());}));};;
return self.withAll_(smalltalk.Array._at(handles).collect_((function(each){return aType._at(each);})));}
,"Natives-instantiation",unescape("of%3A%20aType%20withAll%3A%20handles%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20wrapping%20handles%20with%20a%20type%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%28aType%20includes%3A%20%24%2C%29%20ifTrue%3A%20%5B%20%7C%20stream%20%7C%0D%0A%09%09stream%20%3A%3D%20aType%20stream.%0D%0A%09%09%5Ehandles%20collect%3A%20%5B%3Ah%7C%20%7C%20cls%20%7C%0D%0A%09%09%09cls%20%3A%3D%20stream%20upTo%3A%20%24%2C.%0D%0A%09%09%09cls%20isEmpty%20ifTrue%3A%20%5B%20h%20%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09%09cls%20%3A%3D%20Smalltalk%20at%3A%20cls%20ifAbsent%3A%20%5B%0D%0A%09%09%09%09%09self%20error%3A%20%27Missing%20global%20%27%20%2Ccls%0D%0A%09%09%09%09%5D.%0D%0A%09%09%09%09cls%20@%20h%0D%0A%09%09%09%5D%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Eself%20withAll%3A%20%28Array%20@%20handles%20collect%3A%20%5B%3Aeach%7C%20aType%20@%20each%20%5D%29"));
smalltalk.bind(smalltalk.Collection,"wrap:",0
,function(anObject){var self=this;

return anObject;}
,"alias",unescape("wrap%3A%20anObject%0D%0A%09%22%20Private%20-%20Return%20an%20element%20holding%20anObject.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5EanObject"));
smalltalk.bind(smalltalk.Collection,"unwrap:",0
,function(anObject){var self=this;

return anObject;}
,"alias",unescape("unwrap%3A%20anObject%0D%0A%09%22%20Private%20-%20Return%20object%20held%20by%20anObject.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5EanObject"));
smalltalk.bind(smalltalk.Collection,"size",0
,function(){var self=this;
return self.subclassResponsibility_("size");}
,"accessing",unescape("size%0D%0A%09%22%20Return%20the%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23size"));
smalltalk.bind(smalltalk.Collection,"readStream",0
,function(){var self=this;
return self.stream();}
,"streams",unescape("readStream%0D%0A%09%22%20Return%20a%20read%20stream%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20stream"));
smalltalk.bind(smalltalk.Collection,"writeStream",0
,function(){var self=this;
return self.stream();}
,"streams",unescape("writeStream%0D%0A%09%22%20Return%20a%20write%20stream%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20stream"));
smalltalk.bind(smalltalk.Collection,"stream",0
,function(){var self=this;
return self.streamClass().on_(self);}
,"streams",unescape("stream%0D%0A%09%22%20Return%20a%20stream%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20streamClass%20on%3A%20self"));
smalltalk.bind(smalltalk.Collection,"streamClass",0
,function(){var self=this;
return self.$klass.streamClass();}
,"streams",unescape("streamClass%0D%0A%09%22%20Private%20-%20Return%20the%20support%20for%20streamming%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20class%20streamClass"));
smalltalk.bind(smalltalk.Collection,"objectNotFound",0
,function(){var self=this;
self.smalltalk().objectNotFound(nil,nil,self);
return self.error_("Object is not in the collection");}
,"error",unescape("objectNotFound%0D%0A%09%22%20Private%20-%20Generates%20an%20error.%0D%0A%09An%20object%20was%20not%20found%20in%20the%20receiver%27s%20contents.%0D%0A%09%22%0D%0A%0D%0A%09self%20smalltalk%20%23objectNotFound%3A%20nil%20at%3A%20nil%20in%3A%20self.%0D%0A%09%5Eself%20error%3A%20%27Object%20is%20not%20in%20the%20collection%27"));
smalltalk.bind(smalltalk.Collection,"invalidIndex:",0
,function(anIndex){var self=this;
return self.error_("Invalid index "._comma(anIndex));}
,"error",unescape("invalidIndex%3A%20anIndex%0D%0A%09%22%20Private%20-%20Generates%20an%20error.%0D%0A%09An%20index%20to%20the%20receiver%27s%20contents%20is%20invalid/wrong.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%27Invalid%20index%20%27%20%2CanIndex%20"));
smalltalk.bind(smalltalk.Collection,"objectNotFound:",0
,function(anObject){var self=this;
self.smalltalk().objectNotFound(anObject,nil,self);
return self.error_("Object not found: "._comma(self.toString_(anObject)));}
,"error",unescape("objectNotFound%3A%20anObject%0D%0A%09%22%20Private%20-%20Generates%20an%20error.%0D%0A%09anObject%20was%20not%20found%20in%20the%20receiver%27s%20contents.%0D%0A%09%22%0D%0A%0D%0A%09self%20smalltalk%20%23objectNotFound%3A%20anObject%20at%3A%20nil%20in%3A%20self.%0D%0A%09%5Eself%20error%3A%20%27Object%20not%20found%3A%20%27%20%2C%28self%20toString%3A%20anObject%29"));
smalltalk.bind(smalltalk.Collection,"objectNotFoundAt:",0
,function(indexOrKey){var self=this;
self.smalltalk().objectNotFound(nil,indexOrKey,self);
return self.error_("Object not found at "._comma(self.toString_(indexOrKey))._comma(" in ")._comma(self.asLiteral()));}
,"error",unescape("objectNotFoundAt%3A%20indexOrKey%0D%0A%09%22%20Private%20-%20Generates%20an%20error.%0D%0A%09The%20object%20at%20key/index%20was%20not%20found%20in%20the%20receiver%27s%20contents.%0D%0A%09%22%0D%0A%0D%0A%09self%20smalltalk%20%23objectNotFound%3A%20nil%20at%3A%20indexOrKey%20in%3A%20self.%0D%0A%09%5Eself%20error%3A%20%27Object%20not%20found%20at%20%27%0D%0A%09%09%2C%28self%20toString%3A%20indexOrKey%29%0D%0A%09%09%2C%27%20in%20%27%20%2Cself%20asLiteral"));
smalltalk.bind(smalltalk.Collection,"add:",0
,function(anObject){var self=this;
return self.subclassResponsibility_("add:");}
,"adding",unescape("add%3A%20anObject%0D%0A%09%22%20Add%20anObject%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23add%3A"));
smalltalk.bind(smalltalk.Collection,"addAll:",0
,function(aCollection){var self=this;
aCollection.do_((function(each){return self.add_(each);}));
return aCollection;}
,"adding",unescape("addAll%3A%20aCollection%0D%0A%09%22%20Add%20objects%20in%20aCollection%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09aCollection%20do%3A%20%5B%3Aeach%20%7C%20self%20add%3A%20each%20%5D.%0D%0A%09%5EaCollection"));
smalltalk.bind(smalltalk.Collection,",","_comma"
,function(aCollection){var self=this;
return (function($rec){$rec.addAll_(aCollection);return $rec.yourself();})(self.copy());}
,"copying",unescape("%2C%20aCollection%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20with%20all%20contents%20of%20aCollection.%20%22%0D%0A%0D%0A%09%5Eself%20copy%0D%0A%09%09addAll%3A%20aCollection%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Collection,"copyWith:",0
,function(anObject){var self=this;
return (function($rec){$rec.add_(anObject);return $rec.yourself();})(self.copy());}
,"copying",unescape("copyWith%3A%20anObject%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20with%20anObject.%20%22%0D%0A%0D%0A%09%5Eself%20copy%20add%3A%20anObject%3B%20yourself"));
smalltalk.bind(smalltalk.Collection,"copyWithAll:",0
,function(aCollection){var self=this;
return (function($rec){$rec.addAll_(aCollection);return $rec.yourself();})(self.copy());}
,"copying",unescape("copyWithAll%3A%20aCollection%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20with%20all%20contents%20of%20aCollection.%20%22%0D%0A%0D%0A%09%5Eself%20copy%0D%0A%09%09addAll%3A%20aCollection%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Collection,"asArray",0
,function(){var self=this;
var result=nil;
result=smalltalk.Array.$$new();
self.do_((function(each){return result.add_(each);}));
return result;}
,"converting",unescape("asArray%0D%0A%09%22%20Return%20an%20array%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20Array%20new.%0D%0A%09self%20do%3A%20%5B%3Aeach%7C%20result%20add%3A%20each%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Collection,"do:",0
,function(aBlock){var self=this;
return self.subclassResponsibility_("do:");}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23do%3A"));
smalltalk.bind(smalltalk.Collection,"detect:ifNone:",0
,function(aBlock,anotherBlock){var self=this;
return self.subclassResponsibility_("detect:ifNone:");}
,"enumerating",unescape("detect%3A%20aBlock%20ifNone%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver%20that%20return%20true%20when%20evaluated%20with%20aBlock%20or%20the%20result%20of%20evaluating%20anotherBlock.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23detect%3AifNone%3A"));
smalltalk.bind(smalltalk.Collection,"reverseDo:",0
,function(aBlock){var self=this;
return self.subclassResponsibility_("reverseDo:");}
,"enumerating",unescape("reverseDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver%20in%20reverse%20order%20of%20iteration.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23reverseDo%3A"));
smalltalk.bind(smalltalk.Collection,"collect:",0
,function(aBlock){var self=this;
var result=nil;
result=self.species().$$new();
self.do_((function(each){return result.add_(aBlock.value_(each));}));
return result;}
,"enumerating",unescape("collect%3A%20aBlock%0D%0A%09%22%20Return%20a%20collection%20with%20the%20result%20of%20evaluating%20aBlock%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20species%20new.%0D%0A%09self%20do%3A%20%5B%3Aeach%20%7C%20result%20add%3A%20%28aBlock%20value%3A%20each%29%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Collection,"select:",0
,function(aBlock){var self=this;
var result=nil;
result=self.species().$$new();
self.do_((function(each){return ((aBlock.value_(each)).mustBeBoolean()==true ? (function(){return result.add_(each);})() : nil);}));
return result;}
,"enumerating",unescape("select%3A%20aBlock%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver%20which%20evaluates%20aBlock%20to%20true.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20species%20new.%0D%0A%09self%20do%3A%20%5B%3Aeach%7C%0D%0A%09%20%20%20%20%28aBlock%20value%3A%20each%29%20ifTrue%3A%20%5B%20result%20add%3A%20each%20%5D%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Collection,"detect:",0
,function(aBlock){var self=this;
return self.detect_ifNone_(aBlock,(function(){return self.objectNotFound();}));}
,"enumerating",unescape("detect%3A%20aBlock%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver%20that%20return%20true%20when%20evaluated%20with%20aBlock.%0D%0A%09Generates%20an%20error%20if%20there%20is%20no%20object%20that%20evaluates%20aBlock%20to%20true.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20detect%3A%20aBlock%20ifNone%3A%20%5B%20self%20objectNotFound%20%5D"));
smalltalk.bind(smalltalk.Collection,"do:andBetweenDo:",0
,function(aBlock,separatorBlock){var self=this;
return self.do_separatedBy_(aBlock,separatorBlock);}
,"enumerating",unescape("do%3A%20aBlock%20andBetweenDo%3A%20separatorBlock%0D%0A%09%22%20For%20each%20element%20in%20the%20receiver%2C%20evaluate%20aBlock%20with%20that%20element%20as%20the%20argument.%0D%0A%09Evaluate%20the%20separatorBlock%20between%20successive%20elements%20of%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20do%3A%20aBlock%20separatedBy%3A%20separatorBlock"));
smalltalk.bind(smalltalk.Collection,"do:separatedBy:",0
,function(aBlock,anotherBlock){var self=this;
var first=nil;
first=true;
self.do_((function(each){if (first){first=false;} else{anotherBlock.value();};;return aBlock.value_(each);}));
return self;}
,"enumerating",unescape("do%3A%20aBlock%20separatedBy%3A%20anotherBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20contents%20of%20the%20receiver%20evaluating%20anotherBlock%20inbetween%20aBlock%20evaluations.%20%22%0D%0A%0D%0A%09%7C%20first%20%7C%0D%0A%09first%20%3A%3D%20true.%0D%0A%09self%20do%3A%20%5B%3Aeach%20%7C%0D%0A%09%20%20%20%20first%0D%0A%09%09ifTrue%3A%20%5B%20first%20%3A%3D%20false%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20anotherBlock%20value%20%5D.%0D%0A%09%20%20%20%20aBlock%20value%3A%20each%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Collection,"inject:into:",0
,function(anObject,aBlock){var self=this;
var result=nil;
result=anObject;
self.do_((function(each){return result=aBlock.value_value_(result,each);}));
return result;}
,"enumerating",unescape("inject%3A%20anObject%20into%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20result%20of%20previous%20evaluation%20and%20contents%20of%20the%20receiver.%0D%0A%09On%20first%20evaluation%20anObject%20is%20used%20as%20first%20argument%20to%20evaluate%20aBlock.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20anObject.%0D%0A%09self%20do%3A%20%5B%3Aeach%20%7C%20result%20%3A%3D%20aBlock%20value%3A%20result%20value%3A%20each%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Collection,"reject:",0
,function(aBlock){var self=this;
return self.select_((function(each){return aBlock.value_(each)._eq(false);}));}
,"enumerating",unescape("reject%3A%20aBlock%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver%20wich%20evaluates%20aBlock%20to%20false.%20%22%0D%0A%0D%0A%09%5Eself%20select%3A%20%5B%3Aeach%20%7C%20%28aBlock%20value%3A%20each%29%20%3D%20false%5D"));
smalltalk.bind(smalltalk.Collection,"select:thenCollect:",0
,function(selectBlock,collectBlock){var self=this;
return self.select_(selectBlock).collect_(collectBlock);}
,"enumerating",unescape("select%3A%20selectBlock%20thenCollect%3A%20collectBlock%0D%0A%09%22%20Return%20a%20new%20collection%20with%20the%20result%20of%20evaluating%20collectBlock%20on%20the%20contents%20of%20the%20receiver%20wich%20evaluates%20selectBlock%20to%20true.%20%22%0D%0A%0D%0A%09%5E%28self%20select%3A%20selectBlock%29%20collect%3A%20collectBlock"));
smalltalk.bind(smalltalk.Collection,"collect:thenSelect:",0
,function(collectBlock,selectBlock){var self=this;
return self.collect_(collectBlock).select_(selectBlock);}
,"enumerating",unescape("collect%3A%20collectBlock%20thenSelect%3A%20selectBlock%0D%0A%09%22%20Return%20a%20new%20collection%20with%20the%20result%20of%20evaluating%20collectBlock%20on%20the%20contents%20of%20the%20receiver%20wich%20evaluates%20selectBlock%20to%20true.%20%22%0D%0A%0D%0A%09%5E%28self%20collect%3A%20collectBlock%29%20select%3A%20selectBlock"));
smalltalk.bind(smalltalk.Collection,"reversed",0
,function(){var self=this;
return self.subclassResponsibility_("reversed");}
,"copying",unescape("reversed%0D%0A%09%22%20Return%20an%20array%20with%20the%20contents%20of%20the%20receiver%20in%20reverse%20order.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23reversed"));
smalltalk.bind(smalltalk.Collection,"includes:",0
,function(anObject){var self=this;
return self.subclassResponsibility_("includes:");}
,"testing",unescape("includes%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23includes%3A"));
smalltalk.bind(smalltalk.Collection,"notEmpty",0
,function(){var self=this;
return self.isEmpty().not();}
,"testing",unescape("notEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20not%20empty.%20%22%0D%0A%0D%0A%09%5Eself%20isEmpty%20not"));
smalltalk.bind(smalltalk.Collection,"isEmpty",0
,function(){var self=this;
return (nil.is_eqeq_(self.size(),(0)));}
,"testing",unescape("isEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20empty.%20%22%0D%0A%0D%0A%09%5Eself%20size%20%3D%3D%200"));
smalltalk.bind(smalltalk.Collection,"remove:",0
,function(anObject){var self=this;
return self.remove_ifAbsent_(anObject,(function(){return self.objectNotFound_(anObject);}));}
,"removing",unescape("remove%3A%20anObject%0D%0A%09%22%20Remove%20anObject%20from%20receiver%2C%20return%20the%20removed%20object.%20%22%0D%0A%0D%0A%09%5Eself%20remove%3A%20anObject%20ifAbsent%3A%20%5B%20self%20objectNotFound%3A%20anObject%20%5D"));
smalltalk.bind(smalltalk.Collection,"remove:ifAbsent:",0
,function(anObject,aBlock){var self=this;
return self.subclassResponsibility_("remove:ifAbsent:");}
,"removing",unescape("remove%3A%20anObject%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Remove%20anObject%20from%20receiver.%0D%0A%09Return%20the%20removed%20object%20or%20the%20result%20of%20evaluating%20aBlock%20%28if%20the%20object%20is%20not%20found%29.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23remove%3AifAbsent%3A"));
smalltalk.bind(smalltalk.Collection,"removeAll:",0
,function(aCollection){var self=this;
aCollection.do_((function(each){return self.remove_(each);}));
return aCollection;}
,"removing",unescape("removeAll%3A%20aCollection%0D%0A%09%22%20Remove%20aCollection%20from%20receiver.%20%22%0D%0A%0D%0A%09aCollection%20do%3A%20%5B%3Aeach%7C%20self%20remove%3A%20each%20%5D.%0D%0A%09%5EaCollection"));
smalltalk.bind(smalltalk.Collection,"removeAll",0
,function(){var self=this;
self.removeAll_(self.shallowCopy());
return self;}
,"removing",unescape("removeAll%0D%0A%09%22%20Remove%20all%20elements%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20removeAll%3A%20self%20shallowCopy"));
smalltalk.bind(smalltalk.Collection,"occurrencesOf:",0
,function(anObject){var self=this;
return self.inject_into_((0),(function(total,each){return ((each._eq(anObject)).mustBeBoolean()==true ? (function(){return total._plus((1));})() : (function(){return total;})());}));}
,"counting",unescape("occurrencesOf%3A%20anObject%0D%0A%09%22%20Return%20the%20occurrences%20of%20anObject%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20inject%3A%200%20into%3A%20%5B%3Atotal%20%3Aeach%7C%0D%0A%09%09each%20%3D%20anObject%20ifTrue%3A%20%5B%20total%20+%201%20%5D%20ifFalse%3A%20%5Btotal%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Collection,"shallowCopy",0
,function(){var self=this;
return self.collect_((function(each){return each;}));}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20collect%3A%20%5B%3Aeach%7C%20each%20%5D"));
smalltalk.bind(smalltalk.Collection,"deepCopy",0
,function(){var self=this;
return self.collect_((function(each){return each.deepCopy();}));}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20collect%3A%20%5B%3Aeach%7C%20each%20deepCopy%20%5D"));
smalltalk.bind(smalltalk.Collection,"on:",0
,function(target){var self=this;
var array=nil;
array=self.asArray();
return (1).to_by_value_(array.size(),(1),(function(index,block){return block.value_(target.at_ifAbsent_(array.at_(index),(function(){return nil;})));}));}
,"iterating",unescape("on%3A%20target%0D%0A%09%22%20Return%20an%20iterable%20on%20the%20target%20contents%20accessed%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20array%20%7C%0D%0A%09array%20%3A%3D%20self%20asArray.%0D%0A%09%5E1%20to%3A%20array%20size%20by%3A%201%20value%3A%20%5B%3Aindex%20%3Ablock%7C%0D%0A%09%09block%20value%3A%20%28target%20at%3A%20%28array%20at%3A%20index%29%20ifAbsent%3A%5B%5D%29%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Collection,"asSet",0
,function(){var self=this;
return smalltalk.Set.withAll_(self);}
,"converting",unescape("asSet%0D%0A%09%22%20Return%20the%20receiver%20contents%20in%20a%20Set.%20%22%0D%0A%0D%0A%09%5ESet%20withAll%3A%20self"));
smalltalk.bind(smalltalk.Collection,"asHashedSet",0
,function(){var self=this;
return smalltalk.HashedSet.withAll_(self);}
,"converting",unescape("asHashedSet%0D%0A%09%22%20Return%20the%20receiver%20contents%20in%20a%20HashedSet.%20%22%0D%0A%0D%0A%09%5EHashedSet%20withAll%3A%20self"));
smalltalk.bind(smalltalk.Collection,"asParser",0
,function(){var self=this;
return smalltalk.PPChoiceParser.withAll_(self.asArray());}
,"Compiler-converting",unescape("asParser%0D%0A%09%22%20Return%20a%20parser%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EPPChoiceParser%20withAll%3A%20self%20asArray"));
smalltalk.bind(smalltalk.Collection,"fileOut",0
,function(){var self=this;
return smalltalk.Exporter.fileOut_(self);}
,"Tools-fileOut",unescape("fileOut%0D%0A%09%22%20Return%20fileOut%20definition%20and%20code%20of%20classes%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EExporter%20fileOut%3A%20self"));
smalltalk.bind(smalltalk.Collection,"export","$$export"
,function(){var self=this;
return smalltalk.Exporter.export_(self);}
,"Tools-export",unescape("export%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20definition%20and%20source%20code%20of%20classes%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EExporter%20export%3A%20self"));
smalltalk.bind(smalltalk.Collection,"emmit",0
,function(){var self=this;
return self.asArray().compose_with_((function(each){return self.emmit_(each);}),unescape("%0D%0A"));}
,"Builders-emmiting",unescape("emmit%0D%0A%09%22%20Emmit%20%28fileIn%20%26%20save%20generated%20code%20of%29%20the%20fileNames%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20asArray%20compose%3A%20%5B%3Aeach%7C%20self%20emmit%3A%20each%20%5D%20with%3A%20%27%0D%0A%27"));
smalltalk.bind(smalltalk.Collection,"emmitSt",0
,function(){var self=this;
return self.collect_((function(module){return module._comma(".")._comma(smalltalk.String.sourceFileExtension());})).emmit();}
,"Builders-emmiting",unescape("emmitSt%0D%0A%09%22%20Emmit%20the%20code%20of%20source%20modules%20%28st%20files%29%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28self%20collect%3A%20%5B%3Amodule%7C%20module%20%2C%24.%20%2CString%20sourceFileExtension%20%5D%29%20emmit"));
smalltalk.bind(smalltalk.Collection,"handles",0
,function(){var self=this;
return self.asArray().handles();}
,"Natives-converting",unescape("handles%0D%0A%09%22%20Return%20an%20array%20with%20the%20handles%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20asArray%20handles"));
smalltalk.bind(smalltalk.SequenceableCollection,"at:",0
,function(anIndex){var self=this;
return self.at_ifAbsent_(anIndex,(function(){return self.objectNotFoundAt_(anIndex);}));}
,"accessing",unescape("at%3A%20anIndex%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver%20at%20anIndex%20position.%20%22%0D%0A%0D%0A%09%5Eself%20at%3A%20anIndex%20ifAbsent%3A%20%5B%20self%20objectNotFoundAt%3A%20anIndex%20%5D"));
smalltalk.bind(smalltalk.SequenceableCollection,"at:ifAbsent:",0
,function(anIndex,aBlock){var self=this;
return self.subclassResponsibility_("at:ifAbsent:");}
,"accessing",unescape("at%3A%20anIndex%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver%20at%20anIndex%20position%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23at%3AifAbsent%3A"));
smalltalk.bind(smalltalk.SequenceableCollection,"at:put:",0
,function(anIndex,anObject){var self=this;
return self.subclassResponsibility_("at:put:");}
,"accessing",unescape("at%3A%20anIndex%20put%3A%20anObject%0D%0A%09%22%20Set%20the%20object%20in%20the%20receiver%20at%20anIndex%20position.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23at%3Aput%3A"));
smalltalk.bind(smalltalk.SequenceableCollection,"at:ifAbsentPut:",0
,function(aKey,aBlock){var self=this;
return self.at_ifAbsent_(aKey,(function(){return self.at_put_(aKey,aBlock.value());}));}
,"accessing",unescape("at%3A%20aKey%20ifAbsentPut%3A%20aBlock%0D%0A%09%22%20Return%20the%20value%20at%20aKey%2C%20or%20add%20the%20result%20of%20evaluating%20aBlock%20to%20the%20receiver%20if%20it%20is%20absent.%20%22%0D%0A%0D%0A%09%5Eself%20at%3A%20aKey%20ifAbsent%3A%20%5B%20self%20at%3A%20aKey%20put%3A%20aBlock%20value%20%5D"));
smalltalk.bind(smalltalk.SequenceableCollection,"copyFrom:to:",0
,function(anIndex,anotherIndex){var self=this;
return self.subclassResponsibility_("copyFrom:to:");}
,"copying",unescape("copyFrom%3A%20anIndex%20to%3A%20anotherIndex%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23copyFrom%3Ato%3A"));
smalltalk.bind(smalltalk.SequenceableCollection,"indexOf:",0
,function(anObject){var self=this;
return self.indexOf_ifAbsent_(anObject,(function(){return (0);}));}
,"testing",unescape("indexOf%3A%20anObject%0D%0A%09%22%20Return%20the%20position%20of%20anObject%20in%20the%20receiver%2C%20or%200.%20%22%0D%0A%0D%0A%09%5Eself%20indexOf%3A%20anObject%20ifAbsent%3A%20%5B%200%20%5D"));
smalltalk.bind(smalltalk.SequenceableCollection,"indexOf:ifAbsent:",0
,function(anObject,aBlock){var self=this;

	for(var i=0;i<self.length;i++){ if (self[i]._eq(anObject)) { return i + 1; } }
	return aBlock.value();
return self;}
,"testing",unescape("indexOf%3A%20anObject%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20position%20of%20anObject%20in%20the%20receiver%2C%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27detection%27%7D"));
smalltalk.bind(smalltalk.SequenceableCollection,"do:",0
,function(aBlock){var self=this;
return self.implementedBySubclass_("do:");}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23do%3A%20"));
smalltalk.bind(smalltalk.SequenceableCollection,"detect:ifNone:",0
,function(aBlock,anotherBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);

	for(var i = 0; i < self.length; i++) if(fn(self[i])) return self[i];
	return anotherBlock.value();
return self;}
,"enumerating",unescape("detect%3A%20aBlock%20ifNone%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver%20that%20return%20true%20when%20evaluated%20with%20aBlock%20or%20the%20result%20of%20evaluating%20anotherBlock.%20%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27s8%27%09self%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28aBlock%20value%3A%20each%29%20ifTrue%3A%20%5B%20%5Eeach%20%5D%0D%0A%09%5D.%0D%0A%09%5EanotherBlock%20value%27%0D%0A%09doc%27detection%27%7D"));
smalltalk.bind(smalltalk.SequenceableCollection,"reversed",0
,function(){var self=this;

	var result = new Array();
	var len=self.length; var i=len;
	while(i--){result[len-i-1]= self[i];}
	return result;
return self;}
,"copying",unescape("reversed%0D%0A%09%22%20Return%20an%20array%20with%20the%20contents%20of%20the%20receiver%20in%20reverse%20order.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27converting%27%7D"));
smalltalk.bind(smalltalk.SequenceableCollection,"isEmpty",0
,function(){var self=this;
return (this.length)?false:true;}
,"testing",unescape("isEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20empty.%0D%0A%09Note%3A%20%5Eself%20size%20%3D%200%20is%20slower%20than%20inlined%20impl.%0D%0A%09%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27testing%27%7D"));
smalltalk.bind(smalltalk.SequenceableCollection,"reverseDo:",0
,function(aBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
var i=self.length;while(i--){fn(self[i]);};
return self;}
,"enumerating",unescape("reverseDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver%20in%20reverse%20order%20of%20iteration.%20%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27s8%27%09self%20size%20to%3A%201%20by%3A%20-1%20do%3A%20%5B%3Ai%7C%0D%0A%09%09aBlock%20value%3A%20%28self%20at%3A%20i%29%0D%0A%09%5D%27%20doc%27%28reversed%29%20iteration%27%7D"));
smalltalk.bind(smalltalk.SequenceableCollection,"includes:",0
,function(anObject){var self=this;

if (!(self.isObject_(anObject))){return self.indexOf(anObject)._gt((-1));};;

	if (anObject.isNumber() && (self.indexOf(anObject) > -1)) return true;
	if (anObject.isString() && (self.indexOf(anObject) > -1)) return true;
	var i = self.length;
	while (i--) { if (anObject._eq(self[i])) { return true; } }
	return false;
return self;}
,"testing",unescape("includes%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%28self%20isObject%3A%20anObject%29%20ifFalse%3A%20%5B%20%5E%28self%20%23indexOf%3A%20anObject%29%20%3E%20-1%20%5D.%0D%0A%09%7B%27%27%20doc%27native%20detection%27%7D"));
smalltalk.bind(smalltalk.SequenceableCollection,"occurrencesOf:",0
,function(anObject){var self=this;

	var i = self.length; var count = 0;
	while (i--) { if (self[i]._eq(anObject)) { count++; } }
	return count;
return self;}
,"counting",unescape("occurrencesOf%3A%20anObject%0D%0A%09%22%20Return%20the%20occurrences%20of%20anObject%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20counting%27%7D"));
smalltalk.bind(smalltalk.SequenceableCollection,"first",0
,function(){var self=this;
return self.at_((1));}
,"accessing",unescape("first%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20at%3A%201"));
smalltalk.bind(smalltalk.SequenceableCollection,"fourth",0
,function(){var self=this;
return self.at_((4));}
,"accessing",unescape("fourth%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20at%3A%204"));
smalltalk.bind(smalltalk.SequenceableCollection,"last",0
,function(){var self=this;
return self.at_(self.size());}
,"accessing",unescape("last%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20at%3A%20self%20size"));
smalltalk.bind(smalltalk.SequenceableCollection,"second",0
,function(){var self=this;
return self.at_((2));}
,"accessing",unescape("second%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20at%3A%202"));
smalltalk.bind(smalltalk.SequenceableCollection,"third",0
,function(){var self=this;
return self.at_((3));}
,"accessing",unescape("third%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20at%3A%203"));
smalltalk.bind(smalltalk.SequenceableCollection,"removeLast",0
,function(){var self=this;
return self.remove_(self.last());}
,"removing",unescape("removeLast%0D%0A%09%22%20Remove%20the%20last%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20remove%3A%20self%20last"));
smalltalk.bind(smalltalk.SequenceableCollection,"removeFirst",0
,function(){var self=this;
return self.remove_(self.first());}
,"removing",unescape("removeFirst%0D%0A%09%22%20Remove%20the%20first%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20remove%3A%20self%20first"));
smalltalk.bind(smalltalk.SequenceableCollection,"addLast:",0
,function(anObject){var self=this;
return self.add_(anObject);}
,"adding",unescape("addLast%3A%20anObject%0D%0A%09%22%20Add%20anObject%20to%20the%20receiver%20%28at%20end%29.%20%22%0D%0A%0D%0A%09%5Eself%20add%3A%20anObject"));
smalltalk.bind(smalltalk.SequenceableCollection,"atAll:put:",0
,function(indices,anObject){var self=this;
indices.do_((function(index){return self.at_put_(index,anObject);}));
return self;}
,"accessing",unescape("atAll%3A%20indices%20put%3A%20anObject%0D%0A%09%22%20Return%20the%20receiver%20after%20replacing%20those%20elements%2C%20indexed%20by%20the%20indices%2C%20with%20anObject.%20%22%0D%0A%0D%0A%09indices%20do%3A%20%5B%20%3Aindex%20%7C%20self%20at%3A%20index%20put%3A%20anObject%20%5D"));
smalltalk.bind(smalltalk.SequenceableCollection,"atAllPut:",0
,function(anObject){var self=this;
(1).to_(self.size()).do_((function(index){return self.at_put_(index,anObject);}));
return self;}
,"accessing",unescape("atAllPut%3A%20anObject%0D%0A%09%22%20Replace%20all%20elements%20of%20the%20receiver%20with%20anObject.%20%22%0D%0A%0D%0A%09%281%20to%3A%20self%20size%29%20do%3A%20%5B%20%3Aindex%20%7C%20self%20at%3A%20index%20put%3A%20anObject%20%5D"));
smalltalk.bind(smalltalk.SequenceableCollection,"withIndexDo:",0
,function(aBlock){var self=this;
(1).to_(self.size()).do_((function(index){return aBlock.value_value_(self.at_(index),index);}));
return self;}
,"enumerating",unescape("withIndexDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20all%20the%20index%20positions%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%281%20to%3A%20self%20size%29%20do%3A%20%5B%20%3Aindex%20%7C%0D%0A%09%09aBlock%20value%3A%20%28self%20at%3A%20index%29%20value%3A%20index%0D%0A%09%5D"));
smalltalk.bind(smalltalk.SequenceableCollection,"=","_eq"
,function(aCollection){var self=this;
var index=nil;
if ((nil.is_eqeq_(self,aCollection))){return true;};;
if (!((nil.is_eqeq_(self.$klass,aCollection.$klass)))){return false;};;
index=self.size();
if (index._no_eq(aCollection.size())){return false;};;
while(!(function(){return index._lt_eq((0));})()){if (!(self.at_(index)._eq(aCollection.at_(index)))){return false;};;
index=index._minus((1));};;
return true;}
,"testing",unescape("%3D%20aCollection%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09self%20%3D%3D%20aCollection%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09self%20class%20%3D%3D%20aCollection%20class%20ifFalse%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09index%20%3A%3D%20self%20size.%0D%0A%09index%20%7E%3D%20aCollection%20size%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%5B%20index%20%3C%3D%200%20%5D%20whileFalse%3A%20%5B%0D%0A%09%09%28self%20at%3A%20index%29%20%3D%20%28aCollection%20at%3A%20index%29%20ifFalse%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%09index%20%3A%3D%20index%20-%201%0D%0A%09%5D.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.SequenceableCollection,"beginsWith:",0
,function(aCollection){var self=this;
if (self.size()._lt(aCollection.size())){return false;};;
if (self.size()._eq(aCollection.size())){return aCollection._eq(self);};;
return aCollection._eq(self.copyFrom_to_((1),aCollection.size()));}
,"testing",unescape("beginsWith%3A%20aCollection%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20begins%20with%20aCollection.%20%22%0D%0A%0D%0A%09self%20size%20%3C%20aCollection%20size%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09self%20size%20%3D%20aCollection%20size%20ifTrue%3A%20%5B%20%5EaCollection%20%3D%20self%20%5D.%0D%0A%09%5EaCollection%20%3D%20%28self%20copyFrom%3A%201%20to%3A%20aCollection%20size%29"));
smalltalk.bind(smalltalk.SequenceableCollection,"endsWith:",0
,function(aCollection){var self=this;
if (self.size()._lt(aCollection.size())){return false;};;
if (self.size()._eq(aCollection.size())){return aCollection._eq(self);};;
return aCollection._eq(self.copyFrom_to_(self.size()._minus(aCollection.size())._plus((1)),self.size()));}
,"testing",unescape("endsWith%3A%20aCollection%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20ends%20with%20aCollection.%20%22%0D%0A%0D%0A%09self%20size%20%3C%20aCollection%20size%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09self%20size%20%3D%20aCollection%20size%20ifTrue%3A%20%5B%20%5EaCollection%20%3D%20self%20%5D.%0D%0A%09%5EaCollection%20%3D%20%28self%20copyFrom%3A%20self%20size%20-%20aCollection%20size%20+%201%20to%3A%20self%20size%29"));
smalltalk.bind(smalltalk.SequenceableCollection,"copyReplaceFrom:to:with:",0
,function(start,stop,aCollection){var self=this;
return self.copyFrom_to_((1),start._minus((1)))._comma(aCollection)._comma(self.copyFrom_to_(stop._plus((1)),self.size()));}
,"copying",unescape("copyReplaceFrom%3A%20start%20to%3A%20stop%20with%3A%20aCollection%0D%0A%09%22%20Return%20a%20collection%20containing%20a%20copy%20of%20the%20receiver%20with%20the%20elements%20at%20index%20positions%20from%20start%20through%20stop%20replaced%20with%20the%20elements%20of%20aCollection.%20%22%0D%0A%0D%0A%09%5E%28self%20copyFrom%3A%201%20to%3A%20start%20-%201%29%0D%0A%09%09%2C%20aCollection%0D%0A%09%09%2C%20%28self%20copyFrom%3A%20stop%20+%201%20to%3A%20self%20size%29"));
smalltalk.bind(smalltalk.SequenceableCollection,"copyWithout:",0
,function(anObject){var $s8Ret$={name:"stReturn"};try{var self=this;
var index=nil;
index=self.indexOf_ifAbsent_(anObject,(function(){return (function(){$s8Ret$.result=(function(){return self.copy()})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return self.copyFrom_to_((1),index._minus((1)))._comma(self.copyFrom_to_(index._plus((1)),self.size()).copyWithout_(anObject))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"copying",unescape("copyWithout%3A%20anObject%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20excluding%20anObject%2C%20if%20any.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09index%20%3A%3D%20self%20indexOf%3A%20anObject%20ifAbsent%3A%20%5B%20%5Eself%20copy%20%5D.%0D%0A%09%5E%28self%20copyFrom%3A%201%20to%3A%20index%20-%201%29%0D%0A%09%09%2C%28%28self%20copyFrom%3A%20index%20+%201%20to%3A%20self%20size%29%0D%0A%09%09%09copyWithout%3A%20anObject%29"));
smalltalk.bind(smalltalk.SequenceableCollection,"copyWithAll:",0
,function(aCollection){var self=this;
return self._comma(aCollection);}
,"copying",unescape("copyWithAll%3A%20aCollection%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20with%20all%20contents%20of%20aCollection.%20%22%0D%0A%0D%0A%09%5Eself%20%2C%20aCollection%20"));
smalltalk.bind(smalltalk.SequenceableCollection,"findFirst:",0
,function(aBlock){var self=this;
return self.findFirst_ifAbsent_(aBlock,(function(){return self.errorAbsentObject();}));}
,"searching",unescape("findFirst%3A%20aBlock%0D%0A%09%22%20Return%20the%20index%20of%20the%20first%20element%20of%20the%20receiver%20that%20causes%20aBlock%20to%20evaluate%20to%20true.%0D%0A%20%20%20%20%20%20%20%20%20If%20no%20such%20element%20is%20found%2C%20report%20an%20error.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20findFirst%3A%20aBlock%20ifAbsent%3A%20%5B%20self%20errorAbsentObject%20%5D"));
smalltalk.bind(smalltalk.SequenceableCollection,"findFirst:ifAbsent:",0
,function(aBlock,absentBlock){var self=this;
var index=nil;
var size=nil;
size=self.size();
index=(1);
while((function(){return index._lt_eq(size);})()){if (aBlock.value_(self.at_(index))){return index;};;
index=index._plus((1));};;
return absentBlock.value();}
,"searching",unescape("findFirst%3A%20aBlock%20ifAbsent%3A%20absentBlock%0D%0A%09%22%20Return%20the%20index%20of%20the%20first%20element%20of%20the%20receiver%20that%20causes%20aBlock%20to%20evaluate%20to%20true.%0D%0A%09If%20no%20such%20element%20is%20found%2C%20return%20the%20result%20of%20evaluating%20the%20argument%20block.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20index%20size%20%7C%0D%0A%09size%20%3A%3D%20self%20size.%0D%0A%09index%20%3A%3D%201.%0D%0A%09%5B%20index%20%3C%3D%20size%20%5D%20whileTrue%3A%20%5B%0D%0A%09%09%28aBlock%20value%3A%20%28self%20at%3A%20index%29%29%20ifTrue%3A%20%5B%20%5Eindex%20%5D.%0D%0A%09%09index%20%3A%3D%20index%20+%201%0D%0A%09%5D.%0D%0A%09%5EabsentBlock%20value"));
smalltalk.bind(smalltalk.SequenceableCollection,"findLast:",0
,function(aBlock){var self=this;
return self.findLast_ifAbsent_(aBlock,(function(){return self.errorAbsentObject();}));}
,"searching",unescape("findLast%3A%20aBlock%0D%0A%09%22%20Return%20the%20index%20of%20the%20last%20element%20of%20the%20receiver%20that%20causes%20aBlock%20to%20evaluate%20to%20true.%0D%0A%09If%20no%20such%20element%20is%20found%2C%20report%20an%20error.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20findLast%3A%20aBlock%20ifAbsent%3A%20%5B%20self%20errorAbsentObject%20%5D"));
smalltalk.bind(smalltalk.SequenceableCollection,"findLast:ifAbsent:",0
,function(aBlock,absentBlock){var self=this;
var index=nil;
index=self.size();
while((function(){return index._gt((0));})()){if (aBlock.value_(self.at_(index))){return index;};;
index=index._minus((1));};;
return absentBlock.value();}
,"searching",unescape("findLast%3A%20aBlock%20ifAbsent%3A%20absentBlock%0D%0A%09%22%20Return%20the%20index%20of%20the%20last%20element%20of%20the%20receiver%20that%20causes%20aBlock%20to%20evaluate%20to%20true.%0D%0A%09If%20no%20such%20element%20is%20found%2C%20return%20the%20result%20of%20evaluating%20the%20argument%20block.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09index%20%3A%3D%20self%20size.%0D%0A%09%5B%20index%20%3E%200%20%5D%20whileTrue%3A%20%5B%0D%0A%09%09%28aBlock%20value%3A%20%28self%20at%3A%20index%29%29%20ifTrue%3A%20%5B%20%5Eindex%20%5D.%0D%0A%09%09index%20%3A%3D%20index%20-%201%0D%0A%09%5D.%0D%0A%09%5EabsentBlock%20value"));
smalltalk.bind(smalltalk.SequenceableCollection,"replaceFrom:to:with:",0
,function(start,stop,aCollection){var self=this;
if (!(stop._minus(start)._plus((1))._eq(aCollection.size()))){return self.error_("Replacement has wrong size");};;
self.replaceFrom_to_with_startingAt_(start,stop,aCollection,(1));
return self;}
,"replace",unescape("replaceFrom%3A%20start%20to%3A%20stop%20with%3A%20aCollection%0D%0A%09%22%20Return%20the%20receiver.%0D%0A%09Replace%20the%20elements%20of%20the%20receiver%20at%20index%20positions%20start%20through%20stop%2C%20with%20the%20elements%20of%20aCollection.%20%22%0D%0A%0D%0A%09stop%20-%20start%20+%201%20%3D%20aCollection%20size%20ifFalse%3A%20%5B%0D%0A%09%09%5Eself%20error%3A%20%27Replacement%20has%20wrong%20size%27%0D%0A%09%5D.%0D%0A%09self%0D%0A%09%09replaceFrom%3A%20start%0D%0A%09%09to%3A%20stop%0D%0A%09%09with%3A%20aCollection%0D%0A%09%09startingAt%3A%201"));
smalltalk.bind(smalltalk.SequenceableCollection,"replaceFrom:to:with:startingAt:",0
,function(start,stop,aCollection,repStart){var self=this;
var index1=nil;
var index2=nil;
if ((nil.is_eqeq_(self,aCollection)).and_((function(){return repStart._lt(start);}))){index2=repStart._plus(stop)._minus(start);
index1=stop;
while((function(){return start._lt_eq(index1);})()){self.at_put_(index1,aCollection.at_(index2));
index1=index1._minus((1));
index2=index2._minus((1));};;
return self;};;
index2=repStart;
index1=start;
while((function(){return index1._lt_eq(stop);})()){self.at_put_(index1,aCollection.at_(index2));
index1=index1._plus((1));
index2=index2._plus((1));};;
return self;}
,"replace",unescape("replaceFrom%3A%20start%0D%0A%20%20%20%20to%3A%20stop%0D%0A%20%20%20%20with%3A%20aCollection%0D%0A%20%20%20%20startingAt%3A%20repStart%0D%0A%09%22%20Return%20the%20receiver.%0D%0A%09Replace%20the%20elements%20of%20the%20receiver%20at%20index%20positions%20start%20through%20stop%20with%20consecutive%0D%0A%20%20%20%20%20%20%20%20%20elements%20of%20aCollection%20beginning%20at%20index%20position%20repStart.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20index1%20index2%20%7C%0D%0A%09%28self%20%3D%3D%20aCollection%20and%3A%20%5B%20repStart%20%3C%20start%20%5D%29%20ifTrue%3A%20%5B%20%22do%20backward%20move%20for%20same%20object%22%0D%0A%09%09index2%20%3A%3D%20repStart%20+%20stop%20-%20start.%0D%0A%09%09index1%20%3A%3D%20stop.%0D%0A%09%09%5B%20start%20%3C%3D%20index1%20%5D%20whileTrue%3A%20%5B%0D%0A%09%09%09self%20at%3A%20index1%20put%3A%20%28aCollection%20at%3A%20index2%29.%0D%0A%09%09%09index1%20%3A%3D%20index1%20-%201.%0D%0A%09%09%09index2%20%3A%3D%20index2%20-%201%0D%0A%09%09%5D.%0D%0A%09%09%5Eself%0D%0A%09%5D.%0D%0A%09index2%20%3A%3D%20repStart.%0D%0A%09index1%20%3A%3D%20start.%0D%0A%09%5B%20index1%20%3C%3D%20stop%20%5D%20whileTrue%3A%20%5B%0D%0A%09%09self%20at%3A%20index1%20put%3A%20%28aCollection%20at%3A%20index2%29.%0D%0A%09%09index1%20%3A%3D%20index1%20+%201.%0D%0A%09%09index2%20%3A%3D%20index2%20+%201%0D%0A%09%5D"));
smalltalk.bind(smalltalk.SequenceableCollection,"replaceFrom:to:with:startingWith:",0
,function(start,stop,replacement,replacementStart){var self=this;
return self.replaceFrom_to_with_startingAt_(start,stop,replacement,replacementStart);}
,"replace",unescape("replaceFrom%3A%20start%0D%0A%20%20%20%20to%3A%20stop%0D%0A%20%20%20%20with%3A%20replacement%0D%0A%20%20%20%20startingWith%3A%20replacementStart%0D%0A%09%22%20Replace%20the%20elements%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09replaceFrom%3A%20start%0D%0A%09%09to%3A%20stop%0D%0A%09%09with%3A%20replacement%0D%0A%09%09startingAt%3A%20replacementStart"));
smalltalk.bind(smalltalk.SequenceableCollection,"replaceFrom:to:withObject:",0
,function(start,stop,anObject){var self=this;
var index=nil;
index=start;
while((function(){return index._lt_eq(stop);})()){self.at_put_(index,anObject);
index=index._plus((1));};;
return anObject;}
,"replace",unescape("replaceFrom%3A%20start%20to%3A%20stop%20withObject%3A%20anObject%0D%0A%09%22Replace%20each%20of%20the%20elements%20of%20the%20receiver%20at%20index%20positions%20start%20through%20stop%20with%20anObject.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09index%20%3A%3D%20start.%0D%0A%09%5B%20index%20%3C%3D%20stop%20%5D%20whileTrue%3A%20%5B%0D%0A%09%09self%20at%3A%20index%20put%3A%20anObject.%0D%0A%09%09index%20%3A%3D%20index%20+%201%0D%0A%09%5D.%0D%0A%09%5EanObject"));
smalltalk.bind(smalltalk.SequenceableCollection,"with:do:",0
,function(aCollection,aBlock){var self=this;
var first=nil;
var second=nil;
first=self.stream();
second=aCollection.stream();
while(!(function(){return first.atEnd().or_((function(){return second.atEnd();}));})()){aBlock.value_value_(first.next(),second.next());};;
return self;}
,"iterating",unescape("with%3A%20aCollection%20do%3A%20aBlock%0D%0A%09%22%20For%20each%20pair%20of%20elements%20evaluate%20aBlock%20with%20those%20elements%20as%20the%20arguments.%20%22%0D%0A%0D%0A%09%7C%20first%20second%20%7C%0D%0A%09first%09%3A%3D%20self%20stream.%0D%0A%09second%09%3A%3D%20aCollection%20stream.%0D%0A%09%5B%20first%20atEnd%20or%3A%20%5B%20second%20atEnd%20%5D%20%5D%20whileFalse%3A%20%5B%0D%0A%09%09aBlock%20value%3A%20first%20next%20value%3A%20second%20next%0D%0A%09%5D."));
smalltalk.bind(smalltalk.SequenceableCollection,"sort",0
,function(){var self=this;
self.sort_(nil);
return self;}
,"sorting",unescape("sort%0D%0A%09%22%20Sort%20%28inplace%29%20the%20receiver%27s%20contents%20with%20default%20criteria.%20%22%0D%0A%0D%0A%09self%20sort%3A%20nil"));
smalltalk.bind(smalltalk.SequenceableCollection,"sort:",0
,function(aBlock){var self=this;
self.implementedBySubclass_("sort:");
return self;}
,"sorting",unescape("sort%3A%20aBlock%0D%0A%09%22%20Sort%20%28inplace%29%20the%20receiver%27s%20contents%20with%20aBlock.%20%22%0D%0A%0D%0A%09self%20implementedBySubclass%3A%20%23sort%3A"));
smalltalk.bind(smalltalk.SequenceableCollection,"from:to:by:",0
,function(start,end,increment){var self=this;
return start.to_by_value_(end,increment,(function(index,block){return block.value_(self.at_(index));}));}
,"iterating",unescape("from%3A%20start%20to%3A%20end%20by%3A%20increment%0D%0A%09%22%20Return%20an%20iterable%20on%20the%20receiver%27s%20contents%20from%20start%20to%20end.%20%22%0D%0A%0D%0A%09%5Estart%20to%3A%20end%20by%3A%20increment%20value%3A%20%5B%3Aindex%20%3Ablock%7C%0D%0A%09%09block%20value%3A%20%28self%20at%3A%20index%29%0D%0A%09%5D"));
smalltalk.bind(smalltalk.SequenceableCollection,"from:to:",0
,function(start,end){var self=this;
return self.from_to_by_(start,end,((start._lt_eq(end)).mustBeBoolean()==true ? (function(){return (1);})() : (function(){return (-1);})()));}
,"iterating",unescape("from%3A%20start%20to%3A%20end%0D%0A%09%22%20Return%20an%20iterable%20on%20the%20receiver%27s%20contents%20from%20start%20to%20end.%20%22%0D%0A%0D%0A%09%5Eself%09from%3A%20start%20to%3A%20end%0D%0A%09%09by%3A%20%28start%20%3C%3D%20end%20ifTrue%3A%20%5B1%5D%20ifFalse%3A%20%5B-1%5D%29%20"));
smalltalk.bind(smalltalk.SequenceableCollection,"asActionSequence",0
,function(){var self=this;
return smalltalk.ActionSequence.withAll_(self);}
,"converting",unescape("asActionSequence%0D%0A%09%22%20Return%20an%20ActionSequence%20from%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EActionSequence%20withAll%3A%20self"));
smalltalk.bind(smalltalk.String.$klass,"fromString:",0
,function(aString){var self=this;
return new self.$fn(aString);
return self;}
,"instantiation",unescape("fromString%3A%20aString%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27instatiation%27%7D"));
smalltalk.bind(smalltalk.String.$klass,"new","$$new"
,function(){var self=this;
return "";}
,"instantiation",unescape("new%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27instantiation%27%7D"));
smalltalk.bind(smalltalk.String.$klass,"cr",0
,function(){var self=this;
return "\r";}
,"constants",unescape("cr%0D%0A%09%22%20Return%20the%20singular%20instance.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20constant%27%7D"));
smalltalk.bind(smalltalk.String.$klass,"lf",0
,function(){var self=this;
return "\n";}
,"constants",unescape("lf%0D%0A%09%22%20Return%20the%20singular%20instance.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20constant%27%7D"));
smalltalk.bind(smalltalk.String.$klass,"crlf",0
,function(){var self=this;
return "\r\n";}
,"constants",unescape("crlf%0D%0A%09%22%20Return%20the%20singular%20instance.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20constant%27%7D"));
smalltalk.bind(smalltalk.String.$klass,"space",0
,function(){var self=this;
return " ";}
,"constants",unescape("space%0D%0A%09%22%20Return%20the%20singular%20instance.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20constant%27%7D"));
smalltalk.bind(smalltalk.String.$klass,"tab",0
,function(){var self=this;
return "\t";}
,"constants",unescape("tab%0D%0A%09%22%20Return%20the%20singular%20instance.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20constant%27%7D"));
smalltalk.bind(smalltalk.String.$klass,"zero",0
,function(){var self=this;
return "\0";}
,"constants",unescape("zero%0D%0A%09%22%20Return%20the%20singular%20instance.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20constant%27%7D"));
smalltalk.bind(smalltalk.String.$klass,"with:",0
,function(aCharacter){var self=this;
return aCharacter;}
,"instantiation",unescape("with%3A%20aCharacter%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EaCharacter"));
smalltalk.bind(smalltalk.String.$klass,"with:with:",0
,function(aCharacter,bCharacter){var self=this;
return aCharacter._comma(bCharacter);}
,"instantiation",unescape("with%3A%20aCharacter%20with%3A%20bCharacter%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EaCharacter%20%2C%20bCharacter"));
smalltalk.bind(smalltalk.String.$klass,"with:with:with:",0
,function(aCharacter,bCharacter,cCharacter){var self=this;
return aCharacter._comma(bCharacter)._comma(cCharacter);}
,"instantiation",unescape("with%3A%20aCharacter%20with%3A%20bCharacter%20with%3A%20cCharacter%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EaCharacter%20%2C%20bCharacter%20%2C%20cCharacter"));
smalltalk.bind(smalltalk.String.$klass,"with:with:with:with:",0
,function(aCharacter,bCharacter,cCharacter,dCharacter){var self=this;
return aCharacter._comma(bCharacter)._comma(cCharacter)._comma(dCharacter);}
,"instantiation",unescape("with%3A%20aCharacter%20with%3A%20bCharacter%0D%0A%09with%3A%20cCharacter%20with%3A%20dCharacter%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EaCharacter%20%2C%20bCharacter%20%2C%20cCharacter%20%2C%20dCharacter"));
smalltalk.bind(smalltalk.String.$klass,"fromCharCode:",0
,function(anArrayOrNumber){var self=this;

	if (anArrayOrNumber.isNumber()) return String.fromCharCode(anArrayOrNumber);
	return String.fromCharCode.apply(null,anArrayOrNumber.asArray());;
return self;}
,"instantiation",unescape("fromCharCode%3A%20anArrayOrNumber%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20conversion%27%7D"));
smalltalk.bind(smalltalk.String.$klass,"doItSelector",0
,function(){var self=this;
return "s8DoIt";}
,"constants",unescape("doItSelector%0D%0A%09%22%20Return%20the%20selector%20to%20use%20for%20doIt%20evaluations.%0D%0A%09Warning%3A%20this%20selector%20will%20not%20be%20dump%20on%20image%20nor%20change%20log%20expressions.%0D%0A%09%22%0D%0A%0D%0A%09%5E%23s8DoIt"));
smalltalk.bind(smalltalk.String.$klass,"nativeFileExtension",0
,function(){var self=this;
return self.nativeLanguage().toString();}
,"fileNames",unescape("nativeFileExtension%0D%0A%09%22%20Return%20the%20file%20extension%20to%20use%20for%20files%20with%20native%20code.%20%22%0D%0A%0D%0A%09%5Eself%20nativeLanguage%20toString"));
smalltalk.bind(smalltalk.String.$klass,"sourceFileExtension",0
,function(){var self=this;
return "st";}
,"fileNames",unescape("sourceFileExtension%0D%0A%09%22%20Return%20the%20file%20extension%20to%20use%20for%20source%20files.%20%22%0D%0A%0D%0A%09%5E%23st%20"));
smalltalk.bind(smalltalk.String.$klass,"nativeLibraryExtension",0
,function(){var self=this;
return self.sourceFileExtension()._comma(".")._comma(self.nativeFileExtension());}
,"fileNames",unescape("nativeLibraryExtension%0D%0A%09%22%20Return%20the%20file%20extension%20to%20use%20for%20native%20library%20files.%20%22%0D%0A%0D%0A%09%5Eself%20sourceFileExtension%20%2C%24.%20%2Cself%20nativeFileExtension%20"));
smalltalk.bind(smalltalk.String.$klass,"binarySelectorChars",0
,function(){var self=this;
return unescape("%7C%26+/%3D%3E%3C%2C@%7E%21*-%25%5C%3F%5C%5C");}
,"constants",unescape("binarySelectorChars%0D%0A%09%22%20Private%20-%20Return%20the%20characters%20that%20can%20be%20included%20in%20binary%20selectors.%20%22%0D%0A%0D%0A%09%5E%27%7C%26+/%3D%3E%3C%2C@%7E%21*-%25%5C%3F%5C%5C%27%20"));
smalltalk.bind(smalltalk.String.$klass,"isBinarySelector:",0
,function(aString){var $s8Ret$={name:"stReturn"};try{var self=this;
var chars=nil;
chars=self.binarySelectorChars();
aString.do_((function(c){return ((chars.includes_(c)).mustBeBoolean()==true ? nil : (function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();})());}));
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"private",unescape("isBinarySelector%3A%20aString%0D%0A%09%22%20Private%20-%20Return%20true%20if%20aString%20is%20a%20binary%20selector.%20%22%0D%0A%0D%0A%09%7C%20chars%20%7C%0D%0A%09chars%20%3A%3D%20self%20binarySelectorChars.%0D%0A%09aString%20do%3A%20%5B%3Ac%7C%20%28chars%20includes%3A%20c%29%20ifFalse%3A%20%5B%20%5Efalse%20%5D%20%5D.%0D%0A%09%5Etrue%20"));
smalltalk.bind(smalltalk.String.$klass,"isKeywordSelector:",0
,function(aString){var self=this;

return (nil.isNil_(smalltalk.RegularExpression.fromString_(unescape("%5B%5Ea-zA-Z0-9_%3A%5D")).exec_(aString)));}
,"private",unescape("isKeywordSelector%3A%20aString%0D%0A%09%22%20Private%20-%20Return%20true%20if%20aString%20is%20a%20keyword%20selector.%20%22%0D%0A%0D%0A%09%7B%27%27%20js%27self%27%20doc%27native%20testing%27%7D.%0D%0A%09%5E%28%28RegularExpression%20fromString%3A%20%27%5B%5Ea-zA-Z0-9_%3A%5D%27%29%20exec%3A%20aString%29%20isNil%20"));
smalltalk.bind(smalltalk.String.$klass,"isMessageSelector:",0
,function(aString){var self=this;
return self.isKeywordSelector_(aString).or_((function(){return self.isBinarySelector_(aString);}));}
,"private",unescape("isMessageSelector%3A%20aString%0D%0A%09%22%20Private%20-%20Return%20true%20if%20aString%20is%20a%20message%20selector.%20%22%0D%0A%0D%0A%09%5E%28self%20isKeywordSelector%3A%20aString%29%0D%0A%09or%3A%20%5B%20self%20isBinarySelector%3A%20aString%20%5D%20"));
smalltalk.bind(smalltalk.String.$klass,"reservedArray",0
,function(){var self=this;
return unescape("abstract%20as%20boolean%20break%20byte%20case%20catch%20char%20class%0D%0A%09continue%20const%20debugger%20default%20delete%20do%20double%20else%0D%0A%09enum%20export%20extends%20false%20final%20finally%20float%20for%20function%0D%0A%09goto%20if%20implements%20import%20in%20instanceof%20int%20interface%20is%0D%0A%09long%20namespace%20native%20new%20null%20package%20private%0D%0A%09protected%20public%20return%20short%20static%20super%20switch%0D%0A%09synchronized%20this%20throw%20throws%20transient%20true%20try%20typeof%0D%0A%09use%20var%20void%20volatile%20while%20with").asArrayOfSubstrings();}
,"js",unescape("reservedArray%0D%0A%09%22%20Private%20-%20Return%20the%20reserved%20names%20of%20native%20language.%20%22%0D%0A%0D%0A%09%5E%27abstract%20as%20boolean%20break%20byte%20case%20catch%20char%20class%0D%0A%09continue%20const%20debugger%20default%20delete%20do%20double%20else%0D%0A%09enum%20export%20extends%20false%20final%20finally%20float%20for%20function%0D%0A%09goto%20if%20implements%20import%20in%20instanceof%20int%20interface%20is%0D%0A%09long%20namespace%20native%20new%20null%20package%20private%0D%0A%09protected%20public%20return%20short%20static%20super%20switch%0D%0A%09synchronized%20this%20throw%20throws%20transient%20true%20try%20typeof%0D%0A%09use%20var%20void%20volatile%20while%20with%27%20asArrayOfSubstrings"));
smalltalk.bind(smalltalk.String.$klass,"reservedWords",0
,function(){var self=this;
return self.basicAt_ifAbsentPut_("$reservedSet",(function(){return smalltalk.HashedSet.withAll_(self.reservedArray());}));}
,"js",unescape("reservedWords%0D%0A%09%22%20Private%20-%20Return%20the%20reserved%20names%20of%20native%20language.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27%24reservedSet%27%20ifAbsentPut%3A%20%5B%0D%0A%09%09HashedSet%20withAll%3A%20self%20reservedArray%0D%0A%09%5D"));
smalltalk.bind(smalltalk.String.$klass,"streamClass",0
,function(){var self=this;
return smalltalk.StringStream;}
,"streams",unescape("streamClass%0D%0A%09%22%20Return%20the%20streamming%20support%20for%20the%20receiver%27s%20instances.%20%22%0D%0A%0D%0A%09%5EStringStream"));
smalltalk.bind(smalltalk.String.$klass,"@","_at"
,function(aHandle){var self=this;
return (((nil.isNil_(aHandle)===false)).mustBeBoolean()==true ? (function(){return self.toString_(aHandle);})() : nil);}
,"Natives-converting",unescape("@%20aHandle%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20from%20aHandle.%20%22%0D%0A%0D%0A%09%5EaHandle%20notNil%20ifTrue%3A%20%5B%20self%20toString%3A%20aHandle%20%5D"));
smalltalk.bind(smalltalk.String.$klass,"reservedLuaWords",0
,function(){var self=this;
return ["and", "break", "do", "else", "elseif", "end", "false", "for", "function", "if", "in", "local", "nil", "not", "or", "repeat", "return", "then", "true", "until", "while"];}
,"Lua-lua",unescape("reservedLuaWords%0D%0A%09%22%20Private%20-%20Return%20the%20reserved%20names%20of%20Lua%20language.%20%22%0D%0A%0D%0A%09%5E%23%28%0D%0A%09and%09break%09do%09else%09elseif%0D%0A%09end%09%23false%09for%09function%09if%0D%0A%09in%09local%09%23nil%09not%09or%0D%0A%09repeat%09return%09then%09%23true%09until%0D%0A%09while%0D%0A%09%29"));
smalltalk.bind(smalltalk.String.$klass,"forbiddenLuaSelectors",0
,function(){var self=this;
return [];}
,"Lua-lua",unescape("forbiddenLuaSelectors%0D%0A%09%22%20Private%20-%20Return%20the%20platform%20forbidden%20selectors.%20%22%0D%0A%0D%0A%09%5E%23%28%20%22class%20new%22%20%29"));
smalltalk.bind(smalltalk.String,"=","_eq"
,function(anObject){var self=this;

	if (nil.isNil_(anObject)) return false;
	if (!(anObject.isString)) return false;
	if (!(anObject.isString())) return false;
	if (self == anObject) return true;
	return ((""+self) == (""+anObject));
return self;}
,"comparing",unescape("%3D%20anObject%0D%0A%20%20%20%20%22%20Return%20true%20if%20the%20receiver%20is%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20testing%27%7D"));
smalltalk.bind(smalltalk.String,"size",0
,function(){var self=this;
return self.length;}
,"accessing",unescape("size%0D%0A%09%22%20Return%20the%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20string%20size%27%7D"));
smalltalk.bind(smalltalk.String,"at:",0
,function(anIndex){var self=this;
return self.charAt(anIndex - 1);}
,"accessing",unescape("at%3A%20anIndex%0D%0A%09%22%20Return%20the%20receiver%20at%20anIndex.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20indexing%27%7D"));
smalltalk.bind(smalltalk.String,"at:put:",0
,function(anIndex,anObject){var self=this;
return self.errorReadOnly_("at:put:");}
,"accessing",unescape("at%3A%20anIndex%20put%3A%20anObject%0D%0A%09%22%20Set%20the%20receiver%20at%20anIndex.%20%22%0D%0A%0D%0A%09%5Eself%20errorReadOnly%3A%20%23at%3Aput%3A"));
smalltalk.bind(smalltalk.String,"at:ifAbsent:",0
,function(anIndex,aBlock){var self=this;
return self.at_(anIndex).ifNil_(aBlock);}
,"accessing",unescape("at%3A%20anIndex%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20receiver%20at%20anIndex%20position%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5E%28self%20at%3A%20anIndex%29%20ifNil%3A%20aBlock"));
smalltalk.bind(smalltalk.String,"do:",0
,function(aBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
for(var i=0;i<self.length;i++){fn(self.charAt(i));};
return self;}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27s8%27%091%20to%3A%20self%20size%20do%3A%20%5B%3Ai%7C%0D%0A%09%09aBlock%20value%3A%20%28self%20at%3A%20i%29%0D%0A%09%5D%27%20doc%27native%20iteration%27%7D"));
smalltalk.bind(smalltalk.String,"detect:ifNone:",0
,function(aBlock,anotherBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
for(var i = 0; i < self.length; i++)
		if(fn(self.charAt(i))) return self.charAt(i);
	return anotherBlock.value();
return self;}
,"enumerating",unescape("detect%3A%20aBlock%20ifNone%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver%20that%20return%20true%20when%20evaluated%20with%20aBlock%20or%20the%20result%20of%20evaluating%20anotherBlock.%20%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27s8%27%09self%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28aBlock%20value%3A%20each%29%20ifTrue%3A%20%5B%20%5Eeach%20%5D%0D%0A%09%5D.%0D%0A%09%5EanotherBlock%20value%27%20doc%27native%20detection%27%7D"));
smalltalk.bind(smalltalk.String,"includes:",0
,function(anObject){var self=this;

return self.indexOf_(anObject)._gt((0));}
,"testing",unescape("includes%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5E%28self%20indexOf%3A%20anObject%29%20%3E%200"));
smalltalk.bind(smalltalk.String,"withIndexDo:",0
,function(aBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
for(var i=0;i<self.length;i++){fn(self.charAt(i), i+1);};
return self;}
,"enumerating",unescape("withIndexDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20all%20the%20index%20positions%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27s8%27%091%20to%3A%20self%20size%20do%3A%20%5B%3Ai%7C%0D%0A%09%09aBlock%20value%3A%20%28self%20at%3A%20i%29%20value%3A%20i%0D%0A%09%5D%27%20doc%27native%20iteration%27%7D"));
smalltalk.bind(smalltalk.String,"copyWithout:",0
,function(anObject){var self=this;
return self.replace_with_(anObject,"");}
,"copying",unescape("copyWithout%3A%20anObject%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20excluding%20anObject%2C%20if%20any.%20%22%0D%0A%0D%0A%09%5Eself%20replace%3A%20anObject%20with%3A%20%27%27"));
smalltalk.bind(smalltalk.String,"asSymbol",0
,function(){var self=this;
return self;}
,"conversion",unescape("asSymbol%0D%0A%09%22%20Return%20a%20symbol%20with%20the%20same%20characters%20as%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20%22we%20do%20not%20discriminate%20singular%20strings%22"));
smalltalk.bind(smalltalk.String,"argumentCount",0
,function(){var self=this;
if (self.isEmpty()){return (0);};;
if (self.$klass.binarySelectorChars().includes_(self.first())){return (1);};;
return self.occurrencesOf_(":");}
,"evaluating",unescape("argumentCount%0D%0A%09%22%20Return%20the%20number%20of%20arguments%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20isEmpty%20ifTrue%3A%20%5B%20%5E0%20%5D.%0D%0A%09%28self%20class%20binarySelectorChars%20includes%3A%20self%20first%29%20ifTrue%3A%20%5B%20%5E1%20%5D.%0D%0A%09%5Eself%20occurrencesOf%3A%20%24%3A"));
smalltalk.bind(smalltalk.String,"isLiteral",0
,function(){var self=this;

return self.notEmpty().and_((function(){return self.$klass.isMessageSelector_(self);}));}
,"testing",unescape("isLiteral%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20can%20be%20output%20using%20literal%20notation.%20%22%0D%0A%0D%0A%09%7B%27%27%20js%27self%27%20doc%27native%20testing%27%7D.%0D%0A%09%5Eself%20notEmpty%20and%3A%20%5B%20self%20class%20isMessageSelector%3A%20self%20%5D"));
smalltalk.bind(smalltalk.String,"asLiteral",0
,function(){var self=this;
return ((self.isLiteral()).mustBeBoolean()==true ? (function(){return unescape("%23")._comma(self);})() : (function(){return self.printString();})());}
,"converting",unescape("asLiteral%0D%0A%09%22%20Return%20the%20receiver%20in%20literal%20notation.%20%22%0D%0A%0D%0A%09%5Eself%20isLiteral%20ifTrue%3A%20%5B%20%24%23%20%2Cself%20%5D%20ifFalse%3A%20%5B%20self%20printString%20%5D"));
smalltalk.bind(smalltalk.String,"occurrencesOf:",0
,function(aString){var self=this;
var result=nil;
var index=nil;

if (!(aString.isString())){result=self.match(aString);
if ((nil.isNil_(result))){return (0);};;
return result.length;};;
if (aString.size()._eq((1))){result=self.match(smalltalk.RegularExpression.fromString_flag_(unescape("%5B")._comma(aString)._comma(unescape("%5D")),"g"));
if ((nil.isNil_(result))){return (0);};;
return result.length;};;
result=(0);
index=(0);
while((function(){index=self.indexOf_startingAt_(aString,index._plus((1)));return index._gt((1));})()){result=result._plus((1));};;
return result;}
,"testing",unescape("occurrencesOf%3A%20aString%0D%0A%09%22%20Return%20the%20times%20aString%20can%20be%20found%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20index%20%7C%0D%0A%09%7B%27%27%20doc%27native%20counting%27%7D.%0D%0A%09aString%20isString%20ifFalse%3A%20%5B%0D%0A%09%09result%20%3A%3D%20self%20%23match%3A%20aString.%0D%0A%09%09result%20isNil%20ifTrue%3A%20%5B%20%5E0%20%5D.%0D%0A%09%09%5Eresult%23length%0D%0A%09%5D.%0D%0A%09aString%20size%20%3D%201%20ifTrue%3A%20%5B%0D%0A%09%09result%20%3A%3D%20self%20%23match%3A%20%28RegularExpression%0D%0A%09%09%09%09fromString%3A%20%27%5B%27%2CaString%2C%27%5D%27%0D%0A%09%09%09%09flag%3A%20%27g%27%29.%0D%0A%09%09result%20isNil%20ifTrue%3A%20%5B%20%5E0%20%5D.%0D%0A%09%09%5Eresult%23length%0D%0A%09%5D.%0D%0A%09result%20%3A%3D%200.%20index%20%3A%3D%200.%0D%0A%09%5B%09index%20%3A%3D%20self%20indexOf%3A%20aString%20startingAt%3A%20index%20+%201.%0D%0A%09%09index%20%3E%201%20%5D%20whileTrue%3A%20%5B%20result%20%3A%3D%20result%20+%201%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.String,"startsWith:",0
,function(aString){var self=this;
if (self.startsWith) { return self.startsWith(aString) };
return self.beginsWith_(aString);}
,"testing",unescape("startsWith%3A%20aString%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20begins%20with%20the%20characters%20of%20aString.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20testing%27%7D.%0D%0A%09%5Eself%20beginsWith%3A%20aString"));
smalltalk.bind(smalltalk.String,"escaped",0
,function(){var self=this;
return escape(self);
return self;}
,"converting",unescape("escaped%0D%0A%09%22%20Return%20the%20receiver%20with%20escape%20codes.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20conversion%27%7D"));
smalltalk.bind(smalltalk.String,"unescaped",0
,function(){var self=this;
return unescape(self);
return self;}
,"converting",unescape("unescaped%0D%0A%09%22%20Return%20the%20receiver%20from%20encoded%20URI%20escape%20codes.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20conversion%27%7D"));
smalltalk.bind(smalltalk.String,"escapedCode",0
,function(){var self=this;
var escaped=nil;
escaped=self.escaped();
if (escaped._eq(self)){return unescape("%22")._comma(escaped)._comma(unescape("%22"));};;
return unescape("unescape%28%22")._comma(escaped)._comma(unescape("%22%29"));}
,"converting",unescape("escapedCode%0D%0A%09%22%20Return%20the%20code%20to%20reproduce%20receiver%20from%20code.%20%22%0D%0A%0D%0A%09%7C%20escaped%20%7C%0D%0A%09escaped%20%3A%3D%20self%20escaped.%0D%0A%09escaped%20%3D%20self%20ifTrue%3A%20%5B%20%5E%24%22%20%2Cescaped%20%2C%24%22%20%5D.%0D%0A%09%5E%27unescape%28%22%27%20%2Cescaped%20%2C%27%22%29%27"));
smalltalk.bind(smalltalk.String,"asEncodedURI",0
,function(){var self=this;
return encodeURIComponent(self);
return self;}
,"converting",unescape("asEncodedURI%0D%0A%09%22%20Return%20the%20receiver%20encoded%20as%20an%20URI%20component.%0D%0A%09Encodes%20special%20characters%20and%20it%20encodes%20the%20following%20characters%3A%20%2C%20/%20%3F%20%3A%20@%20%26%20%3D%20+%20%24%20%23%0D%0A%09%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20conversion%27%7D"));
smalltalk.bind(smalltalk.String,"asDecodedURI",0
,function(){var self=this;
return decodeURIComponent(self);
return self;}
,"converting",unescape("asDecodedURI%0D%0A%09%22%20Return%20the%20receiver%20decoded%20as%20an%20URI%20component.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20conversion%27%7D"));
smalltalk.bind(smalltalk.String,"add:",0
,function(anObject){var self=this;
return self.errorReadOnly_("add:");}
,"adding",unescape("add%3A%20anObject%0D%0A%09%22%20Add%20anObject%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20errorReadOnly%3A%20%23add%3A"));
smalltalk.bind(smalltalk.String,"-->","_minus_minus_gt"
,function(anObject){var self=this;
return smalltalk.Association.key_value_(self.asArrayOfSubstringsSeparatedBy_("."),anObject);}
,"converting",unescape("--%3E%20anObject%0D%0A%09%22%20Return%20a%20far-association%20of%20the%20receiver%20to%20anObject.%20%22%0D%0A%0D%0A%09%5EAssociation%20key%3A%20%28self%20asArrayOfSubstringsSeparatedBy%3A%20%24.%29%20value%3A%20anObject"));
smalltalk.bind(smalltalk.String,",","_comma"
,function(aCollection){var self=this;

	if (nil.isNil_(aCollection)) return self+""; // for JSC under iOS
	return self + aCollection.toString();
return self;}
,"copying",unescape("%2C%20aCollection%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20with%20all%20contents%20of%20aCollection.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20composition%27%7D"));
smalltalk.bind(smalltalk.String,"copyWith:",0
,function(anObject){var self=this;
return self + anObject.toString();}
,"copying",unescape("copyWith%3A%20anObject%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20with%20anObject.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20composition%27%7D"));
smalltalk.bind(smalltalk.String,"copyFrom:to:",0
,function(anIndex,anotherIndex){var self=this;
return self.substring(anIndex - 1,anotherIndex);}
,"copying",unescape("copyFrom%3A%20anIndex%20to%3A%20anotherIndex%0D%0A%09%22%20Return%20the%20receiver%20between%20limits.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20copy%27%7D"));
smalltalk.bind(smalltalk.String,"shallowCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.String,"deepCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.String,"reversed",0
,function(){var self=this;
var result=nil;

result="";
self.reverseDo_((function(c){return result=result._comma(c);}));
return result;}
,"copying",unescape("reversed%0D%0A%09%22%20Return%20a%20string%20with%20the%20contents%20of%20the%20receiver%20in%20reverse%20order.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09%7B%27self%27%20doc%27native%20conversion%27%7D.%0D%0A%09result%20%3A%3D%20%27%27.%0D%0A%09self%20reverseDo%3A%20%5B%3Ac%7C%20result%20%3A%3D%20result%20%2C%20c%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.String,"indexOf:",0
,function(aString){var self=this;
return self.indexOf(aString) + 1;
return self;}
,"accessing",unescape("indexOf%3A%20aString%0D%0A%09%22%20Return%20the%20position%20of%20first%20occurrence%20of%20aString%20%28or%200%20if%20not%20present%29.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20detection%27%7D"));
smalltalk.bind(smalltalk.String,"indexOf:ifAbsent:",0
,function(aString,aBlock){var self=this;
var result=nil;
result=self.indexOf_(aString);
return result._eq((0)).ifTrue_ifFalse_(aBlock,(function(){return result;}));}
,"accessing",unescape("indexOf%3A%20aString%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20position%20of%20first%20occurrence%20of%20aString%20%28or%20the%20result%20of%20evaluating%20aBlock%29.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20indexOf%3A%20aString.%0D%0A%09%5Eresult%20%3D%200%20ifTrue%3A%20aBlock%20ifFalse%3A%20%5B%20result%20%5D"));
smalltalk.bind(smalltalk.String,"indexOf:startingAt:",0
,function(aString,position){var self=this;
return self.indexOf(aString,position-1) + 1;
return self;}
,"accessing",unescape("indexOf%3A%20aString%20startingAt%3A%20position%0D%0A%09%22%20Return%20the%20position%20of%20first%20occurrence%20of%20aString%20after%20position%20%28or%200%20if%20not%20present%29.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20detection%27%7D"));
smalltalk.bind(smalltalk.String,"lastIndexOf:",0
,function(aString){var self=this;
return self.lastIndexOf(aString) + 1;
return self;}
,"accessing",unescape("lastIndexOf%3A%20aString%0D%0A%09%22%20Return%20the%20position%20of%20last%20occurrence%20of%20aString%20%28or%200%20if%20not%20present%29.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20detection%27%7D"));
smalltalk.bind(smalltalk.String,"lastIndexOf:ifAbsent:",0
,function(aString,aBlock){var self=this;
var result=nil;
result=self.lastIndexOf_(aString);
return result._eq((0)).ifTrue_ifFalse_(aBlock,(function(){return result;}));}
,"accessing",unescape("lastIndexOf%3A%20aString%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20position%20of%20last%20occurrence%20of%20aString%20%28or%20the%20result%20of%20evaluating%20aBlock%29.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20lastIndexOf%3A%20aString.%0D%0A%09%5Eresult%20%3D%200%20ifTrue%3A%20aBlock%20ifFalse%3A%20%5B%20result%20%5D"));
smalltalk.bind(smalltalk.String,"lastIndexOf:startingAt:",0
,function(aString,position){var self=this;
return self.lastIndexOf(aString,position-1) + 1;
return self;}
,"accessing",unescape("lastIndexOf%3A%20aString%20startingAt%3A%20position%0D%0A%09%22%20Return%20the%20position%20of%20last%20occurrence%20of%20aString%20before%20position%20%28or%200%20if%20not%20present%29.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20detection%27%7D"));
smalltalk.bind(smalltalk.String,"upTo:startingAt:",0
,function(aString,position){var self=this;
var index=nil;
index=self.indexOf_startingAt_(aString,position);
index=((index._eq((0))).mustBeBoolean()==true ? (function(){return self.size();})() : (function(){return index._minus((1));})());
return self.copyFrom_to_(position,index);}
,"accessing",unescape("upTo%3A%20aString%20startingAt%3A%20position%0D%0A%09%22%20Return%20the%20receiver%20upto%20first%20occurrence%20of%20aString%20after%20position.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09index%20%3A%3D%20self%20indexOf%3A%20aString%20startingAt%3A%20position.%0D%0A%09index%20%3A%3D%20index%20%3D%200%20ifTrue%3A%20%5B%20self%20size%20%5D%20ifFalse%3A%20%5B%20index%20-%201%20%5D.%0D%0A%09%5Eself%20copyFrom%3A%20position%20to%3A%20index"));
smalltalk.bind(smalltalk.String,"upTo:",0
,function(aCharacter){var self=this;
return self.upTo_startingAt_(aCharacter,(1));}
,"accessing",unescape("upTo%3A%20aCharacter%0D%0A%09%22%20Return%20the%20receiver%20upto%20first%20occurrence%20of%20aCharacter.%20%22%0D%0A%0D%0A%09%5Eself%20upTo%3A%20aCharacter%20startingAt%3A%201"));
smalltalk.bind(smalltalk.String,"upToLast:",0
,function(aString){var $s8Ret$={name:"stReturn"};try{var self=this;
var index=nil;
index=self.lastIndexOf_ifAbsent_(aString,(function(){return (function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return self.copyFrom_to_((1),index._minus((1)))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("upToLast%3A%20aString%0D%0A%09%22%20Return%20the%20receiver%20upto%20last%20occurrence%20of%20aString.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09index%20%3A%3D%20self%20lastIndexOf%3A%20aString%20ifAbsent%3A%20%5B%20%5Eself%20%5D.%0D%0A%09%5Eself%20copyFrom%3A%201%20to%3A%20index%20-%201"));
smalltalk.bind(smalltalk.String,"fromLast:",0
,function(aString){var $s8Ret$={name:"stReturn"};try{var self=this;
var index=nil;
index=self.lastIndexOf_ifAbsent_(aString,(function(){return (function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return self.copyFrom_to_(index._plus(aString.size()),self.size())})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("fromLast%3A%20aString%0D%0A%09%22%20Return%20the%20receiver%20contents%20from%20last%20occurrence%20of%20aString.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09index%20%3A%3D%20self%20lastIndexOf%3A%20aString%20ifAbsent%3A%20%5B%20%5Eself%20%5D.%0D%0A%09%5Eself%20copyFrom%3A%20index%20+%20aString%20size%20to%3A%20self%20size"));
smalltalk.bind(smalltalk.String,"asJsName",0
,function(){var $s8Ret$={name:"stReturn"};try{var self=this;
if (smalltalk.String.reservedWords().includes_(self)){((self._eq("toString")).mustBeBoolean()==true ? nil : (function(){return (function(){$s8Ret$.result=(function(){return "$$"._comma(self)})();throw($s8Ret$)})();})());};;
(function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"js",unescape("asJsName%0D%0A%09%22%20Private%20-%20Return%20the%20receiver%20as%20a%20valid%20native%20name.%20%22%0D%0A%0D%0A%09%28String%20reservedWords%20includes%3A%20self%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20%3D%20%23toString%20ifFalse%3A%20%5B%20%5E%27%24%24%27%20%2Cself%20%5D%0D%0A%09%5D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.String,"asNativeName",0
,function(){var self=this;
return self.asJsName();}
,"js",unescape("asNativeName%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20valid%20native%20name.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20conversion%27%7D"));
smalltalk.bind(smalltalk.String,"asPrivateSelector",0
,function(){var self=this;
var result=nil;
result = self
	.replace(/:/g,"_")
	.replace(/[+]/g,"_plus")
	.replace(/-/g,"_minus")
	.replace(/[*]/g,"_star")
	.replace(/[/]/g,"_slash")
	.replace(/[|]/g,"_bar")
	.replace(/[&]/g,"_amp")
	.replace(/>/g,"_gt")
	.replace(/</g,"_lt")
	.replace(/=/g,"_eq")
	.replace(/,/g,"_comma")
	.replace(/[@]/g,"_at")
	.replace(/~/g,"_no")
	.replace(/\\/g,"_bk");
return result;}
,"converting",unescape("asPrivateSelector%0D%0A%09%22%20Private%20-%20Return%20the%20receiver%20as%20an%20encoded%20selector.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09%7B%27%27%20doc%27native%20conversion%27%7D.%0D%0A%09%5Eresult%20"));
smalltalk.bind(smalltalk.String,"asSelector",0
,function(){var self=this;
if (self.startsWith_(unescape("%23"))){return self.upTo_startingAt_(":",(2));};;
return self.asPrivateSelector().asNativeName();}
,"converting",unescape("asSelector%0D%0A%09%22%20Private%20-%20Return%20the%20receiver%20as%20a%20valid%20native%20selector.%20%22%0D%0A%0D%0A%09%28self%20startsWith%3A%20%24%23%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20upTo%3A%20%24%3A%20startingAt%3A%202%0D%0A%09%5D.%0D%0A%09%5Eself%20asPrivateSelector%20asNativeName%20"));
smalltalk.bind(smalltalk.String,"asNativeSelector",0
,function(){var $s8Ret$={name:"stReturn"};try{var self=this;
var operations=nil;
if (self._eq("$")){return "";};;
if (self._eq("_bk")){return (92).asCharacter();};;
if (self.startsWith_("$$")){return self.copyFrom_to_((3),self.size()).asNativeSelector();};;
operations=[[unescape("+"), "_plus"], [unescape("-"), "_minus"], [unescape("*"), "_star"], [unescape("/"), "_slash"], [unescape("%7C"), "_bar"], [unescape("%26"), "_amp"], [unescape("%3E"), "_gt"], [unescape("%3C"), "_lt"], [unescape("%3D"), "_eq"], [unescape("%2C"), "_comma"], [unescape("@"), "_at"], [unescape("%7E"), "_no"]];
operations.add_(smalltalk.Array.with_with_((92).asCharacter(),"_bk"));
operations.do_((function(tuple){return ((self.startsWith_(tuple.last())).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return operations.reversed().inject_into_(self,(function(result,each){return result.replace_with_(each.last(),each.first());}))})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return self.replace_with_("_",":")})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"converting",unescape("asNativeSelector%0D%0A%09%22%20Return%20a%20smalltalk%20selector%20from%20a%20native%20function%20name.%0D%0A%09Note%3A%20it%20must%20be%20the%20reverse%20operation%20of%20%23asSelector%0D%0A%09%22%0D%0A%0D%0A%09%7C%20operations%20%7C%0D%0A%09self%20%3D%20%27%24%27%20ifTrue%3A%20%5B%20%5E%27%27%20%5D.%0D%0A%09self%20%3D%20%23_bk%20ifTrue%3A%20%5B%20%5E92%20asCharacter%20%5D.%0D%0A%09%28self%20startsWith%3A%20%27%24%24%27%29%20ifTrue%3A%20%5B%0D%0A%09%09%5E%28self%20copyFrom%3A%203%20to%3A%20self%20size%29%20asNativeSelector%0D%0A%09%5D.%0D%0A%09operations%20%3A%3D%20%23%28%0D%0A%09%09%23%28%20+%20_plus%20%29%0D%0A%09%09%23%28%20-%20_minus%20%29%0D%0A%09%09%23%28%20*%20_star%20%29%0D%0A%09%09%23%28%20/%20_slash%20%29%0D%0A%09%09%23%28%20%7C%20_bar%20%29%0D%0A%09%09%23%28%20%26%20_amp%20%29%0D%0A%09%09%23%28%20%3E%20_gt%20%29%0D%0A%09%09%23%28%20%3C%20_lt%20%29%0D%0A%09%09%23%28%20%3D%20_eq%20%29%0D%0A%09%09%23%28%20%2C%20_comma%20%29%0D%0A%09%09%23%28%20@%20_at%20%29%0D%0A%09%09%23%28%20%7E%20_no%20%29%0D%0A%09%29.%0D%0A%09operations%20add%3A%20%28Array%20with%3A%2092%20asCharacter%20with%3A%20%23_bk%29.%0D%0A%09operations%20do%3A%20%5B%3Atuple%7C%0D%0A%09%09%28self%20startsWith%3A%20tuple%20last%29%20ifTrue%3A%20%5B%0D%0A%09%09%09%5Eoperations%20reversed%20inject%3A%20self%20into%3A%20%5B%3Aresult%20%3Aeach%7C%0D%0A%09%09%09%09result%20replace%3A%20each%20last%20with%3A%20each%20first%0D%0A%09%09%09%5D%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5Eself%20replace%3A%20%24_%20with%3A%20%24%3A"));
smalltalk.bind(smalltalk.String,"asStSelector",0
,function(){var self=this;
self.note_(unescape("OBSOLETE%20-%20%23asStSelector%20%28please%20use%20%23asNativeSelector%29"));
return self.asNativeSelector();}
,"converting",unescape("asStSelector%0D%0A%09%22%20OBSOLETE%20-%20Return%20a%20smalltalk%20selector%20from%20a%20native%20function%20name.%0D%0A%09Note%3A%20use%20%23asNativeSelector%0D%0A%09%22%0D%0A%0D%0A%09self%20note%3A%20%27OBSOLETE%20-%20%23asStSelector%20%28please%20use%20%23asNativeSelector%29%27.%0D%0A%09%5Eself%20asNativeSelector"));
smalltalk.bind(smalltalk.String,"slashEncoded",0
,function(){var self=this;
var encoded=nil;
var bar=nil;
bar=(92).asCharacter();
encoded=self;
encoded=encoded.replace_with_(bar,bar._comma(bar));
encoded=encoded.replace_with_(smalltalk.String.lf(),bar._comma("n"));
encoded=encoded.replace_with_(smalltalk.String.cr(),bar._comma("r"));
encoded=encoded.replace_with_(smalltalk.String.tab(),bar._comma("t"));
encoded=encoded.replace_with_(unescape("%22"),bar._comma(unescape("%22")));
return unescape("%22")._comma(encoded)._comma(unescape("%22"));}
,"converting",unescape("slashEncoded%0D%0A%09%22%20Private%20-%20Return%20the%20native%20representation%20of%20the%20receiver%20slashed.%20%22%0D%0A%0D%0A%09%7C%20encoded%20bar%20%7C%0D%0A%09bar%20%3A%3D%2092%20asCharacter.%0D%0A%09encoded%20%3A%3D%20self.%0D%0A%09encoded%20%3A%3D%20encoded%20replace%3A%20bar%20with%3A%20bar%20%2Cbar.%0D%0A%09encoded%20%3A%3D%20encoded%20replace%3A%20String%20lf%20with%3A%20bar%20%2C%24n.%0D%0A%09encoded%20%3A%3D%20encoded%20replace%3A%20String%20cr%20with%3A%20bar%20%2C%24r.%0D%0A%09encoded%20%3A%3D%20encoded%20replace%3A%20String%20tab%20with%3A%20bar%20%2C%24t.%0D%0A%09encoded%20%3A%3D%20encoded%20replace%3A%20%24%22%20with%3A%20bar%20%2C%24%22.%0D%0A%09%5E%24%22%20%2Cencoded%20%2C%24%22"));
smalltalk.bind(smalltalk.String,"asNative:",0
,function(lang){var self=this;
if (lang._eq("lua")){return self.slashEncoded();};;

	var bar=unescape("%22");
	if(self.search(/^[a-zA-Z0-9_:.$ ]*$/) == -1)
		return "unescape(" +bar +escape(self) +bar +")";
	else
		return bar +self +bar;
return self;}
,"converting",unescape("asNative%3A%20lang%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09lang%20%3D%20%23lua%20ifTrue%3A%20%5B%20%5Eself%20slashEncoded%20%5D.%0D%0A%09%7B%27%27%20doc%27native%20conversion%27%7D"));
smalltalk.bind(smalltalk.String,"replace:with:",0
,function(aString,anotherString){var self=this;

if ((nil.is_eqeq_(aString,unescape("%5C")))){return self.replace_with_(unescape("%5C%5C"),anotherString);};;
return self.replaceRegexp_with_(smalltalk.RegularExpression.fromString_flag_(aString,"g"),anotherString);}
,"regularExpressions",unescape("replace%3A%20aString%20with%3A%20anotherString%0D%0A%09%22%20Return%20the%20result%20of%20replacement%20of%20aString%20with%20anotherString%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27self%27%20doc%27native%20conversion%27%7D.%0D%0A%09aString%20%3D%3D%20%24%5C%20ifTrue%3A%20%5B%20%5Eself%20replace%3A%20%27%5C%5C%27%20with%3A%20anotherString%20%5D.%0D%0A%09%5Eself%0D%0A%09%09replaceRegexp%3A%20%28RegularExpression%20fromString%3A%20aString%20flag%3A%20%27g%27%29%0D%0A%09%09with%3A%20anotherString"));
smalltalk.bind(smalltalk.String,"replaceRegexp:with:",0
,function(aRegexp,aString){var self=this;
return self.replace(aRegexp,aString);}
,"regularExpressions",unescape("replaceRegexp%3A%20aRegexp%20with%3A%20aString%0D%0A%09%22%20Return%20the%20result%20of%20replacement%20of%20aRegexp%20with%20aString%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20conversion%27%7D"));
smalltalk.bind(smalltalk.String,"tokenize:",0
,function(aString){var self=this;
return self.split(aString);
return self;}
,"converting",unescape("tokenize%3A%20aString%0D%0A%09%22%20Return%20the%20split%20tokenization%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20conversion%27%7D"));
smalltalk.bind(smalltalk.String,"asArrayOfSubstrings",0
,function(){var self=this;
var result=nil;
var start=nil;
var i=nil;
result=smalltalk.Array.$$new();
i=(1);
self.do_((function(c){if ((nil.isNil_(start))){((c.isSeparator()).mustBeBoolean()==true ? nil : (function(){return start=i;})());} else{((c.isSeparator()).mustBeBoolean()==true ? (function(){result.add_(self.copyFrom_to_(start,i._minus((1))));return start=nil;})() : nil);};;return i=i._plus((1));}));
if ((nil.isNil_(start)===false)){result.add_(self.copyFrom_to_(start,self.size()));};;
return result;}
,"converting",unescape("asArrayOfSubstrings%0D%0A%09%22%20Return%20an%20array%20of%20substrings%20from%20the%20receiver.%0D%0A%09The%20receiver%20is%20divided%20into%20substrings%20by%20one%20or%20more%20occurrences%20of%20separators.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20result%20start%20i%20%7C%0D%0A%09result%20%3A%3D%20Array%20new.%20i%20%3A%3D%201.%0D%0A%09self%20do%3A%20%5B%3Ac%7C%0D%0A%09%09start%20isNil%20ifTrue%3A%20%5B%20%22in%20separator%22%0D%0A%09%09%09c%20isSeparator%20ifFalse%3A%20%5B%20%22start%20of%20substring%22%0D%0A%09%09%09%09start%20%3A%3D%20i.%0D%0A%09%09%09%5D%0D%0A%09%09%5D%20ifFalse%3A%20%5B%20%22in%20word%22%0D%0A%09%09%09c%20isSeparator%20ifTrue%3A%20%5B%20%22end%20of%20substring%22%0D%0A%09%09%09%09result%20add%3A%20%28self%20copyFrom%3A%20start%20to%3A%20i%20-%201%29.%0D%0A%09%09%09%09start%20%3A%3D%20nil.%0D%0A%09%09%09%5D.%0D%0A%09%09%5D.%0D%0A%09%09i%20%3A%3D%20i%20+%201.%0D%0A%09%5D.%0D%0A%09start%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09result%20add%3A%20%28self%20copyFrom%3A%20start%20to%3A%20self%20size%29%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.String,"asArrayOfSubstringsSeparatedBy:",0
,function(aCharacter){var self=this;
return self.tokenize_(aCharacter);}
,"converting",unescape("asArrayOfSubstringsSeparatedBy%3A%20aCharacter%0D%0A%09%22%20Return%20an%20array%20of%20substrings%20from%20the%20receiver.%0D%0A%09The%20receiver%20is%20divided%20into%20substrings%20by%20one%20or%20more%20occurrences%20of%20aCharacter.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20tokenize%3A%20aCharacter"));
smalltalk.bind(smalltalk.String,"match:",0
,function(aRegexp){var self=this;
return self.search(aRegexp) != -1;}
,"regularExpressions",unescape("match%3A%20aRegexp%0D%0A%09%22%20Return%20the%20match%20index%20of%20aRegexp%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20testing%27%7D"));
smalltalk.bind(smalltalk.String,"asString",0
,function(){var self=this;
return self;}
,"converting",unescape("asString%0D%0A%09%22%20Return%20the%20text%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.String,"between:and:",0
,function(min,max){var self=this;
return min._lt_eq(self).and_((function(){return self._lt_eq(max);}));}
,"testing",unescape("between%3A%20min%20and%3A%20max%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20between%20arguments.%20%22%0D%0A%0D%0A%09%5Emin%20%3C%3D%20self%20and%3A%20%5B%20self%20%3C%3D%20max%20%5D"));
smalltalk.bind(smalltalk.String,"isLetter",0
,function(){var self=this;
return self.search("[A-Za-z]$")==0;}
,"testing",unescape("isLetter%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20one%20character%20letter.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20testing%27%7D"));
smalltalk.bind(smalltalk.String,"isDigit",0
,function(){var self=this;
return self.search("[0-9]$")==0;}
,"testing",unescape("isDigit%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20one%20character%20numeric%20digit.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20testing%27%7D"));
smalltalk.bind(smalltalk.String,"isAlphaNumeric",0
,function(){var self=this;

return self.isDigit().or_((function(){return self.isLetter();}));}
,"testing",unescape("isAlphaNumeric%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20an%20alphanumeric%20character.%20%22%0D%0A%0D%0A%09%7B%27self%27%20doc%27native%20testing%27%7D.%0D%0A%09%5Eself%20isDigit%20or%3A%20%5Bself%20isLetter%5D"));
smalltalk.bind(smalltalk.String,"isUppercase",0
,function(){var self=this;
return self.isUpperCase();}
,"testing",unescape("isUppercase%0D%0A%09%22%20Return%20true%20if%20all%20the%20receiver%20characters%20are%20uppercase.%20%22%0D%0A%0D%0A%09%5Eself%20isUpperCase"));
smalltalk.bind(smalltalk.String,"isUpperCase",0
,function(){var self=this;

return self.match_(unescape("%5B%5EA-Z%5D")).not();}
,"testing",unescape("isUpperCase%0D%0A%09%22%20Return%20true%20if%20all%20the%20receiver%20characters%20are%20uppercase.%20%22%0D%0A%0D%0A%09%7B%27self%27%20doc%27native%20testing%27%7D.%0D%0A%09%5E%28self%20match%3A%20%27%5B%5EA-Z%5D%27%29%20not"));
smalltalk.bind(smalltalk.String,"isLowerCase",0
,function(){var self=this;

return self.match_(unescape("%5B%5Ea-z%5D")).not();}
,"testing",unescape("isLowerCase%0D%0A%09%22%20Return%20true%20if%20all%20the%20receiver%20characters%20are%20lowercase.%20%22%0D%0A%0D%0A%09%7B%27self%27%20doc%27native%20testing%27%7D.%0D%0A%09%5E%28self%20match%3A%20%27%5B%5Ea-z%5D%27%29%20not"));
smalltalk.bind(smalltalk.String,"isLowercase",0
,function(){var self=this;
return self.isLowerCase();}
,"testing",unescape("isLowercase%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20one%20character%20lowercase.%20%22%0D%0A%0D%0A%09%5Eself%20isLowerCase"));
smalltalk.bind(smalltalk.String,"asUppercase",0
,function(){var self=this;
return self.toUpperCase();}
,"converting",unescape("asUppercase%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20converted%20to%20uppercase.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20testing%27%7D"));
smalltalk.bind(smalltalk.String,"asUpperCase",0
,function(){var self=this;
return self.asUppercase();}
,"converting",unescape("asUpperCase%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20converted%20to%20uppercase.%20%22%0D%0A%0D%0A%09%5Eself%20asUppercase"));
smalltalk.bind(smalltalk.String,"asLowercase",0
,function(){var self=this;
return self.toLowerCase();}
,"converting",unescape("asLowercase%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20converted%20to%20lowercase.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20testing%27%7D"));
smalltalk.bind(smalltalk.String,"asLowerCase",0
,function(){var self=this;
return self.asLowercase();}
,"converting",unescape("asLowerCase%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20converted%20to%20lowercase.%20%22%0D%0A%0D%0A%09%5Eself%20asLowercase"));
smalltalk.bind(smalltalk.String,"asSmalltalkName:",0
,function(upCase){var $s8Ret$={name:"stReturn"};try{var self=this;
var up=nil;
var result=nil;
if (upCase.not().and_((function(){return (nil.isNil_(self.detect_ifNone_((function(c){return c._gt_eq("a");}),(function(){return nil;}))));}))){return self.asLowercase().asSmalltalkName_(upCase);};;
up=upCase;
result=self.inject_into_("",(function(s,each){return ((unescape("_%3A.%26%25%24%3F").includes_(each)).mustBeBoolean()==true ? (function(){up=true;return s;})() : (function(){return s._comma(((up).mustBeBoolean()==true ? (function(){up=false;return each.asUppercase();})() : (function(){return each;})()));})());}));
if (result.isEmpty()){return "_";};;
if (result.first().isDigit()){return "_"._comma(result);};;
if (upCase){((result.first().isLowercase()).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return result.first().asUppercase()._comma(result.copyFrom_to_((2),result.size()))})();throw($s8Ret$)})();})() : nil);} else{((result.first().isUppercase()).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return result.first().asLowercase()._comma(result.copyFrom_to_((2),result.size()))})();throw($s8Ret$)})();})() : nil);};;
(function(){$s8Ret$.result=(function(){return result})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"converting",unescape("asSmalltalkName%3A%20upCase%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20converted%20to%20a%20smalltalk%20global%20name.%20%22%0D%0A%0D%0A%09%7C%20up%20result%20%7C%0D%0A%09%28upCase%20not%20and%3A%20%5B%0D%0A%09%09%28self%20detect%3A%20%5B%3Ac%7C%20c%20%3E%3D%20%24a%20%5D%20ifNone%3A%20%5B%5D%29%20isNil%0D%0A%09%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20asLowercase%20asSmalltalkName%3A%20upCase%0D%0A%09%5D.%0D%0A%09up%20%3A%3D%20upCase.%0D%0A%09result%20%3A%3D%20self%20inject%3A%20%27%27%20into%3A%20%5B%3As%20%3Aeach%7C%0D%0A%09%09%28%27_%3A.%26%25%24%3F%27%20includes%3A%20each%29%20ifTrue%3A%20%5B%20up%20%3A%3D%20true.%20s%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20s%20%2C%20%28up%0D%0A%09%09%09ifTrue%3A%20%5B%20%20up%20%3A%3D%20false.%20each%20asUppercase%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20each%20%5D%29%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09result%20isEmpty%20ifTrue%3A%20%5B%20%5E%24_%20%5D.%0D%0A%09result%20first%20isDigit%20ifTrue%3A%20%5B%20%5E%24_%20%2Cresult%20%5D.%0D%0A%09upCase%20ifTrue%3A%20%5B%0D%0A%09%09result%20first%20isLowercase%20ifTrue%3A%20%5B%0D%0A%09%09%09%5Eresult%20first%20asUppercase%20%2C%28result%20copyFrom%3A%202%20to%3A%20result%20size%29%0D%0A%09%09%5D%0D%0A%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09result%20first%20isUppercase%20ifTrue%3A%20%5B%0D%0A%09%09%09%5Eresult%20first%20asLowercase%20%2C%28result%20copyFrom%3A%202%20to%3A%20result%20size%29%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.String,"asGlobalName",0
,function(){var self=this;
return self.asSmalltalkName_(true);}
,"converting",unescape("asGlobalName%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20converted%20to%20a%20smalltalk%20global%20name.%20%22%0D%0A%0D%0A%09%5Eself%20asSmalltalkName%3A%20true"));
smalltalk.bind(smalltalk.String,"asLocalName",0
,function(){var self=this;
return self.asSmalltalkName_(false);}
,"converting",unescape("asLocalName%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20converted%20to%20a%20smalltalk%20local%20name.%20%22%0D%0A%0D%0A%09%5Eself%20asSmalltalkName%3A%20false"));
smalltalk.bind(smalltalk.String,"digitValue",0
,function(){var self=this;
return "0123456789ABCDEF".indexOf_(self)._minus((1)).max_((0)._minus((1)));}
,"converting",unescape("digitValue%0D%0A%09%22%20Return%20the%20digit%20value%20of%20the%20receiver%20%28or%20-1%29.%20%22%0D%0A%0D%0A%09%5E%28%270123456789ABCDEF%27%20indexOf%3A%20self%29%20-%201%20max%3A%200%20-%201"));
smalltalk.bind(smalltalk.String,"asNumber",0
,function(){var self=this;
if (self.includes_("r")){return smalltalk.Number.readFrom_(self.readStream());};;
return Number(self);}
,"converting",unescape("asNumber%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20Number.%20%22%0D%0A%0D%0A%09%28self%20includes%3A%20%24r%29%20ifTrue%3A%20%5B%0D%0A%09%09%5ENumber%20readFrom%3A%20self%20readStream%0D%0A%09%5D.%0D%0A%09%5E%7B%27%27%20doc%27native%20converting%27%7D"));
smalltalk.bind(smalltalk.String,"errorReadOnly",0
,function(){var self=this;
return self.error_("Strings are inmutable. Attempt to change "._comma(self.asLiteral()));}
,"error",unescape("errorReadOnly%0D%0A%09%22%20DEPRECATED%2C%20Private%20-%20The%20receiver%20is%20read-only%20instance.%20%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%27Strings%20are%20inmutable.%20Attempt%20to%20change%20%27%20%2Cself%20asLiteral"));
smalltalk.bind(smalltalk.String,"errorReadOnly:",0
,function(selector){var self=this;
return self.error_("Strings are inmutable. "._comma(selector.asLiteral())._comma(" sent to ")._comma(self.asLiteral()));}
,"error",unescape("errorReadOnly%3A%20selector%0D%0A%09%22%20Private%20-%20The%20receiver%20is%20read-only%20instance.%20%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%27Strings%20are%20inmutable.%20%27%0D%0A%09%09%2Cselector%20asLiteral%0D%0A%09%09%2C%27%20sent%20to%20%27%20%2Cself%20asLiteral"));
smalltalk.bind(smalltalk.String,"printString",0
,function(){var self=this;
var body=nil;
body=((self.includes_(unescape("%27"))).mustBeBoolean()==true ? (function(){return self.asArrayOfSubstringsSeparatedBy_(unescape("%27")).join_(unescape("%27%27"));})() : (function(){return self;})());
return unescape("%27")._comma(body)._comma(unescape("%27"));}
,"printing",unescape("printString%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20body%20%7C%0D%0A%09body%20%3A%3D%20%28self%20includes%3A%20%24%27%29%20ifTrue%3A%20%5B%0D%0A%09%09%28self%20asArrayOfSubstringsSeparatedBy%3A%20%24%27%29%20join%3A%20%27%27%27%27%27%27%0D%0A%09%5D%20ifFalse%3A%20%5B%20self%20%5D.%0D%0A%09%5E%24%27%20%2Cbody%20%2C%24%27"));
smalltalk.bind(smalltalk.String,"isString",0
,function(){var self=this;
return true;}
,"testing",unescape("isString%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20String.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.String,"isBackSlash",0
,function(){var self=this;
return self.asciiValue()._eq((92));}
,"testing",unescape("isBackSlash%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20back-slash%20string.%20%22%0D%0A%0D%0A%09%5Eself%20asciiValue%20%3D%2092%20"));
smalltalk.bind(smalltalk.String,">","_gt"
,function(aString){var self=this;
return String(self) > aString;}
,"comparing",unescape("%3E%20aString%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20greater%20than%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20compare%27%7D"));
smalltalk.bind(smalltalk.String,"<","_lt"
,function(aString){var self=this;
return String(self) < aString;}
,"comparing",unescape("%3C%20aString%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20smaller%20than%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20compare%27%7D"));
smalltalk.bind(smalltalk.String,">=","_gt_eq"
,function(aString){var self=this;
return String(self) >= aString;}
,"comparing",unescape("%3E%3D%20aString%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20greater%20or%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20compare%27%7D"));
smalltalk.bind(smalltalk.String,"<=","_lt_eq"
,function(aString){var self=this;
return String(self) <= aString;}
,"comparing",unescape("%3C%3D%20aString%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20smaller%20or%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20compare%27%7D"));
smalltalk.bind(smalltalk.String,"remove:ifAbsent:",0
,function(anObject,aBlock){var self=this;
return self.errorReadOnly_("remove:ifAbsent:");}
,"removing",unescape("remove%3A%20anObject%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Remove%20anObject%20from%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20errorReadOnly%3A%20%23remove%3AifAbsent%3A"));
smalltalk.bind(smalltalk.String,"removeAll:",0
,function(aCollection){var self=this;
return self.errorReadOnly_("removeAll:");}
,"removing",unescape("removeAll%3A%20aCollection%0D%0A%09%22%20Remove%20aCollection%20from%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20errorReadOnly%3A%20%23removeAll%3A"));
smalltalk.bind(smalltalk.String,"removeAll",0
,function(){var self=this;
return self.errorReadOnly_("removeAll");}
,"removing",unescape("removeAll%0D%0A%09%22%20Remove%20all%20elements%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20errorReadOnly%3A%20%23removeAll"));
smalltalk.bind(smalltalk.String,"asJSONObject",0
,function(){var self=this;

return self._comma("");}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself%20%2C%20%27%27%0D%0A%22Note%3A%09we%20need%20to%20normalize%20result%20to%20do%20not%20destroy%20typeof%20checkings%0D%0A%09Do%20NOT%20modify%20this%20method%20if%20you%20do%20not%20know%20what%20you%20are%20doing.%0D%0A%09If%20you%20remove%20the%20%2C%27%27%20computation%2C%20the%20returned%20object%20will%20return%20%27object%27%20and%20not%20%27string%27%20as%20result%20of%20typeof%20operation.%0D%0A%22"));
smalltalk.bind(smalltalk.String,"isCharacter",0
,function(){var self=this;
return self.length == 1;}
,"testing",unescape("isCharacter%0D%0A%09%22%20Implemented%20to%20support%20Character%20protocoll%20for%20one-character%20Strings.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20testing%27%7D"));
smalltalk.bind(smalltalk.String,"asciiValue",0
,function(){var self=this;
return self.charCodeAt(0);}
,"testing",unescape("asciiValue%0D%0A%09%22%20Return%20the%20number%20corresponding%20to%20the%20ASCII%20encoding%20of%20the%20first%20character%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20converting%27%7D"));
smalltalk.bind(smalltalk.String,"isSeparator",0
,function(){var self=this;
return (new RegExp("\\s|\\u0085")).test(self);}
,"testing",unescape("isSeparator%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20either%20a%20space%2C%20tab%2C%20new-line%2C%20form-feed%20character%2C%20or%20double-byte%20space.%0D%0A%09Note%20that%20new-line%20has%20been%20extended%20to%20support%20NEL%20character.%0D%0A%09See%20https%3A//en.wikipedia.org/wiki/Newline%0D%0A%09%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20testing%27%7D"));
smalltalk.bind(smalltalk.String,"asAsciiZ",0
,function(){var self=this;
return self._comma(self.$klass.zero());}
,"converting",unescape("asAsciiZ%0D%0A%09%22%20Return%20the%20receiver%20with%20ascii%20zero%20appended.%20%22%0D%0A%0D%0A%09%5Eself%20%2C%20self%20class%20zero"));
smalltalk.bind(smalltalk.String,"trimFirst:last:",0
,function(firstBlock,lastBlock){var $s8Ret$={name:"stReturn"};try{var self=this;
var start=nil;
var stop=nil;
start=(1);
stop=self.size();
while((function(){if (start._gt(stop)){(function(){$s8Ret$.result=(function(){return smalltalk.String.$$new()})();throw($s8Ret$)})();};;return firstBlock.value_(self.at_(start));})()){start=start._plus((1));};;
while((function(){if (stop._lt(start)){(function(){$s8Ret$.result=(function(){return smalltalk.String.$$new()})();throw($s8Ret$)})();};;return lastBlock.value_(self.at_(stop));})()){stop=stop._minus((1));};;
(function(){$s8Ret$.result=(function(){return self.copyFrom_to_(start,stop)})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"selecting",unescape("trimFirst%3A%20firstBlock%20last%3A%20lastBlock%0D%0A%09%22%20Return%20a%20String%20containing%20the%20result%20of%20trimming%20characters%20matching%20blocks%20from%20both%20sides%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20start%20stop%20%7C%0D%0A%09start%20%3A%3D%201.%20stop%20%3A%3D%20self%20size.%0D%0A%09%5B%09start%20%3E%20stop%20ifTrue%3A%20%5B%20%5EString%20new%20%5D.%0D%0A%09%09firstBlock%20value%3A%20%28self%20at%3A%20start%29%0D%0A%09%5D%20whileTrue%3A%20%5B%20start%20%3A%3D%20start%20+%201%20%5D.%0D%0A%09%5B%09stop%20%3C%20start%20ifTrue%3A%20%5B%20%5EString%20new%20%5D.%0D%0A%09%09lastBlock%20value%3A%20%28self%20at%3A%20stop%29%0D%0A%09%5D%20whileTrue%3A%20%5B%20stop%20%3A%3D%20stop%20-%201%20%5D.%0D%0A%09%5Eself%20copyFrom%3A%20start%20to%3A%20stop"));
smalltalk.bind(smalltalk.String,"trim:",0
,function(aBlock){var self=this;
return self.trimFirst_last_(aBlock,aBlock);}
,"selecting",unescape("trim%3A%20aBlock%0D%0A%09%22%20Return%20a%20String%20containing%20the%20result%20of%20trimming%20characters%20matching%20aBlock%20from%20both%20sides%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20trimFirst%3A%20aBlock%20last%3A%20aBlock"));
smalltalk.bind(smalltalk.String,"trimFirst:",0
,function(aBlock){var self=this;
return self.trimFirst_last_(aBlock,(function(c){return false;}));}
,"selecting",unescape("trimFirst%3A%20aBlock%0D%0A%09%22%20Return%20a%20String%20containing%20the%20result%20of%20trimming%20characters%20matching%20aBlock%20from%20start%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20trimFirst%3A%20aBlock%20last%3A%20%5B%3Ac%7C%20false%20%5D"));
smalltalk.bind(smalltalk.String,"trimLast:",0
,function(aBlock){var self=this;
return self.trimFirst_last_((function(c){return false;}),aBlock);}
,"selecting",unescape("trimLast%3A%20aBlock%0D%0A%09%22%20Return%20a%20String%20containing%20the%20result%20of%20trimming%20characters%20matching%20aBlock%20from%20end%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20trimFirst%3A%20%5B%3Ac%7C%20false%20%5D%20last%3A%20aBlock"));
smalltalk.bind(smalltalk.String,"trimBlanks",0
,function(){var self=this;
return self.trim_((function(c){return c.isSeparator();}));}
,"selecting",unescape("trimBlanks%0D%0A%09%22%20Return%20a%20String%20containing%20the%20receiver%20string%20with%20leading%20and%20trailing%20blanks%20removed.%20%22%0D%0A%0D%0A%09%5Eself%20trim%3A%20%5B%3Ac%7C%20c%20isSeparator%20%5D"));
smalltalk.bind(smalltalk.String,"to:",0
,function(end){var self=this;
return self.to_by_(end,((self._lt_eq(end)).mustBeBoolean()==true ? (function(){return (1);})() : (function(){return (-1);})()));}
,"iterating",unescape("to%3A%20end%0D%0A%09%22%20Return%20an%20iterable%20from%20the%20receiver%20to%20end.%20%22%0D%0A%0D%0A%09%5Eself%20to%3A%20end%20by%3A%20%28self%20%3C%3D%20end%20ifTrue%3A%20%5B1%5D%20ifFalse%3A%20%5B-1%5D%29"));
smalltalk.bind(smalltalk.String,"to:by:",0
,function(end,increment){var self=this;
return self.asciiValue().to_by_value_(end.asciiValue(),increment,(function(index,block){return block.value_(index.asCharacter());}));}
,"iterating",unescape("to%3A%20end%20by%3A%20increment%0D%0A%09%22%20Return%20an%20iterable%20from%20the%20receiver%20to%20end.%20%22%0D%0A%0D%0A%09%5Eself%20asciiValue%0D%0A%09%09to%3A%20end%20asciiValue%0D%0A%09%09by%3A%20increment%0D%0A%09%09value%3A%20%5B%3Aindex%20%3Ablock%7C%20block%20value%3A%20index%20asCharacter%20%5D"));
smalltalk.bind(smalltalk.String,"writeStream",0
,function(){var self=this;
return smalltalk.StringWriteStream.on_(self);}
,"streams",unescape("writeStream%0D%0A%09%22%20Return%20a%20write%20stream%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EStringWriteStream%20on%3A%20self"));
smalltalk.bind(smalltalk.String,"evaluateFor:",0
,function(anObject){var self=this;
return anObject.perform_(self);}
,"evaluating",unescape("evaluateFor%3A%20anObject%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20with%20anObject.%20%22%0D%0A%0D%0A%09%5EanObject%20perform%3A%20self"));
smalltalk.bind(smalltalk.String,"asParser",0
,function(){var self=this;
return ((self.isCharacter()).mustBeBoolean()==true ? (function(){return smalltalk.PPCharacterParser.in_(self);})() : (function(){return smalltalk.PPStringParser.$$new().string_(self);})());}
,"Compiler-converting",unescape("asParser%0D%0A%09%22%20Return%20a%20parser%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20isCharacter%0D%0A%09ifTrue%3A%20%20%5B%20PPCharacterParser%20in%3A%20self%20%5D%0D%0A%09ifFalse%3A%20%5B%20PPStringParser%20new%20string%3A%20self%20%5D"));
smalltalk.bind(smalltalk.String,"asChoiceParser",0
,function(){var self=this;
return smalltalk.PPChoiceParser.withAll_(self.asArray().collect_((function(each){return each.asParser();})));}
,"Compiler-converting",unescape("asChoiceParser%0D%0A%09%22%20Return%20a%20parser%20of%20choices%20for%20each%20character%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EPPChoiceParser%20withAll%3A%20%28self%20asArray%20collect%3A%20%5B%3Aeach%20%7C%20each%20asParser%20%5D%29"));
smalltalk.bind(smalltalk.String,"asCharacterParser",0
,function(){var self=this;
return smalltalk.PPCharacterParser.$$new().string_(self);}
,"Compiler-converting",unescape("asCharacterParser%0D%0A%09%22%20Return%20a%20parser%20on%20the%20characters%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EPPCharacterParser%20new%20string%3A%20self"));
smalltalk.bind(smalltalk.String,"showIt",0
,function(){var self=this;
return self.print_(self.logEvaluate());}
,"Tools-evaluating",unescape("showIt%0D%0A%09%22%20Show/Print%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20print%3A%20self%20logEvaluate"));
smalltalk.bind(smalltalk.String,"logEvaluate",0
,function(){var self=this;
return (function(){return self.evaluate();}).logEvaluate_(self);}
,"Tools-evaluating",unescape("logEvaluate%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%0D%0A%09The%20source%20of%20evaluation%20will%20be%20logged.%0D%0A%09%22%0D%0A%0D%0A%09%5E%5B%20self%20evaluate%20%5D%20logEvaluate%3A%20self%20"));
smalltalk.bind(smalltalk.String,"evaluate",0
,function(){var self=this;
return self.smalltalk().compilerSupport().evaluate_(self);}
,"Tools-evaluating",unescape("evaluate%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20smalltalk%20compilerSupport%20evaluate%3A%20self%20"));
smalltalk.bind(smalltalk.String,"evaluateIn:",0
,function(aReceiver){var self=this;
var target=nil;
target=((self.isObject_(aReceiver).and_((function(){return (nil.isNil_(aReceiver)===false);}))).mustBeBoolean()==true ? (function(){return aReceiver;})() : nil);
return target.smalltalk().compilerSupport().evaluate_to_(self,target);}
,"Tools-evaluating",unescape("evaluateIn%3A%20aReceiver%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20in%20the%20context%20of%20aReceiver.%20%22%0D%0A%0D%0A%09%7C%20target%20%7C%0D%0A%09target%20%3A%3D%20%28%28self%20isObject%3A%20aReceiver%29%20and%3A%20%5BaReceiver%20notNil%5D%29%20ifTrue%3A%20%5B%20aReceiver%20%5D.%0D%0A%09%5Etarget%20smalltalk%20compilerSupport%20evaluate%3A%20self%20to%3A%20target"));
smalltalk.bind(smalltalk.String,"doIt",0
,function(){var self=this;
return self.smalltalk().compilerSupport().doIt_(self);}
,"Tools-evaluating",unescape("doIt%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20or%20the%20error%20occurred%20during%20compilation%20or%20evaluation.%20%22%0D%0A%0D%0A%09%5Eself%20smalltalk%20compilerSupport%20doIt%3A%20self"));
smalltalk.bind(smalltalk.String,"fileIn",0
,function(){var self=this;
return self.stream().fileIn();}
,"Tools-fileIn",unescape("fileIn%0D%0A%09%22%20FileIn%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20stream%20fileIn"));
smalltalk.bind(smalltalk.String,"fileOut",0
,function(){var self=this;
return smalltalk.Exporter.fileOut_(self);}
,"Tools-fileOut",unescape("fileOut%0D%0A%09%22%20FileOut%20the%20classes/categories%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EExporter%20fileOut%3A%20self"));
smalltalk.bind(smalltalk.String,"export","$$export"
,function(){var self=this;
return smalltalk.Exporter.export_(self);}
,"Tools-fileOut",unescape("export%0D%0A%09%22%20Export%20the%20classes/categories%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EExporter%20export%3A%20self"));
smalltalk.bind(smalltalk.String,"asComment",0
,function(){var self=this;
var prologue=nil;
var epilogue=nil;
if (self.includes_(smalltalk.String.cr()).or_((function(){return self.includes_(smalltalk.String.lf());}))){prologue="/* ";
epilogue="*/ ";} else{prologue="// ";
epilogue="";};;
return prologue._comma(self)._comma(epilogue);}
,"Tools-converting",unescape("asComment%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20source%20code%20comment.%20%22%0D%0A%0D%0A%09%7C%20prologue%20epilogue%20%7C%0D%0A%09%28%28self%20includes%3A%20String%20cr%29%20or%3A%20%5Bself%20includes%3A%20String%20lf%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09prologue%20%3A%3D%20%7B%27%27%7D.%0D%0A%09%09epilogue%20%3A%3D%20%7B%27%27%7D.%0D%0A%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09prologue%20%3A%3D%20%7B%27%27%7D.%0D%0A%09%09epilogue%20%3A%3D%20%27%27%0D%0A%09%5D.%0D%0A%09%5Eprologue%20%2Cself%20%2Cepilogue"));
smalltalk.bind(smalltalk.String,"senders",0
,function(){var self=this;
return smalltalk.Smalltalk.sendersOf_(self);}
,"Tools-query",unescape("senders%0D%0A%09%22%20Return%20the%20senders%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5ESmalltalk%20sendersOf%3A%20self"));
smalltalk.bind(smalltalk.String,"implementors",0
,function(){var self=this;
return smalltalk.Smalltalk.implementorsOf_(self);}
,"Tools-query",unescape("implementors%0D%0A%09%22%20Return%20the%20implementors%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5ESmalltalk%20implementorsOf%3A%20self"));
smalltalk.bind(smalltalk.String,"ms:",0
,function(aBlock){var self=this;
self.note_(self._comma(unescape("%20%28"))._comma(aBlock.ms())._comma(unescape("ms%29")));
return self;}
,"Tools-debug",unescape("ms%3A%20aBlock%0D%0A%09%22%20Print%20the%20time%20spent%20evaluating%20aBlock%20titled%20with%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20note%3A%20self%20%2C%27%20%28%27%20%2CaBlock%20ms%20%2C%27ms%29%27"));
smalltalk.bind(smalltalk.String,"emmitFiles",0
,function(){var self=this;
var folder=nil;
var fileNames=nil;
var stream=nil;
var line=nil;
folder=self;
fileNames=[];
stream=folder._comma("emmit.txt").fileContents().stream();
while(!(function(){return stream.atEnd();})()){line=stream.nextLine().upTo_(unescape("%3B")).trimBlanks();
((line.notEmpty()).mustBeBoolean()==true ? (function(){return ((line.first()._eq(unescape("@"))).mustBeBoolean()==true ? (function(){folder=line.copyFrom_to_((2),line.size()).trimBlanks();return ((folder.isEmpty()).mustBeBoolean()==true ? (function(){return folder=self;})() : nil);})() : (function(){return fileNames.add_(folder._comma(line));})());})() : nil);};;
fileNames.emmitAll();
return self;}
,"Builders-emmiting",unescape("emmitFiles%0D%0A%09%22%20Emmit%20all%20the%20files%20listed%20in%20emmit.txt%20%22%0D%0A%0D%0A%09%7C%20folder%20fileNames%20stream%20line%20%7C%0D%0A%09folder%20%3A%3D%20self.%0D%0A%09fileNames%20%3A%3D%20%23%28%29.%0D%0A%09stream%20%3A%3D%20%28folder%20%2C%27emmit.txt%27%29%20fileContents%20stream.%0D%0A%09%5B%20stream%20atEnd%20%5D%20whileFalse%3A%20%5B%0D%0A%09%09line%20%3A%3D%20%28stream%20nextLine%20upTo%3A%20%24%3B%29%20trimBlanks.%0D%0A%09%09line%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%09%09line%20first%20%3D%20%24@%20ifTrue%3A%20%5B%0D%0A%09%09%09%09folder%20%3A%3D%20%28line%20copyFrom%3A%202%20to%3A%20line%20size%29%20trimBlanks.%0D%0A%09%09%09%09folder%20isEmpty%20ifTrue%3A%20%5B%20folder%20%3A%3D%20self%20%5D%0D%0A%09%09%09%5D%20ifFalse%3A%20%5B%20fileNames%20add%3A%20folder%20%2Cline%20%5D%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09fileNames%20emmitAll"));
smalltalk.bind(smalltalk.String,"outputToFile:",0
,function(fileName){var self=this;
return fileWrite(fileName,self,"wb");}
,"Snapshot-io",unescape("outputToFile%3A%20fileName%0D%0A%09%22%20Save%20the%20receiver%27s%20contents%20onto%20a%20file%20at%20fileName.%20%22%0D%0A%0D%0A%09%5E%23%20%23fileWrite%3A%20fileName%20string%3A%20self%20mode%3A%20%23wb"));
smalltalk.bind(smalltalk.String,"appendToFile:",0
,function(fileName){var self=this;
return fileWrite(fileName,self,"ab");}
,"Snapshot-io",unescape("appendToFile%3A%20fileName%0D%0A%09%22%20Append%20the%20receiver%27s%20contents%20onto%20a%20file%20at%20fileName.%20%22%0D%0A%0D%0A%09%5E%23%20%23fileWrite%3A%20fileName%20string%3A%20self%20mode%3A%20%23ab"));
smalltalk.bind(smalltalk.String,"asLuaName",0
,function(){var self=this;
if (self.first()._eq("_")){return self;};;
if (smalltalk.String.reservedLuaWords().includes_(self)){return "__"._comma(self);};;
return self;}
,"Lua-lua",unescape("asLuaName%0D%0A%09%22%20Private%20-%20Return%20the%20receiver%20as%20a%20valid%20Lua%20name.%20%22%0D%0A%0D%0A%09self%20first%20%3D%20%24_%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09%28String%20reservedLuaWords%20includes%3A%20self%29%20ifTrue%3A%20%5B%20%5E%27__%27%2Cself%20%5D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.String,"asLuaSelector",0
,function(){var self=this;
if (smalltalk.String.forbiddenLuaSelectors().includes_(self)){return "_"._comma(self);};;
if (self.startsWith_(unescape("%23"))){return self.upTo_startingAt_(":",(2));};;
return self.asPrivateSelector().asLuaName();}
,"Lua-lua",unescape("asLuaSelector%0D%0A%09%22%20Private%20-%20Return%20the%20receiver%20as%20a%20valid%20native%20selector.%20%22%0D%0A%0D%0A%09%28String%20forbiddenLuaSelectors%20includes%3A%20self%29%20ifTrue%3A%20%5B%20%5E%24_%20%2Cself%20%5D.%0D%0A%09%28self%20startsWith%3A%20%24%23%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20upTo%3A%20%24%3A%20startingAt%3A%202%0D%0A%09%5D.%0D%0A%09%5Eself%20asPrivateSelector%20asLuaName%20"));
smalltalk.bind(smalltalk.String,"asLua",0
,function(){var self=this;
var encoded=nil;
var bar=nil;
bar=(92).asCharacter();
encoded=self;
encoded=encoded.replace_with_(bar,bar._comma(bar));
encoded=encoded.replace_with_(smalltalk.String.lf(),bar._comma("n"));
encoded=encoded.replace_with_(smalltalk.String.cr(),bar._comma("r"));
encoded=encoded.replace_with_(smalltalk.String.tab(),bar._comma("t"));
encoded=encoded.replace_with_(unescape("%22"),bar._comma(unescape("%22")));
return unescape("%22")._comma(encoded)._comma(unescape("%22"));}
,"Lua-lua",unescape("asLua%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20encoded%20bar%20%7C%0D%0A%09bar%20%3A%3D%2092%20asCharacter.%0D%0A%09encoded%20%3A%3D%20self.%0D%0A%09encoded%20%3A%3D%20encoded%20replace%3A%20bar%20with%3A%20bar%20%2Cbar.%0D%0A%09encoded%20%3A%3D%20encoded%20replace%3A%20String%20lf%20with%3A%20bar%20%2C%24n.%0D%0A%09encoded%20%3A%3D%20encoded%20replace%3A%20String%20cr%20with%3A%20bar%20%2C%24r.%0D%0A%09encoded%20%3A%3D%20encoded%20replace%3A%20String%20tab%20with%3A%20bar%20%2C%24t.%0D%0A%09encoded%20%3A%3D%20encoded%20replace%3A%20%24%22%20with%3A%20bar%20%2C%24%22.%0D%0A%09%5E%24%22%20%2Cencoded%20%2C%24%22"));
smalltalk.bind(smalltalk.String,"fileContents",0
,function(){var self=this;
return read(self);}
,"io",unescape("fileContents%0D%0A%09%22%20Returns%20the%20contents%20of%20file%20at%20%28fileName%20given%20by%29%20the%20receiver.%0D%0A%09Return%20nil%20if%20the%20file%20do%20not%20exist%20or%20contents%20can%27t%20be%20read.%0D%0A%09%22%0D%0A%0D%0A%09%5E%23%20%23read%3A%20self"));
smalltalk.bind(smalltalk.Array.$klass,"new","$$new"
,function(){var self=this;
return [];}
,"instantiation",unescape("new%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%29"));
smalltalk.bind(smalltalk.Array.$klass,"basicNew:",0
,function(size){var self=this;
return new Array(size);}
,"instantiation",unescape("basicNew%3A%20size%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20of%20specified%20size.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20instantiation%27%7D"));
smalltalk.bind(smalltalk.Array.$klass,"new:",0
,function(size){var self=this;
return self.basicNew_(size).atAllPut_(nil);}
,"instantiation",unescape("new%3A%20size%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20of%20specified%20size.%20%22%0D%0A%0D%0A%09%5E%28self%20basicNew%3A%20size%29%20atAllPut%3A%20nil"));
smalltalk.bind(smalltalk.Array.$klass,"with:",0
,function(anObject){var self=this;
return [ anObject ];
return (function($rec){$rec.add_(anObject);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("with%3A%20anObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20instantiation%27%7D.%0D%0A%09%5Eself%20new%0D%0A%09%09add%3A%20anObject%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Array.$klass,"with:with:",0
,function(firstObject,secondObject){var self=this;
return [ firstObject, secondObject ];
return (function($rec){$rec.add_(firstObject);$rec.add_(secondObject);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("with%3A%20firstObject%20with%3A%20secondObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20instantiation%27%7D.%0D%0A%09%5Eself%20new%0D%0A%09%09add%3A%20firstObject%3B%0D%0A%09%09add%3A%20secondObject%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Array.$klass,"with:with:with:",0
,function(firstObject,secondObject,thirdObject){var self=this;
return [ firstObject, secondObject, thirdObject ];
return (function($rec){$rec.add_(firstObject);$rec.add_(secondObject);$rec.add_(thirdObject);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("with%3A%20firstObject%20with%3A%20secondObject%20with%3A%20thirdObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20instantiation%27%7D.%0D%0A%09%5Eself%20new%0D%0A%09%09add%3A%20firstObject%3B%0D%0A%09%09add%3A%20secondObject%3B%0D%0A%09%09add%3A%20thirdObject%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Array.$klass,"with:with:with:with:",0
,function(firstObject,secondObject,thirdObject,fourthObject){var self=this;
return [ firstObject, secondObject, thirdObject, fourthObject ];
return (function($rec){$rec.add_(firstObject);$rec.add_(secondObject);$rec.add_(thirdObject);$rec.add_(fourthObject);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("with%3A%20firstObject%20with%3A%20secondObject%20with%3A%20thirdObject%20with%3A%20fourthObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20instantiation%27%7D.%0D%0A%09%5Eself%20new%0D%0A%09%09add%3A%20firstObject%3B%0D%0A%09%09add%3A%20secondObject%3B%0D%0A%09%09add%3A%20thirdObject%3B%0D%0A%09%09add%3A%20fourthObject%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Array.$klass,"withAll:",0
,function(aCollection){var self=this;
return [].concat(aCollection.asArray());
return self;}
,"instantiation",unescape("withAll%3A%20aCollection%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20instantiation%27%7D"));
smalltalk.bind(smalltalk.Array.$klass,"slice:",0
,function(anIterable){var self=this;
return Array.prototype.slice.call(anIterable);}
,"util",unescape("slice%3A%20anIterable%0D%0A%09%22%20Return%20an%20Array%20instance%20from%20contents%20of%20anIterable.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20instantiation%27%7D"));
smalltalk.bind(smalltalk.Array.$klass,"@@","_at_at"
,function(hArray){var self=this;

return hArray;}
,"Natives-instantiation",unescape("@@%20hArray%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20wrapping%20native%20handle.%20%22%0D%0A%0D%0A%09%7B%27%27%7D.%0D%0A%09%5EhArray"));
smalltalk.bind(smalltalk.Array.$klass,"@","_at"
,function(natives){var self=this;
var result=nil;
result=self.$$new();
if ((nil.isNil_(natives))){return result;};;
result.addAll_(natives);
return result;}
,"Natives-instantiation",unescape("@%20natives%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20containing%20the%20values%20in%20native%20object.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20new.%0D%0A%09natives%20isNil%20ifTrue%3A%20%5B%20%5Eresult%20%5D.%0D%0A%09%7B%27%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Array,"size",0
,function(){var self=this;
return self.length;}
,"accessing",unescape("size%0D%0A%09%22%20Return%20the%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20size%27%7D"));
smalltalk.bind(smalltalk.Array,"at:",0
,function(anIndex){var self=this;
var value = self[anIndex - 1];
	if(value === undefined) { return self.objectNotFoundAt_(anIndex); }
	return value;
return self;}
,"accessing",unescape("at%3A%20anIndex%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver%20at%20anIndex.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20indexing%27%7D"));
smalltalk.bind(smalltalk.Array,"at:ifAbsent:",0
,function(anIndex,aBlock){var self=this;
var value = self[anIndex - 1];
	if(value === undefined) { return aBlock.value(); }
	return value;
return self;}
,"accessing",unescape("at%3A%20anIndex%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver%20at%20anIndex%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20indexing%27%7D"));
smalltalk.bind(smalltalk.Array,"at:put:",0
,function(anIndex,anObject){var self=this;
return self[anIndex - 1] = anObject;
return self;}
,"accessing",unescape("at%3A%20anIndex%20put%3A%20anObject%0D%0A%09%22%20Set%20the%20contents%20of%20the%20receiver%20at%20anIndex.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20indexing%27%7D"));
smalltalk.bind(smalltalk.Array,"do:",0
,function(aBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
for(var i=0;i<self.length;i++){fn(self[i]);};
return self;}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27s8%27%091%20to%3A%20self%20size%20do%3A%20%5B%3Ai%7C%0D%0A%09%09aBlock%20value%3A%20%28self%20at%3A%20i%29%0D%0A%09%5D%27%20doc%27native%20iteration%27%7D"));
smalltalk.bind(smalltalk.Array,"isArray",0
,function(){var self=this;
return (nil.is_eqeq_(self.$klass,smalltalk.Array));}
,"testing",unescape("isArray%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20an%20Array.%20%22%0D%0A%0D%0A%09%5Eself%20class%20%3D%3D%20Array"));
smalltalk.bind(smalltalk.Array,"addAll:",0
,function(aCollection){var self=this;
self.push.apply(self, aCollection.asArray());
return aCollection;}
,"adding",unescape("addAll%3A%20aCollection%0D%0A%09%22%20Add%20objects%20in%20aCollection%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20composing%27%7D.%0D%0A%09%5EaCollection"));
smalltalk.bind(smalltalk.Array,"shallowCopy",0
,function(){var self=this;
var result=nil;
result = self.slice(0);
return result;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09%7B%27%27%20doc%27native%20copy%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Array,"deepCopy",0
,function(){var self=this;
return self.collect_((function(each){return each.deepCopy();}));}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20collect%3A%20%5B%3Aeach%20%7C%20each%20deepCopy%20%5D"));
smalltalk.bind(smalltalk.Array,"copyFrom:to:",0
,function(anIndex,anotherIndex){var self=this;
var result=nil;
result=self.species().$$new();
anIndex.to_do_(anotherIndex,(function(each){return result.add_(self.at_(each));}));
return result;}
,"copying",unescape("copyFrom%3A%20anIndex%20to%3A%20anotherIndex%0D%0A%09%22%20Return%20the%20receiver%20contents%20in%20the%20interval.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20species%20new.%0D%0A%09anIndex%20to%3A%20anotherIndex%20do%3A%20%5B%3Aeach%20%7C%20result%20add%3A%20%28self%20at%3A%20each%29%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Array,"copyWithAll:",0
,function(aCollection){var self=this;
return self.concat(aCollection.asArray());
return self;}
,"copying",unescape("copyWithAll%3A%20aCollection%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20with%20all%20contents%20of%20aCollection.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20composition%27%7D"));
smalltalk.bind(smalltalk.Array,",","_comma"
,function(aCollection){var self=this;
return self.concat(aCollection.asArray());
return self;}
,"copying",unescape("%2C%20aCollection%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20with%20all%20contents%20of%20aCollection.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20composition%27%7D"));
smalltalk.bind(smalltalk.Array,"join:",0
,function(aString){var self=this;
return self.join(aString);}
,"composing",unescape("join%3A%20aString%0D%0A%09%22%20Return%20a%20string%20with%20the%20receiver%27s%20contents%20separated%20by%20aString.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20composition%27%7D"));
smalltalk.bind(smalltalk.Array,"compose:with:",0
,function(aBlock,aString){var self=this;
return self.collect_thenSelect_(aBlock,(function(each){return (nil.isNil_(each)===false);})).join_(aString);}
,"composing",unescape("compose%3A%20aBlock%20with%3A%20aString%0D%0A%09%22%20Return%20a%20string%20composed%20with%20the%20result%20of%20evaluating%20aBlock%20into%20receiver%27s%20contents.%20%22%0D%0A%0D%0A%09%5E%28self%09collect%3A%20aBlock%0D%0A%09%09thenSelect%3A%20%5B%3Aeach%7C%20each%20notNil%20%5D%29%0D%0A%09%09join%3A%20aString"));
smalltalk.bind(smalltalk.Array,"compose:",0
,function(aBlock){var self=this;
return self.compose_with_(aBlock,"");}
,"composing",unescape("compose%3A%20aBlock%0D%0A%09%22%20Return%20a%20string%20composed%20with%20the%20result%20of%20evaluating%20aBlock%20into%20receiver%27s%20contents.%20%22%0D%0A%0D%0A%09%5Eself%20compose%3A%20aBlock%20with%3A%20%27%27"));
smalltalk.bind(smalltalk.Array,"asNative:",0
,function(lang){var self=this;
var list=nil;
list=self.collect_((function(each){return each.asNative_(lang);})).join_(unescape("%2C%20"));
if (lang._eq("lua")){return "smalltalk:newArray"._comma(unescape("%28"))._comma(unescape("%7B"))._comma(list)._comma(unescape("%7D"))._comma(unescape("%29"));};;
return unescape("%5B")._comma(list)._comma(unescape("%5D"));}
,"converting",unescape("asNative%3A%20lang%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20list%20%7C%0D%0A%09list%20%3A%3D%20%28self%20collect%3A%20%5B%3Aeach%20%7C%20each%20asNative%3A%20lang%20%5D%29%20join%3A%20%27%2C%20%27.%0D%0A%09lang%20%3D%20%23lua%20ifTrue%3A%20%5B%20%5E%27smalltalk%3AnewArray%27%20%2C%24%28%20%2C%24%7B%20%2Clist%20%2C%24%7D%20%2C%24%29%20%5D.%0D%0A%09%5E%24%5B%20%2Clist%20%2C%24%5D"));
smalltalk.bind(smalltalk.Array,"sort:",0
,function(aBlock){var self=this;
var fn=nil;
fn=self.callableOf_(aBlock);
if (nil.isNil_(aBlock)) { self.sort(); return self }
	var criteria = function(a, b) {
		if(fn(a,b)) {return -1} else {return 1}
	}
	self.sort(criteria);
return self;}
,"sorting",unescape("sort%3A%20aBlock%0D%0A%09%22%20Sort%20%28inplace%29%20the%20receiver%27s%20contents%20with%20aBlock.%20%22%0D%0A%0D%0A%09%7C%20fn%20%7C%20fn%20%3A%3D%20self%20callableOf%3A%20aBlock.%0D%0A%09%7B%27%27%20doc%27native%20sorting%27%7D"));
smalltalk.bind(smalltalk.Array,"asArray",0
,function(){var self=this;
return self;}
,"converting",unescape("asArray%0D%0A%09%22%20Return%20an%20array%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Array,"remove:ifAbsent:",0
,function(anObject,aBlock){var self=this;
for(var i=0;i<self.length;i++) {
		if(self[i] == anObject) {
			self.splice(i,1); return anObject;
		}
	}; return aBlock.value();
return self;}
,"removing",unescape("remove%3A%20anObject%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Remove%20anObject%20from%20the%20receiver.%0D%0A%09Return%20the%20result%20of%20evaluating%20aBlock%20if%20anObject%20is%20not%20in%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20remotion%27%7D"));
smalltalk.bind(smalltalk.Array,"add:",0
,function(anObject){var self=this;
self.push(anObject);
return anObject;}
,"adding",unescape("add%3A%20anObject%0D%0A%09%22%20Add%20anObject%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20addition%27%7D.%0D%0A%09%5EanObject"));
smalltalk.bind(smalltalk.Array,"addFirst:",0
,function(anObject){var self=this;
self.splice(0,0,anObject);
return anObject;}
,"adding",unescape("addFirst%3A%20anObject%0D%0A%09%22%20Add%20anObject%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20addition%27%7D.%0D%0A%09%5EanObject"));
smalltalk.bind(smalltalk.Array,"addLast:",0
,function(anObject){var self=this;
self.push(anObject);
return anObject;}
,"adding",unescape("addLast%3A%20anObject%0D%0A%09%22%20Add%20anObject%20to%20the%20receiver%20%28at%20end%29.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20addition%27%7D.%0D%0A%09%5EanObject"));
smalltalk.bind(smalltalk.Array,"removeAll",0
,function(){var self=this;
self.splice(0,self.size());
return self;}
,"removing",unescape("removeAll%0D%0A%09%22%20Remove%20all%20the%20objects%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20remotion%27%7D"));
smalltalk.bind(smalltalk.Array,"removeFirst",0
,function(){var self=this;
var result=nil;
result=self.first();
self.splice(0,1);
return result;}
,"removing",unescape("removeFirst%0D%0A%09%22%20Remove%20the%20first%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20first.%0D%0A%09%7B%27%27%20doc%27native%20remotion%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Array,"removeIndex:",0
,function(index){var self=this;
self.splice(index-1,1);
return self;}
,"removing",unescape("removeIndex%3A%20index%0D%0A%09%22%20Remove%20the%20element%20at%20index.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20remotion%27%7D"));
smalltalk.bind(smalltalk.Array,"removeLast",0
,function(){var self=this;
return self.pop();}
,"removing",unescape("removeLast%0D%0A%09%22%20Remove%20the%20last%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20remotion%27%7D"));
smalltalk.bind(smalltalk.Array,"sorted",0
,function(){var self=this;
return self.copy().sort_(nil);}
,"sorting",unescape("sorted%0D%0A%09%22%20Return%20a%20sorted%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20copy%20sort%3A%20nil"));
smalltalk.bind(smalltalk.Array,"sorted:",0
,function(aBlock){var self=this;
return self.copy().sort_(aBlock);}
,"sorting",unescape("sorted%3A%20aBlock%0D%0A%09%22%20Return%20a%20sorted%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20copy%20sort%3A%20aBlock"));
smalltalk.bind(smalltalk.Array,"atAllPut:",0
,function(anObject){var self=this;
for(var i=0;i<self.length;i++){ self[i] = anObject; };
return self;}
,"accessing",unescape("atAllPut%3A%20anObject%0D%0A%09%22%20Replace%20all%20elements%20of%20the%20receiver%20with%20anObject.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20operation%27%7D"));
smalltalk.bind(smalltalk.Array,"first",0
,function(){var self=this;
var result=self[0];
	if (typeof(result)!="undefined") return result;
return self.at_((1));}
,"accessing",unescape("first%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20indexing%27%7D.%0D%0A%09%5Eself%20at%3A%201"));
smalltalk.bind(smalltalk.Array,"second",0
,function(){var self=this;
var result=self[1];
	if (typeof(result)!="undefined") return result;
return self.at_((2));}
,"accessing",unescape("second%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20indexing%27%7D.%0D%0A%09%5Eself%20at%3A%202"));
smalltalk.bind(smalltalk.Array,"third",0
,function(){var self=this;
var result=self[2];
	if (typeof(result)!="undefined") return result;
return self.at_((3));}
,"accessing",unescape("third%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20indexing%27%7D.%0D%0A%09%5Eself%20at%3A%203"));
smalltalk.bind(smalltalk.Array,"fourth",0
,function(){var self=this;
	var result=self[3];
	if (typeof(result)!="undefined") return result;
return self.at_((4));}
,"accessing",unescape("fourth%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20indexing%27%7D.%0D%0A%09%5Eself%20at%3A%204"));
smalltalk.bind(smalltalk.Array,"fifth",0
,function(){var self=this;
var result=self[4];
	if (typeof(result)!="undefined") return result;
return self.at_((5));}
,"accessing",unescape("fifth%0D%0A%09%22%20Return%20the%20fifth%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20indexing%27%7D.%0D%0A%09%5Eself%20at%3A%205"));
smalltalk.bind(smalltalk.Array,"sixth",0
,function(){var self=this;
var result=self[5];
	if (typeof(result)!="undefined") return result;
return self.at_((6));}
,"accessing",unescape("sixth%0D%0A%09%22%20Return%20the%20sixth%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20indexing%27%7D.%0D%0A%09%5Eself%20at%3A%206"));
smalltalk.bind(smalltalk.Array,"withIndexDo:",0
,function(aBlock){var self=this;
var index=nil;
index=(1);
self.do_((function(each){aBlock.value_value_(each,index);return index=index._plus((1));}));
return self;}
,"enumerating",unescape("withIndexDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20all%20the%20index%20positions%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09index%20%3A%3D%201.%0D%0A%09self%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09aBlock%20value%3A%20each%20value%3A%20index.%0D%0A%09%09index%20%3A%3D%20index%20+%201%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Array,"printString",0
,function(){var $s8Ret$={name:"stReturn"};try{var self=this;
var result=nil;
if (self.isEmpty()){return unescape("%23%28%29");};;
result=unescape("%23%28%20");
self.do_((function(each){if (result.size()._gt((10000))){(function(){$s8Ret$.result=(function(){return result._comma(unescape("...%29"))})();throw($s8Ret$)})();};;return result=result._comma(((self.isObject_(each)).mustBeBoolean()==true ? (function(){return each.printString();})() : (function(){return each;})()))._comma(" ");}));
(function(){$s8Ret$.result=(function(){return result._comma(unescape("%29"))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"printing",unescape("printString%0D%0A%09%22%20DEBUG%20-%20Temporary%20printing%20method%20%28must%20check%20for%20recursion%29.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09self%20isEmpty%20ifTrue%3A%20%5B%20%5E%27%23%28%29%27%20%5D.%0D%0A%09result%20%3A%3D%20%27%23%28%20%27.%0D%0A%09self%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09result%20size%20%3E%2010000%20ifTrue%3A%20%5B%20%5Eresult%2C%27...%29%27%20%5D.%0D%0A%09%09result%20%3A%3D%20result%20%2C%20%28%0D%0A%09%09%09%28self%20isObject%3A%20each%29%0D%0A%09%09%09ifTrue%3A%20%5B%20each%20printString%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20each%20%5D%29%20%2C%27%20%27%0D%0A%09%5D.%0D%0A%09%5Eresult%2C%27%29%27"));
smalltalk.bind(smalltalk.Array,"asLiteral",0
,function(){var self=this;
var literals=nil;
literals=self.collect_((function(each){return ((self.isObject_(each)).mustBeBoolean()==true ? (function(){return each.asLiteral();})() : (function(){return each.toString();})());}));
return unescape("%23%28")._comma(literals.join_(" "))._comma(unescape("%29"));}
,"converting",unescape("asLiteral%0D%0A%09%22%20Return%20the%20receiver%20in%20literal%20notation.%20%22%0D%0A%0D%0A%09%7C%20literals%20%7C%0D%0A%09literals%20%3A%3D%20self%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09%28self%20isObject%3A%20each%29%0D%0A%09%09ifTrue%3A%20%5B%20each%20asLiteral%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20each%20toString%20%5D%0D%0A%09%5D.%0D%0A%09%5E%27%23%28%27%20%2C%28literals%20join%3A%20%24%20%29%20%2C%24%29"));
smalltalk.bind(smalltalk.Array,"asJSONObject",0
,function(){var self=this;
var result=nil;

result=self.collect_((function(each){return ((self.isObject_(each)).mustBeBoolean()==true ? (function(){return each.asJSONObject();})() : (function(){return each;})());}));

return result;}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09%7B%27self%27%20doc%27native%20conversion%27%7D.%0D%0A%09result%20%3A%3D%20self%20collect%3A%20%5B%3Aeach%7C%20%0D%0A%09%09%28self%20isObject%3A%20each%29%0D%0A%09%09ifTrue%3A%20%5B%20each%20asJSONObject%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20each%20%5D%0D%0A%09%5D.%0D%0A%09%7B%27self%27%20doc%27native%20closing%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Array,"json",0
,function(){var self=this;
var result=nil;

result=self.collect_((function(each){return ((self.isObject_(each)).mustBeBoolean()==true ? (function(){return each.json();})() : (function(){return each;})());}));

return result;}
,"converting",unescape("json%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09%7B%27self%27%20doc%27native%20conversion%27%7D.%0D%0A%09result%20%3A%3D%20self%20collect%3A%20%5B%3Aeach%7C%20%0D%0A%09%09%28self%20isObject%3A%20each%29%0D%0A%09%09ifTrue%3A%20%5B%20each%20json%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20each%20%5D%0D%0A%09%5D.%0D%0A%09%7B%27self%27%20doc%27native%20closing%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Array,"emmitAllIn:",0
,function(folder){var self=this;
return self.collect_((function(each){return folder._comma(each);})).emmitAll();}
,"Builders-emmiting",unescape("emmitAllIn%3A%20folder%0D%0A%09%22%20Emmit%28fileIn%20%26%20save%20generated%20code%20of%29%20the%20fileNames%20in%20the%20receiver.%0D%0A%09Note%20the%20receiver%20MUST%20include%20the%20file%20separator%20at%20end%0D%0A%09%22%0D%0A%0D%0A%09%5E%28self%20collect%3A%20%5B%3Aeach%7C%20folder%20%2C%20each%20%5D%29%20emmitAll"));
smalltalk.bind(smalltalk.Array,"emmitAll",0
,function(){var self=this;
if (self.isEmpty()){return self;};;
(function(){self.emmit_(smalltalk.Array.with_(self.removeFirst()));return self.emmitAll();}).valueDeferred_((500));
return self;}
,"Builders-emmiting",unescape("emmitAll%0D%0A%09%22%20Deferred%20%28consuming%29%20emmit%20of%20fileNames%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20isEmpty%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09%5B%09self%20emmit%3A%20%28Array%20with%3A%20self%20removeFirst%29.%0D%0A%09%09self%20emmitAll%0D%0A%09%5D%20valueDeferred%3A%20500"));
smalltalk.bind(smalltalk.Array,"handles",0
,function(){var self=this;
return self.collect_((function(each){return each.handle();}));}
,"Natives-converting",unescape("handles%0D%0A%09%22%20Return%20the%20result%20of%20collecting%20the%20handles%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20collect%3A%20%5B%3Aeach%7C%20each%20handle%20%5D"));
smalltalk.bind(smalltalk.Array,"asLua",0
,function(){var self=this;
return "smalltalk:newArray"._comma(unescape("%28"))._comma(unescape("%7B"))._comma(self.collect_((function(each){return each.asLua();})).join_(unescape("%2C%20")))._comma(unescape("%7D"))._comma(unescape("%29"));}
,"Lua-lua",unescape("asLua%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%27smalltalk%3AnewArray%27%20%2C%24%28%20%2C%24%7B%0D%0A%09%09%2C%28%28self%20collect%3A%20%5B%3Aeach%20%7C%20each%20asLua%20%5D%29%20join%3A%20%27%2C%20%27%29%0D%0A%09%2C%24%7D%20%2C%24%29%20"));
smalltalk.bind(smalltalk.Array,"__tostring",0
,function(){var self=this;
return self.asLiteral();}
,"Lua-printing",unescape("__tostring%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20asLiteral"));
smalltalk.bind(smalltalk.Array,"addAllLua:",0
,function(natives){var self=this;
if (self.typeOf_(natives)._eq("userdata")){var count=nil;
count=smalltalk.JavaArray.getLength_(smalltalk.Lua._at(natives));
(1).to_do_(count,(function(i){return self.add_(smalltalk.JavaArray.nativeClass().handle().get(natives,i._minus((1))));}));
return self;};;
self.addAll_(natives);
return self;}
,"Lua-private",unescape("addAllLua%3A%20natives%0D%0A%09%22%20Private%20-%20Add%20the%20native%20values%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%28self%20typeOf%3A%20natives%29%20%3D%20%23userdata%20ifTrue%3A%20%5B%20%7C%20count%20%7C%0D%0A%09%09count%20%3A%3D%20JavaArray%20getLength%3A%20Lua@natives.%0D%0A%09%091%20to%3A%20count%20do%3A%20%5B%3Ai%7C%0D%0A%09%09%09self%20add%3A%20%28JavaArray%20nativeClass%20handle%0D%0A%09%09%09%09%23get%3A%20natives%20index%3A%20i%20-%201%29.%0D%0A%09%09%5D.%0D%0A%09%09%5Eself%0D%0A%09%5D.%0D%0A%09%7B%27%27js%27self.addAll_%28natives%29%27%7D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.ArrayedContents,"contents",0
,function(){var self=this;
if ((nil.isNil_(self["@contents"]))){self["@contents"]=smalltalk.Array.$$new();};;
return self["@contents"];}
,"private",unescape("contents%0D%0A%09%22%20Private%20-%20Return%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09contents%20isNil%20ifTrue%3A%20%5B%20contents%20%3A%3D%20Array%20new%20%5D.%0D%0A%09%5Econtents"));
smalltalk.bind(smalltalk.ArrayedContents,"add:",0
,function(anObject){var self=this;
return self.contents().add_(anObject);}
,"adding",unescape("add%3A%20anObject%0D%0A%09%22%20Add%20anObject%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20add%3A%20anObject"));
smalltalk.bind(smalltalk.ArrayedContents,"at:ifAbsent:",0
,function(anIndex,aBlock){var self=this;
return self.contents().at_ifAbsent_(anIndex,aBlock);}
,"accessing",unescape("at%3A%20anIndex%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver%20at%20anIndex%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20at%3A%20anIndex%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.ArrayedContents,"at:put:",0
,function(anIndex,anObject){var self=this;
return self.contents().at_put_(anIndex,anObject);}
,"accessing",unescape("at%3A%20anIndex%20put%3A%20anObject%0D%0A%09%22%20Set%20the%20contents%20of%20the%20receiver%20at%20anIndex.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20at%3A%20anIndex%20put%3A%20anObject"));
smalltalk.bind(smalltalk.ArrayedContents,"atAllPut:",0
,function(anObject){var self=this;
return self.contents().atAllPut_(anObject);}
,"accessing",unescape("atAllPut%3A%20anObject%0D%0A%09%22%20Replace%20all%20elements%20of%20the%20receiver%20with%20anObject.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20atAllPut%3A%20anObject"));
smalltalk.bind(smalltalk.ArrayedContents,"copyFrom:to:",0
,function(anIndex,anotherIndex){var self=this;
var result=nil;
result=self.species().$$new();
anIndex.to_do_(anotherIndex,(function(each){return result.add_(self.at_(each));}));
return result;}
,"copying",unescape("copyFrom%3A%20anIndex%20to%3A%20anotherIndex%0D%0A%09%22%20Return%20the%20receiver%20contents%20in%20the%20interval.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20species%20new.%0D%0A%09anIndex%20to%3A%20anotherIndex%20do%3A%20%5B%3Aeach%20%7C%20result%20add%3A%20%28self%20at%3A%20each%29%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.ArrayedContents,"indexOf:ifAbsent:",0
,function(anObject,aBlock){var self=this;
return self.contents().indexOf_ifAbsent_(anObject,aBlock);}
,"testing",unescape("indexOf%3A%20anObject%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20position%20of%20anObject%20in%20the%20receiver%2C%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20indexOf%3A%20anObject%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.ArrayedContents,"detect:ifNone:",0
,function(aBlock,anotherBlock){var self=this;
return self.contents().detect_ifNone_(aBlock,anotherBlock);}
,"enumerating",unescape("detect%3A%20aBlock%20ifNone%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver%20that%20return%20true%20when%20evaluated%20with%20aBlock%20or%20the%20result%20of%20evaluating%20anotherBlock.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20detect%3A%20aBlock%20ifNone%3A%20anotherBlock"));
smalltalk.bind(smalltalk.ArrayedContents,"do:",0
,function(aBlock){var self=this;
return self.contents().do_(aBlock);}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20do%3A%20aBlock"));
smalltalk.bind(smalltalk.ArrayedContents,"includes:",0
,function(anObject){var self=this;
return self.contents().includes_(anObject);}
,"testing",unescape("includes%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20includes%3A%20anObject"));
smalltalk.bind(smalltalk.ArrayedContents,"isEmpty",0
,function(){var self=this;
return self.contents().isEmpty();}
,"testing",unescape("isEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20empty.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20isEmpty"));
smalltalk.bind(smalltalk.ArrayedContents,"occurrencesOf:",0
,function(anObject){var self=this;
return self.contents().occurrencesOf_(anObject);}
,"counting",unescape("occurrencesOf%3A%20anObject%0D%0A%09%22%20Return%20the%20occurrences%20of%20anObject%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20occurrencesOf%3A%20anObject"));
smalltalk.bind(smalltalk.ArrayedContents,"reverseDo:",0
,function(aBlock){var self=this;
return self.contents().reverseDo_(aBlock);}
,"enumerating",unescape("reverseDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver%20in%20reverse%20order%20of%20iteration.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20reverseDo%3A%20aBlock"));
smalltalk.bind(smalltalk.ArrayedContents,"reversed",0
,function(){var self=this;
return self.$klass.withAll_(self.contents().reversed());}
,"copying",unescape("reversed%0D%0A%09%22%20Return%20an%20array%20with%20the%20contents%20of%20the%20receiver%20in%20reverse%20order.%20%22%0D%0A%0D%0A%09%5Eself%20class%20withAll%3A%20self%20contents%20reversed"));
smalltalk.bind(smalltalk.ArrayedContents,"remove:ifAbsent:",0
,function(anObject,aBlock){var self=this;
return self.contents().remove_ifAbsent_(anObject,aBlock);}
,"removing",unescape("remove%3A%20anObject%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Remove%20anObject%20from%20the%20receiver.%0D%0A%09Return%20the%20result%20of%20evaluating%20aBlock%20if%20anObject%20is%20not%20in%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20contents%20remove%3A%20anObject%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.ArrayedContents,"removeAll",0
,function(){var self=this;
self["@contents"]=nil;
return self;}
,"removing",unescape("removeAll%0D%0A%09%22%20Remove%20all%20the%20objects%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09contents%20%3A%3D%20nil"));
smalltalk.bind(smalltalk.ArrayedContents,"removeFirst",0
,function(){var self=this;
return self.contents().removeFirst();}
,"removing",unescape("removeFirst%0D%0A%09%22%20Remove%20the%20first%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20removeFirst"));
smalltalk.bind(smalltalk.ArrayedContents,"removeLast",0
,function(){var self=this;
return self.contents().removeLast();}
,"removing",unescape("removeLast%0D%0A%09%22%20Remove%20the%20last%20object%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20removeLast"));
smalltalk.bind(smalltalk.ArrayedContents,"removeIndex:",0
,function(index){var self=this;
return self.contents().removeIndex_(index);}
,"removing",unescape("removeIndex%3A%20index%0D%0A%09%22%20Remove%20the%20element%20at%20index.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20removeIndex%3A%20index"));
smalltalk.bind(smalltalk.ArrayedContents,"size",0
,function(){var self=this;
return self.contents().size();}
,"accesing",unescape("size%0D%0A%09%22%20Return%20the%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20size"));
smalltalk.bind(smalltalk.ArrayedContents,"withIndexDo:",0
,function(aBlock){var self=this;
return self.contents().withIndexDo_(aBlock);}
,"enumerating",unescape("withIndexDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20all%20the%20index%20positions%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20contents%20withIndexDo%3A%20aBlock"));
smalltalk.bind(smalltalk.ComposedAssociations.$klass,",","_comma"
,function(anObject){var self=this;
return self.$$new()._comma(anObject);}
,"composing",unescape("%2C%20anObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20with%20the%20argument.%20%22%0D%0A%0D%0A%09%5Eself%20new%20%2CanObject"));
smalltalk.bind(smalltalk.ComposedAssociations,",","_comma"
,function(anObject){var self=this;
if (anObject.isAssociation()){return (function($rec){$rec.add_(anObject);return $rec.yourself();})(self);};;
return (((nil.isNil_(anObject))).mustBeBoolean()==true ? (function(){return self;})() : (function(){return smalltalk.superImplementor(smalltalk.ComposedAssociations,"_comma").apply(self, [anObject]);})());}
,"composing",unescape("%2C%20anObject%0D%0A%09%22%20Return%20the%20receiver%20composed%20with%20the%20argument.%20%22%0D%0A%0D%0A%09anObject%20isAssociation%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20add%3A%20anObject%3B%20yourself%0D%0A%09%5D.%0D%0A%09%5EanObject%20isNil%20ifTrue%3A%20%5B%20self%20%5D%20ifFalse%3A%20%5B%20super%20%2C%20anObject%20%5D"));
smalltalk.bind(smalltalk.ComposedAssociations,"hashObject",0
,function(){var self=this;
return {};}
,"private",unescape("hashObject%0D%0A%09%22%20Private%20-%20Return%20an%20empty%20hash%20object.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20instantiation%27%7D"));
smalltalk.bind(smalltalk.ComposedAssociations,"install:value:into:",0
,function(key,value,jsObject){var $s8Ret$={name:"stReturn"};try{var self=this;
if (self.isObject_(key).and_((function(){return key.isArray();}))){if (key.size()._eq((1))){return self.install_value_into_(key.first(),value,jsObject);};;
((key.size()._gt((1))).mustBeBoolean()==true ? (function(){var target=nil;
target=((function(){var $1$=(jsObject[key.first()]);if(nil.isNil_($1$))return ((function(){return (jsObject[self.toString_(key.first())]=(self.hashObject()));}).value());return $1$;})());return (function(){$s8Ret$.result=(function(){return self.install_value_into_(key.copyFrom_to_((2),key.size()),value,target)})();throw($s8Ret$)})();})() : nil);};;
(function(){$s8Ret$.result=(function(){return (jsObject[self.toString_(key)]=(value))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"private",unescape("install%3A%20key%20value%3A%20value%20into%3A%20jsObject%0D%0A%09%22%20Private%20-%20Install%20a%20key/value%20pair%20into%20jsObject.%20%22%0D%0A%0D%0A%09%28%28self%20isObject%3A%20key%29%20and%3A%20%5B%20key%20isArray%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09key%20size%20%3D%201%20ifTrue%3A%20%5B%20%5Eself%20install%3A%20key%20first%20value%3A%20value%20into%3A%20jsObject%20%5D.%0D%0A%09%09key%20size%20%3E%201%20ifTrue%3A%20%5B%20%7C%20target%20%7C%0D%0A%09%09%09target%20%3A%3D%20jsObject%20basicAt%3A%20key%20first%20ifAbsent%3A%20%5B%0D%0A%09%09%09%09jsObject%20basicAt%3A%20%28self%20toString%3A%20key%20first%29%20put%3A%20self%20hashObject%0D%0A%09%09%09%5D.%0D%0A%09%09%09%5Eself%20install%3A%20%28key%20copyFrom%3A%202%20to%3A%20key%20size%29%20value%3A%20value%20into%3A%20target%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5EjsObject%0D%0A%09%09basicAt%3A%20%28self%20toString%3A%20key%29%20%22it%20is%20IMPORTANT%20to%20send%20%23toString%20here%20%28v8%7B4.7.0%7D%20bug%20will%20expose%20%7Bobject%7D%20instead%20of%20%7Bstring%7D%20Ale-14/oct/2015%29%22%0D%0A%09%09put%3A%20value"));
smalltalk.bind(smalltalk.ComposedAssociations,"asJSONObject",0
,function(){var self=this;
var result=nil;
var aValue=nil;
result=self.hashObject();
self.do_((function(assoc){aValue=((self.isObject_(assoc.value())).mustBeBoolean()==true ? (function(){return assoc.value().json();})() : (function(){return assoc.value();})());return self.install_value_into_(assoc.key(),aValue,result);}));
return result;}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%7C%20result%20aValue%20%7C%0D%0A%09result%20%3A%3D%20self%20hashObject.%0D%0A%09self%20do%3A%20%5B%3Aassoc%20%7C%0D%0A%09%09aValue%20%3A%3D%20%28self%20isObject%3A%20assoc%20value%29%0D%0A%09%09%09ifTrue%3A%20%5B%20assoc%20value%20json%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20assoc%20value%20%5D.%0D%0A%09%09self%20install%3A%20assoc%20key%20value%3A%20aValue%20into%3A%20result%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.ComposedAssociations,"handle",0
,function(){var self=this;
return self.json();}
,"converting",unescape("handle%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20native%20object.%20%22%0D%0A%0D%0A%09%5Eself%20json"));
smalltalk.bind(smalltalk.ComposedAssociations,"associationAt:ifAbsent:",0
,function(aKey,aBlock){var $s8Ret$={name:"stReturn"};try{var self=this;
self.reverseDo_((function(assoc){var value=nil;
if (assoc.key()._eq(aKey)){(function(){$s8Ret$.result=(function(){return assoc})();throw($s8Ret$)})();};;value=assoc.value();return ((self.isObject_(value).and_((function(){return smalltalk.is_kindOf_(value,(self.$klass));}))).mustBeBoolean()==true ? (function(){value=value.associationAt_ifAbsent_(aKey,(function(){return nil;}));return (((nil.isNil_(value)===false)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return value})();throw($s8Ret$)})();})() : nil);})() : nil);}));
(function(){$s8Ret$.result=(function(){return aBlock.evaluate()})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("associationAt%3A%20aKey%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20association%20at%20aKey%20%28or%20the%20result%20of%20evaluating%20aBlock%29.%0D%0A%09Note%3A%20the%20search%20will%20introspect%20in%20case%20of%20composite%20values.%0D%0A%09%22%0D%0A%0D%0A%09self%20reverseDo%3A%20%5B%3Aassoc%7C%20%7C%20value%20%7C%0D%0A%09%09assoc%20key%20%3D%20aKey%20ifTrue%3A%20%5B%20%5Eassoc%20%5D.%0D%0A%09%09value%20%3A%3D%20assoc%20value.%0D%0A%09%09%28%28self%20isObject%3A%20value%29%20and%3A%20%5Bvalue%20isKindOf%3A%20self%20class%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%09value%20%3A%3D%20value%20associationAt%3A%20aKey%20ifAbsent%3A%20%5B%5D.%0D%0A%09%09%09value%20notNil%20ifTrue%3A%20%5B%20%5Evalue%20%5D.%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5EaBlock%20evaluate"));
smalltalk.bind(smalltalk.ComposedAssociations,"@","_at"
,function(aKey){var $s8Ret$={name:"stReturn"};try{var self=this;
var assoc=nil;
assoc=self.associationAt_ifAbsent_(aKey,(function(){return (function(){$s8Ret$.result=(function(){return nil})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return assoc.value()})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("@%20aKey%0D%0A%09%22%20Return%20the%20value%20at%20aKey%20%28or%20nil%29.%0D%0A%09Note%3A%20the%20search%20will%20introspect%20in%20case%20of%20composite%20values.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20assoc%20%7C%0D%0A%09assoc%20%3A%3D%20self%20associationAt%3A%20aKey%20ifAbsent%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%5Eassoc%20value"));
smalltalk.bind(smalltalk.ComposedObjects,",","_comma"
,function(anObject){var self=this;
if ((nil.isNil_(anObject))){return self;};;
if (anObject.isArray().or_((function(){return smalltalk.responds_to_(anObject,("asArray"));}))){anObject.asArray().do_((function(each){return self._comma(each);}));
return self;};;
if ((nil.isNil_(self["@species"])===false).and_((function(){return smalltalk.is_kindOf_(anObject,(self["@species"])).not();}))){self.error_("Invalid object to compose");};;
self.add_(anObject);
return self;}
,"composing",unescape("%2C%20anObject%0D%0A%09%22%20Return%20the%20receiver%20composed%20with%20the%20argument.%20%22%0D%0A%0D%0A%09anObject%20isNil%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09%28anObject%20isArray%20or%3A%20%5B%20anObject%20respondsTo%3A%20%23asArray%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09anObject%20asArray%20do%3A%20%5B%3Aeach%7C%20self%20%2C%20each%20%5D.%0D%0A%09%09%5Eself%0D%0A%09%5D.%0D%0A%0D%0A%09%28species%20notNil%20and%3A%20%5B%20%28anObject%20isKindOf%3A%20species%29%20not%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20error%3A%20%27Invalid%20object%20to%20compose%27%0D%0A%09%5D.%0D%0A%0D%0A%09self%20add%3A%20anObject."));
smalltalk.bind(smalltalk.ComposedObjects,"species:",0
,function(aBehavior){var self=this;
self["@species"]=aBehavior;
return self;}
,"accessing",unescape("species%3A%20aBehavior%0D%0A%09%22%20Set%20the%20species%20to%20compose%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09species%20%3A%3D%20aBehavior"));
smalltalk.bind(smalltalk.Interval.$klass,"from:to:by:value:",0
,function(start,end,step,aBlock){var self=this;
return self.basicNew().initialize_to_by_value_(start,end,step,aBlock);}
,"instantiation",unescape("from%3A%20start%20to%3A%20end%20by%3A%20step%20value%3A%20aBlock%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicNew%20initialize%3A%20start%20to%3A%20end%20by%3A%20step%20value%3A%20aBlock"));
smalltalk.bind(smalltalk.Interval,"initialize:to:by:value:",0
,function(from,to,increment,aBlock){var self=this;
self["@start"]=from;
self["@end"]=to;
self["@step"]=increment;
self["@value"]=aBlock;
return self;}
,"initialize",unescape("initialize%3A%20from%20to%3A%20to%20by%3A%20increment%20value%3A%20aBlock%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%20%22%0D%0A%0D%0A%09start%20%3A%3D%20from.%0D%0A%09end%20%3A%3D%20to.%0D%0A%09step%20%3A%3D%20increment.%0D%0A%09value%20%3A%3D%20aBlock"));
smalltalk.bind(smalltalk.Interval,"start",0
,function(){var self=this;
return self["@start"];}
,"accessing",unescape("start%0D%0A%09%22%20Return%20the%20start%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Estart%20"));
smalltalk.bind(smalltalk.Interval,"end",0
,function(){var self=this;
return self["@end"];}
,"accessing",unescape("end%0D%0A%09%22%20Return%20the%20end%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eend%20"));
smalltalk.bind(smalltalk.Interval,"step",0
,function(){var self=this;
return self["@step"];}
,"accessing",unescape("step%0D%0A%09%22%20Return%20the%20step%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Estep%20"));
smalltalk.bind(smalltalk.Interval,"start:",0
,function(from){var self=this;
self["@start"]=from;
return self;}
,"accessing",unescape("start%3A%20from%0D%0A%09%22%20Set%20the%20start%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09start%20%3A%3D%20from%20"));
smalltalk.bind(smalltalk.Interval,"end:",0
,function(to){var self=this;
self["@end"]=to;
return self;}
,"accessing",unescape("end%3A%20to%0D%0A%09%22%20Set%20the%20end%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09end%20%3A%3D%20to%20"));
smalltalk.bind(smalltalk.Interval,"step:",0
,function(increment){var self=this;
self["@step"]=increment;
return self;}
,"accessing",unescape("step%3A%20increment%0D%0A%09%22%20Set%20the%20step%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09step%20%3A%3D%20increment%20"));
smalltalk.bind(smalltalk.Interval,"species",0
,function(){var self=this;
return smalltalk.Array;}
,"species",unescape("species%0D%0A%09%22%20Return%20the%20species%20to%20instantiate%20objects%20like%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EArray%20"));
smalltalk.bind(smalltalk.Interval,"subclassResponsibility:",0
,function(aSelector){var self=this;
return self.invalidMessage_(aSelector);}
,"error",unescape("subclassResponsibility%3A%20aSelector%0D%0A%09%22%20Private%20-%20The%20concrete%20implementation%20on%20the%20receiver%20is%20not%20complete.%0D%0A%09Note%3A%20Unimplemented%20methods%20handle%20invalid%20messages%20sent%20to%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20aSelector"));
smalltalk.bind(smalltalk.Interval,"size",0
,function(){var self=this;
return (0).max_(self["@end"]._minus(self["@start"])._slash_slash(self["@step"])._plus((1)));}
,"accessing",unescape("size%0D%0A%09%22%20Return%20the%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E0%20max%3A%20end%20-%20start%20//%20step%20+%201"));
smalltalk.bind(smalltalk.Interval,"consume:",0
,function(aBlock){var self=this;
while(!(function(){return self.isEmpty();})()){aBlock.value_(self["@start"]);
self["@start"]=self["@start"]._plus(self["@step"]);};;
return self;}
,"iterating",unescape("consume%3A%20aBlock%0D%0A%09%22%20Private%20-%20Evaluates%20aBlock%20while%20the%20receiver%20is%20not%20empty.%20%22%0D%0A%0D%0A%09%5B%20self%20isEmpty%20%5D%20whileFalse%3A%20%5B%0D%0A%09%09aBlock%20value%3A%20start.%0D%0A%09%09start%20%3A%3D%20start%20+%20step%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.Interval,"atEnd",0
,function(){var self=this;
return self.size()._eq((0));}
,"testing",unescape("atEnd%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20at%20end.%20%22%0D%0A%0D%0A%09%5Eself%20size%20%3D%200%20"));
smalltalk.bind(smalltalk.Interval,"isEmpty",0
,function(){var self=this;
return self.size()._eq((0));}
,"testing",unescape("isEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20empty.%20%22%0D%0A%0D%0A%09%5Eself%20size%20%3D%200%20"));
smalltalk.bind(smalltalk.Interval,"notEmpty",0
,function(){var self=this;
return self.isEmpty().not();}
,"testing",unescape("notEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20not%20empty.%20%22%0D%0A%0D%0A%09%5Eself%20isEmpty%20not"));
smalltalk.bind(smalltalk.Interval,"reversed",0
,function(){var self=this;
return self.$klass.from_to_by_value_(self["@end"],self["@start"],self["@step"].negated(),self["@value"]);}
,"copying",unescape("reversed%0D%0A%09%22%20Return%20an%20Interval%20with%20the%20contents%20of%20the%20receiver%20in%20reverse%20order.%20%22%0D%0A%0D%0A%09%5Eself%20class%20from%3A%20end%20to%3A%20start%20by%3A%20step%20negated%20value%3A%20value%20"));
smalltalk.bind(smalltalk.Interval,"reverseDo:",0
,function(aBlock){var self=this;
return self.reversed().do_(aBlock);}
,"enumerating",unescape("reverseDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver%20in%20reverse%20order%20of%20iteration.%20%22%0D%0A%0D%0A%09%5Eself%20reversed%20do%3A%20aBlock%20"));
smalltalk.bind(smalltalk.Interval,"do:",0
,function(aBlock){var self=this;
self["@start"].to_by_do_(self["@end"],self["@step"],(function(i){return self["@value"].value_value_(i,aBlock);}));
return self;}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09start%20to%3A%20end%20by%3A%20step%20do%3A%20%5B%3Ai%7C%0D%0A%09%09value%20value%3A%20i%20value%3A%20aBlock%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Interval,"+","_plus"
,function(aNumber){var self=this;
return self["@start"]._plus(aNumber).to_by_value_(self["@end"]._plus(aNumber),self["@step"],self["@value"]);}
,"arithmetic",unescape("+%20aNumber%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%5Estart%20+%20aNumber%20to%3A%20end%20+%20aNumber%20by%3A%20step%20value%3A%20value"));
smalltalk.bind(smalltalk.Interval,"-","_minus"
,function(aNumber){var self=this;
return self["@start"]._minus(aNumber).to_by_value_(self["@end"]._minus(aNumber),self["@step"],self["@value"]);}
,"arithmetic",unescape("-%20aNumber%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%5Estart%20-%20aNumber%20to%3A%20end%20-%20aNumber%20by%3A%20step%20value%3A%20value"));
smalltalk.bind(smalltalk.Interval,"printString",0
,function(){var self=this;
return unescape("%28")._comma(self["@start"])._comma("to:")._comma(self["@end"])._comma(unescape("%29"));}
,"printing",unescape("printString%0D%0A%09%22%20Return%20the%20string%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%24%28%20%2Cstart%20%2C%23to%3A%20%2Cend%20%2C%24%29%20"));
smalltalk.bind(smalltalk.Interval,"toString",0
,function(){var self=this;
if ((nil.isNil_(self["@start"])).or_((function(){return (nil.isNil_(self["@end"]));}))){return smalltalk.superImplementor(smalltalk.Interval,"toString").apply(self, []);};;
return self.printString();}
,"converting",unescape("toString%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20String.%20%22%0D%0A%0D%0A%09%28start%20isNil%20or%3A%20%5Bend%20isNil%5D%29%20ifTrue%3A%20%5B%20%5Esuper%20toString%20%5D.%0D%0A%09%5Eself%20printString%20"));
smalltalk.bind(smalltalk.Interval,"__tostring",0
,function(){var self=this;
return self.toString();}
,"Lua-printing",unescape("__tostring%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.Set.$klass,"newContents:",0
,function(contents){var self=this;
return self.basicNew().initialize_(contents);}
,"private",unescape("newContents%3A%20contents%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20initialized%20from%20given%20contents.%20%22%0D%0A%0D%0A%09%5Eself%20basicNew%20initialize%3A%20contents"));
smalltalk.bind(smalltalk.Set.$klass,"basicNew:",0
,function(size){var self=this;
return self.$$new();}
,"instantiation",unescape("basicNew%3A%20size%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20of%20specified%20size.%20%22%0D%0A%0D%0A%09%5Eself%20new"));
smalltalk.bind(smalltalk.Set,"initialize",0
,function(){var self=this;
smalltalk.superImplementor(smalltalk.Set,"initialize").apply(self, []);
self["@contents"]=[];
return self;}
,"initialize",unescape("initialize%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20initialize.%0D%0A%09contents%20%3A%3D%20%23%28%29."));
smalltalk.bind(smalltalk.Set,"initialize:",0
,function(initializer){var self=this;
self["@contents"]=initializer;
return self;}
,"initialize",unescape("initialize%3A%20initializer%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%20%22%0D%0A%0D%0A%09contents%20%3A%3D%20initializer."));
smalltalk.bind(smalltalk.Set,"size",0
,function(){var self=this;
return self["@contents"].size();}
,"accessing",unescape("size%0D%0A%09%22%20Return%20the%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Econtents%20size"));
smalltalk.bind(smalltalk.Set,"basicAdd:",0
,function(anObject){var self=this;
self["@contents"].add_(anObject);
return anObject;}
,"private",unescape("basicAdd%3A%20anObject%0D%0A%09%22%20Private%20-%20Add%20anObject%20to%20the%20receiver%27s%20contents.%20%22%0D%0A%0D%0A%09contents%20add%3A%20anObject.%0D%0A%09%5EanObject"));
smalltalk.bind(smalltalk.Set,"includes:",0
,function(anObject){var self=this;
return self["@contents"].includes_(anObject);}
,"testing",unescape("includes%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Econtents%20includes%3A%20anObject"));
smalltalk.bind(smalltalk.Set,"add:",0
,function(anObject){var self=this;
if (self.includes_(anObject)){return anObject;};;
return self.basicAdd_(anObject);}
,"adding",unescape("add%3A%20anObject%0D%0A%09%22%20Add%20anObject%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%28self%20includes%3A%20anObject%29%20ifTrue%3A%20%5B%20%5EanObject%20%5D.%0D%0A%09%5Eself%20basicAdd%3A%20anObject"));
smalltalk.bind(smalltalk.Set,"remove:ifAbsent:",0
,function(anObject,aBlock){var self=this;
return self["@contents"].remove_ifAbsent_(anObject,aBlock);}
,"removing",unescape("remove%3A%20anObject%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Remove%20anObject%20from%20the%20receiver.%0D%0A%09Return%20the%20result%20of%20evaluating%20aBlock%20if%20anObject%20is%20not%20in%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%5Econtents%20remove%3A%20anObject%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.Set,"copyFrom:to:",0
,function(anIndex,anotherIndex){var self=this;
return self.shouldNotImplement_("copyFrom:to:");}
,"copying",unescape("copyFrom%3A%20anIndex%20to%3A%20anotherIndex%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20from%20anIndex%20to%20anotherIndex.%20%22%0D%0A%0D%0A%09%5Eself%20shouldNotImplement%3A%20%23copyFrom%3Ato%3A"));
smalltalk.bind(smalltalk.Set,"asNative:",0
,function(lang){var self=this;
var send=nil;
send=((lang._eq("lua")).mustBeBoolean()==true ? (function(){return ":";})() : (function(){return ".";})());
return "smalltalk."._comma(self.$klass.name())._comma(send)._comma(unescape("newContents_%28"))._comma(self["@contents"].asNative_(lang))._comma(unescape("%29"));}
,"converting",unescape("asNative%3A%20lang%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20send%20%7C%0D%0A%09send%20%3A%3D%20lang%20%3D%20%23lua%20ifTrue%3A%20%5B%20%24%3A%20%5D%20ifFalse%3A%20%5B%20%24.%20%5D.%0D%0A%09%5E%27smalltalk.%27%20%2Cself%20class%20name%0D%0A%09%09%2Csend%20%2C%27newContents_%28%27%0D%0A%09%09%2C%28contents%20asNative%3A%20lang%29%0D%0A%09%09%2C%27%29%27"));
smalltalk.bind(smalltalk.Set,"=","_eq"
,function(aSet){var $s8Ret$={name:"stReturn"};try{var self=this;
if (!(self.$klass._eq(aSet.$klass))){return false;};;
if (!(self.size()._eq(aSet.size()))){return false;};;
self.do_((function(each){return ((aSet.includes_(each)).mustBeBoolean()==true ? nil : (function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();})());}));
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"comparing",unescape("%3D%20aSet%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09self%20class%20%3D%20aSet%20class%20ifFalse%3A%20%5B%5Efalse%5D.%0D%0A%09self%20size%20%3D%20aSet%20size%20ifFalse%3A%20%5B%5Efalse%5D.%0D%0A%09self%20do%3A%20%5B%3Aeach%20%7C%20%28aSet%20includes%3A%20each%29%20ifFalse%3A%20%5B%5Efalse%5D%20%5D.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.Set,"asArray",0
,function(){var self=this;
return self["@contents"].copy();}
,"converting",unescape("asArray%0D%0A%09%22%20Return%20an%20array%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Econtents%20copy"));
smalltalk.bind(smalltalk.Set,"toString",0
,function(){var self=this;
return self.$klass.name()._comma(" withAll: ")._comma(self.asArray().printString());}
,"converting",unescape("toString%0D%0A%09%22%20Convert%20the%20receiver%20to%20a%20String.%20%22%0D%0A%0D%0A%09%5Eself%20class%20name%2C%27%20withAll%3A%20%27%2Cself%20asArray%20printString"));
smalltalk.bind(smalltalk.Set,"sorted",0
,function(){var self=this;
return self.asArray().sort();}
,"sorting",unescape("sorted%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver%20sorted.%20%22%0D%0A%0D%0A%09%5Eself%20asArray%20sort"));
smalltalk.bind(smalltalk.Set,"sorted:",0
,function(aBlock){var self=this;
return self.asArray().sort_(aBlock);}
,"sorting",unescape("sorted%3A%20aBlock%0D%0A%09%22%20Return%20a%20sorted%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20asArray%20sort%3A%20aBlock"));
smalltalk.bind(smalltalk.Set,"do:",0
,function(aBlock){var self=this;
return self["@contents"].do_(aBlock);}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Econtents%20do%3A%20aBlock"));
smalltalk.bind(smalltalk.Set,"detect:ifNone:",0
,function(aBlock,anotherBlock){var self=this;
return self["@contents"].detect_ifNone_(aBlock,anotherBlock);}
,"enumerating",unescape("detect%3A%20aBlock%20ifNone%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20object%20in%20the%20receiver%20that%20return%20true%20when%20evaluated%20with%20aBlock%20or%20the%20result%20of%20evaluating%20anotherBlock.%20%22%0D%0A%0D%0A%09%5Econtents%20detect%3A%20aBlock%20ifNone%3A%20anotherBlock"));
smalltalk.bind(smalltalk.Set,"reverseDo:",0
,function(aBlock){var self=this;
return self["@contents"].reverseDo_(aBlock);}
,"enumerating",unescape("reverseDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20contents%20of%20the%20receiver%20in%20reverse%20order%20of%20iteration.%20%22%0D%0A%0D%0A%09%5Econtents%20reverseDo%3A%20aBlock"));
smalltalk.bind(smalltalk.Set,"collect:",0
,function(aBlock){var self=this;
return self.$klass.withAll_(self["@contents"].collect_(aBlock));}
,"enumerating",unescape("collect%3A%20aBlock%0D%0A%09%22%20Return%20a%20collection%20with%20the%20result%20of%20evaluating%20aBlock%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20class%20withAll%3A%20%28contents%20collect%3A%20aBlock%29"));
smalltalk.bind(smalltalk.Set,"do:separatedBy:",0
,function(aBlock,anotherBlock){var self=this;
return self["@contents"].do_separatedBy_(aBlock,anotherBlock);}
,"enumerating",unescape("do%3A%20aBlock%20separatedBy%3A%20anotherBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20contents%20of%20the%20receiver%20evaluating%20anotherBlock%20inbetween%20aBlock%20evaluations.%20%22%0D%0A%0D%0A%09%5Econtents%20do%3A%20aBlock%20separatedBy%3A%20anotherBlock"));
smalltalk.bind(smalltalk.Set,"inject:into:",0
,function(anObject,aBlock){var self=this;
return self["@contents"].inject_into_(anObject,aBlock);}
,"enumerating",unescape("inject%3A%20anObject%20into%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20result%20of%20previous%20evaluation%20and%20contents%20of%20the%20receiver.%0D%0A%09On%20first%20evaluation%20anObject%20is%20used%20as%20first%20argument%20to%20evaluate%20aBlock.%0D%0A%09%22%0D%0A%0D%0A%09%5Econtents%20inject%3A%20anObject%20into%3A%20aBlock"));
smalltalk.bind(smalltalk.Set,"select:",0
,function(aBlock){var self=this;
return self.$klass.withAll_(self["@contents"].select_(aBlock));}
,"enumerating",unescape("select%3A%20aBlock%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20with%20elements%20wich%20evaluates%20aBlock%20to%20true.%20%22%0D%0A%0D%0A%09%5Eself%20class%20withAll%3A%20%28contents%20select%3A%20aBlock%29"));
smalltalk.bind(smalltalk.Set,"reject:",0
,function(aBlock){var self=this;
return self.$klass.withAll_(self["@contents"].reject_(aBlock));}
,"enumerating",unescape("reject%3A%20aBlock%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver%20wich%20evaluates%20aBlock%20to%20false.%20%22%0D%0A%0D%0A%09%5Eself%20class%20withAll%3A%20%28contents%20reject%3A%20aBlock%29"));
smalltalk.bind(smalltalk.Set,"reversed",0
,function(){var self=this;
return self["@contents"].reversed();}
,"copying",unescape("reversed%0D%0A%09%22%20Return%20an%20array%20with%20the%20contents%20of%20the%20receiver%20in%20reverse%20order.%20%22%0D%0A%0D%0A%09%5Econtents%20reversed"));
smalltalk.bind(smalltalk.Set,"notEmpty",0
,function(){var self=this;
return self["@contents"].notEmpty();}
,"testing",unescape("notEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20not%20empty.%20%22%0D%0A%0D%0A%09%5Econtents%20notEmpty"));
smalltalk.bind(smalltalk.Set,"isEmpty",0
,function(){var self=this;
return self["@contents"].isEmpty();}
,"testing",unescape("isEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20empty.%20%22%0D%0A%0D%0A%09%5Econtents%20isEmpty"));
smalltalk.bind(smalltalk.Set,"occurrencesOf:",0
,function(anObject){var self=this;
return ((self.includes_(anObject)).mustBeBoolean()==true ? (function(){return (1);})() : (function(){return (0);})());}
,"counting",unescape("occurrencesOf%3A%20anObject%0D%0A%09%22%20Return%20the%20occurrences%20of%20anObject%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28self%20includes%3A%20anObject%29%20ifTrue%3A%20%5B%201%20%5D%20ifFalse%3A%20%5B%200%20%5D"));
smalltalk.bind(smalltalk.Set,"asSet",0
,function(){var self=this;
return self;}
,"converting",unescape("asSet%0D%0A%09%22%20Return%20the%20receiver%20contents%20in%20a%20Set.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Set,"shallowCopy",0
,function(){var self=this;
return self.species().newContents_(self["@contents"].copy());}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20species%20newContents%3A%20contents%20copy"));
smalltalk.bind(smalltalk.Set,"deepCopy",0
,function(){var self=this;
return self.species().newContents_(self["@contents"].deepCopy());}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20species%20newContents%3A%20contents%20deepCopy"));
smalltalk.bind(smalltalk.Set,"asLua",0
,function(){var self=this;
return "smalltalk."._comma(self.$klass.name())._comma(unescape("%3AnewContents_%28"))._comma(self["@contents"].asLua())._comma(unescape("%29"));}
,"Lua-lua",unescape("asLua%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%27smalltalk.%27%20%2Cself%20class%20name%0D%0A%09%2C%20%27%3AnewContents_%28%27%0D%0A%09%09%2C%20contents%20asLua%0D%0A%09%2C%27%29%27"));
smalltalk.bind(smalltalk.Set,"__tostring",0
,function(){var self=this;
return self.toString();}
,"Lua-printing",unescape("__tostring%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.HashedSet,"initialize",0
,function(){var self=this;
smalltalk.superImplementor(smalltalk.HashedSet,"initialize").apply(self, []);
self.initializeHash();
return self;}
,"initialize",unescape("initialize%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20initialize.%0D%0A%09self%20initializeHash."));
smalltalk.bind(smalltalk.HashedSet,"initialize:",0
,function(initializer){var self=this;
self["@contents"]=initializer;
self.rehash();
return self;}
,"initialize",unescape("initialize%3A%20initializer%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%20%22%0D%0A%0D%0A%09contents%20%3A%3D%20initializer.%0D%0A%09self%20rehash."));
smalltalk.bind(smalltalk.HashedSet,"initializeHash",0
,function(){var self=this;
self["@hash"]={};
return self;}
,"private",unescape("initializeHash%0D%0A%09%22%20Private%20-%20Initialize%20the%20hash%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09hash%20%3A%3D%20%7B%27%27%20doc%27get%20native%20sentinel%27%7D."));
smalltalk.bind(smalltalk.HashedSet,"rehash",0
,function(){var self=this;
self.initializeHash();
self["@contents"].do_((function(each){return self.privateAdd_(self.wrap_(each));}));
return self;}
,"private",unescape("rehash%0D%0A%09%22%20Private%20-%20Rehash%20the%20receiver%20contents.%20%22%0D%0A%0D%0A%09self%20initializeHash.%0D%0A%09contents%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09self%20privateAdd%3A%20%28self%20wrap%3A%20each%29%0D%0A%09%5D"));
smalltalk.bind(smalltalk.HashedSet,"asHashedSet",0
,function(){var self=this;
return self;}
,"converting",unescape("asHashedSet%0D%0A%09%22%20Return%20the%20receiver%20contents%20in%20a%20HashedSet.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.HashedSet,"indexOf:ifAbsent:",0
,function(anObject,aBlock){var self=this;
if (!(self.includes_(anObject))){return aBlock.value();};;
return self["@contents"].indexOf_ifAbsent_(anObject,aBlock);}
,"private",unescape("indexOf%3A%20anObject%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20position%20of%20anObject%20in%20the%20receiver%2C%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%28self%20includes%3A%20anObject%29%20ifFalse%3A%20%5B%20%5EaBlock%20value%20%5D.%0D%0A%09%5Econtents%20indexOf%3A%20anObject%20ifAbsent%3A%20aBlock"));
smalltalk.bind(smalltalk.HashedSet,"withIndexDo:",0
,function(aBlock){var self=this;
return self["@contents"].withIndexDo_(aBlock);}
,"private",unescape("withIndexDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20all%20the%20index%20positions%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Econtents%20withIndexDo%3A%20aBlock"));
smalltalk.bind(smalltalk.HashedSet,"removeIndex:",0
,function(index){var self=this;
return self.remove_(self["@contents"].at_(index));}
,"private",unescape("removeIndex%3A%20index%0D%0A%09%22%20Remove%20the%20element%20at%20index.%20%22%0D%0A%0D%0A%09%5Eself%20remove%3A%20%28contents%20at%3A%20index%29"));
smalltalk.bind(smalltalk.HashedSet,"privateAdd:",0
,function(cookie){var self=this;

(self["@hash"][cookie]=(cookie));
return self;}
,"private",unescape("privateAdd%3A%20cookie%0D%0A%09%22%20Private%20-%20Register%20a%20cookie%20onto%20the%20receiver%27s%20hash.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09hash%20basicAt%3A%20cookie%20put%3A%20cookie"));
smalltalk.bind(smalltalk.HashedSet,"privateRemove:",0
,function(cookie){var self=this;
delete self["@hash"][cookie];
return self;}
,"private",unescape("privateRemove%3A%20cookie%0D%0A%09%22%20Private%20-%20Remove%20a%20cookie%20in%20the%20receiver%27s%20hash.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20remotion%27%7D"));
smalltalk.bind(smalltalk.HashedSet,"includes:",0
,function(anObject){var self=this;
var cookie=nil;
cookie=self.wrap_(anObject);
return Object.prototype.hasOwnProperty.call(self["@hash"],cookie);}
,"testing",unescape("includes%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20cookie%20%7C%0D%0A%09cookie%20%3A%3D%20self%20wrap%3A%20anObject.%0D%0A%09%5E%7B%27%27%20doc%27testing%27%7D"));
smalltalk.bind(smalltalk.HashedSet,"add:",0
,function(anObject){var self=this;
if (self.includes_(anObject)){return anObject;};;
self.privateAdd_(self.wrap_(anObject));
return self.basicAdd_(anObject);}
,"adding",unescape("add%3A%20anObject%0D%0A%09%22%20Add%20anObject%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09%28self%20includes%3A%20anObject%29%20ifTrue%3A%20%5B%20%5EanObject%20%5D.%0D%0A%09self%20privateAdd%3A%20%28self%20wrap%3A%20anObject%29.%0D%0A%09%5Eself%20basicAdd%3A%20anObject"));
smalltalk.bind(smalltalk.HashedSet,"remove:ifAbsent:",0
,function(anObject,aBlock){var self=this;
if (!(self.includes_(anObject))){return aBlock.value();};;
self["@contents"].remove_ifAbsent_(anObject,(function(){return nil;}));
self.privateRemove_(self.wrap_(anObject));
return anObject;}
,"removing",unescape("remove%3A%20anObject%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Remove%20anObject%20from%20the%20receiver.%0D%0A%09Return%20the%20result%20of%20evaluating%20aBlock%20if%20anObject%20is%20not%20in%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%28self%20includes%3A%20anObject%29%20ifFalse%3A%20%5B%20%5EaBlock%20value%20%5D.%0D%0A%09contents%20remove%3A%20anObject%20ifAbsent%3A%20%5B%20%5D.%0D%0A%09self%20privateRemove%3A%20%28self%20wrap%3A%20anObject%29.%0D%0A%09%5EanObject"));
smalltalk.bind(smalltalk.KeyedCollection.$klass,"basicNew:",0
,function(size){var self=this;
return self.$$new();}
,"instantiation",unescape("basicNew%3A%20size%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20of%20specified%20size.%20%22%0D%0A%0D%0A%09%5Eself%20new"));
smalltalk.bind(smalltalk.KeyedCollection.$klass,"@","_at"
,function(jsObject){var self=this;
var result=nil;
var parts=nil;
if ((nil.isNil_(jsObject))){return nil;};;
result=self.$$new();
parts=Object.getOwnPropertyNames(jsObject);
parts.do_((function(key){return result.at_put_(key,((function(){var $1$=(jsObject[key]);if(nil.isNil_($1$))return nil;return $1$;})()));}));
return result;}
,"instantiation",unescape("@%20jsObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20with%20the%20properties%20owned%20by%20jsObject%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20result%20parts%20%7C%0D%0A%09jsObject%20isNil%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09result%20%3A%3D%20self%20new.%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D.%0D%0A%09parts%20do%3A%20%5B%3Akey%7C%0D%0A%09%09result%20at%3A%20key%20put%3A%20%28jsObject%20basicAt%3A%20key%29%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.KeyedCollection.$klass,"fromTuples:",0
,function(tuples){var self=this;
var result=nil;
result=self.$$new();
tuples.do_((function(tuple){return result.at_put_(tuple.first(),tuple.last());}));
return result;}
,"instantiation",unescape("fromTuples%3A%20tuples%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20new.%0D%0A%09tuples%20do%3A%20%5B%3Atuple%7C%20result%20at%3A%20tuple%20first%20put%3A%20tuple%20last%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.KeyedCollection,",","_comma"
,function(aCollection){var self=this;
return self.shouldNotImplement_(unescape("%2C"));}
,"copying",unescape("%2C%20aCollection%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20with%20all%20contents%20of%20aCollection.%20%22%0D%0A%0D%0A%09%5Eself%20shouldNotImplement%3A%20%23%2C"));
smalltalk.bind(smalltalk.KeyedCollection,"=","_eq"
,function(anObject){var $s8Ret$={name:"stReturn"};try{var self=this;
if (!(self.$klass._eq(anObject.$klass))){return false;};;
if (!(self.size()._eq(anObject.size()))){return false;};;
self.keysAndValuesDo_((function(key,value){return ((anObject.at_ifAbsent_(key,(function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();}))._eq(value)).mustBeBoolean()==true ? nil : (function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();})());}));
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"comparing",unescape("%3D%20anObject%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09self%20class%20%3D%20anObject%20class%20ifFalse%3A%20%5B%5Efalse%5D.%0D%0A%09self%20size%20%3D%20anObject%20size%20ifFalse%3A%20%5B%5Efalse%5D.%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%20%7C%0D%0A%09%09%28anObject%20at%3A%20key%20ifAbsent%3A%20%5B%5Efalse%5D%29%0D%0A%09%09%09%3D%20value%20ifFalse%3A%20%5B%5Efalse%5D%0D%0A%09%5D.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.KeyedCollection,"add:",0
,function(anAssociation){var self=this;
self.at_put_(anAssociation.key(),anAssociation.value());
return anAssociation;}
,"adding",unescape("add%3A%20anAssociation%0D%0A%09%22%20Bind%20anAssociation%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20at%3A%20anAssociation%20key%20put%3A%20anAssociation%20value.%0D%0A%09%5EanAssociation"));
smalltalk.bind(smalltalk.KeyedCollection,"addAll:",0
,function(aKeyedCollection){var self=this;
smalltalk.superImplementor(smalltalk.KeyedCollection,"addAll_").apply(self, [aKeyedCollection.associations()]);
return aKeyedCollection;}
,"adding",unescape("addAll%3A%20aKeyedCollection%0D%0A%09%22%20Bind%20all%20associations%20in%20aKeyedCollection%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20addAll%3A%20aKeyedCollection%20associations.%0D%0A%09%5EaKeyedCollection"));
smalltalk.bind(smalltalk.KeyedCollection,"newHash",0
,function(){var self=this;
return {};}
,"private",unescape("newHash%0D%0A%09%22%20Private%20-%20Return%20a%20new%20hash%20object.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20instantiation%27%7D"));
smalltalk.bind(smalltalk.KeyedCollection,"asJSONObject",0
,function(){var self=this;
var result=nil;
var value=nil;
result=self.newHash();
self.keysAndValuesDo_((function(key,aValue){value=((self.isObject_(aValue)).mustBeBoolean()==true ? (function(){return aValue.asJSONObject();})() : (function(){return aValue;})());return (result[key]=(value));}));
return result;}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%7C%20result%20value%20%7C%0D%0A%09result%20%3A%3D%20self%20newHash.%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3AaValue%20%7C%0D%0A%09%09value%20%3A%3D%20%28self%20isObject%3A%20aValue%29%0D%0A%09%09%09ifTrue%3A%20%5B%20aValue%20asJSONObject%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20aValue%20%5D.%0D%0A%09%09result%20basicAt%3A%20key%20put%3A%20value%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.KeyedCollection,"associations",0
,function(){var self=this;
var result=nil;
result=smalltalk.Array.$$new();
self.associationsDo_((function(each){return result.add_(each);}));
return result;}
,"accessing",unescape("associations%0D%0A%09%22%20Return%20the%20associations%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20Array%20new.%0D%0A%09self%20associationsDo%3A%20%5B%3Aeach%7C%20result%20add%3A%20each%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.KeyedCollection,"associationsDo:",0
,function(aBlock){var self=this;
self.keysAndValuesDo_((function(key,value){return aBlock.value_(smalltalk.Association.key_value_(key,value));}));
return self;}
,"enumerating",unescape("associationsDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20the%20receiver%27s%20associations.%20%22%0D%0A%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%7C%0D%0A%09%09aBlock%20value%3A%20%28Association%20key%3A%20key%20value%3A%20value%29%0D%0A%09%5D."));
smalltalk.bind(smalltalk.KeyedCollection,"at:",0
,function(aKey){var self=this;
return self.at_ifAbsent_(aKey,(function(){return self.objectNotFoundAt_(aKey);}));}
,"accessing",unescape("at%3A%20aKey%0D%0A%09%22%20Return%20the%20value%20at%20aKey.%0D%0A%09Signal%20an%20error%20if%20the%20key%20is%20not%20present.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20at%3A%20aKey%20ifAbsent%3A%20%5B%20self%20objectNotFoundAt%3A%20aKey%20%5D"));
smalltalk.bind(smalltalk.KeyedCollection,"at:ifAbsent:",0
,function(aKey,aBlock){var self=this;
return self.implementedBySubclass_("at:ifAbsent:");}
,"accessing",unescape("at%3A%20aKey%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20value%20at%20aKey%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23at%3AifAbsent%3A"));
smalltalk.bind(smalltalk.KeyedCollection,"values",0
,function(){var self=this;
return self.implementedBySubclass_("values");}
,"accessing",unescape("values%0D%0A%09%22%20Return%20the%20values%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23values"));
smalltalk.bind(smalltalk.KeyedCollection,"at:ifAbsentPut:",0
,function(aKey,aBlock){var self=this;
return self.at_ifAbsent_(aKey,(function(){return self.at_put_(aKey,aBlock.value());}));}
,"accessing",unescape("at%3A%20aKey%20ifAbsentPut%3A%20aBlock%0D%0A%09%22%20Return%20the%20value%20at%20aKey%2C%20or%20add%20the%20result%20of%20evaluating%20aBlock%20to%20the%20receiver%20if%20it%20is%20absent.%20%22%0D%0A%0D%0A%09%5Eself%20at%3A%20aKey%20ifAbsent%3A%20%5B%20self%20at%3A%20aKey%20put%3A%20aBlock%20value%20%5D"));
smalltalk.bind(smalltalk.KeyedCollection,"at:put:",0
,function(aKey,aValue){var self=this;
return self.implementedBySubclass_("at:put:");}
,"accessing",unescape("at%3A%20aKey%20put%3A%20aValue%0D%0A%09%22%20Set%20the%20value%20at%20aKey%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23at%3Aput%3A"));
smalltalk.bind(smalltalk.KeyedCollection,"detect:ifNone:",0
,function(aBlock,anotherBlock){var self=this;
return self.implementedBySubclass_("detect:ifNone:");}
,"enumerating",unescape("detect%3A%20aBlock%20ifNone%3A%20anotherBlock%0D%0A%09%22%20Detect%20the%20value%20who%20matches%20aBlock.%0D%0A%09Return%20the%20result%20of%20evaluating%20aBlock%20if%20no%20value%20matches%20detection%20block.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23detect%3AifNone%3A"));
smalltalk.bind(smalltalk.KeyedCollection,"do:",0
,function(aBlock){var self=this;
return self.implementedBySubclass_("do:");}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20for%20all%20values%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23do%3A"));
smalltalk.bind(smalltalk.KeyedCollection,"includes:",0
,function(anObject){var self=this;
return self.implementedBySubclass_("includes:");}
,"testing",unescape("includes%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23includes%3A"));
smalltalk.bind(smalltalk.KeyedCollection,"keyForValue:ifAbsent:",0
,function(anObject,aBlock){var $s8Ret$={name:"stReturn"};try{var self=this;
self.keysAndValuesDo_((function(key,value){return ((anObject._eq(value)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return key})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return aBlock.value()})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("keyForValue%3A%20anObject%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20key%20in%20the%20receiver%20for%20value%20anObject%2C%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%7C%20anObject%20%3D%20value%20ifTrue%3A%20%5B%20%5Ekey%20%5D%20%5D.%0D%0A%09%5EaBlock%20value"));
smalltalk.bind(smalltalk.KeyedCollection,"keys",0
,function(){var self=this;
return self.implementedBySubclass_("keys");}
,"accessing",unescape("keys%0D%0A%09%22%20Return%20the%20keys%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23keys"));
smalltalk.bind(smalltalk.KeyedCollection,"keysAndValuesDo:",0
,function(aBlock){var self=this;
return self.implementedBySubclass_("keysAndValuesDo:");}
,"enumerating",unescape("keysAndValuesDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20key/value%20pairs%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23keysAndValuesDo%3A"));
smalltalk.bind(smalltalk.KeyedCollection,"keysDo:",0
,function(aBlock){var self=this;
return self.implementedBySubclass_("keysDo:");}
,"enumerating",unescape("keysDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20keys%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23keysDo%3A"));
smalltalk.bind(smalltalk.KeyedCollection,"includesKey:",0
,function(aKey){var self=this;
return self.implementedBySubclass_("includesKey:");}
,"testing",unescape("includesKey%3A%20aKey%0D%0A%09%22%20Return%20true%20if%20the%20receiver%27s%20keys%20include%20aKey.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23includesKey%3A"));
smalltalk.bind(smalltalk.KeyedCollection,"removeKey:ifAbsent:",0
,function(aKey,aBlock){var self=this;
return self.implementedBySubclass_("removeKey:ifAbsent:");}
,"removing",unescape("removeKey%3A%20aKey%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Remove%20the%20binding%20at%20aKey.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23removeKey%3AifAbsent%3A"));
smalltalk.bind(smalltalk.KeyedCollection,"removeKeys:",0
,function(aCollection){var self=this;
aCollection.do_((function(each){return self.removeKey_ifAbsent_(each,(function(){return nil;}));}));
return self;}
,"removing",unescape("removeKeys%3A%20aCollection%0D%0A%09%22%20Remove%20aCollection%20of%20keys%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09aCollection%20do%3A%20%5B%3Aeach%7C%20self%20removeKey%3A%20each%20ifAbsent%3A%20%5B%5D%20%5D"));
smalltalk.bind(smalltalk.KeyedCollection,"removeAll",0
,function(){var self=this;
self.initialize();
return self;}
,"removing",unescape("removeAll%0D%0A%09%22%20Remove%20all%20elements%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20initialize"));
smalltalk.bind(smalltalk.KeyedCollection,"isEmpty",0
,function(){var self=this;
return self.size()._eq((0));}
,"testing",unescape("isEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20empty.%20%22%0D%0A%0D%0A%09%5Eself%20size%20%3D%200"));
smalltalk.bind(smalltalk.KeyedCollection,"at:ifPresent:",0
,function(aKey,aBlock){var $s8Ret$={name:"stReturn"};try{var self=this;
var aValue=nil;
aValue=self.at_ifAbsent_(aKey,(function(){return (function(){$s8Ret$.result=(function(){return nil})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return aBlock.value_(aValue)})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("at%3A%20aKey%20ifPresent%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20aBlock%20if%20there%20is%20a%20value%20at%20aKey%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20aValue%20%7C%0D%0A%09aValue%20%3A%3D%20self%20at%3A%20aKey%20ifAbsent%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%5EaBlock%20value%3A%20aValue"));
smalltalk.bind(smalltalk.KeyedCollection,"at:ifPresent:ifAbsent:",0
,function(aKey,aBlock,anotherBlock){var $s8Ret$={name:"stReturn"};try{var self=this;
var aValue=nil;
aValue=self.at_ifAbsent_(aKey,(function(){return (function(){$s8Ret$.result=(function(){return anotherBlock.value()})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return aBlock.value_(aValue)})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("at%3A%20aKey%20ifPresent%3A%20aBlock%20ifAbsent%3A%20anotherBlock%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20aBlock%20if%20there%20is%20a%20value%20at%20aKey%20%28or%20the%20result%20of%20evaluating%20anotherBlock%29.%20%22%0D%0A%0D%0A%09%7C%20aValue%20%7C%0D%0A%09aValue%20%3A%3D%20self%20at%3A%20aKey%20ifAbsent%3A%20%5B%20%5EanotherBlock%20value%20%5D.%0D%0A%09%5EaBlock%20value%3A%20aValue"));
smalltalk.bind(smalltalk.KeyedCollection,"collect:",0
,function(aBlock){var self=this;
var result=nil;
result=self.species().$$new();
self.keysAndValuesDo_((function(key,value){return result.at_put_(key,aBlock.value_(value));}));
return result;}
,"enumerating",unescape("collect%3A%20aBlock%0D%0A%09%22%20Return%20aKeyedCollection%20with%20keys%20and%20result%20of%20evaluating%20aBlock%20with%20value.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20species%20new.%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%20%7C%0D%0A%09%09result%20at%3A%20key%20put%3A%20%28aBlock%20value%3A%20value%29%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.KeyedCollection,"select:",0
,function(aBlock){var self=this;
var result=nil;
result=self.species().$$new();
self.keysAndValuesDo_((function(key,value){return ((aBlock.value_(value)).mustBeBoolean()==true ? (function(){return result.at_put_(key,value);})() : nil);}));
return result;}
,"enumerating",unescape("select%3A%20aBlock%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver%20wich%20evaluates%20aBlock%20to%20true.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20species%20new.%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%20%7C%0D%0A%09%09%28aBlock%20value%3A%20value%29%20ifTrue%3A%20%5B%20result%20at%3A%20key%20put%3A%20value%20%5D%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.KeyedCollection,"shallowCopy",0
,function(){var self=this;
var result=nil;
result=self.species().$$new();
self.keysAndValuesDo_((function(key,value){return result.at_put_(key,value);}));
return result;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20species%20new.%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%20%7C%20result%20at%3A%20key%20put%3A%20value%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.KeyedCollection,"tuples",0
,function(){var self=this;
var result=nil;
result=smalltalk.Array.$$new();
self.keysAndValuesDo_((function(key,value){return result.add_(smalltalk.Array.with_with_(key,value));}));
return result;}
,"accessing",unescape("tuples%0D%0A%09%22%20Return%20an%20array%20of%20tuples%20built%20from%20the%20receiver%27s%20contents.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20Array%20new.%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%7C%0D%0A%09%09result%20add%3A%20%28Array%20with%3A%20key%20with%3A%20value%29%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.KeyedCollection,"copyFrom:to:",0
,function(anIndex,anotherIndex){var self=this;
return self.shouldNotImplement_("copyFrom:to:");}
,"copying",unescape("copyFrom%3A%20anIndex%20to%3A%20anotherIndex%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20from%20anIndex%20to%20anotherIndex.%20%22%0D%0A%0D%0A%09%5Eself%20shouldNotImplement%3A%20%23copyFrom%3Ato%3A"));
smalltalk.bind(smalltalk.KeyedCollection,"deepCopy",0
,function(){var self=this;
var result=nil;
result=self.species().$$new();
self.keysAndValuesDo_((function(key,value){return result.at_put_(key,value.deepCopy());}));
return result;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20species%20new.%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%20%7C%0D%0A%09%09result%20at%3A%20key%20put%3A%20value%20deepCopy%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.KeyedCollection,"remove:",0
,function(aKey){var self=this;
self.removeKey_(aKey);
return self;}
,"removing",unescape("remove%3A%20aKey%0D%0A%09%22%20Remove%20the%20binding%20at%20aKey.%20%22%0D%0A%0D%0A%09self%20removeKey%3A%20aKey"));
smalltalk.bind(smalltalk.KeyedCollection,"removeKey:",0
,function(aKey){var self=this;
return self.removeKey_ifAbsent_(aKey,(function(){return self.objectNotFoundAt_(aKey);}));}
,"removing",unescape("removeKey%3A%20aKey%0D%0A%09%22%20Remove%20the%20binding%20at%20aKey.%0D%0A%09Signal%20an%20error%20if%20the%20key%20is%20not%20present.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20removeKey%3A%20aKey%20ifAbsent%3A%20%5B%20self%20objectNotFoundAt%3A%20aKey%20%5D"));
smalltalk.bind(smalltalk.KeyedCollection,"printString",0
,function(){var self=this;
return self.$klass.name()._comma(" fromTuples: ")._comma(self.tuples().asLiteral());}
,"converting",unescape("printString%0D%0A%09%22%20Return%20a%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20class%20name%20%2C%27%20fromTuples%3A%20%27%20%2Cself%20tuples%20asLiteral"));
smalltalk.bind(smalltalk.KeyedCollection,"isDictionary",0
,function(){var self=this;
return true;}
,"testing",unescape("isDictionary%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20dictionary.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.Dictionary.$klass,"basicNew",0
,function(){var self=this;

return smalltalk.superImplementor(smalltalk.Dictionary.$klass,"basicNew").apply(self, []);}
,"instantiation",unescape("basicNew%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27self%27%20doc%27native%20instantiation%27%7D.%0D%0A%09%5Esuper%20basicNew"));
smalltalk.bind(smalltalk.Dictionary.$klass,"fromKeys:and:",0
,function(keys,values){var self=this;
return self.basicNew().initializeKeys_values_(keys,values);}
,"instantiation",unescape("fromKeys%3A%20keys%20and%3A%20values%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicNew%20initializeKeys%3A%20keys%20values%3A%20values"));
smalltalk.bind(smalltalk.Dictionary,"initialize",0
,function(){var self=this;
smalltalk.superImplementor(smalltalk.Dictionary,"initialize").apply(self, []);
self["@keys"]=smalltalk.HashedSet.$$new();
self["@values"]=[];
return self;}
,"initialize",unescape("initialize%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20initialize.%0D%0A%09keys%20%3A%3D%20HashedSet%20new.%0D%0A%09values%20%3A%3D%20%23%28%29."));
smalltalk.bind(smalltalk.Dictionary,"initializeKeys:values:",0
,function(privateKeys,privateValues){var self=this;
smalltalk.superImplementor(smalltalk.Dictionary,"initialize").apply(self, []);
self["@keys"]=smalltalk.HashedSet.newContents_(privateKeys);
self["@values"]=privateValues.asArray();
return self;}
,"initialize",unescape("initializeKeys%3A%20privateKeys%20values%3A%20privateValues%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20initialize.%0D%0A%09keys%20%3A%3D%20HashedSet%20newContents%3A%20privateKeys.%0D%0A%09values%20%3A%3D%20privateValues%20asArray."));
smalltalk.bind(smalltalk.Dictionary,"size",0
,function(){var self=this;
return self["@keys"].size();}
,"accessing",unescape("size%0D%0A%09%22%20Return%20the%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ekeys%20size"));
smalltalk.bind(smalltalk.Dictionary,"isEmpty",0
,function(){var self=this;
return self["@keys"].isEmpty();}
,"testing",unescape("isEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20empty.%20%22%0D%0A%0D%0A%09%5Ekeys%20isEmpty"));
smalltalk.bind(smalltalk.Dictionary,"lookUpKey:",0
,function(aKey){var self=this;
return self["@keys"].indexOf_ifAbsent_(aKey,(function(){return nil;}));}
,"private",unescape("lookUpKey%3A%20aKey%0D%0A%09%22%20Private%20-%20Return%20the%20index%20where%20aKey%20is%20found%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Ekeys%20indexOf%3A%20aKey%20ifAbsent%3A%20%5B%20nil%20%5D"));
smalltalk.bind(smalltalk.Dictionary,"keysAndValuesDo:",0
,function(aBlock){var self=this;
self["@keys"].withIndexDo_((function(aKey,index){return aBlock.value_value_(aKey,self["@values"].at_(index));}));
return self;}
,"enumerating",unescape("keysAndValuesDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20key/value%20pairs%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09keys%20withIndexDo%3A%20%5B%3AaKey%20%3Aindex%20%7C%20aBlock%20value%3A%20aKey%20value%3A%20%28values%20at%3A%20index%29%20%5D"));
smalltalk.bind(smalltalk.Dictionary,"keys",0
,function(){var self=this;
return self["@keys"].copy();}
,"accessing",unescape("keys%0D%0A%09%22%20Return%20the%20keys%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ekeys%20copy"));
smalltalk.bind(smalltalk.Dictionary,"keysDo:",0
,function(aBlock){var self=this;
self["@keys"].do_(aBlock);
return self;}
,"enumerating",unescape("keysDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20keys%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09keys%20do%3A%20aBlock"));
smalltalk.bind(smalltalk.Dictionary,"values",0
,function(){var self=this;
return self["@values"].copy();}
,"accessing",unescape("values%0D%0A%09%22%20Return%20the%20values%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Evalues%20copy"));
smalltalk.bind(smalltalk.Dictionary,"at:put:",0
,function(aKey,aValue){var self=this;
var index=nil;
index=self.lookUpKey_(aKey);
if ((nil.isNil_(index))){self["@keys"].add_(aKey);
self["@values"].add_(aValue);} else{self["@values"].at_put_(index,aValue);};;
return aValue;}
,"accessing",unescape("at%3A%20aKey%20put%3A%20aValue%0D%0A%09%22%20Set%20the%20value%20at%20aKey%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09index%20%3A%3D%20self%20lookUpKey%3A%20aKey.%0D%0A%09index%20isNil%0D%0A%09%09ifTrue%3A%20%5B%20keys%20add%3A%20aKey.%20values%20add%3A%20aValue.%5D%0D%0A%09%09ifFalse%3A%20%5B%20values%20at%3A%20index%20put%3A%20aValue%20%5D.%0D%0A%09%5EaValue"));
smalltalk.bind(smalltalk.Dictionary,"at:ifAbsent:",0
,function(aKey,aBlock){var self=this;
var index=nil;
index=self.lookUpKey_(aKey);
if ((nil.isNil_(index))){return aBlock.value();};;
return self["@values"].at_(index);}
,"accessing",unescape("at%3A%20aKey%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20value%20at%20aKey%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09index%20%3A%3D%20self%20lookUpKey%3A%20aKey.%0D%0A%09index%20isNil%20ifTrue%3A%20%5B%20%5EaBlock%20value%20%5D.%0D%0A%09%5Evalues%20at%3A%20index"));
smalltalk.bind(smalltalk.Dictionary,"at:ifAbsentPut:",0
,function(aKey,aBlock){var self=this;
var index=nil;
index=self.lookUpKey_(aKey);
if ((nil.isNil_(index))){return self.at_put_(aKey,aBlock.value());};;
return self["@values"].at_(index);}
,"accessing",unescape("at%3A%20aKey%20ifAbsentPut%3A%20aBlock%0D%0A%09%22%20Return%20the%20value%20at%20aKey%2C%20or%20add%20the%20result%20of%20evaluating%20aBlock%20to%20the%20receiver%20if%20it%20is%20absent.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09index%20%3A%3D%20self%20lookUpKey%3A%20aKey.%0D%0A%09index%20isNil%20ifTrue%3A%20%5B%20%5Eself%20at%3A%20aKey%20put%3A%20aBlock%20value%20%5D.%0D%0A%09%5Evalues%20at%3A%20index"));
smalltalk.bind(smalltalk.Dictionary,"do:",0
,function(aBlock){var self=this;
return self["@values"].do_(aBlock);}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20for%20all%20values%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Evalues%20do%3A%20aBlock"));
smalltalk.bind(smalltalk.Dictionary,"detect:ifNone:",0
,function(aBlock,anotherBlock){var self=this;
return self["@values"].detect_ifNone_(aBlock,anotherBlock);}
,"enumerating",unescape("detect%3A%20aBlock%20ifNone%3A%20anotherBlock%0D%0A%09%22%20Detect%20the%20value%20who%20matches%20aBlock.%0D%0A%09Return%20the%20result%20of%20evaluating%20aBlock%20if%20no%20value%20matches%20detection%20block.%0D%0A%09%22%0D%0A%0D%0A%09%5Evalues%20detect%3A%20aBlock%20ifNone%3A%20anotherBlock"));
smalltalk.bind(smalltalk.Dictionary,"includes:",0
,function(anObject){var self=this;
return self["@values"].includes_(anObject);}
,"testing",unescape("includes%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Evalues%20includes%3A%20anObject"));
smalltalk.bind(smalltalk.Dictionary,"includesKey:",0
,function(aKey){var self=this;
return self["@keys"].includes_(aKey);}
,"enumerating",unescape("includesKey%3A%20aKey%0D%0A%09%22%20Return%20true%20if%20the%20receiver%27s%20keys%20include%20aKey.%20%22%0D%0A%0D%0A%09%5Ekeys%20includes%3A%20aKey"));
smalltalk.bind(smalltalk.Dictionary,"removeKey:ifAbsent:",0
,function(aKey,aBlock){var self=this;
var index=nil;
index=self.lookUpKey_(aKey);
if ((nil.isNil_(index))){return aBlock.value();};;
self["@keys"].removeIndex_(index);
self["@values"].removeIndex_(index);
return self;}
,"removing",unescape("removeKey%3A%20aKey%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Remove%20the%20binding%20at%20aKey.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09index%20%3A%3D%20self%20lookUpKey%3A%20aKey.%0D%0A%09index%20isNil%20ifTrue%3A%20%5B%20%5EaBlock%20value%20%5D.%0D%0A%09keys%20removeIndex%3A%20index.%0D%0A%09values%20removeIndex%3A%20index."));
smalltalk.bind(smalltalk.Dictionary,"keyForValue:ifAbsent:",0
,function(anObject,aBlock){var $s8Ret$={name:"stReturn"};try{var self=this;
self.keysAndValuesDo_((function(key,aValue){return (((nil.is_eqeq_(anObject,aValue))).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return key})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return aBlock.value()})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("keyForValue%3A%20anObject%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20key%20in%20the%20receiver%20for%20value%20anObject%2C%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3AaValue%7C%20anObject%20%3D%3D%20aValue%20ifTrue%3A%20%5B%20%5Ekey%20%5D%20%5D.%0D%0A%09%5EaBlock%20value"));
smalltalk.bind(smalltalk.Dictionary,"asNative:",0
,function(lang){var self=this;
var send=nil;
send=((lang._eq("lua")).mustBeBoolean()==true ? (function(){return ":";})() : (function(){return ".";})());
return "smalltalk."._comma(self.$klass.name())._comma(send)._comma(unescape("fromKeys_and_%28"))._comma(self["@keys"].asArray().asNative_(lang))._comma(unescape("%2C"))._comma(self["@values"].asNative_(lang))._comma(unescape("%29"));}
,"converting",unescape("asNative%3A%20lang%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20send%20%7C%0D%0A%09send%20%3A%3D%20lang%20%3D%20%23lua%20ifTrue%3A%20%5B%24%3A%5D%20ifFalse%3A%20%5B%24.%5D.%0D%0A%09%5E%27smalltalk.%27%20%2Cself%20class%20name%0D%0A%09%09%2Csend%20%2C%27fromKeys_and_%28%27%0D%0A%09%09%09%2C%28keys%20asArray%20asNative%3A%20lang%29%0D%0A%09%09%09%2C%27%2C%27%20%2C%28values%20asNative%3A%20lang%29%0D%0A%09%09%2C%27%29%27"));
smalltalk.bind(smalltalk.Dictionary,"asLua",0
,function(){var self=this;
return "smalltalk."._comma(self.$klass.name())._comma(unescape("%3AfromKeys_and_%28"))._comma(self["@keys"].asArray().asLua())._comma(unescape("%2C"))._comma(self["@values"].asLua())._comma(unescape("%29"));}
,"Lua-lua",unescape("asLua%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%27smalltalk.%27%20%2Cself%20class%20name%0D%0A%09%2C%20%27%3AfromKeys_and_%28%27%0D%0A%09%09%2C%20keys%20asArray%20asLua%0D%0A%09%09%2C%27%2C%27%20%2Cvalues%20asLua%0D%0A%09%2C%27%29%27"));
smalltalk.bind(smalltalk.PoolDictionary.$klass,"@@","_at_at"
,function(aHash){var self=this;
return (function($rec){$rec.instVarAt_put_("hash",aHash);return $rec.yourself();})(self.basicNew());}
,"instantiation",unescape("@@%20aHash%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%09%22%20The%20instance%20will%20hack%20aHash%20violating%20encapsulation%20of%20aHash%20and%20instance.%0D%0A%09%20Both%20will%20share%20internal%20state.%20%22%0D%0A%0D%0A%09%5Eself%20basicNew%0D%0A%09%09instVarAt%3A%20%23hash%20put%3A%20aHash%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.PoolDictionary,"initialize",0
,function(){var self=this;
smalltalk.superImplementor(smalltalk.PoolDictionary,"initialize").apply(self, []);
self["@hash"]=self.newHash();
return self;}
,"initialize",unescape("initialize%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20initialize.%0D%0A%09hash%20%3A%3D%20self%20newHash."));
smalltalk.bind(smalltalk.PoolDictionary,"validKey:",0
,function(aKey){var self=this;

return aKey;}
,"initialize",unescape("validKey%3A%20aKey%0D%0A%09%22%20Private%20-%20Returns%20a%20valid%20key%20to%20store%20a%20value%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%22%28self%20typeOf%3A%20aKey%29%20%3D%20%23string%20ifFalse%3A%20%5B%20%5Eself%20error%3A%20%27Invalid%20key%20%28must%20be%20aString%29.%27%20%5D.%22%0D%0A%09%5EaKey%20"));
smalltalk.bind(smalltalk.PoolDictionary,"at:put:",0
,function(aKey,aValue){var self=this;
self["@size"]=nil;

(self["@hash"][self.validKey_(aKey)]=(self.wrap_(aValue)));
return aValue;}
,"accessing",unescape("at%3A%20aKey%20put%3A%20aValue%0D%0A%09%22%20Set%20the%20value%20at%20aKey%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09size%20%3A%3D%20nil.%0D%0A%09%7B%27self%27%7D.%0D%0A%09hash%09basicAt%3A%20%28self%20validKey%3A%20aKey%29%0D%0A%09%09put%3A%20%28self%20wrap%3A%20aValue%29.%0D%0A%09%5EaValue"));
smalltalk.bind(smalltalk.PoolDictionary,"removeKey:ifAbsent:",0
,function(aKey,aBlock){var self=this;
if (!(self.includesKey_(aKey))){return aBlock.value();};;
self["@size"]=nil;
delete self["@hash"][aKey];
return self;}
,"removing",unescape("removeKey%3A%20aKey%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Remove%20the%20binding%20at%20aKey.%20%22%0D%0A%0D%0A%09%28self%20includesKey%3A%20aKey%29%20ifFalse%3A%20%5B%20%5EaBlock%20value%20%5D.%0D%0A%09size%20%3A%3D%20nil.%0D%0A%09%7B%27%27%20doc%27native%20remotion%27%7D"));
smalltalk.bind(smalltalk.PoolDictionary,"size",0
,function(){var self=this;
if ((nil.isNil_(self["@size"]))){self["@size"]=self.keys().size();};;
return self["@size"];}
,"accessing",unescape("size%0D%0A%09%22%20Return%20the%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09size%20isNil%20ifTrue%3A%20%5B%20size%20%3A%3D%20self%20keys%20size%20%5D.%0D%0A%09%5Esize"));
smalltalk.bind(smalltalk.PoolDictionary,"at:ifAbsent:",0
,function(aKey,aBlock){var self=this;
var found = Object.prototype.hasOwnProperty.call(self["@hash"],aKey);
	if(found) return self["@hash"][aKey];
	return aBlock.value();
return self;}
,"accessing",unescape("at%3A%20aKey%20ifAbsent%3A%20aBlock%0D%0A%09%22%20Return%20the%20value%20at%20aKey%20or%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20indexing%27%7D"));
smalltalk.bind(smalltalk.PoolDictionary,"includesKey:",0
,function(aKey){var self=this;
return Object.prototype.hasOwnProperty.call(self["@hash"],aKey);}
,"testing",unescape("includesKey%3A%20aKey%0D%0A%09%22%20Return%20true%20if%20the%20receiver%27s%20keys%20include%20aKey.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20testing%27%7D"));
smalltalk.bind(smalltalk.PoolDictionary,"keys",0
,function(){var self=this;
return Object.getOwnPropertyNames(self["@hash"]);
return self;}
,"accessing",unescape("keys%0D%0A%09%22%20Return%20the%20keys%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.PoolDictionary,"keysAndValuesDo:",0
,function(aBlock){var self=this;
self.keysDo_((function(key){return aBlock.value_value_(key,self.at_(key));}));
return self;}
,"enumerating",unescape("keysAndValuesDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20key/value%20pairs%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20keysDo%3A%20%5B%3Akey%20%7C%0D%0A%09%09aBlock%20value%3A%20key%20value%3A%20%28self%20at%3A%20key%29%0D%0A%09%5D"));
smalltalk.bind(smalltalk.PoolDictionary,"values",0
,function(){var self=this;
return self.keys().collect_((function(key){return self.at_(key);}));}
,"accessing",unescape("values%0D%0A%09%22%20Return%20the%20values%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20keys%20collect%3A%20%5B%3Akey%7C%20self%20at%3A%20key%20%5D"));
smalltalk.bind(smalltalk.PoolDictionary,"asNative:",0
,function(lang){var self=this;
var send=nil;
send=((lang._eq("lua")).mustBeBoolean()==true ? (function(){return ":";})() : (function(){return ".";})());
return "smalltalk."._comma(self.$klass.name())._comma(send)._comma(unescape("fromTuples_%28"))._comma(self.tuples().asNative_(lang))._comma(unescape("%29"));}
,"converting",unescape("asNative%3A%20lang%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20send%20%7C%0D%0A%09send%20%3A%3D%20lang%20%3D%20%23lua%20ifTrue%3A%20%5B%24%3A%5D%20ifFalse%3A%20%5B%24.%5D.%0D%0A%09%5E%27smalltalk.%27%20%2Cself%20class%20name%0D%0A%09%09%2Csend%20%2C%27fromTuples_%28%27%0D%0A%09%09%2C%28self%20tuples%20asNative%3A%20lang%29%0D%0A%09%09%2C%27%29%27"));
smalltalk.bind(smalltalk.PoolDictionary,"detect:ifNone:",0
,function(aBlock,anotherBlock){var $s8Ret$={name:"stReturn"};try{var self=this;
self.keysAndValuesDo_((function(key,value){return ((aBlock.value_(value)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return value})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return anotherBlock.value()})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"enumerating",unescape("detect%3A%20aBlock%20ifNone%3A%20anotherBlock%0D%0A%09%22%20Detect%20the%20value%20who%20matches%20aBlock.%0D%0A%09Return%20the%20result%20of%20evaluating%20aBlock%20if%20no%20value%20matches%20detection%20block.%0D%0A%09%22%0D%0A%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%20%7C%0D%0A%09%09%28aBlock%20value%3A%20value%29%20ifTrue%3A%20%5B%20%5Evalue%20%5D%0D%0A%09%5D.%0D%0A%09%5EanotherBlock%20value"));
smalltalk.bind(smalltalk.PoolDictionary,"do:",0
,function(aBlock){var self=this;
self.keysAndValuesDo_((function(key,value){return aBlock.value_(value);}));
return self;}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20for%20all%20values%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%20%7C%20aBlock%20value%3A%20value%20%5D"));
smalltalk.bind(smalltalk.PoolDictionary,"includes:",0
,function(anObject){var $s8Ret$={name:"stReturn"};try{var self=this;
self.keysAndValuesDo_((function(key,value){return ((anObject._eq(value)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"testing",unescape("includes%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%20%7C%0D%0A%09%09anObject%20%3D%20value%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09%5D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.PoolDictionary,"keysDo:",0
,function(aBlock){var self=this;
self.keys().do_(aBlock);
return self;}
,"enumerating",unescape("keysDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20with%20the%20keys%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20keys%20do%3A%20aBlock"));
smalltalk.bind(smalltalk.PoolDictionary,"@","_at"
,function(keyOrPath){var $s8Ret$={name:"stReturn"};try{var self=this;
var result=nil;
if (!(keyOrPath.isArray())){return self.at_ifAbsent_(keyOrPath,(function(){return nil;}));};;
result=keyOrPath.inject_into_(self,(function(pool,key){var value=nil;
if ((nil.isNil_((value=pool._at(key))))){(function(){$s8Ret$.result=(function(){return nil})();throw($s8Ret$)})();};;return value;}));
(function(){$s8Ret$.result=(function(){return result})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("@%20keyOrPath%0D%0A%09%22%20Return%20the%20result%20of%20accessing%20the%20receiver%20at%20key%20or%20path%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09keyOrPath%20isArray%20ifFalse%3A%20%5B%20%5Eself%20at%3A%20keyOrPath%20ifAbsent%3A%20%5B%5D%20%5D.%0D%0A%09result%20%3A%3D%20keyOrPath%20inject%3A%20self%20into%3A%20%5B%3Apool%20%3Akey%20%7C%20%7C%20value%20%7C%0D%0A%09%09%28value%20%3A%3D%20pool%20@%20key%29%20isNil%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%09value%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.PoolDictionary,"asLua",0
,function(){var self=this;
return "smalltalk."._comma(self.$klass.name())._comma(unescape("%3AfromTuples_%28"))._comma(self.tuples().asLua())._comma(unescape("%29"));}
,"Lua-lua",unescape("asLua%0D%0A%09%22%20Return%20the%20native%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%27smalltalk.%27%2Cself%20class%20name%0D%0A%09%2C%20%27%3AfromTuples_%28%27%0D%0A%09%09%2C%20self%20tuples%20asLua%0D%0A%09%2C%27%29%27"));
smalltalk.bind(smalltalk.RegularExpression.$klass,"fromString:flag:",0
,function(aString,anotherString){var self=this;
return new RegExp(aString, anotherString);}
,"instantiation",unescape("fromString%3A%20aString%20flag%3A%20anotherString%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%20doc%27native%20instantiation%27%7D"));
smalltalk.bind(smalltalk.RegularExpression.$klass,"fromString:",0
,function(aString){var self=this;
return self.fromString_flag_(aString,"");}
,"instantiation",unescape("fromString%3A%20aString%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%20%20%20%20%5Eself%20fromString%3A%20aString%20flag%3A%20%27%27"));
smalltalk.bind(smalltalk.RegularExpression,"compile:",0
,function(aString){var self=this;
return self.compile(aString);}
,"evaluating",unescape("compile%3A%20aString%0D%0A%09%22%20Return%20the%20result%20of%20compilation%20of%20aString.%20%22%0D%0A%0D%0A%09%5Eself%20%23compile%3A%20aString"));
smalltalk.bind(smalltalk.RegularExpression,"exec:",0
,function(aString){var self=this;
return self.exec(aString);}
,"evaluating",unescape("exec%3A%20aString%0D%0A%09%22%20Return%20the%20result%20of%20excecution%20of%20aString.%20%22%0D%0A%0D%0A%09%5Eself%20%23exec%3A%20aString"));
smalltalk.bind(smalltalk.RegularExpression,"test:",0
,function(aString){var self=this;
return self.test(aString);}
,"evaluating",unescape("test%3A%20aString%0D%0A%09%22%20Return%20the%20result%20of%20testing%20of%20aString.%20%22%0D%0A%0D%0A%09%5Eself%20%23test%3A%20aString"));
smalltalk.bind(smalltalk.RegularExpression,"shallowCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.RegularExpression,"deepCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.RegularExpression,"asJSONObject",0
,function(){var self=this;
return self;}
,"json",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Exception,"captureStack",0
,function(){var self=this;

return self;}
,"private",unescape("captureStack%0D%0A%09%22%20Private%2C%20Internal%20-%20Capture%20the%20native%20stack%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27self%27%20doc%27Must%20return%20the%20native%20stack%27%7D"));
smalltalk.bind(smalltalk.Exception,"nativeStack",0
,function(){var self=this;
return ((function(){var $1$=(self[unescape("@stack")]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"private",unescape("nativeStack%0D%0A%09%22%20Private%2C%20Internal%20-%20Return%20the%20native%20stack%20of%20the%20receiver%20%28or%20nil%29.%0D%0A%09Note%20that%20the%20native%20stack%20can%20be%20nativelly%20captured%20during%20signal.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27@stack%27"));
smalltalk.bind(smalltalk.Exception,"nativeException",0
,function(){var self=this;
return ((function(){var $1$=(self[unescape("@nativeException")]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"private",unescape("nativeException%0D%0A%09%22%20Private%2C%20Internal%20-%20Return%20the%20native%20exception%20associated%20with%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27@nativeException%27"));
smalltalk.bind(smalltalk.Exception,"nativeException:",0
,function(nex){var self=this;
return (self[unescape("@nativeException")]=(nex));}
,"private",unescape("nativeException%3A%20nex%0D%0A%09%22%20Private%2C%20Internal%20-%20Set%20the%20native%20exception%20associated%20with%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27@nativeException%27%20put%3A%20nex"));
smalltalk.bind(smalltalk.Exception,"stack",0
,function(){var self=this;
var nex=nil;
var stk=nil;
stk=self.nativeStack();
if (stk.size()._gt((0))){return stk;};;
nex=self.nativeException();
if ((nil.isNil_(nex))){return "";};;
stk=((function(){var $1$=(nex["stack"]);if(nil.isNil_($1$))return nil;return $1$;})());
return (((nil.isNil_(stk))).mustBeBoolean()==true ? (function(){return "";})() : (function(){return stk;})());}
,"stack",unescape("stack%0D%0A%09%22%20Private%20-%20Return%20a%20stack%20trace%20string%20%28or%20an%20empty%20string%29.%20%22%0D%0A%0D%0A%09%7C%20nex%20stk%20%7C%0D%0A%09stk%20%3A%3D%20self%20nativeStack.%0D%0A%09stk%20size%20%3E%200%20ifTrue%3A%20%5B%20%5Estk%20%5D.%0D%0A%09nex%20%3A%3D%20self%20nativeException.%0D%0A%09nex%20isNil%20ifTrue%3A%20%5B%20%5E%27%27%20%5D.%0D%0A%09stk%20%3A%3D%20nex%20basicAt%3A%20%23stack.%0D%0A%09%5Estk%20isNil%20ifTrue%3A%20%5B%27%27%5D%20ifFalse%3A%20%5Bstk%5D"));
smalltalk.bind(smalltalk.Exception,"toString",0
,function(){var self=this;
var txt=nil;
var stk=nil;
txt=self.messageText();
if ((nil.isNil_(txt))){txt=unescape("%7Bst%3A%20")._comma(self.description())._comma(unescape("%7D"));};;
stk=self.stack();
if (stk.size()._eq((0))){return txt;};;
return (function($rec){$rec.nextPutAll_(txt);$rec.cr();$rec.nextPutAll_(stk);return $rec.contents();})("".writeStream());}
,"converting",unescape("toString%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20String.%20%22%0D%0A%0D%0A%09%7C%20txt%20stk%20%7C%0D%0A%09txt%20%3A%3D%20self%20messageText.%0D%0A%09txt%20isNil%20ifTrue%3A%20%5B%20txt%20%3A%3D%20%27%7Bst%3A%20%27%20%2Cself%20description%20%2C%24%7D%20%5D.%0D%0A%09stk%20%3A%3D%20self%20stack.%0D%0A%09stk%20size%20%3D%200%20ifTrue%3A%20%5B%20%5Etxt%20%5D.%0D%0A%09%5E%27%27%20writeStream%0D%0A%09%09nextPutAll%3A%20txt%3B%20cr%3B%0D%0A%09%09nextPutAll%3A%20stk%3B%0D%0A%09%09contents"));
smalltalk.bind(smalltalk.Exception,"messageText",0
,function(){var self=this;
return self["@messageText"];}
,"accessing",unescape("messageText%0D%0A%09%22%20Return%20the%20messageText%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EmessageText"));
smalltalk.bind(smalltalk.Exception,"description",0
,function(){var self=this;
var txt=nil;
var ex=nil;
txt=self.messageText();
if ((nil.isNil_(txt))){txt="";};;
txt=self.toString_(txt);
ex=self.nativeException();
if ((nil.isNil_(ex)===false)){txt=txt._comma(unescape("%20%5B"))._comma(self.typeOf_(ex))._comma(unescape("%5D%20"))._comma(self.toString_(ex));};;
return self.$klass.name()._comma(unescape("%20-%20"))._comma(txt);}
,"accessing",unescape("description%0D%0A%09%22%20Return%20the%20description%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20txt%20ex%20%7C%0D%0A%09txt%20%3A%3D%20self%20messageText.%0D%0A%09txt%20isNil%20ifTrue%3A%20%5B%20txt%20%3A%3D%20%27%27%20%5D.%0D%0A%09txt%20%3A%3D%20self%20toString%3A%20txt.%0D%0A%09ex%20%3A%3D%20self%20nativeException.%0D%0A%09ex%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09txt%20%3A%3D%20txt%0D%0A%09%09%09%2C%27%20%5B%27%20%2C%28self%20typeOf%3A%20ex%29%0D%0A%09%09%09%2C%27%5D%20%27%20%2C%28self%20toString%3A%20ex%29%0D%0A%09%5D.%0D%0A%09%5Eself%20class%20name%20%2C%27%20-%20%27%20%2Ctxt%20"));
smalltalk.bind(smalltalk.Exception,"messageText:",0
,function(aString){var self=this;
self["@messageText"]=aString;
return self;}
,"accessing",unescape("messageText%3A%20aString%0D%0A%09%22%20Set%20the%20messageText%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09messageText%20%3A%3D%20aString"));
smalltalk.bind(smalltalk.Exception,"signal",0
,function(){var self=this;
self.note_(self.description());
throw(self);
return self;}
,"signaling",unescape("signal%0D%0A%09%22%20Signal/throw%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20note%3A%20self%20description.%0D%0A%09%7B%27%27%20doc%27jump%20into%20native%20exception%27%7D"));
smalltalk.bind(smalltalk.Exception,"__tostring",0
,function(){var self=this;
return self.toString();}
,"Lua-printing",unescape("__tostring%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.Error.$klass,"signal:",0
,function(aString){var self=this;
return (function($rec){$rec.messageText_(aString);$rec.captureStack();return $rec.signal();})(self.$$new());}
,"instantiation",unescape("signal%3A%20aString%0D%0A%09%22%20Signal%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09messageText%3A%20aString%3B%0D%0A%09%09captureStack%3B%0D%0A%09%09signal"));
smalltalk.bind(smalltalk.MessageNotUnderstood,"message",0
,function(){var self=this;
return self["@message"];}
,"accessing",unescape("message%0D%0A%09%22%20Return%20the%20message%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Emessage"));
smalltalk.bind(smalltalk.MessageNotUnderstood,"message:",0
,function(aMessage){var self=this;
self["@message"]=aMessage;
return self;}
,"accessing",unescape("message%3A%20aMessage%0D%0A%09%22%20Set%20the%20message%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09message%20%3A%3D%20aMessage"));
smalltalk.bind(smalltalk.MessageNotUnderstood,"receiver",0
,function(){var self=this;
return self["@receiver"];}
,"accessing",unescape("receiver%0D%0A%09%22%20Return%20the%20receiver%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ereceiver"));
smalltalk.bind(smalltalk.MessageNotUnderstood,"receiver:",0
,function(anObject){var self=this;
self["@receiver"]=anObject;
return self;}
,"accessing",unescape("receiver%3A%20anObject%0D%0A%09%22%20Set%20the%20receiver%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09receiver%20%3A%3D%20anObject"));
smalltalk.bind(smalltalk.MessageNotUnderstood,"messageText",0
,function(){var self=this;
var type=nil;
if ((nil.isNil_(self["@message"]))){return "unknown";};;
type=((self.isObject_(self["@receiver"])).mustBeBoolean()==true ? (function(){return "instance of "._comma(self["@receiver"].$klass.name());})() : (function(){return "native object";})());
return ((self["@message"].isString()).mustBeBoolean()==true ? (function(){return self["@message"].asLiteral();})() : (function(){return self["@message"].selector();})())._comma(" not understood by ")._comma(type);}
,"accessing",unescape("messageText%0D%0A%09%22%20Return%20the%20message%20text%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20type%20%7C%0D%0A%09message%20isNil%20ifTrue%3A%20%5B%20%5E%23unknown%20%5D.%0D%0A%09type%20%3A%3D%20%28self%20isObject%3A%20receiver%29%0D%0A%09%09ifTrue%3A%20%5B%20%27instance%20of%20%27%20%2Creceiver%20class%20name%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20%27native%20object%27%20%5D.%0D%0A%09%5E%28message%20isString%0D%0A%09%09ifTrue%3A%20%5B%20message%20asLiteral%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20message%20selector%20%5D%29%0D%0A%09%09%2C%27%20not%20understood%20by%20%27%20%2Ctype"));
smalltalk.bind(smalltalk.Association.$klass,"key:",0
,function(aKey){var self=this;
return self.$$new().key_(aKey);}
,"instantiation",unescape("key%3A%20aKey%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%20key%3A%20aKey"));
smalltalk.bind(smalltalk.Association.$klass,"key:value:",0
,function(aKey,aValue){var self=this;
return self.key_(aKey).value_(aValue);}
,"instantiation",unescape("key%3A%20aKey%20value%3A%20aValue%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28self%20key%3A%20aKey%29%20value%3A%20aValue"));
smalltalk.bind(smalltalk.Association,"=","_eq"
,function(anAssociation){var self=this;
return self.$klass._eq(anAssociation.$klass).and_((function(){return self.key()._eq(anAssociation.key()).and_((function(){return self.value()._eq(anAssociation.value());}));}));}
,"comparing",unescape("%3D%20anAssociation%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%5Eself%20class%20%3D%20anAssociation%20class%20and%3A%20%5B%0D%0A%09%20%20%20%20self%20key%20%3D%20anAssociation%20key%20and%3A%20%5B%0D%0A%09%09self%20value%20%3D%20anAssociation%20value%5D%5D"));
smalltalk.bind(smalltalk.Association,"key:",0
,function(aKey){var self=this;
self["@key"]=aKey;
return self;}
,"accessing",unescape("key%3A%20aKey%0D%0A%09%22%20Set%20the%20key%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09key%20%3A%3D%20aKey"));
smalltalk.bind(smalltalk.Association,"key",0
,function(){var self=this;
return self["@key"];}
,"accessing",unescape("key%0D%0A%09%22%20Return%20the%20key%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ekey"));
smalltalk.bind(smalltalk.Association,"value:",0
,function(aValue){var self=this;
self["@value"]=aValue;
return self;}
,"accessing",unescape("value%3A%20aValue%0D%0A%09%22%20Set%20the%20value%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09value%20%3A%3D%20aValue"));
smalltalk.bind(smalltalk.Association,"value",0
,function(){var self=this;
return self["@value"];}
,"accessing",unescape("value%0D%0A%09%22%20Return%20the%20value%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Evalue"));
smalltalk.bind(smalltalk.Association,",","_comma"
,function(anAssociation){var self=this;
return smalltalk.ComposedAssociations.with_with_(self,anAssociation);}
,"composing",unescape("%2C%20anAssociation%0D%0A%09%22%20Return%20the%20receiver%20composed%20with%20the%20argument.%20%22%0D%0A%0D%0A%09%5EComposedAssociations%20with%3A%20self%20with%3A%20anAssociation"));
smalltalk.bind(smalltalk.Association,"asJSONObject",0
,function(){var self=this;
return smalltalk.ComposedAssociations.with_(self).json();}
,"converting",unescape("asJSONObject%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20hash%20object.%20%22%0D%0A%0D%0A%09%5E%28ComposedAssociations%20with%3A%20self%29%20json"));
smalltalk.bind(smalltalk.Association,"isAssociation",0
,function(){var self=this;
return true;}
,"testing",unescape("isAssociation%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20anAssociation.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.Association,"printString",0
,function(){var self=this;
return self.toString();}
,"printing",unescape("printString%0D%0A%09%22%20Return%20a%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20toString%20"));
smalltalk.bind(smalltalk.Association,"toString",0
,function(){var self=this;
var string=nil;
string=(function(x){return (function(){return (((nil.isNil_(x))).mustBeBoolean()==true ? (function(){return "nil";})() : (function(){return ((self.isObject_(x)).mustBeBoolean()==true ? (function(){return x.asLiteral();})() : (function(){return self.toString_(x);})());})());}).on_do_(smalltalk.Exception,(function(ex){return unescape("%3F%3F%3F%20")._comma(ex.description());}));});
return unescape("%28")._comma(string.value_(self["@key"]))._comma(unescape("%20-%3E%20"))._comma(string.value_(self["@value"]))._comma(unescape("%29"));}
,"converting",unescape("toString%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20String.%20%22%0D%0A%0D%0A%09%7C%20string%20%7C%0D%0A%09string%20%3A%3D%20%5B%3Ax%7C%20%5B%0D%0A%09%09x%20isNil%20ifTrue%3A%20%5B%20%23nil%20%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09%28self%20isObject%3A%20x%29%20ifTrue%3A%20%5B%20x%20asLiteral%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20self%20toString%3A%20x%20%5D%20%5D%0D%0A%09%09%5D%20on%3A%20Exception%20do%3A%20%5B%3Aex%7C%20%27%3F%3F%3F%20%27%20%2Cex%20description%20%5D%20%5D.%0D%0A%09%5E%24%28%20%2C%28string%20value%3A%20key%29%20%2C%27%20-%3E%20%27%20%2C%28string%20value%3A%20value%29%20%2C%24%29"));
smalltalk.bind(smalltalk.Association,"__tostring",0
,function(){var self=this;
return self.toString();}
,"Lua-printing",unescape("__tostring%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.ClassBuilder,"superclass:subclass:",0
,function(aClass,aString){var self=this;
return self.superclass_subclass_instanceVariableNames_category_(aClass,aString,"",nil);}
,"instantiation",unescape("superclass%3A%20aClass%20subclass%3A%20aString%0D%0A%09%22%20Create/Register%20a%20new%20class%20in%20the%20system.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09superclass%3A%20aClass%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20%27%27%0D%0A%09%09category%3A%20nil"));
smalltalk.bind(smalltalk.ClassBuilder,"superclass:subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:",0
,function(aClass,aString,aString2,classVarNames,poolVarNames,aString3){var self=this;
var result=nil;
var aCategory=nil;
aCategory=(((nil.isNil_(aString3))).mustBeBoolean()==true ? (function(){return (((nil.isNil_(aClass))).mustBeBoolean()==true ? (function(){return "unclassified";})() : (function(){return aClass.category();})());})() : (function(){return aString3;})());
result=self.addSubclassOf_named_instanceVariableNames_(aClass,aString,self.instanceVariableNamesFor_(aString2));
self.setupClass_(result);
(function($rec){$rec.classVariableNames_(classVarNames);$rec.poolDictionaryNames_(poolVarNames);return $rec.category_(aCategory);})(result);
if ((nil.isNil_(aClass)===false).and_((function(){return smalltalk.Smalltalk.includesKey_("Exporter");}))){self.print_(smalltalk.Exporter.$$new().exportNew_(result));};;
return result;}
,"instantiation",unescape("superclass%3A%20aClass%0D%0A%09subclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09classVariableNames%3A%20classVarNames%0D%0A%09poolDictionaries%3A%20poolVarNames%0D%0A%09category%3A%20aString3%0D%0A%09%22%20Create/Register%20a%20new%20class%20in%20the%20system.%20%22%0D%0A%0D%0A%09%7C%20result%20aCategory%20%7C%0D%0A%09aCategory%20%3A%3D%20aString3%20isNil%20ifTrue%3A%20%5B%0D%0A%09%09%09aClass%20isNil%20ifTrue%3A%20%5B%20%23unclassified%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20aClass%20category%20%5D%0D%0A%09%09%5D%20ifFalse%3A%20%5B%20aString3%20%5D.%0D%0A%09result%20%3A%3D%20self%0D%0A%09%09addSubclassOf%3A%20aClass%20named%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20%28self%20instanceVariableNamesFor%3A%20aString2%29.%0D%0A%09self%20setupClass%3A%20result.%0D%0A%09result%20%0D%0A%09%09classVariableNames%3A%20classVarNames%3B%0D%0A%09%09poolDictionaryNames%3A%20poolVarNames%3B%0D%0A%09%09category%3A%20aCategory.%0D%0A%09%28aClass%20notNil%20and%3A%20%5B%20Smalltalk%20includesKey%3A%20%23Exporter%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20print%3A%20%28Exporter%20new%20exportNew%3A%20result%29%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.ClassBuilder,"superclass:subclass:instanceVariableNames:category:",0
,function(aClass,aString,aString2,aString3){var self=this;
return self.superclass_subclass_instanceVariableNames_classVariableNames_poolDictionaries_category_(aClass,aString,aString2,"","",aString3);}
,"instantiation",unescape("superclass%3A%20aClass%0D%0A%09subclass%3A%20aString%0D%0A%09instanceVariableNames%3A%20aString2%0D%0A%09category%3A%20aString3%0D%0A%09%22%20Create/Register%20a%20new%20class%20in%20the%20system.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09superclass%3A%20aClass%0D%0A%09%09subclass%3A%20aString%0D%0A%09%09instanceVariableNames%3A%20aString2%0D%0A%09%09classVariableNames%3A%20%27%27%0D%0A%09%09poolDictionaries%3A%20%27%27%0D%0A%09%09category%3A%20aString3"));
smalltalk.bind(smalltalk.ClassBuilder,"class:comment:",0
,function(aClass,aString){var self=this;
if ((nil.isNil_(aClass)).or_((function(){return aClass.comment()._eq(aString);}))){return self;};;
aClass.basicComment_(aString);
if (smalltalk.Smalltalk.includesKey_("Exporter")){self.print_(smalltalk.Exporter.$$new().exportCommentOf_(aClass));};;
return self;}
,"edition",unescape("class%3A%20aClass%20comment%3A%20aString%0D%0A%09%22%20Set/Register%20the%20class%20comment%20in%20the%20system.%20%22%0D%0A%0D%0A%09%28aClass%20isNil%20or%3A%20%5B%20aClass%20comment%20%3D%20aString%20%5D%29%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09aClass%20basicComment%3A%20aString.%0D%0A%09%28Smalltalk%20includesKey%3A%20%23Exporter%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20print%3A%20%28Exporter%20new%20exportCommentOf%3A%20aClass%29%0D%0A%09%5D."));
smalltalk.bind(smalltalk.ClassBuilder,"basicClass:instanceVariableNames:",0
,function(aMetaClass,anArray){var self=this;
return self.smalltalk().metaVarNames(aMetaClass,anArray);}
,"private",unescape("basicClass%3A%20aMetaClass%20instanceVariableNames%3A%20anArray%0D%0A%09%22%20Private%20-%20Register%20instance%20variables%20onto%20aMetaClass.%20%22%0D%0A%0D%0A%09%5Eself%20smalltalk%20%23metaVarNames%3A%20aMetaClass%20array%3A%20anArray"));
smalltalk.bind(smalltalk.ClassBuilder,"class:instanceVariableNames:",0
,function(aMetaClass,aString){var $s8Ret$={name:"stReturn"};try{var self=this;
var varNames=nil;
if (!(aMetaClass.isMetaclass())){return self.error_(aMetaClass.name()._comma(" is not a metaclass"));};;
varNames=self.instanceVariableNamesFor_(aString);
if (!(varNames._eq(aMetaClass.instanceVariableNames()))){((self.basicClass_instanceVariableNames_(aMetaClass,varNames)).mustBeBoolean()==true ? nil : (function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();})());};;
self.setupClass_(aMetaClass);
if (smalltalk.Smalltalk.includesKey_("Exporter")){self.print_(smalltalk.Exporter.$$new().exportMetaDefinitionOf_(aMetaClass.instanceClass()));};;
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"metaclass",unescape("class%3A%20aMetaClass%20instanceVariableNames%3A%20aString%0D%0A%09%22%20Register%20instance%20variable%20names%20onto%20aMetaClass.%20%22%0D%0A%0D%0A%09%7C%20varNames%20%7C%0D%0A%09aMetaClass%20isMetaclass%20ifFalse%3A%20%5B%0D%0A%09%09%5Eself%20error%3A%20aMetaClass%20name%2C%20%27%20is%20not%20a%20metaclass%27%0D%0A%09%5D.%0D%0A%09varNames%20%3A%3D%20self%20instanceVariableNamesFor%3A%20aString.%0D%0A%09varNames%20%3D%20aMetaClass%20instanceVariableNames%20ifFalse%3A%20%5B%0D%0A%09%09%28self%20basicClass%3A%20aMetaClass%20instanceVariableNames%3A%20varNames%29%20ifFalse%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%5D.%0D%0A%09self%20setupClass%3A%20aMetaClass.%0D%0A%09%28Smalltalk%20includesKey%3A%20%23Exporter%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20print%3A%20%28Exporter%20new%20exportMetaDefinitionOf%3A%20aMetaClass%20instanceClass%29%0D%0A%09%5D.%0D%0A%09%5Etrue%20"));
smalltalk.bind(smalltalk.ClassBuilder,"instanceVariableNamesFor:",0
,function(aString){var self=this;
var result=nil;
result=aString.asArrayOfSubstrings();
if (!(result.size()._eq(result.asSet().size()))){return self.error_("Invalid instance variable names "._comma(result.asLiteral()));};;
return result;}
,"private",unescape("instanceVariableNamesFor%3A%20aString%0D%0A%09%22%20Private%20-%20Return%20the%20instance%20variable%20names%20string.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20aString%20asArrayOfSubstrings.%0D%0A%09result%20size%20%3D%20result%20asSet%20size%20ifFalse%3A%20%5B%0D%0A%09%09%5Eself%20error%3A%20%27Invalid%20instance%20variable%20names%20%27%20%2Cresult%20asLiteral%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.ClassBuilder,"addSubclassOf:named:instanceVariableNames:",0
,function(aClass,aString,aCollection){var self=this;

	smalltalk.addClass(aString, aClass, aCollection);
	return smalltalk[aString];
return self;}
,"private",unescape("addSubclassOf%3A%20aClass%20named%3A%20aString%20instanceVariableNames%3A%20aCollection%0D%0A%09%22%20Private%20-%20Add%20the%20subclass%20to%20system.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20instantiation%27%7D"));
smalltalk.bind(smalltalk.ClassBuilder,"setupClass:",0
,function(aClass){var self=this;
return self;}
,"private",unescape("setupClass%3A%20aClass%0D%0A%09%22%20Private%20-%20Bind/initialize%20the%20class%20delegation%20chain.%20%22"));
smalltalk.bind(smalltalk.Stream.$klass,"on:",0
,function(aCollection){var self=this;
return (function($rec){$rec.collection_(aCollection);$rec.readLimit_(aCollection.size());return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("on%3A%20aCollection%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%20%0D%0A%09%09collection%3A%20aCollection%3B%0D%0A%09%09readLimit%3A%20aCollection%20size%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Stream,"collection",0
,function(){var self=this;
return self["@collection"];}
,"accessing",unescape("collection%0D%0A%09%22%20Private%20-%20Return%20the%20collection%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ecollection"));
smalltalk.bind(smalltalk.Stream,"collection:",0
,function(aCollection){var self=this;
self["@collection"]=aCollection;
return self;}
,"accessing",unescape("collection%3A%20aCollection%0D%0A%09%22%20Private%20-%20Set%20the%20collection%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09collection%20%3A%3D%20aCollection"));
smalltalk.bind(smalltalk.Stream,"setCollection:",0
,function(aCollection){var self=this;
self.collection_(aCollection);
return self;}
,"accessing",unescape("setCollection%3A%20aCollection%0D%0A%09%22%20Private%20-%20Set%20the%20receiver%20stream%20collection%20to%20aCollection.%20%22%0D%0A%0D%0A%09self%20collection%3A%20aCollection"));
smalltalk.bind(smalltalk.Stream,"initialize",0
,function(){var self=this;
smalltalk.superImplementor(smalltalk.Stream,"initialize").apply(self, []);
self["@position"]=(0);
return self;}
,"initialize",unescape("initialize%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20initialize.%0D%0A%09position%20%3A%3D%200."));
smalltalk.bind(smalltalk.Stream,"position",0
,function(){var self=this;
return self["@position"];}
,"accessing",unescape("position%0D%0A%09%22%20Return%20the%20position%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eposition"));
smalltalk.bind(smalltalk.Stream,"position:",0
,function(anInteger){var self=this;
self["@position"]=anInteger;
return self;}
,"accessing",unescape("position%3A%20anInteger%0D%0A%09%22%20Set%20the%20position%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09position%20%3A%3D%20anInteger"));
smalltalk.bind(smalltalk.Stream,"readLimit",0
,function(){var self=this;
if ((nil.is_eqeq_(self["@readLimit"],nil))){self["@readLimit"]=self.collection().size();};;
return self["@readLimit"];}
,"accessing",unescape("readLimit%0D%0A%09%22%20Private%20-%20Return%20the%20read%20limit%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09readLimit%20%3D%3D%20nil%20ifTrue%3A%20%5B%20readLimit%20%3A%3D%20self%20collection%20size%20%5D.%0D%0A%09%5EreadLimit"));
smalltalk.bind(smalltalk.Stream,"readLimit:",0
,function(anInteger){var self=this;
self["@readLimit"]=anInteger;
return self;}
,"accessing",unescape("readLimit%3A%20anInteger%0D%0A%09%22%20Private%20-%20Set%20the%20stream%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09readLimit%20%3A%3D%20anInteger"));
smalltalk.bind(smalltalk.Stream,"contents",0
,function(){var self=this;
var result=nil;
result=self.collection();
if (self.readLimit()._eq(result.size())){return result;};;
return result.copyFrom_to_((1),self.readLimit());}
,"accessing",unescape("contents%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20collection.%0D%0A%09self%20readLimit%20%3D%20result%20size%20ifTrue%3A%20%5B%20%5Eresult%20%5D.%0D%0A%09%5Eresult%20copyFrom%3A%201%20to%3A%20self%20readLimit"));
smalltalk.bind(smalltalk.Stream,"size",0
,function(){var self=this;
return self.readLimit();}
,"accessing",unescape("size%0D%0A%09%22%20Return%20the%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20readLimit"));
smalltalk.bind(smalltalk.Stream,"reset",0
,function(){var self=this;
self.position_((0));
return self;}
,"actions",unescape("reset%0D%0A%09%22%20Reset%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20position%3A%200"));
smalltalk.bind(smalltalk.Stream,"close",0
,function(){var self=this;
return self;}
,"actions",unescape("close%0D%0A%09%22%20Close%20the%20receiver.%20%22"));
smalltalk.bind(smalltalk.Stream,"flush",0
,function(){var self=this;
return self;}
,"io",unescape("flush%0D%0A%09%22%20Flush%20the%20receiver.%20%22"));
smalltalk.bind(smalltalk.Stream,"resetContents",0
,function(){var self=this;
(function($rec){$rec.reset();return $rec.readLimit_((0));})(self);
return self;}
,"actions",unescape("resetContents%0D%0A%09%22%20Reset%20the%20receiver%20contents.%20%22%0D%0A%0D%0A%09self%20reset%3B%20readLimit%3A%200"));
smalltalk.bind(smalltalk.Stream,"do:",0
,function(aBlock){var self=this;
while(!(function(){return self.atEnd();})()){aBlock.value_(self.next());};;
return self;}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20streaming%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5B%20self%20atEnd%20%5D%20whileFalse%3A%20%5B%20aBlock%20value%3A%20self%20next%20%5D"));
smalltalk.bind(smalltalk.Stream,"setToEnd",0
,function(){var self=this;
self.position_(self.size());
return self;}
,"positioning",unescape("setToEnd%0D%0A%09%22%20Position%20the%20receiver%20at%20end.%20%22%0D%0A%0D%0A%09self%20position%3A%20self%20size"));
smalltalk.bind(smalltalk.Stream,"skip:",0
,function(anInteger){var self=this;
self.position_(anInteger._plus(self.position()).min_(self.size()).max_((0)));
return self;}
,"positioning",unescape("skip%3A%20anInteger%0D%0A%09%22%20Skip%20next%20anInteger%20readable%20elements%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20position%3A%20%28%28anInteger%20+%20self%20position%20min%3A%20self%20size%29%20max%3A%200%29"));
smalltalk.bind(smalltalk.Stream,"next",0
,function(){var self=this;
self.position_(self.position()._plus((1)));
return self["@collection"].at_(self.position());}
,"reading",unescape("next%0D%0A%09%22%20Return%20the%20next%20readable%20element%20in%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09self%20position%3A%20self%20position%20+%201.%20%0D%0A%09%5Ecollection%20at%3A%20self%20position"));
smalltalk.bind(smalltalk.Stream,"next:",0
,function(anInteger){var $s8Ret$={name:"stReturn"};try{var self=this;
var result=nil;
result=self.collection().species().$$new();
(function(){for(var $1$=(anInteger);$1$>0;$1$--){(function(){if (self.atEnd()){(function(){$s8Ret$.result=(function(){return result})();throw($s8Ret$)})();};;return result.add_(self.next());})()};return nil})();
(function(){$s8Ret$.result=(function(){return result})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"reading",unescape("next%3A%20anInteger%0D%0A%09%22%20Return%20the%20next%20anInteger%20elements%20read%20from%20the%20receiver.%0D%0A%09If%20the%20receiver%20reach%20its%20end%2C%20the%20returned%20collection%20has%20less%20elements.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20collection%20species%20new.%0D%0A%09anInteger%20timesRepeat%3A%20%5B%0D%0A%09%09self%20atEnd%20ifTrue%3A%20%5B%20%5Eresult%20%5D.%0D%0A%09%09result%20add%3A%20self%20next.%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Stream,"nextPut:",0
,function(anObject){var self=this;
self.position_(self.position()._plus((1)));
self.collection().at_put_(self["@position"],anObject);
self.readLimit_(self.readLimit().max_(self["@position"]));
return self;}
,"writing",unescape("nextPut%3A%20anObject%0D%0A%09%22%20Put%20anObject%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20position%3A%20self%20position%20+%201.%0D%0A%09self%20collection%20at%3A%20position%20put%3A%20anObject.%0D%0A%09self%20readLimit%3A%20%28self%20readLimit%20max%3A%20position%29"));
smalltalk.bind(smalltalk.Stream,"nextPutAll:",0
,function(aCollection){var self=this;
aCollection.do_((function(each){return self.nextPut_(each);}));
return self;}
,"writing",unescape("nextPutAll%3A%20aCollection%0D%0A%09%22%20Put%20aCollection%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09aCollection%20do%3A%20%5B%3Aeach%20%7C%20self%20nextPut%3A%20each%20%5D"));
smalltalk.bind(smalltalk.Stream,"cr",0
,function(){var self=this;
self.nextPutAll_(smalltalk.String.crlf());
return self;}
,"writing",unescape("cr%0D%0A%09%22%20Put%20a%20line%20break%20sequence%20onto%20the%20receiver.%0D%0A%09Note%3A%20this%20implementation%20MUST%20be%20consistent%20with%20%23nextLine%0D%0A%09%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20String%20crlf"));
smalltalk.bind(smalltalk.Stream,"space",0
,function(){var self=this;
self.nextPutAll_(smalltalk.String.space());
return self;}
,"writing",unescape("space%0D%0A%09%22%20Put%20a%20space%20sequence%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20String%20space"));
smalltalk.bind(smalltalk.Stream,"tab",0
,function(){var self=this;
self.nextPutAll_(smalltalk.String.tab());
return self;}
,"writing",unescape("tab%0D%0A%09%22%20Put%20a%20tab%20sequence%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20String%20tab"));
smalltalk.bind(smalltalk.Stream,"peek",0
,function(){var self=this;
return ((self.atEnd()).mustBeBoolean()==true ? nil : (function(){return self.collection().at_(self.position()._plus((1)));})());}
,"reading",unescape("peek%0D%0A%09%22%20Return%20the%20next%20readable%20element%20from%20the%20receiver%20without%20advancing%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20atEnd%20ifFalse%3A%20%5B%0D%0A%09%20%20%20%20self%20collection%20at%3A%20self%20position%20+%201%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Stream,"peekFor:",0
,function(anObject){var self=this;
return anObject._eq(self.peek()).and_((function(){if (!(self.atEnd())){self.next();};;return true;}));}
,"reading",unescape("peekFor%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20the%20next%20readable%20element%20from%20the%20receiver%20equals%20anObject.%0D%0A%09Advance%20the%20receiver%20if%20the%20result%20is%20true.%0D%0A%09%22%0D%0A%0D%0A%09%5EanObject%20%3D%20self%20peek%20and%3A%20%5B%0D%0A%09%09self%20atEnd%20ifFalse%3A%20%5B%20self%20next%20%5D.%0D%0A%09%09true%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Stream,"peekForAll:",0
,function(aCollection){var $s8Ret$={name:"stReturn"};try{var self=this;
var back=nil;
back=self.position();
aCollection.do_((function(each){return ((self.peekFor_(each)).mustBeBoolean()==true ? nil : (function(){self.position_(back);return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();})());}));
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"reading",unescape("peekForAll%3A%20aCollection%0D%0A%09%22%20Return%20true%20if%20the%20next%20readable%20elements%20from%20the%20receiver%20matches%20aCollection.%0D%0A%09Advance%20the%20receiver%20if%20the%20result%20is%20true.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20back%20%7C%0D%0A%09back%20%3A%3D%20self%20position.%0D%0A%09aCollection%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28self%20peekFor%3A%20each%29%20ifFalse%3A%20%5B%0D%0A%09%09%09self%20position%3A%20back.%0D%0A%09%09%09%5Efalse%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.Stream,"atEnd",0
,function(){var self=this;
return self.position()._gt_eq(self.size());}
,"testing",unescape("atEnd%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20at%20end.%20%22%0D%0A%0D%0A%09%5Eself%20position%20%3E%3D%20self%20size"));
smalltalk.bind(smalltalk.Stream,"atStart",0
,function(){var self=this;
return self.position()._eq((0));}
,"testing",unescape("atStart%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20at%20start%20position.%20%22%0D%0A%0D%0A%09%5Eself%20position%20%3D%200"));
smalltalk.bind(smalltalk.Stream,"isEmpty",0
,function(){var self=this;
return self.size()._eq((0));}
,"testing",unescape("isEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20empty.%20%22%0D%0A%0D%0A%09%5Eself%20size%20%3D%200"));
smalltalk.bind(smalltalk.Stream,"notEmpty",0
,function(){var self=this;
return self.isEmpty().not();}
,"testing",unescape("notEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20not%20empty.%20%22%0D%0A%0D%0A%09%5Eself%20isEmpty%20not"));
smalltalk.bind(smalltalk.Stream,"upTo:",0
,function(anObject){var self=this;
var result=nil;
result=self.collection().species().$$new();
while(!(function(){return self.atEnd().or_((function(){return self.peek()._eq(anObject);}));})()){result.add_(self.next());};;
if (!(self.atEnd())){self.next();};;
return result;}
,"reading",unescape("upTo%3A%20anObject%0D%0A%09%22%20Return%20the%20elements%20read%20from%20the%20receiver%20upto%20anObject%20%28or%20end%29.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20collection%20species%20new.%0D%0A%09%5B%20self%20atEnd%20or%3A%20%5B%20self%20peek%20%3D%20anObject%20%5D%5D%20whileFalse%3A%20%5B%0D%0A%09%09result%20add%3A%20self%20next%0D%0A%09%5D.%0D%0A%09self%20atEnd%20ifFalse%3A%20%5B%20self%20next%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Stream,"upToEnd",0
,function(){var self=this;
var result=nil;
result=self.collection().copyFrom_to_(self.position()._plus((1)),self.readLimit());
self.setToEnd();
return result;}
,"reading",unescape("upToEnd%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver%20upto%20end.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20collection%20copyFrom%3A%20self%20position%20+%201%20to%3A%20self%20readLimit.%0D%0A%09self%20setToEnd.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Stream,"nextLine",0
,function(){var self=this;
var result=nil;
var start=nil;
start=self.position();
result=self.upTo_(smalltalk.String.cr());
if (result.includes_(smalltalk.String.lf())){return (function($rec){$rec.position_(start);return $rec.upTo_(smalltalk.String.lf());})(self);};;
if (self.peek()._eq(smalltalk.String.lf())){self.next();};;
return result;}
,"reading",unescape("nextLine%0D%0A%09%22%20Return%20the%20characters%20read%20from%20the%20receiver%20up%20to%20the%20line%20delimiter.%20%22%0D%0A%0D%0A%09%7C%20result%20start%20%7C%0D%0A%09start%20%3A%3D%20self%20position.%0D%0A%09result%20%3A%3D%20self%20upTo%3A%20String%20cr.%0D%0A%09%28result%20includes%3A%20String%20lf%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20position%3A%20start%3B%20upTo%3A%20String%20lf%0D%0A%09%5D.%0D%0A%09%28self%20peek%20%3D%20String%20lf%29%20ifTrue%3A%20%5B%20self%20next%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Stream,"skipTo:",0
,function(anObject){var $s8Ret$={name:"stReturn"};try{var self=this;
while(!(function(){return self.atEnd();})()){((self.next()._eq(anObject)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();})() : nil);};;
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"reading",unescape("skipTo%3A%20anObject%0D%0A%09%22%20Advance%20the%20receiver%20upto%20anObject%20%28or%20end%29.%20%22%0D%0A%0D%0A%09%5B%20self%20atEnd%20%5D%20whileFalse%3A%20%5B%0D%0A%09%09self%20next%20%3D%20anObject%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09%5D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Stream,"skipSeparators",0
,function(){var $s8Ret$={name:"stReturn"};try{var self=this;
while((function(){if (self.atEnd()){(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();};;return self.peek().isSeparator();})()){self.next();};;
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"reading",unescape("skipSeparators%0D%0A%09%22%20Advance%20the%20receiver%20upto%20a%20non-separator%20%28or%20end%29.%0D%0A%09Return%20true%20%20if%20a%20non-separator%20contents%20is%20found%2C%0D%0A%09return%20false%20if%20run%20upto%20end%20of%20stream.%0D%0A%09%22%0D%0A%0D%0A%09%5B%20self%20atEnd%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%20%20self%20peek%20isSeparator%0D%0A%09%5D%20whileTrue%3A%20%5B%20self%20next%20%5D.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.Stream,"nextChunk",0
,function(){var self=this;
var result=nil;
result=self.upTo_(unescape("%21"));
while((function(){return self.peek()._eq(unescape("%21"));})()){self.next();
result=result._comma(unescape("%21"))._comma(self.upTo_(unescape("%21")));};;
return result.trimFirst_((function(c){return c.asciiValue()._lt_eq((32));}));}
,"Tools-fileIn",unescape("nextChunk%0D%0A%09%22%20FileIn%20-%20Return%20the%20next%20chunk%20read%20from%20the%20receiver.%0D%0A%09Trailing%20non-printing%20characters%20are%20skipped.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20upTo%3A%20%24%21.%0D%0A%09%5B%20self%20peek%20%3D%20%24%21%20%5D%20whileTrue%3A%20%5B%0D%0A%09%09self%20next.%0D%0A%09%09result%20%3A%3D%20result%2C%20%27%21%27%2C%20%28self%20upTo%3A%20%24%21%29%0D%0A%09%5D.%0D%0A%09%5Eresult%20trimFirst%3A%20%5B%3Ac%7C%20c%20asciiValue%20%3C%3D%2032%20%5D"));
smalltalk.bind(smalltalk.Stream,"fileInto:",0
,function(aReceiver){var self=this;
var scanner=nil;
while(!(function(){return self.atEnd();})()){if ((nil.isNil_(scanner))){scanner=aReceiver;};;
scanner=scanner.scanFrom_(self);};;
return scanner;}
,"Tools-fileIn",unescape("fileInto%3A%20aReceiver%0D%0A%09%22%20FileIn%20-%20FileIn%20the%20receiver%27s%20contents%20%28in%20chunk%20format%29%20using%20aReceiver%20as%20default%20scanner.%20%22%0D%0A%0D%0A%09%7C%20scanner%20%7C%0D%0A%09%5Bself%20atEnd%5D%20whileFalse%3A%20%5B%0D%0A%09%09scanner%20isNil%20ifTrue%3A%20%5B%20scanner%20%3A%3D%20aReceiver%20%5D.%0D%0A%09%09scanner%20%3A%3D%20scanner%20scanFrom%3A%20self%0D%0A%09%5D.%0D%0A%09%5Escanner"));
smalltalk.bind(smalltalk.Stream,"fileIn",0
,function(){var self=this;
return self.fileInto_(nil);}
,"Tools-fileIn",unescape("fileIn%0D%0A%09%22%20FileIn%20-%20FileIn%20the%20receiver%27s%20contents%20%28in%20chunk%20format%29.%20%22%0D%0A%0D%0A%09%5Eself%20fileInto%3A%20nil"));
smalltalk.bind(smalltalk.Stream,"nextChunkPut:",0
,function(aString){var self=this;
var eoc=nil;
var parts=nil;
eoc=unescape("%21");
if ((nil.isNil_(aString))){self.nextPut_(eoc);
return nil;};;
if (aString.isEmpty()){self.nextPut_(" ");};;
aString.asArrayOfSubstringsSeparatedBy_(eoc).do_separatedBy_((function(each){return self.nextPutAll_(each);}),(function(){return self.nextPutAll_(eoc._comma(eoc));}));
self.nextPut_(eoc);
return aString;}
,"Tools-fileOut",unescape("nextChunkPut%3A%20aString%0D%0A%09%22%20Output%20aString%20in%20chunk%20format.%20%22%0D%0A%0D%0A%09%7C%20eoc%20parts%20%7C%0D%0A%09eoc%20%3A%3D%20%24%21.%0D%0A%09aString%20isNil%20ifTrue%3A%20%5B%20%20self%20nextPut%3A%20eoc.%20%5Enil%20%5D.%0D%0A%09aString%20isEmpty%20ifTrue%3A%20%5B%20self%20nextPut%3A%20%24%20%5D.%0D%0A%09%28aString%20asArrayOfSubstringsSeparatedBy%3A%20eoc%29%0D%0A%09%09do%3A%20%5B%3Aeach%7C%20self%20nextPutAll%3A%20each%20%5D%0D%0A%09%09separatedBy%3A%20%5B%20self%20nextPutAll%3A%20eoc%2Ceoc%20%5D.%0D%0A%09self%20nextPut%3A%20eoc.%0D%0A%09%5EaString"));
smalltalk.bind(smalltalk.StringStream,"next:",0
,function(anInteger){var self=this;
var start=nil;
var result=nil;
var stop=nil;
start=self.position();
stop=start._plus(anInteger).min_(self.readLimit());
result=self.collection().copyFrom_to_(start._plus((1)),stop);
self.position_(stop);
return result;}
,"reading",unescape("next%3A%20anInteger%0D%0A%09%22%20Return%20the%20next%20anInteger%20elements%20read%20from%20the%20receiver.%0D%0A%09If%20the%20receiver%20reach%20its%20end%2C%20the%20returned%20collection%20has%20less%20elements.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20start%20result%20stop%20%7C%0D%0A%09start%20%3A%3D%20self%20position.%0D%0A%09stop%20%3A%3D%20start%20+%20anInteger%20min%3A%20self%20readLimit.%0D%0A%09result%20%3A%3D%20self%20collection%20copyFrom%3A%20start%20+%201%20to%3A%20stop.%0D%0A%09self%20position%3A%20stop.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.StringStream,"nextPut:",0
,function(aString){var self=this;
self.nextPutAll_(aString);
return self;}
,"writing",unescape("nextPut%3A%20aString%0D%0A%09%22%20Put%20aString%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20aString"));
smalltalk.bind(smalltalk.StringStream,"nextPutAll:",0
,function(aString){var self=this;
var result=nil;
if (self.position()._eq(self.collection().size())){result=self.collection()._comma(aString);} else{var substring=nil;
substring=self.collection().copyFrom_to_((1),self.position());
result=((substring.isEmpty()).mustBeBoolean()==true ? (function(){return aString;})() : (function(){return substring._comma(aString);})());
substring=self.collection().copyFrom_to_(self.position()._plus((1))._plus(aString.size()),self.collection().size());
result=((substring.isEmpty()).mustBeBoolean()==true ? (function(){return result;})() : (function(){return result._comma(substring);})());};;
self.collection_(result);
self.position_(self.position()._plus(aString.size()));
self.readLimit_(self.readLimit().max_(self["@position"]));
return self;}
,"writing",unescape("nextPutAll%3A%20aString%0D%0A%09%22%20Put%20aString%20onto%20the%20receiver%20at%20current%20position%3B%20expanding%20internal%20contents.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09self%20position%20%3D%20self%20collection%20size%0D%0A%09ifTrue%3A%20%5B%20result%20%3A%3D%20self%20collection%20%2C%20aString%20%5D%0D%0A%09ifFalse%3A%20%5B%20%7C%20substring%20%7C%0D%0A%09%09substring%20%3A%3D%20self%20collection%20copyFrom%3A%201%20to%3A%20self%20position.%0D%0A%09%09result%20%3A%3D%20substring%20isEmpty%20ifTrue%3A%20%5B%20aString%20%5D%20ifFalse%3A%20%5B%20substring%20%2C%20aString%20%5D.%0D%0A%09%09substring%20%3A%3D%20self%20collection%0D%0A%09%09%09copyFrom%3A%20self%20position%20+%201%20+%20aString%20size%0D%0A%09%09%09to%3A%20self%20collection%20size.%0D%0A%09%09result%20%3A%3D%20substring%20isEmpty%20ifTrue%3A%20%5B%20result%20%5D%20ifFalse%3A%20%5B%20result%20%2C%20substring%20%5D.%0D%0A%09%5D.%0D%0A%09self%20collection%3A%20result.%0D%0A%09self%20position%3A%20self%20position%20+%20aString%20size.%0D%0A%09self%20readLimit%3A%20%28self%20readLimit%20max%3A%20position%29"));
smalltalk.bind(smalltalk.StringStream,"skipTo:",0
,function(anObject){var self=this;
var index=nil;
if (!(anObject.isString())){self.setToEnd();
return false;};;
index=self.collection().indexOf_startingAt_(anObject,self.position()._plus((1)));
if (index.between_and_(self.position()._plus((1)),self.readLimit())){self.position_(index._plus(anObject.size())._minus((1)));
return true;};;
self.setToEnd();
return false;}
,"reading",unescape("skipTo%3A%20anObject%0D%0A%09%22%20Advance%20the%20receiver%20upto%20anObject%20%28or%20end%29.%20%22%0D%0A%0D%0A%09%7C%20index%20%7C%0D%0A%09anObject%20isString%20ifFalse%3A%20%5B%20self%20setToEnd.%20%5Efalse%20%5D.%0D%0A%09index%20%3A%3D%20self%20collection%20indexOf%3A%20anObject%20startingAt%3A%20self%20position%20+%201.%0D%0A%09%28index%20between%3A%20self%20position%20+%201%20and%3A%20self%20readLimit%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20position%3A%20index%20+%20anObject%20size%20-%201.%0D%0A%09%09%5Etrue%0D%0A%09%5D.%0D%0A%09self%20setToEnd.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.StringStream,"upTo:",0
,function(anObject){var self=this;
var start=nil;
var found=nil;
if (!(anObject.isString())){return self.upToEnd();};;
start=self.position()._plus((1));
found=((self.skipTo_(anObject)).mustBeBoolean()==true ? (function(){return anObject.size();})() : (function(){return (0);})());
return self.collection().copyFrom_to_(start,self.position()._minus(found));}
,"reading",unescape("upTo%3A%20anObject%0D%0A%09%22%20Return%20the%20elements%20read%20from%20the%20receiver%20upto%20anObject%20%28or%20end%29.%20%22%0D%0A%0D%0A%09%7C%20start%20found%20%7C%0D%0A%09anObject%20isString%20ifFalse%3A%20%5B%20%5Eself%20upToEnd%20%5D.%0D%0A%09start%20%3A%3D%20self%20position%20+%201.%0D%0A%09found%20%3A%3D%20%28self%20skipTo%3A%20anObject%29%20ifTrue%3A%20%5B%20anObject%20size%20%5D%20ifFalse%3A%20%5B%200%20%5D.%0D%0A%09%5Eself%20collection%20copyFrom%3A%20start%20to%3A%20self%20position%20-%20found"));
smalltalk.bind(smalltalk.StringStream,"peek",0
,function(){var self=this;
return self["@collection"].charAt(self.position())||nil;
return self;}
,"reading",unescape("peek%0D%0A%09%22%20Return%20the%20next%20readable%20element%20from%20the%20receiver%20without%20advancing%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.StringStream,"next",0
,function(){var self=this;
var result=nil;
result=self.peek();
if ((nil.isNil_(result)===false)){self["@position"]=self["@position"]._plus((1));};;
return result;}
,"reading",unescape("next%0D%0A%09%22%20Return%20the%20next%20readable%20element%20in%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20peek.%0D%0A%09result%20notNil%20ifTrue%3A%20%5B%20position%20%3A%3D%20position%20+%201%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.StringStream,"peekForAll:",0
,function(aCollection){var self=this;
if (!(aCollection.isString())){return smalltalk.superImplementor(smalltalk.StringStream,"peekForAll_").apply(self, [aCollection]);};;
var start = self.position();
	for (var i=0;i<aCollection.length; i++){
		if (self["@collection"].charCodeAt(start+i) != aCollection.charCodeAt(i)) return false;
	};
self["@position"]=self["@position"]._plus(aCollection.size());
return true;}
,"reading",unescape("peekForAll%3A%20aCollection%0D%0A%09%22%20Return%20true%20if%20the%20next%20readable%20elements%20from%20the%20receiver%20matches%20aCollection.%0D%0A%09Advance%20the%20receiver%20if%20the%20result%20is%20true.%0D%0A%09%22%0D%0A%0D%0A%09aCollection%20isString%20ifFalse%3A%20%5B%20%5Esuper%20peekForAll%3A%20aCollection%20%5D.%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D.%0D%0A%09position%20%3A%3D%20position%20+%20aCollection%20size.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.StringStream,"upToSeparator",0
,function(){var self=this;
var start=nil;
start=self.position()._plus((1));
while((function(){return self.atEnd().not().and_((function(){return self.peek().isSeparator().not();}));})()){self.next();};;
return self.collection().copyFrom_to_(start,self.position());}
,"reading",unescape("upToSeparator%0D%0A%09%22%20Return%20the%20elements%20read%20from%20the%20receiver%20upto%20a%20separator%20%28or%20end%29.%20%22%0D%0A%0D%0A%09%7C%20start%20%7C%0D%0A%09start%20%3A%3D%20self%20position%20+%201.%0D%0A%09%5Bself%20atEnd%20not%20and%3A%20%5Bself%20peek%20isSeparator%20not%5D%5D%20whileTrue%3A%20%5B%0D%0A%09%09self%20next%0D%0A%09%5D.%0D%0A%09%5Eself%20collection%20copyFrom%3A%20start%20to%3A%20self%20position"));
smalltalk.bind(smalltalk.StringStream,"nextMessagePattern",0
,function(){var self=this;
var part=nil;
var selector=nil;
var args=nil;
args=[];
self.skipSeparators();
selector=self.upToSeparator();
if (selector.isEmpty()){return nil;};;
if (selector.argumentCount()._gt((0))){(function(){while((function(){self.skipSeparators();args.add_(self.upToSeparator());self.skipSeparators();part=((self.peek()._eq(unescape("%22"))).mustBeBoolean()==true ? (function(){return "";})() : (function(){return self.upToSeparator();})());return part.notEmpty().and_((function(){return part.last()._eq(":");}));})()){(function(){return selector=selector._comma(part);})()};return nil})();};;
part=smalltalk.Message.selector_arguments_(selector,args);
self.skipSeparators();
if (self.peek()._eq(unescape("%22"))){part.comment_((function($rec){$rec.next();return $rec.upTo_(unescape("%22"));})(self).trimBlanks());};;
return part;}
,"tools",unescape("nextMessagePattern%0D%0A%09%22%20Return%20a%20message%20pattern%20read%20from%20the%20receiver%20%28or%20nil%29.%0D%0A%09Note%3A%20it%20is%20assumed%20that%20there%20is%20a%20correct%20message%20header%20in%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20part%20selector%20args%20%7C%0D%0A%09args%20%3A%3D%20%23%28%29.%0D%0A%09self%20skipSeparators.%0D%0A%09selector%20%3A%3D%20self%20upToSeparator.%0D%0A%09selector%20isEmpty%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09selector%20argumentCount%20%3E%200%20ifTrue%3A%20%5B%0D%0A%09%09%5B%20self%20skipSeparators.%0D%0A%09%09%20%20args%20add%3A%20self%20upToSeparator.%0D%0A%09%09%20%20self%20skipSeparators.%0D%0A%09%09%20%20part%20%3A%3D%20self%20peek%20%3D%20%24%22%20ifTrue%3A%20%5B%20%27%27%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20self%20upToSeparator%20%5D.%0D%0A%09%09%20%20part%20notEmpty%20and%3A%20%5B%20part%20last%20%3D%20%24%3A%20%5D%0D%0A%09%09%5D%20whileTrue%3A%20%5B%0D%0A%09%09%20%20selector%20%3A%3D%20selector%20%2C%20part%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09part%20%3A%3D%20Message%20selector%3A%20selector%20arguments%3A%20args.%0D%0A%09self%20skipSeparators.%0D%0A%09self%20peek%20%3D%20%24%22%20ifTrue%3A%20%5B%0D%0A%09%09part%20comment%3A%20%28self%20next%3B%20upTo%3A%20%24%22%29%20trimBlanks%0D%0A%09%5D.%0D%0A%09%5Epart"));
smalltalk.bind(smalltalk.StringWriteStream,"collection",0
,function(){var self=this;
return self.invalidMessage_("collection");}
,"accessing",unescape("collection%0D%0A%09%22%20Private%20-%20Return%20the%20collection%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23collection"));
smalltalk.bind(smalltalk.StringWriteStream,"collection:",0
,function(aCollection){var self=this;
if (!(aCollection.isString())){self.error_("Collection must be a String");};;
self["@collection"]=smalltalk.Array.with_(aCollection);
return self;}
,"accessing",unescape("collection%3A%20aCollection%0D%0A%09%22%20Private%20-%20Set%20the%20collection%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09aCollection%20isString%20ifFalse%3A%20%5B%20self%20error%3A%20%27Collection%20must%20be%20a%20String%27%20%5D.%0D%0A%09collection%20%3A%3D%20Array%20with%3A%20aCollection"));
smalltalk.bind(smalltalk.StringWriteStream,"contents",0
,function(){var self=this;
return self["@collection"].join_("");}
,"accessing",unescape("contents%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ecollection%20join%3A%20%27%27"));
smalltalk.bind(smalltalk.StringWriteStream,"position",0
,function(){var self=this;
return self.readLimit();}
,"accessing",unescape("position%0D%0A%09%22%20Return%20the%20position%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20readLimit"));
smalltalk.bind(smalltalk.StringWriteStream,"position:",0
,function(anInteger){var self=this;
return self.invalidMessage_("position:");}
,"accessing",unescape("position%3A%20anInteger%0D%0A%09%22%20Set%20the%20position%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23position%3A"));
smalltalk.bind(smalltalk.StringWriteStream,"readLimit",0
,function(){var self=this;
return self["@readLimit"];}
,"accessing",unescape("readLimit%0D%0A%09%22%20Private%20-%20Return%20the%20read%20limit%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EreadLimit"));
smalltalk.bind(smalltalk.StringWriteStream,"readLimit:",0
,function(anInteger){var self=this;
self["@readLimit"]=anInteger;
return self;}
,"accessing",unescape("readLimit%3A%20anInteger%0D%0A%09%22%20Private%20-%20Set%20the%20stream%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09readLimit%20%3A%3D%20anInteger"));
smalltalk.bind(smalltalk.StringWriteStream,"setCollection:",0
,function(aCollection){var self=this;
return self.invalidMessage_("setCollection:");}
,"accessing",unescape("setCollection%3A%20aCollection%0D%0A%09%22%20Private%20-%20Set%20the%20receiver%20stream%20collection%20to%20aCollection.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23setCollection%3A"));
smalltalk.bind(smalltalk.StringWriteStream,"size",0
,function(){var self=this;
return self.readLimit();}
,"accessing",unescape("size%0D%0A%09%22%20Return%20the%20size%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20readLimit"));
smalltalk.bind(smalltalk.StringWriteStream,"close",0
,function(){var self=this;
return self;}
,"actions",unescape("close%0D%0A%09%22%20Close%20the%20receiver.%20%22"));
smalltalk.bind(smalltalk.StringWriteStream,"reset",0
,function(){var self=this;
self.initialize();
return self;}
,"actions",unescape("reset%0D%0A%09%22%20Reset%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20initialize"));
smalltalk.bind(smalltalk.StringWriteStream,"do:",0
,function(aBlock){var self=this;
return self.invalidMessage_("do:");}
,"enumerating",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20streaming%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23do%3A"));
smalltalk.bind(smalltalk.StringWriteStream,"initialize",0
,function(){var self=this;
smalltalk.superImplementor(smalltalk.StringWriteStream,"initialize").apply(self, []);
self["@collection"]=smalltalk.Array.$$new();
self["@readLimit"]=self["@position"]=(0);
return self;}
,"initialize",unescape("initialize%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20initialize.%0D%0A%09collection%20%3A%3D%20Array%20new.%0D%0A%09readLimit%20%3A%3D%20position%20%3A%3D%200."));
smalltalk.bind(smalltalk.StringWriteStream,"setToEnd",0
,function(){var self=this;
return self;}
,"positioning",unescape("setToEnd%0D%0A%09%22%20Position%20the%20receiver%20at%20end.%20%22"));
smalltalk.bind(smalltalk.StringWriteStream,"skip:",0
,function(anInteger){var self=this;
return self.invalidMessage_("skip:");}
,"positioning",unescape("skip%3A%20anInteger%0D%0A%09%22%20Skip%20next%20anInteger%20readable%20elements%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23skip%3A"));
smalltalk.bind(smalltalk.StringWriteStream,"next",0
,function(){var self=this;
return self.invalidMessage_("next");}
,"reading",unescape("next%0D%0A%09%22%20Return%20the%20next%20readable%20element%20in%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23next"));
smalltalk.bind(smalltalk.StringWriteStream,"next:",0
,function(anInteger){var self=this;
return self.invalidMessage_("next");}
,"reading",unescape("next%3A%20anInteger%0D%0A%09%22%20Return%20the%20next%20anInteger%20elements%20read%20from%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23next"));
smalltalk.bind(smalltalk.StringWriteStream,"nextLine",0
,function(){var self=this;
return self.invalidMessage_("nextLine");}
,"reading",unescape("nextLine%0D%0A%09%22%20Return%20the%20characters%20read%20from%20the%20receiver%20up%20to%20the%20line%20delimiter.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23nextLine"));
smalltalk.bind(smalltalk.StringWriteStream,"peek",0
,function(){var self=this;
return self.invalidMessage_("peek");}
,"reading",unescape("peek%0D%0A%09%22%20Return%20the%20next%20readable%20element%20from%20the%20receiver%20without%20advancing%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23peek"));
smalltalk.bind(smalltalk.StringWriteStream,"upTo:",0
,function(anObject){var self=this;
return self.invalidMessage_("upTo:");}
,"reading",unescape("upTo%3A%20anObject%0D%0A%09%22%20Return%20the%20elements%20read%20from%20the%20receiver%20upto%20anObject%20%28or%20end%29.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23upTo%3A"));
smalltalk.bind(smalltalk.StringWriteStream,"upToEnd",0
,function(){var self=this;
return self.invalidMessage_("upToEnd");}
,"reading",unescape("upToEnd%0D%0A%09%22%20Return%20the%20contents%20of%20the%20receiver%20upto%20end.%20%22%0D%0A%0D%0A%09%5Eself%20invalidMessage%3A%20%23upToEnd"));
smalltalk.bind(smalltalk.StringWriteStream,"atEnd",0
,function(){var self=this;
return true;}
,"testing",unescape("atEnd%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20at%20end.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.StringWriteStream,"nextPut:",0
,function(anObject){var self=this;
var string=nil;
string=anObject.asString();
self["@collection"].add_(string);
self["@readLimit"]=self["@readLimit"]._plus(string.size());
return self;}
,"writing",unescape("nextPut%3A%20anObject%0D%0A%09%22%20Put%20anObject%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20string%20%7C%0D%0A%09string%20%3A%3D%20%20anObject%20asString.%0D%0A%09collection%20add%3A%20string.%0D%0A%09readLimit%20%3A%3D%20readLimit%20+%20string%20size"));
smalltalk.bind(smalltalk.StringWriteStream,"nextPutAll:",0
,function(aCollection){var self=this;
self.nextPut_(aCollection);
return self;}
,"writing",unescape("nextPutAll%3A%20aCollection%0D%0A%09%22%20Put%20aCollection%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20nextPut%3A%20aCollection"));
smalltalk.bind(smalltalk.StringWriteStream,"removeLastInput",0
,function(){var self=this;
self["@readLimit"]=self["@readLimit"]._minus(self["@collection"].removeLast().size());
return self;}
,"back",unescape("removeLastInput%0D%0A%09%22%20Remove%20last%20input%20recorded%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09readLimit%20%3A%3D%20readLimit%20-%20collection%20removeLast%20size"));
smalltalk.bind(smalltalk.Random,"next",0
,function(){var self=this;
return Math.random();
return self;}
,"accessing",unescape("next%0D%0A%09%22%20Return%20the%20next%20random%20number.%20%22%0D%0A%0D%0A%09%7B%27%27%20doc%27native%20accessing%27%7D"));
smalltalk.bind(smalltalk.Random,"next:",0
,function(anInteger){var self=this;
return (1).to_(anInteger).collect_((function(each){return self.next();}));}
,"accessing",unescape("next%3A%20anInteger%0D%0A%09%22%20Return%20the%20next%20anInteger%20random%20numbers.%20%22%0D%0A%0D%0A%09%5E%281%20to%3A%20anInteger%29%20collect%3A%20%5B%3Aeach%20%7C%20self%20next%20%5D"));
smalltalk.bind(smalltalk.Point.$klass,"x:y:",0
,function(x,y){var self=this;
return (function($rec){$rec.x_(x);return $rec.y_(y);})(self.basicNew());}
,"instantiation",unescape("x%3A%20x%20y%3A%20y%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicNew%20x%3A%20x%3B%20y%3A%20y"));
smalltalk.bind(smalltalk.Point.$klass,"zero",0
,function(){var self=this;
return self.basicNew().x_y_((0),(0));}
,"instantiation",unescape("zero%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicNew%20x%3A%200%20y%3A%200"));
smalltalk.bind(smalltalk.Point,"x",0
,function(){var self=this;
return self["@x"];}
,"accessing",unescape("x%0D%0A%09%22%20Return%20the%20x%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ex"));
smalltalk.bind(smalltalk.Point,"y",0
,function(){var self=this;
return self["@y"];}
,"accessing",unescape("y%0D%0A%09%22%20Return%20the%20y%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ey"));
smalltalk.bind(smalltalk.Point,"x:y:",0
,function(xValue,yValue){var self=this;
self["@x"]=xValue;
self["@y"]=yValue;
return self;}
,"accessing",unescape("x%3A%20xValue%20y%3A%20yValue%0D%0A%09%22%20Set%20the%20receiver%27s%20coordinates.%20%22%0D%0A%0D%0A%09x%20%3A%3D%20xValue.%0D%0A%09y%20%3A%3D%20yValue"));
smalltalk.bind(smalltalk.Point,"y:",0
,function(aNumber){var self=this;
self["@y"]=(((nil.isNil_(aNumber))).mustBeBoolean()==true ? (function(){return (0);})() : (function(){return aNumber;})());
return self;}
,"accessing",unescape("y%3A%20aNumber%0D%0A%09%22%20Set%20the%20y%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09y%20%3A%3D%20aNumber%20isNil%20ifTrue%3A%20%5B%200%20%5D%20ifFalse%3A%20%5B%20aNumber%20%5D"));
smalltalk.bind(smalltalk.Point,"x:",0
,function(aNumber){var self=this;
self["@x"]=(((nil.isNil_(aNumber))).mustBeBoolean()==true ? (function(){return (0);})() : (function(){return aNumber;})());
return self;}
,"accessing",unescape("x%3A%20aNumber%0D%0A%09%22%20Set%20the%20x%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09x%20%3A%3D%20aNumber%20isNil%20ifTrue%3A%20%5B%200%20%5D%20ifFalse%3A%20%5B%20aNumber%20%5D"));
smalltalk.bind(smalltalk.Point,"*","_star"
,function(aPoint){var self=this;
if (self.isNumber_(aPoint)){return self["@x"]._star(aPoint)._at(self["@y"]._star(aPoint));};;
return smalltalk.Point.x_y_(self["@x"]._star(aPoint.asPoint().x()),self["@y"]._star(aPoint.asPoint().y()));}
,"arithmetic",unescape("*%20aPoint%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20aPoint%29%20ifTrue%3A%20%5B%20%5Ex%20*%20aPoint%20@%20%28y%20*%20aPoint%29%20%5D.%0D%0A%09%5EPoint%09x%3A%20x%20*%20aPoint%20asPoint%20x%0D%0A%09%09y%3A%20y%20*%20aPoint%20asPoint%20y"));
smalltalk.bind(smalltalk.Point,"+","_plus"
,function(aPoint){var self=this;
if (self.isNumber_(aPoint)){return self["@x"]._plus(aPoint)._at(self["@y"]._plus(aPoint));};;
return smalltalk.Point.x_y_(self["@x"]._plus(aPoint.asPoint().x()),self["@y"]._plus(aPoint.asPoint().y()));}
,"arithmetic",unescape("+%20aPoint%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20aPoint%29%20ifTrue%3A%20%5B%20%5Ex%20+%20aPoint%20@%20%28y%20+%20aPoint%29%20%5D.%0D%0A%09%5EPoint%09x%3A%20x%20+%20aPoint%20asPoint%20x%0D%0A%09%09y%3A%20y%20+%20aPoint%20asPoint%20y"));
smalltalk.bind(smalltalk.Point,"-","_minus"
,function(aPoint){var self=this;
if (self.isNumber_(aPoint)){return self["@x"]._minus(aPoint)._at(self["@y"]._minus(aPoint));};;
return smalltalk.Point.x_y_(self["@x"]._minus(aPoint.asPoint().x()),self["@y"]._minus(aPoint.asPoint().y()));}
,"arithmetic",unescape("-%20aPoint%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20aPoint%29%20ifTrue%3A%20%5B%20%5Ex%20-%20aPoint%20@%20%28y%20-%20aPoint%29%20%5D.%0D%0A%09%5EPoint%09x%3A%20x%20-%20aPoint%20asPoint%20x%0D%0A%09%09y%3A%20y%20-%20aPoint%20asPoint%20y"));
smalltalk.bind(smalltalk.Point,"/","_slash"
,function(aPoint){var self=this;
if (self.isNumber_(aPoint)){return self["@x"]._slash(aPoint)._at(self["@y"]._slash(aPoint));};;
return smalltalk.Point.x_y_(self["@x"]._slash(aPoint.asPoint().x()),self["@y"]._slash(aPoint.asPoint().y()));}
,"arithmetic",unescape("/%20aPoint%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20aPoint%29%20ifTrue%3A%20%5B%20%5Ex%20/%20aPoint%20@%20%28y%20/%20aPoint%29%20%5D.%0D%0A%09%5EPoint%09x%3A%20x%20/%20aPoint%20asPoint%20x%0D%0A%09%09y%3A%20y%20/%20aPoint%20asPoint%20y"));
smalltalk.bind(smalltalk.Point,"//","_slash_slash"
,function(aPoint){var self=this;
if (self.isNumber_(aPoint)){return self["@x"]._slash_slash(aPoint)._at(self["@y"]._slash_slash(aPoint));};;
return smalltalk.Point.x_y_(self["@x"]._slash_slash(aPoint.asPoint().x()),self["@y"]._slash_slash(aPoint.asPoint().y()));}
,"arithmetic",unescape("//%20aPoint%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20aPoint%29%20ifTrue%3A%20%5B%20%5Ex%20//%20aPoint%20@%20%28y%20//%20aPoint%29%20%5D.%0D%0A%09%5EPoint%09x%3A%20x%20//%20aPoint%20asPoint%20x%0D%0A%09%09y%3A%20y%20//%20aPoint%20asPoint%20y"));
smalltalk.bind(smalltalk.Point,unescape("%5C%5C"),"_bk_bk"
,function(aPoint){var self=this;
if (self.isNumber_(aPoint)){return self["@x"]._bk_bk(aPoint)._at(self["@y"]._bk_bk(aPoint));};;
return smalltalk.Point.x_y_(self["@x"]._bk_bk(aPoint.asPoint().x()),self["@y"]._bk_bk(aPoint.asPoint().y()));}
,"arithmetic",unescape("%5C%5C%5C%5C%20aPoint%0D%0A%09%22%20Return%20the%20result%20of%20operation.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20aPoint%29%20ifTrue%3A%20%5B%20%5Ex%20%5C%5C%20aPoint%20@%20%28y%20%5C%5C%20aPoint%29%20%5D.%0D%0A%09%5EPoint%09x%3A%20x%20%5C%5C%20aPoint%20asPoint%20x%0D%0A%09%09y%3A%20y%20%5C%5C%20aPoint%20asPoint%20y"));
smalltalk.bind(smalltalk.Point,"<","_lt"
,function(aPoint){var self=this;
if (self.isNumber_(aPoint)){return self._lt(aPoint.asPoint());};;
return self["@x"]._lt(aPoint.x()).and_((function(){return self["@y"]._lt(aPoint.y());}));}
,"comparing",unescape("%3C%20aPoint%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20smaller%20than%20argument.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20aPoint%29%20ifTrue%3A%20%5B%20%5Eself%20%3C%20aPoint%20asPoint%20%5D.%0D%0A%09%5Ex%20%3C%20aPoint%20x%20and%3A%20%5B%20y%20%3C%20aPoint%20y%20%5D"));
smalltalk.bind(smalltalk.Point,"<=","_lt_eq"
,function(aPoint){var self=this;
if (self.isNumber_(aPoint)){return self._lt_eq(aPoint.asPoint());};;
return self["@x"]._lt_eq(aPoint.x()).and_((function(){return self["@y"]._lt_eq(aPoint.y());}));}
,"comparing",unescape("%3C%3D%20aPoint%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20smaller%20or%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20aPoint%29%20ifTrue%3A%20%5B%20%5Eself%20%3C%3D%20aPoint%20asPoint%20%5D.%0D%0A%09%5Ex%20%3C%3D%20aPoint%20x%20and%3A%20%5B%20y%20%3C%3D%20aPoint%20y%20%5D"));
smalltalk.bind(smalltalk.Point,">","_gt"
,function(aPoint){var self=this;
if (self.isNumber_(aPoint)){return self._gt(aPoint.asPoint());};;
return self["@x"]._gt(aPoint.x()).and_((function(){return self["@y"]._gt(aPoint.y());}));}
,"comparing",unescape("%3E%20aPoint%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20greater%20than%20argument.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20aPoint%29%20ifTrue%3A%20%5B%20%5Eself%20%3E%20aPoint%20asPoint%20%5D.%0D%0A%09%5Ex%20%3E%20aPoint%20x%20and%3A%20%5B%20y%20%3E%20aPoint%20y%20%5D"));
smalltalk.bind(smalltalk.Point,">=","_gt_eq"
,function(aPoint){var self=this;
if (self.isNumber_(aPoint)){return self._gt_eq(aPoint.asPoint());};;
return self["@x"]._gt_eq(aPoint.x()).and_((function(){return self["@y"]._gt_eq(aPoint.y());}));}
,"comparing",unescape("%3E%3D%20aPoint%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20greater%20or%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%28self%20isNumber%3A%20aPoint%29%20ifTrue%3A%20%5B%20%5Eself%20%3E%3D%20aPoint%20asPoint%20%5D.%0D%0A%09%5Ex%20%3E%3D%20aPoint%20x%20and%3A%20%5B%20y%20%3E%3D%20aPoint%20y%20%5D"));
smalltalk.bind(smalltalk.Point,"rounded",0
,function(){var self=this;
return self["@x"].rounded()._at(self["@y"].rounded());}
,"converting",unescape("rounded%0D%0A%09%22%20Return%20the%20receiver%20rounded.%20%22%0D%0A%0D%0A%09%5Ex%20rounded%20@%20y%20rounded"));
smalltalk.bind(smalltalk.Point,"truncated",0
,function(){var self=this;
return self["@x"].truncated()._at(self["@y"].truncated());}
,"converting",unescape("truncated%0D%0A%09%22%20Return%20the%20receiver%20truncated.%20%22%0D%0A%0D%0A%09%5Ex%20truncated%20@%20y%20truncated"));
smalltalk.bind(smalltalk.Point,"asPoint",0
,function(){var self=this;
return self;}
,"converting",unescape("asPoint%0D%0A%09%22%20Return%20the%20receiver%20as%20a%20Point.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.Point,"printString",0
,function(){var self=this;
return self["@x"].toString()._comma(unescape("%20@%20"))._comma(self["@y"].toString());}
,"printing",unescape("printString%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ex%20toString%20%2C%27%20@%20%27%20%2Cy%20toString"));
smalltalk.bind(smalltalk.Point,"toString",0
,function(){var self=this;
return self["@x"].toString()._comma(unescape("@"))._comma(self["@y"].toString());}
,"printing",unescape("toString%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ex%20toString%20%2C%24@%20%2Cy%20toString"));
smalltalk.bind(smalltalk.Point,"asLiteral",0
,function(){var self=this;
var literal=nil;
literal=(function(v){var string=nil;
string=((self.isObject_(v)).mustBeBoolean()==true ? (function(){return v.asLiteral();})() : (function(){return v.toString();})());return ((string.isEmpty().or_((function(){return string.first().isDigit().or_((function(){return unescape("%23%27-+").includes_(string.first());}));}))).mustBeBoolean()==true ? (function(){return string;})() : (function(){return unescape("%28")._comma(string)._comma(unescape("%29"));})());});
return literal.value_(self["@x"])._comma(unescape("@"))._comma(literal.value_(self["@y"]));}
,"converting",unescape("asLiteral%0D%0A%09%22%20Return%20the%20receiver%20in%20literal%20notation.%20%22%0D%0A%0D%0A%09%7C%20literal%20%7C%0D%0A%09literal%20%3A%3D%20%5B%3Av%7C%20%7C%20string%20%7C%0D%0A%09%09string%20%3A%3D%20%28self%20isObject%3A%20v%29%20ifTrue%3A%20%5Bv%20asLiteral%5D%20ifFalse%3A%20%5Bv%20toString%5D.%0D%0A%09%09%28string%20isEmpty%20or%3A%20%5Bstring%20first%20isDigit%20or%3A%20%5B%27%23%27%27-+%27%20includes%3A%20string%20first%5D%5D%29%0D%0A%09%09ifTrue%3A%20%5Bstring%5D%20ifFalse%3A%20%5B%20%24%28%20%2Cstring%20%2C%24%29%20%5D%0D%0A%09%5D.%0D%0A%09%5E%28literal%20value%3A%20x%29%20%2C%24@%20%2C%28literal%20value%3A%20y%29"));
smalltalk.bind(smalltalk.Point,"shallowCopy",0
,function(){var self=this;
return self.$klass.basicNew().x_y_(self.x(),self.y());}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20class%20basicNew%0D%0A%09%09x%3A%20self%20x%20y%3A%20self%20y"));
smalltalk.bind(smalltalk.Point,"=","_eq"
,function(aPoint){var self=this;
if ((nil.isNil_(aPoint))){return false;};;
if (!(aPoint.isMemberOf_(self.$klass))){return smalltalk.responds_to_(aPoint,("asPoint")).and_((function(){return aPoint.asPoint()._eq(self);}));};;
return self["@x"]._eq(aPoint.x()).and_((function(){return self["@y"]._eq(aPoint.y());}));}
,"testing",unescape("%3D%20aPoint%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09aPoint%20isNil%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%28aPoint%20isMemberOf%3A%20self%20class%29%20ifFalse%3A%20%5B%0D%0A%09%09%5E%28aPoint%20respondsTo%3A%20%23asPoint%29%20and%3A%20%5B%0D%0A%09%09%09aPoint%20asPoint%20%3D%20self%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Ex%20%3D%20aPoint%20x%20and%3A%20%5B%20y%20%3D%20aPoint%20y%20%5D"));
smalltalk.bind(smalltalk.Point,"min:",0
,function(aPoint){var self=this;
return self["@x"].min_(aPoint.x())._at(self["@y"].min_(aPoint.y()));}
,"magnitude",unescape("min%3A%20aPoint%0D%0A%09%22%20Return%20the%20minimum%20of%20the%20receiver%20and%20aPoint.%20%22%0D%0A%0D%0A%09%5E%28x%20min%3A%20aPoint%20x%29%20@%20%28y%20min%3A%20aPoint%20y%29"));
smalltalk.bind(smalltalk.Point,"max:",0
,function(aPoint){var self=this;
return self["@x"].max_(aPoint.x())._at(self["@y"].max_(aPoint.y()));}
,"magnitude",unescape("max%3A%20aPoint%0D%0A%09%22%20Return%20the%20maximum%20of%20the%20receiver%20and%20aPoint.%20%22%0D%0A%0D%0A%09%5E%28x%20max%3A%20aPoint%20x%29%20@%20%28y%20max%3A%20aPoint%20y%29"));
smalltalk.bind(smalltalk.Point,"extent:",0
,function(extent){var self=this;
return smalltalk.Rectangle.leftTop_rightBottom_(self,self._plus(extent));}
,"converting",unescape("extent%3A%20extent%0D%0A%09%22%20Return%20a%20Rectangle%20from%20the%20receiver%20with%20specified%20extent.%20%22%0D%0A%0D%0A%09%5ERectangle%20leftTop%3A%20self%20rightBottom%3A%20self%20+%20extent"));
smalltalk.bind(smalltalk.Point,"rightBottom:",0
,function(rightBottom){var self=this;
return smalltalk.Rectangle.leftTop_rightBottom_(self,rightBottom);}
,"converting",unescape("rightBottom%3A%20rightBottom%0D%0A%09%22%20Return%20a%20Rectangle%20from%20the%20receiver%20to%20rightBottom.%20%22%0D%0A%0D%0A%09%5ERectangle%20leftTop%3A%20self%20rightBottom%3A%20rightBottom"));
smalltalk.bind(smalltalk.Point,"to:",0
,function(rightBottom){var self=this;
return smalltalk.Rectangle.leftTop_rightBottom_(self,rightBottom);}
,"converting",unescape("to%3A%20rightBottom%0D%0A%09%22%20Return%20a%20Rectangle%20from%20the%20receiver%20to%20rightBottom.%20%22%0D%0A%0D%0A%09%5ERectangle%20leftTop%3A%20self%20rightBottom%3A%20rightBottom"));
smalltalk.bind(smalltalk.Point,"__tostring",0
,function(){var self=this;
return self.toString();}
,"Lua-printing",unescape("__tostring%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.Rectangle.$klass,"leftTop:extent:",0
,function(leftTop,extent){var self=this;
return self.$$new().leftTop_rightBottom_(leftTop,leftTop._plus(extent));}
,"instantiation",unescape("leftTop%3A%20leftTop%20extent%3A%20extent%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%20leftTop%3A%20leftTop%20rightBottom%3A%20leftTop%20+%20extent"));
smalltalk.bind(smalltalk.Rectangle.$klass,"leftTop:rightBottom:",0
,function(leftTop,rightBottom){var self=this;
return self.$$new().leftTop_rightBottom_(leftTop,rightBottom);}
,"instantiation",unescape("leftTop%3A%20leftTop%20rightBottom%3A%20rightBottom%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%20leftTop%3A%20leftTop%20rightBottom%3A%20rightBottom"));
smalltalk.bind(smalltalk.Rectangle,"=","_eq"
,function(aRectangle){var self=this;
return aRectangle.isRectangle().and_((function(){return self["@leftTop"]._eq(aRectangle.leftTop()).and_((function(){return self["@rightBottom"]._eq(aRectangle.rightBottom());}));}));}
,"testing",unescape("%3D%20aRectangle%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20equal%20to%20argument.%20%22%0D%0A%0D%0A%09%5EaRectangle%20isRectangle%0D%0A%09and%3A%20%5B%20leftTop%20%3D%20aRectangle%20leftTop%0D%0A%09and%3A%20%5B%20rightBottom%20%3D%20aRectangle%20rightBottom%20%5D%5D"));
smalltalk.bind(smalltalk.Rectangle,"area",0
,function(){var self=this;
return self.height()._star(self.width());}
,"accessing",unescape("area%0D%0A%09%22%20Return%20the%20area%20of%20the%20receiver.%22%0D%0A%0D%0A%09%5Eself%20height%20*%20self%20width"));
smalltalk.bind(smalltalk.Rectangle,"areasOutside:",0
,function(aRectangle){var self=this;
if (!(self.intersects_(aRectangle))){return smalltalk.Array.with_(self);};;
if (aRectangle.containsPoint_(self.topLeft()).and_((function(){return aRectangle.containsPoint_(self.bottomRight());}))){return [];};;
return smalltalk.Array.with_with_with_with_(((aRectangle.top()._gt(self.top())).mustBeBoolean()==true ? (function(){return self.topLeft().rightBottom_(self.right()._at(aRectangle.top()));})() : nil),((aRectangle.bottom()._lt(self.bottom())).mustBeBoolean()==true ? (function(){return self.left()._at(aRectangle.bottom()).rightBottom_(self.bottomRight());})() : nil),((aRectangle.left()._gt(self.left())).mustBeBoolean()==true ? (function(){return self.left()._at(self.top().max_(aRectangle.top())).rightBottom_(aRectangle.left()._at(self.bottom().min_(aRectangle.bottom())));})() : nil),((aRectangle.right()._lt(self.right())).mustBeBoolean()==true ? (function(){return aRectangle.right()._at(self.top().max_(aRectangle.top())).rightBottom_(self.right()._at(self.bottom().min_(aRectangle.bottom())));})() : nil)).select_((function(each){return (nil.isNil_(each)===false);}));}
,"accessing",unescape("areasOutside%3A%20aRectangle%0D%0A%09%22Return%20a%20collection%20of%20rectangles%20where%20the%20receiver%20do%20not%20intersect%20aRectangle.%20%22%0D%0A%0D%0A%09%28self%20intersects%3A%20aRectangle%29%20ifFalse%3A%20%5B%20%5EArray%20with%3A%20self%20%5D.%0D%0A%09%28%28aRectangle%20containsPoint%3A%20self%20topLeft%29%0D%0A%09and%3A%20%5BaRectangle%20containsPoint%3A%20self%20bottomRight%5D%29%0D%0A%20%20%20%20%20%20%20%20ifTrue%3A%20%5B%20%5E%23%28%29%20%5D.%0D%0A%09%5E%28Array%0D%0A%09with%3A%20%28aRectangle%20top%20%3E%20self%20top%20ifTrue%3A%20%5B%20self%20topLeft%20rightBottom%3A%20self%20right%20@%20aRectangle%20top%5D%29%0D%0A%09with%3A%20%28aRectangle%20bottom%20%3C%20self%20bottom%20ifTrue%3A%20%5B%20self%20left%20@%20aRectangle%20bottom%20rightBottom%3A%20self%20bottomRight%5D%29%0D%0A%09with%3A%20%28aRectangle%20left%20%3E%20self%20left%20ifTrue%3A%20%5B%20self%20left%20@%20%28self%20top%20max%3A%20aRectangle%20top%29%20rightBottom%3A%20aRectangle%20left%20@%20%28self%20bottom%20min%3A%20aRectangle%20bottom%29%20%5D%29%0D%0A%09with%3A%20%28aRectangle%20right%20%3C%20self%20right%20ifTrue%3A%20%5B%20aRectangle%20right%20@%20%28self%20top%20max%3A%20aRectangle%20top%29%20rightBottom%3A%20self%20right%20@%20%28self%20bottom%20min%3A%20aRectangle%20bottom%29%20%5D%29%0D%0A%09%29%20select%3A%20%5B%3Aeach%7C%20each%20notNil%20%5D"));
smalltalk.bind(smalltalk.Rectangle,"bottom",0
,function(){var self=this;
return self["@rightBottom"].y();}
,"accessing",unescape("bottom%0D%0A%09%22%20Return%20the%20bottom%20of%20the%20receiver.%22%0D%0A%0D%0A%09%5ErightBottom%20y"));
smalltalk.bind(smalltalk.Rectangle,"bottomCenter",0
,function(){var self=this;
return self.center().x()._at(self.bottom());}
,"accessing",unescape("bottomCenter%0D%0A%09%22%20Return%20the%20center%20of%20the%20bottom%20edge%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20center%20x%20@%20self%20bottom"));
smalltalk.bind(smalltalk.Rectangle,"bottomLeft",0
,function(){var self=this;
return self.left()._at(self.bottom());}
,"accessing",unescape("bottomLeft%0D%0A%09%22%20Return%20the%20bottom-left%20corner%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20left%20@%20self%20bottom"));
smalltalk.bind(smalltalk.Rectangle,"bottomRight",0
,function(){var self=this;
return self["@rightBottom"];}
,"accessing",unescape("bottomRight%0D%0A%09%22%20Return%20the%20bottom-right%20corner%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5ErightBottom"));
smalltalk.bind(smalltalk.Rectangle,"center",0
,function(){var self=this;
return self["@leftTop"]._plus(self["@rightBottom"])._slash_slash((2));}
,"accessing",unescape("center%0D%0A%09%22%20Return%20the%20center%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EleftTop%20+%20rightBottom%20//%202"));
smalltalk.bind(smalltalk.Rectangle,"centerTop",0
,function(){var self=this;
return self.right()._plus(self.left())._slash_slash((2))._at(self.top());}
,"accessing",unescape("centerTop%0D%0A%09%22%20Return%20the%20center%20top%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20right%20+%20self%20left%20//%202%20@%20self%20top"));
smalltalk.bind(smalltalk.Rectangle,"contains:",0
,function(aRectangle){var self=this;
return self.left()._lt_eq(aRectangle.left()).and_((function(){return aRectangle.right()._lt_eq(self.right()).and_((function(){return self.bottom()._gt_eq(aRectangle.bottom()).and_((function(){return aRectangle.top()._gt_eq(self.top());}));}));}));}
,"testing",unescape("contains%3A%20aRectangle%0D%0A%09%22%20Return%20true%20if%20aRectangle%20lies%20completely%20within%20or%20on%20the%20edge%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20left%20%3C%3D%20aRectangle%20left%0D%0A%09and%3A%20%5B%20aRectangle%20right%20%3C%3D%20self%20right%0D%0A%09and%3A%20%5B%20self%20bottom%20%3E%3D%20aRectangle%20bottom%0D%0A%09and%3A%20%5B%20aRectangle%20top%20%3E%3D%20self%20top%20%5D%5D%5D"));
smalltalk.bind(smalltalk.Rectangle,"containsPoint:",0
,function(aPoint){var self=this;
return self.left()._lt_eq(aPoint.x()).and_((function(){return aPoint.x()._lt(self.right()).and_((function(){return self.bottom()._gt(aPoint.y()).and_((function(){return aPoint.y()._gt_eq(self.top());}));}));}));}
,"testing",unescape("containsPoint%3A%20aPoint%0D%0A%09%22%20Return%20true%20if%20aPoint%20is%20contained%20within%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28self%20left%20%3C%3D%20aPoint%20x%29%0D%0A%09and%3A%20%5B%20aPoint%20x%20%3C%20self%20right%0D%0A%09and%3A%20%5B%20self%20bottom%20%3E%20aPoint%20y%0D%0A%09and%3A%20%5B%20aPoint%20y%20%3E%3D%20self%20top%20%5D%5D%5D"));
smalltalk.bind(smalltalk.Rectangle,"expandBy:",0
,function(delta){var self=this;
if (delta.isRectangle()){return self["@leftTop"]._minus(delta.leftTop()).rightBottom_(self["@rightBottom"]._plus(delta.rightBottom()));};;
return self["@leftTop"]._minus(delta).rightBottom_(self["@rightBottom"]._plus(delta));}
,"accessing",unescape("expandBy%3A%20delta%0D%0A%09%22%20Return%20a%20Rectangle%20obtained%20from%20the%20receiver%20expanded%20by%20delta.%20%22%0D%0A%0D%0A%09delta%20isRectangle%20ifTrue%3A%20%5B%0D%0A%20%20%20%20%20%20%20%20%20%20%20%20%5EleftTop%20-%20delta%20leftTop%20rightBottom%3A%20rightBottom%20+%20delta%20rightBottom%0D%0A%09%5D.%0D%0A%09%5EleftTop%20-%20delta%20rightBottom%3A%20rightBottom%20+%20delta"));
smalltalk.bind(smalltalk.Rectangle,"extent",0
,function(){var self=this;
return self.width()._at(self.height());}
,"accessing",unescape("extent%0D%0A%09%22%20Return%20receiver%27s%20width%20and%20height.%22%0D%0A%0D%0A%09%5Eself%20width%20@%20self%20height"));
smalltalk.bind(smalltalk.Rectangle,"extent:",0
,function(aPoint){var self=this;
self.width_(aPoint.x());
self.height_(aPoint.y());
return self;}
,"accessing",unescape("extent%3A%20aPoint%0D%0A%09%22%20Set%20the%20extent%20of%20receiver%20to%20aPoint.%20%22%0D%0A%0D%0A%09self%20width%3A%20aPoint%20x.%0D%0A%09self%20height%3A%20aPoint%20y"));
smalltalk.bind(smalltalk.Rectangle,"height",0
,function(){var self=this;
return self["@leftTop"].y()._minus(self["@rightBottom"].y()).abs();}
,"accessing",unescape("height%0D%0A%09%22%20Return%20the%20height%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28leftTop%20y%20-%20rightBottom%20y%29%20abs"));
smalltalk.bind(smalltalk.Rectangle,"height:",0
,function(aNumber){var self=this;
self["@rightBottom"].y_(self["@leftTop"].y()._plus(aNumber));
return self;}
,"accessing",unescape("height%3A%20aNumber%0D%0A%09%22%20Change%20the%20height%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09rightBottom%20y%3A%20leftTop%20y%20+%20aNumber"));
smalltalk.bind(smalltalk.Rectangle,"insetBy:",0
,function(delta){var self=this;
if (delta.isRectangle()){return self["@leftTop"]._plus(delta.leftTop()).rightBottom_(self["@rightBottom"]._minus(delta.rightBottom()));};;
return self["@leftTop"]._plus(delta).rightBottom_(self["@rightBottom"]._minus(delta));}
,"accessing",unescape("insetBy%3A%20delta%0D%0A%09%22%20Return%20the%20receiver%20inset%20by%20delta.%20%22%0D%0A%0D%0A%09delta%20isRectangle%20ifTrue%3A%20%5B%0D%0A%20%20%20%20%20%20%20%20%20%20%20%20%5EleftTop%20+%20delta%20leftTop%20rightBottom%3A%20rightBottom%20-%20delta%20rightBottom%0D%0A%09%5D.%0D%0A%09%5EleftTop%20+%20delta%20rightBottom%3A%20rightBottom%20-%20delta"));
smalltalk.bind(smalltalk.Rectangle,"intersect:",0
,function(aRectangle){var self=this;
if ((nil.isNil_(aRectangle))){return self;};;
return self["@leftTop"].max_(aRectangle.leftTop()).rightBottom_(self["@rightBottom"].min_(aRectangle.rightBottom()));}
,"accessing",unescape("intersect%3A%20aRectangle%0D%0A%09%22%20Return%20the%20rectangle%20in%20which%20the%20receiver%20and%20aRectangle%20overlap.%20%22%0D%0A%0D%0A%09aRectangle%20isNil%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09%5E%28leftTop%20max%3A%20aRectangle%20leftTop%29%0D%0A%20%20%20%20%20%20%20%20rightBottom%3A%20%28rightBottom%20min%3A%20aRectangle%20rightBottom%29"));
smalltalk.bind(smalltalk.Rectangle,"intersects:",0
,function(aRectangle){var self=this;
return self["@leftTop"]._lt(aRectangle.rightBottom()).and_((function(){return aRectangle.leftTop()._lt(self["@rightBottom"]);}));}
,"accessing",unescape("intersects%3A%20aRectangle%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20and%20aRectangle%20have%20any%20area%20in%20common.%20%22%0D%0A%0D%0A%09%5EleftTop%20%3C%20aRectangle%20rightBottom%0D%0A%09and%3A%20%5B%20aRectangle%20leftTop%20%3C%20rightBottom%20%5D"));
smalltalk.bind(smalltalk.Rectangle,"isRectangle",0
,function(){var self=this;
return true;}
,"testing",unescape("isRectangle%0D%0A%09%22%20Return%20true%20if%20receiver%20is%20a%20Rectangle.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.Rectangle,"left",0
,function(){var self=this;
return self["@leftTop"].x();}
,"accessing",unescape("left%0D%0A%09%22%20Return%20the%20left%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EleftTop%20x"));
smalltalk.bind(smalltalk.Rectangle,"leftBottom",0
,function(){var self=this;
return self.left()._at(self.bottom());}
,"accessing",unescape("leftBottom%0D%0A%09%22%20Return%20the%20left%20bottom%20corner%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20left%20@%20self%20bottom"));
smalltalk.bind(smalltalk.Rectangle,"leftCenter",0
,function(){var self=this;
return self.left()._at(self.center().y());}
,"accessing",unescape("leftCenter%0D%0A%09%22%20Return%20the%20center%20of%20the%20left%20edge%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20left%20@%20self%20center%20y"));
smalltalk.bind(smalltalk.Rectangle,"leftTop",0
,function(){var self=this;
return self["@leftTop"];}
,"accessing",unescape("leftTop%0D%0A%09%22%20Return%20the%20left%20top%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EleftTop"));
smalltalk.bind(smalltalk.Rectangle,"leftTop:",0
,function(aPoint){var self=this;
self["@leftTop"]=aPoint;
return self;}
,"accessing",unescape("leftTop%3A%20aPoint%0D%0A%09%22%20Set%20the%20leftTop%20of%20the%20receiver.%22%0D%0A%0D%0A%09leftTop%20%3A%3D%20aPoint"));
smalltalk.bind(smalltalk.Rectangle,"leftTop:rightBottom:",0
,function(leftTopPoint,rightBottomPoint){var self=this;
self["@leftTop"]=leftTopPoint;
self["@rightBottom"]=rightBottomPoint;
return self;}
,"accessing",unescape("leftTop%3A%20leftTopPoint%20rightBottom%3A%20rightBottomPoint%0D%0A%09%22Set%20the%20receiver%27s%20coordinates.%20%22%0D%0A%0D%0A%09leftTop%20%3A%3D%20leftTopPoint.%0D%0A%09rightBottom%20%3A%3D%20rightBottomPoint."));
smalltalk.bind(smalltalk.Rectangle,"merge:",0
,function(aRectangle){var self=this;
if ((nil.isNil_(aRectangle))){return self;};;
return self["@leftTop"].min_(aRectangle.leftTop()).rightBottom_(self["@rightBottom"].max_(aRectangle.rightBottom()));}
,"accessing",unescape("merge%3A%20aRectangle%0D%0A%09%22%20Return%20a%20Rectangle%20representing%20the%20area%20in%20which%20the%20receiver%20and%20aRectangle%20merged%20together.%20%22%0D%0A%0D%0A%09aRectangle%20isNil%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09%5E%28leftTop%20min%3A%20aRectangle%20leftTop%29%20rightBottom%3A%20%28rightBottom%20max%3A%20aRectangle%20rightBottom%29"));
smalltalk.bind(smalltalk.Rectangle,"moveBy:",0
,function(aPoint){var self=this;
self["@leftTop"]=self["@leftTop"]._plus(aPoint);
self["@rightBottom"]=self["@rightBottom"]._plus(aPoint);
return self;}
,"accessing",unescape("moveBy%3A%20aPoint%0D%0A%09%22%20Move%20the%20receiver%20by%20aPoint.%20%22%0D%0A%0D%0A%09leftTop%20%3A%3D%20leftTop%20+%20aPoint.%0D%0A%09rightBottom%20%3A%3D%20rightBottom%20+%20aPoint"));
smalltalk.bind(smalltalk.Rectangle,"moveTo:",0
,function(aPoint){var self=this;
self.moveBy_(aPoint._minus(self["@leftTop"]));
return self;}
,"accessing",unescape("moveTo%3A%20aPoint%0D%0A%09%22%20Move%20the%20receiver%20to%20aPoint.%20%22%0D%0A%0D%0A%09self%20moveBy%3A%20aPoint%20-%20leftTop"));
smalltalk.bind(smalltalk.Rectangle,"moveToLeftTop:",0
,function(aPoint){var self=this;
self.moveBy_(aPoint._minus(self.leftTop()));
return self;}
,"accessing",unescape("moveToLeftTop%3A%20aPoint%0D%0A%09%22%20Move%20the%20receiver%20to%20leftTop.%20%22%0D%0A%0D%0A%09self%20moveBy%3A%20aPoint%20-%20self%20leftTop"));
smalltalk.bind(smalltalk.Rectangle,"nonIntersections:",0
,function(aRectangle){var self=this;
var areas=nil;
if (!(self.intersects_(aRectangle))){return smalltalk.Array.with_(self);};;
if (aRectangle.containsPoint_(self["@leftTop"]).and_((function(){return aRectangle.containsPoint_(self["@rightBottom"]);}))){return [];};;
return smalltalk.Array.with_with_with_with_(((aRectangle.top()._gt(self.top())).mustBeBoolean()==true ? (function(){return self["@leftTop"].rightBottom_(self["@rightBottom"].x()._at(aRectangle.top()));})() : nil),((aRectangle.bottom()._lt(self.bottom())).mustBeBoolean()==true ? (function(){return self["@leftTop"].x()._at(aRectangle.bottom()).rightBottom_(self["@rightBottom"]);})() : nil),((aRectangle.left()._gt(self.left())).mustBeBoolean()==true ? (function(){return self["@leftTop"].x()._at(self["@leftTop"].y().max_(aRectangle.top())).rightBottom_(aRectangle.left()._at(self["@rightBottom"].y().min_(aRectangle.bottom())));})() : nil),((aRectangle.right()._lt(self.right())).mustBeBoolean()==true ? (function(){return aRectangle.right()._at(self["@leftTop"].y().max_(aRectangle.top())).rightBottom_(self["@rightBottom"].x()._at(self["@rightBottom"].y().min_(aRectangle.bottom())));})() : nil)).select_((function(each){return (nil.isNil_(each)===false);}));}
,"accessing",unescape("nonIntersections%3A%20aRectangle%0D%0A%09%22%20Return%20the%20rectangles%20describing%20areas%20of%20the%20receiver%20outside%20aRectangle.%20%22%0D%0A%0D%0A%09%7C%20areas%20%7C%0D%0A%09%28self%20intersects%3A%20aRectangle%29%20ifFalse%3A%20%5B%20%5EArray%20with%3A%20self%20%5D.%0D%0A%09%28%28aRectangle%20containsPoint%3A%20leftTop%29%0D%0A%09%09and%3A%20%5B%20aRectangle%20containsPoint%3A%20rightBottom%20%5D%29%0D%0A%09%09ifTrue%3A%20%5B%20%5E%23%28%29%20%5D.%0D%0A%09%5E%28Array%0D%0A%09with%3A%20%28aRectangle%20top%20%3E%20self%20top%20ifTrue%3A%20%5B%20leftTop%20rightBottom%3A%20rightBottom%20x%20@%20aRectangle%20top%20%5D%29%0D%0A%09with%3A%20%28aRectangle%20bottom%20%3C%20self%20bottom%20ifTrue%3A%20%5B%20leftTop%20x%20@%20aRectangle%20bottom%20rightBottom%3A%20rightBottom%20%5D%29%0D%0A%09with%3A%20%28aRectangle%20left%20%3E%20self%20left%20ifTrue%3A%20%5B%20leftTop%20x%20@%20%28leftTop%20y%20max%3A%20aRectangle%20top%29%0D%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20rightBottom%3A%20aRectangle%20left%20@%20%28rightBottom%20y%20min%3A%20aRectangle%20bottom%29%20%5D%29%0D%0A%09with%3A%20%28aRectangle%20right%20%3C%20self%20right%20ifTrue%3A%20%5B%20aRectangle%20right%20@%20%28leftTop%20y%20max%3A%20aRectangle%20top%29%0D%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20rightBottom%3A%20rightBottom%20x%20@%20%28rightBottom%20y%20min%3A%20aRectangle%20bottom%29%20%5D%29%0D%0A%09%29%20select%3A%20%5B%3Aeach%7C%20each%20notNil%20%5D"));
smalltalk.bind(smalltalk.Rectangle,"toString",0
,function(){var self=this;
return self["@leftTop"].toString()._comma(" to: ")._comma(self["@rightBottom"].toString());}
,"printing",unescape("toString%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EleftTop%20toString%20%2C%27%20to%3A%20%27%20%2CrightBottom%20toString%20"));
smalltalk.bind(smalltalk.Rectangle,"printString",0
,function(){var self=this;

return self.toString();}
,"printing",unescape("printString%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.Rectangle,"relativeTo:",0
,function(aRectangle){var self=this;
var extent=nil;
extent=aRectangle.extent();
if (aRectangle.extent().x()._eq((0)).or_((function(){return aRectangle.extent().y()._eq((0));}))){return self;};;
return self["@leftTop"]._minus(aRectangle.leftTop())._slash(extent).extent_(self.extent()._slash(extent));}
,"accessing",unescape("relativeTo%3A%20aRectangle%0D%0A%09%22%20Return%20a%20Rectangle%20proportionally%20relative%20to%20aRectangle.%20%22%0D%0A%0D%0A%09%7C%20extent%20%7C%0D%0A%09extent%20%3A%3D%20aRectangle%20extent.%0D%0A%09%28aRectangle%20extent%20x%20%3D%200%20or%3A%20%5B%20aRectangle%20extent%20y%20%3D%200%20%5D%29%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09%5EleftTop%20-%20aRectangle%20leftTop%20/%20extent%20extent%3A%20self%20extent%20/%20extent"));
smalltalk.bind(smalltalk.Rectangle,"right",0
,function(){var self=this;
return self["@rightBottom"].x();}
,"accessing",unescape("right%0D%0A%09%22%20Return%20the%20receiver%27s%20right%20side.%20%22%0D%0A%0D%0A%09%5ErightBottom%20x"));
smalltalk.bind(smalltalk.Rectangle,"rightBottom",0
,function(){var self=this;
return self["@rightBottom"];}
,"accessing",unescape("rightBottom%0D%0A%09%22%20Return%20the%20right%20bottom%20corner.%20%22%0D%0A%0D%0A%09%5ErightBottom"));
smalltalk.bind(smalltalk.Rectangle,"rightBottom:",0
,function(aPoint){var self=this;
self["@rightBottom"]=aPoint;
return self;}
,"accessing",unescape("rightBottom%3A%20aPoint%0D%0A%09%22%20Set%20the%20right%20bottom%20corner%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09rightBottom%20%3A%3D%20aPoint"));
smalltalk.bind(smalltalk.Rectangle,"rightCenter",0
,function(){var self=this;
return self.right()._at(self.center().y());}
,"accessing",unescape("rightCenter%0D%0A%09%22%20Return%20the%20center%20of%20the%20right%20edge%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20right%20@%20self%20center%20y"));
smalltalk.bind(smalltalk.Rectangle,"rightTop",0
,function(){var self=this;
return self.right()._at(self.top());}
,"accessing",unescape("rightTop%0D%0A%20%20%20%20%20%20%20%20%22%20Return%20the%20right%20top%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20right%20@%20self%20top"));
smalltalk.bind(smalltalk.Rectangle,"rounded",0
,function(){var self=this;
self["@leftTop"]=self["@leftTop"].rounded();
self["@rightBottom"]=self["@rightBottom"].rounded();
return self;}
,"accessing",unescape("rounded%0D%0A%09%22%20Set%20the%20receiver%20with%20coordinates%20rounded%20to%20integers.%20%22%0D%0A%0D%0A%09leftTop%20%3A%3D%20leftTop%20rounded.%0D%0A%09rightBottom%20%3A%3D%20rightBottom%20rounded."));
smalltalk.bind(smalltalk.Rectangle,"scaleBy:",0
,function(delta){var self=this;
return self["@leftTop"]._star(delta).rightBottom_(self["@rightBottom"]._star(delta));}
,"accessing",unescape("scaleBy%3A%20delta%0D%0A%09%22%20Return%20the%20receiver%20multiplied%20by%20delta.%20%22%0D%0A%0D%0A%09%5EleftTop%20*%20delta%20rightBottom%3A%20rightBottom%20*%20delta"));
smalltalk.bind(smalltalk.Rectangle,"scaleTo:",0
,function(aRectangle){var self=this;
return aRectangle.scaleBy_(self.extent()).truncated().moveBy_(self.leftTop());}
,"accessing",unescape("scaleTo%3A%20aRectangle%0D%0A%09%22%20Return%20a%20Rectangle%20whose%20size%20is%20proportional%20to%20the%20receiver%20with%20ratios%20specified%20by%20aRectangle.%20%22%0D%0A%0D%0A%09%5E%28aRectangle%20scaleBy%3A%20self%20extent%29%20truncated%20moveBy%3A%20self%20leftTop"));
smalltalk.bind(smalltalk.Rectangle,"top",0
,function(){var self=this;
return self["@leftTop"].y();}
,"accessing",unescape("top%0D%0A%09%22%20Return%20the%20top%20side%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EleftTop%20y"));
smalltalk.bind(smalltalk.Rectangle,"topCenter",0
,function(){var self=this;
return self.center().x()._at(self.top());}
,"accessing",unescape("topCenter%0D%0A%09%22%20Return%20the%20center%20of%20the%20top%20edge%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20center%20x%20@%20self%20top"));
smalltalk.bind(smalltalk.Rectangle,"topLeft",0
,function(){var self=this;
return self["@leftTop"];}
,"accessing",unescape("topLeft%0D%0A%09%22%20Return%20the%20top-left%20corner%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EleftTop"));
smalltalk.bind(smalltalk.Rectangle,"topLeft:",0
,function(aPoint){var self=this;
self["@leftTop"]=aPoint;
return self;}
,"accessing",unescape("topLeft%3A%20aPoint%0D%0A%09%22Set%20the%20top%20left%20corner%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09leftTop%20%3A%3D%20aPoint"));
smalltalk.bind(smalltalk.Rectangle,"topRight",0
,function(){var self=this;
return self.right()._at(self.top());}
,"accessing",unescape("topRight%0D%0A%09%22%20Return%20the%20top-right%20corner%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20right%20@%20self%20top"));
smalltalk.bind(smalltalk.Rectangle,"translateBy:",0
,function(delta){var self=this;
return self["@leftTop"]._plus(delta).rightBottom_(self["@rightBottom"]._plus(delta));}
,"accessing",unescape("translateBy%3A%20delta%0D%0A%09%22%20Return%20the%20receiver%20with%20position%20incremented%20by%20delta.%20%22%0D%0A%0D%0A%09%5EleftTop%20+%20delta%20rightBottom%3A%20rightBottom%20+%20delta"));
smalltalk.bind(smalltalk.Rectangle,"truncated",0
,function(){var self=this;
self["@leftTop"]=self["@leftTop"].truncated();
self["@rightBottom"]=self["@rightBottom"].truncated();
return self;}
,"accessing",unescape("truncated%0D%0A%09%22%20Set%20the%20receiver%20with%20the%20coordinates%20truncated%20to%20integers.%20%22%0D%0A%0D%0A%09leftTop%20%3A%3D%20leftTop%20truncated.%0D%0A%09rightBottom%20%3A%3D%20rightBottom%20truncated."));
smalltalk.bind(smalltalk.Rectangle,"width",0
,function(){var self=this;
return self["@rightBottom"].x()._minus(self["@leftTop"].x()).abs();}
,"accessing",unescape("width%0D%0A%09%22%20Return%20the%20width%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28rightBottom%20x%20-%20leftTop%20x%29%20abs"));
smalltalk.bind(smalltalk.Rectangle,"width:",0
,function(anInteger){var self=this;
self["@rightBottom"].x_(self["@leftTop"].x()._plus(anInteger));
return self;}
,"accessing",unescape("width%3A%20anInteger%0D%0A%09%22%20Set%20the%20width%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09rightBottom%20x%3A%20leftTop%20x%20+%20anInteger"));
smalltalk.bind(smalltalk.Rectangle,"__tostring",0
,function(){var self=this;
return self.toString();}
,"Lua-printing",unescape("__tostring%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.EvaluableAction,",","_comma"
,function(anAction){var self=this;
return smalltalk.ActionSequence.with_with_(self,anAction);}
,"composing",unescape("%2C%20anAction%0D%0A%09%22%20Return%20the%20receiver%20composed%20with%20the%20argument.%20%22%0D%0A%0D%0A%09%5EActionSequence%20with%3A%20self%20with%3A%20anAction"));
smalltalk.bind(smalltalk.Message.$klass,"selector:arguments:",0
,function(aString,anArray){var self=this;
return (function($rec){$rec.selector_(aString);$rec.arguments_(anArray);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("selector%3A%20aString%20arguments%3A%20anArray%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09selector%3A%20aString%3B%0D%0A%09%09arguments%3A%20anArray%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Message.$klass,"receiver:selector:arguments:",0
,function(anObject,aString,anArray){var self=this;
return (function($rec){$rec.receiver_(anObject);$rec.selector_(aString);$rec.arguments_(anArray);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("receiver%3A%20anObject%20selector%3A%20aString%20arguments%3A%20anArray%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09receiver%3A%20anObject%3B%0D%0A%09%09selector%3A%20aString%3B%0D%0A%09%09arguments%3A%20anArray%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Message,"selector",0
,function(){var self=this;
return self["@selector"];}
,"accessing",unescape("selector%0D%0A%09%22%20Return%20the%20selector%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eselector"));
smalltalk.bind(smalltalk.Message,"selector:",0
,function(aString){var self=this;
self["@selector"]=aString;
return self;}
,"accessing",unescape("selector%3A%20aString%0D%0A%09%22%20Set%20the%20selector%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09selector%20%3A%3D%20aString"));
smalltalk.bind(smalltalk.Message,"arguments:",0
,function(anArray){var self=this;
self["@arguments"]=anArray;
return self;}
,"accessing",unescape("arguments%3A%20anArray%0D%0A%09%22%20Set%20the%20arguments%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09arguments%20%3A%3D%20anArray"));
smalltalk.bind(smalltalk.Message,"arguments",0
,function(){var self=this;
return self["@arguments"];}
,"accessing",unescape("arguments%0D%0A%09%22%20Return%20the%20arguments%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Earguments"));
smalltalk.bind(smalltalk.Message,"receiver",0
,function(){var self=this;
return self["@receiver"];}
,"accessing",unescape("receiver%0D%0A%09%22%20Return%20the%20receiver%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ereceiver"));
smalltalk.bind(smalltalk.Message,"receiver:",0
,function(anObject){var self=this;
self["@receiver"]=anObject;
return self;}
,"accessing",unescape("receiver%3A%20anObject%0D%0A%09%22%20Set%20the%20receiver%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09receiver%20%3A%3D%20anObject"));
smalltalk.bind(smalltalk.Message,"comment",0
,function(){var self=this;
return ((function(){var $1$=(self[unescape("@comment")]);if(nil.isNil_($1$))return nil;return $1$;})());}
,"accessing",unescape("comment%0D%0A%09%22%20Return%20the%20comment%20of%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20basicAt%3A%20%27@comment%27"));
smalltalk.bind(smalltalk.Message,"comment:",0
,function(aString){var self=this;
if ((nil.isNil_(aString))){return self.basicDelete_(unescape("@comment"));};;
(self[unescape("@comment")]=(aString));
return self;}
,"accessing",unescape("comment%3A%20aString%0D%0A%09%22%20Set%20the%20comment%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09aString%20isNil%20ifTrue%3A%20%5B%20%5Eself%20basicDelete%3A%20%27@comment%27%20%5D.%0D%0A%09self%20basicAt%3A%20%27@comment%27%20put%3A%20aString"));
smalltalk.bind(smalltalk.Message,"perform",0
,function(){var self=this;
return self.receiver().perform_withArguments_(self.selector(),self.arguments());}
,"private",unescape("perform%0D%0A%09%22%20Return%20the%20result%20of%20sending%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20receiver%0D%0A%09%09perform%3A%20self%20selector%0D%0A%09%09withArguments%3A%20self%20arguments"));
smalltalk.bind(smalltalk.Message,"performDeferred",0
,function(){var self=this;
(function(){return self.perform();}).valueDeferred();
return self;}
,"evaluating",unescape("performDeferred%0D%0A%09%22%20Evaluate%20the%20receiver%20deferred%20in%20time.%0D%0A%09Return%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%5B%20self%20perform%20%5D%20valueDeferred."));
smalltalk.bind(smalltalk.Message,"evaluate",0
,function(){var self=this;
return self.perform();}
,"evaluating",unescape("evaluate%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20perform"));
smalltalk.bind(smalltalk.Message,"evaluateFor:",0
,function(anObject){var self=this;
return self.evaluateWithArguments_(smalltalk.Array.with_(anObject));}
,"evaluating",unescape("evaluateFor%3A%20anObject%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20with%20anObject.%20%22%0D%0A%0D%0A%09%5Eself%20evaluateWithArguments%3A%20%28Array%20with%3A%20anObject%29"));
smalltalk.bind(smalltalk.Message,"evaluateWithArguments:",0
,function(anArray){var self=this;
return self.receiver().perform_withArguments_(self.selector(),self.collectArguments_(anArray));}
,"evaluating",unescape("evaluateWithArguments%3A%20anArray%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20receiver%0D%0A%09%09perform%3A%20self%20selector%0D%0A%09%09withArguments%3A%20%28self%20collectArguments%3A%20anArray%29"));
smalltalk.bind(smalltalk.Message,"initializeArguments",0
,function(){var self=this;
self["@arguments"]=smalltalk.Array.new_(self.selector().argumentCount());
return self;}
,"private",unescape("initializeArguments%0D%0A%09%22%20Private%20-%20Initialize%20the%20arguments%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09arguments%20%3A%3D%20Array%20new%3A%20self%20selector%20argumentCount"));
smalltalk.bind(smalltalk.Message,"argumentCount",0
,function(){var self=this;
if ((nil.isNil_(self["@arguments"]))){self.initializeArguments();};;
return self["@arguments"].size();}
,"private",unescape("argumentCount%0D%0A%09%22%20Return%20the%20number%20of%20arguments%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09arguments%20isNil%20ifTrue%3A%20%5B%20self%20initializeArguments%20%5D.%0D%0A%09%5Earguments%20size"));
smalltalk.bind(smalltalk.Message,"collectArguments:",0
,function(evaluationArguments){var self=this;
if (self["@arguments"].size()._eq(evaluationArguments.size())){return evaluationArguments;};;
return evaluationArguments.copyFrom_to_((1),self.argumentCount().min_(evaluationArguments.size()));}
,"private",unescape("collectArguments%3A%20evaluationArguments%0D%0A%09%22%20Private%20-%20Return%20the%20arguments%20to%20evaluate%20the%20receiver.%20%22%0D%0A%0D%0A%09arguments%20size%20%3D%20evaluationArguments%20size%20ifTrue%3A%20%5B%20%5EevaluationArguments%20%5D.%0D%0A%09%5EevaluationArguments%20copyFrom%3A%201%20to%3A%20%28self%20argumentCount%20min%3A%20evaluationArguments%20size%29"));
smalltalk.bind(smalltalk.Message,"resolve:with:",0
,function(what,aPoolOrBlock){var self=this;
if (!(aPoolOrBlock.isMemberOf_(smalltalk.PoolDictionary))){return aPoolOrBlock.evaluateWith_(what);};;
if ((nil.isNil_(what))){return what;};;
if (!(self.isObject_(what))){return what;};;
if (what.isString()){return aPoolOrBlock.at_ifAbsent_(what,(function(){return what;}));};;
return what;}
,"resolution",unescape("resolve%3A%20what%20with%3A%20aPoolOrBlock%0D%0A%09%22%20Private%20-%20Return%20the%20receiver%20to%20be%20evaluated%20with%20mappings%20given%20by%20aPoolOrBlock.%20%22%0D%0A%0D%0A%09%28aPoolOrBlock%20isMemberOf%3A%20PoolDictionary%29%20ifFalse%3A%20%5B%0D%0A%09%09%5EaPoolOrBlock%20evaluateWith%3A%20what%0D%0A%09%5D.%0D%0A%09what%20isNil%20ifTrue%3A%20%5B%20%5Ewhat%20%5D.%0D%0A%09%28self%20isObject%3A%20what%29%20ifFalse%3A%20%5B%20%5Ewhat%20%5D.%0D%0A%09what%20isString%20ifTrue%3A%20%5B%20%5EaPoolOrBlock%20at%3A%20what%20ifAbsent%3A%20%5B%20what%20%5D%20%5D.%0D%0A%09%5Ewhat"));
smalltalk.bind(smalltalk.Message,"resolveWith:",0
,function(aPoolOrBlock){var self=this;
var args=nil;
args=(((nil.isNil_(self["@arguments"])===false)).mustBeBoolean()==true ? (function(){return self["@arguments"].collect_((function(each){return self.resolve_with_(each,aPoolOrBlock);}));})() : nil);
return self.$klass.receiver_selector_arguments_(self.resolve_with_(self["@receiver"],aPoolOrBlock),self.selector(),args);}
,"resolution",unescape("resolveWith%3A%20aPoolOrBlock%0D%0A%09%22%20Return%20a%20message%20set%20to%20be%20evaluated%20with%20mappings%20given%20by%20aPoolOrBlock.%20%22%0D%0A%0D%0A%09%7C%20args%20%7C%0D%0A%09args%20%3A%3D%20arguments%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09arguments%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09%09self%20resolve%3A%20each%20with%3A%20aPoolOrBlock%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Eself%20class%0D%0A%09%09receiver%3A%20%28self%20resolve%3A%20receiver%20with%3A%20aPoolOrBlock%29%0D%0A%09%09selector%3A%20self%20selector%0D%0A%09%09arguments%3A%20args"));
smalltalk.bind(smalltalk.Message,"isMessage",0
,function(){var self=this;
return true;}
,"testing",unescape("isMessage%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20message.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.LinkMessage,"resultAction",0
,function(){var self=this;
return self["@resultAction"];}
,"accessing",unescape("resultAction%0D%0A%09%22%20Return%20the%20resultAction%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EresultAction"));
smalltalk.bind(smalltalk.LinkMessage,"resultAction:",0
,function(anObject){var self=this;
self["@resultAction"]=anObject;
return self;}
,"accessing",unescape("resultAction%3A%20anObject%0D%0A%09%22%20Set%20the%20resultAction%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09resultAction%20%3A%3D%20anObject"));
smalltalk.bind(smalltalk.LinkMessage,"evaluate",0
,function(){var self=this;
return self.evaluateWithArguments_([]);}
,"evaluating",unescape("evaluate%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20evaluateWithArguments%3A%20%23%28%29"));
smalltalk.bind(smalltalk.LinkMessage,"collectArguments:",0
,function(evaluationArguments){var self=this;
return self.arguments().collect_((function(each){return each.evaluateWithArguments_(evaluationArguments);}));}
,"evaluating",unescape("collectArguments%3A%20evaluationArguments%0D%0A%09%22%20Private%20-%20Return%20the%20arguments%20to%20evaluate%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20arguments%20collect%3A%20%5B%3Aeach%7C%20each%20evaluateWithArguments%3A%20evaluationArguments%20%5D"));
smalltalk.bind(smalltalk.ActionSequence.$klass,"withAll:",0
,function(aCollection){var self=this;
return (function($rec){$rec.contents_(aCollection);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("withAll%3A%20aCollection%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09contents%3A%20aCollection%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.ActionSequence.$klass,"with:",0
,function(anObject){var self=this;
return self.withAll_(smalltalk.Array.with_(anObject));}
,"instantiation",unescape("with%3A%20anObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20withAll%3A%20%28Array%20with%3A%20anObject%29"));
smalltalk.bind(smalltalk.ActionSequence.$klass,"with:with:",0
,function(anObject,bObject){var self=this;
return self.withAll_(smalltalk.Array.with_with_(anObject,bObject));}
,"instantiation",unescape("with%3A%20anObject%20with%3A%20bObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20withAll%3A%20%28Array%20with%3A%20anObject%20with%3A%20bObject%29"));
smalltalk.bind(smalltalk.ActionSequence.$klass,"with:with:with:",0
,function(anObject,bObject,cObject){var self=this;
return self.withAll_(smalltalk.Array.with_with_with_(anObject,bObject,cObject));}
,"instantiation",unescape("with%3A%20anObject%20with%3A%20bObject%20with%3A%20cObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20withAll%3A%20%28Array%20with%3A%20anObject%20with%3A%20bObject%20with%3A%20cObject%29"));
smalltalk.bind(smalltalk.ActionSequence,"contents:",0
,function(aCollection){var self=this;
self["@contents"]=aCollection;
return self;}
,"private",unescape("contents%3A%20aCollection%0D%0A%09%22%20Private%20-%20Set%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09contents%20%3A%3D%20aCollection"));
smalltalk.bind(smalltalk.ActionSequence,"contents",0
,function(){var self=this;
if ((nil.isNil_(self["@contents"]))){self["@contents"]=smalltalk.Array.$$new();};;
return self["@contents"];}
,"private",unescape("contents%0D%0A%09%22%20Private%20-%20Return%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09contents%20isNil%20ifTrue%3A%20%5B%20contents%20%3A%3D%20Array%20new%20%5D.%0D%0A%09%5Econtents"));
smalltalk.bind(smalltalk.ActionSequence,"isEmpty",0
,function(){var self=this;
return (nil.isNil_(self["@contents"])).or_((function(){return self["@contents"].isEmpty();}));}
,"testing",unescape("isEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20empty.%20%22%0D%0A%0D%0A%09%5Econtents%20isNil%20or%3A%20%5B%20contents%20isEmpty%20%5D"));
smalltalk.bind(smalltalk.ActionSequence,"notEmpty",0
,function(){var self=this;
return (nil.isNil_(self["@contents"])===false).and_((function(){return self["@contents"].notEmpty();}));}
,"testing",unescape("notEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20not%20empty.%20%22%0D%0A%0D%0A%09%5Econtents%20notNil%20and%3A%20%5B%20contents%20notEmpty%20%5D"));
smalltalk.bind(smalltalk.ActionSequence,"asArray",0
,function(){var self=this;
if (self.isEmpty()){return [];};;
return self["@contents"].asArray();}
,"converting",unescape("asArray%0D%0A%09%22%20Return%20an%20array%20with%20the%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20isEmpty%20ifTrue%3A%20%5B%20%5E%23%28%29%20%5D.%0D%0A%09%5Econtents%20asArray"));
smalltalk.bind(smalltalk.ActionSequence,"asActionSequence",0
,function(){var self=this;
return self;}
,"converting",unescape("asActionSequence%0D%0A%09%22%20Return%20an%20ActionSequence%20from%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.ActionSequence,",","_comma"
,function(anAction){var self=this;
if (self.isEmpty()){return anAction.asActionSequence();};;
return self.$klass.withAll_(self.contents()._comma(anAction.asActionSequence().contents()));}
,"converting",unescape("%2C%20anAction%0D%0A%09%22%20Return%20an%20action%20composed%20with%20the%20receiver%20and%20anAction.%20%22%0D%0A%0D%0A%09self%20isEmpty%20ifTrue%3A%20%5B%20%5EanAction%20asActionSequence%20%5D.%0D%0A%09%5Eself%20class%20withAll%3A%20self%20contents%20%2C%20anAction%20asActionSequence%20contents"));
smalltalk.bind(smalltalk.ActionSequence,"copyWith:",0
,function(anObject){var self=this;
return self.$klass.withAll_(self.contents().copyWith_(anObject));}
,"converting",unescape("copyWith%3A%20anObject%0D%0A%09%22%20Return%20an%20action%20composed%20with%20the%20receiver%20and%20anObject.%20%22%0D%0A%0D%0A%09%5Eself%20class%20withAll%3A%20%28self%20contents%20copyWith%3A%20anObject%29"));
smalltalk.bind(smalltalk.ActionSequence,"do:",0
,function(aBlock){var self=this;
return ((self.notEmpty()).mustBeBoolean()==true ? (function(){return self.contents().do_(aBlock);})() : nil);}
,"converting",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20each%20object%20in%20the%20receiver%27s%20contents.%20%22%0D%0A%0D%0A%09%5Eself%20notEmpty%20ifTrue%3A%20%5B%20self%20contents%20do%3A%20aBlock%20%5D"));
smalltalk.bind(smalltalk.ActionSequence,"includes:",0
,function(anObject){var self=this;
return self.notEmpty().and_((function(){return self.contents().includes_(anObject);}));}
,"testing",unescape("includes%3A%20anObject%0D%0A%09%22%20Return%20true%20if%20anObject%20is%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20notEmpty%20and%3A%20%5B%20self%20contents%20includes%3A%20anObject%20%5D"));
smalltalk.bind(smalltalk.ActionSequence,"select:",0
,function(aBlock){var self=this;
return self.$klass.withAll_(self.asArray().select_(aBlock));}
,"enumerating",unescape("select%3A%20aBlock%0D%0A%09%22%20Return%20the%20a%20copy%20of%20the%20receiver%20with%20elements%20wich%20evaluates%20aBlock%20to%20true.%20%22%0D%0A%0D%0A%09%5Eself%20class%20withAll%3A%20%28self%20asArray%20select%3A%20aBlock%29"));
smalltalk.bind(smalltalk.ActionSequence,"reject:",0
,function(aBlock){var self=this;
return self.$klass.withAll_(self.asArray().reject_(aBlock));}
,"enumerating",unescape("reject%3A%20aBlock%0D%0A%09%22%20Return%20the%20a%20copy%20of%20the%20receiver%20with%20elements%20wich%20evaluates%20aBlock%20to%20false.%20%22%0D%0A%0D%0A%09%5Eself%20class%20withAll%3A%20%28self%20asArray%20reject%3A%20aBlock%29"));
smalltalk.bind(smalltalk.ActionSequence,"evaluate",0
,function(){var self=this;
var result=nil;
self.do_((function(each){return result=each.evaluate();}));
return result;}
,"evaluating",unescape("evaluate%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09self%20do%3A%20%5B%3Aeach%7C%20result%20%3A%3D%20each%20evaluate%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.ActionSequence,"evaluateWithArguments:",0
,function(anArray){var self=this;
var result=nil;
self.do_((function(each){return result=each.evaluateWithArguments_(anArray);}));
return result;}
,"evaluating",unescape("evaluateWithArguments%3A%20anArray%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09self%20do%3A%20%5B%3Aeach%7C%20result%20%3A%3D%20each%20evaluateWithArguments%3A%20anArray%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.ActionSequence,"evaluateFor:",0
,function(anObject){var self=this;
var result=nil;
self.do_((function(each){return result=each.evaluateFor_(anObject);}));
return result;}
,"evaluating",unescape("evaluateFor%3A%20anObject%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20receiver%20with%20anObject.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09self%20do%3A%20%5B%3Aeach%7C%20result%20%3A%3D%20each%20evaluateFor%3A%20anObject%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.ActionSequence,"reduced",0
,function(){var self=this;
return ((self.notEmpty()).mustBeBoolean()==true ? (function(){return ((self["@contents"].size()._eq((1))).mustBeBoolean()==true ? (function(){return self["@contents"].first();})() : (function(){return self;})());})() : nil);}
,"events",unescape("reduced%0D%0A%09%22%20Return%20the%20receiver%20reduced%20to%20minimal%20evaluable%20expression.%20%22%0D%0A%0D%0A%09%5Eself%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%09contents%20size%20%3D%201%0D%0A%09%09ifTrue%3A%20%5B%20contents%20first%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20self%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.ActionSequence,"evaluateWithSelector:withArguments:",0
,function(aSelector,array){var self=this;
var result=nil;
self.do_((function(each){return result=each.receiver().perform_withArguments_(aSelector,array);}));
return result;}
,"changes",unescape("evaluateWithSelector%3A%20aSelector%20withArguments%3A%20array%0D%0A%09%22%20Private%20-%20Return%20the%20result%20of%20evaluating%20the%20receiver%20with%20aSelector%20and%20arguments%20array.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09self%20do%3A%20%5B%3Aeach%7C%20result%20%3A%3D%20each%20receiver%20perform%3A%20aSelector%20withArguments%3A%20array%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.ActionSequence,"resolveWith:",0
,function(aPoolOrBlock){var self=this;
return self.$klass.withAll_(self.asArray().collect_((function(each){return ((smalltalk.responds_to_(each,("resolveWith:"))).mustBeBoolean()==true ? (function(){return each.resolveWith_(aPoolOrBlock);})() : (function(){return each;})());})));}
,"resolution",unescape("resolveWith%3A%20aPoolOrBlock%0D%0A%09%22%20Return%20an%20instance%20set%20to%20be%20evaluated%20with%20mappings%20given%20by%20aPoolOrBlock.%20%22%0D%0A%0D%0A%09%5Eself%20class%20withAll%3A%20%28self%20asArray%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09%28each%20respondsTo%3A%20%23resolveWith%3A%29%0D%0A%09%09ifTrue%3A%20%5B%20each%20resolveWith%3A%20aPoolOrBlock%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20each%20%5D%0D%0A%09%5D%29"));
smalltalk.bind(smalltalk.EventManager.$klass,"eventsTriggered",0
,function(){var self=this;
if ((nil.isNil_(self["@eventsTriggered"]))){self.initializeEventsTriggered();};;
return self["@eventsTriggered"];}
,"events",unescape("eventsTriggered%0D%0A%09%22%20Return%20all%20the%20events%20managed%20by%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09eventsTriggered%20isNil%20ifTrue%3A%20%5B%20self%20initializeEventsTriggered%20%5D.%0D%0A%09%5EeventsTriggered"));
smalltalk.bind(smalltalk.EventManager.$klass,"eventsTriggered:",0
,function(aCollection){var self=this;
self["@eventsTriggered"]=aCollection;
return self;}
,"events",unescape("eventsTriggered%3A%20aCollection%0D%0A%09%22%20Set%20all%20the%20events%20managed%20by%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09eventsTriggered%20%3A%3D%20aCollection"));
smalltalk.bind(smalltalk.EventManager.$klass,"initializeEventsTriggered",0
,function(){var self=this;
self.eventsTriggered_(self.constructEventsTriggered());
return self;}
,"events",unescape("initializeEventsTriggered%0D%0A%09%22%20Private%20-%20Initialize%20all%20the%20events%20managed%20by%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20eventsTriggered%3A%20self%20constructEventsTriggered"));
smalltalk.bind(smalltalk.EventManager,"eventTable",0
,function(){var self=this;
return (((nil.isNil_(self["@handlers"]))).mustBeBoolean()==true ? (function(){return self.eventTableCreate();})() : (function(){return self["@handlers"];})());}
,"events",unescape("eventTable%0D%0A%09%22%20Private%20-%20Return%20the%20mapping%20event%20names%20to%20actions%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ehandlers%20isNil%0D%0A%09%09ifTrue%3A%20%5B%20self%20eventTableCreate%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20handlers%20%5D"));
smalltalk.bind(smalltalk.EventManager,"eventTableForEdit",0
,function(){var self=this;
if ((nil.isNil_(self["@handlers"]))){self["@handlers"]=self.eventTableCreate();};;
return self["@handlers"];}
,"events",unescape("eventTableForEdit%0D%0A%09%22%20Private%20-%20Return%20the%20mapping%20event%20names%20to%20actions%20of%20the%20receiver.%0D%0A%09Set%20the%20internal%20map%20if%20not%20present.%0D%0A%09%22%0D%0A%0D%0A%09handlers%20isNil%20ifTrue%3A%20%5B%20handlers%20%3A%3D%20self%20eventTableCreate%20%5D.%0D%0A%09%5Ehandlers"));
smalltalk.bind(smalltalk.EventManager,"releaseEventTable",0
,function(){var self=this;
self["@handlers"]=nil;
return self;}
,"events",unescape("releaseEventTable%0D%0A%09%22%20Private%20-%20Release%20the%20event%20table.%20%22%0D%0A%0D%0A%09handlers%20%3A%3D%20nil"));
smalltalk.bind(smalltalk.S8Encoder.$klass,"for:",0
,function(aCompiler){var self=this;
return self.support().$$new().compiler_(aCompiler);}
,"instantiation",unescape("for%3A%20aCompiler%0D%0A%09%22%20Private%20-%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20support%20new%20compiler%3A%20aCompiler"));
smalltalk.bind(smalltalk.S8Encoder.$klass,"support",0
,function(){var self=this;
var lang=nil;
lang=smalltalk.Smalltalk.nativeLanguage();
return self.allSubclasses().detect_ifNone_((function(cls){return cls.supportsLanguage_(lang);}),(function(){return self.invalidMessage_("support");}));}
,"instantiation",unescape("support%0D%0A%09%22%20Private%20-%20Return%20the%20support%20to%20encode%20for%20native%20platform.%20%22%0D%0A%0D%0A%09%7C%20lang%20%7C%0D%0A%09lang%20%3A%3D%20Smalltalk%20nativeLanguage.%0D%0A%09%5Eself%20allSubclasses%0D%0A%09%09detect%3A%20%5B%3Acls%7C%20cls%20supportsLanguage%3A%20lang%20%5D%0D%0A%09%09ifNone%3A%20%5B%20self%20invalidMessage%3A%20%23support%20%5D"));
smalltalk.bind(smalltalk.S8Encoder.$klass,"supportsLanguage:",0
,function(aLanguage){var self=this;
return false;}
,"instantiation",unescape("supportsLanguage%3A%20aLanguage%0D%0A%09%22%20Private%20-%20Return%20true%20if%20instances%20of%20the%20receiver%20can%20encode%20to%20aLanguage.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.S8Encoder.$klass,"export:",0
,function(spec){var self=this;
return self.$$new().export_(spec);}
,"Tools-export",unescape("export%3A%20spec%0D%0A%09%22%20Return%20the%20native%20code%20result%20of%20encoding%20spec.%20%22%0D%0A%0D%0A%09%5Eself%20new%20export%3A%20spec"));
smalltalk.bind(smalltalk.S8Encoder,"messageSendConnector",0
,function(){var self=this;
return self.implementedBySubclass_("messageSendConnector");}
,"encoding",unescape("messageSendConnector%0D%0A%09%22%20Private%20-%20Return%20the%20connector%20to%20use%20for%20sending%20a%20message.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23messageSendConnector%20"));
smalltalk.bind(smalltalk.S8Encoder,"propertyAccessConnector",0
,function(){var self=this;
return self.messageSendConnector();}
,"encoding",unescape("propertyAccessConnector%0D%0A%09%22%20Private%20-%20Return%20the%20connector%20to%20use%20for%20accessing%20a%20native%20property.%20%22%0D%0A%0D%0A%09%5Eself%20messageSendConnector%20"));
smalltalk.bind(smalltalk.S8Encoder,"codeToGetGlobal:",0
,function(aName){var self=this;
return "smalltalk"._comma(self.propertyAccessConnector())._comma(aName);}
,"encoding",unescape("codeToGetGlobal%3A%20aName%0D%0A%09%22%20Private%20-%20Return%20the%20native%20code%20to%20get%20global%20at%20aName.%20%22%0D%0A%0D%0A%09%5E%23smalltalk%20%2Cself%20propertyAccessConnector%20%2CaName%20"));
smalltalk.bind(smalltalk.S8Encoder,"nativeUndefined",0
,function(){var self=this;
return self.implementedBySubclass_("nativeUndefined");}
,"encoding",unescape("nativeUndefined%0D%0A%09%22%20Private%20-%20Return%20the%20code%20for%20undefined.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23nativeUndefined%20"));
smalltalk.bind(smalltalk.S8Encoder,"nativeNameOfMetaClass:",0
,function(aClass){var self=this;
return self.implementedBySubclass_("nativeNameOfMetaClass:");}
,"encoding",unescape("nativeNameOfMetaClass%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20native%20name%20for%20metaclass%20aClass.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23nativeNameOfMetaClass%3A%20"));
smalltalk.bind(smalltalk.S8Encoder,"nativeNameOfClass:",0
,function(aClass){var self=this;
if (aClass.isMetaclass()){return self.nativeNameOfMetaClass_(aClass);};;
return aClass.name();}
,"encoding",unescape("nativeNameOfClass%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20native%20name%20for%20aClass.%20%22%0D%0A%0D%0A%09aClass%20isMetaclass%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20nativeNameOfMetaClass%3A%20aClass%0D%0A%09%5D.%0D%0A%09%5EaClass%20name%20"));
smalltalk.bind(smalltalk.S8Encoder,"smalltalkNameFor:",0
,function(aClass){var self=this;
if ((nil.isNil_(aClass))){return self.nativeUndefined();};;
return self.codeToGetGlobal_(self.nativeNameOfClass_(aClass));}
,"export",unescape("smalltalkNameFor%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20global%20name%20of%20aClass%20%28can%20be%20nil%29.%20%22%0D%0A%0D%0A%09aClass%20isNil%20ifTrue%3A%20%5B%20%5Eself%20nativeUndefined%20%5D.%0D%0A%09%5Eself%20codeToGetGlobal%3A%20%28self%20nativeNameOfClass%3A%20aClass%29%20"));
smalltalk.bind(smalltalk.S8Encoder,"compiler",0
,function(){var self=this;
return self["@compiler"];}
,"accessing",unescape("compiler%0D%0A%09%22%20Return%20the%20compiler%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ecompiler"));
smalltalk.bind(smalltalk.S8Encoder,"compiler:",0
,function(aCompiler){var self=this;
self["@compiler"]=aCompiler;
return self;}
,"accessing",unescape("compiler%3A%20aCompiler%0D%0A%09%22%20Set%20the%20compiler%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09compiler%20%3A%3D%20aCompiler"));
smalltalk.bind(smalltalk.S8Encoder,"compilerEvent:",0
,function(selector){var self=this;
return self["@compiler"].triggerEvent_(selector);}
,"events",unescape("compilerEvent%3A%20selector%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20compiler%27s%20event.%20%22%0D%0A%0D%0A%09%5Ecompiler%20triggerEvent%3A%20selector"));
smalltalk.bind(smalltalk.S8Encoder,"compilerEvent:ifNotHandled:",0
,function(selector,aBlock){var self=this;
return self["@compiler"].triggerEvent_ifNotHandled_(selector,aBlock);}
,"events",unescape("compilerEvent%3A%20selector%20ifNotHandled%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20compiler%27s%20event.%20%22%0D%0A%0D%0A%09%5Ecompiler%20triggerEvent%3A%20selector%20ifNotHandled%3A%20aBlock"));
smalltalk.bind(smalltalk.S8Encoder,"compilerEvent:with:",0
,function(selector,anObject){var self=this;
return self["@compiler"].triggerEvent_with_(selector,anObject);}
,"events",unescape("compilerEvent%3A%20selector%20with%3A%20anObject%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20compiler%27s%20event%20with%20anObject.%20%22%0D%0A%0D%0A%09%5Ecompiler%0D%0A%09%09triggerEvent%3A%20selector%0D%0A%09%09with%3A%20anObject"));
smalltalk.bind(smalltalk.S8Encoder,"compilerEvent:with:ifNotHandled:",0
,function(selector,anObject,aBlock){var self=this;
return self["@compiler"].triggerEvent_with_ifNotHandled_(selector,anObject,aBlock);}
,"events",unescape("compilerEvent%3A%20selector%20with%3A%20anObject%20ifNotHandled%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20compiler%27s%20event%20with%20anObject.%20%22%0D%0A%0D%0A%09%5Ecompiler%0D%0A%09%09triggerEvent%3A%20selector%0D%0A%09%09with%3A%20anObject%0D%0A%09%09ifNotHandled%3A%20aBlock"));
smalltalk.bind(smalltalk.S8Encoder,"compilerEvent:withArguments:",0
,function(selector,array){var self=this;
return self["@compiler"].triggerEvent_withArguments_(selector,array);}
,"events",unescape("compilerEvent%3A%20selector%20withArguments%3A%20array%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20compiler%27s%20event%20with%20the%20arguments.%20%22%0D%0A%0D%0A%09%5Ecompiler%0D%0A%09%09triggerEvent%3A%20selector%0D%0A%09%09withArguments%3A%20array"));
smalltalk.bind(smalltalk.S8Encoder,"compilerEvent:withArguments:ifNotHandled:",0
,function(selector,array,aBlock){var self=this;
return self["@compiler"].triggerEvent_withArguments_ifNotHandled_(selector,array,aBlock);}
,"events",unescape("compilerEvent%3A%20selector%20withArguments%3A%20array%20ifNotHandled%3A%20aBlock%0D%0A%09%22%20Return%20the%20result%20of%20triggering%20the%20compiler%27s%20event%20with%20the%20arguments.%20%22%0D%0A%0D%0A%09%5Ecompiler%0D%0A%09%09triggerEvent%3A%20selector%0D%0A%09%09withArguments%3A%20array%0D%0A%09%09ifNotHandled%3A%20aBlock"));
smalltalk.bind(smalltalk.S8Encoder,"currentClass",0
,function(){var self=this;
return self["@compiler"].currentClass();}
,"accessing",unescape("currentClass%0D%0A%09%22%20Return%20the%20current%20class%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ecompiler%20currentClass"));
smalltalk.bind(smalltalk.S8Encoder,"ignore:",0
,function(aNode){var self=this;
return self;}
,"visiting",unescape("ignore%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"asSelector:",0
,function(aSelector){var self=this;
return aSelector.asSelector();}
,"converting",unescape("asSelector%3A%20aSelector%0D%0A%09%22%20Return%20the%20native%20representation%20of%20aSelector.%20%22%0D%0A%0D%0A%09%5EaSelector%20asSelector"));
smalltalk.bind(smalltalk.S8Encoder,"knownVariables",0
,function(){var self=this;
return self["@compiler"].knownVariables();}
,"variables",unescape("knownVariables%0D%0A%09%22%20Return%20the%20knownVariables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ecompiler%20knownVariables"));
smalltalk.bind(smalltalk.S8Encoder,"localName:",0
,function(aName){var self=this;
if (self.pseudoVariables().includes_(aName)){return aName;};;
return self.asSelector_(aName);}
,"private",unescape("localName%3A%20aName%0D%0A%09%22%20Private%20-%20Return%20a%20valid%20local%20name%20for%20aName.%20%22%0D%0A%0D%0A%09%28self%20pseudoVariables%20includes%3A%20aName%29%20ifTrue%3A%20%5B%20%5EaName%20%5D.%0D%0A%09%5Eself%20asSelector%3A%20aName"));
smalltalk.bind(smalltalk.S8Encoder,"alienName:",0
,function(aName){var self=this;
return aName;}
,"private",unescape("alienName%3A%20aName%0D%0A%09%22%20Private%20-%20Return%20a%20valid%20native/alien%20name%20for%20aName.%20%22%0D%0A%0D%0A%09%5EaName"));
smalltalk.bind(smalltalk.S8Encoder,"nativeLanguage",0
,function(){var self=this;
return nil;}
,"accessing",unescape("nativeLanguage%0D%0A%09%22%20Return%20the%20nativeLanguage%20of%20the%20receiver.%0D%0A%09The%20default%20action%20return%20nil%20%28the%20default%20native%20language%29.%0D%0A%09%22%0D%0A%0D%0A%09%5Enil"));
smalltalk.bind(smalltalk.S8Encoder,"emmitedCode",0
,function(){var self=this;
return self.implementedBySubclass_("emmitedCode");}
,"results",unescape("emmitedCode%0D%0A%09%22%20Return%20the%20emmited%20code%20during%20compilation%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23emmitedCode"));
smalltalk.bind(smalltalk.S8Encoder,"endCompiling:",0
,function(aMethodNode){var self=this;
return self;}
,"private",unescape("endCompiling%3A%20aMethodNode%0D%0A%09%22%20Private%20-%20Finish%20emmiting%20code%20for%20aMethodNode.%20%22"));
smalltalk.bind(smalltalk.S8Encoder,"startCompiling:",0
,function(aNode){var self=this;
return self;}
,"private",unescape("startCompiling%3A%20aNode%0D%0A%09%22%20Private%20-%20Set%20the%20receiver%20to%20start%20emmiting%20code%20for%20aNode.%20%22"));
smalltalk.bind(smalltalk.S8Encoder,"prepare:",0
,function(aNode){var self=this;
if (aNode.isParseFailure()){return nil;};;
aNode.do_((function(each,parent){return each.prepareIn_for_(parent,self);}));
return self;}
,"prepare",unescape("prepare%3A%20aNode%0D%0A%09%22%20Private%20-%20Prepare%20aNode%20for%20traversal.%20%22%0D%0A%0D%0A%09aNode%20isParseFailure%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09aNode%20do%3A%20%5B%3Aeach%20%3Aparent%7C%0D%0A%09%09each%20prepareIn%3A%20parent%20for%3A%20self%0D%0A%09%5D"));
smalltalk.bind(smalltalk.S8Encoder,"pseudoVariables",0
,function(){var self=this;
return self["@compiler"].pseudoVariables();}
,"variables",unescape("pseudoVariables%0D%0A%09%22%20Return%20the%20pseudoVariables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ecompiler%20pseudoVariables"));
smalltalk.bind(smalltalk.S8Encoder,"tempVariables",0
,function(){var self=this;
return self["@compiler"].tempVariables();}
,"variables",unescape("tempVariables%0D%0A%09%22%20Return%20the%20tempVariables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ecompiler%20tempVariables"));
smalltalk.bind(smalltalk.S8Encoder,"unknownVariables",0
,function(){var self=this;
return self["@compiler"].unknownVariables();}
,"variables",unescape("unknownVariables%0D%0A%09%22%20Return%20the%20unknownVariables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ecompiler%20unknownVariables"));
smalltalk.bind(smalltalk.S8Encoder,"visit:",0
,function(aNode){var self=this;
return aNode.accept_(self);}
,"visiting",unescape("visit%3A%20aNode%0D%0A%09%5EaNode%20accept%3A%20self"));
smalltalk.bind(smalltalk.S8Encoder,"visitAssignmentNode:",0
,function(aNode){var self=this;
self.visitNode_(aNode);
return self;}
,"visiting",unescape("visitAssignmentNode%3A%20aNode%0D%0A%09self%20visitNode%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"visitBlockNode:",0
,function(aNode){var self=this;
self.visitNode_(aNode);
return self;}
,"visiting",unescape("visitBlockNode%3A%20aNode%0D%0A%09self%20visitNode%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"visitBlockSequenceNode:",0
,function(aNode){var self=this;
self.visitSequenceNode_(aNode);
return self;}
,"visiting",unescape("visitBlockSequenceNode%3A%20aNode%0D%0A%09self%20visitSequenceNode%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"visitCascadeNode:",0
,function(aNode){var self=this;
self.visitNode_(aNode);
return self;}
,"visiting",unescape("visitCascadeNode%3A%20aNode%0D%0A%09self%20visitNode%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"visitMacroAssignment:to:",0
,function(aVariableNode,anExpressionNode){var self=this;
return false;}
,"visiting",unescape("visitMacroAssignment%3A%20aVariableNode%20to%3A%20anExpressionNode%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20generated%20code%20to%20a%20macro%20assignment%20to%20aVariableNode.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.S8Encoder,"visitMethodNode:",0
,function(aNode){var self=this;
self.visitNode_(aNode);
return self;}
,"visiting",unescape("visitMethodNode%3A%20aNode%0D%0A%09%22%20Visit%20a%20method%20node.%20%22%0D%0A%0D%0A%09self%20visitNode%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"visitNode:",0
,function(aNode){var self=this;
self.ignore_(aNode);
return self;}
,"visiting",unescape("visitNode%3A%20aNode%0D%0A%09self%20ignore%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"visitReturnNode:",0
,function(aNode){var self=this;
self.visitNode_(aNode);
return self;}
,"visiting",unescape("visitReturnNode%3A%20aNode%0D%0A%09self%20visitNode%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"visitSendNode:",0
,function(aNode){var self=this;
self.visitNode_(aNode);
return self;}
,"visiting",unescape("visitSendNode%3A%20aNode%0D%0A%09self%20visitNode%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"visitSequenceNode:",0
,function(aNode){var self=this;
self.visitNode_(aNode);
return self;}
,"visiting",unescape("visitSequenceNode%3A%20aNode%0D%0A%09self%20visitNode%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"visitValueNode:",0
,function(aNode){var self=this;
self.visitNode_(aNode);
return self;}
,"visiting",unescape("visitValueNode%3A%20aNode%0D%0A%09self%20visitNode%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"visitVariableNode:",0
,function(aNode){var self=this;
self.ignore_(aNode);
return self;}
,"visiting",unescape("visitVariableNode%3A%20aNode%0D%0A%09self%20ignore%3A%20aNode"));
smalltalk.bind(smalltalk.S8Encoder,"translate:by:",0
,function(aNode,delta){var self=this;
if (delta._eq((0))){return self;};;
self.compilerEvent_withArguments_("move:by:",smalltalk.Array.with_with_(aNode,delta));
return self;}
,"private",unescape("translate%3A%20aNode%20by%3A%20delta%0D%0A%09%22%20Private%20-%20Translate%20the%20contents%20of%20aNode%20by%20delta%20characters%20in%20source%20code.%20%22%0D%0A%0D%0A%09delta%20%3D%200%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09self%09compilerEvent%3A%20%23move%3Aby%3A%0D%0A%09%09withArguments%3A%20%28Array%20with%3A%20aNode%20with%3A%20delta%29."));
smalltalk.bind(smalltalk.S8Encoder,"loadRequirementsOf:",0
,function(aClass){var self=this;
return self.implementedBySubclass_("loadRequirementsOf:");}
,"export",unescape("loadRequirementsOf%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20requirements%20to%20load%20aClass.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23loadRequirementsOf%3A%20"));
smalltalk.bind(smalltalk.S8Encoder,"exportDefinitionOf:on:",0
,function(aClass,aStream){var self=this;
return self.implementedBySubclass_("exportDefinitionOf:on:");}
,"export",unescape("exportDefinitionOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23exportDefinitionOf%3Aon%3A%20"));
smalltalk.bind(smalltalk.S8Encoder,"exportMetaDefinitionOf:on:",0
,function(aClass,aStream){var self=this;
return self.implementedBySubclass_("exportMetaDefinitionOf:on:");}
,"export",unescape("exportMetaDefinitionOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20class%20side%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23exportMetaDefinitionOf%3Aon%3A%20"));
smalltalk.bind(smalltalk.S8Encoder,"exportCommentOf:on:",0
,function(aClass,aStream){var self=this;
return self.implementedBySubclass_("exportCommentOf:on:");}
,"export",unescape("exportCommentOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20the%20comment%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23exportCommentOf%3Aon%3A%20"));
smalltalk.bind(smalltalk.S8Encoder,"export:method:on:",0
,function(aClass,aMethod,aStream){var self=this;
return self.implementedBySubclass_("export:method:on:");}
,"export",unescape("export%3A%20aClass%20method%3A%20aMethod%20on%3A%20aStream%0D%0A%09%22%20Export%20a%20method%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23export%3Amethod%3Aon%3A%20"));
smalltalk.bind(smalltalk.S8Encoder,"arrayLiteralStart",0
,function(){var self=this;
return self.implementedBySubclass_("arrayLiteralStart");}
,"literals",unescape("arrayLiteralStart%0D%0A%09%22%20Private%20-%20Return%20the%20code%20string%20to%20start%20exporting%20an%20array%20literal.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23arrayLiteralStart"));
smalltalk.bind(smalltalk.S8Encoder,"arrayLiteralEnd",0
,function(){var self=this;
return self.implementedBySubclass_("arrayLiteralEnd");}
,"literals",unescape("arrayLiteralEnd%0D%0A%09%22%20Private%20-%20Return%20the%20code%20string%20to%20end%20exporting%20an%20array%20literal.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23arrayLiteralEnd"));
smalltalk.bind(smalltalk.S8Encoder,"escaped:",0
,function(string){var self=this;
return unescape("%22")._comma(string)._comma(unescape("%22"));}
,"literals",unescape("escaped%3A%20string%0D%0A%09%22%20Private%20-%20Return%20the%20code%20string%20to%20export%20string%20literal.%20%22%0D%0A%0D%0A%09%5E%24%22%20%2Cstring%20%2C%24%22"));
smalltalk.bind(smalltalk.S8Encoder,"argumentsSeparatorOp",0
,function(){var self=this;
return unescape("%2C");}
,"constants",unescape("argumentsSeparatorOp%0D%0A%09%22%20Private%20-%20Return%20the%20OP%20for%20separating%20arguments.%20%22%0D%0A%0D%0A%09%5E%24%2C%20"));
smalltalk.bind(smalltalk.S8Encoder,"export:literals:on:",0
,function(tags,anArray,aStream){var self=this;
if ((nil.isNil_(tags)===false)){aStream.nextPutAll_(tags.first());};;
anArray.do_separatedBy_((function(each){return aStream.nextPutAll_(((each.isString()).mustBeBoolean()==true ? (function(){return unescape("%22")._comma(each)._comma(unescape("%22"));})() : (function(){return each.printString();})()));}),(function(){return aStream.nextPutAll_(self.argumentsSeparatorOp());}));
if ((nil.isNil_(tags)===false)){aStream.nextPutAll_(tags.last());};;
return self;}
,"literals",unescape("export%3A%20tags%20literals%3A%20anArray%20on%3A%20aStream%0D%0A%09%22%20Private%20-%20Export%20the%20native%20code%20for%20elements%20in%20anArray%20between%20tags.%20%22%0D%0A%0D%0A%09tags%20notNil%20ifTrue%3A%20%5B%20aStream%20nextPutAll%3A%20tags%20first%20%5D.%0D%0A%09anArray%0D%0A%09%20%20%20%20do%3A%20%5B%3Aeach%20%7C%20aStream%20nextPutAll%3A%20%28%0D%0A%09%09each%20isString%20ifTrue%3A%20%5B%20%24%22%20%2Ceach%20%2C%24%22%20%5D%20ifFalse%3A%20%5B%20each%20printString%20%5D%0D%0A%09%20%20%20%20%29%20%5D%20separatedBy%3A%20%5B%20aStream%20nextPutAll%3A%20self%20argumentsSeparatorOp%20%5D.%0D%0A%09tags%20notNil%20ifTrue%3A%20%5B%20aStream%20nextPutAll%3A%20tags%20last%20%5D"));
smalltalk.bind(smalltalk.S8Encoder,"exportArrayLiteral:on:newLine:",0
,function(anArray,aStream,newLine){var self=this;
self.export_literals_on_(smalltalk.Array.with_with_(self.arrayLiteralStart(),self.arrayLiteralEnd()),anArray,aStream);
if (newLine){aStream.cr();};;
return self;}
,"literals",unescape("exportArrayLiteral%3A%20anArray%20on%3A%20aStream%20newLine%3A%20newLine%0D%0A%09%22%20Private%20-%20Export%20the%20native%20code%20for%20elements%20in%20anArray.%20%22%0D%0A%0D%0A%09self%09export%3A%20%28Array%0D%0A%09%09%09with%3A%20self%20arrayLiteralStart%0D%0A%09%09%09with%3A%20self%20arrayLiteralEnd%29%0D%0A%09%09literals%3A%20anArray%20on%3A%20aStream.%0D%0A%09newLine%20ifTrue%3A%20%5B%20aStream%20cr%20%5D"));
smalltalk.bind(smalltalk.S8Encoder,"sourceToTry:",0
,function(sourceCode){var self=this;
return sourceCode;}
,"directives",unescape("sourceToTry%3A%20sourceCode%0D%0A%09%22%20Return%20the%20code%20to%20emmit%20to%20try%20execution%20of%20sourceCode.%20%22%0D%0A%0D%0A%09%5EsourceCode%20"));
smalltalk.bind(smalltalk.S8Encoder,"sourceToDirectCall:with:",0
,function(nativeSelector,arg){var self=this;
return self.implementedBySubclass_("sourceToDirectCall:with:");}
,"directives",unescape("sourceToDirectCall%3A%20nativeSelector%20with%3A%20arg%0D%0A%09%22%20Return%20the%20code%20to%20emmit%20to%20direct%20native%20call%20to%20selector%20with%20argument.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23sourceToDirectCall%3Awith%3A%20"));
smalltalk.bind(smalltalk.S8Encoder,"export:",0
,function(spec){var self=this;
var exporter=nil;
exporter=smalltalk.Exporter.$$new();
exporter.encoder_(self);
if (spec._eq("all")){return exporter.exportAll();};;
return exporter.export_(spec);}
,"Tools-export",unescape("export%3A%20spec%0D%0A%09%22%20Return%20the%20native%20code%20result%20of%20encoding%20spec.%20%22%0D%0A%0D%0A%09%7C%20exporter%20%7C%0D%0A%09exporter%20%3A%3D%20Exporter%20new.%0D%0A%09exporter%20encoder%3A%20self.%0D%0A%09spec%20%3D%20%23all%20ifTrue%3A%20%5B%20%5Eexporter%20exportAll%20%5D.%0D%0A%22OBSOLETE-%0D%0A%09spec%20isArray%20ifTrue%3A%20%5B%20%5Eexporter%20exportAll%3A%20spec%20%5D.%0D%0A%09%5Eexporter%20exportAll%3A%20%28Array%20with%3A%20spec%29%0D%0A%22%09%5Eexporter%20export%3A%20spec"));
smalltalk.bind(smalltalk.OptimizingEncoder.$klass,"initializeInlines",0
,function(){var self=this;
self["@inlines"]=smalltalk.PoolDictionary.$$new();
self.$klass.selectors().select_((function(msg){return msg.startsWith_("initialize").and_((function(){return msg.endsWith_("Inlines:");}));})).do_((function(each){return self.perform_with_(each,self["@inlines"]);}));
return self["@inlines"];}
,"inlines",unescape("initializeInlines%0D%0A%09%22%20Private%20-%20Initialize%20the%20pool%20of%20inlines%20known%20by%20the%20receiver.%0D%0A%09Note%20that%20initialize%20defaults%20from%20methods%20implementing%20%23initialize*Inlines%3A%20in%20the%20receiver%0D%0A%09%22%0D%0A%0D%0A%09inlines%20%3A%3D%20PoolDictionary%20new.%0D%0A%09%28self%20class%20selectors%0D%0A%09%09select%3A%20%5B%3Amsg%7C%20%28msg%20startsWith%3A%20%23initialize%29%20and%3A%20%5B%20msg%20endsWith%3A%20%23Inlines%3A%20%5D%20%5D%29%0D%0A%09%09do%3A%20%5B%3Aeach%7C%20self%20perform%3A%20each%20with%3A%20inlines%20%5D.%0D%0A%09%5Einlines%20"));
smalltalk.bind(smalltalk.OptimizingEncoder.$klass,"inlines",0
,function(){var self=this;
if ((nil.isNil_(self["@inlines"]))){self.initializeInlines();};;
return self["@inlines"];}
,"inlines",unescape("inlines%0D%0A%09%22%20Private%20-%20Return%20the%20pool%20of%20inlines.%20%22%0D%0A%0D%0A%09inlines%20isNil%20ifTrue%3A%20%5B%20self%20initializeInlines%20%5D.%0D%0A%09%5Einlines"));
smalltalk.bind(smalltalk.OptimizingEncoder.$klass,"register:into:",0
,function(literals,pool){var self=this;
literals.do_((function(tuple){var tokens=nil;
var value=nil;
if (!(tuple.isArray())){self.error_("Must be array "._comma(tuple));};;if (!(tuple.size()._eq((2)))){self.error_(unescape("Invalid%20tuple%20size%20%28must%20be%202%29"));};;tokens=tuple.first();value=tuple.last();if (tokens.isString()){tokens=smalltalk.Array.with_(tokens);};;return tokens.do_((function(token){if (!(token.isString())){self.error_("Must be string "._comma(token.asLiteral()));};;return ((token.isEmpty().or_((function(){return token.first().isLowercase();}))).mustBeBoolean()==true ? (function(){return pool.at_put_(token,value);})() : (function(){var options=nil;
options=pool.at_ifAbsentPut_(token,(function(){return smalltalk.PoolDictionary.$$new();}));return value.do_((function(pair){if (!(pair.isArray())){self.error_("Must be array "._comma(tuple));};;if (!(pair.size()._eq((2)))){self.error_(unescape("Invalid%20pair%20size%20%28must%20be%202%29%20-%20")._comma(pair.asLiteral()));};;value=pair.last();return ((pair.first().isString()).mustBeBoolean()==true ? (function(){return smalltalk.Array.with_(pair.first());})() : (function(){return pair.first();})()).do_((function(selector){if (!(selector.isString())){self.error_("Must be string "._comma(selector.asLiteral()));};;return options.at_put_(selector,value);}));}));})());}));}));
return self;}
,"inlines",unescape("register%3A%20literals%20into%3A%20pool%0D%0A%09%22%20Private%20-%20Register%20literals%20array%20onto%20pool%20dictionary.%0D%0A%09literals%20must%20be%20a%20collection%20of%20arrays%20like%3A%0D%0A%09%09%23%28token%20inlines%29%0D%0A%09%09%23%28ClassName%20%23%28%0D%0A%09%09%09%23%28selector%20inlines%29%0D%0A%09%09%29%29%0D%0A%09inlines%20can%20be%20a%20string%20or%20array%20of%20strings%20to%20be%20used%20to%20encode%20inlines%20in%20source%0D%0A%09token%20must%20be%20lowercase%20and%20ClassName%20must%20be%20uppercase%20names%0D%0A%09token%20%26%20className%20can%20be%20arrays%20for%20multiple%20registrations%0D%0A%09%22%0D%0A%0D%0A%09literals%20do%3A%20%5B%3Atuple%7C%20%7C%20tokens%20value%20%7C%0D%0A%09%09tuple%20isArray%20ifFalse%3A%20%5B%20self%20error%3A%20%27Must%20be%20array%20%27%20%2Ctuple%20%5D.%0D%0A%09%09tuple%20size%20%3D%202%20ifFalse%3A%20%5B%20self%20error%3A%20%27Invalid%20tuple%20size%20%28must%20be%202%29%27%20%5D.%0D%0A%09%09tokens%20%3A%3D%20tuple%20first.%20value%20%3A%3D%20tuple%20last.%0D%0A%09%09tokens%20isString%20ifTrue%3A%20%5B%20tokens%20%3A%3D%20Array%20with%3A%20tokens%20%5D.%0D%0A%09%09tokens%20do%3A%20%5B%3Atoken%7C%0D%0A%09%09%09token%20isString%20ifFalse%3A%20%5B%20self%20error%3A%20%27Must%20be%20string%20%27%20%2Ctoken%20asLiteral%20%5D.%0D%0A%09%09%09%28token%20isEmpty%20or%3A%20%5B%20token%20first%20isLowercase%20%5D%29%20ifTrue%3A%20%5B%20%22%23%28token%20inlines%29%20%22%0D%0A%09%09%09%09pool%20at%3A%20token%20put%3A%20value%0D%0A%09%09%09%5D%20ifFalse%3A%20%5B%20%7C%20options%20%7C%20%22%23%28ClassName%20pairs%29%22%0D%0A%09%09%09%09options%20%3A%3D%20pool%20at%3A%20token%20ifAbsentPut%3A%20%5BPoolDictionary%20new%5D.%0D%0A%09%09%09%09value%20do%3A%20%5B%3Apair%7C%20%22%23%28selector%20inlines%29%22%0D%0A%09%09%09%09%09pair%20isArray%20ifFalse%3A%20%5B%20self%20error%3A%20%27Must%20be%20array%20%27%20%2Ctuple%20%5D.%0D%0A%09%09%09%09%09pair%20size%20%3D%202%20ifFalse%3A%20%5B%20self%20error%3A%20%27Invalid%20pair%20size%20%28must%20be%202%29%20-%20%27%20%2Cpair%20asLiteral%20%5D.%0D%0A%09%09%09%09%09value%20%3A%3D%20pair%20last.%0D%0A%09%09%09%09%09%28pair%20first%20isString%20ifTrue%3A%20%5BArray%20with%3A%20pair%20first%5D%20ifFalse%3A%20%5Bpair%20first%5D%29%20do%3A%20%5B%3Aselector%7C%0D%0A%09%09%09%09%09%09selector%20isString%20ifFalse%3A%20%5B%20self%20error%3A%20%27Must%20be%20string%20%27%20%2Cselector%20asLiteral%20%5D.%0D%0A%09%09%09%09%09%09options%20at%3A%20selector%20put%3A%20value%0D%0A%09%09%09%09%09%5D%0D%0A%09%09%09%09%5D%0D%0A%09%09%09%5D%0D%0A%09%09%5D%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.OptimizingEncoder.$klass,"inlinesFor:compiler:",0
,function(token,aCompiler){var $s8Ret$={name:"stReturn"};try{var self=this;
var result=nil;
if ((nil.isNil_(token)===false)){result=self.inlines().at_ifAbsent_(token,(function(){return nil;}));
(((nil.isNil_(result)===false)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return result})();throw($s8Ret$)})();})() : nil);};;
(function(){$s8Ret$.result=(function(){return self.inlines()._at(smalltalk.Array.with_with_(aCompiler.currentClass().name(),aCompiler.selector()))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"inlines",unescape("inlinesFor%3A%20token%20compiler%3A%20aCompiler%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20stream%20with%20native%20source%20chunks%20to%20inject%20while%20generating%20code.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09token%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09result%20%3A%3D%20self%20inlines%20at%3A%20token%20ifAbsent%3A%20%5B%5D.%0D%0A%09%09result%20notNil%20ifTrue%3A%20%5B%20%5Eresult%20%5D.%0D%0A%09%5D.%0D%0A%09%5Eself%20inlines%20@%20%28Array%0D%0A%09%09with%3A%20aCompiler%20currentClass%20name%0D%0A%09%09with%3A%20aCompiler%20selector%29%20"));
smalltalk.bind(smalltalk.OptimizingEncoder.$klass,"unknownVariable:in:",0
,function(aNode,anEncoder){var self=this;
self.note_("Unknown variable "._comma(aNode.value().asString()));
return self.error_("Unknown variable "._comma(aNode.value()));}
,"variables",unescape("unknownVariable%3A%20aNode%20in%3A%20anEncoder%0D%0A%09%22%20Private%20-%20anEncoder%20has%20found%20an%20unknown%20name%20reference.%0D%0A%09The%20default%20action%20will%20thrown%20an%20error%20%28stop%20compilation%29%2C%0D%0A%09%20this%20method%20can%20be%20refined/changed%20to%20silently%20ignore%20the%0D%0A%09%20unknown%20to%20let%20the%20compiler%20produce%20code%20to%20refer%20code%0D%0A%09%20in%20outer%20native%20contexts.%0D%0A%09See%20implementors%20of%20%23visitVariableNode%3A%0D%0A%09%22%0D%0A%0D%0Aself%20note%3A%20%27Unknown%20variable%20%27%20%2CaNode%20value%20asString.%0D%0A%09%5Eself%20error%3A%20%27Unknown%20variable%20%27%2CaNode%20value"));
smalltalk.bind(smalltalk.OptimizingEncoder.$klass,"bind:in:",0
,function(aName,anEncoder){var $s8Ret$={name:"stReturn"};try{var self=this;
aName.detect_ifNone_((function(c){return c._no_eq("_");}),(function(){smalltalk.Smalltalk.nativeObjectAt_ifAbsent_(aName,(function(){return (smalltalk.Smalltalk.global()[aName]=(nil));}));anEncoder.knownVariables().add_(aName);return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"variables",unescape("bind%3A%20aName%20in%3A%20anEncoder%0D%0A%09%22%20Private%20-%20Return%20true%20if%20aName%20has%20been%20bound%20to%20anEncoder.%0D%0A%09Note%20that%20the%20process%20of%20binding%20must%20register%20aName%20onto%20knownVariables%20of%20encoder.%20%22%0D%0A%0D%0A%09%22we%20use%20underscores%20as%20native%20cookies%22%0D%0A%09aName%20detect%3A%20%5B%3Ac%7C%20c%20%7E%3D%20%23_%20%5D%20ifNone%3A%20%5B%20%22all%20underscores%22%0D%0A%09%09Smalltalk%20nativeObjectAt%3A%20aName%20ifAbsent%3A%20%5B%0D%0A%09%09%09Smalltalk%20global%20basicAt%3A%20aName%20put%3A%20nil%0D%0A%09%09%5D.%0D%0A%09%09anEncoder%20knownVariables%20add%3A%20aName.%0D%0A%09%09%5Etrue%0D%0A%09%5D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.OptimizingEncoder,"addOptimizations:",0
,function(array){var self=this;
array.do_((function(opt){return self.optimizationMap().addAll_(self.optimizationLiteralsFor_(opt));}));
return self;}
,"optimization",unescape("addOptimizations%3A%20array%0D%0A%09%22%20Private%20-%20Add%20an%20array%20of%20optimizations%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09array%20do%3A%20%5B%3Aopt%7C%0D%0A%09%09self%20optimizationMap%20addAll%3A%0D%0A%09%09%09%28self%20optimizationLiteralsFor%3A%20opt%29%0D%0A%09%5D"));
smalltalk.bind(smalltalk.OptimizingEncoder,"allBlockNoArguments:",0
,function(aNode){var self=this;
return self.receiverIsBlockNoArguments_(aNode).and_((function(){return self.hasBlockNoArguments_(aNode);}));}
,"optimization",unescape("allBlockNoArguments%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20receiver%20and%20arguments%20of%20aNode%20are%20blocks%20with%20no%20arguments.%20%22%0D%0A%0D%0A%09%5E%28self%20receiverIsBlockNoArguments%3A%20aNode%29%20and%3A%20%5B%0D%0A%09%09self%20hasBlockNoArguments%3A%20aNode%0D%0A%09%5D"));
smalltalk.bind(smalltalk.OptimizingEncoder,"arg1HasNumber:",0
,function(aNode){var self=this;
return aNode.arguments().at_((1)).hasNumber();}
,"optimization",unescape("arg1HasNumber%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20argument%20of%20aNode%20is%20a%20number%20literal.%20%22%0D%0A%0D%0A%09%5E%28aNode%20arguments%20at%3A%201%29%20hasNumber"));
smalltalk.bind(smalltalk.OptimizingEncoder,"arg2HasNumber:",0
,function(aNode){var self=this;
return aNode.arguments().at_((2)).hasNumber();}
,"optimization",unescape("arg2HasNumber%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20argument%20of%20aNode%20is%20a%20number%20literal.%20%22%0D%0A%0D%0A%09%5E%28aNode%20arguments%20at%3A%202%29%20hasNumber"));
smalltalk.bind(smalltalk.OptimizingEncoder,"arg3HasNumber:",0
,function(aNode){var self=this;
return aNode.arguments().at_((3)).hasNumber();}
,"optimization",unescape("arg3HasNumber%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20argument%20of%20aNode%20is%20a%20number%20literal.%20%22%0D%0A%0D%0A%09%5E%28aNode%20arguments%20at%3A%203%29%20hasNumber"));
smalltalk.bind(smalltalk.OptimizingEncoder,"argumentsHasNumber:",0
,function(aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
aNode.arguments().do_((function(each){return ((each.hasNumber()).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"optimization",unescape("argumentsHasNumber%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20any%20argument%20of%20aNode%20is%20a%20number%20literal.%20%22%0D%0A%0D%0A%09aNode%20arguments%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20hasNumber%20ifTrue%3A%20%5B%20%5Etrue%20%5D%0D%0A%09%5D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.OptimizingEncoder,"argumentsHasString:",0
,function(aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
aNode.arguments().do_((function(each){return ((each.hasString()).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"optimization",unescape("argumentsHasString%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20any%20argument%20of%20aNode%20is%20a%20string%20literal.%20%22%0D%0A%0D%0A%09aNode%20arguments%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20hasString%20ifTrue%3A%20%5B%20%5Etrue%20%5D%0D%0A%09%5D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.OptimizingEncoder,"defaultKnownOptimizations",0
,function(){var self=this;
return [];}
,"defaults",unescape("defaultKnownOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20default%20knownOptimizations%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%29"));
smalltalk.bind(smalltalk.OptimizingEncoder,"defaultOptimizations",0
,function(){var self=this;
return [];}
,"defaults",unescape("defaultOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20default%20optimizations%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%29"));
smalltalk.bind(smalltalk.OptimizingEncoder,"hasBlockArguments:",0
,function(aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
aNode.arguments().do_((function(each){return ((smalltalk.is_kindOf_(each,(smalltalk.BlockNode))).mustBeBoolean()==true ? nil : (function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();})());}));
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"optimization",unescape("hasBlockArguments%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20arguments%20of%20aNode%20are%20blocks.%20%22%0D%0A%0D%0A%09aNode%20arguments%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28each%20isKindOf%3A%20BlockNode%29%20ifFalse%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%5D.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.OptimizingEncoder,"hasBlockNoArguments:",0
,function(aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
if (!(self.hasBlockArguments_(aNode))){return false;};;
aNode.arguments().do_((function(each){return ((each.parameters().isEmpty()).mustBeBoolean()==true ? nil : (function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();})());}));
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"optimization",unescape("hasBlockNoArguments%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20arguments%20of%20aNode%20are%20blocks%20with%20no%20parameters.%20%22%0D%0A%0D%0A%09%28self%20hasBlockArguments%3A%20aNode%29%20ifFalse%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09aNode%20arguments%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20parameters%20isEmpty%20ifFalse%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%5D.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.OptimizingEncoder,"hasBlockOneArguments:",0
,function(aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
if (!(self.hasBlockArguments_(aNode))){return false;};;
aNode.arguments().do_((function(each){return ((each.parameters().size()._eq((1))).mustBeBoolean()==true ? nil : (function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();})());}));
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"optimization",unescape("hasBlockOneArguments%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20arguments%20of%20aNode%20are%20blocks%20with%20ONE%20parameter.%20%22%0D%0A%0D%0A%09%28self%20hasBlockArguments%3A%20aNode%29%20ifFalse%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09aNode%20arguments%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20parameters%20size%20%3D%201%20ifFalse%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%5D.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.OptimizingEncoder,"hasNumber:",0
,function(aNode){var self=this;
return self.receiverHasNumber_(aNode).or_((function(){return self.argumentsHasNumber_(aNode);}));}
,"optimization",unescape("hasNumber%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20the%20receiver%20or%20arguments%20in%20aNode%20has%20numeric%20value.%20%22%0D%0A%0D%0A%09%5E%28self%20receiverHasNumber%3A%20aNode%29%20or%3A%20%5B%0D%0A%09%09self%20argumentsHasNumber%3A%20aNode%0D%0A%09%5D"));
smalltalk.bind(smalltalk.OptimizingEncoder,"hasString:",0
,function(aNode){var self=this;
return self.receiverHasString_(aNode).or_((function(){return self.argumentsHasString_(aNode);}));}
,"optimization",unescape("hasString%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20the%20receiver%20or%20arguments%20in%20aNode%20has%20string%20value.%20%22%0D%0A%0D%0A%09%5E%28self%20receiverHasString%3A%20aNode%29%20or%3A%20%5B%0D%0A%09%09self%20argumentsHasString%3A%20aNode%0D%0A%09%5D"));
smalltalk.bind(smalltalk.OptimizingEncoder,"noClient:",0
,function(aNode){var self=this;
return aNode.client()._eq("none");}
,"optimization",unescape("noClient%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20aNode%20has%20no%20client.%20%22%0D%0A%0D%0A%09%5EaNode%20client%20%3D%20%23none%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"hasClient:",0
,function(aNode){var self=this;
var client=nil;
client=aNode.client();
return (nil.isNil_(client)===false).and_((function(){return client._no_eq("none");}));}
,"optimization",unescape("hasClient%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20aNode%20has%20a%20client.%20%22%0D%0A%0D%0A%09%7C%20client%20%7C%0D%0A%09client%20%3A%3D%20aNode%20client.%0D%0A%09%5Eclient%20notNil%20and%3A%20%5B%20client%20%7E%3D%20%23none%20%5D%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"initializeKnownOptimizations",0
,function(){var self=this;
self["@knownOptimizations"]=self.defaultKnownOptimizations();
return self;}
,"initialize",unescape("initializeKnownOptimizations%0D%0A%09%22%20Private%20-%20Initialize%20the%20knownOptimizations%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09knownOptimizations%20%3A%3D%20self%20defaultKnownOptimizations"));
smalltalk.bind(smalltalk.OptimizingEncoder,"initializeOptimizationMap",0
,function(){var self=this;
self["@optimizationMap"]=self.compilerEvent_ifNotHandled_("needsOptimizationMap",(function(){return [];}));
self.compilerEvent_ifNotHandled_("initializeOptimizations",(function(){return self.addOptimizations_(self.defaultOptimizations());}));
self.compilerEvent_("customizeOptimizations");
return self;}
,"initialize",unescape("initializeOptimizationMap%0D%0A%09%22%20Private%20-%20Initialize%20the%20optimization%20map%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09optimizationMap%20%3A%3D%20self%0D%0A%09%09compilerEvent%3A%20%23needsOptimizationMap%0D%0A%09%09ifNotHandled%3A%20%5B%20%23%28%29%20%5D.%0D%0A%09self%20compilerEvent%3A%20%23initializeOptimizations%20ifNotHandled%3A%20%5B%0D%0A%09%09self%20addOptimizations%3A%20self%20defaultOptimizations%0D%0A%09%5D.%0D%0A%09self%20compilerEvent%3A%20%23customizeOptimizations"));
smalltalk.bind(smalltalk.OptimizingEncoder,"knownOptimizations",0
,function(){var self=this;
if ((nil.isNil_(self["@knownOptimizations"]))){self.initializeKnownOptimizations();};;
return self["@knownOptimizations"];}
,"accessing",unescape("knownOptimizations%0D%0A%09%22%20Return%20the%20knownOptimizations%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09knownOptimizations%20isNil%20ifTrue%3A%20%5B%20self%20initializeKnownOptimizations%20%5D.%0D%0A%09%5EknownOptimizations"));
smalltalk.bind(smalltalk.OptimizingEncoder,"knownOptimizations:",0
,function(anObject){var self=this;
self["@knownOptimizations"]=anObject;
return self;}
,"accessing",unescape("knownOptimizations%3A%20anObject%0D%0A%09%22%20Set%20the%20knownOptimizations%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09knownOptimizations%20%3A%3D%20anObject"));
smalltalk.bind(smalltalk.OptimizingEncoder,"matchOptimizedCondition:with:",0
,function(condition,aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
if (["hasNumber:", "receiverHasNumber:", "argumentsHasNumber:", "hasString:", "receiverHasString:", "argumentsHasString:", "arg1HasNumber:", "arg2HasNumber:", "arg3HasNumber:", "hasBlockArguments:", "hasBlockNoArguments:", "hasBlockOneArguments:", "allBlockNoArguments:", "receiverIsBlockNoArguments:", "receiverIsBlock:", "noClient:", "hasClient:"].includes_(condition)){return self.perform_withArguments_(condition,smalltalk.Array.with_(aNode));};;
if (condition.isArray()){condition.detect_ifNone_((function(one){return false._eq(self.matchOptimizedCondition_with_(one,aNode));}),(function(){return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();};;
if (condition._eq("true")){(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();};;
if (condition._eq("false")){(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();};;
self.note_(unescape("Unknown%20optimize-condition%3A%20")._comma(condition));
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"optimization",unescape("matchOptimizedCondition%3A%20condition%20with%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20aNode%20matches%20the%20condition.%20%22%0D%0A%0D%0A%09%28%23%28%0D%0A%09hasNumber%3A%20receiverHasNumber%3A%20argumentsHasNumber%3A%0D%0A%09hasString%3A%20receiverHasString%3A%20argumentsHasString%3A%0D%0A%09arg1HasNumber%3A%20arg2HasNumber%3A%20arg3HasNumber%3A%0D%0A%09hasBlockArguments%3A%20hasBlockNoArguments%3A%20hasBlockOneArguments%3A%0D%0A%09allBlockNoArguments%3A%20receiverIsBlockNoArguments%3A%20receiverIsBlock%3A%0D%0A%09noClient%3A%20hasClient%3A%0D%0A%09%29%20includes%3A%20condition%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20perform%3A%20condition%20withArguments%3A%20%28Array%20with%3A%20aNode%29%0D%0A%09%5D.%0D%0A%09condition%20isArray%20ifTrue%3A%20%5B%0D%0A%09%09condition%20detect%3A%20%5B%3Aone%7C%0D%0A%09%09%09false%20%3D%20%28self%20matchOptimizedCondition%3A%20one%20with%3A%20aNode%29%0D%0A%09%09%5D%20ifNone%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09%09%5Efalse%0D%0A%09%5D.%0D%0A%09condition%20%3D%20%23true%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09condition%20%3D%20%23false%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09self%20note%3A%20%27Unknown%20optimize-condition%3A%20%27%2Ccondition.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.OptimizingEncoder,"optimizationLiteralsFor:",0
,function(opt){var $s8Ret$={name:"stReturn"};try{var self=this;
var array=nil;
if (!(opt.isString())){return smalltalk.Array.with_(opt);};;
array=self.knownOptimizations();
array.do_((function(pair){return ((pair.first()._eq(opt)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return pair.last().inject_into_([],(function(total,tuple){return (function($rec){$rec.addAll_(self.optimizationLiteralsFor_(tuple));return $rec.yourself();})(total);}))})();throw($s8Ret$)})();})() : nil);}));
array.do_((function(pair){return pair.last().do_((function(tuple){return ((tuple.isArray().and_((function(){return tuple.first()._eq(opt);}))).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return smalltalk.Array.with_(tuple)})();throw($s8Ret$)})();})() : nil);}));}));
(function(){$s8Ret$.result=(function(){return []})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"optimization",unescape("optimizationLiteralsFor%3A%20opt%0D%0A%09%22%20Private%20-%20Return%20the%20optimization%20literals%20array%20defined%20for%20opt.%0D%0A%09opt%20can%20be%20an%20optimization%20family%20or%20message%20selector%0D%0A%09%22%0D%0A%0D%0A%09%7C%20array%20%7C%0D%0A%09opt%20isString%20ifFalse%3A%20%5B%20%5EArray%20with%3A%20opt%20%5D.%0D%0A%09array%20%3A%3D%20self%20knownOptimizations.%0D%0A%09array%20do%3A%20%5B%3Apair%7C%0D%0A%09%09pair%20first%20%3D%20opt%20ifTrue%3A%20%5B%0D%0A%09%09%09%5Epair%20last%20inject%3A%20%23%28%29%20into%3A%20%5B%3Atotal%20%3Atuple%7C%0D%0A%09%09%09%09total%20addAll%3A%20%28self%20optimizationLiteralsFor%3A%20tuple%29%3B%20yourself%0D%0A%09%09%09%5D%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09array%20do%3A%20%5B%3Apair%7C%0D%0A%09%09pair%20last%20do%3A%20%5B%3Atuple%7C%0D%0A%09%09%09%28tuple%20isArray%20and%3A%20%5Btuple%20first%20%3D%20opt%5D%29%20ifTrue%3A%20%5B%20%5EArray%20with%3A%20tuple%20%5D%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5E%23%28%29"));
smalltalk.bind(smalltalk.OptimizingEncoder,"optimizationMap",0
,function(){var self=this;
if ((nil.isNil_(self["@optimizationMap"]))){self.initializeOptimizationMap();};;
return self["@optimizationMap"];}
,"accessing",unescape("optimizationMap%0D%0A%09%22%20Return%20the%20message%20send%20optimization%20map%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09optimizationMap%20isNil%20ifTrue%3A%20%5B%20self%20initializeOptimizationMap%20%5D.%0D%0A%09%5EoptimizationMap"));
smalltalk.bind(smalltalk.OptimizingEncoder,"optimizationMap:",0
,function(aSpec){var self=this;
self["@optimizationMap"]=aSpec;
return self;}
,"accessing",unescape("optimizationMap%3A%20aSpec%0D%0A%09%22%20Set%20the%20current%20optimizationMap%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09optimizationMap%20%3A%3D%20aSpec"));
smalltalk.bind(smalltalk.OptimizingEncoder,"emmitCode:",0
,function(code){var self=this;
return self.implementedBySubclass_("emmitCode:");}
,"generation",unescape("emmitCode%3A%20code%0D%0A%09%22%20Private%20-%20Emmit%20code%20onto%20output%20stream.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23emmitCode%3A%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"emmitInlinedCode:",0
,function(code){var self=this;
return self.emmitCode_(code);}
,"generation",unescape("emmitInlinedCode%3A%20code%0D%0A%09%22%20Private%20-%20Emmit%20inlined%20code%20onto%20output%20stream.%20%22%0D%0A%0D%0A%09%5Eself%20emmitCode%3A%20code%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"macroToken",0
,function(){var self=this;
return unescape("%25");}
,"macros",unescape("macroToken%0D%0A%09%22%20Private%20-%20Return%20the%20%28default%29%20macro%20token.%20%22%0D%0A%0D%0A%09%5E%24%25%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"macroStart",0
,function(){var self=this;
return self.macroToken();}
,"macros",unescape("macroStart%0D%0A%09%22%20Private%20-%20Return%20the%20macro%20start%20token.%20%22%0D%0A%0D%0A%09%5Eself%20macroToken%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"macroEnd",0
,function(){var self=this;
return self.macroToken();}
,"macros",unescape("macroEnd%0D%0A%09%22%20Private%20-%20Return%20the%20macro%20end%20token.%20%22%0D%0A%0D%0A%09%5Eself%20macroToken%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"macroStream:",0
,function(macro){var self=this;
return macro.readStream();}
,"macros",unescape("macroStream%3A%20macro%0D%0A%09%22%20Private%20-%20Return%20a%20code%20stream%20from%20macro%20string.%20%22%0D%0A%0D%0A%09%5Emacro%20readStream%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"expandMacrosIn:with:",0
,function(input,aNode){var self=this;
while(!(function(){return input.atEnd();})()){var code=nil;
code=input.upTo_(self.macroStart());
if (code.size()._gt((0))){self.emmitInlinedCode_(code);};;
((input.atEnd()).mustBeBoolean()==true ? nil : (function(){return self.visitMacroIn_with_(input,aNode);})());};;
return self;}
,"macros",unescape("expandMacrosIn%3A%20input%20with%3A%20aNode%0D%0A%09%22%20Private%20-%20Parse%20input%20expanding%20macros.%0D%0A%09Expandable%20macros%20are%20defined%20between%20macroToken%0D%0A%09%22%0D%0A%0D%0A%09%5Binput%20atEnd%5D%20whileFalse%3A%20%5B%20%7C%20code%20%7C%0D%0A%09%09code%20%3A%3D%20input%20upTo%3A%20self%20macroStart.%0D%0A%09%09code%20size%20%3E%200%20ifTrue%3A%20%5B%20self%20emmitInlinedCode%3A%20code%20%5D.%0D%0A%09%09input%20atEnd%20ifFalse%3A%20%5B%20self%20visitMacroIn%3A%20input%20with%3A%20aNode%20%5D%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"optimize:macro:",0
,function(aNode,template){var $s8Ret$={name:"stReturn"};try{var self=this;
if (template.isArray()){template.detect_ifNone_((function(one){return self.visitOptimizedNode_as_(aNode,one);}),(function(){return (function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();};;
if (!(template.isString())){(function(){$s8Ret$.result=(function(){return (nil.isNil_(template.value_value_(aNode,self))===false)})();throw($s8Ret$)})();};;
self.expandMacrosIn_with_(self.macroStream_(template),aNode);
(function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"macros",unescape("optimize%3A%20aNode%20macro%3A%20template%0D%0A%09%22%20Private%20-%20Generate%20the%20code%20for%20aNode%20using%20a%20macro%20template.%0D%0A%09Return%20true%20if%20expanded%20the%20macro%2C%20or%20leave%20the%20receiver%20unchanged%20returning%20false%0D%0A%09%22%0D%0A%0D%0A%09template%20isArray%20ifTrue%3A%20%5B%20%22conditional%20templates%22%0D%0A%09%09template%20detect%3A%20%5B%3Aone%7C%20self%20visitOptimizedNode%3A%20aNode%20as%3A%20one%20%5D%0D%0A%09%09%09ifNone%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%09%5Etrue%0D%0A%09%5D.%0D%0A%09template%20isString%20ifFalse%3A%20%5B%20%22an%20evaluable%20template%20%28must%20return%20nil%20if%20failed%20to%20expand%29%22%0D%0A%09%09%5E%28template%20value%3A%20aNode%20value%3A%20self%29%20notNil%0D%0A%09%5D.%0D%0A%09self%20expandMacrosIn%3A%20%28self%20macroStream%3A%20template%29%20with%3A%20aNode.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.OptimizingEncoder,"receiverHasNumber:",0
,function(aNode){var self=this;
return aNode.receiver().hasNumber();}
,"optimization",unescape("receiverHasNumber%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20the%20receiver%20of%20aNode%20is%20a%20number%20literal.%20%22%0D%0A%0D%0A%09%5EaNode%20receiver%20hasNumber"));
smalltalk.bind(smalltalk.OptimizingEncoder,"receiverHasString:",0
,function(aNode){var self=this;
return aNode.receiver().hasString();}
,"optimization",unescape("receiverHasString%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20the%20receiver%20of%20aNode%20is%20a%20string%20literal.%20%22%0D%0A%0D%0A%09%5EaNode%20receiver%20hasString"));
smalltalk.bind(smalltalk.OptimizingEncoder,"receiverIsBlock:",0
,function(aNode){var self=this;
return smalltalk.is_kindOf_(aNode.receiver(),(smalltalk.BlockNode));}
,"optimization",unescape("receiverIsBlock%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20receiver%20of%20aNode%20is%20block.%22%0D%0A%0D%0A%09%5EaNode%20receiver%20isKindOf%3A%20BlockNode%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"receiverIsBlockNoArguments:",0
,function(aNode){var self=this;
return self.receiverIsBlock_(aNode).and_((function(){return aNode.receiver().parameters().isEmpty();}));}
,"optimization",unescape("receiverIsBlockNoArguments%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20receiver%20of%20aNode%20is%20block%20with%20no%20arguments.%20%22%0D%0A%0D%0A%09%5E%28self%20receiverIsBlock%3A%20aNode%29%20and%3A%20%5B%0D%0A%09%09aNode%20receiver%20parameters%20isEmpty%0D%0A%09%5D"));
smalltalk.bind(smalltalk.OptimizingEncoder,"removeOptimizationLiteral:",0
,function(aLiteral){var self=this;
self.optimizationMap().remove_(aLiteral);
return self;}
,"optimization",unescape("removeOptimizationLiteral%3A%20aLiteral%0D%0A%09%22%20Private%20-%20Remove%20the%20optimization%20from%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20optimizationMap%20remove%3A%20aLiteral%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"removeOptimizations:",0
,function(array){var self=this;
array.do_((function(opt){return self.optimizationLiteralsFor_(opt).do_((function(each){return self.removeOptimizationLiteral_(each);}));}));
return self;}
,"optimization",unescape("removeOptimizations%3A%20array%0D%0A%09%22%20Private%20-%20Remove%20an%20array%20of%20optimizations%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09array%20do%3A%20%5B%3Aopt%7C%0D%0A%09%09%28self%20optimizationLiteralsFor%3A%20opt%29%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%09self%20removeOptimizationLiteral%3A%20each%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.OptimizingEncoder,"inlineToken:",0
,function(string){var self=this;
return string.copyFrom_to_((2),string.size());}
,"inlines",unescape("inlineToken%3A%20string%0D%0A%09%22%20Private%20-%20Return%20the%20inline%20key%20to%20search%20for%20contents%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Estring%20copyFrom%3A%202%20to%3A%20string%20size%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"inlineKey:",0
,function(string){var self=this;
if (string.size()._eq((0))){return "";};;
if (string.startsWith_(unescape("%23"))){return self.inlineToken_(string);};;
return nil;}
,"inlines",unescape("inlineKey%3A%20string%0D%0A%09%22%20Private%20-%20Return%20the%20inline%20key%20to%20search%20for%20inline%20contents%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09string%20size%20%3D%200%20ifTrue%3A%20%5B%20%5E%27%27%20%5D.%0D%0A%09%28string%20startsWith%3A%20%24%23%29%20ifTrue%3A%20%5B%20%5Eself%20inlineToken%3A%20string%20%5D.%0D%0A%09%5Enil%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"inlinesFor:",0
,function(string){var self=this;
var chunks=nil;
if ((nil.isNil_(self["@inlines"])===false)){return self["@inlines"];};;
chunks=self.$klass.inlinesFor_compiler_(self.inlineKey_(string),self.compiler());
if ((nil.isNil_(chunks))){chunks=[];};;
if (chunks.isString()){chunks=smalltalk.Array.with_(chunks);};;
self["@inlines"]=chunks.stream();
return self["@inlines"];}
,"inlines",unescape("inlinesFor%3A%20string%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20stream%20with%20native%20source%20chunks%20to%20inject%20while%20generating%20code.%20%22%0D%0A%0D%0A%09%7C%20chunks%20%7C%0D%0A%09inlines%20notNil%20ifTrue%3A%20%5B%20%5Einlines%20%5D.%0D%0A%09chunks%20%3A%3D%20self%20class%20inlinesFor%3A%20%28self%20inlineKey%3A%20string%29%20compiler%3A%20self%20compiler.%0D%0A%09chunks%20isNil%20ifTrue%3A%20%5B%20chunks%20%3A%3D%20%23%28%29%20%5D.%0D%0A%09chunks%20isString%20ifTrue%3A%20%5B%20chunks%20%3A%3D%20Array%20with%3A%20chunks%20%5D.%0D%0A%09inlines%20%3A%3D%20chunks%20stream.%0D%0A%09%5Einlines"));
smalltalk.bind(smalltalk.OptimizingEncoder,"missingNativeCodeFor:as:",0
,function(aNode,lang){var self=this;
return self.error_("Missing native code for language "._comma(lang));}
,"inlines",unescape("missingNativeCodeFor%3A%20aNode%20as%3A%20lang%0D%0A%09%22%20Private%20-%20Return%20the%20native%20language%20code%20for%20missing%20expression%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%27Missing%20native%20code%20for%20language%20%27%20%2Clang%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"inline:source:",0
,function(aNode,string){var self=this;
var chunks=nil;
chunks=self.inlinesFor_(string);
return ((chunks.atEnd()).mustBeBoolean()==true ? nil : (function(){return chunks.next();})());}
,"inlines",unescape("inline%3A%20aNode%20source%3A%20string%0D%0A%09%22%20Return%20the%20string%20source%20for%20inline%20node.%20%22%0D%0A%0D%0A%09%7C%20chunks%20%7C%0D%0A%09chunks%20%3A%3D%20self%20inlinesFor%3A%20string.%0D%0A%09%5Echunks%20atEnd%20ifFalse%3A%20%5B%20chunks%20next%20%5D%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitInlineNode:",0
,function(aNode){var self=this;
var string=nil;
var lang=nil;
lang=self.nativeLanguage();
if ((nil.isNil_(lang))){lang=smalltalk.Smalltalk.nativeLanguage();};;
string=aNode.sourceFor_in_(lang,self);
if (string._eq("self")){return nil;};;
if (string.size()._eq((0))){string=self.missingNativeCodeFor_as_(aNode,lang);};;
return (((nil.isNil_(string)===false)).mustBeBoolean()==true ? (function(){return self.optimize_macro_(aNode,string);})() : nil);}
,"inlines",unescape("visitInlineNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Encode%20an%20inline%20node%20containing%20native%20language%20code.%20%22%0D%0A%0D%0A%09%7C%20string%20lang%20%7C%0D%0A%09lang%20%3A%3D%20self%20nativeLanguage.%0D%0A%09lang%20isNil%20ifTrue%3A%20%20%5B%20lang%20%3A%3D%20Smalltalk%20nativeLanguage%20%5D.%0D%0A%09string%20%3A%3D%20aNode%20sourceFor%3A%20lang%20in%3A%20self.%0D%0A%09string%20%3D%20%23self%20ifTrue%3A%20%5B%20%5Enil%20%5D.%20%22inlined%20%27self%27%20is%20encoded%20a%20no-operation%20%28optional%20inline%29%22%0D%0A%09string%20size%20%3D%200%20ifTrue%3A%20%20%5B%20string%20%3A%3D%20self%20missingNativeCodeFor%3A%20aNode%20as%3A%20lang%20%5D.%0D%0A%09%5Estring%20notNil%20ifTrue%3A%20%5B%20self%20optimize%3A%20aNode%20macro%3A%20string%20%5D%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"startCompiling:",0
,function(aNode){var self=this;
self["@inlines"]=nil;
return smalltalk.superImplementor(smalltalk.OptimizingEncoder,"startCompiling_").apply(self, [aNode]);}
,"private",unescape("startCompiling%3A%20aNode%0D%0A%09%22%20Private%20-%20Set%20the%20receiver%20to%20start%20emmiting%20code%20for%20aNode.%20%22%0D%0A%0D%0A%09inlines%20%3A%3D%20nil.%0D%0A%09%5Esuper%20startCompiling%3A%20aNode"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitInlined:",0
,function(aNode){var self=this;
if (!(smalltalk.is_kindOf_(aNode,(smalltalk.BlockNode)))){return self.error_(unescape("Invalid%20block%20node%20%28")._comma(aNode)._comma(unescape("%29.")));};;
aNode.nodes().do_((function(each){return self.visit_(each);}));
return self;}
,"optimization",unescape("visitInlined%3A%20aNode%0D%0A%09%22%20Private%20-%20Encode%20aNode%20as%20inlined%20sequence%20of%20statements.%20%22%0D%0A%0D%0A%09%28aNode%20isKindOf%3A%20BlockNode%29%20ifFalse%3A%20%5B%0D%0A%09%09%5Eself%20error%3A%20%27Invalid%20block%20node%20%28%27%20%2CaNode%20%2C%27%29.%27%0D%0A%09%5D.%0D%0A%09aNode%20nodes%20do%3A%20%5B%3Aeach%20%7C%20self%20visit%3A%20each%20%5D."));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitInlinedMacro:in:with:",0
,function(macro,input,aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
if (macro._eq("inlinedReceiver")){return self.visitInlined_(aNode.receiver());};;
if (macro.startsWith_("inlined")){var args=nil;
args=aNode.arguments();
(1).to_do_(args.size(),(function(i){return (("inlined"._comma(i.asString())._eq(macro)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return self.visitInlined_(args.at_(i))})();throw($s8Ret$)})();})() : nil);}));};;
(function(){$s8Ret$.result=(function(){return self.error_(unescape("Invalid%20macro%20%5B")._comma(macro)._comma(unescape("%5D")))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"optimizing",unescape("visitInlinedMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20inlined%20macro%20in%20input.%20%22%0D%0A%0D%0A%09macro%20%3D%20%23inlinedReceiver%20ifTrue%3A%20%5B%20%5Eself%20visitInlined%3A%20aNode%20receiver%20%5D.%0D%0A%09%28macro%20startsWith%3A%20%23inlined%29%20ifTrue%3A%20%5B%20%7C%20args%20%7C%0D%0A%09%09args%20%3A%3D%20aNode%20arguments.%0D%0A%09%091%20to%3A%20args%20size%20do%3A%20%5B%3Ai%7C%0D%0A%09%09%09%23inlined%20%2Ci%20asString%20%3D%20macro%20ifTrue%3A%20%5B%0D%0A%09%09%09%09%5Eself%20visitInlined%3A%20%28args%20at%3A%20i%29%0D%0A%09%09%09%5D.%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5Eself%20error%3A%20%27Invalid%20macro%20%5B%27%2Cmacro%2C%27%5D%27%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitSeqMacro:in:with:",0
,function(macro,input,aNode){var self=this;
var args=nil;
var i=nil;
if (macro._eq("sequence")){return self.visitSequenceNode_(aNode);};;
i=macro.copyFrom_to_((4),macro.size()).asNumber();
args=aNode.arguments();
if (i.between_and_((1),args.size())){var block=nil;
block=args.at_(i);
if (!(block.parameters().size()._eq((0)))){self.error_(unescape("Invalid%20block%20%28has%20parameters%29"));};;
return self.visitSequenceNode_(block.nodes().first());};;
return self.error_(unescape("Invalid%20macro%20%5B")._comma(macro)._comma(unescape("%5D")));}
,"optimizing",unescape("visitSeqMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20argument%20macro%20in%20input.%20%22%0D%0A%0D%0A%09%7C%20args%20i%20%7C%0D%0A%09macro%20%3D%20%23sequence%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20visitSequenceNode%3A%20aNode%0D%0A%09%5D.%0D%0A%09i%20%3A%3D%20%28macro%20copyFrom%3A%204%20to%3A%20macro%20size%29%20asNumber.%0D%0A%09args%20%3A%3D%20aNode%20arguments.%0D%0A%09%28i%20between%3A%201%20and%3A%20args%20size%29%20ifTrue%3A%20%5B%20%7C%20block%20%7C%0D%0A%09%09block%20%3A%3D%20args%20at%3A%20i.%0D%0A%09%09block%20parameters%20size%20%3D%200%20ifFalse%3A%20%5B%0D%0A%09%09%09self%20error%3A%20%27Invalid%20block%20%28has%20parameters%29%27%0D%0A%09%09%5D.%0D%0A%09%09%5Eself%20visitSequenceNode%3A%20block%20nodes%20first%0D%0A%09%5D.%0D%0A%09%5Eself%20error%3A%20%27Invalid%20macro%20%5B%27%2Cmacro%2C%27%5D%27%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitArgMacro:in:with:",0
,function(macro,input,aNode){var self=this;
var args=nil;
var i=nil;
i=macro.copyFrom_to_((4),macro.size()).asNumber();
args=aNode.arguments();
if (i.between_and_((1),args.size())){return self.visit_(args.at_(i));};;
return self.error_(unescape("Invalid%20macro%20%5B")._comma(macro)._comma(unescape("%5D")));}
,"optimizing",unescape("visitArgMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20argument%20macro%20in%20input.%20%22%0D%0A%0D%0A%09%7C%20args%20i%20%7C%0D%0A%09i%20%3A%3D%20%28macro%20copyFrom%3A%204%20to%3A%20macro%20size%29%20asNumber.%0D%0A%09args%20%3A%3D%20aNode%20arguments.%0D%0A%09%28i%20between%3A%201%20and%3A%20args%20size%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20visit%3A%20%28args%20at%3A%20i%29%0D%0A%09%5D.%0D%0A%09%5Eself%20error%3A%20%27Invalid%20macro%20%5B%27%2Cmacro%2C%27%5D%27%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"basicMacro:in:with:",0
,function(macro,input,aNode){var self=this;
if (macro._eq("lang")){return self.nextPutAll_(self.nativeLanguage());};;
if (macro._eq("receiver")){return self.visitReceiver_(aNode.receiver());};;
if (macro.startsWith_("arg")){return self.visitArgMacro_in_with_(macro,input,aNode);};;
if (macro.startsWith_("seq")){return self.visitSeqMacro_in_with_(macro,input,aNode);};;
if (macro.startsWith_(unescape("%21"))){return (function($rec){$rec.next();return $rec.fileInto_(self);})(macro.stream());};;
if (macro.startsWith_("inlined")){return self.visitInlinedMacro_in_with_(macro,input,aNode);};;
if (smalltalk.responds_to_(self,(macro))){var args=nil;
var count=nil;
count=macro.argumentCount();
if (count._eq((0))){return self.perform_(macro);};;
args=smalltalk.Array.with_with_(aNode,input).copyFrom_to_((1),count.min_((2)));
return self.perform_withArguments_(macro,args);};;
return self.error_(unescape("Missing%20macro%20%5B")._comma(macro)._comma(unescape("%5D")));}
,"macros",unescape("basicMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20aNode%20encoding/expanding%20basic%20macros%20in%20input.%20%22%0D%0A%0D%0A%09macro%20%3D%20%23lang%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20nextPutAll%3A%20self%20nativeLanguage%0D%0A%09%5D.%0D%0A%09macro%20%3D%20%23receiver%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20visitReceiver%3A%20aNode%20receiver%0D%0A%09%5D.%0D%0A%09%28macro%20startsWith%3A%20%23arg%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20visitArgMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%5D.%0D%0A%09%28macro%20startsWith%3A%20%23seq%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20visitSeqMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%5D.%0D%0A%09%28macro%20startsWith%3A%20%24%21%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Emacro%20stream%20next%3B%20fileInto%3A%20self%0D%0A%09%5D.%0D%0A%09%28macro%20startsWith%3A%20%23inlined%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20visitInlinedMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%5D.%0D%0A%09%28self%20respondsTo%3A%20macro%29%20ifTrue%3A%20%5B%20%7C%20args%20count%20%7C%0D%0A%09%09count%20%3A%3D%20macro%20argumentCount.%0D%0A%09%09count%20%3D%200%20ifTrue%3A%20%5B%20%5Eself%20perform%3A%20macro%20%5D.%0D%0A%09%09args%20%3A%3D%20%28Array%20with%3A%20aNode%20with%3A%20input%29%0D%0A%09%09%09copyFrom%3A%201%20to%3A%20%28count%20min%3A%202%29.%0D%0A%09%09%5Eself%20perform%3A%20macro%20withArguments%3A%20args%0D%0A%09%5D.%0D%0A%09%5Eself%20error%3A%20%27Missing%20macro%20%5B%27%20%2Cmacro%20%2C%24%5D%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitUppercaseMacro:in:with:",0
,function(macro,input,aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
if (macro._eq("Lang")){return self.nextPutAll_(self.nativeLanguage().asGlobalName());};;
smalltalk.Smalltalk.at_ifAbsent_(macro,(function(){return (function(){$s8Ret$.result=(function(){return self.basicMacro_in_with_(macro,input,aNode)})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return self.nextPutAll_(self.codeToGetGlobal_(macro))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"optimizing",unescape("visitUppercaseMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%22%20Private%20-%20Expand%20global%20macro%20in%20input.%20%22%0D%0A%0D%0A%09macro%20%3D%20%23Lang%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20nextPutAll%3A%20self%20nativeLanguage%20asGlobalName%0D%0A%09%5D.%0D%0A%09Smalltalk%20at%3A%20macro%20ifAbsent%3A%20%5B%0D%0A%09%09%5Eself%20basicMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%20%0D%0A%09%5D.%0D%0A%09%5Eself%20nextPutAll%3A%20%28self%20codeToGetGlobal%3A%20macro%29%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitMacroSend:in:with:",0
,function(macro,input,aNode){var self=this;
return self.nextPutAll_("smalltalk"._comma(macro));}
,"optimizing",unescape("visitMacroSend%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%22%20Private%20-%20Expand%20root%20macro%20in%20input.%20%22%0D%0A%0D%0A%09%5Eself%20nextPutAll%3A%20%23smalltalk%20%2Cmacro%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitMacro:in:with:",0
,function(macro,input,aNode){var self=this;
if (macro.startsWith_(self.messageSendConnector())){return self.visitMacroSend_in_with_(macro,input,aNode);};;
if (macro.first().isUppercase()){return self.visitUppercaseMacro_in_with_(macro,input,aNode);};;
return self.basicMacro_in_with_(macro,input,aNode);}
,"optimizing",unescape("visitMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20aNode%20encoding/expanding%20macro%20in%20input.%20%22%0D%0A%0D%0A%09%28macro%20startsWith%3A%20self%20messageSendConnector%29%20ifTrue%3A%20%5B%20%22%20%25.msgAPI%25%20%22%0D%0A%09%09%5Eself%20visitMacroSend%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%5D.%0D%0A%09macro%20first%20isUppercase%20ifTrue%3A%20%5B%20%22%20%25GlobalName%25%20%22%0D%0A%09%09%5Eself%20visitUppercaseMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%20%0D%0A%09%5D.%0D%0A%09%5Eself%20basicMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitMacroIn:with:",0
,function(input,aNode){var self=this;
var macro=nil;
macro=input.upTo_(self.macroEnd());
if (macro.isEmpty()){self.emmitCode_(self.macroEnd());} else{self.visitMacro_in_with_(macro,input,aNode);};;
return self;}
,"optimizing",unescape("visitMacroIn%3A%20input%20with%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20next%20macro%20in%20input.%20%22%0D%0A%0D%0A%09%7C%20macro%20%7C%0D%0A%09macro%20%3A%3D%20input%20upTo%3A%20self%20macroEnd.%0D%0A%09macro%20isEmpty%0D%0A%09%09ifTrue%3A%20%5B%20self%20emmitCode%3A%20self%20macroEnd%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20self%20visitMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%20%5D%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitOptimizedNode:as:",0
,function(aNode,mapping){var self=this;
var macro=nil;
macro=mapping.last();
if (mapping.size()._eq((2))){return self.optimize_macro_(aNode,macro);};;
if (mapping.size()._eq((3))){return self.matchOptimizedCondition_with_(mapping.at_((2)),aNode).and_((function(){return self.optimize_macro_(aNode,macro);}));};;
self.note_("Compiler optimization"._comma(unescape("%20for%20%23"))._comma(aNode.selector())._comma(unescape("%20-%20not%20implemented%20"))._comma(mapping.asLiteral()));
return false;}
,"visiting",unescape("visitOptimizedNode%3A%20aNode%20as%3A%20mapping%0D%0A%09%22%20Private%20-%20Generate%20the%20code%20for%20aNode%20using%20custom%20mapping.%0D%0A%09Return%20true%20if%20the%20code%20can%20be%20generated.%0D%0A%09If%20the%20optimization%20fails%2C%20the%20receiver%20remains%20intact.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20macro%20%7C%0D%0A%09macro%20%3A%3D%20mapping%20last.%0D%0A%0D%0A%09mapping%20size%20%3D%202%20ifTrue%3A%20%5B%20%22%28selector%20macro%29%22%0D%0A%09%09%5Eself%20optimize%3A%20aNode%20macro%3A%20macro%0D%0A%09%5D.%0D%0A%09mapping%20size%20%3D%203%20ifTrue%3A%20%5B%22%28selector%20condition%20macro%29%22%0D%0A%09%09%5E%28self%20matchOptimizedCondition%3A%20%28mapping%20at%3A2%29%20with%3A%20aNode%29%0D%0A%09%09%09and%3A%20%5B%20self%20optimize%3A%20aNode%20macro%3A%20macro%20%5D%0D%0A%09%5D.%0D%0A%0D%0A%09self%20note%3A%20%27Compiler%20optimization%27%0D%0A%09%09%2C%27%20for%20%23%27%2CaNode%20selector%0D%0A%09%09%2C%27%20-%20not%20implemented%20%27%0D%0A%09%09%2Cmapping%20asLiteral.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitOptimizedSend:",0
,function(aNode){var self=this;
return self.visitOptimizedSend_with_(aNode,self.optimizationMap());}
,"visiting",unescape("visitOptimizedSend%3A%20aNode%0D%0A%09%22%20Private%20-%20Generate%20optimal%20code%20for%20aNode%20using%20optimization%20map.%0D%0A%09Return%20true%20if%20optimization%20was%20successfull.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20visitOptimizedSend%3A%20aNode%20with%3A%20self%20optimizationMap"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitOptimizedSend:with:",0
,function(aNode,mapping){var $s8Ret$={name:"stReturn"};try{var self=this;
var argCount=nil;
argCount=aNode.arguments().size();
mapping.do_((function(tuple){return ((tuple.first().isNumber().and_((function(){return tuple.first()._eq(argCount);}))).mustBeBoolean()==true ? (function(){return ((self.visitOptimizedSend_with_(aNode,tuple.last())).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();})() : nil);})() : (function(){return ((tuple.first()._eq(aNode.selector())).mustBeBoolean()==true ? (function(){return ((self.visitOptimizedNode_as_(aNode,tuple)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();})() : nil);})() : nil);})());}));
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"visiting",unescape("visitOptimizedSend%3A%20aNode%20with%3A%20mapping%0D%0A%09%22%20Private%20-%20Generate%20optimal%20code%20for%20aNode%20using%20mapping.%0D%0A%09Return%20true%20if%20the%20code%20has%20been%20generated.%0D%0A%09Leave%20the%20receiver%20intact%20if%20the%20optimization%20do%20not%20apply.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20argCount%20%7C%0D%0A%09argCount%20%3A%3D%20aNode%20arguments%20size.%0D%0A%09mapping%20do%3A%20%5B%3Atuple%7C%0D%0A%09%09%28tuple%20first%20isNumber%20and%3A%20%5B%20tuple%20first%20%3D%20argCount%20%5D%29%20ifTrue%3A%20%5B%20%22arguments%20size%20match%22%0D%0A%09%09%09%28self%20visitOptimizedSend%3A%20aNode%20with%3A%20tuple%20last%29%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09tuple%20first%20%3D%20aNode%20selector%20ifTrue%3A%20%5B%20%22selector%20match%22%0D%0A%09%09%09%09%28self%20visitOptimizedNode%3A%20aNode%20as%3A%20tuple%29%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09%09%09%5D.%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitReceiver:",0
,function(aNode){var self=this;
return self.implementedBySubclass_("visitReceiver:");}
,"visiting",unescape("visitReceiver%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20the%20node%20%28as%20receiver%29%20emmitting%20output.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23visitReceiver%3A"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitSendNode:",0
,function(aNode){var self=this;
return self.implementedBySubclass_("visitSendNode:");}
,"visiting",unescape("visitSendNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20the%20node%20%28message%20send%20node%29%20emmitting%20output.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23visitSendNode%3A"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitNode:index:",0
,function(aNode,index){var self=this;
return self.implementedBySubclass_("visitNode:index:");}
,"visiting",unescape("visitNode%3A%20aNode%20index%3A%20index%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20indexed%20access.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23visitNode%3Aindex%3A%20"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitNodeIndices:doing:",0
,function(aNode,aBlock){var self=this;
aBlock.evaluate();
aNode.indices().do_((function(each){return self.visitNode_index_(aNode,each);}));
return self;}
,"visiting",unescape("visitNodeIndices%3A%20aNode%20doing%3A%20aBlock%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20indexed%20access.%20%22%0D%0A%0D%0A%09aBlock%20evaluate.%0D%0A%09aNode%20indices%20do%3A%20%5B%3Aeach%7C%20self%20visitNode%3A%20aNode%20index%3A%20each%20%5D."));
smalltalk.bind(smalltalk.OptimizingEncoder,"missingVariable:",0
,function(aNode){var self=this;
self.compilerEvent_withArguments_ifNotHandled_("missingVariable:in:",smalltalk.Array.with_with_(aNode,self),(function(){var unknown=nil;
unknown=aNode.value();return ((self.$klass.bind_in_(unknown,self)).mustBeBoolean()==true ? nil : (function(){self.unknownVariables().add_(unknown);return self.$klass.unknownVariable_in_(aNode,self);})());}));
return self;}
,"variables",unescape("missingVariable%3A%20aNode%0D%0A%09%22%20Private%20-%20Try%20to%20bind%20to%20access%20to%20variable%20in%20aNode.%20%22%0D%0A%0D%0A%09self%09compilerEvent%3A%20%23missingVariable%3Ain%3A%0D%0A%09%09withArguments%3A%20%28Array%20with%3A%20aNode%20with%3A%20self%29%0D%0A%09%09ifNotHandled%3A%20%5B%20%7C%20unknown%20%7C%0D%0A%09%09%09unknown%20%3A%3D%20aNode%20value.%0D%0A%09%09%09%28self%20class%20bind%3A%20unknown%20in%3A%20self%29%20ifFalse%3A%20%5B%0D%0A%09%09%09%09self%20unknownVariables%20add%3A%20unknown.%0D%0A%09%09%09%09self%20class%20unknownVariable%3A%20aNode%20in%3A%20self.%0D%0A%09%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.OptimizingEncoder,"codeVisitingLocal:",0
,function(aNode){var self=this;
return aNode.localNameIn_(self);}
,"variables",unescape("codeVisitingLocal%3A%20aNode%0D%0A%09%22%20Private%20-%20Returns%20code%20to%20access%20local%20value.%20%22%0D%0A%0D%0A%09%5EaNode%20localNameIn%3A%20self"));
smalltalk.bind(smalltalk.OptimizingEncoder,"codeVisitingVariableNode:",0
,function(aNode){var self=this;
var code=nil;
code=self.currentClass().codeFor_accessing_(self,aNode);
if ((nil.isNil_(code)).and_((function(){return aNode.needsVerification();}))){((self.knownVariables().includes_(aNode.value())).mustBeBoolean()==true ? nil : (function(){return self.missingVariable_(aNode);})());};;
if ((nil.isNil_(code)).or_((function(){return self.knownVariables().includes_(aNode.value());}))){code=self.codeVisitingLocal_(aNode);};;
return code;}
,"variables",unescape("codeVisitingVariableNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Returns%20code%20to%20access%20variable%20value.%20%22%0D%0A%0D%0A%09%7C%20code%20%7C%0D%0A%09code%20%3A%3D%20self%20currentClass%20codeFor%3A%20self%20accessing%3A%20aNode.%0D%0A%09%28code%20isNil%20and%3A%20%5BaNode%20needsVerification%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%28self%20knownVariables%20includes%3A%20aNode%20value%29%20ifFalse%3A%20%5B%0D%0A%09%09%09self%20missingVariable%3A%20aNode%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%28code%20isNil%20or%3A%20%5Bself%20knownVariables%20includes%3A%20aNode%20value%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09code%20%3A%3D%20self%20codeVisitingLocal%3A%20aNode%0D%0A%09%5D.%0D%0A%09%5Ecode"));
smalltalk.bind(smalltalk.OptimizingEncoder,"visitVariableNode:",0
,function(aNode){var self=this;
self.visitNodeIndices_doing_(aNode,(function(){return self.emmitCode_(self.codeVisitingVariableNode_(aNode));}));
return self;}
,"variables",unescape("visitVariableNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20access%20variable%20aNode.%20%22%0D%0A%0D%0A%09self%20visitNodeIndices%3A%20aNode%20doing%3A%20%5B%0D%0A%09%09self%20emmitCode%3A%20%28self%20codeVisitingVariableNode%3A%20aNode%29%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeEncoder,"indentation",0
,function(){var self=this;
if ((nil.isNil_(self["@indentation"]))){self["@indentation"]=(0);};;
return self["@indentation"];}
,"indentation",unescape("indentation%0D%0A%09%22%20Private%20-%20Return%20the%20indentation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09indentation%20isNil%20ifTrue%3A%20%5B%20indentation%20%3A%3D%200%20%5D.%0D%0A%09%5Eindentation%20"));
smalltalk.bind(smalltalk.NativeEncoder,"indent",0
,function(){var self=this;
self["@indentation"]=self.indentation()._plus((1));
return self;}
,"indentation",unescape("indent%0D%0A%09%22%20Private%20-%20Enter%20the%20next%20level%20of%20indentation.%20%22%0D%0A%0D%0A%09indentation%20%3A%3D%20self%20indentation%20+%201"));
smalltalk.bind(smalltalk.NativeEncoder,"dedent",0
,function(){var self=this;
self["@indentation"]=self.indentation()._minus((1)).max_((0));
return self;}
,"indentation",unescape("dedent%0D%0A%09%22%20Private%20-%20Exit%20the%20current%20level%20of%20indentation.%20%22%0D%0A%0D%0A%09indentation%20%3A%3D%20self%20indentation%20-%201%20max%3A%200"));
smalltalk.bind(smalltalk.NativeEncoder,"cr",0
,function(){var self=this;
self["@stream"].cr();
(function(){for(var $1$=(self.indentation());$1$>0;$1$--){(function(){return self.tab();})()};return nil})();
return self;}
,"writing",unescape("cr%0D%0A%09%22%20Private%20-%20Emmit%20a%20new%20line%20onto%20output%20stream.%20%22%0D%0A%0D%0A%09stream%20cr.%0D%0A%09self%20indentation%20timesRepeat%3A%20%5B%20self%20tab%20%5D"));
smalltalk.bind(smalltalk.NativeEncoder,"tab",0
,function(){var self=this;
self["@stream"].tab();
return self;}
,"writing",unescape("tab%0D%0A%09%22%20Private%20-%20Emmit%20a%20tab%20onto%20output%20stream.%20%22%0D%0A%0D%0A%09stream%20tab"));
smalltalk.bind(smalltalk.NativeEncoder,"space",0
,function(){var self=this;
self["@stream"].space();
return self;}
,"writing",unescape("space%0D%0A%09%22%20Private%20-%20Emmit%20a%20space%20onto%20output%20stream.%20%22%0D%0A%0D%0A%09stream%20space"));
smalltalk.bind(smalltalk.NativeEncoder,"nextPutAll:",0
,function(aString){var self=this;
self["@stream"].nextPutAll_(aString);
return self;}
,"writing",unescape("nextPutAll%3A%20aString%0D%0A%09%22%20Private%20-%20Emmit%20aString%20onto%20output%20stream.%20%22%0D%0A%0D%0A%09stream%20nextPutAll%3A%20aString"));
smalltalk.bind(smalltalk.NativeEncoder,"assignmentOp",0
,function(){var self=this;
return unescape("%3D");}
,"constants",unescape("assignmentOp%0D%0A%09%22%20Private%20-%20Return%20the%20OP%20for%20encoding%20assignment.%20%22%0D%0A%0D%0A%09%5E%24%3D%20"));
smalltalk.bind(smalltalk.NativeEncoder,"separatorOp",0
,function(){var self=this;
return unescape("%3B");}
,"constants",unescape("separatorOp%0D%0A%09%22%20Private%20-%20Return%20the%20OP%20for%20separating%20statements.%20%22%0D%0A%0D%0A%09%5E%24%3B%20"));
smalltalk.bind(smalltalk.NativeEncoder,"returnOpStart",0
,function(){var self=this;
return "return ";}
,"constants",unescape("returnOpStart%0D%0A%09%22%20Private%20-%20Return%20the%20OP%20for%20starting%20return%20expressions.%20%22%0D%0A%0D%0A%09%5E%27return%20%27%20"));
smalltalk.bind(smalltalk.NativeEncoder,"returnOpEnd",0
,function(){var self=this;
return nil;}
,"constants",unescape("returnOpEnd%0D%0A%09%22%20Private%20-%20Return%20the%20OP%20for%20ending%20return%20expressions.%20%22%0D%0A%0D%0A%09%5Enil%20"));
smalltalk.bind(smalltalk.NativeEncoder,"returnOps",0
,function(){var self=this;
return smalltalk.Array.with_with_(self.returnOpStart(),self.returnOpEnd());}
,"constants",unescape("returnOps%0D%0A%09%22%20Private%20-%20Return%20the%20OPs%20for%20return%20expression.%20%22%0D%0A%0D%0A%09%5EArray%09with%3A%20self%20returnOpStart%0D%0A%09%09with%3A%20self%20returnOpEnd%20"));
smalltalk.bind(smalltalk.NativeEncoder,"between:do:",0
,function(separators,aBlock){var self=this;
if ((nil.isNil_(separators.first())===false)){self["@stream"].nextPutAll_(separators.first());};;
aBlock.evaluate();
if ((nil.isNil_(separators.last())===false)){self["@stream"].nextPutAll_(separators.last());};;
return self;}
,"writing",unescape("between%3A%20separators%20do%3A%20aBlock%0D%0A%09%22%20Private%20-%20Emmit%20aBlock%20between%20separators%20onto%20output%20stream.%20%22%0D%0A%0D%0A%09separators%20first%20notNil%20ifTrue%3A%20%5B%20stream%20nextPutAll%3A%20separators%20first%20%5D.%0D%0A%09aBlock%20evaluate.%0D%0A%09separators%20last%20notNil%20ifTrue%3A%20%5B%20stream%20nextPutAll%3A%20separators%20last%20%5D%20"));
smalltalk.bind(smalltalk.NativeEncoder,"nextArgumentsPut:",0
,function(aBlock){var self=this;
self.between_do_(unescape("%28%29"),aBlock);
return self;}
,"writing",unescape("nextArgumentsPut%3A%20aBlock%0D%0A%09%22%20Private%20-%20Emmit%20arguments%20list%20evaluating%20aBlock%20onto%20output%20stream.%20%22%0D%0A%0D%0A%09self%20between%3A%20%27%28%29%27%20do%3A%20aBlock%20"));
smalltalk.bind(smalltalk.NativeEncoder,"nextIndexPut:",0
,function(aBlock){var self=this;
self.between_do_(unescape("%5B%5D"),aBlock);
return self;}
,"writing",unescape("nextIndexPut%3A%20aBlock%0D%0A%09%22%20Private%20-%20Emmit%20a%20series%20between%20indexing%20notation%20evaluating%20aBlock%20onto%20output%20stream.%20%22%0D%0A%0D%0A%09self%20between%3A%20%27%5B%5D%27%20do%3A%20aBlock%20"));
smalltalk.bind(smalltalk.NativeEncoder,"nextBlockPut:",0
,function(aBlock){var self=this;
(function($rec){$rec.indent();return $rec.cr();})(self);
aBlock.evaluate();
(function($rec){$rec.dedent();return $rec.cr();})(self);
return self;}
,"writing",unescape("nextBlockPut%3A%20aBlock%0D%0A%09%22%20Private%20-%20Evaluates%20aBlock%20on%20the%20next%20level%20of%20indentation.%20%22%0D%0A%0D%0A%09self%20indent%3Bcr.%0D%0A%09aBlock%20evaluate.%0D%0A%09self%20dedent%3Bcr%20"));
smalltalk.bind(smalltalk.NativeEncoder,"nextReturn:put:",0
,function(early,aBlock){var self=this;
self.between_do_(self.returnOps(),aBlock);
return self;}
,"writing",unescape("nextReturn%3A%20early%20put%3A%20aBlock%0D%0A%09%22%20Private%20-%20Emmit%20a%20return%20OP%20evaluating%20aBlock%20onto%20output%20stream.%20%22%0D%0A%0D%0A%09self%20between%3A%20self%20returnOps%20do%3A%20aBlock%20"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitCode:",0
,function(code){var self=this;
self.nextPutAll_(code);
return self;}
,"generation",unescape("emmitCode%3A%20code%0D%0A%09%22%20Private%20-%20Emmit%20code%20onto%20output%20stream.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20code"));
smalltalk.bind(smalltalk.NativeEncoder,"codeToAccessProperty:in:",0
,function(aName,aNode){var self=this;
return unescape("self%5B%22")._comma(aName)._comma(unescape("%22%5D"));}
,"encoding",unescape("codeToAccessProperty%3A%20aName%20in%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20the%20source%20code%20to%20access%20property%20aName.%20%22%0D%0A%0D%0A%09%5E%27self%5B%22%27%2CaName%2C%27%22%5D%27"));
smalltalk.bind(smalltalk.NativeEncoder,"codeToAccessInstanceVariable:in:",0
,function(aName,aNode){var self=this;
return self.codeToAccessProperty_in_(unescape("@")._comma(aName),aNode);}
,"encoding",unescape("codeToAccessInstanceVariable%3A%20aName%20in%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20the%20source%20code%20to%20access%20instance%20variable%20aName.%20%22%0D%0A%0D%0A%09%5Eself%20codeToAccessProperty%3A%20%27@%27%2C%20aName%20in%3A%20aNode"));
smalltalk.bind(smalltalk.NativeEncoder,"defaultRequirementsFor:",0
,function(aClass){var self=this;
return [];}
,"export",unescape("defaultRequirementsFor%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20default%20%28collection%20of%29%20requirements%20to%20load%20aClass.%0D%0A%09Return%20an%20empty%20collection.%0D%0A%09This%20method%20can%20be%20refined%20by%20subclasses.%0D%0A%09%22%0D%0A%0D%0A%09%5E%23%28%29%20"));
smalltalk.bind(smalltalk.NativeEncoder,"acceptAssignment:",0
,function(aNode){var self=this;
return self.compilerEvent_with_ifNotHandled_("acceptAssignmentTo:",aNode.left(),(function(){return true;}));}
,"assignment",unescape("acceptAssignment%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20assignment%20aNode%20can%20be%20accepted.%20%22%0D%0A%0D%0A%09%5Eself%09compilerEvent%3A%20%23acceptAssignmentTo%3A%0D%0A%09%09with%3A%20aNode%20left%0D%0A%09%09ifNotHandled%3A%20%5B%20true%20%5D"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitFunctionStart:",0
,function(fName){var self=this;
self.nextPutAll_("function");
return self;}
,"generation",unescape("emmitFunctionStart%3A%20fName%0D%0A%09%22%20Private%20-%20Emmit%20the%20code%20of%20function%20start%20definition.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20%23function%20"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitFunctionArguments:",0
,function(argList){var self=this;
self.nextArgumentsPut_((function(){return self.nextPutAll_(argList);}));
return self;}
,"generation",unescape("emmitFunctionArguments%3A%20argList%0D%0A%09%22%20Private%20-%20Emmit%20the%20code%20of%20function%20arguments.%20%22%0D%0A%0D%0A%09self%20nextArgumentsPut%3A%20%5B%0D%0A%09%09self%20nextPutAll%3A%20argList%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitFunctionBody:",0
,function(aBlock){var self=this;
return self.implementedBySubclass_("emmitFunctionBody:");}
,"generation",unescape("emmitFunctionBody%3A%20aBlock%0D%0A%09%22%20Private%20-%20Emmit%20the%20code%20of%20function%20body%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23emmitFunctionBody%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitFunction:arguments:doing:",0
,function(fName,argList,aBlock){var self=this;
self.compilerEvent_withArguments_ifNotHandled_("startFunction:with:",smalltalk.Array.with_with_(fName,argList),(function(){self.emmitFunctionStart_(fName);self.compilerEvent_with_("startFunctionName:",fName);return self.emmitFunctionArguments_(argList);}));
self.emmitFunctionBody_((function(){self.compilerEvent_withArguments_("startFunctionBody:with:",smalltalk.Array.with_with_(fName,argList));return aBlock.evaluate();}));
self.compilerEvent_with_("endFunction:",fName);
return self;}
,"generation",unescape("emmitFunction%3A%20fName%20arguments%3A%20argList%20doing%3A%20aBlock%0D%0A%09%22%20Private%20-%20Emmit%20function%20on%20receiver%27s%20code%20stream.%20%22%0D%0A%0D%0A%09self%0D%0A%09%09compilerEvent%3A%20%23startFunction%3Awith%3A%0D%0A%09%09withArguments%3A%20%28Array%20with%3A%20fName%20with%3A%20argList%29%0D%0A%09%09ifNotHandled%3A%20%5B%0D%0A%09%09%09self%20emmitFunctionStart%3A%20fName.%0D%0A%09%09%09self%20compilerEvent%3A%20%23startFunctionName%3A%20with%3A%20fName.%0D%0A%09%09%09self%20emmitFunctionArguments%3A%20argList%0D%0A%09%09%5D.%0D%0A%09self%20emmitFunctionBody%3A%20%5B%0D%0A%09%09self%09compilerEvent%3A%20%23startFunctionBody%3Awith%3A%0D%0A%09%09%09withArguments%3A%20%28Array%20with%3A%20fName%20with%3A%20argList%29.%0D%0A%09%09aBlock%20evaluate%0D%0A%09%5D.%0D%0A%09self%20compilerEvent%3A%20%23endFunction%3A%20with%3A%20fName."));
smalltalk.bind(smalltalk.NativeEncoder,"emmitFunction:node:doing:",0
,function(fName,aNode,aBlock){var self=this;
return self.emmitFunction_node_arguments_doing_(fName,aNode,aNode.argumentNames(),aBlock);}
,"generation",unescape("emmitFunction%3A%20fName%20node%3A%20aNode%20doing%3A%20aBlock%0D%0A%09%22%20Private%20-%20Generate%20function%20for%20aNode.%20%22%0D%0A%0D%0A%09%5Eself%09emmitFunction%3A%20fName%0D%0A%09%09node%3A%20aNode%20arguments%3A%20aNode%20argumentNames%0D%0A%09%09doing%3A%20aBlock"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitFunctionDoing:",0
,function(aBlock){var self=this;
return self.emmitFunction_arguments_doing_("","",aBlock);}
,"generation",unescape("emmitFunctionDoing%3A%20aBlock%0D%0A%09%22%20Private%20-%20Emmit%20function%20on%20receiver%27s%20code%20stream.%20%22%0D%0A%0D%0A%09%5Eself%20emmitFunction%3A%20%27%27%20arguments%3A%20%27%27%20doing%3A%20aBlock"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitReturn:early:",0
,function(aBlockOrString,early){var self=this;
self.compilerEvent_with_("aboutReturn:",aBlockOrString);
self.nextReturn_put_(early,(function(){self.compilerEvent_with_("return:",aBlockOrString);return ((aBlockOrString.isString()).mustBeBoolean()==true ? (function(){return self.nextPutAll_(aBlockOrString);})() : (function(){return aBlockOrString.evaluate();})());}));
if ((nil.isNil_(aBlockOrString)===false)){self.compilerEvent_withArguments_("returned:early:",smalltalk.Array.with_with_(aBlockOrString,early));};;
return self;}
,"generation",unescape("emmitReturn%3A%20aBlockOrString%20early%3A%20early%0D%0A%09%22%20Private%20-%20Generate%20return%20code.%20%22%0D%0A%0D%0A%09self%20compilerEvent%3A%20%23aboutReturn%3A%20with%3A%20aBlockOrString.%0D%0A%09self%20nextReturn%3A%20early%20put%3A%20%5B%0D%0A%09%09self%20compilerEvent%3A%20%23return%3A%20with%3A%20aBlockOrString.%0D%0A%09%09aBlockOrString%20isString%0D%0A%09%09%09ifTrue%3A%20%5B%20self%20nextPutAll%3A%20aBlockOrString%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20aBlockOrString%20evaluate%20%5D%0D%0A%09%5D.%0D%0A%09aBlockOrString%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09self%09compilerEvent%3A%20%23returned%3Aearly%3A%0D%0A%09%09%09withArguments%3A%20%28Array%20with%3A%20aBlockOrString%20with%3A%20early%29.%0D%0A%09%5D."));
smalltalk.bind(smalltalk.NativeEncoder,"emmitReturn:",0
,function(aBlockOrString){var self=this;
return self.emmitReturn_early_(aBlockOrString,false);}
,"generation",unescape("emmitReturn%3A%20aBlockOrString%0D%0A%09%22%20Private%20-%20Generate%20return%20code.%20%22%0D%0A%0D%0A%09%5Eself%20emmitReturn%3A%20aBlockOrString%20early%3A%20false"));
smalltalk.bind(smalltalk.NativeEncoder,"defaultOptimizations",0
,function(){var self=this;
return ["s8"];}
,"defaults",unescape("defaultOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20default%20optimizations%20of%20the%20receiver.%0D%0A%09Note%20that%20returned%20optimizations%20MUST%20be%20provided%20by%20the%20subclasses.%20%22%0D%0A%0D%0A%09%5E%23%28%20s8%20%22magnitude%20math%22%20%29"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitedCode",0
,function(){var self=this;
return (((nil.isNil_(self["@stream"])===false)).mustBeBoolean()==true ? (function(){return self["@stream"].contents();})() : nil);}
,"results",unescape("emmitedCode%0D%0A%09%22%20Return%20the%20emmited%20code%20during%20compilation%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Estream%20notNil%20ifTrue%3A%20%5B%20stream%20contents%20%5D"));
smalltalk.bind(smalltalk.NativeEncoder,"arrayLiteralStart",0
,function(){var self=this;
return unescape("%5B");}
,"literals",unescape("arrayLiteralStart%0D%0A%09%22%20Private%20-%20Return%20the%20code%20string%20to%20start%20exporting%20an%20array%20literal.%20%22%0D%0A%0D%0A%09%5E%24%5B"));
smalltalk.bind(smalltalk.NativeEncoder,"arrayLiteralEnd",0
,function(){var self=this;
return unescape("%5D");}
,"literals",unescape("arrayLiteralEnd%0D%0A%09%22%20Private%20-%20Return%20the%20code%20string%20to%20end%20exporting%20an%20array%20literal.%20%22%0D%0A%0D%0A%09%5E%24%5D"));
smalltalk.bind(smalltalk.NativeEncoder,"escaped:",0
,function(string){var self=this;
return string.escapedCode();}
,"literals",unescape("escaped%3A%20string%0D%0A%09%22%20Private%20-%20Return%20the%20code%20string%20to%20export%20string%20literal.%20%22%0D%0A%0D%0A%09%5Estring%20escapedCode"));
smalltalk.bind(smalltalk.NativeEncoder,"nativeSourceFor:",0
,function(aMethod){var self=this;
return aMethod.compiledSource();}
,"private",unescape("nativeSourceFor%3A%20aMethod%0D%0A%09%22%20Private%20-%20Return%20the%20code%20of%20native%20function%20implementing%20aMethod.%20%22%0D%0A%0D%0A%09%5EaMethod%20compiledSource"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitMethodContents:",0
,function(aNode){var self=this;
var code=nil;
var delta=nil;
code=self.generatedCodeFor_while_(aNode,(function(){self.compilerEvent_withArguments_ifNotHandled_("enterMethod:",smalltalk.Array.with_(aNode),(function(){return self.emmitEnterMethod_(aNode);}));return aNode.nodes().do_((function(each){return self.visit_(each);}));}));
delta=self.generate_early_return_(self["@functionName"],self["@earlyReturn"],code);
self.translate_by_(aNode,delta);
return self;}
,"generation",unescape("emmitMethodContents%3A%20aNode%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20contents%20of%20method%20aNode.%20%22%0D%0A%0D%0A%09%7C%20code%20delta%20%7C%0D%0A%09code%20%3A%3D%20self%20generatedCodeFor%3A%20aNode%20while%3A%20%5B%0D%0A%09%09self%20compilerEvent%3A%20%23enterMethod%3A%20withArguments%3A%20%28Array%20with%3A%20aNode%29%0D%0A%09%09%09ifNotHandled%3A%20%5B%20self%20emmitEnterMethod%3A%20aNode%20%5D.%0D%0A%09%09aNode%20nodes%20do%3A%20%5B%3Aeach%20%7C%20self%20visit%3A%20each%20%5D.%0D%0A%09%5D.%0D%0A%09delta%20%3A%3D%20self%20generate%3A%20functionName%20early%3A%20earlyReturn%20return%3A%20code.%0D%0A%09self%20translate%3A%20aNode%20by%3A%20delta."));
smalltalk.bind(smalltalk.NativeEncoder,"emmitMethod:",0
,function(aNode){var self=this;
self["@nestedBlocks"]=(0);
self["@earlyReturn"]=false;
self.compiler().resetVariables();
self["@functionName"]=self.asSelector_(aNode.selector());
aNode.validateArguments();
self.emmitFunction_node_doing_(self["@functionName"],aNode,(function(){self.compilerEvent_with_("startMethod:",aNode);self.emmitMethodContents_(aNode);return self.compilerEvent_with_("endMethod:",aNode);}));
return self;}
,"generation",unescape("emmitMethod%3A%20aNode%0D%0A%09%22%20Private%20-%20Generate%20code%20of%20method%20aNode.%20%22%0D%0A%0D%0A%09nestedBlocks%20%3A%3D%200.%0D%0A%09earlyReturn%20%3A%3D%20false.%0D%0A%09self%20compiler%20resetVariables.%0D%0A%0D%0A%09functionName%20%3A%3D%20self%20asSelector%3A%20aNode%20selector.%0D%0A%09aNode%20validateArguments.%0D%0A%09self%20emmitFunction%3A%20functionName%20node%3A%20aNode%20doing%3A%20%5B%0D%0A%09%09self%20compilerEvent%3A%20%23startMethod%3A%20with%3A%20aNode.%0D%0A%09%09self%20emmitMethodContents%3A%20aNode.%0D%0A%09%09self%20compilerEvent%3A%20%23endMethod%3A%20with%3A%20aNode.%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeEncoder,"saveState",0
,function(){var self=this;
return smalltalk.Array.with_with_with_(self["@stream"],self["@startOffset"],self.indentation());}
,"state",unescape("saveState%0D%0A%09%22%20Private%20-%20Return%20the%20state%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EArray%09with%3A%20stream%0D%0A%09%09with%3A%20startOffset%0D%0A%09%09with%3A%20self%20indentation%20"));
smalltalk.bind(smalltalk.NativeEncoder,"restoreState:",0
,function(array){var self=this;
self["@stream"]=array.first();
self["@startOffset"]=array.second();
self["@indentation"]=array.third();
return self;}
,"state",unescape("restoreState%3A%20array%0D%0A%09%22%20Private%20-%20Restore%20the%20state%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09stream%20%3A%3D%20array%20first.%0D%0A%09startOffset%20%3A%3D%20array%20second.%0D%0A%09indentation%20%3A%3D%20array%20third.%20"));
smalltalk.bind(smalltalk.NativeEncoder,"generatedCodeFor:while:",0
,function(aNode,aBlock){var self=this;
var backup=nil;
var result=nil;
backup=self.saveState();
self["@startOffset"]=self["@stream"].position();
self["@stream"]="".writeStream();
self["@indentation"]=(0);
self.visit_doing_(aNode,aBlock);
result=self["@stream"].contents();
self.restoreState_(backup);
return result;}
,"generation",unescape("generatedCodeFor%3A%20aNode%20while%3A%20aBlock%0D%0A%09%22%20Private%20-%20Return%20the%20code%20generated%20during%20evaluation%20of%20aBlock.%20%22%0D%0A%0D%0A%09%7C%20backup%20result%20%7C%0D%0A%09backup%20%3A%3D%20self%20saveState.%0D%0A%09startOffset%20%3A%3D%20stream%20position.%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09indentation%20%3A%3D%200.%0D%0A%09self%20visit%3A%20aNode%20doing%3A%20aBlock.%0D%0A%09result%20%3A%3D%20stream%20contents.%0D%0A%09self%20restoreState%3A%20backup.%0D%0A%09%5Eresult%20"));
smalltalk.bind(smalltalk.NativeEncoder,"includeSourceCode",0
,function(){var self=this;
return true;}
,"generation",unescape("includeSourceCode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20source%20code%20is%20included%20as%20method%20argument.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.NativeEncoder,"isOperation:",0
,function(aNode){var self=this;
return aNode.arguments().size()._eq((1)).and_((function(){return aNode.selector().size()._gt((1)).and_((function(){return aNode.selector().first()._eq(unescape("%23")).and_((function(){return unescape("%7C%26%5C%5C+*/%3D%3E%3C%2C@%25%7E-").includes_(aNode.selector().at_((2)));}));}));}));}
,"operations",unescape("isOperation%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20aNode%20is%20an%20operation.%20%22%0D%0A%0D%0A%09%5EaNode%20arguments%20size%20%3D%201%20and%3A%20%5B%0D%0A%09%09aNode%20selector%20size%20%3E%201%20and%3A%20%5B%0D%0A%09%09aNode%20selector%20first%20%3D%20%24%23%20and%3A%20%5B%0D%0A%09%09%27%7C%26%5C%5C+*/%3D%3E%3C%2C@%25%7E-%27%20includes%3A%20%28aNode%20selector%20at%3A%202%29%0D%0A%09%09%5D%5D%5D"));
smalltalk.bind(smalltalk.NativeEncoder,"loadRequirementsOf:",0
,function(aClass){var self=this;
var requirements=nil;
requirements=self.defaultRequirementsFor_(aClass);
aClass.loadRequirementsInto_for_(requirements,self);
return requirements;}
,"export",unescape("loadRequirementsOf%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20requirements%20to%20load%20aClass.%0D%0A%09It%20must%20be%20a%20collection%20of%20requirements.%0D%0A%09aRequirement%20must%20implement%20%23fileOutOn%3A%20or%20must%20be%20aString%28native%20code%29%20to%20execute%20just%20before%20defining%20aClass.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20requirements%20%7C%0D%0A%09requirements%20%3A%3D%20self%20defaultRequirementsFor%3A%20aClass.%0D%0A%09aClass%20loadRequirementsInto%3A%20requirements%20for%3A%20self.%0D%0A%09%5Erequirements"));
smalltalk.bind(smalltalk.NativeEncoder,"position",0
,function(){var self=this;
return self["@stream"].position()._plus(self["@startOffset"]);}
,"accessing",unescape("position%0D%0A%09%22%20Return%20the%20current%20position%20in%20output%20stream.%20%22%0D%0A%0D%0A%09%5Estream%20position%20+%20startOffset"));
smalltalk.bind(smalltalk.NativeEncoder,"startCompiling:",0
,function(aNode){var self=this;
smalltalk.superImplementor(smalltalk.NativeEncoder,"startCompiling_").apply(self, [aNode]);
self["@stream"]="".writeStream();
self["@startOffset"]=self["@stream"].position();
self["@indentation"]=nil;
return self;}
,"private",unescape("startCompiling%3A%20aNode%0D%0A%09%22%20Private%20-%20Set%20the%20receiver%20to%20start%20emmiting%20code%20for%20aNode.%20%22%0D%0A%0D%0A%09super%20startCompiling%3A%20aNode.%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09startOffset%20%3A%3D%20stream%20position.%0D%0A%09indentation%20%3A%3D%20nil.%20"));
smalltalk.bind(smalltalk.NativeEncoder,"stream",0
,function(){var self=this;
return self["@stream"];}
,"accessing",unescape("stream%0D%0A%09%22%20Return%20the%20current%20stream%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Estream"));
smalltalk.bind(smalltalk.NativeEncoder,"visit:",0
,function(aNode){var self=this;
return self.visit_doing_(aNode,(function(){return smalltalk.superImplementor(smalltalk.NativeEncoder,"visit_").apply(self, [aNode]);}));}
,"visiting",unescape("visit%3A%20aNode%0D%0A%09%5Eself%20visit%3A%20aNode%20doing%3A%20%5B%20super%20visit%3A%20aNode%20%5D"));
smalltalk.bind(smalltalk.NativeEncoder,"visit:doing:",0
,function(aNode,aBlock){var self=this;
var cookie=nil;
var result=nil;
cookie=self.compilerEvent_with_ifNotHandled_("needsCookie:",aNode,(function(){return nil;}));
if ((nil.isNil_(cookie)===false)){self.compilerEvent_with_("startVisit:",cookie);};;
result=aBlock.evaluate();
if ((nil.isNil_(cookie)===false)){self.compilerEvent_with_("endVisit:",cookie);};;
return result;}
,"visiting",unescape("visit%3A%20aNode%20doing%3A%20aBlock%0D%0A%09%7C%20cookie%20result%20%7C%0D%0A%09cookie%20%3A%3D%20self%0D%0A%09%09compilerEvent%3A%20%23needsCookie%3A%20with%3A%20aNode%0D%0A%09%09ifNotHandled%3A%20%5B%20nil%20%5D.%0D%0A%09cookie%20notNil%20ifTrue%3A%20%5B%20self%20compilerEvent%3A%20%23startVisit%3A%20with%3A%20cookie%20%5D.%0D%0A%09result%20%3A%3D%20aBlock%20evaluate.%0D%0A%09cookie%20notNil%20ifTrue%3A%20%5B%20self%20compilerEvent%3A%20%23endVisit%3A%20with%3A%20cookie%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeEncoder,"visitArgument:",0
,function(aNode){var self=this;
if (aNode.isAssignment()){return self.visitLocalAssignment_(aNode);};;
return self.visit_(aNode);}
,"visiting",unescape("visitArgument%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20the%20argument%20emmitting%20output.%20%22%0D%0A%0D%0A%09aNode%20isAssignment%20ifTrue%3A%20%5B%20%5Eself%20visitLocalAssignment%3A%20aNode%20%5D.%0D%0A%09%5Eself%20visit%3A%20aNode"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitAssignment:",0
,function(aNode){var self=this;
(function($rec){$rec.visit_(aNode.left());$rec.nextPutAll_(self.assignmentOp());return $rec.visit_(aNode.right());})(self);
return self;}
,"generation",unescape("emmitAssignment%3A%20aNode%0D%0A%09%22%20Private%20-%20Emmit%20the%20code%20for%20assignment.%20%22%0D%0A%0D%0A%09self%0D%0A%09%09visit%3A%20aNode%20left%3B%0D%0A%09%09nextPutAll%3A%20self%20assignmentOp%3B%0D%0A%09%09visit%3A%20aNode%20right"));
smalltalk.bind(smalltalk.NativeEncoder,"visitAssignmentNode:",0
,function(aNode){var self=this;
if (self.visitMacroAssignment_(aNode)){return self;};;
if (self.acceptAssignment_(aNode)){self.emmitAssignment_(aNode);} else{self.compilerEvent_with_ifNotHandled_("invalidAssignment:",aNode,(function(){return self.error_("Invalid assignment to "._comma(aNode.left().value()));}));};;
return self;}
,"visiting",unescape("visitAssignmentNode%3A%20aNode%0D%0A%0D%0A%09%28self%20visitMacroAssignment%3A%20aNode%29%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09%28self%20acceptAssignment%3A%20aNode%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20emmitAssignment%3A%20aNode%0D%0A%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09self%09compilerEvent%3A%20%23invalidAssignment%3A%20with%3A%20aNode%0D%0A%09%09%09ifNotHandled%3A%20%5B%20self%20error%3A%20%27Invalid%20assignment%20to%20%27%20%2CaNode%20left%20value%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeEncoder,"visitNode:in:cr:",0
,function(aNode,aParentNode,withCr){var self=this;
if ((nil.isNil_(self.visit_(aNode))===false)){self.nextPutAll_(self.separatorOp());
((withCr).mustBeBoolean()==true ? (function(){return self.cr();})() : nil);};;
return self;}
,"visiting",unescape("visitNode%3A%20aNode%20in%3A%20aParentNode%20cr%3A%20withCr%0D%0A%09%22%20Private%20-%20Visit%20aNode%20as%20part%20of%20sequence%20in%20aParentNode.%20%22%0D%0A%0D%0A%09%28self%20visit%3A%20aNode%29%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09self%20nextPutAll%3A%20self%20separatorOp.%0D%0A%09%09withCr%20ifTrue%3A%20%5B%20self%20cr%20%5D.%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeEncoder,"visitEmptyNode:",0
,function(aNode){var self=this;
return self;}
,"visiting",unescape("visitEmptyNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20an%20empty%20node%2C%20aNode.%20%22%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitNodesIn:cr:",0
,function(aNode,withCr){var self=this;
var array=nil;
array=aNode.nodes();
if (array.isEmpty()){return self.visitEmptyNode_(aNode);};;
array.do_separatedBy_((function(one){return self.visitNode_in_cr_(one,aNode,false);}),(function(){return ((withCr).mustBeBoolean()==true ? (function(){return self.cr();})() : nil);}));
return self;}
,"visiting",unescape("visitNodesIn%3A%20aNode%20cr%3A%20withCr%0D%0A%09%22%20Private%20-%20Visit%20the%20nodes%20contained%20in%20aNode.%20%22%0D%0A%0D%0A%09%7C%20array%20%7C%0D%0A%09array%20%3A%3D%20aNode%20nodes.%0D%0A%09array%20isEmpty%20ifTrue%3A%20%5B%20%5Eself%20visitEmptyNode%3A%20aNode%20%5D.%0D%0A%09array%20do%3A%20%5B%3Aone%7C%0D%0A%09%09self%20visitNode%3A%20one%20in%3A%20aNode%20cr%3A%20false%0D%0A%09%5D%20separatedBy%3A%20%5B%20withCr%20ifTrue%3A%20%5B%20self%20cr%20%5D%20%5D"));
smalltalk.bind(smalltalk.NativeEncoder,"visitArgMacro:in:with:",0
,function(macro,input,aNode){var self=this;
if (macro._eq("argList")){return self.visitArguments_(aNode.arguments());};;
return smalltalk.superImplementor(smalltalk.NativeEncoder,"visitArgMacro_in_with_").apply(self, [macro,input,aNode]);}
,"optimizing",unescape("visitArgMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20argument%20macro%20in%20input.%20%22%0D%0A%0D%0A%09macro%20%3D%20%23argList%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20visitArguments%3A%20aNode%20arguments%0D%0A%09%5D.%0D%0A%09%5Esuper%20visitArgMacro%3A%20macro%20in%3A%20input%20with%3A%20aNode"));
smalltalk.bind(smalltalk.NativeEncoder,"defineTemporary:",0
,function(aName){var self=this;
(function($rec){$rec.nextPutAll_("var "._comma(self.localName_(aName))._comma(unescape("%3Dnil%3B")));return $rec.cr();})(self);
return self;}
,"encoding",unescape("defineTemporary%3A%20aName%0D%0A%09%22%20Private%20-%20Return%20the%20code%20for%20defining%20temporary%20variable%20with%20aName.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20%27var%20%27%20%2C%28self%20localName%3A%20aName%29%20%2C%27%3Dnil%3B%27%20%3Bcr%20"));
smalltalk.bind(smalltalk.NativeEncoder,"withNodesIn:do:atEnd:",0
,function(aNode,aBlock,endBlock){var self=this;
var array=nil;
array=aNode.nodes().asArray();
(1).to_do_(array.size()._minus((1)),(function(index){return aBlock.evaluateWith_(array.at_(index));}));
return endBlock.evaluateWith_(((array.notEmpty()).mustBeBoolean()==true ? (function(){return array.last();})() : nil));}
,"visiting",unescape("withNodesIn%3A%20aNode%20do%3A%20aBlock%20atEnd%3A%20endBlock%0D%0A%09%22%20Evaluates%20aBlock%20for%20nodes%20in%20aNode%20%28without%20last%29.%0D%0A%09Return%20the%20result%20of%20evaluating%20lastBlock%20with%20the%20last%20node%20in%20aNode%20%28or%20nil%29.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20array%20%7C%0D%0A%09array%20%3A%3D%20aNode%20nodes%20asArray.%0D%0A%091%20to%3A%20array%20size%20-%201%20do%3A%20%5B%3Aindex%7C%0D%0A%09%09aBlock%20evaluateWith%3A%20%28array%20at%3A%20index%29%0D%0A%09%5D.%0D%0A%09%5EendBlock%20evaluateWith%3A%20%28array%20notEmpty%20ifTrue%3A%20%5B%20array%20last%20%5D%29%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitTemporaries:",0
,function(aNode){var self=this;
aNode.tempNames().do_((function(aName){self.tempVariables().add_(aName);return self.defineTemporary_(aName);}));
return self;}
,"visiting",unescape("visitTemporaries%3A%20aNode%0D%0A%09%22%20Private%20-%20Emmit%20code%20for%20local%20variables%20in%20aNode.%20%22%0D%0A%0D%0A%09aNode%20tempNames%20do%3A%20%5B%3AaName%20%7C%0D%0A%09%09self%20tempVariables%20add%3A%20aName.%0D%0A%09%09self%20defineTemporary%3A%20aName%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.NativeEncoder,"startSequence:",0
,function(aNode){var self=this;
return self;}
,"contexts",unescape("startSequence%3A%20aNode%0D%0A%09%22%20Private%20-%20Prepare%20context%20for%20sequence%20aNode.%20%22%20"));
smalltalk.bind(smalltalk.NativeEncoder,"endSequence:",0
,function(aNode){var self=this;
return self;}
,"contexts",unescape("endSequence%3A%20aNode%0D%0A%09%22%20Private%20-%20Leave%20context%20for%20sequence%20aNode.%20%22%20"));
smalltalk.bind(smalltalk.NativeEncoder,"sequenceSpace:",0
,function(aNode){var self=this;
return self;}
,"contexts",unescape("sequenceSpace%3A%20aNode%0D%0A%09%22%20Private%20-%20aNode%20has%20been%20visited%20in%20a%20sequence.%20%22%20"));
smalltalk.bind(smalltalk.NativeEncoder,"sequenceExit:",0
,function(aNode){var self=this;
self.emmitReturn_((function(){return self.visit_(aNode);}));
return self;}
,"contexts",unescape("sequenceExit%3A%20aNode%0D%0A%09%22%20Private%20-%20A%20block%20sequence%20is%20exiting%20with%20aNode.%20%22%0D%0A%0D%0A%09self%20emmitReturn%3A%20%5B%20self%20visit%3A%20aNode%20%5D%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitBlockSequenceNode:",0
,function(aNode){var self=this;
self["@nestedBlocks"]=self["@nestedBlocks"]._plus((1));
if (aNode.nodes().isEmpty()){self.emmitReturn_("nil");} else{(function($rec){$rec.startSequence_(aNode);$rec.visitTemporaries_(aNode);$rec.withNodesIn_do_atEnd_(aNode,(function(each){self.visitNode_in_cr_(each,aNode,false);return self.sequenceSpace_(each);}),(function(last){return self.sequenceExit_(last);}));return $rec.endSequence_(aNode);})(self);};;
self["@nestedBlocks"]=self["@nestedBlocks"]._minus((1));
return self;}
,"visiting",unescape("visitBlockSequenceNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20the%20block%20sequence%20emmitting%20output.%20%22%0D%0A%0D%0A%09nestedBlocks%20%3A%3D%20nestedBlocks%20+%201.%0D%0A%09aNode%20nodes%20isEmpty%0D%0A%09ifTrue%3A%20%5B%20self%20emmitReturn%3A%20%23nil%20%5D%0D%0A%09ifFalse%3A%20%5B%0D%0A%09%09self%09startSequence%3A%20aNode%3B%0D%0A%09%09%09visitTemporaries%3A%20aNode%3B%0D%0A%09%09%09withNodesIn%3A%20aNode%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%09%09self%20visitNode%3A%20each%20in%3A%20aNode%20cr%3A%20false.%0D%0A%09%09%09%09self%20sequenceSpace%3A%20each%0D%0A%09%09%09%5D%20atEnd%3A%20%5B%3Alast%7C%20self%20sequenceExit%3A%20last%20%5D%3B%0D%0A%09%09%09endSequence%3A%20aNode.%0D%0A%09%5D.%0D%0A%09nestedBlocks%20%3A%3D%20nestedBlocks%20-%201%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitSequenceNode:",0
,function(aNode){var self=this;
(function($rec){$rec.startSequence_(aNode);$rec.visitTemporaries_(aNode);$rec.visitNodesIn_cr_(aNode,true);return $rec.endSequence_(aNode);})(self);
return self;}
,"visiting",unescape("visitSequenceNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20the%20sequence%20emmitting%20output.%20%22%0D%0A%0D%0A%09self%09startSequence%3A%20aNode%3B%0D%0A%09%09visitTemporaries%3A%20aNode%3B%0D%0A%09%09visitNodesIn%3A%20aNode%20cr%3A%20true%3B%0D%0A%09%09endSequence%3A%20aNode.%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitExtendedSequenceNode:",0
,function(aNode){var self=this;
aNode.directives().do_((function(each){return self.visitDirectiveNode_(each);}));
return self.visitSequenceNode_(aNode);}
,"visiting",unescape("visitExtendedSequenceNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20an%20extended%20sequence%20node.%20%22%0D%0A%0D%0A%09aNode%20directives%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09self%20visitDirectiveNode%3A%20each%0D%0A%09%5D.%0D%0A%09%5Eself%20visitSequenceNode%3A%20aNode"));
smalltalk.bind(smalltalk.NativeEncoder,"visitMacroAssignment:",0
,function(aNode){var self=this;
return aNode.left().acceptMacroAssignment_with_(aNode.right(),self);}
,"visiting",unescape("visitMacroAssignment%3A%20aNode%0D%0A%09%5EaNode%20left%20acceptMacroAssignment%3A%20aNode%20right%20with%3A%20self"));
smalltalk.bind(smalltalk.NativeEncoder,"visitMacroAssignment:to:",0
,function(aVariableNode,anExpressionNode){var $s8Ret$={name:"stReturn"};try{var self=this;
var aName=nil;
var aClass=nil;
if (aVariableNode.indices().notEmpty()){return false;};;
aName=aVariableNode.value();
aClass=self.currentClass().instanceClass();
aClass.classVariableNamesDo_((function(each){return ((each._eq(aName)).mustBeBoolean()==true ? (function(){self.visitClassVariableAssignment_in_to_(each,aClass,anExpressionNode);return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();})() : nil);}));
aClass.sharedPoolsAndNamesDo_((function(poolName,pool){return ((pool.includesKey_(aName)).mustBeBoolean()==true ? (function(){self.visitPoolVariableAssignment_in_to_(aName,poolName,anExpressionNode);return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"visiting",unescape("visitMacroAssignment%3A%20aVariableNode%20to%3A%20anExpressionNode%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20generated%20code%20to%20a%20macro%20assignment%20to%20aVariableNode.%20%22%0D%0A%0D%0A%09%7C%20aName%20aClass%20%7C%0D%0A%09aVariableNode%20indices%20notEmpty%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09aName%20%3A%3D%20aVariableNode%20value.%0D%0A%09aClass%20%3A%3D%20self%20currentClass%20instanceClass.%0D%0A%09aClass%20classVariableNamesDo%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20%3D%20aName%20ifTrue%3A%20%5B%0D%0A%09%09%09self%20visitClassVariableAssignment%3A%20each%0D%0A%09%09%09%09in%3A%20aClass%0D%0A%09%09%09%09to%3A%20anExpressionNode.%0D%0A%09%09%09%5Etrue%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09aClass%20sharedPoolsAndNamesDo%3A%20%5B%3ApoolName%20%3Apool%7C%0D%0A%09%09%28pool%20includesKey%3A%20aName%29%20ifTrue%3A%20%5B%0D%0A%09%09%09self%20visitPoolVariableAssignment%3A%20aName%0D%0A%09%09%09%09in%3A%20poolName%0D%0A%09%09%09%09to%3A%20anExpressionNode.%0D%0A%09%09%09%5Etrue%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.NativeEncoder,"visitReceiver:",0
,function(aNode){var self=this;
if (aNode.isAssignment()){return self.visitLocalAssignment_(aNode);};;
return self.visit_(aNode);}
,"visiting",unescape("visitReceiver%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20the%20node%20%28as%20receiver%29%20emmitting%20output.%20%22%0D%0A%0D%0A%09aNode%20isAssignment%20ifTrue%3A%20%5B%20%5Eself%20visitLocalAssignment%3A%20aNode.%20%5D.%0D%0A%09%5Eself%20visit%3A%20aNode"));
smalltalk.bind(smalltalk.NativeEncoder,"visitLocalAssignment:",0
,function(aNode){var self=this;
self.nextArgumentsPut_((function(){return self.visit_(aNode);}));
return self;}
,"visiting",unescape("visitLocalAssignment%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20a%20node%20that%20is%20an%20assignment%20in%20a%20local%20expression.%20%22%0D%0A%0D%0A%09self%20nextArgumentsPut%3A%20%5B%20self%20visit%3A%20aNode%20%5D%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitArguments:",0
,function(anArray){var self=this;
anArray.do_separatedBy_((function(each){return self.visitArgument_(each);}),(function(){return self.nextPutAll_(self.argumentsSeparatorOp());}));
return self;}
,"visiting",unescape("visitArguments%3A%20anArray%0D%0A%0D%0A%09anArray%0D%0A%09%20%20%20%20do%3A%20%5B%3Aeach%20%7C%20self%20visitArgument%3A%20each%20%5D%0D%0A%09%20%20%20%20separatedBy%3A%20%5B%20self%20nextPutAll%3A%20self%20argumentsSeparatorOp%20%5D%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitSendArguments:",0
,function(aSendNode){var self=this;
self.visitArguments_(aSendNode.arguments());
return self;}
,"visiting",unescape("visitSendArguments%3A%20aSendNode%0D%0A%0D%0A%09self%20visitArguments%3A%20aSendNode%20arguments%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitSendNode:",0
,function(aNode){var self=this;
if (aNode.receiver().isSuper()){return self.visitSendToSuper_(aNode);};;
if (self.visitOptimizedSend_(aNode)){return self;};;
if (self.isOperation_(aNode)){return self.visitOperation_(aNode);};;
return self.visitNormalSend_(aNode);}
,"visiting",unescape("visitSendNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20the%20node%20%28message%20send%20node%29%20emmitting%20output.%20%22%0D%0A%0D%0A%09aNode%20receiver%20isSuper%20ifTrue%3A%20%5B%20%5Eself%20visitSendToSuper%3A%20aNode%20%5D.%0D%0A%09%28self%20visitOptimizedSend%3A%20aNode%29%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09%28self%20isOperation%3A%20aNode%29%20ifTrue%3A%20%5B%20%5Eself%20visitOperation%3A%20aNode%20%5D.%0D%0A%09%5Eself%20visitNormalSend%3A%20aNode"));
smalltalk.bind(smalltalk.NativeEncoder,"visitValueNode:",0
,function(aNode){var self=this;
self.nextPutAll_(aNode.value().asNative_(self.nativeLanguage()));
return self;}
,"visiting",unescape("visitValueNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20the%20node%20emmitting%20output.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20%28aNode%20value%20asNative%3A%20self%20nativeLanguage%29"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitEnterMethod:",0
,function(aMethodNode){var self=this;
return self.implementedBySubclass_("emmitEnterMethod:");}
,"generation",unescape("emmitEnterMethod%3A%20aMethodNode%0D%0A%09%22%20Private%20-%20Generate%20home%20context%20definitions%2C%20entering%20aMethodNode.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23emmitEnterMethod%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"emmitFunction:node:arguments:doing:",0
,function(fName,aNode,argumentNames,aBlock){var self=this;
return self.implementedBySubclass_("emmitFunction:node:arguments:doing:");}
,"generation",unescape("emmitFunction%3A%20fName%20node%3A%20aNode%20arguments%3A%20argumentNames%20doing%3A%20aBlock%0D%0A%09%22%20Private%20-%20Generate%20function%20for%20aNode.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23emmitFunction%3Anode%3Aarguments%3Adoing%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"generate:early:return:",0
,function(fName,early,code){var self=this;
return self.implementedBySubclass_("generate:early:return:");}
,"generation",unescape("generate%3A%20fName%20early%3A%20early%20return%3A%20code%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20catch%20early%20return%20of%20code%20expression%20in%20fName.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23generate%3Aearly%3Areturn%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitBlockNode:",0
,function(aNode){var self=this;
return self.implementedBySubclass_("visitBlockNode:");}
,"visiting",unescape("visitBlockNode%3A%20aNode%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23visitBlockNode%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitReturnNode:",0
,function(aNode){var self=this;
return self.implementedBySubclass_("visitReturnNode:");}
,"visiting",unescape("visitReturnNode%3A%20aNode%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23visitReturnNode%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitCascadeNode:",0
,function(aNode){var self=this;
return self.implementedBySubclass_("visitCascadeNode:");}
,"visiting",unescape("visitCascadeNode%3A%20aNode%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23visitCascadeNode%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitClassVariableAssignment:in:to:",0
,function(aClassVariableName,aClass,anExpressionNode){var self=this;
return self.implementedBySubclass_("visitClassVariableAssignment:in:to:");}
,"visiting",unescape("visitClassVariableAssignment%3A%20aClassVariableName%20in%3A%20aClass%20to%3A%20anExpressionNode%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23visitClassVariableAssignment%3Ain%3Ato%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitPoolVariableAssignment:in:to:",0
,function(aName,poolName,anExpressionNode){var self=this;
return self.implementedBySubclass_("visitPoolVariableAssignment:in:to:");}
,"visiting",unescape("visitPoolVariableAssignment%3A%20aName%20in%3A%20poolName%20to%3A%20anExpressionNode%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23visitPoolVariableAssignment%3Ain%3Ato%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitDirectiveNode:",0
,function(aNode){var self=this;
return self.implementedBySubclass_("visitDirectiveNode:");}
,"visiting",unescape("visitDirectiveNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20a%20directive%20node.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23visitDirectiveNode%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitMethodNode:",0
,function(aMethodNode){var self=this;
return self.implementedBySubclass_("visitMethodNode:");}
,"visiting",unescape("visitMethodNode%3A%20aMethodNode%0D%0A%09%22%20Visit%20a%20method%20node.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23visitMethodNode%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitOperation:",0
,function(aNode){var self=this;
return self.implementedBySubclass_("visitOperation:");}
,"visiting",unescape("visitOperation%3A%20aNode%0D%0A%09%22%20Private%20-%20aNode%20is%20an%20operation%20and%20selector%20is%20mapped%20to%20target%20operation.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23visitOperation%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"isNativeCall:",0
,function(aNode){var self=this;
return aNode.selector().startsWith_(unescape("%23"));}
,"query",unescape("isNativeCall%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20true%20if%20aNode%20is%20a%20send%20calling%20to%20native%20function.%20%22%0D%0A%0D%0A%09%5EaNode%20selector%20startsWith%3A%20%24%23%20"));
smalltalk.bind(smalltalk.NativeEncoder,"encodeNormalSend:",0
,function(aNode){var self=this;
return self.implementedBySubclass_("encodeNormalSend:");}
,"encoding",unescape("encodeNormalSend%3A%20aNode%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20direct/inlined%20message%20send.%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23encodeNormalSend%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"encodeSuperSend:",0
,function(aNode){var self=this;
return self.implementedBySubclass_("encodeSuperSend:");}
,"encoding",unescape("encodeSuperSend%3A%20aNode%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20super%20message%20send.%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23encodeSuperSend%3A%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitNormalSend:",0
,function(aNode){var self=this;
self.compilerEvent_with_("aboutSend:",aNode);
self.encodeNormalSend_(aNode);
self.compilerEvent_with_("send:",aNode);
return self;}
,"visiting",unescape("visitNormalSend%3A%20aNode%0D%0A%09%22%20Visit%20direct/inlined%20message%20send.%22%0D%0A%0D%0A%09self%20compilerEvent%3A%20%23aboutSend%3A%20with%3A%20aNode.%0D%0A%09self%20encodeNormalSend%3A%20aNode.%0D%0A%09self%20compilerEvent%3A%20%23send%3A%20with%3A%20aNode.%20"));
smalltalk.bind(smalltalk.NativeEncoder,"visitSendToSuper:",0
,function(aNode){var self=this;
self.compilerEvent_with_("aboutSend:",aNode);
self.encodeSuperSend_(aNode);
self.compilerEvent_with_("send:",aNode);
return self;}
,"visiting",unescape("visitSendToSuper%3A%20aNode%0D%0A%09%22%20Generate%20code%20for%20super%20message%20send.%22%0D%0A%0D%0A%09self%20compilerEvent%3A%20%23aboutSend%3A%20with%3A%20aNode.%0D%0A%09self%20encodeSuperSend%3A%20aNode.%0D%0A%09self%20compilerEvent%3A%20%23send%3A%20with%3A%20aNode.%20"));
smalltalk.bind(smalltalk.NativeEncoder,"currentMethodInfo",0
,function(){var self=this;
return self.compiler().currentClass().name()._comma(unescape("%3E%3E%23"))._comma(self.compiler().selector());}
,"debug",unescape("currentMethodInfo%0D%0A%09%22%20Private%20-%20Returns%20the%20debug%20info%20for%20current%20method.%20%22%0D%0A%0D%0A%09%5Eself%20compiler%20currentClass%20name%20%0D%0A%09%09%2C%27%3E%3E%23%27%20%2Cself%20compiler%20selector"));
smalltalk.bind(smalltalk.JSEncoder.$klass,"supportsLanguage:",0
,function(aLanguage){var self=this;
return (nil.isNil_(aLanguage)).or_((function(){return ["js", "javascript"].includes_(aLanguage);}));}
,"instantiation",unescape("supportsLanguage%3A%20aLanguage%0D%0A%09%22%20Private%20-%20Return%20true%20if%20instances%20of%20the%20receiver%20can%20encode%20to%20aLanguage.%20%22%0D%0A%0D%0A%09%5EaLanguage%20isNil%20or%3A%20%5B%20%23%28js%20javascript%29%20includes%3A%20aLanguage%20%5D%20"));
smalltalk.bind(smalltalk.JSEncoder.$klass,"initializeKernelInlines:",0
,function(pool){var self=this;
return self.register_into_(self.kernelInlines(),pool);}
,"inlines",unescape("initializeKernelInlines%3A%20pool%0D%0A%09%22%20Private%20-%20Initialize%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5Eself%20register%3A%20self%20kernelInlines%20into%3A%20pool%20"));
smalltalk.bind(smalltalk.JSEncoder.$klass,"kernelInlines",0
,function(){var self=this;
return [["Object", [["print:", "self"], ["halt", "debugger"], ["toString", "self"], ["toString:", unescape("%22%22+anObject")], ["isObject:", unescape("return%20%28anObject%3D%3D0%20%7C%7C%20anObject%20%26%26%20anObject.%24klass%29%3Ftrue%3Afalse")], ["typeOf:", "typeof anObject"], ["callableOf:", "self"], ["basicDelete:", unescape("delete%20self%5BaString%5D")], ["basicPerform:withArguments:", ["self", unescape("fn.apply%28self%2CanArray%29")]], ["asJSONObject", unescape("%7B%7D")], ["printingWith:do:", "self"], ["isFile:olderThan:", "self"]]], ["Smalltalk", [["includesKey:", "undefined"], ["eval:", unescape("return%20eval%28anExpression%29")]]], ["Smalltalk class", [["global", unescape("%28function%28%29%7Breturn%20this%3B%7D%29.call%28null%29")], ["gc", unescape("if%28typeof%20gc%21%3D%22undefined%22%29%7Bgc%28%29%3B%20return%20true%7D")]]], ["Behavior", [["basicNew", unescape("new%20self.%24fn%28%29")], ["fn", "self"], ["prototype", "self"], ["privateMethods", "self"]]], ["CompiledMethod", [["asClosure", "self"], ["printString", "self"]]], ["Number", [[unescape("%3D"), unescape("%0D%0A%09if%20%28nil.isNil_%28anObject%29%29%20return%20false%3B%0D%0A%09if%20%28%21%28anObject.isNumber%29%29%20return%20false%3B%0D%0A%09if%20%28%21%28anObject.isNumber%28%29%29%29%20return%20false%3B%0D%0A%09if%20%28self%20%3D%3D%20anObject%29%20return%20true%3B%0D%0A%09return%20%28%280+self-anObject%29%20%3D%3D%200%29")], [unescape("%3E"), unescape("self%20%3E%20aNumber.mustBeNumber%28%29")], [unescape("%3C"), unescape("self%20%3C%20aNumber.mustBeNumber%28%29")], [unescape("%3E%3D"), unescape("self%20%3E%3D%20aNumber.mustBeNumber%28%29")], [unescape("%3C%3D"), unescape("self%20%3C%3D%20aNumber.mustBeNumber%28%29")], [unescape("+"), unescape("self%20+%20aNumber.mustBeNumber%28%29")], [unescape("-"), unescape("self%20-%20aNumber.mustBeNumber%28%29")], [unescape("*"), unescape("self%20*%20aNumber.mustBeNumber%28%29")], [unescape("/"), unescape("self%20/%20aNumber.mustBeNumber%28%29")], ["sin", unescape("Math.sin%28self%29")], ["cos", unescape("Math.cos%28self%29")], ["tan", unescape("Math.tan%28self%29")], ["arcCos", unescape("Math.acos%28self%29")], ["arcSin", unescape("Math.asin%28self%29")], ["arcTan", unescape("Math.atan%28self%29")], ["atan2:", unescape("Math.atan2%28self%2CaNumber.mustBeNumber%28%29%29")], ["sqrt", unescape("Math.sqrt%28self%29")], [unescape("%3C%3C"), unescape("self%20%3C%3C%20aNumber.mustBeNumber%28%29")], [unescape("%3E%3E"), unescape("self%20%3E%3E%20aNumber.mustBeNumber%28%29")], ["raisedTo:", unescape("Math.pow%28self%2CaNumber.mustBeNumber%28%29%29")], ["max:", unescape("Math.max%28self%2CaNumber%29")], ["min:", unescape("Math.min%28self%2CaNumber%29")], ["rounded", unescape("Math.round%28self%29")], ["truncated", unescape("Math.floor%28self%29")], ["timesRepeat:", unescape("for%28var%20i%3D1%3Bi%3C%3Dself%3Bi++%29%7Bfn%28i%29%3B%7D")], ["to:do:", unescape("for%28var%20i%3Dself%3Bi%3C%3DaNumber%3Bi++%29%7Bfn%28i%29%3B%7D")], ["to:by:do:", unescape("if%20%28increment%3E0%29%20%7B%20for%28var%20i%3Dself%3Bi%3C%3DaNumber%3Bi%3Di+increment%29%7Bfn%28i%29%3B%7D%20%7D%0D%0A%09else%20%7B%20for%28var%20i%3Dself%3BaNumber%3C%3Di%3Bi%3Di+increment%29%7Bfn%28i%29%3B%7D%20%7D")], ["printString", unescape("String%28self%29")], ["toString:", unescape("self.toString%28radix%29")], ["isNumber", unescape("return%20isNaN%28self%29.not%28%29")], ["asJSONObject", "self"], ["clearInterval", unescape("clearInterval%28self%29")], ["clearTimeout", unescape("clearTimeout%28self%29")]]], ["Number class", [["pi", "Math.PI"], ["basicNew", "self"]]], ["BlockClosure", [["compiledSource", "self"], ["whileTrue:", unescape("while%28self%28%29%29%20%7Bfn%28%29%7D")], ["whileFalse:", unescape("while%28%21self%28%29%29%20%7Bfn%28%29%7D")], ["value", unescape("self%28%29")], ["value:", unescape("self%28anObject%29")], ["value:value:", unescape("self%28arg1%2Carg2%29")], ["value:value:value:", unescape("self%28arg1%2Carg2%2Carg3%29")], ["value:value:value:value:", unescape("self%28arg1%2Carg2%2Carg3%2Carg4%29")], ["valueWithPossibleArguments:", unescape("self.apply%28null%2Carray%29")], ["try:catch:", unescape("%0D%0A%09var%20result%3B%0D%0A%09try%7B%20result%20%3D%20fn%28%29%20%7D%20catch%28e%29%20%7B%0D%0A%09%09result%20%3D%20ab%28%20self.smalltalkErrorOf_%28e%29%20%29%0D%0A%09%7D%3B%0D%0A%09return%20result")], ["valueWithTimeout:", unescape("setTimeout%28self%2CaNumber%29")], ["valueWithInterval:", unescape("setInterval%28self%2CaNumber%29")], ["ensure:", unescape("try%7Breturn%20self%28%29%3B%7D%20finally%20%7Bfn%28%29%3B%7D")], ["ifCurtailed:", unescape("%0D%0A%09try%7B%09result%20%3D%20self.value%28%29%3B%0D%0A%09%09curtailed%20%3D%20false%3B%0D%0A%09%7D%20finally%20%7B%0D%0A%09%09if%20%28curtailed%29%20exceptionBlock.value%28%29%3B%0D%0A%09%7D%3B%0D%0A%09return%20result")], ["argumentCount", "self.length"]]], ["BlockClosure class", [["defaultSortBlock", "self"]]], ["Boolean", [[unescape("%3D"), unescape("%0D%0A%09if%20%28nil.isNil_%28anObject%29%29%20return%20false%3B%0D%0A%09if%20%28%21%28self.%24klass%20%3D%3D%3D%20anObject.%24klass%29%29%20return%20false%3B%0D%0A%09return%20%28self.valueOf%28%29%20%3D%3D%20anObject.valueOf%28%29%29")], ["ifTrue:ifFalse:", unescape("if%28self%20%3D%3D%20true%29%20%7B%20return%20aBlock.value%28%29%3B%20%7D%20else%20%7B%20return%20anotherBlock.value%28%29%3B%20%7D")], ["not", unescape("self%20%3D%3D%20false")], ["asJSONObject", "self"]]], ["DateTime", [["year", unescape("self.getFullYear%28%29")], ["month", unescape("self.getMonth%28%29%20+%201")], ["month:", unescape("self.setMonth%28aNumber%20-%201%29")], ["day", unescape("self.getDay%28%29%20+%201")], ["day:", unescape("self.setDay%28aNumber%20-%201%29")], ["year:", unescape("self.setFullYear%28aNumber%29")], ["dayOfMonth", unescape("self.getDate%28%29")], ["dayOfMonth:", unescape("self.setDate%28aNumber%29")], ["asString", "self"], ["time", unescape("self.getTime%28%29")], ["time:", unescape("self.setTime%28aNumber%29")], ["asDateString", unescape("self.toDateString%28%29")], ["asTimeString", unescape("self.toTimeString%28%29")], ["asLocaleString", unescape("self.toLocaleString%28%29")], ["hours:", unescape("self.setHours%28aNumber%29")], ["minutes:", unescape("self.setMinutes%28aNumber%29")], ["seconds:", unescape("self.setSeconds%28aNumber%29")], ["milliseconds:", unescape("self.setMilliseconds%28aNumber%29")], ["hours", unescape("self.getHours%28%29")], ["minutes", unescape("self.getMinutes%28%29")], ["seconds", unescape("self.getSeconds%28%29")], ["milliseconds", unescape("self.getMilliseconds%28%29")], [unescape("%3C"), unescape("self%20%3C%20aDateTime")], [unescape("%3E"), unescape("self%20%3E%20aDateTime")], [unescape("%3C%3D"), unescape("self%20%3C%3D%20aDateTime")], [unescape("%3E%3D"), unescape("self%20%3E%3D%20aDateTime")], [unescape("-"), unescape("self%20-%20aDateTime")], [unescape("+"), unescape("self%20+%20aDateTime")], ["isValid", unescape("isFinite%28self%29")], ["asJSONObject", "self"]]], ["DateTime class", [["new:", unescape("return%20new%20Date%28anObject%29")], ["today", "self"], ["now", "self"], ["msNow", "self"]]], ["UndefinedObject", [["asJSONObject", "null"]]], ["Collection", [["wrap:", "self"], ["unwrap:", "self"]]], ["SequenceableCollection", [["indexOf:ifAbsent:", unescape("%0D%0A%09for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%7B%20if%20%28self%5Bi%5D._eq%28anObject%29%29%20%7B%20return%20i%20+%201%3B%20%7D%20%7D%0D%0A%09return%20aBlock.value%28%29")], ["detect:ifNone:", unescape("%0D%0A%09for%28var%20i%20%3D%200%3B%20i%20%3C%20self.length%3B%20i++%29%20if%28fn%28self%5Bi%5D%29%29%20return%20self%5Bi%5D%3B%0D%0A%09return%20anotherBlock.value%28%29")], ["reversed", unescape("%0D%0A%09var%20result%20%3D%20new%20Array%28%29%3B%0D%0A%09var%20len%3Dself.length%3B%20var%20i%3Dlen%3B%0D%0A%09while%28i--%29%7Bresult%5Blen-i-1%5D%3D%20self%5Bi%5D%3B%7D%0D%0A%09return%20result")], ["isEmpty", unescape("%28this.length%29%3Ffalse%3Atrue")], ["reverseDo:", unescape("var%20i%3Dself.length%3Bwhile%28i--%29%7Bfn%28self%5Bi%5D%29%3B%7D")], ["includes:", ["self", unescape("%0D%0A%09if%20%28anObject.isNumber%28%29%20%26%26%20%28self.indexOf%28anObject%29%20%3E%20-1%29%29%20return%20true%3B%0D%0A%09if%20%28anObject.isString%28%29%20%26%26%20%28self.indexOf%28anObject%29%20%3E%20-1%29%29%20return%20true%3B%0D%0A%09var%20i%20%3D%20self.length%3B%0D%0A%09while%20%28i--%29%20%7B%20if%20%28anObject._eq%28self%5Bi%5D%29%29%20%7B%20return%20true%3B%20%7D%20%7D%0D%0A%09return%20false")]], ["occurrencesOf:", unescape("%0D%0A%09var%20i%20%3D%20self.length%3B%20var%20count%20%3D%200%3B%0D%0A%09while%20%28i--%29%20%7B%20if%20%28self%5Bi%5D._eq%28anObject%29%29%20%7B%20count++%3B%20%7D%20%7D%0D%0A%09return%20count")]]], ["String", [[unescape("%3D"), unescape("%0D%0A%09if%20%28nil.isNil_%28anObject%29%29%20return%20false%3B%0D%0A%09if%20%28%21%28anObject.isString%29%29%20return%20false%3B%0D%0A%09if%20%28%21%28anObject.isString%28%29%29%29%20return%20false%3B%0D%0A%09if%20%28self%20%3D%3D%20anObject%29%20return%20true%3B%0D%0A%09return%20%28%28%22%22+self%29%20%3D%3D%20%28%22%22+anObject%29%29")], ["size", "self.length"], ["at:", unescape("self.charAt%28anIndex%20-%201%29")], ["do:", unescape("for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%7Bfn%28self.charAt%28i%29%29%3B%7D")], ["detect:ifNone:", unescape("for%28var%20i%20%3D%200%3B%20i%20%3C%20self.length%3B%20i++%29%0D%0A%09%09if%28fn%28self.charAt%28i%29%29%29%20return%20self.charAt%28i%29%3B%0D%0A%09return%20anotherBlock.value%28%29")], ["includes:", "self"], ["withIndexDo:", unescape("for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%7Bfn%28self.charAt%28i%29%2C%20i+1%29%3B%7D")], ["isLiteral", "self"], ["occurrencesOf:", "self"], ["startsWith:", unescape("if%20%28self.startsWith%29%20%7B%20return%20self.startsWith%28aString%29%20%7D")], ["escaped", unescape("return%20escape%28self%29")], ["unescaped", unescape("return%20unescape%28self%29")], ["asEncodedURI", unescape("return%20encodeURIComponent%28self%29")], ["asDecodedURI", unescape("return%20decodeURIComponent%28self%29")], [unescape("%2C"), unescape("%0D%0A%09if%20%28nil.isNil_%28aCollection%29%29%20return%20self+%22%22%3B%20//%20for%20JSC%20under%20iOS%0D%0A%09return%20self%20+%20aCollection.toString%28%29")], ["copyWith:", unescape("self%20+%20anObject.toString%28%29")], ["copyFrom:to:", unescape("self.substring%28anIndex%20-%201%2CanotherIndex%29")], ["reversed", "self"], ["indexOf:", unescape("return%20self.indexOf%28aString%29%20+%201")], ["indexOf:startingAt:", unescape("return%20self.indexOf%28aString%2Cposition-1%29%20+%201")], ["lastIndexOf:", unescape("return%20self.lastIndexOf%28aString%29%20+%201")], ["lastIndexOf:startingAt:", unescape("return%20self.lastIndexOf%28aString%2Cposition-1%29%20+%201")], ["asNativeName", unescape("self.asJsName%28%29")], ["asPrivateSelector", unescape("result%20%3D%20self%0D%0A%09.replace%28/%3A/g%2C%22_%22%29%0D%0A%09.replace%28/%5B+%5D/g%2C%22_plus%22%29%0D%0A%09.replace%28/-/g%2C%22_minus%22%29%0D%0A%09.replace%28/%5B*%5D/g%2C%22_star%22%29%0D%0A%09.replace%28/%5B/%5D/g%2C%22_slash%22%29%0D%0A%09.replace%28/%5B%7C%5D/g%2C%22_bar%22%29%0D%0A%09.replace%28/%5B%26%5D/g%2C%22_amp%22%29%0D%0A%09.replace%28/%3E/g%2C%22_gt%22%29%0D%0A%09.replace%28/%3C/g%2C%22_lt%22%29%0D%0A%09.replace%28/%3D/g%2C%22_eq%22%29%0D%0A%09.replace%28/%2C/g%2C%22_comma%22%29%0D%0A%09.replace%28/%5B@%5D/g%2C%22_at%22%29%0D%0A%09.replace%28/%7E/g%2C%22_no%22%29%0D%0A%09.replace%28/%5C%5C/g%2C%22_bk%22%29")], ["asNative:", unescape("%0D%0A%09var%20bar%3Dunescape%28%22%25%2522%22%29%3B%0D%0A%09if%28self.search%28/%5E%5Ba-zA-Z0-9_%3A.%24%20%5D*%24/%29%20%3D%3D%20-1%29%0D%0A%09%09return%20%22unescape%28%22%20+bar%20+escape%28self%29%20+bar%20+%22%29%22%3B%0D%0A%09else%0D%0A%09%09return%20bar%20+self%20+bar")], ["replace:with:", "self"], ["replaceRegexp:with:", unescape("self.replace%28aRegexp%2CaString%29")], ["tokenize:", unescape("return%20self.split%28aString%29")], ["match:", unescape("self.search%28aRegexp%29%20%21%3D%20-1")], ["isLetter", unescape("self.search%28%22%5BA-Za-z%5D%24%22%29%3D%3D0")], ["isDigit", unescape("self.search%28%22%5B0-9%5D%24%22%29%3D%3D0")], ["isAlphaNumeric", "self"], ["isUpperCase", "self"], ["isLowerCase", "self"], ["asUppercase", unescape("self.toUpperCase%28%29")], ["asLowercase", unescape("self.toLowerCase%28%29")], ["asNumber", unescape("Number%28self%29")], [unescape("%3E"), unescape("String%28self%29%20%3E%20aString")], [unescape("%3C"), unescape("String%28self%29%20%3C%20aString")], [unescape("%3E%3D"), unescape("String%28self%29%20%3E%3D%20aString")], [unescape("%3C%3D"), unescape("String%28self%29%20%3C%3D%20aString")], ["asJSONObject", "self"], ["isCharacter", unescape("self.length%20%3D%3D%201")], ["asciiValue", unescape("self.charCodeAt%280%29")], ["isSeparator", unescape("%28new%20RegExp%28%22%5C%5Cs%7C%5C%5Cu0085%22%29%29.test%28self%29")]]], ["String class", [["fromString:", unescape("return%20new%20self.%24fn%28aString%29")], ["new", unescape("%22%22")], ["cr", unescape("%22%5Cr%22")], ["lf", unescape("%22%5Cn%22")], ["crlf", unescape("%22%5Cr%5Cn%22")], ["space", unescape("%22%20%22")], ["tab", unescape("%22%5Ct%22")], ["zero", unescape("%22%5C0%22")], ["fromCharCode:", unescape("%0D%0A%09if%20%28anArrayOrNumber.isNumber%28%29%29%20return%20String.fromCharCode%28anArrayOrNumber%29%3B%0D%0A%09return%20String.fromCharCode.apply%28null%2CanArrayOrNumber.asArray%28%29%29%3B")], ["basicNew", "self"]]], ["Array", [["size", "self.length"], ["at:", unescape("var%20value%20%3D%20self%5BanIndex%20-%201%5D%3B%0D%0A%09if%28value%20%3D%3D%3D%20undefined%29%20%7B%20return%20self.objectNotFoundAt_%28anIndex%29%3B%20%7D%0D%0A%09return%20value")], ["at:ifAbsent:", unescape("var%20value%20%3D%20self%5BanIndex%20-%201%5D%3B%0D%0A%09if%28value%20%3D%3D%3D%20undefined%29%20%7B%20return%20aBlock.value%28%29%3B%20%7D%0D%0A%09return%20value")], ["at:put:", unescape("return%20self%5BanIndex%20-%201%5D%20%3D%20anObject")], ["do:", unescape("for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%7Bfn%28self%5Bi%5D%29%3B%7D")], ["addAll:", unescape("self.push.apply%28self%2C%20aCollection.asArray%28%29%29")], ["shallowCopy", unescape("result%20%3D%20self.slice%280%29")], ["copyWithAll:", unescape("return%20self.concat%28aCollection.asArray%28%29%29")], [unescape("%2C"), unescape("return%20self.concat%28aCollection.asArray%28%29%29")], ["join:", unescape("self.join%28aString%29")], ["sort:", unescape("if%20%28nil.isNil_%28aBlock%29%29%20%7B%20self.sort%28%29%3B%20return%20self%20%7D%0D%0A%09var%20criteria%20%3D%20function%28a%2C%20b%29%20%7B%0D%0A%09%09if%28fn%28a%2Cb%29%29%20%7Breturn%20-1%7D%20else%20%7Breturn%201%7D%0D%0A%09%7D%0D%0A%09self.sort%28criteria%29")], ["remove:ifAbsent:", unescape("for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%20%7B%0D%0A%09%09if%28self%5Bi%5D%20%3D%3D%20anObject%29%20%7B%0D%0A%09%09%09self.splice%28i%2C1%29%3B%20return%20anObject%3B%0D%0A%09%09%7D%0D%0A%09%7D%3B%20return%20aBlock.value%28%29")], ["add:", unescape("self.push%28anObject%29")], ["addFirst:", unescape("self.splice%280%2C0%2CanObject%29")], ["addLast:", unescape("self.push%28anObject%29")], ["removeAll", unescape("self.splice%280%2Cself.size%28%29%29")], ["removeFirst", unescape("self.splice%280%2C1%29")], ["removeIndex:", unescape("self.splice%28index-1%2C1%29")], ["removeLast", unescape("self.pop%28%29")], ["atAllPut:", unescape("for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%7B%20self%5Bi%5D%20%3D%20anObject%3B%20%7D")], ["first", unescape("var%20result%3Dself%5B0%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result")], ["second", unescape("var%20result%3Dself%5B1%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result")], ["third", unescape("var%20result%3Dself%5B2%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result")], ["fourth", unescape("%09var%20result%3Dself%5B3%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result")], ["fifth", unescape("var%20result%3Dself%5B4%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result")], ["sixth", unescape("var%20result%3Dself%5B5%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result")], ["asJSONObject", ["self", "self"]], ["json", ["self", "self"]]]], ["Array class", [["basicNew:", unescape("new%20Array%28size%29")], ["with:", unescape("return%20%5B%20anObject%20%5D")], ["with:with:", unescape("return%20%5B%20firstObject%2C%20secondObject%20%5D")], ["with:with:with:", unescape("return%20%5B%20firstObject%2C%20secondObject%2C%20thirdObject%20%5D")], ["with:with:with:with:", unescape("return%20%5B%20firstObject%2C%20secondObject%2C%20thirdObject%2C%20fourthObject%20%5D")], ["withAll:", unescape("return%20%5B%5D.concat%28aCollection.asArray%28%29%29")], ["slice:", unescape("Array.prototype.slice.call%28anIterable%29")]]], ["ComposedAssociations", [["hashObject", unescape("%7B%7D")]]], ["HashedSet", [["initializeHash", unescape("%7B%7D")], ["privateAdd:", "self"], ["privateRemove:", unescape("delete%20self%5B%22@hash%22%5D%5Bcookie%5D")], ["includes:", unescape("Object.prototype.hasOwnProperty.call%28self%5B%22@hash%22%5D%2Ccookie%29")]]], ["KeyedCollection", [["newHash", unescape("%7B%7D")]]], ["KeyedCollection class", [[unescape("@"), unescape("parts%3DObject.getOwnPropertyNames%28jsObject%29")]]], ["Dictionary class", [["basicNew", "self"]]], ["PoolDictionary", [["validKey:", "self"], ["at:put:", "self"], ["removeKey:ifAbsent:", unescape("delete%20self%5B%22@hash%22%5D%5BaKey%5D")], ["at:ifAbsent:", unescape("var%20found%20%3D%20Object.prototype.hasOwnProperty.call%28self%5B%22@hash%22%5D%2CaKey%29%3B%0D%0A%09if%28found%29%20return%20self%5B%22@hash%22%5D%5BaKey%5D%3B%0D%0A%09return%20aBlock.value%28%29")], ["includesKey:", unescape("Object.prototype.hasOwnProperty.call%28self%5B%22@hash%22%5D%2CaKey%29")], ["keys", unescape("return%20Object.getOwnPropertyNames%28self%5B%22@hash%22%5D%29")]]], ["RegularExpression class", [["fromString:flag:", unescape("new%20RegExp%28aString%2C%20anotherString%29")]]], ["Exception", [["captureStack", "self"], ["signal", unescape("throw%28self%29")]]], ["ClassBuilder", [["addSubclassOf:named:instanceVariableNames:", unescape("%0D%0A%09smalltalk.addClass%28aString%2C%20aClass%2C%20aCollection%29%3B%0D%0A%09return%20smalltalk%5BaString%5D")]]], ["StringStream", [["peek", unescape("return%20self%5B%22@collection%22%5D.charAt%28self.position%28%29%29%7C%7Cnil")], ["peekForAll:", unescape("var%20start%20%3D%20self.position%28%29%3B%0D%0A%09for%20%28var%20i%3D0%3Bi%3CaCollection.length%3B%20i++%29%7B%0D%0A%09%09if%20%28self%5B%22@collection%22%5D.charCodeAt%28start+i%29%20%21%3D%20aCollection.charCodeAt%28i%29%29%20return%20false%3B%0D%0A%09%7D")]]], ["Random", [["next", unescape("return%20Math.random%28%29")]]], ["Rectangle", [["printString", "self"]]]];}
,"inlines",unescape("kernelInlines%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5E%23%28%0D%0A%23%28Object%20%23%28%0D%0A%23%28print%3A%09%09%27self%27%29%0D%0A%23%28halt%09%09%27debugger%27%29%0D%0A%23%28toString%09%27self%27%29%0D%0A%23%28toString%3A%09%27%22%22+anObject%27%29%0D%0A%23%28isObject%3A%09%27return%20%28anObject%3D%3D0%20%7C%7C%20anObject%20%26%26%20anObject.%24klass%29%3Ftrue%3Afalse%27%29%0D%0A%23%28typeOf%3A%09%27typeof%20anObject%27%29%0D%0A%23%28callableOf%3A%09%27self%27%29%0D%0A%23%28basicDelete%3A%09%27delete%20self%5BaString%5D%27%29%0D%0A%23%28basicPerform%3AwithArguments%3A%20%23%28%27self%27%20%27fn.apply%28self%2CanArray%29%27%29%29%0D%0A%23%28asJSONObject%09%27%7B%7D%27%29%0D%0A%23%28printingWith%3Ado%3A%09%27self%27%29%0D%0A%23%28isFile%3AolderThan%3A%20%27self%27%29%0D%0A%29%29%0D%0A%23%28Smalltalk%20%23%28%0D%0A%23%28includesKey%3A%09%27undefined%27%29%0D%0A%23%28eval%3A%09%27return%20eval%28anExpression%29%27%29%0D%0A%29%29%0D%0A%23%28%27Smalltalk%20class%27%20%23%28%0D%0A%23%28global%09%27%28function%28%29%7Breturn%20this%3B%7D%29.call%28null%29%27%29%0D%0A%23%28gc%09%27if%28typeof%20gc%21%3D%22undefined%22%29%7Bgc%28%29%3B%20return%20true%7D%27%29%0D%0A%29%29%0D%0A%23%28Behavior%20%23%28%0D%0A%23%28basicNew%09%27new%20self.%24fn%28%29%27%29%0D%0A%23%28fn%09%09%27self%27%29%0D%0A%23%28prototype%09%27self%27%29%0D%0A%23%28privateMethods%09%27self%27%29%0D%0A%29%29%0D%0A%23%28CompiledMethod%20%23%28%0D%0A%23%28asClosure%09%27self%27%29%0D%0A%23%28printString%09%27self%27%29%0D%0A%29%29%0D%0A%23%28Number%20%23%28%0D%0A%23%28%3D%20%27%0D%0A%09if%20%28nil.isNil_%28anObject%29%29%20return%20false%3B%0D%0A%09if%20%28%21%28anObject.isNumber%29%29%20return%20false%3B%0D%0A%09if%20%28%21%28anObject.isNumber%28%29%29%29%20return%20false%3B%0D%0A%09if%20%28self%20%3D%3D%20anObject%29%20return%20true%3B%0D%0A%09return%20%28%280+self-anObject%29%20%3D%3D%200%29%27%29%0D%0A%23%28%3E%20%27self%20%3E%20aNumber.mustBeNumber%28%29%27%29%0D%0A%23%28%3C%20%27self%20%3C%20aNumber.mustBeNumber%28%29%27%29%0D%0A%23%28%3E%3D%20%27self%20%3E%3D%20aNumber.mustBeNumber%28%29%27%29%0D%0A%23%28%3C%3D%20%27self%20%3C%3D%20aNumber.mustBeNumber%28%29%27%29%0D%0A%23%28+%20%27self%20+%20aNumber.mustBeNumber%28%29%27%29%0D%0A%23%28-%20%27self%20-%20aNumber.mustBeNumber%28%29%27%29%0D%0A%23%28*%20%27self%20*%20aNumber.mustBeNumber%28%29%27%29%0D%0A%23%28/%20%27self%20/%20aNumber.mustBeNumber%28%29%27%29%0D%0A%23%28sin%20%27Math.sin%28self%29%27%29%0D%0A%23%28cos%20%27Math.cos%28self%29%27%29%0D%0A%23%28tan%20%27Math.tan%28self%29%27%29%0D%0A%23%28arcCos%20%27Math.acos%28self%29%27%29%0D%0A%23%28arcSin%20%27Math.asin%28self%29%27%29%0D%0A%23%28arcTan%20%27Math.atan%28self%29%27%29%0D%0A%23%28atan2%3A%20%27Math.atan2%28self%2CaNumber.mustBeNumber%28%29%29%27%29%0D%0A%23%28sqrt%20%27Math.sqrt%28self%29%27%29%0D%0A%23%28%3C%3C%20%27self%20%3C%3C%20aNumber.mustBeNumber%28%29%27%29%0D%0A%23%28%3E%3E%20%27self%20%3E%3E%20aNumber.mustBeNumber%28%29%27%29%0D%0A%23%28raisedTo%3A%20%27Math.pow%28self%2CaNumber.mustBeNumber%28%29%29%27%29%0D%0A%23%28max%3A%20%27Math.max%28self%2CaNumber%29%27%29%0D%0A%23%28min%3A%20%27Math.min%28self%2CaNumber%29%27%29%0D%0A%23%28rounded%20%27Math.round%28self%29%27%29%0D%0A%23%28truncated%20%27Math.floor%28self%29%27%29%0D%0A%23%28timesRepeat%3A%20%27for%28var%20i%3D1%3Bi%3C%3Dself%3Bi++%29%7Bfn%28i%29%3B%7D%27%29%0D%0A%23%28to%3Ado%3A%20%27for%28var%20i%3Dself%3Bi%3C%3DaNumber%3Bi++%29%7Bfn%28i%29%3B%7D%27%29%0D%0A%23%28to%3Aby%3Ado%3A%20%27if%20%28increment%3E0%29%20%7B%20for%28var%20i%3Dself%3Bi%3C%3DaNumber%3Bi%3Di+increment%29%7Bfn%28i%29%3B%7D%20%7D%0D%0A%09else%20%7B%20for%28var%20i%3Dself%3BaNumber%3C%3Di%3Bi%3Di+increment%29%7Bfn%28i%29%3B%7D%20%7D%27%29%0D%0A%23%28printString%20%27String%28self%29%27%29%0D%0A%23%28toString%3A%20%27self.toString%28radix%29%27%29%0D%0A%23%28isNumber%20%27return%20isNaN%28self%29.not%28%29%27%29%0D%0A%23%28asJSONObject%20%27self%27%29%0D%0A%23%28clearInterval%20%27clearInterval%28self%29%27%29%0D%0A%23%28clearTimeout%20%27clearTimeout%28self%29%27%29%0D%0A%29%29%0D%0A%23%28%27Number%20class%27%20%23%28%0D%0A%23%28pi%20%27Math.PI%27%29%0D%0A%23%28basicNew%20%27self%27%29%0D%0A%29%29%0D%0A%23%28BlockClosure%20%23%28%0D%0A%23%28compiledSource%20%27self%27%29%0D%0A%23%28whileTrue%3A%20%27while%28self%28%29%29%20%7Bfn%28%29%7D%27%29%0D%0A%23%28whileFalse%3A%20%27while%28%21self%28%29%29%20%7Bfn%28%29%7D%27%29%0D%0A%23%28value%20%27self%28%29%27%29%0D%0A%23%28value%3A%20%27self%28anObject%29%27%29%0D%0A%23%28value%3Avalue%3A%20%27self%28arg1%2Carg2%29%27%29%0D%0A%23%28value%3Avalue%3Avalue%3A%20%27self%28arg1%2Carg2%2Carg3%29%27%29%0D%0A%23%28value%3Avalue%3Avalue%3Avalue%3A%20%27self%28arg1%2Carg2%2Carg3%2Carg4%29%27%29%0D%0A%23%28valueWithPossibleArguments%3A%20%27self.apply%28null%2Carray%29%27%29%0D%0A%23%28try%3Acatch%3A%20%27%0D%0A%09var%20result%3B%0D%0A%09try%7B%20result%20%3D%20fn%28%29%20%7D%20catch%28e%29%20%7B%0D%0A%09%09result%20%3D%20ab%28%20self.smalltalkErrorOf_%28e%29%20%29%0D%0A%09%7D%3B%0D%0A%09return%20result%27%29%0D%0A%23%28valueWithTimeout%3A%20%27setTimeout%28self%2CaNumber%29%27%29%0D%0A%23%28valueWithInterval%3A%20%27setInterval%28self%2CaNumber%29%27%29%0D%0A%23%28ensure%3A%20%27try%7Breturn%20self%28%29%3B%7D%20finally%20%7Bfn%28%29%3B%7D%27%29%0D%0A%23%28ifCurtailed%3A%20%27%0D%0A%09try%7B%09result%20%3D%20self.value%28%29%3B%0D%0A%09%09curtailed%20%3D%20false%3B%0D%0A%09%7D%20finally%20%7B%0D%0A%09%09if%20%28curtailed%29%20exceptionBlock.value%28%29%3B%0D%0A%09%7D%3B%0D%0A%09return%20result%27%29%0D%0A%23%28argumentCount%20%27self.length%27%29%0D%0A%29%29%0D%0A%23%28%27BlockClosure%20class%27%20%23%28%0D%0A%23%28defaultSortBlock%20%27self%27%29%0D%0A%29%29%0D%0A%23%28Boolean%20%23%28%0D%0A%23%28%3D%09%27%0D%0A%09if%20%28nil.isNil_%28anObject%29%29%20return%20false%3B%0D%0A%09if%20%28%21%28self.%24klass%20%3D%3D%3D%20anObject.%24klass%29%29%20return%20false%3B%0D%0A%09return%20%28self.valueOf%28%29%20%3D%3D%20anObject.valueOf%28%29%29%27%29%0D%0A%23%28ifTrue%3AifFalse%3A%20%27if%28self%20%3D%3D%20true%29%20%7B%20return%20aBlock.value%28%29%3B%20%7D%20else%20%7B%20return%20anotherBlock.value%28%29%3B%20%7D%27%29%0D%0A%23%28not%20%09%27self%20%3D%3D%20false%27%29%0D%0A%23%28asJSONObject%20%27self%27%29%0D%0A%29%29%0D%0A%23%28DateTime%20%23%28%0D%0A%23%28year%20%27self.getFullYear%28%29%27%29%0D%0A%23%28month%20%27self.getMonth%28%29%20+%201%27%29%0D%0A%23%28month%3A%20%27self.setMonth%28aNumber%20-%201%29%27%29%0D%0A%23%28day%20%27self.getDay%28%29%20+%201%27%29%0D%0A%23%28day%3A%20%27self.setDay%28aNumber%20-%201%29%27%29%0D%0A%23%28year%3A%20%27self.setFullYear%28aNumber%29%27%29%0D%0A%23%28dayOfMonth%20%27self.getDate%28%29%27%29%0D%0A%23%28dayOfMonth%3A%20%27self.setDate%28aNumber%29%27%29%0D%0A%23%28asString%20%27self%27%29%0D%0A%23%28time%20%27self.getTime%28%29%27%29%0D%0A%23%28time%3A%20%27self.setTime%28aNumber%29%27%29%0D%0A%23%28asDateString%20%27self.toDateString%28%29%27%29%0D%0A%23%28asTimeString%20%27self.toTimeString%28%29%27%29%0D%0A%23%28asLocaleString%20%27self.toLocaleString%28%29%27%29%0D%0A%23%28hours%3A%20%27self.setHours%28aNumber%29%27%29%0D%0A%23%28minutes%3A%20%27self.setMinutes%28aNumber%29%27%29%0D%0A%23%28seconds%3A%20%27self.setSeconds%28aNumber%29%27%29%0D%0A%23%28milliseconds%3A%20%27self.setMilliseconds%28aNumber%29%27%29%0D%0A%23%28hours%20%27self.getHours%28%29%27%29%0D%0A%23%28minutes%20%27self.getMinutes%28%29%27%29%0D%0A%23%28seconds%20%27self.getSeconds%28%29%27%29%0D%0A%23%28milliseconds%20%27self.getMilliseconds%28%29%27%29%0D%0A%23%28%3C%20%27self%20%3C%20aDateTime%27%29%0D%0A%23%28%3E%20%27self%20%3E%20aDateTime%27%29%0D%0A%23%28%3C%3D%20%27self%20%3C%3D%20aDateTime%27%29%0D%0A%23%28%3E%3D%20%27self%20%3E%3D%20aDateTime%27%29%0D%0A%23%28-%20%27self%20-%20aDateTime%27%29%0D%0A%23%28+%20%27self%20+%20aDateTime%27%29%0D%0A%23%28isValid%20%27isFinite%28self%29%27%29%0D%0A%23%28asJSONObject%20%27self%27%29%0D%0A%29%29%0D%0A%23%28%27DateTime%20class%27%20%23%28%0D%0A%23%28new%3A%20%27return%20new%20Date%28anObject%29%27%29%0D%0A%23%28today%20%27self%27%29%0D%0A%23%28now%20%27self%27%29%0D%0A%23%28msNow%20%27self%27%29%0D%0A%29%29%0D%0A%23%28UndefinedObject%20%20%23%28%0D%0A%23%28asJSONObject%20%27null%27%29%0D%0A%29%29%0D%0A%23%28Collection%20%20%23%28%0D%0A%23%28wrap%3A%20%27self%27%29%0D%0A%23%28unwrap%3A%20%27self%27%29%0D%0A%29%29%0D%0A%23%28SequenceableCollection%20%23%28%0D%0A%23%28indexOf%3AifAbsent%3A%20%27%0D%0A%09for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%7B%20if%20%28self%5Bi%5D._eq%28anObject%29%29%20%7B%20return%20i%20+%201%3B%20%7D%20%7D%0D%0A%09return%20aBlock.value%28%29%27%29%0D%0A%23%28detect%3AifNone%3A%20%27%0D%0A%09for%28var%20i%20%3D%200%3B%20i%20%3C%20self.length%3B%20i++%29%20if%28fn%28self%5Bi%5D%29%29%20return%20self%5Bi%5D%3B%0D%0A%09return%20anotherBlock.value%28%29%27%29%0D%0A%23%28reversed%20%27%0D%0A%09var%20result%20%3D%20new%20Array%28%29%3B%0D%0A%09var%20len%3Dself.length%3B%20var%20i%3Dlen%3B%0D%0A%09while%28i--%29%7Bresult%5Blen-i-1%5D%3D%20self%5Bi%5D%3B%7D%0D%0A%09return%20result%27%29%0D%0A%23%28isEmpty%20%27%28this.length%29%3Ffalse%3Atrue%27%29%0D%0A%23%28reverseDo%3A%20%20%27var%20i%3Dself.length%3Bwhile%28i--%29%7Bfn%28self%5Bi%5D%29%3B%7D%27%29%0D%0A%23%28includes%3A%20%23%28%27self%27%20%27%0D%0A%09if%20%28anObject.isNumber%28%29%20%26%26%20%28self.indexOf%28anObject%29%20%3E%20-1%29%29%20return%20true%3B%0D%0A%09if%20%28anObject.isString%28%29%20%26%26%20%28self.indexOf%28anObject%29%20%3E%20-1%29%29%20return%20true%3B%0D%0A%09var%20i%20%3D%20self.length%3B%0D%0A%09while%20%28i--%29%20%7B%20if%20%28anObject._eq%28self%5Bi%5D%29%29%20%7B%20return%20true%3B%20%7D%20%7D%0D%0A%09return%20false%27%29%29%0D%0A%23%28occurrencesOf%3A%20%27%0D%0A%09var%20i%20%3D%20self.length%3B%20var%20count%20%3D%200%3B%0D%0A%09while%20%28i--%29%20%7B%20if%20%28self%5Bi%5D._eq%28anObject%29%29%20%7B%20count++%3B%20%7D%20%7D%0D%0A%09return%20count%27%29%0D%0A%29%29%0D%0A%23%28String%20%23%28%0D%0A%23%28%3D%09%27%0D%0A%09if%20%28nil.isNil_%28anObject%29%29%20return%20false%3B%0D%0A%09if%20%28%21%28anObject.isString%29%29%20return%20false%3B%0D%0A%09if%20%28%21%28anObject.isString%28%29%29%29%20return%20false%3B%0D%0A%09if%20%28self%20%3D%3D%20anObject%29%20return%20true%3B%0D%0A%09return%20%28%28%22%22+self%29%20%3D%3D%20%28%22%22+anObject%29%29%27%29%0D%0A%23%28size%09%27self.length%27%29%0D%0A%23%28at%3A%09%27self.charAt%28anIndex%20-%201%29%27%29%0D%0A%23%28do%3A%20%09%27for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%7Bfn%28self.charAt%28i%29%29%3B%7D%27%29%0D%0A%23%28detect%3AifNone%3A%20%27for%28var%20i%20%3D%200%3B%20i%20%3C%20self.length%3B%20i++%29%0D%0A%09%09if%28fn%28self.charAt%28i%29%29%29%20return%20self.charAt%28i%29%3B%0D%0A%09return%20anotherBlock.value%28%29%27%29%0D%0A%23%28includes%3A%20%27self%27%29%0D%0A%23%28withIndexDo%3A%20%27for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%7Bfn%28self.charAt%28i%29%2C%20i+1%29%3B%7D%27%29%0D%0A%23%28isLiteral%20%27self%27%29%0D%0A%23%28occurrencesOf%3A%20%27self%27%29%0D%0A%23%28startsWith%3A%20%27if%20%28self.startsWith%29%20%7B%20return%20self.startsWith%28aString%29%20%7D%27%29%0D%0A%23%28escaped%20%27return%20escape%28self%29%27%29%0D%0A%23%28unescaped%20%27return%20unescape%28self%29%27%29%0D%0A%23%28asEncodedURI%20%27return%20encodeURIComponent%28self%29%27%29%0D%0A%23%28asDecodedURI%20%27return%20decodeURIComponent%28self%29%27%29%0D%0A%23%28%2C%20%27%0D%0A%09if%20%28nil.isNil_%28aCollection%29%29%20return%20self+%22%22%3B%20//%20for%20JSC%20under%20iOS%0D%0A%09return%20self%20+%20aCollection.toString%28%29%27%29%0D%0A%23%28copyWith%3A%20%27self%20+%20anObject.toString%28%29%27%29%0D%0A%23%28copyFrom%3Ato%3A%20%27self.substring%28anIndex%20-%201%2CanotherIndex%29%27%29%0D%0A%23%28reversed%20%27self%27%29%0D%0A%23%28indexOf%3A%20%27return%20self.indexOf%28aString%29%20+%201%27%29%0D%0A%23%28indexOf%3AstartingAt%3A%20%27return%20self.indexOf%28aString%2Cposition-1%29%20+%201%27%29%0D%0A%23%28lastIndexOf%3A%20%27return%20self.lastIndexOf%28aString%29%20+%201%27%29%0D%0A%23%28lastIndexOf%3AstartingAt%3A%20%27return%20self.lastIndexOf%28aString%2Cposition-1%29%20+%201%27%29%0D%0A%23%28asNativeName%20%27self.asJsName%28%29%27%29%0D%0A%23%28asPrivateSelector%20%27result%20%3D%20self%0D%0A%09.replace%28/%3A/g%2C%22_%22%29%0D%0A%09.replace%28/%5B+%5D/g%2C%22_plus%22%29%0D%0A%09.replace%28/-/g%2C%22_minus%22%29%0D%0A%09.replace%28/%5B*%5D/g%2C%22_star%22%29%0D%0A%09.replace%28/%5B/%5D/g%2C%22_slash%22%29%0D%0A%09.replace%28/%5B%7C%5D/g%2C%22_bar%22%29%0D%0A%09.replace%28/%5B%26%5D/g%2C%22_amp%22%29%0D%0A%09.replace%28/%3E/g%2C%22_gt%22%29%0D%0A%09.replace%28/%3C/g%2C%22_lt%22%29%0D%0A%09.replace%28/%3D/g%2C%22_eq%22%29%0D%0A%09.replace%28/%2C/g%2C%22_comma%22%29%0D%0A%09.replace%28/%5B@%5D/g%2C%22_at%22%29%0D%0A%09.replace%28/%7E/g%2C%22_no%22%29%0D%0A%09.replace%28/%5C%5C/g%2C%22_bk%22%29%27%29%0D%0A%23%28asNative%3A%20%27%0D%0A%09var%20bar%3Dunescape%28%22%25%2522%22%29%3B%0D%0A%09if%28self.search%28/%5E%5Ba-zA-Z0-9_%3A.%24%20%5D*%24/%29%20%3D%3D%20-1%29%0D%0A%09%09return%20%22unescape%28%22%20+bar%20+escape%28self%29%20+bar%20+%22%29%22%3B%0D%0A%09else%0D%0A%09%09return%20bar%20+self%20+bar%27%29%0D%0A%23%28replace%3Awith%3A%20%27self%27%29%0D%0A%23%28replaceRegexp%3Awith%3A%20%27self.replace%28aRegexp%2CaString%29%27%29%0D%0A%23%28tokenize%3A%20%27return%20self.split%28aString%29%27%29%0D%0A%23%28match%3A%20%27self.search%28aRegexp%29%20%21%3D%20-1%27%29%0D%0A%23%28isLetter%20%27self.search%28%22%5BA-Za-z%5D%24%22%29%3D%3D0%27%29%0D%0A%23%28isDigit%20%27self.search%28%22%5B0-9%5D%24%22%29%3D%3D0%27%29%0D%0A%23%28isAlphaNumeric%20%27self%27%29%0D%0A%23%28isUpperCase%20%27self%27%29%0D%0A%23%28isLowerCase%20%27self%27%29%0D%0A%23%28asUppercase%20%27self.toUpperCase%28%29%27%29%0D%0A%23%28asLowercase%20%27self.toLowerCase%28%29%27%29%0D%0A%23%28asNumber%20%27Number%28self%29%27%29%0D%0A%23%28%3E%20%27String%28self%29%20%3E%20aString%27%29%0D%0A%23%28%3C%20%27String%28self%29%20%3C%20aString%27%29%0D%0A%23%28%3E%3D%20%27String%28self%29%20%3E%3D%20aString%27%29%0D%0A%23%28%3C%3D%20%27String%28self%29%20%3C%3D%20aString%27%29%0D%0A%23%28asJSONObject%20%27self%27%29%0D%0A%23%28isCharacter%20%27self.length%20%3D%3D%201%27%29%0D%0A%23%28asciiValue%20%27self.charCodeAt%280%29%27%29%0D%0A%23%28isSeparator%20%27%28new%20RegExp%28%22%5C%5Cs%7C%5C%5Cu0085%22%29%29.test%28self%29%27%29%0D%0A%29%29%0D%0A%23%28%27String%20class%27%20%23%28%0D%0A%23%28fromString%3A%20%27return%20new%20self.%24fn%28aString%29%27%29%0D%0A%23%28new%09%27%22%22%27%29%0D%0A%23%28cr%20%09%27%22%5Cr%22%27%29%0D%0A%23%28lf%20%09%27%22%5Cn%22%27%29%0D%0A%23%28crlf%20%09%27%22%5Cr%5Cn%22%27%29%0D%0A%23%28space%09%27%22%20%22%27%29%0D%0A%23%28tab%09%27%22%5Ct%22%27%29%0D%0A%23%28zero%09%27%22%5C0%22%27%29%0D%0A%23%28fromCharCode%3A%20%27%0D%0A%09if%20%28anArrayOrNumber.isNumber%28%29%29%20return%20String.fromCharCode%28anArrayOrNumber%29%3B%0D%0A%09return%20String.fromCharCode.apply%28null%2CanArrayOrNumber.asArray%28%29%29%3B%27%29%0D%0A%23%28basicNew%20%27self%27%29%0D%0A%29%29%0D%0A%23%28Array%20%23%28%0D%0A%23%28size%20%27self.length%27%29%0D%0A%23%28at%3A%20%27var%20value%20%3D%20self%5BanIndex%20-%201%5D%3B%0D%0A%09if%28value%20%3D%3D%3D%20undefined%29%20%7B%20return%20self.objectNotFoundAt_%28anIndex%29%3B%20%7D%0D%0A%09return%20value%27%29%0D%0A%23%28at%3AifAbsent%3A%20%27var%20value%20%3D%20self%5BanIndex%20-%201%5D%3B%0D%0A%09if%28value%20%3D%3D%3D%20undefined%29%20%7B%20return%20aBlock.value%28%29%3B%20%7D%0D%0A%09return%20value%27%29%0D%0A%23%28at%3Aput%3A%20%27return%20self%5BanIndex%20-%201%5D%20%3D%20anObject%27%29%0D%0A%23%28do%3A%20%27for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%7Bfn%28self%5Bi%5D%29%3B%7D%27%29%0D%0A%23%28addAll%3A%20%27self.push.apply%28self%2C%20aCollection.asArray%28%29%29%27%29%0D%0A%23%28shallowCopy%20%27result%20%3D%20self.slice%280%29%27%29%0D%0A%23%28copyWithAll%3A%20%27return%20self.concat%28aCollection.asArray%28%29%29%27%29%0D%0A%23%28%2C%20%27return%20self.concat%28aCollection.asArray%28%29%29%27%29%0D%0A%23%28join%3A%20%27self.join%28aString%29%27%29%0D%0A%23%28sort%3A%20%27if%20%28nil.isNil_%28aBlock%29%29%20%7B%20self.sort%28%29%3B%20return%20self%20%7D%0D%0A%09var%20criteria%20%3D%20function%28a%2C%20b%29%20%7B%0D%0A%09%09if%28fn%28a%2Cb%29%29%20%7Breturn%20-1%7D%20else%20%7Breturn%201%7D%0D%0A%09%7D%0D%0A%09self.sort%28criteria%29%27%29%0D%0A%23%28remove%3AifAbsent%3A%20%27for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%20%7B%0D%0A%09%09if%28self%5Bi%5D%20%3D%3D%20anObject%29%20%7B%0D%0A%09%09%09self.splice%28i%2C1%29%3B%20return%20anObject%3B%0D%0A%09%09%7D%0D%0A%09%7D%3B%20return%20aBlock.value%28%29%27%29%0D%0A%23%28add%3A%20%27self.push%28anObject%29%27%29%0D%0A%23%28addFirst%3A%20%27self.splice%280%2C0%2CanObject%29%27%29%0D%0A%23%28addLast%3A%20%27self.push%28anObject%29%27%29%0D%0A%23%28removeAll%20%27self.splice%280%2Cself.size%28%29%29%27%29%0D%0A%23%28removeFirst%20%27self.splice%280%2C1%29%27%29%0D%0A%23%28removeIndex%3A%20%27self.splice%28index-1%2C1%29%27%29%0D%0A%23%28removeLast%20%27self.pop%28%29%27%29%0D%0A%23%28atAllPut%3A%20%27for%28var%20i%3D0%3Bi%3Cself.length%3Bi++%29%7B%20self%5Bi%5D%20%3D%20anObject%3B%20%7D%27%29%0D%0A%23%28first%20%27var%20result%3Dself%5B0%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result%27%29%0D%0A%23%28second%20%27var%20result%3Dself%5B1%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result%27%29%0D%0A%23%28third%20%27var%20result%3Dself%5B2%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result%27%29%0D%0A%23%28fourth%20%27%09var%20result%3Dself%5B3%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result%27%29%0D%0A%23%28fifth%20%27var%20result%3Dself%5B4%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result%27%29%0D%0A%23%28sixth%20%27var%20result%3Dself%5B5%5D%3B%0D%0A%09if%20%28typeof%28result%29%21%3D%22undefined%22%29%20return%20result%27%29%0D%0A%23%28asJSONObject%20%23%28%27self%27%20%27self%27%29%29%0D%0A%23%28json%20%23%28%27self%27%20%27self%27%29%29%0D%0A%29%29%0D%0A%23%28%27Array%20class%27%20%23%28%0D%0A%23%28basicNew%3A%20%27new%20Array%28size%29%27%29%0D%0A%23%28with%3A%20%27return%20%5B%20anObject%20%5D%27%29%0D%0A%23%28with%3Awith%3A%20%27return%20%5B%20firstObject%2C%20secondObject%20%5D%27%29%0D%0A%23%28with%3Awith%3Awith%3A%20%27return%20%5B%20firstObject%2C%20secondObject%2C%20thirdObject%20%5D%27%29%0D%0A%23%28with%3Awith%3Awith%3Awith%3A%20%27return%20%5B%20firstObject%2C%20secondObject%2C%20thirdObject%2C%20fourthObject%20%5D%27%29%0D%0A%23%28withAll%3A%20%27return%20%5B%5D.concat%28aCollection.asArray%28%29%29%27%29%0D%0A%23%28slice%3A%20%27Array.prototype.slice.call%28anIterable%29%27%29%0D%0A%29%29%0D%0A%23%28ComposedAssociations%20%23%28%0D%0A%23%28hashObject%20%27%7B%7D%27%29%0D%0A%29%29%0D%0A%23%28HashedSet%20%23%28%0D%0A%23%28initializeHash%20%27%7B%7D%27%29%0D%0A%23%28privateAdd%3A%20%27self%27%29%0D%0A%23%28privateRemove%3A%20%27delete%20self%5B%22@hash%22%5D%5Bcookie%5D%27%29%0D%0A%23%28includes%3A%20%27Object.prototype.hasOwnProperty.call%28self%5B%22@hash%22%5D%2Ccookie%29%27%29%0D%0A%29%29%0D%0A%23%28KeyedCollection%20%23%28%0D%0A%23%28newHash%20%27%7B%7D%27%29%0D%0A%29%29%0D%0A%23%28%27KeyedCollection%20class%27%20%23%28%0D%0A%23%28@%20%27parts%3DObject.getOwnPropertyNames%28jsObject%29%27%29%0D%0A%29%29%0D%0A%23%28%27Dictionary%20class%27%20%23%28%0D%0A%23%28basicNew%20%27self%27%29%0D%0A%29%29%0D%0A%23%28PoolDictionary%20%23%28%0D%0A%23%28validKey%3A%20%27self%27%29%0D%0A%23%28at%3Aput%3A%20%27self%27%29%0D%0A%23%28removeKey%3AifAbsent%3A%20%27delete%20self%5B%22@hash%22%5D%5BaKey%5D%27%29%0D%0A%23%28at%3AifAbsent%3A%20%27var%20found%20%3D%20Object.prototype.hasOwnProperty.call%28self%5B%22@hash%22%5D%2CaKey%29%3B%0D%0A%09if%28found%29%20return%20self%5B%22@hash%22%5D%5BaKey%5D%3B%0D%0A%09return%20aBlock.value%28%29%27%29%0D%0A%23%28includesKey%3A%20%27Object.prototype.hasOwnProperty.call%28self%5B%22@hash%22%5D%2CaKey%29%27%29%0D%0A%23%28keys%20%27return%20Object.getOwnPropertyNames%28self%5B%22@hash%22%5D%29%27%29%0D%0A%29%29%0D%0A%23%28%27RegularExpression%20class%27%20%23%28%0D%0A%23%28fromString%3Aflag%3A%20%27new%20RegExp%28aString%2C%20anotherString%29%27%29%0D%0A%29%29%0D%0A%23%28Exception%20%23%28%0D%0A%23%28captureStack%20%27self%27%29%0D%0A%23%28signal%20%27throw%28self%29%27%29%0D%0A%29%29%0D%0A%23%28ClassBuilder%20%23%28%0D%0A%23%28addSubclassOf%3Anamed%3AinstanceVariableNames%3A%20%27%0D%0A%09smalltalk.addClass%28aString%2C%20aClass%2C%20aCollection%29%3B%0D%0A%09return%20smalltalk%5BaString%5D%27%29%0D%0A%29%29%0D%0A%23%28StringStream%20%23%28%0D%0A%23%28peek%20%27return%20self%5B%22@collection%22%5D.charAt%28self.position%28%29%29%7C%7Cnil%27%29%0D%0A%23%28peekForAll%3A%20%27var%20start%20%3D%20self.position%28%29%3B%0D%0A%09for%20%28var%20i%3D0%3Bi%3CaCollection.length%3B%20i++%29%7B%0D%0A%09%09if%20%28self%5B%22@collection%22%5D.charCodeAt%28start+i%29%20%21%3D%20aCollection.charCodeAt%28i%29%29%20return%20false%3B%0D%0A%09%7D%27%29%0D%0A%29%29%0D%0A%23%28Random%20%23%28%0D%0A%23%28next%20%27return%20Math.random%28%29%27%29%0D%0A%29%29%0D%0A%23%28Rectangle%20%23%28%0D%0A%23%28printString%20%27self%27%29%0D%0A%29%29%0D%0A%09%29%20"));
smalltalk.bind(smalltalk.JSEncoder.$klass,"initializeNativeInlines:",0
,function(pool){var self=this;
return self.register_into_(self.nativeInlines(),pool);}
,"inlines",unescape("initializeNativeInlines%3A%20pool%0D%0A%09%22%20Private%20-%20Initialize%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5Eself%20register%3A%20self%20nativeInlines%20into%3A%20pool%20"));
smalltalk.bind(smalltalk.JSEncoder.$klass,"nativeInlines",0
,function(){var self=this;
return [["NativeObject", [["jsObject", "Object"], ["ownPropertyNames", unescape("return%20Object.getOwnPropertyNames%28self%5B%22@handle%22%5D%29")]]], ["NativeObject class", [["undefined", "undefined"], ["codeToNumber", unescape("%22%20%7C%7C%200%22")], ["newInstanceOf:withAll:", "self"]]], ["NativeHandle", [["hasOwnProperty:", unescape("Object.prototype.hasOwnProperty.call%28self%5B%22@handle%22%5D%2CaName%29")], ["delete:", unescape("delete%20self%5B%22@handle%22%5D%5BaName%5D")]]], ["NativeLibraryObject class", [["basicLoad:", unescape("load%28url%29")]]], ["NativeLibraryObject", [["newHandleOf:", unescape("result%20%3D%20new%20aConstructor%28%29")], ["newHandleOf:with:", unescape("result%20%3D%20new%20aConstructor%28arg1%29")], ["newHandleOf:with:with:", unescape("result%20%3D%20new%20aConstructor%28arg1%2Carg2%29")], ["newHandleOf:with:with:with:", unescape("result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%29")], ["newHandleOf:with:with:with:with:", unescape("result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%29")], ["newHandleOf:with:with:with:with:with:", unescape("result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%29")], ["newHandleOf:with:with:with:with:with:with:", unescape("result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%29")], ["newHandleOf:with:with:with:with:with:with:with:", unescape("result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%2Carg7%29")], ["newHandleOf:with:with:with:with:with:with:with:with:", unescape("result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%2Carg7%2Carg8%29")]]], ["Array", [["addAllLua:", unescape("self.addAll_%28natives%29")]]], ["Array class", [[unescape("@@"), "self"], [unescape("@"), unescape("result.addAll_%28natives%29")], ["basicNew", "self"]]], ["Boolean class", [[unescape("@"), unescape("%0D%0A%09if%20%28aBOOL%20%3D%3D%3D%20true%29%20return%20true%3B%0D%0A%09if%20%28aBOOL%20%3D%3D%3D%201%29%20return%20true%3B%0D%0A%09if%20%28%281%20%26%20aBOOL%29%20+%200%20%3D%3D%201%29%20return%20true")]]], ["Number", [["int32", "self"], ["int64", "self"], ["float32", "self"], ["float64", "self"]]]];}
,"inlines",unescape("nativeInlines%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5E%23%28%0D%0A%23%28NativeObject%20%23%28%0D%0A%23%28jsObject%20%27Object%27%29%0D%0A%23%28ownPropertyNames%20%27return%20Object.getOwnPropertyNames%28self%5B%22@handle%22%5D%29%27%29%0D%0A%29%29%0D%0A%23%28%27NativeObject%20class%27%20%23%28%0D%0A%23%28undefined%20%27undefined%27%29%0D%0A%23%28codeToNumber%20%27%22%20%7C%7C%200%22%27%29%0D%0A%23%28newInstanceOf%3AwithAll%3A%20%27self%27%29%0D%0A%29%29%0D%0A%23%28NativeHandle%20%23%28%0D%0A%23%28hasOwnProperty%3A%20%27Object.prototype.hasOwnProperty.call%28self%5B%22@handle%22%5D%2CaName%29%27%29%0D%0A%23%28delete%3A%20%27delete%20self%5B%22@handle%22%5D%5BaName%5D%27%29%0D%0A%29%29%0D%0A%23%28%27NativeLibraryObject%20class%27%20%23%28%0D%0A%23%28basicLoad%3A%20%27load%28url%29%27%29%0D%0A%29%29%0D%0A%23%28NativeLibraryObject%20%20%23%28%0D%0A%23%28newHandleOf%3A%20%27result%20%3D%20new%20aConstructor%28%29%27%29%0D%0A%23%28newHandleOf%3Awith%3A%20%27result%20%3D%20new%20aConstructor%28arg1%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3A%20%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%2Carg7%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%2Carg7%2Carg8%29%27%29%0D%0A%29%29%0D%0A%23%28Array%20%23%28%0D%0A%23%28addAllLua%3A%20%27self.addAll_%28natives%29%27%29%0D%0A%29%29%0D%0A%23%28%27Array%20class%27%20%23%28%0D%0A%23%28@@%20%27self%27%29%0D%0A%23%28@%20%27result.addAll_%28natives%29%27%29%0D%0A%23%28basicNew%20%27self%27%29%0D%0A%29%29%0D%0A%23%28%27Boolean%20class%27%20%23%28%0D%0A%23%28@%20%27%0D%0A%09if%20%28aBOOL%20%3D%3D%3D%20true%29%20return%20true%3B%0D%0A%09if%20%28aBOOL%20%3D%3D%3D%201%29%20return%20true%3B%0D%0A%09if%20%28%281%20%26%20aBOOL%29%20+%200%20%3D%3D%201%29%20return%20true%27%29%0D%0A%29%29%0D%0A%23%28Number%20%23%28%0D%0A%23%28int32%20%27self%27%29%0D%0A%23%28int64%20%27self%27%29%0D%0A%23%28float32%20%27self%27%29%0D%0A%23%28float64%20%27self%27%29%0D%0A%29%29%0D%0A%09%29%20"));
smalltalk.bind(smalltalk.JSEncoder.$klass,"initializeToolsInlines:",0
,function(pool){var self=this;
return self.register_into_(self.toolsInlines(),pool);}
,"inlines",unescape("initializeToolsInlines%3A%20pool%0D%0A%09%22%20Private%20-%20Initialize%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5Eself%20register%3A%20self%20toolsInlines%20into%3A%20pool%20"));
smalltalk.bind(smalltalk.JSEncoder.$klass,"toolsInlines",0
,function(){var self=this;
return [["NativeReader", [["initializeObject", unescape("%7B%7D")]]], ["String", [["asComment", [unescape("%22/*%20%22"), unescape("%22*/%20%22"), unescape("%22//%20%22")]]]], ["MethodReferences", [["dot", unescape("%22.%22")], ["localDef", unescape("%22var%20%22")], ["normalSendStarter:", "self"], ["superSendPatternOf:", unescape("self.superJSSendPatternOf_%28aSelector%29")], ["compiledSourceStart", unescape("%22%7B%22")]]], ["Class", [["hackDNU", unescape("mth.fn_%28function%28%29%7B%0D%0A%09%09this.print_%28%22//DNU%20%22+selector%29%3B%0D%0A%09%09this.doesNotUnderstand_%28%0D%0A%09%09%09smalltalk.Message.receiver_selector_arguments_%0D%0A%09%09%09%09%28this%2Cselector%2C%5B%5D.slice.call%28arguments%29%29%29%7D%0D%0A%09%09%29%3B")]]]];}
,"inlines",unescape("toolsInlines%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5E%23%28%0D%0A%23%28NativeReader%20%23%28%0D%0A%23%28initializeObject%20%27%7B%7D%27%29%0D%0A%29%29%0D%0A%23%28String%20%20%23%28%0D%0A%23%28asComment%20%23%28%27%22/*%20%22%27%20%09%27%22*/%20%22%27%20%09%27%22//%20%22%27%29%29%0D%0A%29%29%0D%0A%23%28MethodReferences%20%23%28%0D%0A%23%28dot%09%09%27%22.%22%27%29%0D%0A%23%28localDef%09%27%22var%20%22%27%29%0D%0A%23%28normalSendStarter%3A%09%27self%27%29%0D%0A%23%28superSendPatternOf%3A%09%27self.superJSSendPatternOf_%28aSelector%29%27%29%0D%0A%23%28compiledSourceStart%09%27%22%7B%22%27%29%0D%0A%29%29%0D%0A%23%28Class%20%23%28%0D%0A%23%28hackDNU%20%27mth.fn_%28function%28%29%7B%0D%0A%09%09this.print_%28%22//DNU%20%22+selector%29%3B%0D%0A%09%09this.doesNotUnderstand_%28%0D%0A%09%09%09smalltalk.Message.receiver_selector_arguments_%0D%0A%09%09%09%09%28this%2Cselector%2C%5B%5D.slice.call%28arguments%29%29%29%7D%0D%0A%09%09%29%3B%27%29%0D%0A%29%29%0D%0A%09%29%20"));
smalltalk.bind(smalltalk.JSEncoder.$klass,"initializeSnapshotInlines:",0
,function(pool){var self=this;
return self.register_into_(self.snapshotInlines(),pool);}
,"inlines",unescape("initializeSnapshotInlines%3A%20pool%0D%0A%09%22%20Private%20-%20Initialize%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5Eself%20register%3A%20self%20snapshotInlines%20into%3A%20pool%20"));
smalltalk.bind(smalltalk.JSEncoder.$klass,"snapshotInlines",0
,function(){var self=this;
return [["Smalltalk", [["exit", unescape("quit%28%29")]]], ["Snapshot", [["bootHeader", unescape("%22%22")], ["bootFooter", unescape("%22%22")], ["launchExpression", unescape("%22smalltalk.launchImage%28%29%22")]]]];}
,"inlines",unescape("snapshotInlines%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5E%23%28%0D%0A%23%28Smalltalk%20%23%28%0D%0A%23%28exit%09%27quit%28%29%27%29%0D%0A%29%29%0D%0A%23%28Snapshot%20%23%28%0D%0A%23%28bootHeader%09%27%22%22%27%29%0D%0A%23%28bootFooter%09%27%22%22%27%29%0D%0A%23%28launchExpression%20%27%22smalltalk.launchImage%28%29%22%27%29%0D%0A%29%29%0D%0A%09%29%20"));
smalltalk.bind(smalltalk.JSEncoder.$klass,"initializeOtherInlines:",0
,function(pool){var self=this;
return self.register_into_(self.otherInlines(),pool);}
,"inlines",unescape("initializeOtherInlines%3A%20pool%0D%0A%09%22%20Private%20-%20Initialize%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5Eself%20register%3A%20self%20otherInlines%20into%3A%20pool%20"));
smalltalk.bind(smalltalk.JSEncoder.$klass,"otherInlines",0
,function(){var self=this;
return [];}
,"inlines",unescape("otherInlines%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5E%23%28%0D%0A%22%23%28ClassName%20%23%28%0D%0A%23%28selector%20%27...%27%29%0D%0A%29%29%22%0D%0A%09%29%20"));
smalltalk.bind(smalltalk.JSEncoder,"nativeLanguage",0
,function(){var self=this;
return "js";}
,"accessing",unescape("nativeLanguage%0D%0A%09%22%20Return%20the%20nativeLanguage%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23js%20"));
smalltalk.bind(smalltalk.JSEncoder,"nativeUndefined",0
,function(){var self=this;
return "undefined";}
,"encoding",unescape("nativeUndefined%0D%0A%09%22%20Private%20-%20Return%20the%20code%20for%20undefined.%20%22%0D%0A%0D%0A%09%5E%23undefined%20"));
smalltalk.bind(smalltalk.JSEncoder,"nativeNameOfMetaClass:",0
,function(aClass){var self=this;
return aClass.instanceClass().name()._comma(".$klass");}
,"encoding",unescape("nativeNameOfMetaClass%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20native%20name%20for%20metaclass%20aClass.%20%22%0D%0A%0D%0A%09%5EaClass%20instanceClass%20name%20%2C%27.%24klass%27%20"));
smalltalk.bind(smalltalk.JSEncoder,"missingNativeCodeFor:as:",0
,function(aNode,lang){var self=this;
return unescape("%28self.invalidMessage_%28%22")._comma(self["@compiler"].selector())._comma(unescape("%22%29%29"));}
,"inlines",unescape("missingNativeCodeFor%3A%20aNode%20as%3A%20lang%0D%0A%09%22%20Private%20-%20Return%20the%20native%20language%20code%20for%20missing%20expression%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5E%27%28self.invalidMessage_%28%22%27%20%2Ccompiler%20selector%20%2C%27%22%29%29%27"));
smalltalk.bind(smalltalk.JSEncoder,"defaultRequirementsFor:",0
,function(aClass){var self=this;
var constructorName=nil;
var code=nil;
constructorName=aClass.constructorName();
if ((nil.isNil_(constructorName))){return [];};;
return smalltalk.Array.with_(unescape("if%28")._comma(constructorName)._comma(unescape("%20%26%26%20%21"))._comma(self.smalltalkNameFor_(aClass))._comma(unescape("%29%20smalltalk.mapClassName%28%22"))._comma(aClass.name())._comma(unescape("%22%2C%22"))._comma(aClass.category())._comma(unescape("%22%2C"))._comma(constructorName)._comma(unescape("%2C"))._comma(self.smalltalkNameFor_(aClass.superclass()))._comma(unescape("%29%3B")));}
,"export",unescape("defaultRequirementsFor%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20default%20requirements%20to%20load%20aClass.%0D%0A%09Detect%20native%20bindings%20and%20generate%20code%20to%20rebind%20before%20class%20definition.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20constructorName%20code%20%7C%0D%0A%09constructorName%20%3A%3D%20aClass%20constructorName.%0D%0A%09constructorName%20isNil%20ifTrue%3A%20%5B%20%5E%23%28%29%20%5D.%0D%0A%09%5EArray%20with%3A%0D%0A%09%09%27if%28%27%2CconstructorName%2C%27%20%26%26%20%21%27%0D%0A%09%09%2C%20%28self%20smalltalkNameFor%3A%20aClass%29%0D%0A%09%09%2C%27%29%20smalltalk.mapClassName%28%22%27%0D%0A%09%09%2CaClass%20name%20%2C%27%22%2C%22%27%0D%0A%09%09%2CaClass%20category%2C%20%27%22%2C%27%0D%0A%09%09%2CconstructorName%2C%20%27%2C%27%0D%0A%09%09%2C%28self%20smalltalkNameFor%3A%20aClass%20superclass%29%0D%0A%09%09%2C%27%29%3B%27"));
smalltalk.bind(smalltalk.JSEncoder,"emmitEnterMethod:",0
,function(aMethodNode){var self=this;
(function($rec){$rec.nextPutAll_(unescape("var%20self%3Dthis%3B"));return $rec.cr();})(self);
return self;}
,"generation",unescape("emmitEnterMethod%3A%20aMethodNode%0D%0A%09%22%20Private%20-%20Generate%20home%20context%20definitions%2C%20entering%20aMethodNode.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20%27var%20self%3Dthis%3B%27%3B%20cr%20"));
smalltalk.bind(smalltalk.JSEncoder,"emmitFunctionBody:",0
,function(aBlock){var self=this;
self.nextPutAll_(unescape("%7B"));
aBlock.evaluate();
self.nextPutAll_(unescape("%7D"));
return self;}
,"generation",unescape("emmitFunctionBody%3A%20aBlock%0D%0A%09%22%20Private%20-%20Emmit%20the%20code%20of%20function%20body%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20%24%7B.%0D%0A%09aBlock%20evaluate.%0D%0A%09self%20nextPutAll%3A%20%24%7D.%20"));
smalltalk.bind(smalltalk.JSEncoder,"emmitFunction:node:arguments:doing:",0
,function(fName,aNode,argumentNames,aBlock){var self=this;
var aStream=nil;
aStream="".writeStream();
argumentNames.do_separatedBy_((function(each){self.tempVariables().add_(each);return aStream.nextPutAll_(self.localName_(each));}),(function(){return aStream.nextPutAll_(unescape("%2C"));}));
self.emmitFunction_arguments_doing_(fName,aStream.contents(),aBlock);
return self;}
,"generation",unescape("emmitFunction%3A%20fName%20node%3A%20aNode%20arguments%3A%20argumentNames%20doing%3A%20aBlock%0D%0A%09%22%20Private%20-%20Generate%20function%20for%20aNode.%20%22%0D%0A%0D%0A%09%7C%20aStream%20%7C%0D%0A%09aStream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09argumentNames%20do%3A%20%5B%3Aeach%20%7C%20%0D%0A%09%09self%20tempVariables%20add%3A%20each.%0D%0A%09%09aStream%20nextPutAll%3A%20%28self%20localName%3A%20each%29%20%5D%0D%0A%09%20%20%20%20separatedBy%3A%20%5B%20aStream%20nextPutAll%3A%20%27%2C%27%20%5D.%0D%0A%09self%09emmitFunction%3A%20fName%0D%0A%09%09arguments%3A%20aStream%20contents%0D%0A%09%09doing%3A%20aBlock"));
smalltalk.bind(smalltalk.JSEncoder,"export:method:on:",0
,function(aClass,aMethod,aStream){var self=this;
var fName=nil;
var selectorName=nil;
var bar=nil;
var source=nil;
source=self.nativeSourceFor_(aMethod);
if ((nil.isNil_(source))){return false;};;
if ((nil.isNil_(aMethod))){self.error_("Method is missing");};;
bar=unescape("%22").asString();
fName=aMethod.selector();
if (fName.first().isBackSlash()){selectorName=self.escaped_(aMethod.selector());
fName=self.escaped_(self.asSelector_(fName));} else{selectorName=bar._comma(aMethod.selector())._comma(bar);
((self.asSelector_(fName)._eq(aMethod.selector().replace_with_(":","_"))).mustBeBoolean()==true ? (function(){return fName="0";})() : (function(){return fName=bar._comma(self.asSelector_(fName))._comma(bar);})());};;
aStream.nextPutAll_(unescape("smalltalk.bind%28")._comma(self.smalltalkNameFor_(aClass))._comma(unescape("%2C"))._comma(selectorName)._comma(unescape("%2C"))._comma(fName)._comma(unescape("%0D%0A%2C"))._comma(source));
if ((nil.isNil_(aMethod.source())===false).and_((function(){return aMethod.source().notEmpty();}))){(function($rec){$rec.cr();return $rec.nextPutAll_(unescape("%2C")._comma(self.escaped_(aMethod.category()))._comma(unescape("%2C"))._comma(self.escaped_(aMethod.source())));})(aStream);};;
if (smalltalk.responds_to_(aMethod,("stamp")).and_((function(){return (nil.isNil_(aMethod.stamp())===false);}))){aStream.nextPutAll_(unescape("%2C")._comma(self.escaped_(aMethod.stamp())));};;
aStream.nextPutAll_(unescape("%29%3B%0D%0A"));
return self;}
,"export",unescape("export%3A%20aClass%20method%3A%20aMethod%20on%3A%20aStream%0D%0A%09%22%20Export%20a%20method%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%7C%20fName%20selectorName%20bar%20source%20%7C%0D%0A%09source%20%3A%3D%20self%20nativeSourceFor%3A%20aMethod.%0D%0A%09source%20isNil%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09aMethod%20isNil%20ifTrue%3A%20%5B%20self%20error%3A%20%27Method%20is%20missing%27%20%5D.%0D%0A%09bar%20%3A%3D%20%24%22%20asString.%0D%0A%09fName%20%3A%3D%20aMethod%20selector.%0D%0A%09fName%20first%20isBackSlash%20ifTrue%3A%20%5B%0D%0A%09%09selectorName%20%3A%3D%20self%20escaped%3A%20aMethod%20selector.%0D%0A%09%09fName%20%3A%3D%20self%20escaped%3A%20%28self%20asSelector%3A%20fName%29.%0D%0A%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09selectorName%20%3A%3D%20bar%20%2CaMethod%20selector%2C%20bar.%0D%0A%09%09%28self%20asSelector%3A%20fName%29%20%3D%20%28aMethod%20selector%20replace%3A%20%24%3A%20with%3A%20%24_%29%20ifTrue%3A%20%5B%0D%0A%09%09%09fName%20%3A%3D%20%270%27%20%22we%20exploit%20the%20fact%20that%200%20isNil%20to%20signal%20a%20compact%20selector%22%0D%0A%09%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09fName%20%3A%3D%20bar%20%2C%28self%20asSelector%3A%20fName%29%20%2Cbar%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09aStream%0D%0A%09%09nextPutAll%3A%20%27smalltalk.bind%28%27%20%2C%28self%20smalltalkNameFor%3A%20aClass%29%0D%0A%09%09%09%2C%27%2C%27%20%2CselectorName%20%2C%27%2C%27%20%2CfName%20%2C%27%0D%0A%2C%27%09%09%09%2Csource.%0D%0A%09%28aMethod%20source%20notNil%20and%3A%20%5B%20aMethod%20source%20notEmpty%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20aStream%20cr%3B%20%22warning%3A%20this%20cr%20is%20important%20for%20debugger%22%0D%0A%09%09nextPutAll%3A%20%27%2C%27%20%2C%28self%20escaped%3A%20aMethod%20category%29%0D%0A%09%09%09%2C%27%2C%27%20%2C%28self%20escaped%3A%20aMethod%20source%29.%0D%0A%09%5D.%0D%0A%09%28%28aMethod%20respondsTo%3A%20%23stamp%29%20and%3A%20%5B%20aMethod%20stamp%20notNil%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09aStream%20nextPutAll%3A%20%27%2C%27%20%2C%28self%20escaped%3A%20aMethod%20stamp%29%0D%0A%09%5D.%0D%0A%09aStream%20nextPutAll%3A%20%27%29%3B%0D%0A%27"));
smalltalk.bind(smalltalk.JSEncoder,"exportCommentOf:on:",0
,function(aClass,aStream){var self=this;
if (aClass.comment().size()._gt((0))){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));$rec.nextPutAll_(unescape(".%24comment%3D")._comma(self.escaped_(aClass.comment())));$rec.nextPutAll_(unescape("%3B"));return $rec.cr();})(aStream);};;
return self;}
,"export",unescape("exportCommentOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20the%20comment%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09aClass%20comment%20size%20%3E%200%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20aStream%0D%0A%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27.%24comment%3D%27%20%2C%28self%20escaped%3A%20aClass%20comment%29%3B%0D%0A%09%09nextPutAll%3A%20%27%3B%27%3B%20cr%0D%0A%09%5D.%20"));
smalltalk.bind(smalltalk.JSEncoder,"exportDefinitionOf:on:",0
,function(aClass,aStream){var self=this;
self.loadRequirementsOf_(aClass).do_((function(each){return ((each.isString()).mustBeBoolean()==true ? (function(){return (function($rec){$rec.nextPutAll_(each);return $rec.cr();})(aStream);})() : (function(){return aStream.fileOutOn_(aStream);})());}));
(function($rec){$rec.nextPutAll_(unescape("smalltalk.addClass%28%22")._comma(aClass.name())._comma(unescape("%22%2C")));$rec.nextPutAll_(self.smalltalkNameFor_(aClass.superclass()));return $rec.nextPutAll_(unescape("%2C"));})(aStream);
self.exportArrayLiteral_on_newLine_(aClass.instanceVariableNames(),aStream,false);
(function($rec){$rec.nextPutAll_(unescape("%2C%22")._comma(aClass.category())._comma(unescape("%22")));$rec.nextPutAll_(unescape("%29%3B"));return $rec.cr();})(aStream);
if ((nil.isNil_(aClass.classVariables())===false)){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));$rec.nextPutAll_(unescape(".%24classVariableNames%3D")._comma(self.escaped_(aClass.classVariablesString())));$rec.nextPutAll_(unescape("%3B"));return $rec.cr();})(aStream);};;
if ((nil.isNil_(aClass.poolDictionaries())===false)){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));return $rec.nextPutAll_(unescape(".%24poolDictionaries%3D"));})(aStream);
self.exportArrayLiteral_on_newLine_(aClass.poolDictionaries(),aStream,true);};;
if (aClass.propertyNames().notEmpty()){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));return $rec.nextPutAll_(unescape(".%24propertyNames%3D"));})(aStream);
self.exportArrayLiteral_on_newLine_(aClass.propertyNames(),aStream,true);};;
self.exportCommentOf_on_(aClass,aStream);
return self;}
,"export",unescape("exportDefinitionOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%28self%20loadRequirementsOf%3A%20aClass%29%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20isString%0D%0A%09%09ifTrue%3A%20%5B%20aStream%20nextPutAll%3A%20each%3Bcr%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20aStream%20fileOutOn%3A%20aStream%20%5D.%0D%0A%09%5D.%0D%0A%09aStream%20%0D%0A%09%20%20%20%20nextPutAll%3A%20%27smalltalk.addClass%28%22%27%2C%20aClass%20name%2C%20%27%22%2C%27%3B%0D%0A%09%20%20%20%20nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%20superclass%29%3B%0D%0A%09%20%20%20%20nextPutAll%3A%20%27%2C%27.%0D%0A%09self%20exportArrayLiteral%3A%20aClass%20instanceVariableNames%20on%3A%20aStream%20newLine%3A%20false.%0D%0A%09aStream%0D%0A%09%20%20%20%20nextPutAll%3A%20%27%2C%22%27%20%2CaClass%20category%2C%20%24%22%3B%0D%0A%09%20%20%20%20nextPutAll%3A%20%27%29%3B%27%3B%20cr.%0D%0A%09aClass%20classVariables%20notNil%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20aStream%0D%0A%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27.%24classVariableNames%3D%27%20%2C%28self%20escaped%3A%20aClass%20classVariablesString%29%3B%0D%0A%09%09nextPutAll%3A%20%27%3B%27%3B%20cr%0D%0A%09%5D.%0D%0A%09aClass%20poolDictionaries%20notNil%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20aStream%0D%0A%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27.%24poolDictionaries%3D%27.%0D%0A%09%20%20%20self%20exportArrayLiteral%3A%20aClass%20poolDictionaries%20on%3A%20aStream%20newLine%3A%20true.%0D%0A%09%5D.%0D%0A%09aClass%20propertyNames%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20aStream%0D%0A%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27.%24propertyNames%3D%27.%0D%0A%09%20%20%20self%20exportArrayLiteral%3A%20aClass%20propertyNames%20on%3A%20aStream%20newLine%3A%20true.%0D%0A%09%5D.%0D%0A%09self%20exportCommentOf%3A%20aClass%20on%3A%20aStream."));
smalltalk.bind(smalltalk.JSEncoder,"exportMetaDefinitionOf:on:",0
,function(aClass,aStream){var self=this;
if (aClass.$klass.instanceVariableNames().notEmpty()){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass.$klass));return $rec.nextPutAll_(unescape(".%24iVarNames%3D"));})(aStream);
self.exportArrayLiteral_on_newLine_(aClass.$klass.instanceVariableNames(),aStream,true);};;
if (aClass.$klass.propertyNames().notEmpty()){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass.$klass));return $rec.nextPutAll_(unescape(".%24propertyNames%3D"));})(aStream);
self.exportArrayLiteral_on_newLine_(aClass.$klass.propertyNames(),aStream,true);};;
return self;}
,"export",unescape("exportMetaDefinitionOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20class%20side%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09aClass%20class%20instanceVariableNames%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%09aStream%20%0D%0A%09%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%20class%29%3B%0D%0A%09%09%09nextPutAll%3A%20%27.%24iVarNames%3D%27.%0D%0A%09%09self%20exportArrayLiteral%3A%20aClass%20class%20instanceVariableNames%20on%3A%20aStream%20newLine%3A%20true.%0D%0A%09%5D.%0D%0A%0D%0A%09aClass%20class%20propertyNames%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%09aStream%20%0D%0A%09%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%20class%29%3B%0D%0A%09%09%09nextPutAll%3A%20%27.%24propertyNames%3D%27.%0D%0A%09%09self%20exportArrayLiteral%3A%20aClass%20class%20propertyNames%20on%3A%20aStream%20newLine%3A%20true.%0D%0A%09%5D."));
smalltalk.bind(smalltalk.JSEncoder,"sourceToTry:",0
,function(sourceCode){var self=this;
return "try"._comma(unescape("%7B"))._comma(sourceCode)._comma(unescape("%7D"))._comma(unescape("%20catch%28%24%24e%29%20%7B%0D%0A%09if%20%28%24%24e.directiveError%29%20%7B%20return%28this.error_%28%24%24e.directiveError%29%29%3B%20%7D%0D%0A%09if%20%28%21%24%24e.directiveFailed%29%20%7B%20throw%28%24%24e%29%3B%20%7D%20%7D"));}
,"directives",unescape("sourceToTry%3A%20sourceCode%0D%0A%09%22%20Return%20the%20code%20to%20emmit%20to%20try%20execution%20of%20sourceCode.%20%22%0D%0A%0D%0A%09%5E%27try%27%20%2C%24%7B%20%2CsourceCode%20%2C%24%7D%20%2C%27%20catch%28%24%24e%29%20%7B%0D%0A%09if%20%28%24%24e.directiveError%29%20%7B%20return%28this.error_%28%24%24e.directiveError%29%29%3B%20%7D%0D%0A%09if%20%28%21%24%24e.directiveFailed%29%20%7B%20throw%28%24%24e%29%3B%20%7D%20%7D%27"));
smalltalk.bind(smalltalk.JSEncoder,"sourceToDirectCall:with:",0
,function(nativeSelector,arg){var self=this;
return "return this."._comma(nativeSelector)._comma(unescape("%28"))._comma(arg)._comma(unescape("%2C"))._comma("arguments")._comma(unescape("%29%3B"));}
,"directives",unescape("sourceToDirectCall%3A%20nativeSelector%20with%3A%20arg%0D%0A%09%22%20Return%20the%20code%20to%20emmit%20to%20direct%20native%20call%20to%20selector%20with%20argument.%20%22%0D%0A%0D%0A%09%5E%27return%20this.%27%20%2CnativeSelector%20%2C%27%28%27%20%2Carg%20%2C%27%2C%27%20%2C%27arguments%27%20%2C%27%29%3B%27%20"));
smalltalk.bind(smalltalk.JSEncoder,"generate:early:return:",0
,function(fName,early,code){var self=this;
var prev=nil;
if (!(early)){self.nextPutAll_(code);
return (0);};;
prev=unescape("var%20%24s8Ret%24%3D%7Bname%3A%22stReturn%22%7D%3Btry")._comma(unescape("%7B"));
(function($rec){$rec.nextPutAll_(prev);$rec.nextPutAll_(code);return $rec.nextPutAll_(unescape("%7D")._comma(unescape("%20catch%28%24%24ex%29%20%7Bif%28%24%24ex%20%3D%3D%3D%20%24s8Ret%24%29"))._comma(unescape("%7B")));})(self);
self.emmitReturn_early_("$$ex.result",true);
self.nextPutAll_(unescape("%7D")._comma(unescape("%20throw%28%24%24ex%29%3B"))._comma(unescape("%7D")));
return prev.size();}
,"generation",unescape("generate%3A%20fName%20early%3A%20early%20return%3A%20code%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20catch%20early%20return%20of%20code%20expression%20in%20fName.%20%22%0D%0A%0D%0A%09%7C%20prev%20%7C%0D%0A%09early%20ifFalse%3A%20%5B%20self%20nextPutAll%3A%20code.%20%5E0%20%5D.%0D%0A%09prev%20%3A%3D%20%27var%20%24s8Ret%24%3D%7Bname%3A%22stReturn%22%7D%3Btry%27%20%2C%24%7B.%0D%0A%09self%09nextPutAll%3A%20prev%3B%0D%0A%09%09nextPutAll%3A%20code%3B%0D%0A%09%09nextPutAll%3A%20%24%7D%20%2C%27%20catch%28%24%24ex%29%20%7Bif%28%24%24ex%20%3D%3D%3D%20%24s8Ret%24%29%27%20%2C%24%7B.%0D%0A%09self%20emmitReturn%3A%20%27%24%24ex.result%27%20early%3A%20true.%0D%0A%09self%20nextPutAll%3A%20%24%7D%20%2C%27%20throw%28%24%24ex%29%3B%27%20%2C%24%7D.%0D%0A%09%5Eprev%20size"));
smalltalk.bind(smalltalk.JSEncoder,"newMethodFunctionName",0
,function(){var self=this;
return "smalltalk.newMethod";}
,"generation",unescape("newMethodFunctionName%0D%0A%09%22%20Private%20-%20Return%20the%20function%20to%20call%20when%20installing%20a%20new%20method.%20%22%0D%0A%0D%0A%09%5E%27smalltalk.newMethod%27"));
smalltalk.bind(smalltalk.JSEncoder,"visitBlockNode:",0
,function(aNode){var self=this;
self.nextArgumentsPut_((function(){return self.emmitFunction_node_arguments_doing_("",aNode,aNode.parameterNames(),(function(){return self.visitNodesIn_cr_(aNode,false);}));}));
return self;}
,"visiting",unescape("visitBlockNode%3A%20aNode%0D%0A%0D%0A%09self%20nextArgumentsPut%3A%20%5B%0D%0A%09%09self%09emmitFunction%3A%20%27%27%20node%3A%20aNode%0D%0A%09%09%09arguments%3A%20aNode%20parameterNames%0D%0A%09%09%09doing%3A%20%5B%20self%20visitNodesIn%3A%20aNode%20cr%3A%20false%20%5D%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.JSEncoder,"visitCascadeNode:",0
,function(aNode){var self=this;
if (!(self.tempVariables().includes_("$rec"))){self.tempVariables().add_("$rec");};;
self.compilerEvent_with_("aboutCascade:",aNode);
self.nextArgumentsPut_((function(){return self.emmitFunction_arguments_doing_("","$rec",(function(){var index=nil;
var rec=nil;
index=(0);return aNode.nodes().do_((function(each){index=index._plus((1));if (index._eq(aNode.nodes().size())){self.emmitReturn_(nil);};;rec=smalltalk.VariableNode.$$new();(function($rec){$rec.value_("$rec");return $rec.start_end_(each.start(),each.start());})(rec);each.receiver_(rec);self.visit_(each);if (index._eq(aNode.nodes().size())){self.compilerEvent_withArguments_("returned:early:",smalltalk.Array.with_with_(nil,false));};;return self.nextPutAll_(unescape("%3B"));}));}));}));
self.nextArgumentsPut_((function(){return self.visit_(aNode.receiver());}));
self.compilerEvent_with_("cascade:",aNode);
return self;}
,"visiting",unescape("visitCascadeNode%3A%20aNode%0D%0A%0D%0A%09%28self%20tempVariables%20includes%3A%20%27%24rec%27%29%20ifFalse%3A%20%5B%20self%20tempVariables%20add%3A%20%27%24rec%27%20%5D.%0D%0A%09self%20compilerEvent%3A%20%23aboutCascade%3A%20with%3A%20aNode.%0D%0A%09self%20nextArgumentsPut%3A%20%5B%0D%0A%09%09self%20emmitFunction%3A%20%27%27%20arguments%3A%20%27%24rec%27%20doing%3A%20%5B%20%7C%20index%20rec%20%7C%0D%0A%09%09%09index%20%3A%3D%200.%0D%0A%09%09%09aNode%20nodes%20do%3A%20%5B%3Aeach%20%7C%0D%0A%09%09%09%20%20%20%20index%20%3A%3D%20index%20+%201.%0D%0A%09%09%09%20%20%20%20index%20%3D%20aNode%20nodes%20size%20ifTrue%3A%20%5B%20self%20emmitReturn%3A%20nil%20%5D.%0D%0A%09%09%09%20%20%20%20rec%20%3A%3D%20VariableNode%20new.%0D%0A%09%09%09%20%20%20%20rec%20value%3A%20%27%24rec%27%3B%20start%3A%20each%20start%20end%3A%20each%20start.%0D%0A%09%09%09%20%20%20%20each%20receiver%3A%20rec.%0D%0A%09%09%09%20%20%20%20self%20visit%3A%20each.%0D%0A%09%09%09%20%20%20%20index%20%3D%20aNode%20nodes%20size%20ifTrue%3A%20%5B%0D%0A%09%09%09%09self%20compilerEvent%3A%20%23returned%3Aearly%3A%20withArguments%3A%20%28Array%20with%3A%20nil%20with%3A%20false%29%0D%0A%09%09%09%20%20%20%20%5D.%0D%0A%09%09%09%20%20%20%20self%20nextPutAll%3A%20%27%3B%27%0D%0A%09%09%09%5D.%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09self%20nextArgumentsPut%3A%20%5B%20self%20visit%3A%20aNode%20receiver%20%5D.%0D%0A%09self%20compilerEvent%3A%20%23cascade%3A%20with%3A%20aNode."));
smalltalk.bind(smalltalk.JSEncoder,"visitClassVariableAssignment:in:to:",0
,function(aClassVariableName,aClass,anExpressionNode){var self=this;
(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));$rec.nextPutAll_(unescape(".classVariableAt_put_%28%22"));$rec.nextPutAll_(aClassVariableName);return $rec.nextPutAll_(unescape("%22%2C"));})(self);
self.visit_(anExpressionNode);
self.nextPutAll_(unescape("%29"));
return self;}
,"visiting",unescape("visitClassVariableAssignment%3A%20aClassVariableName%20in%3A%20aClass%20to%3A%20anExpressionNode%0D%0A%0D%0A%09self%0D%0A%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27.classVariableAt_put_%28%22%27%3B%0D%0A%09%09nextPutAll%3A%20aClassVariableName%3B%0D%0A%09%09nextPutAll%3A%20%27%22%2C%27.%0D%0A%09self%20visit%3A%20anExpressionNode.%0D%0A%09self%09nextPutAll%3A%20%27%29%27"));
smalltalk.bind(smalltalk.JSEncoder,"visitDirectiveNode:",0
,function(aNode){var self=this;
if (smalltalk.responds_to_(aNode,("asNative:for:on:"))){var code=nil;
code=aNode.asNative_for_on_(self["@functionName"],self,self["@stream"]);
return (((nil.isNil_(code)===false)).mustBeBoolean()==true ? (function(){return (function($rec){$rec.nextPutAll_(code);return $rec.cr();})(self);})() : nil);};;
self.visitNode_in_cr_(aNode,nil,true);
return self;}
,"visiting",unescape("visitDirectiveNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20a%20directive%20node.%20%22%0D%0A%0D%0A%09%28aNode%20respondsTo%3A%20%23asNative%3Afor%3Aon%3A%29%20ifTrue%3A%20%5B%0D%0A%09%09%7C%20code%20%7C%0D%0A%09%09code%20%3A%3D%20aNode%0D%0A%09%09%09asNative%3A%20functionName%0D%0A%09%09%09for%3A%20self%20on%3A%20stream.%0D%0A%09%09%5Ecode%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09self%20nextPutAll%3A%20code%3B%20cr%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09self%20visitNode%3A%20aNode%20in%3A%20nil%20cr%3A%20true"));
smalltalk.bind(smalltalk.JSEncoder,"visitGlobalReferenceNode:",0
,function(aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
var aName=nil;
var aClass=nil;
var lookupPools=nil;
aName=aNode.value();
aClass=self.currentClass().instanceClass();
aClass.classVariableNamesDo_((function(each){return ((each._eq(aName)).mustBeBoolean()==true ? (function(){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));$rec.nextPutAll_(unescape(".classVariableAt_%28%22"));$rec.nextPutAll_(aName);return $rec.nextPutAll_(unescape("%22%29"));})(self);return (function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();})() : nil);}));
aClass.sharedPoolsAndNamesDo_((function(poolName,pool){return ((pool.includesKey_(aName)).mustBeBoolean()==true ? (function(){(function($rec){$rec.nextPutAll_(unescape("smalltalk.poolDictionary_%28%22"));$rec.nextPutAll_(poolName);$rec.nextPutAll_(unescape("%22%29.at_%28%22"));$rec.nextPutAll_(aName);return $rec.nextPutAll_(unescape("%22%29"));})(self);return (function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();})() : nil);}));
self.nextPutAll_(self.codeToGetGlobal_(aNode.value()));
(function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"visiting",unescape("visitGlobalReferenceNode%3A%20aNode%0D%0A%0D%0A%09%7C%20aName%20aClass%20lookupPools%20%7C%0D%0A%09aName%20%3A%3D%20aNode%20value.%0D%0A%09aClass%20%3A%3D%20self%20currentClass%20instanceClass.%0D%0A%09aClass%20classVariableNamesDo%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20%3D%20aName%20ifTrue%3A%20%5B%0D%0A%09%09%09self%0D%0A%09%09%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09%09%09nextPutAll%3A%20%27.classVariableAt_%28%22%27%3B%0D%0A%09%09%09%09nextPutAll%3A%20aName%3B%0D%0A%09%09%09%09nextPutAll%3A%20%27%22%29%27.%0D%0A%09%09%09%5Eself%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09aClass%20sharedPoolsAndNamesDo%3A%20%5B%3ApoolName%20%3Apool%7C%0D%0A%09%09%28pool%20includesKey%3A%20aName%29%20ifTrue%3A%20%5B%0D%0A%09%09%09self%0D%0A%09%09%09%09nextPutAll%3A%20%27smalltalk.poolDictionary_%28%22%27%3B%0D%0A%09%09%09%09nextPutAll%3A%20poolName%3B%0D%0A%09%09%09%09nextPutAll%3A%20%27%22%29.at_%28%22%27%3B%0D%0A%09%09%09%09nextPutAll%3A%20aName%3B%0D%0A%09%09%09%09nextPutAll%3A%20%27%22%29%27.%0D%0A%09%09%09%5Eself%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09self%20nextPutAll%3A%20%28self%20codeToGetGlobal%3A%20aNode%20value%29."));
smalltalk.bind(smalltalk.JSEncoder,"visitMethodNode:",0
,function(aMethodNode){var self=this;
(function($rec){$rec.nextPutAll_(self.newMethodFunctionName()._comma(unescape("%28")));$rec.nextPutAll_(self.smalltalkNameFor_(self.currentClass()));$rec.nextPutAll_(unescape("%2C%22")._comma(aMethodNode.selector())._comma(unescape("%22%2C")));return $rec.cr();})(self);
self.emmitMethod_(aMethodNode);
if (self.includeSourceCode()){(function($rec){$rec.cr();return $rec.nextPutAll_(unescape("%2C")._comma(self.escaped_(aMethodNode.source())));})(self);};;
self.nextPutAll_(unescape("%29"));
return self;}
,"visiting",unescape("visitMethodNode%3A%20aMethodNode%0D%0A%09%22%20Visit%20a%20method%20node.%20%22%0D%0A%0D%0A%09self%0D%0A%09%09nextPutAll%3A%20self%20newMethodFunctionName%2C%20%27%28%27%3B%0D%0A%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20self%20currentClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27%2C%22%27%2C%20aMethodNode%20selector%2C%20%27%22%2C%27%3B%20cr.%0D%0A%09self%20emmitMethod%3A%20aMethodNode.%0D%0A%09self%20includeSourceCode%20ifTrue%3A%20%5B%0D%0A%09%09self%09cr%3B%20%22warning%3A%20this%20cr%20is%20important%20for%20debugger%22%0D%0A%09%09%09nextPutAll%3A%20%27%2C%27%20%2C%28self%20escaped%3A%20aMethodNode%20source%29%0D%0A%09%5D.%0D%0A%09self%20nextPutAll%3A%20%27%29%27"));
smalltalk.bind(smalltalk.JSEncoder,"visitNode:index:",0
,function(aNode,index){var $s8Ret$={name:"stReturn"};try{var self=this;
var selector=nil;
if (index.isMemberOf_(smalltalk.ValueNode).and_((function(){return index.hasString();}))){selector=self.asSelector_(index.value());
((selector.includes_(" ").or_((function(){return selector.includes_("_");}))).mustBeBoolean()==true ? nil : (function(){(function($rec){$rec.nextPutAll_(".");return $rec.nextPutAll_(selector);})(self);return (function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();})());};;
self.nextIndexPut_((function(){return self.visit_(index);}));
(function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"visiting",unescape("visitNode%3A%20aNode%20index%3A%20index%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20indexed%20access.%20%22%0D%0A%0D%0A%09%7C%20selector%20%7C%0D%0A%09%28%28index%20isMemberOf%3A%20ValueNode%29%20and%3A%20%5B%20index%20hasString%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09selector%20%3A%3D%20self%20asSelector%3A%20index%20value.%0D%0A%09%09%28%28selector%20includes%3A%20%24%20%29%20or%3A%20%5B%20selector%20includes%3A%20%24_%20%5D%29%20ifFalse%3A%20%5B%0D%0A%09%09%09self%20nextPutAll%3A%20%24.%3B%20nextPutAll%3A%20selector.%0D%0A%09%09%09%5Eself%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09self%20nextIndexPut%3A%20%5B%20self%20visit%3A%20index%20%5D%20"));
smalltalk.bind(smalltalk.JSEncoder,"messageSendConnector",0
,function(){var self=this;
return ".";}
,"encoding",unescape("messageSendConnector%0D%0A%09%22%20Private%20-%20Return%20the%20connector%20to%20use%20for%20sending%20a%20message.%20%22%0D%0A%0D%0A%09%5E%24.%20"));
smalltalk.bind(smalltalk.JSEncoder,"messageSendConnector:",0
,function(aNode){var self=this;
if ((nil.isNil_(aNode.receiver().messageSendConnector()))){return "";};;
return self.messageSendConnector();}
,"encoding",unescape("messageSendConnector%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20the%20connector%20to%20use%20for%20sending%20message.%20%22%0D%0A%0D%0A%09aNode%20receiver%20messageSendConnector%20isNil%20ifTrue%3A%20%5B%20%5E%27%27%20%5D.%0D%0A%09%5Eself%20messageSendConnector%20"));
smalltalk.bind(smalltalk.JSEncoder,"encodeNormalSend:",0
,function(aNode){var self=this;
(function($rec){$rec.visitReceiver_(aNode.receiver());$rec.nextPutAll_(self.messageSendConnector_(aNode));$rec.nextPutAll_(self.asSelector_(aNode.selector()));return $rec.nextArgumentsPut_((function(){return self.visitSendArguments_(aNode);}));})(self);
return self;}
,"encoding",unescape("encodeNormalSend%3A%20aNode%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20message%20send.%20%22%0D%0A%0D%0A%09self%09visitReceiver%3A%20aNode%20receiver%3B%0D%0A%09%09nextPutAll%3A%20%28self%20messageSendConnector%3A%20aNode%29%3B%0D%0A%09%09nextPutAll%3A%20%28self%20asSelector%3A%20aNode%20selector%29%3B%0D%0A%09%09nextArgumentsPut%3A%20%5B%20self%20visitSendArguments%3A%20aNode%20%5D%20"));
smalltalk.bind(smalltalk.JSEncoder,"visitOperation:",0
,function(aNode){var self=this;
self.compilerEvent_with_("aboutOperation:",aNode);
self.nextArgumentsPut_((function(){return self.visitReceiver_(aNode.receiver());}));
self.nextPutAll_(self.asSelector_(aNode.selector()));
self.visitSendArguments_(aNode);
self.compilerEvent_with_("operation:",aNode);
return self;}
,"visiting",unescape("visitOperation%3A%20aNode%0D%0A%09%22%20Private%20-%20aNode%20is%20an%20operation%20and%20selector%20is%20mapped%20to%20target%20operation.%20%22%0D%0A%0D%0A%09self%20compilerEvent%3A%20%23aboutOperation%3A%20with%3A%20aNode.%0D%0A%09self%20nextArgumentsPut%3A%20%5B%20self%20visitReceiver%3A%20aNode%20receiver%20%5D.%0D%0A%09self%20nextPutAll%3A%20%28self%20asSelector%3A%20aNode%20selector%29.%0D%0A%09self%20visitSendArguments%3A%20aNode.%0D%0A%09self%20compilerEvent%3A%20%23operation%3A%20with%3A%20aNode."));
smalltalk.bind(smalltalk.JSEncoder,"visitPoolVariableAssignment:in:to:",0
,function(aName,poolName,anExpressionNode){var self=this;
(function($rec){$rec.nextPutAll_(unescape("smalltalk.poolDictionary_%28%22"));$rec.nextPutAll_(poolName);$rec.nextPutAll_(unescape("%22%29.at_put_%28%22"));$rec.nextPutAll_(aName);return $rec.nextPutAll_(unescape("%22%2C"));})(self);
self.visit_(anExpressionNode);
self.nextPutAll_(unescape("%29"));
return self;}
,"visiting",unescape("visitPoolVariableAssignment%3A%20aName%20in%3A%20poolName%20to%3A%20anExpressionNode%0D%0A%0D%0A%09self%0D%0A%09%09nextPutAll%3A%20%27smalltalk.poolDictionary_%28%22%27%3B%0D%0A%09%09nextPutAll%3A%20poolName%3B%20nextPutAll%3A%20%27%22%29.at_put_%28%22%27%3B%0D%0A%09%09nextPutAll%3A%20aName%3B%20nextPutAll%3A%20%27%22%2C%27.%0D%0A%09self%20visit%3A%20anExpressionNode.%0D%0A%09self%09nextPutAll%3A%20%27%29%27"));
smalltalk.bind(smalltalk.JSEncoder,"visitReturnNode:",0
,function(aNode){var self=this;
var theReturn=nil;
if (self["@nestedBlocks"]._gt((0))){self["@earlyReturn"]=true;};;
theReturn=(function(){return self.emmitReturn_((function(){return aNode.nodes().do_((function(each){return self.visit_(each);}));}));});
if (!(self["@earlyReturn"])){return theReturn.value();};;
(function($rec){$rec.nextArgumentsPut_((function(){return self.emmitFunctionDoing_((function(){self.nextPutAll_(unescape("%24s8Ret%24.result%3D"));(function($rec){$rec.nextArgumentsPut_((function(){return self.emmitFunctionDoing_(theReturn);}));return $rec.nextArgumentsPut_((function(){return nil;}));})(self);return self.nextPutAll_(unescape("%3Bthrow%28%24s8Ret%24%29"));}));}));return $rec.nextArgumentsPut_((function(){return nil;}));})(self);
return self;}
,"visiting",unescape("visitReturnNode%3A%20aNode%0D%0A%0D%0A%09%7C%20theReturn%20%7C%0D%0A%09nestedBlocks%20%3E%200%20ifTrue%3A%20%5B%20earlyReturn%20%3A%3D%20true%20%5D.%0D%0A%09theReturn%20%3A%3D%20%5B%20self%20emmitReturn%3A%20%5B%0D%0A%09%09aNode%20nodes%20do%3A%20%5B%3Aeach%20%7C%20self%20visit%3A%20each%20%5D%0D%0A%09%5D%20%5D.%0D%0A%09earlyReturn%20ifFalse%3A%20%5B%20%5EtheReturn%20value%20%5D.%0D%0A%09self%20nextArgumentsPut%3A%20%5B%0D%0A%09%09self%20emmitFunctionDoing%3A%20%5B%0D%0A%09%09%09self%20nextPutAll%3A%20%27%24s8Ret%24.result%3D%27.%0D%0A%09%09%09self%20nextArgumentsPut%3A%20%5B%0D%0A%09%09%09%09self%20emmitFunctionDoing%3A%20theReturn.%0D%0A%09%09%09%5D%3B%20nextArgumentsPut%3A%20%5B%5D.%0D%0A%09%09%09self%20nextPutAll%3A%20%27%3Bthrow%28%24s8Ret%24%29%27%0D%0A%09%09%5D%0D%0A%09%5D%3B%20nextArgumentsPut%3A%20%5B%5D%20"));
smalltalk.bind(smalltalk.JSEncoder,"encodeSuperSend:",0
,function(aNode){var self=this;
self.nextPutAll_("smalltalk.superImplementor");
self.nextArgumentsPut_((function(){return self.nextPutAll_(self.smalltalkNameFor_(self.currentClass())._comma(unescape("%2C%22"))._comma(self.asSelector_(aNode.selector()))._comma(unescape("%22")));}));
self.nextPutAll_(unescape(".apply%28self%2C%20"));
self.nextIndexPut_((function(){return self.visitSendArguments_(aNode);}));
self.nextPutAll_(unescape("%29"));
return self;}
,"encoding",unescape("encodeSuperSend%3A%20aNode%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20super%20message%20send.%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20%27smalltalk.superImplementor%27.%0D%0A%09self%20nextArgumentsPut%3A%20%5B%0D%0A%09%09self%20nextPutAll%3A%0D%0A%09%09%09%28self%20smalltalkNameFor%3A%20self%20currentClass%29%0D%0A%09%09%09%2C%27%2C%22%27%2C%28self%20asSelector%3A%20aNode%20selector%29%20%2C%24%22%0D%0A%09%5D.%0D%0A%09self%20nextPutAll%3A%20%27.apply%28self%2C%20%27.%0D%0A%09self%20nextIndexPut%3A%20%5B%20self%20visitSendArguments%3A%20aNode%20%5D.%0D%0A%09self%20nextPutAll%3A%20%27%29%27.%20"));
smalltalk.bind(smalltalk.JSEncoder,"coreOptimizations",0
,function(){var self=this;
return ["core", [["class", unescape("%25receiver%25.%24klass")], ["isNil", unescape("%28nil.isNil_%28%25receiver%25%29%29")], ["notNil", unescape("%28nil.isNil_%28%25receiver%25%29%3D%3D%3Dfalse%29")], [unescape("%3D%3D"), unescape("%28nil.is_eqeq_%28%25receiver%25%2C%25arg1%25%29%29")], [unescape("%3D%3D%3D"), unescape("%28%25receiver%25%20%3D%3D%3D%20%25arg1%25%29")], [unescape("%7C%7C"), unescape("%28%25receiver%25%20%7C%7C%20%25arg1%25%29")], ["basicAt:", unescape("%28%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%5B%25arg1%25%5D%29%3Bif%28nil.isNil_%28%241%24%29%29return%20nil%3Breturn%20%241%24%3B%7D%29%28%29%29")], ["ifUndefined:", "hasBlockNoArguments:", unescape("%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%29%3Bif%20%28typeof%28%241%24%29%3D%3D%22undefined%22%29%20return%20%25arg1%25%28%29%3Breturn%20%241%24%3B%7D%29%28%29")], ["basicAt:ifAbsent:", unescape("%28%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%5B%25arg1%25%5D%29%3Bif%28nil.isNil_%28%241%24%29%29return%20%28%25arg2%25.value%28%29%29%3Breturn%20%241%24%3B%7D%29%28%29%29")], ["basicAt:put:", unescape("%28%25receiver%25%5B%25arg1%25%5D%3D%28%25arg2%25%29%29")]]];}
,"optimization",unescape("coreOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20optimization%20set.%20%22%0D%0A%0D%0A%09%5E%23%28core%20%20%23%28%0D%0A%09%09%23%28class%09%27%25receiver%25.%24klass%27%20%29%0D%0A%09%09%23%28isNil%09%27%28nil.isNil_%28%25receiver%25%29%29%27%20%29%0D%0A%09%09%23%28notNil%20%27%28nil.isNil_%28%25receiver%25%29%3D%3D%3Dfalse%29%27%20%29%0D%0A%09%09%23%28%3D%3D%09%27%28nil.is_eqeq_%28%25receiver%25%2C%25arg1%25%29%29%27%20%29%0D%0A%09%09%23%28%3D%3D%3D%09%27%28%25receiver%25%20%3D%3D%3D%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%7C%7C%09%27%28%25receiver%25%20%7C%7C%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28basicAt%3A%20%27%28%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%5B%25arg1%25%5D%29%3Bif%28nil.isNil_%28%241%24%29%29return%20nil%3Breturn%20%241%24%3B%7D%29%28%29%29%27%20%29%0D%0A%09%09%23%28ifUndefined%3A%20hasBlockNoArguments%3A%20%27%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%29%3Bif%20%28typeof%28%241%24%29%3D%3D%22undefined%22%29%20return%20%25arg1%25%28%29%3Breturn%20%241%24%3B%7D%29%28%29%27%20%29%0D%0A%09%09%23%28basicAt%3AifAbsent%3A%20%27%28%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%5B%25arg1%25%5D%29%3Bif%28nil.isNil_%28%241%24%29%29return%20%28%25arg2%25.value%28%29%29%3Breturn%20%241%24%3B%7D%29%28%29%29%27%20%29%0D%0A%09%09%23%28basicAt%3Aput%3A%20%27%28%25receiver%25%5B%25arg1%25%5D%3D%28%25arg2%25%29%29%27%20%29%0D%0A%09%29%29"));
smalltalk.bind(smalltalk.JSEncoder,"s8Optimizations",0
,function(){var self=this;
return ["s8", ["core", ["ifTrue:", "hasBlockNoArguments:", [["ifTrue:", "noClient:", unescape("if%20%28%25receiver%25%29%7B%25seq1%25%7D%3B")], ["ifTrue:", unescape("%28%28%25receiver%25%29.mustBeBoolean%28%29%3D%3Dtrue%20%3F%20%25arg1%25%28%29%20%3A%20nil%29")]]], ["ifFalse:", "hasBlockNoArguments:", [["ifFalse:", "noClient:", unescape("if%20%28%21%28%25receiver%25%29%29%7B%25seq1%25%7D%3B")], ["ifFalse:", unescape("%28%28%25receiver%25%29.mustBeBoolean%28%29%3D%3Dtrue%20%3F%20nil%20%3A%20%25arg1%25%28%29%29")]]], ["ifTrue:ifFalse:", "hasBlockNoArguments:", [["ifTrue:ifFalse:", "noClient:", unescape("if%20%28%25receiver%25%29%7B%25seq1%25%7D%20else%7B%25seq2%25%7D%3B")], ["ifTrue:ifFalse:", unescape("%28%28%25receiver%25%29.mustBeBoolean%28%29%3D%3Dtrue%20%3F%20%25arg1%25%28%29%20%3A%20%25arg2%25%28%29%29")]]], ["ifFalse:ifTrue:", "hasBlockNoArguments:", [["ifFalse:ifTrue:", "noClient:", unescape("if%20%28%25receiver%25%29%7B%25seq2%25%7D%20else%7B%25seq1%25%7D%3B")], ["ifFalse:ifTrue:", unescape("%28%28%25receiver%25%29.mustBeBoolean%28%29%3D%3Dtrue%20%3F%20%25arg2%25%28%29%20%3A%20%25arg1%25%28%29%29")]]], ["whileTrue", "receiverIsBlockNoArguments:", unescape("%28function%28%29%7Bwhile%28%25receiver%25%28%29%29%7B%7D%3Breturn%20nil%7D%29%28%29")], ["whileFalse", "receiverIsBlockNoArguments:", unescape("%28function%28%29%7Bwhile%28%21%25receiver%25%28%29%29%7B%7D%3Breturn%20nil%7D%29%28%29")], ["orNil", unescape("%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%29%3Breturn%20%28nil.isNil_%28%241%24%29%29%20%3F%20nil%20%3A%20%241%24%3B%7D%29%28%29")], ["whileTrue:", "allBlockNoArguments:", [["whileTrue:", "noClient:", unescape("while%28%25receiver%25%28%29%29%7B%25seq1%25%7D%3B")], ["whileTrue:", unescape("%28function%28%29%7Bwhile%28%25receiver%25%28%29%29%7B%25arg1%25%28%29%7D%3Breturn%20nil%7D%29%28%29")]]], ["whileFalse:", "allBlockNoArguments:", [["whileFalse:", "noClient:", unescape("while%28%21%25receiver%25%28%29%29%7B%25seq1%25%7D%3B")], ["whileFalse:", unescape("%28function%28%29%7Bwhile%28%21%25receiver%25%28%29%29%7B%25arg1%25%28%29%7D%3Breturn%20nil%7D%29%28%29")]]], ["ifNil:", "hasBlockArguments:", unescape("%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%29%3Breturn%20%28nil.isNil_%28%241%24%29%29%20%3F%20%25arg1%25%28%29%20%3A%20%241%24%3B%7D%29%28%29")], ["ifNotNil:", "hasBlockArguments:", unescape("%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%29%3Breturn%20%28nil.isNil_%28%241%24%29%3D%3D%3Dfalse%29%20%3F%20%25arg1%25%28%25receiver%25%29%20%3A%20%241%24%3B%7D%29%28%29")], ["timesRepeat:", "hasBlockNoArguments:", unescape("%28function%28%29%7Bfor%28var%20%241%24%3D%28%25receiver%25%29%3B%241%24%3E0%3B%241%24--%29%7B%25arg1%25%28%29%7D%3Breturn%20nil%7D%29%28%29")], ["isKindOf:", unescape("smalltalk.is_kindOf_%28%25receiver%25%2C%28%25arg1%25%29%29")], ["respondsTo:", unescape("smalltalk.responds_to_%28%25receiver%25%2C%28%25arg1%25%29%29")], ["ifNil:ifNotNil:", "hasBlockNoArguments:", unescape("%28nil.isNil_%28%25receiver%25%29%29%20%3F%20%25arg1%25%28%29%20%3A%20%25arg2%25%28%29")], ["ifNotNil:ifNil:", "hasBlockNoArguments:", unescape("%28nil.isNil_%28%25receiver%25%29%3D%3D%3Dfalse%29%20%3F%20%25arg1%25%28%29%20%3A%20%25arg2%25%28%29")]]];}
,"optimization",unescape("s8Optimizations%0D%0A%09%22%20Private%20-%20Return%20the%20optimization%20set.%20%22%0D%0A%0D%0A%09%5E%23%28s8%09%23%28%0D%0A%09%09core%0D%0A%09%09%23%28ifTrue%3A%20hasBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28ifTrue%3A%20noClient%3A%20%27if%20%28%25receiver%25%29%7B%25seq1%25%7D%3B%27%20%29%0D%0A%09%09%09%23%28ifTrue%3A%20%27%28%28%25receiver%25%29.mustBeBoolean%28%29%3D%3Dtrue%20%3F%20%25arg1%25%28%29%20%3A%20nil%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28ifFalse%3A%20hasBlockNoArguments%3A%20%20%23%28%0D%0A%09%09%09%23%28ifFalse%3A%20noClient%3A%20%27if%20%28%21%28%25receiver%25%29%29%7B%25seq1%25%7D%3B%27%20%29%0D%0A%09%09%09%23%28ifFalse%3A%20%27%28%28%25receiver%25%29.mustBeBoolean%28%29%3D%3Dtrue%20%3F%20nil%20%3A%20%25arg1%25%28%29%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28ifTrue%3AifFalse%3A%20hasBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28ifTrue%3AifFalse%3A%20noClient%3A%20%27if%20%28%25receiver%25%29%7B%25seq1%25%7D%20else%7B%25seq2%25%7D%3B%27%20%29%0D%0A%09%09%09%23%28ifTrue%3AifFalse%3A%20%27%28%28%25receiver%25%29.mustBeBoolean%28%29%3D%3Dtrue%20%3F%20%25arg1%25%28%29%20%3A%20%25arg2%25%28%29%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28ifFalse%3AifTrue%3A%20hasBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28ifFalse%3AifTrue%3A%20noClient%3A%20%27if%20%28%25receiver%25%29%7B%25seq2%25%7D%20else%7B%25seq1%25%7D%3B%27%20%29%0D%0A%09%09%09%23%28ifFalse%3AifTrue%3A%20%27%28%28%25receiver%25%29.mustBeBoolean%28%29%3D%3Dtrue%20%3F%20%25arg2%25%28%29%20%3A%20%25arg1%25%28%29%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28whileTrue%20receiverIsBlockNoArguments%3A%20%27%28function%28%29%7Bwhile%28%25receiver%25%28%29%29%7B%7D%3Breturn%20nil%7D%29%28%29%27%20%29%0D%0A%09%09%23%28whileFalse%20receiverIsBlockNoArguments%3A%20%27%28function%28%29%7Bwhile%28%21%25receiver%25%28%29%29%7B%7D%3Breturn%20nil%7D%29%28%29%27%20%29%0D%0A%09%09%23%28orNil%20%27%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%29%3Breturn%20%28nil.isNil_%28%241%24%29%29%20%3F%20nil%20%3A%20%241%24%3B%7D%29%28%29%27%20%29%0D%0A%09%09%23%28whileTrue%3A%20allBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28whileTrue%3A%20noClient%3A%20%27while%28%25receiver%25%28%29%29%7B%25seq1%25%7D%3B%27%20%29%0D%0A%09%09%09%23%28whileTrue%3A%20%27%28function%28%29%7Bwhile%28%25receiver%25%28%29%29%7B%25arg1%25%28%29%7D%3Breturn%20nil%7D%29%28%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28whileFalse%3A%20allBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28whileFalse%3A%20noClient%3A%20%27while%28%21%25receiver%25%28%29%29%7B%25seq1%25%7D%3B%27%20%29%0D%0A%09%09%09%23%28whileFalse%3A%20%27%28function%28%29%7Bwhile%28%21%25receiver%25%28%29%29%7B%25arg1%25%28%29%7D%3Breturn%20nil%7D%29%28%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28ifNil%3A%20hasBlockArguments%3A%20%27%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%29%3Breturn%20%28nil.isNil_%28%241%24%29%29%20%3F%20%25arg1%25%28%29%20%3A%20%241%24%3B%7D%29%28%29%27%20%29%0D%0A%09%09%23%28ifNotNil%3A%20hasBlockArguments%3A%20%27%28function%28%29%7Bvar%20%241%24%3D%28%25receiver%25%29%3Breturn%20%28nil.isNil_%28%241%24%29%3D%3D%3Dfalse%29%20%3F%20%25arg1%25%28%25receiver%25%29%20%3A%20%241%24%3B%7D%29%28%29%27%20%29%0D%0A%09%09%23%28timesRepeat%3A%20hasBlockNoArguments%3A%20%27%28function%28%29%7Bfor%28var%20%241%24%3D%28%25receiver%25%29%3B%241%24%3E0%3B%241%24--%29%7B%25arg1%25%28%29%7D%3Breturn%20nil%7D%29%28%29%27%20%29%0D%0A%09%09%23%28isKindOf%3A%20%27smalltalk.is_kindOf_%28%25receiver%25%2C%28%25arg1%25%29%29%27%20%29%0D%0A%09%09%23%28respondsTo%3A%20%27smalltalk.responds_to_%28%25receiver%25%2C%28%25arg1%25%29%29%27%20%29%0D%0A%09%09%23%28ifNil%3AifNotNil%3A%20hasBlockNoArguments%3A%20%27%28nil.isNil_%28%25receiver%25%29%29%20%3F%20%25arg1%25%28%29%20%3A%20%25arg2%25%28%29%27%20%29%0D%0A%09%09%23%28ifNotNil%3AifNil%3A%20hasBlockNoArguments%3A%20%27%28nil.isNil_%28%25receiver%25%29%3D%3D%3Dfalse%29%20%3F%20%25arg1%25%28%29%20%3A%20%25arg2%25%28%29%27%20%29%0D%0A%09%29%29%20"));
smalltalk.bind(smalltalk.JSEncoder,"magnitudeOptimizations",0
,function(){var self=this;
return ["magnitude", [[unescape("%3E"), "hasNumber:", unescape("%28%25receiver%25%20%3E%20%25arg1%25%29")], [unescape("%3C"), "hasNumber:", unescape("%28%25receiver%25%20%3C%20%25arg1%25%29")], [unescape("%3E%3D"), "hasNumber:", unescape("%28%25receiver%25%20%3E%3D%20%25arg1%25%29")], [unescape("%3C%3D"), "hasNumber:", unescape("%28%25receiver%25%20%3C%3D%20%25arg1%25%29")]]];}
,"optimization",unescape("magnitudeOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20optimization%20set.%20%22%0D%0A%0D%0A%09%5E%23%28magnitude%20%23%28%0D%0A%09%09%23%28%3E%20%20hasNumber%3A%20%27%28%25receiver%25%20%3E%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%3C%20%20hasNumber%3A%20%27%28%25receiver%25%20%3C%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%3E%3D%20hasNumber%3A%20%27%28%25receiver%25%20%3E%3D%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%3C%3D%20hasNumber%3A%20%27%28%25receiver%25%20%3C%3D%20%25arg1%25%29%27%20%29%0D%0A%09%29%29"));
smalltalk.bind(smalltalk.JSEncoder,"mathOptimizations",0
,function(){var self=this;
return ["math", [[unescape("+"), "hasNumber:", unescape("%28%25receiver%25%20+%20%25arg1%25%29")], [unescape("-"), "hasNumber:", unescape("%28%25receiver%25%20-%20%25arg1%25%29")], [unescape("*"), "hasNumber:", unescape("%28%25receiver%25%20*%20%25arg1%25%29")], [unescape("/"), "hasNumber:", unescape("%28%25receiver%25%20/%20%25arg1%25%29")], [unescape("%3D"), "hasNumber:", unescape("%28%25receiver%25%20%3D%3D%20%25arg1%25%29")]]];}
,"optimization",unescape("mathOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20optimization%20set.%20%22%0D%0A%0D%0A%09%5E%23%28math%20%23%28%0D%0A%09%09%23%28+%20hasNumber%3A%20%27%28%25receiver%25%20+%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%23-%20hasNumber%3A%20%27%28%25receiver%25%20-%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28*%20hasNumber%3A%20%27%28%25receiver%25%20*%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28/%20hasNumber%3A%20%27%28%25receiver%25%20/%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%3D%20hasNumber%3A%20%27%28%25receiver%25%20%3D%3D%20%25arg1%25%29%27%20%29%0D%0A%09%29%29"));
smalltalk.bind(smalltalk.JSEncoder,"defaultKnownOptimizations",0
,function(){var self=this;
return (function($rec){$rec.add_(self.s8Optimizations());$rec.add_(self.magnitudeOptimizations());$rec.add_(self.mathOptimizations());return $rec.yourself();})(smalltalk.Array.with_(self.coreOptimizations()));}
,"defaults",unescape("defaultKnownOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20default%20knownOptimizations%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28Array%09with%3A%20self%20coreOptimizations%29%0D%0A%09%09add%3A%20self%20s8Optimizations%3B%0D%0A%09%09add%3A%20self%20magnitudeOptimizations%3B%0D%0A%09%09add%3A%20self%20mathOptimizations%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.LuaEncoder.$klass,"supportsLanguage:",0
,function(aLanguage){var self=this;
return aLanguage._eq("lua");}
,"instantiation",unescape("supportsLanguage%3A%20aLanguage%0D%0A%09%22%20Private%20-%20Return%20true%20if%20instances%20of%20the%20receiver%20can%20encode%20to%20aLanguage.%20%22%0D%0A%0D%0A%09%5EaLanguage%20%3D%20%23lua%20"));
smalltalk.bind(smalltalk.LuaEncoder.$klass,"initializeKernelInlines:",0
,function(pool){var self=this;
return self.register_into_(self.kernelInlines(),pool);}
,"inlines",unescape("initializeKernelInlines%3A%20pool%0D%0A%09%22%20Private%20-%20Initialize%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5Eself%20register%3A%20self%20kernelInlines%20into%3A%20pool%20"));
smalltalk.bind(smalltalk.LuaEncoder.$klass,"kernelInlines",0
,function(){var self=this;
return [["Object", [["halt", unescape("debug.debug%28%29")], ["toString:", unescape("tostring%28anObject%29")], ["isObject:", unescape("%0D%0A%09local%20status%2C%20result%20%3D%20pcall%28function%28%29%20return%20type%28anObject.yourself%29%3D%3D%22function%22%20end%29%0D%0A%09if%20true%20then%20return%20status%20and%20result%20end")], ["typeOf:", unescape("type%28anObject%29")], ["basicDelete:", unescape("if%20self%3D%3Dnil%20then%20return%20else%20self%5BaString%5D%3Dnil%20end")], ["basicPerform:withArguments:", ["self", unescape("fn%28self%2C%28table.unpack%20or%20unpack%29%28anArray%29%29")]], ["asJSONObject", unescape("%7B%7D")]]], ["Smalltalk", [["includesKey:", unescape("%7B%7D")], ["eval:", unescape("%0D%0A%09--%20%28req%20v5.3%29%20local%20fn%2Cerr%20%20%3D%20load%28anExpression%29%0D%0A%09function%20f%28%29%20local%20r%20%3D%20anExpression%3B%20anExpression%3D%20%22%22%3B%20return%20r%20end%20--%20%28req%20v5.1%29%0D%0A%09local%20fn%2Cerr%20%20%3D%20load%28f%29%0D%0A%09if%20fn%20then%20return%20fn%28%29%20else%0D%0A%09%09self%3Anote_%28%22%20eval%3A%20Error%20%22..err%29%20--the%20expression%20must%20be%20a%20lua%20expression%20%28including%20return%29%0D%0A%09%09self%3Anote_%28%22%20expression%3D%20%22..anExpression%29%0D%0A%09%09return%20nil%0D%0A%09end")]]], ["Smalltalk class", [["global", "_G"], ["gc", unescape("if%20collectgarbage%28%29%20then%20return%20true%20end")]]], ["Behavior", [["basicNew", unescape("self%5B%22%24fn%22%5D%28%29")]]], ["CompiledMethod", []], ["Boolean", [["not", unescape("self%20%3D%3D%20false")], [unescape("%3D"), unescape("%0D%0A%09if%20anObject%3D%3Dnil%20then%20return%20false%20end%0D%0A%09if%20true%20then%20return%20self%3D%3DanObject%20end")], ["ifTrue:ifFalse:", unescape("%0D%0A%09if%20%28self%3D%3Dtrue%29%0D%0A%09%09then%20return%20aBlock%3Avalue%28%29%0D%0A%09%09else%20return%20anotherBlock%3Avalue%28%29%20end")]]], ["Number", [[unescape("%3D"), unescape("%0D%0A%09if%20%28type%28anObject%29%3D%3D%22number%22%29%0D%0A%09%09then%20return%20self%3D%3DanObject%0D%0A%09%09else%20return%20false%0D%0A%09%09end")], [unescape("%3E"), unescape("self%20%3E%20aNumber")], [unescape("%3E%3D"), unescape("self%20%3E%3D%20aNumber")], [unescape("%3C"), unescape("self%20%3C%20aNumber")], [unescape("%3C%3D"), unescape("self%20%3C%3D%20aNumber")], [unescape("+"), unescape("self%20+%20aNumber")], [unescape("-"), unescape("self%20-%20aNumber")], [unescape("*"), unescape("self%20*%20aNumber")], [unescape("/"), unescape("self%20/%20aNumber")], ["sin", unescape("math.sin%28self%29")], ["cos", unescape("math.cos%28self%29")], ["tan", unescape("math.tan%28self%29")], ["arcCos", unescape("math.acos%28self%29")], ["arcSin", unescape("math.asin%28self%29")], ["arcTan", unescape("math.atan%28self%29")], ["atan2:", unescape("math.atan2%28self%2CaNumber%29")], ["sqrt", unescape("math.sqrt%28self%29")], [unescape("%3C%3C"), unescape("bit.lshift%28self%2CaNumber%29")], [unescape("%3E%3E"), unescape("bit.rshift%28self%2CaNumber%29")], ["raisedTo:", unescape("math.pow%28self%2CaNumber%29")], ["max:", unescape("math.max%28self%2CaNumber%29")], ["min:", unescape("math.min%28self%2CaNumber%29")], ["rounded", unescape("self%20+%200.5%20-%20%28self%20+%200.5%29%20%25%25%201")], ["truncated", unescape("math.floor%28self%29")], ["timesRepeat:", unescape("for%20i%3D1%2Cself%20do%20fn%28i%29%20end")], ["to:do:", unescape("for%20i%3Dself%2CaNumber%20do%20fn%28i%29%20end")], ["to:by:do:", unescape("for%20i%3Dself%2CaNumber%2Cincrement%20do%20fn%28i%29%20end")], ["printString", unescape("tostring%28self%29")], ["toString:", unescape("%25%3Abasen%25%28self%2Cradix%29")], ["isNumber", unescape("if%20%28self%7E%3Dself%29%20then%20return%20false%20else%20return%20%28self%7E%3Dmath.huge%20and%20self%7E%3D-math.huge%29%20end")], ["clearInterval", unescape("if%20true%20then%20self%3AnotImplementedYet_%28%22clearInterval%22%29%20end")], ["clearTimeout", unescape("if%20true%20then%20self%3AnotImplementedYet_%28%22clearTimeout%22%29%20end")]]], ["Number class", [["pi", "math.pi"]]], ["BlockClosure", [["whileTrue:", unescape("while%20self%28%29%20do%20fn%28%29%20end")], ["whileFalse:", unescape("while%20%28self%28%29%3D%3Dfalse%29%20do%20fn%28%29%20end")], ["valueWithPossibleArguments:", unescape("self%28%28table.unpack%20or%20unpack%29%28array%29%29")], ["try:catch:", unescape("%0D%0A%09local%20success%2C%20result%20%3D%20pcall%28function%28%29%20return%20fn%28%29%20end%29%0D%0A%09if%20success%0D%0A%09%09then%20return%20result%0D%0A%09%09else%20return%20ab%28%20self%3AsmalltalkErrorOf_%28result%29%20%29%20end")], ["valueWithTimeout:", unescape("self%3AnotImplementedYet_%28%22valueWithTimeout%3A%22%29")], ["valueWithInterval:", unescape("self%3AnotImplementedYet_%28%22valueWithInterval%3A%22%29")], ["ensure:", unescape("%0D%0A%09local%20success%2C%20result%20%3D%20pcall%28function%28%29%20return%20self%28%29%20end%29%0D%0A%09fn%28%29%0D%0A%09if%20true%20then%20return%20result%20end")], ["ifCurtailed:", unescape("%0D%0A%09local%20curtailed%20%3D%20true%0D%0A%09local%20success%2C%20result%20%3D%20pcall%28function%28%29%0D%0A%09%09local%20r%20%3D%20self%28%29%0D%0A%09%09curtailed%20%3D%20false%0D%0A%09%09return%20r%0D%0A%09end%29%0D%0A%09if%20%28curtailed%29%0D%0A%09%09then%20exceptionBlock%3Avalue%28%29%0D%0A%09%09else%20return%20result%0D%0A%09%09end")], ["argumentCount", unescape("self%3AnotImplementedYet_%28%22argumentCount%22%29")]]], ["UndefinedObject", [["asJSONObject", "nil"]]], ["Collection", [["wrap:", unescape("if%20%25%3AisNil_%25%28anObject%29%20--anObject%3D%3Dnil%0D%0A%09then%20return%20smalltalk.UndefinedObject%3Aalias%28%29%0D%0A%09else%20return%20anObject%20end")], ["unwrap:", unescape("--if%20anObject%3D%3Dnil%20then%20return%20nil%20end%0D%0A%09--if%20%28type%28anObject%29%3D%3D%22table%22%29%20and%20%25%3AisNil_%25%28anObject%29%0D%0A%09if%20%25%3AisNil_%25%28anObject%29%0D%0A%09%09then%20return%20nil%0D%0A%09%09else%20return%20anObject%20end")]]], ["HashedSet", [["initializeHash", unescape("%7B%7D")], ["includes:", unescape("%28self%5B%22@hash%22%5D%5Bcookie%5D%29%7E%3Dnil")], ["privateRemove:", unescape("self%5B%22@hash%22%5D%5Bcookie%5D%20%3D%20nil")]]], ["SequenceableCollection", [["indexOf:ifAbsent:", unescape("%0D%0A%09for%20i%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09if%20%28%25%3AisObject_equalTo_%25%28self%3Aunwrap_%28value%29%2CanObject%29%29%20then%20return%20i%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20aBlock%3Avalue%28%29%20end")], ["do:", unescape("%0D%0A%09for%20_%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09fn%28self%3Aunwrap_%28value%29%29%0D%0A%09end")], ["detect:ifNone:", unescape("%0D%0A%09for%20_%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09local%20v%3Dself%3Aunwrap_%28value%29%0D%0A%09%09if%20%28fn%28v%29%29%20then%20return%20v%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20anotherBlock%3Avalue%28%29%20end")], ["reversed", unescape("%0D%0A%09local%20result%20%3D%20smalltalk.Array%3Anew%28%29%0D%0A%09local%20top%20%3D%20%23self%20+%201%0D%0A%09for%20i%2Cv%20in%20ipairs%28self%29%20do%20result%5Btop%20-%20i%5D%20%3D%20v%20end%0D%0A%09if%20true%20then%20return%20result%20end")], ["isEmpty", unescape("%23self%3D%3D0")], ["reverseDo:", unescape("for%20i%20%3D%20%23self%2C1%2C-1%20do%20fn%28self%3Aunwrap_%28self%5Bi%5D%29%29%20end")], ["includes:", [unescape("%0D%0A%09for%20_%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09if%20%28%25%3AisObject_equalTo_%25%28self%3Aunwrap_%28value%29%2CanObject%29%29%20then%20return%20true%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20false%20end"), "if true then end"]], ["occurrencesOf:", unescape("%0D%0A%09local%20count%20%3D%200%0D%0A%09for%20_%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09if%20%28%25%3AisObject_equalTo_%25%28self%3Aunwrap_%28value%29%2CanObject%29%29%20then%20count%20%3D%20count%20+%201%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20count%20end")]]], ["String class", [["fromString:", unescape("if%20true%20then%20tostring%28aString%29%20end")], ["new", unescape("%22%22")], ["cr", unescape("%22%5Cr%22")], ["lf", unescape("%22%5Cn%22")], ["crlf", unescape("%22%5Cr%5Cn%22")], ["space", unescape("%22%20%22")], ["tab", unescape("%22%5Ct%22")], ["zero", unescape("string.char%280%29")], ["fromCharCode:", unescape("%0D%0A%09if%20%28type%28anArrayOrNumber%29%3D%3D%22number%22%29%20then%0D%0A%09%09return%20string.char%28anArrayOrNumber%29%0D%0A%09end%0D%0A%09if%20true%20then%0D%0A%09%09local%20fup%20%3D%20table.unpack%20or%20unpack%0D%0A%09%09return%20%28string.char%28fup%28anArrayOrNumber%29%29%29%0D%0A%09end")]]], ["String", [[unescape("%3D"), unescape("if%20true%20then%20return%20self%3D%3DanObject%20end")], ["size", unescape("%23self")], ["at:", unescape("self%3Asub%28anIndex%2CanIndex%29")], ["do:", unescape("for%20c%20in%20self%3Agmatch%28%22.%22%29%20do%20fn%28c%29%20end")], ["withIndexDo:", unescape("for%20i%3D1%2C%23self%20do%20fn%28self%3Asub%28i%2Ci%29%2Ci%29%20end")], ["detect:ifNone:", unescape("%0D%0A%09for%20c%20in%20self%3Agmatch%28%22.%22%29%20do%0D%0A%09%09if%20%28fn%28c%29%29%20then%20return%20c%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20anotherBlock%3Avalue%28%29%20end")], ["isLiteral", unescape("%0D%0A%09if%20%23self%3D%3D0%20then%20return%20false%20end%0D%0A%09if%20smalltalk.String%3AisBinarySelector_%28self%29%20then%20return%20true%20end%0D%0A%09if%20true%20then%20return%20string.find%28self%2C%22%5B%5Ea-zA-Z0-9_%3A%5D+%22%29%3D%3Dnil%20end")], ["occurrencesOf:", unescape("%0D%0A%09local%20j%0D%0A%09local%20k%2Ccount%20%3D%200%2C0%0D%0A%09repeat%09j%2C%20k%20%3D%20self%3Afind%28aString%2C%20k%20+%201%2C%20true%29%0D%0A%09%09if%20%28j%7E%3Dnil%29%20then%20count%3Dcount+1%20end%0D%0A%09until%20j%20%3D%3D%20nil%0D%0A%09if%20true%20then%20return%20count%20end")], ["startsWith:", unescape("if%20true%20then%20return%20%28self%3Asub%281%2C%20%23aString%29%20%3D%3D%20aString%29%20end")], ["escaped", unescape("self%3AnotImplementedYet_%28%22escaped%22%29")], ["unescaped", unescape("%0D%0A%09local%20x16%20%3D%20function%28x%29%20return%20string.char%28tonumber%28x%2C%2016%29%29%20end%0D%0A%09if%20true%20then%20return%20self%3Agsub%28%22%25%25%25%25%28%25%25x%25%25x%29%22%2C%20x16%29%20end")], ["asEncodedURI", unescape("self%3AnotImplementedYet_%28%22asEncodedURI%22%29")], ["asDecodedURI", unescape("self%3AnotImplementedYet_%28%22asDecodedURI%22%29")], [unescape("%2C"), unescape("if%20true%20then%20return%20self..%28%25%3AasString_%25%28aCollection%29%29%20end")], ["copyWith:", unescape("self..%28%25%3AasString_%25%28anObject%29%29")], ["copyFrom:to:", unescape("self%3Asub%28anIndex%2CanotherIndex%29")], ["reversed", unescape("if%20true%20then%20return%20self%3Areverse%28%29%20end")], ["includes:", "local ignored"], ["indexOf:", unescape("%0D%0A%09local%20index%20%3D%20self%3Afind%28aString%2C1%2Ctrue%29%0D%0A%09if%20not%20index%0D%0A%09%09then%20return%200%0D%0A%09%09else%20return%20index%20end")], ["indexOf:startingAt:", unescape("%0D%0A%09local%20index%20%3D%20self%3Afind%28aString%2Cposition%2Ctrue%29%0D%0A%09if%20not%20index%0D%0A%09%09then%20return%200%0D%0A%09%09else%20return%20index%20end")], ["lastIndexOf:", unescape("%0D%0A%09local%20i%20%3D%200%0D%0A%09local%20j%20%3D%200%0D%0A%09local%20k%20%3D%200%0D%0A%09repeat%09i%20%3D%20j%0D%0A%09%09j%2C%20k%20%3D%20self%3Afind%28aString%2C%20k%20+%201%2C%20true%29%0D%0A%09until%20j%20%3D%3D%20nil%0D%0A%09if%20true%20then%20return%20i%20end")], ["lastIndexOf:startingAt:", unescape("%0D%0A%09local%20i%2C%20j%0D%0A%09local%20k%20%3D%20position%0D%0A%09repeat%09i%20%3D%20j%0D%0A%09%09j%2C%20k%20%3D%20self%3Afind%28aString%2C%20k%20+%201%2C%20true%29%0D%0A%09until%20j%20%3D%3D%20nil%0D%0A%09if%20true%20then%20return%20i%20end")], ["asNativeName", unescape("self%3AasLuaName%28%29")], ["asPrivateSelector", unescape("local%20map%20%3D%20%7B%0D%0A%09%09%5B%22%3A%22%5D%3D%22_%22%2C%0D%0A%09%09%5B%22+%22%5D%3D%22_plus%22%2C%0D%0A%09%09%5B%22-%22%5D%3D%22_minus%22%2C%0D%0A%09%09%5B%22*%22%5D%3D%22_star%22%2C%0D%0A%09%09%5B%22/%22%5D%3D%22_slash%22%2C%0D%0A%09%09%5B%22%7C%22%5D%3D%22_bar%22%2C%0D%0A%09%09%5B%22%26%22%5D%3D%22_amp%22%2C%0D%0A%09%09%5B%22%3E%22%5D%3D%22_gt%22%2C%0D%0A%09%09%5B%22%3C%22%5D%3D%22_lt%22%2C%0D%0A%09%09%5B%22%3D%22%5D%3D%22_eq%22%2C%0D%0A%09%09%5B%22%2C%22%5D%3D%22_comma%22%2C%0D%0A%09%09%5B%22@%22%5D%3D%22_at%22%2C%0D%0A%09%09%5B%22%7E%22%5D%3D%22_no%22%2C%0D%0A%09%09%5B%22%5C%5C%22%5D%3D%22_bk%22%0D%0A%09%7D%3B%0D%0A%09result%20%3D%20self%3Agsub%28%22.%22%2Cfunction%28c%29%20return%20map%5Bc%5D%20end%29")], ["asNative:", unescape("if%20true%20then%20return%20self%3AslashEncoded%28%29%20end")], ["replace:with:", unescape("%0D%0A%09local%20result%3D%22%22%0D%0A%09local%20i%2Cj%0D%0A%09local%20k%20%3D%200%0D%0A%09repeat%09i%3Dk+1%0D%0A%09%09j%2C%20k%20%3D%20self%3Afind%28aString%2C%20i%2C%20true%29%0D%0A%09%09if%20%28j%3D%3Dnil%29%20then%0D%0A%09%09%09result%3Dresult..%28self%3Asub%28i%29%29%0D%0A%09%09else%0D%0A%09%09%09result%3Dresult..%28self%3Asub%28i%2Cj-1%29..anotherString%29%0D%0A%09%09end%0D%0A%09until%20j%20%3D%3D%20nil%0D%0A%09if%20true%20then%20return%20result%20end")], ["replaceRegexp:with:", unescape("self%3AnotImplementedYet_%28%22replaceRegexp%3Awith%3A%22%29")], ["tokenize:", unescape("%0D%0A%09local%20result%3Dsmalltalk.Array%3Anew%28%29%0D%0A%09local%20i%2Cj%0D%0A%09local%20k%20%3D%200%0D%0A%09repeat%09i%3Dk+1%0D%0A%09%09j%2C%20k%20%3D%20self%3Afind%28aString%2C%20i%2C%20true%29%0D%0A%09%09if%20%28j%3D%3Dnil%29%20then%0D%0A%09%09%09result%3Aadd_%28self%3Asub%28i%29%29%0D%0A%09%09else%0D%0A%09%09%09result%3Aadd_%28self%3Asub%28i%2Cj-%23aString%29%29%0D%0A%09%09end%0D%0A%09until%20j%20%3D%3D%20nil%0D%0A%09if%20true%20then%20return%20result%20end")], ["match:", unescape("self%3AnotImplementedYet_%28%22match%3A%22%29")], ["isLetter", unescape("%23self%3D%3D1%20and%20string.find%28self%2C%22%25%25a%22%29%3D%3D1")], ["isDigit", unescape("%23self%3D%3D1%20and%20string.find%28self%2C%22%25%25d%22%29%3D%3D1")], ["isUpperCase", unescape("%0D%0A%09if%20%28%23self%3D%3D0%29%20then%20return%20false%20end%0D%0A%09for%20c%20in%20self%3Agmatch%28%22.%22%29%20do%0D%0A%09%09if%20%28c%3C%22A%22%20or%20c%3E%22Z%22%29%20then%20return%20false%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20true%20end")], ["isLowerCase", unescape("%0D%0A%09if%20%28%23self%3D%3D0%29%20then%20return%20false%20end%0D%0A%09for%20c%20in%20self%3Agmatch%28%22.%22%29%20do%0D%0A%09%09if%20%28c%3C%22a%22%20or%20c%3E%22z%22%29%20then%20return%20false%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20true%20end")], ["asUppercase", unescape("self%3Aupper%28%29")], ["asLowercase", unescape("self%3Alower%28%29")], ["asNumber", unescape("tonumber%28self%29")], [unescape("%3E"), unescape("self%20%3E%20aString")], [unescape("%3C"), unescape("self%20%3C%20aString")], [unescape("%3E%3D"), unescape("self%20%3E%3D%20aString")], [unescape("%3C%3D"), unescape("self%20%3C%3D%20aString")], ["isCharacter", unescape("%23self%20%3D%3D%201")], ["asciiValue", unescape("string.byte%28self%29")], ["isSeparator", unescape("%23self%3D%3D1%20and%20self%3Afind%28%22%25%25s%22%29%3D%3D1")]]], ["Array class", [["basicNew:", unescape("%25%3AnewArray%25%28%29")], ["with:", "self"], ["with:with:", "self"], ["with:with:with:", "self"], ["with:with:with:with:", "self"], ["withAll:", unescape("%0D%0A%09local%20result%20%3D%20aCollection%3AasArray%28%29%0D%0A%09if%20aCollection%3D%3Dresult%0D%0A%09%09then%20return%20result%3Acopy%28%29%0D%0A%09%09else%20return%20result%0D%0A%09end")], ["slice:", unescape("self%3AnotImplementedYet_%28%22slice%3A%22%29")]]], ["Array", [["size", unescape("%23self")], ["at:", unescape("%0D%0A%09local%20value%20%3D%20self%5BanIndex%5D%0D%0A%09if%20value%3D%3Dnil%20then%20return%20self%3AobjectNotFoundAt_%28anIndex%29%20end%0D%0A%09if%20true%20then%20return%20self%3Aunwrap_%28value%29%20end")], ["at:ifAbsent:", unescape("%0D%0A%09local%20value%20%3D%20self%5BanIndex%5D%0D%0A%09if%20%28value%3D%3Dnil%29%20then%20return%20aBlock%3Avalue%28%29%20end%0D%0A%09if%20true%20then%20return%20self%3Aunwrap_%28value%29%20end")], ["at:put:", unescape("%0D%0A%09self%5BanIndex%5D%3Dself%3Awrap_%28anObject%29%0D%0A%09if%20true%20then%20return%20anObject%20end")], ["do:", unescape("%0D%0A%09for%20_%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09fn%28self%3Aunwrap_%28value%29%29%0D%0A%09end")], ["addAll:", unescape("%0D%0A%09local%20array%20%3D%20aCollection%3AasArray%28%29%0D%0A%09for%20i%3D1%2C%23array%20do%20self%5B%23self+1%5D%3Dself%3Awrap_%28array%5Bi%5D%29%20end")], ["shallowCopy", unescape("%0D%0A%09result%20%3D%20self%3Aclass%28%29%3Anew%28%29%0D%0A%09for%20i%3D1%2C%23self%20do%20result%5B%23result+1%5D%3Dself%5Bi%5D%20end")], ["copyWithAll:", unescape("%0D%0A%09local%20result%20%3D%20self%3Acopy%28%29%0D%0A%09result%3AaddAll_%28aCollection%29%0D%0A%09if%20true%20then%20return%20result%20end")], [unescape("%2C"), unescape("%0D%0A%09local%20result%20%3D%20self%3Acopy%28%29%0D%0A%09result%3AaddAll_%28aCollection%29%0D%0A%09if%20true%20then%20return%20result%20end")], ["join:", unescape("table.concat%28self%2CaString%29")], ["sort:", unescape("table.sort%28self%2Cfn%29")], ["remove:ifAbsent:", unescape("%0D%0A%09for%20i%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09if%20%28%25%3AisObject_equalTo_%25%28self%3Aunwrap_%28value%29%2CanObject%29%29%20then%20return%20table.remove%28self%2Ci%29%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20aBlock%3Avalue%28%29%20end")], ["add:", unescape("self%5B%23self+1%5D%3Dself%3Awrap_%28anObject%29")], ["addFirst:", unescape("table.insert%28self%2C1%2Cself%3Awrap_%28anObject%29%29")], ["addLast:", unescape("self%5B%23self+1%5D%3Dself%3Awrap_%28anObject%29")], ["removeAll", unescape("for%20i%3D%23self%2C1%2C-1%20do%20self%5Bi%5D%3Dnil%20end")], ["removeFirst", unescape("table.remove%28self%2C1%29")], ["removeIndex:", unescape("table.remove%28self%2Cindex%29")], ["removeLast", unescape("self%3Aunwrap_%28table.remove%28self%29%29")], ["atAllPut:", unescape("%0D%0A%09local%20what%3Dself%3Awrap_%28anObject%29%0D%0A%09for%20i%3D1%2C%23self%20do%20self%5Bi%5D%3Dwhat%20end")], ["first", unescape("%0D%0A%09local%20value%3Dself%5B1%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end")], ["second", unescape("%0D%0A%09local%20value%3Dself%5B2%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end")], ["third", unescape("%0D%0A%09local%20value%3Dself%5B3%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end")], ["fourth", unescape("%09%0D%0A%09local%20value%3Dself%5B4%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end")], ["fifth", unescape("%0D%0A%09local%20value%3Dself%5B5%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end")], ["sixth", unescape("%0D%0A%09local%20value%3Dself%5B6%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end")]]], ["RegularExpression class", [["fromString:flag:", unescape("self%3AnotImplementedYet_%28%22fromString%3Aflag%3A%22%29")]]], ["Exception", [["signal", unescape("error%28self%29")]]], ["ComposedAssociations", [["hashObject", unescape("%7B%7D")]]], ["KeyedCollection class", [[unescape("@"), unescape("%0D%0A%09parts%20%3D%20smalltalk.Array%3Anew%28%29%0D%0A%09for%20k%2C_%20in%20pairs%28jsObject%29%20do%20parts%3Aadd_%28k%29%20end")]]], ["KeyedCollection", [["newHash", unescape("%7B%7D")]]], ["Dictionary class", [["basicNew", unescape("if%20true%20then%20return%20smalltalk.PoolDictionary%3Anew%28%29%20end")]]], ["PoolDictionary", [["removeKey:ifAbsent:", unescape("self%5B%22@hash%22%5D%5Bself%3Awrap_%28aKey%29%5D%3Dnil")], ["at:ifAbsent:", unescape("%0D%0A%09local%20found%20%3D%20self%5B%22@hash%22%5D%5Bself%3Awrap_%28aKey%29%5D%0D%0A%09if%20found%7E%3Dnil%0D%0A%09%09then%20return%20self%3Aunwrap_%28found%29%0D%0A%09%09else%20return%20aBlock%3Avalue%28%29%0D%0A%09end")], ["includesKey:", unescape("self%5B%22@hash%22%5D%5Bself%3Awrap_%28aKey%29%5D%7E%3Dnil")], ["keys", unescape("%0D%0A%09local%20result%3Dsmalltalk.Array%3Anew%28%29%0D%0A%09for%20k%2C_%20in%20pairs%28self%5B%22@hash%22%5D%29%20do%20result%3Aadd_%28k%29%20end%0D%0A%09if%20true%20then%20return%20result%20end")]]], ["ClassBuilder", [["addSubclassOf:named:instanceVariableNames:", unescape("%0D%0A%09%25%3AaddClass%25%28aString%2C%20aClass%2C%20aCollection%29%0D%0A%09if%20true%20then%20return%20smalltalk%5BaString%5D%20end")]]], ["StringStream", [["peek", unescape("%0D%0A%09local%20c%20%3D%20self%5B%22@collection%22%5D%3Aat_%28self%3Aposition%28%29+1%29%0D%0A%09if%20c%3D%3D%22%22%20then%20return%20nil%20else%20return%20c%20end")], ["peekForAll:", unescape("%0D%0A%09local%20start%3Dself%3Aposition%28%29%0D%0A%09if%20%28self%5B%22@collection%22%5D%3Asub%28start+1%2Cstart+%23aCollection%29%7E%3DaCollection%29%20then%20return%20false%20end")]]], ["Random", [["next", unescape("if%20true%20then%20return%20math.random%28%29%20end")]]], ["DateTime class", [["new:", unescape("if%20true%20then%20return%20self%3AnotImplementedYet_%28%22new%3A%22%29%20end")], ["today", unescape("if%20true%20then%20return%20self%3Anew%28%29%20end")], ["now", unescape("if%20true%20then%20return%20self%3Atoday%28%29%20end")], ["msNow", unescape("if%20true%20then%20return%20os.clock%28%29*1000%20end")]]], ["DateTime", [[unescape("%3C"), unescape("self%3AnotImplementedYet_%28%22%3C%22%29")], [unescape("%3E"), unescape("self%3AnotImplementedYet_%28%22%3E%22%29")], [unescape("%3C%3D"), unescape("self%3AnotImplementedYet_%28%22%3C%3D%22%29")], [unescape("%3E%3D"), unescape("self%3AnotImplementedYet_%28%22%3E%3D%22%29")], [unescape("-"), unescape("self%3AnotImplementedYet_%28%22-%22%29")], [unescape("+"), unescape("self%3AnotImplementedYet_%28%22+%22%29")], ["isValid", unescape("self%3AnotImplementedYet_%28%22isValid%22%29")]]]];}
,"inlines",unescape("kernelInlines%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5E%23%28%0D%0A%23%28Object%20%23%28%0D%0A%23%28halt%09%09%27debug.debug%28%29%27%29%0D%0A%22%23%28toString%09%27local%20unused%27%29%22%0D%0A%23%28toString%3A%09%27tostring%28anObject%29%27%29%0D%0A%23%28isObject%3A%09%27%0D%0A%09local%20status%2C%20result%20%3D%20pcall%28function%28%29%20return%20type%28anObject.yourself%29%3D%3D%22function%22%20end%29%0D%0A%09if%20true%20then%20return%20status%20and%20result%20end%27%29%0D%0A%23%28typeOf%3A%09%27type%28anObject%29%27%29%0D%0A%22%23%28callableOf%3A%09%27local%20unused%27%29%22%0D%0A%23%28basicDelete%3A%09%27if%20self%3D%3Dnil%20then%20return%20else%20self%5BaString%5D%3Dnil%20end%27%29%0D%0A%23%28basicPerform%3AwithArguments%3A%09%23%28%27self%27%20%27fn%28self%2C%28table.unpack%20or%20unpack%29%28anArray%29%29%27%29%29%0D%0A%23%28asJSONObject%09%27%7B%7D%27%29%0D%0A%29%29%0D%0A%23%28Smalltalk%20%23%28%0D%0A%23%28includesKey%3A%09%27%7B%7D%27%29%0D%0A%23%28eval%3A%09%27%0D%0A%09--%20%28req%20v5.3%29%20local%20fn%2Cerr%20%20%3D%20load%28anExpression%29%0D%0A%09function%20f%28%29%20local%20r%20%3D%20anExpression%3B%20anExpression%3D%20%22%22%3B%20return%20r%20end%20--%20%28req%20v5.1%29%0D%0A%09local%20fn%2Cerr%20%20%3D%20load%28f%29%0D%0A%09if%20fn%20then%20return%20fn%28%29%20else%0D%0A%09%09self%3Anote_%28%22%20eval%3A%20Error%20%22..err%29%20--the%20expression%20must%20be%20a%20lua%20expression%20%28including%20return%29%0D%0A%09%09self%3Anote_%28%22%20expression%3D%20%22..anExpression%29%0D%0A%09%09return%20nil%0D%0A%09end%27%29%0D%0A%29%29%0D%0A%23%28%27Smalltalk%20class%27%20%23%28%0D%0A%23%28global%09%27_G%27%29%0D%0A%23%28gc%09%27if%20collectgarbage%28%29%20then%20return%20true%20end%27%29%0D%0A%29%29%0D%0A%23%28Behavior%20%23%28%0D%0A%23%28basicNew%09%27self%5B%22%24fn%22%5D%28%29%27%29%0D%0A%22%23%28fn%09%09%27local%20unused%27%29%22%0D%0A%22%23%28prototype%09%27local%20unused%27%29%22%0D%0A%22%23%28privateMethods%09%27local%20unused%27%29%22%0D%0A%29%29%0D%0A%23%28CompiledMethod%20%23%28%0D%0A%22%23%28asClosure%09%27local%20unused%27%29%22%0D%0A%29%29%0D%0A%23%28Boolean%20%23%28%0D%0A%23%28not%20%27self%20%3D%3D%20false%27%29%0D%0A%23%28%3D%09%27%0D%0A%09if%20anObject%3D%3Dnil%20then%20return%20false%20end%0D%0A%09if%20true%20then%20return%20self%3D%3DanObject%20end%27%29%0D%0A%23%28ifTrue%3AifFalse%3A%20%27%0D%0A%09if%20%28self%3D%3Dtrue%29%0D%0A%09%09then%20return%20aBlock%3Avalue%28%29%0D%0A%09%09else%20return%20anotherBlock%3Avalue%28%29%20end%27%29%0D%0A%29%29%0D%0A%23%28Number%20%23%28%0D%0A%23%28%3D%20%27%0D%0A%09if%20%28type%28anObject%29%3D%3D%22number%22%29%0D%0A%09%09then%20return%20self%3D%3DanObject%0D%0A%09%09else%20return%20false%0D%0A%09%09end%27%29%0D%0A%23%28%3E%20%27self%20%3E%20aNumber%27%29%0D%0A%23%28%3E%3D%20%27self%20%3E%3D%20aNumber%27%29%0D%0A%23%28%3C%20%27self%20%3C%20aNumber%27%29%0D%0A%23%28%3C%3D%20%27self%20%3C%3D%20aNumber%27%29%0D%0A%23%28+%20%27self%20+%20aNumber%27%29%0D%0A%23%28-%20%27self%20-%20aNumber%27%29%0D%0A%23%28*%20%27self%20*%20aNumber%27%29%0D%0A%23%28/%20%27self%20/%20aNumber%27%29%0D%0A%23%28sin%20%27math.sin%28self%29%27%29%0D%0A%23%28cos%20%27math.cos%28self%29%27%29%0D%0A%23%28tan%20%27math.tan%28self%29%27%29%0D%0A%23%28arcCos%20%27math.acos%28self%29%27%29%0D%0A%23%28arcSin%20%27math.asin%28self%29%27%29%0D%0A%23%28arcTan%20%27math.atan%28self%29%27%29%0D%0A%23%28atan2%3A%20%27math.atan2%28self%2CaNumber%29%27%29%0D%0A%23%28sqrt%20%27math.sqrt%28self%29%27%29%0D%0A%23%28%3C%3C%20%27bit.lshift%28self%2CaNumber%29%27%29%0D%0A%23%28%3E%3E%20%27bit.rshift%28self%2CaNumber%29%27%29%0D%0A%23%28raisedTo%3A%20%27math.pow%28self%2CaNumber%29%27%29%0D%0A%23%28max%3A%20%27math.max%28self%2CaNumber%29%27%29%0D%0A%23%28min%3A%20%27math.min%28self%2CaNumber%29%27%29%0D%0A%23%28rounded%20%27self%20+%200.5%20-%20%28self%20+%200.5%29%20%25%25%201%27%29%0D%0A%23%28truncated%20%27math.floor%28self%29%27%29%0D%0A%23%28timesRepeat%3A%20%27for%20i%3D1%2Cself%20do%20fn%28i%29%20end%27%29%0D%0A%23%28to%3Ado%3A%20%27for%20i%3Dself%2CaNumber%20do%20fn%28i%29%20end%27%29%0D%0A%23%28to%3Aby%3Ado%3A%20%27for%20i%3Dself%2CaNumber%2Cincrement%20do%20fn%28i%29%20end%27%29%0D%0A%23%28printString%20%27tostring%28self%29%27%29%0D%0A%23%28toString%3A%20%27%25%3Abasen%25%28self%2Cradix%29%27%29%0D%0A%23%28isNumber%20%27if%20%28self%7E%3Dself%29%20then%20return%20false%20else%20return%20%28self%7E%3Dmath.huge%20and%20self%7E%3D-math.huge%29%20end%27%29%0D%0A%23%28clearInterval%20%27if%20true%20then%20self%3AnotImplementedYet_%28%22clearInterval%22%29%20end%27%29%0D%0A%23%28clearTimeout%20%27if%20true%20then%20self%3AnotImplementedYet_%28%22clearTimeout%22%29%20end%27%29%0D%0A%29%29%0D%0A%23%28%27Number%20class%27%20%23%28%0D%0A%23%28pi%20%27math.pi%27%29%0D%0A%29%29%0D%0A%23%28BlockClosure%20%23%28%0D%0A%23%28whileTrue%3A%20%27while%20self%28%29%20do%20fn%28%29%20end%27%29%0D%0A%23%28whileFalse%3A%20%27while%20%28self%28%29%3D%3Dfalse%29%20do%20fn%28%29%20end%27%29%0D%0A%23%28valueWithPossibleArguments%3A%20%27self%28%28table.unpack%20or%20unpack%29%28array%29%29%27%29%0D%0A%23%28try%3Acatch%3A%20%27%0D%0A%09local%20success%2C%20result%20%3D%20pcall%28function%28%29%20return%20fn%28%29%20end%29%0D%0A%09if%20success%0D%0A%09%09then%20return%20result%0D%0A%09%09else%20return%20ab%28%20self%3AsmalltalkErrorOf_%28result%29%20%29%20end%27%29%0D%0A%23%28valueWithTimeout%3A%20%27self%3AnotImplementedYet_%28%22valueWithTimeout%3A%22%29%27%29%0D%0A%23%28valueWithInterval%3A%20%27self%3AnotImplementedYet_%28%22valueWithInterval%3A%22%29%27%29%0D%0A%23%28ensure%3A%20%27%0D%0A%09local%20success%2C%20result%20%3D%20pcall%28function%28%29%20return%20self%28%29%20end%29%0D%0A%09fn%28%29%0D%0A%09if%20true%20then%20return%20result%20end%27%29%0D%0A%23%28ifCurtailed%3A%20%27%0D%0A%09local%20curtailed%20%3D%20true%0D%0A%09local%20success%2C%20result%20%3D%20pcall%28function%28%29%0D%0A%09%09local%20r%20%3D%20self%28%29%0D%0A%09%09curtailed%20%3D%20false%0D%0A%09%09return%20r%0D%0A%09end%29%0D%0A%09if%20%28curtailed%29%0D%0A%09%09then%20exceptionBlock%3Avalue%28%29%0D%0A%09%09else%20return%20result%0D%0A%09%09end%27%29%0D%0A%23%28argumentCount%20%27self%3AnotImplementedYet_%28%22argumentCount%22%29%27%29%0D%0A%29%29%0D%0A%23%28UndefinedObject%20%20%23%28%0D%0A%23%28asJSONObject%20%27nil%27%29%0D%0A%29%29%0D%0A%23%28Collection%20%20%23%28%0D%0A%23%28wrap%3A%20%27if%20%25%3AisNil_%25%28anObject%29%20--anObject%3D%3Dnil%0D%0A%09then%20return%20smalltalk.UndefinedObject%3Aalias%28%29%0D%0A%09else%20return%20anObject%20end%27%29%0D%0A%23%28unwrap%3A%20%27--if%20anObject%3D%3Dnil%20then%20return%20nil%20end%0D%0A%09--if%20%28type%28anObject%29%3D%3D%22table%22%29%20and%20%25%3AisNil_%25%28anObject%29%0D%0A%09if%20%25%3AisNil_%25%28anObject%29%0D%0A%09%09then%20return%20nil%0D%0A%09%09else%20return%20anObject%20end%27%29%0D%0A%29%29%0D%0A%23%28HashedSet%20%23%28%0D%0A%23%28initializeHash%20%27%7B%7D%27%29%0D%0A%23%28includes%3A%20%27%28self%5B%22@hash%22%5D%5Bcookie%5D%29%7E%3Dnil%27%29%0D%0A%22%23%28privateAdd%3A%20%27local%20unused%27%29%22%0D%0A%23%28privateRemove%3A%20%27self%5B%22@hash%22%5D%5Bcookie%5D%20%3D%20nil%27%29%0D%0A%29%29%0D%0A%23%28SequenceableCollection%20%23%28%0D%0A%23%28indexOf%3AifAbsent%3A%20%27%0D%0A%09for%20i%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09if%20%28%25%3AisObject_equalTo_%25%28self%3Aunwrap_%28value%29%2CanObject%29%29%20then%20return%20i%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20aBlock%3Avalue%28%29%20end%27%29%0D%0A%23%28do%3A%20%27%0D%0A%09for%20_%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09fn%28self%3Aunwrap_%28value%29%29%0D%0A%09end%27%29%0D%0A%23%28detect%3AifNone%3A%20%27%0D%0A%09for%20_%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09local%20v%3Dself%3Aunwrap_%28value%29%0D%0A%09%09if%20%28fn%28v%29%29%20then%20return%20v%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20anotherBlock%3Avalue%28%29%20end%27%29%0D%0A%23%28reversed%20%27%0D%0A%09local%20result%20%3D%20smalltalk.Array%3Anew%28%29%0D%0A%09local%20top%20%3D%20%23self%20+%201%0D%0A%09for%20i%2Cv%20in%20ipairs%28self%29%20do%20result%5Btop%20-%20i%5D%20%3D%20v%20end%0D%0A%09if%20true%20then%20return%20result%20end%27%29%0D%0A%23%28isEmpty%20%27%23self%3D%3D0%27%29%0D%0A%23%28reverseDo%3A%20%20%27for%20i%20%3D%20%23self%2C1%2C-1%20do%20fn%28self%3Aunwrap_%28self%5Bi%5D%29%29%20end%27%29%0D%0A%23%28includes%3A%20%23%28%27%0D%0A%09for%20_%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09if%20%28%25%3AisObject_equalTo_%25%28self%3Aunwrap_%28value%29%2CanObject%29%29%20then%20return%20true%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20false%20end%27%20%27if%20true%20then%20end%27%29%29%0D%0A%23%28occurrencesOf%3A%20%27%0D%0A%09local%20count%20%3D%200%0D%0A%09for%20_%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09if%20%28%25%3AisObject_equalTo_%25%28self%3Aunwrap_%28value%29%2CanObject%29%29%20then%20count%20%3D%20count%20+%201%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20count%20end%27%29%0D%0A%29%29%0D%0A%23%28%27String%20class%27%20%23%28%0D%0A%23%28fromString%3A%20%27if%20true%20then%20tostring%28aString%29%20end%27%29%0D%0A%23%28new%09%27%22%22%27%29%0D%0A%23%28cr%20%09%27%22%5Cr%22%27%29%0D%0A%23%28lf%20%09%27%22%5Cn%22%27%29%0D%0A%23%28crlf%20%09%27%22%5Cr%5Cn%22%27%29%0D%0A%23%28space%09%27%22%20%22%27%29%0D%0A%23%28tab%09%27%22%5Ct%22%27%29%0D%0A%23%28zero%09%27string.char%280%29%27%29%0D%0A%23%28fromCharCode%3A%20%27%0D%0A%09if%20%28type%28anArrayOrNumber%29%3D%3D%22number%22%29%20then%0D%0A%09%09return%20string.char%28anArrayOrNumber%29%0D%0A%09end%0D%0A%09if%20true%20then%0D%0A%09%09local%20fup%20%3D%20table.unpack%20or%20unpack%0D%0A%09%09return%20%28string.char%28fup%28anArrayOrNumber%29%29%29%0D%0A%09end%27%29%0D%0A%29%29%0D%0A%23%28String%20%23%28%0D%0A%23%28%3D%09%27if%20true%20then%20return%20self%3D%3DanObject%20end%27%29%0D%0A%23%28size%09%27%23self%27%29%0D%0A%23%28at%3A%09%27self%3Asub%28anIndex%2CanIndex%29%27%29%0D%0A%23%28do%3A%20%09%27for%20c%20in%20self%3Agmatch%28%22.%22%29%20do%20fn%28c%29%20end%27%29%0D%0A%23%28withIndexDo%3A%20%27for%20i%3D1%2C%23self%20do%20fn%28self%3Asub%28i%2Ci%29%2Ci%29%20end%27%29%0D%0A%23%28detect%3AifNone%3A%20%27%0D%0A%09for%20c%20in%20self%3Agmatch%28%22.%22%29%20do%0D%0A%09%09if%20%28fn%28c%29%29%20then%20return%20c%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20anotherBlock%3Avalue%28%29%20end%27%29%0D%0A%23%28isLiteral%20%27%0D%0A%09if%20%23self%3D%3D0%20then%20return%20false%20end%0D%0A%09if%20smalltalk.String%3AisBinarySelector_%28self%29%20then%20return%20true%20end%0D%0A%09if%20true%20then%20return%20string.find%28self%2C%22%5B%5Ea-zA-Z0-9_%3A%5D+%22%29%3D%3Dnil%20end%27%29%0D%0A%23%28occurrencesOf%3A%20%27%0D%0A%09local%20j%0D%0A%09local%20k%2Ccount%20%3D%200%2C0%0D%0A%09repeat%09j%2C%20k%20%3D%20self%3Afind%28aString%2C%20k%20+%201%2C%20true%29%0D%0A%09%09if%20%28j%7E%3Dnil%29%20then%20count%3Dcount+1%20end%0D%0A%09until%20j%20%3D%3D%20nil%0D%0A%09if%20true%20then%20return%20count%20end%27%29%0D%0A%23%28startsWith%3A%20%27if%20true%20then%20return%20%28self%3Asub%281%2C%20%23aString%29%20%3D%3D%20aString%29%20end%27%29%0D%0A%23%28escaped%20%27self%3AnotImplementedYet_%28%22escaped%22%29%27%29%0D%0A%23%28unescaped%20%27%0D%0A%09local%20x16%20%3D%20function%28x%29%20return%20string.char%28tonumber%28x%2C%2016%29%29%20end%0D%0A%09if%20true%20then%20return%20self%3Agsub%28%22%25%25%25%25%28%25%25x%25%25x%29%22%2C%20x16%29%20end%27%29%0D%0A%23%28asEncodedURI%20%27self%3AnotImplementedYet_%28%22asEncodedURI%22%29%27%29%0D%0A%23%28asDecodedURI%20%27self%3AnotImplementedYet_%28%22asDecodedURI%22%29%27%29%0D%0A%23%28%2C%20%27if%20true%20then%20return%20self..%28%25%3AasString_%25%28aCollection%29%29%20end%27%29%0D%0A%23%28copyWith%3A%20%27self..%28%25%3AasString_%25%28anObject%29%29%27%29%0D%0A%23%28copyFrom%3Ato%3A%20%27self%3Asub%28anIndex%2CanotherIndex%29%27%29%0D%0A%23%28reversed%20%27if%20true%20then%20return%20self%3Areverse%28%29%20end%27%29%0D%0A%23%28includes%3A%20%27local%20ignored%27%29%0D%0A%23%28indexOf%3A%20%27%0D%0A%09local%20index%20%3D%20self%3Afind%28aString%2C1%2Ctrue%29%0D%0A%09if%20not%20index%0D%0A%09%09then%20return%200%0D%0A%09%09else%20return%20index%20end%27%29%0D%0A%23%28indexOf%3AstartingAt%3A%20%27%0D%0A%09local%20index%20%3D%20self%3Afind%28aString%2Cposition%2Ctrue%29%0D%0A%09if%20not%20index%0D%0A%09%09then%20return%200%0D%0A%09%09else%20return%20index%20end%27%29%0D%0A%23%28lastIndexOf%3A%20%27%0D%0A%09local%20i%20%3D%200%0D%0A%09local%20j%20%3D%200%0D%0A%09local%20k%20%3D%200%0D%0A%09repeat%09i%20%3D%20j%0D%0A%09%09j%2C%20k%20%3D%20self%3Afind%28aString%2C%20k%20+%201%2C%20true%29%0D%0A%09until%20j%20%3D%3D%20nil%0D%0A%09if%20true%20then%20return%20i%20end%27%29%0D%0A%23%28lastIndexOf%3AstartingAt%3A%20%27%0D%0A%09local%20i%2C%20j%0D%0A%09local%20k%20%3D%20position%0D%0A%09repeat%09i%20%3D%20j%0D%0A%09%09j%2C%20k%20%3D%20self%3Afind%28aString%2C%20k%20+%201%2C%20true%29%0D%0A%09until%20j%20%3D%3D%20nil%0D%0A%09if%20true%20then%20return%20i%20end%27%29%0D%0A%23%28asNativeName%20%27self%3AasLuaName%28%29%27%29%0D%0A%23%28asPrivateSelector%20%27local%20map%20%3D%20%7B%0D%0A%09%09%5B%22%3A%22%5D%3D%22_%22%2C%0D%0A%09%09%5B%22+%22%5D%3D%22_plus%22%2C%0D%0A%09%09%5B%22-%22%5D%3D%22_minus%22%2C%0D%0A%09%09%5B%22*%22%5D%3D%22_star%22%2C%0D%0A%09%09%5B%22/%22%5D%3D%22_slash%22%2C%0D%0A%09%09%5B%22%7C%22%5D%3D%22_bar%22%2C%0D%0A%09%09%5B%22%26%22%5D%3D%22_amp%22%2C%0D%0A%09%09%5B%22%3E%22%5D%3D%22_gt%22%2C%0D%0A%09%09%5B%22%3C%22%5D%3D%22_lt%22%2C%0D%0A%09%09%5B%22%3D%22%5D%3D%22_eq%22%2C%0D%0A%09%09%5B%22%2C%22%5D%3D%22_comma%22%2C%0D%0A%09%09%5B%22@%22%5D%3D%22_at%22%2C%0D%0A%09%09%5B%22%7E%22%5D%3D%22_no%22%2C%0D%0A%09%09%5B%22%5C%5C%22%5D%3D%22_bk%22%0D%0A%09%7D%3B%0D%0A%09result%20%3D%20self%3Agsub%28%22.%22%2Cfunction%28c%29%20return%20map%5Bc%5D%20end%29%27%29%0D%0A%23%28asNative%3A%20%27if%20true%20then%20return%20self%3AslashEncoded%28%29%20end%27%29%0D%0A%23%28replace%3Awith%3A%20%27%0D%0A%09local%20result%3D%22%22%0D%0A%09local%20i%2Cj%0D%0A%09local%20k%20%3D%200%0D%0A%09repeat%09i%3Dk+1%0D%0A%09%09j%2C%20k%20%3D%20self%3Afind%28aString%2C%20i%2C%20true%29%0D%0A%09%09if%20%28j%3D%3Dnil%29%20then%0D%0A%09%09%09result%3Dresult..%28self%3Asub%28i%29%29%0D%0A%09%09else%0D%0A%09%09%09result%3Dresult..%28self%3Asub%28i%2Cj-1%29..anotherString%29%0D%0A%09%09end%0D%0A%09until%20j%20%3D%3D%20nil%0D%0A%09if%20true%20then%20return%20result%20end%27%29%0D%0A%23%28replaceRegexp%3Awith%3A%20%27self%3AnotImplementedYet_%28%22replaceRegexp%3Awith%3A%22%29%27%29%0D%0A%23%28tokenize%3A%20%27%0D%0A%09local%20result%3Dsmalltalk.Array%3Anew%28%29%0D%0A%09local%20i%2Cj%0D%0A%09local%20k%20%3D%200%0D%0A%09repeat%09i%3Dk+1%0D%0A%09%09j%2C%20k%20%3D%20self%3Afind%28aString%2C%20i%2C%20true%29%0D%0A%09%09if%20%28j%3D%3Dnil%29%20then%0D%0A%09%09%09result%3Aadd_%28self%3Asub%28i%29%29%0D%0A%09%09else%0D%0A%09%09%09result%3Aadd_%28self%3Asub%28i%2Cj-%23aString%29%29%0D%0A%09%09end%0D%0A%09until%20j%20%3D%3D%20nil%0D%0A%09if%20true%20then%20return%20result%20end%27%29%0D%0A%23%28match%3A%20%27self%3AnotImplementedYet_%28%22match%3A%22%29%27%29%0D%0A%23%28isLetter%20%27%23self%3D%3D1%20and%20string.find%28self%2C%22%25%25a%22%29%3D%3D1%27%29%0D%0A%23%28isDigit%20%27%23self%3D%3D1%20and%20string.find%28self%2C%22%25%25d%22%29%3D%3D1%27%29%0D%0A%22%23%28isAlphaNumeric%20%27local%20unused%27%29%22%0D%0A%23%28isUpperCase%20%27%0D%0A%09if%20%28%23self%3D%3D0%29%20then%20return%20false%20end%0D%0A%09for%20c%20in%20self%3Agmatch%28%22.%22%29%20do%0D%0A%09%09if%20%28c%3C%22A%22%20or%20c%3E%22Z%22%29%20then%20return%20false%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20true%20end%27%29%0D%0A%23%28isLowerCase%20%27%0D%0A%09if%20%28%23self%3D%3D0%29%20then%20return%20false%20end%0D%0A%09for%20c%20in%20self%3Agmatch%28%22.%22%29%20do%0D%0A%09%09if%20%28c%3C%22a%22%20or%20c%3E%22z%22%29%20then%20return%20false%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20true%20end%27%29%0D%0A%23%28asUppercase%20%27self%3Aupper%28%29%27%29%0D%0A%23%28asLowercase%20%27self%3Alower%28%29%27%29%0D%0A%23%28asNumber%20%27tonumber%28self%29%27%29%0D%0A%23%28%3E%20%27self%20%3E%20aString%27%29%0D%0A%23%28%3C%20%27self%20%3C%20aString%27%29%0D%0A%23%28%3E%3D%20%27self%20%3E%3D%20aString%27%29%0D%0A%23%28%3C%3D%20%27self%20%3C%3D%20aString%27%29%0D%0A%23%28isCharacter%20%27%23self%20%3D%3D%201%27%29%0D%0A%23%28asciiValue%20%27string.byte%28self%29%27%29%0D%0A%23%28isSeparator%20%27%23self%3D%3D1%20and%20self%3Afind%28%22%25%25s%22%29%3D%3D1%27%29%0D%0A%29%29%0D%0A%23%28%27Array%20class%27%20%23%28%0D%0A%23%28basicNew%3A%20%27%25%3AnewArray%25%28%29%27%29%0D%0A%23%28with%3A%20%27self%27%29%0D%0A%23%28with%3Awith%3A%20%27self%27%29%0D%0A%23%28with%3Awith%3Awith%3A%20%27self%27%29%0D%0A%23%28with%3Awith%3Awith%3Awith%3A%20%27self%27%29%0D%0A%23%28withAll%3A%20%27%0D%0A%09local%20result%20%3D%20aCollection%3AasArray%28%29%0D%0A%09if%20aCollection%3D%3Dresult%0D%0A%09%09then%20return%20result%3Acopy%28%29%0D%0A%09%09else%20return%20result%0D%0A%09end%27%29%0D%0A%23%28slice%3A%20%27self%3AnotImplementedYet_%28%22slice%3A%22%29%27%29%0D%0A%29%29%0D%0A%23%28Array%20%23%28%0D%0A%23%28size%20%27%23self%27%29%0D%0A%23%28at%3A%20%27%0D%0A%09local%20value%20%3D%20self%5BanIndex%5D%0D%0A%09if%20value%3D%3Dnil%20then%20return%20self%3AobjectNotFoundAt_%28anIndex%29%20end%0D%0A%09if%20true%20then%20return%20self%3Aunwrap_%28value%29%20end%27%29%0D%0A%23%28at%3AifAbsent%3A%20%27%0D%0A%09local%20value%20%3D%20self%5BanIndex%5D%0D%0A%09if%20%28value%3D%3Dnil%29%20then%20return%20aBlock%3Avalue%28%29%20end%0D%0A%09if%20true%20then%20return%20self%3Aunwrap_%28value%29%20end%27%29%0D%0A%23%28at%3Aput%3A%20%27%0D%0A%09self%5BanIndex%5D%3Dself%3Awrap_%28anObject%29%0D%0A%09if%20true%20then%20return%20anObject%20end%27%29%0D%0A%23%28do%3A%20%27%0D%0A%09for%20_%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09fn%28self%3Aunwrap_%28value%29%29%0D%0A%09end%27%29%0D%0A%23%28addAll%3A%20%27%0D%0A%09local%20array%20%3D%20aCollection%3AasArray%28%29%0D%0A%09for%20i%3D1%2C%23array%20do%20self%5B%23self+1%5D%3Dself%3Awrap_%28array%5Bi%5D%29%20end%27%29%0D%0A%23%28shallowCopy%20%27%0D%0A%09result%20%3D%20self%3Aclass%28%29%3Anew%28%29%0D%0A%09for%20i%3D1%2C%23self%20do%20result%5B%23result+1%5D%3Dself%5Bi%5D%20end%27%29%0D%0A%23%28copyWithAll%3A%20%27%0D%0A%09local%20result%20%3D%20self%3Acopy%28%29%0D%0A%09result%3AaddAll_%28aCollection%29%0D%0A%09if%20true%20then%20return%20result%20end%27%29%0D%0A%23%28%2C%20%27%0D%0A%09local%20result%20%3D%20self%3Acopy%28%29%0D%0A%09result%3AaddAll_%28aCollection%29%0D%0A%09if%20true%20then%20return%20result%20end%27%29%0D%0A%23%28join%3A%20%27table.concat%28self%2CaString%29%27%29%0D%0A%23%28sort%3A%20%27table.sort%28self%2Cfn%29%27%29%0D%0A%23%28remove%3AifAbsent%3A%20%27%0D%0A%09for%20i%2Cvalue%20in%20ipairs%28self%29%20do%0D%0A%09%09if%20%28%25%3AisObject_equalTo_%25%28self%3Aunwrap_%28value%29%2CanObject%29%29%20then%20return%20table.remove%28self%2Ci%29%20end%0D%0A%09end%0D%0A%09if%20true%20then%20return%20aBlock%3Avalue%28%29%20end%27%29%0D%0A%23%28add%3A%20%27self%5B%23self+1%5D%3Dself%3Awrap_%28anObject%29%27%29%0D%0A%23%28addFirst%3A%20%27table.insert%28self%2C1%2Cself%3Awrap_%28anObject%29%29%27%29%0D%0A%23%28addLast%3A%20%27self%5B%23self+1%5D%3Dself%3Awrap_%28anObject%29%27%29%0D%0A%23%28removeAll%20%27for%20i%3D%23self%2C1%2C-1%20do%20self%5Bi%5D%3Dnil%20end%27%29%0D%0A%23%28removeFirst%20%27table.remove%28self%2C1%29%27%29%0D%0A%23%28removeIndex%3A%20%27table.remove%28self%2Cindex%29%27%29%0D%0A%23%28removeLast%20%27self%3Aunwrap_%28table.remove%28self%29%29%27%29%0D%0A%23%28atAllPut%3A%20%27%0D%0A%09local%20what%3Dself%3Awrap_%28anObject%29%0D%0A%09for%20i%3D1%2C%23self%20do%20self%5Bi%5D%3Dwhat%20end%27%29%0D%0A%23%28first%20%27%0D%0A%09local%20value%3Dself%5B1%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end%27%29%0D%0A%23%28second%20%27%0D%0A%09local%20value%3Dself%5B2%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end%27%29%0D%0A%23%28third%20%27%0D%0A%09local%20value%3Dself%5B3%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end%27%29%0D%0A%23%28fourth%20%27%09%0D%0A%09local%20value%3Dself%5B4%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end%27%29%0D%0A%23%28fifth%20%27%0D%0A%09local%20value%3Dself%5B5%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end%27%29%0D%0A%23%28sixth%20%27%0D%0A%09local%20value%3Dself%5B6%5D%0D%0A%09if%20%28value%7E%3Dnil%29%20then%20return%20self%3Aunwrap_%28value%29%20end%27%29%0D%0A%29%29%0D%0A%23%28%27RegularExpression%20class%27%20%23%28%0D%0A%23%28fromString%3Aflag%3A%20%27self%3AnotImplementedYet_%28%22fromString%3Aflag%3A%22%29%27%29%0D%0A%29%29%0D%0A%23%28Exception%20%23%28%0D%0A%23%28signal%20%27error%28self%29%27%29%0D%0A%29%29%0D%0A%23%28ComposedAssociations%20%23%28%0D%0A%23%28hashObject%20%27%7B%7D%27%29%0D%0A%29%29%0D%0A%23%28%27KeyedCollection%20class%27%20%23%28%0D%0A%23%28@%20%27%0D%0A%09parts%20%3D%20smalltalk.Array%3Anew%28%29%0D%0A%09for%20k%2C_%20in%20pairs%28jsObject%29%20do%20parts%3Aadd_%28k%29%20end%27%29%0D%0A%29%29%0D%0A%23%28KeyedCollection%20%23%28%0D%0A%23%28newHash%20%27%7B%7D%27%29%0D%0A%29%29%0D%0A%23%28%27Dictionary%20class%27%20%23%28%0D%0A%23%28basicNew%20%27if%20true%20then%20return%20smalltalk.PoolDictionary%3Anew%28%29%20end%27%29%0D%0A%29%29%0D%0A%23%28PoolDictionary%20%23%28%0D%0A%22%23%28validKey%3A%20%27local%20unused%27%29%22%0D%0A%22%23%28at%3Aput%3A%20%27local%20unused%27%29%22%0D%0A%23%28removeKey%3AifAbsent%3A%20%27self%5B%22@hash%22%5D%5Bself%3Awrap_%28aKey%29%5D%3Dnil%27%29%0D%0A%23%28at%3AifAbsent%3A%20%27%0D%0A%09local%20found%20%3D%20self%5B%22@hash%22%5D%5Bself%3Awrap_%28aKey%29%5D%0D%0A%09if%20found%7E%3Dnil%0D%0A%09%09then%20return%20self%3Aunwrap_%28found%29%0D%0A%09%09else%20return%20aBlock%3Avalue%28%29%0D%0A%09end%27%29%0D%0A%23%28includesKey%3A%20%27self%5B%22@hash%22%5D%5Bself%3Awrap_%28aKey%29%5D%7E%3Dnil%27%29%0D%0A%23%28keys%20%27%0D%0A%09local%20result%3Dsmalltalk.Array%3Anew%28%29%0D%0A%09for%20k%2C_%20in%20pairs%28self%5B%22@hash%22%5D%29%20do%20result%3Aadd_%28k%29%20end%0D%0A%09if%20true%20then%20return%20result%20end%27%29%0D%0A%29%29%0D%0A%23%28ClassBuilder%20%23%28%0D%0A%23%28addSubclassOf%3Anamed%3AinstanceVariableNames%3A%20%27%0D%0A%09%25%3AaddClass%25%28aString%2C%20aClass%2C%20aCollection%29%0D%0A%09if%20true%20then%20return%20smalltalk%5BaString%5D%20end%27%29%0D%0A%29%29%0D%0A%23%28StringStream%20%23%28%0D%0A%23%28peek%20%27%0D%0A%09local%20c%20%3D%20self%5B%22@collection%22%5D%3Aat_%28self%3Aposition%28%29+1%29%0D%0A%09if%20c%3D%3D%22%22%20then%20return%20nil%20else%20return%20c%20end%27%29%0D%0A%23%28peekForAll%3A%20%27%0D%0A%09local%20start%3Dself%3Aposition%28%29%0D%0A%09if%20%28self%5B%22@collection%22%5D%3Asub%28start+1%2Cstart+%23aCollection%29%7E%3DaCollection%29%20then%20return%20false%20end%27%29%0D%0A%29%29%0D%0A%23%28Random%20%23%28%0D%0A%23%28next%20%27if%20true%20then%20return%20math.random%28%29%20end%27%29%0D%0A%29%29%0D%0A%23%28%27DateTime%20class%27%20%23%28%0D%0A%23%28new%3A%20%27if%20true%20then%20return%20self%3AnotImplementedYet_%28%22new%3A%22%29%20end%27%29%0D%0A%23%28today%20%27if%20true%20then%20return%20self%3Anew%28%29%20end%27%29%0D%0A%23%28now%20%27if%20true%20then%20return%20self%3Atoday%28%29%20end%27%29%0D%0A%23%28msNow%20%27if%20true%20then%20return%20os.clock%28%29*1000%20end%27%29%0D%0A%29%29%0D%0A%23%28DateTime%20%23%28%0D%0A%23%28%3C%20%27self%3AnotImplementedYet_%28%22%3C%22%29%27%29%0D%0A%23%28%3E%20%27self%3AnotImplementedYet_%28%22%3E%22%29%27%29%0D%0A%23%28%3C%3D%20%27self%3AnotImplementedYet_%28%22%3C%3D%22%29%27%29%0D%0A%23%28%3E%3D%20%27self%3AnotImplementedYet_%28%22%3E%3D%22%29%27%29%0D%0A%23%28-%20%27self%3AnotImplementedYet_%28%22-%22%29%27%29%0D%0A%23%28+%20%27self%3AnotImplementedYet_%28%22+%22%29%27%29%0D%0A%23%28isValid%20%27self%3AnotImplementedYet_%28%22isValid%22%29%27%29%0D%0A%29%29%0D%0A%09%29%20"));
smalltalk.bind(smalltalk.LuaEncoder.$klass,"initializeNativeInlines:",0
,function(pool){var self=this;
return self.register_into_(self.nativeInlines(),pool);}
,"inlines",unescape("initializeNativeInlines%3A%20pool%0D%0A%09%22%20Private%20-%20Initialize%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5Eself%20register%3A%20self%20nativeInlines%20into%3A%20pool%20"));
smalltalk.bind(smalltalk.LuaEncoder.$klass,"nativeInlines",0
,function(){var self=this;
return [["NativeObject class", [["undefined", "nil"], ["codeToNumber", unescape("%22%20%22")]]], ["NativeObject", [["jsObject", unescape("self%3AinvalidMessage_%28%22jsObject%22%29")], ["ownPropertyNames", unescape("%0D%0A%09local%20result%20%3D%20smalltalk.Array%3Anew%28%29%0D%0A%09for%20k%2C_%20in%20pairs%28self%5B%22@handle%22%5D%29%20do%0D%0A%09%09result%3Aadd_%28k%29%0D%0A%09end%0D%0A%09if%20true%20then%20return%20result%20end")]]], ["NativeHandle", [["hasOwnProperty:", unescape("rawget%28self%5B%22@handle%22%5D%2CaName%29%7E%3Dnil")], ["delete:", unescape("self%5B%22@handle%22%5D%5BaName%5D%3Dnil")]]], ["NativeLibraryObject class", [["basicLoad:", unescape("require%28url%29")]]], ["NativeLibraryObject", [["newHandleOf:", unescape("result%20%3D%20aConstructor%28%29")], ["newHandleOf:with:", unescape("result%20%3D%20aConstructor%28arg1%29")], ["newHandleOf:with:with:", unescape("result%20%3D%20aConstructor%28arg1%2Carg2%29")], ["newHandleOf:with:with:with:", unescape("result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%29")], ["newHandleOf:with:with:with:with:", unescape("result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%29")], ["newHandleOf:with:with:with:with:with:", unescape("result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%29")], ["newHandleOf:with:with:with:with:with:with:", unescape("result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%29")], ["newHandleOf:with:with:with:with:with:with:with:", unescape("result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%2Carg7%29")], ["newHandleOf:with:with:with:with:with:with:with:with:", unescape("result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%2Carg7%2Carg8%29")]]], ["Array", [["addAllLua:", unescape("for%20_%2Cvalue%20in%20pairs%28natives%29%20do%20self%3Aadd_%28value%29%20end")]]], ["Array class", [[unescape("@@"), "self"], [unescape("@"), unescape("result%3AaddAllLua_%28natives%29")]]], ["Boolean class", [[unescape("@"), unescape("%0D%0A%09if%20%28aBOOL%3D%3Dtrue%29%20then%20return%20true%20end%0D%0A%09if%20%28aBOOL%3D%3D1%29%20then%20return%20true%20end%0D%0A%09if%20%28aBOOL%3D%3Dfalse%29%20then%20return%20false%20end%0D%0A%09if%20%28aBOOL+0%3D%3D1%29%20then%20return%20true%20end")]]]];}
,"inlines",unescape("nativeInlines%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5E%23%28%0D%0A%23%28%27NativeObject%20class%27%20%23%28%0D%0A%23%28undefined%20%27nil%27%29%0D%0A%23%28codeToNumber%20%27%22%20%22%27%29%0D%0A%29%29%0D%0A%23%28NativeObject%20%23%28%0D%0A%23%28jsObject%20%27self%3AinvalidMessage_%28%22jsObject%22%29%27%29%0D%0A%23%28ownPropertyNames%20%27%0D%0A%09local%20result%20%3D%20smalltalk.Array%3Anew%28%29%0D%0A%09for%20k%2C_%20in%20pairs%28self%5B%22@handle%22%5D%29%20do%0D%0A%09%09result%3Aadd_%28k%29%0D%0A%09end%0D%0A%09if%20true%20then%20return%20result%20end%27%29%0D%0A%29%29%0D%0A%23%28NativeHandle%20%23%28%0D%0A%23%28hasOwnProperty%3A%20%27rawget%28self%5B%22@handle%22%5D%2CaName%29%7E%3Dnil%27%29%0D%0A%23%28delete%3A%20%27self%5B%22@handle%22%5D%5BaName%5D%3Dnil%27%29%0D%0A%29%29%0D%0A%23%28%27NativeLibraryObject%20class%27%20%23%28%0D%0A%23%28basicLoad%3A%20%27require%28url%29%27%29%0D%0A%29%29%0D%0A%23%28NativeLibraryObject%20%20%23%28%0D%0A%23%28newHandleOf%3A%20%27result%20%3D%20aConstructor%28%29%27%29%0D%0A%23%28newHandleOf%3Awith%3A%20%27result%20%3D%20aConstructor%28arg1%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3A%20%27result%20%3D%20aConstructor%28arg1%2Carg2%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%2Carg7%29%27%29%0D%0A%23%28newHandleOf%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3Awith%3A%20%27result%20%3D%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%2Carg7%2Carg8%29%27%29%0D%0A%29%29%0D%0A%23%28Array%20%23%28%0D%0A%23%28addAllLua%3A%20%27for%20_%2Cvalue%20in%20pairs%28natives%29%20do%20self%3Aadd_%28value%29%20end%27%29%0D%0A%29%29%0D%0A%23%28%27Array%20class%27%20%23%28%0D%0A%23%28@@%20%27self%27%29%0D%0A%23%28@%20%27result%3AaddAllLua_%28natives%29%27%29%0D%0A%29%29%0D%0A%23%28%27Boolean%20class%27%20%23%28%0D%0A%23%28@%20%27%0D%0A%09if%20%28aBOOL%3D%3Dtrue%29%20then%20return%20true%20end%0D%0A%09if%20%28aBOOL%3D%3D1%29%20then%20return%20true%20end%0D%0A%09if%20%28aBOOL%3D%3Dfalse%29%20then%20return%20false%20end%0D%0A%09if%20%28aBOOL+0%3D%3D1%29%20then%20return%20true%20end%27%29%0D%0A%29%29%0D%0A%09%29%20"));
smalltalk.bind(smalltalk.LuaEncoder.$klass,"initializeToolsInlines:",0
,function(pool){var self=this;
return self.register_into_(self.toolsInlines(),pool);}
,"inlines",unescape("initializeToolsInlines%3A%20pool%0D%0A%09%22%20Private%20-%20Initialize%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5Eself%20register%3A%20self%20toolsInlines%20into%3A%20pool%20"));
smalltalk.bind(smalltalk.LuaEncoder.$klass,"toolsInlines",0
,function(){var self=this;
return [["NativeReader", [["initializeObject", unescape("%7B%7D")]]], ["String", [["asComment", [unescape("%22--%5B%5B%20%22"), unescape("%22%20--%5D%5D%22"), unescape("%22--%20%22")]]]], ["MethodReferences", [["dot", unescape("%22%3A%22")], ["localDef", unescape("%22local%20%22")], ["superSendPatternOf:", unescape("self%3AsuperLuaSendPatternOf_%28aSelector%29")], ["compiledSourceStart", unescape("%22%28%22")]]], ["Class", [["hackDNU", unescape("self%3AnotImplementedYet_%28%22hackDNU%22%29")]]]];}
,"inlines",unescape("toolsInlines%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5E%23%28%0D%0A%23%28NativeReader%20%23%28%0D%0A%23%28initializeObject%20%27%7B%7D%27%29%0D%0A%29%29%0D%0A%23%28String%20%20%23%28%0D%0A%23%28asComment%20%23%28%27%22--%5B%5B%20%22%27%20%27%22%20--%5D%5D%22%27%20%27%22--%20%22%27%29%29%0D%0A%29%29%0D%0A%23%28MethodReferences%20%23%28%0D%0A%23%28dot%09%09%27%22%3A%22%27%29%0D%0A%23%28localDef%09%27%22local%20%22%27%29%0D%0A%23%28superSendPatternOf%3A%09%27self%3AsuperLuaSendPatternOf_%28aSelector%29%27%29%0D%0A%23%28compiledSourceStart%09%27%22%28%22%27%29%0D%0A%29%29%0D%0A%23%28Class%20%23%28%0D%0A%23%28hackDNU%20%27self%3AnotImplementedYet_%28%22hackDNU%22%29%27%29%0D%0A%29%29%0D%0A%09%29%20"));
smalltalk.bind(smalltalk.LuaEncoder.$klass,"initializeSnapshotInlines:",0
,function(pool){var self=this;
return self.register_into_(self.snapshotInlines(),pool);}
,"inlines",unescape("initializeSnapshotInlines%3A%20pool%0D%0A%09%22%20Private%20-%20Initialize%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5Eself%20register%3A%20self%20snapshotInlines%20into%3A%20pool%20"));
smalltalk.bind(smalltalk.LuaEncoder.$klass,"snapshotInlines",0
,function(){var self=this;
return [["Smalltalk", [["exit", unescape("quit%28%29")]]], ["Snapshot", [["bootHeader", unescape("%5B%5Bsmalltalk%20%3D%20%28function%28%29%20%0D%0A%5D%5D")], ["bootFooter", unescape("%5B%5B%0D%0Aend%29%28%29%0D%0A%5D%5D")], ["launchExpression", unescape("%22%25%3AlaunchImage%25%28%29%22")]]]];}
,"inlines",unescape("snapshotInlines%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5E%23%28%0D%0A%23%28Smalltalk%20%23%28%0D%0A%23%28exit%09%27quit%28%29%27%29%0D%0A%29%29%0D%0A%23%28Snapshot%20%23%28%0D%0A%23%28bootHeader%20%27%5B%5Bsmalltalk%20%3D%20%28function%28%29%20%0D%0A%5D%5D%27%29%0D%0A%23%28bootFooter%20%27%5B%5B%0D%0Aend%29%28%29%0D%0A%5D%5D%27%29%0D%0A%23%28launchExpression%20%27%22%25%3AlaunchImage%25%28%29%22%27%29%0D%0A%29%29%0D%0A%09%29%20"));
smalltalk.bind(smalltalk.LuaEncoder.$klass,"initializeOtherInlines:",0
,function(pool){var self=this;
return self.register_into_(self.otherInlines(),pool);}
,"inlines",unescape("initializeOtherInlines%3A%20pool%0D%0A%09%22%20Private%20-%20Initialize%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5Eself%20register%3A%20self%20otherInlines%20into%3A%20pool%20"));
smalltalk.bind(smalltalk.LuaEncoder.$klass,"otherInlines",0
,function(){var self=this;
return [];}
,"inlines",unescape("otherInlines%0D%0A%09%22%20Private%20-%20Return%20the%20inlines%20for%20methods%20in%20the%20category.%20%22%0D%0A%0D%0A%09%5E%23%28%0D%0A%22%23%28ClassName%20%23%28%0D%0A%23%28selector%20%27...%27%29%0D%0A%29%29%22%0D%0A%09%29%20"));
smalltalk.bind(smalltalk.LuaEncoder,"nativeLanguage",0
,function(){var self=this;
return "lua";}
,"accessing",unescape("nativeLanguage%0D%0A%09%22%20Return%20the%20nativeLanguage%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23lua%20"));
smalltalk.bind(smalltalk.LuaEncoder,"nativeUndefined",0
,function(){var self=this;
return "nil";}
,"encoding",unescape("nativeUndefined%0D%0A%09%22%20Private%20-%20Return%20the%20code%20for%20undefined.%20%22%0D%0A%0D%0A%09%5E%23nil%20"));
smalltalk.bind(smalltalk.LuaEncoder,"nativeNameOfMetaClass:",0
,function(aClass){var self=this;
return aClass.instanceClass().name()._comma(unescape("%5B%22%24klass%22%5D"));}
,"encoding",unescape("nativeNameOfMetaClass%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20native%20name%20for%20metaclass%20aClass.%20%22%0D%0A%0D%0A%09%5EaClass%20instanceClass%20name%20%2C%27%5B%22%24klass%22%5D%27%20"));
smalltalk.bind(smalltalk.LuaEncoder,"callEscape",0
,function(){var self=this;
return unescape("%23_");}
,"accessing",unescape("callEscape%0D%0A%09%22%20Private%20-%20Return%20the%20selector%20sequence%20to%20escape%20OO%20message%20send%20selector%2C%20and%20produce%20code%20for%20function%20call.%0D%0A%09Note%20that%20an%20alien%20selector%20starting%20with%20two%20%24%24%20are%20understood%20as%20a%20function%20call%20to%20function%20starting%20with%20%24%0D%0A%09%22%0D%0A%0D%0A%09%5E%27%23_%27"));
smalltalk.bind(smalltalk.LuaEncoder,"missingNativeCodeFor:as:",0
,function(aNode,lang){var self=this;
return unescape("%28function%28%29%20self%3AinvalidMessage_%28%22")._comma(self["@compiler"].selector())._comma(unescape("%22%29%20end%29%28%29"));}
,"inlines",unescape("missingNativeCodeFor%3A%20aNode%20as%3A%20lang%0D%0A%09%22%20Private%20-%20Return%20the%20native%20language%20code%20for%20missing%20expression%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5E%27%28function%28%29%20self%3AinvalidMessage_%28%22%27%20%2Ccompiler%20selector%20%2C%27%22%29%20end%29%28%29%27"));
smalltalk.bind(smalltalk.LuaEncoder,"canExportMethod:",0
,function(aMethod){var self=this;
return true;}
,"private",unescape("canExportMethod%3A%20aMethod%0D%0A%09%22%20Private%20-%20Return%20true%20if%20the%20receiver%20can/want%20to%20export%20method%20aMethod.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.LuaEncoder,"asSelector:",0
,function(aSelector){var self=this;
return aSelector.asLuaSelector();}
,"converting",unescape("asSelector%3A%20aSelector%0D%0A%09%22%20Return%20the%20native%20representation%20of%20aSelector.%20%22%0D%0A%0D%0A%09%5EaSelector%20asLuaSelector"));
smalltalk.bind(smalltalk.LuaEncoder,"nativeSourceFor:",0
,function(aMethod){var self=this;
return ((self.canExportMethod_(aMethod)).mustBeBoolean()==true ? (function(){return self.privateSourceFor_(aMethod);})() : nil);}
,"private",unescape("nativeSourceFor%3A%20aMethod%0D%0A%09%22%20Private%20-%20Return%20the%20code%20of%20native%20function%20implementing%20aMethod.%20%22%0D%0A%0D%0A%09%5E%28self%20canExportMethod%3A%20aMethod%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20privateSourceFor%3A%20aMethod%0D%0A%09%5D"));
smalltalk.bind(smalltalk.LuaEncoder,"export:method:on:",0
,function(aClass,aMethod,aStream){var self=this;
var fName=nil;
var selectorName=nil;
var bar=nil;
var source=nil;
source=self.nativeSourceFor_(aMethod);
if ((nil.isNil_(source))){return false;};;
if ((nil.isNil_(aMethod))){self.error_("Method is missing");};;
bar=unescape("%22");
fName=aMethod.selector();
if (fName.first().isBackSlash()){selectorName=self.escaped_(aMethod.selector());
fName=self.escaped_(self.asSelector_(fName));} else{selectorName=bar._comma(aMethod.selector())._comma(bar);
((self.asSelector_(fName)._eq(aMethod.selector().replace_with_(":","_"))).mustBeBoolean()==true ? (function(){return fName=(0);})() : (function(){return fName=bar._comma(self.asSelector_(fName))._comma(bar);})());};;
aStream.nextPutAll_(unescape("smalltalk%3Abind%28")._comma(self.smalltalkNameFor_(aClass))._comma(unescape("%2C"))._comma(selectorName)._comma(unescape("%2C"))._comma(fName)._comma(unescape("%0D%0A%2C"))._comma(source));
if ((nil.isNil_(aMethod.source())===false).and_((function(){return aMethod.source().notEmpty();}))){(function($rec){$rec.cr();return $rec.nextPutAll_(unescape("%2C")._comma(self.escaped_(aMethod.category()))._comma(unescape("%2C"))._comma(self.escaped_(aMethod.source()))._comma(unescape("%2C"))._comma(self.escaped_(source)));})(aStream);};;
if (smalltalk.responds_to_(aMethod,("stamp")).and_((function(){return (nil.isNil_(aMethod.stamp())===false);}))){aStream.nextPutAll_(unescape("%2C")._comma(self.escaped_(aMethod.stamp())));};;
aStream.nextPutAll_(unescape("%29%0D%0A"));
return true;}
,"export",unescape("export%3A%20aClass%20method%3A%20aMethod%20on%3A%20aStream%0D%0A%09%22%20Export%20a%20method%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%7C%20fName%20selectorName%20bar%20source%20%7C%0D%0A%09source%20%3A%3D%20self%20nativeSourceFor%3A%20aMethod.%0D%0A%09source%20isNil%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09aMethod%20isNil%20ifTrue%3A%20%5B%20self%20error%3A%20%27Method%20is%20missing%27%20%5D.%0D%0A%09bar%20%3A%3D%20%24%22.%0D%0A%09fName%20%3A%3D%20aMethod%20selector.%0D%0A%09fName%20first%20isBackSlash%20ifTrue%3A%20%5B%0D%0A%09%09selectorName%20%3A%3D%20self%20escaped%3A%20aMethod%20selector.%0D%0A%09%09fName%20%3A%3D%20self%20escaped%3A%20%28self%20asSelector%3A%20fName%29.%0D%0A%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09selectorName%20%3A%3D%20bar%20%2CaMethod%20selector%2C%20bar.%0D%0A%09%09%28self%20asSelector%3A%20fName%29%20%3D%20%28aMethod%20selector%20replace%3A%20%24%3A%20with%3A%20%24_%29%20ifTrue%3A%20%5B%0D%0A%09%09%09fName%20%3A%3D%200%0D%0A%09%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09fName%20%3A%3D%20bar%20%2C%28self%20asSelector%3A%20fName%29%20%2Cbar%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09aStream%0D%0A%09%09nextPutAll%3A%20%27smalltalk%3Abind%28%27%20%2C%28self%20smalltalkNameFor%3A%20aClass%29%0D%0A%09%09%09%2C%27%2C%27%20%2CselectorName%20%2C%27%2C%27%20%2CfName%20%2C%27%0D%0A%2C%27%09%09%09%2Csource.%0D%0A%09%28aMethod%20source%20notNil%20and%3A%20%5B%20aMethod%20source%20notEmpty%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20aStream%20cr%3B%20%22warning%3A%20this%20cr%20is%20important%20for%20debugger%22%0D%0A%09%09nextPutAll%3A%20%27%2C%27%20%2C%28self%20escaped%3A%20aMethod%20category%29%0D%0A%09%09%09%2C%27%2C%27%20%2C%28self%20escaped%3A%20aMethod%20source%29%0D%0A%09%09%09%2C%27%2C%27%20%2C%28self%20escaped%3A%20source%29.%0D%0A%09%5D.%0D%0A%09%28%28aMethod%20respondsTo%3A%20%23stamp%29%20and%3A%20%5B%20aMethod%20stamp%20notNil%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09aStream%20nextPutAll%3A%20%27%2C%27%20%2C%28self%20escaped%3A%20aMethod%20stamp%29%0D%0A%09%5D.%0D%0A%09aStream%20nextPutAll%3A%20%27%29%0D%0A%27.%09%5Etrue"));
smalltalk.bind(smalltalk.LuaEncoder,"arrayLiteralStart",0
,function(){var self=this;
return "smalltalk:newArray"._comma(unescape("%28"))._comma(unescape("%7B"));}
,"arrays",unescape("arrayLiteralStart%0D%0A%09%22%20Private%20-%20Return%20the%20code%20string%20to%20start%20exporting%20an%20array%20literal.%20%22%0D%0A%0D%0A%09%5E%27smalltalk%3AnewArray%27%20%2C%24%28%20%2C%24%7B"));
smalltalk.bind(smalltalk.LuaEncoder,"arrayLiteralEnd",0
,function(){var self=this;
return unescape("%7D")._comma(unescape("%29"));}
,"arrays",unescape("arrayLiteralEnd%0D%0A%09%22%20Private%20-%20Return%20the%20code%20string%20to%20end%20exporting%20an%20array%20literal.%20%22%0D%0A%0D%0A%09%5E%24%7D%20%2C%24%29"));
smalltalk.bind(smalltalk.LuaEncoder,"escaped:",0
,function(string){var self=this;
return string.asLua();}
,"literals",unescape("escaped%3A%20string%0D%0A%09%22%20Private%20-%20Return%20the%20code%20string%20to%20export%20string%20literal.%20%22%0D%0A%0D%0A%09%5Estring%20asLua"));
smalltalk.bind(smalltalk.LuaEncoder,"exportCommentOf:on:",0
,function(aClass,aStream){var self=this;
if (aClass.comment().size()._gt((0))){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));$rec.nextPutAll_(unescape("%5B%22%24comment%22%5D%3D")._comma(self.escaped_(aClass.comment())));return $rec.cr();})(aStream);};;
return self;}
,"export",unescape("exportCommentOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20the%20comment%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09aClass%20comment%20size%20%3E%200%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20aStream%0D%0A%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27%5B%22%24comment%22%5D%3D%27%20%2C%28self%20escaped%3A%20aClass%20comment%29%3B%20cr%0D%0A%09%5D"));
smalltalk.bind(smalltalk.LuaEncoder,"exportDefinitionOf:on:",0
,function(aClass,aStream){var self=this;
self.loadRequirementsOf_(aClass).do_((function(each){return ((each.isString()).mustBeBoolean()==true ? (function(){return (function($rec){$rec.nextPutAll_(each);return $rec.cr();})(aStream);})() : (function(){return aStream.fileOutOn_(aStream);})());}));
(function($rec){$rec.nextPutAll_(unescape("smalltalk%3AaddClass%28%22")._comma(aClass.name())._comma(unescape("%22%2C")));$rec.nextPutAll_(self.smalltalkNameFor_(aClass.superclass()));return $rec.nextPutAll_(unescape("%2C"));})(aStream);
self.exportArrayLiteral_on_newLine_(aClass.instanceVariableNames(),aStream,false);
(function($rec){$rec.nextPutAll_(unescape("%2C%22")._comma(aClass.category())._comma(unescape("%22")));$rec.nextPutAll_(unescape("%29"));return $rec.cr();})(aStream);
if ((nil.isNil_(aClass.classVariables())===false)){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));$rec.nextPutAll_(unescape("%5B%22%24classVariableNames%22%5D%3D")._comma(self.escaped_(aClass.classVariablesString())));return $rec.cr();})(aStream);};;
if ((nil.isNil_(aClass.poolDictionaries())===false)){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));return $rec.nextPutAll_(unescape("%5B%22%24poolDictionaries%22%5D%3D"));})(aStream);
self.exportArrayLiteral_on_newLine_(aClass.poolDictionaries(),aStream,true);};;
if (aClass.propertyNames().notEmpty()){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));return $rec.nextPutAll_(unescape("%5B%22%24propertyNames%22%5D%3D"));})(aStream);
self.exportArrayLiteral_on_newLine_(aClass.propertyNames(),aStream,true);};;
self.exportCommentOf_on_(aClass,aStream);
return self;}
,"export",unescape("exportDefinitionOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%28self%20loadRequirementsOf%3A%20aClass%29%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20isString%0D%0A%09%09ifTrue%3A%20%5B%20aStream%20nextPutAll%3A%20each%3Bcr%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20aStream%20fileOutOn%3A%20aStream%20%5D.%0D%0A%09%5D.%0D%0A%09aStream%20%0D%0A%09%20%20%20%20nextPutAll%3A%20%27smalltalk%3AaddClass%28%22%27%2C%20aClass%20name%2C%20%27%22%2C%27%3B%0D%0A%09%20%20%20%20nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%20superclass%29%3B%0D%0A%09%20%20%20%20nextPutAll%3A%20%27%2C%27.%0D%0A%09self%20exportArrayLiteral%3A%20aClass%20instanceVariableNames%20on%3A%20aStream%20newLine%3A%20false.%0D%0A%09aStream%0D%0A%09%20%20%20%20nextPutAll%3A%20%27%2C%22%27%20%2CaClass%20category%2C%20%24%22%3B%0D%0A%09%20%20%20%20nextPutAll%3A%20%27%29%27%3B%20cr.%0D%0A%09aClass%20classVariables%20notNil%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20aStream%0D%0A%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27%5B%22%24classVariableNames%22%5D%3D%27%20%2C%28self%20escaped%3A%20aClass%20classVariablesString%29%3B%20cr%0D%0A%09%5D.%0D%0A%09aClass%20poolDictionaries%20notNil%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20aStream%0D%0A%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27%5B%22%24poolDictionaries%22%5D%3D%27.%0D%0A%09%20%20%20self%20exportArrayLiteral%3A%20aClass%20poolDictionaries%20on%3A%20aStream%20newLine%3A%20true.%0D%0A%09%5D.%0D%0A%09aClass%20propertyNames%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20aStream%0D%0A%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27%5B%22%24propertyNames%22%5D%3D%27.%0D%0A%09%20%20%20self%20exportArrayLiteral%3A%20aClass%20propertyNames%20on%3A%20aStream%20newLine%3A%20true.%0D%0A%09%5D.%0D%0A%09self%20exportCommentOf%3A%20aClass%20on%3A%20aStream."));
smalltalk.bind(smalltalk.LuaEncoder,"exportMetaDefinitionOf:on:",0
,function(aClass,aStream){var self=this;
if (aClass.$klass.instanceVariableNames().notEmpty()){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass.$klass));return $rec.nextPutAll_(unescape("%5B%22%24iVarNames%22%5D%3D"));})(aStream);
self.exportArrayLiteral_on_newLine_(aClass.$klass.instanceVariableNames(),aStream,true);};;
if (aClass.$klass.propertyNames().notEmpty()){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass.$klass));return $rec.nextPutAll_(unescape("%5B%22%24propertyNames%22%5D%3D"));})(aStream);
self.exportArrayLiteral_on_newLine_(aClass.$klass.propertyNames(),aStream,true);};;
return self;}
,"export",unescape("exportMetaDefinitionOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20class%20side%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09aClass%20class%20instanceVariableNames%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%09aStream%20%0D%0A%09%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%20class%29%3B%0D%0A%09%09%09nextPutAll%3A%20%27%5B%22%24iVarNames%22%5D%3D%27.%0D%0A%09%09self%20exportArrayLiteral%3A%20aClass%20class%20instanceVariableNames%20on%3A%20aStream%20newLine%3A%20true.%0D%0A%09%5D.%0D%0A%0D%0A%09aClass%20class%20propertyNames%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%09aStream%20%0D%0A%09%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%20class%29%3B%0D%0A%09%09%09nextPutAll%3A%20%27%5B%22%24propertyNames%22%5D%3D%27.%0D%0A%09%09self%20exportArrayLiteral%3A%20aClass%20class%20propertyNames%20on%3A%20aStream%20newLine%3A%20true.%0D%0A%09%5D."));
smalltalk.bind(smalltalk.LuaEncoder,"privateSourceFor:",0
,function(aMethod){var $s8Ret$={name:"stReturn"};try{var self=this;
var code=nil;
if (smalltalk.Smalltalk.isRunningOn_(self.nativeLanguage())){code=((function(){var $1$=(aMethod["$nativeSource"]);if(nil.isNil_($1$))return ((function(){return nil;}).value());return $1$;})());
(((nil.isNil_(code)===false)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return code})();throw($s8Ret$)})();})() : nil);};;
code=(function(){return self.privateCodeFor_(aMethod);}).on_do_(smalltalk.Exception,(function(ex){self.note_(aMethod);return self.privateNIYSourceFor_with_(aMethod,ex.description());}));
(function(){$s8Ret$.result=(function(){return unescape("%28%0D%0A")._comma(code)._comma(unescape("%0D%0A%29"))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"lua",unescape("privateSourceFor%3A%20aMethod%0D%0A%09%22%20Private%20-%20Return%20the%20native%20code%20implementing%20aMethod.%20%22%0D%0A%0D%0A%09%7C%20code%20%7C%0D%0A%09%28Smalltalk%20isRunningOn%3A%20self%20nativeLanguage%29%20ifTrue%3A%20%5B%0D%0A%09%09code%20%3A%3D%20aMethod%20basicAt%3A%20%27%24nativeSource%27%20ifAbsent%3A%20%5B%5D.%0D%0A%09%09code%20notNil%20ifTrue%3A%20%5B%20%5Ecode%20%5D.%0D%0A%09%5D.%0D%0A%0D%0A%09code%20%3A%3D%20%5B%20self%20privateCodeFor%3A%20aMethod%20%5D%20on%3A%20Exception%20do%3A%20%5B%3Aex%7C%0D%0A%09%09self%20note%3A%20aMethod.%0D%0A%09%09self%20privateNIYSourceFor%3A%20aMethod%20with%3A%20ex%20description%0D%0A%09%5D.%0D%0A%09%5E%27%28%0D%0A%27%09%2Ccode%20%2C%27%0D%0A%29%27"));
smalltalk.bind(smalltalk.LuaEncoder,"privateNIYSourceFor:with:",0
,function(aMethod,description){var self=this;
return self.privateMethodDefinition_(aMethod)._comma(self.privateMethodBody_with_(aMethod,description));}
,"niy",unescape("privateNIYSourceFor%3A%20aMethod%20with%3A%20description%0D%0A%09%22%20Private%20-%20Return%20the%20native%20code%20implementing%20aMethod.%20%22%0D%0A%0D%0A%09%5E%28self%20privateMethodDefinition%3A%20aMethod%29%0D%0A%09%2C%28self%20privateMethodBody%3A%20aMethod%20with%3A%20description%29"));
smalltalk.bind(smalltalk.LuaEncoder,"privateMethodDefinition:",0
,function(aMethod){var self=this;
var args=nil;
args=(function($rec){$rec.upTo_(unescape("%28"));return $rec.upTo_(unescape("%29"));})(aMethod.compiledSource().stream());
if (args.notEmpty()){args=args.asArrayOfSubstringsSeparatedBy_(unescape("%2C")).inject_into_("",(function(total,each){return total._comma(unescape("%2C"))._comma(each.asLuaName());}));};;
return unescape("function%28self")._comma(args)._comma(unescape("%29"));}
,"niy",unescape("privateMethodDefinition%3A%20aMethod%0D%0A%09%22%20Private%20-%20Return%20the%20%28Lua%29%20method%20definition%20of%20aMethod.%20%22%0D%0A%0D%0A%09%7C%20args%20%7C%0D%0A%09args%20%3A%3D%20aMethod%20compiledSource%20stream%20upTo%3A%20%24%28%3B%20upTo%3A%20%24%29.%0D%0A%09args%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%09args%20%3A%3D%20%28args%20asArrayOfSubstringsSeparatedBy%3A%20%24%2C%29%0D%0A%09%09%09inject%3A%20%27%27%20into%3A%20%5B%3Atotal%20%3Aeach%7C%20total%20%2C%27%2C%27%20%2Ceach%20asLuaName%20%5D.%0D%0A%09%5D.%0D%0A%09%5E%27function%28self%27%20%2Cargs%20%2C%27%29%27"));
smalltalk.bind(smalltalk.LuaEncoder,"privateMethodBody:with:",0
,function(aMethod,description){var self=this;
return unescape("%0D%0A%09print%28%22NIY%3A%23privateMethodBody%3A%20")._comma(aMethod)._comma(unescape("%20-%20"))._comma(description)._comma(unescape("%22%29%0D%0A%09return%20self%0D%0Aend"));}
,"niy",unescape("privateMethodBody%3A%20aMethod%20with%3A%20description%0D%0A%09%22%20Private%20-%20Return%20the%20%28Lua%29%20body%20of%20aMethod.%20%22%0D%0A%0D%0A%09%5E%27%0D%0A%09print%28%22NIY%3A%23privateMethodBody%3A%20%27%2CaMethod%2C%27%20-%20%27%2Cdescription%2C%27%22%29%0D%0A%09return%20self%0D%0Aend%27"));
smalltalk.bind(smalltalk.LuaEncoder,"privateCodeFor:",0
,function(aMethod){var self=this;
self.compiler_(aMethod.classField().compiler());
return (function($rec){$rec.encoder_(self);return $rec.compile_(aMethod.source());})(self.compiler());}
,"compiling",unescape("privateCodeFor%3A%20aMethod%0D%0A%09%22%20Private%20-%20Return%20the%20native%20code%20translated%20from%20sources%20of%20aMethod.%20%22%0D%0A%0D%0A%09self%20compiler%3A%20aMethod%20classField%20compiler.%0D%0A%09%5Eself%20compiler%0D%0A%09%09encoder%3A%20self%3B%0D%0A%09%09compile%3A%20aMethod%20source"));
smalltalk.bind(smalltalk.LuaEncoder,"emmitEnterMethod:",0
,function(aMethodNode){var self=this;
return self;}
,"generation",unescape("emmitEnterMethod%3A%20aMethodNode%0D%0A%09%22%20Private%20-%20Generate%20home%20context%20definitions%2C%20entering%20aMethodNode.%20%22"));
smalltalk.bind(smalltalk.LuaEncoder,"emmitFunctionBody:",0
,function(aBlock){var self=this;
self.cr();
aBlock.evaluate();
(function($rec){$rec.cr();return $rec.nextPutAll_("end");})(self);
return self;}
,"generation",unescape("emmitFunctionBody%3A%20aBlock%0D%0A%09%22%20Private%20-%20Emmit%20the%20code%20of%20function%20body%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09self%20cr.%0D%0A%09aBlock%20evaluate.%0D%0A%09self%20cr%3B%20nextPutAll%3A%20%23end%20"));
smalltalk.bind(smalltalk.LuaEncoder,"newMethodFunctionName",0
,function(){var self=this;
return "smalltalk:newMethod";}
,"generation",unescape("newMethodFunctionName%0D%0A%09%22%20Private%20-%20Return%20the%20function%20to%20call%20when%20installing%20a%20new%20method.%20%22%0D%0A%0D%0A%09%5E%27smalltalk%3AnewMethod%27"));
smalltalk.bind(smalltalk.LuaEncoder,"emmitFunction:node:withSelf:arguments:doing:",0
,function(fName,aNode,aBoolean,argumentNames,aBlock){var self=this;
var aStream=nil;
aStream="".writeStream();
if (aBoolean){aStream.nextPutAll_("self");
((argumentNames.notEmpty()).mustBeBoolean()==true ? (function(){return aStream.nextPutAll_(self.argumentsSeparatorOp());})() : nil);};;
argumentNames.do_separatedBy_((function(each){self.tempVariables().add_(each);return aStream.nextPutAll_(self.localName_(each));}),(function(){return aStream.nextPutAll_(self.argumentsSeparatorOp());}));
self.emmitFunction_arguments_doing_(fName,aStream.contents(),aBlock);
return self;}
,"generation",unescape("emmitFunction%3A%20fName%20node%3A%20aNode%20withSelf%3A%20aBoolean%20arguments%3A%20argumentNames%20doing%3A%20aBlock%0D%0A%09%22%20Private%20-%20Generate%20function%20for%20aNode.%20%22%0D%0A%0D%0A%09%7C%20aStream%20%7C%0D%0A%09aStream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09aBoolean%20ifTrue%3A%20%5B%0D%0A%09%09aStream%20nextPutAll%3A%20%23self.%0D%0A%09%09argumentNames%20notEmpty%20ifTrue%3A%20%5B%20aStream%20nextPutAll%3A%20self%20argumentsSeparatorOp%20%5D.%0D%0A%09%5D.%0D%0A%09argumentNames%20do%3A%20%5B%3Aeach%20%7C%20%0D%0A%09%09self%20tempVariables%20add%3A%20each.%0D%0A%09%09aStream%20nextPutAll%3A%20%28self%20localName%3A%20each%29%20%5D%0D%0A%09%20%20%20%20separatedBy%3A%20%5B%20aStream%20nextPutAll%3A%20self%20argumentsSeparatorOp%20%5D.%0D%0A%09self%09emmitFunction%3A%20fName%0D%0A%09%09arguments%3A%20aStream%20contents%0D%0A%09%09doing%3A%20aBlock"));
smalltalk.bind(smalltalk.LuaEncoder,"emmitFunction:node:arguments:doing:",0
,function(fName,aNode,argumentNames,aBlock){var self=this;
return self.emmitFunction_node_withSelf_arguments_doing_(fName,aNode,false,argumentNames,aBlock);}
,"generation",unescape("emmitFunction%3A%20fName%20node%3A%20aNode%20arguments%3A%20argumentNames%20doing%3A%20aBlock%0D%0A%09%22%20Private%20-%20Generate%20function%20for%20aNode.%20%22%0D%0A%0D%0A%09%5Eself%09emmitFunction%3A%20fName%20node%3A%20aNode%0D%0A%09%09withSelf%3A%20false%20arguments%3A%20argumentNames%0D%0A%09%09doing%3A%20aBlock"));
smalltalk.bind(smalltalk.LuaEncoder,"emmitFunction:node:doing:",0
,function(fName,aNode,aBlock){var self=this;
return self.emmitFunction_node_withSelf_arguments_doing_(fName,aNode,true,aNode.argumentNames(),aBlock);}
,"generation",unescape("emmitFunction%3A%20fName%20node%3A%20aNode%20doing%3A%20aBlock%0D%0A%09%22%20Private%20-%20Generate%20function%20for%20aNode.%20%22%0D%0A%0D%0A%09%5Eself%09emmitFunction%3A%20fName%20node%3A%20aNode%0D%0A%09%09withSelf%3A%20true%20arguments%3A%20aNode%20argumentNames%0D%0A%09%09doing%3A%20aBlock"));
smalltalk.bind(smalltalk.LuaEncoder,"visitMethodNode:",0
,function(aNode){var self=this;
var fnStart=nil;
var fnEnd=nil;
var nativeSource=nil;
if (!(smalltalk.Smalltalk.isRunningOn_(self.nativeLanguage()))){return self.emmitMethod_(aNode);};;
self.nextPutAll_(self.newMethodFunctionName());
self.nextArgumentsPut_((function(){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(self.currentClass()));$rec.nextPutAll_(unescape("%2C%22")._comma(aNode.selector())._comma(unescape("%22%2C")));return $rec.cr();})(self);fnStart=self["@stream"].position();self.emmitMethod_(aNode);fnEnd=self["@stream"].position();return ((self.includeSourceCode()).mustBeBoolean()==true ? (function(){nativeSource=self["@stream"].contents().copyFrom_to_(fnStart._plus((1)),fnEnd._plus((1)));return (function($rec){$rec.cr();$rec.nextPutAll_(unescape("%2C")._comma(self.escaped_(aNode.source())));return $rec.nextPutAll_(unescape("%2C")._comma(self.escaped_(nativeSource)));})(self);})() : nil);}));
return self;}
,"visiting",unescape("visitMethodNode%3A%20aNode%0D%0A%09%22%20Visit%20a%20method%20node.%20%22%0D%0A%0D%0A%09%7C%20fnStart%20fnEnd%20nativeSource%20%7C%0D%0A%09%28Smalltalk%20isRunningOn%3A%20self%20nativeLanguage%29%20ifFalse%3A%20%5B%20%5Eself%20emmitMethod%3A%20aNode%20%5D.%0D%0A%0D%0A%09%22return%20the%20code%20for%20method%20ready%20to%20be%20injected%20in%20the%20system%22%0D%0A%09self%20nextPutAll%3A%20self%20newMethodFunctionName.%0D%0A%09self%20nextArgumentsPut%3A%20%5B%0D%0A%09%09self%0D%0A%09%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20self%20currentClass%29%3B%0D%0A%09%09%09nextPutAll%3A%20%27%2C%22%27%2C%20aNode%20selector%2C%20%27%22%2C%27%3B%20cr.%0D%0A%09%09fnStart%20%3A%3D%20stream%20position.%0D%0A%09%09self%20emmitMethod%3A%20aNode.%0D%0A%09%09fnEnd%20%3A%3D%20stream%20position.%0D%0A%09%09self%20includeSourceCode%20ifTrue%3A%20%5B%0D%0A%09%09%09nativeSource%20%3A%3D%20stream%20contents%20copyFrom%3A%20fnStart+1%20to%3A%20fnEnd+1.%0D%0A%09%09%09self%09cr%3B%20%22warning%3A%20this%20cr%20is%20important%20for%20debugger%22%0D%0A%09%09%09%09nextPutAll%3A%20%27%2C%27%20%2C%28self%20escaped%3A%20aNode%20source%29%0D%0A%09%09%09%09%3BnextPutAll%3A%20%27%2C%27%20%2C%28self%20escaped%3A%20nativeSource%29%0D%0A%09%09%5D%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.LuaEncoder,"visitDirectiveNode:",0
,function(aNode){var self=this;
if (smalltalk.responds_to_(aNode,("asNative:for:on:"))){(function($rec){$rec.nextPutAll_(aNode.asNative_for_on_(self["@functionName"],self,self["@stream"]));return $rec.cr();})(self);
return self;};;
self.visit_(aNode);
(function($rec){$rec.nextPutAll_(self.separatorOp());return $rec.cr();})(self);
return self;}
,"visiting",unescape("visitDirectiveNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20a%20directive%20node.%20%22%0D%0A%0D%0A%09%28aNode%20respondsTo%3A%20%23asNative%3Afor%3Aon%3A%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20nextPutAll%3A%20%28aNode%0D%0A%09%09%09asNative%3A%20functionName%0D%0A%09%09%09for%3A%20self%20on%3A%20stream%29%3B%20cr.%0D%0A%09%09%5Eself%0D%0A%09%5D.%0D%0A%0D%0A%09self%20visit%3A%20aNode.%0D%0A%09self%20nextPutAll%3A%20self%20separatorOp%3B%20cr"));
smalltalk.bind(smalltalk.LuaEncoder,"emmitedCode",0
,function(){var self=this;
if ((nil.isNil_(self["@stream"]))){return nil;};;
if (smalltalk.Smalltalk.isRunningOn_(self.nativeLanguage())){return "return "._comma(self["@stream"].contents());};;
return self["@stream"].contents();}
,"results",unescape("emmitedCode%0D%0A%09%22%20Return%20the%20emmited%20code%20during%20compilation%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09stream%20isNil%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%28Smalltalk%20isRunningOn%3A%20self%20nativeLanguage%29%20ifTrue%3A%20%5B%0D%0A%09%09%22the%20code%20must%20be%20valid%20native%20code%20including%20return%20expression%22%0D%0A%09%09%5E%27return%20%27%20%2Cstream%20contents%0D%0A%09%5D.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.LuaEncoder,"returnOpStart",0
,function(){var self=this;
return unescape("return%20%28");}
,"constants",unescape("returnOpStart%0D%0A%09%22%20Private%20-%20Return%20the%20OP%20for%20starting%20return%20expressions.%20%22%0D%0A%0D%0A%09%5E%27return%20%28%27%20"));
smalltalk.bind(smalltalk.LuaEncoder,"returnOpEnd",0
,function(){var self=this;
return unescape("%29");}
,"constants",unescape("returnOpEnd%0D%0A%09%22%20Private%20-%20Return%20the%20OP%20for%20ending%20return%20expressions.%20%22%0D%0A%0D%0A%09%5E%27%29%27%20"));
smalltalk.bind(smalltalk.LuaEncoder,"visitValueNode:",0
,function(aNode){var self=this;
self.nextPutAll_(aNode.value().asLua());
return self;}
,"visiting",unescape("visitValueNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20the%20node%20emmitting%20output.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20aNode%20value%20asLua"));
smalltalk.bind(smalltalk.LuaEncoder,"visitNode:index:",0
,function(aNode,index){var self=this;
self.nextPutAll_(unescape("%5B"));
self.visit_(index);
self.nextPutAll_(unescape("%5D"));
return self;}
,"visiting",unescape("visitNode%3A%20aNode%20index%3A%20index%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20indexed%20access.%20%22%0D%0A%0D%0A%22ToDo%3A%09%28%28index%20isMemberOf%3A%20ValueNode%29%20and%3A%20%5B%20index%20hasString%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%7C%20selector%20%7C%0D%0A%09%09selector%20%3A%3D%20self%20asSelector%3A%20index%20value.%0D%0A%09%09%28%28selector%20includes%3A%20%24%20%29%20or%3A%20%5B%20selector%20includes%3A%20%24_%20%5D%29%20ifFalse%3A%20%5B%0D%0A%09%09%09self%20nextPutAll%3A%20%27.%27%3B%20nextPutAll%3A%20selector.%0D%0A%09%09%09%5Eself%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%22%09self%20nextPutAll%3A%20%27%5B%27.%0D%0A%09self%20visit%3A%20index.%0D%0A%09self%20nextPutAll%3A%20%27%5D%27."));
smalltalk.bind(smalltalk.LuaEncoder,"propertyAccessConnector",0
,function(){var self=this;
return ".";}
,"encoding",unescape("propertyAccessConnector%0D%0A%09%22%20Private%20-%20Return%20the%20connector%20to%20use%20for%20accessing%20a%20native%20property.%20%22%0D%0A%0D%0A%09%5E%24.%20"));
smalltalk.bind(smalltalk.LuaEncoder,"messageSendConnector",0
,function(){var self=this;
return ":";}
,"encoding",unescape("messageSendConnector%0D%0A%09%22%20Private%20-%20Return%20the%20connector%20to%20use%20for%20sending%20a%20message.%20%22%0D%0A%0D%0A%09%5E%24%3A%20"));
smalltalk.bind(smalltalk.LuaEncoder,"functionCallConnector",0
,function(){var self=this;
return ".";}
,"encoding",unescape("functionCallConnector%0D%0A%09%22%20Private%20-%20Return%20the%20connector%20to%20use%20for%20calling%20a%20function.%20%22%0D%0A%0D%0A%09%5E%24.%20"));
smalltalk.bind(smalltalk.LuaEncoder,"messageSendConnector:",0
,function(aNode){var self=this;
if (aNode.receiver().messageSendConnector().size()._eq((0))){return "";};;
if (self.isNativeCall_(aNode)){if (aNode.selector().startsWith_(self.callEscape())){return self.functionCallConnector();};;
return self.messageSendConnector();};;
return self.messageSendConnector();}
,"encoding",unescape("messageSendConnector%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20the%20connector%20to%20use%20for%20sending%20message.%20%22%0D%0A%0D%0A%09aNode%20receiver%20messageSendConnector%20size%20%3D%200%20ifTrue%3A%20%5B%20%5E%27%27%20%5D.%0D%0A%09%28self%20isNativeCall%3A%20aNode%29%20ifTrue%3A%20%5B%20%22calling%20an%20alien%22%0D%0A%09%09%28aNode%20selector%20startsWith%3A%20self%20callEscape%29%20ifTrue%3A%20%5B%0D%0A%09%09%09%5Eself%20functionCallConnector%0D%0A%09%09%5D.%0D%0A%09%09%5Eself%20messageSendConnector%0D%0A%09%5D.%0D%0A%09%5Eself%20messageSendConnector%20"));
smalltalk.bind(smalltalk.LuaEncoder,"asPrivateSelector:",0
,function(aSelector){var self=this;
var result=nil;
result=self.asSelector_(aSelector);
if (aSelector.startsWith_(self.callEscape())){return result.copyFrom_to_((2),result.size());};;
return result;}
,"encoding",unescape("asPrivateSelector%3A%20aSelector%0D%0A%09%22%20Private%20-%20Validate%20and%20returns%20a%20valid%20native%20selector%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20asSelector%3A%20aSelector.%0D%0A%09%28aSelector%20startsWith%3A%20self%20callEscape%29%20ifTrue%3A%20%5B%20%22call%20to%20an%20alien%20starting%20with%20_%22%0D%0A%09%09%5Eresult%20copyFrom%3A%202%20to%3A%20result%20size%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.LuaEncoder,"visitPrivateMessage:",0
,function(aNode){var self=this;
(function($rec){$rec.nextPutAll_(self.messageSendConnector_(aNode));return $rec.nextPutAll_(self.asPrivateSelector_(aNode.selector()));})(self);
return self;}
,"visiting",unescape("visitPrivateMessage%3A%20aNode%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20native%20message%20send.%22%0D%0A%0D%0A%09self%0D%0A%09%09nextPutAll%3A%20%28self%20messageSendConnector%3A%20aNode%29%3B%0D%0A%09%09nextPutAll%3A%20%28self%20asPrivateSelector%3A%20aNode%20selector%29"));
smalltalk.bind(smalltalk.LuaEncoder,"encodeNormalSend:",0
,function(aNode){var self=this;
self.visitReceiver_(aNode.receiver());
self.visitPrivateMessage_(aNode);
self.nextArgumentsPut_((function(){return self.visitSendArguments_(aNode);}));
return self;}
,"encoding",unescape("encodeNormalSend%3A%20aNode%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20direct/inlined%20message%20send.%22%0D%0A%0D%0A%09self%20visitReceiver%3A%20aNode%20receiver.%0D%0A%09self%20visitPrivateMessage%3A%20aNode.%0D%0A%09self%20nextArgumentsPut%3A%20%5B%20self%20visitSendArguments%3A%20aNode%20%5D%20"));
smalltalk.bind(smalltalk.LuaEncoder,"visitReceiver:",0
,function(aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
self.nextArgumentsPut_((function(){var last=nil;
last=self["@stream"].position();if (aNode.isAssignment()){self.visitLocalAssignment_(aNode);} else{self.visit_(aNode);};;return ((self["@stream"].position()._eq(last)).mustBeBoolean()==true ? (function(){self["@stream"].removeLastInput();return (function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"visiting",unescape("visitReceiver%3A%20aNode%0D%0A%09%22%20Private%20-%20Visit%20the%20node%20%28as%20receiver%29%20emmitting%20output.%20%22%0D%0A%0D%0A%09self%20nextArgumentsPut%3A%20%5B%20%7C%20last%20%7C%0D%0A%09%09last%20%3A%3D%20stream%20position.%0D%0A%09%09aNode%20isAssignment%0D%0A%09%09%09ifTrue%3A%20%5B%20self%20visitLocalAssignment%3A%20aNode%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20self%20visit%3A%20aNode%20%5D.%0D%0A%09%09stream%20position%20%3D%20last%20ifTrue%3A%20%5B%0D%0A%09%09%09stream%20removeLastInput.%0D%0A%09%09%09%5Eself%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.LuaEncoder,"localName:",0
,function(aName){var self=this;
if (self.pseudoVariables().includes_(aName)){return aName;};;
return self.asSelector_(aName);}
,"private",unescape("localName%3A%20aName%0D%0A%09%22%20Private%20-%20Return%20a%20valid%20%28native%2C%20local%29%20name%20for%20aName.%20%22%0D%0A%0D%0A%09%28self%20pseudoVariables%20includes%3A%20aName%29%20ifTrue%3A%20%5B%20%5EaName%20%5D.%0D%0A%09%5Eself%20asSelector%3A%20aName"));
smalltalk.bind(smalltalk.LuaEncoder,"visitGlobalReferenceNode:",0
,function(aNode){var $s8Ret$={name:"stReturn"};try{var self=this;
var aName=nil;
var aClass=nil;
var lookupPools=nil;
aName=aNode.value();
aClass=self.currentClass().instanceClass();
aClass.classVariableNamesDo_((function(each){return ((each._eq(aName)).mustBeBoolean()==true ? (function(){(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));$rec.nextPutAll_(unescape("%3AclassVariableAt_%28%22"));$rec.nextPutAll_(aName);return $rec.nextPutAll_(unescape("%22%29"));})(self);return (function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();})() : nil);}));
aClass.sharedPoolsAndNamesDo_((function(poolName,pool){return ((pool.includesKey_(aName)).mustBeBoolean()==true ? (function(){(function($rec){$rec.nextPutAll_(unescape("%28smalltalk%3ApoolDictionary_%28%22"));$rec.nextPutAll_(poolName);$rec.nextPutAll_(unescape("%22%29%29%3Aat_%28%22"));$rec.nextPutAll_(aName);return $rec.nextPutAll_(unescape("%22%29"));})(self);return (function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();})() : nil);}));
(function($rec){$rec.nextPutAll_("smalltalk.");return $rec.nextPutAll_(aNode.value());})(self);
(function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"visiting",unescape("visitGlobalReferenceNode%3A%20aNode%0D%0A%0D%0A%09%7C%20aName%20aClass%20lookupPools%20%7C%0D%0A%09aName%20%3A%3D%20aNode%20value.%0D%0A%09aClass%20%3A%3D%20self%20currentClass%20instanceClass.%0D%0A%09aClass%20classVariableNamesDo%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20%3D%20aName%20ifTrue%3A%20%5B%0D%0A%09%09%09self%0D%0A%09%09%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09%09%09nextPutAll%3A%20%27%3AclassVariableAt_%28%22%27%3B%0D%0A%09%09%09%09nextPutAll%3A%20aName%3B%0D%0A%09%09%09%09nextPutAll%3A%20%27%22%29%27.%0D%0A%09%09%09%5Eself%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09aClass%20sharedPoolsAndNamesDo%3A%20%5B%3ApoolName%20%3Apool%7C%0D%0A%09%09%28pool%20includesKey%3A%20aName%29%20ifTrue%3A%20%5B%0D%0A%09%09%09self%0D%0A%09%09%09%09nextPutAll%3A%20%27%28smalltalk%3ApoolDictionary_%28%22%27%3B%0D%0A%09%09%09%09nextPutAll%3A%20poolName%3B%0D%0A%09%09%09%09nextPutAll%3A%20%27%22%29%29%3Aat_%28%22%27%3B%0D%0A%09%09%09%09nextPutAll%3A%20aName%3B%0D%0A%09%09%09%09nextPutAll%3A%20%27%22%29%27.%0D%0A%09%09%09%5Eself%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09self%0D%0A%09%09nextPutAll%3A%20%27smalltalk.%27%3B%0D%0A%09%09nextPutAll%3A%20aNode%20value."));
smalltalk.bind(smalltalk.LuaEncoder,"visitBlockNode:",0
,function(aNode){var self=this;
self.nextArgumentsPut_((function(){return self.emmitFunction_node_arguments_doing_("",aNode,aNode.parameterNames(),(function(){return aNode.nodes().do_((function(each){return self.visit_(each);}));}));}));
return self;}
,"visiting",unescape("visitBlockNode%3A%20aNode%0D%0A%0D%0A%09self%20nextArgumentsPut%3A%20%5B%0D%0A%09%09self%09emmitFunction%3A%20%27%27%20node%3A%20aNode%0D%0A%09%09%09arguments%3A%20aNode%20parameterNames%0D%0A%09%09%09doing%3A%20%5B%20aNode%20nodes%20do%3A%20%5B%3Aeach%20%7C%20self%20visit%3A%20each%20%5D%20%5D%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.LuaEncoder,"defineTemporary:",0
,function(aName){var self=this;
(function($rec){$rec.nextPutAll_("local "._comma(self.localName_(aName))._comma(unescape("%3Dnil")));return $rec.cr();})(self);
return self;}
,"encoding",unescape("defineTemporary%3A%20aName%0D%0A%09%22%20Private%20-%20Return%20the%20code%20for%20defining%20temporary%20variable%20with%20aName.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20%27local%20%27%2C%20%28self%20localName%3A%20aName%29%2C%20%27%3Dnil%27%20%3Bcr%20"));
smalltalk.bind(smalltalk.LuaEncoder,"visitCascadeNode:",0
,function(aNode){var self=this;
var target=nil;
target="_s8_rec";
if (!(self.tempVariables().includes_(target))){self.tempVariables().add_(target);};;
self.compilerEvent_with_("aboutCascade:",aNode);
(function($rec){$rec.nextArgumentsPut_((function(){self.cr();self.emmitFunction_arguments_doing_("",target,(function(){var index=nil;
var rec=nil;
index=(0);self.tab();return aNode.nodes().do_((function(each){index=index._plus((1));if (index._eq(aNode.nodes().size())){self.nextPutAll_(self.returnOpStart());};;rec=smalltalk.VariableNode.$$new();(function($rec){$rec.value_(target);return $rec.start_end_(each.start(),each.start());})(rec);each.receiver_(rec);self.visit_(each);return ((index._eq(aNode.nodes().size())).mustBeBoolean()==true ? (function(){self.compilerEvent_withArguments_("returned:early:",smalltalk.Array.with_with_(nil,false));return self.nextPutAll_(self.returnOpEnd());})() : (function(){return (function($rec){$rec.nextPutAll_(self.separatorOp());$rec.cr();return $rec.tab();})(self);})());}));}));return self.cr();}));return $rec.nextArgumentsPut_((function(){return self.visit_(aNode.receiver());}));})(self);
self.compilerEvent_with_("cascade:",aNode);
return self;}
,"visiting",unescape("visitCascadeNode%3A%20aNode%0D%0A%0D%0A%09%7C%20target%20%7C%0D%0A%09target%20%3A%3D%20%23_s8_rec.%0D%0A%09%28self%20tempVariables%20includes%3A%20target%29%20ifFalse%3A%20%5B%20self%20tempVariables%20add%3A%20target%20%5D.%0D%0A%09self%20compilerEvent%3A%20%23aboutCascade%3A%20with%3A%20aNode.%0D%0A%09self%20nextArgumentsPut%3A%20%5B%0D%0A%09%09self%20cr.%0D%0A%09%09self%20emmitFunction%3A%20%27%27%20arguments%3A%20target%20doing%3A%20%5B%20%7C%20index%20rec%20%7C%0D%0A%09%09%09index%20%3A%3D%200.%20self%20tab.%0D%0A%09%09%09aNode%20nodes%20do%3A%20%5B%3Aeach%20%7C%0D%0A%09%09%09%20%20%20%20index%20%3A%3D%20index%20+%201.%0D%0A%09%09%09%20%20%20%20index%20%3D%20aNode%20nodes%20size%20ifTrue%3A%20%5B%20self%20nextPutAll%3A%20self%20returnOpStart%20%5D.%0D%0A%09%09%09%20%20%20%20rec%20%3A%3D%20VariableNode%20new.%0D%0A%09%09%09%20%20%20%20rec%20value%3A%20target%3B%20start%3A%20each%20start%20end%3A%20each%20start.%0D%0A%09%09%09%20%20%20%20each%20receiver%3A%20rec.%0D%0A%09%09%09%20%20%20%20self%20visit%3A%20each.%0D%0A%09%09%09%20%20%20%20index%20%3D%20aNode%20nodes%20size%20ifTrue%3A%20%5B%0D%0A%09%09%09%09%09self%09compilerEvent%3A%20%23returned%3Aearly%3A%0D%0A%09%09%09%09%09%09withArguments%3A%20%28Array%20with%3A%20nil%20with%3A%20false%29.%0D%0A%09%09%09%09%09self%20nextPutAll%3A%20self%20returnOpEnd%0D%0A%09%09%09%20%20%20%20%09%5D%20ifFalse%3A%20%5B%20self%20nextPutAll%3A%20self%20separatorOp%3B%20cr%3Btab%20%5D%0D%0A%09%09%09%5D%0D%0A%09%09%5D.%0D%0A%09%09self%20cr%0D%0A%09%5D%3B%20nextArgumentsPut%3A%20%5B%20self%20visit%3A%20aNode%20receiver%20%5D.%0D%0A%09self%20compilerEvent%3A%20%23cascade%3A%20with%3A%20aNode."));
smalltalk.bind(smalltalk.LuaEncoder,"emmitAssignment:",0
,function(aNode){var self=this;
(function($rec){$rec.nextArgumentsPut_((function(){var tab=nil;
tab=smalltalk.String.tab();(function($rec){$rec.nextPutAll_(unescape("function%28%29"));return $rec.cr();})(self);(function($rec){$rec.tab();return $rec.nextPutAll_(unescape("local%20_s8_value%3D"));})(self);self.nextArgumentsPut_((function(){return self.visit_(aNode.right());}));(function($rec){$rec.cr();return $rec.tab();})(self);self.visit_(aNode.left());(function($rec){$rec.nextPutAll_(unescape("%3D_s8_value"));$rec.cr();$rec.tab();$rec.nextPutAll_("return _s8_value");return $rec.cr();})(self);return self.nextPutAll_("end");}));return $rec.nextArgumentsPut_((function(){return nil;}));})(self);
return self;}
,"generation",unescape("emmitAssignment%3A%20aNode%0D%0A%09%22%20Private%20-%20Emmit%20the%20code%20for%20assignment.%20%22%0D%0A%0D%0A%09self%20nextArgumentsPut%3A%20%5B%20%7C%20tab%20%7C%0D%0A%09%09tab%20%3A%3D%20String%20tab.%0D%0A%09%09self%20nextPutAll%3A%20%27function%28%29%27%3B%20cr.%0D%0A%09%09self%20tab%3BnextPutAll%3A%20%27local%20_s8_value%3D%27.%0D%0A%09%09self%20nextArgumentsPut%3A%20%5B%20self%20visit%3A%20aNode%20right%20%5D.%0D%0A%09%09self%20cr%3B%20tab.%0D%0A%09%09self%20visit%3A%20aNode%20left.%0D%0A%09%09self%09nextPutAll%3A%20%27%3D_s8_value%27%20%22%3B%20%27%22%3B%0D%0A%09%09%09%20cr%3B%20tab%3B%0D%0A%09%09%09nextPutAll%3A%20%27return%20_s8_value%27%3B%20cr.%0D%0A%09%09self%20nextPutAll%3A%20%23end%0D%0A%09%5D%3B%20nextArgumentsPut%3A%20%5B%5D%20"));
smalltalk.bind(smalltalk.LuaEncoder,"visitClassVariableAssignment:in:to:",0
,function(aClassVariableName,aClass,anExpressionNode){var self=this;
(function($rec){$rec.nextPutAll_(self.smalltalkNameFor_(aClass));$rec.nextPutAll_(unescape("%3AclassVariableAt_put_%28%22"));$rec.nextPutAll_(aClassVariableName);return $rec.nextPutAll_(unescape("%22%2C"));})(self);
self.visit_(anExpressionNode);
self.nextPutAll_(unescape("%29"));
return self;}
,"visiting",unescape("visitClassVariableAssignment%3A%20aClassVariableName%20in%3A%20aClass%20to%3A%20anExpressionNode%0D%0A%0D%0A%09self%0D%0A%09%09nextPutAll%3A%20%28self%20smalltalkNameFor%3A%20aClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27%3AclassVariableAt_put_%28%22%27%3B%0D%0A%09%09nextPutAll%3A%20aClassVariableName%3B%20nextPutAll%3A%20%27%22%2C%27.%0D%0A%09self%20visit%3A%20anExpressionNode.%0D%0A%09self%09nextPutAll%3A%20%27%29%27"));
smalltalk.bind(smalltalk.LuaEncoder,"visitPoolVariableAssignment:in:to:",0
,function(aName,poolName,anExpressionNode){var self=this;
(function($rec){$rec.nextPutAll_(unescape("smalltalk%3ApoolDictionary_%28%22"));$rec.nextPutAll_(poolName);$rec.nextPutAll_(unescape("%22%29%3Aat_put_%28%22"));$rec.nextPutAll_(aName);return $rec.nextPutAll_(unescape("%22%2C"));})(self);
self.visit_(anExpressionNode);
self.nextPutAll_(unescape("%29"));
return self;}
,"visiting",unescape("visitPoolVariableAssignment%3A%20aName%20in%3A%20poolName%20to%3A%20anExpressionNode%0D%0A%0D%0A%09self%0D%0A%09%09nextPutAll%3A%20%27smalltalk%3ApoolDictionary_%28%22%27%3B%0D%0A%09%09nextPutAll%3A%20poolName%3B%20nextPutAll%3A%20%27%22%29%3Aat_put_%28%22%27%3B%0D%0A%09%09nextPutAll%3A%20aName%3B%20nextPutAll%3A%20%27%22%2C%27.%0D%0A%09self%20visit%3A%20anExpressionNode.%0D%0A%09self%09nextPutAll%3A%20%27%29%27"));
smalltalk.bind(smalltalk.LuaEncoder,"visitOperation:",0
,function(aNode){var self=this;
self.compilerEvent_with_("aboutOperation:",aNode);
self.nextArgumentsPut_((function(){return self.visitReceiver_(aNode.receiver());}));
self.nextPutAll_(self.asPrivateSelector_(aNode.selector()));
self.visitSendArguments_(aNode);
self.compilerEvent_with_("operation:",aNode);
return self;}
,"visiting",unescape("visitOperation%3A%20aNode%0D%0A%09%22%20Private%20-%20aNode%20is%20an%20operation%20and%20selector%20is%20mapped%20to%20target%20operation.%20%22%0D%0A%0D%0A%09self%20compilerEvent%3A%20%23aboutOperation%3A%20with%3A%20aNode.%0D%0A%09self%20nextArgumentsPut%3A%20%5B%20self%20visitReceiver%3A%20aNode%20receiver%20%5D.%0D%0A%09self%20nextPutAll%3A%20%28self%20asPrivateSelector%3A%20aNode%20selector%29.%0D%0A%09self%20visitSendArguments%3A%20aNode.%0D%0A%09self%20compilerEvent%3A%20%23operation%3A%20with%3A%20aNode."));
smalltalk.bind(smalltalk.LuaEncoder,"encodeSuperSend:",0
,function(aNode){var self=this;
self.nextPutAll_(unescape("smalltalk%3AsuperImplementor%28")._comma(self.smalltalkNameFor_(self.currentClass()))._comma(unescape("%2C%22"))._comma(self.asPrivateSelector_(aNode.selector()))._comma(unescape("%22%29%28self")));
if (aNode.arguments().notEmpty()){self.nextPutAll_(self.argumentsSeparatorOp());
self.visitSendArguments_(aNode);};;
self.nextPutAll_(unescape("%29"));
return self;}
,"encoding",unescape("encodeSuperSend%3A%20aNode%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20super%20message%20send.%22%0D%0A%0D%0A%09self%0D%0A%09%09nextPutAll%3A%20%27smalltalk%3AsuperImplementor%28%27%0D%0A%09%09%09%2C%28self%20smalltalkNameFor%3A%20self%20currentClass%29%0D%0A%09%09%09%2C%27%2C%22%27%2C%28self%20asPrivateSelector%3A%20aNode%20selector%29%0D%0A%09%09%09%2C%27%22%29%28self%27.%0D%0A%09aNode%20arguments%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%09self%20nextPutAll%3A%20self%20argumentsSeparatorOp.%0D%0A%09%09self%20visitSendArguments%3A%20aNode%0D%0A%09%5D.%0D%0A%09self%20nextPutAll%3A%20%27%29%27.%20"));
smalltalk.bind(smalltalk.LuaEncoder,"visitReturnNode:",0
,function(aNode){var self=this;
var theReturn=nil;
if (self["@nestedBlocks"]._gt((0))){self["@earlyReturn"]=true;};;
theReturn=(function(){return self.emmitReturn_((function(){return aNode.nodes().do_((function(each){return self.visit_(each);}));}));});
if (!(self["@earlyReturn"])){return theReturn.value();};;
(function($rec){$rec.nextArgumentsPut_((function(){return self.emmitFunctionDoing_((function(){(function($rec){$rec.nextPutAll_(unescape("_s8Ret_.hasResult%3Dtrue"));return $rec.cr();})(self);self.nextPutAll_(unescape("_s8Ret_.result%3D"));(function($rec){$rec.nextArgumentsPut_((function(){return (function($rec){$rec.emmitFunctionDoing_(theReturn);return $rec.cr();})(self);}));return $rec.nextArgumentsPut_((function(){return nil;}));})(self);return (function($rec){$rec.cr();return $rec.nextPutAll_(unescape("error%28_s8Ret_%2C0%29"));})(self);}));}));return $rec.nextArgumentsPut_((function(){return nil;}));})(self);
return self;}
,"visiting",unescape("visitReturnNode%3A%20aNode%0D%0A%0D%0A%09%7C%20theReturn%20%7C%0D%0A%09nestedBlocks%20%3E%200%20ifTrue%3A%20%5B%20earlyReturn%20%3A%3D%20true%20%5D.%0D%0A%09theReturn%20%3A%3D%20%5B%20self%20emmitReturn%3A%20%5B%0D%0A%09%09aNode%20nodes%20do%3A%20%5B%3Aeach%20%7C%20self%20visit%3A%20each%20%5D%0D%0A%09%5D%20%5D.%0D%0A%09earlyReturn%20ifFalse%3A%20%5B%20%5EtheReturn%20value%20%5D.%0D%0A%0D%0A%09self%20nextArgumentsPut%3A%20%5B%0D%0A%09%09self%20emmitFunctionDoing%3A%20%5B%0D%0A%09%09%09self%20nextPutAll%3A%20%27_s8Ret_.hasResult%3Dtrue%27%3Bcr.%0D%0A%09%09%09self%20nextPutAll%3A%20%27_s8Ret_.result%3D%27.%0D%0A%09%09%09self%20nextArgumentsPut%3A%20%5B%0D%0A%09%09%09%09self%20emmitFunctionDoing%3A%20theReturn%3B%20cr%0D%0A%09%09%09%5D%3B%20nextArgumentsPut%3A%20%5B%5D.%0D%0A%09%09%09self%20cr%3BnextPutAll%3A%20%27error%28_s8Ret_%2C0%29%27%0D%0A%09%09%5D%0D%0A%09%5D%3B%20nextArgumentsPut%3A%20%5B%5D%20"));
smalltalk.bind(smalltalk.LuaEncoder,"generate:early:return:",0
,function(fName,early,code){var self=this;
var prev=nil;
if (!(early)){self.nextPutAll_(code);
return (0);};;
prev=unescape("%0D%0Alocal%20_s8Ret_%3D%7B%5B%22name%22%5D%3D%22stReturn%22%7D%0D%0Alocal%20_s8_f%20%3D%20function%28%29%0D%0A");
(function($rec){$rec.nextPutAll_(prev);$rec.nextPutAll_(code);return $rec.nextPutAll_(unescape("%0D%0Aend%0D%0Alocal%20_s8_status%2C%20_s8_result%20%3D%20pcall%28_s8_f%29%0D%0Aif%20_s8_status%20then%20return%20_s8_result%0D%0Aelse%0D%0A%20%20%20%20if%20%28_s8Ret_.hasResult%29%20then%0D%0A%20%20%20%20%20%20"));})(self);
self.emmitReturn_early_("_s8Ret_.result",true);
self.nextPutAll_(unescape("%0D%0A%20%20%20%20end%0D%0Aend%0D%0Aerror%28_s8_result%2C0%29%0D%0A"));
return prev.size();}
,"generation",unescape("generate%3A%20fName%20early%3A%20early%20return%3A%20code%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20catch%20early%20return%20of%20code%20expression%20in%20fName.%20%22%0D%0A%0D%0A%09%7C%20prev%20%7C%0D%0A%09early%20ifFalse%3A%20%5B%20self%20nextPutAll%3A%20code.%20%5E0%20%5D.%0D%0A%09prev%20%3A%3D%20%27%0D%0Alocal%20_s8Ret_%3D%7B%5B%22name%22%5D%3D%22stReturn%22%7D%0D%0Alocal%20_s8_f%20%3D%20function%28%29%0D%0A%27.%0D%0A%09self%09nextPutAll%3A%20prev%3B%0D%0A%09%09nextPutAll%3A%20code%3B%0D%0A%09%09nextPutAll%3A%20%27%0D%0Aend%0D%0Alocal%20_s8_status%2C%20_s8_result%20%3D%20pcall%28_s8_f%29%0D%0Aif%20_s8_status%20then%20return%20_s8_result%0D%0Aelse%0D%0A%20%20%20%20if%20%28_s8Ret_.hasResult%29%20then%0D%0A%20%20%20%20%20%20%27.%09self%20emmitReturn%3A%20%27_s8Ret_.result%27%20early%3A%20true.%0D%0A%09self%20nextPutAll%3A%20%27%0D%0A%20%20%20%20end%0D%0Aend%0D%0Aerror%28_s8_result%2C0%29%0D%0A%27.%0D%0A%09%5Eprev%20size"));
smalltalk.bind(smalltalk.LuaEncoder,"coreOptimizations",0
,function(){var self=this;
return ["core", [["class", unescape("%25receiver%25%5B%22%24klass%22%5D")], ["isNil", unescape("%25%3AisNil_%25%28%25receiver%25%29")], ["notNil", unescape("%25%3AnotNil_%25%28%25receiver%25%29")], [unescape("%3D"), unescape("%25%3AisObject_equalTo_%25%28%25receiver%25%2C%25arg1%25%29")], [unescape("%7E%3D"), unescape("%28not%20%25%3AisObject_equalTo_%25%28%25receiver%25%2C%25arg1%25%29%29")], [unescape("%3D%3D"), unescape("%28%25receiver%25%3D%3D%25arg1%25%29")], [unescape("%7E%7E"), unescape("%28%25receiver%25%7E%3D%25arg1%25%29")], [unescape("%3D%3D%3D"), unescape("%28%25receiver%25%20%3D%3D%20%25arg1%25%29")], [unescape("%7C%7C"), unescape("%28%25receiver%25%20or%20%25arg1%25%29")], [unescape("%2C"), unescape("%25%3Aobject_comma_%25%28%25receiver%25%2C%25arg1%25%29")], ["basicAt:", unescape("%25receiver%25%5B%25arg1%25%5D")], ["basicAt:ifAbsent:", unescape("%25%3Abasic_at_ifAbsent_%25%28%25receiver%25%2C%25arg1%25%2C%25arg2%25%29")], ["basicAt:put:", unescape("%25%3Abasic_at_put_%25%28%25receiver%25%2C%25arg1%25%2C%25arg2%25%29")], ["ifUndefined:", unescape("%25%3AifUndefined_do_%25%28%25receiver%25%2C%25arg1%25%29")]]];}
,"optimization",unescape("coreOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20optimization%20set.%20%22%0D%0A%0D%0A%09%5E%23%28core%20%20%23%28%0D%0A%09%09%23%28class%09%27%25receiver%25%5B%22%24klass%22%5D%27%20%29%0D%0A%09%09%23%28isNil%09%27%25%3AisNil_%25%28%25receiver%25%29%27%20%29%0D%0A%09%09%23%28notNil%09%27%25%3AnotNil_%25%28%25receiver%25%29%27%20%29%0D%0A%09%09%23%28%3D%09%27%25%3AisObject_equalTo_%25%28%25receiver%25%2C%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%7E%3D%09%27%28not%20%25%3AisObject_equalTo_%25%28%25receiver%25%2C%25arg1%25%29%29%27%20%29%0D%0A%09%09%23%28%3D%3D%09%27%28%25receiver%25%3D%3D%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%7E%7E%09%27%28%25receiver%25%7E%3D%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%3D%3D%3D%09%27%28%25receiver%25%20%3D%3D%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%7C%7C%09%27%28%25receiver%25%20or%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%2C%09%27%25%3Aobject_comma_%25%28%25receiver%25%2C%25arg1%25%29%27%20%29%0D%0A%09%09%23%28basicAt%3A%09%27%25receiver%25%5B%25arg1%25%5D%27%20%29%0D%0A%09%09%23%28basicAt%3AifAbsent%3A%20%27%25%3Abasic_at_ifAbsent_%25%28%25receiver%25%2C%25arg1%25%2C%25arg2%25%29%27%20%29%0D%0A%09%09%23%28basicAt%3Aput%3A%09%27%25%3Abasic_at_put_%25%28%25receiver%25%2C%25arg1%25%2C%25arg2%25%29%27%20%29%0D%0A%09%09%23%28ifUndefined%3A%09%27%25%3AifUndefined_do_%25%28%25receiver%25%2C%25arg1%25%29%27%20%29%0D%0A%09%29%29"));
smalltalk.bind(smalltalk.LuaEncoder,"s8Optimizations",0
,function(){var self=this;
return ["s8", ["core", ["ifTrue:", "hasBlockNoArguments:", [["ifTrue:", "noClient:", unescape("if%20%25receiver%25%3D%3Dtrue%20then%20%25seq1%25%20end")], ["ifTrue:", unescape("%25%3Aif_true_false_%25%28%25receiver%25%2C%25arg1%25%2Cnil%29")]]], ["ifFalse:", "hasBlockNoArguments:", [["ifFalse:", "noClient:", unescape("if%20not%20%28%25receiver%25%29%3D%3Dtrue%20then%20%25seq1%25%20end")], ["ifFalse:", unescape("%25%3Aif_true_false_%25%28%25receiver%25%2Cnil%2C%25arg1%25%29")]]], ["ifTrue:ifFalse:", "hasBlockNoArguments:", [["ifTrue:ifFalse:", "noClient:", unescape("if%20%25receiver%25%3D%3Dtrue%20then%20%25seq1%25%20else%20%25seq2%25%20end")], ["ifTrue:ifFalse:", unescape("%25%3Aif_true_false_%25%28%25receiver%25%2C%25arg1%25%2C%25arg2%25%29")]]], ["ifFalse:ifTrue:", "hasBlockNoArguments:", [["ifFalse:ifTrue:", "noClient:", unescape("if%20%25receiver%25%3D%3Dtrue%20then%20%25seq2%25%20else%20%25seq1%25%20end")], ["ifFalse:ifTrue:", unescape("%25%3Aif_true_false_%25%28%25receiver%25%2C%25arg2%25%2C%25arg1%25%29")]]], ["whileTrue", "receiverIsBlockNoArguments:", [["whileTrue", "noClient:", unescape("while%20%25receiver%25%28%29%20do%20end")], ["whileTrue", unescape("%25%3Awhile_do_%25%28%25receiver%25%2Cnil%29")]]], ["whileFalse", "receiverIsBlockNoArguments:", [["whileFalse", "noClient:", unescape("while%20not%20%25receiver%25%28%29%20do%20end")], ["whileFalse", unescape("%25%3AwhileNot_do_%25%28%25receiver%25%2Cnil%29")]]], ["whileTrue:", "receiverIsBlockNoArguments:", [["whileTrue:", "noClient:", unescape("while%20%25receiver%25%28%29%20do%20%25seq1%25%20end")], ["whileTrue:", unescape("%25%3Awhile_do_%25%28%25receiver%25%2C%25arg1%25%29")]]], ["whileFalse:", "receiverIsBlockNoArguments:", [["whileFalse:", "noClient:", unescape("while%20not%20%25receiver%25%28%29%20do%20%25seq1%25%20end")], ["whileFalse:", unescape("%25%3AwhileNot_do_%25%28%25receiver%25%2C%25arg1%25%29")]]], ["orNil", unescape("%25%3AorNil_%25%28%25receiver%25%29")], ["timesRepeat:", unescape("%25%3Atimes_repeat_%25%28%25receiver%25%2C%25arg1%25%29")], ["isKindOf:", unescape("%25%3Ais_kindOf_%25%28%25receiver%25%2C%28%25arg1%25%29%29")], ["respondsTo:", unescape("%25%3Aresponds_to_%25%28%25receiver%25%2C%28%25arg1%25%29%29")], ["ifNil:", unescape("%25%3Aif_nil_notNil_%25%28%25receiver%25%2C%25arg1%25%2Cnil%29")], ["ifNotNil:", unescape("%25%3Aif_nil_notNil_%25%28%25receiver%25%2Cnil%2C%25arg1%25%29")], ["ifNil:ifNotNil:", unescape("%25%3Aif_nil_notNil_%25%28%25receiver%25%2C%25arg1%25%2C%25arg2%25%29")], ["ifNotNil:ifNil:", unescape("%25%3Aif_nil_notNil_%25%28%25receiver%25%2C%25arg2%25%2C%25arg1%25%29")]]];}
,"optimization",unescape("s8Optimizations%0D%0A%09%22%20Private%20-%20Return%20the%20optimization%20set.%20%22%0D%0A%0D%0A%09%5E%23%28s8%09%23%28%0D%0A%09%09core%0D%0A%09%09%23%28ifTrue%3A%20hasBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28ifTrue%3A%20noClient%3A%20%27if%20%25receiver%25%3D%3Dtrue%20then%20%25seq1%25%20end%27%20%29%0D%0A%09%09%09%23%28ifTrue%3A%20%27%25%3Aif_true_false_%25%28%25receiver%25%2C%25arg1%25%2Cnil%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28ifFalse%3A%20hasBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28ifFalse%3A%20noClient%3A%20%27if%20not%20%28%25receiver%25%29%3D%3Dtrue%20then%20%25seq1%25%20end%27%20%29%0D%0A%09%09%09%23%28ifFalse%3A%20%27%25%3Aif_true_false_%25%28%25receiver%25%2Cnil%2C%25arg1%25%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28ifTrue%3AifFalse%3A%20hasBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28ifTrue%3AifFalse%3A%20noClient%3A%20%27if%20%25receiver%25%3D%3Dtrue%20then%20%25seq1%25%20else%20%25seq2%25%20end%27%20%29%0D%0A%09%09%09%23%28ifTrue%3AifFalse%3A%20%27%25%3Aif_true_false_%25%28%25receiver%25%2C%25arg1%25%2C%25arg2%25%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28ifFalse%3AifTrue%3A%20hasBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28ifFalse%3AifTrue%3A%20noClient%3A%20%27if%20%25receiver%25%3D%3Dtrue%20then%20%25seq2%25%20else%20%25seq1%25%20end%27%20%29%0D%0A%09%09%09%23%28ifFalse%3AifTrue%3A%20%27%25%3Aif_true_false_%25%28%25receiver%25%2C%25arg2%25%2C%25arg1%25%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28whileTrue%20receiverIsBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28whileTrue%20noClient%3A%20%27while%20%25receiver%25%28%29%20do%20end%27%20%29%0D%0A%09%09%09%23%28whileTrue%20%27%25%3Awhile_do_%25%28%25receiver%25%2Cnil%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28whileFalse%20receiverIsBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28whileFalse%20noClient%3A%20%27while%20not%20%25receiver%25%28%29%20do%20end%27%20%29%0D%0A%09%09%09%23%28whileFalse%20%27%25%3AwhileNot_do_%25%28%25receiver%25%2Cnil%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28whileTrue%3A%20receiverIsBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28whileTrue%3A%20noClient%3A%20%27while%20%25receiver%25%28%29%20do%20%25seq1%25%20end%27%20%29%0D%0A%09%09%09%23%28whileTrue%3A%20%27%25%3Awhile_do_%25%28%25receiver%25%2C%25arg1%25%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28whileFalse%3A%20receiverIsBlockNoArguments%3A%20%23%28%0D%0A%09%09%09%23%28whileFalse%3A%20noClient%3A%20%27while%20not%20%25receiver%25%28%29%20do%20%25seq1%25%20end%27%20%29%0D%0A%09%09%09%23%28whileFalse%3A%20%27%25%3AwhileNot_do_%25%28%25receiver%25%2C%25arg1%25%29%27%20%29%0D%0A%09%09%29%29%0D%0A%09%09%23%28orNil%20%27%25%3AorNil_%25%28%25receiver%25%29%27%20%29%0D%0A%09%09%23%28timesRepeat%3A%20%27%25%3Atimes_repeat_%25%28%25receiver%25%2C%25arg1%25%29%27%20%29%0D%0A%09%09%23%28isKindOf%3A%20%27%25%3Ais_kindOf_%25%28%25receiver%25%2C%28%25arg1%25%29%29%27%20%29%0D%0A%09%09%23%28respondsTo%3A%20%27%25%3Aresponds_to_%25%28%25receiver%25%2C%28%25arg1%25%29%29%27%20%29%0D%0A%09%09%23%28ifNil%3A%20%27%25%3Aif_nil_notNil_%25%28%25receiver%25%2C%25arg1%25%2Cnil%29%27%20%29%0D%0A%09%09%23%28ifNotNil%3A%20%27%25%3Aif_nil_notNil_%25%28%25receiver%25%2Cnil%2C%25arg1%25%29%27%20%29%0D%0A%09%09%23%28ifNil%3AifNotNil%3A%20%27%25%3Aif_nil_notNil_%25%28%25receiver%25%2C%25arg1%25%2C%25arg2%25%29%27%20%29%0D%0A%09%09%23%28ifNotNil%3AifNil%3A%20%27%25%3Aif_nil_notNil_%25%28%25receiver%25%2C%25arg2%25%2C%25arg1%25%29%27%20%29%0D%0A%09%29%29"));
smalltalk.bind(smalltalk.LuaEncoder,"magnitudeOptimizations",0
,function(){var self=this;
return ["magnitude", [[unescape("%3E"), unescape("%28%25receiver%25%20%3E%20%25arg1%25%29")], [unescape("%3C"), unescape("%28%25receiver%25%20%3C%20%25arg1%25%29")], [unescape("%3E%3D"), unescape("%28%25receiver%25%20%3E%3D%20%25arg1%25%29")], [unescape("%3C%3D"), unescape("%28%25receiver%25%20%3C%3D%20%25arg1%25%29")]]];}
,"optimization",unescape("magnitudeOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20optimization%20set.%20%22%0D%0A%0D%0A%09%5E%23%28magnitude%20%23%28%0D%0A%09%09%23%28%3E%20%09%27%28%25receiver%25%20%3E%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%3C%20%09%27%28%25receiver%25%20%3C%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%3E%3D%09%27%28%25receiver%25%20%3E%3D%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%3C%3D%09%27%28%25receiver%25%20%3C%3D%20%25arg1%25%29%27%20%29%0D%0A%09%29%29"));
smalltalk.bind(smalltalk.LuaEncoder,"mathOptimizations",0
,function(){var self=this;
return ["math", [["max:", unescape("%25%3Amath_max_%25%28%25receiver%25%2C%25arg1%25%29")], ["min:", unescape("%25%3Amath_min_%25%28%25receiver%25%2C%25arg1%25%29")], ["mustBeNumber", unescape("%25%3AmustBeNumber_%25%28%25receiver%25%29")], [unescape("+"), unescape("%28%25receiver%25%20+%20%25arg1%25%29")], [unescape("-"), unescape("%28%25receiver%25%20-%20%25arg1%25%29")], [unescape("*"), unescape("%28%25receiver%25%20*%20%25arg1%25%29")], [unescape("/"), unescape("%28%25receiver%25%20/%20%25arg1%25%29")]]];}
,"optimization",unescape("mathOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20optimization%20set.%20%22%0D%0A%0D%0A%09%5E%23%28math%20%23%28%0D%0A%09%09%23%28max%3A%09%27%25%3Amath_max_%25%28%25receiver%25%2C%25arg1%25%29%27%20%29%0D%0A%09%09%23%28min%3A%09%27%25%3Amath_min_%25%28%25receiver%25%2C%25arg1%25%29%27%20%29%0D%0A%09%09%23%28mustBeNumber%09%27%25%3AmustBeNumber_%25%28%25receiver%25%29%27%20%29%0D%0A%09%09%23%28+%09%27%28%25receiver%25%20+%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28%23-%09%27%28%25receiver%25%20-%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28*%09%27%28%25receiver%25%20*%20%25arg1%25%29%27%20%29%0D%0A%09%09%23%28/%09%27%28%25receiver%25%20/%20%25arg1%25%29%27%20%29%0D%0A%09%29%29"));
smalltalk.bind(smalltalk.LuaEncoder,"printingOptimizations",0
,function(){var self=this;
return ["printing", [["toString", unescape("%25%3AtoString_%25%28%25receiver%25%29")], ["printString", unescape("%25%3AprintString_%25%28%25receiver%25%29")], ["asString", unescape("%25%3AasString_%25%28%25receiver%25%29")], ["asCharacter", unescape("%25%3AasCharacter_%25%28%25receiver%25%29")]]];}
,"optimization",unescape("printingOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20optimization%20set.%20%22%0D%0A%0D%0A%09%5E%23%28printing%20%23%28%0D%0A%09%09%23%28toString%09%27%25%3AtoString_%25%28%25receiver%25%29%27%20%29%0D%0A%09%09%23%28printString%09%27%25%3AprintString_%25%28%25receiver%25%29%27%20%29%0D%0A%09%09%23%28asString%09%27%25%3AasString_%25%28%25receiver%25%29%27%20%29%0D%0A%09%09%23%28asCharacter%09%27%25%3AasCharacter_%25%28%25receiver%25%29%27%20%29%0D%0A%09%29%29"));
smalltalk.bind(smalltalk.LuaEncoder,"booleanOptimizations",0
,function(){var self=this;
return ["boolean", [["isBoolean", unescape("%28type%28%25receiver%25%29%3D%3D%22boolean%22%29")], ["and:", "hasBlockNoArguments:", unescape("%25%3Aboolean_and_%25%28%25receiver%25%2C%25arg1%25%29")], ["or:", "hasBlockNoArguments:", unescape("%25%3Aboolean_or_%25%28%25receiver%25%2C%25arg1%25%29")], ["not", unescape("%25%3AbooleanNot_%25%28%25receiver%25%29")]]];}
,"optimization",unescape("booleanOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20optimization%20set.%20%22%0D%0A%0D%0A%09%5E%23%28boolean%20%23%28%0D%0A%09%09%23%28isBoolean%20%27%28type%28%25receiver%25%29%3D%3D%22boolean%22%29%27%20%29%0D%0A%09%09%23%28and%3A%20hasBlockNoArguments%3A%20%27%25%3Aboolean_and_%25%28%25receiver%25%2C%25arg1%25%29%27%20%29%0D%0A%09%09%23%28or%3A%20hasBlockNoArguments%3A%20%20%27%25%3Aboolean_or_%25%28%25receiver%25%2C%25arg1%25%29%27%20%29%0D%0A%09%09%23%28not%20%27%25%3AbooleanNot_%25%28%25receiver%25%29%27%20%29%0D%0A%09%29%29"));
smalltalk.bind(smalltalk.LuaEncoder,"defaultKnownOptimizations",0
,function(){var self=this;
return (function($rec){$rec.add_(self.s8Optimizations());$rec.add_(self.magnitudeOptimizations());$rec.add_(self.booleanOptimizations());$rec.add_(self.printingOptimizations());return $rec.yourself();})(smalltalk.Array.with_(self.coreOptimizations()));}
,"defaults",unescape("defaultKnownOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20default%20knownOptimizations%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28Array%09with%3A%20self%20coreOptimizations%29%0D%0A%09%09add%3A%20self%20s8Optimizations%3B%0D%0A%09%09add%3A%20self%20magnitudeOptimizations%3B%0D%0A%22%09%09add%3A%20self%20mathOptimizations%3B%22%0D%0A%09%09add%3A%20self%20booleanOptimizations%3B%0D%0A%09%09add%3A%20self%20printingOptimizations%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.LuaEncoder,"defaultOptimizations",0
,function(){var self=this;
return ["s8", "magnitude", "boolean", "printing"];}
,"defaults",unescape("defaultOptimizations%0D%0A%09%22%20Private%20-%20Return%20the%20default%20optimizations%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%20s8%20magnitude%20%22math%22%20boolean%20%22evaluation%22%20printing%20%29"));
smalltalk.bind(smalltalk.Compiler.$klass,"constructEventsTriggered",0
,function(){var self=this;
return ["needsCookie:", "startVisit:", "endVisit:", "startMethod:", "enterMethod:", "endMethod:", "startFunction:with:", "startFunctionName:", "startFunctionBody:with:", "endFunction:", "aboutReturn:", "return:", "returned:early:", "aboutSend:", "send:", "aboutOperation:", "operation:", "aboutCascade:", "cascade:", "aboutToCompile:", "setTransformers:", "needsParser", "aboutToParse:", "needsEncoder", "aboutToEncode:", "aboutToRecompile:", "recompiled:", "startCompiling:", "endCompiling:", "needsOptimizationMap", "initializeOptimizations", "customizeOptimizations", "unknownVariable:", "acceptAssignmentTo:", "invalidAssignment:", "move:by:"]._comma(smalltalk.superImplementor(smalltalk.Compiler.$klass,"constructEventsTriggered").apply(self, []));}
,"events",unescape("constructEventsTriggered%0D%0A%09%22%20Private%20-%20Return%20the%20events%20managed%20by%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%09%23needsCookie%3A%20%23startVisit%3A%20%23endVisit%3A%0D%0A%09%09%23startMethod%3A%20%23enterMethod%3A%20%23endMethod%3A%0D%0A%09%09%23startFunction%3Awith%3A%20%23startFunctionName%3A%20%23startFunctionBody%3Awith%3A%20%23endFunction%3A%0D%0A%09%09%23aboutReturn%3A%20%23return%3A%20%23returned%3Aearly%3A%0D%0A%09%09%23aboutSend%3A%20%23send%3A%0D%0A%09%09%23aboutOperation%3A%20%23operation%3A%0D%0A%09%09%23aboutCascade%3A%20%23cascade%3A%0D%0A%09%09%23aboutToCompile%3A%20%23setTransformers%3A%0D%0A%09%09%23needsParser%20%23aboutToParse%3A%0D%0A%09%09%23needsEncoder%20%23aboutToEncode%3A%0D%0A%09%09%23aboutToRecompile%3A%20%23recompiled%3A%0D%0A%09%09%23startCompiling%3A%20%23endCompiling%3A%0D%0A%09%09%23needsOptimizationMap%20%23initializeOptimizations%20%23customizeOptimizations%0D%0A%09%09%23unknownVariable%3A%20%23acceptAssignmentTo%3A%20%23invalidAssignment%3A%0D%0A%09%09%23move%3Aby%3A%0D%0A%09%29%20%2Csuper%20constructEventsTriggered"));
smalltalk.bind(smalltalk.Compiler.$klass,"debugCookieFrom:",0
,function(aStream){var self=this;
var nextOffset=nil;
nextOffset=(function(){return aStream.nextChunk().asNumber();});
return smalltalk.Array.with_with_with_with_(aStream.nextChunk(),nextOffset.value()._at(nextOffset.value()),nextOffset.value()._at(nextOffset.value()),aStream.nextChunk().asArrayOfSubstrings());}
,"private",unescape("debugCookieFrom%3A%20aStream%0D%0A%09%22%20Private%20-%20Return%20the%20next%20debug%20cookie%20encoded%20in%20aStream%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20nextOffset%20%7C%0D%0A%09nextOffset%20%3A%3D%20%5B%20aStream%20nextChunk%20asNumber%20%5D.%0D%0A%09%5EArray%09%22ref.%20%23debugInfo%3AforClass%3A%20%22%0D%0A%09%09with%3A%20aStream%20nextChunk%0D%0A%09%09with%3A%20nextOffset%20value%20@%20nextOffset%20value%0D%0A%09%09with%3A%20nextOffset%20value%20@%20nextOffset%20value%0D%0A%09%09with%3A%20aStream%20nextChunk%20asArrayOfSubstrings"));
smalltalk.bind(smalltalk.Compiler.$klass,"debugCookiesFrom:",0
,function(aStream){var self=this;
var result=nil;
var cookie=nil;
result=smalltalk.Array.$$new();
while(!(function(){return (function($rec){$rec.skipSeparators();return $rec.atEnd();})(aStream);})()){cookie=self.debugCookieFrom_(aStream);
(((nil.isNil_(cookie)===false)).mustBeBoolean()==true ? (function(){return result.add_(cookie);})() : nil);};;
return result;}
,"private",unescape("debugCookiesFrom%3A%20aStream%0D%0A%09%22%20Private%20-%20Return%20the%20debug%20cookies%20encoded%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20result%20cookie%20%7C%0D%0A%09result%20%3A%3D%20Array%20new.%0D%0A%09%5B%20aStream%20skipSeparators%3B%20atEnd%20%5D%20whileFalse%3A%20%5B%0D%0A%09%09cookie%20%3A%3D%20self%20debugCookieFrom%3A%20aStream.%0D%0A%09%09cookie%20notNil%20ifTrue%3A%20%5B%20result%20add%3A%20cookie%20%5D.%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Compiler.$klass,"debugInfo:",0
,function(aMethod){var self=this;
return aMethod.basicAt_ifAbsentPut_("$debugInfo",(function(){return self.debugInfo_forClass_(aMethod.source(),aMethod.classField());}));}
,"debug",unescape("debugInfo%3A%20aMethod%0D%0A%09%22%20Return%20the%20debug%20information%20of%20aMethod.%20%22%0D%0A%0D%0A%09%5EaMethod%20basicAt%3A%20%27%24debugInfo%27%20ifAbsentPut%3A%20%5B%0D%0A%09%09self%20debugInfo%3A%20aMethod%20source%20forClass%3A%20aMethod%20classField%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Compiler.$klass,"debugInfo:forClass:",0
,function(aString,aClass){var self=this;
var compiler=nil;
var output=nil;
var cookies=nil;
var aNode=nil;
var delta=nil;
output="".writeStream();
cookies=[];
delta=(0);
compiler=self.$$new();
(function($rec){$rec.useTemplates_(false);$rec.when_do_("needsCookie:",(function(one){return cookies.add_(smalltalk.Array.with_with_(one,compiler.encoder().position()));}));$rec.when_do_("endVisit:",(function(cookie){return cookie.add_(compiler.encoder().position());}));return $rec.when_do_("move:by:",(function(one,distance){return delta=delta._plus(distance);}));})(compiler);
(function($rec){$rec.nextChunkPut_(aClass.name());$rec.nextChunkPut_(aString);$rec.nextChunkPut_(compiler.compile_forClass_(aString,aClass));return $rec.yourself();})(output);
cookies.do_((function(tuple){aNode=tuple.first();(function($rec){$rec.nextChunkPut_(aNode.$klass.name());$rec.nextChunkPut_(aNode.start().asString());$rec.nextChunkPut_(aNode.end().asString());$rec.nextChunkPut_(tuple.second()._plus(delta).asString());$rec.nextChunkPut_(tuple.third()._plus(delta).asString());return $rec.yourself();})(output);aNode.bindings().do_((function(each){return output.nextPutAll_(" "._comma(each));}));return output.nextChunkPut_(" ");}));
return output.contents();}
,"debug",unescape("debugInfo%3A%20aString%20forClass%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20aString%20with%20information%20of%20compiling%20aString%20as%20a%20method%20implemented%20in%20aClass.%20%22%0D%0A%0D%0A%09%7C%20compiler%20output%20cookies%20aNode%20delta%20%7C%0D%0A%09output%20%3A%3D%20%27%27%20writeStream.%0D%0A%09cookies%20%3A%3D%20%23%28%29.%20delta%20%3A%3D%200.%0D%0A%09compiler%20%3A%3D%20self%20new.%0D%0A%09compiler%0D%0A%09%09useTemplates%3A%20false%3B%0D%0A%09%09when%3A%20%23needsCookie%3A%20do%3A%20%5B%3Aone%7C%20cookies%20add%3A%20%28Array%20with%3A%20one%20with%3A%20compiler%20encoder%20position%29%20%5D%3B%0D%0A%09%09when%3A%20%23endVisit%3A%20do%3A%20%5B%3Acookie%7C%20cookie%20add%3A%20compiler%20encoder%20position%20%5D%3B%0D%0A%09%09when%3A%20%23move%3Aby%3A%20do%3A%20%5B%3Aone%20%3Adistance%7C%20delta%20%3A%3D%20delta%20+%20distance%20%5D.%0D%0A%09output%0D%0A%09%09nextChunkPut%3A%20aClass%20name%3B%0D%0A%09%09nextChunkPut%3A%20aString%3B%0D%0A%09%09nextChunkPut%3A%20%28compiler%20compile%3A%20aString%20forClass%3A%20aClass%29%3B%0D%0A%09%09yourself.%0D%0A%09cookies%20do%3A%20%5B%3Atuple%7C%0D%0A%09%09aNode%20%3A%3D%20tuple%20first.%0D%0A%09%09output%0D%0A%09%09%09nextChunkPut%3A%20aNode%20class%20name%3B%0D%0A%09%09%09nextChunkPut%3A%20aNode%20start%20asString%3B%0D%0A%09%09%09nextChunkPut%3A%20aNode%20end%20asString%3B%0D%0A%09%09%09nextChunkPut%3A%20%28tuple%20second%20+%20delta%29%20asString%3B%0D%0A%09%09%09nextChunkPut%3A%20%28tuple%20third%20+%20delta%29%20asString%3B%0D%0A%09%09%09yourself.%0D%0A%09%09aNode%20bindings%20do%3A%20%5B%3Aeach%7C%20output%20nextPutAll%3A%20%27%20%27%20%2Ceach%20%5D.%0D%0A%09%09output%20nextChunkPut%3A%20%27%20%27.%0D%0A%09%5D.%0D%0A%09%5Eoutput%20contents"));
smalltalk.bind(smalltalk.Compiler.$klass,"debugInfoFrom:",0
,function(aString){var self=this;
return self.debugInfoFromStream_(aString.stream());}
,"debug",unescape("debugInfoFrom%3A%20aString%0D%0A%09%22%20Private%20-%20Return%20the%20debug%20information%20decoding%20aString%20contents.%0D%0A%09Note%3A%20aString%20must%20be%20result%20of%20sending%20%23getInfo%3A%20to%20the%20receiver.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20debugInfoFromStream%3A%20aString%20stream"));
smalltalk.bind(smalltalk.Compiler.$klass,"debugInfoFromStream:",0
,function(aStream){var self=this;
return smalltalk.Array.with_with_with_with_(aStream.nextChunk(),aStream.nextChunk(),aStream.nextChunk(),self.debugCookiesFrom_(aStream));}
,"private",unescape("debugInfoFromStream%3A%20aStream%0D%0A%09%22%20Private%20-%20Return%20the%20debug%20information%20encoded%20in%20aStream.%20%22%0D%0A%0D%0A%09%5EArray%09%22ref.%20%23debugInfo%3AforClass%3A%20%22%0D%0A%09%09with%3A%20aStream%20nextChunk%0D%0A%09%09with%3A%20aStream%20nextChunk%0D%0A%09%09with%3A%20aStream%20nextChunk%0D%0A%09%09with%3A%20%28self%20debugCookiesFrom%3A%20aStream%29"));
smalltalk.bind(smalltalk.Compiler.$klass,"doIt:",0
,function(smalltalkExpression){var self=this;
return self.doIt_to_(smalltalkExpression,nil);}
,"evaluation",unescape("doIt%3A%20smalltalkExpression%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20smalltalk%20expression%20in%20the%20global%20context%20or%20a%20description%20of%20an%20error%20situation%20occurred%20evaluating%20the%20code.%20%22%0D%0A%0D%0A%09%5Eself%20doIt%3A%20smalltalkExpression%20to%3A%20nil"));
smalltalk.bind(smalltalk.Compiler.$klass,"doItOBSOLETE:to:",0
,function(smalltalkExpression,aReceiver){var self=this;
var result=nil;
(function(){return result=(((nil.isNil_(aReceiver))).mustBeBoolean()==true ? (function(){return self.evaluate_(smalltalkExpression);})() : (function(){return self.evaluate_to_(smalltalkExpression,aReceiver);})());}).on_do_(smalltalk.Exception,(function(error){result=error;if (self.isObject_(error).and_((function(){return error.isString().not();}))){result=error.$klass.name()._comma(": ")._comma(error.messageText());};;return self.note_(result);}));
return result;}
,"evaluation",unescape("doItOBSOLETE%3A%20smalltalkExpression%20to%3A%20aReceiver%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20smalltalk%20expression%20in%20the%20context%20of%20aReceiver%20or%20a%20description%20of%20an%20error%20situation%20occurred%20evaluating%20the%20code.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09%5B%20result%20%3A%3D%20aReceiver%20isNil%0D%0A%09%09%09ifTrue%3A%20%20%5B%20self%20evaluate%3A%20smalltalkExpression%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20self%20evaluate%3A%20smalltalkExpression%20to%3A%20aReceiver%20%5D%0D%0A%09%5D%20on%3A%20Exception%20do%3A%20%5B%3Aerror%7C%0D%0A%09%09result%20%3A%3D%20error.%0D%0A%09%09%28%28self%20isObject%3A%20error%29%20and%3A%20%5B%20error%20isString%20not%20%5D%29%0D%0A%09%09ifTrue%3A%20%5B%20result%20%3A%3D%20error%20class%20name%20%2C%27%3A%20%27%20%2Cerror%20messageText%20%5D.%0D%0A%09%09self%20note%3A%20result%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Compiler.$klass,"doIt:to:",0
,function(smalltalkExpression,aReceiver){var self=this;
var result=nil;
(function(){return result=(((nil.isNil_(aReceiver))).mustBeBoolean()==true ? (function(){return self.evaluate_(smalltalkExpression);})() : (function(){return self.evaluate_to_(smalltalkExpression,aReceiver);})());}).on_do_(smalltalk.Exception,(function(error){result=error;if (self.isObject_(error).and_((function(){return smalltalk.responds_to_(error,("description"));}))){result=error.description();};;return self.note_(result);}));
return result;}
,"evaluation",unescape("doIt%3A%20smalltalkExpression%20to%3A%20aReceiver%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20smalltalk%20expression%20in%20the%20context%20of%20aReceiver%20or%20a%20description%20of%20an%20error%20situation%20occurred%20evaluating%20the%20code.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09%5B%20result%20%3A%3D%20aReceiver%20isNil%0D%0A%09%09%09ifTrue%3A%20%20%5B%20self%20evaluate%3A%20smalltalkExpression%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20self%20evaluate%3A%20smalltalkExpression%20to%3A%20aReceiver%20%5D%0D%0A%09%5D%20on%3A%20Exception%20do%3A%20%5B%3Aerror%7C%0D%0A%09%09result%20%3A%3D%20error.%0D%0A%09%09%28%28self%20isObject%3A%20error%29%20and%3A%20%5B%20error%20respondsTo%3A%20%23description%20%5D%29%0D%0A%09%09ifTrue%3A%20%5B%20result%20%3A%3D%20error%20description%20%5D.%0D%0A%09%09self%20note%3A%20result%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Compiler.$klass,"evaluate:",0
,function(smalltalkExpression){var self=this;
return self.$$new().evaluate_(smalltalkExpression);}
,"evaluation",unescape("evaluate%3A%20smalltalkExpression%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20smalltalk%20expression%20in%20a%20global%20context.%20%22%0D%0A%0D%0A%09%5Eself%20new%20evaluate%3A%20smalltalkExpression"));
smalltalk.bind(smalltalk.Compiler.$klass,"evaluate:in:to:",0
,function(expression,aClass,aReceiver){var self=this;
return aClass.compiler().evaluate_in_to_(expression,aClass,aReceiver);}
,"evaluation",unescape("evaluate%3A%20expression%20in%3A%20aClass%20to%3A%20aReceiver%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20expression%20with%20aReceiver.%20%22%0D%0A%0D%0A%09%5EaClass%20compiler%20evaluate%3A%20expression%20in%3A%20aClass%20to%3A%20aReceiver"));
smalltalk.bind(smalltalk.Compiler.$klass,"targetFor:",0
,function(aReceiver){var self=this;
return ((self.isObject_(aReceiver)).mustBeBoolean()==true ? (function(){return aReceiver;})() : (function(){return smalltalk.NativeHandle.support()._at(aReceiver);})());}
,"evaluation",unescape("targetFor%3A%20aReceiver%0D%0A%09%22%20Return%20the%20object%20to%20use%20for%20compiling%20expressions%20to%20be%20evaluated%20in%20aReceiver.%20%22%0D%0A%0D%0A%09%5E%28self%20isObject%3A%20aReceiver%29%0D%0A%09%09ifTrue%3A%20%5B%20aReceiver%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20NativeHandle%20support%20@%20aReceiver%20%5D%20"));
smalltalk.bind(smalltalk.Compiler.$klass,"evaluate:to:",0
,function(expression,aReceiver){var self=this;
var target=nil;
target=self.targetFor_(aReceiver);
return self.evaluate_in_to_(expression,target.$klass,target);}
,"evaluation",unescape("evaluate%3A%20expression%20to%3A%20aReceiver%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20expression%20with%20aReceiver.%20%22%0D%0A%0D%0A%09%7C%20target%20%7C%0D%0A%09target%20%3A%3D%20self%20targetFor%3A%20aReceiver.%0D%0A%09%5Eself%20evaluate%3A%20expression%20in%3A%20target%20class%20to%3A%20target"));
smalltalk.bind(smalltalk.Compiler.$klass,"for:",0
,function(aBehavior){var self=this;
return (function($rec){$rec.currentClass_(aBehavior);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("for%3A%20aBehavior%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09currentClass%3A%20aBehavior%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Compiler.$klass,"getInfo:",0
,function(aString){var self=this;
var stream=nil;
var aClass=nil;
var selector=nil;
var source=nil;
var aMethod=nil;
stream=aString.stream();
aClass=stream.nextChunk().doIt();
selector=stream.nextChunk();
source=stream.nextChunk();
aMethod=aClass._gt_gt(selector);
if ((nil.isNil_(aMethod)===false).and_((function(){return source.trimBlanks().isEmpty().or_((function(){return aMethod.source()._eq(source);}));}))){return self.debugInfo_(aMethod);};;
return self.debugInfo_forClass_(source,aClass);}
,"debug",unescape("getInfo%3A%20aString%0D%0A%09%22%20Private%20-%20Return%20the%20information%20of%20compiling%20aString%20as%20a%20chunked%20spec.%20%22%0D%0A%0D%0A%09%7C%20stream%20aClass%20selector%20source%20aMethod%20%7C%0D%0A%09stream%20%3A%3D%20aString%20stream.%0D%0A%09aClass%20%3A%3D%20stream%20nextChunk%20doIt.%0D%0A%09selector%20%3A%3D%20stream%20nextChunk.%0D%0A%09source%20%3A%3D%20stream%20nextChunk.%0D%0A%09aMethod%20%3A%3D%20aClass%20%3E%3E%20selector.%0D%0A%09%28aMethod%20notNil%20and%3A%20%5B%20source%20trimBlanks%20isEmpty%20or%3A%20%5B%0D%0A%09%09aMethod%20source%20%3D%20source%20%5D%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20debugInfo%3A%20aMethod%0D%0A%09%5D.%0D%0A%09%5Eself%20debugInfo%3A%20source%20forClass%3A%20aClass"));
smalltalk.bind(smalltalk.Compiler,"parseMethod:",0
,function(aString){var $s8Ret$={name:"stReturn"};try{var self=this;
var result=nil;
self.triggerEvent_with_ifNotHandled_("aboutToParse:",aString,(function(){result=self.nodeFromTemplateIn_(aString);return (((nil.isNil_(result)===false)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return result})();throw($s8Ret$)})();})() : nil);}));
result=self.parser().parse_(aString.readStream());
(function(){$s8Ret$.result=(function(){return result})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"parsing",unescape("parseMethod%3A%20aString%0D%0A%09%22%20Return%20the%20result%20of%20parsing%20aString%20as%20a%20method%20source%20code.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09self%09triggerEvent%3A%20%23aboutToParse%3A%0D%0A%09%09with%3A%20aString%20ifNotHandled%3A%20%5B%0D%0A%09%09%09result%20%3A%3D%20self%20nodeFromTemplateIn%3A%20aString.%0D%0A%09%09%09result%20notNil%20ifTrue%3A%20%5B%20%5Eresult%20%5D%0D%0A%09%09%5D.%0D%0A%09result%20%3A%3D%20self%20parser%20parse%3A%20aString%20readStream.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Compiler,"parse:",0
,function(aString){var self=this;
self.error_(unescape("OBSOLETE%20message%20%23parse%3A%20")._comma(aString.printString()));
return self.parseMethod_(aString);}
,"parsing",unescape("parse%3A%20aString%0D%0A%09%22%20OBSOLETE%20-%20Return%20the%20result%20of%20parsing%20aString.%20%22%0D%0A%0D%0A%09self%20error%3A%20%27OBSOLETE%20message%20%23parse%3A%20%27%20%2CaString%20printString.%0D%0A%09%5Eself%20parseMethod%3A%20aString%20"));
smalltalk.bind(smalltalk.Compiler,"compile:",0
,function(methodSource){var self=this;
var aNode=nil;
aNode=self.parseMethod_(methodSource);
self.triggerEvent_with_("aboutToCompile:",aNode);
return self.emmitNode_(aNode);}
,"private",unescape("compile%3A%20methodSource%0D%0A%09%22%20Return%20the%20native%20code%20result%20of%20compiling%20methodSource%20for%20currentClass.%20%22%0D%0A%0D%0A%09%7C%20aNode%20%7C%0D%0A%09aNode%20%3A%3D%20self%20parseMethod%3A%20methodSource.%0D%0A%09self%20triggerEvent%3A%20%23aboutToCompile%3A%20with%3A%20aNode.%0D%0A%09%5Eself%20emmitNode%3A%20aNode"));
smalltalk.bind(smalltalk.Compiler,"compile:forClass:",0
,function(methodSource,aClass){var self=this;
self.currentClass_(aClass);
return self.compile_(methodSource);}
,"compiling",unescape("compile%3A%20methodSource%20forClass%3A%20aClass%0D%0A%09%22%20Return%20the%20native%20code%20result%20of%20compiling%20methodSource%20as%20a%20method%20implemented%20by%20aClass.%20%22%0D%0A%0D%0A%09self%20currentClass%3A%20aClass.%0D%0A%09%5Eself%20compile%3A%20methodSource%20"));
smalltalk.bind(smalltalk.Compiler,"compile:forClass:category:",0
,function(methodSource,aClass,category){var self=this;
var method=nil;
if ((nil.isNil_(methodSource)).or_((function(){return methodSource.isEmpty();}))){return nil;};;
method=self.load_forClass_(methodSource,aClass);
method.category_(category);
aClass.addCompiledMethod_(method);
return method;}
,"compiling",unescape("compile%3A%20methodSource%20forClass%3A%20aClass%20category%3A%20category%0D%0A%09%22%20Return%20the%20compiled%20method%20result%20of%20installing%20a%20new%20method%20compiled%20from%20methodSource%20in%20aClass.%20%22%0D%0A%0D%0A%09%7C%20method%20%7C%0D%0A%09%28methodSource%20isNil%20or%3A%20%5B%20methodSource%20isEmpty%20%5D%29%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09method%20%3A%3D%20self%20load%3A%20methodSource%20forClass%3A%20aClass.%0D%0A%09method%20category%3A%20category.%0D%0A%09aClass%20addCompiledMethod%3A%20method.%0D%0A%09%5Emethod"));
smalltalk.bind(smalltalk.Compiler,"selector",0
,function(){var self=this;
return self["@selector"];}
,"accessing",unescape("selector%0D%0A%09%22%20Return%20the%20selector%20of%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Eselector%20"));
smalltalk.bind(smalltalk.Compiler,"currentClass",0
,function(){var self=this;
return self["@currentClass"];}
,"accessing",unescape("currentClass%0D%0A%09%22%20Return%20the%20current%20class%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EcurrentClass"));
smalltalk.bind(smalltalk.Compiler,"currentClass:",0
,function(aClass){var self=this;
self["@currentClass"]=aClass;
return self;}
,"accessing",unescape("currentClass%3A%20aClass%0D%0A%09%22%20Set%20the%20current%20class%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09currentClass%20%3A%3D%20aClass"));
smalltalk.bind(smalltalk.Compiler,"emmitNode:",0
,function(aNode){var self=this;
self.startCompiling_(aNode);
self.prepare_(aNode);
self.triggerEvent_with_("aboutToEncode:",aNode);
self.encoder().visit_(aNode);
self.endCompiling_(aNode);
return self.encoder().emmitedCode();}
,"private",unescape("emmitNode%3A%20aNode%0D%0A%09%22%20Private%20-%20Return%20the%20native%20code%20for%20aNode.%20%22%0D%0A%0D%0A%09self%20startCompiling%3A%20aNode.%0D%0A%09self%20prepare%3A%20aNode.%0D%0A%09self%20triggerEvent%3A%20%23aboutToEncode%3A%20with%3A%20aNode.%0D%0A%09self%20encoder%20visit%3A%20aNode.%0D%0A%09self%20endCompiling%3A%20aNode.%0D%0A%09%5Eself%20encoder%20emmitedCode"));
smalltalk.bind(smalltalk.Compiler,"encoder",0
,function(){var self=this;
if ((nil.isNil_(self["@encoder"]))){self.initializeEncoder();};;
return self["@encoder"];}
,"accessing",unescape("encoder%0D%0A%09%22%20Return%20the%20encoder%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09encoder%20isNil%20ifTrue%3A%20%5B%20self%20initializeEncoder%20%5D.%0D%0A%09%5Eencoder"));
smalltalk.bind(smalltalk.Compiler,"encoder:",0
,function(anEncoder){var self=this;
self["@encoder"]=anEncoder;
return self;}
,"accessing",unescape("encoder%3A%20anEncoder%0D%0A%09%22%20Set%20the%20encoder%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09encoder%20%3A%3D%20anEncoder"));
smalltalk.bind(smalltalk.Compiler,"endCompiling:",0
,function(aMethodNode){var self=this;
self.encoder().endCompiling_(aMethodNode);
self.triggerEvent_with_("endCompiling:",aMethodNode);
return self;}
,"private",unescape("endCompiling%3A%20aMethodNode%0D%0A%09%22%20Private%20-%20Finish%20emmiting%20code%20for%20aMethodNode.%20%22%0D%0A%0D%0A%09self%20encoder%20endCompiling%3A%20aMethodNode.%0D%0A%09self%20triggerEvent%3A%20%23endCompiling%3A%20with%3A%20aMethodNode."));
smalltalk.bind(smalltalk.Compiler,"eval:",0
,function(nativeExpression){var self=this;
return smalltalk.Smalltalk.current().eval_(nativeExpression);}
,"evaluation",unescape("eval%3A%20nativeExpression%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20a%20native%20expression%20in%20global%20context.%20%22%0D%0A%0D%0A%09%5ESmalltalk%20current%20eval%3A%20nativeExpression"));
smalltalk.bind(smalltalk.Compiler,"eval:as:",0
,function(nativeExpression,aState){var self=this;
return smalltalk.Smalltalk.current().eval_as_(nativeExpression,aState);}
,"evaluation",unescape("eval%3A%20nativeExpression%20as%3A%20aState%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20a%20native%20expression%20in%20global%20context.%0D%0A%09The%20state%20of%20the%20system%20change%20to%20state%20while%20evaluating%20the%20expression.%0D%0A%09%22%0D%0A%0D%0A%09%5ESmalltalk%20current%20eval%3A%20nativeExpression%20as%3A%20aState"));
smalltalk.bind(smalltalk.Compiler,"evaluate:",0
,function(smalltalkExpression){var self=this;
return self.evaluate_in_to_(smalltalkExpression,nil.$klass,nil);}
,"doIt",unescape("evaluate%3A%20smalltalkExpression%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20smalltalk%20expression%20in%20global%20context.%20%22%0D%0A%0D%0A%09%5Eself%20evaluate%3A%20smalltalkExpression%20in%3A%20nil%20class%20to%3A%20nil"));
smalltalk.bind(smalltalk.Compiler,"compiledMethodToEvaluate:in:",0
,function(smalltalkExpression,aClass){var self=this;
var result=nil;
var doIt=nil;
var node=nil;
var code=nil;
doIt=smalltalk.String.doItSelector();
self.currentClass_(aClass);
node=self.parseMethod_(doIt._comma(unescape("%20%5E%5B"))._comma(smalltalkExpression)._comma(unescape("%5D%20value")));
code=self.emmitNode_(node);
result=self.eval_(code);
result.category_(doIt);
aClass.addCompiledMethod_(result);
return result;}
,"evaluation",unescape("compiledMethodToEvaluate%3A%20smalltalkExpression%20in%3A%20aClass%0D%0A%09%22%20Return%20a%20compiled%20method%20to%20evaluate%20the%20smalltalk%20expression%20in%20aClass.%20%22%0D%0A%0D%0A%09%7C%20result%20doIt%20node%20code%20%7C%0D%0A%09doIt%20%3A%3D%20String%20doItSelector.%0D%0A%09self%20currentClass%3A%20aClass.%0D%0A%09node%20%3A%3D%20self%20parseMethod%3A%20doIt%2C%27%20%5E%5B%27%2C%20smalltalkExpression%2C%20%27%5D%20value%27.%0D%0A%09code%20%3A%3D%20self%20emmitNode%3A%20node.%0D%0A%09result%20%3A%3D%20self%20eval%3A%20code.%0D%0A%09result%20category%3A%20doIt.%0D%0A%09aClass%20addCompiledMethod%3A%20result.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Compiler,"evaluate:in:to:",0
,function(smalltalkExpression,aClass,aReceiver){var self=this;
var result=nil;
var cm=nil;
cm=self.compiledMethodToEvaluate_in_(smalltalkExpression,aClass);
(function(){return result=aReceiver.perform_(cm.selector());}).ensure_((function(){return cm.removeFromSystem();}));
return result;}
,"evaluation",unescape("evaluate%3A%20smalltalkExpression%20in%3A%20aClass%20to%3A%20aReceiver%0D%0A%09%22%20Return%20the%20result%20of%20evaluating%20the%20smalltalk%20expression%20with%20aReceiver.%20%22%0D%0A%0D%0A%09%7C%20result%20cm%20%7C%0D%0A%09cm%20%3A%3D%20self%20compiledMethodToEvaluate%3A%20smalltalkExpression%20in%3A%20aClass.%0D%0A%09%5B%20result%20%3A%3D%20aReceiver%20perform%3A%20cm%20selector.%0D%0A%09%5D%20ensure%3A%20%5B%20cm%20removeFromSystem%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Compiler,"initialize",0
,function(){var self=this;
smalltalk.superImplementor(smalltalk.Compiler,"initialize").apply(self, []);
self.resetVariables();
return self;}
,"initialize",unescape("initialize%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20initialize.%0D%0A%09self%20resetVariables."));
smalltalk.bind(smalltalk.Compiler,"initializeEncoder",0
,function(){var self=this;
self["@encoder"]=self.triggerEvent_ifNotHandled_("needsEncoder",(function(){return smalltalk.S8Encoder.for_(self);}));
return self;}
,"private",unescape("initializeEncoder%0D%0A%09%22%20Private%20-%20Initialize%20the%20encoder%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09encoder%20%3A%3D%20self%0D%0A%09%09triggerEvent%3A%20%23needsEncoder%0D%0A%09%09ifNotHandled%3A%20%5B%20S8Encoder%20for%3A%20self%20%5D"));
smalltalk.bind(smalltalk.Compiler,"knownVariables",0
,function(){var self=this;
return self.pseudoVariables()._comma(self.tempVariables());}
,"variables",unescape("knownVariables%0D%0A%09%22%20Return%20the%20knownVariables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20pseudoVariables%20%2C%20self%20tempVariables"));
smalltalk.bind(smalltalk.Compiler,"load:forClass:",0
,function(aString,aClass){var self=this;
var nativeCode=nil;
nativeCode=self.compile_forClass_(aString,aClass);
return self.eval_(nativeCode);}
,"compiling",unescape("load%3A%20aString%20forClass%3A%20aClass%0D%0A%09%22%20Return%20the%20CompiledMethod%20result%20of%20compiling%20aString%20as%20a%20method%20implemented%20in%20aClass.%20%22%0D%0A%0D%0A%09%7C%20nativeCode%20%7C%0D%0A%09nativeCode%20%3A%3D%20self%20compile%3A%20aString%20forClass%3A%20aClass.%0D%0A%09%5Eself%20eval%3A%20nativeCode"));
smalltalk.bind(smalltalk.Compiler,"localName:",0
,function(aName){var self=this;
return self.encoder().localName_(aName);}
,"private",unescape("localName%3A%20aName%0D%0A%09%22%20Private%20-%20Return%20a%20valid%20%28native%2C%20local%29%20name%20for%20aName.%20%22%0D%0A%0D%0A%09%5Eself%20encoder%20localName%3A%20aName"));
smalltalk.bind(smalltalk.Compiler,"alienName:",0
,function(aName){var self=this;
return self.encoder().alienName_(aName);}
,"private",unescape("alienName%3A%20aName%0D%0A%09%22%20Private%20-%20Return%20a%20valid%20native%20name%20for%20aName.%20%22%0D%0A%0D%0A%09%5Eself%20encoder%20alienName%3A%20aName"));
smalltalk.bind(smalltalk.Compiler,"nativeLanguage",0
,function(){var self=this;
return self.encoder().nativeLanguage();}
,"accessing",unescape("nativeLanguage%0D%0A%0D%0A%09%5Eself%20encoder%20nativeLanguage"));
smalltalk.bind(smalltalk.Compiler,"nodeFromTemplateIn:",0
,function(aString){var $s8Ret$={name:"stReturn"};try{var self=this;
var src=nil;
var driver=nil;
var msg=nil;
var count=nil;
var found=nil;
if (aString.isEmpty().or_((function(){return aString.last()._eq(unescape("%22")).not();}))){return nil;};;
src=aString.upToLast_(unescape("%22")).fromLast_(unescape("%22")).stream();
if (src.peek()._eq(unescape("%23"))){src.next();} else{(("template"._eq(src.nextLine().upTo_(" "))).mustBeBoolean()==true ? nil : (function(){return (function(){$s8Ret$.result=(function(){return nil})();throw($s8Ret$)})();})());};;
driver=src.upTo_(" ");
if (driver._eq("self")){driver=self;};;
if (driver._eq("class")){driver=self.currentClass();};;
if (driver._eq("encoder")){driver=self.encoder();};;
if (driver._eq("api")){driver="APIBuilder";};;
if (driver.isString()){driver=smalltalk.Smalltalk.at_ifAbsent_(driver,(function(){return nil;}));};;
msg=src.upTo_(" ");
if (!(smalltalk.responds_to_(driver,(msg)))){(function(){$s8Ret$.result=(function(){return nil})();throw($s8Ret$)})();};;
count=msg.occurrencesOf_(":");
found=(function(mth){return (((nil.isNil_(mth)===false)).mustBeBoolean()==true ? (function(){return (function($rec){$rec.source_(aString);return $rec.yourself();})(mth);})() : nil);});
if (count._eq((0))){(function(){$s8Ret$.result=(function(){return found.value_(driver.perform_(msg))})();throw($s8Ret$)})();};;
if (count._eq((1))){(function(){$s8Ret$.result=(function(){return found.value_(driver.perform_with_(msg,src.upToEnd()))})();throw($s8Ret$)})();};;
(function(){$s8Ret$.result=(function(){return nil})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"templates",unescape("nodeFromTemplateIn%3A%20aString%0D%0A%09%22%20Return%20the%20nodes%20syntetized%20from%20inlined%20templates%20in%20aString%20%28or%20nil%29.%0D%0A%09Inlined%20templates%20are%20detected%20from%20last%20comment%20in%20source.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20src%20driver%20msg%20count%20found%20%7C%0D%0A%09%28aString%20isEmpty%20or%3A%20%5B%28aString%20last%20%3D%20%24%22%29%20not%5D%29%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09src%20%3A%3D%20%28%28aString%20upToLast%3A%20%24%22%29%20fromLast%3A%20%24%22%29%20stream.%0D%0A%09src%20peek%20%3D%20%24%23%20ifTrue%3A%20%5B%20src%20next%20%5D%20ifFalse%3A%20%5B%0D%0A%09%09%23template%20%3D%20%28src%20nextLine%20upTo%3A%20%24%20%29%20ifFalse%3A%20%5B%20%5Enil%20%5D%0D%0A%09%5D.%0D%0A%09driver%20%3A%3D%20src%20upTo%3A%20%24%20.%0D%0A%09driver%20%3D%20%23self%20%20ifTrue%3A%20%5B%20driver%20%3A%3D%20self%20%5D.%0D%0A%09driver%20%3D%20%23class%20ifTrue%3A%20%5B%20driver%20%3A%3D%20self%20currentClass%20%5D.%0D%0A%09driver%20%3D%20%23encoder%20ifTrue%3A%20%5B%20driver%20%3A%3D%20self%20encoder%20%5D.%0D%0A%09driver%20%3D%20%23api%20%20%20ifTrue%3A%20%5B%20driver%20%3A%3D%20%23APIBuilder%20%5D.%0D%0A%09driver%20isString%20ifTrue%3A%20%5B%20driver%20%3A%3D%20Smalltalk%20at%3A%20driver%20ifAbsent%3A%20%5B%5D%20%5D.%0D%0A%09msg%20%3A%3D%20src%20upTo%3A%20%24%20.%0D%0A%09%28driver%20respondsTo%3A%20msg%29%20ifFalse%3A%20%5B%20%5Enil%20%5D.%0D%0A%09count%20%3A%3D%20msg%20occurrencesOf%3A%20%24%3A.%0D%0A%09found%20%3A%3D%20%5B%3Amth%7C%20mth%20notNil%20ifTrue%3A%20%5B%20mth%20source%3A%20aString%3B%20yourself%20%5D%20%5D.%0D%0A%09count%20%3D%200%20ifTrue%3A%20%5B%20%5Efound%20value%3A%20%28driver%20perform%3A%20msg%29%20%5D.%0D%0A%09count%20%3D%201%20ifTrue%3A%20%5B%20%5Efound%20value%3A%20%28driver%20perform%3A%20msg%20with%3A%20src%20upToEnd%29%20%5D.%0D%0A%09%5Enil"));
smalltalk.bind(smalltalk.Compiler,"parser",0
,function(){var self=this;
return self.triggerEvent_ifNotHandled_("needsParser",(function(){return smalltalk.SmalltalkParser.$$new();}));}
,"private",unescape("parser%0D%0A%09%22%20Private%20-%20Return%20the%20parser%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%09triggerEvent%3A%20%23needsParser%0D%0A%09%09ifNotHandled%3A%20%5B%20SmalltalkParser%20new%20%5D"));
smalltalk.bind(smalltalk.Compiler,"prepare:",0
,function(aMethodNode){var self=this;
self.encoder().prepare_(aMethodNode);
self.transformers().do_((function(each){return each.visit_(aMethodNode);}));
return self;}
,"prepare",unescape("prepare%3A%20aMethodNode%0D%0A%09%22%20Private%20-%20Apply%20tranformations/optimization%20to%20aMethodNode.%20%22%0D%0A%0D%0A%09self%20encoder%20prepare%3A%20aMethodNode.%0D%0A%09self%20transformers%20do%3A%20%5B%3Aeach%7C%20each%20visit%3A%20aMethodNode%20%5D."));
smalltalk.bind(smalltalk.Compiler,"pseudoVariables",0
,function(){var self=this;
return ["self", "super", "true", "false", "nil", "thisContext"];}
,"variables",unescape("pseudoVariables%0D%0A%09%22%20Return%20the%20pseudoVariables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%20%23self%20%23super%20%23true%20%23false%20%23nil%20%23thisContext%20%29"));
smalltalk.bind(smalltalk.Compiler,"recompile:",0
,function(aClass){var self=this;
self.triggerEvent_with_("aboutToRecompile:",aClass);
aClass.methodDictionary().do_((function(each){return self.compile_forClass_category_(each.source(),aClass,each.category());}));
self.triggerEvent_with_("recompiled:",aClass);
if (!(aClass.isMetaclass())){self.recompile_(aClass.$klass);};;
return self;}
,"compiling",unescape("recompile%3A%20aClass%0D%0A%09%22%20Re-compile%20all%20the%20methods%20defined%20by%20aClass.%20%22%0D%0A%0D%0A%09self%20triggerEvent%3A%20%23aboutToRecompile%3A%20with%3A%20aClass.%0D%0A%09aClass%20methodDictionary%20do%3A%20%5B%3Aeach%20%7C%0D%0A%22Debug%09%09self%20note%3A%20each.%22%0D%0A%09%09self%20compile%3A%20each%20source%20forClass%3A%20aClass%20category%3A%20each%20category%0D%0A%09%5D.%0D%0A%09self%20triggerEvent%3A%20%23recompiled%3A%20with%3A%20aClass.%0D%0A%09aClass%20isMetaclass%20ifFalse%3A%20%5B%20self%20recompile%3A%20aClass%20class%20%5D."));
smalltalk.bind(smalltalk.Compiler,"resetVariables",0
,function(){var self=this;
self["@unknownVariables"]=[];
self["@tempVariables"]=[];
return self;}
,"variables",unescape("resetVariables%0D%0A%09%22%20Reset%20the%20variables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09unknownVariables%20%3A%3D%20%23%28%29.%0D%0A%09tempVariables%20%3A%3D%20%23%28%29."));
smalltalk.bind(smalltalk.Compiler,"smalltalkNameFor:",0
,function(aClass){var self=this;
return self.encoder().smalltalkNameFor_(aClass);}
,"names",unescape("smalltalkNameFor%3A%20aClass%0D%0A%09%22%20Return%20the%20global%20name%20of%20aClass%20%28can%20be%20nil%29.%20%22%0D%0A%0D%0A%09%5Eself%20encoder%20smalltalkNameFor%3A%20aClass"));
smalltalk.bind(smalltalk.Compiler,"startCompiling:",0
,function(aNode){var self=this;
if (!(aNode.isParseFailure())){self["@selector"]=aNode.selector();};;
self.triggerEvent_with_("startCompiling:",aNode);
self.encoder().startCompiling_(aNode);
return self;}
,"private",unescape("startCompiling%3A%20aNode%0D%0A%09%22%20Private%20-%20Set%20the%20receiver%20to%20start%20emmiting%20code%20for%20aNode.%20%22%0D%0A%0D%0A%09aNode%20isParseFailure%20ifFalse%3A%20%5B%0D%0A%09%09selector%20%3A%3D%20aNode%20selector%0D%0A%09%5D.%0D%0A%09self%20triggerEvent%3A%20%23startCompiling%3A%20with%3A%20aNode.%0D%0A%09self%20encoder%20startCompiling%3A%20aNode."));
smalltalk.bind(smalltalk.Compiler,"tempVariables",0
,function(){var self=this;
return self["@tempVariables"];}
,"variables",unescape("tempVariables%0D%0A%09%22%20Return%20the%20tempVariables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EtempVariables"));
smalltalk.bind(smalltalk.Compiler,"transformers",0
,function(){var self=this;
if ((nil.isNil_(self["@transformers"]))){self["@transformers"]=smalltalk.Array.$$new();
self.triggerEvent_with_("setTransformers:",self["@transformers"]);};;
return self["@transformers"];}
,"accessing",unescape("transformers%0D%0A%09%22%20Return%20the%20node%20transformers%20installed%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09transformers%20isNil%20ifTrue%3A%20%5B%0D%0A%09%09transformers%20%3A%3D%20Array%20new.%0D%0A%09%09self%20triggerEvent%3A%20%23setTransformers%3A%20with%3A%20transformers.%0D%0A%09%5D.%0D%0A%09%5Etransformers"));
smalltalk.bind(smalltalk.Compiler,"unknownVariables",0
,function(){var self=this;
return self["@unknownVariables"];}
,"variables",unescape("unknownVariables%0D%0A%09%22%20Return%20the%20unknownVariables%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EunknownVariables"));
smalltalk.bind(smalltalk.Compiler,"useTemplates",0
,function(){var self=this;
return (nil.isNil_(self["@useTemplates"])===false).and_((function(){return self["@useTemplates"];}));}
,"accessing",unescape("useTemplates%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20use%20templates.%20%22%0D%0A%0D%0A%09%5EuseTemplates%20notNil%20and%3A%20%5B%20useTemplates%20%5D"));
smalltalk.bind(smalltalk.Compiler,"useTemplates:",0
,function(aBoolean){var self=this;
self["@useTemplates"]=aBoolean;
return self;}
,"accessing",unescape("useTemplates%3A%20aBoolean%0D%0A%09%22%20Set%20the%20receiver%20to%20consider%20inlined%20templates.%20%22%0D%0A%0D%0A%09useTemplates%20%3A%3D%20aBoolean"));
smalltalk.bind(smalltalk.SourceCodeParser.$klass,"parse:",0
,function(aStream){var self=this;
return self.$$new().parse_(aStream);}
,"instantiation",unescape("parse%3A%20aStream%0D%0A%09%22%20Return%20the%20result%20of%20parsing%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%5Eself%20new%20parse%3A%20aStream%20"));
smalltalk.bind(smalltalk.SourceCodeParser.$klass,"debug:",0
,function(aStream){var self=this;
return smalltalk.PPParser.debugWhile_((function(){return self.parse_(aStream);}));}
,"debug",unescape("debug%3A%20aStream%0D%0A%09%5EPPParser%20debugWhile%3A%20%5B%0D%0A%09%09self%20parse%3A%20aStream%0D%0A%09%5D"));
smalltalk.bind(smalltalk.SourceCodeParser,"parse:",0
,function(aStream){var self=this;
return self.parser().parse_(aStream);}
,"parsing",unescape("parse%3A%20aStream%0D%0A%09%22%20Return%20the%20result%20of%20parsing%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%5Eself%20parser%20parse%3A%20aStream"));
smalltalk.bind(smalltalk.SourceCodeParser,"gc",0
,function(){var self=this;
if (smalltalk.Smalltalk.includesKey_("JSC")){smalltalk.Smalltalk.gc();};;
return self;}
,"gc",unescape("gc%0D%0A%09%22%20Private%20-%20Perform%20%28preventive%29%20garbage%20collection.%0D%0A%09Note%3A%20JavascriptCore%20VM%20can%20crash%20reporting%0D%0A%09%20%27memory%20issues%27%20if%20we%20do%20not%20GC%20asap.%0D%0A%09%20This%20issue%20will%20persist%20while%20Apple%20continue%0D%0A%09%20%20distributing%20a%20version%20of%20JSC%20with%20issue%20reported%0D%0A%09%09swiki%3AU8/232%0D%0A%09%09https%3A//bugs.webkit.org/show_bug.cgi%3Fid%3D160027%0D%0A%09%22%0D%0A%0D%0A%09%28Smalltalk%20includesKey%3A%20%23JSC%29%20ifTrue%3A%20%5B%20Smalltalk%20gc%20%5D."));
smalltalk.bind(smalltalk.SourceCodeParser,"separator",0
,function(){var self=this;
return smalltalk.PPCharacterParser.separator();}
,"parsers",unescape("separator%0D%0A%09%22%20Return%20the%20parser%20for%20separator.%20%22%0D%0A%0D%0A%09%5EPPCharacterParser%20separator"));
smalltalk.bind(smalltalk.SourceCodeParser,"parser",0
,function(){var self=this;
return self.implementedBySubclass_("parser");}
,"parsers",unescape("parser%0D%0A%09%22%20Return%20the%20method%20parser.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23parser"));
smalltalk.bind(smalltalk.SourceCodeParser,"charParser:",0
,function(options){var self=this;
return smalltalk.PPCharacterParser.for_doing_(unescape("a-zA-Z0-9")._comma(options),(function(c){return c.isAlphaNumeric().or_((function(){return options.includes_(c);}));}));}
,"parsers",unescape("charParser%3A%20options%0D%0A%09%22%20Private%20-%20Return%20the%20character%20parser.%20%22%0D%0A%0D%0A%09%5EPPCharacterParser%20for%3A%20%27a-zA-Z0-9%27%2Coptions%20doing%3A%20%5B%3Ac%7C%0D%0A%09%09c%20isAlphaNumeric%20or%3A%20%5B%20options%20includes%3A%20c%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.SourceCodeParser,"charParserOr:",0
,function(aCharacter){var self=this;
return smalltalk.PPCharacterParser.for_doing_(unescape("a-zA-Z0-9")._comma(aCharacter),(function(c){return c.isAlphaNumeric().or_((function(){return c._eq(aCharacter);}));}));}
,"parsers",unescape("charParserOr%3A%20aCharacter%0D%0A%09%22%20Private%20-%20Return%20the%20character%20parser.%20%22%0D%0A%0D%0A%09%5EPPCharacterParser%20for%3A%20%27a-zA-Z0-9%27%2CaCharacter%20doing%3A%20%5B%3Ac%7C%0D%0A%09%09c%20isAlphaNumeric%20or%3A%20%5B%20c%20%3D%20aCharacter%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.SourceCodeParser,",","_comma"
,function(anObject){var self=this;
return smalltalk.PPParser._comma(anObject);}
,"parsers",unescape("%2C%20anObject%0D%0A%09%22%20Private%20-%20Return%20the%20parser%20to%20start%20parsing%20anObject.%20%22%0D%0A%0D%0A%09%5EPPParser%20%2C%20anObject"));
smalltalk.bind(smalltalk.SourceCodeParser,"optional:",0
,function(anObject){var self=this;
return smalltalk.PPParser.optional_(anObject);}
,"parsers",unescape("optional%3A%20anObject%0D%0A%09%22%20Private%20-%20Return%20the%20parser%20to%20parse%20an%20optional%20element%20anObject.%20%22%0D%0A%0D%0A%09%5EPPParser%20optional%3A%20anObject"));
smalltalk.bind(smalltalk.SourceCodeParser,"gc:",0
,function(anObject){var self=this;
return self._comma(anObject)._eq_eq_gt((function(node){self.gc();return node;}));}
,"parsers",unescape("gc%3A%20anObject%0D%0A%09%22%20Private%20-%20Return%20the%20parser%20to%20parse%20anObject.%20%22%0D%0A%0D%0A%09%5E%28self%20%2CanObject%29%20%3D%3D%3E%20%5B%3Anode%7C%20self%20gc.%20node%20%5D"));
smalltalk.bind(smalltalk.SmalltalkParser.$klass,"constructEventsTriggered",0
,function(){var self=this;
return ["needsDirectiveParser:ws:"]._comma(smalltalk.superImplementor(smalltalk.SmalltalkParser.$klass,"constructEventsTriggered").apply(self, []));}
,"events",unescape("constructEventsTriggered%0D%0A%09%22%20Private%20-%20Return%20the%20events%20managed%20by%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%09%23needsDirectiveParser%3Aws%3A%0D%0A%09%29%20%2C%20super%20constructEventsTriggered"));
smalltalk.bind(smalltalk.SmalltalkParser,"missingDirective:ws:",0
,function(number,ws){var self=this;
return smalltalk.PPCharacterParser.notIn_(unescape("%3E")).star().flatten()._comma(unescape("%3E"))._at_at("missingDirective")._eq_eq_gt((function(node){return self.error_(unescape("Invalid%20directive%20-%20")._comma(node.first()));}));}
,"directives",unescape("missingDirective%3A%20number%20ws%3A%20ws%0D%0A%09%22%20Private%20-%20Return%20the%20parser%20for%20missing%20support%20of%20directives.%20%22%0D%0A%0D%0A%09%5E%28PPCharacterParser%20notIn%3A%20%24%3E%29%20star%20flatten%0D%0A%09%2C%20%24%3E%20@@%23missingDirective%0D%0A%09%3D%3D%3E%20%5B%3Anode%7C%20self%20error%3A%20%27Invalid%20directive%20-%20%27%20%2Cnode%20first%20%5D"));
smalltalk.bind(smalltalk.SmalltalkParser,"jsDirective:ws:",0
,function(number,ws){var self=this;
return self._comma("js:")._slash("javascript:")._comma(smalltalk.PPInlineParser.upTo_(unescape("%3A%3E")))._at_at("jsDirective")._eq_eq_gt((function(node){return smalltalk.JSDirectiveNode.code_protected_(node.second(),node.first()._eq("javascript:"));}));}
,"directives",unescape("jsDirective%3A%20number%20ws%3A%20ws%0D%0A%09%22%20Private%20-%20Return%20the%20parser%20for%20native%20directives.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%09%2C%20%23js%3A%20/%20%23javascript%3A%0D%0A%09%09%2C%20%28PPInlineParser%20upTo%3A%20%27%3A%3E%27%29%0D%0A%09%09@@%23jsDirective%0D%0A%09%3D%3D%3E%20%5B%3Anode%7C%20JSDirectiveNode%0D%0A%09%09code%3A%20node%20second%0D%0A%09%09protected%3A%20node%20first%20%3D%20%23javascript%3A%0D%0A%09%5D"));
smalltalk.bind(smalltalk.SmalltalkParser,"optDirective:ws:",0
,function(number,ws){var self=this;
return self._comma("opt:")._slash(unescape("opt+%3A"))._slash(unescape("opt-%3A"))._comma(smalltalk.PPInlineParser.upTo_(unescape("%3A%3E")))._at_at("optDirective")._eq_eq_gt((function(node){return (function($rec){$rec.args_(node.second().asArrayOfSubstrings());$rec.remove_(node.first()._eq(unescape("opt-%3A")));return $rec.yourself();})(smalltalk.InlineOptimizationNode.$$new());}));}
,"directives",unescape("optDirective%3A%20number%20ws%3A%20ws%0D%0A%09%22%20Private%20-%20Return%20the%20parser%20for%20compiler%20optimizations%20directive.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%09%2C%20%23opt%3A%20/%20%23opt+%3A%20/%20%23opt-%3A%0D%0A%09%09%2C%28PPInlineParser%20upTo%3A%20%27%3A%3E%27%29%0D%0A%09%09@@%23optDirective%0D%0A%09%3D%3D%3E%20%5B%3Anode%7C%20InlineOptimizationNode%20new%0D%0A%09%09args%3A%20node%20second%20asArrayOfSubstrings%3B%0D%0A%09%09remove%3A%20node%20first%20%3D%20%23opt-%3A%3B%0D%0A%09%09yourself%0D%0A%09%5D"));
smalltalk.bind(smalltalk.SmalltalkParser,"defaultDirective:ws:",0
,function(number,ws){var self=this;
return self.optDirective_ws_(number,ws)._slash(self.jsDirective_ws_(number,ws))._slash(self.missingDirective_ws_(number,ws));}
,"directives",unescape("defaultDirective%3A%20number%20ws%3A%20ws%0D%0A%09%22%20Private%20-%20Return%20the%20parser%20for%20default%20support%20of%20directives.%0D%0A%09We%20support%20inlined%20native%20directives.%0D%0A%09%22%0D%0A%0D%0A%09%5E%20%28self%20optDirective%3A%20number%20ws%3A%20ws%29%0D%0A%09/%20%28self%20jsDirective%3A%20number%20ws%3A%20ws%29%0D%0A%09/%20%28self%20missingDirective%3A%20number%20ws%3A%20ws%29"));
smalltalk.bind(smalltalk.SmalltalkParser,"directive:ws:",0
,function(number,ws){var self=this;
var parser=nil;
parser=self.triggerEvent_withArguments_ifNotHandled_("needsDirectiveParser:ws:",smalltalk.Array.with_with_(number,ws),(function(){return self.defaultDirective_ws_(number,ws);}));
return self._comma(unescape("%3C"))._comma(ws._comma(parser)._comma(ws))._at_at("directive")._eq_eq_gt((function(node){return node.second().second();}));}
,"directives",unescape("directive%3A%20number%20ws%3A%20ws%0D%0A%09%22%20Private%20-%20Return%20the%20parser%20for%20directives.%20%22%0D%0A%0D%0A%09%7C%20parser%20%7C%0D%0A%09parser%20%3A%3D%20self%0D%0A%09%09triggerEvent%3A%20%23needsDirectiveParser%3Aws%3A%0D%0A%09%09withArguments%3A%20%28Array%20with%3A%20number%20with%3A%20ws%29%0D%0A%09%09ifNotHandled%3A%20%5B%20self%20defaultDirective%3A%20number%20ws%3A%20ws%20%5D.%0D%0A%0D%0A%09%5Eself%09%2C%20%24%3C%20%2C%28ws%2C%20parser%2C%20ws%29%20@@%23directive%0D%0A%09%09%3D%3D%3E%20%5B%3Anode%20%7C%20node%20second%20second%20%5D"));
smalltalk.bind(smalltalk.SmalltalkParser,"binarySelectorChars",0
,function(){var self=this;
return smalltalk.String.binarySelectorChars();}
,"constants",unescape("binarySelectorChars%0D%0A%09%22%20Return%20the%20binary%20selector%20characters%20mask.%20%22%0D%0A%0D%0A%09%5EString%20binarySelectorChars%20"));
smalltalk.bind(smalltalk.SmalltalkParser,"binaryCharParser",0
,function(){var self=this;
return smalltalk.PPCharacterParser.in_(self.binarySelectorChars())._at_at("binaryChar");}
,"parsers",unescape("binaryCharParser%0D%0A%09%22%20Return%20the%20binary%20selector%20character%20parser.%20%22%0D%0A%0D%0A%09%5E%28PPCharacterParser%20in%3A%20self%20binarySelectorChars%29%20@@%23binaryChar"));
smalltalk.bind(smalltalk.SmalltalkParser,"symbolCharParser",0
,function(){var self=this;
return self.charParserOr_("_")._at_at("symbolChar");}
,"parsers",unescape("symbolCharParser%0D%0A%09%22%20Return%20the%20symbol%20character%20parser.%20%22%0D%0A%0D%0A%09%5E%28self%20charParserOr%3A%20%24_%29%20@@%23symbolChar"));
smalltalk.bind(smalltalk.SmalltalkParser,"jsSymbolCharParser",0
,function(){var self=this;
return self.charParser_("_$")._at_at("jsSymbolChar");}
,"parsers",unescape("jsSymbolCharParser%0D%0A%09%22%20Private%20-%20Return%20the%20js%20symbol%20character%20parser.%20%22%0D%0A%0D%0A%09%5E%28self%20charParser%3A%20%27_%24%27%29%20@@%23jsSymbolChar"));
smalltalk.bind(smalltalk.SmalltalkParser,"extendedSymbolCharParser",0
,function(){var self=this;
return self.charParser_("_:"._comma(self.binarySelectorChars()))._at_at("extendedSymbolChar");}
,"parsers",unescape("extendedSymbolCharParser%0D%0A%09%22%20Private%20-%20Return%20the%20extended%20symbol%20character%20parser.%20%22%0D%0A%0D%0A%09%5E%28self%20charParser%3A%20%27_%3A%27%20%2C%20self%20binarySelectorChars%29%20@@%23extendedSymbolChar"));
smalltalk.bind(smalltalk.SmalltalkParser,"globalNameParser",0
,function(){var self=this;
return smalltalk.PPCharacterParser.between_and_("A","Z")._comma(self.symbolCharParser().star()).flatten()._at_at("globalNameParser");}
,"parsers",unescape("globalNameParser%0D%0A%09%22%20Return%20the%20parser%20for%20global%20names.%20%22%0D%0A%0D%0A%09%5E%28%28PPCharacterParser%20between%3A%20%24A%20and%3A%20%24Z%29%0D%0A%09%09%2C%20self%20symbolCharParser%20star%29%0D%0A%09%09flatten%20@@%23globalNameParser"));
smalltalk.bind(smalltalk.SmalltalkParser,"comment",0
,function(){var self=this;
return unescape("%22").asParser()._comma(unescape("%22").asParser().not()._comma(smalltalk.PPAnyParser.$$new()).star())._comma(unescape("%22"))._at_at("stComment").flatten()._at_at("comment");}
,"parsers",unescape("comment%0D%0A%09%22%20Return%20the%20parser%20for%20comment.%20%22%0D%0A%0D%0A%09%5E%28%24%22%20asParser%0D%0A%09%2C%28%24%22%20asParser%20not%2C%20PPAnyParser%20new%29%20star%0D%0A%09%2C%20%24%22%20@@%23stComment%29%20flatten%0D%0A%09@@%23comment"));
smalltalk.bind(smalltalk.SmalltalkParser,"nameStart",0
,function(){var self=this;
return smalltalk.PPCharacterParser.for_doing_(unescape("a-z_"),(function(c){return c.between_and_("a","z").or_((function(){return c._eq("_");}));}))._at_at("nameStart");}
,"parsers",unescape("nameStart%0D%0A%09%22%20Return%20the%20parser%20for%20name%20start.%22%0D%0A%0D%0A%09%5E%28PPCharacterParser%20for%3A%20%27a-z_%27%20doing%3A%20%5B%3Ac%7C%0D%0A%09%09%28c%20between%3A%20%24a%20and%3A%20%24z%29%0D%0A%09%09or%3A%20%5B%20c%20%3D%20%24_%20%5D%0D%0A%09%5D%29%20@@%23nameStart"));
smalltalk.bind(smalltalk.SmalltalkParser,"upNameStart",0
,function(){var self=this;
return smalltalk.PPCharacterParser.for_doing_(unescape("A-Za-z_"),(function(c){return c.isLetter().or_((function(){return c._eq("_");}));}))._at_at("upNameStart");}
,"parsers",unescape("upNameStart%0D%0A%09%22%20Return%20the%20parser%20for%20name%20start%20%28can%20be%20uppercase%29.%22%0D%0A%0D%0A%09%5E%28PPCharacterParser%20for%3A%20%27A-Za-z_%27%20doing%3A%20%5B%3Ac%7C%0D%0A%09%09c%20isLetter%20or%3A%20%5B%20c%20%3D%20%24_%20%5D%0D%0A%09%5D%29%20@@%23upNameStart"));
smalltalk.bind(smalltalk.SmalltalkParser,"digits",0
,function(){var self=this;
return smalltalk.PPCharacterParser.between_and_("0","9").plus();}
,"parsers",unescape("digits%0D%0A%09%22%20Return%20the%20parser%20for%20numeric%20digits.%22%0D%0A%0D%0A%09%5E%28PPCharacterParser%20between%3A%20%240%20and%3A%20%249%29%20plus"));
smalltalk.bind(smalltalk.SmalltalkParser,"decimals",0
,function(){var self=this;
return self._comma(".")._comma(self.digits())._at_at("decimals");}
,"parsers",unescape("decimals%0D%0A%09%22%20Return%20the%20parser%20for%20number%20decimals.%22%0D%0A%0D%0A%09%5Eself%09%2C%20%24.%0D%0A%09%09%2C%20self%20digits%0D%0A%09%09@@%23decimals"));
smalltalk.bind(smalltalk.SmalltalkParser,"exponent",0
,function(){var self=this;
return self._comma("e")._comma(self.optional_(unescape("-")))._comma(self.digits())._at_at("exponent");}
,"parsers",unescape("exponent%0D%0A%09%22%20Return%20the%20parser%20for%20number%20exponent.%22%0D%0A%0D%0A%09%5Eself%09%2C%20%24e%0D%0A%09%09%2C%20%28self%20optional%3A%20%24-%29%0D%0A%09%09%2C%20self%20digits%0D%0A%09%09@@%23exponent"));
smalltalk.bind(smalltalk.SmalltalkParser,"radix",0
,function(){var self=this;
return self._comma("r")._comma(smalltalk.PPCharacterParser.for_doing_(unescape("0-9A-Z"),(function(c){return c.between_and_("0","9").or_((function(){return c.between_and_("A","Z");}));})).plus())._at_at("radix");}
,"parsers",unescape("radix%0D%0A%09%22%20Return%20the%20parser%20for%20number%20radix.%22%0D%0A%0D%0A%09%5Eself%09%2C%20%24r%0D%0A%09%09%2C%20%28PPCharacterParser%20for%3A%20%270-9A-Z%27%20doing%3A%20%5B%3Ac%7C%0D%0A%09%09%09%28c%20between%3A%20%240%20and%3A%20%249%29%0D%0A%09%09%09or%3A%20%5B%20c%20between%3A%20%24A%20and%3A%20%24Z%20%5D%0D%0A%09%09%5D%29%20plus%0D%0A%09%09@@%23radix"));
smalltalk.bind(smalltalk.SmalltalkParser,"number",0
,function(){var self=this;
return self.optional_(unescape("-"))._comma(self.digits())._comma(self.decimals().optional())._comma(self.exponent()._slash(self.radix()).optional())._at_at("number");}
,"parsers",unescape("number%0D%0A%09%22%20Return%20the%20parser%20for%20number.%22%0D%0A%0D%0A%09%5E%28self%20optional%3A%20%24-%29%0D%0A%09%09%2C%20self%20digits%20%2C%20self%20decimals%20optional%0D%0A%09%09%2C%20%28self%20exponent%20/%20self%20radix%29%20optional%0D%0A%09%09@@%23number"));
smalltalk.bind(smalltalk.SmalltalkParser,"annotationStart",0
,function(){var self=this;
return self._comma("::");}
,"private",unescape("annotationStart%0D%0A%09%22%20Private%20-%20Return%20the%20parser%20for%20starting%20annotations.%20%22%0D%0A%0D%0A%09%5Eself%20%2C%23%3A%3A%20"));
smalltalk.bind(smalltalk.SmalltalkParser,"methodParserParts",0
,function(){var self=this;
var expression=nil;
var separator=nil;
var comment=nil;
var ws=nil;
var identifier=nil;
var keyword=nil;
var stSelectorPart=nil;
var string=nil;
var symbol=nil;
var number=nil;
var variable=nil;
var reference=nil;
var globalReference=nil;
var literal=nil;
var returnExpression=nil;
var expressionParser=nil;
var keyword=nil;
var unarySelector=nil;
var binarySelector=nil;
var keywordPattern=nil;
var unaryPattern=nil;
var binaryPattern=nil;
var assignment=nil;
var temps=nil;
var blockParamList=nil;
var block=nil;
var expression=nil;
var expressions=nil;
var subexpression=nil;
var statements=nil;
var sequence=nil;
var operand=nil;
var unaryMessage=nil;
var unarySend=nil;
var unaryTail=nil;
var binaryMessage=nil;
var binarySend=nil;
var binaryTail=nil;
var keywordMessage=nil;
var keywordSend=nil;
var keywordPair=nil;
var cascade=nil;
var message=nil;
var inline=nil;
var jsIndex=nil;
var character=nil;
var directive=nil;
var arrayLiteral=nil;
var arrayVariable=nil;
var literalArray=nil;
var aSymbol=nil;
var stIdentifier=nil;
var jsIdentifier=nil;
var noReference=nil;
var alienReference=nil;
var alienIdentifier=nil;
var emptyArray=nil;
var hashedString=nil;
var annotation=nil;
var argument=nil;
separator=self.separator();
comment=self.comment();
ws=separator._slash(comment)._at_at("sp").star()._at_at("ws");
string=self._comma(unescape("%27"))._comma(smalltalk.PPStStringParser.$$new())._at_at("string")._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(node.second());}));
hashedString=self._comma(unescape("%23")._comma(unescape("%27")))._comma(smalltalk.PPStStringParser.$$new())._at_at("hashedString")._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(node.second());}));
stIdentifier=self.nameStart()._comma(self.symbolCharParser().star())._at_at("stId").flatten()._at_at("stIdentifier");
stSelectorPart=self.upNameStart()._comma(self.symbolCharParser().star())._at_at("stSel").flatten()._at_at("stSelectorPart");
jsIdentifier=self._comma(unescape("%23"))._comma(self.jsSymbolCharParser().plus())._at_at("jsId").flatten()._at_at("jsIdentifier");
alienIdentifier=self.jsSymbolCharParser().plus().flatten()._at_at("alienIdentifier");
identifier=jsIdentifier._slash(stIdentifier)._at_at("identifier");
keyword=stSelectorPart._slash(identifier)._comma(":").flatten()._at_at("keyword");
annotation=self.annotationStart()._comma(string._slash(stIdentifier)._slash(self.globalNameParser()))._at_at("annotation")._eq_eq_gt((function(node){return node.last();}));
argument=identifier._comma(annotation.star())._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(node.first()).annotations_(node.last());}));
symbol=self._comma(unescape("%23"))._comma(self.extendedSymbolCharParser().plus().flatten())._at_at("symbol")._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(node.second());}));
number=self.number().flatten()._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(node.asNumber());}));
character=self._comma("$")._comma(smalltalk.PPAnyParser.$$new())._at_at("character")._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(node.at_((2)));}));
directive=self.directive_ws_(number,ws);
variable=identifier._eq_eq_gt((function(token){return smalltalk.VariableNode.$$new().value_(token);}));
noReference=self._comma(unescape("%23"))._at_at("noReference")._eq_eq_gt((function(token){return smalltalk.NoReceiverNode.$$new();}));
literal=smalltalk.PPDelegateParser.$$new()._at_at("literalDelegate");
expression=smalltalk.PPDelegateParser.$$new()._at_at("expressionDelegate");
arrayVariable=self._comma("true")._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(true);}))._slash(self._comma("false")._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(false);})))._slash(self._comma("nil")._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(nil);})))._at_at("arrayVariable");
aSymbol=self.extendedSymbolCharParser().plus().flatten()._at_at("aSymbol")._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(node);}));
arrayLiteral=literal._slash(arrayVariable)._slash(aSymbol)._slash(variable);
literalArray=self.gc_(unescape("%23%28"))._comma(ws._comma(arrayLiteral)._comma(ws).star())._comma(unescape("%29"))._at_at("literalArray")._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(smalltalk.Array.withAll_(node.second().collect_((function(each){return each.second().value();}))));}));
emptyArray=self._comma(unescape("%23%28"))._comma(ws)._comma(unescape("%29"))._at_at("emptyArray")._eq_eq_gt((function(node){return smalltalk.ValueNode.with_(smalltalk.Array.$$new());}));
globalReference=self.globalNameParser()._at_at("globalReference")._eq_eq_gt((function(token){self.gc();return smalltalk.GlobalReferenceNode.$$new().value_(token);}));
alienReference=self._comma(unescape("%23")._comma(unescape("%7B")))._comma(alienIdentifier)._comma(unescape("%7D"))._at_at("alienReference")._eq_eq_gt((function(node){return smalltalk.AlienNode.$$new().value_(node.second());}));
jsIndex=self.gc_(unescape("%5B"))._comma(expression)._comma(unescape("%5D"))._slash(self._comma(unescape("%23"))._comma(alienIdentifier));
reference=variable._slash(globalReference)._slash(alienReference)._at_at("ref")._comma(jsIndex.star())._comma(annotation.star())._at_at("reference")._eq_eq_gt((function(nodes){return nodes.first().reference_(nodes);}));
binarySelector=self.optional_(unescape("%23"))._comma(self.binaryCharParser().plus()).flatten()._at_at("binarySelector");
unarySelector=stSelectorPart._slash(identifier)._at_at("unarySelector");
keywordPattern=ws._comma(keyword)._at_at("kw")._comma(ws)._comma(argument)._at_at("kwId").plus()._at_at("keywordPattern")._eq_eq_gt((function(nodes){return smalltalk.Array.with_with_(nodes.collect_((function(each){return each.at_((2));})).join_(""),nodes.collect_((function(each){return each.at_((4));})));}));
binaryPattern=ws._comma(binarySelector)._comma(ws)._comma(argument)._at_at("binaryPattern")._eq_eq_gt((function(node){return smalltalk.Array.with_with_(node.second(),smalltalk.Array.with_(node.fourth()));}));
unaryPattern=ws._comma(unarySelector)._at_at("unaryPattern")._eq_eq_gt((function(node){return smalltalk.Array.with_with_(node.second(),smalltalk.Array.$$new());}));
expressions=expression._comma(ws._comma(".")._comma(ws)._comma(expression)._at_at("anExpression")._eq_eq_gt((function(node){return node.fourth();})).star())._at_at("expressions")._eq_eq_gt((function(node){return smalltalk.Array.with_(node.first())._comma(node.second());}));
assignment=reference._comma(ws)._comma(unescape("%3A%3D"))._comma(ws)._comma(expression)._at_at("assignment")._eq_eq_gt((function(node){return (function($rec){$rec.left_(node.first());return $rec.right_(node.at_((5)));})(smalltalk.AssignmentNode.$$new());}));
returnExpression=self.gc_(unescape("%5E"))._at_at("caret")._comma(ws)._comma(expression)._comma(annotation.star())._comma(ws)._comma(".".asParser().optional())._at_at("returnExpression")._eq_eq_gt((function(node){return (function($rec){$rec.addNode_(node.at_((3)));$rec.annotations_(node.at_((4)));return $rec.yourself();})(smalltalk.ReturnNode.$$new());}));
temps=self._comma(unescape("%7C"))._comma(ws._comma(argument).star())._comma(ws)._comma(unescape("%7C"))._at_at("temps")._eq_eq_gt((function(node){return node.second().collect_((function(each){return each.second();}));}));
blockParamList=self._comma(":")._comma(ws)._comma(argument)._comma(ws).plus()._comma(unescape("%7C"))._at_at("blockParamList")._eq_eq_gt((function(node){return node.first().collect_((function(each){return each.third();}));}));
subexpression=self.gc_(unescape("%28"))._comma(ws)._comma(expression)._comma(ws)._comma(unescape("%29"))._at_at("subexpression")._eq_eq_gt((function(node){return node.third();}));
statements=returnExpression._at_at("stat1")._eq_eq_gt((function(node){return smalltalk.Array.with_(node);}))._slash(expressions._comma(ws)._comma(".")._comma(ws)._comma(returnExpression)._at_at("stat2")._eq_eq_gt((function(node){return (function($rec){$rec.add_(node.at_((5)));return $rec.yourself();})(node.first());})))._slash(expressions._comma(self.optional_("."))._at_at("stat3")._eq_eq_gt((function(node){return node.first();})));
sequence=temps.optional()._at_at("temps")._comma(ws)._comma(directive.star())._comma(ws)._comma(statements.optional())._comma(ws)._at_at("sequence")._eq_eq_gt((function(node){return (function($rec){$rec.temps_(node.first());$rec.directives_(node.third());$rec.nodes_(node.at_((5)));return $rec.yourself();})(smalltalk.ExtendedSequenceNode.$$new());}));
block=self.gc_(unescape("%5B"))._comma(ws)._comma(blockParamList.optional())._comma(ws)._comma(sequence.optional())._comma(ws)._comma(unescape("%5D"))._at_at("block")._eq_eq_gt((function(node){return (function($rec){$rec.parameters_(node.third());return $rec.addNode_(node.at_((5)).asBlockSequenceNode());})(smalltalk.BlockNode.$$new());}));
operand=block._slash(literal)._slash(reference)._slash(noReference)._slash(subexpression)._at_at("operand");
literal.parser_(symbol._slash(number)._slash(string)._slash(character)._slash(emptyArray)._slash(literalArray)._slash(hashedString)._at_at("literalDelegate"));
unaryMessage=ws._comma(unarySelector)._comma(self._comma(":").not())._at_at("unaryMessage")._eq_eq_gt((function(node){return smalltalk.SendNode.$$new().selector_(node.second());}));
unaryTail=smalltalk.PPDelegateParser.$$new()._at_at("unaryTailDelegate");
unaryTail.parser_(unaryMessage._comma(unaryTail.optional())._at_at("unaryTailDelegate")._eq_eq_gt((function(node){return (((nil.isNil_(node.second()))).mustBeBoolean()==true ? (function(){return node.first();})() : (function(){return node.second().valueForReceiver_(node.first());})());})));
unarySend=operand._comma(unaryTail.optional())._at_at("unarySend")._eq_eq_gt((function(node){return (((nil.isNil_(node.second()))).mustBeBoolean()==true ? (function(){return node.first();})() : (function(){return node.second().valueForReceiver_(node.first());})());}));
binaryMessage=ws._comma(binarySelector)._comma(ws)._comma(unarySend._slash(operand))._at_at("binaryMessage")._eq_eq_gt((function(node){return (function($rec){$rec.selector_(node.second());return $rec.arguments_(smalltalk.Array.with_(node.fourth()));})(smalltalk.SendNode.$$new());}));
binaryTail=smalltalk.PPDelegateParser.$$new()._at_at("binaryTailDelegate");
binaryTail.parser_(binaryMessage._comma(binaryTail.optional())._eq_eq_gt((function(node){return (((nil.isNil_(node.second()))).mustBeBoolean()==true ? (function(){return node.first();})() : (function(){return node.second().valueForReceiver_(node.first());})());})));
binarySend=unarySend._comma(binaryTail.optional())._at_at("binarySend")._eq_eq_gt((function(node){return (((nil.isNil_(node.second()))).mustBeBoolean()==true ? (function(){return node.first();})() : (function(){return node.second().valueForReceiver_(node.first());})());}));
keywordPair=keyword._comma(ws)._comma(binarySend)._at_at("keywordPair");
keywordMessage=ws._comma(keywordPair).plus()._at_at("keywordMessage")._eq_eq_gt((function(nodes){return (function($rec){$rec.selector_(nodes.collect_((function(each){return each.second().first();})).join_(""));return $rec.arguments_(nodes.collect_((function(each){return each.second().third();})));})(smalltalk.SendNode.$$new());}));
keywordSend=binarySend._comma(keywordMessage)._at_at("keywordSend")._eq_eq_gt((function(node){return node.second().valueForReceiver_(node.first());}));
message=binaryMessage._slash(unaryMessage)._slash(keywordMessage)._at_at("message");
cascade=keywordSend._slash(binarySend)._at_at("head")._comma(ws._comma(unescape("%3B"))._comma(message).plus())._at_at("cascade")._eq_eq_gt((function(node){return node.first().cascadeNodeWithMessages_(node.second().collect_((function(each){return each.third();})));}));
inline=self._comma(unescape("%7B"))._comma(stIdentifier.optional()._comma(string)._comma(ws).plus())._comma(unescape("%7D"))._at_at("inline")._eq_eq_gt((function(node){return smalltalk.InlineNode.sources_parser_(node.second(),ws._comma(sequence.optional()));}));
expression.parser_(assignment._slash(cascade)._slash(keywordSend)._slash(binarySend)._slash(inline)._at_at("expression"));
return smalltalk.Array.with_with_with_(ws,keywordPattern._slash(binaryPattern)._slash(unaryPattern)._at_at("header"),sequence.optional()._at_at("body"));}
,"private",unescape("methodParserParts%0D%0A%09%22%20Private%20-%20Return%20a%20parser%20for%20parts%20of%20a%20method.%20%22%0D%0A%0D%0A%20%7C%20expression%20separator%20comment%20ws%20identifier%20keyword%20stSelectorPart%0D%0A%20%20string%20symbol%20number%20variable%20reference%20globalReference%0D%0A%20%20literal%20returnExpression%20expressionParser%20keyword%20unarySelector%0D%0A%20%20binarySelector%20keywordPattern%20unaryPattern%20binaryPattern%20assignment%0D%0A%20%20temps%20blockParamList%20block%20expression%20expressions%20subexpression%0D%0A%20%20statements%20sequence%20operand%20unaryMessage%20unarySend%20unaryTail%0D%0A%20%20binaryMessage%20binarySend%20binaryTail%20keywordMessage%20keywordSend%0D%0A%20%20keywordPair%20cascade%20message%20inline%20jsIndex%20character%20directive%0D%0A%20%20arrayLiteral%20arrayVariable%20literalArray%20aSymbol%0D%0A%20%20stIdentifier%20jsIdentifier%20noReference%20alienReference%0D%0A%20%20alienIdentifier%20emptyArray%20hashedString%20annotation%20argument%0D%0A%20%7C%0D%0A%0D%0A%20separator%20%3A%3D%20self%20separator.%0D%0A%20comment%20%3A%3D%20self%20comment.%0D%0A%20ws%20%3A%3D%20%28separator%20/%20comment%20@@%23sp%29%20star%20@@%23ws.%0D%0A%20%0D%0A%20string%20%3A%3D%20self%20%2C%24%27%20%2CPPStStringParser%20new%20@@%23string%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%7C%20ValueNode%20with%3A%20node%20second%20%5D.%0D%0A%20%0D%0A%20hashedString%20%3A%3D%20self%20%2C%28%24%23%2C%24%27%29%20%2CPPStStringParser%20new%20@@%23hashedString%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%7C%20ValueNode%20with%3A%20node%20second%20%5D.%0D%0A%0D%0A%20stIdentifier%20%3A%3D%20%28self%20nameStart%2C%20self%20symbolCharParser%20star%20@@%23stId%29%20flatten%20@@%23stIdentifier.%0D%0A%20stSelectorPart%20%3A%3D%20%28self%20upNameStart%2C%20self%20symbolCharParser%20star%20@@%23stSel%29%20flatten%20@@%23stSelectorPart.%0D%0A%20jsIdentifier%20%3A%3D%20%28self%20%2C%24%23%20%2Cself%20jsSymbolCharParser%20plus%20@@%23jsId%29%20flatten%20@@%23jsIdentifier.%0D%0A%20alienIdentifier%20%3A%3D%20self%20jsSymbolCharParser%20plus%20flatten%20@@%23alienIdentifier.%0D%0A%20identifier%20%3A%3D%20jsIdentifier%20/%20stIdentifier%20@@%23identifier.%0D%0A%0D%0A%20keyword%20%3A%3D%20%28stSelectorPart%20/%20identifier%2C%20%24%3A%29%20flatten%20@@%23keyword.%0D%0A%0D%0A%20annotation%20%3A%3D%20self%20annotationStart%0D%0A%09%2C%28string%20/%20stIdentifier%20/%20self%20globalNameParser%29%20@@%23annotation%0D%0A%09%3D%3D%3E%5B%3Anode%7C%20node%20last%20%5D.%0D%0A%20argument%20%3A%3D%20identifier%20%2Cannotation%20star%20%3D%3D%3E%20%5B%3Anode%7C%0D%0A%09%28ValueNode%20with%3A%20node%20first%29%20annotations%3A%20node%20last%0D%0A%20%5D.%0D%0A%0D%0A%20symbol%20%3A%3D%20self%20%2C%24%23%20%2Cself%20extendedSymbolCharParser%20plus%20flatten%20@@%23symbol%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20ValueNode%20with%3A%20node%20second%20%5D.%0D%0A%0D%0A%20number%20%3A%3D%20self%20number%20flatten%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20ValueNode%20with%3A%20node%20asNumber%20%5D.%0D%0A%0D%0A%20character%20%3A%3D%20self%20%2C%24%24%20%2CPPAnyParser%20new%20@@%23character%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20ValueNode%20with%3A%20%28node%20at%3A%202%29%20%5D.%0D%0A%0D%0A%20directive%20%3A%3D%20self%20directive%3A%20number%20ws%3A%20ws.%0D%0A%0D%0A%20variable%20%3A%3D%20identifier%20%3D%3D%3E%20%5B%3Atoken%20%7C%20VariableNode%20new%20value%3A%20token%20%5D.%0D%0A%20noReference%20%3A%3D%20self%20%2C%24%23%20@@%23noReference%20%3D%3D%3E%20%5B%3Atoken%20%7C%20NoReceiverNode%20new%20%5D.%0D%0A%20literal%20%3A%3D%20PPDelegateParser%20new%20@@%23literalDelegate.%0D%0A%20expression%20%3A%3D%20PPDelegateParser%20new%20@@%23expressionDelegate.%0D%0A%0D%0A%20arrayVariable%20%3A%3D%0D%0A%09%20%20%28self%20%2C%23true%20%20%3D%3D%3E%20%5B%3Anode%7C%20ValueNode%20with%3A%20true%20%5D%29%0D%0A%09/%20%28self%20%2C%23false%20%3D%3D%3E%20%5B%3Anode%7C%20ValueNode%20with%3A%20false%20%5D%29%0D%0A%09/%20%28self%20%2C%23nil%20%20%20%3D%3D%3E%20%5B%3Anode%7C%20ValueNode%20with%3A%20nil%20%5D%29%0D%0A%09@@%23arrayVariable.%0D%0A%0D%0A%20aSymbol%20%3A%3D%20self%20extendedSymbolCharParser%20plus%20flatten%20@@%23aSymbol%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%7C%20ValueNode%20with%3A%20node%20%5D.%0D%0A%0D%0A%20arrayLiteral%20%3A%3D%20literal%20/%20arrayVariable%20/%20aSymbol%20/%20variable.%0D%0A%20literalArray%20%3A%3D%20%28self%20gc%3A%20%27%23%28%27%29%0D%0A%09%2C%28ws%20%2CarrayLiteral%20%2Cws%29%20star%20%2C%24%29%0D%0A%09@@%23literalArray%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20ValueNode%20with%3A%20%28Array%0D%0A%09withAll%3A%20%28node%20second%0D%0A%09%09collect%3A%20%5B%3Aeach%20%7C%20each%20second%20value%5D%29%29%20%5D.%0D%0A%20emptyArray%20%3A%3D%20self%20%2C%27%23%28%27%20%2Cws%20%2C%24%29%20@@%23emptyArray%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20ValueNode%20with%3A%20Array%20new%20%5D.%0D%0A%0D%0A%20globalReference%20%3A%3D%20self%20globalNameParser%20@@%23globalReference%0D%0A%20%20%3D%3D%3E%20%5B%3Atoken%20%7C%20self%20gc.%20GlobalReferenceNode%20new%20value%3A%20token%20%5D.%0D%0A%20alienReference%20%3A%3D%20self%20%2C%28%24%23%20%2C%24%7B%29%20%2CalienIdentifier%2C%20%24%7D%20@@%23alienReference%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20AlienNode%20new%20value%3A%20node%20second%20%5D.%0D%0A%0D%0A%20jsIndex%20%3A%3D%20%28self%20gc%3A%20%24%5B%29%20%2Cexpression%2C%20%24%5D%20/%20%28self%20%2C%24%23%20%2CalienIdentifier%29.%0D%0A%20reference%20%3A%3D%20%28variable%20/%20globalReference%20/%20alienReference%20@@%23ref%29%0D%0A%09%2CjsIndex%20star%20%2Cannotation%20star%20@@%23reference%0D%0A%20%20%3D%3D%3E%20%5B%3Anodes%20%7C%20nodes%20first%20reference%3A%20nodes%20%5D.%0D%0A%0D%0A%20binarySelector%20%3A%3D%20%28%0D%0A%09%28self%20optional%3A%20%24%23%29%0D%0A%09%2Cself%20binaryCharParser%20plus%0D%0A%09%29%20flatten%20@@%23binarySelector.%0D%0A%0D%0A%20unarySelector%20%3A%3D%20stSelectorPart%20/%20identifier%20@@%23unarySelector.%0D%0A%0D%0A%20keywordPattern%20%3A%3D%0D%0A%09%28ws%2C%20keyword%20%20@@%23kw%20%2C%0D%0A%09%20ws%2C%20argument%20@@%23kwId%0D%0A%09%29%20plus%20@@%23keywordPattern%0D%0A%20%20%3D%3D%3E%20%5B%3Anodes%20%7C%20Array%0D%0A%20%20%20with%3A%20%28%28nodes%20collect%3A%20%5B%3Aeach%20%7C%20each%20at%3A%202%5D%29%20join%3A%20%27%27%29%0D%0A%20%20%20with%3A%20%28nodes%20collect%3A%20%5B%3Aeach%20%7C%20each%20at%3A%204%5D%29%20%5D.%0D%0A%0D%0A%20binaryPattern%20%3A%3D%20ws%2C%20binarySelector%2C%20ws%2C%20argument%20@@%23binaryPattern%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20Array%20with%3A%20node%20second%20with%3A%20%28Array%20with%3A%20node%20fourth%29%5D.%0D%0A%0D%0A%20unaryPattern%20%3A%3D%20ws%2C%20unarySelector%20@@%23unaryPattern%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20Array%20with%3A%20node%20second%20with%3A%20Array%20new%5D.%0D%0A%20%0D%0A%20expressions%20%3A%3D%20expression%2C%20%28%28ws%2C%20%24.%2C%20ws%2C%20expression%29%20@@%23anExpression%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20fourth%5D%29%20star%20@@%23expressions%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20%28Array%20with%3A%20node%20first%29%20%2Cnode%20second%20%5D.%0D%0A%0D%0A%20assignment%20%3A%3D%20reference%2C%20ws%2C%20%27%3A%3D%27%2C%20ws%2C%20expression%20@@%23assignment%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20AssignmentNode%20new%20left%3A%20node%20first%3B%20right%3A%20%28node%20at%3A%205%29%5D.%0D%0A%0D%0AreturnExpression%20%3A%3D%20%28self%20gc%3A%20%24%5E%29%20@@%23caret%0D%0A%09%2C%20ws%2C%20expression%20%2Cannotation%20star%0D%0A%09%2C%20ws%2C%20%24.%20asParser%20optional%09@@%23returnExpression%0D%0A%20%20%20%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20ReturnNode%20new%0D%0A%09addNode%3A%20%28node%20at%3A%203%29%3B%0D%0A%09annotations%3A%20%28node%20at%3A%204%29%3B%0D%0A%09yourself%20%5D.%0D%0A%0D%0A%20temps%20%3A%3D%20self%2C%20%24%7C%2C%20%28ws%2C%20argument%29%20star%2C%20ws%2C%20%24%7C%20@@%23temps%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20second%20collect%3A%20%5B%3Aeach%20%7C%20each%20second%20%5D%20%5D.%0D%0A%0D%0A%20blockParamList%20%3A%3D%20%28self%20%2C%24%3A%20%2Cws%20%2Cargument%20%2Cws%29%20plus%20%2C%24%7C%20@@%23blockParamList%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20first%20collect%3A%20%5B%3Aeach%20%7C%20each%20third%20%5D%20%5D.%0D%0A%0D%0A%20subexpression%20%3A%3D%20%28self%20gc%3A%20%24%28%29%20%2Cws%20%2Cexpression%20%2Cws%20%2C%24%29%20@@%23subexpression%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20third%20%5D.%0D%0A%0D%0A%20statements%20%3A%3D%20%28returnExpression%20@@%23stat1%20%3D%3D%3E%20%5B%3Anode%20%7C%20Array%20with%3A%20node%5D%29%0D%0A%20%20/%20%28expressions%20%2Cws%20%2C%24.%20%2Cws%20%2CreturnExpression%20@@%23stat2%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20first%20add%3A%20%28node%20at%3A%205%29%3B%20yourself%20%5D%29%0D%0A%20%20/%20%28expressions%20%2C%28self%20optional%3A%20%24.%29%20@@%23stat3%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20first%20%5D%29.%0D%0A%0D%0A%20sequence%20%3A%3D%20temps%20optional%20@@%23temps%0D%0A%09%2C%20ws%2C%20directive%20star%0D%0A%09%2C%20ws%2C%20statements%20optional%0D%0A%09%2C%20ws%20@@%23sequence%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20ExtendedSequenceNode%20new%0D%0A%09temps%3A%20node%20first%3B%0D%0A%09directives%3A%20node%20third%3B%0D%0A%09nodes%3A%20%28node%20at%3A%205%29%3B%0D%0A%09yourself%20%5D.%0D%0A%0D%0A%20block%20%3A%3D%20%28self%20gc%3A%20%24%5B%29%0D%0A%09%2Cws%20%2CblockParamList%20optional%0D%0A%09%2Cws%20%2Csequence%20optional%0D%0A%09%2Cws%20%2C%24%5D%20@@%23block%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20BlockNode%20new%0D%0A%09parameters%3A%20node%20third%3B%0D%0A%09addNode%3A%20%28node%20at%3A%205%29%20asBlockSequenceNode%20%5D.%0D%0A%0D%0A%20operand%20%3A%3D%20block%20/%20literal%20/%20reference%20/%20noReference%20/%20subexpression%20@@%23operand.%0D%0A%0D%0A%20literal%20parser%3A%20symbol%20/%20number%20/%20string%20/%20character%0D%0A%09/%20emptyArray%20/%20literalArray%20/%20hashedString%20@@%23literalDelegate.%0D%0A%0D%0A%20unaryMessage%20%3A%3D%20ws%20%2CunarySelector%20%2C%28self%20%2C%24%3A%29%20not%20@@%23unaryMessage%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20SendNode%20new%20selector%3A%20node%20second%20%5D.%0D%0A%0D%0A%20unaryTail%20%3A%3D%20PPDelegateParser%20new%20@@%23unaryTailDelegate.%0D%0A%20unaryTail%20parser%3A%20%28unaryMessage%2C%20unaryTail%20optional%20@@%23unaryTailDelegate%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20second%20isNil%20ifTrue%3A%20%5B%20node%20first%20%5D%0D%0A%09ifFalse%3A%20%5B%20node%20second%20valueForReceiver%3A%20node%20first%20%5D%20%5D%29.%0D%0A%0D%0A%20unarySend%20%3A%3D%20operand%2C%20unaryTail%20optional%20@@%23unarySend%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20second%20isNil%20ifTrue%3A%20%5B%20node%20first%20%5D%0D%0A%09ifFalse%3A%20%5B%20node%20second%20valueForReceiver%3A%20node%20first%20%5D%20%5D.%0D%0A%0D%0A%20binaryMessage%20%3A%3D%20ws%2C%20binarySelector%2C%20ws%2C%20%28unarySend%20/%20operand%29%20@@%23binaryMessage%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20SendNode%20new%0D%0A%09selector%3A%20node%20second%3B%0D%0A%09arguments%3A%20%28Array%20with%3A%20node%20fourth%29%20%5D.%0D%0A%0D%0A%20binaryTail%20%3A%3D%20PPDelegateParser%20new%20@@%23binaryTailDelegate.%0D%0A%20binaryTail%20parser%3A%20%28binaryMessage%2C%20binaryTail%20optional%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20second%20isNil%20ifTrue%3A%20%5B%20node%20first%20%5D%0D%0A%09ifFalse%3A%20%5B%20node%20second%20valueForReceiver%3A%20node%20first%20%5D%20%5D%29.%0D%0A%0D%0A%20binarySend%20%3A%3D%20unarySend%2C%20binaryTail%20optional%20@@%23binarySend%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20second%20isNil%20ifTrue%3A%20%5B%20node%20first%20%5D%0D%0A%09ifFalse%3A%20%5B%20node%20second%20valueForReceiver%3A%20node%20first%20%5D%20%5D.%0D%0A%0D%0A%20keywordPair%20%3A%3D%20keyword%2C%20ws%2C%20binarySend%20@@%23keywordPair.%0D%0A%0D%0A%20keywordMessage%20%3A%3D%20%28ws%2C%20keywordPair%29%20plus%20@@%23keywordMessage%0D%0A%20%20%3D%3D%3E%20%5B%3Anodes%20%7C%20SendNode%20new%0D%0A%09selector%3A%20%28%28nodes%20collect%3A%20%5B%3Aeach%20%7C%20each%20second%20first%5D%29%20join%3A%20%27%27%29%3B%0D%0A%09arguments%3A%20%28nodes%20collect%3A%20%5B%3Aeach%20%7C%20each%20second%20third%5D%29%5D.%0D%0A%0D%0A%20keywordSend%20%3A%3D%20binarySend%2C%20keywordMessage%20@@%23keywordSend%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20second%20valueForReceiver%3A%20node%20first%5D.%0D%0A%0D%0A%20message%20%3A%3D%20binaryMessage%20/%20unaryMessage%20/%20keywordMessage%20@@%23message.%0D%0A%0D%0A%20cascade%20%3A%3D%20%28keywordSend%20/%20binarySend%20@@%23head%29%0D%0A%09%2C%28ws%2C%20%24%3B%2C%20message%29%20plus%20@@%23cascade%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20node%20first%20cascadeNodeWithMessages%3A%20%0D%0A%09%28node%20second%20collect%3A%20%5B%3Aeach%20%7C%20each%20third%20%5D%29%5D.%0D%0A%0D%0A%20inline%20%3A%3D%20self%20%2C%24%7B%0D%0A%09%2C%28stIdentifier%20optional%20%2Cstring%20%2Cws%29%20plus%0D%0A%09%2C%24%7D%20@@%23inline%0D%0A%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20InlineNode%20sources%3A%20node%20second%20parser%3A%20ws%20%2Csequence%20optional%20%5D.%0D%0A%0D%0A%20expression%20parser%3A%20assignment%20/%20cascade%20/%20keywordSend%20/%20binarySend%20/%20inline%20@@%23expression.%0D%0A%0D%0A%5EArray%09with%3A%20ws%0D%0A%09with%3A%20keywordPattern%20/%20binaryPattern%20/%20unaryPattern%20@@%23header%0D%0A%09with%3A%20sequence%20optional%20@@%23body"));
smalltalk.bind(smalltalk.SmalltalkParser,"parser",0
,function(){var self=this;
var parts=nil;
var ws=nil;
var header=nil;
var body=nil;
var method=nil;
parts=self.methodParserParts();
ws=parts.first();
header=parts.second();
body=parts.third();
method=ws._comma(header)._comma(ws)._comma(body)._comma(ws)._at_at("impl").withSource()._eq_eq_gt((function(node){return (function($rec){$rec.selector_(node.first().second().first());$rec.arguments_(node.first().second().second());$rec.addNode_(node.first().fourth().shouldReturn());$rec.source_(node.second());return $rec.yourself();})(smalltalk.MethodNode.$$new());}));
self.gc();
return method._comma(smalltalk.PPEOFParser.$$new())._at_at("method")._eq_eq_gt((function(nodes){return nodes.first();}));}
,"parsers",unescape("parser%0D%0A%09%22%20Return%20the%20method%20parser.%20%22%0D%0A%0D%0A%09%7C%20parts%20ws%20header%20body%20method%20%7C%0D%0A%09parts%20%3A%3D%20self%20methodParserParts.%0D%0A%09ws%20%3A%3D%20parts%20first.%0D%0A%09header%20%3A%3D%20parts%20second.%0D%0A%09body%20%3A%3D%20parts%20third.%0D%0A%0D%0A%09method%20%3A%3D%20%28ws%20%2Cheader%0D%0A%09%09%2Cws%20%2Cbody%0D%0A%09%09%2Cws%20@@%23impl%29%20withSource%0D%0A%09%20%20%3D%3D%3E%20%5B%3Anode%20%7C%20MethodNode%20new%0D%0A%09%09selector%3A%20node%20first%20second%20first%3B%0D%0A%09%09arguments%3A%20node%20first%20second%20second%3B%0D%0A%09%09addNode%3A%20node%20first%20fourth%20shouldReturn%3B%0D%0A%09%09source%3A%20node%20second%3B%0D%0A%09%09yourself%20%5D.%0D%0A%20%0D%0A%09self%20gc.%0D%0A%20%09%5Emethod%20%2CPPEOFParser%20new%20@@%23method%0D%0A%09%20%20%20%3D%3D%3E%20%5B%3Anodes%20%7C%20nodes%20first%20%5D"));
smalltalk.bind(smalltalk.SmalltalkParser,"methodHeaderParser",0
,function(){var self=this;
var parts=nil;
var ws=nil;
var header=nil;
parts=self.methodParserParts();
ws=parts.first();
header=parts.second();
return ws._comma(header)._at_at("methodHeader")._eq_eq_gt((function(nodes){return nodes.second();}));}
,"parsers",unescape("methodHeaderParser%0D%0A%09%22%20Return%20a%20parser%20for%20method%20header.%20%22%0D%0A%0D%0A%09%7C%20parts%20ws%20header%20%7C%0D%0A%09parts%20%3A%3D%20self%20methodParserParts.%0D%0A%09ws%20%3A%3D%20parts%20first.%0D%0A%09header%20%3A%3D%20parts%20second.%0D%0A%09%5Ews%20%2Cheader%20@@%23methodHeader%0D%0A%09%09%3D%3D%3E%20%5B%3Anodes%7C%20nodes%20second%20%5D"));
smalltalk.bind(smalltalk.Exporter.$klass,"constructEventsTriggered",0
,function(){var self=this;
return ["isBehavior:", "needsClasses", "needsRoots", "needsEncoderFor:"]._comma(smalltalk.superImplementor(smalltalk.Exporter.$klass,"constructEventsTriggered").apply(self, []));}
,"events",unescape("constructEventsTriggered%0D%0A%09%22%20Private%20-%20Return%20the%20events%20managed%20by%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%09%23isBehavior%3A%0D%0A%09%09%23needsClasses%20%23needsRoots%0D%0A%09%09%23needsEncoderFor%3A%0D%0A%09%29%20%2Csuper%20constructEventsTriggered"));
smalltalk.bind(smalltalk.Exporter.$klass,"fileOutCategories:",0
,function(categories){var self=this;
return self.$$new().fileOutCategories_(categories);}
,"fileOut",unescape("fileOutCategories%3A%20categories%0D%0A%09%22%20Return%20the%20result%20of%20fileOut%20all%20the%20classes%20in%20categories.%20%22%0D%0A%0D%0A%09%5Eself%20new%20fileOutCategories%3A%20categories"));
smalltalk.bind(smalltalk.Exporter.$klass,"fileOutCategory:",0
,function(aString){var self=this;
return self.$$new().fileOutCategory_(aString);}
,"fileOut",unescape("fileOutCategory%3A%20aString%0D%0A%09%22%20Return%20the%20result%20of%20fileOut%20all%20the%20classes%20in%20category%20aString.%20%22%0D%0A%0D%0A%09%5Eself%20new%20fileOutCategory%3A%20aString"));
smalltalk.bind(smalltalk.Exporter.$klass,"fileOutAll:",0
,function(elements){var self=this;
return self.$$new().fileOutAll_(elements);}
,"fileOut",unescape("fileOutAll%3A%20elements%0D%0A%09%22%20Return%20the%20result%20of%20fileOut%20elements%20%28classes%2C%20methods%20and/or%20categories%29.%20%22%0D%0A%0D%0A%09%5Eself%20new%20fileOutAll%3A%20elements"));
smalltalk.bind(smalltalk.Exporter.$klass,"fileOutAll",0
,function(){var self=this;
return self.$$new().fileOutAll();}
,"fileOut",unescape("fileOutAll%0D%0A%09%22%20Return%20the%20result%20of%20fileOut%20of%20all%20classes%20in%20the%20system.%20%22%0D%0A%0D%0A%09%5Eself%20new%20fileOutAll%20"));
smalltalk.bind(smalltalk.Exporter.$klass,"fileOut:",0
,function(spec){var self=this;
return self.$$new().fileOutSpec_(spec);}
,"fileOut",unescape("fileOut%3A%20spec%0D%0A%09%22%20Return%20the%20result%20of%20fileOut%20of%20literals%20in%20spec.%20%22%0D%0A%0D%0A%09%5Eself%20new%20fileOutSpec%3A%20spec"));
smalltalk.bind(smalltalk.Exporter.$klass,"exportCategories:",0
,function(categories){var self=this;
return self.$$new().exportCategories_(categories);}
,"export",unescape("exportCategories%3A%20categories%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20all%20the%20classes%20in%20categories.%20%22%0D%0A%0D%0A%09%5Eself%20new%20exportCategories%3A%20categories"));
smalltalk.bind(smalltalk.Exporter.$klass,"exportCategory:",0
,function(aString){var self=this;
return self.$$new().exportCategory_(aString);}
,"export",unescape("exportCategory%3A%20aString%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20all%20the%20classes%20in%20category%20aString.%20%22%0D%0A%0D%0A%09%5Eself%20new%20exportCategory%3A%20aString"));
smalltalk.bind(smalltalk.Exporter.$klass,"export:",0
,function(spec){var self=this;
return self.$$new().exportSpec_(spec);}
,"export",unescape("export%3A%20spec%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20literals%20in%20spec.%20%22%0D%0A%0D%0A%09%5Eself%20new%20exportSpec%3A%20spec"));
smalltalk.bind(smalltalk.Exporter.$klass,"exportAll",0
,function(){var self=this;
return self.$$new().exportAll();}
,"export",unescape("exportAll%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20all%20classes%20in%20the%20system.%20%22%0D%0A%0D%0A%09%5Eself%20new%20exportAll"));
smalltalk.bind(smalltalk.Exporter.$klass,"exportAll:",0
,function(elements){var self=this;
return self.$$new().exportAll_(elements);}
,"export",unescape("exportAll%3A%20elements%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20all%20elements%20%28classes%2C%20methods%20and/or%20categories%29.%20%22%0D%0A%0D%0A%09%5Eself%20new%20exportAll%3A%20elements"));
smalltalk.bind(smalltalk.Exporter.$klass,"allClasses",0
,function(){var self=this;
return self.$$new().allClasses();}
,"accessing",unescape("allClasses%0D%0A%09%22%20OBSOLETE%2C%20Private%20-%20Return%20all%20the%20classes%20in%20the%20system%20%28in%20fileIn%20order%29.%20%22%0D%0A%0D%0A%09%5Eself%20new%20allClasses"));
smalltalk.bind(smalltalk.Exporter,"encoder:",0
,function(anEncoder){var self=this;
self["@encoder"]=anEncoder;
return self;}
,"accessing",unescape("encoder%3A%20anEncoder%0D%0A%09%22%20Set%20the%20encoder%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09encoder%20%3A%3D%20anEncoder"));
smalltalk.bind(smalltalk.Exporter,"setEncoderFor:",0
,function(aClass){var self=this;
var anEncoder=nil;
anEncoder=self.triggerEvent_with_ifNotHandled_("needsEncoderFor:",aClass,(function(){return aClass.compiler().encoder();}));
self.encoder_(anEncoder);
return self;}
,"accessing",unescape("setEncoderFor%3A%20aClass%0D%0A%09%22%20Set%20the%20encoder%20of%20the%20receiver%20to%20emmit%20code%20for%20aClass.%20%22%0D%0A%0D%0A%09%7C%20anEncoder%20%7C%0D%0A%09anEncoder%20%3A%3D%20self%20triggerEvent%3A%20%23needsEncoderFor%3A%20with%3A%20aClass%20ifNotHandled%3A%20%5B%0D%0A%09%09aClass%20compiler%20encoder%0D%0A%09%5D.%0D%0A%09self%20encoder%3A%20anEncoder%20"));
smalltalk.bind(smalltalk.Exporter,"encoderFor:",0
,function(aClass){var self=this;
if ((nil.isNil_(self["@encoder"]))){self.setEncoderFor_(aClass);};;
return self["@encoder"];}
,"accessing",unescape("encoderFor%3A%20aClass%0D%0A%09%22%20Return%20the%20encoder%20of%20the%20receiver.%0D%0A%09If%20the%20encoder%20is%20nil%2C%20use%20the%20default%20encoder%20for%20aClass.%0D%0A%09%22%0D%0A%0D%0A%09encoder%20isNil%20ifTrue%3A%20%5B%20self%20setEncoderFor%3A%20aClass%20%5D.%0D%0A%09%5Eencoder"));
smalltalk.bind(smalltalk.Exporter,"encoder",0
,function(){var self=this;
if ((nil.isNil_(self["@encoder"]))){self.setEncoderFor_(smalltalk.Object);};;
return self["@encoder"];}
,"accessing",unescape("encoder%0D%0A%09%22%20Return%20the%20encoder%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09encoder%20isNil%20ifTrue%3A%20%5B%20self%20setEncoderFor%3A%20Object%20%5D.%0D%0A%09%5Eencoder"));
smalltalk.bind(smalltalk.Exporter,"fileOutAll",0
,function(){var self=this;
return self.fileOutAll_(self.allClasses());}
,"fileOut",unescape("fileOutAll%0D%0A%09%22%20Return%20the%20result%20of%20fileOut%20of%20all%20classes%20in%20the%20system.%20%22%0D%0A%0D%0A%09%5Eself%20fileOutAll%3A%20self%20allClasses"));
smalltalk.bind(smalltalk.Exporter,"isBehavior:",0
,function(anObject){var self=this;
return self.triggerEvent_with_ifNotHandled_("isBehavior:",anObject,(function(){return smalltalk.is_kindOf_(anObject,(smalltalk.Behavior)).or_((function(){return anObject.isClass();}));}));}
,"testing",unescape("isBehavior%3A%20anObject%0D%0A%09%22%20Private%20-%20Return%20true%20if%20anObject%20is%20a%20Behavior.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20%23isBehavior%3A%20with%3A%20anObject%20ifNotHandled%3A%20%5B%0D%0A%09%09%28anObject%20isKindOf%3A%20Behavior%29%20or%3A%20%5B%20anObject%20isClass%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Exporter,"fileOutCollection:",0
,function(array){var self=this;
var cr=nil;
cr=unescape("%0D%0A");
return array.inject_into_([],(function(total,each){return (function($rec){$rec.add_(cr);$rec.add_(self.fileOutSpec_(each));return $rec.yourself();})(total);})).join_("");}
,"fileOut",unescape("fileOutCollection%3A%20array%0D%0A%09%22%20Return%20the%20result%20of%20fileOut%20of%20array%20contents.%20%22%0D%0A%0D%0A%09%7C%20cr%20%7C%0D%0A%09cr%20%3A%3D%20%27%0D%0A%27.%09%5E%28array%20inject%3A%20%23%28%29%20into%3A%20%5B%3Atotal%20%3Aeach%7C%0D%0A%09%09total%20add%3A%20cr%3B%20add%3A%20%28self%20fileOutSpec%3A%20each%29%3B%20yourself%0D%0A%09%09%5D%29%20join%3A%20%27%27"));
smalltalk.bind(smalltalk.Exporter,"fileOutSpec:",0
,function(spec){var self=this;
var aName=nil;
if ((nil.isNil_(spec))){return "";};;
if (!(spec.isString())){if (self.isBehavior_(spec)){return self.fileOut_(spec);};;
return self.fileOutCollection_(spec);};;
if (spec.includes_(unescape("%2C"))){return self.fileOutSpec_(spec.asArrayOfSubstringsSeparatedBy_(unescape("%2C")));};;
if (spec.includes_(" ")){return self.fileOutSpec_(spec.asArrayOfSubstringsSeparatedBy_(" "));};;
aName=spec.trimBlanks();
if (smalltalk.Smalltalk.includesKey_(aName)){return self.fileOutSpec_(smalltalk.Smalltalk.at_(aName));};;
return self.fileOutCategories_(aName);}
,"fileOut",unescape("fileOutSpec%3A%20spec%0D%0A%09%22%20Return%20the%20result%20of%20fileOut%20of%20literals%20in%20spec.%20%22%0D%0A%0D%0A%09%7C%20aName%20%7C%0D%0A%09spec%20isNil%20ifTrue%3A%20%5B%20%5E%27%27%20%5D.%0D%0A%09spec%20isString%20ifFalse%3A%20%5B%0D%0A%09%09%28self%20isBehavior%3A%20spec%29%20ifTrue%3A%20%5B%20%5Eself%20fileOut%3A%20spec%20%5D.%0D%0A%09%09%5Eself%20fileOutCollection%3A%20spec%0D%0A%09%5D.%0D%0A%09%28spec%20includes%3A%20%24%2C%29%20ifTrue%3A%20%5B%20%5Eself%20fileOutSpec%3A%20%28spec%20asArrayOfSubstringsSeparatedBy%3A%20%24%2C%29%20%5D.%0D%0A%09%28spec%20includes%3A%20%24%20%29%20ifTrue%3A%20%5B%20%5Eself%20fileOutSpec%3A%20%28spec%20asArrayOfSubstringsSeparatedBy%3A%20%24%20%29%20%5D.%0D%0A%09aName%20%3A%3D%20spec%20trimBlanks.%0D%0A%09%28Smalltalk%20includesKey%3A%20aName%29%20ifTrue%3A%20%5B%20%5Eself%20fileOutSpec%3A%20%28Smalltalk%20at%3A%20aName%29%20%5D.%0D%0A%09%5Eself%20fileOutCategories%3A%20aName"));
smalltalk.bind(smalltalk.Exporter,"exportCollection:",0
,function(array){var self=this;
var cr=nil;
cr=unescape("%0D%0A");
return array.inject_into_([],(function(total,each){return (function($rec){$rec.add_(cr);$rec.add_(self.exportSpec_(each));return $rec.yourself();})(total);})).join_("");}
,"export",unescape("exportCollection%3A%20array%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20array%20contents.%20%22%0D%0A%0D%0A%09%7C%20cr%20%7C%0D%0A%09cr%20%3A%3D%20%27%0D%0A%27.%09%5E%28array%20inject%3A%20%23%28%29%20into%3A%20%5B%3Atotal%20%3Aeach%7C%0D%0A%09%09total%20add%3A%20cr%3B%20add%3A%20%28self%20exportSpec%3A%20each%29%3B%20yourself%0D%0A%09%09%5D%29%20join%3A%20%27%27"));
smalltalk.bind(smalltalk.Exporter,"exportSpec:",0
,function(spec){var self=this;
var aName=nil;
if ((nil.isNil_(spec))){return "";};;
if (!(spec.isString())){if (self.isBehavior_(spec)){return self.export_(spec);};;
return self.exportCollection_(spec);};;
if (spec.includes_(unescape("%2C"))){return self.exportSpec_(spec.asArrayOfSubstringsSeparatedBy_(unescape("%2C")));};;
if (spec.includes_(" ")){return self.exportSpec_(spec.asArrayOfSubstringsSeparatedBy_(" "));};;
aName=spec.trimBlanks();
if (smalltalk.Smalltalk.includesKey_(aName)){return self.export_(smalltalk.Smalltalk.at_(aName));};;
return self.exportCategories_(aName);}
,"export",unescape("exportSpec%3A%20spec%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20literals%20in%20spec.%20%22%0D%0A%0D%0A%09%7C%20aName%20%7C%0D%0A%09spec%20isNil%20ifTrue%3A%20%5B%20%5E%27%27%20%5D.%0D%0A%09spec%20isString%20ifFalse%3A%20%5B%0D%0A%09%09%28self%20isBehavior%3A%20spec%29%20ifTrue%3A%20%5B%20%5Eself%20export%3A%20spec%20%5D.%0D%0A%09%09%5Eself%20exportCollection%3A%20spec%0D%0A%09%5D.%0D%0A%09%28spec%20includes%3A%20%24%2C%29%20ifTrue%3A%20%5B%20%5Eself%20exportSpec%3A%20%28spec%20asArrayOfSubstringsSeparatedBy%3A%20%24%2C%29%20%5D.%0D%0A%09%28spec%20includes%3A%20%24%20%29%20ifTrue%3A%20%5B%20%5Eself%20exportSpec%3A%20%28spec%20asArrayOfSubstringsSeparatedBy%3A%20%24%20%29%20%5D.%0D%0A%09aName%20%3A%3D%20spec%20trimBlanks.%0D%0A%09%28Smalltalk%20includesKey%3A%20aName%29%20ifTrue%3A%20%5B%20%5Eself%20export%3A%20%28Smalltalk%20at%3A%20aName%29%20%5D.%0D%0A%09%5Eself%20exportCategories%3A%20aName"));
smalltalk.bind(smalltalk.Exporter,"rootClasses",0
,function(){var self=this;
return self.triggerEvent_ifNotHandled_("needsRoots",(function(){return smalltalk.Smalltalk.rootClasses();}));}
,"accessing",unescape("rootClasses%0D%0A%09%22%20Private%20-%20Return%20all%20the%20root%20classes%20in%20the%20system%20%28in%20fileIn%20order%29.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20%23needsRoots%20ifNotHandled%3A%20%5B%0D%0A%09%09Smalltalk%20rootClasses%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Exporter,"allClasses",0
,function(){var self=this;
return self.triggerEvent_ifNotHandled_("needsClasses",(function(){return self.rootClasses().inject_into_([],(function(total,each){return (function($rec){$rec.addAll_(each.withAllSubclasses());return $rec.yourself();})(total);}));}));}
,"accessing",unescape("allClasses%0D%0A%09%22%20Private%20-%20Return%20all%20the%20classes%20in%20the%20system%20%28in%20fileIn%20order%29.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20%23needsClasses%20ifNotHandled%3A%20%5B%0D%0A%09%09self%20rootClasses%20inject%3A%20%23%28%29%20into%3A%20%5B%3Atotal%20%3Aeach%7C%0D%0A%09%09%09total%20addAll%3A%20each%20withAllSubclasses%3B%20yourself%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Exporter,"matchCategory:",0
,function(aString){var self=this;
return self.matchCategory_from_(aString,smalltalk.Smalltalk.rootClasses());}
,"categories",unescape("matchCategory%3A%20aString%0D%0A%09%22%20Return%20the%20result%20of%20filtering%20all%20the%20classes%20%28and%20orphan%20methods%29%20in%20category%20aString.%20%22%0D%0A%0D%0A%09%5Eself%20matchCategory%3A%20aString%20from%3A%20Smalltalk%20rootClasses"));
smalltalk.bind(smalltalk.Exporter,"matchCategory:from:",0
,function(aString,rootClasses){var self=this;
var selection=nil;
var selected=nil;
var rejected=nil;
var orphanMethods=nil;
var fileOutMethods=nil;
selected=smalltalk.Array.$$new();
rejected=smalltalk.Array.$$new();
orphanMethods=smalltalk.Array.$$new();
selection=((aString.includes_(unescape("-"))).mustBeBoolean()==true ? (function(){return (function(each){return each.category().asString()._eq(aString);});})() : (function(){return (function(each){return each.category().asString().upTo_(unescape("-"))._eq(aString);});})());
rootClasses.do_((function(root){return root.instanceClass().withAllSubclasses().do_((function(each){return ((selection.value_(each)).mustBeBoolean()==true ? (function(){return selected;})() : (function(){return rejected;})()).add_(each);}));}));
fileOutMethods=(function(cls){return cls.methodsDo_((function(aMethod){return ((selection.value_(aMethod)).mustBeBoolean()==true ? (function(){return orphanMethods.add_(aMethod);})() : nil);}));});
rejected.do_((function(each){return (function($rec){$rec.value_(each);return $rec.value_(each.$klass);})(fileOutMethods);}));
return smalltalk.Array.with_with_with_(selected,rejected,orphanMethods);}
,"categories",unescape("matchCategory%3A%20aString%20from%3A%20rootClasses%0D%0A%09%22%20Return%20the%20result%20of%20filtering%20all%20the%20classes%20%28and%20orphan%20methods%29%20in%20category%20aString.%20%22%0D%0A%0D%0A%09%7C%20selection%20selected%20rejected%20orphanMethods%20fileOutMethods%20%7C%0D%0A%09selected%20%3A%3D%20Array%20new.%0D%0A%09rejected%20%3A%3D%20Array%20new.%0D%0A%09orphanMethods%20%3A%3D%20Array%20new.%0D%0A%09selection%20%3A%3D%20%28aString%20includes%3A%20%24-%29%0D%0A%09%09ifTrue%3A%20%5B%20%5B%3Aeach%7C%20each%20category%20asString%20%3D%20aString%20%5D%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20%5B%3Aeach%7C%20%28each%20category%20asString%20upTo%3A%20%24-%29%20%3D%20aString%20%5D%20%5D.%0D%0A%09rootClasses%20do%3A%20%5B%3Aroot%7C%0D%0A%09%09root%20instanceClass%20withAllSubclasses%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%09%28%28selection%20value%3A%20each%29%0D%0A%09%09%09%09ifTrue%3A%20%5B%20selected%20%5D%20ifFalse%3A%20%5B%20rejected%20%5D%29%0D%0A%09%09%09%09add%3A%20each%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09fileOutMethods%20%3A%3D%20%5B%3Acls%7C%0D%0A%09%09cls%20methodsDo%3A%20%5B%3AaMethod%20%7C%0D%0A%09%09%09%28selection%20value%3A%20aMethod%29%20ifTrue%3A%20%5B%20orphanMethods%20add%3A%20aMethod%20%5D%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09rejected%20do%3A%20%5B%3Aeach%7C%20fileOutMethods%20value%3A%20each%3B%20value%3A%20each%20class%20%5D.%0D%0A%09%5EArray%20with%3A%20selected%20with%3A%20rejected%20with%3A%20orphanMethods"));
smalltalk.bind(smalltalk.Exporter,"fileOutCategories:",0
,function(categories){var self=this;
if (categories.isString()){return self.fileOutCategories_(categories.asArrayOfSubstrings());};;
return categories.asArray().inject_into_([],(function(total,each){return (function($rec){$rec.add_(self.fileOutCategory_(each));return $rec.yourself();})(total);})).join_("");}
,"fileOut",unescape("fileOutCategories%3A%20categories%0D%0A%09%22%20Return%20the%20result%20of%20fileOut%20all%20the%20classes%20in%20categories.%20%22%0D%0A%0D%0A%09categories%20isString%20ifTrue%3A%20%5B%20%5Eself%20fileOutCategories%3A%20categories%20asArrayOfSubstrings%20%5D.%0D%0A%09%5E%28categories%20asArray%20inject%3A%20%23%28%29%20into%3A%20%5B%3Atotal%20%3Aeach%7C%0D%0A%09%09total%20add%3A%20%28self%20fileOutCategory%3A%20each%29%3B%20yourself%0D%0A%09%5D%29%20join%3A%20%27%27"));
smalltalk.bind(smalltalk.Exporter,"fileOutCategory:",0
,function(aString){var self=this;
var stream=nil;
var match=nil;
stream="".writeStream();
match=self.matchCategory_(aString);
self.fileOutAll_on_(match.first(),stream);
self.fileOutMethods_on_(match.last(),stream);
return stream.contents();}
,"fileOut",unescape("fileOutCategory%3A%20aString%0D%0A%09%22%20Return%20the%20result%20of%20fileOut%20all%20the%20classes%20in%20category%20aString.%20%22%0D%0A%0D%0A%09%7C%20stream%20match%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09match%20%3A%3D%20self%20matchCategory%3A%20aString.%0D%0A%09self%20fileOutAll%3A%20match%20first%20on%3A%20stream.%0D%0A%09self%20fileOutMethods%3A%20match%20last%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"fileOut:",0
,function(aClass){var self=this;
var stream=nil;
stream="".writeStream();
self.fileOut_on_(aClass,stream);
return stream.contents();}
,"fileOut",unescape("fileOut%3A%20aClass%0D%0A%09%22%20Return%20fileOut%20definition%20and%20code%20of%20a%20class.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09self%20fileOut%3A%20aClass%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"fileOutClass:",0
,function(aClass){var self=this;
return self.fileOut_(aClass);}
,"fileOut",unescape("fileOutClass%3A%20aClass%0D%0A%09%22%20Return%20fileOut%20definition%20and%20code%20of%20a%20class.%20%22%0D%0A%0D%0A%09%5Eself%20fileOut%3A%20aClass"));
smalltalk.bind(smalltalk.Exporter,"fileOutAll:",0
,function(elements){var self=this;
var stream=nil;
stream="".writeStream();
self.fileOutAll_on_(elements,stream);
return stream.contents();}
,"fileOut",unescape("fileOutAll%3A%20elements%0D%0A%09%22%20Return%20fileOut%20definition%20and%20code%20of%20elements%20%28classes%2C%20methods%20and/or%20categories%29.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09self%20fileOutAll%3A%20elements%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"fileOutMethod:of:",0
,function(aMethod,aClass){var self=this;
var stream=nil;
stream="".writeStream();
self.fileOutMethod_of_on_(aMethod,aClass,stream);
return stream.contents();}
,"fileOut",unescape("fileOutMethod%3A%20aMethod%20of%3A%20aClass%0D%0A%09%22%20Return%20fileOut%20method%20of%20a%20class.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09self%20fileOutMethod%3A%20aMethod%20of%3A%20aClass%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"fileOutAllMethodsOf:on:",0
,function(aClass,stream){var self=this;
self.fileOutMethodsOf_on_(aClass.$klass,stream);
self.fileOutMethodsOf_on_(aClass,stream);
return self;}
,"fileOut",unescape("fileOutAllMethodsOf%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20FileOut%20all%20methods%20of%20aClass%20%28and%20metaclass%29%20onto%20stream.%20%22%0D%0A%0D%0A%09self%20fileOutMethodsOf%3A%20aClass%20class%20on%3A%20stream.%0D%0A%09self%20fileOutMethodsOf%3A%20aClass%20on%3A%20stream."));
smalltalk.bind(smalltalk.Exporter,"fileOutAll:on:",0
,function(elements,stream){var self=this;
if (elements.isString()){return self.fileOutAll_on_(elements.asArrayOfSubstrings(),stream);};;
elements.do_((function(each){return ((self.isBehavior_(each)).mustBeBoolean()==true ? (function(){return self.fileOutDefinitionOfClass_on_(each,stream);})() : nil);}));
stream.cr();
elements.do_((function(each){return ((self.isBehavior_(each)).mustBeBoolean()==true ? (function(){return self.fileOutAllMethodsOf_on_(each,stream);})() : (function(){return ((each.isString()).mustBeBoolean()==true ? (function(){return (function($rec){$rec.nextPutAll_(self.fileOutCategory_(each));return $rec.cr();})(stream);})() : (function(){return self.fileOutMethod_of_on_(each,each.classField(),stream);})());})());}));
return self;}
,"fileOut",unescape("fileOutAll%3A%20elements%20on%3A%20stream%0D%0A%09%22%20FileOut%20definition%20and%20code%20of%20elements%20%28classes%2C%20methods%20and/or%20categories%29%20onto%20stream.%20%22%0D%0A%0D%0A%09elements%20isString%20ifTrue%3A%20%5B%20%5Eself%20fileOutAll%3A%20elements%20asArrayOfSubstrings%20on%3A%20stream%20%5D.%0D%0A%09elements%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28self%20isBehavior%3A%20each%29%20ifTrue%3A%20%5B%0D%0A%09%09%09self%20fileOutDefinitionOfClass%3A%20each%20on%3A%20stream%0D%0A%09%09%5D.%0D%0A%09%5D.%09stream%20cr.%0D%0A%09elements%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28self%20isBehavior%3A%20each%29%20ifTrue%3A%20%5B%0D%0A%09%09%09self%20fileOutAllMethodsOf%3A%20each%20on%3A%20stream%0D%0A%09%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09each%20isString%0D%0A%09%09%09ifTrue%3A%20%5B%20stream%20nextPutAll%3A%20%28self%20fileOutCategory%3A%20each%29%3Bcr%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20self%20fileOutMethod%3A%20each%20of%3A%20each%20classField%20on%3A%20stream%20%5D%0D%0A%09%09%5D%0D%0A%09%5D."));
smalltalk.bind(smalltalk.Exporter,"fileOutDefinitionOfClass:on:",0
,function(aClass,stream){var self=this;
self.fileOutDefinitionOf_on_(aClass,stream);
self.fileOutDefinitionOf_on_(aClass.$klass,stream);
return self;}
,"fileOut",unescape("fileOutDefinitionOfClass%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20FileOut%20definition%20of%20aClass%20%28and%20metaclass%29%20onto%20stream.%20%22%0D%0A%0D%0A%09self%20fileOutDefinitionOf%3A%20aClass%20on%3A%20stream.%0D%0A%09self%20fileOutDefinitionOf%3A%20aClass%20class%20on%3A%20stream"));
smalltalk.bind(smalltalk.Exporter,"fileOutDefinitionOf:",0
,function(aClass){var self=this;
var stream=nil;
stream="".writeStream();
self.fileOutDefinitionOfClass_on_(aClass.instanceClass(),stream);
return stream.contents();}
,"fileOut",unescape("fileOutDefinitionOf%3A%20aClass%0D%0A%09%22%20Return%20the%20definition%20of%20a%20class.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09self%20fileOutDefinitionOfClass%3A%20aClass%20instanceClass%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"fileOut:on:",0
,function(aClass,stream){var self=this;
self.fileOutAll_on_(smalltalk.Array.with_(aClass),stream);
return self;}
,"fileOut",unescape("fileOut%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20FileOut%20definition%20and%20code%20of%20a%20class%20onto%20stream.%20%22%0D%0A%0D%0A%09self%20fileOutAll%3A%20%28Array%20with%3A%20aClass%29%20on%3A%20stream."));
smalltalk.bind(smalltalk.Exporter,"fileOutDefinitionOf:on:",0
,function(aClass,stream){var self=this;
if (aClass.isMetaclass()){if (self.fileOutStrings_of_on_("instanceVariableNames",aClass,stream)){(function($rec){$rec.nextChunkPut_("");return $rec.cr();})(stream);};;
if (self.fileOutArray_of_on_("propertyNames",aClass,stream)){(function($rec){$rec.nextChunkPut_("");return $rec.cr();})(stream);};;
return self;};;
(function($rec){$rec.nextPutAll_(aClass.superclass().asString());$rec.cr();$rec.tab();return $rec.nextPutAll_(unescape("subclass%3A%20%23"));})(stream);
if (!(self.fileOutStrings_of_on_("instanceVariableNames",aClass,stream))){(function($rec){$rec.nextPutAll_(self.globalNameFor_(aClass));$rec.cr();$rec.tab();$rec.nextPutAll_("instanceVariableNames: ");return $rec.nextPutAll_("".printString());})(stream);};;
if ((nil.isNil_(aClass.classVariables())===false)){(function($rec){$rec.cr();$rec.tab();$rec.nextPutAll_("classVariableNames: ");return $rec.nextPutAll_(aClass.classVariablesString().printString());})(stream);};;
if ((nil.isNil_(aClass.poolDictionaries())===false)){(function($rec){$rec.cr();$rec.tab();$rec.nextPutAll_("poolDictionaries: ");return $rec.nextPutAll_(aClass.poolDictionariesString().printString());})(stream);};;
(function($rec){$rec.cr();$rec.tab();$rec.nextPutAll_("category: ");return $rec.nextPutAll_(aClass.category().asLiteral());})(stream);
(function($rec){$rec.nextChunkPut_("");return $rec.cr();})(stream);
if (self.fileOutArray_of_on_("propertyNames",aClass,stream)){(function($rec){$rec.nextChunkPut_("");return $rec.cr();})(stream);};;
if ((nil.isNil_(aClass.comment())===false).and_((function(){return aClass.comment().notEmpty();}))){stream.nextPutAll_(self.globalNameFor_(aClass)._comma(" comment: ")._comma(aClass.comment().printString()));
(function($rec){$rec.nextChunkPut_("");return $rec.cr();})(stream);};;
return self;}
,"fileOut",unescape("fileOutDefinitionOf%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20FileOut%20definition%20of%20a%20class%20onto%20stream.%20%22%0D%0A%0D%0A%09aClass%20isMetaclass%20ifTrue%3A%20%5B%0D%0A%09%09%28self%20fileOutStrings%3A%20%23instanceVariableNames%20of%3A%20aClass%20on%3A%20stream%29%20ifTrue%3A%20%5B%0D%0A%09%09%09stream%20nextChunkPut%3A%20%27%27%3Bcr.%0D%0A%09%09%5D.%0D%0A%09%09%28self%20fileOutArray%3A%20%23propertyNames%20of%3A%20aClass%20on%3A%20stream%29%20ifTrue%3A%20%5B%0D%0A%09%09%09stream%20nextChunkPut%3A%20%27%27%3Bcr.%0D%0A%09%09%5D.%0D%0A%09%09%5Eself%0D%0A%09%5D.%0D%0A%0D%0A%09stream%0D%0A%09%09nextPutAll%3A%20aClass%20superclass%20asString%3B%20cr%3Btab%3B%0D%0A%09%09nextPutAll%3A%20%27subclass%3A%20%23%27.%0D%0A%09%28self%20fileOutStrings%3A%20%23instanceVariableNames%20of%3A%20aClass%20on%3A%20stream%29%20ifFalse%3A%20%5B%0D%0A%09%09stream%0D%0A%09%09%09nextPutAll%3A%20%28self%20globalNameFor%3A%20aClass%29%3B%20cr%3Btab%3B%0D%0A%09%09%09nextPutAll%3A%20%27instanceVariableNames%3A%20%27%3B%0D%0A%09%09%09nextPutAll%3A%20%27%27%20printString%0D%0A%09%5D.%0D%0A%09aClass%20classVariables%20notNil%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20stream%09%20cr%3Btab%3B%0D%0A%09%09nextPutAll%3A%20%27classVariableNames%3A%20%27%3B%0D%0A%09%09nextPutAll%3A%20aClass%20classVariablesString%20printString%0D%0A%09%5D.%0D%0A%09aClass%20poolDictionaries%20notNil%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20stream%09%20cr%3Btab%3B%0D%0A%09%09nextPutAll%3A%20%27poolDictionaries%3A%20%27%3B%0D%0A%09%09nextPutAll%3A%20aClass%20poolDictionariesString%20printString%0D%0A%09%5D.%0D%0A%09stream%20cr%3Btab%3B%0D%0A%09%09nextPutAll%3A%20%27category%3A%20%27%3B%0D%0A%09%09nextPutAll%3A%20aClass%20category%20asLiteral.%0D%0A%09stream%20nextChunkPut%3A%20%27%27%3Bcr.%0D%0A%09%28self%20fileOutArray%3A%20%23propertyNames%20of%3A%20aClass%20on%3A%20stream%29%20ifTrue%3A%20%5B%0D%0A%09%09stream%20nextChunkPut%3A%20%27%27%3Bcr.%0D%0A%09%5D.%0D%0A%09%28aClass%20comment%20notNil%20and%3A%20%5BaClass%20comment%20notEmpty%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09stream%20nextPutAll%3A%20%28self%20globalNameFor%3A%20aClass%29%2C%27%20comment%3A%20%27%2C%20aClass%20comment%20printString.%0D%0A%09%09stream%20nextChunkPut%3A%20%27%27%3Bcr.%0D%0A%09%5D."));
smalltalk.bind(smalltalk.Exporter,"fileOutStrings:of:on:",0
,function(selector,aClass,stream){var self=this;
var strings=nil;
strings=aClass.perform_(selector);
if (strings.isEmpty()){return false;};;
(function($rec){$rec.nextPutAll_(self.globalNameFor_(aClass));$rec.cr();$rec.tab();return $rec.nextPutAll_(selector._comma(unescape("%3A%20%27")));})(stream);
strings.do_((function(each){return stream.nextPutAll_(" "._comma(each));}));
stream.nextPutAll_(unescape("%20%27"));
return true;}
,"fileOut",unescape("fileOutStrings%3A%20selector%20of%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20Private%20-%20FileOut%20selector%20strings%20of%20a%20class%20onto%20stream.%0D%0A%09Return%20true%20if%20code%20has%20been%20filedOut.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20strings%20%7C%0D%0A%09strings%20%3A%3D%20aClass%20perform%3A%20selector.%0D%0A%09strings%20isEmpty%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09stream%0D%0A%09%09nextPutAll%3A%20%28self%20globalNameFor%3A%20aClass%29%3B%20cr%3Btab%3B%0D%0A%09%09nextPutAll%3A%20selector%2C%27%3A%20%27%27%27.%0D%0A%09strings%20do%3A%20%5B%3Aeach%7C%20stream%20nextPutAll%3A%20%27%20%27%2Ceach%20%5D.%0D%0A%09stream%20nextPutAll%3A%20%27%20%27%27%27.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.Exporter,"fileOutArray:of:on:",0
,function(selector,aClass,stream){var self=this;
var strings=nil;
strings=aClass.perform_(selector);
if (strings.isEmpty()){return false;};;
(function($rec){$rec.nextPutAll_(self.globalNameFor_(aClass));return $rec.nextPutAll_(" "._comma(selector)._comma(": ")._comma(strings.asLiteral()));})(stream);
return true;}
,"fileOut",unescape("fileOutArray%3A%20selector%20of%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20Private%20-%20FileOut%20selector%20arrayed%20strings%20of%20a%20class%20onto%20stream.%0D%0A%09Return%20true%20if%20code%20has%20been%20filedOut.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20strings%20%7C%0D%0A%09strings%20%3A%3D%20aClass%20perform%3A%20selector.%0D%0A%09strings%20isEmpty%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09stream%0D%0A%09%09nextPutAll%3A%20%28self%20globalNameFor%3A%20aClass%29%3B%0D%0A%09%09nextPutAll%3A%20%27%20%27%2Cselector%2C%27%3A%20%27%2C%20strings%20asLiteral.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.Exporter,"fileOutMethodsOf:on:",0
,function(aClass,stream){var self=this;
var selectors=nil;
selectors=aClass.selectors();
if (selectors.isEmpty()){return self;};;
stream.cr();
self.fileOutSelectors_of_on_(selectors,aClass,stream);
return self;}
,"fileOut",unescape("fileOutMethodsOf%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20FileOut%20methods%20of%20a%20class%20onto%20stream.%20%22%0D%0A%0D%0A%09%7C%20selectors%20%7C%0D%0A%09selectors%20%3A%3D%20aClass%20selectors.%0D%0A%09selectors%20isEmpty%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09stream%20cr.%0D%0A%09self%0D%0A%09%09fileOutSelectors%3A%20selectors%0D%0A%09%09of%3A%20aClass%20on%3A%20stream"));
smalltalk.bind(smalltalk.Exporter,"fileOutClusters",0
,function(){var self=this;
return ["APIBuilder"].select_((function(key){return smalltalk.Smalltalk.includesKey_(key);})).collect_((function(each){return smalltalk.Smalltalk.at_(each).$$default();}));}
,"private",unescape("fileOutClusters%0D%0A%09%22%20Private%20-%20Return%20the%20fileOut%20chustering%20dumpers%20of%20the%20receiver.%0D%0A%09A%20fileOutCluster%20is%20an%20object%20that%20potentially%20dump%20clustered%20actions%20for%20selectors%20implemented%20by%20aClass.%0D%0A%09The%20fileOut%20write%20%28smalltalk%29code%20for%20multiple%20selectors%20removing%20them%20from%20selectors%20collection.%0D%0A%09Note%3A%20the%20selectors%20collection%20is%20modified%20during%20fileOut.%0D%0A%09The%20default%20answer%20is%20an%20empty%20collection%20of%20custed%20or%20a%20collection%20with%20default%20APIBuilder.%0D%0A%09This%20method%20can%20be%20refined%20by%20the%20subclasses.%0D%0A%09%22%0D%0A%0D%0A%09%5E%28%23%28%20%23APIBuilder%20%29%0D%0A%09%09select%3A%20%5B%3Akey%7C%20Smalltalk%20includesKey%3A%20key%20%5D%29%0D%0A%09%09collect%3A%20%5B%3Aeach%7C%20%28Smalltalk%20at%3A%20each%29%20default%20%5D"));
smalltalk.bind(smalltalk.Exporter,"fileOutSelectors:of:on:",0
,function(selectors,aClass,stream){var self=this;
self.fileOutClusters().do_((function(each){return each.fileOut_selectors_of_on_(self,selectors,aClass,stream);}));
self.fileOutMethods_on_(selectors.collect_((function(each){return aClass.compiledMethodAt_(each);})),stream);
return self;}
,"fileOut",unescape("fileOutSelectors%3A%20selectors%20of%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20FileOut%20methods%20implementing%20selectors%20of%20a%20class%20onto%20stream.%20%22%0D%0A%0D%0A%09self%20fileOutClusters%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09each%0D%0A%09%09%09fileOut%3A%20self%20selectors%3A%20selectors%0D%0A%09%09%09of%3A%20aClass%20on%3A%20stream%0D%0A%09%5D.%0D%0A%09self%09fileOutMethods%3A%20%28selectors%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09%09aClass%20compiledMethodAt%3A%20each%20%5D%29%0D%0A%09%09on%3A%20stream"));
smalltalk.bind(smalltalk.Exporter,"fileOutMethods:on:",0
,function(array,stream){var self=this;
var sorted=nil;
sorted=array.sorted_((function(a,b){return a.category()._lt(b.category()).or_((function(){return a.category()._eq(b.category()).and_((function(){return a.selector()._lt(b.selector());}));}));}));
sorted.do_((function(each){return self.fileOutMethod_of_on_(each,each.classField(),stream);}));
return self;}
,"fileOut",unescape("fileOutMethods%3A%20array%20on%3A%20stream%0D%0A%09%22%20FileOut%20array%20of%20methods%20of%20a%20class%20onto%20stream.%20%22%0D%0A%0D%0A%09%7C%20sorted%20%7C%0D%0A%09sorted%20%3A%3D%20array%20sorted%3A%20%5B%3Aa%20%3Ab%7C%0D%0A%09%09a%20category%20%3C%20b%20category%0D%0A%09%09or%3A%20%5B%20a%20category%20%3D%20b%20category%0D%0A%09%09and%3A%20%5B%20a%20selector%20%3C%20b%20selector%20%5D%5D%0D%0A%09%5D.%0D%0A%0D%0A%09sorted%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09self%09fileOutMethod%3A%20each%0D%0A%09%09%09of%3A%20each%20classField%20on%3A%20stream%0D%0A%09%5D."));
smalltalk.bind(smalltalk.Exporter,"isASymbol:",0
,function(aString){var self=this;
return (nil.isNil_(smalltalk.RegularExpression.fromString_(unescape("%5B%5Ea-zA-Z0-9_%3A%5D")).exec_(aString)));}
,"private",unescape("isASymbol%3A%20aString%0D%0A%09%22%20Private%20-%20Return%20true%20if%20aString%20can%20be%20output%20with%20symbol%20notation.%20%22%0D%0A%0D%0A%09%5E%28%28RegularExpression%20fromString%3A%20%27%5B%5Ea-zA-Z0-9_%3A%5D%27%29%20exec%3A%20aString%29%20isNil"));
smalltalk.bind(smalltalk.Exporter,"fileOutMethod:on:",0
,function(aMethod,aStream){var self=this;
return self.fileOutMethod_of_on_(aMethod,aMethod.classField(),aStream);}
,"fileOut",unescape("fileOutMethod%3A%20aMethod%20on%3A%20aStream%0D%0A%09%22%20Return%20the%20fileOut%20of%20aMethod.%20%22%0D%0A%0D%0A%09%5Eself%0D%0A%09%09fileOutMethod%3A%20aMethod%0D%0A%09%09of%3A%20aMethod%20classField%0D%0A%09%09on%3A%20aStream"));
smalltalk.bind(smalltalk.Exporter,"fileOutMethod:",0
,function(aMethod){var self=this;
var stream=nil;
stream="".writeStream();
self.fileOutMethod_on_(aMethod,stream);
return stream.contents();}
,"fileOut",unescape("fileOutMethod%3A%20aMethod%0D%0A%09%22%20Return%20the%20fileOut%20of%20aMethod.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09self%20fileOutMethod%3A%20aMethod%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"fileOutMethod:of:on:",0
,function(aMethod,aClass,stream){var self=this;
var stamp=nil;
stamp=((aMethod.stamp().size()._gt((0))).mustBeBoolean()==true ? (function(){return " stamp: "._comma(aMethod.stamp().asLiteral());})() : (function(){return "";})());
(function($rec){$rec.cr();$rec.nextChunkPut_(nil);$rec.nextChunkPut_(" "._comma(self.globalNameFor_(aClass))._comma(" methodsFor: ")._comma(aMethod.category().asLiteral())._comma(stamp)._comma(" "));$rec.cr();$rec.nextChunkPut_(aMethod.source());$rec.nextChunkPut_(" ");return $rec.cr();})(stream);
return self;}
,"fileOut",unescape("fileOutMethod%3A%20aMethod%20of%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20FileOut%20method%20of%20a%20class%20onto%20stream.%20%22%0D%0A%0D%0A%09%7C%20stamp%20%7C%0D%0A%09stamp%20%3A%3D%20aMethod%20stamp%20size%20%3E%200%0D%0A%09%09ifTrue%3A%20%5B%20%27%20stamp%3A%20%27%20%2C%20aMethod%20stamp%20asLiteral%20%5D%0D%0A%09%09ifFalse%3A%20%5B%27%27%5D.%0D%0A%09stream%09cr%3B%20nextChunkPut%3A%20nil%3B%0D%0A%09%09nextChunkPut%3A%20%27%20%27%0D%0A%09%09%09%2C%28self%20globalNameFor%3A%20aClass%29%0D%0A%09%09%09%2C%27%20methodsFor%3A%20%27%20%2CaMethod%20category%20asLiteral%0D%0A%09%09%09%2Cstamp%20%2C%27%20%27%3B%20cr%3B%0D%0A%09%09nextChunkPut%3A%20aMethod%20source%3B%0D%0A%09%09nextChunkPut%3A%20%27%20%27%3B%20cr"));
smalltalk.bind(smalltalk.Exporter,"exportCategory:",0
,function(aString){var self=this;
var stream=nil;
var match=nil;
stream="".writeStream();
match=self.matchCategory_(aString);
match.first().do_((function(each){return stream.nextPutAll_(self.export_(each));}));
match.last().do_((function(aMethod){return self.export_method_on_(aMethod.classField(),aMethod,stream);}));
return stream.contents();}
,"export",unescape("exportCategory%3A%20aString%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20all%20the%20classes%20in%20category%20aString.%20%22%0D%0A%0D%0A%09%7C%20stream%20match%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09match%20%3A%3D%20self%20matchCategory%3A%20aString.%0D%0A%09match%20first%20do%3A%20%5B%3Aeach%7C%20stream%20nextPutAll%3A%20%28self%20export%3A%20each%29%20%5D.%0D%0A%09match%20last%20do%3A%20%5B%3AaMethod%20%7C%0D%0A%09%09self%20export%3A%20aMethod%20classField%20method%3A%20aMethod%20on%3A%20stream%0D%0A%09%5D.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"exportCategories:",0
,function(categories){var self=this;
if (categories.isString()){return self.exportCategories_(categories.asArrayOfSubstrings());};;
return categories.asArray().inject_into_([],(function(total,each){return (function($rec){$rec.add_(self.exportCategory_(each));return $rec.yourself();})(total);})).join_("");}
,"export",unescape("exportCategories%3A%20categories%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20all%20the%20classes%20in%20categories.%20%22%0D%0A%0D%0A%09categories%20isString%20ifTrue%3A%20%5B%20%5Eself%20exportCategories%3A%20categories%20asArrayOfSubstrings%20%5D.%0D%0A%09%5E%28categories%20asArray%20inject%3A%20%23%28%29%20into%3A%20%5B%3Atotal%20%3Aeach%7C%0D%0A%09%09total%20add%3A%20%28self%20exportCategory%3A%20each%29%3B%20yourself%0D%0A%09%5D%29%20join%3A%20%27%27"));
smalltalk.bind(smalltalk.Exporter,"exportAllMethodsOf:on:",0
,function(aClass,stream){var self=this;
self.exportMethodsOf_on_(aClass.$klass,stream);
self.exportMethodsOf_on_(aClass,stream);
return self;}
,"export",unescape("exportAllMethodsOf%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20Export%20all%20methods%20of%20a%20class%20%28and%20metaclass%29%20onto%20stream.%20%22%0D%0A%0D%0A%09self%20exportMethodsOf%3A%20aClass%20class%20on%3A%20stream.%0D%0A%09self%20exportMethodsOf%3A%20aClass%20on%3A%20stream."));
smalltalk.bind(smalltalk.Exporter,"exportAllMethodsOn:",0
,function(stream){var self=this;
self.allClasses().do_((function(aClass){return self.exportAllMethodsOf_on_(aClass,stream);}));
return self;}
,"export",unescape("exportAllMethodsOn%3A%20stream%0D%0A%09%22%20Export%20all%20methods%20in%20the%20system%20onto%20stream.%20%22%0D%0A%0D%0A%09self%20allClasses%20do%3A%20%5B%3AaClass%7C%0D%0A%09%09self%20exportAllMethodsOf%3A%20aClass%20on%3A%20stream%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Exporter,"exportAllMethods",0
,function(){var self=this;
var stream=nil;
stream="".writeStream();
self.exportAllMethodsOn_(stream);
return stream.contents();}
,"export",unescape("exportAllMethods%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20all%20methods%20in%20the%20system.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09self%20exportAllMethodsOn%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"exportAllDefinitionsOn:",0
,function(stream){var self=this;
self.allClasses().do_((function(aClass){return self.exportDefinitionOfClass_on_(aClass,stream);}));
return self;}
,"export",unescape("exportAllDefinitionsOn%3A%20stream%0D%0A%09%22%20Export%20all%20class%20definitions%20in%20the%20system%20onto%20stream.%20%22%0D%0A%0D%0A%09self%20allClasses%20do%3A%20%5B%3AaClass%7C%0D%0A%09%09self%20exportDefinitionOfClass%3A%20aClass%20on%3A%20stream%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Exporter,"exportAllDefinitions",0
,function(){var self=this;
var stream=nil;
stream="".writeStream();
self.exportAllDefinitionsOn_(stream);
return stream.contents();}
,"export",unescape("exportAllDefinitions%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20all%20class%20definitions%20in%20the%20system.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09self%20exportAllDefinitionsOn%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"exportDefinitionOfClass:on:",0
,function(aClass,stream){var self=this;
self.exportDefinitionOf_on_(aClass,stream);
self.exportMetaDefinitionOf_on_(aClass,stream);
return self;}
,"export",unescape("exportDefinitionOfClass%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20Export%20definition%20of%20aClass%20%28and%20metaclass%29%20onto%20stream.%20%22%0D%0A%0D%0A%09self%20exportDefinitionOf%3A%20aClass%20on%3A%20stream.%0D%0A%09self%20exportMetaDefinitionOf%3A%20aClass%20on%3A%20stream."));
smalltalk.bind(smalltalk.Exporter,"export:",0
,function(aClass){var self=this;
var stream=nil;
stream="".writeStream();
self.exportDefinitionOfClass_on_(aClass,stream);
self.exportAllMethodsOf_on_(aClass,stream);
return stream.contents();}
,"export",unescape("export%3A%20aClass%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20definition%20and%20source%20code%20of%20a%20class.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09self%20exportDefinitionOfClass%3A%20aClass%20on%3A%20stream.%0D%0A%09self%20exportAllMethodsOf%3A%20aClass%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"exportNew:",0
,function(aClass){var self=this;
return self.exportDefinitionOf_(aClass);}
,"export",unescape("exportNew%3A%20aClass%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20definition%20of%20aClass.%20%22%0D%0A%0D%0A%09%5Eself%20exportDefinitionOf%3A%20aClass"));
smalltalk.bind(smalltalk.Exporter,"exportCommentOf:",0
,function(aClass){var self=this;
var stream=nil;
stream="".writeStream();
self.exportCommentOf_on_(aClass,stream);
return stream.contents();}
,"export",unescape("exportCommentOf%3A%20aClass%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20comment%20of%20a%20class.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20writeStream.%0D%0A%09self%20exportCommentOf%3A%20aClass%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"exportCommentOf:on:",0
,function(aClass,aStream){var self=this;
return self.encoderFor_(aClass).exportCommentOf_on_(aClass,aStream);}
,"private",unescape("exportCommentOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20comment%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%5E%28self%20encoderFor%3A%20aClass%29%20exportCommentOf%3A%20aClass%20on%3A%20aStream"));
smalltalk.bind(smalltalk.Exporter,"exportDefinitionOf:on:",0
,function(aClass,aStream){var self=this;
return self.encoderFor_(aClass).exportDefinitionOf_on_(aClass,aStream);}
,"private",unescape("exportDefinitionOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%5E%28self%20encoderFor%3A%20aClass%29%20exportDefinitionOf%3A%20aClass%20on%3A%20aStream"));
smalltalk.bind(smalltalk.Exporter,"exportMetaDefinitionOf:on:",0
,function(aClass,aStream){var self=this;
return self.encoderFor_(aClass).exportMetaDefinitionOf_on_(aClass,aStream);}
,"private",unescape("exportMetaDefinitionOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20class%20side%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%5E%28self%20encoderFor%3A%20aClass%29%20exportMetaDefinitionOf%3A%20aClass%20on%3A%20aStream"));
smalltalk.bind(smalltalk.Exporter,"smalltalkNameFor:",0
,function(aClass){var self=this;
return self.encoderFor_(aClass).smalltalkNameFor_(aClass);}
,"private",unescape("smalltalkNameFor%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20native%20reference%20expression%20of%20aClass%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5E%28self%20encoderFor%3A%20aClass%29%20smalltalkNameFor%3A%20aClass"));
smalltalk.bind(smalltalk.Exporter,"globalNameFor:",0
,function(aClass){var self=this;
return (((nil.isNil_(aClass))).mustBeBoolean()==true ? (function(){return "nil";})() : (function(){return aClass.name();})());}
,"private",unescape("globalNameFor%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20smalltalk%20name%20of%20aClass%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5EaClass%20isNil%20ifTrue%3A%20%5B%20%23nil%20%5D%20ifFalse%3A%20%5B%20aClass%20name%20%5D"));
smalltalk.bind(smalltalk.Exporter,"exportAll",0
,function(){var self=this;
return self.exportClasses_(self.allClasses());}
,"export",unescape("exportAll%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20all%20classes%20in%20the%20system.%20%22%0D%0A%0D%0A%09%5Eself%20exportClasses%3A%20self%20allClasses"));
smalltalk.bind(smalltalk.Exporter,"exportAllCategories",0
,function(){var self=this;
var categories=nil;
categories=smalltalk.Array.$$new();
self.allClasses().do_((function(each){return ((categories.includes_(each.category())).mustBeBoolean()==true ? nil : (function(){return categories.add_(each.category());})());}));
return categories.inject_into_([],(function(total,each){return (function($rec){$rec.add_(self.exportCategory_(each));return $rec.yourself();})(total);})).join_("");}
,"export",unescape("exportAllCategories%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20all%20categories%20in%20the%20system.%20%22%0D%0A%0D%0A%09%7C%20categories%20%7C%0D%0A%09categories%20%3A%3D%20Array%20new.%0D%0A%09self%20allClasses%20do%3A%20%5B%3Aeach%20%7C%0D%0A%09%09%28categories%20includes%3A%20each%20category%29%20ifFalse%3A%20%5B%0D%0A%09%09%09categories%20add%3A%20each%20category%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5E%28categories%20inject%3A%20%23%28%29%20into%3A%20%5B%3Atotal%20%3Aeach%20%7C%20total%20add%3A%20%28self%20exportCategory%3A%20each%29%3B%20yourself%20%5D%29%20join%3A%20%27%27"));
smalltalk.bind(smalltalk.Exporter,"exportDefinitionOf:",0
,function(aClass){var self=this;
var stream=nil;
stream=smalltalk.String.$$new().writeStream();
self.exportDefinitionOf_on_(aClass,stream);
return stream.contents();}
,"export",unescape("exportDefinitionOf%3A%20aClass%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20definition%20of%20aClass.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20String%20new%20writeStream.%0D%0A%09self%20exportDefinitionOf%3A%20aClass%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"exportMetaDefinitionOf:",0
,function(aClass){var self=this;
var stream=nil;
stream=smalltalk.String.$$new().writeStream();
self.exportMetaDefinitionOf_on_(aClass,stream);
return stream.contents();}
,"export",unescape("exportMetaDefinitionOf%3A%20aClass%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20class%20side%20definition%20of%20aClass.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20String%20new%20writeStream.%0D%0A%09self%20exportMetaDefinitionOf%3A%20aClass%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"exportClasses:on:",0
,function(classes,stream){var self=this;
return self.exportAll_on_(classes,stream);}
,"export",unescape("exportClasses%3A%20classes%20on%3A%20stream%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20class%20definitions%20onto%20stream.%20%22%0D%0A%0D%0A%09%5Eself%20exportAll%3A%20classes%20on%3A%20stream"));
smalltalk.bind(smalltalk.Exporter,"exportAll:on:",0
,function(elements,stream){var self=this;
if (elements.isString()){return self.exportAll_on_(elements.asArrayOfSubstrings(),stream);};;
elements.do_((function(each){return ((self.isBehavior_(each)).mustBeBoolean()==true ? (function(){return self.exportDefinitionOfClass_on_(each,stream);})() : nil);}));
stream.cr();
elements.do_((function(each){return ((self.isBehavior_(each)).mustBeBoolean()==true ? (function(){return self.exportAllMethodsOf_on_(each,stream);})() : (function(){return ((each.isString()).mustBeBoolean()==true ? (function(){return (function($rec){$rec.nextPutAll_(self.exportCategory_(each));return $rec.cr();})(stream);})() : (function(){return self.exportMethod_on_(each,stream);})());})());}));
return self;}
,"export",unescape("exportAll%3A%20elements%20on%3A%20stream%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20elements%20%28classes%2C%20methods%20and/or%20categories%29%20onto%20stream.%20%22%0D%0A%0D%0A%09elements%20isString%20ifTrue%3A%20%5B%20%5Eself%20exportAll%3A%20elements%20asArrayOfSubstrings%20on%3A%20stream%20%5D.%0D%0A%09elements%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28self%20isBehavior%3A%20each%29%20ifTrue%3A%20%5B%0D%0A%09%09%09self%20exportDefinitionOfClass%3A%20each%20on%3A%20stream%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09stream%20cr.%0D%0A%09elements%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28self%20isBehavior%3A%20each%29%20ifTrue%3A%20%5B%0D%0A%09%09%09self%20exportAllMethodsOf%3A%20each%20on%3A%20stream%0D%0A%09%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09each%20isString%0D%0A%09%09%09ifTrue%3A%20%5B%20stream%20nextPutAll%3A%20%28self%20exportCategory%3A%20each%29%3Bcr%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20self%20exportMethod%3A%20each%20on%3A%20stream%20%5D%0D%0A%09%09%5D%0D%0A%09%5D."));
smalltalk.bind(smalltalk.Exporter,"exportAll:",0
,function(elements){var self=this;
var stream=nil;
stream=smalltalk.String.$$new().writeStream();
self.exportAll_on_(elements,stream);
return stream.contents();}
,"export",unescape("exportAll%3A%20elements%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20elements%20%28classes%2C%20methods%20and/or%20categories%29.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20String%20new%20writeStream.%0D%0A%09self%20exportAll%3A%20elements%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"exportClasses:",0
,function(classes){var self=this;
var stream=nil;
stream=smalltalk.String.$$new().writeStream();
self.exportClasses_on_(classes,stream);
return stream.contents();}
,"export",unescape("exportClasses%3A%20classes%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20classes.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20String%20new%20writeStream.%0D%0A%09self%20exportClasses%3A%20classes%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"exportObject:on:",0
,function(anObject,stream){var self=this;
stream.nextPutAll_(anObject.asNative());
return self;}
,"export",unescape("exportObject%3A%20anObject%20on%3A%20stream%0D%0A%09%22%20Export%20a%20%28native%29literal%20representation%20of%20anObject%20onto%20stream.%20%22%0D%0A%0D%0A%09stream%20nextPutAll%3A%20anObject%20asNative"));
smalltalk.bind(smalltalk.Exporter,"exportObject:",0
,function(anObject){var self=this;
var stream=nil;
stream=smalltalk.String.$$new().writeStream();
self.exportObject_on_(anObject,stream);
return stream.contents();}
,"export",unescape("exportObject%3A%20anObject%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20a%20literal%20representation%20of%20anObject.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20String%20new%20writeStream.%0D%0A%09self%20exportObject%3A%20anObject%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"export:method:",0
,function(aClass,aMethod){var self=this;
var stream=nil;
stream=smalltalk.String.$$new().writeStream();
self.export_method_on_(aClass,aMethod,stream);
return stream.contents();}
,"export",unescape("export%3A%20aClass%20method%3A%20aMethod%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20aClass%20method.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20String%20new%20writeStream.%0D%0A%09self%20export%3A%20aClass%20method%3A%20aMethod%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"exportMethod:",0
,function(aMethod){var self=this;
return self.export_method_(aMethod.classField(),aMethod);}
,"export",unescape("exportMethod%3A%20aMethod%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20aMethod.%20%22%0D%0A%0D%0A%09%5Eself%20export%3A%20aMethod%20classField%20method%3A%20aMethod"));
smalltalk.bind(smalltalk.Exporter,"exportMethod:on:",0
,function(aMethod,aStream){var self=this;
return self.export_method_on_(aMethod.classField(),aMethod,aStream);}
,"export",unescape("exportMethod%3A%20aMethod%20on%3A%20aStream%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20aMethod%20on%20stream.%20%22%0D%0A%0D%0A%22%09%28aMethod%20respondsTo%3A%20%23classField%29%20ifFalse%3A%20%5B%20%5Eself%20note%3A%20%27Warning%3A%20broken%20method%20%27%2CaMethod%20%5D.%0D%0A%22%09%5Eself%20export%3A%20aMethod%20classField%20method%3A%20aMethod%20on%3A%20aStream"));
smalltalk.bind(smalltalk.Exporter,"exportMethods:on:",0
,function(aCollection,aStream){var self=this;
aCollection.do_((function(each){return self.exportMethod_on_(each,aStream);}));
return self;}
,"export",unescape("exportMethods%3A%20aCollection%20on%3A%20aStream%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20aCollection%20of%20methods%20on%20stream.%20%22%0D%0A%0D%0A%09aCollection%20do%3A%20%5B%3Aeach%7C%20self%20exportMethod%3A%20each%20on%3A%20aStream%20%5D"));
smalltalk.bind(smalltalk.Exporter,"exportMethods:",0
,function(aCollection){var self=this;
var stream=nil;
stream=smalltalk.String.$$new().writeStream();
self.exportMethods_on_(aCollection,stream);
return stream.contents();}
,"Tools-export",unescape("exportMethods%3A%20aCollection%0D%0A%09%22%20Return%20the%20result%20of%20exporting%20aCollection%20of%20methods.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20String%20new%20writeStream.%0D%0A%09self%20exportMethods%3A%20aCollection%20on%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Exporter,"exportMethodsOf:on:",0
,function(aClass,aStream){var self=this;
aClass.methodsDo_((function(each){return self.export_method_on_(aClass,each,aStream);}));
return self;}
,"Tools-private",unescape("exportMethodsOf%3A%20aClass%20on%3A%20aStream%0D%0A%09%22%20Export%20methods%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09aClass%20methodsDo%3A%20%5B%3Aeach%7C%0D%0A%09%09self%20export%3A%20aClass%20method%3A%20each%20on%3A%20aStream%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Exporter,"export:method:on:",0
,function(aClass,aMethod,aStream){var self=this;
if (smalltalk.responds_to_(aMethod,("exportFor:on:"))){return aMethod.exportFor_on_(self,aStream);};;
return self.encoderFor_(aClass).export_method_on_(aClass,aMethod,aStream);}
,"Tools-export",unescape("export%3A%20aClass%20method%3A%20aMethod%20on%3A%20aStream%0D%0A%09%22%20Export%20a%20method%20definition%20of%20aClass%20onto%20aStream.%20%22%0D%0A%0D%0A%09%28aMethod%20respondsTo%3A%20%23exportFor%3Aon%3A%29%20ifTrue%3A%20%5B%20%5EaMethod%20exportFor%3A%20self%20on%3A%20aStream%20%5D.%0D%0A%09%5E%28self%20encoderFor%3A%20aClass%29%20export%3A%20aClass%20method%3A%20aMethod%20on%3A%20aStream"));
smalltalk.bind(smalltalk.MethodReferences.$klass,"constructEventsTriggered",0
,function(){var self=this;
return ["found:", "found:pattern:in:", "needsMethods", "searchStarted", "searchFinished"]._comma(smalltalk.superImplementor(smalltalk.MethodReferences.$klass,"constructEventsTriggered").apply(self, []));}
,"Tools-events",unescape("constructEventsTriggered%0D%0A%09%22%20Private%20-%20Return%20the%20events%20managed%20by%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%09%23found%3A%20%23found%3Apattern%3Ain%3A%0D%0A%09%09%23needsMethods%0D%0A%09%09%23searchStarted%20%23searchFinished%0D%0A%09%29%20%2C%20super%20constructEventsTriggered"));
smalltalk.bind(smalltalk.MethodReferences.$klass,"of:",0
,function(selector){var self=this;
return self.$$new().referencesOf_(selector);}
,"Tools-testing",unescape("of%3A%20selector%0D%0A%09%22%20Return%20the%20references%20to%20selector.%20%22%0D%0A%0D%0A%09%5Eself%20new%20referencesOf%3A%20selector"));
smalltalk.bind(smalltalk.MethodReferences.$klass,"ofGlobal:",0
,function(globalName){var self=this;
return self.$$new().referencesOfGlobal_(globalName);}
,"Tools-testing",unescape("ofGlobal%3A%20globalName%0D%0A%09%22%20Return%20the%20references%20to%20global%20globalName.%20%22%0D%0A%0D%0A%09%5Eself%20new%20referencesOfGlobal%3A%20globalName"));
smalltalk.bind(smalltalk.MethodReferences.$klass,"findClasses:",0
,function(what){var self=this;
return self.$$new().findClasses_(what);}
,"Tools-testing",unescape("findClasses%3A%20what%0D%0A%09%22%20Find%20classes%20in%20the%20system%20matching%20what.%20%22%0D%0A%0D%0A%09%5Eself%20new%20findClasses%3A%20what"));
smalltalk.bind(smalltalk.MethodReferences.$klass,"globals",0
,function(){var self=this;
var tool=nil;
var result=nil;
tool=self.$$new();
result=smalltalk.PoolDictionary.$$new();
tool.when_do_("found:pattern:in:",(function(mth,mask,stream){var global=nil;
global=smalltalk.SmalltalkParser.$$new().globalNameParser().parse_(stream);return ((global.isParseFailure()).mustBeBoolean()==true ? nil : (function(){return result.at_ifAbsentPut_(global,(function(){return smalltalk.Array.$$new();})).add_(mth);})());}));
tool.searchGlobalReferences();
return result;}
,"Tools-testing",unescape("globals%0D%0A%09%22%20Return%20the%20references%20to%20all%20global%20names.%20%22%0D%0A%0D%0A%09%7C%20tool%20result%20%7C%0D%0A%09tool%20%3A%3D%20self%20new.%0D%0A%09result%20%3A%3D%20PoolDictionary%20new.%0D%0A%09tool%20when%3A%20%23found%3Apattern%3Ain%3A%20do%3A%20%5B%3Amth%20%3Amask%20%3Astream%7C%20%7C%20global%20%7C%0D%0A%09%09global%20%3A%3D%20SmalltalkParser%20new%20globalNameParser%20parse%3A%20stream.%0D%0A%09%09global%20isParseFailure%20ifFalse%3A%20%5B%0D%0A%09%09%09%28result%20at%3A%20global%20ifAbsentPut%3A%20%5BArray%20new%5D%29%20add%3A%20mth%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09tool%20searchGlobalReferences.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.MethodReferences.$klass,"methodOverwrites",0
,function(){var self=this;
return self.$$new().methodOverwrites();}
,"Tools-testing",unescape("methodOverwrites%0D%0A%09%22%20Find%20the%20methods%20that%20are%20implemented/overwritten%20by%20subclasses.%20%22%0D%0A%0D%0A%09%5Eself%20new%20methodOverwrites"));
smalltalk.bind(smalltalk.MethodReferences.$klass,"variableOverwrites",0
,function(){var self=this;
return self.$$new().variableOverwrites();}
,"Tools-testing",unescape("variableOverwrites%0D%0A%09%22%20Find%20the%20methods%20that%20has%20potential%20overwrite%20of%20variable%20names.%20%22%0D%0A%0D%0A%09%5Eself%20new%20variableOverwrites"));
smalltalk.bind(smalltalk.MethodReferences.$klass,"findMessages:",0
,function(what){var self=this;
return self.$$new().findMessages_(what);}
,"Tools-testing",unescape("findMessages%3A%20what%0D%0A%09%22%20Find%20messages%20in%20the%20system%20matching%20what.%20%22%0D%0A%0D%0A%09%5Eself%20new%20findMessages%3A%20what"));
smalltalk.bind(smalltalk.MethodReferences,"initializeResults",0
,function(){var self=this;
self["@results"]=smalltalk.Array.$$new();
return self;}
,"Tools-initialize",unescape("initializeResults%0D%0A%09%22%20Private%20-%20Initialize%20the%20results%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09results%20%3A%3D%20Array%20new"));
smalltalk.bind(smalltalk.MethodReferences,"reset",0
,function(){var self=this;
self.results_(nil);
return self;}
,"Tools-initialize",unescape("reset%0D%0A%09%22%20Reset%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20results%3A%20nil"));
smalltalk.bind(smalltalk.MethodReferences,"rootClasses:",0
,function(array){var self=this;
self["@rootClasses"]=array;
return self;}
,"Tools-accessing",unescape("rootClasses%3A%20array%0D%0A%09%22%20Set%20the%20rootClasses%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09rootClasses%20%3A%3D%20array"));
smalltalk.bind(smalltalk.MethodReferences,"results",0
,function(){var self=this;
if ((nil.isNil_(self["@results"]))){self.initializeResults();};;
return self["@results"];}
,"accessing",unescape("results%0D%0A%09%22%20Return%20the%20results%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09results%20isNil%20ifTrue%3A%20%5B%20self%20initializeResults%20%5D.%0D%0A%09%5Eresults"));
smalltalk.bind(smalltalk.MethodReferences,"results:",0
,function(array){var self=this;
self["@results"]=array;
return self;}
,"Tools-accessing",unescape("results%3A%20array%0D%0A%09%22%20Set%20the%20results%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09results%20%3A%3D%20array"));
smalltalk.bind(smalltalk.MethodReferences,"rootClasses",0
,function(){var self=this;
if ((nil.isNil_(self["@rootClasses"]))){return smalltalk.Smalltalk.rootClasses();};;
return self["@rootClasses"];}
,"Tools-accessing",unescape("rootClasses%0D%0A%09%22%20Return%20the%20rootClasses%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09rootClasses%20isNil%20ifTrue%3A%20%5B%20%5ESmalltalk%20rootClasses%20%5D.%0D%0A%09%5ErootClasses"));
smalltalk.bind(smalltalk.MethodReferences,"allMethodsIn:",0
,function(aCollection){var self=this;
var result=nil;
result=smalltalk.Array.$$new();
self.rootClasses().do_((function(root){return root.instanceClass().withAllSubclasses().do_((function(cls){cls.methodsDo_((function(each){return result.add_(each);}));return cls.$klass.methodsDo_((function(each){return result.add_(each);}));}));}));
return result;}
,"Tools-accessing",unescape("allMethodsIn%3A%20aCollection%0D%0A%09%22%20Return%20the%20methods%20defined%20by%20aCollection%20of%20classes.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20Array%20new.%0D%0A%09self%20rootClasses%20do%3A%20%5B%3Aroot%7C%0D%0A%09%09root%20instanceClass%20withAllSubclasses%20do%3A%20%5B%3Acls%7C%0D%0A%09%09%09cls%20methodsDo%3A%20%5B%3Aeach%7C%20result%20add%3A%20each%20%5D.%0D%0A%09%09%09cls%20class%20methodsDo%3A%20%5B%3Aeach%7C%20result%20add%3A%20each%20%5D.%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.MethodReferences,"allMethods",0
,function(){var self=this;
return self.triggerEvent_ifNotHandled_("needsMethods",(function(){return self.allMethodsIn_(self.rootClasses());}));}
,"Tools-accessing",unescape("allMethods%0D%0A%09%22%20Return%20the%20methods%20to%20search%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%09triggerEvent%3A%20%23needsMethods%0D%0A%09%09ifNotHandled%3A%20%5B%20self%20allMethodsIn%3A%20self%20rootClasses%20%5D"));
smalltalk.bind(smalltalk.MethodReferences,"search:with:",0
,function(what,targetBlock){var self=this;
var atoms=nil;
atoms=self.allMethods();
self.triggerEvent_("searchStarted");
atoms=atoms.select_((function(aMethod){var source=nil;
source=targetBlock.value_(aMethod);return (((nil.isNil_(source)===false).and_((function(){return source.toString().includes_(what);}))).mustBeBoolean()==true ? (function(){self.results().add_(aMethod);self.triggerEvent_with_("found:",aMethod);if (self.hasActionForEvent_("found:pattern:in:")){var stream=nil;
var args=nil;
stream=source.toString().stream();
args=smalltalk.Array.with_with_with_(aMethod,what,stream);
(function(){while(!(function(){return (function($rec){$rec.upTo_(what);return $rec.atEnd();})(stream);})()){(function(){return self.triggerEvent_withArguments_("found:pattern:in:",args);})()};return nil})();};;return true;})() : (function(){return false;})());}));
self.triggerEvent_("searchFinished");
return atoms;}
,"Tools-searching",unescape("search%3A%20what%20with%3A%20targetBlock%0D%0A%09%22%20Find%20what%20on%20target%20of%20selected%20methods.%20%22%0D%0A%0D%0A%09%7C%20atoms%20%7C%0D%0A%09atoms%20%3A%3D%20self%20allMethods.%0D%0A%09self%20triggerEvent%3A%20%23searchStarted.%0D%0A%09atoms%20%3A%3D%20atoms%20select%3A%20%5B%3AaMethod%7C%20%7C%20source%20%7C%0D%0A%09%09source%20%3A%3D%20targetBlock%20value%3A%20aMethod.%0D%0A%09%09%28source%20notNil%20and%3A%20%5Bsource%20toString%20includes%3A%20what%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%09self%20results%20add%3A%20aMethod.%0D%0A%09%09%09self%20triggerEvent%3A%20%23found%3A%20with%3A%20aMethod.%0D%0A%09%09%09%28self%20hasActionForEvent%3A%20%23found%3Apattern%3Ain%3A%29%20ifTrue%3A%20%5B%20%7C%20stream%20args%20%7C%0D%0A%09%09%09%09stream%20%3A%3D%20source%20toString%20stream.%0D%0A%09%09%09%09args%20%3A%3D%20Array%20with%3A%20aMethod%20with%3A%20what%20with%3A%20stream.%0D%0A%09%09%09%09%5B%20stream%20upTo%3A%20what%3B%20atEnd%20%5D%20whileFalse%3A%20%5B%0D%0A%09%09%09%09%09self%20triggerEvent%3A%20%23found%3Apattern%3Ain%3A%20withArguments%3A%20args%0D%0A%09%09%09%09%5D%0D%0A%09%09%09%5D.%0D%0A%09%09%09true%0D%0A%09%09%5D%20ifFalse%3A%20%5B%20false%20%5D%0D%0A%09%5D.%0D%0A%09self%20triggerEvent%3A%20%23searchFinished.%0D%0A%09%5Eatoms"));
smalltalk.bind(smalltalk.MethodReferences,"search:",0
,function(what){var self=this;
return self.search_with_(what,(function(aMethod){return aMethod.compiledSource();}));}
,"Tools-searching",unescape("search%3A%20what%0D%0A%09%22%20Find%20references%20to%20what%20in%20selected%20methods.%0D%0A%09Note%3A%20search%20is%20realized%20on%20native%20source%20of%20native%20code%20of%20methods.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20search%3A%20what%20with%3A%20%5B%3AaMethod%7C%20aMethod%20compiledSource%20%5D"));
smalltalk.bind(smalltalk.MethodReferences,"searchSource:",0
,function(what){var self=this;
return self.search_with_(what,(function(aMethod){return aMethod.source();}));}
,"Tools-searching",unescape("searchSource%3A%20what%0D%0A%09%22%20Find%20what%20in%20source%20of%20selected%20methods.%20%22%0D%0A%0D%0A%09%5Eself%20search%3A%20what%20with%3A%20%5B%3AaMethod%7C%20aMethod%20source%20%5D"));
smalltalk.bind(smalltalk.MethodReferences,"inlines",0
,function(){var self=this;
return self.searchSource_(unescape("%7B")._comma(unescape("%27")));}
,"Tools-searching",unescape("inlines%0D%0A%09%22%20Find%20native%20inlines%20in%20selected%20methods.%20%22%0D%0A%0D%0A%09%5Eself%20searchSource%3A%20%24%7B%20%2C%24%27%20"));
smalltalk.bind(smalltalk.MethodReferences,"searchCategory:",0
,function(aName){var self=this;
var found=nil;
self.triggerEvent_("searchStarted");
found=smalltalk.Exporter.$$new().matchCategory_from_(aName,self.rootClasses());
found=found.first()._comma(found.last());
found.do_((function(each){self.results().add_(each);return self.triggerEvent_with_("found:",each);}));
self.triggerEvent_("searchFinished");
return found;}
,"Tools-searching",unescape("searchCategory%3A%20aName%0D%0A%09%22%20Find%20%28source%29%20references%20to%20category%20starting%20from%20root%20classes.%20%22%0D%0A%0D%0A%09%7C%20found%20%7C%0D%0A%09self%20triggerEvent%3A%20%23searchStarted.%0D%0A%09found%20%3A%3D%20Exporter%20new%20matchCategory%3A%20aName%20from%3A%20self%20rootClasses.%0D%0A%09found%20%3A%3D%20found%20first%20%2C%20found%20last.%0D%0A%09found%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%09self%20results%20add%3A%20each.%0D%0A%09%09%09self%20triggerEvent%3A%20%23found%3A%20with%3A%20each.%0D%0A%09%5D.%0D%0A%09self%20triggerEvent%3A%20%23searchFinished.%0D%0A%09%5Efound"));
smalltalk.bind(smalltalk.MethodReferences,"sendPatternsOf:",0
,function(aSelector){var self=this;
return smalltalk.Array.with_with_(self.normalSendPatternOf_(aSelector),self.superSendPatternOf_(aSelector));}
,"Tools-patterns",unescape("sendPatternsOf%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20the%20search%20patterns%20for%20references%20of%20aSelector.%20%22%0D%0A%0D%0A%09%5EArray%0D%0A%09%09with%3A%20%28self%20normalSendPatternOf%3A%20aSelector%29%0D%0A%09%09with%3A%20%28self%20superSendPatternOf%3A%20aSelector%29"));
smalltalk.bind(smalltalk.MethodReferences,"dot",0
,function(){var self=this;
return ".";}
,"Tools-patterns",unescape("dot%0D%0A%09%22%20Private%20-%20Return%20the%20message%20send%20pattern%20starter.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%7D"));
smalltalk.bind(smalltalk.MethodReferences,"localDef",0
,function(){var self=this;
return "var ";}
,"Tools-patterns",unescape("localDef%0D%0A%09%22%20Private%20-%20Return%20the%20variable%20definition%20starter.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%7D"));
smalltalk.bind(smalltalk.MethodReferences,"normalSendStarter:",0
,function(aSelector){var self=this;

return self.dot();}
,"Tools-patterns",unescape("normalSendStarter%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20the%20pattern%20for%20detecting%20normal%20senders%20of%20aSelector.%20%22%0D%0A%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Eself%20dot"));
smalltalk.bind(smalltalk.MethodReferences,"normalSendPatternOf:",0
,function(aSelector){var self=this;
return self.normalSendStarter_(aSelector)._comma(aSelector.asSelector())._comma(unescape("%28"));}
,"Tools-patterns",unescape("normalSendPatternOf%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20the%20search%20pattern%20for%20normal%20senders%20of%20aSelector.%20%22%0D%0A%0D%0A%09%5E%28self%20normalSendStarter%3A%20aSelector%29%0D%0A%09%09%2CaSelector%20asSelector%0D%0A%09%09%2C%24%28%20"));
smalltalk.bind(smalltalk.MethodReferences,"superJSSendPatternOf:",0
,function(aSelector){var self=this;
return unescape("%22")._comma(aSelector.asSelector())._comma(unescape("%22"))._comma(unescape("%29.apply%28self%2C"));}
,"Tools-patterns",unescape("superJSSendPatternOf%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20the%20search%20pattern%20for%20senders%20%28to%20super%29%20of%20aSelector.%20%22%0D%0A%0D%0A%09%5E%24%22%20%2CaSelector%20asSelector%20%2C%24%22%20%2C%27%29.apply%28self%2C%27"));
smalltalk.bind(smalltalk.MethodReferences,"superSendPatternOf:",0
,function(aSelector){var self=this;
return self.superJSSendPatternOf_(aSelector);}
,"Tools-patterns",unescape("superSendPatternOf%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20the%20search%20pattern%20for%20senders%20%28to%20super%29%20of%20aSelector.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%7D"));
smalltalk.bind(smalltalk.MethodReferences,"instVarPatternOf:",0
,function(instVarName){var self=this;
return unescape("self%5B%22@")._comma(instVarName.asNativeName())._comma(unescape("%22%5D"));}
,"Tools-patterns",unescape("instVarPatternOf%3A%20instVarName%0D%0A%09%22%20Private%20-%20Return%20the%20search%20pattern%20for%20instance%20variable%20access.%20%22%0D%0A%0D%0A%09%5E%27self%5B%22@%27%20%2CinstVarName%20asNativeName%20%2C%27%22%5D%27"));
smalltalk.bind(smalltalk.MethodReferences,"globalPatternOf:",0
,function(globalName){var self=this;
return "smalltalk."._comma(globalName);}
,"Tools-patterns",unescape("globalPatternOf%3A%20globalName%0D%0A%09%22%20Private%20-%20Return%20the%20search%20pattern%20for%20global%20variable%20access.%20%22%0D%0A%0D%0A%09%5E%27smalltalk.%27%20%2CglobalName"));
smalltalk.bind(smalltalk.MethodReferences,"types",0
,function(){var self=this;
return ["referencesOf:", "referencesOfGlobal:", "referencesOfClassVariable:", "referencesOfInstanceVariable:", "findClasses:", "findMessages:", "searchCategory:", "search:", "searchSource:", "globalReferences", "missingGlobals", "inlines", "methodOverwrites", "variableOverwrites"];}
,"Tools-types",unescape("types%0D%0A%09%22%20Private%20-%20Return%20the%20types%20of%20search%20implemented%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%09%23referencesOf%3A%20%23referencesOfGlobal%3A%0D%0A%09%09%23referencesOfClassVariable%3A%20%0D%0A%09%09%23referencesOfInstanceVariable%3A%0D%0A%09%09%23findClasses%3A%20%23findMessages%3A%0D%0A%09%09%23searchCategory%3A%0D%0A%09%09%23search%3A%20%23searchSource%3A%0D%0A%09%09%23globalReferences%20%23missingGlobals%20%23inlines%0D%0A%09%09%23methodOverwrites%20%23variableOverwrites%0D%0A%09%29"));
smalltalk.bind(smalltalk.MethodReferences,"search:type:",0
,function(aName,typeSelector){var self=this;
var selector=nil;
selector=((typeSelector.includes_(":")).mustBeBoolean()==true ? (function(){return typeSelector.upTo_(":")._comma(":");})() : (function(){return typeSelector.upTo_(" ");})());
return ((smalltalk.responds_to_(self,(selector))).mustBeBoolean()==true ? (function(){return self.perform_with_(selector,aName.trimBlanks());})() : nil);}
,"Tools-searching",unescape("search%3A%20aName%20type%3A%20typeSelector%0D%0A%09%22%20Return%20the%20result%20of%20searching%20for%20aName%20as%20a%20search%20of%20specified%20type%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20selector%20%7C%0D%0A%09selector%20%3A%3D%20%28typeSelector%20includes%3A%20%24%3A%29%0D%0A%09ifTrue%3A%20%5B%20%28typeSelector%20upTo%3A%20%24%3A%29%20%2C%24%3A%20%5D%0D%0A%09ifFalse%3A%20%5B%20typeSelector%20upTo%3A%20%24%20%20%5D.%0D%0A%09%5E%28self%20respondsTo%3A%20selector%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20perform%3A%20selector%20with%3A%20aName%20trimBlanks%0D%0A%09%5D"));
smalltalk.bind(smalltalk.MethodReferences,"sendersOf:",0
,function(aSelector){var self=this;
return self.sendPatternsOf_(aSelector).inject_into_([],(function(total,each){return (function($rec){$rec.addAll_(self.search_(each));return $rec.yourself();})(total);}));}
,"Tools-searching",unescape("sendersOf%3A%20aSelector%0D%0A%09%22%20Find%20the%20requested%20elements%20from%20root%20classes.%20%22%0D%0A%0D%0A%09%5E%28self%20sendPatternsOf%3A%20aSelector%29%20inject%3A%20%23%28%29%20into%3A%20%5B%3Atotal%20%3Aeach%7C%0D%0A%09%09total%20addAll%3A%20%28self%20search%3A%20each%29%3B%20yourself%0D%0A%09%5D"));
smalltalk.bind(smalltalk.MethodReferences,"referencesOf:",0
,function(aSelector){var self=this;
(function($rec){$rec.add_(unescape("---%20Implementors%20of%20%23")._comma(aSelector)._comma(unescape("%20---")));$rec.addAll_(self.implementorsOf_(aSelector));return $rec.add_(unescape("---%20Senders%20of%20%23")._comma(aSelector)._comma(unescape("%20---")));})(self.results());
self.sendersOf_(aSelector);
return self["@results"];}
,"Tools-searching",unescape("referencesOf%3A%20aSelector%0D%0A%09%22%20Find%20the%20references%20to%20message%20%28aSelector%29.%20%22%0D%0A%0D%0A%09self%20results%0D%0A%09%09add%3A%20%27---%20Implementors%20of%20%23%27%20%2CaSelector%20%2C%27%20---%27%3B%0D%0A%09%09addAll%3A%20%28self%20implementorsOf%3A%20aSelector%29%3B%0D%0A%09%09add%3A%20%27---%20Senders%20of%20%23%27%20%2CaSelector%20%2C%27%20---%27.%0D%0A%09self%20sendersOf%3A%20aSelector.%0D%0A%09%5Eresults"));
smalltalk.bind(smalltalk.MethodReferences,"referencesOfInstanceVariable:",0
,function(instVarName){var self=this;
return self.search_(self.instVarPatternOf_(instVarName));}
,"Tools-searching",unescape("referencesOfInstanceVariable%3A%20instVarName%0D%0A%09%22%20Find%20the%20requested%20elements%20from%20root%20classes.%20%22%0D%0A%0D%0A%09%5Eself%20search%3A%20%28self%20instVarPatternOf%3A%20instVarName%29"));
smalltalk.bind(smalltalk.MethodReferences,"settersOfInstanceVariable:",0
,function(instVarName){var self=this;
return self.search_(self.instVarPatternOf_(instVarName)._comma(unescape("%3D")));}
,"Tools-searching",unescape("settersOfInstanceVariable%3A%20instVarName%0D%0A%09%22%20Find%20the%20requested%20elements%20from%20root%20classes.%20%22%0D%0A%0D%0A%09%5Eself%20search%3A%20%28self%20instVarPatternOf%3A%20instVarName%29%20%2C%27%3D%27"));
smalltalk.bind(smalltalk.MethodReferences,"referencesOfGlobal:",0
,function(globalName){var self=this;
return self.search_(self.globalPatternOf_(globalName));}
,"Tools-searching",unescape("referencesOfGlobal%3A%20globalName%0D%0A%09%22%20Find%20the%20requested%20elements%20from%20root%20classes.%20%22%0D%0A%0D%0A%09%5Eself%20search%3A%20%28self%20globalPatternOf%3A%20globalName%29"));
smalltalk.bind(smalltalk.MethodReferences,"searchGlobalReferences",0
,function(){var self=this;
return self.search_(self.globalPatternOf_(""));}
,"Tools-searching",unescape("searchGlobalReferences%0D%0A%09%22%20Find%20the%20requested%20elements%20from%20root%20classes.%20%22%0D%0A%0D%0A%09%5Eself%20search%3A%20%28self%20globalPatternOf%3A%20%27%27%29"));
smalltalk.bind(smalltalk.MethodReferences,"settersOfGlobalVariable:",0
,function(globalName){var self=this;
return self.search_(self.globalPatternOf_(globalName)._comma(unescape("%3D")));}
,"Tools-searching",unescape("settersOfGlobalVariable%3A%20globalName%0D%0A%09%22%20Find%20the%20requested%20elements%20from%20root%20classes.%20%22%0D%0A%0D%0A%09%5Eself%20search%3A%20%28self%20globalPatternOf%3A%20globalName%29%20%2C%27%3D%27"));
smalltalk.bind(smalltalk.MethodReferences,"referencesOfClassVariable:",0
,function(classVarName){var self=this;
return (function($rec){$rec.add_(unescape("---%20Getters%20of%20")._comma(classVarName)._comma(unescape("%20---")));$rec.addAll_(self.gettersOfClassVariable_(classVarName));$rec.add_(unescape("---%20Setters%20of%20")._comma(classVarName)._comma(unescape("%20---")));$rec.addAll_(self.settersOfClassVariable_(classVarName));return $rec.yourself();})(self.results());}
,"Tools-searching",unescape("referencesOfClassVariable%3A%20classVarName%0D%0A%09%22%20Find%20the%20requested%20elements%20from%20root%20classes.%20%22%0D%0A%0D%0A%09%5Eself%20results%0D%0A%09%09add%3A%20%27---%20Getters%20of%20%27%20%2CclassVarName%20%2C%27%20---%27%3B%0D%0A%09%09addAll%3A%20%28self%20gettersOfClassVariable%3A%20classVarName%29%3B%0D%0A%09%09add%3A%20%27---%20Setters%20of%20%27%20%2CclassVarName%20%2C%27%20---%27%3B%0D%0A%09%09addAll%3A%20%28self%20settersOfClassVariable%3A%20classVarName%29%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.MethodReferences,"gettersOfClassVariable:",0
,function(classVarName){var self=this;
return self.search_(self.dot()._comma(unescape("classVariableAt_%28%22"))._comma(classVarName)._comma(unescape("%22%29")));}
,"Tools-searching",unescape("gettersOfClassVariable%3A%20classVarName%0D%0A%09%22%20Find%20the%20requested%20elements%20from%20root%20classes.%20%22%0D%0A%0D%0A%09%5Eself%20search%3A%20self%20dot%2C%27classVariableAt_%28%22%27%2CclassVarName%2C%27%22%29%27"));
smalltalk.bind(smalltalk.MethodReferences,"settersOfClassVariable:",0
,function(classVarName){var self=this;
return self.search_(self.dot()._comma(unescape("classVariableAt_put_%28%22"))._comma(classVarName)._comma(unescape("%22%2C")));}
,"Tools-searching",unescape("settersOfClassVariable%3A%20classVarName%0D%0A%09%22%20Find%20the%20requested%20elements%20from%20root%20classes.%20%22%0D%0A%0D%0A%09%5Eself%20search%3A%20self%20dot%2C%27classVariableAt_put_%28%22%27%2CclassVarName%2C%27%22%2C%27"));
smalltalk.bind(smalltalk.MethodReferences,"globalReferences",0
,function(){var self=this;
var pool=nil;
pool=self.$klass.globals();
pool.keys().sorted().do_((function(aName){return (function($rec){$rec.add_(unescape("---%20References%20to%3A%20")._comma(aName)._comma(unescape("%20---")));return $rec.addAll_(pool.at_(aName));})(self.results());}));
return self["@results"];}
,"Tools-searching",unescape("globalReferences%0D%0A%09%22%20Find%20the%20references%20to%20globals.%20%22%0D%0A%0D%0A%09%7C%20pool%20%7C%0D%0A%09pool%20%3A%3D%20self%20class%20globals.%0D%0A%09pool%20keys%20sorted%20do%3A%20%5B%3AaName%7C%0D%0A%09%09self%20results%0D%0A%09%09%09add%3A%20%27---%20References%20to%3A%20%27%2CaName%2C%27%20---%27%3B%0D%0A%09%09%09addAll%3A%20%28pool%20at%3A%20aName%29%0D%0A%09%5D.%0D%0A%09%5Eresults"));
smalltalk.bind(smalltalk.MethodReferences,"missingGlobals",0
,function(){var self=this;
var pool=nil;
pool=self.$klass.globals();
pool.keys().sorted().reject_((function(key){return smalltalk.Smalltalk.includesKey_(key);})).do_((function(aName){return (function($rec){$rec.add_(unescape("---%20Missing%20global%3A%20")._comma(aName)._comma(unescape("%20---")));return $rec.addAll_(pool.at_(aName));})(self.results());}));
if (self.results().isEmpty()){self.results().add_(unescape("---%20All%20global%20references%20are%20bound%20to%20Smalltalk%20---"));};;
return self["@results"];}
,"Tools-searching",unescape("missingGlobals%0D%0A%09%22%20Find%20the%20references%20to%20missing%20globals.%20%22%0D%0A%0D%0A%09%7C%20pool%20%7C%0D%0A%09pool%20%3A%3D%20self%20class%20globals.%0D%0A%09%28pool%20keys%20sorted%0D%0A%09%09reject%3A%20%5B%3Akey%7C%20%28Smalltalk%20includesKey%3A%20key%29%20%5D%29%0D%0A%09%09do%3A%20%5B%3AaName%7C%0D%0A%09%09%09self%20results%0D%0A%09%09%09%09add%3A%20%27---%20Missing%20global%3A%20%27%2CaName%2C%27%20---%27%3B%0D%0A%09%09%09%09addAll%3A%20%28pool%20at%3A%20aName%29%0D%0A%09%09%5D.%0D%0A%09self%20results%20isEmpty%20ifTrue%3A%20%5B%0D%0A%09%09self%20results%20add%3A%20%27---%20All%20global%20references%20are%20bound%20to%20Smalltalk%20---%27%0D%0A%09%5D.%0D%0A%09%5Eresults"));
smalltalk.bind(smalltalk.MethodReferences,"implementorsOf:",0
,function(aSelector){var self=this;
return smalltalk.Smalltalk.current().implementorsOf_(aSelector);}
,"Tools-private",unescape("implementorsOf%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20the%20implementors%20of%20aSelector.%20%22%0D%0A%0D%0A%09%5ESmalltalk%20current%20implementorsOf%3A%20aSelector"));
smalltalk.bind(smalltalk.MethodReferences,"compiledSourceStart",0
,function(){var self=this;
return "{";}
,"Tools-patterns",unescape("compiledSourceStart%0D%0A%09%22%20Private%20-%20Return%20the%20token%20for%20start%20of%20compiled%20code.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%7D"));
smalltalk.bind(smalltalk.MethodReferences,"messagesStreamOn:",0
,function(aCompiledMethod){var self=this;
var stream=nil;
var nativeMessage=nil;
nativeMessage=self.nativeLanguage()._comma("MessagesStreamOn:");
if (smalltalk.responds_to_(self,(nativeMessage))){return self.perform_with_(nativeMessage,aCompiledMethod);};;
stream=aCompiledMethod.compiledSource().stream();
stream.upTo_(self.compiledSourceStart());
return stream;}
,"Tools-patterns",unescape("messagesStreamOn%3A%20aCompiledMethod%0D%0A%09%22%20Private%20-%20Return%20the%20stream%20for%20reading%20messages%20on%20compiled%20code%20of%20aCompiledMethod.%20%22%0D%0A%0D%0A%09%7C%20stream%20nativeMessage%20%7C%0D%0A%09nativeMessage%20%3A%3D%20self%20nativeLanguage%20%2C%23MessagesStreamOn%3A.%0D%0A%09%28self%20respondsTo%3A%20nativeMessage%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20perform%3A%20nativeMessage%20with%3A%20aCompiledMethod%0D%0A%09%5D.%0D%0A%09stream%20%3A%3D%20aCompiledMethod%20compiledSource%20stream.%0D%0A%09stream%20upTo%3A%20self%20compiledSourceStart.%0D%0A%09%5Estream%20"));
smalltalk.bind(smalltalk.MethodReferences,"nextMessageIn:",0
,function(stream){var self=this;
var dot=nil;
dot=self.dot();
return (function($rec){$rec.upTo_(dot);return $rec.upTo_(unescape("%28"));})(stream).asArrayOfSubstringsSeparatedBy_(dot).last().asNativeSelector();}
,"Tools-messages",unescape("nextMessageIn%3A%20stream%0D%0A%09%22%20Private%20-%20Return%20the%20next%20sent%20encoded%20in%20native%20source%20stream.%20%22%0D%0A%0D%0A%09%7C%20dot%20%7C%0D%0A%09dot%20%3A%3D%20self%20dot.%0D%0A%09%5E%28%28stream%20upTo%3A%20dot%3B%20upTo%3A%20%24%28%0D%0A%09%29%20asArrayOfSubstringsSeparatedBy%3A%20dot%29%0D%0A%09last%20asNativeSelector"));
smalltalk.bind(smalltalk.MethodReferences,"messagesOf:",0
,function(aCompiledMethod){var self=this;
var stream=nil;
var selectors=nil;
var getMessage=nil;
stream=self.messagesStreamOn_(aCompiledMethod);
if (stream.atEnd()){return [];};;
getMessage=self.nativeLanguage()._comma("NextMessageIn:");
if (!(smalltalk.responds_to_(self,(getMessage)))){getMessage="nextMessageIn:";};;
selectors=smalltalk.HashedSet.$$new();
while(!(function(){return stream.atEnd();})()){selectors.add_(self.perform_with_(getMessage,stream));};;
selectors.remove_ifAbsent_("",(function(){return nil;}));
return selectors.asArray().select_((function(each){return each.isLiteral();})).sorted();}
,"Tools-accessing",unescape("messagesOf%3A%20aCompiledMethod%0D%0A%09%22%20Return%20all%20the%20messages%20that%20can%20be%20sent%20during%20execution%20of%20aCompiledMethod.%20%22%0D%0A%0D%0A%09%7C%20stream%20selectors%20getMessage%20%7C%0D%0A%09stream%20%3A%3D%20self%20messagesStreamOn%3A%20aCompiledMethod.%0D%0A%09stream%20atEnd%20ifTrue%3A%20%5B%20%5E%23%28%29%20%5D.%0D%0A%09getMessage%20%3A%3D%20self%20nativeLanguage%20%2C%23NextMessageIn%3A.%0D%0A%09%28self%20respondsTo%3A%20getMessage%29%20ifFalse%3A%20%5B%0D%0A%09%09getMessage%20%3A%3D%20%23nextMessageIn%3A%0D%0A%09%5D.%0D%0A%09selectors%20%3A%3D%20HashedSet%20new.%0D%0A%09%5B%20stream%20atEnd%20%5D%20whileFalse%3A%20%5B%0D%0A%09%09selectors%20add%3A%20%28self%20perform%3A%20getMessage%20with%3A%20stream%29%0D%0A%09%5D.%0D%0A%09selectors%20remove%3A%20%27%27%20ifAbsent%3A%20%5B%5D.%0D%0A%09%5E%28selectors%20asArray%20select%3A%20%5B%3Aeach%7C%20each%20isLiteral%20%5D%29%20sorted"));
smalltalk.bind(smalltalk.MethodReferences,"globalReferencesOf:",0
,function(aCompiledMethod){var self=this;
var result=nil;
result=smalltalk.HashedSet.$$new();
(function($rec){$rec.when_do_("needsMethods",(function(){return smalltalk.Array.with_(aCompiledMethod);}));$rec.when_do_("found:pattern:in:",(function(mth,mask,stream){var global=nil;
global=smalltalk.SmalltalkParser.$$new().globalNameParser().parse_(stream);return ((global.isParseFailure()).mustBeBoolean()==true ? nil : (function(){return result.add_(global);})());}));return $rec.yourself();})(self);
self.searchGlobalReferences();
return result.asArray().sorted();}
,"Tools-accessing",unescape("globalReferencesOf%3A%20aCompiledMethod%0D%0A%09%22%20Return%20all%20the%20global%20reference%20names%20in%20aCompiledMethod.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20HashedSet%20new.%0D%0A%09self%0D%0A%09%09when%3A%20%23needsMethods%20do%3A%20%5B%20Array%20with%3A%20aCompiledMethod%20%5D%3B%0D%0A%09%09when%3A%20%23found%3Apattern%3Ain%3A%20do%3A%20%5B%3Amth%20%3Amask%20%3Astream%7C%20%7C%20global%20%7C%0D%0A%09%09%09global%20%3A%3D%20SmalltalkParser%20new%20globalNameParser%20parse%3A%20stream.%0D%0A%09%09%09global%20isParseFailure%20ifFalse%3A%20%5B%20result%20add%3A%20global%20%5D.%0D%0A%09%09%5D%3B%0D%0A%09%09yourself.%0D%0A%09self%20searchGlobalReferences.%0D%0A%09%5Eresult%20asArray%20sorted"));
smalltalk.bind(smalltalk.MethodReferences,"findClasses:",0
,function(what){var self=this;
var keys=nil;
var system=nil;
if (what.isString()){return self.findClasses_(what.trimBlanks().asArrayOfSubstrings());};;
self.triggerEvent_("searchStarted");
keys=what.collect_((function(each){return each.asString().asLowercase();}));
system=smalltalk.Smalltalk.current();
if (keys.notEmpty()){system.classes().do_((function(aClass){var key=nil;
key=aClass.name().asLowercase();return (((nil.isNil_(keys.detect_ifNone_((function(one){return key.includes_(one);}),(function(){return nil;})))===false)).mustBeBoolean()==true ? (function(){self.results().add_(aClass);return self.triggerEvent_with_("found:",aClass);})() : nil);}));};;
self.triggerEvent_("searchFinished");
return self.results();}
,"Tools-searching",unescape("findClasses%3A%20what%0D%0A%09%22%20Find%20classes%20matching%20what.%20%22%0D%0A%0D%0A%09%7C%20keys%20system%20%7C%0D%0A%09what%20isString%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20findClasses%3A%20what%20trimBlanks%20asArrayOfSubstrings%0D%0A%09%5D.%0D%0A%09self%20triggerEvent%3A%20%23searchStarted.%0D%0A%09keys%20%3A%3D%20what%20collect%3A%20%5B%3Aeach%7C%20each%20asString%20asLowercase%20%5D.%0D%0A%09system%20%3A%3D%20Smalltalk%20current.%0D%0A%09keys%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%09system%20classes%20do%3A%20%5B%3AaClass%7C%20%7C%20key%20%7C%0D%0A%09%09%09key%20%3A%3D%20aClass%20name%20asLowercase.%0D%0A%09%09%09%28keys%20detect%3A%20%5B%3Aone%7C%20key%20includes%3A%20one%20%5D%20ifNone%3A%20%5B%5D%29%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09%09self%20results%20add%3A%20aClass.%0D%0A%09%09%09%09self%20triggerEvent%3A%20%23found%3A%20with%3A%20aClass.%0D%0A%09%09%09%5D%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09self%20triggerEvent%3A%20%23searchFinished.%0D%0A%09%5Eself%20results"));
smalltalk.bind(smalltalk.MethodReferences,"methodOverwritesOf:",0
,function(aClass){var self=this;
var map=nil;
var subclasses=nil;
subclasses=aClass.allSubclasses();
map=smalltalk.PoolDictionary.$$new();
if (subclasses.isEmpty()){return map;};;
aClass.methodsDo_((function(mth){var selector=nil;
selector=mth.selector();map.at_put_(selector,smalltalk.Array.with_(mth));return subclasses.do_((function(each){return ((each.selectors().includes_(selector)).mustBeBoolean()==true ? (function(){return map.at_(selector).add_(each._gt_gt(selector));})() : nil);}));}));
subclasses.do_((function(each){return self.methodOverwritesOf_(each).keysAndValuesDo_((function(key,value){return map.at_ifAbsentPut_(key,(function(){return smalltalk.Array.$$new();})).addAll_(value);}));}));
map.keys().asArray().select_((function(key){return map.at_(key).size()._lt_eq((1));})).do_((function(key){return map.removeKey_(key);}));
return map;}
,"Tools-private",unescape("methodOverwritesOf%3A%20aClass%0D%0A%09%22%20Private%20-%20Find%20the%20methods%20subimplemented%20by%20subclasses%20of%20aClass.%20%22%0D%0A%0D%0A%09%7C%20map%20subclasses%20%7C%0D%0A%09subclasses%20%3A%3D%20aClass%20allSubclasses.%0D%0A%09map%20%3A%3D%20PoolDictionary%20new.%0D%0A%09subclasses%20isEmpty%20ifTrue%3A%20%5B%20%5Emap%20%5D.%0D%0A%09aClass%20methodsDo%3A%20%5B%3Amth%7C%20%7C%20selector%20%7C%0D%0A%09%09selector%20%3A%3D%20mth%20selector.%0D%0A%09%09map%20at%3A%20selector%20put%3A%20%28Array%20with%3A%20mth%29.%0D%0A%09%09subclasses%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%09%28each%20selectors%20includes%3A%20selector%29%20ifTrue%3A%20%5B%0D%0A%09%09%09%09%28map%20at%3A%20selector%29%20add%3A%20each%20%3E%3E%20selector%0D%0A%09%09%09%5D.%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09subclasses%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28self%20methodOverwritesOf%3A%20each%29%20keysAndValuesDo%3A%20%5B%3Akey%20%3Avalue%7C%0D%0A%09%09%09%28map%20at%3A%20key%20ifAbsentPut%3A%20%5BArray%20new%5D%29%20addAll%3A%20value%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%28map%20keys%20asArray%0D%0A%09%09select%3A%20%5B%3Akey%7C%20%28map%20at%3A%20key%29%20size%20%3C%3D%201%20%5D%0D%0A%09%09%29%20do%3A%20%5B%3Akey%7C%20map%20removeKey%3A%20key%20%5D.%0D%0A%09%5Emap"));
smalltalk.bind(smalltalk.MethodReferences,"addMethodOverwrites:",0
,function(map){var self=this;
map.associationsDo_((function(assoc){return assoc.value().do_((function(each){self.triggerEvent_with_("found:",each);return self.results().add_(each);}));}));
return self;}
,"Tools-private",unescape("addMethodOverwrites%3A%20map%0D%0A%09%22%20Private%20-%20Register%20the%20methods%20overwrites%20in%20map.%20%22%0D%0A%0D%0A%09map%20associationsDo%3A%20%5B%3Aassoc%7C%0D%0A%09%09assoc%20value%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%09self%20triggerEvent%3A%20%23found%3A%20with%3A%20each.%0D%0A%09%09%09self%20results%20add%3A%20each.%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.MethodReferences,"methodOverwrites",0
,function(){var self=this;
self.triggerEvent_("searchStarted");
smalltalk.Smalltalk.rootClasses().do_((function(aClass){self.addMethodOverwrites_(self.methodOverwritesOf_(aClass));return self.addMethodOverwrites_(self.methodOverwritesOf_(aClass.$klass));}));
self.triggerEvent_("searchFinished");
return self.results();}
,"Tools-searching",unescape("methodOverwrites%0D%0A%09%22%20Find%20the%20methods%20that%20are%20implemented/overwritten%20by%20subclasses.%20%22%0D%0A%0D%0A%09self%20triggerEvent%3A%20%23searchStarted.%0D%0A%09Smalltalk%20rootClasses%20do%3A%20%5B%3AaClass%7C%0D%0A%09%09self%20addMethodOverwrites%3A%20%28self%20methodOverwritesOf%3A%20aClass%29.%0D%0A%09%09self%20addMethodOverwrites%3A%20%28self%20methodOverwritesOf%3A%20aClass%20class%29.%0D%0A%09%5D.%0D%0A%09self%20triggerEvent%3A%20%23searchFinished.%0D%0A%09%5Eself%20results"));
smalltalk.bind(smalltalk.MethodReferences,"hasArgumentOverwrites:in:",0
,function(names,stream){var $s8Ret$={name:"stReturn"};try{var self=this;
var args=nil;
args=(function($rec){$rec.upTo_(unescape("%28"));return $rec.upTo_(unescape("%29"));})(stream).asArrayOfSubstringsSeparatedBy_(unescape("%2C"));
args.do_((function(each){return ((names.includes_(each)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"Tools-private",unescape("hasArgumentOverwrites%3A%20names%20in%3A%20stream%0D%0A%09%22%20Private%20-%20Return%20true%20if%20stream%20of%20code%20has%20argument%20name%20conflicts.%20%22%0D%0A%0D%0A%09%7C%20args%20%7C%0D%0A%09args%20%3A%3D%20%28stream%20upTo%3A%20%24%28%3B%20upTo%3A%20%24%29%29%20asArrayOfSubstringsSeparatedBy%3A%20%24%2C.%0D%0A%09args%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28names%20includes%3A%20each%29%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09%5D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.MethodReferences,"hasLocalOverwrites:in:",0
,function(names,stream){var $s8Ret$={name:"stReturn"};try{var self=this;
var candidate=nil;
while(!(function(){return stream.atEnd();})()){candidate=(function($rec){$rec.upTo_(self.localDef());return $rec.upTo_(unescape("%3D"));})(stream).trimBlanks();
((names.includes_(candidate)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return true})();throw($s8Ret$)})();})() : nil);};;
(function(){$s8Ret$.result=(function(){return false})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"Tools-private",unescape("hasLocalOverwrites%3A%20names%20in%3A%20stream%0D%0A%09%22%20Private%20-%20Return%20true%20if%20stream%20of%20code%20has%20local%20name%20conflicts.%20%22%0D%0A%0D%0A%09%7C%20candidate%20%7C%0D%0A%09%5Bstream%20atEnd%5D%20whileFalse%3A%20%5B%0D%0A%09%09candidate%20%3A%3D%20%28stream%20upTo%3A%20self%20localDef%3B%20upTo%3A%20%24%3D%29%20trimBlanks.%0D%0A%09%09%28names%20includes%3A%20candidate%29%20ifTrue%3A%20%5B%0D%0A%09%09%09%5Etrue%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.MethodReferences,"addVariableOverwrites:in:",0
,function(names,mth){var self=this;
var stream=nil;
stream=mth.compiledSource().stream();
if (self.hasArgumentOverwrites_in_(names,stream).or_((function(){return self.hasLocalOverwrites_in_(names,stream);}))){self.triggerEvent_with_("found:",mth);
self.results().add_(mth);};;
return self;}
,"Tools-private",unescape("addVariableOverwrites%3A%20names%20in%3A%20mth%0D%0A%09%22%20Private%20-%20Register%20the%20methods%20that%20has%20name%20overwrites.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20mth%20compiledSource%20stream.%0D%0A%09%28%28self%20hasArgumentOverwrites%3A%20names%20in%3A%20stream%29%0D%0A%09or%3A%20%5B%20self%20hasLocalOverwrites%3A%20names%20in%3A%20stream%20%5D%29%0D%0A%09ifTrue%3A%20%5B%0D%0A%09%09self%20triggerEvent%3A%20%23found%3A%20with%3A%20mth.%0D%0A%09%09self%20results%20add%3A%20mth.%0D%0A%09%5D"));
smalltalk.bind(smalltalk.MethodReferences,"variableOverwrites",0
,function(){var self=this;
self.triggerEvent_("searchStarted");
smalltalk.Smalltalk.classes().do_((function(aClass){var names=nil;
names=aClass.allInstVarNames();aClass.methodsDo_((function(mth){return self.addVariableOverwrites_in_(names,mth);}));names=aClass.$klass.allInstVarNames();return aClass.$klass.methodsDo_((function(mth){return self.addVariableOverwrites_in_(names,mth);}));}));
self.triggerEvent_("searchFinished");
return self.results();}
,"Tools-searching",unescape("variableOverwrites%0D%0A%09%22%20Find%20the%20methods%20that%20has%20potential%20overwrite%20of%20variable%20names.%20%22%0D%0A%0D%0A%09self%20triggerEvent%3A%20%23searchStarted.%0D%0A%09Smalltalk%20classes%20do%3A%20%5B%3AaClass%7C%20%7C%20names%20%7C%0D%0A%09%09names%20%3A%3D%20aClass%20allInstVarNames.%0D%0A%09%09aClass%20methodsDo%3A%20%5B%3Amth%7C%0D%0A%09%09%09self%20addVariableOverwrites%3A%20names%20in%3A%20mth.%0D%0A%09%09%5D.%0D%0A%09%09names%20%3A%3D%20aClass%20class%20allInstVarNames.%0D%0A%09%09aClass%20class%20methodsDo%3A%20%5B%3Amth%7C%0D%0A%09%09%09self%20addVariableOverwrites%3A%20names%20in%3A%20mth.%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09self%20triggerEvent%3A%20%23searchFinished.%0D%0A%09%5Eself%20results"));
smalltalk.bind(smalltalk.MethodReferences,"findMessages:",0
,function(what){var self=this;
var keys=nil;
if (what.isString()){return self.findMessages_(what.trimBlanks().asArrayOfSubstrings());};;
self.triggerEvent_("searchStarted");
keys=what.collect_((function(each){return each.asString();}));
if (keys.notEmpty()){var all=nil;
var collect=nil;
all=self.allMethods();
all.do_((function(mth){var current=nil;
current=mth.selector();return (((nil.isNil_(keys.detect_ifNone_((function(one){return current.includes_(one);}),(function(){return nil;})))===false)).mustBeBoolean()==true ? (function(){self.results().add_(mth);return self.triggerEvent_with_("found:",mth);})() : nil);}));};;
self.triggerEvent_("searchFinished");
return self.results();}
,"Tools-searching",unescape("findMessages%3A%20what%0D%0A%09%22%20Find%20messages%20matching%20what.%20%22%0D%0A%0D%0A%09%7C%20keys%20%7C%0D%0A%09what%20isString%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20findMessages%3A%20what%20trimBlanks%20asArrayOfSubstrings%0D%0A%09%5D.%0D%0A%09self%20triggerEvent%3A%20%23searchStarted.%0D%0A%09keys%20%3A%3D%20what%20collect%3A%20%5B%3Aeach%7C%20each%20asString%20%5D.%0D%0A%09keys%20notEmpty%20ifTrue%3A%20%5B%20%7C%20all%20collect%20%7C%0D%0A%09%09all%20%3A%3D%20self%20allMethods.%0D%0A%09%09all%20do%3A%20%5B%3Amth%7C%20%7C%20current%20%7C%0D%0A%09%09%09current%20%3A%3D%20mth%20selector.%0D%0A%09%09%09%28keys%20detect%3A%20%5B%3Aone%7C%20current%20includes%3A%20one%20%5D%20ifNone%3A%20%5B%5D%29%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09%09self%20results%20add%3A%20mth.%0D%0A%09%09%09%09self%20triggerEvent%3A%20%23found%3A%20with%3A%20mth.%0D%0A%09%09%09%5D.%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09self%20triggerEvent%3A%20%23searchFinished.%0D%0A%09%5Eself%20results"));
smalltalk.bind(smalltalk.MethodReferences,"superLuaSendPatternOf:",0
,function(aSelector){var self=this;
return unescape("%22")._comma(aSelector.asSelector())._comma(unescape("%22"))._comma(unescape("%29%28self"));}
,"Tools-patterns",unescape("superLuaSendPatternOf%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20the%20search%20pattern%20for%20senders%20%28to%20super%29%20of%20aSelector.%20%22%0D%0A%0D%0A%09%5E%24%22%20%2CaSelector%20asSelector%20%2C%24%22%20%2C%27%29%28self%27"));
smalltalk.bind(smalltalk.SystemManager.$klass,"constructEventsTriggered",0
,function(){var self=this;
return smalltalk.superImplementor(smalltalk.SystemManager.$klass,"constructEventsTriggered").apply(self, [])._comma(["aboutToBind:to:", "methodBound:to:", "aboutToRemove:from:", "methodRemoved:from:", "aboutToChange:to:", "aboutToCreateClass:with:", "classChanged:", "classShapeChanged:", "object:at:notFoundIn:", "log:", "memoryWarning:"]);}
,"events",unescape("constructEventsTriggered%0D%0A%09%22%20Private%20-%20Return%20the%20events%20managed%20by%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Esuper%20constructEventsTriggered%20%2C%23%28%0D%0A%09%09%23aboutToBind%3Ato%3A%0D%0A%09%09%23methodBound%3Ato%3A%0D%0A%09%09%23aboutToRemove%3Afrom%3A%0D%0A%09%09%23methodRemoved%3Afrom%3A%0D%0A%09%09%23aboutToChange%3Ato%3A%0D%0A%09%09%23aboutToCreateClass%3Awith%3A%0D%0A%09%09%23classChanged%3A%0D%0A%09%09%23classShapeChanged%3A%0D%0A%09%09%23object%3Aat%3AnotFoundIn%3A%0D%0A%09%09%23log%3A%0D%0A%09%09%23memoryWarning%3A%0D%0A%09%29"));
smalltalk.bind(smalltalk.SystemManager.$klass,"currentOrNil",0
,function(){var self=this;
return self.classVariableAt_ifAbsent_("Current",(function(){return nil;}));}
,"singleton",unescape("currentOrNil%0D%0A%09%22%20Return%20the%20current%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20classVariableAt%3A%20%23Current%20ifAbsent%3A%20%5B%20nil%20%5D"));
smalltalk.bind(smalltalk.SystemManager.$klass,"current",0
,function(){var self=this;
if ((nil.isNil_(smalltalk.SystemManager.classVariableAt_("Current")))){self.current_(self.$$new());};;
return smalltalk.SystemManager.classVariableAt_("Current");}
,"singleton",unescape("current%0D%0A%09%22%20Return%20the%20current%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09Current%20isNil%20ifTrue%3A%20%5B%20self%20current%3A%20self%20new%20%5D.%0D%0A%09%5ECurrent"));
smalltalk.bind(smalltalk.SystemManager.$klass,"current:",0
,function(aSystemManager){var self=this;
smalltalk.SystemManager.classVariableAt_put_("Current",aSystemManager);
aSystemManager.install();
return self;}
,"singleton",unescape("current%3A%20aSystemManager%0D%0A%09%22%20Set%20the%20current%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09Current%20%3A%3D%20aSystemManager.%0D%0A%09aSystemManager%20install."));
smalltalk.bind(smalltalk.SystemManager.$klass,"log:",0
,function(aString){var self=this;
if ((nil.isNil_(smalltalk.SystemManager.classVariableAt_("Current"))===false)){smalltalk.SystemManager.classVariableAt_("Current").log_(aString);};;
return self;}
,"log",unescape("log%3A%20aString%0D%0A%09%22%20Log%20aString%20onto%20loggin%20stream.%20%22%0D%0A%0D%0A%09Current%20notNil%20ifTrue%3A%20%5B%20Current%20log%3A%20aString%20%5D"));
smalltalk.bind(smalltalk.SystemManager.$klass,"memoryWarning:",0
,function(anArgument){var self=this;
if ((nil.isNil_(smalltalk.SystemManager.classVariableAt_("Current"))===false)){smalltalk.SystemManager.classVariableAt_("Current").memoryWarning_(anArgument);};;
return self;}
,"memory",unescape("memoryWarning%3A%20anArgument%0D%0A%09%22%20Trigger%20a%20memory%20warning%20event.%20%22%0D%0A%0D%0A%09Current%20notNil%20ifTrue%3A%20%5B%20Current%20memoryWarning%3A%20anArgument%20%5D"));
smalltalk.bind(smalltalk.SystemManager,"install",0
,function(){var self=this;
return self;}
,"install",unescape("install%0D%0A%09%22%20Private%20-%20Install%20the%20receiver%20as%20the%20source%20manager%20of%20the%20system.%0D%0A%09The%20default%20action%20is%20to%20do%20nothing.%0D%0A%09This%20method%20can%20be%20refined%20by%20subclasses.%0D%0A%09%22"));
smalltalk.bind(smalltalk.SystemManager,"aboutToBind:to:",0
,function(aMethod,aClass){var self=this;
return self.triggerEvent_with_with_("aboutToBind:to:",aMethod,aClass);}
,"tracking-methods",unescape("aboutToBind%3A%20aMethod%20to%3A%20aClass%0D%0A%09%22%20Private%20-%20The%20method%20will%20be%20bound%20to%20aClass.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20%23aboutToBind%3Ato%3A%20with%3A%20aMethod%20with%3A%20aClass%20"));
smalltalk.bind(smalltalk.SystemManager,"methodBound:to:",0
,function(aMethod,aClass){var self=this;
return self.triggerEvent_with_with_("methodBound:to:",aMethod,aClass);}
,"tracking-methods",unescape("methodBound%3A%20aMethod%20to%3A%20aClass%0D%0A%09%22%20Private%20-%20The%20method%20has%20been%20bound%20to%20aClass.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20%23methodBound%3Ato%3A%20with%3A%20aMethod%20with%3A%20aClass%20"));
smalltalk.bind(smalltalk.SystemManager,"aboutToRemove:from:",0
,function(aMethod,aClass){var self=this;
return self.triggerEvent_with_with_("aboutToRemove:from:",aMethod,aClass);}
,"tracking-methods",unescape("aboutToRemove%3A%20aMethod%20from%3A%20aClass%0D%0A%09%22%20Private%20-%20The%20method%20will%20be%20removed%20from%20aClass.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20%23aboutToRemove%3Afrom%3A%20with%3A%20aMethod%20with%3A%20aClass%20"));
smalltalk.bind(smalltalk.SystemManager,"methodRemoved:from:",0
,function(aMethod,aClass){var self=this;
return self.triggerEvent_with_with_("methodRemoved:from:",aMethod,aClass);}
,"tracking-methods",unescape("methodRemoved%3A%20aMethod%20from%3A%20aClass%0D%0A%09%22%20Private%20-%20The%20method%20has%20been%20removed%20from%20aClass.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20%23methodRemoved%3Afrom%3A%20with%3A%20aMethod%20with%3A%20aClass%20"));
smalltalk.bind(smalltalk.SystemManager,"aboutToChange:superclass:iVarNames:category:",0
,function(aClass,superclass,iVarNames,category){var self=this;
return self.triggerEvent_with_with_("aboutToChange:to:",aClass,smalltalk.Array.with_with_with_(superclass,iVarNames,category));}
,"tracking-classes",unescape("aboutToChange%3A%20aClass%20superclass%3A%20superclass%20iVarNames%3A%20iVarNames%20category%3A%20category%0D%0A%09%22%20Private%20-%20aClass%20is%20about%20to%20be%20changed.%20%22%0D%0A%0D%0A%09%5Eself%09triggerEvent%3A%20%23aboutToChange%3Ato%3A%0D%0A%09%09with%3A%20aClass%0D%0A%09%09with%3A%20%28Array%20with%3A%20superclass%20with%3A%20iVarNames%20with%3A%20category%29"));
smalltalk.bind(smalltalk.SystemManager,"aboutToCreateClass:superclass:iVarNames:category:",0
,function(globalName,superclass,iVarNames,category){var self=this;
return self.triggerEvent_with_with_("aboutToCreateClass:with:",globalName,smalltalk.Array.with_with_with_(superclass,iVarNames,category));}
,"tracking-classes",unescape("aboutToCreateClass%3A%20globalName%20superclass%3A%20superclass%20iVarNames%3A%20iVarNames%20category%3A%20category%0D%0A%09%22%20Private%20-%20Attemp%20to%20create%20Class%20with%20name%20globalName.%20%22%0D%0A%0D%0A%09%5Eself%09triggerEvent%3A%20%23aboutToCreateClass%3Awith%3A%0D%0A%09%09with%3A%20globalName%0D%0A%09%09with%3A%20%28Array%20with%3A%20superclass%20with%3A%20iVarNames%20with%3A%20category%29"));
smalltalk.bind(smalltalk.SystemManager,"classChanged:",0
,function(aBehavior){var self=this;
return self.triggerEvent_with_("classChanged:",aBehavior);}
,"tracking-classes",unescape("classChanged%3A%20aBehavior%0D%0A%09%22%20Private%20-%20aBehavior%20has%20been%20changed.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20%23classChanged%3A%20with%3A%20aBehavior%20"));
smalltalk.bind(smalltalk.SystemManager,"classShapeChanged:",0
,function(aBehavior){var self=this;
return self.triggerEvent_with_ifNotHandled_("classShapeChanged:",aBehavior,(function(){return aBehavior.recompileAll();}));}
,"tracking-classes",unescape("classShapeChanged%3A%20aBehavior%0D%0A%09%22%20Private%20-%20aBehavior%20has%20changed%20shape.%20%22%0D%0A%0D%0A%09%5Eself%09triggerEvent%3A%20%23classShapeChanged%3A%0D%0A%09%09with%3A%20aBehavior%0D%0A%09%09ifNotHandled%3A%20%5B%20aBehavior%20recompileAll%20%5D"));
smalltalk.bind(smalltalk.SystemManager,"log:",0
,function(aString){var self=this;
return self.triggerEvent_with_("log:",aString);}
,"log",unescape("log%3A%20aString%0D%0A%09%22%20Private%20-%20Log%20aString%20onto%20loggin%20device.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20%23log%3A%20with%3A%20aString"));
smalltalk.bind(smalltalk.SystemManager,"nextPutAll:",0
,function(aString){var self=this;
return self.log_(aString);}
,"streaming",unescape("nextPutAll%3A%20aString%0D%0A%09%22%20Log%20aString%20when%20streaming.%20%22%0D%0A%0D%0A%09%5Eself%20log%3A%20aString"));
smalltalk.bind(smalltalk.SystemManager,"cr",0
,function(){var self=this;
return self;}
,"streaming",unescape("cr%0D%0A%09%22%20Ignore%20the%20output%2C%20we%20log%20as%20line%20oriented%20streaming%20service.%20%22"));
smalltalk.bind(smalltalk.SystemManager,"memoryWarning:",0
,function(anArgument){var self=this;
return self.triggerEvent_with_("memoryWarning:",anArgument);}
,"memory",unescape("memoryWarning%3A%20anArgument%0D%0A%09%22%20Trigger%20a%20memory%20warning%20event.%0D%0A%09The%20execution%20environment%20can%20signal%20memory%20warning/low%20conditions%2C%20we%20trigger%20an%20event%20to%20be%20handled%20at%20application%20level.%0D%0A%09Note%3A%20It%20is%20frequent%20procedure%20to%20clean%20cached%20resources%20that%20can%20be%20reconstructed%20later%2C%20if%20required.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20%23memoryWarning%3A%20with%3A%20anArgument"));
smalltalk.bind(smalltalk.SystemManager,"object:at:notFoundIn:",0
,function(anObject,indexOrKey,aCollection){var self=this;
return self.triggerEvent_withArguments_ifNotHandled_("object:at:notFoundIn:",smalltalk.Array.with_with_with_(anObject,indexOrKey,aCollection),(function(){return self.print_(unescape("Object/key%28")._comma((((nil.isNil_(indexOrKey))).mustBeBoolean()==true ? (function(){return anObject;})() : (function(){return indexOrKey;})()).toString())._comma(unescape("%29%20not%20found%20in%20"))._comma(aCollection.printString()));}));}
,"failures",unescape("object%3A%20anObject%20at%3A%20indexOrKey%20notFoundIn%3A%20aCollection%0D%0A%09%22%20Trigger%20a%20missing%20object%20event.%20%22%0D%0A%0D%0A%09%5Eself%20triggerEvent%3A%20%23object%3Aat%3AnotFoundIn%3A%0D%0A%09%09withArguments%3A%20%28Array%20with%3A%20anObject%20with%3A%20indexOrKey%20with%3A%20aCollection%29%0D%0A%09%09ifNotHandled%3A%20%5B%0D%0A%09%09%09self%20print%3A%20%27Object/key%28%27%0D%0A%09%09%09%09%2C%28indexOrKey%20isNil%20ifTrue%3A%20%5BanObject%5D%20ifFalse%3A%20%5BindexOrKey%5D%29%20toString%0D%0A%09%09%09%09%2C%27%29%20not%20found%20in%20%27%2CaCollection%20printString%0D%0A%09%09%5D"));
smalltalk.bind(smalltalk.CompilerNode.$klass,"with:",0
,function(aNode){var self=this;
return (function($rec){$rec.addNode_(aNode);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("with%3A%20aNode%0D%0A%09%5Eself%20new%0D%0A%09%09addNode%3A%20aNode%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.CompilerNode,"nodes",0
,function(){var self=this;
if ((nil.isNil_(self["@nodes"]))){self["@nodes"]=smalltalk.Array.$$new();};;
return self["@nodes"];}
,"accessing",unescape("nodes%0D%0A%09nodes%20isNil%20ifTrue%3A%20%5B%20nodes%20%3A%3D%20Array%20new%20%5D.%0D%0A%09%5Enodes"));
smalltalk.bind(smalltalk.CompilerNode,"nodes:",0
,function(aCollection){var self=this;
self["@nodes"]=aCollection;
return self;}
,"accessing",unescape("nodes%3A%20aCollection%0D%0A%09nodes%20%3A%3D%20aCollection"));
smalltalk.bind(smalltalk.CompilerNode,"isEmpty",0
,function(){var self=this;
return self["@nodes"].size()._eq((0));}
,"testing",unescape("isEmpty%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20empty.%20%22%0D%0A%0D%0A%09%5Enodes%20size%20%3D%200%20"));
smalltalk.bind(smalltalk.CompilerNode,"annotations",0
,function(){var self=this;
if ((nil.isNil_(self["@annotations"]))){self["@annotations"]=smalltalk.Array.$$new();};;
return self["@annotations"];}
,"accessing",unescape("annotations%0D%0A%09annotations%20isNil%20ifTrue%3A%20%5B%20annotations%20%3A%3D%20Array%20new%20%5D.%0D%0A%09%5Eannotations"));
smalltalk.bind(smalltalk.CompilerNode,"annotations:",0
,function(aCollection){var self=this;
self["@annotations"]=aCollection;
return self;}
,"accessing",unescape("annotations%3A%20aCollection%0D%0A%09annotations%20%3A%3D%20aCollection"));
smalltalk.bind(smalltalk.CompilerNode,"addNode:",0
,function(aNode){var self=this;
self.nodes().add_(aNode);
return self;}
,"accessing",unescape("addNode%3A%20aNode%0D%0A%09%22%20Add%20a%20Node%20to%20the%20nodes%20of%20the%20receiver.%0D%0A%09Return%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20nodes%20add%3A%20aNode"));
smalltalk.bind(smalltalk.CompilerNode,"start",0
,function(){var self=this;
return self["@start"];}
,"accessing",unescape("start%0D%0A%09%5Estart"));
smalltalk.bind(smalltalk.CompilerNode,"end",0
,function(){var self=this;
return self["@end"];}
,"accessing",unescape("end%0D%0A%09%5Eend"));
smalltalk.bind(smalltalk.CompilerNode,"start:end:",0
,function(startOffset,endOffset){var self=this;
self["@start"]=startOffset;
self["@end"]=endOffset;
return self;}
,"accessing",unescape("start%3A%20startOffset%20end%3A%20endOffset%0D%0A%09%22%20Set%20the%20offsets%20of%20the%20receiver%20in%20source%20code.%20%22%0D%0A%0D%0A%09start%20%3A%3D%20startOffset.%0D%0A%09end%20%3A%3D%20endOffset."));
smalltalk.bind(smalltalk.CompilerNode,"sourceRange",0
,function(){var self=this;
return self["@start"].to_(self["@end"]);}
,"accessing",unescape("sourceRange%0D%0A%09%22%20Return%20the%20receiver%27s%20sourceRange.%20%22%0D%0A%0D%0A%09%5Estart%20to%3A%20end%20"));
smalltalk.bind(smalltalk.CompilerNode,"client",0
,function(){var self=this;
return self["@client"];}
,"accessing",unescape("client%0D%0A%09%22%20Private%20-%20Return%20an%20object%20that%20require%20the%20result%20of%20evaluation%20of%20the%20receiver.%0D%0A%09Note%20that%20client%20can%20be%3A%0D%0A%09%09aNode%20that%20will%20demmand%20result%20on%20runtime%0D%0A%09%09nil%20in%20case%20of%20not%20known%0D%0A%09%09%23none%20if%20granted%20that%20the%20result%20of%20evaluation%20of%20the%20receiver%20will%20be%20discarded/unused.%0D%0A%09%22%0D%0A%0D%0A%09%5Eclient%20"));
smalltalk.bind(smalltalk.CompilerNode,"client:",0
,function(anObject){var self=this;
self["@client"]=anObject;
return self;}
,"accessing",unescape("client%3A%20anObject%0D%0A%09%22%20Private%20-%20Set%20an%20object%20that%20require%20the%20result%20of%20evaluation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09client%20%3A%3D%20anObject%20"));
smalltalk.bind(smalltalk.CompilerNode,"prepareIn:for:",0
,function(aNode,anEncoder){var self=this;
if (self["@nodes"].size()._lt((1))){return self;};;
self["@nodes"].do_((function(each){return each.client_("none");}));
self["@nodes"].last().client_(self);
return self;}
,"prepare",unescape("prepareIn%3A%20aNode%20for%3A%20anEncoder%0D%0A%09%22%20Prepare%20the%20receiver%20to%20be%20encoded%20by%20anEncoder%20in%20aNode.%20%22%0D%0A%0D%0A%09nodes%20size%20%3C%201%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09nodes%20do%3A%20%5B%3Aeach%7C%20each%20client%3A%20%23none%20%5D.%0D%0A%09nodes%20last%20client%3A%20self"));
smalltalk.bind(smalltalk.CompilerNode,"sourceRange:",0
,function(interval){var self=this;
self.start_end_(interval.start(),interval.end());
return self;}
,"accessing",unescape("sourceRange%3A%20interval%0D%0A%09%22%20Set%20the%20source%20range%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20start%3A%20interval%20start%20end%3A%20interval%20end%20"));
smalltalk.bind(smalltalk.CompilerNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitNode%3A%20self"));
smalltalk.bind(smalltalk.CompilerNode,"acceptMacroAssignment:with:",0
,function(anExpressionNode,aVisitor){var self=this;
return aVisitor.visitMacroAssignment_to_(self,anExpressionNode);}
,"visiting",unescape("acceptMacroAssignment%3A%20anExpressionNode%20with%3A%20aVisitor%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20can%20resolve%20assignment%20on%20aVisitor.%20%22%0D%0A%0D%0A%09%5EaVisitor%20visitMacroAssignment%3A%20self%20to%3A%20anExpressionNode"));
smalltalk.bind(smalltalk.CompilerNode,"isSuper",0
,function(){var self=this;
return false;}
,"testing",unescape("isSuper%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20super.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"hasNumber",0
,function(){var self=this;
return false;}
,"testing",unescape("hasNumber%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20has%20a%20numeric%20literal%20value.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"hasString",0
,function(){var self=this;
return false;}
,"testing",unescape("hasString%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20has%20a%20string%20literal%20value.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"isAssignment",0
,function(){var self=this;
return false;}
,"testing",unescape("isAssignment%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"isBlock",0
,function(){var self=this;
return false;}
,"testing",unescape("isBlock%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"isCascade",0
,function(){var self=this;
return false;}
,"testing",unescape("isCascade%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"isInline",0
,function(){var self=this;
return false;}
,"testing",unescape("isInline%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"isReturn",0
,function(){var self=this;
return false;}
,"testing",unescape("isReturn%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"isSend",0
,function(){var self=this;
return false;}
,"testing",unescape("isSend%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"isSequence",0
,function(){var self=this;
return false;}
,"testing",unescape("isSequence%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"isValue",0
,function(){var self=this;
return false;}
,"testing",unescape("isValue%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"isVariable",0
,function(){var self=this;
return false;}
,"testing",unescape("isVariable%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.CompilerNode,"bindings",0
,function(){var self=this;
return [];}
,"bindings",unescape("bindings%0D%0A%09%22%20Return%20the%20bindings%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%29"));
smalltalk.bind(smalltalk.CompilerNode,"do:",0
,function(aBlock){var self=this;
self.do_in_(aBlock,nil);
return self;}
,"iteration",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20do%3A%20aBlock%20in%3A%20nil%20"));
smalltalk.bind(smalltalk.CompilerNode,"do:in:",0
,function(aBlock,aNode){var self=this;
aBlock.evaluateWithArguments_(smalltalk.Array.with_with_(self,aNode));
if ((nil.isNil_(self["@nodes"])===false)){self["@nodes"].do_((function(each){return each.do_in_(aBlock,self);}));};;
return self;}
,"iteration",unescape("do%3A%20aBlock%20in%3A%20aNode%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09aBlock%20evaluateWithArguments%3A%20%28Array%20with%3A%20self%20with%3A%20aNode%29.%0D%0A%09nodes%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09nodes%20do%3A%20%5B%3Aeach%7C%20each%20do%3A%20aBlock%20in%3A%20self%20%5D%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.CompilerNode,"with:except:do:",0
,function(testBlock,except,aBlock){var self=this;
if ((nil.is_eqeq_(except.evaluateWith_(self),true))){return nil;};;
if ((nil.isNil_(self["@nodes"])===false)){self["@nodes"].do_((function(each){return ((testBlock.value_(each)).mustBeBoolean()==true ? (function(){return aBlock.value_value_(each,self);})() : (function(){return each.with_except_do_(testBlock,except,aBlock);})());}));};;
return self;}
,"iteration",unescape("with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20nodes%20%28returning%20true%20when%20evaluated%20with%20testBlock%29%20in%20the%20local%20context%20of%20receiver.%20%22%0D%0A%0D%0A%09%28except%20evaluateWith%3A%20self%29%20%3D%3D%20true%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09nodes%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09nodes%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%09%28testBlock%20value%3A%20each%29%0D%0A%09%09%09ifTrue%3A%20%5B%20aBlock%20value%3A%20each%20value%3A%20self%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20each%20with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%20%5D%0D%0A%09%09%5D%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.CompilerNode,"select:",0
,function(aBlock){var self=this;
var result=nil;
result=[];
self.do_((function(each){return (((nil.is_eqeq_(aBlock.evaluateWith_(each),true))).mustBeBoolean()==true ? (function(){return result.add_(each);})() : nil);}));
return result;}
,"enumerating",unescape("select%3A%20aBlock%0D%0A%09%22%20Returns%20the%20nodes%20in%20the%20receiver%20that%20returns%20true%20when%20evaluated%20with%20aBlock.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20%23%28%29.%0D%0A%09self%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28aBlock%20evaluateWith%3A%20each%29%20%3D%3D%20true%20ifTrue%3A%20%5B%0D%0A%09%09%09result%20add%3A%20each%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.CompilerNode,"messageSendConnector",0
,function(){var self=this;
return ".";}
,"accessing",unescape("messageSendConnector%0D%0A%09%22%20Return%20the%20connector%20used%20to%20encode%20message%20sent%20to%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5E%24.%20"));
smalltalk.bind(smalltalk.MethodNode,"selector",0
,function(){var self=this;
return self["@selector"];}
,"accessing",unescape("selector%0D%0A%09%5Eselector"));
smalltalk.bind(smalltalk.MethodNode,"selector:",0
,function(aString){var self=this;
self["@selector"]=aString;
return self;}
,"accessing",unescape("selector%3A%20aString%0D%0A%09selector%20%3A%3D%20aString"));
smalltalk.bind(smalltalk.MethodNode,"arguments",0
,function(){var self=this;
return (((nil.isNil_(self["@arguments"]))).mustBeBoolean()==true ? (function(){return [];})() : (function(){return self["@arguments"];})());}
,"accessing",unescape("arguments%0D%0A%09%5Earguments%20isNil%20ifTrue%3A%20%5B%20%23%28%29%20%5D%20ifFalse%3A%20%5B%20arguments%20%5D"));
smalltalk.bind(smalltalk.MethodNode,"arguments:",0
,function(aCollection){var self=this;
self["@arguments"]=aCollection;
return self;}
,"accessing",unescape("arguments%3A%20aCollection%0D%0A%09arguments%20%3A%3D%20aCollection"));
smalltalk.bind(smalltalk.MethodNode,"argumentNames",0
,function(){var self=this;
return self.arguments().collect_((function(each){return ((each.isString()).mustBeBoolean()==true ? (function(){return each;})() : (function(){return each.value();})());}));}
,"accessing",unescape("argumentNames%0D%0A%09%22%20Return%20the%20names%20of%20the%20arguments%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20arguments%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20isString%0D%0A%09%09ifTrue%3A%20%5Beach%5D%0D%0A%09%09ifFalse%3A%20%5Beach%20value%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.MethodNode,"source",0
,function(){var self=this;
return self["@source"];}
,"accessing",unescape("source%0D%0A%09%5Esource"));
smalltalk.bind(smalltalk.MethodNode,"source:",0
,function(aString){var self=this;
self["@source"]=aString;
return self;}
,"accessing",unescape("source%3A%20aString%0D%0A%09source%20%3A%3D%20aString"));
smalltalk.bind(smalltalk.MethodNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitMethodNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitMethodNode%3A%20self"));
smalltalk.bind(smalltalk.MethodNode,"validateArguments",0
,function(){var self=this;
var names=nil;
names=smalltalk.HashedSet.withAll_(self.argumentNames());
if (!(names.size()._eq(self.arguments().size()))){return self.error_("Invalid argument names "._comma(self.argumentNames().asLiteral()));};;
self.nodes().do_((function(each){return each.validateNames_(names);}));
return self;}
,"validation",unescape("validateArguments%0D%0A%09%22%20Ensure%20the%20nodes%20of%20the%20receiver%20can%20be%20emmited%20with%20the%20defined%20arguments.%20%22%0D%0A%0D%0A%09%7C%20names%20%7C%0D%0A%09names%20%3A%3D%20HashedSet%20withAll%3A%20self%20argumentNames.%0D%0A%09names%20size%20%3D%20self%20arguments%20size%20ifFalse%3A%20%5B%0D%0A%09%09%5Eself%20error%3A%20%27Invalid%20argument%20names%20%27%20%2Cself%20argumentNames%20asLiteral%0D%0A%09%5D.%0D%0A%09self%20nodes%20do%3A%20%5B%3Aeach%7C%20each%20validateNames%3A%20names%20%5D"));
smalltalk.bind(smalltalk.MethodNode,"sendNodes",0
,function(){var self=this;
return self.select_((function(each){return each.isSend();}));}
,"accessing",unescape("sendNodes%0D%0A%09%22%20Returns%20the%20send%20nodes%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20select%3A%20%5B%3Aeach%7C%20each%20isSend%20%5D"));
smalltalk.bind(smalltalk.MethodNode,"messages",0
,function(){var self=this;
return self.sendNodes().collect_((function(each){return each.selector();})).asSet().sorted();}
,"accessing",unescape("messages%0D%0A%09%22%20Returns%20the%20messages%20potentially%20sent%20when%20evaluating%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28self%20sendNodes%0D%0A%09%09collect%3A%20%5B%3Aeach%7C%20each%20selector%20%5D%29%0D%0A%09%09asSet%20sorted"));
smalltalk.bind(smalltalk.MethodNode,"values",0
,function(){var self=this;
var result=nil;
result=[];
result.addAll_(self.arguments());
self.do_((function(each){if (each.isSequence()){result.addAll_(each.temps());};;return ((each.isBlock()).mustBeBoolean()==true ? (function(){return result.addAll_(each.parameters());})() : nil);}));
return result;}
,"accessing",unescape("values%0D%0A%09%22%20Returns%20the%20values%20bound%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20%23%28%29.%0D%0A%09result%20addAll%3A%20self%20arguments.%0D%0A%09self%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20isSequence%20ifTrue%3A%20%5B%20result%20addAll%3A%20each%20temps%20%5D.%0D%0A%09%09each%20isBlock%20ifTrue%3A%20%5B%20result%20addAll%3A%20each%20parameters%20%5D.%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.SendNode,"selector",0
,function(){var self=this;
return self["@selector"];}
,"accessing",unescape("selector%0D%0A%09%5Eselector"));
smalltalk.bind(smalltalk.SendNode,"selector:",0
,function(aString){var self=this;
self["@selector"]=aString;
return self;}
,"accessing",unescape("selector%3A%20aString%0D%0A%09selector%20%3A%3D%20aString"));
smalltalk.bind(smalltalk.SendNode,"arguments",0
,function(){var self=this;
return (((nil.isNil_(self["@arguments"]))).mustBeBoolean()==true ? (function(){return [];})() : (function(){return self["@arguments"];})());}
,"accessing",unescape("arguments%0D%0A%09%5Earguments%20isNil%20ifTrue%3A%20%5B%20%23%28%29%20%5D%20ifFalse%3A%20%5B%20arguments%20%5D"));
smalltalk.bind(smalltalk.SendNode,"arguments:",0
,function(aCollection){var self=this;
self["@arguments"]=aCollection;
return self;}
,"accessing",unescape("arguments%3A%20aCollection%0D%0A%09arguments%20%3A%3D%20aCollection"));
smalltalk.bind(smalltalk.SendNode,"receiver",0
,function(){var self=this;
return self["@receiver"];}
,"accessing",unescape("receiver%0D%0A%09%5Ereceiver"));
smalltalk.bind(smalltalk.SendNode,"receiver:",0
,function(aNode){var self=this;
self["@receiver"]=aNode;
return self;}
,"accessing",unescape("receiver%3A%20aNode%0D%0A%09receiver%20%3A%3D%20aNode"));
smalltalk.bind(smalltalk.SendNode,"valueForReceiver:",0
,function(anObject){var self=this;
return (function($rec){$rec.receiver_((((nil.isNil_(self.receiver()))).mustBeBoolean()==true ? (function(){return anObject;})() : (function(){return self.receiver().valueForReceiver_(anObject);})()));$rec.selector_(self.selector());$rec.arguments_(self.arguments());$rec.start_end_(self.start(),self.end());return $rec.yourself();})(smalltalk.SendNode.$$new());}
,"accessing",unescape("valueForReceiver%3A%20anObject%0D%0A%09%5ESendNode%20new%0D%0A%09%20%20%20%20receiver%3A%20%28self%20receiver%20isNil%20ifTrue%3A%20%5BanObject%5D%0D%0A%09%09ifFalse%3A%20%5Bself%20receiver%20valueForReceiver%3A%20anObject%5D%29%3B%0D%0A%09%20%20%20%20selector%3A%20self%20selector%3B%0D%0A%09%20%20%20%20arguments%3A%20self%20arguments%3B%0D%0A%09%20%20%20%20start%3A%20self%20start%20end%3A%20self%20end%3B%0D%0A%09%20%20%20%20yourself"));
smalltalk.bind(smalltalk.SendNode,"cascadeNodeWithMessages:",0
,function(aCollection){var self=this;
var first=nil;
var sends=nil;
first=(function($rec){$rec.selector_(self.selector());$rec.arguments_(self.arguments());$rec.start_end_(self.start(),self.end());return $rec.yourself();})(smalltalk.SendNode.$$new());
sends=smalltalk.Array.with_(first)._comma(aCollection);
return (function($rec){$rec.receiver_(self.receiver());$rec.nodes_(sends);$rec.start_end_(sends.first().start(),sends.last().end());return $rec.yourself();})(smalltalk.CascadeNode.$$new());}
,"accessing",unescape("cascadeNodeWithMessages%3A%20aCollection%0D%0A%09%7C%20first%20sends%20%7C%0D%0A%09first%20%3A%3D%20SendNode%20new%0D%0A%09%20%20%20%20selector%3A%20self%20selector%3B%0D%0A%09%20%20%20%20arguments%3A%20self%20arguments%3B%0D%0A%09%20%20%20%20start%3A%20self%20start%20end%3A%20self%20end%3B%0D%0A%09%20%20%20%20yourself.%0D%0A%09sends%20%3A%3D%20%28Array%20with%3A%20first%29%2C%20aCollection.%0D%0A%09%5ECascadeNode%20new%0D%0A%09%20%20%20%20receiver%3A%20self%20receiver%3B%0D%0A%09%20%20%20%20nodes%3A%20sends%3B%0D%0A%09%20%20%20%20start%3A%20sends%20first%20start%20end%3A%20sends%20last%20end%3B%0D%0A%09%20%20%20%20yourself"));
smalltalk.bind(smalltalk.SendNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitSendNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitSendNode%3A%20self"));
smalltalk.bind(smalltalk.SendNode,"isSend",0
,function(){var self=this;
return true;}
,"testing",unescape("isSend%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.SendNode,"do:in:",0
,function(aBlock,aNode){var self=this;
smalltalk.superImplementor(smalltalk.SendNode,"do_in_").apply(self, [aBlock,aNode]);
if ((nil.isNil_(self["@receiver"])===false)){self["@receiver"].do_in_(aBlock,self);};;
self.arguments().do_((function(each){return each.do_in_(aBlock,self);}));
return self;}
,"iteration",unescape("do%3A%20aBlock%20in%3A%20aNode%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20do%3A%20aBlock%20in%3A%20aNode.%0D%0A%09receiver%20notNil%20ifTrue%3A%20%5B%20receiver%20do%3A%20aBlock%20in%3A%20self%20%5D.%0D%0A%09self%20arguments%20do%3A%20%5B%3Aeach%7C%20each%20do%3A%20aBlock%20in%3A%20self%20%5D."));
smalltalk.bind(smalltalk.SendNode,"with:except:do:",0
,function(testBlock,except,aBlock){var self=this;
if ((nil.isNil_(smalltalk.superImplementor(smalltalk.SendNode,"with_except_do_").apply(self, [testBlock,except,aBlock])))){return nil;};;
if ((nil.isNil_(self["@receiver"])===false)){((testBlock.value_(self["@receiver"])).mustBeBoolean()==true ? (function(){return aBlock.value_value_(self["@receiver"],self);})() : (function(){return self["@receiver"].with_except_do_(testBlock,except,aBlock);})());};;
self.arguments().do_((function(each){return ((testBlock.value_(each)).mustBeBoolean()==true ? (function(){return aBlock.value_value_(each,self);})() : (function(){return each.with_except_do_(testBlock,except,aBlock);})());}));
return self;}
,"iteration",unescape("with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20nodes%20%28returning%20true%20when%20evaluated%20with%20testBlock%29%20in%20the%20local%20context%20of%20receiver.%20%22%0D%0A%0D%0A%09%28super%20with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%29%20isNil%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09receiver%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%28testBlock%20value%3A%20receiver%29%0D%0A%09%09ifTrue%3A%20%5B%20aBlock%20value%3A%20receiver%20value%3A%20self%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20receiver%20with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%20%5D%0D%0A%09%5D.%0D%0A%09self%20arguments%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28testBlock%20value%3A%20each%29%0D%0A%09%09ifTrue%3A%20%5B%20aBlock%20value%3A%20each%20value%3A%20self%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20each%20with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%20%5D%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.SendNode,"prepareIn:for:",0
,function(aNode,anEncoder){var self=this;
smalltalk.superImplementor(smalltalk.SendNode,"prepareIn_for_").apply(self, [aNode,anEncoder]);
if ((nil.isNil_(self["@receiver"])===false)){self["@receiver"].client_(self);};;
self.arguments().do_((function(each){return each.client_(self);}));
return self;}
,"prepare",unescape("prepareIn%3A%20aNode%20for%3A%20anEncoder%0D%0A%09%22%20Prepare%20the%20receiver%20to%20be%20encoded%20by%20anEncoder%20in%20aNode.%20%22%0D%0A%0D%0A%09super%20prepareIn%3A%20aNode%20for%3A%20anEncoder.%0D%0A%09receiver%20notNil%20ifTrue%3A%20%5B%20receiver%20client%3A%20self%20%5D.%0D%0A%09self%20arguments%20do%3A%20%5B%3Aeach%7C%20each%20client%3A%20self%20%5D%20"));
smalltalk.bind(smalltalk.CascadeNode,"receiver",0
,function(){var self=this;
return self["@receiver"];}
,"accessing",unescape("receiver%0D%0A%09%5Ereceiver"));
smalltalk.bind(smalltalk.CascadeNode,"receiver:",0
,function(aNode){var self=this;
self["@receiver"]=aNode;
return self;}
,"accessing",unescape("receiver%3A%20aNode%0D%0A%09receiver%20%3A%3D%20aNode"));
smalltalk.bind(smalltalk.CascadeNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitCascadeNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitCascadeNode%3A%20self"));
smalltalk.bind(smalltalk.CascadeNode,"isCascade",0
,function(){var self=this;
return true;}
,"testing",unescape("isCascade%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.CascadeNode,"do:in:",0
,function(aBlock,aNode){var self=this;
smalltalk.superImplementor(smalltalk.CascadeNode,"do_in_").apply(self, [aBlock,aNode]);
if ((nil.isNil_(self["@receiver"])===false)){self["@receiver"].do_in_(aBlock,self);};;
return self;}
,"iteration",unescape("do%3A%20aBlock%20in%3A%20aNode%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20do%3A%20aBlock%20in%3A%20aNode.%0D%0A%09receiver%20notNil%20ifTrue%3A%20%5B%20receiver%20do%3A%20aBlock%20in%3A%20self%20%5D."));
smalltalk.bind(smalltalk.CascadeNode,"with:except:do:",0
,function(testBlock,except,aBlock){var self=this;
if ((nil.isNil_(smalltalk.superImplementor(smalltalk.CascadeNode,"with_except_do_").apply(self, [testBlock,except,aBlock])))){return nil;};;
if ((nil.isNil_(self["@receiver"])===false)){((testBlock.value_(self["@receiver"])).mustBeBoolean()==true ? (function(){return aBlock.value_value_(self["@receiver"],self);})() : (function(){return self["@receiver"].with_except_do_(testBlock,except,aBlock);})());};;
return self;}
,"iteration",unescape("with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20nodes%20%28returning%20true%20when%20evaluated%20with%20testBlock%29%20in%20the%20local%20context%20of%20receiver.%20%22%0D%0A%0D%0A%09%28super%20with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%29%20isNil%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09receiver%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%28testBlock%20value%3A%20receiver%29%0D%0A%09%09ifTrue%3A%20%5B%20aBlock%20value%3A%20receiver%20value%3A%20self%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20receiver%20with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%20%5D%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.BlockNode,"parameterNames",0
,function(){var self=this;
var result=nil;
result=self.parameters().collect_((function(each){return ((each.isString()).mustBeBoolean()==true ? (function(){return each;})() : (function(){return each.value();})());}));
if (!(result.asSet().size()._eq(result.size()))){return self.error_("Invalid names "._comma(result.asLiteral()));};;
return result;}
,"accessing",unescape("parameterNames%0D%0A%09%22%20Return%20the%20names%20of%20the%20parameters%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20self%20parameters%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20isString%0D%0A%09%09ifTrue%3A%20%5Beach%5D%0D%0A%09%09ifFalse%3A%20%5Beach%20value%5D%0D%0A%09%5D.%0D%0A%09result%20asSet%20size%20%3D%20result%20size%20ifFalse%3A%20%5B%0D%0A%09%09%5Eself%20error%3A%20%27Invalid%20names%20%27%20%2Cresult%20asLiteral%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.BlockNode,"parameters",0
,function(){var self=this;
if ((nil.isNil_(self["@parameters"]))){self["@parameters"]=smalltalk.Array.$$new();};;
return self["@parameters"];}
,"accessing",unescape("parameters%0D%0A%09parameters%20isNil%20ifTrue%3A%20%5B%20parameters%20%3A%3D%20Array%20new%20%5D.%0D%0A%09%5Eparameters"));
smalltalk.bind(smalltalk.BlockNode,"parameters:",0
,function(aCollection){var self=this;
self["@parameters"]=aCollection;
return self;}
,"accessing",unescape("parameters%3A%20aCollection%0D%0A%09parameters%20%3A%3D%20aCollection"));
smalltalk.bind(smalltalk.BlockNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitBlockNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitBlockNode%3A%20self"));
smalltalk.bind(smalltalk.BlockNode,"bindings",0
,function(){var self=this;
return self.parameterNames();}
,"bindings",unescape("bindings%0D%0A%09%22%20Return%20the%20bindings%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20parameterNames"));
smalltalk.bind(smalltalk.BlockNode,"isBlock",0
,function(){var self=this;
return true;}
,"testing",unescape("isBlock%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.BlockNode,"do:in:",0
,function(aBlock,aNode){var self=this;
smalltalk.superImplementor(smalltalk.BlockNode,"do_in_").apply(self, [aBlock,aNode]);
self.parameters().do_((function(each){return each.do_in_(aBlock,self);}));
return self;}
,"iteration",unescape("do%3A%20aBlock%20in%3A%20aNode%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20do%3A%20aBlock%20in%3A%20aNode.%0D%0A%09self%20parameters%20do%3A%20%5B%3Aeach%7C%20each%20do%3A%20aBlock%20in%3A%20self%20%5D."));
smalltalk.bind(smalltalk.SequenceNode,"tempNames",0
,function(){var self=this;
return self.temps().collect_((function(each){return ((each.isString()).mustBeBoolean()==true ? (function(){return each;})() : (function(){return each.value();})());}));}
,"accessing",unescape("tempNames%0D%0A%09%22%20Return%20the%20names%20of%20the%20temps%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20temps%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20isString%0D%0A%09%09ifTrue%3A%20%5Beach%5D%0D%0A%09%09ifFalse%3A%20%5Beach%20value%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.SequenceNode,"temps",0
,function(){var self=this;
return (((nil.isNil_(self["@temps"]))).mustBeBoolean()==true ? (function(){return [];})() : (function(){return self["@temps"];})());}
,"accessing",unescape("temps%0D%0A%09%5Etemps%20isNil%20ifTrue%3A%20%5B%20%23%28%29%20%5D%20ifFalse%3A%20%5B%20temps%20%5D"));
smalltalk.bind(smalltalk.SequenceNode,"temps:",0
,function(aCollection){var self=this;
self["@temps"]=aCollection;
return self;}
,"accessing",unescape("temps%3A%20aCollection%0D%0A%09temps%20%3A%3D%20aCollection"));
smalltalk.bind(smalltalk.SequenceNode,"asBlockSequenceNode",0
,function(){var self=this;
return (function($rec){$rec.nodes_(self.nodes());$rec.temps_(self.temps());$rec.start_end_(self.start(),self.end());return $rec.yourself();})(smalltalk.BlockSequenceNode.$$new());}
,"converting",unescape("asBlockSequenceNode%0D%0A%09%5EBlockSequenceNode%20new%0D%0A%09%20%20%20%20nodes%3A%20self%20nodes%3B%0D%0A%09%20%20%20%20temps%3A%20self%20temps%3B%0D%0A%09%20%20%20%20start%3A%20self%20start%20end%3A%20self%20end%3B%0D%0A%09%20%20%20%20yourself"));
smalltalk.bind(smalltalk.SequenceNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitSequenceNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitSequenceNode%3A%20self"));
smalltalk.bind(smalltalk.SequenceNode,"bindings",0
,function(){var self=this;
return self.tempNames();}
,"bindings",unescape("bindings%0D%0A%09%22%20Return%20the%20bindings%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20tempNames"));
smalltalk.bind(smalltalk.SequenceNode,"isSequence",0
,function(){var self=this;
return true;}
,"testing",unescape("isSequence%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.SequenceNode,"shouldReturn",0
,function(){var self=this;
if (self.nodes().notEmpty().and_((function(){return self.nodes().last().isReturn();}))){return self;};;
self.addNode_(smalltalk.ReturnNode.variable_at_("self",self.end()));
return self;}
,"return",unescape("shouldReturn%0D%0A%09%22%20Ensure%20the%20receiver%20includes%20a%20return%20node.%20%22%0D%0A%0D%0A%09%28self%20nodes%20notEmpty%20and%3A%20%5B%0D%0A%09%09self%20nodes%20last%20isReturn%0D%0A%09%5D%29%20ifTrue%3A%20%5B%20%5Eself%5D.%0D%0A%0D%0A%09self%20addNode%3A%20%28ReturnNode%20variable%3A%20%23self%20at%3A%20self%20end%29."));
smalltalk.bind(smalltalk.SequenceNode,"validateNames:",0
,function(names){var $s8Ret$={name:"stReturn"};try{var self=this;
self.tempNames().do_((function(temp){return ((names.includes_(temp)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return self.error_(unescape("Invalid%20temp%20%28")._comma(temp)._comma(unescape("%29")))})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"validation",unescape("validateNames%3A%20names%0D%0A%09%22%20Validate%20the%20receiver%20can%20be%20emmited%20in%20a%20context%20with%20names.%20%22%0D%0A%0D%0A%09self%20tempNames%20do%3A%20%5B%3Atemp%7C%0D%0A%09%09%28names%20includes%3A%20temp%29%20ifTrue%3A%20%5B%0D%0A%09%09%09%5Eself%20error%3A%20%27Invalid%20temp%20%28%27%2Ctemp%2C%27%29%27%0D%0A%09%09%5D.%0D%0A%09%5D"));
smalltalk.bind(smalltalk.BlockSequenceNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitBlockSequenceNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitBlockSequenceNode%3A%20self"));
smalltalk.bind(smalltalk.ExtendedSequenceNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitExtendedSequenceNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitExtendedSequenceNode%3A%20self"));
smalltalk.bind(smalltalk.ExtendedSequenceNode,"directives",0
,function(){var self=this;
return (((nil.isNil_(self["@directives"]))).mustBeBoolean()==true ? (function(){return [];})() : (function(){return self["@directives"];})());}
,"accessing",unescape("directives%0D%0A%09%22%20Return%20the%20directives%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Edirectives%20isNil%0D%0A%09%09ifTrue%3A%20%5B%20%23%28%29%20%5D%0D%0A%09%09ifFalse%3A%20%5Bdirectives%5D"));
smalltalk.bind(smalltalk.ExtendedSequenceNode,"directives:",0
,function(aCollection){var self=this;
self["@directives"]=aCollection;
return self;}
,"accessing",unescape("directives%3A%20aCollection%0D%0A%09%22%20Set%20the%20directives%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09directives%20%3A%3D%20aCollection"));
smalltalk.bind(smalltalk.AssignmentNode,"left",0
,function(){var self=this;
return self["@left"];}
,"accessing",unescape("left%0D%0A%09%5Eleft"));
smalltalk.bind(smalltalk.AssignmentNode,"left:",0
,function(aNode){var self=this;
self["@left"]=aNode;
return self;}
,"accessing",unescape("left%3A%20aNode%0D%0A%09left%20%3A%3D%20aNode"));
smalltalk.bind(smalltalk.AssignmentNode,"right",0
,function(){var self=this;
return self["@right"];}
,"accessing",unescape("right%0D%0A%09%5Eright"));
smalltalk.bind(smalltalk.AssignmentNode,"right:",0
,function(aNode){var self=this;
self["@right"]=aNode;
return self;}
,"accessing",unescape("right%3A%20aNode%0D%0A%09right%20%3A%3D%20aNode"));
smalltalk.bind(smalltalk.AssignmentNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitAssignmentNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitAssignmentNode%3A%20self"));
smalltalk.bind(smalltalk.AssignmentNode,"isAssignment",0
,function(){var self=this;
return true;}
,"testing",unescape("isAssignment%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20an%20assignment%20node.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.AssignmentNode,"do:in:",0
,function(aBlock,aNode){var self=this;
smalltalk.superImplementor(smalltalk.AssignmentNode,"do_in_").apply(self, [aBlock,aNode]);
if ((nil.isNil_(self["@left"])===false)){self["@left"].do_in_(aBlock,self);};;
if ((nil.isNil_(self["@right"])===false)){self["@right"].do_in_(aBlock,self);};;
return self;}
,"iteration",unescape("do%3A%20aBlock%20in%3A%20aNode%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20contents%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09super%20do%3A%20aBlock%20in%3A%20aNode.%0D%0A%09left%20%20notNil%20ifTrue%3A%20%5B%20left%20%20do%3A%20aBlock%20in%3A%20self%20%5D.%0D%0A%09right%20notNil%20ifTrue%3A%20%5B%20right%20do%3A%20aBlock%20in%3A%20self%20%5D."));
smalltalk.bind(smalltalk.AssignmentNode,"with:except:do:",0
,function(testBlock,except,aBlock){var self=this;
if ((nil.isNil_(smalltalk.superImplementor(smalltalk.AssignmentNode,"with_except_do_").apply(self, [testBlock,except,aBlock])))){return nil;};;
if ((nil.isNil_(self["@right"])===false)){((testBlock.value_(self["@right"])).mustBeBoolean()==true ? (function(){return aBlock.value_value_(self["@right"],self);})() : (function(){return self["@right"].with_except_do_(testBlock,except,aBlock);})());};;
return self;}
,"iteration",unescape("with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%0D%0A%09%22%20Evaluate%20aBlock%20for%20all%20nodes%20%28returning%20true%20when%20evaluated%20with%20testBlock%29%20in%20the%20local%20context%20of%20receiver.%20%22%0D%0A%0D%0A%09%28super%20with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%29%20isNil%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09right%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%28testBlock%20value%3A%20right%29%0D%0A%09%09ifTrue%3A%20%5B%20aBlock%20value%3A%20right%20value%3A%20self%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20right%20with%3A%20testBlock%20except%3A%20except%20do%3A%20aBlock%20%5D%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.AssignmentNode,"prepareIn:for:",0
,function(aNode,anEncoder){var self=this;
smalltalk.superImplementor(smalltalk.AssignmentNode,"prepareIn_for_").apply(self, [aNode,anEncoder]);
self["@right"].client_(self);
return self;}
,"prepare",unescape("prepareIn%3A%20aNode%20for%3A%20anEncoder%0D%0A%09%22%20Prepare%20the%20receiver%20to%20be%20encoded%20by%20anEncoder%20in%20aNode.%20%22%0D%0A%0D%0A%09super%20prepareIn%3A%20aNode%20for%3A%20anEncoder.%0D%0A%09right%20client%3A%20self"));
smalltalk.bind(smalltalk.ReturnNode.$klass,"variable:at:",0
,function(aName,offset){var self=this;
return (function($rec){$rec.addNode_((function($rec){$rec.value_(aName);$rec.start_end_(offset,offset);return $rec.yourself();})(smalltalk.VariableNode.$$new()));$rec.start_end_(offset,offset);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("variable%3A%20aName%20at%3A%20offset%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09addNode%3A%20%28VariableNode%20new%0D%0A%09%09%09value%3A%20aName%3B%0D%0A%09%09%09start%3A%20offset%20end%3A%20offset%3B%0D%0A%09%09%09yourself%29%3B%0D%0A%09%09start%3A%20offset%20end%3A%20offset%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.ReturnNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitReturnNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitReturnNode%3A%20self"));
smalltalk.bind(smalltalk.ReturnNode,"isReturn",0
,function(){var self=this;
return true;}
,"testing",unescape("isReturn%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.ValueNode.$klass,"with:",0
,function(aValue){var self=this;
return self.$$new().value_(aValue);}
,"instantiation",unescape("with%3A%20aValue%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%20value%3A%20aValue"));
smalltalk.bind(smalltalk.ValueNode,"value",0
,function(){var self=this;
return self["@value"];}
,"accessing",unescape("value%0D%0A%09%5Evalue"));
smalltalk.bind(smalltalk.ValueNode,"value:",0
,function(anObject){var self=this;
self["@value"]=anObject;
return self;}
,"accessing",unescape("value%3A%20anObject%0D%0A%09value%20%3A%3D%20anObject"));
smalltalk.bind(smalltalk.ValueNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitValueNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitValueNode%3A%20self"));
smalltalk.bind(smalltalk.ValueNode,"hasNumber",0
,function(){var self=this;
return self["@value"].isNumber();}
,"testing",unescape("hasNumber%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20has%20a%20numeric%20literal%20value.%20%22%0D%0A%0D%0A%09%5Evalue%20isNumber"));
smalltalk.bind(smalltalk.ValueNode,"hasString",0
,function(){var self=this;
return self["@value"].isString();}
,"testing",unescape("hasString%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20has%20a%20string%20literal%20value.%20%22%0D%0A%0D%0A%09%5Evalue%20isString"));
smalltalk.bind(smalltalk.ValueNode,"isValue",0
,function(){var self=this;
return true;}
,"testing",unescape("isValue%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.VariableNode,"localNameIn:",0
,function(aCompiler){var self=this;
return aCompiler.localName_(self.value());}
,"accessing",unescape("localNameIn%3A%20aCompiler%0D%0A%09%22%20Return%20the%20name%20of%20the%20receiver%20to%20be%20used%20by%20aCompiler%20locally.%20%22%0D%0A%0D%0A%09%5EaCompiler%20localName%3A%20self%20value%20"));
smalltalk.bind(smalltalk.VariableNode,"needsVerification",0
,function(){var self=this;
return true;}
,"testing",unescape("needsVerification%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20need%20verification.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.VariableNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitVariableNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitVariableNode%3A%20self"));
smalltalk.bind(smalltalk.VariableNode,"isSuper",0
,function(){var self=this;
return self["@value"]._eq("super");}
,"testing",unescape("isSuper%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20super.%20%22%0D%0A%0D%0A%09%5Evalue%20%3D%20%27super%27"));
smalltalk.bind(smalltalk.VariableNode,"indices",0
,function(){var self=this;
return (((nil.isNil_(self["@indices"]))).mustBeBoolean()==true ? (function(){return smalltalk.Array.$$new();})() : (function(){return self["@indices"];})());}
,"accessing",unescape("indices%0D%0A%09%22%20Return%20the%20indexing%20references%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eindices%20isNil%20ifTrue%3A%20%5B%20Array%20new%20%5D%20ifFalse%3A%20%5B%20indices%20%5D"));
smalltalk.bind(smalltalk.VariableNode,"reference:",0
,function(chain){var self=this;
self["@indices"]=((chain.size()._gt((1)).and_((function(){return chain.second().notEmpty();}))).mustBeBoolean()==true ? (function(){return chain.second().collect_((function(tuple){return ((tuple.second().isString()).mustBeBoolean()==true ? (function(){return smalltalk.ValueNode.$$new().value_(tuple.second());})() : (function(){return tuple.second();})());}));})() : nil);
if (chain.size()._gt((2)).and_((function(){return chain.third().notEmpty();}))){self.annotations_(chain.third());};;
return self;}
,"visiting",unescape("reference%3A%20chain%0D%0A%09%22%20The%20receiver%20is%20a%20reference%20defined%20by%20chain.%20%22%0D%0A%0D%0A%09indices%20%3A%3D%20%28chain%20size%20%3E%201%20and%3A%20%5B%20chain%20second%20notEmpty%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09chain%20second%20collect%3A%20%5B%3Atuple%7C%0D%0A%09%09%09tuple%20second%20isString%0D%0A%09%09%09ifTrue%3A%20%5B%20ValueNode%20new%20value%3A%20tuple%20second%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20tuple%20second%20%5D%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%28chain%20size%20%3E%202%20and%3A%20%5B%20chain%20third%20notEmpty%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20annotations%3A%20chain%20third%0D%0A%09%5D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.VariableNode,"isVariable",0
,function(){var self=this;
return true;}
,"testing",unescape("isVariable%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.GlobalReferenceNode,"accept:",0
,function(aVisitor){var self=this;
aVisitor.visitGlobalReferenceNode_(self);
return self;}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09aVisitor%20visitGlobalReferenceNode%3A%20self"));
smalltalk.bind(smalltalk.NoReceiverNode,"accept:",0
,function(aVisitor){var self=this;
return self;}
,"visiting",unescape("accept%3A%20aVisitor"));
smalltalk.bind(smalltalk.NoReceiverNode,"messageSendConnector",0
,function(){var self=this;
return nil;}
,"accessing",unescape("messageSendConnector%0D%0A%09%22%20Return%20the%20connector%20used%20to%20encode%20message%20sent%20to%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Enil%20"));
smalltalk.bind(smalltalk.AlienNode,"localNameIn:",0
,function(aCompiler){var self=this;
return aCompiler.alienName_(self.value());}
,"accessing",unescape("localNameIn%3A%20aCompiler%0D%0A%09%22%20Return%20the%20name%20of%20the%20receiver%20to%20be%20used%20by%20aCompiler%20locally.%20%22%0D%0A%0D%0A%09%5EaCompiler%20alienName%3A%20self%20value%20"));
smalltalk.bind(smalltalk.AlienNode,"needsVerification",0
,function(){var self=this;
return false;}
,"testing",unescape("needsVerification%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20need%20verification.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.InlineNode.$klass,"sources:parser:",0
,function(tuples,parser){var self=this;
return self.basicNew().initialize_parser_(tuples,parser);}
,"instantiation",unescape("sources%3A%20tuples%20parser%3A%20parser%0D%0A%09%5Eself%20basicNew%0D%0A%09%09initialize%3A%20tuples%0D%0A%09%09parser%3A%20parser%20"));
smalltalk.bind(smalltalk.InlineNode,"initialize:parser:",0
,function(tuples,parser){var self=this;
self["@sources"]=tuples;
self.capture_with_("s8",parser);
return self;}
,"initialize",unescape("initialize%3A%20tuples%20parser%3A%20parser%0D%0A%09%22%20Private%20-%20Initialize%20the%20receiver%20%22%0D%0A%0D%0A%09sources%20%3A%3D%20tuples.%0D%0A%09self%20capture%3A%20%23s8%20with%3A%20parser.%20"));
smalltalk.bind(smalltalk.InlineNode,"sourceFor:in:",0
,function(lang,encoder){var $s8Ret$={name:"stReturn"};try{var self=this;
var $$default=nil;
self["@sources"].do_((function(tuple){var code=nil;
code=tuple.second().value();if (tuple.first()._eq(lang)){(function(){$s8Ret$.result=(function(){return code})();throw($s8Ret$)})();};;return (((nil.isNil_(tuple.first()))).mustBeBoolean()==true ? (function(){return $$default=code;})() : nil);}));
if ((nil.isNil_(encoder)===false)){var result=nil;
result=encoder.inline_source_(self,$$default);
(((nil.isNil_(result)===false)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return result})();throw($s8Ret$)})();})() : nil);};;
(function(){$s8Ret$.result=(function(){return $$default})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"accessing",unescape("sourceFor%3A%20lang%20in%3A%20encoder%0D%0A%09%22%20Return%20the%20native%20source%20code%20to%20inline%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20default%20%7C%0D%0A%09sources%20do%3A%20%5B%3Atuple%7C%20%7C%20code%20%7C%0D%0A%09%09code%20%3A%3D%20tuple%20second%20value.%0D%0A%09%09tuple%20first%20%3D%20lang%20ifTrue%3A%20%5B%20%5Ecode%20%5D.%0D%0A%09%09tuple%20first%20isNil%20ifTrue%3A%20%5B%20default%20%3A%3D%20code%20%5D.%0D%0A%09%5D.%0D%0A%09encoder%20notNil%20ifTrue%3A%20%5B%20%7C%20result%20%7C%0D%0A%09%09result%20%3A%3D%20encoder%20inline%3A%20self%20source%3A%20default.%0D%0A%09%09result%20notNil%20ifTrue%3A%20%5B%20%5Eresult%20%5D%0D%0A%09%5D.%0D%0A%09%5Edefault"));
smalltalk.bind(smalltalk.InlineNode,"accept:",0
,function(aVisitor){var self=this;
return aVisitor.visitInlineNode_(self);}
,"visiting",unescape("accept%3A%20aVisitor%0D%0A%09%5EaVisitor%20visitInlineNode%3A%20self"));
smalltalk.bind(smalltalk.InlineNode,"isInline",0
,function(){var self=this;
return true;}
,"testing",unescape("isInline%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20of%20this%20type.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.InlineNode,"capture:with:",0
,function(lang,parser){var self=this;
self["@sources"].do_((function(tuple){return ((tuple.first()._eq(lang)).mustBeBoolean()==true ? (function(){return self.parse_at_with_(tuple.second().value(),tuple.second().start(),parser);})() : nil);}));
return self;}
,"internal",unescape("capture%3A%20lang%20with%3A%20parser%0D%0A%09%22%20Private%20-%20Capture%20internal%20contents%20from%20sources.%20%22%0D%0A%0D%0A%09sources%20do%3A%20%5B%3Atuple%7C%20tuple%20first%20%3D%20lang%20ifTrue%3A%20%5B%0D%0A%09%09self%20parse%3A%20tuple%20second%20value%20at%3A%20tuple%20second%20start%20with%3A%20parser%09%0D%0A%09%5D%5D"));
smalltalk.bind(smalltalk.InlineNode,"parse:at:with:",0
,function(code,offset,parser){var self=this;
var stream=nil;
var array=nil;
stream="".stream();
(function(){for(var $1$=(offset);$1$>0;$1$--){(function(){return stream.nextPut_(" ");})()};return nil})();
(function($rec){$rec.nextPutAll_(code);return $rec.position_(offset);})(stream);
array=parser._comma(smalltalk.PPEOFParser.$$new()).parse_(stream);
if (!(array.isArray().and_((function(){return array.size()._gt((1));})))){self.error_("Invalid inline code "._comma(code.asString()));};;
self.addNode_(array.second());
return self;}
,"internal",unescape("parse%3A%20code%20at%3A%20offset%20with%3A%20parser%0D%0A%09%22%20Private%20-%20Parse%20internal%20contents%20from%20code%20using%20parser.%20%22%0D%0A%0D%0A%09%7C%20stream%20array%20%7C%0D%0A%09stream%20%3A%3D%20%27%27%20stream.%0D%0A%09offset%20timesRepeat%3A%20%5B%20stream%20nextPut%3A%20%24%20%5D.%0D%0A%09stream%20nextPutAll%3A%20code%3B%20position%3A%20offset.%0D%0A%09array%20%3A%3D%20parser%20%2CPPEOFParser%20new%20parse%3A%20stream.%0D%0A%09%28array%20isArray%20and%3A%20%5Barray%20size%20%3E%201%5D%29%20ifFalse%3A%20%5B%0D%0A%09%09self%20error%3A%20%27Invalid%20inline%20code%20%27%20%2Ccode%20asString%0D%0A%09%5D.%0D%0A%09self%20addNode%3A%20array%20second"));
smalltalk.bind(smalltalk.DirectiveNode,"description",0
,function(){var self=this;
return self.$klass.name();}
,"visiting",unescape("description%0D%0A%09%22%20Return%20the%20description%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20class%20name"));
smalltalk.bind(smalltalk.DirectiveNode,"selector",0
,function(){var self=this;
return "directive:arguments:".asSelector();}
,"visiting",unescape("selector%0D%0A%09%22%20Private%20-%20Return%20the%20internal%20selector%20used%20to%20execute%20directive.%20%22%0D%0A%0D%0A%09%5E%23directive%3Aarguments%3A%20asSelector"));
smalltalk.bind(smalltalk.DirectiveNode,"literal",0
,function(){var self=this;
return self.implementedBySubclass();}
,"visiting",unescape("literal%0D%0A%09%22%20Private%20-%20Return%20the%20literal%20to%20embedd%20in%20directive%20call.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass"));
smalltalk.bind(smalltalk.DirectiveNode,"asNative:for:on:",0
,function(selector,anEncoder,aStream){var self=this;
var call=nil;
call=anEncoder.sourceToDirectCall_with_(self.selector(),self.literal());
return anEncoder.sourceToTry_(call);}
,unescape("code%20generation"),unescape("asNative%3A%20selector%20for%3A%20anEncoder%20on%3A%20aStream%0D%0A%09%22%20Return%20the%20native%20code%20to%20emmit%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20call%20%7C%0D%0A%09call%20%3A%3D%20anEncoder%0D%0A%09%09sourceToDirectCall%3A%20self%20selector%0D%0A%09%09with%3A%20self%20literal.%0D%0A%09%5EanEncoder%20sourceToTry%3A%20call%20"));
smalltalk.bind(smalltalk.JSDirectiveNode.$klass,"code:protected:",0
,function(aCode,useTry){var self=this;
return (function($rec){$rec.code_(aCode);$rec.protected_(useTry);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("code%3A%20aCode%20protected%3A%20useTry%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09code%3A%20aCode%3B%0D%0A%09%09protected%3A%20useTry%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.JSDirectiveNode,"code:",0
,function(aCode){var self=this;
self["@code"]=aCode;
return self;}
,"accessing",unescape("code%3A%20aCode%0D%0A%09%22%20Set%20the%20code%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09code%20%3A%3D%20aCode"));
smalltalk.bind(smalltalk.JSDirectiveNode,"nativeCodeFor:",0
,function(anEncoder){var self=this;
if (self["@code"].isString()){return self["@code"];};;
return self["@code"].evaluateWith_(anEncoder);}
,"accessing",unescape("nativeCodeFor%3A%20anEncoder%0D%0A%09%22%20Private%20-%20Return%20the%20native%20code%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09code%20isString%20ifTrue%3A%20%5B%20%5Ecode%20%5D.%0D%0A%09%5Ecode%20evaluateWith%3A%20anEncoder%20"));
smalltalk.bind(smalltalk.JSDirectiveNode,"protected:",0
,function(aBoolean){var self=this;
self["@protected"]=aBoolean;
return self;}
,"accessing",unescape("protected%3A%20aBoolean%0D%0A%09%22%20Set%20the%20receiver%20to%20use%20try/catch%20to%20handle%20failure.%20%22%0D%0A%0D%0A%09protected%20%3A%3D%20aBoolean"));
smalltalk.bind(smalltalk.JSDirectiveNode,"asNative:for:on:",0
,function(selector,anEncoder,aStream){var self=this;
var string=nil;
string=self.nativeCodeFor_(anEncoder);
if ((nil.is_eqeq_(self["@protected"],true))){return anEncoder.sourceToTry_(string);};;
return string;}
,unescape("code%20generation"),unescape("asNative%3A%20selector%20for%3A%20anEncoder%20on%3A%20aStream%0D%0A%09%22%20Return%20the%20native%20code%20to%20emmit%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20string%20%7C%0D%0A%09string%20%3A%3D%20self%20nativeCodeFor%3A%20anEncoder.%0D%0A%09protected%20%3D%3D%20true%20ifTrue%3A%20%5B%0D%0A%09%09%5EanEncoder%20sourceToTry%3A%20string%0D%0A%09%5D.%0D%0A%09%5Estring%20"));
smalltalk.bind(smalltalk.InlineOptimizationNode,"args",0
,function(){var self=this;
return self["@args"];}
,"accessing",unescape("args%0D%0A%09%22%20Return%20the%20args%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eargs"));
smalltalk.bind(smalltalk.InlineOptimizationNode,"args:",0
,function(anObject){var self=this;
self["@args"]=anObject;
return self;}
,"accessing",unescape("args%3A%20anObject%0D%0A%09%22%20Set%20the%20args%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09args%20%3A%3D%20anObject"));
smalltalk.bind(smalltalk.InlineOptimizationNode,"remove",0
,function(){var self=this;
return self["@remove"];}
,"accessing",unescape("remove%0D%0A%09%22%20Return%20the%20remove%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eremove"));
smalltalk.bind(smalltalk.InlineOptimizationNode,"remove:",0
,function(anObject){var self=this;
self["@remove"]=anObject;
return self;}
,"accessing",unescape("remove%3A%20anObject%0D%0A%09%22%20Set%20the%20remove%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09remove%20%3A%3D%20anObject"));
smalltalk.bind(smalltalk.InlineOptimizationNode,"asNative:for:on:",0
,function(selector,anEncoder,aStream){var self=this;
if (self.remove()){anEncoder.removeOptimizations_(self["@args"]);} else{anEncoder.addOptimizations_(self["@args"]);};;
return nil;}
,unescape("code%20generation"),unescape("asNative%3A%20selector%20for%3A%20anEncoder%20on%3A%20aStream%0D%0A%09%22%20Return%20the%20native%20code%20to%20emmit%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20remove%0D%0A%09%09ifTrue%3A%20%5B%20anEncoder%20removeOptimizations%3A%20args%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20anEncoder%20addOptimizations%3A%20args%20%5D.%0D%0A%09%5Enil%20"));
smalltalk.bind(smalltalk.PPFailure.$klass,"reason:at:",0
,function(aString,anInteger){var self=this;
return self.$$new().reason_at_(aString,anInteger);}
,"instantiation",unescape("reason%3A%20aString%20at%3A%20anInteger%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%20reason%3A%20aString%20at%3A%20anInteger"));
smalltalk.bind(smalltalk.PPFailure,"position",0
,function(){var self=this;
if ((nil.isNil_(self["@position"]))){return (0);};;
return self["@position"];}
,"accessing",unescape("position%0D%0A%09%22%20Return%20the%20position%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09position%20isNil%20ifTrue%3A%20%5B%20%5E0%20%5D.%0D%0A%09%5Eposition"));
smalltalk.bind(smalltalk.PPFailure,"position:",0
,function(aNumber){var self=this;
self["@position"]=aNumber;
return self;}
,"accessing",unescape("position%3A%20aNumber%0D%0A%09%22%20Set%20the%20position%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09position%20%3A%3D%20aNumber"));
smalltalk.bind(smalltalk.PPFailure,"toString",0
,function(){var self=this;
return "Failure "._comma(self.reason().printString())._comma(unescape("%28@"))._comma(self.position())._comma(unescape("%29"));}
,"printing",unescape("toString%0D%0A%09%22%20Return%20the%20receiver%20converted%20to%20String.%20%22%0D%0A%0D%0A%09%5E%27Failure%20%27%2Cself%20reason%20printString%20%2C%27%28@%27%2Cself%20position%2C%27%29%27"));
smalltalk.bind(smalltalk.PPFailure,"reason",0
,function(){var self=this;
if ((nil.isNil_(self["@reason"]))){return smalltalk.String.$$new();};;
return self["@reason"];}
,"accessing",unescape("reason%0D%0A%09%22%20Return%20the%20reason%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09reason%20isNil%20ifTrue%3A%20%5B%20%5EString%20new%20%5D.%0D%0A%09%5Ereason"));
smalltalk.bind(smalltalk.PPFailure,"reason:",0
,function(aString){var self=this;
self["@reason"]=""._comma(aString);
return self;}
,"accessing",unescape("reason%3A%20aString%0D%0A%09%22%20Set%20the%20reason%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09reason%20%3A%3D%20%27%27%20%2CaString"));
smalltalk.bind(smalltalk.PPFailure,"reason:at:",0
,function(aString,anInteger){var self=this;
(function($rec){$rec.reason_(aString);return $rec.position_(anInteger);})(self);
return self;}
,"accessing",unescape("reason%3A%20aString%20at%3A%20anInteger%0D%0A%09%22%20Set%20the%20reason%20and%20position%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%0D%0A%09%09reason%3A%20aString%3B%20%0D%0A%09%09position%3A%20anInteger"));
smalltalk.bind(smalltalk.PPFailure,"isParseFailure",0
,function(){var self=this;
return true;}
,"testing",unescape("isParseFailure%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20a%20parse%20failure.%20%22%0D%0A%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.PPFailure,"accept:",0
,function(anEncoder){var self=this;
var description=nil;
description="PPFailure: "._comma(self.reason())._comma(" at: ")._comma(self.position().asString())._comma(" source: ")._comma(anEncoder.stream().contents());
self.print_(description);
self.error_(description);
return self;}
,"bug",unescape("accept%3A%20anEncoder%0D%0A%09%22%20Report%20compilation%20failure%20and%20error.%20%22%0D%0A%0D%0A%09%7C%20description%20%7C%0D%0A%09description%20%3A%3D%20%27PPFailure%3A%20%27%2C%20self%20reason%0D%0A%09%09%2C%27%20at%3A%20%27%20%2Cself%20position%20asString%0D%0A%09%09%2C%27%20source%3A%20%27%20%2CanEncoder%20stream%20contents.%0D%0A%09self%20print%3A%20description.%0D%0A%09self%20error%3A%20description"));
smalltalk.bind(smalltalk.PPFailure,"__tostring",0
,function(){var self=this;
return self.toString();}
,"Lua-printing",unescape("__tostring%0D%0A%09%22%20Return%20the%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20toString"));
smalltalk.bind(smalltalk.PPParser.$klass,",","_comma"
,function(anObject){var self=this;
return anObject.asParser();}
,"instantiation",unescape("%2C%20anObject%0D%0A%09%22%20Return%20the%20receiver%20composed%20with%20the%20argument.%20%22%0D%0A%0D%0A%09%5EanObject%20asParser"));
smalltalk.bind(smalltalk.PPParser.$klass,"optional:",0
,function(anObject){var self=this;
return self._comma(anObject).optional();}
,"instantiation",unescape("optional%3A%20anObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20to%20parse%20an%20optional%20element%20anObject.%20%22%0D%0A%0D%0A%09%5E%28self%20%2CanObject%29%20optional"));
smalltalk.bind(smalltalk.PPParser.$klass,"debug:or:",0
,function(action,aBlock){var self=this;
var show=nil;
var result=nil;
show=((function(){var $1$=(smalltalk.PPParser["debugOn"]);if(nil.isNil_($1$))return ((function(){result=aBlock.evaluate();return false;}).value());return $1$;})());
if (show._eq(true)){result=action.evaluate();};;
return result;}
,"debug",unescape("debug%3A%20action%20or%3A%20aBlock%0D%0A%09%22%20Log%20evaluation%20of%20action%20if%20in%20debugging%20mode%2C%0D%0A%09%20else%20returns%20the%20result%20of%20evaluation%20of%20aBlock.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20show%20result%20%7C%0D%0A%09show%20%3A%3D%20PPParser%20basicAt%3A%20%23debugOn%20ifAbsent%3A%5B%20result%20%3A%3D%20aBlock%20evaluate.%20false%20%5D.%0D%0A%09show%20%3D%20true%20ifTrue%3A%20%5B%20result%20%3A%3D%20action%20evaluate%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.PPParser.$klass,"debugWhile:",0
,function(aBlock){var self=this;
return (function(){(smalltalk.PPParser["debugOn"]=(true));return aBlock.evaluate();}).ensure_((function(){return (smalltalk.PPParser["debugOn"]=(false));}));}
,"debug",unescape("debugWhile%3A%20aBlock%0D%0A%09%5E%5B%09PPParser%20basicAt%3A%20%23debugOn%20put%3A%20true.%0D%0A%09%09aBlock%20evaluate%0D%0A%09%5D%20ensure%3A%20%5B%0D%0A%09%09PPParser%20basicAt%3A%20%23debugOn%20put%3A%20false%0D%0A%09%5D"));
smalltalk.bind(smalltalk.PPParser,"set:start:end:",0
,function(anElement,start,end){var self=this;
if (smalltalk.responds_to_(anElement,("start:end:"))){anElement.start_end_(start,end);};;
return self;}
,"private",unescape("set%3A%20anElement%20start%3A%20start%20end%3A%20end%0D%0A%09%22%20Private%20-%20Set%20start%26end%20source%20offest%20of%20anElement.%20%22%0D%0A%0D%0A%09%28anElement%20respondsTo%3A%20%23start%3Aend%3A%29%20ifTrue%3A%20%5B%0D%0A%09%09anElement%20start%3A%20start%20end%3A%20end%0D%0A%09%5D"));
smalltalk.bind(smalltalk.PPParser,"flatten",0
,function(){var self=this;
return smalltalk.PPFlattenParser.on_(self);}
,"operations",unescape("flatten%0D%0A%09%22%20Return%20a%20flattened%20version%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EPPFlattenParser%20on%3A%20self"));
smalltalk.bind(smalltalk.PPParser,"withSource",0
,function(){var self=this;
return smalltalk.PPSourceParser.on_(self);}
,"operations",unescape("withSource%0D%0A%09%22%20Return%20a%20source%20parser%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EPPSourceParser%20on%3A%20self"));
smalltalk.bind(smalltalk.PPParser,"==>","_eq_eq_gt"
,function(aBlock){var self=this;
return smalltalk.PPActionParser.on_block_(self,aBlock);}
,"operations",unescape("%3D%3D%3E%20aBlock%0D%0A%09%22%20Return%20an%20action%20parser%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EPPActionParser%20on%3A%20self%20block%3A%20aBlock"));
smalltalk.bind(smalltalk.PPParser,",","_comma"
,function(aParser){var self=this;
return smalltalk.PPSequenceParser.with_with_(self,aParser);}
,"operations",unescape("%2C%20aParser%0D%0A%09%22%20Return%20the%20receiver%20composed%20with%20the%20argument.%20%22%0D%0A%0D%0A%09%5EPPSequenceParser%20with%3A%20self%20with%3A%20aParser"));
smalltalk.bind(smalltalk.PPParser,"/","_slash"
,function(aParser){var self=this;
return smalltalk.PPChoiceParser.with_with_(self,aParser);}
,"operations",unescape("/%20aParser%0D%0A%09%22%20Return%20a%20choice%20composite%20version%20of%20the%20receiver%20and%20aParser.%20%22%0D%0A%0D%0A%09%5EPPChoiceParser%20with%3A%20self%20with%3A%20aParser"));
smalltalk.bind(smalltalk.PPParser,"@","_at"
,function(tag){var self=this;
(self["debugTag"]=(tag));
return self._eq_eq_gt((function(node){self.note_(tag._comma(unescape("%3D%3D%3E"))._comma(node.printString()));return node;}));}
,"debug",unescape("@%20tag%0D%0A%09%22%20DEBUG%20-%20Return%20a%20notification%20wrapper%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20basicAt%3A%20%23debugTag%20put%3A%20tag.%0D%0A%09%5Eself%20%3D%3D%3E%20%5B%3Anode%7C%20self%20note%3A%20tag%20%2C%27%3D%3D%3E%27%2Cnode%20printString.%20node%20%5D"));
smalltalk.bind(smalltalk.PPParser,"@@","_at_at"
,function(tag){var self=this;
(self["debugTag"]=(tag));
return self;}
,"debug",unescape("@@%20tag%0D%0A%09%22%20DEBUG%20-%20Tag%20the%20receiver%20for%20debugging.%20%22%0D%0A%0D%0A%09self%20basicAt%3A%20%23debugTag%20put%3A%20tag"));
smalltalk.bind(smalltalk.PPParser,"plus",0
,function(){var self=this;
return smalltalk.PPRepeatingParser.on_min_(self,(1))._at_at("plus");}
,"operations",unescape("plus%0D%0A%09%22%20Return%20an%20iterative%20version%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28PPRepeatingParser%20on%3A%20self%20min%3A%201%29%20@@%23plus"));
smalltalk.bind(smalltalk.PPParser,"star",0
,function(){var self=this;
return smalltalk.PPRepeatingParser.on_min_(self,(0))._at_at("star");}
,"operations",unescape("star%0D%0A%09%22%20Return%20an%20iterative%20version%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28PPRepeatingParser%20on%3A%20self%20min%3A%200%29%20@@%23star"));
smalltalk.bind(smalltalk.PPParser,"not",0
,function(){var self=this;
return smalltalk.PPNotParser.on_(self)._at_at("not");}
,"operations",unescape("not%0D%0A%09%22%20Return%20the%20negation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%28PPNotParser%20on%3A%20self%29%20@@%23not"));
smalltalk.bind(smalltalk.PPParser,"optional",0
,function(){var self=this;
return self._slash(smalltalk.PPEpsilonParser.$$new())._at_at("opt");}
,"operations",unescape("optional%0D%0A%09%22%20Return%20an%20optional%20version%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20/%20PPEpsilonParser%20new%20@@%23opt"));
smalltalk.bind(smalltalk.PPParser,"isParseFailure:",0
,function(what){var self=this;
return what.isParseFailure();}
,"test",unescape("isParseFailure%3A%20what%0D%0A%09%22%20Private%20-%20Return%20true%20if%20what%20is%20a%20parsing%20failure.%20%22%0D%0A%0D%0A%09%5Ewhat%20isParseFailure"));
smalltalk.bind(smalltalk.PPParser,"valueToMemoize:start:",0
,function(aStream,start){var self=this;
var end=nil;
var node=nil;
node=self.parse_(aStream);
end=aStream.position();
self.set_start_end_(node,start,end);
return smalltalk.Array.with_with_(node,end);}
,"memo",unescape("valueToMemoize%3A%20aStream%20start%3A%20start%0D%0A%09%22%20Private%20-%20Return%20the%20value%20to%20memoize%20as%20result%20of%20parsing%20en%20element%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20end%20node%20%7C%0D%0A%09node%20%3A%3D%20self%20parse%3A%20aStream.%0D%0A%09end%20%3A%3D%20aStream%20position.%0D%0A%09self%20set%3A%20node%20start%3A%20start%20end%3A%20end.%0D%0A%09%5EArray%20with%3A%20node%20with%3A%20end"));
smalltalk.bind(smalltalk.PPParser,"memoizedParse:",0
,function(aStream){var self=this;
var start=nil;
var value=nil;
if ((nil.isNil_(self["@memo"]))){self["@memo"]=smalltalk.PoolDictionary.$$new();};;
start=aStream.position();
value=self["@memo"].at_ifAbsentPut_(start,(function(){return self.valueToMemoize_start_(aStream,start);}));
aStream.position_(value.second());
return value.first();}
,"memo",unescape("memoizedParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Evaluate%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20start%20value%20%7C%0D%0A%09memo%20isNil%20ifTrue%3A%20%5B%20memo%20%3A%3D%20PoolDictionary%20new%20%5D.%0D%0A%09start%20%3A%3D%20aStream%20position.%0D%0A%09value%20%3A%3D%20memo%20at%3A%20start%20ifAbsentPut%3A%20%5B%0D%0A%09%09self%20valueToMemoize%3A%20aStream%20start%3A%20start%0D%0A%09%5D.%0D%0A%09aStream%20position%3A%20value%20second.%0D%0A%09%5Evalue%20first"));
smalltalk.bind(smalltalk.PPParser,"basicParse:",0
,function(aStream){var self=this;
self.subclassResponsibility_("basicParse:");
return self;}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09self%20subclassResponsibility%3A%20%23basicParse%3A"));
smalltalk.bind(smalltalk.PPParser,"parseLog:",0
,function(aStream){var self=this;
var result=nil;
var old=nil;
old=aStream.position();
result=self.basicParse_(aStream);
if (!(self.isParseFailure_(result))){self.note_(unescape("%5B")._comma(old)._comma(unescape("-"))._comma(aStream.position())._comma(unescape("%5D%20"))._comma(self.printString()));};;
return result;}
,"parsing",unescape("parseLog%3A%20aStream%0D%0A%09%22%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20result%20old%20%7C%0D%0A%09old%20%3A%3D%20aStream%20position.%0D%0A%09result%20%3A%3D%20self%20basicParse%3A%20aStream.%0D%0A%09%28self%20isParseFailure%3A%20result%29%20ifFalse%3A%20%5B%0D%0A%09%09self%20note%3A%20%24%5B%20%2Cold%20%2C%24-%20%2CaStream%20position%20%2C%27%5D%20%27%20%2Cself%20printString%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.PPParser,"parse:",0
,function(aStream){var self=this;
return smalltalk.PPParser.debug_or_((function(){return self.parseLog_(aStream);}),(function(){return self.basicParse_(aStream);}));}
,"parsing",unescape("parse%3A%20aStream%0D%0A%09%22%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%5EPPParser%0D%0A%09%09debug%3A%20%5Bself%20parseLog%3A%20aStream%5D%0D%0A%09%09or%3A%20%5Bself%20basicParse%3A%20aStream%5D"));
smalltalk.bind(smalltalk.PPParser,"parseAll:",0
,function(aStream){var self=this;
var result=nil;
result=smalltalk.PPSequenceParser.with_with_(self,smalltalk.PPEOFParser.$$new()).memoizedParse_(aStream);
return ((self.isParseFailure_(result)).mustBeBoolean()==true ? (function(){return self.error_(result.messageFor_(aStream.contents()));})() : (function(){return result.first();})());}
,"parsing",unescape("parseAll%3A%20aStream%0D%0A%09%22%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20%28PPSequenceParser%0D%0A%09%09with%3A%20self%20with%3A%20PPEOFParser%20new%29%0D%0A%09%09memoizedParse%3A%20aStream.%0D%0A%09%5E%28self%20isParseFailure%3A%20result%29%0D%0A%09%20%20%20%20ifTrue%3A%20%5B%20self%20error%3A%20%28result%20messageFor%3A%20aStream%20contents%29%20%5D%0D%0A%09%20%20%20%20ifFalse%3A%20%5B%20result%20first%20%5D"));
smalltalk.bind(smalltalk.PPParser,"asParser",0
,function(){var self=this;
return self;}
,"converting",unescape("asParser%0D%0A%09%22%20Return%20a%20parser%20on%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.PPParser,"printStringDetail",0
,function(){var self=this;
return "";}
,"printing",unescape("printStringDetail%0D%0A%09%22%20Private%20-%20Return%20a%20printable%20detail%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%27%27"));
smalltalk.bind(smalltalk.PPParser,"printString",0
,function(){var self=this;
var tag=nil;
tag=((function(){var $1$=(self["debugTag"]);if(nil.isNil_($1$))return nil;return $1$;})());
if ((nil.isNil_(tag)===false)){return unescape("@@")._comma(tag)._comma(unescape("@@"));};;
return "a"._comma(self.$klass.name())._comma((((nil.isNil_(tag))).mustBeBoolean()==true ? (function(){return self.printStringDetail();})() : (function(){return unescape("@@")._comma(tag);})()));}
,"printing",unescape("printString%0D%0A%09%22%20Return%20a%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20tag%20%7C%0D%0A%09tag%20%3A%3D%20self%20basicAt%3A%20%23debugTag.%0D%0A%09tag%20notNil%20ifTrue%3A%20%5B%20%5E%23@@%20%2Ctag%20%2C%23@@%20%5D.%20%22compact%20notation%22%0D%0A%0D%0A%09%5E%23a%20%2Cself%20class%20name%20%2C%28tag%20isNil%0D%0A%09%09ifTrue%3A%20%5B%20self%20printStringDetail%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20%27@@%27%2Ctag%20%5D%29"));
smalltalk.bind(smalltalk.PPEOFParser,"basicParse:",0
,function(aStream){var self=this;
if (aStream.atEnd()){return nil;};;
return smalltalk.PPFailure.reason_at_(unescape("EOF%20expected%20%28or%20invalid%20character%20in%20input%29"),aStream.position());}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09aStream%20atEnd%20ifTrue%3A%20%5B%20%5Enil%20%5D%20.%0D%0A%09%5EPPFailure%20reason%3A%20%27EOF%20expected%20%28or%20invalid%20character%20in%20input%29%27%20at%3A%20aStream%20position"));
smalltalk.bind(smalltalk.PPAnyParser,"basicParse:",0
,function(aStream){var self=this;
if (!(aStream.atEnd())){return aStream.next();};;
return smalltalk.PPFailure.reason_at_("End of stream",aStream.position());}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09aStream%20atEnd%20ifFalse%3A%20%5B%20%5EaStream%20next%20%5D.%0D%0A%09%5EPPFailure%20reason%3A%20%27End%20of%20stream%27%20at%3A%20aStream%20position"));
smalltalk.bind(smalltalk.PPEpsilonParser,"basicParse:",0
,function(aStream){var self=this;
return nil;}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%5Enil"));
smalltalk.bind(smalltalk.PPStringParser,"string",0
,function(){var self=this;
return self["@string"];}
,"accessing",unescape("string%0D%0A%09%22%20Return%20the%20string%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Estring"));
smalltalk.bind(smalltalk.PPStringParser,"string:",0
,function(aString){var self=this;
self["@string"]=aString;
return self;}
,"accessing",unescape("string%3A%20aString%0D%0A%09%22%20Set%20the%20string%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09string%20%3A%3D%20aString"));
smalltalk.bind(smalltalk.PPStringParser,"basicParse:",0
,function(aStream){var self=this;
var back=nil;
var reason=nil;
if (self["@string"].isEmpty()){return self["@string"];};;
back=aStream.position();
if (aStream.peekForAll_(self["@string"])){return self["@string"];};;
reason="Expecting "._comma(self["@string"].printString())._comma(" at ")._comma(back.printString())._comma(" read ")._comma(aStream.next_(self["@string"].size()).printString());
aStream.position_(back);
return smalltalk.PPFailure.reason_at_(reason,back);}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20back%20reason%20%7C%0D%0A%09string%20isEmpty%20ifTrue%3A%20%5B%20%5Estring%20%5D.%0D%0A%09back%20%3A%3D%20aStream%20position.%0D%0A%09%28aStream%20peekForAll%3A%20string%29%20ifTrue%3A%20%5B%20%5Estring%20%5D.%0D%0A%09reason%20%3A%3D%20%27Expecting%20%27%2C%20string%20printString%0D%0A%09%09%2C%27%20at%20%27%2Cback%20printString%0D%0A%09%09%2C%27%20read%20%27%2C%28aStream%20next%3A%20string%20size%29%20printString.%0D%0A%09aStream%20position%3A%20back.%0D%0A%09%5EPPFailure%20reason%3A%20reason%20at%3A%20back"));
smalltalk.bind(smalltalk.PPStringParser,"printStringDetail",0
,function(){var self=this;
return self["@string"].printString();}
,"printing",unescape("printStringDetail%0D%0A%09%22%20Private%20-%20Return%20a%20printable%20detail%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Estring%20printString"));
smalltalk.bind(smalltalk.PPStStringParser,"basicParse:",0
,function(aStream){var $s8Ret$={name:"stReturn"};try{var self=this;
var position=nil;
var result=nil;
position=aStream.position();
result="";
while(!(function(){result=result._comma(aStream.upTo_(unescape("%27")));return aStream.atEnd();})()){if (!(aStream.peek()._eq(unescape("%27")))){return result;};;
result=result._comma(aStream.next());};;
if (aStream.position()._gt(position)){aStream.position_(aStream.position()._minus((1)));
((aStream.next()._eq(unescape("%27"))).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return result})();throw($s8Ret$)})();})() : nil);};;
aStream.position_(position);
(function(){$s8Ret$.result=(function(){return smalltalk.PPFailure.reason_at_("Unfinished String",position)})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20position%20result%20%7C%0D%0A%09position%20%3A%3D%20aStream%20position.%0D%0A%09result%20%3A%3D%20%27%27.%0D%0A%09%5B%20result%20%3A%3D%20result%20%2C%20%28aStream%20upTo%3A%20%24%27%29.%0D%0A%09%20%20aStream%20atEnd%20%5D%20whileFalse%3A%20%5B%0D%0A%09%09aStream%20peek%20%3D%20%24%27%20ifFalse%3A%20%5B%20%5Eresult%20%5D.%0D%0A%09%09result%20%3A%3D%20result%20%2CaStream%20next.%0D%0A%09%5D.%0D%0A%09aStream%20position%20%3E%20position%20ifTrue%3A%20%5B%0D%0A%09%09aStream%20position%3A%20aStream%20position%20-%201.%0D%0A%09%09aStream%20next%20%3D%20%24%27%20ifTrue%3A%20%5B%20%5Eresult%20%5D.%0D%0A%09%5D.%0D%0A%09aStream%20position%3A%20position.%0D%0A%09%5EPPFailure%20reason%3A%20%27Unfinished%20String%27%20at%3A%20position"));
smalltalk.bind(smalltalk.PPCharacterParser.$klass,"separator",0
,function(){var self=this;
return self.$$new().separator();}
,"constants",unescape("separator%0D%0A%09%22%20Return%20the%20parser%20for%20separator.%20%22%0D%0A%0D%0A%09%5Eself%20new%20separator"));
smalltalk.bind(smalltalk.PPCharacterParser.$klass,"in:",0
,function(aCollection){var self=this;
var tag=nil;
tag=aCollection.asString();
if (aCollection.size()._eq((1))){tag=aCollection.first();
return self.for_doing_(tag.asString(),(function(c){return tag._eq(c);}));};;
return self.for_doing_(tag,(function(c){return aCollection.includes_(c);}));}
,"constants",unescape("in%3A%20aCollection%0D%0A%09%22%20Return%20the%20parser%20for%20input%20matching%20any%20element%20in%20aCollection.%20%22%0D%0A%0D%0A%09%7C%20tag%20%7C%0D%0A%09tag%20%3A%3D%20aCollection%20asString.%0D%0A%09aCollection%20size%20%3D%201%20ifTrue%3A%20%5B%0D%0A%09%09tag%20%3A%3D%20aCollection%20first.%0D%0A%09%09%5Eself%20for%3A%20tag%20asString%20doing%3A%20%5B%3Ac%7C%20tag%20%3D%20c%20%5D%0D%0A%09%5D.%0D%0A%09%5Eself%20for%3A%20tag%20doing%3A%20%5B%3Ac%7C%20aCollection%20includes%3A%20c%20%5D"));
smalltalk.bind(smalltalk.PPCharacterParser.$klass,"notIn:",0
,function(aCollection){var self=this;
var tag=nil;
tag=aCollection.asString();
if (aCollection.size()._eq((1))){tag=aCollection.first();
return self.for_doing_(tag.asString(),(function(c){return tag._no_eq(c);}));};;
return self.for_doing_(tag,(function(c){return aCollection.includes_(c).not();}));}
,"constants",unescape("notIn%3A%20aCollection%0D%0A%09%22%20Return%20the%20parser%20for%20input%20matching%20any%20element%20in%20aCollection.%20%22%0D%0A%0D%0A%09%7C%20tag%20%7C%0D%0A%09tag%20%3A%3D%20aCollection%20asString.%0D%0A%09aCollection%20size%20%3D%201%20ifTrue%3A%20%5B%0D%0A%09%09tag%20%3A%3D%20aCollection%20first.%0D%0A%09%09%5Eself%20for%3A%20tag%20asString%20doing%3A%20%5B%3Ac%7C%20tag%20%7E%3D%20c%20%5D%0D%0A%09%5D.%0D%0A%09%5Eself%20for%3A%20tag%20doing%3A%20%5B%3Ac%7C%20%28aCollection%20includes%3A%20c%29%20not%20%5D"));
smalltalk.bind(smalltalk.PPCharacterParser.$klass,"between:and:",0
,function(start,end){var self=this;
return self.for_doing_(""._comma(start)._comma(unescape("-"))._comma(end),(function(c){return c.between_and_(start,end);}));}
,"constants",unescape("between%3A%20start%20and%3A%20end%0D%0A%09%22%20Return%20the%20parser%20for%20input%20between%20start%20and%20end.%20%22%0D%0A%0D%0A%09%5Eself%09for%3A%20%27%27%2Cstart%2C%27-%27%2Cend%0D%0A%09%09doing%3A%20%5B%3Ac%7C%20c%20between%3A%20start%20and%3A%20end%20%5D"));
smalltalk.bind(smalltalk.PPCharacterParser.$klass,"for:doing:",0
,function(label,aBlock){var self=this;
return self.$$new().do_for_(aBlock,label);}
,"constants",unescape("for%3A%20label%20doing%3A%20aBlock%0D%0A%09%22%20Return%20a%20parser%20that%20evaluates%20aBlock%20to%20match%20label.%20%22%0D%0A%0D%0A%09%5Eself%20new%20do%3A%20aBlock%20for%3A%20label"));
smalltalk.bind(smalltalk.PPCharacterParser.$klass,"regExpCache",0
,function(){var self=this;
if ((nil.isNil_(smalltalk.PPCharacterParser.classVariableAt_("RegExpCache")))){smalltalk.PPCharacterParser.classVariableAt_put_("RegExpCache",smalltalk.PoolDictionary.$$new());};;
return smalltalk.PPCharacterParser.classVariableAt_("RegExpCache");}
,"constants",unescape("regExpCache%0D%0A%09%22%20Private%20-%20Return%20the%20regexp%20cache.%20%22%0D%0A%0D%0A%09RegExpCache%20isNil%20ifTrue%3A%20%5B%0D%0A%09%09RegExpCache%20%3A%3D%20PoolDictionary%20new%0D%0A%09%5D.%0D%0A%09%5ERegExpCache"));
smalltalk.bind(smalltalk.PPCharacterParser,"do:for:",0
,function(aBlock,aString){var self=this;
self["@theString"]=aString;
self["@regexp"]=aBlock;
return self;}
,"accessing",unescape("do%3A%20aBlock%20for%3A%20aString%0D%0A%09%22%20Set%20the%20receiver%20to%20match%20aString%20using%20aBlock.%20%22%0D%0A%0D%0A%09theString%20%3A%3D%20aString.%0D%0A%09regexp%20%3A%3D%20aBlock"));
smalltalk.bind(smalltalk.PPCharacterParser,"string:",0
,function(aString){var self=this;
self["@theString"]=aString;
self["@regexp"]=self.$klass.regExpCache().at_ifAbsentPut_(aString,(function(){return smalltalk.RegularExpression.fromString_(unescape("%5B")._comma(aString)._comma(unescape("%5D")));}));
return self;}
,"accessing",unescape("string%3A%20aString%0D%0A%09%22%20Set%20the%20string%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09theString%20%3A%3D%20aString.%0D%0A%09regexp%20%3A%3D%20self%20class%20regExpCache%20at%3A%20aString%20ifAbsentPut%3A%20%5B%0D%0A%09%09RegularExpression%20fromString%3A%20%27%5B%27%2C%20aString%2C%20%27%5D%27%0D%0A%09%5D"));
smalltalk.bind(smalltalk.PPCharacterParser,"match:",0
,function(aString){var self=this;
if ((nil.isNil_(aString))){return false;};;
if ((nil.isNil_(self["@regexp"]))){return aString.isSeparator();};;
if (self["@regexp"].isValuable()){return self["@regexp"].value_(aString);};;
if (self["@theString"]._eq(aString)){return true;};;
return aString.match_(self["@regexp"]);}
,"private",unescape("match%3A%20aString%0D%0A%09%22%20Private%20-%20Match%20the%20receiver%20to%20string.%20%22%0D%0A%0D%0A%09aString%20isNil%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09regexp%20isNil%20ifTrue%3A%20%5B%20%5EaString%20isSeparator%20%5D.%0D%0A%09regexp%20isValuable%20ifTrue%3A%20%5B%20%5Eregexp%20value%3A%20aString%20%5D.%0D%0A%09theString%20%3D%20aString%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09%5EaString%20match%3A%20regexp"));
smalltalk.bind(smalltalk.PPCharacterParser,"matchFailureReason",0
,function(){var self=this;
return "Could not match "._comma((((nil.isNil_(self["@theString"]))).mustBeBoolean()==true ? (function(){return "separator";})() : (function(){return self["@theString"].printString();})()));}
,"accessing",unescape("matchFailureReason%0D%0A%09%22%20Private%20-%20Return%20the%20reason%20for%20mismatch.%20%22%0D%0A%0D%0A%09%5E%27Could%20not%20match%20%27%20%2C%28theString%20isNil%0D%0A%09%09ifTrue%3A%20%5B%23separator%5D%0D%0A%09%09ifFalse%3A%20%5BtheString%20printString%5D%29"));
smalltalk.bind(smalltalk.PPCharacterParser,"basicParse:",0
,function(aStream){var self=this;
return ((self.match_(aStream.peek())).mustBeBoolean()==true ? (function(){return aStream.next();})() : (function(){return smalltalk.PPFailure.reason_at_(self.matchFailureReason(),aStream.position());})());}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%5E%28self%20match%3A%20aStream%20peek%29%0D%0A%09%20ifTrue%3A%20%20%5B%20aStream%20next%20%5D%0D%0A%09%20ifFalse%3A%20%5B%20PPFailure%0D%0A%09%09reason%3A%20self%20matchFailureReason%0D%0A%09%09at%3A%20aStream%20position%0D%0A%09%5D"));
smalltalk.bind(smalltalk.PPCharacterParser,"separator",0
,function(){var self=this;
self["@theString"]=self["@regexp"]=nil;
self._at_at("separator");
return self;}
,"separator",unescape("separator%0D%0A%09%22%20Set%20the%20receiver%20to%20parse%20separator.%20%22%0D%0A%0D%0A%09theString%20%3A%3D%20regexp%20%3A%3D%20nil.%0D%0A%09self%20@@%23separator"));
smalltalk.bind(smalltalk.PPCharacterParser,"printStringDetail",0
,function(){var self=this;
if (self["@theString"].isCharacter()){if (self["@theString"].between_and_("0","z")){return "$"._comma(self["@theString"]);};;
return unescape("%23")._comma(self["@theString"].asciiValue());};;
return self["@theString"].printString();}
,"printing",unescape("printStringDetail%0D%0A%09%22%20Private%20-%20Return%20a%20printable%20detail%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09theString%20isCharacter%20ifTrue%3A%20%5B%0D%0A%09%09%28theString%20between%3A%20%240%20and%3A%20%24z%29%20ifTrue%3A%20%5B%0D%0A%09%09%09%5E%24%24%20%2CtheString%0D%0A%09%09%5D.%0D%0A%09%09%5E%24%23%20%2CtheString%20asciiValue%0D%0A%09%5D.%0D%0A%09%5EtheString%20printString"));
smalltalk.bind(smalltalk.PPListParser.$klass,"withAll:",0
,function(aCollection){var self=this;
return (function($rec){$rec.parsers_(aCollection);return $rec.yourself();})(self.basicNew());}
,"instantiation",unescape("withAll%3A%20aCollection%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%20%20%20%20%5Eself%20basicNew%0D%0A%09%09parsers%3A%20aCollection%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.PPListParser.$klass,"with:with:",0
,function(aParser,anotherParser){var self=this;
return self.withAll_(smalltalk.Array.with_with_(aParser,anotherParser));}
,"instantiation",unescape("with%3A%20aParser%20with%3A%20anotherParser%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20withAll%3A%20%28Array%20with%3A%20aParser%20with%3A%20anotherParser%29"));
smalltalk.bind(smalltalk.PPListParser,"parsers",0
,function(){var self=this;
if ((nil.isNil_(self["@parsers"]))){return [];};;
return self["@parsers"];}
,"accessing",unescape("parsers%0D%0A%09%22%20Return%20the%20parsers%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09parsers%20isNil%20ifTrue%3A%20%5B%20%5E%23%28%29%20%5D.%0D%0A%09%5Eparsers"));
smalltalk.bind(smalltalk.PPListParser,"parsers:",0
,function(aCollection){var self=this;
self["@parsers"]=aCollection.collect_((function(each){return each.asParser();}));
return self;}
,"accessing",unescape("parsers%3A%20aCollection%0D%0A%09%22%20Set%20the%20parsers%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09parsers%20%3A%3D%20aCollection%20collect%3A%20%5B%3Aeach%7C%20each%20asParser%20%5D"));
smalltalk.bind(smalltalk.PPListParser,"copyWith:",0
,function(aParser){var self=this;
return self.$klass.withAll_(self.parsers().copyWith_(aParser));}
,"copying",unescape("copyWith%3A%20aParser%0D%0A%09%22%20Return%20a%20copy%20of%20the%20receiver%20with%20aParser.%20%22%0D%0A%0D%0A%09%5Eself%20class%20withAll%3A%20%28self%20parsers%20copyWith%3A%20aParser%29"));
smalltalk.bind(smalltalk.PPListParser,"op",0
,function(){var self=this;
return unescape("+");}
,"printing",unescape("op%0D%0A%09%22%20Private%20-%20Return%20the%20operator%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%24+"));
smalltalk.bind(smalltalk.PPListParser,"printString",0
,function(){var self=this;
var stream=nil;
if ((nil.isNil_(((function(){var $1$=(self["debugTag"]);if(nil.isNil_($1$))return nil;return $1$;})()))===false)){return smalltalk.superImplementor(smalltalk.PPListParser,"printString").apply(self, []);};;
stream=unescape("%7B").writeStream();
self.parsers().do_separatedBy_((function(each){return stream.nextPutAll_(each.printString());}),(function(){return stream.nextPutAll_(self.op());}));
return stream.contents();}
,"printing",unescape("printString%0D%0A%09%22%20Return%20a%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09%28self%20basicAt%3A%20%23debugTag%29%20notNil%20ifTrue%3A%20%5B%20%5Esuper%20printString%20%5D.%0D%0A%09stream%20%3A%3D%20%24%7B%20writeStream.%0D%0A%09self%20parsers%0D%0A%09%09do%3A%20%5B%3Aeach%7C%20stream%20nextPutAll%3A%20each%20printString%20%5D%0D%0A%09%09separatedBy%3A%20%5Bstream%20nextPutAll%3A%20self%20op%5D.%0D%0A%09%5Estream%20contents%20"));
smalltalk.bind(smalltalk.PPSequenceParser,",","_comma"
,function(aRule){var self=this;
return self.copyWith_(aRule);}
,"copying",unescape("%2C%20aRule%0D%0A%09%22%20Return%20the%20receiver%20composed%20with%20the%20argument.%20%22%0D%0A%0D%0A%09%5Eself%20copyWith%3A%20aRule"));
smalltalk.bind(smalltalk.PPSequenceParser,"basicParse:",0
,function(aStream){var $s8Ret$={name:"stReturn"};try{var self=this;
var start=nil;
var result=nil;
var element=nil;
result=[];
start=aStream.position();
self.parsers().do_((function(each){element=each.memoizedParse_(aStream);result.add_(element);return ((self.isParseFailure_(element)).mustBeBoolean()==true ? (function(){aStream.position_(start);return (function(){$s8Ret$.result=(function(){return element})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return result})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20start%20result%20element%20%7C%0D%0A%09result%20%3A%3D%20%23%28%29.%0D%0A%09start%20%3A%3D%20aStream%20position.%0D%0A%09self%20parsers%20do%3A%20%5B%3Aeach%20%7C%0D%0A%09%09element%20%3A%3D%20each%20memoizedParse%3A%20aStream.%0D%0A%09%09result%20add%3A%20element.%0D%0A%09%09%28self%20isParseFailure%3A%20element%29%20ifTrue%3A%20%5B%0D%0A%09%09%09aStream%20position%3A%20start.%0D%0A%09%09%09%5Eelement%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.PPSequenceParser,"op",0
,function(){var self=this;
return unescape("%2C");}
,"printing",unescape("op%0D%0A%09%22%20Private%20-%20Return%20the%20operator%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%24%2C"));
smalltalk.bind(smalltalk.PPChoiceParser,"/","_slash"
,function(aRule){var self=this;
return self.copyWith_(aRule);}
,"copying",unescape("/%20aRule%0D%0A%09%22%20Return%20the%20receiver%20composed%20with%20aRule.%20%22%0D%0A%0D%0A%09%5Eself%20copyWith%3A%20aRule"));
smalltalk.bind(smalltalk.PPChoiceParser,"basicParse:",0
,function(aStream){var self=this;
var result=nil;
self.parsers().detect_ifNone_((function(each){result=each.memoizedParse_(aStream);return self.isParseFailure_(result).not();}),(function(){return nil;}));
return result;}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09self%20parsers%20detect%3A%20%5B%3Aeach%20%7C%0D%0A%09%09result%20%3A%3D%20each%20memoizedParse%3A%20aStream.%0D%0A%09%09%28self%20isParseFailure%3A%20result%29%20not%0D%0A%09%5D%20ifNone%3A%20%5B%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.PPChoiceParser,"op",0
,function(){var self=this;
return unescape("/");}
,"printing",unescape("op%0D%0A%09%22%20Private%20-%20Return%20the%20operator%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%24/"));
smalltalk.bind(smalltalk.PPDelegateParser.$klass,"on:",0
,function(aParser){var self=this;
return (function($rec){$rec.parser_(aParser);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("on%3A%20aParser%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%20%20%20%20%5Eself%20new%0D%0A%09%09parser%3A%20aParser%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.PPDelegateParser,"parser",0
,function(){var self=this;
return self["@parser"];}
,"accessing",unescape("parser%0D%0A%09%22%20Return%20the%20parser%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eparser"));
smalltalk.bind(smalltalk.PPDelegateParser,"parser:",0
,function(aParser){var self=this;
self["@parser"]=aParser;
return self;}
,"accessing",unescape("parser%3A%20aParser%0D%0A%09%22%20Set%20the%20parser%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09parser%20%3A%3D%20aParser"));
smalltalk.bind(smalltalk.PPDelegateParser,"basicParse:",0
,function(aStream){var self=this;
return self.parser().memoizedParse_(aStream);}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%5Eself%20parser%20memoizedParse%3A%20aStream"));
smalltalk.bind(smalltalk.PPDelegateParser,"printStringDetail",0
,function(){var self=this;
return unescape("%5B")._comma(self["@parser"].printString())._comma(unescape("%5D"));}
,"printing",unescape("printStringDetail%0D%0A%09%22%20Private%20-%20Return%20a%20printable%20detail%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%27%5B%27%20%2Cparser%20printString%20%2C%27%5D%27"));
smalltalk.bind(smalltalk.PPAndParser,"opParse:",0
,function(aStream){var self=this;
var element=nil;
var position=nil;
position=aStream.position();
element=self.parser().memoizedParse_(aStream);
aStream.position_(position);
return element;}
,"parsing",unescape("opParse%3A%20aStream%0D%0A%09%22%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20element%20position%20%7C%0D%0A%09position%20%3A%3D%20aStream%20position.%0D%0A%09element%20%3A%3D%20self%20parser%20memoizedParse%3A%20aStream.%0D%0A%09aStream%20position%3A%20position.%0D%0A%09%5Eelement"));
smalltalk.bind(smalltalk.PPAndParser,"basicParse:",0
,function(aStream){var self=this;
return self.opParse_(aStream);}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%5Eself%20opParse%3A%20aStream"));
smalltalk.bind(smalltalk.PPNotParser,"basicParse:",0
,function(aStream){var self=this;
var element=nil;
element=self.opParse_(aStream);
if (self.isParseFailure_(element)){return nil;};;
return smalltalk.PPFailure.reason_at_(element,aStream.position());}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20element%20%7C%0D%0A%09element%20%3A%3D%20self%20opParse%3A%20aStream.%0D%0A%09%28self%20isParseFailure%3A%20element%29%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%5EPPFailure%20reason%3A%20element%20at%3A%20aStream%20position"));
smalltalk.bind(smalltalk.PPActionParser.$klass,"on:block:",0
,function(aParser,aBlock){var self=this;
return (function($rec){$rec.parser_(aParser);$rec.block_(aBlock);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("on%3A%20aParser%20block%3A%20aBlock%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09parser%3A%20aParser%3B%0D%0A%09%09block%3A%20aBlock%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.PPActionParser,"block",0
,function(){var self=this;
return self["@block"];}
,"accessing",unescape("block%0D%0A%09%22%20Return%20the%20block%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eblock"));
smalltalk.bind(smalltalk.PPActionParser,"block:",0
,function(aBlock){var self=this;
self["@block"]=aBlock;
return self;}
,"accessing",unescape("block%3A%20aBlock%0D%0A%09%22%20Set%20the%20block%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09block%20%3A%3D%20aBlock"));
smalltalk.bind(smalltalk.PPActionParser,"basicParse:",0
,function(aStream){var self=this;
var element=nil;
element=self.parser().memoizedParse_(aStream);
if (self.isParseFailure_(element)){return element;};;
return self.block().value_(element);}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20element%20%7C%0D%0A%09element%20%3A%3D%20self%20parser%20memoizedParse%3A%20aStream.%0D%0A%09%28self%20isParseFailure%3A%20element%29%20ifTrue%3A%20%5B%20%5Eelement%20%5D.%0D%0A%09%5Eself%20block%20value%3A%20element"));
smalltalk.bind(smalltalk.PPActionParser,"printString",0
,function(){var self=this;
return unescape("%7B")._comma(self["@parser"].printString())._comma(unescape("%7D"));}
,"printing",unescape("printString%0D%0A%09%22%20Return%20a%20printable%20representation%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%24%7B%20%2Cparser%20printString%20%2C%24%7D%20"));
smalltalk.bind(smalltalk.PPFlattenParser,"basicParse:",0
,function(aStream){var self=this;
var start=nil;
var element=nil;
start=aStream.position();
element=self.parser().memoizedParse_(aStream);
if (self.isParseFailure_(element)){return element;};;
return aStream.collection().copyFrom_to_(start._plus((1)),aStream.position());}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20start%20element%20%7C%0D%0A%09start%20%3A%3D%20aStream%20position.%0D%0A%09element%20%3A%3D%20self%20parser%20memoizedParse%3A%20aStream.%0D%0A%09%28self%20isParseFailure%3A%20element%29%20ifTrue%3A%20%5B%20%5Eelement%20%5D.%0D%0A%09%5EaStream%20collection%20%0D%0A%09%09copyFrom%3A%20start%20+%201%20%0D%0A%09%09to%3A%20aStream%20position"));
smalltalk.bind(smalltalk.PPSourceParser,"basicParse:",0
,function(aStream){var self=this;
var start=nil;
var element=nil;
var result=nil;
start=aStream.position();
element=self.parser().memoizedParse_(aStream);
if (self.isParseFailure_(element)){return element;};;
result=aStream.collection().copyFrom_to_(start._plus((1)),aStream.position());
return smalltalk.Array.with_with_(element,result);}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20start%20element%20result%20%7C%0D%0A%09start%20%3A%3D%20aStream%20position.%0D%0A%09element%20%3A%3D%20self%20parser%20memoizedParse%3A%20aStream.%0D%0A%09%28self%20isParseFailure%3A%20element%29%20ifTrue%3A%20%5B%20%5Eelement%20%5D.%0D%0A%09result%20%3A%3D%20aStream%20collection%0D%0A%09%09copyFrom%3A%20start%20+%201%0D%0A%09%09to%3A%20aStream%20position.%0D%0A%09%5EArray%20with%3A%20element%20with%3A%20result"));
smalltalk.bind(smalltalk.PPRepeatingParser.$klass,"on:min:",0
,function(aParser,aNumber){var self=this;
return (function($rec){$rec.parser_(aParser);$rec.min_(aNumber);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("on%3A%20aParser%20min%3A%20aNumber%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09parser%3A%20aParser%3B%0D%0A%09%09min%3A%20aNumber%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.PPRepeatingParser,"min",0
,function(){var self=this;
return self["@min"];}
,"accessing",unescape("min%0D%0A%09%22%20Return%20the%20min%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Emin"));
smalltalk.bind(smalltalk.PPRepeatingParser,"min:",0
,function(aNumber){var self=this;
self["@min"]=aNumber;
return self;}
,"accessing",unescape("min%3A%20aNumber%0D%0A%09%22%20Set%20the%20min%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09min%20%3A%3D%20aNumber"));
smalltalk.bind(smalltalk.PPRepeatingParser,"basicParse:",0
,function(aStream){var self=this;
var start=nil;
var element=nil;
var result=nil;
var failure=nil;
start=aStream.position();
result=smalltalk.Array.$$new();
while((function(){return result.size()._lt(self.min()).and_((function(){return (nil.isNil_(failure));}));})()){element=self.parser().memoizedParse_(aStream);
((self.isParseFailure_(element)).mustBeBoolean()==true ? (function(){aStream.position_(start);return failure=element;})() : (function(){return result.addLast_(element);})());};;
if ((nil.isNil_(failure)===false)){return failure;};;
while((function(){return (nil.isNil_(failure));})()){element=self.parser().memoizedParse_(aStream);
((self.isParseFailure_(element)).mustBeBoolean()==true ? (function(){return failure=element;})() : (function(){return result.addLast_(element);})());};;
return result;}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20start%20element%20result%20failure%20%7C%0D%0A%09start%20%3A%3D%20aStream%20position.%0D%0A%09result%20%3A%3D%20Array%20new.%0D%0A%09%5B%20%28result%20size%20%3C%20self%20min%29%20and%3A%20%5B%20failure%20isNil%20%5D%0D%0A%09%5D%20whileTrue%3A%20%5B%0D%0A%09%20%20%20%20element%20%3A%3D%20self%20parser%20memoizedParse%3A%20aStream.%0D%0A%09%20%20%20%20%28self%20isParseFailure%3A%20element%29%0D%0A%09%09ifFalse%3A%20%5B%20result%20addLast%3A%20element%20%5D%0D%0A%09%09ifTrue%3A%20%5B%20aStream%20position%3A%20start.%0D%0A%09%09%09%20%20failure%20%3A%3D%20element%20%5D%0D%0A%09%5D.%0D%0A%09failure%20notNil%20ifTrue%3A%20%5B%20%5Efailure%20%5D.%0D%0A%09%5Bfailure%20isNil%5D%20whileTrue%3A%20%5B%0D%0A%09%09element%20%3A%3D%20self%20parser%20memoizedParse%3A%20aStream.%0D%0A%09%20%09%28self%20isParseFailure%3A%20element%29%0D%0A%09%09%09ifTrue%3A%20%5B%20failure%20%3A%3D%20element%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20result%20addLast%3A%20element%20%5D%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.PPRepeatingParser,"printStringDetail",0
,function(){var self=this;
return unescape("%5Bmin%3D")._comma(self["@min"])._comma(unescape("%5D"));}
,"printing",unescape("printStringDetail%0D%0A%09%22%20Private%20-%20Return%20a%20printable%20detail%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%27%5Bmin%3D%27%20%2Cmin%20%2C%27%5D%27"));
smalltalk.bind(smalltalk.PPInlineParser.$klass,"upTo:",0
,function(aString){var self=this;
return (function($rec){$rec.terminator_(aString);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("upTo%3A%20aString%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%20terminator%3A%20aString%3B%20yourself"));
smalltalk.bind(smalltalk.PPInlineParser,"terminator:",0
,function(aString){var self=this;
self["@terminator"]=aString;
return self;}
,"accessing",unescape("terminator%3A%20aString%0D%0A%09%22%20Set%20terminator%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09terminator%20%3A%3D%20aString"));
smalltalk.bind(smalltalk.PPInlineParser,"basicParse:",0
,function(aStream){var self=this;
var result=nil;
result=aStream.upTo_(self["@terminator"]);
if (!(aStream.atEnd())){return result;};;
return smalltalk.PPFailure.reason_at_(unescape("Missing%20inline%20terminator%20%28")._comma(self["@terminator"])._comma(unescape("%29")),aStream.position());}
,"parsing",unescape("basicParse%3A%20aStream%0D%0A%09%22%20Private%20-%20Parse%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20aStream%20upTo%3A%20terminator.%0D%0A%09aStream%20atEnd%20ifFalse%3A%20%5B%20%5Eresult%20%5D.%0D%0A%09%5EPPFailure%0D%0A%09%09reason%3A%20%27Missing%20inline%20terminator%20%28%27%2Cterminator%2C%27%29%27%0D%0A%09%09at%3A%20aStream%20position"));
smalltalk.bind(smalltalk.FileInReader,"scanFrom:",0
,function(aStream){var self=this;
return self.subclassResponsibility_("scanFrom:");}
,"fileIn",unescape("scanFrom%3A%20aStream%0D%0A%09%22%20FileIn%20-%20Return%20the%20result%20of%20evaluating%20the%20next%20chunk%20of%20data%20from%20aStream.%0D%0A%09The%20method%20must%20be%20implemented%20by%20chunk%20readers%20to%20support%20custom%20fileIn%20features.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23scanFrom%3A"));
smalltalk.bind(smalltalk.ChunkReader.$klass,"doing:until:",0
,function(aBlock,terminationBlock){var self=this;
return (function($rec){$rec.actionBlock_(aBlock);$rec.terminationBlock_(terminationBlock);return $rec.yourself();})(self.$$new());}
,"instantiation",unescape("doing%3A%20aBlock%20until%3A%20terminationBlock%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%0D%0A%09%09actionBlock%3A%20aBlock%3B%0D%0A%09%09terminationBlock%3A%20terminationBlock%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.ChunkReader.$klass,"doing:",0
,function(aBlock){var self=this;
return self.doing_until_(aBlock,(function(){return true;}));}
,"instantiation",unescape("doing%3A%20aBlock%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20doing%3A%20aBlock%20until%3A%20%5B%20true%20%5D"));
smalltalk.bind(smalltalk.ChunkReader.$klass,"until:",0
,function(terminationBlock){var self=this;
return self.doing_until_(nil,terminationBlock);}
,"instantiation",unescape("until%3A%20terminationBlock%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20doing%3A%20nil%20until%3A%20terminationBlock"));
smalltalk.bind(smalltalk.ChunkReader.$klass,"ignore",0
,function(){var self=this;
return self.$$new();}
,"instantiation",unescape("ignore%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%0D%0A%09The%20returned%20reader%20will%20ignore/absorb%20next%20chunk.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20new"));
smalltalk.bind(smalltalk.ChunkReader.$klass,"javascriptChunk",0
,function(){var self=this;
return self.nativeChunk();}
,"instantiation",unescape("javascriptChunk%0D%0A%09%22%20Obsolete%20-%20use%20%23nativeChunk%20%22%0D%0A%0D%0A%09%5Eself%20nativeChunk%20"));
smalltalk.bind(smalltalk.ChunkReader.$klass,"javascript",0
,function(){var self=this;
return self.$$native();}
,"instantiation",unescape("javascript%0D%0A%09%22%20Obsolete%20-%20use%20%23native%20%22%0D%0A%0D%0A%09%5Eself%20native%20"));
smalltalk.bind(smalltalk.ChunkReader.$klass,"nativeChunk",0
,function(){var self=this;
return self.doing_((function(aStream){return smalltalk.Smalltalk.current().eval_(aStream.nextChunk());}));}
,"instantiation",unescape("nativeChunk%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%0D%0A%09The%20returned%20reader%20will%20evaluate%20next%20chunk%20as%20native%20code%20%28in%20global%20context%29.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20doing%3A%20%5B%3AaStream%7C%20Smalltalk%20current%20eval%3A%20aStream%20nextChunk%20%5D"));
smalltalk.bind(smalltalk.ChunkReader.$klass,"native","$$native"
,function(){var self=this;
return self.doing_((function(aStream){return smalltalk.Smalltalk.current().eval_(aStream.upToEnd());}));}
,"instantiation",unescape("native%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%0D%0A%09The%20returned%20reader%20will%20evaluate%20following%20contents%20as%20native%20code%20%28in%20global%20context%29.%0D%0A%09The%20stream%20will%20be%20consumed%20up%20to%20end.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20doing%3A%20%5B%3AaStream%7C%20Smalltalk%20current%20eval%3A%20aStream%20upToEnd%20%5D"));
smalltalk.bind(smalltalk.ChunkReader,"actionBlock",0
,function(){var self=this;
return self["@actionBlock"];}
,"accessing",unescape("actionBlock%0D%0A%09%22%20Return%20the%20actionBlock%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EactionBlock"));
smalltalk.bind(smalltalk.ChunkReader,"actionBlock:",0
,function(aBlock){var self=this;
self["@actionBlock"]=aBlock;
return self;}
,"accessing",unescape("actionBlock%3A%20aBlock%0D%0A%09%22%20Set%20the%20actionBlock%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09actionBlock%20%3A%3D%20aBlock"));
smalltalk.bind(smalltalk.ChunkReader,"terminationBlock",0
,function(){var self=this;
return self["@terminationBlock"];}
,"accessing",unescape("terminationBlock%0D%0A%09%22%20Return%20the%20terminationBlock%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5EterminationBlock"));
smalltalk.bind(smalltalk.ChunkReader,"terminationBlock:",0
,function(aBlock){var self=this;
self["@terminationBlock"]=aBlock;
return self;}
,"accessing",unescape("terminationBlock%3A%20aBlock%0D%0A%09%22%20Set%20the%20terminationBlock%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09terminationBlock%20%3A%3D%20aBlock"));
smalltalk.bind(smalltalk.ChunkReader,"scanFrom:",0
,function(aStream){var $s8Ret$={name:"stReturn"};try{var self=this;
var chunk=nil;
if ((nil.isNil_(self.actionBlock()))){chunk=aStream.nextChunk();
((chunk.isEmpty()).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return nil})();throw($s8Ret$)})();})() : nil);} else{self.actionBlock().value_(aStream);};;
if ((nil.isNil_(self.terminationBlock())).or_((function(){return (nil.is_eqeq_(true,self.terminationBlock().value()));}))){(function(){$s8Ret$.result=(function(){return nil})();throw($s8Ret$)})();};;
(function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"fileIn",unescape("scanFrom%3A%20aStream%0D%0A%09%22%20Scan%20the%20next%20chunk%20of%20data%20from%20aStream.%20%22%0D%0A%0D%0A%09%7C%20chunk%20%7C%0D%0A%09self%20actionBlock%20isNil%20ifTrue%3A%20%5B%0D%0A%09%09chunk%20%3A%3D%20aStream%20nextChunk.%0D%0A%09%09chunk%20isEmpty%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%5D%20ifFalse%3A%20%5B%20self%20actionBlock%20value%3A%20aStream%20%5D.%0D%0A%09%28self%20terminationBlock%20isNil%0D%0A%09%09or%3A%20%5B%20true%20%3D%3D%20self%20terminationBlock%20value%20%5D%29%0D%0A%09%09ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.ClassCategoryReader,"class:category:",0
,function(aClass,aString){var self=this;
self["@class"]=aClass;
self["@category"]=aString;
return self;}
,"accessing",unescape("class%3A%20aClass%20category%3A%20aString%0D%0A%09%22%20Private%20-%20Set%20the%20class%20and%20category%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09class%20%3A%3D%20aClass.%0D%0A%09category%20%3A%3D%20aString"));
smalltalk.bind(smalltalk.ClassCategoryReader,"class:category:stamp:",0
,function(aClass,aString,aStamp){var self=this;
self.class_category_(aClass,aString);
self["@stamp"]=aStamp;
return self;}
,"accessing",unescape("class%3A%20aClass%20category%3A%20aString%20stamp%3A%20aStamp%0D%0A%09%22%20Private%20-%20Set%20the%20class%20and%20category%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20class%3A%20aClass%20category%3A%20aString.%0D%0A%09stamp%20%3A%3D%20aStamp"));
smalltalk.bind(smalltalk.ClassCategoryReader,"scanFrom:",0
,function(aStream){var self=this;
var chunk=nil;
chunk=aStream.nextChunk();
return (function(){return ((chunk.isEmpty()).mustBeBoolean()==true ? nil : (function(){return self.compileMethod_(chunk);})());}).logEvaluate_(chunk);}
,"fileIn",unescape("scanFrom%3A%20aStream%0D%0A%09%22%20Scan%20the%20next%20chunk%20of%20data%20from%20aStream.%20%22%0D%0A%0D%0A%09%7C%20chunk%20%7C%0D%0A%09chunk%20%3A%3D%20aStream%20nextChunk.%0D%0A%09%5E%5B%20chunk%20isEmpty%20ifFalse%3A%20%5B%20self%20compileMethod%3A%20chunk%20%5D%0D%0A%09%5D%20logEvaluate%3A%20chunk"));
smalltalk.bind(smalltalk.ClassCategoryReader,"compileMethod:",0
,function(aString){var self=this;
var aMethod=nil;
aMethod=self["@class"].compiler().compile_forClass_category_(aString,self["@class"],self["@category"]);
if (self["@stamp"].size()._gt((0))){aMethod.stamp_(self["@stamp"]);};;
return self;}
,"private",unescape("compileMethod%3A%20aString%0D%0A%09%22%20Compile%20aString%20as%20a%20method%20source.%20%22%0D%0A%0D%0A%09%7C%20aMethod%20%7C%0D%0A%09aMethod%20%3A%3D%20class%20compiler%0D%0A%09%09compile%3A%20aString%0D%0A%09%09forClass%3A%20class%0D%0A%09%09category%3A%20category.%0D%0A%09stamp%20size%20%3E%200%20ifTrue%3A%20%5B%20aMethod%20stamp%3A%20stamp%20%5D%20"));
smalltalk.bind(smalltalk.ClassCommentReader,"class:",0
,function(aClass){var self=this;
self["@class"]=aClass;
return self;}
,"accessing",unescape("class%3A%20aClass%0D%0A%09%22%20Set%20the%20destination%20class%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09class%20%3A%3D%20aClass"));
smalltalk.bind(smalltalk.ClassCommentReader,"scanFrom:",0
,function(aStream){var self=this;
self.comment_(aStream.nextChunk());
return nil;}
,"fileIn",unescape("scanFrom%3A%20aStream%0D%0A%09%22%20FileIn%20-%20Parse%20next%20chunk%20in%20aStream%20as%20a%20comment%20for%20target%20class.%20%22%0D%0A%0D%0A%09self%20comment%3A%20aStream%20nextChunk.%0D%0A%09%5Enil"));
smalltalk.bind(smalltalk.ClassCommentReader,"comment:",0
,function(aString){var self=this;
self["@class"].comment_(((aString.isEmpty()).mustBeBoolean()==true ? nil : (function(){return aString;})()));
return self;}
,"private",unescape("comment%3A%20aString%0D%0A%09%22%20Set%20the%20comment%20of%20target%20class.%20%22%0D%0A%0D%0A%09class%20comment%3A%20%28aString%20isEmpty%20ifFalse%3A%20%5B%20aString%20%5D%29"));
smalltalk.bind(smalltalk.SkipFileInReader.$klass,"upTo:",0
,function(aToken){var self=this;
return self.$$new().limit_(aToken);}
,"instantiation",unescape("upTo%3A%20aToken%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20new%20limit%3A%20aToken"));
smalltalk.bind(smalltalk.SkipFileInReader,"scanFrom:",0
,function(aStream){var self=this;
if ((nil.isNil_(self["@limit"]))){aStream.setToEnd();} else{aStream.skipTo_(self["@limit"]);};;
return nil;}
,"fileIn",unescape("scanFrom%3A%20aStream%0D%0A%09%22%20Consume/Ignore%20contents%20in%20aStream.%20%22%0D%0A%0D%0A%09limit%20isNil%0D%0A%09%09ifTrue%3A%20%5B%20aStream%20setToEnd%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20aStream%20skipTo%3A%20limit%20%5D.%0D%0A%09%5Enil"));
smalltalk.bind(smalltalk.SkipFileInReader,"limit:",0
,function(aToken){var self=this;
self["@limit"]=aToken;
return self;}
,"accessing",unescape("limit%3A%20aToken%0D%0A%09%22%20Set%20the%20token%20limit.%20%22%0D%0A%0D%0A%09limit%20%3A%3D%20aToken"));
smalltalk.bind(smalltalk.AssociationsReader,"keyFrom:",0
,function(aStream){var self=this;
if (aStream.peek()._eq(unescape("%23"))){return (function($rec){$rec.next();return $rec.upTo_(" ");})(aStream).asSymbol();};;
if (aStream.peek()._eq(unescape("%27"))){return (function($rec){$rec.next();return $rec.upTo_(unescape("%27"));})(aStream);};;
return aStream.upTo_(" ");}
,"reading",unescape("keyFrom%3A%20aStream%0D%0A%09%22%20Private%20-%20Return%20the%20name%20of%20the%20current%20binding%20in%20aStream.%20%22%0D%0A%0D%0A%09aStream%20peek%20%3D%20%24%23%20ifTrue%3A%20%5B%20%5E%28aStream%20next%3B%20upTo%3A%20%24%20%29%20asSymbol%20%5D.%0D%0A%09aStream%20peek%20%3D%20%24%27%20ifTrue%3A%20%5B%20%5EaStream%20next%3B%20upTo%3A%20%24%27%20%5D.%0D%0A%09%5EaStream%20upTo%3A%20%24%20"));
smalltalk.bind(smalltalk.AssociationsReader,"valueFrom:",0
,function(aStream){var self=this;
return self.compilerSupport().evaluate_(aStream.upToEnd());}
,"reading",unescape("valueFrom%3A%20aStream%0D%0A%09%22%20Private%20-%20Return%20the%20value%20of%20the%20current%20binding%20in%20aStream.%20%22%0D%0A%0D%0A%09%5Eself%20compilerSupport%20evaluate%3A%20aStream%20upToEnd%20"));
smalltalk.bind(smalltalk.AssociationsReader,"compilerSupport",0
,function(){var self=this;
return self.smalltalk().compilerSupport();}
,"evaluating",unescape("compilerSupport%0D%0A%09%22%20Private%20-%20Return%20compiler%20in%20use%20to%20get%20values%20from%20stream.%20%22%0D%0A%0D%0A%09%5Eself%20smalltalk%20compilerSupport"));
smalltalk.bind(smalltalk.AssociationsReader,"bind:",0
,function(tuples){var self=this;
return self.subclassResponsibility_("bind:");}
,"binding",unescape("bind%3A%20tuples%0D%0A%09%22%20Private%20-%20Bind%20the%20filedIn%20tuples%20to%20target.%0D%0A%09Must%20return%20the%20result%20of%20the%20fileIn/binding%20process.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23bind%3A%20"));
smalltalk.bind(smalltalk.AssociationsReader,"scanFrom:",0
,function(aStream){var self=this;
var chunk=nil;
var tuples=nil;
tuples=smalltalk.Array.$$new();
while((function(){return aStream.atEnd().not().and_((function(){chunk=aStream.nextChunk().trimBlanks();return chunk.notEmpty();}));})()){chunk=chunk.stream();
tuples.add_(smalltalk.Array.with_with_(self.keyFrom_(chunk),self.valueFrom_(chunk)));};;
return self.bind_(tuples);}
,"fileIn",unescape("scanFrom%3A%20aStream%0D%0A%09%22%20Consume%20chunks%20in%20aStream%20until%20empty%20chunk.%0D%0A%09The%20chunks%20are%20considered%20associative%20bindings%20with%20the%20syntax%3A%0D%0A%09%09%3Ckey%3E%20%3Cexpression%3E%0D%0A%09Where%0D%0A%09%09key%20is%20the%20keyword%20of%20the%20binding%20%28name%2C%20string%20or%20symbol%29.%0D%0A%09%09expression%20is%20a%20valid%20smalltalk%20expression%20that%20will%20be%20evaluated%20to%20generate%20the%20value%20of%20the%20binding.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20chunk%20tuples%20%7C%0D%0A%09tuples%20%3A%3D%20Array%20new.%0D%0A%09%5B%20aStream%20atEnd%20not%20and%3A%20%5B%0D%0A%09%20%20chunk%20%3A%3D%20aStream%20nextChunk%20trimBlanks.%0D%0A%09%20%20chunk%20notEmpty%20%5D%5D%20whileTrue%3A%20%5B%0D%0A%09%20%20chunk%20%3A%3D%20chunk%20stream.%0D%0A%09%20%20tuples%20add%3A%20%28Array%0D%0A%09%09with%3A%20%28self%20keyFrom%3A%20chunk%29%0D%0A%09%09with%3A%20%28self%20valueFrom%3A%20chunk%29%29.%0D%0A%09%5D.%0D%0A%09%5Eself%20bind%3A%20tuples"));
smalltalk.bind(smalltalk.PoolDictionaryReader,"scope:",0
,function(aScope){var self=this;
self["@scope"]=aScope;
return self;}
,"accessing",unescape("scope%3A%20aScope%0D%0A%09%22%20Set%20the%20scope%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09scope%20%3A%3D%20aScope"));
smalltalk.bind(smalltalk.PoolDictionaryReader,"bind:",0
,function(tuples){var self=this;
if (self["@scope"].isString()){return smalltalk.Smalltalk.definePool_with_(self["@scope"],tuples);};;
if ((nil.isNil_(self["@scope"]))){self["@scope"]=smalltalk.PoolDictionary.$$new();};;
tuples.do_((function(each){return self["@scope"].at_put_(each.first(),each.last());}));
return self["@scope"];}
,"binding",unescape("bind%3A%20tuples%0D%0A%09%22%20Private%20-%20Bind%20the%20filedIn%20tuples%20to%20target.%0D%0A%09Must%20return%20the%20result%20of%20the%20fileIn/binding%20process.%0D%0A%09%22%0D%0A%0D%0A%09scope%20isString%20ifTrue%3A%20%5B%0D%0A%09%09%5ESmalltalk%20definePool%3A%20scope%20with%3A%20tuples%0D%0A%09%5D.%0D%0A%09scope%20isNil%20ifTrue%3A%20%5B%20scope%20%3A%3D%20PoolDictionary%20new%20%5D.%0D%0A%09tuples%20do%3A%20%5B%3Aeach%7C%20scope%20at%3A%20each%20first%20put%3A%20each%20last%20%5D.%0D%0A%09%5Escope"));
smalltalk.bind(smalltalk.PoolDictionaryReader,"valueFrom:",0
,function(aStream){var self=this;
var code=nil;
var result=nil;
var word=nil;
code=aStream.upToEnd();
word=code.trimBlanks();
if (word._eq("nil").or_((function(){return word.isEmpty();}))){return nil;};;
result=word.asNumber();
if (result.isNumber()){return result;};;
return smalltalk.superImplementor(smalltalk.PoolDictionaryReader,"valueFrom_").apply(self, [code.stream()]);}
,"reading",unescape("valueFrom%3A%20aStream%0D%0A%09%22%20Private%20-%20Return%20the%20value%20of%20the%20current%20binding%20in%20aStream.%20%22%0D%0A%0D%0A%09%7C%20code%20result%20word%20%7C%0D%0A%09code%20%3A%3D%20aStream%20upToEnd.%0D%0A%09word%20%3A%3D%20code%20trimBlanks.%0D%0A%09%28word%20%3D%20%23nil%20or%3A%20%5B%20word%20isEmpty%20%5D%29%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09result%20%3A%3D%20word%20asNumber.%0D%0A%09result%20isNumber%20ifTrue%3A%20%5B%20%5Eresult%20%5D.%0D%0A%09%5Esuper%20valueFrom%3A%20code%20stream%20"));
smalltalk.bind(smalltalk.NativeReader,"bind:",0
,function(tuples){var self=this;
if ((nil.isNil_(self["@object"]))){self.initializeObject();};;
tuples.do_((function(each){return (self["@object"][each.first()]=(each.last()));}));
return self["@object"];}
,"binding",unescape("bind%3A%20tuples%0D%0A%09%22%20Private%20-%20Bind%20the%20filedIn%20tuples%20to%20target.%0D%0A%09Must%20return%20the%20result%20of%20the%20fileIn/binding%20process.%0D%0A%09%22%0D%0A%0D%0A%09object%20isNil%20ifTrue%3A%20%5B%20self%20initializeObject%20%5D.%0D%0A%09tuples%20do%3A%20%5B%3Aeach%7C%20object%20basicAt%3A%20each%20first%20put%3A%20each%20last%20%5D.%0D%0A%09%5Eobject"));
smalltalk.bind(smalltalk.NativeReader,"object:",0
,function(anObject){var self=this;
self["@object"]=anObject;
return self;}
,"accessing",unescape("object%3A%20anObject%0D%0A%09%22%20Set%20the%20object%20where%20the%20bindings%20will%20be%20stored.%20%22%0D%0A%0D%0A%09object%20%3A%3D%20anObject"));
smalltalk.bind(smalltalk.NativeReader,"initializeObject",0
,function(){var self=this;
self["@object"]={};
return self;}
,"initialize",unescape("initializeObject%0D%0A%09%22%20Private%20-%20Initialize%20the%20object%20where%20the%20bindings%20will%20be%20stored.%20%22%0D%0A%0D%0A%09object%20%3A%3D%20%7B%27%27%7D"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"default","$$default"
,function(){var self=this;
if ((nil.isNil_(self["@default"]))){self["@default"]=self.$$new();};;
return self["@default"];}
,"accessing",unescape("default%0D%0A%09%22%20Return%20the%20default%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09default%20isNil%20ifTrue%3A%20%5B%20default%20%3A%3D%20self%20new%20%5D.%0D%0A%09%5Edefault"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"saveBuilders",0
,function(){var self=this;
return (nil.is_eqeq_(self["@saveBuilders"],true));}
,"accessing",unescape("saveBuilders%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20set%20to%20save%20builders%20in%20snapshot.%20%22%0D%0A%0D%0A%09%5EsaveBuilders%20%3D%3D%20true"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"saveBuilders:",0
,function(aBoolean){var self=this;
self["@saveBuilders"]=aBoolean;
return self;}
,"accessing",unescape("saveBuilders%3A%20aBoolean%0D%0A%09%22%20Set%20to%20true%20if%20the%20receiver%20save%20builders%20in%20snapshot.%20%22%0D%0A%0D%0A%09saveBuilders%20%3A%3D%20aBoolean"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"build:doing:",0
,function(token,aBlock){var self=this;
var builder=nil;
var result=nil;
var success=nil;
builder=self.$$default();
builder.open_(token);
success=false;
(function(){result=aBlock.value();return success=true;}).ensure_((function(){return builder.close_success_(token,success);}));
return result;}
,"building",unescape("build%3A%20token%20doing%3A%20aBlock%0D%0A%09%22%20Process%20a%20%28build...%3A%20spec%29%20definition%20marking%20methods%20generated%20while%20evaluating%20aBlock.%0D%0A%09Return%20the%20result%20of%20evaluating%20aBlock.%20%22%0D%0A%0D%0A%09%7C%20builder%20result%20success%20%7C%0D%0A%09builder%20%3A%3D%20self%20default.%0D%0A%09builder%20open%3A%20token.%0D%0A%09success%20%3A%3D%20false.%0D%0A%09%5B%09result%20%3A%3D%20aBlock%20value.%0D%0A%09%09success%20%3A%3D%20true.%0D%0A%09%5D%20ensure%3A%20%5B%20builder%20close%3A%20token%20success%3A%20success%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"register:",0
,function(aMethod){var self=this;
return (nil.isNil_(self["@default"])===false).and_((function(){return self["@default"].register_(aMethod);}));}
,"building",unescape("register%3A%20aMethod%0D%0A%09%22%20Register%20aMethod%20as%20related%20to%20current%20builder.%20%22%0D%0A%0D%0A%09%5Edefault%20notNil%20and%3A%20%5B%20default%20register%3A%20aMethod%20%5D"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"imageSaved:on:",0
,function(snapshot,stream){var self=this;
var code=nil;
if (self.saveBuilders().and_((function(){return (code=self.codeForBuilders()).notEmpty();}))){(function($rec){$rec.cr();return $rec.nextPutAll_(code);})(stream);};;
return smalltalk.superImplementor(smalltalk.APIBuilder.$klass,"imageSaved_on_").apply(self, [snapshot,stream]);}
,"snapshot",unescape("imageSaved%3A%20snapshot%20on%3A%20stream%0D%0A%09%22%20Private%20-%20A%20system%20snapshot%20has%20been%20taken%2C%20the%20receiver%20can%20restore%20state.%20%22%0D%0A%0D%0A%09%7C%20code%20%7C%0D%0A%09%28self%20saveBuilders%20and%3A%20%5B%0D%0A%09%09%28code%20%3A%3D%20self%20codeForBuilders%29%20notEmpty%20%5D%29%0D%0A%09%09ifTrue%3A%20%5B%20stream%20cr%3BnextPutAll%3A%20code%20%5D.%0D%0A%09%5Esuper%20imageSaved%3A%20snapshot%20on%3A%20stream"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"codeForBuilders",0
,function(){var self=this;
return self.codeFor_(smalltalk.Smalltalk.classes());}
,"code",unescape("codeForBuilders%0D%0A%09%22%20Return%20the%20%28native%29%20code%20to%20export%20the%20builders%20of%20the%20system.%20%22%0D%0A%0D%0A%09%5Eself%20codeFor%3A%20Smalltalk%20classes"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"codeFor:",0
,function(classes){var self=this;
var builders=nil;
builders=self.buildersFor_(classes);
if (builders.isEmpty()){return "";};;
return "smalltalk."._comma(self.name())._comma(unescape(".builders_%28%0D%0A"))._comma(smalltalk.Exporter.$$new().exportObject_(builders))._comma(unescape("%0D%0A%29%3B"));}
,"code",unescape("codeFor%3A%20classes%0D%0A%09%22%20Return%20the%20%28native%29%20code%20to%20export%20the%20builders%20defined%20by%20a%20collection%20of%20classes.%20%22%0D%0A%0D%0A%09%7C%20builders%20%7C%0D%0A%09builders%20%3A%3D%20self%20buildersFor%3A%20classes.%0D%0A%09builders%20isEmpty%20ifTrue%3A%20%5B%20%5E%27%27%20%5D.%0D%0A%09%5E%27smalltalk.%27%2C%20self%20name%20%2C%27.builders_%28%0D%0A%27%09%2C%28Exporter%20new%20exportObject%3A%20builders%29%20%2C%27%0D%0A%29%3B%27"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"buildersFor:",0
,function(classes){var self=this;
return classes.collect_((function(cls){return smalltalk.Array.with_with_with_(cls.name(),self.buildersIn_(cls.$klass),self.buildersIn_(cls));})).reject_((function(tuple){return tuple.second().isEmpty().and_((function(){return tuple.third().isEmpty();}));}));}
,"private",unescape("buildersFor%3A%20classes%0D%0A%09%22%20Private%20-%20Return%20the%20builders%20defined%20by%20a%20collection%20of%20classes.%0D%0A%09%23%28%20className%20%23%28--classbuilders--%29%20%23%28--instanceBuilders---%29%29%0D%0A%09%22%0D%0A%0D%0A%09%5E%28classes%20collect%3A%20%5B%3Acls%7C%0D%0A%09%09Array%0D%0A%09%09%09with%3A%20cls%20name%0D%0A%09%09%09with%3A%20%28self%20buildersIn%3A%20cls%20class%29%0D%0A%09%09%09with%3A%20%28self%20buildersIn%3A%20cls%29%0D%0A%09%5D%29%20reject%3A%20%5B%3Atuple%7C%0D%0A%09%09tuple%20second%20isEmpty%20and%3A%20%5Btuple%20third%20isEmpty%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"builders",0
,function(){var self=this;
return self.buildersFor_(smalltalk.Smalltalk.classes());}
,"private",unescape("builders%0D%0A%09%22%20Private%20-%20Return%20the%20builders%20defined%20in%20this%20system.%20%22%0D%0A%0D%0A%09%5Eself%20buildersFor%3A%20Smalltalk%20classes"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"buildersIn:",0
,function(aClass){var self=this;
var map=nil;
map=smalltalk.Array.$$new();
aClass.methodsDo_((function(mth){var assoc=nil;
assoc=mth.builder();return (((nil.isNil_(assoc)===false)).mustBeBoolean()==true ? (function(){return map.detect_ifNone_((function(tuple){return tuple.first()._eq(assoc);}),(function(){return map.add_(smalltalk.Array.with_with_(assoc,smalltalk.Array.$$new()));})).second().add_(mth.selector());})() : nil);}));
return map.collect_((function(tuple){return smalltalk.Array.with_with_with_(tuple.first().key(),tuple.first().value(),tuple.last());}));}
,"private",unescape("buildersIn%3A%20aClass%0D%0A%09%22%20Private%20-%20Return%20the%20builders%20used%20in%20aClass.%0D%0A%09%23%28%20builderType%20builderSpec%20%23%28%20selectors%20%29%20%29%0D%0A%09%22%0D%0A%0D%0A%09%7C%20map%20%7C%0D%0A%09map%20%3A%3D%20Array%20new.%0D%0A%09aClass%20methodsDo%3A%20%5B%3Amth%7C%20%7C%20assoc%20%7C%0D%0A%09%09assoc%20%3A%3D%20mth%20builder.%0D%0A%09%09assoc%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09%28map%09detect%3A%20%5B%3Atuple%7C%20tuple%20first%20%3D%20assoc%20%5D%0D%0A%09%09%09%09ifNone%3A%20%5B%20map%20add%3A%20%28Array%20with%3A%20assoc%20with%3A%20Array%20new%29%20%5D%29%0D%0A%09%09%09%09second%20add%3A%20mth%20selector%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Emap%20collect%3A%20%5B%3Atuple%7C%20Array%0D%0A%09%09with%3A%20tuple%20first%20key%0D%0A%09%09with%3A%20tuple%20first%20value%0D%0A%09%09with%3A%20tuple%20last%0D%0A%09%5D"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"builders:",0
,function(spec){var self=this;
spec.do_((function(builder){var cls=nil;
cls=smalltalk.Smalltalk.at_ifAbsent_(builder.first(),(function(){return nil;}));return (((nil.isNil_(cls)===false)).mustBeBoolean()==true ? (function(){self.setBuilders_in_(builder.second(),cls.$klass);return self.setBuilders_in_(builder.third(),cls);})() : nil);}));
return self;}
,"private",unescape("builders%3A%20spec%0D%0A%09%22%20Private%20-%20Set%20the%20builders%20on%20this%20system%20according%20to%20bindings%20defined%20in%20spec.%0D%0A%09Note%20that%20missing%20bindings%20will%20be%20ignored%20and%20no%20check%20is%20done%20to%20ensure%20the%20bindings%20are%20in%20sync%20with%20target.%0D%0A%09%22%0D%0A%0D%0A%09spec%20do%3A%20%5B%3Abuilder%7C%20%7C%20cls%20%7C%0D%0A%09%09cls%20%3A%3D%20Smalltalk%20at%3A%20builder%20first%20ifAbsent%3A%20%5B%5D.%0D%0A%09%09cls%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09self%20setBuilders%3A%20builder%20second%20in%3A%20cls%20class.%0D%0A%09%09%09self%20setBuilders%3A%20builder%20third%20in%3A%20cls.%0D%0A%09%09%5D.%0D%0A%09%5D"));
smalltalk.bind(smalltalk.APIBuilder.$klass,"setBuilders:in:",0
,function(array,aClass){var self=this;
var builder=nil;
array.do_((function(map){builder=smalltalk.Association.key_value_(map.first(),map.second());return map.last().do_((function(selector){var mth=nil;
mth=aClass.compiledMethodAt_(selector);return (((nil.isNil_(mth)===false)).mustBeBoolean()==true ? (function(){return mth.builder_(builder);})() : nil);}));}));
return self;}
,"private",unescape("setBuilders%3A%20array%20in%3A%20aClass%0D%0A%09%22%20Private%20-%20Set%20the%20builders%20in%20array%20on%20aClass.%20%22%0D%0A%0D%0A%09%7C%20builder%20%7C%0D%0A%09array%20do%3A%20%5B%3Amap%7C%0D%0A%09%09builder%20%3A%3D%20Association%20key%3A%20map%20first%20value%3A%20map%20second.%0D%0A%09%09map%20last%20do%3A%20%5B%3Aselector%7C%20%7C%20mth%20%7C%0D%0A%09%09%09mth%20%3A%3D%20aClass%20compiledMethodAt%3A%20selector.%0D%0A%09%09%09mth%20notNil%20ifTrue%3A%20%5B%20mth%20builder%3A%20builder%20%5D.%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.APIBuilder,"register:",0
,function(aMethod){var self=this;
if ((nil.isNil_(self["@token"]))){return false;};;
if ((nil.isNil_(self["@bound"]))){self["@bound"]=smalltalk.Array.$$new();};;
self["@bound"].add_(aMethod);
return true;}
,"private",unescape("register%3A%20aMethod%0D%0A%09%22%20Private%20-%20Register%20aMethod%20as%20related%20to%20the%20receiver.%20%22%0D%0A%0D%0A%09token%20isNil%20ifTrue%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09bound%20isNil%20ifTrue%3A%20%5B%20bound%20%3A%3D%20Array%20new%20%5D.%0D%0A%09bound%20add%3A%20aMethod.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.APIBuilder,"open:",0
,function(spec){var self=this;
if ((nil.isNil_(self["@parents"]))){self["@parents"]=smalltalk.Array.$$new();};;
if ((nil.isNil_(self["@token"])===false)){self["@parents"].addLast_(self["@token"]);};;
self["@token"]=spec;
return self;}
,"private",unescape("open%3A%20spec%0D%0A%09%22%20Private%20-%20The%20spec%20will%20be%20generated.%0D%0A%09Prepare%20the%20receiver%20to%20track%20generated%20methods%20while%20building%20spec.%0D%0A%09%22%0D%0A%0D%0A%09parents%20isNil%20ifTrue%3A%20%5B%20parents%20%3A%3D%20Array%20new%20%5D.%0D%0A%09token%20notNil%20ifTrue%3A%20%5B%20parents%20addLast%3A%20token%20%5D.%0D%0A%09token%20%3A%3D%20spec."));
smalltalk.bind(smalltalk.APIBuilder,"close:success:",0
,function(spec,success){var self=this;
if (!(self["@token"]._eq(spec))){return self.error_("Builders out of sync.");};;
if ((nil.isNil_(self["@parents"])===false).and_((function(){return self["@parents"].notEmpty();}))){self["@token"]=self["@parents"].removeLast();
return success;};;
if ((nil.isNil_(self["@bound"])===false)){self["@bound"].do_((function(each){return self.mark_(each);}));
self["@token"]=self["@parents"]=self["@bound"]=nil;};;
return success;}
,"private",unescape("close%3A%20spec%20success%3A%20success%0D%0A%09%22%20Private%20-%20The%20spec%20has%20been%20implemented%2C%20backtrack%20to%20parent%20or%20mark%20collected%20methods.%20%22%0D%0A%0D%0A%09token%20%3D%20spec%20ifFalse%3A%20%5B%20%5Eself%20error%3A%20%27Builders%20out%20of%20sync.%27%20%5D.%0D%0A%09%28parents%20notNil%20and%3A%20%5B%20parents%20notEmpty%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09token%20%3A%3D%20parents%20removeLast.%0D%0A%09%09%5Esuccess%0D%0A%09%5D.%0D%0A%09bound%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09bound%20do%3A%20%5B%3Aeach%7C%20self%20mark%3A%20each%20%5D.%0D%0A%09%09token%20%3A%3D%20parents%20%3A%3D%20bound%20%3A%3D%20nil.%0D%0A%09%5D.%0D%0A%09%5Esuccess"));
smalltalk.bind(smalltalk.APIBuilder,"mark:",0
,function(aMethod){var self=this;
aMethod.builder_(self["@token"]);
return self;}
,"private",unescape("mark%3A%20aMethod%0D%0A%09%22%20Private%20-%20Mark%20aMethod%20as%20dependent%20of%20current%20token.%20%22%0D%0A%0D%0A%09aMethod%20builder%3A%20token"));
smalltalk.bind(smalltalk.APIBuilder,"fileOut:selectors:of:on:",0
,function(exporter,selectors,aClass,stream){var self=this;
var specs=nil;
specs=smalltalk.Set.$$new();
selectors.copy().do_((function(selector){var assoc=nil;
assoc=aClass.compiledMethodAt_(selector).builder();return (((nil.isNil_(assoc)===false)).mustBeBoolean()==true ? (function(){specs.add_(assoc);return selectors.remove_(selector);})() : nil);}));
specs.asArray().sorted_((function(a,b){return a.key()._lt_eq(b.key());})).do_((function(assoc){stream.nextPutAll_(aClass.name()._comma(" build")._comma(assoc.key()));if (assoc.value().notEmpty()){stream.nextPut_(":");
self.fileOutSpec_on_(assoc.value(),stream);};;return (function($rec){$rec.nextChunkPut_(" ");return $rec.cr();})(stream);}));
return self;}
,"fileOut",unescape("fileOut%3A%20exporter%20selectors%3A%20selectors%20of%3A%20aClass%20on%3A%20stream%0D%0A%09%22%20Fileout%20clustered%20sources%20for%20selectors%20onto%20stream.%0D%0A%09Note%3A%20Remove%20the%20clustered%20message%20selectors%20from%20selectors%20collection%20while%20fileOut.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20specs%20%7C%0D%0A%09specs%20%3A%3D%20Set%20new.%0D%0A%09selectors%20copy%20do%3A%20%5B%3Aselector%7C%20%7C%20assoc%20%7C%0D%0A%09%09assoc%20%3A%3D%20%28aClass%20compiledMethodAt%3A%20selector%29%20builder.%0D%0A%09%09assoc%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09specs%20add%3A%20assoc.%0D%0A%09%09%09selectors%20remove%3A%20selector.%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%28specs%20asArray%20sorted%3A%20%5B%3Aa%20%3Ab%7C%20a%20key%20%3C%3D%20b%20key%20%5D%29%20do%3A%20%5B%3Aassoc%7C%0D%0A%09%09stream%20nextPutAll%3A%20aClass%20name%20%2C%27%20build%27%20%2Cassoc%20key.%0D%0A%09%09assoc%20value%20notEmpty%20ifTrue%3A%20%5B%0D%0A%09%09%09stream%20nextPut%3A%20%24%3A.%0D%0A%09%09%09self%20fileOutSpec%3A%20assoc%20value%20on%3A%20stream%0D%0A%09%09%5D.%0D%0A%09%09stream%20nextChunkPut%3A%20%24%20%3Bcr%0D%0A%09%5D."));
smalltalk.bind(smalltalk.APIBuilder,"fileOutSpec:on:",0
,function(array,stream){var self=this;
(function($rec){$rec.nextPutAll_(unescape("%20%23%28"));return $rec.cr();})(stream);
array.do_((function(each){return (function($rec){$rec.tab();$rec.nextPutAll_(each.asLiteral());return $rec.cr();})(stream);}));
stream.nextPut_(unescape("%29"));
return self;}
,"fileOut",unescape("fileOutSpec%3A%20array%20on%3A%20stream%0D%0A%09%22%20Private%20-%20Fileout%20API%20spec%20in%20array%20onto%20stream.%20%22%0D%0A%0D%0A%09stream%20nextPutAll%3A%20%27%20%23%28%27%3Bcr.%0D%0A%09array%20do%3A%20%5B%3Aeach%7C%20stream%20tab%3B%20nextPutAll%3A%20each%20asLiteral%3Bcr%20%5D.%0D%0A%09stream%20nextPut%3A%20%24%29."));
smalltalk.bind(smalltalk.Snapshot.$klass,"contents",0
,function(){var self=this;
return self.$$new().contents();}
,"accessing",unescape("contents%0D%0A%09%22%20Return%20a%20snapshot%20of%20current%20system%20contents.%20%22%0D%0A%0D%0A%09%5Eself%20new%20contents"));
smalltalk.bind(smalltalk.Snapshot.$klass,"outputToFile:",0
,function(fileName){var self=this;
smalltalk.String.doItSelector().implementors().do_((function(each){return each.removeFromSystem();}));
self.contents().outputToFile_(fileName);
return self;}
,"saveImage",unescape("outputToFile%3A%20fileName%0D%0A%09%22%20Save%20a%20complete%20system%20snapshot%20in%20a%20file%20at%20fileName.%20%22%0D%0A%0D%0A%09String%20doItSelector%20implementors%20do%3A%20%5B%3Aeach%7C%20each%20removeFromSystem%20%5D.%0D%0A%09self%20contents%20outputToFile%3A%20fileName"));
smalltalk.bind(smalltalk.Snapshot,"contents",0
,function(){var self=this;
var stream=nil;
stream=smalltalk.String.$$new().writeStream();
self.saveImageOn_(stream);
return stream.contents();}
,"accessing",unescape("contents%0D%0A%09%22%20Return%20a%20snapshot%20of%20current%20system%20contents.%20%22%0D%0A%0D%0A%09%7C%20stream%20%7C%0D%0A%09stream%20%3A%3D%20String%20new%20writeStream.%0D%0A%09self%20saveImageOn%3A%20stream.%0D%0A%09%5Estream%20contents"));
smalltalk.bind(smalltalk.Snapshot,"saveImageOn:",0
,function(stream){var self=this;
return self.saveImage_on_(self.smalltalk(),stream);}
,"saveImage",unescape("saveImageOn%3A%20stream%0D%0A%09%22%20Save%20the%20current%20system%20contents%20onto%20stream.%20%22%0D%0A%0D%0A%09%5Eself%20saveImage%3A%20self%20smalltalk%20on%3A%20stream"));
smalltalk.bind(smalltalk.Snapshot,"saveImage:on:",0
,function(aSmalltalk,stream){var self=this;
(function($rec){$rec.writeBoot_on_(aSmalltalk,stream);$rec.write_on_("Prologue",stream);$rec.writeContents_on_(aSmalltalk,stream);$rec.write_on_("Epilogue",stream);$rec.writeLaunchExpressionOn_(stream);return $rec.yourself();})(self);
return self;}
,"saveImage",unescape("saveImage%3A%20aSmalltalk%20on%3A%20stream%0D%0A%09%22%20Save%20aSmalltalk%20system%20contents%20onto%20stream.%20%22%0D%0A%0D%0A%09self%0D%0A%09%09writeBoot%3A%20aSmalltalk%20on%3A%20stream%3B%0D%0A%09%09write%3A%20%23Prologue%20on%3A%20stream%3B%0D%0A%09%09writeContents%3A%20aSmalltalk%20on%3A%20stream%3B%0D%0A%09%09write%3A%20%23Epilogue%20on%3A%20stream%3B%0D%0A%09%09writeLaunchExpressionOn%3A%20stream%3B%0D%0A%09%09yourself"));
smalltalk.bind(smalltalk.Snapshot,"writeLicense:on:",0
,function(aSmalltalk,aStream){var self=this;
(function($rec){$rec.nextPutAll_(aSmalltalk.licenseTerms().asComment());return $rec.cr();})(aStream);
return self;}
,"writing",unescape("writeLicense%3A%20aSmalltalk%20on%3A%20aStream%0D%0A%09%22%20Private%20-%20Write%20the%20license%20terms%20onto%20aStream.%20%22%0D%0A%0D%0A%09aStream%20nextPutAll%3A%20aSmalltalk%20licenseTerms%20asComment%3B%20cr"));
smalltalk.bind(smalltalk.Snapshot,"writeJSBootOn:",0
,function(aStream){var self=this;
self.writeLicense_on_(smalltalk.Smalltalk.current(),aStream);
self.writeGlobals_on_(["StObject", "StBehavior", "StClass", "StMetaclass", "StMethod", "StUndefinedObject"],aStream);
self.writeNilDefinitionOn_(aStream);
self.writeGlobals_on_(["StSystemDictionary"],aStream);
self.writeSystemDefinitionOn_(aStream);
return self;}
,"js",unescape("writeJSBootOn%3A%20aStream%0D%0A%09%22%20Private%20-%20Write%20the%20jsBoot%20section%20of%20the%20image%20onto%20aStream.%20%22%0D%0A%0D%0A%09self%20writeLicense%3A%20Smalltalk%20current%20on%3A%20aStream.%0D%0A%09self%20writeGlobals%3A%20%23%28%0D%0A%09%09%23StObject%20%23StBehavior%20%23StClass%20%23StMetaclass%0D%0A%09%09%23StMethod%0D%0A%09%09%23StUndefinedObject%0D%0A%09%09%29%20on%3A%20aStream.%0D%0A%09self%20writeNilDefinitionOn%3A%20aStream.%0D%0A%09self%20writeGlobals%3A%20%23%28%20%23StSystemDictionary%20%29%20on%3A%20aStream.%0D%0A%09self%20writeSystemDefinitionOn%3A%20aStream."));
smalltalk.bind(smalltalk.Snapshot,"bootHeader",0
,function(){var self=this;
return "";}
,"encoding",unescape("bootHeader%0D%0A%09%22%20Private%20-%20Return%20the%20header%20for%20boot%20contents.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%7D"));
smalltalk.bind(smalltalk.Snapshot,"bootFooter",0
,function(){var self=this;
return "";}
,"encoding",unescape("bootFooter%0D%0A%09%22%20Private%20-%20Return%20the%20footer%20for%20boot%20contents.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%7D"));
smalltalk.bind(smalltalk.Snapshot,"writeBoot:on:",0
,function(aSmalltalk,aStream){var $s8Ret$={name:"stReturn"};try{var self=this;
var boot=nil;
var bootName=nil;
var save=nil;
save=(function(){var text=nil;
self.writeLicense_on_(aSmalltalk,aStream);text=self.bootHeader()._comma(boot)._comma(self.bootFooter());self.writeString_on_(text,aStream);return text;});
boot=aSmalltalk.at_ifAbsent_("BootSector",(function(){return nil;}));
if (smalltalk.responds_to_(boot,("for:"))){boot=boot.for_(self.nativeLanguage()).contents();
(((nil.isNil_(boot)===false)).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return self.writeString_on_(boot,aStream)})();throw($s8Ret$)})();})() : nil);};;
bootName="S8_Boot_"._comma(smalltalk.String.nativeFileExtension());
boot=aSmalltalk.at_ifAbsent_(bootName,(function(){return nil;}));
if ((nil.isNil_(boot)===false)){(function(){$s8Ret$.result=(function(){return save.value()})();throw($s8Ret$)})();};;
if (self.write_on_("Boot",aStream)){(function(){$s8Ret$.result=(function(){return self})();throw($s8Ret$)})();};;
bootName=unescape("s8/start/boot.")._comma(smalltalk.String.nativeFileExtension());
boot=bootName.fileContents();
if ((nil.isNil_(boot)===false)){(function(){$s8Ret$.result=(function(){return aSmalltalk.at_put_(bootName,save.value())})();throw($s8Ret$)})();};;
(function(){$s8Ret$.result=(function(){return self.error_("Missing boot resource")})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"writing",unescape("writeBoot%3A%20aSmalltalk%20on%3A%20aStream%0D%0A%09%22%20Private%20-%20Write%20the%20boot%20section%20of%20the%20image%20onto%20aStream.%20%22%0D%0A%0D%0A%09%7C%20boot%20bootName%20save%20%7C%0D%0A%09save%20%3A%3D%20%5B%20%7C%20text%20%7C%0D%0A%09%09self%20writeLicense%3A%20aSmalltalk%20on%3A%20aStream.%0D%0A%09%09text%20%3A%3D%20self%20bootHeader%20%2Cboot%20%2Cself%20bootFooter.%0D%0A%09%09self%20writeString%3A%20text%20on%3A%20aStream.%0D%0A%09%09text%0D%0A%09%5D.%0D%0A%0D%0A%09boot%20%3A%3D%20aSmalltalk%20at%3A%20%23BootSector%20ifAbsent%3A%20%5B%5D.%0D%0A%09%28boot%20respondsTo%3A%20%23for%3A%29%20ifTrue%3A%20%5B%0D%0A%09%09boot%20%3A%3D%20%28boot%20for%3A%20self%20nativeLanguage%29%20contents.%0D%0A%09%09boot%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09%5Eself%20writeString%3A%20boot%20on%3A%20aStream%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%0D%0A%09bootName%20%3A%3D%20%27S8_Boot_%27%20%2CString%20nativeFileExtension.%0D%0A%09boot%20%3A%3D%20aSmalltalk%20at%3A%20bootName%20ifAbsent%3A%20%5B%5D.%0D%0A%09boot%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%22embedded%20boot%20sector%22%0D%0A%09%09%5Esave%20value%0D%0A%09%5D.%0D%0A%0D%0A%09%28self%20write%3A%20%23Boot%20on%3A%20aStream%29%20ifTrue%3A%20%5B%0D%0A%09%09%22custom%20implementation%20by%20the%20receiver%22%0D%0A%09%09%5Eself%0D%0A%09%5D.%0D%0A%0D%0A%09bootName%20%3A%3D%20%27s8/start/boot.%27%20%2CString%20nativeFileExtension.%0D%0A%09boot%20%3A%3D%20bootName%20fileContents.%0D%0A%09boot%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%22embedding%20from%20external%20boot%20file%22%0D%0A%09%09%5EaSmalltalk%20at%3A%20bootName%20put%3A%20save%20value%0D%0A%09%5D.%0D%0A%09%5Eself%20error%3A%20%27Missing%20boot%20resource%27"));
smalltalk.bind(smalltalk.Snapshot,"write:on:",0
,function(section,aStream){var self=this;
var msg=nil;
aStream.cr();
msg="write"._comma(self.nativeLanguage().asUppercase())._comma(section)._comma("On:");
if (smalltalk.responds_to_(self,(msg))){self.perform_with_(msg,aStream);
return true;};;
return false;}
,"writing",unescape("write%3A%20section%20on%3A%20aStream%0D%0A%09%22%20Private%20-%20Write%20the%20section%20of%20the%20image%20onto%20aStream.%20%22%0D%0A%0D%0A%09%7C%20msg%20%7C%0D%0A%09aStream%20cr.%0D%0A%09msg%3A%3D%20%23write%20%2Cself%20nativeLanguage%20asUppercase%20%2Csection%20%2C%23On%3A.%0D%0A%09%28self%20respondsTo%3A%20msg%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20perform%3A%20msg%20with%3A%20aStream.%0D%0A%09%09%5Etrue%0D%0A%09%5D.%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.Snapshot,"writeJSPrologueOn:",0
,function(aStream){var self=this;
self.writeString_on_(unescape("smalltalk.%24state%20%3D%20%22js-toolset%22"),aStream);
self.writeGlobals_on_(["smalltalk.st", "smalltalk.doIt", "fileIn", "fileInJS", "doit", "exportCategory", "saveImage", "saveImageAs"],aStream);
self.writeString_on_(unescape("smalltalk.%24state%20%3D%20%22initialized%22"),aStream);
aStream.cr();
return self;}
,"js",unescape("writeJSPrologueOn%3A%20aStream%0D%0A%09%22%20Private%20-%20Write%20the%20js%20prologue%20section%20of%20the%20image%20onto%20aStream.%20%22%0D%0A%0D%0A%09self%20writeString%3A%20%27smalltalk.%24state%20%3D%20%22js-toolset%22%27%20on%3A%20aStream.%0D%0A%09self%20writeGlobals%3A%20%23%28%0D%0A%09%09%27smalltalk.st%27%20%27smalltalk.doIt%27%0D%0A%09%09%23fileIn%20%23fileInJS%0D%0A%09%09%23doit%20%23exportCategory%0D%0A%09%09%23saveImage%20%23saveImageAs%0D%0A%09%09%29%20on%3A%20aStream.%0D%0A%09self%20writeString%3A%20%27smalltalk.%24state%20%3D%20%22initialized%22%27%20on%3A%20aStream.%0D%0A%09aStream%20cr."));
smalltalk.bind(smalltalk.Snapshot,"launchExpression",0
,function(){var self=this;
return "smalltalk.launchImage()";}
,"encoding",unescape("launchExpression%0D%0A%09%22%20Private%20-%20Return%20the%20native%20launch%20expression.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%7D"));
smalltalk.bind(smalltalk.Snapshot,"writeLaunchExpressionOn:",0
,function(aStream){var self=this;
var launch=nil;
launch=self.launchExpression();
if ((nil.isNil_(launch)===false)){self.writeString_on_(launch,aStream);};;
return self;}
,"writing",unescape("writeLaunchExpressionOn%3A%20aStream%0D%0A%09%22%20Private%20-%20Write%20the%20epilogue%20section%20of%20the%20image%20onto%20aStream.%20%22%0D%0A%0D%0A%09%7C%20launch%20%7C%0D%0A%09launch%20%3A%3D%20self%20launchExpression.%0D%0A%09launch%20notNil%20ifTrue%3A%20%5B%20self%20writeString%3A%20launch%20on%3A%20aStream%20%5D."));
smalltalk.bind(smalltalk.Snapshot,"writeString:on:",0
,function(aString,aStream){var self=this;
(function($rec){$rec.cr();return $rec.nextPutAll_(aString);})(aStream);
return self;}
,"writing",unescape("writeString%3A%20aString%20on%3A%20aStream%0D%0A%09%22%20Private%20-%20Write%20aString%20onto%20aStream.%20%22%0D%0A%0D%0A%09aStream%20cr%3B%20nextPutAll%3A%20aString"));
smalltalk.bind(smalltalk.Snapshot,"globalNotFound:",0
,function(aGlobal){var self=this;
return self.error_("Global not found: "._comma(aGlobal));}
,"error",unescape("globalNotFound%3A%20aGlobal%0D%0A%09%22%20Private%20-%20A%20global%20at%20native%20level%20has%20not%20been%20found.%20%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%27Global%20not%20found%3A%20%27%2CaGlobal"));
smalltalk.bind(smalltalk.Snapshot,"writeGlobals:on:",0
,function(globals,aStream){var self=this;
var object=nil;
var varName=nil;
globals.do_((function(each){object=smalltalk.Smalltalk.current().eval_(each);if ((nil.isNil_(object))){object=self.globalNotFound_(each);};;return (((nil.isNil_(object)===false)).mustBeBoolean()==true ? (function(){varName=((each.includes_(".")).mustBeBoolean()==true ? (function(){return each;})() : (function(){return "var "._comma(each);})());return self.writeString_on_(varName._comma(unescape("%20%3D%20"))._comma(object.toString()),aStream);})() : nil);}));
return self;}
,"writing",unescape("writeGlobals%3A%20globals%20on%3A%20aStream%0D%0A%09%22%20Private%20-%20Write%20the%20prologue%20section%20of%20the%20image%20onto%20aStream.%20%22%0D%0A%0D%0A%09%7C%20object%20varName%20%7C%0D%0A%09globals%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09object%20%3A%3D%20Smalltalk%20current%20eval%3A%20each.%0D%0A%09%09object%20isNil%20ifTrue%3A%20%5B%20object%20%3A%3D%20self%20globalNotFound%3A%20each%20%5D.%0D%0A%09%09object%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09varName%20%3A%3D%20%28each%20includes%3A%20%24.%29%20ifTrue%3A%20%5B%20each%20%5D%20ifFalse%3A%20%5B%20%27var%20%27%2Ceach%20%5D.%0D%0A%09%09%09self%20writeString%3A%20varName%2C%27%20%3D%20%27%2C%20object%20toString%20on%3A%20aStream.%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Snapshot,"writeNilDefinitionOn:",0
,function(aStream){var self=this;
var sample=nil;
sample=nil;
self.writeString_on_(unescape("var%20nil%20%3D%20new%20StUndefinedObject%28%29"),aStream);
self.writeString_on_(unescape("nil.toString%20%3D%20")._comma(sample.toString.toString()),aStream);
self.writeString_on_(unescape("nil.isNil_%20%3D%20")._comma(sample["isNil_"].toString()),aStream);
self.writeString_on_(unescape("nil.is_eqeq_%20%3D%20")._comma(sample["is_eqeq_"].toString()),aStream);
return self;}
,"js",unescape("writeNilDefinitionOn%3A%20aStream%0D%0A%09%22%20Private%20-%20Write%20the%20definition%20of%20nil%20object%20onto%20aStream.%20%22%0D%0A%0D%0A%09%7C%20sample%20%7C%0D%0A%09sample%20%3A%3D%20nil.%0D%0A%09self%20writeString%3A%20%27var%20nil%20%3D%20new%20StUndefinedObject%28%29%27%20on%3A%20aStream.%0D%0A%09self%20writeString%3A%20%27nil.toString%20%3D%20%27%2C%20sample%23toString%20toString%20on%3A%20aStream.%0D%0A%09self%20writeString%3A%20%27nil.isNil_%20%3D%20%27%2C%20sample%23isNil_%20toString%20on%3A%20aStream.%0D%0A%09self%20writeString%3A%20%27nil.is_eqeq_%20%3D%20%27%2C%20sample%23is_eqeq_%20toString%20on%3A%20aStream."));
smalltalk.bind(smalltalk.Snapshot,"writeSystemDefinitionOn:",0
,function(aStream){var self=this;
self.writeString_on_(unescape("var%20smalltalk%20%3D%20new%20StSystemDictionary%28%29"),aStream);
self.writeString_on_(unescape("smalltalk.toString%20%3D%20function%28%29%20%7B%20return%20%22smalltalk%22%20%7D"),aStream);
self.writeString_on_(unescape("smalltalk.startup%28%29%3B"),aStream);
return self;}
,"js",unescape("writeSystemDefinitionOn%3A%20aStream%0D%0A%09%22%20Private%20-%20Write%20the%20definition%20of%20smalltalk%20object%20onto%20aStream.%20%22%0D%0A%0D%0A%09self%20writeString%3A%20%27var%20smalltalk%20%3D%20new%20StSystemDictionary%28%29%27%20on%3A%20aStream.%0D%0A%09self%20writeString%3A%20%27smalltalk.toString%20%3D%20function%28%29%20%7B%20return%20%22smalltalk%22%20%7D%27%20on%3A%20aStream.%0D%0A%09self%20writeString%3A%20%27smalltalk.startup%28%29%3B%27%20on%3A%20aStream."));
smalltalk.bind(smalltalk.Snapshot,"writeContents:on:",0
,function(aSmalltalk,aStream){var self=this;
var exporter=nil;
var globals=nil;
var classes=nil;
aStream.cr();
exporter=self.exporterFor_(aSmalltalk);
globals=self.globalsIn_(aSmalltalk);
classes=self.classesToExportFrom_with_(aSmalltalk,exporter).collect_((function(each){return each.aboutToSaveImage_on_(self,aStream);})).select_((function(each){return (nil.isNil_(each)===false);}));
exporter.exportClasses_on_(classes,aStream);
self.exportGlobals_with_on_(globals,exporter,aStream);
classes.do_((function(each){return each.imageSaved_on_(self,aStream);}));
return self;}
,"writing",unescape("writeContents%3A%20aSmalltalk%20on%3A%20aStream%0D%0A%09%22%20Private%20-%20Write%20the%20image%20section%20of%20aSmalltalk%20system%20onto%20aStream.%20%22%0D%0A%0D%0A%09%7C%20exporter%20globals%20classes%20%7C%0D%0A%09aStream%20cr.%0D%0A%09exporter%20%3A%3D%20self%20exporterFor%3A%20aSmalltalk.%0D%0A%09globals%20%3A%3D%20self%20globalsIn%3A%20aSmalltalk.%0D%0A%09classes%20%3A%3D%20%28%28self%20classesToExportFrom%3A%20aSmalltalk%20with%3A%20exporter%29%0D%0A%09%09collect%3A%20%5B%3Aeach%7C%20each%20aboutToSaveImage%3A%20self%20on%3A%20aStream%20%5D%29%0D%0A%09%09select%3A%20%5B%3Aeach%7C%20each%20notNil%20%5D.%0D%0A%22%09self%20exportGlobalNames%3A%20globals%20keys%20sorted%20in%3A%20aSmalltalk%20on%3A%20aStream.%0D%0A%22%09exporter%20exportClasses%3A%20classes%20on%3A%20aStream.%0D%0A%09self%20exportGlobals%3A%20globals%20with%3A%20exporter%20on%3A%20aStream.%0D%0A%09classes%20do%3A%20%5B%3Aeach%7C%20each%20imageSaved%3A%20self%20on%3A%20aStream%20%5D."));
smalltalk.bind(smalltalk.Snapshot,"exporterFor:",0
,function(aSmalltalk){var self=this;
return smalltalk.Exporter.$$new();}
,"exporting",unescape("exporterFor%3A%20aSmalltalk%0D%0A%09%22%20Private%20-%20Return%20the%20exporter%20to%20use%20to%20write%20contents%20of%20aSmalltalk%20system.%20%22%0D%0A%0D%0A%09%5EExporter%20new"));
smalltalk.bind(smalltalk.Snapshot,"globalsIn:",0
,function(aSmalltalk){var self=this;
var result=nil;
result=smalltalk.PoolDictionary.$$new();
aSmalltalk.globalsDo_((function(key){return result.at_put_(key,((function(){var $1$=(aSmalltalk[key]);if(nil.isNil_($1$))return nil;return $1$;})()));}));
return result;}
,"exporting",unescape("globalsIn%3A%20aSmalltalk%0D%0A%09%22%20Private%20-%20Return%20the%20globals%20defined%20in%20aSmalltalk%20system.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20PoolDictionary%20new.%0D%0A%09aSmalltalk%20globalsDo%3A%20%5B%3Akey%7C%20result%20at%3A%20key%20put%3A%20%28aSmalltalk%20basicAt%3A%20key%29%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.Snapshot,"classesToExportFrom:with:",0
,function(aSmalltalk,exporter){var self=this;
return aSmalltalk.rootClasses().inject_into_([],(function(total,each){total.addAll_(each.withAllSubclasses());return total;}));}
,"exporting",unescape("classesToExportFrom%3A%20aSmalltalk%20with%3A%20exporter%0D%0A%09%22%20Private%20-%20Return%20the%20classes%20to%20export%20from%20aSmalltalk%20system.%20%22%0D%0A%0D%0A%09%5EaSmalltalk%20rootClasses%0D%0A%09%09inject%3A%20%23%28%29%20into%3A%20%5B%3Atotal%20%3Aeach%7C%20total%20addAll%3A%20each%20withAllSubclasses.%20total%20%5D"));
smalltalk.bind(smalltalk.Snapshot,"exportGlobalNames:in:on:",0
,function(names,aSmalltalk,aStream){var self=this;
names.do_separatedBy_((function(key){return aStream.nextPutAll_(unescape("smalltalk%5B%22")._comma(key)._comma(unescape("%22%5D")));}),(function(){return aStream.nextPutAll_(unescape("%20%2C%20"));}));
(function($rec){$rec.nextPutAll_(unescape("%3Dnil"));return $rec.cr();})(aStream);
return self;}
,"exporting",unescape("exportGlobalNames%3A%20names%20in%3A%20aSmalltalk%20on%3A%20aStream%0D%0A%09%22%20Private%20-%20Export%20global%20names%20in%20aSmalltalk%20system.%20%22%0D%0A%0D%0A%09names%0D%0A%09%09do%3A%20%5B%3Akey%7C%20aStream%20nextPutAll%3A%20%27smalltalk%5B%22%27%2Ckey%2C%27%22%5D%27%20%5D%0D%0A%09%09separatedBy%3A%20%5B%20aStream%20nextPutAll%3A%20%27%20%2C%20%27%20%5D.%0D%0A%09aStream%20nextPutAll%3A%20%27%3Dnil%27%3B%20cr."));
smalltalk.bind(smalltalk.Snapshot,"exportGlobals:with:on:",0
,function(globals,exporter,aStream){var self=this;
globals.associationsDo_((function(binding){return self.writeString_on_(unescape("smalltalk%5B%22")._comma(binding.key())._comma(unescape("%22%5D%3D"))._comma(exporter.exportObject_(binding.value())),aStream);}));
return self;}
,"exporting",unescape("exportGlobals%3A%20globals%20with%3A%20exporter%20on%3A%20aStream%0D%0A%09%22%20Private%20-%20Export%20global%20objects%20onto%20aStream.%20%22%0D%0A%0D%0A%09globals%20associationsDo%3A%20%5B%3Abinding%7C%0D%0A%09%09self%20writeString%3A%20%27smalltalk%5B%22%27%2Cbinding%20key%2C%27%22%5D%3D%27%0D%0A%09%09%09%2C%28exporter%20exportObject%3A%20binding%20value%29%0D%0A%09%09%09on%3A%20aStream%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Transcript.$klass,"current",0
,function(){var self=this;
if ((nil.isNil_(smalltalk.Transcript.classVariableAt_("Current")))){self.current_(self.$$new());};;
return smalltalk.Transcript.classVariableAt_("Current");}
,"singleton",unescape("current%0D%0A%09%22%20Return%20the%20current%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09Current%20isNil%20ifTrue%3A%20%5B%20self%20current%3A%20self%20new%20%5D.%0D%0A%09%5ECurrent"));
smalltalk.bind(smalltalk.Transcript.$klass,"current:",0
,function(aTranscript){var self=this;
smalltalk.Transcript.classVariableAt_put_("Current",aTranscript);
return self;}
,"singleton",unescape("current%3A%20aTranscript%0D%0A%09%22%20Set%20the%20current%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09Current%20%3A%3D%20aTranscript"));
smalltalk.bind(smalltalk.Transcript.$klass,"cr",0
,function(){var self=this;
return self.current().cr();}
,"io",unescape("cr%0D%0A%09%22%20Output%20a%20line%20break%20onto%20the%20current%20instance.%20%22%0D%0A%0D%0A%09%5Eself%20current%20cr"));
smalltalk.bind(smalltalk.Transcript.$klass,"nextPutAll:",0
,function(aString){var self=this;
return self.current().nextPutAll_(aString);}
,"io",unescape("nextPutAll%3A%20aString%0D%0A%09%22%20Output%20aString%20onto%20the%20current%20instance.%20%22%0D%0A%0D%0A%09%5Eself%20current%20nextPutAll%3A%20aString"));
smalltalk.bind(smalltalk.Transcript.$klass,"show:",0
,function(aString){var self=this;
return self.current().show_(aString);}
,"io",unescape("show%3A%20aString%0D%0A%09%22%20Output%20aString%20onto%20the%20current%20instance%20%28ensure%20result%20is%20shown%29.%20%22%0D%0A%0D%0A%09%5Eself%20current%20show%3A%20aString"));
smalltalk.bind(smalltalk.Transcript,"initializeStreams",0
,function(){var self=this;
self["@streams"]=smalltalk.Array.$$new();
return self;}
,"initialize",unescape("initializeStreams%0D%0A%09%22%20Private%20-%20Initialize%20the%20streams%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09streams%20%3A%3D%20Array%20new"));
smalltalk.bind(smalltalk.Transcript,"streams",0
,function(){var self=this;
if ((nil.isNil_(self["@streams"]))){self.initializeStreams();};;
return self["@streams"];}
,"accessing",unescape("streams%0D%0A%09%22%20Return%20the%20streams%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09streams%20isNil%20ifTrue%3A%20%5B%20self%20initializeStreams%20%5D.%0D%0A%09%5Estreams"));
smalltalk.bind(smalltalk.Transcript,"streams:",0
,function(anObject){var self=this;
self["@streams"]=anObject;
return self;}
,"accessing",unescape("streams%3A%20anObject%0D%0A%09%22%20Set%20the%20streams%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09streams%20%3A%3D%20anObject"));
smalltalk.bind(smalltalk.Transcript,"cr",0
,function(){var self=this;
self.streams().do_((function(each){return each.cr();}));
return self;}
,"io",unescape("cr%0D%0A%09%22%20Output%20a%20line%20break%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20streams%20do%3A%20%5B%3Aeach%7C%20each%20cr%20%5D"));
smalltalk.bind(smalltalk.Transcript,"nextPutAll:",0
,function(aString){var self=this;
self.streams().do_((function(each){return each.nextPutAll_(aString.asString());}));
return self;}
,"io",unescape("nextPutAll%3A%20aString%0D%0A%09%22%20Output%20aString%20onto%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20streams%20do%3A%20%5B%3Aeach%7C%20each%20nextPutAll%3A%20aString%20asString%20%5D"));
smalltalk.bind(smalltalk.Transcript,"show:",0
,function(aString){var self=this;
self.nextPutAll_(aString);
return self;}
,"io",unescape("show%3A%20aString%0D%0A%09%22%20Output%20aString%20onto%20the%20receiver%20%28ensure%20result%20is%20shown%29.%20%22%0D%0A%0D%0A%09self%20nextPutAll%3A%20aString"));
smalltalk.bind(smalltalk.Transcript,"flush",0
,function(){var self=this;
self.streams().do_((function(each){return ((smalltalk.responds_to_(each,("flush"))).mustBeBoolean()==true ? (function(){return each.flush();})() : nil);}));
return self;}
,"io",unescape("flush%0D%0A%09%22%20Flush%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20streams%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09%28each%20respondsTo%3A%20%23flush%29%20ifTrue%3A%20%5B%20each%20flush%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.Transcript,"inUse",0
,function(){var self=this;
return (nil.isNil_(self["@streams"])===false).and_((function(){return self["@streams"].notEmpty();}));}
,"query",unescape("inUse%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20in%20use.%20%22%0D%0A%0D%0A%09%5Estreams%20notNil%20and%3A%20%5Bstreams%20notEmpty%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"undefined",0
,function(){var self=this;
return undefined;}
,"constants",unescape("undefined%0D%0A%09%22%20Private%20-%20Return%20the%20native%20representation%20for%20undefined.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%7D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"nativeNameFromSelector:",0
,function(aSymbol){var self=this;
return aSymbol;}
,"builders",unescape("nativeNameFromSelector%3A%20aSymbol%0D%0A%09%22%20Private%20-%20Return%20the%20property%20name%20for%20selector%20aSymbol.%20%22%0D%0A%0D%0A%09%5EaSymbol"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildGetterCode:",0
,function(aSelector){var self=this;
if (self.instanceVariableNames().includes_(aSelector)){return smalltalk.superImplementor(smalltalk.NativeObject.$klass,"buildGetterCode_").apply(self, [aSelector]);};;
return unescape("%5Eself%20handle%20basicAt%3A%20")._comma(self.nativeNameFromSelector_(aSelector).asLiteral());}
,"builders-private",unescape("buildGetterCode%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20source%20code%20to%20get%20aSelector.%20%22%0D%0A%0D%0A%09%28self%20instanceVariableNames%20includes%3A%20aSelector%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Esuper%20buildGetterCode%3A%20aSelector%0D%0A%09%5D.%0D%0A%09%5E%27%5Eself%20handle%20basicAt%3A%20%27%0D%0A%09%09%2C%28self%20nativeNameFromSelector%3A%20aSelector%29%20asLiteral"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildGetter:",0
,function(spec){var self=this;
if (spec.isString().or_((function(){return spec.isArray().and_((function(){return spec.second()._eq("initialize");}));}))){return smalltalk.superImplementor(smalltalk.NativeObject.$klass,"buildGetter_").apply(self, [spec]);};;
return self.buildTypedGetter_(spec);}
,"builders",unescape("buildGetter%3A%20spec%0D%0A%09%22%20Build%20method%20for%20getter.%20%22%0D%0A%0D%0A%09%28spec%20isString%20or%3A%20%5B%0D%0A%09%09spec%20isArray%20and%3A%20%5B%0D%0A%09%09spec%20second%20%3D%20%23initialize%20%5D%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Esuper%20buildGetter%3A%20spec%0D%0A%09%5D.%0D%0A%09%5Eself%20buildTypedGetter%3A%20spec"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildSetterCode:",0
,function(aSelector){var self=this;
if (self.instanceVariableNames().includes_(aSelector)){return smalltalk.superImplementor(smalltalk.NativeObject.$klass,"buildSetterCode_").apply(self, [aSelector]);};;
return "self handle basicAt: "._comma(self.nativeNameFromSelector_(aSelector).asLiteral())._comma(" put: anObject");}
,"builders-private",unescape("buildSetterCode%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20source%20code%20to%20set%20aSelector.%20%22%0D%0A%0D%0A%09%28self%20instanceVariableNames%20includes%3A%20aSelector%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Esuper%20buildSetterCode%3A%20aSelector%0D%0A%09%5D.%0D%0A%09%5E%27self%20handle%20basicAt%3A%20%27%0D%0A%09%09%2C%28self%20nativeNameFromSelector%3A%20aSelector%29%20asLiteral%0D%0A%09%09%2C%20%27%20put%3A%20anObject%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildSetter:",0
,function(spec){var self=this;
if (spec.isString().or_((function(){return spec.isArray().and_((function(){return spec.second()._eq("initialize");}));}))){return smalltalk.superImplementor(smalltalk.NativeObject.$klass,"buildSetter_").apply(self, [spec]);};;
return self.buildTypedSetter_(spec);}
,"builders",unescape("buildSetter%3A%20spec%0D%0A%09%22%20Private%20-%20Return%20source%20code%20for%20setter.%20%22%0D%0A%0D%0A%09%28spec%20isString%20or%3A%20%5B%0D%0A%09%09spec%20isArray%20and%3A%20%5B%0D%0A%09%09spec%20second%20%3D%20%23initialize%20%5D%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Esuper%20buildSetter%3A%20spec%0D%0A%09%5D.%0D%0A%09%5Eself%20buildTypedSetter%3A%20spec"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildTypedGetter:",0
,function(tuple){var self=this;
if (tuple.isString()){return self.buildGetter_(tuple);};;
return self.build_doing_("TypedGetter"._minus_gt(tuple),(function(){return self.implement_comment_as_category_(self.stMessageHeader_(tuple.first()),"Return the "._comma(tuple.first())._comma(" of the receiver."),self.apiPrologueFor_(tuple.last())._comma("self handle basicAt: ")._comma(self.nativeNameFromSelector_(tuple.first()).asLiteral())._comma(self.apiEpilogueFor_(tuple.last())),"accessing");}));}
,"builders",unescape("buildTypedGetter%3A%20tuple%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20get%20property%20from%20spec%20array.%20%22%0D%0A%0D%0A%09tuple%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildGetter%3A%20tuple%20%5D.%0D%0A%09%5Eself%20build%3A%20%23TypedGetter%20-%3E%20tuple%20doing%3A%20%5B%0D%0A%09%09self%0D%0A%09%09%09implement%3A%20%28self%20stMessageHeader%3A%20tuple%20first%29%0D%0A%09%09%09comment%3A%20%27Return%20the%20%27%20%2Ctuple%20first%20%2C%27%20of%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%09%20%28self%20apiPrologueFor%3A%20tuple%20last%29%0D%0A%09%09%09%09%2C%27self%20handle%20basicAt%3A%20%27%0D%0A%09%09%09%09%2C%28self%20nativeNameFromSelector%3A%20tuple%20first%29%20asLiteral%0D%0A%09%09%09%09%2C%28self%20apiEpilogueFor%3A%20tuple%20last%29%0D%0A%09%09%09category%3A%20%23accessing%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildTypedSetter:",0
,function(tuple){var self=this;
if (tuple.isString()){return self.buildSetter_(tuple);};;
return self.build_doing_("TypedSetter"._minus_gt(tuple),(function(){return self.implement_comment_as_category_(self.stMessageHeader_(tuple.first()._comma(": anObject")),"Set the "._comma(tuple.first())._comma(" of the receiver."),"self handle basicAt: "._comma(self.nativeNameFromSelector_(tuple.first()).asLiteral())._comma(" put: anObject handle"),"accessing");}));}
,"builders",unescape("buildTypedSetter%3A%20tuple%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20set%20property%20to%20spec%20array.%20%22%0D%0A%0D%0A%09tuple%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildSetter%3A%20tuple%20%5D.%0D%0A%09%5Eself%20build%3A%20%23TypedSetter%20-%3E%20tuple%20doing%3A%20%5B%0D%0A%09%09self%0D%0A%09%09%09implement%3A%20%28self%20stMessageHeader%3A%20tuple%20first%20%2C%20%27%3A%20anObject%27%29%0D%0A%09%09%09comment%3A%20%27Set%20the%20%27%20%2Ctuple%20first%20%2C%27%20of%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%27self%20handle%20basicAt%3A%20%27%0D%0A%09%09%09%09%2C%28self%20nativeNameFromSelector%3A%20tuple%20first%29%20asLiteral%0D%0A%09%09%09%09%2C%27%20put%3A%20anObject%20handle%27%0D%0A%09%09%09category%3A%20%23accessing%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildTypedGetters:",0
,function(specs){var self=this;
return self.build_doing_("TypedGetters"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildTypedGetter_(each);}));}));}
,"builders",unescape("buildTypedGetters%3A%20specs%0D%0A%09%22%20Build%20%28instance%29%20methods%20to%20get%20properties.%20%22%0D%0A%0D%0A%09%5Eself%20build%3A%20%23TypedGetters%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildTypedGetter%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildTypedAccessors:",0
,function(tuples){var self=this;
return self.build_doing_("TypedAccessors"._minus_gt(tuples),(function(){return tuples.compose_((function(each){return self.buildTypedGetter_(each)._comma(self.buildTypedSetter_(each));}));}));}
,"builders",unescape("buildTypedAccessors%3A%20tuples%0D%0A%09%22%20Build%20%28instance%29%20methods%20to%20get/set%20property%20from/to%20tuples.%20%22%0D%0A%09%0D%0A%09%5Eself%20build%3A%20%23TypedAccessors%20-%3E%20tuples%20doing%3A%20%5B%0D%0A%09%09tuples%20compose%3A%20%5B%3Aeach%7C%0D%0A%09%09%09%28self%20buildTypedGetter%3A%20each%29%0D%0A%09%09%09%2C%28self%20buildTypedSetter%3A%20each%29%0D%0A%09%09%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildFastGetter:",0
,function(aSelector){var self=this;
var type=nil;
var jsName=nil;
if (aSelector.isString()){type="";
jsName=self.nativeNameFromSelector_(aSelector);} else{type=aSelector.last();
jsName=self.nativeNameFromSelector_(aSelector.first());};;
return self.build_doing_("FastGetter"._minus_gt(aSelector),(function(){return self.implement_comment_as_category_(self.stMessageHeader_(aSelector),"Return the "._comma(aSelector)._comma(" of the receiver."),unescape("%5E")._comma(type)._comma(unescape("handle%23"))._comma(jsName),"accessing");}));}
,"builders",unescape("buildFastGetter%3A%20aSelector%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20get%20native%20property.%20%22%0D%0A%0D%0A%09%7C%20type%20jsName%20%7C%0D%0A%09aSelector%20isString%0D%0A%09%09ifTrue%3A%20%5B%20type%20%3A%3D%20%27%27.%20jsName%20%3A%3D%20self%20nativeNameFromSelector%3A%20aSelector%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20type%20%3A%3D%20aSelector%20last.%20jsName%20%3A%3D%20self%20nativeNameFromSelector%3A%20aSelector%20first%20%5D.%0D%0A%09%5Eself%20build%3A%20%23FastGetter%20-%3E%20aSelector%20doing%3A%20%5B%0D%0A%09%09self%0D%0A%09%09%09implement%3A%20%28self%20stMessageHeader%3A%20aSelector%29%0D%0A%09%09%09comment%3A%20%27Return%20the%20%27%20%2CaSelector%20%2C%27%20of%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%27%5E%27%20%2Ctype%20%2C%27handle%23%27%20%2CjsName%20category%3A%20%23accessing%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildFastSetter:",0
,function(aSelector){var self=this;
var jsName=nil;
if (aSelector.isString()){jsName=self.nativeNameFromSelector_(aSelector);} else{jsName=self.nativeNameFromSelector_(aSelector.first());};;
return self.build_doing_("FastSetter"._minus_gt(aSelector),(function(){return self.implement_comment_as_category_(self.stMessageHeader_(aSelector)._comma(": anObject"),"Set the "._comma(aSelector)._comma(" of the receiver."),unescape("handle%23")._comma(jsName)._comma(unescape("%20%3A%3D%20anObject")),"accessing");}));}
,"builders",unescape("buildFastSetter%3A%20aSelector%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20set%20native%20property.%20%22%0D%0A%0D%0A%09%7C%20jsName%20%7C%0D%0A%09aSelector%20isString%0D%0A%09%09ifTrue%3A%20%5B%20jsName%20%3A%3D%20self%20nativeNameFromSelector%3A%20aSelector%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20jsName%20%3A%3D%20self%20nativeNameFromSelector%3A%20aSelector%20first%20%5D.%0D%0A%09%5Eself%20build%3A%20%23FastSetter%20-%3E%20aSelector%20doing%3A%20%5B%0D%0A%09%09self%0D%0A%09%09%09implement%3A%20%28self%20stMessageHeader%3A%20aSelector%29%20%2C%27%3A%20anObject%27%0D%0A%09%09%09comment%3A%20%27Set%20the%20%27%20%2CaSelector%20%2C%27%20of%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%27handle%23%27%20%2CjsName%20%2C%27%20%3A%3D%20anObject%27%20category%3A%20%23accessing%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildFastAccessor:",0
,function(accessor){var self=this;
if (accessor.isString()){return self.build_doing_("FastAccessor"._minus_gt(accessor),(function(){return self.buildFastGetter_(accessor)._comma(self.buildFastSetter_(accessor));}));};;
return self.error_(unescape("Invalid%20%28Fast%29accessors."));}
,"builders",unescape("buildFastAccessor%3A%20accessor%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20access%20native%20property.%20%22%0D%0A%0D%0A%09accessor%20isString%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20build%3A%20%23FastAccessor%20-%3E%20accessor%20doing%3A%20%5B%0D%0A%09%09%09%28self%20buildFastGetter%3A%20accessor%29%0D%0A%09%09%09%2C%28self%20buildFastSetter%3A%20accessor%29%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Eself%20error%3A%20%27Invalid%20%28Fast%29accessors.%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildFastGetters:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildFastGetters_(specs.asArrayOfSubstrings());};;
return self.build_doing_("FastGetters"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildFastGetter_(each);}));}));}
,"builders",unescape("buildFastGetters%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20get%20properties.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildFastGetters%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23FastGetters%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildFastGetter%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildFastAccessors:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildFastAccessors_(specs.asArrayOfSubstrings());};;
return self.build_doing_("FastAccessors"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildFastAccessor_(each);}));}));}
,"builders",unescape("buildFastAccessors%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20access%20properties.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildFastAccessors%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23FastAccessors%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildFastAccessor%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"codeToNumber",0
,function(){var self=this;
return " || 0";}
,"constants",unescape("codeToNumber%0D%0A%09%22%20Private%20-%20Return%20the%20code%20to%20transform%20value%20to%20a%20number.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%7D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"handleAt:",0
,function(aSelector){var self=this;
return unescape("handle%23")._comma(self.nativeNameFromSelector_(aSelector));}
,"constants",unescape("handleAt%3A%20aSelector%0D%0A%09%22%20Private%20-%20Return%20the%20code%20for%20accessing%20instance%20handle.%20%22%0D%0A%0D%0A%09%5E%27handle%23%27%20%2C%28self%20nativeNameFromSelector%3A%20aSelector%29"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildNumberGetter:",0
,function(aSelector){var self=this;
return self.build_doing_("NumberGetter"._minus_gt(aSelector),(function(){return self.implement_comment_as_category_(self.stMessageHeader_(aSelector),"Return the "._comma(aSelector)._comma(" of the receiver."),unescape("%5E")._comma(self.handleAt_(aSelector))._comma(self.codeToNumber()),"accessing");}));}
,"builders",unescape("buildNumberGetter%3A%20aSelector%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20get%20native%20property.%20%22%0D%0A%0D%0A%09%5Eself%20build%3A%20%23NumberGetter%20-%3E%20aSelector%20doing%3A%20%5B%0D%0A%09%09self%0D%0A%09%09%09implement%3A%20%28self%20stMessageHeader%3A%20aSelector%29%0D%0A%09%09%09comment%3A%20%27Return%20the%20%27%20%2CaSelector%20%2C%27%20of%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%24%5E%20%2C%28self%20handleAt%3A%20aSelector%29%20%2Cself%20codeToNumber%0D%0A%09%09%09category%3A%20%23accessing%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildNumberAccessor:",0
,function(accessor){var self=this;
if (accessor.isString()){return self.build_doing_("NumberAccessor"._minus_gt(accessor),(function(){return self.buildNumberGetter_(accessor)._comma(self.buildSetter_(accessor));}));};;
return self.error_(unescape("Invalid%20%28Number%29accessors."));}
,"builders",unescape("buildNumberAccessor%3A%20accessor%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20access%20native%20property.%20%22%0D%0A%0D%0A%09accessor%20isString%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20build%3A%20%23NumberAccessor%20-%3E%20accessor%20doing%3A%20%5B%0D%0A%09%09%09%28self%20buildNumberGetter%3A%20accessor%29%0D%0A%09%09%09%2C%28self%20buildSetter%3A%20accessor%29%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Eself%20error%3A%20%27Invalid%20%28Number%29accessors.%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildNumberGetters:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildNumberGetters_(specs.asArrayOfSubstrings());};;
return self.build_doing_("NumberGetters"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildNumberGetter_(each);}));}));}
,"builders",unescape("buildNumberGetters%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20get%20properties.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildNumberGetters%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23NumberGetters%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildNumberGetter%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildNumberAccessors:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildNumberAccessors_(specs.asArrayOfSubstrings());};;
return self.build_doing_("NumberAccessors"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildNumberAccessor_(each);}));}));}
,"builders",unescape("buildNumberAccessors%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20access%20properties.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildNumberAccessors%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23NumberAccessors%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildNumberAccessor%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildBooleanGetter:",0
,function(aSelector){var self=this;
return self.build_doing_("BooleanGetter"._minus_gt(aSelector),(function(){return self.implement_comment_as_category_(self.stMessageHeader_(aSelector),"Return the "._comma(aSelector)._comma(" of the receiver."),unescape("%5Etrue%20%3D%3D%20")._comma(self.handleAt_(aSelector)),"accessing");}));}
,"builders",unescape("buildBooleanGetter%3A%20aSelector%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20get%20native%20property.%20%22%0D%0A%0D%0A%09%5Eself%20build%3A%20%23BooleanGetter%20-%3E%20aSelector%20doing%3A%20%5B%0D%0A%09%09self%0D%0A%09%09%09implement%3A%20%28self%20stMessageHeader%3A%20aSelector%29%0D%0A%09%09%09comment%3A%20%27Return%20the%20%27%20%2CaSelector%20%2C%27%20of%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%27%5Etrue%20%3D%3D%20%27%20%2C%28self%20handleAt%3A%20aSelector%29%0D%0A%09%09%09category%3A%20%23accessing%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildBooleanAccessor:",0
,function(accessor){var self=this;
if (accessor.isString()){return self.build_doing_("BooleanAccessor"._minus_gt(accessor),(function(){return self.buildBooleanGetter_(accessor)._comma(self.buildSetter_(accessor));}));};;
return self.error_(unescape("Invalid%20%28Boolean%29accessors."));}
,"builders",unescape("buildBooleanAccessor%3A%20accessor%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20access%20native%20property.%20%22%0D%0A%0D%0A%09accessor%20isString%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20build%3A%20%23BooleanAccessor%20-%3E%20accessor%20doing%3A%20%5B%0D%0A%09%09%09%28self%20buildBooleanGetter%3A%20accessor%29%0D%0A%09%09%09%2C%28self%20buildSetter%3A%20accessor%29%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Eself%20error%3A%20%27Invalid%20%28Boolean%29accessors.%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildBooleanGetters:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildBooleanGetters_(specs.asArrayOfSubstrings());};;
return self.build_doing_("BooleanGetters"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildBooleanGetter_(each);}));}));}
,"builders",unescape("buildBooleanGetters%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20get%20properties.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildBooleanGetters%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23BooleanGetters%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildBooleanGetter%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildBooleanAccessors:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildBooleanAccessors_(specs.asArrayOfSubstrings());};;
return self.build_doing_("BooleanAccessors"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildBooleanAccessor_(each);}));}));}
,"builders",unescape("buildBooleanAccessors%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20access%20properties.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildBooleanAccessors%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23BooleanAccessors%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildBooleanAccessor%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildIsBooleanGetter:",0
,function(aSelector){var self=this;
return self.build_doing_("IsBooleanGetter"._minus_gt(aSelector),(function(){var isSelector=nil;
isSelector="is"._comma(aSelector.asGlobalName());return self.implement_comment_as_category_(self.stMessageHeader_(isSelector),"Return the "._comma(aSelector)._comma(" of the receiver."),unescape("%5Etrue%20%3D%3D%20handle%23")._comma(self.nativeNameFromSelector_(isSelector)),"accessing");}));}
,"builders",unescape("buildIsBooleanGetter%3A%20aSelector%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20get%20native%20property.%20%22%0D%0A%0D%0A%09%5Eself%20build%3A%20%23IsBooleanGetter%20-%3E%20aSelector%20doing%3A%20%5B%20%7C%20isSelector%20%7C%0D%0A%09%09isSelector%20%3A%3D%20%20%23is%20%2CaSelector%20asGlobalName.%0D%0A%09%09self%0D%0A%09%09%09implement%3A%20%28self%20stMessageHeader%3A%20isSelector%29%0D%0A%09%09%09comment%3A%20%27Return%20the%20%27%20%2CaSelector%20%2C%27%20of%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%27%5Etrue%20%3D%3D%20handle%23%27%20%2C%28self%20nativeNameFromSelector%3A%20isSelector%29%0D%0A%09%09%09category%3A%20%23accessing%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildIsBooleanAccessor:",0
,function(accessor){var self=this;
if (accessor.isString()){return self.build_doing_("IsBooleanAccessor"._minus_gt(accessor),(function(){return self.buildIsBooleanGetter_(accessor)._comma(self.buildSetter_(accessor));}));};;
return self.error_(unescape("Invalid%20%28IsBoolean%29accessors."));}
,"builders",unescape("buildIsBooleanAccessor%3A%20accessor%0D%0A%09%22%20Build%20%28instance%29%20method%20to%20access%20native%20property.%20%22%0D%0A%0D%0A%09accessor%20isString%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20build%3A%20%23IsBooleanAccessor%20-%3E%20accessor%20doing%3A%20%5B%0D%0A%09%09%09%28self%20buildIsBooleanGetter%3A%20accessor%29%0D%0A%09%09%09%2C%28self%20buildSetter%3A%20accessor%29%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5Eself%20error%3A%20%27Invalid%20%28IsBoolean%29accessors.%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildIsBooleanGetters:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildIsBooleanGetters_(specs.asArrayOfSubstrings());};;
return self.build_doing_("IsBooleanGetters"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildIsBooleanGetter_(each);}));}));}
,"builders",unescape("buildIsBooleanGetters%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20get%20properties.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildIsBooleanGetters%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23IsBooleanGetters%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildIsBooleanGetter%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildIsBooleanAccessors:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildIsBooleanAccessors_(specs.asArrayOfSubstrings());};;
return self.build_doing_("IsBooleanAccessors"._minus_gt(specs),(function(){return specs.compose_((function(each){return self.buildIsBooleanAccessor_(each);}));}));}
,"builders",unescape("buildIsBooleanAccessors%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20access%20properties.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildIsBooleanAccessors%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23IsBooleanAccessors%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20compose%3A%20%5B%3Aeach%7C%20self%20buildIsBooleanAccessor%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildSubclasses:",0
,function(aCollection){var self=this;
var result=nil;
aCollection.do_((function(tuple){return result=((tuple.isString()).mustBeBoolean()==true ? (function(){return self.buildSubclass_arguments_typed_(tuple,[],[]);})() : (function(){return ((tuple.size()._eq((2))).mustBeBoolean()==true ? (function(){return self.buildSubclass_arguments_typed_(tuple.first(),tuple.second(),[]);})() : (function(){return self.buildSubclass_arguments_typed_(tuple.first(),tuple.second(),tuple.last());})());})());}));
return result;}
,"builder",unescape("buildSubclasses%3A%20aCollection%0D%0A%09%22%20Private%20-%20Build%20aCollection%20of%20subclasses%20of%20the%20receiver%20as%20wrappers%20to%20the%20library.%0D%0A%09Return%20the%20last%20class%20created%20%28or%20nil%29.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09aCollection%20do%3A%20%5B%3Atuple%7C%0D%0A%09%09result%20%3A%3D%20tuple%20isString%0D%0A%09%09ifTrue%3A%20%5B%20self%20buildSubclass%3A%20tuple%20arguments%3A%20%23%28%29%20typed%3A%20%23%28%29%20%5D%0D%0A%09%09ifFalse%3A%20%5B%0D%0A%09%09%09tuple%20size%20%3D%202%0D%0A%09%09%09ifTrue%3A%20%5B%20self%20buildSubclass%3A%20tuple%20first%20arguments%3A%20tuple%20second%20typed%3A%20%23%28%29%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20self%20buildSubclass%3A%20tuple%20first%20arguments%3A%20tuple%20second%20typed%3A%20tuple%20last%20%5D%0D%0A%09%09%20%5D%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildHierarchy:",0
,function(aCollection){var self=this;
var stream=nil;
var aClass=nil;
stream=aCollection.stream();
while(!(function(){return stream.atEnd();})()){aClass=self.buildSubclasses_(smalltalk.Array.with_(stream.next()));
((stream.peek().isArray()).mustBeBoolean()==true ? (function(){return aClass.buildHierarchy_(stream.next());})() : nil);};;
return self;}
,"builder",unescape("buildHierarchy%3A%20aCollection%0D%0A%09%22%20Private%20-%20Build%20aCollection%20of%20subclasses%20of%20the%20receiver%20as%20wrappers%20to%20the%20library.%20%22%0D%0A%20%0D%0A%09%7C%20stream%20aClass%20%7C%0D%0A%09stream%20%3A%3D%20aCollection%20stream.%0D%0A%09%5B%20stream%20atEnd%20%5D%20whileFalse%3A%20%5B%0D%0A%09%09aClass%20%3A%3D%20self%20buildSubclasses%3A%20%28Array%20with%3A%20stream%20next%29.%0D%0A%09%09stream%20peek%20isArray%20ifTrue%3A%20%5B%0D%0A%09%09%09aClass%20buildHierarchy%3A%20stream%20next%0D%0A%09%09%5D.%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildSubclass:arguments:typed:",0
,function(globalClassName,argList,typedArguments){var self=this;
var cls=nil;
var clsName=nil;
clsName=globalClassName.first().asUppercase()._comma(globalClassName.copyFrom_to_((2),globalClassName.size()));
clsName=self.classToken()._comma(clsName);
self.subclass_instanceVariableNames_category_(clsName,"",self.category());
return smalltalk.Smalltalk.at_(clsName);}
,"builder",unescape("buildSubclass%3A%20globalClassName%20arguments%3A%20argList%20typed%3A%20typedArguments%0D%0A%09%22%20Private%20-%20Build%20a%20subclass%20of%20the%20receiver%20implementing%20a%20wrapper%20to%20the%20library.%0D%0A%09Note%3A%20argList%20and%20typed%20arguments%20are%20ignored%3B%20see%20subclasses%20to%20find%20concrete%20implementors.%0D%0A%09Return%20the%20created%20class.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20cls%20clsName%20%7C%0D%0A%09clsName%20%3A%3D%20globalClassName%20first%20asUppercase%0D%0A%09%09%09%2C%28globalClassName%20copyFrom%3A%202%20to%3A%20globalClassName%20size%29.%0D%0A%09clsName%20%3A%3D%20self%20classToken%20%2CclsName.%0D%0A%09self%0D%0A%09%09subclass%3A%20clsName%0D%0A%09%09instanceVariableNames%3A%20%27%27%0D%0A%09%09category%3A%20self%20category.%0D%0A%09%5ESmalltalk%20at%3A%20clsName"));
smalltalk.bind(smalltalk.NativeObject.$klass,"classToken",0
,function(){var self=this;
return "";}
,"builder",unescape("classToken%0D%0A%09%22%20Private%20-%20Return%20the%20classToken%20to%20prepend/tag%20the%20receiver.%0D%0A%09The%20default%20answer%20is%20empty.%0D%0A%09The%20subclasses%20can%20subclass%20to%20build%20tagged%20subclasses.%0D%0A%09%22%0D%0A%0D%0A%09%5E%27%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"classToken:",0
,function(aToken){var self=this;
return self.buildClassToken_(aToken);}
,"builder",unescape("classToken%3A%20aToken%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20return%20the%20classToken%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20buildClassToken%3A%20aToken"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildClassToken:",0
,function(aToken){var self=this;
self.build_doing_("ClassToken"._minus_gt(aToken),(function(){return self.implement_comment_as_category_("classToken",unescape("Private%20-%20Return%20the%20classToken%20to%20prepend/tag%20the%20receiver."),unescape("%5E")._comma(aToken.asLiteral()),"builder");}));
return self;}
,"builder",unescape("buildClassToken%3A%20aToken%0D%0A%09%22%20Generate%20code%20to%20return%20the%20classToken%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20build%3A%20%23ClassToken%20-%3E%20aToken%20doing%3A%20%5B%0D%0A%09%09self%0D%0A%09%09%09implement%3A%20%23classToken%0D%0A%09%09%09comment%3A%20%27Private%20-%20Return%20the%20classToken%20to%20prepend/tag%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%24%5E%20%2CaToken%20asLiteral%0D%0A%09%09%09category%3A%20%23builder%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildFunctions:",0
,function(specs){var self=this;
self.build_doing_("Functions"._minus_gt(specs),(function(){return specs.do_((function(spec){return self.buildFunction_(spec);}));}));
return self;}
,"builders",unescape("buildFunctions%3A%20specs%0D%0A%09%22%20Generate%20code%20from%20specs%20as%20functions%20implemented%20by%20instance%20handle.%20%22%0D%0A%0D%0A%09self%20build%3A%20%23Functions%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20do%3A%20%5B%3Aspec%7C%20self%20buildFunction%3A%20spec%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildBooleanFunctions:",0
,function(specs){var self=this;
if (specs.isString()){return self.buildBooleanFunctions_(specs.asArrayOfSubstrings());};;
return self.build_doing_("BooleanFunctions"._minus_gt(specs),(function(){return self.build_functions_("Boolean",specs);}));}
,"builders",unescape("buildBooleanFunctions%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20evaluate%20simple%20functions%20returning%20a%20Boolean.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20buildBooleanFunctions%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20build%3A%20%23BooleanFunctions%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09self%20build%3A%20%23Boolean%20functions%3A%20specs%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"build:functions:",0
,function(aClassName,specs){var self=this;
if (specs.isString()){return self.build_functions_(aClassName,specs.asArrayOfSubstrings());};;
return self.buildFunctions_(specs.collect_((function(each){return ((each.isString()).mustBeBoolean()==true ? (function(){return smalltalk.Array.with_with_with_(each,[],aClassName);})() : (function(){if (!(each.isArray().and_((function(){return each.size()._eq((2));})))){self.error_("Invalid literal spec "._comma(each));};;return each.copyWith_(aClassName);})());})));}
,"builders",unescape("build%3A%20aClassName%20functions%3A%20specs%0D%0A%09%22%20Build%20methods%20to%20evaluate%20simple%20functions%20returning%20an%20instance%20of%20aClassName.%20%22%0D%0A%0D%0A%09specs%20isString%20ifTrue%3A%20%5B%20%5Eself%20build%3A%20aClassName%20functions%3A%20specs%20asArrayOfSubstrings%20%5D.%0D%0A%09%5Eself%20buildFunctions%3A%20%28specs%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09each%20isString%0D%0A%09%09ifTrue%3A%20%5B%20Array%20with%3A%20each%20with%3A%20%23%28%29%20with%3A%20aClassName%20%5D%0D%0A%09%09ifFalse%3A%20%5B%0D%0A%09%09%09%28each%20isArray%20and%3A%20%5Beach%20size%20%3D%202%5D%29%20ifFalse%3A%20%5B%0D%0A%09%09%09%09self%20error%3A%20%27Invalid%20literal%20spec%20%27%20%2Ceach%0D%0A%09%09%09%5D.%0D%0A%09%09%09each%20copyWith%3A%20aClassName%0D%0A%09%09%5D%0D%0A%09%5D%29"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiSupportFor:",0
,function(literal){var self=this;
var support=nil;
support=smalltalk.NativeHandle.support();
if (smalltalk.responds_to_(support,("handleAPIFor:")).and_((function(){return support.handleAPIFor_(literal);}))){return support;};;
return self;}
,"api",unescape("apiSupportFor%3A%20literal%0D%0A%09%22%20Private%20-%20Return%20the%20support%20to%20encode%20literal%20as%20native%20api.%0D%0A%09Note%3A%20literal%20can%20be%20a%20String%20or%20Array%20of%20api%20annotation%20literals.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20support%20%7C%0D%0A%09support%20%3A%3D%20NativeHandle%20support.%0D%0A%09%28%28support%20respondsTo%3A%20%23handleAPIFor%3A%29%0D%0A%09%09and%3A%20%5Bsupport%20handleAPIFor%3A%20literal%5D%29%0D%0A%09%09ifTrue%3A%20%5B%20%5Esupport%20%5D.%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiEpilogueFor:",0
,function(aReturnType){var self=this;
return self.apiSupportFor_(aReturnType).nativeEpilogueFor_(aReturnType);}
,"builders",unescape("apiEpilogueFor%3A%20aReturnType%0D%0A%09%22%20Private%20-%20Return%20the%20epilogue%20code%20to%20use%20to%20wrap%20a%20return.%20%22%0D%0A%0D%0A%09%5E%28self%20apiSupportFor%3A%20aReturnType%29%0D%0A%09%09nativeEpilogueFor%3A%20aReturnType"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiPrologueFor:",0
,function(aReturnType){var self=this;
return self.apiSupportFor_(aReturnType).nativePrologueFor_(aReturnType);}
,"builders",unescape("apiPrologueFor%3A%20aReturnType%0D%0A%09%22%20Private%20-%20Return%20the%20prologue%20to%20use%20to%20wrap%20a%20return.%20%22%0D%0A%0D%0A%09%5E%28self%20apiSupportFor%3A%20aReturnType%29%0D%0A%09%09nativePrologueFor%3A%20aReturnType"));
smalltalk.bind(smalltalk.NativeObject.$klass,"nativeEpilogueFor:",0
,function(aReturnType){var self=this;
return unescape("%29");}
,"builders",unescape("nativeEpilogueFor%3A%20aReturnType%0D%0A%09%22%20Private%20-%20Return%20the%20epilogue%20code%20to%20use%20to%20wrap%20a%20return.%20%22%0D%0A%0D%0A%09%5E%24%29"));
smalltalk.bind(smalltalk.NativeObject.$klass,"nativePrologueFor:",0
,function(aReturnType){var self=this;
if (aReturnType.isEmpty()){return unescape("%5Eself%20ignoreReturn%3A%20%28");};;
if (aReturnType.last()._eq(unescape("%28"))){return unescape("%5E")._comma(aReturnType);};;
if (aReturnType.last()._eq(unescape("+"))){return self.apiPrologueFor_(aReturnType.upTo_(unescape("+"))._comma(unescape("%5B%5D")));};;
if (aReturnType.includes_(unescape("%5B"))){return unescape("%5E")._comma(self.apiPrologueForArray_(aReturnType));};;
if (aReturnType.includes_(unescape("%3C"))){return unescape("%5E")._comma(self.apiPrologueForTuple_(aReturnType));};;
return unescape("%5E")._comma(self.apiWrapperFor_(aReturnType))._comma(unescape("%20@%20%28"));}
,"builders",unescape("nativePrologueFor%3A%20aReturnType%0D%0A%09%22%20Private%20-%20Return%20the%20prologue%20to%20use%20to%20wrap%20a%20return.%0D%0A%09aReturnType%20can%20be%3A%0D%0A%09%09-%20empty%20string%20to%20ignore%20api%20return%20value%20%28will%20return%20the%20receiver%29%0D%0A%09%09-%20suffixed%20by%20%5B...%5D%20or%20+%20to%20return%20an%20arrayed%20result%0D%0A%09%09-%20suffixed%20by%20type%20spec%20GlobalName%0D%0A%09%09-%20smalltalk%20code%20ending%20with%20%28%0D%0A%09%09-%20a%20global%20name%20understanding%20%23@%0D%0A%09%22%0D%0A%0D%0A%09aReturnType%20isEmpty%20ifTrue%3A%20%5B%20%5E%27%5Eself%20ignoreReturn%3A%20%28%27%20%5D.%0D%0A%09aReturnType%20last%20%3D%20%24%28%20ifTrue%3A%20%5B%20%5E%24%5E%20%2CaReturnType%20%5D.%0D%0A%09aReturnType%20last%20%3D%20%24+%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20apiPrologueFor%3A%20%28aReturnType%20upTo%3A%20%24+%29%20%2C%27%5B%5D%27%0D%0A%09%5D.%0D%0A%09%28aReturnType%20includes%3A%20%24%5B%29%20ifTrue%3A%20%5B%0D%0A%09%09%5E%24%5E%20%2C%28self%20apiPrologueForArray%3A%20aReturnType%29%0D%0A%09%5D.%0D%0A%09%28aReturnType%20includes%3A%20%24%3C%29%20ifTrue%3A%20%5B%0D%0A%09%09%5E%24%5E%20%2C%28self%20apiPrologueForTuple%3A%20aReturnType%29%0D%0A%09%5D.%0D%0A%0D%0A%09%5E%24%5E%20%2C%28self%20apiWrapperFor%3A%20aReturnType%29%20%2C%27%20@%20%28%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiPrologueForArray:",0
,function(aReturnType){var self=this;
var type=nil;
var stream=nil;
var kind=nil;
stream=aReturnType.stream();
type=self.apiWrapperFor_(stream.upTo_(unescape("%5B")));
kind=stream.upTo_(unescape("%5D"));
if (!(stream.atEnd())){return self.error_("Invalid brackets for array");};;
if (kind.isEmpty()){return type._comma(unescape("%20withAll%3A%20%28"));};;
if (!(kind.includes_(unescape("%2C")))){return type._comma(" of: ")._comma(self.apiWrapperFor_(kind))._comma(unescape("%20withAll%3A%20%28"));};;
return type._comma(" of: ")._comma(kind.printString())._comma(unescape("%20withAll%3A%20%28"));}
,"builders",unescape("apiPrologueForArray%3A%20aReturnType%0D%0A%09%22%20Private%20-%20Return%20the%20prologue%20to%20use%20to%20wrap%20an%20arrayed%20return.%0D%0A%09The%20returned%20code%20will%20be%20like%3A%0D%0A%09%09GlobalName%20withAll%3A%20%28%0D%0A%09%09GlobalName%20of%3A%20ElementClass%20withAll%3A%20%28%0D%0A%09%09GlobalName%20of%3A%20%27Type1%2CType2%2CType3%27%20withAll%3A%20%28%0D%0A%09%22%0D%0A%0D%0A%09%7C%20type%20stream%20kind%20%7C%0D%0A%09stream%20%3A%3D%20aReturnType%20stream.%0D%0A%09type%20%3A%3D%20self%20apiWrapperFor%3A%20%28stream%20upTo%3A%20%24%5B%29.%0D%0A%09kind%20%3A%3D%20stream%20upTo%3A%20%24%5D.%0D%0A%09stream%20atEnd%20ifFalse%3A%20%5B%20%5Eself%20error%3A%20%27Invalid%20brackets%20for%20array%27%20%5D.%0D%0A%09kind%20isEmpty%20ifTrue%3A%20%5B%20%5Etype%20%2C%27%20withAll%3A%20%28%27%5D.%0D%0A%09%28kind%20includes%3A%20%24%2C%29%20ifFalse%3A%20%5B%0D%0A%09%09%5Etype%20%2C%27%20of%3A%20%27%20%2C%28self%20apiWrapperFor%3A%20kind%29%20%2C%27%20withAll%3A%20%28%27%0D%0A%09%5D.%09%0D%0A%09%5Etype%20%2C%27%20of%3A%20%27%20%2Ckind%20printString%20%2C%27%20withAll%3A%20%28%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiPrologueForTuple:",0
,function(aReturnType){var self=this;
var type=nil;
var stream=nil;
var kind=nil;
stream=aReturnType.stream();
type=self.apiWrapperFor_(stream.upTo_(unescape("%3C")));
kind=stream.upTo_(unescape("%3E"));
if (!(stream.atEnd())){return self.error_("Invalid code for typle");};;
if (kind.isEmpty()){return self.error_("Empty types for tuple");};;
if (!(kind.includes_(unescape("%2C")))){return type._comma(" of: ")._comma(self.apiWrapperFor_(kind))._comma(unescape("%20with%3A%20%28"));};;
return type._comma(" of: ")._comma(kind.printString())._comma(unescape("%20with%3A%20%28"));}
,"builders",unescape("apiPrologueForTuple%3A%20aReturnType%0D%0A%09%22%20Private%20-%20Return%20the%20prologue%20to%20use%20to%20wrap%20a%20tuple%20return.%0D%0A%09The%20returned%20code%20will%20be%20like%3A%0D%0A%09%09GlobalName%20of%3A%20%27Type1%2CType2%2CType3%27%20with%3A%20%28%0D%0A%09%22%0D%0A%0D%0A%09%7C%20type%20stream%20kind%20%7C%0D%0A%09stream%20%3A%3D%20aReturnType%20stream.%0D%0A%09type%20%3A%3D%20self%20apiWrapperFor%3A%20%28stream%20upTo%3A%20%24%3C%29.%0D%0A%09kind%20%3A%3D%20stream%20upTo%3A%20%24%3E.%0D%0A%09stream%20atEnd%20ifFalse%3A%20%5B%20%5Eself%20error%3A%20%27Invalid%20code%20for%20typle%27%20%5D.%0D%0A%09kind%20isEmpty%20ifTrue%3A%20%20%5B%20%5Eself%20error%3A%20%27Empty%20types%20for%20tuple%27%20%5D.%0D%0A%09%28kind%20includes%3A%20%24%2C%29%20ifFalse%3A%20%5B%0D%0A%09%09%5Etype%20%2C%27%20of%3A%20%27%20%2C%28self%20apiWrapperFor%3A%20kind%29%20%2C%27%20with%3A%20%28%27%0D%0A%09%5D.%09%0D%0A%09%5Etype%20%2C%27%20of%3A%20%27%20%2Ckind%20printString%20%2C%27%20with%3A%20%28%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiWrapperFor:",0
,function(aName){var $s8Ret$={name:"stReturn"};try{var self=this;
smalltalk.Smalltalk.at_ifAbsent_(aName,(function(){return (function(){$s8Ret$.result=(function(){return self.defaultAPIWrapperFor_(aName)})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return aName})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"builders",unescape("apiWrapperFor%3A%20aName%0D%0A%09%22%20Private%20-%20Return%20the%20name%20to%20use%20to%20wrap%20API%20structures%20of%20type%20aName.%20%22%0D%0A%0D%0A%09Smalltalk%20at%3A%20aName%20ifAbsent%3A%20%5B%20%5Eself%20defaultAPIWrapperFor%3A%20aName%20%5D.%0D%0A%09%5EaName"));
smalltalk.bind(smalltalk.NativeObject.$klass,"defaultAPIWrapperFor:",0
,function(aName){var self=this;
return aName;}
,"builders",unescape("defaultAPIWrapperFor%3A%20aName%0D%0A%09%22%20Private%20-%20Return%20the%20default%20name%20to%20use%20to%20wrap%20API%20structures%20of%20type%20aName.%0D%0A%09Default%20implementation%20will%20defer%20lookup%20upto%20runtime.%0D%0A%09%22%0D%0A%0D%0A%09%5EaName"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildFunction:",0
,function(spec){var $s8Ret$={name:"stReturn"};try{var self=this;
if (spec.isArray()){if (spec.size()._eq((3)).and_((function(){return spec.last().isString();}))){return self.buildFunction_(smalltalk.Array.with_with_(smalltalk.Array.with_with_(spec.first(),spec.second()),spec.last()));};;
((spec.size()._eq((2)).and_((function(){return spec.last().isString();}))).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return self.buildFunction_prologue_epilogue_(spec.first(),self.apiPrologueFor_(spec.second()),self.apiEpilogueFor_(spec.second()))})();throw($s8Ret$)})();})() : nil);};;
(function(){$s8Ret$.result=(function(){return self.buildFunction_prologue_epilogue_(spec,unescape("%5E"),"")})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"builders",unescape("buildFunction%3A%20spec%0D%0A%09%22%20Private%20-%20Generate%20code%20from%20spec%20as%20functions%20on%20instance%20handle.%0D%0A%09spec%20can%20be%3A%0D%0A%09%09aSymbol%20%28e.g.%20any%20smalltalk%20message%20selector%29%0D%0A%09%09%23%28%20selector%20%23ReturnType%20%29%0D%0A%09%09%23%28%20selector%20%23%28%20argumentNames%20%29%20%23ReturnType%20%29%0D%0A%09%09%23%28%20%23%28%20selector%20%23%28%20argumentNames%20%29%20%29%20%23ReturnType%20%29%0D%0A%09%09%23%28%20selector%20%23%28%20argumentNames%20%29%20%29%0D%0A%09argumentNames%20must%20match%20selector%20pattern%0D%0A%09argumentNames%20can%20include%20kwargsStart%20to%20define%20keyword%20arguments%0D%0A%09if%20argumentNames%20is%20not%20given%2C%20it%20will%20be%20computed%20from%20selector%20pattern%0D%0A%09The%20function%20will%20return%20the%20result%20of%20calling%20the%20handle%2C%20wrapped%20to%20a%20ReturnType%0D%0A%09See%20implementation%20of%20%23apiPrologueFor%3A%20for%20documentation%20on%20%23ReturnType%0D%0A%09%22%0D%0A%0D%0A%09spec%20isArray%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20%28spec%20size%20%3D%203%20and%3A%20%5Bspec%20last%20isString%5D%29%20ifTrue%3A%20%5B%20%22%23%28%20selector%20%23%28%20argumentNames%20%29%20%23ReturnType%20%29%22%0D%0A%09%09%5Eself%20buildFunction%3A%20%28Array%0D%0A%09%09%09with%3A%20%28Array%20with%3A%20spec%20first%20with%3A%20spec%20second%29%0D%0A%09%09%09with%3A%20spec%20last%29%0D%0A%09%20%20%20%20%5D.%20%0D%0A%09%20%20%20%20%28spec%20size%20%3D%202%20and%3A%20%5Bspec%20last%20isString%5D%29%20ifTrue%3A%20%5B%20%22%23%28%20selector%20%23ReturnType%20%29%22%0D%0A%09%09%5Eself%0D%0A%09%09%09buildFunction%3A%20spec%20first%0D%0A%09%09%09prologue%3A%20%28self%20apiPrologueFor%3A%20spec%20second%29%0D%0A%09%09%09epilogue%3A%20%28self%20apiEpilogueFor%3A%20spec%20second%29%0D%0A%09%20%20%20%20%5D.%20%0D%0A%09%5D.%0D%0A%09%5Eself%20buildFunction%3A%20spec%20prologue%3A%20%24%5E%20epilogue%3A%20%27%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildFunction:prologue:epilogue:",0
,function(spec,prologue,epilogue){var self=this;
var args=nil;
if (spec.isArray()){return self.buildFunction_arguments_prologue_epilogue_(spec.first(),spec.last(),prologue,epilogue);};;
args=(1).to_(spec.occurrencesOf_(":")).collect_((function(i){return "arg"._comma(i.asString());}));
if (args.size()._eq((1))){args=["anObject"];};;
return self.buildFunction_arguments_prologue_epilogue_(spec,args,prologue,epilogue);}
,"builders",unescape("buildFunction%3A%20spec%20prologue%3A%20prologue%20epilogue%3A%20epilogue%20%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20spec%20to%20apply%20on%20instance%20handle.%20%22%0D%0A%0D%0A%09%7C%20args%20%7C%0D%0A%09spec%20isArray%20ifTrue%3A%20%5B%20%22%23%28%20selector%20%23%28argumentNames%29%20%29%22%0D%0A%09%09%5Eself%09buildFunction%3A%20spec%20first%20arguments%3A%20spec%20last%0D%0A%09%09%09prologue%3A%20prologue%20epilogue%3A%20epilogue%0D%0A%09%5D.%0D%0A%09args%20%3A%3D%20%20%281%20to%3A%20%28spec%20occurrencesOf%3A%20%24%3A%29%29%20collect%3A%20%5B%3Ai%7C%20%23arg%20%2C%20i%20asString%20%5D.%0D%0A%09args%20size%20%3D%201%20ifTrue%3A%20%5B%20args%20%3A%3D%20%23%28%20%23anObject%20%29%20%5D.%0D%0A%09%5Eself%0D%0A%09%09buildFunction%3A%20spec%20arguments%3A%20args%0D%0A%09%09prologue%3A%20prologue%20epilogue%3A%20epilogue%20%20"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiSelector:forArguments:",0
,function(callName,args){var self=this;
var parts=nil;
if (args.size()._eq(callName.argumentCount())){return callName;};;
parts=callName.asArrayOfSubstringsSeparatedBy_(":");
if (parts.last().isEmpty()){parts.removeLast();};;
parts.size()._plus((1)).to_do_(args.size(),(function(i){var argName=nil;
argName=args.at_(i);if (argName.isArray()){argName=argName.first();};;return parts.add_(argName.asSmalltalkName_(false));}));
return parts.inject_into_("",(function(total,part){return total._comma(part)._comma(":");}));}
,"builders",unescape("apiSelector%3A%20callName%20forArguments%3A%20args%0D%0A%09%22%20Private%20-%20Return%20the%20api%20selector%20to%20implement%20a%20native%20call%20with%20arguments.%20%22%0D%0A%0D%0A%09%7C%20parts%20%7C%0D%0A%09args%20size%20%3D%20callName%20argumentCount%20ifTrue%3A%20%5B%20%5EcallName%20%5D.%0D%0A%09parts%20%3A%3D%20callName%20asArrayOfSubstringsSeparatedBy%3A%20%24%3A.%0D%0A%09parts%20last%20isEmpty%20ifTrue%3A%20%5B%20parts%20removeLast%20%5D.%0D%0A%09parts%20size%20+%201%20to%3A%20args%20size%20do%3A%20%5B%3Ai%7C%20%7C%20argName%20%7C%0D%0A%09%09argName%20%3A%3D%20args%20at%3A%20i.%0D%0A%09%09argName%20isArray%20ifTrue%3A%20%5B%20argName%20%3A%3D%20argName%20first%20%5D.%0D%0A%09%09parts%20add%3A%20%28argName%20asSmalltalkName%3A%20false%29%0D%0A%09%5D.%0D%0A%09%5Eparts%20inject%3A%20%27%27%20into%3A%20%5B%3Atotal%20%3Apart%7C%20total%20%2Cpart%20%2C%24%3A%20%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiArgumentsFrom:",0
,function(array){var self=this;
return self.apiSupportFor_(array).nativeArgumentsFrom_(array);}
,"api",unescape("apiArgumentsFrom%3A%20array%0D%0A%09%22%20Private%20-%20Return%20the%20arguments%20for%20API%20call.%20%22%0D%0A%0D%0A%09%5E%28self%20apiSupportFor%3A%20array%29%0D%0A%09%09nativeArgumentsFrom%3A%20array"));
smalltalk.bind(smalltalk.NativeObject.$klass,"nativeArgumentsFrom:",0
,function(array){var self=this;
var tag=nil;
tag="_";
return array.collect_((function(each){return ((each.isString().and_((function(){return each.first()._eq(tag);}))).mustBeBoolean()==true ? (function(){var varName=nil;
varName=each.copyFrom_to_((2),each.size());return ((each.last()._eq(tag)).mustBeBoolean()==true ? (function(){return smalltalk.Array.with_with_(varName.copyFrom_to_((1),varName.size()._minus((1))),"handles");})() : (function(){return smalltalk.Array.with_with_(varName,"json");})());})() : (function(){return each;})());}));}
,"builders",unescape("nativeArgumentsFrom%3A%20array%0D%0A%09%22%20Private%20-%20Return%20the%20arguments%20for%20API%20call.%20%22%0D%0A%0D%0A%09%7C%20tag%20%7C%0D%0A%09tag%20%3A%3D%20%24_.%0D%0A%09%5Earray%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09%28each%20isString%20and%3A%20%5B%20each%20first%20%3D%20tag%20%5D%29%20ifTrue%3A%20%5B%20%7C%20varName%20%7C%0D%0A%09%09%09varName%20%3A%3D%20each%20copyFrom%3A%202%20to%3A%20each%20size.%0D%0A%09%09%09each%20last%20%3D%20tag%0D%0A%09%09%09ifTrue%3A%20%5B%20Array%20with%3A%20%28varName%20copyFrom%3A%201%20to%3A%20varName%20size%20-%201%29%20with%3A%20%23handles%20%5D%0D%0A%09%09%09ifFalse%3A%20%5B%20Array%20with%3A%20varName%20with%3A%20%23json%20%5D%0D%0A%09%09%5D%20ifFalse%3A%20%5B%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiSelectorAndFunctionName:forArguments:",0
,function(callName,args){var self=this;
var msgSelector=nil;
var apiSelector=nil;
if (!(callName.isString().or_((function(){return callName.isArray();})))){return self.error_("CallName must be aString or anArray");};;
msgSelector=self.apiSelector_forArguments_(((callName.isString()).mustBeBoolean()==true ? (function(){return self.apiMessage_for_(callName,args);})() : (function(){return self.apiMessage_for_(callName.first(),args);})()),args);
apiSelector=self.apiSelector_forArguments_(((callName.isString()).mustBeBoolean()==true ? (function(){return callName;})() : (function(){return callName.last();})()),args);
return smalltalk.Array.with_with_(msgSelector,apiSelector);}
,"builders",unescape("apiSelectorAndFunctionName%3A%20callName%20forArguments%3A%20args%0D%0A%09%22%20Private%20-%20Return%20the%20selector%20and%20api%20call%20to%20implement%20a%20native%20call%20with%20arguments.%20%22%0D%0A%0D%0A%09%7C%20msgSelector%20apiSelector%20%7C%0D%0A%09%28callName%20isString%20or%3A%20%5BcallName%20isArray%5D%29%20ifFalse%3A%20%5B%0D%0A%09%09%5Eself%20error%3A%20%27CallName%20must%20be%20aString%20or%20anArray%27%0D%0A%09%5D.%0D%0A%09msgSelector%20%3A%3D%20self%20apiSelector%3A%20%28callName%20isString%0D%0A%09%09ifTrue%3A%20%5B%20self%20apiMessage%3A%20callName%20for%3A%20args%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20self%20apiMessage%3A%20callName%20first%20for%3A%20args%20%5D%29%20forArguments%3A%20args.%0D%0A%09apiSelector%20%3A%3D%20self%20apiSelector%3A%20%28callName%20isString%0D%0A%09%09ifTrue%3A%20%5B%20callName%20%5D%20ifFalse%3A%20%5B%20callName%20last%20%5D%29%20forArguments%3A%20args.%0D%0A%0D%0A%09%5EArray%20with%3A%20msgSelector%20with%3A%20apiSelector"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiMessage:for:",0
,function(callName,args){var self=this;
var result=nil;
result=callName.copyWithout_(unescape("%21"));
if (self.nativeLanguage()._eq("lua")){((callName.startsWith_("_")).mustBeBoolean()==true ? (function(){return result=callName.copyFrom_to_((2),callName.size());})() : nil);};;
if (result.argumentCount()._gt((0))){return result;};;
return result.asLocalName();}
,"builders",unescape("apiMessage%3A%20callName%20for%3A%20args%0D%0A%09%22%20Private%20-%20Return%20the%20message%20to%20implement%20for%20calling%20function%20callName.%0D%0A%09The%20default%20action%20use%20the%20callName%20as%20local%20name.%0D%0A%09It%20can%20be%20refined%20by%20subclasses%20to%20edit%20the%20name%20for%20better%20smalltalk%20looking%20of%20the%20message.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20callName%20copyWithout%3A%20%24%21.%0D%0A%09self%20nativeLanguage%20%3D%20%23lua%20ifTrue%3A%20%5B%0D%0A%09%09%28callName%20startsWith%3A%20%24_%29%20ifTrue%3A%20%5B%20result%20%3A%3D%20callName%20copyFrom%3A%202%20to%3A%20callName%20size%20%5D.%0D%0A%09%5D.%0D%0A%09result%20argumentCount%20%3E%200%20ifTrue%3A%20%5B%20%5Eresult%20%5D.%0D%0A%09%5Eresult%20asLocalName%20"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiEncode:call:args:on:header:",0
,function(delegate,aSelector,array,code,header){var self=this;
var pair=nil;
var args=nil;
var msgStream=nil;
var apiStream=nil;
var argStream=nil;
var msg=nil;
var space=nil;
args=self.apiArgumentsFrom_(array);
pair=self.apiSelectorAndFunctionName_forArguments_(aSelector,args);
msgStream=pair.first().stream();
space=((pair.first().includes_(":")).mustBeBoolean()==true ? (function(){return ": ";})() : (function(){return " ";})());
apiStream=pair.last().stream();
argStream=args.stream();
code.nextPutAll_(delegate);
code.nextPut_(apiStream.next());
header.nextPut_(msgStream.next());
while(!(function(){return argStream.atEnd();})()){var param=nil;
var expr=nil;
if (argStream.peek().isString()){param=argStream.next();
expr=param;} else{param=argStream.peek().first();
expr=param._comma(" ")._comma(argStream.next().last());};;
code.nextPutAll_(apiStream.upTo_(":")._comma(": ")._comma(expr)._comma(" "));
header.nextPutAll_(msgStream.upTo_(":")._comma(space)._comma(param)._comma(" "));};;
code.nextPutAll_(apiStream.upToEnd());
header.nextPutAll_(msgStream.upToEnd());
return self;}
,"builders",unescape("apiEncode%3A%20delegate%0D%0A%09call%3A%20aSelector%20args%3A%20array%0D%0A%09on%3A%20code%20header%3A%20header%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20call%20function%20onto%20code%26header%20streams.%20%22%0D%0A%0D%0A%09%7C%20pair%20args%20msgStream%20apiStream%20argStream%20msg%20space%20%7C%0D%0A%09args%20%3A%3D%20self%20apiArgumentsFrom%3A%20array.%0D%0A%09pair%20%3A%3D%20self%20apiSelectorAndFunctionName%3A%20aSelector%20forArguments%3A%20args.%0D%0A%09msgStream%20%3A%3D%20pair%20first%20stream.%0D%0A%09space%20%3A%3D%20%28pair%20first%20includes%3A%20%24%3A%29%20ifTrue%3A%20%5B%20%27%3A%20%27%20%5D%20ifFalse%3A%20%5B%20%24%20%5D.%0D%0A%09apiStream%20%3A%3D%20pair%20last%20stream.%0D%0A%09argStream%20%3A%3D%20args%20stream.%0D%0A%0D%0A%09code%09nextPutAll%3A%20delegate.%0D%0A%09code%09nextPut%3A%20apiStream%20next.%0D%0A%09header%09nextPut%3A%20msgStream%20next.%0D%0A%0D%0A%09%5B%20argStream%20atEnd%20%5D%20whileFalse%3A%20%5B%20%7C%20param%20expr%20%7C%0D%0A%09%09argStream%20peek%20isString%0D%0A%09%09ifTrue%3A%20%5B%20param%20%3A%3D%20argStream%20next.%20expr%20%3A%3D%20param%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20param%20%3A%3D%20argStream%20peek%20first.%20expr%20%3A%3D%20param%20%2C%27%20%27%20%2CargStream%20next%20last.%20%5D.%0D%0A%09%09code%20nextPutAll%3A%20%28apiStream%20upTo%3A%20%24%3A%29%20%2C%27%3A%20%27%20%2Cexpr%20%2C%27%20%27.%0D%0A%09%09header%20nextPutAll%3A%20%28msgStream%20upTo%3A%20%24%3A%29%20%2Cspace%20%2Cparam%20%2C%27%20%27.%0D%0A%09%5D.%0D%0A%0D%0A%09code%20%20%20nextPutAll%3A%20apiStream%20upToEnd.%0D%0A%09header%20nextPutAll%3A%20msgStream%20upToEnd."));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiEncode:call:args:kwargs:on:header:",0
,function(delegate,aSelector,array,kwarray,code,header){var self=this;
var pair=nil;
var args=nil;
var kwargs=nil;
var msgStream=nil;
var argStream=nil;
var msg=nil;
var space=nil;
args=self.apiArgumentsFrom_(array);
kwargs=self.apiArgumentsFrom_(kwarray);
pair=self.apiSelectorAndFunctionName_forArguments_(aSelector,args._comma(kwargs));
msgStream=pair.first().stream();
space=((pair.first().includes_(":")).mustBeBoolean()==true ? (function(){return ": ";})() : (function(){return " ";})());
argStream=args._comma(kwargs).stream();
header.nextPut_(msgStream.next());
while(!(function(){return argStream.atEnd();})()){var param=nil;
if (argStream.peek().isString()){param=argStream.next();} else{param=argStream.next().first();};;
header.nextPutAll_(msgStream.upTo_(":")._comma(space)._comma(param)._comma(" "));};;
header.nextPutAll_(msgStream.upToEnd());
code.nextPut_(unescape("%28"));
code.nextPutAll_(self.apikw_call_args_kwargs_header_(delegate,pair.last(),args,kwargs,header));
code.nextPut_(unescape("%29"));
return self;}
,"builders",unescape("apiEncode%3A%20delegate%0D%0A%09call%3A%20aSelector%20args%3A%20array%20kwargs%3A%20kwarray%0D%0A%09on%3A%20code%20header%3A%20header%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20call%20function%20onto%20code%26header%20streams.%20%22%0D%0A%0D%0A%09%7C%20pair%20args%20kwargs%20msgStream%20argStream%20msg%20space%20%7C%0D%0A%09args%20%3A%3D%20self%20apiArgumentsFrom%3A%20array.%0D%0A%09kwargs%20%3A%3D%20self%20apiArgumentsFrom%3A%20kwarray.%0D%0A%09pair%20%3A%3D%20self%20apiSelectorAndFunctionName%3A%20aSelector%20forArguments%3A%20args%20%2Ckwargs.%0D%0A%09msgStream%20%3A%3D%20pair%20first%20stream.%0D%0A%09space%20%3A%3D%20%28pair%20first%20includes%3A%20%24%3A%29%20ifTrue%3A%20%5B%20%27%3A%20%27%20%5D%20ifFalse%3A%20%5B%20%24%20%5D.%0D%0A%09argStream%20%3A%3D%20%28args%20%2Ckwargs%29%20stream.%0D%0A%0D%0A%09header%09nextPut%3A%20msgStream%20next.%0D%0A%09%5B%20argStream%20atEnd%20%5D%20whileFalse%3A%20%5B%20%7C%20param%20%7C%0D%0A%09%09argStream%20peek%20isString%0D%0A%09%09ifTrue%3A%20%5B%20param%20%3A%3D%20argStream%20next%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20param%20%3A%3D%20argStream%20next%20first%20%5D.%0D%0A%09%09header%20nextPutAll%3A%20%28msgStream%20upTo%3A%20%24%3A%29%20%2Cspace%20%2Cparam%20%2C%27%20%27.%0D%0A%09%5D.%0D%0A%09header%20nextPutAll%3A%20msgStream%20upToEnd.%0D%0A%0D%0A%09code%20nextPut%3A%20%24%28.%0D%0A%09code%20nextPutAll%3A%20%28self%0D%0A%09%09apikw%3A%20delegate%20call%3A%20pair%20last%0D%0A%09%09args%3A%20args%20kwargs%3A%20kwargs%0D%0A%09%09header%3A%20header%29.%0D%0A%09code%20nextPut%3A%20%24%29."));
smalltalk.bind(smalltalk.NativeObject.$klass,"apikw:call:args:kwargs:header:",0
,function(delegate,aSelector,array,kwarray,header){var $s8Ret$={name:"stReturn"};try{var self=this;
var callArgs=nil;
callArgs="self call: "._comma(self.apikw_delegate_(aSelector,delegate))._comma(unescape("%0D%0A%09args%3A%20"))._comma(self.apikwArguments_named_(array,false));
if (kwarray.isEmpty()){((aSelector.includes_(unescape("%21"))).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return callArgs._comma(" kwargs: nil")})();throw($s8Ret$)})();})() : (function(){if (array.notEmpty()){header.nextPutAll_("kwargs");};;header.nextPutAll_(": kwargs");return (function(){$s8Ret$.result=(function(){return callArgs._comma(" kwargs: kwargs")})();throw($s8Ret$)})();})());};;
(function(){$s8Ret$.result=(function(){return callArgs._comma(" kwargs: ")._comma(self.apikwArguments_named_(kwarray,true))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"builders",unescape("apikw%3A%20delegate%20call%3A%20aSelector%20args%3A%20array%20kwargs%3A%20kwarray%20header%3A%20header%0D%0A%09%22%20Private%20-%20Returns%20the%20code%20for%20a%20keyword%20function%20call.%20%22%0D%0A%0D%0A%09%7C%20callArgs%20%7C%0D%0A%09callArgs%20%3A%3D%20%27self%20call%3A%20%27%20%2C%28self%20apikw%3A%20aSelector%20delegate%3A%20delegate%29%20%2C%27%0D%0A%09args%3A%20%27%20%20%20%20%20%2C%28self%20apikwArguments%3A%20array%20named%3A%20false%29.%0D%0A%09kwarray%20isEmpty%20ifTrue%3A%20%5B%0D%0A%09%09%28aSelector%20includes%3A%20%24%21%29%0D%0A%09%09ifTrue%3A%20%5B%20%5EcallArgs%20%2C%27%20kwargs%3A%20nil%27%20%5D%0D%0A%09%09ifFalse%3A%20%5B%0D%0A%09%09%09array%20notEmpty%20ifTrue%3A%20%5B%20header%20nextPutAll%3A%20%23kwargs%20%5D.%0D%0A%09%09%09header%20nextPutAll%3A%20%27%3A%20kwargs%27.%0D%0A%09%09%09%5EcallArgs%20%2C%27%20kwargs%3A%20kwargs%27%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09%5EcallArgs%20%2C%27%20kwargs%3A%20%27%20%2C%28self%20apikwArguments%3A%20kwarray%20named%3A%20true%29"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apikw:delegate:",0
,function(aSelector,delegate){var self=this;
return unescape("%28")._comma(delegate.upTo_(unescape("%23")))._comma("basicAt: ")._comma(aSelector.upTo_(":").asLiteral())._comma(unescape("%29"));}
,"builders",unescape("apikw%3A%20aSelector%20delegate%3A%20delegate%0D%0A%09%22%20Private%20-%20Returns%20the%20code%20to%20access%20a%20keyword%20function.%20%22%0D%0A%0D%0A%09%5E%24%28%0D%0A%09%2C%28delegate%20upTo%3A%20%24%23%29%0D%0A%09%2C%27basicAt%3A%20%27%09%2C%28aSelector%20upTo%3A%20%23%3A%29%20asLiteral%0D%0A%09%2C%24%29"));
smalltalk.bind(smalltalk.NativeObject.$klass,"kwargsStart",0
,function(){var self=this;
return unescape("%7C");}
,"builders",unescape("kwargsStart%0D%0A%09%22%20Private%20-%20Returns%20the%20tag%20used%20for%20keyword%20arguments.%20%22%0D%0A%0D%0A%09%5E%23%7C"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apikwArguments:named:",0
,function(array,named){var self=this;
var element=nil;
element=(function(arg){var kw=nil;
var v=nil;
if (arg.isArray()){kw=arg.first();
v=arg.first()._comma(" ")._comma(arg.last());} else{kw=v=arg;};;return ((named).mustBeBoolean()==true ? (function(){return unescape("%28Array%20with%3A%20")._comma(kw.asLiteral())._comma(" with: ")._comma(v)._comma(unescape("%29"));})() : (function(){return v;})());});
if (array.isEmpty()){return unescape("%23%28%29");};;
if (array.size()._lt_eq((3))){return array.inject_into_(unescape("%28")._comma("Array"),(function(total,each){return total._comma(" with: ")._comma(element.value_(each));}))._comma(unescape("%29"));};;
return unescape("%28Array%20new")._comma(array.inject_into_("",(function(total,each){return total._comma(" add: ")._comma(element.value_(each))._comma(unescape("%3B"));})))._comma(unescape("%20yourself%29"));}
,"builders",unescape("apikwArguments%3A%20array%20named%3A%20named%0D%0A%09%22%20Private%20-%20Returns%20the%20code%20to%20pass%20arguments%20to%20kw%20function%20call.%20%22%0D%0A%0D%0A%09%7C%20element%20%7C%0D%0A%09element%20%3A%3D%20%5B%3Aarg%7C%20%7C%20kw%20v%20%7C%0D%0A%09%09arg%20isArray%0D%0A%09%09ifTrue%3A%20%5B%20kw%20%3A%3D%20arg%20first.%20v%20%3A%3D%20arg%20first%20%2C%27%20%27%2Carg%20last%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20kw%20%3A%3D%20v%20%3A%3D%20arg%20%5D.%0D%0A%09%09named%20ifTrue%3A%20%5B%27%28Array%20with%3A%20%27%20%2Ckw%20asLiteral%2C%27%20with%3A%20%27%20%2Cv%20%2C%24%29%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20v%20%5D%0D%0A%09%5D.%0D%0A%09array%20isEmpty%20ifTrue%3A%20%5B%20%5E%27%23%28%29%27%5D.%0D%0A%09array%20size%20%3C%3D%203%20ifTrue%3A%20%5B%0D%0A%09%09%5E%28array%20inject%3A%20%24%28%20%2C%23Array%20into%3A%20%5B%3Atotal%20%3Aeach%7C%0D%0A%09%09%09total%2C%27%20with%3A%20%27%2C%28element%20value%3A%20each%29%0D%0A%09%09%5D%29%20%2C%24%29%0D%0A%09%5D.%0D%0A%09%5E%27%28Array%20new%27%0D%0A%09%2C%28array%20inject%3A%20%27%27%20into%3A%20%5B%3Atotal%20%3Aeach%7C%0D%0A%09%09total%2C%27%20add%3A%20%27%2C%28element%20value%3A%20each%29%20%2C%24%3B%0D%0A%09%5D%29%20%2C%27%20yourself%29%27%20"));
smalltalk.bind(smalltalk.NativeObject.$klass,"apiEncode:call:arguments:on:header:",0
,function(delegate,aSelector,array,code,header){var self=this;
if (aSelector.includes_(unescape("%21")).or_((function(){return array.includes_(self.kwargsStart());}))){var stream=nil;
stream=array.readStream();
return self.apiEncode_call_args_kwargs_on_header_(delegate,aSelector,stream.upTo_(self.kwargsStart()),stream.upToEnd(),code,header);};;
return self.apiEncode_call_args_on_header_(delegate,aSelector,array,code,header);}
,"builders",unescape("apiEncode%3A%20delegate%0D%0A%09call%3A%20aSelector%20arguments%3A%20array%0D%0A%09on%3A%20code%20header%3A%20header%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20call%20function%20onto%20code%26header%20streams.%20%22%0D%0A%0D%0A%09%28%28aSelector%20includes%3A%20%24%21%29%20or%3A%20%5Barray%20includes%3A%20self%20kwargsStart%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%7C%20stream%20%7C%20stream%20%3A%3D%20array%20readStream.%0D%0A%09%09%5Eself%09apiEncode%3A%20delegate%20call%3A%20aSelector%0D%0A%09%09%09args%3A%20%28stream%20upTo%3A%20self%20kwargsStart%29%20kwargs%3A%20stream%20upToEnd%0D%0A%09%09%09on%3A%20code%20header%3A%20header%0D%0A%09%5D.%0D%0A%09%5Eself%09apiEncode%3A%20delegate%0D%0A%09%09call%3A%20aSelector%20args%3A%20array%0D%0A%09%09on%3A%20code%20header%3A%20header"));
smalltalk.bind(smalltalk.NativeObject.$klass,"codeForInstanceReceiver",0
,function(){var self=this;
return unescape("self%20handle%20%23");}
,"private",unescape("codeForInstanceReceiver%0D%0A%09%22%20Private%20-%20Return%20the%20code%20for%20the%20receiver%20of%20instance%20function%20call.%0D%0A%09Default%20answer%20asumes%20a%20wrapper%20calling%20with%20syntax%20sugar.%0D%0A%09This%20method%20can%20be%20refined%20to%20adapt%20to%20custom%20platform%20calls.%0D%0A%09%22%0D%0A%0D%0A%09%5E%27self%20handle%20%23%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildFunction:arguments:prologue:epilogue:",0
,function(spec,args,prologue,epilogue){var self=this;
var header=nil;
var code=nil;
if (args.includes_(self.kwargsStart())){var array=nil;
array=args.stream().upTo_(self.kwargsStart());
((args.last()._eq(self.kwargsStart())).mustBeBoolean()==true ? (function(){return self.buildFunction_arguments_prologue_epilogue_(spec,array,prologue,epilogue);})() : (function(){array.add_(self.kwargsStart());return self.buildFunction_arguments_prologue_epilogue_(spec,array,prologue,epilogue);})());};;
header="".writeStream();
code="".writeStream();
(function($rec){$rec.apiEncode_call_arguments_on_header_(self.codeForInstanceReceiver(),spec,args,code,header);return $rec.implement_comment_as_category_(self.stMessageHeader_(header.contents()),"Delegate to handle.",prologue._comma(code.contents())._comma(epilogue).replace_with_(unescape("%21"),unescape("%21")._comma(unescape("%21"))),"wrappers");})(self);
return self;}
,"builders",unescape("buildFunction%3A%20spec%20arguments%3A%20args%20prologue%3A%20prologue%20epilogue%3A%20epilogue%20%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20delegate%20message%20on%20instance%20handle.%20%22%0D%0A%0D%0A%09%7C%20header%20code%20%7C%0D%0A%09%28args%20includes%3A%20self%20kwargsStart%29%20ifTrue%3A%20%5B%20%7C%20array%20%7C%0D%0A%09%09array%20%3A%3D%20args%20stream%20upTo%3A%20self%20kwargsStart.%0D%0A%09%09args%20last%20%3D%20self%20kwargsStart%0D%0A%09%09ifTrue%3A%20%5B%20self%09buildFunction%3A%20spec%20arguments%3A%20array%0D%0A%09%09%09%09prologue%3A%20prologue%20epilogue%3A%20epilogue%20%5D%0D%0A%09%09ifFalse%3A%20%5B%0D%0A%09%09%09array%20add%3A%20self%20kwargsStart.%0D%0A%09%09%09self%09buildFunction%3A%20spec%20arguments%3A%20array%0D%0A%09%09%09%09prologue%3A%20prologue%20epilogue%3A%20epilogue%20%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09header%20%3A%3D%20%27%27%20writeStream.%20code%20%3A%3D%20%27%27%20writeStream.%0D%0A%09self%0D%0A%09%20%20apiEncode%3A%20self%20codeForInstanceReceiver%0D%0A%09%09call%3A%20spec%20arguments%3A%20args%0D%0A%09%09on%3A%20code%20header%3A%20header%3B%0D%0A%09%20%20implement%3A%20%28self%20stMessageHeader%3A%20header%20contents%29%0D%0A%09%09comment%3A%20%27Delegate%20to%20handle.%27%0D%0A%09%09as%3A%09%28%28prologue%0D%0A%09%09%09%2C%20code%20contents%0D%0A%09%09%09%2C%20epilogue%29%20replace%3A%20%24%21%20with%3A%20%24%21%20%2C%24%21%29%0D%0A%09%09category%3A%20%23wrappers"));
smalltalk.bind(smalltalk.NativeObject.$klass,"codeForStaticReceiver",0
,function(){var self=this;
return unescape("self%20nativeClass%20handle%20%23");}
,"private",unescape("codeForStaticReceiver%0D%0A%09%22%20Private%20-%20Return%20the%20code%20for%20the%20receiver%20of%20static%20function%20call.%0D%0A%09Default%20answer%20asumes%20a%20nativeClass%20wrapper%20calling%20with%20syntax%20sugar.%0D%0A%09This%20method%20can%20be%20refined%20to%20adapt%20to%20custom%20platform%20calls.%0D%0A%09%22%0D%0A%0D%0A%09%5E%27self%20nativeClass%20handle%20%23%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"nativeClass",0
,function(){var self=this;
return self.implementedBySubclass_("nativeClass");}
,"private",unescape("nativeClass%0D%0A%09%22%20Private%20-%20Return%20the%20wrapper%20for%20native%20binding%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23nativeClass"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildConstants",0
,function(){var self=this;
return self.buildConstants_(self.constants());}
,"builders",unescape("buildConstants%0D%0A%09%22%20Build%20the%20methods%20for%20constants%20defined%20in%20the%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%5Eself%20buildConstants%3A%20self%20constants"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildConstants:",0
,function(constants){var self=this;
return self.build_doing_("Constants"._minus_gt(constants),(function(){return ((constants.isString()).mustBeBoolean()==true ? (function(){var stream=nil;
stream=constants.stream();return (function(){while(!(function(){return stream.atEnd();})()){(function(){return self.buildConstant_(stream.nextLine().trimBlanks());})()};return nil})();})() : (function(){return constants.do_((function(each){return self.buildConstant_(each);}));})());}));}
,"builders",unescape("buildConstants%3A%20constants%0D%0A%09%22%20Build%20the%20methods%20for%20constants.%20%22%0D%0A%0D%0A%09%5Eself%20build%3A%20%23Constants%20-%3E%20constants%20doing%3A%20%5B%0D%0A%09%09constants%20isString%20ifTrue%3A%20%5B%20%7C%20stream%20%7C%0D%0A%09%09%09stream%20%3A%3D%20constants%20stream.%0D%0A%09%09%09%5Bstream%20atEnd%5D%20whileFalse%3A%20%5B%0D%0A%09%09%09%09self%20buildConstant%3A%20stream%20nextLine%20trimBlanks%0D%0A%09%09%09%5D%0D%0A%09%09%5D%20ifFalse%3A%20%5B%20constants%20do%3A%20%5B%3Aeach%7C%20self%20buildConstant%3A%20each%20%5D%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildConstant:",0
,function(spec){var self=this;
var selector=nil;
if (spec.isEmpty()){return self;};;
selector=spec.fromLast_(".").asSmalltalkName_(false);
self.$klass.implement_comment_as_category_(selector,"Return the constant value for "._comma(spec),unescape("%5E")._comma(self.codeToGetConstant_(spec)),"constants");
return self;}
,"builders",unescape("buildConstant%3A%20spec%0D%0A%09%22%20Private%20-%20Build%20the%20accessor%20for%20constant%20spec.%20%22%0D%0A%0D%0A%09%7C%20selector%20%7C%0D%0A%09spec%20isEmpty%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09selector%20%3A%3D%20%28spec%20fromLast%3A%20%24.%29%20asSmalltalkName%3A%20false.%0D%0A%09self%20class%0D%0A%09%09implement%3A%20selector%0D%0A%09%09comment%3A%20%27Return%20the%20constant%20value%20for%20%27%20%2Cspec%0D%0A%09%09as%3A%20%24%5E%20%2C%28self%20codeToGetConstant%3A%20spec%29%20category%3A%20%23constants"));
smalltalk.bind(smalltalk.NativeObject.$klass,"library",0
,function(){var self=this;
return self.$$new();}
,"private",unescape("library%0D%0A%09%22%20Private%20-%20Returns%20the%20library%20of%20the%20receiver.%0D%0A%09A%20new%20instance%20is%20returned%20by%20default.%0D%0A%09This%20method%20can%20be%20refined%20by%20subclasses%20%28e.g.%20implementing%20singleton%29.%20%22%0D%0A%0D%0A%09%5Eself%20new%20"));
smalltalk.bind(smalltalk.NativeObject.$klass,"codeToGetConstant:",0
,function(spec){var self=this;
if (spec.isArray()){return spec.last()._comma(unescape("%20@%20%28"))._comma(self.codeToGetConstant_(spec.first()))._comma(unescape("%29"));};;
return unescape("self%20library%20@%20")._comma(spec.asLiteral());}
,"builders",unescape("codeToGetConstant%3A%20spec%0D%0A%09%22%20Private%20-%20Returns%20the%20code%20to%20get%20the%20constant%20value%20for%20spec.%20%22%0D%0A%0D%0A%09spec%20isArray%20ifTrue%3A%20%5B%0D%0A%09%09%5Espec%20last%20%2C%27%20@%20%28%27%20%2C%28self%20codeToGetConstant%3A%20spec%20first%29%20%2C%24%29%0D%0A%09%5D.%0D%0A%09%5E%27self%20library%20@%20%27%20%2Cspec%20asLiteral%20"));
smalltalk.bind(smalltalk.NativeObject.$klass,"constants",0
,function(){var self=this;
return self.constantsIn_(self.library());}
,"builders",unescape("constants%0D%0A%09%22%20Returns%20the%20constants%20implemented%20by%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%5Eself%20constantsIn%3A%20self%20library"));
smalltalk.bind(smalltalk.NativeObject.$klass,"constantsIn:",0
,function(aNativeObject){var self=this;
return aNativeObject.keys().select_((function(key){return ["Number", "String"].includes_(self.typeOf_(aNativeObject._at(key)));})).reject_((function(key){return key.startsWith_("_");})).sorted();}
,"builders",unescape("constantsIn%3A%20aNativeObject%0D%0A%09%22%20Returns%20the%20constants%20implemented%20by%20aNativeObject.%20%22%0D%0A%0D%0A%09%5E%28%28aNativeObject%20keys%0D%0A%09%09select%3A%20%5B%3Akey%7C%20%23%28Number%20String%29%20includes%3A%20%28self%20typeOf%3A%20aNativeObject@key%29%20%5D%29%0D%0A%09%09reject%3A%20%5B%3Akey%7C%20key%20startsWith%3A%20%24_%20%5D%29%20sorted"));
smalltalk.bind(smalltalk.NativeObject.$klass,"newInstanceOf:withAll:",0
,function(aName,handlesArray){var self=this;
var fn=nil;
fn=self.instantiatorFor_(aName);

return fn.evaluateWithArguments_(handlesArray);}
,"api",unescape("newInstanceOf%3A%20aName%20withAll%3A%20handlesArray%0D%0A%09%22%20Private%20-%20Returns%20the%20handle%20result%20of%20instatiating%20native%20object%20of%20type%20aName.%20%22%0D%0A%0D%0A%09%7C%20fn%20%7C%0D%0A%09fn%20%3A%3D%20self%20instantiatorFor%3A%20aName.%0D%0A%09%7B%27self%27%7D.%0D%0A%09%5Efn%20evaluateWithArguments%3A%20handlesArray"));
smalltalk.bind(smalltalk.NativeObject.$klass,"instantiatorFor:",0
,function(aName){var self=this;
return self.library()._at(aName);}
,"api",unescape("instantiatorFor%3A%20aName%0D%0A%09%22%20Private%20-%20Returns%20the%20function%20to%20evaluate%20when%20creating%20native%20instance.%20%22%0D%0A%0D%0A%09%5Eself%20library%20@%20aName"));
smalltalk.bind(smalltalk.NativeObject.$klass,"instantiate:withAll:",0
,function(aNativeObjectClass,handlesArray){var self=this;
return aNativeObjectClass._at(self.newInstanceOf_withAll_(aNativeObjectClass.name(),handlesArray));}
,"api",unescape("instantiate%3A%20aNativeObjectClass%20withAll%3A%20handlesArray%0D%0A%09%22%20Returns%20the%20result%20of%20instatiating%20aCVObjectClass.%20%22%0D%0A%0D%0A%09%5EaNativeObjectClass%20@%20%28self%0D%0A%09%09newInstanceOf%3A%20aNativeObjectClass%20name%0D%0A%09%09withAll%3A%20handlesArray%29"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildSubclasses:in:",0
,function(classNames,packageName){var self=this;
return self.buildSubclasses_in_reference_(classNames,packageName,nil);}
,"builders",unescape("buildSubclasses%3A%20classNames%20in%3A%20packageName%0D%0A%09%22%20Private%20-%20Generate%20subclasses%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20buildSubclasses%3A%20classNames%20in%3A%20packageName%20reference%3A%20nil"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildSubclasses:in:reference:",0
,function(classSpecs,packageName,urlSite){var self=this;
classSpecs.do_((function(spec){return self.buildSubclass_in_reference_(spec,packageName,urlSite);}));
return self;}
,"builders",unescape("buildSubclasses%3A%20classSpecs%20in%3A%20packageName%20reference%3A%20urlSite%0D%0A%09%22%20Private%20-%20Generate%20subclasses%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09classSpecs%20do%3A%20%5B%3Aspec%7C%0D%0A%09%09self%20buildSubclass%3A%20spec%20in%3A%20packageName%20reference%3A%20urlSite%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildSubclass:in:",0
,function(spec,packageName){var self=this;
return self.buildSubclass_in_reference_(spec,packageName,nil);}
,"builders",unescape("buildSubclass%3A%20spec%20in%3A%20packageName%0D%0A%09%22%20Private%20-%20Generate%20a%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20buildSubclass%3A%20spec%20in%3A%20packageName%20reference%3A%20nil"));
smalltalk.bind(smalltalk.NativeObject.$klass,"urlReference",0
,function(){var self=this;
return "none";}
,"builders",unescape("urlReference%0D%0A%09%22%20Private%20-%20Returns%20the%20URL%20of%20package%20site.%20%22%0D%0A%0D%0A%09%5E%23none%20"));
smalltalk.bind(smalltalk.NativeObject.$klass,"urlSelector",0
,function(){var self=this;
return nil;}
,"builders",unescape("urlSelector%0D%0A%09%22%20Private%20-%20Returns%20the%20selector%20for%20urlSite.%20%22%0D%0A%0D%0A%09%5Enil%20"));
smalltalk.bind(smalltalk.NativeObject.$klass,"urlSelectors",0
,function(){var self=this;
return smalltalk.Array.with_(self.urlSelector());}
,"builders",unescape("urlSelectors%0D%0A%09%22%20Private%20-%20Returns%20the%20selector%20for%20urlSite.%20%22%0D%0A%0D%0A%09%5EArray%20with%3A%20self%20urlSelector%20"));
smalltalk.bind(smalltalk.NativeObject.$klass,"referenceTo:in:at:",0
,function(topic,packageName,urlSite){var self=this;
if (urlSite._eq(self.urlSelector())){return self.referenceTo_in_at_(topic,packageName,self.urlReference());};;
return urlSite._comma(unescape("/"))._comma((((nil.isNil_(packageName)===false)).mustBeBoolean()==true ? (function(){return packageName.replace_with_(".",unescape("/"))._comma(unescape("/"));})() : (function(){return "";})()))._comma(topic)._comma(".html");}
,"builders",unescape("referenceTo%3A%20topic%20in%3A%20packageName%20at%3A%20urlSite%0D%0A%09%22%20Private%20-%20Returns%20the%20URL%20to%20find%20documentation%20on%20topic.%20%22%0D%0A%0D%0A%09urlSite%20%3D%20self%20urlSelector%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%09referenceTo%3A%20topic%20in%3A%20packageName%0D%0A%09%09%09at%3A%20self%20urlReference%0D%0A%09%5D.%0D%0A%09%5EurlSite%20%2C%24/%0D%0A%09%09%2C%28packageName%20notNil%20ifTrue%3A%20%5B%20%28packageName%20replace%3A%20%24.%20with%3A%20%24/%29%20%2C%24/%20%5D%20ifFalse%3A%20%5B%27%27%5D%29%0D%0A%09%09%2Ctopic%20%2C%27.html%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildSubclass:in:reference:",0
,function(spec,packageName,urlSite){var self=this;
var cls=nil;
var className=nil;
if (!(spec.isString())){cls=spec.first();
if (cls.isString()){cls=smalltalk.Smalltalk.at_ifAbsent_(cls,(function(){return self.buildSubclass_in_reference_(cls,packageName,urlSite);}));} else{self.error_("Invalid superClass name: "._comma(cls));};;
return spec.copyFrom_to_((2),spec.size()).collect_((function(each){return cls.buildSubclass_in_reference_(each,packageName,urlSite);}));};;
className=spec;
cls=(((nil.isNil_(packageName)===false)).mustBeBoolean()==true ? (function(){return self.subclass_category_(className,packageName);})() : (function(){return self.subclass_(className);})());
if ((nil.isNil_(urlSite)).and_((function(){return (nil.isNil_(packageName)===false).and_((function(){return self.urlSelectors().includes_(packageName.upTo_("."));}));}))){urlSite=self.urlSelector();};;
if ((nil.isNil_(urlSite)===false).and_((function(){return urlSite._no_eq("none");}))){cls.comment_("See "._comma(self.referenceTo_in_at_(className,packageName,urlSite)));};;
if (smalltalk.responds_to_(cls,("packagePrefix:"))){cls.packagePrefix_(packageName);};;
return cls;}
,"builders",unescape("buildSubclass%3A%20spec%20in%3A%20packageName%20reference%3A%20urlSite%0D%0A%09%22%20Private%20-%20Generate%20a%20subclass%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20cls%20className%20%7C%0D%0A%09spec%20isString%20ifFalse%3A%20%5B%20%22spec%20can%20be%20%23%28superClassName%20className...%29%20%22%0D%0A%09%09cls%20%3A%3D%20spec%20first.%0D%0A%09%09cls%20isString%20ifTrue%3A%20%5B%0D%0A%09%09%09cls%20%3A%3D%20Smalltalk%20at%3A%20cls%20ifAbsent%3A%20%5B%0D%0A%09%09%09%09self%20buildSubclass%3A%20cls%20in%3A%20packageName%20reference%3A%20urlSite%0D%0A%09%09%09%5D%0D%0A%09%09%5D%20ifFalse%3A%20%5B%20self%20error%3A%20%27Invalid%20superClass%20name%3A%20%27%2Ccls%20%5D.%0D%0A%09%09%5E%28spec%20copyFrom%3A%202%20to%3A%20spec%20size%29%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09%09cls%20buildSubclass%3A%20each%20in%3A%20packageName%20reference%3A%20urlSite%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09className%20%3A%3D%20spec.%0D%0A%09cls%20%3A%3D%20packageName%20notNil%0D%0A%09%09ifTrue%3A%20%5B%20self%20subclass%3A%20className%20category%3A%20packageName%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20self%20subclass%3A%20className%20%5D.%0D%0A%09%28urlSite%20isNil%20and%3A%20%5BpackageName%20notNil%20and%3A%20%5B%0D%0A%09%09self%20urlSelectors%20includes%3A%20%28packageName%20upTo%3A%20%24.%29%0D%0A%09%09%5D%5D%29%20ifTrue%3A%20%5B%20urlSite%20%3A%3D%20self%20urlSelector%20%5D.%0D%0A%09%28urlSite%20notNil%20and%3A%20%5BurlSite%20%7E%3D%20%23none%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09cls%20comment%3A%20%27See%20%27%20%2C%28self%20referenceTo%3A%20className%20in%3A%20packageName%20at%3A%20urlSite%29%0D%0A%09%5D.%0D%0A%09%28cls%20respondsTo%3A%20%23packagePrefix%3A%29%20ifTrue%3A%20%5B%0D%0A%09%09cls%20packagePrefix%3A%20packageName%0D%0A%09%5D.%0D%0A%09%5Ecls"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildConstructors:",0
,function(selectors){var self=this;
self.build_doing_("Constructors"._minus_gt(selectors),(function(){return selectors.do_((function(each){return self.buildConstructor_(each);}));}));
return self;}
,"builders",unescape("buildConstructors%3A%20selectors%0D%0A%09%22%20Generate%20code%20for%20constructors%20implemented%20by%20native%20class.%20%22%0D%0A%0D%0A%09self%20build%3A%20%23Constructors%20-%3E%20selectors%20doing%3A%20%5B%0D%0A%09%09selectors%20do%3A%20%5B%3Aeach%7C%20self%20buildConstructor%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildConstructor:",0
,function(selector){var $s8Ret$={name:"stReturn"};try{var self=this;
if (selector.isArray()){if (selector.size()._eq((3)).and_((function(){return selector.last().isString();}))){return self.buildStaticFunction_(selector);};;
((selector.size()._eq((2)).and_((function(){return selector.last().isString();}))).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return self.buildStaticFunction_(selector)})();throw($s8Ret$)})();})() : nil);};;
(function(){$s8Ret$.result=(function(){return self.buildConstructorFor_(selector)})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"builders",unescape("buildConstructor%3A%20selector%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20constructor%20of%20native%20class.%20%22%0D%0A%0D%0A%09selector%20isArray%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20%28selector%20size%20%3D%203%20and%3A%20%5Bselector%20last%20isString%5D%29%20ifTrue%3A%20%5B%20%22%23%28%20selector%20%23%28%20argumentNames%20%29%20%23RETURNClass%20%29%22%0D%0A%09%09%5Eself%20buildStaticFunction%3A%20selector%0D%0A%09%20%20%20%20%5D.%20%0D%0A%09%20%20%20%20%28selector%20size%20%3D%202%20and%3A%20%5Bselector%20last%20isString%5D%29%20ifTrue%3A%20%5B%20%22%23%28%20selector%20%23RETURNClass%20%29%22%0D%0A%09%09%5Eself%20buildStaticFunction%3A%20selector%0D%0A%09%20%20%20%20%5D.%20%0D%0A%09%5D.%0D%0A%09%5Eself%20buildConstructorFor%3A%20selector"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildConstructorFor:",0
,function(selector){var self=this;
var args=nil;
if (selector.isArray()){return self.buildConstructor_arguments_(selector.first(),selector.last());};;
args=(1).to_(selector.occurrencesOf_(":")).collect_((function(i){return "arg"._comma(i.asString());}));
if (args.size()._eq((1))){args=["anObject"];};;
return self.buildConstructor_arguments_(selector,args);}
,"builders",unescape("buildConstructorFor%3A%20selector%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20constructor%20of%20native%20class.%20%22%0D%0A%0D%0A%09%7C%20args%20%7C%0D%0A%09selector%20isArray%20ifTrue%3A%20%5B%20%22%23%28%20selector%20%23%28argumentNames%29%20%29%22%0D%0A%09%09%5Eself%09buildConstructor%3A%20selector%20first%0D%0A%09%09%09arguments%3A%20selector%20last%0D%0A%09%5D.%0D%0A%09args%20%3A%3D%20%20%281%20to%3A%20%28selector%20occurrencesOf%3A%20%24%3A%29%29%20collect%3A%20%5B%3Ai%7C%20%23arg%20%2C%20i%20asString%20%5D.%0D%0A%09args%20size%20%3D%201%20ifTrue%3A%20%5B%20args%20%3A%3D%20%23%28%20%23anObject%20%29%20%5D.%0D%0A%09%5Eself%20buildConstructor%3A%20selector%20arguments%3A%20args"));
smalltalk.bind(smalltalk.NativeObject.$klass,"constructorSupport",0
,function(){var self=this;
return "self";}
,"builders",unescape("constructorSupport%0D%0A%09%22%20Private%20-%20Returns%20the%20instantiation%20support%20for%20constructor.%20%22%0D%0A%0D%0A%09%5E%23self"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildConstructor:arguments:",0
,function(aSelector,array){var self=this;
var selector=nil;
var args=nil;
var header=nil;
var code=nil;
var input=nil;
var argStream=nil;
var msg=nil;
var $$long=nil;
var adder=nil;
var comma=nil;
args=self.apiArgumentsFrom_(array);
selector=self.apiSelector_forArguments_(aSelector,args);
$$long=args.size()._gt((4));
input=selector.stream();
argStream=args.stream();
header="".writeStream();
code="".writeStream();
code.nextPutAll_((($$long).mustBeBoolean()==true ? (function(){return unescape("%23%28%29");})() : (function(){return "Array";})()));
adder=(($$long).mustBeBoolean()==true ? (function(){return " add: ";})() : (function(){return " with: ";})());
comma=(($$long).mustBeBoolean()==true ? (function(){return unescape("%3B");})() : (function(){return "";})());
header.nextPut_(input.next().asLowercase());
if (argStream.atEnd()){code.nextPutAll_(" new");};;
while(!(function(){return argStream.atEnd();})()){var param=nil;
var expr=nil;
if (argStream.peek().isString()){param=argStream.next();
expr=param;} else{param=argStream.peek().first();
expr=param._comma(" ")._comma(argStream.next().last());};;
msg=input.upTo_(":")._comma(": ");
code.nextPutAll_(adder._comma(expr)._comma(comma));
header.nextPutAll_(msg._comma(param)._comma(" "));};;
code.nextPutAll_((($$long).mustBeBoolean()==true ? (function(){return " yourself";})() : (function(){return "";})()));
msg=input.upToEnd();
code.nextPutAll_(msg);
header.nextPutAll_(msg);
self.$klass.implement_comment_as_category_(self.stMessageHeader_(header.contents()),"Returns an instance of the receiver.",unescape("%5E")._comma(self.constructorSupport())._comma(unescape("%20instantiate%3A%20self%20withAll%3A%20%28"))._comma(code.contents())._comma(unescape("%29")),"instantiation");
return self;}
,"builders",unescape("buildConstructor%3A%20aSelector%20arguments%3A%20array%0D%0A%09%22%20Private%20-%20Generate%20code%20for%20constructor%20of%20native%20class.%20%22%0D%0A%0D%0A%09%7C%20selector%20args%20header%20code%20input%20argStream%20msg%20long%20adder%20comma%20%7C%0D%0A%09args%20%3A%3D%20self%20apiArgumentsFrom%3A%20array.%0D%0A%09selector%20%3A%3D%20self%20apiSelector%3A%20aSelector%20forArguments%3A%20args.%0D%0A%09long%20%3A%3D%20args%20size%20%3E%204.%0D%0A%09input%20%3A%3D%20selector%20stream.%0D%0A%09argStream%20%3A%3D%20args%20stream.%0D%0A%09header%20%3A%3D%20%27%27%20writeStream.%20code%20%3A%3D%20%27%27%20writeStream.%0D%0A%09code%20nextPutAll%3A%20%28long%20ifTrue%3A%20%5B%20%27%23%28%29%27%20%5D%20ifFalse%3A%20%5B%20%23Array%20%5D%29.%0D%0A%09adder%20%3A%3D%20long%20ifTrue%3A%20%5B%27%20add%3A%20%27%5D%20ifFalse%3A%20%5B%27%20with%3A%20%27%5D.%0D%0A%09comma%20%3A%3D%20long%20ifTrue%3A%20%5B%24%3B%5D%20ifFalse%3A%20%5B%27%27%5D.%0D%0A%09header%20nextPut%3A%20input%20next%20asLowercase.%0D%0A%09argStream%20atEnd%20ifTrue%3A%20%5B%20code%20nextPutAll%3A%20%27%20new%27%20%5D.%0D%0A%09%5B%20argStream%20atEnd%20%5D%20whileFalse%3A%20%5B%20%7C%20param%20expr%20%7C%0D%0A%09%09argStream%20peek%20isString%0D%0A%09%09ifTrue%3A%20%5B%20param%20%3A%3D%20argStream%20next.%20expr%20%3A%3D%20param%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20param%20%3A%3D%20argStream%20peek%20first.%20expr%20%3A%3D%20param%20%2C%27%20%27%20%2CargStream%20next%20last.%20%5D.%0D%0A%09%09msg%20%3A%3D%20%28input%20upTo%3A%20%24%3A%29%20%2C%27%3A%20%27.%0D%0A%09%09code%20%20%20nextPutAll%3A%20adder%20%2Cexpr%20%2Ccomma.%0D%0A%09%09header%20nextPutAll%3A%20msg%20%2Cparam%20%2C%27%20%27.%0D%0A%09%5D.%0D%0A%09code%20nextPutAll%3A%20%28long%20ifTrue%3A%20%5B%20%27%20yourself%27%20%5D%20ifFalse%3A%20%5B%20%27%27%20%5D%29.%0D%0A%09msg%20%3A%3D%20input%20upToEnd.%0D%0A%09code%20nextPutAll%3A%20msg.%0D%0A%09header%20nextPutAll%3A%20msg.%0D%0A%09self%20class%0D%0A%09%09implement%3A%20%28self%20stMessageHeader%3A%20header%20contents%29%0D%0A%09%09%20%20comment%3A%20%27Returns%20an%20instance%20of%20the%20receiver.%27%0D%0A%09%09%20%20as%3A%20%24%5E%20%2Cself%20constructorSupport%20%2C%27%20instantiate%3A%20self%20withAll%3A%20%28%27%20%2Ccode%20contents%20%2C%27%29%27%0D%0A%09%09%20%20category%3A%20%23instantiation"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildStaticFunctions:",0
,function(specs){var self=this;
self.build_doing_("StaticFunctions"._minus_gt(specs),(function(){return specs.do_((function(each){return self.buildStaticFunction_(each);}));}));
return self;}
,"builders",unescape("buildStaticFunctions%3A%20specs%0D%0A%09%22%20Generate%20code%20from%20specs%20as%20functions%20implemented%20implemented%20by%20native%20class.%20%22%0D%0A%0D%0A%09self%20build%3A%20%23StaticFunctions%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09specs%20do%3A%20%5B%3Aeach%7C%20self%20buildStaticFunction%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildStaticFunction:",0
,function(spec){var $s8Ret$={name:"stReturn"};try{var self=this;
if (spec.isArray()){if (spec.size()._eq((3)).and_((function(){return spec.last().isString();}))){return self.buildStaticFunction_(smalltalk.Array.with_with_(smalltalk.Array.with_with_(spec.first(),spec.second()),spec.last()));};;
((spec.size()._eq((2)).and_((function(){return spec.last().isString();}))).mustBeBoolean()==true ? (function(){return (function(){$s8Ret$.result=(function(){return self.buildStaticFunction_prologue_epilogue_(spec.first(),self.apiPrologueFor_(spec.second()),self.apiEpilogueFor_(spec.second()))})();throw($s8Ret$)})();})() : nil);};;
(function(){$s8Ret$.result=(function(){return self.buildStaticFunction_prologue_epilogue_(spec,unescape("%5E"),"")})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"builders",unescape("buildStaticFunction%3A%20spec%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20delegate%20message%20implemented%20as%20functions%20on%20native%20class.%20%22%0D%0A%0D%0A%09spec%20isArray%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20%28spec%20size%20%3D%203%20and%3A%20%5Bspec%20last%20isString%5D%29%20ifTrue%3A%20%5B%20%22%23%28%20selector%20%23%28%20argumentNames%20%29%20%23RETURNClass%20%29%22%0D%0A%09%09%5Eself%20buildStaticFunction%3A%20%28Array%0D%0A%09%09%09with%3A%20%28Array%20with%3A%20spec%20first%20with%3A%20spec%20second%29%0D%0A%09%09%09with%3A%20spec%20last%29%0D%0A%09%20%20%20%20%5D.%20%0D%0A%09%20%20%20%20%28spec%20size%20%3D%202%20and%3A%20%5Bspec%20last%20isString%5D%29%20ifTrue%3A%20%5B%20%22%23%28%20selector%20%23RETURNClass%20%29%22%0D%0A%09%09%5Eself%0D%0A%09%09%09buildStaticFunction%3A%20spec%20first%0D%0A%09%09%09prologue%3A%20%28self%20apiPrologueFor%3A%20spec%20second%29%0D%0A%09%09%09epilogue%3A%20%28self%20apiEpilogueFor%3A%20spec%20second%29%0D%0A%09%20%20%20%20%5D.%0D%0A%09%5D.%0D%0A%09%5Eself%20buildStaticFunction%3A%20spec%20prologue%3A%20%24%5E%20epilogue%3A%20%27%27"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildStaticFunction:prologue:epilogue:",0
,function(spec,prologue,epilogue){var self=this;
var args=nil;
if (spec.isArray()){return self.buildStaticFunction_arguments_prologue_epilogue_(spec.first(),spec.last(),prologue,epilogue);};;
args=(1).to_(spec.occurrencesOf_(":")).collect_((function(i){return "arg"._comma(i.asString());}));
if (args.size()._eq((1))){args=["anObject"];};;
return self.buildStaticFunction_arguments_prologue_epilogue_(spec,args,prologue,epilogue);}
,"builders",unescape("buildStaticFunction%3A%20spec%20prologue%3A%20prologue%20epilogue%3A%20epilogue%20%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20delegate%20message%20on%20native%20class.%20%22%0D%0A%0D%0A%09%7C%20args%20%7C%0D%0A%09spec%20isArray%20ifTrue%3A%20%5B%20%22%23%28%20selector%20%23%28argumentNames%29%20%29%22%0D%0A%09%09%5Eself%09buildStaticFunction%3A%20spec%20first%0D%0A%09%09%09arguments%3A%20spec%20last%0D%0A%09%09%09prologue%3A%20prologue%20epilogue%3A%20epilogue%0D%0A%09%5D.%0D%0A%09args%20%3A%3D%20%20%281%20to%3A%20%28spec%20occurrencesOf%3A%20%24%3A%29%29%20collect%3A%20%5B%3Ai%7C%20%23arg%20%2C%20i%20asString%20%5D.%0D%0A%09args%20size%20%3D%201%20ifTrue%3A%20%5B%20args%20%3A%3D%20%23%28%20%23anObject%20%29%20%5D.%0D%0A%09%5Eself%0D%0A%09%09buildStaticFunction%3A%20spec%20arguments%3A%20args%0D%0A%09%09prologue%3A%20prologue%20epilogue%3A%20epilogue%20"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildStaticFunction:arguments:prologue:epilogue:",0
,function(spec,args,prologue,epilogue){var self=this;
var header=nil;
var code=nil;
if (args.includes_(self.kwargsStart())){var array=nil;
array=args.stream().upTo_(self.kwargsStart());
((args.last()._eq(self.kwargsStart())).mustBeBoolean()==true ? (function(){return self.buildStaticFunction_arguments_prologue_epilogue_(spec,array,prologue,epilogue);})() : (function(){array.add_(self.kwargsStart());return self.buildStaticFunction_arguments_prologue_epilogue_(spec,array,prologue,epilogue);})());};;
header="".writeStream();
code="".writeStream();
self.apiEncode_call_arguments_on_header_(self.codeForStaticReceiver(),spec,args,code,header);
self.$klass.implement_comment_as_category_(self.stMessageHeader_(header.contents()),"Delegated to native class.",prologue._comma(code.contents())._comma(epilogue).replace_with_(unescape("%21"),unescape("%21")._comma(unescape("%21"))),"wrappers");
return self;}
,"builders",unescape("buildStaticFunction%3A%20spec%20arguments%3A%20args%20prologue%3A%20prologue%20epilogue%3A%20epilogue%20%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20delegate%20message%20on%20native%20class.%20%22%0D%0A%0D%0A%09%7C%20header%20code%20%7C%0D%0A%09%28args%20includes%3A%20self%20kwargsStart%29%20ifTrue%3A%20%5B%20%7C%20array%20%7C%0D%0A%09%09array%20%3A%3D%20args%20stream%20upTo%3A%20self%20kwargsStart.%0D%0A%09%09args%20last%20%3D%20self%20kwargsStart%0D%0A%09%09ifTrue%3A%20%5B%20self%09buildStaticFunction%3A%20spec%20arguments%3A%20array%0D%0A%09%09%09%09prologue%3A%20prologue%20epilogue%3A%20epilogue%20%5D%0D%0A%09%09ifFalse%3A%20%5B%0D%0A%09%09%09array%20add%3A%20self%20kwargsStart.%0D%0A%09%09%09self%09buildStaticFunction%3A%20spec%20arguments%3A%20array%0D%0A%09%09%09%09prologue%3A%20prologue%20epilogue%3A%20epilogue%20%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09header%20%3A%3D%20%27%27%20writeStream.%20code%20%3A%3D%20%27%27%20writeStream.%0D%0A%09self%0D%0A%09%20%20apiEncode%3A%20self%20codeForStaticReceiver%0D%0A%09%09call%3A%20spec%20arguments%3A%20args%0D%0A%09%09on%3A%20code%20header%3A%20header.%0D%0A%09self%20class%0D%0A%09%09implement%3A%20%28self%20stMessageHeader%3A%20header%20contents%29%0D%0A%09%09%20%20comment%3A%20%27Delegated%20to%20native%20class.%27%0D%0A%09%09%20%20as%3A%09%28%28prologue%0D%0A%09%09%09%2C%20code%20contents%0D%0A%09%09%09%2C%20epilogue%29%20replace%3A%20%24%21%20with%3A%20%24%21%20%2C%24%21%29%0D%0A%09%09%20%20category%3A%20%23wrappers"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildNativeFunctions:",0
,function(specs){var self=this;
self.build_doing_("NativeFunctions"._minus_gt(specs),(function(){var tuples=nil;
tuples=specs.collect_((function(spec){return ((spec.isString()).mustBeBoolean()==true ? (function(){return "_"._comma(spec);})() : (function(){var selector=nil;
selector=((spec.first().isString()).mustBeBoolean()==true ? (function(){return "_"._comma(spec.first());})() : (function(){return (function($rec){$rec.at_put_((1),"_"._comma(spec.first().first()));return $rec.yourself();})(spec.first().copy());})());return smalltalk.Array.with_(selector)._comma(spec.copyFrom_to_((2),spec.size()));})());}));return tuples.do_((function(spec){return self.buildFunction_(spec);}));}));
return self;}
,"builders",unescape("buildNativeFunctions%3A%20specs%0D%0A%09%22%20Generate%20code%20from%20specs%20as%20raw%20functions%20implemented%20by%20instance%20handle.%20%22%0D%0A%0D%0A%09self%20build%3A%20%23NativeFunctions%20-%3E%20specs%20doing%3A%20%5B%20%7C%20tuples%20%7C%0D%0A%09%09tuples%20%3A%3D%20specs%20collect%3A%20%5B%3Aspec%7C%0D%0A%09%09%09spec%20isString%20ifTrue%3A%20%5B%20%24_%20%2Cspec%20%5D%20ifFalse%3A%20%5B%20%7C%20selector%20%7C%0D%0A%09%09%09%09selector%20%3A%3D%20spec%20first%20isString%20ifTrue%3A%20%5B%24_%20%2Cspec%20first%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09%09%09spec%20first%20copy%20at%3A%201%20put%3A%20%28%24_%20%2Cspec%20first%20first%29%3B%20yourself%0D%0A%09%09%09%09%5D.%0D%0A%09%09%09%09%28Array%20with%3A%20selector%29%2C%20%28spec%20copyFrom%3A%202%20to%3A%20spec%20size%29%0D%0A%09%09%09%5D%0D%0A%09%09%5D.%0D%0A%09%09tuples%20do%3A%20%5B%3Aspec%7C%20self%20buildFunction%3A%20spec%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildNativeStaticFunctions:",0
,function(specs){var self=this;
self.build_doing_("NativeStaticFunctions"._minus_gt(specs),(function(){var tuples=nil;
tuples=specs.collect_((function(spec){return ((spec.isString()).mustBeBoolean()==true ? (function(){return "_"._comma(spec);})() : (function(){var selector=nil;
selector=((spec.first().isString()).mustBeBoolean()==true ? (function(){return "_"._comma(spec.first());})() : (function(){return (function($rec){$rec.at_put_((1),"_"._comma(spec.first().first()));return $rec.yourself();})(spec.first().copy());})());return smalltalk.Array.with_(selector)._comma(spec.copyFrom_to_((2),spec.size()));})());}));return tuples.do_((function(spec){return self.buildStaticFunction_(spec);}));}));
return self;}
,"builders",unescape("buildNativeStaticFunctions%3A%20specs%0D%0A%09%22%20Generate%20code%20from%20specs%20as%20raw%20functions%20implemented%20by%20class%20handle.%20%22%0D%0A%0D%0A%09self%20build%3A%20%23NativeStaticFunctions%20-%3E%20specs%20doing%3A%20%5B%20%7C%20tuples%20%7C%0D%0A%09%09tuples%20%3A%3D%20specs%20collect%3A%20%5B%3Aspec%7C%0D%0A%09%09%09spec%20isString%20ifTrue%3A%20%5B%20%24_%20%2Cspec%20%5D%20ifFalse%3A%20%5B%20%7C%20selector%20%7C%0D%0A%09%09%09%09selector%20%3A%3D%20spec%20first%20isString%20ifTrue%3A%20%5B%24_%20%2Cspec%20first%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09%09%09spec%20first%20copy%20at%3A%201%20put%3A%20%28%24_%20%2Cspec%20first%20first%29%3B%20yourself%0D%0A%09%09%09%09%5D.%0D%0A%09%09%09%09%28Array%20with%3A%20selector%29%2C%20%28spec%20copyFrom%3A%202%20to%3A%20spec%20size%29%0D%0A%09%09%09%5D%0D%0A%09%09%5D.%0D%0A%09%09tuples%20do%3A%20%5B%3Aspec%7C%20self%20buildStaticFunction%3A%20spec%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildMethods:",0
,function(specs){var self=this;
self.build_doing_("Methods"._minus_gt(specs),(function(){return self.buildFunctions_(specs);}));
return self;}
,"builders",unescape("buildMethods%3A%20specs%0D%0A%09%22%20Generate%20code%20from%20specs%20as%20OO%20methods%20implemented%20by%20instance%20handle.%20%22%0D%0A%0D%0A%09self%20build%3A%20%23Methods%20-%3E%20specs%20doing%3A%20%5B%0D%0A%09%09self%20buildFunctions%3A%20specs%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"@","_at"
,function(handle){var self=this;
return self.basicNew().handle_(handle);}
,"instantiation",unescape("@%20handle%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicNew%20handle%3A%20handle"));
smalltalk.bind(smalltalk.NativeObject.$klass,"empty",0
,function(){var self=this;
return self._at(smalltalk.Object.$$new().json());}
,"instantiation",unescape("empty%0D%0A%09%22%20Return%20an%20empty%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20@%20Object%20new%20json"));
smalltalk.bind(smalltalk.NativeObject.$klass,"for:",0
,function(handle){var self=this;
return self.basicNew().handle_(handle);}
,"instantiation",unescape("for%3A%20handle%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20basicNew%20handle%3A%20handle"));
smalltalk.bind(smalltalk.NativeObject.$klass,"withAll:",0
,function(handles){var self=this;
return smalltalk.Array._at(handles).collect_((function(each){return self._at(each);}));}
,"instantiation",unescape("withAll%3A%20handles%0D%0A%09%22%20Return%20a%20collection%20of%20instance%20of%20the%20receiver%20with%20handles.%20%22%0D%0A%0D%0A%09%5EArray%20@%20handles%20collect%3A%20%5B%3Aeach%7C%20self%20@%20each%20%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"of:withAll:",0
,function(aType,handles){var self=this;
return self.withAll_(smalltalk.Array._at(handles));}
,"instantiation",unescape("of%3A%20aType%20withAll%3A%20handles%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20wrapping%20handles%20with%20a%20type%20%28or%20nil%29.%0D%0A%09Default%20implementation%20ignore%20the%20type.%0D%0A%09This%20method%20can%20be%20refined%20by%20subclasses%20to%20support%20typed%20wrappers.%0D%0A%09aType%20can%20be%20a%20class%20for%20type%20of%20elements%20or%20aString%20of%20comma%20separated%20type%20names.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20withAll%3A%20Array%20@%20handles"));
smalltalk.bind(smalltalk.NativeObject.$klass,"of:with:",0
,function(aType,aHandle){var self=this;
if ((nil.isNil_(aHandle))){return nil;};;
return self.basicOf_with_(aType,aHandle);}
,"instantiation",unescape("of%3A%20aType%20with%3A%20aHandle%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20wrapping%20aHandle%20with%20a%20type%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09aHandle%20isNil%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%5Eself%20basicOf%3A%20aType%20with%3A%20aHandle"));
smalltalk.bind(smalltalk.NativeObject.$klass,"basicOf:with:",0
,function(aType,aHandle){var self=this;
return self.implementedBySubclass_("basicOf:with:");}
,"instantiation",unescape("basicOf%3A%20aType%20with%3A%20aHandle%0D%0A%09%22%20Private%20-%20Return%20an%20instance%20of%20the%20receiver%20wrapping%20aHandle%20with%20a%20type.%0D%0A%09aType%20can%20be%20a%20class%20for%20type%20of%20element%20or%20aString%20of%20comma%20separated%20type%20names.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20implementedBySubclass%3A%20%23basicOf%3Awith%3A"));
smalltalk.bind(smalltalk.NativeObject.$klass,"html5Events",0
,function(){var self=this;
return [];}
,"events",unescape("html5Events%0D%0A%09%22%20Private%20-%20Return%20the%20html5%20events+notifications%20managed%20by%20instances%20of%20the%20receiver.%0D%0A%09Default%20answer%20is%20empty%2C%20this%20method%20can%20be%20refined%20by%20subclasses%20to%20handle%20html5%20events.%0D%0A%09%22%0D%0A%09%5E%23%28%29"));
smalltalk.bind(smalltalk.NativeObject.$klass,"localEvents",0
,function(){var self=this;
return [];}
,"events",unescape("localEvents%0D%0A%09%22%20Private%20-%20Return%20the%20smalltalk%20events%20managed%20by%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5E%23%28%29"));
smalltalk.bind(smalltalk.NativeObject.$klass,"constructEventsTriggered",0
,function(){var self=this;
return self.html5Events()._comma(self.localEvents())._comma(smalltalk.superImplementor(smalltalk.NativeObject.$klass,"constructEventsTriggered").apply(self, []));}
,"events",unescape("constructEventsTriggered%0D%0A%09%22%20Private%20-%20Return%20the%20events%20managed%20by%20instances%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20html5Events%0D%0A%09%2Cself%20localEvents%0D%0A%09%2Csuper%20constructEventsTriggered"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildHTML5Events:",0
,function(selectors){var self=this;
self.build_doing_("HTML5Events"._minus_gt(selectors),(function(){self.$klass.implement_comment_as_category_("html5Events",unescape("Private%20-%20Return%20the%20html5%20events%20managed%20by%20instances%20of%20the%20receiver."),unescape("%5E%23%28%20")._comma(selectors.inject_into_("",(function(total,each){return total._comma(unescape("%23"))._comma(each)._comma(" ");})))._comma(unescape("%29%20%2C%20super%20html5Events")),"events");return self.buildGetters_(selectors.collect_((function(each){return self.selectorForHTML5Event_(each);})));}));
return self;}
,"builder",unescape("buildHTML5Events%3A%20selectors%0D%0A%09%22%20Private%20-%20Generate%20code%20to%20route%20events%20to%20handle.%20%22%0D%0A%0D%0A%09self%20build%3A%20%23HTML5Events%20-%3E%20selectors%20doing%3A%20%5B%0D%0A%09%09self%20class%0D%0A%09%09%09implement%3A%20%23html5Events%0D%0A%09%09%09comment%3A%20%27Private%20-%20Return%20the%20html5%20events%20managed%20by%20instances%20of%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%27%5E%23%28%20%27%20%2C%28selectors%20inject%3A%20%27%27%20into%3A%20%5B%3Atotal%20%3Aeach%7C%0D%0A%09%09%09%09total%20%2C%27%23%27%2Ceach%20%2C%27%20%27%0D%0A%09%09%09%09%5D%29%2C%27%29%20%2C%20super%20html5Events%27%0D%0A%09%09%09category%3A%20%23events.%0D%0A%09%09self%20buildGetters%3A%20%28selectors%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09%09self%20selectorForHTML5Event%3A%20each%0D%0A%09%09%5D%29.%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"selectorForEvent:",0
,function(selector){var self=this;
if (self.localEvents().includes_(selector)){return nil;};;
return ((self.html5Events().includes_(selector)).mustBeBoolean()==true ? (function(){return self.selectorForHTML5Event_(selector);})() : nil);}
,"events",unescape("selectorForEvent%3A%20selector%0D%0A%09%22%20Private%20-%20Return%20the%20on...%20selector%20to%20use%20to%20register%20binding%20to%20selector.%20%22%0D%0A%0D%0A%09%28self%20localEvents%20includes%3A%20selector%29%20ifTrue%3A%20%5B%20%5Enil%20%5D.%0D%0A%09%5E%28self%20html5Events%20includes%3A%20selector%29%20ifTrue%3A%20%5B%0D%0A%09%09self%20selectorForHTML5Event%3A%20selector%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject.$klass,"selectorForHTML5Event:",0
,function(selector){var self=this;
return "on"._comma(selector.upTo_(":").asSmalltalkName_(true));}
,"events",unescape("selectorForHTML5Event%3A%20selector%0D%0A%09%22%20Private%20-%20Return%20the%20on...%20selector%20to%20use%20to%20register%20binding%20to%20selector.%20%22%0D%0A%0D%0A%09%5E%23on%20%2C%20%28%28selector%20upTo%3A%20%24%3A%29%20asSmalltalkName%3A%20true%29"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildLuaFunctions:",0
,function(specs){var self=this;
self.buildNativeFunctions_(specs);
return self;}
,"Lua-builders",unescape("buildLuaFunctions%3A%20specs%0D%0A%09%22%20OBSOLETE%20-%20Generate%20code%20from%20specs%20as%20raw%20functions%20implemented%20by%20instance%20handle.%20%22%0D%0A%0D%0A%09self%20buildNativeFunctions%3A%20specs"));
smalltalk.bind(smalltalk.NativeObject.$klass,"buildLuaStaticFunctions:",0
,function(specs){var self=this;
self.buildNativeStaticFunctions_(specs);
return self;}
,"Lua-builders",unescape("buildLuaStaticFunctions%3A%20specs%0D%0A%09%22%20OBSOLETE%20-%20Generate%20code%20from%20specs%20as%20raw%20functions%20implemented%20by%20class%20handle.%20%22%0D%0A%0D%0A%09self%20buildNativeStaticFunctions%3A%20specs"));
smalltalk.bind(smalltalk.NativeObject,"eval:",0
,function(code){var self=this;
return smalltalk.Smalltalk.current().eval_(code);}
,"eval",unescape("eval%3A%20code%0D%0A%09%22%20Private%20-%20Return%20the%20result%20of%20evaluating%20%28native%29%20code.%20%22%0D%0A%0D%0A%09%5ESmalltalk%20current%20eval%3A%20code"));
smalltalk.bind(smalltalk.NativeObject,"ignoreReturn:",0
,function(value){var self=this;
return self;}
,"api",unescape("ignoreReturn%3A%20value%0D%0A%09%22%20Private%20-%20Ignore%20the%20argument%20value.%0D%0A%09Return%20the%20receiver.%20%22"));
smalltalk.bind(smalltalk.NativeObject,"@@","_at_at"
,function(path){var self=this;
if (path.isString()){return self._at_at(path.asArrayOfSubstringsSeparatedBy_("."));};;
return path.inject_into_(self.handle(),(function(h,each){return (((nil.isNil_(h)===false)).mustBeBoolean()==true ? (function(){return ((function(){var $1$=(h[each]);if(nil.isNil_($1$))return nil;return $1$;})());})() : nil);}));}
,"accessing",unescape("@@%20path%0D%0A%09%22%20Return%20the%20property%20at%20path%20of%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09path%20isString%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20@@%20%28path%20asArrayOfSubstringsSeparatedBy%3A%20%24.%29%0D%0A%09%5D.%0D%0A%09%5Epath%20inject%3A%20self%20handle%20into%3A%20%5B%3Ah%20%3Aeach%7C%0D%0A%09%09h%20notNil%20ifTrue%3A%20%5B%20h%20basicAt%3A%20each%20%5D%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject,"<@","_lt_at"
,function(path){var self=this;
return smalltalk.NativeHandle._at(self._at_at(path));}
,"accessing",unescape("%3C@%20path%0D%0A%09%22%20Return%20the%20native%20property%20at%20path%20of%20the%20receiver%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5ENativeHandle%20@%20%28self%20@@%20path%29"));
smalltalk.bind(smalltalk.NativeObject,"as:",0
,function(aNativeObjectClass){var self=this;
var result=nil;
if (smalltalk.is_kindOf_(self,(aNativeObjectClass))){return self;};;
result=aNativeObjectClass._at(self.handle());
if ((nil.is_eqeq_(result,self))){return self;};;
result.copyPropertiesFrom_(self);
return result;}
,"converting",unescape("as%3A%20aNativeObjectClass%0D%0A%09%22%20Return%20the%20receiver%20casted%20to%20aNativeObjectClass.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09%28self%20isKindOf%3A%20aNativeObjectClass%29%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09result%20%3A%3D%20aNativeObjectClass%20@%20self%20handle.%0D%0A%09result%20%3D%3D%20self%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%09result%20copyPropertiesFrom%3A%20self.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeObject,"handle",0
,function(){var self=this;
if ((nil.isNil_(self["@handle"]))){self.initializeHandle();};;
return self["@handle"];}
,"accessing",unescape("handle%0D%0A%09%22%20Return%20the%20native%20handle.%20%22%0D%0A%0D%0A%09handle%20isNil%20ifTrue%3A%20%5B%20self%20initializeHandle%20%5D.%0D%0A%09%5Ehandle"));
smalltalk.bind(smalltalk.NativeObject,"handleOrNil",0
,function(){var self=this;
return self["@handle"];}
,"private",unescape("handleOrNil%0D%0A%09%22%20Return%20the%20native%20handle%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5Ehandle"));
smalltalk.bind(smalltalk.NativeObject,"=","_eq"
,function(anObject){var self=this;
if ((nil.is_eqeq_(self,anObject))){return true;};;
if (!(self.isObject_(anObject))){return false;};;
return smalltalk.responds_to_(anObject,("handleOrNil")).and_((function(){return (nil.is_eqeq_(self["@handle"],anObject.handleOrNil()));}));}
,"testing",unescape("%3D%20anObject%0D%0A%0D%0A%09self%20%3D%3D%20anObject%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%0D%0A%09%28self%20isObject%3A%20anObject%29%20ifFalse%3A%20%5B%20%5Efalse%20%5D.%0D%0A%09%5E%28anObject%20respondsTo%3A%20%23handleOrNil%29%20and%3A%20%5B%0D%0A%09%09handle%20%3D%3D%20anObject%20handleOrNil%0D%0A%09%5D%20"));
smalltalk.bind(smalltalk.NativeObject,"handle:",0
,function(anAlien){var self=this;
self["@handle"]=anAlien;
if ((nil.isNil_(self["@handle"]))){return nil;};;
return self;}
,"accessing",unescape("handle%3A%20anAlien%0D%0A%09%22%20Set%20the%20native%20handle%20to%20a%20potentialy%20native%28non-S8%29%20object.%0D%0A%09Return%20the%20receiver%2C%20or%20nil%20if%20the%20receiver%20is%20void.%0D%0A%09%22%0D%0A%0D%0A%09handle%20%3A%3D%20anAlien.%0D%0A%09handle%20isNil%20ifTrue%3A%20%5B%20%5Enil%20%5D."));
smalltalk.bind(smalltalk.NativeObject,"jsonHiddenNames",0
,function(){var self=this;
return ["handle"];}
,"converting",unescape("jsonHiddenNames%0D%0A%09%22%20Private%20-%20Return%20the%20names%20of%20the%20instance%20variables%20to%20skip%20when%20mapping%20the%20receiver%20to%20JSON%20structure.%20%22%0D%0A%0D%0A%09%5E%23%28%20%23handle%20%29"));
smalltalk.bind(smalltalk.NativeObject,"basicJson",0
,function(){var self=this;
return smalltalk.superImplementor(smalltalk.NativeObject,"json").apply(self, []);}
,"converting",unescape("basicJson%0D%0A%09%22%20Convert%20the%20receiver%20to%20JSON%20structure.%20%22%0D%0A%0D%0A%09%5Esuper%20json"));
smalltalk.bind(smalltalk.NativeObject,"json",0
,function(){var self=this;
return self.handle();}
,"converting",unescape("json%0D%0A%09%22%20Convert%20the%20receiver%20to%20JSON%20structure.%20%22%0D%0A%0D%0A%09%5Eself%20handle"));
smalltalk.bind(smalltalk.NativeObject,"jsObject",0
,function(){var self=this;
return Object;}
,"private",unescape("jsObject%0D%0A%09%22%20Private%20-%20Return%20the%20%28native%29Object.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%7D"));
smalltalk.bind(smalltalk.NativeObject,"ownPropertyNames",0
,function(){var self=this;
return Object.getOwnPropertyNames(self["@handle"]);
return self;}
,"accessing",unescape("ownPropertyNames%0D%0A%09%22%20Return%20the%20owned%20property%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%7D"));
smalltalk.bind(smalltalk.NativeObject,"@","_at"
,function(propertyName){var self=this;
return self.handleAt_(propertyName);}
,"accessing",unescape("@%20propertyName%0D%0A%09%22%20Return%20the%20property%20at%20propertyName%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20handleAt%3A%20propertyName"));
smalltalk.bind(smalltalk.NativeObject,"handleAt:",0
,function(propertyName){var self=this;
return self["@handle"][propertyName];}
,"accessing",unescape("handleAt%3A%20propertyName%0D%0A%09%22%20Return%20the%20property%20at%20propertyName%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ehandle%5BpropertyName%5D"));
smalltalk.bind(smalltalk.NativeObject,"handleAt:put:",0
,function(propertyName,anObject){var self=this;
return self["@handle"][propertyName]=anObject;}
,"accessing",unescape("handleAt%3A%20propertyName%20put%3A%20anObject%0D%0A%09%22%20Set%20the%20property%20at%20propertyName%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Ehandle%5BpropertyName%5D%20%3A%3D%20anObject"));
smalltalk.bind(smalltalk.NativeObject,"initializeHandle",0
,function(){var self=this;
return self.error_("Missing "._comma(self.$klass.name())._comma(unescape("%3E%3E%23initializeHandle")));}
,"initialize",unescape("initializeHandle%0D%0A%09%22%20Private%20-%20Initialize%20the%20native%20handle.%0D%0A%09This%20method%20must%20be%20implemented%20by%20subclasses.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20error%3A%20%27Missing%20%27%0D%0A%09%09%2Cself%20class%20name%0D%0A%09%09%2C%27%3E%3E%23initializeHandle%27"));
smalltalk.bind(smalltalk.NativeObject,"releaseHandle",0
,function(){var self=this;
self["@handle"]=nil;
return self;}
,"release",unescape("releaseHandle%0D%0A%09%22%20Private%20-%20Release%20the%20native%20handle.%0D%0A%09The%20default%20implementation%20clear%20handle.%0D%0A%09This%20method%20can%20be%20refined%20by%20subclasses%20to%20free%20external%20resources.%0D%0A%09%22%0D%0A%0D%0A%09handle%20%3A%3D%20nil"));
smalltalk.bind(smalltalk.NativeObject,"release",0
,function(){var self=this;
self.releaseHandle();
return smalltalk.superImplementor(smalltalk.NativeObject,"release").apply(self, []);}
,"release",unescape("release%0D%0A%09%22%20Private%20-%20Release%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20releaseHandle.%0D%0A%09%5Esuper%20release"));
smalltalk.bind(smalltalk.NativeObject,"copyPropertiesFrom:",0
,function(aNativeObject){var self=this;
aNativeObject.ownPropertyNames().do_((function(key){return (self[key]=(((function(){var $1$=(aNativeObject[key]);if(nil.isNil_($1$))return nil;return $1$;})())));}));
return self;}
,"copying",unescape("copyPropertiesFrom%3A%20aNativeObject%0D%0A%09%22%20Private%20-%20Copy%20receiver%27s%20properties%20from%20aNativeObject.%20%22%0D%0A%0D%0A%09aNativeObject%20ownPropertyNames%20do%3A%20%5B%3Akey%7C%0D%0A%09%09self%20basicAt%3A%20key%20put%3A%20%28aNativeObject%20basicAt%3A%20key%29%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject,"shallowCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("shallowCopy%0D%0A%09%22%20Return%20a%20shallow%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.NativeObject,"deepCopy",0
,function(){var self=this;
return self;}
,"copying",unescape("deepCopy%0D%0A%09%22%20Return%20a%20deep%20copy%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself"));
smalltalk.bind(smalltalk.NativeObject,"closureForEvent:",0
,function(selector){var self=this;
var argumentCount=nil;
argumentCount=selector.argumentCount();
if (argumentCount._eq((0))){return (function(){return self.triggerEvent_(selector);});};;
if (argumentCount._eq((1))){return (function(arg1){return self.triggerEvent_with_(selector,arg1);});};;
if (argumentCount._eq((2))){return (function(arg1,arg2){return self.triggerEvent_with_with_(selector,arg1,arg2);});};;
if (argumentCount._eq((3))){return (function(arg1,arg2,arg3){return self.triggerEvent_withArguments_(selector,smalltalk.Array.with_with_with_(arg1,arg2,arg3));});};;
if (argumentCount._eq((4))){return (function(arg1,arg2,arg3,arg4){return self.triggerEvent_withArguments_(selector,smalltalk.Array.with_with_with_with_(arg1,arg2,arg3,arg4));});};;
return self.error_("Invalid argument count");}
,"events",unescape("closureForEvent%3A%20selector%0D%0A%09%22%20Private%20-%20Return%20the%20closure%20to%20handle%20event%20selector.%20%22%0D%0A%0D%0A%09%7C%20argumentCount%20%7C%0D%0A%09argumentCount%20%3A%3D%20selector%20argumentCount.%0D%0A%09argumentCount%20%3D%200%20ifTrue%3A%20%5B%20%5E%5B%20self%20triggerEvent%3A%20selector%20%5D%20%5D.%0D%0A%09argumentCount%20%3D%201%20ifTrue%3A%20%5B%20%5E%5B%3Aarg1%7C%20self%20triggerEvent%3A%20selector%20with%3A%20arg1%20%5D%20%5D.%0D%0A%09argumentCount%20%3D%202%20ifTrue%3A%20%5B%20%5E%5B%3Aarg1%20%3Aarg2%7C%20self%20triggerEvent%3A%20selector%20with%3A%20arg1%20with%3A%20arg2%20%5D%20%5D.%0D%0A%09argumentCount%20%3D%203%20ifTrue%3A%20%5B%20%5E%5B%3Aarg1%20%3Aarg2%20%3Aarg3%7C%20self%20triggerEvent%3A%20selector%20withArguments%3A%20%28Array%20with%3A%20arg1%20with%3A%20arg2%20with%3A%20arg3%29%20%5D%20%5D.%0D%0A%09argumentCount%20%3D%204%20ifTrue%3A%20%5B%20%5E%5B%3Aarg1%20%3Aarg2%20%3Aarg3%20%3Aarg4%7C%20self%20triggerEvent%3A%20selector%20withArguments%3A%20%28Array%20with%3A%20arg1%20with%3A%20arg2%20with%3A%20arg3%20with%3A%20arg4%29%20%5D%20%5D.%0D%0A%09%5Eself%20error%3A%20%27Invalid%20argument%20count%27"));
smalltalk.bind(smalltalk.NativeObject,"when:evaluate:",0
,function(selector,anAction){var self=this;
var result=nil;
var needsRegistration=nil;
needsRegistration=(nil.isNil_(self.eventTable().at_ifAbsent_(selector,(function(){return nil;}))));
result=smalltalk.superImplementor(smalltalk.NativeObject,"when_evaluate_").apply(self, [selector,anAction]);
if (needsRegistration){self.registerEvent_doing_(selector,self.closureForEvent_(selector));};;
return result;}
,"events",unescape("when%3A%20selector%20evaluate%3A%20anAction%0D%0A%09%22%20Register%20anAction%20to%20the%20be%20evaluated%20when%20the%20receiver%20triggers%20the%20event.%20%22%0D%0A%0D%0A%09%7C%20result%20needsRegistration%20%7C%0D%0A%09needsRegistration%20%3A%3D%20%28self%20eventTable%20at%3A%20selector%20ifAbsent%3A%20%5Bnil%5D%29%20isNil.%0D%0A%09result%20%3A%3D%20super%20when%3A%20selector%20evaluate%3A%20anAction.%0D%0A%09needsRegistration%20ifTrue%3A%20%5B%0D%0A%09%09self%09registerEvent%3A%20selector%0D%0A%09%09%09doing%3A%20%28self%20closureForEvent%3A%20selector%29.%0D%0A%09%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeObject,"registerEvent:doing:",0
,function(selector,closure){var self=this;
var onSelector=nil;
onSelector=self.$klass.selectorForEvent_(selector);
if ((nil.isNil_(onSelector))){return self;};;
return self.bindEvent_doing_(onSelector,closure);}
,"events",unescape("registerEvent%3A%20selector%20doing%3A%20closure%0D%0A%09%22%20Private%20-%20Register%20basic%20event%20handler.%20%22%0D%0A%0D%0A%09%7C%20onSelector%20%7C%0D%0A%09onSelector%20%3A%3D%20self%20class%20selectorForEvent%3A%20selector.%0D%0A%09onSelector%20isNil%20ifTrue%3A%20%5B%20%5Eself%20%5D.%20%22local/smalltalk%20event%22%0D%0A%09%5Eself%20bindEvent%3A%20onSelector%20doing%3A%20closure"));
smalltalk.bind(smalltalk.NativeObject,"bindEvent:doing:",0
,function(onSelector,closure){var self=this;
var $$native=nil;
$$native=self.handle();
if ((nil.isNil_($$native[onSelector]))){$$native[onSelector]=closure;} else{$$native[onSelector].addListener(closure.json());};;
return self;}
,"events",unescape("bindEvent%3A%20onSelector%20doing%3A%20closure%0D%0A%09%22%20Private%20-%20Bind%20the%20event%20handler.%20%22%0D%0A%0D%0A%09%7C%20native%20%7C%0D%0A%09native%20%3A%3D%20self%20handle.%0D%0A%09native%5BonSelector%5D%20isNil%20ifTrue%3A%20%5B%0D%0A%09%09native%5BonSelector%5D%20%3A%3D%20closure%0D%0A%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09native%5BonSelector%5D%20%23addListener%3A%20closure%20json%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeObject,"description",0
,function(){var self=this;
return self.asNativeHandle().description();}
,"debugging",unescape("description%0D%0A%09%22%20Return%20a%20string%20description%20from%20receiver%27s%20properties.%20%22%0D%0A%0D%0A%09%5Eself%20asNativeHandle%20description"));
smalltalk.bind(smalltalk.NativeObject,"asDictionary",0
,function(){var self=this;
var result=nil;
result=smalltalk.PoolDictionary.$$new();
self.ownPropertyNames().do_((function(key){return result.at_put_(key,self["@handle"][key]);}));
return result;}
,"converting",unescape("asDictionary%0D%0A%09%22%20Convert%20the%20receiver%20to%20a%28Pool%29Dictionary.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20PoolDictionary%20new.%0D%0A%09self%20ownPropertyNames%20do%3A%20%5B%3Akey%7C%20result%20at%3A%20key%20put%3A%20handle%5Bkey%5D%20%5D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeObject,"asNativeHandle",0
,function(){var self=this;
return smalltalk.NativeHandle._at(self.handle());}
,"converting",unescape("asNativeHandle%0D%0A%09%22%20Convert%20the%20receiver%20to%20NativeHandle%20instance%20%28for%20reflection%29.%20%22%0D%0A%0D%0A%09%5ENativeHandle%20@%20self%20handle"));
smalltalk.bind(smalltalk.NativeObject,"asJS",0
,function(){var self=this;
return smalltalk.NativeHandle._at(self.handle());}
,"converting",unescape("asJS%0D%0A%09%22%20Convert%20the%20receiver%20to%20JS%20instance%20%28for%20reflection%29.%20%22%0D%0A%0D%0A%09%5ENativeHandle%20@%20self%20handle"));
smalltalk.bind(smalltalk.NativeHandle.$klass,"supportsLanguage:",0
,function(language){var self=this;
return false;}
,"instantiation",unescape("supportsLanguage%3A%20language%0D%0A%09%22%20Private%20-%20Return%20true%20if%20instances%20of%20the%20receiver%20implement%20native%20wrappers%20for%20language.%0D%0A%09Returns%20false.%20This%20method%20may%20be%20refined%20by%20subclasses.%20%22%0D%0A%0D%0A%09%5Efalse"));
smalltalk.bind(smalltalk.NativeHandle.$klass,"support",0
,function(){var $s8Ret$={name:"stReturn"};try{var self=this;
var lang=nil;
if ((nil.isNil_(smalltalk.NativeHandle.classVariableAt_("Support"))===false)){return smalltalk.NativeHandle.classVariableAt_("Support");};;
lang=self.nativeLanguage();
smalltalk.NativeHandle.allSubclasses().do_((function(cls){return ((cls.supportsLanguage_(lang)).mustBeBoolean()==true ? (function(){smalltalk.NativeHandle.classVariableAt_put_("Support",cls);return (function(){$s8Ret$.result=(function(){return smalltalk.NativeHandle.classVariableAt_("Support")})();throw($s8Ret$)})();})() : nil);}));
(function(){$s8Ret$.result=(function(){return self.error_("Native handle support missing")})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"instantiation",unescape("support%0D%0A%09%22%20Private%20-%20Return%20the%20support%20for%20hosting%20execution%20environment.%20%22%0D%0A%0D%0A%09%7C%20lang%20%7C%0D%0A%09Support%20notNil%20ifTrue%3A%20%5B%20%5ESupport%20%5D.%0D%0A%09lang%20%3A%3D%20self%20nativeLanguage.%0D%0A%09NativeHandle%20allSubclasses%20do%3A%20%5B%3Acls%7C%0D%0A%09%09%28cls%20supportsLanguage%3A%20lang%29%20ifTrue%3A%20%5B%0D%0A%09%09%09Support%20%3A%3D%20cls.%0D%0A%09%09%09%5ESupport%0D%0A%09%09%5D.%0D%0A%09%5D.%0D%0A%09%5Eself%20error%3A%20%27Native%20handle%20support%20missing%27"));
smalltalk.bind(smalltalk.NativeHandle.$klass,"@>","_at_gt"
,function(nativeObject){var self=this;
var h=nil;
h=self._at(nativeObject);
return (((nil.isNil_(h)===false)).mustBeBoolean()==true ? (function(){return h.description();})() : (function(){return "nil";})());}
,"converting",unescape("@%3E%20nativeObject%0D%0A%09%22%20Return%20the%20description%20of%20nativeObject.%20%22%0D%0A%0D%0A%09%7C%20h%20%7C%0D%0A%09h%20%3A%3D%20self%20@%20nativeObject.%0D%0A%09%5Eh%20notNil%0D%0A%09%09ifTrue%3A%20%5B%20h%20description%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20%23nil%20%5D"));
smalltalk.bind(smalltalk.NativeHandle.$klass,"instanceHandleFromString:",0
,function(aString){var self=this;
return smalltalk.Smalltalk.nativeObjectAt_ifAbsent_(aString,(function(){return nil;}));}
,"private",unescape("instanceHandleFromString%3A%20aString%0D%0A%09%22%20Private%20-%20Return%20a%20%28native%29handle%20obtained%20from%20aString%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%5ESmalltalk%20nativeObjectAt%3A%20aString%20ifAbsent%3A%20%5B%5D"));
smalltalk.bind(smalltalk.NativeHandle.$klass,"@","_at"
,function(nativeObject){var self=this;
if ((nil.is_eqeq_(self,smalltalk.NativeHandle))){return self.support()._at(nativeObject);};;
if (self.isObject_(nativeObject).and_((function(){return nativeObject.isString();}))){return smalltalk.superImplementor(smalltalk.NativeHandle.$klass,"_at").apply(self, [self.instanceHandleFromString_(nativeObject)]);};;
return smalltalk.superImplementor(smalltalk.NativeHandle.$klass,"_at").apply(self, [nativeObject]);}
,"instantiation",unescape("@%20nativeObject%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20%3D%3D%20NativeHandle%20ifTrue%3A%20%5B%20%5Eself%20support%20@%20nativeObject%20%5D.%0D%0A%09%28%28self%20isObject%3A%20nativeObject%29%20and%3A%20%5B%20nativeObject%20isString%20%5D%29%20ifTrue%3A%20%5B%0D%0A%09%09%5Esuper%20@%20%28self%20instanceHandleFromString%3A%20nativeObject%29%0D%0A%09%5D.%0D%0A%09%5Esuper%20@%20nativeObject"));
smalltalk.bind(smalltalk.NativeHandle.$klass,"chunks",0
,function(){var self=this;
return smalltalk.NativeReader.$$new();}
,"utilities",unescape("chunks%0D%0A%09%22%20Return%20a%20NativeReader%20to%20fileIn%20properties%20in%20new%20native%20object.%20%22%0D%0A%0D%0A%09%5ENativeReader%20new"));
smalltalk.bind(smalltalk.NativeHandle.$klass,"@>>","_at_gt_gt"
,function(nativeObject){var self=this;
var h=nil;
h=self._at(nativeObject);
return (((nil.isNil_(h)===false)).mustBeBoolean()==true ? (function(){return h.description_owned_((1),true);})() : (function(){return "nil";})());}
,"converting",unescape("@%3E%3E%20nativeObject%0D%0A%09%22%20Return%20the%20%28owned%29%20description%20of%20nativeObject.%20%22%0D%0A%0D%0A%09%7C%20h%20%7C%0D%0A%09h%20%3A%3D%20self%20@%20nativeObject.%0D%0A%09%5Eh%20notNil%0D%0A%09%09ifTrue%3A%20%5B%20h%20description%3A%201%20owned%3A%20true%20%5D%0D%0A%09%09ifFalse%3A%20%5B%20%23nil%20%5D"));
smalltalk.bind(smalltalk.NativeHandle,"keys",0
,function(){var self=this;
return self.propertyNames().sorted();}
,"accessing",unescape("keys%0D%0A%09%22%20Return%20the%20sorted%20property%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20propertyNames%20sorted"));
smalltalk.bind(smalltalk.NativeHandle,"properties",0
,function(){var self=this;
return self.keys().collect_((function(each){return smalltalk.Array.with_with_(each,self.handleAt_(each));}));}
,"accessing",unescape("properties%0D%0A%09%22%20Return%20an%20array%20with%20property%20name%20and%20values.%20%22%0D%0A%0D%0A%09%5Eself%20keys%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09Array%20with%3A%20each%20with%3A%20%28self%20handleAt%3A%20each%29%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeHandle,"propertiesDo:",0
,function(aBlock){var self=this;
self.propertyNames().do_((function(each){return aBlock.value_value_(each,self.handleAt_(each));}));
return self;}
,"iterating",unescape("propertiesDo%3A%20aBlock%0D%0A%09%22%20Evaluate%20aTwoArgumentBlock%20with%20each%20property%20%28name%20%26%20value%29%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09self%20propertyNames%20do%3A%20%5B%3Aeach%7C%20aBlock%20value%3A%20each%20value%3A%20%28self%20handleAt%3A%20each%29%20%5D"));
smalltalk.bind(smalltalk.NativeHandle,"description:",0
,function(level){var self=this;
return self.description_owned_(level,false);}
,"debugging",unescape("description%3A%20level%0D%0A%09%22%20Return%20the%20description%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20description%3A%20level%20owned%3A%20false%20"));
smalltalk.bind(smalltalk.NativeHandle,"description:owned:",0
,function(level,owned){var self=this;
var text=nil;
var data=nil;
var cr=nil;
text=(function(h){return (((nil.isNil_(h))).mustBeBoolean()==true ? nil : (function(){return ((self.isObject_(h)).mustBeBoolean()==true ? (function(){return (((nil.is_eqeq_(self.typeOf_(h),"function"))).mustBeBoolean()==true ? (function(){return unescape("%28")._comma(h.toString().upTo_(unescape("%7B")))._comma(unescape("%7B...%7D%29"));})() : (function(){return h.printString();})());})() : (function(){return (function(){return ((level._gt((1))).mustBeBoolean()==true ? (function(){return self.$klass._at(h).description_owned_(level._minus((1)),owned);})() : (function(){return self.toString_(h);})());}).on_do_(smalltalk.Exception,(function(err){return unescape("%3F%3F%3F");}));})());})());});
data=(function(key){var value=nil;
(function(){return value=((function(){var $1$=(self["@handle"][key]);if(nil.isNil_($1$))return nil;return $1$;})());}).on_do_(smalltalk.Exception,(function(ex){return value=unescape("%5B")._comma(key)._comma(unescape("%5D%20"))._comma(ex.description());}));return value;});
cr=unescape("%0D%0A");
return ((owned).mustBeBoolean()==true ? (function(){return self.ownPropertyNames();})() : (function(){return self.propertyNames();})()).sorted().inject_into_(unescape("%7B"),(function(total,each){return total._comma(cr)._comma(each)._comma(": ")._comma(text.value_(data.value_(each)))._comma(unescape("%3B"));}))._comma(cr)._comma(unescape("%7D"));}
,"debugging",unescape("description%3A%20level%20owned%3A%20owned%0D%0A%09%22%20Return%20the%20description%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7C%20text%20data%20cr%20%7C%20%0D%0A%09text%20%3A%3D%20%5B%3Ah%7C%0D%0A%09%09h%20isNil%20ifFalse%3A%20%5B%0D%0A%09%09%09%28self%20isObject%3A%20h%29%20ifTrue%3A%20%5B%0D%0A%09%09%09%09%28self%20typeOf%3A%20h%29%20%3D%3D%20%23function%0D%0A%09%09%09%09ifTrue%3A%20%5B%20%24%28%20%2C%28h%20toString%20upTo%3A%20%24%7B%29%20%2C%27%7B...%7D%29%27%20%5D%0D%0A%09%09%09%09ifFalse%3A%20%5B%20h%20printString%20%5D%0D%0A%09%09%09%5D%20ifFalse%3A%20%5B%0D%0A%09%09%09%09%5B%20level%20%3E%201%0D%0A%09%09%09%09%09ifTrue%3A%20%5B%20%28self%20class%20@%20h%29%20description%3A%20level%20-%201%20owned%3A%20owned%20%5D%0D%0A%09%09%09%09%09ifFalse%3A%20%5B%20self%20toString%3A%20h%20%5D%0D%0A%09%09%09%09%5D%20on%3A%20Exception%20do%3A%20%5B%3Aerr%7C%20%27%3F%3F%3F%27%20%5D%0D%0A%09%09%09%5D%0D%0A%09%09%5D%0D%0A%09%5D.%0D%0A%09data%20%3A%3D%20%5B%3Akey%7C%20%7C%20value%20%7C%0D%0A%09%09%5B%20value%20%3A%3D%20handle%20basicAt%3A%20key%20%5D%20on%3A%20Exception%20do%3A%20%5B%3Aex%7C%0D%0A%09%09%09value%20%3A%3D%20%27%5B%27%2Ckey%2C%27%5D%20%27%20%2Cex%20description%0D%0A%09%09%5D.%20value%0D%0A%09%5D.%0D%0A%09cr%20%3A%3D%20%27%0D%0A%27.%09%5E%28%28owned%0D%0A%09ifTrue%3A%20%5Bself%20ownPropertyNames%5D%0D%0A%09ifFalse%3A%20%5Bself%20propertyNames%5D%29%20sorted%0D%0A%09%09inject%3A%20%24%7B%20into%3A%20%5B%3Atotal%20%3Aeach%7C%20total%20%2Ccr%0D%0A%20%09%09%09%2Ceach%20%2C%27%3A%20%27%0D%0A%09%09%09%2C%28text%20value%3A%20%28data%20value%3Aeach%29%29%0D%0A%09%09%09%2C%27%3B%27%20%5D%0D%0A%09%09%29%09%2Ccr%20%2C%24%7D"));
smalltalk.bind(smalltalk.NativeHandle,"description",0
,function(){var self=this;
return self.description_((1));}
,"debugging",unescape("description%0D%0A%09%22%20Return%20the%20description%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20description%3A%201"));
smalltalk.bind(smalltalk.NativeHandle,"isFunction:",0
,function(aHandle){var self=this;
return (nil.is_eqeq_(self.typeOf_(aHandle),"function"));}
,"private",unescape("isFunction%3A%20aHandle%0D%0A%09%22%20Return%20true%20if%20aHandle%20is%20a%20function.%20%22%0D%0A%0D%0A%09%5E%28self%20typeOf%3A%20aHandle%29%20%3D%3D%20%23function"));
smalltalk.bind(smalltalk.NativeHandle,"functionNames",0
,function(){var self=this;
return self.propertyNames().select_((function(each){return self.isFunction_(self.handleAt_(each));}));}
,"accessing",unescape("functionNames%0D%0A%09%22%20Return%20the%20names%20of%20property%20of%20the%20receiver%20of%20type%20function.%20%22%0D%0A%0D%0A%09%5Eself%20propertyNames%20select%3A%20%5B%3Aeach%7C%20self%20isFunction%3A%20%28self%20handleAt%3A%20each%29%20%5D"));
smalltalk.bind(smalltalk.NativeHandle,"functions",0
,function(){var self=this;
return self.functionNames().collect_((function(each){return smalltalk.Array.with_with_(each,self.handleAt_(each));}));}
,"accessing",unescape("functions%0D%0A%09%22%20Return%20the%20functions%20of%20the%20receiver%20as%20a%20collection%20of%20tuples%20%23%28functionName%20aClosure%29.%20%22%0D%0A%0D%0A%09%5Eself%20functionNames%20collect%3A%20%5B%3Aeach%7C%0D%0A%09%09Array%20with%3A%20each%20with%3A%20%28self%20handleAt%3A%20each%29%0D%0A%09%5D"));
smalltalk.bind(smalltalk.NativeHandle,"ownFunctionNames",0
,function(){var self=this;
return self.ownPropertyNames().select_((function(each){return self.isFunction_(self.handleAt_(each));}));}
,"accessing",unescape("ownFunctionNames%0D%0A%09%22%20Return%20the%20names%20of%20functions%20owned/implemented%20by%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20ownPropertyNames%20select%3A%20%5B%3Aeach%7C%20self%20isFunction%3A%20%28self%20handleAt%3A%20each%29%20%5D"));
smalltalk.bind(smalltalk.NativeHandle,"hasOwnProperty:",0
,function(aName){var self=this;
return Object.prototype.hasOwnProperty.call(self["@handle"],aName);}
,"testing",unescape("hasOwnProperty%3A%20aName%0D%0A%09%22%20Return%20true%20if%20receiver%20own%20a%20property%20named%20aName.%20%22%0D%0A%0D%0A%09%5E%7B%27%27%7D"));
smalltalk.bind(smalltalk.NativeHandle,"delete:",0
,function(aName){var self=this;
delete self["@handle"][aName];
return self;}
,"actions",unescape("delete%3A%20aName%0D%0A%09%22%20Delete%20the%20property%20named%20aName%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27%7D"));
smalltalk.bind(smalltalk.JS.$klass,"supportsLanguage:",0
,function(language){var self=this;
return language._eq("js");}
,"instantiation",unescape("supportsLanguage%3A%20language%0D%0A%09%22%20Private%20-%20Return%20true%20if%20instances%20of%20the%20receiver%20implement%20native%20wrappers%20for%20language.%20%22%0D%0A%0D%0A%09%5Elanguage%20%3D%20%23js"));
smalltalk.bind(smalltalk.JS,"propertyNames",0
,function(){var self=this;
return Object.keys(self.handle());
return self;}
,"accessing",unescape("propertyNames%0D%0A%09%22%20Return%20the%20property%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27js%27return%20Object.keys%28self.handle%28%29%29%27%7D"));
smalltalk.bind(smalltalk.JS,"prototype",0
,function(){var self=this;
return smalltalk.JS._at(Object.getPrototypeOf(self["@handle"]));
return self;}
,"accessing",unescape("prototype%0D%0A%09%22%20Return%20the%20prototype%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27js%27return%20smalltalk.JS._at%28Object.getPrototypeOf%28self%5B%22@handle%22%5D%29%29%27%7D"));
smalltalk.bind(smalltalk.JS,"functionNamesAndArguments",0
,function(){var self=this;
var argList=nil;
argList=(function(source){return (function($rec){$rec.upTo_(unescape("%28"));return $rec.upTo_(unescape("%29"));})(source).asArrayOfSubstringsSeparatedBy_(unescape("%2C")).collect_((function(arg){return arg.trimBlanks();}));});
return self.functions().collect_((function(tuple){return smalltalk.Array.with_with_(tuple.first(),argList.value_(tuple.last().toString().stream()));}));}
,"accessing",unescape("functionNamesAndArguments%0D%0A%09%22%20Return%20the%20functions%20of%20the%20receiver%20as%20a%20collection%20of%20tuples%20%23%28functionName%20argumentsArray%29.%20%22%0D%0A%0D%0A%09%7C%20argList%20%7C%0D%0A%09argList%20%3A%3D%20%5B%3Asource%7C%0D%0A%09%09%28%28source%20upTo%3A%20%24%28%3B%20upTo%3A%20%24%29%29%20asArrayOfSubstringsSeparatedBy%3A%20%24%2C%29%0D%0A%09%09%09collect%3A%20%5B%3Aarg%7C%20arg%20trimBlanks%20%5D%0D%0A%09%5D.%0D%0A%09%5Eself%20functions%20collect%3A%20%5B%3Atuple%7C%20Array%20with%3A%20tuple%20first%20with%3A%20%28argList%20value%3A%20tuple%20last%20toString%20stream%29%20%5D"));
smalltalk.bind(smalltalk.JS,"defineProperty:value:writable:enumerable:configurable:",0
,function(aName,aValue,writable,enumerable,configurable){var self=this;
return self.defineProperty_as_(aName,smalltalk.Array.with_with_with_with_(smalltalk.Array.with_with_("value",aValue),smalltalk.Array.with_with_("writable",writable),smalltalk.Array.with_with_("enumerable",enumerable),smalltalk.Array.with_with_("configurable",configurable)));}
,"properties",unescape("defineProperty%3A%20aName%0D%0A%09value%3A%20aValue%0D%0A%09writable%3A%20writable%0D%0A%09enumerable%3A%20enumerable%0D%0A%09configurable%3A%20configurable%0D%0A%09%22%20Define%20the%20property%20named%20aName%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20defineProperty%3A%20aName%20as%3A%20%28%20Array%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23value%20with%3A%20aValue%29%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23writable%20with%3A%20writable%29%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23enumerable%20with%3A%20enumerable%29%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23configurable%20with%3A%20configurable%29%0D%0A%09%29"));
smalltalk.bind(smalltalk.JS,"defineAccessor:get:set:enumerable:configurable:",0
,function(aName,aBlock,oneArgumentBlock,enumerable,configurable){var self=this;
return self.defineProperty_as_(aName,smalltalk.Array.with_with_with_with_(smalltalk.Array.with_with_("get",aBlock),smalltalk.Array.with_with_("set",oneArgumentBlock),smalltalk.Array.with_with_("enumerable",enumerable),smalltalk.Array.with_with_("configurable",configurable)));}
,"properties",unescape("defineAccessor%3A%20aName%0D%0A%09get%3A%20aBlock%0D%0A%09set%3A%20oneArgumentBlock%0D%0A%09enumerable%3A%20enumerable%0D%0A%09configurable%3A%20configurable%0D%0A%09%22%20Define%20the%20Accessor%20property%20named%20aName%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20defineProperty%3A%20aName%20as%3A%20%28%20Array%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23get%20with%3A%20aBlock%29%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23set%20with%3A%20oneArgumentBlock%29%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23enumerable%20with%3A%20enumerable%29%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23configurable%20with%3A%20configurable%29%0D%0A%09%29"));
smalltalk.bind(smalltalk.JS,"defineProperty:value:writable:enumerable:",0
,function(aName,aValue,writable,enumerable){var self=this;
return self.defineProperty_as_(aName,smalltalk.Array.with_with_with_(smalltalk.Array.with_with_("value",aValue),smalltalk.Array.with_with_("writable",writable),smalltalk.Array.with_with_("enumerable",enumerable)));}
,"properties",unescape("defineProperty%3A%20aName%0D%0A%09value%3A%20aValue%0D%0A%09writable%3A%20writable%0D%0A%09enumerable%3A%20enumerable%0D%0A%09%22%20Define%20the%20property%20named%20aName%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20defineProperty%3A%20aName%20as%3A%20%28%20Array%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23value%20with%3A%20aValue%29%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23writable%20with%3A%20writable%29%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23enumerable%20with%3A%20enumerable%29%0D%0A%09%29"));
smalltalk.bind(smalltalk.JS,"defineProperty:value:writable:",0
,function(aName,aValue,writable){var self=this;
return self.defineProperty_as_(aName,smalltalk.Array.with_with_(smalltalk.Array.with_with_("value",aValue),smalltalk.Array.with_with_("writable",writable)));}
,"properties",unescape("defineProperty%3A%20aName%0D%0A%09value%3A%20aValue%0D%0A%09writable%3A%20writable%0D%0A%09%22%20Define%20the%20property%20named%20aName%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20defineProperty%3A%20aName%20as%3A%20%28%20Array%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23value%20with%3A%20aValue%29%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23writable%20with%3A%20writable%29%0D%0A%09%29"));
smalltalk.bind(smalltalk.JS,"defineProperty:value:",0
,function(aName,aValue){var self=this;
return self.defineProperty_as_(aName,smalltalk.Array.with_(smalltalk.Array.with_with_("value",aValue)));}
,"properties",unescape("defineProperty%3A%20aName%20value%3A%20aValue%0D%0A%09%22%20Define%20the%20property%20named%20aName%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20defineProperty%3A%20aName%20as%3A%20%28%20Array%0D%0A%09%09with%3A%20%28Array%20with%3A%20%23value%20with%3A%20aValue%29%0D%0A%09%29"));
smalltalk.bind(smalltalk.JS,"defineProperty:",0
,function(aName){var self=this;
return self.defineProperty_as_(aName,smalltalk.Object.$$new());}
,"properties",unescape("defineProperty%3A%20aName%0D%0A%09%22%20Define%20the%20property%20named%20aName%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20defineProperty%3A%20aName%20as%3A%20Object%20new"));
smalltalk.bind(smalltalk.JS,"defineProperty:as:",0
,function(aName,descriptor){var self=this;
if (descriptor.isArray()){return self.defineProperty_as_(aName,descriptor.inject_into_(smalltalk.Object.$$new(),(function(result,tuple){if ((nil.isNil_(tuple.last())===false)){(result[tuple.first()]=(tuple.last()));};;return result;})));};;
return Object.defineProperty(self["@handle"],aName,descriptor);
return self;}
,"properties",unescape("defineProperty%3A%20aName%20as%3A%20descriptor%0D%0A%09%22%20Define%20the%20property%20named%20aName%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09descriptor%20isArray%20ifTrue%3A%20%5B%0D%0A%09%09%5Eself%20defineProperty%3A%20aName%20as%3A%20%28descriptor%0D%0A%09%09%09inject%3A%20Object%20new%20into%3A%20%5B%3Aresult%20%3Atuple%7C%0D%0A%09%09%09%09tuple%20last%20notNil%20ifTrue%3A%20%5B%0D%0A%09%09%09%09%09result%20basicAt%3A%20tuple%20first%20put%3A%20tuple%20last%0D%0A%09%09%09%09%5D.%0D%0A%09%09%09%09result%0D%0A%09%09%09%5D%29%0D%0A%09%5D.%0D%0A%0D%0A%09%7B%27%27js%27return%20Object.defineProperty%28self%5B%22@handle%22%5D%2CaName%2Cdescriptor%29%27%7D"));
smalltalk.bind(smalltalk.JS,"defineProperties:",0
,function(descriptors){var self=this;
return Object.defineProperties(self["@handle"],descriptors);
return self;}
,"properties",unescape("defineProperties%3A%20descriptors%0D%0A%09%22%20Define%20the%20properties%20in%20the%20receiver.%20%22%0D%0A%0D%0A%09%7B%27%27js%27return%20Object.defineProperties%28self%5B%22@handle%22%5D%2Cdescriptors%29%27%7D"));
smalltalk.bind(smalltalk.JS,"preventExtensions",0
,function(){var self=this;
return Object.preventExtensions(self["@handle"]);
return self;}
,"actions",unescape("preventExtensions%0D%0A%09%22%20Set%20the%20receiver%20as%20non-extensible.%20%22%0D%0A%0D%0A%09%7B%27%27js%27return%20Object.preventExtensions%28self%5B%22@handle%22%5D%29%27%7D"));
smalltalk.bind(smalltalk.JS,"freeze",0
,function(){var self=this;
return Object.freeze(self["@handle"]);
return self;}
,"actions",unescape("freeze%0D%0A%09%22%20Prevents%20the%20modification%20of%20existing%20property%20attributes%20and%20values%2C%20and%20prevents%20the%20addition%20of%20new%20properties.%20%22%0D%0A%0D%0A%09%7B%27%27js%27return%20Object.freeze%28self%5B%22@handle%22%5D%29%27%7D"));
smalltalk.bind(smalltalk.JS,"seal",0
,function(){var self=this;
return Object.seal(self["@handle"]);
return self;}
,"actions",unescape("seal%0D%0A%09%22%20Prevents%20the%20modification%20of%20attributes%20of%20existing%20properties%2C%20and%20prevents%20the%20addition%20of%20new%20properties.%20%22%0D%0A%0D%0A%09%7B%27%27js%27return%20Object.seal%28self%5B%22@handle%22%5D%29%27%7D"));
smalltalk.bind(smalltalk.JS,"isExtensible",0
,function(){var self=this;
return Object.isExtensible(self["@handle"]);
return self;}
,"testing",unescape("isExtensible%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20extensible.%20%22%0D%0A%0D%0A%09%7B%27%27js%27return%20Object.isExtensible%28self%5B%22@handle%22%5D%29%27%7D"));
smalltalk.bind(smalltalk.JS,"isFrozen",0
,function(){var self=this;
return Object.isFrozen(self["@handle"]);
return self;}
,"testing",unescape("isFrozen%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20frozen.%20%22%0D%0A%0D%0A%09%7B%27%27js%27return%20Object.isFrozen%28self%5B%22@handle%22%5D%29%27%7D"));
smalltalk.bind(smalltalk.JS,"isSealed",0
,function(){var self=this;
return Object.isSealed(self["@handle"]);
return self;}
,"testing",unescape("isSealed%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20sealed.%20%22%0D%0A%0D%0A%09%7B%27%27js%27return%20Object.isSealed%28self%5B%22@handle%22%5D%29%27%7D"));
smalltalk.bind(smalltalk.JS,"create:with:",0
,function(prototype,descriptors){var self=this;
return smalltalk.JS._at(Object.create(prototype,descriptors));
return self;}
,"creation",unescape("create%3A%20prototype%20with%3A%20descriptors%0D%0A%09%22%20Return%20a%20JS%20object%20that%20has%20been%20created%20with%20the%20specified%20prototype%2C%20and%20that%20optionally%20contains%20specified%20properties.%20%22%0D%0A%0D%0A%09%7B%27%27js%27return%20smalltalk.JS._at%28Object.create%28prototype%2Cdescriptors%29%29%27%7D"));
smalltalk.bind(smalltalk.JS,"createWith:",0
,function(descriptors){var self=this;
return self.create_with_(self["@handle"],descriptors);}
,"creation",unescape("createWith%3A%20descriptors%0D%0A%09%22%20Return%20a%20JS%20object%20that%20has%20been%20created%20with%20the%20handle%20prototype%2C%20and%20that%20optionally%20contains%20specified%20properties.%20%22%0D%0A%0D%0A%09%5Eself%20create%3A%20handle%20with%3A%20descriptors"));
smalltalk.bind(smalltalk.Lua.$klass,"supportsLanguage:",0
,function(language){var self=this;
return language._eq("lua");}
,"instantiation",unescape("supportsLanguage%3A%20language%0D%0A%09%22%20Private%20-%20Return%20true%20if%20instances%20of%20the%20receiver%20implement%20native%20wrappers%20for%20language.%20%22%0D%0A%0D%0A%09%5Elanguage%20%3D%20%23lua"));
smalltalk.bind(smalltalk.Lua.$klass,"global",0
,function(){var self=this;
return self._at(smalltalk.Smalltalk.global());}
,"accessing",unescape("global%0D%0A%09%22%20Return%20an%20instance%20wrapping%20global.%20%22%0D%0A%0D%0A%09%5Eself%20@%20Smalltalk%20global"));
smalltalk.bind(smalltalk.Lua.$klass,"version",0
,function(){var self=this;
return self.global()._at("_VERSION");}
,"accessing",unescape("version%0D%0A%09%22%20Return%20the%20host%20version.%20%22%0D%0A%0D%0A%09%5Eself%20global%20@%20%23_VERSION"));
smalltalk.bind(smalltalk.Lua.$klass,"arg",0
,function(){var self=this;
return smalltalk.Array._at(self.global()._at("arg"));}
,"accessing",unescape("arg%0D%0A%09%22%20Return%20the%20main%20arguments%20%28anArray%20of%20strings%29.%20%22%0D%0A%0D%0A%09%5EArray%20@%20%28self%20global%20@%20%23arg%29"));
smalltalk.bind(smalltalk.Lua.$klass,"require:",0
,function(aName){var self=this;
return self._at(require(aName));}
,"modules",unescape("require%3A%20aName%0D%0A%09%22%20Return%20an%20instance%20of%20the%20receiver%20obtained%20from%20loading%20module%20aName.%20%28or%20nil%29%20%22%0D%0A%0D%0A%09%5Eself%20@%20%28%23%20%23require%3A%20aName%29"));
smalltalk.bind(smalltalk.Lua,"propertyNames",0
,function(){var self=this;
return self.ownPropertyNames();}
,"accessing",unescape("propertyNames%0D%0A%09%22%20Return%20the%20property%20names%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20ownPropertyNames"));
smalltalk.bind(smalltalk.Lua,"do:",0
,function(aBlock){var self=this;
	self.invalidMessage_("do:");
return self;}
,"iteration",unescape("do%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20on%20each%20element%20of%20the%20receiver.%20%22%0D%0A%0D%0A%7B%27%27js%27%09self.invalidMessage_%28%22do%3A%22%29%27%0D%0Alua%27%09for%20item%20in%20self%3Ahandle%28%29%20do%20aBlock%28item%29%20end%27%7D."));
smalltalk.bind(smalltalk.Lua,"pairsDo:",0
,function(aBlock){var self=this;
	self.invalidMessage_("pairsDo:");
return self;}
,"iteration",unescape("pairsDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20on%20each%20key/value%20pair%20of%20the%20receiver.%20%22%0D%0A%0D%0A%7B%27%27js%27%09self.invalidMessage_%28%22pairsDo%3A%22%29%27%0D%0Alua%27%09for%20k%2Cv%20in%20pairs%28self%3Ahandle%28%29%29%20do%20aBlock%28k%2Cv%29%20end%27%7D."));
smalltalk.bind(smalltalk.Lua,"keysDo:",0
,function(aBlock){var self=this;
	self.invalidMessage_("keysDo:");
return self;}
,"iteration",unescape("keysDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20on%20each%20key%20of%20the%20receiver.%20%22%0D%0A%0D%0A%7B%27%27js%27%09self.invalidMessage_%28%22keysDo%3A%22%29%27%0D%0Alua%27%09for%20k%2C_%20in%20pairs%28self%3Ahandle%28%29%29%20do%20aBlock%28k%29%20end%27%7D."));
smalltalk.bind(smalltalk.Lua,"valuesDo:",0
,function(aBlock){var self=this;
	self.invalidMessage_("valuesDo:");
return self;}
,"iteration",unescape("valuesDo%3A%20aBlock%0D%0A%09%22%20Evaluates%20aBlock%20on%20each%20value%20of%20the%20receiver.%20%22%0D%0A%0D%0A%7B%27%27js%27%09self.invalidMessage_%28%22valuesDo%3A%22%29%27%0D%0Alua%27%09for%20_%2Cv%20in%20pairs%28self%3Ahandle%28%29%29%20do%20aBlock%28k%2Cv%29%20end%27%7D."));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"globalName",0
,function(){var self=this;
return self.subclassResponsibility_("globalName");}
,"private",unescape("globalName%0D%0A%09%22%20Private%20-%20Return%20the%20name%20of%20the%20global%20to%20detect/access%20the%20library%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23globalName"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"topGlobal",0
,function(){var self=this;
return smalltalk.Smalltalk.nativeObjectAt_ifAbsent_("window",(function(){return self.error_("Missing window support");}));}
,"private",unescape("topGlobal%0D%0A%09%22%20Private%20-%20Return%20the%20object%20to%20access%20the%20global%20during%20detection%20stage.%0D%0A%09The%20default%20implementation%20return%20window%20global%20%28present%20when%20running%20in%20a%20web%20browser%29.%0D%0A%09This%20method%20must%20be%20refined%20by%20the%20subclasses%20requiring%20other%20root%20dependences.%0D%0A%09Can%20return%20nil%20to%20do%20not%20check%20dependents.%0D%0A%09%22%0D%0A%0D%0A%09%5ESmalltalk%20nativeObjectAt%3A%20%23window%20ifAbsent%3A%20%5B%20self%20error%3A%20%27Missing%20window%20support%27%20%5D."));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"requiredFeatures",0
,function(){var self=this;
return [];}
,"private",unescape("requiredFeatures%0D%0A%09%22%20Private%20-%20Return%20the%20names%20of%20the%20required%20properties/features%20to%20execute%20the%20receiver.%0D%0A%09The%20default%20implementation%20do%20not%20check%20dependents.%0D%0A%09This%20method%20can%20be%20refined%20by%20subclasses%20to%20ensure%20features%20to%20run%20the%20receiver%20are%20present%20in%20this%20system.%0D%0A%09%22%0D%0A%0D%0A%09%5E%23%28%29"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"classNamed:",0
,function(className){var self=this;
var library=nil;
library=self.library();
return library[className];}
,"private",unescape("classNamed%3A%20className%0D%0A%09%22%20Private%20-%20Return%20the%20class%20named%20className%20in%20the%20receiver%27s%20library.%0D%0A%09The%20default%20implementation%20get%20the%20class%20from%20library.%0D%0A%09Subclasses%20can%20refine%20to%20obtain%20classes%20from%20anywhere%0D%0A%09%09e.g.%20global%20scope%20with%3A%20%5Eself%20global%3A%20className.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20library%20%7C%0D%0A%09library%20%3A%3D%20self%20library.%0D%0A%09%5Elibrary%5BclassName%5D"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"sourceCode",0
,function(){var self=this;
return self.subclassResponsibility_("sourceCode");}
,"private",unescape("sourceCode%0D%0A%09%22%20Private%20-%20Return%20the%20%28native%29%20source%20code%20of%20the%20library%20as%20a%20String.%0D%0A%09This%20method%20can%20be%20implemented%20by%20subclasses%20loading%20sources%20from%20external%20media%20or%20simply%20returning%20aString%20embedded%20in%20this%20method.%0D%0A%09A%20reference/link%20to%20source%20origin%20is%20appreciated%20by%20users.%0D%0A%09CAUTION%3A%20S8%20is%20MIT%2C%20so%20you%20MUST%20embedd%20here%20MIT%20compatible%20sources.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20subclassResponsibility%3A%20%23sourceCode"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"libraryBound:",0
,function(aClassName){var self=this;
self.bindingListeners().do_((function(each){return each.libraryBound_(aClassName);}));
return self;}
,"binding",unescape("libraryBound%3A%20aClassName%0D%0A%09%22%20Private%20-%20The%20library%20of%20aClassName%20has%20just%20been%20bound%20to%20this%20system.%20%22%0D%0A%0D%0A%09self%20bindingListeners%20do%3A%20%5B%3Aeach%7C%20each%20libraryBound%3A%20aClassName%20%5D"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"bindingListeners",0
,function(){var self=this;
if ((nil.isNil_(smalltalk.NativeLibraryObject.classVariableAt_("BindingListeners")))){smalltalk.NativeLibraryObject.classVariableAt_put_("BindingListeners",smalltalk.Array.$$new());};;
return smalltalk.NativeLibraryObject.classVariableAt_("BindingListeners");}
,"binding",unescape("bindingListeners%0D%0A%09%22%20Private%20-%20Return%20the%20collection%20of%20listeners%20to%20library%20binding%20notifications%20of%20this%20system.%20%22%0D%0A%0D%0A%09BindingListeners%20isNil%20ifTrue%3A%20%5B%20BindingListeners%20%3A%3D%20Array%20new%20%5D.%0D%0A%09%5EBindingListeners"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"globalName:",0
,function(aName){var self=this;
return self.$klass.implement_comment_as_category_("globalName",unescape("Private%20-%20Return%20the%20name%20of%20the%20global%20to%20detect/access%20the%20library%20of%20the%20receiver."),unescape("%5E")._comma(aName.printString()),"private");}
,"builder",unescape("globalName%3A%20aName%0D%0A%09%22%20Generate%20code%20to%20return%20the%20name%20of%20the%20global%20to%20detect/access%20the%20library%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20class%0D%0A%09%09implement%3A%20%23globalName%0D%0A%09%09comment%3A%20%27Private%20-%20Return%20the%20name%20of%20the%20global%20to%20detect/access%20the%20library%20of%20the%20receiver.%27%0D%0A%09%09as%3A%20%27%5E%27%20%2CaName%20printString%0D%0A%09%09category%3A%20%23private"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"libraryURL:",0
,function(anURL){var self=this;
return self.$klass.implement_comment_as_category_("libraryURL",unescape("Private%20-%20Return%20the%20URL%20to%20use%20when%20loading%20library%20of%20the%20receiver."),unescape("%5E")._comma(anURL.printString()),"private");}
,"builder",unescape("libraryURL%3A%20anURL%0D%0A%09%22%20Generate%20the%20method%20to%20return%20libraryURL.%20%22%0D%0A%0D%0A%09%5Eself%20class%0D%0A%09%09implement%3A%20%23libraryURL%0D%0A%09%09comment%3A%20%27Private%20-%20Return%20the%20URL%20to%20use%20when%20loading%20library%20of%20the%20receiver.%27%0D%0A%09%09as%3A%20%27%5E%27%20%2CanURL%20printString%0D%0A%09%09category%3A%20%23private"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"globalClassNameFrom:",0
,function(globalClassName){var self=this;
return globalClassName;}
,"private",unescape("globalClassNameFrom%3A%20globalClassName%0D%0A%09%22%20Private%20-%20Return%20the%20global%20class%20name%20for%20global%20globalClassName.%0D%0A%09It%20must%20return%20the%20%28global%29%20name%20of%20the%20constructor%20used%20to%20instantiate%20instance%20handles.%0D%0A%09The%20default%20action%20is%20to%20return%20the%20globalClassName.%0D%0A%09This%20method%20can%20be%20refined%20by%20subclasses%20to%20adapt%20name%20to%20%28case%20sensitive%29%20library%20constructor%20names.%0D%0A%09%22%0D%0A%0D%0A%09%5EglobalClassName"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"selectorForConstantNamed:",0
,function(aName){var $s8Ret$={name:"stReturn"};try{var self=this;
aName.detect_ifNone_((function(c){return c.isLowercase();}),(function(){return (function(){$s8Ret$.result=(function(){return aName.asLowercase().asSmalltalkName_(false)})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return aName.asSmalltalkName_(false)})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"builder",unescape("selectorForConstantNamed%3A%20aName%0D%0A%09%22%20Private%20-%20Return%20the%20selector%20to%20use%20for%20constant%20aName.%20%22%0D%0A%0D%0A%09aName%20detect%3A%20%5B%3Ac%7C%20c%20isLowercase%20%5D%20ifNone%3A%20%5B%0D%0A%09%09%5EaName%20asLowercase%20asSmalltalkName%3A%20false%0D%0A%09%5D.%0D%0A%09%5EaName%20asSmalltalkName%3A%20false"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"constants:",0
,function(names){var self=this;
names.do_((function(each){return self.$klass.implement_comment_as_category_(self.selectorForConstantNamed_(each),"Return the value of "._comma(each),unescape("%5Eself%20library%20basicAt%3A%20%23")._comma(each),"accessing");}));
return self;}
,"builder",unescape("constants%3A%20names%0D%0A%09%22%20Private%20-%20Build%20methods%20to%20access%20library%20globals.%20%22%0D%0A%0D%0A%09names%20do%3A%20%5B%3Aeach%7C%0D%0A%09%09self%20class%0D%0A%09%09%09implement%3A%20%28self%20selectorForConstantNamed%3A%20each%29%0D%0A%09%09%09comment%3A%20%27Return%20the%20value%20of%20%27%2Ceach%0D%0A%09%09%09as%3A%20%27%5Eself%20library%20basicAt%3A%20%23%27%2Ceach%0D%0A%09%09%09category%3A%20%23accessing%0D%0A%09%5D."));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"buildSubclass:arguments:typed:",0
,function(globalClassName,argList,typedArguments){var self=this;
var cls=nil;
var clsName=nil;
clsName=globalClassName.first().asUppercase()._comma(globalClassName.copyFrom_to_((2),globalClassName.size()));
clsName=self.classToken()._comma(clsName);
self.subclass_instanceVariableNames_category_(clsName,"",self.category());
if ((nil.isNil_(argList)).or_((function(){return argList._eq("nil");}))){return self;};;
cls=smalltalk.Smalltalk.at_(clsName);
cls.$klass.implement_comment_as_category_("globalClassName",unescape("Private%20-%20Return%20the%20internal%20class%20name."),unescape("%5Eself%20globalClassNameFrom%3A%20%23")._comma(globalClassName),"private");
if (argList.isEmpty()){cls.implement_comment_as_category_("initializeHandle",unescape("Private%20-%20Initialize%20the%20receiver%27s%20handle."),unescape("self%20handle%3A%20%28self%20newHandleOf%3A%20self%20class%20globalClassName%29"),"initialize");} else{var selector=nil;
var args=nil;
var current=nil;
selector=argList.inject_into_("",(function(total,each){return total._comma(each)._comma(": ")._comma(each)._comma(" ");}));
args=argList.inject_into_("",(function(total,each){if (typedArguments._eq("all").or_((function(){return typedArguments.includes_(each);}))){current=" handle ";} else{current=" ";};;return total._comma("with: ")._comma(each)._comma(current);}));
cls.$klass.implement_comment_as_category_(selector,"Return an instance of the receiver.",unescape("%5Eself%20basicNew%20")._comma(selector),"instantiation");
cls.implement_comment_as_category_(selector,unescape("Private%20-%20Initialize%20the%20receiver."),unescape("self%20handle%3A%20%28self%20newHandleOf%3A%20self%20class%20globalClassName%20")._comma(args)._comma(unescape("%29")),"initialize");};;
return self;}
,"builder",unescape("buildSubclass%3A%20globalClassName%20arguments%3A%20argList%20typed%3A%20typedArguments%0D%0A%09%22%20Private%20-%20Build%20a%20subclass%20of%20the%20receiver%20implementing%20a%20wrapper%20to%20the%20library.%20%22%0D%0A%0D%0A%09%7C%20cls%20clsName%20%7C%0D%0A%09clsName%20%3A%3D%20globalClassName%20first%20asUppercase%0D%0A%09%09%09%2C%28globalClassName%20copyFrom%3A%202%20to%3A%20globalClassName%20size%29.%0D%0A%09clsName%20%3A%3D%20self%20classToken%20%2CclsName.%0D%0A%09self%0D%0A%09%09subclass%3A%20clsName%0D%0A%09%09instanceVariableNames%3A%20%27%27%0D%0A%09%09category%3A%20self%20category.%0D%0A%09%28argList%20isNil%20or%3A%20%5B%20argList%20%3D%20%23nil%20%5D%29%20ifTrue%3A%20%5B%20%5Eself%20%5D.%0D%0A%0D%0A%09cls%20%3A%3D%20Smalltalk%20at%3A%20clsName.%0D%0A%09cls%20class%0D%0A%09%09implement%3A%20%23globalClassName%0D%0A%09%09comment%3A%20%27Private%20-%20Return%20the%20internal%20class%20name.%27%0D%0A%09%09as%3A%20%27%5Eself%20globalClassNameFrom%3A%20%23%27%20%2CglobalClassName%0D%0A%09%09category%3A%20%23private.%0D%0A%09argList%20isEmpty%20ifTrue%3A%20%5B%0D%0A%09%09cls%0D%0A%09%09%09implement%3A%20%23initializeHandle%0D%0A%09%09%09comment%3A%20%27Private%20-%20Initialize%20the%20receiver%27%27s%20handle.%27%0D%0A%09%09%09as%3A%20%27self%20handle%3A%20%28self%20newHandleOf%3A%20self%20class%20globalClassName%29%27%0D%0A%09%09%09category%3A%20%23initialize.%0D%0A%09%5D%20ifFalse%3A%20%5B%20%7C%20selector%20args%20current%20%7C%0D%0A%09%09selector%20%3A%3D%20argList%20inject%3A%20%27%27%20into%3A%20%5B%3Atotal%20%3Aeach%7C%20total%20%2Ceach%2C%27%3A%20%27%2Ceach%2C%27%20%27%20%5D.%0D%0A%09%09args%20%3A%3D%20argList%20inject%3A%20%27%27%20into%3A%20%5B%3Atotal%20%3Aeach%7C%0D%0A%09%09%09%28typedArguments%20%3D%20%23all%0D%0A%09%09%09%20%20or%3A%20%5B%20typedArguments%20includes%3A%20each%20%5D%29%0D%0A%09%09%09%20%20ifTrue%3A%20%5B%20current%20%3A%3D%20%27%20handle%20%27%20%5D%0D%0A%09%09%09%20%20ifFalse%3A%20%5B%20current%20%3A%3D%20%27%20%27%20%5D.%0D%0A%09%09%09total%20%2C%27with%3A%20%27%2Ceach%2C%20current%0D%0A%09%09%5D.%0D%0A%09%09cls%20class%0D%0A%09%09%09implement%3A%20selector%0D%0A%09%09%09comment%3A%20%27Return%20an%20instance%20of%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%27%5Eself%20basicNew%20%27%2Cselector%0D%0A%09%09%09category%3A%20%23instantiation.%0D%0A%09%09cls%0D%0A%09%09%09implement%3A%20selector%0D%0A%09%09%09comment%3A%20%27Private%20-%20Initialize%20the%20receiver.%27%0D%0A%09%09%09as%3A%20%27self%20handle%3A%20%28self%20newHandleOf%3A%20self%20class%20globalClassName%20%27%20%2Cargs%20%2C%27%29%27%0D%0A%09%09%09category%3A%20%23initialize.%0D%0A%09%5D."));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"global:",0
,function(globalName){var self=this;
var result=nil;
result=smalltalk.Smalltalk.nativeObjectAt_ifAbsent_(globalName,(function(){return nil;}));
if ((nil.isNil_(result)===false)){return result;};;
if (!(self.isValid())){return nil;};;
self.libraryLoaded_(self.loadLibrary());
return smalltalk.Smalltalk.nativeObjectAt_ifAbsent_(globalName,(function(){return nil;}));}
,"accessing",unescape("global%3A%20globalName%0D%0A%09%22%20Private%20-%20Return%20the%20native%20library%20global%20object%20%28or%20nil%29.%20%22%0D%0A%0D%0A%09%7C%20result%20%7C%0D%0A%09result%20%3A%3D%20Smalltalk%20nativeObjectAt%3A%20globalName%20ifAbsent%3A%20%5B%20nil%20%5D.%0D%0A%09result%20notNil%20ifTrue%3A%20%5B%20%5Eresult%20%5D.%0D%0A%09self%20isValid%20ifFalse%3A%20%5B%20%5Enil%20%5D.%0D%0A%09self%20libraryLoaded%3A%20self%20loadLibrary.%0D%0A%09%5ESmalltalk%20nativeObjectAt%3A%20globalName%20ifAbsent%3A%20%5B%20nil%20%5D"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"library",0
,function(){var self=this;
return self.global_(self.globalName());}
,"validation",unescape("library%0D%0A%09%22%20Check%20support%20to%20run%20this%20framework%20and%20try%20to%20load%20the%20library%20implementation.%0D%0A%09Return%20the%20native%20library%20object%20if%20success.%0D%0A%09%22%0D%0A%0D%0A%09%5Eself%20global%3A%20self%20globalName"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"libraryOrNil",0
,function(){var self=this;
return smalltalk.Smalltalk.nativeObjectAt_ifAbsent_(self.globalName(),(function(){return nil;}));}
,"validation",unescape("libraryOrNil%0D%0A%09%22%20Quick%20Check%20-%20Return%20true%20if%20the%20library%20is%20already%20bound.%20%22%0D%0A%0D%0A%09%5ESmalltalk%20nativeObjectAt%3A%20self%20globalName%20ifAbsent%3A%20%5B%20nil%20%5D"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"nativeClass",0
,function(){var self=this;
return smalltalk.NativeHandle._at(self.library());}
,"private",unescape("nativeClass%0D%0A%09%22%20Private%20-%20Return%20the%20wrapper%20for%20native%20binding%20of%20the%20receiver.%0D%0A%09The%20default%20result%20is%20a%20NativeHandle%20instance%20wrapping%20the%20receiver%27s%20library%2C%0D%0A%09%20so%20static%28class%20side%29%20builders%20generate%20bindings%20accessing%20receiver%27s%20library.%0D%0A%09%22%0D%0A%0D%0A%09%5ENativeHandle%20@%20self%20library"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"libraryURLs",0
,function(){var self=this;
return smalltalk.Array.with_(self.libraryURL()).select_((function(each){return (nil.isNil_(each)===false);}));}
,"private",unescape("libraryURLs%0D%0A%09%22%20Private%20-%20Return%20a%20collection%20of%20URL%20to%20load%20library%20of%20the%20receiver.%0D%0A%09Return%20an%20empty%20collection%2C%20the%20subclasses%20can%20complement%20addresses%20to%20load%20library%20from%20external%20resources.%0D%0A%09%22%0D%0A%0D%0A%09%5E%28Array%20with%3A%20self%20libraryURL%29%20select%3A%20%5B%3Aeach%7C%20each%20notNil%20%5D"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"libraryURL",0
,function(){var self=this;
return nil;}
,"private",unescape("libraryURL%0D%0A%09%22%20Private%20-%20Return%20the%20URL%20to%20use%20when%20loading%20library%20of%20the%20receiver.%0D%0A%09Default%20implementation%20do%20not%20provide%20support%20to%20load%20library.%0D%0A%09%22%0D%0A%0D%0A%09%5Enil"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"basicLoad:",0
,function(url){var self=this;
load(url);
return self;}
,"loading",unescape("basicLoad%3A%20url%0D%0A%09%22%20Private%20-%20Try%20to%20load%20the%20library%20at%20url.%20%22%0D%0A%0D%0A%09%7B%27%27js%27load%28url%29%27%7D"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"onLoad:",0
,function(aBlock){var $s8Ret$={name:"stReturn"};try{var self=this;
var dom=nil;
var success=nil;
if ((nil.isNil_(self.libraryOrNil())===false)){return aBlock.value_(self.library());};;
success=(function(){smalltalk.NativeLibraryObject.libraryBound_(self.name());return aBlock.value_(self.library());});
dom=smalltalk.Smalltalk.at_ifAbsent_("DOM",(function(){self.libraryURLs().do_((function(each){return self.basicLoad_(each);}));return (function(){$s8Ret$.result=(function(){return success.value()})();throw($s8Ret$)})();}));
(function(){$s8Ret$.result=(function(){return dom.document().loadAll_onLoad_(self.libraryURLs(),(function(){return success.value();}))})();throw($s8Ret$)})();} catch($$ex) {if($$ex === $s8Ret$){return $$ex.result} throw($$ex);}}
,"private",unescape("onLoad%3A%20aBlock%0D%0A%09%22%20Private%20-%20Load%20the%20library%20of%20the%20receiver%20and%20evaluates%20aBlock.%20%22%0D%0A%0D%0A%09%7C%20dom%20success%20%7C%0D%0A%09self%20libraryOrNil%20notNil%20ifTrue%3A%20%5B%20%5EaBlock%20value%3A%20self%20library%20%5D.%0D%0A%09success%20%3A%3D%20%5B%0D%0A%09%09NativeLibraryObject%20libraryBound%3A%20self%20name.%0D%0A%09%09aBlock%20value%3A%20self%20library%0D%0A%09%5D.%0D%0A%09dom%20%3A%3D%20Smalltalk%20at%3A%20%23DOM%20ifAbsent%3A%20%5B%0D%0A%09%09self%20libraryURLs%20do%3A%20%5B%3Aeach%7C%20self%20basicLoad%3A%20each%20%5D.%0D%0A%09%09%5Esuccess%20value%0D%0A%09%5D.%0D%0A%09%5Edom%20document%0D%0A%09%09loadAll%3A%20self%20libraryURLs%0D%0A%09%09onLoad%3A%20%5B%20success%20value%20%5D"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"loadLibrary",0
,function(){var self=this;
if (self.libraryURLs().notEmpty()){return self.onLoad_((function(ignored){return nil;}));};;
return smalltalk.Smalltalk.current().eval_as_(self.sourceCode(),smalltalk.Array.with_with_("loadLibrary",self.name()));}
,"private",unescape("loadLibrary%0D%0A%09%22%20Private%20-%20Load%20the%20library%20of%20the%20receiver.%0D%0A%09Default%20implementation%20load%20the%20library%20from%20remote%20URL%20or%20sourceCode.%0D%0A%09%22%0D%0A%0D%0A%09self%20libraryURLs%20notEmpty%20ifTrue%3A%20%5B%20%5Eself%20onLoad%3A%20%5B%3Aignored%7C%20%5D%20%5D.%0D%0A%09%5ESmalltalk%20current%0D%0A%09%09eval%3A%20self%20sourceCode%0D%0A%09%09as%3A%20%28Array%20with%3A%20%23loadLibrary%20with%3A%20self%20name%29"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"libraryLoaded:",0
,function(loadResult){var self=this;
return self;}
,"private",unescape("libraryLoaded%3A%20loadResult%0D%0A%09%22%20Private%20-%20The%20library%20of%20the%20receiver%20has%20been%20loaded.%0D%0A%09Default%20implementation%20do%20nothing.%0D%0A%09This%20method%20can%20be%20refined%20by%20subclasses%20to%20perform%20postLoading%20actions.%0D%0A%09The%20loadResult%20argument%20is%20the%20result%20of%20the%20just%20evaluated%20load%20process.%0D%0A%09%22"));
smalltalk.bind(smalltalk.NativeLibraryObject.$klass,"isValid",0
,function(){var self=this;
var global=nil;
global=self.topGlobal();
if ((nil.isNil_(global))){return true;};;
self.requiredFeatures().do_((function(each){return (((nil.isNil_(global[each]))).mustBeBoolean()==true ? (function(){return self.error_("Missing support for "._comma(each));})() : nil);}));
return true;}
,"validation",unescape("isValid%0D%0A%09%22%20Return%20true%20if%20the%20receiver%20is%20valid%20in%20this%20%28browser%29%20context.%0D%0A%09Generate%20error%20if%20the%20minimum%20support%20to%20run%20the%20library%20is%20not%20available.%0D%0A%09%22%0D%0A%0D%0A%09%7C%20global%20%7C%0D%0A%09global%20%3A%3D%20self%20topGlobal.%0D%0A%09global%20isNil%20ifTrue%3A%20%5B%20%5Etrue%20%5D.%20%22always%20valid%22%0D%0A%09self%20requiredFeatures%20do%3A%20%5B%3Aeach%7C%0D%0A%09%20%20global%5Beach%5D%20isNil%20ifTrue%3A%20%5B%0D%0A%09%20%20%20%20self%20error%3A%20%27Missing%20support%20for%20%27%2Ceach%0D%0A%09%20%20%5D.%0D%0A%09%5D.%0D%0A%09%5Etrue"));
smalltalk.bind(smalltalk.NativeLibraryObject,"library",0
,function(){var self=this;
return self.$klass.library();}
,"private",unescape("library%0D%0A%09%22%20Private%20-%20Return%20the%20library%20of%20the%20receiver.%20%22%0D%0A%0D%0A%09%5Eself%20class%20library"));
smalltalk.bind(smalltalk.NativeLibraryObject,"classNamed:",0
,function(className){var self=this;
return self.$klass.classNamed_(className);}
,"private",unescape("classNamed%3A%20className%0D%0A%09%22%20Private%20-%20Return%20the%20class%20named%20className%20in%20the%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%5Eself%20class%20classNamed%3A%20className"));
smalltalk.bind(smalltalk.NativeLibraryObject,"newHandleOf:",0
,function(className){var self=this;
var result=nil;
var aConstructor=nil;
aConstructor=self.classNamed_(className);
result = new aConstructor();
return result;}
,"private",unescape("newHandleOf%3A%20className%0D%0A%09%22%20Private%20-%20Return%20a%20new%20instance%20of%20the%20class%20named%20className%20in%20the%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%7C%20result%20aConstructor%20%7C%0D%0A%09aConstructor%20%3A%3D%20self%20classNamed%3A%20className.%0D%0A%09%7B%27%27js%27result%20%3D%20new%20aConstructor%28%29%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeLibraryObject,"newHandleOf:with:",0
,function(className,arg1){var self=this;
var result=nil;
var aConstructor=nil;
aConstructor=self.classNamed_(className);
result = new aConstructor(arg1);
return result;}
,"private",unescape("newHandleOf%3A%20className%20with%3A%20arg1%0D%0A%09%22%20Private%20-%20Return%20a%20new%20instance%20of%20the%20class%20named%20className%20in%20the%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%7C%20result%20aConstructor%20%7C%0D%0A%09aConstructor%20%3A%3D%20self%20classNamed%3A%20className.%0D%0A%09%7B%27%27js%27result%20%3D%20new%20aConstructor%28arg1%29%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:",0
,function(className,arg1,arg2){var self=this;
var result=nil;
var aConstructor=nil;
aConstructor=self.classNamed_(className);
result = new aConstructor(arg1,arg2);
return result;}
,"private",unescape("newHandleOf%3A%20className%20with%3A%20arg1%20with%3A%20arg2%0D%0A%09%22%20Private%20-%20Return%20a%20new%20instance%20of%20the%20class%20named%20className%20in%20the%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%7C%20result%20aConstructor%20%7C%0D%0A%09aConstructor%20%3A%3D%20self%20classNamed%3A%20className.%0D%0A%09%7B%27%27js%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%29%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:",0
,function(className,arg1,arg2,arg3){var self=this;
var result=nil;
var aConstructor=nil;
aConstructor=self.classNamed_(className);
result = new aConstructor(arg1,arg2,arg3);
return result;}
,"private",unescape("newHandleOf%3A%20className%20with%3A%20arg1%20with%3A%20arg2%20with%3A%20arg3%0D%0A%09%22%20Private%20-%20Return%20a%20new%20instance%20of%20the%20class%20named%20className%20in%20the%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%7C%20result%20aConstructor%20%7C%0D%0A%09aConstructor%20%3A%3D%20self%20classNamed%3A%20className.%0D%0A%09%7B%27%27js%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%29%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:with:",0
,function(className,arg1,arg2,arg3,arg4){var self=this;
var result=nil;
var aConstructor=nil;
aConstructor=self.classNamed_(className);
result = new aConstructor(arg1,arg2,arg3,arg4);
return result;}
,"private",unescape("newHandleOf%3A%20className%20with%3A%20arg1%20with%3A%20arg2%20with%3A%20arg3%20with%3A%20arg4%0D%0A%09%22%20Private%20-%20Return%20a%20new%20instance%20of%20the%20class%20named%20className%20in%20the%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%7C%20result%20aConstructor%20%7C%0D%0A%09aConstructor%20%3A%3D%20self%20classNamed%3A%20className.%0D%0A%09%7B%27%27js%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%29%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:with:with:",0
,function(className,arg1,arg2,arg3,arg4,arg5){var self=this;
var result=nil;
var aConstructor=nil;
aConstructor=self.classNamed_(className);
result = new aConstructor(arg1,arg2,arg3,arg4,arg5);
return result;}
,"private",unescape("newHandleOf%3A%20className%20with%3A%20arg1%20with%3A%20arg2%20with%3A%20arg3%20with%3A%20arg4%20with%3A%20arg5%0D%0A%09%22%20Private%20-%20Return%20a%20new%20instance%20of%20the%20class%20named%20className%20in%20the%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%7C%20result%20aConstructor%20%7C%0D%0A%09aConstructor%20%3A%3D%20self%20classNamed%3A%20className.%0D%0A%09%7B%27%27js%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%29%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:with:with:with:",0
,function(className,arg1,arg2,arg3,arg4,arg5,arg6){var self=this;
var result=nil;
var aConstructor=nil;
aConstructor=self.classNamed_(className);
result = new aConstructor(arg1,arg2,arg3,arg4,arg5,arg6);
return result;}
,"private",unescape("newHandleOf%3A%20className%20with%3A%20arg1%20with%3A%20arg2%20with%3A%20arg3%20with%3A%20arg4%20with%3A%20arg5%20with%3A%20arg6%0D%0A%09%22%20Private%20-%20Return%20a%20new%20instance%20of%20the%20class%20named%20className%20in%20the%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%7C%20result%20aConstructor%20%7C%0D%0A%09aConstructor%20%3A%3D%20self%20classNamed%3A%20className.%0D%0A%09%7B%27%27js%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%29%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:with:with:with:with:",0
,function(className,arg1,arg2,arg3,arg4,arg5,arg6,arg7){var self=this;
var result=nil;
var aConstructor=nil;
aConstructor=self.classNamed_(className);
result = new aConstructor(arg1,arg2,arg3,arg4,arg5,arg6,arg7);
return result;}
,"private",unescape("newHandleOf%3A%20className%20with%3A%20arg1%20with%3A%20arg2%20with%3A%20arg3%20with%3A%20arg4%20with%3A%20arg5%20with%3A%20arg6%20with%3A%20arg7%0D%0A%09%22%20Private%20-%20Return%20a%20new%20instance%20of%20the%20class%20named%20className%20in%20the%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%7C%20result%20aConstructor%20%7C%0D%0A%09aConstructor%20%3A%3D%20self%20classNamed%3A%20className.%0D%0A%09%7B%27%27js%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%2Carg7%29%27%7D.%0D%0A%09%5Eresult"));
smalltalk.bind(smalltalk.NativeLibraryObject,"newHandleOf:with:with:with:with:with:with:with:with:",0
,function(className,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8){var self=this;
var result=nil;
var aConstructor=nil;
aConstructor=self.classNamed_(className);
result = new aConstructor(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8);
return result;}
,"private",unescape("newHandleOf%3A%20className%20with%3A%20arg1%20with%3A%20arg2%20with%3A%20arg3%20with%3A%20arg4%20with%3A%20arg5%20with%3A%20arg6%20with%3A%20arg7%20with%3A%20arg8%0D%0A%09%22%20Private%20-%20Return%20a%20new%20instance%20of%20the%20class%20named%20className%20in%20the%20receiver%27s%20library.%20%22%0D%0A%0D%0A%09%7C%20result%20aConstructor%20%7C%0D%0A%09aConstructor%20%3A%3D%20self%20classNamed%3A%20className.%0D%0A%09%7B%27%27js%27result%20%3D%20new%20aConstructor%28arg1%2Carg2%2Carg3%2Carg4%2Carg5%2Carg6%2Carg7%2Carg8%29%27%7D.%0D%0A%09%5Eresult"));

smalltalk["$imageFileName"]="s8.image.js"
smalltalk["$nativeLanguage"]="js"
smalltalk["useManager"]=true
smalltalk["$state"]="running"
smalltalk["OverwriteMethods"]=true
smalltalk["DumpOverwritenMethods"]=false

smalltalk.launchImage()